diff -pruN 1:13.23.1~dfsg-2/addons/app_mysql.c 1:16.1.1~dfsg-1/addons/app_mysql.c
--- 1:13.23.1~dfsg-2/addons/app_mysql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/app_mysql.c	2018-12-26 18:45:13.000000000 +0000
@@ -218,7 +218,7 @@ static int add_identifier(struct ast_cha
 		ast_log(LOG_WARNING, "Unable to lock identifiers list\n");
 		return -1;
 	} else {
-		i = malloc(sizeof(*i));
+		i = ast_malloc(sizeof(*i));
 		AST_LIST_TRAVERSE(headp, j, entries) {
 			if (j->identifier > maxidentifier) {
 				maxidentifier = j->identifier;
@@ -247,7 +247,7 @@ static int del_identifier(int identifier
 			if ((i->identifier == identifier) &&
 			    (i->identifier_type == identifier_type)) {
 				AST_LIST_REMOVE(headp, i, entries);
-				free(i);
+				ast_free(i);
 				found = 1;
 				break;
 			}
diff -pruN 1:13.23.1~dfsg-2/addons/cdr_mysql.c 1:16.1.1~dfsg-1/addons/cdr_mysql.c
--- 1:13.23.1~dfsg-2/addons/cdr_mysql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/cdr_mysql.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,8 +42,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <mysql/mysql.h>
 #include <mysql/errmsg.h>
 
@@ -127,7 +125,9 @@ static char *handle_cli_cdr_mysql_status
 		return CLI_SHOWUSAGE;
 
 	if (connected) {
-		char status[256], status2[100] = "";
+		char status[256];
+		char status2[100] = "";
+		char buf[362]; /* 256+100+" for "+NULL */
 		int ctime = time(NULL) - connect_time;
 		if (dbport)
 			snprintf(status, 255, "Connected to %s@%s, port %d", ast_str_buffer(dbname), ast_str_buffer(hostname), dbport);
@@ -140,17 +140,10 @@ static char *handle_cli_cdr_mysql_status
 			snprintf(status2, 99, " with username %s", ast_str_buffer(dbuser));
 		if (ast_str_strlen(dbtable))
 			snprintf(status2, 99, " using table %s", ast_str_buffer(dbtable));
-		if (ctime > 31536000) {
-			ast_cli(a->fd, "%s%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 31536000, (ctime % 31536000) / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 86400) {
-			ast_cli(a->fd, "%s%s for %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 3600) {
-			ast_cli(a->fd, "%s%s for %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 60) {
-			ast_cli(a->fd, "%s%s for %d minutes, %d seconds.\n", status, status2, ctime / 60, ctime % 60);
-		} else {
-			ast_cli(a->fd, "%s%s for %d seconds.\n", status, status2, ctime);
-		}
+
+		snprintf(buf, sizeof(buf), "%s%s for ", status, status2);
+		ast_cli_print_timestr_fromseconds(a->fd, ctime, buf);
+
 		if (records == totalrecords)
 			ast_cli(a->fd, "  Wrote %d records since last restart.\n", totalrecords);
 		else
@@ -455,6 +448,140 @@ static int my_load_config_number(struct
 	return 0;
 }
 
+/** Connect to MySQL. Initializes the connection.
+ *
+ * * Assumes the read-write lock for columns is held.
+ * * Caller should allocate and free cfg
+ * */
+static int my_connect_db(struct ast_config *cfg)
+{
+	struct ast_variable *var;
+	char *temp;
+	MYSQL_ROW row;
+	MYSQL_RES *result;
+	char sqldesc[128];
+#if MYSQL_VERSION_ID >= 50013
+	my_bool my_bool_true = 1;
+#endif
+
+	mysql_init(&mysql);
+
+	if (timeout && mysql_options(&mysql, MYSQL_OPT_CONNECT_TIMEOUT, (char *)&timeout) != 0) {
+		ast_log(LOG_ERROR, "cdr_mysql: mysql_options returned (%d) %s\n", mysql_errno(&mysql), mysql_error(&mysql));
+	}
+
+#if MYSQL_VERSION_ID >= 50013
+	/* Add option for automatic reconnection */
+	if (mysql_options(&mysql, MYSQL_OPT_RECONNECT, &my_bool_true) != 0) {
+		ast_log(LOG_ERROR, "cdr_mysql: mysql_options returned (%d) %s\n", mysql_errno(&mysql), mysql_error(&mysql));
+	}
+#endif
+
+	if ((ssl_ca && ast_str_strlen(ssl_ca)) || (ssl_cert && ast_str_strlen(ssl_cert)) || (ssl_key && ast_str_strlen(ssl_key))) {
+		mysql_ssl_set(&mysql,
+			ssl_key ? ast_str_buffer(ssl_key) : NULL,
+			ssl_cert ? ast_str_buffer(ssl_cert) : NULL,
+			ssl_ca ? ast_str_buffer(ssl_ca) : NULL,
+			NULL, NULL);
+	}
+	temp = dbsock && ast_str_strlen(dbsock) ? ast_str_buffer(dbsock) : NULL;
+
+	configure_connection_charset();
+
+	if (!mysql_real_connect(&mysql, ast_str_buffer(hostname), ast_str_buffer(dbuser), ast_str_buffer(password), ast_str_buffer(dbname), dbport, temp, ssl_ca && ast_str_strlen(ssl_ca) ? CLIENT_SSL : 0)) {
+		ast_log(LOG_ERROR, "Failed to connect to mysql database %s on %s.\n", ast_str_buffer(dbname), ast_str_buffer(hostname));
+		connected = 0;
+		records = 0;
+
+		return AST_MODULE_LOAD_SUCCESS;	/* May be reconnected later */
+	}
+
+	ast_debug(1, "Successfully connected to MySQL database.\n");
+	connected = 1;
+	records = 0;
+	connect_time = time(NULL);
+
+	/* Get table description */
+	snprintf(sqldesc, sizeof(sqldesc), "DESC %s", dbtable ? ast_str_buffer(dbtable) : "cdr");
+	if (mysql_query(&mysql, sqldesc)) {
+		ast_log(LOG_ERROR, "Unable to query table description!!  Logging disabled.\n");
+		mysql_close(&mysql);
+		connected = 0;
+
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (!(result = mysql_store_result(&mysql))) {
+		ast_log(LOG_ERROR, "Unable to query table description!!  Logging disabled.\n");
+		mysql_close(&mysql);
+		connected = 0;
+
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	while ((row = mysql_fetch_row(result))) {
+		struct column *entry;
+		char *cdrvar = "", *staticvalue = "";
+
+		ast_debug(1, "Got a field '%s' of type '%s'\n", row[0], row[1]);
+		/* Check for an alias or a static value */
+		for (var = ast_variable_browse(cfg, "columns"); var; var = var->next) {
+			if (strncmp(var->name, "alias", 5) == 0 && strcasecmp(var->value, row[0]) == 0 ) {
+				char *alias = ast_strdupa(var->name + 5);
+				cdrvar = ast_strip(alias);
+				ast_verb(3, "Found alias %s for column %s\n", cdrvar, row[0]);
+				break;
+			} else if (strncmp(var->name, "static", 6) == 0 && strcasecmp(var->value, row[0]) == 0) {
+				char *item = ast_strdupa(var->name + 6);
+				item = ast_strip(item);
+				if (item[0] == '"' && item[strlen(item) - 1] == '"') {
+					/* Remove surrounding quotes */
+					item[strlen(item) - 1] = '\0';
+					item++;
+				}
+				staticvalue = item;
+			}
+		}
+
+		entry = ast_calloc(sizeof(char), sizeof(*entry) + strlen(row[0]) + 1 + strlen(cdrvar) + 1 + strlen(staticvalue) + 1 + strlen(row[1]) + 1);
+		if (!entry) {
+			ast_log(LOG_ERROR, "Out of memory creating entry for column '%s'\n", row[0]);
+			mysql_free_result(result);
+			return AST_MODULE_LOAD_DECLINE;
+		}
+
+		entry->name = (char *)entry + sizeof(*entry);
+		strcpy(entry->name, row[0]);
+
+		if (!ast_strlen_zero(cdrvar)) {
+			entry->cdrname = entry->name + strlen(row[0]) + 1;
+			strcpy(entry->cdrname, cdrvar);
+		} else { /* Point to same place as the column name */
+			entry->cdrname = (char *)entry + sizeof(*entry);
+		}
+
+		if (!ast_strlen_zero(staticvalue)) {
+			entry->staticvalue = entry->cdrname + strlen(entry->cdrname) + 1;
+			strcpy(entry->staticvalue, staticvalue);
+			ast_debug(1, "staticvalue length: %d\n", (int) strlen(staticvalue) );
+			entry->type = entry->staticvalue + strlen(entry->staticvalue) + 1;
+		} else {
+			entry->type = entry->cdrname + strlen(entry->cdrname) + 1;
+		}
+		strcpy(entry->type, row[1]);
+
+		ast_debug(1, "Entry name '%s'\n", entry->name);
+		ast_debug(1, "   cdrname '%s'\n", entry->cdrname);
+		ast_debug(1, "    static '%s'\n", entry->staticvalue);
+		ast_debug(1, "      type '%s'\n", entry->type);
+
+		AST_LIST_INSERT_TAIL(&columns, entry, list);
+	}
+	mysql_free_result(result);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
 static int my_load_module(int reload)
 {
 	int res;
@@ -465,14 +592,7 @@ static int my_load_module(int reload)
 	 * rescan the table layout. */
 	struct ast_flags config_flags = { 0 };
 	struct column *entry;
-	char *temp;
 	struct ast_str *compat;
-	MYSQL_ROW row;
-	MYSQL_RES *result;
-	char sqldesc[128];
-#if MYSQL_VERSION_ID >= 50013
-	my_bool my_bool_true = 1;
-#endif
 
 	/* Cannot use a conditionally different flag, because the table layout may
 	 * have changed, which is not detectable by config file change detection,
@@ -569,129 +689,12 @@ static int my_load_module(int reload)
 		ast_debug(1, "Got DB charset of %s\n", ast_str_buffer(dbcharset));
 	}
 
-	mysql_init(&mysql);
-
-	if (timeout && mysql_options(&mysql, MYSQL_OPT_CONNECT_TIMEOUT, (char *)&timeout) != 0) {
-		ast_log(LOG_ERROR, "cdr_mysql: mysql_options returned (%d) %s\n", mysql_errno(&mysql), mysql_error(&mysql));
-	}
-
-#if MYSQL_VERSION_ID >= 50013
-	/* Add option for automatic reconnection */
-	if (mysql_options(&mysql, MYSQL_OPT_RECONNECT, &my_bool_true) != 0) {
-		ast_log(LOG_ERROR, "cdr_mysql: mysql_options returned (%d) %s\n", mysql_errno(&mysql), mysql_error(&mysql));
-	}
-#endif
-
-	if ((ssl_ca && ast_str_strlen(ssl_ca)) || (ssl_cert && ast_str_strlen(ssl_cert)) || (ssl_key && ast_str_strlen(ssl_key))) {
-		mysql_ssl_set(&mysql,
-			ssl_key ? ast_str_buffer(ssl_key) : NULL,
-			ssl_cert ? ast_str_buffer(ssl_cert) : NULL,
-			ssl_ca ? ast_str_buffer(ssl_ca) : NULL,
-			NULL, NULL);
-	}
-	temp = dbsock && ast_str_strlen(dbsock) ? ast_str_buffer(dbsock) : NULL;
-
-	configure_connection_charset();
-
-	if (!mysql_real_connect(&mysql, ast_str_buffer(hostname), ast_str_buffer(dbuser), ast_str_buffer(password), ast_str_buffer(dbname), dbport, temp, ssl_ca && ast_str_strlen(ssl_ca) ? CLIENT_SSL : 0)) {
-		ast_log(LOG_ERROR, "Failed to connect to mysql database %s on %s.\n", ast_str_buffer(dbname), ast_str_buffer(hostname));
-		connected = 0;
-		records = 0;
-	} else {
-		ast_debug(1, "Successfully connected to MySQL database.\n");
-		connected = 1;
-		records = 0;
-		connect_time = time(NULL);
-
-		/* Get table description */
-		snprintf(sqldesc, sizeof(sqldesc), "DESC %s", dbtable ? ast_str_buffer(dbtable) : "cdr");
-		if (mysql_query(&mysql, sqldesc)) {
-			ast_log(LOG_ERROR, "Unable to query table description!!  Logging disabled.\n");
-			mysql_close(&mysql);
-			connected = 0;
-			AST_RWLIST_UNLOCK(&columns);
-			ast_config_destroy(cfg);
-			free_strings();
-
-			return AST_MODULE_LOAD_DECLINE;
-		}
-
-		if (!(result = mysql_store_result(&mysql))) {
-			ast_log(LOG_ERROR, "Unable to query table description!!  Logging disabled.\n");
-			mysql_close(&mysql);
-			connected = 0;
-			AST_RWLIST_UNLOCK(&columns);
-			ast_config_destroy(cfg);
-			free_strings();
-
-			return AST_MODULE_LOAD_DECLINE;
-		}
-
-		while ((row = mysql_fetch_row(result))) {
-			struct column *entry;
-			char *cdrvar = "", *staticvalue = "";
-
-			ast_debug(1, "Got a field '%s' of type '%s'\n", row[0], row[1]);
-			/* Check for an alias or a static value */
-			for (var = ast_variable_browse(cfg, "columns"); var; var = var->next) {
-				if (strncmp(var->name, "alias", 5) == 0 && strcasecmp(var->value, row[0]) == 0 ) {
-					char *alias = ast_strdupa(var->name + 5);
-					cdrvar = ast_strip(alias);
-					ast_verb(3, "Found alias %s for column %s\n", cdrvar, row[0]);
-					break;
-				} else if (strncmp(var->name, "static", 6) == 0 && strcasecmp(var->value, row[0]) == 0) {
-					char *item = ast_strdupa(var->name + 6);
-					item = ast_strip(item);
-					if (item[0] == '"' && item[strlen(item) - 1] == '"') {
-						/* Remove surrounding quotes */
-						item[strlen(item) - 1] = '\0';
-						item++;
-					}
-					staticvalue = item;
-				}
-			}
-
-			entry = ast_calloc(sizeof(char), sizeof(*entry) + strlen(row[0]) + 1 + strlen(cdrvar) + 1 + strlen(staticvalue) + 1 + strlen(row[1]) + 1);
-			if (!entry) {
-				ast_log(LOG_ERROR, "Out of memory creating entry for column '%s'\n", row[0]);
-				res = -1;
-				break;
-			}
-
-			entry->name = (char *)entry + sizeof(*entry);
-			strcpy(entry->name, row[0]);
-
-			if (!ast_strlen_zero(cdrvar)) {
-				entry->cdrname = entry->name + strlen(row[0]) + 1;
-				strcpy(entry->cdrname, cdrvar);
-			} else { /* Point to same place as the column name */
-				entry->cdrname = (char *)entry + sizeof(*entry);
-			}
-
-			if (!ast_strlen_zero(staticvalue)) {
-				entry->staticvalue = entry->cdrname + strlen(entry->cdrname) + 1;
-				strcpy(entry->staticvalue, staticvalue);
-				ast_debug(1, "staticvalue length: %d\n", (int) strlen(staticvalue) );
-				entry->type = entry->staticvalue + strlen(entry->staticvalue) + 1;
-			} else {
-				entry->type = entry->cdrname + strlen(entry->cdrname) + 1;
-			}
-			strcpy(entry->type, row[1]);
-
-			ast_debug(1, "Entry name '%s'\n", entry->name);
-			ast_debug(1, "   cdrname '%s'\n", entry->cdrname);
-			ast_debug(1, "    static '%s'\n", entry->staticvalue);
-			ast_debug(1, "      type '%s'\n", entry->type);
-
-			AST_LIST_INSERT_TAIL(&columns, entry, list);
-		}
-		mysql_free_result(result);
-	}
+	res = my_connect_db(cfg);
 	AST_RWLIST_UNLOCK(&columns);
 	ast_config_destroy(cfg);
-	if (res < 0) {
+	if (res != AST_MODULE_LOAD_SUCCESS) {
 		my_unload_module(0);
-		return AST_MODULE_LOAD_DECLINE;
+		return res;
 	}
 
 	if (!reload) {
@@ -739,4 +742,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload,
+	.requires = "cdr",
 );
diff -pruN 1:13.23.1~dfsg-2/addons/chan_mobile.c 1:16.1.1~dfsg-1/addons/chan_mobile.c
--- 1:13.23.1~dfsg-2/addons/chan_mobile.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/chan_mobile.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,8 +42,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pthread.h>
 #include <signal.h>
 
@@ -4818,8 +4816,8 @@ e_cleanup:
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Bluetooth Mobile Device Channel Driver",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
 );
diff -pruN 1:13.23.1~dfsg-2/addons/chan_ooh323.c 1:16.1.1~dfsg-1/addons/chan_ooh323.c
--- 1:13.23.1~dfsg-2/addons/chan_ooh323.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/chan_ooh323.c	2018-12-26 18:45:13.000000000 +0000
@@ -359,6 +359,7 @@ static int  gDTMFCodec = 101;
 static int  gFAXdetect = FAXDETECT_CNG;
 static int  gT38Support = T38_FAXGW;
 static char gGatekeeper[100];
+static char gRASIP[2+8*4+7];	/* Max for IPv6 addr */
 static enum RasGatekeeperMode gRasGkMode = RasNoGatekeeper;
 
 static int  gIsGateway = 0;
@@ -590,7 +591,7 @@ static struct ooh323_pvt *ooh323_alloc(i
 
 	pvt->call_reference = callref;
 	if (callToken)
-		pvt->callToken = strdup(callToken);
+		pvt->callToken = ast_strdup(callToken);
 
 	/* whether to use gk for this call */
 	if (gRasGkMode == RasNoGatekeeper)
@@ -697,8 +698,8 @@ static struct ast_channel *ooh323_reques
 	}
 
 	if (peer) {
-		p->username = strdup(peer->name);
-		p->host = strdup(peer->ip);
+		p->username = ast_strdup(peer->name);
+		p->host = ast_strdup(peer->ip);
 		p->port = peer->port;
 		/* Disable gk as we are going to call a known peer*/
 		/* OO_SETFLAG(p->flags, H323_DISABLEGK); */
@@ -759,9 +760,9 @@ static struct ast_channel *ooh323_reques
 		p->directrtp = gDirectRTP;
 		p->earlydirect = gEarlyDirect;
 
-		p->username = strdup(dest);
+		p->username = ast_strdup(dest);
 
-		p->host = strdup(dest);
+		p->host = ast_strdup(dest);
 		if (port > 0) {
 			p->port = port;
 		}
@@ -1016,22 +1017,22 @@ static int ooh323_call(struct ast_channe
 	ast_mutex_lock(&p->lock);
 	ast_set_flag(p, H323_OUTGOING);
 	if (ast_channel_connected(ast)->id.number.valid && ast_channel_connected(ast)->id.number.str) {
-		free(p->callerid_num);
-		p->callerid_num = strdup(ast_channel_connected(ast)->id.number.str);
+		ast_free(p->callerid_num);
+		p->callerid_num = ast_strdup(ast_channel_connected(ast)->id.number.str);
 	}
 
 	if (ast_channel_connected(ast)->id.name.valid && ast_channel_connected(ast)->id.name.str) {
-		free(p->callerid_name);
-		p->callerid_name = strdup(ast_channel_connected(ast)->id.name.str);
+		ast_free(p->callerid_name);
+		p->callerid_name = ast_strdup(ast_channel_connected(ast)->id.name.str);
 	} else if (ast_channel_connected(ast)->id.number.valid && ast_channel_connected(ast)->id.number.str) {
-		free(p->callerid_name);
-		p->callerid_name = strdup(ast_channel_connected(ast)->id.number.str);
+		ast_free(p->callerid_name);
+		p->callerid_name = ast_strdup(ast_channel_connected(ast)->id.number.str);
 	} else {
 		ast_channel_connected(ast)->id.name.valid = 1;
-		free(ast_channel_connected(ast)->id.name.str);
-		ast_channel_connected(ast)->id.name.str = strdup(gCallerID);
-		free(p->callerid_name);
-		p->callerid_name = strdup(ast_channel_connected(ast)->id.name.str);
+		ast_free(ast_channel_connected(ast)->id.name.str);
+		ast_channel_connected(ast)->id.name.str = ast_strdup(gCallerID);
+		ast_free(p->callerid_name);
+		p->callerid_name = ast_strdup(ast_channel_connected(ast)->id.name.str);
 	}
 
 	/* Retrieve vars */
@@ -1044,7 +1045,7 @@ static int ooh323_call(struct ast_channe
 	if ((val = pbx_builtin_getvar_helper(ast, "CALLER_H323DIALEDDIGITS"))) {
 		ast_copy_string(p->caller_dialedDigits, val, sizeof(p->caller_dialedDigits));
       		if(!p->callerid_num)
-			p->callerid_num = strdup(val);
+			p->callerid_num = ast_strdup(val);
 	}
 
 	if ((val = pbx_builtin_getvar_helper(ast, "CALLER_H323EMAIL"))) {
@@ -1165,7 +1166,7 @@ static int ooh323_answer(struct ast_chan
 	if (p) {
 
 		ast_mutex_lock(&p->lock);
-		callToken = (p->callToken ? strdup(p->callToken) : NULL);
+		callToken = (p->callToken ? ast_strdup(p->callToken) : NULL);
 		if (ast_channel_state(ast) != AST_STATE_UP) {
 			ast_channel_lock(ast);
 			if (!p->alertsent) {
@@ -1183,7 +1184,7 @@ static int ooh323_answer(struct ast_chan
 			ooAnswerCall(p->callToken);
 		}
 		if (callToken) {
-			free(callToken);
+			ast_free(callToken);
 		}
 		ast_mutex_unlock(&p->lock);
 	}
@@ -1290,7 +1291,7 @@ static int ooh323_indicate(struct ast_ch
 	if (!p) return -1;
 
 	ast_mutex_lock(&p->lock);
-	callToken = (p->callToken ? strdup(p->callToken) : NULL);
+	callToken = (p->callToken ? ast_strdup(p->callToken) : NULL);
 	ast_mutex_unlock(&p->lock);
 
 	if (!callToken) {
@@ -1456,7 +1457,7 @@ static int ooh323_indicate(struct ast_ch
 		ast_verb(0, "++++  ooh323_indicate %d on %s is %d\n", condition, callToken, res);
 	}
 
-   	free(callToken);
+   	ast_free(callToken);
 	return res;
 }
 
@@ -1862,25 +1863,25 @@ int ooh323_onReceivedSetup(ooCallData *c
 
 
 	if (call->remoteDisplayName) {
-		p->callerid_name = strdup(call->remoteDisplayName);
+		p->callerid_name = ast_strdup(call->remoteDisplayName);
 	}
 
 	if (ooCallGetCallingPartyNumber(call, number, OO_MAX_NUMBER_LENGTH) == OO_OK) {
-		p->callerid_num = strdup(number);
+		p->callerid_num = ast_strdup(number);
 	}
 
 	if (call->remoteAliases) {
 		for (alias = call->remoteAliases; alias; alias = alias->next) {
 			if (alias->type == T_H225AliasAddress_h323_ID) {
 				if (!p->callerid_name) {
-					p->callerid_name = strdup(alias->value);
+					p->callerid_name = ast_strdup(alias->value);
 				}
 				ast_copy_string(p->caller_h323id, alias->value, sizeof(p->caller_h323id));
 				}
          else if(alias->type == T_H225AliasAddress_dialedDigits)
          {
             if(!p->callerid_num)
-               p->callerid_num = strdup(alias->value);
+               p->callerid_num = ast_strdup(alias->value);
 				ast_copy_string(p->caller_dialedDigits, alias->value,
 															sizeof(p->caller_dialedDigits));
          }
@@ -1921,7 +1922,7 @@ int ooh323_onReceivedSetup(ooCallData *c
       	user = find_user(p->callerid_name, call->remoteIP);
       	if(user && (user->incominglimit == 0 || user->inUse < user->incominglimit)) {
 		ast_mutex_lock(&user->lock);
-		p->username = strdup(user->name);
+		p->username = ast_strdup(user->name);
  		p->neighbor.user = user->mUseIP ? ast_strdup(user->mIP) :
 						  ast_strdup(user->name);
 		ast_copy_string(p->context, user->context, sizeof(p->context));
@@ -1967,7 +1968,7 @@ int ooh323_onReceivedSetup(ooCallData *c
 		ast_mutex_unlock(&user->lock);
 	} else {
 	 if (!OO_TESTFLAG(p->flags,H323_DISABLEGK)) {
-		p->username = strdup(call->remoteIP);
+		p->username = ast_strdup(call->remoteIP);
 		p->directrtp = gDirectRTP;
 		p->earlydirect = gEarlyDirect;
 	} else {
@@ -2320,7 +2321,7 @@ int onCallCleared(ooCallData *call)
 		}
 		ast_mutex_unlock(&userl.lock);
 
-		free(user);
+		ast_free(user);
 	}
 
 	if (gH323Debug)
@@ -2352,13 +2353,13 @@ void ooh323_delete_peer(struct ooh323_pe
 			}
 		ast_mutex_unlock(&peerl.lock);
 
-      if(peer->h323id)   free(peer->h323id);
-      if(peer->email)    free(peer->email);
-      if(peer->url)      free(peer->url);
-      if(peer->e164)     free(peer->e164);
+		ast_free(peer->h323id);
+		ast_free(peer->email);
+		ast_free(peer->url);
+		ast_free(peer->e164);
 
-      ao2_cleanup(peer->cap);
-		free(peer);
+		ao2_cleanup(peer->cap);
+		ast_free(peer);
 	}
 
 	if (gH323Debug)
@@ -2725,7 +2726,7 @@ static int ooh323_do_reload(void)
 	if (gRasGkMode == RasUseSpecificGatekeeper ||
 		gRasGkMode == RasDiscoverGatekeeper) {
 		ooGkClientInit(gRasGkMode, (gRasGkMode == RasUseSpecificGatekeeper) ?
-								gGatekeeper : 0, 0);
+								gGatekeeper : 0, gRASIP, 0);
 		ooGkClientStart(gH323ep.gkClient);
 	}
 
@@ -2847,8 +2848,8 @@ int reload_config(int reload)
 		while (cur) {
 			prev = cur;
 	  		cur = cur->next;
-	  		free(prev->value);
-	  		free(prev);
+	  		ast_free(prev->value);
+	  		ast_free(prev);
 		}
 		gAliasList = NULL;
 		ooH323EpClearAllAliases();
@@ -2868,6 +2869,7 @@ int reload_config(int reload)
 	gTRCLVL = OOTRCLVLERR;
 	gRasGkMode = RasNoGatekeeper;
 	gGatekeeper[0] = '\0';
+	gRASIP[0] = '\0';
 	gRTPTimeout = 60;
 	gNat = FALSE;
 	gRTDRInterval = 0;
@@ -2963,7 +2965,7 @@ int reload_config(int reload)
 	  			ast_copy_string(gCallerID, v->value, sizeof(gCallerID));
 	 		}
 			pNewAlias->type =  T_H225AliasAddress_h323_ID;
-			pNewAlias->value = strdup(v->value);
+			pNewAlias->value = ast_strdup(v->value);
 			pNewAlias->next = gAliasList;
 			gAliasList = pNewAlias;
 			pNewAlias = NULL;
@@ -2984,7 +2986,7 @@ int reload_config(int reload)
 					return 1;
 				}
 				pNewAlias->type =  T_H225AliasAddress_dialedDigits;
-				pNewAlias->value = strdup(v->value);
+				pNewAlias->value = ast_strdup(v->value);
 				pNewAlias->next = gAliasList;
 				gAliasList = pNewAlias;
 				pNewAlias = NULL;
@@ -2999,7 +3001,7 @@ int reload_config(int reload)
 				return 1;
 			}
 			pNewAlias->type =  T_H225AliasAddress_email_ID;
-			pNewAlias->value = strdup(v->value);
+			pNewAlias->value = ast_strdup(v->value);
 			pNewAlias->next = gAliasList;
 			gAliasList = pNewAlias;
 			pNewAlias = NULL;
@@ -3028,6 +3030,9 @@ int reload_config(int reload)
 				gRasGkMode = RasUseSpecificGatekeeper;
             			ast_copy_string(gGatekeeper, v->value, sizeof(gGatekeeper));
 			}
+		} else if (!strcasecmp(v->name, "localras")) {
+		        ast_copy_string(gRASIP, v->value, sizeof(gRASIP));
+			ast_verb(3, "  == Setting RAS IP to %s\n", gRASIP);
 		} else if (!strcasecmp(v->name, "logfile")) {
          		ast_copy_string(gLogFile, v->value, sizeof(gLogFile));
 		} else if (!strcasecmp(v->name, "context")) {
@@ -3923,9 +3928,9 @@ static int load_module(void)
 
 		/* Gatekeeper */
 		if (gRasGkMode == RasUseSpecificGatekeeper)
-			ooGkClientInit(gRasGkMode, gGatekeeper, 0);
+			ooGkClientInit(gRasGkMode, gGatekeeper, gRASIP, 0);
 		else if (gRasGkMode == RasDiscoverGatekeeper)
-			ooGkClientInit(gRasGkMode, 0, 0);
+			ooGkClientInit(gRasGkMode, 0, gRASIP, 0);
 
 		/* Register callbacks */
 		ooH323EpSetH323Callbacks(h323Callbacks);
@@ -4012,7 +4017,7 @@ static void *do_monitor(void *data)
 			ooGkClientDestroy();
 			ast_verb(0, "Restart stopped gatekeeper client\n");
 			ooGkClientInit(gRasGkMode, (gRasGkMode == RasUseSpecificGatekeeper) ?
-									gGatekeeper : 0, 0);
+									gGatekeeper : 0, gRASIP, 0);
 			ooGkClientStart(gH323ep.gkClient);
 		}
 
@@ -4144,22 +4149,22 @@ int ooh323_destroy(struct ooh323_pvt *p)
 		}
 
 		if (cur->username) {
-			free(cur->username);
+			ast_free(cur->username);
 			cur->username = 0;
 		}
 
 		if (cur->host) {
-			free(cur->host);
+			ast_free(cur->host);
 			cur->host = 0;
 		}
 
 		if (cur->callerid_name) {
-			free(cur->callerid_name);
+			ast_free(cur->callerid_name);
 			cur->callerid_name = 0;
 		}
 
 		if (cur->callerid_num) {
-			free(cur->callerid_num);
+			ast_free(cur->callerid_num);
 			cur->callerid_num = 0;
 		}
 
@@ -4202,7 +4207,7 @@ int ooh323_destroy(struct ooh323_pvt *p)
 	  	user->inUse--;
 	  	ast_mutex_unlock(&user->lock);
 	  }
-	  free(cur->neighbor.user);
+	  ast_free(cur->neighbor.user);
 	 }
       } else {
 /* outgoing limit decrement here !!! */
@@ -4232,10 +4237,10 @@ int delete_peers()
 		cur = cur->next;
 
 		ast_mutex_destroy(&prev->lock);
-      if(prev->h323id)   free(prev->h323id);
-      if(prev->email)    free(prev->email);
-      if(prev->url)      free(prev->url);
-      if(prev->e164)     free(prev->e164);
+		ast_free(prev->h323id);
+		ast_free(prev->email);
+		ast_free(prev->url);
+		ast_free(prev->e164);
       if(prev->rtpmask) {
 		ast_mutex_lock(&prev->rtpmask->lock);
 		prev->rtpmask->inuse--;
@@ -4243,10 +4248,10 @@ int delete_peers()
 	 	if (prev->rtpmask->inuse == 0) {
 	  		regfree(&prev->rtpmask->regex);
 			ast_mutex_destroy(&prev->rtpmask->lock);
-	  		free(prev->rtpmask);
+	  		ast_free(prev->rtpmask);
       		}
       }
-		free(prev);
+		ast_free(prev);
 
 		if (cur == peerl.peers) {
 			break;
@@ -4274,11 +4279,11 @@ int delete_users()
 	 		if (prev->rtpmask->inuse == 0) {
 	  			regfree(&prev->rtpmask->regex);
 				ast_mutex_destroy(&prev->rtpmask->lock);
-	  			free(prev->rtpmask);
+	  			ast_free(prev->rtpmask);
       			}
       		}
       	ao2_cleanup(prev->cap);
-		free(prev);
+		ast_free(prev);
 		if (cur == userl.users) {
 			break;
 		}
@@ -4390,8 +4395,8 @@ static int unload_module(void)
 	while (cur) {
 	  prev = cur;
 	  cur = cur->next;
-	  free(prev->value);
-	  free(prev);
+	  ast_free(prev->value);
+	  ast_free(prev);
 	}
 	gAliasList = NULL;
 
@@ -4555,7 +4560,7 @@ static int ooh323_set_rtp_peer(struct as
 		memset(&p->redirip, 0, sizeof(p->redirip));
 	}
 
-	callToken = (p->callToken ? strdup(p->callToken) : NULL);
+	callToken = (p->callToken ? ast_strdup(p->callToken) : NULL);
 
 	if (!callToken) {
 		if (gH323Debug) {
@@ -4583,7 +4588,7 @@ static int ooh323_set_rtp_peer(struct as
 	}
 
 	ast_mutex_unlock(&p->lock);
-	free(callToken);
+	ast_free(callToken);
 	return 0;
 
 }
@@ -5236,9 +5241,10 @@ void ast_ooh323c_exit()
 #endif
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Objective Systems H323 Channel",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-			.load = load_module,
-			.unload = unload_module,
-			.reload = reload_module,
-			.load_pri = AST_MODPRI_CHANNEL_DRIVER
-			);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "udptl",
+);
diff -pruN 1:13.23.1~dfsg-2/addons/chan_ooh323.h 1:16.1.1~dfsg-1/addons/chan_ooh323.h
--- 1:13.23.1~dfsg-2/addons/chan_ooh323.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/chan_ooh323.h	2018-12-26 18:45:13.000000000 +0000
@@ -38,7 +38,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
-#include <sys/signal.h>
 
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/addons/format_mp3.c 1:16.1.1~dfsg-1/addons/format_mp3.c
--- 1:13.23.1~dfsg-2/addons/format_mp3.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/format_mp3.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "mp3/mpg123.h"
 #include "mp3/mpglib.h"
 
diff -pruN 1:13.23.1~dfsg-2/addons/Makefile 1:16.1.1~dfsg-1/addons/Makefile
--- 1:13.23.1~dfsg-2/addons/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -14,15 +14,17 @@
 MENUSELECT_CATEGORY=ADDONS
 MENUSELECT_DESCRIPTION=Add-ons (See README-addons.txt)
 
-H323OBJS:=ooCmdChannel.o ooLogChan.o ooUtils.o ooGkClient.o context.o \
-	ooDateTime.o decode.o dlist.o encode.o errmgmt.o \
-	memheap.o ootrace.o oochannels.o ooh245.o ooports.o \
-	ooq931.o ooCapability.o ooSocket.o perutil.o eventHandler.o \
-	ooCalls.o ooStackCmds.o ooh323.o ooh323ep.o printHandler.o \
-	rtctype.o ooTimer.o h323/H235-SECURITY-MESSAGESDec.o \
-	h323/H235-SECURITY-MESSAGESEnc.o h323/H323-MESSAGES.o h323/H323-MESSAGESDec.o \
-	h323/H323-MESSAGESEnc.o h323/MULTIMEDIA-SYSTEM-CONTROL.o \
-	h323/MULTIMEDIA-SYSTEM-CONTROLDec.o h323/MULTIMEDIA-SYSTEM-CONTROLEnc.o
+OOH323C:=ooCmdChannel.c ooLogChan.c ooUtils.c ooGkClient.c context.c \
+	ooDateTime.c decode.c dlist.c encode.c errmgmt.c \
+	memheap.c ootrace.c oochannels.c ooh245.c ooports.c \
+	ooq931.c ooCapability.c ooSocket.c perutil.c eventHandler.c \
+	ooCalls.c ooStackCmds.c ooh323.c ooh323ep.c printHandler.c \
+	rtctype.c ooTimer.c h323/H235-SECURITY-MESSAGESDec.c \
+	h323/H235-SECURITY-MESSAGESEnc.c h323/H323-MESSAGES.c h323/H323-MESSAGESDec.c \
+	h323/H323-MESSAGESEnc.c h323/MULTIMEDIA-SYSTEM-CONTROL.c \
+	h323/MULTIMEDIA-SYSTEM-CONTROLDec.c h323/MULTIMEDIA-SYSTEM-CONTROLEnc.c
+
+H323SOURCE:=$(addprefix ooh323c/src/,$(OOH323C)) ooh323cDriver.c
 
 H323CFLAGS:=-Iooh323c/src -Iooh323c/src/h323
 
@@ -57,22 +59,14 @@ endif
 
 include $(ASTTOPDIR)/Makefile.moddir_rules
 
-clean::
-	if [ -f mp3/Makefile ] ; then $(MAKE) -C mp3 clean ; fi
-	rm -f $(addprefix ooh323c/src/,$(H323OBJS))
-
-ifneq ($(wildcard mp3/Makefile),)
-format_mp3.so: mp3/common.o mp3/dct64_i386.o mp3/decode_ntom.o mp3/layer3.o mp3/tabinit.o mp3/interface.o
-endif
-
 chan_mobile.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
-chan_ooh323.o: _ASTCFLAGS+=$(H323CFLAGS)
 chan_ooh323.so: _ASTCFLAGS+=$(H323CFLAGS)
-chan_ooh323.so: $(addprefix ooh323c/src/,$(H323OBJS)) chan_ooh323.o ooh323cDriver.o
-$(addprefix ooh323c/src/,$(H323OBJS)) chan_ooh323.o ooh323cDriver.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_ooh323)
+$(call MOD_ADD_C,chan_ooh323,$(H323SOURCE))
 
-.PHONY: check_mp3
+ifneq ($(wildcard mp3/Makefile),)
+$(call MOD_ADD_C,format_mp3,mp3/common.c mp3/dct64_i386.c mp3/decode_ntom.c mp3/layer3.c mp3/tabinit.c mp3/interface.c)
 
-ifeq ($(wildcard mp3/Makefile),)
-.PHONY: format_mp3.o format_mp3.so
+.PHONY: check_mp3
+else
+.PHONY: check_mp3 format_mp3.o format_mp3.so
 endif
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/context.c 1:16.1.1~dfsg-1/addons/ooh323c/src/context.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/context.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/context.c	2018-12-26 18:45:13.000000000 +0000
@@ -15,6 +15,7 @@
  *****************************************************************************/
 #include "asterisk.h"
 #include "asterisk/lock.h"
+#include "asterisk/utils.h"
 
 #include "ooasn1.h"
 #include <stdlib.h>
@@ -157,11 +158,11 @@ int setPERBuffer (OOCTXT* pctxt,
 OOCTXT* newContext ()
 {
    /* OOCTXT* pctxt = (OOCTXT*) ASN1CRTMALLOC0 (sizeof(OOCTXT)); */
-   OOCTXT* pctxt = (OOCTXT*) malloc (sizeof(OOCTXT));
+   OOCTXT* pctxt = ast_malloc(sizeof(OOCTXT));
    if (pctxt) {
       if (initContext(pctxt) != ASN_OK) {
          /* ASN1CRTFREE0 (pctxt); */
-	 free(pctxt);
+	 ast_free(pctxt);
          pctxt = 0;
          return (pctxt);
       }
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/errmgmt.c 1:16.1.1~dfsg-1/addons/ooh323c/src/errmgmt.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/errmgmt.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/errmgmt.c	2018-12-26 18:45:13.000000000 +0000
@@ -18,6 +18,7 @@
 
 #include "asterisk.h"
 #include "asterisk/lock.h"
+#include "asterisk/utils.h"
 #include <stdlib.h>
 #include "ooasn1.h"
 
@@ -89,7 +90,7 @@ int errAddStrParm (ASN1ErrInfo* pErrInfo
 #if defined(_NO_THREADS) || !defined(_NO_MALLOC)
    if (pErrInfo->parmcnt < ASN_K_MAXERRP) {
       /* char* tmpstr = (char*) ASN1CRTMALLOC0 (strlen(errprm_p)+1); */
-      char* tmpstr = (char*) malloc (strlen(errprm_p)+1);
+      char* tmpstr = ast_malloc(strlen(errprm_p) + 1);
       strcpy (tmpstr, errprm_p);
       pErrInfo->parms[pErrInfo->parmcnt] = tmpstr;
       pErrInfo->parmcnt++;
@@ -118,7 +119,7 @@ void errFreeParms (ASN1ErrInfo* pErrInfo
 
    for (i = 0; i < pErrInfo->parmcnt; i++)
       /* ASN1CRTFREE0 ((char*)pErrInfo->parms[i]); */
-      free ((char*)pErrInfo->parms[i]);
+      ast_free((char*)pErrInfo->parms[i]);
 #endif
 
    pErrInfo->parmcnt = 0;
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/memheap.c 1:16.1.1~dfsg-1/addons/ooh323c/src/memheap.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/memheap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/memheap.c	2018-12-26 18:45:13.000000000 +0000
@@ -15,6 +15,7 @@
  *****************************************************************************/
 #include "asterisk.h"
 #include "asterisk/lock.h"
+#include "asterisk/utils.h"
 #include <stdlib.h>
 #include "memheap.h"
 
@@ -176,13 +177,13 @@ void* memHeapAlloc (void** ppvMemHeap, i
 
       /* allocate raw block */
 
-      data = malloc (nbytes);
+      data = ast_malloc(nbytes);
       if (data == NULL) {
          return NULL;
       }
       pMemLink = memHeapAddBlock (ppMemLink, data, RTMEMMALLOC | RTMEMRAW);
       if (pMemLink == 0) {
-         free (data);
+         ast_free(data);
          return NULL;
       }
       /* save size of the RAW memory block behind the pMemLink */
@@ -333,7 +334,7 @@ void* memHeapAlloc (void** ppvMemHeap, i
             ((((ASN1UINT)dataUnits) * 8u) + sizeof (OSMemBlk));
       }
 
-      pmem = (ASN1OCTET*) malloc (allocSize + sizeof (OSMemLink));
+      pmem = (ASN1OCTET*) ast_malloc(allocSize + sizeof (OSMemLink));
       if (0 != pmem) {
          OSMemElemDescr* pElem;
 
@@ -357,7 +358,7 @@ void* memHeapAlloc (void** ppvMemHeap, i
 
          if (memHeapAddBlock (ppMemLink, pMemBlk, RTMEMSTD | RTMEMLINK) == 0)
          {
-            free (pmem);
+            ast_free(pmem);
 	    ast_mutex_unlock(&pMemHeap->pLock);
             return NULL;
          }
@@ -437,12 +438,12 @@ void memHeapFreePtr (void** ppvMemHeap,
          if ((pMemLink->blockType & RTMEMLINK) &&
              (pMemLink->blockType & RTMEMMALLOC))
          {
-            free (pMemLink);
+            ast_free(pMemLink);
          }
          else {
             if (pMemLink->blockType & RTMEMMALLOC)
-               free (pMemLink->pMemBlk);
-            free (pMemLink);
+               ast_free(pMemLink->pMemBlk);
+            ast_free(pMemLink);
          }
 	 ast_mutex_unlock(&pMemHeap->pLock);
          return;
@@ -553,11 +554,11 @@ void memHeapFreePtr (void** ppvMemHeap,
             FILLFREEMEM (pMemBlk->plink, sizeof (*pMemBlk->plink));
             FILLFREEMEM (pMemBlk->data, (pMemBlk->nunits * 8u));
 
-            free (pMemBlk->plink);
+            ast_free(pMemBlk->plink);
 
             if (!(blockType & RTMEMLINK)) {
                FILLFREEMEM (pMemBlk, sizeof (*pMemBlk));
-               free (pMemBlk);
+               ast_free(pMemBlk);
             }
             RTMEMDIAG2 ("memHeapFreePtr: pMemBlk = 0x%x was freed\n",
                              pMemBlk);
@@ -748,7 +749,7 @@ void* memHeapRealloc (void** ppvMemHeap,
            pMemLink->pMemBlk == mem_p)
       {
          if (pMemLink->blockType & RTMEMMALLOC) {
-             void *newMemBlk = realloc (pMemLink->pMemBlk, nbytes_);
+             void *newMemBlk = ast_realloc(pMemLink->pMemBlk, nbytes_);
              if (newMemBlk == 0)
                 return 0;
              pMemLink->pMemBlk = newMemBlk;
@@ -1038,8 +1039,8 @@ void memHeapFreeAll (void** ppvMemHeap)
          if (((pMemLink2->blockType & RTMEMSTD) ||
               (pMemLink2->blockType & RTMEMMALLOC)) &&
               !(pMemLink2->blockType & RTMEMLINK))
-            free (pMemLink2->pMemBlk);
-         free (pMemLink2);
+            ast_free(pMemLink2->pMemBlk);
+         ast_free(pMemLink2);
       }
    }
    ast_mutex_unlock(&pMemHeap->pLock);
@@ -1075,13 +1076,13 @@ void memHeapRelease (void** ppvMemHeap)
          pMemLink2 = pMemLink;
          pMemLink  = pMemLink2->pnext;
 
-         free (pMemLink2);
+         ast_free(pMemLink2);
       }
 
       if ((*ppMemHeap)->flags & RT_MH_FREEHEAPDESC) {
          pMemHeap = *ppMemHeap;
          ast_mutex_destroy(&pMemHeap->pLock);
-         free (*ppMemHeap);
+         ast_free(*ppMemHeap);
       }
       *ppMemHeap = 0;
    }
@@ -1216,8 +1217,7 @@ static OSMemLink* memHeapAddBlock (OSMem
    if (blockType & RTMEMLINK)
       pMemLink = (OSMemLink*) (((ASN1OCTET*)pMemBlk) - sizeof (OSMemLink));
    else {
-      pMemLink = (OSMemLink*) malloc (
-         sizeof(OSMemLink) + sizeof (int));
+      pMemLink = ast_malloc(sizeof(OSMemLink) + sizeof(int));
       if (pMemLink == 0) return 0;
       /* An extra integer is necessary to save a size of a RAW memory block
          to perform rtMemRealloc through malloc/memcpy/free */
@@ -1340,7 +1340,7 @@ int memHeapCreate (void** ppvMemHeap)
    OSMemHeap* pMemHeap;
    if (ppvMemHeap == 0) return ASN_E_INVPARAM;
 
-   pMemHeap = (OSMemHeap*) malloc (sizeof (OSMemHeap));
+   pMemHeap = ast_malloc(sizeof (OSMemHeap));
    if (pMemHeap == NULL) return ASN_E_NOMEM;
    memset (pMemHeap, 0, sizeof (OSMemHeap));
    pMemHeap->defBlkSize = g_defBlkSize;
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooasn1.h 1:16.1.1~dfsg-1/addons/ooh323c/src/ooasn1.h
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooasn1.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooasn1.h	2018-12-26 18:45:13.000000000 +0000
@@ -664,7 +664,7 @@ EXTERN int setPERBufferUsingCtxt (OOCTXT
 /**
  * This function adds an integer parameter to an error information structure.
  * Parameter substitution is done in much the same way as it is done in C
- * printf statments. The base error message specification that goes along with
+ * printf statements. The base error message specification that goes along with
  * a particular status code may have variable fields built in using '%'
  * modifiers. These would be replaced with actual parameter data.
  *
@@ -680,7 +680,7 @@ EXTERN int errAddIntParm (ASN1ErrInfo* p
 /**
  * This function adds an string parameter to an error information structure.
  * Parameter substitution is done in much the same way as it is done in C
- * printf statments. The base error message specification that goes along with
+ * printf statements. The base error message specification that goes along with
  * a particular status code may have variable fields built in using '%'
  * modifiers. These would be replaced with actual parameter data.
  *
@@ -697,7 +697,7 @@ EXTERN int errAddStrParm (ASN1ErrInfo* p
 /**
  * This function adds an unsigned integer parameter to an error information
  * structure. Parameter substitution is done in much the same way as it is done
- * in C printf statments. The base error message specification that goes along
+ * in C printf statements. The base error message specification that goes along
  * with a particular status code may have variable fields built in using '%'
  * modifiers. These would be replaced with actual parameter data.
  *
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooCalls.c 1:16.1.1~dfsg-1/addons/ooh323c/src/ooCalls.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooCalls.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooCalls.c	2018-12-26 18:45:13.000000000 +0000
@@ -16,6 +16,7 @@
 
 #include "asterisk.h"
 #include "asterisk/lock.h"
+#include "asterisk/utils.h"
 
 #include "ootrace.h"
 #include "ootypes.h"
@@ -369,13 +370,13 @@ int ooCleanCall(OOH323CallData *call)
 	if ((call->rtpMask->inuse) == 0) {
 		regfree(&call->rtpMask->regex);
 		ast_mutex_destroy(&call->rtpMask->lock);
-		free(call->rtpMask);
+		ast_free(call->rtpMask);
 	}
    }
 
    if ((pctxt = call->msgctxt) != NULL) {
    	freeContext(pctxt);
-   	free(pctxt);
+   	ast_free(pctxt);
    	call->msgctxt = NULL;
    }
 /* May !!!! Fix it !! */
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/oochannels.c 1:16.1.1~dfsg-1/addons/ooh323c/src/oochannels.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/oochannels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/oochannels.c	2018-12-26 18:45:13.000000000 +0000
@@ -959,7 +959,7 @@ int ooMonitorCallChannels(OOH323CallData
    ast_cond_destroy(&call->gkWait);
    pctxt = call->pctxt;
    freeContext(pctxt);
-   free(pctxt);
+   ast_free(pctxt);
 
    return OO_OK;
 }
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooCmdChannel.c 1:16.1.1~dfsg-1/addons/ooh323c/src/ooCmdChannel.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooCmdChannel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooCmdChannel.c	2018-12-26 18:45:13.000000000 +0000
@@ -57,13 +57,13 @@ int ooCreateCallCmdConnection(OOH323Call
 
     OOTRACEINFO2("INFO: create cmd connect for call: %lx\n", call);
 
-   call->CmdChanLock = calloc(1, sizeof(ast_mutex_t));
+   call->CmdChanLock = ast_calloc(1, sizeof(ast_mutex_t));
    ast_mutex_init(call->CmdChanLock);
 
 
    if ((ret = socketpair(PF_LOCAL, SOCK_STREAM, 0, thePipe)) == -1) {
       ast_mutex_destroy(call->CmdChanLock);
-      free(call->CmdChanLock);
+      ast_free(call->CmdChanLock);
       call->CmdChanLock = NULL;
       return OO_FAILED;
    }
@@ -94,7 +94,7 @@ int ooCloseCallCmdConnection(OOH323CallD
    call->CmdChan = 0;
    ast_mutex_unlock(call->CmdChanLock);
    ast_mutex_destroy(call->CmdChanLock);
-   free(call->CmdChanLock);
+   ast_free(call->CmdChanLock);
    call->CmdChanLock = NULL;
 
    return OO_OK;
@@ -267,9 +267,9 @@ int ooReadAndProcessStackCommand()
             default: OOTRACEERR1("ERROR:Unknown command\n");
          }
       }
-      if(cmd.param1) free(cmd.param1);
-      if(cmd.param2) free(cmd.param2);
-      if(cmd.param3) free(cmd.param3);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
+      ast_free(cmd.param3);
    }
 
 
@@ -302,7 +302,7 @@ int ooReadAndProcessCallStackCommand(OOH
       bPoint +=  sizeof(OOStackCommand);
 
       if (cmd.plen1 > 0) {
-	cmd.param1 = malloc(cmd.plen1 + 1);
+	cmd.param1 = ast_malloc(cmd.plen1 + 1);
 	if (!cmd.param1)
 		return OO_FAILED;
 	memset(cmd.param1, 0, cmd.plen1 + 1);
@@ -311,7 +311,7 @@ int ooReadAndProcessCallStackCommand(OOH
       }
 
       if (cmd.plen2 > 0) {
-	cmd.param2 = malloc(cmd.plen2 + 1);
+	cmd.param2 = ast_malloc(cmd.plen2 + 1);
 	if (!cmd.param2)
 		return OO_FAILED;
 	memset(cmd.param2, 0, cmd.plen2 + 1);
@@ -320,7 +320,7 @@ int ooReadAndProcessCallStackCommand(OOH
       }
 
       if (cmd.plen3 > 0) {
-	cmd.param3 = malloc(cmd.plen3 + 1);
+	cmd.param3 = ast_malloc(cmd.plen3 + 1);
 	if (!cmd.param3)
 		return OO_FAILED;
 	memset(cmd.param3, 0, cmd.plen3 + 1);
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooGkClient.c 1:16.1.1~dfsg-1/addons/ooh323c/src/ooGkClient.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooGkClient.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooGkClient.c	2018-12-26 18:45:13.000000000 +0000
@@ -49,7 +49,7 @@ static ASN1OBJID gProtocolID = {
 };
 
 int ooGkClientInit(enum RasGatekeeperMode eGkMode,
-              char *szGkAddr, int iGkPort )
+              char *szGkAddr, char *szRasAddr, int iGkPort )
 {
    ooGkClient *pGkClient=NULL;
    OOInterface *cur=NULL;
@@ -69,7 +69,11 @@ int ooGkClientInit(enum RasGatekeeperMod
    pGkClient->rrqRetries = 0;
    pGkClient->grqRetries = 0;
 
-   strcpy(pGkClient->localRASIP, gH323ep.signallingIP);
+   if (szRasAddr && *szRasAddr) {
+      strcpy(pGkClient->localRASIP, szRasAddr);
+   } else {
+      strcpy(pGkClient->localRASIP, gH323ep.signallingIP);
+   }
 #ifndef _WIN32
    if(!strcmp(pGkClient->localRASIP, "0.0.0.0") ||
       !strcmp(pGkClient->localRASIP, "127.0.0.1"))
@@ -90,7 +94,6 @@ int ooGkClientInit(enum RasGatekeeperMod
       }
       if(cur)
       {
-         OOTRACEINFO2("Using local RAS Ip address %s\n", cur->addr);
          strcpy(pGkClient->localRASIP, cur->addr);
       }
       else{
@@ -98,6 +101,7 @@ int ooGkClientInit(enum RasGatekeeperMod
          return OO_FAILED;
       }
    }
+   OOTRACEINFO2("Using local RAS Ip address %s\n", pGkClient->localRASIP);
 #endif
    if(OO_OK != ooGkClientSetGkMode(pGkClient, eGkMode, szGkAddr, iGkPort))
    {
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooGkClient.h 1:16.1.1~dfsg-1/addons/ooh323c/src/ooGkClient.h
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooGkClient.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooGkClient.h	2018-12-26 18:45:13.000000000 +0000
@@ -223,7 +223,7 @@ struct OOH323CallData;
  *
  */
 EXTERN int ooGkClientInit
-   (enum RasGatekeeperMode eGkMode, char *szGkAddr, int iGkPort );
+   (enum RasGatekeeperMode eGkMode, char *szGkAddr, char *szRasAddr, int iGkPort );
 
 /**
  * This function is used to print the gatekeeper client configuration
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooSocket.c 1:16.1.1~dfsg-1/addons/ooh323c/src/ooSocket.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooSocket.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooSocket.c	2018-12-26 18:45:13.000000000 +0000
@@ -15,8 +15,6 @@
  *****************************************************************************/
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #ifdef SOLARIS
 #include <sys/sockio.h>
 #endif
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/ooStackCmds.c 1:16.1.1~dfsg-1/addons/ooh323c/src/ooStackCmds.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/ooStackCmds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/ooStackCmds.c	2018-12-26 18:45:13.000000000 +0000
@@ -79,7 +79,7 @@ OOStkCmdStat ooMakeCall
 
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_MAKECALL;
-   cmd.param1 = (void*) malloc(strlen(dest)+1);
+   cmd.param1 = ast_malloc(strlen(dest)+1);
    if(!cmd.param1)
    {
       return OO_STKCMD_MEMERR;
@@ -87,10 +87,10 @@ OOStkCmdStat ooMakeCall
    strcpy((char*)cmd.param1, dest);
 
 
-   cmd.param2 = (void*) malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(strlen(callToken)+1);
    if(!cmd.param2)
    {
-      free(cmd.param1);
+      ast_free(cmd.param1);
       return OO_STKCMD_MEMERR;
    }
 
@@ -101,11 +101,11 @@ OOStkCmdStat ooMakeCall
       cmd.param3 = 0;
    }
    else {
-      cmd.param3 = (void*) malloc(sizeof(ooCallOptions));
+      cmd.param3 = ast_malloc(sizeof(ooCallOptions));
       if(!cmd.param3)
       {
-         free(cmd.param1);
-         free(cmd.param2);
+         ast_free(cmd.param1);
+         ast_free(cmd.param2);
          return OO_STKCMD_MEMERR;
       }
       memcpy((void*)cmd.param3, opts, sizeof(ooCallOptions));
@@ -113,9 +113,9 @@ OOStkCmdStat ooMakeCall
 
    if(ooWriteStackCommand(&cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
-      if(cmd.param3) free(cmd.param3);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
+      if(cmd.param3) ast_free(cmd.param3);
       return OO_STKCMD_WRITEERR;
    }
 
@@ -149,7 +149,7 @@ OOStkCmdStat ooRunCall
 
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_MAKECALL;
-   cmd.param1 = (void*) malloc(strlen(dest)+1);
+   cmd.param1 = ast_malloc(strlen(dest)+1);
    if(!cmd.param1)
    {
       return OO_STKCMD_MEMERR;
@@ -158,10 +158,10 @@ OOStkCmdStat ooRunCall
    cmd.plen1 = strlen(dest);
 
 
-   cmd.param2 = (void*) malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(strlen(callToken)+1);
    if(!cmd.param2)
    {
-      free(cmd.param1);
+      ast_free(cmd.param1);
       return OO_STKCMD_MEMERR;
    }
 
@@ -173,11 +173,11 @@ OOStkCmdStat ooRunCall
       cmd.param3 = 0;
    }
    else {
-      cmd.param3 = (void*) malloc(sizeof(ooCallOptions));
+      cmd.param3 = ast_malloc(sizeof(ooCallOptions));
       if(!cmd.param3)
       {
-         free(cmd.param1);
-         free(cmd.param2);
+         ast_free(cmd.param1);
+         ast_free(cmd.param2);
          return OO_STKCMD_MEMERR;
       }
       memcpy((void*)cmd.param3, opts, sizeof(ooCallOptions));
@@ -186,16 +186,16 @@ OOStkCmdStat ooRunCall
 
    if(ooWriteCallStackCommand(call, &cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
-      if(cmd.param3) free(cmd.param3);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
+      if(cmd.param3) ast_free(cmd.param3);
       return OO_STKCMD_WRITEERR;
    }
 
 
-   free(cmd.param1);
-   free(cmd.param2);
-   if(cmd.param3) free(cmd.param3);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
+   if(cmd.param3) ast_free(cmd.param3);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -223,7 +223,7 @@ OOStkCmdStat ooManualRingback(const char
 
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_MANUALRINGBACK;
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
    if(!cmd.param1)
    {
       return OO_STKCMD_MEMERR;
@@ -233,11 +233,11 @@ OOStkCmdStat ooManualRingback(const char
 
    if(ooWriteCallStackCommand(call,&cmd) != OO_OK)
    {
-      free(cmd.param1);
+      ast_free(cmd.param1);
       return OO_STKCMD_WRITEERR;
    }
 
-   free(cmd.param1);
+   ast_free(cmd.param1);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -267,7 +267,7 @@ OOStkCmdStat ooManualProgress(const char
 
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_MANUALPROGRESS;
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
    if(!cmd.param1)
    {
       return OO_STKCMD_MEMERR;
@@ -277,11 +277,11 @@ OOStkCmdStat ooManualProgress(const char
 
    if(ooWriteCallStackCommand(call, &cmd) != OO_OK)
    {
-      free(cmd.param1);
+      ast_free(cmd.param1);
       return OO_STKCMD_WRITEERR;
    }
 
-   free(cmd.param1);
+   ast_free(cmd.param1);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -309,7 +309,7 @@ OOStkCmdStat ooAnswerCall(const char *ca
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_ANSCALL;
 
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
    if(!cmd.param1)
    {
       return OO_STKCMD_MEMERR;
@@ -319,11 +319,11 @@ OOStkCmdStat ooAnswerCall(const char *ca
 
    if(ooWriteCallStackCommand(call, &cmd) != OO_OK)
    {
-      free(cmd.param1);
+      ast_free(cmd.param1);
       return OO_STKCMD_WRITEERR;
    }
 
-   free(cmd.param1);
+   ast_free(cmd.param1);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -350,12 +350,12 @@ OOStkCmdStat ooForwardCall(const char* c
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_FWDCALL;
 
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
-   cmd.param2 = (void*) malloc(strlen(dest)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(strlen(dest)+1);
    if(!cmd.param1 || !cmd.param2)
    {
-      if(cmd.param1)   free(cmd.param1);  /* Release memory */
-      if(cmd.param2)   free(cmd.param2);
+      if(cmd.param1)   ast_free(cmd.param1);  /* Release memory */
+      if(cmd.param2)   ast_free(cmd.param2);
       return OO_STKCMD_MEMERR;
    }
    strcpy((char*)cmd.param1, callToken);
@@ -365,12 +365,12 @@ OOStkCmdStat ooForwardCall(const char* c
 
    if(ooWriteCallStackCommand(call, &cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
       return OO_STKCMD_WRITEERR;
    }
-   free(cmd.param1);
-   free(cmd.param2);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -399,14 +399,14 @@ OOStkCmdStat ooHangCall(const char* call
 
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_HANGCALL;
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
-   cmd.param2 = (void*) malloc(sizeof(OOCallClearReason));
-   cmd.param3 = (void*) malloc(sizeof(int));
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(sizeof(OOCallClearReason));
+   cmd.param3 = ast_malloc(sizeof(int));
    if(!cmd.param1 || !cmd.param2 || !cmd.param3)
    {
-      if(cmd.param1)   free(cmd.param1); /* Release memory */
-      if(cmd.param2)   free(cmd.param2);
-      if(cmd.param3)   free(cmd.param3);
+      if(cmd.param1)   ast_free(cmd.param1); /* Release memory */
+      if(cmd.param2)   ast_free(cmd.param2);
+      if(cmd.param3)   ast_free(cmd.param3);
       return OO_STKCMD_MEMERR;
    }
    strcpy((char*)cmd.param1, callToken);
@@ -418,14 +418,14 @@ OOStkCmdStat ooHangCall(const char* call
 
    if(ooWriteCallStackCommand(call, &cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
-      free(cmd.param3);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
+      ast_free(cmd.param3);
       return OO_STKCMD_WRITEERR;
    }
-   free(cmd.param1);
-   free(cmd.param2);
-   free(cmd.param3);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
+   ast_free(cmd.param3);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -473,12 +473,12 @@ OOStkCmdStat ooSendDTMFDigit(const char
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_SENDDIGIT;
 
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
-   cmd.param2 = (void*) malloc(strlen(dtmf)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(strlen(dtmf)+1);
    if(!cmd.param1 || !cmd.param2)
    {
-      if(cmd.param1)   free(cmd.param1); /* Release memory */
-      if(cmd.param2)   free(cmd.param2);
+      if(cmd.param1)   ast_free(cmd.param1); /* Release memory */
+      if(cmd.param2)   ast_free(cmd.param2);
       return OO_STKCMD_MEMERR;
    }
    strcpy((char*)cmd.param1, callToken);
@@ -488,12 +488,12 @@ OOStkCmdStat ooSendDTMFDigit(const char
 
    if(ooWriteCallStackCommand(call,&cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
       return OO_STKCMD_WRITEERR;
    }
-   free(cmd.param1);
-   free(cmd.param2);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -521,12 +521,12 @@ OOStkCmdStat ooSetANI(const char *callTo
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_SETANI;
 
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
-   cmd.param2 = (void*) malloc(strlen(ani)+1);
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(strlen(ani)+1);
    if(!cmd.param1 || !cmd.param2)
    {
-      if(cmd.param1)   free(cmd.param1); /* Release memory */
-      if(cmd.param2)   free(cmd.param2);
+      if(cmd.param1)   ast_free(cmd.param1); /* Release memory */
+      if(cmd.param2)   ast_free(cmd.param2);
       return OO_STKCMD_MEMERR;
    }
    strcpy((char*)cmd.param1, callToken);
@@ -536,12 +536,12 @@ OOStkCmdStat ooSetANI(const char *callTo
 
    if(ooWriteCallStackCommand(call,&cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
       return OO_STKCMD_WRITEERR;
    }
-   free(cmd.param1);
-   free(cmd.param2);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
 
    return OO_STKCMD_SUCCESS;
 }
@@ -631,12 +631,12 @@ OOStkCmdStat ooRequestChangeMode(const c
    memset(&cmd, 0, sizeof(OOStackCommand));
    cmd.type = OO_CMD_REQMODE;
 
-   cmd.param1 = (void*) malloc(strlen(callToken)+1);
-   cmd.param2 = (void*) malloc(sizeof(int));
+   cmd.param1 = ast_malloc(strlen(callToken)+1);
+   cmd.param2 = ast_malloc(sizeof(int));
    if(!cmd.param1 || !cmd.param2)
    {
-      if(cmd.param1)   free(cmd.param1); /* Release memory */
-      if(cmd.param2)   free(cmd.param2);
+      ast_free(cmd.param1); /* Release memory */
+      ast_free(cmd.param2);
       return OO_STKCMD_MEMERR;
    }
    strcpy((char*)cmd.param1, callToken);
@@ -646,12 +646,12 @@ OOStkCmdStat ooRequestChangeMode(const c
 
    if(ooWriteCallStackCommand(call,&cmd) != OO_OK)
    {
-      free(cmd.param1);
-      free(cmd.param2);
+      ast_free(cmd.param1);
+      ast_free(cmd.param2);
       return OO_STKCMD_WRITEERR;
    }
-   free(cmd.param1);
-   free(cmd.param2);
+   ast_free(cmd.param1);
+   ast_free(cmd.param2);
 
    return OO_STKCMD_SUCCESS;
 }
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323c/src/printHandler.c 1:16.1.1~dfsg-1/addons/ooh323c/src/printHandler.c
--- 1:13.23.1~dfsg-2/addons/ooh323c/src/printHandler.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323c/src/printHandler.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,6 +19,7 @@
 */
 #include "asterisk.h"
 #include "asterisk/lock.h"
+#include "asterisk/utils.h"
 
 #include <stdlib.h>
 /* #ifdef HAVE_MALLOC_H
@@ -126,10 +127,10 @@ void printBitStrValue (ASN1UINT numbits,
    indent ();
    OOTRACEDBGB2("%s\n", bitStrToString (numbits, data, s, numbits+8));
 #else
-   char* s = (char*)malloc(numbits + 8);
+   char* s = ast_malloc(numbits + 8);
    indent ();
    OOTRACEDBGB2("%s\n", bitStrToString (numbits, data, s, numbits+8));
-   free(s);
+   ast_free(s);
 #endif
 }
 
@@ -141,10 +142,10 @@ void printOctStrValue (ASN1UINT numocts,
    indent ();
    OOTRACEDBGB2 ("%s\n", octStrToString (numocts, data, s, bufsiz));
 #else
-   char* s = (char*)malloc(bufsiz);
+   char* s = ast_malloc(bufsiz);
    indent ();
    OOTRACEDBGB2 ("%s\n", octStrToString (numocts, data, s, bufsiz));
-   free(s);
+   ast_free(s);
 #endif
 }
 
diff -pruN 1:13.23.1~dfsg-2/addons/ooh323cDriver.c 1:16.1.1~dfsg-1/addons/ooh323cDriver.c
--- 1:13.23.1~dfsg-2/addons/ooh323cDriver.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/ooh323cDriver.c	2018-12-26 18:45:13.000000000 +0000
@@ -121,7 +121,7 @@ void* ooh323c_call_thread(void* dummy)
 
  close(mycthread->thePipe[0]);
  close(mycthread->thePipe[1]);
- free(mycthread);
+ ast_free(mycthread);
  ast_module_unref(myself);
  ast_update_use_count();
  return NULL;
@@ -155,7 +155,7 @@ int ooh323c_start_call_thread(ooCallData
 	ast_module_ref(myself);
 	if ((socketpair(PF_LOCAL, SOCK_STREAM, 0, cur->thePipe)) == -1) {
 		ast_log(LOG_ERROR, "Can't create thread pipe for call %s\n", call->callToken);
-		free(cur);
+		ast_free(cur);
 		return -1;
 	}
 	cur->inUse = TRUE;
@@ -173,7 +173,7 @@ int ooh323c_start_call_thread(ooCallData
 		ast_mutex_destroy(&cur->lock);
 		close(cur->thePipe[0]);
 		close(cur->thePipe[1]);
-		free(cur);
+		ast_free(cur);
   		return -1;
  	}
 
diff -pruN 1:13.23.1~dfsg-2/addons/res_config_mysql.c 1:16.1.1~dfsg-1/addons/res_config_mysql.c
--- 1:13.23.1~dfsg-2/addons/res_config_mysql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/addons/res_config_mysql.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include <mysql/mysql.h>
@@ -1556,9 +1554,10 @@ static char *handle_cli_realtime_mysql_s
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "MySQL RealTime Configuration Driver",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_adsiprog.c 1:16.1.1~dfsg-1/apps/app_adsiprog.c
--- 1:13.23.1~dfsg-2/apps/app_adsiprog.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_adsiprog.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,13 +36,11 @@
 
 /*** MODULEINFO
 	<depend>res_adsi</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <netinet/in.h>
 #include <ctype.h>
 
@@ -1612,8 +1610,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Asterisk ADSI Programming Application",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.nonoptreq = "res_adsi",
-		);
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_adsi",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_agent_pool.c 1:16.1.1~dfsg-1/apps/app_agent_pool.c
--- 1:13.23.1~dfsg-2/apps/app_agent_pool.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_agent_pool.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/cli.h"
 #include "asterisk/app.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_alarmreceiver.c 1:16.1.1~dfsg-1/apps/app_alarmreceiver.c
--- 1:13.23.1~dfsg-2/apps/app_alarmreceiver.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_alarmreceiver.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 #include <sys/wait.h>
 #include <sys/time.h>
@@ -995,8 +993,8 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Alarm Receiver for Asterisk",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
 );
diff -pruN 1:13.23.1~dfsg-2/apps/app_amd.c 1:16.1.1~dfsg-1/apps/app_amd.c
--- 1:13.23.1~dfsg-2/apps/app_amd.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_amd.c	2018-12-26 18:45:13.000000000 +0000
@@ -43,8 +43,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
@@ -85,7 +83,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			</parameter>
 			<parameter name="maximumNumberOfWords" required="false">
 				<para>Is the maximum number of words in a greeting</para>
-				<para>If this is REACHED, then the result is detection as a MACHINE</para>
+				<para>If this is exceeded, then the result is detection as a MACHINE</para>
 			</parameter>
 			<parameter name="silenceThreshold" required="false">
 				<para>What is the average level of noise from 0 to 32767 which if not exceeded, should be considered silence?</para>
@@ -154,7 +152,7 @@ static int dfltAfterGreetingSilence = 80
 static int dfltTotalAnalysisTime    = 5000;
 static int dfltMinimumWordLength    = 100;
 static int dfltBetweenWordsSilence  = 50;
-static int dfltMaximumNumberOfWords = 3;
+static int dfltMaximumNumberOfWords = 2;
 static int dfltSilenceThreshold     = 256;
 static int dfltMaximumWordLength    = 5000; /* Setting this to a large default so it is not used unless specify it in the configs or command line */
 
@@ -380,7 +378,7 @@ static void isAnsweringMachine(struct as
 					sprintf(amdCause , "MAXWORDLENGTH-%d", consecutiveVoiceDuration);
 					break;
 				}
-				if (iWordsCount >= maximumNumberOfWords) {
+				if (iWordsCount > maximumNumberOfWords) {
 					ast_verb(3, "AMD: Channel [%s]. ANSWERING MACHINE: iWordsCount:%d\n", ast_channel_name(chan), iWordsCount);
 					ast_frfree(f);
 					strcpy(amdStatus , "MACHINE");
@@ -541,8 +539,8 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Answering Machine Detection Application",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
 );
diff -pruN 1:13.23.1~dfsg-2/apps/app_authenticate.c 1:16.1.1~dfsg-1/apps/app_authenticate.c
--- 1:13.23.1~dfsg-2/apps/app_authenticate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_authenticate.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_bridgeaddchan.c 1:16.1.1~dfsg-1/apps/app_bridgeaddchan.c
--- 1:13.23.1~dfsg-2/apps/app_bridgeaddchan.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_bridgeaddchan.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,122 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Alec Davis <sivad.a@paradise.net.nz>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Application to place the channel into an existing Bridge
+ *
+ * \author Alec Davis
+ *
+ * \ingroup applications
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/file.h"
+#include "asterisk/module.h"
+#include "asterisk/channel.h"
+#include "asterisk/bridge.h"
+#include "asterisk/features.h"
+
+/*** DOCUMENTATION
+	<application name="BridgeAdd" language="en_US">
+		<synopsis>
+			Join a bridge that contains the specified channel.
+		</synopsis>
+		<syntax>
+			<parameter name="name">
+				<para>Name of the channel in an existing bridge
+				</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>This application places the incoming channel into
+			the bridge containing the specified channel. The specified
+			channel only needs to be the prefix of a full channel name
+			IE. 'SIP/cisco0001'.
+			</para>
+		</description>
+	</application>
+ ***/
+
+static const char app[] = "BridgeAdd";
+
+static int bridgeadd_exec(struct ast_channel *chan, const char *data)
+{
+	struct ast_channel *c_ref;
+	struct ast_bridge_features chan_features;
+	struct ast_bridge *bridge;
+	char *c_name;
+
+	/* Answer the channel if needed */
+	if (ast_channel_state(chan) != AST_STATE_UP) {
+		ast_answer(chan);
+	}
+
+	if (!(c_ref = ast_channel_get_by_name_prefix(data, strlen(data)))) {
+		ast_log(LOG_WARNING, "Channel %s not found\n", data);
+		return -1;
+	}
+
+	c_name = ast_strdupa(ast_channel_name(c_ref));
+
+	ast_channel_lock(c_ref);
+	bridge = ast_channel_get_bridge(c_ref);
+	ast_channel_unlock(c_ref);
+
+	ast_channel_unref(c_ref);
+
+	if (!bridge) {
+		ast_log(LOG_WARNING, "Channel %s is not in a bridge\n", c_name);
+		return -1;
+	}
+
+	ast_verb(3, "%s is joining %s in bridge %s\n", ast_channel_name(chan),
+		c_name, bridge->uniqueid);
+
+	if (ast_bridge_features_init(&chan_features)
+		|| ast_bridge_join(bridge, chan, NULL, &chan_features, NULL, 0)) {
+
+		ast_log(LOG_WARNING, "%s failed to join %s in bridge %s\n", ast_channel_name(chan),
+			 c_name, bridge->uniqueid);
+
+		ast_bridge_features_cleanup(&chan_features);
+		ao2_cleanup(bridge);
+		return -1;
+	}
+
+	ast_bridge_features_cleanup(&chan_features);
+	ao2_cleanup(bridge);
+	return 0;
+}
+
+static int unload_module(void)
+{
+	return ast_unregister_application(app);
+}
+
+static int load_module(void)
+{
+	return ast_register_application_xml(app, bridgeadd_exec);
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Bridge Add Channel Application");
diff -pruN 1:13.23.1~dfsg-2/apps/app_bridgewait.c 1:16.1.1~dfsg-1/apps/app_bridgewait.c
--- 1:13.23.1~dfsg-2/apps/app_bridgewait.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_bridgewait.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_cdr.c 1:16.1.1~dfsg-1/apps/app_cdr.c
--- 1:13.23.1~dfsg-2/apps/app_cdr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_cdr.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_celgenuserevent.c 1:16.1.1~dfsg-1/apps/app_celgenuserevent.c
--- 1:13.23.1~dfsg-2/apps/app_celgenuserevent.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_celgenuserevent.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/app.h"
 #include "asterisk/channel.h"
@@ -102,7 +100,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Generate an User-Defined CEL event",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-				.load = load_module,
-				.unload = unload_module,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "cel",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_chanisavail.c 1:16.1.1~dfsg-1/apps/app_chanisavail.c
--- 1:13.23.1~dfsg-2/apps/app_chanisavail.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_chanisavail.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/ioctl.h>
 
 #include "asterisk/lock.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_channelredirect.c 1:16.1.1~dfsg-1/apps/app_channelredirect.c
--- 1:13.23.1~dfsg-2/apps/app_channelredirect.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_channelredirect.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_chanspy.c 1:16.1.1~dfsg-1/apps/app_chanspy.c
--- 1:13.23.1~dfsg-2/apps/app_chanspy.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_chanspy.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <errno.h>
 
@@ -117,6 +115,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						either a single group or a colon-delimited list of groups, such
 						as <literal>sales:support:accounting</literal>.</para></note>
 					</option>
+					<option name="l">
+						<para>Allow usage of a long queue to store audio frames.</para>
+						<note><para>This may introduce some delay in the received audio feed, but will improve the audio quality.</para></note>
+					</option>
 					<option name="n" argsep="@">
 						<para>Say the name of the person being spied on if that person has recorded
 						his/her name. If a context is specified, then that voicemail context will
@@ -262,6 +264,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						either a single group or a colon-delimited list of groups, such
 						as <literal>sales:support:accounting</literal>.</para></note>
 					</option>
+					<option name="l">
+						<para>Allow usage of a long queue to store audio frames.</para>
+						<note><para>This may introduce some delay in the received audio feed, but will improve the audio quality.</para></note>
+					</option>
 					<option name="n" argsep="@">
 						<para>Say the name of the person being spied on if that person has recorded
 						his/her name. If a context is specified, then that voicemail context will
@@ -386,6 +392,7 @@ enum {
 	OPTION_STOP              = (1 << 17),
 	OPTION_EXITONHANGUP      = (1 << 18),   /* Hang up when the spied-on channel hangs up. */
 	OPTION_UNIQUEID          = (1 << 19),	/* The chanprefix is a channel uniqueid or fully specified channel name. */
+	OPTION_LONG_QUEUE        = (1 << 20),	/* Allow usage of a long queue to store audio frames. */
 };
 
 enum {
@@ -407,6 +414,7 @@ AST_APP_OPTIONS(spy_opts, {
 	AST_APP_OPTION_ARG('e', OPTION_ENFORCED, OPT_ARG_ENFORCED),
 	AST_APP_OPTION('E', OPTION_EXITONHANGUP),
 	AST_APP_OPTION_ARG('g', OPTION_GROUP, OPT_ARG_GROUP),
+	AST_APP_OPTION('l', OPTION_LONG_QUEUE),
 	AST_APP_OPTION_ARG('n', OPTION_NAME, OPT_ARG_NAME),
 	AST_APP_OPTION('o', OPTION_READONLY),
 	AST_APP_OPTION('q', OPTION_QUIET),
@@ -496,14 +504,23 @@ static struct ast_generator spygen = {
 	.generate = spy_generate,
 };
 
-static int start_spying(struct ast_autochan *autochan, const char *spychan_name, struct ast_audiohook *audiohook)
+static int start_spying(struct ast_autochan *autochan, const char *spychan_name, struct ast_audiohook *audiohook, struct ast_flags *flags)
 {
 	int res;
 
 	ast_autochan_channel_lock(autochan);
 	ast_log(LOG_NOTICE, "Attaching %s to %s\n", spychan_name, ast_channel_name(autochan->chan));
 
-	ast_set_flag(audiohook, AST_AUDIOHOOK_TRIGGER_SYNC | AST_AUDIOHOOK_SMALL_QUEUE);
+	if (ast_test_flag(flags, OPTION_READONLY)) {
+		ast_set_flag(audiohook, AST_AUDIOHOOK_MUTE_WRITE);
+	} else {
+		ast_set_flag(audiohook, AST_AUDIOHOOK_TRIGGER_SYNC);
+	}
+	if (ast_test_flag(flags, OPTION_LONG_QUEUE)) {
+		ast_debug(9, "Using a long queue to store audio frames in spy audiohook\n");
+	} else {
+		ast_set_flag(audiohook, AST_AUDIOHOOK_SMALL_QUEUE);
+	}
 	res = ast_audiohook_attach(autochan->chan, audiohook);
 	ast_autochan_channel_unlock(autochan);
 	return res;
@@ -586,7 +603,7 @@ static void publish_chanspy_message(stru
 
 static int attach_barge(struct ast_autochan *spyee_autochan,
 	struct ast_autochan **spyee_bridge_autochan, struct ast_audiohook *bridge_whisper_audiohook,
-	const char *spyer_name, const char *name)
+	const char *spyer_name, const char *name, struct ast_flags *flags)
 {
 	int retval = 0;
 	struct ast_autochan *internal_bridge_autochan;
@@ -609,7 +626,7 @@ static int attach_barge(struct ast_autoc
 		return -1;
 	}
 
-	if (start_spying(internal_bridge_autochan, spyer_name, bridge_whisper_audiohook)) {
+	if (start_spying(internal_bridge_autochan, spyer_name, bridge_whisper_audiohook, flags)) {
 		ast_log(LOG_WARNING, "Unable to attach barge audiohook on spyee '%s'. Barge mode disabled.\n", name);
 		retval = -1;
 	}
@@ -660,7 +677,7 @@ static int channel_spy(struct ast_channe
 	*/
 	ast_audiohook_init(&csth.spy_audiohook, AST_AUDIOHOOK_TYPE_SPY, "ChanSpy", 0);
 
-	if (start_spying(spyee_autochan, spyer_name, &csth.spy_audiohook)) {
+	if (start_spying(spyee_autochan, spyer_name, &csth.spy_audiohook, flags)) {
 		ast_audiohook_destroy(&csth.spy_audiohook);
 		return 0;
 	}
@@ -671,7 +688,7 @@ static int channel_spy(struct ast_channe
 		*/
 		ast_audiohook_init(&csth.whisper_audiohook, AST_AUDIOHOOK_TYPE_WHISPER, "ChanSpy", 0);
 
-		if (start_spying(spyee_autochan, spyer_name, &csth.whisper_audiohook)) {
+		if (start_spying(spyee_autochan, spyer_name, &csth.whisper_audiohook, flags)) {
 			ast_log(LOG_WARNING, "Unable to attach whisper audiohook to spyee %s. Whisper mode disabled!\n", name);
 		}
 	}
@@ -721,7 +738,7 @@ static int channel_spy(struct ast_channe
 			 * be attached and we'll need to continue attempting to attach the barge
 			 * audio hook. */
 			if (!bridge_connected && attach_barge(spyee_autochan, &spyee_bridge_autochan,
-					&csth.bridge_whisper_audiohook, spyer_name, name) == 0) {
+					&csth.bridge_whisper_audiohook, spyer_name, name, flags) == 0) {
 				bridge_connected = 1;
 			}
 
diff -pruN 1:13.23.1~dfsg-2/apps/app_confbridge.c 1:16.1.1~dfsg-1/apps/app_confbridge.c
--- 1:13.23.1~dfsg-2/apps/app_confbridge.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_confbridge.c	2018-12-26 18:45:13.000000000 +0000
@@ -43,8 +43,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -73,6 +71,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/json.h"
 #include "asterisk/format_cache.h"
 #include "asterisk/taskprocessor.h"
+#include "asterisk/stream.h"
+#include "asterisk/message.h"
 
 /*** DOCUMENTATION
 	<application name="ConfBridge" language="en_US">
@@ -504,6 +504,10 @@ const char *conf_get_sound(enum conf_sou
 		return S_OR(custom_sounds->muted, "conf-muted");
 	case CONF_SOUND_UNMUTED:
 		return S_OR(custom_sounds->unmuted, "conf-unmuted");
+	case CONF_SOUND_BINAURAL_ON:
+		return S_OR(custom_sounds->binauralon, "confbridge-binaural-on");
+	case CONF_SOUND_BINAURAL_OFF:
+		return S_OR(custom_sounds->binauraloff, "confbridge-binaural-off");
 	case CONF_SOUND_ONLY_ONE:
 		return S_OR(custom_sounds->onlyone, "conf-onlyone");
 	case CONF_SOUND_THERE_ARE:
@@ -545,6 +549,7 @@ const char *conf_get_sound(enum conf_sou
 	return "";
 }
 
+
 static void send_conf_stasis(struct confbridge_conference *conference, struct ast_channel *chan,
 	struct stasis_message_type *type, struct ast_json *extras, int channel_topic)
 {
@@ -571,6 +576,10 @@ static void send_conf_stasis(struct conf
 		return;
 	}
 
+	if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_ENABLE_EVENTS)) {
+		conf_send_event_to_participants(conference, chan, msg);
+	}
+
 	if (channel_topic) {
 		stasis_publish(ast_channel_topic(chan), msg);
 	} else {
@@ -672,7 +681,7 @@ static void set_rec_filename(struct conf
 	if (ast_strlen_zero(rec_file)) {
 		ast_str_set(filename, 0, "confbridge-%s-%u.wav", conference->name,
 			(unsigned int) now);
-	} else {
+	} else if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_RECORD_FILE_TIMESTAMP)) {
 		/* insert time before file extension */
 		ext = strrchr(rec_file, '.');
 		if (ext) {
@@ -681,10 +690,13 @@ static void set_rec_filename(struct conf
 		} else {
 			ast_str_set(filename, 0, "%s-%u", rec_file, (unsigned int) now);
 		}
+	} else {
+		ast_str_set(filename, 0, "%s", rec_file);
 	}
-	if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_RECORD_FILE_APPEND)) {
-		ast_str_append(filename, 0, ",a");
-	}
+	ast_str_append(filename, 0, ",%s%s,%s",
+		ast_test_flag(&conference->b_profile, BRIDGE_OPT_RECORD_FILE_APPEND) ? "a" : "",
+		conference->b_profile.rec_options,
+		conference->b_profile.rec_command);
 }
 
 static int is_new_rec_file(const char *rec_file, struct ast_str **orig_rec_file)
@@ -703,6 +715,11 @@ static int is_new_rec_file(const char *r
 	return 0;
 }
 
+struct confbridge_conference *conf_find_bridge(const char *conference_name)
+{
+	return ao2_find(conference_bridges, conference_name, OBJ_KEY);
+}
+
 /*!
  * \internal
  * \brief Returns whether or not conference is being recorded.
@@ -1094,13 +1111,15 @@ static void destroy_conference_bridge(vo
 		if (conference->playback_queue) {
 			struct hangup_data hangup;
 			hangup_data_init(&hangup, conference);
-			ast_taskprocessor_push(conference->playback_queue, hangup_playback, &hangup);
 
-			ast_mutex_lock(&hangup.lock);
-			while (!hangup.hungup) {
-				ast_cond_wait(&hangup.cond, &hangup.lock);
+			if (!ast_taskprocessor_push(conference->playback_queue, hangup_playback, &hangup)) {
+				ast_mutex_lock(&hangup.lock);
+				while (!hangup.hungup) {
+					ast_cond_wait(&hangup.cond, &hangup.lock);
+				}
+				ast_mutex_unlock(&hangup.lock);
 			}
-			ast_mutex_unlock(&hangup.lock);
+
 			hangup_data_destroy(&hangup);
 		} else {
 			/* Playback queue is not yet allocated. Just hang up the channel straight */
@@ -1532,9 +1551,25 @@ static struct confbridge_conference *joi
 		ast_bridge_set_internal_sample_rate(conference->bridge, conference->b_profile.internal_sample_rate);
 		/* Set the internal mixing interval on the bridge from the bridge profile */
 		ast_bridge_set_mixing_interval(conference->bridge, conference->b_profile.mix_interval);
+		ast_bridge_set_binaural_active(conference->bridge, ast_test_flag(&conference->b_profile, BRIDGE_OPT_BINAURAL_ACTIVE));
 
 		if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER)) {
 			ast_bridge_set_talker_src_video_mode(conference->bridge);
+		} else if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_VIDEO_SRC_SFU)) {
+			ast_bridge_set_sfu_video_mode(conference->bridge);
+			ast_bridge_set_video_update_discard(conference->bridge, conference->b_profile.video_update_discard);
+			ast_bridge_set_remb_send_interval(conference->bridge, conference->b_profile.remb_send_interval);
+			if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE)) {
+				ast_brige_set_remb_behavior(conference->bridge, AST_BRIDGE_VIDEO_SFU_REMB_AVERAGE);
+			} else if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_REMB_BEHAVIOR_LOWEST)) {
+				ast_brige_set_remb_behavior(conference->bridge, AST_BRIDGE_VIDEO_SFU_REMB_LOWEST);
+			} else if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST)) {
+				ast_brige_set_remb_behavior(conference->bridge, AST_BRIDGE_VIDEO_SFU_REMB_HIGHEST);
+			}
+		}
+
+		if (ast_test_flag(&conference->b_profile, BRIDGE_OPT_ENABLE_EVENTS)) {
+			ast_bridge_set_send_sdp_label(conference->bridge, 1);
 		}
 
 		/* Link it into the conference bridges container */
@@ -2286,6 +2321,25 @@ static int join_callback(struct ast_brid
 	return 0;
 }
 
+struct confbridge_hook_data {
+	struct confbridge_conference *conference;
+	struct confbridge_user *user;
+	enum ast_bridge_hook_type hook_type;
+};
+
+static int send_event_hook_callback(struct ast_bridge_channel *bridge_channel, void *data)
+{
+	struct confbridge_hook_data *hook_data = data;
+
+	if (hook_data->hook_type == AST_BRIDGE_HOOK_TYPE_JOIN) {
+		send_join_event(hook_data->user, hook_data->conference);
+	} else {
+		send_leave_event(hook_data->user, hook_data->conference);
+	}
+
+	return 0;
+}
+
 /*! \brief The ConfBridge application */
 static int confbridge_exec(struct ast_channel *chan, const char *data)
 {
@@ -2303,6 +2357,9 @@ static int confbridge_exec(struct ast_ch
 		.tech_args.silence_threshold = DEFAULT_SILENCE_THRESHOLD,
 		.tech_args.drop_silence = 0,
 	};
+	struct confbridge_hook_data *join_hook_data;
+	struct confbridge_hook_data *leave_hook_data;
+
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(conf_name);
 		AST_APP_ARG(b_profile_name);
@@ -2439,11 +2496,11 @@ static int confbridge_exec(struct ast_ch
 		user.tech_args.drop_silence = 1;
 	}
 
-	if (ast_test_flag(&user.u_profile, USER_OPT_JITTERBUFFER) && ast_module_check("func_jitterbuffer.so")) {
+	if (ast_test_flag(&user.u_profile, USER_OPT_JITTERBUFFER)) {
 		ast_func_write(chan, "JITTERBUFFER(adaptive)", "default");
 	}
 
-	if (ast_test_flag(&user.u_profile, USER_OPT_DENOISE) && ast_module_check("codec_speex.so")) {
+	if (ast_test_flag(&user.u_profile, USER_OPT_DENOISE)) {
 		ast_func_write(chan, "DENOISE(rx)", "on");
 	}
 
@@ -2485,8 +2542,39 @@ static int confbridge_exec(struct ast_ch
 
 	conf_moh_unsuspend(&user);
 
-	/* Join our conference bridge for real */
-	send_join_event(&user, conference);
+	join_hook_data = ast_malloc(sizeof(*join_hook_data));
+	if (!join_hook_data) {
+		res = -1;
+		goto confbridge_cleanup;
+	}
+	join_hook_data->user = &user;
+	join_hook_data->conference = conference;
+	join_hook_data->hook_type = AST_BRIDGE_HOOK_TYPE_JOIN;
+	res = ast_bridge_join_hook(&user.features, send_event_hook_callback,
+		join_hook_data, ast_free_ptr, 0);
+	if (res) {
+		ast_free(join_hook_data);
+		ast_log(LOG_ERROR, "Couldn't add bridge join hook for channel '%s'\n", ast_channel_name(chan));
+		goto confbridge_cleanup;
+	}
+
+	leave_hook_data = ast_malloc(sizeof(*leave_hook_data));
+	if (!leave_hook_data) {
+		/* join_hook_data is cleaned up by ast_bridge_features_cleanup via the goto */
+		res = -1;
+		goto confbridge_cleanup;
+	}
+	leave_hook_data->user = &user;
+	leave_hook_data->conference = conference;
+	leave_hook_data->hook_type = AST_BRIDGE_HOOK_TYPE_LEAVE;
+	res = ast_bridge_leave_hook(&user.features, send_event_hook_callback,
+		leave_hook_data, ast_free_ptr, 0);
+	if (res) {
+		/* join_hook_data is cleaned up by ast_bridge_features_cleanup via the goto */
+		ast_free(leave_hook_data);
+		ast_log(LOG_ERROR, "Couldn't add bridge leave hook for channel '%s'\n", ast_channel_name(chan));
+		goto confbridge_cleanup;
+	}
 
 	if (ast_bridge_join_hook(&user.features, join_callback, NULL, NULL, 0)) {
 		async_play_sound_ready(user.chan);
@@ -2508,8 +2596,6 @@ static int confbridge_exec(struct ast_ch
 		pbx_builtin_setvar_helper(chan, "CONFBRIDGE_RESULT", "HANGUP");
 	}
 
-	send_leave_event(&user, conference);
-
 	/* if we're shutting down, don't attempt to do further processing */
 	if (ast_shutting_down()) {
 		/*
@@ -2584,6 +2670,20 @@ static int action_toggle_mute(struct con
 			conference->b_profile.sounds)) < 0;
 }
 
+static int action_toggle_binaural(struct confbridge_conference *conference,
+		struct confbridge_user *user,
+		struct ast_bridge_channel *bridge_channel)
+{
+	unsigned int binaural;
+	ast_bridge_channel_lock_bridge(bridge_channel);
+	binaural = !bridge_channel->binaural_suspended;
+	bridge_channel->binaural_suspended = binaural;
+	ast_bridge_unlock(bridge_channel->bridge);
+	return play_file(bridge_channel, NULL, (binaural ?
+				conf_get_sound(CONF_SOUND_BINAURAL_OFF, user->b_profile.sounds) :
+				conf_get_sound(CONF_SOUND_BINAURAL_ON, user->b_profile.sounds))) < 0;
+}
+
 static int action_toggle_mute_participants(struct confbridge_conference *conference, struct confbridge_user *user)
 {
 	struct confbridge_user *cur_user = NULL;
@@ -2808,6 +2908,9 @@ static int execute_menu_entry(struct con
 		case MENU_ACTION_TOGGLE_MUTE:
 			res |= action_toggle_mute(conference, user, bridge_channel);
 			break;
+		case MENU_ACTION_TOGGLE_BINAURAL:
+			action_toggle_binaural(conference, user, bridge_channel);
+			break;
 		case MENU_ACTION_ADMIN_TOGGLE_MUTE_PARTICIPANTS:
 			if (!isadmin) {
 				break;
@@ -2887,7 +2990,9 @@ static int execute_menu_entry(struct con
 			break;
 		case MENU_ACTION_SET_SINGLE_VIDEO_SRC:
 			ao2_lock(conference);
-			ast_bridge_set_single_src_video_mode(conference->bridge, bridge_channel->chan);
+			if (!ast_test_flag(&conference->b_profile, BRIDGE_OPT_VIDEO_SRC_SFU)) {
+				ast_bridge_set_single_src_video_mode(conference->bridge, bridge_channel->chan);
+			}
 			ao2_unlock(conference);
 			break;
 		case MENU_ACTION_RELEASE_SINGLE_VIDEO_SRC:
@@ -4091,8 +4196,9 @@ static int load_module(void)
 	}
 
 	/* Create a container to hold the conference bridges */
-	conference_bridges = ao2_container_alloc(CONFERENCE_BRIDGE_BUCKETS,
-		conference_bridge_hash_cb, conference_bridge_cmp_cb);
+	conference_bridges = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		CONFERENCE_BRIDGE_BUCKETS,
+		conference_bridge_hash_cb, NULL, conference_bridge_cmp_cb);
 	if (!conference_bridges) {
 		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
@@ -4116,7 +4222,7 @@ static int load_module(void)
 	res |= ast_manager_register_xml("ConfbridgeUnlock", EVENT_FLAG_CALL, action_confbridgeunlock);
 	res |= ast_manager_register_xml("ConfbridgeLock", EVENT_FLAG_CALL, action_confbridgelock);
 	res |= ast_manager_register_xml("ConfbridgeStartRecord", EVENT_FLAG_SYSTEM, action_confbridgestartrecord);
-	res |= ast_manager_register_xml("ConfbridgeStopRecord", EVENT_FLAG_CALL, action_confbridgestoprecord);
+	res |= ast_manager_register_xml("ConfbridgeStopRecord", EVENT_FLAG_SYSTEM, action_confbridgestoprecord);
 	res |= ast_manager_register_xml("ConfbridgeSetSingleVideoSrc", EVENT_FLAG_CALL, action_confbridgesetsinglevideosrc);
 	if (res) {
 		unload_module();
@@ -4137,4 +4243,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
+	.optional_modules = "codec_speex,func_jitterbuffer",
 );
diff -pruN 1:13.23.1~dfsg-2/apps/app_controlplayback.c 1:16.1.1~dfsg-1/apps/app_controlplayback.c
--- 1:13.23.1~dfsg-2/apps/app_controlplayback.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_controlplayback.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/app.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_dahdiras.c 1:16.1.1~dfsg-1/apps/app_dahdiras.c
--- 1:13.23.1~dfsg-2/apps/app_dahdiras.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_dahdiras.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,21 +27,14 @@
 
 /*** MODULEINFO
 	<depend>dahdi</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/ioctl.h>
 #include <sys/wait.h>
-#ifdef __linux__
-#include <sys/signal.h>
-#else
 #include <signal.h>
-#endif /* __linux__ */
-
 #include <fcntl.h>
 
 #include <dahdi/user.h>
@@ -233,4 +226,4 @@ static int load_module(void)
 	return ((ast_register_application_xml(app, dahdiras_exec)) ? AST_MODULE_LOAD_DECLINE : AST_MODULE_LOAD_SUCCESS);
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "DAHDI ISDN Remote Access Server");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "DAHDI ISDN Remote Access Server");
diff -pruN 1:13.23.1~dfsg-2/apps/app_db.c 1:16.1.1~dfsg-1/apps/app_db.c
--- 1:13.23.1~dfsg-2/apps/app_db.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_db.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -44,25 +42,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/lock.h"
 
 /*** DOCUMENTATION
-	<application name="DBdel" language="en_US">
-		<synopsis>
-			Delete a key from the asterisk database.
-		</synopsis>
-		<syntax argsep="/">
-			<parameter name="family" required="true" />
-			<parameter name="key" required="true" />
-		</syntax>
-		<description>
-			<para>This application will delete a <replaceable>key</replaceable> from the Asterisk
-			database.</para>
-			<note><para>This application has been DEPRECATED in favor of the DB_DELETE function.</para></note>
-		</description>
-		<see-also>
-			<ref type="function">DB_DELETE</ref>
-			<ref type="application">DBdeltree</ref>
-			<ref type="function">DB</ref>
-		</see-also>
-	</application>
 	<application name="DBdeltree" language="en_US">
 		<synopsis>
 			Delete a family or keytree from the asterisk database.
@@ -77,13 +56,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 		</description>
 		<see-also>
 			<ref type="function">DB_DELETE</ref>
-			<ref type="application">DBdel</ref>
 			<ref type="function">DB</ref>
 		</see-also>
 	</application>
  ***/
 
-static const char d_app[] = "DBdel";
 static const char dt_app[] = "DBdeltree";
 
 static int deltree_exec(struct ast_channel *chan, const char *data)
@@ -119,53 +96,14 @@ static int deltree_exec(struct ast_chann
 	return 0;
 }
 
-static int del_exec(struct ast_channel *chan, const char *data)
-{
-	char *argv, *family, *key;
-	static int deprecation_warning = 0;
-
-	if (!deprecation_warning) {
-		deprecation_warning = 1;
-		ast_log(LOG_WARNING, "The DBdel application has been deprecated in favor of the DB_DELETE dialplan function!\n");
-	}
-
-	argv = ast_strdupa(data);
-
-	if (strchr(argv, '/')) {
-		family = strsep(&argv, "/");
-		key = strsep(&argv, "\0");
-		if (!family || !key) {
-			ast_debug(1, "Ignoring; Syntax error in argument\n");
-			return 0;
-		}
-		ast_verb(3, "DBdel: family=%s, key=%s\n", family, key);
-		if (ast_db_del(family, key))
-			ast_verb(3, "DBdel: Error deleting key from database.\n");
-	} else {
-		ast_debug(1, "Ignoring, no parameters\n");
-	}
-
-	return 0;
-}
-
 static int unload_module(void)
 {
-	int retval;
-
-	retval = ast_unregister_application(dt_app);
-	retval |= ast_unregister_application(d_app);
-
-	return retval;
+	return ast_unregister_application(dt_app);
 }
 
 static int load_module(void)
 {
-	int retval;
-
-	retval = ast_register_application_xml(d_app, del_exec);
-	retval |= ast_register_application_xml(dt_app, deltree_exec);
-
-	return retval;
+	return ast_register_application_xml(dt_app, deltree_exec);
 }
 
 AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Database Access Functions");
diff -pruN 1:13.23.1~dfsg-2/apps/app_dial.c 1:16.1.1~dfsg-1/apps/app_dial.c
--- 1:13.23.1~dfsg-2/apps/app_dial.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_dial.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,10 +32,8 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
 
@@ -68,6 +66,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/bridge_after.h"
 #include "asterisk/features_config.h"
 #include "asterisk/max_forwards.h"
+#include "asterisk/stream.h"
 
 /*** DOCUMENTATION
 	<application name="Dial" language="en_US">
@@ -143,11 +142,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 					a call to be answered. Exit to that extension if it exists in the
 					current context, or the context defined in the <variable>EXITCONTEXT</variable> variable,
 					if it exists.</para>
-					<note>
-						<para>Many SIP and ISDN phones cannot send DTMF digits until the call is
-						connected.  If you wish to use this option with these phones, you
-						can use the <literal>Answer</literal> application before dialing.</para>
-					</note>
+					<para>NOTE: Many SIP and ISDN phones cannot send DTMF digits until the call is
+					connected.  If you wish to use this option with these phones, you
+					can use the <literal>Answer</literal> application before dialing.</para>
 				</option>
 				<option name="D" argsep=":">
 					<argument name="called" />
@@ -178,21 +175,15 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 					<argument name="priority" required="true" />
 					<para>When the caller hangs up, transfer the <emphasis>called</emphasis> party
 					to the specified destination and <emphasis>start</emphasis> execution at that location.</para>
-					<note>
-						<para>Any channel variables you want the called channel to inherit from the caller channel must be
-						prefixed with one or two underbars ('_').</para>
-					</note>
+					<para>NOTE: Any channel variables you want the called channel to inherit from the caller channel must be
+					prefixed with one or two underbars ('_').</para>
 				</option>
 				<option name="F">
 					<para>When the caller hangs up, transfer the <emphasis>called</emphasis> party to the next priority of the current extension
 					and <emphasis>start</emphasis> execution at that location.</para>
-					<note>
-						<para>Any channel variables you want the called channel to inherit from the caller channel must be
-						prefixed with one or two underbars ('_').</para>
-					</note>
-					<note>
-						<para>Using this option from a Macro() or GoSub() might not make sense as there would be no return points.</para>
-					</note>
+					<para>NOTE: Any channel variables you want the called channel to inherit from the caller channel must be
+					prefixed with one or two underbars ('_').</para>
+					<para>NOTE: Using this option from a Macro() or GoSub() might not make sense as there would be no return points.</para>
 				</option>
 				<option name="g">
 					<para>Proceed with dialplan execution at the next priority in the current extension if the
@@ -205,9 +196,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 					<para>If the call is answered, transfer the calling party to
 					the specified <replaceable>priority</replaceable> and the called party to the specified
 					<replaceable>priority</replaceable> plus one.</para>
-					<note>
-						<para>You cannot use any additional action post answer options in conjunction with this option.</para>
-					</note>
+					<para>NOTE: You cannot use any additional action post answer options in conjunction with this option.</para>
 				</option>
 				<option name="h">
 					<para>Allow the called party to hang up by sending the DTMF sequence
@@ -216,12 +205,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 				<option name="H">
 					<para>Allow the calling party to hang up by sending the DTMF sequence
 					defined for disconnect in <filename>features.conf</filename>.</para>
-					<note>
-						<para>Many SIP and ISDN phones cannot send DTMF digits until the call is
-						connected.  If you wish to allow DTMF disconnect before the dialed
-						party answers with these phones, you can use the <literal>Answer</literal>
-						application before dialing.</para>
-					</note>
+					<para>NOTE: Many SIP and ISDN phones cannot send DTMF digits until the call is
+					connected.  If you wish to allow DTMF disconnect before the dialed
+					party answers with these phones, you can use the <literal>Answer</literal>
+					application before dialing.</para>
 				</option>
 				<option name="i">
 					<para>Asterisk will ignore any forwarding requests it may receive on this dial attempt.</para>
@@ -317,14 +304,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 							</value>
 						</variable>
 					</variablelist>
-					<note>
-						<para>You cannot use any additional action post answer options in conjunction
-						with this option. Also, pbx services are run on the peer (called) channel,
-						so you will not be able to set timeouts via the <literal>TIMEOUT()</literal> function in this macro.</para>
-					</note>
-					<warning><para>Be aware of the limitations that macros have, specifically with regards to use of
+					<para>NOTE: You cannot use any additional action post answer options in conjunction
+					with this option. Also, pbx services are run on the peer (called) channel,
+					so you will not be able to set timeouts via the <literal>TIMEOUT()</literal> function in this macro.</para>
+					<para>WARNING: Be aware of the limitations that macros have, specifically with regards to use of
 					the <literal>WaitExten</literal> application. For more information, see the documentation for
-					<literal>Macro()</literal>.</para></warning>
+					<literal>Macro()</literal>.</para>
+					<para>NOTE: Macros are deprecated, GoSub should be used instead,
+					see the <literal>U</literal> option.</para>
 				</option>
 				<option name="n">
 					<argument name="delete">
@@ -389,10 +376,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						to send no cause.  See the <filename>causes.h</filename> file for the
 						full list of valid causes and names.
 						</para>
-					<note>
-						<para>chan_sip does not support setting the cause on a CANCEL to anything
-						other than ANSWERED_ELSEWHERE.</para>
-					</note>
+					<para>NOTE: chan_sip does not support setting the cause on a CANCEL to anything
+					other than ANSWERED_ELSEWHERE.</para>
 				</option>
 				<option name="r">
 					<para>Default: Indicate ringing to the calling party, even if the called party isn't actually ringing. Pass no audio to the calling
@@ -427,7 +412,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 				</option>
 				<option name="U" argsep="^">
 					<argument name="x" required="true">
-						<para>Name of the subroutine to execute via <literal>Gosub</literal></para>
+						<para>Name of the subroutine context to execute via <literal>Gosub</literal>.
+						The subroutine execution starts in the named context at the s exten and priority 1.</para>
 					</argument>
 					<argument name="arg" multiple="true" required="false">
 						<para>Arguments for the <literal>Gosub</literal> routine</para>
@@ -456,11 +442,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 							</value>
 						</variable>
 					</variablelist>
-					<note>
-						<para>You cannot use any additional action post answer options in conjunction
-						with this option. Also, pbx services are run on the peer (called) channel,
-						so you will not be able to set timeouts via the <literal>TIMEOUT()</literal> function in this routine.</para>
-					</note>
+					<para>NOTE: You cannot use any additional action post answer options in conjunction
+					with this option. Also, pbx services are run on the <emphasis>called</emphasis> channel,
+					so you will not be able to set timeouts via the <literal>TIMEOUT()</literal> function in this routine.</para>
 				</option>
 				<option name="u">
 					<argument name = "x" required="true">
@@ -545,27 +529,29 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<example title="Dial with pre-dial subroutines">
 			[default]
 
-			exten => callee_channel,1,NoOp()
+			exten => callee_channel,1,NoOp(ARG1=${ARG1} ARG2=${ARG2})
 			 same => n,Log(NOTICE, I'm called on channel ${CHANNEL} prior to it starting the dial attempt)
 			 same => n,Return()
 
-			exten => called_channel,1,NoOp()
+			exten => called_channel,1,NoOp(ARG1=${ARG1} ARG2=${ARG2})
 			 same => n,Log(NOTICE, I'm called on outbound channel ${CHANNEL} prior to it being used to dial someone)
 			 same => n,Return()
 
 			exten => _X.,1,NoOp()
-			 same => n,Dial(PJSIP/alice,,b(default^called_channel^1)B(default^callee_channel^1))
+			 same => n,Dial(PJSIP/alice,,b(default^called_channel^1(my_gosub_arg1^my_gosub_arg2))B(default^callee_channel^1(my_gosub_arg1^my_gosub_arg2)))
 			 same => n,Hangup()
 			</example>
 			<example title="Dial with post-answer subroutine executed on outbound channel">
-			[default]
+			[my_gosub_routine]
 
-			exten => called_channel,1,NoOp()
+			exten => s,1,NoOp(ARG1=${ARG1} ARG2=${ARG2})
 			 same => n,Playback(hello)
 			 same => n,Return()
 
+			[default]
+
 			exten => _X.,1,NoOp()
-			 same => n,Dial(PJSIP/alice,,U(default^called_channel^1))
+			 same => n,Dial(PJSIP/alice,,U(my_gosub_routine^my_gosub_arg1^my_gosub_arg2))
 			 same => n,Hangup()
 			</example>
 			<example title="Dial into ConfBridge using 'G' option">
@@ -967,19 +953,21 @@ static void do_forward(struct chanlist *
 	/* If we have been told to ignore forwards, just set this channel to null and continue processing extensions normally */
 	if (ast_test_flag64(peerflags, OPT_IGNORE_FORWARDING)) {
 		ast_verb(3, "Forwarding %s to '%s/%s' prevented.\n", ast_channel_name(in), tech, stuff);
+		ast_channel_publish_dial_forward(in, original, NULL, NULL, "CANCEL",
+			ast_channel_call_forward(original));
 		c = o->chan = NULL;
 		cause = AST_CAUSE_BUSY;
 	} else {
-		struct ast_format_cap *nativeformats;
+		struct ast_stream_topology *topology;
 
 		ast_channel_lock(in);
-		nativeformats = ao2_bump(ast_channel_nativeformats(in));
+		topology = ast_stream_topology_clone(ast_channel_get_stream_topology(in));
 		ast_channel_unlock(in);
 
 		/* Setup parameters */
-		c = o->chan = ast_request(tech, nativeformats, NULL, in, stuff, &cause);
+		c = o->chan = ast_request_with_stream_topology(tech, topology, NULL, in, stuff, &cause);
 
-		ao2_cleanup(nativeformats);
+		ast_stream_topology_free(topology);
 
 		if (c) {
 			if (single && !caller_entertained) {
@@ -1186,9 +1174,6 @@ static struct ast_channel *wait_for_answ
 	int prestart = num.busy + num.congestion + num.nochan;
 	int orig = *to;
 	struct ast_channel *peer = NULL;
-#ifdef HAVE_EPOLL
-	struct chanlist *epollo;
-#endif
 	struct chanlist *outgoing = AST_LIST_FIRST(out_chans);
 	/* single is set if only one destination is enabled */
 	int single = outgoing && !AST_LIST_NEXT(outgoing, node);
@@ -1227,12 +1212,6 @@ static struct ast_channel *wait_for_answ
 
 	is_cc_recall = ast_cc_is_recall(in, &cc_recall_core_id, NULL);
 
-#ifdef HAVE_EPOLL
-	AST_LIST_TRAVERSE(out_chans, epollo, node) {
-		ast_poll_channel_add(in, epollo->chan);
-	}
-#endif
-
 	while ((*to = ast_remaining_ms(start, orig)) && !peer) {
 		struct chanlist *o;
 		int pos = 0; /* how many channels do we handle */
@@ -1359,9 +1338,6 @@ static struct ast_channel *wait_for_answ
 			f = ast_read(winner);
 			if (!f) {
 				ast_channel_hangupcause_set(in, ast_channel_hangupcause(c));
-#ifdef HAVE_EPOLL
-				ast_poll_channel_del(in, c);
-#endif
 				ast_channel_publish_dial(in, c, NULL, ast_hangup_cause_to_dial_status(ast_channel_hangupcause(c)));
 				ast_hangup(c);
 				c = o->chan = NULL;
@@ -1482,6 +1458,7 @@ static struct ast_channel *wait_for_answ
 							pa->sentringing++;
 						}
 					}
+					ast_channel_publish_dial(in, c, NULL, "RINGING");
 					break;
 				case AST_CONTROL_PROGRESS:
 					ast_verb(3, "%s is making progress passing it to %s\n", ast_channel_name(c), ast_channel_name(in));
@@ -1501,6 +1478,7 @@ static struct ast_channel *wait_for_answ
 							dtmf_progress);
 						ast_dtmf_stream(c, in, dtmf_progress, 250, 0);
 					}
+					ast_channel_publish_dial(in, c, NULL, "PROGRESS");
 					break;
 				case AST_CONTROL_VIDUPDATE:
 				case AST_CONTROL_SRCUPDATE:
@@ -1573,6 +1551,7 @@ static struct ast_channel *wait_for_answ
 					}
 					if (!ast_test_flag64(outgoing, OPT_RINGBACK))
 						ast_indicate(in, AST_CONTROL_PROCEEDING);
+					ast_channel_publish_dial(in, c, NULL, "PROCEEDING");
 					break;
 				case AST_CONTROL_HOLD:
 					/* XXX this should be saved like AST_CONTROL_CONNECTED_LINE for !single || caller_entertained */
@@ -1754,12 +1733,20 @@ static struct ast_channel *wait_for_answ
 						ast_indicate(o->chan, f->subclass.integer);
 						break;
 					case AST_CONTROL_CONNECTED_LINE:
+						if (ast_test_flag64(o, OPT_IGNORE_CONNECTEDLINE)) {
+							ast_verb(3, "Connected line update to %s prevented.\n", ast_channel_name(o->chan));
+							break;
+						}
 						if (ast_channel_connected_line_sub(in, o->chan, f, 1) &&
 							ast_channel_connected_line_macro(in, o->chan, f, 0, 1)) {
 							ast_indicate_data(o->chan, f->subclass.integer, f->data.ptr, f->datalen);
 						}
 						break;
 					case AST_CONTROL_REDIRECTING:
+						if (ast_test_flag64(o, OPT_IGNORE_CONNECTEDLINE)) {
+							ast_verb(3, "Redirecting update to %s prevented.\n", ast_channel_name(o->chan));
+							break;
+						}
 						if (ast_channel_redirecting_sub(in, o->chan, f, 1) &&
 							ast_channel_redirecting_macro(in, o->chan, f, 0, 1)) {
 							ast_indicate_data(o->chan, f->subclass.integer, f->data.ptr, f->datalen);
@@ -1785,13 +1772,6 @@ skip_frame:;
 		publish_dial_end_event(in, out_chans, NULL, "NOANSWER");
 	}
 
-#ifdef HAVE_EPOLL
-	AST_LIST_TRAVERSE(out_chans, epollo, node) {
-		if (epollo->chan)
-			ast_poll_channel_del(in, epollo->chan);
-	}
-#endif
-
 	if (is_cc_recall) {
 		ast_cc_completed(in, "Recall completed!");
 	}
@@ -2462,7 +2442,7 @@ static int dial_exec_full(struct ast_cha
 		char *tech = strsep(&number, "/");
 		size_t tech_len;
 		size_t number_len;
-		struct ast_format_cap *nativeformats;
+		struct ast_stream_topology *topology;
 
 		num_dialed++;
 		if (ast_strlen_zero(number)) {
@@ -2514,13 +2494,13 @@ static int dial_exec_full(struct ast_cha
 		 */
 		ast_party_connected_line_copy(&tmp->connected, ast_channel_connected(chan));
 
-		nativeformats = ao2_bump(ast_channel_nativeformats(chan));
+		topology = ast_stream_topology_clone(ast_channel_get_stream_topology(chan));
 
 		ast_channel_unlock(chan);
 
-		tc = ast_request(tmp->tech, nativeformats, NULL, chan, tmp->number, &cause);
+		tc = ast_request_with_stream_topology(tmp->tech, topology, NULL, chan, tmp->number, &cause);
 
-		ao2_cleanup(nativeformats);
+		ast_stream_topology_free(topology);
 
 		if (!tc) {
 			/* If we can't, just go on to the next call */
@@ -2863,7 +2843,7 @@ static int dial_exec_full(struct ast_cha
 			chans[0] = chan;
 			chans[1] = peer;
 
-			/* we need to stream the announcement while monitoring the caller for a hangup */
+			/* we need to stream the announcement to the called party when the OPT_ARG_ANNOUNCE (-A) is setted */
 
 			/* stream the file */
 			res = ast_streamfile(peer, opt_args[OPT_ARG_ANNOUNCE], ast_channel_language(peer));
@@ -3391,4 +3371,9 @@ static int load_module(void)
 	return res;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Dialing Application");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Dialing Application",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "ccss",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_dictate.c 1:16.1.1~dfsg-1/apps/app_dictate.c
--- 1:13.23.1~dfsg-2/apps/app_dictate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_dictate.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include "asterisk/paths.h" /* use ast_config_AST_SPOOL_DIR */
diff -pruN 1:13.23.1~dfsg-2/apps/app_directed_pickup.c 1:16.1.1~dfsg-1/apps/app_directed_pickup.c
--- 1:13.23.1~dfsg-2/apps/app_directed_pickup.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_directed_pickup.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_directory.c 1:16.1.1~dfsg-1/apps/app_directory.c
--- 1:13.23.1~dfsg-2/apps/app_directory.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_directory.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
  ***/
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 
 #include "asterisk/paths.h" /* use ast_config_AST_SPOOL_DIR */
diff -pruN 1:13.23.1~dfsg-2/apps/app_disa.c 1:16.1.1~dfsg-1/apps/app_disa.c
--- 1:13.23.1~dfsg-2/apps/app_disa.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_disa.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 #include <sys/time.h>
 
diff -pruN 1:13.23.1~dfsg-2/apps/app_dumpchan.c 1:16.1.1~dfsg-1/apps/app_dumpchan.c
--- 1:13.23.1~dfsg-2/apps/app_dumpchan.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_dumpchan.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_echo.c 1:16.1.1~dfsg-1/apps/app_echo.c
--- 1:13.23.1~dfsg-2/apps/app_echo.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_echo.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_exec.c 1:16.1.1~dfsg-1/apps/app_exec.c
--- 1:13.23.1~dfsg-2/apps/app_exec.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_exec.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
  ***/
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_externalivr.c 1:16.1.1~dfsg-1/apps/app_externalivr.c
--- 1:13.23.1~dfsg-2/apps/app_externalivr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_externalivr.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,8 +37,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/lock.h"
@@ -152,10 +150,12 @@ struct gen_state {
 };
 
 static int eivr_comm(struct ast_channel *chan, struct ivr_localuser *u,
-	int *eivr_events_fd, int *eivr_commands_fd, int *eivr_errors_fd,
+	struct ast_iostream *eivr_events,
+	struct ast_iostream *eivr_commands,
+	struct ast_iostream *eivr_errors,
 	const struct ast_str *args, const struct ast_flags flags);
 
-static void send_eivr_event(FILE *handle, const char event, const char *data,
+static void send_eivr_event(struct ast_iostream *stream, const char event, const char *data,
 	const struct ast_channel *chan)
 {
 	struct ast_str *tmp = ast_str_create(12);
@@ -164,9 +164,11 @@ static void send_eivr_event(FILE *handle
 	if (data) {
 		ast_str_append(&tmp, 0, ",%s", data);
 	}
+	ast_str_append(&tmp, 0, "\n");
+	ast_iostream_write(stream, ast_str_buffer(tmp), strlen(ast_str_buffer(tmp)));
+	ast_str_truncate(tmp, -1);
 
-	fprintf(handle, "%s\n", ast_str_buffer(tmp));
-	ast_debug(1, "sent '%s'\n", ast_str_buffer(tmp));
+	ast_debug(1, "sent '%s'", ast_str_buffer(tmp));
 	ast_free(tmp);
 }
 
@@ -395,6 +397,8 @@ static int app_exec(struct ast_channel *
 	int child_stdin[2] = { -1, -1 };
 	int child_stdout[2] = { -1, -1 };
 	int child_stderr[2] = { -1, -1 };
+	struct ast_iostream *stream_stdin = NULL, *stream_stdout = NULL,
+		*stream_stderr = NULL;
 	int res = -1;
 	int pid;
 
@@ -526,7 +530,7 @@ static int app_exec(struct ast_channel *
 			goto exit;
 		}
 
-		res = eivr_comm(chan, u, &ser->fd, &ser->fd, NULL, comma_delim_args, flags);
+		res = eivr_comm(chan, u, ser->stream, ser->stream, NULL, comma_delim_args, flags);
 
 	} else {
 		if (pipe(child_stdin)) {
@@ -568,7 +572,12 @@ static int app_exec(struct ast_channel *
 			child_stdout[1] = -1;
 			close(child_stderr[1]);
 			child_stderr[1] = -1;
-			res = eivr_comm(chan, u, &child_stdin[1], &child_stdout[0], &child_stderr[0], comma_delim_args, flags);
+
+			stream_stdin  = ast_iostream_from_fd(&child_stdin[1]);
+			stream_stdout = ast_iostream_from_fd(&child_stdout[0]);
+			stream_stderr = ast_iostream_from_fd(&child_stderr[0]);
+
+			res = eivr_comm(chan, u, stream_stdin, stream_stdout, stream_stderr, comma_delim_args, flags);
 		}
 	}
 
@@ -576,6 +585,15 @@ static int app_exec(struct ast_channel *
 	if (u->gen_active) {
 		ast_deactivate_generator(chan);
 	}
+	if (stream_stdin) {
+		ast_iostream_close(stream_stdin);
+	}
+	if (stream_stdout) {
+		ast_iostream_close(stream_stdout);
+	}
+	if (stream_stderr) {
+		ast_iostream_close(stream_stderr);
+	}
 	if (child_stdin[0] > -1) {
 		close(child_stdin[0]);
 	}
@@ -604,46 +622,25 @@ static int app_exec(struct ast_channel *
 }
 
 static int eivr_comm(struct ast_channel *chan, struct ivr_localuser *u,
- 				int *eivr_events_fd, int *eivr_commands_fd, int *eivr_errors_fd,
- 				const struct ast_str *args, const struct ast_flags flags)
+				struct ast_iostream *eivr_events,
+				struct ast_iostream *eivr_commands,
+				struct ast_iostream *eivr_errors,
+				const struct ast_str *args, const struct ast_flags flags)
 {
+	char input[1024];
 	struct playlist_entry *entry;
 	struct ast_frame *f;
 	int ms;
  	int exception;
  	int ready_fd;
-	int waitfds[2] = { *eivr_commands_fd, (eivr_errors_fd) ? *eivr_errors_fd : -1 };
+	int waitfds[2];
+	int r;
  	struct ast_channel *rchan;
  	int res = -1;
-	int test_available_fd = -1;
 	int hangup_info_sent = 0;
 
- 	FILE *eivr_commands = NULL;
- 	FILE *eivr_errors = NULL;
- 	FILE *eivr_events = NULL;
-
-	if (!(eivr_events = fdopen(*eivr_events_fd, "w"))) {
-		ast_chan_log(LOG_ERROR, chan, "Could not open stream to send events\n");
-		goto exit;
-	}
-	if (!(eivr_commands = fdopen(*eivr_commands_fd, "r"))) {
-		ast_chan_log(LOG_ERROR, chan, "Could not open stream to receive commands\n");
-		goto exit;
-	}
-	if (eivr_errors_fd) {  /* if opening a socket connection, error stream will not be used */
- 		if (!(eivr_errors = fdopen(*eivr_errors_fd, "r"))) {
- 			ast_chan_log(LOG_ERROR, chan, "Could not open stream to receive errors\n");
- 			goto exit;
- 		}
-	}
-
-	test_available_fd = open("/dev/null", O_RDONLY);
-
- 	setvbuf(eivr_events, NULL, _IONBF, 0);
- 	setvbuf(eivr_commands, NULL, _IONBF, 0);
- 	if (eivr_errors) {
-		setvbuf(eivr_errors, NULL, _IONBF, 0);
-	}
+	waitfds[0] = ast_iostream_get_fd(eivr_commands);
+	waitfds[1] = eivr_errors ? ast_iostream_get_fd(eivr_errors) : -1;
 
 	while (1) {
 		if (ast_test_flag(ast_channel_flags(chan), AST_FLAG_ZOMBIE)) {
@@ -667,7 +664,7 @@ static int eivr_comm(struct ast_channel
  		errno = 0;
  		exception = 0;
 
- 		rchan = ast_waitfor_nandfds(&chan, 1, waitfds, (eivr_errors_fd) ? 2 : 1, &exception, &ready_fd, &ms);
+		rchan = ast_waitfor_nandfds(&chan, 1, waitfds, (eivr_errors) ? 2 : 1, &exception, &ready_fd, &ms);
 
  		if (ast_channel_state(chan) == AST_STATE_UP && !AST_LIST_EMPTY(&u->finishlist)) {
  			AST_LIST_LOCK(&u->finishlist);
@@ -715,15 +712,18 @@ static int eivr_comm(struct ast_channel
  				break;
  			}
  			ast_frfree(f);
- 		} else if (ready_fd == *eivr_commands_fd) {
- 			char input[1024];
-
- 			if (exception || (dup2(*eivr_commands_fd, test_available_fd) == -1) || feof(eivr_commands)) {
+ 		} else if (ready_fd == waitfds[0]) {
+ 			if (exception) {
  				ast_chan_log(LOG_ERROR, chan, "Child process went away\n");
   				break;
   			}
 
-			if (!fgets(input, sizeof(input), eivr_commands)) {
+			r = ast_iostream_gets(eivr_commands, input, sizeof(input));
+			if (r <= 0) {
+				if (r == 0) {
+					ast_chan_log(LOG_ERROR, chan, "Child process went away\n");
+					break;
+				}
 				continue;
 			}
 
@@ -869,16 +869,19 @@ static int eivr_comm(struct ast_channel
  				else
  					ast_chan_log(LOG_WARNING, chan, "Unknown option requested: %s\n", &input[2]);
  			}
- 		} else if (eivr_errors_fd && (ready_fd == *eivr_errors_fd)) {
- 			char input[1024];
-
- 			if (exception || feof(eivr_errors)) {
+ 		} else if (ready_fd == waitfds[1]) {
+ 			if (exception) {
  				ast_chan_log(LOG_ERROR, chan, "Child process went away\n");
  				break;
  			}
- 			if (fgets(input, sizeof(input), eivr_errors)) {
+
+ 			r = ast_iostream_gets(eivr_errors, input, sizeof(input));
+ 			if (r > 0) {
  				ast_chan_log(LOG_NOTICE, chan, "stderr: %s\n", ast_strip(input));
- 			}
+ 			} else if (r == 0) {
+ 				ast_chan_log(LOG_ERROR, chan, "Child process went away\n");
+ 				break;
+			}
  		} else if ((ready_fd < 0) && ms) {
  			if (errno == 0 || errno == EINTR)
  				continue;
@@ -888,23 +891,7 @@ static int eivr_comm(struct ast_channel
  		}
  	}
 
-	exit:
-	if (test_available_fd > -1) {
-		close(test_available_fd);
-	}
-	if (eivr_events) {
- 		fclose(eivr_events);
-		*eivr_events_fd = -1;
-	}
-	if (eivr_commands) {
-		fclose(eivr_commands);
-		*eivr_commands_fd = -1;
-	}
-	if (eivr_errors) {
-		fclose(eivr_errors);
-		*eivr_errors_fd = -1;
-	}
-  	return res;
+	return res;
 }
 
 static int unload_module(void)
diff -pruN 1:13.23.1~dfsg-2/apps/app_fax.c 1:16.1.1~dfsg-1/apps/app_fax.c
--- 1:13.23.1~dfsg-2/apps/app_fax.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_fax.c	2018-12-26 18:45:13.000000000 +0000
@@ -16,13 +16,12 @@
 	<defaultenabled>no</defaultenabled>
 	<depend>spandsp</depend>
 	<conflict>res_fax</conflict>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
+	<replacement>res_fax</replacement>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -996,7 +995,7 @@ static int load_module(void)
 
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Simple FAX Application",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		);
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_festival.c 1:16.1.1~dfsg-1/apps/app_festival.c
--- 1:13.23.1~dfsg-2/apps/app_festival.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_festival.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,8 +42,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
diff -pruN 1:13.23.1~dfsg-2/apps/app_flash.c 1:16.1.1~dfsg-1/apps/app_flash.c
--- 1:13.23.1~dfsg-2/apps/app_flash.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_flash.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dahdi/user.h>
 
 #include "asterisk/lock.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_followme.c 1:16.1.1~dfsg-1/apps/app_followme.c
--- 1:13.23.1~dfsg-2/apps/app_followme.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_followme.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_SPOOL_DIR */
@@ -168,6 +166,8 @@ struct call_followme {
 	char context[AST_MAX_CONTEXT];  /*!< Context to dial from */
 	unsigned int active;		/*!< Profile is active (1), or disabled (0). */
 	int realtime;           /*!< Cached from realtime */
+	/*! Allow callees to accept/reject the forwarded call */
+	unsigned int enable_callee_prompt:1;
 	char takecall[MAX_YN_STRING];	/*!< Digit mapping to take a call */
 	char nextindp[MAX_YN_STRING];	/*!< Digit mapping to decline a call */
 	char callfromprompt[PATH_MAX];	/*!< Sound prompt name and path */
@@ -199,6 +199,8 @@ struct fm_args {
 	unsigned int pending_out_connected_update:1;
 	/*! TRUE if caller has a pending hold request for the winning call. */
 	unsigned int pending_hold:1;
+	/*! TRUE if callees will be prompted to answer */
+	unsigned int enable_callee_prompt:1;
 	/*! Music On Hold Class suggested by caller hold for winning call. */
 	char suggested_moh[MAX_MUSICCLASS];
 	char context[AST_MAX_CONTEXT];
@@ -270,6 +272,7 @@ static const char *defaultmoh = "default
 
 static char takecall[MAX_YN_STRING] = "1";
 static char nextindp[MAX_YN_STRING] = "2";
+static int enable_callee_prompt = 1;
 static char callfromprompt[PATH_MAX] = "followme/call-from";
 static char norecordingprompt[PATH_MAX] = "followme/no-recording";
 static char optionsprompt[PATH_MAX] = "followme/options";
@@ -322,6 +325,7 @@ static struct call_followme *alloc_profi
 
 static void init_profile(struct call_followme *f, int activate)
 {
+	f->enable_callee_prompt = enable_callee_prompt;
 	f->context[0] = '\0';
 	ast_copy_string(f->moh, defaultmoh, sizeof(f->moh));
 	ast_copy_string(f->takecall, takecall, sizeof(f->takecall));
@@ -348,6 +352,8 @@ static void profile_set_param(struct cal
 		ast_copy_string(f->moh, val, sizeof(f->moh));
 	else if (!strcasecmp(param, "context"))
 		ast_copy_string(f->context, val, sizeof(f->context));
+	else if (!strcasecmp(param, "enable_callee_prompt"))
+		f->enable_callee_prompt = ast_true(val);
 	else if (!strcasecmp(param, "takecall"))
 		ast_copy_string(f->takecall, val, sizeof(f->takecall));
 	else if (!strcasecmp(param, "declinecall"))
@@ -405,6 +411,7 @@ static int reload_followme(int reload)
 	char *numberstr;
 	int timeout;
 	int numorder;
+	const char* enable_callee_prompt_str;
 	const char *takecallstr;
 	const char *declinecallstr;
 	const char *tmpstr;
@@ -437,6 +444,12 @@ static int reload_followme(int reload)
 			featuredigittimeout = 5000;
 	}
 
+	if ((enable_callee_prompt_str = ast_variable_retrieve(cfg, "general",
+					"enable_callee_prompt")) &&
+			!ast_strlen_zero(enable_callee_prompt_str)) {
+		enable_callee_prompt = ast_true(enable_callee_prompt_str);
+	}
+
 	if ((takecallstr = ast_variable_retrieve(cfg, "general", "takecall")) && !ast_strlen_zero(takecallstr)) {
 		ast_copy_string(takecall, takecallstr, sizeof(takecall));
 	}
@@ -665,26 +678,30 @@ static struct ast_channel *wait_for_winn
 			if (tmpuser->digts && (tmpuser->digts > featuredigittimeout)) {
 				ast_verb(3, "<%s> We've been waiting for digits longer than we should have.\n",
 					ast_channel_name(tmpuser->ochan));
-				if (!ast_strlen_zero(tpargs->namerecloc)) {
-					tmpuser->state = 1;
-					tmpuser->digts = 0;
-					if (!ast_streamfile(tmpuser->ochan, callfromname, ast_channel_language(tmpuser->ochan))) {
-						ast_sched_runq(ast_channel_sched(tmpuser->ochan));
+				if (tpargs->enable_callee_prompt) {
+					if (!ast_strlen_zero(tpargs->namerecloc)) {
+						tmpuser->state = 1;
+						tmpuser->digts = 0;
+						if (!ast_streamfile(tmpuser->ochan, callfromname, ast_channel_language(tmpuser->ochan))) {
+							ast_sched_runq(ast_channel_sched(tmpuser->ochan));
+						} else {
+							ast_log(LOG_WARNING, "Unable to playback %s.\n", callfromname);
+							clear_caller(tmpuser);
+							continue;
+						}
 					} else {
-						ast_log(LOG_WARNING, "Unable to playback %s.\n", callfromname);
-						clear_caller(tmpuser);
-						continue;
+						tmpuser->state = 2;
+						tmpuser->digts = 0;
+						if (!ast_streamfile(tmpuser->ochan, tpargs->norecordingprompt, ast_channel_language(tmpuser->ochan)))
+							ast_sched_runq(ast_channel_sched(tmpuser->ochan));
+						else {
+							ast_log(LOG_WARNING, "Unable to playback %s.\n", tpargs->norecordingprompt);
+							clear_caller(tmpuser);
+							continue;
+						}
 					}
 				} else {
-					tmpuser->state = 2;
-					tmpuser->digts = 0;
-					if (!ast_streamfile(tmpuser->ochan, tpargs->norecordingprompt, ast_channel_language(tmpuser->ochan)))
-						ast_sched_runq(ast_channel_sched(tmpuser->ochan));
-					else {
-						ast_log(LOG_WARNING, "Unable to playback %s.\n", tpargs->norecordingprompt);
-						clear_caller(tmpuser);
-						continue;
-					}
+					tmpuser->state = 3;
 				}
 			}
 			if (ast_channel_stream(tmpuser->ochan)) {
@@ -801,23 +818,28 @@ static struct ast_channel *wait_for_winn
 						/* If call has been answered, then the eventual hangup is likely to be normal hangup */
 						ast_channel_hangupcause_set(winner, AST_CAUSE_NORMAL_CLEARING);
 						ast_channel_hangupcause_set(caller, AST_CAUSE_NORMAL_CLEARING);
-						ast_verb(3, "Starting playback of %s\n", callfromname);
-						if (!ast_strlen_zero(tpargs->namerecloc)) {
-							if (!ast_streamfile(winner, callfromname, ast_channel_language(winner))) {
-								ast_sched_runq(ast_channel_sched(winner));
-								tmpuser->state = 1;
+						if (tpargs->enable_callee_prompt) {
+							ast_verb(3, "Starting playback of %s\n", callfromname);
+							if (!ast_strlen_zero(tpargs->namerecloc)) {
+								if (!ast_streamfile(winner, callfromname, ast_channel_language(winner))) {
+									ast_sched_runq(ast_channel_sched(winner));
+									tmpuser->state = 1;
+								} else {
+									ast_log(LOG_WARNING, "Unable to playback %s.\n", callfromname);
+									clear_caller(tmpuser);
+								}
 							} else {
-								ast_log(LOG_WARNING, "Unable to playback %s.\n", callfromname);
-								clear_caller(tmpuser);
+								tmpuser->state = 2;
+								if (!ast_streamfile(tmpuser->ochan, tpargs->norecordingprompt, ast_channel_language(tmpuser->ochan)))
+									ast_sched_runq(ast_channel_sched(tmpuser->ochan));
+								else {
+									ast_log(LOG_WARNING, "Unable to playback %s.\n", tpargs->norecordingprompt);
+									clear_caller(tmpuser);
+								}
 							}
 						} else {
+							ast_verb(3, "Skip playback of caller name / norecording\n");
 							tmpuser->state = 2;
-							if (!ast_streamfile(tmpuser->ochan, tpargs->norecordingprompt, ast_channel_language(tmpuser->ochan)))
-								ast_sched_runq(ast_channel_sched(tmpuser->ochan));
-							else {
-								ast_log(LOG_WARNING, "Unable to playback %s.\n", tpargs->norecordingprompt);
-								clear_caller(tmpuser);
-							}
 						}
 						break;
 					case AST_CONTROL_BUSY:
@@ -838,9 +860,11 @@ static struct ast_channel *wait_for_winn
 						break;
 					case AST_CONTROL_RINGING:
 						ast_verb(3, "%s is ringing\n", ast_channel_name(winner));
+						ast_channel_publish_dial(caller, winner, NULL, "RINGING");
 						break;
 					case AST_CONTROL_PROGRESS:
 						ast_verb(3, "%s is making progress\n", ast_channel_name(winner));
+						ast_channel_publish_dial(caller, winner, NULL, "PROGRESS");
 						break;
 					case AST_CONTROL_VIDUPDATE:
 						ast_verb(3, "%s requested a video update\n", ast_channel_name(winner));
@@ -850,6 +874,7 @@ static struct ast_channel *wait_for_winn
 						break;
 					case AST_CONTROL_PROCEEDING:
 						ast_verb(3, "%s is proceeding\n", ast_channel_name(winner));
+						ast_channel_publish_dial(caller, winner, NULL, "PROCEEDING");
 						break;
 					case AST_CONTROL_HOLD:
 						ast_verb(3, "%s placed call on hold\n", ast_channel_name(winner));
@@ -890,6 +915,11 @@ static struct ast_channel *wait_for_winn
 						/* Ignore going off hook and flash */
 						break;
 					case AST_CONTROL_CONNECTED_LINE:
+						if (ast_test_flag(&tpargs->followmeflags, FOLLOWMEFLAG_IGNORE_CONNECTEDLINE)) {
+							ast_verb(3, "Connected line update from %s prevented.\n",
+								ast_channel_name(winner));
+							break;
+						}
 						if (!tmpuser) {
 							/*
 							 * Hold connected line update from caller until we have a
@@ -905,11 +935,6 @@ static struct ast_channel *wait_for_winn
 								tpargs->pending_in_connected_update = 1;
 							}
 							ast_party_connected_line_free(&connected);
-							break;
-						}
-						if (ast_test_flag(&tpargs->followmeflags, FOLLOWMEFLAG_IGNORE_CONNECTEDLINE)) {
-							ast_verb(3, "Connected line update from %s prevented.\n",
-								ast_channel_name(winner));
 						} else {
 							ast_verb(3,
 								"%s connected line has changed. Saving it until answer.\n",
@@ -941,6 +966,11 @@ static struct ast_channel *wait_for_winn
 						break;
 					}
 				}
+				if (!tpargs->enable_callee_prompt && tmpuser) {
+					ast_debug(1, "Taking call with no prompt\n");
+					ast_frfree(f);
+					return tmpuser->ochan;
+				}
 				if (tmpuser && tmpuser->state == 3 && f->frametype == AST_FRAME_DTMF) {
 					int cmp_len;
 
@@ -1380,6 +1410,7 @@ static int app_exec(struct ast_channel *
 
 	/* Lock the profile lock and copy out everything we need to run with before unlocking it again */
 	ast_mutex_lock(&f->lock);
+	targs->enable_callee_prompt = f->enable_callee_prompt;
 	targs->mohclass = ast_strdupa(f->moh);
 	ast_copy_string(targs->context, f->context, sizeof(targs->context));
 	ast_copy_string(targs->takecall, f->takecall, sizeof(targs->takecall));
@@ -1619,8 +1650,8 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Find-Me/Follow-Me Application",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_forkcdr.c 1:16.1.1~dfsg-1/apps/app_forkcdr.c
--- 1:13.23.1~dfsg-2/apps/app_forkcdr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_forkcdr.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -217,4 +215,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Fork The CDR into 2 separate entities");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Fork The CDR into 2 separate entities",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_getcpeid.c 1:16.1.1~dfsg-1/apps/app_getcpeid.c
--- 1:13.23.1~dfsg-2/apps/app_getcpeid.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_getcpeid.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,13 +26,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -139,8 +137,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Get ADSI CPE ID",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.nonoptreq = "res_adsi",
-		);
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_adsi",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_ices.c 1:16.1.1~dfsg-1/apps/app_ices.c
--- 1:13.23.1~dfsg-2/apps/app_ices.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_ices.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,13 +28,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 #include <fcntl.h>
 #include <sys/time.h>
@@ -213,4 +211,4 @@ static int load_module(void)
 	return ast_register_application_xml(app, ices_exec);
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Encode and Stream via icecast and ices");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Encode and Stream via icecast and ices");
diff -pruN 1:13.23.1~dfsg-2/apps/app_image.c 1:16.1.1~dfsg-1/apps/app_image.c
--- 1:13.23.1~dfsg-2/apps/app_image.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_image.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,13 +26,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/image.h"
@@ -106,4 +104,4 @@ static int load_module(void)
 	return ast_register_application_xml(app, sendimage_exec);
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Image Transmission Application");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Image Transmission Application");
diff -pruN 1:13.23.1~dfsg-2/apps/app_ivrdemo.c 1:16.1.1~dfsg-1/apps/app_ivrdemo.c
--- 1:13.23.1~dfsg-2/apps/app_ivrdemo.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_ivrdemo.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_jack.c 1:16.1.1~dfsg-1/apps/app_jack.c
--- 1:13.23.1~dfsg-2/apps/app_jack.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_jack.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,8 +42,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <limits.h>
 
 #include <jack/jack.h>
@@ -497,7 +495,7 @@ static int init_jack_data(struct ast_cha
 				jack_port_name(jack_data->output_port));
 		}
 
-		free((void *) ports);
+		jack_free(ports);
 
 		break;
 	}
@@ -528,7 +526,7 @@ static int init_jack_data(struct ast_cha
 				jack_port_name(jack_data->input_port));
 		}
 
-		free((void *) ports);
+		jack_free(ports);
 
 		break;
 	}
diff -pruN 1:13.23.1~dfsg-2/apps/app_macro.c 1:16.1.1~dfsg-1/apps/app_macro.c
--- 1:13.23.1~dfsg-2/apps/app_macro.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_macro.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,14 +26,13 @@
  */
 
 /*** MODULEINFO
-	<support_level>core</support_level>
+	<defaultenabled>no</defaultenabled>
+	<support_level>deprecated</support_level>
 	<replacement>app_stack (GoSub)</replacement>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -192,8 +191,8 @@ static struct ast_exten *find_matching_p
 	int priority, const char *callerid, int iter, int *had_error)
 {
 	struct ast_exten *e;
-	struct ast_include *i;
 	struct ast_context *c2;
+	int idx;
 
 	if (iter >= AST_PBX_MAX_STACK) {
 		if (!(*had_error)) {
@@ -220,7 +219,9 @@ static struct ast_exten *find_matching_p
 	}
 
 	/* No match; run through includes */
-	for (i=ast_walk_context_includes(c, NULL); i; i=ast_walk_context_includes(c, i)) {
+	for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+		const struct ast_include *i = ast_context_includes_get(c, idx);
+
 		for (c2=ast_walk_contexts(NULL); c2; c2=ast_walk_contexts(c2)) {
 			if (!strcmp(ast_get_context_name(c2), ast_get_include_name(i))) {
 				e = find_matching_priority(c2, exten, priority, callerid, iter + 1, had_error);
@@ -261,12 +262,19 @@ static int _macro_exec(struct ast_channe
 	int save_in_subroutine;
 	struct ast_datastore *macro_store = ast_channel_datastore_find(chan, &macro_ds_info, NULL);
 	int had_infinite_include_error = 0;
+	static int deprecation_notice = 0;
 
 	if (ast_strlen_zero(data)) {
 		ast_log(LOG_WARNING, "Macro() requires arguments. See \"core show application macro\" for help.\n");
 		return -1;
 	}
 
+	if (!deprecation_notice) {
+		deprecation_notice = 1;
+		ast_log(LOG_WARNING, "Macro() is deprecated and will be removed from a future version of Asterisk.\n");
+		ast_log(LOG_WARNING, "Dialplan should be updated to use Gosub instead.\n");
+	}
+
 	do {
 		if (macro_store) {
 			break;
@@ -678,4 +686,4 @@ static int load_module(void)
 	return res;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Extension Macros");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Extension Macros");
diff -pruN 1:13.23.1~dfsg-2/apps/app_meetme.c 1:16.1.1~dfsg-1/apps/app_meetme.c
--- 1:13.23.1~dfsg-2/apps/app_meetme.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_meetme.c	2018-12-26 18:45:13.000000000 +0000
@@ -47,8 +47,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dahdi/user.h>
 
 #include "asterisk/lock.h"
@@ -71,7 +69,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/dial.h"
 #include "asterisk/causes.h"
 #include "asterisk/paths.h"
-#include "asterisk/data.h"
 #include "asterisk/test.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_channels.h"
@@ -1627,8 +1624,14 @@ static struct ast_conference *build_conf
 
 	ast_format_cap_append(cap_slin, ast_format_slin, 0);
 	/* Make a new one */
-	if (!(cnf = ast_calloc(1, sizeof(*cnf))) ||
-		!(cnf->usercontainer = ao2_container_alloc(1, NULL, user_no_cmp))) {
+	cnf = ast_calloc(1, sizeof(*cnf));
+	if (!cnf) {
+		goto cnfout;
+	}
+
+	cnf->usercontainer = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NULL, user_no_cmp);
+	if (!cnf->usercontainer) {
 		goto cnfout;
 	}
 
@@ -3591,7 +3594,7 @@ static int conf_run(struct ast_channel *
 	}
 
 	/* Reduce background noise from each participant */
-	if (!ast_test_flag64(confflags, CONFFLAG_DONT_DENOISE) && ast_module_check("func_speex.so")) {
+	if (!ast_test_flag64(confflags, CONFFLAG_DONT_DENOISE)) {
 		ast_func_write(chan, "DENOISE(rx)", "on");
 	}
 
@@ -7398,13 +7401,6 @@ static void sla_station_destructor(void
 	ast_string_field_free_memory(station);
 }
 
-static int sla_trunk_hash(const void *obj, const int flags)
-{
-	const struct sla_trunk *trunk = obj;
-
-	return ast_str_case_hash(trunk->name);
-}
-
 static int sla_trunk_cmp(void *obj, void *arg, int flags)
 {
 	struct sla_trunk *trunk = obj, *trunk2 = arg;
@@ -7412,13 +7408,6 @@ static int sla_trunk_cmp(void *obj, void
 	return !strcasecmp(trunk->name, trunk2->name) ? CMP_MATCH | CMP_STOP : 0;
 }
 
-static int sla_station_hash(const void *obj, const int flags)
-{
-	const struct sla_station *station = obj;
-
-	return ast_str_case_hash(station->name);
-}
-
 static int sla_station_cmp(void *obj, void *arg, int flags)
 {
 	struct sla_station *station = obj, *station2 = arg;
@@ -7872,8 +7861,10 @@ static int sla_load_config(int reload)
 	if (!reload) {
 		ast_mutex_init(&sla.lock);
 		ast_cond_init(&sla.cond, NULL);
-		sla_trunks = ao2_container_alloc(1, sla_trunk_hash, sla_trunk_cmp);
-		sla_stations = ao2_container_alloc(1, sla_station_hash, sla_station_cmp);
+		sla_trunks = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+			NULL, sla_trunk_cmp);
+		sla_stations = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+			NULL, sla_station_cmp);
 	}
 
 	if (!(cfg = ast_config_load(SLA_CONFIG_FILE, config_flags))) {
@@ -8008,186 +7999,6 @@ static int load_config(int reload)
 	return sla_load_config(reload);
 }
 
-#define MEETME_DATA_EXPORT(MEMBER)					\
-	MEMBER(ast_conference, confno, AST_DATA_STRING)			\
-	MEMBER(ast_conference, dahdiconf, AST_DATA_INTEGER)		\
-	MEMBER(ast_conference, users, AST_DATA_INTEGER)			\
-	MEMBER(ast_conference, markedusers, AST_DATA_INTEGER)		\
-	MEMBER(ast_conference, maxusers, AST_DATA_INTEGER)		\
-	MEMBER(ast_conference, isdynamic, AST_DATA_BOOLEAN)		\
-	MEMBER(ast_conference, locked, AST_DATA_BOOLEAN)		\
-	MEMBER(ast_conference, recordingfilename, AST_DATA_STRING)	\
-	MEMBER(ast_conference, recordingformat, AST_DATA_STRING)	\
-	MEMBER(ast_conference, pin, AST_DATA_PASSWORD)			\
-	MEMBER(ast_conference, pinadmin, AST_DATA_PASSWORD)		\
-	MEMBER(ast_conference, start, AST_DATA_TIMESTAMP)		\
-	MEMBER(ast_conference, endtime, AST_DATA_TIMESTAMP)
-
-AST_DATA_STRUCTURE(ast_conference, MEETME_DATA_EXPORT);
-
-#define MEETME_USER_DATA_EXPORT(MEMBER)					\
-	MEMBER(ast_conf_user, user_no, AST_DATA_INTEGER)		\
-	MEMBER(ast_conf_user, talking, AST_DATA_BOOLEAN)		\
-	MEMBER(ast_conf_user, dahdichannel, AST_DATA_BOOLEAN)		\
-	MEMBER(ast_conf_user, jointime, AST_DATA_TIMESTAMP)		\
-	MEMBER(ast_conf_user, kicktime, AST_DATA_TIMESTAMP)		\
-	MEMBER(ast_conf_user, timelimit, AST_DATA_MILLISECONDS)		\
-	MEMBER(ast_conf_user, play_warning, AST_DATA_MILLISECONDS)	\
-	MEMBER(ast_conf_user, warning_freq, AST_DATA_MILLISECONDS)
-
-AST_DATA_STRUCTURE(ast_conf_user, MEETME_USER_DATA_EXPORT);
-
-static int user_add_provider_cb(void *obj, void *arg, int flags)
-{
-	struct ast_data *data_meetme_user;
-	struct ast_data *data_meetme_user_channel;
-	struct ast_data *data_meetme_user_volume;
-
-	struct ast_conf_user *user = obj;
-	struct ast_data *data_meetme_users = arg;
-
-	data_meetme_user = ast_data_add_node(data_meetme_users, "user");
-	if (!data_meetme_user) {
-		return 0;
-	}
-	/* user structure */
-	ast_data_add_structure(ast_conf_user, data_meetme_user, user);
-
-	/* user's channel */
-	data_meetme_user_channel = ast_data_add_node(data_meetme_user, "channel");
-	if (!data_meetme_user_channel) {
-		return 0;
-	}
-
-	ast_channel_data_add_structure(data_meetme_user_channel, user->chan, 1);
-
-	/* volume structure */
-	data_meetme_user_volume = ast_data_add_node(data_meetme_user, "listen-volume");
-	if (!data_meetme_user_volume) {
-		return 0;
-	}
-	ast_data_add_int(data_meetme_user_volume, "desired", user->listen.desired);
-	ast_data_add_int(data_meetme_user_volume, "actual", user->listen.actual);
-
-	data_meetme_user_volume = ast_data_add_node(data_meetme_user, "talk-volume");
-	if (!data_meetme_user_volume) {
-		return 0;
-	}
-	ast_data_add_int(data_meetme_user_volume, "desired", user->talk.desired);
-	ast_data_add_int(data_meetme_user_volume, "actual", user->talk.actual);
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Implements the meetme data provider.
- */
-static int meetme_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ast_conference *cnf;
-	struct ast_data *data_meetme, *data_meetme_users;
-
-	AST_LIST_LOCK(&confs);
-	AST_LIST_TRAVERSE(&confs, cnf, list) {
-		data_meetme = ast_data_add_node(data_root, "meetme");
-		if (!data_meetme) {
-			continue;
-		}
-
-		ast_data_add_structure(ast_conference, data_meetme, cnf);
-
-		if (ao2_container_count(cnf->usercontainer)) {
-			data_meetme_users = ast_data_add_node(data_meetme, "users");
-			if (!data_meetme_users) {
-				ast_data_remove_node(data_root, data_meetme);
-				continue;
-			}
-
-			ao2_callback(cnf->usercontainer, OBJ_NODATA, user_add_provider_cb, data_meetme_users);
-		}
-
-		if (!ast_data_search_match(search, data_meetme)) {
-			ast_data_remove_node(data_root, data_meetme);
-		}
-	}
-	AST_LIST_UNLOCK(&confs);
-
-	return 0;
-}
-
-static const struct ast_data_handler meetme_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = meetme_data_provider_get
-};
-
-static const struct ast_data_entry meetme_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/application/meetme/list", &meetme_data_provider),
-};
-
-#ifdef TEST_FRAMEWORK
-AST_TEST_DEFINE(test_meetme_data_provider)
-{
-	struct ast_channel *chan;
-	struct ast_conference *cnf;
-	struct ast_data *node;
-	struct ast_data_query query = {
-		.path = "/asterisk/application/meetme/list",
-		.search = "list/meetme/confno=9898"
-	};
-
-	switch (cmd) {
-	case TEST_INIT:
-		info->name = "meetme_get_data_test";
-		info->category = "/main/data/app_meetme/list/";
-		info->summary = "Meetme data provider unit test";
-		info->description =
-			"Tests whether the Meetme data provider implementation works as expected.";
-		return AST_TEST_NOT_RUN;
-	case TEST_EXECUTE:
-		break;
-	}
-
-	chan = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "MeetMeTest");
-	if (!chan) {
-		ast_test_status_update(test, "Channel allocation failed\n");
-		return AST_TEST_FAIL;
-	}
-
-	ast_channel_unlock(chan);
-
-	cnf = build_conf("9898", "", "1234", 1, 1, 1, chan, test);
-	if (!cnf) {
-		ast_test_status_update(test, "Build of test conference 9898 failed\n");
-		ast_hangup(chan);
-		return AST_TEST_FAIL;
-	}
-
-	node = ast_data_get(&query);
-	if (!node) {
-		ast_test_status_update(test, "Data query for test conference 9898 failed\n");
-		dispose_conf(cnf);
-		ast_hangup(chan);
-		return AST_TEST_FAIL;
-	}
-
-	if (strcmp(ast_data_retrieve_string(node, "meetme/confno"), "9898")) {
-		ast_test_status_update(test, "Query returned the wrong conference\n");
-		dispose_conf(cnf);
-		ast_hangup(chan);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	ast_data_free(node);
-	dispose_conf(cnf);
-	ast_hangup(chan);
-
-	return AST_TEST_PASS;
-}
-#endif
-
 static int unload_module(void)
 {
 	int res = 0;
@@ -8204,11 +8015,6 @@ static int unload_module(void)
 	res |= ast_unregister_application(slastation_app);
 	res |= ast_unregister_application(slatrunk_app);
 
-#ifdef TEST_FRAMEWORK
-	AST_TEST_UNREGISTER(test_meetme_data_provider);
-#endif
-	ast_data_unregister(NULL);
-
 	ast_devstate_prov_del("Meetme");
 	ast_devstate_prov_del("SLA");
 
@@ -8252,11 +8058,6 @@ static int load_module(void)
 	res |= ast_register_application_xml(slastation_app, sla_station_exec);
 	res |= ast_register_application_xml(slatrunk_app, sla_trunk_exec);
 
-#ifdef TEST_FRAMEWORK
-	AST_TEST_REGISTER(test_meetme_data_provider);
-#endif
-	ast_data_register_multiple(meetme_data_providers, ARRAY_LEN(meetme_data_providers));
-
 	res |= ast_devstate_prov_add("Meetme", meetmestate);
 	res |= ast_devstate_prov_add("SLA", sla_state);
 
@@ -8273,9 +8074,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "MeetMe conference bridge",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
+	.optional_modules = "func_speex",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_milliwatt.c 1:16.1.1~dfsg-1/apps/app_milliwatt.c
--- 1:13.23.1~dfsg-2/apps/app_milliwatt.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_milliwatt.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_minivm.c 1:16.1.1~dfsg-1/apps/app_minivm.c
--- 1:13.23.1~dfsg-2/apps/app_minivm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_minivm.c	2018-12-26 18:45:13.000000000 +0000
@@ -146,8 +146,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -3592,8 +3590,8 @@ static int unload_module(void)
 
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Mini VoiceMail (A minimal Voicemail e-mail System)",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_mixmonitor.c 1:16.1.1~dfsg-1/apps/app_mixmonitor.c
--- 1:13.23.1~dfsg-2/apps/app_mixmonitor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_mixmonitor.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/paths.h"	/* use ast_config_AST_MONITOR_DIR */
 #include "asterisk/stringfields.h"
 #include "asterisk/file.h"
@@ -311,12 +309,12 @@ struct vm_recipient {
 
 struct mixmonitor {
 	struct ast_audiohook audiohook;
-	struct ast_callid *callid;
 	char *filename;
 	char *filename_read;
 	char *filename_write;
 	char *post_process;
 	char *name;
+	ast_callid callid;
 	unsigned int flags;
 	struct ast_autochan *autochan;
 	struct mixmonitor_ds *mixmonitor_ds;
@@ -561,9 +559,6 @@ static void mixmonitor_free(struct mixmo
 		/* clean stringfields */
 		ast_string_field_free_memory(mixmonitor);
 
-		if (mixmonitor->callid) {
-			ast_callid_unref(mixmonitor->callid);
-		}
 		ast_free(mixmonitor);
 	}
 }
@@ -1561,4 +1556,9 @@ static int load_module(void)
 	return res;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Mixed Audio Monitoring Application");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Mixed Audio Monitoring Application",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.optional_modules = "func_periodic_hook",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_morsecode.c 1:16.1.1~dfsg-1/apps/app_morsecode.c
--- 1:13.23.1~dfsg-2/apps/app_morsecode.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_morsecode.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_mp3.c 1:16.1.1~dfsg-1/apps/app_mp3.c
--- 1:13.23.1~dfsg-2/apps/app_mp3.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_mp3.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <sys/types.h>
 #include <signal.h>
diff -pruN 1:13.23.1~dfsg-2/apps/app_nbscat.c 1:16.1.1~dfsg-1/apps/app_nbscat.c
--- 1:13.23.1~dfsg-2/apps/app_nbscat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_nbscat.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,13 +26,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <fcntl.h>
 #include <sys/time.h>
 #include <sys/socket.h>
@@ -222,4 +220,4 @@ static int load_module(void)
 	return ast_register_application_xml(app, NBScat_exec);
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Silly NBS Stream Application");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Silly NBS Stream Application");
diff -pruN 1:13.23.1~dfsg-2/apps/app_originate.c 1:16.1.1~dfsg-1/apps/app_originate.c
--- 1:13.23.1~dfsg-2/apps/app_originate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_originate.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -74,6 +72,34 @@ static const char app_originate[] = "Ori
 			<parameter name="timeout" required="false">
 				<para>Timeout in seconds. Default is 30 seconds.</para>
 			</parameter>
+			<parameter name="options" required="false">
+				<optionlist>
+				<option name="a">
+					<para>Originate asynchronously.  In other words, continue in the dialplan
+					without waiting for the originated channel to answer.</para>
+				</option>
+				<option name="b" argsep="^">
+					<para>Before originating the outgoing call, Gosub to the specified
+					location using the newly created channel.</para>
+					<argument name="context" required="false" />
+					<argument name="exten" required="false" />
+					<argument name="priority" required="true" hasparams="optional" argsep="^">
+						<argument name="arg1" multiple="true" required="true" />
+						<argument name="argN" />
+					</argument>
+				</option>
+				<option name="B" argsep="^">
+					<para>Before originating the outgoing call, Gosub to the specified
+					location using the current channel.</para>
+					<argument name="context" required="false" />
+					<argument name="exten" required="false" />
+					<argument name="priority" required="true" hasparams="optional" argsep="^">
+						<argument name="arg1" multiple="true" required="true" />
+						<argument name="argN" />
+					</argument>
+				</option>
+				</optionlist>
+			</parameter>
 		</syntax>
 		<description>
 		<para>This application originates an outbound call and connects it to a specified extension or application.  This application will block until the outgoing call fails or gets answered.  At that point, this application will exit with the status variable set and dialplan processing will continue.</para>
@@ -97,6 +123,27 @@ static const char app_originate[] = "Ori
 	</application>
  ***/
 
+
+enum {
+	OPT_PREDIAL_CALLEE =    (1 << 0),
+	OPT_PREDIAL_CALLER =    (1 << 1),
+	OPT_ASYNC =             (1 << 2),
+};
+
+enum {
+	OPT_ARG_PREDIAL_CALLEE,
+	OPT_ARG_PREDIAL_CALLER,
+	/* note: this entry _MUST_ be the last one in the enum */
+	OPT_ARG_ARRAY_SIZE,
+};
+
+AST_APP_OPTIONS(originate_exec_options, BEGIN_OPTIONS
+	AST_APP_OPTION('a', OPT_ASYNC),
+	AST_APP_OPTION_ARG('b', OPT_PREDIAL_CALLEE, OPT_ARG_PREDIAL_CALLEE),
+	AST_APP_OPTION_ARG('B', OPT_PREDIAL_CALLER, OPT_ARG_PREDIAL_CALLER),
+END_OPTIONS );
+
+
 static int originate_exec(struct ast_channel *chan, const char *data)
 {
 	AST_DECLARE_APP_ARGS(args,
@@ -106,7 +153,11 @@ static int originate_exec(struct ast_cha
 		AST_APP_ARG(arg2);
 		AST_APP_ARG(arg3);
 		AST_APP_ARG(timeout);
+		AST_APP_ARG(options);
 	);
+	struct ast_flags64 opts = { 0, };
+	char *opt_args[OPT_ARG_ARRAY_SIZE];
+	char *predial_callee = NULL;
 	char *parse;
 	char *chantech, *chandata;
 	int res = -1;
@@ -160,6 +211,25 @@ static int originate_exec(struct ast_cha
 		goto return_cleanup;
 	}
 
+	if (!ast_strlen_zero(args.options) &&
+		ast_app_parse_options64(originate_exec_options, &opts, opt_args, args.options)) {
+		ast_log(LOG_ERROR, "Invalid options: '%s'\n", args.options);
+		goto return_cleanup;
+	}
+
+	/* PREDIAL: Run gosub on the caller's channel */
+	if (ast_test_flag64(&opts, OPT_PREDIAL_CALLER)
+		&& !ast_strlen_zero(opt_args[OPT_ARG_PREDIAL_CALLER])) {
+		ast_replace_subargument_delimiter(opt_args[OPT_ARG_PREDIAL_CALLER]);
+		ast_app_exec_sub(NULL, chan, opt_args[OPT_ARG_PREDIAL_CALLER], 0);
+	}
+
+	if (ast_test_flag64(&opts, OPT_PREDIAL_CALLEE)
+		&& !ast_strlen_zero(opt_args[OPT_ARG_PREDIAL_CALLEE])) {
+		ast_replace_subargument_delimiter(opt_args[OPT_ARG_PREDIAL_CALLEE]);
+		predial_callee = opt_args[OPT_ARG_PREDIAL_CALLEE];
+	}
+
 	if (strcasecmp(args.type, "exten") && strcasecmp(args.type, "app")) {
 		ast_log(LOG_ERROR, "Incorrect type, it should be 'exten' or 'app': %s\n",
 				args.type);
@@ -184,16 +254,20 @@ static int originate_exec(struct ast_cha
 		ast_debug(1, "Originating call to '%s/%s' and connecting them to extension %s,%s,%d\n",
 				chantech, chandata, args.arg1, exten, priority);
 
-		res = ast_pbx_outgoing_exten(chantech, cap_slin, chandata,
+		res = ast_pbx_outgoing_exten_predial(chantech, cap_slin, chandata,
 				timeout * 1000, args.arg1, exten, priority, &outgoing_status,
-				AST_OUTGOING_WAIT, NULL, NULL, NULL, NULL, NULL, 0, NULL);
+				ast_test_flag64(&opts, OPT_ASYNC) ? AST_OUTGOING_NO_WAIT : AST_OUTGOING_WAIT,
+				NULL, NULL, NULL, NULL, NULL, 0, NULL,
+				predial_callee);
 	} else {
 		ast_debug(1, "Originating call to '%s/%s' and connecting them to %s(%s)\n",
 				chantech, chandata, args.arg1, S_OR(args.arg2, ""));
 
-		res = ast_pbx_outgoing_app(chantech, cap_slin, chandata,
+		res = ast_pbx_outgoing_app_predial(chantech, cap_slin, chandata,
 				timeout * 1000, args.arg1, args.arg2, &outgoing_status,
-				AST_OUTGOING_WAIT, NULL, NULL, NULL, NULL, NULL, NULL);
+				ast_test_flag64(&opts, OPT_ASYNC) ? AST_OUTGOING_NO_WAIT : AST_OUTGOING_WAIT,
+				NULL, NULL, NULL, NULL, NULL, NULL,
+				predial_callee);
 	}
 
 	/*
diff -pruN 1:13.23.1~dfsg-2/apps/app_osplookup.c 1:16.1.1~dfsg-1/apps/app_osplookup.c
--- 1:13.23.1~dfsg-2/apps/app_osplookup.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_osplookup.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <osp/osp.h>
 #include <osp/osputils.h>
 #include <osp/ospb64.h>
@@ -52,7 +50,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 #include "asterisk/cli.h"
-#include "asterisk/astosp.h"
 
 /*** DOCUMENTATION
 	<application name="OSPAuth" language="en_US">
@@ -438,6 +435,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 	</application>
  ***/
 
+/* OSP Return statuses */
+#define AST_OSP_SUCCESS	((char*)"SUCCESS")	/* Return status, success */
+#define AST_OSP_FAILED	((char*)"FAILED")	/* Return status, failed */
+#define AST_OSP_ERROR	((char*)"ERROR")	/* Return status, error */
+
 /* OSP Buffer Sizes */
 #define OSP_SIZE_INTSTR		((unsigned int)16)			/* OSP signed/unsigned int string buffer size */
 #define OSP_SIZE_NORSTR		((unsigned int)256)			/* OSP normal string buffer size */
diff -pruN 1:13.23.1~dfsg-2/apps/app_page.c 1:16.1.1~dfsg-1/apps/app_page.c
--- 1:13.23.1~dfsg-2/apps/app_page.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_page.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -442,4 +440,9 @@ static int load_module(void)
 	return ast_register_application_xml(app_page, page_exec);
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Page Multiple Phones");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Page Multiple Phones",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "app_confbridge",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_playback.c 1:16.1.1~dfsg-1/apps/app_playback.c
--- 1:13.23.1~dfsg-2/apps/app_playback.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_playback.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -572,8 +570,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Sound File Playback Application",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_playtones.c 1:16.1.1~dfsg-1/apps/app_playtones.c
--- 1:13.23.1~dfsg-2/apps/app_playtones.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_playtones.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_privacy.c 1:16.1.1~dfsg-1/apps/app_privacy.c
--- 1:13.23.1~dfsg-2/apps/app_privacy.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_privacy.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_queue.c 1:16.1.1~dfsg-1/apps/app_queue.c
--- 1:13.23.1~dfsg-2/apps/app_queue.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_queue.c	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 1999 - 2006, Digium, Inc.
+ * Copyright (C) 1999 - 2018, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
@@ -69,10 +69,8 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <netinet/in.h>
 #include <ctype.h>
 
@@ -101,7 +99,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/taskprocessor.h"
 #include "asterisk/aoc.h"
 #include "asterisk/callerid.h"
-#include "asterisk/data.h"
 #include "asterisk/term.h"
 #include "asterisk/dial.h"
 #include "asterisk/stasis_channels.h"
@@ -110,7 +107,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/stasis_bridges.h"
 #include "asterisk/core_local.h"
 #include "asterisk/mixmonitor.h"
-#include "asterisk/core_unreal.h"
 #include "asterisk/bridge_basic.h"
 #include "asterisk/max_forwards.h"
 
@@ -137,6 +133,27 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<parameter name="queuename" required="true" />
 			<parameter name="options">
 				<optionlist>
+					<option name="b" argsep="^">
+						<para>Before initiating an outgoing call, <literal>Gosub</literal> to the specified
+						location using the newly created channel.  The <literal>Gosub</literal> will be
+						executed for each destination channel.</para>
+						<argument name="context" required="false" />
+						<argument name="exten" required="false" />
+						<argument name="priority" required="true" hasparams="optional" argsep="^">
+							<argument name="arg1" multiple="true" required="true" />
+							<argument name="argN" />
+						</argument>
+					</option>
+					<option name="B" argsep="^">
+						<para>Before initiating the outgoing call(s), <literal>Gosub</literal> to the
+						specified location using the current channel.</para>
+						<argument name="context" required="false" />
+						<argument name="exten" required="false" />
+						<argument name="priority" required="true" hasparams="optional" argsep="^">
+							<argument name="arg1" multiple="true" required="true" />
+							<argument name="argN" />
+						</argument>
+					</option>
 					<option name="C">
 						<para>Mark all calls as "answered elsewhere" when cancelled.</para>
 					</option>
@@ -152,21 +169,15 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						<argument name="priority" required="true" />
 						<para>When the caller hangs up, transfer the <emphasis>called member</emphasis>
 						to the specified destination and <emphasis>start</emphasis> execution at that location.</para>
-						<note>
-							<para>Any channel variables you want the called channel to inherit from the caller channel must be
-							prefixed with one or two underbars ('_').</para>
-						</note>
+						<para>NOTE: Any channel variables you want the called channel to inherit from the caller channel must be
+						prefixed with one or two underbars ('_').</para>
 					</option>
 					<option name="F">
 						<para>When the caller hangs up, transfer the <emphasis>called member</emphasis> to the next priority of
 						the current extension and <emphasis>start</emphasis> execution at that location.</para>
-						<note>
-							<para>Any channel variables you want the called channel to inherit from the caller channel must be
-							prefixed with one or two underbars ('_').</para>
-						</note>
-						<note>
-							<para>Using this option from a Macro() or GoSub() might not make sense as there would be no return points.</para>
-						</note>
+						<para>NOTE: Any channel variables you want the called channel to inherit from the caller channel must be
+						prefixed with one or two underbars ('_').</para>
+						<para>NOTE: Using this option from a Macro() or GoSub() might not make sense as there would be no return points.</para>
 					</option>
 					<option name="h">
 						<para>Allow <emphasis>callee</emphasis> to hang up by pressing <literal>*</literal>.</para>
@@ -239,9 +250,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			</parameter>
 			<parameter name="macro">
 				<para>Will run a macro on the called party's channel (the queue member) once the parties are connected.</para>
+				<para>NOTE: Macros are deprecated, GoSub should be used instead.</para>
 			</parameter>
 			<parameter name="gosub">
-				<para>Will run a gosub on the called party's channel (the queue member) once the parties are connected.</para>
+				<para>Will run a gosub on the called party's channel (the queue member)
+				once the parties are connected.  The subroutine execution starts in the
+				named context at the s exten and priority 1.</para>
 			</parameter>
 			<parameter name="rule">
 				<para>Will cause the queue's defaultrule to be overridden by the rule specified.</para>
@@ -288,6 +302,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -329,6 +344,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -366,6 +382,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -410,6 +427,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -451,6 +469,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -482,11 +501,30 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
 		</see-also>
 	</application>
+	<application name="QueueUpdate" language="en_US">
+		<synopsis>
+			Writes to the queue_log file for OutBound calls and updates Realtime Data.
+			Is used at h extension to be able to have all the parameters.
+		</synopsis>
+		<syntax>
+			<parameter name="queuename" required="true" />
+			<parameter name="uniqueid" required="true" />
+			<parameter name="agent" required="true" />
+			<parameter name="status" required="true" />
+			<parameter name="talktime" required="true" />
+			<parameter name="params" required="false" />
+		</syntax>
+		<description>
+			<para>Allows you to write Outbound events into the queue log.</para>
+			<para>Example: exten => h,1,QueueUpdate(${QUEUE}, ${UNIQUEID}, ${AGENT}, ${DIALSTATUS}, ${ANSWEREDTIME}, ${DIALEDTIME} | ${DIALEDNUMBER})</para>
+		</description>
+	</application>
 	<function name="QUEUE_VARIABLES" language="en_US">
 		<synopsis>
 			Return Queue information in variables.
@@ -536,6 +574,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -599,6 +638,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -626,6 +666,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -652,6 +693,35 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
+			<ref type="function">QUEUE_WAITING_COUNT</ref>
+			<ref type="function">QUEUE_MEMBER_LIST</ref>
+			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
+		</see-also>
+	</function>
+	<function name="QUEUE_GET_CHANNEL" language="en_US">
+		<synopsis>
+			Return caller at the specified position in a queue.
+		</synopsis>
+		<syntax>
+			<parameter name="queuename" required="true" />
+			<parameter name="position" />
+		</syntax>
+		<description>
+			<para>Returns the caller channel at <replaceable>position</replaceable> in the specified <replaceable>queuename</replaceable>.</para>
+			<para>If <replaceable>position</replaceable> is unspecified the first channel is returned.</para>
+		</description>
+		<see-also>
+			<ref type="application">Queue</ref>
+			<ref type="application">QueueLog</ref>
+			<ref type="application">AddQueueMember</ref>
+			<ref type="application">RemoveQueueMember</ref>
+			<ref type="application">PauseQueueMember</ref>
+			<ref type="application">UnpauseQueueMember</ref>
+			<ref type="function">QUEUE_VARIABLES</ref>
+			<ref type="function">QUEUE_MEMBER</ref>
+			<ref type="function">QUEUE_MEMBER_COUNT</ref>
+			<ref type="function">QUEUE_EXISTS</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -678,6 +748,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -704,6 +775,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
@@ -732,21 +804,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<ref type="function">QUEUE_MEMBER</ref>
 			<ref type="function">QUEUE_MEMBER_COUNT</ref>
 			<ref type="function">QUEUE_EXISTS</ref>
+			<ref type="function">QUEUE_GET_CHANNEL</ref>
 			<ref type="function">QUEUE_WAITING_COUNT</ref>
 			<ref type="function">QUEUE_MEMBER_LIST</ref>
 			<ref type="function">QUEUE_MEMBER_PENALTY</ref>
 		</see-also>
 	</function>
-	<manager name="Queues" language="en_US">
-		<synopsis>
-			Queues.
-		</synopsis>
-		<syntax>
-		</syntax>
-		<description>
-			<para>Show queues information.</para>
-		</description>
-	</manager>
 	<manager name="QueueStatus" language="en_US">
 		<synopsis>
 			Show queue status.
@@ -953,6 +1016,26 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<para>Reset the statistics for a queue.</para>
 		</description>
 	</manager>
+	<manager name="QueueChangePriorityCaller" language="en_US">
+		<synopsis>
+			Change priority of a caller on queue.
+		</synopsis>
+		<syntax>
+			<xi:include xpointer="xpointer(/docs/manager[@name='Login']/syntax/parameter[@name='ActionID'])" />
+			<parameter name="Queue" required="true">
+				<para>The name of the queue to take action on.</para>
+			</parameter>
+			<parameter name="Caller" required="true">
+				<para>The caller (channel) to change priority on queue.</para>
+			</parameter>
+
+			<parameter name="Priority" required="true">
+				<para>Priority value for change for caller on queue.</para>
+			</parameter>
+		</syntax>
+		<description>
+		</description>
+	</manager>
 
 	<managerEvent language="en_US" name="QueueMemberStatus">
 		<managerEventInstance class="EVENT_FLAG_AGENT">
@@ -986,6 +1069,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 				<parameter name="LastCall">
 					<para>The time this member last took a call, expressed in seconds since 00:00, Jan 1, 1970 UTC.</para>
 				</parameter>
+				<parameter name="LastPause">
+					<para>The time when started last paused the queue member.</para>
+				</parameter>
 				<parameter name="InCall">
 					<para>Set to 1 if member is in call. Set to 0 after LastCall time is updated.</para>
 					<enumlist>
@@ -1254,15 +1340,21 @@ enum {
 	OPT_CALLER_AUTOMIXMON =      (1 << 16),
 	OPT_CALLEE_AUTOMON =         (1 << 17),
 	OPT_CALLER_AUTOMON =         (1 << 18),
+	OPT_PREDIAL_CALLEE =         (1 << 19),
+	OPT_PREDIAL_CALLER =         (1 << 20),
 };
 
 enum {
 	OPT_ARG_CALLEE_GO_ON = 0,
+	OPT_ARG_PREDIAL_CALLEE,
+	OPT_ARG_PREDIAL_CALLER,
 	/* note: this entry _MUST_ be the last one in the enum */
 	OPT_ARG_ARRAY_SIZE
 };
 
 AST_APP_OPTIONS(queue_exec_options, BEGIN_OPTIONS
+	AST_APP_OPTION_ARG('b', OPT_PREDIAL_CALLEE, OPT_ARG_PREDIAL_CALLEE),
+	AST_APP_OPTION_ARG('B', OPT_PREDIAL_CALLER, OPT_ARG_PREDIAL_CALLER),
 	AST_APP_OPTION('C', OPT_MARK_AS_ANSWERED),
 	AST_APP_OPTION('c', OPT_GO_ON),
 	AST_APP_OPTION('d', OPT_DATA_QUALITY),
@@ -1296,9 +1388,9 @@ enum {
 };
 
 enum {
-     QUEUE_AUTOPAUSE_OFF = 0,
-     QUEUE_AUTOPAUSE_ON,
-     QUEUE_AUTOPAUSE_ALL
+	QUEUE_AUTOPAUSE_OFF = 0,
+	QUEUE_AUTOPAUSE_ON,
+	QUEUE_AUTOPAUSE_ALL
 };
 
 enum queue_reload_mask {
@@ -1349,6 +1441,7 @@ static const struct autopause {
 #define	RES_OUTOFMEMORY	(-2)		/*!< Out of memory */
 #define	RES_NOSUCHQUEUE	(-3)		/*!< No such queue */
 #define RES_NOT_DYNAMIC (-4)		/*!< Member is not dynamic */
+#define RES_NOT_CALLER  (-5)		/*!< Caller not found */
 
 static char *app = "Queue";
 
@@ -1362,6 +1455,8 @@ static char *app_upqm = "UnpauseQueueMem
 
 static char *app_ql = "QueueLog" ;
 
+static char *app_qupd = "QueueUpdate";
+
 /*! \brief Persistent Members astdb family */
 static const char * const pm_family = "Queue/PersistentMembers";
 
@@ -1471,6 +1566,7 @@ struct queue_ent {
 	char announce[PATH_MAX];               /*!< Announcement to play for member when call is answered */
 	char context[AST_MAX_CONTEXT];         /*!< Context when user exits queue */
 	char digits[AST_MAX_EXTENSION];        /*!< Digits entered while in queue */
+	const char *predial_callee;            /*!< Gosub app arguments for outgoing calls.  NULL if not supplied. */
 	int valid_digits;                      /*!< Digits entered correspond to valid extension. Exited */
 	int pos;                               /*!< Where we are in the queue */
 	int prio;                              /*!< Our priority */
@@ -1484,11 +1580,12 @@ struct queue_ent {
 	int pending;                           /*!< Non-zero if we are attempting to call a member */
 	int max_penalty;                       /*!< Limit the members that can take this call to this penalty or lower */
 	int min_penalty;                       /*!< Limit the members that can take this call to this penalty or higher */
+	int raise_penalty;                     /*!< Float lower penalty mambers to a minimum penalty */
 	int linpos;                            /*!< If using linear strategy, what position are we at? */
 	int linwrapped;                        /*!< Is the linpos wrapped? */
 	time_t start;                          /*!< When we started holding */
 	time_t expire;                         /*!< When this entry should expire (time out of queue) */
-	int cancel_answered_elsewhere;	       /*!< Whether we should force the CAE flag on this call (C) option*/
+	int cancel_answered_elsewhere;         /*!< Whether we should force the CAE flag on this call (C) option*/
 	struct ast_channel *chan;              /*!< Our channel */
 	AST_LIST_HEAD_NOLOCK(,penalty_rule) qe_rules; /*!< Local copy of the queue's penalty rules */
 	struct penalty_rule *pr;               /*!< Pointer to the next penalty rule to implement */
@@ -1511,8 +1608,10 @@ struct member {
 	char reason_paused[80];              /*!< Reason of paused if member is paused */
 	int queuepos;                        /*!< In what order (pertains to certain strategies) should this member be called? */
 	int callcompletedinsl;               /*!< Whether the current call was completed within service level */
+	int wrapuptime;                      /*!< Wrapup Time */
 	time_t starttime;                    /*!< The time at which the member answered the current caller. */
 	time_t lastcall;                     /*!< When last successful call was hungup */
+	time_t lastpause;                    /*!< When started the last pause */
 	struct call_queue *lastqueue;	     /*!< Last queue we received a call */
 	unsigned int dead:1;                 /*!< Used to detect members deleted in realtime */
 	unsigned int delme:1;                /*!< Flag to delete entry on reload */
@@ -1545,8 +1644,10 @@ struct penalty_rule {
 	int time;                           /*!< Number of seconds that need to pass before applying this rule */
 	int max_value;                      /*!< The amount specified in the penalty rule for max penalty */
 	int min_value;                      /*!< The amount specified in the penalty rule for min penalty */
+	int raise_value;                      /*!< The amount specified in the penalty rule for min penalty */
 	int max_relative;                   /*!< Is the max adjustment relative? 1 for relative, 0 for absolute */
 	int min_relative;                   /*!< Is the min adjustment relative? 1 for relative, 0 for absolute */
+	int raise_relative;                   /*!< Is the min adjustment relative? 1 for relative, 0 for absolute */
 	AST_LIST_ENTRY(penalty_rule) list;  /*!< Next penalty_rule */
 };
 
@@ -1609,6 +1710,7 @@ struct call_queue {
 	unsigned int timeoutrestart:1;
 	unsigned int announceholdtime:2;
 	unsigned int announceposition:3;
+	unsigned int announceposition_only_up:1; /*!< Only announce position if it has improved */
 	int strategy:4;
 	unsigned int realtime:1;
 	unsigned int found:1;
@@ -1628,6 +1730,7 @@ struct call_queue {
 	int talktime;                       /*!< Current avg talktime, based on the same exponential average */
 	int callscompleted;                 /*!< Number of queue calls completed */
 	int callsabandoned;                 /*!< Number of queue calls abandoned */
+	int callsabandonedinsl;             /*!< Number of queue calls abandoned in servicelevel */
 	int servicelevel;                   /*!< seconds setting for servicelevel*/
 	int callscompletedinsl;             /*!< Number of calls answered with servicelevel*/
 	char monfmt[8];                     /*!< Format to use when recording calls */
@@ -1649,7 +1752,7 @@ struct call_queue {
 	int memberdelay;                    /*!< Seconds to delay connecting member to caller */
 	int autofill;                       /*!< Ignore the head call status and ring an available agent */
 
-	struct ao2_container *members;             /*!< Head of the list of members */
+	struct ao2_container *members;      /*!< Head of the list of members */
 	struct queue_ent *head;             /*!< Head of the list of callers */
 	AST_LIST_ENTRY(call_queue) list;    /*!< Next call queue */
 	AST_LIST_HEAD_NOLOCK(, penalty_rule) rules; /*!< The list of penalty rules to invoke */
@@ -1746,6 +1849,21 @@ static int queue_cmp_cb(void *obj, void
 	return !strcasecmp(q->name, q2->name) ? CMP_MATCH | CMP_STOP : 0;
 }
 
+/*!
+ * \brief Return wrapuptime
+ *
+ * This function checks if wrapuptime in member is set and return this value.
+ * Otherwise return value the wrapuptime in the queue configuration
+ * \return integer value
+ */
+static int get_wrapuptime(struct call_queue *q, struct member *member)
+{
+	if (member->wrapuptime) {
+		return member->wrapuptime;
+	}
+	return q->wrapuptime;
+}
+
 /*! \internal
  * \brief ao2_callback, Decreases queuepos of all followers with a queuepos greater than arg.
  * \param obj the member being acted on
@@ -1800,50 +1918,13 @@ static void queue_member_follower_remova
 	ao2_callback(queue->members, OBJ_NODATA | OBJ_MULTIPLE, queue_member_decrement_followers, &pos);
 }
 
-#ifdef REF_DEBUG
-#define queue_ref(q)				_queue_ref(q, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define queue_unref(q)				_queue_unref(q, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define queue_t_ref(q, tag)			_queue_ref(q, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define queue_t_unref(q, tag)		_queue_unref(q, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define queues_t_link(c, q, tag)	__ao2_link_debug(c, q, 0, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define queues_t_unlink(c, q, tag)	__ao2_unlink_debug(c, q, 0, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-static inline struct call_queue *_queue_ref(struct call_queue *q, const char *tag, const char *file, int line, const char *filename)
-{
-	__ao2_ref_debug(q, 1, tag, file, line, filename);
-	return q;
-}
-
-static inline struct call_queue *_queue_unref(struct call_queue *q, const char *tag, const char *file, int line, const char *filename)
-{
-	if (q) {
-		__ao2_ref_debug(q, -1, tag, file, line, filename);
-	}
-	return NULL;
-}
-
-#else
-
-#define queue_t_ref(q, tag)			queue_ref(q)
-#define queue_t_unref(q, tag)		queue_unref(q)
+#define queue_ref(q)				ao2_bump(q)
+#define queue_unref(q)				({ ao2_cleanup(q); NULL; })
+#define queue_t_ref(q, tag)			ao2_t_bump(q, tag)
+#define queue_t_unref(q, tag)		({ ao2_t_cleanup(q, tag); NULL; })
 #define queues_t_link(c, q, tag)	ao2_t_link(c, q, tag)
 #define queues_t_unlink(c, q, tag)	ao2_t_unlink(c, q, tag)
 
-static inline struct call_queue *queue_ref(struct call_queue *q)
-{
-	ao2_ref(q, 1);
-	return q;
-}
-
-static inline struct call_queue *queue_unref(struct call_queue *q)
-{
-	if (q) {
-		ao2_ref(q, -1);
-	}
-	return NULL;
-}
-#endif
-
 /*! \brief Set variables of queue */
 static void set_queue_variables(struct call_queue *q, struct ast_channel *chan)
 {
@@ -2102,7 +2183,12 @@ static void queue_publish_multi_channel_
 		return;
 	}
 
-	ast_multi_channel_blob_add_channel(payload, "caller", caller_snapshot);
+	if (caller_snapshot) {
+		ast_multi_channel_blob_add_channel(payload, "caller", caller_snapshot);
+	} else {
+		ast_debug(1, "Empty caller_snapshot; sending incomplete event\n");
+	}
+
 	if (agent_snapshot) {
 		ast_multi_channel_blob_add_channel(payload, "agent", agent_snapshot);
 	}
@@ -2154,6 +2240,7 @@ static void queue_publish_member_blob(st
 	RAII_VAR(struct stasis_message *, msg, NULL, ao2_cleanup);
 
 	if (!blob || !type) {
+		ast_json_unref(blob);
 		return;
 	}
 
@@ -2173,7 +2260,7 @@ static void queue_publish_member_blob(st
 
 static struct ast_json *queue_member_blob_create(struct call_queue *q, struct member *mem)
 {
-	return ast_json_pack("{s: s, s: s, s: s, s: s, s: s, s: i, s: i, s: i, s: i, s: i, s: i, s: s, s: i}",
+	return ast_json_pack("{s: s, s: s, s: s, s: s, s: s, s: i, s: i, s: i, s: i, s: i, s: i, s: i, s: s, s: i}",
 		"Queue", q->name,
 		"MemberName", mem->membername,
 		"Interface", mem->interface,
@@ -2182,6 +2269,7 @@ static struct ast_json *queue_member_blo
 		"Penalty", mem->penalty,
 		"CallsTaken", mem->calls,
 		"LastCall", (int)mem->lastcall,
+		"LastPause", (int)mem->lastpause,
 		"InCall", mem->starttime ? 1 : 0,
 		"Status", mem->status,
 		"Paused", mem->paused,
@@ -2195,7 +2283,7 @@ static struct ast_json *queue_member_blo
  * is available, the function immediately returns 0. If no members are available,
  * then -1 is returned.
  */
-static int get_member_status(struct call_queue *q, int max_penalty, int min_penalty, enum empty_conditions conditions, int devstate)
+static int get_member_status(struct call_queue *q, int max_penalty, int min_penalty, int raise_penalty, enum empty_conditions conditions, int devstate)
 {
 	struct member *member;
 	struct ao2_iterator mem_iter;
@@ -2203,7 +2291,12 @@ static int get_member_status(struct call
 	ao2_lock(q);
 	mem_iter = ao2_iterator_init(q->members, 0);
 	for (; (member = ao2_iterator_next(&mem_iter)); ao2_ref(member, -1)) {
-		if ((max_penalty != INT_MAX && member->penalty > max_penalty) || (min_penalty != INT_MAX && member->penalty < min_penalty)) {
+		int penalty = member->penalty;
+		if (raise_penalty != INT_MAX && penalty < raise_penalty) {
+			ast_debug(4, "%s is having his penalty raised up from %d to %d\n", member->membername, penalty, raise_penalty);
+			penalty = raise_penalty;
+		}
+		if ((max_penalty != INT_MAX && penalty > max_penalty) || (min_penalty != INT_MAX && penalty < min_penalty)) {
 			if (conditions & QUEUE_EMPTY_PENALTY) {
 				ast_debug(4, "%s is unavailable because his penalty is not between %d and %d\n", member->membername, min_penalty, max_penalty);
 				continue;
@@ -2246,8 +2339,12 @@ static int get_member_status(struct call
 			if (member->paused && (conditions & QUEUE_EMPTY_PAUSED)) {
 				ast_debug(4, "%s is unavailable because he is paused'\n", member->membername);
 				break;
-			} else if ((conditions & QUEUE_EMPTY_WRAPUP) && member->lastcall && q->wrapuptime && (time(NULL) - q->wrapuptime < member->lastcall)) {
-				ast_debug(4, "%s is unavailable because it has only been %d seconds since his last call (wrapup time is %d)\n", member->membername, (int) (time(NULL) - member->lastcall), q->wrapuptime);
+			} else if ((conditions & QUEUE_EMPTY_WRAPUP)
+				&& member->lastcall
+				&& get_wrapuptime(q, member)
+				&& (time(NULL) - get_wrapuptime(q, member) < member->lastcall)) {
+				ast_debug(4, "%s is unavailable because it has only been %d seconds since his last call (wrapup time is %d)\n",
+					member->membername, (int) (time(NULL) - member->lastcall), get_wrapuptime(q, member));
 				break;
 			} else {
 				ao2_ref(member, -1);
@@ -2264,7 +2361,7 @@ static int get_member_status(struct call
 
 	if (!devstate && (conditions & QUEUE_EMPTY_RINGING)) {
 		/* member state still may be RINGING due to lag in event message - check again with device state */
-		return get_member_status(q, max_penalty, min_penalty, conditions, 1);
+		return get_member_status(q, max_penalty, min_penalty, raise_penalty, conditions, 1);
 	}
 	return -1;
 }
@@ -2373,6 +2470,7 @@ static void update_status(struct call_qu
 static int is_member_available(struct call_queue *q, struct member *mem)
 {
 	int available = 0;
+	int wrapuptime;
 
 	switch (mem->status) {
 		case AST_DEVICE_INVALID:
@@ -2396,7 +2494,8 @@ static int is_member_available(struct ca
 	}
 
 	/* Let wrapuptimes override device state availability */
-	if (mem->lastcall && q->wrapuptime && (time(NULL) - q->wrapuptime < mem->lastcall)) {
+	wrapuptime = get_wrapuptime(q, mem);
+	if (mem->lastcall && wrapuptime && (time(NULL) - wrapuptime < mem->lastcall)) {
 		available = 0;
 	}
 	return available;
@@ -2521,7 +2620,7 @@ static int extensionstate2devicestate(in
 	return state;
 }
 
-static int extension_state_cb(char *context, char *exten, struct ast_state_cb_info *info, void *data)
+static int extension_state_cb(const char *context, const char *exten, struct ast_state_cb_info *info, void *data)
 {
 	struct ao2_iterator miter, qiter;
 	struct member *m;
@@ -2588,6 +2687,9 @@ static struct member *create_queue_membe
 		cur->ringinuse = ringinuse;
 		cur->penalty = penalty;
 		cur->paused = paused;
+		if (paused) {
+			time(&cur->lastpause); /* Update time of last pause */
+		}
 		ast_copy_string(cur->interface, interface, sizeof(cur->interface));
 		if (!ast_strlen_zero(state_interface)) {
 			ast_copy_string(cur->state_interface, state_interface, sizeof(cur->state_interface));
@@ -2674,6 +2776,7 @@ static void init_queue(struct call_queue
 	q->announcefrequency = 0;
 	q->minannouncefrequency = DEFAULT_MIN_ANNOUNCE_FREQUENCY;
 	q->announceholdtime = 1;
+	q->announceposition_only_up = 0;
 	q->announcepositionlimit = 10; /* Default 10 positions */
 	q->announceposition = ANNOUNCEPOSITION_YES; /* Default yes */
 	q->roundingseconds = 0; /* Default - don't announce seconds */
@@ -2702,10 +2805,11 @@ static void init_queue(struct call_queue
 	q->autopausedelay = 0;
 	if (!q->members) {
 		if (q->strategy == QUEUE_STRATEGY_LINEAR || q->strategy == QUEUE_STRATEGY_RRORDERED) {
-			/* linear strategy depends on order, so we have to place all members in a single bucket */
-			q->members = ao2_container_alloc(1, member_hash_fn, member_cmp_fn);
+			/* linear strategy depends on order, so we have to place all members in a list */
+			q->members = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, member_cmp_fn);
 		} else {
-			q->members = ao2_container_alloc(37, member_hash_fn, member_cmp_fn);
+			q->members = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+				member_hash_fn, NULL, member_cmp_fn);
 		}
 	}
 	q->found = 1;
@@ -2757,6 +2861,7 @@ static void clear_queue(struct call_queu
 	q->callscompleted = 0;
 	q->callsabandoned = 0;
 	q->callscompletedinsl = 0;
+	q->callsabandonedinsl = 0;
 	q->talktime = 0;
 
 	if (q->members) {
@@ -2784,7 +2889,7 @@ static void clear_queue(struct call_queu
 */
 static int insert_penaltychange(const char *list_name, const char *content, const int linenum)
 {
-	char *timestr, *maxstr, *minstr, *contentdup;
+	char *timestr, *maxstr, *minstr, *raisestr, *contentdup;
 	struct penalty_rule *rule = NULL, *rule_iter;
 	struct rule_list *rl_iter;
 	int penaltychangetime, inserted = 0;
@@ -2802,8 +2907,16 @@ static int insert_penaltychange(const ch
 	}
 
 	*maxstr++ = '\0';
-	timestr = contentdup;
+	if ((minstr = strchr(maxstr,','))) {
+		*minstr++ = '\0';
+		if ((raisestr = strchr(minstr,','))) {
+			*raisestr++ = '\0';
+		}
+	} else {
+		raisestr = NULL;
+	}
 
+	timestr = contentdup;
 	if ((penaltychangetime = atoi(timestr)) < 0) {
 		ast_log(LOG_WARNING, "Improper time parameter specified for penaltychange rule at line %d. Ignoring.\n", linenum);
 		ast_free(rule);
@@ -2812,10 +2925,6 @@ static int insert_penaltychange(const ch
 
 	rule->time = penaltychangetime;
 
-	if ((minstr = strchr(maxstr,','))) {
-		*minstr++ = '\0';
-	}
-
 	/* The last check will evaluate true if either no penalty change is indicated for a given rule
 	 * OR if a min penalty change is indicated but no max penalty change is */
 	if (*maxstr == '+' || *maxstr == '-' || *maxstr == '\0') {
@@ -2833,6 +2942,15 @@ static int insert_penaltychange(const ch
 		rule->min_relative = 1;
 	}
 
+	if (!ast_strlen_zero(raisestr)) {
+		if (*raisestr == '+' || *raisestr == '-') {
+			rule->raise_relative = 1;
+		}
+		rule->raise_value = atoi(raisestr);
+	} else { /*there was no raise specified, so assume this means no change*/
+		rule->raise_relative = 1;
+	}
+
 	/*We have the rule made, now we need to insert it where it belongs*/
 	AST_LIST_TRAVERSE(&rule_lists, rl_iter, list){
 		if (strcasecmp(rl_iter->name, list_name)) {
@@ -2889,9 +3007,10 @@ static int load_realtime_rules(void)
 		return 0;
 	}
 	while ((rulecat = ast_category_browse(cfg, rulecat))) {
-		const char *timestr, *maxstr, *minstr, *rule_name;
+		const char *timestr, *maxstr, *minstr, *raisestr, *rule_name;
 		int penaltychangetime, rule_exists = 0, inserted = 0;
-		int max_penalty = 0, min_penalty = 0, min_relative = 0, max_relative = 0;
+		int max_penalty = 0, min_penalty = 0, raise_penalty = 0;
+		int min_relative = 0, max_relative = 0, raise_relative = 0;
 		struct penalty_rule *new_penalty_rule = NULL;
 
 		rule_name = ast_variable_retrieve(cfg, rulecat, "rule_name");
@@ -2942,11 +3061,22 @@ static int load_realtime_rules(void)
 				min_relative = 1;
 			}
 		}
+		if (!(raisestr = ast_variable_retrieve(cfg, rulecat, "raise_penalty")) ||
+			ast_strlen_zero(raisestr) || sscanf(raisestr, "%30d", &raise_penalty) != 1) {
+			raise_penalty = 0;
+			raise_relative = 1;
+		} else {
+			if (*raisestr == '+' || *raisestr == '-') {
+				raise_relative = 1;
+			}
+		}
 		new_penalty_rule->time = penaltychangetime;
 		new_penalty_rule->max_relative = max_relative;
 		new_penalty_rule->max_value = max_penalty;
 		new_penalty_rule->min_relative = min_relative;
 		new_penalty_rule->min_value = min_penalty;
+		new_penalty_rule->raise_relative = raise_relative;
+		new_penalty_rule->raise_value = raise_penalty;
 		AST_LIST_TRAVERSE_SAFE_BEGIN(&new_rl->rules, pr_iter, list) {
 			if (new_penalty_rule->time < pr_iter->time) {
 				AST_LIST_INSERT_BEFORE_CURRENT(new_penalty_rule, list);
@@ -3098,6 +3228,8 @@ static void queue_set_param(struct call_
 		} else {
 			q->announceposition = ANNOUNCEPOSITION_NO;
 		}
+	} else if (!strcasecmp(param, "announce-position-only-up")) {
+		q->announceposition_only_up = ast_true(val);
 	} else if (!strcasecmp(param, "announce-position-limit")) {
 		q->announcepositionlimit = atoi(val);
 	} else if (!strcasecmp(param, "periodic-announce")) {
@@ -3259,6 +3391,7 @@ static void rt_handle_member_record(stru
 	int penalty = 0;
 	int paused  = 0;
 	int found = 0;
+	int wrapuptime = 0;
 	int ringinuse = q->ringinuse;
 
 	const char *config_val;
@@ -3268,6 +3401,7 @@ static void rt_handle_member_record(stru
 	const char *state_interface = S_OR(ast_variable_retrieve(member_config, category, "state_interface"), interface);
 	const char *penalty_str = ast_variable_retrieve(member_config, category, "penalty");
 	const char *paused_str = ast_variable_retrieve(member_config, category, "paused");
+	const char *wrapuptime_str = ast_variable_retrieve(member_config, category, "wrapuptime");
 
 	if (ast_strlen_zero(rt_uniqueid)) {
 		ast_log(LOG_WARNING, "Realtime field uniqueid is empty for member %s\n", S_OR(membername, "NULL"));
@@ -3290,6 +3424,13 @@ static void rt_handle_member_record(stru
 		}
 	}
 
+	if (wrapuptime_str) {
+		wrapuptime = atoi(wrapuptime_str);
+		if (wrapuptime < 0) {
+			wrapuptime = 0;
+		}
+	}
+
 	if ((config_val = ast_variable_retrieve(member_config, category, realtime_ringinuse_field))) {
 		if (ast_true(config_val)) {
 			ringinuse = 1;
@@ -3316,6 +3457,7 @@ static void rt_handle_member_record(stru
 			}
 			m->penalty = penalty;
 			m->ringinuse = ringinuse;
+			m->wrapuptime = wrapuptime;
 			found = 1;
 			ao2_ref(m, -1);
 			break;
@@ -3368,7 +3510,7 @@ static void destroy_queue(void *obj)
 	ast_string_field_free_memory(q);
 	for (i = 0; i < MAX_PERIODIC_ANNOUNCEMENTS; i++) {
 		if (q->sound_periodicannounce[i]) {
-			free(q->sound_periodicannounce[i]);
+			ast_free(q->sound_periodicannounce[i]);
 		}
 	}
 	ao2_ref(q->members, -1);
@@ -3688,7 +3830,7 @@ static int join_queue(char *queuename, s
 	/* This is our one */
 	if (q->joinempty) {
 		int status = 0;
-		if ((status = get_member_status(q, qe->max_penalty, qe->min_penalty, q->joinempty, 0))) {
+		if ((status = get_member_status(q, qe->max_penalty, qe->min_penalty, qe->raise_penalty, q->joinempty, 0))) {
 			*reason = QUEUE_JOINEMPTY;
 			ao2_unlock(q);
 			queue_t_unref(q, "Done with realtime queue");
@@ -3838,6 +3980,11 @@ static int say_position(struct queue_ent
 		return 0;
 	}
 
+	/* Only announce if the caller's queue position has improved since last time */
+	if (qe->parent->announceposition_only_up && qe->last_pos_said <= qe->pos) {
+		return 0;
+	}
+
 	if (ringing) {
 		ast_indicate(qe->chan,-1);
 	} else {
@@ -4243,6 +4390,8 @@ static int member_status_available(int s
  */
 static int can_ring_entry(struct queue_ent *qe, struct callattempt *call)
 {
+	int wrapuptime;
+
 	if (call->member->paused) {
 		ast_debug(1, "%s paused, can't receive call\n", call->interface);
 		return 0;
@@ -4253,8 +4402,12 @@ static int can_ring_entry(struct queue_e
 		return 0;
 	}
 
-	if ((call->lastqueue && call->lastqueue->wrapuptime && (time(NULL) - call->lastcall < call->lastqueue->wrapuptime))
-		|| (!call->lastqueue && qe->parent->wrapuptime && (time(NULL) - call->lastcall < qe->parent->wrapuptime))) {
+	if (call->lastqueue) {
+		wrapuptime = get_wrapuptime(call->lastqueue, call->member);
+	} else {
+		wrapuptime = get_wrapuptime(qe->parent, call->member);
+	}
+	if (wrapuptime && time(NULL) - call->lastcall < wrapuptime) {
 		ast_debug(1, "Wrapuptime not yet expired on queue %s for %s\n",
 			(call->lastqueue ? call->lastqueue->name : qe->parent->name),
 			call->interface);
@@ -4421,6 +4574,11 @@ static int ring_entry(struct queue_ent *
 	ast_channel_unlock(tmp->chan);
 	ast_channel_unlock(qe->chan);
 
+	/* PREDIAL: Run gosub on the callee's channel */
+	if (qe->predial_callee) {
+		ast_pre_call(tmp->chan, qe->predial_callee);
+	}
+
 	/* Place the call, but don't wait on the answer */
 	if ((res = ast_call(tmp->chan, location, 0))) {
 		/* Again, keep going even if there's an error */
@@ -4477,6 +4635,16 @@ static struct callattempt *find_best(str
 static int ring_one(struct queue_ent *qe, struct callattempt *outgoing, int *busies)
 {
 	int ret = 0;
+	struct callattempt *cur;
+
+	if (qe->predial_callee) {
+		ast_autoservice_start(qe->chan);
+		for (cur = outgoing; cur; cur = cur->q_next) {
+			if (cur->stillgoing && cur->chan) {
+				ast_autoservice_start(cur->chan);
+			}
+		}
+	}
 
 	while (ret == 0) {
 		struct callattempt *best = find_best(outgoing);
@@ -4485,18 +4653,23 @@ static int ring_one(struct queue_ent *qe
 			break;
 		}
 		if (qe->parent->strategy == QUEUE_STRATEGY_RINGALL) {
-			struct callattempt *cur;
 			/* Ring everyone who shares this best metric (for ringall) */
 			for (cur = outgoing; cur; cur = cur->q_next) {
 				if (cur->stillgoing && !cur->chan && cur->metric <= best->metric) {
 					ast_debug(1, "(Parallel) Trying '%s' with metric %d\n", cur->interface, cur->metric);
 					ret |= ring_entry(qe, cur, busies);
+					if (qe->predial_callee && cur->chan) {
+						ast_autoservice_start(cur->chan);
+					}
 				}
 			}
 		} else {
 			/* Ring just the best channel */
 			ast_debug(1, "Trying '%s' with metric %d\n", best->interface, best->metric);
 			ret = ring_entry(qe, best, busies);
+			if (qe->predial_callee && cur->chan) {
+				ast_autoservice_start(best->chan);
+			}
 		}
 
 		/* If we have timed out, break out */
@@ -4506,6 +4679,14 @@ static int ring_one(struct queue_ent *qe
 			break;
 		}
 	}
+	if (qe->predial_callee) {
+		for (cur = outgoing; cur; cur = cur->q_next) {
+			if (cur->stillgoing && cur->chan) {
+				ast_autoservice_stop(cur->chan);
+			}
+		}
+		ast_autoservice_stop(qe->chan);
+	}
 
 	return ret;
 }
@@ -4622,6 +4803,9 @@ static int say_periodic_announcement(str
 /*! \brief Record that a caller gave up on waiting in queue */
 static void record_abandoned(struct queue_ent *qe)
 {
+	int callabandonedinsl = 0;
+	time_t now;
+
 	RAII_VAR(struct ast_json *, blob, NULL, ast_json_unref);
 
 	pbx_builtin_setvar_helper(qe->chan, "ABANDONED", "TRUE");
@@ -4634,6 +4818,13 @@ static void record_abandoned(struct queu
 			     "OriginalPosition", qe->opos,
 			     "HoldTime", (int)(time(NULL) - qe->start));
 
+
+	time(&now);
+	callabandonedinsl = ((now - qe->start) <= qe->parent->servicelevel);
+	if (callabandonedinsl) {
+		qe->parent->callsabandonedinsl++;
+	}
+
 	qe->parent->callsabandoned++;
 	ao2_unlock(qe->parent);
 
@@ -4756,24 +4947,15 @@ static struct callattempt *wait_for_answ
 	char membername[80] = "";
 	long starttime = 0;
 	long endtime = 0;
-#ifdef HAVE_EPOLL
-	struct callattempt *epollo;
-#endif
 	char *inchan_name;
 	struct timeval start_time_tv = ast_tvnow();
+	int canceled_by_caller = 0; /* 1 when caller hangs up or press digit or press * */
 
 	ast_channel_lock(qe->chan);
 	inchan_name = ast_strdupa(ast_channel_name(qe->chan));
 	ast_channel_unlock(qe->chan);
 
 	starttime = (long) time(NULL);
-#ifdef HAVE_EPOLL
-	for (epollo = outgoing; epollo; epollo = epollo->q_next) {
-		if (epollo->chan) {
-			ast_poll_channel_add(in, epollo->chan);
-		}
-	}
-#endif
 
 	while ((*to = ast_remaining_ms(start_time_tv, orig)) && !peer) {
 		int numlines, retry, pos = 1;
@@ -4874,6 +5056,8 @@ static struct callattempt *wait_for_answ
 				/* Before processing channel, go ahead and check for forwarding */
 				if (!ast_strlen_zero(ast_channel_call_forward(o->chan)) && !forwardsallowed) {
 					ast_verb(3, "Forwarding %s to '%s' prevented.\n", inchan_name, ast_channel_call_forward(o->chan));
+					ast_channel_publish_dial_forward(qe->chan, o->chan, NULL, NULL,
+						"CANCEL", ast_channel_call_forward(o->chan));
 					numnochan++;
 					do_hang(o);
 					winner = NULL;
@@ -5094,6 +5278,8 @@ static struct callattempt *wait_for_answ
 						case AST_CONTROL_RINGING:
 							ast_verb(3, "%s is ringing\n", ochan_name);
 
+							ast_channel_publish_dial(qe->chan, o->chan, on, "RINGING");
+
 							/* Start ring indication when the channel is ringing, if specified */
 							if (qe->ring_when_ringing) {
 								ast_moh_stop(qe->chan);
@@ -5195,29 +5381,33 @@ static struct callattempt *wait_for_answ
 			if (!f || ((f->frametype == AST_FRAME_CONTROL) && (f->subclass.integer == AST_CONTROL_HANGUP))) {
 				/* Got hung up */
 				*to = -1;
-				publish_dial_end_event(in, outgoing, NULL, "CANCEL");
 				if (f) {
 					if (f->data.uint32) {
 						ast_channel_hangupcause_set(in, f->data.uint32);
 					}
 					ast_frfree(f);
 				}
-				return NULL;
-			}
-
-			if ((f->frametype == AST_FRAME_DTMF) && caller_disconnect && (f->subclass.integer == '*')) {
+				canceled_by_caller = 1;
+			} else if ((f->frametype == AST_FRAME_DTMF) && caller_disconnect && (f->subclass.integer == '*')) {
 				ast_verb(3, "User hit %c to disconnect call.\n", f->subclass.integer);
 				*to = 0;
-				publish_dial_end_event(in, outgoing, NULL, "CANCEL");
 				ast_frfree(f);
-				return NULL;
-			}
-			if ((f->frametype == AST_FRAME_DTMF) && valid_exit(qe, f->subclass.integer)) {
+				canceled_by_caller = 1;
+			} else if ((f->frametype == AST_FRAME_DTMF) && valid_exit(qe, f->subclass.integer)) {
 				ast_verb(3, "User pressed digit: %c\n", f->subclass.integer);
 				*to = 0;
-				publish_dial_end_event(in, outgoing, NULL, "CANCEL");
 				*digit = f->subclass.integer;
 				ast_frfree(f);
+				canceled_by_caller = 1;
+			}
+			/* When caller hung up or pressed * or digit. */
+			if (canceled_by_caller) {
+				publish_dial_end_event(in, outgoing, NULL, "CANCEL");
+				for (o = start; o; o = o->call_next) {
+					if (o->chan) {
+						ast_queue_log(qe->parent->name, ast_channel_uniqueid(qe->chan), o->member->membername, "RINGCANCELED", "%d", (int) ast_tvdiff_ms(ast_tvnow(), start_time_tv));
+					}
+				}
 				return NULL;
 			}
 
@@ -5231,12 +5421,20 @@ static struct callattempt *wait_for_answ
 				case AST_FRAME_CONTROL:
 					switch (f->subclass.integer) {
 					case AST_CONTROL_CONNECTED_LINE:
+						if (o->block_connected_update) {
+							ast_verb(3, "Connected line update to %s prevented.\n", ast_channel_name(o->chan));
+							break;
+						}
 						if (ast_channel_connected_line_sub(in, o->chan, f, 1) &&
 							ast_channel_connected_line_macro(in, o->chan, f, 0, 1)) {
 							ast_indicate_data(o->chan, f->subclass.integer, f->data.ptr, f->datalen);
 						}
 						break;
 					case AST_CONTROL_REDIRECTING:
+						if (o->block_connected_update) {
+							ast_verb(3, "Redirecting update to %s prevented.\n", ast_channel_name(o->chan));
+							break;
+						}
 						if (ast_channel_redirecting_sub(in, o->chan, f, 1) &&
 							ast_channel_redirecting_macro(in, o->chan, f, 0, 1)) {
 							ast_indicate_data(o->chan, f->subclass.integer, f->data.ptr, f->datalen);
@@ -5268,14 +5466,6 @@ skip_frame:;
 		publish_dial_end_event(qe->chan, outgoing, NULL, "NOANSWER");
 	}
 
-#ifdef HAVE_EPOLL
-	for (epollo = outgoing; epollo; epollo = epollo->q_next) {
-		if (epollo->chan) {
-			ast_poll_channel_del(in, epollo->chan);
-		}
-	}
-#endif
-
 	return peer;
 }
 
@@ -5390,6 +5580,32 @@ static void update_qe_rule(struct queue_
 			qe->min_penalty, ast_channel_name(qe->chan), qe->pr->time);
 	}
 
+	if (qe->raise_penalty != INT_MAX) {
+		char raise_penalty_str[20];
+		int raise_penalty;
+
+		if (qe->pr->raise_relative) {
+			raise_penalty = qe->raise_penalty + qe->pr->raise_value;
+		} else {
+			raise_penalty = qe->pr->raise_value;
+		}
+
+		/* a relative change to the penalty could put it below 0 */
+		if (raise_penalty < 0) {
+			raise_penalty = 0;
+		}
+
+		if (max_penalty != INT_MAX && raise_penalty > max_penalty) {
+			raise_penalty = max_penalty;
+		}
+
+		snprintf(raise_penalty_str, sizeof(raise_penalty_str), "%d", raise_penalty);
+		pbx_builtin_setvar_helper(qe->chan, "QUEUE_RAISE_PENALTY", raise_penalty_str);
+		qe->raise_penalty = raise_penalty;
+		ast_debug(3, "Setting raised penalty to %d for caller %s since %d seconds have elapsed\n",
+			qe->raise_penalty, ast_channel_name(qe->chan), qe->pr->time);
+	}
+
 	qe->pr = AST_LIST_NEXT(qe->pr, list);
 }
 
@@ -5423,7 +5639,7 @@ static int wait_our_turn(struct queue_en
 		if (qe->parent->leavewhenempty) {
 			int status = 0;
 
-			if ((status = get_member_status(qe->parent, qe->max_penalty, qe->min_penalty, qe->parent->leavewhenempty, 0))) {
+			if ((status = get_member_status(qe->parent, qe->max_penalty, qe->min_penalty, qe->raise_penalty, qe->parent->leavewhenempty, 0))) {
 				*reason = QUEUE_LEAVEEMPTY;
 				ast_queue_log(qe->parent->name, ast_channel_uniqueid(qe->chan), "NONE", "EXITEMPTY", "%d|%d|%ld", qe->pos, qe->opos, (long) (time(NULL) - qe->start));
 				res = -1;
@@ -5560,10 +5776,15 @@ static int calc_metric(struct call_queue
 	/* disregarding penalty on too few members? */
 	int membercount = ao2_container_count(q->members);
 	unsigned char usepenalty = (membercount <= q->penaltymemberslimit) ? 0 : 1;
+	int penalty = mem->penalty;
 
 	if (usepenalty) {
-		if ((qe->max_penalty != INT_MAX && mem->penalty > qe->max_penalty) ||
-			(qe->min_penalty != INT_MAX && mem->penalty < qe->min_penalty)) {
+		if (qe->raise_penalty != INT_MAX && penalty < qe->raise_penalty) {
+			/* Low penalty is raised up to the current minimum */
+			penalty = qe->raise_penalty;
+		}
+		if ((qe->max_penalty != INT_MAX && penalty > qe->max_penalty) ||
+			(qe->min_penalty != INT_MAX && penalty < qe->min_penalty)) {
 			return -1;
 		}
 	} else {
@@ -5574,7 +5795,7 @@ static int calc_metric(struct call_queue
 	switch (q->strategy) {
 	case QUEUE_STRATEGY_RINGALL:
 		/* Everyone equal, except for penalty */
-		tmp->metric = mem->penalty * 1000000 * usepenalty;
+		tmp->metric = penalty * 1000000 * usepenalty;
 		break;
 	case QUEUE_STRATEGY_LINEAR:
 		if (pos < qe->linpos) {
@@ -5586,7 +5807,7 @@ static int calc_metric(struct call_queue
 			}
 			tmp->metric = pos;
 		}
-		tmp->metric += mem->penalty * 1000000 * usepenalty;
+		tmp->metric += penalty * 1000000 * usepenalty;
 		break;
 	case QUEUE_STRATEGY_RRORDERED:
 	case QUEUE_STRATEGY_RRMEMORY:
@@ -5600,18 +5821,18 @@ static int calc_metric(struct call_queue
 			}
 			tmp->metric = pos;
 		}
-		tmp->metric += mem->penalty * 1000000 * usepenalty;
+		tmp->metric += penalty * 1000000 * usepenalty;
 		break;
 	case QUEUE_STRATEGY_RANDOM:
 		tmp->metric = ast_random() % 1000;
-		tmp->metric += mem->penalty * 1000000 * usepenalty;
+		tmp->metric += penalty * 1000000 * usepenalty;
 		break;
 	case QUEUE_STRATEGY_WRANDOM:
-		tmp->metric = ast_random() % ((1 + mem->penalty) * 1000);
+		tmp->metric = ast_random() % ((1 + penalty) * 1000);
 		break;
 	case QUEUE_STRATEGY_FEWESTCALLS:
 		tmp->metric = mem->calls;
-		tmp->metric += mem->penalty * 1000000 * usepenalty;
+		tmp->metric += penalty * 1000000 * usepenalty;
 		break;
 	case QUEUE_STRATEGY_LEASTRECENT:
 		if (!mem->lastcall) {
@@ -5619,7 +5840,7 @@ static int calc_metric(struct call_queue
 		} else {
 			tmp->metric = 1000000 - (time(NULL) - mem->lastcall);
 		}
-		tmp->metric += mem->penalty * 1000000 * usepenalty;
+		tmp->metric += penalty * 1000000 * usepenalty;
 		break;
 	default:
 		ast_log(LOG_WARNING, "Can't calculate metric for unknown strategy %d\n", q->strategy);
@@ -5759,10 +5980,6 @@ struct queue_stasis_data {
 	struct local_optimization caller_optimize;
 	/*! Local channel optimization details for the member */
 	struct local_optimization member_optimize;
-	/*! Member channel */
-	struct ast_channel *member_channel;
-	/*! Caller channel */
-	struct ast_channel *caller_channel;
 };
 
 /*!
@@ -5780,9 +5997,6 @@ static void queue_stasis_data_destructor
 	ao2_cleanup(queue_data->member);
 	queue_unref(queue_data->queue);
 	ast_string_field_free_memory(queue_data);
-
-	ao2_ref(queue_data->member_channel, -1);
-	ao2_ref(queue_data->caller_channel, -1);
 }
 
 /*!
@@ -5830,15 +6044,6 @@ static struct queue_stasis_data *queue_s
 	ao2_ref(mem, +1);
 	queue_data->member = mem;
 
-	/*
-	 * During transfers it's possible for both the member and/or caller
-	 * channel(s) to not be available. Adding a reference here ensures
-	 * that the channels remain until app_queue is completely done with
-	 * them.
-	 */
-	queue_data->member_channel = ao2_bump(peer);
-	queue_data->caller_channel = ao2_bump(qe->chan);
-
 	return queue_data;
 }
 
@@ -5850,10 +6055,9 @@ static struct queue_stasis_data *queue_s
  * attended transfer was completed.
  *
  * \param queue_data Data pertaining to the particular call in the queue.
- * \param caller The channel snapshot for the caller channel in the queue.
  * \param atxfer_msg The stasis attended transfer message data.
  */
-static void log_attended_transfer(struct queue_stasis_data *queue_data, struct ast_channel_snapshot *caller,
+static void log_attended_transfer(struct queue_stasis_data *queue_data,
 		struct ast_attended_transfer_message *atxfer_msg)
 {
 	RAII_VAR(struct ast_str *, transfer_str, ast_str_create(32), ast_free);
@@ -5882,7 +6086,7 @@ static void log_attended_transfer(struct
 		return;
 	}
 
-	ast_queue_log(queue_data->queue->name, caller->uniqueid, queue_data->member->membername, "ATTENDEDTRANSFER", "%s|%ld|%ld|%d",
+	ast_queue_log(queue_data->queue->name, queue_data->caller_uniqueid, queue_data->member->membername, "ATTENDEDTRANSFER", "%s|%ld|%ld|%d",
 			ast_str_buffer(transfer_str),
 			(long) (queue_data->starttime - queue_data->holdstart),
 			(long) (time(NULL) - queue_data->starttime), queue_data->caller_pos);
@@ -5970,7 +6174,7 @@ static void handle_blind_transfer(void *
 	context = transfer_msg->context;
 
 	ast_debug(3, "Detected blind transfer in queue %s\n", queue_data->queue->name);
-	ast_queue_log(queue_data->queue->name, caller_snapshot->uniqueid, queue_data->member->membername,
+	ast_queue_log(queue_data->queue->name, queue_data->caller_uniqueid, queue_data->member->membername,
 			"BLINDTRANSFER", "%s|%s|%ld|%ld|%d",
 			exten, context,
 			(long) (queue_data->starttime - queue_data->holdstart),
@@ -6033,8 +6237,7 @@ static void handle_attended_transfer(voi
 	ao2_unlock(queue_data);
 
 	ast_debug(3, "Detected attended transfer in queue %s\n", queue_data->queue->name);
-
-	log_attended_transfer(queue_data, caller_snapshot, atxfer_msg);
+	log_attended_transfer(queue_data, atxfer_msg);
 
 	send_agent_complete(queue_data->queue->name, caller_snapshot, member_snapshot, queue_data->member,
 			queue_data->holdstart, queue_data->starttime, TRANSFER);
@@ -6232,7 +6435,7 @@ static void handle_hangup(void *userdata
 	ast_debug(3, "Detected hangup of queue %s channel %s\n", reason == CALLER ? "caller" : "member",
 			channel_blob->snapshot->name);
 
-	ast_queue_log(queue_data->queue->name, caller_snapshot->uniqueid, queue_data->member->membername,
+	ast_queue_log(queue_data->queue->name, queue_data->caller_uniqueid, queue_data->member->membername,
 			reason == CALLER ? "COMPLETECALLER" : "COMPLETEAGENT", "%ld|%ld|%d",
 		(long) (queue_data->starttime - queue_data->holdstart),
 		(long) (time(NULL) - queue_data->starttime), queue_data->caller_pos);
@@ -6761,6 +6964,7 @@ static int try_calling(struct queue_ent
 
 				ast_channel_publish_dial(qe->chan, peer, member->interface, ast_hangup_cause_to_dial_status(ast_channel_hangupcause(peer)));
 				ast_autoservice_chan_hangup_peer(qe->chan, peer);
+				pending_members_remove(member);
 				ao2_ref(member, -1);
 				goto out;
 			} else if (ast_check_hangup(qe->chan)) {
@@ -6771,6 +6975,7 @@ static int try_calling(struct queue_ent
 				qe->handled = -1;
 				ast_channel_publish_dial(qe->chan, peer, member->interface, ast_hangup_cause_to_dial_status(ast_channel_hangupcause(peer)));
 				ast_autoservice_chan_hangup_peer(qe->chan, peer);
+				pending_members_remove(member);
 				ao2_ref(member, -1);
 				return -1;
 			}
@@ -6790,6 +6995,7 @@ static int try_calling(struct queue_ent
 			record_abandoned(qe);
 			ast_channel_publish_dial(qe->chan, peer, member->interface, ast_hangup_cause_to_dial_status(ast_channel_hangupcause(peer)));
 			ast_autoservice_chan_hangup_peer(qe->chan, peer);
+			pending_members_remove(member);
 			ao2_ref(member, -1);
 			return -1;
 		}
@@ -7054,13 +7260,14 @@ static void dump_queue_members(struct ca
 			continue;
 		}
 
-		ast_str_append(&value, 0, "%s%s;%d;%d;%s;%s",
+		ast_str_append(&value, 0, "%s%s;%d;%d;%s;%s;%s",
 			ast_str_strlen(value) ? "|" : "",
 			cur_member->interface,
 			cur_member->penalty,
 			cur_member->paused,
 			cur_member->membername,
-			cur_member->state_interface);
+			cur_member->state_interface,
+			cur_member->reason_paused);
 
 		ao2_ref(cur_member, -1);
 	}
@@ -7137,7 +7344,7 @@ static int remove_from_queue(const char
  * \retval RES_EXISTS queue exists but no members
  * \retval RES_OUT_OF_MEMORY queue exists but not enough memory to create member
 */
-static int add_to_queue(const char *queuename, const char *interface, const char *membername, int penalty, int paused, int dump, const char *state_interface)
+static int add_to_queue(const char *queuename, const char *interface, const char *membername, int penalty, int paused, int dump, const char *state_interface, const char *reason_paused)
 {
 	struct call_queue *q;
 	struct member *new_member, *old_member;
@@ -7152,8 +7359,10 @@ static int add_to_queue(const char *queu
 	ao2_lock(q);
 	if ((old_member = interface_exists(q, interface)) == NULL) {
 		if ((new_member = create_queue_member(interface, membername, penalty, paused, state_interface, q->ringinuse))) {
-			new_member->ringinuse = q->ringinuse;
 			new_member->dynamic = 1;
+			if (reason_paused) {
+				ast_copy_string(new_member->reason_paused, reason_paused, sizeof(new_member->reason_paused));
+			}
 			member_add_to_queue(q, new_member);
 			queue_publish_member_blob(queue_member_added_type(), queue_member_blob_create(q, new_member));
 
@@ -7182,6 +7391,39 @@ static int add_to_queue(const char *queu
 	return res;
 }
 
+
+/*! \brief Change priority caller into a queue
+ * \retval RES_NOSUCHQUEUE queue does not exist
+ * \retval RES_OKAY change priority
+ * \retval RES_NOT_CALLER queue exists but no caller
+*/
+static int change_priority_caller_on_queue(const char *queuename, const char *caller, int priority)
+{
+	struct call_queue *q;
+	struct queue_ent *qe;
+	int res = RES_NOSUCHQUEUE;
+
+	/*! \note Ensure the appropriate realtime queue is loaded.  Note that this
+	 * short-circuits if the queue is already in memory. */
+	if (!(q = find_load_queue_rt_friendly(queuename))) {
+		return res;
+	}
+
+	ao2_lock(q);
+	res = RES_NOT_CALLER;
+	for (qe = q->head; qe; qe = qe->next) {
+		if (strcmp(ast_channel_name(qe->chan), caller) == 0) {
+			ast_debug(1, "%s Caller new prioriry %d in queue %s\n",
+			             caller, priority, queuename);
+			qe->prio = priority;
+			res = RES_OKAY;
+		}
+	}
+	ao2_unlock(q);
+	return res;
+}
+
+
 static int publish_queue_member_pause(struct call_queue *q, struct member *member, const char *reason)
 {
 	struct ast_json *json_blob = queue_member_blob_create(q, member);
@@ -7227,6 +7469,9 @@ static void set_queue_member_pause(struc
 	}
 
 	mem->paused = paused;
+	if (paused) {
+		time(&mem->lastpause); /* update last pause field */
+	}
 	if (paused && !ast_strlen_zero(reason)) {
 		ast_copy_string(mem->reason_paused, reason, sizeof(mem->reason_paused));
 	} else {
@@ -7279,7 +7524,7 @@ static int set_member_paused(const char
 					 * but since this affects all queues, we cannot.
 					 */
 					ast_queue_log("NONE", "NONE", mem->membername,
-						(paused ? "PAUSEALL" : "UNPAUSEALL"), "%s", "");
+						(paused ? "PAUSEALL" : "UNPAUSEALL"), "%s", S_OR(reason, ""));
 				}
 
 				set_queue_member_pause(q, mem, reason, paused);
@@ -7505,6 +7750,7 @@ static void reload_queue_members(void)
 	int penalty = 0;
 	char *paused_tok;
 	int paused = 0;
+	char *reason_paused;
 	struct ast_db_entry *db_tree;
 	struct ast_db_entry *entry;
 	struct call_queue *cur_queue;
@@ -7551,6 +7797,7 @@ static void reload_queue_members(void)
 			paused_tok = strsep(&member, ";");
 			membername = strsep(&member, ";");
 			state_interface = strsep(&member, ";");
+			reason_paused = strsep(&member, ";");
 
 			if (!penalty_tok) {
 				ast_log(LOG_WARNING, "Error parsing persistent member string for '%s' (penalty)\n", queue_name);
@@ -7572,9 +7819,10 @@ static void reload_queue_members(void)
 				break;
 			}
 
-			ast_debug(1, "Reload Members: Queue: %s  Member: %s  Name: %s  Penalty: %d  Paused: %d\n", queue_name, interface, membername, penalty, paused);
+			ast_debug(1, "Reload Members: Queue: %s  Member: %s  Name: %s  Penalty: %d  Paused: %d ReasonPause: %s\n",
+			              queue_name, interface, membername, penalty, paused, reason_paused);
 
-			if (add_to_queue(queue_name, interface, membername, penalty, paused, 0, state_interface) == RES_OUTOFMEMORY) {
+			if (add_to_queue(queue_name, interface, membername, penalty, paused, 0, state_interface, reason_paused) == RES_OUTOFMEMORY) {
 				ast_log(LOG_ERROR, "Out of Memory when reloading persistent queue member\n");
 				break;
 			}
@@ -7771,7 +8019,7 @@ static int aqm_exec(struct ast_channel *
 		}
 	}
 
-	switch (add_to_queue(args.queuename, args.interface, args.membername, penalty, 0, queue_persistent_members, args.state_interface)) {
+	switch (add_to_queue(args.queuename, args.interface, args.membername, penalty, 0, queue_persistent_members, args.state_interface, NULL)) {
 	case RES_OKAY:
 		if (ast_strlen_zero(args.membername) || !log_membername_as_agent) {
 			ast_queue_log(args.queuename, ast_channel_uniqueid(chan), args.interface, "ADDMEMBER", "%s", "");
@@ -7856,8 +8104,10 @@ static void copy_rules(struct queue_ent
 			new_pr->time = pr_iter->time;
 			new_pr->max_value = pr_iter->max_value;
 			new_pr->min_value = pr_iter->min_value;
+			new_pr->raise_value = pr_iter->raise_value;
 			new_pr->max_relative = pr_iter->max_relative;
 			new_pr->min_relative = pr_iter->min_relative;
+			new_pr->raise_relative = pr_iter->raise_relative;
 			AST_LIST_INSERT_TAIL(&qe->qe_rules, new_pr, list);
 		}
 	}
@@ -7883,9 +8133,10 @@ static int queue_exec(struct ast_channel
 	const char *user_priority;
 	const char *max_penalty_str;
 	const char *min_penalty_str;
+	const char *raise_penalty_str;
 	int prio;
 	int qcontinue = 0;
-	int max_penalty, min_penalty;
+	int max_penalty, min_penalty, raise_penalty;
 	enum queue_result reason = QUEUE_UNKNOWN;
 	/* whether to exit Queue application after the timeout hits */
 	int tries = 0;
@@ -7997,6 +8248,18 @@ static int queue_exec(struct ast_channel
 	} else {
 		min_penalty = INT_MAX;
 	}
+
+	if ((raise_penalty_str = pbx_builtin_getvar_helper(chan, "QUEUE_RAISE_PENALTY"))) {
+		if (sscanf(raise_penalty_str, "%30d", &raise_penalty) == 1) {
+			ast_debug(1, "%s: Got raise penalty %d from ${QUEUE_RAISE_PENALTY}.\n", ast_channel_name(chan), raise_penalty);
+		} else {
+			ast_log(LOG_WARNING, "${QUEUE_RAISE_PENALTY}: Invalid value (%s), channel %s.\n",
+				raise_penalty_str, ast_channel_name(chan));
+			raise_penalty = INT_MAX;
+		}
+	} else {
+		raise_penalty = INT_MAX;
+	}
 	ast_channel_unlock(chan);
 
 	if (ast_test_flag(&opts, OPT_RINGING)) {
@@ -8026,6 +8289,7 @@ static int queue_exec(struct ast_channel
 	qe.prio = prio;
 	qe.max_penalty = max_penalty;
 	qe.min_penalty = min_penalty;
+	qe.raise_penalty = raise_penalty;
 	qe.last_pos_said = 0;
 	qe.last_pos = 0;
 	qe.last_periodic_announce_time = time(NULL);
@@ -8042,6 +8306,21 @@ static int queue_exec(struct ast_channel
 		S_OR(args.url, ""),
 		S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, ""),
 		qe.opos);
+
+	/* PREDIAL: Preprocess any callee gosub arguments. */
+	if (ast_test_flag(&opts, OPT_PREDIAL_CALLEE)
+		&& !ast_strlen_zero(opt_args[OPT_ARG_PREDIAL_CALLEE])) {
+		ast_replace_subargument_delimiter(opt_args[OPT_ARG_PREDIAL_CALLEE]);
+		qe.predial_callee = opt_args[OPT_ARG_PREDIAL_CALLEE];
+	}
+
+	/* PREDIAL: Run gosub on the caller's channel */
+	if (ast_test_flag(&opts, OPT_PREDIAL_CALLER)
+		&& !ast_strlen_zero(opt_args[OPT_ARG_PREDIAL_CALLER])) {
+		ast_replace_subargument_delimiter(opt_args[OPT_ARG_PREDIAL_CALLER]);
+		ast_app_exec_sub(NULL, chan, opt_args[OPT_ARG_PREDIAL_CALLER], 0);
+	}
+
 	copy_rules(&qe, args.rule);
 	qe.pr = AST_LIST_FIRST(&qe.qe_rules);
 check_turns:
@@ -8115,7 +8394,7 @@ check_turns:
 
 		if (qe.parent->leavewhenempty) {
 			int status = 0;
-			if ((status = get_member_status(qe.parent, qe.max_penalty, qe.min_penalty, qe.parent->leavewhenempty, 0))) {
+			if ((status = get_member_status(qe.parent, qe.max_penalty, qe.min_penalty, qe.raise_penalty, qe.parent->leavewhenempty, 0))) {
 				record_abandoned(&qe);
 				reason = QUEUE_LEAVEEMPTY;
 				ast_queue_log(args.queuename, ast_channel_uniqueid(chan), "NONE", "EXITEMPTY", "%d|%d|%ld", qe.pos, qe.opos, (long)(time(NULL) - qe.start));
@@ -8145,14 +8424,15 @@ check_turns:
 			break;
 		}
 
-		/* If using dynamic realtime members, we should regenerate the member list for this queue */
-		update_realtime_members(qe.parent);
 		/* OK, we didn't get anybody; wait for 'retry' seconds; may get a digit to exit with */
 		res = wait_a_bit(&qe);
 		if (res) {
 			goto stop;
 		}
 
+		/* If using dynamic realtime members, we should regenerate the member list for this queue */
+		update_realtime_members(qe.parent);
+
 		/* Since this is a priority queue and
 		 * it is not sure that we are still at the head
 		 * of the queue, go and check for our turn again.
@@ -8364,7 +8644,7 @@ static int queue_function_mem_read(struc
 			while ((m = ao2_iterator_next(&mem_iter))) {
 				/* Count the agents who are logged in, not paused and not wrapping up */
 				if ((m->status == AST_DEVICE_NOT_INUSE) && (!m->paused) &&
-						!(m->lastcall && q->wrapuptime && ((now - q->wrapuptime) < m->lastcall))) {
+						!(m->lastcall && get_wrapuptime(q, m) && ((now - get_wrapuptime(q, m)) < m->lastcall))) {
 					count++;
 				}
 				ao2_ref(m, -1);
@@ -8509,6 +8789,85 @@ static int queue_function_qac_dep(struct
 	return 0;
 }
 
+/*! \brief Dialplan function QUEUE_GET_CHANNEL() Get caller channel waiting at specified position in the queue */
+static int queue_function_queuegetchannel(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
+{
+	int position;
+	char *parse;
+	struct call_queue *q;
+	struct ast_variable *var;
+
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(queuename);
+		AST_APP_ARG(position);
+	);
+
+	buf[0] = '\0';
+
+	if (ast_strlen_zero(data)) {
+		ast_log(LOG_ERROR, "Missing argument. QUEUE_GET_CHANNEL(<queuename>,<position>)\n");
+		return -1;
+	}
+
+	parse = ast_strdupa(data);
+	AST_STANDARD_APP_ARGS(args, parse);
+
+	if (ast_strlen_zero(args.queuename)) {
+		ast_log (LOG_ERROR, "The <queuename> parameter is required.\n");
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.position)) {
+		position = 1;
+	} else {
+		if (sscanf(args.position, "%30d", &position) != 1) {
+			ast_log (LOG_ERROR, "<position> parameter must be an integer.\n");
+			return -1;
+		}
+		if (position < 1) {
+			ast_log (LOG_ERROR, "<position> parameter must be an integer greater than zero.\n");
+			return -1;
+		}
+	}
+
+	{
+		struct call_queue tmpq = {
+			.name = args.queuename,
+		};
+
+		q = ao2_t_find(queues, &tmpq, OBJ_POINTER, "Find for QUEUE_GET_CHANNEL()");
+	}
+	if (q) {
+		ao2_lock(q);
+		if (q->count >= position) {
+			struct queue_ent *qe;
+
+			for (qe = q->head; qe; qe = qe->next) {
+				if (qe->pos == position) {
+					ast_copy_string(buf, ast_channel_name(qe->chan), len);
+					break;
+				}
+			}
+		}
+		ao2_unlock(q);
+		queue_t_unref(q, "Done with reference in QUEUE_GET_CHANNEL()");
+		return 0;
+	}
+
+	var = ast_load_realtime("queues", "name", args.queuename, SENTINEL);
+	if (var) {
+		/* if the queue is realtime but was not found in memory, this
+		 * means that the queue had been deleted from memory since it was
+		 * "dead."
+		 */
+		ast_variables_destroy(var);
+		return 0;
+	}
+
+	ast_log(LOG_WARNING, "queue %s was not found\n", args.queuename);
+	return 0;
+}
+
 /*! \brief Dialplan function QUEUE_WAITING_COUNT() Get number callers waiting in a specific queue */
 static int queue_function_queuewaitingcount(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
 {
@@ -8684,6 +9043,11 @@ static struct ast_custom_function queuem
 	.read = queue_function_qac_dep,
 };
 
+static struct ast_custom_function queuegetchannel_function = {
+	.name = "QUEUE_GET_CHANNEL",
+	.read = queue_function_queuegetchannel,
+};
+
 static struct ast_custom_function queuewaitingcount_function = {
 	.name = "QUEUE_WAITING_COUNT",
 	.read = queue_function_queuewaitingcount,
@@ -8824,12 +9188,14 @@ static void reload_single_member(const c
 	struct member tmpmem;
 	int penalty;
 	int ringinuse;
+	int wrapuptime;
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(interface);
 		AST_APP_ARG(penalty);
 		AST_APP_ARG(membername);
 		AST_APP_ARG(state_interface);
 		AST_APP_ARG(ringinuse);
+		AST_APP_ARG(wrapuptime);
 	);
 
 	if (ast_strlen_zero(memberdata)) {
@@ -8884,11 +9250,23 @@ static void reload_single_member(const c
 		ringinuse = q->ringinuse;
 	}
 
+	if (!ast_strlen_zero(args.wrapuptime)) {
+		tmp = args.wrapuptime;
+		ast_strip(tmp);
+		wrapuptime = atoi(tmp);
+		if (wrapuptime < 0) {
+			wrapuptime = 0;
+		}
+	} else {
+		wrapuptime = 0;
+	}
+
 	/* Find the old position in the list */
 	ast_copy_string(tmpmem.interface, interface, sizeof(tmpmem.interface));
 	cur = ao2_find(q->members, &tmpmem, OBJ_POINTER);
 
 	if ((newm = create_queue_member(interface, membername, penalty, cur ? cur->paused : 0, state_interface, ringinuse))) {
+		newm->wrapuptime = wrapuptime;
 		if (cur) {
 			/* Round Robin Queue Position must be copied if this is replacing an existing member */
 			ao2_lock(q->members);
@@ -8945,6 +9323,7 @@ static void reload_single_queue(struct a
 {
 	int new;
 	struct call_queue *q = NULL;
+	struct member *member;
 	/*We're defining a queue*/
 	struct call_queue tmpq = {
 		.name = queuename,
@@ -8954,6 +9333,8 @@ static void reload_single_queue(struct a
 	const int member_reload = ast_test_flag(mask, QUEUE_RELOAD_MEMBER);
 	int prev_weight = 0;
 	struct ast_variable *var;
+	struct ao2_iterator mem_iter;
+
 	if (!(q = ao2_t_find(queues, &tmpq, OBJ_POINTER, "Find queue for reload"))) {
 		if (queue_reload) {
 			/* Make one then */
@@ -9022,6 +9403,20 @@ static void reload_single_queue(struct a
 		}
 	}
 
+	/* Update ringinuse for dynamic members */
+	if (member_reload) {
+		ao2_lock(q->members);
+		mem_iter = ao2_iterator_init(q->members, AO2_ITERATOR_DONTLOCK);
+		while ((member = ao2_iterator_next(&mem_iter))) {
+			if (member->dynamic) {
+				member->ringinuse = q->ringinuse;
+			}
+			ao2_ref(member, -1);
+		}
+		ao2_iterator_destroy(&mem_iter);
+		ao2_unlock(q->members);
+	}
+
 	/* At this point, we've determined if the queue has a weight, so update use_weight
 	 * as appropriate
 	 */
@@ -9237,6 +9632,8 @@ static char *__queues_show(struct manses
 	queue_iter = ao2_iterator_init(queues, AO2_ITERATOR_DONTLOCK);
 	while ((q = ao2_t_iterator_next(&queue_iter, "Iterate through queues"))) {
 		float sl;
+		float sl2;
+
 		struct call_queue *realtime_queue = NULL;
 
 		ao2_lock(q);
@@ -9269,12 +9666,16 @@ static char *__queues_show(struct manses
 			ast_str_append(&out, 0, "unlimited");
 		}
 		sl = 0;
+		sl2 = 0;
 		if (q->callscompleted > 0) {
 			sl = 100 * ((float) q->callscompletedinsl / (float) q->callscompleted);
 		}
-		ast_str_append(&out, 0, ") in '%s' strategy (%ds holdtime, %ds talktime), W:%d, C:%d, A:%d, SL:%2.1f%% within %ds",
-			int2strat(q->strategy), q->holdtime, q->talktime, q->weight,
-			q->callscompleted, q->callsabandoned,sl,q->servicelevel);
+		if (q->callscompleted + q->callsabandoned > 0) {
+			sl2 =100 * (((float)q->callsabandonedinsl + (float)q->callscompletedinsl) / ((float)q->callsabandoned + (float)q->callscompleted));
+		}
+
+		ast_str_append(&out, 0, ") in '%s' strategy (%ds holdtime, %ds talktime), W:%d, C:%d, A:%d, SL:%2.1f%%, SL2:%2.1f%% within %ds",
+			int2strat(q->strategy), q->holdtime, q->talktime, q->weight, q->callscompleted, q->callsabandoned, sl, sl2, q->servicelevel);
 		do_print(s, fd, ast_str_buffer(out));
 		if (!ao2_container_count(q->members)) {
 			do_print(s, fd, "   No Members");
@@ -9299,11 +9700,21 @@ static char *__queues_show(struct manses
 
 				ast_str_append(&out, 0, " (ringinuse %s)", mem->ringinuse ? "enabled" : "disabled");
 
-				ast_str_append(&out, 0, "%s%s%s%s%s%s%s%s%s%s%s%s (%s%s%s)",
+				ast_str_append(&out, 0, "%s%s%s%s%s%s%s%s%s",
 					mem->dynamic ? ast_term_color(COLOR_CYAN, COLOR_BLACK) : "", mem->dynamic ? " (dynamic)" : "", ast_term_reset(),
 					mem->realtime ? ast_term_color(COLOR_MAGENTA, COLOR_BLACK) : "", mem->realtime ? " (realtime)" : "", ast_term_reset(),
-					mem->paused ? ast_term_color(COLOR_BROWN, COLOR_BLACK) : "", mem->paused ? " (paused)" : "", ast_term_reset(),
-					mem->starttime ? ast_term_color(COLOR_BROWN, COLOR_BLACK) : "", mem->starttime ? " (in call)" : "", ast_term_reset(),
+					mem->starttime ? ast_term_color(COLOR_BROWN, COLOR_BLACK) : "", mem->starttime ? " (in call)" : "", ast_term_reset());
+				if (mem->paused) {
+					if (ast_strlen_zero(mem->reason_paused)) {
+						ast_str_append(&out, 0, " %s(paused was %ld secs ago)%s",
+							ast_term_color(COLOR_BROWN, COLOR_BLACK), (long) (time(NULL) - mem->lastpause), ast_term_reset());
+					} else {
+						ast_str_append(&out, 0, " %s(paused:%s was %ld secs ago)%s", ast_term_color(COLOR_BROWN, COLOR_BLACK),
+							mem->reason_paused,  (long) (time(NULL) - mem->lastcall), ast_term_reset());
+					}
+				}
+
+				ast_str_append(&out, 0, " (%s%s%s)",
 					ast_term_color(
 						mem->status == AST_DEVICE_UNAVAILABLE || mem->status == AST_DEVICE_UNKNOWN ?
 							COLOR_RED : COLOR_GREEN, COLOR_BLACK),
@@ -9483,19 +9894,6 @@ static char *queue_show(struct ast_cli_e
 	return __queues_show(NULL, a->fd, a->argc, a->argv);
 }
 
-/*!\brief callback to display queues status in manager
-   \addtogroup Group_AMI
- */
-static int manager_queues_show(struct mansession *s, const struct message *m)
-{
-	static const char * const a[] = { "queue", "show" };
-
-	__queues_show(s, -1, 2, a);
-	astman_append(s, "\r\n\r\n");	/* Properly terminate Manager output */
-
-	return RESULT_SUCCESS;
-}
-
 static int manager_queue_rule_show(struct mansession *s, const struct message *m)
 {
 	const char *rule = astman_get_header(m, "Rule");
@@ -9622,6 +10020,7 @@ static int manager_queues_status(struct
 	struct call_queue *q;
 	struct queue_ent *qe;
 	float sl = 0;
+	float sl2 = 0;
 	struct member *mem;
 	struct ao2_iterator queue_iter;
 	struct ao2_iterator mem_iter;
@@ -9640,6 +10039,8 @@ static int manager_queues_status(struct
 		/* List queue properties */
 		if (ast_strlen_zero(queuefilter) || !strcasecmp(q->name, queuefilter)) {
 			sl = ((q->callscompleted > 0) ? 100 * ((float)q->callscompletedinsl / (float)q->callscompleted) : 0);
+			sl2 = (((q->callscompleted + q->callsabandoned) > 0) ? 100 * (((float)q->callsabandonedinsl + (float)q->callscompletedinsl) / ((float)q->callsabandoned + (float)q->callscompleted)) : 0);
+
 			astman_append(s, "Event: QueueParams\r\n"
 				"Queue: %s\r\n"
 				"Max: %d\r\n"
@@ -9651,11 +10052,12 @@ static int manager_queues_status(struct
 				"Abandoned: %d\r\n"
 				"ServiceLevel: %d\r\n"
 				"ServicelevelPerf: %2.1f\r\n"
+				"ServicelevelPerf2: %2.1f\r\n"
 				"Weight: %d\r\n"
 				"%s"
 				"\r\n",
 				q->name, q->maxlen, int2strat(q->strategy), q->count, q->holdtime, q->talktime, q->callscompleted,
-				q->callsabandoned, q->servicelevel, sl, q->weight, idText);
+				q->callsabandoned, q->servicelevel, sl, sl2, q->weight, idText);
 			++q_items;
 
 			/* List Queue Members */
@@ -9671,6 +10073,7 @@ static int manager_queues_status(struct
 						"Penalty: %d\r\n"
 						"CallsTaken: %d\r\n"
 						"LastCall: %d\r\n"
+						"LastPause: %d\r\n"
 						"InCall: %d\r\n"
 						"Status: %d\r\n"
 						"Paused: %d\r\n"
@@ -9678,7 +10081,7 @@ static int manager_queues_status(struct
 						"%s"
 						"\r\n",
 						q->name, mem->membername, mem->interface, mem->state_interface, mem->dynamic ? "dynamic" : "static",
-						mem->penalty, mem->calls, (int)mem->lastcall, mem->starttime ? 1 : 0, mem->status,
+						mem->penalty, mem->calls, (int)mem->lastcall, (int)mem->lastpause, mem->starttime ? 1 : 0, mem->status,
 						mem->paused, mem->reason_paused, idText);
 					++q_items;
 				}
@@ -9756,7 +10159,7 @@ static int manager_add_queue_member(stru
 		paused = abs(ast_true(paused_s));
 	}
 
-	switch (add_to_queue(queuename, interface, membername, penalty, paused, queue_persistent_members, state_interface)) {
+	switch (add_to_queue(queuename, interface, membername, penalty, paused, queue_persistent_members, state_interface, NULL)) {
 	case RES_OKAY:
 		if (ast_strlen_zero(membername) || !log_membername_as_agent) {
 			ast_queue_log(queuename, "MANAGER", interface, "ADDMEMBER", "%s", paused ? "PAUSED" : "");
@@ -10010,6 +10413,50 @@ static int manager_queue_member_penalty(
 	return 0;
 }
 
+static int manager_change_priority_caller_on_queue(struct mansession *s, const struct message *m)
+{
+	const char *queuename, *caller, *priority_s;
+	int priority = 0;
+
+	queuename = astman_get_header(m, "Queue");
+	caller = astman_get_header(m, "Caller");
+	priority_s = astman_get_header(m, "Priority");
+
+	if (ast_strlen_zero(queuename)) {
+		astman_send_error(s, m, "'Queue' not specified.");
+		return 0;
+	}
+
+	if (ast_strlen_zero(caller)) {
+		astman_send_error(s, m, "'Caller' not specified.");
+		return 0;
+	}
+
+	if (ast_strlen_zero(priority_s)) {
+		astman_send_error(s, m, "'Priority' not specified.");
+		return 0;
+	} else if (sscanf(priority_s, "%30d", &priority) != 1) {
+		astman_send_error(s, m, "'Priority' need integer.");
+		return 0;
+	}
+
+	switch (change_priority_caller_on_queue(queuename, caller, priority)) {
+	case RES_OKAY:
+		astman_send_ack(s, m, "Priority change for caller on queue");
+		break;
+	case RES_NOSUCHQUEUE:
+		astman_send_error(s, m, "Unable to change priority caller on queue: No such queue");
+		break;
+	case RES_NOT_CALLER:
+		astman_send_error(s, m, "Unable to change priority caller on queue: No such caller");
+		break;
+	}
+
+	return 0;
+}
+
+
+
 static char *handle_queue_add_member(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	const char *queuename, *interface, *membername = NULL, *state_interface = NULL;
@@ -10062,7 +10509,7 @@ static char *handle_queue_add_member(str
 		state_interface = a->argv[11];
 	}
 
-	switch (add_to_queue(queuename, interface, membername, penalty, 0, queue_persistent_members, state_interface)) {
+	switch (add_to_queue(queuename, interface, membername, penalty, 0, queue_persistent_members, state_interface, NULL)) {
 	case RES_OKAY:
 		if (ast_strlen_zero(membername) || !log_membername_as_agent) {
 			ast_queue_log(queuename, "CLI", interface, "ADDMEMBER", "%s", "");
@@ -10197,6 +10644,57 @@ static char *handle_queue_remove_member(
 	return res;
 }
 
+
+
+static char *handle_queue_change_priority_caller(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	const char *queuename, *caller;
+	int priority;
+	char *res = CLI_FAILURE;
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "queue priority caller";
+		e->usage =
+			"Usage: queue priority caller <channel> on <queue> to <priority>\n"
+			"       Change the priority of a channel on a queue.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	if (a->argc != 8) {
+		return CLI_SHOWUSAGE;
+	} else if (strcmp(a->argv[4], "on")) {
+		return CLI_SHOWUSAGE;
+	} else if (strcmp(a->argv[6], "to")) {
+		return CLI_SHOWUSAGE;
+	} else if (sscanf(a->argv[7], "%30d", &priority) != 1) {
+		ast_log (LOG_ERROR, "<priority> parameter must be an integer.\n");
+		return CLI_SHOWUSAGE;
+	}
+
+	caller = a->argv[3];
+	queuename = a->argv[5];
+
+	switch (change_priority_caller_on_queue(queuename, caller, priority)) {
+	case RES_OKAY:
+		res = CLI_SUCCESS;
+		break;
+	case RES_NOSUCHQUEUE:
+		ast_cli(a->fd, "Unable change priority caller %s on queue '%s': No such queue\n", caller, queuename);
+		break;
+	case RES_NOT_CALLER:
+		ast_cli(a->fd, "Unable to change priority caller '%s' on queue '%s': Not there\n", caller, queuename);
+
+		break;
+	}
+
+	return res;
+}
+
+
+
 static char *complete_queue_pause_member(const char *line, const char *word, int pos, int state)
 {
 	/* 0 - queue; 1 - pause; 2 - member; 3 - <interface>; 4 - queue; 5 - <queue>; 6 - reason; 7 - <reason> */
@@ -10441,7 +10939,7 @@ static char *handle_queue_rule_show(stru
 		if (ast_strlen_zero(rule) || !strcasecmp(rl_iter->name, rule)) {
 			ast_cli(a->fd, "Rule: %s\n", rl_iter->name);
 			AST_LIST_TRAVERSE(&rl_iter->rules, pr_iter, list) {
-				ast_cli(a->fd, "\tAfter %d seconds, adjust QUEUE_MAX_PENALTY %s %d and adjust QUEUE_MIN_PENALTY %s %d\n", pr_iter->time, pr_iter->max_relative ? "by" : "to", pr_iter->max_value, pr_iter->min_relative ? "by" : "to", pr_iter->min_value);
+				ast_cli(a->fd, "\tAfter %d seconds, adjust QUEUE_MAX_PENALTY %s %d, adjust QUEUE_MIN_PENALTY %s %d and adjust QUEUE_RAISE_PENALTY %s %d\n", pr_iter->time, pr_iter->max_relative ? "by" : "to", pr_iter->max_value, pr_iter->min_relative ? "by" : "to", pr_iter->min_value, pr_iter->raise_relative ? "by" : "to", pr_iter->raise_value);
 			}
 		}
 	}
@@ -10552,284 +11050,97 @@ static char *handle_queue_reload(struct
 	return CLI_SUCCESS;
 }
 
-static struct ast_cli_entry cli_queue[] = {
-	AST_CLI_DEFINE(queue_show, "Show status of a specified queue"),
-	AST_CLI_DEFINE(handle_queue_rule_show, "Show the rules defined in queuerules.conf"),
-	AST_CLI_DEFINE(handle_queue_add_member, "Add a channel to a specified queue"),
-	AST_CLI_DEFINE(handle_queue_remove_member, "Removes a channel from a specified queue"),
-	AST_CLI_DEFINE(handle_queue_pause_member, "Pause or unpause a queue member"),
-	AST_CLI_DEFINE(handle_queue_set_member_penalty, "Set penalty for a channel of a specified queue"),
-	AST_CLI_DEFINE(handle_queue_set_member_ringinuse, "Set ringinuse for a channel of a specified queue"),
-	AST_CLI_DEFINE(handle_queue_reload, "Reload queues, members, queue rules, or parameters"),
-	AST_CLI_DEFINE(handle_queue_reset, "Reset statistics for a queue"),
-};
-
-/* struct call_queue astdata mapping. */
-#define DATA_EXPORT_CALL_QUEUE(MEMBER)					\
-	MEMBER(call_queue, name, AST_DATA_STRING)			\
-	MEMBER(call_queue, moh, AST_DATA_STRING)			\
-	MEMBER(call_queue, announce, AST_DATA_STRING)			\
-	MEMBER(call_queue, context, AST_DATA_STRING)			\
-	MEMBER(call_queue, membermacro, AST_DATA_STRING)		\
-	MEMBER(call_queue, membergosub, AST_DATA_STRING)		\
-	MEMBER(call_queue, defaultrule, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_next, AST_DATA_STRING)			\
-	MEMBER(call_queue, sound_thereare, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_calls, AST_DATA_STRING)		\
-	MEMBER(call_queue, queue_quantity1, AST_DATA_STRING)		\
-	MEMBER(call_queue, queue_quantity2, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_holdtime, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_minutes, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_minute, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_seconds, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_thanks, AST_DATA_STRING)		\
-	MEMBER(call_queue, sound_callerannounce, AST_DATA_STRING)	\
-	MEMBER(call_queue, sound_reporthold, AST_DATA_STRING)		\
-	MEMBER(call_queue, dead, AST_DATA_BOOLEAN)			\
-	MEMBER(call_queue, ringinuse, AST_DATA_BOOLEAN)			\
-	MEMBER(call_queue, announce_to_first_user, AST_DATA_BOOLEAN)	\
-	MEMBER(call_queue, setinterfacevar, AST_DATA_BOOLEAN)		\
-	MEMBER(call_queue, setqueuevar, AST_DATA_BOOLEAN)		\
-	MEMBER(call_queue, setqueueentryvar, AST_DATA_BOOLEAN)		\
-	MEMBER(call_queue, reportholdtime, AST_DATA_BOOLEAN)		\
-	MEMBER(call_queue, wrapped, AST_DATA_BOOLEAN)			\
-	MEMBER(call_queue, timeoutrestart, AST_DATA_BOOLEAN)		\
-	MEMBER(call_queue, announceholdtime, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, realtime, AST_DATA_BOOLEAN)			\
-	MEMBER(call_queue, found, AST_DATA_BOOLEAN)			\
-	MEMBER(call_queue, announcepositionlimit, AST_DATA_INTEGER)	\
-	MEMBER(call_queue, announcefrequency, AST_DATA_SECONDS)		\
-	MEMBER(call_queue, minannouncefrequency, AST_DATA_SECONDS)	\
-	MEMBER(call_queue, periodicannouncefrequency, AST_DATA_SECONDS)	\
-	MEMBER(call_queue, numperiodicannounce, AST_DATA_INTEGER)	\
-	MEMBER(call_queue, randomperiodicannounce, AST_DATA_INTEGER)	\
-	MEMBER(call_queue, roundingseconds, AST_DATA_SECONDS)		\
-	MEMBER(call_queue, holdtime, AST_DATA_SECONDS)			\
-	MEMBER(call_queue, talktime, AST_DATA_SECONDS)			\
-	MEMBER(call_queue, callscompleted, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, callsabandoned, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, servicelevel, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, callscompletedinsl, AST_DATA_INTEGER)	\
-	MEMBER(call_queue, monfmt, AST_DATA_STRING)			\
-	MEMBER(call_queue, montype, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, count, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, maxlen, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, wrapuptime, AST_DATA_SECONDS)		\
-	MEMBER(call_queue, retry, AST_DATA_SECONDS)			\
-	MEMBER(call_queue, timeout, AST_DATA_SECONDS)			\
-	MEMBER(call_queue, weight, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, autopause, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, timeoutpriority, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, rrpos, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, memberdelay, AST_DATA_INTEGER)		\
-	MEMBER(call_queue, autofill, AST_DATA_INTEGER)			\
-	MEMBER(call_queue, members, AST_DATA_CONTAINER)
-
-AST_DATA_STRUCTURE(call_queue, DATA_EXPORT_CALL_QUEUE);
-
-/* struct member astdata mapping. */
-#define DATA_EXPORT_MEMBER(MEMBER)					\
-	MEMBER(member, interface, AST_DATA_STRING)			\
-	MEMBER(member, state_interface, AST_DATA_STRING)		\
-	MEMBER(member, membername, AST_DATA_STRING)			\
-	MEMBER(member, penalty, AST_DATA_INTEGER)			\
-	MEMBER(member, calls, AST_DATA_INTEGER)				\
-	MEMBER(member, dynamic, AST_DATA_INTEGER)			\
-	MEMBER(member, realtime, AST_DATA_INTEGER)			\
-	MEMBER(member, status, AST_DATA_INTEGER)			\
-	MEMBER(member, paused, AST_DATA_BOOLEAN)			\
-	MEMBER(member, rt_uniqueid, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(member, DATA_EXPORT_MEMBER);
-
-#define DATA_EXPORT_QUEUE_ENT(MEMBER)						\
-	MEMBER(queue_ent, moh, AST_DATA_STRING)					\
-	MEMBER(queue_ent, announce, AST_DATA_STRING)				\
-	MEMBER(queue_ent, context, AST_DATA_STRING)				\
-	MEMBER(queue_ent, digits, AST_DATA_STRING)				\
-	MEMBER(queue_ent, valid_digits, AST_DATA_INTEGER)			\
-	MEMBER(queue_ent, pos, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, prio, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, last_pos_said, AST_DATA_INTEGER)			\
-	MEMBER(queue_ent, last_periodic_announce_time, AST_DATA_INTEGER)	\
-	MEMBER(queue_ent, last_periodic_announce_sound, AST_DATA_INTEGER)	\
-	MEMBER(queue_ent, last_pos, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, opos, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, handled, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, pending, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, max_penalty, AST_DATA_INTEGER)			\
-	MEMBER(queue_ent, min_penalty, AST_DATA_INTEGER)			\
-	MEMBER(queue_ent, linpos, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, linwrapped, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, start, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, expire, AST_DATA_INTEGER)				\
-	MEMBER(queue_ent, cancel_answered_elsewhere, AST_DATA_INTEGER)
-
-AST_DATA_STRUCTURE(queue_ent, DATA_EXPORT_QUEUE_ENT);
-
 /*!
- * \internal
- * \brief Add a queue to the data_root node.
- * \param[in] search The search tree.
- * \param[in] data_root The main result node.
- * \param[in] queue The queue to add.
- */
-static void queues_data_provider_get_helper(const struct ast_data_search *search,
-	struct ast_data *data_root, struct call_queue *queue)
+ * \brief Update Queue with data of an outgoing call
+*/
+static int qupd_exec(struct ast_channel *chan, const char *data)
 {
-	struct ao2_iterator im;
-	struct member *member;
-	struct queue_ent *qe;
-	struct ast_data *data_queue, *data_members = NULL, *enum_node;
-	struct ast_data *data_member, *data_callers = NULL, *data_caller, *data_caller_channel;
+	int oldtalktime;
+	char *parse;
+	struct call_queue *q;
+	struct member *mem;
+	int newtalktime = 0;
 
-	data_queue = ast_data_add_node(data_root, "queue");
-	if (!data_queue) {
-		return;
+	AST_DECLARE_APP_ARGS(args,
+			AST_APP_ARG(queuename);
+			AST_APP_ARG(uniqueid);
+			AST_APP_ARG(agent);
+			AST_APP_ARG(status);
+			AST_APP_ARG(talktime);
+			AST_APP_ARG(params););
+
+	if (ast_strlen_zero(data)) {
+		ast_log(LOG_WARNING, "QueueUpdate requires arguments (queuename,uniqueid,agent,status,talktime,params[totaltime,callednumber])\n");
+		return -1;
 	}
 
-	ast_data_add_structure(call_queue, data_queue, queue);
+	parse = ast_strdupa(data);
 
-	ast_data_add_str(data_queue, "strategy", int2strat(queue->strategy));
-	ast_data_add_int(data_queue, "membercount", ao2_container_count(queue->members));
+	AST_STANDARD_APP_ARGS(args, parse);
 
-	/* announce position */
-	enum_node = ast_data_add_node(data_queue, "announceposition");
-	if (!enum_node) {
-		return;
-	}
-	switch (queue->announceposition) {
-	case ANNOUNCEPOSITION_LIMIT:
-		ast_data_add_str(enum_node, "text", "limit");
-		break;
-	case ANNOUNCEPOSITION_MORE_THAN:
-		ast_data_add_str(enum_node, "text", "more");
-		break;
-	case ANNOUNCEPOSITION_YES:
-		ast_data_add_str(enum_node, "text", "yes");
-		break;
-	case ANNOUNCEPOSITION_NO:
-		ast_data_add_str(enum_node, "text", "no");
-		break;
-	default:
-		ast_data_add_str(enum_node, "text", "unknown");
-		break;
+	if (ast_strlen_zero(args.queuename) || ast_strlen_zero(args.uniqueid) || ast_strlen_zero(args.agent) || ast_strlen_zero(args.status)) {
+		ast_log(LOG_WARNING, "Missing argument to QueueUpdate (queuename,uniqueid,agent,status,talktime,params[totaltime|callednumber])\n");
+		return -1;
 	}
-	ast_data_add_int(enum_node, "value", queue->announceposition);
-
-	/* add queue members */
-	im = ao2_iterator_init(queue->members, 0);
-	while ((member = ao2_iterator_next(&im))) {
-		if (!data_members) {
-			data_members = ast_data_add_node(data_queue, "members");
-			if (!data_members) {
-				ao2_ref(member, -1);
-				continue;
-			}
-		}
 
-		data_member = ast_data_add_node(data_members, "member");
-		if (!data_member) {
-			ao2_ref(member, -1);
-			continue;
-		}
-
-		ast_data_add_structure(member, data_member, member);
+	if (!ast_strlen_zero(args.talktime)) {
+		newtalktime = atoi(args.talktime);
+	}
 
-		ao2_ref(member, -1);
+	q = find_load_queue_rt_friendly(args.queuename);
+	if (!q) {
+		ast_log(LOG_WARNING, "QueueUpdate could not find requested queue '%s'\n", args.queuename);
+		return 0;
 	}
-	ao2_iterator_destroy(&im);
 
-	/* include the callers inside the result. */
-	if (queue->head) {
-		for (qe = queue->head; qe; qe = qe->next) {
-			if (!data_callers) {
-				data_callers = ast_data_add_node(data_queue, "callers");
-				if (!data_callers) {
-					continue;
-				}
-			}
+	ao2_lock(q);
+	if (q->members) {
+		struct ao2_iterator mem_iter = ao2_iterator_init(q->members, 0);
+		while ((mem = ao2_iterator_next(&mem_iter))) {
+			if (!strcasecmp(mem->membername, args.agent)) {
+				if (!strcasecmp(args.status, "ANSWER")) {
+					oldtalktime = q->talktime;
+					q->talktime = (((oldtalktime << 2) - oldtalktime) + newtalktime) >> 2;
+					time(&mem->lastcall);
+					mem->calls++;
+					mem->lastqueue = q;
+					q->callscompleted++;
 
-			data_caller = ast_data_add_node(data_callers, "caller");
-			if (!data_caller) {
-				continue;
-			}
+					if (newtalktime <= q->servicelevel) {
+						q->callscompletedinsl++;
+					}
+				} else {
 
-			ast_data_add_structure(queue_ent, data_caller, qe);
+					time(&mem->lastcall);
+					q->callsabandoned++;
+				}
 
-			/* add the caller channel. */
-			data_caller_channel = ast_data_add_node(data_caller, "channel");
-			if (!data_caller_channel) {
-				continue;
+				ast_queue_log(args.queuename, args.uniqueid, args.agent, "OUTCALL", "%s|%s|%s", args.status, args.talktime, args.params);
 			}
 
-			ast_channel_data_add_structure(data_caller_channel, qe->chan, 1);
+			ao2_ref(mem, -1);
 		}
-	}
 
-	/* if this queue doesn't match remove the added queue. */
-	if (!ast_data_search_match(search, data_queue)) {
-		ast_data_remove_node(data_root, data_queue);
+		ao2_iterator_destroy(&mem_iter);
 	}
-}
 
-/*!
- * \internal
- * \brief Callback used to generate the queues tree.
- * \param[in] search The search pattern tree.
- * \retval NULL on error.
- * \retval non-NULL The generated tree.
- */
-static int queues_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ao2_iterator i;
-	struct call_queue *queue, *queue_realtime = NULL;
-	struct ast_config *cfg;
-
-	/* load realtime queues. */
-	cfg = ast_load_realtime_multientry("queues", "name LIKE", "%", SENTINEL);
-	if (cfg) {
-		char *category = NULL;
-		while ((category = ast_category_browse(cfg, category))) {
-			const char *queuename = ast_variable_retrieve(cfg, category, "name");
-			if ((queue = find_load_queue_rt_friendly(queuename))) {
-				queue_unref(queue);
-			}
-		}
-		ast_config_destroy(cfg);
-	}
-
-	/* static queues. */
-	i = ao2_iterator_init(queues, 0);
-	while ((queue = ao2_iterator_next(&i))) {
-		ao2_lock(queue);
-		if (queue->realtime) {
-			queue_realtime = find_load_queue_rt_friendly(queue->name);
-			if (!queue_realtime) {
-				ao2_unlock(queue);
-				queue_unref(queue);
-				continue;
-			}
-			queue_unref(queue_realtime);
-		}
-
-		queues_data_provider_get_helper(search, data_root, queue);
-		ao2_unlock(queue);
-		queue_unref(queue);
-	}
-	ao2_iterator_destroy(&i);
+	ao2_unlock(q);
+	queue_t_unref(q, "Done with temporary pointer");
 
 	return 0;
 }
 
-static const struct ast_data_handler queues_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = queues_data_provider_get
-};
-
-static const struct ast_data_entry queue_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/application/queue/list", &queues_data_provider),
+static struct ast_cli_entry cli_queue[] = {
+	AST_CLI_DEFINE(queue_show, "Show status of a specified queue"),
+	AST_CLI_DEFINE(handle_queue_rule_show, "Show the rules defined in queuerules.conf"),
+	AST_CLI_DEFINE(handle_queue_add_member, "Add a channel to a specified queue"),
+	AST_CLI_DEFINE(handle_queue_remove_member, "Removes a channel from a specified queue"),
+	AST_CLI_DEFINE(handle_queue_pause_member, "Pause or unpause a queue member"),
+	AST_CLI_DEFINE(handle_queue_set_member_penalty, "Set penalty for a channel of a specified queue"),
+	AST_CLI_DEFINE(handle_queue_set_member_ringinuse, "Set ringinuse for a channel of a specified queue"),
+	AST_CLI_DEFINE(handle_queue_reload, "Reload queues, members, queue rules, or parameters"),
+	AST_CLI_DEFINE(handle_queue_reset, "Reset statistics for a queue"),
+	AST_CLI_DEFINE(handle_queue_change_priority_caller, "Change priority caller on queue"),
 };
 
 static struct stasis_message_router *agent_router;
@@ -10861,33 +11172,34 @@ static int unload_module(void)
 
 	ast_cli_unregister_multiple(cli_queue, ARRAY_LEN(cli_queue));
 	ast_manager_unregister("QueueStatus");
-	ast_manager_unregister("Queues");
 	ast_manager_unregister("QueueRule");
 	ast_manager_unregister("QueueSummary");
 	ast_manager_unregister("QueueAdd");
 	ast_manager_unregister("QueueRemove");
 	ast_manager_unregister("QueuePause");
 	ast_manager_unregister("QueueLog");
+	ast_manager_unregister("QueueUpdate");
 	ast_manager_unregister("QueuePenalty");
 	ast_manager_unregister("QueueReload");
 	ast_manager_unregister("QueueReset");
 	ast_manager_unregister("QueueMemberRingInUse");
+	ast_manager_unregister("QueueChangePriorityCaller");
 	ast_unregister_application(app_aqm);
 	ast_unregister_application(app_rqm);
 	ast_unregister_application(app_pqm);
 	ast_unregister_application(app_upqm);
 	ast_unregister_application(app_ql);
+	ast_unregister_application(app_qupd);
 	ast_unregister_application(app);
 	ast_custom_function_unregister(&queueexists_function);
 	ast_custom_function_unregister(&queuevar_function);
 	ast_custom_function_unregister(&queuemembercount_function);
 	ast_custom_function_unregister(&queuemembercount_dep);
 	ast_custom_function_unregister(&queuememberlist_function);
+	ast_custom_function_unregister(&queuegetchannel_function);
 	ast_custom_function_unregister(&queuewaitingcount_function);
 	ast_custom_function_unregister(&queuememberpenalty_function);
 
-	ast_data_unregister(NULL);
-
 	device_state_sub = stasis_unsubscribe_and_join(device_state_sub);
 
 	ast_unload_realtime("queue_members");
@@ -10916,13 +11228,14 @@ static int load_module(void)
 	struct stasis_topic *queue_topic;
 	struct stasis_topic *manager_topic;
 
-	queues = ao2_container_alloc(MAX_QUEUE_BUCKETS, queue_hash_cb, queue_cmp_cb);
+	queues = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_QUEUE_BUCKETS,
+		queue_hash_cb, NULL, queue_cmp_cb);
 	if (!queues) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	pending_members = ao2_container_alloc(
-		MAX_CALL_ATTEMPT_BUCKETS, pending_members_hash, pending_members_cmp);
+	pending_members = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_CALL_ATTEMPT_BUCKETS, pending_members_hash, NULL, pending_members_cmp);
 	if (!pending_members) {
 		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
@@ -10964,8 +11277,6 @@ static int load_module(void)
 		reload_queue_members();
 	}
 
-	ast_data_register_multiple(queue_data_providers, ARRAY_LEN(queue_data_providers));
-
 	err |= ast_cli_register_multiple(cli_queue, ARRAY_LEN(cli_queue));
 	err |= ast_register_application_xml(app, queue_exec);
 	err |= ast_register_application_xml(app_aqm, aqm_exec);
@@ -10973,7 +11284,7 @@ static int load_module(void)
 	err |= ast_register_application_xml(app_pqm, pqm_exec);
 	err |= ast_register_application_xml(app_upqm, upqm_exec);
 	err |= ast_register_application_xml(app_ql, ql_exec);
-	err |= ast_manager_register_xml("Queues", 0, manager_queues_show);
+	err |= ast_register_application_xml(app_qupd, qupd_exec);
 	err |= ast_manager_register_xml("QueueStatus", 0, manager_queues_status);
 	err |= ast_manager_register_xml("QueueSummary", 0, manager_queues_summary);
 	err |= ast_manager_register_xml("QueueAdd", EVENT_FLAG_AGENT, manager_add_queue_member);
@@ -10985,11 +11296,13 @@ static int load_module(void)
 	err |= ast_manager_register_xml("QueueRule", 0, manager_queue_rule_show);
 	err |= ast_manager_register_xml("QueueReload", 0, manager_queue_reload);
 	err |= ast_manager_register_xml("QueueReset", 0, manager_queue_reset);
+	err |= ast_manager_register_xml("QueueChangePriorityCaller", 0,  manager_change_priority_caller_on_queue);
 	err |= ast_custom_function_register(&queuevar_function);
 	err |= ast_custom_function_register(&queueexists_function);
 	err |= ast_custom_function_register(&queuemembercount_function);
 	err |= ast_custom_function_register(&queuemembercount_dep);
 	err |= ast_custom_function_register(&queuememberlist_function);
+	err |= ast_custom_function_register(&queuegetchannel_function);
 	err |= ast_custom_function_register(&queuewaitingcount_function);
 	err |= ast_custom_function_register(&queuememberpenalty_function);
 
@@ -10998,6 +11311,8 @@ static int load_module(void)
 	if (!device_state_sub) {
 		err = -1;
 	}
+	stasis_subscription_accept_message_type(device_state_sub, ast_device_state_message_type());
+	stasis_subscription_set_filter(device_state_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	manager_topic = ast_manager_get_topic();
 	queue_topic = ast_queue_topic_all();
@@ -11080,10 +11395,10 @@ static struct member *find_member_by_que
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "True Call Queueing",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
-		.nonoptreq = "res_monitor",
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
+	.optional_modules = "res_monitor",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_read.c 1:16.1.1~dfsg-1/apps/app_read.c
--- 1:13.23.1~dfsg-2/apps/app_read.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_read.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_readexten.c 1:16.1.1~dfsg-1/apps/app_readexten.c
--- 1:13.23.1~dfsg-2/apps/app_readexten.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_readexten.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_record.c 1:16.1.1~dfsg-1/apps/app_record.c
--- 1:13.23.1~dfsg-2/apps/app_record.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_record.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_saycounted.c 1:16.1.1~dfsg-1/apps/app_saycounted.c
--- 1:13.23.1~dfsg-2/apps/app_saycounted.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_saycounted.c	2018-12-26 18:45:13.000000000 +0000
@@ -114,8 +114,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_sayunixtime.c 1:16.1.1~dfsg-1/apps/app_sayunixtime.c
--- 1:13.23.1~dfsg-2/apps/app_sayunixtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_sayunixtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_senddtmf.c 1:16.1.1~dfsg-1/apps/app_senddtmf.c
--- 1:13.23.1~dfsg-2/apps/app_senddtmf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_senddtmf.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_sendtext.c 1:16.1.1~dfsg-1/apps/app_sendtext.c
--- 1:13.23.1~dfsg-2/apps/app_sendtext.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_sendtext.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_setcallerid.c 1:16.1.1~dfsg-1/apps/app_setcallerid.c
--- 1:13.23.1~dfsg-2/apps/app_setcallerid.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_setcallerid.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,130 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Mark Spencer <markster@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- *
- * \brief App to set callerid presentation
- *
- * \author Mark Spencer <markster@digium.com>
- *
- * \ingroup applications
- */
-
-/*** MODULEINFO
-	<defaultenabled>no</defaultenabled>
-	<support_level>deprecated</support_level>
-	<replacement>func_callerid</replacement>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include "asterisk/lock.h"
-#include "asterisk/file.h"
-#include "asterisk/channel.h"
-#include "asterisk/pbx.h"
-#include "asterisk/module.h"
-#include "asterisk/translate.h"
-#include "asterisk/image.h"
-#include "asterisk/callerid.h"
-
-/*** DOCUMENTATION
-	<application name="SetCallerPres" language="en_US">
-		<synopsis>
-			Set CallerID Presentation.
-		</synopsis>
-		<syntax>
-			<parameter name="presentation" required="true">
-				<enumlist>
-					<enum name="allowed_not_screened">
-						<para>Presentation Allowed, Not Screened.</para>
-					</enum>
-					<enum name="allowed_passed_screen">
-						<para>Presentation Allowed, Passed Screen.</para>
-					</enum>
-					<enum name="allowed_failed_screen">
-						<para>Presentation Allowed, Failed Screen.</para>
-					</enum>
-					<enum name="allowed">
-						<para>Presentation Allowed, Network Number.</para>
-					</enum>
-					<enum name="prohib_not_screened">
-						<para>Presentation Prohibited, Not Screened.</para>
-					</enum>
-					<enum name="prohib_passed_screen">
-						<para>Presentation Prohibited, Passed Screen.</para>
-					</enum>
-					<enum name="prohib_failed_screen">
-						<para>Presentation Prohibited, Failed Screen.</para>
-					</enum>
-					<enum name="prohib">
-						<para>Presentation Prohibited, Network Number.</para>
-					</enum>
-					<enum name="unavailable">
-						<para>Number Unavailable.</para>
-					</enum>
-				</enumlist>
-			</parameter>
-		</syntax>
-		<description>
-			<para>Set Caller*ID presentation on a call.</para>
-		</description>
-	</application>
- ***/
-
-static char *app2 = "SetCallerPres";
-
-static int setcallerid_pres_exec(struct ast_channel *chan, const char *data)
-{
-	int pres = -1;
-	static int deprecated = 0;
-
-	if (!deprecated) {
-		deprecated = 1;
-		ast_log(LOG_WARNING, "SetCallerPres is deprecated.  Please use Set(CALLERPRES()=%s) instead.\n", (char *)data);
-	}
-
-	/* For interface consistency, permit the argument to be specified as a number */
-	if (sscanf(data, "%30d", &pres) != 1 || pres < 0 || pres > 255 || (pres & 0x9c)) {
-		pres = ast_parse_caller_presentation(data);
-	}
-
-	if (pres < 0) {
-		ast_log(LOG_WARNING, "'%s' is not a valid presentation (see 'show application SetCallerPres')\n",
-			(char *) data);
-		return 0;
-	}
-
-	/* Set the combined caller id presentation. */
-	ast_channel_caller(chan)->id.name.presentation = pres;
-	ast_channel_caller(chan)->id.number.presentation = pres;
-	return 0;
-}
-
-static int unload_module(void)
-{
-	return ast_unregister_application(app2);
-}
-
-static int load_module(void)
-{
-	return ast_register_application_xml(app2, setcallerid_pres_exec);
-}
-
-AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Set CallerID Presentation Application");
diff -pruN 1:13.23.1~dfsg-2/apps/app_skel.c 1:16.1.1~dfsg-1/apps/app_skel.c
--- 1:13.23.1~dfsg-2/apps/app_skel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_skel.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h> /* log10 */
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -582,7 +580,9 @@ static void *skel_config_alloc(void)
 		goto error;
 	}
 
-	if (!(cfg->levels = ao2_container_alloc(LEVEL_BUCKETS, skel_level_hash, skel_level_cmp))) {
+	cfg->levels = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, LEVEL_BUCKETS,
+		skel_level_hash, NULL, skel_level_cmp);
+	if (!cfg->levels) {
 		goto error;
 	}
 
@@ -727,7 +727,9 @@ static int load_module(void)
 	if (aco_info_init(&cfg_info)) {
 		goto error;
 	}
-	if (!(games = ao2_container_alloc(1, NULL, NULL))) {
+
+	games = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
+	if (!games) {
 		goto error;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/apps/app_sms.c 1:16.1.1~dfsg-1/apps/app_sms.c
--- 1:13.23.1~dfsg-2/apps/app_sms.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_sms.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dirent.h>
 #include <ctype.h>
 #include <sys/stat.h>
@@ -88,6 +86,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 					<option name="o">
 						<para>The body should be coded as octets not 7-bit symbols.</para>
 					</option>
+					<option name="n">
+						<para>Do not log any SMS content to log file (privacy).</para>
+					</option>
 				</optionlist>
 			</parameter>
 			<parameter name="addr" />
@@ -220,6 +221,7 @@ typedef struct sms_s {
 	unsigned char sent_rel:1;     /*!< have sent REL message... */
 	unsigned char smsc:1;        /*!< we are SMSC */
 	unsigned char rx:1;          /*!< this is a received message */
+	unsigned char nolog:1;       /*!< do not log plain text SMS content (privacy) */
 	char queue[30];              /*!< queue name */
 	char oa[20];                 /*!< originating address */
 	char da[20];                 /*!< destination address */
@@ -790,20 +792,25 @@ static void sms_log(sms_t * h, char stat
 			status, h->rx ? 'I' : 'O', h->smsc ? 'S' : 'M', mrs, h->queue,
 			S_OR(h->oa, "-"), S_OR(h->da, "-") );
 		p = line + strlen(line);
-		for (n = 0; n < h->udl; n++) {
-			if (h->ud[n] == '\\') {
-				*p++ = '\\';
-				*p++ = '\\';
-			} else if (h->ud[n] == '\n') {
-				*p++ = '\\';
-				*p++ = 'n';
-			} else if (h->ud[n] == '\r') {
-				*p++ = '\\';
-				*p++ = 'r';
-			} else if (h->ud[n] < 32 || h->ud[n] == 127) {
-				*p++ = 0xbf;
-			} else {
-				*p++ = h->ud[n];
+
+		if (h->nolog) {
+			p += snprintf(p, 1000 - strlen(line), "udl=%d", h->udl);
+		} else {
+			for (n = 0; n < h->udl; n++) {
+				if (h->ud[n] == '\\') {
+					*p++ = '\\';
+					*p++ = '\\';
+				} else if (h->ud[n] == '\n') {
+					*p++ = '\\';
+					*p++ = 'n';
+				} else if (h->ud[n] == '\r') {
+					*p++ = '\\';
+					*p++ = 'r';
+				} else if (h->ud[n] < 32 || h->ud[n] == 127) {
+					*p++ = 0xbf;
+				} else {
+					*p++ = h->ud[n];
+				}
 			}
 		}
 		*p++ = '\n';
@@ -1853,6 +1860,7 @@ enum sms_flags {
 	OPTION_PAUSE	= (1 << 3),             /* pause before sending data, in ms */
 	OPTION_SRR	= (1 << 4),                 /* set srr */
 	OPTION_DCS	= (1 << 5),                 /* set dcs */
+	OPTIONS_NO_LOG	= (1 << 6),             /* Don't log SMS content */
 };
 
 enum sms_opt_args {
@@ -1866,6 +1874,7 @@ AST_APP_OPTIONS(sms_options, {
 	AST_APP_OPTION('t', OPTION_TWO),
 	AST_APP_OPTION('r', OPTION_SRR),
 	AST_APP_OPTION('o', OPTION_DCS),
+	AST_APP_OPTION('n', OPTIONS_NO_LOG),
 	AST_APP_OPTION_ARG('p', OPTION_PAUSE, OPTION_ARG_PAUSE),
 	} );
 
@@ -1926,6 +1935,7 @@ static int sms_exec(struct ast_channel *
 
 	h.smsc = ast_test_flag(&flags, OPTION_BE_SMSC);
 	h.protocol = ast_test_flag(&flags, OPTION_TWO) ? 2 : 1;
+	h.nolog = ast_test_flag(&flags, OPTIONS_NO_LOG) ? 1 : 0;
 	if (!ast_strlen_zero(sms_opts[OPTION_ARG_PAUSE])) {
 		h.opause_0 = atoi(sms_opts[OPTION_ARG_PAUSE]);
 	}
diff -pruN 1:13.23.1~dfsg-2/apps/app_softhangup.c 1:16.1.1~dfsg-1/apps/app_softhangup.c
--- 1:13.23.1~dfsg-2/apps/app_softhangup.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_softhangup.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_speech_utils.c 1:16.1.1~dfsg-1/apps/app_speech_utils.c
--- 1:13.23.1~dfsg-2/apps/app_speech_utils.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_speech_utils.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -1002,8 +1000,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Dialplan Speech Applications",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.nonoptreq = "res_speech",
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_speech",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_stack.c 1:16.1.1~dfsg-1/apps/app_stack.c
--- 1:13.23.1~dfsg-2/apps/app_stack.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_stack.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
@@ -1280,7 +1278,7 @@ static int unload_module(void)
 {
 	ast_install_stack_functions(NULL);
 
-	ast_agi_unregister(ast_module_info->self, &gosub_agi_command);
+	ast_agi_unregister(&gosub_agi_command);
 
 	ast_unregister_application(app_return);
 	ast_unregister_application(app_pop);
@@ -1318,9 +1316,9 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT | AST_MODFLAG_LOAD_ORDER, "Dialplan subroutines (Gosub, Return, etc)",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-		.nonoptreq = "res_agi",
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.optional_modules = "res_agi",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_stasis.c 1:16.1.1~dfsg-1/apps/app_stasis.c
--- 1:13.23.1~dfsg-2/apps/app_stasis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_stasis.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
@@ -127,24 +125,17 @@ static int app_exec(struct ast_channel *
 
 static int load_module(void)
 {
-	int r = 0;
-
-	stasis_app_ref();
-	r |= ast_register_application_xml(stasis, app_exec);
-	return r;
+	return ast_register_application_xml(stasis, app_exec);
 }
 
 static int unload_module(void)
 {
-	int r = 0;
-	r |= ast_unregister_application(stasis);
-	stasis_app_unref();
-	return r;
+	return ast_unregister_application(stasis);
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Stasis dialplan application",
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis",
-	);
+	.requires = "res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_statsd.c 1:16.1.1~dfsg-1/apps/app_statsd.c
--- 1:13.23.1~dfsg-2/apps/app_statsd.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_statsd.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,428 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Tyler Cambron <tcambron@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+ /*** MODULEINFO
+	<depend>res_statsd</depend>
+	<defaultenabled>no</defaultenabled>
+	<support_level>extended</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <math.h>
+
+#include "asterisk/module.h"
+#include "asterisk/logger.h"
+#include "asterisk/app.h"
+#include "asterisk/pbx.h"
+#include "asterisk/strings.h"
+#include "asterisk/statsd.h"
+
+/*** DOCUMENTATION
+	<application name="StatsD" language="en_US">
+		<synopsis>
+			Allow statistics to be passed to the StatsD server from the dialplan.
+		</synopsis>
+		<syntax>
+			<parameter name="metric_type" required="true">
+				<para>The metric type to be sent to StatsD. Valid metric types
+				are 'g' for gauge, 'c' for counter, 'ms' for timer, and 's' for
+				sets.</para>
+			</parameter>
+			<parameter name="statistic_name" required="true">
+				<para>The name of the variable to be sent to StatsD. Statistic
+				names cannot contain the pipe (|) character.</para>
+			</parameter>
+			<parameter name="value" required="true">
+				<para>The value of the variable to be sent to StatsD. Values
+				must be numeric. Values for gauge and counter metrics can be
+				sent with a '+' or '-' to update a value after the value has
+				been initialized. Only counters can be initialized as negative.
+				Sets can send a string as the value parameter, but the string
+				cannot contain the pipe character.</para>
+			</parameter>
+			<parameter name="sample_rate">
+				<para>The value of the sample rate to be sent to StatsD. Sample
+				rates less than or equal to 0 will never be sent and sample rates
+				greater than or equal to 1 will always be sent. Any rate
+				between 1 and 0 will be compared to a randomly generated value,
+				and if it is greater than the random value, it will be sent.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>This dialplan application sends statistics to the StatsD
+			server specified inside of <literal>statsd.conf</literal>.</para>
+		</description>
+	</application>
+ ***/
+
+static const char app[] = "StatsD";
+
+/*!
+ * \brief Check to ensure the value is within the allowed range.
+ *
+ * \param value The value of the statistic to be sent to StatsD.
+ *
+ * This function checks to see if the value given to the StatsD daialplan
+ * application is within the allowed range of [-2^63, 2^63] as specified by StatsD.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int value_in_range(const char *value) {
+	double numerical_value = strtod(value, NULL);
+
+	if (numerical_value < pow(-2, 63) || numerical_value > pow(2, 63)) {
+		ast_log(AST_LOG_WARNING, "Value %lf out of range!\n", numerical_value);
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Check to ensure the value is within the allowed range.
+ *
+ * \param value The value of the statistic to be sent to StatsD.
+ *
+ * This function checks to see if the value given to the StatsD daialplan
+ * application is within the allowed range of [0, 2^64] as specified by StatsD.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int non_neg_value_range(const char *value) {
+	double numerical_value = strtod(value, NULL);
+
+	if (numerical_value < 0 || numerical_value > pow(2, 64)) {
+		ast_log(AST_LOG_WARNING, "Value %lf out of range!\n", numerical_value);
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Check to ensure the metric type is a valid metric type.
+ *
+ * \param metric The metric type to be sent to StatsD.
+ *
+ * This function checks to see if the metric type given to the StatsD dialplan
+ * is a valid metric type. Metric types are determined by StatsD.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_metric(const char *metric)
+{
+	const char *valid_metrics[] = {"g","s","ms","c"};
+	int i;
+
+	if (ast_strlen_zero(metric)) {
+		ast_log(AST_LOG_ERROR, "Missing metric type argument.\n");
+		return 1;
+	}
+
+	for (i = 0; i < ARRAY_LEN(valid_metrics); i++) {
+		if (!strcmp(valid_metrics[i], metric)) {
+			return 0;
+		}
+	}
+
+	ast_log(AST_LOG_ERROR, "Invalid metric type %s.\n", metric);
+
+	return 1;
+}
+
+/*!
+ * \brief Check to ensure that a numeric value is valid.
+ *
+ * \param numeric_value The numeric value to be sent to StatsD.
+ *
+ * This function checks to see if a number to be sent to StatsD is actually
+ * a valid number. One decimal is allowed.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_numeric(const char *numeric_value) {
+	const char *num;
+	int decimal_counter = 0;
+
+	num = numeric_value;
+	while (*num) {
+		if (!isdigit(*num++)) {
+			if (strstr(numeric_value, ".") != NULL && decimal_counter == 0) {
+				decimal_counter++;
+				continue;
+			}
+			ast_log(AST_LOG_ERROR, "%s is not a number!\n", numeric_value);
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Determines the actual value of a number by looking for a leading + or -.
+ *
+ * \param raw_value The entire numeric string to be sent to StatsD.
+ *
+ * This function checks to see if the numeric string contains valid characters
+ * and then isolates the actual number to be sent for validation. Returns the
+ * result of the numeric validation.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int determine_actual_value(const char *raw_value) {
+	const char *actual_value;
+
+	if ((raw_value[0] == '+') || (raw_value[0] == '-')) {
+		actual_value = &raw_value[1];
+		if (ast_strlen_zero(actual_value)) {
+			ast_log(AST_LOG_ERROR, "Value argument %s only contains a sign"
+				" operator.\n", raw_value);
+			return 1;
+		}
+	} else {
+		actual_value = &raw_value[0];
+	}
+
+	return validate_numeric(actual_value);
+}
+
+/*!
+ * \brief Check to ensure the statistic name is valid.
+ *
+ * \param name The variable name to be sent to StatsD.
+ *
+ * This function checks to see if the statistic name given to the StatsD
+ * dialplan application is valid by ensuring that the name does not have any
+ * invalid characters.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_name(const char *name)
+{
+	if (ast_strlen_zero(name) || (strstr(name, "|") != NULL)) {
+		ast_log(AST_LOG_ERROR, "Statistic name %s is missing or contains a pipe (|)"
+			" character.\n", name);
+		return 1;
+	}
+
+	return 0;
+}
+
+
+/*!
+ * \brief Calls the appropriate functions to validate a gauge metric.
+ *
+ * \param statistic_name The statistic name to be sent to StatsD.
+ * \param value The value to be sent to StatsD.
+ *
+ * This function calls other validating functions to correctly validate each
+ * input based on allowable input for a gauge metric.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_metric_type_gauge(const char *statistic_name, const char *value) {
+
+	if (ast_strlen_zero(value)) {
+		ast_log(AST_LOG_ERROR, "Missing value argument.\n");
+		return 1;
+	}
+
+	if (validate_name(statistic_name) || determine_actual_value(value)
+		|| value_in_range(value)) {
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Calls the appropriate functions to validate a counter metric.
+ *
+ * \param statistic_name The statistic name to be sent to StatsD.
+ * \param value The value to be sent to StatsD.
+ *
+ * This function calls other validating functions to correctly validate each
+ * input based on allowable input for a counter metric.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_metric_type_counter(const char *statistic_name, const char *value) {
+
+	if (ast_strlen_zero(value)) {
+		ast_log(AST_LOG_ERROR, "Missing value argument.\n");
+		return 1;
+	}
+
+	if (validate_name(statistic_name) || determine_actual_value(value)
+		|| value_in_range(value)) {
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Calls the appropriate functions to validate a timer metric.
+ *
+ * \param statistic_name The statistic name to be sent to StatsD.
+ * \param value The value to be sent to StatsD.
+ *
+ * This function calls other validating functions to correctly validate each
+ * input based on allowable input for a timer metric.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_metric_type_timer(const char *statistic_name, const char *value) {
+
+	if (ast_strlen_zero(value)) {
+		ast_log(AST_LOG_ERROR, "Missing value argument.\n");
+		return 1;
+	}
+
+	if (validate_name(statistic_name) || validate_numeric(value)
+		|| non_neg_value_range(value)) {
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Calls the appropriate functions to validate a set metric.
+ *
+ * \param statistic_name The statistic name to be sent to StatsD.
+ * \param value The value to be sent to StatsD.
+ *
+ * This function calls other validating functions to correctly validate each
+ * input based on allowable input for a set metric.
+ *
+ * \retval zero on success.
+ * \retval 1 on error.
+ */
+static int validate_metric_type_set(const char *statistic_name, const char *value) {
+	if (ast_strlen_zero(value)) {
+		ast_log(AST_LOG_ERROR, "Missing value argument.\n");
+		return 1;
+	}
+
+	if (validate_name(statistic_name)) {
+		return 1;
+	}
+
+	if (strstr(value, "|") != NULL) {
+		ast_log(AST_LOG_ERROR, "Pipe (|) character is not allowed for value %s"
+			" in a set metric.\n", value);
+		return 1;
+	}
+
+	return 0;
+}
+
+static int statsd_exec(struct ast_channel *chan, const char *data)
+{
+	char *stats;
+	double numerical_rate = 1.0;
+
+	AST_DECLARE_APP_ARGS(args,
+			AST_APP_ARG(metric_type);
+			AST_APP_ARG(statistic_name);
+			AST_APP_ARG(value);
+			AST_APP_ARG(sample_rate);
+	);
+
+	if (!data) {
+		ast_log(AST_LOG_ERROR, "No parameters were provided. Correct format is "
+			"StatsD(metric_type,statistic_name,value[,sample_rate]). Sample rate is the "
+			"only optional parameter.\n");
+		return 1;
+	}
+
+	stats = ast_strdupa(data);
+	AST_STANDARD_APP_ARGS(args, stats);
+
+	if (validate_metric(args.metric_type)) {
+		return 1;
+	}
+
+	if (!strcmp(args.metric_type, "g")) {
+		if (validate_metric_type_gauge(args.statistic_name, args.value)) {
+			ast_log(AST_LOG_ERROR, "Invalid input for a gauge metric.\n");
+			return 1;
+		}
+	}
+	else if (!strcmp(args.metric_type, "c")) {
+		if (validate_metric_type_counter(args.statistic_name, args.value)) {
+			ast_log(AST_LOG_ERROR, "Invalid input for a counter metric.\n");
+			return 1;
+		}
+	}
+	else if (!strcmp(args.metric_type, "ms")) {
+		if (validate_metric_type_timer(args.statistic_name, args.value)) {
+			ast_log(AST_LOG_ERROR, "Invalid input for a timer metric.\n");
+			return 1;
+		}
+	}
+	else if (!strcmp(args.metric_type, "s")) {
+		if (validate_metric_type_set(args.statistic_name, args.value)) {
+			ast_log(AST_LOG_ERROR, "Invalid input for a set metric.\n");
+			return 1;
+		}
+	}
+
+	if (args.sample_rate) {
+
+		if (validate_numeric(args.sample_rate)) {
+			return 1;
+		}
+
+		numerical_rate = strtod(args.sample_rate, NULL);
+	}
+
+	ast_statsd_log_string(args.statistic_name, args.metric_type, args.value,
+		numerical_rate);
+
+	return 0;
+}
+
+static int unload_module(void)
+{
+	return ast_unregister_application(app);
+}
+
+static int load_module(void)
+{
+	return ast_register_application_xml(app, statsd_exec);
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "StatsD Dialplan Application",
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_statsd",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_stream_echo.c 1:16.1.1~dfsg-1/apps/app_stream_echo.c
--- 1:13.23.1~dfsg-2/apps/app_stream_echo.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_stream_echo.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,345 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Kevin Harwell <kharwell@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Stream echo application
+ *
+ * \author Kevin Harwell <kharwell@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/app.h"
+#include "asterisk/conversions.h"
+#include "asterisk/file.h"
+#include "asterisk/module.h"
+#include "asterisk/channel.h"
+#include "asterisk/stream.h"
+
+/*** DOCUMENTATION
+	<application name="StreamEcho" language="en_US">
+		<synopsis>
+			Echo media, up to 'N' streams of a type, and DTMF back to the calling party
+		</synopsis>
+		<syntax>
+		<parameter name="num" required="false">
+			<para>The number of streams of a type to echo back. If '0' is specified then
+			all streams of a type are removed.</para>
+		</parameter>
+		<parameter name="type" required="false">
+			<para>The media type of the stream(s) to add or remove (in the case of "num"
+			being '0'). This can be set to either "audio" or "video" (default). If "num"
+			is empty (i.e. not specified) then this parameter is ignored.</para>
+		</parameter>
+		</syntax>
+		<description>
+			<para>If a "num" (the number of streams) is not given then this simply echos
+			back any media or DTMF frames (note, however if '#' is detected then the
+			application exits) read from the calling channel back to itself. This means
+			for any relevant frame read from a particular stream it is written back out
+			to the associated write stream in a one to one fashion.
+			</para>
+			<para>However if a "num" is specified, and if the calling channel allows it
+			(a new offer is made requesting the allowance of additional streams) then any
+			any media received, like before, is echoed back onto each stream. However, in
+			this case a relevant frame received on a stream of the given "type" is also
+			echoed back out to the other streams of that same type. It should be noted that
+			when operating in this mode only the first stream found of the given "type" is
+			allowed from the original offer. And this first stream found is also the only
+			stream of that "type" granted read (send/receive) capabilities in the new offer
+			whereas the additional ones are set to receive only.</para>
+			<note><para>This does not echo CONTROL, MODEM, or NULL frames.</para></note>
+		</description>
+	</application>
+ ***/
+
+static const char app[] = "StreamEcho";
+
+static int stream_echo_write_error(struct ast_channel *chan, struct ast_frame *frame, int pos)
+{
+	char frame_type[32];
+	const char *media_type;
+	struct ast_stream *stream;
+
+	ast_frame_type2str(frame->frametype, frame_type, sizeof(frame_type));
+
+	stream = pos < 0 ?
+		ast_channel_get_default_stream(chan, ast_format_get_type(frame->subclass.format)) :
+		ast_stream_topology_get_stream(ast_channel_get_stream_topology(chan), pos);
+
+	media_type = ast_codec_media_type2str(ast_stream_get_type(stream));
+
+	ast_log(LOG_ERROR, "%s - unable to write frame type '%s' to stream type '%s' at "
+		"position '%d'\n", ast_channel_name(chan), frame_type, media_type,
+		ast_stream_get_position(stream));
+
+	return -1;
+}
+
+static int stream_echo_write(struct ast_channel *chan, struct ast_frame *frame,
+	enum ast_media_type type, int one_to_one)
+{
+	int i;
+	int num;
+	struct ast_stream_topology *topology;
+
+	/*
+	 * Since this is an echo application if we get a frame in on a stream
+	 * we simply want to echo it back out onto the same stream number.
+	 */
+	num = ast_channel_is_multistream(chan) ? frame->stream_num : -1;
+	if (ast_write_stream(chan, num, frame)) {
+		return stream_echo_write_error(chan, frame, num);
+	}
+
+	/*
+	 * If the frame's type and given type don't match, or we are operating in
+	 * a one to one stream echo mode then there is nothing left to do.
+	 *
+	 * Note, if the channel is not multi-stream capable then one_to_one will
+	 * always be true, so it is safe to also not check for that here too.
+	 */
+	if (one_to_one || !frame->subclass.format ||
+	    ast_format_get_type(frame->subclass.format) != type) {
+		return 0;
+	}
+
+	/*
+	 * However, if we are operating in a single stream echoed to many stream
+	 * mode, and the frame's type matches the given type then we also need to
+	 * find the other streams of the same type and write out to those streams
+	 * as well.
+	 *
+	 * If we are here, then it's accepted that whatever stream number the frame
+	 * was read from for the given type is the only one set to send/receive,
+	 * while the others of the same type are set to receive only. Since we
+	 * shouldn't assume any order to the streams, we'll loop back through all
+	 * streams in the channel's topology writing only to those of the same type.
+	 * And, of course also not the stream which has already been written to.
+	 */
+	topology = ast_channel_get_stream_topology(chan);
+
+	for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
+		struct ast_stream *stream = ast_stream_topology_get_stream(topology, i);
+		if (num != i && ast_stream_get_type(stream) == type) {
+			if (ast_write_stream(chan, i, frame)) {
+				return stream_echo_write_error(chan, frame, i);
+			}
+		}
+	}
+
+	return 0;
+}
+
+static int stream_echo_perform(struct ast_channel *chan,
+	struct ast_stream_topology *topology, enum ast_media_type type)
+{
+	int update_sent = 0;
+	int request_change = topology != NULL;
+	int one_to_one = 1;
+
+	while (ast_waitfor(chan, -1) > -1) {
+		struct ast_frame *f;
+
+		if (request_change) {
+			/* Request a change to the new topology */
+			if (ast_channel_request_stream_topology_change(chan, topology, NULL)) {
+				ast_log(LOG_WARNING, "Request stream topology change not supported "
+					"by channel '%s'\n", ast_channel_name(chan));
+			}
+			request_change = 0;
+		}
+
+		f = ast_read_stream(chan);
+		if (!f) {
+			return -1;
+		}
+
+		if ((f->frametype == AST_FRAME_DTMF) && (f->subclass.integer == '#')) {
+			ast_frfree(f);
+			break;
+		}
+
+		f->delivery.tv_sec = 0;
+		f->delivery.tv_usec = 0;
+
+		if (f->frametype == AST_FRAME_CONTROL) {
+			if (f->subclass.integer == AST_CONTROL_VIDUPDATE && !update_sent) {
+				if (stream_echo_write(chan, f, type, one_to_one)) {
+					ast_frfree(f);
+					return -1;
+				}
+				update_sent = 1;
+			} else if (f->subclass.integer == AST_CONTROL_SRCCHANGE) {
+				update_sent = 0;
+			} else if (f->subclass.integer == AST_CONTROL_STREAM_TOPOLOGY_CHANGED) {
+				update_sent = 0;
+				one_to_one = 0; /* Switch writing to one to many */
+			}
+		} else if (f->frametype == AST_FRAME_VIDEO && !update_sent){
+			struct ast_frame frame = {
+				.frametype = AST_FRAME_CONTROL,
+				.subclass.integer = AST_CONTROL_VIDUPDATE,
+			};
+			stream_echo_write(chan, &frame, type, one_to_one);
+			update_sent = 1;
+		}
+
+		if (f->frametype != AST_FRAME_CONTROL &&
+		    f->frametype != AST_FRAME_MODEM &&
+		    f->frametype != AST_FRAME_NULL &&
+		    stream_echo_write(chan, f, type, one_to_one)) {
+			ast_frfree(f);
+			return -1;
+		}
+
+		ast_frfree(f);
+	}
+
+	return 0;
+}
+
+static struct ast_stream_topology *stream_echo_topology_alloc(
+	struct ast_stream_topology *original, unsigned int num, enum ast_media_type type)
+{
+	int i, n = num;
+	struct ast_stream_topology *res = ast_stream_topology_alloc();
+
+	if (!res) {
+		return NULL;
+	}
+
+	/*
+	 * Clone every stream of a type not matching the given one. If the type
+	 * matches clone only the first stream found for the given type. Then for
+	 * that stream clone it again up to num - 1 times. Ignore any other streams
+	 * of the same matched type in the original topology.
+	 *
+	 * So for instance if the original stream contains 1 audio stream and 2 video
+	 * streams (video stream 'A' and video stream 'B'), num is '3', and the given
+	 * type is 'video' then the resulting topology will contain a clone of the
+	 * audio stream along with 3 clones of video stream 'A'. Video stream 'B' is
+	 * not copied over.
+	 */
+	for (i = 0; i < ast_stream_topology_get_count(original); ++i) {
+		struct ast_stream *stream = ast_stream_topology_get_stream(original, i);
+
+		if (!n && ast_stream_get_type(stream) == type) {
+			/* Don't copy any[more] of the given type */
+			continue;
+		}
+
+		if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+			/* Don't copy removed/declined streams */
+			continue;
+		}
+
+		do {
+			stream = ast_stream_clone(stream, NULL);
+
+			if (!stream || ast_stream_topology_append_stream(res, stream) < 0) {
+				ast_stream_free(stream);
+				ast_stream_topology_free(res);
+				return NULL;
+			}
+
+			if (ast_stream_get_type(stream) != type) {
+				/* Do not multiply non matching streams */
+				break;
+			}
+
+			/*
+			 * Since num is not zero yet (i.e. this is first stream found to
+			 * match on the type) and the types match then loop num - 1 times
+			 * cloning the same stream.
+			 */
+			ast_stream_set_state(stream, n == num ?
+			     AST_STREAM_STATE_SENDRECV : AST_STREAM_STATE_RECVONLY);
+		} while (--n);
+	}
+
+	return res;
+}
+
+static int stream_echo_exec(struct ast_channel *chan, const char *data)
+{
+	int res;
+	unsigned int num = 0;
+	enum ast_media_type type;
+	char *parse;
+	struct ast_stream_topology *topology;
+
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(num);
+		AST_APP_ARG(type);
+	);
+
+	parse = ast_strdupa(data);
+	AST_STANDARD_APP_ARGS(args, parse);
+
+	if (ast_strlen_zero(args.num)) {
+		/*
+		 * If a number is not given then no topology is to be created
+		 * and renegotiated. The app will just echo back each stream
+		 * received to itself.
+		 */
+		return stream_echo_perform(chan, NULL, AST_MEDIA_TYPE_UNKNOWN);
+	}
+
+	if (ast_str_to_uint(args.num, &num)) {
+		ast_log(LOG_ERROR, "Failed to parse the first parameter '%s' into a"
+			" greater than or equal to zero\n", args.num);
+		return -1;
+	}
+
+	type = ast_strlen_zero(args.type) ? AST_MEDIA_TYPE_VIDEO :
+		ast_media_type_from_str(args.type);
+
+	topology = stream_echo_topology_alloc(
+		ast_channel_get_stream_topology(chan), num, type);
+	if (!topology) {
+		ast_log(LOG_ERROR, "Unable to create '%u' streams of type '%s' to"
+			" the topology\n", num, ast_codec_media_type2str(type));
+		return -1;
+	}
+
+	res = stream_echo_perform(chan, topology, type);
+
+	if (ast_channel_get_stream_topology(chan) != topology) {
+		ast_stream_topology_free(topology);
+	}
+
+	return res;
+}
+
+static int unload_module(void)
+{
+	return ast_unregister_application(app);
+}
+
+static int load_module(void)
+{
+	return ast_register_application_xml(app, stream_echo_exec);
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Stream Echo Application");
diff -pruN 1:13.23.1~dfsg-2/apps/app_system.c 1:16.1.1~dfsg-1/apps/app_system.c
--- 1:13.23.1~dfsg-2/apps/app_system.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_system.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_talkdetect.c 1:16.1.1~dfsg-1/apps/app_talkdetect.c
--- 1:13.23.1~dfsg-2/apps/app_talkdetect.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_talkdetect.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_test.c 1:16.1.1~dfsg-1/apps/app_test.c
--- 1:13.23.1~dfsg-2/apps/app_test.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_test.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
diff -pruN 1:13.23.1~dfsg-2/apps/app_transfer.c 1:16.1.1~dfsg-1/apps/app_transfer.c
--- 1:13.23.1~dfsg-2/apps/app_transfer.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_transfer.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_url.c 1:16.1.1~dfsg-1/apps/app_url.c
--- 1:13.23.1~dfsg-2/apps/app_url.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_url.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,13 +26,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/app.h"
@@ -179,4 +177,4 @@ static int load_module(void)
 	return ast_register_application_xml(app, sendurl_exec);
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Send URL Applications");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Send URL Applications");
diff -pruN 1:13.23.1~dfsg-2/apps/app_userevent.c 1:16.1.1~dfsg-1/apps/app_userevent.c
--- 1:13.23.1~dfsg-2/apps/app_userevent.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_userevent.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/manager.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_verbose.c 1:16.1.1~dfsg-1/apps/app_verbose.c
--- 1:13.23.1~dfsg-2/apps/app_verbose.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_verbose.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/app.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_voicemail.c 1:16.1.1~dfsg-1/apps/app_voicemail.c
--- 1:13.23.1~dfsg-2/apps/app_voicemail.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_voicemail.c	2018-12-26 18:45:13.000000000 +0000
@@ -100,8 +100,6 @@
 #endif
 #endif
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/paths.h"	/* use ast_config_AST_SPOOL_DIR */
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -464,6 +462,23 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 		<description>
 		</description>
 	</manager>
+	<manager name="VoicemailUserStatus" language="en_US">
+		<synopsis>
+			Show the status of given voicemail user's info.
+		</synopsis>
+		<syntax>
+			<xi:include xpointer="xpointer(/docs/manager[@name='Login']/syntax/parameter[@name='ActionID'])" />
+			<parameter name="Context" required="true">
+				<para>The context you want to check.</para>
+			</parameter>
+			<parameter name="Mailbox" required="true">
+				<para>The mailbox you want to check.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Retrieves the status of the given voicemail user.</para>
+		</description>
+	</manager>
 	<manager name="VoicemailRefresh" language="en_US">
 		<synopsis>
 			Tell Asterisk to poll mailboxes for a change
@@ -795,11 +810,16 @@ struct baseio {
 	unsigned char iobuf[BASEMAXINLINE];
 };
 
+#define MAX_VM_MBOX_ID_LEN (AST_MAX_EXTENSION)
+#define MAX_VM_CONTEXT_LEN (AST_MAX_CONTEXT)
+/* MAX_VM_MAILBOX_LEN allows enough room for the '@' and NULL terminator */
+#define MAX_VM_MAILBOX_LEN (MAX_VM_MBOX_ID_LEN + MAX_VM_CONTEXT_LEN)
+
 /*! Structure for linked list of users
  * Use ast_vm_user_destroy() to free one of these structures. */
 struct ast_vm_user {
-	char context[AST_MAX_CONTEXT];   /*!< Voicemail context */
-	char mailbox[AST_MAX_EXTENSION]; /*!< Mailbox id, unique within vm context */
+	char context[MAX_VM_CONTEXT_LEN];/*!< Voicemail context */
+	char mailbox[MAX_VM_MBOX_ID_LEN];/*!< Mailbox id, unique within vm context */
 	char password[80];               /*!< Secret pin code, numbers only */
 	char fullname[80];               /*!< Full name, for directory app */
 	char *email;                     /*!< E-mail address */
@@ -1106,6 +1126,8 @@ static int write_password_to_file(const
 static const char *substitute_escapes(const char *value);
 static int message_range_and_existence_check(struct vm_state *vms, const char *msg_ids [], size_t num_msgs, int *msg_nums, struct ast_vm_user *vmu);
 static void notify_new_state(struct ast_vm_user *vmu);
+static int append_vmu_info_astman(struct mansession *s, struct ast_vm_user *vmu, const char* event_name, const char* actionid);
+
 
 /*!
  * Place a message in the indicated folder
@@ -2021,6 +2043,10 @@ static void free_user(struct ast_vm_user
 		return;
 	}
 
+	if (!ast_strlen_zero(vmu->mailbox)) {
+		ast_delete_mwi_state_full(vmu->mailbox, vmu->context, NULL);
+	}
+
 	ast_free(vmu->email);
 	vmu->email = NULL;
 	ast_free(vmu->emailbody);
@@ -6083,22 +6109,33 @@ static int has_voicemail(const char *mai
 	return 0;
 }
 
-
+/*!
+ * \brief Check the given mailbox's message count.
+ * \param mailbox The @ delimited string for user@context. If no context is found, uses 'default' for the context.
+ * \param urgentmsgs  urgent message count.
+ * \param newmsgs new message count.
+ * \param oldmsgs old message count pointer
+ * \return -1 if error occurred, 0 otherwise.
+ */
 static int inboxcount2(const char *mailbox, int *urgentmsgs, int *newmsgs, int *oldmsgs)
 {
 	char tmp[256];
 	char *context;
 
 	/* If no mailbox, return immediately */
-	if (ast_strlen_zero(mailbox))
+	if (ast_strlen_zero(mailbox)) {
 		return 0;
+	}
 
-	if (newmsgs)
+	if (newmsgs) {
 		*newmsgs = 0;
-	if (oldmsgs)
+	}
+	if (oldmsgs) {
 		*oldmsgs = 0;
-	if (urgentmsgs)
+	}
+	if (urgentmsgs) {
 		*urgentmsgs = 0;
+	}
 
 	if (strchr(mailbox, ',')) {
 		int tmpnew, tmpold, tmpurgent;
@@ -6108,15 +6145,18 @@ static int inboxcount2(const char *mailb
 		mb = tmp;
 		while ((cur = strsep(&mb, ", "))) {
 			if (!ast_strlen_zero(cur)) {
-				if (inboxcount2(cur, urgentmsgs ? &tmpurgent : NULL, newmsgs ? &tmpnew : NULL, oldmsgs ? &tmpold : NULL))
+				if (inboxcount2(cur, urgentmsgs ? &tmpurgent : NULL, newmsgs ? &tmpnew : NULL, oldmsgs ? &tmpold : NULL)) {
 					return -1;
-				else {
-					if (newmsgs)
+				} else {
+					if (newmsgs) {
 						*newmsgs += tmpnew;
-					if (oldmsgs)
+					}
+					if (oldmsgs) {
 						*oldmsgs += tmpold;
-					if (urgentmsgs)
+					}
+					if (urgentmsgs) {
 						*urgentmsgs += tmpurgent;
+					}
 				}
 			}
 		}
@@ -6125,17 +6165,21 @@ static int inboxcount2(const char *mailb
 
 	ast_copy_string(tmp, mailbox, sizeof(tmp));
 
-	if ((context = strchr(tmp, '@')))
+	if ((context = strchr(tmp, '@'))) {
 		*context++ = '\0';
-	else
+	} else {
 		context = "default";
+	}
 
-	if (newmsgs)
+	if (newmsgs) {
 		*newmsgs = __has_voicemail(context, tmp, "INBOX", 0);
-	if (oldmsgs)
+	}
+	if (oldmsgs) {
 		*oldmsgs = __has_voicemail(context, tmp, "Old", 0);
-	if (urgentmsgs)
+	}
+	if (urgentmsgs) {
 		*urgentmsgs = __has_voicemail(context, tmp, "Urgent", 0);
+	}
 
 	return 0;
 }
@@ -8588,6 +8632,8 @@ static int play_message_datetime(struct
 		res = ast_say_date_with_format(chan, t, AST_DIGIT_ANY, ast_channel_language(chan), "'vm-received' Q 'digits/at' HM", NULL);
 	} else if (!strncasecmp(ast_channel_language(chan), "gr", 2)) {     /* GREEK syntax */
 		res = ast_say_date_with_format(chan, t, AST_DIGIT_ANY, ast_channel_language(chan), "'vm-received' q  H 'digits/kai' M ", NULL);
+	} else if (!strncasecmp(ast_channel_language(chan), "is", 2)) {     /* ICELANDIC syntax */
+		res = ast_say_date_with_format(chan, t, AST_DIGIT_ANY, ast_channel_language(chan), "'vm-received' Q 'digits/at' HM", NULL);
 	} else if (!strncasecmp(ast_channel_language(chan), "it", 2)) {     /* ITALIAN syntax */
 		res = ast_say_date_with_format(chan, t, AST_DIGIT_ANY, ast_channel_language(chan), "'vm-received' q 'digits/at' 'digits/hours' k 'digits/e' M 'digits/minutes'", NULL);
 	} else if (!strcasecmp(ast_channel_language(chan),"ja")) {     /* Japanese syntax */
@@ -8814,6 +8860,12 @@ static int play_message(struct ast_chann
 				res = wait_file2(chan, vms, "vm-number");
 				res = ast_say_number(chan, vms->curmsg + 1, AST_DIGIT_ANY, ast_channel_language(chan), "f");
 			}
+		/* ICELANDIC syntax */
+		} else if (!strncasecmp(ast_channel_language(chan), "is", 2)) {
+			res = wait_file2(chan, vms, "vm-message");
+			if (vms->curmsg && (vms->curmsg != vms->lastmsg)) {
+				res = ast_say_number(chan, vms->curmsg + 1, AST_DIGIT_ANY, ast_channel_language(chan), "n");
+			}
 		/* VIETNAMESE syntax */
 		} else if (!strncasecmp(ast_channel_language(chan), "vi", 2)) {
 			if (!vms->curmsg) {
@@ -9554,6 +9606,75 @@ static int vm_intro_en(struct ast_channe
 	return res;
 }
 
+/* ICELANDIC syntax */
+static int vm_intro_is(struct ast_channel *chan, struct vm_state *vms)
+{
+	int res;
+
+	/* Introduce messages they have */
+	res = ast_play_and_wait(chan, "vm-youhave");
+	if (!res) {
+		if (vms->urgentmessages) {
+			/* Digits 1-4 are spoken in neutral and plural when talking about messages,
+			   however, feminine is used for 1 as it is the same as the neutral for plural,
+			   and singular neutral is the same after 1. */
+			if (vms->urgentmessages < 5) {
+				char recname[16];
+				if (vms->urgentmessages == 1)
+					snprintf(recname, sizeof(recname), "digits/1kvk");
+				else
+					snprintf(recname, sizeof(recname), "digits/%dhk", vms->urgentmessages);
+				res = ast_play_and_wait(chan, recname);
+			} else if (!res)
+				res = ast_play_and_wait(chan, "vm-Urgent");
+			if ((vms->oldmessages || vms->newmessages) && !res) {
+				res = ast_play_and_wait(chan, "vm-and");
+			} else if (!res)
+				res = ast_play_and_wait(chan, "vm-messages");
+		}
+		if (vms->newmessages) {
+			if (vms->newmessages < 5) {
+				char recname[16];
+				if (vms->newmessages == 1)
+					snprintf(recname, sizeof(recname), "digits/1kvk");
+				else
+					snprintf(recname, sizeof(recname), "digits/%dhk", vms->newmessages);
+				res = ast_play_and_wait(chan, recname);
+			} else
+				res = say_and_wait(chan, vms->newmessages, ast_channel_language(chan));
+			if (!res)
+				res = ast_play_and_wait(chan, "vm-INBOX");
+			if (vms->oldmessages && !res)
+				res = ast_play_and_wait(chan, "vm-and");
+			else if (!res)
+				res = ast_play_and_wait(chan, "vm-messages");
+		}
+		if (!res && vms->oldmessages) {
+			if (vms->oldmessages < 5) {
+				char recname[16];
+				if (vms->oldmessages == 1)
+					snprintf(recname, sizeof(recname), "digits/1kvk");
+				else
+					snprintf(recname, sizeof(recname), "digits/%dhk", vms->oldmessages);
+				res = ast_play_and_wait(chan, recname);
+			} else
+				res = say_and_wait(chan, vms->oldmessages, ast_channel_language(chan));
+			if (!res)
+				res = ast_play_and_wait(chan, "vm-Old");
+			if (!res)
+				res = ast_play_and_wait(chan, "vm-messages");
+		}
+		if (!res) {
+			if (!vms->urgentmessages && !vms->oldmessages && !vms->newmessages) {
+				res = ast_play_and_wait(chan, "vm-no");
+				if (!res)
+					res = ast_play_and_wait(chan, "vm-messages");
+			}
+		}
+	}
+	return res;
+}
+
 /* ITALIAN syntax */
 static int vm_intro_it(struct ast_channel *chan, struct vm_state *vms)
 {
@@ -10218,6 +10339,8 @@ static int vm_intro(struct ast_channel *
 		return vm_intro_gr(chan, vms);
 	} else if (!strncasecmp(ast_channel_language(chan), "he", 2)) {  /* HEBREW syntax */
 		return vm_intro_he(chan, vms);
+	} else if (!strncasecmp(ast_channel_language(chan), "is", 2)) {  /* ICELANDIC syntax */
+		return vm_intro_is(chan, vms);
 	} else if (!strncasecmp(ast_channel_language(chan), "it", 2)) {  /* ITALIAN syntax */
 		return vm_intro_it(chan, vms);
 	} else if (!strncasecmp(ast_channel_language(chan), "ja", 2)) {  /* JAPANESE syntax */
@@ -12243,7 +12366,7 @@ static int append_mailbox(const char *co
 	char *stringp;
 	char *s;
 	struct ast_vm_user *vmu;
-	char *mailbox_full;
+	char mailbox_full[MAX_VM_MAILBOX_LEN];
 	int new = 0, old = 0, urgent = 0;
 	char secretfn[PATH_MAX] = "";
 
@@ -12282,10 +12405,10 @@ static int append_mailbox(const char *co
 		read_password_from_file(secretfn, vmu->password, sizeof(vmu->password));
 	}
 
-	mailbox_full = ast_alloca(strlen(box) + strlen(context) + 1);
-	strcpy(mailbox_full, box);
-	strcat(mailbox_full, "@");
-	strcat(mailbox_full, context);
+	snprintf(mailbox_full, MAX_VM_MAILBOX_LEN, "%s%s%s",
+		box,
+		ast_strlen_zero(context) ? "" : "@",
+		context);
 
 	inboxcount2(mailbox_full, &urgent, &new, &old);
 #ifdef IMAP_STORAGE
@@ -12896,147 +13019,6 @@ static struct ast_cli_entry cli_voicemai
 	AST_CLI_DEFINE(handle_voicemail_reload, "Reload voicemail configuration"),
 };
 
-#ifdef IMAP_STORAGE
-	#define DATA_EXPORT_VM_USERS(USER)					\
-		USER(ast_vm_user, context, AST_DATA_STRING)			\
-		USER(ast_vm_user, mailbox, AST_DATA_STRING)			\
-		USER(ast_vm_user, password, AST_DATA_PASSWORD)			\
-		USER(ast_vm_user, fullname, AST_DATA_STRING)			\
-		USER(ast_vm_user, email, AST_DATA_STRING)			\
-		USER(ast_vm_user, emailsubject, AST_DATA_STRING)		\
-		USER(ast_vm_user, emailbody, AST_DATA_STRING)			\
-		USER(ast_vm_user, pager, AST_DATA_STRING)			\
-		USER(ast_vm_user, serveremail, AST_DATA_STRING)			\
-		USER(ast_vm_user, fromstring, AST_DATA_STRING)			\
-		USER(ast_vm_user, language, AST_DATA_STRING)			\
-		USER(ast_vm_user, zonetag, AST_DATA_STRING)			\
-		USER(ast_vm_user, callback, AST_DATA_STRING)			\
-		USER(ast_vm_user, dialout, AST_DATA_STRING)			\
-		USER(ast_vm_user, uniqueid, AST_DATA_STRING)			\
-		USER(ast_vm_user, exit, AST_DATA_STRING)			\
-		USER(ast_vm_user, attachfmt, AST_DATA_STRING)			\
-		USER(ast_vm_user, flags, AST_DATA_UNSIGNED_INTEGER)		\
-		USER(ast_vm_user, saydurationm, AST_DATA_INTEGER)		\
-		USER(ast_vm_user, maxmsg, AST_DATA_INTEGER)			\
-		USER(ast_vm_user, maxdeletedmsg, AST_DATA_INTEGER)		\
-		USER(ast_vm_user, maxsecs, AST_DATA_INTEGER)			\
-		USER(ast_vm_user, imapuser, AST_DATA_STRING)			\
-		USER(ast_vm_user, imappassword, AST_DATA_STRING)		\
-		USER(ast_vm_user, imapvmshareid, AST_DATA_STRING)		\
-		USER(ast_vm_user, volgain, AST_DATA_DOUBLE)
-#else
-	#define DATA_EXPORT_VM_USERS(USER)					\
-		USER(ast_vm_user, context, AST_DATA_STRING)			\
-		USER(ast_vm_user, mailbox, AST_DATA_STRING)			\
-		USER(ast_vm_user, password, AST_DATA_PASSWORD)			\
-		USER(ast_vm_user, fullname, AST_DATA_STRING)			\
-		USER(ast_vm_user, email, AST_DATA_STRING)			\
-		USER(ast_vm_user, emailsubject, AST_DATA_STRING)		\
-		USER(ast_vm_user, emailbody, AST_DATA_STRING)			\
-		USER(ast_vm_user, pager, AST_DATA_STRING)			\
-		USER(ast_vm_user, serveremail, AST_DATA_STRING)			\
-		USER(ast_vm_user, fromstring, AST_DATA_STRING)			\
-		USER(ast_vm_user, language, AST_DATA_STRING)			\
-		USER(ast_vm_user, zonetag, AST_DATA_STRING)			\
-		USER(ast_vm_user, callback, AST_DATA_STRING)			\
-		USER(ast_vm_user, dialout, AST_DATA_STRING)			\
-		USER(ast_vm_user, uniqueid, AST_DATA_STRING)			\
-		USER(ast_vm_user, exit, AST_DATA_STRING)			\
-		USER(ast_vm_user, attachfmt, AST_DATA_STRING)			\
-		USER(ast_vm_user, flags, AST_DATA_UNSIGNED_INTEGER)		\
-		USER(ast_vm_user, saydurationm, AST_DATA_INTEGER)		\
-		USER(ast_vm_user, maxmsg, AST_DATA_INTEGER)			\
-		USER(ast_vm_user, maxdeletedmsg, AST_DATA_INTEGER)		\
-		USER(ast_vm_user, maxsecs, AST_DATA_INTEGER)			\
-		USER(ast_vm_user, volgain, AST_DATA_DOUBLE)
-#endif
-
-AST_DATA_STRUCTURE(ast_vm_user, DATA_EXPORT_VM_USERS);
-
-#define DATA_EXPORT_VM_ZONES(ZONE)			\
-	ZONE(vm_zone, name, AST_DATA_STRING)		\
-	ZONE(vm_zone, timezone, AST_DATA_STRING)	\
-	ZONE(vm_zone, msg_format, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(vm_zone, DATA_EXPORT_VM_ZONES);
-
-/*!
- * \internal
- * \brief Add voicemail user to the data_root.
- * \param[in] search The search tree.
- * \param[in] data_root The main result node.
- * \param[in] user The voicemail user.
- */
-static int vm_users_data_provider_get_helper(const struct ast_data_search *search,
-    struct ast_data *data_root, struct ast_vm_user *user)
-{
-	struct ast_data *data_user, *data_zone;
-	struct ast_data *data_state;
-	struct vm_zone *zone = NULL;
-	int urgentmsg = 0, newmsg = 0, oldmsg = 0;
-	char ext_context[256] = "";
-
-	data_user = ast_data_add_node(data_root, "user");
-	if (!data_user) {
-		return -1;
-	}
-
-	ast_data_add_structure(ast_vm_user, data_user, user);
-
-	AST_LIST_LOCK(&zones);
-	AST_LIST_TRAVERSE(&zones, zone, list) {
-		if (!strcmp(zone->name, user->zonetag)) {
-			break;
-		}
-	}
-	AST_LIST_UNLOCK(&zones);
-
-	/* state */
-	data_state = ast_data_add_node(data_user, "state");
-	if (!data_state) {
-		return -1;
-	}
-	snprintf(ext_context, sizeof(ext_context), "%s@%s", user->mailbox, user->context);
-	inboxcount2(ext_context, &urgentmsg, &newmsg, &oldmsg);
-	ast_data_add_int(data_state, "urgentmsg", urgentmsg);
-	ast_data_add_int(data_state, "newmsg", newmsg);
-	ast_data_add_int(data_state, "oldmsg", oldmsg);
-
-	if (zone) {
-		data_zone = ast_data_add_node(data_user, "zone");
-		ast_data_add_structure(vm_zone, data_zone, zone);
-	}
-
-	if (!ast_data_search_match(search, data_user)) {
-		ast_data_remove_node(data_root, data_user);
-	}
-
-	return 0;
-}
-
-static int vm_users_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ast_vm_user *user;
-
-	AST_LIST_LOCK(&users);
-	AST_LIST_TRAVERSE(&users, user, list) {
-		vm_users_data_provider_get_helper(search, data_root, user);
-	}
-	AST_LIST_UNLOCK(&users);
-
-	return 0;
-}
-
-static const struct ast_data_handler vm_users_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = vm_users_data_provider_get
-};
-
-static const struct ast_data_entry vm_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/application/voicemail/list", &vm_users_data_provider)
-};
-
 static void poll_subscribed_mailbox(struct mwi_sub *mwi_sub)
 {
 	int new = 0, old = 0, urgent = 0;
@@ -13307,6 +13289,127 @@ static void stop_poll_thread(void)
 	poll_thread = AST_PTHREADT_NULL;
 }
 
+/*!
+ * \brief Append vmu info string into given astman with event_name.
+ * \return 0 failed. 1 otherwise.
+*/
+static int append_vmu_info_astman(
+		struct mansession *s,
+		struct ast_vm_user *vmu,
+		const char* event_name,
+		const char* actionid
+		)
+{
+	int new;
+	int old;
+	char *mailbox;
+	int ret;
+
+	if((s == NULL) || (vmu == NULL) || (event_name == NULL) || (actionid == NULL)) {
+		ast_log(LOG_ERROR, "Wrong input parameter.");
+		return 0;
+	}
+
+	/* create mailbox string */
+	if (!ast_strlen_zero(vmu->context)) {
+		ret = ast_asprintf(&mailbox, "%s@%s", vmu->mailbox, vmu->context);
+	} else {
+		ret = ast_asprintf(&mailbox, "%s", vmu->mailbox);
+	}
+	if (ret == -1) {
+		ast_log(LOG_ERROR, "Could not create mailbox string. err[%s]\n", strerror(errno));
+		return 0;
+	}
+
+	/* get mailbox count */
+	ret = inboxcount(mailbox, &new, &old);
+	ast_free(mailbox);
+	if (ret == -1) {
+		ast_log(LOG_ERROR, "Could not get mailbox count. user[%s], context[%s]\n",
+			vmu->mailbox ?: "", vmu->context ?: "");
+		return 0;
+	}
+
+	astman_append(s,
+		"Event: %s\r\n"
+		"%s"
+		"VMContext: %s\r\n"
+		"VoiceMailbox: %s\r\n"
+		"Fullname: %s\r\n"
+		"Email: %s\r\n"
+		"Pager: %s\r\n"
+		"ServerEmail: %s\r\n"
+		"FromString: %s\r\n"
+		"MailCommand: %s\r\n"
+		"Language: %s\r\n"
+		"TimeZone: %s\r\n"
+		"Callback: %s\r\n"
+		"Dialout: %s\r\n"
+		"UniqueID: %s\r\n"
+		"ExitContext: %s\r\n"
+		"SayDurationMinimum: %d\r\n"
+		"SayEnvelope: %s\r\n"
+		"SayCID: %s\r\n"
+		"AttachMessage: %s\r\n"
+		"AttachmentFormat: %s\r\n"
+		"DeleteMessage: %s\r\n"
+		"VolumeGain: %.2f\r\n"
+		"CanReview: %s\r\n"
+		"CallOperator: %s\r\n"
+		"MaxMessageCount: %d\r\n"
+		"MaxMessageLength: %d\r\n"
+		"NewMessageCount: %d\r\n"
+		"OldMessageCount: %d\r\n"
+#ifdef IMAP_STORAGE
+		"IMAPUser: %s\r\n"
+		"IMAPServer: %s\r\n"
+		"IMAPPort: %s\r\n"
+		"IMAPFlags: %s\r\n"
+#endif
+		"\r\n",
+
+		event_name,
+		actionid,
+		vmu->context,
+		vmu->mailbox,
+		vmu->fullname,
+		vmu->email,
+		vmu->pager,
+		ast_strlen_zero(vmu->serveremail) ? serveremail : vmu->serveremail,
+		ast_strlen_zero(vmu->fromstring) ? fromstring : vmu->fromstring,
+		mailcmd,
+		vmu->language,
+		vmu->zonetag,
+		vmu->callback,
+		vmu->dialout,
+		vmu->uniqueid,
+		vmu->exit,
+		vmu->saydurationm,
+		ast_test_flag(vmu, VM_ENVELOPE) ? "Yes" : "No",
+		ast_test_flag(vmu, VM_SAYCID) ? "Yes" : "No",
+		ast_test_flag(vmu, VM_ATTACH) ? "Yes" : "No",
+		vmu->attachfmt,
+		ast_test_flag(vmu, VM_DELETE) ? "Yes" : "No",
+		vmu->volgain,
+		ast_test_flag(vmu, VM_REVIEW) ? "Yes" : "No",
+		ast_test_flag(vmu, VM_OPERATOR) ? "Yes" : "No",
+		vmu->maxmsg,
+		vmu->maxsecs,
+		new,
+		old
+#ifdef IMAP_STORAGE
+		,
+		vmu->imapuser,
+		vmu->imapserver,
+		vmu->imapport,
+		vmu->imapflags
+#endif
+		);
+
+	return 1;
+
+}
+
 static int manager_voicemail_refresh(struct mansession *s, const struct message *m)
 {
 	const char *context = astman_get_header(m, "Context");
@@ -13343,6 +13446,51 @@ static int manager_voicemail_refresh(str
 	return RESULT_SUCCESS;
 }
 
+static int manager_status_voicemail_user(struct mansession *s, const struct message *m)
+{
+	struct ast_vm_user *vmu = NULL;
+	const char *id = astman_get_header(m, "ActionID");
+	char actionid[128];
+	struct ast_vm_user svm;
+	int ret;
+
+	const char *context = astman_get_header(m, "Context");
+	const char *mailbox = astman_get_header(m, "Mailbox");
+
+	if ((ast_strlen_zero(context) || ast_strlen_zero(mailbox))) {
+		astman_send_error(s, m, "Need 'Context' and 'Mailbox' parameters.");
+		return RESULT_SUCCESS;
+	}
+
+	actionid[0] = '\0';
+	if (!ast_strlen_zero(id)) {
+		snprintf(actionid, sizeof(actionid), "ActionID: %s\r\n", id);
+	}
+
+	/* find user */
+	memset(&svm, 0, sizeof(svm));
+	vmu = find_user(&svm, context, mailbox);
+	if (!vmu) {
+		/* could not find it */
+		astman_send_ack(s, m, "There is no voicemail user of the given info.");
+		return RESULT_SUCCESS;
+	}
+
+	astman_send_listack(s, m, "Voicemail user detail will follow", "start");
+
+	/* append vmu info event */
+	ret = append_vmu_info_astman(s, vmu, "VoicemailUserDetail", actionid);
+	free_user(vmu);
+	if(ret == 0) {
+		ast_log(LOG_ERROR, "Could not append voicemail user info.");
+	}
+
+	astman_send_list_complete_start(s, m, "VoicemailUserDetailComplete", 1);
+	astman_send_list_complete_end(s);
+
+	return RESULT_SUCCESS;
+}
+
 /*! \brief Manager list voicemail users command */
 static int manager_list_voicemail_users(struct mansession *s, const struct message *m)
 {
@@ -13350,6 +13498,7 @@ static int manager_list_voicemail_users(
 	const char *id = astman_get_header(m, "ActionID");
 	char actionid[128];
 	int num_users = 0;
+	int ret;
 
 	actionid[0] = '\0';
 	if (!ast_strlen_zero(id)) {
@@ -13367,103 +13516,12 @@ static int manager_list_voicemail_users(
 	astman_send_listack(s, m, "Voicemail user list will follow", "start");
 
 	AST_LIST_TRAVERSE(&users, vmu, list) {
-		int new, old;
-		int ret;
-		char *mailbox;
-
-		/* create mailbox string */
-		if (!ast_strlen_zero(vmu->context)) {
-			ret = ast_asprintf(&mailbox, "%s@%s", vmu->mailbox, vmu->context);
-		} else {
-			ret = ast_asprintf(&mailbox, "%s", vmu->mailbox);
-		}
-		if (ret == -1) {
-			ast_log(LOG_ERROR, "Could not create mailbox string. err[%s]\n", strerror(errno));
-			continue;
-		}
-
-		/* get mailbox count */
-		ret = inboxcount(mailbox, &new, &old);
-		ast_free(mailbox);
-		if (ret == -1) {
-			ast_log(LOG_ERROR, "Could not get mailbox count. username[%s], context[%s]\n",
-				vmu->mailbox ?: "", vmu->context ?: "");
+		/* append vmu info event */
+		ret = append_vmu_info_astman(s, vmu, "VoicemailUserEntry", actionid);
+		if(ret == 0) {
+			ast_log(LOG_ERROR, "Could not append voicemail user info.");
 			continue;
 		}
-
-		astman_append(s,
-			"Event: VoicemailUserEntry\r\n"
-			"%s"
-			"VMContext: %s\r\n"
-			"VoiceMailbox: %s\r\n"
-			"Fullname: %s\r\n"
-			"Email: %s\r\n"
-			"Pager: %s\r\n"
-			"ServerEmail: %s\r\n"
-			"FromString: %s\r\n"
-			"MailCommand: %s\r\n"
-			"Language: %s\r\n"
-			"TimeZone: %s\r\n"
-			"Callback: %s\r\n"
-			"Dialout: %s\r\n"
-			"UniqueID: %s\r\n"
-			"ExitContext: %s\r\n"
-			"SayDurationMinimum: %d\r\n"
-			"SayEnvelope: %s\r\n"
-			"SayCID: %s\r\n"
-			"AttachMessage: %s\r\n"
-			"AttachmentFormat: %s\r\n"
-			"DeleteMessage: %s\r\n"
-			"VolumeGain: %.2f\r\n"
-			"CanReview: %s\r\n"
-			"CallOperator: %s\r\n"
-			"MaxMessageCount: %d\r\n"
-			"MaxMessageLength: %d\r\n"
-			"NewMessageCount: %d\r\n"
-			"OldMessageCount: %d\r\n"
-#ifdef IMAP_STORAGE
-			"IMAPUser: %s\r\n"
-			"IMAPServer: %s\r\n"
-			"IMAPPort: %s\r\n"
-			"IMAPFlags: %s\r\n"
-#endif
-			"\r\n",
-			actionid,
-			vmu->context,
-			vmu->mailbox,
-			vmu->fullname,
-			vmu->email,
-			vmu->pager,
-			ast_strlen_zero(vmu->serveremail) ? serveremail : vmu->serveremail,
-			ast_strlen_zero(vmu->fromstring) ? fromstring : vmu->fromstring,
-			mailcmd,
-			vmu->language,
-			vmu->zonetag,
-			vmu->callback,
-			vmu->dialout,
-			vmu->uniqueid,
-			vmu->exit,
-			vmu->saydurationm,
-			ast_test_flag(vmu, VM_ENVELOPE) ? "Yes" : "No",
-			ast_test_flag(vmu, VM_SAYCID) ? "Yes" : "No",
-			ast_test_flag(vmu, VM_ATTACH) ? "Yes" : "No",
-			vmu->attachfmt,
-			ast_test_flag(vmu, VM_DELETE) ? "Yes" : "No",
-			vmu->volgain,
-			ast_test_flag(vmu, VM_REVIEW) ? "Yes" : "No",
-			ast_test_flag(vmu, VM_OPERATOR) ? "Yes" : "No",
-			vmu->maxmsg,
-			vmu->maxsecs,
-			new,
-			old
-#ifdef IMAP_STORAGE
-			,
-			vmu->imapuser,
-			vmu->imapserver,
-			vmu->imapport,
-			vmu->imapflags
-#endif
-			);
 		++num_users;
 	}
 
@@ -14995,8 +15053,8 @@ static int unload_module(void)
 	res |= ast_custom_function_unregister(&mailbox_exists_acf);
 	res |= ast_custom_function_unregister(&vm_info_acf);
 	res |= ast_manager_unregister("VoicemailUsersList");
+	res |= ast_manager_unregister("VoicemailUserStatus");
 	res |= ast_manager_unregister("VoicemailRefresh");
-	res |= ast_data_unregister(NULL);
 #ifdef TEST_FRAMEWORK
 	res |= AST_TEST_UNREGISTER(test_voicemail_vmsayname);
 	res |= AST_TEST_UNREGISTER(test_voicemail_msgcount);
@@ -15048,7 +15106,9 @@ static int load_module(void)
 	my_umask = umask(0);
 	umask(my_umask);
 
-	if (!(inprocess_container = ao2_container_alloc(573, inprocess_hash_fn, inprocess_cmp_fn))) {
+	inprocess_container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 573,
+		inprocess_hash_fn, NULL, inprocess_cmp_fn);
+	if (!inprocess_container) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -15073,6 +15133,7 @@ static int load_module(void)
 	res |= ast_custom_function_register(&mailbox_exists_acf);
 	res |= ast_custom_function_register(&vm_info_acf);
 	res |= ast_manager_register_xml("VoicemailUsersList", EVENT_FLAG_CALL | EVENT_FLAG_REPORTING, manager_list_voicemail_users);
+	res |= ast_manager_register_xml("VoicemailUserStatus", EVENT_FLAG_CALL | EVENT_FLAG_REPORTING, manager_status_voicemail_user);
 	res |= ast_manager_register_xml("VoicemailRefresh", EVENT_FLAG_USER, manager_voicemail_refresh);
 #ifdef TEST_FRAMEWORK
 	res |= AST_TEST_REGISTER(test_voicemail_vmsayname);
@@ -15106,7 +15167,6 @@ static int load_module(void)
 	}
 
 	ast_cli_register_multiple(cli_voicemail, ARRAY_LEN(cli_voicemail));
-	ast_data_register_multiple(vm_data_providers, ARRAY_LEN(vm_data_providers));
 
 #ifdef TEST_FRAMEWORK
 	ast_install_vm_test_functions(vm_test_create_user, vm_test_destroy_user);
@@ -16490,9 +16550,9 @@ play2_msg_cleanup:
  */
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, tdesc,
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.nonoptreq = "res_adsi,res_smdi",
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.optional_modules = "res_adsi,res_smdi",
+);
diff -pruN 1:13.23.1~dfsg-2/apps/app_waitforring.c 1:16.1.1~dfsg-1/apps/app_waitforring.c
--- 1:13.23.1~dfsg-2/apps/app_waitforring.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_waitforring.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_waitforsilence.c 1:16.1.1~dfsg-1/apps/app_waitforsilence.c
--- 1:13.23.1~dfsg-2/apps/app_waitforsilence.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_waitforsilence.c	2018-12-26 18:45:13.000000000 +0000
@@ -44,8 +44,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_waituntil.c 1:16.1.1~dfsg-1/apps/app_waituntil.c
--- 1:13.23.1~dfsg-2/apps/app_waituntil.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_waituntil.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/apps/app_while.c 1:16.1.1~dfsg-1/apps/app_while.c
--- 1:13.23.1~dfsg-2/apps/app_while.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_while.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
@@ -117,8 +115,8 @@ static const char *get_index(struct ast_
 static struct ast_exten *find_matching_priority(struct ast_context *c, const char *exten, int priority, const char *callerid)
 {
 	struct ast_exten *e;
-	struct ast_include *i;
 	struct ast_context *c2;
+	int idx;
 
 	for (e=ast_walk_context_extensions(c, NULL); e; e=ast_walk_context_extensions(c, e)) {
 		if (ast_extension_match(ast_get_extension_name(e), exten)) {
@@ -137,7 +135,9 @@ static struct ast_exten *find_matching_p
 	}
 
 	/* No match; run through includes */
-	for (i=ast_walk_context_includes(c, NULL); i; i=ast_walk_context_includes(c, i)) {
+	for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+		const struct ast_include *i = ast_context_includes_get(c, idx);
+
 		for (c2=ast_walk_contexts(NULL); c2; c2=ast_walk_contexts(c2)) {
 			if (!strcmp(ast_get_context_name(c2), ast_get_include_name(i))) {
 				e = find_matching_priority(c2, exten, priority, callerid);
diff -pruN 1:13.23.1~dfsg-2/apps/app_zapateller.c 1:16.1.1~dfsg-1/apps/app_zapateller.c
--- 1:13.23.1~dfsg-2/apps/app_zapateller.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/app_zapateller.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/apps/confbridge/confbridge_manager.c 1:16.1.1~dfsg-1/apps/confbridge/confbridge_manager.c
--- 1:13.23.1~dfsg-2/apps/confbridge/confbridge_manager.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/confbridge/confbridge_manager.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/bridge.h"
 #include "asterisk/stasis.h"
@@ -35,6 +33,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/manager.h"
 #include "asterisk/stasis_message_router.h"
 #include "include/confbridge.h"
+#include "asterisk/message.h"
+#include "asterisk/stream.h"
 
 /*** DOCUMENTATION
 	<managerEvent language="en_US" name="ConfbridgeStart">
@@ -229,12 +229,305 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 static struct stasis_message_router *bridge_state_router;
 static struct stasis_message_router *channel_state_router;
 
+STASIS_MESSAGE_TYPE_DEFN(confbridge_start_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_end_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_join_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_leave_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_start_record_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_stop_record_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_mute_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_unmute_type);
+STASIS_MESSAGE_TYPE_DEFN(confbridge_talking_type);
+/*
+ * The welcome message is defined here but is only sent
+ * to participants and only when events are enabled.
+ * At the current time, no actual stasis or AMI events
+ * are generated for this type.
+ */
+STASIS_MESSAGE_TYPE_DEFN(confbridge_welcome_type);
+
+const char *confbridge_event_type_to_string(struct stasis_message_type *event_type)
+{
+	if (event_type == confbridge_start_type()) {
+		return "ConfbridgeStart";
+	} else if (event_type == confbridge_end_type()) {
+		return "ConfbridgeEnd";
+	} else if (event_type == confbridge_join_type()) {
+		return "ConfbridgeJoin";
+	} else if (event_type == confbridge_leave_type()) {
+		return "ConfbridgeLeave";
+	} else if (event_type == confbridge_start_record_type()) {
+		return "ConfbridgeRecord";
+	} else if (event_type == confbridge_stop_record_type()) {
+		return "ConfbridgeStopRecord";
+	} else if (event_type == confbridge_mute_type()) {
+		return "ConfbridgeMute";
+	} else if (event_type == confbridge_unmute_type()) {
+		return "ConfbridgeUnmute";
+	} else if (event_type == confbridge_talking_type()) {
+		return "ConfbridgeTalking";
+	} else if (event_type == confbridge_welcome_type()) {
+		return "ConfbridgeWelcome";
+	} else {
+		return "unknown";
+	}
+}
+
+static struct ast_json *channel_to_json(struct ast_channel_snapshot *channel_snapshot,
+	struct ast_json *conf_blob, struct ast_json *labels_blob)
+{
+	struct ast_json *json_channel = ast_channel_snapshot_to_json(channel_snapshot, NULL);
+
+	if (!json_channel) {
+		return NULL;
+	}
+
+	/* These items are removed for privacy reasons. */
+	ast_json_object_del(json_channel, "dialplan");
+	ast_json_object_del(json_channel, "connected");
+	ast_json_object_del(json_channel, "accountcode");
+
+	/* conf_blob contains flags such as talking, admin, mute, etc. */
+	if (conf_blob) {
+		struct ast_json *conf_copy = ast_json_copy(conf_blob);
+
+		if (!conf_copy) {
+			ast_json_unref(json_channel);
+			return NULL;
+		}
+		ast_json_object_del(conf_copy, "conference");
+		ast_json_object_update(json_channel, conf_copy);
+		ast_json_unref(conf_copy);
+	}
+
+	/* labels_blob contains the msid labels to correlate to streams. */
+	if (labels_blob) {
+		ast_json_object_update(json_channel, labels_blob);
+	}
+
+	return json_channel;
+}
+
+static struct ast_json *bridge_to_json(struct ast_bridge_snapshot *bridge_snapshot)
+{
+	struct ast_json *json_bridge = ast_bridge_snapshot_to_json(bridge_snapshot, NULL);
+
+	if (!json_bridge) {
+		return NULL;
+	}
+
+	/* These items have no use in the context of bridge participant info. */
+	ast_json_object_del(json_bridge, "technology");
+	ast_json_object_del(json_bridge, "bridge_type");
+	ast_json_object_del(json_bridge, "bridge_class");
+	ast_json_object_del(json_bridge, "creator");
+	ast_json_object_del(json_bridge, "channels");
+
+	return json_bridge;
+}
+
+static struct ast_json *pack_bridge_and_channels(
+	struct ast_json *json_bridge, struct ast_json *json_channels,
+	struct stasis_message * msg)
+{
+	const struct timeval *tv = stasis_message_timestamp(msg);
+	const char *msg_name = confbridge_event_type_to_string(stasis_message_type(msg));
+	const char *fmt = ast_json_typeof(json_channels) == AST_JSON_ARRAY ?
+		"{s: s, s: o, s: o, s: o }" : "{s: s, s: o, s: o, s: [ o ] }";
+
+	return ast_json_pack(fmt,
+		"type", msg_name,
+		"timestamp", ast_json_timeval(*tv, NULL),
+		"bridge", json_bridge,
+		"channels", json_channels);
+}
+
+static struct ast_json *pack_snapshots(	struct ast_bridge_snapshot *bridge_snapshot,
+	struct ast_channel_snapshot *channel_snapshot, 	struct ast_json *conf_blob,
+	struct ast_json *labels_blob, struct stasis_message * msg)
+{
+	struct ast_json *json_bridge;
+	struct ast_json *json_channel;
+
+	json_bridge = bridge_to_json(bridge_snapshot);
+	json_channel = channel_to_json(channel_snapshot, conf_blob, labels_blob);
+
+	return pack_bridge_and_channels(json_bridge, json_channel, msg);
+}
+
+static void send_message(const char *msg_name, char *conf_name, struct ast_json *json_object,
+	struct ast_channel *chan)
+{
+	struct ast_msg_data *data_msg;
+	struct ast_msg_data_attribute attrs[] = {
+		{ .type = AST_MSG_DATA_ATTR_FROM, conf_name },
+		{ .type = AST_MSG_DATA_ATTR_CONTENT_TYPE, .value = "application/x-asterisk-confbridge-event+json"},
+		{ .type = AST_MSG_DATA_ATTR_BODY, },
+	};
+	char *json;
+	int rc = 0;
+	struct ast_frame f;
+	struct ast_bridge_channel *bridge_chan;
+
+	bridge_chan = ast_channel_get_bridge_channel(chan);
+	if (!bridge_chan) {
+		/* Don't complain if we can't get the bridge_chan. The channel is probably gone. */
+		return;
+	}
+
+	json = ast_json_dump_string_format(json_object, AST_JSON_PRETTY);
+	if (!json) {
+		ast_log(LOG_ERROR, "Unable to convert json_object for %s message to string\n", msg_name);
+		return;
+	}
+	attrs[2].value = json;
+
+	data_msg = ast_msg_data_alloc(AST_MSG_DATA_SOURCE_TYPE_IN_DIALOG, attrs, ARRAY_LEN(attrs));
+	if (!data_msg) {
+		ast_log(LOG_ERROR, "Unable to create %s message for channel '%s'\n", msg_name,
+			ast_channel_name(chan));
+		ast_json_free(json);
+		return;
+	}
+
+	memset(&f, 0, sizeof(f));
+	f.frametype = AST_FRAME_TEXT_DATA;
+	f.data.ptr = data_msg;
+	f.datalen = ast_msg_data_get_length(data_msg);
+
+	rc = ast_bridge_channel_queue_frame(bridge_chan, &f);
+	ast_free(data_msg);
+	if (rc != 0) {
+		/* Don't complain if we can't send a leave message. The channel is probably gone. */
+		if (strcmp(confbridge_event_type_to_string(confbridge_leave_type()), msg_name) != 0) {
+			ast_log(LOG_ERROR, "Failed to queue %s message to '%s'\n%s\n", msg_name,
+				ast_channel_name(chan), json);
+		}
+		ast_json_free(json);
+		return;
+	}
+
+	ast_debug(3, "Queued %s message to '%s'\n%s\n", msg_name, ast_channel_name(chan), json);
+	ast_json_free(json);
+}
+
+void conf_send_event_to_participants(struct confbridge_conference *conference,
+	struct ast_channel *chan, struct stasis_message *msg)
+{
+	struct ast_bridge_blob *obj = stasis_message_data(msg);
+	struct ast_json *extras = obj->blob;
+	struct user_profile u_profile = {{0}};
+	int source_send_events = 0;
+	int source_echo_events = 0;
+	struct ast_json* json_channels = NULL;
+	struct confbridge_user *user;
+	const char *msg_name = confbridge_event_type_to_string(stasis_message_type(msg));
+
+	ast_debug(3, "Distributing %s event to participants\n", msg_name);
+
+	/* This could be a channel level event or a bridge level event */
+	if (chan) {
+		if (!conf_find_user_profile(chan, NULL, &u_profile)) {
+			ast_log(LOG_ERROR, "Unable to retrieve user profile for channel '%s'\n",
+				ast_channel_name(chan));
+			return;
+		}
+		source_send_events = ast_test_flag(&u_profile, USER_OPT_SEND_EVENTS);
+		source_echo_events = ast_test_flag(&u_profile, USER_OPT_ECHO_EVENTS);
+		ast_debug(3, "send_events: %d  echo_events: %d for profile %s\n",
+			source_send_events, source_echo_events, u_profile.name);
+	}
+
+	/* Now send a message to the participants with the json string. */
+	ao2_lock(conference);
+	AST_LIST_TRAVERSE(&conference->active_list, user, list) {
+		struct ast_json *json_object;
+
+		/*
+		 * If the msg type is join, we need to capture all targets channel info so we can
+		 * send a welcome message to the source channel with all current participants.
+		 */
+		if (source_send_events && stasis_message_type(msg) == confbridge_join_type()) {
+			struct ast_channel_snapshot *target_snapshot;
+			struct ast_json *target_json_channel;
+
+			target_snapshot = ast_channel_snapshot_get_latest(ast_channel_uniqueid(user->chan));
+			if (!target_snapshot) {
+				ast_log(LOG_ERROR, "Unable to get a channel snapshot for '%s'\n",
+					ast_channel_name(user->chan));
+				continue;
+			}
+
+			target_json_channel = channel_to_json(target_snapshot, extras, NULL);
+			ao2_ref(target_snapshot, -1);
+
+			if (!json_channels) {
+				json_channels = ast_json_array_create();
+				if (!json_channels) {
+					ast_log(LOG_ERROR, "Unable to allocate json array\n");
+					ast_json_unref(target_json_channel);
+					return;
+				}
+			}
+
+			ast_json_array_append(json_channels, target_json_channel);
+		}
+
+		/* Don't send a message to the user that triggered the event. */
+		if (!source_echo_events && user->chan == chan) {
+			ast_debug(3, "Skipping queueing %s message to '%s'. Same channel.\n", msg_name,
+				ast_channel_name(user->chan));
+			continue;
+		}
+
+		/* Don't send a message to users in profiles not sending events. */
+		if (!ast_test_flag(&user->u_profile, USER_OPT_SEND_EVENTS)) {
+			ast_debug(3, "Skipping queueing %s message to '%s'. Not receiving events.\n", msg_name,
+				ast_channel_name(user->chan));
+			continue;
+		}
+
+		json_object = pack_snapshots(obj->bridge, obj->channel, extras, NULL, msg);
+
+		if (!json_object) {
+			ast_log(LOG_ERROR, "Unable to convert %s message to json\n", msg_name);
+			continue;
+		}
+
+		send_message(msg_name, conference->name, json_object, user->chan);
+		ast_json_unref(json_object);
+	}
+	ao2_unlock(conference);
+
+	/*
+	 * If this is a join event, send the welcome message to just the joining user
+	 * if it's not audio-only or otherwise restricted.
+	 */
+	if (source_send_events && json_channels
+		&& stasis_message_type(msg) == confbridge_join_type()) {
+		struct ast_json *json_object;
+		struct ast_json *json_bridge;
+		const char *welcome_msg_name = confbridge_event_type_to_string(confbridge_welcome_type());
+
+		json_bridge = bridge_to_json(obj->bridge);
+		json_object = pack_bridge_and_channels(json_bridge, json_channels, msg);
+		if (!json_object) {
+			ast_log(LOG_ERROR, "Unable to convert ConfbridgeWelcome message to json\n");
+			return;
+		}
+		ast_json_string_set(ast_json_object_get(json_object, "type"), welcome_msg_name);
+
+		send_message(welcome_msg_name, conference->name, json_object, chan);
+		ast_json_unref(json_object);
+	}
+}
+
 static void confbridge_publish_manager_event(
 	struct stasis_message *message,
-	const char *event,
 	struct ast_str *extra_text)
 {
 	struct ast_bridge_blob *blob = stasis_message_data(message);
+	const char *event = confbridge_event_type_to_string(stasis_message_type(message));
 	const char *conference_name;
 	RAII_VAR(struct ast_str *, bridge_text, NULL, ast_free);
 	RAII_VAR(struct ast_str *, channel_text, NULL, ast_free);
@@ -251,7 +544,10 @@ static void confbridge_publish_manager_e
 	ast_assert(conference_name != NULL);
 
 	if (blob->channel) {
+		struct confbridge_conference *conference = conf_find_bridge(conference_name);
+
 		channel_text = ast_manager_build_channel_state_string(blob->channel);
+		ao2_cleanup(conference);
 	}
 
 	manager_event(EVENT_FLAG_CALL, event,
@@ -293,13 +589,13 @@ static int get_muted_header(struct ast_s
 static void confbridge_start_cb(void *data, struct stasis_subscription *sub,
 	struct stasis_message *message)
 {
-	confbridge_publish_manager_event(message, "ConfbridgeStart", NULL);
+	confbridge_publish_manager_event(message, NULL);
 }
 
 static void confbridge_end_cb(void *data, struct stasis_subscription *sub,
 	struct stasis_message *message)
 {
-	confbridge_publish_manager_event(message, "ConfbridgeEnd", NULL);
+	confbridge_publish_manager_event(message, NULL);
 }
 
 static void confbridge_leave_cb(void *data, struct stasis_subscription *sub,
@@ -308,7 +604,7 @@ static void confbridge_leave_cb(void *da
 	struct ast_str *extra_text = NULL;
 
 	if (!get_admin_header(&extra_text, message)) {
-		confbridge_publish_manager_event(message, "ConfbridgeLeave", extra_text);
+		confbridge_publish_manager_event(message, extra_text);
 	}
 	ast_free(extra_text);
 }
@@ -320,7 +616,7 @@ static void confbridge_join_cb(void *dat
 
 	if (!get_admin_header(&extra_text, message)
 		&& !get_muted_header(&extra_text, message)) {
-		confbridge_publish_manager_event(message, "ConfbridgeJoin", extra_text);
+		confbridge_publish_manager_event(message, extra_text);
 	}
 	ast_free(extra_text);
 }
@@ -328,13 +624,13 @@ static void confbridge_join_cb(void *dat
 static void confbridge_start_record_cb(void *data, struct stasis_subscription *sub,
 	struct stasis_message *message)
 {
-	confbridge_publish_manager_event(message, "ConfbridgeRecord", NULL);
+	confbridge_publish_manager_event(message, NULL);
 }
 
 static void confbridge_stop_record_cb(void *data, struct stasis_subscription *sub,
 	struct stasis_message *message)
 {
-	confbridge_publish_manager_event(message, "ConfbridgeStopRecord", NULL);
+	confbridge_publish_manager_event(message, NULL);
 }
 
 static void confbridge_mute_cb(void *data, struct stasis_subscription *sub,
@@ -343,7 +639,7 @@ static void confbridge_mute_cb(void *dat
 	struct ast_str *extra_text = NULL;
 
 	if (!get_admin_header(&extra_text, message)) {
-		confbridge_publish_manager_event(message, "ConfbridgeMute", extra_text);
+		confbridge_publish_manager_event(message, extra_text);
 	}
 	ast_free(extra_text);
 }
@@ -354,7 +650,7 @@ static void confbridge_unmute_cb(void *d
 	struct ast_str *extra_text = NULL;
 
 	if (!get_admin_header(&extra_text, message)) {
-		confbridge_publish_manager_event(message, "ConfbridgeUnmute", extra_text);
+		confbridge_publish_manager_event(message, extra_text);
 	}
 	ast_free(extra_text);
 }
@@ -375,20 +671,10 @@ static void confbridge_talking_cb(void *
 	}
 
 	if (!get_admin_header(&extra_text, message)) {
-		confbridge_publish_manager_event(message, "ConfbridgeTalking", extra_text);
+		confbridge_publish_manager_event(message, extra_text);
 	}
 }
 
-STASIS_MESSAGE_TYPE_DEFN(confbridge_start_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_end_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_join_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_leave_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_start_record_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_stop_record_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_mute_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_unmute_type);
-STASIS_MESSAGE_TYPE_DEFN(confbridge_talking_type);
-
 void manager_confbridge_shutdown(void) {
 	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_start_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_end_type);
@@ -399,6 +685,7 @@ void manager_confbridge_shutdown(void) {
 	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_mute_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_unmute_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_talking_type);
+	STASIS_MESSAGE_TYPE_CLEANUP(confbridge_welcome_type);
 
 	if (bridge_state_router) {
 		stasis_message_router_unsubscribe(bridge_state_router);
@@ -422,6 +709,7 @@ int manager_confbridge_init(void)
 	STASIS_MESSAGE_TYPE_INIT(confbridge_mute_type);
 	STASIS_MESSAGE_TYPE_INIT(confbridge_unmute_type);
 	STASIS_MESSAGE_TYPE_INIT(confbridge_talking_type);
+	STASIS_MESSAGE_TYPE_INIT(confbridge_welcome_type);
 
 	bridge_state_router = stasis_message_router_create(
 		ast_bridge_topic_all_cached());
@@ -566,5 +854,7 @@ int manager_confbridge_init(void)
 		return -1;
 	}
 
+	/* FYI: confbridge_welcome_type is never routed */
+
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/apps/confbridge/conf_chan_announce.c 1:16.1.1~dfsg-1/apps/confbridge/conf_chan_announce.c
--- 1:13.23.1~dfsg-2/apps/confbridge/conf_chan_announce.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/confbridge/conf_chan_announce.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/bridge.h"
 #include "asterisk/core_unreal.h"
@@ -103,7 +101,7 @@ static struct ast_channel *announce_requ
 	ao2_ref(pvt->bridge, +1);
 
 	chan = ast_unreal_new_channels(&pvt->base, conf_announce_get_tech(),
-		AST_STATE_UP, AST_STATE_UP, NULL, NULL, assignedids, requestor, NULL);
+		AST_STATE_UP, AST_STATE_UP, NULL, NULL, assignedids, requestor, 0);
 	if (chan) {
 		ast_answer(pvt->base.owner);
 		ast_answer(pvt->base.chan);
diff -pruN 1:13.23.1~dfsg-2/apps/confbridge/conf_chan_record.c 1:16.1.1~dfsg-1/apps/confbridge/conf_chan_record.c
--- 1:13.23.1~dfsg-2/apps/confbridge/conf_chan_record.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/confbridge/conf_chan_record.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/bridge.h"
 #include "asterisk/format_cache.h"
@@ -67,10 +65,10 @@ static struct ast_channel *rec_request(c
 	if (!capabilities) {
 		return NULL;
 	}
-	ast_format_cap_append_by_type(capabilities, AST_MEDIA_TYPE_UNKNOWN);
+	ast_format_cap_append_by_type(capabilities, AST_MEDIA_TYPE_AUDIO);
 
 	chan = ast_channel_alloc(1, AST_STATE_UP, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0,
-		"CBRec/conf-%s-uid-%08x",
+		"CBRec/%s-%08x",
 		conf_name, (unsigned) generated_seqno);
 	if (!chan) {
 		return NULL;
diff -pruN 1:13.23.1~dfsg-2/apps/confbridge/conf_config_parser.c 1:16.1.1~dfsg-1/apps/confbridge/conf_config_parser.c
--- 1:13.23.1~dfsg-2/apps/confbridge/conf_config_parser.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/confbridge/conf_config_parser.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,7 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/logger.h"
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
@@ -69,6 +68,23 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 				<configOption name="admin">
 					<synopsis>Sets if the user is an admin or not</synopsis>
 				</configOption>
+				<configOption name="send_events" default="no">
+					<synopsis>Sets if events are send to the user</synopsis>
+					<description><para>If events are enabled for this bridge and this option is
+					set, users will receive events like join, leave, talking, etc. via text
+					messages.  For users accessing the bridge via chan_pjsip, this means
+					in-dialog MESSAGE messages.  This is most useful for WebRTC participants
+					where the browser application can use the messages to alter the user
+					interface.</para></description>
+				</configOption>
+				<configOption name="echo_events" default="yes">
+					<synopsis>Sets if events are echoed back to the user that
+					triggered them</synopsis>
+					<description><para>If events are enabled for this user and this option
+					is set, the user will receive events they trigger, talking, mute, etc.
+					If not set, they will not receive their own events.
+					</para></description>
+				</configOption>
 				<configOption name="marked">
 					<synopsis>Sets if this is a marked user or not</synopsis>
 				</configOption>
@@ -278,6 +294,13 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						or 80.
 					</para></description>
 				</configOption>
+				<configOption name="binaural_active">
+					<synopsis>If true binaural conferencing with stereo audio is active</synopsis>
+					<description><para>
+						Activates binaural mixing for a conference bridge.
+						Binaural features are disabled by default.
+					</para></description>
+				</configOption>
 				<configOption name="record_conference">
 					<synopsis>Record the conference starting with the first active user's entrance and ending with the last active user's exit</synopsis>
 					<description><para>
@@ -303,7 +326,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 					</para></description>
 				</configOption>
 				<configOption name="record_file_append" default="yes">
-					<synopsis>Append record file when starting/stopping on same conference recording</synopsis>
+					<synopsis>Append to record file when starting/stopping on same conference recording</synopsis>
 					<description><para>
 						When <replaceable>record_file_append</replaceable> is set to yes, stopping and starting recording on a
 						conference adds the new portion to end of current record_file. When this is
@@ -311,6 +334,28 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						on a conference.
 					</para></description>
 				</configOption>
+				<configOption name="record_file_timestamp" default="yes">
+					<synopsis>Append the start time to the record_file name so that it is unique.</synopsis>
+					<description><para>
+						When <replaceable>record_file_timestamp</replaceable> is set to yes, the start time is appended to
+						<replaceable>record_file</replaceable> so that the filename is unique. This allows you to specify
+						a <replaceable>record_file</replaceable> but not overwrite existing recordings.
+					</para></description>
+				</configOption>
+				<configOption name="record_options" default="">
+					<synopsis>Pass additional options to MixMonitor when recording</synopsis>
+					<description><para>
+						Pass additional options to MixMonitor when <replaceable>record_conference</replaceable> is set to yes.
+						See <literal>MixMonitor</literal> for available options.
+					</para></description>
+				</configOption>
+				<configOption name="record_command" default="">
+					<synopsis>Execute a command after recording ends</synopsis>
+					<description><para>
+						Executes the specified command when recording ends. Any strings matching <literal>^{X}</literal> will be
+						unescaped to <variable>X</variable>. All variables will be evaluated at the time ConfBridge is called.
+					</para></description>
+				</configOption>
 				<configOption name="regcontext">
 					<synopsis>The name of the context into which to register the name of the conference bridge as NoOP() at priority 1</synopsis>
 					<description><para>
@@ -357,6 +402,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 								is the single source of video distribution among all participants. If
 								that user leaves, the marked user to join after them becomes the source.</para>
 							</enum>
+							<enum name="sfu">
+								<para>Selective Forwarding Unit - Sets multi-stream
+								operation for a multi-party video conference.</para>
+							</enum>
 						</enumlist>
 					</description>
 				</configOption>
@@ -391,6 +440,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 							<enum name="sound_kicked"><para>The sound played to a user who has been kicked from the conference.</para></enum>
 							<enum name="sound_muted"><para>The sound played when the mute option it toggled on.</para></enum>
 							<enum name="sound_unmuted"><para>The sound played when the mute option it toggled off.</para></enum>
+							<enum name="sound_binaural_on"><para>The sound played when binaural auudio is turned on.</para></enum>
+							<enum name="sound_binaural_off"><para>The sound played when the binaural audio is turned off.</para></enum>
 							<enum name="sound_only_person"><para>The sound played when the user is the only person in the conference.</para></enum>
 							<enum name="sound_only_one"><para>The sound played to a user when there is only one other
 										person is in the conference.</para></enum>
@@ -414,6 +465,60 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						</enumlist>
 					</description>
 				</configOption>
+				<configOption name="video_update_discard" default="2000">
+					<synopsis>Sets the amount of time in milliseconds after sending a video update to discard subsequent video updates</synopsis>
+					<description><para>
+						Sets the amount of time in milliseconds after sending a video update request
+						that subsequent video updates should be discarded. This means that if we
+						send a video update we will discard any other video update requests until
+						after the configured amount of time has elapsed. This prevents flooding of
+						video update requests from clients.
+					</para></description>
+				</configOption>
+				<configOption name="remb_send_interval" default="0">
+					<synopsis>Sets the interval in milliseconds that a combined REMB frame will be sent to video sources</synopsis>
+					<description><para>
+						Sets the interval in milliseconds that a combined REMB frame will be sent
+						to video sources. This is done by taking all REMB frames that have been
+						received since the last REMB frame was sent, making a combined value,
+						and sending it to the source. A REMB frame contains receiver estimated
+						maximum bitrate information. By creating a combined REMB frame the
+						sender of video can be influenced on the bitrate they choose, allowing
+						better quality for all receivers.
+					</para></description>
+				</configOption>
+				<configOption name="remb_behavior" default="average">
+					<synopsis>Sets how REMB reports are generated from multiple sources</synopsis>
+					<description><para>
+						Sets how REMB reports are combined from multiple sources to form one. A REMB report
+						consists of information about the receiver estimated maximum bitrate. As a source
+						stream may be forwarded to multiple receivers the reports must be combined into
+						a single one which is sent to the sender.</para>
+						<enumlist>
+							<enum name="average">
+								<para>The average of all estimated maximum bitrates is taken and sent
+								to the sender.</para>
+							</enum>
+							<enum name="lowest">
+								<para>The lowest estimated maximum bitrate is forwarded to the sender.</para>
+							</enum>
+							<enum name="highest">
+								<para>The highest estimated maximum bitrate is forwarded to the sender.</para>
+							</enum>
+						</enumlist>
+					</description>
+				</configOption>
+				<configOption name="enable_events" default="no">
+					<synopsis>Enables events for this bridge</synopsis>
+					<description><para>
+						If enabled, recipients who joined the bridge via a channel driver
+						that supports Enhanced Messaging (currently only chan_pjsip) will
+						receive in-dialog messages containing a JSON body describing the
+						event.  The Content-Type header will be
+						<literal>text/x-ast-confbridge-event</literal>.
+						This feature must also be enabled in user profiles.</para>
+					</description>
+				</configOption>
 				<configOption name="template">
 					<synopsis>When using the CONFBRIDGE dialplan function, use a bridge profile as a template for creating a new temporary profile</synopsis>
 				</configOption>
@@ -478,6 +583,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 							Toggle turning on and off mute.  Mute will make the user silent
 							to everyone else, but the user will still be able to listen in.
 							</para></enum>
+						<enum name="toggle_binaural"><para>
+							Toggle turning on and off binaural audio processing.
+							</para></enum>
 						<enum name="no_op"><para>
 							This action does nothing (No Operation). Its only real purpose exists for
 							being able to reserve a sequence in the config as a menu exit sequence.</para></enum>
@@ -882,6 +990,10 @@ static int set_sound(const char *sound_n
 		ast_string_field_set(sounds, muted, sound_file);
 	} else if (!strcasecmp(sound_name, "sound_unmuted")) {
 		ast_string_field_set(sounds, unmuted, sound_file);
+	} else if (!strcasecmp(sound_name, "sound_binaural_on")) {
+		ast_string_field_set(sounds, binauralon, sound_file);
+	} else if (!strcasecmp(sound_name, "sound_binaural_off")) {
+		ast_string_field_set(sounds, binauraloff, sound_file);
 	} else if (!strcasecmp(sound_name, "sound_there_are")) {
 		ast_string_field_set(sounds, thereare, sound_file);
 	} else if (!strcasecmp(sound_name, "sound_other_in_party")) {
@@ -1107,6 +1219,7 @@ static int add_action_to_menu_entry(stru
 	switch (id) {
 	case MENU_ACTION_NOOP:
 	case MENU_ACTION_TOGGLE_MUTE:
+	case MENU_ACTION_TOGGLE_BINAURAL:
 	case MENU_ACTION_INCREASE_LISTENING:
 	case MENU_ACTION_DECREASE_LISTENING:
 	case MENU_ACTION_INCREASE_TALKING:
@@ -1215,6 +1328,8 @@ static int add_menu_entry(struct conf_me
 		ast_copy_string(menu_entry->dtmf, dtmf, sizeof(menu_entry->dtmf));
 		if (!strcasecmp(action, "toggle_mute")) {
 			res |= add_action_to_menu_entry(menu_entry, MENU_ACTION_TOGGLE_MUTE, NULL);
+		} else if (!strcasecmp(action, "toggle_binaural")) {
+			res |= add_action_to_menu_entry(menu_entry, MENU_ACTION_TOGGLE_BINAURAL, NULL);
 		} else if (!strcasecmp(action, "no_op")) {
 			res |= add_action_to_menu_entry(menu_entry, MENU_ACTION_NOOP, NULL);
 		} else if (!strcasecmp(action, "increase_listening_volume")) {
@@ -1391,6 +1506,12 @@ static char *handle_cli_confbridge_show_
 	ast_cli(a->fd,"Admin:                   %s\n",
 		u_profile.flags & USER_OPT_ADMIN ?
 		"true" : "false");
+	ast_cli(a->fd,"Send Events:             %s\n",
+		u_profile.flags & USER_OPT_SEND_EVENTS ?
+		"true" : "false");
+	ast_cli(a->fd,"Echo Events:             %s\n",
+		u_profile.flags & USER_OPT_ECHO_EVENTS ?
+		"true" : "false");
 	ast_cli(a->fd,"Marked User:             %s\n",
 		u_profile.flags & USER_OPT_MARKEDUSER ?
 		"true" : "false");
@@ -1563,10 +1684,20 @@ static char *handle_cli_confbridge_show_
 		b_profile.flags & BRIDGE_OPT_RECORD_FILE_APPEND ?
 		"yes" : "no");
 
+	ast_cli(a->fd,"Record File Timestamp: %s\n",
+		b_profile.flags & BRIDGE_OPT_RECORD_FILE_TIMESTAMP ?
+		"yes" : "no");
+
 	ast_cli(a->fd,"Record File:          %s\n",
 		ast_strlen_zero(b_profile.rec_file) ? "Auto Generated" :
 		b_profile.rec_file);
 
+	ast_cli(a->fd,"Record Options:       %s\n",
+		b_profile.rec_options);
+
+	ast_cli(a->fd,"Record Command:       %s\n",
+		b_profile.rec_command);
+
 	if (b_profile.max_members) {
 		ast_cli(a->fd,"Max Members:          %u\n", b_profile.max_members);
 	} else {
@@ -1576,8 +1707,10 @@ static char *handle_cli_confbridge_show_
 	ast_cli(a->fd,"Registration context: %s\n", b_profile.regcontext);
 
 	switch (b_profile.flags
-		& (BRIDGE_OPT_VIDEO_SRC_LAST_MARKED | BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
-			| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER)) {
+		& (BRIDGE_OPT_VIDEO_SRC_LAST_MARKED |
+			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED |
+			BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER |
+			BRIDGE_OPT_VIDEO_SRC_SFU)) {
 	case BRIDGE_OPT_VIDEO_SRC_LAST_MARKED:
 		ast_cli(a->fd, "Video Mode:           last_marked\n");
 		break;
@@ -1587,6 +1720,9 @@ static char *handle_cli_confbridge_show_
 	case BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER:
 		ast_cli(a->fd, "Video Mode:           follow_talker\n");
 		break;
+	case BRIDGE_OPT_VIDEO_SRC_SFU:
+		ast_cli(a->fd, "Video Mode:           sfu\n");
+		break;
 	case 0:
 		ast_cli(a->fd, "Video Mode:           no video\n");
 		break;
@@ -1596,6 +1732,30 @@ static char *handle_cli_confbridge_show_
 		break;
 	}
 
+	ast_cli(a->fd,"Video Update Discard: %u\n", b_profile.video_update_discard);
+	ast_cli(a->fd,"REMB Send Interval: %u\n", b_profile.remb_send_interval);
+
+	switch (b_profile.flags
+		& (BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE | BRIDGE_OPT_REMB_BEHAVIOR_LOWEST
+			| BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST)) {
+	case BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE:
+		ast_cli(a->fd, "REMB Behavior:           average\n");
+		break;
+	case BRIDGE_OPT_REMB_BEHAVIOR_LOWEST:
+		ast_cli(a->fd, "REMB Behavior:           lowest\n");
+		break;
+	case BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST:
+		ast_cli(a->fd, "REMB Behavior:           highest\n");
+		break;
+	default:
+		ast_assert(0);
+		break;
+	}
+
+	ast_cli(a->fd,"Enable Events:             %s\n",
+		b_profile.flags & BRIDGE_OPT_ENABLE_EVENTS ?
+		"yes" : "no");
+
 	ast_cli(a->fd,"sound_only_person:    %s\n", conf_get_sound(CONF_SOUND_ONLY_PERSON, b_profile.sounds));
 	ast_cli(a->fd,"sound_only_one:       %s\n", conf_get_sound(CONF_SOUND_ONLY_ONE, b_profile.sounds));
 	ast_cli(a->fd,"sound_has_joined:     %s\n", conf_get_sound(CONF_SOUND_HAS_JOINED, b_profile.sounds));
@@ -1603,6 +1763,8 @@ static char *handle_cli_confbridge_show_
 	ast_cli(a->fd,"sound_kicked:         %s\n", conf_get_sound(CONF_SOUND_KICKED, b_profile.sounds));
 	ast_cli(a->fd,"sound_muted:          %s\n", conf_get_sound(CONF_SOUND_MUTED, b_profile.sounds));
 	ast_cli(a->fd,"sound_unmuted:        %s\n", conf_get_sound(CONF_SOUND_UNMUTED, b_profile.sounds));
+	ast_cli(a->fd,"sound_binaural_on:    %s\n", conf_get_sound(CONF_SOUND_BINAURAL_ON, b_profile.sounds));
+	ast_cli(a->fd,"sound_binaural_off:   %s\n", conf_get_sound(CONF_SOUND_BINAURAL_OFF, b_profile.sounds));
 	ast_cli(a->fd,"sound_there_are:      %s\n", conf_get_sound(CONF_SOUND_THERE_ARE, b_profile.sounds));
 	ast_cli(a->fd,"sound_other_in_party: %s\n", conf_get_sound(CONF_SOUND_OTHER_IN_PARTY, b_profile.sounds));
 	ast_cli(a->fd,"sound_place_into_conference: %s\n", conf_get_sound(CONF_SOUND_PLACE_IN_CONF, b_profile.sounds));
@@ -1731,6 +1893,9 @@ static char *handle_cli_confbridge_show_
 			case MENU_ACTION_TOGGLE_MUTE:
 				ast_cli(a->fd, "toggle_mute");
 				break;
+			case MENU_ACTION_TOGGLE_BINAURAL:
+				ast_cli(a->fd, "toggle_binaural");
+				break;
 			case MENU_ACTION_NOOP:
 				ast_cli(a->fd, "no_op");
 				break;
@@ -1822,15 +1987,21 @@ void *confbridge_cfg_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->user_profiles = ao2_container_alloc(283, user_hash_cb, user_cmp_cb))) {
+	cfg->user_profiles = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 283,
+		user_hash_cb, NULL, user_cmp_cb);
+	if (!cfg->user_profiles) {
 		goto error;
 	}
 
-	if (!(cfg->bridge_profiles = ao2_container_alloc(283, bridge_hash_cb, bridge_cmp_cb))) {
+	cfg->bridge_profiles = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 283,
+		bridge_hash_cb, NULL, bridge_cmp_cb);
+	if (!cfg->bridge_profiles) {
 		goto error;
 	}
 
-	if (!(cfg->menus = ao2_container_alloc(283, menu_hash_cb, menu_cmp_cb))) {
+	cfg->menus = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 283,
+		menu_hash_cb, NULL, menu_cmp_cb);
+	if (!cfg->menus) {
 		goto error;
 	}
 
@@ -1891,25 +2062,60 @@ static int video_mode_handler(const stru
 		ast_set_flags_to(b_profile,
 			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
 				| BRIDGE_OPT_VIDEO_SRC_LAST_MARKED
-				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER,
+				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER
+				| BRIDGE_OPT_VIDEO_SRC_SFU,
 			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED);
 	} else if (!strcasecmp(var->value, "last_marked")) {
 		ast_set_flags_to(b_profile,
 			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
 				| BRIDGE_OPT_VIDEO_SRC_LAST_MARKED
-				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER,
+				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER
+				| BRIDGE_OPT_VIDEO_SRC_SFU,
 			BRIDGE_OPT_VIDEO_SRC_LAST_MARKED);
 	} else if (!strcasecmp(var->value, "follow_talker")) {
 		ast_set_flags_to(b_profile,
 			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
 				| BRIDGE_OPT_VIDEO_SRC_LAST_MARKED
-				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER,
+				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER
+				| BRIDGE_OPT_VIDEO_SRC_SFU,
 			BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER);
+	} else if (!strcasecmp(var->value, "sfu")) {
+		ast_set_flags_to(b_profile,
+			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
+				| BRIDGE_OPT_VIDEO_SRC_LAST_MARKED
+				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER
+				| BRIDGE_OPT_VIDEO_SRC_SFU,
+			BRIDGE_OPT_VIDEO_SRC_SFU);
 	} else if (!strcasecmp(var->value, "none")) {
 		ast_clear_flag(b_profile,
 			BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED
 				| BRIDGE_OPT_VIDEO_SRC_LAST_MARKED
-				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER);
+				| BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER
+				| BRIDGE_OPT_VIDEO_SRC_SFU);
+	} else {
+		return -1;
+	}
+	return 0;
+}
+
+static int remb_behavior_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
+{
+	struct bridge_profile *b_profile = obj;
+
+	if (strcasecmp(var->name, "remb_behavior")) {
+		return -1;
+	}
+
+	ast_clear_flag(b_profile, BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE |
+		BRIDGE_OPT_REMB_BEHAVIOR_LOWEST |
+		BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST);
+
+	if (!strcasecmp(var->value, "average")) {
+		ast_set_flag(b_profile, BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE);
+	} else if (!strcasecmp(var->value, "lowest")) {
+		ast_set_flag(b_profile, BRIDGE_OPT_REMB_BEHAVIOR_LOWEST);
+	} else if (!strcasecmp(var->value, "highest")) {
+		ast_set_flag(b_profile, BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST);
 	} else {
 		return -1;
 	}
@@ -2103,6 +2309,8 @@ int conf_load_config(void)
 	/* User options */
 	aco_option_register(&cfg_info, "type", ACO_EXACT, user_types, NULL, OPT_NOOP_T, 0, 0);
 	aco_option_register(&cfg_info, "admin", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_ADMIN);
+	aco_option_register(&cfg_info, "send_events", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_SEND_EVENTS);
+	aco_option_register(&cfg_info, "echo_events", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_ECHO_EVENTS);
 	aco_option_register(&cfg_info, "marked", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_MARKEDUSER);
 	aco_option_register(&cfg_info, "startmuted", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_STARTMUTED);
 	aco_option_register(&cfg_info, "music_on_hold_when_empty", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_MUSICONHOLD);
@@ -2126,6 +2334,7 @@ int conf_load_config(void)
 	aco_option_register(&cfg_info, "dsp_talking_threshold", ACO_EXACT, user_types, __stringify(DEFAULT_TALKING_THRESHOLD), OPT_UINT_T, 0, FLDSET(struct user_profile, talking_threshold));
 	aco_option_register(&cfg_info, "jitterbuffer", ACO_EXACT, user_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct user_profile, flags), USER_OPT_JITTERBUFFER);
 	aco_option_register(&cfg_info, "timeout", ACO_EXACT, user_types, "0", OPT_UINT_T, 0, FLDSET(struct user_profile, timeout));
+
 	/* This option should only be used with the CONFBRIDGE dialplan function */
 	aco_option_register_custom(&cfg_info, "template", ACO_EXACT, user_types, NULL, user_template_handler, 0);
 
@@ -2135,15 +2344,23 @@ int conf_load_config(void)
 	aco_option_register(&cfg_info, "jitterbuffer", ACO_EXACT, bridge_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), USER_OPT_JITTERBUFFER);
 	/* "auto" will fail to parse as a uint, but we use PARSE_DEFAULT to set the value to 0 in that case, which is the value that auto resolves to */
 	aco_option_register(&cfg_info, "internal_sample_rate", ACO_EXACT, bridge_types, "0", OPT_UINT_T, PARSE_DEFAULT, FLDSET(struct bridge_profile, internal_sample_rate), 0);
+	aco_option_register(&cfg_info, "binaural_active", ACO_EXACT, bridge_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), BRIDGE_OPT_BINAURAL_ACTIVE);
 	aco_option_register_custom(&cfg_info, "mixing_interval", ACO_EXACT, bridge_types, "20", mix_interval_handler, 0);
 	aco_option_register(&cfg_info, "record_conference", ACO_EXACT, bridge_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), BRIDGE_OPT_RECORD_CONFERENCE);
 	aco_option_register_custom(&cfg_info, "video_mode", ACO_EXACT, bridge_types, NULL, video_mode_handler, 0);
 	aco_option_register(&cfg_info, "record_file_append", ACO_EXACT, bridge_types, "yes", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), BRIDGE_OPT_RECORD_FILE_APPEND);
+	aco_option_register(&cfg_info, "record_file_timestamp", ACO_EXACT, bridge_types, "yes", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), BRIDGE_OPT_RECORD_FILE_TIMESTAMP);
 	aco_option_register(&cfg_info, "max_members", ACO_EXACT, bridge_types, "0", OPT_UINT_T, 0, FLDSET(struct bridge_profile, max_members));
 	aco_option_register(&cfg_info, "record_file", ACO_EXACT, bridge_types, NULL, OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct bridge_profile, rec_file));
+	aco_option_register(&cfg_info, "record_options", ACO_EXACT, bridge_types, NULL, OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct bridge_profile, rec_options));
+	aco_option_register(&cfg_info, "record_command", ACO_EXACT, bridge_types, NULL, OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct bridge_profile, rec_command));
 	aco_option_register(&cfg_info, "regcontext", ACO_EXACT, bridge_types, NULL, OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct bridge_profile, regcontext));
 	aco_option_register(&cfg_info, "language", ACO_EXACT, bridge_types, "en", OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct bridge_profile, language));
 	aco_option_register_custom(&cfg_info, "sound_", ACO_PREFIX, bridge_types, NULL, sound_option_handler, 0);
+	aco_option_register(&cfg_info, "video_update_discard", ACO_EXACT, bridge_types, "2000", OPT_UINT_T, 0, FLDSET(struct bridge_profile, video_update_discard));
+	aco_option_register(&cfg_info, "remb_send_interval", ACO_EXACT, bridge_types, "0", OPT_UINT_T, 0, FLDSET(struct bridge_profile, remb_send_interval));
+	aco_option_register_custom(&cfg_info, "remb_behavior", ACO_EXACT, bridge_types, "average", remb_behavior_handler, 0);
+	aco_option_register(&cfg_info, "enable_events", ACO_EXACT, bridge_types, "no", OPT_BOOLFLAG_T, 1, FLDSET(struct bridge_profile, flags), BRIDGE_OPT_ENABLE_EVENTS);
 	/* This option should only be used with the CONFBRIDGE dialplan function */
 	aco_option_register_custom(&cfg_info, "template", ACO_EXACT, bridge_types, NULL, bridge_template_handler, 0);
 
diff -pruN 1:13.23.1~dfsg-2/apps/confbridge/include/confbridge.h 1:16.1.1~dfsg-1/apps/confbridge/include/confbridge.h
--- 1:13.23.1~dfsg-2/apps/confbridge/include/confbridge.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/confbridge/include/confbridge.h	2018-12-26 18:45:13.000000000 +0000
@@ -30,11 +30,11 @@
 #include "asterisk/bridge_features.h"
 #include "conf_state.h"
 
-/* Maximum length of a conference bridge name */
+/*! Maximum length of a conference bridge name */
 #define MAX_CONF_NAME AST_MAX_EXTENSION
-/* Maximum length of a conference pin */
+/*! Maximum length of a conference pin */
 #define MAX_PIN     80
-/* Maximum length of bridge/user/menu profile names */
+/*! Maximum length of bridge/user/menu profile names */
 #define MAX_PROFILE_NAME 128
 
 #define DEFAULT_USER_PROFILE "default_user"
@@ -65,6 +65,8 @@ enum user_profile_flags {
 	USER_OPT_ANNOUNCEUSERCOUNTALL = (1 << 14), /*!< Sets if the number of users should be announced to everyone. */
 	USER_OPT_JITTERBUFFER =  (1 << 15), /*!< Places a jitterbuffer on the user. */
 	USER_OPT_ANNOUNCE_JOIN_LEAVE_REVIEW = (1 << 16), /*!< modifies ANNOUNCE_JOIN_LEAVE - user reviews the recording before continuing */
+	USER_OPT_SEND_EVENTS = (1 << 17), /*!< Send text message events to users */
+	USER_OPT_ECHO_EVENTS = (1 << 18), /*!< Send events only to the admin(s) */
 };
 
 enum bridge_profile_flags {
@@ -73,6 +75,13 @@ enum bridge_profile_flags {
 	BRIDGE_OPT_VIDEO_SRC_FIRST_MARKED = (1 << 2), /*!< Set if conference should feed video of first marked user to all participants. */
 	BRIDGE_OPT_VIDEO_SRC_FOLLOW_TALKER = (1 << 3), /*!< Set if conference set the video feed to follow the loudest talker.  */
 	BRIDGE_OPT_RECORD_FILE_APPEND = (1 << 4), /*!< Set if the record file should be appended to between start/stops.  */
+	BRIDGE_OPT_RECORD_FILE_TIMESTAMP = (1 << 5), /*!< Set if the record file should have a timestamp appended */
+	BRIDGE_OPT_BINAURAL_ACTIVE = (1 << 6), /*!< Set if binaural convolution is activated */
+	BRIDGE_OPT_VIDEO_SRC_SFU = (1 << 7), /*!< Selective forwarding unit */
+	BRIDGE_OPT_REMB_BEHAVIOR_AVERAGE = (1 << 8), /*!< The average of all REMB reports is sent to the sender */
+	BRIDGE_OPT_REMB_BEHAVIOR_LOWEST = (1 << 9), /*!< The lowest estimated maximum bitrate is sent to the sender */
+	BRIDGE_OPT_REMB_BEHAVIOR_HIGHEST = (1 << 10), /*!< The highest estimated maximum bitrate is sent to the sender */
+	BRIDGE_OPT_ENABLE_EVENTS = (1 << 11), /*!< Enable sending events to participants */
 };
 
 enum conf_menu_action_id {
@@ -94,6 +103,7 @@ enum conf_menu_action_id {
 	MENU_ACTION_RELEASE_SINGLE_VIDEO_SRC,
 	MENU_ACTION_PARTICIPANT_COUNT,
 	MENU_ACTION_ADMIN_TOGGLE_MUTE_PARTICIPANTS,
+	MENU_ACTION_TOGGLE_BINAURAL,
 };
 
 /*! The conference menu action contains both
@@ -171,6 +181,8 @@ enum conf_sounds {
 	CONF_SOUND_PARTICIPANTS_MUTED,
 	CONF_SOUND_PARTICIPANTS_UNMUTED,
 	CONF_SOUND_BEGIN,
+	CONF_SOUND_BINAURAL_ON,
+	CONF_SOUND_BINAURAL_OFF,
 };
 
 struct bridge_profile_sounds {
@@ -198,6 +210,8 @@ struct bridge_profile_sounds {
 		AST_STRING_FIELD(participantsmuted);
 		AST_STRING_FIELD(participantsunmuted);
 		AST_STRING_FIELD(begin);
+		AST_STRING_FIELD(binauralon);
+		AST_STRING_FIELD(binauraloff);
 	);
 };
 
@@ -205,12 +219,16 @@ struct bridge_profile {
 	char name[MAX_PROFILE_NAME];
 	char language[MAX_LANGUAGE];		  /*!< Language used for playback_chan */
 	char rec_file[PATH_MAX];
+	char rec_options[128];
+	char rec_command[128];
 	unsigned int flags;
 	unsigned int max_members;          /*!< The maximum number of participants allowed in the conference */
 	unsigned int internal_sample_rate; /*!< The internal sample rate of the bridge. 0 when set to auto adjust mode. */
 	unsigned int mix_interval;  /*!< The internal mixing interval used by the bridge. When set to 0 the bridgewill use a default interval. */
 	struct bridge_profile_sounds *sounds;
 	char regcontext[AST_MAX_CONTEXT];
+	unsigned int video_update_discard; /*!< Amount of time after sending a video update request that subsequent requests should be discarded */
+	unsigned int remb_send_interval; /*!< Interval at which a combined REMB frame is sent to video sources */
 };
 
 /*! \brief The structure that represents a conference bridge */
@@ -611,6 +629,26 @@ struct stasis_message_type *confbridge_u
 struct stasis_message_type *confbridge_talking_type(void);
 
 /*!
+ * \since 15.5
+ * \brief get the confbridge welcome stasis message type
+ *
+ * \retval stasis message type for confbridge welcome messages if it's available
+ * \retval NULL if it isn't
+ */
+struct stasis_message_type *confbridge_welcome_type(void);
+
+/*!
+ * \since 15.5
+ * \brief Get the string representation of a confbridge stasis message type
+ *
+ * \param event_type The confbridge event type such as 'confbridge_welcome_type()'
+ *
+ * \retval The string representation of the message type
+ * \retval "unknown" if not found
+ */
+const char *confbridge_event_type_to_string(struct stasis_message_type *event_type);
+
+/*!
  * \since 12.0
  * \brief register stasis message routers to handle manager events for confbridge messages
  *
@@ -651,4 +689,28 @@ struct ast_channel_tech *conf_announce_g
  * \retval -1 on error.
  */
 int conf_announce_channel_push(struct ast_channel *ast);
+
+/*!
+ * \brief Find a confbridge by name.
+ * \since 13.22.0
+ * \since 15.5.0
+ *
+ * \param confbridge_name The name to search for
+ *
+ * \return ConfBridge (which must be unreffed) or NULL.
+ */
+struct confbridge_conference *conf_find_bridge(const char *conference_name);
+
+/*!
+ * \brief Send events to bridge participants.
+ * \since 15.7
+ * \since 16.1
+ *
+ * \param conference The conference bridge
+ * \param chan The channel triggering the action
+ * \param msg The stasis message describing the event
+ */
+void conf_send_event_to_participants(struct confbridge_conference *conference,
+	struct ast_channel *chan, struct stasis_message *msg);
+
 #endif
diff -pruN 1:13.23.1~dfsg-2/apps/Makefile 1:16.1.1~dfsg-1/apps/Makefile
--- 1:13.23.1~dfsg-2/apps/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/apps/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,7 @@ all: _all
 
 include $(ASTTOPDIR)/Makefile.moddir_rules
 
-clean::
-	rm -f confbridge/*.o confbridge/*.i confbridge/*.gcda confbridge/*.gcno
+$(call MOD_ADD_C,app_confbridge,$(wildcard confbridge/*.c))
 
 app_confbridge.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 app_meetme.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
@@ -36,9 +35,6 @@ app_minivm.o: _ASTCFLAGS+=$(AST_NO_FORMA
 app_voicemail.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 app_while.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 
-app_confbridge.so: $(subst .c,.o,$(wildcard confbridge/*.c))
-$(subst .c,.o,$(wildcard confbridge/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,app_confbridge)
-
 ifneq ($(findstring $(OSARCH), mingw32 cygwin ),)
   LIBS+= -lres_ael_share.so -lres_monitor.so -lres_speech.so
   LIBS+= -lres_smdi.so
diff -pruN 1:13.23.1~dfsg-2/asterisk-13.23.1-summary.html 1:16.1.1~dfsg-1/asterisk-13.23.1-summary.html
--- 1:13.23.1~dfsg-2/asterisk-13.23.1-summary.html	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/asterisk-13.23.1-summary.html	1970-01-01 00:00:00.000000000 +0000
@@ -1,25 +0,0 @@
-<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"><html xmlns="http://www.w3.org/1999/xhtml"><title>Release Summary - asterisk-13.23.1</title><h1 align="center"><a name="top">Release Summary</a></h1><h3 align="center">asterisk-13.23.1</h3><h3 align="center">Date: 2018-09-20</h3><h3 align="center">&lt;asteriskteam@digium.com&gt;</h3><hr><h2 align="center">Table of Contents</h2><ol>
-<li><a href="#summary">Summary</a></li>
-<li><a href="#contributors">Contributors</a></li>
-<li><a href="#closed_issues">Closed Issues</a></li>
-<li><a href="#commits">Other Changes</a></li>
-<li><a href="#diffstat">Diffstat</a></li>
-</ol><hr><a name="summary"><h2 align="center">Summary</h2></a><center><a href="#top">[Back to Top]</a></center><p>This release has been made to address one or more security vulnerabilities that have been identified. A security advisory document has been published for each vulnerability that includes additional information. Users of versions of Asterisk that are affected are strongly encouraged to review the advisories and determine what action they should take to protect their systems from these issues.</p><p>Security Advisories:</p><ul>
-<li><a href="http://downloads.asterisk.org/pub/security/AST-2018-009.html">AST-2018-009</a></li>
-</ul><p>The data in this summary reflects changes that have been made since the previous release, asterisk-13.23.0.</p><hr><a name="contributors"><h2 align="center">Contributors</h2></a><center><a href="#top">[Back to Top]</a></center><p>This table lists the people who have submitted code, those that have tested patches, as well as those that reported issues on the issue tracker that were resolved in this release. For coders, the number is how many of their patches (of any size) were committed into this release. For testers, the number is the number of times their name was listed as assisting with testing a patch. Finally, for reporters, the number is the number of issues that they reported that were affected by commits that went into this release.</p><table width="100%" border="0">
-<tr><th width="33%">Coders</th><th width="33%">Testers</th><th width="33%">Reporters</th></tr>
-<tr valign="top"><td width="33%">2 George Joseph <gjoseph@digium.com><br/>1 Sean Bright <sean.bright@gmail.com><br/></td><td width="33%"><td width="33%">1 Sean Bright <sean.bright@gmail.com><br/></td></tr>
-</table><hr><a name="closed_issues"><h2 align="center">Closed Issues</h2></a><center><a href="#top">[Back to Top]</a></center><p>This is a list of all issues from the issue tracker that were closed by changes that went into this release.</p><h3>Security</h3><h4>Category: Resources/res_http_websocket</h4><a href="https://issues.asterisk.org/jira/browse/ASTERISK-28013">ASTERISK-28013</a>: res_http_websocket: Crash when reading HTTP Upgrade requests<br/>Reported by: Sean Bright<ul>
-<li><a href="https://code.asterisk.org/code/changelog/asterisk?cs=e6b0c4d27e0392a7b4b4b6717a6d1e0ea049b550">[e6b0c4d27e]</a> Sean Bright -- AST-2018-009: Fix crash processing websocket HTTP Upgrade requests</li>
-</ul><br><hr><a name="commits"><h2 align="center">Commits Not Associated with an Issue</h2></a><center><a href="#top">[Back to Top]</a></center><p>This is a list of all changes that went into this release that did not reference a JIRA issue.</p><table width="100%" border="1">
-<tr><th>Revision</th><th>Author</th><th>Summary</th></tr>
-<tr><td><a href="https://code.asterisk.org/code/changelog/asterisk?cs=43e816ead907bd5c0fd4acf46941bdccec7acac1">43e816ead9</a></td><td>George Joseph</td><td>CI: Fix typo in testsuite git checkout</td></tr>
-<tr><td><a href="https://code.asterisk.org/code/changelog/asterisk?cs=5f777e483a950e0ae2cc1ef5c7fd34c9a99a1243">5f777e483a</a></td><td>George Joseph</td><td>CI: Use proper credentials for Security testsuite checkout</td></tr>
-</table><hr><a name="diffstat"><h2 align="center">Diffstat Results</h2></a><center><a href="#top">[Back to Top]</a></center><p>This is a summary of the changes to the source code that went into this release that was generated using the diffstat utility.</p><pre>asterisk-13.23.0-summary.html       |  276 --------------
-asterisk-13.23.0-summary.txt        |  688 -----------------------------------
-b/.version                          |    2
-b/ChangeLog                         |    4
-b/asterisk-13.23.0-rc1-summary.html |  285 ++++++++++++++
-b/asterisk-13.23.0-rc1-summary.txt  |  696 ++++++++++++++++++++++++++++++++++++
-b/res/res_http_websocket.c          |   22 -
-7 files changed, 993 insertions(+), 980 deletions(-)</pre><br></html>
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/asterisk-13.23.1-summary.txt 1:16.1.1~dfsg-1/asterisk-13.23.1-summary.txt
--- 1:13.23.1~dfsg-2/asterisk-13.23.1-summary.txt	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/asterisk-13.23.1-summary.txt	1970-01-01 00:00:00.000000000 +0000
@@ -1,111 +0,0 @@
-                                Release Summary
-
-                                asterisk-13.23.1
-
-                                Date: 2018-09-20
-
-                           <asteriskteam@digium.com>
-
-     ----------------------------------------------------------------------
-
-                               Table of Contents
-
-    1. Summary
-    2. Contributors
-    3. Closed Issues
-    4. Other Changes
-    5. Diffstat
-
-     ----------------------------------------------------------------------
-
-                                    Summary
-
-                                 [Back to Top]
-
-   This release has been made to address one or more security vulnerabilities
-   that have been identified. A security advisory document has been published
-   for each vulnerability that includes additional information. Users of
-   versions of Asterisk that are affected are strongly encouraged to review
-   the advisories and determine what action they should take to protect their
-   systems from these issues.
-
-   Security Advisories:
-
-     * AST-2018-009
-
-   The data in this summary reflects changes that have been made since the
-   previous release, asterisk-13.23.0.
-
-     ----------------------------------------------------------------------
-
-                                  Contributors
-
-                                 [Back to Top]
-
-   This table lists the people who have submitted code, those that have
-   tested patches, as well as those that reported issues on the issue tracker
-   that were resolved in this release. For coders, the number is how many of
-   their patches (of any size) were committed into this release. For testers,
-   the number is the number of times their name was listed as assisting with
-   testing a patch. Finally, for reporters, the number is the number of
-   issues that they reported that were affected by commits that went into
-   this release.
-
-   Coders                   Testers                  Reporters                
-   2 George Joseph                                   1 Sean Bright            
-   1 Sean Bright            
-
-     ----------------------------------------------------------------------
-
-                                 Closed Issues
-
-                                 [Back to Top]
-
-   This is a list of all issues from the issue tracker that were closed by
-   changes that went into this release.
-
-  Security
-
-    Category: Resources/res_http_websocket
-
-   ASTERISK-28013: res_http_websocket: Crash when reading HTTP Upgrade
-   requests
-   Reported by: Sean Bright
-     * [e6b0c4d27e] Sean Bright -- AST-2018-009: Fix crash processing
-       websocket HTTP Upgrade requests
-
-     ----------------------------------------------------------------------
-
-                      Commits Not Associated with an Issue
-
-                                 [Back to Top]
-
-   This is a list of all changes that went into this release that did not
-   reference a JIRA issue.
-
-   +------------------------------------------------------------------------+
-   | Revision   | Author        | Summary                                   |
-   |------------+---------------+-------------------------------------------|
-   | 43e816ead9 | George Joseph | CI: Fix typo in testsuite git checkout    |
-   |------------+---------------+-------------------------------------------|
-   | 5f777e483a | George Joseph | CI: Use proper credentials for Security   |
-   |            |               | testsuite checkout                        |
-   +------------------------------------------------------------------------+
-
-     ----------------------------------------------------------------------
-
-                                Diffstat Results
-
-                                 [Back to Top]
-
-   This is a summary of the changes to the source code that went into this
-   release that was generated using the diffstat utility.
-
- asterisk-13.23.0-summary.html       |  276 --------------
- asterisk-13.23.0-summary.txt        |  688 -----------------------------------
- b/.version                          |    2
- b/ChangeLog                         |    4
- b/asterisk-13.23.0-rc1-summary.html |  285 ++++++++++++++
- b/asterisk-13.23.0-rc1-summary.txt  |  696 ++++++++++++++++++++++++++++++++++++
- b/res/res_http_websocket.c          |   22 -
- 7 files changed, 993 insertions(+), 980 deletions(-)
diff -pruN 1:13.23.1~dfsg-2/asterisk-16.1.1-summary.html 1:16.1.1~dfsg-1/asterisk-16.1.1-summary.html
--- 1:13.23.1~dfsg-2/asterisk-16.1.1-summary.html	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/asterisk-16.1.1-summary.html	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,30 @@
+<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"><html xmlns="http://www.w3.org/1999/xhtml"><title>Release Summary - asterisk-16.1.1</title><h1 align="center"><a name="top">Release Summary</a></h1><h3 align="center">asterisk-16.1.1</h3><h3 align="center">Date: 2018-12-26</h3><h3 align="center">&lt;asteriskteam@digium.com&gt;</h3><hr><h2 align="center">Table of Contents</h2><ol>
+<li><a href="#summary">Summary</a></li>
+<li><a href="#contributors">Contributors</a></li>
+<li><a href="#closed_issues">Closed Issues</a></li>
+<li><a href="#diffstat">Diffstat</a></li>
+</ol><hr><a name="summary"><h2 align="center">Summary</h2></a><center><a href="#top">[Back to Top]</a></center><p>This release is a point release of an existing major version. The changes included were made to address problems that have been identified in this release series, or are minor, backwards compatible new features or improvements. Users should be able to safely upgrade to this version if this release series is already in use. Users considering upgrading from a previous version are strongly encouraged to review the UPGRADE.txt document as well as the CHANGES document for information about upgrading to this release series.</p><p>The data in this summary reflects changes that have been made since the previous release, asterisk-16.1.0.</p><hr><a name="contributors"><h2 align="center">Contributors</h2></a><center><a href="#top">[Back to Top]</a></center><p>This table lists the people who have submitted code, those that have tested patches, as well as those that reported issues on the issue tracker that were resolved in this release. For coders, the number is how many of their patches (of any size) were committed into this release. For testers, the number is the number of times their name was listed as assisting with testing a patch. Finally, for reporters, the number is the number of issues that they reported that were affected by commits that went into this release.</p><table width="100%" border="0">
+<tr><th width="33%">Coders</th><th width="33%">Testers</th><th width="33%">Reporters</th></tr>
+<tr valign="top"><td width="33%">1 George Joseph <gjoseph@digium.com><br/></td><td width="33%"><td width="33%">1 abelbeck <lonnie@abelbeck.com><br/></td></tr>
+</table><hr><a name="closed_issues"><h2 align="center">Closed Issues</h2></a><center><a href="#top">[Back to Top]</a></center><p>This is a list of all issues from the issue tracker that were closed by changes that went into this release.</p><h3>Bug</h3><h4>Category: Applications/app_voicemail</h4><a href="https://issues.asterisk.org/jira/browse/ASTERISK-28222">ASTERISK-28222</a>: Regression: MWI polling no longer works<br/>Reported by: abelbeck<ul>
+<li><a href="https://code.asterisk.org/code/changelog/asterisk?cs=227c8bb8023e79d4c4923a3b6e2f275ac942a5cb">[227c8bb802]</a> George Joseph -- Revert "stasis_cache:  Stop caching stasis subscription change messages"</li>
+</ul><br><hr><a name="diffstat"><h2 align="center">Diffstat Results</h2></a><center><a href="#top">[Back to Top]</a></center><p>This is a summary of the changes to the source code that went into this release that was generated using the diffstat utility.</p><pre>asterisk-16.1.0-summary.html                           |  620 -------
+asterisk-16.1.0-summary.txt                            | 1442 ----------------
+b/.version                                             |    2
+b/ChangeLog                                            |    4
+b/asterisk-16.1.0-rc1-summary.html                     |  625 +++++++
+b/asterisk-16.1.0-rc1-summary.txt                      | 1443 +++++++++++++++++
+b/contrib/realtime/mssql/mssql_cdr.sql                 |    3
+b/contrib/realtime/mssql/mssql_config.sql              |    3
+b/contrib/realtime/mssql/mssql_voicemail.sql           |    3
+b/contrib/realtime/mysql/mysql_cdr.sql                 |    3
+b/contrib/realtime/mysql/mysql_config.sql              |    3
+b/contrib/realtime/mysql/mysql_voicemail.sql           |    3
+b/contrib/realtime/oracle/oracle_cdr.sql               |    3
+b/contrib/realtime/oracle/oracle_config.sql            |    3
+b/contrib/realtime/oracle/oracle_voicemail.sql         |    3
+b/contrib/realtime/postgresql/postgresql_cdr.sql       |    3
+b/contrib/realtime/postgresql/postgresql_config.sql    |    3
+b/contrib/realtime/postgresql/postgresql_voicemail.sql |    3
+b/main/stasis_cache.c                                  |   21
+19 files changed, 2093 insertions(+), 2100 deletions(-)</pre><br></html>
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/asterisk-16.1.1-summary.txt 1:16.1.1~dfsg-1/asterisk-16.1.1-summary.txt
--- 1:13.23.1~dfsg-2/asterisk-16.1.1-summary.txt	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/asterisk-16.1.1-summary.txt	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,100 @@
+                                Release Summary
+
+                                asterisk-16.1.1
+
+                                Date: 2018-12-26
+
+                           <asteriskteam@digium.com>
+
+     ----------------------------------------------------------------------
+
+                               Table of Contents
+
+    1. Summary
+    2. Contributors
+    3. Closed Issues
+    4. Diffstat
+
+     ----------------------------------------------------------------------
+
+                                    Summary
+
+                                 [Back to Top]
+
+   This release is a point release of an existing major version. The changes
+   included were made to address problems that have been identified in this
+   release series, or are minor, backwards compatible new features or
+   improvements. Users should be able to safely upgrade to this version if
+   this release series is already in use. Users considering upgrading from a
+   previous version are strongly encouraged to review the UPGRADE.txt
+   document as well as the CHANGES document for information about upgrading
+   to this release series.
+
+   The data in this summary reflects changes that have been made since the
+   previous release, asterisk-16.1.0.
+
+     ----------------------------------------------------------------------
+
+                                  Contributors
+
+                                 [Back to Top]
+
+   This table lists the people who have submitted code, those that have
+   tested patches, as well as those that reported issues on the issue tracker
+   that were resolved in this release. For coders, the number is how many of
+   their patches (of any size) were committed into this release. For testers,
+   the number is the number of times their name was listed as assisting with
+   testing a patch. Finally, for reporters, the number is the number of
+   issues that they reported that were affected by commits that went into
+   this release.
+
+   Coders                   Testers                  Reporters                
+   1 George Joseph                                   1 abelbeck               
+
+     ----------------------------------------------------------------------
+
+                                 Closed Issues
+
+                                 [Back to Top]
+
+   This is a list of all issues from the issue tracker that were closed by
+   changes that went into this release.
+
+  Bug
+
+    Category: Applications/app_voicemail
+
+   ASTERISK-28222: Regression: MWI polling no longer works
+   Reported by: abelbeck
+     * [227c8bb802] George Joseph -- Revert "stasis_cache: Stop caching
+       stasis subscription change messages"
+
+     ----------------------------------------------------------------------
+
+                                Diffstat Results
+
+                                 [Back to Top]
+
+   This is a summary of the changes to the source code that went into this
+   release that was generated using the diffstat utility.
+
+ asterisk-16.1.0-summary.html                           |  620 -------
+ asterisk-16.1.0-summary.txt                            | 1442 ----------------
+ b/.version                                             |    2
+ b/ChangeLog                                            |    4
+ b/asterisk-16.1.0-rc1-summary.html                     |  625 +++++++
+ b/asterisk-16.1.0-rc1-summary.txt                      | 1443 +++++++++++++++++
+ b/contrib/realtime/mssql/mssql_cdr.sql                 |    3
+ b/contrib/realtime/mssql/mssql_config.sql              |    3
+ b/contrib/realtime/mssql/mssql_voicemail.sql           |    3
+ b/contrib/realtime/mysql/mysql_cdr.sql                 |    3
+ b/contrib/realtime/mysql/mysql_config.sql              |    3
+ b/contrib/realtime/mysql/mysql_voicemail.sql           |    3
+ b/contrib/realtime/oracle/oracle_cdr.sql               |    3
+ b/contrib/realtime/oracle/oracle_config.sql            |    3
+ b/contrib/realtime/oracle/oracle_voicemail.sql         |    3
+ b/contrib/realtime/postgresql/postgresql_cdr.sql       |    3
+ b/contrib/realtime/postgresql/postgresql_config.sql    |    3
+ b/contrib/realtime/postgresql/postgresql_voicemail.sql |    3
+ b/main/stasis_cache.c                                  |   21
+ 19 files changed, 2093 insertions(+), 2100 deletions(-)
diff -pruN 1:13.23.1~dfsg-2/autoconf/ast_ext_lib.m4 1:16.1.1~dfsg-1/autoconf/ast_ext_lib.m4
--- 1:13.23.1~dfsg-2/autoconf/ast_ext_lib.m4	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/autoconf/ast_ext_lib.m4	2018-12-26 18:45:13.000000000 +0000
@@ -203,3 +203,105 @@ if test "x${PBX_$1}" = "x1"; then
    LIBS="${ast_ext_lib_check_shared_saved_libs}"
 fi
 ])
+
+# Check for existence of a given package ($1), either looking up a function
+# in a library, or, if no function is supplied, only check for the
+# existence of the header files.  Then compile, link and run the supplied
+# code fragment to make the final determination.
+
+# AST_EXT_LIB_EXTRA_CHECK([package], [library], [function], [header],
+#	 [extra libs], [extra cflags], [AC_LANG_PROGRAM(extra check code...)],
+#    ["checking for" display string], ["HAVE_package_" extra variable to set])
+AC_DEFUN([AST_EXT_LIB_EXTRA_CHECK],
+[
+if test "x${PBX_$1}" != "x1" -a "${USE_$1}" != "no"; then
+   pbxlibdir=""
+   # if --with-$1=DIR has been specified, use it.
+   if test "x${$1_DIR}" != "x"; then
+      if test -d ${$1_DIR}/lib; then
+         pbxlibdir="-L${$1_DIR}/lib"
+      else
+         pbxlibdir="-L${$1_DIR}"
+      fi
+   fi
+   m4_ifval([$3], [
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} $6"
+      AC_CHECK_LIB([$2], [$3], [AST_$1_FOUND=yes], [AST_$1_FOUND=no], [${pbxlibdir} $5])
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+   ], [
+      # empty lib, assume only headers
+      AST_$1_FOUND=yes
+   ])
+
+   # now check for the header.
+   if test "${AST_$1_FOUND}" = "yes"; then
+      $1_LIB="${pbxlibdir} -l$2 $5"
+      # if --with-$1=DIR has been specified, use it.
+      if test "x${$1_DIR}" != "x"; then
+         $1_INCLUDE="-I${$1_DIR}/include"
+      fi
+      $1_INCLUDE="${$1_INCLUDE} $6"
+      m4_ifval([$4], [
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${$1_INCLUDE}"
+         AC_CHECK_HEADER([$4], [$1_HEADER_FOUND=1], [$1_HEADER_FOUND=0])
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+      ], [
+         # no header, assume found
+         $1_HEADER_FOUND="1"
+      ])
+   fi
+   # Validate the package with the supplied code.
+   if test "x${$1_HEADER_FOUND}" = "x1" ; then
+      AC_MSG_CHECKING(for $8)
+      ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+      CPPFLAGS="${CPPFLAGS} ${$1_INCLUDE}"
+      ast_ext_lib_check_saved_LIBS="${LIBS}"
+      LIBS="${$1_LIB}"
+      AC_LINK_IFELSE(
+         [$7],
+         [
+            if test "x${cross_compiling}" = "xyes" ; then
+               $1_VALIDATED="1"
+               AC_MSG_RESULT([yes (guessed for cross-compile)])
+            else
+               ./conftest$EXEEXT
+               if test $? -eq 0 ; then
+                  $1_VALIDATED="1"
+                  AC_MSG_RESULT(yes)
+               else
+                  AC_MSG_RESULT(no)
+               fi
+            fi
+         ],
+         [
+            AC_MSG_RESULT(no)
+         ]
+      )
+      CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+      LIBS="${ast_ext_lib_check_saved_LIBS}"
+   fi
+
+   if test "x${$1_VALIDATED}" = "x1" ; then
+      m4_ifval([$3], [], [
+         # only checking headers -> no library
+         $1_LIB=""
+      ])
+      PBX_$1=1
+      cat >>confdefs.h <<_ACEOF
+[@%:@define] HAVE_$1 1
+_ACEOF
+      m4_ifval([$9], [
+      cat >>confdefs.h <<_ACEOF
+[@%:@define] HAVE_$1_$9 1
+_ACEOF
+            ])
+   else
+      $1_LIB=""
+      $1_INCLUDE=""
+   fi
+fi
+m4_ifval([$9], [AH_TEMPLATE(m4_bpatsubst([[HAVE_$1_$9]], [(.*)]), [Define if $8])])
+])
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_builtin_features.c 1:16.1.1~dfsg-1/bridges/bridge_builtin_features.c
--- 1:13.23.1~dfsg-2/bridges/bridge_builtin_features.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_builtin_features.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -521,4 +519,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Built in bridging features");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Built in bridging features",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.optional_modules = "res_monitor",
+);
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_builtin_interval_features.c 1:16.1.1~dfsg-1/bridges/bridge_builtin_interval_features.c
--- 1:13.23.1~dfsg-2/bridges/bridge_builtin_interval_features.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_builtin_interval_features.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_holding.c 1:16.1.1~dfsg-1/bridges/bridge_holding.c
--- 1:13.23.1~dfsg-2/bridges/bridge_holding.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_holding.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_native_rtp.c 1:16.1.1~dfsg-1/bridges/bridge_native_rtp.c
--- 1:13.23.1~dfsg-2/bridges/bridge_native_rtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_native_rtp.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -705,6 +703,18 @@ static int native_rtp_bridge_compatible_
 		return 0;
 	}
 
+	if (glue0->audio.instance && glue1->audio.instance) {
+		unsigned int framing_inst0, framing_inst1;
+		framing_inst0 = ast_rtp_codecs_get_framing(ast_rtp_instance_get_codecs(glue0->audio.instance));
+		framing_inst1 = ast_rtp_codecs_get_framing(ast_rtp_instance_get_codecs(glue1->audio.instance));
+		if (framing_inst0 != framing_inst1) {
+			/* ptimes are asymmetric on the two call legs so we can't use the native bridge */
+			ast_debug(1, "Asymmetric ptimes on the two call legs (%u != %u). Cannot native bridge in RTP\n",
+				framing_inst0, framing_inst1);
+			return 0;
+		}
+	}
+
 	read_ptime0 = ast_format_cap_get_format_framing(cap0, ast_channel_rawreadformat(bc0->chan));
 	read_ptime1 = ast_format_cap_get_format_framing(cap1, ast_channel_rawreadformat(bc1->chan));
 	write_ptime0 = ast_format_cap_get_format_framing(cap0, ast_channel_rawwriteformat(bc0->chan));
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_simple.c 1:16.1.1~dfsg-1/bridges/bridge_simple.c
--- 1:13.23.1~dfsg-2/bridges/bridge_simple.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_simple.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,6 +42,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/bridge.h"
 #include "asterisk/bridge_technology.h"
 #include "asterisk/frame.h"
+#include "asterisk/stream.h"
+
+static void simple_bridge_stream_topology_changed(struct ast_bridge *bridge,
+		struct ast_bridge_channel *bridge_channel);
 
 static int simple_bridge_join(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel)
 {
@@ -58,7 +60,13 @@ static int simple_bridge_join(struct ast
 		return 0;
 	}
 
-	return ast_channel_make_compatible(c0, c1);
+	if (ast_channel_make_compatible(c0, c1)) {
+		return -1;
+	}
+
+	/* Align stream topologies */
+	simple_bridge_stream_topology_changed(bridge, NULL);
+	return 0;
 }
 
 static int simple_bridge_write(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel, struct ast_frame *frame)
@@ -102,8 +110,102 @@ static struct ast_bridge_technology simp
 	.preference = AST_BRIDGE_PREFERENCE_BASE_1TO1MIX,
 	.join = simple_bridge_join,
 	.write = simple_bridge_write,
+	.stream_topology_changed = simple_bridge_stream_topology_changed,
 };
 
+static struct ast_stream_topology *simple_bridge_request_stream_topology_update(
+	struct ast_stream_topology *existing_topology,
+	struct ast_stream_topology *requested_topology)
+{
+	struct ast_stream *stream;
+	struct ast_format_cap *audio_formats = NULL;
+	struct ast_stream_topology *new_topology;
+	int i;
+
+	new_topology = ast_stream_topology_clone(requested_topology);
+	if (!new_topology) {
+		return NULL;
+	}
+
+	/* We find an existing stream with negotiated audio formats that we can place into
+	 * any audio streams in the new topology to ensure that negotiation succeeds. Some
+	 * endpoints incorrectly terminate the call if SDP negotiation fails.
+	 */
+	for (i = 0; i < ast_stream_topology_get_count(existing_topology); ++i) {
+		stream = ast_stream_topology_get_stream(existing_topology, i);
+
+		if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO ||
+			ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+			continue;
+		}
+
+		audio_formats = ast_stream_get_formats(stream);
+		break;
+	}
+
+	if (audio_formats) {
+		for (i = 0; i < ast_stream_topology_get_count(new_topology); ++i) {
+			stream = ast_stream_topology_get_stream(new_topology, i);
+
+			if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO ||
+				ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+				continue;
+			}
+
+			ast_format_cap_append_from_cap(ast_stream_get_formats(stream), audio_formats,
+				AST_MEDIA_TYPE_AUDIO);
+		}
+	}
+
+	return new_topology;
+}
+
+static void simple_bridge_stream_topology_changed(struct ast_bridge *bridge,
+		struct ast_bridge_channel *bridge_channel)
+{
+	struct ast_channel *req_chan;
+	struct ast_channel *existing_chan;
+	struct ast_stream_topology *req_top;
+	struct ast_stream_topology *existing_top;
+	struct ast_stream_topology *new_top;
+
+	if (bridge_channel) {
+		ast_bridge_channel_stream_map(bridge_channel);
+
+		if (ast_channel_get_stream_topology_change_source(bridge_channel->chan)
+			== &simple_bridge) {
+			return;
+		}
+	}
+
+	req_chan = AST_LIST_FIRST(&bridge->channels)->chan;
+	existing_chan = AST_LIST_LAST(&bridge->channels)->chan;
+	if (req_chan == existing_chan) {
+		/* Wait until both channels are in the bridge to align topologies. */
+		return;
+	}
+
+	/* Align topologies according to size or first channel to join */
+	ast_channel_lock_both(req_chan, existing_chan);
+	req_top = ast_channel_get_stream_topology(req_chan);
+	existing_top = ast_channel_get_stream_topology(existing_chan);
+	if (ast_stream_topology_get_count(req_top) < ast_stream_topology_get_count(existing_top)) {
+		SWAP(req_top, existing_top);
+		SWAP(req_chan, existing_chan);
+	}
+	new_top = simple_bridge_request_stream_topology_update(existing_top, req_top);
+	ast_channel_unlock(req_chan);
+	ast_channel_unlock(existing_chan);
+
+	if (!new_top) {
+		/* Failure.  We'll just have to live with the current topology. */
+		return;
+	}
+
+	ast_channel_request_stream_topology_change(existing_chan, new_top, &simple_bridge);
+	ast_stream_topology_free(new_top);
+}
+
 static int unload_module(void)
 {
 	ast_bridge_technology_unregister(&simple_bridge);
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_softmix/bridge_softmix_binaural.c 1:16.1.1~dfsg-1/bridges/bridge_softmix/bridge_softmix_binaural.c
--- 1:13.23.1~dfsg-2/bridges/bridge_softmix/bridge_softmix_binaural.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_softmix/bridge_softmix_binaural.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,580 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Frank Haase, Dennis Guse
+ *
+ * Frank Haase <fra.haase@gmail.com>
+ * Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Multi-party software based binaural mixing
+ *
+ * \author Frank Haase <fra.haase@googlemail.com>
+ * \author Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * \ingroup bridges
+ */
+
+#include "include/bridge_softmix_internal.h"
+
+#ifdef BINAURAL_RENDERING
+  #include "include/hrirs_configuration.h"
+#endif
+
+/*! The number of prealloced channels when a bridge will be created. */
+#define CONVOLVE_CHANNEL_PREALLOC 3
+/*! Max size of the convolve buffer. */
+#define CONVOLVE_MAX_BUFFER 4096
+/*! The default sample size in an binaural environment with a two-channel
+ * codec at 48kHz is 960 samples.
+ */
+#define CONVOLUTION_SAMPLE_SIZE 960
+
+#ifdef BINAURAL_RENDERING
+  #if SOFTMIX_BINAURAL_SAMPLE_RATE != HRIRS_SAMPLE_RATE
+	  #error HRIRs are required to be SOFTMIX_BINAURAL_SAMPLE_RATE Hz. Please adjust hrirs.h accordingly.
+	#endif
+  #if CONVOLUTION_SAMPLE_SIZE < HRIRS_IMPULSE_LEN
+	  #error HRIRS_IMPULSE_LEN cannot be longer than CONVOLUTION_SAMPLE_SIZE. Please adjust hrirs.h accordingly.
+	#endif
+#endif
+
+void reset_channel_pair(struct convolve_channel_pair *channel_pair,
+		unsigned int default_sample_size)
+{
+	memset(channel_pair->chan_left.overlap_add, 0, sizeof(float) * default_sample_size);
+	memset(channel_pair->chan_right.overlap_add, 0, sizeof(float) * default_sample_size);
+}
+
+void random_binaural_pos_change(struct softmix_bridge_data *softmix_data)
+{
+	/*
+	 * We perform a shuffle of all channels, even the ones that aren't used at the
+	 * moment of shuffling now. This has the efect that new members will be placed
+	 * randomly too.
+	 */
+	unsigned int i;
+	unsigned int j;
+	struct convolve_channel_pair *tmp;
+
+	if (softmix_data->convolve.chan_size < 2) {
+		return;
+	}
+
+	srand(time(NULL));
+	for (i = softmix_data->convolve.chan_size - 1; i > 0; i--) {
+		j = rand() % (i + 1);
+		tmp = softmix_data->convolve.cchan_pair[i];
+		reset_channel_pair(tmp, softmix_data->default_sample_size);
+		softmix_data->convolve.cchan_pair[i] = softmix_data->convolve.cchan_pair[j];
+		softmix_data->convolve.cchan_pair[j] = tmp;
+	}
+}
+
+int do_convolve(struct convolve_channel *chan, int16_t *in_samples,
+		unsigned int in_sample_size, unsigned int hrtf_length)
+{
+#ifdef BINAURAL_RENDERING
+	unsigned int i;
+
+	if (in_sample_size != CONVOLUTION_SAMPLE_SIZE) {
+		return -1;
+	}
+
+	/* FFT setting real part */
+	for (i = 0; i < CONVOLUTION_SAMPLE_SIZE; i++) {
+		chan->fftw_in[i] = in_samples[i] * (FLT_MAX / SHRT_MAX);
+	}
+
+	for (i = CONVOLUTION_SAMPLE_SIZE; i < hrtf_length; i++) {
+		chan->fftw_in[i] = 0;
+	}
+	fftw_execute(chan->fftw_plan);
+
+	/* Imaginary mulitplication (frequency space). */
+	/* First FFTW result has never an imaginary part. */
+	chan->fftw_in[0] = chan->fftw_out[0] * chan->hrtf[0];
+	for (i = 1; i < (hrtf_length / 2); i++) {
+		/* Real part */
+		chan->fftw_in[i] = (chan->fftw_out[i] * chan->hrtf[i]) -
+				(chan->fftw_out[hrtf_length - i] * chan->hrtf[hrtf_length - i]);
+		/* Imaginary part */
+		chan->fftw_in[hrtf_length - i] = (chan->fftw_out[i] * chan->hrtf[hrtf_length - i]) +
+				(chan->fftw_out[hrtf_length - i] * chan->hrtf[i]);
+	}
+
+	/* The last (if even) FFTW result has never an imaginary part. */
+	if (hrtf_length % 2 == 0) {
+		chan->fftw_in[hrtf_length / 2] = chan->fftw_out[hrtf_length / 2] *
+				chan->hrtf[hrtf_length / 2];
+	}
+
+	/* iFFT */
+	fftw_execute(chan->fftw_plan_inverse);
+	/* Remove signal increase due to iFFT. */
+	for (i = 0; i < hrtf_length; i++) {
+		chan->fftw_out[i] = chan->fftw_out[i] / (hrtf_length / 2);
+	}
+
+	/* Save the block for overlapp add in the next itteration. */
+	for (i = 0; i < in_sample_size; i++) {
+		chan->overlap_add[i] += chan->fftw_out[i];
+	}
+
+	/* Copy real part to the output, ignore the complex part. */
+	for (i = 0; i < in_sample_size; i++) {
+		chan->out_data[i] = chan->overlap_add[i] * (SHRT_MAX / FLT_MAX);
+		chan->overlap_add[i] = chan->fftw_out[i + in_sample_size];
+	}
+#endif
+	return 0;
+}
+
+struct convolve_channel_pair *do_convolve_pair(struct convolve_data *data,
+		unsigned int pos_id, int16_t *in_samples, unsigned int in_sample_size,
+		const char *channel_name)
+{
+	struct convolve_channel_pair *chan_pair;
+
+	/* If a position has no active member we will not convolve. */
+	if (data->pos_ids[pos_id] != 1) {
+		ast_log(LOG_ERROR, "Channel %s: Channel pair has no active member! (pos id = %d)\n",
+				channel_name, pos_id);
+		return NULL;
+	}
+
+	chan_pair = data->cchan_pair[pos_id];
+	if (do_convolve(&chan_pair->chan_left, in_samples, in_sample_size, data->hrtf_length)) {
+		ast_log(LOG_ERROR, "Channel %s: Binaural processing failed.", channel_name);
+		return NULL;
+	}
+
+	if (do_convolve(&chan_pair->chan_right, in_samples, in_sample_size, data->hrtf_length)) {
+		ast_log(LOG_ERROR, "Channel %s: Binaural processing failed.", channel_name);
+		return NULL;
+	}
+
+	return chan_pair;
+}
+
+float *get_hrir(unsigned int chan_pos, unsigned int chan_side)
+{
+#ifdef BINAURAL_RENDERING
+	if (chan_side == HRIRS_CHANNEL_LEFT) {
+		return hrirs_left[ast_binaural_positions[chan_pos]];
+	} else if (chan_side == HRIRS_CHANNEL_RIGHT) {
+		return hrirs_right[ast_binaural_positions[chan_pos]];
+	}
+#else
+	ast_log(LOG_ERROR, "Requesting data for the binaural conference feature without "
+			"it beeing active.\n");
+#endif
+
+	return NULL;
+}
+
+int init_convolve_channel(struct convolve_channel *channel, unsigned int hrtf_len,
+		unsigned int chan_pos, unsigned int chan_side, unsigned int default_sample_size)
+{
+#ifdef BINAURAL_RENDERING
+	unsigned int j;
+	float *hrir;
+
+	/* Prepare FFTW. */
+	channel->fftw_in = (double *) fftw_malloc(sizeof(double) * (hrtf_len + 1));
+	if (channel->fftw_in == NULL) {
+		return -1;
+	}
+
+	channel->fftw_out = (double *) fftw_malloc(sizeof(double) * (hrtf_len + 1));
+	if (channel->fftw_out == NULL) {
+		fftw_free(channel->fftw_in);
+		return -1;
+	}
+
+	memset(channel->fftw_in, 0, sizeof(double) * (hrtf_len + 1));
+	memset(channel->fftw_out, 0, sizeof(double) * (hrtf_len + 1));
+
+	channel->fftw_plan = fftw_plan_r2r_1d(hrtf_len, channel->fftw_in, channel->fftw_out,
+			FFTW_R2HC, FFTW_PATIENT);
+	channel->fftw_plan_inverse = fftw_plan_r2r_1d(hrtf_len, channel->fftw_in, channel->fftw_out,
+			FFTW_HC2R, FFTW_PATIENT);
+	channel->out_data = ast_calloc(CONVOLVE_MAX_BUFFER, sizeof(int16_t));
+	if (channel->out_data == NULL) {
+		fftw_free(channel->fftw_in);
+		fftw_free(channel->fftw_out);
+		return -1;
+	}
+
+	/* Reuse positions if all positions are already used. */
+	chan_pos = chan_pos % HRIRS_IMPULSE_SIZE;
+
+	/* Get HRTF for the channels spatial position. */
+	hrir = get_hrir(chan_pos, chan_side);
+	if (hrir == NULL) {
+		fftw_free(channel->fftw_in);
+		fftw_free(channel->fftw_out);
+		ast_free(channel->out_data);
+		return -1;
+	}
+
+	for (j = 0; j < HRIRS_IMPULSE_LEN; j++) {
+		channel->fftw_in[j] = hrir[j];
+	}
+
+	for (j = HRIRS_IMPULSE_LEN; j < hrtf_len; j++) {
+		channel->fftw_in[j] = 0;
+	}
+
+	fftw_execute(channel->fftw_plan);
+	channel->hrtf = (double *) fftw_malloc(sizeof(double) * hrtf_len);
+	if (channel->hrtf == NULL) {
+		fftw_free(channel->fftw_in);
+		fftw_free(channel->fftw_out);
+		ast_free(channel->out_data);
+		return -1;
+	}
+
+	for (j = 0; j < hrtf_len; j++) {
+		channel->hrtf[j] = channel->fftw_out[j];
+	}
+	channel->overlap_add = ast_calloc(default_sample_size, sizeof(float));
+
+	return 0;
+#endif
+	return -1;
+}
+
+int init_convolve_channel_pair(struct convolve_channel_pair *cchan_pair,
+		unsigned int hrtf_len, unsigned int chan_pos, unsigned int default_sample_size)
+{
+#ifdef BINAURAL_RENDERING
+	unsigned int hrirs_pos = chan_pos * 2;
+	int success = 0;
+
+	ast_debug(3, "Binaural pos for the new channel pair will be L: %d R: %d (pos id = %d)\n",
+			hrirs_pos, hrirs_pos + 1, chan_pos);
+	success = init_convolve_channel(&cchan_pair->chan_left, hrtf_len, chan_pos, HRIRS_CHANNEL_LEFT,
+			default_sample_size);
+	if (success == -1) {
+		return success;
+	}
+
+	success = init_convolve_channel(&cchan_pair->chan_right, hrtf_len, chan_pos,
+			HRIRS_CHANNEL_RIGHT, default_sample_size);
+	if (success == -1) {
+		free_convolve_channel(&cchan_pair->chan_left);
+	}
+
+	return success;
+#else
+	ast_log(LOG_ERROR, "Requesting data for the binaural conference feature "
+			"without it beeing active.\n");
+
+	return -1;
+#endif
+}
+
+int init_convolve_data(struct convolve_data *data, unsigned int default_sample_size)
+{
+	unsigned int i;
+	unsigned int j;
+	int success;
+	success = 0;
+
+	data->pos_ids = ast_calloc(sizeof(int), sizeof(int) * CONVOLVE_CHANNEL_PREALLOC);
+	if (data->pos_ids == NULL) {
+		return -1;
+	}
+	data->chan_size = CONVOLVE_CHANNEL_PREALLOC;
+	data->number_channels = 0;
+	data->cchan_pair = ast_malloc(sizeof(struct convolve_channel_pair *) *
+			CONVOLVE_CHANNEL_PREALLOC);
+	if (data->cchan_pair == NULL) {
+		ast_free(data->pos_ids);
+		return -1;
+	}
+
+	for (i = 0; i < CONVOLVE_CHANNEL_PREALLOC; i++) {
+		data->cchan_pair[i] = ast_malloc(sizeof(struct convolve_channel_pair));
+		if (data->cchan_pair[i] == NULL) {
+			ast_free(data->pos_ids);
+			for (j = 0; j < i; j++) {
+				ast_free(data->cchan_pair[j]);
+			}
+			ast_free(data->cchan_pair);
+			return -1;
+		}
+	}
+
+	data->hrtf_length = (default_sample_size * 2) - 1;
+	for (i = 0; i < CONVOLVE_CHANNEL_PREALLOC; i++) {
+		success = init_convolve_channel_pair(data->cchan_pair[i], data->hrtf_length, i,
+				default_sample_size);
+		if (success == -1) {
+			ast_free(data->pos_ids);
+			for (j = 0; j < i; j++) {
+				free_convolve_channel_pair(data->cchan_pair[j]);
+			}
+			for (j = 0; j < CONVOLVE_CHANNEL_PREALLOC; j++) {
+				ast_free(data->cchan_pair[j]);
+			}
+			return -1;
+		}
+	}
+
+	return success;
+}
+
+void free_convolve_channel(struct convolve_channel *cchan)
+{
+#ifdef BINAURAL_RENDERING
+	fftw_free(cchan->fftw_out);
+	fftw_free(cchan->fftw_in);
+	fftw_free(cchan->hrtf);
+	ast_free(cchan->overlap_add);
+	ast_free(cchan->out_data);
+	fftw_destroy_plan(cchan->fftw_plan);
+	fftw_destroy_plan(cchan->fftw_plan_inverse);
+#endif
+}
+
+void free_convolve_channel_pair(struct convolve_channel_pair *cchan_pair)
+{
+	free_convolve_channel(&cchan_pair->chan_left);
+	free_convolve_channel(&cchan_pair->chan_right);
+}
+
+void free_convolve_data(struct convolve_data *data)
+{
+	unsigned int i;
+	ast_free(data->pos_ids);
+	for (i = 0; i < data->chan_size; i++) {
+		free_convolve_channel_pair(data->cchan_pair[i]);
+		ast_free(data->cchan_pair[i]);
+	}
+	ast_free(data->cchan_pair);
+}
+
+int set_binaural_data_join(struct convolve_data *data, unsigned int default_sample_size)
+{
+	struct convolve_channel_pair **cchan_pair_tmp;
+	unsigned int i;
+	int *pos_ids_tmp;
+
+	/* Raise the number of input channels. */
+	data->number_channels++;
+	/* We realloc another channel pair if we are out of prealloced ones. */
+	/* We have prealloced one at the beginning of a conference and if a member leaves. */
+	if (data->chan_size < data->number_channels)  {
+		data->chan_size += 1;
+
+		pos_ids_tmp = ast_realloc(data->pos_ids, data->chan_size * sizeof(int));
+		if (pos_ids_tmp) {
+			data->pos_ids = pos_ids_tmp;
+		} else {
+			goto binaural_join_fails;
+		}
+
+		data->pos_ids[data->chan_size - 1] = 0;
+		cchan_pair_tmp = ast_realloc(data->cchan_pair,
+				data->chan_size * sizeof(struct convolve_channel_pair *));
+		if (cchan_pair_tmp) {
+			data->cchan_pair = cchan_pair_tmp;
+		} else {
+			goto binaural_join_fails;
+		}
+
+		data->cchan_pair[data->chan_size - 1] = ast_malloc(sizeof(struct convolve_channel_pair));
+		if (data->cchan_pair[data->chan_size - 1] == NULL) {
+			goto binaural_join_fails;
+		}
+
+		i = init_convolve_channel_pair(data->cchan_pair[data->chan_size - 1], data->hrtf_length,
+				data->chan_size - 1, default_sample_size);
+		if (i == -1) {
+			goto binaural_join_fails;
+		}
+	}
+
+	for (i = 0; i < data->chan_size; i++) {
+		if (data->pos_ids[i] == 0) {
+			data->pos_ids[i] = 1;
+			break;
+		}
+	}
+
+	return i;
+
+binaural_join_fails:
+	data->number_channels--;
+	data->chan_size -= 1;
+
+	return -1;
+}
+
+void set_binaural_data_leave(struct convolve_data *data, unsigned int pos,
+		unsigned int default_sample_size)
+{
+	if (pos >= data->chan_size || data->pos_ids[pos] == 0) {
+		return;
+	}
+
+	reset_channel_pair(data->cchan_pair[pos], default_sample_size);
+	data->number_channels--;
+	data->pos_ids[pos] = 0;
+}
+
+void softmix_process_write_binaural_audio(struct softmix_channel *sc,
+		unsigned int default_sample_size)
+{
+	unsigned int i;
+
+	if (sc->write_frame.samples % default_sample_size != 0) {
+		return;
+	}
+
+	/* If binaural is suspended, the source audio (mono) will be removed. */
+	if (sc->binaural_suspended) {
+		for (i = 0; i < default_sample_size; i++) {
+			ast_slinear_saturated_subtract(&sc->final_buf[i * 2], &sc->our_buf[i]);
+			ast_slinear_saturated_subtract(&sc->final_buf[(i * 2) + 1], &sc->our_buf[i]);
+		}
+		return;
+	}
+
+  /* If binaural is NOT suspended, the source audio (binaural) will be removed. */
+	for (i = 0; i < default_sample_size; i++) {
+		ast_slinear_saturated_subtract(&sc->final_buf[i * 2],
+				&sc->our_chan_pair->chan_left.out_data[i]);
+		ast_slinear_saturated_subtract(&sc->final_buf[(i * 2) + 1],
+				&sc->our_chan_pair->chan_right.out_data[i]);
+	}
+}
+
+void check_binaural_position_change(struct ast_bridge *bridge,
+		struct softmix_bridge_data *softmix_data)
+{
+	unsigned int pos_change;
+	struct ast_bridge_channel *bridge_channel;
+
+	/*
+	 * We only check binaural things if binaural is activated by the config
+	 * and at least one binaural channel joined.
+	 */
+	if (!(bridge->softmix.binaural_active && softmix_data->convolve.binaural_active)) {
+		return;
+	}
+	/*
+	 * Before we pull any audio, we must check if any channel requests a
+	 * change of binaural positions.
+	 */
+	pos_change = 0;
+	AST_LIST_TRAVERSE(&bridge->channels, bridge_channel, entry) {
+		if (!bridge_channel->binaural_pos_change) {
+			continue;
+		}
+		ast_bridge_channel_lock_bridge(bridge_channel);
+		bridge_channel->binaural_pos_change = 0;
+		ast_bridge_unlock(bridge_channel->bridge);
+		pos_change = 1;
+	}
+
+	if (pos_change) {
+		random_binaural_pos_change(softmix_data);
+	}
+}
+
+void add_binaural_mixing(struct ast_bridge *bridge, struct softmix_bridge_data *softmix_data,
+		unsigned int softmix_samples, struct softmix_mixing_array *mixing_array,
+		struct softmix_channel *sc, const char *channel_name)
+{
+	struct convolve_channel_pair *pair;
+
+	pair = NULL;
+	/* We only check binaural things if at least one binaural channel joined. */
+	if (!(bridge->softmix.binaural_active && softmix_data->convolve.binaural_active
+			&& (softmix_samples % CONVOLUTION_SAMPLE_SIZE) == 0)) {
+		return;
+	}
+
+	if (!sc->is_announcement) {
+		pair = do_convolve_pair(&softmix_data->convolve, sc->binaural_pos,
+				mixing_array->buffers[mixing_array->used_entries], softmix_samples, channel_name);
+	}
+	sc->our_chan_pair = pair;
+	mixing_array->chan_pairs[mixing_array->used_entries] = pair;
+}
+
+void binaural_mixing(struct ast_bridge *bridge, struct softmix_bridge_data *softmix_data,
+		struct softmix_mixing_array *mixing_array, int16_t *bin_buf, int16_t *ann_buf)
+{
+	unsigned int idx;
+	unsigned int x;
+
+	if (!(bridge->softmix.binaural_active && softmix_data->convolve.binaural_active)) {
+		return;
+	}
+	/* mix it like crazy (binaural channels) */
+	memset(bin_buf, 0, MAX_DATALEN);
+	memset(ann_buf, 0, MAX_DATALEN);
+
+	for (idx = 0; idx < mixing_array->used_entries; idx++) {
+		if (mixing_array->chan_pairs[idx] == NULL) {
+			for (x = 0; x < softmix_data->default_sample_size; x++) {
+				ast_slinear_saturated_add(bin_buf + (x * 2), mixing_array->buffers[idx] + x);
+				ast_slinear_saturated_add(bin_buf + (x * 2) + 1, mixing_array->buffers[idx] + x);
+				ann_buf[x * 2] = mixing_array->buffers[idx][x];
+				ann_buf[(x * 2) + 1] = mixing_array->buffers[idx][x];
+			}
+		} else {
+			for (x = 0; x < softmix_data->default_sample_size; x++) {
+				ast_slinear_saturated_add(bin_buf + (x * 2),
+						mixing_array->chan_pairs[idx]->chan_left.out_data + x);
+				ast_slinear_saturated_add(bin_buf + (x * 2) + 1,
+						mixing_array->chan_pairs[idx]->chan_right.out_data + x);
+			}
+		}
+	}
+}
+
+void create_binaural_frame(struct ast_bridge_channel *bridge_channel,
+		struct softmix_channel *sc, int16_t *bin_buf, int16_t *ann_buf,
+		unsigned int softmix_datalen, unsigned int softmix_samples, int16_t *buf)
+{
+	unsigned int i;
+
+	sc->write_frame.datalen = softmix_datalen * 2;
+	sc->write_frame.samples = softmix_samples * 2;
+	if (!bridge_channel->binaural_suspended) {
+		sc->binaural_suspended = 0;
+		if (sc->is_announcement) {
+			memcpy(sc->final_buf, ann_buf, softmix_datalen * 2);
+		} else {
+			memcpy(sc->final_buf, bin_buf, softmix_datalen * 2);
+		}
+		return;
+	}
+
+	/*
+	 * Mark that binaural output is suspended, since we use two channel audio
+	 * we copy the same signals into both channels.
+	 */
+	sc->binaural_suspended = 1;
+	for (i = 0; i < softmix_samples; i++) {
+		sc->final_buf[i * 2] = buf[i];
+		sc->final_buf[(i * 2) + 1] = buf[i];
+	}
+}
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/bridge_softmix_internal.h 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/bridge_softmix_internal.h
--- 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/bridge_softmix_internal.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/bridge_softmix_internal.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,442 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2011, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ * David Vossel <dvossel@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Multi-party software based channel mixing (header)
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ * \author David Vossel <dvossel@digium.com>
+ *
+ * \ingroup bridges
+ */
+
+#ifndef _ASTERISK_BRIDGE_SOFTMIX_INTERNAL_H
+#define _ASTERISK_BRIDGE_SOFTMIX_INTERNAL_H
+
+#include "asterisk.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <sys/time.h>
+#include <signal.h>
+#include <errno.h>
+#include <unistd.h>
+
+#include "asterisk/module.h"
+#include "asterisk/channel.h"
+#include "asterisk/bridge.h"
+#include "asterisk/bridge_technology.h"
+#include "asterisk/frame.h"
+#include "asterisk/options.h"
+#include "asterisk/logger.h"
+#include "asterisk/slinfactory.h"
+#include "asterisk/astobj2.h"
+#include "asterisk/timing.h"
+#include "asterisk/translate.h"
+#include "asterisk/rtp_engine.h"
+#include "asterisk/vector.h"
+
+#ifdef BINAURAL_RENDERING
+#include <fftw3.h>
+#endif
+
+#if defined(__Darwin__) || defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__DragonFly__) || defined(__NetBSD__) || defined(__CYGWIN__)
+#include <float.h>
+#else
+#include <values.h>
+#endif
+
+#define MAX_DATALEN 8096
+#define DEFAULT_ENERGY_HISTORY_LEN 150
+
+/*! Setting the sample rate to 48000 by default if binaural is activated. */
+#define SOFTMIX_BINAURAL_SAMPLE_RATE 48000
+/*! We only support 20 ms interval length with binaural data at the moment. */
+#define BINAURAL_MIXING_INTERVAL 20
+
+struct convolve_channel {
+	/*! The head related transfer function used for convolving */
+	double *hrtf;
+	/*! Input signals for fftw */
+	double *fftw_in;
+	/*! Output signals from the fftw */
+	double *fftw_out;
+	/*! Signals for overlap add */
+	float *overlap_add;
+	/*! The resulting data after the convolution */
+	int16_t *out_data;
+#ifdef BINAURAL_RENDERING
+	/*! The fftw plan for binaural signaling */
+	fftw_plan fftw_plan;
+	/*! The inverse fftw plan for binaural signaling */
+	fftw_plan fftw_plan_inverse;
+#endif
+};
+
+struct convolve_channel_pair {
+	/*! The left channel of a stereo channel pair */
+	struct convolve_channel chan_left;
+	/*! The right channel of a stereo channel pair */
+	struct convolve_channel chan_right;
+};
+
+struct convolve_data {
+	/*! A count of all channels potentialy having input data for the conference. */
+	int number_channels;
+	/*! Will set to true if there is at least one binaural output.
+	 * Only if set to true data will be convolved. */
+	int binaural_active;
+	/*! The length of the head related transfer function */
+	unsigned int hrtf_length;
+	/*! Number of channels available for convolving.
+	 * We do not delete a channel when a member leaves, cause we can reuse it for the next one. */
+	int chan_size;
+	/*! The positions of the single channels in the virtual room */
+	int *pos_ids;
+	/*! Each channel has a stereo pair of channels for the convolution */
+	struct convolve_channel_pair **cchan_pair;
+};
+
+struct video_follow_talker_data {
+	/*! audio energy history */
+	int energy_history[DEFAULT_ENERGY_HISTORY_LEN];
+	/*! The current slot being used in the history buffer, this
+	 *  increments and wraps around */
+	int energy_history_cur_slot;
+	/*! The current energy sum used for averages. */
+	int energy_accum;
+	/*! The current energy average */
+	int energy_average;
+};
+
+struct softmix_remb_collector;
+
+/*! \brief Structure which contains per-channel mixing information */
+struct softmix_channel {
+	/*! Lock to protect this structure */
+	ast_mutex_t lock;
+	/*! Factory which contains audio read in from the channel */
+	struct ast_slinfactory factory;
+	/*! Frame that contains mixed audio to be written out to the channel */
+	struct ast_frame write_frame;
+	/*! Current expected read slinear format. */
+	struct ast_format *read_slin_format;
+	/*! DSP for detecting silence */
+	struct ast_dsp *dsp;
+	/*!
+	 * \brief TRUE if a channel is talking.
+	 *
+	 * \note This affects how the channel's audio is mixed back to
+	 * it.
+	 */
+	unsigned int talking:1;
+	/*! TRUE if the channel provided audio for this mixing interval */
+	unsigned int have_audio:1;
+	/*! We set binaural also as channel data, to have better tracking.
+	 *  It is also present in transpvt.
+	 */
+	unsigned int binaural:1;
+	/*! TRUE if this is an announcement channel (data will not be convolved) */
+	unsigned int is_announcement:1;
+	/*! The position of the channel in the virtual room represented by an id
+	 *	This ID has to be set even if the channel has no binaural output!
+	 */
+	unsigned int binaural_pos;
+	/*! The channel pair for this channel */
+	struct convolve_channel_pair *our_chan_pair;
+	/*! Marks the channel for suspending all binaural activity on the output */
+	unsigned int binaural_suspended:1;
+	/*! Channel sample rate, stored to retrieve it after unsuspending the channel */
+	int rate;
+	/*! Buffer containing final mixed audio from all sources */
+	short final_buf[MAX_DATALEN];
+	/*! Buffer containing only the audio from the channel */
+	short our_buf[MAX_DATALEN];
+	/*! Data pertaining to talker mode for video conferencing */
+	struct video_follow_talker_data video_talker;
+	/*! The ideal stream topology for the channel */
+	struct ast_stream_topology *topology;
+	/*! The latest REMB report from this participant */
+	struct ast_rtp_rtcp_feedback_remb remb;
+	/*! The REMB collector for this channel, collects REMB from all video receivers */
+	struct softmix_remb_collector *remb_collector;
+	/*! The bridge streams which are feeding us video sources */
+	AST_VECTOR(, int) video_sources;
+};
+
+struct softmix_bridge_data {
+	struct ast_timer *timer;
+	/*!
+	 * \brief Bridge pointer passed to the softmix mixing thread.
+	 *
+	 * \note Does not need a reference because the bridge will
+	 * always exist while the mixing thread exists even if the
+	 * bridge is no longer actively using the softmix technology.
+	 */
+	struct ast_bridge *bridge;
+	/*! Lock for signaling the mixing thread. */
+	ast_mutex_t lock;
+	/*! Condition, used if we need to wake up the mixing thread. */
+	ast_cond_t cond;
+	/*! Thread handling the mixing */
+	pthread_t thread;
+	unsigned int internal_rate;
+	unsigned int internal_mixing_interval;
+	/*! TRUE if the mixing thread should stop */
+	unsigned int stop:1;
+	/*! The default sample size (e.g. using Opus at 48khz and 20 ms mixing
+	 * interval, sample size is 960) */
+	unsigned int default_sample_size;
+	/*! All data needed for binaural signaling */
+	struct convolve_data convolve;
+	/*! TRUE if the first attempt to init binaural rendering data was done
+	 * (does not guarantee success)
+	 */
+	unsigned int binaural_init;
+	/*! The last time a video update was sent into the bridge */
+	struct timeval last_video_update;
+	/*! The last time a REMB frame was sent to each source of video */
+	struct timeval last_remb_update;
+	/*! Per-bridge stream REMB collectors, which flow back to video source */
+	AST_VECTOR(, struct softmix_remb_collector *) remb_collectors;
+};
+
+struct softmix_mixing_array {
+	unsigned int max_num_entries;
+	unsigned int used_entries;
+	int16_t **buffers;
+	/*! Stereo channel pairs used to store convolved binaural signals */
+	struct convolve_channel_pair **chan_pairs;
+};
+
+/*!
+ * \brief Deletes left over signals on a channel that it can be reused.
+ *
+ * \param channel_pair The channel pair which contains the left and right audio channel.
+ * \param default_sample_size The sample size which the channel pair uses.
+ */
+void reset_channel_pair(struct convolve_channel_pair *channel_pair,
+		unsigned int default_sample_size);
+
+/*!
+ * \brief Randomly changes the virtual positions of conference participants.
+ *
+ * \param softmix_data The structure containing all position informations.
+ */
+void random_binaural_pos_change(struct softmix_bridge_data *softmix_data);
+
+/*!
+ * \brief Binaural convolving of audio data for a channel.
+ *
+ * \param chan The channel that will contain the binaural audio data as result.
+ * \param in_samples The audio data which will be convolved.
+ * \param in_sample_size The size of the audio data.
+ * \param hrtf_length The length of the head related transfer function used to convolve the audio.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int do_convolve(struct convolve_channel *chan, int16_t *in_samples,
+		unsigned int in_sample_size, unsigned int hrtf_length);
+
+/*!
+ * \brief Binaural convolving of audio data for a channel pair (left and right channel).
+ *
+ * \param data  Contains the left and right audio channel.
+ * \param pos_id The position the channel has in the virtual enviroment.
+ * \param in_samples The audio data which will be convolved for both channels.
+ * \param in_sample_size The size of the audio data.
+ * \param channel_name The name of the channel
+ *
+ * \retval The channel pair with convolved audio on success.
+ * \retval NULL failure
+ */
+struct convolve_channel_pair *do_convolve_pair(struct convolve_data *data,
+		unsigned int pos_id, int16_t *in_samples, unsigned int in_sample_size,
+		const char *channel_name);
+
+/*!
+ * \brief Provides a head related impulse response for the given position in the virtual
+ * enviroment.
+ *
+ * \param chan_pos The position of the channel in the virtual enviroment.
+ * \param chan_side 0 for the left audio channel, 1 for the right.
+ *
+ * \retval The hrir for the given position in the virtual room for either the left or right
+ *  channels.
+ * \retval NULL on failure.
+ *
+ */
+float *get_hrir(unsigned int chan_pos, unsigned int chan_side);
+
+/*!
+ * \brief Initializes all data needed for binaural audio processing.
+ *
+ * \param channel The channel used for binaural audio processing.
+ * \param hrtf_len The length of the head related impulse response used for binaural processing.
+ * \param chan_pos The position of the channel in the virtual enviroment.
+ * \param chan_side 0 for the left audio channel, 1 for the right.
+ * \param default_sample_size The default size of audio samples.
+ *
+ * \retval 0 on success
+ * \retval -1 on failure
+ */
+int init_convolve_channel(struct convolve_channel *channel, unsigned int hrtf_len,
+		unsigned int chan_pos, unsigned int chan_side, unsigned int default_sample_size);
+
+/*!
+ * \brief Initializies all data needed for binaural audio processing of a channel pair
+ * (left and right).
+ *
+ * \param cchan_pair The channel pair used for binaural audio processing.
+ * \param hrtf_len The length of the head related impulse response used for binaural processing.
+ * \param chan_pos The position of the channel in the virtual enviroment.
+ * \param default_sample_size The default size of audio samples.
+ *
+ * \retval 0 on success
+ * \retval -1 on failure
+ */
+int init_convolve_channel_pair(struct convolve_channel_pair *cchan_pair,
+		unsigned int hrtf_len, unsigned int chan_pos, unsigned int default_sample_size);
+
+/*!
+ * \brief Preinits a specific number of channels (CONVOVLE_CHANNEL_PREALLOC)
+ * at the beginning of a conference.
+ *
+ * \param data Contains all channels and data needed for binaural processing
+ *  (e.g. head related transfer functions).
+ * \param default_sample_size The default size of audio samples.
+ *
+ * \retval 0 on success
+ * \retval -1 on failure
+ */
+int init_convolve_data(struct convolve_data *data, unsigned int default_sample_size);
+
+/*!
+ * \brief Frees all data needed for binaural processing by an audio channel.
+ *
+ * \param cchan The channel to clean up.
+ */
+void free_convolve_channel(struct convolve_channel *cchan);
+
+/*!
+ * \brief Frees all data needed for binaural processing by a pair of audio channels
+ *  (left and right).
+ *
+ * \param cchan_pair The channel pair to clean up.
+ */
+void free_convolve_channel_pair(struct convolve_channel_pair *cchan_pair);
+
+/*!
+ * \brief Frees all channels and data needed for binaural audio processing.
+ *
+ * \param data Contains all channels and data for the cleanup process.
+ */
+void free_convolve_data(struct convolve_data *data);
+
+/*!
+ * \brief Joins a channel into a virtual enviroment build with the help of binaural sythesis.
+ *
+ * \param data Contains all channels and data needed for binaural processing
+ *  (e.g. head related transfer functions).
+ * \param default_sample_size The default size of audio samples.
+ *
+ * \retval The position of the channel in the virtual enviroment.
+ * \retval -1 on failure
+ */
+int set_binaural_data_join(struct convolve_data *data, unsigned int default_sample_size);
+
+/*!
+ * \brief Removes a channel from the binaural conference bridge. Marks the position in
+ *  the virtual room as unused that it can be reused by the next channel which enters the
+ *  conference.
+ *
+ * \param data Contains all channels and data needed for binaural processing
+ *  (e.g. head related transfer functions).
+ * \param pos The position of the channel in the virtual enviroment.
+ * \param default_sample_size The default size of audio samples.
+ */
+void set_binaural_data_leave(struct convolve_data *data, unsigned int pos,
+		unsigned int default_sample_size);
+
+/*!
+ * \brief Writes the binaural audio to a channel.
+ *
+ * \param sc The softmix channel.
+ * \param default_sample_size The default size of audio samples.
+ */
+void softmix_process_write_binaural_audio(struct softmix_channel *sc,
+		unsigned int default_sample_size);
+
+/*!
+ * \brief Checks if a position change in the virual enviroment is requested by one of
+ * the participants.
+ *
+ * \param bridge The conference bridge.
+ * \param softmix_data The data used by the softmix bridge.
+ */
+void check_binaural_position_change(struct ast_bridge *bridge,
+		struct softmix_bridge_data *softmix_data);
+
+/*!
+ * \brief Processes audio data with the binaural synthesis and adds the result to the mixing array.
+ *
+ * \param bridge The conference bridge needed to check if binaural processing is active or not.
+ * \param softmix_data Contains all data for the softmix bridge and for the binaural processing.
+ * \param softmix_samples The sample size.
+ * \param mixing_array The array which holds all audio data for mixing.
+ * \param sc The channel which contains the audio data to process.
+ * \param channel_name The name of the channel
+ */
+void add_binaural_mixing(struct ast_bridge *bridge, struct softmix_bridge_data *softmix_data,
+		unsigned int softmix_samples, struct softmix_mixing_array *mixing_array,
+		struct softmix_channel *sc, const char *channel_name);
+
+/*!
+ * \brief Mixes all binaural audio data contained in the mixing array.
+ *
+ * \param bridge The conference bridge needed to check if binaural processing is active or not.
+ * \param softmix_data Contains all data for the softmix bridge and for the binaural processing.
+ * \param mixing_array The array which holds all audio data for mixing.
+ * \param bin_buf The buffer that will contain the mixing results.
+ * \param ann_buf The buffer that will contain mixed announcements in an interleaved format.
+ */
+void binaural_mixing(struct ast_bridge *bridge, struct softmix_bridge_data *softmix_data,
+		struct softmix_mixing_array *mixing_array, int16_t *bin_buf, int16_t *ann_buf);
+
+/*!
+ * \brief Creates a frame out of binaural audio data.
+ *
+ * \param bridge_channel Contains the information if binaural processing is active or not.
+ *  If active binaural audio data will be copied, if not mono data will be provided in an
+ *  interleaved format.
+ * \param sc The softmix channel holding all informations for the process.
+ * \param bin_buf The buffer that contains all mixing results.
+ * \param ann_buf The buffer that contains mixed announcements in an interleaved format.
+ * \param softmix_datalen The size of the audio data.
+ * \param softmix_samples The number of audio samples.
+ * \param buf The buffer that contains all mono mixing results, used if binaural processing is
+ *  inactive.
+ */
+void create_binaural_frame(struct ast_bridge_channel *bridge_channel,
+		struct softmix_channel *sc, int16_t *bin_buf, int16_t *ann_buf,
+		unsigned int softmix_datalen, unsigned int softmix_samples, int16_t *buf);
+
+#endif /* _ASTERISK_BRIDGE_SOFTMIX_INTERNAL_H */
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/hrirs_configuration.h 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/hrirs_configuration.h
--- 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/hrirs_configuration.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/hrirs_configuration.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,63 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Frank Haase, Dennis Guse
+ *
+ * Frank Haase <fra.haase@gmail.com>
+ * Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Multi-party software binaural channel mixing (header)
+ *
+ * \author Frank Haase <fra.haase@googlemail.com>
+ * \author Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * \ingroup bridges
+ */
+
+#ifndef _ASTERISK_HRIRS_CONFIGURATION_H
+#define _ASTERISK_HRIRS_CONFIGURATION_H
+
+#include "hrirs.h"
+
+/*! The size of possible positions in the virtual enviroment build with the help
+ * of binaural audio processing.
+ */
+#define POSITION_SIZE 181
+
+#if POSITION_SIZE != HRIRS_IMPULSE_SIZE
+#error "The conference is designed for 181 individual places at the moment. If you want to change this please alter the positions array first."
+#endif
+
+/*! The offset for the left channel audio channel. */
+#define HRIRS_CHANNEL_LEFT 0
+/*! The offset for the right channel audio channel. */
+#define HRIRS_CHANNEL_RIGHT 1
+
+/*! The ast_binaural_positions array contains a specific plan to order conference
+ * participants in the virtual enviroment.
+ */
+static unsigned int ast_binaural_positions[POSITION_SIZE] = {
+        90, 80, 100, 70, 110, 60, 120, 50, 130, 40, 140, 20, 160, 0, 180, 85, 95, 75, 105, 65, 115,
+        55, 125, 45, 135, 30, 150, 10, 170, 87, 93, 82, 98, 77, 103, 72, 108, 67, 113, 62, 118, 57,
+        123, 52, 128, 47, 133, 42, 138, 35, 145, 25, 155, 15, 165, 5, 175, 88, 92, 86, 83, 97, 81,
+        99, 78, 102, 76, 104, 73, 94, 107, 71, 109, 68, 112, 66, 114, 63, 117, 61, 119, 58, 122, 56,
+        124, 53, 127, 51, 129, 48, 132, 46, 134, 43, 137, 41, 139, 37, 143, 32, 148, 27, 153, 22,
+        158, 17, 163, 12, 168, 7, 96, 173, 2, 178, 89, 91, 84, 79, 101, 74, 106, 69, 111, 64, 116,
+        59, 121, 54, 126, 49, 131, 44, 136, 38, 142, 36, 144, 33, 147, 31, 149, 28, 152, 26, 154,
+        23, 157, 21, 159, 162, 16, 164, 13, 167, 11, 169, 8, 172, 6, 174, 3, 177, 1, 179, 39, 141,
+        34, 146, 29, 151, 24, 156, 19, 161, 14, 166, 9, 171, 4, 18, 176 };
+
+#endif /* _ASTERISK_HRIRS_CONFIGURATION_H */
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/hrirs.h 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/hrirs.h
--- 1:13.23.1~dfsg-2/bridges/bridge_softmix/include/hrirs.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_softmix/include/hrirs.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,50730 @@
+/* Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Frank Haase, Dennis Guse
+ *
+ * Frank Haase <fra.haase@gmail.com>
+ * Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * Copyright (c) 2001 The Regents of the University of California. All Rights Reserved.
+ *
+ * The HRIRs used here are obtained from The CIPIC HRTF Database
+ * (http://interface.cipic.ucdavis.edu/CIL_html/CIL_HRTF_database.htm)
+ * Note that the above mentioned material is Copyright (c) 2001 The
+ * Regents of the University of California. All Rights Reserved.
+ *
+ * Download the file
+ * http://interface.cipic.ucdavis.edu/data/special_kemar_hrir.tar and
+ * uncompress it in the folder where this Matlab script resides. Finally,
+ * run the script.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Multi-party software binaural channel HRIRS
+ *
+ * \author Frank Haase <fra.haase@googlemail.com>
+ * \author Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * \ingroup bridges
+ */
+
+/*
+ * This file was created with command:
+ * $ conf_bridge_binaural_hrir_importer sounds/hrirs_48k.wav 0 180
+ */
+#define HRIRS_IMPULSE_LEN 557
+#define HRIRS_IMPULSE_SIZE 181
+#define HRIRS_SAMPLE_RATE 48000
+
+float hrirs_left[HRIRS_IMPULSE_SIZE][HRIRS_IMPULSE_LEN] = {
+{-0.0000610351562500, 0.0002441406250000, -0.0001220703125000, 0.0000915527343750,
+-0.0003051757812500, 0.0003967285156250, -0.0000305175781250, 0.0001220703125000,
+0.0002136230468750, -0.0003356933593750, 0.0000000000000000, -0.0001220703125000,
+0.0002746582031250, -0.0001220703125000, 0.0004272460937500, -0.0003662109375000,
+0.0001525878906250, -0.0001220703125000, 0.0000305175781250, 0.0002441406250000,
+0.0002441406250000, 0.0000305175781250, -0.0003662109375000, 0.0001831054687500,
+-0.0004577636718750, 0.0003356933593750, -0.0002136230468750, 0.0000915527343750,
+-0.0004272460937500, -0.0001220703125000, -0.0005493164062500, 0.0000305175781250,
+0.0002441406250000, -0.0001220703125000, 0.0001525878906250, -0.0002441406250000,
+0.0001220703125000, -0.0000610351562500, 0.0007629394531250, 0.0000610351562500,
+0.0006408691406250, -0.0000610351562500, 0.0000610351562500, 0.0000915527343750,
+0.0003967285156250, 0.0002136230468750, 0.0000610351562500, 0.0000610351562500,
+-0.0007324218750000, -0.0000305175781250, -0.0003051757812500, 0.0000305175781250,
+-0.0004272460937500, -0.0000610351562500, -0.0008544921875000, -0.0001220703125000,
+-0.0003051757812500, -0.0000610351562500, 0.0001831054687500, 0.0000305175781250,
+-0.0002441406250000, -0.0000305175781250, 0.0006103515625000, 0.0000610351562500,
+0.0007934570312500, 0.0001220703125000, 0.0004272460937500, 0.0000610351562500,
+0.0005798339843750, 0.0000000000000000, 0.0007934570312500, 0.0000305175781250,
+-0.0002441406250000, -0.0000305175781250, -0.0000610351562500, -0.0000915527343750,
+-0.0001831054687500, -0.0000305175781250, -0.0008850097656250, 0.0000000000000000,
+-0.0006713867187500, -0.0000305175781250, -0.0004577636718750, 0.0000305175781250,
+-0.0008239746093750, 0.0000305175781250, -0.0003356933593750, 0.0000305175781250,
+0.0002441406250000, 0.0000000000000000, -0.0002136230468750, 0.0000305175781250,
+0.0007324218750000, 0.0000000000000000, 0.0007629394531250, -0.0000305175781250,
+0.0005798339843750, 0.0000000000000000, 0.0009155273437500, 0.0000305175781250,
+0.0009765625000000, 0.0000000000000000, 0.0002746582031250, 0.0000000000000000,
+0.0005798339843750, 0.0000305175781250, 0.0002136230468750, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0000915527343750, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, -0.0009155273437500, -0.0000305175781250,
+-0.0008239746093750, 0.0000000000000000, -0.0012207031250000, 0.0000305175781250,
+-0.0011291503906250, 0.0000000000000000, -0.0007019042968750, 0.0000305175781250,
+-0.0009765625000000, 0.0000000000000000, -0.0005493164062500, 0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, -0.0004577636718750, 0.0000000000000000,
+0.0006713867187500, -0.0000305175781250, 0.0005187988281250, -0.0000305175781250,
+0.0007019042968750, -0.0000305175781250, 0.0014648437500000, 0.0000000000000000,
+0.0010681152343750, 0.0000000000000000, 0.0014953613281250, 0.0000000000000000,
+0.0016784667968750, 0.0000000000000000, 0.0014038085937500, 0.0000305175781250,
+0.0014648437500000, 0.0000000000000000, 0.0016174316406250, 0.0000000000000000,
+0.0009155273437500, 0.0000000000000000, 0.0011901855468750, 0.0000000000000000,
+0.0008544921875000, -0.0000305175781250, 0.0002441406250000, 0.0000000000000000,
+0.0007324218750000, 0.0000305175781250, -0.0003662109375000, 0.0000000000000000,
+-0.0000915527343750, -0.0000305175781250, -0.0003356933593750, 0.0000000000000000,
+-0.0011291503906250, 0.0000000000000000, -0.0006103515625000, 0.0000000000000000,
+-0.0014038085937500, 0.0000000000000000, -0.0012817382812500, 0.0000000000000000,
+-0.0013122558593750, 0.0000305175781250, -0.0017395019531250, 0.0000000000000000,
+-0.0014648437500000, 0.0000000000000000, -0.0017700195312500, 0.0000000000000000,
+-0.0017395019531250, 0.0000000000000000, -0.0017089843750000, 0.0000000000000000,
+-0.0018615722656250, -0.0000305175781250, -0.0017089843750000, 0.0000000000000000,
+-0.0018005371093750, -0.0000305175781250, -0.0017395019531250, 0.0000000000000000,
+-0.0016784667968750, 0.0000305175781250, -0.0017395019531250, -0.0000305175781250,
+-0.0016479492187500, 0.0000305175781250, -0.0015258789062500, 0.0000000000000000,
+-0.0014953613281250, 0.0000000000000000, -0.0013732910156250, 0.0000000000000000,
+-0.0011901855468750, 0.0000000000000000, -0.0011291503906250, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, -0.0008850097656250, 0.0000000000000000,
+-0.0006408691406250, 0.0000000000000000, -0.0003051757812500, 0.0000000000000000,
+-0.0003967285156250, -0.0000305175781250, 0.0001831054687500, 0.0000000000000000,
+0.0001220703125000, -0.0000305175781250, 0.0003662109375000, 0.0000000000000000,
+0.0008544921875000, -0.0000305175781250, 0.0006713867187500, 0.0000000000000000,
+0.0011291503906250, 0.0000305175781250, 0.0011901855468750, 0.0000000000000000,
+0.0010986328125000, 0.0000000000000000, 0.0012512207031250, 0.0000000000000000,
+0.0012817382812500, 0.0000000000000000, 0.0009765625000000, -0.0000305175781250,
+0.0010986328125000, 0.0000000000000000, 0.0007934570312500, 0.0000000000000000,
+0.0003967285156250, 0.0000000000000000, 0.0005798339843750, -0.0000305175781250,
+-0.0002136230468750, 0.0000000000000000, -0.0001831054687500, 0.0000305175781250,
+-0.0003051757812500, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+-0.0006408691406250, 0.0000305175781250, -0.0008239746093750, 0.0000000000000000,
+-0.0009765625000000, 0.0000305175781250, -0.0007019042968750, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0006713867187500, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, 0.0000000000000000, 0.0000000000000000,
+0.0001525878906250, 0.0000000000000000, 0.0006408691406250, 0.0000305175781250,
+0.0004882812500000, 0.0000305175781250, 0.0005187988281250, 0.0000000000000000,
+0.0007019042968750, 0.0000000000000000, 0.0004272460937500, 0.0000000000000000,
+0.0001220703125000, -0.0000305175781250, 0.0003051757812500, -0.0000305175781250,
+-0.0002746582031250, 0.0000000000000000, -0.0003662109375000, 0.0000000000000000,
+-0.0002441406250000, 0.0000000000000000, -0.0004882812500000, 0.0000000000000000,
+-0.0004882812500000, 0.0000000000000000, -0.0000610351562500, -0.0000305175781250,
+-0.0001525878906250, 0.0000000000000000, -0.0000305175781250, 0.0000000000000000,
+0.0003662109375000, -0.0000305175781250, 0.0002136230468750, -0.0000610351562500,
+0.0001831054687500, 0.0000000000000000, 0.0003356933593750, -0.0000305175781250,
+0.0001220703125000, 0.0000000000000000, -0.0001525878906250, 0.0000610351562500,
+0.0000305175781250, 0.0000305175781250, -0.0001831054687500, 0.0000000000000000,
+-0.0003356933593750, 0.0000610351562500, -0.0001220703125000, 0.0000000000000000,
+0.0000000000000000, -0.0000610351562500, -0.0001220703125000, -0.0000305175781250,
+0.0001525878906250, 0.0000000000000000, 0.0002441406250000, -0.0000610351562500,
+0.0001220703125000, 0.0000000000000000, 0.0000915527343750, 0.0000610351562500,
+0.0001831054687500, 0.0000000000000000, -0.0001220703125000, 0.0000000000000000,
+-0.0001220703125000, 0.0000610351562500, -0.0000610351562500, 0.0000000000000000,
+-0.0001525878906250, -0.0000610351562500, -0.0001831054687500, -0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, 0.0000610351562500, -0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, 0.0001220703125000, 0.0000610351562500,
+0.0000915527343750, 0.0000610351562500, -0.0000915527343750, -0.0000305175781250,
+-0.0000610351562500, 0.0000305175781250, 0.0000000000000000, 0.0000000000000000,
+-0.0000610351562500, -0.0000915527343750, -0.0000915527343750, -0.0000610351562500,
+0.0000610351562500, 0.0000000000000000, 0.0000610351562500, -0.0000305175781250,
+0.0000305175781250, -0.0000305175781250, 0.0000610351562500, 0.0000915527343750,
+0.0000915527343750, 0.0000610351562500, -0.0000610351562500, -0.0000305175781250,
+-0.0000610351562500, 0.0000305175781250, 0.0000000000000000, 0.0000000000000000,
+-0.0000610351562500, -0.0001525878906250, -0.0000610351562500, -0.0000915527343750,
+0.0000610351562500, 0.0000305175781250, 0.0000915527343750, -0.0000610351562500,
+0.0000000000000000, 0.0000000000000000, 0.0000610351562500, 0.0002136230468750,
+0.0000915527343750, 0.0001220703125000, -0.0000610351562500, 0.0000305175781250,
+-0.0000305175781250, 0.0001220703125000, -0.0000305175781250, 0.0000305175781250,
+-0.0000305175781250, -0.0002441406250000, -0.0000610351562500, -0.0000915527343750,
+0.0000305175781250, -0.0001525878906250, 0.0000305175781250, -0.0002746582031250,
+0.0000305175781250, -0.0000610351562500, 0.0000610351562500, 0.0001831054687500,
+0.0000610351562500, 0.0000000000000000, -0.0000305175781250, 0.0002136230468750,
+-0.0000305175781250, 0.0003662109375000, -0.0000305175781250, 0.0001525878906250,
+-0.0000305175781250, 0.0000610351562500, -0.0000305175781250, 0.0002136230468750,
+0.0000000000000000, -0.0002441406250000, 0.0000000000000000, -0.0003051757812500,
+0.0000000000000000, -0.0001525878906250, 0.0000305175781250, -0.0003356933593750,
+0.0000305175781250, -0.0003662109375000, 0.0000000000000000, 0.0000915527343750,
+0.0000305175781250, -0.0000305175781250, 0.0000000000000000, 0.0000915527343750,
+-0.0000610351562500, 0.0004882812500000, 0.0000000000000000, 0.0003662109375000,
+0.0000000000000000, 0.0003051757812500, 0.0000000000000000, 0.0004882812500000,
+0.0000000000000000, 0.0001831054687500, 0.0000000000000000, -0.0001220703125000,
+0.0000000000000000, 0.0000610351562500, 0.0000000000000000, -0.0004882812500000,
+0.0000000000000000, -0.0005798339843750, 0.0000000000000000, -0.0004882812500000,
+0.0000000000000000, -0.0006408691406250, 0.0000000000000000, -0.0006408691406250,
+0.0000000000000000, -0.0001525878906250, 0.0000000000000000, -0.0002746582031250,
+0.0000000000000000, 0.0000000000000000, 0.0000000000000000, 0.0005187988281250,
+0.0000305175781250, 0.0003051757812500, 0.0000000000000000, 0.0007629394531250,
+0.0000000000000000, 0.0008850097656250, 0.0000000000000000, 0.0007324218750000,
+-0.0000305175781250, 0.0008850097656250, -0.0000305175781250, 0.0009460449218750,
+0.0000000000000000, 0.0005493164062500, 0.0000000000000000, 0.0006408691406250,
+-0.0000305175781250, 0.0003967285156250, 0.0000000000000000, -0.0000610351562500,
+0.0000305175781250, 0.0001220703125000, -0.0000305175781250, -0.0004882812500000,
+0.0000000000000000, -0.0004882812500000, 0.0000000000000000, -0.0006713867187500,
+0.0000000000000000, -0.0011291503906250, 0.0000000000000000, -0.0009765625000000,
+0.0000000000000000, -0.0013122558593750, -0.0000305175781250, -0.0013732910156250,
+0.0000000000000000, -0.0013732910156250, 0.0000000000000000, -0.0015258789062500,
+0.0000000000000000, -0.0015258789062500, 0.0000000000000000, -0.0015258789062500,
+0.0000000000000000, -0.0014953613281250, 0.0000000000000000, -0.0014648437500000,
+0.0000000000000000, -0.0014648437500000, 0.0000000000000000, -0.0014343261718750,
+0.0000000000000000, -0.0014343261718750, 0.0000000000000000, -0.0013427734375000,
+0.0000305175781250, -0.0013427734375000, -0.0000305175781250, -0.0013732910156250,
+0.0000000000000000, -0.0012817382812500, 0.0000000000000000, -0.0014343261718750,
+0.0000000000000000, -0.0013732910156250, 0.0000305175781250, -0.0013732910156250,
+0.0000000000000000, -0.0014953613281250, 0.0000305175781250, -0.0014343261718750,
+0.0000000000000000, -0.0016174316406250, 0.0000000000000000, -0.0015563964843750,
+0.0000000000000000},
+{-0.0000915527343750, 0.0001525878906250, 0.0000000000000000, 0.0000000000000000,
+-0.0000305175781250, -0.0001831054687500, -0.0000915527343750, -0.0000915527343750,
+0.0000610351562500, -0.0000915527343750, 0.0000915527343750, -0.0002441406250000,
+0.0000305175781250, -0.0000305175781250, 0.0000305175781250, 0.0002136230468750,
+0.0000610351562500, 0.0000000000000000, -0.0000305175781250, 0.0001831054687500,
+-0.0000610351562500, 0.0003356933593750, 0.0000000000000000, 0.0001220703125000,
+-0.0000305175781250, -0.0000305175781250, -0.0000610351562500, 0.0001220703125000,
+0.0000000000000000, -0.0002746582031250, 0.0000305175781250, -0.0003662109375000,
+0.0000000000000000, -0.0001831054687500, 0.0000305175781250, -0.0002746582031250,
+0.0000610351562500, -0.0003356933593750, -0.0000305175781250, 0.0001525878906250,
+-0.0000305175781250, 0.0000915527343750, 0.0000000000000000, 0.0001220703125000,
+-0.0000305175781250, 0.0004882812500000, -0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, 0.0001220703125000, 0.0000000000000000, 0.0002746582031250,
+0.0000000000000000, 0.0000305175781250, 0.0000305175781250, -0.0004577636718750,
+0.0000610351562500, -0.0001831054687500, 0.0000000000000000, -0.0004577636718750,
+-0.0000305175781250, -0.0006408691406250, 0.0000000000000000, -0.0003051757812500,
+0.0000000000000000, -0.0001525878906250, 0.0000000000000000, -0.0003356933593750,
+0.0000000000000000, 0.0003662109375000, 0.0000000000000000, 0.0003967285156250,
+0.0000000000000000, 0.0004272460937500, 0.0000305175781250, 0.0008239746093750,
+0.0000305175781250, 0.0007019042968750, 0.0000000000000000, 0.0004882812500000,
+0.0000000000000000, 0.0007019042968750, 0.0000000000000000, 0.0003051757812500,
+-0.0000305175781250, -0.0001831054687500, 0.0000000000000000, 0.0001220703125000,
+0.0000305175781250, -0.0006408691406250, -0.0000305175781250, -0.0007019042968750,
+0.0000000000000000, -0.0007324218750000, 0.0000000000000000, -0.0011291503906250,
+0.0000000000000000, -0.0010375976562500, -0.0000305175781250, -0.0009765625000000,
+0.0000000000000000, -0.0010986328125000, 0.0000000000000000, -0.0008544921875000,
+0.0000000000000000, -0.0005493164062500, -0.0000305175781250, -0.0007019042968750,
+0.0000000000000000, -0.0000915527343750, 0.0000000000000000, -0.0001220703125000,
+0.0000305175781250, 0.0001525878906250, 0.0000000000000000, 0.0007019042968750,
+0.0000000000000000, 0.0005187988281250, 0.0000000000000000, 0.0010681152343750,
+0.0000000000000000, 0.0011291503906250, 0.0000000000000000, 0.0012512207031250,
+-0.0000305175781250, 0.0015563964843750, -0.0000305175781250, 0.0015258789062500,
+0.0000000000000000, 0.0016784667968750, 0.0000000000000000, 0.0017089843750000,
+0.0000000000000000, 0.0017395019531250, 0.0000000000000000, 0.0018310546875000,
+0.0000000000000000, 0.0017700195312500, -0.0000305175781250, 0.0018005371093750,
+0.0000000000000000, 0.0018310546875000, -0.0000305175781250, 0.0018005371093750,
+0.0000000000000000, 0.0018310546875000, -0.0000305175781250, 0.0018005371093750,
+-0.0000305175781250, 0.0019226074218750, 0.0000000000000000, 0.0018615722656250,
+0.0000000000000000, 0.0018615722656250, -0.0000305175781250, 0.0019531250000000,
+0.0000000000000000, 0.0019226074218750, 0.0000000000000000, 0.0020141601562500,
+0.0000305175781250, 0.0019836425781250, 0.0000000000000000, 0.0019836425781250,
+0.0000305175781250, 0.0020446777343750, 0.0000000000000000, 0.0020141601562500,
+-0.0000305175781250, 0.0019226074218750, 0.0000000000000000, 0.0020141601562500,
+0.0000000000000000, 0.0018615722656250, 0.0000000000000000, 0.0017089843750000,
+-0.0000305175781250, 0.0018310546875000, 0.0000000000000000, 0.0011596679687500,
+0.0000000000000000, 0.0014953613281250, 0.0000000000000000, 0.0010681152343750,
+0.0000305175781250, 0.0003662109375000, 0.0000000000000000, 0.0008850097656250,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, -0.0002441406250000,
+0.0000000000000000, -0.0004272460937500, 0.0000000000000000, -0.0014648437500000,
+-0.0000305175781250, -0.0009765625000000, 0.0000000000000000, -0.0015563964843750,
+0.0000000000000000, -0.0018005371093750, 0.0000000000000000, -0.0014038085937500,
+0.0000000000000000, -0.0014343261718750, 0.0000305175781250, -0.0016479492187500,
+-0.0000305175781250, -0.0004882812500000, -0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0002746582031250, 0.0000000000000000, 0.0007934570312500,
+0.0000000000000000, 0.0001831054687500, 0.0000305175781250, 0.0011901855468750,
+0.0000305175781250, 0.0013427734375000, 0.0000000000000000, 0.0010375976562500,
+0.0000000000000000, 0.0013427734375000, 0.0000305175781250, 0.0014038085937500,
+-0.0000305175781250, 0.0005493164062500, -0.0000305175781250, 0.0008239746093750,
+0.0000000000000000, 0.0003967285156250, 0.0000000000000000, -0.0005493164062500,
+-0.0000305175781250, -0.0001220703125000, 0.0000305175781250, -0.0008544921875000,
+0.0000305175781250, -0.0010681152343750, -0.0000305175781250, -0.0007629394531250,
+0.0000000000000000, -0.0008850097656250, 0.0000000000000000, -0.0010070800781250,
+0.0000000000000000, -0.0000915527343750, -0.0000305175781250, -0.0002746582031250,
+0.0000000000000000, 0.0000000000000000, 0.0000000000000000, 0.0007629394531250,
+-0.0000305175781250, 0.0004577636718750, 0.0000305175781250, 0.0005798339843750,
+0.0000305175781250, 0.0008544921875000, 0.0000000000000000, 0.0004882812500000,
+0.0000000000000000, 0.0001220703125000, 0.0000305175781250, 0.0004272460937500,
+-0.0000305175781250, -0.0003662109375000, -0.0000610351562500, -0.0003967285156250,
+-0.0000305175781250, -0.0003051757812500, -0.0000305175781250, -0.0006713867187500,
+-0.0000610351562500, -0.0006103515625000, 0.0000610351562500, -0.0001525878906250,
+0.0000610351562500, -0.0003662109375000, 0.0000305175781250, -0.0000915527343750,
+0.0000915527343750, 0.0005187988281250, 0.0000915527343750, 0.0002136230468750,
+-0.0000610351562500, 0.0004272460937500, -0.0000305175781250, 0.0006713867187500,
+-0.0000305175781250, 0.0002746582031250, -0.0001220703125000, -0.0000305175781250,
+-0.0000915527343750, 0.0002746582031250, 0.0000000000000000, -0.0004272460937500,
+-0.0000305175781250, -0.0006408691406250, 0.0000305175781250, -0.0002746582031250,
+0.0001831054687500, -0.0003356933593750, 0.0000915527343750, -0.0005493164062500,
+0.0000305175781250, 0.0003356933593750, 0.0000915527343750, 0.0003356933593750,
+0.0000000000000000, 0.0001525878906250, -0.0002136230468750, 0.0005493164062500,
+-0.0000915527343750, 0.0006103515625000, -0.0001220703125000, -0.0001220703125000,
+-0.0002746582031250, 0.0000000000000000, -0.0000610351562500, 0.0000305175781250,
+0.0002136230468750, -0.0006408691406250, 0.0001220703125000, 0.0005187988281250,
+-0.0003051757812500, -0.0002136230468750, -0.0003051757812500, 0.0001220703125000,
+-0.0000915527343750, 0.0000000000000000, -0.0000915527343750, -0.0004577636718750,
+-0.0002746582031250, -0.0004272460937500, 0.0003051757812500, 0.0000610351562500,
+0.0003356933593750, -0.0000915527343750, 0.0001831054687500, -0.0000305175781250,
+0.0005187988281250, 0.0004272460937500, 0.0005187988281250, 0.0003662109375000,
+-0.0001525878906250, 0.0000000000000000, 0.0000000000000000, 0.0001831054687500,
+-0.0001220703125000, 0.0000610351562500, -0.0007934570312500, -0.0003051757812500,
+-0.0005493164062500, -0.0001831054687500, -0.0001220703125000, -0.0000305175781250,
+-0.0005187988281250, -0.0001831054687500, -0.0000915527343750, -0.0000610351562500,
+0.0007019042968750, 0.0002441406250000, 0.0002441406250000, 0.0001220703125000,
+0.0005187988281250, 0.0000610351562500, 0.0009460449218750, 0.0001831054687500,
+0.0003051757812500, 0.0000610351562500, -0.0002441406250000, -0.0001525878906250,
+0.0003051757812500, -0.0000915527343750, -0.0006713867187500, -0.0000305175781250,
+-0.0008850097656250, -0.0001525878906250, -0.0004272460937500, -0.0000305175781250,
+-0.0005798339843750, 0.0001220703125000, -0.0008239746093750, 0.0000610351562500,
+0.0003662109375000, 0.0000610351562500, 0.0002136230468750, 0.0000915527343750,
+0.0002441406250000, 0.0000305175781250, 0.0010375976562500, -0.0000915527343750,
+0.0008544921875000, -0.0000305175781250, 0.0003051757812500, -0.0000610351562500,
+0.0007324218750000, -0.0000915527343750, 0.0002746582031250, -0.0000610351562500,
+-0.0006103515625000, 0.0000610351562500, -0.0000915527343750, 0.0000000000000000,
+-0.0007629394531250, 0.0000610351562500, -0.0010375976562500, 0.0000915527343750,
+-0.0005798339843750, 0.0000305175781250, -0.0006408691406250, -0.0000610351562500,
+-0.0008850097656250, -0.0000305175781250, 0.0002746582031250, 0.0000000000000000,
+0.0000610351562500, -0.0000610351562500, 0.0002441406250000, -0.0000305175781250,
+0.0011291503906250, 0.0000305175781250, 0.0007324218750000, 0.0000000000000000,
+0.0008850097656250, 0.0000000000000000, 0.0012512207031250, 0.0000305175781250,
+0.0007629394531250, 0.0000000000000000, 0.0004272460937500, -0.0000305175781250,
+0.0007934570312500, -0.0000305175781250, -0.0003356933593750, 0.0000000000000000,
+-0.0001525878906250, -0.0000305175781250, -0.0003967285156250, 0.0000000000000000,
+-0.0012512207031250, 0.0000305175781250, -0.0007934570312500, 0.0000305175781250,
+-0.0013427734375000, 0.0000000000000000, -0.0015258789062500, 0.0000000000000000,
+-0.0011596679687500, 0.0000000000000000, -0.0011596679687500, -0.0000305175781250,
+-0.0013427734375000, -0.0000305175781250, -0.0003662109375000, 0.0000000000000000,
+-0.0007629394531250, 0.0000000000000000, -0.0003051757812500, -0.0000305175781250,
+0.0006713867187500, 0.0000000000000000, 0.0000000000000000, 0.0000305175781250,
+0.0013122558593750, 0.0000000000000000, 0.0011901855468750, 0.0000000000000000,
+0.0012817382812500, 0.0000000000000000, 0.0020751953125000, 0.0000000000000000,
+0.0016784667968750, 0.0000000000000000, 0.0020141601562500, 0.0000000000000000,
+0.0021667480468750, 0.0000305175781250, 0.0019531250000000, 0.0000000000000000,
+0.0018920898437500, 0.0000000000000000, 0.0020751953125000, 0.0000000000000000,
+0.0013732910156250, 0.0000000000000000, 0.0017089843750000, 0.0000000000000000,
+0.0014038085937500, 0.0000000000000000, 0.0007324218750000, -0.0000305175781250,
+0.0012512207031250, 0.0000000000000000, 0.0003662109375000, -0.0000305175781250,
+0.0005493164062500, 0.0000000000000000, 0.0003967285156250, 0.0000000000000000,
+-0.0001831054687500, 0.0000000000000000, 0.0003356933593750, 0.0000000000000000,
+-0.0002746582031250, -0.0000305175781250, -0.0001220703125000, 0.0000000000000000,
+-0.0001525878906250, 0.0000000000000000, -0.0004577636718750, -0.0000305175781250,
+-0.0001220703125000, -0.0000305175781250, -0.0004577636718750, 0.0000000000000000,
+-0.0003662109375000, 0.0000000000000000, -0.0002746582031250, 0.0000000000000000,
+-0.0003967285156250, -0.0000305175781250, -0.0002136230468750, 0.0000000000000000,
+-0.0001525878906250, 0.0000000000000000, -0.0000305175781250, 0.0000000000000000,
+0.0001220703125000, 0.0000305175781250, 0.0002441406250000, 0.0000000000000000,
+0.0003662109375000, 0.0000000000000000, 0.0006408691406250, 0.0000000000000000,
+0.0006713867187500, 0.0000000000000000, 0.0008544921875000, 0.0000000000000000,
+0.0011291503906250, 0.0000000000000000, 0.0010681152343750, 0.0000000000000000,
+0.0014648437500000, 0.0000000000000000, 0.0014648437500000, 0.0000305175781250,
+0.0014648437500000, 0.0000000000000000, 0.0016784667968750, -0.0000305175781250,
+0.0016174316406250},
+{-0.0010986328125000, 0.0001525878906250, -0.0006408691406250, 0.0000610351562500,
+-0.0005798339843750, 0.0000000000000000, -0.0010375976562500, 0.0000610351562500,
+-0.0004272460937500, 0.0000610351562500, 0.0002441406250000, -0.0000610351562500,
+-0.0003051757812500, -0.0000610351562500, 0.0007324218750000, -0.0000305175781250,
+0.0009155273437500, -0.0000610351562500, 0.0005798339843750, -0.0000305175781250,
+0.0009765625000000, 0.0000915527343750, 0.0010375976562500, 0.0000305175781250,
+0.0000610351562500, 0.0000305175781250, 0.0003967285156250, 0.0000305175781250,
+0.0000000000000000, 0.0000305175781250, -0.0010070800781250, -0.0000610351562500,
+-0.0004577636718750, -0.0000305175781250, -0.0010681152343750, 0.0000000000000000,
+-0.0013732910156250, -0.0000305175781250, -0.0009460449218750, 0.0000000000000000,
+-0.0009460449218750, 0.0000305175781250, -0.0011596679687500, 0.0000305175781250,
+-0.0001220703125000, 0.0000000000000000, -0.0003967285156250, 0.0000000000000000,
+-0.0000305175781250, 0.0000000000000000, 0.0008850097656250, 0.0000000000000000,
+0.0003662109375000, -0.0000305175781250, 0.0012817382812500, 0.0000305175781250,
+0.0014038085937500, 0.0000000000000000, 0.0012207031250000, 0.0000000000000000,
+0.0015563964843750, 0.0000000000000000, 0.0015258789062500, 0.0000305175781250,
+0.0008850097656250, 0.0000000000000000, 0.0012817382812500, 0.0000000000000000,
+0.0007324218750000, 0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+0.0006103515625000, -0.0000305175781250, -0.0008239746093750, 0.0000305175781250,
+-0.0006408691406250, 0.0000000000000000, -0.0009460449218750, -0.0000305175781250,
+-0.0019531250000000, 0.0000000000000000, -0.0013732910156250, 0.0000000000000000,
+-0.0021972656250000, -0.0000305175781250, -0.0022888183593750, 0.0000000000000000,
+-0.0021362304687500, 0.0000000000000000, -0.0024719238281250, 0.0000305175781250,
+-0.0024414062500000, 0.0000000000000000, -0.0021667480468750, 0.0000000000000000,
+-0.0024108886718750, 0.0000000000000000, -0.0021362304687500, 0.0000000000000000,
+-0.0017700195312500, 0.0000000000000000, -0.0021362304687500, 0.0000000000000000,
+-0.0013427734375000, 0.0000000000000000, -0.0015869140625000, -0.0000305175781250,
+-0.0013732910156250, 0.0000000000000000, -0.0008544921875000, 0.0000305175781250,
+-0.0013732910156250, 0.0000000000000000, -0.0007629394531250, 0.0000000000000000,
+-0.0009155273437500, 0.0000000000000000, -0.0008850097656250, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0006408691406250, 0.0000000000000000,
+-0.0007019042968750, 0.0000000000000000, -0.0004882812500000, 0.0000000000000000,
+-0.0007324218750000, 0.0000000000000000, -0.0007324218750000, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, -0.0010070800781250, 0.0000305175781250,
+-0.0010986328125000, 0.0000000000000000, -0.0012207031250000, 0.0000000000000000,
+-0.0014648437500000, 0.0000305175781250, -0.0014648437500000, 0.0000305175781250,
+-0.0016174316406250, 0.0000305175781250, -0.0017700195312500, 0.0000000000000000,
+-0.0017395019531250, 0.0000000000000000, -0.0018615722656250, 0.0000000000000000,
+-0.0019226074218750, 0.0000000000000000, -0.0018310546875000, 0.0000305175781250,
+-0.0018310546875000, 0.0000000000000000, -0.0018615722656250, 0.0000305175781250,
+-0.0013732910156250, 0.0000305175781250, -0.0014953613281250, 0.0000305175781250,
+-0.0011291503906250, 0.0000000000000000, -0.0005187988281250, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, 0.0001525878906250, -0.0000305175781250,
+0.0001220703125000, 0.0000000000000000, 0.0003051757812500, -0.0000305175781250,
+0.0010070800781250, 0.0000000000000000, 0.0007629394531250, -0.0000610351562500,
+0.0011596679687500, 0.0000000000000000, 0.0013122558593750, 0.0000305175781250,
+0.0010375976562500, 0.0000000000000000, 0.0009765625000000, -0.0000305175781250,
+0.0010986328125000, 0.0000000000000000, 0.0003356933593750, 0.0000000000000000,
+0.0004577636718750, 0.0000000000000000, 0.0000915527343750, 0.0000305175781250,
+-0.0005493164062500, 0.0000000000000000, -0.0002746582031250, 0.0000610351562500,
+-0.0007934570312500, 0.0000000000000000, -0.0009765625000000, 0.0000305175781250,
+-0.0007324218750000, 0.0000305175781250, -0.0007019042968750, -0.0000610351562500,
+-0.0008239746093750, 0.0000000000000000, -0.0000610351562500, 0.0000000000000000,
+-0.0001525878906250, 0.0000000000000000, 0.0000000000000000, 0.0000305175781250,
+0.0006103515625000, 0.0000000000000000, 0.0004272460937500, 0.0000305175781250,
+0.0004577636718750, 0.0000000000000000, 0.0006713867187500, 0.0000000000000000,
+0.0003356933593750, 0.0000610351562500, 0.0000000000000000, -0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, -0.0003356933593750, 0.0000000000000000,
+-0.0004272460937500, -0.0000610351562500, -0.0002441406250000, 0.0000000000000000,
+-0.0003967285156250, 0.0000000000000000, -0.0004882812500000, 0.0000000000000000,
+0.0000915527343750, 0.0000305175781250, 0.0000610351562500, 0.0000915527343750,
+0.0000305175781250, 0.0000305175781250, 0.0003967285156250, 0.0000000000000000,
+0.0003051757812500, 0.0000305175781250, 0.0000000000000000, -0.0000610351562500,
+0.0001525878906250, -0.0000915527343750, 0.0000305175781250, -0.0000305175781250,
+-0.0003051757812500, -0.0000305175781250, -0.0001831054687500, -0.0000305175781250,
+-0.0001525878906250, 0.0000305175781250, -0.0002441406250000, 0.0000915527343750,
+-0.0000610351562500, 0.0000000000000000, 0.0002136230468750, 0.0000000000000000,
+0.0000610351562500, 0.0000305175781250, 0.0001525878906250, -0.0000305175781250,
+0.0003356933593750, -0.0000915527343750, 0.0000915527343750, 0.0000000000000000,
+-0.0001220703125000, 0.0000305175781250, 0.0000305175781250, -0.0000305175781250,
+-0.0001220703125000, 0.0000305175781250, -0.0002136230468750, 0.0000610351562500,
+-0.0000915527343750, 0.0000000000000000, 0.0000610351562500, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, 0.0000305175781250, -0.0000610351562500,
+0.0001220703125000, -0.0000915527343750, 0.0000305175781250, -0.0000305175781250,
+-0.0000915527343750, 0.0000000000000000, 0.0000610351562500, 0.0000305175781250,
+-0.0000610351562500, 0.0000915527343750, -0.0001525878906250, 0.0001220703125000,
+0.0000000000000000, 0.0000610351562500, 0.0000915527343750, 0.0000000000000000,
+0.0000000000000000, 0.0000305175781250, 0.0000305175781250, -0.0000610351562500,
+0.0001220703125000, -0.0001220703125000, 0.0000000000000000, -0.0000305175781250,
+-0.0000915527343750, 0.0000610351562500, 0.0000305175781250, 0.0000000000000000,
+-0.0000610351562500, 0.0001220703125000, -0.0001220703125000, 0.0001831054687500,
+-0.0000305175781250, 0.0000000000000000, 0.0000915527343750, -0.0000610351562500,
+-0.0000305175781250, 0.0000305175781250, 0.0000610351562500, -0.0002136230468750,
+0.0001220703125000, -0.0002746582031250, 0.0000610351562500, -0.0000915527343750,
+-0.0000305175781250, -0.0000305175781250, 0.0000305175781250, -0.0002136230468750,
+-0.0000305175781250, 0.0002441406250000, -0.0000915527343750, 0.0002746582031250,
+-0.0000305175781250, 0.0001525878906250, 0.0000305175781250, 0.0002746582031250,
+-0.0000305175781250, 0.0003356933593750, 0.0000305175781250, -0.0001831054687500,
+0.0000610351562500, -0.0001220703125000, 0.0000305175781250, -0.0001220703125000,
+-0.0000305175781250, -0.0004882812500000, 0.0000305175781250, -0.0003662109375000,
+-0.0000305175781250, -0.0000610351562500, -0.0000610351562500, -0.0002441406250000,
+-0.0000305175781250, -0.0000305175781250, 0.0000000000000000, 0.0004577636718750,
+-0.0000610351562500, 0.0002136230468750, 0.0000305175781250, 0.0003356933593750,
+0.0000610351562500, 0.0005493164062500, 0.0000610351562500, 0.0001831054687500,
+0.0000000000000000, -0.0000915527343750, 0.0000305175781250, 0.0001831054687500,
+-0.0000305175781250, -0.0004882812500000, -0.0000305175781250, -0.0005798339843750,
+-0.0000305175781250, -0.0003967285156250, 0.0000000000000000, -0.0006103515625000,
+-0.0000305175781250, -0.0006713867187500, 0.0000000000000000, -0.0000610351562500,
+0.0000305175781250, -0.0002136230468750, 0.0000000000000000, 0.0001220703125000,
+0.0000000000000000, 0.0007019042968750, 0.0000305175781250, 0.0004882812500000,
+0.0000000000000000, 0.0007629394531250, 0.0000000000000000, 0.0009460449218750,
+-0.0000305175781250, 0.0006713867187500, 0.0000000000000000, 0.0005493164062500,
+-0.0000305175781250, 0.0007019042968750, 0.0000000000000000, -0.0000915527343750,
+0.0000000000000000, 0.0000305175781250, 0.0000000000000000, -0.0002746582031250,
+0.0000610351562500, -0.0009155273437500, 0.0000000000000000, -0.0006713867187500,
+0.0000000000000000, -0.0011596679687500, 0.0000000000000000, -0.0012512207031250,
+0.0000000000000000, -0.0011901855468750, 0.0000000000000000, -0.0013732910156250,
+0.0000000000000000, -0.0013122558593750, 0.0000000000000000, -0.0010375976562500,
+-0.0000305175781250, -0.0011291503906250, -0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0004577636718750, 0.0000305175781250, -0.0005798339843750,
+0.0000000000000000, 0.0000305175781250, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, 0.0003051757812500, 0.0000000000000000, 0.0007324218750000,
+0.0000305175781250, 0.0006713867187500, 0.0000000000000000, 0.0010070800781250,
+0.0000000000000000, 0.0010986328125000, 0.0000000000000000, 0.0011901855468750,
+-0.0000305175781250, 0.0013732910156250, 0.0000000000000000, 0.0013427734375000,
+0.0000000000000000, 0.0014953613281250, 0.0000000000000000, 0.0015869140625000,
+0.0000000000000000, 0.0015563964843750, 0.0000000000000000, 0.0016174316406250,
+0.0000000000000000, 0.0016784667968750, 0.0000000000000000, 0.0016479492187500,
+0.0000000000000000, 0.0017089843750000, 0.0000000000000000, 0.0016784667968750,
+0.0000000000000000, 0.0016479492187500, 0.0000000000000000, 0.0016784667968750,
+0.0000000000000000, 0.0015258789062500, 0.0000000000000000, 0.0016479492187500,
+0.0000000000000000, 0.0015563964843750, 0.0000305175781250, 0.0014038085937500,
+0.0000000000000000, 0.0015563964843750, 0.0000000000000000, 0.0010375976562500,
+0.0000305175781250, 0.0012207031250000, 0.0000305175781250, 0.0009765625000000,
+-0.0000305175781250, 0.0005187988281250, 0.0000000000000000, 0.0008850097656250,
+-0.0000305175781250, -0.0001831054687500, 0.0000000000000000, 0.0000915527343750,
+0.0000000000000000, -0.0002441406250000, 0.0000000000000000, -0.0010986328125000,
+0.0000000000000000, -0.0005493164062500, -0.0000305175781250, -0.0016479492187500,
+0.0000305175781250, -0.0016174316406250, -0.0000305175781250, -0.0014648437500000,
+-0.0000305175781250, -0.0020141601562500, -0.0000305175781250, -0.0019226074218750,
+0.0000305175781250, -0.0014343261718750, 0.0000000000000000, -0.0018310546875000,
+0.0000305175781250, -0.0012207031250000, 0.0000000000000000, -0.0004272460937500,
+0.0000000000000000, -0.0010070800781250, 0.0000000000000000, 0.0005493164062500,
+0.0000000000000000, 0.0003967285156250, 0.0000000000000000, 0.0005798339843750,
+-0.0000305175781250, 0.0016784667968750, 0.0000000000000000, 0.0012207031250000,
+0.0000000000000000, 0.0013122558593750, 0.0000000000000000, 0.0016784667968750,
+0.0000305175781250, 0.0010986328125000, 0.0000000000000000, 0.0006408691406250,
+0.0000305175781250, 0.0010681152343750, 0.0000000000000000, -0.0003356933593750,
+-0.0000305175781250},
+{-0.0000305175781250, 0.0002746582031250, -0.0000610351562500, 0.0004577636718750,
+0.0000000000000000, 0.0000610351562500, 0.0000000000000000, -0.0005187988281250,
+-0.0000305175781250, -0.0002136230468750, 0.0000000000000000, -0.0008239746093750,
+0.0000000000000000, -0.0010070800781250, -0.0000305175781250, -0.0008239746093750,
+0.0000305175781250, -0.0008850097656250, 0.0000000000000000, -0.0009765625000000,
+0.0000000000000000, -0.0003356933593750, 0.0000000000000000, -0.0004577636718750,
+0.0000000000000000, -0.0001220703125000, 0.0000000000000000, 0.0005493164062500,
+0.0000000000000000, 0.0002746582031250, -0.0000305175781250, 0.0009765625000000,
+0.0000305175781250, 0.0010681152343750, -0.0000305175781250, 0.0010986328125000,
+-0.0000305175781250, 0.0014343261718750, 0.0000305175781250, 0.0013427734375000,
+-0.0000305175781250, 0.0013427734375000, 0.0000000000000000, 0.0014343261718750,
+0.0000000000000000, 0.0011901855468750, 0.0000000000000000, 0.0009765625000000,
+0.0000000000000000, 0.0010375976562500, -0.0000305175781250, 0.0005187988281250,
+0.0000000000000000, 0.0004577636718750, 0.0000000000000000, 0.0002746582031250,
+0.0000305175781250, -0.0001220703125000, 0.0000305175781250, -0.0000610351562500,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, -0.0006103515625000,
+0.0000000000000000, -0.0007019042968750, 0.0000305175781250, -0.0009460449218750,
+0.0000000000000000, -0.0008850097656250, 0.0000000000000000, -0.0010986328125000,
+0.0000000000000000, -0.0012512207031250, 0.0000305175781250, -0.0012207031250000,
+0.0000000000000000, -0.0012817382812500, 0.0000305175781250, -0.0013427734375000,
+0.0000000000000000, -0.0012817382812500, 0.0000000000000000, -0.0013427734375000,
+0.0000000000000000, -0.0012817382812500, 0.0000000000000000, -0.0012207031250000,
+0.0000000000000000, -0.0012817382812500, 0.0000000000000000, -0.0010986328125000,
+-0.0000305175781250, -0.0012207031250000, 0.0000000000000000, -0.0010375976562500,
+-0.0000305175781250, -0.0008239746093750, 0.0000000000000000, -0.0010375976562500,
+0.0000000000000000, -0.0003967285156250, 0.0000000000000000, -0.0005798339843750,
+0.0000000000000000, -0.0002746582031250, 0.0000000000000000, 0.0003051757812500,
+0.0000305175781250, -0.0001525878906250, 0.0000000000000000, 0.0008544921875000,
+0.0000000000000000, 0.0006408691406250, 0.0000000000000000, 0.0009155273437500,
+-0.0000305175781250, 0.0017089843750000, 0.0000000000000000, 0.0012207031250000,
+0.0000000000000000, 0.0018615722656250, 0.0000000000000000, 0.0020141601562500,
+0.0000000000000000, 0.0016479492187500, 0.0000305175781250, 0.0018615722656250,
+0.0000000000000000, 0.0020141601562500, -0.0000305175781250, 0.0009765625000000,
+0.0000000000000000, 0.0013732910156250, -0.0000305175781250, 0.0007324218750000,
+-0.0000305175781250, -0.0003662109375000, -0.0000305175781250, 0.0003051757812500,
+0.0000000000000000, -0.0011596679687500, -0.0000305175781250, -0.0011596679687500,
+0.0000610351562500, -0.0010070800781250, 0.0000305175781250, -0.0018920898437500,
+0.0000305175781250, -0.0016784667968750, 0.0000000000000000, -0.0011901855468750,
+0.0000000000000000, -0.0016174316406250, -0.0000305175781250, -0.0009460449218750,
+0.0000000000000000, -0.0000915527343750, 0.0000000000000000, -0.0007019042968750,
+0.0000000000000000, 0.0007019042968750, 0.0000000000000000, 0.0007629394531250,
+0.0000000000000000, 0.0006713867187500, 0.0000305175781250, 0.0014038085937500,
+0.0000305175781250, 0.0012512207031250, 0.0000000000000000, 0.0007934570312500,
+0.0000000000000000, 0.0010986328125000, 0.0000000000000000, 0.0005798339843750,
+-0.0000305175781250, -0.0002136230468750, -0.0000305175781250, 0.0002136230468750,
+0.0000305175781250, -0.0007629394531250, 0.0000000000000000, -0.0009460449218750,
+0.0000000000000000, -0.0006408691406250, 0.0000610351562500, -0.0009460449218750,
+0.0000305175781250, -0.0010375976562500, -0.0000305175781250, -0.0000915527343750,
+0.0000000000000000, -0.0003051757812500, 0.0000000000000000, -0.0000915527343750,
+-0.0000610351562500, 0.0006713867187500, -0.0000610351562500, 0.0003967285156250,
+0.0000305175781250, 0.0005493164062500, -0.0000305175781250, 0.0008239746093750,
+-0.0000305175781250, 0.0004272460937500, 0.0000915527343750, 0.0001525878906250,
+0.0000305175781250, 0.0003967285156250, 0.0000000000000000, -0.0003662109375000,
+0.0000305175781250, -0.0004577636718750, 0.0000305175781250, -0.0003356933593750,
+-0.0000915527343750, -0.0006408691406250, -0.0000305175781250, -0.0006408691406250,
+-0.0000305175781250, 0.0000000000000000, -0.0000915527343750, -0.0002136230468750,
+-0.0000305175781250, 0.0000000000000000, 0.0001220703125000, 0.0007629394531250,
+0.0000610351562500, 0.0004272460937500, 0.0000915527343750, 0.0003967285156250,
+0.0001525878906250, 0.0007629394531250, 0.0000610351562500, 0.0002441406250000,
+-0.0000610351562500, -0.0004272460937500, 0.0000305175781250, 0.0000000000000000,
+-0.0001525878906250, -0.0004577636718750, -0.0002136230468750, -0.0008239746093750,
+-0.0000610351562500, -0.0002441406250000, 0.0000000000000000, 0.0001220703125000,
+-0.0001220703125000, -0.0003051757812500, 0.0002136230468750, 0.0004577636718750,
+0.0002441406250000, 0.0006713867187500, 0.0001525878906250, 0.0003051757812500,
+0.0001831054687500, 0.0001831054687500, -0.0003051757812500, -0.0003356933593750,
+0.0002136230468750, 0.0004577636718750, -0.0000915527343750, 0.0005798339843750,
+0.0000915527343750, 0.0002136230468750, 0.0005187988281250, 0.0000610351562500,
+0.0003967285156250, 0.0003967285156250, 0.0000000000000000, -0.0003051757812500,
+0.0003356933593750, -0.0005187988281250, -0.0000305175781250, -0.0001525878906250,
+-0.0006408691406250, -0.0001525878906250, -0.0003051757812500, -0.0003356933593750,
+-0.0003967285156250, 0.0002441406250000, -0.0007934570312500, 0.0003051757812500,
+-0.0002136230468750, 0.0000915527343750, 0.0003967285156250, 0.0002441406250000,
+-0.0001220703125000, 0.0003662109375000, 0.0007019042968750, -0.0000915527343750,
+0.0010070800781250, -0.0001525878906250, 0.0003662109375000, -0.0000305175781250,
+0.0002746582031250, -0.0002136230468750, 0.0007019042968750, -0.0002441406250000,
+-0.0006103515625000, 0.0001525878906250, -0.0007019042968750, 0.0001525878906250,
+-0.0003356933593750, 0.0000000000000000, -0.0008850097656250, 0.0001831054687500,
+-0.0009765625000000, 0.0002136230468750, 0.0001831054687500, -0.0000610351562500,
+-0.0001220703125000, -0.0000305175781250, 0.0001220703125000, 0.0000305175781250,
+0.0012207031250000, -0.0001525878906250, 0.0007934570312500, -0.0001831054687500,
+0.0004882812500000, 0.0000305175781250, 0.0010375976562500, 0.0000305175781250,
+0.0003662109375000, 0.0000305175781250, -0.0005493164062500, 0.0001525878906250,
+0.0000610351562500, 0.0000915527343750, -0.0009460449218750, -0.0000305175781250,
+-0.0012512207031250, -0.0000305175781250, -0.0006713867187500, 0.0000000000000000,
+-0.0007324218750000, -0.0000915527343750, -0.0010375976562500, -0.0000915527343750,
+0.0003356933593750, 0.0000000000000000, 0.0001220703125000, 0.0000000000000000,
+0.0002746582031250, 0.0000000000000000, 0.0013122558593750, 0.0000610351562500,
+0.0008850097656250, 0.0000610351562500, 0.0007019042968750, -0.0000305175781250,
+0.0012512207031250, 0.0000000000000000, 0.0005187988281250, 0.0000000000000000,
+-0.0002136230468750, -0.0000305175781250, 0.0004272460937500, -0.0000610351562500,
+-0.0009155273437500, 0.0000305175781250, -0.0008850097656250, 0.0000000000000000,
+-0.0007934570312500, 0.0000305175781250, -0.0015563964843750, 0.0000305175781250,
+-0.0014038085937500, 0.0000000000000000, -0.0010375976562500, -0.0000305175781250,
+-0.0013732910156250, -0.0000305175781250, -0.0008544921875000, 0.0000305175781250,
+-0.0001831054687500, 0.0000305175781250, -0.0007019042968750, 0.0000000000000000,
+0.0007324218750000, 0.0000000000000000, 0.0005493164062500, 0.0000000000000000,
+0.0007629394531250, 0.0000000000000000, 0.0017395019531250, 0.0000000000000000,
+0.0012207031250000, 0.0000000000000000, 0.0016479492187500, 0.0000000000000000,
+0.0018920898437500, 0.0000305175781250, 0.0014953613281250, 0.0000305175781250,
+0.0013427734375000, 0.0000000000000000, 0.0016479492187500, 0.0000305175781250,
+0.0004882812500000, 0.0000000000000000, 0.0008850097656250, 0.0000305175781250,
+0.0003356933593750, 0.0000000000000000, -0.0007629394531250, 0.0000000000000000,
+0.0000305175781250, -0.0000305175781250, -0.0015563964843750, 0.0000000000000000,
+-0.0014038085937500, 0.0000305175781250, -0.0015258789062500, 0.0000305175781250,
+-0.0025329589843750, -0.0000305175781250, -0.0020141601562500, 0.0000000000000000,
+-0.0027465820312500, 0.0000000000000000, -0.0028076171875000, 0.0000000000000000,
+-0.0027160644531250, 0.0000000000000000, -0.0030517578125000, 0.0000000000000000,
+-0.0029907226562500, 0.0000000000000000, -0.0028381347656250, 0.0000305175781250,
+-0.0029907226562500, 0.0000000000000000, -0.0028381347656250, 0.0000000000000000,
+-0.0026550292968750, 0.0000000000000000, -0.0028686523437500, 0.0000000000000000,
+-0.0025634765625000, 0.0000000000000000, -0.0026855468750000, 0.0000000000000000,
+-0.0026245117187500, 0.0000000000000000, -0.0023498535156250, 0.0000305175781250,
+-0.0025939941406250, -0.0000305175781250, -0.0022888183593750, 0.0000000000000000,
+-0.0024108886718750, 0.0000305175781250, -0.0023803710937500, 0.0000305175781250,
+-0.0022277832031250, 0.0000000000000000, -0.0024108886718750, 0.0000305175781250,
+-0.0023193359375000, 0.0000000000000000, -0.0024108886718750, -0.0000305175781250,
+-0.0024719238281250, 0.0000000000000000, -0.0024414062500000, 0.0000305175781250,
+-0.0025024414062500, -0.0000305175781250, -0.0024414062500000, 0.0000000000000000,
+-0.0024719238281250, 0.0000000000000000, -0.0024108886718750, 0.0000000000000000,
+-0.0023193359375000, -0.0000305175781250, -0.0023803710937500, -0.0000305175781250,
+-0.0019226074218750, -0.0000610351562500, -0.0020751953125000, -0.0000305175781250,
+-0.0017700195312500, 0.0000000000000000, -0.0012512207031250, 0.0000000000000000,
+-0.0014953613281250, 0.0000000000000000, -0.0005187988281250, 0.0000305175781250,
+-0.0006103515625000, 0.0000305175781250, -0.0002441406250000, 0.0000305175781250,
+0.0005798339843750, 0.0000000000000000, 0.0002746582031250, 0.0000000000000000,
+0.0010375976562500, -0.0000610351562500, 0.0011901855468750, -0.0000610351562500,
+0.0010986328125000, 0.0000000000000000, 0.0014953613281250, -0.0000305175781250,
+0.0014038085937500, 0.0000000000000000, 0.0010375976562500, 0.0000305175781250,
+0.0012817382812500, 0.0000305175781250, 0.0008544921875000, 0.0000305175781250,
+0.0002441406250000, 0.0000000000000000, 0.0005493164062500, 0.0000000000000000,
+-0.0004272460937500, 0.0000000000000000, -0.0004577636718750, 0.0000000000000000,
+-0.0005798339843750, -0.0000305175781250, -0.0012207031250000, 0.0000305175781250,
+-0.0010070800781250, 0.0000000000000000, -0.0008239746093750, -0.0000305175781250,
+-0.0010986328125000, 0.0000305175781250, -0.0006408691406250, 0.0000000000000000,
+-0.0001220703125000, 0.0000000000000000, -0.0004882812500000, 0.0000000000000000,
+0.0004272460937500, 0.0000305175781250, 0.0005493164062500, 0.0000000000000000,
+0.0003967285156250, 0.0000000000000000, 0.0007934570312500, 0.0000000000000000,
+0.0007324218750000, -0.0000610351562500, 0.0002441406250000, 0.0000305175781250,
+0.0003967285156250, 0.0000610351562500, 0.0001220703125000, 0.0000000000000000,
+-0.0004577636718750},
+{-0.0017089843750000, 0.0000305175781250, -0.0020446777343750, 0.0000305175781250,
+-0.0021362304687500, 0.0000000000000000, -0.0012817382812500, 0.0000000000000000,
+-0.0017395019531250, 0.0000000000000000, -0.0011291503906250, 0.0000000000000000,
+-0.0002441406250000, 0.0000305175781250, -0.0009765625000000, 0.0000305175781250,
+0.0007629394531250, 0.0000000000000000, 0.0004577636718750, 0.0000305175781250,
+0.0007934570312500, 0.0000305175781250, 0.0019531250000000, -0.0000305175781250,
+0.0011901855468750, 0.0000000000000000, 0.0024108886718750, 0.0000000000000000,
+0.0023498535156250, 0.0000305175781250, 0.0023498535156250, 0.0000000000000000,
+0.0030212402343750, 0.0000305175781250, 0.0027160644531250, 0.0000305175781250,
+0.0030212402343750, 0.0000000000000000, 0.0031127929687500, 0.0000000000000000,
+0.0029907226562500, -0.0000305175781250, 0.0030822753906250, -0.0000305175781250,
+0.0030822753906250, 0.0000305175781250, 0.0029602050781250, 0.0000305175781250,
+0.0030212402343750, 0.0000000000000000, 0.0029296875000000, 0.0000305175781250,
+0.0028381347656250, 0.0000000000000000, 0.0029907226562500, 0.0000000000000000,
+0.0027770996093750, 0.0000000000000000, 0.0028076171875000, 0.0000305175781250,
+0.0027770996093750, 0.0000305175781250, 0.0026855468750000, 0.0000000000000000,
+0.0028076171875000, 0.0000000000000000, 0.0026855468750000, 0.0000000000000000,
+0.0027465820312500, 0.0000000000000000, 0.0026550292968750, 0.0000000000000000,
+0.0025939941406250, 0.0000000000000000, 0.0026245117187500, 0.0000000000000000,
+0.0024719238281250, -0.0000305175781250, 0.0025024414062500, 0.0000000000000000,
+0.0023193359375000, 0.0000000000000000, 0.0020751953125000, -0.0000305175781250,
+0.0021362304687500, -0.0000305175781250, 0.0014953613281250, -0.0000305175781250,
+0.0016479492187500, 0.0000000000000000, 0.0012817382812500, 0.0000000000000000,
+0.0005798339843750, 0.0000000000000000, 0.0008239746093750, 0.0000305175781250,
+-0.0002136230468750, -0.0000305175781250, -0.0001831054687500, 0.0000000000000000,
+-0.0004577636718750, 0.0000305175781250, -0.0012207031250000, 0.0000000000000000,
+-0.0010070800781250, -0.0000305175781250, -0.0014953613281250, 0.0000000000000000,
+-0.0016479492187500, -0.0000305175781250, -0.0014648437500000, 0.0000000000000000,
+-0.0014953613281250, 0.0000000000000000, -0.0015869140625000, 0.0000305175781250,
+-0.0008544921875000, 0.0000305175781250, -0.0010070800781250, 0.0000305175781250,
+-0.0006103515625000, 0.0000000000000000, 0.0001220703125000, -0.0000305175781250,
+-0.0002136230468750, 0.0000305175781250, 0.0007019042968750, 0.0000305175781250,
+0.0008544921875000, 0.0000610351562500, 0.0007629394531250, 0.0000000000000000,
+0.0011596679687500, 0.0000000000000000, 0.0010986328125000, 0.0000000000000000,
+0.0005798339843750, -0.0000305175781250, 0.0008239746093750, 0.0000000000000000,
+0.0003967285156250, 0.0000000000000000, -0.0003051757812500, -0.0000305175781250,
+0.0000305175781250, -0.0000305175781250, -0.0007019042968750, 0.0000305175781250,
+-0.0008850097656250, 0.0000305175781250, -0.0006103515625000, 0.0000000000000000,
+-0.0007934570312500, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+-0.0000610351562500, -0.0000305175781250, -0.0001525878906250, 0.0000000000000000,
+0.0000305175781250, 0.0000000000000000, 0.0006713867187500, -0.0000610351562500,
+0.0004272460937500, -0.0000305175781250, 0.0003662109375000, 0.0000915527343750,
+0.0006408691406250, 0.0000000000000000, 0.0002441406250000, 0.0000000000000000,
+-0.0001831054687500, 0.0000305175781250, 0.0000915527343750, 0.0000305175781250,
+-0.0003051757812500, -0.0000305175781250, -0.0005187988281250, 0.0000000000000000,
+-0.0002746582031250, 0.0000305175781250, -0.0000915527343750, -0.0000305175781250,
+-0.0002746582031250, -0.0000305175781250, 0.0002136230468750, 0.0000305175781250,
+0.0003356933593750, 0.0000305175781250, 0.0001220703125000, 0.0000000000000000,
+0.0002746582031250, 0.0000610351562500, 0.0003356933593750, 0.0000915527343750,
+-0.0001220703125000, 0.0000000000000000, -0.0001220703125000, 0.0000000000000000,
+-0.0000915527343750, 0.0000000000000000, -0.0003356933593750, -0.0000610351562500,
+-0.0003356933593750, -0.0000915527343750, 0.0000915527343750, 0.0000610351562500,
+-0.0000915527343750, 0.0000000000000000, -0.0000610351562500, -0.0000305175781250,
+0.0002746582031250, 0.0000610351562500, 0.0002136230468750, 0.0000610351562500,
+-0.0000305175781250, -0.0000610351562500, 0.0000915527343750, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, -0.0002136230468750, -0.0000915527343750,
+-0.0001831054687500, -0.0000915527343750, 0.0000610351562500, 0.0000610351562500,
+-0.0000610351562500, 0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+0.0001831054687500, 0.0000915527343750, 0.0001220703125000, 0.0001220703125000,
+0.0000000000000000, 0.0000000000000000, 0.0000610351562500, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, -0.0001525878906250, -0.0000915527343750,
+-0.0001220703125000, -0.0001220703125000, 0.0000915527343750, 0.0000610351562500,
+0.0000000000000000, 0.0000610351562500, 0.0000000000000000, 0.0000000000000000,
+0.0001831054687500, 0.0000915527343750, 0.0000915527343750, 0.0001220703125000,
+0.0000000000000000, -0.0001525878906250, 0.0000915527343750, -0.0000915527343750,
+0.0000305175781250, -0.0000610351562500, -0.0001525878906250, -0.0002441406250000,
+-0.0000915527343750, -0.0002441406250000, 0.0000000000000000, 0.0001220703125000,
+-0.0000305175781250, 0.0000305175781250, 0.0000000000000000, 0.0000610351562500,
+0.0001220703125000, 0.0003662109375000, 0.0000915527343750, 0.0002746582031250,
+0.0000000000000000, -0.0001220703125000, 0.0000305175781250, 0.0001525878906250,
+0.0000000000000000, 0.0000000000000000, -0.0000610351562500, -0.0004882812500000,
+-0.0000610351562500, -0.0002746582031250, 0.0000000000000000, -0.0001831054687500,
+0.0000000000000000, -0.0004272460937500, -0.0000305175781250, -0.0001220703125000,
+0.0000610351562500, 0.0003051757812500, 0.0000305175781250, 0.0000305175781250,
+0.0000305175781250, 0.0003356933593750, 0.0000610351562500, 0.0005798339843750,
+0.0000000000000000, 0.0002441406250000, -0.0000305175781250, 0.0000610351562500,
+-0.0000305175781250, 0.0002746582031250, -0.0000305175781250, -0.0004272460937500,
+-0.0000610351562500, -0.0004882812500000, 0.0000000000000000, -0.0003662109375000,
+0.0000610351562500, -0.0006408691406250, 0.0000305175781250, -0.0006713867187500,
+0.0000000000000000, -0.0000610351562500, 0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, 0.0000610351562500, -0.0000305175781250, 0.0007324218750000,
+-0.0000305175781250, 0.0004577636718750, 0.0000000000000000, 0.0005798339843750,
+-0.0000305175781250, 0.0008850097656250, 0.0000000000000000, 0.0004882812500000,
+0.0000305175781250, 0.0001220703125000, 0.0000000000000000, 0.0003662109375000,
+0.0000305175781250, -0.0005493164062500, 0.0000000000000000, -0.0005493164062500,
+0.0000000000000000, -0.0006713867187500, 0.0000000000000000, -0.0012512207031250,
+0.0000000000000000, -0.0010681152343750, -0.0000305175781250, -0.0009460449218750,
+0.0000000000000000, -0.0012207031250000, -0.0000305175781250, -0.0008544921875000,
+0.0000305175781250, -0.0004577636718750, 0.0000000000000000, -0.0006713867187500,
+0.0000000000000000, 0.0003051757812500, 0.0000305175781250, 0.0002441406250000,
+0.0000000000000000, 0.0005187988281250, -0.0000305175781250, 0.0012207031250000,
+0.0000000000000000, 0.0009765625000000, 0.0000000000000000, 0.0015258789062500,
+0.0000000000000000, 0.0015869140625000, 0.0000000000000000, 0.0015258789062500,
+0.0000000000000000, 0.0017395019531250, 0.0000000000000000, 0.0017089843750000,
+0.0000305175781250, 0.0016174316406250, 0.0000000000000000, 0.0015869140625000,
+0.0000000000000000, 0.0014343261718750, 0.0000000000000000, 0.0011901855468750,
+0.0000305175781250, 0.0012817382812500, 0.0000000000000000, 0.0009460449218750,
+0.0000000000000000, 0.0008239746093750, 0.0000305175781250, 0.0007019042968750,
+0.0000000000000000, 0.0004882812500000, 0.0000000000000000, 0.0005187988281250,
+0.0000000000000000, 0.0003356933593750, 0.0000000000000000, 0.0001220703125000,
+0.0000000000000000, 0.0001525878906250, -0.0000305175781250, 0.0001220703125000,
+0.0000000000000000, -0.0000305175781250, 0.0000305175781250, 0.0001831054687500,
+0.0000000000000000, 0.0000610351562500, 0.0000000000000000, 0.0001220703125000,
+0.0000000000000000, 0.0002746582031250, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, 0.0005187988281250, 0.0000000000000000, 0.0003662109375000,
+0.0000000000000000, 0.0005493164062500, 0.0000000000000000, 0.0008850097656250,
+0.0000000000000000, 0.0005798339843750, 0.0000000000000000, 0.0012817382812500,
+0.0000000000000000, 0.0011291503906250, 0.0000000000000000, 0.0013122558593750,
+-0.0000305175781250, 0.0018310546875000, 0.0000000000000000, 0.0014953613281250,
+0.0000000000000000, 0.0021667480468750, 0.0000000000000000, 0.0021362304687500,
+-0.0000305175781250, 0.0019836425781250, 0.0000000000000000, 0.0023803710937500,
+0.0000000000000000, 0.0023193359375000, -0.0000305175781250, 0.0018005371093750,
+0.0000000000000000, 0.0021972656250000, -0.0000305175781250, 0.0015563964843750,
+0.0000000000000000, 0.0008544921875000, -0.0000305175781250, 0.0014953613281250,
+0.0000000000000000, -0.0002746582031250, 0.0000000000000000, -0.0000610351562500,
+-0.0000305175781250, -0.0003967285156250, -0.0000610351562500, -0.0016784667968750,
+0.0000305175781250, -0.0011291503906250, 0.0000000000000000, -0.0018005371093750,
+-0.0000305175781250, -0.0020751953125000, 0.0000000000000000, -0.0015258789062500,
+0.0000000000000000, -0.0014953613281250, 0.0000000000000000, -0.0018310546875000,
+0.0000000000000000, -0.0003356933593750, 0.0000305175781250, -0.0006713867187500,
+0.0000000000000000, -0.0001525878906250, -0.0000305175781250, 0.0011596679687500,
+0.0000000000000000, 0.0004882812500000, 0.0000000000000000, 0.0013427734375000,
+-0.0000305175781250, 0.0016784667968750, 0.0000000000000000, 0.0011596679687500,
+0.0000305175781250, 0.0010986328125000, 0.0000305175781250, 0.0013732910156250,
+-0.0000305175781250, 0.0000610351562500, 0.0000000000000000, 0.0002441406250000,
+-0.0000305175781250, -0.0000915527343750, -0.0000610351562500, -0.0010986328125000,
+-0.0000305175781250, -0.0007019042968750, 0.0000000000000000, -0.0010070800781250,
+0.0000000000000000, -0.0014038085937500, 0.0000000000000000, -0.0007934570312500,
+0.0000610351562500, -0.0003967285156250, 0.0000305175781250, -0.0007629394531250,
+0.0000610351562500, 0.0003662109375000, 0.0000305175781250, 0.0004577636718750,
+0.0000305175781250, 0.0003051757812500, -0.0000305175781250, 0.0008544921875000,
+-0.0000610351562500, 0.0007934570312500, 0.0000000000000000, 0.0003051757812500,
+-0.0000915527343750, 0.0005493164062500, 0.0000000000000000, 0.0001525878906250,
+0.0000915527343750, -0.0005493164062500, 0.0000000000000000, -0.0001220703125000,
+0.0000915527343750, -0.0006408691406250, 0.0001220703125000, -0.0008850097656250,
+0.0000610351562500, -0.0004882812500000, -0.0000305175781250, -0.0003356933593750,
+0.0000000000000000, -0.0005798339843750, -0.0000915527343750, 0.0003967285156250,
+-0.0001525878906250, 0.0003967285156250, -0.0000305175781250, 0.0002746582031250,
+-0.0000305175781250, 0.0008544921875000, -0.0000915527343750, 0.0007629394531250,
+0.0001525878906250},
+{-0.0000305175781250, -0.0013122558593750, -0.0000305175781250, -0.0011291503906250,
+0.0000000000000000, -0.0009765625000000, 0.0000000000000000, -0.0009765625000000,
+0.0000305175781250, -0.0009765625000000, 0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0010681152343750, -0.0000305175781250, -0.0009460449218750,
+0.0000000000000000, -0.0009765625000000, 0.0000000000000000, -0.0011901855468750,
+0.0000305175781250, -0.0010986328125000, 0.0000000000000000, -0.0013732910156250,
+0.0000000000000000, -0.0012512207031250, 0.0000000000000000, -0.0014343261718750,
+0.0000000000000000, -0.0017089843750000, 0.0000000000000000, -0.0014343261718750,
+0.0000000000000000, -0.0021057128906250, -0.0000305175781250, -0.0019836425781250,
+0.0000000000000000, -0.0020446777343750, 0.0000305175781250, -0.0024414062500000,
+0.0000000000000000, -0.0022277832031250, 0.0000000000000000, -0.0024719238281250,
+0.0000000000000000, -0.0026245117187500, 0.0000305175781250, -0.0023803710937500,
+0.0000000000000000, -0.0023803710937500, 0.0000305175781250, -0.0025329589843750,
+-0.0000305175781250, -0.0014648437500000, 0.0000000000000000, -0.0018920898437500,
+-0.0000305175781250, -0.0013427734375000, 0.0000000000000000, -0.0002441406250000,
+0.0000000000000000, -0.0010681152343750, 0.0000000000000000, 0.0007324218750000,
+0.0000000000000000, 0.0007324218750000, -0.0000305175781250, 0.0008239746093750,
+0.0000000000000000, 0.0019836425781250, 0.0000000000000000, 0.0015258789062500,
+0.0000000000000000, 0.0016479492187500, 0.0000000000000000, 0.0020751953125000,
+0.0000000000000000, 0.0013122558593750, 0.0000000000000000, 0.0008239746093750,
+-0.0000305175781250, 0.0013427734375000, 0.0000000000000000, -0.0004272460937500,
+0.0000000000000000, -0.0001831054687500, 0.0000000000000000, -0.0004577636718750,
+0.0000000000000000, -0.0017700195312500, 0.0000000000000000, -0.0012207031250000,
+0.0000000000000000, -0.0015258789062500, 0.0000000000000000, -0.0019226074218750,
+0.0000000000000000, -0.0012817382812500, -0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0012207031250000, -0.0000305175781250, 0.0002136230468750,
+-0.0000305175781250, 0.0001831054687500, -0.0000305175781250, 0.0003356933593750,
+0.0000305175781250, 0.0013122558593750, 0.0000305175781250, 0.0010375976562500,
+0.0000305175781250, 0.0007934570312500, 0.0000610351562500, 0.0012207031250000,
+0.0000000000000000, 0.0006408691406250, -0.0000610351562500, -0.0001525878906250,
+-0.0000305175781250, 0.0002746582031250, -0.0000305175781250, -0.0007019042968750,
+-0.0000915527343750, -0.0008850097656250, 0.0000305175781250, -0.0006408691406250,
+0.0000915527343750, -0.0009155273437500, 0.0000000000000000, -0.0009460449218750,
+0.0000610351562500, -0.0001220703125000, 0.0000915527343750, -0.0003662109375000,
+0.0000000000000000, -0.0000610351562500, -0.0000305175781250, 0.0006713867187500,
+0.0000305175781250, 0.0003662109375000, -0.0000915527343750, 0.0005493164062500,
+-0.0001220703125000, 0.0008544921875000, 0.0000000000000000, 0.0003356933593750,
+0.0000000000000000, -0.0000610351562500, -0.0000915527343750, 0.0003662109375000,
+0.0001220703125000, -0.0005798339843750, 0.0001525878906250, -0.0007324218750000,
+0.0000610351562500, -0.0004272460937500, 0.0000305175781250, -0.0007324218750000,
+0.0001831054687500, -0.0008544921875000, -0.0000915527343750, 0.0002746582031250,
+-0.0001220703125000, 0.0001220703125000, -0.0000610351562500, 0.0001220703125000,
+-0.0002136230468750, 0.0009460449218750, -0.0001831054687500, 0.0007629394531250,
+0.0000610351562500, -0.0000305175781250, 0.0000305175781250, 0.0003356933593750,
+0.0000610351562500, 0.0000610351562500, 0.0003356933593750, -0.0009155273437500,
+0.0002746582031250, -0.0005493164062500, 0.0000000000000000, -0.0003051757812500,
+0.0001220703125000, -0.0007019042968750, -0.0000305175781250, -0.0003356933593750,
+-0.0004882812500000, 0.0007629394531250, 0.0001220703125000, -0.0003967285156250,
+0.0002441406250000, -0.0001525878906250, 0.0005493164062500, -0.0007019042968750,
+0.0000915527343750, -0.0001831054687500, -0.0003356933593750, 0.0006103515625000,
+0.0000610351562500, 0.0002746582031250, -0.0004577636718750, 0.0001831054687500,
+-0.0007629394531250, 0.0006103515625000, -0.0002746582031250, 0.0001525878906250,
+-0.0000915527343750, -0.0004272460937500, -0.0004272460937500, -0.0001220703125000,
+0.0006408691406250, -0.0002441406250000, 0.0007324218750000, -0.0005798339843750,
+0.0003356933593750, -0.0001831054687500, 0.0007324218750000, 0.0002746582031250,
+0.0009155273437500, 0.0000000000000000, -0.0003967285156250, 0.0001831054687500,
+-0.0001525878906250, 0.0004272460937500, -0.0002441406250000, 0.0001525878906250,
+-0.0012817382812500, -0.0001525878906250, -0.0010070800781250, 0.0000915527343750,
+-0.0001525878906250, -0.0001220703125000, -0.0007019042968750, -0.0003051757812500,
+-0.0001525878906250, -0.0000610351562500, 0.0010986328125000, 0.0000915527343750,
+0.0005493164062500, -0.0000915527343750, 0.0007019042968750, 0.0001220703125000,
+0.0012817382812500, 0.0002746582031250, 0.0004577636718750, 0.0000610351562500,
+-0.0003967285156250, -0.0000305175781250, 0.0003356933593750, 0.0000915527343750,
+-0.0010375976562500, -0.0001220703125000, -0.0013427734375000, -0.0001831054687500,
+-0.0007019042968750, -0.0000305175781250, -0.0009460449218750, 0.0000610351562500,
+-0.0012512207031250, -0.0000610351562500, 0.0003967285156250, 0.0001220703125000,
+0.0001831054687500, 0.0001220703125000, 0.0002441406250000, 0.0000610351562500,
+0.0013732910156250, 0.0000610351562500, 0.0009765625000000, 0.0000610351562500,
+0.0006408691406250, -0.0000610351562500, 0.0011901855468750, -0.0001220703125000,
+0.0004272460937500, -0.0000305175781250, -0.0004882812500000, 0.0000000000000000,
+0.0002136230468750, -0.0000610351562500, -0.0011291503906250, 0.0000610351562500,
+-0.0013122558593750, 0.0000915527343750, -0.0009460449218750, 0.0000610351562500,
+-0.0014343261718750, 0.0000000000000000, -0.0015258789062500, 0.0000305175781250,
+-0.0003662109375000, -0.0000305175781250, -0.0007934570312500, 0.0000000000000000,
+-0.0003051757812500, -0.0000305175781250, 0.0008239746093750, 0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, 0.0013122558593750, 0.0000305175781250,
+0.0014343261718750, 0.0000305175781250, 0.0012207031250000, 0.0000305175781250,
+0.0016479492187500, 0.0000000000000000, 0.0016479492187500, 0.0000000000000000,
+0.0008239746093750, 0.0000000000000000, 0.0012207031250000, -0.0000305175781250,
+0.0006103515625000, 0.0000000000000000, -0.0003356933593750, 0.0000000000000000,
+0.0003662109375000, 0.0000000000000000, -0.0012512207031250, 0.0000305175781250,
+-0.0010986328125000, 0.0000305175781250, -0.0012512207031250, 0.0000305175781250,
+-0.0023193359375000, 0.0000000000000000, -0.0018005371093750, 0.0000000000000000,
+-0.0021362304687500, -0.0000305175781250, -0.0024719238281250, 0.0000000000000000,
+-0.0019836425781250, 0.0000305175781250, -0.0018310546875000, 0.0000305175781250,
+-0.0022583007812500, 0.0000610351562500, -0.0009460449218750, 0.0000000000000000,
+-0.0013732910156250, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+0.0001831054687500, 0.0000000000000000, -0.0006408691406250, -0.0000305175781250,
+0.0010070800781250, 0.0000000000000000, 0.0007019042968750, 0.0000610351562500,
+0.0009460449218750, 0.0000000000000000, 0.0020751953125000, 0.0000000000000000,
+0.0014038085937500, 0.0000305175781250, 0.0023498535156250, -0.0000305175781250,
+0.0022583007812500, 0.0000000000000000, 0.0022583007812500, 0.0000000000000000,
+0.0027770996093750, 0.0000000000000000, 0.0024414062500000, 0.0000000000000000,
+0.0028076171875000, 0.0000305175781250, 0.0027160644531250, 0.0000305175781250,
+0.0026855468750000, 0.0000000000000000, 0.0028381347656250, 0.0000305175781250,
+0.0026855468750000, -0.0000305175781250, 0.0027465820312500, -0.0000305175781250,
+0.0027160644531250, 0.0000000000000000, 0.0026550292968750, 0.0000305175781250,
+0.0027160644531250, 0.0000000000000000, 0.0026245117187500, 0.0000000000000000,
+0.0025024414062500, 0.0000000000000000, 0.0024719238281250, 0.0000000000000000,
+0.0023193359375000, -0.0000305175781250, 0.0021667480468750, -0.0000305175781250,
+0.0021057128906250, 0.0000000000000000, 0.0016784667968750, 0.0000305175781250,
+0.0017089843750000, -0.0000305175781250, 0.0014038085937500, -0.0000305175781250,
+0.0009460449218750, 0.0000000000000000, 0.0010986328125000, -0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, 0.0003051757812500, 0.0000000000000000,
+-0.0000610351562500, 0.0000305175781250, -0.0007934570312500, 0.0000000000000000,
+-0.0005493164062500, 0.0000000000000000, -0.0013427734375000, 0.0000305175781250,
+-0.0014343261718750, 0.0000000000000000, -0.0014343261718750, 0.0000000000000000,
+-0.0018920898437500, -0.0000305175781250, -0.0018005371093750, 0.0000000000000000,
+-0.0015869140625000, -0.0000305175781250, -0.0018005371093750, 0.0000000000000000,
+-0.0013732910156250, 0.0000305175781250, -0.0008544921875000, 0.0000305175781250,
+-0.0011596679687500, 0.0000000000000000, 0.0000000000000000, 0.0000305175781250,
+-0.0000610351562500, 0.0000000000000000, 0.0002136230468750, -0.0000305175781250,
+0.0010070800781250, 0.0000000000000000, 0.0007019042968750, 0.0000000000000000,
+0.0011291503906250, 0.0000305175781250, 0.0013732910156250, 0.0000000000000000,
+0.0009765625000000, -0.0000305175781250, 0.0008544921875000, 0.0000305175781250,
+0.0010375976562500, 0.0000305175781250, -0.0000305175781250, -0.0000305175781250,
+0.0000915527343750, 0.0000305175781250, -0.0001831054687500, 0.0000000000000000,
+-0.0010070800781250, -0.0000305175781250, -0.0006713867187500, -0.0000305175781250,
+-0.0008239746093750, 0.0000000000000000, -0.0011596679687500, -0.0000305175781250,
+-0.0007019042968750, 0.0000305175781250, -0.0002746582031250, 0.0000305175781250,
+-0.0005798339843750, 0.0000305175781250, 0.0003356933593750, 0.0000000000000000,
+0.0004272460937500, 0.0000000000000000, 0.0003051757812500, 0.0000305175781250,
+0.0007019042968750, -0.0000610351562500, 0.0006713867187500, -0.0000610351562500,
+0.0000915527343750, 0.0000000000000000, 0.0002746582031250, 0.0000305175781250,
+0.0000915527343750, 0.0000000000000000, -0.0006103515625000, 0.0000610351562500,
+-0.0003051757812500, 0.0000610351562500, -0.0002746582031250, -0.0000305175781250,
+-0.0005798339843750, 0.0000305175781250, -0.0002136230468750, 0.0000000000000000,
+0.0002136230468750, -0.0000610351562500, -0.0000915527343750, 0.0000000000000000,
+0.0003051757812500, 0.0000000000000000, 0.0005187988281250, -0.0000610351562500,
+0.0001525878906250, 0.0000000000000000, -0.0000305175781250, 0.0001220703125000,
+0.0001831054687500, 0.0000610351562500, -0.0002746582031250, 0.0000610351562500,
+-0.0003967285156250, 0.0000915527343750, -0.0001831054687500, 0.0000305175781250,
+-0.0002136230468750, -0.0000915527343750, -0.0003051757812500, -0.0000305175781250,
+0.0001831054687500, -0.0000305175781250, 0.0002136230468750, -0.0000915527343750,
+0.0000915527343750, -0.0000305175781250, 0.0002441406250000, 0.0001220703125000,
+0.0003051757812500, 0.0000305175781250, -0.0001220703125000, 0.0000000000000000,
+-0.0000915527343750, 0.0000610351562500, 0.0000000000000000, 0.0000305175781250,
+-0.0001525878906250, -0.0001220703125000, -0.0001525878906250, -0.0001220703125000,
+0.0001220703125000, 0.0000000000000000, 0.0000915527343750, -0.0000610351562500,
+0.0000305175781250},
+{-0.0020446777343750, 0.0000305175781250, -0.0018615722656250, 0.0000000000000000,
+-0.0016479492187500, -0.0000305175781250, -0.0015563964843750, 0.0000305175781250,
+-0.0011596679687500, 0.0000000000000000, -0.0011901855468750, 0.0000000000000000,
+-0.0008239746093750, 0.0000000000000000, -0.0003967285156250, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, 0.0003662109375000, -0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, 0.0005798339843750, 0.0000000000000000,
+0.0012207031250000, 0.0000305175781250, 0.0010070800781250, 0.0000000000000000,
+0.0015869140625000, 0.0000305175781250, 0.0016784667968750, 0.0000000000000000,
+0.0015258789062500, 0.0000305175781250, 0.0017089843750000, 0.0000000000000000,
+0.0018005371093750, -0.0000305175781250, 0.0012207031250000, -0.0000305175781250,
+0.0014038085937500, 0.0000305175781250, 0.0009460449218750, 0.0000305175781250,
+0.0002746582031250, 0.0000000000000000, 0.0006103515625000, 0.0000000000000000,
+-0.0005493164062500, -0.0000305175781250, -0.0005187988281250, 0.0000000000000000,
+-0.0006713867187500, -0.0000610351562500, -0.0014038085937500, 0.0000000000000000,
+-0.0011901855468750, 0.0000305175781250, -0.0011901855468750, 0.0000305175781250,
+-0.0014953613281250, 0.0000000000000000, -0.0010375976562500, 0.0000000000000000,
+-0.0006103515625000, -0.0000305175781250, -0.0009155273437500, 0.0000000000000000,
+0.0002441406250000, 0.0000000000000000, 0.0002746582031250, -0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, 0.0010986328125000, 0.0000305175781250,
+0.0008544921875000, 0.0000305175781250, 0.0006713867187500, 0.0000305175781250,
+0.0009460449218750, 0.0000000000000000, 0.0004882812500000, -0.0000305175781250,
+-0.0001220703125000, 0.0000000000000000, 0.0001831054687500, 0.0000000000000000,
+-0.0006103515625000, 0.0000000000000000, -0.0007934570312500, -0.0000610351562500,
+-0.0005187988281250, 0.0000305175781250, -0.0006713867187500, 0.0000610351562500,
+-0.0007629394531250, 0.0000000000000000, 0.0000305175781250, 0.0000610351562500,
+-0.0000915527343750, 0.0000610351562500, 0.0000000000000000, 0.0000000000000000,
+0.0006408691406250, -0.0000610351562500, 0.0004577636718750, -0.0000305175781250,
+0.0001831054687500, 0.0000000000000000, 0.0004577636718750, -0.0000610351562500,
+0.0001525878906250, 0.0000000000000000, -0.0004272460937500, 0.0000610351562500,
+-0.0001220703125000, 0.0000305175781250, -0.0003051757812500, 0.0000610351562500,
+-0.0005493164062500, 0.0000610351562500, -0.0001831054687500, 0.0000305175781250,
+0.0001525878906250, -0.0000915527343750, -0.0001220703125000, -0.0000305175781250,
+0.0002746582031250, -0.0000305175781250, 0.0004882812500000, -0.0001220703125000,
+0.0001220703125000, 0.0000000000000000, 0.0000000000000000, 0.0001220703125000,
+0.0002441406250000, -0.0000305175781250, -0.0002136230468750, 0.0000305175781250,
+-0.0003662109375000, 0.0001220703125000, -0.0001220703125000, 0.0000000000000000,
+-0.0001220703125000, -0.0000915527343750, -0.0002441406250000, -0.0000305175781250,
+0.0001525878906250, -0.0000610351562500, 0.0001831054687500, -0.0000915527343750,
+0.0000610351562500, -0.0000305175781250, 0.0000915527343750, 0.0000915527343750,
+0.0000610351562500, 0.0000610351562500, -0.0001525878906250, 0.0000305175781250,
+-0.0001525878906250, 0.0000610351562500, 0.0000000000000000, 0.0000000000000000,
+-0.0000305175781250, -0.0001220703125000, -0.0001831054687500, -0.0000305175781250,
+0.0001220703125000, 0.0000000000000000, 0.0001525878906250, -0.0001220703125000,
+0.0000000000000000, -0.0000305175781250, 0.0000610351562500, 0.0001220703125000,
+0.0001220703125000, 0.0000610351562500, -0.0000915527343750, 0.0000305175781250,
+-0.0001525878906250, 0.0001220703125000, -0.0000305175781250, 0.0000000000000000,
+-0.0000305175781250, -0.0002136230468750, -0.0001220703125000, -0.0001220703125000,
+0.0001220703125000, -0.0000915527343750, 0.0001525878906250, -0.0001831054687500,
+0.0000610351562500, 0.0000000000000000, 0.0000305175781250, 0.0002746582031250,
+0.0001220703125000, 0.0000610351562500, -0.0000915527343750, 0.0001220703125000,
+-0.0001220703125000, 0.0003356933593750, -0.0000305175781250, 0.0000305175781250,
+-0.0000610351562500, -0.0003662109375000, -0.0000915527343750, -0.0000915527343750,
+0.0000915527343750, -0.0002746582031250, 0.0001220703125000, -0.0005493164062500,
+0.0000000000000000, -0.0001831054687500, 0.0000915527343750, 0.0000915527343750,
+0.0001220703125000, -0.0001525878906250, -0.0000610351562500, 0.0003356933593750,
+-0.0000610351562500, 0.0005187988281250, 0.0000000000000000, 0.0002136230468750,
+-0.0000610351562500, 0.0002746582031250, -0.0000915527343750, 0.0004272460937500,
+0.0000915527343750, -0.0003662109375000, 0.0000305175781250, -0.0003356933593750,
+0.0000000000000000, -0.0002136230468750, 0.0000610351562500, -0.0007019042968750,
+0.0000915527343750, -0.0006713867187500, 0.0000000000000000, -0.0001220703125000,
+-0.0000305175781250, -0.0003356933593750, 0.0000000000000000, 0.0000000000000000,
+-0.0000305175781250, 0.0006408691406250, -0.0000305175781250, 0.0003051757812500,
+0.0000000000000000, 0.0005493164062500, 0.0000000000000000, 0.0007934570312500,
+0.0000000000000000, 0.0003356933593750, 0.0000305175781250, -0.0000305175781250,
+0.0000305175781250, 0.0002441406250000, -0.0000305175781250, -0.0006408691406250,
+0.0000000000000000, -0.0007629394531250, -0.0000305175781250, -0.0007019042968750,
+-0.0000305175781250, -0.0010986328125000, 0.0000000000000000, -0.0010070800781250,
+-0.0000305175781250, -0.0004882812500000, 0.0000000000000000, -0.0007324218750000,
+0.0000000000000000, -0.0002746582031250, 0.0000305175781250, 0.0003662109375000,
+0.0000305175781250, 0.0000610351562500, 0.0000305175781250, 0.0008544921875000,
+0.0000305175781250, 0.0010070800781250, 0.0000000000000000, 0.0009155273437500,
+-0.0000305175781250, 0.0012512207031250, 0.0000000000000000, 0.0012207031250000,
+0.0000305175781250, 0.0008544921875000, 0.0000305175781250, 0.0010375976562500,
+0.0000305175781250, 0.0006713867187500, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, 0.0003967285156250, 0.0000000000000000, -0.0004577636718750,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, -0.0007324218750000,
+-0.0000305175781250, -0.0013122558593750, 0.0000305175781250, -0.0011596679687500,
+0.0000000000000000, -0.0017089843750000, 0.0000305175781250, -0.0017700195312500,
+0.0000610351562500, -0.0018310546875000, 0.0000305175781250, -0.0021057128906250,
+0.0000305175781250, -0.0019836425781250, -0.0000305175781250, -0.0020751953125000,
+0.0000305175781250, -0.0020446777343750, 0.0000305175781250, -0.0020141601562500,
+0.0000000000000000, -0.0020751953125000, -0.0000305175781250, -0.0020141601562500,
+0.0000000000000000, -0.0019836425781250, -0.0000305175781250, -0.0018615722656250,
+0.0000000000000000, -0.0018615722656250, 0.0000000000000000, -0.0018615722656250,
+0.0000305175781250, -0.0017700195312500, 0.0000000000000000, -0.0019531250000000,
+0.0000305175781250, -0.0018920898437500, 0.0000000000000000, -0.0018920898437500,
+-0.0000305175781250, -0.0020751953125000, 0.0000000000000000, -0.0019836425781250,
+0.0000000000000000, -0.0021972656250000, 0.0000305175781250, -0.0021057128906250,
+0.0000000000000000, -0.0021972656250000, -0.0000610351562500, -0.0024108886718750,
+0.0000305175781250, -0.0022277832031250, 0.0000305175781250, -0.0024108886718750,
+0.0000000000000000, -0.0025024414062500, 0.0000000000000000, -0.0024414062500000,
+0.0000305175781250, -0.0024719238281250, 0.0000000000000000, -0.0025024414062500,
+0.0000000000000000, -0.0020141601562500, 0.0000000000000000, -0.0023498535156250,
+-0.0000305175781250, -0.0018310546875000, 0.0000000000000000, -0.0012817382812500,
+0.0000000000000000, -0.0018005371093750, -0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, -0.0005798339843750, 0.0000305175781250, -0.0001220703125000,
+0.0000000000000000, 0.0011596679687500, 0.0000000000000000, 0.0004272460937500,
+-0.0000305175781250, 0.0016174316406250, -0.0000305175781250, 0.0018615722656250,
+0.0000000000000000, 0.0014648437500000, -0.0000305175781250, 0.0018920898437500,
+0.0000000000000000, 0.0019531250000000, 0.0000000000000000, 0.0008544921875000,
+0.0000305175781250, 0.0013427734375000, 0.0000000000000000, 0.0006408691406250,
+0.0000000000000000, -0.0006408691406250, 0.0000305175781250, 0.0001220703125000,
+-0.0000305175781250, -0.0014038085937500, -0.0000305175781250, -0.0014648437500000,
+0.0000000000000000, -0.0012512207031250, 0.0000305175781250, -0.0019226074218750,
+0.0000000000000000, -0.0018310546875000, 0.0000305175781250, -0.0010070800781250,
+0.0000305175781250, -0.0014038085937500, 0.0000000000000000, -0.0007934570312500,
+0.0000000000000000, 0.0003051757812500, 0.0000305175781250, -0.0002441406250000,
+-0.0000305175781250, 0.0009765625000000, -0.0000305175781250, 0.0012207031250000,
+0.0000000000000000, 0.0009155273437500, 0.0000305175781250, 0.0012512207031250,
+-0.0000305175781250, 0.0013122558593750, 0.0000305175781250, 0.0002136230468750,
+0.0000610351562500, 0.0005493164062500, 0.0000305175781250, 0.0001220703125000,
+0.0000000000000000, -0.0010070800781250, 0.0000610351562500, -0.0005493164062500,
+-0.0000610351562500, -0.0009155273437500, -0.0000915527343750, -0.0012817382812500,
+-0.0000305175781250, -0.0007629394531250, 0.0000000000000000, -0.0003662109375000,
+-0.0000915527343750, -0.0007629394531250, 0.0000915527343750, 0.0003356933593750,
+0.0000915527343750, 0.0003356933593750, 0.0000305175781250, 0.0002746582031250,
+0.0000610351562500, 0.0008850097656250, 0.0000915527343750, 0.0007629394531250,
+-0.0000915527343750, 0.0002746582031250, -0.0000915527343750, 0.0006103515625000,
+-0.0000305175781250, 0.0001220703125000, -0.0001220703125000, -0.0006713867187500,
+-0.0001525878906250, -0.0002746582031250, 0.0000915527343750, -0.0006713867187500,
+0.0000305175781250, -0.0010375976562500, 0.0000305175781250, -0.0004882812500000,
+0.0001831054687500, -0.0001220703125000, 0.0001831054687500, -0.0005187988281250,
+-0.0000305175781250, 0.0006408691406250, 0.0000305175781250, 0.0008239746093750,
+-0.0000305175781250, 0.0003356933593750, -0.0002441406250000, 0.0006103515625000,
+-0.0001831054687500, 0.0008544921875000, -0.0000610351562500, -0.0004882812500000,
+-0.0001525878906250, -0.0004272460937500, 0.0000000000000000, -0.0002441406250000,
+0.0003356933593750, -0.0009155273437500, 0.0001220703125000, -0.0009460449218750,
+0.0001831054687500, 0.0001220703125000, 0.0003662109375000, -0.0000915527343750,
+0.0001525878906250, -0.0000915527343750, -0.0003051757812500, 0.0010070800781250,
+-0.0002441406250000, -0.0008850097656250, 0.0004882812500000, 0.0003051757812500,
+0.0004882812500000, -0.0001831054687500, 0.0001525878906250, -0.0000305175781250,
+0.0001220703125000, 0.0007934570312500, 0.0003967285156250, 0.0006713867187500,
+-0.0005493164062500, -0.0001220703125000, -0.0005493164062500, 0.0001525878906250,
+-0.0003051757812500, 0.0000610351562500, -0.0007629394531250, -0.0007324218750000,
+-0.0007934570312500, -0.0005187988281250, 0.0002746582031250, -0.0000610351562500,
+0.0000305175781250, -0.0002746582031250, 0.0001831054687500, -0.0000915527343750,
+0.0011291503906250, 0.0005798339843750, 0.0008239746093750, 0.0003356933593750,
+0.0001831054687500, 0.0000915527343750, 0.0007324218750000, 0.0003662109375000,
+0.0000915527343750, 0.0001525878906250, -0.0012207031250000, -0.0003662109375000,
+-0.0004882812500000},
+{0.0000000000000000, -0.0015869140625000, 0.0000000000000000, -0.0018615722656250,
+-0.0000305175781250, -0.0004577636718750, 0.0000000000000000, -0.0008239746093750,
+0.0000305175781250, -0.0002136230468750, 0.0000305175781250, 0.0010986328125000,
+0.0000000000000000, 0.0004272460937500, 0.0000305175781250, 0.0014953613281250,
+0.0000305175781250, 0.0017700195312500, 0.0000000000000000, 0.0012817382812500,
+0.0000000000000000, 0.0015563964843750, 0.0000305175781250, 0.0017089843750000,
+-0.0000305175781250, 0.0003662109375000, -0.0000305175781250, 0.0007629394531250,
+0.0000000000000000, 0.0002441406250000, 0.0000000000000000, -0.0009765625000000,
+0.0000000000000000, -0.0004577636718750, 0.0000305175781250, -0.0012817382812500,
+0.0000305175781250, -0.0016174316406250, 0.0000305175781250, -0.0011596679687500,
+0.0000000000000000, -0.0010986328125000, 0.0000610351562500, -0.0013427734375000,
+-0.0000305175781250, 0.0000305175781250, -0.0000610351562500, -0.0001525878906250,
+-0.0000305175781250, 0.0000610351562500, -0.0000305175781250, 0.0011291503906250,
+-0.0000610351562500, 0.0007934570312500, 0.0000915527343750, 0.0006713867187500,
+0.0000610351562500, 0.0010986328125000, 0.0000305175781250, 0.0004882812500000,
+0.0000305175781250, -0.0001525878906250, 0.0000610351562500, 0.0003356933593750,
+-0.0000610351562500, -0.0006408691406250, -0.0000610351562500, -0.0007629394531250,
+-0.0000305175781250, -0.0005798339843750, -0.0000915527343750, -0.0009765625000000,
+-0.0000915527343750, -0.0009155273437500, 0.0000610351562500, -0.0001525878906250,
+0.0000000000000000, -0.0004272460937500, 0.0000305175781250, -0.0000610351562500,
+0.0001831054687500, 0.0008239746093750, 0.0001220703125000, 0.0004272460937500,
+-0.0000610351562500, 0.0005187988281250, 0.0000000000000000, 0.0009765625000000,
+-0.0000305175781250, 0.0003051757812500, -0.0002441406250000, -0.0003967285156250,
+-0.0001831054687500, 0.0001831054687500, -0.0000305175781250, -0.0007629394531250,
+-0.0001525878906250, -0.0011901855468750, 0.0000000000000000, -0.0004882812500000,
+0.0002746582031250, -0.0003051757812500, 0.0000915527343750, -0.0007629394531250,
+0.0001525878906250, 0.0006103515625000, 0.0002746582031250, 0.0007629394531250,
+0.0000305175781250, 0.0003051757812500, -0.0002441406250000, 0.0007324218750000,
+-0.0000610351562500, 0.0008850097656250, -0.0002746582031250, -0.0003967285156250,
+-0.0004882812500000, -0.0003051757812500, -0.0002136230468750, -0.0001525878906250,
+0.0001525878906250, -0.0009460449218750, 0.0003662109375000, 0.0008850097656250,
+-0.0005493164062500, -0.0005187988281250, -0.0003662109375000, -0.0001525878906250,
+-0.0001525878906250, -0.0000915527343750, -0.0004272460937500, -0.0007019042968750,
+-0.0006103515625000, -0.0007324218750000, 0.0003662109375000, 0.0002136230468750,
+0.0002746582031250, 0.0001220703125000, 0.0002441406250000, 0.0000610351562500,
+0.0009765625000000, 0.0006103515625000, 0.0007934570312500, 0.0006408691406250,
+-0.0000915527343750, -0.0001220703125000, 0.0003662109375000, 0.0000610351562500,
+-0.0000610351562500, 0.0000610351562500, -0.0012817382812500, -0.0005798339843750,
+-0.0007019042968750, -0.0004882812500000, -0.0005187988281250, 0.0000000000000000,
+-0.0011901855468750, -0.0001525878906250, -0.0003662109375000, -0.0000610351562500,
+0.0007934570312500, 0.0004272460937500, 0.0000000000000000, 0.0003051757812500,
+0.0008850097656250, 0.0000000000000000, 0.0014953613281250, 0.0001831054687500,
+0.0005187988281250, 0.0000610351562500, 0.0000305175781250, -0.0003356933593750,
+0.0007324218750000, -0.0001831054687500, -0.0009765625000000, -0.0000305175781250,
+-0.0011596679687500, -0.0001525878906250, -0.0006713867187500, -0.0000610351562500,
+-0.0013122558593750, 0.0002441406250000, -0.0014038085937500, 0.0001525878906250,
+0.0002441406250000, 0.0000305175781250, -0.0001831054687500, 0.0001220703125000,
+0.0000915527343750, 0.0000305175781250, 0.0014953613281250, -0.0001525878906250,
+0.0009765625000000, -0.0001220703125000, 0.0006103515625000, -0.0000305175781250,
+0.0012512207031250, -0.0001220703125000, 0.0004577636718750, -0.0000610351562500,
+-0.0006103515625000, 0.0001220703125000, 0.0001525878906250, 0.0000610351562500,
+-0.0011291503906250, 0.0000000000000000, -0.0014038085937500, 0.0000915527343750,
+-0.0009155273437500, 0.0000305175781250, -0.0011901855468750, -0.0001220703125000,
+-0.0014343261718750, 0.0000000000000000, 0.0000610351562500, 0.0000000000000000,
+-0.0003356933593750, -0.0000610351562500, 0.0000305175781250, 0.0000000000000000,
+0.0014038085937500, 0.0000915527343750, 0.0007629394531250, 0.0000610351562500,
+0.0011291503906250, -0.0000305175781250, 0.0016174316406250, 0.0000305175781250,
+0.0010375976562500, 0.0000305175781250, 0.0007324218750000, -0.0000610351562500,
+0.0011596679687500, -0.0000610351562500, -0.0003051757812500, 0.0000000000000000,
+-0.0000610351562500, -0.0000305175781250, -0.0003967285156250, 0.0000000000000000,
+-0.0014648437500000, 0.0000915527343750, -0.0009460449218750, 0.0000610351562500,
+-0.0016479492187500, 0.0000000000000000, -0.0019226074218750, 0.0000000000000000,
+-0.0014648437500000, -0.0000305175781250, -0.0014953613281250, -0.0000305175781250,
+-0.0017700195312500, -0.0000305175781250, -0.0005187988281250, 0.0000305175781250,
+-0.0009765625000000, 0.0000000000000000, -0.0003967285156250, 0.0000305175781250,
+0.0007324218750000, 0.0000305175781250, -0.0000915527343750, 0.0000305175781250,
+0.0014648437500000, -0.0000305175781250, 0.0013732910156250, 0.0000000000000000,
+0.0014038085937500, 0.0000000000000000, 0.0022583007812500, 0.0000000000000000,
+0.0019531250000000, 0.0000305175781250, 0.0020751953125000, 0.0000305175781250,
+0.0023498535156250, 0.0000305175781250, 0.0019836425781250, 0.0000305175781250,
+0.0018005371093750, 0.0000000000000000, 0.0021362304687500, 0.0000000000000000,
+0.0011596679687500, -0.0000305175781250, 0.0014953613281250, 0.0000305175781250,
+0.0011901855468750, 0.0000305175781250, 0.0003356933593750, 0.0000000000000000,
+0.0009155273437500, 0.0000305175781250, -0.0001525878906250, 0.0000305175781250,
+0.0000610351562500, 0.0000305175781250, -0.0000915527343750, 0.0000305175781250,
+-0.0007629394531250, 0.0000000000000000, -0.0001831054687500, 0.0000000000000000,
+-0.0008850097656250, -0.0000305175781250, -0.0007629394531250, -0.0000305175781250,
+-0.0007324218750000, 0.0000305175781250, -0.0011596679687500, 0.0000305175781250,
+-0.0007324218750000, 0.0000305175781250, -0.0011596679687500, 0.0000000000000000,
+-0.0010375976562500, 0.0000000000000000, -0.0009765625000000, 0.0000305175781250,
+-0.0011596679687500, 0.0000305175781250, -0.0009155273437500, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+-0.0006103515625000, 0.0000000000000000, -0.0004272460937500, 0.0000000000000000,
+-0.0002441406250000, -0.0000305175781250, 0.0000915527343750, 0.0000000000000000,
+0.0000610351562500, 0.0000000000000000, 0.0004272460937500, 0.0000305175781250,
+0.0007324218750000, 0.0000000000000000, 0.0007324218750000, 0.0000000000000000,
+0.0012817382812500, 0.0000305175781250, 0.0013122558593750, 0.0000305175781250,
+0.0013427734375000, 0.0000000000000000, 0.0017395019531250, 0.0000000000000000,
+0.0016784667968750, 0.0000000000000000, 0.0016784667968750, -0.0000305175781250,
+0.0018310546875000, 0.0000000000000000, 0.0014953613281250, 0.0000305175781250,
+0.0012512207031250, -0.0000610351562500, 0.0014648437500000, -0.0000610351562500,
+0.0004577636718750, 0.0000000000000000, 0.0005798339843750, 0.0000000000000000,
+0.0002441406250000, 0.0000000000000000, -0.0005798339843750, 0.0000305175781250,
+-0.0002746582031250, 0.0000305175781250, -0.0010986328125000, 0.0000305175781250,
+-0.0012207031250000, 0.0000000000000000, -0.0011291503906250, 0.0000000000000000,
+-0.0014343261718750, 0.0000610351562500, -0.0014038085937500, 0.0000000000000000,
+-0.0007934570312500, 0.0000305175781250, -0.0010070800781250, 0.0000305175781250,
+-0.0005493164062500, 0.0000000000000000, 0.0002441406250000, 0.0000000000000000,
+-0.0001831054687500, 0.0000000000000000, 0.0007629394531250, -0.0000305175781250,
+0.0009460449218750, 0.0000000000000000, 0.0007019042968750, 0.0000305175781250,
+0.0009765625000000, 0.0000305175781250, 0.0010070800781250, 0.0000000000000000,
+0.0001831054687500, 0.0000305175781250, 0.0003967285156250, 0.0000305175781250,
+0.0000305175781250, -0.0000305175781250, -0.0007324218750000, -0.0000305175781250,
+-0.0004577636718750, 0.0000000000000000, -0.0007019042968750, -0.0000305175781250,
+-0.0010070800781250, -0.0000610351562500, -0.0005798339843750, 0.0000000000000000,
+-0.0002441406250000, 0.0000305175781250, -0.0005798339843750, -0.0000305175781250,
+0.0003967285156250, 0.0000915527343750, 0.0005187988281250, 0.0000915527343750,
+0.0002441406250000, 0.0000000000000000, 0.0005798339843750, -0.0000305175781250,
+0.0006103515625000, 0.0000305175781250, -0.0000915527343750, -0.0000915527343750,
+0.0000000000000000, -0.0000915527343750, -0.0000610351562500, 0.0000000000000000,
+-0.0006408691406250, 0.0000305175781250, -0.0004577636718750, -0.0000305175781250,
+-0.0001220703125000, 0.0000305175781250, -0.0003662109375000, 0.0000915527343750,
+-0.0000915527343750, 0.0000305175781250, 0.0004272460937500, 0.0000000000000000,
+0.0001831054687500, 0.0000610351562500, 0.0001525878906250, -0.0000915527343750,
+0.0004272460937500, -0.0001220703125000, 0.0001220703125000, -0.0000305175781250,
+-0.0003051757812500, 0.0000000000000000, -0.0000305175781250, -0.0000915527343750,
+-0.0002441406250000, 0.0000610351562500, -0.0004882812500000, 0.0001525878906250,
+-0.0002136230468750, 0.0000305175781250, 0.0000915527343750, 0.0000000000000000,
+-0.0000915527343750, 0.0000610351562500, 0.0001525878906250, -0.0000915527343750,
+0.0003051757812500, -0.0001220703125000, 0.0001220703125000, 0.0000000000000000,
+-0.0000915527343750, 0.0000305175781250, -0.0000915527343750, 0.0000000000000000,
+-0.0000915527343750, 0.0000610351562500, -0.0002441406250000, 0.0001220703125000,
+-0.0000915527343750, 0.0000000000000000, 0.0000915527343750, -0.0000305175781250,
+-0.0000610351562500, -0.0000305175781250, 0.0000915527343750, -0.0000915527343750,
+0.0002136230468750, -0.0001525878906250, 0.0000305175781250, 0.0000000000000000,
+-0.0001220703125000, 0.0000610351562500, 0.0000610351562500, -0.0000305175781250,
+-0.0000915527343750, 0.0000915527343750, -0.0002441406250000, 0.0001831054687500,
+-0.0000610351562500, 0.0000000000000000, 0.0001220703125000, -0.0001220703125000,
+-0.0000305175781250, -0.0000305175781250, 0.0000915527343750, -0.0002136230468750,
+0.0002136230468750, -0.0003051757812500, 0.0000610351562500, -0.0000610351562500,
+-0.0000915527343750, 0.0001220703125000, 0.0000305175781250, -0.0000915527343750,
+-0.0000915527343750, 0.0002441406250000, -0.0001831054687500, 0.0004272460937500,
+-0.0000610351562500, 0.0000915527343750, 0.0000610351562500, -0.0000610351562500,
+-0.0000610351562500, 0.0001831054687500, 0.0000610351562500, -0.0003662109375000,
+0.0001831054687500, -0.0004882812500000, 0.0000610351562500, -0.0002746582031250,
+-0.0000305175781250, -0.0003356933593750, 0.0000305175781250, -0.0004577636718750,
+-0.0000610351562500, 0.0002746582031250, -0.0001220703125000, 0.0002136230468750,
+-0.0000305175781250, 0.0001525878906250, 0.0000305175781250, 0.0005798339843750,
+-0.0000610351562500, 0.0005187988281250, 0.0000610351562500, -0.0000305175781250,
+0.0000915527343750},
+{-0.0009765625000000, 0.0000305175781250, -0.0012512207031250, 0.0000305175781250,
+-0.0008544921875000, 0.0000000000000000, -0.0009155273437500, 0.0000000000000000,
+-0.0010986328125000, 0.0000305175781250, -0.0000915527343750, -0.0000305175781250,
+-0.0001831054687500, -0.0000305175781250, 0.0000305175781250, 0.0000000000000000,
+0.0008239746093750, 0.0000000000000000, 0.0005493164062500, -0.0000610351562500,
+0.0005798339843750, 0.0000610351562500, 0.0008544921875000, 0.0000610351562500,
+0.0003662109375000, 0.0000000000000000, -0.0001525878906250, 0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, -0.0005187988281250, 0.0000000000000000,
+-0.0007324218750000, -0.0000305175781250, -0.0004577636718750, 0.0000000000000000,
+-0.0005493164062500, -0.0000305175781250, -0.0007019042968750, -0.0000610351562500,
+0.0001525878906250, 0.0000305175781250, 0.0001525878906250, 0.0000915527343750,
+0.0000000000000000, 0.0000305175781250, 0.0004882812500000, 0.0000305175781250,
+0.0004577636718750, 0.0000915527343750, -0.0000610351562500, -0.0000915527343750,
+0.0000915527343750, -0.0000915527343750, -0.0000305175781250, -0.0000305175781250,
+-0.0006103515625000, -0.0000305175781250, -0.0004272460937500, -0.0000915527343750,
+-0.0001525878906250, 0.0000915527343750, -0.0003356933593750, 0.0001220703125000,
+-0.0001525878906250, 0.0000305175781250, 0.0003662109375000, 0.0000610351562500,
+0.0001220703125000, 0.0001525878906250, 0.0001525878906250, -0.0000610351562500,
+0.0003662109375000, -0.0001220703125000, 0.0000915527343750, 0.0000000000000000,
+-0.0002746582031250, -0.0000305175781250, -0.0000305175781250, -0.0001220703125000,
+-0.0001831054687500, 0.0000610351562500, -0.0003967285156250, 0.0000915527343750,
+-0.0001525878906250, 0.0000305175781250, 0.0001831054687500, -0.0000305175781250,
+0.0001220703125000, -0.0000305175781250, 0.0000000000000000, -0.0000915527343750,
+0.0001831054687500, -0.0001525878906250, 0.0000610351562500, -0.0000610351562500,
+-0.0002136230468750, -0.0000305175781250, 0.0000000000000000, -0.0000610351562500,
+-0.0000610351562500, 0.0000915527343750, -0.0002441406250000, 0.0001525878906250,
+-0.0000610351562500, 0.0000000000000000, 0.0001831054687500, -0.0000305175781250,
+0.0000305175781250, 0.0000610351562500, 0.0000610351562500, -0.0001220703125000,
+0.0001831054687500, -0.0002136230468750, 0.0000305175781250, -0.0000610351562500,
+-0.0001831054687500, 0.0000000000000000, -0.0000305175781250, -0.0001220703125000,
+-0.0000610351562500, 0.0001525878906250, -0.0001831054687500, 0.0002746582031250,
+-0.0000305175781250, 0.0000610351562500, 0.0002136230468750, 0.0000000000000000,
+0.0000000000000000, 0.0001220703125000, 0.0000305175781250, -0.0003356933593750,
+0.0001831054687500, -0.0003967285156250, 0.0000610351562500, -0.0002136230468750,
+-0.0001525878906250, -0.0002441406250000, -0.0000305175781250, -0.0003967285156250,
+-0.0000610351562500, 0.0003051757812500, -0.0001525878906250, 0.0002441406250000,
+-0.0000305175781250, 0.0001220703125000, 0.0000915527343750, 0.0004882812500000,
+0.0000000000000000, 0.0004882812500000, 0.0000610351562500, -0.0002136230468750,
+0.0001525878906250, -0.0000915527343750, 0.0000305175781250, -0.0001220703125000,
+-0.0000610351562500, -0.0007019042968750, 0.0000000000000000, -0.0005187988281250,
+-0.0000610351562500, -0.0002441406250000, -0.0000915527343750, -0.0005493164062500,
+-0.0000305175781250, -0.0001831054687500, 0.0000305175781250, 0.0004272460937500,
+0.0000000000000000, 0.0000610351562500, 0.0000305175781250, 0.0004272460937500,
+0.0000915527343750, 0.0007019042968750, 0.0000000000000000, 0.0002746582031250,
+0.0000000000000000, -0.0000610351562500, 0.0000610351562500, 0.0003051757812500,
+-0.0000305175781250, -0.0005187988281250, -0.0000610351562500, -0.0006408691406250,
+0.0000000000000000, -0.0005187988281250, 0.0000305175781250, -0.0008239746093750,
+-0.0000305175781250, -0.0008239746093750, 0.0000305175781250, -0.0001831054687500,
+0.0000610351562500, -0.0003051757812500, 0.0000305175781250, 0.0000000000000000,
+0.0000305175781250, 0.0006713867187500, 0.0000305175781250, 0.0003662109375000,
+-0.0000305175781250, 0.0007324218750000, -0.0000305175781250, 0.0009765625000000,
+0.0000000000000000, 0.0006408691406250, 0.0000000000000000, 0.0004882812500000,
+-0.0000305175781250, 0.0006408691406250, 0.0000000000000000, -0.0001220703125000,
+0.0000000000000000, -0.0000610351562500, -0.0000305175781250, -0.0002441406250000,
+0.0000000000000000, -0.0007629394531250, 0.0000305175781250, -0.0006103515625000,
+-0.0000305175781250, -0.0009155273437500, -0.0000305175781250, -0.0010681152343750,
+0.0000000000000000, -0.0009460449218750, -0.0000305175781250, -0.0009460449218750,
+0.0000000000000000, -0.0009460449218750, 0.0000610351562500, -0.0006103515625000,
+0.0000000000000000, -0.0007019042968750, 0.0000000000000000, -0.0004577636718750,
+0.0000000000000000, -0.0000305175781250, 0.0000305175781250, -0.0001831054687500,
+0.0000000000000000, 0.0003967285156250, -0.0000305175781250, 0.0004577636718750,
+0.0000000000000000, 0.0005187988281250, 0.0000000000000000, 0.0008850097656250,
+0.0000000000000000, 0.0007934570312500, 0.0000000000000000, 0.0010070800781250,
+0.0000000000000000, 0.0010375976562500, 0.0000305175781250, 0.0010375976562500,
+0.0000305175781250, 0.0010681152343750, 0.0000000000000000, 0.0010681152343750,
+0.0000000000000000, 0.0010375976562500, 0.0000000000000000, 0.0010681152343750,
+0.0000305175781250, 0.0010681152343750, 0.0000000000000000, 0.0009765625000000,
+0.0000000000000000, 0.0010070800781250, 0.0000305175781250, 0.0010070800781250,
+-0.0000305175781250, 0.0010070800781250, 0.0000000000000000, 0.0010375976562500,
+-0.0000305175781250, 0.0010681152343750, 0.0000305175781250, 0.0010070800781250,
+-0.0000305175781250, 0.0011291503906250, 0.0000000000000000, 0.0010986328125000,
+-0.0000305175781250, 0.0011596679687500, 0.0000000000000000, 0.0012207031250000,
+0.0000000000000000, 0.0011901855468750, 0.0000000000000000, 0.0011596679687500,
+0.0000305175781250, 0.0012817382812500, 0.0000000000000000, 0.0010986328125000,
+-0.0000610351562500, 0.0009765625000000, 0.0000000000000000, 0.0011596679687500,
+0.0000000000000000, 0.0004882812500000, 0.0000000000000000, 0.0007629394531250,
+0.0000305175781250, 0.0004272460937500, 0.0000000000000000, -0.0002746582031250,
+-0.0000305175781250, 0.0002746582031250, 0.0000000000000000, -0.0008850097656250,
+0.0000000000000000, -0.0007629394531250, 0.0000000000000000, -0.0007934570312500,
+-0.0000610351562500, -0.0014953613281250, 0.0000000000000000, -0.0012817382812500,
+0.0000000000000000, -0.0011596679687500, -0.0000305175781250, -0.0014953613281250,
+0.0000305175781250, -0.0010070800781250, 0.0000000000000000, -0.0004882812500000,
+-0.0000305175781250, -0.0009155273437500, 0.0000000000000000, 0.0004577636718750,
+0.0000000000000000, 0.0002746582031250, 0.0000000000000000, 0.0004882812500000,
+-0.0000305175781250, 0.0015563964843750, 0.0000305175781250, 0.0011291503906250,
+0.0000000000000000, 0.0012512207031250, 0.0000305175781250, 0.0016479492187500,
+0.0000000000000000, 0.0009765625000000, 0.0000610351562500, 0.0004882812500000,
+0.0000610351562500, 0.0009765625000000, -0.0000610351562500, -0.0005493164062500,
+-0.0000305175781250, -0.0004577636718750, 0.0000000000000000, -0.0005493164062500,
+-0.0000305175781250, -0.0015563964843750, -0.0000610351562500, -0.0012817382812500,
+0.0000000000000000, -0.0011291503906250, 0.0000000000000000, -0.0015258789062500,
+0.0000000000000000, -0.0008544921875000, 0.0000610351562500, -0.0001831054687500,
+0.0000305175781250, -0.0006408691406250, -0.0000610351562500, 0.0007019042968750,
+0.0000000000000000, 0.0008239746093750, 0.0000000000000000, 0.0005493164062500,
+-0.0001220703125000, 0.0011291503906250, -0.0000915527343750, 0.0010986328125000,
+0.0000610351562500, 0.0002441406250000, 0.0000000000000000, 0.0004882812500000,
+0.0000000000000000, 0.0001220703125000, 0.0000915527343750, -0.0007934570312500,
+0.0001220703125000, -0.0003662109375000, 0.0000305175781250, -0.0007629394531250,
+0.0000305175781250, -0.0010986328125000, 0.0000000000000000, -0.0006408691406250,
+-0.0001525878906250, -0.0003967285156250, -0.0000915527343750, -0.0007019042968750,
+-0.0000610351562500, 0.0003356933593750, -0.0000915527343750, 0.0003356933593750,
+0.0000000000000000, 0.0002746582031250, 0.0001831054687500, 0.0009155273437500,
+0.0000915527343750, 0.0007934570312500, 0.0000610351562500, 0.0000610351562500,
+0.0001831054687500, 0.0004577636718750, 0.0000000000000000, -0.0000305175781250,
+-0.0001525878906250, -0.0010681152343750, -0.0000305175781250, -0.0005187988281250,
+-0.0001525878906250, -0.0006713867187500, -0.0003051757812500, -0.0012512207031250,
+-0.0000915527343750, -0.0004272460937500, 0.0000915527343750, 0.0004882812500000,
+-0.0000610351562500, -0.0001831054687500, 0.0002136230468750, 0.0006713867187500,
+0.0003356933593750, 0.0011596679687500, 0.0001220703125000, 0.0003356933593750,
+0.0000915527343750, -0.0001220703125000, 0.0002136230468750, 0.0005187988281250,
+-0.0003356933593750, -0.0007324218750000, -0.0003967285156250, -0.0010070800781250,
+-0.0001831054687500, -0.0004882812500000, -0.0003051757812500, -0.0003967285156250,
+0.0004577636718750, 0.0005798339843750, -0.0003662109375000, -0.0007629394531250,
+0.0000915527343750, -0.0008544921875000, -0.0001525878906250, -0.0003051757812500,
+-0.0008239746093750, -0.0002441406250000, -0.0006408691406250, -0.0006408691406250,
+0.0000000000000000, 0.0004577636718750, -0.0004577636718750, 0.0007019042968750,
+-0.0000305175781250, 0.0001831054687500, 0.0009155273437500, 0.0003356933593750,
+0.0004882812500000, 0.0006103515625000, 0.0005187988281250, -0.0003662109375000,
+0.0010681152343750, -0.0005187988281250, 0.0002441406250000, -0.0001525878906250,
+-0.0006713867187500, -0.0004272460937500, 0.0000610351562500, -0.0005187988281250,
+-0.0010070800781250, 0.0002136230468750, -0.0015258789062500, 0.0003051757812500,
+-0.0006103515625000, 0.0000610351562500, -0.0002136230468750, 0.0003356933593750,
+-0.0009460449218750, 0.0003967285156250, 0.0009155273437500, -0.0001220703125000,
+0.0010681152343750, -0.0001525878906250, 0.0004577636718750, -0.0000305175781250,
+0.0010375976562500, -0.0003356933593750, 0.0013122558593750, -0.0003356933593750,
+-0.0004882812500000, 0.0001220703125000, -0.0001525878906250, 0.0001220703125000,
+-0.0003051757812500, 0.0000305175781250, -0.0017089843750000, 0.0002441406250000,
+-0.0012817382812500, 0.0002441406250000, -0.0005798339843750, -0.0000915527343750,
+-0.0012512207031250, -0.0000305175781250, -0.0004577636718750, 0.0000000000000000,
+0.0009155273437500, -0.0001831054687500, 0.0000915527343750, -0.0002136230468750,
+0.0009765625000000, 0.0000610351562500, 0.0015258789062500, 0.0000000000000000,
+0.0007324218750000, 0.0000305175781250, 0.0003662109375000, 0.0001831054687500,
+0.0009155273437500, 0.0001525878906250, -0.0006713867187500, -0.0000305175781250,
+-0.0006408691406250, 0.0000305175781250, -0.0005798339843750, 0.0000000000000000,
+-0.0015258789062500, -0.0000915527343750, -0.0012817382812500, -0.0000915527343750,
+-0.0006408691406250, 0.0000610351562500, -0.0011901855468750, 0.0000305175781250,
+-0.0005187988281250, 0.0000000000000000, 0.0005187988281250, 0.0000915527343750,
+-0.0001831054687500, 0.0000915527343750, 0.0009765625000000, -0.0000305175781250,
+0.0012207031250000, -0.0000305175781250, 0.0008850097656250, -0.0000305175781250,
+0.0012512207031250},
+{0.0000000000000000, 0.0004577636718750, 0.0001525878906250, -0.0000305175781250,
+0.0000610351562500, 0.0003356933593750, 0.0000610351562500, -0.0006713867187500,
+0.0001525878906250, -0.0007629394531250, 0.0000305175781250, -0.0005187988281250,
+-0.0001525878906250, -0.0008850097656250, -0.0000305175781250, -0.0009460449218750,
+-0.0001220703125000, 0.0001525878906250, -0.0002441406250000, -0.0001831054687500,
+-0.0000915527343750, 0.0000915527343750, 0.0001220703125000, 0.0011291503906250,
+-0.0000610351562500, 0.0007629394531250, 0.0001831054687500, 0.0003662109375000,
+0.0003051757812500, 0.0009460449218750, 0.0000610351562500, 0.0002136230468750,
+0.0000305175781250, -0.0009765625000000, 0.0001220703125000, -0.0003662109375000,
+-0.0003051757812500, -0.0006103515625000, -0.0003356933593750, -0.0013122558593750,
+-0.0001220703125000, -0.0004272460937500, -0.0002441406250000, 0.0004882812500000,
+-0.0003356933593750, -0.0002441406250000, 0.0002136230468750, 0.0007019042968750,
+0.0002136230468750, 0.0011901855468750, 0.0001525878906250, 0.0004882812500000,
+0.0005187988281250, -0.0001525878906250, -0.0004577636718750, -0.0003662109375000,
+0.0000915527343750, 0.0006713867187500, -0.0003967285156250, 0.0010375976562500,
+0.0000000000000000, 0.0003051757812500, 0.0007934570312500, -0.0002136230468750,
+0.0004272460937500, 0.0003051757812500, 0.0002136230468750, -0.0005798339843750,
+0.0007019042968750, -0.0009765625000000, 0.0000610351562500, -0.0003356933593750,
+-0.0008239746093750, -0.0000305175781250, -0.0002441406250000, -0.0004577636718750,
+-0.0008239746093750, 0.0004272460937500, -0.0013427734375000, 0.0007019042968750,
+-0.0004882812500000, 0.0002136230468750, 0.0000915527343750, 0.0001831054687500,
+-0.0005798339843750, 0.0004882812500000, 0.0009155273437500, -0.0003051757812500,
+0.0012512207031250, -0.0004272460937500, 0.0004577636718750, -0.0001525878906250,
+0.0007324218750000, -0.0002136230468750, 0.0011901855468750, -0.0004272460937500,
+-0.0007629394531250, 0.0002136230468750, -0.0006713867187500, 0.0003356933593750,
+-0.0004577636718750, 0.0000610351562500, -0.0015258789062500, 0.0002136230468750,
+-0.0014343261718750, 0.0003356933593750, 0.0000305175781250, -0.0001831054687500,
+-0.0005493164062500, -0.0002136230468750, -0.0000915527343750, -0.0000610351562500,
+0.0014343261718750, -0.0002136230468750, 0.0007934570312500, -0.0003051757812500,
+0.0007324218750000, 0.0001525878906250, 0.0014343261718750, 0.0001525878906250,
+0.0005493164062500, 0.0000610351562500, -0.0005798339843750, 0.0001525878906250,
+0.0002441406250000, 0.0002441406250000, -0.0010986328125000, -0.0000915527343750,
+-0.0014648437500000, -0.0000915527343750, -0.0007934570312500, -0.0000305175781250,
+-0.0008239746093750, -0.0001220703125000, -0.0012207031250000, -0.0001831054687500,
+0.0002441406250000, 0.0000610351562500, 0.0000305175781250, 0.0000915527343750,
+0.0001525878906250, 0.0000000000000000, 0.0012512207031250, 0.0000915527343750,
+0.0008544921875000, 0.0001220703125000, 0.0006408691406250, -0.0000305175781250,
+0.0011901855468750, -0.0000610351562500, 0.0005187988281250, 0.0000000000000000,
+-0.0002746582031250, -0.0000610351562500, 0.0003662109375000, -0.0000915527343750,
+-0.0007934570312500, 0.0000305175781250, -0.0009155273437500, 0.0000915527343750,
+-0.0006713867187500, 0.0000000000000000, -0.0011291503906250, 0.0000305175781250,
+-0.0011291503906250, 0.0000305175781250, -0.0005187988281250, -0.0000305175781250,
+-0.0007629394531250, -0.0000305175781250, -0.0004272460937500, -0.0000305175781250,
+0.0002746582031250, 0.0000000000000000, -0.0001525878906250, -0.0000305175781250,
+0.0006408691406250, 0.0000000000000000, 0.0007324218750000, 0.0000610351562500,
+0.0005798339843750, 0.0000000000000000, 0.0009155273437500, -0.0000305175781250,
+0.0008850097656250, 0.0000305175781250, 0.0003051757812500, -0.0000305175781250,
+0.0006713867187500, -0.0000610351562500, 0.0002136230468750, 0.0000000000000000,
+-0.0005187988281250, 0.0000305175781250, 0.0000000000000000, 0.0000000000000000,
+-0.0010986328125000, 0.0000000000000000, -0.0010681152343750, 0.0000305175781250,
+-0.0010375976562500, 0.0000000000000000, -0.0016784667968750, 0.0000000000000000,
+-0.0014343261718750, 0.0000000000000000, -0.0012817382812500, 0.0000000000000000,
+-0.0016784667968750, 0.0000305175781250, -0.0011901855468750, 0.0000305175781250,
+-0.0007019042968750, 0.0000000000000000, -0.0012512207031250, 0.0000610351562500,
+0.0002136230468750, -0.0000305175781250, -0.0003051757812500, 0.0000000000000000,
+0.0001525878906250, 0.0000305175781250, 0.0014343261718750, -0.0000305175781250,
+0.0004882812500000, 0.0000000000000000, 0.0019836425781250, 0.0000000000000000,
+0.0018005371093750, 0.0000305175781250, 0.0019531250000000, -0.0000305175781250,
+0.0027770996093750, 0.0000000000000000, 0.0021362304687500, -0.0000305175781250,
+0.0029602050781250, 0.0000000000000000, 0.0028076171875000, 0.0000000000000000,
+0.0028381347656250, 0.0000305175781250, 0.0032958984375000, 0.0000000000000000,
+0.0028686523437500, 0.0000000000000000, 0.0032653808593750, 0.0000000000000000,
+0.0031127929687500, 0.0000305175781250, 0.0029907226562500, 0.0000305175781250,
+0.0031127929687500, 0.0000000000000000, 0.0028686523437500, 0.0000000000000000,
+0.0026245117187500, 0.0000305175781250, 0.0025329589843750, 0.0000000000000000,
+0.0021972656250000, 0.0000000000000000, 0.0018005371093750, 0.0000000000000000,
+0.0016174316406250, 0.0000000000000000, 0.0009460449218750, 0.0000000000000000,
+0.0009765625000000, -0.0000305175781250, 0.0004272460937500, 0.0000000000000000,
+-0.0002441406250000, 0.0000000000000000, -0.0000915527343750, 0.0000000000000000,
+-0.0010681152343750, 0.0000305175781250, -0.0010681152343750, 0.0000305175781250,
+-0.0012512207031250, 0.0000305175781250, -0.0019226074218750, 0.0000000000000000,
+-0.0017700195312500, -0.0000305175781250, -0.0018920898437500, 0.0000305175781250,
+-0.0021057128906250, 0.0000000000000000, -0.0017700195312500, 0.0000000000000000,
+-0.0015869140625000, 0.0000000000000000, -0.0017700195312500, -0.0000305175781250,
+-0.0007934570312500, -0.0000305175781250, -0.0009155273437500, -0.0000305175781250,
+-0.0005187988281250, 0.0000000000000000, 0.0003356933593750, 0.0000000000000000,
+0.0000305175781250, 0.0000000000000000, 0.0008239746093750, 0.0000305175781250,
+0.0009155273437500, 0.0000305175781250, 0.0007324218750000, -0.0000305175781250,
+0.0009460449218750, 0.0000305175781250, 0.0009765625000000, 0.0000305175781250,
+0.0002746582031250, -0.0000305175781250, 0.0004272460937500, -0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, -0.0007019042968750, 0.0000000000000000,
+-0.0003662109375000, -0.0000305175781250, -0.0008544921875000, 0.0000305175781250,
+-0.0011291503906250, 0.0000305175781250, -0.0006408691406250, 0.0000305175781250,
+-0.0004272460937500, 0.0000305175781250, -0.0007324218750000, 0.0000305175781250,
+0.0002441406250000, -0.0000305175781250, 0.0003051757812500, -0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, 0.0008850097656250, -0.0000305175781250,
+0.0007934570312500, -0.0000915527343750, 0.0002441406250000, 0.0000305175781250,
+0.0005493164062500, 0.0000000000000000, 0.0001220703125000, 0.0000000000000000,
+-0.0006408691406250, 0.0000305175781250, -0.0002441406250000, 0.0000305175781250,
+-0.0006103515625000, -0.0000305175781250, -0.0009460449218750, -0.0000305175781250,
+-0.0004882812500000, -0.0000305175781250, -0.0001525878906250, -0.0000915527343750,
+-0.0004882812500000, -0.0000610351562500, 0.0003662109375000, 0.0000915527343750,
+0.0005493164062500, 0.0000305175781250, 0.0001831054687500, 0.0000305175781250,
+0.0003051757812500, 0.0001220703125000, 0.0004577636718750, 0.0000915527343750,
+-0.0003051757812500, -0.0000610351562500, -0.0003356933593750, -0.0000305175781250,
+-0.0001525878906250, -0.0000305175781250, -0.0005187988281250, -0.0001220703125000,
+-0.0005187988281250, -0.0000915527343750, 0.0000610351562500, 0.0000610351562500,
+-0.0000305175781250, 0.0000305175781250, -0.0000305175781250, 0.0000000000000000,
+0.0004272460937500, 0.0001525878906250, 0.0003662109375000, 0.0000915527343750,
+-0.0000305175781250, -0.0000915527343750, 0.0001220703125000, -0.0000305175781250,
+0.0000000000000000, 0.0000000000000000, -0.0004577636718750, -0.0001525878906250,
+-0.0002746582031250, -0.0001220703125000, -0.0000610351562500, 0.0000610351562500,
+-0.0002136230468750, 0.0000305175781250, -0.0001220703125000, 0.0000305175781250,
+0.0003051757812500, 0.0000915527343750, 0.0002136230468750, 0.0000610351562500,
+-0.0001220703125000, -0.0001525878906250, 0.0000610351562500, -0.0000610351562500,
+0.0000000000000000, -0.0000305175781250, -0.0002746582031250, -0.0001831054687500,
+-0.0001831054687500, -0.0001220703125000, 0.0000305175781250, 0.0000610351562500,
+-0.0001220703125000, 0.0000915527343750, -0.0000610351562500, 0.0000000000000000,
+0.0002441406250000, 0.0001220703125000, 0.0001525878906250, 0.0001220703125000,
+-0.0000610351562500, -0.0001525878906250, 0.0000305175781250, -0.0001525878906250,
+-0.0000305175781250, -0.0000610351562500, -0.0002746582031250, -0.0001831054687500,
+-0.0001831054687500, -0.0002441406250000, 0.0000305175781250, 0.0001525878906250,
+-0.0001220703125000, 0.0001831054687500, -0.0000610351562500, 0.0000305175781250,
+0.0001831054687500, 0.0002441406250000, 0.0001525878906250, 0.0002441406250000,
+0.0000000000000000, -0.0002746582031250, 0.0000915527343750, -0.0002136230468750,
+0.0000305175781250, -0.0001525878906250, -0.0002441406250000, -0.0005187988281250,
+-0.0001220703125000, -0.0004882812500000, 0.0000000000000000, 0.0000610351562500,
+-0.0001220703125000, -0.0000915527343750, -0.0000305175781250, 0.0000000000000000,
+0.0002136230468750, 0.0005798339843750, 0.0000915527343750, 0.0003967285156250,
+0.0000000000000000, 0.0000610351562500, 0.0000610351562500, 0.0003662109375000,
+0.0000000000000000, 0.0000610351562500, -0.0001220703125000, -0.0005798339843750,
+-0.0000610351562500, -0.0002441406250000, 0.0000000000000000, -0.0004272460937500,
+-0.0000915527343750, -0.0007019042968750, -0.0000610351562500, -0.0003356933593750,
+0.0001220703125000, 0.0000000000000000, 0.0000000000000000, -0.0002746582031250,
+0.0000000000000000, 0.0003356933593750, 0.0000610351562500, 0.0005187988281250,
+0.0000305175781250, 0.0002441406250000, -0.0000610351562500, 0.0003356933593750,
+0.0000000000000000, 0.0004882812500000, 0.0000000000000000, -0.0002136230468750,
+-0.0000305175781250, -0.0001525878906250, -0.0000305175781250, -0.0001831054687500,
+0.0000305175781250, -0.0006713867187500, 0.0000000000000000, -0.0005493164062500,
+0.0000305175781250, -0.0003051757812500, 0.0000305175781250, -0.0004577636718750,
+0.0000305175781250, -0.0002441406250000, -0.0000305175781250, 0.0002136230468750,
+-0.0000305175781250, 0.0000305175781250, -0.0000305175781250, 0.0003051757812500,
+-0.0000305175781250, 0.0004272460937500, 0.0000000000000000, 0.0002136230468750,
+0.0000000000000000, 0.0001220703125000, -0.0000305175781250, 0.0002746582031250,
+0.0000000000000000, -0.0001831054687500, 0.0000000000000000, -0.0001831054687500,
+0.0000000000000000, -0.0001831054687500, 0.0000305175781250, -0.0003662109375000,
+0.0000305175781250, -0.0003356933593750, 0.0000000000000000, -0.0001525878906250,
+-0.0000610351562500, -0.0002746582031250, 0.0000000000000000, -0.0000305175781250,
+0.0000000000000000, 0.0003051757812500, -0.0000305175781250, 0.0001525878906250,
+0.0000305175781250},
+{-0.0005187988281250, -0.0001220703125000, 0.0000610351562500, 0.0000000000000000,
+-0.0000610351562500, -0.0000610351562500, -0.0000305175781250, 0.0000000000000000,
+0.0003967285156250, 0.0001525878906250, 0.0003356933593750, 0.0000915527343750,
+-0.0000610351562500, -0.0000610351562500, 0.0000610351562500, 0.0000000000000000,
+0.0000305175781250, 0.0000000000000000, -0.0003967285156250, -0.0001220703125000,
+-0.0003051757812500, -0.0001220703125000, 0.0001220703125000, 0.0000610351562500,
+-0.0000305175781250, -0.0000305175781250, -0.0000915527343750, -0.0000610351562500,
+0.0002136230468750, 0.0001220703125000, 0.0001831054687500, 0.0001220703125000,
+-0.0001220703125000, -0.0001220703125000, -0.0000915527343750, -0.0000610351562500,
+0.0000000000000000, -0.0000610351562500, -0.0002136230468750, -0.0002136230468750,
+-0.0002441406250000, -0.0001831054687500, 0.0000610351562500, 0.0000610351562500,
+-0.0000305175781250, 0.0000305175781250, -0.0000305175781250, -0.0000305175781250,
+0.0002136230468750, 0.0001220703125000, 0.0002136230468750, 0.0001525878906250,
+-0.0000610351562500, -0.0002136230468750, -0.0000305175781250, -0.0001220703125000,
+0.0000000000000000, -0.0001220703125000, -0.0002441406250000, -0.0003967285156250,
+-0.0002136230468750, -0.0003967285156250, 0.0000610351562500, 0.0000915527343750,
+0.0000305175781250, -0.0000305175781250, -0.0000305175781250, 0.0000000000000000,
+0.0002136230468750, 0.0004882812500000, 0.0001525878906250, 0.0003356933593750,
+-0.0000305175781250, -0.0000610351562500, 0.0000000000000000, 0.0001831054687500,
+0.0000610351562500, -0.0000305175781250, -0.0001525878906250, -0.0006408691406250,
+-0.0001220703125000, -0.0003967285156250, 0.0000305175781250, -0.0003051757812500,
+-0.0000610351562500, -0.0005798339843750, 0.0000000000000000, -0.0002441406250000,
+0.0001525878906250, 0.0002746582031250, 0.0001220703125000, -0.0000610351562500,
+0.0000000000000000, 0.0003662109375000, 0.0000305175781250, 0.0006103515625000,
+0.0000305175781250, 0.0002441406250000, -0.0000915527343750, 0.0000915527343750,
+-0.0000915527343750, 0.0002746582031250, 0.0000000000000000, -0.0003051757812500,
+-0.0000305175781250, -0.0003967285156250, 0.0000000000000000, -0.0002136230468750,
+0.0000915527343750, -0.0003967285156250, 0.0000305175781250, -0.0004882812500000,
+0.0000000000000000, -0.0000305175781250, 0.0000915527343750, -0.0000610351562500,
+0.0000305175781250, 0.0000000000000000, -0.0000305175781250, 0.0003356933593750,
+-0.0000305175781250, 0.0002746582031250, 0.0000000000000000, 0.0000305175781250,
+-0.0000305175781250, 0.0001220703125000, 0.0000000000000000, 0.0000000000000000,
+0.0000610351562500, -0.0003051757812500, 0.0000000000000000, -0.0002441406250000,
+0.0000305175781250, -0.0000305175781250, 0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, 0.0001220703125000, -0.0000305175781250, 0.0005798339843750,
+0.0000000000000000, 0.0003051757812500, -0.0000305175781250, 0.0008850097656250,
+-0.0000610351562500, 0.0010681152343750, -0.0000305175781250, 0.0007629394531250,
+0.0000610351562500, 0.0008544921875000, 0.0000000000000000, 0.0008850097656250,
+0.0000000000000000, -0.0001525878906250, 0.0000305175781250, 0.0000610351562500,
+0.0000000000000000, -0.0005493164062500, 0.0000000000000000, -0.0016784667968750,
+-0.0000305175781250, -0.0012817382812500, 0.0000305175781250, -0.0026245117187500,
+-0.0000305175781250, -0.0027160644531250, 0.0000305175781250, -0.0030822753906250,
+0.0000000000000000, -0.0039978027343750, 0.0000000000000000, -0.0036621093750000,
+0.0000000000000000, -0.0042114257812500, 0.0000000000000000, -0.0043029785156250,
+0.0000000000000000, -0.0042114257812500, 0.0000000000000000, -0.0043029785156250,
+0.0000305175781250, -0.0040893554687500, 0.0000000000000000, -0.0038146972656250,
+-0.0000305175781250, -0.0034484863281250, 0.0000000000000000, -0.0032043457031250,
+0.0000000000000000, -0.0031127929687500, 0.0000000000000000, -0.0029296875000000,
+0.0000305175781250, -0.0022888183593750, 0.0000000000000000, -0.0017089843750000,
+0.0000000000000000, -0.0017395019531250, 0.0000000000000000, -0.0015563964843750,
+-0.0000305175781250, -0.0010681152343750, -0.0000305175781250, -0.0017395019531250,
+0.0000000000000000, -0.0014343261718750, -0.0000305175781250, -0.0016479492187500,
+0.0000000000000000, -0.0022583007812500, 0.0000000000000000, -0.0018920898437500,
+0.0000000000000000, -0.0029907226562500, 0.0000000000000000, -0.0025024414062500,
+0.0000000000000000, -0.0033264160156250, 0.0000000000000000, -0.0046386718750000,
+0.0000000000000000, -0.0035705566406250, 0.0000000000000000, -0.0054626464843750,
+0.0000305175781250, -0.0054321289062500, 0.0000305175781250, -0.0054626464843750,
+0.0000305175781250, -0.0066223144531250, -0.0000305175781250, -0.0061340332031250,
+0.0000000000000000, -0.0061340332031250, 0.0000000000000000, -0.0067749023437500,
+0.0000000000000000, -0.0057983398437500, 0.0000000000000000, -0.0051574707031250,
+0.0000000000000000, -0.0058898925781250, 0.0000305175781250, -0.0030212402343750,
+0.0000000000000000, -0.0038452148437500, 0.0000000000000000, -0.0028991699218750,
+0.0000305175781250, -0.0003967285156250, -0.0000305175781250, -0.0018005371093750,
+0.0000000000000000, 0.0010375976562500, 0.0000610351562500, 0.0011901855468750,
+0.0000000000000000, 0.0009765625000000, -0.0000610351562500, 0.0023193359375000,
+0.0000305175781250, 0.0019836425781250, 0.0000000000000000, 0.0014038085937500,
+0.0000000000000000, 0.0019226074218750, 0.0000610351562500, 0.0013122558593750,
+0.0000305175781250, 0.0002746582031250, 0.0000000000000000, 0.0008544921875000,
+0.0000000000000000, -0.0001220703125000, 0.0000305175781250, -0.0003967285156250,
+0.0000305175781250, -0.0000610351562500, 0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, -0.0003356933593750, -0.0000305175781250, 0.0001525878906250,
+-0.0000305175781250, 0.0001220703125000, -0.0000305175781250, 0.0001220703125000,
+0.0000305175781250, 0.0004882812500000, 0.0000000000000000, 0.0004272460937500,
+0.0000305175781250, 0.0000610351562500, 0.0000915527343750, 0.0003051757812500,
+0.0000000000000000, -0.0000305175781250, -0.0000610351562500, -0.0005798339843750,
+0.0000000000000000, -0.0002441406250000, -0.0000305175781250, -0.0005798339843750,
+-0.0000610351562500, -0.0008850097656250, 0.0000305175781250, -0.0004577636718750,
+0.0000915527343750, -0.0002746582031250, 0.0000000000000000, -0.0005187988281250,
+0.0000610351562500, 0.0003356933593750, 0.0001220703125000, 0.0003967285156250,
+0.0000000000000000, 0.0003051757812500, -0.0000915527343750, 0.0007324218750000,
+0.0000000000000000, 0.0006408691406250, -0.0000610351562500, 0.0002136230468750,
+-0.0002136230468750, 0.0005187988281250, -0.0000610351562500, 0.0001220703125000,
+0.0000610351562500, -0.0004882812500000, -0.0000305175781250, -0.0001525878906250,
+0.0001525878906250, -0.0007324218750000, 0.0002136230468750, -0.0010070800781250,
+0.0000305175781250, -0.0004882812500000, 0.0000000000000000, -0.0003662109375000,
+0.0000915527343750, -0.0007019042968750, -0.0001831054687500, 0.0005493164062500,
+-0.0002746582031250, 0.0005493164062500, -0.0001220703125000, 0.0003356933593750,
+-0.0000915527343750, 0.0010070800781250, -0.0001831054687500, 0.0010070800781250,
+0.0001831054687500, -0.0002136230468750, 0.0001525878906250, 0.0001220703125000,
+0.0000305175781250, -0.0001220703125000, 0.0003051757812500, -0.0013732910156250,
+0.0002441406250000, -0.0010070800781250, -0.0001831054687500, -0.0001831054687500,
+-0.0000915527343750, -0.0007934570312500, -0.0001525878906250, -0.0002136230468750,
+-0.0004882812500000, 0.0012512207031250, -0.0003967285156250, 0.0005798339843750,
+0.0000305175781250, 0.0004272460937500, -0.0002136230468750, 0.0010986328125000,
+-0.0000610351562500, 0.0004882812500000, 0.0006408691406250, -0.0010681152343750,
+-0.0002136230468750, 0.0004272460937500, -0.0003662109375000, 0.0003051757812500,
+-0.0007934570312500, 0.0010375976562500, -0.0001220703125000, 0.0002441406250000,
+0.0004882812500000, -0.0008850097656250, -0.0000305175781250, -0.0003051757812500,
+0.0005798339843750, -0.0003967285156250, 0.0010375976562500, -0.0009460449218750,
+0.0003356933593750, -0.0003051757812500, 0.0000000000000000, 0.0005798339843750,
+0.0004882812500000, 0.0000610351562500, -0.0009460449218750, 0.0003662109375000,
+-0.0011596679687500, 0.0007934570312500, -0.0005187988281250, 0.0003051757812500,
+-0.0008850097656250, -0.0003356933593750, -0.0011901855468750, 0.0001220703125000,
+0.0006103515625000, -0.0003051757812500, 0.0004272460937500, -0.0007019042968750,
+0.0002746582031250, -0.0002441406250000, 0.0014343261718750, 0.0001831054687500,
+0.0013122558593750, -0.0001831054687500, -0.0000915527343750, 0.0002746582031250,
+0.0004882812500000, 0.0005187988281250, 0.0000610351562500, 0.0001220703125000,
+-0.0015258789062500, -0.0000915527343750, -0.0008850097656250, 0.0001525878906250,
+-0.0006103515625000, -0.0002136230468750, -0.0013427734375000, -0.0004272460937500,
+-0.0004577636718750, -0.0001525878906250, 0.0007324218750000, 0.0000305175781250,
+-0.0000610351562500, -0.0001831054687500, 0.0008544921875000, 0.0001525878906250,
+0.0013732910156250, 0.0003356933593750, 0.0005493164062500, 0.0000915527343750,
+0.0002441406250000, 0.0000000000000000, 0.0007629394531250, 0.0001220703125000,
+-0.0006408691406250, -0.0001220703125000, -0.0007934570312500, -0.0002441406250000,
+-0.0003662109375000, -0.0000610351562500, -0.0007934570312500, 0.0000305175781250,
+-0.0009155273437500, -0.0000610351562500, -0.0000305175781250, 0.0000915527343750,
+-0.0002136230468750, 0.0001831054687500, -0.0000915527343750, 0.0000610351562500,
+0.0005493164062500, -0.0000305175781250, 0.0003051757812500, 0.0001220703125000,
+0.0001525878906250, -0.0000610351562500, 0.0005187988281250, -0.0001220703125000,
+0.0001525878906250, -0.0000610351562500, -0.0003967285156250, 0.0000305175781250,
+0.0000000000000000, -0.0000610351562500, -0.0003967285156250, 0.0000915527343750,
+-0.0007324218750000, 0.0000915527343750, -0.0001831054687500, 0.0000305175781250,
+0.0000915527343750, 0.0000000000000000, -0.0003356933593750, 0.0000305175781250,
+0.0009765625000000, -0.0000305175781250, 0.0008544921875000, -0.0000915527343750,
+0.0008850097656250, 0.0000000000000000, 0.0017700195312500, 0.0000610351562500,
+0.0014953613281250, 0.0000000000000000, 0.0010681152343750, 0.0000000000000000,
+0.0017395019531250, 0.0000305175781250, 0.0006713867187500, 0.0000610351562500,
+-0.0002746582031250, -0.0000610351562500, 0.0007324218750000, -0.0000305175781250,
+-0.0025634765625000, 0.0000000000000000, -0.0016784667968750, -0.0000610351562500,
+-0.0026550292968750, 0.0000000000000000, -0.0054321289062500, 0.0000305175781250,
+-0.0036926269531250, 0.0000305175781250, -0.0059814453125000, 0.0000000000000000,
+-0.0065612792968750, 0.0000305175781250, -0.0054321289062500, 0.0000000000000000,
+-0.0061035156250000, -0.0000305175781250, -0.0067749023437500, 0.0000000000000000,
+-0.0031433105468750, 0.0000000000000000, -0.0050354003906250, 0.0000000000000000,
+-0.0028991699218750, 0.0000000000000000, 0.0005187988281250, 0.0000305175781250,
+-0.0026245117187500, 0.0000000000000000, 0.0049133300781250, 0.0000000000000000,
+0.0027770996093750, 0.0000000000000000, 0.0046691894531250, -0.0000305175781250,
+0.0104675292968750, -0.0000305175781250, 0.0065307617187500, 0.0000000000000000,
+0.0132446289062500, 0.0000305175781250, 0.0121765136718750, 0.0000610351562500,
+0.0129089355468750},
+{-0.0010070800781250, 0.0006713867187500, -0.0003967285156250, 0.0001831054687500,
+-0.0004577636718750, -0.0008850097656250, -0.0008239746093750, -0.0003967285156250,
+0.0006713867187500, -0.0003051757812500, 0.0006713867187500, -0.0007934570312500,
+0.0003662109375000, -0.0003051757812500, 0.0011291503906250, 0.0005493164062500,
+0.0012207031250000, 0.0001220703125000, -0.0003662109375000, 0.0002136230468750,
+0.0000000000000000, 0.0006408691406250, -0.0001831054687500, 0.0001525878906250,
+-0.0015869140625000, -0.0003967285156250, -0.0011596679687500, -0.0000305175781250,
+-0.0002746582031250, -0.0002441406250000, -0.0010070800781250, -0.0005187988281250,
+-0.0003356933593750, -0.0001525878906250, 0.0011596679687500, 0.0002441406250000,
+0.0004577636718750, -0.0000610351562500, 0.0006103515625000, 0.0002136230468750,
+0.0012817382812500, 0.0004272460937500, 0.0004882812500000, 0.0001220703125000,
+-0.0003662109375000, -0.0002136230468750, 0.0003051757812500, 0.0000305175781250,
+-0.0007629394531250, -0.0002136230468750, -0.0011596679687500, -0.0003662109375000,
+-0.0004882812500000, -0.0000915527343750, -0.0003051757812500, 0.0001525878906250,
+-0.0007629394531250, -0.0000610351562500, 0.0003051757812500, 0.0000610351562500,
+0.0005187988281250, 0.0002441406250000, 0.0000915527343750, 0.0000610351562500,
+0.0002441406250000, -0.0001220703125000, 0.0004882812500000, 0.0000305175781250,
+-0.0003051757812500, -0.0000915527343750, -0.0003967285156250, -0.0002136230468750,
+-0.0000915527343750, -0.0000610351562500, -0.0003662109375000, 0.0000610351562500,
+-0.0004882812500000, 0.0000000000000000, 0.0005798339843750, 0.0000305175781250,
+0.0002136230468750, 0.0001525878906250, 0.0006103515625000, 0.0000000000000000,
+0.0017395019531250, -0.0000610351562500, 0.0011291503906250, 0.0000000000000000,
+0.0008544921875000, 0.0000305175781250, 0.0017395019531250, -0.0001220703125000,
+0.0004577636718750, -0.0000305175781250, -0.0008239746093750, 0.0000610351562500,
+0.0003051757812500, 0.0000305175781250, -0.0030212402343750, 0.0000610351562500,
+-0.0028686523437500, 0.0000610351562500, -0.0028991699218750, 0.0000305175781250,
+-0.0049743652343750, -0.0000610351562500, -0.0042114257812500, -0.0000305175781250,
+-0.0028686523437500, 0.0000000000000000, -0.0046386718750000, -0.0000305175781250,
+-0.0017395019531250, -0.0000305175781250, 0.0011901855468750, 0.0000610351562500,
+-0.0016174316406250, 0.0000305175781250, 0.0069274902343750, -0.0000305175781250,
+0.0049743652343750, 0.0000610351562500, 0.0070190429687500, 0.0000305175781250,
+0.0138854980468750, -0.0000610351562500, 0.0098876953125000, -0.0000305175781250,
+0.0144348144531250, 0.0000305175781250, 0.0164794921875000, 0.0000000000000000,
+0.0124206542968750, 0.0000000000000000, 0.0130004882812500, 0.0000305175781250,
+0.0159301757812500, 0.0000610351562500, 0.0016479492187500, 0.0000000000000000,
+0.0086059570312500, 0.0000305175781250, -0.0003662109375000, 0.0000305175781250,
+-0.0138854980468750, 0.0000000000000000, -0.0011596679687500, 0.0000305175781250,
+-0.0329895019531250, 0.0000305175781250, -0.0226135253906250, 0.0000305175781250,
+-0.0323791503906250, 0.0000610351562500, -0.0581665039062500, 0.0000610351562500,
+-0.0392761230468750, 0.0000610351562500, -0.0732727050781250, 0.0000610351562500,
+-0.0659484863281250, 0.0000610351562500, -0.0714721679687500, 0.0000305175781250,
+-0.0931091308593750, 0.0000610351562500, -0.0736083984375000, 0.0000305175781250,
+-0.0965881347656250, 0.0000915527343750, -0.0910034179687500, 0.0000305175781250,
+-0.0900573730468750, 0.0000305175781250, -0.1019897460937500, 0.0000610351562500,
+-0.0890197753906250, 0.0000305175781250, -0.0983581542968750, 0.0000610351562500,
+-0.0944213867187500, 0.0000610351562500, -0.0891418457031250, 0.0000610351562500,
+-0.0917968750000000, 0.0000915527343750, -0.0848999023437500, 0.0000610351562500,
+-0.0762634277343750, 0.0000915527343750, -0.0733032226562500, 0.0000610351562500,
+-0.0650939941406250, 0.0000610351562500, -0.0552368164062500, 0.0000610351562500,
+-0.0518798828125000, 0.0000305175781250, -0.0380554199218750, 0.0000000000000000,
+-0.0373229980468750, 0.0000305175781250, -0.0285949707031250, 0.0000305175781250,
+-0.0160217285156250, 0.0000305175781250, -0.0191650390625000, 0.0000305175781250,
+-0.0023803710937500, 0.0000915527343750, -0.0025939941406250, 0.0000305175781250,
+0.0005187988281250, 0.0000305175781250, 0.0119018554687500, 0.0000000000000000,
+0.0089416503906250, 0.0000305175781250, 0.0142822265625000, 0.0000000000000000,
+0.0163879394531250, 0.0000000000000000, 0.0142211914062500, 0.0000000000000000,
+0.0147399902343750, 0.0000305175781250, 0.0156250000000000, -0.0000305175781250,
+0.0092773437500000, 0.0000000000000000, 0.0104675292968750, -0.0000305175781250,
+0.0072937011718750, 0.0000000000000000, 0.0010986328125000, 0.0000305175781250,
+0.0036010742187500, 0.0000305175781250, -0.0028076171875000, 0.0000305175781250,
+-0.0035400390625000, 0.0000305175781250, -0.0029907226562500, 0.0000305175781250,
+-0.0057678222656250, 0.0000000000000000, -0.0053405761718750, -0.0000610351562500,
+-0.0034179687500000, 0.0000305175781250, -0.0043334960937500, -0.0000305175781250,
+-0.0026550292968750, 0.0000000000000000, 0.0000305175781250, 0.0000610351562500,
+-0.0013122558593750, 0.0000000000000000, 0.0012817382812500, 0.0000000000000000,
+0.0019531250000000, -0.0000305175781250, 0.0011596679687500, -0.0000305175781250,
+0.0016174316406250, -0.0000305175781250, 0.0018615722656250, 0.0000000000000000,
+0.0002136230468750, 0.0000610351562500, 0.0003356933593750, -0.0000610351562500,
+0.0000915527343750, 0.0000305175781250, -0.0010681152343750, 0.0000915527343750,
+-0.0007629394531250, 0.0000305175781250, -0.0003967285156250, 0.0000000000000000,
+-0.0007629394531250, 0.0000610351562500, -0.0002441406250000, 0.0000305175781250,
+0.0006408691406250, -0.0000915527343750, 0.0002136230468750, -0.0000305175781250,
+0.0003967285156250, 0.0000000000000000, 0.0007934570312500, -0.0000610351562500,
+0.0002746582031250, 0.0000000000000000, -0.0003356933593750, 0.0000610351562500,
+0.0000610351562500, 0.0000305175781250, -0.0004577636718750, 0.0000305175781250,
+-0.0007934570312500, 0.0000915527343750, -0.0003356933593750, 0.0000305175781250,
+0.0000000000000000, -0.0001220703125000, -0.0003051757812500, 0.0000000000000000,
+0.0002746582031250, -0.0000915527343750, 0.0004882812500000, -0.0001525878906250,
+0.0001525878906250, -0.0000305175781250, 0.0000305175781250, 0.0000915527343750,
+0.0002746582031250, 0.0000000000000000, -0.0003662109375000, 0.0000305175781250,
+-0.0004882812500000, 0.0001831054687500, -0.0002136230468750, 0.0000305175781250,
+-0.0003356933593750, -0.0001525878906250, -0.0005187988281250, 0.0000000000000000,
+0.0002136230468750, -0.0001220703125000, 0.0002746582031250, -0.0002136230468750,
+0.0000610351562500, -0.0000915527343750, 0.0002441406250000, 0.0001220703125000,
+0.0003662109375000, 0.0000305175781250, -0.0002441406250000, -0.0000305175781250,
+-0.0002441406250000, 0.0000915527343750, -0.0001220703125000, 0.0000305175781250,
+-0.0002746582031250, -0.0002136230468750, -0.0002136230468750, -0.0001525878906250,
+0.0001525878906250, 0.0000000000000000, 0.0001220703125000, -0.0000915527343750,
+-0.0000305175781250, -0.0000305175781250, 0.0001220703125000, 0.0001220703125000,
+0.0002136230468750, 0.0000610351562500, -0.0001831054687500, -0.0000305175781250,
+-0.0002136230468750, 0.0000915527343750, -0.0000610351562500, 0.0000000000000000,
+-0.0001525878906250, -0.0002441406250000, -0.0002441406250000, -0.0001525878906250,
+0.0001525878906250, 0.0000000000000000, 0.0001525878906250, -0.0001525878906250,
+0.0000000000000000, -0.0000610351562500, 0.0000915527343750, 0.0001831054687500,
+0.0001525878906250, 0.0001525878906250, -0.0001525878906250, -0.0000610351562500,
+-0.0002746582031250, 0.0000305175781250, -0.0001220703125000, -0.0000915527343750,
+-0.0001220703125000, -0.0004577636718750, -0.0002441406250000, -0.0003051757812500,
+0.0000915527343750, -0.0000915527343750, 0.0001525878906250, -0.0003356933593750,
+0.0000305175781250, -0.0000915527343750, 0.0000915527343750, 0.0003662109375000,
+0.0001525878906250, 0.0001525878906250, -0.0000915527343750, 0.0001525878906250,
+-0.0001525878906250, 0.0003967285156250, -0.0000610351562500, 0.0000610351562500,
+-0.0000915527343750, -0.0003356933593750, -0.0001525878906250, -0.0000915527343750,
+0.0000915527343750, -0.0003051757812500, 0.0001220703125000, -0.0005187988281250,
+0.0000305175781250, -0.0002441406250000, 0.0000915527343750, -0.0000305175781250,
+0.0001220703125000, -0.0002136230468750, -0.0000305175781250, 0.0000610351562500,
+-0.0000610351562500, 0.0002136230468750, 0.0000305175781250, 0.0000305175781250,
+-0.0000610351562500, 0.0000000000000000, -0.0000915527343750, 0.0000610351562500,
+0.0000305175781250, 0.0000610351562500, 0.0000305175781250, -0.0001525878906250,
+0.0000305175781250, 0.0001220703125000, 0.0000610351562500, 0.0003967285156250,
+0.0000610351562500, 0.0002441406250000, -0.0000305175781250, 0.0004577636718750,
+0.0000000000000000, 0.0007019042968750, 0.0000000000000000, 0.0000915527343750,
+-0.0000610351562500, -0.0002746582031250, -0.0000610351562500, 0.0000915527343750,
+0.0000305175781250, -0.0014953613281250, -0.0000305175781250, -0.0016479492187500,
+0.0000000000000000, -0.0013427734375000, 0.0000305175781250, -0.0022583007812500,
+0.0000305175781250, -0.0021667480468750, -0.0000305175781250, 0.0005798339843750,
+0.0000305175781250, -0.0004272460937500, 0.0000000000000000, 0.0015563964843750,
+0.0000000000000000, 0.0051879882812500, 0.0000000000000000, 0.0034179687500000,
+0.0000000000000000, 0.0059509277343750, 0.0000000000000000, 0.0077819824218750,
+0.0000000000000000, 0.0041198730468750, 0.0000610351562500, 0.0031127929687500,
+0.0000305175781250, 0.0046997070312500, 0.0000000000000000, -0.0105590820312500,
+0.0000000000000000, -0.0063781738281250, 0.0000000000000000, -0.0167846679687500,
+-0.0000305175781250, -0.0333862304687500, 0.0000000000000000, -0.0262756347656250,
+0.0000000000000000, -0.0551147460937500, -0.0000305175781250, -0.0534973144531250,
+-0.0000305175781250, -0.0661315917968750, 0.0000305175781250, -0.0904235839843750,
+0.0000305175781250, -0.0820312500000000, 0.0000000000000000, -0.1137084960937500,
+0.0000305175781250, -0.1163330078125000, 0.0000000000000000, -0.1253051757812500,
+-0.0000305175781250, -0.1463012695312500, 0.0000305175781250, -0.1408081054687500,
+0.0000305175781250, -0.1610412597656250, -0.0000305175781250, -0.1639709472656250,
+0.0000000000000000, -0.1705322265625000, 0.0000000000000000, -0.1831970214843750,
+0.0000000000000000, -0.1787719726562500, 0.0000305175781250, -0.1934814453125000,
+0.0000000000000000, -0.1973876953125000, 0.0000305175781250, -0.1996154785156250,
+0.0000305175781250, -0.2065124511718750, 0.0000000000000000, -0.2060852050781250,
+0.0000000000000000, -0.2098999023437500, 0.0000000000000000, -0.2114868164062500,
+-0.0000305175781250, -0.2106933593750000, -0.0000305175781250, -0.2119140625000000,
+0.0000000000000000, -0.2128906250000000, 0.0000000000000000, -0.2082824707031250,
+-0.0000305175781250, -0.2135314941406250, 0.0000000000000000, -0.2075500488281250,
+-0.0000305175781250, -0.2004699707031250, -0.0000305175781250, -0.2072753906250000,
+-0.0000305175781250, -0.1828918457031250, 0.0000000000000000, -0.1912536621093750,
+-0.0000305175781250, -0.1777038574218750, 0.0000305175781250, -0.1558227539062500,
+0.0000000000000000},
+{-0.0000610351562500, -0.0003662109375000, -0.0001831054687500, -0.0001525878906250,
+0.0001220703125000, -0.0002746582031250, 0.0001831054687500, -0.0004577636718750,
+0.0000305175781250, -0.0001831054687500, 0.0000610351562500, 0.0001831054687500,
+0.0001220703125000, -0.0000610351562500, -0.0001220703125000, 0.0000915527343750,
+-0.0002136230468750, 0.0002746582031250, -0.0000305175781250, 0.0000915527343750,
+-0.0000610351562500, -0.0000610351562500, -0.0001220703125000, 0.0000305175781250,
+0.0000915527343750, 0.0001525878906250, 0.0000915527343750, -0.0000305175781250,
+0.0000305175781250, 0.0001525878906250, 0.0000610351562500, 0.0004577636718750,
+0.0000915527343750, 0.0002746582031250, -0.0000610351562500, -0.0000915527343750,
+-0.0000610351562500, 0.0003356933593750, -0.0000305175781250, -0.0003662109375000,
+-0.0000610351562500, -0.0014038085937500, -0.0000915527343750, -0.0007324218750000,
+0.0000305175781250, -0.0013122558593750, 0.0000610351562500, -0.0021667480468750,
+0.0000305175781250, -0.0004577636718750, 0.0000305175781250, 0.0007019042968750,
+0.0000610351562500, -0.0004882812500000, -0.0000305175781250, 0.0037841796875000,
+-0.0000610351562500, 0.0045776367187500, 0.0000000000000000, 0.0024108886718750,
+-0.0000305175781250, 0.0044250488281250, -0.0000305175781250, 0.0047302246093750,
+0.0000000000000000, -0.0105590820312500, 0.0000000000000000, -0.0035095214843750,
+0.0000000000000000, -0.0202026367187500, 0.0000305175781250, -0.0411376953125000,
+0.0000000000000000, -0.0286254882812500, 0.0000000000000000, -0.0752868652343750,
+0.0000000000000000, -0.0705261230468750, 0.0000000000000000, -0.0866088867187500,
+-0.0000305175781250, -0.1245422363281250, -0.0000305175781250, -0.1097106933593750,
+0.0000000000000000, -0.1471252441406250, 0.0000305175781250, -0.1509399414062500,
+0.0000305175781250, -0.1511535644531250, 0.0000305175781250, -0.1688842773437500,
+0.0000305175781250, -0.1636962890625000, 0.0000000000000000, -0.1610412597656250,
+0.0000305175781250, -0.1652526855468750, 0.0000305175781250, -0.1506042480468750,
+0.0000000000000000, -0.1370239257812500, 0.0000305175781250, -0.1420288085937500,
+0.0000000000000000, -0.1075439453125000, 0.0000305175781250, -0.1041259765625000,
+0.0000305175781250, -0.0880737304687500, 0.0000610351562500, -0.0588989257812500,
+0.0000610351562500, -0.0616149902343750, 0.0000610351562500, -0.0301513671875000,
+0.0000305175781250, -0.0196228027343750, 0.0000610351562500, -0.0055847167968750,
+0.0000000000000000, 0.0146179199218750, 0.0000000000000000, 0.0137329101562500,
+0.0000305175781250, 0.0403747558593750, 0.0000610351562500, 0.0588073730468750,
+0.0000915527343750, 0.0590820312500000, 0.0000305175781250, 0.0661621093750000,
+0.0000000000000000, 0.0796508789062500, 0.0000305175781250, 0.0625000000000000,
+0.0000000000000000, 0.0734863281250000, 0.0000305175781250, 0.0671081542968750,
+0.0000000000000000, 0.0506591796875000, 0.0000000000000000, 0.0628662109375000,
+0.0000305175781250, 0.0254821777343750, 0.0000610351562500, 0.0442810058593750,
+0.0000305175781250, 0.0216979980468750, 0.0000000000000000, -0.0166015625000000,
+0.0000000000000000, 0.0140991210937500, 0.0000000000000000, -0.0665588378906250,
+0.0000000000000000, -0.0498657226562500, 0.0000305175781250, -0.0735778808593750,
+0.0000305175781250, -0.1333007812500000, 0.0000305175781250, -0.0943603515625000,
+0.0000305175781250, -0.1772460937500000, 0.0000305175781250, -0.1656494140625000,
+0.0000305175781250, -0.1785583496093750, 0.0000000000000000, -0.2352600097656250,
+0.0000305175781250, -0.2043457031250000, 0.0000000000000000, -0.2447814941406250,
+-0.0000305175781250, -0.2521362304687500, 0.0000000000000000, -0.2363586425781250,
+0.0000000000000000, -0.2489929199218750, 0.0000305175781250, -0.2535705566406250,
+0.0000000000000000, -0.2068786621093750, 0.0000000000000000, -0.2268066406250000,
+0.0000305175781250, -0.1941833496093750, -0.0000305175781250, -0.1424560546875000,
+-0.0000305175781250, -0.1766662597656250, 0.0000305175781250, -0.0900573730468750,
+0.0000000000000000, -0.1002197265625000, 0.0000000000000000, -0.0833740234375000,
+0.0000305175781250, -0.0214233398437500, 0.0000305175781250, -0.0494079589843750,
+-0.0000305175781250, -0.0059204101562500, -0.0000305175781250, -0.0004882812500000,
+-0.0000305175781250, -0.0041198730468750, 0.0000000000000000, 0.0137023925781250,
+-0.0000305175781250, 0.0091552734375000, 0.0000305175781250, 0.0067138671875000,
+0.0000305175781250, 0.0104370117187500, 0.0000000000000000, 0.0064086914062500,
+0.0000305175781250, 0.0003356933593750, 0.0000610351562500, 0.0033264160156250,
+-0.0000610351562500, -0.0024108886718750, -0.0000915527343750, -0.0038146972656250,
+-0.0000305175781250, -0.0023193359375000, -0.0000610351562500, -0.0035705566406250,
+-0.0000915527343750, -0.0038452148437500, 0.0000915527343750, -0.0009765625000000,
+0.0001220703125000, -0.0013732910156250, 0.0000305175781250, -0.0007934570312500,
+0.0000305175781250, 0.0014038085937500, 0.0000915527343750, 0.0006103515625000,
+-0.0000610351562500, 0.0009155273437500, -0.0000915527343750, 0.0015258789062500,
+-0.0000305175781250, 0.0007629394531250, -0.0000915527343750, 0.0000915527343750,
+-0.0001220703125000, 0.0006103515625000, 0.0000915527343750, -0.0003051757812500,
+0.0001220703125000, -0.0005798339843750, 0.0000305175781250, -0.0001525878906250,
+0.0000915527343750, -0.0001525878906250, 0.0001525878906250, -0.0003662109375000,
+-0.0001220703125000, 0.0002746582031250, -0.0000610351562500, 0.0005187988281250,
+-0.0000305175781250, 0.0000915527343750, -0.0002441406250000, 0.0000000000000000,
+-0.0002136230468750, 0.0003051757812500, 0.0000305175781250, -0.0005798339843750,
+-0.0000915527343750, -0.0007629394531250, 0.0000000000000000, -0.0002746582031250,
+0.0003051757812500, -0.0003967285156250, 0.0001831054687500, -0.0006713867187500,
+0.0000000000000000, 0.0006408691406250, 0.0001525878906250, 0.0007934570312500,
+-0.0000305175781250, 0.0003051757812500, -0.0004272460937500, 0.0007629394531250,
+-0.0002136230468750, 0.0010375976562500, -0.0001831054687500, -0.0006713867187500,
+-0.0004882812500000, -0.0006103515625000, -0.0000915527343750, -0.0003051757812500,
+0.0003051757812500, -0.0011901855468750, 0.0000000000000000, -0.0012207031250000,
+0.0002746582031250, 0.0002441406250000, 0.0005493164062500, -0.0000305175781250,
+0.0002136230468750, -0.0000305175781250, -0.0002746582031250, 0.0013732910156250,
+-0.0004577636718750, -0.0011901855468750, 0.0005493164062500, 0.0004577636718750,
+0.0004577636718750, -0.0001831054687500, 0.0001525878906250, -0.0000305175781250,
+0.0003662109375000, 0.0009765625000000, 0.0006103515625000, 0.0008850097656250,
+-0.0005187988281250, -0.0002746582031250, -0.0004882812500000, 0.0000000000000000,
+-0.0003356933593750, -0.0000305175781250, -0.0010681152343750, -0.0010070800781250,
+-0.0009460449218750, -0.0008544921875000, 0.0001525878906250, 0.0000610351562500,
+-0.0002746582031250, -0.0002746582031250, 0.0000305175781250, -0.0001525878906250,
+0.0011901855468750, 0.0007934570312500, 0.0007629394531250, 0.0005493164062500,
+0.0002441406250000, 0.0000000000000000, 0.0008544921875000, 0.0003356933593750,
+0.0002441406250000, 0.0001220703125000, -0.0010070800781250, -0.0006408691406250,
+-0.0003051757812500, -0.0004272460937500, -0.0004882812500000, -0.0000610351562500,
+-0.0011291503906250, -0.0003356933593750, -0.0004577636718750, -0.0001831054687500,
+0.0004272460937500, 0.0004272460937500, -0.0001525878906250, 0.0002746582031250,
+0.0001831054687500, 0.0000305175781250, 0.0006408691406250, 0.0003051757812500,
+0.0002136230468750, 0.0000915527343750, -0.0003356933593750, -0.0003967285156250,
+0.0000915527343750, -0.0002441406250000, 0.0000610351562500, -0.0000915527343750,
+-0.0004882812500000, -0.0002441406250000, 0.0002746582031250, -0.0000610351562500,
+0.0012817382812500, 0.0003051757812500, 0.0004882812500000, 0.0001525878906250,
+0.0007934570312500, 0.0000305175781250, 0.0019531250000000, 0.0001831054687500,
+-0.0000305175781250, 0.0000610351562500, -0.0017089843750000, -0.0002136230468750,
+0.0001220703125000, -0.0000915527343750, -0.0033874511718750, -0.0000610351562500,
+-0.0043945312500000, -0.0001831054687500, -0.0019531250000000, -0.0000610351562500,
+-0.0025024414062500, 0.0001831054687500, -0.0039062500000000, 0.0000915527343750,
+0.0045776367187500, -0.0000305175781250, 0.0023193359375000, 0.0000915527343750,
+0.0037536621093750, 0.0000305175781250, 0.0117492675781250, -0.0001525878906250,
+0.0082702636718750, -0.0001220703125000, 0.0037841796875000, 0.0000000000000000,
+0.0124816894531250, -0.0000610351562500, -0.0030822753906250, 0.0000000000000000,
+-0.0153198242187500, 0.0001220703125000, -0.0018615722656250, 0.0000915527343750,
+-0.0572814941406250, 0.0000305175781250, -0.0406799316406250, 0.0000305175781250,
+-0.0684204101562500, 0.0000305175781250, -0.1190490722656250, 0.0000000000000000,
+-0.0856933593750000, 0.0000305175781250, -0.1685485839843750, 0.0000610351562500,
+-0.1604614257812500, 0.0000305175781250, -0.1736755371093750, 0.0000305175781250,
+-0.2297058105468750, 0.0000305175781250, -0.2012939453125000, 0.0001220703125000,
+-0.2296447753906250, -0.0000305175781250, -0.2447814941406250, 0.0000000000000000,
+-0.2177124023437500, 0.0000000000000000, -0.2159423828125000, -0.0000610351562500,
+-0.2339172363281250, 0.0000000000000000, -0.1503295898437500, -0.0000305175781250,
+-0.1871032714843750, -0.0000305175781250, -0.1301879882812500, -0.0000915527343750,
+-0.0468139648437500, -0.0000915527343750, -0.1203308105468750, -0.0000305175781250,
+0.0575866699218750, -0.0000915527343750, 0.0165100097656250, -0.0001525878906250,
+0.0634155273437500, -0.0001220703125000, 0.1908264160156250, -0.0000915527343750,
+0.0997924804687500, -0.0000915527343750, 0.2805786132812500, -0.0000915527343750,
+0.2435607910156250, -0.0000915527343750, 0.2745361328125000, -0.0001220703125000,
+0.4007873535156250, -0.0001831054687500, 0.3213806152343750, -0.0001525878906250,
+0.4457092285156250, -0.0001525878906250, 0.4286804199218750, -0.0001220703125000,
+0.4397583007812500, -0.0001525878906250, 0.5110778808593750, -0.0001220703125000,
+0.4535217285156250, -0.0001525878906250, 0.5212707519531250, -0.0000915527343750,
+0.5078430175781250, -0.0001220703125000, 0.5019531250000000, -0.0001220703125000,
+0.5344543457031250, -0.0001220703125000, 0.5009460449218750, -0.0001220703125000,
+0.5220336914062500, -0.0001220703125000, 0.5114746093750000, -0.0001525878906250,
+0.4958496093750000, -0.0000915527343750, 0.5010375976562500, -0.0001220703125000,
+0.4836120605468750, -0.0001525878906250, 0.4528503417968750, -0.0000915527343750,
+0.4474487304687500, -0.0001525878906250, 0.4190063476562500, -0.0001525878906250,
+0.3813476562500000, -0.0001525878906250, 0.3710327148437500, -0.0001220703125000,
+0.3111267089843750, -0.0001220703125000, 0.3122863769531250, -0.0001220703125000,
+0.2676391601562500, -0.0001220703125000, 0.2079772949218750, -0.0001220703125000,
+0.2244262695312500, -0.0001525878906250, 0.1175231933593750, -0.0001220703125000,
+0.1272888183593750, -0.0000915527343750, 0.0883789062500000, -0.0000915527343750,
+0.0020141601562500, -0.0000610351562500, 0.0303344726562500, -0.0000915527343750,
+-0.0708618164062500, -0.0000915527343750, -0.0721435546875000, -0.0000305175781250,
+-0.0913696289062500, -0.0000305175781250, -0.1585083007812500, -0.0000305175781250,
+-0.1396789550781250, -0.0000305175781250, -0.1784973144531250, 0.0000000000000000,
+-0.1894836425781250},
+{0.0083007812500000, -0.0000305175781250, 0.0116271972656250, 0.0000915527343750,
+0.0025634765625000, 0.0000305175781250, 0.0041503906250000, -0.0001831054687500,
+0.0093078613281250, -0.0001525878906250, -0.0322570800781250, 0.0000305175781250,
+-0.0124206542968750, -0.0000305175781250, -0.0438537597656250, 0.0000000000000000,
+-0.0908203125000000, 0.0001831054687500, -0.0549011230468750, 0.0001220703125000,
+-0.1434326171875000, 0.0000000000000000, -0.1344909667968750, 0.0000305175781250,
+-0.1454772949218750, 0.0000305175781250, -0.2061157226562500, -0.0000610351562500,
+-0.1795043945312500, -0.0000915527343750, -0.1961364746093750, 0.0000305175781250,
+-0.2187194824218750, 0.0000305175781250, -0.1791687011718750, 0.0000000000000000,
+-0.1627502441406250, 0.0000610351562500, -0.1898498535156250, 0.0000915527343750,
+-0.0513305664062500, -0.0000915527343750, -0.0980834960937500, -0.0001220703125000,
+-0.0146179199218750, -0.0000915527343750, 0.1170959472656250, -0.0001525878906250,
+0.0196228027343750, -0.0001525878906250, 0.2698974609375000, -0.0000610351562500,
+0.2206420898437500, -0.0000915527343750, 0.2865600585937500, -0.0000915527343750,
+0.4734191894531250, -0.0000610351562500, 0.3590087890625000, -0.0000915527343750,
+0.5651550292968750, -0.0001220703125000, 0.5540771484375000, -0.0001525878906250,
+0.5686340332031250, -0.0001525878906250, 0.6908264160156250, -0.0000305175781250,
+0.6331481933593750, -0.0000915527343750, 0.6972656250000000, 0.0000000000000000,
+0.7143249511718750, -0.0000305175781250, 0.6896362304687500, 0.0000305175781250,
+0.7058410644531250, 0.0000915527343750, 0.7147827148437500, 0.0000610351562500,
+0.6542968750000000, 0.0000610351562500, 0.6873168945312500, 0.0000610351562500,
+0.6541442871093750, 0.0000610351562500, 0.5924072265625000, 0.0001220703125000,
+0.6428527832031250, 0.0001220703125000, 0.5401306152343750, 0.0000915527343750,
+0.5627136230468750, 0.0001220703125000, 0.5421752929687500, 0.0001220703125000,
+0.4743957519531250, 0.0001220703125000, 0.5333557128906250, 0.0001220703125000,
+0.4571838378906250, 0.0001831054687500, 0.4742126464843750, 0.0001220703125000,
+0.4725341796875000, 0.0001220703125000, 0.4301147460937500, 0.0001525878906250,
+0.4719848632812500, 0.0001525878906250, 0.4333496093750000, 0.0001220703125000,
+0.4460754394531250, 0.0001525878906250, 0.4577636718750000, 0.0001831054687500,
+0.4414367675781250, 0.0001831054687500, 0.4655761718750000, 0.0001220703125000,
+0.4785156250000000, 0.0001525878906250, 0.4913330078125000, 0.0001220703125000,
+0.5095520019531250, 0.0000915527343750, 0.5263061523437500, 0.0000915527343750,
+0.5427246093750000, 0.0001220703125000, 0.5643310546875000, 0.0000915527343750,
+0.5682678222656250, 0.0000915527343750, 0.5830383300781250, 0.0000305175781250,
+0.6027526855468750, 0.0000610351562500, 0.5990905761718750, 0.0000610351562500,
+0.6097717285156250, -0.0000305175781250, 0.6153259277343750, 0.0000610351562500,
+0.6008300781250000, 0.0000305175781250, 0.5996093750000000, -0.0000305175781250,
+0.6055603027343750, 0.0000000000000000, 0.5392150878906250, -0.0000610351562500,
+0.5599975585937500, -0.0000610351562500, 0.5082397460937500, -0.0000305175781250,
+0.4350280761718750, -0.0000610351562500, 0.4738769531250000, -0.0000610351562500,
+0.3220214843750000, -0.0000915527343750, 0.3400878906250000, -0.0000610351562500,
+0.2742614746093750, -0.0000610351562500, 0.1441345214843750, -0.0000915527343750,
+0.1986694335937500, -0.0001220703125000, 0.0292358398437500, -0.0001220703125000,
+0.0314941406250000, -0.0001220703125000, 0.0014038085937500, -0.0000610351562500,
+-0.1085510253906250, -0.0000305175781250, -0.0688171386718750, -0.0001220703125000,
+-0.1364746093750000, 0.0000305175781250, -0.1553344726562500, 0.0000305175781250,
+-0.1380920410156250, -0.0000305175781250, -0.1545104980468750, 0.0000000000000000,
+-0.1577758789062500, 0.0000305175781250, -0.1083679199218750, -0.0000305175781250,
+-0.1213684082031250, -0.0000610351562500, -0.0967407226562500, -0.0000305175781250,
+-0.0484619140625000, 0.0000610351562500, -0.0718383789062500, -0.0000305175781250,
+-0.0201416015625000, 0.0000610351562500, -0.0151062011718750, 0.0000915527343750,
+-0.0152893066406250, 0.0000305175781250, 0.0088806152343750, 0.0000000000000000,
+0.0026245117187500, -0.0000305175781250, 0.0029296875000000, -0.0000610351562500,
+0.0068969726562500, -0.0000915527343750, 0.0031127929687500, 0.0000000000000000,
+-0.0019226074218750, -0.0000305175781250, 0.0005187988281250, -0.0000610351562500,
+-0.0016784667968750, 0.0000915527343750, -0.0033569335937500, 0.0000915527343750,
+-0.0014648437500000, 0.0000000000000000, -0.0000305175781250, -0.0000305175781250,
+-0.0011901855468750, 0.0000915527343750, 0.0007934570312500, -0.0001220703125000,
+0.0014953613281250, -0.0001220703125000, 0.0005187988281250, -0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, 0.0008239746093750, -0.0001220703125000,
+-0.0003356933593750, 0.0000610351562500, -0.0007324218750000, 0.0001525878906250,
+-0.0001831054687500, 0.0000000000000000, -0.0001831054687500, 0.0000000000000000,
+-0.0004577636718750, 0.0001220703125000, 0.0002441406250000, -0.0001220703125000,
+0.0004577636718750, -0.0001525878906250, 0.0000915527343750, -0.0000305175781250,
+0.0000305175781250, -0.0000610351562500, 0.0002441406250000, -0.0001831054687500,
+-0.0003051757812500, 0.0000610351562500, -0.0005187988281250, 0.0001220703125000,
+-0.0001831054687500, 0.0000000000000000, -0.0000610351562500, 0.0000000000000000,
+-0.0003051757812500, 0.0000915527343750, 0.0002136230468750, -0.0001220703125000,
+0.0003356933593750, -0.0001831054687500, 0.0000610351562500, -0.0000305175781250,
+-0.0000915527343750, 0.0000000000000000, -0.0000915527343750, -0.0000305175781250,
+-0.0001525878906250, 0.0000000000000000, -0.0003356933593750, 0.0000915527343750,
+-0.0001220703125000, -0.0000305175781250, 0.0000915527343750, -0.0001220703125000,
+-0.0001220703125000, 0.0000305175781250, 0.0000915527343750, -0.0001220703125000,
+0.0002746582031250, -0.0002441406250000, 0.0000305175781250, -0.0000915527343750,
+-0.0001831054687500, 0.0000610351562500, 0.0000000000000000, -0.0000610351562500,
+-0.0001220703125000, 0.0000000000000000, -0.0003662109375000, 0.0001220703125000,
+-0.0000915527343750, -0.0000610351562500, 0.0001525878906250, -0.0002441406250000,
+-0.0000610351562500, -0.0001525878906250, 0.0000915527343750, -0.0000915527343750,
+0.0003051757812500, -0.0002136230468750, 0.0000305175781250, -0.0000915527343750,
+-0.0002136230468750, 0.0002136230468750, -0.0000305175781250, 0.0000915527343750,
+-0.0001220703125000, 0.0000305175781250, -0.0003051757812500, 0.0002136230468750,
+-0.0000915527343750, 0.0000000000000000, 0.0001220703125000, -0.0003051757812500,
+-0.0000610351562500, -0.0001220703125000, 0.0000915527343750, 0.0000915527343750,
+0.0002441406250000, -0.0001220703125000, 0.0000305175781250, -0.0000305175781250,
+-0.0001525878906250, 0.0004272460937500, 0.0000000000000000, 0.0002746582031250,
+-0.0000610351562500, -0.0006103515625000, -0.0002441406250000, -0.0002746582031250,
+-0.0000915527343750, -0.0002746582031250, 0.0000610351562500, -0.0013732910156250,
+-0.0000305175781250, -0.0011291503906250, 0.0000915527343750, 0.0010375976562500,
+0.0001831054687500, 0.0002136230468750, 0.0000610351562500, 0.0005493164062500,
+-0.0000305175781250, 0.0038452148437500, 0.0000610351562500, 0.0026855468750000,
+-0.0000610351562500, -0.0050964355468750, -0.0001525878906250, 0.0025024414062500,
+-0.0000610351562500, -0.0142211914062500, 0.0000000000000000, -0.0303955078125000,
+-0.0000305175781250, -0.0166015625000000, 0.0000305175781250, -0.0654907226562500,
+0.0001220703125000, -0.0609436035156250, 0.0000610351562500, -0.0723876953125000,
+0.0000305175781250, -0.1112365722656250, 0.0000610351562500, -0.0959167480468750,
+-0.0000305175781250, -0.1021118164062500, -0.0000915527343750, -0.1207885742187500,
+-0.0000610351562500, -0.0810241699218750, -0.0000305175781250, -0.0541992187500000,
+-0.0000610351562500, -0.0758056640625000, 0.0000305175781250, 0.0495300292968750,
+0.0000305175781250, 0.0273437500000000, 0.0000000000000000, 0.0885925292968750,
+0.0000305175781250, 0.2033386230468750, 0.0000610351562500, 0.1488342285156250,
+-0.0000610351562500, 0.2929077148437500, -0.0000610351562500, 0.2959899902343750,
+0.0000000000000000, 0.3153991699218750, 0.0000000000000000, 0.4041442871093750,
+-0.0000305175781250, 0.3754577636718750, 0.0000610351562500, 0.4122314453125000,
+0.0000305175781250, 0.4290466308593750, 0.0000305175781250, 0.4094238281250000,
+0.0000305175781250, 0.4059753417968750, 0.0000000000000000, 0.4099121093750000,
+-0.0000305175781250, 0.3624267578125000, -0.0000610351562500, 0.3666381835937500,
+0.0000000000000000, 0.3320922851562500, -0.0000305175781250, 0.2805175781250000,
+-0.0000305175781250, 0.2956848144531250, 0.0000000000000000, 0.2190856933593750,
+0.0000000000000000, 0.2077331542968750, 0.0000000000000000, 0.1836242675781250,
+0.0000000000000000, 0.1311645507812500, 0.0000305175781250, 0.1366882324218750,
+0.0000000000000000, 0.0901489257812500, -0.0000305175781250, 0.0741882324218750,
+0.0000000000000000, 0.0563659667968750, 0.0000000000000000, 0.0293884277343750,
+-0.0000305175781250, 0.0315856933593750, 0.0000000000000000, -0.0009155273437500,
+0.0000000000000000, -0.0235290527343750, 0.0000000000000000, -0.0231628417968750,
+0.0000305175781250, -0.0302124023437500, 0.0000305175781250, -0.0454406738281250,
+-0.0000610351562500, -0.0256347656250000, -0.0000305175781250, -0.0380554199218750,
+0.0000000000000000, -0.0289611816406250, 0.0000000000000000, -0.0097961425781250,
+-0.0000610351562500, -0.0252380371093750, -0.0000305175781250, 0.0234680175781250,
+0.0000000000000000, -0.0022583007812500, 0.0000000000000000, 0.0271911621093750,
+0.0000000000000000, 0.0764770507812500, -0.0000305175781250, 0.0380554199218750,
+0.0000000000000000, 0.1557006835937500, -0.0000610351562500, 0.1248779296875000,
+-0.0000305175781250, 0.1725463867187500, 0.0000000000000000, 0.2669677734375000,
+-0.0000305175781250, 0.2007751464843750, 0.0000000000000000, 0.3692321777343750,
+0.0000000000000000, 0.3299560546875000, -0.0000305175781250, 0.3825683593750000,
+0.0000305175781250, 0.5145263671875000, 0.0000000000000000, 0.4292602539062500,
+0.0000000000000000, 0.6030883789062500, -0.0000305175781250, 0.5871887207031250,
+0.0000000000000000, 0.5936889648437500, 0.0000000000000000, 0.7033996582031250,
+0.0000000000000000, 0.6585083007812500, 0.0000000000000000, 0.6726074218750000,
+0.0000305175781250, 0.7183532714843750, 0.0000305175781250, 0.6428833007812500,
+0.0000000000000000, 0.5941772460937500, -0.0000305175781250, 0.6524047851562500,
+0.0000610351562500, 0.4279785156250000, 0.0000305175781250, 0.4888610839843750,
+0.0000610351562500, 0.3830566406250000, 0.0000915527343750, 0.1737365722656250,
+0.0000915527343750, 0.2944335937500000, 0.0000305175781250, 0.0174255371093750,
+0.0000610351562500, 0.0299987792968750, 0.0000305175781250, -0.0063476562500000,
+-0.0000610351562500, -0.1809692382812500, 0.0000000000000000, -0.1044616699218750,
+0.0000610351562500, -0.1994018554687500, 0.0000000000000000, -0.2271423339843750,
+0.0000000000000000, -0.1939697265625000, 0.0000305175781250, -0.2118225097656250,
+0.0000305175781250, -0.2202758789062500, -0.0000610351562500, -0.1451721191406250,
+-0.0000305175781250, -0.1622009277343750, -0.0000305175781250, -0.1290283203125000,
+-0.0000915527343750},
+{-0.0000305175781250, 0.3160400390625000, 0.0000000000000000, 0.3126220703125000,
+-0.0000610351562500, 0.3286132812500000, 0.0000000000000000, 0.2206115722656250,
+0.0000305175781250, 0.2425537109375000, 0.0000000000000000, 0.1859436035156250,
+-0.0000305175781250, 0.0814514160156250, 0.0000000000000000, 0.1233520507812500,
+-0.0000305175781250, -0.0210571289062500, -0.0000610351562500, -0.0212097167968750,
+-0.0000305175781250, -0.0508728027343750, 0.0000000000000000, -0.1446228027343750,
+0.0000000000000000, -0.1138305664062500, 0.0000305175781250, -0.1896362304687500,
+0.0000305175781250, -0.1954650878906250, 0.0000000000000000, -0.2021789550781250,
+0.0000000000000000, -0.2429199218750000, 0.0000305175781250, -0.2324523925781250,
+0.0000000000000000, -0.2580261230468750, 0.0000000000000000, -0.2601013183593750,
+0.0000610351562500, -0.2605590820312500, 0.0000000000000000, -0.2698669433593750,
+0.0000000000000000, -0.2619018554687500, 0.0000305175781250, -0.2692871093750000,
+0.0000305175781250, -0.2641296386718750, 0.0000305175781250, -0.2657775878906250,
+0.0000305175781250, -0.2727050781250000, 0.0000000000000000, -0.2640380859375000,
+0.0000305175781250, -0.2704162597656250, 0.0000305175781250, -0.2625427246093750,
+0.0000305175781250, -0.2679748535156250, 0.0000305175781250, -0.2770385742187500,
+0.0000305175781250, -0.2648315429687500, 0.0000000000000000, -0.2878417968750000,
+0.0000305175781250, -0.2834472656250000, 0.0000305175781250, -0.2873229980468750,
+0.0000305175781250, -0.3028564453125000, 0.0000305175781250, -0.2961425781250000,
+0.0000000000000000, -0.3211669921875000, 0.0000000000000000, -0.3146362304687500,
+0.0000305175781250, -0.3257141113281250, 0.0000000000000000, -0.3464355468750000,
+0.0000000000000000, -0.3313293457031250, 0.0000305175781250, -0.3673095703125000,
+0.0000305175781250, -0.3642578125000000, 0.0000000000000000, -0.3645019531250000,
+0.0000305175781250, -0.3868103027343750, 0.0000305175781250, -0.3794250488281250,
+0.0000000000000000, -0.3788452148437500, -0.0000610351562500, -0.3941650390625000,
+-0.0000305175781250, -0.3692016601562500, 0.0000000000000000, -0.3576660156250000,
+0.0000305175781250, -0.3786315917968750, 0.0000305175781250, -0.2630920410156250,
+0.0000000000000000, -0.3168029785156250, -0.0000305175781250, -0.2381591796875000,
+-0.0000610351562500, -0.1211242675781250, -0.0000610351562500, -0.2143859863281250,
+-0.0000305175781250, 0.0605773925781250, -0.0000610351562500, 0.0011291503906250,
+-0.0000305175781250, 0.1068420410156250, 0.0000000000000000, 0.3385925292968750,
+-0.0000305175781250, 0.2033996582031250, 0.0000000000000000, 0.4976806640625000,
+0.0000000000000000, 0.4801940917968750, -0.0000305175781250, 0.4971923828125000,
+-0.0000610351562500, 0.6837768554687500, -0.0000305175781250, 0.6147460937500000,
+0.0000000000000000, 0.6313171386718750, 0.0000000000000000, 0.6900024414062500,
+-0.0000305175781250, 0.5909729003906250, 0.0000610351562500, 0.5202026367187500,
+0.0000305175781250, 0.5898437500000000, 0.0000000000000000, 0.3266601562500000,
+0.0000610351562500, 0.3740234375000000, 0.0000000000000000, 0.2809143066406250,
+-0.0000915527343750, 0.0599365234375000, -0.0000305175781250, 0.1621093750000000,
+0.0000305175781250, -0.0723266601562500, -0.0000305175781250, -0.0885620117187500,
+0.0000000000000000, -0.0924987792968750, 0.0001220703125000, -0.2107849121093750,
+0.0000915527343750, -0.1747741699218750, 0.0000000000000000, -0.1756591796875000,
+0.0000305175781250, -0.2087402343750000, 0.0000000000000000, -0.1649169921875000,
+-0.0001831054687500, -0.1201477050781250, -0.0001525878906250, -0.1507873535156250,
+-0.0000305175781250, -0.0588989257812500, -0.0000915527343750, -0.0612792968750000,
+-0.0000305175781250, -0.0518493652343750, 0.0001220703125000, 0.0028991699218750,
+0.0000915527343750, -0.0177001953125000, 0.0000000000000000, 0.0032348632812500,
+0.0000915527343750, 0.0121765136718750, -0.0000305175781250, 0.0037231445312500,
+-0.0002136230468750, 0.0038452148437500, -0.0001220703125000, 0.0070800781250000,
+-0.0001220703125000, -0.0018005371093750, -0.0002136230468750, -0.0032348632812500,
+-0.0000610351562500, -0.0008544921875000, 0.0001831054687500, -0.0028686523437500,
+0.0000000000000000, -0.0036926269531250, 0.0001220703125000, 0.0004882812500000,
+0.0002441406250000, 0.0004577636718750, 0.0000000000000000, -0.0002441406250000,
+-0.0001831054687500, 0.0015563964843750, 0.0000000000000000, 0.0016174316406250,
+-0.0003051757812500, -0.0003356933593750, -0.0004272460937500, -0.0003051757812500,
+-0.0001831054687500, 0.0001525878906250, -0.0000305175781250, -0.0006713867187500,
+-0.0002441406250000, -0.0008544921875000, 0.0002441406250000, 0.0004882812500000,
+0.0003662109375000, 0.0005493164062500, 0.0001831054687500, 0.0000000000000000,
+0.0002441406250000, 0.0003662109375000, 0.0003662109375000, 0.0007019042968750,
+-0.0003356933593750, -0.0007019042968750, -0.0003356933593750, -0.0007629394531250,
+-0.0002136230468750, -0.0003051757812500, -0.0006103515625000, -0.0005798339843750,
+0.0004272460937500, 0.0007324218750000, -0.0000610351562500, -0.0007324218750000,
+0.0003662109375000, -0.0006103515625000, 0.0000000000000000, -0.0001525878906250,
+-0.0006408691406250, -0.0003051757812500, -0.0003356933593750, -0.0006408691406250,
+-0.0002441406250000, 0.0005187988281250, -0.0005493164062500, 0.0007324218750000,
+-0.0001525878906250, 0.0001525878906250, 0.0003356933593750, 0.0003051757812500,
+-0.0000305175781250, 0.0006713867187500, 0.0003662109375000, -0.0005187988281250,
+0.0004882812500000, -0.0007324218750000, 0.0003662109375000, -0.0002136230468750,
+0.0003662109375000, -0.0004577636718750, 0.0003662109375000, -0.0007019042968750,
+0.0003662109375000, 0.0003356933593750, 0.0004882812500000, 0.0003967285156250,
+-0.0000610351562500, 0.0000610351562500, -0.0002746582031250, 0.0003662109375000,
+0.0001831054687500, 0.0005493164062500, -0.0018615722656250, -0.0003051757812500,
+-0.0019836425781250, -0.0003356933593750, -0.0008239746093750, -0.0001220703125000,
+-0.0018615722656250, -0.0004272460937500, -0.0025634765625000, -0.0005187988281250,
+0.0037841796875000, 0.0002136230468750, 0.0023803710937500, 0.0002441406250000,
+0.0017089843750000, 0.0000610351562500, 0.0090637207031250, 0.0003051757812500,
+0.0071716308593750, 0.0003356933593750, -0.0144653320312500, -0.0002441406250000,
+0.0032043457031250, -0.0002136230468750, -0.0295715332031250, -0.0000610351562500,
+-0.0657653808593750, -0.0002441406250000, -0.0347595214843750, -0.0003356933593750,
+-0.1291503906250000, 0.0001220703125000, -0.1205139160156250, 0.0001525878906250,
+-0.1248168945312500, 0.0000305175781250, -0.1901245117187500, 0.0001831054687500,
+-0.1668395996093750, 0.0002441406250000, -0.1335144042968750, -0.0001831054687500,
+-0.1834716796875000, -0.0001831054687500, -0.1004333496093750, -0.0000915527343750,
+-0.0203247070312500, -0.0002441406250000, -0.0961608886718750, -0.0002441406250000,
+0.1483764648437500, -0.0000305175781250, 0.0954589843750000, 0.0000000000000000,
+0.1718750000000000, -0.0000610351562500, 0.3688659667968750, 0.0000000000000000,
+0.2542724609375000, 0.0000305175781250, 0.4536437988281250, -0.0001220703125000,
+0.4649353027343750, -0.0001525878906250, 0.4519653320312500, -0.0000610351562500,
+0.5510559082031250, -0.0000305175781250, 0.5249328613281250, -0.0001220703125000,
+0.4800720214843750, 0.0001220703125000, 0.5305175781250000, 0.0001525878906250,
+0.4505310058593750, 0.0000915527343750, 0.3665161132812500, 0.0001220703125000,
+0.4372253417968750, 0.0001831054687500, 0.1932678222656250, 0.0000915527343750,
+0.2441711425781250, 0.0000610351562500, 0.1556396484375000, 0.0000915527343750,
+-0.0392150878906250, 0.0001525878906250, 0.0824584960937500, 0.0001525878906250,
+-0.1699523925781250, 0.0001220703125000, -0.1471557617187500, 0.0001831054687500,
+-0.1752624511718750, 0.0001220703125000, -0.3342285156250000, 0.0000305175781250,
+-0.2558288574218750, 0.0000915527343750, -0.3575134277343750, 0.0000305175781250,
+-0.3742675781250000, 0.0000305175781250, -0.3543090820312500, -0.0000305175781250,
+-0.3918457031250000, 0.0000000000000000, -0.3932189941406250, 0.0000610351562500,
+-0.3603820800781250, -0.0000610351562500, -0.3801574707031250, 0.0000000000000000,
+-0.3604431152343750, -0.0000305175781250, -0.3271789550781250, -0.0000915527343750,
+-0.3571777343750000, -0.0000610351562500, -0.2940368652343750, -0.0000305175781250,
+-0.3093261718750000, -0.0000610351562500, -0.2996215820312500, -0.0000305175781250,
+-0.2549133300781250, 0.0000000000000000, -0.2824401855468750, -0.0000305175781250,
+-0.2401428222656250, -0.0000305175781250, -0.2429199218750000, -0.0000305175781250,
+-0.2402648925781250, -0.0000305175781250, -0.2182006835937500, 0.0000000000000000,
+-0.2340393066406250, -0.0000305175781250, -0.2107849121093750, 0.0000000000000000,
+-0.2144165039062500, -0.0000305175781250, -0.2143554687500000, 0.0000000000000000,
+-0.2019348144531250, 0.0000000000000000, -0.2109985351562500, -0.0000305175781250,
+-0.1992492675781250, 0.0000000000000000, -0.2037963867187500, -0.0000305175781250,
+-0.2044677734375000, 0.0000000000000000, -0.1965026855468750, -0.0000305175781250,
+-0.2038574218750000, -0.0000305175781250, -0.2048645019531250, 0.0000000000000000,
+-0.2093811035156250, -0.0000610351562500, -0.2155456542968750, -0.0000610351562500,
+-0.2205200195312500, 0.0000000000000000, -0.2306213378906250, -0.0000305175781250,
+-0.2446594238281250, -0.0000305175781250, -0.2445678710937500, -0.0000610351562500,
+-0.2603759765625000, -0.0000305175781250, -0.2777099609375000, -0.0000610351562500,
+-0.2731018066406250, -0.0000610351562500, -0.3054199218750000, 0.0000000000000000,
+-0.3032226562500000, -0.0000305175781250, -0.3119812011718750, -0.0000305175781250,
+-0.3390197753906250, -0.0000305175781250, -0.3319091796875000, -0.0000305175781250,
+-0.3377685546875000, -0.0000305175781250, -0.3494262695312500, -0.0000305175781250,
+-0.3274230957031250, 0.0000000000000000, -0.3177795410156250, 0.0000305175781250,
+-0.3349304199218750, -0.0000305175781250, -0.2527770996093750, 0.0000915527343750,
+-0.2767333984375000, 0.0000915527343750, -0.2135925292968750, 0.0000915527343750,
+-0.1224365234375000, 0.0000915527343750, -0.1685791015625000, 0.0000915527343750,
+0.0170593261718750, 0.0000610351562500, -0.0044555664062500, 0.0000610351562500,
+0.0632934570312500, 0.0000915527343750, 0.2150573730468750, 0.0000915527343750,
+0.1500854492187500, 0.0000610351562500, 0.3133850097656250, 0.0000915527343750,
+0.3303527832031250, 0.0001525878906250, 0.3320312500000000, 0.0000610351562500,
+0.4168090820312500, 0.0000610351562500, 0.3965454101562500, 0.0000610351562500,
+0.3557739257812500, -0.0000915527343750, 0.3990478515625000, -0.0000915527343750,
+0.3243713378906250, -0.0000610351562500, 0.2373657226562500, -0.0001220703125000,
+0.2948303222656250, -0.0000915527343750, 0.1031188964843750, 0.0000000000000000,
+0.1145935058593750, -0.0000610351562500, 0.0716247558593750, -0.0000915527343750,
+-0.0651245117187500, 0.0000000000000000, -0.0120849609375000, -0.0000305175781250,
+-0.0935363769531250, -0.0000610351562500, -0.1192626953125000, -0.0000610351562500,
+-0.0914611816406250, 0.0000000000000000, -0.1026306152343750, -0.0000610351562500,
+-0.1116943359375000, -0.0000915527343750, -0.0538330078125000, 0.0000610351562500,
+-0.0621948242187500, 0.0000610351562500, -0.0488891601562500, 0.0000000000000000,
+-0.0049133300781250, 0.0000610351562500, -0.0220642089843750, 0.0000915527343750,
+-0.0011291503906250},
+{-0.2001037597656250, 0.0000305175781250, -0.2456665039062500, 0.0000305175781250,
+-0.2269592285156250, 0.0000000000000000, -0.2497253417968750, 0.0000305175781250,
+-0.2909240722656250, 0.0000610351562500, -0.2571411132812500, 0.0000610351562500,
+-0.3307495117187500, 0.0000305175781250, -0.3139343261718750, 0.0000915527343750,
+-0.3263854980468750, 0.0000610351562500, -0.3778686523437500, 0.0000610351562500,
+-0.3444824218750000, 0.0000305175781250, -0.3950805664062500, 0.0000610351562500,
+-0.3894653320312500, 0.0000915527343750, -0.3923339843750000, 0.0000610351562500,
+-0.4198608398437500, 0.0000610351562500, -0.3983459472656250, 0.0000610351562500,
+-0.4223632812500000, 0.0000305175781250, -0.4167175292968750, 0.0000000000000000,
+-0.4132690429687500, 0.0000305175781250, -0.4241027832031250, 0.0000610351562500,
+-0.4111633300781250, 0.0000305175781250, -0.4167175292968750, 0.0000915527343750,
+-0.4142761230468750, 0.0000915527343750, -0.4050598144531250, 0.0000305175781250,
+-0.4025268554687500, 0.0000610351562500, -0.3982543945312500, 0.0000915527343750,
+-0.3796081542968750, 0.0000610351562500, -0.3769836425781250, 0.0000305175781250,
+-0.3611145019531250, 0.0000610351562500, -0.3398742675781250, 0.0000610351562500,
+-0.3381042480468750, 0.0000610351562500, -0.3048400878906250, 0.0000915527343750,
+-0.3048400878906250, 0.0000610351562500, -0.2859802246093750, 0.0000915527343750,
+-0.2561340332031250, 0.0000915527343750, -0.2636413574218750, 0.0000915527343750,
+-0.2211914062500000, 0.0000610351562500, -0.2213745117187500, 0.0000305175781250,
+-0.2101745605468750, 0.0000610351562500, -0.1787109375000000, 0.0000305175781250,
+-0.1869506835937500, 0.0000305175781250, -0.1699523925781250, 0.0000305175781250,
+-0.1628417968750000, 0.0000610351562500, -0.1697998046875000, -0.0000305175781250,
+-0.1674804687500000, -0.0000305175781250, -0.1642150878906250, 0.0000000000000000,
+-0.1912536621093750, -0.0000305175781250, -0.1862792968750000, -0.0000305175781250,
+-0.2048034667968750, -0.0000305175781250, -0.2355651855468750, -0.0000610351562500,
+-0.2236633300781250, -0.0000610351562500, -0.2579040527343750, 0.0000305175781250,
+-0.2665405273437500, -0.0000305175781250, -0.2499389648437500, 0.0000305175781250,
+-0.2595825195312500, 0.0000610351562500, -0.2648315429687500, 0.0000000000000000,
+-0.1848449707031250, 0.0000305175781250, -0.2168579101562500, 0.0000000000000000,
+-0.1430358886718750, 0.0000305175781250, -0.0445251464843750, 0.0000305175781250,
+-0.1026000976562500, 0.0000305175781250, 0.1078491210937500, 0.0000610351562500,
+0.0925903320312500, 0.0001220703125000, 0.1409606933593750, 0.0000915527343750,
+0.2926940917968750, 0.0000915527343750, 0.2304077148437500, 0.0001220703125000,
+0.3198547363281250, -0.0000305175781250, 0.3576965332031250, -0.0000305175781250,
+0.3060607910156250, -0.0000305175781250, 0.3072509765625000, -0.0001220703125000,
+0.3303833007812500, -0.0001220703125000, 0.1734008789062500, -0.0000610351562500,
+0.2041931152343750, -0.0000915527343750, 0.1385803222656250, -0.0000305175781250,
+-0.0032653808593750, 0.0000610351562500, 0.0639038085937500, -0.0000305175781250,
+-0.0621948242187500, -0.0000610351562500, -0.0830383300781250, -0.0000305175781250,
+-0.0627441406250000, -0.0000610351562500, -0.1054382324218750, -0.0001525878906250,
+-0.1021728515625000, -0.0001525878906250, -0.0579833984375000, 0.0000305175781250,
+-0.0721435546875000, 0.0000000000000000, -0.0534973144531250, 0.0000000000000000,
+-0.0098571777343750, 0.0001525878906250, -0.0296936035156250, 0.0001220703125000,
+-0.0032653808593750, -0.0000915527343750, 0.0046997070312500, 0.0000305175781250,
+-0.0018310546875000, 0.0000000000000000, 0.0020751953125000, -0.0001831054687500,
+0.0031738281250000, -0.0001525878906250, -0.0011901855468750, 0.0000305175781250,
+-0.0020751953125000, -0.0000610351562500, -0.0005798339843750, -0.0000610351562500,
+-0.0009155273437500, 0.0001220703125000, -0.0016479492187500, 0.0001220703125000,
+0.0003967285156250, -0.0000915527343750, 0.0009155273437500, 0.0000000000000000,
+0.0000915527343750, -0.0000305175781250, 0.0002441406250000, -0.0001525878906250,
+0.0006103515625000, -0.0001525878906250, -0.0003051757812500, -0.0000305175781250,
+-0.0005187988281250, 0.0000000000000000, -0.0001525878906250, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, 0.0000915527343750, 0.0000000000000000,
+0.0000305175781250, -0.0000915527343750, 0.0002441406250000, -0.0000915527343750,
+0.0000305175781250, -0.0000305175781250, -0.0002441406250000, -0.0000305175781250,
+-0.0000305175781250, -0.0000610351562500, -0.0000305175781250, -0.0000305175781250,
+-0.0002136230468750, 0.0000610351562500, -0.0000915527343750, -0.0000610351562500,
+0.0002136230468750, -0.0002136230468750, 0.0000915527343750, -0.0000915527343750,
+-0.0000610351562500, 0.0000305175781250, 0.0001220703125000, -0.0001220703125000,
+0.0000000000000000, -0.0000305175781250, -0.0003662109375000, 0.0004882812500000,
+-0.0002441406250000, 0.0003356933593750, 0.0000305175781250, -0.0003967285156250,
+-0.0001525878906250, -0.0000915527343750, -0.0000915527343750, -0.0002441406250000,
+0.0002746582031250, -0.0012817382812500, 0.0001525878906250, -0.0010375976562500,
+-0.0000610351562500, 0.0008239746093750, 0.0000305175781250, 0.0006713867187500,
+0.0000000000000000, -0.0012512207031250, -0.0003356933593750, 0.0013732910156250,
+-0.0002441406250000, 0.0017089843750000, 0.0000000000000000, -0.0172729492187500,
+-0.0001220703125000, -0.0062866210937500, -0.0000610351562500, -0.0245971679687500,
+0.0002441406250000, -0.0531311035156250, 0.0001831054687500, -0.0346069335937500,
+-0.0000610351562500, -0.0681152343750000, 0.0000305175781250, -0.0812072753906250,
+0.0000000000000000, -0.0521545410156250, -0.0002136230468750, -0.0534667968750000,
+-0.0001220703125000, -0.0691528320312500, 0.0000000000000000, 0.0423583984375000,
+-0.0000915527343750, 0.0096130371093750, -0.0000305175781250, 0.0644226074218750,
+0.0001525878906250, 0.1773071289062500, 0.0000610351562500, 0.1179504394531250,
+-0.0000305175781250, 0.2245483398437500, 0.0000610351562500, 0.2477722167968750,
+0.0000305175781250, 0.2131958007812500, -0.0001525878906250, 0.2416687011718750,
+-0.0000915527343750, 0.2512207031250000, 0.0000000000000000, 0.1430969238281250,
+-0.0000915527343750, 0.1734313964843750, -0.0000305175781250, 0.1118774414062500,
+0.0000915527343750, -0.0021362304687500, 0.0000000000000000, 0.0509948730468750,
+0.0000000000000000, -0.0784606933593750, 0.0000305175781250, -0.0925292968750000,
+0.0000305175781250, -0.0905456542968750, -0.0000610351562500, -0.1496887207031250,
+0.0000000000000000, -0.1355285644531250, -0.0000305175781250, -0.1242065429687500,
+-0.0000610351562500, -0.1368408203125000, 0.0000000000000000, -0.1196899414062500,
+0.0000610351562500, -0.0927124023437500, -0.0000305175781250, -0.1052856445312500,
+0.0000000000000000, -0.0922851562500000, 0.0000610351562500, -0.0808105468750000,
+0.0000000000000000, -0.1021728515625000, 0.0000000000000000, -0.1127014160156250,
+0.0000000000000000, -0.1018371582031250, -0.0000610351562500, -0.1681213378906250,
+-0.0000305175781250, -0.1567077636718750, 0.0000000000000000, -0.1877136230468750,
+0.0000305175781250, -0.2489318847656250, -0.0000610351562500, -0.2255249023437500,
+0.0000000000000000, -0.3018798828125000, 0.0000610351562500, -0.3034973144531250,
+0.0000000000000000, -0.3244018554687500, -0.0000305175781250, -0.3774719238281250,
+0.0000305175781250, -0.3603515625000000, 0.0000000000000000, -0.4088134765625000,
+0.0000000000000000, -0.4148559570312500, -0.0000305175781250, -0.4212951660156250,
+0.0000305175781250, -0.4463806152343750, 0.0000305175781250, -0.4386291503906250,
+0.0000305175781250, -0.4553527832031250, 0.0000305175781250, -0.4555358886718750,
+0.0000305175781250, -0.4572143554687500, 0.0000000000000000, -0.4656372070312500,
+0.0000305175781250, -0.4591979980468750, 0.0000305175781250, -0.4642639160156250,
+0.0000000000000000, -0.4611511230468750, 0.0000305175781250, -0.4618530273437500,
+0.0000610351562500, -0.4657287597656250, 0.0000000000000000, -0.4599304199218750,
+0.0000305175781250, -0.4658508300781250, 0.0000000000000000, -0.4637756347656250,
+0.0000000000000000, -0.4625549316406250, 0.0000305175781250, -0.4663085937500000,
+0.0000305175781250, -0.4649963378906250, 0.0000610351562500, -0.4661865234375000,
+0.0000000000000000, -0.4649963378906250, 0.0000000000000000, -0.4659729003906250,
+-0.0000610351562500, -0.4685974121093750, -0.0000305175781250, -0.4644165039062500,
+0.0000000000000000, -0.4575195312500000, 0.0000305175781250, -0.4629211425781250,
+0.0000000000000000, -0.4503784179687500, 0.0000000000000000, -0.4405822753906250,
+0.0000000000000000, -0.4501953125000000, -0.0000305175781250, -0.4033203125000000,
+0.0000000000000000, -0.4237670898437500, 0.0000000000000000, -0.3912353515625000,
+0.0000000000000000, -0.3445739746093750, -0.0000305175781250, -0.3835144042968750,
+0.0000305175781250, -0.2720031738281250, 0.0000000000000000, -0.2980957031250000,
+0.0000000000000000, -0.2668762207031250, 0.0000000000000000, -0.1777343750000000,
+0.0000000000000000, -0.2302551269531250, 0.0000000000000000, -0.1427917480468750,
+-0.0000610351562500, -0.1268615722656250, 0.0000000000000000, -0.1479797363281250,
+0.0000305175781250, -0.1158752441406250, -0.0000305175781250, -0.1124572753906250,
+0.0000305175781250, -0.1892089843750000, 0.0000610351562500, -0.1513366699218750,
+0.0000305175781250, -0.2055664062500000, -0.0000305175781250, -0.3009948730468750,
+0.0000305175781250, -0.2425537109375000, -0.0000305175781250, -0.3538513183593750,
+-0.0000305175781250, -0.3682556152343750, 0.0000000000000000, -0.3323059082031250,
+0.0000305175781250, -0.3754272460937500, 0.0000305175781250, -0.3802185058593750,
+0.0000305175781250, -0.2324218750000000, 0.0000305175781250, -0.3004150390625000,
+-0.0000305175781250, -0.1752319335937500, -0.0001220703125000, 0.0083312988281250,
+-0.0000305175781250, -0.0997314453125000, -0.0000610351562500, 0.2403564453125000,
+-0.0001220703125000, 0.2265319824218750, -0.0000610351562500, 0.2843322753906250,
+0.0000305175781250, 0.5127868652343750, -0.0000305175781250, 0.4230957031250000,
+0.0000305175781250, 0.5095825195312500, 0.0000915527343750, 0.5771179199218750,
+0.0000000000000000, 0.4806518554687500, -0.0000915527343750, 0.4392089843750000,
+-0.0000610351562500, 0.4960632324218750, -0.0000610351562500, 0.2311401367187500,
+-0.0001525878906250, 0.2761840820312500, -0.0000610351562500, 0.1903686523437500,
+0.0000915527343750, -0.0231018066406250, 0.0000305175781250, 0.0830688476562500,
+0.0000610351562500, -0.1060791015625000, 0.0002136230468750, -0.1308288574218750,
+0.0000000000000000, -0.1053161621093750, -0.0000915527343750, -0.1790771484375000,
+0.0000305175781250, -0.1687316894531250, -0.0001831054687500, -0.1098632812500000,
+-0.0003051757812500, -0.1367492675781250, -0.0001220703125000, -0.1009216308593750,
+-0.0000305175781250, -0.0315551757812500, -0.0001525878906250, -0.0671997070312500,
+0.0001831054687500, -0.0086669921875000, 0.0002746582031250, 0.0016784667968750,
+0.0000610351562500, -0.0078735351562500, 0.0000915527343750, 0.0098876953125000,
+0.0002136230468750, 0.0086059570312500, -0.0002136230468750, -0.0005187988281250,
+-0.0002441406250000, 0.0003967285156250, -0.0000915527343750, 0.0010070800781250,
+-0.0003356933593750, -0.0036621093750000, -0.0003356933593750, -0.0032653808593750,
+0.0001220703125000, 0.0001525878906250, 0.0000305175781250, -0.0003662109375000,
+0.0000610351562500},
+{0.0000000000000000, 0.0528259277343750, 0.0000305175781250, 0.0470886230468750,
+0.0000305175781250, 0.0380249023437500, 0.0000305175781250, 0.0481262207031250,
+0.0000305175781250, 0.0220031738281250, 0.0000610351562500, 0.0362548828125000,
+0.0000305175781250, 0.0167846679687500, 0.0000305175781250, -0.0118103027343750,
+0.0000000000000000, 0.0115966796875000, 0.0000000000000000, -0.0602416992187500,
+0.0000305175781250, -0.0401306152343750, 0.0000305175781250, -0.0737915039062500,
+0.0000000000000000, -0.1332092285156250, 0.0000000000000000, -0.0887145996093750,
+0.0000305175781250, -0.2080078125000000, 0.0000305175781250, -0.1773071289062500,
+0.0000305175781250, -0.2211914062500000, 0.0000305175781250, -0.3193359375000000,
+0.0000610351562500, -0.2548217773437500, 0.0000305175781250, -0.3966369628906250,
+0.0000000000000000, -0.3810729980468750, 0.0000000000000000, -0.3933410644531250,
+0.0000305175781250, -0.4866943359375000, -0.0000305175781250, -0.4445495605468750,
+0.0000000000000000, -0.4738159179687500, 0.0000610351562500, -0.5079956054687500,
+0.0000305175781250, -0.4566040039062500, -0.0000305175781250, -0.4304809570312500,
+0.0000305175781250, -0.4668273925781250, -0.0000610351562500, -0.3184204101562500,
+-0.0000610351562500, -0.3529968261718750, -0.0000305175781250, -0.3026123046875000,
+0.0000000000000000, -0.1724853515625000, -0.0000305175781250, -0.2399291992187500,
+0.0000000000000000, -0.1406555175781250, 0.0000610351562500, -0.1161193847656250,
+0.0000000000000000, -0.1522827148437500, -0.0000305175781250, -0.1300048828125000,
+0.0000305175781250, -0.1176452636718750, -0.0000610351562500, -0.2128295898437500,
+-0.0001220703125000, -0.1897888183593750, 0.0000000000000000, -0.2167968750000000,
+0.0000610351562500, -0.3056335449218750, 0.0000305175781250, -0.2711486816406250,
+0.0000305175781250, -0.2866516113281250, 0.0000915527343750, -0.3374023437500000,
+0.0000000000000000, -0.2471618652343750, -0.0001220703125000, -0.1833190917968750,
+0.0000000000000000, -0.2445068359375000, -0.0000610351562500, 0.0302124023437500,
+-0.0001525878906250, -0.0245971679687500, 0.0000000000000000, 0.0776367187500000,
+0.0000610351562500, 0.3213500976562500, -0.0000915527343750, 0.2100524902343750,
+0.0000915527343750, 0.4246520996093750, 0.0001831054687500, 0.4639282226562500,
+0.0000000000000000, 0.4107055664062500, -0.0000915527343750, 0.4804687500000000,
+0.0000305175781250, 0.4879455566406250, -0.0000915527343750, 0.3202209472656250,
+-0.0002136230468750, 0.3880004882812500, -0.0000915527343750, 0.2796630859375000,
+0.0000000000000000, 0.0898132324218750, -0.0001220703125000, 0.1987915039062500,
+0.0001525878906250, -0.0452575683593750, 0.0002136230468750, -0.0586547851562500,
+0.0000915527343750, -0.0528259277343750, 0.0000610351562500, -0.1733093261718750,
+0.0001831054687500, -0.1405029296875000, -0.0001525878906250, -0.1148986816406250,
+-0.0002136230468750, -0.1494750976562500, -0.0000915527343750, -0.1076965332031250,
+-0.0002441406250000, -0.0477600097656250, -0.0003051757812500, -0.0832824707031250,
+0.0000915527343750, -0.0145263671875000, 0.0000000000000000, -0.0041809082031250,
+0.0000305175781250, -0.0126342773437500, 0.0003356933593750, 0.0104064941406250,
+0.0002746582031250, 0.0073852539062500, 0.0000000000000000, -0.0004577636718750,
+0.0001220703125000, 0.0014648437500000, 0.0000000000000000, 0.0015563964843750,
+-0.0003967285156250, -0.0032043457031250, -0.0002136230468750, -0.0026855468750000,
+-0.0002746582031250, -0.0000610351562500, -0.0004577636718750, -0.0006103515625000,
+-0.0002441406250000, -0.0007934570312500, 0.0001525878906250, 0.0013427734375000,
+0.0005798339843750, -0.0021362304687500, -0.0007934570312500, 0.0002136230468750,
+-0.0000305175781250, -0.0009460449218750, -0.0005187988281250, -0.0007019042968750,
+-0.0018005371093750, 0.0008239746093750, -0.0013732910156250, 0.0007324218750000,
+0.0001220703125000, -0.0001831054687500, -0.0012207031250000, -0.0001525878906250,
+0.0004882812500000, 0.0001831054687500, 0.0041503906250000, -0.0000610351562500,
+0.0018005371093750, -0.0002441406250000, -0.0009460449218750, 0.0003051757812500,
+0.0071411132812500, 0.0005493164062500, -0.0115356445312500, 0.0001525878906250,
+-0.0235900878906250, -0.0001831054687500, -0.0089721679687500, 0.0001831054687500,
+-0.0773315429687500, -0.0003051757812500, -0.0614013671875000, -0.0005798339843750,
+-0.0820922851562500, -0.0001525878906250, -0.1470031738281250, 0.0001525878906250,
+-0.1143493652343750, -0.0002136230468750, -0.1154174804687500, 0.0002441406250000,
+-0.1648559570312500, 0.0005187988281250, -0.0726318359375000, 0.0000915527343750,
+-0.0046997070312500, -0.0002441406250000, -0.0813293457031250, 0.0001220703125000,
+0.1817626953125000, -0.0003356933593750, 0.1334533691406250, -0.0006103515625000,
+0.1945800781250000, -0.0001831054687500, 0.4058227539062500, 0.0000610351562500,
+0.3057556152343750, -0.0002136230468750, 0.3971862792968750, 0.0002136230468750,
+0.4656982421875000, 0.0004272460937500, 0.3595275878906250, 0.0001525878906250,
+0.3187866210937500, 0.0000000000000000, 0.3898620605468750, 0.0001525878906250,
+0.1112365722656250, -0.0000915527343750, 0.1606140136718750, -0.0002441406250000,
+0.1003723144531250, -0.0000305175781250, -0.1098937988281250, 0.0000915527343750,
+-0.0086669921875000, 0.0000000000000000, -0.1309509277343750, 0.0001525878906250,
+-0.1651611328125000, 0.0003051757812500, -0.1265869140625000, 0.0000915527343750,
+-0.1473999023437500, -0.0001220703125000, -0.1604003906250000, 0.0000610351562500,
+-0.1155395507812500, -0.0001220703125000, -0.1138000488281250, -0.0002136230468750,
+-0.1340637207031250, -0.0000915527343750, -0.1160583496093750, 0.0000610351562500,
+-0.1098937988281250, -0.0000915527343750, -0.1979064941406250, 0.0000610351562500,
+-0.1583251953125000, 0.0001831054687500, -0.2145385742187500, 0.0000915527343750,
+-0.3107299804687500, -0.0000305175781250, -0.2479553222656250, 0.0000000000000000,
+-0.3994750976562500, 0.0000305175781250, -0.3879394531250000, -0.0000610351562500,
+-0.4044494628906250, 0.0000000000000000, -0.5013732910156250, 0.0001525878906250,
+-0.4574890136718750, 0.0000610351562500, -0.4937133789062500, -0.0000305175781250,
+-0.5205993652343750, 0.0000610351562500, -0.4716491699218750, 0.0000000000000000,
+-0.4578247070312500, -0.0001220703125000, -0.4895935058593750, -0.0000610351562500,
+-0.3429260253906250, -0.0000915527343750, -0.3951416015625000, -0.0000915527343750,
+-0.3190002441406250, -0.0000915527343750, -0.1810302734375000, -0.0000305175781250,
+-0.2805786132812500, -0.0000610351562500, -0.0618591308593750, -0.0001220703125000,
+-0.0928955078125000, -0.0001220703125000, -0.0481262207031250, -0.0001220703125000,
+0.1025390625000000, -0.0001525878906250, 0.0127563476562500, -0.0001220703125000,
+0.1568908691406250, -0.0000610351562500, 0.1549377441406250, -0.0000915527343750,
+0.1506347656250000, -0.0000610351562500, 0.2230834960937500, -0.0000305175781250,
+0.1913146972656250, 0.0000000000000000, 0.2135620117187500, 0.0000000000000000,
+0.2278442382812500, -0.0000610351562500, 0.2105712890625000, 0.0000000000000000,
+0.2072753906250000, 0.0000305175781250, 0.2190246582031250, -0.0000305175781250,
+0.1828918457031250, 0.0001220703125000, 0.1951599121093750, 0.0000915527343750,
+0.1875000000000000, 0.0000610351562500, 0.1611938476562500, 0.0000915527343750,
+0.1855773925781250, 0.0000610351562500, 0.1651306152343750, 0.0000610351562500,
+0.1701049804687500, 0.0000305175781250, 0.1755676269531250, 0.0000610351562500,
+0.1682434082031250, 0.0000915527343750, 0.1816406250000000, 0.0000610351562500,
+0.1833190917968750, 0.0000610351562500, 0.1856079101562500, 0.0000915527343750,
+0.1967773437500000, 0.0001220703125000, 0.2041625976562500, 0.0000610351562500,
+0.2052612304687500, 0.0000610351562500, 0.2240600585937500, 0.0000305175781250,
+0.2306213378906250, 0.0000000000000000, 0.2355346679687500, 0.0000305175781250,
+0.2451477050781250, 0.0000305175781250, 0.2449340820312500, 0.0000000000000000,
+0.2442321777343750, 0.0000305175781250, 0.2476806640625000, 0.0000000000000000,
+0.2374877929687500, 0.0000000000000000, 0.2293395996093750, -0.0000305175781250,
+0.2312927246093750, 0.0000305175781250, 0.1931152343750000, -0.0000915527343750,
+0.2011108398437500, -0.0000610351562500, 0.1736450195312500, -0.0000610351562500,
+0.1340942382812500, -0.0000610351562500, 0.1482238769531250, -0.0000610351562500,
+0.0606384277343750, -0.0000915527343750, 0.0712890625000000, -0.0000915527343750,
+0.0292053222656250, -0.0000915527343750, -0.0485229492187500, -0.0000915527343750,
+-0.0185852050781250, -0.0000610351562500, -0.1276550292968750, -0.0000915527343750,
+-0.1251831054687500, -0.0000915527343750, -0.1550598144531250, -0.0000610351562500,
+-0.2356262207031250, -0.0000915527343750, -0.2099609375000000, -0.0001220703125000,
+-0.2709655761718750, -0.0000610351562500, -0.2831726074218750, -0.0000610351562500,
+-0.2724914550781250, -0.0000305175781250, -0.2929992675781250, 0.0000610351562500,
+-0.2933654785156250, -0.0000305175781250, -0.2572937011718750, 0.0000000000000000,
+-0.2698364257812500, 0.0000305175781250, -0.2458496093750000, 0.0000000000000000,
+-0.2037658691406250, -0.0000305175781250, -0.2255859375000000, -0.0000305175781250,
+-0.1834716796875000, 0.0000000000000000, -0.1760559082031250, -0.0000305175781250,
+-0.1854553222656250, 0.0000000000000000, -0.1739501953125000, 0.0000305175781250,
+-0.1731262207031250, 0.0000305175781250, -0.1912231445312500, 0.0000000000000000,
+-0.1958618164062500, 0.0000000000000000, -0.1826782226562500, 0.0000000000000000,
+-0.1916809082031250, -0.0000915527343750, -0.1978759765625000, -0.0000610351562500,
+-0.1051025390625000, -0.0000305175781250, -0.1430053710937500, -0.0000305175781250,
+-0.0636596679687500, 0.0000000000000000, 0.0529479980468750, 0.0001220703125000,
+-0.0168762207031250, 0.0000915527343750, 0.1709594726562500, 0.0000000000000000,
+0.1765747070312500, 0.0000915527343750, 0.1740112304687500, 0.0000000000000000,
+0.2771606445312500, -0.0001525878906250, 0.2509155273437500, -0.0000610351562500,
+0.1946105957031250, -0.0000610351562500, 0.2444152832031250, -0.0001831054687500,
+0.1677551269531250, -0.0000915527343750, 0.0620117187500000, 0.0001220703125000,
+0.1288757324218750, -0.0000305175781250, -0.0208740234375000, 0.0000000000000000,
+-0.0341186523437500, 0.0000305175781250, -0.0244445800781250, -0.0000610351562500,
+-0.0908203125000000, -0.0002136230468750, -0.0773315429687500, -0.0001525878906250,
+-0.0500488281250000, -0.0000610351562500, -0.0672302246093750, -0.0001220703125000,
+-0.0480346679687500, -0.0000610351562500, -0.0106201171875000, 0.0000915527343750,
+-0.0283813476562500, 0.0000000000000000, -0.0036926269531250, 0.0000000000000000,
+0.0034179687500000, 0.0000305175781250, -0.0025024414062500, 0.0000000000000000,
+0.0012512207031250, -0.0000915527343750, 0.0021972656250000, -0.0000610351562500,
+-0.0010375976562500, 0.0000000000000000, -0.0018615722656250, -0.0000305175781250,
+-0.0006408691406250, -0.0000610351562500, -0.0001220703125000, -0.0000915527343750,
+-0.0000915527343750, -0.0001220703125000, 0.0001831054687500, -0.0000610351562500,
+0.0005798339843750, -0.0001525878906250, 0.0002136230468750, -0.0000610351562500,
+-0.0002441406250000, 0.0002136230468750, 0.0001525878906250, 0.0000000000000000,
+-0.0000305175781250, -0.0000915527343750, -0.0003967285156250, 0.0002441406250000,
+-0.0001220703125000, -0.0002746582031250, 0.0002136230468750, -0.0011291503906250,
+0.0000305175781250},
+{-0.0187377929687500, 0.0000305175781250, -0.0278625488281250, 0.0000305175781250,
+0.0157470703125000, 0.0000610351562500, 0.0114746093750000, 0.0000305175781250,
+0.0299072265625000, 0.0000915527343750, 0.0667724609375000, 0.0000915527343750,
+0.0557250976562500, 0.0000610351562500, 0.1102905273437500, 0.0000305175781250,
+0.1088867187500000, 0.0000610351562500, 0.1247558593750000, 0.0000610351562500,
+0.1678161621093750, 0.0000000000000000, 0.1584472656250000, 0.0000610351562500,
+0.1877441406250000, 0.0000305175781250, 0.1969909667968750, 0.0000000000000000,
+0.1871643066406250, -0.0000305175781250, 0.1961975097656250, 0.0000305175781250,
+0.2004699707031250, 0.0000000000000000, 0.1478271484375000, -0.0000610351562500,
+0.1674194335937500, 0.0000000000000000, 0.1211242675781250, -0.0000305175781250,
+0.0554199218750000, -0.0001525878906250, 0.0885925292968750, -0.0001220703125000,
+-0.0364685058593750, -0.0000915527343750, -0.0316162109375000, -0.0001220703125000,
+-0.0658874511718750, -0.0001220703125000, -0.1574401855468750, 0.0000000000000000,
+-0.1218566894531250, -0.0000610351562500, -0.1852111816406250, -0.0000305175781250,
+-0.2024230957031250, 0.0000000000000000, -0.1836242675781250, -0.0000305175781250,
+-0.1949462890625000, -0.0000610351562500, -0.2005004882812500, -0.0000610351562500,
+-0.1634521484375000, 0.0000000000000000, -0.1676635742187500, -0.0000610351562500,
+-0.1638488769531250, -0.0000610351562500, -0.1377563476562500, 0.0000305175781250,
+-0.1460876464843750, 0.0000305175781250, -0.1544189453125000, 0.0000000000000000,
+-0.1492004394531250, 0.0000305175781250, -0.1526489257812500, 0.0000000000000000,
+-0.1718139648437500, -0.0000915527343750, -0.1679687500000000, -0.0000610351562500,
+-0.1212158203125000, -0.0000610351562500, -0.1570129394531250, -0.0001220703125000,
+-0.0887145996093750, -0.0000305175781250, -0.0065002441406250, 0.0001220703125000,
+-0.0629882812500000, 0.0000000000000000, 0.1187133789062500, 0.0000610351562500,
+0.1120300292968750, 0.0001525878906250, 0.1240539550781250, 0.0000305175781250,
+0.2398986816406250, -0.0001220703125000, 0.2024841308593750, -0.0000305175781250,
+0.1757202148437500, -0.0001220703125000, 0.2240600585937500, -0.0002136230468750,
+0.1516418457031250, -0.0001220703125000, 0.0632324218750000, 0.0000305175781250,
+0.1197509765625000, -0.0000915527343750, -0.0185852050781250, -0.0000305175781250,
+-0.0261230468750000, 0.0000915527343750, -0.0173339843750000, 0.0000000000000000,
+-0.0820617675781250, -0.0001220703125000, -0.0689392089843750, -0.0000610351562500,
+-0.0430908203125000, 0.0000000000000000, -0.0588073730468750, -0.0000305175781250,
+-0.0416259765625000, -0.0000915527343750, -0.0074462890625000, 0.0000305175781250,
+-0.0236816406250000, 0.0000000000000000, -0.0030822753906250, -0.0000915527343750,
+0.0032348632812500, -0.0001220703125000, -0.0018920898437500, 0.0000000000000000,
+0.0000000000000000, 0.0001525878906250, -0.0004577636718750, 0.0001220703125000,
+-0.0006713867187500, -0.0000915527343750, -0.0015258789062500, 0.0001831054687500,
+-0.0008239746093750, -0.0001525878906250, 0.0003662109375000, -0.0008544921875000,
+-0.0005187988281250, -0.0003967285156250, 0.0000915527343750, -0.0003967285156250,
+0.0006408691406250, -0.0010375976562500, 0.0002136230468750, -0.0004577636718750,
+-0.0003051757812500, 0.0017700195312500, -0.0000610351562500, 0.0005798339843750,
+0.0000610351562500, -0.0067749023437500, -0.0002136230468750, 0.0007019042968750,
+-0.0001220703125000, -0.0141906738281250, 0.0002746582031250, -0.0317077636718750,
+0.0001831054687500, -0.0184020996093750, -0.0001525878906250, -0.0494079589843750,
+-0.0000610351562500, -0.0585937500000000, 0.0000000000000000, -0.0352478027343750,
+-0.0002136230468750, -0.0401611328125000, -0.0002441406250000, -0.0519104003906250,
+0.0001525878906250, 0.0455627441406250, 0.0001220703125000, 0.0185546875000000,
+0.0000000000000000, 0.0565185546875000, 0.0001220703125000, 0.1526794433593750,
+0.0002136230468750, 0.1051330566406250, -0.0001525878906250, 0.1467285156250000,
+-0.0001831054687500, 0.1838684082031250, -0.0000610351562500, 0.1257934570312500,
+-0.0001525878906250, 0.0939025878906250, -0.0002136230468750, 0.1327819824218750,
+0.0000915527343750, -0.0029602050781250, 0.0001220703125000, 0.0051574707031250,
+0.0000000000000000, -0.0043640136718750, 0.0000610351562500, -0.0876770019531250,
+0.0001525878906250, -0.0593261718750000, -0.0001220703125000, -0.0652160644531250,
+-0.0001220703125000, -0.0815734863281250, -0.0000305175781250, -0.0699768066406250,
+-0.0001220703125000, -0.0498962402343750, -0.0001831054687500, -0.0581970214843750,
+0.0000305175781250, -0.0788574218750000, 0.0000305175781250, -0.0553588867187500,
+0.0000000000000000, -0.0979003906250000, 0.0000610351562500, -0.1484375000000000,
+0.0000610351562500, -0.1116943359375000, -0.0000915527343750, -0.1999511718750000,
+-0.0000610351562500, -0.2080993652343750, -0.0000305175781250, -0.1964721679687500,
+-0.0001220703125000, -0.2372436523437500, -0.0001525878906250, -0.2311706542968750,
+0.0000305175781250, -0.1656799316406250, 0.0000000000000000, -0.1914367675781250,
+0.0000000000000000, -0.1305236816406250, 0.0000610351562500, -0.0425720214843750,
+0.0000915527343750, -0.0853881835937500, -0.0000305175781250, 0.0467834472656250,
+0.0000000000000000, 0.0530700683593750, -0.0000305175781250, 0.0631408691406250,
+-0.0000610351562500, 0.1363525390625000, -0.0000610351562500, 0.1148071289062500,
+0.0000000000000000, 0.1255493164062500, -0.0000305175781250, 0.1427001953125000,
+-0.0000610351562500, 0.1213989257812500, 0.0000610351562500, 0.1026916503906250,
+0.0000305175781250, 0.1137084960937500, 0.0000000000000000, 0.0704956054687500,
+0.0000000000000000, 0.0690307617187500, 0.0000000000000000, 0.0680236816406250,
+-0.0000305175781250, 0.0440063476562500, -0.0000305175781250, 0.0495605468750000,
+0.0000000000000000, 0.0594482421875000, 0.0000000000000000, 0.0502624511718750,
+-0.0000610351562500, 0.0711975097656250, 0.0000000000000000, 0.0946350097656250,
+0.0000305175781250, 0.0823364257812500, 0.0000000000000000, 0.1326904296875000,
+0.0000305175781250, 0.1344299316406250, -0.0000305175781250, 0.1549987792968750,
+0.0000305175781250, 0.1965637207031250, 0.0000305175781250, 0.1879577636718750,
+0.0000610351562500, 0.2285461425781250, 0.0000305175781250, 0.2389831542968750,
+0.0000000000000000, 0.2552795410156250, 0.0000610351562500, 0.2826232910156250,
+0.0000000000000000, 0.2796630859375000, 0.0000305175781250, 0.3113098144531250,
+0.0000305175781250, 0.3281250000000000, 0.0000610351562500, 0.3324279785156250,
+0.0000305175781250, 0.3466186523437500, 0.0000610351562500, 0.3581542968750000,
+0.0000305175781250, 0.3496398925781250, 0.0000305175781250, 0.3609619140625000,
+0.0000000000000000, 0.3584594726562500, 0.0000305175781250, 0.3488159179687500,
+0.0000305175781250, 0.3590087890625000, 0.0000305175781250, 0.3311462402343750,
+0.0000305175781250, 0.3513793945312500, 0.0000915527343750, 0.3295898437500000,
+0.0000000000000000, 0.2968750000000000, 0.0000000000000000, 0.3278503417968750,
+0.0000305175781250, 0.2469482421875000, 0.0000610351562500, 0.2674865722656250,
+0.0000305175781250, 0.2398071289062500, 0.0000000000000000, 0.1769409179687500,
+0.0000000000000000, 0.2233276367187500, 0.0000305175781250, 0.1314697265625000,
+0.0000000000000000, 0.1445922851562500, 0.0000000000000000, 0.1315002441406250,
+0.0000000000000000, 0.0692138671875000, 0.0000305175781250, 0.1054382324218750,
+-0.0000305175781250, 0.0658874511718750, -0.0000305175781250, 0.0545349121093750,
+0.0000000000000000, 0.0804748535156250, -0.0000305175781250, 0.0772705078125000,
+0.0000000000000000, 0.0650634765625000, 0.0000305175781250, 0.1280212402343750,
+-0.0000305175781250, 0.1166992187500000, 0.0000305175781250, 0.1311950683593750,
+0.0000305175781250, 0.1864013671875000, 0.0000305175781250, 0.1654968261718750,
+0.0000000000000000, 0.1624145507812500, 0.0000000000000000, 0.1978454589843750,
+0.0000305175781250, 0.1249389648437500, 0.0000000000000000, 0.0736083984375000,
+-0.0000305175781250, 0.1309204101562500, 0.0000915527343750, -0.0929870605468750,
+0.0000915527343750, -0.0434570312500000, 0.0000610351562500, -0.1288146972656250,
+0.0000305175781250, -0.3221130371093750, 0.0000915527343750, -0.2140808105468750,
+-0.0000610351562500, -0.4138183593750000, -0.0000305175781250, -0.4318847656250000,
+0.0000000000000000, -0.4111633300781250, 0.0000000000000000, -0.5028076171875000,
+-0.0000610351562500, -0.4894409179687500, 0.0000000000000000, -0.4224548339843750,
+0.0000610351562500, -0.4605407714843750, -0.0000305175781250, -0.3949890136718750,
+-0.0000305175781250, -0.2984619140625000, 0.0000305175781250, -0.3547668457031250,
+-0.0000915527343750, -0.2211303710937500, -0.0001831054687500, -0.2138671875000000,
+-0.0000915527343750, -0.2175598144531250, -0.0001220703125000, -0.1530761718750000,
+-0.0001525878906250, -0.1700134277343750, 0.0000610351562500, -0.1825866699218750,
+0.0000305175781250, -0.1734924316406250, 0.0000000000000000, -0.1737670898437500,
+0.0000305175781250, -0.1962585449218750, 0.0001220703125000, -0.1933898925781250,
+-0.0001831054687500, -0.1287536621093750, -0.0001525878906250, -0.1780395507812500,
+-0.0001220703125000, -0.0871582031250000, -0.0002136230468750, 0.0207519531250000,
+-0.0002441406250000, -0.0588378906250000, 0.0000000000000000, 0.1953735351562500,
+-0.0000305175781250, 0.1750793457031250, 0.0000000000000000, 0.2132873535156250,
+0.0001831054687500, 0.3937072753906250, 0.0001525878906250, 0.3231506347656250,
+-0.0000305175781250, 0.3395385742187500, 0.0000610351562500, 0.4090881347656250,
+-0.0000915527343750, 0.3016052246093750, -0.0002746582031250, 0.2118530273437500,
+-0.0001525878906250, 0.2918701171875000, -0.0001831054687500, 0.0376892089843750,
+-0.0003356933593750, 0.0480346679687500, -0.0000915527343750, 0.0221557617187500,
+0.0000610351562500, -0.1379089355468750, -0.0000610351562500, -0.0812072753906250,
+0.0003662109375000, -0.1026611328125000, 0.0004577636718750, -0.1426391601562500,
+0.0002441406250000, -0.0964355468750000, 0.0003967285156250, -0.0543212890625000,
+0.0004882812500000, -0.0871582031250000, -0.0006713867187500, -0.0187988281250000,
+-0.0004882812500000, -0.0120239257812500, -0.0004577636718750, -0.0210266113281250,
+-0.0016174316406250, 0.0140380859375000, 0.0024108886718750, -0.0847473144531250,
+-0.0150756835937500, -0.0074157714843750, 0.0045471191406250, -0.0197143554687500,
+-0.0243225097656250, -0.0163879394531250, -0.0504760742187500, 0.0079345703125000,
+-0.0275268554687500, 0.0038757324218750, -0.0924072265625000, -0.0003662109375000,
+-0.0920410156250000, 0.0015563964843750, -0.0841064453125000, 0.0012512207031250,
+-0.1257019042968750, -0.0019226074218750, -0.1171569824218750, -0.0015563964843750,
+-0.0330200195312500, 0.0000000000000000, -0.0915832519531250, -0.0001831054687500,
+0.0117492675781250, -0.0004272460937500, 0.1453552246093750, 0.0003051757812500,
+0.0478515625000000, 0.0003967285156250, 0.2974548339843750, -0.0001525878906250,
+0.2997741699218750, -0.0003051757812500, 0.2778320312500000, 0.0000305175781250,
+0.4158325195312500, 0.0000915527343750, 0.3887939453125000, -0.0001831054687500,
+0.2364807128906250, 0.0001831054687500, 0.3381042480468750, 0.0003662109375000,
+0.1950073242187500, 0.0000610351562500, -0.0309448242187500, -0.0001525878906250,
+0.1176147460937500, 0.0000610351562500, -0.1513061523437500, -0.0000915527343750,
+-0.1828918457031250},
+{-0.0000610351562500, 0.1758422851562500, -0.0000305175781250, 0.1729125976562500,
+-0.0000305175781250, 0.1650085449218750, -0.0001220703125000, 0.2125854492187500,
+-0.0000610351562500, 0.2018432617187500, 0.0000000000000000, 0.1256408691406250,
+0.0000305175781250, 0.1765136718750000, -0.0000305175781250, 0.0843200683593750,
+0.0000000000000000, -0.0334777832031250, 0.0000305175781250, 0.0430603027343750,
+-0.0001220703125000, -0.2059326171875000, -0.0001220703125000, -0.1896057128906250,
+-0.0000610351562500, -0.2392578125000000, -0.0001220703125000, -0.4133300781250000,
+-0.0001220703125000, -0.3414611816406250, 0.0000000000000000, -0.4272460937500000,
+0.0000305175781250, -0.4696350097656250, -0.0000305175781250, -0.4138793945312500,
+0.0000915527343750, -0.4044189453125000, 0.0000610351562500, -0.4314575195312500,
+-0.0001525878906250, -0.2937316894531250, -0.0001220703125000, -0.3164062500000000,
+-0.0000610351562500, -0.2788085937500000, -0.0001831054687500, -0.1691894531250000,
+-0.0001525878906250, -0.2179260253906250, 0.0000305175781250, -0.1504211425781250,
+-0.0000305175781250, -0.1341247558593750, -0.0000305175781250, -0.1491699218750000,
+0.0001525878906250, -0.1388244628906250, 0.0001525878906250, -0.1364135742187500,
+-0.0000305175781250, -0.1232910156250000, 0.0000610351562500, -0.1515502929687500,
+-0.0000305175781250, -0.0877075195312500, -0.0002136230468750, -0.0379333496093750,
+-0.0001525878906250, -0.0882568359375000, -0.0001525878906250, 0.1231079101562500,
+-0.0002746582031250, 0.0842590332031250, -0.0000915527343750, 0.1418151855468750,
+0.0000610351562500, 0.3206481933593750, -0.0000915527343750, 0.2403869628906250,
+0.0003356933593750, 0.3129882812500000, 0.0003967285156250, 0.3784790039062500,
+0.0001831054687500, 0.2777709960937500, 0.0003051757812500, 0.2221069335937500,
+0.0003967285156250, 0.2878417968750000, -0.0006408691406250, 0.0474548339843750,
+-0.0004882812500000, 0.0697631835937500, -0.0003662109375000, 0.0397338867187500,
+-0.0012207031250000, -0.1218261718750000, -0.0010681152343750, -0.0603942871093750,
+0.0010986328125000, -0.0947265625000000, 0.0003356933593750, -0.1301574707031250,
+0.0004272460937500, -0.0946960449218750, 0.0031127929687500, -0.0442504882812500,
+-0.0288696289062500, -0.0435791015625000, -0.0953063964843750, -0.0894165039062500,
+-0.0848999023437500, -0.1177062988281250, -0.0709533691406250, -0.0782775878906250,
+-0.0991210937500000, -0.0449218750000000, -0.1044006347656250, -0.0707702636718750,
+0.0138549804687500, -0.0144653320312500, -0.0425720214843750, -0.0082702636718750,
+0.0484313964843750, -0.0140075683593750, 0.1950378417968750, 0.0065917968750000,
+0.1004333496093750, 0.0032348632812500, 0.3073120117187500, -0.0006408691406250,
+0.3326416015625000, 0.0008544921875000, 0.2768859863281250, 0.0008544921875000,
+0.3580627441406250, -0.0015563964843750, 0.3654785156250000, -0.0014038085937500,
+0.1401367187500000, 0.0002441406250000, 0.2308349609375000, 0.0001220703125000,
+0.1054382324218750, -0.0001831054687500, -0.1473693847656250, 0.0003967285156250,
+0.0004272460937500, 0.0004882812500000, -0.2015686035156250, -0.0000915527343750,
+-0.2761840820312500, -0.0001831054687500, -0.1613464355468750, 0.0000305175781250,
+-0.1620788574218750, 0.0000305175781250, -0.2288818359375000, -0.0000305175781250,
+0.0150756835937500, 0.0000000000000000, -0.0097045898437500, 0.0001220703125000,
+-0.0114440917968750, 0.0000000000000000, 0.1535949707031250, -0.0003051757812500,
+0.1080627441406250, -0.0001831054687500, -0.0261230468750000, 0.0000000000000000,
+0.0833435058593750, -0.0002136230468750, -0.0762023925781250, -0.0000305175781250,
+-0.3026123046875000, 0.0004272460937500, -0.1456298828125000, 0.0001525878906250,
+-0.4718627929687500, 0.0000000000000000, -0.5018310546875000, 0.0002441406250000,
+-0.4371032714843750, 0.0000610351562500, -0.5710754394531250, -0.0003356933593750,
+-0.5664672851562500, -0.0001831054687500, -0.3495178222656250, -0.0001525878906250,
+-0.4533691406250000, -0.0003051757812500, -0.3105773925781250, -0.0001831054687500,
+-0.0610656738281250, 0.0000610351562500, -0.2210693359375000, 0.0000000000000000,
+0.1013793945312500, -0.0000915527343750, 0.1062927246093750, -0.0000305175781250,
+0.0965270996093750, -0.0001220703125000, 0.2626647949218750, -0.0002441406250000,
+0.2107238769531250, -0.0002441406250000, 0.1876831054687500, 0.0000305175781250,
+0.2439575195312500, -0.0000610351562500, 0.1757202148437500, -0.0000305175781250,
+0.0971374511718750, 0.0002136230468750, 0.1595764160156250, 0.0001525878906250,
+0.0044250488281250, -0.0000305175781250, 0.0111389160156250, 0.0000305175781250,
+0.0062561035156250, 0.0000305175781250, -0.0892028808593750, -0.0000915527343750,
+-0.0552062988281250, -0.0000610351562500, -0.0307922363281250, 0.0000305175781250,
+-0.0833740234375000, 0.0000000000000000, 0.0093383789062500, -0.0000610351562500,
+0.0870666503906250, -0.0000610351562500, 0.0036010742187500, 0.0000305175781250,
+0.2753601074218750, -0.0001525878906250, 0.2035217285156250, -0.0001831054687500,
+0.3020019531250000, -0.0001525878906250, 0.5280456542968750, -0.0001525878906250,
+0.3830566406250000, -0.0001831054687500, 0.6717834472656250, -0.0000610351562500,
+0.6470336914062500, -0.0000305175781250, 0.6855773925781250, -0.0000915527343750,
+0.8690795898437500, -0.0000915527343750, 0.7750854492187500, -0.0000610351562500,
+0.9118041992187500, -0.0000610351562500, 0.9224243164062500, -0.0000915527343750,
+0.9091796875000000, -0.0000305175781250, 0.9695434570312500, 0.0000305175781250,
+0.9526977539062500, 0.0000000000000000, 0.9431152343750000, 0.0000610351562500,
+0.9671325683593750, 0.0000610351562500, 0.9442138671875000, 0.0000610351562500,
+0.9173889160156250, 0.0000305175781250, 0.9448242187500000, 0.0000000000000000,
+0.8828735351562500, 0.0000915527343750, 0.8976440429687500, 0.0000610351562500,
+0.8855285644531250, 0.0000915527343750, 0.8445434570312500, 0.0000915527343750,
+0.8799743652343750, 0.0000915527343750, 0.8346252441406250, 0.0000610351562500,
+0.8443298339843750, 0.0000610351562500, 0.8459777832031250, 0.0000915527343750,
+0.8236389160156250, 0.0001220703125000, 0.8484497070312500, 0.0000915527343750,
+0.8296813964843750, 0.0000915527343750, 0.8383483886718750, 0.0000915527343750,
+0.8470458984375000, 0.0000915527343750, 0.8410644531250000, 0.0001220703125000,
+0.8560180664062500, 0.0001220703125000, 0.8645324707031250, 0.0000915527343750,
+0.8780517578125000, 0.0000915527343750, 0.8859863281250000, 0.0000915527343750,
+0.8934326171875000, 0.0000915527343750, 0.9101562500000000, 0.0000305175781250,
+0.9073791503906250, 0.0000915527343750, 0.9143066406250000, 0.0000610351562500,
+0.9155883789062500, 0.0000305175781250, 0.9130859375000000, 0.0000305175781250,
+0.9202270507812500, 0.0000610351562500, 0.9031982421875000, 0.0000000000000000,
+0.9080810546875000, -0.0000305175781250, 0.8924560546875000, -0.0000305175781250,
+0.8757019042968750, 0.0000000000000000, 0.8854980468750000, -0.0000305175781250,
+0.8171081542968750, -0.0000610351562500, 0.8363647460937500, -0.0000305175781250,
+0.7861328125000000, -0.0000610351562500, 0.7148132324218750, -0.0001220703125000,
+0.7514648437500000, -0.0001220703125000, 0.5904846191406250, -0.0001220703125000,
+0.6167907714843750, -0.0001525878906250, 0.5299072265625000, -0.0001525878906250,
+0.3802185058593750, -0.0001220703125000, 0.4478759765625000, -0.0001525878906250,
+0.2103576660156250, -0.0001220703125000, 0.2240905761718750, -0.0000610351562500,
+0.1663208007812500, -0.0000915527343750, -0.0021362304687500, -0.0000915527343750,
+0.0660705566406250, -0.0000915527343750, -0.0659484863281250, -0.0000610351562500,
+-0.0900573730468750, -0.0000610351562500, -0.0708312988281250, -0.0000305175781250,
+-0.1179809570312500, 0.0000610351562500, -0.1122436523437500, 0.0000000000000000,
+-0.0492248535156250, 0.0000610351562500, -0.0683593750000000, 0.0001220703125000,
+-0.0349426269531250, 0.0000610351562500, 0.0335083007812500, 0.0000000000000000,
+0.0036621093750000, 0.0000610351562500, 0.0481567382812500, 0.0000000000000000,
+0.0713195800781250, -0.0000915527343750, 0.0396423339843750, -0.0000305175781250,
+0.0327148437500000, 0.0000000000000000, 0.0506591796875000, -0.0000610351562500,
+-0.0364685058593750, 0.0000305175781250, -0.0280761718750000, 0.0000915527343750,
+-0.0444641113281250, 0.0000305175781250, -0.1081542968750000, -0.0000305175781250,
+-0.0844116210937500, 0.0000000000000000, -0.0963439941406250, -0.0000610351562500,
+-0.1134643554687500, -0.0001220703125000, -0.0941162109375000, 0.0000000000000000,
+-0.0748291015625000, 0.0000305175781250, -0.0880432128906250, -0.0000305175781250,
+-0.0720214843750000, 0.0000610351562500, -0.0622863769531250, 0.0001525878906250,
+-0.0756835937500000, 0.0000305175781250, -0.0863037109375000, -0.0000305175781250,
+-0.0763854980468750, 0.0000610351562500, -0.0767517089843750, -0.0000305175781250,
+-0.1005554199218750, -0.0000915527343750, -0.0520629882812500, 0.0000000000000000,
+-0.0139465332031250, 0.0000915527343750, -0.0503540039062500, 0.0000305175781250,
+0.0881958007812500, 0.0000305175781250, 0.0761413574218750, 0.0000915527343750,
+0.0884399414062500, 0.0000305175781250, 0.1856994628906250, 0.0000000000000000,
+0.1547241210937500, 0.0000000000000000, 0.1275329589843750, 0.0000610351562500,
+0.1705627441406250, 0.0001831054687500, 0.1088256835937500, -0.0001831054687500,
+0.0302429199218750, -0.0004272460937500, 0.0828857421875000, -0.0000915527343750,
+-0.0282897949218750, -0.0007324218750000, -0.0394592285156250, -0.0011291503906250,
+-0.0234985351562500, -0.0003356933593750, -0.0654602050781250, 0.0010681152343750,
+-0.0569458007812500, 0.0005493164062500, -0.0162048339843750, -0.0027770996093750,
+-0.0313415527343750, 0.0018310546875000, -0.0281677246093750, -0.0062866210937500,
+0.0023498535156250, -0.0135192871093750, -0.0090637207031250, -0.0063171386718750,
+-0.0018920898437500, -0.0368347167968750, 0.0030822753906250, -0.0369873046875000,
+0.0000000000000000, -0.0309448242187500, -0.0014953613281250, -0.0506286621093750,
+-0.0004882812500000, -0.0492248535156250, -0.0003662109375000, 0.0068664550781250,
+-0.0013122558593750, -0.0201416015625000, -0.0007629394531250, 0.0222473144531250,
+0.0005187988281250, 0.0990295410156250, 0.0001525878906250, 0.0575866699218750,
+-0.0001220703125000, 0.1185607910156250, 0.0001525878906250, 0.1456909179687500,
+0.0001220703125000, 0.1010742187500000, -0.0003356933593750, 0.0918884277343750,
+-0.0003051757812500, 0.1170349121093750, 0.0001220703125000, 0.0115966796875000,
+0.0000610351562500, 0.0187683105468750, -0.0000610351562500, 0.0149230957031250,
+0.0000915527343750, -0.0492553710937500, 0.0002136230468750, -0.0314636230468750,
+-0.0001525878906250, -0.0239257812500000, -0.0002136230468750, -0.0355834960937500,
+-0.0000305175781250, -0.0348815917968750, -0.0000610351562500, -0.0158386230468750,
+-0.0001525878906250, -0.0185241699218750, 0.0000915527343750, -0.0669555664062500,
+0.0002136230468750, -0.0441589355468750, 0.0000305175781250, -0.0760192871093750,
+0.0000305175781250, -0.1395568847656250, 0.0000915527343750, -0.1051635742187500,
+-0.0000915527343750, -0.1394653320312500, -0.0001831054687500, -0.1674804687500000,
+-0.0000305175781250, -0.1213073730468750, 0.0000000000000000, -0.0968322753906250,
+-0.0001220703125000, -0.1271667480468750, 0.0000610351562500, -0.0195922851562500,
+0.0001831054687500, -0.0253601074218750, 0.0000305175781250, -0.0111083984375000,
+0.0000305175781250},
+{-0.0924377441406250, 0.0000000000000000, -0.0733337402343750, 0.0000305175781250,
+-0.0946960449218750, 0.0000610351562500, -0.0184631347656250, -0.0000610351562500,
+-0.0101623535156250, -0.0000610351562500, -0.0209960937500000, 0.0000000000000000,
+0.0116577148437500, -0.0000610351562500, 0.0119018554687500, -0.0000610351562500,
+-0.0420227050781250, 0.0000610351562500, -0.0243530273437500, 0.0001220703125000,
+-0.0462341308593750, 0.0000000000000000, -0.1026611328125000, 0.0000000000000000,
+-0.0778503417968750, 0.0000305175781250, -0.0959777832031250, -0.0000610351562500,
+-0.1144104003906250, -0.0001220703125000, -0.0895385742187500, 0.0000000000000000,
+-0.0700073242187500, 0.0000915527343750, -0.0865783691406250, -0.0000305175781250,
+-0.0538330078125000, -0.0000305175781250, -0.0457153320312500, 0.0000610351562500,
+-0.0559387207031250, 0.0000305175781250, -0.0547485351562500, -0.0000305175781250,
+-0.0498352050781250, -0.0000610351562500, -0.0467224121093750, 0.0000305175781250,
+-0.0660705566406250, 0.0001220703125000, -0.0240478515625000, -0.0001831054687500,
+0.0094604492187500, -0.0002136230468750, -0.0242919921875000, 0.0000305175781250,
+0.0968322753906250, -0.0007324218750000, 0.0910339355468750, -0.0010070800781250,
+0.0946044921875000, -0.0001831054687500, 0.1725158691406250, 0.0001525878906250,
+0.1505432128906250, -0.0007324218750000, 0.1065979003906250, 0.0000305175781250,
+0.1447753906250000, 0.0027465820312500, 0.0974121093750000, -0.0043945312500000,
+-0.0034484863281250, -0.0098571777343750, 0.0032958984375000, -0.0091552734375000,
+-0.0500793457031250, -0.0370483398437500, -0.0523376464843750, -0.0286865234375000,
+-0.0335083007812500, -0.0271606445312500, -0.0511779785156250, -0.0520019531250000,
+-0.0585632324218750, -0.0452880859375000, -0.0188598632812500, -0.0104370117187500,
+-0.0238342285156250, -0.0354919433593750, -0.0213623046875000, 0.0075378417968750,
+0.0035705566406250, 0.0688781738281250, -0.0043945312500000, 0.0294799804687500,
+-0.0014648437500000, 0.1062622070312500, 0.0022277832031250, 0.1256103515625000,
+0.0003356933593750, 0.0918579101562500, -0.0018005371093750, 0.1035461425781250,
+-0.0011596679687500, 0.1189880371093750, -0.0002441406250000, 0.0297241210937500,
+-0.0008544921875000, 0.0398254394531250, -0.0006408691406250, 0.0297241210937500,
+0.0004577636718750, -0.0324707031250000, 0.0002746582031250, -0.0130920410156250,
+-0.0001831054687500, -0.0112304687500000, -0.0000915527343750, -0.0220947265625000,
+0.0000000000000000, -0.0216064453125000, -0.0002441406250000, -0.0061035156250000,
+-0.0003051757812500, -0.0073242187500000, 0.0000305175781250, -0.0555725097656250,
+0.0001525878906250, -0.0328979492187500, -0.0000305175781250, -0.0622253417968750,
+-0.0000305175781250, -0.1257934570312500, 0.0000610351562500, -0.0935668945312500,
+-0.0001525878906250, -0.1173400878906250, -0.0002136230468750, -0.1461181640625000,
+-0.0000915527343750, -0.1018066406250000, -0.0000305175781250, -0.0710754394531250,
+-0.0001525878906250, -0.1032104492187500, 0.0000305175781250, -0.0098266601562500,
+0.0001831054687500, -0.0072937011718750, -0.0000305175781250, -0.0144348144531250,
+-0.0000610351562500, 0.0308532714843750, 0.0000000000000000, 0.0234375000000000,
+-0.0001220703125000, -0.0046081542968750, -0.0002136230468750, 0.0063781738281250,
+-0.0000915527343750, -0.0019531250000000, 0.0000305175781250, -0.0368041992187500,
+-0.0000915527343750, -0.0249938964843750, 0.0000610351562500, 0.0121154785156250,
+0.0001220703125000, -0.0244140625000000, 0.0000000000000000, 0.0505371093750000,
+0.0000000000000000, 0.1333618164062500, 0.0000610351562500, 0.0809936523437500,
+-0.0000305175781250, 0.2818908691406250, -0.0000915527343750, 0.2692565917968750,
+-0.0000610351562500, 0.3267822265625000, 0.0000305175781250, 0.4776916503906250,
+-0.0000305175781250, 0.4168701171875000, 0.0000610351562500, 0.5252685546875000,
+0.0000915527343750, 0.5495910644531250, 0.0000305175781250, 0.5221557617187500,
+0.0000610351562500, 0.5501403808593750, 0.0000915527343750, 0.5527038574218750,
+-0.0000305175781250, 0.4828796386718750, -0.0000305175781250, 0.5009460449218750,
+0.0000305175781250, 0.4580688476562500, 0.0000000000000000, 0.3840942382812500,
+-0.0000305175781250, 0.4128112792968750, 0.0000305175781250, 0.3078308105468750,
+0.0000915527343750, 0.3061523437500000, 0.0000305175781250, 0.2855529785156250,
+0.0000305175781250, 0.2178955078125000, 0.0000305175781250, 0.2356262207031250,
+0.0000000000000000, 0.1813964843750000, 0.0000000000000000, 0.1755371093750000,
+0.0000305175781250, 0.1612854003906250, 0.0000000000000000, 0.1248779296875000,
+0.0000000000000000, 0.1304626464843750, 0.0000000000000000, 0.0942382812500000,
+0.0000610351562500, 0.0851135253906250, 0.0000305175781250, 0.0755004882812500,
+0.0000305175781250, 0.0533752441406250, 0.0000610351562500, 0.0531616210937500,
+-0.0000305175781250, 0.0402526855468750, 0.0000000000000000, 0.0287475585937500,
+0.0000000000000000, 0.0240478515625000, -0.0000305175781250, 0.0184631347656250,
+0.0000000000000000, 0.0142211914062500, 0.0000610351562500, 0.0197143554687500,
+0.0000610351562500, 0.0024108886718750, 0.0000000000000000, 0.0110473632812500,
+0.0000000000000000, 0.0244445800781250, 0.0000305175781250, 0.0058593750000000,
+0.0000000000000000, 0.0514526367187500, 0.0000000000000000, 0.0375671386718750,
+0.0000000000000000, 0.0542907714843750, -0.0000305175781250, 0.0919189453125000,
+0.0000000000000000, 0.0656127929687500, 0.0000000000000000, 0.1246337890625000,
+0.0000000000000000, 0.1111145019531250, 0.0000000000000000, 0.1311950683593750,
+0.0000305175781250, 0.1790771484375000, 0.0000000000000000, 0.1503906250000000,
+-0.0000305175781250, 0.2189636230468750, 0.0000000000000000, 0.2096557617187500,
+0.0000305175781250, 0.2281494140625000, 0.0000305175781250, 0.2757263183593750,
+0.0000000000000000, 0.2457885742187500, -0.0000305175781250, 0.3085937500000000,
+0.0000000000000000, 0.2979125976562500, -0.0000305175781250, 0.3155822753906250,
+-0.0000305175781250, 0.3591003417968750, -0.0000305175781250, 0.3312072753906250,
+0.0000000000000000, 0.4077758789062500, -0.0000610351562500, 0.3858337402343750,
+-0.0000305175781250, 0.4205322265625000, 0.0000000000000000, 0.4835815429687500,
+0.0000000000000000, 0.4373474121093750, 0.0000000000000000, 0.5700073242187500,
+0.0000000000000000, 0.5420532226562500, -0.0000305175781250, 0.5945739746093750,
+-0.0000915527343750, 0.7084960937500000, -0.0000610351562500, 0.6417846679687500,
+-0.0000305175781250, 0.7787780761718750, -0.0000610351562500, 0.7788391113281250,
+-0.0000305175781250, 0.7649230957031250, 0.0000305175781250, 0.8406982421875000,
+0.0000000000000000, 0.8235168457031250, 0.0000000000000000, 0.7478027343750000,
+0.0000305175781250, 0.8016662597656250, 0.0000000000000000, 0.7060241699218750,
+-0.0000610351562500, 0.5867614746093750, -0.0000305175781250, 0.6701049804687500,
+0.0000610351562500, 0.4246520996093750, 0.0000000000000000, 0.4506530761718750,
+0.0000305175781250, 0.3981628417968750, 0.0001220703125000, 0.2196350097656250,
+0.0000915527343750, 0.2942504882812500, -0.0000305175781250, 0.1739501953125000,
+0.0000000000000000, 0.1506958007812500, 0.0000000000000000, 0.1691284179687500,
+-0.0001220703125000, 0.1334838867187500, -0.0001220703125000, 0.1389770507812500,
+-0.0000305175781250, 0.1570129394531250, -0.0000610351562500, 0.1614074707031250,
+-0.0000610351562500, 0.1450805664062500, 0.0000610351562500, 0.1536865234375000,
+0.0000000000000000, 0.1634521484375000, -0.0000305175781250, 0.0534057617187500,
+0.0000610351562500, 0.1001892089843750, -0.0000305175781250, 0.0117492675781250,
+-0.0001525878906250, -0.1180725097656250, -0.0000915527343750, -0.0320434570312500,
+-0.0001220703125000, -0.2635803222656250, -0.0002136230468750, -0.2528686523437500,
+-0.0001220703125000, -0.2724914550781250, -0.0000305175781250, -0.4216918945312500,
+-0.0000915527343750, -0.3670959472656250, 0.0002136230468750, -0.3641662597656250,
+0.0002441406250000, -0.4231262207031250, 0.0000610351562500, -0.3353576660156250,
+0.0001220703125000, -0.2477111816406250, 0.0002441406250000, -0.3161315917968750,
+-0.0004882812500000, -0.1352233886718750, -0.0004272460937500, -0.1350097656250000,
+-0.0002746582031250, -0.1349182128906250, -0.0007324218750000, -0.0391540527343750,
+-0.0007629394531250, -0.0661926269531250, 0.0008544921875000, -0.0688476562500000,
+0.0008850097656250, -0.0611572265625000, -0.0006408691406250, -0.0499877929687500,
+0.0015869140625000, -0.0642395019531250, 0.0016174316406250, -0.0708312988281250,
+-0.0146484375000000, 0.0538330078125000, -0.0046081542968750, 0.0025024414062500,
+-0.0246582031250000, 0.0769958496093750, -0.0505371093750000, 0.2253112792968750,
+-0.0319824218750000, 0.1415710449218750, -0.0785217285156250, 0.2505493164062500,
+-0.0848083496093750, 0.2936706542968750, -0.0669860839843750, 0.2296447753906250,
+-0.0739440917968750, 0.1891174316406250, 0.1296691894531250, 0.1123352050781250,
+0.2735900878906250, 0.2359619140625000, 0.2972717285156250, 0.2599487304687500,
+0.2092590332031250, 0.1910400390625000, 0.1994628906250000, 0.1734619140625000,
+0.2606201171875000, 0.2186889648437500, -0.0165405273437500, 0.0360412597656250,
+0.0311889648437500, 0.0531921386718750, -0.0245666503906250, 0.0317687988281250,
+-0.2474670410156250, -0.0921630859375000, -0.1540222167968750, -0.0492553710937500,
+-0.1788024902343750, -0.0618896484375000, -0.2746276855468750, -0.0915832519531250,
+-0.1338195800781250, -0.0600891113281250, -0.0028686523437500, -0.0275878906250000,
+-0.1159973144531250, -0.0492858886718750, 0.1441345214843750, -0.0088195800781250,
+0.1800231933593750, -0.0019836425781250, 0.0877685546875000, -0.0077514648437500,
+0.1762695312500000, 0.0040588378906250, 0.2073974609375000, 0.0028076171875000,
+-0.1308898925781250, -0.0004272460937500, -0.0194396972656250, -0.0001525878906250,
+-0.1567077636718750, 0.0003356933593750, -0.4968872070312500, -0.0007629394531250,
+-0.3207092285156250, -0.0009155273437500, -0.5058288574218750, 0.0000305175781250,
+-0.6077880859375000, 0.0002746582031250, -0.4500732421875000, -0.0000610351562500,
+-0.4114990234375000, -0.0000915527343750, -0.5037536621093750, 0.0000915527343750,
+-0.1015319824218750, -0.0001220703125000, -0.1683654785156250, -0.0003051757812500,
+-0.0827636718750000, -0.0001220703125000, 0.2221984863281250, 0.0000915527343750,
+0.0833740234375000, -0.0000610351562500, 0.2283935546875000, -0.0001220703125000,
+0.2940368652343750, 0.0000000000000000, 0.2145385742187500, -0.0000305175781250,
+0.2067871093750000, -0.0001525878906250, 0.2510375976562500, -0.0001525878906250,
+0.1177062988281250, 0.0000305175781250, 0.1247863769531250, 0.0000000000000000,
+0.1294860839843750, -0.0000305175781250, 0.0515136718750000, 0.0001831054687500,
+0.0729370117187500, 0.0001831054687500, 0.1216125488281250, -0.0001220703125000,
+0.0713500976562500, -0.0000915527343750, 0.1474609375000000, -0.0000610351562500,
+0.2345275878906250, -0.0002746582031250, 0.1615295410156250, -0.0002746582031250,
+0.3753356933593750, -0.0000305175781250, 0.3368530273437500, 0.0000000000000000,
+0.4011840820312500, -0.0000915527343750, 0.5679931640625000, -0.0000610351562500,
+0.4786682128906250, 0.0000305175781250, 0.6627807617187500, -0.0001525878906250,
+0.6630554199218750, -0.0002136230468750, 0.6695861816406250, -0.0000915527343750,
+0.7725524902343750, -0.0000915527343750, 0.7329101562500000, -0.0001525878906250,
+0.7465515136718750},
+{-0.0005798339843750, -0.3108825683593750, 0.0007629394531250, -0.3648681640625000,
+0.0009460449218750, -0.4243164062500000, -0.0008239746093750, -0.3346252441406250,
+0.0008239746093750, -0.2791442871093750, 0.0012512207031250, -0.3408813476562500,
+-0.0143432617187500, -0.1370544433593750, -0.0048522949218750, -0.1429748535156250,
+-0.0228271484375000, -0.1310424804687500, -0.0473632812500000, -0.0095520019531250,
+-0.0300292968750000, -0.0460510253906250, -0.0688171386718750, -0.0469970703125000,
+-0.0787048339843750, -0.0275268554687500, -0.0531005859375000, -0.0293579101562500,
+-0.0636596679687500, -0.0567932128906250, -0.0745239257812500, -0.0539855957031250,
+0.0472106933593750, 0.0350341796875000, 0.0094909667968750, -0.0060729980468750,
+0.0549011230468750, 0.0425415039062500, 0.2493591308593750, 0.2209472656250000,
+0.2226257324218750, -0.0434570312500000, -0.0920715332031250, 0.0476989746093750,
+0.0332946777343750, -0.0287475585937500, -0.0459594726562500, 0.0578918457031250,
+-0.2491760253906250, 0.1772155761718750, -0.1765136718750000, 0.1119689941406250,
+-0.1326599121093750, 0.1967468261718750, -0.2265625000000000, 0.2427978515625000,
+-0.1006469726562500, 0.1677551269531250, 0.0587463378906250, 0.1419677734375000,
+-0.0548095703125000, 0.1878967285156250, 0.1592712402343750, 0.0194702148437500,
+0.2184448242187500, 0.0302124023437500, 0.0982971191406250, 0.0194091796875000,
+0.1278076171875000, -0.0850219726562500, 0.1889343261718750, -0.0520019531250000,
+-0.1893005371093750, -0.0525817871093750, -0.0994262695312500, -0.0771484375000000,
+-0.2052307128906250, -0.0512390136718750, -0.5395812988281250, -0.0191955566406250,
+-0.3821105957031250, -0.0381469726562500, -0.4755249023437500, -0.0064392089843750,
+-0.6070556640625000, 0.0000915527343750, -0.4131774902343750, -0.0053710937500000,
+-0.2805175781250000, 0.0026855468750000, -0.4293518066406250, 0.0023803710937500,
+-0.0100708007812500, -0.0005493164062500, -0.0466613769531250, -0.0007934570312500,
+-0.0089721679687500, -0.0000305175781250, 0.2623596191406250, -0.0004577636718750,
+0.1594543457031250, -0.0008544921875000, 0.2152709960937500, 0.0000305175781250,
+0.2806091308593750, 0.0003356933593750, 0.2106628417968750, 0.0000610351562500,
+0.1579589843750000, 0.0000000000000000, 0.2079467773437500, 0.0000915527343750,
+0.1057128906250000, -0.0000610351562500, 0.0912780761718750, -0.0001525878906250,
+0.1239013671875000, -0.0000305175781250, 0.0921020507812500, 0.0000915527343750,
+0.0836791992187500, 0.0000305175781250, 0.1821899414062500, -0.0001831054687500,
+0.1429138183593750, -0.0000915527343750, 0.2030334472656250, -0.0000305175781250,
+0.3031921386718750, -0.0002136230468750, 0.2377014160156250, -0.0002136230468750,
+0.4121704101562500, 0.0000000000000000, 0.3858337402343750, 0.0000000000000000,
+0.4335021972656250, -0.0001220703125000, 0.5626525878906250, 0.0000000000000000,
+0.4908447265625000, 0.0000305175781250, 0.6326293945312500, -0.0001525878906250,
+0.6347351074218750, -0.0001831054687500, 0.6338195800781250, -0.0000610351562500,
+0.7112426757812500, -0.0000915527343750, 0.6859741210937500, -0.0001525878906250,
+0.6744995117187500, 0.0000610351562500, 0.7051696777343750, 0.0001220703125000,
+0.6531982421875000, 0.0000000000000000, 0.6072082519531250, 0.0000000000000000,
+0.6472473144531250, 0.0000610351562500, 0.4976501464843750, -0.0000305175781250,
+0.5300598144531250, -0.0000915527343750, 0.4682617187500000, 0.0000610351562500,
+0.3478698730468750, 0.0000610351562500, 0.4258422851562500, -0.0000305175781250,
+0.2342834472656250, 0.0000610351562500, 0.2681884765625000, 0.0001525878906250,
+0.2074890136718750, 0.0000305175781250, 0.0644226074218750, 0.0000000000000000,
+0.1543579101562500, 0.0000305175781250, -0.0498046875000000, 0.0000610351562500,
+-0.0177612304687500, 0.0000000000000000, -0.0734252929687500, 0.0000305175781250,
+-0.2216491699218750, 0.0000915527343750, -0.1313781738281250, 0.0000915527343750,
+-0.3152770996093750, 0.0000305175781250, -0.2975463867187500, 0.0000915527343750,
+-0.3227844238281250, 0.0000610351562500, -0.4385375976562500, -0.0000305175781250,
+-0.3763732910156250, 0.0000000000000000, -0.4769287109375000, 0.0000000000000000,
+-0.4782104492187500, 0.0000610351562500, -0.4771118164062500, 0.0000610351562500,
+-0.5298156738281250, 0.0000000000000000, -0.5107727050781250, 0.0000305175781250,
+-0.5236206054687500, -0.0000305175781250, -0.5336303710937500, -0.0000305175781250,
+-0.5214538574218750, 0.0000305175781250, -0.5184020996093750, -0.0000305175781250,
+-0.5277404785156250, 0.0000000000000000, -0.4983215332031250, 0.0000305175781250,
+-0.5080566406250000, 0.0000305175781250, -0.4951477050781250, 0.0000000000000000,
+-0.4705200195312500, 0.0000305175781250, -0.4904785156250000, 0.0000610351562500,
+-0.4471130371093750, 0.0000305175781250, -0.4572753906250000, 0.0000305175781250,
+-0.4469299316406250, -0.0000305175781250, -0.4150390625000000, 0.0000000000000000,
+-0.4376525878906250, 0.0000305175781250, -0.4032592773437500, 0.0000305175781250,
+-0.4099426269531250, 0.0000000000000000, -0.4055175781250000, 0.0000000000000000,
+-0.3834838867187500, 0.0000000000000000, -0.4017639160156250, 0.0000000000000000,
+-0.3817749023437500, -0.0000610351562500, -0.3836364746093750, -0.0000305175781250,
+-0.3861083984375000, 0.0000000000000000, -0.3776855468750000, -0.0000305175781250,
+-0.3828125000000000, -0.0000305175781250, -0.3716430664062500, 0.0000000000000000,
+-0.3740234375000000, 0.0000610351562500, -0.3623657226562500, 0.0000305175781250,
+-0.3507385253906250, 0.0000610351562500, -0.3580932617187500, 0.0000610351562500,
+-0.3125915527343750, 0.0000000000000000, -0.3250732421875000, 0.0000305175781250,
+-0.2967224121093750, 0.0000610351562500, -0.2507629394531250, 0.0000610351562500,
+-0.2764282226562500, 0.0000305175781250, -0.1992492675781250, 0.0000610351562500,
+-0.2055358886718750, 0.0000610351562500, -0.1876220703125000, 0.0000305175781250,
+-0.1327819824218750, 0.0000610351562500, -0.1558532714843750, 0.0000610351562500,
+-0.1101684570312500, 0.0000305175781250, -0.1090087890625000, -0.0000305175781250,
+-0.0961608886718750, 0.0000305175781250, -0.0700988769531250, 0.0000000000000000,
+-0.0819702148437500, -0.0000305175781250, -0.0164794921875000, 0.0000915527343750,
+-0.0387268066406250, 0.0000915527343750, 0.0201721191406250, 0.0000610351562500,
+0.0946350097656250, 0.0000915527343750, 0.0495605468750000, 0.0000915527343750,
+0.2416076660156250, -0.0000305175781250, 0.2158813476562500, 0.0000305175781250,
+0.2807922363281250, 0.0000610351562500, 0.4392395019531250, 0.0000305175781250,
+0.3680419921875000, 0.0000000000000000, 0.4974975585937500, 0.0000305175781250,
+0.5352478027343750, 0.0000305175781250, 0.4835815429687500, -0.0000305175781250,
+0.5119323730468750, -0.0000610351562500, 0.5302734375000000, -0.0000305175781250,
+0.3385925292968750, -0.0001525878906250, 0.3936157226562500, -0.0001525878906250,
+0.2852478027343750, -0.0000915527343750, 0.0853271484375000, -0.0001525878906250,
+0.1892395019531250, -0.0001831054687500, -0.0310668945312500, -0.0000610351562500,
+-0.0538940429687500, -0.0000610351562500, -0.0349121093750000, -0.0000915527343750,
+-0.1270751953125000, -0.0000915527343750, -0.1098022460937500, -0.0000305175781250,
+-0.0683288574218750, -0.0000610351562500, -0.0867309570312500, -0.0000915527343750,
+-0.0727539062500000, -0.0001220703125000, -0.0257873535156250, -0.0000305175781250,
+-0.0416564941406250, -0.0000305175781250, -0.0598144531250000, -0.0005798339843750,
+-0.0377807617187500, -0.0004882812500000, -0.0651245117187500, -0.0002136230468750,
+-0.1104431152343750, -0.0007019042968750, -0.0866699218750000, -0.0008544921875000,
+-0.1103515625000000, 0.0007629394531250, -0.1286621093750000, 0.0012512207031250,
+-0.1022338867187500, -0.0018615722656250, -0.0860900878906250, -0.0007934570312500,
+-0.1033325195312500, 0.0010070800781250, -0.0571899414062500, -0.0179748535156250,
+-0.0547180175781250, -0.0105895996093750, -0.0546569824218750, -0.0200805664062500,
+-0.0376892089843750, -0.0440673828125000, -0.0424499511718750, -0.0314025878906250,
+-0.0124816894531250, -0.0288696289062500, -0.0305175781250000, -0.0488281250000000,
+0.0045776367187500, -0.0141601562500000, 0.0694274902343750, 0.0300292968750000,
+0.0678710937500000, 0.0227050781250000, 0.1416931152343750, 0.0971069335937500,
+0.1363220214843750, 0.0832214355468750, 0.1088867187500000, 0.0736694335937500,
+0.1439208984375000, 0.1224975585937500, 0.1524047851562500, 0.1145019531250000,
+0.0537109375000000, 0.0643310546875000, 0.0845336914062500, 0.0853576660156250,
+0.0518493652343750, 0.0590209960937500, -0.0408935546875000, -0.0000915527343750,
+0.0040283203125000, 0.0263366699218750, -0.0414123535156250, 0.0006408691406250,
+-0.0643005371093750, -0.0127258300781250, -0.0384826660156250, -0.0052490234375000,
+-0.0313720703125000, 0.0021057128906250, -0.0448913574218750, -0.0015869140625000,
+-0.0099792480468750, -0.0347290039062500, -0.0075988769531250, -0.0136413574218750,
+-0.0104675292968750, -0.0461730957031250, 0.0036621093750000, -0.1016235351562500,
+0.0011291503906250, -0.0703430175781250, -0.0009460449218750, -0.1095886230468750,
+0.0001525878906250, -0.1315612792968750, 0.0001525878906250, -0.0980224609375000,
+-0.0014038085937500, -0.0840454101562500, -0.0014038085937500, -0.1065368652343750,
+-0.0001525878906250, -0.0480346679687500, -0.0002441406250000, -0.0419921875000000,
+-0.0003967285156250, -0.0556945800781250, 0.0001525878906250, -0.0370788574218750,
+0.0002136230468750, -0.0329284667968750, -0.0002136230468750, -0.0610656738281250,
+-0.0002441406250000, -0.0681152343750000, -0.0000610351562500, -0.0395507812500000,
+-0.0000915527343750, -0.0449523925781250, -0.0001831054687500, -0.0603332519531250,
+0.0000305175781250, 0.0742187500000000, 0.0001220703125000, 0.0301818847656250,
+0.0000000000000000, 0.1061096191406250, -0.0000915527343750, 0.2490234375000000,
+0.0000610351562500, 0.1718444824218750, -0.0000915527343750, 0.3338928222656250,
+-0.0001831054687500, 0.3500366210937500, -0.0000610351562500, 0.3376770019531250,
+0.0000610351562500, 0.4102783203125000, -0.0000610351562500, 0.3961181640625000,
+0.0000000000000000, 0.3518676757812500, 0.0000915527343750, 0.3813781738281250,
+-0.0000305175781250, 0.3280944824218750, -0.0001220703125000, 0.2565002441406250,
+-0.0000305175781250, 0.2941284179687500, -0.0000915527343750, 0.1663513183593750,
+-0.0001831054687500, 0.1713867187500000, -0.0000305175781250, 0.1346740722656250,
+0.0000305175781250, 0.0450439453125000, -0.0000610351562500, 0.0809631347656250,
+0.0000305175781250, -0.0210876464843750, 0.0000610351562500, -0.0188903808593750,
+0.0000000000000000, -0.0461120605468750, 0.0000000000000000, -0.1149597167968750,
+0.0000305175781250, -0.0856018066406250, -0.0000305175781250, -0.1666870117187500,
+-0.0000915527343750, -0.1665344238281250, 0.0000000000000000, -0.1855773925781250,
+0.0000610351562500, -0.2409667968750000, -0.0000305175781250, -0.2201232910156250,
+0.0000305175781250, -0.2632446289062500, 0.0000610351562500, -0.2703552246093750,
+0.0000610351562500, -0.2690429687500000, -0.0000610351562500, -0.2863769531250000,
+0.0000305175781250, -0.2810974121093750, 0.0000000000000000, -0.2762451171875000,
+-0.0000305175781250, -0.2801513671875000, 0.0000305175781250, -0.2651062011718750,
+0.0000915527343750, -0.2492980957031250, 0.0000000000000000, -0.2545166015625000,
+0.0000610351562500, -0.2243652343750000, 0.0000915527343750, -0.2183227539062500,
+0.0000305175781250},
+{-0.0603637695312500, 0.0606689453125000, -0.0570678710937500, 0.0610046386718750,
+-0.0403747558593750, 0.1234741210937500, -0.0387573242187500, 0.1129150390625000,
+0.0286254882812500, 0.0632934570312500, -0.0102539062500000, 0.0929565429687500,
+0.0227661132812500, 0.0717163085937500, 0.0971984863281250, 0.0140991210937500,
+0.0457763671875000, 0.0413513183593750, 0.1307067871093750, 0.0025634765625000,
+0.1504516601562500, -0.0105895996093750, 0.1108703613281250, -0.0009460449218750,
+0.1275329589843750, -0.0007019042968750, 0.1464233398437500, -0.0051879882812500,
+0.0373535156250000, -0.0262451171875000, 0.0596618652343750, -0.0065917968750000,
+0.0369567871093750, -0.0407104492187500, -0.0514221191406250, -0.0883178710937500,
+-0.0159301757812500, -0.0579223632812500, -0.0392456054687500, -0.1113891601562500,
+-0.0606079101562500, -0.1285095214843750, -0.0375061035156250, -0.1009216308593750,
+-0.0214233398437500, -0.1009521484375000, -0.0361938476562500, -0.1149597167968750,
+-0.0067138671875000, -0.0612792968750000, -0.0031127929687500, -0.0590820312500000,
+-0.0070190429687500, -0.0682678222656250, 0.0028991699218750, -0.0478820800781250,
+0.0017089843750000, -0.0457763671875000, -0.0007629394531250, -0.0618896484375000,
+-0.0003662109375000, -0.0708923339843750, -0.0000305175781250, -0.0377807617187500,
+-0.0010375976562500, -0.0332946777343750, -0.0012207031250000, -0.0540466308593750,
+-0.0000915527343750, 0.0910034179687500, -0.0000305175781250, 0.0503845214843750,
+-0.0003051757812500, 0.1168518066406250, 0.0000305175781250, 0.2573242187500000,
+0.0001525878906250, 0.1818847656250000, -0.0001831054687500, 0.3161926269531250,
+-0.0002441406250000, 0.3392944335937500, -0.0000915527343750, 0.3106384277343750,
+-0.0000610351562500, 0.3538208007812500, -0.0001525878906250, 0.3550415039062500,
+-0.0000305175781250, 0.2843933105468750, 0.0000305175781250, 0.3102722167968750,
+-0.0000305175781250, 0.2601318359375000, -0.0001525878906250, 0.1817016601562500,
+-0.0000610351562500, 0.2274780273437500, -0.0001525878906250, 0.1068115234375000,
+-0.0002441406250000, 0.1127929687500000, -0.0001525878906250, 0.0799255371093750,
+0.0000305175781250, -0.0038452148437500, -0.0000915527343750, 0.0329589843750000,
+-0.0000915527343750, -0.0661010742187500, 0.0000000000000000, -0.0656127929687500,
+-0.0000915527343750, -0.0878906250000000, -0.0001831054687500, -0.1544799804687500,
+-0.0000915527343750, -0.1278076171875000, -0.0000610351562500, -0.1786804199218750,
+-0.0001831054687500, -0.1873779296875000, -0.0000915527343750, -0.1784057617187500,
+0.0000305175781250, -0.1965332031250000, -0.0000610351562500, -0.1948547363281250,
+-0.0000610351562500, -0.1659851074218750, 0.0000000000000000, -0.1768798828125000,
+-0.0000305175781250, -0.1524353027343750, -0.0000915527343750, -0.1155395507812500,
+-0.0000610351562500, -0.1303100585937500, -0.0000915527343750, -0.0678710937500000,
+-0.0000915527343750, -0.0670776367187500, -0.0000305175781250, -0.0557556152343750,
+-0.0000305175781250, -0.0147399902343750, -0.0000305175781250, -0.0249633789062500,
+-0.0000305175781250, -0.0041198730468750, 0.0000000000000000, 0.0035400390625000,
+-0.0000610351562500, -0.0034790039062500, -0.0000305175781250, 0.0004272460937500,
+-0.0000305175781250, 0.0052185058593750, -0.0000305175781250, -0.0151672363281250,
+-0.0000610351562500, -0.0097351074218750, -0.0000610351562500, -0.0226440429687500,
+-0.0000305175781250, -0.0453491210937500, -0.0000305175781250, -0.0367736816406250,
+-0.0000610351562500, -0.0687561035156250, -0.0000305175781250, -0.0691833496093750,
+-0.0000305175781250, -0.0789184570312500, -0.0000610351562500, -0.1008300781250000,
+-0.0000610351562500, -0.0932617187500000, -0.0000610351562500, -0.1209411621093750,
+-0.0000610351562500, -0.1241149902343750, -0.0000305175781250, -0.1290588378906250,
+-0.0000305175781250, -0.1446838378906250, -0.0000610351562500, -0.1427612304687500,
+0.0000000000000000, -0.1587829589843750, -0.0000305175781250, -0.1578369140625000,
+-0.0000305175781250, -0.1651000976562500, -0.0000305175781250, -0.1777648925781250,
+-0.0000305175781250, -0.1710815429687500, 0.0000000000000000, -0.1881713867187500,
+0.0000000000000000, -0.1882629394531250, 0.0000000000000000, -0.1938171386718750,
+0.0000000000000000, -0.2069702148437500, 0.0000000000000000, -0.2039794921875000,
+-0.0000305175781250, -0.2175903320312500, 0.0000305175781250, -0.2176818847656250,
+-0.0000305175781250, -0.2265014648437500, 0.0000000000000000, -0.2385559082031250,
+0.0000000000000000, -0.2338562011718750, -0.0000915527343750, -0.2576293945312500,
+-0.0000915527343750, -0.2524108886718750, -0.0000305175781250, -0.2656860351562500,
+-0.0000915527343750, -0.2885131835937500, -0.0000610351562500, -0.2783813476562500,
+-0.0000305175781250, -0.3191223144531250, -0.0000610351562500, -0.3118591308593750,
+-0.0000305175781250, -0.3230590820312500, -0.0000305175781250, -0.3559570312500000,
+-0.0000305175781250, -0.3399658203125000, -0.0000915527343750, -0.3631591796875000,
+-0.0001220703125000, -0.3739013671875000, -0.0000915527343750, -0.3549499511718750,
+-0.0000610351562500, -0.3566284179687500, -0.0000915527343750, -0.3683166503906250,
+-0.0000305175781250, -0.2950744628906250, 0.0000305175781250, -0.3241882324218750,
+-0.0000305175781250, -0.2683715820312500, -0.0000610351562500, -0.1877136230468750,
+-0.0000305175781250, -0.2430419921875000, -0.0000610351562500, -0.0823669433593750,
+-0.0001525878906250, -0.1079406738281250, -0.0000915527343750, -0.0522460937500000,
+0.0000000000000000, 0.0711059570312500, -0.0000610351562500, 0.0015869140625000,
+0.0000305175781250, 0.1760864257812500, 0.0000610351562500, 0.1555480957031250,
+0.0000305175781250, 0.2110595703125000, -0.0000610351562500, 0.3431396484375000,
+0.0000305175781250, 0.2767333984375000, -0.0000915527343750, 0.4466857910156250,
+-0.0001220703125000, 0.4412536621093750, -0.0000610351562500, 0.4703979492187500,
+-0.0000610351562500, 0.5823974609375000, -0.0000915527343750, 0.5355224609375000,
+0.0000915527343750, 0.5959472656250000, 0.0000915527343750, 0.6173706054687500,
+0.0000610351562500, 0.5903320312500000, 0.0001831054687500, 0.5982971191406250,
+0.0001831054687500, 0.6102600097656250, -0.0001831054687500, 0.5385131835937500,
+-0.0000915527343750, 0.5610351562500000, -0.0000610351562500, 0.5170288085937500,
+-0.0003967285156250, 0.4487609863281250, -0.0003662109375000, 0.4931640625000000,
+0.0005187988281250, 0.3550109863281250, 0.0006103515625000, 0.3804016113281250,
+-0.0007629394531250, 0.3309631347656250, 0.0005187988281250, 0.2160339355468750,
+0.0008850097656250, 0.2766723632812500, -0.0118103027343750, 0.1479187011718750,
+-0.0042724609375000, 0.1417846679687500, -0.0186767578125000, 0.1415710449218750,
+-0.0385437011718750, 0.0809936523437500, -0.0241394042968750, 0.1002807617187500,
+-0.0523986816406250, 0.0739135742187500, -0.0619506835937500, 0.0835571289062500,
+-0.0382385253906250, 0.0457763671875000, -0.0428161621093750, 0.0177612304687500,
+-0.0552673339843750, 0.0503845214843750, 0.0478820800781250, -0.1168823242187500,
+0.0130920410156250, -0.0768432617187500, 0.0668029785156250, -0.1484375000000000,
+0.1809692382812500, -0.3118591308593750, 0.1198120117187500, -0.2284851074218750,
+0.1936645507812500, -0.3299560546875000, 0.2398376464843750, -0.3833923339843750,
+0.1556396484375000, -0.2956237792968750, 0.1265258789062500, -0.2704772949218750,
+0.1790466308593750, -0.3250427246093750, -0.0526428222656250, -0.1226806640625000,
+-0.0312194824218750, -0.1373901367187500, -0.0366516113281250, -0.1355285644531250,
+-0.2461547851562500, 0.0400390625000000, 0.0039062500000000, -0.2932739257812500,
+0.1537780761718750, -0.0610961914062500, 0.2085266113281250, -0.1352539062500000,
+0.0702819824218750, -0.0910644531250000, 0.0177307128906250, 0.0338439941406250,
+0.1182861328125000, -0.0017395019531250, -0.2425537109375000, -0.0117797851562500,
+-0.2084350585937500, 0.0092468261718750, -0.2385864257812500, 0.0050354003906250,
+-0.4871215820312500, -0.0282592773437500, -0.4005737304687500, -0.0231933593750000,
+-0.3400878906250000, 0.0533447265625000, -0.4581909179687500, 0.0166625976562500,
+-0.2872924804687500, 0.0615844726562500, -0.0836791992187500, 0.1614074707031250,
+-0.2298278808593750, 0.1115722656250000, 0.1080322265625000, 0.1468505859375000,
+0.1210632324218750, 0.1906738281250000, 0.1000976562500000, 0.1253356933593750,
+0.2607727050781250, 0.0773315429687500, 0.2251586914062500, 0.1234436035156250,
+0.1763610839843750, -0.0053710937500000, 0.2153625488281250, -0.0080871582031250,
+0.1872863769531250, -0.0042419433593750, 0.1163940429687500, -0.0697021484375000,
+0.1482543945312500, -0.0548095703125000, 0.1411132812500000, -0.0360107421875000,
+0.1137695312500000, -0.0505371093750000, 0.1495971679687500, -0.0350341796875000,
+0.1818847656250000, -0.0059204101562500, 0.1551208496093750, -0.0194702148437500,
+0.2324523925781250, -0.0026245117187500, 0.2192687988281250, 0.0028991699218750,
+0.2503051757812500, -0.0013122558593750, 0.3050231933593750, 0.0005798339843750,
+0.2706909179687500, 0.0012207031250000, 0.4072570800781250, -0.0003051757812500,
+0.3674316406250000, -0.0008850097656250, 0.4372863769531250, -0.0002746582031250,
+0.5701599121093750, 0.0000000000000000, 0.4858703613281250, -0.0002441406250000,
+0.6530151367187500, 0.0000000000000000, 0.6670532226562500, 0.0002136230468750,
+0.6310119628906250, 0.0000610351562500, 0.7058410644531250, -0.0000610351562500,
+0.7026367187500000, -0.0000610351562500, 0.5528564453125000, 0.0000610351562500,
+0.6349487304687500, 0.0000610351562500, 0.5113830566406250, 0.0000000000000000,
+0.3264770507812500, 0.0000610351562500, 0.4577636718750000, 0.0001220703125000,
+0.1369323730468750, -0.0000305175781250, 0.1669311523437500, -0.0000610351562500,
+0.1141052246093750, 0.0000305175781250, -0.0982971191406250, 0.0000915527343750,
+-0.0013122558593750, 0.0000000000000000, -0.1686096191406250, 0.0000305175781250,
+-0.1759643554687500, 0.0000915527343750, -0.1829833984375000, 0.0000000000000000,
+-0.2682800292968750, -0.0000915527343750, -0.2325439453125000, -0.0000305175781250,
+-0.2824707031250000, -0.0000610351562500, -0.2871093750000000, -0.0000915527343750,
+-0.2875061035156250, -0.0000305175781250, -0.3121032714843750, 0.0000305175781250,
+-0.3025512695312500, 0.0000305175781250, -0.3114013671875000, -0.0000305175781250,
+-0.3173828125000000, 0.0000000000000000, -0.3142089843750000, -0.0000610351562500,
+-0.3144531250000000, -0.0000915527343750, -0.3181762695312500, -0.0000915527343750,
+-0.3121948242187500, -0.0000305175781250, -0.3122558593750000, -0.0000610351562500,
+-0.3116149902343750, -0.0000305175781250, -0.3067932128906250, 0.0000610351562500,
+-0.3059997558593750, 0.0000610351562500, -0.3039855957031250, -0.0000305175781250,
+-0.3025817871093750, -0.0000305175781250, -0.2998962402343750, 0.0000000000000000,
+-0.2963562011718750, 0.0000000000000000, -0.2941589355468750, 0.0000000000000000,
+-0.2900695800781250, 0.0000610351562500, -0.2878417968750000, 0.0001220703125000,
+-0.2817687988281250, 0.0000610351562500, -0.2777099609375000, 0.0000305175781250,
+-0.2781677246093750, 0.0000305175781250, -0.2630004882812500, 0.0000000000000000,
+-0.2662963867187500, 0.0000000000000000, -0.2539978027343750, 0.0000305175781250,
+-0.2375183105468750, 0.0000305175781250, -0.2452392578125000, 0.0000305175781250,
+-0.2116088867187500, 0.0000305175781250, -0.2178955078125000, 0.0000305175781250,
+-0.2012939453125000, 0.0000305175781250, -0.1720275878906250, 0.0000305175781250,
+-0.1882019042968750},
+{0.2118225097656250, -0.0136718750000000, 0.1962280273437500, -0.0140380859375000,
+0.1408691406250000, 0.0647888183593750, 0.1605834960937500, 0.0368957519531250,
+0.1566772460937500, 0.0690917968750000, 0.1090087890625000, 0.1566772460937500,
+0.1226501464843750, 0.1175842285156250, 0.1613159179687500, 0.1268005371093750,
+0.1355285644531250, 0.1672058105468750, 0.1660461425781250, 0.1080017089843750,
+0.2200012207031250, 0.0505065917968750, 0.1897888183593750, 0.0946655273437500,
+0.2463073730468750, -0.0148010253906250, 0.2379455566406250, -0.0236816406250000,
+0.2675781250000000, -0.0135192871093750, 0.3056030273437500, -0.0597229003906250,
+0.2740173339843750, -0.0526428222656250, 0.4207458496093750, -0.0270996093750000,
+0.3740234375000000, -0.0372314453125000, 0.4507751464843750, -0.0272827148437500,
+0.6018066406250000, -0.0013732910156250, 0.5105590820312500, -0.0117492675781250,
+0.6709594726562500, -0.0015258789062500, 0.7063293457031250, 0.0028686523437500,
+0.6325378417968750, -0.0001525878906250, 0.6725158691406250, 0.0001525878906250,
+0.7025146484375000, 0.0006408691406250, 0.4535217285156250, -0.0000305175781250,
+0.5443420410156250, -0.0005493164062500, 0.4093627929687500, -0.0001220703125000,
+0.1555786132812500, 0.0003356933593750, 0.3118896484375000, 0.0001220703125000,
+-0.0139160156250000, 0.0001220703125000, -0.0099182128906250, 0.0002746582031250,
+-0.0221862792968750, 0.0001831054687500, -0.2009277343750000, 0.0000305175781250,
+-0.1327209472656250, 0.0000305175781250, -0.1970214843750000, 0.0000305175781250,
+-0.2190246582031250, 0.0000915527343750, -0.2084350585937500, 0.0000000000000000,
+-0.2209472656250000, -0.0000305175781250, -0.2208251953125000, 0.0000000000000000,
+-0.2389526367187500, -0.0000305175781250, -0.2253112792968750, -0.0001220703125000,
+-0.2509155273437500, -0.0000305175781250, -0.2798156738281250, 0.0000915527343750,
+-0.2557373046875000, 0.0000305175781250, -0.3327636718750000, 0.0000305175781250,
+-0.3203735351562500, 0.0000915527343750, -0.3408203125000000, 0.0000000000000000,
+-0.3985595703125000, -0.0000915527343750, -0.3667602539062500, 0.0000000000000000,
+-0.4176025390625000, 0.0000610351562500, -0.4205017089843750, 0.0000000000000000,
+-0.4105834960937500, 0.0000000000000000, -0.4321289062500000, 0.0000610351562500,
+-0.4277954101562500, 0.0000610351562500, -0.4018859863281250, -0.0000610351562500,
+-0.4188537597656250, -0.0000305175781250, -0.3907165527343750, -0.0000610351562500,
+-0.3562622070312500, -0.0001220703125000, -0.3843994140625000, -0.0000915527343750,
+-0.3037719726562500, -0.0000610351562500, -0.3224182128906250, -0.0000610351562500,
+-0.2960815429687500, -0.0000915527343750, -0.2349548339843750, -0.0000915527343750,
+-0.2761840820312500, -0.0000610351562500, -0.1838989257812500, -0.0001220703125000,
+-0.2023620605468750, -0.0001220703125000, -0.1808471679687500, -0.0000915527343750,
+-0.1141662597656250, -0.0000610351562500, -0.1560363769531250, -0.0000915527343750,
+-0.0773925781250000, -0.0000305175781250, -0.0887451171875000, 0.0000000000000000,
+-0.0746154785156250, -0.0000305175781250, -0.0236206054687500, -0.0000915527343750,
+-0.0592956542968750, -0.0000305175781250, -0.0043029785156250, -0.0000610351562500,
+-0.0125427246093750, -0.0000915527343750, -0.0080871582031250, -0.0000610351562500,
+0.0251159667968750, -0.0000610351562500, 0.0027160644531250, -0.0000610351562500,
+0.0329895019531250, -0.0000915527343750, 0.0284729003906250, -0.0000610351562500,
+0.0291137695312500, -0.0000610351562500, 0.0463867187500000, -0.0001220703125000,
+0.0343017578125000, -0.0000610351562500, 0.0484313964843750, -0.0000915527343750,
+0.0449523925781250, -0.0000915527343750, 0.0463256835937500, -0.0001220703125000,
+0.0550231933593750, -0.0001220703125000, 0.0464172363281250, -0.0001525878906250,
+0.0575256347656250, -0.0001220703125000, 0.0561523437500000, -0.0000915527343750,
+0.0553283691406250, -0.0000610351562500, 0.0609741210937500, -0.0000610351562500,
+0.0577697753906250, -0.0000610351562500, 0.0620422363281250, -0.0000915527343750,
+0.0597534179687500, -0.0000915527343750, 0.0635070800781250, -0.0000610351562500,
+0.0668640136718750, -0.0000610351562500, 0.0625305175781250, -0.0000915527343750,
+0.0853881835937500, -0.0000610351562500, 0.0772705078125000, -0.0000305175781250,
+0.0928344726562500, -0.0000305175781250, 0.1176452636718750, -0.0000305175781250,
+0.1037292480468750, -0.0000305175781250, 0.1499938964843750, 0.0000000000000000,
+0.1476135253906250, 0.0000000000000000, 0.1603088378906250, -0.0000610351562500,
+0.1957702636718750, -0.0000915527343750, 0.1819763183593750, -0.0000305175781250,
+0.2008666992187500, -0.0000610351562500, 0.2115173339843750, -0.0000915527343750,
+0.1894836425781250, -0.0000915527343750, 0.1824340820312500, -0.0000610351562500,
+0.1915283203125000, -0.0000915527343750, 0.1213073730468750, -0.0000915527343750,
+0.1347045898437500, -0.0001220703125000, 0.0965881347656250, -0.0000915527343750,
+0.0281982421875000, -0.0001525878906250, 0.0592041015625000, -0.0001525878906250,
+-0.0258789062500000, -0.0001220703125000, -0.0330810546875000, -0.0000915527343750,
+-0.0324401855468750, -0.0000915527343750, -0.0733642578125000, -0.0001220703125000,
+-0.0647277832031250, -0.0000610351562500, -0.0595397949218750, -0.0001220703125000,
+-0.0713195800781250, -0.0001220703125000, -0.0586547851562500, -0.0000915527343750,
+-0.0414428710937500, -0.0000610351562500, -0.0530700683593750, -0.0001525878906250,
+-0.0297851562500000, -0.0002136230468750, -0.0315551757812500, -0.0000915527343750,
+-0.0227355957031250, -0.0001831054687500, -0.0122070312500000, -0.0005187988281250,
+-0.0216674804687500, -0.0003967285156250, 0.0413818359375000, 0.0000305175781250,
+0.0115356445312500, -0.0003051757812500, 0.0735168457031250, -0.0001220703125000,
+0.1561279296875000, 0.0013427734375000, 0.1024475097656250, 0.0006103515625000,
+0.2796325683593750, -0.0042419433593750, 0.2715454101562500, 0.0007629394531250,
+0.2845153808593750, -0.0086975097656250, 0.4034118652343750, -0.0194396972656250,
+0.3645935058593750, -0.0101013183593750, 0.3181152343750000, -0.0311584472656250,
+0.3832702636718750, -0.0363464355468750, 0.2771911621093750, -0.0204467773437500,
+0.1506042480468750, -0.0252075195312500, 0.2355651855468750, -0.0337524414062500,
+0.0089416503906250, 0.0332336425781250, 0.0066528320312500, 0.0133972167968750,
+0.0076904296875000, 0.0353393554687500, -0.1124267578125000, 0.0996398925781250,
+-0.0805969238281250, 0.0671081542968750, -0.0675659179687500, 0.0837402343750000,
+-0.0889892578125000, 0.1117858886718750, -0.0744018554687500, 0.0733337402343750,
+-0.0403442382812500, 0.0406188964843750, -0.0523376464843750, 0.0704040527343750,
+-0.0683898925781250, 0.0064392089843750, -0.0518493652343750, -0.0002136230468750,
+-0.0663146972656250, 0.0078430175781250, -0.1066589355468750, -0.0120544433593750,
+-0.1015625000000000, -0.0105590820312500, -0.0849304199218750, -0.0212707519531250,
+-0.1003417968750000, -0.0078430175781250, -0.0890502929687500, -0.0293884277343750,
+-0.0649719238281250, -0.0541076660156250, -0.0801086425781250, -0.0334777832031250,
+-0.0572814941406250, -0.0984191894531250, -0.0572204589843750, -0.1036376953125000,
+-0.0457763671875000, -0.0908508300781250, -0.0363464355468750, -0.1171569824218750,
+-0.0482177734375000, -0.1185302734375000, 0.0260620117187500, -0.0721130371093750,
+-0.0001525878906250, -0.0812072753906250, 0.0357360839843750, -0.0747985839843750,
+0.1181640625000000, -0.0384826660156250, 0.0743713378906250, -0.0475769042968750,
+0.1187744140625000, -0.0502014160156250, 0.1502990722656250, -0.0504455566406250,
+0.1014099121093750, -0.0319213867187500, 0.0788269042968750, -0.0334167480468750,
+0.1125183105468750, -0.0449218750000000, 0.0079345703125000, 0.0632934570312500,
+0.0130615234375000, 0.0274353027343750, 0.0100402832031250, 0.0859069824218750,
+-0.0516967773437500, 0.2008972167968750, -0.0335693359375000, 0.1374511718750000,
+-0.0300903320312500, 0.2487182617187500, -0.0446777343750000, 0.2668151855468750,
+-0.0297851562500000, 0.2431335449218750, -0.0090332031250000, 0.2800292968750000,
+-0.0206298828125000, 0.2806701660156250, -0.0030822753906250, 0.2212219238281250,
+0.0009460449218750, 0.2435913085937500, -0.0026245117187500, 0.2076110839843750,
+0.0011291503906250, 0.1471862792968750, 0.0013122558593750, 0.1826782226562500,
+-0.0005493164062500, 0.0822143554687500, -0.0007324218750000, 0.0928955078125000,
+-0.0002136230468750, 0.0657043457031250, -0.0003967285156250, -0.0097656250000000,
+-0.0007324218750000, 0.0261230468750000, -0.0000305175781250, -0.0380859375000000,
+0.0000915527343750, -0.0497436523437500, -0.0000610351562500, -0.0340270996093750,
+-0.0000610351562500, -0.0550842285156250, 0.0000305175781250, -0.0568847656250000,
+-0.0000610351562500, -0.0080261230468750, -0.0001220703125000, -0.0224304199218750,
+-0.0000305175781250, 0.0092163085937500, 0.0001220703125000, 0.0643615722656250,
+-0.0000305175781250, 0.0397033691406250, -0.0000305175781250, 0.1037292480468750,
+0.0000915527343750, 0.1130065917968750, 0.0000305175781250, 0.1100158691406250,
+-0.0000610351562500, 0.1377563476562500, 0.0000000000000000, 0.1337890625000000,
+0.0000000000000000, 0.1172485351562500, -0.0000915527343750, 0.1274414062500000,
+-0.0000610351562500, 0.1090393066406250, 0.0000305175781250, 0.0824584960937500,
+0.0000305175781250, 0.0967712402343750, -0.0000305175781250, 0.0623168945312500,
+0.0000305175781250, 0.0603027343750000, -0.0000305175781250, 0.0609741210937500,
+-0.0000610351562500, 0.0437622070312500, -0.0000610351562500, 0.0484619140625000,
+0.0000000000000000, 0.0541687011718750, -0.0000305175781250, 0.0495300292968750,
+-0.0000610351562500, 0.0591735839843750, -0.0000305175781250, 0.0718383789062500,
+0.0000000000000000, 0.0680236816406250, -0.0000610351562500, 0.0887451171875000,
+-0.0000610351562500, 0.0903625488281250, -0.0000610351562500, 0.0955505371093750,
+-0.0001525878906250, 0.1107788085937500, -0.0000915527343750, 0.1090393066406250,
+-0.0000610351562500, 0.1139221191406250, -0.0000610351562500, 0.1198120117187500,
+-0.0000610351562500, 0.1173095703125000, 0.0000000000000000, 0.1151428222656250,
+0.0000000000000000, 0.1176757812500000, -0.0000610351562500, 0.1060180664062500,
+0.0000000000000000, 0.1091918945312500, -0.0000610351562500, 0.1035766601562500,
+-0.0000610351562500, 0.0923461914062500, -0.0000915527343750, 0.0976562500000000,
+-0.0000610351562500, 0.0821228027343750, -0.0000610351562500, 0.0827026367187500,
+-0.0000915527343750, 0.0780639648437500, -0.0000915527343750, 0.0676879882812500,
+-0.0000915527343750, 0.0731811523437500, -0.0001525878906250, 0.0596618652343750,
+-0.0001220703125000, 0.0615844726562500, -0.0000610351562500, 0.0566406250000000,
+-0.0000915527343750, 0.0466613769531250, -0.0001220703125000, 0.0534057617187500,
+-0.0000610351562500, 0.0382080078125000, -0.0000610351562500, 0.0412597656250000,
+-0.0000610351562500, 0.0370788574218750, -0.0000610351562500, 0.0263061523437500,
+-0.0000915527343750, 0.0321655273437500, -0.0000610351562500, 0.0108032226562500,
+-0.0000915527343750, 0.0161743164062500, -0.0000610351562500, 0.0032348632812500,
+0.0000610351562500, -0.0150451660156250, 0.0000000000000000, -0.0018005371093750,
+-0.0000305175781250, -0.0494079589843750, -0.0000305175781250, -0.0325317382812500,
+0.0000000000000000, -0.0585632324218750, 0.0000000000000000, -0.1021118164062500,
+-0.0000305175781250, -0.0697326660156250, 0.0000000000000000, -0.1598205566406250,
+0.0000305175781250},
+{-0.0241088867187500, 0.1052856445312500, -0.0354003906250000, 0.1170043945312500,
+-0.0246582031250000, 0.0935058593750000, -0.0049438476562500, 0.0232849121093750,
+-0.0148620605468750, 0.0571594238281250, -0.0021057128906250, 0.0099487304687500,
+0.0014343261718750, -0.0071411132812500, -0.0016784667968750, 0.0197143554687500,
+0.0005493164062500, 0.0182495117187500, 0.0009765625000000, 0.0055236816406250,
+-0.0004882812500000, 0.0735778808593750, -0.0007934570312500, 0.0628662109375000,
+-0.0003051757812500, 0.0848999023437500, -0.0002746582031250, 0.1426696777343750,
+-0.0005493164062500, 0.1171569824218750, -0.0000915527343750, 0.1470642089843750,
+0.0000610351562500, 0.1622009277343750, -0.0000915527343750, 0.1417846679687500,
+-0.0001220703125000, 0.1351318359375000, -0.0000610351562500, 0.1439819335937500,
+-0.0000610351562500, 0.1023254394531250, -0.0000915527343750, 0.1021728515625000,
+-0.0000915527343750, 0.0958862304687500, 0.0000610351562500, 0.0688171386718750,
+0.0000000000000000, 0.0771179199218750, -0.0000305175781250, 0.0768127441406250,
+0.0000305175781250, 0.0679016113281250, 0.0000305175781250, 0.0797424316406250,
+-0.0000610351562500, 0.0924377441406250, -0.0000305175781250, 0.0837402343750000,
+0.0000000000000000, 0.1064758300781250, -0.0000305175781250, 0.1086730957031250,
+0.0000000000000000, 0.1072082519531250, 0.0000305175781250, 0.1184082031250000,
+0.0000000000000000, 0.1157226562500000, -0.0000610351562500, 0.1006774902343750,
+0.0000000000000000, 0.1076965332031250, 0.0000000000000000, 0.0928344726562500,
+-0.0000915527343750, 0.0710449218750000, -0.0000610351562500, 0.0792236328125000,
+0.0000000000000000, 0.0426025390625000, -0.0000610351562500, 0.0424499511718750,
+-0.0000610351562500, 0.0279235839843750, 0.0000000000000000, -0.0018310546875000,
+0.0000000000000000, 0.0065307617187500, -0.0000610351562500, -0.0233764648437500,
+-0.0000305175781250, -0.0295715332031250, -0.0000610351562500, -0.0317993164062500,
+-0.0001220703125000, -0.0461120605468750, -0.0001220703125000, -0.0462341308593750,
+-0.0000610351562500, -0.0495605468750000, -0.0001220703125000, -0.0544128417968750,
+-0.0001220703125000, -0.0548095703125000, -0.0001220703125000, -0.0542297363281250,
+-0.0000915527343750, -0.0568847656250000, -0.0001220703125000, -0.0551757812500000,
+-0.0001220703125000, -0.0570373535156250, -0.0000610351562500, -0.0592041015625000,
+-0.0001220703125000, -0.0581359863281250, -0.0001220703125000, -0.0582885742187500,
+-0.0000915527343750, -0.0612487792968750, -0.0000915527343750, -0.0590820312500000,
+-0.0000610351562500, -0.0628356933593750, -0.0000610351562500, -0.0673217773437500,
+-0.0000305175781250, -0.0640563964843750, -0.0000305175781250, -0.0756225585937500,
+-0.0000305175781250, -0.0747375488281250, -0.0000305175781250, -0.0774230957031250,
+-0.0000305175781250, -0.0852661132812500, -0.0000305175781250, -0.0810241699218750,
+0.0000000000000000, -0.0869445800781250, 0.0000610351562500, -0.0906372070312500,
+0.0000000000000000, -0.0874938964843750, -0.0000305175781250, -0.0880432128906250,
+0.0000000000000000, -0.0905151367187500, 0.0000000000000000, -0.0763549804687500,
+-0.0000305175781250, -0.0836181640625000, 0.0000000000000000, -0.0747070312500000,
+0.0000610351562500, -0.0592346191406250, 0.0000305175781250, -0.0705261230468750,
+0.0000915527343750, -0.0436706542968750, 0.0000305175781250, -0.0460205078125000,
+0.0000610351562500, -0.0494995117187500, 0.0000305175781250, -0.0332336425781250,
+0.0000305175781250, -0.0370788574218750, 0.0001220703125000, -0.0546875000000000,
+0.0001220703125000, -0.0361328125000000, 0.0001220703125000, -0.0671997070312500,
+0.0001525878906250, -0.0989379882812500, 0.0001525878906250, -0.0697631835937500,
+0.0001220703125000, -0.1678771972656250, 0.0001831054687500, -0.1428833007812500,
+0.0001220703125000, -0.1870727539062500, 0.0000610351562500, -0.2765502929687500,
+0.0001220703125000, -0.2247314453125000, 0.0000915527343750, -0.3427124023437500,
+0.0000610351562500, -0.3390502929687500, 0.0000915527343750, -0.3481140136718750,
+0.0001831054687500, -0.4196166992187500, 0.0001525878906250, -0.3893737792968750,
+0.0000915527343750, -0.4016723632812500, 0.0001831054687500, -0.4269714355468750,
+0.0000610351562500, -0.3870544433593750, -0.0002136230468750, -0.3589172363281250,
+-0.0000610351562500, -0.3862609863281250, 0.0001525878906250, -0.2825317382812500,
+0.0000000000000000, -0.2941894531250000, -0.0001525878906250, -0.2705078125000000,
+0.0007934570312500, -0.1939697265625000, 0.0005187988281250, -0.2289428710937500,
+-0.0046081542968750, -0.1925048828125000, -0.0001525878906250, -0.1823425292968750,
+-0.0088500976562500, -0.1885986328125000, -0.0189208984375000, -0.1914672851562500,
+-0.0110168457031250, -0.1928405761718750, -0.0330505371093750, -0.1350708007812500,
+-0.0368957519531250, -0.1833801269531250, -0.0245361328125000, -0.0851135253906250,
+-0.0333862304687500, 0.0185241699218750, -0.0382385253906250, -0.0673217773437500,
+0.0267944335937500, 0.2223815917968750, 0.0025634765625000, 0.1786804199218750,
+0.0400085449218750, 0.2517700195312500, 0.1159667968750000, 0.4779663085937500,
+0.0748901367187500, 0.3754272460937500, 0.1268005371093750, 0.5231323242187500,
+0.1592407226562500, 0.5637512207031250, 0.1007385253906250, 0.5187072753906250,
+0.0804443359375000, 0.5458984375000000, 0.1196899414062500, 0.5578002929687500,
+-0.0356750488281250, 0.4752502441406250, -0.0218811035156250, 0.4927062988281250,
+-0.0296630859375000, 0.4655151367187500, -0.1366271972656250, 0.4005126953125000,
+-0.1047973632812500, 0.4323120117187500, -0.0500183105468750, 0.3558044433593750,
+-0.1061096191406250, 0.3677978515625000, -0.0330810546875000, 0.3298339843750000,
+0.0859680175781250, 0.2677917480468750, 0.0160217285156250, 0.3080139160156250,
+0.0896606445312500, 0.1765136718750000, 0.1525573730468750, 0.1936950683593750,
+0.0406799316406250, 0.1615295410156250, -0.0169067382812500, 0.0634765625000000,
+0.0597229003906250, 0.1101989746093750, -0.2232360839843750, 0.0382080078125000,
+-0.2106018066406250, 0.0317993164062500, -0.1996765136718750, 0.0279846191406250,
+-0.4184875488281250, -0.0307617187500000, -0.1030273437500000, 0.1120605468750000,
+0.1310729980468750, 0.0146179199218750, 0.1035461425781250, 0.0373229980468750,
+0.0939941406250000, 0.0086364746093750, 0.1488647460937500, -0.0256042480468750,
+0.1511535644531250, -0.0042724609375000, 0.1187438964843750, -0.1103515625000000,
+0.1132507324218750, -0.0816040039062500, 0.1348876953125000, -0.1268005371093750,
+0.1313781738281250, -0.2378234863281250, 0.1167907714843750, -0.1826477050781250,
+0.1878662109375000, -0.2218933105468750, 0.1793823242187500, -0.2705688476562500,
+0.1853332519531250, -0.1955566406250000, 0.2313842773437500, -0.1417541503906250,
+0.2180480957031250, -0.1957702636718750, 0.2300109863281250, -0.0458984375000000,
+0.2204284667968750, -0.0397949218750000, 0.2609863281250000, -0.0486145019531250,
+0.2786865234375000, 0.0198364257812500, 0.2444458007812500, 0.0072326660156250,
+0.4303894042968750, -0.0078125000000000, 0.3760986328125000, -0.0068664550781250,
+0.4510498046875000, 0.0105285644531250, 0.6351318359375000, 0.0066223144531250,
+0.5361328125000000, -0.0062866210937500, 0.6375427246093750, 0.0742187500000000,
+0.7049255371093750, 0.0633239746093750, 0.5873718261718750, 0.0714721679687500,
+0.5516662597656250, 0.1343688964843750, 0.6246337890625000, 0.1157226562500000,
+0.2774658203125000, 0.0798645019531250, 0.3533020019531250, 0.1117553710937500,
+0.2402648925781250, 0.0683898925781250, -0.0520629882812500, 0.0004272460937500,
+0.0974731445312500, 0.0395507812500000, -0.1437072753906250, -0.0249023437500000,
+-0.1803283691406250, -0.0402221679687500, -0.1431884765625000, -0.0237731933593750,
+-0.2233276367187500, -0.0370788574218750, -0.2166748046875000, -0.0404663085937500,
+-0.1895446777343750, -0.0143432617187500, -0.1892395019531250, -0.0167236328125000,
+-0.2099304199218750, -0.0143127441406250, -0.1995239257812500, 0.0020446777343750,
+-0.1872253417968750, -0.0025634765625000, -0.2919616699218750, -0.0003967285156250,
+-0.2531433105468750, 0.0015869140625000, -0.3067016601562500, 0.0004577636718750,
+-0.4155883789062500, -0.0003662109375000, -0.3504028320312500, -0.0001525878906250,
+-0.4616699218750000, 0.0000610351562500, -0.4792480468750000, -0.0000610351562500,
+-0.4532775878906250, -0.0000610351562500, -0.4932556152343750, 0.0001525878906250,
+-0.4952087402343750, 0.0001525878906250, -0.4126892089843750, 0.0000305175781250,
+-0.4499816894531250, 0.0000305175781250, -0.3914489746093750, 0.0000305175781250,
+-0.2987670898437500, 0.0000305175781250, -0.3596496582031250, 0.0000305175781250,
+-0.2041015625000000, -0.0000305175781250, -0.2167968750000000, 0.0000305175781250,
+-0.1918029785156250, 0.0000000000000000, -0.0908203125000000, -0.0000305175781250,
+-0.1401672363281250, -0.0000610351562500, -0.0711364746093750, 0.0000610351562500,
+-0.0613708496093750, 0.0000915527343750, -0.0688171386718750, 0.0000610351562500,
+-0.0421447753906250, 0.0001220703125000, -0.0468750000000000, 0.0000610351562500,
+-0.0478515625000000, 0.0000000000000000, -0.0443420410156250, -0.0000305175781250,
+-0.0451965332031250, 0.0000305175781250, -0.0476684570312500, 0.0000000000000000,
+-0.0444335937500000, 0.0000000000000000, -0.0399475097656250, -0.0000305175781250,
+-0.0429687500000000, 0.0000000000000000, -0.0357360839843750, -0.0000305175781250,
+-0.0278930664062500, -0.0000915527343750, -0.0333251953125000, -0.0000305175781250,
+-0.0168151855468750, -0.0000610351562500, -0.0190429687500000, -0.0000915527343750,
+-0.0154113769531250, -0.0000610351562500, -0.0037536621093750, -0.0000610351562500,
+-0.0100708007812500, -0.0000610351562500, -0.0007019042968750, -0.0000610351562500,
+-0.0007324218750000, -0.0000610351562500, -0.0021972656250000, -0.0000915527343750,
+0.0015563964843750, -0.0001525878906250, -0.0010070800781250, -0.0001220703125000,
+-0.0039367675781250, -0.0001220703125000, -0.0035400390625000, -0.0001525878906250,
+-0.0057067871093750, -0.0001220703125000, -0.0093994140625000, -0.0001220703125000,
+-0.0094909667968750, -0.0001220703125000, -0.0119934082031250, -0.0000915527343750,
+-0.0120849609375000, -0.0001220703125000, -0.0134277343750000, -0.0001220703125000,
+-0.0152587890625000, -0.0001220703125000, -0.0148925781250000, -0.0001220703125000,
+-0.0201110839843750, -0.0000915527343750, -0.0189514160156250, -0.0000915527343750,
+-0.0239562988281250, -0.0001220703125000, -0.0294189453125000, -0.0000915527343750,
+-0.0256652832031250, -0.0000915527343750, -0.0409240722656250, -0.0001220703125000,
+-0.0380249023437500, -0.0001220703125000, -0.0432434082031250, -0.0000915527343750,
+-0.0556640625000000, -0.0001220703125000, -0.0487365722656250, -0.0001220703125000,
+-0.0612792968750000, -0.0001220703125000, -0.0621032714843750, -0.0000610351562500,
+-0.0604858398437500, -0.0000610351562500, -0.0659179687500000, -0.0000610351562500,
+-0.0627136230468750, -0.0001220703125000, -0.0533142089843750, -0.0000610351562500,
+-0.0582275390625000, -0.0000610351562500, -0.0474548339843750, -0.0000610351562500,
+-0.0333251953125000, 0.0000000000000000, -0.0396423339843750, -0.0000610351562500,
+-0.0124816894531250, -0.0000610351562500, -0.0153808593750000, 0.0000000000000000,
+-0.0063781738281250, -0.0000305175781250, 0.0157470703125000, -0.0000305175781250,
+0.0072021484375000, -0.0000305175781250, 0.0266113281250000, 0.0000000000000000,
+0.0281372070312500, 0.0000000000000000, 0.0305480957031250, -0.0000305175781250,
+0.0403137207031250},
+{-0.4284362792968750, -0.0089111328125000, -0.4179382324218750, 0.0020446777343750,
+-0.4476928710937500, -0.0005187988281250, -0.3241882324218750, -0.0004272460937500,
+-0.3546142578125000, 0.0006408691406250, -0.3118591308593750, 0.0002441406250000,
+-0.2063293457031250, -0.0003662109375000, -0.2622375488281250, -0.0003967285156250,
+-0.1539916992187500, 0.0000305175781250, -0.1499938964843750, 0.0000000000000000,
+-0.1481018066406250, -0.0000610351562500, -0.0926208496093750, 0.0000915527343750,
+-0.1123046875000000, 0.0001220703125000, -0.0971984863281250, 0.0000000000000000,
+-0.0897216796875000, -0.0000305175781250, -0.0926208496093750, 0.0000000000000000,
+-0.0932006835937500, 0.0000610351562500, -0.0921020507812500, 0.0000305175781250,
+-0.0765380859375000, 0.0000000000000000, -0.0885009765625000, 0.0000000000000000,
+-0.0660705566406250, 0.0000305175781250, -0.0409545898437500, 0.0000000000000000,
+-0.0622558593750000, -0.0000610351562500, 0.0013732910156250, 0.0000305175781250,
+-0.0100708007812500, 0.0000610351562500, 0.0056762695312500, -0.0000305175781250,
+0.0528564453125000, 0.0000000000000000, 0.0253601074218750, 0.0000305175781250,
+0.0670776367187500, 0.0000000000000000, 0.0708312988281250, -0.0000915527343750,
+0.0647583007812500, -0.0000305175781250, 0.0810852050781250, 0.0000305175781250,
+0.0761718750000000, -0.0000305175781250, 0.0657043457031250, 0.0000000000000000,
+0.0739135742187500, 0.0000305175781250, 0.0604248046875000, 0.0000000000000000,
+0.0441589355468750, -0.0000610351562500, 0.0567932128906250, -0.0000305175781250,
+0.0226745605468750, 0.0000305175781250, 0.0304870605468750, 0.0000000000000000,
+0.0206298828125000, -0.0000305175781250, -0.0049133300781250, 0.0000305175781250,
+0.0127258300781250, 0.0000305175781250, -0.0188598632812500, 0.0000000000000000,
+-0.0131225585937500, -0.0000305175781250, -0.0191040039062500, 0.0000000000000000,
+-0.0399475097656250, 0.0000000000000000, -0.0247192382812500, 0.0000000000000000,
+-0.0505981445312500, 0.0000000000000000, -0.0459289550781250, 0.0000305175781250,
+-0.0485839843750000, -0.0000305175781250, -0.0654907226562500, -0.0000305175781250,
+-0.0545349121093750, 0.0000000000000000, -0.0672607421875000, -0.0000610351562500,
+-0.0651245117187500, -0.0000610351562500, -0.0644836425781250, -0.0000610351562500,
+-0.0702209472656250, 0.0000000000000000, -0.0628662109375000, -0.0000915527343750,
+-0.0672607421875000, 0.0000000000000000, -0.0654296875000000, 0.0000305175781250,
+-0.0637817382812500, 0.0000000000000000, -0.0653381347656250, 0.0000000000000000,
+-0.0631713867187500, 0.0000000000000000, -0.0671997070312500, 0.0000305175781250,
+-0.0643005371093750, 0.0000000000000000, -0.0691223144531250, 0.0000000000000000,
+-0.0747375488281250, 0.0000305175781250, -0.0695495605468750, 0.0000305175781250,
+-0.0869445800781250, 0.0000305175781250, -0.0840454101562500, 0.0000610351562500,
+-0.0907897949218750, 0.0000610351562500, -0.1058654785156250, 0.0000305175781250,
+-0.0990905761718750, 0.0000000000000000, -0.1153869628906250, 0.0000000000000000,
+-0.1176147460937500, 0.0000305175781250, -0.1156311035156250, 0.0000305175781250,
+-0.1223754882812500, 0.0000305175781250, -0.1203308105468750, 0.0000000000000000,
+-0.1077575683593750, 0.0000305175781250, -0.1142883300781250, 0.0000610351562500,
+-0.0983276367187500, 0.0000305175781250, -0.0786132812500000, 0.0000000000000000,
+-0.0885620117187500, -0.0000305175781250, -0.0468139648437500, 0.0000305175781250,
+-0.0490112304687500, 0.0000305175781250, -0.0349731445312500, -0.0000305175781250,
+-0.0036010742187500, -0.0000915527343750, -0.0161132812500000, -0.0000305175781250,
+0.0173645019531250, -0.0001220703125000, 0.0168457031250000, -0.0003051757812500,
+0.0245666503906250, 0.0000915527343750, 0.0445556640625000, 0.0004882812500000,
+0.0346374511718750, 0.0000305175781250, 0.0657653808593750, -0.0002746582031250,
+0.0618286132812500, 0.0015258789062500, 0.0734252929687500, -0.0030212402343750,
+0.0992126464843750, -0.0057067871093750, 0.0867919921875000, -0.0018615722656250,
+0.1148986816406250, -0.0175170898437500, 0.1179809570312500, -0.0151367187500000,
+0.1143493652343750, -0.0147705078125000, 0.1264953613281250, -0.0277709960937500,
+0.1250915527343750, -0.0242309570312500, 0.1107482910156250, -0.0043029785156250,
+0.1161804199218750, -0.0205383300781250, 0.1094970703125000, 0.0054626464843750,
+0.0925903320312500, 0.0407409667968750, 0.0991516113281250, 0.0143432617187500,
+0.0965881347656250, 0.0664367675781250, 0.0877685546875000, 0.0748596191406250,
+0.1016845703125000, 0.0569152832031250, 0.1137084960937500, 0.0722045898437500,
+0.1024475097656250, 0.0784301757812500, 0.1315917968750000, 0.0248413085937500,
+0.1315612792968750, 0.0374145507812500, 0.1323547363281250, 0.0269775390625000,
+0.1454772949218750, -0.0170593261718750, 0.1408386230468750, -0.0008850097656250,
+0.1562194824218750, -0.0112609863281250, 0.1452331542968750, -0.0161743164062500,
+0.1701354980468750, -0.0207824707031250, 0.1966857910156250, -0.0205993652343750,
+0.1739501953125000, -0.0149230957031250, 0.2472534179687500, -0.0563049316406250,
+0.2499084472656250, -0.0459594726562500, 0.2356262207031250, -0.0545349121093750,
+0.2765502929687500, -0.0935668945312500, 0.2743835449218750, -0.0793457031250000,
+0.1729736328125000, -0.0654907226562500, 0.2206726074218750, -0.0832214355468750,
+0.1558227539062500, -0.0644531250000000, -0.0025634765625000, -0.0231323242187500,
+0.0102844238281250, -0.0303649902343750, -0.0726623535156250, -0.0321044921875000,
+-0.0746154785156250, -0.0218811035156250, -0.0479431152343750, -0.0294494628906250,
+-0.0764160156250000, -0.0465393066406250, -0.0862121582031250, -0.0365905761718750,
+-0.0416564941406250, -0.0171508789062500, -0.0390625000000000, -0.0461730957031250,
+-0.0533447265625000, 0.0100708007812500, -0.0390930175781250, 0.0686950683593750,
+-0.0319213867187500, 0.0231018066406250, -0.0705566406250000, 0.1616821289062500,
+-0.0685119628906250, 0.1582031250000000, -0.0616455078125000, 0.1694030761718750,
+-0.0813293457031250, 0.2538452148437500, -0.0816345214843750, 0.2272949218750000,
+-0.0541076660156250, 0.2390136718750000, -0.0596313476562500, 0.2600402832031250,
+-0.0571289062500000, 0.2301025390625000, -0.0383605957031250, 0.2093811035156250,
+-0.0430603027343750, 0.2283935546875000, -0.0374755859375000, 0.1506652832031250,
+-0.0439453125000000, 0.1637878417968750, -0.0226745605468750, 0.1405944824218750,
+-0.0086059570312500, 0.0762939453125000, -0.0272216796875000, 0.1072998046875000,
+0.0475463867187500, 0.0698547363281250, 0.0377197265625000, 0.0520935058593750,
+0.0480957031250000, 0.0787963867187500, 0.1063537597656250, 0.0848388671875000,
+0.0863037109375000, 0.0695190429687500, 0.0729370117187500, 0.1255493164062500,
+0.0993652343750000, 0.1257629394531250, 0.0640258789062500, 0.1188049316406250,
+0.0191345214843750, 0.1476440429687500, 0.0502014160156250, 0.1437683105468750,
+-0.0123291015625000, 0.1031799316406250, -0.0181274414062500, 0.1193542480468750,
+-0.0088806152343750, 0.0956115722656250, -0.0330810546875000, 0.0476989746093750,
+-0.0309143066406250, 0.0701293945312500, -0.0140686035156250, 0.0359497070312500,
+-0.0200195312500000, 0.0240783691406250, -0.0154113769531250, 0.0365295410156250,
+-0.0004272460937500, 0.0345458984375000, -0.0067749023437500, 0.0284118652343750,
+-0.0007934570312500, 0.0454101562500000, 0.0016174316406250, 0.0496520996093750,
+-0.0002441406250000, 0.0361022949218750, 0.0001220703125000, 0.0372314453125000,
+0.0005798339843750, 0.0419616699218750, -0.0000610351562500, -0.0121154785156250,
+-0.0004272460937500, -0.0014343261718750, -0.0001220703125000, -0.0333862304687500,
+0.0001220703125000, -0.0892333984375000, -0.0001525878906250, -0.0657348632812500,
+0.0000915527343750, -0.1308898925781250, 0.0001831054687500, -0.1382446289062500,
+0.0001220703125000, -0.1371154785156250, 0.0000305175781250, -0.1678466796875000,
+0.0000610351562500, -0.1620178222656250, 0.0000610351562500, -0.1500549316406250,
+0.0000305175781250, -0.1612243652343750, 0.0000305175781250, -0.1440429687500000,
+0.0001525878906250, -0.1209411621093750, 0.0001220703125000, -0.1335144042968750,
+0.0000610351562500, -0.0931091308593750, 0.0000610351562500, -0.0937805175781250,
+0.0001220703125000, -0.0881347656250000, 0.0000610351562500, -0.0631103515625000,
+0.0000305175781250, -0.0721740722656250, 0.0000915527343750, -0.0600891113281250,
+0.0000610351562500, -0.0560607910156250, 0.0000610351562500, -0.0603332519531250,
+0.0001220703125000, -0.0596923828125000, 0.0000915527343750, -0.0598449707031250,
+0.0000305175781250, -0.0684204101562500, 0.0000610351562500, -0.0700073242187500,
+0.0000610351562500, -0.0750122070312500, 0.0000915527343750, -0.0831909179687500,
+0.0000610351562500, -0.0821228027343750, 0.0001220703125000, -0.0849609375000000,
+0.0001525878906250, -0.0895996093750000, 0.0001220703125000, -0.0876770019531250,
+0.0001525878906250, -0.0865173339843750, 0.0001220703125000, -0.0901184082031250,
+0.0000610351562500, -0.0795288085937500, 0.0000610351562500, -0.0855712890625000,
+0.0000610351562500, -0.0792541503906250, 0.0000610351562500, -0.0677490234375000,
+0.0000610351562500, -0.0767517089843750, 0.0000610351562500, -0.0553283691406250,
+0.0000610351562500, -0.0595703125000000, 0.0000610351562500, -0.0532836914062500,
+0.0000610351562500, -0.0366210937500000, 0.0000610351562500, -0.0456237792968750,
+0.0000305175781250, -0.0241394042968750, 0.0000305175781250, -0.0275573730468750,
+0.0000305175781250, -0.0194396972656250, 0.0000305175781250, -0.0030212402343750,
+0.0000000000000000, -0.0139160156250000, 0.0000305175781250, 0.0130920410156250,
+0.0000305175781250, 0.0105590820312500, 0.0000000000000000, 0.0168151855468750,
+0.0000000000000000, 0.0362854003906250, 0.0000305175781250, 0.0267333984375000,
+0.0000000000000000, 0.0415344238281250, -0.0000305175781250, 0.0457153320312500,
+-0.0000305175781250, 0.0378112792968750, 0.0000305175781250, 0.0422363281250000,
+0.0000000000000000, 0.0470886230468750, 0.0000305175781250, 0.0170593261718750,
+0.0000305175781250, 0.0307006835937500, -0.0000305175781250, 0.0098266601562500,
+-0.0000610351562500, -0.0206909179687500, -0.0000305175781250, 0.0050354003906250,
+-0.0000915527343750, -0.0606384277343750, -0.0001220703125000, -0.0492248535156250,
+-0.0000610351562500, -0.0693664550781250, -0.0000610351562500, -0.1198730468750000,
+-0.0001220703125000, -0.0923156738281250, -0.0000305175781250, -0.1556396484375000,
+0.0000305175781250, -0.1515808105468750, -0.0000305175781250, -0.1555786132812500,
+-0.0000915527343750, -0.1962890625000000, -0.0000305175781250, -0.1804199218750000,
+-0.0001831054687500, -0.1807556152343750, -0.0003051757812500, -0.1957397460937500,
+-0.0000610351562500, -0.1750183105468750, 0.0003356933593750, -0.1539611816406250,
+-0.0000915527343750, -0.1712036132812500, -0.0011291503906250, -0.1334838867187500,
+0.0012207031250000, -0.1273803710937500, -0.0045166015625000, -0.1427612304687500,
+-0.0086975097656250, -0.1282043457031250, -0.0037536621093750, -0.1232299804687500,
+-0.0218505859375000, -0.1958618164062500, -0.0213623046875000, -0.1654663085937500,
+-0.0180664062500000, -0.2230529785156250, -0.0310058593750000, -0.3142089843750000,
+-0.0296936035156250, -0.2606811523437500, 0.0056152343750000, -0.3840942382812500,
+-0.0145874023437500, -0.3972778320312500, 0.0166015625000000, -0.3778076171875000,
+0.0679626464843750, -0.4322509765625000, 0.0368041992187500, -0.4298095703125000,
+0.0930480957031250},
+{0.0000305175781250, -0.1015930175781250, 0.0000305175781250, -0.1217346191406250,
+0.0000305175781250, -0.1207580566406250, 0.0000305175781250, -0.1257629394531250,
+0.0000610351562500, -0.1407470703125000, 0.0000000000000000, -0.1361999511718750,
+0.0000305175781250, -0.1431579589843750, 0.0000000000000000, -0.1467590332031250,
+0.0000000000000000, -0.1415405273437500, 0.0000000000000000, -0.1405944824218750,
+0.0000000000000000, -0.1427612304687500, 0.0000915527343750, -0.1254577636718750,
+0.0000610351562500, -0.1278991699218750, 0.0000915527343750, -0.1177978515625000,
+0.0000915527343750, -0.1007995605468750, 0.0000915527343750, -0.1071166992187500,
+0.0000915527343750, -0.0836486816406250, 0.0000915527343750, -0.0821228027343750,
+0.0000915527343750, -0.0738525390625000, 0.0000610351562500, -0.0574340820312500,
+0.0000305175781250, -0.0635070800781250, 0.0000610351562500, -0.0454406738281250,
+0.0000915527343750, -0.0404663085937500, 0.0000610351562500, -0.0373229980468750,
+0.0000915527343750, -0.0287170410156250, 0.0001220703125000, -0.0286560058593750,
+0.0000305175781250, -0.0205078125000000, 0.0000610351562500, -0.0201416015625000,
+0.0000610351562500, -0.0164794921875000, 0.0000305175781250, -0.0104064941406250,
+0.0000610351562500, -0.0131530761718750, 0.0000610351562500, -0.0059204101562500,
+0.0000305175781250, -0.0032958984375000, 0.0000305175781250, -0.0036010742187500,
+0.0000915527343750, -0.0012512207031250, 0.0000915527343750, -0.0007019042968750,
+0.0000000000000000, -0.0035095214843750, 0.0000000000000000, -0.0010986328125000,
+0.0000000000000000, 0.0003356933593750, 0.0000610351562500, -0.0013122558593750,
+0.0000000000000000, 0.0012817382812500, 0.0000610351562500, 0.0061645507812500,
+0.0000915527343750, 0.0044250488281250, 0.0000305175781250, 0.0101318359375000,
+0.0000610351562500, 0.0176086425781250, 0.0000610351562500, 0.0140380859375000,
+-0.0000305175781250, 0.0229187011718750, 0.0000000000000000, 0.0262451171875000,
+0.0000000000000000, 0.0223388671875000, -0.0000305175781250, 0.0237121582031250,
+-0.0000305175781250, 0.0271301269531250, 0.0000305175781250, 0.0149230957031250,
+0.0000305175781250, 0.0193481445312500, 0.0000305175781250, 0.0161743164062500,
+0.0000305175781250, 0.0068969726562500, 0.0000610351562500, 0.0123596191406250,
+-0.0001525878906250, -0.0040283203125000, -0.0001525878906250, 0.0036621093750000,
+0.0000610351562500, -0.0160522460937500, 0.0000610351562500, -0.0349426269531250,
+-0.0001525878906250, -0.0156555175781250, -0.0002441406250000, -0.0872802734375000,
+0.0010986328125000, -0.0701293945312500, -0.0025939941406250, -0.1009216308593750,
+-0.0043640136718750, -0.1661376953125000, -0.0015563964843750, -0.1295471191406250,
+-0.0165100097656250, -0.2010498046875000, -0.0140075683593750, -0.2058410644531250,
+-0.0149536132812500, -0.1995849609375000, -0.0289306640625000, -0.2328796386718750,
+-0.0245056152343750, -0.2264404296875000, -0.0042724609375000, -0.2083129882812500,
+-0.0221252441406250, -0.2154541015625000, 0.0086669921875000, -0.2066650390625000,
+0.0492248535156250, -0.1817321777343750, 0.0205078125000000, -0.1904296875000000,
+0.0787353515625000, -0.2085266113281250, 0.0906066894531250, -0.1816101074218750,
+0.0643310546875000, -0.2269897460937500, 0.0782470703125000, -0.2780761718750000,
+0.0888977050781250, -0.2413635253906250, 0.0001831054687500, -0.3574523925781250,
+0.0184326171875000, -0.3595275878906250, 0.0000000000000000, -0.3544311523437500,
+-0.0780029296875000, -0.4187622070312500, -0.0495605468750000, -0.4061889648437500,
+-0.0330200195312500, -0.3527832031250000, -0.0698242187500000, -0.3901977539062500,
+-0.0226440429687500, -0.3392944335937500, 0.0447692871093750, -0.2561950683593750,
+0.0023193359375000, -0.3047180175781250, 0.0507812500000000, -0.2121276855468750,
+0.0902099609375000, -0.2005615234375000, 0.0186157226562500, -0.2124023437500000,
+-0.0171203613281250, -0.1844177246093750, 0.0335998535156250, -0.1886901855468750,
+-0.1434631347656250, -0.1760864257812500, -0.1351318359375000, -0.1986694335937500,
+-0.1335449218750000, -0.1411132812500000, -0.2425537109375000, -0.1025390625000000,
+-0.2151794433593750, -0.1486816406250000, -0.1432800292968750, 0.0677795410156250,
+-0.2011413574218750, 0.0163269042968750, -0.1225280761718750, 0.1138305664062500,
+0.0023193359375000, 0.3174133300781250, -0.0702514648437500, 0.2093200683593750,
+0.0594787597656250, 0.4132385253906250, 0.0825805664062500, 0.4404296875000000,
+0.0595092773437500, 0.4037170410156250, 0.0953979492187500, 0.4764099121093750,
+0.0968017578125000, 0.4725646972656250, 0.0812683105468750, 0.3892822265625000,
+0.0748291015625000, 0.4141845703125000, 0.0931396484375000, 0.3951416015625000,
+0.1153564453125000, 0.2831420898437500, 0.1792297363281250, 0.4488220214843750,
+0.2096862792968750, 0.3499450683593750, 0.1744995117187500, 0.4035644531250000,
+0.2399902343750000, 0.3583068847656250, 0.2838134765625000, 0.2793273925781250,
+0.2338562011718750, 0.3116149902343750, 0.4234924316406250, 0.2590332031250000,
+0.3952636718750000, 0.2630615234375000, 0.4280090332031250, 0.2352905273437500,
+0.5784606933593750, 0.1968383789062500, 0.5160217285156250, 0.2259826660156250,
+0.5041198730468750, 0.1121215820312500, 0.5878906250000000, 0.1304321289062500,
+0.4498901367187500, 0.1025390625000000, 0.3240356445312500, 0.0140075683593750,
+0.4326477050781250, 0.0520935058593750, 0.0811157226562500, -0.0076599121093750,
+0.1235961914062500, -0.0093994140625000, 0.0539245605468750, -0.0265502929687500,
+-0.1949768066406250, -0.0601196289062500, -0.0804443359375000, -0.0372009277343750,
+-0.2401123046875000, -0.1206970214843750, -0.2733764648437500, -0.1113281250000000,
+-0.2471313476562500, -0.1193237304687500, -0.2943420410156250, -0.1838684082031250,
+-0.2917175292968750, -0.1635437011718750, -0.2751770019531250, -0.1295471191406250,
+-0.2772827148437500, -0.1639404296875000, -0.2879333496093750, -0.1140136718750000,
+-0.2801818847656250, -0.0408325195312500, -0.2750244140625000, -0.0848693847656250,
+-0.3259887695312500, -0.0106201171875000, -0.3076477050781250, 0.0067443847656250,
+-0.3344726562500000, -0.0104370117187500, -0.3896179199218750, 0.0016479492187500,
+-0.3595886230468750, 0.0059509277343750, -0.4084777832031250, 0.0048828125000000,
+-0.4218750000000000, -0.0083007812500000, -0.3983459472656250, 0.0167846679687500,
+-0.4069824218750000, 0.0415344238281250, -0.4166259765625000, 0.0215454101562500,
+-0.3434448242187500, 0.0704040527343750, -0.3672790527343750, 0.0805053710937500,
+-0.3300781250000000, 0.0603332519531250, -0.2585449218750000, 0.0705261230468750,
+-0.2998657226562500, 0.0794982910156250, -0.2064208984375000, 0.0194091796875000,
+-0.2117309570312500, 0.0280151367187500, -0.1974182128906250, 0.0169067382812500,
+-0.1401367187500000, -0.0288696289062500, -0.1680297851562500, -0.0127868652343750,
+-0.1141967773437500, -0.0194091796875000, -0.1177673339843750, -0.0303955078125000,
+-0.1044311523437500, -0.0192871093750000, -0.0700988769531250, -0.0079345703125000,
+-0.0892028808593750, -0.0149230957031250, -0.0301208496093750, -0.0026550292968750,
+-0.0413818359375000, -0.0000305175781250, -0.0174865722656250, -0.0021972656250000,
+0.0337524414062500, 0.0006713867187500, 0.0045166015625000, 0.0005798339843750,
+0.0610961914062500, -0.0001525878906250, 0.0683288574218750, -0.0003051757812500,
+0.0568847656250000, -0.0000915527343750, 0.0789184570312500, 0.0000305175781250,
+0.0794372558593750, -0.0000915527343750, 0.0436706542968750, 0.0000000000000000,
+0.0604858398437500, 0.0001220703125000, 0.0411987304687500, 0.0000610351562500,
+0.0021362304687500, 0.0000000000000000, 0.0265502929687500, 0.0000000000000000,
+-0.0145568847656250, 0.0000000000000000, -0.0180664062500000, 0.0000305175781250,
+-0.0129699707031250, 0.0000000000000000, -0.0303649902343750, -0.0000305175781250,
+-0.0279846191406250, 0.0000000000000000, -0.0206604003906250, 0.0000000000000000,
+-0.0268249511718750, -0.0000610351562500, -0.0217590332031250, 0.0000000000000000,
+-0.0130310058593750, 0.0000610351562500, -0.0206298828125000, 0.0000305175781250,
+-0.0065917968750000, 0.0000000000000000, -0.0091857910156250, 0.0000305175781250,
+-0.0075683593750000, 0.0000305175781250, 0.0020446777343750, -0.0000610351562500,
+-0.0034790039062500, -0.0000305175781250, 0.0031127929687500, 0.0000610351562500,
+0.0038146972656250, 0.0000915527343750, 0.0017089843750000, 0.0000305175781250,
+0.0041198730468750, 0.0000610351562500, 0.0046081542968750, 0.0000610351562500,
+0.0011596679687500, 0.0000000000000000, 0.0033874511718750, 0.0000305175781250,
+-0.0002746582031250, 0.0000610351562500, -0.0053100585937500, 0.0000305175781250,
+-0.0017089843750000, 0.0000000000000000, -0.0111694335937500, 0.0000610351562500,
+-0.0098266601562500, 0.0000610351562500, -0.0134582519531250, 0.0000305175781250,
+-0.0202636718750000, 0.0000000000000000, -0.0159606933593750, 0.0000610351562500,
+-0.0297241210937500, 0.0000305175781250, -0.0273437500000000, 0.0000305175781250,
+-0.0341186523437500, 0.0000305175781250, -0.0466308593750000, 0.0000610351562500,
+-0.0415344238281250, 0.0000305175781250, -0.0627746582031250, 0.0000305175781250,
+-0.0595703125000000, 0.0000915527343750, -0.0690307617187500, 0.0000305175781250,
+-0.0867309570312500, -0.0000305175781250, -0.0783386230468750, 0.0000305175781250,
+-0.1073303222656250, 0.0000000000000000, -0.1034545898437500, 0.0000000000000000,
+-0.1127929687500000, 0.0000000000000000, -0.1354064941406250, 0.0000000000000000,
+-0.1256103515625000, 0.0000305175781250, -0.1538391113281250, 0.0001220703125000,
+-0.1528930664062500, 0.0000610351562500, -0.1574096679687500, 0.0000915527343750,
+-0.1760559082031250, 0.0000610351562500, -0.1687316894531250, 0.0000305175781250,
+-0.1761474609375000, 0.0000610351562500, -0.1806030273437500, 0.0001525878906250,
+-0.1725158691406250, 0.0000305175781250, -0.1693420410156250, 0.0000000000000000,
+-0.1725158691406250, 0.0000915527343750, -0.1509094238281250, -0.0000305175781250,
+-0.1526794433593750, -0.0000915527343750, -0.1489257812500000, 0.0000305175781250,
+-0.1323547363281250, 0.0000915527343750, -0.1372375488281250, -0.0000610351562500,
+-0.1396484375000000, 0.0001831054687500, -0.1308593750000000, 0.0008239746093750,
+-0.1436157226562500, -0.0012512207031250, -0.1576232910156250, -0.0017089843750000,
+-0.1464843750000000, -0.0001525878906250, -0.1740722656250000, -0.0098571777343750,
+-0.1755371093750000, -0.0067749023437500, -0.1719665527343750, -0.0100402832031250,
+-0.1847534179687500, -0.0207824707031250, -0.1827087402343750, -0.0154113769531250,
+-0.1675109863281250, -0.0118408203125000, -0.1748962402343750, -0.0223083496093750,
+-0.1590270996093750, -0.0033874511718750, -0.1385803222656250, 0.0140380859375000,
+-0.1498413085937500, -0.0029602050781250, -0.1052246093750000, 0.0439453125000000,
+-0.1113586425781250, 0.0421752929687500, -0.0913085937500000, 0.0384521484375000,
+-0.0538940429687500, 0.0658264160156250, -0.0727844238281250, 0.0606384277343750,
+-0.0182800292968750, 0.0354309082031250, -0.0218200683593750, 0.0505676269531250,
+-0.0079040527343750, 0.0326843261718750, 0.0303039550781250, -0.0019226074218750,
+0.0138244628906250, 0.0176086425781250, 0.0532531738281250, -0.0069580078125000,
+0.0543518066406250, -0.0144042968750000, 0.0559997558593750, -0.0090942382812500,
+0.0778808593750000, -0.0103759765625000, 0.0706481933593750, -0.0118713378906250,
+0.0719909667968750, -0.0252380371093750, 0.0772094726562500, -0.0146179199218750,
+0.0725708007812500},
+{-0.0320129394531250, 0.0000915527343750, -0.0352783203125000, 0.0001220703125000,
+-0.0534973144531250, 0.0001220703125000, -0.0441894531250000, 0.0001220703125000,
+-0.0591735839843750, 0.0000915527343750, -0.0591125488281250, 0.0000610351562500,
+-0.0593566894531250, 0.0000915527343750, -0.0671081542968750, 0.0001220703125000,
+-0.0631408691406250, 0.0000915527343750, -0.0667114257812500, 0.0000915527343750,
+-0.0669555664062500, 0.0001831054687500, -0.0649414062500000, 0.0001831054687500,
+-0.0652770996093750, 0.0001220703125000, -0.0655517578125000, 0.0001831054687500,
+-0.0643920898437500, 0.0001525878906250, -0.0640869140625000, 0.0001220703125000,
+-0.0642700195312500, 0.0000610351562500, -0.0639038085937500, 0.0000915527343750,
+-0.0636901855468750, 0.0000610351562500, -0.0649108886718750, 0.0001220703125000,
+-0.0645751953125000, 0.0001220703125000, -0.0650024414062500, 0.0001220703125000,
+-0.0654907226562500, 0.0001220703125000, -0.0645446777343750, 0.0000915527343750,
+-0.0672302246093750, 0.0000915527343750, -0.0665588378906250, 0.0000610351562500,
+-0.0673522949218750, 0.0001220703125000, -0.0693969726562500, 0.0001525878906250,
+-0.0678710937500000, 0.0000915527343750, -0.0706481933593750, 0.0001220703125000,
+-0.0706176757812500, 0.0001525878906250, -0.0707397460937500, 0.0001220703125000,
+-0.0715332031250000, 0.0001220703125000, -0.0710449218750000, 0.0000915527343750,
+-0.0759582519531250, 0.0001220703125000, -0.0729064941406250, 0.0001220703125000,
+-0.0788879394531250, 0.0000305175781250, -0.0852661132812500, 0.0000305175781250,
+-0.0802307128906250, 0.0000305175781250, -0.1025085449218750, -0.0001831054687500,
+-0.0974731445312500, -0.0001831054687500, -0.1092834472656250, -0.0000915527343750,
+-0.1314086914062500, -0.0001525878906250, -0.1213989257812500, -0.0002136230468750,
+-0.1509094238281250, 0.0000915527343750, -0.1517944335937500, 0.0004577636718750,
+-0.1535644531250000, -0.0009155273437500, -0.1701049804687500, -0.0009460449218750,
+-0.1651000976562500, 0.0000000000000000, -0.1661071777343750, -0.0071716308593750,
+-0.1690673828125000, -0.0041809082031250, -0.1685485839843750, -0.0079345703125000,
+-0.1649169921875000, -0.0167236328125000, -0.1650695800781250, -0.0115051269531250,
+-0.1757507324218750, -0.0129089355468750, -0.1685485839843750, -0.0205688476562500,
+-0.1781005859375000, -0.0057373046875000, -0.1940612792968750, 0.0045471191406250,
+-0.1836853027343750, -0.0080871582031250, -0.2025146484375000, 0.0328979492187500,
+-0.2052612304687500, 0.0288085937500000, -0.2009582519531250, 0.0302429199218750,
+-0.2068786621093750, 0.0583496093750000, -0.2063903808593750, 0.0496826171875000,
+-0.1960754394531250, 0.0363464355468750, -0.2001953125000000, 0.0512390136718750,
+-0.1898803710937500, 0.0332641601562500, -0.1784667968750000, 0.0058898925781250,
+-0.1860961914062500, 0.0241088867187500, -0.1506042480468750, -0.0035705566406250,
+-0.1592102050781250, -0.0093383789062500, -0.1399536132812500, -0.0034484863281250,
+-0.1065979003906250, -0.0085449218750000, -0.1241149902343750, -0.0100708007812500,
+-0.0672302246093750, -0.0148925781250000, -0.0740661621093750, -0.0070495605468750,
+-0.0558471679687500, -0.0213623046875000, -0.0125427246093750, -0.0381164550781250,
+-0.0342407226562500, -0.0242919921875000, 0.0149536132812500, -0.0494079589843750,
+0.0157165527343750, -0.0558776855468750, 0.0209960937500000, -0.0440673828125000,
+0.0500183105468750, -0.0452880859375000, 0.0389709472656250, -0.0523681640625000,
+0.0505065917968750, -0.0357360839843750, 0.0546875000000000, -0.0294799804687500,
+0.0517272949218750, -0.0440673828125000, 0.0508117675781250, -0.0484619140625000,
+0.0515136718750000, -0.0360717773437500, 0.0620422363281250, -0.0592346191406250,
+0.0515441894531250, -0.0724182128906250, 0.0715637207031250, -0.0374450683593750,
+0.0924377441406250, -0.0295715332031250, 0.0749206542968750, -0.0535888671875000,
+0.1334838867187500, 0.0655212402343750, 0.1236267089843750, 0.0408020019531250,
+0.1412353515625000, 0.0756530761718750, 0.1935424804687500, 0.1927795410156250,
+0.1917724609375000, 0.1756286621093750, 0.2616577148437500, 0.2245178222656250,
+0.2360534667968750, 0.2233581542968750, 0.2457580566406250, 0.2126770019531250,
+0.3069763183593750, 0.2373657226562500, 0.2762756347656250, 0.2344360351562500,
+0.2791748046875000, 0.2033691406250000, 0.3201904296875000, 0.2171936035156250,
+0.2460632324218750, 0.1901550292968750, 0.1917419433593750, 0.1505737304687500,
+0.2543640136718750, 0.1741027832031250, 0.0468750000000000, 0.1136779785156250,
+0.0718688964843750, 0.1099243164062500, 0.0448303222656250, 0.1134643554687500,
+-0.1022033691406250, 0.0853271484375000, -0.0462036132812500, 0.0893859863281250,
+-0.0721130371093750, 0.0983276367187500, -0.1073303222656250, 0.0978393554687500,
+-0.0711059570312500, 0.0880126953125000, -0.0365295410156250, 0.0914001464843750,
+-0.0613708496093750, 0.0968322753906250, -0.0325012207031250, 0.0467834472656250,
+-0.0165405273437500, 0.0587158203125000, -0.0363769531250000, 0.0426330566406250,
+-0.0466003417968750, -0.0035400390625000, -0.0340576171875000, 0.0160217285156250,
+-0.0544128417968750, -0.0001525878906250, -0.0636901855468750, -0.0111389160156250,
+-0.0488586425781250, -0.0030517578125000, -0.0437927246093750, 0.0054321289062500,
+-0.0535583496093750, -0.0005187988281250, -0.0283813476562500, -0.0191345214843750,
+-0.0267028808593750, 0.0003356933593750, -0.0285949707031250, -0.0358276367187500,
+-0.0207519531250000, -0.0768127441406250, -0.0218200683593750, -0.0463562011718750,
+-0.0095825195312500, -0.1367492675781250, -0.0201416015625000, -0.1331787109375000,
+0.0003662109375000, -0.1414184570312500, 0.0244445800781250, -0.1973571777343750,
+0.0043029785156250, -0.1772460937500000, 0.0521240234375000, -0.1869812011718750,
+0.0543518066406250, -0.2007141113281250, 0.0444335937500000, -0.1811523437500000,
+0.0659179687500000, -0.1658020019531250, 0.0656738281250000, -0.1787414550781250,
+0.0298461914062500, -0.1466979980468750, 0.0450134277343750, -0.1437377929687500,
+0.0287475585937500, -0.1463623046875000, -0.0093078613281250, -0.1330871582031250,
+0.0116577148437500, -0.1353149414062500, -0.0144042968750000, -0.1416015625000000,
+-0.0212097167968750, -0.1399841308593750, -0.0123291015625000, -0.1400756835937500,
+-0.0168762207031250, -0.1462097167968750, -0.0198364257812500, -0.1452331542968750,
+-0.0067443847656250, -0.1326599121093750, -0.0083007812500000, -0.1387329101562500,
+-0.0074768066406250, -0.1253356933593750, 0.0005493164062500, -0.1066894531250000,
+-0.0021972656250000, -0.1163330078125000, -0.0005798339843750, -0.0869445800781250,
+0.0005493164062500, -0.0859985351562500, -0.0000610351562500, -0.0830078125000000,
+-0.0003051757812500, -0.0655517578125000, -0.0000915527343750, -0.0702209472656250,
+-0.0000305175781250, -0.0665893554687500, -0.0001831054687500, -0.0632324218750000,
+-0.0000915527343750, -0.0663146972656250, 0.0001525878906250, -0.0686035156250000,
+0.0000610351562500, -0.0673522949218750, 0.0001831054687500, -0.0730285644531250,
+0.0001831054687500, -0.0729980468750000, 0.0001831054687500, -0.0755310058593750,
+0.0002136230468750, -0.0803527832031250, 0.0002136230468750, -0.0788879394531250,
+0.0001831054687500, -0.0840454101562500, 0.0002136230468750, -0.0849609375000000,
+0.0001831054687500, -0.0841369628906250, 0.0001220703125000, -0.0857849121093750,
+0.0001831054687500, -0.0859069824218750, 0.0001525878906250, -0.0834045410156250,
+0.0000915527343750, -0.0841674804687500, 0.0000915527343750, -0.0816955566406250,
+0.0000915527343750, -0.0783691406250000, 0.0000915527343750, -0.0797424316406250,
+0.0000305175781250, -0.0736083984375000, 0.0000610351562500, -0.0728149414062500,
+0.0000610351562500, -0.0715637207031250, 0.0000610351562500, -0.0678405761718750,
+0.0000610351562500, -0.0682067871093750, 0.0000305175781250, -0.0666198730468750,
+0.0000305175781250, -0.0659790039062500, 0.0000610351562500, -0.0654602050781250,
+0.0000610351562500, -0.0646667480468750, 0.0000305175781250, -0.0646972656250000,
+0.0000610351562500, -0.0635070800781250, 0.0000610351562500, -0.0644836425781250,
+0.0000610351562500, -0.0628051757812500, 0.0000915527343750, -0.0612487792968750,
+0.0000915527343750, -0.0607604980468750, 0.0000000000000000, -0.0485229492187500,
+0.0000305175781250, -0.0541381835937500, 0.0000305175781250, -0.0420532226562500,
+0.0000305175781250, -0.0266113281250000, -0.0000305175781250, -0.0377502441406250,
+0.0000000000000000, -0.0057067871093750, 0.0000000000000000, -0.0106506347656250,
+0.0000000000000000, -0.0023803710937500, 0.0000610351562500, 0.0211181640625000,
+0.0000305175781250, 0.0097045898437500, 0.0000000000000000, 0.0368652343750000,
+0.0000305175781250, 0.0305786132812500, -0.0000305175781250, 0.0463562011718750,
+-0.0000915527343750, 0.0687866210937500, -0.0000915527343750, 0.0505065917968750,
+0.0001220703125000, 0.1111450195312500, 0.0002441406250000, 0.1013488769531250,
+-0.0005187988281250, 0.1140441894531250, -0.0000915527343750, 0.1635437011718750,
+0.0001525878906250, 0.1424560546875000, -0.0066833496093750, 0.1504211425781250,
+-0.0033264160156250, 0.1726074218750000, -0.0087890625000000, 0.1369934082031250,
+-0.0190429687500000, 0.1095886230468750, -0.0127258300781250, 0.1372070312500000,
+-0.0157470703125000, 0.0527648925781250, -0.0256958007812500, 0.0601806640625000,
+-0.0050964355468750, 0.0495300292968750, 0.0091552734375000, -0.0059814453125000,
+-0.0070190429687500, 0.0150146484375000, 0.0481567382812500, -0.0082092285156250,
+0.0466308593750000, -0.0133972167968750, 0.0412292480468750, -0.0120849609375000,
+0.0743713378906250, -0.0173645019531250, 0.0696105957031250, -0.0132141113281250,
+0.0244140625000000, -0.0297851562500000, 0.0467529296875000, -0.0192565917968750,
+0.0189208984375000, -0.0417480468750000, -0.0415039062500000, -0.0648803710937500,
+-0.0116271972656250, -0.0450744628906250, -0.0303344726562500, -0.1250610351562500,
+-0.0563354492187500, -0.1080932617187500, -0.0202331542968750, -0.1371459960937500,
+0.0115051269531250, -0.2084960937500000, -0.0157470703125000, -0.1752319335937500,
+0.0341491699218750, -0.2363586425781250, 0.0563354492187500, -0.2457275390625000,
+0.0114440917968750, -0.2362976074218750, 0.0029907226562500, -0.2564086914062500,
+0.0318603515625000, -0.2551879882812500, -0.0949096679687500, -0.2445373535156250,
+-0.0840759277343750, -0.2431335449218750, -0.0899047851562500, -0.2500610351562500,
+-0.1762390136718750, -0.2482299804687500, -0.1523132324218750, -0.2447509765625000,
+-0.1082763671875000, -0.2745666503906250, -0.1504516601562500, -0.2700500488281250,
+-0.0915222167968750, -0.2725219726562500, -0.0049438476562500, -0.2963256835937500,
+-0.0588684082031250, -0.2904663085937500, 0.0387573242187500, -0.2760314941406250,
+0.0536499023437500, -0.2923889160156250, 0.0399780273437500, -0.2688293457031250,
+0.0700988769531250, -0.2396545410156250, 0.0682983398437500, -0.2611694335937500,
+0.0685729980468750, -0.1998901367187500, 0.0611877441406250, -0.2068176269531250,
+0.0837707519531250, -0.1916503906250000, 0.1009521484375000, -0.1491088867187500,
+0.0831604003906250, -0.1697387695312500, 0.1427307128906250, -0.1200256347656250,
+0.1407775878906250, -0.1311645507812500, 0.1442871093750000, -0.0951232910156250,
+0.1755371093750000, -0.0542602539062500, 0.1655273437500000, -0.0878295898437500,
+0.1899108886718750, 0.0676574707031250, 0.1776428222656250, 0.0289916992187500,
+0.2082519531250000, 0.0807189941406250, 0.2831726074218750, 0.3150939941406250,
+0.4269409179687500},
+{-0.0000610351562500, -0.0066833496093750, -0.0000915527343750, -0.0082397460937500,
+-0.0000915527343750, -0.0084228515625000, -0.0000610351562500, -0.0087280273437500,
+-0.0000305175781250, -0.0069580078125000, -0.0000305175781250, -0.0075073242187500,
+-0.0000610351562500, -0.0091552734375000, 0.0000000000000000, 0.0037536621093750,
+0.0000915527343750, -0.0028381347656250, 0.0000305175781250, 0.0072631835937500,
+0.0000305175781250, 0.0242614746093750, 0.0000305175781250, 0.0131225585937500,
+0.0000610351562500, 0.0325622558593750, 0.0000305175781250, 0.0376586914062500,
+-0.0000610351562500, 0.0304260253906250, 0.0003967285156250, 0.0338745117187500,
+0.0002136230468750, 0.0382080078125000, -0.0032348632812500, 0.0223693847656250,
+-0.0006103515625000, 0.0196228027343750, -0.0057678222656250, 0.0325622558593750,
+-0.0125732421875000, 0.0282287597656250, -0.0075988769531250, 0.0217895507812500,
+-0.0164489746093750, 0.0802307128906250, -0.0219726562500000, 0.0603027343750000,
+-0.0087890625000000, 0.0881042480468750, -0.0054931640625000, 0.1505126953125000,
+-0.0144348144531250, 0.1177673339843750, 0.0318298339843750, 0.1575927734375000,
+0.0243530273437500, 0.1748352050781250, 0.0302429199218750, 0.1487426757812500,
+0.0685424804687500, 0.1437072753906250, 0.0556335449218750, 0.1566467285156250,
+0.0341186523437500, 0.1045837402343750, 0.0570678710937500, 0.1112976074218750,
+0.0263366699218750, 0.0979919433593750, -0.0214843750000000, 0.0609741210937500,
+0.0082702636718750, 0.0777893066406250, -0.0272521972656250, 0.0375366210937500,
+-0.0487976074218750, 0.0393066406250000, -0.0165100097656250, 0.0314025878906250,
+-0.0002136230468750, 0.0058593750000000, -0.0227050781250000, 0.0186157226562500,
+0.0275878906250000, -0.0070800781250000, 0.0407104492187500, -0.0026245117187500,
+0.0090942382812500, -0.0148620605468750, 0.0145568847656250, -0.0328674316406250,
+0.0313415527343750, -0.0201110839843750, -0.0708618164062500, -0.0822448730468750,
+-0.0558166503906250, -0.0644226074218750, -0.0687255859375000, -0.0978698730468750,
+-0.1482849121093750, -0.1586914062500000, -0.1222229003906250, -0.1221008300781250,
+-0.0950927734375000, -0.2061157226562500, -0.1321716308593750, -0.2056579589843750,
+-0.0820312500000000, -0.2085571289062500, -0.0124511718750000, -0.2552795410156250,
+-0.0551452636718750, -0.2394409179687500, 0.0296325683593750, -0.2492980957031250,
+0.0408020019531250, -0.2570800781250000, 0.0315856933593750, -0.2480468750000000,
+0.0601196289062500, -0.2431335449218750, 0.0563964843750000, -0.2482910156250000,
+0.0599975585937500, -0.2374572753906250, 0.0527648925781250, -0.2377624511718750,
+0.0724792480468750, -0.2374267578125000, 0.0895996093750000, -0.2317810058593750,
+0.0733032226562500, -0.2341308593750000, 0.1263427734375000, -0.2322998046875000,
+0.1228942871093750, -0.2356567382812500, 0.1276245117187500, -0.2278137207031250,
+0.1573791503906250, -0.2249450683593750, 0.1475219726562500, -0.2317810058593750,
+0.1755676269531250, -0.1867675781250000, 0.1591491699218750, -0.2017517089843750,
+0.2025146484375000, -0.1761169433593750, 0.2437133789062500, -0.1296081542968750,
+0.2056274414062500, -0.1569213867187500, 0.3550415039062500, -0.0964965820312500,
+0.3406982421875000, -0.1009216308593750, 0.3468322753906250, -0.0876464843750000,
+0.4879455566406250, -0.0166320800781250, 0.2804870605468750, -0.1459655761718750,
+-0.0443725585937500, -0.0640563964843750, 0.0705871582031250, -0.0933227539062500,
+-0.0208435058593750, -0.0542297363281250, -0.2004394531250000, -0.0153808593750000,
+-0.1227722167968750, -0.0430297851562500, -0.2338256835937500, 0.0939025878906250,
+-0.2531738281250000, 0.0529479980468750, -0.2400817871093750, 0.1249084472656250,
+-0.2808837890625000, 0.2701110839843750, -0.2713623046875000, 0.1908569335937500,
+-0.2500915527343750, 0.3189697265625000, -0.2668457031250000, 0.3504333496093750,
+-0.2476196289062500, 0.3070983886718750, -0.2112426757812500, 0.3298645019531250,
+-0.2314758300781250, 0.3446655273437500, -0.2162475585937500, 0.2617797851562500,
+-0.1929016113281250, 0.2711181640625000, -0.2364501953125000, 0.2572937011718750,
+-0.2623596191406250, 0.2043457031250000, -0.2268371582031250, 0.2240600585937500,
+-0.3414916992187500, 0.1788024902343750, -0.3336181640625000, 0.1894226074218750,
+-0.3349609375000000, 0.1587219238281250, -0.4097595214843750, 0.1131896972656250,
+-0.3888549804687500, 0.1441955566406250, -0.3445434570312500, 0.0537414550781250,
+-0.3872070312500000, 0.0574951171875000, -0.3265075683593750, 0.0462951660156250,
+-0.2442932128906250, -0.0075378417968750, -0.2998046875000000, 0.0130310058593750,
+-0.1638488769531250, -0.0263977050781250, -0.1662902832031250, -0.0207824707031250,
+-0.1558837890625000, -0.0411071777343750, -0.0801696777343750, -0.0769653320312500,
+-0.1076660156250000, -0.0529174804687500, -0.0589904785156250, -0.1027832031250000,
+-0.0626220703125000, -0.1130371093750000, -0.0401000976562500, -0.0936584472656250,
+-0.0038452148437500, -0.1064758300781250, -0.0299987792968750, -0.1141967773437500,
+0.0498962402343750, -0.0502624511718750, 0.0380859375000000, -0.0625915527343750,
+0.0546264648437500, -0.0474853515625000, 0.1125793457031250, 0.0059204101562500,
+0.0844421386718750, -0.0130004882812500, 0.1262817382812500, -0.0000305175781250,
+0.1325378417968750, 0.0079650878906250, 0.1235351562500000, 0.0062561035156250,
+0.1403503417968750, 0.0024719238281250, 0.1392211914062500, 0.0015258789062500,
+0.1076660156250000, 0.0287780761718750, 0.1262207031250000, 0.0200500488281250,
+0.1004333496093750, 0.0287780761718750, 0.0569763183593750, 0.0588989257812500,
+0.0845336914062500, 0.0477905273437500, 0.0350036621093750, 0.0390319824218750,
+0.0284423828125000, 0.0548706054687500, 0.0374755859375000, 0.0325622558593750,
+0.0209960937500000, 0.0024108886718750, 0.0214233398437500, 0.0207519531250000,
+0.0351867675781250, -0.0121154785156250, 0.0333251953125000, -0.0192871093750000,
+0.0321655273437500, -0.0104370117187500, 0.0401000976562500, -0.0172729492187500,
+0.0390014648437500, -0.0195312500000000, 0.0308227539062500, -0.0059204101562500,
+0.0346374511718750, -0.0066833496093750, 0.0308532714843750, -0.0063171386718750,
+0.0207519531250000, 0.0011901855468750, 0.0249938964843750, -0.0007324218750000,
+0.0241088867187500, -0.0002746582031250, 0.0204772949218750, 0.0004882812500000,
+0.0252685546875000, 0.0002136230468750, 0.0315246582031250, -0.0000610351562500,
+0.0285644531250000, -0.0000610351562500, 0.0314941406250000, 0.0000915527343750,
+0.0375976562500000, 0.0000915527343750, 0.0299072265625000, 0.0000305175781250,
+0.0268859863281250, 0.0000915527343750, 0.0365295410156250, 0.0000915527343750,
+0.0083923339843750, 0.0000305175781250, 0.0192871093750000, 0.0000915527343750,
+0.0095825195312500, 0.0000610351562500, -0.0150146484375000, 0.0000610351562500,
+0.0027160644531250, 0.0000610351562500, -0.0282897949218750, 0.0001220703125000,
+-0.0230407714843750, 0.0000305175781250, -0.0263977050781250, 0.0000610351562500,
+-0.0446777343750000, 0.0000610351562500, -0.0294494628906250, 0.0000305175781250,
+-0.0469055175781250, 0.0000915527343750, -0.0432739257812500, 0.0001220703125000,
+-0.0425109863281250, 0.0001220703125000, -0.0514221191406250, 0.0001220703125000,
+-0.0422363281250000, 0.0000915527343750, -0.0498046875000000, 0.0000915527343750,
+-0.0473022460937500, 0.0001220703125000, -0.0444946289062500, 0.0000915527343750,
+-0.0474548339843750, 0.0000915527343750, -0.0425109863281250, 0.0001220703125000,
+-0.0384826660156250, 0.0001220703125000, -0.0364379882812500, 0.0001220703125000,
+-0.0309143066406250, 0.0000610351562500, -0.0254516601562500, 0.0000305175781250,
+-0.0231018066406250, 0.0000610351562500, -0.0131530761718750, 0.0000305175781250,
+-0.0139465332031250, 0.0000305175781250, -0.0056152343750000, 0.0000000000000000,
+0.0051879882812500, 0.0000000000000000, 0.0014038085937500, 0.0000305175781250,
+0.0202026367187500, -0.0001220703125000, 0.0192565917968750, -0.0000610351562500,
+0.0222473144531250, -0.0000610351562500, 0.0357666015625000, -0.0001525878906250,
+0.0321655273437500, -0.0001525878906250, 0.0339355468750000, 0.0000610351562500,
+0.0388183593750000, 0.0000915527343750, 0.0306701660156250, -0.0003356933593750,
+0.0250549316406250, 0.0000610351562500, 0.0306396484375000, 0.0001831054687500,
+0.0060424804687500, -0.0033874511718750, 0.0111694335937500, -0.0012512207031250,
+-0.0007324218750000, -0.0046386718750000, -0.0239562988281250, -0.0098876953125000,
+-0.0136108398437500, -0.0059204101562500, -0.0428771972656250, -0.0114135742187500,
+-0.0428161621093750, -0.0147094726562500, -0.0468444824218750, -0.0071105957031250,
+-0.0644226074218750, -0.0057678222656250, -0.0584411621093750, -0.0111694335937500,
+-0.0719299316406250, 0.0155944824218750, -0.0725708007812500, 0.0084228515625000,
+-0.0732421875000000, 0.0158996582031250, -0.0809936523437500, 0.0404052734375000,
+-0.0778503417968750, 0.0285644531250000, -0.0756530761718750, 0.0347290039062500,
+-0.0799255371093750, 0.0449523925781250, -0.0737609863281250, 0.0304260253906250,
+-0.0644836425781250, 0.0191345214843750, -0.0699157714843750, 0.0309753417968750,
+-0.0675354003906250, 0.0039367675781250, -0.0582580566406250, 0.0027770996093750,
+-0.0796508789062500, 0.0052795410156250, -0.0942382812500000, -0.0059814453125000,
+-0.0802917480468750, -0.0044555664062500, -0.1448974609375000, -0.0052795410156250,
+-0.1387939453125000, -0.0022277832031250, -0.1529235839843750, -0.0114746093750000,
+-0.2002868652343750, -0.0177307128906250, -0.1819763183593750, -0.0092163085937500,
+-0.2021179199218750, -0.0360412597656250, -0.2097167968750000, -0.0343627929687500,
+-0.2033081054687500, -0.0335083007812500, -0.2048645019531250, -0.0491638183593750,
+-0.2058715820312500, -0.0458068847656250, -0.1986389160156250, -0.0396728515625000,
+-0.2003479003906250, -0.0418701171875000, -0.1944580078125000, -0.0447082519531250,
+-0.1877441406250000, -0.0402221679687500, -0.1917114257812500, -0.0380554199218750,
+-0.1691894531250000, -0.0536193847656250, -0.1762084960937500, -0.0545349121093750,
+-0.1592712402343750, -0.0420227050781250, -0.1352233886718750, -0.0507507324218750,
+-0.1499938964843750, -0.0563049316406250, -0.1005249023437500, 0.0174865722656250,
+-0.1086425781250000, -0.0131530761718750, -0.0915527343750000, 0.0324096679687500,
+-0.0523681640625000, 0.1140136718750000, -0.0725402832031250, 0.0619201660156250,
+-0.0226440429687500, 0.1635437011718750, -0.0252685546875000, 0.1689453125000000,
+-0.0164184570312500, 0.1694335937500000, 0.0160522460937500, 0.2198486328125000,
+0.0012207031250000, 0.2029418945312500, 0.0267333984375000, 0.2097473144531250,
+0.0275878906250000, 0.2224731445312500, 0.0302124023437500, 0.2022399902343750,
+0.0434875488281250, 0.1837158203125000, 0.0429687500000000, 0.1853332519531250,
+0.0733032226562500, 0.1293945312500000, 0.0573730468750000, 0.1478576660156250,
+0.0725097656250000, 0.1401062011718750, 0.1020507812500000, 0.0932312011718750,
+0.0808715820312500, 0.1100769042968750, 0.1386413574218750, 0.0938720703125000,
+0.1264343261718750, 0.0858764648437500, 0.1499938964843750, 0.0874633789062500,
+0.1962585449218750, 0.0892944335937500, 0.1658325195312500, 0.0884704589843750,
+0.2456665039062500, 0.0612182617187500, 0.2401428222656250, 0.0740356445312500,
+0.2419128417968750, 0.0517883300781250, 0.2987976074218750, 0.0139160156250000,
+0.2802429199218750, 0.0338134765625000, 0.2299499511718750, -0.0025634765625000,
+0.2796630859375000},
+{-0.0432434082031250, -0.0022888183593750, -0.0473937988281250, -0.0005798339843750,
+-0.0372924804687500, -0.0035400390625000, -0.0195922851562500, -0.0074768066406250,
+-0.0276184082031250, -0.0042419433593750, -0.0001831054687500, -0.0101623535156250,
+-0.0009460449218750, -0.0120239257812500, 0.0059509277343750, -0.0069580078125000,
+0.0264282226562500, -0.0079345703125000, 0.0195617675781250, -0.0110168457031250,
+0.0300598144531250, 0.0093688964843750, 0.0356750488281250, 0.0025329589843750,
+0.0284118652343750, 0.0109252929687500, 0.0267639160156250, 0.0316772460937500,
+0.0299072265625000, 0.0202026367187500, 0.0073852539062500, 0.0314941406250000,
+0.0125427246093750, 0.0391540527343750, -0.0004577636718750, 0.0276489257812500,
+-0.0216674804687500, 0.0228576660156250, -0.0098876953125000, 0.0312500000000000,
+-0.0452880859375000, 0.0073852539062500, -0.0448608398437500, 0.0085144042968750,
+-0.0473937988281250, 0.0086975097656250, -0.0699157714843750, -0.0043029785156250,
+-0.0626220703125000, -0.0010986328125000, -0.0559997558593750, -0.0028381347656250,
+-0.0661315917968750, -0.0017395019531250, -0.0505065917968750, -0.0082092285156250,
+-0.0284118652343750, -0.0118713378906250, -0.0408325195312500, -0.0058593750000000,
+-0.0243835449218750, -0.0294494628906250, -0.0105590820312500, -0.0256958007812500,
+-0.0390625000000000, -0.0278930664062500, -0.0544433593750000, -0.0450744628906250,
+-0.0361938476562500, -0.0393676757812500, -0.1084289550781250, -0.0384826660156250,
+-0.1024780273437500, -0.0420532226562500, -0.1158752441406250, -0.0412902832031250,
+-0.1649780273437500, -0.0364685058593750, -0.1460876464843750, -0.0368957519531250,
+-0.1759033203125000, -0.0463256835937500, -0.1808471679687500, -0.0445861816406250,
+-0.1784667968750000, -0.0395812988281250, -0.1895141601562500, -0.0493469238281250,
+-0.1864318847656250, -0.0499877929687500, -0.1863403320312500, -0.0024414062500000,
+-0.1887207031250000, -0.0284729003906250, -0.1843872070312500, 0.0131225585937500,
+-0.1821899414062500, 0.0764465332031250, -0.1844482421875000, 0.0358886718750000,
+-0.1664123535156250, 0.1365661621093750, -0.1744689941406250, 0.1320495605468750,
+-0.1587829589843750, 0.1416320800781250, -0.1372680664062500, 0.2015991210937500,
+-0.1516418457031250, 0.1759033203125000, -0.1051940917968750, 0.2061767578125000,
+-0.1121826171875000, 0.2174377441406250, -0.0961608886718750, 0.1997680664062500,
+-0.0603942871093750, 0.2029113769531250, -0.0798950195312500, 0.2110900878906250,
+-0.0330505371093750, 0.1577453613281250, -0.0361633300781250, 0.1712951660156250,
+-0.0287475585937500, 0.1532592773437500, 0.0066833496093750, 0.0956420898437500,
+0.0041198730468750, 0.1042785644531250, 0.0281066894531250, 0.0869750976562500,
+0.0200195312500000, 0.0849609375000000, 0.0267639160156250, 0.0874328613281250,
+0.0447998046875000, 0.0833129882812500, 0.0332946777343750, 0.0822143554687500,
+0.0688781738281250, 0.0632019042968750, 0.0593566894531250, 0.0753784179687500,
+0.0767517089843750, 0.0511779785156250, 0.1079406738281250, 0.0184326171875000,
+0.0869445800781250, 0.0394592285156250, 0.1424255371093750, -0.0089416503906250,
+0.1307678222656250, -0.0133361816406250, 0.1541442871093750, -0.0125427246093750,
+0.2007751464843750, -0.0312805175781250, 0.1714172363281250, -0.0273437500000000,
+0.2428894042968750, -0.0469360351562500, 0.2447204589843750, -0.0354919433593750,
+0.2360229492187500, -0.0615844726562500, 0.2788391113281250, -0.0928344726562500,
+0.2718811035156250, -0.0702819824218750, 0.1941223144531250, -0.1405334472656250,
+0.2425537109375000, -0.1378479003906250, 0.1672668457031250, -0.1424865722656250,
+0.0541381835937500, -0.1849670410156250, 0.1292114257812500, -0.1715698242187500,
+-0.0305175781250000, -0.1764526367187500, -0.0422668457031250, -0.1851806640625000,
+-0.0287475585937500, -0.1760864257812500, -0.1002502441406250, -0.1696166992187500,
+-0.0871887207031250, -0.1752014160156250, -0.0520629882812500, -0.1516113281250000,
+-0.0688476562500000, -0.1610412597656250, -0.0561218261718750, -0.1453247070312500,
+-0.0175781250000000, -0.1190490722656250, -0.0322875976562500, -0.1375732421875000,
+-0.0299377441406250, -0.1000061035156250, -0.0190734863281250, -0.0962829589843750,
+-0.0271606445312500, -0.1020202636718750, -0.0402526855468750, -0.0874938964843750,
+-0.0337219238281250, -0.0883178710937500, -0.0310668945312500, -0.0998229980468750,
+-0.0390930175781250, -0.0974731445312500, -0.0297241210937500, -0.0969543457031250,
+-0.0171203613281250, -0.1057739257812500, -0.0257263183593750, -0.1047363281250000,
+-0.0119934082031250, -0.0928039550781250, -0.0107727050781250, -0.0987854003906250,
+-0.0096740722656250, -0.0885925292968750, -0.0050964355468750, -0.0720825195312500,
+-0.0077819824218750, -0.0803527832031250, 0.0095520019531250, -0.0596313476562500,
+0.0017700195312500, -0.0576782226562500, 0.0117492675781250, -0.0548095703125000,
+0.0319213867187500, -0.0437622070312500, 0.0188293457031250, -0.0476379394531250,
+0.0365905761718750, -0.0392150878906250, 0.0432434082031250, -0.0390014648437500,
+0.0312500000000000, -0.0370788574218750, 0.0311279296875000, -0.0310058593750000,
+0.0384216308593750, -0.0334472656250000, 0.0086669921875000, -0.0314025878906250,
+0.0135803222656250, -0.0288696289062500, 0.0089416503906250, -0.0336303710937500,
+-0.0129089355468750, -0.0371093750000000, -0.0039062500000000, -0.0344848632812500,
+-0.0100708007812500, -0.0454711914062500, -0.0152282714843750, -0.0451049804687500,
+-0.0097351074218750, -0.0456848144531250, -0.0063476562500000, -0.0514526367187500,
+-0.0098571777343750, -0.0488891601562500, -0.0024719238281250, -0.0499877929687500,
+-0.0018615722656250, -0.0493774414062500, -0.0025634765625000, -0.0464172363281250,
+0.0003051757812500, -0.0442810058593750, -0.0003356933593750, -0.0429687500000000,
+-0.0004577636718750, -0.0399475097656250, -0.0002136230468750, -0.0365905761718750,
+-0.0002441406250000, -0.0360107421875000, -0.0004272460937500, -0.0350646972656250,
+-0.0004272460937500, -0.0317077636718750, -0.0003051757812500, -0.0337829589843750,
+-0.0003356933593750, -0.0305480957031250, -0.0002441406250000, -0.0302429199218750,
+-0.0002136230468750, -0.0324401855468750, -0.0002746582031250, -0.0293273925781250,
+-0.0001220703125000, -0.0328674316406250, -0.0001525878906250, -0.0288085937500000,
+-0.0001220703125000, -0.0307617187500000, -0.0000610351562500, -0.0354614257812500,
+-0.0001220703125000, -0.0304870605468750, -0.0000610351562500, -0.0400085449218750,
+-0.0000915527343750, -0.0380554199218750, -0.0000610351562500, -0.0396423339843750,
+-0.0000305175781250, -0.0464172363281250, -0.0000305175781250, -0.0431213378906250,
+-0.0000305175781250, -0.0510253906250000, 0.0000000000000000, -0.0481262207031250,
+-0.0000305175781250, -0.0515441894531250, -0.0000305175781250, -0.0588684082031250,
+0.0000000000000000, -0.0540771484375000, -0.0002136230468750, -0.0663452148437500,
+-0.0001831054687500, -0.0637207031250000, -0.0002136230468750, -0.0681762695312500,
+-0.0002441406250000, -0.0767211914062500, -0.0002441406250000, -0.0711059570312500,
+-0.0006713867187500, -0.0926818847656250, 0.0000915527343750, -0.0823059082031250,
+-0.0021362304687500, -0.1008300781250000, -0.0035705566406250, -0.1256713867187500,
+-0.0019226074218750, -0.1053161621093750, -0.0107421875000000, -0.1638793945312500,
+-0.0102539062500000, -0.1560363769531250, -0.0097351074218750, -0.1595764160156250,
+-0.0173645019531250, -0.2009582519531250, -0.0157775878906250, -0.1852111816406250,
+0.0028686523437500, -0.1756591796875000, -0.0078735351562500, -0.1991882324218750,
+0.0089721679687500, -0.1645507812500000, 0.0380859375000000, -0.1283569335937500,
+0.0214843750000000, -0.1588134765625000, 0.0412292480468750, -0.0741577148437500,
+0.0559692382812500, -0.0852355957031250, 0.0313415527343750, -0.0668334960937500,
+0.0182495117187500, -0.0078125000000000, 0.0359191894531250, -0.0357971191406250,
+-0.0153808593750000, 0.0256042480468750, -0.0197448730468750, 0.0170593261718750,
+-0.0063171386718750, 0.0416564941406250, -0.0251464843750000, 0.0920715332031250,
+-0.0290832519531250, 0.0609130859375000, 0.0128784179687500, 0.1262207031250000,
+0.0088500976562500, 0.1330871582031250, 0.0012817382812500, 0.1231079101562500,
+0.0283813476562500, 0.1491394042968750, 0.0280151367187500, 0.1484069824218750,
+-0.0284729003906250, 0.1230773925781250, -0.0069885253906250, 0.1243896484375000,
+-0.0319213867187500, 0.1296081542968750, -0.0968017578125000, 0.1165771484375000,
+-0.0678405761718750, 0.1151428222656250, -0.0782165527343750, 0.1435546875000000,
+-0.1052551269531250, 0.1406250000000000, -0.0681762695312500, 0.1406555175781250,
+-0.0337524414062500, 0.1616210937500000, -0.0609741210937500, 0.1589965820312500,
+0.0083312988281250, 0.1353454589843750, 0.0122985839843750, 0.1517333984375000,
+0.0094604492187500, 0.1264038085937500, 0.0394897460937500, 0.0914611816406250,
+0.0334777832031250, 0.1157531738281250, 0.0404052734375000, 0.0522766113281250,
+0.0360412597656250, 0.0591430664062500, 0.0509338378906250, 0.0430908203125000,
+0.0650939941406250, 0.0001831054687500, 0.0520324707031250, 0.0222778320312500,
+0.0950622558593750, -0.0378417968750000, 0.0923461914062500, -0.0254211425781250,
+0.0987243652343750, -0.0550842285156250, 0.1245422363281250, -0.1069030761718750,
+0.1156921386718750, -0.0737304687500000, 0.1444091796875000, -0.1673583984375000,
+0.1310729980468750, -0.1546020507812500, 0.1659240722656250, -0.1781616210937500,
+0.2026062011718750, -0.2509460449218750, 0.1707458496093750, -0.2160034179687500,
+0.2867736816406250, -0.2619934082031250, 0.2736816406250000, -0.2829895019531250,
+0.2872009277343750, -0.2501220703125000, 0.3753051757812500, -0.2448120117187500,
+0.3430175781250000, -0.2636108398437500, 0.3091125488281250, -0.1840209960937500,
+0.3626098632812500, -0.1905822753906250, 0.2693786621093750, -0.1889648437500000,
+0.1705322265625000, -0.1382751464843750, 0.2453002929687500, -0.1506958007812500,
+0.0149536132812500, -0.1689147949218750, 0.0257568359375000, -0.1574707031250000,
+0.0114746093750000, -0.1626281738281250, -0.2003479003906250, -0.1940917968750000,
+-0.2117309570312500, -0.1367187500000000, -0.1857910156250000, -0.1623229980468750,
+-0.2113342285156250, -0.1455688476562500, -0.1856689453125000, -0.1512756347656250,
+-0.1480712890625000, -0.1809082031250000, -0.1708374023437500, -0.1771850585937500,
+-0.1562500000000000, -0.1291198730468750, -0.1322021484375000, -0.1534729003906250,
+-0.1741943359375000, -0.1226806640625000, -0.2003479003906250, -0.0626525878906250,
+-0.1680908203125000, -0.0922851562500000, -0.2830505371093750, -0.0327758789062500,
+-0.2731628417968750, -0.0376892089843750, -0.2780151367187500, -0.0108032226562500,
+-0.3563232421875000, 0.0273437500000000, -0.3321838378906250, -0.0016784667968750,
+-0.2918395996093750, 0.1260375976562500, -0.3343200683593750, 0.0991210937500000,
+-0.2737121582031250, 0.1425170898437500, -0.1928100585937500, 0.2576904296875000,
+-0.2494506835937500, 0.2028503417968750, -0.1259460449218750, 0.2650451660156250,
+-0.1245422363281250, 0.2955322265625000, -0.1142578125000000, 0.2557983398437500,
+-0.0508728027343750, 0.2449340820312500, -0.0751647949218750, 0.2657165527343750,
+-0.0086669921875000, 0.1965637207031250, -0.0271606445312500, 0.1993408203125000,
+0.0133361816406250, 0.1895446777343750, 0.0787963867187500, 0.1515197753906250,
+0.0341186523437500, 0.1669616699218750, 0.1540222167968750, 0.1183166503906250,
+0.1496887207031250, 0.1306762695312500, 0.1507873535156250, 0.1032409667968750,
+0.2231140136718750},
+{-0.0170593261718750, 0.0215148925781250, -0.0707092285156250, 0.1256713867187500,
+-0.0402832031250000, 0.0727233886718750, -0.0702209472656250, 0.1642761230468750,
+-0.0934143066406250, 0.1889343261718750, -0.0598144531250000, 0.1575622558593750,
+-0.0405578613281250, 0.1695556640625000, -0.0633239746093750, 0.1809692382812500,
+-0.0002441406250000, 0.1284179687500000, -0.0004882812500000, 0.1285400390625000,
+-0.0011291503906250, 0.1324462890625000, 0.0300292968750000, 0.1064758300781250,
+0.0222167968750000, 0.1106872558593750, 0.0318908691406250, 0.1314086914062500,
+0.0294799804687500, 0.1243591308593750, 0.0413208007812500, 0.1323242187500000,
+0.0534362792968750, 0.1546020507812500, 0.0426940917968750, 0.1459655761718750,
+0.0808105468750000, 0.1460571289062500, 0.0778808593750000, 0.1599426269531250,
+0.0845642089843750, 0.1360473632812500, 0.1083068847656250, 0.1174316406250000,
+0.0994873046875000, 0.1370544433593750, 0.1263732910156250, 0.0684509277343750,
+0.1150512695312500, 0.0802307128906250, 0.1465454101562500, 0.0568237304687500,
+0.1804504394531250, 0.0037231445312500, 0.1519470214843750, 0.0329589843750000,
+0.2535095214843750, -0.0362548828125000, 0.2426147460937500, -0.0264587402343750,
+0.2538146972656250, -0.0517883300781250, 0.3294982910156250, -0.1029968261718750,
+0.3020324707031250, -0.0726013183593750, 0.2768859863281250, -0.1672973632812500,
+0.3245849609375000, -0.1531372070312500, 0.2445068359375000, -0.1829833984375000,
+0.1605224609375000, -0.2620239257812500, 0.2243347167968750, -0.2221984863281250,
+0.0238952636718750, -0.2718505859375000, 0.0405883789062500, -0.2971801757812500,
+0.0006103515625000, -0.2551574707031250, -0.1369628906250000, -0.2438354492187500,
+-0.0778503417968750, -0.2705078125000000, -0.1749267578125000, -0.1752014160156250,
+-0.1909179687500000, -0.1793212890625000, -0.1770324707031250, -0.1846313476562500,
+-0.2233581542968750, -0.1135559082031250, -0.1650085449218750, -0.2692565917968750,
+-0.1531677246093750, -0.1472167968750000, -0.1300659179687500, -0.1773071289062500,
+-0.1738891601562500, -0.1700744628906250, -0.2019042968750000, -0.1181945800781250,
+-0.1693725585937500, -0.1256103515625000, -0.2754516601562500, -0.1580505371093750,
+-0.2689208984375000, -0.1515808105468750, -0.2714538574218750, -0.1469421386718750,
+-0.3398132324218750, -0.1721191406250000, -0.3194580078125000, -0.1715393066406250,
+-0.2844543457031250, -0.1141357421875000, -0.3194580078125000, -0.1359252929687500,
+-0.2674255371093750, -0.1082458496093750, -0.1980285644531250, -0.0481262207031250,
+-0.2433776855468750, -0.0767822265625000, -0.1354675292968750, -0.0201416015625000,
+-0.1387023925781250, -0.0263977050781250, -0.1206970214843750, 0.0033264160156250,
+-0.0603332519531250, 0.0450439453125000, -0.0888977050781250, 0.0137939453125000,
+0.0026245117187500, 0.1341247558593750, -0.0213928222656250, 0.1144409179687500,
+0.0299987792968750, 0.1449279785156250, 0.1206054687500000, 0.2451477050781250,
+0.0627746582031250, 0.2030029296875000, 0.1876525878906250, 0.2363281250000000,
+0.1914062500000000, 0.2652893066406250, 0.1813354492187500, 0.2286376953125000,
+0.2456665039062500, 0.2049865722656250, 0.2312927246093750, 0.2279052734375000,
+0.1817016601562500, 0.1666870117187500, 0.2142944335937500, 0.1671447753906250,
+0.1744079589843750, 0.1580810546875000, 0.1017150878906250, 0.1259765625000000,
+0.1451110839843750, 0.1402282714843750, 0.0937500000000000, 0.0910644531250000,
+0.0684509277343750, 0.1018371582031250, 0.1047363281250000, 0.0795593261718750,
+0.1181945800781250, 0.0336303710937500, 0.0932006835937500, 0.0579223632812500,
+0.1373596191406250, 0.0065002441406250, 0.1562194824218750, 0.0042724609375000,
+0.1199645996093750, 0.0000915527343750, 0.1130371093750000, -0.0250549316406250,
+0.1378479003906250, -0.0151062011718750, 0.0481567382812500, -0.0447998046875000,
+0.0618286132812500, -0.0406799316406250, 0.0469970703125000, -0.0483703613281250,
+-0.0132751464843750, -0.0756835937500000, 0.0146789550781250, -0.0646057128906250,
+-0.0346679687500000, -0.0602722167968750, -0.0338745117187500, -0.0764770507812500,
+-0.0383605957031250, -0.0524597167968750, -0.0700683593750000, -0.0238952636718750,
+-0.0579833984375000, -0.0431518554687500, -0.0651855468750000, -0.0028991699218750,
+-0.0751342773437500, 0.0032348632812500, -0.0646057128906250, -0.0034484863281250,
+-0.0569458007812500, 0.0072326660156250, -0.0652465820312500, 0.0075378417968750,
+-0.0422668457031250, 0.0090637207031250, -0.0465393066406250, 0.0029296875000000,
+-0.0400085449218750, 0.0147399902343750, -0.0224609375000000, 0.0279846191406250,
+-0.0339965820312500, 0.0169677734375000, -0.0191955566406250, 0.0354309082031250,
+-0.0165710449218750, 0.0435485839843750, -0.0232849121093750, 0.0286254882812500,
+-0.0209655761718750, 0.0247802734375000, -0.0196838378906250, 0.0339660644531250,
+-0.0321655273437500, -0.0000915527343750, -0.0283203125000000, 0.0006408691406250,
+-0.0358581542968750, 0.0006408691406250, -0.0472106933593750, -0.0183715820312500,
+-0.0385437011718750, -0.0140075683593750, -0.0603942871093750, -0.0090026855468750,
+-0.0553894042968750, -0.0130920410156250, -0.0615539550781250, -0.0091552734375000,
+-0.0778808593750000, -0.0013732910156250, -0.0669860839843750, -0.0046081542968750,
+-0.0896301269531250, -0.0009460449218750, -0.0872192382812500, 0.0003967285156250,
+-0.0905456542968750, -0.0004882812500000, -0.1063537597656250, -0.0001220703125000,
+-0.0993347167968750, -0.0000610351562500, -0.1076965332031250, -0.0001831054687500,
+-0.1111755371093750, -0.0002441406250000, -0.1068725585937500, -0.0002136230468750,
+-0.1074523925781250, -0.0001220703125000, -0.1100463867187500, -0.0001220703125000,
+-0.0998840332031250, -0.0002136230468750, -0.1024475097656250, -0.0001831054687500,
+-0.0984497070312500, -0.0001831054687500, -0.0899963378906250, -0.0002441406250000,
+-0.0953674316406250, -0.0002136230468750, -0.0840454101562500, -0.0001525878906250,
+-0.0856323242187500, -0.0001831054687500, -0.0825195312500000, -0.0001831054687500,
+-0.0746459960937500, -0.0001525878906250, -0.0798339843750000, -0.0001831054687500,
+-0.0683898925781250, 0.0000000000000000, -0.0696716308593750, -0.0000305175781250,
+-0.0674133300781250, -0.0000915527343750, -0.0593872070312500, 0.0001831054687500,
+-0.0625915527343750, 0.0000610351562500, -0.0542907714843750, -0.0009155273437500,
+-0.0553283691406250, 0.0001220703125000, -0.0519409179687500, -0.0017700195312500,
+-0.0460205078125000, -0.0037231445312500, -0.0496520996093750, -0.0017700195312500,
+-0.0353698730468750, -0.0071411132812500, -0.0391235351562500, -0.0071716308593750,
+-0.0311584472656250, -0.0056152343750000, -0.0174865722656250, -0.0087585449218750,
+-0.0249938964843750, -0.0088500976562500, -0.0007629394531250, 0.0011291503906250,
+-0.0034484863281250, -0.0042724609375000, 0.0034790039062500, 0.0037841796875000,
+0.0220336914062500, 0.0169982910156250, 0.0138244628906250, 0.0079345703125000,
+0.0304565429687500, 0.0231323242187500, 0.0326232910156250, 0.0262145996093750,
+0.0276184082031250, 0.0204162597656250, 0.0328979492187500, 0.0241394042968750,
+0.0326232910156250, 0.0266723632812500, 0.0158081054687500, 0.0115966796875000,
+0.0201110839843750, 0.0150756835937500, 0.0112609863281250, 0.0120849609375000,
+-0.0060119628906250, 0.0001831054687500, 0.0008239746093750, 0.0051574707031250,
+-0.0160827636718750, 0.0002746582031250, -0.0179748535156250, -0.0005187988281250,
+-0.0190429687500000, -0.0027770996093750, -0.0280151367187500, -0.0044860839843750,
+-0.0248413085937500, -0.0018615722656250, -0.0223083496093750, -0.0165710449218750,
+-0.0267028808593750, -0.0120849609375000, -0.0179748535156250, -0.0175476074218750,
+-0.0088195800781250, -0.0321044921875000, -0.0147399902343750, -0.0248413085937500,
+0.0054321289062500, -0.0316162109375000, 0.0085754394531250, -0.0351562500000000,
+0.0051269531250000, -0.0321044921875000, 0.0140075683593750, -0.0304260253906250,
+0.0149536132812500, -0.0320434570312500, -0.0065612792968750, -0.0343017578125000,
+-0.0011596679687500, -0.0314941406250000, -0.0056152343750000, -0.0330810546875000,
+-0.0259399414062500, -0.0400695800781250, -0.0193176269531250, -0.0368347167968750,
+-0.0107421875000000, -0.0223999023437500, -0.0167236328125000, -0.0374755859375000,
+-0.0118713378906250, -0.0090942382812500, 0.0023803710937500, 0.0199584960937500,
+-0.0020446777343750, -0.0052795410156250, -0.0111083984375000, 0.0762634277343750,
+-0.0006408691406250, 0.0629272460937500, -0.0170593261718750, 0.0861206054687500,
+-0.0358581542968750, 0.1484680175781250, -0.0222778320312500, 0.1158447265625000,
+-0.0690307617187500, 0.1782836914062500, -0.0619201660156250, 0.1813964843750000,
+-0.0786437988281250, 0.1765747070312500, -0.1179809570312500, 0.2094116210937500,
+-0.0992431640625000, 0.2014465332031250, -0.1349792480468750, 0.1726074218750000,
+-0.1378173828125000, 0.1952209472656250, -0.1373901367187500, 0.1625671386718750,
+-0.1533813476562500, 0.1159057617187500, -0.1484375000000000, 0.1501464843750000,
+-0.1561889648437500, 0.0851440429687500, -0.1572265625000000, 0.0787658691406250,
+-0.1559143066406250, 0.0847167968750000, -0.1604309082031250, 0.0603637695312500,
+-0.1597290039062500, 0.0639953613281250, -0.1489562988281250, 0.0576782226562500,
+-0.1559143066406250, 0.0650024414062500, -0.1433410644531250, 0.0465393066406250,
+-0.1228027343750000, 0.0223388671875000, -0.1222229003906250, 0.0255126953125000,
+-0.0809936523437500, -0.0244140625000000, -0.0972595214843750, -0.0112304687500000,
+-0.0871582031250000, -0.0135192871093750, -0.0533142089843750, -0.0491943359375000,
+-0.0722656250000000, -0.0386352539062500, -0.0314331054687500, -0.0513305664062500,
+-0.0353393554687500, -0.0494689941406250, -0.0266418457031250, -0.0630798339843750,
+-0.0000915527343750, -0.0750122070312500, -0.0148315429687500, -0.0627441406250000,
+0.0191955566406250, -0.1190490722656250, 0.0134277343750000, -0.1094970703125000,
+0.0256652832031250, -0.1260986328125000, 0.0518188476562500, -0.1736145019531250,
+0.0362548828125000, -0.1535034179687500, 0.0800476074218750, -0.1789855957031250,
+0.0733642578125000, -0.1887207031250000, 0.0874938964843750, -0.1769409179687500,
+0.1195068359375000, -0.1773986816406250, 0.1009521484375000, -0.1817626953125000,
+0.1498718261718750, -0.1589355468750000, 0.1415405273437500, -0.1642761230468750,
+0.1593933105468750, -0.1526489257812500, 0.2007751464843750, -0.1338195800781250,
+0.1771850585937500, -0.1456298828125000, 0.2181701660156250, -0.1039428710937500,
+0.2316284179687500, -0.1099243164062500, 0.2047729492187500, -0.1021728515625000,
+0.2124633789062500, -0.0682373046875000, 0.2264404296875000, -0.0797119140625000,
+0.1165466308593750, -0.0771484375000000, 0.1522521972656250, -0.0668640136718750,
+0.0998535156250000, -0.0718688964843750, -0.0114135742187500, -0.0835571289062500,
+0.0500793457031250, -0.0770874023437500, -0.0503845214843750, -0.0623779296875000,
+-0.0679321289062500, -0.0737915039062500, -0.0433959960937500, -0.0560913085937500,
+-0.0715942382812500, -0.0287475585937500, -0.0756835937500000, -0.0437622070312500,
+-0.0353698730468750, -0.0175476074218750, -0.0414123535156250, -0.0105895996093750,
+-0.0401611328125000, -0.0171203613281250, -0.0142517089843750, -0.0122375488281250,
+-0.0212402343750000, -0.0096740722656250, -0.0259704589843750, -0.0178833007812500,
+-0.0205993652343750, -0.0194396972656250, -0.0220031738281250, -0.0134582519531250,
+-0.0300903320312500, -0.0144348144531250, -0.0278930664062500, -0.0163269042968750,
+-0.0196533203125000},
+{-0.0095214843750000, -0.0146484375000000, -0.0112304687500000, 0.0530395507812500,
+-0.0105285644531250, 0.0381774902343750, -0.0069885253906250, 0.0635070800781250,
+-0.0061340332031250, 0.1203613281250000, -0.0090332031250000, 0.0881652832031250,
+-0.0016479492187500, 0.1566467285156250, 0.0017089843750000, 0.1563720703125000,
+-0.0036926269531250, 0.1584167480468750, -0.0052490234375000, 0.1990356445312500,
+-0.0013427734375000, 0.1842346191406250, -0.0124511718750000, 0.1747131347656250,
+-0.0091552734375000, 0.1960144042968750, -0.0191040039062500, 0.1636352539062500,
+-0.0292358398437500, 0.1278381347656250, -0.0207214355468750, 0.1557312011718750,
+-0.0628051757812500, 0.0860900878906250, -0.0527648925781250, 0.0855102539062500,
+-0.0686035156250000, 0.0888061523437500, -0.1067199707031250, 0.0556945800781250,
+-0.0883178710937500, 0.0634460449218750, -0.1214904785156250, 0.0597229003906250,
+-0.1245727539062500, 0.0627136230468750, -0.1250000000000000, 0.0464477539062500,
+-0.1397705078125000, 0.0365905761718750, -0.1344909667968750, 0.0510559082031250,
+-0.1444702148437500, -0.0057983398437500, -0.1448364257812500, 0.0016174316406250,
+-0.1435852050781250, -0.0061340332031250, -0.1509399414062500, -0.0510559082031250,
+-0.1492614746093750, -0.0450134277343750, -0.1307983398437500, -0.0532531738281250,
+-0.1425781250000000, -0.0511779785156250, -0.1310729980468750, -0.0589904785156250,
+-0.1109008789062500, -0.0652770996093750, -0.1255798339843750, -0.0584106445312500,
+-0.0847167968750000, -0.1064147949218750, -0.0911254882812500, -0.0942077636718750,
+-0.0798950195312500, -0.1152954101562500, -0.0502929687500000, -0.1620788574218750,
+-0.0668029785156250, -0.1387939453125000, -0.0293579101562500, -0.1744689941406250,
+-0.0330810546875000, -0.1838989257812500, -0.0238342285156250, -0.1722106933593750,
+0.0015258789062500, -0.1788635253906250, -0.0118713378906250, -0.1816711425781250,
+0.0231933593750000, -0.1545715332031250, 0.0170593261718750, -0.1610412597656250,
+0.0312500000000000, -0.1490478515625000, 0.0592346191406250, -0.1253051757812500,
+0.0415954589843750, -0.1370239257812500, 0.0858459472656250, -0.1040344238281250,
+0.0799255371093750, -0.1087036132812500, 0.0924987792968750, -0.0960388183593750,
+0.1243896484375000, -0.0682067871093750, 0.1075134277343750, -0.0834045410156250,
+0.1515808105468750, -0.0560302734375000, 0.1452636718750000, -0.0513610839843750,
+0.1585998535156250, -0.0553894042968750, 0.1955871582031250, -0.0484924316406250,
+0.1771850585937500, -0.0475158691406250, 0.1998291015625000, -0.0386352539062500,
+0.2163696289062500, -0.0465393066406250, 0.1821899414062500, -0.0331115722656250,
+0.1724853515625000, -0.0144348144531250, 0.1954040527343750, -0.0273132324218750,
+0.0812683105468750, -0.0044250488281250, 0.1106872558593750, 0.0007324218750000,
+0.0696411132812500, -0.0059814453125000, -0.0349426269531250, -0.0028076171875000,
+0.0184020996093750, -0.0002746582031250, -0.0522766113281250, -0.0039672851562500,
+-0.0714111328125000, -0.0092773437500000, -0.0469970703125000, 0.0023193359375000,
+-0.0581970214843750, 0.0093688964843750, -0.0665283203125000, 0.0000915527343750,
+-0.0293884277343750, 0.0321350097656250, -0.0325622558593750, 0.0292358398437500,
+-0.0339965820312500, 0.0356445312500000, -0.0135192871093750, 0.0597229003906250,
+-0.0179443359375000, 0.0506591796875000, -0.0229492187500000, 0.0557250976562500,
+-0.0194702148437500, 0.0645446777343750, -0.0189514160156250, 0.0491943359375000,
+-0.0244750976562500, 0.0370483398437500, -0.0238952636718750, 0.0464172363281250,
+-0.0146789550781250, 0.0128173828125000, -0.0190734863281250, 0.0125427246093750,
+-0.0146789550781250, 0.0086975097656250, -0.0044860839843750, -0.0112609863281250,
+-0.0097351074218750, -0.0050354003906250, -0.0017700195312500, -0.0105285644531250,
+-0.0018005371093750, -0.0148620605468750, 0.0007934570312500, -0.0093688964843750,
+0.0050659179687500, -0.0067749023437500, 0.0012817382812500, -0.0093078613281250,
+0.0151977539062500, 0.0051574707031250, 0.0124511718750000, 0.0025634765625000,
+0.0151062011718750, 0.0091552734375000, 0.0272521972656250, 0.0225830078125000,
+0.0222167968750000, 0.0171508789062500, 0.0200805664062500, 0.0320434570312500,
+0.0266723632812500, 0.0326538085937500, 0.0174865722656250, 0.0342712402343750,
+0.0072326660156250, 0.0427856445312500, 0.0157470703125000, 0.0398559570312500,
+-0.0023193359375000, 0.0429687500000000, -0.0027160644531250, 0.0438232421875000,
+-0.0012817382812500, 0.0420837402343750, -0.0099792480468750, 0.0418090820312500,
+-0.0080566406250000, 0.0416564941406250, -0.0052490234375000, 0.0386352539062500,
+-0.0074768066406250, 0.0380554199218750, -0.0053710937500000, 0.0360717773437500,
+-0.0012512207031250, 0.0333251953125000, -0.0033569335937500, 0.0333862304687500,
+-0.0005493164062500, 0.0297546386718750, 0.0000305175781250, 0.0280761718750000,
+-0.0003967285156250, 0.0265808105468750, 0.0003356933593750, 0.0249938964843750,
+0.0002136230468750, 0.0251770019531250, 0.0001525878906250, 0.0225524902343750,
+0.0002136230468750, 0.0233459472656250, -0.0000610351562500, 0.0201416015625000,
+0.0000000000000000, 0.0173950195312500, 0.0000915527343750, 0.0204467773437500,
+-0.0023193359375000, 0.0104064941406250, -0.0008850097656250, 0.0123901367187500,
+-0.0038452148437500, 0.0094909667968750, -0.0077209472656250, 0.0009155273437500,
+-0.0049133300781250, 0.0043640136718750, -0.0109863281250000, -0.0020446777343750,
+-0.0135192871093750, -0.0034484863281250, -0.0069580078125000, -0.0030212402343750,
+-0.0069274902343750, -0.0046386718750000, -0.0106811523437500, -0.0046081542968750,
+0.0160827636718750, -0.0077209472656250, 0.0107421875000000, -0.0047302246093750,
+0.0168151855468750, -0.0096435546875000, 0.0417480468750000, -0.0148925781250000,
+0.0332336425781250, -0.0101623535156250, 0.0245666503906250, -0.0268554687500000,
+0.0365905761718750, -0.0216674804687500, 0.0183410644531250, -0.0274047851562500,
+-0.0084228515625000, -0.0414733886718750, 0.0072937011718750, -0.0324401855468750,
+-0.0067138671875000, -0.0516662597656250, -0.0198974609375000, -0.0514526367187500,
+-0.0029907226562500, -0.0520629882812500, 0.0122375488281250, -0.0643310546875000,
+-0.0004882812500000, -0.0608215332031250, 0.0107421875000000, -0.0553894042968750,
+0.0263366699218750, -0.0619812011718750, -0.0002746582031250, -0.0522766113281250,
+-0.0205383300781250, -0.0405883789062500, 0.0004577636718750, -0.0498657226562500,
+-0.0513610839843750, -0.0280456542968750, -0.0582275390625000, -0.0310363769531250,
+-0.0423278808593750, -0.0248718261718750, -0.0580749511718750, -0.0075683593750000,
+-0.0633239746093750, -0.0178833007812500, -0.0180053710937500, -0.0079345703125000,
+-0.0267944335937500, 0.0039978027343750, -0.0180969238281250, -0.0221252441406250,
+0.0163269042968750, -0.0375976562500000, 0.0034484863281250, -0.0195617675781250,
+0.0184936523437500, -0.0892028808593750, 0.0201721191406250, -0.0960998535156250,
+0.0264892578125000, -0.0773315429687500, 0.0348205566406250, -0.1069335937500000,
+0.0274047851562500, -0.1136169433593750, 0.0567321777343750, -0.0253906250000000,
+0.0523681640625000, -0.0517578125000000, 0.0585327148437500, -0.0179443359375000,
+0.0792846679687500, 0.0665893554687500, 0.0712585449218750, 0.0300903320312500,
+0.0924682617187500, 0.0951232910156250, 0.0856628417968750, 0.1028747558593750,
+0.1073608398437500, 0.0994873046875000, 0.1299133300781250, 0.1283874511718750,
+0.1104431152343750, 0.1201171875000000, 0.1891174316406250, 0.1105651855468750,
+0.1777343750000000, 0.1227111816406250, 0.1921997070312500, 0.1076660156250000,
+0.2556152343750000, 0.0843200683593750, 0.2303466796875000, 0.0985412597656250,
+0.2253112792968750, 0.0814819335937500, 0.2597351074218750, 0.0728759765625000,
+0.2004699707031250, 0.0838623046875000, 0.1472473144531250, 0.0887756347656250,
+0.1937255859375000, 0.0831298828125000, 0.0442504882812500, 0.0968933105468750,
+0.0614624023437500, 0.1031799316406250, 0.0268554687500000, 0.0900268554687500,
+-0.0800781250000000, 0.0902404785156250, -0.0313110351562500, 0.0988464355468750,
+-0.1223449707031250, 0.0451660156250000, -0.1304016113281250, 0.0628662109375000,
+-0.1287841796875000, 0.0325012207031250, -0.1717834472656250, -0.0226745605468750,
+-0.1587524414062500, 0.0087890625000000, -0.1589355468750000, -0.0664978027343750,
+-0.1692810058593750, -0.0607910156250000, -0.1560058593750000, -0.0795898437500000,
+-0.1391601562500000, -0.1272583007812500, -0.1499328613281250, -0.1007995605468750,
+-0.1436157226562500, -0.1806030273437500, -0.1295166015625000, -0.1689453125000000,
+-0.1529846191406250, -0.1829528808593750, -0.1945190429687500, -0.2794799804687500,
+-0.2567749023437500, -0.1045227050781250, -0.2362670898437500, -0.1960449218750000,
+-0.2692871093750000, -0.1592407226562500, -0.2286987304687500, -0.1919555664062500,
+-0.1848449707031250, -0.2559814453125000, -0.2142028808593750, -0.2287902832031250,
+-0.1399841308593750, -0.2347412109375000, -0.1477050781250000, -0.2653808593750000,
+-0.1213684082031250, -0.2173461914062500, -0.0712585449218750, -0.1712036132812500,
+-0.1009216308593750, -0.2088623046875000, 0.0099487304687500, -0.1230468750000000,
+-0.0143737792968750, -0.1119689941406250, 0.0331726074218750, -0.1287536621093750,
+0.1355285644531250, -0.1018676757812500, 0.0786132812500000, -0.0996398925781250,
+0.1739501953125000, -0.1281738281250000, 0.1957092285156250, -0.1319580078125000,
+0.1601257324218750, -0.1153869628906250, 0.1822814941406250, -0.1215515136718750,
+0.1976928710937500, -0.1312561035156250, 0.1036376953125000, -0.0705261230468750,
+0.1240844726562500, -0.0829467773437500, 0.1044311523437500, -0.0645751953125000,
+0.0282287597656250, -0.0162963867187500, 0.0602722167968750, -0.0373229980468750,
+0.0393676757812500, 0.0132141113281250, 0.0186462402343750, 0.0030517578125000,
+0.0462646484375000, 0.0354003906250000, 0.0624694824218750, 0.0820922851562500,
+0.0442504882812500, 0.0487060546875000, 0.1031799316406250, 0.1494445800781250,
+0.1027221679687500, 0.1454467773437500, 0.0941467285156250, 0.1496887207031250,
+0.1282958984375000, 0.2120666503906250, 0.1245117187500000, 0.1932373046875000,
+0.0619812011718750, 0.1792297363281250, 0.0896911621093750, 0.1993103027343750,
+0.0598144531250000, 0.1745605468750000, -0.0127868652343750, 0.1398010253906250,
+0.0228271484375000, 0.1593933105468750, -0.0067749023437500, 0.1154479980468750,
+-0.0252380371093750, 0.1166076660156250, -0.0121154785156250, 0.1057739257812500,
+-0.0024108886718750, 0.0795898437500000, -0.0090637207031250, 0.0936889648437500,
+-0.0205383300781250, 0.0462036132812500, -0.0073242187500000, 0.0529785156250000,
+-0.0216064453125000, 0.0396118164062500, -0.0463562011718750, 0.0022888183593750,
+-0.0317993164062500, 0.0191955566406250, -0.0508422851562500, -0.0110168457031250,
+-0.0562744140625000, -0.0118103027343750, -0.0515441894531250, -0.0178527832031250,
+-0.0563659667968750, -0.0352478027343750, -0.0567626953125000, -0.0256347656250000,
+-0.0432128906250000, -0.0508728027343750, -0.0518798828125000, -0.0531005859375000,
+-0.0395202636718750, -0.0461120605468750, -0.0191650390625000, -0.0576477050781250,
+-0.0332946777343750, -0.0592346191406250, -0.0139160156250000, -0.0261230468750000,
+-0.0074768066406250, -0.0356445312500000, -0.0183105468750000, -0.0246582031250000,
+-0.0178527832031250, 0.0068359375000000, -0.0130004882812500, -0.0065002441406250,
+-0.0375366210937500, 0.0052490234375000, -0.0327758789062500, 0.0106811523437500,
+-0.0420532226562500},
+{-0.1488037109375000, 0.0900268554687500, -0.1390380859375000, 0.0805358886718750,
+-0.1278991699218750, 0.0834350585937500, -0.1361083984375000, 0.0891418457031250,
+-0.1340637207031250, 0.0383300781250000, -0.1204833984375000, 0.0578308105468750,
+-0.1478881835937500, 0.0262451171875000, -0.1683959960937500, -0.0291137695312500,
+-0.1472167968750000, 0.0047607421875000, -0.2224731445312500, -0.0701293945312500,
+-0.2181091308593750, -0.0682067871093750, -0.2179870605468750, -0.0743713378906250,
+-0.2815246582031250, -0.1452941894531250, -0.2016296386718750, 0.0112304687500000,
+-0.1210327148437500, -0.0814514160156250, -0.1589050292968750, -0.0516662597656250,
+-0.1122436523437500, -0.0823059082031250, -0.0593566894531250, -0.1286315917968750,
+-0.0947570800781250, -0.1052246093750000, 0.0245056152343750, -0.1766052246093750,
+0.0028076171875000, -0.1683044433593750, 0.0419921875000000, -0.1797790527343750,
+0.1428527832031250, -0.2362976074218750, 0.0904846191406250, -0.2143859863281250,
+0.1668701171875000, -0.2037658691406250, 0.1932373046875000, -0.2324218750000000,
+0.1535949707031250, -0.1901855468750000, 0.1577758789062500, -0.1394958496093750,
+0.1777954101562500, -0.1752624511718750, 0.0826721191406250, -0.1066894531250000,
+0.0964965820312500, -0.0934753417968750, 0.0856933593750000, -0.1106567382812500,
+0.0180358886718750, -0.0957946777343750, 0.0430603027343750, -0.0907287597656250,
+0.0425109863281250, -0.1102600097656250, 0.0224609375000000, -0.1179504394531250,
+0.0442199707031250, -0.0980224609375000, 0.0700683593750000, -0.0940246582031250,
+0.0525817871093750, -0.1073303222656250, 0.0828247070312500, -0.0468139648437500,
+0.0850219726562500, -0.0552978515625000, 0.0830688476562500, -0.0404663085937500,
+0.0991210937500000, 0.0018310546875000, 0.0931091308593750, -0.0167541503906250,
+0.0733032226562500, 0.0326538085937500, 0.0932312011718750, 0.0210266113281250,
+0.0586242675781250, 0.0518493652343750, 0.0162658691406250, 0.1007995605468750,
+0.0481262207031250, 0.0679016113281250, -0.0248107910156250, 0.1490173339843750,
+-0.0280761718750000, 0.1533508300781250, -0.0291137695312500, 0.1478271484375000,
+-0.0613098144531250, 0.1880798339843750, -0.0507202148437500, 0.1809082031250000,
+-0.0800170898437500, 0.1530456542968750, -0.0724182128906250, 0.1684265136718750,
+-0.0828857421875000, 0.1484375000000000, -0.1138305664062500, 0.1122131347656250,
+-0.0966186523437500, 0.1298217773437500, -0.0942687988281250, 0.0917968750000000,
+-0.1158447265625000, 0.0939025878906250, -0.0841979980468750, 0.0820007324218750,
+-0.0515441894531250, 0.0569152832031250, -0.0797729492187500, 0.0718994140625000,
+-0.0190734863281250, 0.0269775390625000, -0.0186767578125000, 0.0310363769531250,
+-0.0192260742187500, 0.0230407714843750, 0.0114746093750000, -0.0075988769531250,
+-0.0003662109375000, 0.0045471191406250, 0.0002136230468750, -0.0173950195312500,
+0.0119628906250000, -0.0164184570312500, -0.0101013183593750, -0.0237121582031250,
+-0.0270690917968750, -0.0402526855468750, -0.0098266601562500, -0.0305175781250000,
+-0.0647277832031250, -0.0488891601562500, -0.0600891113281250, -0.0548095703125000,
+-0.0673217773437500, -0.0420532226562500, -0.1019287109375000, -0.0425109863281250,
+-0.0861511230468750, -0.0500183105468750, -0.1040039062500000, -0.0137023925781250,
+-0.1094360351562500, -0.0174865722656250, -0.1007385253906250, -0.0141601562500000,
+-0.1041564941406250, 0.0099182128906250, -0.1077270507812500, 0.0029907226562500,
+-0.0834350585937500, 0.0061340332031250, -0.0950622558593750, 0.0070800781250000,
+-0.0769042968750000, 0.0115966796875000, -0.0512390136718750, 0.0142211914062500,
+-0.0732421875000000, 0.0095825195312500, -0.0180358886718750, 0.0288696289062500,
+-0.0291748046875000, 0.0296630859375000, -0.0156860351562500, 0.0252075195312500,
+0.0239868164062500, 0.0345458984375000, -0.0021667480468750, 0.0353698730468750,
+0.0433654785156250, 0.0121765136718750, 0.0378112792968750, 0.0178527832031250,
+0.0418395996093750, 0.0101013183593750, 0.0692749023437500, -0.0115966796875000,
+0.0528869628906250, -0.0031738281250000, 0.0766906738281250, -0.0106811523437500,
+0.0729980468750000, -0.0158996582031250, 0.0767822265625000, -0.0104370117187500,
+0.0922546386718750, -0.0071411132812500, 0.0811157226562500, -0.0098876953125000,
+0.0997924804687500, -0.0031433105468750, 0.0976562500000000, -0.0020141601562500,
+0.0973205566406250, -0.0031738281250000, 0.1074829101562500, -0.0010681152343750,
+0.1015014648437500, -0.0009460449218750, 0.1053466796875000, -0.0036926269531250,
+0.1047973632812500, -0.0028381347656250, 0.1028137207031250, -0.0034484863281250,
+0.1033935546875000, -0.0061645507812500, 0.1015625000000000, -0.0049743652343750,
+0.1014709472656250, -0.0032958984375000, 0.1002197265625000, -0.0061340332031250,
+0.0991821289062500, -0.0012207031250000, 0.0986328125000000, 0.0034484863281250,
+0.0955505371093750, -0.0012817382812500, 0.0909423828125000, 0.0115356445312500,
+0.0924072265625000, 0.0105895996093750, 0.0834045410156250, 0.0106506347656250,
+0.0745239257812500, 0.0185546875000000, 0.0778503417968750, 0.0160217285156250,
+0.0560302734375000, 0.0134582519531250, 0.0580749511718750, 0.0167846679687500,
+0.0506896972656250, 0.0128173828125000, 0.0332641601562500, 0.0070495605468750,
+0.0388793945312500, 0.0109558105468750, 0.0208435058593750, 0.0045471191406250,
+0.0198364257812500, 0.0039672851562500, 0.0173645019531250, 0.0035705566406250,
+0.0065612792968750, 0.0011901855468750, 0.0087280273437500, 0.0022583007812500,
+0.0022888183593750, -0.0040588378906250, 0.0013122558593750, -0.0010375976562500,
+-0.0000610351562500, -0.0059204101562500, -0.0033264160156250, -0.0137634277343750,
+-0.0024719238281250, -0.0082092285156250, -0.0089111328125000, -0.0186462402343750,
+-0.0080871582031250, -0.0194702148437500, -0.0105895996093750, -0.0184631347656250,
+-0.0167846679687500, -0.0225524902343750, -0.0147399902343750, -0.0217895507812500,
+-0.0169372558593750, -0.0226440429687500, -0.0199279785156250, -0.0218200683593750,
+-0.0170593261718750, -0.0239562988281250, -0.0162658691406250, -0.0265808105468750,
+-0.0188903808593750, -0.0238647460937500, -0.0046691894531250, -0.0256958007812500,
+-0.0114440917968750, -0.0303649902343750, 0.0010375976562500, -0.0190124511718750,
+0.0186462402343750, -0.0139160156250000, 0.0067138671875000, -0.0223388671875000,
+0.0417785644531250, 0.0191040039062500, 0.0388488769531250, 0.0055541992187500,
+0.0466918945312500, 0.0290527343750000, 0.0736083984375000, 0.0695495605468750,
+0.0617065429687500, 0.0433044433593750, 0.0676269531250000, 0.1090698242187500,
+0.0798034667968750, 0.1021423339843750, 0.0601501464843750, 0.1132812500000000,
+0.0432739257812500, 0.1597900390625000, 0.0571899414062500, 0.1380004882812500,
+0.0201416015625000, 0.1585388183593750, 0.0184631347656250, 0.1728515625000000,
+0.0153808593750000, 0.1491394042968750, -0.0034484863281250, 0.1404418945312500,
+0.0040283203125000, 0.1565856933593750, -0.0075378417968750, 0.0973815917968750,
+-0.0131835937500000, 0.1050720214843750, -0.0017700195312500, 0.0977783203125000,
+0.0018920898437500, 0.0567626953125000, -0.0069885253906250, 0.0718688964843750,
+0.0147094726562500, 0.0611572265625000, 0.0204162597656250, 0.0567321777343750,
+0.0082092285156250, 0.0527038574218750, 0.0084838867187500, 0.0527343750000000,
+0.0157775878906250, 0.0570373535156250, -0.0083007812500000, 0.0144653320312500,
+-0.0085449218750000, 0.0299072265625000, -0.0065917968750000, 0.0102233886718750,
+-0.0160522460937500, -0.0341186523437500, -0.0146179199218750, -0.0094604492187500,
+-0.0191345214843750, -0.0428771972656250, -0.0131530761718750, -0.0502319335937500,
+-0.0257568359375000, -0.0490722656250000, -0.0375061035156250, -0.0563049316406250,
+-0.0263366699218750, -0.0534667968750000, -0.0642395019531250, -0.0835266113281250,
+-0.0592651367187500, -0.0696411132812500, -0.0662841796875000, -0.0914611816406250,
+-0.0979003906250000, -0.1390991210937500, -0.0952148437500000, -0.1315307617187500,
+-0.1109619140625000, -0.1639099121093750, -0.1069946289062500, -0.1622314453125000,
+-0.1089782714843750, -0.1551208496093750, -0.1198120117187500, -0.1706848144531250,
+-0.1135559082031250, -0.1693420410156250, -0.1255187988281250, -0.1500549316406250,
+-0.1258544921875000, -0.1559753417968750, -0.1228942871093750, -0.1430053710937500,
+-0.1292419433593750, -0.1231079101562500, -0.1287231445312500, -0.1338195800781250,
+-0.1136169433593750, -0.1007690429687500, -0.1219787597656250, -0.1027221679687500,
+-0.1082153320312500, -0.0966186523437500, -0.0886230468750000, -0.0760803222656250,
+-0.1019592285156250, -0.0835876464843750, -0.0658569335937500, -0.0571899414062500,
+-0.0703430175781250, -0.0643005371093750, -0.0600585937500000, -0.0469055175781250,
+-0.0344238281250000, -0.0170898437500000, -0.0485229492187500, -0.0346069335937500,
+-0.0145874023437500, -0.0016479492187500, -0.0196533203125000, 0.0058593750000000,
+-0.0089416503906250, -0.0022277832031250, 0.0161743164062500, 0.0046081542968750,
+0.0015869140625000, 0.0076293945312500, 0.0382385253906250, -0.0012817382812500,
+0.0333862304687500, -0.0021667480468750, 0.0459594726562500, -0.0002441406250000,
+0.0740356445312500, -0.0039062500000000, 0.0587463378906250, -0.0033874511718750,
+0.0968017578125000, 0.0079650878906250, 0.0928039550781250, -0.0002136230468750,
+0.1028442382812500, 0.0161132812500000, 0.1293029785156250, 0.0357360839843750,
+0.1151733398437500, 0.0208129882812500, 0.1493530273437500, 0.0592041015625000,
+0.1473693847656250, 0.0612182617187500, 0.1484069824218750, 0.0561523437500000,
+0.1720581054687500, 0.0736389160156250, 0.1639404296875000, 0.0711669921875000,
+0.1485900878906250, 0.0509338378906250, 0.1696777343750000, 0.0585937500000000,
+0.1324157714843750, 0.0461425781250000, 0.0959167480468750, 0.0223388671875000,
+0.1296691894531250, 0.0334167480468750, 0.0239257812500000, 0.0150146484375000,
+0.0398864746093750, 0.0106201171875000, 0.0238952636718750, 0.0122985839843750,
+-0.0519714355468750, 0.0061035156250000, -0.0179748535156250, 0.0065612792968750,
+-0.0462646484375000, 0.0097351074218750, -0.0621643066406250, 0.0074462890625000,
+-0.0434570312500000, 0.0090332031250000, -0.0381469726562500, 0.0134887695312500,
+-0.0491638183593750, 0.0113525390625000, -0.0221557617187500, 0.0115051269531250,
+-0.0212402343750000, 0.0121459960937500, -0.0241394042968750, 0.0110778808593750,
+-0.0126953125000000, 0.0088806152343750, -0.0140991210937500, 0.0090026855468750,
+-0.0171508789062500, 0.0141601562500000, -0.0170288085937500, 0.0102233886718750,
+-0.0143737792968750, 0.0178527832031250, -0.0151672363281250, 0.0269775390625000,
+-0.0171203613281250, 0.0212707519531250, -0.0065612792968750, 0.0433349609375000,
+-0.0087890625000000, 0.0417175292968750, -0.0062866210937500, 0.0475158691406250,
+0.0018005371093750, 0.0639038085937500, -0.0019226074218750, 0.0583801269531250,
+0.0044250488281250, 0.0722045898437500, 0.0036621093750000, 0.0737304687500000,
+0.0064697265625000, 0.0746459960937500, 0.0112609863281250, 0.0818481445312500,
+0.0072937011718750, 0.0799255371093750, 0.0165710449218750, 0.0809936523437500,
+0.0166015625000000, 0.0824279785156250, 0.0149536132812500, 0.0793151855468750,
+0.0200500488281250, 0.0761413574218750, 0.0195617675781250, 0.0770263671875000,
+0.0110473632812500, 0.0733642578125000, 0.0152893066406250, 0.0713500976562500,
+0.0095214843750000, 0.0711975097656250, -0.0010375976562500, 0.0699462890625000,
+0.0050354003906250},
+{-0.0961303710937500, -0.1283874511718750, -0.0844421386718750, -0.1358642578125000,
+-0.1016235351562500, -0.1091918945312500, -0.1016235351562500, -0.1133117675781250,
+-0.1033935546875000, -0.1016540527343750, -0.1128234863281250, -0.0785217285156250,
+-0.1074523925781250, -0.0904541015625000, -0.1177062988281250, -0.0618591308593750,
+-0.1186218261718750, -0.0642395019531250, -0.1149597167968750, -0.0502319335937500,
+-0.1195678710937500, -0.0280456542968750, -0.1202392578125000, -0.0417480468750000,
+-0.1032409667968750, 0.0052185058593750, -0.1109008789062500, 0.0056152343750000,
+-0.0981445312500000, 0.0013122558593750, -0.0787353515625000, 0.0259704589843750,
+-0.0916748046875000, 0.0238342285156250, -0.0575561523437500, 0.0031433105468750,
+-0.0624084472656250, 0.0082397460937500, -0.0526428222656250, 0.0086364746093750,
+-0.0282897949218750, -0.0071105957031250, -0.0421142578125000, -0.0047302246093750,
+-0.0085144042968750, 0.0093688964843750, -0.0128479003906250, 0.0004577636718750,
+-0.0010681152343750, 0.0141601562500000, 0.0242919921875000, 0.0339660644531250,
+0.0097656250000000, 0.0205383300781250, 0.0476684570312500, 0.0533447265625000,
+0.0429382324218750, 0.0554809570312500, 0.0535888671875000, 0.0487976074218750,
+0.0808105468750000, 0.0628662109375000, 0.0659790039062500, 0.0621032714843750,
+0.1005249023437500, 0.0364379882812500, 0.0969543457031250, 0.0450134277343750,
+0.1052246093750000, 0.0325012207031250, 0.1296081542968750, 0.0052490234375000,
+0.1170654296875000, 0.0180969238281250, 0.1434020996093750, -0.0001831054687500,
+0.1442565917968750, -0.0060424804687500, 0.1393127441406250, -0.0024719238281250,
+0.1541137695312500, -0.0053100585937500, 0.1522827148437500, -0.0061340332031250,
+0.1224365234375000, -0.0051879882812500, 0.1439208984375000, -0.0054626464843750,
+0.1084289550781250, -0.0052185058593750, 0.0632934570312500, -0.0058288574218750,
+0.0993652343750000, -0.0067443847656250, 0.0075073242187500, 0.0000915527343750,
+0.0174560546875000, -0.0038146972656250, 0.0092773437500000, 0.0040283203125000,
+-0.0510559082031250, 0.0131835937500000, -0.0258483886718750, 0.0067138671875000,
+-0.0417480468750000, 0.0325317382812500, -0.0558776855468750, 0.0274658203125000,
+-0.0400390625000000, 0.0383605957031250, -0.0302429199218750, 0.0609436035156250,
+-0.0409545898437500, 0.0485534667968750, -0.0192260742187500, 0.0741271972656250,
+-0.0169982910156250, 0.0755615234375000, -0.0202331542968750, 0.0725097656250000,
+-0.0127868652343750, 0.0841369628906250, -0.0130920410156250, 0.0805053710937500,
+-0.0153808593750000, 0.0676574707031250, -0.0163269042968750, 0.0737304687500000,
+-0.0127258300781250, 0.0597229003906250, -0.0117492675781250, 0.0407104492187500,
+-0.0146179199218750, 0.0486755371093750, -0.0039062500000000, 0.0150146484375000,
+-0.0053405761718750, 0.0160522460937500, -0.0039062500000000, 0.0038757324218750,
+0.0029296875000000, -0.0224609375000000, -0.0000305175781250, -0.0137634277343750,
+0.0052795410156250, -0.0419616699218750, 0.0042114257812500, -0.0458679199218750,
+0.0066528320312500, -0.0509948730468750, 0.0113220214843750, -0.0671997070312500,
+0.0077209472656250, -0.0637512207031250, 0.0137329101562500, -0.0737609863281250,
+0.0148620605468750, -0.0763549804687500, 0.0113220214843750, -0.0792236328125000,
+0.0126647949218750, -0.0851135253906250, 0.0143127441406250, -0.0835571289062500,
+0.0023193359375000, -0.0878601074218750, 0.0053710937500000, -0.0916137695312500,
+0.0019836425781250, -0.0884704589843750, -0.0090942382812500, -0.0879211425781250,
+-0.0036010742187500, -0.0915222167968750, -0.0043334960937500, -0.0784301757812500,
+-0.0104980468750000, -0.0829467773437500, 0.0001525878906250, -0.0775756835937500,
+0.0093383789062500, -0.0659179687500000, 0.0011596679687500, -0.0734558105468750,
+0.0253601074218750, -0.0566406250000000, 0.0268554687500000, -0.0609741210937500,
+0.0218811035156250, -0.0522460937500000, 0.0329284667968750, -0.0375366210937500,
+0.0329284667968750, -0.0471496582031250, 0.0092773437500000, -0.0141296386718750,
+0.0156555175781250, -0.0204772949218750, 0.0095214843750000, -0.0104980468750000,
+-0.0136413574218750, 0.0159301757812500, -0.0058593750000000, 0.0016479492187500,
+-0.0003051757812500, 0.0259094238281250, -0.0085449218750000, 0.0262145996093750,
+-0.0023498535156250, 0.0278015136718750, 0.0141906738281250, 0.0401000976562500,
+0.0083007812500000, 0.0342712402343750, -0.0056457519531250, 0.0467529296875000,
+0.0056762695312500, 0.0452575683593750, -0.0115966796875000, 0.0466003417968750,
+-0.0402832031250000, 0.0561218261718750, -0.0248107910156250, 0.0516052246093750,
+-0.0365600585937500, 0.0491943359375000, -0.0496520996093750, 0.0581970214843750,
+-0.0323486328125000, 0.0426025390625000, -0.0182495117187500, 0.0290832519531250,
+-0.0301208496093750, 0.0422058105468750, -0.0033264160156250, -0.0028991699218750,
+-0.0007019042968750, 0.0040588378906250, -0.0007629394531250, -0.0081787109375000,
+0.0090332031250000, -0.0428771972656250, 0.0060424804687500, -0.0265197753906250,
+0.0195312500000000, -0.0574035644531250, 0.0145263671875000, -0.0609436035156250,
+0.0234985351562500, -0.0568847656250000, 0.0396118164062500, -0.0701904296875000,
+0.0305175781250000, -0.0692138671875000, 0.0486145019531250, -0.0559692382812500,
+0.0475769042968750, -0.0589294433593750, 0.0556945800781250, -0.0603942871093750,
+0.0660705566406250, -0.0487670898437500, 0.0576477050781250, -0.0491638183593750,
+0.1008605957031250, -0.0729980468750000, 0.0879516601562500, -0.0649414062500000,
+0.1102905273437500, -0.0746154785156250, 0.1571655273437500, -0.1005249023437500,
+0.1320190429687500, -0.0913696289062500, 0.1640014648437500, -0.0998229980468750,
+0.1801452636718750, -0.1030883789062500, 0.1527404785156250, -0.1072692871093750,
+0.1478576660156250, -0.1129150390625000, 0.1626586914062500, -0.1078186035156250,
+0.0805664062500000, -0.1132202148437500, 0.0980224609375000, -0.1263732910156250,
+0.0659179687500000, -0.1004333496093750, -0.0054016113281250, -0.0804138183593750,
+0.0300598144531250, -0.1030883789062500, -0.0456848144531250, -0.0475769042968750,
+-0.0483093261718750, -0.0418395996093750, -0.0534362792968750, -0.0520019531250000,
+-0.0948486328125000, -0.0366821289062500, -0.0799865722656250, -0.0353698730468750,
+-0.0962219238281250, -0.0347595214843750, -0.1032409667968750, -0.0482482910156250,
+-0.0980224609375000, -0.0219421386718750, -0.0977172851562500, 0.0019836425781250,
+-0.1014099121093750, -0.0207214355468750, -0.1060791015625000, 0.0410766601562500,
+-0.0971679687500000, 0.0383605957031250, -0.1165466308593750, 0.0448608398437500,
+-0.1334533691406250, 0.0816955566406250, -0.1175537109375000, 0.0670471191406250,
+-0.1720886230468750, 0.0870666503906250, -0.1674194335937500, 0.0928344726562500,
+-0.1731262207031250, 0.0865173339843750, -0.2104187011718750, 0.0909423828125000,
+-0.1968688964843750, 0.0928955078125000, -0.1952514648437500, 0.0778808593750000,
+-0.2086486816406250, 0.0800476074218750, -0.1876220703125000, 0.0796508789062500,
+-0.1679687500000000, 0.0693054199218750, -0.1828002929687500, 0.0718078613281250,
+-0.1306457519531250, 0.0767822265625000, -0.1390380859375000, 0.0758972167968750,
+-0.1207885742187500, 0.0725097656250000, -0.0464782714843750, 0.0726623535156250,
+0.0785522460937500, 0.0678710937500000, 0.1388549804687500, 0.0690307617187500,
+0.1553039550781250, 0.0703735351562500, 0.1174011230468750, 0.0621948242187500,
+0.1020507812500000, 0.0644226074218750, 0.1303405761718750, 0.0700988769531250,
+0.0500183105468750, 0.0205078125000000, 0.0500183105468750, 0.0390319824218750,
+0.0546569824218750, 0.0103149414062500, 0.0123901367187500, -0.0437011718750000,
+0.0209045410156250, -0.0142822265625000, 0.0473632812500000, -0.0801086425781250,
+0.0324707031250000, -0.0773010253906250, 0.0454711914062500, -0.0908508300781250,
+0.0799255371093750, -0.1316528320312500, 0.0657958984375000, -0.1100769042968750,
+0.0696105957031250, -0.1612243652343750, 0.0812072753906250, -0.1625366210937500,
+0.0692443847656250, -0.1577758789062500, 0.0535888671875000, -0.1865539550781250,
+0.0625915527343750, -0.1809692382812500, 0.0582275390625000, -0.1443176269531250,
+0.0513916015625000, -0.1641540527343750, 0.0514831542968750, -0.1371154785156250,
+0.0627441406250000, -0.0862731933593750, 0.0591735839843750, -0.1124572753906250,
+0.0052490234375000, -0.0761718750000000, 0.0390930175781250, -0.0641174316406250,
+-0.0116271972656250, -0.0771179199218750, -0.0934448242187500, -0.0775756835937500,
+-0.0419006347656250, -0.0719299316406250, -0.1327209472656250, -0.0717773437500000,
+-0.1503601074218750, -0.0844421386718750, -0.1272583007812500, -0.0617370605468750,
+-0.1518554687500000, -0.0404052734375000, -0.1560974121093750, -0.0584411621093750,
+-0.1047058105468750, -0.0067443847656250, -0.1222534179687500, -0.0096435546875000,
+-0.0987854003906250, 0.0003356933593750, -0.0476989746093750, 0.0306701660156250,
+-0.0776367187500000, 0.0164489746093750, -0.0361633300781250, 0.0619201660156250,
+-0.0218811035156250, 0.0519409179687500, -0.0480957031250000, 0.0730895996093750,
+-0.0456237792968750, 0.1181640625000000, -0.0303344726562500, 0.0952148437500000,
+-0.1009521484375000, 0.1347351074218750, -0.0874023437500000, 0.1465454101562500,
+-0.0984191894531250, 0.1293334960937500, -0.1534423828125000, 0.1338195800781250,
+-0.1312255859375000, 0.1416931152343750, -0.1356201171875000, 0.1004028320312500,
+-0.1553649902343750, 0.1069641113281250, -0.1293029785156250, 0.0964355468750000,
+-0.1095581054687500, 0.0664672851562500, -0.1296081542968750, 0.0792541503906250,
+-0.0529785156250000, 0.0470886230468750, -0.0703125000000000, 0.0517272949218750,
+-0.0421142578125000, 0.0391845703125000, 0.0228881835937500, 0.0121765136718750,
+-0.0132446289062500, 0.0270080566406250, 0.0587158203125000, -0.0040588378906250,
+0.0590209960937500, -0.0050964355468750, 0.0633850097656250, -0.0065612792968750,
+0.1029663085937500, -0.0214233398437500, 0.0855712890625000, -0.0156860351562500,
+0.1011352539062500, -0.0307006835937500, 0.1098632812500000, -0.0285949707031250,
+0.0984497070312500, -0.0315551757812500, 0.0973205566406250, -0.0454101562500000,
+0.1044921875000000, -0.0401611328125000, 0.0747070312500000, -0.0333557128906250,
+0.0840148925781250, -0.0426940917968750, 0.0693969726562500, -0.0289611816406250,
+0.0429382324218750, -0.0099487304687500, 0.0617675781250000, -0.0214233398437500,
+0.0171508789062500, -0.0004882812500000, 0.0238342285156250, 0.0039367675781250,
+0.0130615234375000, 0.0008544921875000, -0.0187377929687500, 0.0046997070312500,
+-0.0009765625000000, 0.0046691894531250, -0.0373840332031250, 0.0126342773437500,
+-0.0363159179687500, 0.0078735351562500, -0.0416259765625000, 0.0156860351562500,
+-0.0642395019531250, 0.0299682617187500, -0.0543212890625000, 0.0227050781250000,
+-0.0713195800781250, 0.0285644531250000, -0.0722351074218750, 0.0366821289062500,
+-0.0742492675781250, 0.0243530273437500, -0.0838317871093750, 0.0141601562500000,
+-0.0809020996093750, 0.0225219726562500, -0.0916442871093750, -0.0014953613281250,
+-0.0921020507812500, -0.0041198730468750, -0.0940856933593750, -0.0010375976562500,
+-0.1014709472656250, -0.0107727050781250, -0.0997924804687500, -0.0105590820312500,
+-0.1014709472656250, -0.0000915527343750, -0.1041564941406250, -0.0038146972656250,
+-0.1014404296875000, 0.0000610351562500, -0.0991516113281250, 0.0104370117187500,
+-0.1016235351562500, 0.0056152343750000, -0.0955505371093750, 0.0103149414062500,
+-0.0962829589843750},
+{0.0200805664062500, -0.1491088867187500, 0.0488281250000000, -0.1536560058593750,
+-0.0033264160156250, -0.1438598632812500, -0.0695190429687500, -0.1605834960937500,
+-0.0188903808593750, -0.1614990234375000, -0.1239624023437500, -0.1174621582031250,
+-0.1365356445312500, -0.1321716308593750, -0.1119995117187500, -0.1116027832031250,
+-0.1476745605468750, -0.0642395019531250, -0.1522521972656250, -0.0866699218750000,
+-0.0946655273437500, -0.0627441406250000, -0.1069641113281250, -0.0518188476562500,
+-0.0971374511718750, -0.0602111816406250, -0.0526428222656250, -0.0640258789062500,
+-0.0710754394531250, -0.0601806640625000, -0.0668029785156250, -0.0488891601562500,
+-0.0494384765625000, -0.0616149902343750, -0.0762329101562500, -0.0413818359375000,
+-0.0983886718750000, -0.0155944824218750, -0.0759277343750000, -0.0329589843750000,
+-0.1331176757812500, 0.0125427246093750, -0.1331481933593750, 0.0101318359375000,
+-0.1286010742187500, 0.0200805664062500, -0.1603698730468750, 0.0476684570312500,
+-0.1544799804687500, 0.0343933105468750, -0.1244201660156250, 0.0782470703125000,
+-0.1477661132812500, 0.0710144042968750, -0.1166992187500000, 0.0854797363281250,
+-0.0710449218750000, 0.1245117187500000, -0.1039733886718750, 0.1065063476562500,
+-0.0320434570312500, 0.1253356933593750, -0.0359191894531250, 0.1388549804687500,
+-0.0263366699218750, 0.1197509765625000, 0.0183105468750000, 0.1099243164062500,
+-0.0020751953125000, 0.1215209960937500, 0.0292968750000000, 0.0820007324218750,
+0.0331420898437500, 0.0853881835937500, 0.0321655273437500, 0.0763854980468750,
+0.0447082519531250, 0.0509643554687500, 0.0401611328125000, 0.0627136230468750,
+0.0478820800781250, 0.0297241210937500, 0.0480651855468750, 0.0347595214843750,
+0.0486145019531250, 0.0240783691406250, 0.0543518066406250, -0.0028076171875000,
+0.0530395507812500, 0.0108947753906250, 0.0502319335937500, -0.0136108398437500,
+0.0551757812500000, -0.0153503417968750, 0.0482482910156250, -0.0172729492187500,
+0.0413818359375000, -0.0287170410156250, 0.0480041503906250, -0.0234680175781250,
+0.0248107910156250, -0.0379638671875000, 0.0301208496093750, -0.0380859375000000,
+0.0213623046875000, -0.0357055664062500, 0.0017700195312500, -0.0446777343750000,
+0.0128784179687500, -0.0439453125000000, -0.0112915039062500, -0.0249938964843750,
+-0.0096130371093750, -0.0317993164062500, -0.0140075683593750, -0.0229492187500000,
+-0.0288696289062500, -0.0015258789062500, -0.0205688476562500, -0.0104064941406250,
+-0.0373535156250000, -0.0027770996093750, -0.0347595214843750, 0.0015563964843750,
+-0.0390014648437500, 0.0002136230468750, -0.0510559082031250, -0.0017395019531250,
+-0.0432128906250000, -0.0014953613281250, -0.0571594238281250, 0.0110473632812500,
+-0.0560607910156250, 0.0060729980468750, -0.0576782226562500, 0.0132751464843750,
+-0.0660095214843750, 0.0291442871093750, -0.0610656738281250, 0.0222473144531250,
+-0.0678405761718750, 0.0280151367187500, -0.0662841796875000, 0.0328979492187500,
+-0.0686035156250000, 0.0255432128906250, -0.0734558105468750, 0.0203247070312500,
+-0.0691528320312500, 0.0239257812500000, -0.0800781250000000, 0.0116882324218750,
+-0.0780944824218750, 0.0115661621093750, -0.0805969238281250, 0.0104370117187500,
+-0.0890197753906250, 0.0037231445312500, -0.0845947265625000, 0.0061950683593750,
+-0.0904541015625000, 0.0035095214843750, -0.0910644531250000, 0.0031127929687500,
+-0.0900573730468750, 0.0031433105468750, -0.0914611816406250, 0.0024719238281250,
+-0.0905761718750000, 0.0031127929687500, -0.0927734375000000, 0.0000000000000000,
+-0.0907592773437500, 0.0016784667968750, -0.0930786132812500, -0.0016174316406250,
+-0.0965881347656250, -0.0056762695312500, -0.0936279296875000, -0.0022277832031250,
+-0.1002502441406250, -0.0112915039062500, -0.1001892089843750, -0.0104980468750000,
+-0.1001586914062500, -0.0114746093750000, -0.1038818359375000, -0.0172119140625000,
+-0.1024780273437500, -0.0148315429687500, -0.1012268066406250, -0.0173645019531250,
+-0.1030883789062500, -0.0178222656250000, -0.1007995605468750, -0.0180664062500000,
+-0.0983886718750000, -0.0190734863281250, -0.1003417968750000, -0.0182189941406250,
+-0.0917968750000000, -0.0202636718750000, -0.0950622558593750, -0.0209960937500000,
+-0.0856933593750000, -0.0181884765625000, -0.0751037597656250, -0.0189208984375000,
+-0.0817871093750000, -0.0203857421875000, -0.0508728027343750, -0.0053100585937500,
+-0.0559692382812500, -0.0131530761718750, -0.0420532226562500, 0.0009155273437500,
+-0.0147399902343750, 0.0189514160156250, -0.0273742675781250, 0.0049438476562500,
+0.0078430175781250, 0.0491027832031250, 0.0084533691406250, 0.0392761230468750,
+0.0135498046875000, 0.0555419921875000, 0.0360717773437500, 0.0927429199218750,
+0.0286560058593750, 0.0705871582031250, 0.0349731445312500, 0.1129150390625000,
+0.0420532226562500, 0.1159973144531250, 0.0304870605468750, 0.1098937988281250,
+0.0246887207031250, 0.1295471191406250, 0.0316467285156250, 0.1271667480468750,
+-0.0055847167968750, 0.1043701171875000, 0.0002136230468750, 0.1162414550781250,
+-0.0109558105468750, 0.1017761230468750, -0.0431213378906250, 0.0734863281250000,
+-0.0306396484375000, 0.0896606445312500, -0.0424499511718750, 0.0658264160156250,
+-0.0502624511718750, 0.0615539550781250, -0.0426940917968750, 0.0640563964843750,
+-0.0406188964843750, 0.0586242675781250, -0.0437316894531250, 0.0592041015625000,
+-0.0218505859375000, 0.0445251464843750, -0.0287475585937500, 0.0559082031250000,
+-0.0172729492187500, 0.0350952148437500, 0.0074768066406250, 0.0097656250000000,
+-0.0047607421875000, 0.0304260253906250, 0.0083007812500000, -0.0209655761718750,
+0.0129089355468750, -0.0201110839843750, 0.0132446289062500, -0.0219421386718750,
+0.0148315429687500, -0.0494384765625000, 0.0115966796875000, -0.0407409667968750,
+0.0249328613281250, -0.0539550781250000, 0.0257873535156250, -0.0502319335937500,
+0.0213623046875000, -0.0650939941406250, 0.0249633789062500, -0.0798034667968750,
+0.0273742675781250, -0.0657653808593750, 0.0206604003906250, -0.1184692382812500,
+0.0174865722656250, -0.1108703613281250, 0.0225524902343750, -0.1173400878906250,
+0.0250854492187500, -0.1555786132812500, 0.0214538574218750, -0.1407775878906250,
+0.0282287597656250, -0.1443481445312500, 0.0307006835937500, -0.1564941406250000,
+0.0274658203125000, -0.1433410644531250, 0.0271606445312500, -0.1300659179687500,
+0.0292358398437500, -0.1398620605468750, 0.0252990722656250, -0.1158752441406250,
+0.0257873535156250, -0.1192016601562500, 0.0238647460937500, -0.1071472167968750,
+0.0214538574218750, -0.0841369628906250, 0.0211181640625000, -0.0867309570312500,
+0.0049438476562500, -0.0408020019531250, 0.0128479003906250, -0.0568237304687500,
+0.0064086914062500, -0.0481567382812500, -0.0078735351562500, -0.0086364746093750,
+0.0018920898437500, -0.0231018066406250, -0.0253601074218750, 0.0055236816406250,
+-0.0188598632812500, 0.0075073242187500, -0.0299377441406250, 0.0144653320312500,
+-0.0529479980468750, 0.0325317382812500, -0.0379943847656250, 0.0227661132812500,
+-0.0701904296875000, 0.0375061035156250, -0.0685424804687500, 0.0466308593750000,
+-0.0715332031250000, 0.0306091308593750, -0.0910034179687500, 0.0214538574218750,
+-0.0829162597656250, 0.0336303710937500, -0.0942077636718750, 0.0114440917968750,
+-0.0951232910156250, 0.0024414062500000, -0.0962524414062500, 0.0166931152343750,
+-0.1026000976562500, 0.0220947265625000, -0.0994262695312500, 0.0130004882812500,
+-0.1022949218750000, 0.0284118652343750, -0.1057739257812500, 0.0343017578125000,
+-0.0995178222656250, 0.0268554687500000, -0.0971984863281250, 0.0252380371093750,
+-0.1024780273437500, 0.0285949707031250, -0.0811462402343750, 0.0180053710937500,
+-0.0874633789062500, 0.0198974609375000, -0.0772705078125000, 0.0144348144531250,
+-0.0575561523437500, 0.0049743652343750, -0.0690307617187500, 0.0111999511718750,
+-0.0395202636718750, -0.0012817382812500, -0.0440979003906250, -0.0009155273437500,
+-0.0341491699218750, -0.0030822753906250, -0.0124206542968750, -0.0104980468750000,
+-0.0255432128906250, -0.0057067871093750, 0.0099182128906250, -0.0124816894531250,
+0.0035705566406250, -0.0154724121093750, 0.0162353515625000, -0.0091857910156250,
+0.0448913574218750, -0.0082702636718750, 0.0283508300781250, -0.0119628906250000,
+0.0642700195312500, 0.0066833496093750, 0.0612487792968750, 0.0017700195312500,
+0.0680541992187500, 0.0124511718750000, 0.0913696289062500, 0.0320739746093750,
+0.0793151855468750, 0.0208740234375000, 0.1048583984375000, 0.0396118164062500,
+0.1028747558593750, 0.0461120605468750, 0.1065368652343750, 0.0349731445312500,
+0.1236267089843750, 0.0357055664062500, 0.1152648925781250, 0.0405273437500000,
+0.1239013671875000, 0.0066223144531250, 0.1289978027343750, 0.0128784179687500,
+0.1170043945312500, -0.0039978027343750, 0.1159057617187500, -0.0354614257812500,
+0.1235351562500000, -0.0212097167968750, 0.0788574218750000, -0.0602722167968750,
+0.0960083007812500, -0.0609436035156250, 0.0694274902343750, -0.0672912597656250,
+0.0223999023437500, -0.0912475585937500, 0.0536804199218750, -0.0822448730468750,
+-0.0120849609375000, -0.0970153808593750, -0.0123596191406250, -0.1001586914062500,
+-0.0110778808593750, -0.0970153808593750, -0.0459289550781250, -0.1012878417968750,
+-0.0353393554687500, -0.1006164550781250, -0.0320739746093750, -0.0927429199218750,
+-0.0412292480468750, -0.0954284667968750, -0.0315856933593750, -0.0908203125000000,
+-0.0179138183593750, -0.0830383300781250, -0.0261230468750000, -0.0873718261718750,
+-0.0148010253906250, -0.0755615234375000, -0.0114440917968750, -0.0773620605468750,
+-0.0144653320312500, -0.0749816894531250, -0.0130920410156250, -0.0671081542968750,
+-0.0119628906250000, -0.0719909667968750, -0.0123291015625000, -0.0638122558593750,
+-0.0144958496093750, -0.0652160644531250, -0.0106506347656250, -0.0641174316406250,
+-0.0077819824218750, -0.0593261718750000, -0.0110168457031250, -0.0630187988281250,
+-0.0001831054687500, -0.0554199218750000, -0.0025939941406250, -0.0578918457031250,
+0.0025024414062500, -0.0553894042968750, 0.0118408203125000, -0.0485839843750000,
+0.0064086914062500, -0.0533142089843750, 0.0224914550781250, -0.0474548339843750,
+0.0213012695312500, -0.0467529296875000, 0.0233764648437500, -0.0495910644531250,
+0.0351562500000000, -0.0487670898437500, 0.0302124023437500, -0.0484619140625000,
+0.0263977050781250, -0.0522155761718750, 0.0342407226562500, -0.0517883300781250,
+0.0216674804687500, -0.0514831542968750, 0.0071105957031250, -0.0532531738281250,
+0.0178222656250000, -0.0521850585937500, -0.0058898925781250, -0.0513000488281250,
+-0.0095825195312500, -0.0521240234375000, -0.0028686523437500, -0.0502319335937500,
+-0.0083618164062500, -0.0493774414062500, -0.0105590820312500, -0.0510864257812500,
+-0.0005187988281250, -0.0422363281250000, 0.0033264160156250, -0.0470275878906250,
+-0.0073852539062500, -0.0395507812500000, -0.0100402832031250, -0.0296020507812500,
+-0.0025634765625000, -0.0381469726562500, -0.0325622558593750, -0.0131530761718750,
+-0.0330200195312500, -0.0211791992187500, -0.0286560058593750, -0.0084838867187500,
+-0.0434265136718750, 0.0140686035156250, -0.0428466796875000, -0.0027465820312500,
+-0.0220031738281250, 0.0379943847656250, -0.0270385742187500, 0.0328063964843750,
+-0.0193481445312500, 0.0349121093750000, -0.0007934570312500, 0.0620727539062500,
+-0.0082397460937500, 0.0504760742187500, 0.0058593750000000, 0.0494995117187500,
+0.0050354003906250, 0.0596923828125000, 0.0113830566406250, 0.0444335937500000,
+0.0225830078125000},
+{-0.0252990722656250, 0.0202331542968750, -0.0032958984375000, 0.0437316894531250,
+-0.0176391601562500, 0.0331420898437500, 0.0190429687500000, 0.0414123535156250,
+0.0128784179687500, 0.0498352050781250, 0.0234375000000000, 0.0346374511718750,
+0.0509338378906250, 0.0271606445312500, 0.0351562500000000, 0.0371704101562500,
+0.0689086914062500, -0.0057067871093750, 0.0659790039062500, 0.0009460449218750,
+0.0716247558593750, -0.0133972167968750, 0.0934143066406250, -0.0491943359375000,
+0.0822143554687500, -0.0339660644531250, 0.1025390625000000, -0.0635986328125000,
+0.1019287109375000, -0.0671081542968750, 0.1027832031250000, -0.0672912597656250,
+0.1151123046875000, -0.0798645019531250, 0.1100158691406250, -0.0758972167968750,
+0.1107482910156250, -0.0834655761718750, 0.1179504394531250, -0.0830078125000000,
+0.1040039062500000, -0.0849914550781250, 0.0957031250000000, -0.0906982421875000,
+0.1066284179687500, -0.0882568359375000, 0.0590515136718750, -0.0940246582031250,
+0.0729370117187500, -0.0942077636718750, 0.0518798828125000, -0.0950012207031250,
+0.0076293945312500, -0.0988464355468750, 0.0343627929687500, -0.0975036621093750,
+-0.0170898437500000, -0.0975646972656250, -0.0203552246093750, -0.0996704101562500,
+-0.0158691406250000, -0.0959472656250000, -0.0392761230468750, -0.0929565429687500,
+-0.0341796875000000, -0.0952148437500000, -0.0247497558593750, -0.0836181640625000,
+-0.0309753417968750, -0.0853881835937500, -0.0252990722656250, -0.0797119140625000,
+-0.0131225585937500, -0.0695495605468750, -0.0191650390625000, -0.0741271972656250,
+-0.0119323730468750, -0.0583801269531250, -0.0101623535156250, -0.0589294433593750,
+-0.0101013183593750, -0.0527038574218750, -0.0083923339843750, -0.0402832031250000,
+-0.0089416503906250, -0.0458068847656250, -0.0007019042968750, -0.0324707031250000,
+-0.0037536621093750, -0.0315551757812500, 0.0012512207031250, -0.0299682617187500,
+0.0111083984375000, -0.0228881835937500, 0.0056152343750000, -0.0254211425781250,
+0.0124206542968750, -0.0196838378906250, 0.0152587890625000, -0.0197448730468750,
+0.0099792480468750, -0.0173034667968750, 0.0082397460937500, -0.0126647949218750,
+0.0108642578125000, -0.0148620605468750, 0.0022277832031250, -0.0094604492187500,
+0.0003051757812500, -0.0076293945312500, 0.0046081542968750, -0.0104370117187500,
+0.0034790039062500, -0.0093994140625000, 0.0007629394531250, -0.0074157714843750,
+0.0124206542968750, -0.0180664062500000, 0.0143737792968750, -0.0132446289062500,
+0.0068664550781250, -0.0184936523437500, 0.0101623535156250, -0.0288391113281250,
+0.0143432617187500, -0.0215454101562500, -0.0179138183593750, -0.0382385253906250,
+-0.0118103027343750, -0.0347290039062500, -0.0176391601562500, -0.0394592285156250,
+-0.0450744628906250, -0.0519104003906250, -0.0358886718750000, -0.0438842773437500,
+-0.0302429199218750, -0.0610351562500000, -0.0407409667968750, -0.0588684082031250,
+-0.0277404785156250, -0.0617065429687500, -0.0090942382812500, -0.0740661621093750,
+-0.0186462402343750, -0.0675659179687500, 0.0014648437500000, -0.0735168457031250,
+0.0022888183593750, -0.0802001953125000, 0.0065612792968750, -0.0690917968750000,
+0.0176391601562500, -0.0656433105468750, 0.0119018554687500, -0.0750427246093750,
+0.0296936035156250, -0.0350952148437500, 0.0286865234375000, -0.0468750000000000,
+0.0312805175781250, -0.0298156738281250, 0.0418090820312500, 0.0085144042968750,
+0.0382995605468750, -0.0122070312500000, 0.0513000488281250, 0.0256042480468750,
+0.0449829101562500, 0.0288696289062500, 0.0607299804687500, 0.0251159667968750,
+0.0783996582031250, 0.0419921875000000, 0.0637207031250000, 0.0386962890625000,
+0.1106262207031250, 0.0295410156250000, 0.1081848144531250, 0.0371704101562500,
+0.1084289550781250, 0.0250244140625000, 0.1387023925781250, 0.0102539062500000,
+0.1306762695312500, 0.0211181640625000, 0.1104431152343750, -0.0118408203125000,
+0.1280517578125000, -0.0085754394531250, 0.1006774902343750, -0.0138854980468750,
+0.0651245117187500, -0.0392150878906250, 0.0897216796875000, -0.0298461914062500,
+0.0257568359375000, -0.0273742675781250, 0.0291442871093750, -0.0376892089843750,
+0.0168151855468750, -0.0231323242187500, -0.0255737304687500, -0.0052185058593750,
+-0.0068054199218750, -0.0161437988281250, -0.0383300781250000, 0.0062561035156250,
+-0.0418090820312500, 0.0122680664062500, -0.0418090820312500, -0.0004577636718750,
+-0.0550231933593750, 0.0047607421875000, -0.0511169433593750, 0.0099487304687500,
+-0.0639343261718750, -0.0456848144531250, -0.0592956542968750, -0.0320434570312500,
+-0.0715332031250000, -0.0498352050781250, -0.0865478515625000, -0.1002502441406250,
+-0.0750122070312500, -0.0794982910156250, -0.1119384765625000, -0.1048889160156250,
+-0.1090087890625000, -0.1107482910156250, -0.1134948730468750, -0.1100769042968750,
+-0.1385498046875000, -0.1221008300781250, -0.1293945312500000, -0.1173095703125000,
+-0.1337585449218750, -0.1045837402343750, -0.1408691406250000, -0.1218566894531250,
+-0.1307067871093750, -0.0931091308593750, -0.1243896484375000, -0.0558776855468750,
+-0.1304321289062500, -0.0827636718750000, -0.1010742187500000, -0.0399169921875000,
+-0.1120910644531250, -0.0281677246093750, -0.0863647460937500, -0.0437927246093750,
+-0.0550842285156250, -0.0423583984375000, -0.0781555175781250, -0.0356750488281250,
+0.0112304687500000, -0.0429077148437500, -0.0042419433593750, -0.0555114746093750,
+0.0224609375000000, -0.0333557128906250, 0.0980529785156250, -0.0167541503906250,
+0.0628967285156250, -0.0339050292968750, 0.1038513183593750, 0.0202026367187500,
+0.1250000000000000, 0.0164794921875000, 0.0943603515625000, 0.0207824707031250,
+0.0849914550781250, 0.0535888671875000, 0.1032409667968750, 0.0420532226562500,
+0.0437927246093750, 0.0540771484375000, 0.0425720214843750, 0.0555725097656250,
+0.0513610839843750, 0.0577392578125000, 0.0188293457031250, 0.0704040527343750,
+0.0774230957031250, 0.0460205078125000, 0.0827331542968750, 0.0557556152343750,
+0.0911560058593750, 0.0527038574218750, 0.0814819335937500, 0.0599975585937500,
+0.0704956054687500, 0.0660095214843750, 0.0772399902343750, 0.0603027343750000,
+0.0690307617187500, 0.0783691406250000, 0.0678100585937500, 0.0801696777343750,
+0.0632629394531250, 0.0739746093750000, 0.0621948242187500, 0.0799865722656250,
+0.0653686523437500, 0.0827636718750000, 0.0253295898437500, 0.0636901855468750,
+0.0418701171875000, 0.0658874511718750, 0.0135803222656250, 0.0646972656250000,
+-0.0352172851562500, 0.0528259277343750, -0.0071411132812500, 0.0558166503906250,
+-0.0635986328125000, 0.0539855957031250, -0.0693054199218750, 0.0582580566406250,
+-0.0635375976562500, 0.0445556640625000, -0.0883483886718750, 0.0364685058593750,
+-0.0858459472656250, 0.0475463867187500, -0.0690307617187500, -0.0036926269531250,
+-0.0744018554687500, 0.0075378417968750, -0.0790405273437500, -0.0111389160156250,
+-0.0633850097656250, -0.0554199218750000, -0.0630187988281250, -0.0339050292968750,
+-0.1070861816406250, -0.0793457031250000, -0.0916137695312500, -0.0788269042968750,
+-0.1029663085937500, -0.0865783691406250, -0.1489257812500000, -0.1155395507812500,
+-0.1321411132812500, -0.1015930175781250, -0.1213684082031250, -0.1200866699218750,
+-0.1433715820312500, -0.1293640136718750, -0.1143188476562500, -0.1119689941406250,
+-0.0812377929687500, -0.1080932617187500, -0.1066894531250000, -0.1189270019531250,
+-0.0335083007812500, -0.0715942382812500, -0.0401611328125000, -0.0769348144531250,
+-0.0349426269531250, -0.0695800781250000, 0.0121154785156250, -0.0360717773437500,
+-0.0051879882812500, -0.0480041503906250, 0.0010070800781250, -0.0393371582031250,
+0.0081787109375000, -0.0358886718750000, 0.0051269531250000, -0.0343627929687500,
+0.0007934570312500, -0.0349731445312500, 0.0015258789062500, -0.0381164550781250,
+0.0086975097656250, -0.0108337402343750, 0.0032653808593750, -0.0206604003906250,
+0.0107727050781250, -0.0051879882812500, 0.0216369628906250, 0.0241699218750000,
+0.0130920410156250, 0.0084228515625000, 0.0299072265625000, 0.0437316894531250,
+0.0310363769531250, 0.0413818359375000, 0.0271606445312500, 0.0516052246093750,
+0.0346374511718750, 0.0756225585937500, 0.0339660644531250, 0.0623168945312500,
+0.0173339843750000, 0.0982666015625000, 0.0271301269531250, 0.0982055664062500,
+0.0105895996093750, 0.0967407226562500, -0.0118103027343750, 0.1177978515625000,
+0.0049438476562500, 0.1133422851562500, -0.0372619628906250, 0.0965881347656250,
+-0.0327453613281250, 0.1066589355468750, -0.0425415039062500, 0.0929565429687500,
+-0.0711669921875000, 0.0701904296875000, -0.0545349121093750, 0.0828857421875000,
+-0.0833129882812500, 0.0545043945312500, -0.0843200683593750, 0.0563659667968750,
+-0.0803527832031250, 0.0477905273437500, -0.0944213867187500, 0.0285339355468750,
+-0.0912475585937500, 0.0393371582031250, -0.0777893066406250, 0.0088806152343750,
+-0.0874633789062500, 0.0111999511718750, -0.0708923339843750, 0.0055847167968750,
+-0.0513000488281250, -0.0146484375000000, -0.0672302246093750, -0.0063171386718750,
+-0.0208740234375000, -0.0238952636718750, -0.0285949707031250, -0.0231628417968750,
+-0.0175781250000000, -0.0271606445312500, 0.0162353515625000, -0.0407409667968750,
+-0.0022888183593750, -0.0346679687500000, 0.0299987792968750, -0.0365600585937500,
+0.0296936035156250, -0.0449218750000000, 0.0305786132812500, -0.0311584472656250,
+0.0476684570312500, -0.0186462402343750, 0.0401000976562500, -0.0291442871093750,
+0.0489196777343750, -0.0007934570312500, 0.0499572753906250, 0.0012817382812500,
+0.0484619140625000, -0.0009460449218750, 0.0521240234375000, 0.0110168457031250,
+0.0513000488281250, 0.0093994140625000, 0.0496826171875000, 0.0063476562500000,
+0.0518493652343750, 0.0061035156250000, 0.0482482910156250, 0.0091552734375000,
+0.0440368652343750, 0.0098876953125000, 0.0470581054687500, 0.0080261230468750,
+0.0389099121093750, 0.0157470703125000, 0.0393981933593750, 0.0171813964843750,
+0.0383300781250000, 0.0139465332031250, 0.0335693359375000, 0.0163879394531250,
+0.0355834960937500, 0.0182800292968750, 0.0313110351562500, 0.0046691894531250,
+0.0324401855468750, 0.0083618164062500, 0.0303039550781250, 0.0030822753906250,
+0.0263977050781250, -0.0098571777343750, 0.0281982421875000, -0.0038146972656250,
+0.0189514160156250, -0.0125427246093750, 0.0206604003906250, -0.0145263671875000,
+0.0158691406250000, -0.0129394531250000, 0.0074462890625000, -0.0149536132812500,
+0.0111389160156250, -0.0147705078125000, -0.0030822753906250, -0.0144653320312500,
+-0.0007324218750000, -0.0143432617187500, -0.0078125000000000, -0.0139160156250000,
+-0.0199890136718750, -0.0143737792968750, -0.0142211914062500, -0.0142822265625000,
+-0.0384521484375000, -0.0090332031250000, -0.0348510742187500, -0.0132141113281250,
+-0.0444641113281250, -0.0048217773437500, -0.0651245117187500, 0.0033264160156250,
+-0.0555419921875000, -0.0044250488281250, -0.0787658691406250, 0.0237426757812500,
+-0.0795898437500000, 0.0156860351562500, -0.0817871093750000, 0.0300598144531250,
+-0.0947570800781250, 0.0561218261718750, -0.0904235839843750, 0.0388183593750000,
+-0.0976867675781250, 0.0793762207031250, -0.0987548828125000, 0.0771179199218750,
+-0.0975646972656250, 0.0794982910156250, -0.1014099121093750, 0.1050720214843750,
+-0.1004333496093750, 0.0954284667968750, -0.0910034179687500, 0.0952453613281250,
+-0.0969543457031250, 0.1053771972656250, -0.0856018066406250, 0.0926208496093750,
+-0.0717163085937500, 0.0791320800781250, -0.0819702148437500, 0.0904846191406250,
+-0.0509948730468750},
+{0.0028076171875000, 0.0270080566406250, -0.0358581542968750, 0.0575866699218750,
+-0.0330505371093750, 0.0546569824218750, -0.0387573242187500, 0.0648498535156250,
+-0.0629882812500000, 0.0882873535156250, -0.0520935058593750, 0.0751647949218750,
+-0.0720214843750000, 0.1017456054687500, -0.0714416503906250, 0.1059570312500000,
+-0.0726013183593750, 0.0973510742187500, -0.0856323242187500, 0.1056518554687500,
+-0.0801696777343750, 0.1082458496093750, -0.0772094726562500, 0.0813293457031250,
+-0.0843505859375000, 0.0883789062500000, -0.0737609863281250, 0.0781555175781250,
+-0.0629882812500000, 0.0545043945312500, -0.0717468261718750, 0.0655822753906250,
+-0.0434265136718750, 0.0403442382812500, -0.0478210449218750, 0.0425720214843750,
+-0.0391235351562500, 0.0348815917968750, -0.0168151855468750, 0.0159912109375000,
+-0.0274658203125000, 0.0257263183593750, -0.0046997070312500, 0.0041198730468750,
+-0.0031127929687500, 0.0029602050781250, -0.0009765625000000, 0.0014343261718750,
+0.0120544433593750, -0.0089721679687500, 0.0086975097656250, -0.0055541992187500,
+0.0138244628906250, -0.0174255371093750, 0.0175476074218750, -0.0156860351562500,
+0.0157165527343750, -0.0195312500000000, 0.0155029296875000, -0.0310974121093750,
+0.0174865722656250, -0.0263671875000000, 0.0117492675781250, -0.0251464843750000,
+0.0133056640625000, -0.0317077636718750, 0.0122375488281250, -0.0216674804687500,
+0.0075378417968750, -0.0094299316406250, 0.0100097656250000, -0.0165710449218750,
+0.0076599121093750, -0.0013732910156250, 0.0068664550781250, 0.0021972656250000,
+0.0085144042968750, -0.0015869140625000, 0.0092468261718750, 0.0011596679687500,
+0.0087280273437500, 0.0023803710937500, 0.0091247558593750, -0.0023803710937500,
+0.0105895996093750, -0.0028076171875000, 0.0088806152343750, -0.0012512207031250,
+0.0072631835937500, -0.0021362304687500, 0.0087585449218750, -0.0024108886718750,
+0.0041503906250000, 0.0006408691406250, 0.0045471191406250, 0.0014648437500000,
+0.0045776367187500, -0.0004577636718750, 0.0022583007812500, 0.0000610351562500,
+0.0035705566406250, 0.0012512207031250, 0.0025024414062500, -0.0068359375000000,
+0.0028381347656250, -0.0050964355468750, 0.0026245117187500, -0.0072937011718750,
+0.0020141601562500, -0.0145874023437500, 0.0030822753906250, -0.0112915039062500,
+0.0018310546875000, -0.0118713378906250, 0.0018005371093750, -0.0160827636718750,
+0.0023803710937500, -0.0083923339843750, 0.0023803710937500, -0.0033874511718750,
+0.0023803710937500, -0.0093078613281250, 0.0018310546875000, 0.0150756835937500,
+0.0037231445312500, 0.0079040527343750, 0.0001831054687500, 0.0211791992187500,
+-0.0020141601562500, 0.0439758300781250, 0.0007934570312500, 0.0284729003906250,
+-0.0148620605468750, 0.0660705566406250, -0.0101928710937500, 0.0627746582031250,
+-0.0205078125000000, 0.0674133300781250, -0.0376892089843750, 0.0928344726562500,
+-0.0291748046875000, 0.0817871093750000, -0.0554809570312500, 0.0885925292968750,
+-0.0539855957031250, 0.0967102050781250, -0.0620422363281250, 0.0858764648437500,
+-0.0812072753906250, 0.0787048339843750, -0.0729980468750000, 0.0871582031250000,
+-0.0955200195312500, 0.0661926269531250, -0.0972290039062500, 0.0669555664062500,
+-0.0965576171875000, 0.0669250488281250, -0.1094055175781250, 0.0562438964843750,
+-0.1070861816406250, 0.0592956542968750, -0.0931701660156250, 0.0533142089843750,
+-0.1034240722656250, 0.0572204589843750, -0.0852050781250000, 0.0461120605468750,
+-0.0618286132812500, 0.0369873046875000, -0.0775146484375000, 0.0474243164062500,
+-0.0339355468750000, 0.0076904296875000, -0.0338745117187500, 0.0162048339843750,
+-0.0302734375000000, 0.0044250488281250, -0.0052795410156250, -0.0289306640625000,
+-0.0140380859375000, -0.0125732421875000, -0.0072326660156250, -0.0364990234375000,
+-0.0029296875000000, -0.0400390625000000, -0.0093688964843750, -0.0400695800781250,
+-0.0128173828125000, -0.0479431152343750, -0.0094299316406250, -0.0454406738281250,
+-0.0174560546875000, -0.0643615722656250, -0.0216064453125000, -0.0543823242187500,
+-0.0142211914062500, -0.0720214843750000, -0.0108642578125000, -0.0976562500000000,
+-0.0163269042968750, -0.0787048339843750, -0.0033874511718750, -0.1188964843750000,
+-0.0019531250000000, -0.1221008300781250, -0.0029296875000000, -0.1146545410156250,
+0.0016174316406250, -0.1324157714843750, -0.0007019042968750, -0.1317749023437500,
+-0.0030212402343750, -0.1033325195312500, 0.0016784667968750, -0.1167297363281250,
+-0.0079650878906250, -0.0993652343750000, -0.0196533203125000, -0.0665893554687500,
+-0.0117797851562500, -0.0841064453125000, -0.0217895507812500, -0.0430603027343750,
+-0.0278625488281250, -0.0458984375000000, -0.0210266113281250, -0.0344543457031250,
+-0.0179443359375000, -0.0078430175781250, -0.0219116210937500, -0.0241394042968750,
+-0.0090026855468750, 0.0185852050781250, -0.0122070312500000, 0.0155029296875000,
+-0.0054626464843750, 0.0198059082031250, 0.0061950683593750, 0.0487365722656250,
+-0.0012512207031250, 0.0380554199218750, 0.0190429687500000, 0.0390319824218750,
+0.0177001953125000, 0.0472717285156250, 0.0186462402343750, 0.0403137207031250,
+0.0335388183593750, 0.0281372070312500, 0.0308837890625000, 0.0304565429687500,
+0.0227966308593750, 0.0297851562500000, 0.0266723632812500, 0.0288391113281250,
+0.0273437500000000, 0.0265502929687500, 0.0198974609375000, 0.0267333984375000,
+0.0208435058593750, 0.0283203125000000, 0.0295410156250000, 0.0140686035156250,
+0.0252990722656250, 0.0150146484375000, 0.0283813476562500, 0.0174560546875000,
+0.0389404296875000, 0.0098266601562500, 0.0342102050781250, 0.0091247558593750,
+0.0308227539062500, 0.0180358886718750, 0.0379028320312500, 0.0180358886718750,
+0.0270080566406250, 0.0169067382812500, 0.0150146484375000, 0.0184326171875000,
+0.0243835449218750, 0.0189208984375000, 0.0000610351562500, 0.0284729003906250,
+0.0014343261718750, 0.0238647460937500, -0.0021667480468750, 0.0281982421875000,
+-0.0160827636718750, 0.0427246093750000, -0.0094604492187500, 0.0376586914062500,
+-0.0267028808593750, 0.0263061523437500, -0.0220336914062500, 0.0343322753906250,
+-0.0308532714843750, 0.0279846191406250, -0.0472106933593750, 0.0105895996093750,
+-0.0362854003906250, 0.0166015625000000, -0.0605468750000000, 0.0162658691406250,
+-0.0601501464843750, 0.0108947753906250, -0.0626831054687500, 0.0168762207031250,
+-0.0772094726562500, 0.0237121582031250, -0.0716247558593750, 0.0179443359375000,
+-0.0788574218750000, 0.0258178710937500, -0.0801696777343750, 0.0316772460937500,
+-0.0788574218750000, 0.0178222656250000, -0.0820312500000000, 0.0111083984375000,
+-0.0815734863281250, 0.0205078125000000, -0.0760803222656250, -0.0132141113281250,
+-0.0807800292968750, -0.0114135742187500, -0.0721130371093750, -0.0179443359375000,
+-0.0625915527343750, -0.0391235351562500, -0.0709228515625000, -0.0308227539062500,
+-0.0475769042968750, -0.0533447265625000, -0.0517578125000000, -0.0520935058593750,
+-0.0450439453125000, -0.0583190917968750, -0.0282287597656250, -0.0751342773437500,
+-0.0381164550781250, -0.0678405761718750, -0.0136108398437500, -0.0815429687500000,
+-0.0190429687500000, -0.0835266113281250, -0.0090942382812500, -0.0835876464843750,
+0.0108032226562500, -0.0900268554687500, -0.0026245117187500, -0.0885620117187500,
+0.0307922363281250, -0.0881652832031250, 0.0249023437500000, -0.0918579101562500,
+0.0336608886718750, -0.0842285156250000, 0.0582885742187500, -0.0783081054687500,
+0.0437927246093750, -0.0834655761718750, 0.0715332031250000, -0.0597839355468750,
+0.0699462890625000, -0.0629272460937500, 0.0731201171875000, -0.0540466308593750,
+0.0899047851562500, -0.0345153808593750, 0.0816040039062500, -0.0433654785156250,
+0.0940856933593750, -0.0229797363281250, 0.0959167480468750, -0.0214538574218750,
+0.0928955078125000, -0.0212402343750000, 0.0981445312500000, -0.0106811523437500,
+0.0978393554687500, -0.0135498046875000, 0.0852661132812500, -0.0157775878906250,
+0.0938720703125000, -0.0118408203125000, 0.0785217285156250, -0.0172424316406250,
+0.0604858398437500, -0.0238037109375000, 0.0752868652343750, -0.0195312500000000,
+0.0292053222656250, -0.0315246582031250, 0.0367736816406250, -0.0300292968750000,
+0.0258178710937500, -0.0323486328125000, -0.0087280273437500, -0.0401306152343750,
+0.0095825195312500, -0.0356140136718750, -0.0156860351562500, -0.0434875488281250,
+-0.0201110839843750, -0.0425415039062500, -0.0147094726562500, -0.0420227050781250,
+-0.0227355957031250, -0.0460510253906250, -0.0232849121093750, -0.0428161621093750,
+-0.0158081054687500, -0.0414733886718750, -0.0182800292968750, -0.0409240722656250,
+-0.0169067382812500, -0.0379638671875000, -0.0110168457031250, -0.0350952148437500,
+-0.0133666992187500, -0.0350341796875000, -0.0094909667968750, -0.0329284667968750,
+-0.0090942382812500, -0.0302124023437500, -0.0090942382812500, -0.0301208496093750,
+-0.0064086914062500, -0.0310363769531250, -0.0072631835937500, -0.0297241210937500,
+-0.0114440917968750, -0.0291442871093750, -0.0078430175781250, -0.0303955078125000,
+-0.0142822265625000, -0.0282592773437500, -0.0233764648437500, -0.0256347656250000,
+-0.0174255371093750, -0.0278625488281250, -0.0260925292968750, -0.0261230468750000,
+-0.0318603515625000, -0.0243225097656250, -0.0216979980468750, -0.0277709960937500,
+-0.0173034667968750, -0.0292663574218750, -0.0239868164062500, -0.0268249511718750,
+0.0018005371093750, -0.0373229980468750, -0.0001525878906250, -0.0330810546875000,
+0.0037231445312500, -0.0407409667968750, 0.0203247070312500, -0.0515747070312500,
+0.0146179199218750, -0.0431518554687500, 0.0257568359375000, -0.0683898925781250,
+0.0261535644531250, -0.0642700195312500, 0.0283813476562500, -0.0695800781250000,
+0.0360412597656250, -0.0890502929687500, 0.0335388183593750, -0.0791625976562500,
+0.0384216308593750, -0.0865783691406250, 0.0380554199218750, -0.0934143066406250,
+0.0440063476562500, -0.0832519531250000, 0.0471496582031250, -0.0783081054687500,
+0.0430297851562500, -0.0856018066406250, 0.0738220214843750, -0.0598754882812500,
+0.0650024414062500, -0.0632019042968750, 0.0791625976562500, -0.0587463378906250,
+0.1116638183593750, -0.0407104492187500, 0.0958251953125000, -0.0480346679687500,
+0.1135253906250000, -0.0376281738281250, 0.1254272460937500, -0.0377502441406250,
+0.1069030761718750, -0.0340576171875000, 0.0999145507812500, -0.0282897949218750,
+0.1113586425781250, -0.0329895019531250, 0.0628662109375000, -0.0127258300781250,
+0.0717773437500000, -0.0167236328125000, 0.0552062988281250, -0.0098266601562500,
+0.0154113769531250, 0.0073852539062500, 0.0354309082031250, -0.0004882812500000,
+-0.0039672851562500, 0.0156860351562500, -0.0053405761718750, 0.0143737792968750,
+-0.0081481933593750, 0.0201416015625000, -0.0286865234375000, 0.0340881347656250,
+-0.0209350585937500, 0.0267944335937500, -0.0367126464843750, 0.0344848632812500,
+-0.0342102050781250, 0.0424194335937500, -0.0435180664062500, 0.0303039550781250,
+-0.0574645996093750, 0.0209655761718750, -0.0477600097656250, 0.0301208496093750,
+-0.0774536132812500, 0.0122680664062500, -0.0741577148437500, 0.0102844238281250,
+-0.0795593261718750, 0.0116271972656250, -0.1013793945312500, 0.0044860839843750,
+-0.0925598144531250, 0.0071105957031250, -0.1011352539062500, 0.0111694335937500,
+-0.1062011718750000, 0.0049743652343750, -0.1001586914062500, 0.0166015625000000,
+-0.0990600585937500, 0.0320129394531250, -0.1017456054687500, 0.0211791992187500,
+-0.0845031738281250, 0.0286865234375000, -0.0924377441406250, 0.0411682128906250,
+-0.0732421875000000},
+{0.0875854492187500, -0.0082092285156250, 0.0702819824218750, -0.0073852539062500,
+0.0483093261718750, -0.0046081542968750, 0.0654296875000000, -0.0046386718750000,
+0.0165405273437500, -0.0015258789062500, 0.0230102539062500, -0.0015563964843750,
+0.0141906738281250, -0.0001831054687500, -0.0211486816406250, 0.0025634765625000,
+-0.0046691894531250, 0.0012512207031250, -0.0217590332031250, 0.0015869140625000,
+-0.0280456542968750, 0.0025939941406250, -0.0206298828125000, 0.0001525878906250,
+-0.0210876464843750, -0.0018615722656250, -0.0242004394531250, -0.0005493164062500,
+-0.0166320800781250, -0.0051879882812500, -0.0146789550781250, -0.0058898925781250,
+-0.0202331542968750, -0.0060119628906250, -0.0207824707031250, -0.0077819824218750,
+-0.0176086425781250, -0.0077209472656250, -0.0303955078125000, -0.0096435546875000,
+-0.0313110351562500, -0.0086975097656250, -0.0268554687500000, -0.0113525390625000,
+-0.0321044921875000, -0.0148925781250000, -0.0335693359375000, -0.0126953125000000,
+-0.0131530761718750, -0.0155944824218750, -0.0191955566406250, -0.0180053710937500,
+-0.0102539062500000, -0.0161743164062500, 0.0092163085937500, -0.0157165527343750,
+-0.0002746582031250, -0.0183410644531250, 0.0191040039062500, -0.0142517089843750,
+0.0189514160156250, -0.0156250000000000, 0.0229797363281250, -0.0150756835937500,
+0.0351867675781250, -0.0122985839843750, 0.0293884277343750, -0.0148010253906250,
+0.0429077148437500, -0.0114440917968750, 0.0429077148437500, -0.0091552734375000,
+0.0454101562500000, -0.0151062011718750, 0.0523376464843750, -0.0151367187500000,
+0.0494995117187500, -0.0109558105468750, 0.0661315917968750, -0.0380859375000000,
+0.0606384277343750, -0.0266723632812500, 0.0744323730468750, -0.0402221679687500,
+0.0957031250000000, -0.0681457519531250, 0.0827636718750000, -0.0490722656250000,
+0.1068420410156250, -0.0795593261718750, 0.1138916015625000, -0.0833435058593750,
+0.1019592285156250, -0.0762329101562500, 0.1051025390625000, -0.0867309570312500,
+0.1101989746093750, -0.0867004394531250, 0.0733337402343750, -0.0704650878906250,
+0.0823669433593750, -0.0757141113281250, 0.0660705566406250, -0.0675048828125000,
+0.0330200195312500, -0.0536499023437500, 0.0505371093750000, -0.0615844726562500,
+0.0103149414062500, -0.0320739746093750, 0.0110778808593750, -0.0415649414062500,
+0.0061035156250000, -0.0220947265625000, -0.0177612304687500, 0.0100097656250000,
+-0.0080871582031250, -0.0107421875000000, -0.0256958007812500, 0.0354614257812500,
+-0.0244140625000000, 0.0374755859375000, -0.0318603515625000, 0.0372924804687500,
+-0.0445556640625000, 0.0600891113281250, -0.0359802246093750, 0.0538024902343750,
+-0.0638427734375000, 0.0567016601562500, -0.0602416992187500, 0.0600280761718750,
+-0.0668640136718750, 0.0531005859375000, -0.0887145996093750, 0.0514526367187500,
+-0.0793762207031250, 0.0563659667968750, -0.0899353027343750, 0.0318908691406250,
+-0.0942687988281250, 0.0344543457031250, -0.0897216796875000, 0.0344848632812500,
+-0.0907287597656250, 0.0198364257812500, -0.0918884277343750, 0.0234375000000000,
+-0.0792541503906250, 0.0251159667968750, -0.0859069824218750, 0.0248413085937500,
+-0.0699768066406250, 0.0252075195312500, -0.0540771484375000, 0.0248718261718750,
+-0.0671386718750000, 0.0256958007812500, -0.0097656250000000, 0.0332946777343750,
+-0.0211791992187500, 0.0327758789062500, -0.0004577636718750, 0.0278930664062500,
+0.0509948730468750, 0.0332641601562500, 0.0265197753906250, 0.0372619628906250,
+0.0614013671875000, 0.0092468261718750, 0.0749511718750000, 0.0135192871093750,
+0.0571899414062500, 0.0104980468750000, 0.0571899414062500, -0.0070800781250000,
+0.0679016113281250, -0.0018920898437500, 0.0341491699218750, -0.0177001953125000,
+0.0338439941406250, -0.0101318359375000, 0.0393981933593750, -0.0275878906250000,
+0.0235595703125000, -0.0513305664062500, 0.0246582031250000, -0.0344848632812500,
+0.0466613769531250, -0.0718688964843750, 0.0390930175781250, -0.0765991210937500,
+0.0467834472656250, -0.0741882324218750, 0.0690612792968750, -0.0899963378906250,
+0.0596008300781250, -0.0862731933593750, 0.0640869140625000, -0.0754699707031250,
+0.0701599121093750, -0.0871276855468750, 0.0631103515625000, -0.0686950683593750,
+0.0570678710937500, -0.0442504882812500, 0.0615234375000000, -0.0623474121093750,
+0.0514221191406250, -0.0320739746093750, 0.0523681640625000, -0.0258789062500000,
+0.0477294921875000, -0.0346679687500000, 0.0323791503906250, -0.0289916992187500,
+-0.0241394042968750, -0.0646972656250000, -0.0511779785156250, -0.0248718261718750,
+-0.0505371093750000, -0.0262145996093750, -0.0488891601562500, -0.0310058593750000,
+-0.0520019531250000, -0.0281066894531250, -0.0535278320312500, -0.0255126953125000,
+-0.0571899414062500, -0.0235290527343750, -0.0496826171875000, -0.0329589843750000,
+-0.0667419433593750, -0.0163879394531250, -0.0804138183593750, 0.0008850097656250,
+-0.0652160644531250, -0.0134277343750000, -0.1163635253906250, 0.0206298828125000,
+-0.1156005859375000, 0.0225219726562500, -0.1109619140625000, 0.0208740234375000,
+-0.1400756835937500, 0.0348815917968750, -0.1364440917968750, 0.0314941406250000,
+-0.1040954589843750, 0.0373840332031250, -0.1215515136718750, 0.0348815917968750,
+-0.0963745117187500, 0.0408630371093750, -0.0583190917968750, 0.0502014160156250,
+-0.0835571289062500, 0.0438232421875000, -0.0114440917968750, 0.0520019531250000,
+-0.0209655761718750, 0.0575561523437500, -0.0052795410156250, 0.0481567382812500,
+0.0481872558593750, 0.0435791015625000, 0.0238342285156250, 0.0499877929687500,
+0.0596008300781250, 0.0302124023437500, 0.0638427734375000, 0.0304260253906250,
+0.0613708496093750, 0.0295715332031250, 0.0780334472656250, 0.0204162597656250,
+0.0721130371093750, 0.0233459472656250, 0.0606689453125000, 0.0119628906250000,
+0.0721130371093750, 0.0180969238281250, 0.0574340820312500, 0.0045776367187500,
+0.0337829589843750, -0.0120544433593750, 0.0480346679687500, 0.0005187988281250,
+0.0296325683593750, -0.0361022949218750, 0.0234985351562500, -0.0335998535156250,
+0.0332336425781250, -0.0403442382812500, 0.0315856933593750, -0.0642700195312500,
+0.0285644531250000, -0.0539855957031250, 0.0583801269531250, -0.0742797851562500,
+0.0494079589843750, -0.0752258300781250, 0.0581970214843750, -0.0747070312500000,
+0.0895385742187500, -0.0870056152343750, 0.0755310058593750, -0.0838317871093750,
+0.0642700195312500, -0.0720520019531250, 0.0858154296875000, -0.0819396972656250,
+0.0541687011718750, -0.0656433105468750, 0.0186462402343750, -0.0434875488281250,
+0.0474548339843750, -0.0579528808593750, -0.0246582031250000, -0.0252380371093750,
+-0.0202636718750000, -0.0229187011718750, -0.0264892578125000, -0.0243835449218750,
+-0.0696105957031250, -0.0114746093750000, -0.0520019531250000, -0.0145263671875000,
+-0.0739440917968750, -0.0059204101562500, -0.0804443359375000, -0.0112304687500000,
+-0.0696105957031250, 0.0005798339843750, -0.0737915039062500, 0.0146179199218750,
+-0.0783081054687500, 0.0039062500000000, -0.0466308593750000, 0.0354003906250000,
+-0.0547790527343750, 0.0323181152343750, -0.0429382324218750, 0.0399475097656250,
+-0.0158386230468750, 0.0608215332031250, -0.0311279296875000, 0.0504150390625000,
+-0.0027465820312500, 0.0786132812500000, -0.0030822753906250, 0.0752868652343750,
+-0.0036621093750000, 0.0816650390625000, 0.0118103027343750, 0.1042785644531250,
+0.0064392089843750, 0.0944824218750000, 0.0051574707031250, 0.0998535156250000,
+0.0104370117187500, 0.1083679199218750, 0.0030822753906250, 0.0959472656250000,
+-0.0038452148437500, 0.0863647460937500, 0.0027770996093750, 0.0946960449218750,
+-0.0150756835937500, 0.0679931640625000, -0.0112915039062500, 0.0704040527343750,
+-0.0153503417968750, 0.0628051757812500, -0.0275573730468750, 0.0446166992187500,
+-0.0187683105468750, 0.0538024902343750, -0.0337524414062500, 0.0280761718750000,
+-0.0309753417968750, 0.0302734375000000, -0.0310668945312500, 0.0238342285156250,
+-0.0399475097656250, 0.0054931640625000, -0.0340576171875000, 0.0136718750000000,
+-0.0381774902343750, -0.0044250488281250, -0.0376892089843750, -0.0028686523437500,
+-0.0351562500000000, -0.0090026855468750, -0.0347595214843750, -0.0227661132812500,
+-0.0323791503906250, -0.0153808593750000, -0.0313720703125000, -0.0326843261718750,
+-0.0291748046875000, -0.0350036621093750, -0.0290832519531250, -0.0313415527343750,
+-0.0287475585937500, -0.0375976562500000, -0.0261535644531250, -0.0379333496093750,
+-0.0311584472656250, -0.0255432128906250, -0.0289306640625000, -0.0285949707031250,
+-0.0296936035156250, -0.0239868164062500, -0.0339355468750000, -0.0126342773437500,
+-0.0305786132812500, -0.0171508789062500, -0.0319519042968750, -0.0093994140625000,
+-0.0317687988281250, -0.0086975097656250, -0.0302429199218750, -0.0068969726562500,
+-0.0289611816406250, -0.0039672851562500, -0.0269775390625000, -0.0063476562500000,
+-0.0269470214843750, 0.0071105957031250, -0.0257873535156250, 0.0008239746093750,
+-0.0236511230468750, 0.0119628906250000, -0.0230407714843750, 0.0275268554687500,
+-0.0220947265625000, 0.0156860351562500, -0.0164184570312500, 0.0480041503906250,
+-0.0181274414062500, 0.0440673828125000, -0.0141296386718750, 0.0498657226562500,
+-0.0071716308593750, 0.0726623535156250, -0.0093994140625000, 0.0613403320312500,
+-0.0018615722656250, 0.0734863281250000, -0.0004882812500000, 0.0785217285156250,
+-0.0012207031250000, 0.0714721679687500, 0.0020446777343750, 0.0711059570312500,
+0.0026550292968750, 0.0750122070312500, -0.0007934570312500, 0.0616760253906250,
+0.0003662109375000, 0.0631713867187500, -0.0004577636718750, 0.0621948242187500,
+-0.0032958984375000, 0.0539855957031250, -0.0017395019531250, 0.0571289062500000,
+-0.0034484863281250, 0.0538940429687500, -0.0030517578125000, 0.0552062988281250,
+-0.0034179687500000, 0.0499572753906250, -0.0040283203125000, 0.0469055175781250,
+-0.0028381347656250, 0.0517578125000000, -0.0073852539062500, 0.0262756347656250,
+-0.0048217773437500, 0.0351257324218750, -0.0098571777343750, 0.0221862792968750,
+-0.0145874023437500, -0.0036315917968750, -0.0096740722656250, 0.0119628906250000,
+-0.0307922363281250, -0.0181579589843750, -0.0245666503906250, -0.0191955566406250,
+-0.0365295410156250, -0.0198974609375000, -0.0578613281250000, -0.0344543457031250,
+-0.0456542968750000, -0.0297851562500000, -0.0772094726562500, -0.0405273437500000,
+-0.0791320800781250, -0.0368347167968750, -0.0773620605468750, -0.0478515625000000,
+-0.0953674316406250, -0.0597229003906250, -0.0921630859375000, -0.0489501953125000,
+-0.0688476562500000, -0.0860900878906250, -0.0820007324218750, -0.0797119140625000,
+-0.0622253417968750, -0.0853576660156250, -0.0292663574218750, -0.1145629882812500,
+-0.0473022460937500, -0.1020812988281250, -0.0106201171875000, -0.1015014648437500,
+-0.0058288574218750, -0.1160888671875000, -0.0110168457031250, -0.0943603515625000,
+0.0025024414062500, -0.0744018554687500, 0.0007629394531250, -0.0929870605468750,
+-0.0120849609375000, -0.0426330566406250, -0.0084533691406250, -0.0458068847656250,
+-0.0123596191406250, -0.0401000976562500, -0.0245666503906250, -0.0092468261718750,
+-0.0200195312500000, -0.0205078125000000, -0.0207519531250000, -0.0002136230468750,
+-0.0280761718750000, -0.0021972656250000, -0.0148315429687500, 0.0076293945312500,
+-0.0023193359375000, 0.0240478515625000, -0.0138549804687500, 0.0121154785156250,
+0.0087280273437500, 0.0411071777343750, 0.0157165527343750, 0.0430908203125000,
+0.0040893554687500, 0.0350952148437500, 0.0039978027343750, 0.0468444824218750,
+0.0096435546875000},
+{-0.0319213867187500, -0.0075073242187500, -0.0404052734375000, -0.0013427734375000,
+-0.0225830078125000, -0.0154724121093750, -0.0254516601562500, -0.0144958496093750,
+-0.0214233398437500, -0.0190124511718750, -0.0090332031250000, -0.0303039550781250,
+-0.0169982910156250, -0.0247802734375000, -0.0032043457031250, -0.0335388183593750,
+-0.0039978027343750, -0.0372924804687500, -0.0046691894531250, -0.0305480957031250,
+0.0024719238281250, -0.0304870605468750, -0.0012207031250000, -0.0340881347656250,
+-0.0013122558593750, -0.0117187500000000, -0.0015563964843750, -0.0183410644531250,
+-0.0039062500000000, -0.0060424804687500, -0.0062255859375000, 0.0154724121093750,
+-0.0079040527343750, 0.0022888183593750, -0.0100402832031250, 0.0348205566406250,
+-0.0106201171875000, 0.0313110351562500, -0.0130004882812500, 0.0377502441406250,
+-0.0152893066406250, 0.0597839355468750, -0.0154724121093750, 0.0480041503906250,
+-0.0209045410156250, 0.0672912597656250, -0.0205688476562500, 0.0692138671875000,
+-0.0217285156250000, 0.0663146972656250, -0.0262451171875000, 0.0744628906250000,
+-0.0254516601562500, 0.0734863281250000, -0.0247192382812500, 0.0668640136718750,
+-0.0269775390625000, 0.0703430175781250, -0.0239257812500000, 0.0663452148437500,
+-0.0204772949218750, 0.0585021972656250, -0.0232238769531250, 0.0628967285156250,
+-0.0158996582031250, 0.0549926757812500, -0.0156250000000000, 0.0558776855468750,
+-0.0147399902343750, 0.0515136718750000, -0.0103454589843750, 0.0463256835937500,
+-0.0109863281250000, 0.0507812500000000, -0.0083618164062500, 0.0297851562500000,
+-0.0083618164062500, 0.0369262695312500, -0.0060729980468750, 0.0253906250000000,
+-0.0029907226562500, 0.0040893554687500, -0.0041809082031250, 0.0177612304687500,
+0.0001525878906250, -0.0106201171875000, 0.0012817382812500, -0.0103149414062500,
+0.0005187988281250, -0.0118408203125000, 0.0026855468750000, -0.0269775390625000,
+0.0032348632812500, -0.0213012695312500, -0.0036315917968750, -0.0323791503906250,
+0.0009155273437500, -0.0301818847656250, -0.0079040527343750, -0.0377502441406250,
+-0.0181579589843750, -0.0470886230468750, -0.0102539062500000, -0.0393981933593750,
+-0.0363464355468750, -0.0687255859375000, -0.0324401855468750, -0.0618591308593750,
+-0.0430908203125000, -0.0711059570312500, -0.0653076171875000, -0.0974731445312500,
+-0.0537719726562500, -0.0833740234375000, -0.0785827636718750, -0.0948791503906250,
+-0.0837402343750000, -0.1062622070312500, -0.0734558105468750, -0.0876770019531250,
+-0.0791931152343750, -0.0786132812500000, -0.0830078125000000, -0.0931396484375000,
+-0.0500793457031250, -0.0460510253906250, -0.0559387207031250, -0.0530090332031250,
+-0.0469055175781250, -0.0439147949218750, -0.0203857421875000, -0.0095520019531250,
+-0.0312194824218750, -0.0242614746093750, -0.0152893066406250, -0.0035705566406250,
+-0.0114135742187500, -0.0010070800781250, -0.0149230957031250, 0.0032043457031250,
+-0.0109558105468750, 0.0122375488281250, -0.0106811523437500, 0.0055847167968750,
+-0.0167541503906250, 0.0314941406250000, -0.0153198242187500, 0.0269775390625000,
+-0.0171508789062500, 0.0286254882812500, -0.0253906250000000, 0.0496826171875000,
+-0.0230712890625000, 0.0433959960937500, -0.0089721679687500, 0.0298156738281250,
+-0.0174560546875000, 0.0418090820312500, -0.0104675292968750, 0.0289916992187500,
+0.0086975097656250, 0.0053405761718750, 0.0008850097656250, 0.0199890136718750,
+0.0053405761718750, 0.0074157714843750, 0.0096435546875000, -0.0022277832031250,
+0.0024719238281250, 0.0100402832031250, 0.0001831054687500, 0.0178222656250000,
+0.0040588378906250, 0.0093078613281250, -0.0214538574218750, 0.0263366699218750,
+-0.0175476074218750, 0.0292968750000000, -0.0180358886718750, 0.0262756347656250,
+-0.0386047363281250, 0.0298461914062500, -0.0355224609375000, 0.0289611816406250,
+-0.0166625976562500, 0.0286254882812500, -0.0269165039062500, 0.0292053222656250,
+-0.0193176269531250, 0.0258789062500000, -0.0011901855468750, 0.0251770019531250,
+-0.0127563476562500, 0.0267639160156250, 0.0120544433593750, 0.0091857910156250,
+0.0124206542968750, 0.0111694335937500, 0.0114746093750000, 0.0097045898437500,
+0.0235290527343750, -0.0032348632812500, 0.0202941894531250, -0.0008850097656250,
+0.0220642089843750, 0.0038146972656250, 0.0228576660156250, 0.0007324218750000,
+0.0221862792968750, 0.0010375976562500, 0.0226440429687500, 0.0075683593750000,
+0.0228576660156250, 0.0058593750000000, 0.0214843750000000, -0.0062866210937500,
+0.0201721191406250, -0.0000610351562500, 0.0249633789062500, -0.0101623535156250,
+0.0270996093750000, -0.0263977050781250, 0.0231933593750000, -0.0177001953125000,
+0.0364685058593750, -0.0401306152343750, 0.0373535156250000, -0.0415649414062500,
+0.0322875976562500, -0.0400085449218750, 0.0383605957031250, -0.0497436523437500,
+0.0405578613281250, -0.0489196777343750, 0.0170898437500000, -0.0496826171875000,
+0.0245971679687500, -0.0481872558593750, 0.0151977539062500, -0.0517578125000000,
+-0.0082397460937500, -0.0541381835937500, 0.0028991699218750, -0.0511474609375000,
+-0.0114440917968750, -0.0669250488281250, -0.0143127441406250, -0.0618286132812500,
+-0.0136718750000000, -0.0658874511718750, -0.0167846679687500, -0.0821228027343750,
+-0.0148620605468750, -0.0749511718750000, -0.0273132324218750, -0.0704650878906250,
+-0.0216369628906250, -0.0780639648437500, -0.0305786132812500, -0.0685424804687500,
+-0.0462036132812500, -0.0548706054687500, -0.0361938476562500, -0.0625000000000000,
+-0.0526733398437500, -0.0490722656250000, -0.0542602539062500, -0.0468139648437500,
+-0.0525207519531250, -0.0459899902343750, -0.0586853027343750, -0.0410766601562500,
+-0.0571289062500000, -0.0424804687500000, -0.0581665039062500, -0.0318603515625000,
+-0.0592346191406250, -0.0339355468750000, -0.0566101074218750, -0.0281982421875000,
+-0.0563964843750000, -0.0168762207031250, -0.0579223632812500, -0.0208435058593750,
+-0.0467529296875000, -0.0101013183593750, -0.0520019531250000, -0.0068969726562500,
+-0.0436096191406250, -0.0086364746093750, -0.0311279296875000, -0.0045776367187500,
+-0.0405578613281250, -0.0036926269531250, -0.0165100097656250, -0.0156860351562500,
+-0.0202026367187500, -0.0103149414062500, -0.0136413574218750, -0.0191345214843750,
+0.0037231445312500, -0.0334472656250000, -0.0059509277343750, -0.0248718261718750,
+0.0162048339843750, -0.0464172363281250, 0.0131835937500000, -0.0453491210937500,
+0.0191650390625000, -0.0484008789062500, 0.0350952148437500, -0.0623474121093750,
+0.0258789062500000, -0.0561523437500000, 0.0455932617187500, -0.0619506835937500,
+0.0438537597656250, -0.0650634765625000, 0.0476684570312500, -0.0586853027343750,
+0.0605163574218750, -0.0559082031250000, 0.0536499023437500, -0.0576782226562500,
+0.0673217773437500, -0.0419616699218750, 0.0667114257812500, -0.0431518554687500,
+0.0677795410156250, -0.0362854003906250, 0.0765686035156250, -0.0223693847656250,
+0.0723266601562500, -0.0262756347656250, 0.0718688964843750, -0.0121154785156250,
+0.0768737792968750, -0.0097656250000000, 0.0668640136718750, -0.0060119628906250,
+0.0598754882812500, 0.0029602050781250, 0.0671997070312500, 0.0016174316406250,
+0.0355224609375000, 0.0083007812500000, 0.0432434082031250, 0.0110778808593750,
+0.0303955078125000, 0.0112304687500000, 0.0020141601562500, 0.0143127441406250,
+0.0178833007812500, 0.0155334472656250, -0.0112304687500000, 0.0102233886718750,
+-0.0130004882812500, 0.0147705078125000, -0.0084228515625000, 0.0102539062500000,
+-0.0204772949218750, 0.0032653808593750, -0.0176696777343750, 0.0098266601562500,
+-0.0063781738281250, -0.0046691894531250, -0.0120544433593750, -0.0014343261718750,
+-0.0034790039062500, -0.0052490234375000, 0.0104675292968750, -0.0160827636718750,
+0.0024108886718750, -0.0088500976562500, 0.0209960937500000, -0.0224914550781250,
+0.0211181640625000, -0.0209045410156250, 0.0227355957031250, -0.0229492187500000,
+0.0328369140625000, -0.0315246582031250, 0.0292053222656250, -0.0267333984375000,
+0.0369873046875000, -0.0367431640625000, 0.0345458984375000, -0.0355834960937500,
+0.0431213378906250, -0.0361633300781250, 0.0516052246093750, -0.0422363281250000,
+0.0440979003906250, -0.0390319824218750, 0.0760803222656250, -0.0421752929687500,
+0.0715332031250000, -0.0415649414062500, 0.0775756835937500, -0.0433654785156250,
+0.1030883789062500, -0.0451354980468750, 0.0933227539062500, -0.0419616699218750,
+0.0935668945312500, -0.0518798828125000, 0.1030578613281250, -0.0493164062500000,
+0.0856628417968750, -0.0492248535156250, 0.0702514648437500, -0.0567626953125000,
+0.0820007324218750, -0.0533142089843750, 0.0429992675781250, -0.0483398437500000,
+0.0461730957031250, -0.0543212890625000, 0.0370178222656250, -0.0458679199218750,
+0.0101318359375000, -0.0344238281250000, 0.0217285156250000, -0.0423278808593750,
+-0.0036315917968750, -0.0263977050781250, -0.0032348632812500, -0.0263366699218750,
+-0.0091552734375000, -0.0234985351562500, -0.0244750976562500, -0.0149230957031250,
+-0.0164489746093750, -0.0196228027343750, -0.0414123535156250, -0.0055236816406250,
+-0.0370788574218750, -0.0057373046875000, -0.0452575683593750, -0.0074462890625000,
+-0.0664367675781250, 0.0010986328125000, -0.0559997558593750, 0.0002441406250000,
+-0.0724487304687500, -0.0133056640625000, -0.0755920410156250, -0.0094299316406250,
+-0.0724487304687500, -0.0107116699218750, -0.0774536132812500, -0.0249023437500000,
+-0.0772705078125000, -0.0224609375000000, -0.0723266601562500, -0.0046386718750000,
+-0.0758666992187500, -0.0149536132812500, -0.0667114257812500, 0.0008544921875000,
+-0.0596008300781250, 0.0261535644531250, -0.0664062500000000, 0.0132446289062500,
+-0.0301513671875000, 0.0491333007812500, -0.0407714843750000, 0.0482177734375000,
+-0.0234375000000000, 0.0510559082031250, 0.0131835937500000, 0.0752563476562500,
+-0.0058593750000000, 0.0682067871093750, 0.0298156738281250, 0.0607910156250000,
+0.0379638671875000, 0.0711669921875000, 0.0273132324218750, 0.0579833984375000,
+0.0352172851562500, 0.0396423339843750, 0.0391540527343750, 0.0510253906250000,
+0.0163269042968750, 0.0258178710937500, 0.0174865722656250, 0.0259704589843750,
+0.0197753906250000, 0.0249023437500000, 0.0070495605468750, 0.0092468261718750,
+0.0085449218750000, 0.0150146484375000, 0.0236511230468750, 0.0219421386718750,
+0.0178833007812500, 0.0160522460937500, 0.0249023437500000, 0.0201721191406250,
+0.0416870117187500, 0.0327148437500000, 0.0342712402343750, 0.0289611816406250,
+0.0409851074218750, 0.0226440429687500, 0.0450744628906250, 0.0299377441406250,
+0.0408630371093750, 0.0198669433593750, 0.0388183593750000, 0.0068969726562500,
+0.0405883789062500, 0.0156860351562500, 0.0353393554687500, -0.0090332031250000,
+0.0374450683593750, -0.0068054199218750, 0.0308837890625000, -0.0115661621093750,
+0.0261230468750000, -0.0312805175781250, 0.0313720703125000, -0.0236816406250000,
+-0.0007019042968750, -0.0206604003906250, 0.0072021484375000, -0.0251464843750000,
+-0.0054931640625000, -0.0295104980468750, -0.0365295410156250, -0.0216674804687500,
+-0.0220642089843750, -0.0186157226562500, -0.0400085449218750, -0.0606079101562500,
+-0.0473327636718750, -0.0509643554687500, -0.0426330566406250, -0.0558166503906250,
+-0.0434570312500000, -0.1054077148437500, -0.0632324218750000, -0.0149536132812500,
+-0.1024475097656250, -0.0679931640625000, -0.1012573242187500, -0.0497436523437500,
+-0.0938110351562500, -0.0602111816406250, -0.0998840332031250, -0.0930786132812500,
+-0.1048583984375000, -0.0845642089843750, -0.0812072753906250, -0.0662231445312500,
+-0.0879821777343750},
+{0.0831909179687500, -0.0287170410156250, 0.0911560058593750, -0.0327148437500000,
+0.0785217285156250, -0.0246582031250000, 0.0715637207031250, -0.0182495117187500,
+0.0785217285156250, -0.0251770019531250, 0.0477600097656250, -0.0051574707031250,
+0.0507812500000000, -0.0067749023437500, 0.0400390625000000, -0.0068664550781250,
+0.0159301757812500, 0.0058288574218750, 0.0267944335937500, 0.0023803710937500,
+0.0034790039062500, -0.0031127929687500, 0.0024108886718750, 0.0040893554687500,
+-0.0008544921875000, -0.0043029785156250, -0.0129394531250000, -0.0161132812500000,
+-0.0079956054687500, -0.0074157714843750, -0.0257873535156250, -0.0244750976562500,
+-0.0217285156250000, -0.0225830078125000, -0.0307006835937500, -0.0266113281250000,
+-0.0474243164062500, -0.0379943847656250, -0.0375366210937500, -0.0308227539062500,
+-0.0592651367187500, -0.0458374023437500, -0.0596923828125000, -0.0472106933593750,
+-0.0598144531250000, -0.0417480468750000, -0.0711364746093750, -0.0473937988281250,
+-0.0676879882812500, -0.0489807128906250, -0.0692138671875000, -0.0295104980468750,
+-0.0720825195312500, -0.0342407226562500, -0.0663452148437500, -0.0277709960937500,
+-0.0647888183593750, -0.0107421875000000, -0.0684204101562500, -0.0172119140625000,
+-0.0430603027343750, -0.0033569335937500, -0.0525207519531250, -0.0052795410156250,
+-0.0343322753906250, 0.0028076171875000, -0.0040893554687500, 0.0155639648437500,
+-0.0222778320312500, 0.0065307617187500, 0.0137023925781250, 0.0260314941406250,
+0.0186157226562500, 0.0274658203125000, 0.0115661621093750, 0.0272216796875000,
+0.0239562988281250, 0.0350646972656250, 0.0244140625000000, 0.0326538085937500,
+0.0086059570312500, 0.0401916503906250, 0.0114440917968750, 0.0390625000000000,
+0.0117187500000000, 0.0415649414062500, 0.0000000000000000, 0.0485229492187500,
+0.0022583007812500, 0.0453796386718750, 0.0149230957031250, 0.0477905273437500,
+0.0087585449218750, 0.0508728027343750, 0.0158691406250000, 0.0450439453125000,
+0.0315551757812500, 0.0426025390625000, 0.0242614746093750, 0.0470581054687500,
+0.0324096679687500, 0.0280456542968750, 0.0361022949218750, 0.0310363769531250,
+0.0323486328125000, 0.0260925292968750, 0.0314941406250000, 0.0117187500000000,
+0.0329589843750000, 0.0187683105468750, 0.0292663574218750, 0.0074768066406250,
+0.0302734375000000, 0.0053710937500000, 0.0257263183593750, 0.0091247558593750,
+0.0231933593750000, 0.0066223144531250, 0.0266723632812500, 0.0055541992187500,
+0.0021057128906250, 0.0122680664062500, 0.0095520019531250, 0.0149536132812500,
+-0.0026550292968750, 0.0069885253906250, -0.0289001464843750, 0.0070495605468750,
+-0.0158386230468750, 0.0130004882812500, -0.0343322753906250, -0.0204772949218750,
+-0.0404663085937500, -0.0183105468750000, -0.0352478027343750, -0.0132446289062500,
+-0.0347900390625000, -0.0307617187500000, -0.0374145507812500, -0.0313415527343750,
+-0.0477294921875000, -0.0149230957031250, -0.0382385253906250, -0.0136718750000000,
+-0.0517272949218750, -0.0234680175781250, -0.0885620117187500, -0.0266113281250000,
+-0.1034545898437500, -0.0217285156250000, -0.0621032714843750, -0.0182189941406250,
+-0.0774230957031250, -0.0136413574218750, -0.0676269531250000, -0.0254516601562500,
+-0.0472106933593750, -0.0305786132812500, -0.0549926757812500, -0.0228881835937500,
+-0.0323181152343750, -0.0615539550781250, -0.0407104492187500, -0.0575256347656250,
+-0.0227050781250000, -0.0591125488281250, 0.0030517578125000, -0.0875854492187500,
+-0.0148925781250000, -0.0808105468750000, 0.0381774902343750, -0.0590209960937500,
+0.0331420898437500, -0.0733642578125000, 0.0426025390625000, -0.0573120117187500,
+0.0821533203125000, -0.0251464843750000, 0.0649414062500000, -0.0415954589843750,
+0.0684204101562500, -0.0238037109375000, 0.0859680175781250, -0.0170898437500000,
+0.0631103515625000, -0.0217285156250000, 0.0405578613281250, -0.0222778320312500,
+0.0604858398437500, -0.0214538574218750, 0.0207214355468750, -0.0130004882812500,
+0.0149841308593750, -0.0191955566406250, 0.0264892578125000, -0.0109252929687500,
+0.0148010253906250, 0.0037536621093750, 0.0116882324218750, -0.0042724609375000,
+0.0393066406250000, 0.0072021484375000, 0.0329589843750000, 0.0099792480468750,
+0.0393981933593750, 0.0088195800781250, 0.0624694824218750, 0.0105590820312500,
+0.0534973144531250, 0.0100402832031250, 0.0613403320312500, 0.0188903808593750,
+0.0666198730468750, 0.0156860351562500, 0.0596618652343750, 0.0198669433593750,
+0.0607299804687500, 0.0309448242187500, 0.0639343261718750, 0.0263061523437500,
+0.0265808105468750, 0.0255737304687500, 0.0417785644531250, 0.0314941406250000,
+0.0220031738281250, 0.0235900878906250, -0.0213928222656250, 0.0149536132812500,
+0.0014953613281250, 0.0212707519531250, -0.0251770019531250, 0.0046081542968750,
+-0.0345458984375000, 0.0063476562500000, -0.0276489257812500, 0.0010681152343750,
+-0.0292968750000000, -0.0098571777343750, -0.0299072265625000, -0.0032653808593750,
+-0.0295410156250000, -0.0251464843750000, -0.0254211425781250, -0.0211486816406250,
+-0.0307312011718750, -0.0288085937500000, -0.0355529785156250, -0.0475463867187500,
+-0.0291442871093750, -0.0386657714843750, -0.0420532226562500, -0.0559082031250000,
+-0.0430908203125000, -0.0572814941406250, -0.0393676757812500, -0.0577392578125000,
+-0.0448303222656250, -0.0664672851562500, -0.0448913574218750, -0.0633239746093750,
+-0.0281372070312500, -0.0646362304687500, -0.0364990234375000, -0.0690612792968750,
+-0.0239868164062500, -0.0598449707031250, -0.0022888183593750, -0.0539855957031250,
+-0.0158691406250000, -0.0607604980468750, 0.0061340332031250, -0.0330810546875000,
+0.0111999511718750, -0.0367126464843750, 0.0043334960937500, -0.0314025878906250,
+0.0087280273437500, -0.0115356445312500, 0.0103454589843750, -0.0195922851562500,
+-0.0030517578125000, -0.0083923339843750, 0.0011596679687500, -0.0081176757812500,
+-0.0057983398437500, -0.0035400390625000, -0.0183410644531250, 0.0030517578125000,
+-0.0102844238281250, -0.0021972656250000, -0.0287170410156250, 0.0194091796875000,
+-0.0273742675781250, 0.0155029296875000, -0.0298767089843750, 0.0227966308593750,
+-0.0413208007812500, 0.0403747558593750, -0.0355834960937500, 0.0323791503906250,
+-0.0437622070312500, 0.0530395507812500, -0.0442810058593750, 0.0514526367187500,
+-0.0437316894531250, 0.0572204589843750, -0.0476379394531250, 0.0731811523437500,
+-0.0465698242187500, 0.0655517578125000, -0.0466613769531250, 0.0762939453125000,
+-0.0484008789062500, 0.0810546875000000, -0.0469055175781250, 0.0738220214843750,
+-0.0455322265625000, 0.0723876953125000, -0.0474548339843750, 0.0762634277343750,
+-0.0423278808593750, 0.0595703125000000, -0.0443725585937500, 0.0612487792968750,
+-0.0419921875000000, 0.0571289062500000, -0.0376281738281250, 0.0452880859375000,
+-0.0416564941406250, 0.0499572753906250, -0.0333557128906250, 0.0386962890625000,
+-0.0354003906250000, 0.0376892089843750, -0.0328674316406250, 0.0372924804687500,
+-0.0265502929687500, 0.0303955078125000, -0.0311889648437500, 0.0310974121093750,
+-0.0223388671875000, 0.0349121093750000, -0.0235900878906250, 0.0303955078125000,
+-0.0223999023437500, 0.0351257324218750, -0.0168457031250000, 0.0420837402343750,
+-0.0202331542968750, 0.0359802246093750, -0.0143737792968750, 0.0462341308593750,
+-0.0156555175781250, 0.0462646484375000, -0.0151672363281250, 0.0448608398437500,
+-0.0111694335937500, 0.0492553710937500, -0.0137939453125000, 0.0476989746093750,
+-0.0112915039062500, 0.0460815429687500, -0.0106811523437500, 0.0466308593750000,
+-0.0122680664062500, 0.0476074218750000, -0.0125122070312500, 0.0465698242187500,
+-0.0119934082031250, 0.0461730957031250, -0.0129394531250000, 0.0516662597656250,
+-0.0138244628906250, 0.0509948730468750, -0.0109558105468750, 0.0501708984375000,
+-0.0096435546875000, 0.0547790527343750, -0.0111083984375000, 0.0543823242187500,
+-0.0018005371093750, 0.0435485839843750, -0.0033874511718750, 0.0502014160156250,
+0.0007324218750000, 0.0393371582031250, 0.0090942382812500, 0.0241699218750000,
+0.0057373046875000, 0.0351867675781250, 0.0158081054687500, 0.0090942382812500,
+0.0159301757812500, 0.0107727050781250, 0.0179748535156250, 0.0075683593750000,
+0.0245666503906250, -0.0082092285156250, 0.0222778320312500, -0.0010986328125000,
+0.0271301269531250, -0.0135192871093750, 0.0286254882812500, -0.0121765136718750,
+0.0250854492187500, -0.0177917480468750, 0.0255737304687500, -0.0269470214843750,
+0.0269775390625000, -0.0205383300781250, 0.0138549804687500, -0.0429992675781250,
+0.0173034667968750, -0.0380554199218750, 0.0114135742187500, -0.0466308593750000,
+-0.0016479492187500, -0.0659179687500000, 0.0046691894531250, -0.0545654296875000,
+-0.0050964355468750, -0.0749511718750000, -0.0083007812500000, -0.0776672363281250,
+-0.0027160644531250, -0.0707702636718750, -0.0030212402343750, -0.0784912109375000,
+-0.0054321289062500, -0.0802001953125000, 0.0091857910156250, -0.0547180175781250,
+0.0095825195312500, -0.0655212402343750, 0.0084838867187500, -0.0513916015625000,
+0.0173950195312500, -0.0234985351562500, 0.0169677734375000, -0.0400085449218750,
+0.0027160644531250, -0.0097351074218750, 0.0110168457031250, -0.0084838867187500,
+-0.0037231445312500, -0.0072631835937500, -0.0230712890625000, 0.0075073242187500,
+-0.0099487304687500, 0.0021057128906250, -0.0528259277343750, 0.0143737792968750,
+-0.0512390136718750, 0.0119018554687500, -0.0509643554687500, 0.0177001953125000,
+-0.0771789550781250, 0.0291748046875000, -0.0709838867187500, 0.0212707519531250,
+-0.0559997558593750, 0.0330505371093750, -0.0644836425781250, 0.0379333496093750,
+-0.0548706054687500, 0.0282592773437500, -0.0348510742187500, 0.0267944335937500,
+-0.0436401367187500, 0.0335083007812500, -0.0324707031250000, 0.0103759765625000,
+-0.0288391113281250, 0.0109558105468750, -0.0295410156250000, 0.0148010253906250,
+-0.0275878906250000, 0.0030822753906250, -0.0291748046875000, 0.0038452148437500,
+-0.0249633789062500, 0.0200500488281250, -0.0249023437500000, 0.0139770507812500,
+-0.0229797363281250, 0.0193176269531250, -0.0217590332031250, 0.0360107421875000,
+-0.0234375000000000, 0.0283508300781250, -0.0074462890625000, 0.0301513671875000,
+-0.0095214843750000, 0.0377502441406250, -0.0098571777343750, 0.0275573730468750,
+-0.0003356933593750, 0.0178833007812500, -0.0011901855468750, 0.0257568359375000,
+0.0007934570312500, 0.0060119628906250, -0.0014953613281250, 0.0061340332031250,
+0.0006103515625000, 0.0032348632812500, 0.0089111328125000, -0.0108032226562500,
+0.0064392089843750, -0.0095520019531250, -0.0098876953125000, -0.0155029296875000,
+-0.0013122558593750, -0.0167541503906250, -0.0059204101562500, -0.0123901367187500,
+-0.0219421386718750, -0.0131835937500000, -0.0141601562500000, -0.0172119140625000,
+-0.0262451171875000, -0.0078735351562500, -0.0293884277343750, -0.0052490234375000,
+-0.0232238769531250, -0.0133972167968750, -0.0263061523437500, -0.0134887695312500,
+-0.0299682617187500, -0.0093688964843750, -0.0088195800781250, -0.0374755859375000,
+-0.0134582519531250, -0.0339050292968750, -0.0097961425781250, -0.0363464355468750,
+0.0061035156250000, -0.0570678710937500, 0.0002746582031250, -0.0529785156250000,
+0.0103149414062500, -0.0485534667968750, 0.0093383789062500, -0.0533752441406250,
+0.0133666992187500, -0.0475463867187500, 0.0222778320312500, -0.0401306152343750,
+0.0168457031250000, -0.0456237792968750, 0.0235900878906250, -0.0296936035156250,
+0.0267639160156250, -0.0290222167968750, 0.0225524902343750, -0.0320739746093750,
+0.0206298828125000},
+{0.0028381347656250, -0.0111999511718750, 0.0036010742187500, -0.0215454101562500,
+0.0023193359375000, -0.0154113769531250, 0.0088195800781250, -0.0346069335937500,
+0.0075378417968750, -0.0309448242187500, 0.0105895996093750, -0.0379333496093750,
+0.0169372558593750, -0.0540161132812500, 0.0141906738281250, -0.0448303222656250,
+0.0195922851562500, -0.0615844726562500, 0.0223083496093750, -0.0626525878906250,
+0.0186462402343750, -0.0599060058593750, 0.0183715820312500, -0.0677795410156250,
+0.0209960937500000, -0.0662536621093750, 0.0090332031250000, -0.0545959472656250,
+0.0114746093750000, -0.0630798339843750, 0.0071105957031250, -0.0501403808593750,
+-0.0036315917968750, -0.0323791503906250, 0.0015869140625000, -0.0457763671875000,
+-0.0054016113281250, -0.0158386230468750, -0.0086364746093750, -0.0171203613281250,
+-0.0037536621093750, -0.0136108398437500, -0.0018920898437500, 0.0039978027343750,
+-0.0046081542968750, -0.0033874511718750, 0.0024414062500000, 0.0117492675781250,
+0.0045776367187500, 0.0108642578125000, 0.0020446777343750, 0.0141296386718750,
+0.0037841796875000, 0.0244445800781250, 0.0045166015625000, 0.0190429687500000,
+-0.0074462890625000, 0.0286865234375000, -0.0003662109375000, 0.0301208496093750,
+-0.0149230957031250, 0.0267333984375000, -0.0343017578125000, 0.0303955078125000,
+-0.0203857421875000, 0.0314331054687500, -0.0490417480468750, 0.0185546875000000,
+-0.0548095703125000, 0.0223083496093750, -0.0460815429687500, 0.0203552246093750,
+-0.0520629882812500, 0.0085449218750000, -0.0551757812500000, 0.0126647949218750,
+-0.0384521484375000, 0.0174865722656250, -0.0391540527343750, 0.0118103027343750,
+-0.0399475097656250, 0.0180969238281250, -0.0310363769531250, 0.0287170410156250,
+-0.0320129394531250, 0.0226135253906250, -0.0351257324218750, 0.0301513671875000,
+-0.0333251953125000, 0.0346374511718750, -0.0350646972656250, 0.0264892578125000,
+-0.0388488769531250, 0.0244750976562500, -0.0357055664062500, 0.0305175781250000,
+-0.0350952148437500, 0.0076904296875000, -0.0412292480468750, 0.0111999511718750,
+-0.0305786132812500, 0.0053710937500000, -0.0185546875000000, -0.0123596191406250,
+-0.0277404785156250, -0.0048522949218750, -0.0149230957031250, -0.0170288085937500,
+-0.0115051269531250, -0.0192565917968750, -0.0118713378906250, -0.0181579589843750,
+-0.0079956054687500, -0.0234069824218750, -0.0082397460937500, -0.0229492187500000,
+-0.0035095214843750, -0.0179748535156250, -0.0027770996093750, -0.0202331542968750,
+-0.0063476562500000, -0.0216979980468750, -0.0064697265625000, -0.0170593261718750,
+-0.0047912597656250, -0.0180664062500000, -0.0127868652343750, -0.0309448242187500,
+-0.0108642578125000, -0.0260009765625000, -0.0167846679687500, -0.0339660644531250,
+-0.0254516601562500, -0.0498962402343750, -0.0190124511718750, -0.0415344238281250,
+-0.0303039550781250, -0.0473327636718750, -0.0354309082031250, -0.0554504394531250,
+-0.0269775390625000, -0.0460205078125000, -0.0249633789062500, -0.0369567871093750,
+-0.0309753417968750, -0.0433349609375000, -0.0134277343750000, -0.0296936035156250,
+-0.0146789550781250, -0.0306091308593750, -0.0110473632812500, -0.0264587402343750,
+-0.0010070800781250, -0.0139465332031250, -0.0066528320312500, -0.0197448730468750,
+0.0106201171875000, -0.0229492187500000, 0.0074768066406250, -0.0137634277343750,
+0.0113830566406250, -0.0265197753906250, 0.0258789062500000, -0.0430297851562500,
+0.0195617675781250, -0.0309753417968750, 0.0227661132812500, -0.0472717285156250,
+0.0270385742187500, -0.0561828613281250, 0.0226745605468750, -0.0420227050781250,
+0.0179748535156250, -0.0368957519531250, 0.0213317871093750, -0.0462341308593750,
+0.0211486816406250, -0.0156860351562500, 0.0169067382812500, -0.0187377929687500,
+0.0224914550781250, -0.0147094726562500, 0.0302734375000000, 0.0068359375000000,
+0.0252075195312500, -0.0021972656250000, 0.0258178710937500, -0.0038452148437500,
+0.0338134765625000, 0.0050659179687500, 0.0205078125000000, -0.0059204101562500,
+0.0083312988281250, -0.0205383300781250, 0.0200805664062500, -0.0119628906250000,
+-0.0074157714843750, -0.0246887207031250, -0.0076599121093750, -0.0303039550781250,
+-0.0057983398437500, -0.0213317871093750, -0.0182800292968750, -0.0201416015625000,
+-0.0154724121093750, -0.0254821777343750, -0.0173034667968750, -0.0018920898437500,
+-0.0155639648437500, -0.0050659179687500, -0.0211181640625000, 0.0018005371093750,
+-0.0253906250000000, 0.0203247070312500, -0.0197448730468750, 0.0129699707031250,
+-0.0379943847656250, 0.0287475585937500, -0.0351867675781250, 0.0301513671875000,
+-0.0373535156250000, 0.0304260253906250, -0.0500488281250000, 0.0379638671875000,
+-0.0446777343750000, 0.0358581542968750, -0.0484924316406250, 0.0390930175781250,
+-0.0513000488281250, 0.0387573242187500, -0.0478515625000000, 0.0403747558593750,
+-0.0462646484375000, 0.0431823730468750, -0.0479431152343750, 0.0415344238281250,
+-0.0412902832031250, 0.0466308593750000, -0.0431213378906250, 0.0461425781250000,
+-0.0397338867187500, 0.0469360351562500, -0.0343933105468750, 0.0503234863281250,
+-0.0385131835937500, 0.0491027832031250, -0.0259094238281250, 0.0518798828125000,
+-0.0298767089843750, 0.0520935058593750, -0.0234069824218750, 0.0519104003906250,
+-0.0121154785156250, 0.0537414550781250, -0.0199279785156250, 0.0533142089843750,
+0.0024414062500000, 0.0499877929687500, -0.0014953613281250, 0.0526428222656250,
+0.0065917968750000, 0.0473632812500000, 0.0242309570312500, 0.0413513183593750,
+0.0142517089843750, 0.0453796386718750, 0.0402832031250000, 0.0304565429687500,
+0.0368957519531250, 0.0324707031250000, 0.0442504882812500, 0.0270080566406250,
+0.0633544921875000, 0.0151977539062500, 0.0530700683593750, 0.0212402343750000,
+0.0751953125000000, 0.0068969726562500, 0.0744628906250000, 0.0075073242187500,
+0.0773010253906250, 0.0047607421875000, 0.0912475585937500, -0.0047302246093750,
+0.0856018066406250, -0.0000610351562500, 0.0932312011718750, -0.0067138671875000,
+0.0946960449218750, -0.0086669921875000, 0.0934448242187500, -0.0069885253906250,
+0.0957336425781250, -0.0084228515625000, 0.0953674316406250, -0.0092468261718750,
+0.0950317382812500, -0.0068359375000000, 0.0953063964843750, -0.0079956054687500,
+0.0956726074218750, -0.0060424804687500, 0.0961303710937500, -0.0039062500000000,
+0.0961608886718750, -0.0061035156250000, 0.0948791503906250, 0.0030822753906250,
+0.0987854003906250, -0.0004577636718750, 0.0910644531250000, 0.0040893554687500,
+0.0857543945312500, 0.0134887695312500, 0.0923461914062500, 0.0081176757812500,
+0.0627746582031250, 0.0207519531250000, 0.0693054199218750, 0.0198364257812500,
+0.0572814941406250, 0.0225524902343750, 0.0307922363281250, 0.0310668945312500,
+0.0436706542968750, 0.0269470214843750, 0.0146484375000000, 0.0363159179687500,
+0.0133666992187500, 0.0369873046875000, 0.0100097656250000, 0.0357360839843750,
+-0.0061035156250000, 0.0411071777343750, -0.0007324218750000, 0.0407409667968750,
+-0.0137023925781250, 0.0307922363281250, -0.0122375488281250, 0.0376281738281250,
+-0.0196533203125000, 0.0289306640625000, -0.0309143066406250, 0.0155639648437500,
+-0.0238342285156250, 0.0256958007812500, -0.0458679199218750, 0.0032043457031250,
+-0.0448913574218750, 0.0043640136718750, -0.0473327636718750, 0.0017700195312500,
+-0.0611877441406250, -0.0117797851562500, -0.0565490722656250, -0.0062561035156250,
+-0.0615844726562500, -0.0164184570312500, -0.0638122558593750, -0.0155334472656250,
+-0.0615539550781250, -0.0169982910156250, -0.0625915527343750, -0.0253601074218750,
+-0.0629272460937500, -0.0213623046875000, -0.0530090332031250, -0.0173034667968750,
+-0.0597534179687500, -0.0240783691406250, -0.0459899902343750, -0.0138549804687500,
+-0.0297546386718750, 0.0003662109375000, -0.0422058105468750, -0.0091552734375000,
+-0.0071105957031250, 0.0014648437500000, -0.0077514648437500, 0.0109863281250000,
+-0.0077514648437500, -0.0077819824218750, 0.0122985839843750, -0.0172729492187500,
+0.0071716308593750, -0.0042114257812500, 0.0019531250000000, -0.0539245605468750,
+0.0070495605468750, -0.0503540039062500, 0.0034179687500000, -0.0581970214843750,
+-0.0062561035156250, -0.0917053222656250, -0.0024719238281250, -0.0786132812500000,
+0.0008850097656250, -0.0893859863281250, -0.0041503906250000, -0.1022338867187500,
+0.0028381347656250, -0.0776062011718750, 0.0126953125000000, -0.0657653808593750,
+0.0071411132812500, -0.0824584960937500, 0.0173950195312500, -0.0051574707031250,
+0.0195617675781250, -0.0138854980468750, 0.0179443359375000, -0.0050354003906250,
+0.0205383300781250, 0.0491333007812500, 0.0205688476562500, 0.0317077636718750,
+0.0197143554687500, 0.0352478027343750, 0.0200805664062500, 0.0469970703125000,
+0.0180969238281250, 0.0358581542968750, 0.0177307128906250, 0.0234985351562500,
+0.0187683105468750, 0.0314025878906250, 0.0061950683593750, 0.0155944824218750,
+0.0114746093750000, 0.0120544433593750, 0.0013122558593750, 0.0191650390625000,
+-0.0148315429687500, 0.0149536132812500, -0.0052795410156250, 0.0122680664062500,
+-0.0260314941406250, 0.0407409667968750, -0.0279846191406250, 0.0353698730468750,
+-0.0274353027343750, 0.0354309082031250, -0.0355834960937500, 0.0591430664062500,
+-0.0340576171875000, 0.0547485351562500, -0.0410461425781250, 0.0274047851562500,
+-0.0356750488281250, 0.0419006347656250, -0.0489501953125000, 0.0205688476562500,
+-0.0626525878906250, -0.0173950195312500, -0.0508728027343750, 0.0044250488281250,
+-0.0864562988281250, -0.0230712890625000, -0.0883483886718750, -0.0368652343750000,
+-0.0808715820312500, -0.0182495117187500, -0.0971984863281250, -0.0101013183593750,
+-0.0978698730468750, -0.0213928222656250, -0.0640258789062500, -0.0007019042968750,
+-0.0724792480468750, 0.0044250488281250, -0.0687866210937500, -0.0019531250000000,
+-0.0296630859375000, -0.0000305175781250, -0.0190734863281250, -0.0288696289062500,
+0.0272216796875000, 0.0052185058593750, 0.0047302246093750, -0.0003356933593750,
+0.0194396972656250, -0.0050659179687500, 0.0544433593750000, 0.0060424804687500,
+0.0421447753906250, 0.0092773437500000, 0.0374145507812500, -0.0198059082031250,
+0.0491943359375000, -0.0173950195312500, 0.0384521484375000, -0.0142517089843750,
+0.0203552246093750, -0.0299072265625000, 0.0300903320312500, -0.0295104980468750,
+0.0217285156250000, -0.0177612304687500, 0.0150451660156250, -0.0164794921875000,
+0.0252380371093750, -0.0253906250000000, 0.0325012207031250, -0.0240478515625000,
+0.0253601074218750, -0.0177612304687500, 0.0362854003906250, -0.0459899902343750,
+0.0423889160156250, -0.0470581054687500, 0.0326232910156250, -0.0409851074218750,
+0.0286865234375000, -0.0540771484375000, 0.0356140136718750, -0.0556030273437500,
+0.0157470703125000, -0.0286560058593750, 0.0151672363281250, -0.0356750488281250,
+0.0220336914062500, -0.0292358398437500, 0.0136413574218750, -0.0057067871093750,
+0.0111389160156250, -0.0146179199218750, 0.0364685058593750, -0.0093078613281250,
+0.0333251953125000, -0.0061645507812500, 0.0276184082031250, -0.0065612792968750,
+0.0475463867187500, -0.0077514648437500, 0.0466918945312500, -0.0089721679687500,
+-0.0109252929687500, 0.0019226074218750, 0.0148925781250000, -0.0006408691406250,
+-0.0190124511718750, 0.0011596679687500, -0.0866088867187500, 0.0106201171875000,
+-0.0493774414062500, 0.0076599121093750, -0.1096801757812500, 0.0057678222656250,
+-0.1200866699218750, 0.0077514648437500, -0.1069946289062500, 0.0070800781250000,
+-0.1266479492187500, 0.0034790039062500, -0.1256713867187500, 0.0040588378906250,
+-0.0898132324218750},
+{0.0063476562500000, -0.0281677246093750, 0.0167846679687500, -0.0606994628906250,
+0.0180969238281250, -0.0568237304687500, 0.0174255371093750, -0.0658874511718750,
+0.0213317871093750, -0.0929565429687500, 0.0205078125000000, -0.0801391601562500,
+0.0213928222656250, -0.0871887207031250, 0.0216674804687500, -0.1051940917968750,
+0.0203552246093750, -0.0736389160156250, 0.0210876464843750, -0.0482177734375000,
+0.0214233398437500, -0.0734863281250000, 0.0127563476562500, -0.0063171386718750,
+0.0172424316406250, -0.0047607421875000, 0.0086975097656250, -0.0042724609375000,
+-0.0032653808593750, 0.0289001464843750, 0.0044860839843750, 0.0187377929687500,
+-0.0162048339843750, 0.0252685546875000, -0.0160522460937500, 0.0315856933593750,
+-0.0187683105468750, 0.0233154296875000, -0.0307006835937500, 0.0186462402343750,
+-0.0264587402343750, 0.0246582031250000, -0.0374145507812500, 0.0097045898437500,
+-0.0346069335937500, 0.0046997070312500, -0.0430297851562500, 0.0186462402343750,
+-0.0540771484375000, 0.0220336914062500, -0.0457763671875000, 0.0117187500000000,
+-0.0717468261718750, 0.0389099121093750, -0.0713500976562500, 0.0439147949218750,
+-0.0704650878906250, 0.0363159179687500, -0.0869140625000000, 0.0451965332031250,
+-0.0832824707031250, 0.0462951660156250, -0.0648803710937500, 0.0114440917968750,
+-0.0757446289062500, 0.0252990722656250, -0.0609436035156250, 0.0086669921875000,
+-0.0343627929687500, -0.0310058593750000, -0.0488586425781250, -0.0117492675781250,
+-0.0275268554687500, -0.0295104980468750, -0.0236206054687500, -0.0393981933593750,
+-0.0229492187500000, -0.0275878906250000, -0.0088195800781250, -0.0170288085937500,
+0.0242309570312500, -0.0065612792968750, 0.0290222167968750, -0.0319213867187500,
+0.0399780273437500, -0.0396728515625000, 0.0267333984375000, -0.0195312500000000,
+0.0114440917968750, -0.0086364746093750, 0.0222778320312500, -0.0235290527343750,
+0.0165100097656250, 0.0084228515625000, 0.0070190429687500, 0.0184936523437500,
+0.0189208984375000, -0.0005493164062500, 0.0309448242187500, -0.0025634765625000,
+0.0220031738281250, 0.0091247558593750, 0.0361938476562500, -0.0254516601562500,
+0.0429992675781250, -0.0294799804687500, 0.0305480957031250, -0.0210876464843750,
+0.0268859863281250, -0.0301208496093750, 0.0357360839843750, -0.0330200195312500,
+0.0093383789062500, -0.0220031738281250, 0.0078735351562500, -0.0179748535156250,
+0.0131835937500000, -0.0280761718750000, 0.0022888183593750, -0.0313415527343750,
+0.0021972656250000, -0.0242309570312500, 0.0204772949218750, -0.0453796386718750,
+0.0155334472656250, -0.0491333007812500, 0.0161743164062500, -0.0401916503906250,
+0.0351257324218750, -0.0451965332031250, 0.0297546386718750, -0.0498657226562500,
+-0.0042419433593750, -0.0222167968750000, 0.0216064453125000, -0.0257568359375000,
+-0.0182800292968750, -0.0226440429687500, -0.0755920410156250, -0.0037536621093750,
+-0.0362854003906250, -0.0098571777343750, -0.1159057617187500, -0.0073242187500000,
+-0.1239929199218750, -0.0059509277343750, -0.1140441894531250, -0.0037536621093750,
+-0.1460266113281250, -0.0029296875000000, -0.1408081054687500, -0.0056152343750000,
+-0.1157836914062500, 0.0063171386718750, -0.1310119628906250, 0.0055236816406250,
+-0.1084289550781250, 0.0052185058593750, -0.0736389160156250, 0.0126342773437500,
+-0.0968627929687500, 0.0112304687500000, -0.0534362792968750, 0.0068359375000000,
+-0.0479431152343750, 0.0084838867187500, -0.0548400878906250, 0.0079956054687500,
+-0.0419616699218750, 0.0038146972656250, -0.0424804687500000, 0.0046997070312500,
+-0.0398254394531250, 0.0079345703125000, -0.0440063476562500, 0.0073547363281250,
+-0.0366516113281250, 0.0072937011718750, -0.0282897949218750, 0.0100402832031250,
+-0.0350952148437500, 0.0098876953125000, -0.0167236328125000, 0.0050048828125000,
+-0.0173339843750000, 0.0075378417968750, -0.0174560546875000, 0.0038146972656250,
+-0.0072631835937500, -0.0015869140625000, -0.0099792480468750, 0.0016174316406250,
+-0.0099182128906250, -0.0115661621093750, -0.0080871582031250, -0.0075683593750000,
+-0.0097961425781250, -0.0156250000000000, -0.0118408203125000, -0.0294189453125000,
+-0.0097656250000000, -0.0209350585937500, -0.0108642578125000, -0.0407714843750000,
+-0.0140686035156250, -0.0406188964843750, -0.0075683593750000, -0.0420532226562500,
+-0.0028686523437500, -0.0531616210937500, -0.0081787109375000, -0.0488891601562500,
+0.0077209472656250, -0.0544128417968750, 0.0066223144531250, -0.0560607910156250,
+0.0082397460937500, -0.0531311035156250, 0.0176696777343750, -0.0548095703125000,
+0.0137939453125000, -0.0557861328125000, 0.0196838378906250, -0.0428161621093750,
+0.0191040039062500, -0.0473632812500000, 0.0207824707031250, -0.0400695800781250,
+0.0247802734375000, -0.0256652832031250, 0.0217285156250000, -0.0328369140625000,
+0.0281372070312500, -0.0196838378906250, 0.0270996093750000, -0.0186767578125000,
+0.0273437500000000, -0.0162963867187500, 0.0311584472656250, -0.0106811523437500,
+0.0286254882812500, -0.0140380859375000, 0.0301513671875000, 0.0032653808593750,
+0.0306091308593750, -0.0024414062500000, 0.0283813476562500, 0.0090026855468750,
+0.0277404785156250, 0.0276489257812500, 0.0284423828125000, 0.0168457031250000,
+0.0239868164062500, 0.0473632812500000, 0.0247497558593750, 0.0449523925781250,
+0.0225830078125000, 0.0524291992187500, 0.0187988281250000, 0.0749511718750000,
+0.0210266113281250, 0.0653991699218750, 0.0155639648437500, 0.0816345214843750,
+0.0162048339843750, 0.0866088867187500, 0.0146484375000000, 0.0811462402343750,
+0.0107421875000000, 0.0842895507812500, 0.0133056640625000, 0.0868835449218750,
+0.0093688964843750, 0.0724487304687500, 0.0095214843750000, 0.0758361816406250,
+0.0097351074218750, 0.0709838867187500, 0.0077819824218750, 0.0593566894531250,
+0.0091857910156250, 0.0656127929687500, 0.0101013183593750, 0.0527343750000000,
+0.0092773437500000, 0.0547180175781250, 0.0122070312500000, 0.0492858886718750,
+0.0146484375000000, 0.0399780273437500, 0.0130310058593750, 0.0473327636718750,
+0.0204772949218750, 0.0262756347656250, 0.0197143554687500, 0.0305786132812500,
+0.0211791992187500, 0.0242004394531250, 0.0264892578125000, 0.0075073242187500,
+0.0241699218750000, 0.0171813964843750, 0.0262145996093750, -0.0003356933593750,
+0.0270996093750000, 0.0002441406250000, 0.0256042480468750, -0.0025939941406250,
+0.0251159667968750, -0.0122070312500000, 0.0252380371093750, -0.0068969726562500,
+0.0222473144531250, -0.0234985351562500, 0.0230102539062500, -0.0194396972656250,
+0.0202941894531250, -0.0274047851562500, 0.0169372558593750, -0.0432739257812500,
+0.0188598632812500, -0.0336608886718750, 0.0113525390625000, -0.0515747070312500,
+0.0122375488281250, -0.0538635253906250, 0.0106811523437500, -0.0498657226562500,
+0.0051574707031250, -0.0559082031250000, 0.0077514648437500, -0.0557861328125000,
+0.0057983398437500, -0.0456542968750000, 0.0043334960937500, -0.0489196777343750,
+0.0072326660156250, -0.0435791015625000, 0.0089416503906250, -0.0344543457031250,
+0.0073852539062500, -0.0403137207031250, 0.0127868652343750, -0.0250244140625000,
+0.0131225585937500, -0.0291137695312500, 0.0127563476562500, -0.0229187011718750,
+0.0156860351562500, -0.0097045898437500, 0.0149841308593750, -0.0180358886718750,
+0.0103149414062500, 0.0020446777343750, 0.0136108398437500, 0.0000915527343750,
+0.0072326660156250, 0.0046386718750000, -0.0009765625000000, 0.0192565917968750,
+0.0045471191406250, 0.0119934082031250, -0.0091552734375000, 0.0221557617187500,
+-0.0104675292968750, 0.0255737304687500, -0.0090332031250000, 0.0212402343750000,
+-0.0150146484375000, 0.0219726562500000, -0.0135192871093750, 0.0245666503906250,
+-0.0059204101562500, 0.0182800292968750, -0.0097045898437500, 0.0181884765625000,
+-0.0018005371093750, 0.0189819335937500, 0.0102539062500000, 0.0167236328125000,
+0.0035095214843750, 0.0172119140625000, 0.0137329101562500, 0.0167236328125000,
+0.0194702148437500, 0.0165405273437500, 0.0111694335937500, 0.0175781250000000,
+0.0084228515625000, 0.0177917480468750, 0.0132446289062500, 0.0169372558593750,
+-0.0071716308593750, 0.0190429687500000, -0.0018920898437500, 0.0198364257812500,
+-0.0111999511718750, 0.0174255371093750, -0.0310974121093750, 0.0175170898437500,
+-0.0193481445312500, 0.0188903808593750, -0.0339660644531250, 0.0092163085937500,
+-0.0386962890625000, 0.0136413574218750, -0.0335083007812500, 0.0051574707031250,
+-0.0340270996093750, -0.0071105957031250, -0.0347290039062500, 0.0009460449218750,
+-0.0285034179687500, -0.0205383300781250, -0.0271911621093750, -0.0211181640625000,
+-0.0307617187500000, -0.0206604003906250, -0.0300903320312500, -0.0315246582031250,
+-0.0267028808593750, -0.0292968750000000, -0.0341186523437500, -0.0290222167968750,
+-0.0340576171875000, -0.0314636230468750, -0.0348815917968750, -0.0296936035156250,
+-0.0390014648437500, -0.0270080566406250, -0.0364074707031250, -0.0289001464843750,
+-0.0367736816406250, -0.0264587402343750, -0.0395202636718750, -0.0262756347656250,
+-0.0366821289062500, -0.0272216796875000, -0.0324707031250000, -0.0266418457031250,
+-0.0334472656250000, -0.0269775390625000, -0.0295410156250000, -0.0296936035156250,
+-0.0315551757812500, -0.0301208496093750, -0.0296630859375000, -0.0277099609375000,
+-0.0247802734375000, -0.0276794433593750, -0.0278015136718750, -0.0292968750000000,
+-0.0263061523437500, -0.0225830078125000, -0.0239562988281250, -0.0232543945312500,
+-0.0252075195312500, -0.0218505859375000, -0.0288085937500000, -0.0162658691406250,
+-0.0278320312500000, -0.0181274414062500, -0.0184020996093750, -0.0201721191406250,
+-0.0226135253906250, -0.0183715820312500, -0.0210266113281250, -0.0185241699218750,
+-0.0105285644531250, -0.0189819335937500, -0.0137939453125000, -0.0192871093750000,
+-0.0201416015625000, -0.0315551757812500, -0.0160522460937500, -0.0257568359375000,
+-0.0213317871093750, -0.0305786132812500, -0.0309753417968750, -0.0465698242187500,
+-0.0264587402343750, -0.0406799316406250, -0.0328063964843750, -0.0370788574218750,
+-0.0359191894531250, -0.0433044433593750, -0.0315551757812500, -0.0366516113281250,
+-0.0317687988281250, -0.0256652832031250, -0.0348205566406250, -0.0313110351562500,
+-0.0197143554687500, -0.0263061523437500, -0.0248413085937500, -0.0236206054687500,
+-0.0162658691406250, -0.0244750976562500, -0.0007324218750000, -0.0238647460937500,
+-0.0100097656250000, -0.0238037109375000, 0.0115661621093750, -0.0263061523437500,
+0.0115356445312500, -0.0258178710937500, 0.0110473632812500, -0.0212097167968750,
+0.0227355957031250, -0.0220336914062500, 0.0195922851562500, -0.0246887207031250,
+0.0171508789062500, -0.0051879882812500, 0.0196838378906250, -0.0077209472656250,
+0.0195617675781250, -0.0029907226562500, 0.0157470703125000, 0.0114440917968750,
+0.0163574218750000, 0.0061035156250000, 0.0227050781250000, 0.0164489746093750,
+0.0213317871093750, 0.0169372558593750, 0.0202636718750000, 0.0191955566406250,
+0.0259399414062500, 0.0254821777343750, 0.0260925292968750, 0.0226135253906250,
+0.0128784179687500, 0.0303344726562500, 0.0199279785156250, 0.0311889648437500,
+0.0121765136718750, 0.0306701660156250, -0.0037841796875000, 0.0343017578125000,
+0.0060729980468750, 0.0343322753906250, -0.0075683593750000, 0.0319519042968750,
+-0.0088195800781250, 0.0326843261718750, -0.0079345703125000, 0.0314636230468750,
+-0.0123291015625000, 0.0290832519531250, -0.0103454589843750, 0.0299682617187500,
+-0.0189819335937500, 0.0267333984375000, -0.0148620605468750, 0.0262756347656250,
+-0.0212402343750000},
+{0.0133361816406250, 0.0090637207031250, 0.0029602050781250, 0.0033264160156250,
+-0.0039672851562500, -0.0058288574218750, 0.0043640136718750, -0.0001525878906250,
+-0.0263671875000000, -0.0182800292968750, -0.0248718261718750, -0.0170288085937500,
+-0.0242309570312500, -0.0187377929687500, -0.0415954589843750, -0.0312805175781250,
+-0.0370788574218750, -0.0281066894531250, -0.0310974121093750, -0.0285339355468750,
+-0.0363159179687500, -0.0309143066406250, -0.0293273925781250, -0.0298461914062500,
+-0.0170898437500000, -0.0282592773437500, -0.0231628417968750, -0.0292358398437500,
+-0.0208740234375000, -0.0265808105468750, -0.0141296386718750, -0.0276794433593750,
+-0.0209350585937500, -0.0267944335937500, -0.0285339355468750, -0.0243835449218750,
+-0.0226135253906250, -0.0258789062500000, -0.0334777832031250, -0.0228576660156250,
+-0.0351562500000000, -0.0230712890625000, -0.0310058593750000, -0.0223999023437500,
+-0.0312194824218750, -0.0209655761718750, -0.0306396484375000, -0.0214233398437500,
+-0.0284729003906250, -0.0145568847656250, -0.0282592773437500, -0.0168151855468750,
+-0.0277404785156250, -0.0178833007812500, -0.0271301269531250, -0.0126037597656250,
+-0.0276184082031250, -0.0127258300781250, -0.0240478515625000, -0.0162963867187500,
+-0.0233459472656250, -0.0150451660156250, -0.0259094238281250, -0.0169677734375000,
+-0.0252380371093750, -0.0208129882812500, -0.0226440429687500, -0.0182800292968750,
+-0.0298461914062500, -0.0187377929687500, -0.0312194824218750, -0.0187988281250000,
+-0.0263061523437500, -0.0225830078125000, -0.0268249511718750, -0.0238952636718750,
+-0.0300903320312500, -0.0192260742187500, -0.0178222656250000, -0.0292053222656250,
+-0.0181579589843750, -0.0313415527343750, -0.0195617675781250, -0.0285034179687500,
+-0.0130615234375000, -0.0291748046875000, -0.0140991210937500, -0.0297851562500000,
+-0.0218505859375000, -0.0295410156250000, -0.0181274414062500, -0.0290222167968750,
+-0.0240783691406250, -0.0280151367187500, -0.0346679687500000, -0.0289611816406250,
+-0.0289611816406250, -0.0289916992187500, -0.0366821289062500, -0.0186157226562500,
+-0.0414733886718750, -0.0219116210937500, -0.0340881347656250, -0.0189514160156250,
+-0.0319213867187500, -0.0091857910156250, -0.0368652343750000, -0.0113525390625000,
+-0.0158996582031250, -0.0031738281250000, -0.0206298828125000, -0.0053100585937500,
+-0.0129699707031250, -0.0007629394531250, 0.0046691894531250, 0.0085754394531250,
+-0.0053405761718750, 0.0044860839843750, 0.0142211914062500, 0.0118713378906250,
+0.0152587890625000, 0.0119628906250000, 0.0135192871093750, 0.0158386230468750,
+0.0224914550781250, 0.0217590332031250, 0.0207519531250000, 0.0179748535156250,
+0.0186767578125000, 0.0279541015625000, 0.0192565917968750, 0.0289611816406250,
+0.0210571289062500, 0.0271301269531250, 0.0204772949218750, 0.0315246582031250,
+0.0192871093750000, 0.0314941406250000, 0.0256042480468750, 0.0226440429687500,
+0.0264587402343750, 0.0250854492187500, 0.0224914550781250, 0.0218200683593750,
+0.0245056152343750, 0.0131225585937500, 0.0270690917968750, 0.0162963867187500,
+0.0110778808593750, 0.0128479003906250, 0.0163879394531250, 0.0108947753906250,
+0.0108642578125000, 0.0124206542968750, -0.0037841796875000, 0.0130004882812500,
+0.0043945312500000, 0.0122985839843750, -0.0069580078125000, 0.0136413574218750,
+-0.0079345703125000, 0.0145874023437500, -0.0079345703125000, 0.0125122070312500,
+-0.0122375488281250, 0.0119323730468750, -0.0099182128906250, 0.0130920410156250,
+-0.0168457031250000, 0.0047302246093750, -0.0139770507812500, 0.0067443847656250,
+-0.0179138183593750, 0.0023498535156250, -0.0253295898437500, -0.0061035156250000,
+-0.0200805664062500, -0.0025634765625000, -0.0300292968750000, -0.0128479003906250,
+-0.0306091308593750, -0.0133666992187500, -0.0290527343750000, -0.0153198242187500,
+-0.0330200195312500, -0.0220031738281250, -0.0322875976562500, -0.0204772949218750,
+-0.0293579101562500, -0.0246887207031250, -0.0302734375000000, -0.0263671875000000,
+-0.0294494628906250, -0.0255737304687500, -0.0273742675781250, -0.0267639160156250,
+-0.0281982421875000, -0.0278625488281250, -0.0252990722656250, -0.0247497558593750,
+-0.0281066894531250, -0.0258178710937500, -0.0222167968750000, -0.0244750976562500,
+-0.0173034667968750, -0.0212402343750000, -0.0226440429687500, -0.0226135253906250,
+0.0003051757812500, -0.0193481445312500, -0.0055847167968750, -0.0193176269531250,
+0.0046997070312500, -0.0177917480468750, 0.0260009765625000, -0.0155334472656250,
+0.0140991210937500, -0.0166015625000000, 0.0385131835937500, -0.0111083984375000,
+0.0397644042968750, -0.0108032226562500, 0.0385742187500000, -0.0097656250000000,
+0.0502624511718750, -0.0060729980468750, 0.0464477539062500, -0.0067749023437500,
+0.0417480468750000, -0.0054931640625000, 0.0460510253906250, -0.0042419433593750,
+0.0389404296875000, -0.0050964355468750, 0.0300292968750000, -0.0052795410156250,
+0.0354614257812500, -0.0041198730468750, 0.0201110839843750, -0.0073852539062500,
+0.0209350585937500, -0.0056152343750000, 0.0174560546875000, -0.0068359375000000,
+0.0080566406250000, -0.0101928710937500, 0.0125122070312500, -0.0078430175781250,
+-0.0017700195312500, -0.0111999511718750, 0.0016784667968750, -0.0109558105468750,
+-0.0071716308593750, -0.0107116699218750, -0.0212402343750000, -0.0126647949218750,
+-0.0126037597656250, -0.0118103027343750, -0.0377807617187500, -0.0110473632812500,
+-0.0352783203125000, -0.0135803222656250, -0.0400085449218750, -0.0092773437500000,
+-0.0570373535156250, -0.0064086914062500, -0.0490722656250000, -0.0110168457031250,
+-0.0628967285156250, 0.0048217773437500, -0.0644836425781250, 0.0005493164062500,
+-0.0610046386718750, 0.0046691894531250, -0.0669860839843750, 0.0176086425781250,
+-0.0661315917968750, 0.0102844238281250, -0.0495605468750000, 0.0206604003906250,
+-0.0570068359375000, 0.0220947265625000, -0.0443115234375000, 0.0207824707031250,
+-0.0229797363281250, 0.0246887207031250, -0.0349121093750000, 0.0235290527343750,
+-0.0118103027343750, 0.0215148925781250, -0.0080566406250000, 0.0238342285156250,
+-0.0115051269531250, 0.0188903808593750, -0.0046691894531250, 0.0146789550781250,
+-0.0048522949218750, 0.0192565917968750, -0.0062561035156250, 0.0046997070312500,
+-0.0078430175781250, 0.0064697265625000, -0.0034790039062500, 0.0041198730468750,
+-0.0006713867187500, -0.0052795410156250, -0.0038146972656250, -0.0014648437500000,
+0.0079956054687500, -0.0114440917968750, 0.0068664550781250, -0.0088195800781250,
+0.0087585449218750, -0.0148010253906250, 0.0164489746093750, -0.0252380371093750,
+0.0133666992187500, -0.0184326171875000, 0.0191040039062500, -0.0318603515625000,
+0.0193481445312500, -0.0355834960937500, 0.0192871093750000, -0.0278015136718750,
+0.0226745605468750, -0.0265808105468750, 0.0213012695312500, -0.0316162109375000,
+0.0190429687500000, -0.0241394042968750, 0.0220031738281250, -0.0170288085937500,
+0.0164184570312500, -0.0295715332031250, 0.0106506347656250, -0.0391845703125000,
+0.0146789550781250, -0.0299682617187500, -0.0011596679687500, -0.0504455566406250,
+0.0009460449218750, -0.0523681640625000, -0.0044250488281250, -0.0529479980468750,
+-0.0164794921875000, -0.0612487792968750, -0.0106201171875000, -0.0572814941406250,
+-0.0261535644531250, -0.0636901855468750, -0.0249938964843750, -0.0694885253906250,
+-0.0293579101562500, -0.0556945800781250, -0.0405578613281250, -0.0482788085937500,
+-0.0350646972656250, -0.0593261718750000, -0.0473327636718750, -0.0289916992187500,
+-0.0480957031250000, -0.0273742675781250, -0.0481872558593750, -0.0281982421875000,
+-0.0549621582031250, -0.0160217285156250, -0.0534667968750000, -0.0188598632812500,
+-0.0512084960937500, -0.0098266601562500, -0.0550537109375000, -0.0138244628906250,
+-0.0487365722656250, -0.0072326660156250, -0.0433654785156250, 0.0048217773437500,
+-0.0486450195312500, -0.0010070800781250, -0.0257873535156250, 0.0131225585937500,
+-0.0314636230468750, 0.0097961425781250, -0.0219726562500000, 0.0192565917968750,
+-0.0009460449218750, 0.0370178222656250, -0.0118713378906250, 0.0269775390625000,
+0.0098876953125000, 0.0333862304687500, 0.0106811523437500, 0.0396728515625000,
+0.0127563476562500, 0.0366516113281250, 0.0253601074218750, 0.0330810546875000,
+0.0198669433593750, 0.0340576171875000, 0.0226135253906250, 0.0323181152343750,
+0.0274353027343750, 0.0332336425781250, 0.0213623046875000, 0.0317077636718750,
+0.0106506347656250, 0.0271606445312500, 0.0171203613281250, 0.0327453613281250,
+0.0308227539062500, 0.0265197753906250, 0.0311584472656250, 0.0259094238281250,
+0.0268554687500000, 0.0229187011718750, 0.0276184082031250, 0.0256652832031250,
+0.0314941406250000, 0.0280151367187500, 0.0225524902343750, -0.0024414062500000,
+0.0208435058593750, 0.0063476562500000, 0.0243530273437500, -0.0008850097656250,
+0.0237426757812500, -0.0312194824218750, 0.0213012695312500, -0.0203552246093750,
+0.0232543945312500, -0.0112304687500000, 0.0277404785156250, -0.0225830078125000,
+0.0174560546875000, -0.0121154785156250, 0.0116271972656250, 0.0093994140625000,
+0.0193786621093750, 0.0000610351562500, -0.0138854980468750, 0.0017089843750000,
+-0.0078125000000000, 0.0109558105468750, -0.0179138183593750, -0.0002441406250000,
+-0.0421752929687500, -0.0138549804687500, -0.0294189453125000, -0.0050964355468750,
+-0.0658569335937500, -0.0169372558593750, -0.0606079101562500, -0.0220947265625000,
+-0.0700073242187500, -0.0165710449218750, -0.0979309082031250, -0.0143737792968750,
+-0.0844116210937500, -0.0176086425781250, -0.1081542968750000, -0.0170593261718750,
+-0.1105041503906250, -0.0120544433593750, -0.1076660156250000, -0.0202331542968750,
+-0.1210632324218750, -0.0299377441406250, -0.1176147460937500, -0.0232238769531250,
+-0.0896911621093750, -0.0305786132812500, -0.1078491210937500, -0.0375366210937500,
+-0.0806884765625000, -0.0265197753906250, -0.0375366210937500, -0.0190734863281250,
+-0.0645446777343750, -0.0272827148437500, -0.0160522460937500, -0.0066528320312500,
+-0.0083312988281250, -0.0053405761718750, -0.0176391601562500, -0.0071105957031250,
+-0.0029296875000000, 0.0008239746093750, -0.0030517578125000, -0.0007324218750000,
+-0.0166625976562500, 0.0000305175781250, -0.0117492675781250, -0.0019226074218750,
+-0.0185852050781250, 0.0027465820312500, -0.0327453613281250, 0.0074157714843750,
+-0.0236206054687500, 0.0030822753906250, -0.0313415527343750, 0.0134582519531250,
+-0.0377197265625000, 0.0151367187500000, -0.0278320312500000, 0.0119628906250000,
+-0.0228271484375000, 0.0147705078125000, -0.0292968750000000, 0.0161437988281250,
+-0.0064086914062500, 0.0085754394531250, -0.0116577148437500, 0.0096435546875000,
+-0.0035400390625000, 0.0087585449218750, 0.0154418945312500, 0.0040893554687500,
+0.0047302246093750, 0.0058288574218750, 0.0270996093750000, 0.0020141601562500,
+0.0275573730468750, 0.0032958984375000, 0.0267944335937500, 0.0003967285156250,
+0.0393981933593750, -0.0037536621093750, 0.0360107421875000, -0.0008544921875000,
+0.0278625488281250, -0.0098266601562500, 0.0344238281250000, -0.0079345703125000,
+0.0267333984375000, -0.0125122070312500, 0.0143737792968750, -0.0197448730468750,
+0.0225830078125000, -0.0147705078125000, 0.0072021484375000, -0.0318298339843750,
+0.0079345703125000, -0.0289916992187500, 0.0065612792968750, -0.0338134765625000,
+-0.0025024414062500, -0.0477905273437500, 0.0021057128906250, -0.0413513183593750,
+-0.0033264160156250, -0.0506896972656250, -0.0047607421875000, -0.0533447265625000,
+-0.0027160644531250, -0.0501403808593750, -0.0042419433593750, -0.0527648925781250,
+-0.0052185058593750},
+{0.0122375488281250, 0.0424194335937500, 0.0158081054687500, 0.0371704101562500,
+0.0130004882812500, 0.0368347167968750, 0.0093078613281250, 0.0415649414062500,
+0.0152587890625000, 0.0361022949218750, 0.0251770019531250, 0.0243225097656250,
+0.0340270996093750, 0.0370788574218750, 0.0282287597656250, 0.0306091308593750,
+0.0267944335937500, 0.0401611328125000, 0.0303955078125000, 0.0329895019531250,
+0.0328979492187500, 0.0198974609375000, 0.0298156738281250, 0.0249023437500000,
+0.0311889648437500, 0.0243835449218750, 0.0381164550781250, 0.0238952636718750,
+0.0225830078125000, 0.0180358886718750, 0.0120849609375000, 0.0169982910156250,
+0.0244140625000000, 0.0223999023437500, -0.0198974609375000, -0.0029602050781250,
+-0.0158081054687500, -0.0003662109375000, -0.0208129882812500, -0.0008239746093750,
+-0.0501403808593750, -0.0174865722656250, -0.0397644042968750, -0.0132141113281250,
+-0.0466613769531250, -0.0053710937500000, -0.0474853515625000, -0.0101928710937500,
+-0.0554809570312500, -0.0057678222656250, -0.0611267089843750, 0.0056152343750000,
+-0.0529479980468750, 0.0012207031250000, -0.0838928222656250, -0.0010986328125000,
+-0.0800476074218750, 0.0046997070312500, -0.0850524902343750, -0.0024108886718750,
+-0.1085815429687500, -0.0129394531250000, -0.0982055664062500, -0.0068664550781250,
+-0.0950622558593750, -0.0132446289062500, -0.1117248535156250, -0.0172424316406250,
+-0.0809020996093750, -0.0132446289062500, -0.0530700683593750, -0.0099487304687500,
+-0.0797729492187500, -0.0126647949218750, -0.0090942382812500, -0.0145568847656250,
+-0.0053710937500000, -0.0107116699218750, -0.0146484375000000, -0.0154113769531250,
+0.0158691406250000, -0.0239257812500000, 0.0128479003906250, -0.0197448730468750,
+-0.0123901367187500, -0.0175781250000000, -0.0020141601562500, -0.0244750976562500,
+-0.0117797851562500, -0.0149536132812500, -0.0379333496093750, -0.0031127929687500,
+-0.0251464843750000, -0.0113220214843750, -0.0350341796875000, 0.0043029785156250,
+-0.0451660156250000, 0.0069885253906250, -0.0272216796875000, 0.0036315917968750,
+-0.0173950195312500, 0.0071105957031250, -0.0312194824218750, 0.0074768066406250,
+0.0088195800781250, 0.0060729980468750, 0.0071105957031250, 0.0040588378906250,
+0.0071716308593750, 0.0075988769531250, 0.0290527343750000, 0.0109558105468750,
+0.0218505859375000, 0.0080261230468750, 0.0220336914062500, 0.0131225585937500,
+0.0282897949218750, 0.0155029296875000, 0.0205688476562500, 0.0115051269531250,
+0.0132751464843750, 0.0103454589843750, 0.0195922851562500, 0.0128479003906250,
+0.0012512207031250, 0.0035400390625000, 0.0027465820312500, 0.0047607421875000,
+0.0007324218750000, 0.0022277832031250, -0.0122375488281250, -0.0041503906250000,
+-0.0072326660156250, -0.0008544921875000, -0.0047607421875000, -0.0101928710937500,
+-0.0112915039062500, -0.0086669921875000, -0.0033569335937500, -0.0112915039062500,
+0.0062255859375000, -0.0186462402343750, -0.0014343261718750, -0.0148010253906250,
+0.0150146484375000, -0.0212097167968750, 0.0130004882812500, -0.0218200683593750,
+0.0156555175781250, -0.0221557617187500, 0.0261230468750000, -0.0254211425781250,
+0.0199584960937500, -0.0242919921875000, 0.0304565429687500, -0.0262145996093750,
+0.0301208496093750, -0.0277099609375000, 0.0309753417968750, -0.0256347656250000,
+0.0370788574218750, -0.0252685546875000, 0.0342407226562500, -0.0274047851562500,
+0.0383300781250000, -0.0202026367187500, 0.0390930175781250, -0.0223083496093750,
+0.0381164550781250, -0.0192871093750000, 0.0401916503906250, -0.0140380859375000,
+0.0409240722656250, -0.0178833007812500, 0.0363769531250000, -0.0031738281250000,
+0.0398559570312500, -0.0079040527343750, 0.0368347167968750, 0.0009155273437500,
+0.0315856933593750, 0.0162658691406250, 0.0362548828125000, 0.0068359375000000,
+0.0254211425781250, 0.0290527343750000, 0.0283508300781250, 0.0291442871093750,
+0.0249023437500000, 0.0303344726562500, 0.0166320800781250, 0.0422058105468750,
+0.0231933593750000, 0.0378417968750000, 0.0109558105468750, 0.0432434082031250,
+0.0136413574218750, 0.0440063476562500, 0.0106811523437500, 0.0435485839843750,
+0.0022277832031250, 0.0458068847656250, 0.0084533691406250, 0.0451354980468750,
+-0.0044250488281250, 0.0457458496093750, -0.0016784667968750, 0.0462951660156250,
+-0.0045776367187500, 0.0451660156250000, -0.0140075683593750, 0.0451354980468750,
+-0.0079345703125000, 0.0455627441406250, -0.0173645019531250, 0.0396118164062500,
+-0.0163574218750000, 0.0429382324218750, -0.0159912109375000, 0.0357055664062500,
+-0.0205688476562500, 0.0275268554687500, -0.0168151855468750, 0.0334777832031250,
+-0.0183715820312500, 0.0100708007812500, -0.0183410644531250, 0.0147094726562500,
+-0.0163879394531250, 0.0060119628906250, -0.0156860351562500, -0.0144042968750000,
+-0.0150451660156250, -0.0039672851562500, -0.0130310058593750, -0.0221862792968750,
+-0.0131530761718750, -0.0258178710937500, -0.0116271972656250, -0.0211181640625000,
+-0.0097961425781250, -0.0256958007812500, -0.0104370117187500, -0.0270996093750000,
+-0.0056762695312500, -0.0198059082031250, -0.0065917968750000, -0.0199890136718750,
+-0.0051574707031250, -0.0206604003906250, -0.0011291503906250, -0.0182189941406250,
+-0.0030822753906250, -0.0186462402343750, -0.0010986328125000, -0.0159606933593750,
+-0.0007629394531250, -0.0205688476562500, -0.0016784667968750, -0.0115966796875000,
+-0.0018920898437500, -0.0029907226562500, -0.0023193359375000, -0.0117492675781250,
+-0.0028381347656250, 0.0119323730468750, -0.0033569335937500, 0.0111694335937500,
+-0.0030517578125000, 0.0114135742187500, -0.0028076171875000, 0.0252075195312500,
+-0.0032958984375000, 0.0210876464843750, -0.0021667480468750, 0.0188293457031250,
+-0.0021057128906250, 0.0234985351562500, -0.0017395019531250, 0.0185852050781250,
+-0.0014038085937500, 0.0110778808593750, -0.0014953613281250, 0.0164184570312500,
+0.0028076171875000, 0.0127563476562500, 0.0010070800781250, 0.0085449218750000,
+0.0048217773437500, 0.0145263671875000, 0.0104980468750000, 0.0194396972656250,
+0.0070495605468750, 0.0142211914062500, 0.0153503417968750, 0.0196533203125000,
+0.0157470703125000, 0.0234680175781250, 0.0152282714843750, 0.0177307128906250,
+0.0196228027343750, 0.0146179199218750, 0.0185852050781250, 0.0183410644531250,
+0.0142211914062500, 0.0076599121093750, 0.0173034667968750, 0.0094604492187500,
+0.0132751464843750, 0.0041198730468750, 0.0068664550781250, -0.0043334960937500,
+0.0108337402343750, 0.0017700195312500, 0.0021972656250000, -0.0154113769531250,
+0.0028686523437500, -0.0143432617187500, 0.0004272460937500, -0.0158081054687500,
+-0.0063781738281250, -0.0266418457031250, -0.0030212402343750, -0.0230712890625000,
+-0.0064392089843750, -0.0257873535156250, -0.0085449218750000, -0.0283813476562500,
+-0.0067443847656250, -0.0262756347656250, -0.0066223144531250, -0.0247497558593750,
+-0.0073242187500000, -0.0268554687500000, -0.0005187988281250, -0.0252685546875000,
+-0.0021667480468750, -0.0238342285156250, -0.0032653808593750, -0.0251770019531250,
+0.0031738281250000, -0.0279846191406250, 0.0030822753906250, -0.0271301269531250,
+-0.0098571777343750, -0.0196228027343750, -0.0058593750000000, -0.0246582031250000,
+-0.0122375488281250, -0.0200500488281250, -0.0252380371093750, -0.0093994140625000,
+-0.0182800292968750, -0.0149536132812500, -0.0297241210937500, -0.0092773437500000,
+-0.0356140136718750, -0.0074768066406250, -0.0256652832031250, -0.0086669921875000,
+-0.0216369628906250, -0.0073852539062500, -0.0285339355468750, -0.0073852539062500,
+-0.0103454589843750, -0.0088195800781250, -0.0079040527343750, -0.0080566406250000,
+-0.0136108398437500, -0.0110168457031250, -0.0102539062500000, -0.0138244628906250,
+-0.0107421875000000, -0.0125122070312500, -0.0233459472656250, -0.0214538574218750,
+-0.0209960937500000, -0.0210266113281250, -0.0198059082031250, -0.0177307128906250,
+-0.0264282226562500, -0.0202636718750000, -0.0241699218750000, -0.0211791992187500,
+-0.0218200683593750, -0.0166320800781250, -0.0249023437500000, -0.0142211914062500,
+-0.0209350585937500, -0.0189819335937500, -0.0143737792968750, -0.0202636718750000,
+-0.0185241699218750, -0.0161437988281250, -0.0186462402343750, -0.0300903320312500,
+-0.0138244628906250, -0.0300292968750000, -0.0190429687500000, -0.0301818847656250,
+-0.0260009765625000, -0.0379638671875000, -0.0213317871093750, -0.0352172851562500,
+-0.0266723632812500, -0.0315246582031250, -0.0285644531250000, -0.0392456054687500,
+-0.0263977050781250, -0.0247802734375000, -0.0265808105468750, -0.0090942382812500,
+-0.0269775390625000, -0.0209350585937500, -0.0232849121093750, 0.0075683593750000,
+-0.0247802734375000, 0.0100097656250000, -0.0210266113281250, 0.0072631835937500,
+-0.0166625976562500, 0.0188293457031250, -0.0204162597656250, 0.0178833007812500,
+-0.0098571777343750, 0.0156555175781250, -0.0089416503906250, 0.0155639648437500,
+-0.0127563476562500, 0.0172729492187500, -0.0088195800781250, 0.0182189941406250,
+-0.0074462890625000, 0.0173034667968750, -0.0223388671875000, 0.0184631347656250,
+-0.0197753906250000, 0.0177307128906250, -0.0212707519531250, 0.0202636718750000,
+-0.0331115722656250, 0.0233764648437500, -0.0300598144531250, 0.0206604003906250,
+-0.0249938964843750, 0.0213928222656250, -0.0310668945312500, 0.0239562988281250,
+-0.0226135253906250, 0.0213317871093750, -0.0122070312500000, 0.0179138183593750,
+-0.0202331542968750, 0.0200195312500000, 0.0005798339843750, 0.0178833007812500,
+-0.0023803710937500, 0.0173339843750000, 0.0014953613281250, 0.0177917480468750,
+0.0162658691406250, 0.0182495117187500, 0.0090026855468750, 0.0182189941406250,
+0.0192565917968750, 0.0132751464843750, 0.0206604003906250, 0.0165405273437500,
+0.0192565917968750, 0.0106201171875000, 0.0224304199218750, 0.0021667480468750,
+0.0217590332031250, 0.0069580078125000, 0.0241394042968750, -0.0059204101562500,
+0.0233459472656250, -0.0072021484375000, 0.0242614746093750, -0.0074462890625000,
+0.0273437500000000, -0.0139465332031250, 0.0260620117187500, -0.0120849609375000,
+0.0241699218750000, -0.0097045898437500, 0.0274353027343750, -0.0118103027343750,
+0.0231323242187500, -0.0077819824218750, 0.0178833007812500, -0.0025024414062500,
+0.0222167968750000, -0.0050964355468750, 0.0122375488281250, 0.0014038085937500,
+0.0135498046875000, 0.0024719238281250, 0.0114135742187500, 0.0019226074218750,
+0.0050659179687500, 0.0046691894531250, 0.0086975097656250, 0.0046386718750000,
+-0.0009765625000000, 0.0021362304687500, 0.0005798339843750, 0.0035705566406250,
+-0.0020141601562500, 0.0011291503906250, -0.0097351074218750, -0.0025329589843750,
+-0.0054626464843750, -0.0008850097656250, -0.0100708007812500, -0.0075683593750000,
+-0.0119934082031250, -0.0077514648437500, -0.0074768066406250, -0.0104370117187500,
+-0.0059509277343750, -0.0162353515625000, -0.0068359375000000, -0.0149841308593750,
+0.0047302246093750, -0.0189819335937500, 0.0037841796875000, -0.0216369628906250,
+0.0079956054687500, -0.0204467773437500, 0.0174255371093750, -0.0210571289062500,
+0.0137329101562500, -0.0232543945312500, 0.0198974609375000, -0.0177612304687500,
+0.0216674804687500, -0.0204467773437500, 0.0194702148437500, -0.0185852050781250,
+0.0202636718750000, -0.0137023925781250, 0.0213012695312500, -0.0174865722656250,
+0.0181274414062500, -0.0099792480468750, 0.0176696777343750, -0.0122070312500000,
+0.0184631347656250, -0.0103149414062500, 0.0178222656250000, -0.0046081542968750,
+0.0177612304687500, -0.0088500976562500, 0.0189208984375000, -0.0015258789062500,
+0.0190124511718750},
+{-0.0155639648437500, -0.0321044921875000, -0.0168762207031250, -0.0363464355468750,
+-0.0197753906250000, -0.0279541015625000, -0.0131530761718750, -0.0234069824218750,
+-0.0119323730468750, -0.0292663574218750, -0.0233764648437500, 0.0016784667968750,
+-0.0223693847656250, -0.0017395019531250, -0.0216979980468750, 0.0006713867187500,
+-0.0282592773437500, 0.0215454101562500, -0.0272827148437500, 0.0159912109375000,
+-0.0224914550781250, 0.0164184570312500, -0.0244750976562500, 0.0208129882812500,
+-0.0223083496093750, 0.0169677734375000, -0.0176391601562500, 0.0121765136718750,
+-0.0196533203125000, 0.0152587890625000, -0.0139465332031250, 0.0100402832031250,
+-0.0156860351562500, 0.0098876953125000, -0.0117797851562500, 0.0105285644531250,
+-0.0051269531250000, 0.0066528320312500, -0.0097045898437500, 0.0072021484375000,
+-0.0032653808593750, 0.0158691406250000, 0.0007324218750000, 0.0127258300781250,
+-0.0068359375000000, 0.0145874023437500, -0.0101623535156250, 0.0229492187500000,
+-0.0048828125000000, 0.0205993652343750, -0.0212402343750000, 0.0215759277343750,
+-0.0215759277343750, 0.0238952636718750, -0.0194396972656250, 0.0196533203125000,
+-0.0275573730468750, 0.0174865722656250, -0.0270996093750000, 0.0205688476562500,
+-0.0177612304687500, 0.0061340332031250, -0.0230102539062500, 0.0090026855468750,
+-0.0156555175781250, 0.0029296875000000, -0.0043029785156250, -0.0107727050781250,
+-0.0120849609375000, -0.0040283203125000, 0.0079650878906250, -0.0134582519531250,
+0.0057983398437500, -0.0175781250000000, 0.0085449218750000, -0.0122985839843750,
+0.0218505859375000, -0.0115051269531250, 0.0158691406250000, -0.0140991210937500,
+0.0232543945312500, -0.0030517578125000, 0.0243225097656250, -0.0033264160156250,
+0.0237731933593750, -0.0018310546875000, 0.0265197753906250, 0.0055847167968750,
+0.0255126953125000, 0.0031433105468750, 0.0271606445312500, 0.0012817382812500,
+0.0278625488281250, 0.0046691894531250, 0.0261840820312500, -0.0017700195312500,
+0.0262451171875000, -0.0085754394531250, 0.0273437500000000, -0.0044250488281250,
+0.0208740234375000, -0.0189819335937500, 0.0229797363281250, -0.0184936523437500,
+0.0209655761718750, -0.0226135253906250, 0.0150451660156250, -0.0330505371093750,
+0.0185241699218750, -0.0292358398437500, 0.0160827636718750, -0.0391235351562500,
+0.0143737792968750, -0.0398864746093750, 0.0179748535156250, -0.0404357910156250,
+0.0202331542968750, -0.0455932617187500, 0.0191040039062500, -0.0438842773437500,
+0.0265808105468750, -0.0443420410156250, 0.0270080566406250, -0.0453796386718750,
+0.0281066894531250, -0.0421752929687500, 0.0331115722656250, -0.0395812988281250,
+0.0324707031250000, -0.0404968261718750, 0.0316467285156250, -0.0330810546875000,
+0.0343627929687500, -0.0332031250000000, 0.0316772460937500, -0.0296020507812500,
+0.0279846191406250, -0.0235900878906250, 0.0307617187500000, -0.0256652832031250,
+0.0242309570312500, -0.0171203613281250, 0.0246276855468750, -0.0176391601562500,
+0.0229492187500000, -0.0144348144531250, 0.0186157226562500, -0.0077209472656250,
+0.0209350585937500, -0.0108032226562500, 0.0150756835937500, -0.0036926269531250,
+0.0156250000000000, -0.0032653808593750, 0.0130615234375000, -0.0030212402343750,
+0.0083618164062500, 0.0003051757812500, 0.0110168457031250, -0.0006713867187500,
+0.0038452148437500, 0.0015563964843750, 0.0035400390625000, 0.0009155273437500,
+0.0033569335937500, 0.0019226074218750, -0.0008239746093750, 0.0037231445312500,
+-0.0002441406250000, 0.0018310546875000, 0.0003662109375000, 0.0048522949218750,
+-0.0015869140625000, 0.0041809082031250, 0.0004272460937500, 0.0046386718750000,
+0.0028381347656250, 0.0066833496093750, 0.0007934570312500, 0.0051879882812500,
+0.0070190429687500, 0.0075378417968750, 0.0046386718750000, 0.0083007812500000,
+0.0103454589843750, 0.0062255859375000, 0.0173950195312500, 0.0067443847656250,
+0.0126037597656250, 0.0079040527343750, 0.0323486328125000, -0.0010986328125000,
+0.0303649902343750, 0.0026855468750000, 0.0343017578125000, -0.0025634765625000,
+0.0499877929687500, -0.0121459960937500, 0.0451965332031250, -0.0056457519531250,
+0.0463867187500000, -0.0179748535156250, 0.0516357421875000, -0.0184326171875000,
+0.0465698242187500, -0.0168151855468750, 0.0408630371093750, -0.0230102539062500,
+0.0453796386718750, -0.0219116210937500, 0.0374145507812500, -0.0158996582031250,
+0.0372924804687500, -0.0210266113281250, 0.0359802246093750, -0.0137023925781250,
+0.0322265625000000, -0.0036621093750000, 0.0351562500000000, -0.0110168457031250,
+0.0274658203125000, 0.0064086914062500, 0.0291442871093750, 0.0043334960937500,
+0.0267944335937500, 0.0072021484375000, 0.0204772949218750, 0.0192565917968750,
+0.0236511230468750, 0.0136413574218750, 0.0169677734375000, 0.0213623046875000,
+0.0176696777343750, 0.0232543945312500, 0.0148925781250000, 0.0211791992187500,
+0.0099487304687500, 0.0238037109375000, 0.0132446289062500, 0.0240478515625000,
+0.0038146972656250, 0.0155029296875000, 0.0049438476562500, 0.0199584960937500,
+0.0023193359375000, 0.0122985839843750, -0.0041503906250000, 0.0005798339843750,
+-0.0006408691406250, 0.0068969726562500, -0.0100097656250000, -0.0120849609375000,
+-0.0084228515625000, -0.0126953125000000, -0.0125122070312500, -0.0122985839843750,
+-0.0209045410156250, -0.0230407714843750, -0.0162658691406250, -0.0207824707031250,
+-0.0256652832031250, -0.0122680664062500, -0.0267944335937500, -0.0150756835937500,
+-0.0256958007812500, -0.0132751464843750, -0.0292968750000000, -0.0052490234375000,
+-0.0290222167968750, -0.0068359375000000, -0.0271911621093750, -0.0068969726562500,
+-0.0281982421875000, -0.0052795410156250, -0.0268249511718750, -0.0083007812500000,
+-0.0253906250000000, -0.0083007812500000, -0.0267333984375000, -0.0064697265625000,
+-0.0189819335937500, -0.0271606445312500, -0.0227661132812500, -0.0187683105468750,
+-0.0150451660156250, -0.0290527343750000, -0.0052185058593750, -0.0533447265625000,
+-0.0124206542968750, -0.0421752929687500, 0.0100402832031250, -0.0555114746093750,
+0.0080566406250000, -0.0618896484375000, 0.0117187500000000, -0.0513305664062500,
+0.0280761718750000, -0.0491638183593750, 0.0216674804687500, -0.0555114746093750,
+0.0236816406250000, -0.0317382812500000, 0.0291748046875000, -0.0346984863281250,
+0.0231018066406250, -0.0290527343750000, 0.0157775878906250, -0.0105895996093750,
+0.0209655761718750, -0.0190429687500000, 0.0176696777343750, -0.0150146484375000,
+0.0126953125000000, -0.0103454589843750, 0.0194702148437500, -0.0099792480468750,
+0.0258178710937500, -0.0122985839843750, 0.0211181640625000, -0.0140686035156250,
+0.0312194824218750, -0.0064392089843750, 0.0326538085937500, -0.0067749023437500,
+0.0316162109375000, -0.0041809082031250, 0.0353393554687500, 0.0011596679687500,
+0.0349121093750000, -0.0025024414062500, 0.0321655273437500, 0.0003967285156250,
+0.0321044921875000, 0.0022583007812500, 0.0349426269531250, 0.0050354003906250,
+0.0369873046875000, 0.0085449218750000, 0.0302429199218750, 0.0109252929687500,
+-0.0203857421875000, 0.0046691894531250, -0.0021972656250000, 0.0013122558593750,
+-0.0128479003906250, 0.0107116699218750, -0.0408020019531250, 0.0150146484375000,
+-0.0309448242187500, 0.0082702636718750, -0.0335693359375000, 0.0322875976562500,
+-0.0401611328125000, 0.0326232910156250, -0.0339050292968750, 0.0288085937500000,
+-0.0252075195312500, 0.0399475097656250, -0.0298767089843750, 0.0397338867187500,
+-0.0325927734375000, 0.0262756347656250, -0.0235290527343750, 0.0297546386718750,
+-0.0382690429687500, 0.0262451171875000, -0.0552368164062500, 0.0144653320312500,
+-0.0409851074218750, 0.0191040039062500, -0.0614318847656250, 0.0166625976562500,
+-0.0698547363281250, 0.0149536132812500, -0.0590209960937500, 0.0142822265625000,
+-0.0573120117187500, 0.0154418945312500, -0.0622863769531250, 0.0164184570312500,
+-0.0412902832031250, 0.0039672851562500, -0.0453796386718750, 0.0072021484375000,
+-0.0385742187500000, 0.0047302246093750, -0.0196533203125000, -0.0071716308593750,
+-0.0285034179687500, -0.0034179687500000, -0.0220031738281250, 0.0012207031250000,
+-0.0150451660156250, -0.0025634765625000, -0.0203247070312500, 0.0004272460937500,
+-0.0279541015625000, 0.0086364746093750, -0.0235290527343750, 0.0058898925781250,
+-0.0137329101562500, 0.0019226074218750, -0.0219726562500000, 0.0051879882812500,
+-0.0136413574218750, 0.0025939941406250, 0.0016479492187500, -0.0041198730468750,
+-0.0063171386718750, -0.0018310546875000, 0.0121765136718750, -0.0012207031250000,
+0.0114440917968750, -0.0027465820312500, 0.0121765136718750, -0.0028991699218750,
+0.0239868164062500, -0.0006713867187500, 0.0202026367187500, -0.0008544921875000,
+0.0165405273437500, -0.0072021484375000, 0.0217895507812500, -0.0061340332031250,
+0.0168457031250000, -0.0060729980468750, 0.0068054199218750, -0.0118103027343750,
+0.0114746093750000, -0.0113830566406250, 0.0118103027343750, -0.0018615722656250,
+0.0064697265625000, -0.0059204101562500, 0.0108337402343750, -0.0012817382812500,
+0.0162963867187500, 0.0101318359375000, 0.0117187500000000, 0.0051574707031250,
+0.0232849121093750, 0.0087890625000000, 0.0214843750000000, 0.0117492675781250,
+0.0224304199218750, 0.0092163085937500, 0.0314941406250000, 0.0066833496093750,
+0.0280456542968750, 0.0079956054687500, 0.0237731933593750, 0.0099792480468750,
+0.0303955078125000, 0.0072326660156250, 0.0191345214843750, 0.0108642578125000,
+0.0070495605468750, 0.0158386230468750, 0.0172424316406250, 0.0127258300781250,
+-0.0081176757812500, 0.0200195312500000, -0.0084533691406250, 0.0191650390625000,
+-0.0081481933593750, 0.0217590332031250, -0.0204162597656250, 0.0268859863281250,
+-0.0166015625000000, 0.0232543945312500, -0.0189514160156250, 0.0322265625000000,
+-0.0203857421875000, 0.0314331054687500, -0.0186157226562500, 0.0317077636718750,
+-0.0187683105468750, 0.0371704101562500, -0.0196533203125000, 0.0350646972656250,
+-0.0144653320312500, 0.0351562500000000, -0.0163574218750000, 0.0362243652343750,
+-0.0132446289062500, 0.0343322753906250, -0.0081176757812500, 0.0329895019531250,
+-0.0116882324218750, 0.0338134765625000, -0.0026855468750000, 0.0279846191406250,
+-0.0036315917968750, 0.0297851562500000, -0.0017700195312500, 0.0255737304687500,
+0.0046081542968750, 0.0191650390625000, 0.0017089843750000, 0.0229492187500000,
+0.0073852539062500, 0.0122680664062500, 0.0079956054687500, 0.0134582519531250,
+0.0088500976562500, 0.0101013183593750, 0.0119628906250000, 0.0020446777343750,
+0.0111083984375000, 0.0063476562500000, 0.0142517089843750, -0.0026855468750000,
+0.0152587890625000, -0.0035705566406250, 0.0146789550781250, -0.0017395019531250,
+0.0163879394531250, -0.0053710937500000, 0.0178527832031250, -0.0049133300781250,
+0.0142517089843750, 0.0021667480468750, 0.0166625976562500, 0.0009155273437500,
+0.0145874023437500, 0.0031127929687500, 0.0105590820312500, 0.0095214843750000,
+0.0141601562500000, 0.0077209472656250, 0.0082397460937500, 0.0089721679687500,
+0.0094604492187500, 0.0106811523437500, 0.0084838867187500, 0.0101318359375000,
+0.0049133300781250, 0.0086975097656250, 0.0080566406250000, 0.0098266601562500,
+0.0014953613281250, 0.0134887695312500, 0.0038757324218750, 0.0103454589843750,
+0.0015258789062500, 0.0171508789062500, -0.0043029785156250, 0.0245971679687500,
+-0.0001831054687500, 0.0192260742187500, -0.0079345703125000, 0.0366516113281250,
+-0.0072631835937500, 0.0355834960937500, -0.0085144042968750, 0.0371093750000000,
+-0.0130310058593750},
+{-0.0055236816406250, 0.0062255859375000, 0.0169982910156250, -0.0058288574218750,
+0.0054321289062500, -0.0032043457031250, 0.0108642578125000, 0.0017700195312500,
+0.0184326171875000, -0.0004882812500000, 0.0136413574218750, -0.0001831054687500,
+0.0069274902343750, 0.0048217773437500, 0.0097656250000000, 0.0040283203125000,
+0.0104675292968750, -0.0012817382812500, 0.0071411132812500, 0.0001831054687500,
+0.0122680664062500, -0.0004882812500000, 0.0166931152343750, -0.0050048828125000,
+0.0123596191406250, -0.0041809082031250, 0.0274353027343750, -0.0026855468750000,
+0.0243835449218750, -0.0028991699218750, 0.0273742675781250, -0.0047912597656250,
+0.0403137207031250, -0.0045166015625000, 0.0343933105468750, -0.0031433105468750,
+0.0326232910156250, -0.0104675292968750, 0.0379638671875000, -0.0112609863281250,
+0.0331726074218750, -0.0082092285156250, 0.0262145996093750, -0.0114135742187500,
+0.0298461914062500, -0.0130615234375000, 0.0202941894531250, 0.0020751953125000,
+0.0226135253906250, -0.0022888183593750, 0.0170898437500000, 0.0032653808593750,
+0.0079345703125000, 0.0172119140625000, 0.0140380859375000, 0.0107421875000000,
+-0.0021667480468750, 0.0220947265625000, -0.0020751953125000, 0.0215759277343750,
+-0.0016784667968750, 0.0249328613281250, -0.0104675292968750, 0.0319213867187500,
+-0.0090637207031250, 0.0270996093750000, -0.0078430175781250, 0.0421142578125000,
+-0.0079040527343750, 0.0404052734375000, -0.0105285644531250, 0.0428161621093750,
+-0.0108642578125000, 0.0534057617187500, -0.0085144042968750, 0.0489196777343750,
+-0.0169982910156250, 0.0533447265625000, -0.0154724121093750, 0.0546569824218750,
+-0.0169677734375000, 0.0534973144531250, -0.0227661132812500, 0.0544433593750000,
+-0.0195312500000000, 0.0541381835937500, -0.0239868164062500, 0.0523986816406250,
+-0.0238037109375000, 0.0538940429687500, -0.0234375000000000, 0.0505676269531250,
+-0.0255737304687500, 0.0475769042968750, -0.0242004394531250, 0.0502014160156250,
+-0.0242614746093750, 0.0386962890625000, -0.0248107910156250, 0.0419311523437500,
+-0.0229797363281250, 0.0350341796875000, -0.0218811035156250, 0.0242919921875000,
+-0.0228271484375000, 0.0313415527343750, -0.0176086425781250, 0.0099487304687500,
+-0.0192260742187500, 0.0129699707031250, -0.0167541503906250, 0.0071105957031250,
+-0.0119934082031250, -0.0090942382812500, -0.0151672363281250, -0.0011291503906250,
+-0.0087280273437500, -0.0169677734375000, -0.0097656250000000, -0.0167541503906250,
+-0.0079650878906250, -0.0190734863281250, -0.0038146972656250, -0.0286254882812500,
+-0.0071411132812500, -0.0239257812500000, 0.0013732910156250, -0.0313110351562500,
+-0.0009460449218750, -0.0321350097656250, 0.0019836425781250, -0.0299072265625000,
+0.0094604492187500, -0.0321655273437500, 0.0045166015625000, -0.0314941406250000,
+0.0107421875000000, -0.0253295898437500, 0.0114135742187500, -0.0278320312500000,
+0.0086364746093750, -0.0217590332031250, 0.0096130371093750, -0.0143127441406250,
+0.0091857910156250, -0.0190429687500000, 0.0035400390625000, -0.0017089843750000,
+0.0037231445312500, -0.0050048828125000, 0.0008850097656250, 0.0026855468750000,
+-0.0040893554687500, 0.0178833007812500, -0.0031433105468750, 0.0097045898437500,
+-0.0090637207031250, 0.0298767089843750, -0.0087585449218750, 0.0299072265625000,
+-0.0112304687500000, 0.0325012207031250, -0.0159606933593750, 0.0439147949218750,
+-0.0139770507812500, 0.0391540527343750, -0.0195922851562500, 0.0494079589843750,
+-0.0195922851562500, 0.0472106933593750, -0.0202026367187500, 0.0516662597656250,
+-0.0235290527343750, 0.0610961914062500, -0.0224304199218750, 0.0545654296875000,
+-0.0246582031250000, 0.0637512207031250, -0.0244140625000000, 0.0673522949218750,
+-0.0249633789062500, 0.0600585937500000, -0.0260620117187500, 0.0591735839843750,
+-0.0247497558593750, 0.0632629394531250, -0.0280456542968750, 0.0447692871093750,
+-0.0268249511718750, 0.0484008789062500, -0.0282287597656250, 0.0433349609375000,
+-0.0313720703125000, 0.0286865234375000, -0.0290832519531250, 0.0361938476562500,
+-0.0328674316406250, 0.0253601074218750, -0.0332336425781250, 0.0231628417968750,
+-0.0326232910156250, 0.0250244140625000, -0.0345458984375000, 0.0223388671875000,
+-0.0345764160156250, 0.0215759277343750, -0.0323486328125000, 0.0186157226562500,
+-0.0337829589843750, 0.0220336914062500, -0.0321960449218750, 0.0150146484375000,
+-0.0298156738281250, 0.0072326660156250, -0.0314331054687500, 0.0135192871093750,
+-0.0240173339843750, -0.0009765625000000, -0.0262756347656250, -0.0010375976562500,
+-0.0203552246093750, -0.0026245117187500, -0.0115966796875000, -0.0102539062500000,
+-0.0166320800781250, -0.0066833496093750, -0.0021972656250000, -0.0145874023437500,
+-0.0016174316406250, -0.0150756835937500, -0.0012512207031250, -0.0130310058593750,
+0.0061340332031250, -0.0166015625000000, 0.0040893554687500, -0.0174865722656250,
+0.0046386718750000, -0.0112915039062500, 0.0050048828125000, -0.0123596191406250,
+0.0046386718750000, -0.0130310058593750, 0.0043640136718750, -0.0098876953125000,
+0.0038757324218750, -0.0106506347656250, 0.0043029785156250, -0.0107727050781250,
+0.0045471191406250, -0.0115966796875000, 0.0030517578125000, -0.0110168457031250,
+0.0026855468750000, -0.0101928710937500, 0.0037231445312500, -0.0111999511718750,
+-0.0007629394531250, -0.0087585449218750, 0.0005798339843750, -0.0100097656250000,
+-0.0028991699218750, -0.0078430175781250, -0.0073242187500000, -0.0040588378906250,
+-0.0033874511718750, -0.0066223144531250, -0.0133666992187500, -0.0043640136718750,
+-0.0157775878906250, -0.0024414062500000, -0.0104675292968750, -0.0044555664062500,
+-0.0109863281250000, -0.0060424804687500, -0.0137939453125000, -0.0049438476562500,
+-0.0029907226562500, -0.0073242187500000, -0.0038757324218750, -0.0061645507812500,
+-0.0033569335937500, -0.0105590820312500, 0.0048522949218750, -0.0149841308593750,
+0.0025024414062500, -0.0101013183593750, -0.0072021484375000, -0.0166625976562500,
+-0.0022583007812500, -0.0187377929687500, -0.0071105957031250, -0.0180969238281250,
+-0.0185852050781250, -0.0186157226562500, -0.0136718750000000, -0.0169982910156250,
+-0.0210876464843750, -0.0178222656250000, -0.0236511230468750, -0.0187072753906250,
+-0.0194702148437500, -0.0191040039062500, -0.0183105468750000, -0.0182800292968750,
+-0.0182189941406250, -0.0180358886718750, -0.0089721679687500, -0.0241394042968750,
+-0.0111694335937500, -0.0230102539062500, -0.0122070312500000, -0.0215759277343750,
+-0.0057678222656250, -0.0249328613281250, -0.0057678222656250, -0.0244140625000000,
+-0.0104370117187500, -0.0187988281250000, -0.0103759765625000, -0.0213623046875000,
+-0.0079650878906250, -0.0167236328125000, -0.0076293945312500, -0.0108947753906250,
+-0.0094299316406250, -0.0151367187500000, -0.0120239257812500, 0.0015563964843750,
+-0.0090332031250000, 0.0013732910156250, -0.0084533691406250, -0.0021057128906250,
+-0.0136413574218750, 0.0055847167968750, -0.0137023925781250, 0.0069885253906250,
+-0.0028076171875000, -0.0012817382812500, -0.0050659179687500, -0.0004272460937500,
+-0.0049743652343750, -0.0031738281250000, 0.0034790039062500, -0.0090637207031250,
+0.0021057128906250, -0.0051574707031250, -0.0042419433593750, -0.0093078613281250,
+-0.0003662109375000, -0.0136718750000000, -0.0052795410156250, -0.0057678222656250,
+-0.0142211914062500, -0.0008544921875000, -0.0088500976562500, -0.0070190429687500,
+-0.0159301757812500, 0.0107421875000000, -0.0191040039062500, 0.0126953125000000,
+-0.0141906738281250, 0.0061340332031250, -0.0130615234375000, 0.0114746093750000,
+-0.0160217285156250, 0.0146789550781250, -0.0064086914062500, -0.0037536621093750,
+-0.0072021484375000, -0.0010681152343750, -0.0057373046875000, -0.0051574707031250,
+-0.0002136230468750, -0.0190734863281250, -0.0030822753906250, -0.0124511718750000,
+0.0033874511718750, -0.0170898437500000, 0.0032653808593750, -0.0227355957031250,
+0.0018615722656250, -0.0142517089843750, 0.0053710937500000, -0.0076293945312500,
+0.0052490234375000, -0.0137634277343750, -0.0014343261718750, 0.0000000000000000,
+0.0014038085937500, 0.0025024414062500, -0.0022277832031250, -0.0023193359375000,
+-0.0099182128906250, 0.0015563964843750, -0.0060119628906250, 0.0032653808593750,
+-0.0126342773437500, -0.0127258300781250, -0.0146484375000000, -0.0090332031250000,
+-0.0107421875000000, -0.0157775878906250, -0.0119018554687500, -0.0309753417968750,
+-0.0143737792968750, -0.0245971679687500, -0.0019226074218750, -0.0366821289062500,
+-0.0067443847656250, -0.0390319824218750, -0.0007324218750000, -0.0370178222656250,
+0.0112915039062500, -0.0406494140625000, 0.0032653808593750, -0.0404357910156250,
+0.0201416015625000, -0.0361328125000000, 0.0185241699218750, -0.0366821289062500,
+0.0227661132812500, -0.0360107421875000, 0.0341491699218750, -0.0328674316406250,
+0.0281372070312500, -0.0336303710937500, 0.0444030761718750, -0.0341491699218750,
+0.0429992675781250, -0.0333557128906250, 0.0461730957031250, -0.0337524414062500,
+0.0580444335937500, -0.0349426269531250, 0.0534362792968750, -0.0346374511718750,
+0.0592346191406250, -0.0345153808593750, 0.0617675781250000, -0.0352783203125000,
+0.0596313476562500, -0.0339965820312500, 0.0596923828125000, -0.0329284667968750,
+0.0605163574218750, -0.0338439941406250, 0.0576171875000000, -0.0297241210937500,
+0.0590820312500000, -0.0307006835937500, 0.0553588867187500, -0.0276184082031250,
+0.0520935058593750, -0.0231323242187500, 0.0547180175781250, -0.0254211425781250,
+0.0386047363281250, -0.0169982910156250, 0.0431518554687500, -0.0169982910156250,
+0.0338134765625000, -0.0147094726562500, 0.0173950195312500, -0.0087585449218750,
+0.0261230468750000, -0.0106201171875000, 0.0025634765625000, -0.0061035156250000,
+0.0040588378906250, -0.0049438476562500, -0.0019531250000000, -0.0040283203125000,
+-0.0182800292968750, -0.0015869140625000, -0.0107727050781250, -0.0020446777343750,
+-0.0279235839843750, -0.0003356933593750, -0.0285034179687500, 0.0009460449218750,
+-0.0302429199218750, 0.0009765625000000, -0.0401611328125000, 0.0014038085937500,
+-0.0366821289062500, 0.0022583007812500, -0.0410766601562500, 0.0010681152343750,
+-0.0435791015625000, 0.0021667480468750, -0.0402832031250000, 0.0009460449218750,
+-0.0404968261718750, -0.0005187988281250, -0.0421447753906250, 0.0009460449218750,
+-0.0285034179687500, -0.0050964355468750, -0.0344848632812500, -0.0033264160156250,
+-0.0240478515625000, -0.0060424804687500, -0.0079040527343750, -0.0117797851562500,
+-0.0185546875000000, -0.0087585449218750, 0.0079040527343750, -0.0161743164062500,
+0.0071411132812500, -0.0160522460937500, 0.0100708007812500, -0.0180664062500000,
+0.0254211425781250, -0.0229187011718750, 0.0191955566406250, -0.0207824707031250,
+0.0304260253906250, -0.0274047851562500, 0.0288391113281250, -0.0272827148437500,
+0.0330810546875000, -0.0282897949218750, 0.0414123535156250, -0.0330200195312500,
+0.0357666015625000, -0.0315856933593750, 0.0525817871093750, -0.0314636230468750,
+0.0506896972656250, -0.0341796875000000, 0.0550231933593750, -0.0308227539062500,
+0.0672607421875000, -0.0280151367187500, 0.0610351562500000, -0.0313110351562500,
+0.0718994140625000, -0.0204162597656250, 0.0737915039062500, -0.0224609375000000,
+0.0708923339843750, -0.0195007324218750, 0.0748291015625000, -0.0107421875000000,
+0.0751037597656250, -0.0147094726562500, 0.0636291503906250, -0.0072326660156250,
+0.0682983398437500, -0.0073547363281250, 0.0598449707031250, -0.0076599121093750,
+0.0469360351562500, -0.0028381347656250, 0.0546875000000000, -0.0043029785156250,
+0.0320739746093750},
+{0.0039367675781250, -0.0260314941406250, -0.0057678222656250, -0.0333862304687500,
+-0.0038452148437500, -0.0355834960937500, -0.0064697265625000, -0.0344848632812500,
+-0.0148315429687500, -0.0361938476562500, -0.0109863281250000, -0.0361022949218750,
+-0.0132751464843750, -0.0350036621093750, -0.0175170898437500, -0.0354003906250000,
+-0.0093078613281250, -0.0354003906250000, -0.0048828125000000, -0.0343017578125000,
+-0.0110473632812500, -0.0345764160156250, 0.0147094726562500, -0.0361633300781250,
+0.0089416503906250, -0.0357666015625000, 0.0175476074218750, -0.0355834960937500,
+0.0392456054687500, -0.0369567871093750, 0.0279235839843750, -0.0365295410156250,
+0.0479736328125000, -0.0328369140625000, 0.0488586425781250, -0.0341186523437500,
+0.0495300292968750, -0.0303344726562500, 0.0594787597656250, -0.0252380371093750,
+0.0552673339843750, -0.0273132324218750, 0.0606994628906250, -0.0183410644531250,
+0.0624084472656250, -0.0188903808593750, 0.0584106445312500, -0.0154724121093750,
+0.0589599609375000, -0.0084838867187500, 0.0603637695312500, -0.0115051269531250,
+0.0469360351562500, -0.0030822753906250, 0.0506591796875000, -0.0028076171875000,
+0.0433654785156250, -0.0027465820312500, 0.0299377441406250, 0.0016784667968750,
+0.0362548828125000, 0.0004577636718750, 0.0166625976562500, 0.0002136230468750,
+0.0181884765625000, 0.0011901855468750, 0.0113220214843750, -0.0006408691406250,
+-0.0030212402343750, -0.0026550292968750, 0.0042114257812500, -0.0019531250000000,
+-0.0162658691406250, -0.0053405761718750, -0.0144958496093750, -0.0056457519531250,
+-0.0195617675781250, -0.0067749023437500, -0.0342102050781250, -0.0088500976562500,
+-0.0276489257812500, -0.0081176757812500, -0.0416564941406250, -0.0117492675781250,
+-0.0424499511718750, -0.0120849609375000, -0.0431823730468750, -0.0129089355468750,
+-0.0511169433593750, -0.0149841308593750, -0.0483398437500000, -0.0144958496093750,
+-0.0477294921875000, -0.0162658691406250, -0.0516662597656250, -0.0162963867187500,
+-0.0447998046875000, -0.0162353515625000, -0.0389404296875000, -0.0170898437500000,
+-0.0440673828125000, -0.0166931152343750, -0.0248413085937500, -0.0166931152343750,
+-0.0270080566406250, -0.0171203613281250, -0.0234375000000000, -0.0171813964843750,
+-0.0102539062500000, -0.0169067382812500, -0.0159912109375000, -0.0169067382812500,
+-0.0073547363281250, -0.0172119140625000, -0.0075988769531250, -0.0177612304687500,
+-0.0041503906250000, -0.0172119140625000, 0.0009765625000000, -0.0173950195312500,
+-0.0034790039062500, -0.0184631347656250, 0.0132446289062500, -0.0148315429687500,
+0.0087585449218750, -0.0164489746093750, 0.0155944824218750, -0.0159912109375000,
+0.0303344726562500, -0.0127563476562500, 0.0218200683593750, -0.0151367187500000,
+0.0415649414062500, -0.0160827636718750, 0.0406188964843750, -0.0142822265625000,
+0.0425720214843750, -0.0176696777343750, 0.0545654296875000, -0.0208740234375000,
+0.0492248535156250, -0.0179748535156250, 0.0537414550781250, -0.0254821777343750,
+0.0562744140625000, -0.0254821777343750, 0.0529479980468750, -0.0255737304687500,
+0.0517272949218750, -0.0297546386718750, 0.0529174804687500, -0.0285034179687500,
+0.0463867187500000, -0.0282287597656250, 0.0474853515625000, -0.0294799804687500,
+0.0439147949218750, -0.0260620117187500, 0.0388488769531250, -0.0244750976562500,
+0.0422058105468750, -0.0269775390625000, 0.0291137695312500, -0.0138244628906250,
+0.0319213867187500, -0.0165405273437500, 0.0264892578125000, -0.0131225585937500,
+0.0146789550781250, -0.0018920898437500, 0.0203247070312500, -0.0068359375000000,
+0.0068359375000000, -0.0036926269531250, 0.0077514648437500, -0.0001220703125000,
+0.0040588378906250, -0.0043945312500000, -0.0053710937500000, -0.0092468261718750,
+-0.0006103515625000, -0.0060729980468750, -0.0117492675781250, -0.0048522949218750,
+-0.0128784179687500, -0.0114440917968750, -0.0108032226562500, -0.0007324218750000,
+-0.0156860351562500, 0.0104675292968750, -0.0157775878906250, 0.0021667480468750,
+-0.0066833496093750, 0.0266723632812500, -0.0102844238281250, 0.0262145996093750,
+-0.0065002441406250, 0.0247802734375000, 0.0029602050781250, 0.0394592285156250,
+-0.0013732910156250, 0.0368652343750000, 0.0047302246093750, 0.0229187011718750,
+0.0056762695312500, 0.0307006835937500, 0.0063476562500000, 0.0197448730468750,
+0.0086059570312500, 0.0012512207031250, 0.0071105957031250, 0.0113830566406250,
+0.0133972167968750, -0.0115356445312500, 0.0124816894531250, -0.0107116699218750,
+0.0132141113281250, -0.0136413574218750, 0.0179138183593750, -0.0299377441406250,
+0.0166015625000000, -0.0236206054687500, 0.0169067382812500, -0.0193786621093750,
+0.0171203613281250, -0.0257263183593750, 0.0185852050781250, -0.0211181640625000,
+0.0195007324218750, -0.0093383789062500, 0.0183715820312500, -0.0138854980468750,
+0.0221557617187500, -0.0210266113281250, 0.0221862792968750, -0.0122070312500000,
+0.0223083496093750, -0.0228271484375000, 0.0244750976562500, -0.0405273437500000,
+0.0234985351562500, -0.0315551757812500, 0.0223693847656250, -0.0441894531250000,
+0.0260009765625000, -0.0465698242187500, 0.0168151855468750, -0.0445861816406250,
+0.0104370117187500, -0.0521850585937500, 0.0177307128906250, -0.0514526367187500,
+-0.0115966796875000, -0.0293884277343750, -0.0082702636718750, -0.0385742187500000,
+-0.0138854980468750, -0.0269470214843750, -0.0355529785156250, -0.0020446777343750,
+-0.0261840820312500, -0.0154724121093750, -0.0307312011718750, 0.0020751953125000,
+-0.0383605957031250, 0.0080261230468750, -0.0300903320312500, 0.0022888183593750,
+-0.0150451660156250, 0.0024414062500000, -0.0149230957031250, -0.0150146484375000,
+-0.0234375000000000, 0.0062255859375000, -0.0227355957031250, 0.0035705566406250,
+-0.0256958007812500, -0.0001831054687500, -0.0252380371093750, 0.0041198730468750,
+-0.0226745605468750, 0.0075683593750000, -0.0467834472656250, -0.0010681152343750,
+-0.0375061035156250, -0.0048828125000000, -0.0499267578125000, 0.0012512207031250,
+-0.0777893066406250, 0.0046081542968750, -0.0625915527343750, 0.0006408691406250,
+-0.0755310058593750, 0.0067138671875000, -0.0874023437500000, 0.0092468261718750,
+-0.0688781738281250, 0.0055541992187500, -0.0600585937500000, 0.0044860839843750,
+-0.0723876953125000, 0.0070495605468750, -0.0202026367187500, 0.0021972656250000,
+-0.0282287597656250, -0.0007629394531250, -0.0182800292968750, 0.0060424804687500,
+0.0204162597656250, 0.0101013183593750, 0.0043334960937500, 0.0050354003906250,
+0.0228881835937500, 0.0154113769531250, 0.0290527343750000, 0.0186462402343750,
+0.0229492187500000, 0.0139160156250000, 0.0252075195312500, 0.0135192871093750,
+0.0271606445312500, 0.0163269042968750, 0.0195922851562500, 0.0112304687500000,
+0.0187683105468750, 0.0093688964843750, 0.0234069824218750, 0.0116577148437500,
+0.0216064453125000, 0.0130920410156250, 0.0182495117187500, 0.0115661621093750,
+0.0319824218750000, 0.0084533691406250, 0.0310974121093750, 0.0130310058593750,
+0.0335388183593750, 0.0053100585937500, 0.0415649414062500, -0.0043945312500000,
+0.0369262695312500, 0.0025024414062500, 0.0447692871093750, -0.0109252929687500,
+0.0478820800781250, -0.0133361816406250, 0.0411071777343750, -0.0100708007812500,
+0.0397644042968750, -0.0133056640625000, 0.0442199707031250, -0.0139770507812500,
+0.0288391113281250, -0.0088195800781250, 0.0289001464843750, -0.0089111328125000,
+0.0307617187500000, -0.0092163085937500, 0.0249023437500000, -0.0073242187500000,
+0.0245971679687500, -0.0075073242187500, 0.0205688476562500, -0.0052795410156250,
+0.0263061523437500, -0.0081787109375000, 0.0168151855468750, -0.0025939941406250,
+0.0057373046875000, 0.0031127929687500, 0.0152282714843750, -0.0022888183593750,
+-0.0061035156250000, 0.0123291015625000, -0.0051269531250000, 0.0112915039062500,
+-0.0068664550781250, 0.0131225585937500, -0.0191345214843750, 0.0213928222656250,
+-0.0139160156250000, 0.0177612304687500, -0.0207519531250000, 0.0278015136718750,
+-0.0224304199218750, 0.0246582031250000, -0.0191040039062500, 0.0311584472656250,
+-0.0213317871093750, 0.0421142578125000, -0.0226745605468750, 0.0350036621093750,
+-0.0091857910156250, 0.0507812500000000, -0.0139465332031250, 0.0515136718750000,
+-0.0079345703125000, 0.0504760742187500, 0.0052795410156250, 0.0576477050781250,
+-0.0026245117187500, 0.0558471679687500, 0.0092773437500000, 0.0549011230468750,
+0.0099792480468750, 0.0564575195312500, 0.0089721679687500, 0.0544433593750000,
+0.0145263671875000, 0.0529479980468750, 0.0127868652343750, 0.0545959472656250,
+0.0109252929687500, 0.0459899902343750, 0.0135803222656250, 0.0496520996093750,
+0.0094604492187500, 0.0433044433593750, 0.0055541992187500, 0.0333557128906250,
+0.0097656250000000, 0.0400390625000000, -0.0018615722656250, 0.0229797363281250,
+0.0013122558593750, 0.0245666503906250, -0.0027465820312500, 0.0203552246093750,
+-0.0121154785156250, 0.0091857910156250, -0.0055236816406250, 0.0152282714843750,
+-0.0192871093750000, -0.0009155273437500, -0.0177612304687500, 0.0018005371093750,
+-0.0185241699218750, -0.0041503906250000, -0.0268554687500000, -0.0168762207031250,
+-0.0228271484375000, -0.0095214843750000, -0.0267944335937500, -0.0284118652343750,
+-0.0267639160156250, -0.0265197753906250, -0.0264587402343750, -0.0309143066406250,
+-0.0279541015625000, -0.0446777343750000, -0.0267028808593750, -0.0385437011718750,
+-0.0291748046875000, -0.0493469238281250, -0.0288696289062500, -0.0512390136718750,
+-0.0279846191406250, -0.0497131347656250, -0.0291748046875000, -0.0538024902343750,
+-0.0289306640625000, -0.0535583496093750, -0.0260314941406250, -0.0493469238281250,
+-0.0271301269531250, -0.0514831542968750, -0.0245361328125000, -0.0475463867187500,
+-0.0210266113281250, -0.0422973632812500, -0.0230407714843750, -0.0456542968750000,
+-0.0162963867187500, -0.0348815917968750, -0.0168457031250000, -0.0373840332031250,
+-0.0149230957031250, -0.0313415527343750, -0.0097351074218750, -0.0222167968750000,
+-0.0111389160156250, -0.0283508300781250, -0.0053710937500000, -0.0058593750000000,
+-0.0051879882812500, -0.0107116699218750, -0.0029907226562500, -0.0021057128906250,
+0.0014953613281250, 0.0177612304687500, 0.0003967285156250, 0.0075988769531250,
+0.0062561035156250, 0.0269470214843750, 0.0062561035156250, 0.0283203125000000,
+0.0079345703125000, 0.0289916992187500, 0.0122070312500000, 0.0380249023437500,
+0.0113830566406250, 0.0351867675781250, 0.0153808593750000, 0.0430908203125000,
+0.0160827636718750, 0.0430297851562500, 0.0165405273437500, 0.0451049804687500,
+0.0190429687500000, 0.0507507324218750, 0.0190124511718750, 0.0485534667968750,
+0.0183715820312500, 0.0527648925781250, 0.0195007324218750, 0.0535278320312500,
+0.0173339843750000, 0.0506591796875000, 0.0151672363281250, 0.0523071289062500,
+0.0163574218750000, 0.0530395507812500, 0.0092468261718750, 0.0382080078125000,
+0.0106506347656250, 0.0444335937500000, 0.0058898925781250, 0.0350036621093750,
+-0.0008850097656250, 0.0177307128906250, 0.0027770996093750, 0.0275573730468750,
+-0.0118103027343750, 0.0089416503906250, -0.0102844238281250, 0.0080871582031250,
+-0.0138244628906250, 0.0059509277343750, -0.0250854492187500, -0.0036315917968750,
+-0.0205383300781250, 0.0009155273437500, -0.0253906250000000, -0.0092468261718750,
+-0.0275268554687500, -0.0101013183593750, -0.0256347656250000, -0.0087890625000000,
+-0.0252990722656250, -0.0130004882812500, -0.0258178710937500, -0.0129394531250000,
+-0.0245666503906250, -0.0111389160156250, -0.0240173339843750, -0.0116882324218750,
+-0.0244140625000000},
+{0.0091552734375000, 0.0265808105468750, 0.0049438476562500, 0.0287780761718750,
+0.0011901855468750, 0.0327453613281250, 0.0047607421875000, 0.0304870605468750,
+-0.0040283203125000, 0.0318908691406250, -0.0025024414062500, 0.0345458984375000,
+-0.0045166015625000, 0.0313720703125000, -0.0107421875000000, 0.0284118652343750,
+-0.0070800781250000, 0.0310974121093750, -0.0137329101562500, 0.0257568359375000,
+-0.0140380859375000, 0.0260925292968750, -0.0141906738281250, 0.0247802734375000,
+-0.0178222656250000, 0.0223083496093750, -0.0169372558593750, 0.0243530273437500,
+-0.0175170898437500, 0.0150451660156250, -0.0192565917968750, 0.0184326171875000,
+-0.0173950195312500, 0.0123596191406250, -0.0167541503906250, 0.0023498535156250,
+-0.0193176269531250, 0.0086669921875000, -0.0109252929687500, -0.0075073242187500,
+-0.0138854980468750, -0.0063781738281250, -0.0111694335937500, -0.0091247558593750,
+-0.0035400390625000, -0.0195312500000000, -0.0079345703125000, -0.0148315429687500,
+-0.0009765625000000, -0.0233764648437500, -0.0013427734375000, -0.0236206054687500,
+-0.0011291503906250, -0.0247192382812500, 0.0020141601562500, -0.0295104980468750,
+-0.0004882812500000, -0.0276489257812500, 0.0034484863281250, -0.0329284667968750,
+0.0021972656250000, -0.0326843261718750, 0.0029602050781250, -0.0336303710937500,
+0.0059509277343750, -0.0371093750000000, 0.0036621093750000, -0.0357055664062500,
+0.0070190429687500, -0.0383300781250000, 0.0066223144531250, -0.0383911132812500,
+0.0063781738281250, -0.0385742187500000, 0.0079956054687500, -0.0407714843750000,
+0.0068359375000000, -0.0403442382812500, 0.0078735351562500, -0.0382385253906250,
+0.0071716308593750, -0.0417175292968750, 0.0070495605468750, -0.0359497070312500,
+0.0078125000000000, -0.0300598144531250, 0.0065612792968750, -0.0354309082031250,
+0.0063476562500000, -0.0191040039062500, 0.0071411132812500, -0.0208740234375000,
+0.0050964355468750, -0.0175170898437500, 0.0033874511718750, -0.0063171386718750,
+0.0043334960937500, -0.0107421875000000, -0.0008544921875000, 0.0004577636718750,
+-0.0003051757812500, -0.0001831054687500, -0.0021362304687500, 0.0036621093750000,
+-0.0064697265625000, 0.0119018554687500, -0.0048522949218750, 0.0082092285156250,
+-0.0084838867187500, 0.0204467773437500, -0.0092773437500000, 0.0189514160156250,
+-0.0084533691406250, 0.0224914550781250, -0.0097351074218750, 0.0320434570312500,
+-0.0101928710937500, 0.0268554687500000, -0.0068969726562500, 0.0332031250000000,
+-0.0079040527343750, 0.0362854003906250, -0.0058288574218750, 0.0311279296875000,
+-0.0020141601562500, 0.0297851562500000, -0.0033569335937500, 0.0334777832031250,
+0.0008544921875000, 0.0242004394531250, 0.0018920898437500, 0.0240478515625000,
+0.0021362304687500, 0.0238952636718750, 0.0043945312500000, 0.0202636718750000,
+0.0045776367187500, 0.0214538574218750, 0.0038452148437500, 0.0145568847656250,
+0.0059814453125000, 0.0174865722656250, 0.0018005371093750, 0.0126647949218750,
+-0.0014343261718750, 0.0039367675781250, 0.0019226074218750, 0.0089111328125000,
+-0.0127563476562500, 0.0004272460937500, -0.0105590820312500, -0.0009460449218750,
+-0.0153808593750000, -0.0010070800781250, -0.0278320312500000, -0.0043334960937500,
+-0.0225219726562500, -0.0032958984375000, -0.0307922363281250, -0.0065002441406250,
+-0.0335693359375000, -0.0070495605468750, -0.0310363769531250, -0.0071411132812500,
+-0.0320739746093750, -0.0084838867187500, -0.0327453613281250, -0.0085754394531250,
+-0.0278930664062500, -0.0100708007812500, -0.0289001464843750, -0.0097351074218750,
+-0.0261535644531250, -0.0085144042968750, -0.0219421386718750, -0.0097656250000000,
+-0.0247802734375000, -0.0103149414062500, -0.0173034667968750, -0.0030822753906250,
+-0.0191345214843750, -0.0045776367187500, -0.0148620605468750, -0.0032958984375000,
+-0.0083312988281250, 0.0028991699218750, -0.0131225585937500, 0.0010986328125000,
+0.0012207031250000, 0.0007324218750000, -0.0010070800781250, 0.0034179687500000,
+0.0032653808593750, -0.0002441406250000, 0.0153808593750000, -0.0041198730468750,
+0.0090942382812500, -0.0005187988281250, 0.0139160156250000, -0.0072631835937500,
+0.0187683105468750, -0.0076904296875000, 0.0130310058593750, -0.0066528320312500,
+0.0090026855468750, -0.0089721679687500, 0.0122070312500000, -0.0082397460937500,
+-0.0007324218750000, -0.0080261230468750, 0.0008239746093750, -0.0053405761718750,
+0.0000000000000000, -0.0114440917968750, -0.0092468261718750, -0.0169372558593750,
+-0.0070800781250000, -0.0103759765625000, -0.0054931640625000, -0.0187988281250000,
+-0.0076904296875000, -0.0206298828125000, -0.0048522949218750, -0.0220947265625000,
+-0.0003356933593750, -0.0256042480468750, -0.0031127929687500, -0.0221557617187500,
+-0.0032653808593750, -0.0248107910156250, -0.0003967285156250, -0.0283203125000000,
+-0.0021972656250000, -0.0232543945312500, -0.0043029785156250, -0.0174865722656250,
+-0.0040588378906250, -0.0188293457031250, -0.0151367187500000, -0.0121459960937500,
+-0.0094299316406250, -0.0125732421875000, -0.0119018554687500, -0.0130615234375000,
+-0.0236511230468750, -0.0100097656250000, -0.0184631347656250, -0.0107727050781250,
+-0.0206298828125000, -0.0091552734375000, -0.0266418457031250, -0.0092163085937500,
+-0.0158691406250000, -0.0082397460937500, -0.0075073242187500, -0.0067443847656250,
+-0.0171508789062500, -0.0081481933593750, 0.0040283203125000, -0.0075988769531250,
+0.0059814453125000, -0.0047302246093750, 0.0026855468750000, -0.0084228515625000,
+0.0113220214843750, -0.0141601562500000, 0.0102844238281250, -0.0114135742187500,
+-0.0015869140625000, -0.0101318359375000, 0.0028686523437500, -0.0133056640625000,
+-0.0006408691406250, -0.0104675292968750, -0.0139770507812500, -0.0067443847656250,
+-0.0090637207031250, -0.0092163085937500, -0.0033264160156250, 0.0001220703125000,
+-0.0100097656250000, -0.0016479492187500, -0.0043640136718750, 0.0000000000000000,
+0.0080261230468750, 0.0083007812500000, 0.0025024414062500, 0.0042724609375000,
+0.0038757324218750, -0.0007019042968750, 0.0079345703125000, 0.0057983398437500,
+0.0047912597656250, -0.0035400390625000, 0.0007324218750000, -0.0173034667968750,
+0.0027770996093750, -0.0099182128906250, -0.0016479492187500, -0.0226440429687500,
+-0.0003356933593750, -0.0252380371093750, -0.0044860839843750, -0.0242309570312500,
+-0.0094299316406250, -0.0287780761718750, -0.0051574707031250, -0.0281066894531250,
+-0.0173645019531250, -0.0260314941406250, -0.0172119140625000, -0.0268859863281250,
+-0.0163574218750000, -0.0277404785156250, -0.0232849121093750, -0.0265502929687500,
+-0.0219726562500000, -0.0261840820312500, -0.0161132812500000, -0.0285644531250000,
+-0.0202636718750000, -0.0290832519531250, -0.0143737792968750, -0.0270996093750000,
+-0.0055847167968750, -0.0274658203125000, -0.0115051269531250, -0.0284118652343750,
+0.0011596679687500, -0.0200195312500000, 0.0014648437500000, -0.0219116210937500,
+0.0020446777343750, -0.0186462402343750, 0.0088195800781250, -0.0104980468750000,
+0.0067749023437500, -0.0136413574218750, 0.0101318359375000, -0.0090942382812500,
+0.0107727050781250, -0.0072937011718750, 0.0104370117187500, -0.0084838867187500,
+0.0119018554687500, -0.0079956054687500, 0.0117797851562500, -0.0076293945312500,
+0.0101623535156250, -0.0099487304687500, 0.0111389160156250, -0.0096435546875000,
+0.0100097656250000, -0.0105895996093750, 0.0076293945312500, -0.0128479003906250,
+0.0086975097656250, -0.0119628906250000, 0.0075683593750000, -0.0118408203125000,
+0.0060729980468750, -0.0131530761718750, 0.0068359375000000, -0.0107116699218750,
+0.0072021484375000, -0.0083618164062500, 0.0049133300781250, -0.0098571777343750,
+0.0053710937500000, -0.0038757324218750, 0.0054016113281250, -0.0040893554687500,
+0.0024108886718750, -0.0025024414062500, 0.0006103515625000, 0.0019836425781250,
+0.0010986328125000, 0.0009765625000000, -0.0047302246093750, 0.0049743652343750,
+-0.0045471191406250, 0.0058593750000000, -0.0075073242187500, 0.0068054199218750,
+-0.0118408203125000, 0.0095520019531250, -0.0100708007812500, 0.0098266601562500,
+-0.0195312500000000, 0.0108032226562500, -0.0181579589843750, 0.0121154785156250,
+-0.0217285156250000, 0.0117492675781250, -0.0299377441406250, 0.0113525390625000,
+-0.0258483886718750, 0.0119323730468750, -0.0312500000000000, 0.0096740722656250,
+-0.0335083007812500, 0.0101318359375000, -0.0297241210937500, 0.0088806152343750,
+-0.0289001464843750, 0.0066528320312500, -0.0307922363281250, 0.0075683593750000,
+-0.0236816406250000, 0.0040588378906250, -0.0229797363281250, 0.0038146972656250,
+-0.0231628417968750, 0.0033874511718750, -0.0195617675781250, 0.0013122558593750,
+-0.0198059082031250, 0.0016174316406250, -0.0224914550781250, 0.0014038085937500,
+-0.0207824707031250, 0.0008850097656250, -0.0224609375000000, 0.0013122558593750,
+-0.0263061523437500, 0.0017395019531250, -0.0242004394531250, 0.0014038085937500,
+-0.0255432128906250, 0.0030517578125000, -0.0273132324218750, 0.0026550292968750,
+-0.0245056152343750, 0.0035095214843750, -0.0227050781250000, 0.0050659179687500,
+-0.0244750976562500, 0.0040588378906250, -0.0169982910156250, 0.0057678222656250,
+-0.0180053710937500, 0.0059204101562500, -0.0156555175781250, 0.0061035156250000,
+-0.0097656250000000, 0.0073852539062500, -0.0126647949218750, 0.0072021484375000,
+-0.0079040527343750, 0.0067443847656250, -0.0069885253906250, 0.0086669921875000,
+-0.0087585449218750, 0.0056762695312500, -0.0084228515625000, 0.0036926269531250,
+-0.0083312988281250, 0.0069885253906250, -0.0101013183593750, -0.0045166015625000,
+-0.0118408203125000, -0.0014038085937500, -0.0098571777343750, -0.0046691894531250,
+-0.0091247558593750, -0.0141906738281250, -0.0115051269531250, -0.0089111328125000,
+-0.0047302246093750, -0.0176391601562500, -0.0055541992187500, -0.0186462402343750,
+-0.0047302246093750, -0.0184936523437500, -0.0007019042968750, -0.0222473144531250,
+-0.0028076171875000, -0.0209655761718750, 0.0007324218750000, -0.0225219726562500,
+-0.0002441406250000, -0.0235290527343750, 0.0018615722656250, -0.0202636718750000,
+0.0053100585937500, -0.0192871093750000, 0.0027160644531250, -0.0217895507812500,
+0.0093994140625000, -0.0122070312500000, 0.0083007812500000, -0.0130004882812500,
+0.0109558105468750, -0.0117797851562500, 0.0156250000000000, -0.0051879882812500,
+0.0122985839843750, -0.0073242187500000, 0.0230407714843750, -0.0064392089843750,
+0.0208435058593750, -0.0046081542968750, 0.0240478515625000, -0.0064697265625000,
+0.0335998535156250, -0.0085144042968750, 0.0288391113281250, -0.0075683593750000,
+0.0322570800781250, -0.0097351074218750, 0.0361022949218750, -0.0091552734375000,
+0.0310058593750000, -0.0111999511718750, 0.0272827148437500, -0.0149230957031250,
+0.0307312011718750, -0.0124511718750000, 0.0215759277343750, -0.0100708007812500,
+0.0218811035156250, -0.0156860351562500, 0.0208435058593750, -0.0057373046875000,
+0.0163574218750000, 0.0049743652343750, 0.0184631347656250, -0.0031433105468750,
+0.0101623535156250, 0.0184326171875000, 0.0126647949218750, 0.0203552246093750,
+0.0088195800781250, 0.0165710449218750, -0.0000610351562500, 0.0254516601562500,
+0.0048828125000000, 0.0252685546875000, 0.0002136230468750, 0.0113525390625000,
+-0.0029907226562500, 0.0156555175781250, 0.0004882812500000, 0.0119323730468750,
+0.0032653808593750, -0.0011291503906250, 0.0006713867187500, 0.0028381347656250,
+0.0005798339843750, -0.0013427734375000, 0.0044860839843750, -0.0013732910156250,
+-0.0023193359375000, -0.0028076171875000, -0.0088500976562500, -0.0066528320312500,
+-0.0035705566406250},
+{-0.0247192382812500, -0.0147705078125000, -0.0262451171875000, -0.0127563476562500,
+-0.0227050781250000, -0.0169677734375000, -0.0225219726562500, -0.0168762207031250,
+-0.0212402343750000, -0.0177917480468750, -0.0187988281250000, -0.0202941894531250,
+-0.0199890136718750, -0.0187377929687500, -0.0169067382812500, -0.0222167968750000,
+-0.0162658691406250, -0.0213623046875000, -0.0167541503906250, -0.0226135253906250,
+-0.0155334472656250, -0.0254821777343750, -0.0154418945312500, -0.0236206054687500,
+-0.0177307128906250, -0.0278320312500000, -0.0166931152343750, -0.0274047851562500,
+-0.0172119140625000, -0.0277709960937500, -0.0195922851562500, -0.0305175781250000,
+-0.0184326171875000, -0.0293273925781250, -0.0177001953125000, -0.0304565429687500,
+-0.0185852050781250, -0.0307922363281250, -0.0170898437500000, -0.0299987792968750,
+-0.0149841308593750, -0.0299682617187500, -0.0161437988281250, -0.0300292968750000,
+-0.0145263671875000, -0.0279541015625000, -0.0133056640625000, -0.0283203125000000,
+-0.0155334472656250, -0.0269775390625000, -0.0167236328125000, -0.0249023437500000,
+-0.0152893066406250, -0.0255737304687500, -0.0207519531250000, -0.0216369628906250,
+-0.0209045410156250, -0.0228576660156250, -0.0204162597656250, -0.0195007324218750,
+-0.0233154296875000, -0.0154113769531250, -0.0234069824218750, -0.0186157226562500,
+-0.0204467773437500, -0.0081481933593750, -0.0213623046875000, -0.0097961425781250,
+-0.0214233398437500, -0.0075683593750000, -0.0194396972656250, -0.0001525878906250,
+-0.0204467773437500, -0.0040283203125000, -0.0218200683593750, 0.0021057128906250,
+-0.0221862792968750, 0.0032653808593750, -0.0214233398437500, 0.0018310546875000,
+-0.0222473144531250, 0.0039062500000000, -0.0236511230468750, 0.0039062500000000,
+-0.0187683105468750, -0.0006408691406250, -0.0209350585937500, 0.0023498535156250,
+-0.0178222656250000, -0.0036010742187500, -0.0131225585937500, -0.0108642578125000,
+-0.0165100097656250, -0.0050964355468750, -0.0039978027343750, -0.0192260742187500,
+-0.0075683593750000, -0.0192565917968750, -0.0010070800781250, -0.0190124511718750,
+0.0115356445312500, -0.0263977050781250, 0.0040283203125000, -0.0239562988281250,
+0.0187377929687500, -0.0223693847656250, 0.0197753906250000, -0.0245056152343750,
+0.0190734863281250, -0.0214233398437500, 0.0255432128906250, -0.0173645019531250,
+0.0240783691406250, -0.0189819335937500, 0.0259399414062500, -0.0129699707031250,
+0.0256042480468750, -0.0137023925781250, 0.0270385742187500, -0.0109252929687500,
+0.0297241210937500, -0.0045471191406250, 0.0279846191406250, -0.0076293945312500,
+0.0284423828125000, -0.0076293945312500, 0.0314331054687500, -0.0048522949218750,
+0.0263671875000000, -0.0065917968750000, 0.0221557617187500, -0.0113830566406250,
+0.0265502929687500, -0.0098266601562500, 0.0141906738281250, -0.0029296875000000,
+0.0146179199218750, -0.0082092285156250, 0.0137939453125000, -0.0013427734375000,
+0.0075378417968750, 0.0106201171875000, 0.0097961425781250, 0.0040588378906250,
+0.0022583007812500, 0.0130004882812500, 0.0056762695312500, 0.0173645019531250,
+-0.0020751953125000, 0.0099487304687500, -0.0125427246093750, 0.0072937011718750,
+-0.0051574707031250, 0.0124816894531250, -0.0234069824218750, 0.0010375976562500,
+-0.0242309570312500, -0.0007019042968750, -0.0234680175781250, -0.0007629394531250,
+-0.0319519042968750, -0.0041809082031250, -0.0299072265625000, -0.0021057128906250,
+-0.0299682617187500, -0.0066223144531250, -0.0307312011718750, -0.0070800781250000,
+-0.0302124023437500, -0.0081176757812500, -0.0301513671875000, -0.0109558105468750,
+-0.0299987792968750, -0.0093688964843750, -0.0259094238281250, -0.0090637207031250,
+-0.0278015136718750, -0.0068664550781250, -0.0239562988281250, -0.0182800292968750,
+-0.0181274414062500, -0.0244750976562500, -0.0212707519531250, -0.0144653320312500,
+-0.0140991210937500, -0.0444946289062500, -0.0117797851562500, -0.0450439453125000,
+-0.0166931152343750, -0.0444946289062500, -0.0157775878906250, -0.0604248046875000,
+-0.0125732421875000, -0.0557250976562500, -0.0295410156250000, -0.0459594726562500,
+-0.0253601074218750, -0.0530700683593750, -0.0323486328125000, -0.0466003417968750,
+-0.0588378906250000, -0.0236511230468750, -0.0693664550781250, -0.0574645996093750,
+-0.0451354980468750, -0.0393981933593750, -0.0618896484375000, -0.0517578125000000,
+-0.0458984375000000, -0.0417480468750000, -0.0238037109375000, -0.0230102539062500,
+-0.0364074707031250, -0.0296630859375000, -0.0070190429687500, -0.0247802734375000,
+-0.0086364746093750, -0.0234069824218750, -0.0026550292968750, -0.0220336914062500,
+0.0157165527343750, -0.0190124511718750, 0.0062866210937500, -0.0208740234375000,
+0.0290527343750000, -0.0177001953125000, 0.0306701660156250, -0.0187683105468750,
+0.0267639160156250, -0.0144042968750000, 0.0373535156250000, -0.0090026855468750,
+0.0366821289062500, -0.0133666992187500, 0.0191650390625000, -0.0086975097656250,
+0.0245056152343750, -0.0058288574218750, 0.0226745605468750, -0.0080261230468750,
+0.0064697265625000, -0.0096130371093750, 0.0099182128906250, -0.0086975097656250,
+0.0177001953125000, -0.0092468261718750, 0.0122375488281250, -0.0092468261718750,
+0.0187683105468750, -0.0104675292968750, 0.0302429199218750, -0.0109863281250000,
+0.0228576660156250, -0.0097351074218750, 0.0314331054687500, -0.0144958496093750,
+0.0373840332031250, -0.0141601562500000, 0.0257568359375000, -0.0142211914062500,
+0.0224304199218750, -0.0176696777343750, 0.0306091308593750, -0.0165710449218750,
+-0.0076293945312500, -0.0120849609375000, -0.0034179687500000, -0.0155029296875000,
+-0.0041198730468750, -0.0091857910156250, -0.0282897949218750, -0.0010681152343750,
+-0.0225830078125000, -0.0064697265625000, -0.0201721191406250, 0.0073547363281250,
+-0.0250854492187500, 0.0075988769531250, -0.0202026367187500, 0.0081481933593750,
+-0.0136718750000000, 0.0153503417968750, -0.0184631347656250, 0.0129394531250000,
+-0.0099792480468750, 0.0149841308593750, -0.0084533691406250, 0.0160217285156250,
+-0.0127258300781250, 0.0143432617187500, -0.0100402832031250, 0.0143432617187500,
+-0.0077514648437500, 0.0151672363281250, -0.0235900878906250, 0.0098876953125000,
+-0.0203552246093750, 0.0112915039062500, -0.0211181640625000, 0.0093078613281250,
+-0.0328979492187500, 0.0041809082031250, -0.0295104980468750, 0.0065612792968750,
+-0.0255126953125000, 0.0039062500000000, -0.0299682617187500, 0.0032653808593750,
+-0.0248413085937500, 0.0028686523437500, -0.0176391601562500, 0.0025634765625000,
+-0.0224914550781250, 0.0033264160156250, -0.0103454589843750, -0.0033569335937500,
+-0.0111999511718750, -0.0010070800781250, -0.0095214843750000, -0.0043640136718750,
+-0.0011596679687500, -0.0115661621093750, -0.0049133300781250, -0.0078430175781250,
+-0.0031127929687500, -0.0136718750000000, 0.0006103515625000, -0.0143127441406250,
+-0.0048828125000000, -0.0146484375000000, -0.0090026855468750, -0.0171508789062500,
+-0.0041503906250000, -0.0157470703125000, -0.0163574218750000, -0.0193481445312500,
+-0.0144042968750000, -0.0192260742187500, -0.0165100097656250, -0.0185241699218750,
+-0.0245971679687500, -0.0204162597656250, -0.0197753906250000, -0.0202941894531250,
+-0.0271911621093750, -0.0172424316406250, -0.0274658203125000, -0.0177307128906250,
+-0.0265197753906250, -0.0174255371093750, -0.0300598144531250, -0.0152587890625000,
+-0.0291748046875000, -0.0155944824218750, -0.0269470214843750, -0.0154724121093750,
+-0.0285034179687500, -0.0160827636718750, -0.0267639160156250, -0.0140686035156250,
+-0.0243530273437500, -0.0126342773437500, -0.0266418457031250, -0.0142517089843750,
+-0.0216979980468750, -0.0085449218750000, -0.0240783691406250, -0.0087280273437500,
+-0.0222473144531250, -0.0083618164062500, -0.0179443359375000, -0.0049133300781250,
+-0.0219726562500000, -0.0057373046875000, -0.0146789550781250, -0.0057678222656250,
+-0.0165100097656250, -0.0046997070312500, -0.0152282714843750, -0.0058898925781250,
+-0.0105285644531250, -0.0068054199218750, -0.0147399902343750, -0.0059509277343750,
+-0.0086669921875000, -0.0109252929687500, -0.0104980468750000, -0.0096435546875000,
+-0.0092468261718750, -0.0120544433593750, -0.0048522949218750, -0.0171813964843750,
+-0.0087585449218750, -0.0147399902343750, -0.0025939941406250, -0.0190429687500000,
+-0.0036621093750000, -0.0196838378906250, -0.0035095214843750, -0.0198669433593750,
+0.0000610351562500, -0.0212097167968750, -0.0023803710937500, -0.0208129882812500,
+-0.0002746582031250, -0.0252075195312500, -0.0010986328125000, -0.0232238769531250,
+-0.0015869140625000, -0.0265502929687500, -0.0006713867187500, -0.0323181152343750,
+-0.0025939941406250, -0.0285949707031250, -0.0024108886718750, -0.0333557128906250,
+-0.0024719238281250, -0.0346679687500000, -0.0038757324218750, -0.0325317382812500,
+-0.0048828125000000, -0.0326538085937500, -0.0050354003906250, -0.0332336425781250,
+-0.0065002441406250, -0.0310363769531250, -0.0068664550781250, -0.0309143066406250,
+-0.0071716308593750, -0.0316772460937500, -0.0078735351562500, -0.0311279296875000,
+-0.0081176757812500, -0.0308532714843750, -0.0088500976562500, -0.0325927734375000,
+-0.0087585449218750, -0.0328674316406250, -0.0089416503906250, -0.0308532714843750,
+-0.0094299316406250, -0.0314025878906250, -0.0089111328125000, -0.0324707031250000,
+-0.0088500976562500, -0.0222167968750000, -0.0083923339843750, -0.0254516601562500,
+-0.0080871582031250, -0.0215759277343750, -0.0077209472656250, -0.0113220214843750,
+-0.0068054199218750, -0.0156555175781250, -0.0066223144531250, -0.0095520019531250,
+-0.0064697265625000, -0.0086669921875000, -0.0062255859375000, -0.0065917968750000,
+-0.0058898925781250, -0.0035095214843750, -0.0057983398437500, -0.0058288574218750,
+-0.0073852539062500, 0.0033264160156250, -0.0067749023437500, 0.0024108886718750,
+-0.0078125000000000, 0.0030212402343750, -0.0101318359375000, 0.0086669921875000,
+-0.0094909667968750, 0.0064697265625000, -0.0101318359375000, 0.0082092285156250,
+-0.0109558105468750, 0.0083618164062500, -0.0099792480468750, 0.0083618164062500,
+-0.0095520019531250, 0.0095825195312500, -0.0099487304687500, 0.0092163085937500,
+-0.0053100585937500, 0.0085754394531250, -0.0054321289062500, 0.0096435546875000,
+-0.0058288574218750, 0.0085449218750000, -0.0025329589843750, 0.0070495605468750,
+-0.0024414062500000, 0.0079956054687500, -0.0076599121093750, 0.0041198730468750,
+-0.0054321289062500, 0.0054626464843750, -0.0087585449218750, 0.0032653808593750,
+-0.0143737792968750, -0.0009460449218750, -0.0110168457031250, 0.0015869140625000,
+-0.0204162597656250, -0.0028076171875000, -0.0198669433593750, -0.0023803710937500,
+-0.0207824707031250, -0.0051269531250000, -0.0270690917968750, -0.0082702636718750,
+-0.0242614746093750, -0.0049743652343750, -0.0223999023437500, -0.0148925781250000,
+-0.0262756347656250, -0.0133972167968750, -0.0193481445312500, -0.0156555175781250,
+-0.0108947753906250, -0.0229797363281250, -0.0161132812500000, -0.0184326171875000,
+-0.0070190429687500, -0.0219116210937500, -0.0054016113281250, -0.0243835449218750,
+-0.0045166015625000, -0.0227050781250000, -0.0007019042968750, -0.0209350585937500,
+-0.0027160644531250, -0.0209350585937500, 0.0000610351562500, -0.0214233398437500,
+0.0008239746093750, -0.0223388671875000, 0.0004577636718750, -0.0209960937500000,
+0.0016784667968750, -0.0195617675781250, 0.0018005371093750, -0.0202331542968750,
+-0.0009765625000000, -0.0174865722656250, 0.0002136230468750, -0.0198364257812500,
+-0.0014038085937500, -0.0162048339843750, -0.0047302246093750, -0.0106506347656250,
+-0.0031738281250000, -0.0143432617187500, -0.0063781738281250, -0.0089721679687500,
+-0.0081176757812500},
+{-0.0051879882812500, -0.0100708007812500, -0.0047607421875000, -0.0091552734375000,
+-0.0037536621093750, -0.0118103027343750, -0.0042419433593750, -0.0138854980468750,
+-0.0035095214843750, -0.0113830566406250, -0.0016479492187500, -0.0195007324218750,
+-0.0015869140625000, -0.0180053710937500, 0.0003967285156250, -0.0200500488281250,
+0.0024719238281250, -0.0259094238281250, 0.0021667480468750, -0.0229492187500000,
+0.0073242187500000, -0.0300903320312500, 0.0062255859375000, -0.0291442871093750,
+0.0088195800781250, -0.0321044921875000, 0.0137939453125000, -0.0380249023437500,
+0.0111694335937500, -0.0348510742187500, 0.0160522460937500, -0.0428466796875000,
+0.0166931152343750, -0.0424194335937500, 0.0155029296875000, -0.0436401367187500,
+0.0172729492187500, -0.0489807128906250, 0.0170898437500000, -0.0464477539062500,
+0.0130004882812500, -0.0473327636718750, 0.0150146484375000, -0.0498352050781250,
+0.0108337402343750, -0.0463256835937500, 0.0054626464843750, -0.0430908203125000,
+0.0087890625000000, -0.0458679199218750, -0.0035400390625000, -0.0401916503906250,
+-0.0023193359375000, -0.0398559570312500, -0.0056762695312500, -0.0386352539062500,
+-0.0152893066406250, -0.0363769531250000, -0.0115356445312500, -0.0375366210937500,
+-0.0181884765625000, -0.0274047851562500, -0.0200500488281250, -0.0310974121093750,
+-0.0184020996093750, -0.0256958007812500, -0.0204162597656250, -0.0150756835937500,
+-0.0209655761718750, -0.0211791992187500, -0.0146179199218750, -0.0086059570312500,
+-0.0170593261718750, -0.0093078613281250, -0.0129394531250000, -0.0063171386718750,
+-0.0057373046875000, 0.0017700195312500, -0.0097656250000000, -0.0028076171875000,
+0.0002136230468750, 0.0079040527343750, 0.0003051757812500, 0.0080261230468750,
+0.0001831054687500, 0.0086059570312500, 0.0057678222656250, 0.0150146484375000,
+0.0042114257812500, 0.0132446289062500, 0.0019836425781250, 0.0138549804687500,
+0.0034179687500000, 0.0157470703125000, 0.0019531250000000, 0.0135498046875000,
+-0.0006713867187500, 0.0115966796875000, 0.0004272460937500, 0.0134582519531250,
+-0.0038757324218750, 0.0088806152343750, -0.0027465820312500, 0.0089721679687500,
+-0.0043029785156250, 0.0078125000000000, -0.0081787109375000, 0.0057373046875000,
+-0.0061340332031250, 0.0065917968750000, -0.0100097656250000, -0.0036315917968750,
+-0.0090332031250000, 0.0012207031250000, -0.0107421875000000, -0.0057678222656250,
+-0.0137023925781250, -0.0184936523437500, -0.0114440917968750, -0.0109863281250000,
+-0.0186767578125000, -0.0255432128906250, -0.0176391601562500, -0.0260925292968750,
+-0.0170288085937500, -0.0242919921875000, -0.0226745605468750, -0.0313415527343750,
+-0.0216674804687500, -0.0306396484375000, -0.0132446289062500, -0.0249023437500000,
+-0.0155029296875000, -0.0255432128906250, -0.0128479003906250, -0.0265197753906250,
+-0.0054931640625000, -0.0249633789062500, -0.0078735351562500, -0.0248107910156250,
+-0.0023803710937500, -0.0214233398437500, -0.0015869140625000, -0.0230712890625000,
+-0.0025939941406250, -0.0217590332031250, -0.0005798339843750, -0.0188903808593750,
+-0.0001220703125000, -0.0205383300781250, -0.0018005371093750, -0.0135803222656250,
+-0.0014343261718750, -0.0151672363281250, -0.0005493164062500, -0.0129394531250000,
+-0.0012512207031250, -0.0072021484375000, -0.0017089843750000, -0.0101928710937500,
+0.0006408691406250, -0.0042724609375000, 0.0010375976562500, -0.0046386718750000,
+0.0003662109375000, -0.0027770996093750, 0.0002746582031250, 0.0036010742187500,
+-0.0007629394531250, 0.0022583007812500, -0.0034790039062500, -0.0012817382812500,
+-0.0029602050781250, 0.0025329589843750, -0.0009155273437500, -0.0000610351562500,
+-0.0017700195312500, -0.0067443847656250, -0.0041809082031250, -0.0036621093750000,
+-0.0011291503906250, -0.0057983398437500, 0.0032653808593750, -0.0095214843750000,
+-0.0064086914062500, -0.0036010742187500, -0.0131225585937500, 0.0015563964843750,
+-0.0050964355468750, -0.0034484863281250, -0.0252990722656250, 0.0046081542968750,
+-0.0263366699218750, 0.0065002441406250, -0.0237426757812500, 0.0048828125000000,
+-0.0325927734375000, 0.0070800781250000, -0.0319519042968750, 0.0062561035156250,
+-0.0247802734375000, 0.0001831054687500, -0.0272827148437500, 0.0031433105468750,
+-0.0235595703125000, -0.0010681152343750, -0.0161743164062500, -0.0090026855468750,
+-0.0205688476562500, -0.0053405761718750, -0.0151062011718750, -0.0136413574218750,
+-0.0124511718750000, -0.0141906738281250, -0.0161743164062500, -0.0138549804687500,
+-0.0172424316406250, -0.0177917480468750, -0.0148620605468750, -0.0170288085937500,
+-0.0212707519531250, -0.0166625976562500, -0.0217285156250000, -0.0182189941406250,
+-0.0202026367187500, -0.0151062011718750, -0.0226745605468750, -0.0119018554687500,
+-0.0231018066406250, -0.0149230957031250, -0.0183715820312500, -0.0113525390625000,
+-0.0205688476562500, -0.0092468261718750, -0.0158996582031250, -0.0111999511718750,
+-0.0098266601562500, -0.0121154785156250, -0.0140380859375000, -0.0115661621093750,
+-0.0017700195312500, -0.0153808593750000, -0.0022583007812500, -0.0144348144531250,
+-0.0015258789062500, -0.0155944824218750, 0.0061950683593750, -0.0200500488281250,
+0.0038146972656250, -0.0185852050781250, 0.0030822753906250, -0.0153198242187500,
+0.0058593750000000, -0.0189208984375000, 0.0028686523437500, -0.0130004882812500,
+-0.0004882812500000, -0.0057067871093750, 0.0024108886718750, -0.0111083984375000,
+-0.0048217773437500, 0.0015563964843750, -0.0037231445312500, 0.0022277832031250,
+-0.0059204101562500, 0.0015563964843750, -0.0115661621093750, 0.0072937011718750,
+-0.0085754394531250, 0.0057983398437500, -0.0147705078125000, 0.0038757324218750,
+-0.0153503417968750, 0.0044860839843750, -0.0160827636718750, 0.0028991699218750,
+-0.0195312500000000, 0.0000915527343750, -0.0191955566406250, 0.0008544921875000,
+-0.0226135253906250, 0.0009765625000000, -0.0221252441406250, -0.0009460449218750,
+-0.0254516601562500, 0.0019836425781250, -0.0290222167968750, 0.0046386718750000,
+-0.0268554687500000, 0.0022888183593750, -0.0368652343750000, 0.0102844238281250,
+-0.0352478027343750, 0.0094909667968750, -0.0387573242187500, 0.0112304687500000,
+-0.0469970703125000, 0.0171203613281250, -0.0430297851562500, 0.0146789550781250,
+-0.0509338378906250, 0.0179748535156250, -0.0514831542968750, 0.0184936523437500,
+-0.0516052246093750, 0.0169982910156250, -0.0555725097656250, 0.0172119140625000,
+-0.0540771484375000, 0.0168151855468750, -0.0537719726562500, 0.0141296386718750,
+-0.0557250976562500, 0.0144348144531250, -0.0516662597656250, 0.0118103027343750,
+-0.0486755371093750, 0.0087585449218750, -0.0516052246093750, 0.0100708007812500,
+-0.0392761230468750, 0.0036315917968750, -0.0420532226562500, 0.0037536621093750,
+-0.0375061035156250, 0.0015258789062500, -0.0273132324218750, -0.0033264160156250,
+-0.0331726074218750, -0.0014953613281250, -0.0209960937500000, -0.0063476562500000,
+-0.0216979980468750, -0.0066833496093750, -0.0198669433593750, -0.0075378417968750,
+-0.0120544433593750, -0.0103454589843750, -0.0155334472656250, -0.0090942382812500,
+-0.0094299316406250, -0.0108642578125000, -0.0087585449218750, -0.0110473632812500,
+-0.0083923339843750, -0.0106201171875000, -0.0054016113281250, -0.0110778808593750,
+-0.0064086914062500, -0.0107727050781250, -0.0042114257812500, -0.0109558105468750,
+-0.0038757324218750, -0.0108032226562500, -0.0040588378906250, -0.0104370117187500,
+-0.0028076171875000, -0.0104370117187500, -0.0029907226562500, -0.0101623535156250,
+-0.0054321289062500, -0.0089111328125000, -0.0036315917968750, -0.0093383789062500,
+-0.0065307617187500, -0.0086364746093750, -0.0103149414062500, -0.0074462890625000,
+-0.0076293945312500, -0.0081176757812500, -0.0157165527343750, -0.0062255859375000,
+-0.0145263671875000, -0.0074157714843750, -0.0176086425781250, -0.0053405761718750,
+-0.0236511230468750, -0.0035095214843750, -0.0204772949218750, -0.0059509277343750,
+-0.0317382812500000, 0.0034484863281250, -0.0299682617187500, 0.0001220703125000,
+-0.0341186523437500, 0.0037231445312500, -0.0440063476562500, 0.0127563476562500,
+-0.0397644042968750, 0.0072326660156250, -0.0487670898437500, 0.0152893066406250,
+-0.0502624511718750, 0.0164794921875000, -0.0497436523437500, 0.0143432617187500,
+-0.0535278320312500, 0.0173034667968750, -0.0529174804687500, 0.0175170898437500,
+-0.0503234863281250, 0.0113220214843750, -0.0529479980468750, 0.0149230957031250,
+-0.0483398437500000, 0.0092773437500000, -0.0435180664062500, 0.0009460449218750,
+-0.0477294921875000, 0.0064392089843750, -0.0343627929687500, -0.0083312988281250,
+-0.0359191894531250, -0.0065612792968750, -0.0339050292968750, -0.0089416503906250,
+-0.0250244140625000, -0.0201416015625000, -0.0289611816406250, -0.0158386230468750,
+-0.0233154296875000, -0.0178833007812500, -0.0234375000000000, -0.0215454101562500,
+-0.0214843750000000, -0.0174560546875000, -0.0178222656250000, -0.0142822265625000,
+-0.0202636718750000, -0.0173339843750000, -0.0118408203125000, -0.0071105957031250,
+-0.0130310058593750, -0.0084228515625000, -0.0110473632812500, -0.0070495605468750,
+-0.0047302246093750, 0.0007019042968750, -0.0072631835937500, -0.0014953613281250,
+-0.0022583007812500, -0.0000610351562500, -0.0025634765625000, 0.0031433105468750,
+-0.0001525878906250, -0.0038452148437500, 0.0038452148437500, -0.0078430175781250,
+0.0011901855468750, -0.0025024414062500, 0.0084228515625000, -0.0236511230468750,
+0.0087890625000000, -0.0229187011718750, 0.0073852539062500, -0.0223083496093750,
+0.0107727050781250, -0.0349731445312500, 0.0106811523437500, -0.0324707031250000,
+0.0037231445312500, -0.0230712890625000, 0.0060119628906250, -0.0288391113281250,
+0.0035095214843750, -0.0202941894531250, -0.0033264160156250, -0.0075378417968750,
+-0.0004577636718750, -0.0151977539062500, -0.0034790039062500, 0.0056152343750000,
+-0.0036926269531250, 0.0047912597656250, -0.0048522949218750, 0.0059814453125000,
+-0.0059204101562500, 0.0191650390625000, -0.0040588378906250, 0.0147705078125000,
+-0.0123901367187500, 0.0126342773437500, -0.0104064941406250, 0.0198669433593750,
+-0.0128173828125000, 0.0069885253906250, -0.0201110839843750, -0.0074157714843750,
+-0.0166931152343750, 0.0036010742187500, -0.0203857421875000, -0.0143432617187500,
+-0.0209045410156250, -0.0189819335937500, -0.0218505859375000, -0.0129394531250000,
+-0.0223693847656250, -0.0162353515625000, -0.0211181640625000, -0.0185241699218750,
+-0.0330505371093750, -0.0037231445312500, -0.0283203125000000, -0.0018005371093750,
+-0.0356445312500000, -0.0096130371093750, -0.0489807128906250, -0.0074462890625000,
+-0.0413513183593750, -0.0032043457031250, -0.0581054687500000, -0.0222778320312500,
+-0.0588073730468750, -0.0193176269531250, -0.0580444335937500, -0.0242004394531250,
+-0.0668640136718750, -0.0397644042968750, -0.0643615722656250, -0.0337524414062500,
+-0.0580444335937500, -0.0392761230468750, -0.0635070800781250, -0.0410156250000000,
+-0.0558471679687500, -0.0424194335937500, -0.0350646972656250, -0.0475769042968750,
+-0.0257263183593750, -0.0495605468750000, 0.0020446777343750, -0.0490417480468750,
+-0.0069580078125000, -0.0501403808593750, -0.0042114257812500, -0.0525512695312500,
+0.0092468261718750, -0.0521850585937500, 0.0074462890625000, -0.0496826171875000,
+0.0058898925781250, -0.0556640625000000, 0.0059814453125000, -0.0581970214843750,
+0.0065917968750000, -0.0529479980468750, 0.0061645507812500, -0.0527954101562500,
+0.0054016113281250, -0.0552978515625000, 0.0094299316406250, -0.0335388183593750,
+0.0080871582031250, -0.0387878417968750, 0.0090332031250000, -0.0349731445312500,
+0.0133666992187500},
+{-0.0500488281250000, -0.0057983398437500, -0.0503540039062500, -0.0038452148437500,
+-0.0509338378906250, -0.0056762695312500, -0.0459594726562500, 0.0012207031250000,
+-0.0471191406250000, -0.0001220703125000, -0.0454711914062500, 0.0023803710937500,
+-0.0411682128906250, 0.0083923339843750, -0.0432128906250000, 0.0055541992187500,
+-0.0401611328125000, 0.0110778808593750, -0.0403442382812500, 0.0117187500000000,
+-0.0389404296875000, 0.0099792480468750, -0.0371093750000000, 0.0124511718750000,
+-0.0387878417968750, 0.0129394531250000, -0.0317687988281250, 0.0053100585937500,
+-0.0334167480468750, 0.0080261230468750, -0.0307312011718750, 0.0030822753906250,
+-0.0243225097656250, -0.0051269531250000, -0.0271911621093750, -0.0003356933593750,
+-0.0208740234375000, -0.0122680664062500, -0.0213012695312500, -0.0125732421875000,
+-0.0189819335937500, -0.0120849609375000, -0.0144653320312500, -0.0187988281250000,
+-0.0171508789062500, -0.0178527832031250, -0.0087280273437500, -0.0144958496093750,
+-0.0090637207031250, -0.0175781250000000, -0.0086364746093750, -0.0118713378906250,
+-0.0032958984375000, -0.0043640136718750, -0.0048522949218750, -0.0095214843750000,
+-0.0057678222656250, -0.0008239746093750, -0.0046386718750000, 0.0035705566406250,
+-0.0049133300781250, -0.0035095214843750, -0.0070190429687500, -0.0057678222656250,
+-0.0068054199218750, -0.0012817382812500, -0.0031738281250000, -0.0184020996093750,
+-0.0047912597656250, -0.0179748535156250, -0.0028686523437500, -0.0175781250000000,
+0.0016174316406250, -0.0271301269531250, -0.0003051757812500, -0.0247192382812500,
+0.0018005371093750, -0.0199279785156250, 0.0028381347656250, -0.0252685546875000,
+0.0020751953125000, -0.0162658691406250, 0.0025024414062500, -0.0057067871093750,
+0.0029296875000000, -0.0130920410156250, -0.0002136230468750, 0.0085144042968750,
+0.0021362304687500, 0.0079040527343750, -0.0023803710937500, 0.0077209472656250,
+-0.0067443847656250, 0.0224304199218750, -0.0028076171875000, 0.0192260742187500,
+-0.0197143554687500, 0.0028381347656250, -0.0148620605468750, 0.0112304687500000,
+-0.0235290527343750, 0.0007019042968750, -0.0405883789062500, -0.0198974609375000,
+-0.0307617187500000, -0.0083007812500000, -0.0490722656250000, -0.0231323242187500,
+-0.0513305664062500, -0.0317993164062500, -0.0502929687500000, -0.0198669433593750,
+-0.0574340820312500, -0.0120239257812500, -0.0551757812500000, -0.0194091796875000,
+-0.0561218261718750, -0.0100708007812500, -0.0585632324218750, -0.0039062500000000,
+-0.0530395507812500, -0.0146789550781250, -0.0502929687500000, -0.0189819335937500,
+-0.0541992187500000, -0.0110778808593750, -0.0352172851562500, -0.0386657714843750,
+-0.0381469726562500, -0.0382385253906250, -0.0353393554687500, -0.0367431640625000,
+-0.0138854980468750, -0.0564575195312500, -0.0080871582031250, -0.0227966308593750,
+-0.0017089843750000, -0.0522766113281250, 0.0003051757812500, -0.0455017089843750,
+0.0005187988281250, -0.0468444824218750, -0.0019836425781250, -0.0583190917968750,
+-0.0031433105468750, -0.0572204589843750, 0.0071716308593750, -0.0531005859375000,
+0.0059509277343750, -0.0540771484375000, 0.0043945312500000, -0.0533447265625000,
+0.0113220214843750, -0.0502624511718750, 0.0103759765625000, -0.0510864257812500,
+-0.0027160644531250, -0.0517272949218750, 0.0027465820312500, -0.0536193847656250,
+-0.0047912597656250, -0.0456542968750000, -0.0201416015625000, -0.0412902832031250,
+-0.0120849609375000, -0.0482177734375000, -0.0232543945312500, -0.0270690917968750,
+-0.0295104980468750, -0.0265197753906250, -0.0191955566406250, -0.0274047851562500,
+-0.0145263671875000, -0.0178222656250000, -0.0223083496093750, -0.0199279785156250,
+-0.0072631835937500, -0.0191650390625000, -0.0057983398437500, -0.0195617675781250,
+-0.0076293945312500, -0.0159606933593750, -0.0007324218750000, -0.0145874023437500,
+-0.0024414062500000, -0.0175170898437500, -0.0132446289062500, -0.0004577636718750,
+-0.0054626464843750, -0.0033569335937500, -0.0170288085937500, -0.0005187988281250,
+-0.0357971191406250, 0.0131835937500000, -0.0234985351562500, 0.0085754394531250,
+-0.0361633300781250, 0.0073547363281250, -0.0450439453125000, 0.0113525390625000,
+-0.0355224609375000, 0.0076293945312500, -0.0288391113281250, 0.0018920898437500,
+-0.0350036621093750, 0.0047607421875000, -0.0269470214843750, 0.0002441406250000,
+-0.0247802734375000, 0.0003967285156250, -0.0256347656250000, -0.0016174316406250,
+-0.0240783691406250, -0.0044860839843750, -0.0243835449218750, -0.0019836425781250,
+-0.0213623046875000, -0.0106506347656250, -0.0227355957031250, -0.0098266601562500,
+-0.0203552246093750, -0.0101013183593750, -0.0167541503906250, -0.0156250000000000,
+-0.0187988281250000, -0.0139160156250000, -0.0115966796875000, -0.0125122070312500,
+-0.0135192871093750, -0.0129699707031250, -0.0111694335937500, -0.0131530761718750,
+-0.0047302246093750, -0.0113220214843750, -0.0081787109375000, -0.0108337402343750,
+-0.0036926269531250, -0.0164794921875000, -0.0033569335937500, -0.0140380859375000,
+-0.0028076171875000, -0.0172424316406250, -0.0004272460937500, -0.0233154296875000,
+-0.0019226074218750, -0.0200500488281250, -0.0001525878906250, -0.0292663574218750,
+0.0009155273437500, -0.0276184082031250, -0.0014038085937500, -0.0314636230468750,
+-0.0021667480468750, -0.0392456054687500, -0.0004272460937500, -0.0345764160156250,
+-0.0061645507812500, -0.0444030761718750, -0.0061035156250000, -0.0449829101562500,
+-0.0051269531250000, -0.0440673828125000, -0.0080871582031250, -0.0481567382812500,
+-0.0079956054687500, -0.0473022460937500, -0.0037841796875000, -0.0468750000000000,
+-0.0054931640625000, -0.0468750000000000, -0.0039978027343750, -0.0476684570312500,
+0.0000000000000000, -0.0481567382812500, -0.0022583007812500, -0.0474548339843750,
+0.0004882812500000, -0.0486145019531250, 0.0003662109375000, -0.0497741699218750,
+0.0005187988281250, -0.0477600097656250, 0.0015563964843750, -0.0468139648437500,
+0.0002136230468750, -0.0483703613281250, 0.0023193359375000, -0.0422058105468750,
+0.0016479492187500, -0.0437316894531250, 0.0018310546875000, -0.0407409667968750,
+0.0033569335937500, -0.0355529785156250, 0.0021972656250000, -0.0388488769531250,
+0.0032958984375000, -0.0288391113281250, 0.0036010742187500, -0.0304870605468750,
+0.0032348632812500, -0.0271911621093750, 0.0036926269531250, -0.0193481445312500,
+0.0039062500000000, -0.0236206054687500, 0.0028686523437500, -0.0137634277343750,
+0.0035705566406250, -0.0145874023437500, 0.0024719238281250, -0.0121459960937500,
+0.0012512207031250, -0.0051269531250000, 0.0027160644531250, -0.0084533691406250,
+-0.0006713867187500, -0.0015563964843750, 0.0002746582031250, -0.0007629394531250,
+-0.0012512207031250, -0.0008544921875000, -0.0042724609375000, 0.0024719238281250,
+-0.0020141601562500, 0.0020141601562500, -0.0061645507812500, 0.0017700195312500,
+-0.0059509277343750, 0.0032348632812500, -0.0057678222656250, 0.0009155273437500,
+-0.0079956054687500, -0.0007934570312500, -0.0070495605468750, 0.0010070800781250,
+-0.0068969726562500, -0.0079650878906250, -0.0070190429687500, -0.0055541992187500,
+-0.0063781738281250, -0.0104675292968750, -0.0054931640625000, -0.0192565917968750,
+-0.0053405761718750, -0.0141906738281250, -0.0053405761718750, -0.0269775390625000,
+-0.0046691894531250, -0.0263977050781250, -0.0052185058593750, -0.0284118652343750,
+-0.0057678222656250, -0.0366210937500000, -0.0049438476562500, -0.0335083007812500,
+-0.0072021484375000, -0.0396423339843750, -0.0069274902343750, -0.0393371582031250,
+-0.0078430175781250, -0.0415344238281250, -0.0098266601562500, -0.0456542968750000,
+-0.0088806152343750, -0.0427246093750000, -0.0107116699218750, -0.0509338378906250,
+-0.0117492675781250, -0.0506896972656250, -0.0104064941406250, -0.0497741699218750,
+-0.0107116699218750, -0.0543212890625000, -0.0123291015625000, -0.0537719726562500,
+-0.0072631835937500, -0.0495300292968750, -0.0093078613281250, -0.0517578125000000,
+-0.0067138671875000, -0.0493469238281250, -0.0017700195312500, -0.0451660156250000,
+-0.0050964355468750, -0.0479736328125000, 0.0032043457031250, -0.0395507812500000,
+0.0023193359375000, -0.0422363281250000, 0.0039978027343750, -0.0376586914062500,
+0.0100097656250000, -0.0290222167968750, 0.0072631835937500, -0.0343322753906250,
+0.0111083984375000, -0.0227966308593750, 0.0126953125000000, -0.0223999023437500,
+0.0102539062500000, -0.0211181640625000, 0.0105590820312500, -0.0147399902343750,
+0.0116882324218750, -0.0166625976562500, 0.0031127929687500, -0.0108947753906250,
+0.0057983398437500, -0.0113525390625000, 0.0004577636718750, -0.0082702636718750,
+-0.0091247558593750, -0.0037231445312500, -0.0040893554687500, -0.0067749023437500,
+-0.0138244628906250, 0.0025329589843750, -0.0161437988281250, 0.0025024414062500,
+-0.0141906738281250, 0.0011901855468750, -0.0166931152343750, 0.0068664550781250,
+-0.0165100097656250, 0.0064086914062500, -0.0104675292968750, -0.0034179687500000,
+-0.0126037597656250, 0.0022888183593750, -0.0086364746093750, -0.0037231445312500,
+-0.0008544921875000, -0.0163574218750000, -0.0045166015625000, -0.0090332031250000,
+-0.0018615722656250, -0.0181579589843750, 0.0015563964843750, -0.0205993652343750,
+-0.0032043457031250, -0.0169372558593750, -0.0068969726562500, -0.0190429687500000,
+-0.0039367675781250, -0.0202636718750000, -0.0144653320312500, -0.0082092285156250,
+-0.0151062011718750, -0.0095520019531250, -0.0133666992187500, -0.0132751464843750,
+-0.0180053710937500, -0.0075073242187500, -0.0187072753906250, -0.0054931640625000,
+-0.0138549804687500, -0.0142211914062500, -0.0150146484375000, -0.0143737792968750,
+-0.0124816894531250, -0.0156860351562500, -0.0086669921875000, -0.0204467773437500,
+-0.0120544433593750, -0.0175781250000000, -0.0043334960937500, -0.0178222656250000,
+-0.0040283203125000, -0.0219726562500000, -0.0048522949218750, -0.0160522460937500,
+-0.0024414062500000, -0.0096740722656250, -0.0034484863281250, -0.0153198242187500,
+-0.0013122558593750, -0.0064392089843750, -0.0021362304687500, -0.0036926269531250,
+-0.0008239746093750, -0.0068359375000000, 0.0021057128906250, -0.0070800781250000,
+0.0004882812500000, -0.0058593750000000, 0.0007934570312500, -0.0057983398437500,
+0.0025634765625000, -0.0071716308593750, 0.0016479492187500, -0.0053100585937500,
+-0.0002441406250000, -0.0032348632812500, 0.0010681152343750, -0.0035705566406250,
+0.0021362304687500, 0.0038146972656250, 0.0017395019531250, 0.0019836425781250,
+0.0012512207031250, 0.0020751953125000, 0.0024108886718750, 0.0070495605468750,
+0.0025024414062500, 0.0054931640625000, -0.0033874511718750, 0.0066528320312500,
+-0.0018310546875000, 0.0072326660156250, -0.0021362304687500, 0.0073547363281250,
+-0.0057983398437500, 0.0081176757812500, -0.0055541992187500, 0.0073547363281250,
+-0.0113830566406250, 0.0050048828125000, -0.0072631835937500, 0.0076904296875000,
+-0.0136413574218750, 0.0041198730468750, -0.0231018066406250, -0.0013732910156250,
+-0.0166625976562500, 0.0013427734375000, -0.0306091308593750, -0.0054931640625000,
+-0.0318298339843750, -0.0052185058593750, -0.0298767089843750, -0.0053100585937500,
+-0.0356750488281250, -0.0096130371093750, -0.0346374511718750, -0.0087280273437500,
+-0.0283203125000000, -0.0075683593750000, -0.0326843261718750, -0.0075073242187500,
+-0.0260925292968750, -0.0081787109375000, -0.0155334472656250, -0.0079040527343750,
+-0.0221862792968750, -0.0074157714843750, -0.0132141113281250, -0.0108642578125000,
+-0.0101318359375000, -0.0104370117187500, -0.0131530761718750, -0.0086364746093750,
+-0.0132141113281250, -0.0108642578125000, -0.0115356445312500, -0.0119018554687500,
+-0.0108337402343750},
+{0.0037231445312500, -0.0288696289062500, 0.0023193359375000, -0.0265808105468750,
+0.0025024414062500, -0.0259399414062500, 0.0028076171875000, -0.0252380371093750,
+0.0021362304687500, -0.0242614746093750, 0.0023498535156250, -0.0244750976562500,
+0.0036010742187500, -0.0198669433593750, 0.0035705566406250, -0.0217285156250000,
+0.0035400390625000, -0.0189819335937500, 0.0051574707031250, -0.0132751464843750,
+0.0052490234375000, -0.0166320800781250, 0.0002136230468750, -0.0138244628906250,
+0.0030517578125000, -0.0106201171875000, -0.0010986328125000, -0.0151062011718750,
+-0.0078125000000000, -0.0178833007812500, -0.0039367675781250, -0.0135498046875000,
+-0.0140075683593750, -0.0226135253906250, -0.0140686035156250, -0.0230407714843750,
+-0.0131225585937500, -0.0198364257812500, -0.0186462402343750, -0.0230407714843750,
+-0.0176086425781250, -0.0234375000000000, -0.0115051269531250, -0.0138244628906250,
+-0.0136413574218750, -0.0178222656250000, -0.0111083984375000, -0.0106201171875000,
+-0.0046997070312500, 0.0022277832031250, -0.0073547363281250, -0.0064086914062500,
+-0.0049438476562500, -0.0016174316406250, -0.0024719238281250, 0.0058593750000000,
+-0.0069580078125000, -0.0014038085937500, -0.0105590820312500, -0.0098266601562500,
+-0.0073547363281250, -0.0051574707031250, -0.0126037597656250, -0.0118713378906250,
+-0.0151672363281250, -0.0137023925781250, -0.0131530761718750, -0.0108337402343750,
+-0.0132446289062500, -0.0127563476562500, -0.0140686035156250, -0.0144958496093750,
+-0.0063476562500000, -0.0045776367187500, -0.0096740722656250, -0.0052795410156250,
+-0.0065307617187500, -0.0072937011718750, 0.0021667480468750, -0.0021362304687500,
+-0.0022277832031250, -0.0017700195312500, 0.0011291503906250, -0.0093994140625000,
+0.0028381347656250, -0.0096130371093750, -0.0002746582031250, -0.0058593750000000,
+-0.0017089843750000, -0.0077209472656250, -0.0003662109375000, -0.0101623535156250,
+-0.0036621093750000, -0.0017395019531250, -0.0043945312500000, -0.0026245117187500,
+-0.0044860839843750, -0.0006408691406250, -0.0055847167968750, 0.0068054199218750,
+-0.0050659179687500, 0.0057067871093750, -0.0048217773437500, 0.0064697265625000,
+-0.0047302246093750, 0.0067138671875000, -0.0051879882812500, 0.0067749023437500,
+-0.0051269531250000, 0.0072631835937500, -0.0044860839843750, 0.0068969726562500,
+-0.0065917968750000, 0.0070495605468750, -0.0060729980468750, 0.0077819824218750,
+-0.0061035156250000, 0.0054626464843750, -0.0081176757812500, 0.0042114257812500,
+-0.0078125000000000, 0.0061035156250000, -0.0052795410156250, -0.0003967285156250,
+-0.0043029785156250, -0.0004577636718750, -0.0090332031250000, -0.0011596679687500,
+-0.0103149414062500, -0.0041809082031250, -0.0063476562500000, -0.0025939941406250,
+-0.0195007324218750, -0.0068054199218750, -0.0188903808593750, -0.0070190429687500,
+-0.0184326171875000, -0.0071411132812500, -0.0256958007812500, -0.0087280273437500,
+-0.0239868164062500, -0.0077514648437500, -0.0217590332031250, -0.0103149414062500,
+-0.0237121582031250, -0.0110473632812500, -0.0201416015625000, -0.0087585449218750,
+-0.0169372558593750, -0.0086059570312500, -0.0202941894531250, -0.0097961425781250,
+-0.0085754394531250, -0.0024414062500000, -0.0093688964843750, -0.0042114257812500,
+-0.0090942382812500, -0.0025329589843750, -0.0015563964843750, 0.0046997070312500,
+-0.0034484863281250, 0.0025024414062500, -0.0061950683593750, 0.0003356933593750,
+-0.0039367675781250, 0.0025939941406250, -0.0049743652343750, 0.0010681152343750,
+-0.0090637207031250, -0.0021667480468750, -0.0080871582031250, -0.0002746582031250,
+-0.0071716308593750, -0.0028991699218750, -0.0075378417968750, -0.0037231445312500,
+-0.0090026855468750, -0.0003662109375000, -0.0087280273437500, 0.0005493164062500,
+-0.0079345703125000, -0.0011291503906250, -0.0140991210937500, 0.0068664550781250,
+-0.0124816894531250, 0.0072021484375000, -0.0150146484375000, 0.0073242187500000,
+-0.0210876464843750, 0.0118713378906250, -0.0181884765625000, 0.0114746093750000,
+-0.0220031738281250, 0.0090026855468750, -0.0235595703125000, 0.0108032226562500,
+-0.0230712890625000, 0.0095825195312500, -0.0235900878906250, 0.0064697265625000,
+-0.0242919921875000, 0.0081481933593750, -0.0257568359375000, 0.0061645507812500,
+-0.0241394042968750, 0.0063171386718750, -0.0278320312500000, 0.0062866210937500,
+-0.0309143066406250, 0.0053710937500000, -0.0273742675781250, 0.0062255859375000,
+-0.0381469726562500, 0.0052490234375000, -0.0364074707031250, 0.0054321289062500,
+-0.0373229980468750, 0.0047302246093750, -0.0447387695312500, 0.0038146972656250,
+-0.0414733886718750, 0.0042419433593750, -0.0421752929687500, 0.0007629394531250,
+-0.0440368652343750, 0.0016479492187500, -0.0412597656250000, -0.0006103515625000,
+-0.0388488769531250, -0.0041809082031250, -0.0402526855468750, -0.0025329589843750,
+-0.0351867675781250, -0.0088806152343750, -0.0357360839843750, -0.0088195800781250,
+-0.0335083007812500, -0.0102233886718750, -0.0293884277343750, -0.0145874023437500,
+-0.0314941406250000, -0.0131530761718750, -0.0256652832031250, -0.0151977539062500,
+-0.0254821777343750, -0.0159912109375000, -0.0253601074218750, -0.0145874023437500,
+-0.0222778320312500, -0.0143127441406250, -0.0228271484375000, -0.0149230957031250,
+-0.0216064453125000, -0.0116577148437500, -0.0213317871093750, -0.0121459960937500,
+-0.0203247070312500, -0.0104980468750000, -0.0192871093750000, -0.0077514648437500,
+-0.0197448730468750, -0.0090637207031250, -0.0170593261718750, -0.0045471191406250,
+-0.0171203613281250, -0.0048828125000000, -0.0165405273437500, -0.0032653808593750,
+-0.0148315429687500, 0.0002136230468750, -0.0154113769531250, -0.0015563964843750,
+-0.0139160156250000, 0.0025024414062500, -0.0144042968750000, 0.0032653808593750,
+-0.0133972167968750, 0.0032653808593750, -0.0117492675781250, 0.0048217773437500,
+-0.0130310058593750, 0.0046386718750000, -0.0109863281250000, 0.0046997070312500,
+-0.0106811523437500, 0.0048828125000000, -0.0122985839843750, 0.0048522949218750,
+-0.0120544433593750, 0.0046691894531250, -0.0116882324218750, 0.0044860839843750,
+-0.0173645019531250, 0.0047302246093750, -0.0160827636718750, 0.0050659179687500,
+-0.0184326171875000, 0.0045471191406250, -0.0235290527343750, 0.0043945312500000,
+-0.0210571289062500, 0.0047607421875000, -0.0258178710937500, 0.0025634765625000,
+-0.0258483886718750, 0.0037231445312500, -0.0265808105468750, 0.0019531250000000,
+-0.0293273925781250, -0.0005187988281250, -0.0277099609375000, 0.0016174316406250,
+-0.0299377441406250, -0.0036010742187500, -0.0303649902343750, -0.0032958984375000,
+-0.0289611816406250, -0.0028076171875000, -0.0286254882812500, -0.0058898925781250,
+-0.0288391113281250, -0.0050048828125000, -0.0271911621093750, -0.0021972656250000,
+-0.0261230468750000, -0.0044555664062500, -0.0270996093750000, -0.0008239746093750,
+-0.0271301269531250, 0.0036621093750000, -0.0256042480468750, -0.0000305175781250,
+-0.0288696289062500, 0.0094909667968750, -0.0285644531250000, 0.0089111328125000,
+-0.0286254882812500, 0.0097656250000000, -0.0302124023437500, 0.0157165527343750,
+-0.0292358398437500, 0.0135803222656250, -0.0300903320312500, 0.0157165527343750,
+-0.0298461914062500, 0.0171508789062500, -0.0298767089843750, 0.0143127441406250,
+-0.0307617187500000, 0.0139465332031250, -0.0301513671875000, 0.0160522460937500,
+-0.0290222167968750, 0.0065917968750000, -0.0297851562500000, 0.0087280273437500,
+-0.0283813476562500, 0.0055236816406250, -0.0261840820312500, -0.0026855468750000,
+-0.0272827148437500, 0.0016784667968750, -0.0263061523437500, -0.0042724609375000,
+-0.0255737304687500, -0.0066223144531250, -0.0262145996093750, -0.0031738281250000,
+-0.0268859863281250, -0.0025329589843750, -0.0263366699218750, -0.0041503906250000,
+-0.0254821777343750, 0.0014038085937500, -0.0263977050781250, 0.0019531250000000,
+-0.0250549316406250, 0.0017089843750000, -0.0228576660156250, 0.0048217773437500,
+-0.0238037109375000, 0.0042724609375000, -0.0223388671875000, -0.0011291503906250,
+-0.0219421386718750, 0.0020446777343750, -0.0214233398437500, -0.0018615722656250,
+-0.0210876464843750, -0.0094604492187500, -0.0213012695312500, -0.0057373046875000,
+-0.0167236328125000, -0.0128173828125000, -0.0177307128906250, -0.0139465332031250,
+-0.0170593261718750, -0.0130310058593750, -0.0124206542968750, -0.0154724121093750,
+-0.0137939453125000, -0.0148620605468750, -0.0188293457031250, -0.0126647949218750,
+-0.0153198242187500, -0.0146789550781250, -0.0197143554687500, -0.0115051269531250,
+-0.0281677246093750, -0.0071105957031250, -0.0239868164062500, -0.0096435546875000,
+-0.0278930664062500, -0.0026245117187500, -0.0307922363281250, -0.0018920898437500,
+-0.0277404785156250, -0.0042724609375000, -0.0261840820312500, -0.0024414062500000,
+-0.0279541015625000, -0.0005187988281250, -0.0215454101562500, -0.0023498535156250,
+-0.0227355957031250, -0.0051269531250000, -0.0214538574218750, -0.0024108886718750,
+-0.0160827636718750, 0.0009765625000000, -0.0179748535156250, -0.0000305175781250,
+-0.0170288085937500, 0.0054931640625000, -0.0169677734375000, 0.0053710937500000,
+-0.0142822265625000, 0.0025939941406250, -0.0126647949218750, 0.0054321289062500,
+-0.0151062011718750, 0.0079956054687500, -0.0072021484375000, -0.0017700195312500,
+-0.0070800781250000, -0.0003051757812500, -0.0075988769531250, -0.0052795410156250,
+-0.0048217773437500, -0.0127868652343750, -0.0053710937500000, -0.0064697265625000,
+-0.0013122558593750, -0.0188598632812500, -0.0029602050781250, -0.0204467773437500,
+-0.0027770996093750, -0.0184631347656250, 0.0027770996093750, -0.0238952636718750,
+0.0089111328125000, -0.0018920898437500, -0.0021667480468750, -0.0256042480468750,
+0.0023803710937500, -0.0236206054687500, -0.0035400390625000, -0.0213928222656250,
+-0.0122375488281250, -0.0241394042968750, -0.0074157714843750, -0.0252380371093750,
+-0.0112915039062500, -0.0291442871093750, -0.0159606933593750, -0.0238342285156250,
+-0.0091247558593750, -0.0324707031250000, -0.0043945312500000, -0.0437622070312500,
+-0.0104370117187500, -0.0358581542968750, 0.0024719238281250, -0.0510864257812500,
+0.0050048828125000, -0.0557250976562500, 0.0001525878906250, -0.0466613769531250,
+0.0028076171875000, -0.0469360351562500, 0.0047607421875000, -0.0521545410156250,
+-0.0081481933593750, -0.0313110351562500, -0.0074462890625000, -0.0327148437500000,
+-0.0042114257812500, -0.0315246582031250, -0.0095520019531250, -0.0188903808593750,
+-0.0116882324218750, -0.0227050781250000, -0.0086364746093750, -0.0221557617187500,
+-0.0045166015625000, -0.0216674804687500, -0.0116577148437500, -0.0193481445312500,
+-0.0180053710937500, -0.0182495117187500, -0.0124206542968750, -0.0207824707031250,
+-0.0238952636718750, -0.0135192871093750, -0.0249633789062500, -0.0126953125000000,
+-0.0242614746093750, -0.0134887695312500, -0.0287780761718750, -0.0114135742187500,
+-0.0267944335937500, -0.0114746093750000, -0.0257873535156250, -0.0089721679687500,
+-0.0307922363281250, -0.0108032226562500, -0.0202636718750000, -0.0073852539062500,
+-0.0102233886718750, -0.0023803710937500, -0.0192260742187500, -0.0054931640625000,
+0.0003051757812500, -0.0003356933593750, 0.0019531250000000, 0.0017089843750000,
+0.0003051757812500, -0.0015563964843750, 0.0090332031250000, -0.0016784667968750,
+0.0070800781250000, 0.0001831054687500, 0.0002441406250000, -0.0089416503906250,
+0.0049743652343750, -0.0072937011718750, -0.0004577636718750, -0.0097656250000000,
+-0.0110168457031250, -0.0166015625000000, -0.0048828125000000, -0.0134887695312500,
+-0.0095214843750000, -0.0210266113281250, -0.0150451660156250, -0.0197448730468750,
+-0.0065307617187500},
+{-0.0231933593750000, -0.0039672851562500, -0.0281066894531250, -0.0057983398437500,
+-0.0330810546875000, -0.0086059570312500, -0.0289916992187500, -0.0069274902343750,
+-0.0354614257812500, -0.0120544433593750, -0.0385437011718750, -0.0113830566406250,
+-0.0334472656250000, -0.0117492675781250, -0.0318908691406250, -0.0160827636718750,
+-0.0351867675781250, -0.0148010253906250, -0.0250549316406250, -0.0111389160156250,
+-0.0260620117187500, -0.0144958496093750, -0.0231018066406250, -0.0077209472656250,
+-0.0165710449218750, 0.0008850097656250, -0.0207214355468750, -0.0053710937500000,
+-0.0090942382812500, 0.0031738281250000, -0.0098266601562500, 0.0057983398437500,
+-0.0086975097656250, 0.0056762695312500, -0.0018310546875000, 0.0081787109375000,
+-0.0041198730468750, 0.0062866210937500, 0.0024108886718750, 0.0039672851562500,
+0.0017700195312500, 0.0066833496093750, 0.0021057128906250, 0.0054931640625000,
+0.0063781738281250, 0.0011596679687500, 0.0051879882812500, 0.0020446777343750,
+0.0060424804687500, 0.0010375976562500, 0.0049438476562500, -0.0006713867187500,
+0.0078735351562500, 0.0046081542968750, 0.0111083984375000, 0.0094299316406250,
+0.0076599121093750, 0.0042114257812500, 0.0111083984375000, 0.0044860839843750,
+0.0143127441406250, 0.0115966796875000, 0.0101318359375000, 0.0029907226562500,
+0.0036621093750000, -0.0160827636718750, 0.0005798339843750, 0.0124816894531250,
+0.0035095214843750, 0.0027770996093750, -0.0005798339843750, 0.0150146484375000,
+0.0036926269531250, 0.0017395019531250, 0.0094909667968750, -0.0168151855468750,
+0.0057983398437500, -0.0073852539062500, 0.0083007812500000, -0.0174560546875000,
+0.0112609863281250, -0.0214843750000000, 0.0066833496093750, -0.0183410644531250,
+0.0041809082031250, -0.0179443359375000, 0.0076904296875000, -0.0194396972656250,
+-0.0047912597656250, -0.0222778320312500, -0.0046386718750000, -0.0173950195312500,
+-0.0038452148437500, -0.0250854492187500, -0.0107727050781250, -0.0359802246093750,
+-0.0093383789062500, -0.0286254882812500, -0.0027465820312500, -0.0350952148437500,
+-0.0044250488281250, -0.0419311523437500, -0.0086059570312500, -0.0318908691406250,
+-0.0036926269531250, -0.0242919921875000, -0.0003967285156250, -0.0319213867187500,
+-0.0211486816406250, -0.0158691406250000, -0.0185241699218750, -0.0126647949218750,
+-0.0191650390625000, -0.0176391601562500, -0.0326843261718750, -0.0154418945312500,
+-0.0286560058593750, -0.0137634277343750, -0.0246582031250000, -0.0183105468750000,
+-0.0299377441406250, -0.0199584960937500, -0.0221557617187500, -0.0177307128906250,
+-0.0134582519531250, -0.0172424316406250, -0.0202026367187500, -0.0182800292968750,
+0.0009765625000000, -0.0146484375000000, 0.0011291503906250, -0.0147094726562500,
+-0.0021057128906250, -0.0140686035156250, 0.0073852539062500, -0.0119018554687500,
+0.0073852539062500, -0.0127563476562500, 0.0014038085937500, -0.0103759765625000,
+0.0030212402343750, -0.0107421875000000, 0.0009765625000000, -0.0097351074218750,
+-0.0043640136718750, -0.0073547363281250, -0.0011596679687500, -0.0085754394531250,
+-0.0019531250000000, -0.0077209472656250, -0.0062866210937500, -0.0057983398437500,
+0.0007629394531250, -0.0090942382812500, 0.0066833496093750, -0.0113830566406250,
+0.0001831054687500, -0.0087890625000000, 0.0155334472656250, -0.0180358886718750,
+0.0165100097656250, -0.0168151855468750, 0.0129699707031250, -0.0188293457031250,
+0.0189819335937500, -0.0252380371093750, 0.0195922851562500, -0.0222778320312500,
+0.0098266601562500, -0.0282592773437500, 0.0127563476562500, -0.0281677246093750,
+0.0097351074218750, -0.0288085937500000, 0.0015563964843750, -0.0327758789062500,
+0.0055236816406250, -0.0311889648437500, -0.0028991699218750, -0.0313415527343750,
+-0.0028076171875000, -0.0328063964843750, -0.0028686523437500, -0.0306396484375000,
+-0.0082397460937500, -0.0283508300781250, -0.0067138671875000, -0.0297851562500000,
+-0.0032958984375000, -0.0264282226562500, -0.0065612792968750, -0.0260620117187500,
+-0.0024414062500000, -0.0255737304687500, 0.0033264160156250, -0.0241088867187500,
+-0.0011596679687500, -0.0248107910156250, 0.0078430175781250, -0.0215148925781250,
+0.0071411132812500, -0.0222167968750000, 0.0076599121093750, -0.0215454101562500,
+0.0129089355468750, -0.0186767578125000, 0.0104370117187500, -0.0198669433593750,
+0.0131225585937500, -0.0202941894531250, 0.0137939453125000, -0.0199584960937500,
+0.0130310058593750, -0.0202941894531250, 0.0138549804687500, -0.0216064453125000,
+0.0142822265625000, -0.0217285156250000, 0.0125122070312500, -0.0191345214843750,
+0.0137939453125000, -0.0200805664062500, 0.0122375488281250, -0.0191345214843750,
+0.0104675292968750, -0.0161743164062500, 0.0128479003906250, -0.0173034667968750,
+0.0065917968750000, -0.0167236328125000, 0.0087890625000000, -0.0157470703125000,
+0.0065002441406250, -0.0166015625000000, 0.0010070800781250, -0.0174255371093750,
+0.0046997070312500, -0.0167846679687500, -0.0029296875000000, -0.0180969238281250,
+-0.0017089843750000, -0.0178833007812500, -0.0032958984375000, -0.0184936523437500,
+-0.0084228515625000, -0.0196228027343750, -0.0050964355468750, -0.0189514160156250,
+-0.0114440917968750, -0.0203857421875000, -0.0102844238281250, -0.0207824707031250,
+-0.0115051269531250, -0.0204162597656250, -0.0157470703125000, -0.0209655761718750,
+-0.0128479003906250, -0.0212097167968750, -0.0183410644531250, -0.0192565917968750,
+-0.0173645019531250, -0.0203552246093750, -0.0181884765625000, -0.0188903808593750,
+-0.0219421386718750, -0.0166015625000000, -0.0196228027343750, -0.0183715820312500,
+-0.0224609375000000, -0.0141296386718750, -0.0221557617187500, -0.0143737792968750,
+-0.0220031738281250, -0.0137939453125000, -0.0234375000000000, -0.0107116699218750,
+-0.0219726562500000, -0.0114746093750000, -0.0216064453125000, -0.0109252929687500,
+-0.0225219726562500, -0.0099792480468750, -0.0197143554687500, -0.0104980468750000,
+-0.0175170898437500, -0.0111083984375000, -0.0191040039062500, -0.0108032226562500,
+-0.0119018554687500, -0.0119934082031250, -0.0129394531250000, -0.0119934082031250,
+-0.0109863281250000, -0.0125122070312500, -0.0054931640625000, -0.0137939453125000,
+-0.0075988769531250, -0.0137634277343750, -0.0021362304687500, -0.0140380859375000,
+-0.0019836425781250, -0.0148620605468750, -0.0016479492187500, -0.0146484375000000,
+0.0017089843750000, -0.0137023925781250, 0.0007629394531250, -0.0141906738281250,
+0.0000000000000000, -0.0158081054687500, 0.0013732910156250, -0.0140075683593750,
+-0.0006103515625000, -0.0159606933593750, -0.0031738281250000, -0.0186767578125000,
+-0.0016174316406250, -0.0164489746093750, -0.0050659179687500, -0.0224914550781250,
+-0.0055847167968750, -0.0208740234375000, -0.0058593750000000, -0.0231628417968750,
+-0.0076599121093750, -0.0288391113281250, -0.0073242187500000, -0.0257263183593750,
+-0.0076904296875000, -0.0302124023437500, -0.0086364746093750, -0.0314025878906250,
+-0.0070800781250000, -0.0313720703125000, -0.0061035156250000, -0.0319519042968750,
+-0.0071716308593750, -0.0315551757812500, -0.0015258789062500, -0.0371398925781250,
+-0.0026550292968750, -0.0346374511718750, -0.0001220703125000, -0.0374755859375000,
+0.0052795410156250, -0.0440979003906250, 0.0028991699218750, -0.0403747558593750,
+0.0068664550781250, -0.0434875488281250, 0.0079956054687500, -0.0463256835937500,
+0.0071105957031250, -0.0403137207031250, 0.0080261230468750, -0.0377197265625000,
+0.0080871582031250, -0.0419006347656250, 0.0058898925781250, -0.0249938964843750,
+0.0063171386718750, -0.0279541015625000, 0.0049438476562500, -0.0236816406250000,
+0.0033569335937500, -0.0104370117187500, 0.0047607421875000, -0.0172119140625000,
+-0.0008850097656250, -0.0105895996093750, -0.0003051757812500, -0.0068359375000000,
+-0.0001220703125000, -0.0104675292968750, -0.0033569335937500, -0.0136108398437500,
+-0.0028076171875000, -0.0115356445312500, -0.0028991699218750, -0.0065612792968750,
+-0.0020141601562500, -0.0105895996093750, -0.0037231445312500, -0.0078430175781250,
+-0.0055541992187500, -0.0010681152343750, -0.0039062500000000, -0.0042419433593750,
+-0.0065612792968750, 0.0026550292968750, -0.0064086914062500, 0.0027465820312500,
+-0.0068969726562500, 0.0021362304687500, -0.0089416503906250, 0.0049133300781250,
+-0.0078735351562500, 0.0043029785156250, -0.0072326660156250, 0.0061340332031250,
+-0.0076293945312500, 0.0039062500000000, -0.0078430175781250, 0.0086364746093750,
+-0.0073547363281250, 0.0137023925781250, -0.0067443847656250, 0.0079956054687500,
+-0.0072326660156250, 0.0155334472656250, -0.0077819824218750, 0.0189819335937500,
+-0.0063781738281250, 0.0151367187500000, -0.0060424804687500, 0.0141906738281250,
+-0.0072021484375000, 0.0154418945312500, -0.0014038085937500, 0.0070800781250000,
+-0.0016479492187500, 0.0093688964843750, -0.0035400390625000, 0.0067443847656250,
+-0.0017089843750000, -0.0022583007812500, -0.0030212402343750, -0.0009460449218750,
+-0.0099487304687500, -0.0045776367187500, -0.0082092285156250, -0.0044555664062500,
+-0.0071105957031250, -0.0032043457031250, -0.0117492675781250, -0.0047302246093750,
+-0.0111083984375000, -0.0050659179687500, -0.0045166015625000, -0.0029296875000000,
+-0.0067443847656250, -0.0027465820312500, -0.0055847167968750, -0.0036315917968750,
+0.0009155273437500, -0.0024719238281250, -0.0006103515625000, -0.0021667480468750,
+-0.0025939941406250, -0.0082397460937500, -0.0022583007812500, -0.0074157714843750,
+0.0022583007812500, -0.0070190429687500, 0.0021972656250000, -0.0107727050781250,
+-0.0017395019531250, -0.0102844238281250, 0.0139465332031250, -0.0077209472656250,
+0.0146179199218750, -0.0095825195312500, 0.0112915039062500, -0.0057067871093750,
+0.0181884765625000, -0.0003051757812500, 0.0188293457031250, -0.0037841796875000,
+0.0050659179687500, 0.0000305175781250, 0.0090332031250000, 0.0022583007812500,
+0.0063476562500000, 0.0007629394531250, -0.0057373046875000, 0.0000610351562500,
+-0.0016174316406250, 0.0011596679687500, -0.0032653808593750, 0.0007629394531250,
+-0.0063171386718750, -0.0004882812500000, -0.0027465820312500, 0.0029602050781250,
+0.0009765625000000, 0.0061950683593750, -0.0020751953125000, 0.0036926269531250,
+-0.0007629394531250, 0.0074768066406250, 0.0024108886718750, 0.0090942382812500,
+-0.0018310546875000, 0.0084838867187500, -0.0054016113281250, 0.0089416503906250,
+-0.0023498535156250, 0.0089416503906250, -0.0131225585937500, 0.0079040527343750,
+-0.0116271972656250, 0.0085449218750000, -0.0135192871093750, 0.0083618164062500,
+-0.0211791992187500, 0.0068664550781250, -0.0184020996093750, 0.0072631835937500,
+-0.0238037109375000, 0.0085449218750000, -0.0237121582031250, 0.0082092285156250,
+-0.0242919921875000, 0.0079956054687500, -0.0278930664062500, 0.0094909667968750,
+-0.0265197753906250, 0.0095825195312500, -0.0275268554687500, 0.0046386718750000,
+-0.0281677246093750, 0.0066833496093750, -0.0277099609375000, 0.0030212402343750,
+-0.0275573730468750, -0.0034790039062500, -0.0278015136718750, -0.0003356933593750,
+-0.0267639160156250, -0.0060729980468750, -0.0270996093750000, -0.0073547363281250,
+-0.0258483886718750, -0.0062866210937500, -0.0247802734375000, -0.0077819824218750,
+-0.0254516601562500, -0.0075378417968750, -0.0200195312500000, -0.0050354003906250,
+-0.0215454101562500, -0.0055236816406250, -0.0192871093750000, -0.0038452148437500,
+-0.0134887695312500, -0.0010681152343750, -0.0161437988281250, -0.0021667480468750,
+-0.0146789550781250, 0.0001525878906250, -0.0119018554687500, 0.0007324218750000,
+-0.0148925781250000, 0.0004577636718750, -0.0175781250000000, 0.0012817382812500,
+-0.0142211914062500},
+{-0.0069580078125000, 0.0146789550781250, -0.0042419433593750, 0.0100402832031250,
+-0.0108947753906250, 0.0185241699218750, -0.0116271972656250, 0.0188293457031250,
+-0.0096435546875000, 0.0172119140625000, -0.0113525390625000, 0.0216369628906250,
+-0.0116271972656250, 0.0216674804687500, -0.0076904296875000, 0.0153198242187500,
+-0.0079650878906250, 0.0174865722656250, -0.0076293945312500, 0.0132141113281250,
+-0.0049133300781250, 0.0070495605468750, -0.0050354003906250, 0.0117187500000000,
+-0.0059814453125000, 0.0001220703125000, -0.0064392089843750, -0.0007019042968750,
+-0.0043029785156250, 0.0003356933593750, -0.0019836425781250, -0.0046997070312500,
+-0.0022583007812500, -0.0040588378906250, -0.0007324218750000, -0.0016784667968750,
+-0.0003967285156250, -0.0024719238281250, -0.0008239746093750, -0.0030517578125000,
+-0.0006713867187500, -0.0012512207031250, -0.0002746582031250, -0.0011291503906250,
+0.0004272460937500, -0.0049133300781250, -0.0013427734375000, -0.0033264160156250,
+0.0021057128906250, -0.0061035156250000, 0.0058593750000000, -0.0111999511718750,
+0.0025024414062500, -0.0081481933593750, 0.0082397460937500, -0.0104064941406250,
+0.0099487304687500, -0.0123596191406250, 0.0077819824218750, -0.0102539062500000,
+0.0069885253906250, -0.0090942382812500, 0.0084228515625000, -0.0098876953125000,
+0.0122375488281250, -0.0035400390625000, 0.0083312988281250, -0.0043945312500000,
+0.0126647949218750, -0.0041503906250000, 0.0209960937500000, -0.0002136230468750,
+0.0166320800781250, -0.0006713867187500, 0.0190734863281250, 0.0018005371093750,
+0.0236816406250000, 0.0007019042968750, 0.0163269042968750, 0.0026245117187500,
+0.0107727050781250, 0.0061340332031250, 0.0166320800781250, 0.0044860839843750,
+0.0008544921875000, 0.0078430175781250, 0.0015563964843750, 0.0088195800781250,
+0.0003356933593750, 0.0071716308593750, -0.0099792480468750, 0.0072937011718750,
+-0.0068969726562500, 0.0085754394531250, -0.0053405761718750, 0.0069580078125000,
+-0.0075073242187500, 0.0061035156250000, -0.0075988769531250, 0.0069885253906250,
+-0.0048828125000000, 0.0078430175781250, -0.0050354003906250, 0.0075988769531250,
+-0.0101318359375000, 0.0070800781250000, -0.0076599121093750, 0.0080261230468750,
+-0.0114440917968750, 0.0064086914062500, -0.0181579589843750, 0.0050354003906250,
+-0.0141296386718750, 0.0063781738281250, -0.0195922851562500, 0.0004577636718750,
+-0.0208129882812500, 0.0016174316406250, -0.0205078125000000, -0.0005187988281250,
+-0.0219726562500000, -0.0059509277343750, -0.0213012695312500, -0.0033569335937500,
+-0.0233459472656250, -0.0053100585937500, -0.0231018066406250, -0.0075683593750000,
+-0.0229492187500000, -0.0040893554687500, -0.0242309570312500, -0.0012512207031250,
+-0.0237121582031250, -0.0033569335937500, -0.0225219726562500, 0.0018310546875000,
+-0.0234375000000000, 0.0029296875000000, -0.0217590332031250, 0.0020446777343750,
+-0.0194702148437500, 0.0036010742187500, -0.0206909179687500, 0.0036926269531250,
+-0.0174255371093750, 0.0003662109375000, -0.0178833007812500, 0.0016174316406250,
+-0.0154113769531250, -0.0011901855468750, -0.0126647949218750, -0.0053100585937500,
+-0.0148010253906250, -0.0029907226562500, -0.0058288574218750, -0.0087585449218750,
+-0.0071716308593750, -0.0087890625000000, -0.0059204101562500, -0.0093688964843750,
+0.0010986328125000, -0.0126342773437500, -0.0010070800781250, -0.0113830566406250,
+-0.0021057128906250, -0.0133361816406250, 0.0011596679687500, -0.0138549804687500,
+-0.0020141601562500, -0.0130615234375000, -0.0066223144531250, -0.0134887695312500,
+-0.0033569335937500, -0.0136413574218750, -0.0077514648437500, -0.0106506347656250,
+-0.0086059570312500, -0.0113830566406250, -0.0075378417968750, -0.0095520019531250,
+-0.0083923339843750, -0.0063781738281250, -0.0084228515625000, -0.0078735351562500,
+-0.0085754394531250, -0.0042114257812500, -0.0079345703125000, -0.0038452148437500,
+-0.0090026855468750, -0.0036621093750000, -0.0099182128906250, -0.0020751953125000,
+-0.0090026855468750, -0.0024108886718750, -0.0120849609375000, -0.0012207031250000,
+-0.0114440917968750, -0.0014038085937500, -0.0130004882812500, -0.0003967285156250,
+-0.0156860351562500, 0.0008544921875000, -0.0140075683593750, 0.0001220703125000,
+-0.0187377929687500, 0.0028381347656250, -0.0182189941406250, 0.0023498535156250,
+-0.0194702148437500, 0.0035705566406250, -0.0229187011718750, 0.0056762695312500,
+-0.0211791992187500, 0.0042724609375000, -0.0241088867187500, 0.0079956054687500,
+-0.0248718261718750, 0.0074462890625000, -0.0236511230468750, 0.0079650878906250,
+-0.0244750976562500, 0.0104675292968750, -0.0249633789062500, 0.0091247558593750,
+-0.0200500488281250, 0.0105285644531250, -0.0218505859375000, 0.0105590820312500,
+-0.0187988281250000, 0.0103454589843750, -0.0134887695312500, 0.0108032226562500,
+-0.0166931152343750, 0.0102844238281250, -0.0094299316406250, 0.0103454589843750,
+-0.0094604492187500, 0.0101928710937500, -0.0088195800781250, 0.0093994140625000,
+-0.0047607421875000, 0.0090637207031250, -0.0064086914062500, 0.0088500976562500,
+-0.0037841796875000, 0.0061950683593750, -0.0036010742187500, 0.0072326660156250,
+-0.0036315917968750, 0.0044250488281250, -0.0022277832031250, 0.0010681152343750,
+-0.0025329589843750, 0.0034484863281250, -0.0022277832031250, -0.0041503906250000,
+-0.0018310546875000, -0.0028991699218750, -0.0018005371093750, -0.0046997070312500,
+-0.0015258789062500, -0.0103454589843750, -0.0015563964843750, -0.0071716308593750,
+-0.0024719238281250, -0.0119018554687500, -0.0011291503906250, -0.0133972167968750,
+-0.0034790039062500, -0.0116882324218750, -0.0057067871093750, -0.0128784179687500,
+-0.0034790039062500, -0.0138244628906250, -0.0109558105468750, -0.0081481933593750,
+-0.0098266601562500, -0.0110168457031250, -0.0118408203125000, -0.0064086914062500,
+-0.0172119140625000, 0.0008850097656250, -0.0144653320312500, -0.0041809082031250,
+-0.0211486816406250, 0.0057983398437500, -0.0202026367187500, 0.0063171386718750,
+-0.0223388671875000, 0.0055236816406250, -0.0275573730468750, 0.0103149414062500,
+-0.0245971679687500, 0.0089111328125000, -0.0299072265625000, 0.0053710937500000,
+-0.0304260253906250, 0.0078125000000000, -0.0296020507812500, 0.0043945312500000,
+-0.0314025878906250, -0.0009765625000000, -0.0311889648437500, 0.0016174316406250,
+-0.0314331054687500, -0.0059204101562500, -0.0302124023437500, -0.0058288574218750,
+-0.0330810546875000, -0.0062866210937500, -0.0354919433593750, -0.0113830566406250,
+-0.0329895019531250, -0.0099487304687500, -0.0403442382812500, -0.0067443847656250,
+-0.0406799316406250, -0.0094909667968750, -0.0391845703125000, -0.0056762695312500,
+-0.0426635742187500, 0.0002136230468750, -0.0426940917968750, -0.0034484863281250,
+-0.0361328125000000, 0.0024414062500000, -0.0385437011718750, 0.0042724609375000,
+-0.0351867675781250, 0.0007934570312500, -0.0285339355468750, 0.0010681152343750,
+-0.0319213867187500, 0.0025024414062500, -0.0234680175781250, -0.0077819824218750,
+-0.0250244140625000, -0.0062866210937500, -0.0200805664062500, -0.0069274902343750,
+-0.0124206542968750, -0.0142822265625000, -0.0178527832031250, -0.0126953125000000,
+-0.0034179687500000, -0.0110778808593750, -0.0021057128906250, -0.0130615234375000,
+-0.0051879882812500, -0.0102539062500000, -0.0003356933593750, -0.0066223144531250,
+0.0005798339843750, -0.0087585449218750, -0.0041198730468750, -0.0036926269531250,
+-0.0055236816406250, -0.0049438476562500, -0.0022277832031250, -0.0022888183593750,
+-0.0021057128906250, 0.0041809082031250, -0.0042114257812500, 0.0011596679687500,
+0.0057067871093750, -0.0004272460937500, 0.0040893554687500, 0.0006408691406250,
+0.0054626464843750, 0.0032348632812500, 0.0135498046875000, 0.0026855468750000,
+0.0107727050781250, 0.0008850097656250, 0.0072021484375000, 0.0103149414062500,
+0.0102233886718750, 0.0088500976562500, 0.0079345703125000, 0.0091857910156250,
+0.0010681152343750, 0.0171203613281250, 0.0036621093750000, 0.0159606933593750,
+0.0102844238281250, 0.0080566406250000, 0.0051879882812500, 0.0092163085937500,
+0.0096130371093750, 0.0111083984375000, 0.0257263183593750, 0.0069580078125000,
+0.0281982421875000, 0.0187072753906250, 0.0118408203125000, 0.0205383300781250,
+0.0166931152343750, 0.0225830078125000, 0.0169067382812500, 0.0209045410156250,
+0.0106811523437500, 0.0182189941406250, 0.0099487304687500, 0.0188598632812500,
+0.0098876953125000, 0.0189514160156250, 0.0133972167968750, 0.0187683105468750,
+0.0069274902343750, 0.0185241699218750, 0.0012207031250000, 0.0199890136718750,
+0.0065002441406250, 0.0190429687500000, -0.0105895996093750, 0.0079956054687500,
+-0.0092163085937500, 0.0150146484375000, -0.0104370117187500, 0.0054321289062500,
+-0.0210266113281250, -0.0120849609375000, -0.0168151855468750, -0.0029907226562500,
+-0.0184326171875000, -0.0157470703125000, -0.0216369628906250, -0.0183410644531250,
+-0.0184326171875000, -0.0178527832031250, -0.0140991210937500, -0.0217895507812500,
+-0.0162048339843750, -0.0198364257812500, -0.0173339843750000, -0.0234985351562500,
+-0.0163879394531250, -0.0243835449218750, -0.0138549804687500, -0.0219726562500000,
+-0.0148010253906250, -0.0221862792968750, -0.0163879394531250, -0.0236816406250000,
+-0.0038452148437500, -0.0182495117187500, -0.0071411132812500, -0.0180053710937500,
+-0.0031738281250000, -0.0185546875000000, 0.0074462890625000, -0.0158691406250000,
+0.0025939941406250, -0.0165405273437500, 0.0139160156250000, -0.0200805664062500,
+0.0144348144531250, -0.0172729492187500, 0.0103454589843750, -0.0220031738281250,
+0.0151367187500000, -0.0289916992187500, 0.0170898437500000, -0.0244140625000000,
+0.0042724609375000, -0.0303955078125000, 0.0060119628906250, -0.0335693359375000,
+0.0043640136718750, -0.0289306640625000, -0.0037536621093750, -0.0270385742187500,
+-0.0002136230468750, -0.0296936035156250, -0.0048522949218750, -0.0220336914062500,
+-0.0065917968750000, -0.0218200683593750, -0.0030212402343750, -0.0215148925781250,
+-0.0034790039062500, -0.0182189941406250, -0.0058288574218750, -0.0193176269531250,
+0.0050354003906250, -0.0153503417968750, 0.0025329589843750, -0.0167846679687500,
+0.0042724609375000, -0.0140380859375000, 0.0128479003906250, -0.0093994140625000,
+0.0090332031250000, -0.0121765136718750, 0.0105590820312500, -0.0065307617187500,
+0.0130615234375000, -0.0060729980468750, 0.0100402832031250, -0.0061950683593750,
+0.0079040527343750, -0.0039978027343750, 0.0097045898437500, -0.0047607421875000,
+0.0019226074218750, -0.0029296875000000, 0.0041809082031250, -0.0031433105468750,
+0.0001831054687500, -0.0029296875000000, -0.0077209472656250, -0.0014953613281250,
+-0.0028076171875000, -0.0020446777343750, -0.0108642578125000, -0.0021362304687500,
+-0.0127868652343750, -0.0019531250000000, -0.0097351074218750, -0.0021057128906250,
+-0.0111999511718750, -0.0027160644531250, -0.0123901367187500, -0.0028686523437500,
+-0.0051269531250000, -0.0025024414062500, -0.0074157714843750, -0.0024414062500000,
+-0.0045471191406250, -0.0032653808593750, 0.0020446777343750, -0.0032348632812500,
+-0.0018615722656250, -0.0026245117187500, 0.0049743652343750, -0.0058898925781250,
+0.0052490234375000, -0.0049743652343750, 0.0050964355468750, -0.0062255859375000,
+0.0084533691406250, -0.0089721679687500, 0.0073242187500000, -0.0073242187500000,
+0.0074157714843750, -0.0112915039062500, 0.0084838867187500, -0.0105590820312500,
+0.0072326660156250, -0.0118408203125000, 0.0062561035156250, -0.0150756835937500,
+0.0074462890625000, -0.0131530761718750, 0.0036315917968750, -0.0165100097656250,
+0.0050659179687500},
+{0.0160522460937500, 0.0253601074218750, 0.0165405273437500, 0.0183105468750000,
+0.0111389160156250, 0.0210876464843750, 0.0103759765625000, 0.0322570800781250,
+0.0153198242187500, 0.0303344726562500, -0.0032653808593750, 0.0224609375000000,
+-0.0020751953125000, 0.0263977050781250, -0.0025329589843750, 0.0216979980468750,
+-0.0137634277343750, 0.0108032226562500, -0.0104370117187500, 0.0156860351562500,
+-0.0071716308593750, 0.0163879394531250, -0.0096740722656250, 0.0132446289062500,
+-0.0088806152343750, 0.0111999511718750, -0.0039672851562500, 0.0156555175781250,
+-0.0050354003906250, 0.0173645019531250, -0.0106201171875000, -0.0028076171875000,
+-0.0068969726562500, 0.0025024414062500, -0.0108337402343750, -0.0048828125000000,
+-0.0211486816406250, -0.0234069824218750, -0.0159912109375000, -0.0146789550781250,
+-0.0078125000000000, -0.0275878906250000, -0.0168762207031250, -0.0305786132812500,
+-0.0058593750000000, -0.0267639160156250, 0.0117492675781250, -0.0294189453125000,
+0.0009155273437500, -0.0302734375000000, 0.0162963867187500, -0.0241394042968750,
+0.0212402343750000, -0.0238952636718750, 0.0140991210937500, -0.0250244140625000,
+0.0157165527343750, -0.0229187011718750, 0.0184631347656250, -0.0225219726562500,
+-0.0010681152343750, -0.0242004394531250, 0.0033874511718750, -0.0246887207031250,
+-0.0000610351562500, -0.0244750976562500, -0.0169982910156250, -0.0238037109375000,
+-0.0111999511718750, -0.0240478515625000, -0.0093078613281250, -0.0290222167968750,
+-0.0145568847656250, -0.0270996093750000, -0.0086364746093750, -0.0287475585937500,
+-0.0018005371093750, -0.0350341796875000, -0.0070495605468750, -0.0329895019531250,
+0.0071411132812500, -0.0289611816406250, 0.0060119628906250, -0.0331726074218750,
+0.0059509277343750, -0.0275573730468750, 0.0149536132812500, -0.0190124511718750,
+0.0122985839843750, -0.0238342285156250, 0.0081176757812500, -0.0141601562500000,
+0.0118713378906250, -0.0136108398437500, 0.0076293945312500, -0.0126953125000000,
+0.0006408691406250, -0.0083312988281250, 0.0049438476562500, -0.0104370117187500,
+-0.0024719238281250, -0.0030822753906250, -0.0029907226562500, -0.0046691894531250,
+-0.0031738281250000, -0.0018615722656250, -0.0065917968750000, 0.0049743652343750,
+-0.0050354003906250, 0.0015869140625000, -0.0070495605468750, 0.0071411132812500,
+-0.0081176757812500, 0.0081176757812500, -0.0057067871093750, 0.0072021484375000,
+-0.0050659179687500, 0.0088500976562500, -0.0068054199218750, 0.0084838867187500,
+-0.0011901855468750, 0.0073242187500000, -0.0011901855468750, 0.0078735351562500,
+-0.0016479492187500, 0.0063171386718750, 0.0011901855468750, 0.0045166015625000,
+0.0007324218750000, 0.0057678222656250, -0.0009765625000000, 0.0025024414062500,
+0.0005798339843750, 0.0023803710937500, -0.0011291503906250, 0.0027160644531250,
+-0.0033874511718750, 0.0016479492187500, -0.0012207031250000, 0.0018310546875000,
+-0.0063171386718750, 0.0007019042968750, -0.0053405761718750, 0.0020141601562500,
+-0.0064392089843750, -0.0004577636718750, -0.0101318359375000, -0.0030212402343750,
+-0.0079345703125000, -0.0007019042968750, -0.0114746093750000, -0.0075378417968750,
+-0.0116882324218750, -0.0069885253906250, -0.0113525390625000, -0.0083618164062500,
+-0.0131835937500000, -0.0126647949218750, -0.0129394531250000, -0.0107421875000000,
+-0.0115661621093750, -0.0162353515625000, -0.0130615234375000, -0.0156555175781250,
+-0.0113830566406250, -0.0173645019531250, -0.0090942382812500, -0.0219421386718750,
+-0.0111694335937500, -0.0200195312500000, -0.0062561035156250, -0.0228576660156250,
+-0.0072937011718750, -0.0240478515625000, -0.0062255859375000, -0.0231323242187500,
+-0.0029602050781250, -0.0232238769531250, -0.0055236816406250, -0.0237731933593750,
+-0.0017395019531250, -0.0229797363281250, -0.0028686523437500, -0.0225219726562500,
+-0.0018005371093750, -0.0232238769531250, 0.0010375976562500, -0.0234069824218750,
+-0.0015563964843750, -0.0227661132812500, 0.0030822753906250, -0.0237731933593750,
+0.0022888183593750, -0.0241394042968750, 0.0029296875000000, -0.0229492187500000,
+0.0061645507812500, -0.0227661132812500, 0.0040893554687500, -0.0234985351562500,
+0.0061950683593750, -0.0200195312500000, 0.0059509277343750, -0.0211791992187500,
+0.0049438476562500, -0.0193481445312500, 0.0054626464843750, -0.0164489746093750,
+0.0048522949218750, -0.0185852050781250, 0.0030517578125000, -0.0112915039062500,
+0.0035095214843750, -0.0132446289062500, 0.0021362304687500, -0.0100097656250000,
+0.0001831054687500, -0.0029602050781250, 0.0010070800781250, -0.0067138671875000,
+-0.0024414062500000, 0.0001831054687500, -0.0018310546875000, 0.0012817382812500,
+-0.0027770996093750, 0.0007324218750000, -0.0056457519531250, 0.0036315917968750,
+-0.0042419433593750, 0.0033264160156250, -0.0056152343750000, 0.0018615722656250,
+-0.0062866210937500, 0.0038452148437500, -0.0054321289062500, 0.0012817382812500,
+-0.0051879882812500, -0.0022583007812500, -0.0054626464843750, 0.0005187988281250,
+-0.0041198730468750, -0.0046997070312500, -0.0039367675781250, -0.0056457519531250,
+-0.0039672851562500, -0.0053405761718750, -0.0031433105468750, -0.0065307617187500,
+-0.0030822753906250, -0.0065917968750000, -0.0042724609375000, -0.0105895996093750,
+-0.0032653808593750, -0.0084838867187500, -0.0047912597656250, -0.0115356445312500,
+-0.0066833496093750, -0.0171508789062500, -0.0050354003906250, -0.0141906738281250,
+-0.0100402832031250, -0.0191650390625000, -0.0095825195312500, -0.0191650390625000,
+-0.0104980468750000, -0.0208740234375000, -0.0142211914062500, -0.0240783691406250,
+-0.0130310058593750, -0.0213928222656250, -0.0141296386718750, -0.0262756347656250,
+-0.0153198242187500, -0.0279235839843750, -0.0133666992187500, -0.0249328613281250,
+-0.0123901367187500, -0.0246887207031250, -0.0136718750000000, -0.0267333984375000,
+-0.0077819824218750, -0.0218200683593750, -0.0088195800781250, -0.0222473144531250,
+-0.0064697265625000, -0.0208435058593750, -0.0016784667968750, -0.0166625976562500,
+-0.0042419433593750, -0.0195007324218750, 0.0016174316406250, -0.0212707519531250,
+0.0014038085937500, -0.0172119140625000, 0.0022277832031250, -0.0208435058593750,
+0.0057983398437500, -0.0282287597656250, 0.0040283203125000, -0.0245666503906250,
+0.0066223144531250, -0.0228271484375000, 0.0063476562500000, -0.0279235839843750,
+0.0071411132812500, -0.0198364257812500, 0.0090332031250000, -0.0114135742187500,
+0.0075378417968750, -0.0182189941406250, 0.0100097656250000, -0.0007629394531250,
+0.0107116699218750, -0.0007019042968750, 0.0091857910156250, 0.0001220703125000,
+0.0094299316406250, 0.0101013183593750, 0.0103454589843750, 0.0063476562500000,
+0.0062255859375000, 0.0055236816406250, 0.0069274902343750, 0.0086975097656250,
+0.0054321289062500, 0.0075988769531250, 0.0024108886718750, 0.0043029785156250,
+0.0042419433593750, 0.0051574707031250, -0.0003356933593750, 0.0033569335937500,
+0.0003662109375000, 0.0042724609375000, -0.0014343261718750, 0.0036315917968750,
+-0.0051269531250000, 0.0020751953125000, -0.0025634765625000, 0.0032958984375000,
+-0.0072631835937500, -0.0007324218750000, -0.0075073242187500, -0.0009460449218750,
+-0.0078430175781250, 0.0008544921875000, -0.0105895996093750, 0.0003967285156250,
+-0.0091247558593750, -0.0006408691406250, -0.0071716308593750, -0.0000305175781250,
+-0.0093383789062500, 0.0013427734375000, -0.0065917968750000, 0.0002746582031250,
+-0.0012817382812500, -0.0014953613281250, -0.0013427734375000, -0.0010070800781250,
+0.0032653808593750, -0.0016174316406250, 0.0021667480468750, -0.0018615722656250,
+0.0016784667968750, -0.0019836425781250, 0.0048217773437500, -0.0015563964843750,
+0.0047302246093750, -0.0008544921875000, 0.0035705566406250, -0.0032653808593750,
+0.0029602050781250, -0.0024414062500000, 0.0058898925781250, -0.0029907226562500,
+0.0067749023437500, -0.0052185058593750, 0.0045166015625000, -0.0036315917968750,
+0.0144958496093750, -0.0027465820312500, 0.0135803222656250, -0.0038146972656250,
+0.0120544433593750, -0.0031738281250000, 0.0181579589843750, -0.0010070800781250,
+0.0178527832031250, -0.0008850097656250, 0.0086669921875000, -0.0014343261718750,
+0.0111694335937500, -0.0011596679687500, 0.0099792480468750, -0.0016784667968750,
+0.0018005371093750, -0.0020141601562500, 0.0033874511718750, -0.0008544921875000,
+0.0060119628906250, -0.0027465820312500, 0.0049133300781250, -0.0036621093750000,
+0.0032348632812500, -0.0014343261718750, 0.0050048828125000, -0.0002746582031250,
+0.0058898925781250, -0.0016174316406250, -0.0019226074218750, 0.0018615722656250,
+0.0001831054687500, 0.0025939941406250, -0.0010070800781250, 0.0009460449218750,
+-0.0071716308593750, 0.0016479492187500, -0.0052185058593750, 0.0027465820312500,
+-0.0088500976562500, -0.0021972656250000, -0.0078735351562500, -0.0023803710937500,
+-0.0096130371093750, -0.0011901855468750, -0.0138244628906250, -0.0032653808593750,
+-0.0118408203125000, -0.0033874511718750, -0.0137939453125000, 0.0014038085937500,
+-0.0149230957031250, -0.0001220703125000, -0.0131530761718750, 0.0017089843750000,
+-0.0126647949218750, 0.0067443847656250, -0.0139160156250000, 0.0045166015625000,
+-0.0098266601562500, 0.0046691894531250, -0.0092773437500000, 0.0069580078125000,
+-0.0105590820312500, 0.0031127929687500, -0.0094909667968750, -0.0003662109375000,
+-0.0086364746093750, 0.0024719238281250, -0.0109863281250000, -0.0063781738281250,
+-0.0106811523437500, -0.0065917968750000, -0.0103759765625000, -0.0065307617187500,
+-0.0115356445312500, -0.0112304687500000, -0.0109558105468750, -0.0103149414062500,
+-0.0093383789062500, -0.0090637207031250, -0.0099792480468750, -0.0099487304687500,
+-0.0087585449218750, -0.0089111328125000, -0.0065002441406250, -0.0070800781250000,
+-0.0071411132812500, -0.0079650878906250, -0.0056457519531250, -0.0062561035156250,
+-0.0054931640625000, -0.0058288574218750, -0.0044250488281250, -0.0070190429687500,
+-0.0032348632812500, -0.0069580078125000, -0.0042114257812500, -0.0066223144531250,
+-0.0014953613281250, -0.0101013183593750, -0.0013427734375000, -0.0095825195312500,
+-0.0015563964843750, -0.0104675292968750, -0.0006103515625000, -0.0133056640625000,
+-0.0007629394531250, -0.0123596191406250, -0.0003051757812500, -0.0136718750000000,
+-0.0008850097656250, -0.0138549804687500, -0.0001525878906250, -0.0136718750000000,
+0.0010986328125000, -0.0140075683593750, -0.0000915527343750, -0.0133666992187500,
+-0.0004882812500000, -0.0125732421875000, 0.0013732910156250, -0.0126953125000000,
+-0.0025024414062500, -0.0107116699218750, -0.0058593750000000, -0.0089416503906250,
+-0.0028686523437500, -0.0094909667968750, -0.0129699707031250, -0.0052795410156250,
+-0.0119934082031250, -0.0050659179687500, -0.0137329101562500, -0.0037231445312500,
+-0.0205078125000000, -0.0006713867187500, -0.0178222656250000, -0.0014648437500000,
+-0.0217590332031250, 0.0003967285156250, -0.0223693847656250, 0.0012512207031250,
+-0.0223999023437500, 0.0011596679687500, -0.0243225097656250, 0.0018310546875000,
+-0.0236206054687500, 0.0022277832031250, -0.0238647460937500, 0.0011901855468750,
+-0.0247192382812500, 0.0020141601562500, -0.0231323242187500, 0.0016479492187500,
+-0.0219726562500000, 0.0006408691406250, -0.0231628417968750, 0.0018615722656250,
+-0.0190734863281250, 0.0004577636718750, -0.0194091796875000, 0.0008239746093750,
+-0.0189208984375000, 0.0007324218750000, -0.0162963867187500, -0.0001220703125000,
+-0.0171813964843750, 0.0005798339843750, -0.0160522460937500, -0.0006408691406250,
+-0.0159912109375000, -0.0002136230468750, -0.0158691406250000, -0.0003967285156250,
+-0.0155334472656250},
+{0.0081176757812500, -0.0011596679687500, 0.0014953613281250, -0.0016479492187500,
+0.0081481933593750, -0.0014343261718750, -0.0032043457031250, 0.0000915527343750,
+-0.0068664550781250, 0.0005187988281250, -0.0029296875000000, -0.0009765625000000,
+-0.0028381347656250, -0.0011596679687500, -0.0047607421875000, -0.0000610351562500,
+-0.0031433105468750, -0.0039062500000000, -0.0004272460937500, -0.0038757324218750,
+-0.0051269531250000, -0.0029296875000000, -0.0090942382812500, -0.0050048828125000,
+-0.0048217773437500, -0.0051879882812500, -0.0115661621093750, -0.0003662109375000,
+-0.0135498046875000, -0.0016174316406250, -0.0118408203125000, -0.0002441406250000,
+-0.0124816894531250, 0.0041503906250000, -0.0127258300781250, 0.0021057128906250,
+-0.0111999511718750, 0.0024414062500000, -0.0114135742187500, 0.0043945312500000,
+-0.0119628906250000, 0.0008850097656250, -0.0113830566406250, -0.0026550292968750,
+-0.0106811523437500, -0.0003967285156250, -0.0110778808593750, -0.0052185058593750,
+-0.0123291015625000, -0.0062255859375000, -0.0097351074218750, -0.0053710937500000,
+-0.0073547363281250, -0.0067749023437500, -0.0092468261718750, -0.0071716308593750,
+-0.0048522949218750, -0.0051574707031250, -0.0045166015625000, -0.0052185058593750,
+-0.0041809082031250, -0.0057067871093750, -0.0017089843750000, -0.0045471191406250,
+-0.0021362304687500, -0.0045776367187500, -0.0019836425781250, -0.0079650878906250,
+-0.0010681152343750, -0.0073242187500000, -0.0010375976562500, -0.0087585449218750,
+-0.0014038085937500, -0.0118103027343750, -0.0012512207031250, -0.0105590820312500,
+-0.0016479492187500, -0.0130615234375000, -0.0009460449218750, -0.0136108398437500,
+-0.0023193359375000, -0.0129394531250000, -0.0038452148437500, -0.0139160156250000,
+-0.0025329589843750, -0.0140380859375000, -0.0047607421875000, -0.0107116699218750,
+-0.0052795410156250, -0.0116577148437500, -0.0046081542968750, -0.0097045898437500,
+-0.0049743652343750, -0.0059204101562500, -0.0053405761718750, -0.0072021484375000,
+-0.0047912597656250, -0.0036926269531250, -0.0046386718750000, -0.0028991699218750,
+-0.0051879882812500, -0.0026245117187500, -0.0054321289062500, -0.0009765625000000,
+-0.0053710937500000, -0.0009460449218750, -0.0063476562500000, -0.0006408691406250,
+-0.0062866210937500, 0.0000305175781250, -0.0074768066406250, -0.0008544921875000,
+-0.0083618164062500, -0.0014038085937500, -0.0080261230468750, -0.0004882812500000,
+-0.0134277343750000, -0.0032958984375000, -0.0117187500000000, -0.0028686523437500,
+-0.0151977539062500, -0.0034790039062500, -0.0212097167968750, -0.0056457519531250,
+-0.0176391601562500, -0.0047607421875000, -0.0246276855468750, -0.0065002441406250,
+-0.0254821777343750, -0.0069274902343750, -0.0239257812500000, -0.0067443847656250,
+-0.0262145996093750, -0.0072937011718750, -0.0263061523437500, -0.0074157714843750,
+-0.0227050781250000, -0.0071105957031250, -0.0233459472656250, -0.0078735351562500,
+-0.0225219726562500, -0.0079040527343750, -0.0196228027343750, -0.0074462890625000,
+-0.0207214355468750, -0.0081787109375000, -0.0199279785156250, -0.0083007812500000,
+-0.0191345214843750, -0.0083618164062500, -0.0199584960937500, -0.0085754394531250,
+-0.0206298828125000, -0.0089111328125000, -0.0199890136718750, -0.0089721679687500,
+-0.0209045410156250, -0.0082092285156250, -0.0211791992187500, -0.0089111328125000,
+-0.0208435058593750, -0.0082092285156250, -0.0211791992187500, -0.0071716308593750,
+-0.0212707519531250, -0.0081787109375000, -0.0195312500000000, -0.0061645507812500,
+-0.0205078125000000, -0.0065307617187500, -0.0187988281250000, -0.0059204101562500,
+-0.0162658691406250, -0.0045471191406250, -0.0180358886718750, -0.0056152343750000,
+-0.0142211914062500, -0.0027160644531250, -0.0140075683593750, -0.0032958984375000,
+-0.0144042968750000, -0.0031738281250000, -0.0130004882812500, -0.0010681152343750,
+-0.0134887695312500, -0.0020141601562500, -0.0130615234375000, -0.0020751953125000,
+-0.0134887695312500, -0.0011596679687500, -0.0131225585937500, -0.0024108886718750,
+-0.0126037597656250, -0.0031127929687500, -0.0132141113281250, -0.0016784667968750,
+-0.0111389160156250, -0.0069885253906250, -0.0118408203125000, -0.0051879882812500,
+-0.0101928710937500, -0.0077819824218750, -0.0079345703125000, -0.0130615234375000,
+-0.0094299316406250, -0.0101013183593750, -0.0038757324218750, -0.0168457031250000,
+-0.0039062500000000, -0.0163574218750000, -0.0036010742187500, -0.0157470703125000,
+0.0000000000000000, -0.0198059082031250, -0.0007324218750000, -0.0186767578125000,
+-0.0024414062500000, -0.0156250000000000, -0.0007629394531250, -0.0177001953125000,
+-0.0029296875000000, -0.0149230957031250, -0.0062255859375000, -0.0106506347656250,
+-0.0042419433593750, -0.0135803222656250, -0.0082702636718750, -0.0075988769531250,
+-0.0084838867187500, -0.0072021484375000, -0.0090026855468750, -0.0075988769531250,
+-0.0111999511718750, -0.0051879882812500, -0.0102539062500000, -0.0057373046875000,
+-0.0123901367187500, -0.0048217773437500, -0.0125427246093750, -0.0048217773437500,
+-0.0119628906250000, -0.0056152343750000, -0.0131225585937500, -0.0054931640625000,
+-0.0131835937500000, -0.0050048828125000, -0.0099182128906250, -0.0067749023437500,
+-0.0112915039062500, -0.0077819824218750, -0.0096740722656250, -0.0053710937500000,
+-0.0056457519531250, -0.0047607421875000, -0.0076293945312500, -0.0064392089843750,
+-0.0072326660156250, 0.0024414062500000, -0.0054321289062500, 0.0018310546875000,
+-0.0076904296875000, 0.0014648437500000, -0.0099487304687500, 0.0065612792968750,
+-0.0085449218750000, 0.0052795410156250, -0.0137939453125000, 0.0014343261718750,
+-0.0129089355468750, 0.0036926269531250, -0.0149536132812500, 0.0009765625000000,
+-0.0200195312500000, -0.0043334960937500, -0.0177917480468750, -0.0024719238281250,
+-0.0201721191406250, -0.0077514648437500, -0.0218811035156250, -0.0063171386718750,
+-0.0193786621093750, -0.0090942382812500, -0.0189514160156250, -0.0156555175781250,
+-0.0204162597656250, -0.0127563476562500, -0.0099182128906250, -0.0139465332031250,
+-0.0132446289062500, -0.0172729492187500, -0.0087280273437500, -0.0115661621093750,
+0.0017089843750000, -0.0064697265625000, -0.0035095214843750, -0.0116882324218750,
+0.0054931640625000, -0.0038146972656250, 0.0069274902343750, -0.0011596679687500,
+0.0053100585937500, -0.0025024414062500, 0.0075683593750000, -0.0025939941406250,
+0.0078125000000000, -0.0032653808593750, 0.0093383789062500, 0.0002746582031250,
+0.0071411132812500, -0.0019226074218750, 0.0099792480468750, 0.0033264160156250,
+0.0151062011718750, 0.0096130371093750, 0.0121765136718750, 0.0043029785156250,
+0.0115051269531250, 0.0133056640625000, 0.0142517089843750, 0.0152282714843750,
+0.0121154785156250, 0.0136718750000000, 0.0059204101562500, 0.0167846679687500,
+-0.0104980468750000, 0.0014343261718750, -0.0126953125000000, 0.0128784179687500,
+-0.0158081054687500, 0.0109558105468750, -0.0108032226562500, 0.0099182128906250,
+-0.0059814453125000, 0.0124511718750000, -0.0095214843750000, 0.0132446289062500,
+-0.0047912597656250, 0.0127868652343750, -0.0037231445312500, 0.0112915039062500,
+-0.0035095214843750, 0.0140686035156250, -0.0021972656250000, 0.0167236328125000,
+-0.0027465820312500, 0.0140991210937500, 0.0015869140625000, 0.0185546875000000,
+-0.0003662109375000, 0.0221557617187500, 0.0031127929687500, 0.0148620605468750,
+0.0083007812500000, 0.0099792480468750, 0.0048522949218750, 0.0157165527343750,
+0.0144653320312500, 0.0045776367187500, 0.0150146484375000, 0.0030517578125000,
+0.0112915039062500, 0.0027770996093750, 0.0150451660156250, -0.0008850097656250,
+0.0170288085937500, 0.0008239746093750, 0.0051574707031250, -0.0057373046875000,
+0.0077209472656250, -0.0026550292968750, 0.0041198730468750, -0.0101928710937500,
+-0.0061035156250000, -0.0189514160156250, -0.0010986328125000, -0.0117797851562500,
+-0.0053405761718750, -0.0332641601562500, -0.0090942382812500, -0.0336608886718750,
+-0.0047302246093750, -0.0316162109375000, -0.0019836425781250, -0.0426025390625000,
+-0.0054016113281250, -0.0413818359375000, 0.0017700195312500, -0.0312500000000000,
+0.0032653808593750, -0.0342407226562500, 0.0010375976562500, -0.0321350097656250,
+0.0015258789062500, -0.0235595703125000, 0.0023803710937500, -0.0262756347656250,
+0.0030517578125000, -0.0242919921875000, 0.0025329589843750, -0.0242309570312500,
+0.0005187988281250, -0.0228271484375000, 0.0011596679687500, -0.0210876464843750,
+0.0027465820312500, -0.0228271484375000, -0.0054321289062500, -0.0180053710937500,
+-0.0039367675781250, -0.0180053710937500, -0.0055236816406250, -0.0184326171875000,
+-0.0109558105468750, -0.0162353515625000, -0.0082702636718750, -0.0165405273437500,
+-0.0150451660156250, -0.0169982910156250, -0.0136413574218750, -0.0176391601562500,
+-0.0148010253906250, -0.0156860351562500, -0.0203857421875000, -0.0150146484375000,
+-0.0177001953125000, -0.0166320800781250, -0.0179138183593750, -0.0087890625000000,
+-0.0203857421875000, -0.0101318359375000, -0.0165710449218750, -0.0083312988281250,
+-0.0131835937500000, -0.0021362304687500, -0.0161437988281250, -0.0045471191406250,
+-0.0075683593750000, -0.0011596679687500, -0.0085754394531250, -0.0007934570312500,
+-0.0060424804687500, -0.0001220703125000, -0.0000305175781250, 0.0016479492187500,
+-0.0035705566406250, 0.0005798339843750, 0.0044250488281250, 0.0035705566406250,
+0.0047302246093750, 0.0036010742187500, 0.0039978027343750, 0.0033874511718750,
+0.0080261230468750, 0.0049743652343750, 0.0073852539062500, 0.0047912597656250,
+0.0046081542968750, 0.0041503906250000, 0.0067138671875000, 0.0045471191406250,
+0.0037841796875000, 0.0043334960937500, 0.0000610351562500, 0.0038452148437500,
+0.0032043457031250, 0.0040283203125000, -0.0051879882812500, 0.0026245117187500,
+-0.0032653808593750, 0.0033874511718750, -0.0062255859375000, 0.0018005371093750,
+-0.0127563476562500, -0.0003662109375000, -0.0081481933593750, 0.0010070800781250,
+-0.0168151855468750, -0.0029296875000000, -0.0157775878906250, -0.0028686523437500,
+-0.0165405273437500, -0.0038146972656250, -0.0215148925781250, -0.0062866210937500,
+-0.0184326171875000, -0.0052185058593750, -0.0238037109375000, -0.0090637207031250,
+-0.0226135253906250, -0.0085449218750000, -0.0234069824218750, -0.0101623535156250,
+-0.0272827148437500, -0.0133361816406250, -0.0248107910156250, -0.0115966796875000,
+-0.0274963378906250, -0.0161743164062500, -0.0275268554687500, -0.0156860351562500,
+-0.0263977050781250, -0.0165710449218750, -0.0268554687500000, -0.0197448730468750,
+-0.0262451171875000, -0.0183105468750000, -0.0243835449218750, -0.0204467773437500,
+-0.0243225097656250, -0.0209960937500000, -0.0229492187500000, -0.0207214355468750,
+-0.0210571289062500, -0.0213317871093750, -0.0215148925781250, -0.0214843750000000,
+-0.0188598632812500, -0.0215759277343750, -0.0187377929687500, -0.0220947265625000,
+-0.0176391601562500, -0.0214538574218750, -0.0155639648437500, -0.0213317871093750,
+-0.0159606933593750, -0.0222473144531250, -0.0128173828125000, -0.0184631347656250,
+-0.0128173828125000, -0.0197448730468750, -0.0111694335937500, -0.0180358886718750,
+-0.0084533691406250, -0.0142822265625000, -0.0093383789062500, -0.0162353515625000,
+-0.0060424804687500, -0.0126647949218750, -0.0060729980468750, -0.0119323730468750,
+-0.0046386718750000, -0.0122680664062500, -0.0021057128906250, -0.0105895996093750,
+-0.0028991699218750, -0.0102844238281250, 0.0014648437500000, -0.0122985839843750,
+0.0014343261718750, -0.0114746093750000, 0.0028381347656250, -0.0118408203125000,
+0.0061645507812500, -0.0145263671875000, 0.0054626464843750, -0.0139770507812500,
+0.0086975097656250},
+{0.0054016113281250, -0.0322265625000000, -0.0002136230468750, -0.0231933593750000,
+0.0058288574218750, -0.0230407714843750, -0.0017089843750000, -0.0226745605468750,
+-0.0133666992187500, -0.0184326171875000, -0.0060424804687500, -0.0199584960937500,
+-0.0189819335937500, -0.0172729492187500, -0.0202026367187500, -0.0174865722656250,
+-0.0185852050781250, -0.0170898437500000, -0.0241394042968750, -0.0152587890625000,
+-0.0227050781250000, -0.0161132812500000, -0.0169067382812500, -0.0145263671875000,
+-0.0210876464843750, -0.0142211914062500, -0.0153198242187500, -0.0150451660156250,
+-0.0063476562500000, -0.0149230957031250, -0.0123596191406250, -0.0145874023437500,
+-0.0014038085937500, -0.0161743164062500, -0.0002136230468750, -0.0169372558593750,
+-0.0021057128906250, -0.0151367187500000, 0.0020751953125000, -0.0147705078125000,
+0.0019531250000000, -0.0159912109375000, -0.0021667480468750, -0.0099792480468750,
+0.0000610351562500, -0.0107421875000000, -0.0036315917968750, -0.0095214843750000,
+-0.0080871582031250, -0.0051269531250000, -0.0042114257812500, -0.0067138671875000,
+-0.0169372558593750, -0.0043945312500000, -0.0144042968750000, -0.0042419433593750,
+-0.0176086425781250, -0.0037841796875000, -0.0271911621093750, -0.0022888183593750,
+-0.0216064453125000, -0.0029907226562500, -0.0298461914062500, -0.0017395019531250,
+-0.0294189453125000, -0.0012817382812500, -0.0296325683593750, -0.0018920898437500,
+-0.0341491699218750, -0.0020141601562500, -0.0319213867187500, -0.0018005371093750,
+-0.0349731445312500, -0.0029907226562500, -0.0350952148437500, -0.0029602050781250,
+-0.0346069335937500, -0.0034484863281250, -0.0359497070312500, -0.0042419433593750,
+-0.0355224609375000, -0.0038452148437500, -0.0351562500000000, -0.0049743652343750,
+-0.0352783203125000, -0.0052795410156250, -0.0346984863281250, -0.0048828125000000,
+-0.0341491699218750, -0.0052185058593750, -0.0344238281250000, -0.0054321289062500,
+-0.0330810546875000, -0.0043945312500000, -0.0336608886718750, -0.0041198730468750,
+-0.0326843261718750, -0.0050048828125000, -0.0312194824218750, -0.0048828125000000,
+-0.0325012207031250, -0.0042114257812500, -0.0303039550781250, -0.0079345703125000,
+-0.0301818847656250, -0.0070190429687500, -0.0299377441406250, -0.0086975097656250,
+-0.0288696289062500, -0.0120544433593750, -0.0294799804687500, -0.0106201171875000,
+-0.0284118652343750, -0.0152282714843750, -0.0284423828125000, -0.0151672363281250,
+-0.0282592773437500, -0.0159301757812500, -0.0274353027343750, -0.0192565917968750,
+-0.0275573730468750, -0.0181884765625000, -0.0265197753906250, -0.0182189941406250,
+-0.0267028808593750, -0.0196533203125000, -0.0260925292968750, -0.0175476074218750,
+-0.0251159667968750, -0.0151977539062500, -0.0254211425781250, -0.0169067382812500,
+-0.0236206054687500, -0.0131225585937500, -0.0238647460937500, -0.0123291015625000,
+-0.0231933593750000, -0.0132446289062500, -0.0216674804687500, -0.0118408203125000,
+-0.0219116210937500, -0.0113525390625000, -0.0190124511718750, -0.0147399902343750,
+-0.0190734863281250, -0.0131835937500000, -0.0181579589843750, -0.0149841308593750,
+-0.0158081054687500, -0.0190124511718750, -0.0162353515625000, -0.0167236328125000,
+-0.0146179199218750, -0.0193481445312500, -0.0139465332031250, -0.0214843750000000,
+-0.0140380859375000, -0.0187988281250000, -0.0133056640625000, -0.0174560546875000,
+-0.0124511718750000, -0.0198669433593750, -0.0126342773437500, -0.0145568847656250,
+-0.0119934082031250, -0.0146179199218750, -0.0110778808593750, -0.0143737792968750,
+-0.0109558105468750, -0.0116271972656250, -0.0103454589843750, -0.0127258300781250,
+-0.0082397460937500, -0.0109558105468750, -0.0086364746093750, -0.0101623535156250,
+-0.0060424804687500, -0.0117187500000000, -0.0030212402343750, -0.0117492675781250,
+-0.0042114257812500, -0.0100097656250000, 0.0011596679687500, -0.0139465332031250,
+0.0007629394531250, -0.0149841308593750, 0.0023193359375000, -0.0114746093750000,
+0.0061950683593750, -0.0116577148437500, 0.0043640136718750, -0.0143127441406250,
+0.0076599121093750, -0.0042114257812500, 0.0084533691406250, -0.0047607421875000,
+0.0073852539062500, -0.0056762695312500, 0.0081481933593750, -0.0007324218750000,
+0.0082702636718750, -0.0012817382812500, 0.0048217773437500, -0.0017395019531250,
+0.0061035156250000, -0.0017395019531250, 0.0040283203125000, -0.0025024414062500,
+-0.0003662109375000, -0.0019531250000000, 0.0018005371093750, -0.0011596679687500,
+-0.0001220703125000, -0.0066528320312500, -0.0018615722656250, -0.0070800781250000,
+-0.0003051757812500, -0.0029602050781250, 0.0009460449218750, -0.0040283203125000,
+-0.0000305175781250, -0.0075988769531250, 0.0021972656250000, 0.0021972656250000,
+0.0029907226562500, 0.0024108886718750, 0.0020751953125000, 0.0010375976562500,
+0.0025024414062500, 0.0052490234375000, 0.0030212402343750, 0.0044860839843750,
+-0.0001831054687500, 0.0003662109375000, 0.0010986328125000, 0.0028686523437500,
+-0.0011596679687500, -0.0018615722656250, -0.0051269531250000, -0.0078430175781250,
+-0.0025939941406250, -0.0028686523437500, -0.0062255859375000, -0.0126342773437500,
+-0.0087280273437500, -0.0153503417968750, -0.0035095214843750, -0.0103759765625000,
+-0.0007324218750000, -0.0098876953125000, -0.0047302246093750, -0.0127868652343750,
+0.0045776367187500, -0.0088195800781250, 0.0057373046875000, -0.0068969726562500,
+0.0048217773437500, -0.0095520019531250, 0.0081787109375000, -0.0102233886718750,
+0.0072937011718750, -0.0082092285156250, 0.0047607421875000, -0.0135192871093750,
+0.0061645507812500, -0.0151977539062500, 0.0044860839843750, -0.0115051269531250,
+0.0008239746093750, -0.0114135742187500, 0.0024719238281250, -0.0135803222656250,
+0.0021362304687500, -0.0033874511718750, 0.0005187988281250, -0.0052185058593750,
+0.0021362304687500, -0.0039367675781250, 0.0045776367187500, 0.0039672851562500,
+0.0034790039062500, 0.0016174316406250, 0.0032653808593750, 0.0016784667968750,
+0.0043334960937500, 0.0036315917968750, 0.0036926269531250, 0.0009460449218750,
+0.0020446777343750, -0.0030822753906250, 0.0022277832031250, -0.0023803710937500,
+0.0024414062500000, -0.0038452148437500, 0.0016479492187500, -0.0044860839843750,
+0.0022888183593750, -0.0036926269531250, 0.0036010742187500, -0.0039367675781250,
+0.0025939941406250, -0.0043640136718750, 0.0020751953125000, -0.0007934570312500,
+0.0031433105468750, -0.0021057128906250, 0.0017089843750000, -0.0001220703125000,
+0.0000915527343750, 0.0039062500000000, 0.0011596679687500, 0.0017700195312500,
+-0.0032043457031250, 0.0053100585937500, -0.0023803710937500, 0.0067749023437500,
+-0.0043945312500000, 0.0044250488281250, -0.0083923339843750, 0.0034179687500000,
+-0.0056762695312500, 0.0046691894531250, -0.0079650878906250, 0.0024414062500000,
+-0.0106201171875000, 0.0017395019531250, -0.0073242187500000, 0.0021667480468750,
+-0.0040893554687500, 0.0015258789062500, -0.0066528320312500, 0.0014343261718750,
+-0.0042724609375000, 0.0035400390625000, -0.0019531250000000, 0.0032043457031250,
+-0.0050048828125000, 0.0022277832031250, -0.0075683593750000, 0.0033569335937500,
+-0.0050354003906250, 0.0036315917968750, -0.0072631835937500, -0.0009155273437500,
+-0.0087890625000000, 0.0001220703125000, -0.0084533691406250, -0.0012512207031250,
+-0.0082397460937500, -0.0055541992187500, -0.0083007812500000, -0.0041503906250000,
+-0.0078430175781250, -0.0052490234375000, -0.0072021484375000, -0.0060119628906250,
+-0.0090332031250000, -0.0057983398437500, -0.0103149414062500, -0.0059814453125000,
+-0.0082702636718750, -0.0062561035156250, -0.0111083984375000, -0.0055541992187500,
+-0.0117492675781250, -0.0053405761718750, -0.0101928710937500, -0.0068969726562500,
+-0.0108947753906250, -0.0075073242187500, -0.0112609863281250, -0.0065612792968750,
+-0.0046081542968750, -0.0100097656250000, -0.0064697265625000, -0.0106811523437500,
+-0.0054016113281250, -0.0094909667968750, 0.0003356933593750, -0.0104064941406250,
+-0.0013122558593750, -0.0110473632812500, -0.0009765625000000, -0.0064697265625000,
+0.0000610351562500, -0.0076293945312500, -0.0008544921875000, -0.0057067871093750,
+-0.0018615722656250, -0.0010681152343750, -0.0009155273437500, -0.0028991699218750,
+-0.0014953613281250, -0.0003662109375000, -0.0025024414062500, 0.0006103515625000,
+-0.0014343261718750, -0.0007934570312500, -0.0002136230468750, -0.0012512207031250,
+-0.0014343261718750, -0.0009155273437500, -0.0014953613281250, -0.0032348632812500,
+-0.0001831054687500, -0.0032958984375000, -0.0019531250000000, -0.0045471191406250,
+-0.0039367675781250, -0.0063476562500000, -0.0026245117187500, -0.0054931640625000,
+-0.0065917968750000, -0.0079956054687500, -0.0058898925781250, -0.0082092285156250,
+-0.0078735351562500, -0.0086975097656250, -0.0114440917968750, -0.0104370117187500,
+-0.0093994140625000, -0.0105285644531250, -0.0150451660156250, -0.0118408203125000,
+-0.0149230957031250, -0.0121459960937500, -0.0158691406250000, -0.0133666992187500,
+-0.0195312500000000, -0.0147094726562500, -0.0182495117187500, -0.0147094726562500,
+-0.0202941894531250, -0.0178527832031250, -0.0212402343750000, -0.0176086425781250,
+-0.0202941894531250, -0.0198974609375000, -0.0204772949218750, -0.0231323242187500,
+-0.0211486816406250, -0.0220031738281250, -0.0179748535156250, -0.0269165039062500,
+-0.0188293457031250, -0.0267944335937500, -0.0178222656250000, -0.0285339355468750,
+-0.0149230957031250, -0.0321044921875000, -0.0163574218750000, -0.0304870605468750,
+-0.0149841308593750, -0.0343933105468750, -0.0142517089843750, -0.0342712402343750,
+-0.0153503417968750, -0.0351562500000000, -0.0156860351562500, -0.0374450683593750,
+-0.0150146484375000, -0.0358581542968750, -0.0173034667968750, -0.0384826660156250,
+-0.0167846679687500, -0.0378417968750000, -0.0173950195312500, -0.0378112792968750,
+-0.0192871093750000, -0.0388488769531250, -0.0181579589843750, -0.0368652343750000,
+-0.0189514160156250, -0.0382995605468750, -0.0195312500000000, -0.0369262695312500,
+-0.0183410644531250, -0.0369262695312500, -0.0176391601562500, -0.0378112792968750,
+-0.0182495117187500, -0.0359191894531250, -0.0161743164062500, -0.0390014648437500,
+-0.0157470703125000, -0.0372924804687500, -0.0164794921875000, -0.0371398925781250,
+-0.0158386230468750, -0.0390319824218750, -0.0153198242187500, -0.0365295410156250,
+-0.0179443359375000, -0.0379943847656250, -0.0173645019531250, -0.0383300781250000,
+-0.0178527832031250, -0.0366210937500000, -0.0200195312500000, -0.0365600585937500,
+-0.0191955566406250, -0.0375366210937500, -0.0195922851562500, -0.0344543457031250,
+-0.0199890136718750, -0.0347290039062500, -0.0192260742187500, -0.0341796875000000,
+-0.0188903808593750, -0.0318908691406250, -0.0191650390625000, -0.0324707031250000,
+-0.0169067382812500, -0.0318908691406250, -0.0176086425781250, -0.0314941406250000,
+-0.0161743164062500, -0.0320739746093750, -0.0136108398437500, -0.0325622558593750,
+-0.0152893066406250, -0.0324096679687500, -0.0131225585937500, -0.0332336425781250,
+-0.0124206542968750, -0.0336914062500000, -0.0137023925781250, -0.0323791503906250,
+-0.0141906738281250, -0.0321655273437500, -0.0139465332031250, -0.0332641601562500,
+-0.0149841308593750, -0.0286254882812500, -0.0160522460937500, -0.0309143066406250,
+-0.0149536132812500, -0.0269470214843750, -0.0142517089843750, -0.0217895507812500,
+-0.0154418945312500, -0.0265502929687500, -0.0123596191406250, -0.0144653320312500,
+-0.0128173828125000, -0.0164184570312500, -0.0120544433593750, -0.0135192871093750,
+-0.0101928710937500, -0.0044860839843750, -0.0114746093750000, -0.0091552734375000,
+-0.0074157714843750, -0.0009460449218750, -0.0086669921875000, 0.0001831054687500,
+-0.0064392089843750},
+{-0.0028686523437500, -0.0099792480468750, -0.0033569335937500, -0.0104370117187500,
+-0.0047607421875000, -0.0111083984375000, -0.0043945312500000, -0.0118713378906250,
+-0.0043945312500000, -0.0127563476562500, -0.0045471191406250, -0.0126037597656250,
+-0.0057067871093750, -0.0139160156250000, -0.0061950683593750, -0.0151977539062500,
+-0.0054626464843750, -0.0146789550781250, -0.0093383789062500, -0.0175170898437500,
+-0.0086975097656250, -0.0173339843750000, -0.0101623535156250, -0.0188903808593750,
+-0.0131835937500000, -0.0209655761718750, -0.0118408203125000, -0.0193176269531250,
+-0.0165405273437500, -0.0235900878906250, -0.0161132812500000, -0.0225524902343750,
+-0.0172424316406250, -0.0231018066406250, -0.0208435058593750, -0.0259399414062500,
+-0.0193481445312500, -0.0238037109375000, -0.0212097167968750, -0.0256347656250000,
+-0.0220947265625000, -0.0256347656250000, -0.0211791992187500, -0.0238342285156250,
+-0.0212097167968750, -0.0232849121093750, -0.0215759277343750, -0.0225524902343750,
+-0.0190734863281250, -0.0195007324218750, -0.0199890136718750, -0.0181884765625000,
+-0.0182800292968750, -0.0160827636718750, -0.0156250000000000, -0.0132446289062500,
+-0.0173950195312500, -0.0123901367187500, -0.0131225585937500, -0.0103149414062500,
+-0.0131530761718750, -0.0077514648437500, -0.0130615234375000, -0.0060119628906250,
+-0.0106201171875000, -0.0045776367187500, -0.0112609863281250, -0.0026245117187500,
+-0.0111999511718750, -0.0007934570312500, -0.0105285644531250, 0.0015563964843750,
+-0.0109558105468750, 0.0022888183593750, -0.0115661621093750, 0.0030517578125000,
+-0.0110778808593750, 0.0054321289062500, -0.0118103027343750, 0.0034790039062500,
+-0.0115661621093750, 0.0057983398437500, -0.0117797851562500, 0.0058898925781250,
+-0.0122070312500000, 0.0044555664062500, -0.0116577148437500, 0.0074462890625000,
+-0.0131225585937500, 0.0045166015625000, -0.0126647949218750, 0.0072631835937500,
+-0.0130920410156250, 0.0064697265625000, -0.0142211914062500, 0.0039062500000000,
+-0.0134277343750000, 0.0077514648437500, -0.0147094726562500, 0.0016174316406250,
+-0.0144042968750000, 0.0038757324218750, -0.0147705078125000, 0.0014953613281250,
+-0.0157470703125000, -0.0035095214843750, -0.0148925781250000, 0.0009460449218750,
+-0.0160522460937500, -0.0084838867187500, -0.0161743164062500, -0.0065002441406250,
+-0.0155029296875000, -0.0083923339843750, -0.0155029296875000, -0.0148620605468750,
+-0.0158691406250000, -0.0104064941406250, -0.0151977539062500, -0.0182800292968750,
+-0.0147705078125000, -0.0168762207031250, -0.0161743164062500, -0.0173339843750000,
+-0.0170288085937500, -0.0223083496093750, -0.0162658691406250, -0.0197448730468750,
+-0.0195617675781250, -0.0237426757812500, -0.0198059082031250, -0.0228576660156250,
+-0.0196533203125000, -0.0245666503906250, -0.0212707519531250, -0.0280761718750000,
+-0.0214538574218750, -0.0254821777343750, -0.0205688476562500, -0.0303344726562500,
+-0.0212402343750000, -0.0292358398437500, -0.0209655761718750, -0.0304565429687500,
+-0.0205078125000000, -0.0342712402343750, -0.0210876464843750, -0.0318908691406250,
+-0.0180358886718750, -0.0354309082031250, -0.0198974609375000, -0.0364379882812500,
+-0.0167541503906250, -0.0350036621093750, -0.0125122070312500, -0.0360107421875000,
+-0.0157775878906250, -0.0363769531250000, -0.0074768066406250, -0.0304260253906250,
+-0.0080261230468750, -0.0335693359375000, -0.0065917968750000, -0.0278930664062500,
+-0.0010986328125000, -0.0198059082031250, -0.0035095214843750, -0.0249023437500000,
+-0.0001220703125000, -0.0101623535156250, 0.0008850097656250, -0.0112609863281250,
+-0.0003967285156250, -0.0088500976562500, 0.0002746582031250, 0.0016174316406250,
+0.0006713867187500, -0.0024414062500000, -0.0026855468750000, 0.0003356933593750,
+-0.0013732910156250, 0.0030822753906250, -0.0040588378906250, 0.0003662109375000,
+-0.0081481933593750, -0.0018005371093750, -0.0054016113281250, 0.0000000000000000,
+-0.0117187500000000, -0.0032348632812500, -0.0133666992187500, -0.0034790039062500,
+-0.0098571777343750, -0.0037841796875000, -0.0101623535156250, -0.0053405761718750,
+-0.0123291015625000, -0.0041809082031250, -0.0050659179687500, -0.0048522949218750,
+-0.0041198730468750, -0.0057067871093750, -0.0056457519531250, -0.0040283203125000,
+-0.0034179687500000, -0.0032348632812500, -0.0028381347656250, -0.0043334960937500,
+-0.0058898925781250, 0.0014038085937500, -0.0055236816406250, 0.0013732910156250,
+-0.0057067871093750, -0.0004577636718750, -0.0075378417968750, 0.0013427734375000,
+-0.0068664550781250, 0.0021667480468750, -0.0076904296875000, 0.0000915527343750,
+-0.0073547363281250, 0.0002441406250000, -0.0086059570312500, -0.0017395019531250,
+-0.0098266601562500, -0.0039367675781250, -0.0085449218750000, -0.0018920898437500,
+-0.0124206542968750, -0.0046081542968750, -0.0119934082031250, -0.0053100585937500,
+-0.0133361816406250, -0.0057067871093750, -0.0160827636718750, -0.0066223144531250,
+-0.0143127441406250, -0.0061340332031250, -0.0182495117187500, -0.0089111328125000,
+-0.0182800292968750, -0.0072631835937500, -0.0187072753906250, -0.0100097656250000,
+-0.0206604003906250, -0.0152282714843750, -0.0194091796875000, -0.0125122070312500,
+-0.0218200683593750, -0.0124816894531250, -0.0227966308593750, -0.0156860351562500,
+-0.0204162597656250, -0.0114440917968750, -0.0180358886718750, -0.0026550292968750,
+-0.0119323730468750, -0.0128173828125000, -0.0070190429687500, -0.0141296386718750,
+-0.0066833496093750, -0.0184326171875000, -0.0093078613281250, -0.0147094726562500,
+-0.0097961425781250, -0.0091857910156250, -0.0074157714843750, -0.0115051269531250,
+-0.0120849609375000, -0.0098571777343750, -0.0144958496093750, -0.0083618164062500,
+-0.0095825195312500, -0.0102844238281250, -0.0083007812500000, -0.0128479003906250,
+-0.0118713378906250, -0.0109252929687500, 0.0020751953125000, -0.0050354003906250,
+0.0013122558593750, -0.0095214843750000, 0.0006713867187500, -0.0054016113281250,
+0.0086364746093750, 0.0045166015625000, 0.0068969726562500, 0.0005187988281250,
+0.0017395019531250, 0.0021972656250000, 0.0037536621093750, 0.0056457519531250,
+0.0044250488281250, 0.0003051757812500, -0.0007629394531250, -0.0043945312500000,
+-0.0011901855468750, 0.0000610351562500, 0.0085449218750000, -0.0065002441406250,
+0.0068969726562500, -0.0097045898437500, 0.0064392089843750, -0.0053405761718750,
+0.0125732421875000, -0.0029296875000000, 0.0114440917968750, -0.0054016113281250,
+0.0076599121093750, -0.0020446777343750, 0.0108642578125000, 0.0000305175781250,
+0.0058898925781250, -0.0032348632812500, -0.0003662109375000, -0.0043334960937500,
+0.0045166015625000, -0.0022888183593750, -0.0086669921875000, -0.0123596191406250,
+-0.0092773437500000, -0.0108032226562500, -0.0057983398437500, -0.0123901367187500,
+-0.0110778808593750, -0.0198669433593750, -0.0122680664062500, -0.0171508789062500,
+-0.0032653808593750, -0.0187377929687500, -0.0041198730468750, -0.0199890136718750,
+-0.0049743652343750, -0.0192565917968750, 0.0006103515625000, -0.0187072753906250,
+-0.0003967285156250, -0.0189208984375000, -0.0086669921875000, -0.0191345214843750,
+-0.0031127929687500, -0.0193481445312500, -0.0115356445312500, -0.0183410644531250,
+-0.0231323242187500, -0.0179443359375000, -0.0140380859375000, -0.0188293457031250,
+-0.0346069335937500, -0.0158691406250000, -0.0344238281250000, -0.0152893066406250,
+-0.0338134765625000, -0.0169677734375000, -0.0446472167968750, -0.0166625976562500,
+-0.0412597656250000, -0.0156250000000000, -0.0396118164062500, -0.0196838378906250,
+-0.0429077148437500, -0.0197753906250000, -0.0389099121093750, -0.0192260742187500,
+-0.0341796875000000, -0.0208129882812500, -0.0376586914062500, -0.0209045410156250,
+-0.0279235839843750, -0.0201416015625000, -0.0296630859375000, -0.0199890136718750,
+-0.0266113281250000, -0.0205688476562500, -0.0188903808593750, -0.0209655761718750,
+-0.0235290527343750, -0.0206909179687500, -0.0156555175781250, -0.0208129882812500,
+-0.0159912109375000, -0.0216979980468750, -0.0153503417968750, -0.0205993652343750,
+-0.0114135742187500, -0.0194091796875000, -0.0140991210937500, -0.0205383300781250,
+-0.0088500976562500, -0.0184326171875000, -0.0111694335937500, -0.0184631347656250,
+-0.0089721679687500, -0.0180664062500000, -0.0040588378906250, -0.0171508789062500,
+-0.0078735351562500, -0.0177307128906250, 0.0009155273437500, -0.0147094726562500,
+-0.0010681152343750, -0.0154724121093750, 0.0009765625000000, -0.0141601562500000,
+0.0073547363281250, -0.0110778808593750, 0.0033264160156250, -0.0123596191406250,
+0.0112915039062500, -0.0099182128906250, 0.0098571777343750, -0.0096435546875000,
+0.0112915039062500, -0.0093383789062500, 0.0166931152343750, -0.0081787109375000,
+0.0133056640625000, -0.0087585449218750, 0.0191955566406250, -0.0070190429687500,
+0.0191040039062500, -0.0071716308593750, 0.0195312500000000, -0.0070495605468750,
+0.0229187011718750, -0.0061340332031250, 0.0218200683593750, -0.0065917968750000,
+0.0250244140625000, -0.0058288574218750, 0.0246582031250000, -0.0064697265625000,
+0.0252685546875000, -0.0051574707031250, 0.0278015136718750, -0.0037841796875000,
+0.0273437500000000, -0.0050964355468750, 0.0291442871093750, -0.0008850097656250,
+0.0295410156250000, -0.0009155273437500, 0.0299072265625000, -0.0010070800781250,
+0.0307312011718750, 0.0015563964843750, 0.0304870605468750, 0.0011901855468750,
+0.0318298339843750, -0.0008850097656250, 0.0321044921875000, 0.0010070800781250,
+0.0322265625000000, -0.0015869140625000, 0.0329895019531250, -0.0049133300781250,
+0.0332946777343750, -0.0022277832031250, 0.0338745117187500, -0.0100402832031250,
+0.0344848632812500, -0.0090332031250000, 0.0345458984375000, -0.0109252929687500,
+0.0346984863281250, -0.0172729492187500, 0.0346984863281250, -0.0149230957031250,
+0.0323181152343750, -0.0174560546875000, 0.0335998535156250, -0.0193176269531250,
+0.0316162109375000, -0.0184020996093750, 0.0287475585937500, -0.0176696777343750,
+0.0303955078125000, -0.0186462402343750, 0.0238952636718750, -0.0192565917968750,
+0.0253906250000000, -0.0182189941406250, 0.0230712890625000, -0.0198059082031250,
+0.0173034667968750, -0.0216064453125000, 0.0195922851562500, -0.0202636718750000,
+0.0121154785156250, -0.0231628417968750, 0.0122070312500000, -0.0227661132812500,
+0.0091552734375000, -0.0227966308593750, 0.0030822753906250, -0.0250549316406250,
+0.0054321289062500, -0.0240783691406250, -0.0010986328125000, -0.0216369628906250,
+-0.0021057128906250, -0.0244140625000000, -0.0033569335937500, -0.0204467773437500,
+-0.0071716308593750, -0.0153198242187500, -0.0068664550781250, -0.0194091796875000,
+-0.0108642578125000, -0.0111083984375000, -0.0108032226562500, -0.0109863281250000,
+-0.0114440917968750, -0.0104370117187500, -0.0139160156250000, -0.0065612792968750,
+-0.0131530761718750, -0.0081787109375000, -0.0144042968750000, -0.0017700195312500,
+-0.0149536132812500, -0.0034179687500000, -0.0140991210937500, -0.0015563964843750,
+-0.0141601562500000, 0.0047912597656250, -0.0146484375000000, 0.0021667480468750,
+-0.0118103027343750, 0.0022583007812500, -0.0123596191406250, 0.0033264160156250,
+-0.0107116699218750, 0.0051269531250000, -0.0081481933593750, 0.0046691894531250,
+-0.0095825195312500, 0.0025634765625000, -0.0051269531250000, 0.0092163085937500,
+-0.0050659179687500, 0.0095520019531250, -0.0040893554687500, 0.0079040527343750,
+-0.0010986328125000, 0.0102844238281250, -0.0020141601562500, 0.0107116699218750,
+-0.0000305175781250, 0.0068054199218750, 0.0010070800781250, 0.0084533691406250,
+0.0000610351562500, 0.0058898925781250, 0.0000305175781250, 0.0019836425781250,
+0.0006713867187500},
+{0.0142517089843750, -0.0208740234375000, 0.0137023925781250, -0.0218811035156250,
+0.0115966796875000, -0.0191040039062500, 0.0126647949218750, -0.0193786621093750,
+0.0111389160156250, -0.0182495117187500, 0.0088195800781250, -0.0161743164062500,
+0.0107116699218750, -0.0173950195312500, 0.0068664550781250, -0.0133056640625000,
+0.0079040527343750, -0.0136718750000000, 0.0070190429687500, -0.0129394531250000,
+0.0045166015625000, -0.0101928710937500, 0.0070495605468750, -0.0113830566406250,
+0.0027770996093750, -0.0095825195312500, 0.0037841796875000, -0.0097045898437500,
+0.0035095214843750, -0.0090942382812500, 0.0007324218750000, -0.0079956054687500,
+0.0022583007812500, -0.0089416503906250, -0.0005493164062500, -0.0057983398437500,
+0.0002136230468750, -0.0064086914062500, -0.0002136230468750, -0.0050659179687500,
+-0.0022583007812500, -0.0021057128906250, -0.0010070800781250, -0.0034484863281250,
+-0.0029296875000000, -0.0004882812500000, -0.0026855468750000, 0.0003967285156250,
+-0.0021667480468750, -0.0002441406250000, -0.0025939941406250, 0.0008850097656250,
+-0.0019226074218750, 0.0014038085937500, -0.0025024414062500, -0.0018005371093750,
+-0.0019836425781250, -0.0002136230468750, -0.0022277832031250, -0.0023803710937500,
+-0.0027160644531250, -0.0064086914062500, -0.0016784667968750, -0.0041503906250000,
+-0.0024414062500000, -0.0087585449218750, -0.0020446777343750, -0.0096740722656250,
+-0.0018005371093750, -0.0093078613281250, -0.0020141601562500, -0.0107727050781250,
+-0.0008544921875000, -0.0110778808593750, 0.0000305175781250, -0.0123901367187500,
+-0.0002746582031250, -0.0114135742187500, 0.0013427734375000, -0.0137634277343750,
+0.0029602050781250, -0.0164794921875000, 0.0028381347656250, -0.0146179199218750,
+0.0073852539062500, -0.0193786621093750, 0.0066223144531250, -0.0190124511718750,
+0.0091552734375000, -0.0196838378906250, 0.0137023925781250, -0.0225219726562500,
+0.0123596191406250, -0.0207519531250000, 0.0191040039062500, -0.0229187011718750,
+0.0188293457031250, -0.0240478515625000, 0.0209960937500000, -0.0217895507812500,
+0.0263061523437500, -0.0213623046875000, 0.0241088867187500, -0.0231628417968750,
+0.0270690917968750, -0.0172424316406250, 0.0284118652343750, -0.0182800292968750,
+0.0269165039062500, -0.0171203613281250, 0.0272216796875000, -0.0131530761718750,
+0.0280151367187500, -0.0150756835937500, 0.0230712890625000, -0.0102539062500000,
+0.0242919921875000, -0.0115051269531250, 0.0205993652343750, -0.0087890625000000,
+0.0152893066406250, -0.0039672851562500, 0.0180664062500000, -0.0072937011718750,
+0.0081176757812500, -0.0006103515625000, 0.0091247558593750, 0.0004272460937500,
+0.0056457519531250, -0.0012207031250000, -0.0020141601562500, 0.0001220703125000,
+0.0018005371093750, 0.0010986328125000, -0.0068969726562500, 0.0019836425781250,
+-0.0068359375000000, -0.0003356933593750, -0.0082397460937500, 0.0023803710937500,
+-0.0132141113281250, 0.0065917968750000, -0.0104370117187500, 0.0039672851562500,
+-0.0152893066406250, 0.0059509277343750, -0.0155639648437500, 0.0074462890625000,
+-0.0154418945312500, 0.0065002441406250, -0.0180664062500000, 0.0061950683593750,
+-0.0170593261718750, 0.0066223144531250, -0.0149230957031250, 0.0038452148437500,
+-0.0166931152343750, 0.0054931640625000, -0.0139160156250000, 0.0029602050781250,
+-0.0101013183593750, -0.0012512207031250, -0.0123901367187500, 0.0019836425781250,
+-0.0065917968750000, -0.0011596679687500, -0.0066528320312500, -0.0025634765625000,
+-0.0062866210937500, -0.0019836425781250, -0.0027770996093750, -0.0015869140625000,
+-0.0038146972656250, -0.0009765625000000, -0.0041503906250000, -0.0027770996093750,
+-0.0030822753906250, -0.0021057128906250, -0.0042419433593750, -0.0046997070312500,
+-0.0059814453125000, -0.0070800781250000, -0.0050964355468750, -0.0043334960937500,
+-0.0065917968750000, -0.0100708007812500, -0.0071105957031250, -0.0106811523437500,
+-0.0050048828125000, -0.0110473632812500, -0.0048828125000000, -0.0134582519531250,
+-0.0064086914062500, -0.0119323730468750, 0.0007934570312500, -0.0129089355468750,
+0.0001525878906250, -0.0135803222656250, 0.0000305175781250, -0.0133056640625000,
+0.0041198730468750, -0.0130920410156250, 0.0030212402343750, -0.0129699707031250,
+0.0020751953125000, -0.0131835937500000, 0.0025939941406250, -0.0126647949218750,
+0.0025939941406250, -0.0130310058593750, 0.0018310546875000, -0.0144348144531250,
+0.0014953613281250, -0.0138854980468750, 0.0013427734375000, -0.0108947753906250,
+0.0021972656250000, -0.0120544433593750, 0.0003051757812500, -0.0107116699218750,
+-0.0021972656250000, -0.0069580078125000, -0.0021667480468750, -0.0072631835937500,
+-0.0060119628906250, -0.0048522949218750, -0.0050048828125000, -0.0053100585937500,
+-0.0053710937500000, -0.0048828125000000, -0.0081787109375000, -0.0035095214843750,
+-0.0069885253906250, -0.0042114257812500, -0.0086669921875000, -0.0008239746093750,
+-0.0085144042968750, -0.0018005371093750, -0.0094299316406250, -0.0001220703125000,
+-0.0106811523437500, 0.0034484863281250, -0.0096435546875000, 0.0013732910156250,
+-0.0137329101562500, 0.0035095214843750, -0.0134277343750000, 0.0044555664062500,
+-0.0126647949218750, 0.0031433105468750, -0.0140686035156250, 0.0029907226562500,
+-0.0142211914062500, 0.0033264160156250, -0.0153808593750000, -0.0002136230468750,
+-0.0139465332031250, 0.0003967285156250, -0.0162658691406250, 0.0001525878906250,
+-0.0192565917968750, -0.0025939941406250, -0.0166015625000000, -0.0020751953125000,
+-0.0201416015625000, -0.0018005371093750, -0.0227050781250000, -0.0022888183593750,
+-0.0178527832031250, -0.0019226074218750, -0.0152282714843750, -0.0015258789062500,
+-0.0188293457031250, -0.0018005371093750, -0.0089416503906250, -0.0001525878906250,
+-0.0080871582031250, -0.0010070800781250, -0.0110168457031250, 0.0005493164062500,
+-0.0080261230468750, 0.0033569335937500, -0.0060729980468750, 0.0015869140625000,
+-0.0097045898437500, 0.0019836425781250, -0.0120544433593750, 0.0044555664062500,
+-0.0081481933593750, 0.0009765625000000, -0.0055236816406250, -0.0024719238281250,
+-0.0084228515625000, 0.0002136230468750, -0.0025939941406250, -0.0077514648437500,
+-0.0018615722656250, -0.0069885253906250, -0.0042419433593750, -0.0087585449218750,
+-0.0026855468750000, -0.0147705078125000, -0.0014648437500000, -0.0123901367187500,
+-0.0069274902343750, -0.0153503417968750, -0.0066528320312500, -0.0164184570312500,
+-0.0065917968750000, -0.0159301757812500, -0.0093994140625000, -0.0162658691406250,
+-0.0085449218750000, -0.0162658691406250, -0.0086975097656250, -0.0165405273437500,
+-0.0097351074218750, -0.0157775878906250, -0.0086364746093750, -0.0153198242187500,
+-0.0072937011718750, -0.0156250000000000, -0.0085754394531250, -0.0144348144531250,
+-0.0087280273437500, -0.0113220214843750, -0.0079040527343750, -0.0117187500000000,
+-0.0089111328125000, -0.0069274902343750, -0.0102844238281250, -0.0019226074218750,
+-0.0098876953125000, -0.0036315917968750, -0.0115051269531250, 0.0068969726562500,
+-0.0112609863281250, 0.0064697265625000, -0.0118713378906250, 0.0093078613281250,
+-0.0133972167968750, 0.0168151855468750, -0.0128173828125000, 0.0140686035156250,
+-0.0135803222656250, 0.0194396972656250, -0.0138854980468750, 0.0206298828125000,
+-0.0139465332031250, 0.0196533203125000, -0.0141906738281250, 0.0208740234375000,
+-0.0139770507812500, 0.0201110839843750, -0.0138244628906250, 0.0176696777343750,
+-0.0143127441406250, 0.0177612304687500, -0.0132141113281250, 0.0156250000000000,
+-0.0120239257812500, 0.0126953125000000, -0.0128479003906250, 0.0129089355468750,
+-0.0110168457031250, 0.0083312988281250, -0.0104980468750000, 0.0083312988281250,
+-0.0109558105468750, 0.0058593750000000, -0.0105895996093750, 0.0015563964843750,
+-0.0101318359375000, 0.0028686523437500, -0.0108947753906250, -0.0010986328125000,
+-0.0107116699218750, -0.0019226074218750, -0.0106506347656250, -0.0025634765625000,
+-0.0112609863281250, -0.0046386718750000, -0.0110778808593750, -0.0042419433593750,
+-0.0105590820312500, -0.0052490234375000, -0.0111083984375000, -0.0059204101562500,
+-0.0098571777343750, -0.0068969726562500, -0.0087585449218750, -0.0077514648437500,
+-0.0097961425781250, -0.0077819824218750, -0.0065002441406250, -0.0096435546875000,
+-0.0068664550781250, -0.0091552734375000, -0.0065002441406250, -0.0097045898437500,
+-0.0042724609375000, -0.0111083984375000, -0.0051574707031250, -0.0101318359375000,
+-0.0051574707031250, -0.0120849609375000, -0.0043029785156250, -0.0116271972656250,
+-0.0052795410156250, -0.0117797851562500, -0.0062866210937500, -0.0132141113281250,
+-0.0054016113281250, -0.0128479003906250, -0.0075988769531250, -0.0143737792968750,
+-0.0073852539062500, -0.0138854980468750, -0.0075683593750000, -0.0138244628906250,
+-0.0089721679687500, -0.0146789550781250, -0.0083312988281250, -0.0139770507812500,
+-0.0089721679687500, -0.0147094726562500, -0.0088500976562500, -0.0149841308593750,
+-0.0093078613281250, -0.0139770507812500, -0.0095214843750000, -0.0140380859375000,
+-0.0090332031250000, -0.0147399902343750, -0.0122680664062500, -0.0111999511718750,
+-0.0109863281250000, -0.0131225585937500, -0.0131530761718750, -0.0094909667968750,
+-0.0169372558593750, -0.0050964355468750, -0.0149230957031250, -0.0087890625000000,
+-0.0195617675781250, 0.0022888183593750, -0.0197753906250000, -0.0008850097656250,
+-0.0200805664062500, 0.0036010742187500, -0.0225219726562500, 0.0132751464843750,
+-0.0215759277343750, 0.0067138671875000, -0.0220336914062500, 0.0190734863281250,
+-0.0230407714843750, 0.0192871093750000, -0.0213623046875000, 0.0209350585937500,
+-0.0204162597656250, 0.0280761718750000, -0.0215148925781250, 0.0246276855468750,
+-0.0163269042968750, 0.0295104980468750, -0.0172119140625000, 0.0301208496093750,
+-0.0156555175781250, 0.0278015136718750, -0.0116577148437500, 0.0291748046875000,
+-0.0137023925781250, 0.0298156738281250, -0.0100708007812500, 0.0236206054687500,
+-0.0098266601562500, 0.0258789062500000, -0.0100097656250000, 0.0217895507812500,
+-0.0081176757812500, 0.0148620605468750, -0.0086669921875000, 0.0192871093750000,
+-0.0095520019531250, 0.0106506347656250, -0.0088500976562500, 0.0111083984375000,
+-0.0100708007812500, 0.0090637207031250, -0.0118713378906250, 0.0043334960937500,
+-0.0108642578125000, 0.0078430175781250, -0.0120239257812500, -0.0019531250000000,
+-0.0128479003906250, 0.0008544921875000, -0.0121154785156250, -0.0038452148437500,
+-0.0117492675781250, -0.0125122070312500, -0.0122375488281250, -0.0066528320312500,
+-0.0107727050781250, -0.0218505859375000, -0.0113525390625000, -0.0209045410156250,
+-0.0098266601562500, -0.0227661132812500, -0.0079040527343750, -0.0332336425781250,
+-0.0093688964843750, -0.0292968750000000, -0.0054931640625000, -0.0293579101562500,
+-0.0053100585937500, -0.0338439941406250, -0.0047607421875000, -0.0263061523437500,
+-0.0027770996093750, -0.0201110839843750, -0.0037536621093750, -0.0257568359375000,
+-0.0010070800781250, -0.0055236816406250, 0.0000305175781250, -0.0077819824218750,
+-0.0028381347656250, -0.0041198730468750, -0.0032043457031250, 0.0107116699218750,
+-0.0009765625000000, 0.0049743652343750, -0.0091247558593750, 0.0102233886718750,
+-0.0093078613281250, 0.0148010253906250, -0.0079650878906250, 0.0097351074218750,
+-0.0115356445312500, 0.0057983398437500, -0.0114440917968750, 0.0091247558593750,
+-0.0073852539062500, 0.0028991699218750, -0.0083618164062500, 0.0024719238281250,
+-0.0075683593750000, 0.0029602050781250, -0.0034790039062500, -0.0008239746093750,
+-0.0050354003906250, -0.0007019042968750, -0.0100097656250000, 0.0038146972656250,
+-0.0067749023437500},
+{-0.0079345703125000, 0.0120544433593750, -0.0084228515625000, 0.0121154785156250,
+-0.0075988769531250, 0.0121459960937500, -0.0065917968750000, 0.0129394531250000,
+-0.0074462890625000, 0.0128784179687500, -0.0059509277343750, 0.0135803222656250,
+-0.0057678222656250, 0.0138854980468750, -0.0061950683593750, 0.0138854980468750,
+-0.0057678222656250, 0.0141296386718750, -0.0056457519531250, 0.0145568847656250,
+-0.0067749023437500, 0.0148925781250000, -0.0065002441406250, 0.0149536132812500,
+-0.0069274902343750, 0.0153808593750000, -0.0076293945312500, 0.0157775878906250,
+-0.0072021484375000, 0.0157470703125000, -0.0095825195312500, 0.0173034667968750,
+-0.0085754394531250, 0.0178833007812500, -0.0106201171875000, 0.0167846679687500,
+-0.0136108398437500, 0.0171203613281250, -0.0116577148437500, 0.0184020996093750,
+-0.0165710449218750, 0.0141601562500000, -0.0166320800781250, 0.0165100097656250,
+-0.0166625976562500, 0.0134582519531250, -0.0192260742187500, 0.0088806152343750,
+-0.0184326171875000, 0.0131835937500000, -0.0182800292968750, 0.0038452148437500,
+-0.0189208984375000, 0.0063171386718750, -0.0174560546875000, 0.0049438476562500,
+-0.0162353515625000, -0.0021667480468750, -0.0168151855468750, 0.0016784667968750,
+-0.0132141113281250, -0.0031127929687500, -0.0135803222656250, -0.0039978027343750,
+-0.0125122070312500, -0.0025024414062500, -0.0096740722656250, -0.0038452148437500,
+-0.0107727050781250, -0.0035705566406250, -0.0085449218750000, 0.0005187988281250,
+-0.0082397460937500, -0.0016784667968750, -0.0085449218750000, 0.0026245117187500,
+-0.0076904296875000, 0.0084228515625000, -0.0079040527343750, 0.0044860839843750,
+-0.0088500976562500, 0.0145263671875000, -0.0080871582031250, 0.0135498046875000,
+-0.0094299316406250, 0.0143432617187500, -0.0111389160156250, 0.0208435058593750,
+-0.0099182128906250, 0.0178222656250000, -0.0130004882812500, 0.0183410644531250,
+-0.0133972167968750, 0.0198059082031250, -0.0130004882812500, 0.0187683105468750,
+-0.0140686035156250, 0.0172729492187500, -0.0141906738281250, 0.0178527832031250,
+-0.0138244628906250, 0.0171813964843750, -0.0136413574218750, 0.0169982910156250,
+-0.0140686035156250, 0.0172729492187500, -0.0147399902343750, 0.0177917480468750,
+-0.0143737792968750, 0.0175781250000000, -0.0125427246093750, 0.0151367187500000,
+-0.0140991210937500, 0.0183105468750000, -0.0117187500000000, 0.0127258300781250,
+-0.0083007812500000, 0.0075073242187500, -0.0102539062500000, 0.0125732421875000,
+-0.0050964355468750, -0.0069580078125000, -0.0048522949218750, -0.0030822753906250,
+-0.0052490234375000, -0.0088500976562500, -0.0022888183593750, -0.0249633789062500,
+-0.0029296875000000, -0.0164794921875000, -0.0067138671875000, -0.0281677246093750,
+-0.0050964355468750, -0.0320434570312500, -0.0062255859375000, -0.0254516601562500,
+-0.0109863281250000, -0.0261535644531250, -0.0094909667968750, -0.0289306640625000,
+-0.0053405761718750, -0.0108947753906250, -0.0080566406250000, -0.0145568847656250,
+-0.0052490234375000, -0.0088806152343750, 0.0008544921875000, 0.0071411132812500,
+-0.0018310546875000, 0.0010375976562500, 0.0011291503906250, 0.0091552734375000,
+0.0029602050781250, 0.0117797851562500, -0.0008850097656250, 0.0081176757812500,
+-0.0024108886718750, 0.0095520019531250, 0.0004272460937500, 0.0117492675781250,
+-0.0074157714843750, 0.0021057128906250, -0.0086669921875000, 0.0014953613281250,
+-0.0072326660156250, 0.0020751953125000, -0.0086669921875000, -0.0017395019531250,
+-0.0091552734375000, -0.0007324218750000, -0.0114135742187500, 0.0006103515625000,
+-0.0095214843750000, 0.0000000000000000, -0.0122375488281250, -0.0023498535156250,
+-0.0176391601562500, -0.0015869140625000, -0.0148620605468750, 0.0003356933593750,
+-0.0137634277343750, -0.0098571777343750, -0.0172119140625000, -0.0087280273437500,
+-0.0135192871093750, -0.0092773437500000, -0.0042419433593750, -0.0189819335937500,
+0.0008239746093750, -0.0040893554687500, -0.0058593750000000, -0.0143432617187500,
+-0.0039978027343750, -0.0107116699218750, -0.0029296875000000, -0.0119323730468750,
+-0.0061645507812500, -0.0177307128906250, -0.0072326660156250, -0.0166320800781250,
+0.0002441406250000, -0.0128173828125000, -0.0005187988281250, -0.0144348144531250,
+-0.0012512207031250, -0.0146179199218750, 0.0036010742187500, -0.0102844238281250,
+0.0029602050781250, -0.0105895996093750, -0.0037536621093750, -0.0191955566406250,
+-0.0013122558593750, -0.0173339843750000, -0.0036621093750000, -0.0174865722656250,
+-0.0105590820312500, -0.0250244140625000, -0.0076293945312500, -0.0233459472656250,
+-0.0092468261718750, -0.0128479003906250, -0.0114135742187500, -0.0192871093750000,
+-0.0091552734375000, -0.0105590820312500, -0.0055236816406250, 0.0054626464843750,
+-0.0076904296875000, -0.0032043457031250, -0.0144958496093750, 0.0067749023437500,
+-0.0082397460937500, 0.0108337402343750, -0.0153198242187500, 0.0065307617187500,
+-0.0280456542968750, 0.0061645507812500, -0.0205078125000000, 0.0078735351562500,
+-0.0295410156250000, 0.0020751953125000, -0.0331726074218750, 0.0012817382812500,
+-0.0276794433593750, 0.0028381347656250, -0.0283813476562500, 0.0019836425781250,
+-0.0304260253906250, 0.0010375976562500, -0.0111694335937500, 0.0002441406250000,
+-0.0176086425781250, 0.0020751953125000, -0.0087890625000000, 0.0001525878906250,
+0.0111083984375000, -0.0032043457031250, 0.0010681152343750, -0.0018310546875000,
+0.0162658691406250, -0.0035095214843750, 0.0185241699218750, -0.0032348632812500,
+0.0168762207031250, -0.0052795410156250, 0.0229187011718750, -0.0067138671875000,
+0.0213928222656250, -0.0048522949218750, 0.0180969238281250, -0.0129089355468750,
+0.0201416015625000, -0.0116882324218750, 0.0185546875000000, -0.0132141113281250,
+0.0148010253906250, -0.0196838378906250, 0.0172119140625000, -0.0173339843750000,
+0.0155639648437500, -0.0184326171875000, 0.0146789550781250, -0.0196533203125000,
+0.0150451660156250, -0.0192871093750000, 0.0154113769531250, -0.0182189941406250,
+0.0155029296875000, -0.0182495117187500, 0.0136718750000000, -0.0215148925781250,
+0.0147094726562500, -0.0206298828125000, 0.0141296386718750, -0.0209960937500000,
+0.0121765136718750, -0.0240173339843750, 0.0128784179687500, -0.0234375000000000,
+0.0114135742187500, -0.0217285156250000, 0.0114135742187500, -0.0224304199218750,
+0.0107421875000000, -0.0223388671875000, 0.0094299316406250, -0.0205078125000000,
+0.0100402832031250, -0.0206604003906250, 0.0086975097656250, -0.0229187011718750,
+0.0075683593750000, -0.0221862792968750, 0.0091247558593750, -0.0223388671875000,
+0.0100402832031250, -0.0242309570312500, 0.0086669921875000, -0.0237731933593750,
+0.0119934082031250, -0.0227050781250000, 0.0114135742187500, -0.0232543945312500,
+0.0115356445312500, -0.0229492187500000, 0.0137939453125000, -0.0218200683593750,
+0.0127868652343750, -0.0221862792968750, 0.0124816894531250, -0.0218200683593750,
+0.0136413574218750, -0.0222167968750000, 0.0115966796875000, -0.0213928222656250,
+0.0101318359375000, -0.0206298828125000, 0.0121154785156250, -0.0216369628906250,
+0.0066833496093750, -0.0189208984375000, 0.0082092285156250, -0.0192565917968750,
+0.0064392089843750, -0.0184936523437500, 0.0023803710937500, -0.0167236328125000,
+0.0058288574218750, -0.0175781250000000, -0.0010070800781250, -0.0141906738281250,
+0.0008850097656250, -0.0146179199218750, -0.0006103515625000, -0.0133056640625000,
+-0.0057067871093750, -0.0101623535156250, -0.0021667480468750, -0.0112609863281250,
+-0.0075378417968750, -0.0091552734375000, -0.0063476562500000, -0.0082397460937500,
+-0.0069580078125000, -0.0087280273437500, -0.0100097656250000, -0.0084533691406250,
+-0.0067138671875000, -0.0079345703125000, -0.0110473632812500, -0.0089721679687500,
+-0.0095520019531250, -0.0090332031250000, -0.0104675292968750, -0.0088806152343750,
+-0.0135498046875000, -0.0096130371093750, -0.0104675292968750, -0.0098571777343750,
+-0.0148620605468750, -0.0088195800781250, -0.0137023925781250, -0.0093688964843750,
+-0.0133666992187500, -0.0093383789062500, -0.0158691406250000, -0.0083007812500000,
+-0.0136413574218750, -0.0087890625000000, -0.0137939453125000, -0.0093688964843750,
+-0.0133056640625000, -0.0089111328125000, -0.0116882324218750, -0.0094604492187500,
+-0.0107116699218750, -0.0102844238281250, -0.0100402832031250, -0.0097045898437500,
+-0.0072326660156250, -0.0108642578125000, -0.0072631835937500, -0.0103149414062500,
+-0.0050354003906250, -0.0111694335937500, -0.0022888183593750, -0.0127563476562500,
+-0.0031433105468750, -0.0116271972656250, 0.0021057128906250, -0.0128479003906250,
+0.0016174316406250, -0.0137939453125000, 0.0032043457031250, -0.0125427246093750,
+0.0072326660156250, -0.0118713378906250, 0.0057678222656250, -0.0128479003906250,
+0.0087585449218750, -0.0109558105468750, 0.0094909667968750, -0.0112915039062500,
+0.0086975097656250, -0.0102844238281250, 0.0094299316406250, -0.0089111328125000,
+0.0098876953125000, -0.0101928710937500, 0.0080566406250000, -0.0063781738281250,
+0.0081481933593750, -0.0063781738281250, 0.0072326660156250, -0.0064697265625000,
+0.0056152343750000, -0.0043640136718750, 0.0059509277343750, -0.0047302246093750,
+0.0030517578125000, -0.0053100585937500, 0.0032043457031250, -0.0043029785156250,
+0.0032348632812500, -0.0060119628906250, 0.0013122558593750, -0.0080261230468750,
+0.0015258789062500, -0.0061035156250000, 0.0028381347656250, -0.0084838867187500,
+0.0016174316406250, -0.0104980468750000, 0.0036621093750000, -0.0071411132812500,
+0.0061340332031250, -0.0051269531250000, 0.0044250488281250, -0.0079956054687500,
+0.0101318359375000, -0.0024414062500000, 0.0098266601562500, -0.0017395019531250,
+0.0109863281250000, -0.0023498535156250, 0.0148010253906250, -0.0001220703125000,
+0.0130310058593750, -0.0006103515625000, 0.0145568847656250, -0.0026245117187500,
+0.0156555175781250, -0.0010375976562500, 0.0140991210937500, -0.0034484863281250,
+0.0139160156250000, -0.0072937011718750, 0.0148010253906250, -0.0044860839843750,
+0.0083312988281250, -0.0059509277343750, 0.0106201171875000, -0.0091857910156250,
+0.0054016113281250, -0.0046386718750000, -0.0022888183593750, 0.0001525878906250,
+0.0025939941406250, -0.0037231445312500, -0.0088500976562500, 0.0007629394531250,
+-0.0087890625000000, 0.0033264160156250, -0.0098266601562500, -0.0008239746093750,
+-0.0163574218750000, -0.0022583007812500, -0.0139160156250000, 0.0006103515625000,
+-0.0181579589843750, -0.0082702636718750, -0.0194396972656250, -0.0092163085937500,
+-0.0168457031250000, -0.0071105957031250, -0.0173950195312500, -0.0096435546875000,
+-0.0189514160156250, -0.0101318359375000, -0.0112304687500000, -0.0064697265625000,
+-0.0124206542968750, -0.0075988769531250, -0.0102844238281250, -0.0068969726562500,
+-0.0041198730468750, -0.0035705566406250, -0.0067749023437500, -0.0045776367187500,
+-0.0040893554687500, -0.0043640136718750, -0.0030517578125000, -0.0040588378906250,
+-0.0034179687500000, -0.0053710937500000, -0.0030212402343750, -0.0061645507812500,
+-0.0032958984375000, -0.0051574707031250, -0.0029296875000000, -0.0066833496093750,
+-0.0032348632812500, -0.0071716308593750, -0.0026245117187500, -0.0079956054687500,
+-0.0016174316406250, -0.0090942382812500, -0.0023803710937500, -0.0079956054687500,
+-0.0026855468750000, -0.0078430175781250, -0.0015869140625000, -0.0095214843750000,
+-0.0035095214843750, -0.0077209472656250, -0.0062255859375000, -0.0047302246093750,
+-0.0062255859375000, -0.0054931640625000, -0.0103759765625000, -0.0035400390625000,
+-0.0097045898437500, -0.0034179687500000, -0.0085449218750000, -0.0042724609375000,
+-0.0108337402343750},
+{-0.0090026855468750, -0.0102539062500000, -0.0097045898437500, -0.0093994140625000,
+-0.0110168457031250, -0.0093383789062500, -0.0106506347656250, -0.0097656250000000,
+-0.0117797851562500, -0.0097045898437500, -0.0119323730468750, -0.0092468261718750,
+-0.0115966796875000, -0.0094604492187500, -0.0130004882812500, -0.0099182128906250,
+-0.0119934082031250, -0.0084228515625000, -0.0125427246093750, -0.0087585449218750,
+-0.0127563476562500, -0.0081481933593750, -0.0125732421875000, -0.0072326660156250,
+-0.0135498046875000, -0.0080566406250000, -0.0112915039062500, -0.0044860839843750,
+-0.0122070312500000, -0.0053710937500000, -0.0103149414062500, -0.0045776367187500,
+-0.0075683593750000, -0.0012817382812500, -0.0092163085937500, -0.0024414062500000,
+-0.0033874511718750, -0.0027770996093750, -0.0039978027343750, -0.0013732910156250,
+-0.0024719238281250, -0.0028076171875000, 0.0019226074218750, -0.0044555664062500,
+0.0004882812500000, -0.0032043457031250, 0.0047607421875000, -0.0078430175781250,
+0.0048217773437500, -0.0071716308593750, 0.0046386718750000, -0.0072326660156250,
+0.0070800781250000, -0.0111389160156250, 0.0067138671875000, -0.0104370117187500,
+0.0050964355468750, -0.0060424804687500, 0.0062255859375000, -0.0083312988281250,
+0.0042114257812500, -0.0071716308593750, 0.0014038085937500, -0.0020751953125000,
+0.0027465820312500, -0.0037231445312500, -0.0021667480468750, -0.0041503906250000,
+-0.0021667480468750, -0.0025024414062500, -0.0016784667968750, -0.0046691894531250,
+-0.0040893554687500, -0.0069580078125000, -0.0033874511718750, -0.0052185058593750,
+-0.0011901855468750, -0.0101013183593750, -0.0027770996093750, -0.0103454589843750,
+0.0004272460937500, -0.0086669921875000, 0.0043945312500000, -0.0110778808593750,
+0.0019226074218750, -0.0119018554687500, 0.0099487304687500, -0.0037841796875000,
+0.0099487304687500, -0.0053100585937500, 0.0098571777343750, -0.0057373046875000,
+0.0146179199218750, -0.0001831054687500, 0.0134582519531250, -0.0010375976562500,
+0.0093994140625000, -0.0047912597656250, 0.0117797851562500, -0.0027160644531250,
+0.0080871582031250, -0.0052795410156250, 0.0024414062500000, -0.0108947753906250,
+0.0053710937500000, -0.0081787109375000, -0.0031738281250000, -0.0075988769531250,
+-0.0022583007812500, -0.0103454589843750, -0.0059509277343750, -0.0076599121093750,
+-0.0133056640625000, -0.0041198730468750, -0.0093688964843750, -0.0066833496093750,
+-0.0163269042968750, -0.0007934570312500, -0.0183715820312500, -0.0004272460937500,
+-0.0161437988281250, -0.0010681152343750, -0.0173950195312500, 0.0011596679687500,
+-0.0179443359375000, 0.0002746582031250, -0.0122985839843750, -0.0006408691406250,
+-0.0138854980468750, 0.0006103515625000, -0.0111694335937500, -0.0007934570312500,
+-0.0057678222656250, -0.0032958984375000, -0.0088195800781250, -0.0024108886718750,
+-0.0021667480468750, -0.0047302246093750, -0.0012817382812500, -0.0049133300781250,
+-0.0027465820312500, -0.0039672851562500, -0.0007934570312500, -0.0049133300781250,
+-0.0007019042968750, -0.0057983398437500, -0.0028991699218750, -0.0036315917968750,
+-0.0025634765625000, -0.0033264160156250, -0.0034179687500000, -0.0040588378906250,
+-0.0052490234375000, -0.0039367675781250, -0.0051879882812500, -0.0043945312500000,
+-0.0089416503906250, -0.0065917968750000, -0.0073547363281250, -0.0060119628906250,
+-0.0086364746093750, -0.0062255859375000, -0.0125427246093750, -0.0079040527343750,
+-0.0102539062500000, -0.0074462890625000, -0.0106506347656250, -0.0076599121093750,
+-0.0136108398437500, -0.0081176757812500, -0.0092468261718750, -0.0080566406250000,
+-0.0046386718750000, -0.0079956054687500, -0.0085144042968750, -0.0080261230468750,
+-0.0027770996093750, -0.0063171386718750, -0.0007019042968750, -0.0075988769531250,
+-0.0028076171875000, -0.0065002441406250, -0.0028686523437500, -0.0035705566406250,
+-0.0021362304687500, -0.0048828125000000, -0.0041809082031250, -0.0055541992187500,
+-0.0044860839843750, -0.0041809082031250, -0.0032653808593750, -0.0070190429687500,
+-0.0031127929687500, -0.0094299316406250, -0.0039672851562500, -0.0072021484375000,
+-0.0033264160156250, -0.0153503417968750, -0.0018310546875000, -0.0152282714843750,
+-0.0041809082031250, -0.0148925781250000, -0.0063781738281250, -0.0196228027343750,
+-0.0041503906250000, -0.0187683105468750, -0.0075073242187500, -0.0152893066406250,
+-0.0097351074218750, -0.0180664062500000, -0.0057983398437500, -0.0144042968750000,
+-0.0030822753906250, -0.0088195800781250, -0.0061340332031250, -0.0120544433593750,
+-0.0025939941406250, -0.0028076171875000, -0.0004882812500000, -0.0034179687500000,
+-0.0038146972656250, -0.0012817382812500, -0.0048217773437500, 0.0051879882812500,
+-0.0024414062500000, 0.0024108886718750, -0.0111999511718750, 0.0080871582031250,
+-0.0108642578125000, 0.0089416503906250, -0.0093688964843750, 0.0079956054687500,
+-0.0132751464843750, 0.0101318359375000, -0.0134277343750000, 0.0100402832031250,
+-0.0118103027343750, 0.0070800781250000, -0.0118103027343750, 0.0081481933593750,
+-0.0123901367187500, 0.0061035156250000, -0.0118103027343750, 0.0025024414062500,
+-0.0119323730468750, 0.0041809082031250, -0.0154418945312500, 0.0006408691406250,
+-0.0144042968750000, -0.0002746582031250, -0.0148925781250000, 0.0011596679687500,
+-0.0181274414062500, 0.0007934570312500, -0.0175170898437500, 0.0003051757812500,
+-0.0170898437500000, 0.0044860839843750, -0.0178222656250000, 0.0046691894531250,
+-0.0170898437500000, 0.0056457519531250, -0.0162963867187500, 0.0086669921875000,
+-0.0168762207031250, 0.0082702636718750, -0.0142211914062500, 0.0084228515625000,
+-0.0147399902343750, 0.0098571777343750, -0.0142517089843750, 0.0085449218750000,
+-0.0123901367187500, 0.0070495605468750, -0.0131835937500000, 0.0084838867187500,
+-0.0108337402343750, 0.0051574707031250, -0.0114746093750000, 0.0053710937500000,
+-0.0104980468750000, 0.0049743652343750, -0.0081787109375000, 0.0029602050781250,
+-0.0093078613281250, 0.0039978027343750, -0.0075073242187500, 0.0035400390625000,
+-0.0073547363281250, 0.0033264160156250, -0.0067749023437500, 0.0044250488281250,
+-0.0057067871093750, 0.0051879882812500, -0.0065002441406250, 0.0050048828125000,
+-0.0043029785156250, 0.0066223144531250, -0.0040893554687500, 0.0069274902343750,
+-0.0045776367187500, 0.0083007812500000, -0.0038146972656250, 0.0100708007812500,
+-0.0036010742187500, 0.0100708007812500, -0.0045471191406250, 0.0133056640625000,
+-0.0043640136718750, 0.0140686035156250, -0.0039978027343750, 0.0151977539062500,
+-0.0042724609375000, 0.0175170898437500, -0.0042419433593750, 0.0177612304687500,
+-0.0038757324218750, 0.0184936523437500, -0.0035400390625000, 0.0200805664062500,
+-0.0043029785156250, 0.0200195312500000, -0.0045776367187500, 0.0197448730468750,
+-0.0039062500000000, 0.0211181640625000, -0.0062255859375000, 0.0197448730468750,
+-0.0059509277343750, 0.0210876464843750, -0.0062866210937500, 0.0209960937500000,
+-0.0078430175781250, 0.0199279785156250, -0.0071716308593750, 0.0215759277343750,
+-0.0083007812500000, 0.0190429687500000, -0.0083923339843750, 0.0204467773437500,
+-0.0087280273437500, 0.0197448730468750, -0.0094604492187500, 0.0177917480468750,
+-0.0092773437500000, 0.0199584960937500, -0.0106201171875000, 0.0156555175781250,
+-0.0107421875000000, 0.0178527832031250, -0.0107727050781250, 0.0168762207031250,
+-0.0115966796875000, 0.0134277343750000, -0.0115051269531250, 0.0166015625000000,
+-0.0106506347656250, 0.0111694335937500, -0.0111694335937500, 0.0121459960937500,
+-0.0103454589843750, 0.0108947753906250, -0.0091552734375000, 0.0072631835937500,
+-0.0097351074218750, 0.0094299316406250, -0.0077209472656250, 0.0026245117187500,
+-0.0078735351562500, 0.0041503906250000, -0.0071105957031250, 0.0026855468750000,
+-0.0053405761718750, -0.0021972656250000, -0.0059509277343750, 0.0008239746093750,
+-0.0048828125000000, -0.0048522949218750, -0.0043029785156250, -0.0048522949218750,
+-0.0047302246093750, -0.0042419433593750, -0.0046691894531250, -0.0076599121093750,
+-0.0042724609375000, -0.0073852539062500, -0.0053405761718750, -0.0039672851562500,
+-0.0047607421875000, -0.0065307617187500, -0.0057373046875000, -0.0023193359375000,
+-0.0068664550781250, 0.0026245117187500, -0.0057983398437500, -0.0014953613281250,
+-0.0093688964843750, 0.0111389160156250, -0.0087890625000000, 0.0082702636718750,
+-0.0094909667968750, 0.0113830566406250, -0.0121765136718750, 0.0217285156250000,
+-0.0111083984375000, 0.0164489746093750, -0.0124206542968750, 0.0230407714843750,
+-0.0126647949218750, 0.0244445800781250, -0.0127563476562500, 0.0223999023437500,
+-0.0130615234375000, 0.0239562988281250, -0.0127563476562500, 0.0245971679687500,
+-0.0145568847656250, 0.0213012695312500, -0.0141296386718750, 0.0215759277343750,
+-0.0144042968750000, 0.0212097167968750, -0.0163269042968750, 0.0201110839843750,
+-0.0158691406250000, 0.0206909179687500, -0.0140380859375000, 0.0153808593750000,
+-0.0151977539062500, 0.0173645019531250, -0.0143432617187500, 0.0147094726562500,
+-0.0116882324218750, 0.0090332031250000, -0.0126342773437500, 0.0124206542968750,
+-0.0130004882812500, 0.0076599121093750, -0.0120849609375000, 0.0064697265625000,
+-0.0129699707031250, 0.0074157714843750, -0.0144653320312500, 0.0061035156250000,
+-0.0137023925781250, 0.0059509277343750, -0.0145568847656250, 0.0080871582031250,
+-0.0155334472656250, 0.0068359375000000, -0.0135192871093750, 0.0088500976562500,
+-0.0127563476562500, 0.0114440917968750, -0.0140991210937500, 0.0092468261718750,
+-0.0064086914062500, 0.0153503417968750, -0.0081176757812500, 0.0148925781250000,
+-0.0057067871093750, 0.0142822265625000, 0.0009765625000000, 0.0185241699218750,
+-0.0018920898437500, 0.0181274414062500, 0.0031738281250000, 0.0121154785156250,
+0.0036926269531250, 0.0161743164062500, 0.0021667480468750, 0.0086059570312500,
+0.0048217773437500, -0.0011596679687500, 0.0050048828125000, 0.0058898925781250,
+-0.0035705566406250, -0.0108947753906250, -0.0017700195312500, -0.0101928710937500,
+-0.0026550292968750, -0.0135803222656250, -0.0089111328125000, -0.0252990722656250,
+-0.0072631835937500, -0.0191650390625000, -0.0089721679687500, -0.0213928222656250,
+-0.0101013183593750, -0.0265502929687500, -0.0074768066406250, -0.0197753906250000,
+-0.0068969726562500, -0.0133361816406250, -0.0087280273437500, -0.0178222656250000,
+-0.0005798339843750, -0.0052795410156250, -0.0010986328125000, -0.0037231445312500,
+-0.0012207031250000, -0.0073852539062500, 0.0033569335937500, -0.0042114257812500,
+0.0023498535156250, -0.0025024414062500, 0.0016479492187500, -0.0084533691406250,
+0.0024414062500000, -0.0089721679687500, 0.0014953613281250, -0.0075378417968750,
+-0.0007934570312500, -0.0093994140625000, 0.0051269531250000, -0.0077209472656250,
+-0.0088806152343750, -0.0087585449218750, -0.0025634765625000, -0.0093688964843750,
+-0.0078735351562500, -0.0069274902343750, -0.0174560546875000, -0.0057678222656250,
+-0.0130615234375000, -0.0075683593750000, -0.0184326171875000, -0.0020751953125000,
+-0.0192871093750000, -0.0012207031250000, -0.0191650390625000, -0.0043945312500000,
+-0.0210876464843750, -0.0035705566406250, -0.0200805664062500, -0.0015563964843750,
+-0.0226135253906250, -0.0094604492187500, -0.0226440429687500, -0.0099182128906250,
+-0.0214538574218750, -0.0091247558593750, -0.0229187011718750, -0.0122985839843750,
+-0.0229187011718750, -0.0118713378906250, -0.0159301757812500, -0.0102539062500000,
+-0.0196533203125000, -0.0106811523437500, -0.0112915039062500, -0.0112609863281250,
+-0.0011901855468750, -0.0111083984375000, -0.0094604492187500, -0.0104675292968750,
+0.0102539062500000},
+{-0.0071105957031250, 0.0089721679687500, -0.0105590820312500, -0.0025329589843750,
+-0.0108032226562500, -0.0017700195312500, -0.0107421875000000, -0.0018310546875000,
+-0.0121459960937500, -0.0090637207031250, -0.0115966796875000, -0.0066833496093750,
+-0.0127563476562500, -0.0052795410156250, -0.0124206542968750, -0.0081481933593750,
+-0.0129394531250000, -0.0037231445312500, -0.0142211914062500, 0.0008850097656250,
+-0.0133666992187500, -0.0029296875000000, -0.0137329101562500, 0.0074157714843750,
+-0.0144958496093750, 0.0061950683593750, -0.0133056640625000, 0.0092468261718750,
+-0.0121459960937500, 0.0166625976562500, -0.0131225585937500, 0.0127258300781250,
+-0.0122375488281250, 0.0228881835937500, -0.0112609863281250, 0.0229797363281250,
+-0.0125427246093750, 0.0220947265625000, -0.0137023925781250, 0.0272521972656250,
+-0.0127563476562500, 0.0260620117187500, -0.0140991210937500, 0.0221252441406250,
+-0.0146789550781250, 0.0244140625000000, -0.0143127441406250, 0.0217285156250000,
+-0.0144042968750000, 0.0165710449218750, -0.0145263671875000, 0.0188598632812500,
+-0.0131225585937500, 0.0133361816406250, -0.0143127441406250, 0.0138244628906250,
+-0.0113525390625000, 0.0120849609375000, -0.0085754394531250, 0.0081176757812500,
+-0.0112304687500000, 0.0101623535156250, -0.0035095214843750, 0.0042114257812500,
+-0.0035095214843750, 0.0039672851562500, -0.0036926269531250, 0.0051879882812500,
+0.0003967285156250, 0.0024719238281250, -0.0008544921875000, 0.0020446777343750,
+-0.0023193359375000, 0.0060424804687500, 0.0001831054687500, 0.0052185058593750,
+-0.0044555664062500, 0.0075073242187500, -0.0103759765625000, 0.0116882324218750,
+-0.0061950683593750, 0.0095520019531250, -0.0097045898437500, 0.0115356445312500,
+-0.0128479003906250, 0.0132446289062500, -0.0098876953125000, 0.0106506347656250,
+-0.0076293945312500, 0.0106811523437500, -0.0092468261718750, 0.0118713378906250,
+-0.0040893554687500, -0.0033264160156250, -0.0049133300781250, 0.0009765625000000,
+-0.0032043457031250, -0.0046997070312500, 0.0010375976562500, -0.0192260742187500,
+-0.0009765625000000, -0.0123291015625000, 0.0031127929687500, -0.0212402343750000,
+0.0026550292968750, -0.0256652832031250, 0.0043945312500000, -0.0191955566406250,
+0.0082702636718750, -0.0166625976562500, 0.0058288574218750, -0.0204772949218750,
+0.0069885253906250, -0.0115661621093750, 0.0096130371093750, -0.0110473632812500,
+0.0065307617187500, -0.0119323730468750, 0.0004882812500000, -0.0065612792968750,
+-0.0120544433593750, -0.0219421386718750, -0.0196228027343750, -0.0101928710937500,
+-0.0203247070312500, -0.0137634277343750, -0.0183105468750000, -0.0122070312500000,
+-0.0177612304687500, -0.0068664550781250, -0.0187988281250000, -0.0079650878906250,
+-0.0169982910156250, -0.0068359375000000, -0.0174560546875000, -0.0077514648437500,
+-0.0151367187500000, -0.0045166015625000, -0.0138854980468750, -0.0011596679687500,
+-0.0156860351562500, -0.0045776367187500, -0.0033569335937500, 0.0002746582031250,
+-0.0065307617187500, 0.0028686523437500, -0.0017395019531250, -0.0012817382812500,
+0.0103759765625000, -0.0029296875000000, 0.0049743652343750, -0.0004882812500000,
+0.0114746093750000, -0.0094604492187500, 0.0151062011718750, -0.0092163085937500,
+0.0090942382812500, -0.0087890625000000, 0.0077209472656250, -0.0136413574218750,
+0.0122985839843750, -0.0128784179687500, -0.0024108886718750, -0.0110778808593750,
+-0.0019226074218750, -0.0124511718750000, -0.0007629394531250, -0.0098571777343750,
+-0.0084533691406250, -0.0070190429687500, -0.0067138671875000, -0.0093383789062500,
+-0.0018310546875000, -0.0028381347656250, -0.0053100585937500, -0.0032348632812500,
+-0.0007324218750000, -0.0020141601562500, 0.0062255859375000, 0.0022277832031250,
+0.0021667480468750, -0.0000305175781250, 0.0137329101562500, 0.0022277832031250,
+0.0133972167968750, 0.0038757324218750, 0.0143127441406250, 0.0016479492187500,
+0.0205383300781250, -0.0000610351562500, 0.0183715820312500, 0.0013732910156250,
+0.0247497558593750, -0.0003051757812500, 0.0232543945312500, -0.0010681152343750,
+0.0240173339843750, -0.0006103515625000, 0.0294189453125000, -0.0005798339843750,
+0.0272521972656250, -0.0007629394531250, 0.0263366699218750, -0.0002136230468750,
+0.0281982421875000, 0.0001220703125000, 0.0249023437500000, -0.0014038085937500,
+0.0225219726562500, -0.0015869140625000, 0.0249633789062500, -0.0005798339843750,
+0.0137939453125000, -0.0065612792968750, 0.0172119140625000, -0.0056762695312500,
+0.0121765136718750, -0.0065917968750000, 0.0014038085937500, -0.0108642578125000,
+0.0078735351562500, -0.0093383789062500, -0.0030517578125000, -0.0107727050781250,
+-0.0047607421875000, -0.0111389160156250, -0.0021362304687500, -0.0110168457031250,
+-0.0060119628906250, -0.0115966796875000, -0.0064697265625000, -0.0113525390625000,
+0.0009460449218750, -0.0112304687500000, -0.0021972656250000, -0.0115356445312500,
+0.0025024414062500, -0.0113525390625000, 0.0100097656250000, -0.0108947753906250,
+0.0046997070312500, -0.0111083984375000, 0.0188293457031250, -0.0120239257812500,
+0.0171203613281250, -0.0113830566406250, 0.0194702148437500, -0.0124511718750000,
+0.0289611816406250, -0.0142211914062500, 0.0242309570312500, -0.0133056640625000,
+0.0309143066406250, -0.0146179199218750, 0.0314331054687500, -0.0152282714843750,
+0.0303955078125000, -0.0145568847656250, 0.0333557128906250, -0.0142211914062500,
+0.0327758789062500, -0.0144653320312500, 0.0308532714843750, -0.0142211914062500,
+0.0325012207031250, -0.0136108398437500, 0.0306091308593750, -0.0140380859375000,
+0.0280456542968750, -0.0145568847656250, 0.0304870605468750, -0.0140075683593750,
+0.0254211425781250, -0.0142822265625000, 0.0263671875000000, -0.0143127441406250,
+0.0252380371093750, -0.0141906738281250, 0.0219116210937500, -0.0140380859375000,
+0.0244750976562500, -0.0139770507812500, 0.0194091796875000, -0.0150146484375000,
+0.0204162597656250, -0.0147705078125000, 0.0191345214843750, -0.0152282714843750,
+0.0154418945312500, -0.0164184570312500, 0.0178833007812500, -0.0162353515625000,
+0.0135803222656250, -0.0162658691406250, 0.0139770507812500, -0.0168151855468750,
+0.0132446289062500, -0.0164794921875000, 0.0104370117187500, -0.0159606933593750,
+0.0121154785156250, -0.0163574218750000, 0.0094604492187500, -0.0152587890625000,
+0.0095825195312500, -0.0156250000000000, 0.0100097656250000, -0.0148010253906250,
+0.0089721679687500, -0.0134887695312500, 0.0093688964843750, -0.0142211914062500,
+0.0092468261718750, -0.0122985839843750, 0.0092773437500000, -0.0119934082031250,
+0.0096435546875000, -0.0122985839843750, 0.0097961425781250, -0.0116882324218750,
+0.0102233886718750, -0.0115356445312500, 0.0111083984375000, -0.0114746093750000,
+0.0110473632812500, -0.0121154785156250, 0.0117187500000000, -0.0106811523437500,
+0.0126342773437500, -0.0095520019531250, 0.0123596191406250, -0.0107116699218750,
+0.0131835937500000, -0.0071411132812500, 0.0136413574218750, -0.0077209472656250,
+0.0138549804687500, -0.0064086914062500, 0.0140380859375000, -0.0036621093750000,
+0.0144348144531250, -0.0054016113281250, 0.0162353515625000, -0.0011291503906250,
+0.0158691406250000, -0.0008544921875000, 0.0174560546875000, -0.0015258789062500,
+0.0197143554687500, 0.0004882812500000, 0.0193176269531250, 0.0007324218750000,
+0.0234069824218750, -0.0016479492187500, 0.0231018066406250, -0.0010681152343750,
+0.0247802734375000, -0.0012817382812500, 0.0283508300781250, -0.0027770996093750,
+0.0268554687500000, -0.0022888183593750, 0.0292053222656250, -0.0037536621093750,
+0.0303344726562500, -0.0037231445312500, 0.0292968750000000, -0.0032653808593750,
+0.0295410156250000, -0.0043640136718750, 0.0299682617187500, -0.0047912597656250,
+0.0261840820312500, -0.0016784667968750, 0.0274353027343750, -0.0019836425781250,
+0.0240173339843750, -0.0035095214843750, 0.0195007324218750, -0.0020446777343750,
+0.0220642089843750, -0.0011291503906250, 0.0120849609375000, -0.0062561035156250,
+0.0129394531250000, -0.0057373046875000, 0.0099182128906250, -0.0060729980468750,
+0.0024719238281250, -0.0090332031250000, 0.0061035156250000, -0.0083618164062500,
+0.0006103515625000, -0.0110473632812500, -0.0012817382812500, -0.0105895996093750,
+0.0011291503906250, -0.0108337402343750, 0.0009155273437500, -0.0135803222656250,
+-0.0001525878906250, -0.0129089355468750, 0.0042419433593750, -0.0096435546875000,
+0.0042724609375000, -0.0111999511718750, 0.0048522949218750, -0.0108337402343750,
+0.0076599121093750, -0.0071411132812500, 0.0069885253906250, -0.0077514648437500,
+0.0071105957031250, -0.0099487304687500, 0.0075988769531250, -0.0085144042968750,
+0.0061035156250000, -0.0105590820312500, 0.0049743652343750, -0.0142517089843750,
+0.0058288574218750, -0.0118408203125000, 0.0032043457031250, -0.0118103027343750,
+0.0026550292968750, -0.0151062011718750, 0.0036621093750000, -0.0109558105468750,
+0.0031433105468750, -0.0062866210937500, 0.0025634765625000, -0.0097045898437500,
+0.0055847167968750, -0.0014953613281250, 0.0051879882812500, -0.0010681152343750,
+0.0059814453125000, -0.0023193359375000, 0.0084228515625000, 0.0007324218750000,
+0.0074157714843750, 0.0007934570312500, 0.0079956054687500, 0.0005493164062500,
+0.0096130371093750, 0.0003662109375000, 0.0068664550781250, 0.0000000000000000,
+0.0046691894531250, -0.0003662109375000, 0.0064697265625000, -0.0001831054687500,
+-0.0013732910156250, 0.0009460449218750, 0.0005187988281250, 0.0010070800781250,
+-0.0032348632812500, -0.0004272460937500, -0.0101928710937500, -0.0008239746093750,
+-0.0061645507812500, 0.0003662109375000, -0.0179748535156250, -0.0017395019531250,
+-0.0175476074218750, -0.0018310546875000, -0.0172424316406250, -0.0017700195312500,
+-0.0244140625000000, -0.0028686523437500, -0.0226745605468750, -0.0028076171875000,
+-0.0174560546875000, -0.0026855468750000, -0.0208740234375000, -0.0026550292968750,
+-0.0165710449218750, -0.0026855468750000, -0.0075378417968750, -0.0027465820312500,
+-0.0087280273437500, -0.0028381347656250, -0.0013427734375000, -0.0026550292968750,
+-0.0021972656250000, -0.0027465820312500, -0.0039367675781250, -0.0028686523437500,
+-0.0000915527343750, -0.0022583007812500, -0.0000305175781250, -0.0022888183593750,
+-0.0042724609375000, -0.0058288574218750, -0.0030517578125000, -0.0036621093750000,
+-0.0040283203125000, -0.0070800781250000, -0.0075988769531250, -0.0120849609375000,
+-0.0060729980468750, -0.0086975097656250, -0.0063781738281250, -0.0171508789062500,
+-0.0080871582031250, -0.0179443359375000, -0.0051879882812500, -0.0163574218750000,
+-0.0026550292968750, -0.0194091796875000, -0.0050659179687500, -0.0192565917968750,
+0.0007324218750000, -0.0152893066406250, 0.0006713867187500, -0.0168151855468750,
+0.0010375976562500, -0.0138244628906250, 0.0046997070312500, -0.0092163085937500,
+0.0032958984375000, -0.0122985839843750, 0.0015258789062500, -0.0061950683593750,
+0.0034484863281250, -0.0058898925781250, 0.0006713867187500, -0.0054321289062500,
+-0.0032043457031250, -0.0023193359375000, -0.0005798339843750, -0.0034484863281250,
+-0.0049743652343750, -0.0015563964843750, -0.0054016113281250, -0.0009765625000000,
+-0.0059814453125000, -0.0013427734375000, -0.0083312988281250, -0.0008544921875000,
+-0.0066528320312500, -0.0005798339843750, -0.0072326660156250, -0.0010070800781250,
+-0.0094909667968750, -0.0014343261718750, -0.0070190429687500, -0.0000305175781250,
+-0.0039062500000000, 0.0007019042968750, -0.0060424804687500, -0.0002441406250000,
+-0.0027465820312500, 0.0039062500000000, -0.0015258789062500, 0.0030822753906250,
+-0.0049133300781250},
+{0.0245666503906250, -0.0124816894531250, 0.0201721191406250, -0.0104064941406250,
+0.0142822265625000, -0.0105590820312500, 0.0176696777343750, -0.0120239257812500,
+0.0086059570312500, -0.0093994140625000, 0.0078735351562500, -0.0081176757812500,
+0.0075988769531250, -0.0100402832031250, 0.0028686523437500, -0.0106811523437500,
+0.0038452148437500, -0.0095825195312500, 0.0039672851562500, -0.0136413574218750,
+0.0024414062500000, -0.0136718750000000, 0.0050659179687500, -0.0121154785156250,
+0.0075378417968750, -0.0142211914062500, 0.0053710937500000, -0.0151672363281250,
+0.0106811523437500, -0.0080871582031250, 0.0108947753906250, -0.0095520019531250,
+0.0102233886718750, -0.0081176757812500, 0.0126342773437500, -0.0024719238281250,
+0.0122680664062500, -0.0047607421875000, 0.0099792480468750, -0.0036315917968750,
+0.0105895996093750, -0.0024719238281250, 0.0098876953125000, -0.0021362304687500,
+0.0079956054687500, -0.0023498535156250, 0.0089721679687500, -0.0033264160156250,
+0.0080261230468750, -0.0027160644531250, 0.0078430175781250, -0.0021667480468750,
+0.0075073242187500, -0.0018920898437500, 0.0074462890625000, -0.0017089843750000,
+0.0081787109375000, -0.0022583007812500, 0.0052490234375000, -0.0042419433593750,
+0.0062866210937500, -0.0031433105468750, 0.0043029785156250, -0.0026855468750000,
+0.0014038085937500, -0.0046997070312500, 0.0033874511718750, -0.0046997070312500,
+-0.0028991699218750, -0.0021667480468750, -0.0014648437500000, -0.0029907226562500,
+-0.0054016113281250, -0.0021667480468750, -0.0115966796875000, 0.0003356933593750,
+-0.0074768066406250, -0.0006713867187500, -0.0176696777343750, 0.0003662109375000,
+-0.0178833007812500, 0.0007934570312500, -0.0168457031250000, 0.0000915527343750,
+-0.0226135253906250, 0.0002136230468750, -0.0213623046875000, -0.0001831054687500,
+-0.0123596191406250, -0.0046081542968750, -0.0178222656250000, -0.0015563964843750,
+-0.0136413574218750, -0.0051574707031250, -0.0035400390625000, -0.0109252929687500,
+-0.0102539062500000, -0.0066833496093750, -0.0013732910156250, -0.0171813964843750,
+0.0014038085937500, -0.0178833007812500, -0.0021667480468750, -0.0167541503906250,
+-0.0018005371093750, -0.0217285156250000, -0.0000915527343750, -0.0208740234375000,
+-0.0035400390625000, -0.0148620605468750, -0.0044555664062500, -0.0171508789062500,
+-0.0033264160156250, -0.0143127441406250, -0.0037841796875000, -0.0078430175781250,
+-0.0041503906250000, -0.0102844238281250, -0.0018615722656250, -0.0046081542968750,
+-0.0021972656250000, -0.0043640136718750, -0.0015563964843750, -0.0046081542968750,
+-0.0001831054687500, -0.0018615722656250, -0.0011901855468750, -0.0021972656250000,
+0.0014953613281250, -0.0016784667968750, 0.0017089843750000, -0.0014953613281250,
+0.0006408691406250, -0.0011596679687500, 0.0018615722656250, -0.0008239746093750,
+0.0020446777343750, -0.0008544921875000, -0.0036926269531250, 0.0004577636718750,
+-0.0016174316406250, -0.0000915527343750, -0.0038146972656250, 0.0013732910156250,
+-0.0096740722656250, 0.0030822753906250, -0.0072326660156250, 0.0020751953125000,
+-0.0112304687500000, 0.0071105957031250, -0.0126342773437500, 0.0071105957031250,
+-0.0097351074218750, 0.0072021484375000, -0.0096435546875000, 0.0100402832031250,
+-0.0117187500000000, 0.0096130371093750, -0.0054931640625000, 0.0094299316406250,
+-0.0057067871093750, 0.0097351074218750, -0.0055541992187500, 0.0092773437500000,
+-0.0017395019531250, 0.0086669921875000, -0.0032348632812500, 0.0091552734375000,
+-0.0056762695312500, 0.0092163085937500, -0.0031738281250000, 0.0080566406250000,
+-0.0050048828125000, 0.0103149414062500, -0.0093383789062500, 0.0122680664062500,
+-0.0072631835937500, 0.0105590820312500, -0.0090026855468750, 0.0169067382812500,
+-0.0093688964843750, 0.0163879394531250, -0.0090942382812500, 0.0177612304687500,
+-0.0097656250000000, 0.0226745605468750, -0.0093688964843750, 0.0209960937500000,
+-0.0084228515625000, 0.0220031738281250, -0.0086059570312500, 0.0235290527343750,
+-0.0090026855468750, 0.0207214355468750, -0.0085449218750000, 0.0184631347656250,
+-0.0079650878906250, 0.0197143554687500, -0.0088806152343750, 0.0126647949218750,
+-0.0089111328125000, 0.0130004882812500, -0.0088195800781250, 0.0110473632812500,
+-0.0090942382812500, 0.0058593750000000, -0.0087585449218750, 0.0076293945312500,
+-0.0089111328125000, 0.0043640136718750, -0.0093994140625000, 0.0032043457031250,
+-0.0080871582031250, 0.0039062500000000, -0.0070190429687500, 0.0029907226562500,
+-0.0081787109375000, 0.0025939941406250, -0.0059814453125000, 0.0053405761718750,
+-0.0053710937500000, 0.0044555664062500, -0.0062866210937500, 0.0052795410156250,
+-0.0061950683593750, 0.0076599121093750, -0.0056762695312500, 0.0063171386718750,
+-0.0066528320312500, 0.0076293945312500, -0.0069885253906250, 0.0079345703125000,
+-0.0064697265625000, 0.0071716308593750, -0.0064086914062500, 0.0070190429687500,
+-0.0068054199218750, 0.0069885253906250, -0.0056457519531250, 0.0059509277343750,
+-0.0058288574218750, 0.0055236816406250, -0.0057373046875000, 0.0044860839843750,
+-0.0049133300781250, 0.0031738281250000, -0.0052795410156250, 0.0030212402343750,
+-0.0052185058593750, 0.0017395019531250, -0.0049133300781250, 0.0009460449218750,
+-0.0055236816406250, 0.0000610351562500, -0.0060424804687500, -0.0010070800781250,
+-0.0056152343750000, -0.0014953613281250, -0.0068664550781250, -0.0032653808593750,
+-0.0068359375000000, -0.0045471191406250, -0.0069885253906250, -0.0048828125000000,
+-0.0077819824218750, -0.0056152343750000, -0.0073547363281250, -0.0066833496093750,
+-0.0073852539062500, -0.0062255859375000, -0.0076904296875000, -0.0071411132812500,
+-0.0072631835937500, -0.0071411132812500, -0.0068359375000000, -0.0067138671875000,
+-0.0070800781250000, -0.0078430175781250, -0.0063781738281250, -0.0065612792968750,
+-0.0063171386718750, -0.0077819824218750, -0.0063781738281250, -0.0068969726562500,
+-0.0062255859375000, -0.0052795410156250, -0.0062255859375000, -0.0071716308593750,
+-0.0058898925781250, -0.0035095214843750, -0.0063781738281250, -0.0045776367187500,
+-0.0055847167968750, -0.0036621093750000, -0.0046997070312500, -0.0010070800781250,
+-0.0055847167968750, -0.0031433105468750, -0.0038757324218750, 0.0012512207031250,
+-0.0038146972656250, 0.0005187988281250, -0.0045166015625000, 0.0006713867187500,
+-0.0042419433593750, 0.0035095214843750, -0.0042419433593750, 0.0022277832031250,
+-0.0055847167968750, 0.0035400390625000, -0.0057067871093750, 0.0040893554687500,
+-0.0055541992187500, 0.0033569335937500, -0.0061340332031250, 0.0031738281250000,
+-0.0062866210937500, 0.0034179687500000, -0.0055541992187500, 0.0025939941406250,
+-0.0055236816406250, 0.0019531250000000, -0.0057983398437500, 0.0030517578125000,
+-0.0055236816406250, 0.0032043457031250, -0.0053100585937500, 0.0021362304687500,
+-0.0065307617187500, 0.0059814453125000, -0.0060729980468750, 0.0041503906250000,
+-0.0065917968750000, 0.0071105957031250, -0.0075988769531250, 0.0116577148437500,
+-0.0068969726562500, 0.0081787109375000, -0.0084533691406250, 0.0164184570312500,
+-0.0081481933593750, 0.0158386230468750, -0.0086364746093750, 0.0169372558593750,
+-0.0098266601562500, 0.0223388671875000, -0.0089721679687500, 0.0197753906250000,
+-0.0106506347656250, 0.0218200683593750, -0.0106506347656250, 0.0231323242187500,
+-0.0105285644531250, 0.0211181640625000, -0.0114440917968750, 0.0208129882812500,
+-0.0111999511718750, 0.0216369628906250, -0.0105285644531250, 0.0142822265625000,
+-0.0109252929687500, 0.0167846679687500, -0.0104980468750000, 0.0116882324218750,
+-0.0098571777343750, 0.0029907226562500, -0.0101013183593750, 0.0077819824218750,
+-0.0082397460937500, -0.0016784667968750, -0.0093383789062500, -0.0034484863281250,
+-0.0069885253906250, -0.0014343261718750, -0.0039978027343750, -0.0044250488281250,
+-0.0063476562500000, -0.0045776367187500, -0.0009155273437500, 0.0010070800781250,
+0.0002136230468750, -0.0011291503906250, -0.0025939941406250, 0.0021057128906250,
+-0.0021972656250000, 0.0072937011718750, -0.0007324218750000, 0.0034484863281250,
+-0.0062255859375000, 0.0133972167968750, -0.0059509277343750, 0.0121765136718750,
+-0.0078430175781250, 0.0135192871093750, -0.0118103027343750, 0.0209655761718750,
+-0.0094299316406250, 0.0180053710937500, -0.0130004882812500, 0.0180053710937500,
+-0.0145874023437500, 0.0207214355468750, -0.0129699707031250, 0.0170593261718750,
+-0.0126342773437500, 0.0135498046875000, -0.0131530761718750, 0.0162353515625000,
+-0.0099182128906250, 0.0069274902343750, -0.0116271972656250, 0.0077514648437500,
+-0.0082397460937500, 0.0056457519531250, -0.0033569335937500, -0.0007324218750000,
+-0.0065307617187500, 0.0024414062500000, 0.0003662109375000, -0.0020446777343750,
+0.0012512207031250, -0.0042724609375000, -0.0003051757812500, -0.0013427734375000,
+0.0029296875000000, -0.0008544921875000, 0.0028381347656250, -0.0020141601562500,
+-0.0064086914062500, 0.0072021484375000, -0.0022583007812500, 0.0048522949218750,
+-0.0076599121093750, 0.0068664550781250, -0.0192260742187500, 0.0159606933593750,
+-0.0136108398437500, 0.0132446289062500, -0.0199584960937500, 0.0101928710937500,
+-0.0227355957031250, 0.0147399902343750, -0.0194396972656250, 0.0081787109375000,
+-0.0177307128906250, -0.0074768066406250, -0.0023498535156250, 0.0111694335937500,
+0.0138244628906250, 0.0096740722656250, 0.0112609863281250, 0.0182800292968750,
+0.0116577148437500, 0.0087890625000000, 0.0179443359375000, -0.0026855468750000,
+0.0170593261718750, 0.0040893554687500, 0.0086669921875000, -0.0076599121093750,
+0.0144958496093750, -0.0098876953125000, 0.0064086914062500, -0.0076904296875000,
+-0.0062561035156250, -0.0103759765625000, 0.0023193359375000, -0.0102844238281250,
+-0.0142822265625000, -0.0103759765625000, -0.0174865722656250, -0.0099182128906250,
+-0.0114440917968750, -0.0094604492187500, -0.0155944824218750, -0.0098876953125000,
+-0.0176086425781250, -0.0099487304687500, -0.0006103515625000, -0.0078735351562500,
+-0.0046386718750000, -0.0087280273437500, 0.0001220703125000, -0.0075683593750000,
+0.0151062011718750, -0.0044250488281250, 0.0088195800781250, -0.0059509277343750,
+0.0144653320312500, -0.0081787109375000, 0.0173645019531250, -0.0056152343750000,
+0.0148315429687500, -0.0087890625000000, 0.0144348144531250, -0.0140991210937500,
+0.0154418945312500, -0.0112609863281250, 0.0122985839843750, -0.0144653320312500,
+0.0125427246093750, -0.0176391601562500, 0.0118103027343750, -0.0119018554687500,
+0.0100708007812500, -0.0083618164062500, 0.0108337402343750, -0.0128173828125000,
+0.0070800781250000, -0.0010375976562500, 0.0084228515625000, -0.0004577636718750,
+0.0049438476562500, -0.0012817382812500, 0.0003967285156250, 0.0040283203125000,
+0.0042114257812500, 0.0028076171875000, -0.0034790039062500, 0.0000000000000000,
+-0.0047912597656250, 0.0014038085937500, -0.0024719238281250, 0.0003967285156250,
+-0.0043640136718750, -0.0029296875000000, -0.0048522949218750, -0.0020141601562500,
+0.0001525878906250, -0.0012512207031250, -0.0023193359375000, -0.0016479492187500,
+0.0019531250000000, -0.0020446777343750, 0.0078125000000000, -0.0015869140625000,
+0.0032043457031250, -0.0011901855468750, 0.0153808593750000, -0.0020446777343750,
+0.0144042968750000, -0.0026245117187500, 0.0155639648437500, -0.0019226074218750,
+0.0230407714843750, -0.0010986328125000, 0.0198974609375000, -0.0016784667968750,
+0.0238647460937500, -0.0023498535156250, 0.0243225097656250, -0.0011596679687500,
+0.0243530273437500, -0.0029296875000000, 0.0261230468750000, -0.0052490234375000,
+0.0252380371093750},
+{-0.0120239257812500, -0.0025329589843750, -0.0101928710937500, -0.0055236816406250,
+-0.0104980468750000, 0.0045471191406250, -0.0120544433593750, 0.0039367675781250,
+-0.0102844238281250, 0.0051574707031250, -0.0084533691406250, 0.0114746093750000,
+-0.0098266601562500, 0.0092468261718750, -0.0063171386718750, 0.0123596191406250,
+-0.0054016113281250, 0.0135192871093750, -0.0075073242187500, 0.0121765136718750,
+-0.0071105957031250, 0.0121765136718750, -0.0058593750000000, 0.0122375488281250,
+-0.0121154785156250, 0.0094909667968750, -0.0104370117187500, 0.0107116699218750,
+-0.0129394531250000, 0.0086059570312500, -0.0186767578125000, 0.0054626464843750,
+-0.0153808593750000, 0.0069274902343750, -0.0207519531250000, -0.0005798339843750,
+-0.0226745605468750, 0.0002136230468750, -0.0189514160156250, 0.0011901855468750,
+-0.0188903808593750, -0.0041198730468750, -0.0207824707031250, -0.0037231445312500,
+-0.0099182128906250, 0.0042114257812500, -0.0127258300781250, 0.0012207031250000,
+-0.0086059570312500, 0.0048522949218750, 0.0083007812500000, 0.0188598632812500,
+0.0136108398437500, -0.0021362304687500, 0.0010375976562500, 0.0049133300781250,
+0.0097961425781250, -0.0009765625000000, 0.0010986328125000, 0.0032043457031250,
+-0.0116882324218750, 0.0120849609375000, -0.0040588378906250, 0.0092163085937500,
+-0.0124206542968750, 0.0083923339843750, -0.0173950195312500, 0.0123901367187500,
+-0.0099182128906250, 0.0063781738281250, -0.0076599121093750, -0.0003356933593750,
+-0.0124206542968750, 0.0043334960937500, 0.0071411132812500, -0.0065917968750000,
+0.0048828125000000, -0.0063171386718750, 0.0074462890625000, -0.0072021484375000,
+0.0212097167968750, -0.0135498046875000, 0.0161437988281250, -0.0109863281250000,
+0.0172729492187500, -0.0130920410156250, 0.0215759277343750, -0.0136108398437500,
+0.0165100097656250, -0.0132751464843750, 0.0110778808593750, -0.0143432617187500,
+0.0150146484375000, -0.0140075683593750, 0.0081787109375000, -0.0118408203125000,
+0.0077209472656250, -0.0126037597656250, 0.0069580078125000, -0.0119934082031250,
+0.0037841796875000, -0.0098571777343750, 0.0054931640625000, -0.0105895996093750,
+0.0005187988281250, -0.0105285644531250, 0.0014038085937500, -0.0098266601562500,
+-0.0003662109375000, -0.0106201171875000, -0.0044555664062500, -0.0124511718750000,
+-0.0020446777343750, -0.0117187500000000, -0.0061950683593750, -0.0075988769531250,
+-0.0074462890625000, -0.0104370117187500, -0.0043640136718750, -0.0067138671875000,
+-0.0046691894531250, -0.0000610351562500, -0.0068969726562500, -0.0038452148437500,
+0.0019226074218750, 0.0018310546875000, 0.0004577636718750, 0.0037536621093750,
+0.0024719238281250, 0.0006713867187500, 0.0083923339843750, 0.0005187988281250,
+0.0053710937500000, 0.0022277832031250, 0.0131835937500000, -0.0028381347656250,
+0.0120544433593750, -0.0039062500000000, 0.0140380859375000, -0.0018920898437500,
+0.0203247070312500, -0.0022888183593750, 0.0179138183593750, -0.0033569335937500,
+0.0220947265625000, -0.0005798339843750, 0.0225830078125000, -0.0000305175781250,
+0.0228576660156250, -0.0005493164062500, 0.0249023437500000, -0.0001525878906250,
+0.0237426757812500, -0.0002136230468750, 0.0241394042968750, -0.0003967285156250,
+0.0257263183593750, -0.0002746582031250, 0.0229492187500000, -0.0008850097656250,
+0.0215148925781250, -0.0010986328125000, 0.0238342285156250, -0.0006103515625000,
+0.0140075683593750, -0.0032348632812500, 0.0173034667968750, -0.0025634765625000,
+0.0123291015625000, -0.0035400390625000, 0.0036926269531250, -0.0060119628906250,
+0.0101013183593750, -0.0046997070312500, -0.0073852539062500, -0.0060119628906250,
+-0.0037536621093750, -0.0065612792968750, -0.0086975097656250, -0.0056457519531250,
+-0.0221557617187500, -0.0053100585937500, -0.0138549804687500, -0.0056457519531250,
+-0.0278930664062500, -0.0045776367187500, -0.0275268554687500, -0.0043334960937500,
+-0.0271911621093750, -0.0047302246093750, -0.0344238281250000, -0.0045471191406250,
+-0.0311279296875000, -0.0045166015625000, -0.0321655273437500, -0.0061645507812500,
+-0.0339965820312500, -0.0055847167968750, -0.0311889648437500, -0.0070495605468750,
+-0.0301208496093750, -0.0091247558593750, -0.0329589843750000, -0.0079040527343750,
+-0.0252380371093750, -0.0112915039062500, -0.0281982421875000, -0.0115661621093750,
+-0.0249328613281250, -0.0113830566406250, -0.0182800292968750, -0.0127868652343750,
+-0.0237426757812500, -0.0125427246093750, -0.0114746093750000, -0.0124511718750000,
+-0.0149841308593750, -0.0123901367187500, -0.0121765136718750, -0.0126647949218750,
+-0.0028991699218750, -0.0128173828125000, -0.0093383789062500, -0.0124511718750000,
+0.0010375976562500, -0.0135192871093750, -0.0009460449218750, -0.0133666992187500,
+0.0004882812500000, -0.0134887695312500, 0.0069274902343750, -0.0141296386718750,
+0.0015258789062500, -0.0138854980468750, 0.0092468261718750, -0.0139160156250000,
+0.0072937011718750, -0.0143432617187500, 0.0080871582031250, -0.0136718750000000,
+0.0126953125000000, -0.0132141113281250, 0.0081787109375000, -0.0138244628906250,
+0.0144042968750000, -0.0117797851562500, 0.0125427246093750, -0.0122985839843750,
+0.0126037597656250, -0.0115051269531250, 0.0162048339843750, -0.0097045898437500,
+0.0122680664062500, -0.0107421875000000, 0.0146179199218750, -0.0083923339843750,
+0.0133056640625000, -0.0080871582031250, 0.0118103027343750, -0.0083007812500000,
+0.0123291015625000, -0.0072021484375000, 0.0099792480468750, -0.0070495605468750,
+0.0083007812500000, -0.0070800781250000, 0.0084228515625000, -0.0075073242187500,
+0.0056457519531250, -0.0060424804687500, 0.0029296875000000, -0.0049438476562500,
+0.0031433105468750, -0.0060729980468750, -0.0032653808593750, -0.0030212402343750,
+-0.0028076171875000, -0.0028076171875000, -0.0061035156250000, -0.0034484863281250,
+-0.0119018554687500, -0.0025329589843750, -0.0104064941406250, -0.0024719238281250,
+-0.0184020996093750, -0.0030517578125000, -0.0182800292968750, -0.0032348632812500,
+-0.0202331542968750, -0.0029907226562500, -0.0259094238281250, -0.0028686523437500,
+-0.0240173339843750, -0.0030822753906250, -0.0266418457031250, -0.0032653808593750,
+-0.0278625488281250, -0.0029602050781250, -0.0265197753906250, -0.0031433105468750,
+-0.0267639160156250, -0.0035400390625000, -0.0273132324218750, -0.0033264160156250,
+-0.0229492187500000, -0.0044250488281250, -0.0241088867187500, -0.0035400390625000,
+-0.0216369628906250, -0.0055236816406250, -0.0170898437500000, -0.0078735351562500,
+-0.0192260742187500, -0.0057678222656250, -0.0139465332031250, -0.0092773437500000,
+-0.0137023925781250, -0.0103759765625000, -0.0133361816406250, -0.0081787109375000,
+-0.0106201171875000, -0.0079650878906250, -0.0117492675781250, -0.0093078613281250,
+-0.0104064941406250, -0.0059204101562500, -0.0107421875000000, -0.0051574707031250,
+-0.0088195800781250, -0.0063476562500000, -0.0074462890625000, -0.0057678222656250,
+-0.0090637207031250, -0.0053100585937500, -0.0005493164062500, -0.0081787109375000,
+-0.0027770996093750, -0.0075988769531250, 0.0020141601562500, -0.0083312988281250,
+0.0106506347656250, -0.0112609863281250, 0.0061950683593750, -0.0104370117187500,
+0.0171508789062500, -0.0089416503906250, 0.0181274414062500, -0.0103149414062500,
+0.0172424316406250, -0.0092163085937500, 0.0223388671875000, -0.0069885253906250,
+0.0211486816406250, -0.0081787109375000, 0.0165405273437500, -0.0055847167968750,
+0.0188903808593750, -0.0055541992187500, 0.0150451660156250, -0.0056152343750000,
+0.0085449218750000, -0.0045166015625000, 0.0117492675781250, -0.0053710937500000,
+0.0058593750000000, -0.0057067871093750, 0.0043945312500000, -0.0044860839843750,
+0.0053100585937500, -0.0057678222656250, 0.0038146972656250, -0.0079956054687500,
+0.0037231445312500, -0.0072021484375000, 0.0046691894531250, -0.0078125000000000,
+0.0042419433593750, -0.0079650878906250, 0.0048217773437500, -0.0068969726562500,
+0.0059814453125000, -0.0073547363281250, 0.0053100585937500, -0.0083312988281250,
+0.0055236816406250, -0.0038452148437500, 0.0062561035156250, -0.0041809082031250,
+0.0053100585937500, -0.0042419433593750, 0.0039978027343750, -0.0016174316406250,
+0.0046997070312500, -0.0025634765625000, 0.0043640136718750, -0.0043029785156250,
+0.0039672851562500, -0.0021667480468750, 0.0043029785156250, -0.0058288574218750,
+0.0047302246093750, -0.0097351074218750, 0.0046997070312500, -0.0063781738281250,
+0.0050964355468750, -0.0160217285156250, 0.0048828125000000, -0.0158996582031250,
+0.0054931640625000, -0.0155029296875000, 0.0072326660156250, -0.0214843750000000,
+0.0069580078125000, -0.0201110839843750, 0.0035400390625000, -0.0127258300781250,
+0.0068664550781250, -0.0175476074218750, 0.0027160644531250, -0.0137023925781250,
+-0.0021667480468750, -0.0048522949218750, 0.0033569335937500, -0.0096435546875000,
+-0.0102844238281250, 0.0003051757812500, -0.0098876953125000, 0.0016784667968750,
+-0.0086059570312500, 0.0002746582031250, -0.0153198242187500, 0.0038757324218750,
+-0.0139465332031250, 0.0039367675781250, -0.0115356445312500, 0.0021057128906250,
+-0.0128479003906250, 0.0022583007812500, -0.0119018554687500, 0.0024414062500000,
+-0.0100708007812500, 0.0014648437500000, -0.0114440917968750, 0.0017395019531250,
+-0.0073547363281250, 0.0031738281250000, -0.0084838867187500, 0.0025634765625000,
+-0.0067138671875000, 0.0033569335937500, -0.0029907226562500, 0.0050354003906250,
+-0.0054321289062500, 0.0043640136718750, -0.0012817382812500, 0.0058288574218750,
+-0.0007934570312500, 0.0062561035156250, -0.0014953613281250, 0.0052490234375000,
+0.0000305175781250, 0.0055236816406250, -0.0000610351562500, 0.0063476562500000,
+-0.0015869140625000, 0.0031738281250000, -0.0014953613281250, 0.0035400390625000,
+-0.0013122558593750, 0.0036315917968750, -0.0017395019531250, 0.0010375976562500,
+-0.0019836425781250, 0.0018310546875000, -0.0034179687500000, 0.0069274902343750,
+-0.0021057128906250, 0.0041503906250000, -0.0046386718750000, 0.0079040527343750,
+-0.0077819824218750, 0.0151062011718750, -0.0051879882812500, 0.0114135742187500,
+-0.0093994140625000, 0.0169982910156250, -0.0112609863281250, 0.0191650390625000,
+-0.0074768066406250, 0.0159301757812500, -0.0065002441406250, 0.0158081054687500,
+-0.0090026855468750, 0.0172424316406250, -0.0012512207031250, 0.0092468261718750,
+-0.0019836425781250, 0.0100708007812500, -0.0010681152343750, 0.0074157714843750,
+0.0045166015625000, 0.0008544921875000, 0.0024108886718750, 0.0032958984375000,
+0.0008544921875000, -0.0004577636718750, 0.0036621093750000, -0.0021667480468750,
+0.0020446777343750, -0.0014038085937500, -0.0017395019531250, -0.0018920898437500,
+0.0000305175781250, -0.0028686523437500, -0.0015563964843750, -0.0026550292968750,
+-0.0014648437500000, -0.0025634765625000, -0.0020141601562500, -0.0048522949218750,
+-0.0031127929687500, -0.0062866210937500, -0.0021057128906250, -0.0058288574218750,
+-0.0045776367187500, -0.0112609863281250, -0.0038757324218750, -0.0107727050781250,
+-0.0051574707031250, -0.0129089355468750, -0.0076599121093750, -0.0175781250000000,
+-0.0057983398437500, -0.0157775878906250, -0.0081787109375000, -0.0193786621093750,
+-0.0087585449218750, -0.0203552246093750, -0.0082092285156250, -0.0193786621093750,
+-0.0085144042968750, -0.0199584960937500, -0.0085144042968750, -0.0195922851562500,
+-0.0095214843750000, -0.0158081054687500, -0.0092163085937500, -0.0162048339843750,
+-0.0090942382812500, -0.0131225585937500, -0.0099182128906250, -0.0086059570312500,
+-0.0098571777343750, -0.0094299316406250, -0.0088500976562500, -0.0034790039062500,
+-0.0090026855468750},
+{0.0030517578125000, -0.0162353515625000, 0.0031738281250000, -0.0160522460937500,
+0.0034484863281250, -0.0157470703125000, 0.0033264160156250, -0.0208129882812500,
+0.0030822753906250, -0.0197448730468750, 0.0041809082031250, -0.0141296386718750,
+0.0040588378906250, -0.0173339843750000, 0.0043640136718750, -0.0130004882812500,
+0.0048828125000000, -0.0034790039062500, 0.0048828125000000, -0.0041503906250000,
+0.0079650878906250, 0.0038452148437500, 0.0067749023437500, 0.0028381347656250,
+0.0072326660156250, 0.0017700195312500, 0.0101623535156250, 0.0062561035156250,
+0.0085449218750000, 0.0056152343750000, 0.0071411132812500, 0.0033264160156250,
+0.0106811523437500, 0.0043029785156250, 0.0052490234375000, 0.0036315917968750,
+-0.0003051757812500, 0.0012207031250000, 0.0050964355468750, 0.0020446777343750,
+-0.0074157714843750, 0.0023193359375000, -0.0062561035156250, 0.0015869140625000,
+-0.0078430175781250, 0.0024108886718750, -0.0161437988281250, 0.0034790039062500,
+-0.0122985839843750, 0.0027160644531250, -0.0155639648437500, 0.0037841796875000,
+-0.0184020996093750, 0.0041198730468750, -0.0146789550781250, 0.0037841796875000,
+-0.0127258300781250, 0.0038757324218750, -0.0155029296875000, 0.0039062500000000,
+-0.0072631835937500, 0.0032043457031250, -0.0085754394531250, 0.0032043457031250,
+-0.0069580078125000, 0.0039062500000000, -0.0011596679687500, 0.0032958984375000,
+-0.0043640136718750, 0.0027465820312500, 0.0003662109375000, 0.0077819824218750,
+0.0014953613281250, 0.0057373046875000, -0.0001220703125000, 0.0087890625000000,
+0.0007629394531250, 0.0151062011718750, 0.0012817382812500, 0.0116882324218750,
+-0.0010681152343750, 0.0148925781250000, -0.0007324218750000, 0.0180664062500000,
+-0.0013427734375000, 0.0131225585937500, -0.0027770996093750, 0.0100097656250000,
+-0.0020446777343750, 0.0133972167968750, -0.0048828125000000, 0.0018615722656250,
+-0.0046081542968750, 0.0025024414062500, -0.0037841796875000, 0.0010070800781250,
+-0.0056762695312500, -0.0065917968750000, -0.0058593750000000, -0.0040893554687500,
+-0.0017089843750000, -0.0064392089843750, -0.0030822753906250, -0.0075683593750000,
+-0.0011291503906250, -0.0070190429687500, 0.0028381347656250, -0.0068969726562500,
+0.0004882812500000, -0.0070495605468750, 0.0049133300781250, -0.0089416503906250,
+0.0059509277343750, -0.0080871582031250, 0.0035400390625000, -0.0103759765625000,
+0.0040588378906250, -0.0134887695312500, 0.0058288574218750, -0.0118408203125000,
+0.0011291503906250, -0.0157165527343750, 0.0014648437500000, -0.0164184570312500,
+0.0013122558593750, -0.0148925781250000, -0.0010070800781250, -0.0155639648437500,
+0.0000610351562500, -0.0151062011718750, -0.0023803710937500, -0.0093383789062500,
+-0.0018310546875000, -0.0105285644531250, -0.0028686523437500, -0.0056762695312500,
+-0.0048217773437500, 0.0010070800781250, -0.0034179687500000, -0.0019531250000000,
+-0.0080566406250000, 0.0089721679687500, -0.0072326660156250, 0.0092163085937500,
+-0.0075683593750000, 0.0127258300781250, -0.0107727050781250, 0.0210876464843750,
+-0.0096130371093750, 0.0191955566406250, -0.0096740722656250, 0.0261840820312500,
+-0.0100708007812500, 0.0276184082031250, -0.0102233886718750, 0.0282897949218750,
+-0.0099487304687500, 0.0317077636718750, -0.0096435546875000, 0.0316467285156250,
+-0.0108032226562500, 0.0327148437500000, -0.0108642578125000, 0.0333557128906250,
+-0.0107421875000000, 0.0335693359375000, -0.0112304687500000, 0.0337829589843750,
+-0.0113220214843750, 0.0335998535156250, -0.0111999511718750, 0.0335388183593750,
+-0.0111999511718750, 0.0332336425781250, -0.0112609863281250, 0.0335693359375000,
+-0.0112915039062500, 0.0339660644531250, -0.0112915039062500, 0.0336303710937500,
+-0.0110473632812500, 0.0336303710937500, -0.0113525390625000, 0.0337524414062500,
+-0.0107116699218750, 0.0337829589843750, -0.0101623535156250, 0.0337829589843750,
+-0.0108032226562500, 0.0338745117187500, -0.0089111328125000, 0.0339355468750000,
+-0.0092163085937500, 0.0339965820312500, -0.0087890625000000, 0.0340270996093750,
+-0.0075073242187500, 0.0341796875000000, -0.0080871582031250, 0.0340270996093750,
+-0.0067138671875000, 0.0334167480468750, -0.0067749023437500, 0.0336303710937500,
+-0.0065307617187500, 0.0331420898437500, -0.0055847167968750, 0.0324707031250000,
+-0.0059814453125000, 0.0329589843750000, -0.0058593750000000, 0.0318603515625000,
+-0.0053710937500000, 0.0317687988281250, -0.0061950683593750, 0.0308837890625000,
+-0.0068969726562500, 0.0299987792968750, -0.0062866210937500, 0.0303649902343750,
+-0.0077514648437500, 0.0271606445312500, -0.0077819824218750, 0.0279846191406250,
+-0.0077514648437500, 0.0270996093750000, -0.0082702636718750, 0.0245666503906250,
+-0.0081176757812500, 0.0263061523437500, -0.0086059570312500, 0.0243225097656250,
+-0.0083312988281250, 0.0242309570312500, -0.0087280273437500, 0.0243225097656250,
+-0.0095214843750000, 0.0236816406250000, -0.0091247558593750, 0.0238342285156250,
+-0.0096740722656250, 0.0223083496093750, -0.0100097656250000, 0.0229492187500000,
+-0.0098571777343750, 0.0210571289062500, -0.0099182128906250, 0.0194702148437500,
+-0.0100708007812500, 0.0212707519531250, -0.0095520019531250, 0.0136413574218750,
+-0.0099487304687500, 0.0160827636718750, -0.0092468261718750, 0.0112915039062500,
+-0.0085449218750000, 0.0038452148437500, -0.0091857910156250, 0.0089416503906250,
+-0.0066223144531250, -0.0055847167968750, -0.0070495605468750, -0.0021667480468750,
+-0.0060729980468750, -0.0074462890625000, -0.0041198730468750, -0.0200195312500000,
+-0.0051269531250000, -0.0128173828125000, -0.0017395019531250, -0.0263366699218750,
+-0.0024108886718750, -0.0268859863281250, -0.0012207031250000, -0.0253601074218750,
+0.0016174316406250, -0.0318908691406250, 0.0000305175781250, -0.0303955078125000,
+0.0027160644531250, -0.0253906250000000, 0.0028686523437500, -0.0291442871093750,
+0.0031738281250000, -0.0233459472656250, 0.0045166015625000, -0.0154113769531250,
+0.0039672851562500, -0.0205993652343750, 0.0061340332031250, -0.0067443847656250,
+0.0057983398437500, -0.0065002441406250, 0.0060119628906250, -0.0070190429687500,
+0.0079650878906250, 0.0000305175781250, 0.0073242187500000, -0.0015563964843750,
+0.0049438476562500, -0.0041503906250000, 0.0066833496093750, -0.0025939941406250,
+0.0049743652343750, -0.0029602050781250, 0.0014343261718750, -0.0064086914062500,
+0.0028991699218750, -0.0055847167968750, -0.0012207031250000, -0.0003051757812500,
+-0.0002441406250000, -0.0029296875000000, -0.0025329589843750, 0.0005187988281250,
+-0.0071105957031250, 0.0073242187500000, -0.0046081542968750, 0.0039367675781250,
+-0.0071105957031250, 0.0090942382812500, -0.0075683593750000, 0.0108642578125000,
+-0.0092773437500000, 0.0090332031250000, -0.0104370117187500, 0.0097961425781250,
+-0.0084533691406250, 0.0103454589843750, -0.0160522460937500, 0.0052185058593750,
+-0.0152282714843750, 0.0065917968750000, -0.0160217285156250, 0.0049438476562500,
+-0.0218811035156250, 0.0001525878906250, -0.0199584960937500, 0.0018615722656250,
+-0.0164489746093750, -0.0012512207031250, -0.0199584960937500, -0.0018005371093750,
+-0.0154113769531250, -0.0008850097656250, -0.0085144042968750, -0.0018310546875000,
+-0.0128479003906250, -0.0021057128906250, -0.0050048828125000, 0.0000000000000000,
+-0.0041503906250000, -0.0002746582031250, -0.0044860839843750, 0.0005798339843750,
+-0.0017089843750000, 0.0019531250000000, -0.0028381347656250, 0.0012817382812500,
+-0.0000305175781250, 0.0050659179687500, -0.0001220703125000, 0.0031127929687500,
+0.0002136230468750, 0.0071105957031250, 0.0031738281250000, 0.0129394531250000,
+0.0022583007812500, 0.0091857910156250, -0.0039978027343750, 0.0162353515625000,
+-0.0007019042968750, 0.0174865722656250, -0.0030517578125000, 0.0149841308593750,
+-0.0145568847656250, 0.0162658691406250, -0.0027160644531250, 0.0057983398437500,
+0.0156250000000000, 0.0165405273437500, 0.0113220214843750, 0.0158996582031250,
+0.0112915039062500, 0.0137329101562500, 0.0190734863281250, 0.0166931152343750,
+0.0190734863281250, 0.0179138183593750, 0.0096435546875000, 0.0046386718750000,
+0.0128173828125000, 0.0082397460937500, 0.0099182128906250, 0.0046997070312500,
+0.0008544921875000, -0.0079956054687500, 0.0046997070312500, -0.0033264160156250,
+0.0021057128906250, -0.0026855468750000, -0.0000305175781250, -0.0068359375000000,
+0.0012512207031250, -0.0021667480468750, 0.0023193359375000, 0.0046081542968750,
+0.0017089843750000, 0.0004577636718750, 0.0016784667968750, 0.0023193359375000,
+0.0025634765625000, 0.0061950683593750, 0.0015869140625000, 0.0019226074218750,
+-0.0003356933593750, -0.0025024414062500, 0.0007629394531250, 0.0004882812500000,
+0.0032348632812500, -0.0060424804687500, 0.0007019042968750, -0.0056762695312500,
+0.0030212402343750, -0.0071411132812500, 0.0073852539062500, -0.0112915039062500,
+0.0049133300781250, -0.0093383789062500, 0.0087890625000000, -0.0153808593750000,
+0.0089721679687500, -0.0144653320312500, 0.0088500976562500, -0.0155334472656250,
+0.0108337402343750, -0.0208435058593750, 0.0100708007812500, -0.0187988281250000,
+0.0093994140625000, -0.0163574218750000, 0.0101928710937500, -0.0192260742187500,
+0.0102844238281250, -0.0149536132812500, 0.0087890625000000, -0.0094604492187500,
+0.0086975097656250, -0.0129699707031250, 0.0121154785156250, -0.0043029785156250,
+0.0119934082031250, -0.0043334960937500, 0.0091247558593750, -0.0036010742187500,
+0.0106506347656250, 0.0008850097656250, 0.0121154785156250, -0.0009460449218750,
+-0.0014038085937500, 0.0036010742187500, 0.0036315917968750, 0.0033874511718750,
+-0.0036926269531250, 0.0029907226562500, -0.0175476074218750, 0.0060424804687500,
+-0.0089111328125000, 0.0055847167968750, -0.0263671875000000, 0.0022277832031250,
+-0.0267333984375000, 0.0033264160156250, -0.0254821777343750, 0.0022888183593750,
+-0.0342407226562500, -0.0010375976562500, -0.0316162109375000, 0.0000610351562500,
+-0.0272521972656250, -0.0002746582031250, -0.0315856933593750, -0.0010375976562500,
+-0.0258178710937500, -0.0003356933593750, -0.0185241699218750, 0.0005187988281250,
+-0.0239868164062500, 0.0000915527343750, -0.0082092285156250, 0.0003051757812500,
+-0.0112609863281250, 0.0007934570312500, -0.0060729980468750, 0.0002746582031250,
+0.0063781738281250, -0.0003051757812500, -0.0010986328125000, 0.0001220703125000,
+0.0138244628906250, -0.0005493164062500, 0.0136108398437500, -0.0005187988281250,
+0.0134887695312500, -0.0010986328125000, 0.0211486816406250, -0.0015258789062500,
+0.0181884765625000, -0.0010070800781250, 0.0202941894531250, -0.0036926269531250,
+0.0211486816406250, -0.0032958984375000, 0.0202636718750000, -0.0037841796875000,
+0.0202636718750000, -0.0059204101562500, 0.0203247070312500, -0.0051879882812500,
+0.0202026367187500, -0.0056762695312500, 0.0196228027343750, -0.0059509277343750,
+0.0208129882812500, -0.0057373046875000, 0.0216369628906250, -0.0057067871093750,
+0.0209350585937500, -0.0057067871093750, 0.0246887207031250, -0.0054321289062500,
+0.0234375000000000, -0.0054626464843750, 0.0250244140625000, -0.0053710937500000,
+0.0281372070312500, -0.0050354003906250, 0.0258178710937500, -0.0050659179687500,
+0.0313415527343750, -0.0058593750000000, 0.0299072265625000, -0.0055541992187500,
+0.0314331054687500, -0.0061035156250000, 0.0359191894531250, -0.0073242187500000,
+0.0332031250000000, -0.0068969726562500, 0.0384521484375000, -0.0071105957031250,
+0.0374145507812500, -0.0076599121093750, 0.0381469726562500, -0.0071716308593750,
+0.0415344238281250},
+{-0.0022888183593750, 0.0101013183593750, 0.0016479492187500, 0.0141906738281250,
+-0.0008850097656250, 0.0156860351562500, 0.0019226074218750, 0.0023803710937500,
+0.0031127929687500, 0.0046386718750000, 0.0024108886718750, 0.0028686523437500,
+0.0019531250000000, -0.0072631835937500, 0.0021057128906250, -0.0037536621093750,
+0.0049743652343750, -0.0014038085937500, 0.0025939941406250, -0.0043640136718750,
+0.0065917968750000, -0.0026855468750000, 0.0121459960937500, 0.0021972656250000,
+0.0078735351562500, 0.0009460449218750, 0.0136413574218750, 0.0005493164062500,
+0.0156860351562500, 0.0020446777343750, 0.0133056640625000, 0.0002746582031250,
+0.0134582519531250, -0.0015563964843750, 0.0146179199218750, 0.0001525878906250,
+0.0114440917968750, -0.0047302246093750, 0.0118103027343750, -0.0038757324218750,
+0.0111083984375000, -0.0067138671875000, 0.0090942382812500, -0.0107421875000000,
+0.0102233886718750, -0.0075988769531250, 0.0079345703125000, -0.0169372558593750,
+0.0083007812500000, -0.0171203613281250, 0.0058898925781250, -0.0157165527343750,
+0.0048522949218750, -0.0204162597656250, 0.0067749023437500, -0.0197448730468750,
+-0.0075073242187500, -0.0126342773437500, -0.0024414062500000, -0.0158386230468750,
+-0.0091552734375000, -0.0111999511718750, -0.0239257812500000, -0.0025329589843750,
+-0.0153198242187500, -0.0070495605468750, -0.0279235839843750, 0.0005493164062500,
+-0.0308837890625000, 0.0015258789062500, -0.0256958007812500, 0.0013122558593750,
+-0.0288391113281250, 0.0043029785156250, -0.0311279296875000, 0.0031433105468750,
+-0.0162658691406250, 0.0047302246093750, -0.0209350585937500, 0.0055236816406250,
+-0.0153503417968750, 0.0041809082031250, -0.0019226074218750, 0.0038146972656250,
+-0.0095825195312500, 0.0047607421875000, 0.0037841796875000, 0.0015258789062500,
+0.0039367675781250, 0.0016479492187500, 0.0041809082031250, 0.0016479492187500,
+0.0108947753906250, -0.0003051757812500, 0.0082702636718750, 0.0002136230468750,
+0.0128479003906250, 0.0009460449218750, 0.0127868652343750, 0.0003662109375000,
+0.0130004882812500, 0.0006713867187500, 0.0155639648437500, 0.0018615722656250,
+0.0145263671875000, 0.0014038085937500, 0.0163269042968750, 0.0000000000000000,
+0.0156860351562500, 0.0006713867187500, 0.0180358886718750, -0.0002441406250000,
+0.0200195312500000, -0.0020446777343750, 0.0174560546875000, -0.0011901855468750,
+0.0261535644531250, -0.0025024414062500, 0.0236816406250000, -0.0024719238281250,
+0.0270996093750000, -0.0030212402343750, 0.0344543457031250, -0.0039062500000000,
+0.0291748046875000, -0.0032043457031250, 0.0384216308593750, -0.0050964355468750,
+0.0378112792968750, -0.0051574707031250, 0.0386047363281250, -0.0048828125000000,
+0.0442199707031250, -0.0057373046875000, 0.0415039062500000, -0.0055541992187500,
+0.0449829101562500, -0.0050048828125000, 0.0457458496093750, -0.0051879882812500,
+0.0446166992187500, -0.0048217773437500, 0.0456237792968750, -0.0043334960937500,
+0.0455322265625000, -0.0046386718750000, 0.0423583984375000, -0.0037231445312500,
+0.0445251464843750, -0.0038757324218750, 0.0423889160156250, -0.0037841796875000,
+0.0386047363281250, -0.0030517578125000, 0.0416564941406250, -0.0033569335937500,
+0.0357971191406250, -0.0036315917968750, 0.0370788574218750, -0.0034179687500000,
+0.0359497070312500, -0.0037841796875000, 0.0320129394531250, -0.0043945312500000,
+0.0352478027343750, -0.0040893554687500, 0.0306701660156250, -0.0045471191406250,
+0.0319824218750000, -0.0044860839843750, 0.0316772460937500, -0.0045776367187500,
+0.0290527343750000, -0.0047912597656250, 0.0317382812500000, -0.0045471191406250,
+0.0281066894531250, -0.0052795410156250, 0.0292053222656250, -0.0051879882812500,
+0.0292968750000000, -0.0054016113281250, 0.0272827148437500, -0.0059509277343750,
+0.0295410156250000, -0.0057373046875000, 0.0280151367187500, -0.0066528320312500,
+0.0289001464843750, -0.0064086914062500, 0.0293579101562500, -0.0070495605468750,
+0.0287170410156250, -0.0080566406250000, 0.0303649902343750, -0.0074768066406250,
+0.0297851562500000, -0.0087890625000000, 0.0301208496093750, -0.0089721679687500,
+0.0306701660156250, -0.0089721679687500, 0.0306396484375000, -0.0095214843750000,
+0.0311889648437500, -0.0093383789062500, 0.0315246582031250, -0.0091247558593750,
+0.0316772460937500, -0.0093078613281250, 0.0319213867187500, -0.0092468261718750,
+0.0323486328125000, -0.0087280273437500, 0.0326232910156250, -0.0087890625000000,
+0.0320129394531250, -0.0097961425781250, 0.0323486328125000, -0.0091857910156250,
+0.0319213867187500, -0.0097045898437500, 0.0313415527343750, -0.0110168457031250,
+0.0318603515625000, -0.0103454589843750, 0.0298767089843750, -0.0111083984375000,
+0.0301208496093750, -0.0114135742187500, 0.0301513671875000, -0.0105895996093750,
+0.0291137695312500, -0.0105285644531250, 0.0300598144531250, -0.0107727050781250,
+0.0306701660156250, -0.0072631835937500, 0.0303955078125000, -0.0083312988281250,
+0.0320434570312500, -0.0068359375000000, 0.0334472656250000, -0.0031738281250000,
+0.0325622558593750, -0.0048522949218750, 0.0362548828125000, -0.0032653808593750,
+0.0364990234375000, -0.0020751953125000, 0.0368652343750000, -0.0035705566406250,
+0.0394592285156250, -0.0043640136718750, 0.0388793945312500, -0.0034179687500000,
+0.0357971191406250, -0.0065612792968750, 0.0382385253906250, -0.0067443847656250,
+0.0331420898437500, -0.0054931640625000, 0.0273437500000000, -0.0065307617187500,
+0.0309753417968750, -0.0071105957031250, 0.0169677734375000, -0.0031127929687500,
+0.0187377929687500, -0.0038146972656250, 0.0136413574218750, -0.0028381347656250,
+0.0024719238281250, 0.0003967285156250, 0.0078735351562500, -0.0009765625000000,
+-0.0051879882812500, -0.0003967285156250, -0.0050659179687500, 0.0004882812500000,
+-0.0062561035156250, 0.0001831054687500, -0.0137939453125000, -0.0001220703125000,
+-0.0104370117187500, -0.0003967285156250, -0.0128479003906250, -0.0024719238281250,
+-0.0140991210937500, -0.0010070800781250, -0.0120849609375000, -0.0027465820312500,
+-0.0111694335937500, -0.0055236816406250, -0.0117187500000000, -0.0040283203125000,
+-0.0095825195312500, -0.0104675292968750, -0.0097351074218750, -0.0090942382812500,
+-0.0075988769531250, -0.0099792480468750, -0.0055541992187500, -0.0149841308593750,
+-0.0069580078125000, -0.0133972167968750, -0.0000610351562500, -0.0163574218750000,
+-0.0011901855468750, -0.0166015625000000, 0.0017395019531250, -0.0151672363281250,
+0.0079345703125000, -0.0163879394531250, 0.0049438476562500, -0.0169372558593750,
+0.0099487304687500, -0.0113525390625000, 0.0110778808593750, -0.0136718750000000,
+0.0088500976562500, -0.0099792480468750, 0.0094909667968750, -0.0037841796875000,
+0.0103454589843750, -0.0080871582031250, 0.0060119628906250, 0.0003967285156250,
+0.0062255859375000, 0.0014038085937500, 0.0050048828125000, -0.0005493164062500,
+0.0021667480468750, 0.0024108886718750, 0.0034484863281250, 0.0031433105468750,
+0.0005493164062500, 0.0000000000000000, 0.0003356933593750, 0.0001525878906250,
+0.0006713867187500, -0.0002136230468750, -0.0009765625000000, -0.0023193359375000,
+-0.0010681152343750, -0.0020141601562500, 0.0009155273437500, -0.0018310546875000,
+-0.0002441406250000, -0.0023498535156250, 0.0005798339843750, -0.0017700195312500,
+0.0028076171875000, -0.0009460449218750, 0.0011901855468750, -0.0015258789062500,
+0.0025024414062500, -0.0003356933593750, 0.0036926269531250, 0.0000305175781250,
+0.0024108886718750, 0.0000610351562500, 0.0012207031250000, 0.0001220703125000,
+0.0023193359375000, 0.0000000000000000, 0.0016174316406250, 0.0017089843750000,
+0.0000610351562500, 0.0008544921875000, 0.0021972656250000, 0.0029907226562500,
+0.0046386718750000, 0.0057373046875000, 0.0027160644531250, 0.0037536621093750,
+0.0039367675781250, 0.0086975097656250, 0.0071716308593750, 0.0092468261718750,
+0.0012207031250000, 0.0086975097656250, -0.0031433105468750, 0.0105285644531250,
+0.0020751953125000, 0.0102844238281250, -0.0111694335937500, 0.0096435546875000,
+-0.0107116699218750, 0.0111083984375000, -0.0101928710937500, 0.0083923339843750,
+-0.0170288085937500, 0.0060119628906250, -0.0146789550781250, 0.0084838867187500,
+-0.0135192871093750, 0.0007629394531250, -0.0157165527343750, 0.0015563964843750,
+-0.0119934082031250, 0.0005187988281250, -0.0088500976562500, -0.0048522949218750,
+-0.0123596191406250, -0.0030212402343750, -0.0019226074218750, -0.0047912597656250,
+-0.0034790039062500, -0.0053405761718750, -0.0024414062500000, -0.0055847167968750,
+0.0047302246093750, -0.0061340332031250, 0.0018005371093750, -0.0056457519531250,
+0.0032958984375000, -0.0082092285156250, 0.0052795410156250, -0.0078125000000000,
+0.0020141601562500, -0.0075378417968750, -0.0001220703125000, -0.0096435546875000,
+0.0025024414062500, -0.0090332031250000, -0.0035705566406250, -0.0041198730468750,
+-0.0043640136718750, -0.0069580078125000, -0.0021667480468750, -0.0010681152343750,
+-0.0037536621093750, 0.0068359375000000, -0.0048217773437500, 0.0023193359375000,
+0.0000000000000000, 0.0159301757812500, -0.0007629394531250, 0.0163269042968750,
+-0.0003051757812500, 0.0178833007812500, 0.0027465820312500, 0.0257873535156250,
+0.0014038085937500, 0.0234985351562500, 0.0027465820312500, 0.0260009765625000,
+0.0033569335937500, 0.0267639160156250, 0.0018920898437500, 0.0259399414062500,
+0.0014648437500000, 0.0258178710937500, 0.0025939941406250, 0.0249633789062500,
+0.0000610351562500, 0.0244445800781250, -0.0000610351562500, 0.0239257812500000,
+-0.0001525878906250, 0.0239868164062500, -0.0009765625000000, 0.0241088867187500,
+-0.0005493164062500, 0.0237121582031250, -0.0028991699218750, 0.0246276855468750,
+-0.0020141601562500, 0.0241088867187500, -0.0032958984375000, 0.0237731933593750,
+-0.0055541992187500, 0.0240783691406250, -0.0040893554687500, 0.0234680175781250,
+-0.0079956054687500, 0.0237426757812500, -0.0074768066406250, 0.0230712890625000,
+-0.0081176757812500, 0.0227355957031250, -0.0109252929687500, 0.0227355957031250,
+-0.0096130371093750, 0.0216369628906250, -0.0110778808593750, 0.0214843750000000,
+-0.0114440917968750, 0.0209655761718750, -0.0110168457031250, 0.0194091796875000,
+-0.0113525390625000, 0.0183105468750000, -0.0112915039062500, 0.0179748535156250,
+-0.0104980468750000, 0.0163574218750000, -0.0107727050781250, 0.0155029296875000,
+-0.0103454589843750, 0.0144958496093750, -0.0095214843750000, 0.0130004882812500,
+-0.0099487304687500, 0.0124206542968750, -0.0093383789062500, 0.0122680664062500,
+-0.0092773437500000, 0.0112304687500000, -0.0091857910156250, 0.0107421875000000,
+-0.0089721679687500, 0.0105895996093750, -0.0090942382812500, 0.0098266601562500,
+-0.0085144042968750, 0.0098876953125000, -0.0084838867187500, 0.0086975097656250,
+-0.0085754394531250, 0.0089721679687500, -0.0082702636718750, 0.0093383789062500,
+-0.0082702636718750, 0.0077819824218750, -0.0084228515625000, 0.0102844238281250,
+-0.0084838867187500, 0.0089721679687500, -0.0081481933593750, 0.0098876953125000,
+-0.0080871582031250, 0.0118408203125000, -0.0083312988281250, 0.0096130371093750,
+-0.0075683593750000, 0.0139160156250000, -0.0075073242187500, 0.0120544433593750,
+-0.0077209472656250, 0.0133666992187500, -0.0073242187500000, 0.0168151855468750,
+-0.0072631835937500, 0.0138549804687500, -0.0080871582031250, 0.0196838378906250,
+-0.0078735351562500, 0.0183715820312500, -0.0080261230468750, 0.0198059082031250,
+-0.0086975097656250, 0.0240173339843750, -0.0083618164062500, 0.0213928222656250,
+-0.0086364746093750},
+{0.0004272460937500, -0.0009765625000000, -0.0076599121093750, -0.0043945312500000,
+-0.0072326660156250, -0.0044860839843750, -0.0072021484375000, -0.0046997070312500,
+-0.0115661621093750, -0.0065917968750000, -0.0092163085937500, -0.0059814453125000,
+-0.0090026855468750, -0.0067443847656250, -0.0110778808593750, -0.0079040527343750,
+-0.0077819824218750, -0.0056152343750000, -0.0046081542968750, -0.0046081542968750,
+-0.0075683593750000, -0.0059814453125000, -0.0012817382812500, 0.0026855468750000,
+-0.0015258789062500, 0.0003356933593750, -0.0018615722656250, 0.0053405761718750,
+0.0016784667968750, 0.0144042968750000, 0.0003051757812500, 0.0095214843750000,
+-0.0024719238281250, 0.0198059082031250, -0.0005187988281250, 0.0210266113281250,
+-0.0024719238281250, 0.0203857421875000, -0.0068969726562500, 0.0245971679687500,
+-0.0047302246093750, 0.0234375000000000, -0.0051574707031250, 0.0215454101562500,
+-0.0073852539062500, 0.0218200683593750, -0.0047912597656250, 0.0202636718750000,
+-0.0021362304687500, 0.0176086425781250, -0.0043945312500000, 0.0177307128906250,
+-0.0001220703125000, 0.0174255371093750, -0.0001525878906250, 0.0159301757812500,
+-0.0003662109375000, 0.0159606933593750, 0.0019531250000000, 0.0167236328125000,
+0.0010070800781250, 0.0157775878906250, -0.0002136230468750, 0.0157165527343750,
+0.0008239746093750, 0.0161437988281250, 0.0000000000000000, 0.0146789550781250,
+-0.0020751953125000, 0.0136413574218750, -0.0013122558593750, 0.0140991210937500,
+-0.0020141601562500, 0.0098571777343750, -0.0021362304687500, 0.0100097656250000,
+-0.0025634765625000, 0.0081481933593750, -0.0028991699218750, 0.0044555664062500,
+-0.0024719238281250, 0.0053100585937500, -0.0047912597656250, 0.0016479492187500,
+-0.0040893554687500, 0.0010375976562500, -0.0052795410156250, 0.0002746582031250,
+-0.0075683593750000, -0.0019531250000000, -0.0061950683593750, -0.0018005371093750,
+-0.0086975097656250, -0.0026550292968750, -0.0087890625000000, -0.0036010742187500,
+-0.0086059570312500, -0.0031433105468750, -0.0095825195312500, -0.0030517578125000,
+-0.0091552734375000, -0.0039062500000000, -0.0094604492187500, -0.0018920898437500,
+-0.0094909667968750, -0.0025939941406250, -0.0089721679687500, -0.0024719238281250,
+-0.0089721679687500, -0.0010375976562500, -0.0092163085937500, -0.0019531250000000,
+-0.0078735351562500, -0.0006408691406250, -0.0079345703125000, -0.0007324218750000,
+-0.0080566406250000, -0.0005187988281250, -0.0072631835937500, 0.0000610351562500,
+-0.0073547363281250, -0.0005493164062500, -0.0079345703125000, 0.0007629394531250,
+-0.0077209472656250, 0.0005798339843750, -0.0077819824218750, 0.0004272460937500,
+-0.0082702636718750, 0.0010681152343750, -0.0080566406250000, 0.0010070800781250,
+-0.0079956054687500, 0.0016784667968750, -0.0080261230468750, 0.0012512207031250,
+-0.0080261230468750, 0.0018005371093750, -0.0078735351562500, 0.0024719238281250,
+-0.0078430175781250, 0.0015563964843750, -0.0084533691406250, 0.0028991699218750,
+-0.0082702636718750, 0.0027160644531250, -0.0083618164062500, 0.0032043457031250,
+-0.0089111328125000, 0.0041809082031250, -0.0086975097656250, 0.0036010742187500,
+-0.0081481933593750, 0.0054321289062500, -0.0084533691406250, 0.0052185058593750,
+-0.0079345703125000, 0.0055236816406250, -0.0070190429687500, 0.0067749023437500,
+-0.0073242187500000, 0.0062561035156250, -0.0063476562500000, 0.0071411132812500,
+-0.0061645507812500, 0.0068054199218750, -0.0061340332031250, 0.0076599121093750,
+-0.0055847167968750, 0.0084533691406250, -0.0056457519531250, 0.0077514648437500,
+-0.0059814453125000, 0.0116882324218750, -0.0057678222656250, 0.0102233886718750,
+-0.0059509277343750, 0.0126342773437500, -0.0065002441406250, 0.0165710449218750,
+-0.0062255859375000, 0.0137023925781250, -0.0057067871093750, 0.0202636718750000,
+-0.0063171386718750, 0.0193786621093750, -0.0053405761718750, 0.0208740234375000,
+-0.0041809082031250, 0.0254211425781250, -0.0049743652343750, 0.0227661132812500,
+-0.0028381347656250, 0.0275268554687500, -0.0031127929687500, 0.0275878906250000,
+-0.0024108886718750, 0.0267028808593750, -0.0010070800781250, 0.0290832519531250,
+-0.0019836425781250, 0.0288085937500000, 0.0004272460937500, 0.0255432128906250,
+0.0003051757812500, 0.0269165039062500, 0.0005493164062500, 0.0245361328125000,
+0.0022277832031250, 0.0207519531250000, 0.0015563964843750, 0.0230712890625000,
+0.0011596679687500, 0.0179138183593750, 0.0020141601562500, 0.0181579589843750,
+0.0004577636718750, 0.0178527832031250, -0.0010986328125000, 0.0145568847656250,
+0.0001831054687500, 0.0155029296875000, -0.0032653808593750, 0.0151977539062500,
+-0.0030517578125000, 0.0148010253906250, -0.0042724609375000, 0.0147094726562500,
+-0.0064697265625000, 0.0154113769531250, -0.0052490234375000, 0.0154418945312500,
+-0.0100097656250000, 0.0109863281250000, -0.0098876953125000, 0.0136108398437500,
+-0.0089721679687500, 0.0100097656250000, -0.0117187500000000, 0.0039672851562500,
+-0.0116882324218750, 0.0077209472656250, -0.0068054199218750, -0.0009765625000000,
+-0.0082397460937500, -0.0003662109375000, -0.0056762695312500, -0.0028991699218750,
+-0.0009765625000000, -0.0083007812500000, -0.0035095214843750, -0.0045776367187500,
+0.0018920898437500, -0.0146179199218750, 0.0025939941406250, -0.0139160156250000,
+0.0018920898437500, -0.0142211914062500, 0.0039672851562500, -0.0210876464843750,
+0.0037231445312500, -0.0190429687500000, 0.0019226074218750, -0.0155944824218750,
+0.0033874511718750, -0.0193176269531250, 0.0012207031250000, -0.0131530761718750,
+-0.0015258789062500, -0.0054016113281250, 0.0007324218750000, -0.0102844238281250,
+-0.0055847167968750, 0.0030517578125000, -0.0058593750000000, 0.0032653808593750,
+-0.0040283203125000, 0.0030822753906250, -0.0071411132812500, 0.0096435546875000,
+-0.0078735351562500, 0.0078430175781250, 0.0000915527343750, 0.0085754394531250,
+-0.0026855468750000, 0.0092468261718750, 0.0011291503906250, 0.0086364746093750,
+0.0096740722656250, 0.0082397460937500, 0.0051269531250000, 0.0081787109375000,
+0.0104370117187500, 0.0073242187500000, 0.0137023925781250, 0.0083312988281250,
+0.0085449218750000, 0.0066833496093750, 0.0066833496093750, 0.0045776367187500,
+0.0101623535156250, 0.0061950683593750, -0.0025329589843750, 0.0030822753906250,
+-0.0025329589843750, 0.0021972656250000, -0.0015258789062500, 0.0041198730468750,
+-0.0097045898437500, 0.0050964355468750, -0.0010681152343750, 0.0037536621093750,
+0.0011901855468750, 0.0025939941406250, 0.0012817382812500, 0.0013122558593750,
+0.0014953613281250, 0.0032653808593750, 0.0008544921875000, 0.0053100585937500,
+0.0006103515625000, 0.0039367675781250, 0.0055847167968750, 0.0049133300781250,
+0.0034179687500000, 0.0069885253906250, 0.0058898925781250, 0.0034790039062500,
+0.0114135742187500, 0.0000610351562500, 0.0083007812500000, 0.0030822753906250,
+0.0120849609375000, -0.0013427734375000, 0.0138549804687500, -0.0034179687500000,
+0.0119018554687500, 0.0005798339843750, 0.0119018554687500, 0.0015563964843750,
+0.0127868652343750, -0.0014038085937500, 0.0071716308593750, 0.0085449218750000,
+0.0089416503906250, 0.0086975097656250, 0.0066223144531250, 0.0078125000000000,
+0.0016174316406250, 0.0125122070312500, 0.0038757324218750, 0.0117187500000000,
+-0.0038757324218750, 0.0085754394531250, -0.0018310546875000, 0.0105895996093750,
+-0.0056152343750000, 0.0073547363281250, -0.0133666992187500, 0.0021667480468750,
+-0.0089721679687500, 0.0052185058593750, -0.0165405273437500, 0.0013427734375000,
+-0.0176086425781250, 0.0005187988281250, -0.0160827636718750, 0.0004882812500000,
+-0.0187988281250000, -0.0006713867187500, -0.0185241699218750, 0.0002746582031250,
+-0.0135498046875000, -0.0015869140625000, -0.0165710449218750, -0.0014038085937500,
+-0.0106811523437500, -0.0023803710937500, -0.0030517578125000, -0.0036010742187500,
+-0.0086364746093750, -0.0025939941406250, 0.0037841796875000, -0.0072937011718750,
+0.0039367675781250, -0.0062255859375000, 0.0045166015625000, -0.0083312988281250,
+0.0111083984375000, -0.0127258300781250, 0.0087890625000000, -0.0100402832031250,
+0.0115356445312500, -0.0138244628906250, 0.0123291015625000, -0.0156555175781250,
+0.0122375488281250, -0.0121765136718750, 0.0129089355468750, -0.0111083984375000,
+0.0131225585937500, -0.0132141113281250, 0.0163574218750000, -0.0050659179687500,
+0.0141906738281250, -0.0057373046875000, 0.0180053710937500, -0.0042419433593750,
+0.0229492187500000, 0.0012512207031250, 0.0191345214843750, -0.0010070800781250,
+0.0283203125000000, 0.0022583007812500, 0.0278930664062500, 0.0029907226562500,
+0.0282897949218750, 0.0025024414062500, 0.0338745117187500, 0.0034790039062500,
+0.0320129394531250, 0.0032958984375000, 0.0325622558593750, 0.0026550292968750,
+0.0340270996093750, 0.0032653808593750, 0.0316162109375000, 0.0020751953125000,
+0.0303955078125000, 0.0007324218750000, 0.0322875976562500, 0.0018005371093750,
+0.0237426757812500, -0.0009765625000000, 0.0263366699218750, -0.0010681152343750,
+0.0226440429687500, -0.0009155273437500, 0.0150146484375000, -0.0019836425781250,
+0.0199890136718750, -0.0017395019531250, 0.0104370117187500, -0.0020141601562500,
+0.0114440917968750, -0.0017089843750000, 0.0107116699218750, -0.0023498535156250,
+0.0047302246093750, -0.0032348632812500, 0.0076293945312500, -0.0025939941406250,
+0.0040893554687500, -0.0037536621093750, 0.0035400390625000, -0.0040893554687500,
+0.0041503906250000, -0.0036315917968750, 0.0026855468750000, -0.0036621093750000,
+0.0025024414062500, -0.0039062500000000, 0.0028686523437500, -0.0034179687500000,
+0.0024414062500000, -0.0032653808593750, 0.0028686523437500, -0.0034790039062500,
+0.0034790039062500, -0.0035400390625000, 0.0028991699218750, -0.0034484863281250,
+0.0041198730468750, -0.0034179687500000, 0.0039062500000000, -0.0034790039062500,
+0.0037536621093750, -0.0033874511718750, 0.0044555664062500, -0.0032043457031250,
+0.0040588378906250, -0.0031738281250000, 0.0036621093750000, -0.0028991699218750,
+0.0043945312500000, -0.0029296875000000, 0.0036010742187500, -0.0027160644531250,
+0.0027160644531250, -0.0023803710937500, 0.0036621093750000, -0.0025634765625000,
+0.0014953613281250, -0.0024108886718750, 0.0019836425781250, -0.0021972656250000,
+0.0010681152343750, -0.0026550292968750, -0.0007934570312500, -0.0029296875000000,
+0.0003662109375000, -0.0026550292968750, -0.0018615722656250, -0.0035095214843750,
+-0.0016174316406250, -0.0036010742187500, -0.0020141601562500, -0.0034790039062500,
+-0.0035095214843750, -0.0038452148437500, -0.0028991699218750, -0.0038757324218750,
+-0.0046691894531250, -0.0040893554687500, -0.0044250488281250, -0.0038146972656250,
+-0.0049438476562500, -0.0044860839843750, -0.0063171386718750, -0.0052185058593750,
+-0.0057067871093750, -0.0046386718750000, -0.0073242187500000, -0.0062255859375000,
+-0.0072937011718750, -0.0061340332031250, -0.0076293945312500, -0.0067749023437500,
+-0.0084838867187500, -0.0077819824218750, -0.0081787109375000, -0.0070495605468750,
+-0.0102233886718750, -0.0097351074218750, -0.0100097656250000, -0.0093688964843750,
+-0.0111694335937500, -0.0094604492187500, -0.0130310058593750, -0.0112915039062500,
+-0.0122375488281250, -0.0107421875000000, -0.0149230957031250, -0.0108642578125000,
+-0.0146789550781250, -0.0110473632812500, -0.0150451660156250, -0.0107727050781250,
+-0.0170593261718750, -0.0108032226562500, -0.0161743164062500, -0.0108032226562500,
+-0.0153808593750000, -0.0092468261718750, -0.0165710449218750, -0.0098266601562500,
+-0.0145568847656250},
+{0.0164794921875000, -0.0076293945312500, 0.0188293457031250, -0.0081176757812500,
+0.0242309570312500, -0.0097656250000000, 0.0211791992187500, -0.0089721679687500,
+0.0255126953125000, -0.0085449218750000, 0.0267639160156250, -0.0098571777343750,
+0.0251159667968750, -0.0079650878906250, 0.0259704589843750, -0.0057983398437500,
+0.0267028808593750, -0.0073547363281250, 0.0222473144531250, -0.0032348632812500,
+0.0239868164062500, -0.0031738281250000, 0.0219116210937500, -0.0030822753906250,
+0.0177307128906250, -0.0008850097656250, 0.0203857421875000, -0.0013732910156250,
+0.0135192871093750, -0.0009460449218750, 0.0146179199218750, -0.0008239746093750,
+0.0125732421875000, -0.0004882812500000, 0.0073852539062500, -0.0000305175781250,
+0.0105590820312500, -0.0003051757812500, 0.0047912597656250, -0.0000305175781250,
+0.0047302246093750, 0.0004577636718750, 0.0040283203125000, -0.0002136230468750,
+0.0010375976562500, -0.0008239746093750, 0.0027160644531250, -0.0002441406250000,
+-0.0007629394531250, -0.0009765625000000, -0.0001525878906250, -0.0013427734375000,
+-0.0011596679687500, -0.0010070800781250, -0.0037231445312500, -0.0006713867187500,
+-0.0018005371093750, -0.0009460449218750, -0.0046386718750000, -0.0015563964843750,
+-0.0041503906250000, -0.0009460449218750, -0.0050659179687500, -0.0018310546875000,
+-0.0072631835937500, -0.0031433105468750, -0.0057678222656250, -0.0023193359375000,
+-0.0089721679687500, -0.0035400390625000, -0.0084533691406250, -0.0039062500000000,
+-0.0088500976562500, -0.0035400390625000, -0.0109558105468750, -0.0036926269531250,
+-0.0093994140625000, -0.0037231445312500, -0.0110778808593750, -0.0031433105468750,
+-0.0111694335937500, -0.0032043457031250, -0.0108642578125000, -0.0028686523437500,
+-0.0115966796875000, -0.0022277832031250, -0.0113525390625000, -0.0026245117187500,
+-0.0107727050781250, -0.0018920898437500, -0.0111999511718750, -0.0017395019531250,
+-0.0106201171875000, -0.0021362304687500, -0.0097961425781250, -0.0020751953125000,
+-0.0101928710937500, -0.0018615722656250, -0.0089416503906250, -0.0028991699218750,
+-0.0093078613281250, -0.0028076171875000, -0.0091552734375000, -0.0029907226562500,
+-0.0084228515625000, -0.0036926269531250, -0.0089721679687500, -0.0033874511718750,
+-0.0077819824218750, -0.0038452148437500, -0.0079650878906250, -0.0037841796875000,
+-0.0078430175781250, -0.0041503906250000, -0.0072631835937500, -0.0045166015625000,
+-0.0079650878906250, -0.0041809082031250, -0.0072937011718750, -0.0051269531250000,
+-0.0075073242187500, -0.0051269531250000, -0.0076599121093750, -0.0049743652343750,
+-0.0073242187500000, -0.0051574707031250, -0.0076293945312500, -0.0051269531250000,
+-0.0072937011718750, -0.0058593750000000, -0.0075073242187500, -0.0051879882812500,
+-0.0075073242187500, -0.0061950683593750, -0.0071411132812500, -0.0076904296875000,
+-0.0073547363281250, -0.0066223144531250, -0.0071716308593750, -0.0081176757812500,
+-0.0072631835937500, -0.0084533691406250, -0.0071105957031250, -0.0083923339843750,
+-0.0068664550781250, -0.0086669921875000, -0.0071411132812500, -0.0084533691406250,
+-0.0070190429687500, -0.0090942382812500, -0.0068664550781250, -0.0091857910156250,
+-0.0075073242187500, -0.0085144042968750, -0.0080261230468750, -0.0085754394531250,
+-0.0079040527343750, -0.0090026855468750, -0.0092163085937500, -0.0072326660156250,
+-0.0090942382812500, -0.0074157714843750, -0.0100097656250000, -0.0072021484375000,
+-0.0115356445312500, -0.0061340332031250, -0.0115356445312500, -0.0066833496093750,
+-0.0137329101562500, -0.0064086914062500, -0.0138244628906250, -0.0059204101562500,
+-0.0143127441406250, -0.0072021484375000, -0.0159606933593750, -0.0082092285156250,
+-0.0153503417968750, -0.0071105957031250, -0.0144042968750000, -0.0086975097656250,
+-0.0158691406250000, -0.0099182128906250, -0.0137939453125000, -0.0075378417968750,
+-0.0115966796875000, -0.0061950683593750, -0.0131530761718750, -0.0080566406250000,
+-0.0059204101562500, -0.0033264160156250, -0.0069885253906250, -0.0028076171875000,
+-0.0046997070312500, -0.0040588378906250, 0.0011291503906250, -0.0022888183593750,
+-0.0012512207031250, -0.0019226074218750, 0.0040283203125000, -0.0061645507812500,
+0.0047912597656250, -0.0050048828125000, 0.0048217773437500, -0.0066528320312500,
+0.0068359375000000, -0.0103454589843750, 0.0064086914062500, -0.0081787109375000,
+0.0083618164062500, -0.0119934082031250, 0.0078735351562500, -0.0128479003906250,
+0.0086364746093750, -0.0108947753906250, 0.0101623535156250, -0.0111389160156250,
+0.0091552734375000, -0.0119323730468750, 0.0120849609375000, -0.0082092285156250,
+0.0110778808593750, -0.0097656250000000, 0.0126647949218750, -0.0063781738281250,
+0.0160827636718750, -0.0016174316406250, 0.0142211914062500, -0.0051574707031250,
+0.0162048339843750, 0.0015563964843750, 0.0179748535156250, 0.0022888183593750,
+0.0154418945312500, 0.0011901855468750, 0.0138854980468750, 0.0041198730468750,
+0.0152893066406250, 0.0041503906250000, 0.0089416503906250, 0.0002746582031250,
+0.0101623535156250, 0.0014648437500000, 0.0071411132812500, 0.0003662109375000,
+0.0010986328125000, -0.0032653808593750, 0.0039978027343750, -0.0016784667968750,
+-0.0021972656250000, -0.0018615722656250, -0.0027160644531250, -0.0028991699218750,
+-0.0025939941406250, -0.0023498535156250, -0.0051574707031250, -0.0014953613281250,
+-0.0042114257812500, -0.0017700195312500, -0.0060119628906250, -0.0009460449218750,
+-0.0065612792968750, -0.0010070800781250, -0.0047912597656250, -0.0007629394531250,
+-0.0049743652343750, -0.0004577636718750, -0.0060119628906250, -0.0008850097656250,
+0.0007934570312500, 0.0011291503906250, -0.0007019042968750, 0.0007629394531250,
+0.0007019042968750, 0.0011901855468750, 0.0069274902343750, 0.0030517578125000,
+0.0060119628906250, 0.0032043457031250, 0.0076293945312500, 0.0050354003906250,
+0.0081787109375000, 0.0040283203125000, 0.0066833496093750, 0.0052490234375000,
+0.0066223144531250, 0.0077819824218750, 0.0076904296875000, 0.0061035156250000,
+0.0043640136718750, 0.0071411132812500, 0.0051574707031250, 0.0090332031250000,
+0.0038452148437500, 0.0062561035156250, 0.0013732910156250, 0.0039367675781250,
+0.0033874511718750, 0.0058898925781250, -0.0012207031250000, -0.0001831054687500,
+-0.0013122558593750, -0.0002136230468750, -0.0004882812500000, -0.0001220703125000,
+-0.0024719238281250, -0.0028076171875000, -0.0024414062500000, -0.0022277832031250,
+-0.0015258789062500, -0.0043334960937500, -0.0018615722656250, -0.0037231445312500,
+-0.0010681152343750, -0.0042114257812500, 0.0001831054687500, -0.0068054199218750,
+-0.0004882812500000, -0.0060424804687500, 0.0002441406250000, -0.0047302246093750,
+0.0009155273437500, -0.0070800781250000, 0.0004272460937500, -0.0027770996093750,
+-0.0003051757812500, 0.0013427734375000, 0.0001220703125000, -0.0024414062500000,
+0.0018615722656250, 0.0093688964843750, 0.0007629394531250, 0.0080566406250000,
+0.0016784667968750, 0.0098876953125000, 0.0041809082031250, 0.0182495117187500,
+0.0029296875000000, 0.0148925781250000, 0.0025024414062500, 0.0172424316406250,
+0.0040893554687500, 0.0191955566406250, 0.0017395019531250, 0.0168457031250000,
+-0.0005493164062500, 0.0155334472656250, 0.0018005371093750, 0.0166625976562500,
+-0.0032348632812500, 0.0123291015625000, -0.0025634765625000, 0.0122375488281250,
+-0.0034790039062500, 0.0119323730468750, -0.0065917968750000, 0.0095214843750000,
+-0.0043029785156250, 0.0097961425781250, -0.0081787109375000, 0.0086975097656250,
+-0.0080871582031250, 0.0090942382812500, -0.0080566406250000, 0.0072326660156250,
+-0.0100097656250000, 0.0054931640625000, -0.0092773437500000, 0.0066833496093750,
+-0.0109558105468750, 0.0009765625000000, -0.0116271972656250, 0.0017700195312500,
+-0.0106811523437500, -0.0000610351562500, -0.0111389160156250, -0.0047912597656250,
+-0.0122680664062500, -0.0029907226562500, -0.0086975097656250, -0.0063171386718750,
+-0.0099487304687500, -0.0072937011718750, -0.0083923339843750, -0.0066223144531250,
+-0.0048828125000000, -0.0071716308593750, -0.0070495605468750, -0.0071411132812500,
+-0.0034484863281250, -0.0059204101562500, -0.0033264160156250, -0.0061340332031250,
+-0.0032043457031250, -0.0053710937500000, -0.0013122558593750, -0.0042114257812500,
+-0.0021972656250000, -0.0049743652343750, -0.0020141601562500, -0.0039672851562500,
+-0.0014038085937500, -0.0036315917968750, -0.0020446777343750, -0.0039672851562500,
+-0.0028686523437500, -0.0036926269531250, -0.0024414062500000, -0.0035400390625000,
+-0.0024108886718750, -0.0046997070312500, -0.0030822753906250, -0.0044555664062500,
+-0.0025024414062500, -0.0048828125000000, -0.0016784667968750, -0.0057983398437500,
+-0.0022583007812500, -0.0054321289062500, -0.0021057128906250, -0.0066528320312500,
+-0.0014038085937500, -0.0065002441406250, -0.0024719238281250, -0.0065307617187500,
+-0.0034484863281250, -0.0070800781250000, -0.0024719238281250, -0.0066833496093750,
+-0.0048217773437500, -0.0072021484375000, -0.0046386718750000, -0.0066833496093750,
+-0.0047607421875000, -0.0061645507812500, -0.0059814453125000, -0.0058288574218750,
+-0.0054626464843750, -0.0051574707031250, -0.0066833496093750, -0.0056152343750000,
+-0.0064086914062500, -0.0046997070312500, -0.0066833496093750, -0.0046386718750000,
+-0.0077514648437500, -0.0049743652343750, -0.0072021484375000, -0.0037231445312500,
+-0.0076904296875000, -0.0045471191406250, -0.0079956054687500, -0.0036621093750000,
+-0.0076904296875000, -0.0036621093750000, -0.0075073242187500, -0.0044250488281250,
+-0.0077209472656250, -0.0033264160156250, -0.0072326660156250, -0.0045471191406250,
+-0.0072326660156250, -0.0039672851562500, -0.0069580078125000, -0.0040893554687500,
+-0.0065307617187500, -0.0048828125000000, -0.0067443847656250, -0.0039062500000000,
+-0.0060119628906250, -0.0052185058593750, -0.0060424804687500, -0.0044250488281250,
+-0.0058898925781250, -0.0048828125000000, -0.0054016113281250, -0.0061950683593750,
+-0.0055847167968750, -0.0049133300781250, -0.0052795410156250, -0.0070800781250000,
+-0.0052490234375000, -0.0067749023437500, -0.0052185058593750, -0.0071411132812500,
+-0.0050659179687500, -0.0085449218750000, -0.0051574707031250, -0.0077209472656250,
+-0.0042114257812500, -0.0093078613281250, -0.0047302246093750, -0.0093688964843750,
+-0.0036010742187500, -0.0087585449218750, -0.0020751953125000, -0.0094909667968750,
+-0.0029602050781250, -0.0094909667968750, -0.0005798339843750, -0.0073852539062500,
+-0.0002441406250000, -0.0087585449218750, -0.0004577636718750, -0.0070800781250000,
+0.0005187988281250, -0.0045166015625000, 0.0004577636718750, -0.0065002441406250,
+-0.0004577636718750, -0.0011291503906250, 0.0001220703125000, -0.0020141601562500,
+-0.0007629394531250, -0.0003356933593750, -0.0018310546875000, 0.0037841796875000,
+-0.0009460449218750, 0.0014648437500000, -0.0039978027343750, 0.0062561035156250,
+-0.0033569335937500, 0.0061340332031250, -0.0042724609375000, 0.0058288574218750,
+-0.0068664550781250, 0.0081176757812500, -0.0056152343750000, 0.0072631835937500,
+-0.0074462890625000, 0.0075073242187500, -0.0077819824218750, 0.0076904296875000,
+-0.0077819824218750, 0.0074157714843750, -0.0084533691406250, 0.0071105957031250,
+-0.0081787109375000, 0.0074157714843750, -0.0085754394531250, 0.0079650878906250,
+-0.0088195800781250, 0.0068969726562500, -0.0084228515625000, 0.0087280273437500,
+-0.0082702636718750, 0.0107727050781250, -0.0085754394531250, 0.0092773437500000,
+-0.0076293945312500, 0.0137023925781250, -0.0078735351562500, 0.0138549804687500,
+-0.0068969726562500, 0.0140075683593750, -0.0056152343750000, 0.0162963867187500,
+-0.0064392089843750},
+{-0.0022583007812500, -0.0025024414062500, -0.0040893554687500, -0.0024414062500000,
+-0.0018005371093750, -0.0017395019531250, -0.0016174316406250, -0.0018310546875000,
+-0.0018920898437500, -0.0007934570312500, -0.0007934570312500, 0.0004577636718750,
+-0.0010070800781250, 0.0002136230468750, -0.0013732910156250, 0.0030212402343750,
+-0.0011901855468750, 0.0031738281250000, -0.0010681152343750, 0.0040893554687500,
+-0.0014343261718750, 0.0062255859375000, -0.0014648437500000, 0.0060424804687500,
+-0.0009155273437500, 0.0078735351562500, -0.0009460449218750, 0.0085449218750000,
+-0.0014038085937500, 0.0090026855468750, -0.0012512207031250, 0.0101928710937500,
+-0.0010681152343750, 0.0106201171875000, -0.0033874511718750, 0.0109558105468750,
+-0.0026245117187500, 0.0118408203125000, -0.0035705566406250, 0.0118103027343750,
+-0.0058288574218750, 0.0115661621093750, -0.0046386718750000, 0.0121765136718750,
+-0.0064392089843750, 0.0110168457031250, -0.0065917968750000, 0.0113220214843750,
+-0.0066528320312500, 0.0108947753906250, -0.0075073242187500, 0.0100708007812500,
+-0.0070495605468750, 0.0108032226562500, -0.0075683593750000, 0.0093383789062500,
+-0.0077209472656250, 0.0091857910156250, -0.0071105957031250, 0.0094299316406250,
+-0.0068969726562500, 0.0089721679687500, -0.0072021484375000, 0.0090332031250000,
+-0.0061035156250000, 0.0091247558593750, -0.0061035156250000, 0.0090026855468750,
+-0.0061645507812500, 0.0092163085937500, -0.0056762695312500, 0.0093994140625000,
+-0.0057067871093750, 0.0093078613281250, -0.0058593750000000, 0.0102233886718750,
+-0.0058593750000000, 0.0099182128906250, -0.0057373046875000, 0.0101623535156250,
+-0.0058593750000000, 0.0110473632812500, -0.0059204101562500, 0.0105285644531250,
+-0.0049743652343750, 0.0110473632812500, -0.0054626464843750, 0.0110168457031250,
+-0.0045166015625000, 0.0106201171875000, -0.0032958984375000, 0.0108337402343750,
+-0.0041503906250000, 0.0105590820312500, -0.0020141601562500, 0.0091552734375000,
+-0.0019226074218750, 0.0099792480468750, -0.0020446777343750, 0.0083007812500000,
+-0.0010070800781250, 0.0065307617187500, -0.0011596679687500, 0.0079956054687500,
+-0.0018005371093750, 0.0021057128906250, -0.0015258789062500, 0.0038757324218750,
+-0.0019836425781250, 0.0015258789062500, -0.0027160644531250, -0.0036315917968750,
+-0.0022888183593750, -0.0004577636718750, -0.0036926269531250, -0.0068359375000000,
+-0.0032348632812500, -0.0066833496093750, -0.0038146972656250, -0.0073547363281250,
+-0.0053100585937500, -0.0112304687500000, -0.0043945312500000, -0.0093688964843750,
+-0.0051879882812500, -0.0104064941406250, -0.0057678222656250, -0.0115661621093750,
+-0.0049133300781250, -0.0092163085937500, -0.0044250488281250, -0.0079345703125000,
+-0.0049743652343750, -0.0094299316406250, -0.0042419433593750, -0.0034790039062500,
+-0.0037841796875000, -0.0048828125000000, -0.0043334960937500, -0.0024414062500000,
+-0.0045776367187500, 0.0027465820312500, -0.0042724609375000, -0.0002746582031250,
+-0.0053405761718750, 0.0054626464843750, -0.0050659179687500, 0.0055847167968750,
+-0.0059509277343750, 0.0058898925781250, -0.0070190429687500, 0.0087280273437500,
+-0.0061035156250000, 0.0073547363281250, -0.0083923339843750, 0.0092163085937500,
+-0.0083923339843750, 0.0093078613281250, -0.0083618164062500, 0.0088806152343750,
+-0.0094909667968750, 0.0096130371093750, -0.0092163085937500, 0.0096435546875000,
+-0.0083923339843750, 0.0096435546875000, -0.0093383789062500, 0.0090942382812500,
+-0.0074462890625000, 0.0104064941406250, -0.0057983398437500, 0.0113525390625000,
+-0.0072021484375000, 0.0101318359375000, -0.0006713867187500, 0.0138854980468750,
+-0.0022583007812500, 0.0137634277343750, 0.0007629394531250, 0.0136108398437500,
+0.0071411132812500, 0.0156860351562500, 0.0038452148437500, 0.0150451660156250,
+0.0096130371093750, 0.0135803222656250, 0.0108337402343750, 0.0139770507812500,
+0.0092468261718750, 0.0139770507812500, 0.0110778808593750, 0.0128479003906250,
+0.0115661621093750, 0.0130310058593750, 0.0063781738281250, 0.0141601562500000,
+0.0074768066406250, 0.0135192871093750, 0.0062561035156250, 0.0149841308593750,
+0.0022583007812500, 0.0172119140625000, 0.0042419433593750, 0.0160217285156250,
+0.0019531250000000, 0.0171813964843750, 0.0007019042968750, 0.0191955566406250,
+0.0027160644531250, 0.0143432617187500, 0.0037536621093750, 0.0119934082031250,
+0.0024414062500000, 0.0155334472656250, 0.0047302246093750, -0.0010681152343750,
+0.0047607421875000, 0.0014038085937500, 0.0046386718750000, -0.0030517578125000,
+0.0056762695312500, -0.0166931152343750, 0.0052490234375000, -0.0108642578125000,
+0.0049133300781250, -0.0158081054687500, 0.0051574707031250, -0.0196838378906250,
+0.0045471191406250, -0.0153503417968750, 0.0034790039062500, -0.0127868652343750,
+0.0041198730468750, -0.0154418945312500, 0.0054626464843750, -0.0083618164062500,
+0.0040588378906250, -0.0078430175781250, 0.0052490234375000, -0.0081787109375000,
+0.0093994140625000, -0.0036315917968750, 0.0068359375000000, -0.0161743164062500,
+-0.0043640136718750, -0.0116882324218750, 0.0005798339843750, -0.0134582519531250,
+-0.0028991699218750, -0.0107727050781250, -0.0099487304687500, -0.0081481933593750,
+-0.0072021484375000, -0.0098876953125000, -0.0119323730468750, -0.0017395019531250,
+-0.0122375488281250, -0.0031127929687500, -0.0122680664062500, -0.0012817382812500,
+-0.0148620605468750, 0.0057373046875000, -0.0138854980468750, 0.0032043457031250,
+-0.0129394531250000, 0.0030212402343750, -0.0155639648437500, 0.0051269531250000,
+-0.0096740722656250, 0.0037536621093750, -0.0050354003906250, 0.0007019042968750,
+-0.0094909667968750, 0.0016784667968750, 0.0078430175781250, 0.0039978027343750,
+0.0052795410156250, 0.0022583007812500, 0.0082702636718750, 0.0039978027343750,
+0.0219116210937500, 0.0076293945312500, 0.0168151855468750, 0.0059509277343750,
+0.0178833007812500, 0.0078125000000000, 0.0216064453125000, 0.0089721679687500,
+0.0181274414062500, 0.0063781738281250, 0.0140686035156250, 0.0049438476562500,
+0.0168151855468750, 0.0068969726562500, 0.0126037597656250, 0.0034179687500000,
+0.0113525390625000, 0.0018310546875000, 0.0127868652343750, 0.0040588378906250,
+0.0120544433593750, 0.0049743652343750, 0.0114746093750000, 0.0037231445312500,
+0.0149841308593750, 0.0071411132812500, 0.0139465332031250, 0.0073242187500000,
+0.0149230957031250, 0.0072631835937500, 0.0180358886718750, 0.0091857910156250,
+0.0166931152343750, 0.0087890625000000, 0.0190429687500000, 0.0064697265625000,
+0.0188903808593750, 0.0077819824218750, 0.0187683105468750, 0.0057373046875000,
+0.0202026367187500, 0.0030517578125000, 0.0194396972656250, 0.0046691894531250,
+0.0178833007812500, -0.0025939941406250, 0.0193481445312500, -0.0009765625000000,
+0.0170288085937500, -0.0033874511718750, 0.0142517089843750, -0.0100708007812500,
+0.0158996582031250, -0.0070495605468750, 0.0087280273437500, -0.0103149414062500,
+0.0108337402343750, -0.0119323730468750, 0.0064392089843750, -0.0097961425781250,
+-0.0007019042968750, -0.0092163085937500, 0.0042114257812500, -0.0102233886718750,
+-0.0068664550781250, -0.0063781738281250, -0.0059814453125000, -0.0065002441406250,
+-0.0072326660156250, -0.0063781738281250, -0.0140991210937500, -0.0043334960937500,
+-0.0110168457031250, -0.0048828125000000, -0.0150146484375000, -0.0043640136718750,
+-0.0155639648437500, -0.0043334960937500, -0.0145568847656250, -0.0041503906250000,
+-0.0161437988281250, -0.0036010742187500, -0.0158386230468750, -0.0039978027343750,
+-0.0125427246093750, -0.0041198730468750, -0.0141906738281250, -0.0037841796875000,
+-0.0115661621093750, -0.0040893554687500, -0.0074768066406250, -0.0046997070312500,
+-0.0100708007812500, -0.0044555664062500, -0.0041198730468750, -0.0037841796875000,
+-0.0047302246093750, -0.0041809082031250, -0.0038757324218750, -0.0036926269531250,
+0.0001831054687500, -0.0027465820312500, -0.0017089843750000, -0.0030822753906250,
+0.0012512207031250, -0.0024719238281250, 0.0014648437500000, -0.0025024414062500,
+0.0015258789062500, -0.0019226074218750, 0.0032348632812500, -0.0013122558593750,
+0.0031127929687500, -0.0018005371093750, 0.0032043457031250, -0.0000610351562500,
+0.0037841796875000, 0.0001220703125000, 0.0033264160156250, -0.0002746582031250,
+0.0031127929687500, 0.0004577636718750, 0.0037841796875000, 0.0005493164062500,
+0.0013732910156250, -0.0009460449218750, 0.0026855468750000, -0.0006103515625000,
+0.0018310546875000, -0.0011596679687500, -0.0003662109375000, -0.0024414062500000,
+0.0015563964843750, -0.0019226074218750, -0.0013122558593750, -0.0029602050781250,
+-0.0009155273437500, -0.0029602050781250, -0.0011596679687500, -0.0031127929687500,
+-0.0026855468750000, -0.0037231445312500, -0.0013122558593750, -0.0034179687500000,
+-0.0031738281250000, -0.0040893554687500, -0.0028076171875000, -0.0039367675781250,
+-0.0025634765625000, -0.0042419433593750, -0.0033874511718750, -0.0047912597656250,
+-0.0026550292968750, -0.0043334960937500, -0.0031433105468750, -0.0051879882812500,
+-0.0030822753906250, -0.0050659179687500, -0.0028076171875000, -0.0053100585937500,
+-0.0028686523437500, -0.0058898925781250, -0.0023803710937500, -0.0055541992187500,
+-0.0019226074218750, -0.0066833496093750, -0.0018005371093750, -0.0066223144531250,
+-0.0009765625000000, -0.0064697265625000, -0.0002746582031250, -0.0071105957031250,
+-0.0001220703125000, -0.0070495605468750, 0.0017700195312500, -0.0065307617187500,
+0.0014648437500000, -0.0065307617187500, 0.0028076171875000, -0.0070190429687500,
+0.0046386718750000, -0.0071716308593750, 0.0038146972656250, -0.0066223144531250,
+0.0076293945312500, -0.0072937011718750, 0.0073547363281250, -0.0079040527343750,
+0.0087585449218750, -0.0065307617187500, 0.0116271972656250, -0.0057373046875000,
+0.0101928710937500, -0.0068054199218750, 0.0135498046875000, -0.0039367675781250,
+0.0135498046875000, -0.0040588378906250, 0.0134277343750000, -0.0039978027343750,
+0.0154113769531250, -0.0023193359375000, 0.0149536132812500, -0.0030822753906250,
+0.0133666992187500, -0.0033874511718750, 0.0144348144531250, -0.0024108886718750,
+0.0126953125000000, -0.0042114257812500, 0.0104370117187500, -0.0060729980468750,
+0.0120849609375000, -0.0045166015625000, 0.0075073242187500, -0.0072937011718750,
+0.0080871582031250, -0.0079345703125000, 0.0067749023437500, -0.0070495605468750,
+0.0031738281250000, -0.0073547363281250, 0.0047912597656250, -0.0076904296875000,
+0.0015869140625000, -0.0072631835937500, 0.0014648437500000, -0.0067749023437500,
+0.0014038085937500, -0.0074768066406250, -0.0000915527343750, -0.0079650878906250,
+0.0006408691406250, -0.0074462890625000, -0.0005798339843750, -0.0092468261718750,
+-0.0004577636718750, -0.0093994140625000, -0.0011291503906250, -0.0089416503906250,
+-0.0020141601562500, -0.0095825195312500, -0.0012817382812500, -0.0097656250000000,
+-0.0040588378906250, -0.0090332031250000, -0.0035705566406250, -0.0094299316406250,
+-0.0045166015625000, -0.0081787109375000, -0.0069885253906250, -0.0068359375000000,
+-0.0060424804687500, -0.0079650878906250, -0.0075378417968750, -0.0061950683593750,
+-0.0080871582031250, -0.0058898925781250, -0.0070190429687500, -0.0050048828125000,
+-0.0072021484375000, -0.0039062500000000, -0.0075683593750000, -0.0051879882812500,
+-0.0026550292968750, -0.0029602050781250, -0.0038452148437500, -0.0026245117187500,
+-0.0017089843750000, -0.0024719238281250, 0.0030517578125000, -0.0015563964843750,
+0.0009155273437500, -0.0020751953125000, 0.0045471191406250, -0.0016479492187500,
+0.0051879882812500},
+{-0.0054321289062500, -0.0034484863281250, -0.0056152343750000, -0.0035400390625000,
+-0.0053100585937500, -0.0030212402343750, -0.0054931640625000, -0.0024108886718750,
+-0.0056457519531250, -0.0028381347656250, -0.0044860839843750, -0.0018310546875000,
+-0.0051879882812500, -0.0015258789062500, -0.0044555664062500, -0.0018615722656250,
+-0.0031738281250000, -0.0017089843750000, -0.0044250488281250, -0.0015258789062500,
+-0.0027770996093750, -0.0023803710937500, -0.0026855468750000, -0.0021362304687500,
+-0.0027465820312500, -0.0027160644531250, -0.0023498535156250, -0.0035095214843750,
+-0.0031738281250000, -0.0029602050781250, -0.0023803710937500, -0.0047607421875000,
+-0.0028686523437500, -0.0045471191406250, -0.0026245117187500, -0.0048217773437500,
+-0.0019836425781250, -0.0061035156250000, -0.0030517578125000, -0.0055847167968750,
+-0.0020446777343750, -0.0061950683593750, -0.0026550292968750, -0.0063781738281250,
+-0.0027160644531250, -0.0061645507812500, -0.0020446777343750, -0.0062866210937500,
+-0.0028686523437500, -0.0062866210937500, -0.0023498535156250, -0.0057983398437500,
+-0.0028991699218750, -0.0058593750000000, -0.0029602050781250, -0.0059509277343750,
+-0.0025939941406250, -0.0055236816406250, -0.0033874511718750, -0.0055847167968750,
+-0.0028991699218750, -0.0065612792968750, -0.0029602050781250, -0.0062866210937500,
+-0.0030822753906250, -0.0061340332031250, -0.0031433105468750, -0.0069274902343750,
+-0.0034484863281250, -0.0069274902343750, -0.0026550292968750, -0.0053710937500000,
+-0.0032348632812500, -0.0061340332031250, -0.0026550292968750, -0.0052185058593750,
+-0.0016784667968750, -0.0035095214843750, -0.0023498535156250, -0.0046386718750000,
+-0.0000915527343750, -0.0025024414062500, -0.0007934570312500, -0.0023193359375000,
+0.0007629394531250, -0.0030822753906250, 0.0032348632812500, -0.0021057128906250,
+0.0018920898437500, -0.0019531250000000, 0.0063476562500000, -0.0050048828125000,
+0.0059814453125000, -0.0039672851562500, 0.0073547363281250, -0.0050964355468750,
+0.0110168457031250, -0.0080871582031250, 0.0099182128906250, -0.0066223144531250,
+0.0132751464843750, -0.0084838867187500, 0.0135803222656250, -0.0088500976562500,
+0.0132751464843750, -0.0084838867187500, 0.0147094726562500, -0.0090942382812500,
+0.0143127441406250, -0.0090332031250000, 0.0125122070312500, -0.0078430175781250,
+0.0134582519531250, -0.0083618164062500, 0.0117797851562500, -0.0080566406250000,
+0.0094604492187500, -0.0070495605468750, 0.0108642578125000, -0.0075683593750000,
+0.0060119628906250, -0.0065612792968750, 0.0062255859375000, -0.0066833496093750,
+0.0055847167968750, -0.0069580078125000, 0.0024108886718750, -0.0065307617187500,
+0.0034179687500000, -0.0063781738281250, 0.0022888183593750, -0.0059509277343750,
+0.0017700195312500, -0.0062866210937500, 0.0016479492187500, -0.0061645507812500,
+0.0014038085937500, -0.0056152343750000, 0.0014038085937500, -0.0054931640625000,
+-0.0000915527343750, -0.0046081542968750, 0.0004577636718750, -0.0054016113281250,
+-0.0008239746093750, -0.0046386718750000, -0.0027160644531250, -0.0029602050781250,
+-0.0013732910156250, -0.0036010742187500, -0.0041503906250000, -0.0021362304687500,
+-0.0046081542968750, -0.0022277832031250, -0.0038146972656250, -0.0025329589843750,
+-0.0049133300781250, -0.0016784667968750, -0.0051879882812500, -0.0016479492187500,
+-0.0017395019531250, -0.0024414062500000, -0.0030212402343750, -0.0026550292968750,
+-0.0011596679687500, -0.0016479492187500, 0.0024719238281250, -0.0017089843750000,
+0.0004577636718750, -0.0024719238281250, 0.0048828125000000, 0.0021667480468750,
+0.0051574707031250, 0.0015869140625000, 0.0050659179687500, 0.0019836425781250,
+0.0072631835937500, 0.0048828125000000, 0.0068054199218750, 0.0038757324218750,
+0.0063476562500000, 0.0046691894531250, 0.0061340332031250, 0.0053405761718750,
+0.0076293945312500, 0.0050659179687500, 0.0081481933593750, 0.0044250488281250,
+0.0068664550781250, 0.0046081542968750, 0.0113220214843750, 0.0068054199218750,
+0.0113830566406250, 0.0056152343750000, 0.0108947753906250, 0.0068664550781250,
+0.0132141113281250, 0.0101928710937500, 0.0128479003906250, 0.0085449218750000,
+0.0096130371093750, 0.0074157714843750, 0.0114440917968750, 0.0098266601562500,
+0.0090332031250000, 0.0066833496093750, 0.0041198730468750, 0.0012817382812500,
+0.0047607421875000, 0.0020751953125000, -0.0010375976562500, -0.0019226074218750,
+0.0004882812500000, -0.0020446777343750, 0.0003051757812500, -0.0007324218750000,
+-0.0037841796875000, -0.0021362304687500, -0.0020141601562500, -0.0025329589843750,
+-0.0024108886718750, 0.0007934570312500, -0.0042724609375000, -0.0003662109375000,
+-0.0018615722656250, 0.0017089843750000, 0.0003051757812500, 0.0049133300781250,
+-0.0018920898437500, 0.0026855468750000, 0.0031433105468750, 0.0093078613281250,
+0.0028991699218750, 0.0090026855468750, 0.0034790039062500, 0.0088806152343750,
+0.0062866210937500, 0.0128479003906250, 0.0047912597656250, 0.0119628906250000,
+0.0068054199218750, 0.0105590820312500, 0.0068054199218750, 0.0115661621093750,
+0.0076904296875000, 0.0103759765625000, 0.0093688964843750, 0.0084228515625000,
+0.0079040527343750, 0.0095520019531250, 0.0086364746093750, 0.0075683593750000,
+0.0106506347656250, 0.0073242187500000, 0.0070800781250000, 0.0072631835937500,
+0.0043640136718750, 0.0067749023437500, 0.0074157714843750, 0.0068969726562500,
+-0.0006103515625000, 0.0037536621093750, -0.0000610351562500, 0.0053710937500000,
+-0.0005798339843750, 0.0027465820312500, -0.0052795410156250, -0.0018005371093750,
+-0.0033569335937500, 0.0006713867187500, -0.0052795410156250, -0.0035705566406250,
+-0.0058288574218750, -0.0048522949218750, -0.0053100585937500, -0.0035095214843750,
+-0.0057067871093750, -0.0040283203125000, -0.0057678222656250, -0.0043640136718750,
+-0.0058593750000000, -0.0018005371093750, -0.0057678222656250, -0.0024108886718750,
+-0.0055236816406250, -0.0010681152343750, -0.0057067871093750, 0.0016174316406250,
+-0.0058898925781250, 0.0003662109375000, -0.0043640136718750, 0.0018920898437500,
+-0.0052795410156250, 0.0024719238281250, -0.0037231445312500, 0.0024414062500000,
+-0.0013427734375000, 0.0024719238281250, -0.0032653808593750, 0.0025024414062500,
+-0.0015258789062500, 0.0047607421875000, -0.0005798339843750, 0.0038757324218750,
+-0.0016479492187500, 0.0059204101562500, -0.0015869140625000, 0.0089416503906250,
+-0.0013427734375000, 0.0075378417968750, -0.0056762695312500, 0.0120239257812500,
+-0.0032653808593750, 0.0125122070312500, -0.0059509277343750, 0.0126037597656250,
+-0.0112915039062500, 0.0148010253906250, -0.0078125000000000, 0.0140686035156250,
+-0.0126342773437500, 0.0128479003906250, -0.0134582519531250, 0.0136413574218750,
+-0.0118713378906250, 0.0117492675781250, -0.0134887695312500, 0.0094909667968750,
+-0.0136108398437500, 0.0104980468750000, -0.0085754394531250, 0.0060729980468750,
+-0.0106811523437500, 0.0061340332031250, -0.0084533691406250, 0.0045471191406250,
+-0.0033264160156250, 0.0008544921875000, -0.0063781738281250, 0.0016479492187500,
+-0.0018310546875000, -0.0009155273437500, -0.0015258789062500, -0.0017089843750000,
+-0.0019836425781250, -0.0018615722656250, -0.0000610351562500, -0.0028991699218750,
+-0.0006408691406250, -0.0031433105468750, -0.0010375976562500, -0.0034790039062500,
+-0.0005493164062500, -0.0037231445312500, -0.0010681152343750, -0.0039367675781250,
+-0.0018920898437500, -0.0041503906250000, -0.0014343261718750, -0.0039978027343750,
+-0.0021057128906250, -0.0038757324218750, -0.0021667480468750, -0.0039367675781250,
+-0.0022277832031250, -0.0038452148437500, -0.0023803710937500, -0.0037841796875000,
+-0.0022277832031250, -0.0038452148437500, -0.0029907226562500, -0.0034179687500000,
+-0.0026245117187500, -0.0033874511718750, -0.0031433105468750, -0.0034484863281250,
+-0.0040588378906250, -0.0034179687500000, -0.0034484863281250, -0.0036010742187500,
+-0.0046997070312500, -0.0038757324218750, -0.0046386718750000, -0.0037841796875000,
+-0.0048522949218750, -0.0039062500000000, -0.0056762695312500, -0.0041809082031250,
+-0.0053100585937500, -0.0039978027343750, -0.0061340332031250, -0.0038757324218750,
+-0.0062561035156250, -0.0038757324218750, -0.0059814453125000, -0.0036621093750000,
+-0.0062561035156250, -0.0034790039062500, -0.0064392089843750, -0.0036315917968750,
+-0.0055541992187500, -0.0039062500000000, -0.0058898925781250, -0.0036621093750000,
+-0.0057678222656250, -0.0035095214843750, -0.0050048828125000, -0.0037536621093750,
+-0.0053710937500000, -0.0036010742187500, -0.0053100585937500, -0.0030517578125000,
+-0.0051269531250000, -0.0032958984375000, -0.0054016113281250, -0.0032043457031250,
+-0.0057678222656250, -0.0026550292968750, -0.0056457519531250, -0.0029907226562500,
+-0.0054931640625000, -0.0032043457031250, -0.0058593750000000, -0.0026855468750000,
+-0.0052795410156250, -0.0032958984375000, -0.0046691894531250, -0.0040588378906250,
+-0.0050354003906250, -0.0034179687500000, -0.0034790039062500, -0.0047607421875000,
+-0.0038757324218750, -0.0047912597656250, -0.0026245117187500, -0.0044555664062500,
+-0.0010375976562500, -0.0052185058593750, -0.0021362304687500, -0.0052795410156250,
+0.0016174316406250, -0.0037841796875000, 0.0015563964843750, -0.0047607421875000,
+0.0014038085937500, -0.0028991699218750, 0.0036010742187500, -0.0008544921875000,
+0.0031433105468750, -0.0028381347656250, 0.0010986328125000, 0.0028381347656250,
+0.0024108886718750, 0.0013122558593750, 0.0004577636718750, 0.0030822753906250,
+-0.0024108886718750, 0.0080566406250000, -0.0006713867187500, 0.0052795410156250,
+-0.0056457519531250, 0.0094299316406250, -0.0050048828125000, 0.0103454589843750,
+-0.0063781738281250, 0.0083312988281250, -0.0103149414062500, 0.0093994140625000,
+-0.0083007812500000, 0.0102539062500000, -0.0103454589843750, 0.0034484863281250,
+-0.0117187500000000, 0.0058288574218750, -0.0091857910156250, 0.0021362304687500,
+-0.0080261230468750, -0.0050659179687500, -0.0097045898437500, -0.0011596679687500,
+-0.0041198730468750, -0.0094299316406250, -0.0045166015625000, -0.0102844238281250,
+-0.0034790039062500, -0.0086669921875000, 0.0002136230468750, -0.0121765136718750,
+-0.0012512207031250, -0.0120849609375000, 0.0016479492187500, -0.0058288574218750,
+0.0016479492187500, -0.0083923339843750, 0.0018005371093750, -0.0048217773437500,
+0.0037841796875000, 0.0021362304687500, 0.0031738281250000, -0.0015869140625000,
+0.0019836425781250, 0.0055236816406250, 0.0034179687500000, 0.0057067871093750,
+0.0014038085937500, 0.0056152343750000, -0.0012207031250000, 0.0089721679687500,
+0.0006713867187500, 0.0075988769531250, -0.0044555664062500, 0.0087890625000000,
+-0.0051269531250000, 0.0091857910156250, -0.0029907226562500, 0.0081787109375000,
+-0.0044250488281250, 0.0080261230468750, -0.0056152343750000, 0.0086975097656250,
+0.0005798339843750, 0.0075378417968750, -0.0002136230468750, 0.0063781738281250,
+0.0011291503906250, 0.0080261230468750, 0.0058288574218750, 0.0095825195312500,
+0.0037536621093750, 0.0082092285156250, 0.0039672851562500, 0.0095825195312500,
+0.0052490234375000, 0.0102233886718750, 0.0051574707031250, 0.0093383789062500,
+0.0036621093750000, 0.0087890625000000, 0.0035095214843750, 0.0092773437500000,
+0.0068359375000000, 0.0089721679687500, 0.0055847167968750, 0.0081481933593750,
+0.0072326660156250, 0.0096130371093750, 0.0113525390625000, 0.0108032226562500,
+0.0093383789062500, 0.0095825195312500, 0.0090942382812500, 0.0131835937500000,
+0.0116882324218750, 0.0137023925781250, 0.0076599121093750, 0.0122375488281250,
+0.0036010742187500},
+{-0.0059814453125000, -0.0038146972656250, -0.0064086914062500, -0.0048217773437500,
+-0.0061950683593750, -0.0042419433593750, -0.0061340332031250, -0.0050964355468750,
+-0.0062561035156250, -0.0048217773437500, -0.0061950683593750, -0.0051574707031250,
+-0.0060424804687500, -0.0058898925781250, -0.0060729980468750, -0.0051269531250000,
+-0.0057983398437500, -0.0060729980468750, -0.0061035156250000, -0.0060729980468750,
+-0.0054321289062500, -0.0058898925781250, -0.0048217773437500, -0.0064086914062500,
+-0.0053100585937500, -0.0061645507812500, -0.0033569335937500, -0.0047607421875000,
+-0.0036926269531250, -0.0053100585937500, -0.0027465820312500, -0.0046691894531250,
+-0.0011901855468750, -0.0032348632812500, -0.0020446777343750, -0.0039978027343750,
+0.0007019042968750, -0.0022583007812500, 0.0005493164062500, -0.0020751953125000,
+0.0007019042968750, -0.0022583007812500, 0.0026855468750000, -0.0013732910156250,
+0.0019531250000000, -0.0014343261718750, 0.0008239746093750, -0.0027160644531250,
+0.0020446777343750, -0.0018005371093750, -0.0000610351562500, -0.0027160644531250,
+-0.0025939941406250, -0.0043640136718750, -0.0009765625000000, -0.0031738281250000,
+-0.0057678222656250, -0.0062255859375000, -0.0055236816406250, -0.0058593750000000,
+-0.0063781738281250, -0.0057678222656250, -0.0096130371093750, -0.0078735351562500,
+-0.0080871582031250, -0.0073242187500000, -0.0100708007812500, -0.0059204101562500,
+-0.0108642578125000, -0.0077514648437500, -0.0091552734375000, -0.0049438476562500,
+-0.0090332031250000, -0.0019531250000000, -0.0098266601562500, -0.0046691894531250,
+-0.0048217773437500, 0.0031738281250000, -0.0056457519531250, 0.0022583007812500,
+-0.0044250488281250, 0.0035095214843750, -0.0003356933593750, 0.0094299316406250,
+-0.0018310546875000, 0.0072326660156250, -0.0002746582031250, 0.0083923339843750,
+0.0004577636718750, 0.0107421875000000, -0.0000915527343750, 0.0071411132812500,
+-0.0000610351562500, 0.0045776367187500, 0.0000915527343750, 0.0072937011718750,
+-0.0007629394531250, -0.0014648437500000, -0.0001220703125000, -0.0007629394531250,
+-0.0011901855468750, -0.0019226074218750, -0.0025329589843750, -0.0077819824218750,
+-0.0014953613281250, -0.0056152343750000, -0.0042114257812500, -0.0072631835937500,
+-0.0041503906250000, -0.0086364746093750, -0.0046081542968750, -0.0066528320312500,
+-0.0066833496093750, -0.0061950683593750, -0.0057678222656250, -0.0075683593750000,
+-0.0041198730468750, -0.0015563964843750, -0.0058898925781250, -0.0033569335937500,
+-0.0039062500000000, -0.0008239746093750, -0.0004272460937500, 0.0051574707031250,
+-0.0020446777343750, 0.0018310546875000, 0.0015258789062500, 0.0060119628906250,
+0.0014648437500000, 0.0076293945312500, 0.0017089843750000, 0.0067749023437500,
+0.0035095214843750, 0.0068664550781250, 0.0029907226562500, 0.0068054199218750,
+0.0056457519531250, 0.0078430175781250, 0.0046081542968750, 0.0076904296875000,
+0.0056762695312500, 0.0077209472656250, 0.0089111328125000, 0.0081481933593750,
+0.0075683593750000, 0.0079956054687500, 0.0080871582031250, 0.0079650878906250,
+0.0097045898437500, 0.0083312988281250, 0.0068054199218750, 0.0074768066406250,
+0.0047302246093750, 0.0068359375000000, 0.0069274902343750, 0.0075683593750000,
+-0.0001525878906250, 0.0051574707031250, -0.0000305175781250, 0.0046691894531250,
+-0.0001831054687500, 0.0062561035156250, -0.0040893554687500, 0.0055541992187500,
+-0.0031738281250000, 0.0047302246093750, -0.0039978027343750, 0.0115966796875000,
+-0.0042114257812500, 0.0105590820312500, -0.0038146972656250, 0.0099792480468750,
+-0.0040283203125000, 0.0155944824218750, 0.0087890625000000, 0.0034790039062500,
+0.0168457031250000, 0.0129089355468750, 0.0157165527343750, 0.0109558105468750,
+0.0143737792968750, 0.0104980468750000, 0.0168457031250000, 0.0142211914062500,
+0.0177612304687500, 0.0143737792968750, 0.0108642578125000, 0.0075683593750000,
+0.0118103027343750, 0.0095520019531250, 0.0114135742187500, 0.0075988769531250,
+0.0066223144531250, 0.0015258789062500, 0.0079040527343750, 0.0038757324218750,
+0.0094604492187500, 0.0008544921875000, 0.0076904296875000, 0.0008850097656250,
+0.0092468261718750, 0.0000305175781250, 0.0123901367187500, -0.0018005371093750,
+0.0108337402343750, -0.0004272460937500, 0.0115966796875000, -0.0045166015625000,
+0.0130310058593750, -0.0046386718750000, 0.0105895996093750, -0.0035400390625000,
+0.0082702636718750, -0.0057678222656250, 0.0103454589843750, -0.0059814453125000,
+0.0082702636718750, -0.0000915527343750, 0.0071105957031250, -0.0016784667968750,
+0.0074157714843750, -0.0006713867187500, 0.0080261230468750, 0.0049133300781250,
+0.0078430175781250, 0.0033569335937500, 0.0044555664062500, 0.0009155273437500,
+0.0060424804687500, 0.0023193359375000, 0.0035400390625000, 0.0014953613281250,
+-0.0003051757812500, -0.0015869140625000, 0.0019531250000000, -0.0008239746093750,
+-0.0054931640625000, 0.0003051757812500, -0.0044860839843750, -0.0009765625000000,
+-0.0061035156250000, 0.0007934570312500, -0.0122375488281250, 0.0035705566406250,
+-0.0095520019531250, 0.0018310546875000, -0.0099487304687500, 0.0028686523437500,
+-0.0123596191406250, 0.0042419433593750, -0.0087585449218750, 0.0026855468750000,
+-0.0050964355468750, 0.0011291503906250, -0.0079650878906250, 0.0021362304687500,
+-0.0023193359375000, 0.0003662109375000, -0.0014038085937500, 0.0002746582031250,
+-0.0029602050781250, 0.0000915527343750, -0.0009155273437500, -0.0007324218750000,
+-0.0001525878906250, -0.0002746582031250, -0.0053405761718750, -0.0013427734375000,
+-0.0036010742187500, -0.0011901855468750, -0.0045471191406250, -0.0017700195312500,
+-0.0091857910156250, -0.0027160644531250, -0.0072631835937500, -0.0020751953125000,
+-0.0074462890625000, -0.0042419433593750, -0.0090942382812500, -0.0037841796875000,
+-0.0069885253906250, -0.0049438476562500, -0.0050354003906250, -0.0068969726562500,
+-0.0066833496093750, -0.0056762695312500, -0.0021057128906250, -0.0095520019531250,
+-0.0028381347656250, -0.0095214843750000, -0.0017395019531250, -0.0092468261718750,
+0.0017395019531250, -0.0115661621093750, -0.0002136230468750, -0.0113220214843750,
+0.0025939941406250, -0.0090637207031250, 0.0032653808593750, -0.0101013183593750,
+0.0023193359375000, -0.0087890625000000, 0.0028686523437500, -0.0062255859375000,
+0.0029907226562500, -0.0075988769531250, 0.0012817382812500, -0.0046691894531250,
+0.0015563964843750, -0.0047607421875000, 0.0011901855468750, -0.0042419433593750,
+-0.0001525878906250, -0.0023193359375000, 0.0004577636718750, -0.0032348632812500,
+0.0001220703125000, -0.0019836425781250, -0.0003051757812500, -0.0015258789062500,
+0.0003967285156250, -0.0023498535156250, 0.0006408691406250, -0.0024108886718750,
+-0.0000915527343750, -0.0019836425781250, 0.0019531250000000, -0.0036315917968750,
+0.0014953613281250, -0.0035095214843750, 0.0019226074218750, -0.0037841796875000,
+0.0033264160156250, -0.0047607421875000, 0.0023803710937500, -0.0042724609375000,
+0.0041198730468750, -0.0056457519531250, 0.0037231445312500, -0.0053710937500000,
+0.0038757324218750, -0.0059509277343750, 0.0049133300781250, -0.0072021484375000,
+0.0040283203125000, -0.0065307617187500, 0.0054016113281250, -0.0072631835937500,
+0.0051269531250000, -0.0076293945312500, 0.0051574707031250, -0.0071411132812500,
+0.0059509277343750, -0.0069580078125000, 0.0052490234375000, -0.0072937011718750,
+0.0059204101562500, -0.0064086914062500, 0.0057983398437500, -0.0065307617187500,
+0.0056762695312500, -0.0063171386718750, 0.0059204101562500, -0.0056457519531250,
+0.0054016113281250, -0.0059204101562500, 0.0055541992187500, -0.0053100585937500,
+0.0057678222656250, -0.0051574707031250, 0.0053100585937500, -0.0053710937500000,
+0.0050354003906250, -0.0052185058593750, 0.0050659179687500, -0.0051269531250000,
+0.0039367675781250, -0.0055847167968750, 0.0040588378906250, -0.0058288574218750,
+0.0032653808593750, -0.0051574707031250, 0.0019836425781250, -0.0050659179687500,
+0.0021667480468750, -0.0056457519531250, 0.0004577636718750, -0.0033569335937500,
+0.0003967285156250, -0.0039062500000000, -0.0003356933593750, -0.0033264160156250,
+-0.0015869140625000, -0.0013732910156250, -0.0011291503906250, -0.0024108886718750,
+-0.0031127929687500, -0.0018005371093750, -0.0030212402343750, -0.0007629394531250,
+-0.0035705566406250, -0.0027160644531250, -0.0050354003906250, -0.0037841796875000,
+-0.0043945312500000, -0.0023193359375000, -0.0056762695312500, -0.0074157714843750,
+-0.0059204101562500, -0.0069274902343750, -0.0056762695312500, -0.0073242187500000,
+-0.0062866210937500, -0.0107727050781250, -0.0064392089843750, -0.0094909667968750,
+-0.0054626464843750, -0.0091247558593750, -0.0058898925781250, -0.0108337402343750,
+-0.0054931640625000, -0.0084228515625000, -0.0047302246093750, -0.0059204101562500,
+-0.0056762695312500, -0.0082702636718750, -0.0044250488281250, -0.0032043457031250,
+-0.0045776367187500, -0.0029296875000000, -0.0045471191406250, -0.0039672851562500,
+-0.0037231445312500, -0.0021057128906250, -0.0040283203125000, -0.0022277832031250,
+-0.0036621093750000, -0.0039978027343750, -0.0039062500000000, -0.0037536621093750,
+-0.0039978027343750, -0.0041809082031250, -0.0039978027343750, -0.0054016113281250,
+-0.0039367675781250, -0.0047302246093750, -0.0025329589843750, -0.0052795410156250,
+-0.0033874511718750, -0.0058593750000000, -0.0017395019531250, -0.0049743652343750,
+0.0005187988281250, -0.0042724609375000, -0.0010070800781250, -0.0048522949218750,
+0.0025939941406250, -0.0035400390625000, 0.0026550292968750, -0.0038757324218750,
+0.0028686523437500, -0.0031127929687500, 0.0047912597656250, -0.0018005371093750,
+0.0041503906250000, -0.0027160644531250, 0.0048828125000000, -0.0013427734375000,
+0.0051879882812500, -0.0014038085937500, 0.0046997070312500, -0.0008850097656250,
+0.0045166015625000, 0.0001220703125000, 0.0046081542968750, -0.0006103515625000,
+0.0036926269531250, 0.0010681152343750, 0.0035095214843750, 0.0007019042968750,
+0.0035095214843750, 0.0014648437500000, 0.0030517578125000, 0.0030822753906250,
+0.0030822753906250, 0.0021667480468750, 0.0042419433593750, 0.0039672851562500,
+0.0039367675781250, 0.0045166015625000, 0.0037841796875000, 0.0033569335937500,
+0.0046997070312500, 0.0031738281250000, 0.0045471191406250, 0.0042419433593750,
+0.0027465820312500, 0.0035095214843750, 0.0034790039062500, 0.0025024414062500,
+0.0023803710937500, 0.0035400390625000, 0.0001220703125000, 0.0046691894531250,
+0.0012207031250000, 0.0039367675781250, 0.0002136230468750, 0.0052490234375000,
+-0.0005493164062500, 0.0061645507812500, 0.0008239746093750, 0.0042419433593750,
+0.0018310546875000, 0.0036926269531250, 0.0010681152343750, 0.0053405761718750,
+0.0028686523437500, -0.0007934570312500, 0.0026855468750000, -0.0000305175781250,
+0.0034790039062500, 0.0003967285156250, 0.0048217773437500, -0.0037231445312500,
+0.0039672851562500, -0.0030822753906250, 0.0069580078125000, 0.0005798339843750,
+0.0062866210937500, -0.0013732910156250, 0.0072631835937500, 0.0012512207031250,
+0.0102844238281250, 0.0072326660156250, 0.0100097656250000, 0.0067138671875000,
+0.0128173828125000, 0.0098876953125000, 0.0124511718750000, 0.0095214843750000,
+0.0113220214843750, 0.0094909667968750, 0.0128173828125000, 0.0114135742187500,
+0.0131225585937500, 0.0107421875000000, 0.0092773437500000, 0.0111389160156250,
+0.0106506347656250, 0.0117187500000000, 0.0087890625000000, 0.0104064941406250,
+0.0054321289062500, 0.0097961425781250, 0.0077819824218750, 0.0108642578125000,
+0.0014343261718750},
+{0.0053710937500000, -0.0018920898437500, 0.0044555664062500, -0.0055236816406250,
+0.0047607421875000, -0.0045471191406250, 0.0044250488281250, -0.0056762695312500,
+0.0037231445312500, -0.0090637207031250, 0.0043945312500000, -0.0075073242187500,
+0.0026855468750000, -0.0086669921875000, 0.0031433105468750, -0.0099792480468750,
+0.0024414062500000, -0.0080566406250000, 0.0008239746093750, -0.0070190429687500,
+0.0015869140625000, -0.0086669921875000, -0.0004272460937500, -0.0037536621093750,
+-0.0004577636718750, -0.0043945312500000, -0.0007629394531250, -0.0041503906250000,
+-0.0021057128906250, -0.0008544921875000, -0.0019531250000000, -0.0019836425781250,
+-0.0029296875000000, -0.0028381347656250, -0.0031433105468750, -0.0018310546875000,
+-0.0031738281250000, -0.0028076171875000, -0.0036926269531250, -0.0045471191406250,
+-0.0035400390625000, -0.0037841796875000, -0.0029296875000000, -0.0045776367187500,
+-0.0031433105468750, -0.0048217773437500, -0.0031738281250000, -0.0045166015625000,
+-0.0026550292968750, -0.0047912597656250, -0.0028686523437500, -0.0047912597656250,
+-0.0035400390625000, -0.0036010742187500, -0.0029602050781250, -0.0038146972656250,
+-0.0038146972656250, -0.0038146972656250, -0.0050964355468750, -0.0032653808593750,
+-0.0042724609375000, -0.0033569335937500, -0.0055541992187500, -0.0023803710937500,
+-0.0060424804687500, -0.0029907226562500, -0.0051879882812500, -0.0021972656250000,
+-0.0052795410156250, -0.0010986328125000, -0.0058288574218750, -0.0018310546875000,
+-0.0033569335937500, 0.0012512207031250, -0.0041809082031250, 0.0005798339843750,
+-0.0027465820312500, 0.0011901855468750, -0.0002746582031250, 0.0036010742187500,
+-0.0019836425781250, 0.0025329589843750, 0.0013427734375000, 0.0035400390625000,
+0.0012817382812500, 0.0039978027343750, 0.0012512207031250, 0.0034790039062500,
+0.0030822753906250, 0.0034484863281250, 0.0025024414062500, 0.0036926269531250,
+0.0026245117187500, 0.0031433105468750, 0.0028381347656250, 0.0031433105468750,
+0.0030212402343750, 0.0038757324218750, 0.0031433105468750, 0.0039672851562500,
+0.0030212402343750, 0.0032348632812500, 0.0036621093750000, 0.0035400390625000,
+0.0039062500000000, 0.0046691894531250, 0.0036315917968750, 0.0030212402343750,
+0.0036926269531250, 0.0016784667968750, 0.0039062500000000, 0.0030822753906250,
+0.0022277832031250, -0.0012817382812500, 0.0028991699218750, -0.0006713867187500,
+0.0021362304687500, -0.0006713867187500, 0.0001525878906250, -0.0036010742187500,
+0.0012512207031250, -0.0024719238281250, 0.0002441406250000, -0.0002136230468750,
+-0.0003967285156250, -0.0025634765625000, 0.0009765625000000, 0.0006408691406250,
+0.0014038085937500, 0.0053100585937500, 0.0004272460937500, 0.0023498535156250,
+0.0040588378906250, 0.0084228515625000, 0.0036315917968750, 0.0087585449218750,
+0.0043945312500000, 0.0082397460937500, 0.0071411132812500, 0.0109252929687500,
+0.0066528320312500, 0.0105590820312500, 0.0086364746093750, 0.0095825195312500,
+0.0078735351562500, 0.0099487304687500, 0.0086364746093750, 0.0095520019531250,
+0.0104675292968750, 0.0088195800781250, 0.0089416503906250, 0.0091857910156250,
+0.0107421875000000, 0.0074157714843750, 0.0115966796875000, 0.0080871582031250,
+0.0100097656250000, 0.0070800781250000, 0.0094909667968750, 0.0050659179687500,
+0.0105895996093750, 0.0061340332031250, 0.0077514648437500, 0.0038146972656250,
+0.0083312988281250, 0.0040588378906250, 0.0067749023437500, 0.0031127929687500,
+0.0046691894531250, 0.0009460449218750, 0.0065002441406250, 0.0020751953125000,
+-0.0000305175781250, 0.0005493164062500, 0.0014343261718750, 0.0000000000000000,
+-0.0007019042968750, 0.0003356933593750, -0.0068969726562500, 0.0000305175781250,
+-0.0039367675781250, -0.0001220703125000, -0.0056457519531250, 0.0005493164062500,
+-0.0082397460937500, 0.0000915527343750, -0.0046081542968750, 0.0006103515625000,
+-0.0018005371093750, 0.0013427734375000, -0.0048828125000000, 0.0005798339843750,
+0.0017395019531250, 0.0025024414062500, 0.0018005371093750, 0.0021972656250000,
+0.0013732910156250, 0.0029296875000000, 0.0042419433593750, 0.0042724609375000,
+0.0031433105468750, 0.0033569335937500, 0.0034484863281250, 0.0066528320312500,
+0.0039062500000000, 0.0067443847656250, 0.0030517578125000, 0.0061950683593750,
+0.0024719238281250, 0.0079040527343750, 0.0030212402343750, 0.0078430175781250,
+0.0018310546875000, 0.0050354003906250, 0.0021362304687500, 0.0062866210937500,
+0.0013122558593750, 0.0037536621093750, 0.0002441406250000, -0.0001831054687500,
+0.0011596679687500, 0.0018920898437500, -0.0010986328125000, -0.0027465820312500,
+-0.0003967285156250, -0.0036621093750000, -0.0009155273437500, -0.0032348632812500,
+-0.0027465820312500, -0.0049438476562500, -0.0015563964843750, -0.0048522949218750,
+-0.0032653808593750, -0.0034484863281250, -0.0031738281250000, -0.0036010742187500,
+-0.0025329589843750, -0.0033264160156250, -0.0035400390625000, -0.0021362304687500,
+-0.0036926269531250, -0.0020446777343750, -0.0015258789062500, -0.0026245117187500,
+-0.0019226074218750, -0.0021057128906250, -0.0018310546875000, -0.0018920898437500,
+-0.0003967285156250, -0.0023193359375000, -0.0009155273437500, -0.0019226074218750,
+-0.0003967285156250, -0.0011291503906250, 0.0004272460937500, -0.0014953613281250,
+-0.0017089843750000, -0.0004272460937500, -0.0028686523437500, 0.0008544921875000,
+-0.0009155273437500, 0.0001525878906250, -0.0058593750000000, 0.0025939941406250,
+-0.0058898925781250, 0.0025024414062500, -0.0057678222656250, 0.0035095214843750,
+-0.0081176757812500, 0.0054931640625000, -0.0074462890625000, 0.0048828125000000,
+-0.0077514648437500, 0.0072326660156250, -0.0080566406250000, 0.0073852539062500,
+-0.0074768066406250, 0.0076599121093750, -0.0072021484375000, 0.0090637207031250,
+-0.0073852539062500, 0.0086059570312500, -0.0064086914062500, 0.0087890625000000,
+-0.0065002441406250, 0.0091247558593750, -0.0059814453125000, 0.0085754394531250,
+-0.0052185058593750, 0.0080871582031250, -0.0057067871093750, 0.0083312988281250,
+-0.0040893554687500, 0.0074462890625000, -0.0043334960937500, 0.0072631835937500,
+-0.0040588378906250, 0.0072326660156250, -0.0028991699218750, 0.0067443847656250,
+-0.0034179687500000, 0.0065307617187500, -0.0025634765625000, 0.0065917968750000,
+-0.0027770996093750, 0.0061950683593750, -0.0022888183593750, 0.0061035156250000,
+-0.0014648437500000, 0.0061950683593750, -0.0022277832031250, 0.0058898925781250,
+-0.0007324218750000, 0.0060119628906250, -0.0006713867187500, 0.0054626464843750,
+-0.0011596679687500, 0.0057373046875000, -0.0005798339843750, 0.0061035156250000,
+-0.0005187988281250, 0.0051574707031250, -0.0017395019531250, 0.0062255859375000,
+-0.0015869140625000, 0.0060424804687500, -0.0015258789062500, 0.0061340332031250,
+-0.0023193359375000, 0.0067443847656250, -0.0021972656250000, 0.0065002441406250,
+-0.0016784667968750, 0.0072326660156250, -0.0016784667968750, 0.0069885253906250,
+-0.0016784667968750, 0.0068664550781250, -0.0013732910156250, 0.0072021484375000,
+-0.0012817382812500, 0.0069580078125000, -0.0012817382812500, 0.0071411132812500,
+-0.0013427734375000, 0.0069885253906250, -0.0010681152343750, 0.0073852539062500,
+-0.0007934570312500, 0.0077514648437500, -0.0010375976562500, 0.0072326660156250,
+-0.0010681152343750, 0.0084838867187500, -0.0007019042968750, 0.0082092285156250,
+-0.0016479492187500, 0.0085449218750000, -0.0025024414062500, 0.0096435546875000,
+-0.0019836425781250, 0.0090026855468750, -0.0043945312500000, 0.0093994140625000,
+-0.0043334960937500, 0.0096740722656250, -0.0047912597656250, 0.0090942382812500,
+-0.0063781738281250, 0.0088195800781250, -0.0058898925781250, 0.0091552734375000,
+-0.0066223144531250, 0.0077819824218750, -0.0069885253906250, 0.0082397460937500,
+-0.0066528320312500, 0.0074157714843750, -0.0067138671875000, 0.0060119628906250,
+-0.0068664550781250, 0.0069580078125000, -0.0055236816406250, 0.0045776367187500,
+-0.0059509277343750, 0.0047302246093750, -0.0049743652343750, 0.0046691894531250,
+-0.0034484863281250, 0.0030517578125000, -0.0041809082031250, 0.0033874511718750,
+-0.0019226074218750, 0.0038757324218750, -0.0019226074218750, 0.0032958984375000,
+-0.0016784667968750, 0.0039062500000000, -0.0002441406250000, 0.0050048828125000,
+-0.0006713867187500, 0.0042724609375000, -0.0002136230468750, 0.0049438476562500,
+-0.0000915527343750, 0.0055847167968750, 0.0000915527343750, 0.0039978027343750,
+0.0003356933593750, 0.0035705566406250, 0.0001831054687500, 0.0047302246093750,
+0.0008239746093750, -0.0009460449218750, 0.0008544921875000, 0.0002441406250000,
+0.0012817382812500, -0.0014648437500000, 0.0019836425781250, -0.0064392089843750,
+0.0018005371093750, -0.0040283203125000, 0.0027465820312500, -0.0066833496093750,
+0.0030517578125000, -0.0082092285156250, 0.0023498535156250, -0.0065307617187500,
+0.0021362304687500, -0.0065002441406250, 0.0028381347656250, -0.0073547363281250,
+0.0025939941406250, -0.0016479492187500, 0.0013122558593750, -0.0031738281250000,
+0.0032043457031250, -0.0006408691406250, 0.0050964355468750, 0.0050659179687500,
+0.0036621093750000, 0.0020141601562500, 0.0068664550781250, 0.0054626464843750,
+0.0068664550781250, 0.0080261230468750, 0.0063171386718750, 0.0037536621093750,
+0.0084838867187500, 0.0018310546875000, 0.0083618164062500, 0.0047302246093750,
+0.0033874511718750, -0.0063171386718750, 0.0052490234375000, -0.0061340332031250,
+0.0028381347656250, -0.0063781738281250, -0.0024108886718750, -0.0126953125000000,
+0.0004882812500000, -0.0111083984375000, -0.0037536621093750, -0.0103759765625000,
+-0.0048828125000000, -0.0124816894531250, -0.0033874511718750, -0.0083312988281250,
+-0.0039978027343750, -0.0047302246093750, -0.0043640136718750, -0.0081176757812500,
+-0.0015869140625000, 0.0019836425781250, -0.0025329589843750, 0.0021057128906250,
+-0.0008544921875000, 0.0018615722656250, 0.0015869140625000, 0.0068969726562500,
+-0.0000305175781250, 0.0059204101562500, 0.0064392089843750, 0.0059509277343750,
+0.0050354003906250, 0.0061340332031250, 0.0075683593750000, 0.0069885253906250,
+0.0133361816406250, 0.0074768066406250, 0.0104370117187500, 0.0065002441406250,
+0.0159912109375000, 0.0082702636718750, 0.0174560546875000, 0.0089721679687500,
+0.0151062011718750, 0.0077209472656250, 0.0148620605468750, 0.0067443847656250,
+0.0075988769531250, 0.0068359375000000, 0.0119323730468750, 0.0090942382812500,
+0.0096740722656250, 0.0089416503906250, 0.0115661621093750, 0.0082092285156250,
+0.0155639648437500, 0.0084533691406250, 0.0137023925781250, 0.0088806152343750,
+0.0112304687500000, 0.0070190429687500, 0.0140991210937500, 0.0066833496093750,
+0.0110473632812500, 0.0077209472656250, 0.0059509277343750, 0.0075683593750000,
+0.0089721679687500, 0.0068054199218750, 0.0041503906250000, 0.0087280273437500,
+0.0034790039062500, 0.0089416503906250, 0.0033874511718750, 0.0081787109375000,
+0.0015869140625000, 0.0088195800781250, 0.0024108886718750, 0.0091247558593750,
+-0.0000305175781250, 0.0058288574218750, 0.0005798339843750, 0.0066528320312500,
+-0.0017395019531250, 0.0055541992187500, -0.0044555664062500, 0.0029602050781250,
+-0.0018615722656250, 0.0043640136718750, -0.0068664550781250, 0.0010681152343750,
+-0.0084228515625000, 0.0016784667968750, -0.0058898925781250, 0.0003051757812500,
+-0.0058898925781250, -0.0026855468750000, -0.0073852539062500, -0.0008850097656250,
+-0.0032653808593750, -0.0036010742187500, -0.0026855468750000, -0.0045471191406250,
+-0.0041809082031250},
+{0.0000915527343750, 0.0093078613281250, 0.0001525878906250, 0.0080261230468750,
+0.0012817382812500, 0.0056762695312500, 0.0010681152343750, 0.0071105957031250,
+0.0017089843750000, 0.0041503906250000, 0.0015563964843750, 0.0036010742187500,
+0.0020751953125000, 0.0045471191406250, 0.0027770996093750, 0.0037536621093750,
+0.0024414062500000, 0.0033569335937500, 0.0038146972656250, 0.0051574707031250,
+0.0039062500000000, 0.0052795410156250, 0.0039062500000000, 0.0045471191406250,
+0.0048217773437500, 0.0054016113281250, 0.0048217773437500, 0.0057067871093750,
+0.0036926269531250, 0.0021667480468750, 0.0039978027343750, 0.0035705566406250,
+0.0043640136718750, 0.0010986328125000, 0.0032958984375000, -0.0027770996093750,
+0.0031127929687500, -0.0003356933593750, 0.0057067871093750, -0.0072021484375000,
+0.0055847167968750, -0.0072326660156250, 0.0051269531250000, -0.0069580078125000,
+0.0066223144531250, -0.0111389160156250, 0.0066223144531250, -0.0102844238281250,
+0.0036315917968750, -0.0063476562500000, 0.0055236816406250, -0.0090026855468750,
+0.0021667480468750, -0.0051879882812500, -0.0029602050781250, 0.0012207031250000,
+0.0003967285156250, -0.0023498535156250, -0.0043945312500000, 0.0032043457031250,
+-0.0058898925781250, 0.0056152343750000, -0.0038146972656250, 0.0018615722656250,
+-0.0042419433593750, 0.0009155273437500, -0.0051269531250000, 0.0029296875000000,
+-0.0006103515625000, -0.0044250488281250, -0.0012207031250000, -0.0033569335937500,
+-0.0002136230468750, -0.0057678222656250, 0.0026855468750000, -0.0120849609375000,
+0.0013122558593750, -0.0086364746093750, 0.0055847167968750, -0.0100708007812500,
+0.0044250488281250, -0.0137634277343750, 0.0066528320312500, -0.0082092285156250,
+0.0108642578125000, -0.0031433105468750, 0.0081481933593750, -0.0073547363281250,
+0.0129089355468750, 0.0030212402343750, 0.0141906738281250, 0.0036315917968750,
+0.0124511718750000, 0.0028076171875000, 0.0134277343750000, 0.0072021484375000,
+0.0142517089843750, 0.0065917968750000, 0.0090332031250000, 0.0063781738281250,
+0.0093383789062500, 0.0064086914062500, 0.0103454589843750, 0.0070190429687500,
+0.0074462890625000, 0.0079345703125000, 0.0119323730468750, 0.0066833496093750,
+0.0111389160156250, 0.0056762695312500, 0.0139465332031250, 0.0052185058593750,
+0.0100097656250000, 0.0068969726562500, 0.0058898925781250, 0.0076599121093750,
+0.0091552734375000, 0.0062255859375000, 0.0045166015625000, 0.0096130371093750,
+0.0032043457031250, 0.0104370117187500, 0.0044860839843750, 0.0085449218750000,
+0.0042419433593750, 0.0086059570312500, 0.0038757324218750, 0.0097045898437500,
+0.0026855468750000, 0.0059204101562500, 0.0041503906250000, 0.0055847167968750,
+0.0019531250000000, 0.0067749023437500, -0.0013732910156250, 0.0055236816406250,
+0.0005493164062500, 0.0050048828125000, -0.0030212402343750, 0.0086364746093750,
+-0.0035705566406250, 0.0085449218750000, -0.0022277832031250, 0.0079956054687500,
+-0.0032653808593750, 0.0099487304687500, -0.0041198730468750, 0.0098266601562500,
+-0.0010375976562500, 0.0064392089843750, -0.0003356933593750, 0.0076904296875000,
+-0.0029296875000000, 0.0062561035156250, -0.0028991699218750, 0.0029602050781250,
+-0.0011291503906250, 0.0045471191406250, -0.0080566406250000, 0.0010986328125000,
+-0.0072631835937500, 0.0014038085937500, -0.0080261230468750, 0.0002746582031250,
+-0.0130004882812500, -0.0023498535156250, -0.0112609863281250, -0.0008850097656250,
+-0.0106201171875000, -0.0038146972656250, -0.0130310058593750, -0.0044250488281250,
+-0.0092163085937500, -0.0027160644531250, -0.0054016113281250, -0.0032653808593750,
+-0.0086364746093750, -0.0041198730468750, -0.0003967285156250, 0.0011291503906250,
+-0.0005798339843750, 0.0005493164062500, -0.0006713867187500, 0.0010070800781250,
+0.0037231445312500, 0.0044555664062500, 0.0021667480468750, 0.0035095214843750,
+0.0017700195312500, 0.0037231445312500, 0.0029907226562500, 0.0042114257812500,
+0.0016784667968750, 0.0039062500000000, -0.0003662109375000, 0.0034790039062500,
+0.0009155273437500, 0.0036926269531250, 0.0007019042968750, 0.0035400390625000,
+-0.0011291503906250, 0.0035705566406250, 0.0012817382812500, 0.0032958984375000,
+0.0038146972656250, 0.0032348632812500, 0.0016174316406250, 0.0034179687500000,
+0.0057983398437500, 0.0020141601562500, 0.0062561035156250, 0.0022277832031250,
+0.0056762695312500, 0.0020751953125000, 0.0072021484375000, 0.0012817382812500,
+0.0070800781250000, 0.0014648437500000, 0.0060424804687500, 0.0003356933593750,
+0.0065917968750000, 0.0010986328125000, 0.0061950683593750, -0.0004272460937500,
+0.0053100585937500, -0.0023498535156250, 0.0058288574218750, -0.0010070800781250,
+0.0042724609375000, -0.0043029785156250, 0.0043945312500000, -0.0043945312500000,
+0.0042419433593750, -0.0043945312500000, 0.0032043457031250, -0.0059204101562500,
+0.0036315917968750, -0.0053710937500000, 0.0033264160156250, -0.0062255859375000,
+0.0028991699218750, -0.0061340332031250, 0.0030517578125000, -0.0061645507812500,
+0.0031738281250000, -0.0068054199218750, 0.0028381347656250, -0.0065612792968750,
+0.0029907226562500, -0.0059509277343750, 0.0027770996093750, -0.0064086914062500,
+0.0027770996093750, -0.0058288574218750, 0.0030517578125000, -0.0047607421875000,
+0.0028381347656250, -0.0053100585937500, 0.0029296875000000, -0.0045166015625000,
+0.0031433105468750, -0.0044250488281250, 0.0026855468750000, -0.0043029785156250,
+0.0024719238281250, -0.0039367675781250, 0.0027465820312500, -0.0043334960937500,
+0.0012817382812500, -0.0032653808593750, 0.0018005371093750, -0.0033569335937500,
+0.0011291503906250, -0.0034790039062500, -0.0001525878906250, -0.0028686523437500,
+0.0008544921875000, -0.0029602050781250, -0.0014648437500000, -0.0032348632812500,
+-0.0009765625000000, -0.0031127929687500, -0.0014343261718750, -0.0032958984375000,
+-0.0030212402343750, -0.0036010742187500, -0.0017089843750000, -0.0036315917968750,
+-0.0034790039062500, -0.0036926269531250, -0.0031127929687500, -0.0038757324218750,
+-0.0032043457031250, -0.0037841796875000, -0.0042114257812500, -0.0037536621093750,
+-0.0031127929687500, -0.0039978027343750, -0.0042724609375000, -0.0034179687500000,
+-0.0040893554687500, -0.0035400390625000, -0.0039978027343750, -0.0036010742187500,
+-0.0043945312500000, -0.0031738281250000, -0.0037231445312500, -0.0033569335937500,
+-0.0041809082031250, -0.0040283203125000, -0.0038757324218750, -0.0035095214843750,
+-0.0035400390625000, -0.0045471191406250, -0.0036315917968750, -0.0057983398437500,
+-0.0031127929687500, -0.0049438476562500, -0.0028991699218750, -0.0068054199218750,
+-0.0027465820312500, -0.0072631835937500, -0.0022888183593750, -0.0065917968750000,
+-0.0019836425781250, -0.0070190429687500, -0.0020446777343750, -0.0073852539062500,
+-0.0013732910156250, -0.0056762695312500, -0.0011291503906250, -0.0062255859375000,
+-0.0012207031250000, -0.0055236816406250, -0.0010070800781250, -0.0040893554687500,
+-0.0009460449218750, -0.0050659179687500, -0.0012817382812500, -0.0028991699218750,
+-0.0010681152343750, -0.0028991699218750, -0.0012512207031250, -0.0030517578125000,
+-0.0016174316406250, -0.0018310546875000, -0.0013122558593750, -0.0020751953125000,
+-0.0018920898437500, -0.0028991699218750, -0.0017395019531250, -0.0024719238281250,
+-0.0018005371093750, -0.0030212402343750, -0.0020751953125000, -0.0040283203125000,
+-0.0015869140625000, -0.0034179687500000, -0.0017395019531250, -0.0043640136718750,
+-0.0018615722656250, -0.0043640136718750, -0.0013732910156250, -0.0041198730468750,
+-0.0009765625000000, -0.0046386718750000, -0.0012512207031250, -0.0044860839843750,
+-0.0007019042968750, -0.0028991699218750, -0.0003662109375000, -0.0035705566406250,
+-0.0001525878906250, -0.0027770996093750, 0.0001220703125000, -0.0013122558593750,
+0.0000305175781250, -0.0021362304687500, 0.0004272460937500, 0.0004882812500000,
+0.0006103515625000, -0.0003051757812500, 0.0003967285156250, 0.0007019042968750,
+0.0003356933593750, 0.0032653808593750, 0.0003356933593750, 0.0020141601562500,
+-0.0006103515625000, 0.0041503906250000, -0.0005493164062500, 0.0044860839843750,
+-0.0007324218750000, 0.0038452148437500, -0.0013122558593750, 0.0045471191406250,
+-0.0010681152343750, 0.0046997070312500, -0.0013427734375000, 0.0035400390625000,
+-0.0014038085937500, 0.0035400390625000, -0.0011596679687500, 0.0039062500000000,
+-0.0010986328125000, 0.0034179687500000, -0.0013122558593750, 0.0030822753906250,
+-0.0005187988281250, 0.0045776367187500, -0.0001220703125000, 0.0047912597656250,
+-0.0012817382812500, 0.0035095214843750, -0.0017700195312500, 0.0035095214843750,
+-0.0010070800781250, 0.0042419433593750, -0.0040893554687500, 0.0008544921875000,
+-0.0042724609375000, 0.0012817382812500, -0.0043640136718750, 0.0011901855468750,
+-0.0060729980468750, -0.0015869140625000, -0.0056457519531250, -0.0008544921875000,
+-0.0051574707031250, 0.0016479492187500, -0.0058288574218750, 0.0000610351562500,
+-0.0048217773437500, 0.0021667480468750, -0.0034484863281250, 0.0057067871093750,
+-0.0041503906250000, 0.0036010742187500, -0.0023193359375000, 0.0075378417968750,
+-0.0021057128906250, 0.0075683593750000, -0.0017089843750000, 0.0077514648437500,
+-0.0008544921875000, 0.0099792480468750, -0.0012512207031250, 0.0087890625000000,
+0.0005798339843750, 0.0087585449218750, 0.0003967285156250, 0.0096130371093750,
+0.0007019042968750, 0.0084228515625000, 0.0022583007812500, 0.0073242187500000,
+0.0017700195312500, 0.0082092285156250, 0.0004577636718750, 0.0053405761718750,
+0.0013122558593750, 0.0054016113281250, 0.0011596679687500, 0.0052795410156250,
+-0.0007019042968750, 0.0036315917968750, -0.0002746582031250, 0.0036315917968750,
+0.0019531250000000, 0.0019226074218750, 0.0004272460937500, 0.0029602050781250,
+0.0015563964843750, 0.0018920898437500, 0.0041503906250000, -0.0003051757812500,
+0.0024108886718750, 0.0010375976562500, 0.0049133300781250, -0.0008544921875000,
+0.0050048828125000, -0.0016479492187500, 0.0053405761718750, -0.0010681152343750,
+0.0063781738281250, -0.0012512207031250, 0.0053710937500000, -0.0013732910156250,
+0.0072937011718750, -0.0003356933593750, 0.0075683593750000, -0.0006408691406250,
+0.0071716308593750, -0.0003356933593750, 0.0077819824218750, 0.0003662109375000,
+0.0077514648437500, -0.0000305175781250, 0.0072937011718750, 0.0017395019531250,
+0.0074768066406250, 0.0010986328125000, 0.0075683593750000, 0.0021972656250000,
+0.0072631835937500, 0.0042419433593750, 0.0070495605468750, 0.0029907226562500,
+0.0069885253906250, 0.0051574707031250, 0.0079345703125000, 0.0061950683593750,
+0.0056457519531250, 0.0042114257812500, 0.0043029785156250, 0.0038146972656250,
+0.0064086914062500, 0.0051879882812500, -0.0007324218750000, -0.0003662109375000,
+0.0004272460937500, 0.0007629394531250, -0.0001220703125000, -0.0009765625000000,
+-0.0050964355468750, -0.0058288574218750, -0.0032043457031250, -0.0036010742187500,
+-0.0033264160156250, -0.0058593750000000, -0.0048522949218750, -0.0074157714843750,
+-0.0028991699218750, -0.0054931640625000, -0.0010681152343750, -0.0045776367187500,
+-0.0030517578125000, -0.0057067871093750, 0.0003356933593750, -0.0020751953125000,
+0.0005187988281250, -0.0019836425781250, 0.0005187988281250, -0.0021362304687500,
+0.0016784667968750, -0.0002136230468750, 0.0008239746093750, -0.0002441406250000,
+0.0026855468750000, -0.0008239746093750, 0.0023193359375000, -0.0004882812500000,
+0.0029296875000000, -0.0001220703125000, 0.0046386718750000, -0.0003662109375000,
+0.0034790039062500},
+{0.0010681152343750, 0.0033874511718750, 0.0010375976562500, 0.0036315917968750,
+0.0001831054687500, 0.0034790039062500, 0.0006713867187500, 0.0033264160156250,
+0.0000305175781250, 0.0032958984375000, -0.0012207031250000, 0.0036926269531250,
+-0.0004272460937500, 0.0036621093750000, -0.0010681152343750, 0.0014648437500000,
+-0.0018005371093750, 0.0023498535156250, -0.0007934570312500, 0.0019531250000000,
+0.0002136230468750, -0.0003051757812500, -0.0005798339843750, 0.0004272460937500,
+0.0003967285156250, 0.0012207031250000, 0.0012512207031250, -0.0002441406250000,
+-0.0005187988281250, 0.0019836425781250, -0.0015869140625000, 0.0045471191406250,
+-0.0004882812500000, 0.0025939941406250, -0.0045471191406250, 0.0071716308593750,
+-0.0044250488281250, 0.0069885253906250, -0.0049743652343750, 0.0070190429687500,
+-0.0078125000000000, 0.0094604492187500, -0.0068664550781250, 0.0086059570312500,
+-0.0060119628906250, 0.0093994140625000, -0.0073547363281250, 0.0097351074218750,
+-0.0057678222656250, 0.0084838867187500, -0.0034790039062500, 0.0082397460937500,
+-0.0048522949218750, 0.0091247558593750, -0.0019226074218750, 0.0062866210937500,
+-0.0017700195312500, 0.0066528320312500, -0.0017395019531250, 0.0055847167968750,
+-0.0003967285156250, 0.0034179687500000, -0.0008544921875000, 0.0047302246093750,
+0.0000915527343750, 0.0025329589843750, 0.0004272460937500, 0.0025024414062500,
+-0.0000610351562500, 0.0019836425781250, -0.0004882812500000, 0.0005187988281250,
+-0.0005798339843750, 0.0005798339843750, -0.0006713867187500, -0.0022888183593750,
+-0.0008544921875000, -0.0017395019531250, -0.0004577636718750, -0.0010681152343750,
+-0.0003662109375000, -0.0027160644531250, -0.0007629394531250, -0.0028991699218750,
+0.0016174316406250, -0.0014343261718750, 0.0007629394531250, -0.0016784667968750,
+0.0016784667968750, -0.0010070800781250, 0.0038757324218750, -0.0000610351562500,
+0.0025939941406250, -0.0008239746093750, 0.0051269531250000, 0.0013122558593750,
+0.0048217773437500, 0.0008544921875000, 0.0050048828125000, 0.0015563964843750,
+0.0066223144531250, 0.0035095214843750, 0.0060119628906250, 0.0025024414062500,
+0.0073547363281250, 0.0033264160156250, 0.0071716308593750, 0.0043029785156250,
+0.0071716308593750, 0.0027465820312500, 0.0080871582031250, 0.0018310546875000,
+0.0078735351562500, 0.0029907226562500, 0.0075378417968750, -0.0016174316406250,
+0.0076293945312500, -0.0009460449218750, 0.0076904296875000, -0.0015563964843750,
+0.0075378417968750, -0.0049438476562500, 0.0073852539062500, -0.0037536621093750,
+0.0067443847656250, -0.0046081542968750, 0.0078125000000000, -0.0054931640625000,
+0.0058593750000000, -0.0041503906250000, 0.0041503906250000, -0.0032958984375000,
+0.0062866210937500, -0.0041809082031250, 0.0009460449218750, -0.0011901855468750,
+0.0018005371093750, -0.0012207031250000, 0.0010681152343750, -0.0009460449218750,
+-0.0027770996093750, 0.0009765625000000, -0.0008850097656250, 0.0005798339843750,
+-0.0017089843750000, 0.0003662109375000, -0.0029602050781250, 0.0010375976562500,
+-0.0018920898437500, 0.0010681152343750, -0.0010070800781250, 0.0004882812500000,
+-0.0017089843750000, 0.0007629394531250, 0.0004882812500000, 0.0016174316406250,
+-0.0003967285156250, 0.0017395019531250, 0.0003051757812500, 0.0016784667968750,
+0.0022277832031250, 0.0023498535156250, 0.0008850097656250, 0.0025329589843750,
+0.0038146972656250, 0.0010070800781250, 0.0035400390625000, 0.0015258789062500,
+0.0032348632812500, 0.0005493164062500, 0.0049438476562500, -0.0013732910156250,
+0.0046081542968750, -0.0005798339843750, 0.0037231445312500, -0.0020446777343750,
+0.0043640136718750, -0.0025024414062500, 0.0030822753906250, -0.0024719238281250,
+0.0016479492187500, -0.0026245117187500, 0.0030517578125000, -0.0023193359375000,
+0.0006713867187500, -0.0030517578125000, 0.0004882812500000, -0.0023498535156250,
+0.0006103515625000, -0.0027770996093750, -0.0002441406250000, -0.0036621093750000,
+0.0003662109375000, -0.0027160644531250, -0.0006408691406250, -0.0045776367187500,
+-0.0005187988281250, -0.0041503906250000, -0.0004577636718750, -0.0046997070312500,
+-0.0007324218750000, -0.0060729980468750, -0.0005187988281250, -0.0050354003906250,
+-0.0018920898437500, -0.0073242187500000, -0.0012207031250000, -0.0068664550781250,
+-0.0019531250000000, -0.0072326660156250, -0.0035705566406250, -0.0089416503906250,
+-0.0027770996093750, -0.0080566406250000, -0.0043640136718750, -0.0095825195312500,
+-0.0042419433593750, -0.0095520019531250, -0.0043945312500000, -0.0094299316406250,
+-0.0055236816406250, -0.0101928710937500, -0.0049438476562500, -0.0099182128906250,
+-0.0047302246093750, -0.0094909667968750, -0.0054321289062500, -0.0101623535156250,
+-0.0043029785156250, -0.0096740722656250, -0.0030822753906250, -0.0089111328125000,
+-0.0039367675781250, -0.0096130371093750, -0.0019531250000000, -0.0082092285156250,
+-0.0018615722656250, -0.0082397460937500, -0.0017395019531250, -0.0081176757812500,
+-0.0006408691406250, -0.0074462890625000, -0.0010375976562500, -0.0079650878906250,
+-0.0006408691406250, -0.0072937011718750, -0.0004272460937500, -0.0070495605468750,
+-0.0005798339843750, -0.0072326660156250, -0.0005798339843750, -0.0071411132812500,
+-0.0004577636718750, -0.0069274902343750, -0.0006103515625000, -0.0068664550781250,
+-0.0007629394531250, -0.0070800781250000, -0.0003967285156250, -0.0068054199218750,
+-0.0001831054687500, -0.0065307617187500, -0.0005493164062500, -0.0068969726562500,
+-0.0002746582031250, -0.0062561035156250, 0.0000000000000000, -0.0064086914062500,
+-0.0008544921875000, -0.0061950683593750, -0.0013122558593750, -0.0057983398437500,
+-0.0007629394531250, -0.0062255859375000, -0.0029602050781250, -0.0055541992187500,
+-0.0026855468750000, -0.0057373046875000, -0.0033264160156250, -0.0054626464843750,
+-0.0049438476562500, -0.0049133300781250, -0.0042114257812500, -0.0052795410156250,
+-0.0059204101562500, -0.0039672851562500, -0.0060119628906250, -0.0045776367187500,
+-0.0056457519531250, -0.0037231445312500, -0.0063476562500000, -0.0026245117187500,
+-0.0061950683593750, -0.0036315917968750, -0.0051269531250000, -0.0000610351562500,
+-0.0053100585937500, -0.0009765625000000, -0.0047912597656250, 0.0001525878906250,
+-0.0037841796875000, 0.0028991699218750, -0.0040893554687500, 0.0011901855468750,
+-0.0029296875000000, 0.0051269531250000, -0.0031127929687500, 0.0045166015625000,
+-0.0025024414062500, 0.0057067871093750, -0.0014648437500000, 0.0088195800781250,
+-0.0021362304687500, 0.0069885253906250, -0.0005798339843750, 0.0102844238281250,
+-0.0007324218750000, 0.0104675292968750, -0.0003051757812500, 0.0099182128906250,
+0.0005798339843750, 0.0113220214843750, 0.0000305175781250, 0.0111083984375000,
+0.0024108886718750, 0.0095520019531250, 0.0017089843750000, 0.0102539062500000,
+0.0028076171875000, 0.0090637207031250, 0.0052795410156250, 0.0071411132812500,
+0.0039367675781250, 0.0084533691406250, 0.0057678222656250, 0.0065307617187500,
+0.0063781738281250, 0.0061340332031250, 0.0056457519531250, 0.0064086914062500,
+0.0058593750000000, 0.0061035156250000, 0.0061340332031250, 0.0062561035156250,
+0.0046691894531250, 0.0053100585937500, 0.0047912597656250, 0.0057067871093750,
+0.0046997070312500, 0.0052185058593750, 0.0037231445312500, 0.0039978027343750,
+0.0037536621093750, 0.0046386718750000, 0.0031127929687500, 0.0039367675781250,
+0.0035705566406250, 0.0038757324218750, 0.0025634765625000, 0.0034179687500000,
+0.0011596679687500, 0.0027160644531250, 0.0020141601562500, 0.0029907226562500,
+0.0009765625000000, 0.0018310546875000, 0.0002746582031250, 0.0013732910156250,
+0.0008544921875000, 0.0016784667968750, 0.0008850097656250, 0.0015563964843750,
+0.0007324218750000, 0.0010681152343750, 0.0037841796875000, 0.0015258789062500,
+0.0025939941406250, 0.0020141601562500, 0.0042114257812500, 0.0003662109375000,
+0.0076599121093750, -0.0005798339843750, 0.0061645507812500, 0.0006408691406250,
+0.0090942382812500, -0.0032653808593750, 0.0093383789062500, -0.0030517578125000,
+0.0091247558593750, -0.0030822753906250, 0.0104675292968750, -0.0055236816406250,
+0.0102844238281250, -0.0050659179687500, 0.0097351074218750, -0.0049743652343750,
+0.0101928710937500, -0.0053405761718750, 0.0092773437500000, -0.0042419433593750,
+0.0082092285156250, -0.0040588378906250, 0.0090942382812500, -0.0049743652343750,
+0.0074462890625000, 0.0003356933593750, 0.0076904296875000, -0.0000610351562500,
+0.0075683593750000, -0.0005187988281250, 0.0062561035156250, 0.0027770996093750,
+0.0068359375000000, 0.0022583007812500, 0.0079650878906250, -0.0014953613281250,
+0.0071105957031250, 0.0001525878906250, 0.0082092285156250, -0.0016479492187500,
+0.0101928710937500, -0.0063476562500000, 0.0089111328125000, -0.0043029785156250,
+0.0085144042968750, -0.0038757324218750, 0.0100708007812500, -0.0061950683593750,
+0.0086364746093750, -0.0036621093750000, 0.0047912597656250, 0.0022888183593750,
+0.0042419433593750, -0.0011291503906250, 0.0033874511718750, -0.0019226074218750,
+0.0043029785156250, -0.0039367675781250, 0.0027160644531250, -0.0014038085937500,
+0.0013732910156250, 0.0011901855468750, 0.0026855468750000, -0.0007629394531250,
+-0.0009460449218750, 0.0019226074218750, -0.0014343261718750, 0.0028381347656250,
+-0.0006713867187500, 0.0023498535156250, -0.0021057128906250, 0.0022277832031250,
+-0.0025939941406250, 0.0021667480468750, -0.0001831054687500, 0.0047302246093750,
+-0.0002441406250000, 0.0033569335937500, -0.0013122558593750, 0.0049743652343750,
+-0.0006103515625000, 0.0085144042968750, -0.0002136230468750, 0.0065917968750000,
+-0.0026855468750000, 0.0070800781250000, -0.0020446777343750, 0.0089721679687500,
+-0.0027160644531250, 0.0063476562500000, -0.0052490234375000, 0.0033264160156250,
+-0.0042419433593750, 0.0054016113281250, -0.0031738281250000, 0.0029907226562500,
+-0.0040588378906250, 0.0014038085937500, -0.0036315917968750, 0.0036926269531250,
+-0.0021362304687500, 0.0052185058593750, -0.0025024414062500, 0.0037536621093750,
+-0.0025024414062500, 0.0061950683593750, -0.0023193359375000, 0.0072631835937500,
+-0.0015258789062500, 0.0061340332031250, -0.0012512207031250, 0.0058898925781250,
+-0.0018920898437500, 0.0066833496093750, -0.0001831054687500, 0.0049438476562500,
+-0.0002136230468750, 0.0049743652343750, 0.0003967285156250, 0.0048828125000000,
+0.0016174316406250, 0.0043640136718750, 0.0009155273437500, 0.0046691894531250,
+0.0020446777343750, 0.0022888183593750, 0.0024719238281250, 0.0034179687500000,
+0.0018615722656250, 0.0018615722656250, 0.0018005371093750, -0.0013122558593750,
+0.0023193359375000, 0.0003967285156250, 0.0016784667968750, -0.0016174316406250,
+0.0015563964843750, -0.0028991699218750, 0.0015258789062500, -0.0013122558593750,
+0.0010986328125000, -0.0006408691406250, 0.0012817382812500, -0.0015258789062500,
+0.0020446777343750, 0.0016174316406250, 0.0008544921875000, 0.0014648437500000,
+0.0031127929687500, 0.0016784667968750, 0.0052795410156250, 0.0034179687500000,
+0.0030822753906250, 0.0028076171875000, 0.0086059570312500, 0.0038452148437500,
+0.0082702636718750, 0.0037231445312500, 0.0081787109375000, 0.0041503906250000,
+0.0114746093750000, 0.0052795410156250, 0.0105285644531250, 0.0047607421875000,
+0.0097351074218750, 0.0047607421875000, 0.0112915039062500, 0.0054931640625000,
+0.0087585449218750, 0.0045166015625000, 0.0065612792968750, 0.0034179687500000,
+0.0088806152343750, 0.0041809082031250, 0.0010375976562500, 0.0025024414062500,
+0.0028991699218750},
+{0.0071411132812500, -0.0004882812500000, 0.0065307617187500, -0.0005493164062500,
+0.0075988769531250, -0.0014343261718750, 0.0083923339843750, -0.0015869140625000,
+0.0074157714843750, -0.0006408691406250, 0.0086059570312500, -0.0033264160156250,
+0.0091247558593750, -0.0039672851562500, 0.0084838867187500, -0.0026245117187500,
+0.0085144042968750, -0.0029602050781250, 0.0086669921875000, -0.0036621093750000,
+0.0063781738281250, -0.0000305175781250, 0.0070800781250000, -0.0008544921875000,
+0.0061645507812500, -0.0002441406250000, 0.0039062500000000, 0.0029907226562500,
+0.0050659179687500, 0.0018310546875000, 0.0041503906250000, 0.0007934570312500,
+0.0033874511718750, 0.0018920898437500, 0.0038757324218750, 0.0005798339843750,
+0.0045166015625000, -0.0031738281250000, 0.0014343261718750, 0.0011596679687500,
+-0.0032958984375000, 0.0010070800781250, -0.0028991699218750, 0.0033569335937500,
+-0.0024719238281250, 0.0007934570312500, -0.0037231445312500, -0.0027770996093750,
+-0.0040588378906250, -0.0009765625000000, -0.0021362304687500, -0.0014343261718750,
+-0.0025939941406250, -0.0032043457031250, -0.0026245117187500, -0.0012817382812500,
+-0.0012207031250000, 0.0012207031250000, -0.0015869140625000, -0.0002136230468750,
+-0.0023803710937500, 0.0018920898437500, -0.0020446777343750, 0.0022888183593750,
+-0.0015869140625000, 0.0022277832031250, -0.0016784667968750, 0.0025329589843750,
+-0.0018310546875000, 0.0022888183593750, -0.0018615722656250, 0.0049438476562500,
+-0.0013122558593750, 0.0039062500000000, -0.0016479492187500, 0.0048217773437500,
+-0.0019226074218750, 0.0078735351562500, -0.0015869140625000, 0.0065002441406250,
+-0.0043640136718750, 0.0060729980468750, -0.0039367675781250, 0.0074462890625000,
+-0.0032348632812500, 0.0059509277343750, -0.0052490234375000, 0.0036621093750000,
+-0.0054016113281250, 0.0048217773437500, -0.0009765625000000, 0.0042114257812500,
+-0.0023498535156250, 0.0033874511718750, -0.0005798339843750, 0.0042114257812500,
+0.0037841796875000, 0.0051269531250000, 0.0017700195312500, 0.0046386718750000,
+0.0043945312500000, 0.0053100585937500, 0.0053710937500000, 0.0057067871093750,
+0.0041503906250000, 0.0052795410156250, 0.0043640136718750, 0.0051269531250000,
+0.0051269531250000, 0.0054626464843750, 0.0028076171875000, 0.0047607421875000,
+0.0022888183593750, 0.0047607421875000, 0.0035095214843750, 0.0044860839843750,
+0.0031738281250000, 0.0043334960937500, 0.0026855468750000, 0.0046997070312500,
+0.0061035156250000, 0.0025329589843750, 0.0049438476562500, 0.0031127929687500,
+0.0063476562500000, 0.0021972656250000, 0.0095520019531250, -0.0002746582031250,
+0.0074462890625000, 0.0007629394531250, 0.0098266601562500, -0.0002746582031250,
+0.0112304687500000, -0.0010681152343750, 0.0084228515625000, 0.0000305175781250,
+0.0076904296875000, 0.0006713867187500, 0.0096435546875000, -0.0000915527343750,
+0.0014343261718750, 0.0018310546875000, 0.0032653808593750, 0.0019226074218750,
+0.0010681152343750, 0.0020446777343750, -0.0053710937500000, 0.0028381347656250,
+-0.0021362304687500, 0.0025634765625000, -0.0078735351562500, 0.0038452148437500,
+-0.0083618164062500, 0.0036621093750000, -0.0081176757812500, 0.0038757324218750,
+-0.0110473632812500, 0.0049743652343750, -0.0102233886718750, 0.0046997070312500,
+-0.0086669921875000, 0.0039672851562500, -0.0105285644531250, 0.0045166015625000,
+-0.0080566406250000, 0.0038452148437500, -0.0051269531250000, 0.0028381347656250,
+-0.0079040527343750, 0.0033874511718750, -0.0021667480468750, 0.0019836425781250,
+-0.0029907226562500, 0.0021972656250000, -0.0024414062500000, 0.0013732910156250,
+0.0010681152343750, 0.0001525878906250, -0.0010070800781250, 0.0009765625000000,
+0.0015563964843750, -0.0014343261718750, 0.0016784667968750, -0.0013732910156250,
+0.0013122558593750, -0.0014648437500000, 0.0024108886718750, -0.0028381347656250,
+0.0018005371093750, -0.0023498535156250, 0.0015869140625000, -0.0028686523437500,
+0.0018615722656250, -0.0028381347656250, 0.0010986328125000, -0.0031738281250000,
+0.0006103515625000, -0.0036315917968750, 0.0010681152343750, -0.0033569335937500,
+-0.0010070800781250, -0.0039672851562500, -0.0004577636718750, -0.0041809082031250,
+-0.0012817382812500, -0.0039062500000000, -0.0031127929687500, -0.0040283203125000,
+-0.0018615722656250, -0.0041503906250000, -0.0038452148437500, -0.0034790039062500,
+-0.0039367675781250, -0.0035705566406250, -0.0038757324218750, -0.0034179687500000,
+-0.0047302246093750, -0.0030212402343750, -0.0042419433593750, -0.0032348632812500,
+-0.0041198730468750, -0.0025024414062500, -0.0040893554687500, -0.0027160644531250,
+-0.0035705566406250, -0.0024414062500000, -0.0031738281250000, -0.0017089843750000,
+-0.0032958984375000, -0.0021667480468750, -0.0024414062500000, -0.0016784667968750,
+-0.0022583007812500, -0.0015563964843750, -0.0018920898437500, -0.0018615722656250,
+-0.0014343261718750, -0.0019226074218750, -0.0016479492187500, -0.0018615722656250,
+-0.0005493164062500, -0.0025939941406250, -0.0005187988281250, -0.0026855468750000,
+-0.0000610351562500, -0.0025329589843750, 0.0007934570312500, -0.0029602050781250,
+0.0007324218750000, -0.0030822753906250, 0.0020751953125000, -0.0018310546875000,
+0.0018920898437500, -0.0023803710937500, 0.0025024414062500, -0.0019531250000000,
+0.0037536621093750, -0.0005798339843750, 0.0033264160156250, -0.0012207031250000,
+0.0046997070312500, -0.0008850097656250, 0.0049743652343750, -0.0002441406250000,
+0.0050048828125000, -0.0011596679687500, 0.0057678222656250, -0.0018310546875000,
+0.0056762695312500, -0.0010986328125000, 0.0054931640625000, -0.0030822753906250,
+0.0058593750000000, -0.0032043457031250, 0.0052795410156250, -0.0025329589843750,
+0.0047607421875000, -0.0033264160156250, 0.0051879882812500, -0.0035705566406250,
+0.0032958984375000, -0.0010681152343750, 0.0034484863281250, -0.0018615722656250,
+0.0025634765625000, -0.0006103515625000, 0.0007629394531250, 0.0018615722656250,
+0.0011596679687500, 0.0004272460937500, -0.0015258789062500, 0.0029602050781250,
+-0.0014953613281250, 0.0033874511718750, -0.0022277832031250, 0.0027160644531250,
+-0.0042724609375000, 0.0033874511718750, -0.0038146972656250, 0.0035705566406250,
+-0.0050354003906250, 0.0024719238281250, -0.0055236816406250, 0.0026855468750000,
+-0.0052185058593750, 0.0025939941406250, -0.0054626464843750, 0.0017395019531250,
+-0.0055541992187500, 0.0019836425781250, -0.0037536621093750, 0.0021667480468750,
+-0.0041503906250000, 0.0021667480468750, -0.0036926269531250, 0.0016784667968750,
+-0.0020141601562500, 0.0014343261718750, -0.0024414062500000, 0.0019531250000000,
+-0.0023803710937500, 0.0004272460937500, -0.0019836425781250, 0.0004882812500000,
+-0.0019226074218750, 0.0007324218750000, -0.0019531250000000, -0.0000915527343750,
+-0.0020141601562500, 0.0000000000000000, -0.0018920898437500, 0.0009460449218750,
+-0.0016479492187500, 0.0003967285156250, -0.0019226074218750, 0.0015869140625000,
+-0.0022583007812500, 0.0028991699218750, -0.0019836425781250, 0.0016784667968750,
+-0.0020446777343750, 0.0045166015625000, -0.0019226074218750, 0.0044555664062500,
+-0.0019836425781250, 0.0049438476562500, -0.0019531250000000, 0.0065307617187500,
+-0.0016784667968750, 0.0056152343750000, -0.0026245117187500, 0.0073242187500000,
+-0.0024719238281250, 0.0076599121093750, -0.0021362304687500, 0.0074157714843750,
+-0.0026550292968750, 0.0082092285156250, -0.0026855468750000, 0.0080566406250000,
+-0.0013732910156250, 0.0063781738281250, -0.0018005371093750, 0.0071716308593750,
+-0.0011291503906250, 0.0061950683593750, 0.0001831054687500, 0.0041809082031250,
+-0.0005187988281250, 0.0054321289062500, 0.0006713867187500, 0.0034484863281250,
+0.0014648437500000, 0.0030212402343750, -0.0005798339843750, 0.0031738281250000,
+-0.0014648437500000, 0.0028686523437500, -0.0000305175781250, 0.0032348632812500,
+-0.0049743652343750, 0.0017700195312500, -0.0049743652343750, 0.0020751953125000,
+-0.0047302246093750, 0.0015563964843750, -0.0073852539062500, 0.0003356933593750,
+-0.0069580078125000, 0.0010681152343750, -0.0052185058593750, -0.0002136230468750,
+-0.0061950683593750, -0.0004272460937500, -0.0051574707031250, 0.0004272460937500,
+-0.0025939941406250, 0.0007629394531250, -0.0033264160156250, 0.0008239746093750,
+-0.0029296875000000, 0.0021972656250000, -0.0025939941406250, 0.0022583007812500,
+-0.0024108886718750, 0.0019836425781250, -0.0025634765625000, 0.0023803710937500,
+-0.0027160644531250, 0.0024414062500000, -0.0011901855468750, 0.0022583007812500,
+-0.0013427734375000, 0.0028381347656250, -0.0014953613281250, 0.0013732910156250,
+-0.0007934570312500, 0.0003662109375000, -0.0007324218750000, 0.0016174316406250,
+-0.0006713867187500, -0.0031433105468750, -0.0010070800781250, -0.0028381347656250,
+-0.0003356933593750, -0.0027465820312500, 0.0000915527343750, -0.0057678222656250,
+-0.0005187988281250, -0.0054321289062500, 0.0020141601562500, -0.0031127929687500,
+0.0018005371093750, -0.0040588378906250, 0.0016479492187500, -0.0030822753906250,
+0.0028991699218750, -0.0006103515625000, 0.0027465820312500, -0.0018310546875000,
+0.0032958984375000, -0.0011291503906250, 0.0027160644531250, -0.0003662109375000,
+0.0036010742187500, -0.0009460449218750, 0.0046081542968750, -0.0017700195312500,
+0.0036926269531250, -0.0013122558593750, 0.0063476562500000, 0.0000915527343750,
+0.0061340332031250, -0.0008239746093750, 0.0062561035156250, 0.0006408691406250,
+0.0078735351562500, 0.0031433105468750, 0.0073242187500000, 0.0019531250000000,
+0.0076599121093750, 0.0034790039062500, 0.0077514648437500, 0.0046081542968750,
+0.0076904296875000, 0.0035095214843750, 0.0078735351562500, 0.0029296875000000,
+0.0078735351562500, 0.0037536621093750, 0.0072937011718750, 0.0009155273437500,
+0.0083923339843750, 0.0017395019531250, 0.0063781738281250, 0.0005493164062500,
+0.0044860839843750, -0.0021667480468750, 0.0064086914062500, -0.0005798339843750,
+0.0009765625000000, -0.0026550292968750, 0.0021972656250000, -0.0030822753906250,
+0.0007324218750000, -0.0030517578125000, -0.0038757324218750, -0.0035705566406250,
+-0.0013122558593750, -0.0032653808593750, -0.0030212402343750, -0.0049438476562500,
+-0.0044250488281250, -0.0041198730468750, -0.0027770996093750, -0.0053100585937500,
+-0.0020141601562500, -0.0074157714843750, -0.0032043457031250, -0.0060729980468750,
+0.0006103515625000, -0.0089111328125000, -0.0002136230468750, -0.0089416503906250,
+0.0001831054687500, -0.0085449218750000, 0.0028381347656250, -0.0094909667968750,
+0.0015563964843750, -0.0088806152343750, 0.0032348632812500, -0.0079650878906250,
+0.0033874511718750, -0.0078430175781250, 0.0028381347656250, -0.0069580078125000,
+0.0033874511718750, -0.0057983398437500, 0.0035095214843750, -0.0058288574218750,
+0.0026855468750000, -0.0047607421875000, 0.0029602050781250, -0.0044860839843750,
+0.0025024414062500, -0.0040283203125000, 0.0017395019531250, -0.0033264160156250,
+0.0024108886718750, -0.0033874511718750, 0.0014343261718750, -0.0028381347656250,
+0.0013122558593750, -0.0030517578125000, 0.0014038085937500, -0.0028991699218750,
+0.0013122558593750, -0.0025939941406250, 0.0014648437500000, -0.0032043457031250,
+-0.0000305175781250, -0.0026245117187500, 0.0006713867187500, -0.0031433105468750,
+0.0000915527343750, -0.0032043457031250, -0.0015869140625000, -0.0028076171875000,
+-0.0007324218750000, -0.0036315917968750, -0.0016784667968750, -0.0034179687500000,
+-0.0018005371093750, -0.0030517578125000, -0.0019531250000000, -0.0030822753906250,
+-0.0024719238281250},
+{-0.0048217773437500, 0.0021972656250000, -0.0065917968750000, 0.0044860839843750,
+-0.0064697265625000, 0.0040588378906250, -0.0044250488281250, 0.0024108886718750,
+-0.0053710937500000, 0.0032043457031250, -0.0051879882812500, 0.0025024414062500,
+-0.0034179687500000, 0.0011596679687500, -0.0039672851562500, 0.0020141601562500,
+-0.0026245117187500, -0.0018005371093750, -0.0029907226562500, -0.0011901855468750,
+-0.0021057128906250, -0.0018920898437500, -0.0006103515625000, -0.0050659179687500,
+-0.0015869140625000, -0.0038757324218750, -0.0001220703125000, -0.0031738281250000,
+0.0004272460937500, -0.0046386718750000, -0.0001525878906250, -0.0029602050781250,
+-0.0000610351562500, -0.0002441406250000, 0.0003356933593750, -0.0015258789062500,
+-0.0004882812500000, -0.0001831054687500, -0.0009155273437500, 0.0007934570312500,
+-0.0001220703125000, -0.0002136230468750, 0.0001525878906250, -0.0008850097656250,
+-0.0004577636718750, -0.0001831054687500, 0.0013122558593750, -0.0010375976562500,
+0.0011291503906250, -0.0017700195312500, 0.0011596679687500, -0.0001525878906250,
+0.0021057128906250, 0.0009765625000000, 0.0018310546875000, -0.0000305175781250,
+0.0029602050781250, 0.0028991699218750, 0.0021362304687500, 0.0035705566406250,
+0.0035095214843750, 0.0024719238281250, 0.0052490234375000, 0.0031433105468750,
+0.0036621093750000, 0.0038757324218750, 0.0066528320312500, 0.0010375976562500,
+0.0065917968750000, 0.0014038085937500, 0.0064086914062500, 0.0010070800781250,
+0.0079040527343750, -0.0005798339843750, 0.0073852539062500, 0.0003051757812500,
+0.0074462890625000, -0.0013122558593750, 0.0075988769531250, -0.0012512207031250,
+0.0073547363281250, -0.0012512207031250, 0.0074462890625000, -0.0020446777343750,
+0.0075683593750000, -0.0018005371093750, 0.0059204101562500, -0.0032653808593750,
+0.0068054199218750, -0.0027160644531250, 0.0050354003906250, -0.0036621093750000,
+0.0032043457031250, -0.0054931640625000, 0.0052185058593750, -0.0044250488281250,
+-0.0008850097656250, -0.0056762695312500, -0.0001220703125000, -0.0061035156250000,
+-0.0002441406250000, -0.0049438476562500, -0.0040283203125000, -0.0045471191406250,
+-0.0025634765625000, -0.0047302246093750, -0.0029907226562500, -0.0024108886718750,
+-0.0040283203125000, -0.0018615722656250, -0.0023803710937500, -0.0009765625000000,
+-0.0010681152343750, 0.0006408691406250, -0.0023803710937500, 0.0005493164062500,
+0.0007324218750000, 0.0012512207031250, 0.0004272460937500, 0.0014648437500000,
+0.0010070800781250, 0.0015258789062500, 0.0029602050781250, 0.0015258789062500,
+0.0017700195312500, 0.0012512207031250, 0.0036010742187500, 0.0020446777343750,
+0.0036010742187500, 0.0011901855468750, 0.0036621093750000, 0.0017395019531250,
+0.0047302246093750, 0.0025329589843750, 0.0042114257812500, 0.0010681152343750,
+0.0039062500000000, 0.0033264160156250, 0.0045166015625000, 0.0024414062500000,
+0.0038757324218750, 0.0027160644531250, 0.0028076171875000, 0.0044250488281250,
+0.0033264160156250, 0.0031127929687500, 0.0024414062500000, 0.0050048828125000,
+0.0027770996093750, 0.0046081542968750, 0.0017700195312500, 0.0047912597656250,
+0.0007629394531250, 0.0059814453125000, 0.0018615722656250, 0.0050354003906250,
+-0.0011291503906250, 0.0060729980468750, -0.0008850097656250, 0.0063476562500000,
+-0.0011901855468750, 0.0061950683593750, -0.0029907226562500, 0.0065002441406250,
+-0.0022888183593750, 0.0065307617187500, -0.0032958984375000, 0.0066833496093750,
+-0.0033264160156250, 0.0064086914062500, -0.0033264160156250, 0.0064392089843750,
+-0.0038452148437500, 0.0065917968750000, -0.0036315917968750, 0.0062866210937500,
+-0.0035400390625000, 0.0064086914062500, -0.0038146972656250, 0.0060729980468750,
+-0.0034790039062500, 0.0061340332031250, -0.0030517578125000, 0.0061340332031250,
+-0.0032958984375000, 0.0054931640625000, -0.0027160644531250, 0.0062866210937500,
+-0.0025634765625000, 0.0057067871093750, -0.0027160644531250, 0.0057373046875000,
+-0.0025634765625000, 0.0063171386718750, -0.0024719238281250, 0.0054931640625000,
+-0.0027160644531250, 0.0063781738281250, -0.0027160644531250, 0.0057983398437500,
+-0.0025024414062500, 0.0057983398437500, -0.0025024414062500, 0.0063476562500000,
+-0.0024719238281250, 0.0054321289062500, -0.0019226074218750, 0.0060424804687500,
+-0.0019531250000000, 0.0058288574218750, -0.0016784667968750, 0.0053100585937500,
+-0.0012512207031250, 0.0054626464843750, -0.0014038085937500, 0.0052490234375000,
+-0.0006103515625000, 0.0039062500000000, -0.0007324218750000, 0.0043029785156250,
+-0.0001525878906250, 0.0035095214843750, 0.0007019042968750, 0.0022888183593750,
+0.0002441406250000, 0.0030517578125000, 0.0017700195312500, 0.0010986328125000,
+0.0017700195312500, 0.0012512207031250, 0.0019531250000000, 0.0007934570312500,
+0.0028991699218750, -0.0005187988281250, 0.0026550292968750, 0.0001525878906250,
+0.0030212402343750, -0.0010375976562500, 0.0032653808593750, -0.0013427734375000,
+0.0029296875000000, -0.0008544921875000, 0.0029296875000000, -0.0009765625000000,
+0.0032348632812500, -0.0010681152343750, 0.0016479492187500, 0.0003662109375000,
+0.0021667480468750, 0.0002746582031250, 0.0015258789062500, 0.0007324218750000,
+0.0001525878906250, 0.0020141601562500, 0.0009155273437500, 0.0016784667968750,
+-0.0007324218750000, 0.0020751953125000, -0.0006713867187500, 0.0027770996093750,
+-0.0009155273437500, 0.0018920898437500, -0.0021362304687500, 0.0014648437500000,
+-0.0016784667968750, 0.0023193359375000, -0.0014648437500000, -0.0008850097656250,
+-0.0023193359375000, -0.0003662109375000, -0.0010681152343750, -0.0009765625000000,
+0.0001831054687500, -0.0033874511718750, -0.0009460449218750, -0.0027465820312500,
+0.0021972656250000, -0.0042114257812500, 0.0020751953125000, -0.0043029785156250,
+0.0025329589843750, -0.0040283203125000, 0.0045471191406250, -0.0048522949218750,
+0.0039367675781250, -0.0043029785156250, 0.0047302246093750, -0.0015258789062500,
+0.0049743652343750, -0.0034179687500000, 0.0054321289062500, -0.0001220703125000,
+0.0056762695312500, 0.0045471191406250, 0.0051269531250000, 0.0017700195312500,
+0.0075988769531250, 0.0083923339843750, 0.0073547363281250, 0.0089111328125000,
+0.0076904296875000, 0.0089111328125000, 0.0091552734375000, 0.0119934082031250,
+0.0084838867187500, 0.0111694335937500, 0.0096130371093750, 0.0118103027343750,
+0.0097351074218750, 0.0123596191406250, 0.0095825195312500, 0.0111083984375000,
+0.0100097656250000, 0.0106201171875000, 0.0097656250000000, 0.0112915039062500,
+0.0093383789062500, 0.0074462890625000, 0.0100097656250000, 0.0082702636718750,
+0.0088500976562500, 0.0061950683593750, 0.0077209472656250, 0.0024414062500000,
+0.0086364746093750, 0.0044250488281250, 0.0046997070312500, 0.0007324218750000,
+0.0053710937500000, 0.0004882812500000, 0.0050354003906250, 0.0004577636718750,
+0.0018920898437500, -0.0013122558593750, 0.0025939941406250, -0.0004882812500000,
+0.0032958984375000, -0.0004577636718750, 0.0024108886718750, -0.0012207031250000,
+0.0034484863281250, -0.0005493164062500, 0.0051269531250000, 0.0000305175781250,
+0.0038757324218750, -0.0005187988281250, 0.0041809082031250, 0.0024719238281250,
+0.0056762695312500, 0.0021667480468750, 0.0026245117187500, 0.0015563964843750,
+0.0000305175781250, 0.0034790039062500, 0.0024108886718750, 0.0036926269531250,
+-0.0036926269531250, -0.0002441406250000, -0.0045471191406250, 0.0009460449218750,
+-0.0029602050781250, 0.0003967285156250, -0.0046691894531250, -0.0045776367187500,
+-0.0003356933593750, 0.0034484863281250, -0.0005187988281250, -0.0023193359375000,
+-0.0004272460937500, -0.0003662109375000, -0.0003662109375000, -0.0005798339843750,
+-0.0005493164062500, -0.0036315917968750, -0.0007934570312500, -0.0034790039062500,
+-0.0003051757812500, -0.0005187988281250, 0.0003051757812500, -0.0011596679687500,
+-0.0008239746093750, -0.0010375976562500, -0.0011596679687500, 0.0010681152343750,
+-0.0003967285156250, 0.0005187988281250, -0.0052795410156250, -0.0003662109375000,
+-0.0045471191406250, 0.0000000000000000, -0.0043029785156250, 0.0000915527343750,
+-0.0079956054687500, -0.0009460449218750, -0.0076904296875000, -0.0008850097656250,
+-0.0034790039062500, 0.0015258789062500, -0.0059204101562500, 0.0009155273437500,
+-0.0023498535156250, 0.0013427734375000, 0.0034790039062500, 0.0033569335937500,
+-0.0003051757812500, 0.0027770996093750, 0.0063171386718750, 0.0029296875000000,
+0.0075073242187500, 0.0030212402343750, 0.0066223144531250, 0.0032348632812500,
+0.0084533691406250, 0.0031738281250000, 0.0083007812500000, 0.0028686523437500,
+0.0088500976562500, 0.0044250488281250, 0.0085449218750000, 0.0039978027343750,
+0.0087585449218750, 0.0046386718750000, 0.0095825195312500, 0.0061645507812500,
+0.0092468261718750, 0.0053405761718750, 0.0090637207031250, 0.0055541992187500,
+0.0096130371093750, 0.0063781738281250, 0.0081176757812500, 0.0052185058593750,
+0.0073242187500000, 0.0039062500000000, 0.0084228515625000, 0.0046691894531250,
+0.0022583007812500, 0.0040893554687500, 0.0038452148437500, 0.0035095214843750,
+0.0016174316406250, 0.0040283203125000, -0.0038757324218750, 0.0046691894531250,
+-0.0010681152343750, 0.0043945312500000, -0.0057678222656250, 0.0040588378906250,
+-0.0066833496093750, 0.0043945312500000, -0.0049743652343750, 0.0040283203125000,
+-0.0065307617187500, 0.0032348632812500, -0.0072631835937500, 0.0036010742187500,
+-0.0014038085937500, 0.0037231445312500, -0.0034790039062500, 0.0033874511718750,
+-0.0015869140625000, 0.0036621093750000, 0.0039978027343750, 0.0043334960937500,
+0.0011291503906250, 0.0040893554687500, 0.0039367675781250, 0.0032653808593750,
+0.0051879882812500, 0.0038757324218750, 0.0035095214843750, 0.0031433105468750,
+0.0034179687500000, 0.0017395019531250, 0.0043029785156250, 0.0023803710937500,
+0.0003967285156250, 0.0016174316406250, 0.0012817382812500, 0.0012512207031250,
+-0.0001525878906250, 0.0013427734375000, -0.0033874511718750, 0.0012512207031250,
+-0.0015869140625000, 0.0013122558593750, -0.0045166015625000, 0.0012207031250000,
+-0.0053100585937500, 0.0010375976562500, -0.0043334960937500, 0.0014953613281250,
+-0.0050048828125000, 0.0018005371093750, -0.0054626464843750, 0.0015563964843750,
+-0.0031127929687500, 0.0025329589843750, -0.0039672851562500, 0.0026855468750000,
+-0.0030517578125000, 0.0025329589843750, -0.0008850097656250, 0.0028381347656250,
+-0.0023498535156250, 0.0029296875000000, 0.0001220703125000, 0.0025939941406250,
+0.0000610351562500, 0.0026855468750000, -0.0001220703125000, 0.0025024414062500,
+0.0011596679687500, 0.0023498535156250, 0.0007324218750000, 0.0024719238281250,
+0.0007019042968750, 0.0013427734375000, 0.0011901855468750, 0.0016784667968750,
+0.0006103515625000, 0.0010375976562500, 0.0001220703125000, -0.0003662109375000,
+0.0006713867187500, 0.0002441406250000, -0.0009765625000000, -0.0006713867187500,
+-0.0004882812500000, -0.0010375976562500, -0.0006408691406250, -0.0008239746093750,
+-0.0018005371093750, -0.0008850097656250, -0.0011291503906250, -0.0010375976562500,
+-0.0019226074218750, -0.0010375976562500, -0.0016784667968750, -0.0009460449218750,
+-0.0015258789062500, -0.0012207031250000, -0.0017700195312500, -0.0015563964843750,
+-0.0013427734375000, -0.0013732910156250, -0.0016784667968750, -0.0019226074218750,
+-0.0013427734375000, -0.0019531250000000, -0.0014343261718750, -0.0019836425781250,
+-0.0017700195312500, -0.0022583007812500, -0.0011596679687500, -0.0021667480468750,
+-0.0018615722656250},
+{0.0006408691406250, -0.0003967285156250, 0.0072021484375000, -0.0003051757812500,
+0.0071411132812500, -0.0001220703125000, 0.0074768066406250, 0.0002441406250000,
+0.0110778808593750, 0.0003356933593750, 0.0101013183593750, 0.0000000000000000,
+0.0119018554687500, 0.0015869140625000, 0.0118103027343750, 0.0013732910156250,
+0.0119323730468750, 0.0016479492187500, 0.0137023925781250, 0.0027770996093750,
+0.0130920410156250, 0.0022888183593750, 0.0105590820312500, 0.0024719238281250,
+0.0126037597656250, 0.0025939941406250, 0.0090332031250000, 0.0025634765625000,
+0.0045166015625000, 0.0022583007812500, 0.0076599121093750, 0.0022583007812500,
+-0.0006103515625000, 0.0039367675781250, -0.0002441406250000, 0.0033264160156250,
+-0.0007629394531250, 0.0038146972656250, -0.0060424804687500, 0.0056457519531250,
+-0.0042419433593750, 0.0048828125000000, -0.0036926269531250, 0.0039978027343750,
+-0.0056762695312500, 0.0046386718750000, -0.0034179687500000, 0.0041503906250000,
+-0.0009460449218750, 0.0027160644531250, -0.0030212402343750, 0.0030517578125000,
+0.0020141601562500, 0.0033874511718750, 0.0016174316406250, 0.0031433105468750,
+0.0016174316406250, 0.0032348632812500, 0.0048522949218750, 0.0038146972656250,
+0.0037841796875000, 0.0037536621093750, 0.0023803710937500, 0.0030517578125000,
+0.0039672851562500, 0.0032348632812500, 0.0018310546875000, 0.0032653808593750,
+-0.0011901855468750, 0.0026550292968750, 0.0008239746093750, 0.0026855468750000,
+-0.0025634765625000, 0.0039672851562500, -0.0030212402343750, 0.0036315917968750,
+-0.0024108886718750, 0.0035705566406250, -0.0037231445312500, 0.0046691894531250,
+-0.0037231445312500, 0.0046081542968750, -0.0024108886718750, 0.0030517578125000,
+-0.0030822753906250, 0.0035705566406250, -0.0018615722656250, 0.0029296875000000,
+-0.0001525878906250, 0.0013427734375000, -0.0016174316406250, 0.0019531250000000,
+0.0010681152343750, 0.0011291503906250, 0.0013427734375000, 0.0008850097656250,
+0.0002136230468750, 0.0010375976562500, 0.0010070800781250, 0.0007629394531250,
+0.0013427734375000, 0.0007934570312500, -0.0018005371093750, 0.0012512207031250,
+-0.0006103515625000, 0.0010070800781250, -0.0023498535156250, 0.0013427734375000,
+-0.0054016113281250, 0.0019836425781250, -0.0031433105468750, 0.0016784667968750,
+-0.0077209472656250, 0.0020141601562500, -0.0073547363281250, 0.0022583007812500,
+-0.0077209472656250, 0.0019531250000000, -0.0104675292968750, 0.0016479492187500,
+-0.0089721679687500, 0.0018310546875000, -0.0104064941406250, 0.0016784667968750,
+-0.0108337402343750, 0.0014648437500000, -0.0099487304687500, 0.0016784667968750,
+-0.0100708007812500, 0.0019836425781250, -0.0103759765625000, 0.0017700195312500,
+-0.0081481933593750, 0.0016784667968750, -0.0090637207031250, 0.0018920898437500,
+-0.0079040527343750, 0.0014953613281250, -0.0057678222656250, 0.0009765625000000,
+-0.0072631835937500, 0.0012512207031250, -0.0041809082031250, 0.0003662109375000,
+-0.0045776367187500, 0.0005187988281250, -0.0039978027343750, 0.0000610351562500,
+-0.0020446777343750, -0.0006713867187500, -0.0033874511718750, -0.0002441406250000,
+-0.0011901855468750, -0.0015258789062500, -0.0016784667968750, -0.0016174316406250,
+-0.0014648437500000, -0.0016174316406250, -0.0001220703125000, -0.0022583007812500,
+-0.0012207031250000, -0.0021667480468750, -0.0000305175781250, -0.0017089843750000,
+-0.0003051757812500, -0.0020141601562500, -0.0005187988281250, -0.0016174316406250,
+0.0000305175781250, -0.0010375976562500, -0.0005493164062500, -0.0014038085937500,
+-0.0004882812500000, -0.0008239746093750, -0.0008239746093750, -0.0007019042968750,
+-0.0009460449218750, -0.0007934570312500, -0.0009460449218750, -0.0005798339843750,
+-0.0014648437500000, -0.0005798339843750, -0.0013427734375000, -0.0005798339843750,
+-0.0014038085937500, -0.0007324218750000, -0.0017395019531250, -0.0002136230468750,
+-0.0018005371093750, -0.0000610351562500, -0.0018920898437500, -0.0004882812500000,
+-0.0027160644531250, 0.0017700195312500, -0.0024719238281250, 0.0012512207031250,
+-0.0028686523437500, 0.0017395019531250, -0.0036926269531250, 0.0036010742187500,
+-0.0032043457031250, 0.0028076171875000, -0.0038146972656250, 0.0035705566406250,
+-0.0039978027343750, 0.0039367675781250, -0.0035400390625000, 0.0034179687500000,
+-0.0034179687500000, 0.0035705566406250, -0.0033569335937500, 0.0038757324218750,
+-0.0021362304687500, 0.0018615722656250, -0.0022277832031250, 0.0024108886718750,
+-0.0015869140625000, 0.0021972656250000, -0.0004272460937500, 0.0005493164062500,
+-0.0006713867187500, 0.0010681152343750, 0.0006103515625000, 0.0011901855468750,
+0.0007019042968750, 0.0007934570312500, 0.0007629394531250, 0.0011901855468750,
+0.0014343261718750, 0.0016784667968750, 0.0013427734375000, 0.0013732910156250,
+0.0014343261718750, 0.0021667480468750, 0.0014343261718750, 0.0020446777343750,
+0.0016174316406250, 0.0022583007812500, 0.0016784667968750, 0.0028991699218750,
+0.0015258789062500, 0.0026245117187500, 0.0030212402343750, 0.0031738281250000,
+0.0021667480468750, 0.0030212402343750, 0.0036010742187500, 0.0036010742187500,
+0.0056457519531250, 0.0042114257812500, 0.0041503906250000, 0.0036010742187500,
+0.0074768066406250, 0.0046997070312500, 0.0075378417968750, 0.0048828125000000,
+0.0075683593750000, 0.0044555664062500, 0.0094604492187500, 0.0047912597656250,
+0.0089111328125000, 0.0048828125000000, 0.0079345703125000, 0.0027465820312500,
+0.0090637207031250, 0.0033569335937500, 0.0075378417968750, 0.0027465820312500,
+0.0053405761718750, 0.0007629394531250, 0.0066528320312500, 0.0016174316406250,
+0.0036315917968750, 0.0008544921875000, 0.0036621093750000, 0.0001831054687500,
+0.0034179687500000, 0.0010986328125000, 0.0016479492187500, 0.0016784667968750,
+0.0023193359375000, 0.0010070800781250, 0.0021057128906250, 0.0020446777343750,
+0.0018310546875000, 0.0022277832031250, 0.0016174316406250, 0.0015563964843750,
+0.0016784667968750, 0.0016479492187500, 0.0018615722656250, 0.0019836425781250,
+-0.0002441406250000, 0.0006713867187500, 0.0002441406250000, 0.0006713867187500,
+-0.0007629394531250, 0.0008239746093750, -0.0031127929687500, -0.0000610351562500,
+-0.0021362304687500, 0.0000305175781250, -0.0024719238281250, 0.0021362304687500,
+-0.0037231445312500, 0.0014648437500000, -0.0021667480468750, 0.0014343261718750,
+-0.0005798339843750, 0.0030517578125000, -0.0018005371093750, 0.0027465820312500,
+0.0011291503906250, 0.0024719238281250, 0.0015869140625000, 0.0027770996093750,
+0.0003356933593750, 0.0020446777343750, 0.0008544921875000, 0.0016479492187500,
+0.0015258789062500, 0.0021057128906250, -0.0014953613281250, -0.0014953613281250,
+-0.0013122558593750, -0.0007324218750000, -0.0013122558593750, -0.0009155273437500,
+-0.0030517578125000, -0.0037841796875000, -0.0028381347656250, -0.0034484863281250,
+-0.0023498535156250, -0.0028381347656250, -0.0022277832031250, -0.0035095214843750,
+-0.0028991699218750, -0.0023498535156250, -0.0030212402343750, -0.0012512207031250,
+-0.0023803710937500, -0.0023498535156250, -0.0042114257812500, 0.0012207031250000,
+-0.0043029785156250, 0.0012207031250000, -0.0042419433593750, 0.0011291503906250,
+-0.0050964355468750, 0.0032653808593750, -0.0047912597656250, 0.0029602050781250,
+-0.0046386718750000, 0.0017700195312500, -0.0051574707031250, 0.0022583007812500,
+-0.0044250488281250, 0.0021972656250000, -0.0038146972656250, 0.0014038085937500,
+-0.0044860839843750, 0.0017395019531250, -0.0018615722656250, 0.0015869140625000,
+-0.0025024414062500, 0.0016174316406250, -0.0015563964843750, 0.0017089843750000,
+0.0008544921875000, 0.0016174316406250, -0.0004272460937500, 0.0017700195312500,
+0.0012817382812500, 0.0021057128906250, 0.0018005371093750, 0.0019836425781250,
+0.0011291503906250, 0.0021667480468750, 0.0011596679687500, 0.0027770996093750,
+0.0014038085937500, 0.0026855468750000, 0.0008850097656250, 0.0015258789062500,
+0.0006408691406250, 0.0022888183593750, 0.0008850097656250, 0.0014038085937500,
+0.0010070800781250, -0.0000305175781250, 0.0008850097656250, 0.0009765625000000,
+0.0009460449218750, -0.0014038085937500, 0.0007019042968750, -0.0017395019531250,
+0.0010986328125000, -0.0009460449218750, 0.0012207031250000, -0.0016784667968750,
+0.0006408691406250, -0.0019836425781250, 0.0025939941406250, 0.0004272460937500,
+0.0017089843750000, -0.0002136230468750, 0.0026245117187500, 0.0010375976562500,
+0.0046691894531250, 0.0036010742187500, 0.0033874511718750, 0.0024108886718750,
+0.0055847167968750, 0.0043640136718750, 0.0056762695312500, 0.0053710937500000,
+0.0050048828125000, 0.0044555664062500, 0.0057983398437500, 0.0043334960937500,
+0.0059814453125000, 0.0051269531250000, 0.0053405761718750, 0.0032348632812500,
+0.0053710937500000, 0.0027160644531250, 0.0048217773437500, 0.0033264160156250,
+0.0045776367187500, 0.0025634765625000, 0.0052185058593750, 0.0017700195312500,
+0.0028991699218750, 0.0041809082031250, 0.0033569335937500, 0.0028991699218750,
+0.0028686523437500, 0.0034179687500000, 0.0012817382812500, 0.0055236816406250,
+0.0022583007812500, 0.0038146972656250, 0.0004577636718750, 0.0051269531250000,
+0.0004272460937500, 0.0050964355468750, 0.0007629394531250, 0.0040893554687500,
+0.0001220703125000, 0.0041198730468750, 0.0003356933593750, 0.0038757324218750,
+0.0008239746093750, 0.0021972656250000, 0.0005798339843750, 0.0022888183593750,
+0.0008239746093750, 0.0013732910156250, 0.0012817382812500, -0.0002136230468750,
+0.0010070800781250, 0.0003662109375000, 0.0015258789062500, -0.0012817382812500,
+0.0010986328125000, -0.0014953613281250, 0.0017700195312500, -0.0014648437500000,
+0.0025939941406250, -0.0022583007812500, 0.0016479492187500, -0.0020751953125000,
+0.0030517578125000, -0.0017700195312500, 0.0032653808593750, -0.0019226074218750,
+0.0028076171875000, -0.0014343261718750, 0.0030822753906250, -0.0006713867187500,
+0.0030822753906250, -0.0007629394531250, 0.0023803710937500, 0.0000000000000000,
+0.0027160644531250, 0.0002746582031250, 0.0021362304687500, 0.0004272460937500,
+0.0014038085937500, 0.0008850097656250, 0.0020141601562500, 0.0009460449218750,
+0.0003662109375000, 0.0009155273437500, 0.0006103515625000, 0.0013122558593750,
+0.0001831054687500, 0.0014038085937500, -0.0010070800781250, 0.0012512207031250,
+-0.0003356933593750, 0.0014953613281250, -0.0019836425781250, 0.0014953613281250,
+-0.0018310546875000, 0.0018615722656250, -0.0018005371093750, 0.0018005371093750,
+-0.0028076171875000, 0.0017089843750000, -0.0024414062500000, 0.0022888183593750,
+-0.0019836425781250, 0.0018310546875000, -0.0024108886718750, 0.0020751953125000,
+-0.0017089843750000, 0.0020446777343750, -0.0008850097656250, 0.0017700195312500,
+-0.0013732910156250, 0.0021667480468750, -0.0001220703125000, 0.0014343261718750,
+-0.0001525878906250, 0.0018005371093750, 0.0001525878906250, 0.0017700195312500,
+0.0009765625000000, 0.0012817382812500, 0.0005798339843750, 0.0018005371093750,
+0.0015869140625000, 0.0011596679687500, 0.0016479492187500, 0.0015869140625000,
+0.0017395019531250, 0.0010070800781250, 0.0023803710937500, 0.0003662109375000,
+0.0021667480468750, 0.0012207031250000, 0.0023498535156250, -0.0010375976562500,
+0.0024108886718750, -0.0003662109375000, 0.0023498535156250, -0.0011901855468750,
+0.0024108886718750, -0.0032348632812500, 0.0023498535156250, -0.0020751953125000,
+0.0018615722656250, -0.0045776367187500, 0.0020446777343750, -0.0043029785156250,
+0.0016174316406250},
+{0.0007324218750000, 0.0008544921875000, 0.0006713867187500, 0.0019836425781250,
+0.0001831054687500, 0.0051269531250000, 0.0002441406250000, 0.0037841796875000,
+0.0004272460937500, 0.0055236816406250, 0.0001525878906250, 0.0063171386718750,
+0.0006713867187500, 0.0051879882812500, 0.0009155273437500, 0.0049133300781250,
+0.0002746582031250, 0.0054321289062500, 0.0020751953125000, 0.0030517578125000,
+0.0013427734375000, 0.0026550292968750, 0.0023498535156250, 0.0030212402343750,
+0.0043334960937500, 0.0018005371093750, 0.0028991699218750, 0.0010681152343750,
+0.0046997070312500, 0.0030212402343750, 0.0050659179687500, 0.0019226074218750,
+0.0050048828125000, 0.0019836425781250, 0.0056152343750000, 0.0036010742187500,
+0.0052490234375000, 0.0023803710937500, 0.0047912597656250, 0.0023498535156250,
+0.0055236816406250, 0.0023193359375000, 0.0048522949218750, 0.0013122558593750,
+0.0036621093750000, 0.0005493164062500, 0.0042419433593750, 0.0006103515625000,
+0.0030822753906250, -0.0000305175781250, 0.0032348632812500, -0.0005493164062500,
+0.0028076171875000, -0.0002746582031250, 0.0019226074218750, -0.0001831054687500,
+0.0025024414062500, -0.0006408691406250, 0.0012207031250000, 0.0006408691406250,
+0.0014343261718750, 0.0003662109375000, 0.0013427734375000, 0.0010986328125000,
+0.0005187988281250, 0.0023498535156250, 0.0010375976562500, 0.0018920898437500,
+0.0005187988281250, 0.0038146972656250, 0.0005493164062500, 0.0038452148437500,
+0.0007324218750000, 0.0042724609375000, 0.0003967285156250, 0.0055236816406250,
+0.0004577636718750, 0.0050354003906250, 0.0014038085937500, 0.0057067871093750,
+0.0007934570312500, 0.0059204101562500, 0.0014343261718750, 0.0054931640625000,
+0.0025939941406250, 0.0056152343750000, 0.0017700195312500, 0.0057373046875000,
+0.0028076171875000, 0.0047607421875000, 0.0029907226562500, 0.0048522949218750,
+0.0025329589843750, 0.0044555664062500, 0.0029296875000000, 0.0036315917968750,
+0.0029296875000000, 0.0038452148437500, 0.0016784667968750, 0.0027160644531250,
+0.0020751953125000, 0.0030212402343750, 0.0015563964843750, 0.0024108886718750,
+0.0002746582031250, 0.0014648437500000, 0.0008850097656250, 0.0021972656250000,
+0.0000000000000000, 0.0004882812500000, -0.0000915527343750, 0.0006713867187500,
+-0.0001525878906250, 0.0003051757812500, -0.0004577636718750, -0.0007934570312500,
+-0.0001831054687500, -0.0001220703125000, -0.0008850097656250, -0.0013427734375000,
+-0.0007324218750000, -0.0012207031250000, -0.0007019042968750, -0.0013122558593750,
+-0.0012512207031250, -0.0020751953125000, -0.0010070800781250, -0.0016174316406250,
+-0.0004272460937500, -0.0020141601562500, -0.0008544921875000, -0.0021057128906250,
+-0.0001525878906250, -0.0018920898437500, 0.0008239746093750, -0.0019836425781250,
+0.0002441406250000, -0.0020751953125000, 0.0017089843750000, -0.0015258789062500,
+0.0017395019531250, -0.0018615722656250, 0.0018615722656250, -0.0010986328125000,
+0.0028076171875000, -0.0003356933593750, 0.0023803710937500, -0.0010681152343750,
+0.0025939941406250, 0.0010986328125000, 0.0028686523437500, 0.0006713867187500,
+0.0023803710937500, 0.0010681152343750, 0.0019836425781250, 0.0025634765625000,
+0.0023193359375000, 0.0015869140625000, 0.0016174316406250, 0.0027160644531250,
+0.0014343261718750, 0.0028076171875000, 0.0017089843750000, 0.0022277832031250,
+0.0015258789062500, 0.0025024414062500, 0.0014343261718750, 0.0025329589843750,
+0.0022277832031250, 0.0009155273437500, 0.0018920898437500, 0.0016174316406250,
+0.0025024414062500, 0.0001220703125000, 0.0033874511718750, -0.0018310546875000,
+0.0028381347656250, -0.0003662109375000, 0.0042419433593750, -0.0042114257812500,
+0.0042724609375000, -0.0036621093750000, 0.0043029785156250, -0.0038146972656250,
+0.0050048828125000, -0.0064392089843750, 0.0047607421875000, -0.0053100585937500,
+0.0045776367187500, -0.0050048828125000, 0.0046691894531250, -0.0059204101562500,
+0.0048217773437500, -0.0043029785156250, 0.0046386718750000, -0.0025024414062500,
+0.0045471191406250, -0.0035400390625000, 0.0056762695312500, -0.0004882812500000,
+0.0054931640625000, -0.0000915527343750, 0.0057678222656250, -0.0001525878906250,
+0.0067443847656250, 0.0014343261718750, 0.0064086914062500, 0.0015258789062500,
+0.0063476562500000, 0.0006408691406250, 0.0068054199218750, 0.0015869140625000,
+0.0061950683593750, 0.0006408691406250, 0.0056152343750000, -0.0011291503906250,
+0.0061340332031250, -0.0000305175781250, 0.0044860839843750, -0.0013732910156250,
+0.0047302246093750, -0.0028686523437500, 0.0041809082031250, -0.0007324218750000,
+0.0028991699218750, 0.0006103515625000, 0.0034179687500000, -0.0012207031250000,
+0.0015869140625000, 0.0042419433593750, 0.0016784667968750, 0.0045166015625000,
+0.0012817382812500, 0.0037841796875000, 0.0000305175781250, 0.0063171386718750,
+0.0006103515625000, 0.0062866210937500, 0.0000610351562500, 0.0036315917968750,
+-0.0003967285156250, 0.0046081542968750, 0.0003356933593750, 0.0034484863281250,
+0.0012207031250000, 0.0010070800781250, 0.0006408691406250, 0.0023803710937500,
+-0.0003662109375000, 0.0004272460937500, 0.0007934570312500, -0.0003051757812500,
+-0.0004882812500000, 0.0017089843750000, -0.0027770996093750, 0.0018310546875000,
+-0.0014648437500000, 0.0007629394531250, -0.0032348632812500, 0.0074157714843750,
+-0.0043334960937500, 0.0056457519531250, -0.0023498535156250, 0.0084838867187500,
+-0.0013732910156250, 0.0152893066406250, -0.0029296875000000, 0.0119628906250000,
+0.0001525878906250, 0.0147399902343750, 0.0006408691406250, 0.0174560546875000,
+-0.0002746582031250, 0.0133666992187500, 0.0002136230468750, 0.0106506347656250,
+0.0003967285156250, 0.0133972167968750, -0.0005187988281250, 0.0060424804687500,
+-0.0007019042968750, 0.0054016113281250, -0.0010375976562500, 0.0060119628906250,
+-0.0014953613281250, 0.0033264160156250, -0.0011596679687500, 0.0037841796875000,
+-0.0011291503906250, 0.0030517578125000, -0.0016479492187500, 0.0035095214843750,
+-0.0017395019531250, 0.0023498535156250, -0.0013732910156250, -0.0003051757812500,
+-0.0024108886718750, 0.0037536621093750, 0.0016784667968750, 0.0042724609375000,
+-0.0011901855468750, 0.0055847167968750, 0.0016784667968750, 0.0031738281250000,
+0.0058593750000000, 0.0014343261718750, 0.0037231445312500, 0.0033569335937500,
+0.0067443847656250, -0.0017089843750000, 0.0071411132812500, -0.0023498535156250,
+0.0076293945312500, -0.0004882812500000, 0.0088806152343750, -0.0017089843750000,
+0.0079956054687500, -0.0025634765625000, 0.0107421875000000, 0.0013427734375000,
+0.0104064941406250, 0.0016784667968750, 0.0106506347656250, 0.0002136230468750,
+0.0132751464843750, 0.0012817382812500, 0.0125122070312500, 0.0020751953125000,
+0.0091552734375000, -0.0019836425781250, 0.0119323730468750, -0.0019226074218750,
+0.0075683593750000, -0.0017089843750000, 0.0016479492187500, -0.0037231445312500,
+0.0059509277343750, -0.0034790039062500, -0.0033874511718750, -0.0024414062500000,
+-0.0044250488281250, -0.0028991699218750, -0.0022583007812500, -0.0026245117187500,
+-0.0056457519531250, -0.0014953613281250, -0.0060119628906250, -0.0017700195312500,
+-0.0005798339843750, -0.0020446777343750, -0.0021667480468750, -0.0021057128906250,
+-0.0006713867187500, -0.0017089843750000, 0.0041198730468750, -0.0017089843750000,
+0.0018310546875000, -0.0021972656250000, 0.0039062500000000, -0.0007019042968750,
+0.0051879882812500, -0.0007019042968750, 0.0033264160156250, -0.0011901855468750,
+0.0024414062500000, -0.0007324218750000, 0.0036621093750000, -0.0005187988281250,
+0.0008544921875000, -0.0013122558593750, 0.0007324218750000, -0.0017395019531250,
+0.0011291503906250, -0.0008239746093750, 0.0000000000000000, -0.0004272460937500,
+0.0002746582031250, -0.0012207031250000, 0.0013427734375000, 0.0007324218750000,
+0.0007629394531250, 0.0007019042968750, 0.0011901855468750, 0.0006103515625000,
+0.0026855468750000, 0.0015563964843750, 0.0019836425781250, 0.0012512207031250,
+0.0007629394531250, 0.0013732910156250, 0.0023193359375000, 0.0014038085937500,
+0.0001220703125000, 0.0015563964843750, -0.0025939941406250, 0.0017395019531250,
+-0.0003662109375000, 0.0014648437500000, -0.0049743652343750, 0.0022888183593750,
+-0.0048217773437500, 0.0024108886718750, -0.0050964355468750, 0.0021667480468750,
+-0.0077209472656250, 0.0022583007812500, -0.0065307617187500, 0.0024414062500000,
+-0.0075988769531250, 0.0021362304687500, -0.0079650878906250, 0.0018310546875000,
+-0.0071105957031250, 0.0023193359375000, -0.0071411132812500, 0.0027160644531250,
+-0.0074157714843750, 0.0022888183593750, -0.0045166015625000, 0.0030212402343750,
+-0.0051574707031250, 0.0032653808593750, -0.0040893554687500, 0.0028381347656250,
+-0.0014038085937500, 0.0028076171875000, -0.0025634765625000, 0.0029296875000000,
+-0.0007629394531250, 0.0022583007812500, -0.0001831054687500, 0.0022277832031250,
+-0.0005798339843750, 0.0021057128906250, 0.0000610351562500, 0.0017089843750000,
+0.0003967285156250, 0.0018310546875000, -0.0007934570312500, 0.0015869140625000,
+-0.0000915527343750, 0.0015563964843750, -0.0006103515625000, 0.0016174316406250,
+-0.0017700195312500, 0.0015258789062500, -0.0006713867187500, 0.0015563964843750,
+-0.0023498535156250, 0.0017089843750000, -0.0020751953125000, 0.0017700195312500,
+-0.0022277832031250, 0.0016784667968750, -0.0032348632812500, 0.0017395019531250,
+-0.0024414062500000, 0.0019226074218750, -0.0036010742187500, 0.0011901855468750,
+-0.0034179687500000, 0.0013732910156250, -0.0033264160156250, 0.0012207031250000,
+-0.0039367675781250, 0.0006713867187500, -0.0035400390625000, 0.0008544921875000,
+-0.0033569335937500, 0.0005187988281250, -0.0036621093750000, 0.0005493164062500,
+-0.0031738281250000, 0.0003662109375000, -0.0027465820312500, 0.0000000000000000,
+-0.0030517578125000, 0.0002441406250000, -0.0020751953125000, 0.0000000000000000,
+-0.0021972656250000, -0.0001831054687500, -0.0018310546875000, 0.0000610351562500,
+-0.0010070800781250, 0.0002136230468750, -0.0015563964843750, 0.0000610351562500,
+-0.0004882812500000, 0.0002746582031250, -0.0005187988281250, 0.0004577636718750,
+-0.0003662109375000, 0.0003356933593750, 0.0002746582031250, 0.0003051757812500,
+0.0000000000000000, 0.0003662109375000, 0.0005798339843750, 0.0005187988281250,
+0.0007019042968750, 0.0001220703125000, 0.0006408691406250, 0.0009155273437500,
+0.0010070800781250, 0.0016479492187500, 0.0010070800781250, 0.0009765625000000,
+0.0010070800781250, 0.0025024414062500, 0.0011596679687500, 0.0027770996093750,
+0.0010986328125000, 0.0021972656250000, 0.0010070800781250, 0.0024719238281250,
+0.0011596679687500, 0.0028076171875000, 0.0012207031250000, 0.0017700195312500,
+0.0013427734375000, 0.0018310546875000, 0.0017700195312500, 0.0017700195312500,
+0.0019836425781250, 0.0013122558593750, 0.0018920898437500, 0.0015563964843750,
+0.0031433105468750, 0.0008544921875000, 0.0028991699218750, 0.0007629394531250,
+0.0033264160156250, 0.0013122558593750, 0.0044555664062500, 0.0010986328125000,
+0.0040588378906250, 0.0006408691406250, 0.0043640136718750, 0.0025329589843750,
+0.0047912597656250, 0.0020141601562500, 0.0043334960937500, 0.0027160644531250,
+0.0040588378906250, 0.0042114257812500, 0.0043029785156250, 0.0032348632812500,
+0.0027465820312500, 0.0051269531250000, 0.0032653808593750, 0.0051879882812500,
+0.0021362304687500, 0.0048828125000000, 0.0003356933593750, 0.0057373046875000,
+0.0013427734375000},
+{-0.0036010742187500, 0.0013122558593750, -0.0045776367187500, 0.0012512207031250,
+-0.0019531250000000, 0.0008544921875000, -0.0020141601562500, 0.0006408691406250,
+-0.0020141601562500, 0.0010681152343750, -0.0005798339843750, 0.0013427734375000,
+-0.0008544921875000, 0.0010070800781250, -0.0008850097656250, 0.0018920898437500,
+-0.0003356933593750, 0.0019836425781250, -0.0009155273437500, 0.0019531250000000,
+-0.0012512207031250, 0.0021667480468750, -0.0006713867187500, 0.0020751953125000,
+-0.0029602050781250, 0.0025024414062500, -0.0021667480468750, 0.0022888183593750,
+-0.0026245117187500, 0.0024719238281250, -0.0046691894531250, 0.0030212402343750,
+-0.0034484863281250, 0.0026550292968750, -0.0043029785156250, 0.0029602050781250,
+-0.0046691894531250, 0.0030212402343750, -0.0037536621093750, 0.0029602050781250,
+-0.0035705566406250, 0.0029296875000000, -0.0039978027343750, 0.0028991699218750,
+-0.0019226074218750, 0.0028991699218750, -0.0024719238281250, 0.0029296875000000,
+-0.0014648437500000, 0.0026550292968750, 0.0003662109375000, 0.0025939941406250,
+-0.0009155273437500, 0.0028076171875000, 0.0019226074218750, 0.0019226074218750,
+0.0015258789062500, 0.0020141601562500, 0.0020446777343750, 0.0019226074218750,
+0.0039062500000000, 0.0013732910156250, 0.0027160644531250, 0.0015258789062500,
+0.0045776367187500, 0.0011291503906250, 0.0044860839843750, 0.0012512207031250,
+0.0043029785156250, 0.0009765625000000, 0.0051879882812500, 0.0005493164062500,
+0.0048522949218750, 0.0008239746093750, 0.0047607421875000, 0.0003051757812500,
+0.0048522949218750, 0.0001831054687500, 0.0045776367187500, 0.0004272460937500,
+0.0043334960937500, 0.0003662109375000, 0.0045166015625000, 0.0002746582031250,
+0.0041809082031250, 0.0007324218750000, 0.0041198730468750, 0.0007324218750000,
+0.0042419433593750, 0.0007934570312500, 0.0041503906250000, 0.0010375976562500,
+0.0040588378906250, 0.0010070800781250, 0.0043640136718750, 0.0011291503906250,
+0.0045166015625000, 0.0010681152343750, 0.0044555664062500, 0.0012512207031250,
+0.0045776367187500, 0.0014038085937500, 0.0046997070312500, 0.0012512207031250,
+0.0046081542968750, 0.0015563964843750, 0.0047302246093750, 0.0015869140625000,
+0.0046691894531250, 0.0013427734375000, 0.0045776367187500, 0.0013732910156250,
+0.0047912597656250, 0.0016174316406250, 0.0044250488281250, 0.0009155273437500,
+0.0045776367187500, 0.0009460449218750, 0.0042419433593750, 0.0014038085937500,
+0.0038452148437500, 0.0012817382812500, 0.0042114257812500, 0.0010986328125000,
+0.0030822753906250, 0.0017700195312500, 0.0032958984375000, 0.0019531250000000,
+0.0029296875000000, 0.0018005371093750, 0.0019836425781250, 0.0016479492187500,
+0.0024108886718750, 0.0017700195312500, 0.0013427734375000, 0.0023193359375000,
+0.0013732910156250, 0.0016174316406250, 0.0014038085937500, 0.0024108886718750,
+0.0007629394531250, 0.0034179687500000, 0.0007934570312500, 0.0025634765625000,
+0.0009460449218750, 0.0044250488281250, 0.0006408691406250, 0.0041198730468750,
+0.0010986328125000, 0.0044555664062500, 0.0015869140625000, 0.0059814453125000,
+0.0012512207031250, 0.0051574707031250, 0.0026550292968750, 0.0053710937500000,
+0.0025329589843750, 0.0059814453125000, 0.0029296875000000, 0.0052185058593750,
+0.0040893554687500, 0.0043945312500000, 0.0037536621093750, 0.0049133300781250,
+0.0043334960937500, 0.0039978027343750, 0.0048217773437500, 0.0039367675781250,
+0.0043334960937500, 0.0034484863281250, 0.0043640136718750, 0.0028991699218750,
+0.0047302246093750, 0.0034179687500000, 0.0022277832031250, 0.0018310546875000,
+0.0031433105468750, 0.0018005371093750, 0.0016479492187500, 0.0018920898437500,
+-0.0011291503906250, 0.0010986328125000, 0.0003662109375000, 0.0012817382812500,
+-0.0030212402343750, 0.0017395019531250, -0.0032348632812500, 0.0012817382812500,
+-0.0031127929687500, 0.0019226074218750, -0.0046997070312500, 0.0028381347656250,
+-0.0042724609375000, 0.0022888183593750, -0.0037231445312500, 0.0030212402343750,
+-0.0043945312500000, 0.0033569335937500, -0.0031127929687500, 0.0027770996093750,
+-0.0017700195312500, 0.0025329589843750, -0.0028076171875000, 0.0028991699218750,
+-0.0003356933593750, 0.0022583007812500, -0.0000915527343750, 0.0022583007812500,
+-0.0007629394531250, 0.0017395019531250, -0.0002136230468750, 0.0014953613281250,
+0.0000610351562500, 0.0020141601562500, -0.0000915527343750, -0.0005493164062500,
+-0.0007934570312500, 0.0001525878906250, 0.0007019042968750, -0.0006103515625000,
+0.0020446777343750, -0.0026245117187500, 0.0010375976562500, -0.0016784667968750,
+0.0043640136718750, -0.0043945312500000, 0.0043029785156250, -0.0039672851562500,
+0.0047302246093750, -0.0040283203125000, 0.0071716308593750, -0.0062255859375000,
+0.0063171386718750, -0.0058593750000000, 0.0055236816406250, -0.0042114257812500,
+0.0067749023437500, -0.0048522949218750, 0.0051879882812500, -0.0044555664062500,
+0.0029907226562500, -0.0031127929687500, 0.0043334960937500, -0.0037231445312500,
+0.0012817382812500, -0.0024108886718750, 0.0009155273437500, -0.0026550292968750,
+0.0010986328125000, -0.0024108886718750, -0.0000305175781250, -0.0011291503906250,
+0.0003356933593750, -0.0014038085937500, 0.0000305175781250, -0.0013732910156250,
+-0.0001831054687500, -0.0015258789062500, 0.0002441406250000, -0.0008850097656250,
+0.0007019042968750, -0.0001220703125000, 0.0007629394531250, -0.0001831054687500,
+0.0007324218750000, 0.0009460449218750, 0.0009460449218750, 0.0008239746093750,
+0.0004882812500000, 0.0004272460937500, 0.0002746582031250, 0.0009155273437500,
+0.0006408691406250, 0.0012512207031250, -0.0013122558593750, 0.0004577636718750,
+-0.0007934570312500, 0.0003662109375000, -0.0011901855468750, 0.0003662109375000,
+-0.0028076171875000, 0.0001525878906250, -0.0019836425781250, 0.0003662109375000,
+-0.0027160644531250, 0.0001220703125000, -0.0029907226562500, -0.0001220703125000,
+-0.0025634765625000, 0.0002136230468750, -0.0022277832031250, 0.0004577636718750,
+-0.0025329589843750, 0.0001220703125000, -0.0036010742187500, 0.0009155273437500,
+-0.0028686523437500, 0.0009765625000000, -0.0039062500000000, 0.0011596679687500,
+-0.0058898925781250, 0.0013732910156250, -0.0049438476562500, 0.0013427734375000,
+-0.0059204101562500, 0.0031127929687500, -0.0065002441406250, 0.0026550292968750,
+-0.0062561035156250, 0.0027160644531250, -0.0062866210937500, 0.0043029785156250,
+-0.0063476562500000, 0.0040283203125000, -0.0059814453125000, 0.0030517578125000,
+-0.0063781738281250, 0.0035705566406250, -0.0054931640625000, 0.0028686523437500,
+-0.0048522949218750, 0.0014953613281250, -0.0058288574218750, 0.0019226074218750,
+-0.0021972656250000, 0.0008544921875000, -0.0031127929687500, 0.0007324218750000,
+-0.0021972656250000, 0.0003967285156250, 0.0008239746093750, -0.0003356933593750,
+-0.0007324218750000, -0.0001525878906250, 0.0008850097656250, -0.0009460449218750,
+0.0015258789062500, -0.0014648437500000, 0.0010375976562500, -0.0013122558593750,
+0.0011596679687500, -0.0015258789062500, 0.0012817382812500, -0.0019226074218750,
+0.0010375976562500, -0.0009155273437500, 0.0010681152343750, -0.0014648437500000,
+0.0013122558593750, -0.0008544921875000, 0.0013732910156250, 0.0001831054687500,
+0.0012512207031250, -0.0005493164062500, 0.0014038085937500, 0.0019531250000000,
+0.0012817382812500, 0.0016784667968750, 0.0017089843750000, 0.0025634765625000,
+0.0020751953125000, 0.0047912597656250, 0.0015563964843750, 0.0037841796875000,
+0.0025024414062500, 0.0049133300781250, 0.0022888183593750, 0.0055541992187500,
+0.0026245117187500, 0.0047607421875000, 0.0034179687500000, 0.0044250488281250,
+0.0028991699218750, 0.0048828125000000, 0.0029296875000000, 0.0033264160156250,
+0.0034179687500000, 0.0034790039062500, 0.0031433105468750, 0.0032958984375000,
+0.0025329589843750, 0.0021667480468750, 0.0027160644531250, 0.0025634765625000,
+0.0028686523437500, 0.0025939941406250, 0.0025024414062500, 0.0020446777343750,
+0.0028686523437500, 0.0025329589843750, 0.0033569335937500, 0.0028991699218750,
+0.0029907226562500, 0.0020446777343750, 0.0031433105468750, 0.0033874511718750,
+0.0036926269531250, 0.0028381347656250, 0.0030212402343750, 0.0029296875000000,
+0.0024719238281250, 0.0038757324218750, 0.0031433105468750, 0.0029907226562500,
+0.0014953613281250, 0.0038757324218750, 0.0018920898437500, 0.0035705566406250,
+0.0014953613281250, 0.0032348632812500, 0.0001525878906250, 0.0036010742187500,
+0.0009765625000000, 0.0031127929687500, -0.0000915527343750, 0.0028686523437500,
+-0.0002441406250000, 0.0031738281250000, 0.0000305175781250, 0.0024108886718750,
+-0.0002441406250000, 0.0018005371093750, -0.0002136230468750, 0.0025329589843750,
+0.0005493164062500, 0.0008850097656250, 0.0002746582031250, 0.0007934570312500,
+0.0007019042968750, 0.0004882812500000, 0.0016784667968750, -0.0003051757812500,
+0.0011596679687500, 0.0002136230468750, 0.0017395019531250, -0.0007629394531250,
+0.0019531250000000, -0.0007934570312500, 0.0017700195312500, -0.0009765625000000,
+0.0016784667968750, -0.0014953613281250, 0.0018615722656250, -0.0012207031250000,
+0.0018310546875000, -0.0017395019531250, 0.0018310546875000, -0.0016174316406250,
+0.0018920898437500, -0.0017700195312500, 0.0019836425781250, -0.0021972656250000,
+0.0020446777343750, -0.0020751953125000, 0.0017395019531250, -0.0022888183593750,
+0.0019531250000000, -0.0024108886718750, 0.0018005371093750, -0.0022888183593750,
+0.0014038085937500, -0.0024108886718750, 0.0016174316406250, -0.0025634765625000,
+0.0013122558593750, -0.0018615722656250, 0.0011901855468750, -0.0021362304687500,
+0.0014953613281250, -0.0019531250000000, 0.0015563964843750, -0.0013732910156250,
+0.0014038085937500, -0.0018615722656250, 0.0024414062500000, -0.0010986328125000,
+0.0022277832031250, -0.0011596679687500, 0.0026550292968750, -0.0012512207031250,
+0.0035400390625000, -0.0006713867187500, 0.0032043457031250, -0.0008239746093750,
+0.0043640136718750, -0.0018005371093750, 0.0042724609375000, -0.0011291503906250,
+0.0046386718750000, -0.0021362304687500, 0.0054931640625000, -0.0034484863281250,
+0.0050964355468750, -0.0023803710937500, 0.0055541992187500, -0.0050048828125000,
+0.0059204101562500, -0.0047912597656250, 0.0054016113281250, -0.0051269531250000,
+0.0050659179687500, -0.0069580078125000, 0.0054321289062500, -0.0063171386718750,
+0.0048522949218750, -0.0066528320312500, 0.0046691894531250, -0.0075683593750000,
+0.0048522949218750, -0.0062255859375000, 0.0049438476562500, -0.0053710937500000,
+0.0049133300781250, -0.0064697265625000, 0.0043945312500000, -0.0026855468750000,
+0.0048217773437500, -0.0033569335937500, 0.0042114257812500, -0.0021667480468750,
+0.0032348632812500, 0.0008850097656250, 0.0038146972656250, -0.0008239746093750,
+0.0028991699218750, 0.0014953613281250, 0.0027465820312500, 0.0018920898437500,
+0.0028076171875000, 0.0010375976562500, 0.0025329589843750, 0.0015258789062500,
+0.0024719238281250, 0.0015563964843750, 0.0022277832031250, -0.0001525878906250,
+0.0024108886718750, 0.0002441406250000, 0.0018005371093750, -0.0004272460937500,
+0.0011901855468750, -0.0020141601562500, 0.0016479492187500, -0.0010986328125000,
+-0.0000915527343750, -0.0016479492187500, 0.0001220703125000, -0.0019531250000000,
+-0.0005798339843750, -0.0007019042968750, -0.0019531250000000, 0.0002441406250000,
+-0.0011901855468750, -0.0002441406250000, -0.0033569335937500, 0.0010681152343750,
+-0.0033569335937500},
+{0.0024414062500000, 0.0014953613281250, 0.0023498535156250, 0.0012512207031250,
+0.0022888183593750, 0.0007324218750000, 0.0028381347656250, 0.0000915527343750,
+0.0028381347656250, 0.0000000000000000, 0.0021667480468750, -0.0007019042968750,
+0.0026245117187500, -0.0009155273437500, 0.0022888183593750, -0.0012207031250000,
+0.0014953613281250, -0.0017700195312500, 0.0021667480468750, -0.0018310546875000,
+0.0012817382812500, -0.0019531250000000, 0.0013732910156250, -0.0022277832031250,
+0.0015258789062500, -0.0024414062500000, 0.0011596679687500, -0.0024414062500000,
+0.0013122558593750, -0.0023803710937500, 0.0010070800781250, -0.0024719238281250,
+0.0011901855468750, -0.0022888183593750, 0.0011596679687500, -0.0021057128906250,
+0.0007934570312500, -0.0021667480468750, 0.0009460449218750, -0.0020141601562500,
+0.0014648437500000, -0.0018005371093750, 0.0012817382812500, -0.0019531250000000,
+0.0015869140625000, -0.0019226074218750, 0.0022583007812500, -0.0017700195312500,
+0.0019531250000000, -0.0020141601562500, 0.0025634765625000, -0.0021362304687500,
+0.0025329589843750, -0.0021667480468750, 0.0026245117187500, -0.0024719238281250,
+0.0029296875000000, -0.0027465820312500, 0.0027770996093750, -0.0028991699218750,
+0.0028686523437500, -0.0031433105468750, 0.0030822753906250, -0.0034484863281250,
+0.0027465820312500, -0.0036926269531250, 0.0025024414062500, -0.0038146972656250,
+0.0027770996093750, -0.0039978027343750, 0.0023803710937500, -0.0040588378906250,
+0.0022888183593750, -0.0041809082031250, 0.0025939941406250, -0.0043334960937500,
+0.0025634765625000, -0.0044860839843750, 0.0025024414062500, -0.0044555664062500,
+0.0034790039062500, -0.0041503906250000, 0.0031433105468750, -0.0046997070312500,
+0.0037231445312500, -0.0042419433593750, 0.0046997070312500, -0.0037536621093750,
+0.0041809082031250, -0.0045166015625000, 0.0051879882812500, -0.0021972656250000,
+0.0053405761718750, -0.0031433105468750, 0.0050354003906250, -0.0024108886718750,
+0.0051574707031250, -0.0004272460937500, 0.0052185058593750, -0.0020751953125000,
+0.0046691894531250, 0.0002441406250000, 0.0046386718750000, 0.0000305175781250,
+0.0046691894531250, -0.0004882812500000, 0.0045166015625000, 0.0005493164062500,
+0.0044860839843750, 0.0000915527343750, 0.0041198730468750, -0.0009460449218750,
+0.0044860839843750, -0.0004272460937500, 0.0039062500000000, -0.0016784667968750,
+0.0032043457031250, -0.0032958984375000, 0.0037536621093750, -0.0025939941406250,
+0.0025939941406250, -0.0052185058593750, 0.0026245117187500, -0.0053710937500000,
+0.0025939941406250, -0.0054016113281250, 0.0020446777343750, -0.0068664550781250,
+0.0022583007812500, -0.0064392089843750, 0.0015869140625000, -0.0054016113281250,
+0.0017395019531250, -0.0065002441406250, 0.0012512207031250, -0.0050048828125000,
+0.0005493164062500, -0.0030822753906250, 0.0010070800781250, -0.0046081542968750,
+-0.0003662109375000, -0.0008239746093750, -0.0003051757812500, -0.0012207031250000,
+-0.0005493164062500, -0.0007629394531250, -0.0013122558593750, 0.0017395019531250,
+-0.0009460449218750, 0.0004272460937500, -0.0020751953125000, 0.0010375976562500,
+-0.0018310546875000, 0.0019836425781250, -0.0021362304687500, 0.0008239746093750,
+-0.0033264160156250, -0.0001525878906250, -0.0028076171875000, 0.0009155273437500,
+-0.0024414062500000, -0.0002746582031250, -0.0030822753906250, -0.0003967285156250,
+-0.0027160644531250, 0.0000915527343750, -0.0017089843750000, -0.0001220703125000,
+-0.0020446777343750, -0.0000915527343750, -0.0022277832031250, 0.0007629394531250,
+-0.0021057128906250, 0.0003967285156250, -0.0026550292968750, 0.0007324218750000,
+-0.0030212402343750, 0.0016784667968750, -0.0026855468750000, 0.0011596679687500,
+-0.0038452148437500, 0.0011596679687500, -0.0042114257812500, 0.0018615722656250,
+-0.0037536621093750, 0.0000000000000000, -0.0037231445312500, -0.0017089843750000,
+-0.0039672851562500, -0.0004882812500000, -0.0033264160156250, -0.0041198730468750,
+-0.0035095214843750, -0.0042114257812500, -0.0031127929687500, -0.0031127929687500,
+-0.0025939941406250, -0.0047912597656250, -0.0031738281250000, -0.0048217773437500,
+-0.0013732910156250, -0.0005798339843750, -0.0019531250000000, -0.0018310546875000,
+-0.0009155273437500, 0.0003051757812500, 0.0004272460937500, 0.0048522949218750,
+-0.0007629394531250, 0.0027465820312500, 0.0046386718750000, 0.0054931640625000,
+0.0036621093750000, 0.0066223144531250, 0.0045471191406250, 0.0057983398437500,
+0.0086975097656250, 0.0058288574218750, 0.0071716308593750, 0.0061950683593750,
+0.0085144042968750, 0.0067749023437500, 0.0093688964843750, 0.0060424804687500,
+0.0084533691406250, 0.0071105957031250, 0.0075073242187500, 0.0097656250000000,
+0.0029907226562500, 0.0053405761718750, 0.0008239746093750, 0.0059814453125000,
+0.0017089843750000, 0.0045471191406250, 0.0004882812500000, 0.0062255859375000,
+-0.0009765625000000, 0.0086669921875000, 0.0000915527343750, 0.0074462890625000,
+-0.0017700195312500, 0.0074157714843750, -0.0021667480468750, 0.0087585449218750,
+-0.0018005371093750, 0.0076293945312500, -0.0022277832031250, 0.0057983398437500,
+-0.0021667480468750, 0.0067443847656250, -0.0018310546875000, 0.0058593750000000,
+-0.0016784667968750, 0.0051879882812500, -0.0024108886718750, 0.0057678222656250,
+-0.0027465820312500, 0.0064086914062500, -0.0020141601562500, 0.0058898925781250,
+-0.0033569335937500, 0.0049743652343750, -0.0039672851562500, 0.0061035156250000,
+-0.0028076171875000, 0.0044860839843750, -0.0026855468750000, 0.0023193359375000,
+-0.0033569335937500, 0.0037841796875000, 0.0008544921875000, 0.0000610351562500,
+-0.0000915527343750, -0.0000610351562500, 0.0002441406250000, 0.0002441406250000,
+0.0036621093750000, -0.0014038085937500, 0.0025024414062500, -0.0011901855468750,
+0.0007019042968750, -0.0010681152343750, 0.0023498535156250, -0.0010375976562500,
+0.0003356933593750, -0.0014343261718750, -0.0030212402343750, -0.0017700195312500,
+-0.0010986328125000, -0.0014648437500000, -0.0039978027343750, -0.0021057128906250,
+-0.0049743652343750, -0.0024108886718750, -0.0033569335937500, -0.0019836425781250,
+-0.0034179687500000, -0.0018615722656250, -0.0044860839843750, -0.0022583007812500,
+-0.0008239746093750, -0.0016784667968750, -0.0011291503906250, -0.0016174316406250,
+-0.0010681152343750, -0.0018310546875000, 0.0009765625000000, -0.0017395019531250,
+0.0001525878906250, -0.0017395019531250, 0.0001525878906250, -0.0019226074218750,
+0.0011291503906250, -0.0019836425781250, -0.0003051757812500, -0.0017089843750000,
+-0.0015258789062500, -0.0016174316406250, -0.0001525878906250, -0.0017700195312500,
+-0.0033874511718750, -0.0010375976562500, -0.0034790039062500, -0.0009765625000000,
+-0.0026245117187500, -0.0012207031250000, -0.0039062500000000, -0.0012512207031250,
+-0.0039367675781250, -0.0011291503906250, -0.0015869140625000, -0.0008544921875000,
+-0.0023498535156250, -0.0013122558593750, -0.0012512207031250, -0.0006408691406250,
+0.0009460449218750, 0.0003356933593750, -0.0003051757812500, -0.0003051757812500,
+0.0019531250000000, 0.0003967285156250, 0.0020141601562500, 0.0007019042968750,
+0.0021057128906250, 0.0004577636718750, 0.0034790039062500, 0.0003356933593750,
+0.0028381347656250, 0.0004577636718750, 0.0023193359375000, 0.0007629394531250,
+0.0033569335937500, 0.0005187988281250, 0.0018005371093750, 0.0009765625000000,
+0.0003051757812500, 0.0017089843750000, 0.0019836425781250, 0.0012512207031250,
+-0.0018310546875000, 0.0021667480468750, -0.0010681152343750, 0.0021667480468750,
+-0.0017089843750000, 0.0022277832031250, -0.0042419433593750, 0.0027465820312500,
+-0.0024719238281250, 0.0025329589843750, -0.0046997070312500, 0.0028686523437500,
+-0.0046386718750000, 0.0029296875000000, -0.0043029785156250, 0.0028686523437500,
+-0.0052795410156250, 0.0030517578125000, -0.0047607421875000, 0.0030517578125000,
+-0.0046997070312500, 0.0026245117187500, -0.0047302246093750, 0.0027160644531250,
+-0.0043640136718750, 0.0027465820312500, -0.0041198730468750, 0.0025329589843750,
+-0.0041198730468750, 0.0026245117187500, -0.0029602050781250, 0.0026550292968750,
+-0.0033569335937500, 0.0026550292968750, -0.0027465820312500, 0.0025024414062500,
+-0.0016479492187500, 0.0024108886718750, -0.0025939941406250, 0.0025329589843750,
+-0.0011291503906250, 0.0022888183593750, -0.0012207031250000, 0.0022583007812500,
+-0.0012207031250000, 0.0023803710937500, -0.0004577636718750, 0.0024108886718750,
+-0.0008239746093750, 0.0023498535156250, -0.0006713867187500, 0.0024108886718750,
+-0.0004577636718750, 0.0025634765625000, -0.0007629394531250, 0.0022888183593750,
+-0.0008850097656250, 0.0021362304687500, -0.0005493164062500, 0.0023498535156250,
+-0.0013427734375000, 0.0017700195312500, -0.0010986328125000, 0.0018615722656250,
+-0.0013732910156250, 0.0017089843750000, -0.0019836425781250, 0.0012207031250000,
+-0.0014038085937500, 0.0014953613281250, -0.0024414062500000, 0.0013427734375000,
+-0.0023193359375000, 0.0010986328125000, -0.0024719238281250, 0.0014648437500000,
+-0.0032043457031250, 0.0018310546875000, -0.0028686523437500, 0.0015258789062500,
+-0.0033569335937500, 0.0018310546875000, -0.0033569335937500, 0.0020446777343750,
+-0.0033874511718750, 0.0018920898437500, -0.0036315917968750, 0.0017089843750000,
+-0.0035400390625000, 0.0017700195312500, -0.0036621093750000, 0.0022888183593750,
+-0.0038757324218750, 0.0017089843750000, -0.0039672851562500, 0.0023803710937500,
+-0.0041503906250000, 0.0032043457031250, -0.0043334960937500, 0.0024414062500000,
+-0.0042114257812500, 0.0044250488281250, -0.0043029785156250, 0.0041809082031250,
+-0.0039672851562500, 0.0042114257812500, -0.0036621093750000, 0.0056457519531250,
+-0.0037536621093750, 0.0051574707031250, -0.0027465820312500, 0.0043029785156250,
+-0.0027770996093750, 0.0051269531250000, -0.0021667480468750, 0.0041809082031250,
+-0.0012817382812500, 0.0026855468750000, -0.0015258789062500, 0.0035705566406250,
+0.0002441406250000, 0.0020141601562500, 0.0000305175781250, 0.0017395019531250,
+0.0004272460937500, 0.0019226074218750, 0.0017700195312500, 0.0013122558593750,
+0.0012817382812500, 0.0014343261718750, 0.0016479492187500, 0.0017395019531250,
+0.0020751953125000, 0.0014953613281250, 0.0015869140625000, 0.0017089843750000,
+0.0014038085937500, 0.0021972656250000, 0.0018310546875000, 0.0019226074218750,
+0.0003051757812500, 0.0020751953125000, 0.0006408691406250, 0.0020141601562500,
+0.0004272460937500, 0.0025024414062500, -0.0007629394531250, 0.0029296875000000,
+-0.0003051757812500, 0.0024108886718750, -0.0007324218750000, 0.0033569335937500,
+-0.0008544921875000, 0.0035705566406250, -0.0006713867187500, 0.0030212402343750,
+-0.0007324218750000, 0.0031738281250000, -0.0007324218750000, 0.0034179687500000,
+-0.0001831054687500, 0.0014953613281250, -0.0002746582031250, 0.0022583007812500,
+-0.0001525878906250, 0.0008239746093750, 0.0002441406250000, -0.0014038085937500,
+0.0000610351562500, 0.0001831054687500, -0.0002136230468750, -0.0029296875000000,
+-0.0000610351562500, -0.0032043457031250, -0.0001220703125000, -0.0027160644531250,
+-0.0006713867187500, -0.0037841796875000, -0.0007934570312500, -0.0037231445312500,
+-0.0001220703125000, -0.0032958984375000, -0.0004272460937500, -0.0033874511718750,
+-0.0003967285156250, -0.0030822753906250, 0.0001220703125000, -0.0030212402343750,
+-0.0001220703125000, -0.0033569335937500, 0.0003662109375000, -0.0009460449218750,
+0.0004882812500000, -0.0012817382812500, 0.0002746582031250, -0.0020751953125000,
+0.0003356933593750},
+{-0.0036010742187500, 0.0034179687500000, -0.0039978027343750, 0.0039672851562500,
+-0.0036315917968750, 0.0037231445312500, -0.0042724609375000, 0.0038146972656250,
+-0.0039672851562500, 0.0040588378906250, -0.0042724609375000, 0.0036010742187500,
+-0.0049438476562500, 0.0031738281250000, -0.0043640136718750, 0.0034790039062500,
+-0.0053710937500000, 0.0022277832031250, -0.0050048828125000, 0.0023498535156250,
+-0.0052185058593750, 0.0021972656250000, -0.0058593750000000, 0.0014648437500000,
+-0.0051574707031250, 0.0017395019531250, -0.0061340332031250, 0.0016174316406250,
+-0.0058288574218750, 0.0014648437500000, -0.0058593750000000, 0.0018005371093750,
+-0.0063781738281250, 0.0020141601562500, -0.0057373046875000, 0.0018005371093750,
+-0.0063781738281250, 0.0025024414062500, -0.0060119628906250, 0.0025634765625000,
+-0.0058593750000000, 0.0024108886718750, -0.0061035156250000, 0.0026855468750000,
+-0.0054931640625000, 0.0027465820312500, -0.0056152343750000, 0.0025329589843750,
+-0.0054016113281250, 0.0023498535156250, -0.0049438476562500, 0.0027160644531250,
+-0.0047302246093750, 0.0028076171875000, -0.0043945312500000, 0.0023803710937500,
+-0.0034179687500000, 0.0034179687500000, -0.0036010742187500, 0.0032043457031250,
+-0.0031433105468750, 0.0033264160156250, -0.0023193359375000, 0.0040893554687500,
+-0.0028991699218750, 0.0036315917968750, -0.0019836425781250, 0.0034790039062500,
+-0.0019836425781250, 0.0040588378906250, -0.0021972656250000, 0.0031738281250000,
+-0.0018920898437500, 0.0022583007812500, -0.0020141601562500, 0.0031127929687500,
+-0.0022888183593750, 0.0014343261718750, -0.0021667480468750, 0.0013427734375000,
+-0.0024719238281250, 0.0015563964843750, -0.0027770996093750, 0.0007629394531250,
+-0.0025939941406250, 0.0009460449218750, -0.0032958984375000, 0.0015869140625000,
+-0.0031433105468750, 0.0012207031250000, -0.0032043457031250, 0.0014953613281250,
+-0.0037231445312500, 0.0021362304687500, -0.0033874511718750, 0.0018005371093750,
+-0.0035705566406250, 0.0021972656250000, -0.0036621093750000, 0.0023193359375000,
+-0.0033264160156250, 0.0021972656250000, -0.0032348632812500, 0.0021667480468750,
+-0.0034179687500000, 0.0021972656250000, -0.0019836425781250, 0.0029602050781250,
+-0.0020446777343750, 0.0027160644531250, -0.0018310546875000, 0.0026550292968750,
+-0.0007934570312500, 0.0033264160156250, -0.0009155273437500, 0.0032348632812500,
+-0.0007934570312500, 0.0019836425781250, -0.0004577636718750, 0.0026550292968750,
+-0.0008544921875000, 0.0014953613281250, -0.0010986328125000, -0.0001525878906250,
+-0.0007934570312500, 0.0008850097656250, -0.0016174316406250, -0.0021057128906250,
+-0.0016479492187500, -0.0018920898437500, -0.0013732910156250, -0.0020751953125000,
+-0.0017395019531250, -0.0039978027343750, -0.0018310546875000, -0.0033569335937500,
+-0.0004577636718750, -0.0027465820312500, -0.0008544921875000, -0.0034484863281250,
+-0.0003662109375000, -0.0026855468750000, 0.0009155273437500, -0.0014343261718750,
+0.0004272460937500, -0.0021667480468750, 0.0012512207031250, -0.0009765625000000,
+0.0014343261718750, -0.0010986328125000, 0.0008544921875000, -0.0006408691406250,
+0.0008850097656250, 0.0005798339843750, 0.0011291503906250, -0.0003051757812500,
+-0.0002441406250000, -0.0013732910156250, -0.0003356933593750, -0.0000305175781250,
+-0.0000610351562500, -0.0014648437500000, -0.0005493164062500, -0.0040588378906250,
+-0.0006408691406250, -0.0028381347656250, -0.0001831054687500, -0.0037841796875000,
+-0.0001525878906250, -0.0047607421875000, -0.0002136230468750, -0.0035095214843750,
+-0.0001220703125000, -0.0027160644531250, -0.0000610351562500, -0.0036315917968750,
+0.0000000000000000, -0.0007934570312500, -0.0002441406250000, -0.0009765625000000,
+0.0004882812500000, -0.0012207031250000, 0.0011901855468750, 0.0002136230468750,
+0.0006713867187500, -0.0001525878906250, 0.0015258789062500, -0.0006713867187500,
+0.0018920898437500, -0.0001831054687500, 0.0013122558593750, -0.0010986328125000,
+0.0010986328125000, -0.0029296875000000, 0.0013427734375000, -0.0027160644531250,
+-0.0002441406250000, -0.0018005371093750, 0.0000915527343750, -0.0026550292968750,
+0.0003662109375000, -0.0018615722656250, -0.0003967285156250, -0.0007934570312500,
+-0.0004882812500000, -0.0018005371093750, -0.0004882812500000, 0.0015869140625000,
+0.0000000000000000, 0.0012207031250000, -0.0008544921875000, 0.0015869140625000,
+-0.0018615722656250, 0.0040588378906250, -0.0010375976562500, 0.0032958984375000,
+-0.0025024414062500, 0.0026855468750000, -0.0028991699218750, 0.0037536621093750,
+-0.0023193359375000, 0.0025939941406250, -0.0025024414062500, 0.0008239746093750,
+-0.0027770996093750, 0.0017395019531250, -0.0020446777343750, 0.0002746582031250,
+-0.0018310546875000, -0.0002136230468750, -0.0018615722656250, 0.0005187988281250,
+-0.0017700195312500, 0.0003967285156250, -0.0016479492187500, -0.0001220703125000,
+-0.0013427734375000, 0.0010986328125000, -0.0015563964843750, 0.0011291503906250,
+-0.0011596679687500, 0.0002136230468750, -0.0005493164062500, 0.0000610351562500,
+-0.0009155273437500, 0.0003051757812500, -0.0000915527343750, -0.0010375976562500,
+0.0001525878906250, -0.0013427734375000, -0.0007629394531250, -0.0008544921875000,
+-0.0008544921875000, -0.0010681152343750, -0.0001525878906250, -0.0014343261718750,
+-0.0024108886718750, -0.0005798339843750, -0.0020446777343750, -0.0003051757812500,
+-0.0026855468750000, -0.0005798339843750, -0.0045166015625000, -0.0004272460937500,
+-0.0035400390625000, -0.0003051757812500, -0.0045776367187500, -0.0012512207031250,
+-0.0050048828125000, -0.0014038085937500, -0.0044555664062500, -0.0015258789062500,
+-0.0044860839843750, -0.0022888183593750, -0.0047607421875000, -0.0022583007812500,
+-0.0034484863281250, -0.0013122558593750, -0.0039062500000000, -0.0017700195312500,
+-0.0030822753906250, -0.0009460449218750, -0.0018615722656250, 0.0003967285156250,
+-0.0028686523437500, -0.0003662109375000, -0.0002746582031250, 0.0006408691406250,
+-0.0006408691406250, 0.0007629394531250, -0.0002441406250000, 0.0001220703125000,
+0.0015563964843750, -0.0001220703125000, 0.0005798339843750, -0.0004882812500000,
+0.0017089843750000, -0.0012817382812500, 0.0019836425781250, -0.0019836425781250,
+0.0015563964843750, -0.0023803710937500, 0.0016784667968750, -0.0027770996093750,
+0.0018310546875000, -0.0032043457031250, 0.0012207031250000, -0.0032043457031250,
+0.0013732910156250, -0.0034484863281250, 0.0011596679687500, -0.0034179687500000,
+0.0007629394531250, -0.0031433105468750, 0.0011291503906250, -0.0034179687500000,
+0.0000000000000000, -0.0035400390625000, 0.0004272460937500, -0.0030212402343750,
+-0.0000305175781250, -0.0036010742187500, -0.0012207031250000, -0.0039672851562500,
+-0.0004577636718750, -0.0029296875000000, -0.0007629394531250, -0.0050354003906250,
+-0.0012512207031250, -0.0039672851562500, -0.0004882812500000, -0.0043334960937500,
+0.0000610351562500, -0.0059509277343750, -0.0003356933593750, -0.0044555664062500,
+0.0010681152343750, -0.0065917968750000, 0.0008544921875000, -0.0059204101562500,
+0.0012512207031250, -0.0061035156250000, 0.0023193359375000, -0.0074462890625000,
+0.0017700195312500, -0.0061340332031250, 0.0030517578125000, -0.0078430175781250,
+0.0030517578125000, -0.0076599121093750, 0.0031738281250000, -0.0070800781250000,
+0.0039062500000000, -0.0077514648437500, 0.0038146972656250, -0.0075378417968750,
+0.0038146972656250, -0.0071716308593750, 0.0040588378906250, -0.0072326660156250,
+0.0040283203125000, -0.0069885253906250, 0.0039062500000000, -0.0063781738281250,
+0.0039978027343750, -0.0061645507812500, 0.0037231445312500, -0.0060119628906250,
+0.0039367675781250, -0.0053710937500000, 0.0035705566406250, -0.0054931640625000,
+0.0031433105468750, -0.0056457519531250, 0.0034790039062500, -0.0048522949218750,
+0.0028381347656250, -0.0057983398437500, 0.0028076171875000, -0.0051879882812500,
+0.0029907226562500, -0.0050354003906250, 0.0027770996093750, -0.0054931640625000,
+0.0028991699218750, -0.0047302246093750, 0.0032348632812500, -0.0052490234375000,
+0.0031127929687500, -0.0046081542968750, 0.0034179687500000, -0.0047912597656250,
+0.0038757324218750, -0.0052185058593750, 0.0036315917968750, -0.0041809082031250,
+0.0042724609375000, -0.0056762695312500, 0.0043945312500000, -0.0050964355468750,
+0.0041198730468750, -0.0051269531250000, 0.0043334960937500, -0.0061645507812500,
+0.0044250488281250, -0.0052490234375000, 0.0033874511718750, -0.0057373046875000,
+0.0036621093750000, -0.0057678222656250, 0.0031738281250000, -0.0054016113281250,
+0.0022277832031250, -0.0054931640625000, 0.0026855468750000, -0.0054626464843750,
+0.0021667480468750, -0.0049743652343750, 0.0020141601562500, -0.0050964355468750,
+0.0020141601562500, -0.0048217773437500, 0.0019531250000000, -0.0043640136718750,
+0.0020446777343750, -0.0046386718750000, 0.0014343261718750, -0.0038146972656250,
+0.0016174316406250, -0.0036315917968750, 0.0014648437500000, -0.0036315917968750,
+0.0008850097656250, -0.0032348632812500, 0.0011291503906250, -0.0032958984375000,
+0.0008239746093750, -0.0039062500000000, 0.0007324218750000, -0.0038146972656250,
+0.0004882812500000, -0.0042114257812500, 0.0001525878906250, -0.0050659179687500,
+0.0003051757812500, -0.0049438476562500, -0.0004577636718750, -0.0051879882812500,
+-0.0006713867187500, -0.0056762695312500, -0.0004577636718750, -0.0050354003906250,
+-0.0006713867187500, -0.0047302246093750, -0.0009155273437500, -0.0054931640625000,
+-0.0003967285156250, -0.0029296875000000, -0.0004577636718750, -0.0036621093750000,
+-0.0004882812500000, -0.0030822753906250, -0.0002441406250000, -0.0009155273437500,
+-0.0003967285156250, -0.0018615722656250, -0.0003662109375000, -0.0002441406250000,
+-0.0001831054687500, -0.0000610351562500, -0.0002136230468750, -0.0004272460937500,
+-0.0003356933593750, 0.0002746582031250, -0.0003356933593750, 0.0002746582031250,
+0.0000000000000000, -0.0011596679687500, 0.0000000000000000, -0.0004272460937500,
+-0.0003051757812500, -0.0014038085937500, -0.0002441406250000, -0.0032653808593750,
+-0.0000610351562500, -0.0022888183593750, -0.0015258789062500, -0.0041503906250000,
+-0.0014953613281250, -0.0043334960937500, -0.0014648437500000, -0.0038757324218750,
+-0.0022583007812500, -0.0044250488281250, -0.0021972656250000, -0.0040283203125000,
+-0.0016784667968750, -0.0026245117187500, -0.0022583007812500, -0.0028076171875000,
+-0.0013427734375000, -0.0018310546875000, -0.0000305175781250, -0.0003051757812500,
+-0.0009765625000000, -0.0010375976562500, 0.0002136230468750, -0.0000915527343750,
+0.0003051757812500, 0.0003356933593750, 0.0009155273437500, -0.0002441406250000,
+0.0015258789062500, -0.0004272460937500, 0.0007629394531250, -0.0001525878906250,
+0.0039367675781250, -0.0007934570312500, 0.0036010742187500, -0.0009765625000000,
+0.0037231445312500, -0.0007629394531250, 0.0059204101562500, -0.0007324218750000,
+0.0055236816406250, -0.0008850097656250, 0.0054931640625000, -0.0013732910156250,
+0.0063476562500000, -0.0003967285156250, 0.0047607421875000, -0.0017089843750000,
+0.0036621093750000, -0.0031738281250000, 0.0053710937500000, -0.0018310546875000,
+0.0010986328125000, -0.0057067871093750, 0.0011291503906250, -0.0059814453125000,
+0.0011901855468750, -0.0045471191406250, -0.0007324218750000, -0.0059509277343750,
+0.0000610351562500, -0.0063781738281250, -0.0009765625000000, -0.0012512207031250,
+-0.0008544921875000, -0.0023193359375000, -0.0008239746093750, -0.0013122558593750,
+-0.0018615722656250, 0.0051574707031250, -0.0022583007812500, -0.0054016113281250,
+-0.0019531250000000},
+{0.0028686523437500, 0.0016479492187500, 0.0021972656250000, 0.0010375976562500,
+0.0020446777343750, 0.0015563964843750, 0.0020141601562500, 0.0012207031250000,
+0.0017395019531250, 0.0007324218750000, 0.0016479492187500, 0.0015869140625000,
+0.0016784667968750, -0.0004882812500000, 0.0015869140625000, 0.0002136230468750,
+0.0015258789062500, -0.0003051757812500, 0.0016479492187500, -0.0019836425781250,
+0.0015563964843750, -0.0009765625000000, 0.0011596679687500, -0.0028686523437500,
+0.0012512207031250, -0.0025634765625000, 0.0009460449218750, -0.0030212402343750,
+0.0004272460937500, -0.0043640136718750, 0.0005798339843750, -0.0036621093750000,
+0.0003356933593750, -0.0054626464843750, 0.0003356933593750, -0.0055236816406250,
+0.0002746582031250, -0.0052795410156250, 0.0001525878906250, -0.0059509277343750,
+0.0003356933593750, -0.0055541992187500, -0.0002441406250000, -0.0046386718750000,
+-0.0001831054687500, -0.0048217773437500, -0.0001220703125000, -0.0040588378906250,
+-0.0004577636718750, -0.0029602050781250, -0.0003662109375000, -0.0034179687500000,
+0.0000305175781250, -0.0027160644531250, -0.0001525878906250, -0.0022277832031250,
+0.0000610351562500, -0.0028991699218750, 0.0004882812500000, -0.0029907226562500,
+0.0003051757812500, -0.0022583007812500, 0.0007629394531250, -0.0041198730468750,
+0.0006713867187500, -0.0036926269531250, 0.0006408691406250, -0.0036621093750000,
+0.0010375976562500, -0.0048522949218750, 0.0008850097656250, -0.0044250488281250,
+0.0003967285156250, -0.0047912597656250, 0.0006713867187500, -0.0051269531250000,
+0.0003051757812500, -0.0042724609375000, -0.0002746582031250, -0.0040588378906250,
+-0.0000610351562500, -0.0045471191406250, -0.0008239746093750, -0.0012817382812500,
+-0.0007934570312500, -0.0022277832031250, -0.0007934570312500, -0.0012817382812500,
+-0.0012817382812500, 0.0018005371093750, -0.0012207031250000, 0.0005187988281250,
+-0.0008239746093750, 0.0010070800781250, -0.0010375976562500, 0.0021667480468750,
+-0.0007934570312500, 0.0002136230468750, -0.0005493164062500, -0.0012207031250000,
+-0.0006713867187500, 0.0002136230468750, 0.0007019042968750, -0.0041198730468750,
+0.0002746582031250, -0.0038452148437500, 0.0007324218750000, -0.0036926269531250,
+0.0019226074218750, -0.0059814453125000, 0.0014648437500000, -0.0049438476562500,
+0.0026550292968750, -0.0035400390625000, 0.0025634765625000, -0.0046081542968750,
+0.0025634765625000, -0.0030212402343750, 0.0036926269531250, -0.0004882812500000,
+0.0034179687500000, -0.0020751953125000, 0.0018615722656250, 0.0001831054687500,
+0.0029296875000000, 0.0008850097656250, 0.0019226074218750, -0.0001220703125000,
+-0.0001525878906250, -0.0000915527343750, 0.0008850097656250, 0.0003662109375000,
+-0.0014343261718750, -0.0013427734375000, -0.0009155273437500, -0.0013427734375000,
+-0.0013732910156250, -0.0011596679687500, -0.0033569335937500, -0.0018615722656250,
+-0.0023803710937500, -0.0016174316406250, -0.0030212402343750, -0.0014648437500000,
+-0.0032653808593750, -0.0009765625000000, -0.0031127929687500, -0.0019836425781250,
+-0.0032958984375000, -0.0026855468750000, -0.0032043457031250, -0.0017700195312500,
+-0.0026855468750000, -0.0044860839843750, -0.0029296875000000, -0.0044555664062500,
+-0.0027160644531250, -0.0039672851562500, -0.0018920898437500, -0.0054626464843750,
+-0.0025939941406250, -0.0011901855468750, -0.0036010742187500, -0.0051269531250000,
+-0.0033264160156250, -0.0037536621093750, -0.0031127929687500, -0.0039062500000000,
+-0.0036315917968750, -0.0061035156250000, -0.0037841796875000, -0.0059814453125000,
+-0.0021057128906250, -0.0031433105468750, -0.0023498535156250, -0.0038452148437500,
+-0.0025939941406250, -0.0032348632812500, -0.0018005371093750, -0.0008239746093750,
+-0.0020446777343750, -0.0015563964843750, -0.0032348632812500, -0.0013122558593750,
+-0.0029602050781250, -0.0011901855468750, -0.0031433105468750, -0.0004577636718750,
+-0.0046691894531250, -0.0002441406250000, -0.0044250488281250, -0.0009765625000000,
+-0.0019531250000000, 0.0018005371093750, -0.0032043457031250, 0.0017395019531250,
+-0.0017700195312500, 0.0015869140625000, 0.0012817382812500, 0.0029907226562500,
+-0.0004882812500000, 0.0025329589843750, 0.0005493164062500, 0.0013122558593750,
+0.0019531250000000, 0.0020446777343750, 0.0000915527343750, 0.0013122558593750,
+-0.0016174316406250, -0.0006408691406250, -0.0000610351562500, 0.0001220703125000,
+-0.0020751953125000, 0.0007629394531250, -0.0029296875000000, -0.0001831054687500,
+-0.0011901855468750, 0.0007019042968750, -0.0007324218750000, 0.0024108886718750,
+-0.0018920898437500, 0.0016174316406250, 0.0013122558593750, 0.0018615722656250,
+0.0015258789062500, 0.0024719238281250, 0.0007934570312500, 0.0017089843750000,
+0.0021667480468750, 0.0007629394531250, 0.0022888183593750, 0.0012512207031250,
+-0.0004272460937500, 0.0009460449218750, 0.0004577636718750, 0.0007019042968750,
+-0.0004272460937500, 0.0008544921875000, -0.0028076171875000, 0.0010681152343750,
+-0.0015258789062500, 0.0009765625000000, -0.0036926269531250, 0.0004577636718750,
+-0.0036926269531250, 0.0006103515625000, -0.0035705566406250, 0.0003662109375000,
+-0.0049133300781250, -0.0002746582031250, -0.0045166015625000, -0.0000915527343750,
+-0.0034179687500000, 0.0002746582031250, -0.0041198730468750, 0.0001525878906250,
+-0.0035705566406250, -0.0000305175781250, -0.0019226074218750, 0.0003051757812500,
+-0.0025634765625000, 0.0004272460937500, -0.0023193359375000, -0.0009155273437500,
+-0.0015563964843750, -0.0007324218750000, -0.0023803710937500, -0.0006103515625000,
+-0.0031127929687500, -0.0015258789062500, -0.0021667480468750, -0.0014343261718750,
+-0.0042724609375000, -0.0004882812500000, -0.0039367675781250, -0.0007629394531250,
+-0.0042724609375000, -0.0005187988281250, -0.0057678222656250, 0.0003051757812500,
+-0.0048828125000000, 0.0000610351562500, -0.0053710937500000, 0.0000610351562500,
+-0.0060119628906250, 0.0001525878906250, -0.0047912597656250, 0.0000915527343750,
+-0.0041809082031250, -0.0000915527343750, -0.0052185058593750, -0.0000610351562500,
+-0.0018005371093750, -0.0000305175781250, -0.0028076171875000, -0.0000610351562500,
+-0.0016174316406250, -0.0000915527343750, 0.0011291503906250, -0.0001220703125000,
+-0.0008544921875000, -0.0000915527343750, 0.0026245117187500, 0.0001525878906250,
+0.0022583007812500, -0.0001525878906250, 0.0023803710937500, 0.0004577636718750,
+0.0044860839843750, 0.0010681152343750, 0.0033874511718750, 0.0006408691406250,
+0.0043945312500000, 0.0017700195312500, 0.0046386718750000, 0.0018615722656250,
+0.0040893554687500, 0.0018310546875000, 0.0043334960937500, 0.0023193359375000,
+0.0043945312500000, 0.0022277832031250, 0.0028381347656250, 0.0023803710937500,
+0.0033874511718750, 0.0024108886718750, 0.0027465820312500, 0.0023803710937500,
+0.0013122558593750, 0.0024719238281250, 0.0021362304687500, 0.0025329589843750,
+0.0004272460937500, 0.0025024414062500, 0.0003967285156250, 0.0024108886718750,
+0.0001831054687500, 0.0027160644531250, -0.0008239746093750, 0.0029296875000000,
+-0.0004882812500000, 0.0027160644531250, -0.0013122558593750, 0.0035705566406250,
+-0.0014343261718750, 0.0035705566406250, -0.0015869140625000, 0.0036926269531250,
+-0.0021667480468750, 0.0041809082031250, -0.0021972656250000, 0.0040893554687500,
+-0.0026855468750000, 0.0040893554687500, -0.0030517578125000, 0.0042724609375000,
+-0.0031738281250000, 0.0040283203125000, -0.0035095214843750, 0.0037231445312500,
+-0.0038757324218750, 0.0039062500000000, -0.0034179687500000, 0.0036010742187500,
+-0.0038757324218750, 0.0033874511718750, -0.0037231445312500, 0.0035400390625000,
+-0.0032653808593750, 0.0036315917968750, -0.0036926269531250, 0.0033569335937500,
+-0.0026245117187500, 0.0036926269531250, -0.0030517578125000, 0.0038452148437500,
+-0.0026245117187500, 0.0035095214843750, -0.0016174316406250, 0.0036315917968750,
+-0.0024414062500000, 0.0037536621093750, -0.0014343261718750, 0.0023498535156250,
+-0.0015258789062500, 0.0029296875000000, -0.0011596679687500, 0.0021667480468750,
+-0.0004882812500000, 0.0006713867187500, -0.0009765625000000, 0.0016174316406250,
+0.0001220703125000, 0.0000610351562500, -0.0001525878906250, -0.0000305175781250,
+-0.0003051757812500, 0.0002136230468750, 0.0001525878906250, -0.0004577636718750,
+-0.0003356933593750, -0.0003356933593750, -0.0005493164062500, 0.0001831054687500,
+-0.0005798339843750, 0.0000000000000000, -0.0009765625000000, 0.0001220703125000,
+-0.0013427734375000, 0.0006408691406250, -0.0013427734375000, 0.0005493164062500,
+-0.0019836425781250, 0.0006713867187500, -0.0019836425781250, 0.0007019042968750,
+-0.0020751953125000, 0.0006408691406250, -0.0023193359375000, 0.0007629394531250,
+-0.0021972656250000, 0.0008239746093750, -0.0023498535156250, 0.0005798339843750,
+-0.0021362304687500, 0.0008239746093750, -0.0021057128906250, 0.0002746582031250,
+-0.0021057128906250, -0.0001831054687500, -0.0018310546875000, 0.0003662109375000,
+-0.0023803710937500, -0.0010375976562500, -0.0020141601562500, -0.0009155273437500,
+-0.0024108886718750, -0.0011901855468750, -0.0033264160156250, -0.0019531250000000,
+-0.0029296875000000, -0.0014648437500000, -0.0039062500000000, -0.0027465820312500,
+-0.0041503906250000, -0.0027770996093750, -0.0041198730468750, -0.0023803710937500,
+-0.0044250488281250, -0.0029602050781250, -0.0043945312500000, -0.0029907226562500,
+-0.0039978027343750, -0.0011291503906250, -0.0041503906250000, -0.0016479492187500,
+-0.0037536621093750, -0.0013427734375000, -0.0031433105468750, 0.0003662109375000,
+-0.0033264160156250, -0.0002136230468750, -0.0028076171875000, -0.0010375976562500,
+-0.0025024414062500, -0.0001525878906250, -0.0027770996093750, -0.0009155273437500,
+-0.0028381347656250, -0.0023803710937500, -0.0025634765625000, -0.0017089843750000,
+-0.0035400390625000, -0.0029907226562500, -0.0036010742187500, -0.0029296875000000,
+-0.0030517578125000, -0.0027465820312500, -0.0033874511718750, -0.0036010742187500,
+-0.0035400390625000, -0.0035400390625000, -0.0010681152343750, -0.0021972656250000,
+-0.0016479492187500, -0.0025329589843750, -0.0010375976562500, -0.0021972656250000,
+0.0010375976562500, -0.0009765625000000, 0.0001831054687500, -0.0013122558593750,
+0.0005798339843750, -0.0013427734375000, 0.0010070800781250, -0.0010070800781250,
+0.0004882812500000, -0.0011291503906250, 0.0001220703125000, -0.0014038085937500,
+0.0004577636718750, -0.0011901855468750, -0.0004272460937500, -0.0014038085937500,
+-0.0005187988281250, -0.0014343261718750, -0.0003662109375000, -0.0016174316406250,
+-0.0004882812500000, -0.0019836425781250, -0.0005187988281250, -0.0016479492187500,
+-0.0008850097656250, -0.0006408691406250, -0.0005798339843750, -0.0014343261718750,
+-0.0012207031250000, -0.0008850097656250, -0.0023193359375000, 0.0008850097656250,
+-0.0022888183593750, 0.0007934570312500, -0.0036926269531250, 0.0012512207031250,
+-0.0036926269531250, 0.0013427734375000, -0.0028991699218750, 0.0007324218750000,
+-0.0032653808593750, 0.0006408691406250, -0.0036315917968750, 0.0010986328125000,
+-0.0015869140625000, -0.0003356933593750, -0.0019226074218750, -0.0004882812500000,
+-0.0013732910156250, -0.0004272460937500, 0.0001525878906250, -0.0010986328125000,
+-0.0007019042968750, -0.0009765625000000, -0.0001220703125000, -0.0003356933593750,
+0.0002746582031250, -0.0009155273437500, 0.0000915527343750, -0.0005493164062500,
+0.0001525878906250, 0.0002136230468750, 0.0001220703125000, -0.0005798339843750,
+-0.0003356933593750, 0.0002746582031250, 0.0000305175781250, 0.0003662109375000,
+-0.0008544921875000},
+{-0.0059509277343750, 0.0001831054687500, -0.0050354003906250, 0.0004882812500000,
+-0.0037841796875000, 0.0005493164062500, -0.0044555664062500, 0.0003662109375000,
+-0.0025329589843750, 0.0005798339843750, -0.0026855468750000, 0.0008239746093750,
+-0.0023193359375000, 0.0005493164062500, -0.0010681152343750, 0.0004577636718750,
+-0.0015869140625000, 0.0007019042968750, -0.0004882812500000, -0.0003051757812500,
+-0.0006408691406250, 0.0002136230468750, -0.0006103515625000, -0.0004882812500000,
+-0.0000610351562500, -0.0016784667968750, -0.0004882812500000, -0.0008544921875000,
+0.0000305175781250, -0.0022277832031250, 0.0000305175781250, -0.0023498535156250,
+-0.0003051757812500, -0.0022277832031250, -0.0001525878906250, -0.0027770996093750,
+-0.0000305175781250, -0.0026245117187500, -0.0007934570312500, -0.0027160644531250,
+-0.0005187988281250, -0.0026245117187500, -0.0007324218750000, -0.0027465820312500,
+-0.0012512207031250, -0.0031433105468750, -0.0005798339843750, -0.0028381347656250,
+-0.0012817382812500, -0.0023193359375000, -0.0013122558593750, -0.0028381347656250,
+-0.0011596679687500, -0.0020446777343750, -0.0013732910156250, -0.0007629394531250,
+-0.0013427734375000, -0.0015258789062500, -0.0016174316406250, -0.0008850097656250,
+-0.0014648437500000, -0.0001831054687500, -0.0019226074218750, -0.0013122558593750,
+-0.0023803710937500, -0.0021057128906250, -0.0019226074218750, -0.0011901855468750,
+-0.0030212402343750, -0.0028381347656250, -0.0028686523437500, -0.0032348632812500,
+-0.0028686523437500, -0.0027770996093750, -0.0036010742187500, -0.0028381347656250,
+-0.0033264160156250, -0.0029602050781250, -0.0033264160156250, -0.0028686523437500,
+-0.0034790039062500, -0.0029296875000000, -0.0035095214843750, -0.0025634765625000,
+-0.0034179687500000, -0.0021362304687500, -0.0032653808593750, -0.0024414062500000,
+-0.0027160644531250, -0.0019836425781250, -0.0031433105468750, -0.0018005371093750,
+-0.0022888183593750, -0.0017700195312500, -0.0011596679687500, -0.0017395019531250,
+-0.0017700195312500, -0.0016174316406250, -0.0006713867187500, -0.0009155273437500,
+-0.0005798339843750, -0.0011291503906250, -0.0005493164062500, -0.0008544921875000,
+-0.0000610351562500, -0.0000610351562500, -0.0003662109375000, -0.0003967285156250,
+0.0000000000000000, -0.0005187988281250, 0.0002136230468750, -0.0002136230468750,
+-0.0001831054687500, 0.0000000000000000, -0.0004577636718750, -0.0001831054687500,
+-0.0002441406250000, -0.0004272460937500, -0.0005493164062500, -0.0000915527343750,
+-0.0006103515625000, 0.0005493164062500, -0.0005798339843750, 0.0003051757812500,
+-0.0003356933593750, -0.0002746582031250, -0.0003051757812500, -0.0001220703125000,
+-0.0023193359375000, -0.0007324218750000, -0.0010375976562500, -0.0003967285156250,
+-0.0018005371093750, -0.0004272460937500, -0.0041503906250000, -0.0010681152343750,
+-0.0026855468750000, -0.0008850097656250, -0.0038452148437500, -0.0007934570312500,
+-0.0046386718750000, -0.0009765625000000, -0.0032348632812500, -0.0008850097656250,
+-0.0025329589843750, -0.0007629394531250, -0.0033569335937500, -0.0012207031250000,
+-0.0007934570312500, -0.0006103515625000, -0.0007019042968750, -0.0009155273437500,
+-0.0010070800781250, -0.0010986328125000, -0.0000915527343750, -0.0006713867187500,
+-0.0002441406250000, -0.0010986328125000, 0.0000915527343750, -0.0018005371093750,
+-0.0001525878906250, -0.0014038085937500, 0.0001525878906250, -0.0023193359375000,
+0.0009460449218750, -0.0035705566406250, 0.0004882812500000, -0.0029602050781250,
+-0.0002136230468750, -0.0041503906250000, 0.0008544921875000, -0.0050354003906250,
+-0.0004272460937500, -0.0038757324218750, -0.0024108886718750, -0.0036621093750000,
+-0.0008850097656250, -0.0051574707031250, -0.0021362304687500, -0.0015258789062500,
+-0.0028991699218750, -0.0022277832031250, -0.0018005371093750, -0.0024719238281250,
+-0.0010681152343750, 0.0000000000000000, -0.0015563964843750, -0.0004882812500000,
+-0.0002746582031250, -0.0017700195312500, -0.0000915527343750, -0.0005187988281250,
+-0.0002136230468750, -0.0009765625000000, 0.0005187988281250, -0.0024719238281250,
+0.0006713867187500, -0.0011596679687500, -0.0000610351562500, -0.0023498535156250,
+0.0002136230468750, -0.0025939941406250, -0.0000305175781250, -0.0019226074218750,
+-0.0004882812500000, -0.0018310546875000, -0.0000305175781250, -0.0018920898437500,
+-0.0010681152343750, -0.0012817382812500, -0.0007934570312500, -0.0006408691406250,
+-0.0012512207031250, -0.0011291503906250, -0.0020751953125000, -0.0011596679687500,
+-0.0013427734375000, -0.0000915527343750, -0.0028381347656250, -0.0024719238281250,
+-0.0026245117187500, -0.0012207031250000, -0.0028076171875000, -0.0015258789062500,
+-0.0037536621093750, -0.0033569335937500, -0.0031127929687500, -0.0014343261718750,
+-0.0037841796875000, -0.0026855468750000, -0.0037841796875000, -0.0020446777343750,
+-0.0034790039062500, -0.0015563964843750, -0.0036010742187500, -0.0019226074218750,
+-0.0033874511718750, -0.0007934570312500, -0.0029296875000000, -0.0014343261718750,
+-0.0031738281250000, -0.0008850097656250, -0.0025939941406250, -0.0009765625000000,
+-0.0019836425781250, -0.0017089843750000, -0.0024414062500000, -0.0012207031250000,
+-0.0011596679687500, -0.0024414062500000, -0.0014648437500000, -0.0023803710937500,
+-0.0011596679687500, -0.0028381347656250, -0.0003051757812500, -0.0038757324218750,
+-0.0008850097656250, -0.0036010742187500, 0.0006713867187500, -0.0045166015625000,
+0.0002441406250000, -0.0049133300781250, 0.0005187988281250, -0.0047912597656250,
+0.0019226074218750, -0.0051879882812500, 0.0011901855468750, -0.0056457519531250,
+0.0012817382812500, -0.0047912597656250, 0.0017700195312500, -0.0055847167968750,
+0.0011291503906250, -0.0051574707031250, 0.0003051757812500, -0.0043029785156250,
+0.0009155273437500, -0.0056457519531250, 0.0000610351562500, -0.0039367675781250,
+-0.0002441406250000, -0.0042114257812500, 0.0003356933593750, -0.0040893554687500,
+0.0003967285156250, -0.0029907226562500, 0.0000000000000000, -0.0033569335937500,
+0.0015869140625000, -0.0023193359375000, 0.0013122558593750, -0.0025634765625000,
+0.0016479492187500, -0.0023193359375000, 0.0027770996093750, -0.0015869140625000,
+0.0021972656250000, -0.0020751953125000, 0.0028686523437500, -0.0012817382812500,
+0.0030822753906250, -0.0013427734375000, 0.0028381347656250, -0.0013732910156250,
+0.0030212402343750, -0.0011901855468750, 0.0031127929687500, -0.0015869140625000,
+0.0025939941406250, -0.0011596679687500, 0.0026855468750000, -0.0018005371093750,
+0.0024108886718750, -0.0016784667968750, 0.0019531250000000, -0.0014343261718750,
+0.0021057128906250, -0.0025939941406250, 0.0012512207031250, -0.0014038085937500,
+0.0013122558593750, -0.0022583007812500, 0.0010375976562500, -0.0023803710937500,
+0.0004577636718750, -0.0016784667968750, 0.0006103515625000, -0.0031127929687500,
+-0.0001831054687500, -0.0020141601562500, 0.0000305175781250, -0.0026855468750000,
+-0.0005187988281250, -0.0033264160156250, -0.0013427734375000, -0.0027160644531250,
+-0.0007629394531250, -0.0033874511718750, -0.0020751953125000, -0.0038146972656250,
+-0.0021057128906250, -0.0034179687500000, -0.0018310546875000, -0.0039978027343750,
+-0.0022277832031250, -0.0044555664062500, -0.0021667480468750, -0.0035705566406250,
+-0.0016174316406250, -0.0048522949218750, -0.0016479492187500, -0.0043334960937500,
+-0.0014648437500000, -0.0041198730468750, -0.0011596679687500, -0.0048522949218750,
+-0.0012512207031250, -0.0040283203125000, -0.0006103515625000, -0.0034484863281250,
+-0.0008544921875000, -0.0038146972656250, -0.0004577636718750, -0.0023498535156250,
+0.0003051757812500, -0.0010986328125000, -0.0002136230468750, -0.0020446777343750,
+0.0004272460937500, 0.0006103515625000, 0.0005187988281250, 0.0003967285156250,
+0.0002136230468750, 0.0003662109375000, 0.0002441406250000, 0.0020141601562500,
+0.0001831054687500, 0.0015869140625000, -0.0003356933593750, 0.0007019042968750,
+-0.0001831054687500, 0.0017089843750000, -0.0008239746093750, 0.0002746582031250,
+-0.0015258789062500, -0.0015869140625000, -0.0010375976562500, -0.0002136230468750,
+-0.0025939941406250, -0.0037231445312500, -0.0026245117187500, -0.0037536621093750,
+-0.0023193359375000, -0.0042724609375000, -0.0029602050781250, -0.0065002441406250,
+-0.0029296875000000, -0.0057983398437500, -0.0018310546875000, -0.0067138671875000,
+-0.0020446777343750, -0.0071716308593750, -0.0017089843750000, -0.0062255859375000,
+-0.0008239746093750, -0.0057067871093750, -0.0011596679687500, -0.0060119628906250,
+-0.0003662109375000, -0.0042114257812500, -0.0007019042968750, -0.0033874511718750,
+-0.0000915527343750, -0.0034179687500000, 0.0007934570312500, -0.0028686523437500,
+0.0000305175781250, -0.0025024414062500, 0.0021972656250000, -0.0032653808593750,
+0.0022888183593750, -0.0035095214843750, 0.0016479492187500, -0.0027465820312500,
+0.0025939941406250, -0.0026550292968750, 0.0027770996093750, -0.0029296875000000,
+0.0005798339843750, -0.0003356933593750, 0.0011596679687500, -0.0008544921875000,
+0.0002441406250000, -0.0004272460937500, -0.0017700195312500, 0.0016174316406250,
+-0.0008239746093750, 0.0006713867187500, -0.0023193359375000, 0.0002746582031250,
+-0.0028076171875000, 0.0017089843750000, -0.0021057128906250, 0.0000610351562500,
+-0.0022583007812500, -0.0017395019531250, -0.0027465820312500, -0.0000305175781250,
+-0.0019226074218750, -0.0047302246093750, -0.0012817382812500, -0.0046386718750000,
+-0.0027770996093750, -0.0043945312500000, -0.0039978027343750, -0.0070800781250000,
+-0.0028381347656250, -0.0064086914062500, -0.0045166015625000, -0.0047607421875000,
+-0.0050659179687500, -0.0059509277343750, -0.0044860839843750, -0.0039367675781250,
+-0.0045776367187500, -0.0010986328125000, -0.0047302246093750, -0.0030517578125000,
+-0.0033569335937500, 0.0001220703125000, -0.0033874511718750, 0.0010986328125000,
+-0.0036926269531250, -0.0005798339843750, -0.0031433105468750, -0.0004577636718750,
+-0.0030212402343750, 0.0004272460937500, -0.0039062500000000, -0.0030212402343750,
+-0.0040588378906250, -0.0029907226562500, -0.0037536621093750, -0.0021667480468750,
+-0.0038757324218750, -0.0037231445312500, -0.0046691894531250, -0.0006408691406250,
+-0.0045471191406250, -0.0045471191406250, -0.0051574707031250, -0.0048828125000000,
+-0.0042419433593750, -0.0032653808593750, -0.0038757324218750, -0.0029296875000000,
+-0.0046691894531250, -0.0041198730468750, -0.0007629394531250, -0.0014953613281250,
+-0.0013732910156250, -0.0008850097656250, -0.0010070800781250, -0.0021972656250000,
+0.0019836425781250, -0.0021362304687500, 0.0012207031250000, -0.0015563964843750,
+0.0004272460937500, -0.0039672851562500, 0.0019836425781250, -0.0043640136718750,
+0.0002136230468750, -0.0036315917968750, -0.0022888183593750, -0.0042114257812500,
+-0.0003662109375000, -0.0046081542968750, -0.0029907226562500, -0.0022888183593750,
+-0.0038452148437500, -0.0027465820312500, -0.0022888183593750, -0.0021667480468750,
+-0.0021972656250000, -0.0002746582031250, -0.0029907226562500, -0.0010681152343750,
+0.0000000000000000, -0.0008850097656250, 0.0000610351562500, -0.0005798339843750,
+-0.0003356933593750, -0.0003356933593750, 0.0010375976562500, -0.0005187988281250,
+0.0010375976562500, -0.0010070800781250, -0.0008850097656250, 0.0011596679687500,
+-0.0000305175781250, 0.0009765625000000, -0.0006103515625000, 0.0009155273437500,
+-0.0025939941406250, 0.0021972656250000, -0.0014648437500000, 0.0020141601562500,
+-0.0017089843750000, 0.0011596679687500, -0.0022277832031250, 0.0013122558593750,
+-0.0015258789062500, 0.0013732910156250, -0.0004882812500000, 0.0008850097656250,
+-0.0008544921875000},
+{-0.0008544921875000, 0.0016174316406250, -0.0018310546875000, 0.0015258789062500,
+-0.0007629394531250, -0.0007934570312500, -0.0007019042968750, -0.0002441406250000,
+-0.0007324218750000, -0.0018615722656250, -0.0003662109375000, -0.0045166015625000,
+-0.0006408691406250, -0.0032958984375000, 0.0004882812500000, -0.0060729980468750,
+0.0003051757812500, -0.0064392089843750, 0.0003356933593750, -0.0058288574218750,
+0.0012817382812500, -0.0064392089843750, 0.0010375976562500, -0.0060119628906250,
+0.0001220703125000, -0.0043945312500000, 0.0004882812500000, -0.0046691894531250,
+0.0000915527343750, -0.0039062500000000, -0.0011901855468750, -0.0024108886718750,
+-0.0009155273437500, -0.0029602050781250, -0.0004272460937500, -0.0015869140625000,
+-0.0009460449218750, -0.0010986328125000, -0.0006103515625000, -0.0007934570312500,
+0.0001831054687500, -0.0002441406250000, -0.0003051757812500, -0.0003356933593750,
+-0.0000915527343750, -0.0000610351562500, 0.0003051757812500, 0.0000610351562500,
+-0.0004272460937500, -0.0000610351562500, -0.0008239746093750, 0.0000915527343750,
+-0.0003356933593750, 0.0001831054687500, -0.0031127929687500, -0.0013122558593750,
+-0.0029602050781250, -0.0004577636718750, -0.0024719238281250, -0.0015563964843750,
+-0.0038757324218750, -0.0034179687500000, -0.0040588378906250, -0.0021667480468750,
+-0.0027770996093750, -0.0045471191406250, -0.0028991699218750, -0.0045776367187500,
+-0.0029602050781250, -0.0045166015625000, -0.0022888183593750, -0.0058593750000000,
+-0.0025634765625000, -0.0053100585937500, -0.0031433105468750, -0.0041503906250000,
+-0.0028381347656250, -0.0052490234375000, -0.0032653808593750, -0.0036010742187500,
+-0.0042724609375000, -0.0010681152343750, -0.0039672851562500, -0.0026550292968750,
+-0.0043029785156250, -0.0004882812500000, -0.0045471191406250, 0.0005493164062500,
+-0.0046081542968750, -0.0011596679687500, -0.0047302246093750, -0.0028381347656250,
+-0.0048217773437500, -0.0037231445312500, -0.0012512207031250, -0.0022277832031250,
+-0.0027160644531250, -0.0012512207031250, -0.0014953613281250, -0.0025634765625000,
+0.0011291503906250, -0.0039367675781250, 0.0000610351562500, -0.0029602050781250,
+-0.0007324218750000, -0.0038452148437500, 0.0008850097656250, -0.0048522949218750,
+-0.0004272460937500, -0.0034179687500000, -0.0028686523437500, -0.0022888183593750,
+-0.0014648437500000, -0.0034790039062500, -0.0029602050781250, -0.0013427734375000,
+-0.0036621093750000, -0.0007934570312500, -0.0022888183593750, -0.0018920898437500,
+-0.0017700195312500, -0.0018615722656250, -0.0026245117187500, -0.0013427734375000,
+-0.0007019042968750, -0.0030212402343750, -0.0003051757812500, -0.0032958984375000,
+-0.0010375976562500, -0.0024108886718750, -0.0006408691406250, -0.0027465820312500,
+-0.0000610351562500, -0.0032653808593750, -0.0011901855468750, 0.0002746582031250,
+-0.0014343261718750, -0.0001831054687500, -0.0007324218750000, -0.0001525878906250,
+-0.0006713867187500, 0.0019531250000000, -0.0009460449218750, 0.0014953613281250,
+0.0001525878906250, 0.0013427734375000, 0.0004272460937500, 0.0012817382812500,
+-0.0001831054687500, 0.0015258789062500, 0.0000915527343750, 0.0016784667968750,
+0.0004882812500000, 0.0013732910156250, -0.0025024414062500, 0.0022277832031250,
+-0.0016784667968750, 0.0024108886718750, -0.0029602050781250, 0.0020751953125000,
+-0.0060729980468750, 0.0022277832031250, -0.0044860839843750, 0.0023193359375000,
+-0.0058288574218750, 0.0014648437500000, -0.0072631835937500, 0.0016479492187500,
+-0.0054931640625000, 0.0014343261718750, -0.0045166015625000, 0.0007019042968750,
+-0.0059204101562500, 0.0009765625000000, -0.0015869140625000, 0.0008239746093750,
+-0.0026855468750000, 0.0006408691406250, -0.0013732910156250, 0.0009460449218750,
+0.0022277832031250, 0.0011901855468750, -0.0001220703125000, 0.0010681152343750,
+0.0026550292968750, 0.0014343261718750, 0.0032653808593750, 0.0013122558593750,
+0.0021362304687500, 0.0015258789062500, 0.0022888183593750, 0.0019226074218750,
+0.0023498535156250, 0.0016479492187500, 0.0008239746093750, 0.0016784667968750,
+0.0005187988281250, 0.0020141601562500, 0.0005187988281250, 0.0013427734375000,
+-0.0002136230468750, 0.0007019042968750, -0.0004272460937500, 0.0011596679687500,
+0.0000305175781250, -0.0003051757812500, -0.0005187988281250, -0.0003967285156250,
+-0.0001525878906250, -0.0004577636718750, 0.0003662109375000, -0.0012207031250000,
+-0.0002441406250000, -0.0010070800781250, 0.0013732910156250, -0.0012512207031250,
+0.0010375976562500, -0.0013122558593750, 0.0012512207031250, -0.0013732910156250,
+0.0025024414062500, -0.0015563964843750, 0.0018920898437500, -0.0014343261718750,
+0.0019836425781250, -0.0016174316406250, 0.0023498535156250, -0.0016784667968750,
+0.0017089843750000, -0.0015563964843750, 0.0012817382812500, -0.0015258789062500,
+0.0017395019531250, -0.0015563964843750, -0.0000610351562500, -0.0014038085937500,
+0.0005798339843750, -0.0014343261718750, -0.0002136230468750, -0.0013732910156250,
+-0.0017700195312500, -0.0013122558593750, -0.0005187988281250, -0.0013427734375000,
+-0.0028991699218750, -0.0010375976562500, -0.0024414062500000, -0.0010681152343750,
+-0.0028076171875000, -0.0010375976562500, -0.0043334960937500, -0.0007934570312500,
+-0.0030822753906250, -0.0008544921875000, -0.0048522949218750, -0.0006713867187500,
+-0.0044860839843750, -0.0007019042968750, -0.0044860839843750, -0.0004577636718750,
+-0.0054626464843750, -0.0001831054687500, -0.0047302246093750, -0.0002746582031250,
+-0.0053100585937500, 0.0003967285156250, -0.0051269531250000, 0.0003356933593750,
+-0.0050354003906250, 0.0004882812500000, -0.0052795410156250, 0.0008850097656250,
+-0.0049133300781250, 0.0008239746093750, -0.0049743652343750, 0.0010986328125000,
+-0.0048828125000000, 0.0010375976562500, -0.0046691894531250, 0.0012512207031250,
+-0.0045776367187500, 0.0016174316406250, -0.0044555664062500, 0.0014038085937500,
+-0.0044860839843750, 0.0018005371093750, -0.0043029785156250, 0.0020141601562500,
+-0.0041809082031250, 0.0018920898437500, -0.0041503906250000, 0.0019836425781250,
+-0.0039978027343750, 0.0021667480468750, -0.0039062500000000, 0.0017700195312500,
+-0.0037231445312500, 0.0019226074218750, -0.0037536621093750, 0.0018615722656250,
+-0.0038146972656250, 0.0016784667968750, -0.0035095214843750, 0.0018310546875000,
+-0.0039672851562500, 0.0015869140625000, -0.0038452148437500, 0.0017089843750000,
+-0.0038146972656250, 0.0014953613281250, -0.0040588378906250, 0.0011901855468750,
+-0.0039367675781250, 0.0014648437500000, -0.0042419433593750, 0.0009460449218750,
+-0.0040588378906250, 0.0009155273437500, -0.0041198730468750, 0.0009460449218750,
+-0.0043334960937500, 0.0008239746093750, -0.0041503906250000, 0.0009155273437500,
+-0.0043029785156250, 0.0000610351562500, -0.0042419433593750, 0.0004882812500000,
+-0.0042419433593750, 0.0000000000000000, -0.0043029785156250, -0.0009155273437500,
+-0.0042419433593750, -0.0003051757812500, -0.0043334960937500, -0.0017700195312500,
+-0.0045166015625000, -0.0014648437500000, -0.0046386718750000, -0.0018615722656250,
+-0.0047912597656250, -0.0031127929687500, -0.0050964355468750, -0.0024719238281250,
+-0.0051574707031250, -0.0029602050781250, -0.0055236816406250, -0.0032653808593750,
+-0.0056762695312500, -0.0029602050781250, -0.0057983398437500, -0.0028381347656250,
+-0.0062866210937500, -0.0029296875000000, -0.0058898925781250, -0.0024719238281250,
+-0.0062866210937500, -0.0024719238281250, -0.0060729980468750, -0.0025024414062500,
+-0.0057983398437500, -0.0023498535156250, -0.0061645507812500, -0.0022888183593750,
+-0.0041198730468750, -0.0020446777343750, -0.0048522949218750, -0.0020751953125000,
+-0.0034179687500000, -0.0018920898437500, -0.0012207031250000, -0.0015869140625000,
+-0.0027160644531250, -0.0017700195312500, 0.0007019042968750, -0.0017700195312500,
+0.0004272460937500, -0.0014953613281250, 0.0002746582031250, -0.0019226074218750,
+0.0021667480468750, -0.0022888183593750, 0.0014038085937500, -0.0019836425781250,
+0.0011596679687500, -0.0027160644531250, 0.0017395019531250, -0.0027465820312500,
+0.0009765625000000, -0.0026855468750000, 0.0000915527343750, -0.0028076171875000,
+0.0007019042968750, -0.0027160644531250, -0.0005187988281250, -0.0034484863281250,
+-0.0007629394531250, -0.0034790039062500, -0.0005493164062500, -0.0027465820312500,
+-0.0007934570312500, -0.0028991699218750, -0.0006103515625000, -0.0034790039062500,
+-0.0003967285156250, -0.0011901855468750, 0.0002441406250000, -0.0014038085937500,
+-0.0004272460937500, -0.0010986328125000, -0.0010986328125000, 0.0003662109375000,
+-0.0003051757812500, -0.0001525878906250, -0.0026550292968750, 0.0007629394531250,
+-0.0027465820312500, 0.0009155273437500, -0.0029602050781250, 0.0009155273437500,
+-0.0044250488281250, 0.0014343261718750, -0.0038757324218750, 0.0013122558593750,
+-0.0030822753906250, 0.0007324218750000, -0.0037536621093750, 0.0012512207031250,
+-0.0030212402343750, 0.0004272460937500, -0.0014343261718750, -0.0007019042968750,
+-0.0020446777343750, 0.0002136230468750, -0.0021057128906250, -0.0009765625000000,
+-0.0015563964843750, -0.0012207031250000, -0.0020751953125000, -0.0007629394531250,
+-0.0029296875000000, -0.0011901855468750, -0.0026855468750000, -0.0015258789062500,
+-0.0025634765625000, 0.0003356933593750, -0.0029907226562500, 0.0000915527343750,
+-0.0022583007812500, -0.0003662109375000, -0.0012512207031250, 0.0002441406250000,
+-0.0014038085937500, 0.0001220703125000, 0.0001831054687500, 0.0001525878906250,
+-0.0002746582031250, 0.0000915527343750, -0.0003967285156250, -0.0000915527343750,
+0.0006103515625000, -0.0003356933593750, 0.0001831054687500, -0.0006713867187500,
+-0.0002136230468750, -0.0009765625000000, 0.0002441406250000, -0.0009765625000000,
+-0.0004882812500000, -0.0014038085937500, -0.0014343261718750, -0.0018615722656250,
+-0.0008544921875000, -0.0015563964843750, -0.0023193359375000, -0.0019836425781250,
+-0.0023803710937500, -0.0022583007812500, -0.0021057128906250, -0.0021362304687500,
+-0.0028686523437500, -0.0021972656250000, -0.0028076171875000, -0.0025329589843750,
+-0.0018615722656250, -0.0021057128906250, -0.0019226074218750, -0.0025939941406250,
+-0.0016784667968750, -0.0024719238281250, -0.0008850097656250, -0.0019226074218750,
+-0.0010070800781250, -0.0024719238281250, -0.0008544921875000, -0.0021362304687500,
+-0.0007629394531250, -0.0017395019531250, -0.0004577636718750, -0.0015563964843750,
+-0.0002136230468750, -0.0012207031250000, -0.0004882812500000, -0.0005798339843750,
+0.0001220703125000, -0.0005493164062500, 0.0003662109375000, -0.0001220703125000,
+-0.0000915527343750, 0.0003967285156250, 0.0000915527343750, 0.0007324218750000,
+0.0006103515625000, 0.0008544921875000, -0.0007934570312500, 0.0004882812500000,
+-0.0003662109375000, 0.0010681152343750, -0.0007629394531250, 0.0007324218750000,
+-0.0020751953125000, 0.0002746582031250, -0.0010986328125000, 0.0011901855468750,
+-0.0012512207031250, 0.0001525878906250, -0.0017700195312500, 0.0009155273437500,
+-0.0013732910156250, 0.0010070800781250, -0.0007324218750000, 0.0004882812500000,
+-0.0009460449218750, 0.0015258789062500, -0.0008239746093750, 0.0004577636718750,
+-0.0006103515625000, 0.0015258789062500, -0.0012207031250000, 0.0008239746093750,
+-0.0017700195312500, -0.0005493164062500, -0.0014038085937500, 0.0009460449218750,
+-0.0021362304687500, -0.0021667480468750, -0.0025634765625000, -0.0020446777343750,
+-0.0023193359375000, -0.0025634765625000, -0.0023803710937500, -0.0049133300781250,
+-0.0027465820312500, -0.0042114257812500, -0.0018920898437500, -0.0047607421875000,
+-0.0023193359375000},
+{-0.0005187988281250, -0.0019226074218750, -0.0001525878906250, -0.0027770996093750,
+-0.0006408691406250, -0.0019226074218750, -0.0010375976562500, 0.0002136230468750,
+-0.0008544921875000, -0.0006408691406250, -0.0011901855468750, 0.0004882812500000,
+-0.0011596679687500, 0.0009155273437500, -0.0009765625000000, 0.0007324218750000,
+-0.0006408691406250, 0.0010986328125000, -0.0002136230468750, 0.0010070800781250,
+-0.0011596679687500, 0.0007629394531250, -0.0007629394531250, 0.0010375976562500,
+-0.0014343261718750, 0.0004272460937500, -0.0026550292968750, -0.0001525878906250,
+-0.0020446777343750, 0.0003051757812500, -0.0039672851562500, -0.0015563964843750,
+-0.0040893554687500, -0.0015258789062500, -0.0036926269531250, -0.0011291503906250,
+-0.0043640136718750, -0.0020141601562500, -0.0043945312500000, -0.0019836425781250,
+-0.0029907226562500, -0.0007019042968750, -0.0031127929687500, -0.0013427734375000,
+-0.0031127929687500, -0.0003967285156250, -0.0022888183593750, 0.0014038085937500,
+-0.0025024414062500, 0.0003662109375000, -0.0028076171875000, 0.0004272460937500,
+-0.0027465820312500, 0.0012207031250000, -0.0027465820312500, 0.0007629394531250,
+-0.0031738281250000, 0.0000000000000000, -0.0030517578125000, 0.0003051757812500,
+-0.0011291503906250, -0.0005798339843750, -0.0023193359375000, -0.0004272460937500,
+-0.0014343261718750, -0.0006408691406250, 0.0007019042968750, -0.0014038085937500,
+-0.0008544921875000, -0.0010375976562500, 0.0010986328125000, -0.0015563964843750,
+0.0014038085937500, -0.0018615722656250, 0.0005798339843750, -0.0013427734375000,
+0.0008544921875000, -0.0010375976562500, 0.0009765625000000, -0.0017395019531250,
+0.0000610351562500, -0.0011901855468750, 0.0000610351562500, -0.0011596679687500,
+-0.0004882812500000, -0.0019226074218750, -0.0010681152343750, -0.0023803710937500,
+-0.0005798339843750, -0.0022583007812500, -0.0020751953125000, -0.0028076171875000,
+-0.0018310546875000, -0.0030212402343750, -0.0021362304687500, -0.0026245117187500,
+-0.0032958984375000, -0.0023803710937500, -0.0024719238281250, -0.0021667480468750,
+-0.0025939941406250, -0.0007324218750000, -0.0030517578125000, -0.0003967285156250,
+-0.0022583007812500, -0.0001831054687500, -0.0014953613281250, 0.0006103515625000,
+-0.0019836425781250, 0.0007019042968750, -0.0006713867187500, 0.0003356933593750,
+-0.0006103515625000, 0.0004272460937500, -0.0004882812500000, 0.0002441406250000,
+0.0002136230468750, -0.0000610351562500, 0.0000000000000000, 0.0003662109375000,
+0.0001220703125000, 0.0002136230468750, 0.0003662109375000, 0.0005187988281250,
+0.0001831054687500, 0.0006408691406250, 0.0000915527343750, 0.0009155273437500,
+0.0002441406250000, 0.0015258789062500, -0.0010681152343750, 0.0003051757812500,
+-0.0009155273437500, 0.0011291503906250, -0.0007629394531250, 0.0002746582031250,
+-0.0015563964843750, -0.0011596679687500, -0.0016479492187500, 0.0000915527343750,
+-0.0014343261718750, -0.0028991699218750, -0.0016479492187500, -0.0028991699218750,
+-0.0016174316406250, -0.0030822753906250, -0.0014343261718750, -0.0050964355468750,
+-0.0018005371093750, -0.0049438476562500, -0.0018615722656250, -0.0046997070312500,
+-0.0019531250000000, -0.0058593750000000, -0.0019226074218750, -0.0052185058593750,
+-0.0020141601562500, -0.0042114257812500, -0.0023803710937500, -0.0056457519531250,
+-0.0015258789062500, -0.0037536621093750, -0.0021057128906250, -0.0042724609375000,
+-0.0015258789062500, -0.0044250488281250, -0.0004577636718750, -0.0035095214843750,
+-0.0014953613281250, -0.0044250488281250, 0.0000305175781250, -0.0036010742187500,
+-0.0000915527343750, -0.0041198730468750, -0.0003356933593750, -0.0038146972656250,
+0.0002746582031250, -0.0030822753906250, -0.0001831054687500, -0.0040893554687500,
+-0.0000305175781250, -0.0028686523437500, -0.0002136230468750, -0.0032958984375000,
+-0.0003356933593750, -0.0030517578125000, -0.0002746582031250, -0.0021972656250000,
+-0.0004882812500000, -0.0030517578125000, -0.0007629394531250, -0.0019531250000000,
+-0.0007324218750000, -0.0022888183593750, -0.0009460449218750, -0.0019836425781250,
+-0.0012512207031250, -0.0012817382812500, -0.0012207031250000, -0.0021362304687500,
+-0.0010986328125000, -0.0011596679687500, -0.0014648437500000, -0.0012512207031250,
+-0.0010986328125000, -0.0015258789062500, -0.0006713867187500, -0.0009765625000000,
+-0.0011596679687500, -0.0008850097656250, -0.0003967285156250, -0.0017089843750000,
+-0.0002441406250000, -0.0014648437500000, -0.0006103515625000, -0.0018005371093750,
+-0.0004577636718750, -0.0025329589843750, -0.0002746582031250, -0.0021057128906250,
+-0.0010070800781250, -0.0027465820312500, -0.0008850097656250, -0.0033569335937500,
+-0.0010070800781250, -0.0031433105468750, -0.0015869140625000, -0.0031738281250000,
+-0.0012207031250000, -0.0038452148437500, -0.0014343261718750, -0.0032348632812500,
+-0.0016479492187500, -0.0037231445312500, -0.0013427734375000, -0.0038452148437500,
+-0.0010986328125000, -0.0034790039062500, -0.0013122558593750, -0.0040893554687500,
+-0.0008239746093750, -0.0036010742187500, -0.0008239746093750, -0.0036010742187500,
+-0.0008239746093750, -0.0036010742187500, -0.0005187988281250, -0.0032348632812500,
+-0.0006408691406250, -0.0033264160156250, -0.0004882812500000, -0.0031433105468750,
+-0.0005187988281250, -0.0028076171875000, -0.0004577636718750, -0.0026245117187500,
+-0.0003967285156250, -0.0024414062500000, -0.0004577636718750, -0.0020141601562500,
+-0.0003356933593750, -0.0016174316406250, -0.0003356933593750, -0.0014648437500000,
+-0.0004272460937500, -0.0007324218750000, -0.0003662109375000, -0.0002746582031250,
+-0.0004577636718750, -0.0005493164062500, -0.0007629394531250, 0.0006713867187500,
+-0.0007629394531250, 0.0004882812500000, -0.0010070800781250, 0.0007934570312500,
+-0.0012817382812500, 0.0015563964843750, -0.0011291503906250, 0.0008239746093750,
+-0.0021362304687500, 0.0014343261718750, -0.0018920898437500, 0.0011596679687500,
+-0.0021667480468750, 0.0007934570312500, -0.0029296875000000, 0.0009460449218750,
+-0.0024719238281250, 0.0005187988281250, -0.0032043457031250, 0.0004577636718750,
+-0.0031127929687500, 0.0002441406250000, -0.0031738281250000, 0.0000915527343750,
+-0.0036315917968750, 0.0001220703125000, -0.0032348632812500, -0.0001220703125000,
+-0.0031127929687500, -0.0001525878906250, -0.0034179687500000, -0.0001831054687500,
+-0.0025634765625000, -0.0001220703125000, -0.0018310546875000, -0.0000305175781250,
+-0.0023803710937500, 0.0000610351562500, -0.0003967285156250, 0.0003051757812500,
+-0.0004882812500000, 0.0004882812500000, -0.0003356933593750, 0.0006713867187500,
+0.0009155273437500, 0.0009155273437500, 0.0006103515625000, 0.0009155273437500,
+0.0005798339843750, 0.0005798339843750, 0.0008544921875000, 0.0010375976562500,
+0.0007324218750000, 0.0001525878906250, 0.0005493164062500, -0.0007629394531250,
+0.0006713867187500, 0.0000305175781250, 0.0006103515625000, -0.0019226074218750,
+0.0006103515625000, -0.0018005371093750, 0.0004882812500000, -0.0017395019531250,
+0.0003356933593750, -0.0028991699218750, 0.0004577636718750, -0.0025329589843750,
+0.0005493164062500, -0.0021057128906250, 0.0002136230468750, -0.0027770996093750,
+0.0007629394531250, -0.0020446777343750, 0.0013732910156250, -0.0010070800781250,
+0.0008850097656250, -0.0020141601562500, 0.0016174316406250, -0.0017089843750000,
+0.0019836425781250, -0.0008239746093750, 0.0013732910156250, -0.0021362304687500,
+0.0010070800781250, -0.0031127929687500, 0.0013427734375000, -0.0017395019531250,
+0.0008544921875000, -0.0043640136718750, 0.0004882812500000, -0.0043945312500000,
+0.0008239746093750, -0.0041503906250000, 0.0011596679687500, -0.0054321289062500,
+0.0007934570312500, -0.0049133300781250, 0.0005798339843750, -0.0041809082031250,
+0.0009460449218750, -0.0048522949218750, 0.0001525878906250, -0.0036315917968750,
+-0.0007324218750000, -0.0021057128906250, -0.0001525878906250, -0.0030822753906250,
+-0.0018310546875000, -0.0005187988281250, -0.0014648437500000, -0.0007019042968750,
+-0.0017395019531250, -0.0003051757812500, -0.0030822753906250, 0.0016784667968750,
+-0.0025024414062500, 0.0010070800781250, -0.0031127929687500, 0.0003662109375000,
+-0.0028686523437500, 0.0017700195312500, -0.0034179687500000, 0.0000610351562500,
+-0.0041809082031250, -0.0022583007812500, -0.0034179687500000, -0.0005798339843750,
+-0.0047912597656250, -0.0025939941406250, -0.0051879882812500, -0.0031738281250000,
+-0.0041198730468750, -0.0025634765625000, -0.0040588378906250, -0.0027465820312500,
+-0.0046691894531250, -0.0028991699218750, -0.0019836425781250, -0.0035705566406250,
+-0.0022277832031250, -0.0030212402343750, -0.0023498535156250, -0.0037841796875000,
+-0.0001831054687500, -0.0058898925781250, -0.0029602050781250, -0.0021057128906250,
+-0.0036621093750000, -0.0032958984375000, -0.0039367675781250, -0.0019226074218750,
+-0.0027160644531250, -0.0034790039062500, -0.0020141601562500, -0.0056762695312500,
+-0.0027770996093750, -0.0045166015625000, -0.0010986328125000, -0.0050964355468750,
+-0.0004272460937500, -0.0061340332031250, -0.0011901855468750, -0.0046997070312500,
+-0.0013427734375000, -0.0032653808593750, -0.0007934570312500, -0.0043334960937500,
+-0.0017700195312500, -0.0030212402343750, -0.0019836425781250, -0.0021972656250000,
+-0.0008239746093750, -0.0031433105468750, -0.0005187988281250, -0.0039367675781250,
+-0.0010986328125000, -0.0033264160156250, 0.0014648437500000, -0.0038452148437500,
+0.0015563964843750, -0.0048522949218750, 0.0007019042968750, -0.0036010742187500,
+0.0018310546875000, -0.0024414062500000, 0.0018920898437500, -0.0035705566406250,
+-0.0020141601562500, -0.0013122558593750, -0.0010986328125000, -0.0008239746093750,
+-0.0020751953125000, -0.0014343261718750, -0.0057067871093750, -0.0011596679687500,
+-0.0046081542968750, -0.0008544921875000, -0.0043640136718750, -0.0005798339843750,
+-0.0060424804687500, -0.0011291503906250, -0.0044555664062500, -0.0003662109375000,
+-0.0022277832031250, 0.0007019042968750, -0.0037536621093750, 0.0001525878906250,
+-0.0009460449218750, 0.0012512207031250, -0.0001525878906250, 0.0015258789062500,
+-0.0013427734375000, 0.0010375976562500, -0.0009155273437500, 0.0011596679687500,
+0.0000000000000000, 0.0014343261718750, -0.0024719238281250, 0.0006103515625000,
+-0.0020751953125000, 0.0003967285156250, -0.0018310546875000, 0.0009765625000000,
+-0.0029296875000000, 0.0009460449218750, -0.0021972656250000, 0.0005493164062500,
+-0.0019226074218750, 0.0020141601562500, -0.0020446777343750, 0.0022277832031250,
+-0.0009765625000000, 0.0016479492187500, 0.0000305175781250, 0.0018920898437500,
+-0.0003051757812500, 0.0021667480468750, 0.0012817382812500, 0.0008544921875000,
+0.0020751953125000, 0.0008239746093750, 0.0013122558593750, 0.0010375976562500,
+0.0017089843750000, 0.0006713867187500, 0.0028381347656250, 0.0006713867187500,
+-0.0000915527343750, 0.0009155273437500, 0.0013427734375000, 0.0010681152343750,
+0.0004882812500000, 0.0007934570312500, -0.0018920898437500, 0.0005493164062500,
+0.0003967285156250, 0.0007324218750000, -0.0032348632812500, 0.0005798339843750,
+-0.0020751953125000, 0.0003662109375000, -0.0024414062500000, 0.0004882812500000,
+-0.0048522949218750, 0.0007324218750000, -0.0025939941406250, 0.0005798339843750,
+-0.0045166015625000, 0.0000610351562500, -0.0041809082031250, 0.0004272460937500,
+-0.0032958984375000, -0.0000915527343750, -0.0036315917968750, -0.0010070800781250,
+-0.0028381347656250, -0.0004882812500000, -0.0027465820312500, -0.0012207031250000,
+-0.0021362304687500, -0.0014648437500000, -0.0019836425781250, -0.0013122558593750,
+-0.0019531250000000},
+{-0.0031127929687500, -0.0007019042968750, -0.0041503906250000, 0.0014343261718750,
+-0.0039672851562500, 0.0008544921875000, -0.0024108886718750, -0.0003967285156250,
+-0.0030212402343750, 0.0007019042968750, -0.0025939941406250, -0.0003051757812500,
+-0.0009765625000000, -0.0024108886718750, -0.0015869140625000, -0.0012512207031250,
+-0.0021972656250000, -0.0021667480468750, -0.0016479492187500, -0.0027770996093750,
+-0.0021667480468750, -0.0022277832031250, -0.0038452148437500, -0.0015563964843750,
+-0.0025024414062500, -0.0018615722656250, -0.0015869140625000, -0.0014038085937500,
+-0.0014343261718750, -0.0022277832031250, -0.0017395019531250, -0.0018310546875000,
+-0.0016479492187500, -0.0004882812500000, -0.0011596679687500, -0.0007934570312500,
+-0.0020141601562500, -0.0028686523437500, -0.0024108886718750, -0.0017395019531250,
+-0.0014953613281250, -0.0029602050781250, -0.0010681152343750, -0.0057983398437500,
+-0.0015563964843750, -0.0044555664062500, 0.0004577636718750, -0.0047912597656250,
+0.0005187988281250, -0.0059509277343750, 0.0000000000000000, -0.0046386718750000,
+0.0007934570312500, -0.0030517578125000, 0.0007019042968750, -0.0039978027343750,
+-0.0008850097656250, -0.0028381347656250, -0.0007629394531250, -0.0023498535156250,
+-0.0012817382812500, -0.0030212402343750, -0.0028381347656250, -0.0034484863281250,
+-0.0025634765625000, -0.0031433105468750, -0.0021057128906250, -0.0032958984375000,
+-0.0028686523437500, -0.0039367675781250, -0.0024414062500000, -0.0031127929687500,
+-0.0012207031250000, -0.0020141601562500, -0.0016479492187500, -0.0025634765625000,
+-0.0016479492187500, -0.0016174316406250, -0.0005187988281250, -0.0012817382812500,
+-0.0015258789062500, -0.0014343261718750, -0.0026550292968750, -0.0013427734375000,
+-0.0011901855468750, -0.0012512207031250, -0.0035705566406250, -0.0011596679687500,
+-0.0034179687500000, -0.0014038085937500, -0.0025939941406250, -0.0007934570312500,
+-0.0034790039062500, -0.0002441406250000, -0.0032958984375000, -0.0007934570312500,
+-0.0014038085937500, 0.0002136230468750, -0.0013732910156250, 0.0006103515625000,
+-0.0011291503906250, 0.0001831054687500, 0.0000610351562500, 0.0000305175781250,
+0.0000305175781250, 0.0003356933593750, -0.0000915527343750, 0.0003051757812500,
+0.0007629394531250, -0.0003051757812500, -0.0002136230468750, 0.0004882812500000,
+-0.0010070800781250, 0.0016174316406250, 0.0003051757812500, 0.0009155273437500,
+-0.0031127929687500, 0.0014343261718750, -0.0022583007812500, 0.0021057128906250,
+-0.0025634765625000, 0.0011901855468750, -0.0047302246093750, 0.0004272460937500,
+-0.0031433105468750, 0.0010986328125000, -0.0044555664062500, 0.0001220703125000,
+-0.0037841796875000, -0.0002136230468750, -0.0037231445312500, 0.0002746582031250,
+-0.0043029785156250, 0.0003662109375000, -0.0028381347656250, 0.0001220703125000,
+-0.0045776367187500, 0.0005493164062500, -0.0033874511718750, 0.0006103515625000,
+-0.0031738281250000, 0.0004272460937500, -0.0041503906250000, 0.0004272460937500,
+-0.0026245117187500, 0.0004882812500000, -0.0039978027343750, 0.0005187988281250,
+-0.0029296875000000, 0.0003051757812500, -0.0032043457031250, 0.0004272460937500,
+-0.0043334960937500, 0.0007324218750000, -0.0027160644531250, 0.0004882812500000,
+-0.0046691894531250, 0.0001831054687500, -0.0040283203125000, 0.0004882812500000,
+-0.0038146972656250, 0.0001220703125000, -0.0048828125000000, -0.0005187988281250,
+-0.0037536621093750, -0.0001525878906250, -0.0043640136718750, -0.0006408691406250,
+-0.0044860839843750, -0.0006408691406250, -0.0039062500000000, -0.0006713867187500,
+-0.0039062500000000, -0.0009155273437500, -0.0041198730468750, -0.0007324218750000,
+-0.0029602050781250, -0.0010681152343750, -0.0035095214843750, -0.0010681152343750,
+-0.0028686523437500, -0.0009765625000000, -0.0018005371093750, -0.0011596679687500,
+-0.0029296875000000, -0.0010681152343750, -0.0012817382812500, -0.0010375976562500,
+-0.0016784667968750, -0.0009765625000000, -0.0016479492187500, -0.0010375976562500,
+-0.0006408691406250, -0.0010681152343750, -0.0012207031250000, -0.0010375976562500,
+-0.0007019042968750, -0.0010681152343750, -0.0006408691406250, -0.0010681152343750,
+-0.0005493164062500, -0.0009765625000000, -0.0002441406250000, -0.0009460449218750,
+-0.0005798339843750, -0.0008544921875000, 0.0000000000000000, -0.0006408691406250,
+-0.0002136230468750, -0.0007019042968750, -0.0000915527343750, -0.0005187988281250,
+0.0003051757812500, -0.0001525878906250, -0.0000610351562500, -0.0003051757812500,
+0.0005187988281250, -0.0000610351562500, 0.0002441406250000, 0.0001525878906250,
+0.0003051757812500, -0.0001525878906250, 0.0007324218750000, -0.0003051757812500,
+0.0001525878906250, 0.0000000000000000, 0.0004882812500000, -0.0007629394531250,
+0.0002136230468750, -0.0006713867187500, -0.0001525878906250, -0.0007629394531250,
+-0.0001525878906250, -0.0012817382812500, -0.0005187988281250, -0.0010070800781250,
+-0.0009460449218750, -0.0014648437500000, -0.0008850097656250, -0.0014038085937500,
+-0.0013427734375000, -0.0013732910156250, -0.0018615722656250, -0.0015258789062500,
+-0.0016784667968750, -0.0013732910156250, -0.0028686523437500, -0.0012207031250000,
+-0.0025329589843750, -0.0012512207031250, -0.0028076171875000, -0.0013122558593750,
+-0.0037231445312500, -0.0012207031250000, -0.0031433105468750, -0.0010986328125000,
+-0.0039672851562500, -0.0010681152343750, -0.0039062500000000, -0.0012817382812500,
+-0.0040588378906250, -0.0009460449218750, -0.0045776367187500, -0.0006103515625000,
+-0.0042724609375000, -0.0008544921875000, -0.0049133300781250, -0.0003967285156250,
+-0.0046081542968750, -0.0003662109375000, -0.0045471191406250, -0.0002441406250000,
+-0.0047912597656250, 0.0000915527343750, -0.0041809082031250, -0.0001525878906250,
+-0.0043334960937500, -0.0003967285156250, -0.0041198730468750, 0.0000000000000000,
+-0.0038452148437500, -0.0003051757812500, -0.0038452148437500, -0.0008850097656250,
+-0.0035400390625000, -0.0005798339843750, -0.0036315917968750, -0.0006408691406250,
+-0.0032958984375000, -0.0007324218750000, -0.0035400390625000, -0.0010375976562500,
+-0.0037841796875000, -0.0010070800781250, -0.0032348632812500, -0.0008239746093750,
+-0.0045166015625000, -0.0024108886718750, -0.0045471191406250, -0.0020141601562500,
+-0.0043945312500000, -0.0022888183593750, -0.0052490234375000, -0.0037536621093750,
+-0.0054626464843750, -0.0032958984375000, -0.0045776367187500, -0.0027465820312500,
+-0.0052490234375000, -0.0033264160156250, -0.0048828125000000, -0.0027465820312500,
+-0.0036926269531250, -0.0017089843750000, -0.0041503906250000, -0.0022583007812500,
+-0.0032958984375000, -0.0007629394531250, -0.0025329589843750, -0.0008850097656250,
+-0.0022277832031250, -0.0007629394531250, -0.0018310546875000, 0.0003662109375000,
+-0.0014648437500000, 0.0001220703125000, -0.0018005371093750, -0.0001831054687500,
+-0.0013427734375000, 0.0001220703125000, -0.0019226074218750, -0.0003051757812500,
+-0.0027160644531250, -0.0007019042968750, -0.0021362304687500, -0.0002746582031250,
+-0.0032043457031250, -0.0018005371093750, -0.0033569335937500, -0.0015563964843750,
+-0.0029907226562500, -0.0017089843750000, -0.0036010742187500, -0.0027770996093750,
+-0.0039672851562500, -0.0023193359375000, -0.0027770996093750, -0.0024719238281250,
+-0.0028076171875000, -0.0028686523437500, -0.0025939941406250, -0.0020446777343750,
+-0.0017700195312500, -0.0014343261718750, -0.0020751953125000, -0.0020446777343750,
+-0.0023193359375000, -0.0001831054687500, -0.0020751953125000, -0.0000915527343750,
+-0.0025024414062500, -0.0000305175781250, -0.0032653808593750, 0.0010070800781250,
+-0.0028686523437500, 0.0006408691406250, -0.0026550292968750, -0.0001220703125000,
+-0.0032043457031250, 0.0004882812500000, -0.0025939941406250, 0.0000610351562500,
+-0.0016174316406250, -0.0010070800781250, -0.0020141601562500, -0.0006713867187500,
+-0.0014038085937500, -0.0018310546875000, -0.0011291503906250, -0.0016174316406250,
+-0.0010986328125000, -0.0012207031250000, -0.0010070800781250, -0.0014038085937500,
+-0.0009155273437500, -0.0009155273437500, -0.0001220703125000, -0.0003356933593750,
+-0.0004577636718750, -0.0003356933593750, -0.0003662109375000, -0.0004882812500000,
+0.0002746582031250, -0.0002441406250000, 0.0000305175781250, -0.0003356933593750,
+0.0006103515625000, -0.0021972656250000, 0.0003967285156250, -0.0016174316406250,
+0.0009765625000000, -0.0024414062500000, 0.0017700195312500, -0.0043945312500000,
+0.0010070800781250, -0.0033874511718750, 0.0021362304687500, -0.0036010742187500,
+0.0025939941406250, -0.0040893554687500, 0.0015258789062500, -0.0032348632812500,
+0.0011291503906250, -0.0023498535156250, 0.0018005371093750, -0.0025329589843750,
+0.0000610351562500, -0.0014953613281250, -0.0001831054687500, -0.0011596679687500,
+0.0000000000000000, -0.0015869140625000, -0.0007629394531250, -0.0015563964843750,
+-0.0010070800781250, -0.0011596679687500, -0.0004577636718750, -0.0017089843750000,
+-0.0002746582031250, -0.0019531250000000, -0.0012817382812500, -0.0016174316406250,
+-0.0017700195312500, -0.0010070800781250, -0.0010375976562500, -0.0007629394531250,
+-0.0032958984375000, -0.0016784667968750, -0.0035705566406250, -0.0007934570312500,
+-0.0027770996093750, -0.0015563964843750, -0.0035095214843750, -0.0029907226562500,
+-0.0040283203125000, -0.0018005371093750, -0.0017700195312500, -0.0036315917968750,
+-0.0022888183593750, -0.0039062500000000, -0.0020446777343750, -0.0034484863281250,
+-0.0004272460937500, -0.0040283203125000, -0.0011901855468750, -0.0042114257812500,
+-0.0008850097656250, -0.0037231445312500, -0.0008544921875000, -0.0042724609375000,
+-0.0010375976562500, -0.0040588378906250, -0.0010375976562500, -0.0037536621093750,
+-0.0010986328125000, -0.0047302246093750, -0.0008239746093750, -0.0041198730468750,
+-0.0012512207031250, -0.0047607421875000, -0.0007324218750000, -0.0049438476562500,
+0.0001525878906250, -0.0047607421875000, -0.0003051757812500, -0.0056457519531250,
+-0.0000305175781250, -0.0044555664062500, 0.0003967285156250, -0.0049743652343750,
+-0.0000305175781250, -0.0045471191406250, -0.0002746582031250, -0.0034179687500000,
+0.0001831054687500, -0.0041503906250000, -0.0009155273437500, -0.0028991699218750,
+-0.0007629394531250, -0.0028991699218750, -0.0006103515625000, -0.0031738281250000,
+-0.0011901855468750, -0.0028991699218750, -0.0010986328125000, -0.0032043457031250,
+-0.0009460449218750, -0.0035705566406250, -0.0011901855468750, -0.0037231445312500,
+-0.0008850097656250, -0.0038757324218750, -0.0005798339843750, -0.0042419433593750,
+-0.0010070800781250, -0.0043945312500000, 0.0002746582031250, -0.0036926269531250,
+0.0000915527343750, -0.0042419433593750, 0.0003356933593750, -0.0038757324218750,
+0.0013427734375000, -0.0031738281250000, 0.0009460449218750, -0.0040283203125000,
+0.0011596679687500, -0.0025024414062500, 0.0014343261718750, -0.0026245117187500,
+0.0011291503906250, -0.0024108886718750, 0.0009460449218750, -0.0014038085937500,
+0.0011596679687500, -0.0017089843750000, 0.0002441406250000, -0.0007629394531250,
+0.0004577636718750, -0.0009155273437500, 0.0000000000000000, -0.0006713867187500,
+-0.0008239746093750, -0.0001220703125000, -0.0003051757812500, -0.0006713867187500,
+-0.0016174316406250, 0.0000915527343750, -0.0017089843750000, 0.0000305175781250,
+-0.0014953613281250, 0.0000000000000000, -0.0020751953125000, 0.0000915527343750,
+-0.0020751953125000, -0.0006713867187500, -0.0015258789062500, -0.0001220703125000,
+-0.0016479492187500, -0.0001831054687500, -0.0015869140625000, -0.0007629394531250,
+-0.0012207031250000, -0.0008544921875000, -0.0013122558593750, -0.0009765625000000,
+-0.0014038085937500},
+{-0.0006103515625000, -0.0026550292968750, -0.0006103515625000, -0.0038452148437500,
+-0.0007629394531250, -0.0041503906250000, -0.0002136230468750, -0.0035095214843750,
+0.0002441406250000, -0.0033569335937500, 0.0000610351562500, -0.0034484863281250,
+0.0011596679687500, -0.0026245117187500, 0.0010375976562500, -0.0023498535156250,
+0.0011901855468750, -0.0022888183593750, 0.0021057128906250, -0.0018310546875000,
+0.0019836425781250, -0.0017089843750000, 0.0014343261718750, -0.0022583007812500,
+0.0018920898437500, -0.0018920898437500, 0.0014038085937500, -0.0020446777343750,
+0.0003662109375000, -0.0028076171875000, 0.0008544921875000, -0.0025329589843750,
+0.0002746582031250, -0.0025634765625000, -0.0003662109375000, -0.0021972656250000,
+0.0000305175781250, -0.0027160644531250, 0.0004272460937500, -0.0033569335937500,
+-0.0001831054687500, -0.0026855468750000, -0.0004272460937500, -0.0034484863281250,
+0.0001831054687500, -0.0037536621093750, -0.0010986328125000, -0.0034484863281250,
+-0.0025024414062500, -0.0034790039062500, -0.0015258789062500, -0.0036010742187500,
+-0.0031738281250000, -0.0030822753906250, -0.0036926269531250, -0.0032958984375000,
+-0.0029602050781250, -0.0036621093750000, -0.0030822753906250, -0.0037841796875000,
+-0.0035400390625000, -0.0041198730468750, -0.0018310546875000, -0.0044555664062500,
+-0.0020141601562500, -0.0049743652343750, -0.0017700195312500, -0.0050964355468750,
+-0.0007019042968750, -0.0053405761718750, -0.0011596679687500, -0.0056762695312500,
+-0.0008850097656250, -0.0048522949218750, -0.0006713867187500, -0.0053710937500000,
+-0.0006103515625000, -0.0048522949218750, -0.0007934570312500, -0.0038146972656250,
+-0.0010070800781250, -0.0044860839843750, 0.0001831054687500, -0.0032958984375000,
+-0.0000915527343750, -0.0030822753906250, 0.0000305175781250, -0.0032958984375000,
+0.0009460449218750, -0.0029296875000000, 0.0004882812500000, -0.0029296875000000,
+0.0003662109375000, -0.0031738281250000, 0.0009155273437500, -0.0034484863281250,
+0.0003662109375000, -0.0030822753906250, -0.0003662109375000, -0.0029296875000000,
+0.0003051757812500, -0.0033264160156250, -0.0005187988281250, -0.0018005371093750,
+-0.0006103515625000, -0.0023498535156250, -0.0003662109375000, -0.0017089843750000,
+-0.0005798339843750, -0.0003967285156250, -0.0004577636718750, -0.0014953613281250,
+-0.0001525878906250, 0.0002136230468750, -0.0002441406250000, -0.0001525878906250,
+0.0000915527343750, -0.0000305175781250, 0.0004577636718750, 0.0011596679687500,
+0.0002136230468750, 0.0002136230468750, 0.0007324218750000, 0.0003356933593750,
+0.0008850097656250, 0.0005187988281250, 0.0006408691406250, 0.0003356933593750,
+0.0007324218750000, -0.0001220703125000, 0.0009155273437500, -0.0004577636718750,
+0.0002136230468750, -0.0002441406250000, 0.0004272460937500, -0.0011901855468750,
+0.0002746582031250, -0.0012207031250000, -0.0002441406250000, -0.0010070800781250,
+0.0001220703125000, -0.0021362304687500, -0.0003967285156250, -0.0011291503906250,
+-0.0003051757812500, -0.0020141601562500, -0.0004272460937500, -0.0023498535156250,
+-0.0007629394531250, -0.0018005371093750, -0.0004882812500000, -0.0028686523437500,
+-0.0010070800781250, -0.0025024414062500, -0.0010070800781250, -0.0025024414062500,
+-0.0009460449218750, -0.0031127929687500, -0.0011596679687500, -0.0031738281250000,
+-0.0010986328125000, -0.0028381347656250, -0.0012207031250000, -0.0034179687500000,
+-0.0011291503906250, -0.0028991699218750, -0.0012512207031250, -0.0028686523437500,
+-0.0014038085937500, -0.0033264160156250, -0.0012207031250000, -0.0028076171875000,
+-0.0014343261718750, -0.0033264160156250, -0.0014648437500000, -0.0034179687500000,
+-0.0012207031250000, -0.0033569335937500, -0.0012207031250000, -0.0037536621093750,
+-0.0013427734375000, -0.0039672851562500, -0.0005493164062500, -0.0037841796875000,
+-0.0007324218750000, -0.0042724609375000, -0.0004577636718750, -0.0043334960937500,
+0.0001525878906250, -0.0041809082031250, -0.0000915527343750, -0.0048217773437500,
+0.0003662109375000, -0.0044250488281250, 0.0004577636718750, -0.0048828125000000,
+0.0004577636718750, -0.0048828125000000, 0.0006408691406250, -0.0046081542968750,
+0.0006103515625000, -0.0052795410156250, 0.0005493164062500, -0.0044860839843750,
+0.0006103515625000, -0.0046691894531250, 0.0007019042968750, -0.0043029785156250,
+0.0006408691406250, -0.0034179687500000, 0.0006408691406250, -0.0036621093750000,
+0.0011291503906250, -0.0022583007812500, 0.0010681152343750, -0.0022583007812500,
+0.0013122558593750, -0.0016479492187500, 0.0018005371093750, -0.0003967285156250,
+0.0016479492187500, -0.0007019042968750, 0.0016174316406250, 0.0001525878906250,
+0.0018615722656250, 0.0008544921875000, 0.0014953613281250, 0.0005187988281250,
+0.0010681152343750, 0.0005493164062500, 0.0013122558593750, 0.0010375976562500,
+0.0005493164062500, -0.0003967285156250, 0.0006103515625000, 0.0000915527343750,
+0.0005493164062500, -0.0006408691406250, 0.0001220703125000, -0.0020446777343750,
+0.0003967285156250, -0.0012512207031250, 0.0003662109375000, -0.0030517578125000,
+0.0002746582031250, -0.0030822753906250, 0.0006103515625000, -0.0029907226562500,
+0.0010070800781250, -0.0041198730468750, 0.0009155273437500, -0.0040283203125000,
+0.0008544921875000, -0.0035400390625000, 0.0012207031250000, -0.0039062500000000,
+0.0010375976562500, -0.0036621093750000, 0.0005798339843750, -0.0030212402343750,
+0.0008850097656250, -0.0030517578125000, 0.0007934570312500, -0.0021362304687500,
+0.0007934570312500, -0.0020141601562500, 0.0006713867187500, -0.0019226074218750,
+0.0007324218750000, -0.0012207031250000, 0.0009155273437500, -0.0012817382812500,
+-0.0002746582031250, -0.0020446777343750, 0.0000610351562500, -0.0014038085937500,
+-0.0004272460937500, -0.0019226074218750, -0.0014343261718750, -0.0029296875000000,
+-0.0007629394531250, -0.0019836425781250, -0.0023193359375000, -0.0025939941406250,
+-0.0023193359375000, -0.0024108886718750, -0.0021972656250000, -0.0019531250000000,
+-0.0029907226562500, -0.0019836425781250, -0.0027465820312500, -0.0019226074218750,
+-0.0024108886718750, -0.0018310546875000, -0.0024108886718750, -0.0011596679687500,
+-0.0020751953125000, -0.0018920898437500, -0.0015869140625000, -0.0024108886718750,
+-0.0016174316406250, -0.0014038085937500, -0.0014038085937500, -0.0038146972656250,
+-0.0013122558593750, -0.0030822753906250, -0.0010070800781250, -0.0028686523437500,
+-0.0007934570312500, -0.0043334960937500, -0.0010070800781250, -0.0033874511718750,
+-0.0007934570312500, -0.0039978027343750, -0.0008239746093750, -0.0039672851562500,
+-0.0009460449218750, -0.0028686523437500, -0.0008239746093750, -0.0023498535156250,
+-0.0009460449218750, -0.0024719238281250, -0.0012817382812500, -0.0006713867187500,
+-0.0010681152343750, -0.0001220703125000, -0.0015258789062500, -0.0009460449218750,
+-0.0021362304687500, -0.0006713867187500, -0.0016174316406250, -0.0000305175781250,
+-0.0021057128906250, -0.0027160644531250, -0.0025939941406250, -0.0023193359375000,
+-0.0017395019531250, -0.0022583007812500, -0.0009765625000000, -0.0039062500000000,
+-0.0014953613281250, -0.0032348632812500, -0.0006103515625000, -0.0023803710937500,
+-0.0000610351562500, -0.0030517578125000, -0.0003356933593750, -0.0024414062500000,
+-0.0006103515625000, -0.0002441406250000, 0.0002136230468750, -0.0032043457031250,
+-0.0001220703125000, -0.0014038085937500, -0.0000305175781250, -0.0022277832031250,
+-0.0003662109375000, -0.0016479492187500, -0.0006103515625000, -0.0000610351562500,
+-0.0003356933593750, -0.0002746582031250, -0.0015258789062500, -0.0011596679687500,
+-0.0018920898437500, -0.0006713867187500, -0.0011596679687500, -0.0008239746093750,
+-0.0012207031250000, -0.0018005371093750, -0.0017700195312500, -0.0014648437500000,
+0.0003662109375000, -0.0010681152343750, 0.0007324218750000, -0.0009765625000000,
+-0.0000915527343750, -0.0014038085937500, 0.0009765625000000, -0.0011291503906250,
+0.0020751953125000, -0.0006713867187500, -0.0023498535156250, -0.0029296875000000,
+-0.0005798339843750, -0.0023498535156250, -0.0014953613281250, -0.0029907226562500,
+-0.0054016113281250, -0.0050964355468750, -0.0027770996093750, -0.0039367675781250,
+-0.0034484863281250, -0.0038757324218750, -0.0046386718750000, -0.0046997070312500,
+-0.0024414062500000, -0.0038452148437500, -0.0005187988281250, -0.0025024414062500,
+-0.0017089843750000, -0.0030822753906250, 0.0007629394531250, -0.0026550292968750,
+0.0014038085937500, -0.0021667480468750, 0.0003662109375000, -0.0025634765625000,
+0.0008544921875000, -0.0029907226562500, 0.0013732910156250, -0.0026550292968750,
+-0.0014953613281250, -0.0021972656250000, -0.0007019042968750, -0.0027465820312500,
+-0.0010375976562500, -0.0019531250000000, -0.0031127929687500, -0.0007019042968750,
+-0.0019226074218750, -0.0014953613281250, -0.0022888183593750, -0.0009765625000000,
+-0.0021667480468750, -0.0006103515625000, -0.0021667480468750, -0.0009765625000000,
+-0.0022888183593750, -0.0014953613281250, -0.0016479492187500, -0.0014648437500000,
+-0.0025024414062500, -0.0008544921875000, -0.0022277832031250, -0.0013122558593750,
+-0.0024108886718750, -0.0010070800781250, -0.0029296875000000, -0.0001831054687500,
+-0.0024108886718750, -0.0006103515625000, -0.0039367675781250, -0.0004882812500000,
+-0.0036315917968750, -0.0004882812500000, -0.0036315917968750, -0.0002441406250000,
+-0.0049438476562500, -0.0001220703125000, -0.0046997070312500, -0.0003967285156250,
+-0.0036315917968750, 0.0005798339843750, -0.0047912597656250, 0.0005187988281250,
+-0.0037231445312500, 0.0003967285156250, -0.0022277832031250, 0.0009155273437500,
+-0.0038146972656250, 0.0009155273437500, -0.0009155273437500, 0.0004272460937500,
+-0.0017089843750000, 0.0004882812500000, -0.0013732910156250, 0.0007324218750000,
+0.0004882812500000, 0.0005187988281250, -0.0011596679687500, 0.0003662109375000,
+0.0004272460937500, 0.0012207031250000, 0.0000000000000000, 0.0011901855468750,
+-0.0006103515625000, 0.0010986328125000, -0.0002136230468750, 0.0016479492187500,
+-0.0010986328125000, 0.0015869140625000, -0.0011901855468750, 0.0012207031250000,
+-0.0019226074218750, 0.0013122558593750, -0.0021972656250000, 0.0012817382812500,
+-0.0023803710937500, 0.0010986328125000, -0.0032348632812500, 0.0010986328125000,
+-0.0024414062500000, 0.0010070800781250, -0.0033874511718750, 0.0010681152343750,
+-0.0032653808593750, 0.0009155273437500, -0.0025939941406250, 0.0006408691406250,
+-0.0038146972656250, 0.0007629394531250, -0.0021057128906250, 0.0006103515625000,
+-0.0026855468750000, 0.0005187988281250, -0.0027160644531250, 0.0005493164062500,
+-0.0016784667968750, 0.0005798339843750, -0.0024108886718750, 0.0005187988281250,
+-0.0009460449218750, 0.0003356933593750, -0.0013427734375000, 0.0004882812500000,
+-0.0007324218750000, 0.0003356933593750, 0.0004272460937500, 0.0001220703125000,
+-0.0005798339843750, 0.0001831054687500, 0.0011596679687500, 0.0000305175781250,
+0.0007934570312500, 0.0000610351562500, 0.0008544921875000, -0.0000915527343750,
+0.0018615722656250, -0.0002441406250000, 0.0010070800781250, -0.0000915527343750,
+0.0020141601562500, -0.0007629394531250, 0.0018310546875000, -0.0005493164062500,
+0.0017395019531250, -0.0008239746093750, 0.0023498535156250, -0.0013732910156250,
+0.0019226074218750, -0.0009765625000000, 0.0020751953125000, -0.0017700195312500,
+0.0022888183593750, -0.0017700195312500, 0.0020141601562500, -0.0017089843750000,
+0.0018920898437500, -0.0020141601562500, 0.0020141601562500, -0.0019531250000000,
+0.0014648437500000, -0.0019226074218750, 0.0016784667968750, -0.0019531250000000,
+0.0014038085937500},
+{0.0002746582031250, -0.0021667480468750, -0.0011291503906250, -0.0024108886718750,
+-0.0009460449218750, -0.0044250488281250, -0.0004882812500000, -0.0038757324218750,
+-0.0029602050781250, -0.0032043457031250, -0.0024414062500000, -0.0037536621093750,
+-0.0024414062500000, -0.0032653808593750, -0.0039672851562500, -0.0021057128906250,
+-0.0030212402343750, -0.0025634765625000, -0.0026245117187500, -0.0024414062500000,
+-0.0028076171875000, -0.0021667480468750, -0.0024108886718750, -0.0020751953125000,
+-0.0015563964843750, -0.0023193359375000, -0.0016479492187500, -0.0024414062500000,
+-0.0029907226562500, -0.0011901855468750, -0.0025939941406250, -0.0014648437500000,
+-0.0027770996093750, -0.0013122558593750, -0.0041809082031250, -0.0002136230468750,
+-0.0041198730468750, -0.0005187988281250, -0.0036010742187500, -0.0010070800781250,
+-0.0045166015625000, -0.0008850097656250, -0.0038146972656250, -0.0008544921875000,
+-0.0029296875000000, -0.0013122558593750, -0.0041198730468750, -0.0014038085937500,
+-0.0013427734375000, -0.0006103515625000, -0.0021667480468750, -0.0009765625000000,
+-0.0017089843750000, -0.0007324218750000, 0.0004272460937500, 0.0000305175781250,
+-0.0010070800781250, -0.0003967285156250, -0.0003051757812500, 0.0002441406250000,
+0.0000610351562500, 0.0002136230468750, -0.0007324218750000, 0.0003662109375000,
+-0.0011596679687500, 0.0007019042968750, -0.0009765625000000, 0.0003967285156250,
+-0.0023498535156250, 0.0011596679687500, -0.0024108886718750, 0.0011291503906250,
+-0.0027465820312500, 0.0010070800781250, -0.0038452148437500, 0.0014038085937500,
+-0.0037536621093750, 0.0013732910156250, -0.0038757324218750, 0.0009765625000000,
+-0.0047607421875000, 0.0009460449218750, -0.0039367675781250, 0.0011596679687500,
+-0.0033874511718750, 0.0010681152343750, -0.0047302246093750, 0.0009155273437500,
+-0.0016784667968750, 0.0014648437500000, -0.0027160644531250, 0.0014953613281250,
+-0.0019836425781250, 0.0014038085937500, 0.0003662109375000, 0.0016479492187500,
+-0.0013427734375000, 0.0016174316406250, 0.0013122558593750, 0.0016174316406250,
+0.0010986328125000, 0.0015258789062500, 0.0012817382812500, 0.0015563964843750,
+0.0029602050781250, 0.0017700195312500, 0.0021972656250000, 0.0015869140625000,
+0.0028381347656250, 0.0015563964843750, 0.0033264160156250, 0.0016784667968750,
+0.0030822753906250, 0.0014648437500000, 0.0031127929687500, 0.0011901855468750,
+0.0037536621093750, 0.0012817382812500, 0.0036926269531250, 0.0010681152343750,
+0.0038146972656250, 0.0010375976562500, 0.0037841796875000, 0.0009460449218750,
+0.0036315917968750, 0.0008544921875000, 0.0037536621093750, 0.0008850097656250,
+0.0031127929687500, 0.0004882812500000, 0.0032958984375000, 0.0005798339843750,
+0.0029907226562500, 0.0002746582031250, 0.0025329589843750, -0.0001525878906250,
+0.0031738281250000, 0.0000915527343750, 0.0022583007812500, -0.0006713867187500,
+0.0026245117187500, -0.0005798339843750, 0.0024414062500000, -0.0007324218750000,
+0.0018005371093750, -0.0012512207031250, 0.0025634765625000, -0.0010375976562500,
+0.0013122558593750, -0.0014343261718750, 0.0018310546875000, -0.0014648437500000,
+0.0018005371093750, -0.0015258789062500, 0.0010375976562500, -0.0018005371093750,
+0.0018920898437500, -0.0016784667968750, 0.0011596679687500, -0.0019226074218750,
+0.0016174316406250, -0.0020141601562500, 0.0016784667968750, -0.0019836425781250,
+0.0013122558593750, -0.0020751953125000, 0.0021667480468750, -0.0020751953125000,
+0.0017395019531250, -0.0017700195312500, 0.0020446777343750, -0.0020141601562500,
+0.0023498535156250, -0.0014953613281250, 0.0022277832031250, -0.0010375976562500,
+0.0027465820312500, -0.0015258789062500, 0.0030212402343750, -0.0001220703125000,
+0.0030212402343750, -0.0003051757812500, 0.0028076171875000, -0.0001220703125000,
+0.0028076171875000, 0.0007019042968750, 0.0030212402343750, 0.0002746582031250,
+0.0022583007812500, 0.0007934570312500, 0.0027160644531250, 0.0009460449218750,
+0.0024108886718750, 0.0005187988281250, 0.0017089843750000, 0.0004577636718750,
+0.0024414062500000, 0.0007324218750000, 0.0012817382812500, 0.0001831054687500,
+0.0014648437500000, 0.0000000000000000, 0.0013427734375000, 0.0003356933593750,
+0.0004577636718750, 0.0004272460937500, 0.0007324218750000, 0.0002136230468750,
+-0.0000915527343750, 0.0003967285156250, -0.0001220703125000, 0.0004882812500000,
+-0.0005798339843750, 0.0003356933593750, -0.0014343261718750, 0.0002441406250000,
+-0.0014648437500000, 0.0003051757812500, -0.0023498535156250, -0.0000610351562500,
+-0.0023498535156250, -0.0000610351562500, -0.0024414062500000, -0.0001525878906250,
+-0.0027160644531250, -0.0004272460937500, -0.0022583007812500, -0.0003967285156250,
+-0.0025634765625000, -0.0004577636718750, -0.0021667480468750, -0.0006103515625000,
+-0.0021972656250000, -0.0004272460937500, -0.0025939941406250, -0.0002441406250000,
+-0.0024108886718750, -0.0004882812500000, -0.0030822753906250, -0.0003051757812500,
+-0.0032653808593750, 0.0000915527343750, -0.0035705566406250, -0.0007324218750000,
+-0.0041503906250000, -0.0014953613281250, -0.0043334960937500, -0.0007019042968750,
+-0.0048522949218750, -0.0019226074218750, -0.0053405761718750, -0.0022277832031250,
+-0.0050048828125000, -0.0018005371093750, -0.0050354003906250, -0.0021057128906250,
+-0.0059509277343750, -0.0021667480468750, -0.0042419433593750, -0.0009460449218750,
+-0.0045471191406250, -0.0011596679687500, -0.0049438476562500, -0.0012207031250000,
+-0.0039978027343750, -0.0003356933593750, -0.0038146972656250, -0.0004272460937500,
+-0.0039062500000000, -0.0005798339843750, -0.0038146972656250, -0.0006103515625000,
+-0.0033569335937500, -0.0002746582031250, -0.0034484863281250, -0.0001525878906250,
+-0.0041503906250000, -0.0004882812500000, -0.0026855468750000, -0.0000610351562500,
+-0.0026855468750000, 0.0000305175781250, -0.0028381347656250, -0.0001220703125000,
+-0.0020751953125000, 0.0000610351562500, -0.0022888183593750, 0.0000610351562500,
+-0.0030822753906250, -0.0007019042968750, -0.0018920898437500, -0.0003967285156250,
+-0.0025024414062500, -0.0007324218750000, -0.0040588378906250, -0.0014343261718750,
+-0.0030212402343750, -0.0009460449218750, -0.0038146972656250, -0.0018920898437500,
+-0.0042724609375000, -0.0019531250000000, -0.0036315917968750, -0.0019836425781250,
+-0.0034484863281250, -0.0022583007812500, -0.0037536621093750, -0.0018920898437500,
+-0.0028076171875000, -0.0025634765625000, -0.0032348632812500, -0.0028076171875000,
+-0.0026550292968750, -0.0025329589843750, -0.0018005371093750, -0.0022888183593750,
+-0.0027160644531250, -0.0023498535156250, -0.0009460449218750, -0.0035095214843750,
+-0.0014343261718750, -0.0026855468750000, -0.0013732910156250, -0.0027465820312500,
+-0.0000305175781250, -0.0035095214843750, -0.0006408691406250, -0.0025329589843750,
+-0.0006408691406250, -0.0035400390625000, 0.0000610351562500, -0.0029907226562500,
+-0.0008239746093750, -0.0029907226562500, -0.0017700195312500, -0.0035400390625000,
+-0.0007934570312500, -0.0026550292968750, -0.0018310546875000, -0.0039672851562500,
+-0.0020751953125000, -0.0037231445312500, -0.0014648437500000, -0.0036621093750000,
+-0.0012817382812500, -0.0047302246093750, -0.0012817382812500, -0.0045166015625000,
+-0.0005187988281250, -0.0035400390625000, -0.0003356933593750, -0.0034790039062500,
+-0.0007324218750000, -0.0034790039062500, -0.0006408691406250, -0.0026855468750000,
+-0.0002441406250000, -0.0021362304687500, -0.0007324218750000, -0.0029296875000000,
+-0.0009155273437500, -0.0019531250000000, -0.0006103515625000, -0.0027160644531250,
+-0.0004577636718750, -0.0039062500000000, -0.0005493164062500, -0.0023193359375000,
+-0.0003051757812500, -0.0048828125000000, -0.0001831054687500, -0.0049743652343750,
+-0.0005493164062500, -0.0046997070312500, -0.0007934570312500, -0.0059814453125000,
+-0.0006103515625000, -0.0059204101562500, -0.0014648437500000, -0.0053710937500000,
+-0.0014953613281250, -0.0057983398437500, -0.0014953613281250, -0.0056762695312500,
+-0.0020751953125000, -0.0052490234375000, -0.0021362304687500, -0.0056457519531250,
+-0.0019531250000000, -0.0047912597656250, -0.0021057128906250, -0.0051879882812500,
+-0.0018920898437500, -0.0047607421875000, -0.0016479492187500, -0.0040893554687500,
+-0.0020446777343750, -0.0050354003906250, -0.0014953613281250, -0.0030822753906250,
+-0.0015869140625000, -0.0039367675781250, -0.0010986328125000, -0.0033874511718750,
+-0.0004882812500000, -0.0017395019531250, -0.0011291503906250, -0.0029907226562500,
+-0.0002136230468750, -0.0011901855468750, -0.0001220703125000, -0.0015258789062500,
+-0.0000915527343750, -0.0015869140625000, 0.0001525878906250, -0.0007324218750000,
+-0.0000610351562500, -0.0017395019531250, 0.0006408691406250, -0.0011596679687500,
+0.0003662109375000, -0.0017700195312500, 0.0007019042968750, -0.0019226074218750,
+0.0013732910156250, -0.0015869140625000, 0.0007629394531250, -0.0025329589843750,
+0.0014038085937500, -0.0012512207031250, 0.0017089843750000, -0.0017700195312500,
+0.0010986328125000, -0.0014038085937500, 0.0009765625000000, -0.0003051757812500,
+0.0014038085937500, -0.0010681152343750, 0.0000305175781250, 0.0002136230468750,
+0.0003356933593750, 0.0002441406250000, 0.0001220703125000, 0.0001220703125000,
+-0.0007629394531250, 0.0007019042968750, -0.0001220703125000, 0.0006103515625000,
+-0.0007629394531250, 0.0006103515625000, -0.0006103515625000, 0.0010681152343750,
+-0.0003356933593750, 0.0006103515625000, -0.0005493164062500, 0.0004577636718750,
+-0.0003356933593750, 0.0012817382812500, 0.0000000000000000, -0.0003051757812500,
+-0.0000305175781250, 0.0007324218750000, 0.0002136230468750, 0.0003967285156250,
+0.0006103515625000, -0.0008544921875000, 0.0004882812500000, 0.0005798339843750,
+0.0008850097656250, -0.0014343261718750, 0.0010070800781250, -0.0003051757812500,
+0.0007629394531250, -0.0002746582031250, 0.0007934570312500, -0.0018615722656250,
+0.0009765625000000, -0.0004577636718750, 0.0003662109375000, -0.0015869140625000,
+0.0004882812500000, -0.0010681152343750, 0.0003967285156250, -0.0003967285156250,
+-0.0000305175781250, -0.0008239746093750, 0.0001831054687500, -0.0002441406250000,
+-0.0000305175781250, 0.0001525878906250, -0.0001525878906250, 0.0002441406250000,
+-0.0000305175781250, 0.0002441406250000, 0.0000000000000000, 0.0005493164062500,
+-0.0001220703125000, 0.0008239746093750, -0.0000915527343750, 0.0001831054687500,
+-0.0000915527343750, 0.0010375976562500, -0.0001220703125000, 0.0007934570312500,
+-0.0001220703125000, 0.0002136230468750, -0.0001831054687500, 0.0014343261718750,
+-0.0001525878906250, 0.0000305175781250, -0.0003051757812500, 0.0009155273437500,
+-0.0002136230468750, 0.0007934570312500, -0.0000610351562500, -0.0001220703125000,
+-0.0002746582031250, 0.0013122558593750, 0.0004882812500000, -0.0004272460937500,
+0.0003051757812500, 0.0003967285156250, 0.0008239746093750, 0.0003662109375000,
+0.0016784667968750, -0.0007019042968750, 0.0014038085937500, 0.0007629394531250,
+0.0022888183593750, -0.0001525878906250, 0.0024414062500000, -0.0000305175781250,
+0.0023803710937500, 0.0004272460937500, 0.0026245117187500, 0.0001220703125000,
+0.0025634765625000, 0.0001831054687500, 0.0024414062500000, 0.0001525878906250,
+0.0022888183593750, 0.0003662109375000, 0.0022888183593750, -0.0000610351562500,
+0.0022583007812500, -0.0001831054687500, 0.0020751953125000, 0.0003967285156250,
+0.0020751953125000, -0.0008239746093750, 0.0019531250000000, 0.0000000000000000,
+0.0019531250000000, -0.0006103515625000, 0.0018615722656250, -0.0018615722656250,
+0.0016479492187500},
+{-0.0018615722656250, -0.0015563964843750, -0.0025024414062500, -0.0036010742187500,
+-0.0008239746093750, -0.0012512207031250, -0.0011901855468750, -0.0018005371093750,
+-0.0012207031250000, -0.0021972656250000, -0.0001220703125000, -0.0012817382812500,
+-0.0005187988281250, -0.0025634765625000, -0.0000305175781250, -0.0015563964843750,
+0.0000305175781250, -0.0027160644531250, -0.0001220703125000, -0.0024414062500000,
+0.0001525878906250, -0.0014953613281250, 0.0002136230468750, -0.0032348632812500,
+0.0000915527343750, -0.0007324218750000, 0.0000305175781250, -0.0014953613281250,
+-0.0000305175781250, -0.0010986328125000, -0.0001525878906250, 0.0009765625000000,
+-0.0001525878906250, -0.0002746582031250, -0.0000915527343750, 0.0009765625000000,
+-0.0002441406250000, 0.0013122558593750, -0.0000610351562500, 0.0008544921875000,
+0.0002136230468750, 0.0011291503906250, 0.0000305175781250, 0.0011291503906250,
+0.0003051757812500, 0.0001220703125000, 0.0005798339843750, 0.0006713867187500,
+0.0003967285156250, 0.0001220703125000, 0.0004882812500000, -0.0007934570312500,
+0.0008850097656250, 0.0003967285156250, 0.0002441406250000, -0.0012817382812500,
+0.0005798339843750, -0.0005493164062500, 0.0005187988281250, -0.0005187988281250,
+0.0000305175781250, -0.0012817382812500, 0.0005187988281250, 0.0001220703125000,
+0.0002441406250000, -0.0010986328125000, 0.0001831054687500, 0.0000000000000000,
+0.0003662109375000, 0.0003051757812500, 0.0003356933593750, -0.0002136230468750,
+0.0003967285156250, 0.0014953613281250, 0.0006103515625000, 0.0006408691406250,
+0.0006103515625000, 0.0019836425781250, 0.0007934570312500, 0.0025634765625000,
+0.0010681152343750, 0.0018005371093750, 0.0010070800781250, 0.0029907226562500,
+0.0011596679687500, 0.0031127929687500, 0.0013427734375000, 0.0027465820312500,
+0.0011901855468750, 0.0033264160156250, 0.0011291503906250, 0.0041198730468750,
+0.0013732910156250, 0.0038452148437500, 0.0008850097656250, 0.0039062500000000,
+0.0009460449218750, 0.0047302246093750, 0.0010375976562500, 0.0043334960937500,
+0.0007324218750000, 0.0039367675781250, 0.0008239746093750, 0.0051879882812500,
+0.0009155273437500, 0.0037231445312500, 0.0007934570312500, 0.0045776367187500,
+0.0007629394531250, 0.0046386718750000, 0.0008850097656250, 0.0039062500000000,
+0.0007019042968750, 0.0051879882812500, 0.0006408691406250, 0.0038452148437500,
+0.0005798339843750, 0.0047607421875000, 0.0006103515625000, 0.0045166015625000,
+0.0006103515625000, 0.0036621093750000, 0.0004272460937500, 0.0052795410156250,
+0.0008850097656250, 0.0032653808593750, 0.0006103515625000, 0.0044555664062500,
+0.0007934570312500, 0.0042419433593750, 0.0012207031250000, 0.0028381347656250,
+0.0007934570312500, 0.0046691894531250, 0.0014038085937500, 0.0021362304687500,
+0.0015563964843750, 0.0030822753906250, 0.0016479492187500, 0.0027770996093750,
+0.0020141601562500, 0.0008239746093750, 0.0020141601562500, 0.0023498535156250,
+0.0021972656250000, 0.0005187988281250, 0.0021972656250000, 0.0005798339843750,
+0.0022277832031250, 0.0005798339843750, 0.0022277832031250, -0.0004577636718750,
+0.0020751953125000, -0.0001525878906250, 0.0026550292968750, -0.0005798339843750,
+0.0023498535156250, -0.0006408691406250, 0.0025024414062500, -0.0004577636718750,
+0.0030517578125000, -0.0003356933593750, 0.0024719238281250, -0.0001220703125000,
+0.0025329589843750, 0.0000305175781250, 0.0025329589843750, 0.0006713867187500,
+0.0020751953125000, 0.0003967285156250, 0.0015563964843750, 0.0003356933593750,
+0.0014953613281250, 0.0012207031250000, 0.0013427734375000, -0.0009765625000000,
+0.0009155273437500, 0.0001220703125000, 0.0008239746093750, -0.0007934570312500,
+0.0010375976562500, -0.0029907226562500, 0.0005798339843750, -0.0012512207031250,
+-0.0000305175781250, -0.0038757324218750, -0.0001831054687500, -0.0040588378906250,
+-0.0003662109375000, -0.0036315917968750, -0.0010681152343750, -0.0046386718750000,
+-0.0013732910156250, -0.0043334960937500, -0.0004882812500000, -0.0032043457031250,
+-0.0009765625000000, -0.0040588378906250, -0.0008544921875000, -0.0026550292968750,
+0.0000915527343750, -0.0010681152343750, -0.0004272460937500, -0.0025939941406250,
+-0.0003356933593750, 0.0007324218750000, 0.0001525878906250, 0.0002441406250000,
+-0.0001220703125000, 0.0001831054687500, -0.0004882812500000, 0.0021667480468750,
+-0.0001220703125000, 0.0011596679687500, -0.0004882812500000, 0.0009765625000000,
+-0.0002746582031250, 0.0009155273437500, -0.0004272460937500, 0.0000305175781250,
+-0.0006408691406250, -0.0006408691406250, -0.0001220703125000, -0.0006713867187500,
+-0.0013427734375000, -0.0018005371093750, -0.0010070800781250, -0.0019531250000000,
+-0.0011901855468750, -0.0022583007812500, -0.0023498535156250, -0.0030212402343750,
+-0.0018310546875000, -0.0028381347656250, -0.0017700195312500, -0.0023193359375000,
+-0.0024719238281250, -0.0029907226562500, -0.0014343261718750, -0.0026245117187500,
+-0.0004272460937500, -0.0018615722656250, -0.0014953613281250, -0.0026855468750000,
+0.0007629394531250, -0.0018005371093750, 0.0005493164062500, -0.0018310546875000,
+0.0003967285156250, -0.0020141601562500, 0.0015563964843750, -0.0020141601562500,
+0.0010375976562500, -0.0024414062500000, 0.0014343261718750, -0.0015258789062500,
+0.0012512207031250, -0.0027160644531250, 0.0011901855468750, -0.0019531250000000,
+0.0013732910156250, -0.0006103515625000, 0.0009155273437500, -0.0025024414062500,
+0.0012512207031250, -0.0004272460937500, 0.0010681152343750, -0.0005798339843750,
+0.0006103515625000, -0.0016479492187500, 0.0006103515625000, -0.0014648437500000,
+0.0004882812500000, -0.0018310546875000, -0.0003356933593750, -0.0030822753906250,
+-0.0004577636718750, -0.0041809082031250, -0.0003662109375000, -0.0036315917968750,
+-0.0007019042968750, -0.0025634765625000, -0.0007629394531250, -0.0018920898437500,
+-0.0029602050781250, -0.0037841796875000, -0.0025024414062500, -0.0046081542968750,
+-0.0019226074218750, -0.0037231445312500, -0.0024719238281250, -0.0031127929687500,
+-0.0026245117187500, -0.0039367675781250, -0.0003967285156250, -0.0024719238281250,
+-0.0011291503906250, -0.0020751953125000, -0.0012207031250000, -0.0027770996093750,
+0.0005493164062500, -0.0027770996093750, -0.0002136230468750, -0.0022277832031250,
+-0.0018920898437500, -0.0031433105468750, -0.0010070800781250, -0.0034484863281250,
+-0.0025939941406250, -0.0025634765625000, -0.0050964355468750, -0.0021667480468750,
+-0.0035705566406250, -0.0026550292968750, -0.0052490234375000, -0.0011596679687500,
+-0.0059509277343750, -0.0008544921875000, -0.0044250488281250, -0.0015869140625000,
+-0.0037231445312500, -0.0012817382812500, -0.0043640136718750, -0.0007019042968750,
+-0.0022277832031250, -0.0027160644531250, -0.0016784667968750, -0.0025939941406250,
+-0.0021972656250000, -0.0021362304687500, -0.0019836425781250, -0.0029296875000000,
+-0.0018920898437500, -0.0027160644531250, -0.0023193359375000, -0.0021972656250000,
+-0.0029602050781250, -0.0020446777343750, -0.0026245117187500, -0.0022583007812500,
+-0.0025634765625000, -0.0021667480468750, -0.0034179687500000, -0.0020751953125000,
+-0.0010986328125000, -0.0029296875000000, -0.0018310546875000, -0.0028076171875000,
+-0.0012512207031250, -0.0025939941406250, 0.0011291503906250, -0.0030822753906250,
+0.0002136230468750, -0.0029296875000000, 0.0001831054687500, -0.0021057128906250,
+0.0018615722656250, -0.0021362304687500, 0.0003662109375000, -0.0018310546875000,
+-0.0013427734375000, -0.0010986328125000, 0.0006103515625000, -0.0011596679687500,
+-0.0030822753906250, -0.0008544921875000, -0.0024414062500000, -0.0006408691406250,
+-0.0025634765625000, -0.0005187988281250, -0.0050048828125000, -0.0002136230468750,
+-0.0037536621093750, -0.0001220703125000, -0.0036315917968750, -0.0002136230468750,
+-0.0046997070312500, 0.0000610351562500, -0.0029907226562500, -0.0000915527343750,
+-0.0013122558593750, -0.0003356933593750, -0.0027160644531250, 0.0000305175781250,
+0.0005187988281250, -0.0003662109375000, 0.0005798339843750, -0.0005798339843750,
+0.0005187988281250, -0.0001525878906250, 0.0020751953125000, 0.0000305175781250,
+0.0014953613281250, -0.0001525878906250, 0.0017089843750000, 0.0007324218750000,
+0.0021057128906250, 0.0008239746093750, 0.0014343261718750, 0.0008850097656250,
+0.0010070800781250, 0.0013427734375000, 0.0014343261718750, 0.0012817382812500,
+-0.0001525878906250, 0.0014343261718750, 0.0000305175781250, 0.0014343261718750,
+-0.0001220703125000, 0.0015869140625000, -0.0012207031250000, 0.0017700195312500,
+-0.0008239746093750, 0.0016784667968750, -0.0007324218750000, 0.0021972656250000,
+-0.0011901855468750, 0.0020446777343750, -0.0003967285156250, 0.0021972656250000,
+0.0002441406250000, 0.0026245117187500, -0.0005187988281250, 0.0024108886718750,
+0.0013732910156250, 0.0028381347656250, 0.0008544921875000, 0.0028076171875000,
+0.0013732910156250, 0.0027465820312500, 0.0026855468750000, 0.0029602050781250,
+0.0016174316406250, 0.0029296875000000, 0.0035095214843750, 0.0027770996093750,
+0.0029602050781250, 0.0028686523437500, 0.0029602050781250, 0.0026855468750000,
+0.0041809082031250, 0.0023498535156250, 0.0030212402343750, 0.0025024414062500,
+0.0034484863281250, 0.0021057128906250, 0.0035400390625000, 0.0021362304687500,
+0.0034484863281250, 0.0019836425781250, 0.0036010742187500, 0.0017089843750000,
+0.0036315917968750, 0.0018615722656250, 0.0038452148437500, 0.0013427734375000,
+0.0038452148437500, 0.0014038085937500, 0.0035705566406250, 0.0011901855468750,
+0.0036926269531250, 0.0007629394531250, 0.0041198730468750, 0.0009460449218750,
+0.0033569335937500, 0.0003356933593750, 0.0037536621093750, 0.0003356933593750,
+0.0036315917968750, 0.0001831054687500, 0.0030822753906250, -0.0002441406250000,
+0.0036315917968750, -0.0000610351562500, 0.0029296875000000, -0.0004577636718750,
+0.0031127929687500, -0.0006103515625000, 0.0030822753906250, -0.0003967285156250,
+0.0026550292968750, -0.0004577636718750, 0.0031433105468750, -0.0007324218750000,
+0.0025939941406250, 0.0000610351562500, 0.0025329589843750, -0.0002746582031250,
+0.0025939941406250, 0.0000000000000000, 0.0022888183593750, 0.0006103515625000,
+0.0024414062500000, 0.0001525878906250, 0.0023803710937500, 0.0006103515625000,
+0.0023193359375000, 0.0006408691406250, 0.0024414062500000, 0.0005798339843750,
+0.0025024414062500, 0.0007629394531250, 0.0024108886718750, 0.0005798339843750,
+0.0031433105468750, 0.0006408691406250, 0.0026550292968750, 0.0007324218750000,
+0.0028991699218750, 0.0005187988281250, 0.0034790039062500, 0.0004272460937500,
+0.0025939941406250, 0.0005493164062500, 0.0035400390625000, 0.0006103515625000,
+0.0029296875000000, 0.0003356933593750, 0.0025024414062500, 0.0005187988281250,
+0.0028686523437500, 0.0008239746093750, 0.0021667480468750, 0.0005493164062500,
+0.0021057128906250, 0.0003967285156250, 0.0021667480468750, 0.0006103515625000,
+0.0018920898437500, 0.0002746582031250, 0.0018005371093750, -0.0001831054687500,
+0.0021972656250000, 0.0001220703125000, 0.0017700195312500, -0.0005493164062500,
+0.0020446777343750, -0.0006408691406250, 0.0024108886718750, -0.0004272460937500,
+0.0022888183593750, -0.0006103515625000, 0.0024719238281250, -0.0007934570312500,
+0.0031127929687500, -0.0003356933593750, 0.0031127929687500, -0.0003662109375000,
+0.0031738281250000, -0.0004577636718750, 0.0036010742187500, -0.0002746582031250,
+0.0032958984375000, -0.0003662109375000, 0.0023803710937500, -0.0003662109375000,
+0.0028381347656250},
+{-0.0014343261718750, 0.0000305175781250, -0.0013122558593750, -0.0001525878906250,
+-0.0007324218750000, 0.0003967285156250, -0.0006103515625000, 0.0006408691406250,
+-0.0003051757812500, 0.0004882812500000, 0.0008544921875000, 0.0012817382812500,
+0.0009460449218750, 0.0013427734375000, 0.0014648437500000, 0.0014038085937500,
+0.0023803710937500, 0.0018005371093750, 0.0023498535156250, 0.0017395019531250,
+0.0028686523437500, 0.0021057128906250, 0.0031433105468750, 0.0021057128906250,
+0.0028991699218750, 0.0022277832031250, 0.0028381347656250, 0.0025329589843750,
+0.0029296875000000, 0.0024719238281250, 0.0021362304687500, 0.0025024414062500,
+0.0021972656250000, 0.0025939941406250, 0.0018310546875000, 0.0025939941406250,
+0.0011291503906250, 0.0025634765625000, 0.0015258789062500, 0.0025634765625000,
+0.0007934570312500, 0.0027770996093750, 0.0005798339843750, 0.0027770996093750,
+0.0003662109375000, 0.0026855468750000, -0.0000610351562500, 0.0027770996093750,
+-0.0000610351562500, 0.0027770996093750, -0.0001525878906250, 0.0025024414062500,
+-0.0003967285156250, 0.0026245117187500, -0.0002746582031250, 0.0023498535156250,
+-0.0002136230468750, 0.0020446777343750, -0.0004577636718750, 0.0021972656250000,
+-0.0003051757812500, 0.0016784667968750, -0.0001220703125000, 0.0017089843750000,
+-0.0000915527343750, 0.0015563964843750, -0.0001220703125000, 0.0010986328125000,
+-0.0000305175781250, 0.0012207031250000, 0.0000305175781250, 0.0007934570312500,
+-0.0002136230468750, 0.0006408691406250, 0.0000305175781250, 0.0007324218750000,
+-0.0000305175781250, 0.0005493164062500, -0.0007934570312500, 0.0004272460937500,
+0.0001831054687500, 0.0008544921875000, -0.0003356933593750, 0.0006713867187500,
+-0.0003356933593750, 0.0008239746093750, 0.0004577636718750, 0.0011596679687500,
+-0.0001831054687500, 0.0008850097656250, 0.0001831054687500, 0.0011596679687500,
+0.0002746582031250, 0.0011901855468750, 0.0000915527343750, 0.0010681152343750,
+0.0000915527343750, 0.0011901855468750, 0.0000305175781250, 0.0011596679687500,
+-0.0000610351562500, 0.0010986328125000, -0.0002746582031250, 0.0010986328125000,
+-0.0001220703125000, 0.0009460449218750, 0.0000305175781250, 0.0008544921875000,
+-0.0000915527343750, 0.0009155273437500, 0.0002136230468750, 0.0006103515625000,
+0.0001831054687500, 0.0005493164062500, 0.0005798339843750, 0.0007019042968750,
+0.0008850097656250, 0.0006713867187500, 0.0005798339843750, 0.0004882812500000,
+0.0018310546875000, 0.0006408691406250, 0.0011901855468750, 0.0007019042968750,
+0.0017700195312500, 0.0003662109375000, 0.0028991699218750, 0.0001525878906250,
+0.0016784667968750, 0.0003051757812500, 0.0029602050781250, -0.0003356933593750,
+0.0028076171875000, -0.0003662109375000, 0.0023498535156250, -0.0004272460937500,
+0.0029296875000000, -0.0007629394531250, 0.0028686523437500, -0.0007629394531250,
+0.0022888183593750, -0.0004882812500000, 0.0028381347656250, -0.0007019042968750,
+0.0024719238281250, -0.0007324218750000, 0.0018310546875000, -0.0004272460937500,
+0.0026855468750000, -0.0004272460937500, 0.0013732910156250, -0.0007324218750000,
+0.0018005371093750, -0.0006713867187500, 0.0018920898437500, -0.0006408691406250,
+0.0011291503906250, -0.0007019042968750, 0.0018615722656250, -0.0006713867187500,
+0.0017700195312500, -0.0010986328125000, 0.0016479492187500, -0.0009460449218750,
+0.0018005371093750, -0.0010070800781250, 0.0018920898437500, -0.0015258789062500,
+0.0019531250000000, -0.0013732910156250, 0.0017700195312500, -0.0012817382812500,
+0.0022277832031250, -0.0014648437500000, 0.0017700195312500, -0.0010681152343750,
+0.0015258789062500, -0.0007324218750000, 0.0024108886718750, -0.0011596679687500,
+0.0003967285156250, -0.0003662109375000, 0.0012207031250000, -0.0002136230468750,
+0.0006713867187500, -0.0006713867187500, -0.0011901855468750, -0.0007324218750000,
+0.0000610351562500, -0.0004577636718750, -0.0017395019531250, -0.0012817382812500,
+-0.0017700195312500, -0.0011596679687500, -0.0019531250000000, -0.0011291503906250,
+-0.0029602050781250, -0.0016174316406250, -0.0025939941406250, -0.0013427734375000,
+-0.0036926269531250, -0.0008850097656250, -0.0034790039062500, -0.0009765625000000,
+-0.0039978027343750, -0.0008239746093750, -0.0049133300781250, -0.0003051757812500,
+-0.0040893554687500, -0.0003967285156250, -0.0052795410156250, -0.0007629394531250,
+-0.0049438476562500, -0.0006103515625000, -0.0043640136718750, -0.0006713867187500,
+-0.0053100585937500, -0.0010375976562500, -0.0054016113281250, -0.0008544921875000,
+-0.0032348632812500, -0.0002746582031250, -0.0040893554687500, -0.0007629394531250,
+-0.0034484863281250, -0.0005493164062500, -0.0012207031250000, 0.0001220703125000,
+-0.0023498535156250, -0.0003051757812500, -0.0017089843750000, 0.0001831054687500,
+-0.0006408691406250, 0.0002746582031250, -0.0018310546875000, -0.0010070800781250,
+-0.0028381347656250, -0.0014953613281250, -0.0016784667968750, -0.0007019042968750,
+-0.0033874511718750, -0.0026855468750000, -0.0035095214843750, -0.0028686523437500,
+-0.0034179687500000, -0.0031127929687500, -0.0032958984375000, -0.0041198730468750,
+-0.0021667480468750, -0.0036621093750000, -0.0036621093750000, -0.0038146972656250,
+-0.0028991699218750, -0.0038757324218750, -0.0027465820312500, -0.0034484863281250,
+-0.0040283203125000, -0.0033569335937500, -0.0031433105468750, -0.0035400390625000,
+-0.0023193359375000, -0.0028076171875000, -0.0029296875000000, -0.0023498535156250,
+-0.0019836425781250, -0.0025634765625000, -0.0006408691406250, -0.0020141601562500,
+-0.0016174316406250, -0.0010375976562500, 0.0000305175781250, -0.0028076171875000,
+-0.0002136230468750, -0.0019226074218750, -0.0000915527343750, -0.0018615722656250,
+0.0006408691406250, -0.0033874511718750, -0.0002441406250000, -0.0027465820312500,
+0.0011901855468750, -0.0030822753906250, 0.0003662109375000, -0.0038757324218750,
+0.0006103515625000, -0.0037231445312500, 0.0018920898437500, -0.0039062500000000,
+0.0007019042968750, -0.0052795410156250, 0.0011291503906250, -0.0042419433593750,
+0.0012817382812500, -0.0053100585937500, 0.0006103515625000, -0.0052795410156250,
+0.0003356933593750, -0.0046081542968750, 0.0004272460937500, -0.0059204101562500,
+-0.0005187988281250, -0.0035095214843750, -0.0005493164062500, -0.0044555664062500,
+-0.0008544921875000, -0.0038452148437500, -0.0014648437500000, -0.0018005371093750,
+-0.0011291503906250, -0.0034179687500000, -0.0016174316406250, -0.0016479492187500,
+-0.0018920898437500, -0.0020446777343750, -0.0015869140625000, -0.0021972656250000,
+-0.0014648437500000, -0.0015869140625000, -0.0016479492187500, -0.0026550292968750,
+-0.0008544921875000, -0.0013427734375000, -0.0007019042968750, -0.0020446777343750,
+-0.0009460449218750, -0.0017395019531250, -0.0006713867187500, -0.0005493164062500,
+-0.0004882812500000, -0.0014648437500000, -0.0011901855468750, 0.0004272460937500,
+-0.0007324218750000, 0.0007934570312500, -0.0010070800781250, 0.0008850097656250,
+-0.0018615722656250, 0.0020751953125000, -0.0012207031250000, 0.0023803710937500,
+-0.0012817382812500, 0.0015258789062500, -0.0013122558593750, 0.0026245117187500,
+-0.0011291503906250, 0.0021362304687500, -0.0009155273437500, 0.0010070800781250,
+-0.0006713867187500, 0.0024719238281250, -0.0007629394531250, 0.0007934570312500,
+-0.0003356933593750, 0.0014953613281250, -0.0005187988281250, 0.0017089843750000,
+-0.0007934570312500, 0.0008544921875000, -0.0001220703125000, 0.0020141601562500,
+-0.0010681152343750, 0.0012512207031250, -0.0009460449218750, 0.0018310546875000,
+-0.0005798339843750, 0.0015869140625000, -0.0009155273437500, 0.0009155273437500,
+-0.0008239746093750, 0.0020141601562500, -0.0004272460937500, 0.0003662109375000,
+-0.0006103515625000, 0.0010986328125000, -0.0004272460937500, 0.0007324218750000,
+-0.0001220703125000, -0.0005493164062500, -0.0004882812500000, 0.0009765625000000,
+0.0001220703125000, -0.0005493164062500, -0.0000610351562500, -0.0001831054687500,
+0.0000000000000000, -0.0001831054687500, 0.0005187988281250, -0.0009155273437500,
+0.0003967285156250, 0.0001220703125000, 0.0005493164062500, -0.0006408691406250,
+0.0008239746093750, -0.0001525878906250, 0.0008239746093750, 0.0003967285156250,
+0.0007019042968750, 0.0002746582031250, 0.0008850097656250, 0.0007934570312500,
+0.0011901855468750, 0.0007934570312500, 0.0008544921875000, 0.0012512207031250,
+0.0012512207031250, 0.0013427734375000, 0.0018310546875000, 0.0013427734375000,
+0.0012207031250000, 0.0020141601562500, 0.0021667480468750, 0.0014648437500000,
+0.0020446777343750, 0.0022583007812500, 0.0020446777343750, 0.0022583007812500,
+0.0025939941406250, 0.0018005371093750, 0.0022583007812500, 0.0026855468750000,
+0.0025024414062500, 0.0018920898437500, 0.0025939941406250, 0.0024108886718750,
+0.0024414062500000, 0.0026245117187500, 0.0025024414062500, 0.0021972656250000,
+0.0025024414062500, 0.0028076171875000, 0.0023498535156250, 0.0026550292968750,
+0.0024719238281250, 0.0024719238281250, 0.0024108886718750, 0.0026855468750000,
+0.0022888183593750, 0.0026550292968750, 0.0024719238281250, 0.0021972656250000,
+0.0021972656250000, 0.0025939941406250, 0.0023498535156250, 0.0023498535156250,
+0.0022277832031250, 0.0021362304687500, 0.0020446777343750, 0.0022277832031250,
+0.0023193359375000, 0.0017700195312500, 0.0018005371093750, 0.0016479492187500,
+0.0019836425781250, 0.0014648437500000, 0.0018005371093750, 0.0010681152343750,
+0.0013122558593750, 0.0008850097656250, 0.0015258789062500, 0.0007324218750000,
+0.0010070800781250, 0.0001220703125000, 0.0007934570312500, 0.0004272460937500,
+0.0009155273437500, -0.0000610351562500, 0.0007019042968750, -0.0004882812500000,
+0.0004272460937500, 0.0003967285156250, 0.0009155273437500, -0.0006408691406250,
+0.0009460449218750, 0.0000915527343750, 0.0007629394531250, 0.0003356933593750,
+0.0010375976562500, -0.0001831054687500, 0.0012817382812500, 0.0009155273437500,
+0.0006103515625000, 0.0003356933593750, 0.0008544921875000, 0.0010070800781250,
+0.0009155273437500, 0.0013427734375000, 0.0004577636718750, 0.0011901855468750,
+0.0006408691406250, 0.0020751953125000, 0.0007324218750000, 0.0010986328125000,
+0.0007934570312500, 0.0020446777343750, 0.0005493164062500, 0.0014343261718750,
+0.0004577636718750, 0.0004882812500000, 0.0006713867187500, 0.0021972656250000,
+0.0000000000000000, -0.0008850097656250, 0.0000305175781250, 0.0005187988281250,
+0.0000305175781250, -0.0001220703125000, -0.0003051757812500, -0.0024414062500000,
+-0.0001525878906250, -0.0000915527343750, -0.0003356933593750, -0.0029602050781250,
+-0.0000305175781250, -0.0022583007812500, -0.0002746582031250, -0.0017089843750000,
+-0.0006408691406250, -0.0029602050781250, -0.0001831054687500, -0.0016784667968750,
+-0.0010375976562500, -0.0016174316406250, -0.0010375976562500, -0.0015563964843750,
+-0.0009460449218750, -0.0008850097656250, -0.0012817382812500, -0.0001831054687500,
+-0.0012207031250000, 0.0001220703125000, -0.0014648437500000, 0.0000915527343750,
+-0.0012817382812500, 0.0004882812500000, -0.0013427734375000, 0.0000000000000000,
+-0.0018920898437500, -0.0007629394531250, -0.0018005371093750, -0.0004882812500000,
+-0.0013427734375000, -0.0014648437500000, -0.0019531250000000, -0.0024719238281250,
+-0.0013732910156250, -0.0020141601562500, -0.0003967285156250, -0.0021972656250000,
+-0.0011596679687500, -0.0036010742187500, -0.0002441406250000, -0.0013732910156250,
+-0.0001220703125000, -0.0025939941406250, -0.0006713867187500, -0.0021362304687500,
+-0.0006713867187500},
+{0.0005187988281250, 0.0032348632812500, 0.0011291503906250, 0.0033874511718750,
+0.0007934570312500, 0.0034179687500000, 0.0012817382812500, 0.0034484863281250,
+0.0010070800781250, 0.0031738281250000, 0.0011901855468750, 0.0031127929687500,
+0.0015258789062500, 0.0031433105468750, 0.0010375976562500, 0.0031127929687500,
+0.0021362304687500, 0.0032653808593750, 0.0016784667968750, 0.0030822753906250,
+0.0019531250000000, 0.0029602050781250, 0.0028991699218750, 0.0028686523437500,
+0.0021667480468750, 0.0025329589843750, 0.0029907226562500, 0.0026855468750000,
+0.0029602050781250, 0.0019836425781250, 0.0028381347656250, 0.0022888183593750,
+0.0032653808593750, 0.0026550292968750, 0.0031127929687500, 0.0015869140625000,
+0.0029602050781250, 0.0028686523437500, 0.0031738281250000, 0.0021667480468750,
+0.0029602050781250, 0.0021972656250000, 0.0026855468750000, 0.0030212402343750,
+0.0029602050781250, 0.0019531250000000, 0.0024108886718750, 0.0029602050781250,
+0.0024719238281250, 0.0023803710937500, 0.0025024414062500, 0.0025939941406250,
+0.0022583007812500, 0.0032958984375000, 0.0025329589843750, 0.0021057128906250,
+0.0021972656250000, 0.0037841796875000, 0.0022888183593750, 0.0028991699218750,
+0.0021362304687500, 0.0029296875000000, 0.0018615722656250, 0.0040893554687500,
+0.0020751953125000, 0.0027160644531250, 0.0014953613281250, 0.0035705566406250,
+0.0017395019531250, 0.0036010742187500, 0.0015258789062500, 0.0031433105468750,
+0.0010375976562500, 0.0036010742187500, 0.0013122558593750, 0.0038452148437500,
+0.0006103515625000, 0.0027770996093750, 0.0004577636718750, 0.0039978027343750,
+0.0004272460937500, 0.0034484863281250, 0.0000915527343750, 0.0025024414062500,
+0.0001220703125000, 0.0044250488281250, 0.0002746582031250, 0.0017700195312500,
+0.0004272460937500, 0.0027465820312500, 0.0004272460937500, 0.0025939941406250,
+0.0005493164062500, 0.0007934570312500, 0.0007934570312500, 0.0024108886718750,
+0.0003051757812500, 0.0007019042968750, 0.0004577636718750, 0.0008850097656250,
+0.0003662109375000, 0.0011901855468750, -0.0000305175781250, 0.0003662109375000,
+0.0002136230468750, 0.0007324218750000, -0.0000915527343750, 0.0005493164062500,
+0.0000610351562500, 0.0007324218750000, -0.0000305175781250, 0.0007019042968750,
+-0.0002136230468750, 0.0006713867187500, 0.0001525878906250, 0.0010986328125000,
+-0.0006713867187500, -0.0000915527343750, -0.0003662109375000, 0.0008239746093750,
+-0.0004577636718750, 0.0000610351562500, -0.0010375976562500, -0.0013122558593750,
+-0.0007019042968750, 0.0000610351562500, -0.0012512207031250, -0.0024108886718750,
+-0.0011901855468750, -0.0019531250000000, -0.0014343261718750, -0.0018005371093750,
+-0.0019226074218750, -0.0033264160156250, -0.0015258789062500, -0.0025024414062500,
+-0.0020446777343750, -0.0022277832031250, -0.0020141601562500, -0.0026855468750000,
+-0.0018005371093750, -0.0017395019531250, -0.0018310546875000, -0.0003356933593750,
+-0.0017700195312500, -0.0006103515625000, -0.0016174316406250, 0.0006713867187500,
+-0.0013122558593750, 0.0013122558593750, -0.0014953613281250, 0.0010681152343750,
+-0.0016174316406250, 0.0011901855468750, -0.0010986328125000, 0.0014038085937500,
+-0.0016784667968750, 0.0004272460937500, -0.0013732910156250, -0.0000915527343750,
+-0.0010070800781250, -0.0005798339843750, -0.0010375976562500, -0.0014038085937500,
+-0.0007324218750000, -0.0018005371093750, -0.0006408691406250, -0.0023193359375000,
+0.0000915527343750, -0.0032348632812500, -0.0002746582031250, -0.0030517578125000,
+-0.0008239746093750, -0.0029296875000000, 0.0000610351562500, -0.0038757324218750,
+-0.0015563964843750, -0.0016784667968750, -0.0009155273437500, -0.0018005371093750,
+-0.0013122558593750, -0.0013427734375000, -0.0025939941406250, 0.0003662109375000,
+-0.0013732910156250, -0.0000305175781250, -0.0024108886718750, -0.0006408691406250,
+-0.0022888183593750, 0.0001525878906250, -0.0018005371093750, -0.0008544921875000,
+-0.0021667480468750, -0.0027770996093750, -0.0019531250000000, -0.0022277832031250,
+-0.0014038085937500, -0.0025329589843750, -0.0018005371093750, -0.0038757324218750,
+-0.0013427734375000, -0.0026855468750000, -0.0008850097656250, -0.0011901855468750,
+-0.0016174316406250, -0.0027160644531250, 0.0001525878906250, -0.0010070800781250,
+-0.0004577636718750, -0.0005187988281250, -0.0007934570312500, -0.0014953613281250,
+0.0009155273437500, -0.0018615722656250, -0.0023193359375000, -0.0010986328125000,
+-0.0030822753906250, -0.0003051757812500, -0.0030517578125000, -0.0000305175781250,
+-0.0020751953125000, -0.0010986328125000, -0.0015869140625000, -0.0018005371093750,
+-0.0018615722656250, -0.0010070800781250, -0.0003967285156250, -0.0020751953125000,
+-0.0001220703125000, -0.0024108886718750, -0.0003051757812500, -0.0017395019531250,
+0.0000915527343750, -0.0013732910156250, 0.0000915527343750, -0.0016479492187500,
+-0.0000610351562500, -0.0018005371093750, 0.0000000000000000, -0.0011291503906250,
+0.0000915527343750, -0.0018615722656250, 0.0002746582031250, -0.0028076171875000,
+0.0003967285156250, -0.0019226074218750, 0.0000000000000000, -0.0029602050781250,
+0.0007324218750000, -0.0032653808593750, -0.0001831054687500, -0.0024108886718750,
+-0.0010375976562500, -0.0021362304687500, 0.0003356933593750, -0.0025634765625000,
+-0.0031433105468750, -0.0018005371093750, -0.0026245117187500, -0.0014953613281250,
+-0.0023193359375000, -0.0018615722656250, -0.0044555664062500, -0.0020751953125000,
+-0.0036621093750000, -0.0019836425781250, -0.0024108886718750, -0.0023498535156250,
+-0.0032348632812500, -0.0025024414062500, -0.0019226074218750, -0.0022277832031250,
+0.0000915527343750, -0.0020751953125000, -0.0009155273437500, -0.0023498535156250,
+0.0013427734375000, -0.0019531250000000, 0.0019836425781250, -0.0015563964843750,
+0.0011901855468750, -0.0022583007812500, 0.0016479492187500, -0.0028076171875000,
+0.0022277832031250, -0.0020751953125000, 0.0000000000000000, -0.0031127929687500,
+0.0001831054687500, -0.0034484863281250, 0.0001831054687500, -0.0029602050781250,
+-0.0010070800781250, -0.0027770996093750, -0.0004882812500000, -0.0029296875000000,
+-0.0002136230468750, -0.0026550292968750, -0.0007629394531250, -0.0024414062500000,
+0.0002441406250000, -0.0024719238281250, 0.0012207031250000, -0.0026245117187500,
+0.0003356933593750, -0.0025939941406250, 0.0020751953125000, -0.0019226074218750,
+0.0022888183593750, -0.0023498535156250, 0.0018005371093750, -0.0018005371093750,
+0.0024719238281250, -0.0008544921875000, 0.0027770996093750, -0.0015258789062500,
+0.0013732910156250, -0.0006103515625000, 0.0020141601562500, -0.0004882812500000,
+0.0017395019531250, -0.0007629394531250, 0.0005798339843750, -0.0006713867187500,
+0.0014953613281250, -0.0007019042968750, 0.0007019042968750, -0.0004882812500000,
+0.0007934570312500, -0.0007934570312500, 0.0009460449218750, -0.0003356933593750,
+0.0005798339843750, 0.0003051757812500, 0.0009765625000000, -0.0001525878906250,
+0.0014343261718750, 0.0006103515625000, 0.0011596679687500, 0.0007324218750000,
+0.0018615722656250, 0.0006713867187500, 0.0026245117187500, 0.0008850097656250,
+0.0020446777343750, 0.0009765625000000, 0.0030822753906250, 0.0011596679687500,
+0.0032958984375000, 0.0010986328125000, 0.0032043457031250, 0.0012512207031250,
+0.0036926269531250, 0.0015563964843750, 0.0037841796875000, 0.0015563964843750,
+0.0033264160156250, 0.0016174316406250, 0.0039367675781250, 0.0017700195312500,
+0.0033874511718750, 0.0018310546875000, 0.0026245117187500, 0.0017700195312500,
+0.0035705566406250, 0.0019531250000000, 0.0020446777343750, 0.0018005371093750,
+0.0026550292968750, 0.0019836425781250, 0.0020751953125000, 0.0019836425781250,
+0.0008850097656250, 0.0018615722656250, 0.0021667480468750, 0.0020751953125000,
+-0.0003356933593750, 0.0017395019531250, 0.0008544921875000, 0.0017700195312500,
+0.0004272460937500, 0.0019226074218750, -0.0013732910156250, 0.0017395019531250,
+0.0007019042968750, 0.0018005371093750, -0.0016174316406250, 0.0020141601562500,
+-0.0012817382812500, 0.0020141601562500, -0.0010070800781250, 0.0020751953125000,
+-0.0022277832031250, 0.0023193359375000, -0.0014953613281250, 0.0022888183593750,
+-0.0019836425781250, 0.0024108886718750, -0.0016479492187500, 0.0024414062500000,
+-0.0015563964843750, 0.0025329589843750, -0.0016174316406250, 0.0026245117187500,
+-0.0007324218750000, 0.0026550292968750, -0.0015563964843750, 0.0027770996093750,
+-0.0009765625000000, 0.0028381347656250, -0.0007324218750000, 0.0027160644531250,
+-0.0013427734375000, 0.0027160644531250, -0.0008544921875000, 0.0028076171875000,
+-0.0006713867187500, 0.0026245117187500, -0.0008544921875000, 0.0026550292968750,
+-0.0006103515625000, 0.0025024414062500, 0.0000000000000000, 0.0022888183593750,
+0.0002441406250000, 0.0024719238281250, 0.0002136230468750, 0.0018615722656250,
+0.0012207031250000, 0.0020446777343750, 0.0012817382812500, 0.0017700195312500,
+0.0010986328125000, 0.0012817382812500, 0.0023803710937500, 0.0016174316406250,
+0.0019836425781250, 0.0007324218750000, 0.0023193359375000, 0.0009155273437500,
+0.0025939941406250, 0.0007324218750000, 0.0024108886718750, 0.0000610351562500,
+0.0029907226562500, 0.0004272460937500, 0.0030517578125000, -0.0003051757812500,
+0.0034179687500000, -0.0003051757812500, 0.0034484863281250, -0.0004272460937500,
+0.0033874511718750, -0.0009460449218750, 0.0037536621093750, -0.0008239746093750,
+0.0035705566406250, -0.0011291503906250, 0.0035095214843750, -0.0011596679687500,
+0.0034484863281250, -0.0013427734375000, 0.0032043457031250, -0.0016479492187500,
+0.0030517578125000, -0.0012817382812500, 0.0024719238281250, -0.0014038085937500,
+0.0024719238281250, -0.0017089843750000, 0.0017395019531250, -0.0012817382812500,
+0.0012817382812500, -0.0007934570312500, 0.0018615722656250, -0.0010986328125000,
+0.0001525878906250, -0.0009460449218750, 0.0006713867187500, -0.0007324218750000,
+0.0006103515625000, -0.0007019042968750, -0.0004272460937500, -0.0008544921875000,
+0.0006408691406250, -0.0009155273437500, 0.0000305175781250, -0.0001220703125000,
+0.0002746582031250, -0.0002441406250000, 0.0007629394531250, -0.0000915527343750,
+0.0007019042968750, 0.0005493164062500, 0.0009155273437500, 0.0004272460937500,
+0.0012817382812500, 0.0002136230468750, 0.0012512207031250, 0.0005798339843750,
+0.0010070800781250, 0.0000305175781250, 0.0008850097656250, -0.0007934570312500,
+0.0007019042968750, -0.0002441406250000, -0.0001831054687500, -0.0008850097656250,
+-0.0002746582031250, -0.0014343261718750, -0.0004272460937500, -0.0010070800781250,
+-0.0010375976562500, -0.0008544921875000, -0.0010375976562500, -0.0013732910156250,
+-0.0003662109375000, -0.0003356933593750, -0.0010375976562500, -0.0005187988281250,
+-0.0000305175781250, -0.0006408691406250, 0.0013122558593750, -0.0001831054687500,
+0.0000305175781250, -0.0004272460937500, 0.0016174316406250, -0.0001220703125000,
+0.0016174316406250, -0.0001525878906250, 0.0007019042968750, -0.0002136230468750,
+0.0007019042968750, 0.0000915527343750, 0.0003356933593750, 0.0001831054687500,
+-0.0013732910156250, -0.0006103515625000, -0.0011901855468750, -0.0001831054687500,
+-0.0020141601562500, -0.0006103515625000, -0.0031433105468750, -0.0013732910156250,
+-0.0018920898437500, -0.0007629394531250, -0.0033874511718750, -0.0020751953125000,
+-0.0035095214843750, -0.0017395019531250, -0.0032348632812500, -0.0023498535156250,
+-0.0036315917968750, -0.0032348632812500, -0.0033874511718750, -0.0018615722656250,
+-0.0033874511718750},
+{0.0020751953125000, 0.0023498535156250, 0.0015563964843750, 0.0026550292968750,
+0.0021057128906250, 0.0026855468750000, 0.0022583007812500, 0.0026245117187500,
+0.0019531250000000, 0.0027160644531250, 0.0025329589843750, 0.0027160644531250,
+0.0019531250000000, 0.0026245117187500, 0.0025329589843750, 0.0026855468750000,
+0.0025024414062500, 0.0026855468750000, 0.0021972656250000, 0.0025939941406250,
+0.0030822753906250, 0.0026550292968750, 0.0024719238281250, 0.0023803710937500,
+0.0024719238281250, 0.0025634765625000, 0.0027465820312500, 0.0022888183593750,
+0.0023498535156250, 0.0018310546875000, 0.0020141601562500, 0.0022277832031250,
+0.0023498535156250, 0.0015563964843750, 0.0022277832031250, 0.0015563964843750,
+0.0019226074218750, 0.0014343261718750, 0.0021972656250000, 0.0010375976562500,
+0.0026855468750000, 0.0013122558593750, 0.0021057128906250, 0.0004882812500000,
+0.0027465820312500, 0.0007629394531250, 0.0021667480468750, 0.0004882812500000,
+0.0012512207031250, -0.0001831054687500, 0.0022277832031250, 0.0002746582031250,
+0.0006713867187500, -0.0004577636718750, 0.0010375976562500, -0.0004272460937500,
+0.0011901855468750, -0.0004272460937500, 0.0003967285156250, -0.0007019042968750,
+0.0010375976562500, -0.0004272460937500, 0.0006103515625000, -0.0010986328125000,
+0.0006408691406250, -0.0009155273437500, 0.0012817382812500, -0.0007934570312500,
+0.0013427734375000, -0.0012512207031250, 0.0011596679687500, -0.0011291503906250,
+0.0025634765625000, -0.0009155273437500, 0.0018615722656250, -0.0008850097656250,
+0.0022277832031250, -0.0010375976562500, 0.0034790039062500, -0.0011596679687500,
+0.0025634765625000, -0.0010375976562500, 0.0033874511718750, -0.0007934570312500,
+0.0035400390625000, -0.0010986328125000, 0.0028381347656250, -0.0007629394531250,
+0.0029907226562500, -0.0001831054687500, 0.0033569335937500, -0.0005187988281250,
+0.0016784667968750, -0.0001831054687500, 0.0024414062500000, 0.0000305175781250,
+0.0019531250000000, -0.0000915527343750, 0.0005493164062500, -0.0001220703125000,
+0.0017089843750000, -0.0000610351562500, 0.0006408691406250, -0.0003967285156250,
+0.0003662109375000, -0.0003662109375000, 0.0009155273437500, -0.0002746582031250,
+0.0007934570312500, -0.0004577636718750, 0.0004577636718750, -0.0004577636718750,
+0.0010375976562500, -0.0000915527343750, 0.0007934570312500, -0.0004272460937500,
+0.0003967285156250, -0.0001525878906250, 0.0004882812500000, 0.0002746582031250,
+0.0003051757812500, -0.0001831054687500, -0.0001831054687500, 0.0004577636718750,
+-0.0006713867187500, 0.0003356933593750, -0.0001831054687500, 0.0002746582031250,
+-0.0003051757812500, 0.0007019042968750, -0.0013732910156250, 0.0004882812500000,
+0.0006408691406250, 0.0000610351562500, -0.0004882812500000, 0.0005187988281250,
+0.0001831054687500, -0.0001831054687500, 0.0019531250000000, -0.0010986328125000,
+0.0000305175781250, -0.0003356933593750, 0.0020751953125000, -0.0018005371093750,
+0.0014038085937500, -0.0015258789062500, 0.0003356933593750, -0.0016784667968750,
+0.0010681152343750, -0.0025024414062500, 0.0003356933593750, -0.0018920898437500,
+-0.0010070800781250, -0.0026855468750000, -0.0006103515625000, -0.0020751953125000,
+-0.0011291503906250, -0.0022277832031250, -0.0026245117187500, -0.0032348632812500,
+-0.0022888183593750, -0.0024719238281250, -0.0023193359375000, -0.0024719238281250,
+-0.0028076171875000, -0.0025329589843750, -0.0024719238281250, -0.0021972656250000,
+-0.0018920898437500, -0.0021362304687500, -0.0021972656250000, -0.0024719238281250,
+-0.0017089843750000, -0.0019531250000000, -0.0018005371093750, -0.0018920898437500,
+-0.0017089843750000, -0.0021667480468750, -0.0015869140625000, -0.0023803710937500,
+-0.0022277832031250, -0.0024414062500000, -0.0018615722656250, -0.0019226074218750,
+-0.0020446777343750, -0.0018615722656250, -0.0023803710937500, -0.0018920898437500,
+-0.0022277832031250, -0.0014038085937500, -0.0023498535156250, -0.0012207031250000,
+-0.0033264160156250, -0.0018615722656250, -0.0029296875000000, -0.0016479492187500,
+-0.0032653808593750, -0.0018920898437500, -0.0042114257812500, -0.0027465820312500,
+-0.0036010742187500, -0.0026855468750000, -0.0037536621093750, -0.0030517578125000,
+-0.0039062500000000, -0.0037231445312500, -0.0035400390625000, -0.0036621093750000,
+-0.0030212402343750, -0.0036010742187500, -0.0027770996093750, -0.0042724609375000,
+-0.0027770996093750, -0.0036621093750000, -0.0020446777343750, -0.0039367675781250,
+-0.0019531250000000, -0.0037841796875000, -0.0020446777343750, -0.0029907226562500,
+-0.0012512207031250, -0.0030822753906250, -0.0016784667968750, -0.0022583007812500,
+-0.0013427734375000, -0.0017395019531250, -0.0008850097656250, -0.0014648437500000,
+-0.0010986328125000, -0.0007934570312500, -0.0010681152343750, -0.0003662109375000,
+-0.0002746582031250, -0.0008544921875000, -0.0008850097656250, 0.0000000000000000,
+0.0000305175781250, -0.0002136230468750, 0.0012817382812500, -0.0009765625000000,
+0.0001220703125000, 0.0000610351562500, 0.0016784667968750, -0.0008239746093750,
+0.0019531250000000, -0.0004577636718750, 0.0012512207031250, -0.0001220703125000,
+0.0014953613281250, -0.0001525878906250, 0.0017089843750000, 0.0007019042968750,
+0.0003967285156250, -0.0000610351562500, 0.0003356933593750, 0.0011901855468750,
+0.0002136230468750, 0.0007629394531250, -0.0005798339843750, -0.0000610351562500,
+-0.0005493164062500, 0.0020141601562500, -0.0003662109375000, -0.0003967285156250,
+-0.0006103515625000, 0.0005493164062500, -0.0008239746093750, 0.0007019042968750,
+-0.0006408691406250, -0.0008239746093750, -0.0006103515625000, 0.0004577636718750,
+-0.0013732910156250, -0.0006408691406250, -0.0014343261718750, -0.0006713867187500,
+-0.0016479492187500, -0.0002441406250000, -0.0021972656250000, -0.0003356933593750,
+-0.0021667480468750, -0.0001525878906250, -0.0019836425781250, -0.0000610351562500,
+-0.0026550292968750, 0.0004882812500000, -0.0021057128906250, 0.0002136230468750,
+-0.0014343261718750, -0.0000305175781250, -0.0023193359375000, 0.0007324218750000,
+-0.0008544921875000, -0.0006408691406250, -0.0010986328125000, 0.0000610351562500,
+-0.0009460449218750, 0.0000305175781250, -0.0000915527343750, -0.0008850097656250,
+-0.0006408691406250, 0.0000915527343750, 0.0003967285156250, -0.0008850097656250,
+0.0002746582031250, -0.0004882812500000, 0.0001525878906250, -0.0001831054687500,
+0.0007324218750000, -0.0007324218750000, 0.0005187988281250, -0.0002136230468750,
+0.0005187988281250, 0.0002441406250000, 0.0004882812500000, -0.0002441406250000,
+0.0005187988281250, 0.0002136230468750, 0.0005493164062500, 0.0007019042968750,
+0.0004272460937500, -0.0003051757812500, 0.0006713867187500, 0.0006103515625000,
+0.0006713867187500, 0.0005187988281250, 0.0006408691406250, 0.0007629394531250,
+0.0007934570312500, 0.0014038085937500, 0.0007629394531250, 0.0008544921875000,
+0.0007629394531250, 0.0017089843750000, 0.0008850097656250, 0.0016784667968750,
+0.0009460449218750, 0.0014648437500000, 0.0009155273437500, 0.0020751953125000,
+0.0008850097656250, 0.0022888183593750, 0.0011596679687500, 0.0014343261718750,
+0.0010375976562500, 0.0018005371093750, 0.0010681152343750, 0.0019531250000000,
+0.0012207031250000, 0.0013427734375000, 0.0010681152343750, 0.0016479492187500,
+0.0014953613281250, 0.0018615722656250, 0.0012512207031250, 0.0011291503906250,
+0.0014953613281250, 0.0013427734375000, 0.0019226074218750, 0.0018005371093750,
+0.0014648437500000, 0.0009460449218750, 0.0022583007812500, 0.0017700195312500,
+0.0021057128906250, 0.0017089843750000, 0.0020446777343750, 0.0015258789062500,
+0.0024414062500000, 0.0020141601562500, 0.0020751953125000, 0.0020446777343750,
+0.0022583007812500, 0.0018920898437500, 0.0020751953125000, 0.0022277832031250,
+0.0020141601562500, 0.0021667480468750, 0.0021057128906250, 0.0020446777343750,
+0.0018310546875000, 0.0026855468750000, 0.0018005371093750, 0.0022583007812500,
+0.0019226074218750, 0.0027770996093750, 0.0017089843750000, 0.0029296875000000,
+0.0015869140625000, 0.0025939941406250, 0.0018005371093750, 0.0032043457031250,
+0.0014038085937500, 0.0029296875000000, 0.0016174316406250, 0.0027160644531250,
+0.0015258789062500, 0.0028686523437500, 0.0012817382812500, 0.0026855468750000,
+0.0016174316406250, 0.0019836425781250, 0.0011291503906250, 0.0021362304687500,
+0.0014343261718750, 0.0014648437500000, 0.0012817382812500, 0.0009765625000000,
+0.0008239746093750, 0.0007629394531250, 0.0012512207031250, 0.0000610351562500,
+0.0004577636718750, 0.0001525878906250, 0.0005187988281250, -0.0003967285156250,
+0.0003051757812500, -0.0003356933593750, -0.0002746582031250, 0.0000915527343750,
+0.0000305175781250, -0.0003051757812500, -0.0004272460937500, 0.0009765625000000,
+-0.0003051757812500, 0.0008850097656250, -0.0003051757812500, 0.0016174316406250,
+-0.0004882812500000, 0.0028991699218750, -0.0001831054687500, 0.0025024414062500,
+-0.0007934570312500, 0.0041809082031250, -0.0004577636718750, 0.0042114257812500,
+-0.0007324218750000, 0.0045166015625000, -0.0014343261718750, 0.0055541992187500,
+-0.0010070800781250, 0.0050659179687500, -0.0018310546875000, 0.0052490234375000,
+-0.0018005371093750, 0.0054626464843750, -0.0019531250000000, 0.0046081542968750,
+-0.0023498535156250, 0.0041503906250000, -0.0020446777343750, 0.0044555664062500,
+-0.0027770996093750, 0.0027160644531250, -0.0027160644531250, 0.0031738281250000,
+-0.0026855468750000, 0.0026245117187500, -0.0030212402343750, 0.0013122558593750,
+-0.0028381347656250, 0.0023803710937500, -0.0029296875000000, 0.0010375976562500,
+-0.0029602050781250, 0.0016479492187500, -0.0026245117187500, 0.0016784667968750,
+-0.0024719238281250, 0.0010070800781250, -0.0027770996093750, 0.0018615722656250,
+-0.0015869140625000, 0.0002136230468750, -0.0018310546875000, 0.0010681152343750,
+-0.0016479492187500, 0.0004272460937500, -0.0008850097656250, -0.0007934570312500,
+-0.0013732910156250, 0.0009155273437500, -0.0010070800781250, -0.0013427734375000,
+-0.0008544921875000, -0.0006408691406250, -0.0010681152343750, -0.0001525878906250,
+-0.0012512207031250, -0.0012207031250000, -0.0013427734375000, -0.0002746582031250,
+-0.0009460449218750, -0.0000915527343750, -0.0009460449218750, -0.0002441406250000,
+-0.0015563964843750, 0.0005493164062500, -0.0016479492187500, 0.0013122558593750,
+-0.0011901855468750, 0.0009460449218750, -0.0019531250000000, 0.0014038085937500,
+-0.0018615722656250, 0.0012817382812500, -0.0018920898437500, 0.0004577636718750,
+-0.0023803710937500, 0.0000305175781250, -0.0020141601562500, -0.0000610351562500,
+-0.0021972656250000, -0.0015869140625000, -0.0021667480468750, -0.0019226074218750,
+-0.0020751953125000, -0.0016479492187500, -0.0019836425781250, -0.0020446777343750,
+-0.0018005371093750, -0.0020751953125000, -0.0025024414062500, -0.0002136230468750,
+-0.0018920898437500, -0.0002136230468750, -0.0021057128906250, 0.0001831054687500,
+-0.0030517578125000, 0.0018920898437500, -0.0021362304687500, 0.0018615722656250,
+-0.0016174316406250, 0.0007324218750000, -0.0019531250000000, 0.0017700195312500,
+-0.0009460449218750, 0.0015563964843750, 0.0011291503906250, -0.0005493164062500,
+0.0031738281250000, 0.0032348632812500, 0.0017700195312500, 0.0025024414062500,
+0.0022277832031250, 0.0039672851562500, 0.0017395019531250, 0.0032043457031250,
+0.0008850097656250, 0.0017089843750000, 0.0011291503906250, 0.0025939941406250,
+0.0010681152343750, 0.0022277832031250, 0.0015563964843750, 0.0015563964843750,
+0.0003967285156250},
+{0.0007019042968750, -0.0001525878906250, 0.0006103515625000, 0.0006103515625000,
+0.0006713867187500, 0.0016479492187500, 0.0008544921875000, 0.0003051757812500,
+0.0003356933593750, 0.0027160644531250, 0.0006713867187500, 0.0015869140625000,
+0.0003356933593750, 0.0018615722656250, -0.0002746582031250, 0.0033874511718750,
+0.0004272460937500, 0.0014038085937500, -0.0007019042968750, 0.0033874511718750,
+-0.0005187988281250, 0.0026550292968750, -0.0003662109375000, 0.0021667480468750,
+-0.0009765625000000, 0.0030517578125000, -0.0006713867187500, 0.0018615722656250,
+-0.0007019042968750, 0.0018920898437500, -0.0007019042968750, 0.0018920898437500,
+-0.0009155273437500, 0.0012207031250000, -0.0009460449218750, 0.0008544921875000,
+-0.0007019042968750, 0.0010986328125000, -0.0016174316406250, 0.0004272460937500,
+-0.0013122558593750, 0.0003967285156250, -0.0014038085937500, 0.0008850097656250,
+-0.0020751953125000, 0.0008850097656250, -0.0018005371093750, 0.0009155273437500,
+-0.0023498535156250, 0.0022888183593750, -0.0024108886718750, 0.0021362304687500,
+-0.0026245117187500, 0.0025329589843750, -0.0030517578125000, 0.0036621093750000,
+-0.0030212402343750, 0.0034484863281250, -0.0032958984375000, 0.0042114257812500,
+-0.0034484863281250, 0.0046691894531250, -0.0032958984375000, 0.0042724609375000,
+-0.0033874511718750, 0.0042419433593750, -0.0035400390625000, 0.0046081542968750,
+-0.0025939941406250, 0.0030517578125000, -0.0027465820312500, 0.0035400390625000,
+-0.0024108886718750, 0.0030212402343750, -0.0015563964843750, 0.0015869140625000,
+-0.0018005371093750, 0.0023193359375000, -0.0015869140625000, 0.0011901855468750,
+-0.0013122558593750, 0.0009155273437500, -0.0014343261718750, 0.0009460449218750,
+-0.0014953613281250, 0.0006713867187500, -0.0012207031250000, 0.0009155273437500,
+-0.0014648437500000, 0.0002136230468750, -0.0011291503906250, 0.0007934570312500,
+-0.0009765625000000, 0.0005493164062500, -0.0010375976562500, -0.0003662109375000,
+-0.0005187988281250, 0.0003662109375000, -0.0012512207031250, -0.0005798339843750,
+-0.0005187988281250, -0.0004577636718750, -0.0006713867187500, -0.0005187988281250,
+-0.0013732910156250, -0.0013122558593750, -0.0004272460937500, -0.0010375976562500,
+-0.0012512207031250, -0.0005187988281250, -0.0006713867187500, -0.0011291503906250,
+-0.0006408691406250, -0.0002136230468750, -0.0011596679687500, 0.0006103515625000,
+-0.0002441406250000, -0.0005798339843750, -0.0011901855468750, 0.0015563964843750,
+-0.0007019042968750, 0.0009460449218750, -0.0007629394531250, 0.0007019042968750,
+-0.0015563964843750, 0.0017089843750000, -0.0008239746093750, 0.0006408691406250,
+-0.0006103515625000, 0.0005187988281250, -0.0010681152343750, 0.0003356933593750,
+0.0003967285156250, 0.0003356933593750, 0.0018615722656250, 0.0001525878906250,
+0.0009155273437500, -0.0002136230468750, 0.0030822753906250, 0.0017700195312500,
+0.0035095214843750, 0.0014343261718750, 0.0028381347656250, 0.0015258789062500,
+0.0030517578125000, 0.0037231445312500, 0.0011901855468750, 0.0004577636718750,
+-0.0005493164062500, 0.0029296875000000, 0.0002441406250000, 0.0022277832031250,
+-0.0003662109375000, 0.0026245117187500, -0.0011596679687500, 0.0045166015625000,
+-0.0004272460937500, 0.0048828125000000, -0.0023803710937500, 0.0031127929687500,
+-0.0021667480468750, 0.0041809082031250, -0.0018615722656250, 0.0040893554687500,
+-0.0030517578125000, 0.0022277832031250, -0.0027160644531250, 0.0028381347656250,
+-0.0010375976562500, 0.0027770996093750, -0.0016479492187500, 0.0025634765625000,
+-0.0007934570312500, 0.0025329589843750, 0.0012207031250000, 0.0025939941406250,
+0.0003662109375000, 0.0025329589843750, 0.0004577636718750, 0.0015869140625000,
+0.0010681152343750, 0.0016479492187500, 0.0003356933593750, 0.0013732910156250,
+-0.0009155273437500, 0.0003356933593750, -0.0004882812500000, 0.0003662109375000,
+-0.0001525878906250, 0.0010681152343750, -0.0017700195312500, 0.0007019042968750,
+-0.0005798339843750, 0.0007019042968750, 0.0009155273437500, 0.0016174316406250,
+-0.0012817382812500, 0.0014953613281250, 0.0011901855468750, 0.0006713867187500,
+0.0007934570312500, 0.0011291503906250, -0.0002746582031250, 0.0004577636718750,
+0.0005493164062500, -0.0007629394531250, 0.0001831054687500, -0.0000610351562500,
+-0.0013427734375000, -0.0008239746093750, -0.0011291503906250, -0.0010070800781250,
+-0.0010375976562500, -0.0010375976562500, -0.0021362304687500, -0.0010070800781250,
+-0.0017700195312500, -0.0006103515625000, 0.0002746582031250, -0.0017089843750000,
+-0.0003662109375000, -0.0014343261718750, 0.0005493164062500, -0.0014953613281250,
+0.0026855468750000, -0.0022888183593750, 0.0018310546875000, -0.0020446777343750,
+0.0031127929687500, -0.0022277832031250, 0.0036621093750000, -0.0023193359375000,
+0.0032348632812500, -0.0023193359375000, 0.0036010742187500, -0.0023193359375000,
+0.0039978027343750, -0.0022277832031250, 0.0022277832031250, -0.0026855468750000,
+0.0030517578125000, -0.0025024414062500, 0.0024108886718750, -0.0025939941406250,
+0.0007629394531250, -0.0031738281250000, 0.0020446777343750, -0.0029296875000000,
+0.0009460449218750, -0.0026550292968750, 0.0009460449218750, -0.0029907226562500,
+0.0016174316406250, -0.0025634765625000, 0.0015563964843750, -0.0019531250000000,
+0.0017395019531250, -0.0023193359375000, 0.0024719238281250, -0.0016174316406250,
+0.0024108886718750, -0.0016784667968750, 0.0026550292968750, -0.0014648437500000,
+0.0032348632812500, -0.0009765625000000, 0.0028991699218750, -0.0014038085937500,
+0.0031738281250000, -0.0003967285156250, 0.0036621093750000, -0.0005493164062500,
+0.0029296875000000, -0.0005798339843750, 0.0025939941406250, -0.0000305175781250,
+0.0035095214843750, -0.0003051757812500, 0.0013732910156250, -0.0002441406250000,
+0.0020751953125000, -0.0003662109375000, 0.0014038085937500, -0.0003662109375000,
+-0.0002746582031250, -0.0003662109375000, 0.0010070800781250, -0.0006713867187500,
+-0.0011291503906250, -0.0002441406250000, -0.0008544921875000, -0.0004272460937500,
+-0.0010681152343750, -0.0004882812500000, -0.0023193359375000, -0.0003051757812500,
+-0.0014648437500000, -0.0004882812500000, -0.0029907226562500, -0.0001220703125000,
+-0.0027160644531250, -0.0004577636718750, -0.0024719238281250, -0.0001525878906250,
+-0.0032043457031250, 0.0002746582031250, -0.0025939941406250, -0.0001525878906250,
+-0.0025329589843750, 0.0006103515625000, -0.0028686523437500, 0.0005798339843750,
+-0.0024108886718750, 0.0006713867187500, -0.0020751953125000, 0.0009765625000000,
+-0.0025939941406250, 0.0007324218750000, -0.0015869140625000, 0.0013427734375000,
+-0.0019531250000000, 0.0009460449218750, -0.0016784667968750, 0.0011291503906250,
+-0.0010070800781250, 0.0017700195312500, -0.0018005371093750, 0.0012207031250000,
+-0.0008850097656250, 0.0018615722656250, -0.0010986328125000, 0.0017700195312500,
+-0.0010986328125000, 0.0016174316406250, -0.0004882812500000, 0.0018920898437500,
+-0.0008239746093750, 0.0016174316406250, -0.0004882812500000, 0.0015869140625000,
+-0.0005187988281250, 0.0015258789062500, -0.0006408691406250, 0.0013732910156250,
+-0.0007019042968750, 0.0011901855468750, -0.0008239746093750, 0.0011596679687500,
+-0.0009765625000000, 0.0011291503906250, -0.0010070800781250, 0.0010375976562500,
+-0.0011291503906250, 0.0008850097656250, -0.0010986328125000, 0.0008544921875000,
+-0.0007324218750000, 0.0009155273437500, -0.0012512207031250, 0.0003662109375000,
+-0.0009765625000000, 0.0005798339843750, -0.0007934570312500, 0.0003051757812500,
+-0.0014648437500000, -0.0002136230468750, -0.0016174316406250, 0.0001220703125000,
+-0.0010681152343750, -0.0005798339843750, -0.0020446777343750, -0.0005187988281250,
+-0.0017395019531250, -0.0005798339843750, -0.0008850097656250, -0.0009765625000000,
+-0.0018920898437500, -0.0006713867187500, -0.0005798339843750, -0.0011291503906250,
+-0.0005798339843750, -0.0011901855468750, -0.0005798339843750, -0.0007934570312500,
+0.0001831054687500, -0.0007629394531250, 0.0000610351562500, -0.0010070800781250,
+0.0007019042968750, -0.0005187988281250, 0.0007019042968750, -0.0002746582031250,
+0.0009460449218750, -0.0005798339843750, 0.0015258789062500, -0.0007019042968750,
+0.0014953613281250, -0.0003662109375000, 0.0020141601562500, -0.0006408691406250,
+0.0023193359375000, -0.0006103515625000, 0.0025024414062500, -0.0005187988281250,
+0.0027770996093750, -0.0005493164062500, 0.0028076171875000, -0.0003967285156250,
+0.0030517578125000, -0.0004882812500000, 0.0028991699218750, -0.0004882812500000,
+0.0027770996093750, -0.0003662109375000, 0.0027465820312500, -0.0002441406250000,
+0.0024414062500000, -0.0002746582031250, 0.0023193359375000, -0.0004272460937500,
+0.0022277832031250, -0.0002441406250000, 0.0020141601562500, -0.0002746582031250,
+0.0020751953125000, -0.0006103515625000, 0.0022888183593750, -0.0003967285156250,
+0.0019226074218750, -0.0000610351562500, 0.0028076171875000, -0.0002746582031250,
+0.0025329589843750, 0.0002746582031250, 0.0021972656250000, 0.0010375976562500,
+0.0037231445312500, 0.0007019042968750, 0.0020751953125000, 0.0010986328125000,
+0.0028686523437500, 0.0017700195312500, 0.0026855468750000, 0.0014038085937500,
+0.0014343261718750, 0.0011291503906250, 0.0025024414062500, 0.0018615722656250,
+0.0006713867187500, 0.0001525878906250, 0.0007934570312500, 0.0008544921875000,
+0.0007324218750000, 0.0005493164062500, -0.0003356933593750, -0.0008239746093750,
+0.0001525878906250, 0.0000000000000000, -0.0001525878906250, -0.0011901855468750,
+-0.0003967285156250, -0.0010681152343750, -0.0000610351562500, -0.0010986328125000,
+0.0003051757812500, -0.0016479492187500, 0.0003356933593750, -0.0010070800781250,
+0.0005187988281250, -0.0013427734375000, 0.0012817382812500, -0.0010375976562500,
+0.0014038085937500, -0.0013122558593750, 0.0010070800781250, -0.0019531250000000,
+0.0012817382812500, -0.0014038085937500, 0.0013427734375000, -0.0019836425781250,
+0.0013427734375000, -0.0024719238281250, 0.0013427734375000, -0.0024108886718750,
+0.0012817382812500, -0.0025939941406250, 0.0015258789062500, -0.0028381347656250,
+0.0012817382812500, -0.0018615722656250, 0.0016784667968750, -0.0022888183593750,
+0.0013122558593750, -0.0021972656250000, 0.0003967285156250, -0.0015258789062500,
+0.0004577636718750, -0.0019226074218750, -0.0000305175781250, -0.0007629394531250,
+-0.0009155273437500, -0.0010070800781250, -0.0008544921875000, -0.0010681152343750,
+-0.0004272460937500, -0.0002136230468750, -0.0007934570312500, -0.0005798339843750,
+-0.0000610351562500, -0.0017089843750000, -0.0005187988281250, -0.0012207031250000,
+-0.0006408691406250, -0.0010375976562500, -0.0000610351562500, -0.0016479492187500,
+-0.0005798339843750, -0.0014038085937500, -0.0004577636718750, -0.0017089843750000,
+-0.0013122558593750, -0.0019836425781250, -0.0010986328125000, -0.0018615722656250,
+-0.0006408691406250, -0.0018920898437500, -0.0018920898437500, -0.0020751953125000,
+-0.0011901855468750, -0.0016479492187500, -0.0014038085937500, -0.0011291503906250,
+-0.0021362304687500, -0.0012207031250000, -0.0024108886718750, -0.0006713867187500,
+-0.0028076171875000, 0.0004882812500000, -0.0027465820312500, -0.0007629394531250,
+-0.0033569335937500, 0.0003356933593750, -0.0028686523437500, 0.0004577636718750,
+-0.0019836425781250, -0.0008239746093750, -0.0027465820312500, -0.0001220703125000,
+-0.0017700195312500, -0.0003662109375000, -0.0011291503906250, -0.0011901855468750,
+-0.0016479492187500, -0.0003356933593750, -0.0015869140625000, 0.0004272460937500,
+-0.0007019042968750},
+{-0.0007019042968750, -0.0007629394531250, -0.0006408691406250, -0.0006408691406250,
+-0.0006408691406250, -0.0006408691406250, -0.0005187988281250, -0.0005493164062500,
+-0.0003967285156250, -0.0003356933593750, -0.0002136230468750, -0.0003662109375000,
+0.0003051757812500, -0.0002746582031250, 0.0006713867187500, -0.0000610351562500,
+0.0008850097656250, 0.0001525878906250, 0.0016174316406250, -0.0000610351562500,
+0.0021362304687500, -0.0002746582031250, 0.0019531250000000, 0.0001220703125000,
+0.0029907226562500, -0.0002136230468750, 0.0029907226562500, -0.0004272460937500,
+0.0028076171875000, 0.0000305175781250, 0.0032348632812500, 0.0002136230468750,
+0.0030822753906250, -0.0000915527343750, 0.0021667480468750, 0.0007934570312500,
+0.0028381347656250, 0.0008850097656250, 0.0020751953125000, 0.0007019042968750,
+0.0012512207031250, 0.0011291503906250, 0.0027160644531250, 0.0011596679687500,
+0.0006103515625000, 0.0006408691406250, 0.0017395019531250, 0.0009460449218750,
+0.0021362304687500, 0.0005493164062500, 0.0010681152343750, 0.0000610351562500,
+0.0026855468750000, 0.0006713867187500, 0.0016784667968750, -0.0006103515625000,
+0.0022583007812500, -0.0003051757812500, 0.0022888183593750, -0.0002746582031250,
+0.0016174316406250, -0.0009155273437500, 0.0022888183593750, -0.0002441406250000,
+0.0007324218750000, -0.0006713867187500, 0.0010681152343750, -0.0005798339843750,
+0.0009765625000000, -0.0003356933593750, -0.0001831054687500, -0.0003356933593750,
+0.0001831054687500, -0.0003662109375000, 0.0003051757812500, -0.0010375976562500,
+0.0003051757812500, -0.0009460449218750, 0.0006408691406250, -0.0011291503906250,
+0.0013122558593750, -0.0019226074218750, 0.0018005371093750, -0.0021972656250000,
+0.0018005371093750, -0.0018615722656250, 0.0022277832031250, -0.0024414062500000,
+0.0019226074218750, -0.0017395019531250, 0.0017089843750000, -0.0008850097656250,
+0.0022888183593750, -0.0020141601562500, 0.0007934570312500, -0.0006408691406250,
+0.0012207031250000, -0.0007934570312500, 0.0012512207031250, -0.0005798339843750,
+0.0004882812500000, 0.0002441406250000, 0.0012512207031250, -0.0004882812500000,
+0.0006103515625000, 0.0001525878906250, 0.0004577636718750, 0.0004577636718750,
+0.0003356933593750, 0.0003967285156250, 0.0002746582031250, -0.0001831054687500,
+0.0006408691406250, -0.0012512207031250, -0.0002441406250000, -0.0007019042968750,
+-0.0000305175781250, -0.0010681152343750, 0.0002441406250000, -0.0012207031250000,
+0.0000000000000000, -0.0010986328125000, 0.0001831054687500, -0.0018005371093750,
+-0.0003051757812500, -0.0011901855468750, 0.0002136230468750, -0.0011291503906250,
+-0.0000915527343750, -0.0013732910156250, -0.0010986328125000, -0.0007934570312500,
+-0.0006408691406250, -0.0002136230468750, -0.0009765625000000, -0.0011901855468750,
+-0.0015258789062500, 0.0000305175781250, -0.0014038085937500, 0.0000305175781250,
+-0.0013122558593750, -0.0011596679687500, -0.0018920898437500, 0.0000915527343750,
+-0.0014038085937500, -0.0002441406250000, -0.0021057128906250, -0.0011291503906250,
+-0.0019836425781250, -0.0003662109375000, -0.0016784667968750, 0.0000610351562500,
+-0.0027160644531250, -0.0011596679687500, -0.0009460449218750, 0.0007629394531250,
+-0.0015869140625000, 0.0005798339843750, -0.0015258789062500, 0.0003662109375000,
+0.0000000000000000, 0.0015869140625000, -0.0007019042968750, 0.0016479492187500,
+-0.0008239746093750, 0.0010986328125000, 0.0001220703125000, 0.0018005371093750,
+-0.0003356933593750, 0.0017395019531250, -0.0011291503906250, 0.0012512207031250,
+-0.0000610351562500, 0.0021362304687500, -0.0005493164062500, 0.0016784667968750,
+-0.0002746582031250, 0.0019531250000000, 0.0002746582031250, 0.0019836425781250,
+0.0003967285156250, 0.0016174316406250, 0.0006103515625000, 0.0017395019531250,
+0.0008544921875000, 0.0008850097656250, 0.0014038085937500, 0.0009155273437500,
+0.0009155273437500, 0.0003967285156250, 0.0007324218750000, -0.0006103515625000,
+0.0015869140625000, -0.0002136230468750, -0.0003662109375000, -0.0005798339843750,
+0.0001220703125000, -0.0008544921875000, 0.0002441406250000, -0.0003051757812500,
+-0.0010070800781250, 0.0000915527343750, -0.0003967285156250, -0.0003356933593750,
+-0.0006103515625000, 0.0004272460937500, -0.0007629394531250, 0.0008850097656250,
+-0.0004272460937500, 0.0005798339843750, -0.0001831054687500, 0.0007629394531250,
+-0.0003356933593750, 0.0013732910156250, -0.0003967285156250, 0.0000305175781250,
+0.0000305175781250, 0.0005798339843750, -0.0000305175781250, 0.0003356933593750,
+-0.0002746582031250, -0.0009155273437500, -0.0000305175781250, -0.0003356933593750,
+-0.0001525878906250, -0.0007629394531250, -0.0000610351562500, -0.0011901855468750,
+-0.0001525878906250, -0.0010070800781250, -0.0003662109375000, -0.0010375976562500,
+-0.0002136230468750, -0.0013122558593750, -0.0004882812500000, -0.0007629394531250,
+-0.0004577636718750, -0.0010681152343750, -0.0004577636718750, -0.0008850097656250,
+-0.0005187988281250, -0.0004882812500000, -0.0004272460937500, -0.0010375976562500,
+-0.0004577636718750, -0.0005493164062500, -0.0003662109375000, -0.0004577636718750,
+-0.0003356933593750, -0.0010070800781250, -0.0002746582031250, -0.0009765625000000,
+-0.0001831054687500, -0.0006103515625000, -0.0003662109375000, -0.0013732910156250,
+-0.0000305175781250, -0.0011901855468750, 0.0000000000000000, -0.0014648437500000,
+-0.0003051757812500, -0.0021667480468750, -0.0000610351562500, -0.0019836425781250,
+-0.0001525878906250, -0.0025634765625000, -0.0001525878906250, -0.0031127929687500,
+-0.0000305175781250, -0.0030822753906250, -0.0001525878906250, -0.0034484863281250,
+-0.0002136230468750, -0.0043334960937500, -0.0000610351562500, -0.0038757324218750,
+-0.0002746582031250, -0.0040283203125000, -0.0001525878906250, -0.0045776367187500,
+0.0000915527343750, -0.0046691894531250, -0.0002441406250000, -0.0047912597656250,
+0.0001525878906250, -0.0056152343750000, 0.0001220703125000, -0.0052795410156250,
+-0.0000305175781250, -0.0052795410156250, 0.0002441406250000, -0.0057678222656250,
+0.0002746582031250, -0.0052795410156250, -0.0000305175781250, -0.0052185058593750,
+0.0000610351562500, -0.0054016113281250, 0.0000305175781250, -0.0051574707031250,
+-0.0001525878906250, -0.0050048828125000, -0.0000915527343750, -0.0054626464843750,
+-0.0000610351562500, -0.0049438476562500, 0.0000000000000000, -0.0051879882812500,
+0.0001220703125000, -0.0050964355468750, 0.0002136230468750, -0.0048828125000000,
+0.0003967285156250, -0.0055541992187500, 0.0002441406250000, -0.0045776367187500,
+0.0006408691406250, -0.0055847167968750, 0.0003967285156250, -0.0052185058593750,
+0.0001831054687500, -0.0043945312500000, 0.0006713867187500, -0.0060119628906250,
+-0.0002746582031250, -0.0040588378906250, 0.0000305175781250, -0.0048217773437500,
+-0.0003051757812500, -0.0046386718750000, -0.0010986328125000, -0.0031433105468750,
+-0.0004882812500000, -0.0042419433593750, -0.0012512207031250, -0.0027465820312500,
+-0.0011596679687500, -0.0028381347656250, -0.0010986328125000, -0.0025939941406250,
+-0.0012817382812500, -0.0017089843750000, -0.0007629394531250, -0.0023803710937500,
+-0.0017700195312500, -0.0011901855468750, -0.0010070800781250, -0.0017395019531250,
+-0.0014953613281250, -0.0015869140625000, -0.0025634765625000, -0.0009155273437500,
+-0.0013732910156250, -0.0021362304687500, -0.0029296875000000, -0.0010070800781250,
+-0.0025329589843750, -0.0018920898437500, -0.0024108886718750, -0.0018005371093750,
+-0.0030822753906250, -0.0010681152343750, -0.0022583007812500, -0.0025634765625000,
+-0.0031127929687500, -0.0003662109375000, -0.0027770996093750, -0.0013427734375000,
+-0.0025634765625000, -0.0007629394531250, -0.0028991699218750, 0.0012207031250000,
+-0.0022888183593750, -0.0003662109375000, -0.0022277832031250, 0.0021667480468750,
+-0.0017089843750000, 0.0021362304687500, -0.0014953613281250, 0.0020141601562500,
+-0.0014038085937500, 0.0033874511718750, -0.0006408691406250, 0.0030822753906250,
+-0.0011596679687500, 0.0032043457031250, -0.0007019042968750, 0.0039672851562500,
+-0.0006103515625000, 0.0030822753906250, -0.0009765625000000, 0.0024108886718750,
+-0.0002136230468750, 0.0034790039062500, -0.0011596679687500, 0.0009155273437500,
+-0.0006408691406250, 0.0012817382812500, -0.0006713867187500, 0.0012207031250000,
+-0.0012817382812500, -0.0005493164062500, -0.0005493164062500, 0.0001220703125000,
+-0.0014953613281250, 0.0004577636718750, -0.0012512207031250, -0.0001525878906250,
+-0.0011291503906250, 0.0005798339843750, -0.0017395019531250, 0.0013122558593750,
+-0.0014343261718750, 0.0002441406250000, -0.0013732910156250, 0.0017395019531250,
+-0.0016174316406250, 0.0013732910156250, -0.0014038085937500, 0.0011291503906250,
+-0.0012512207031250, 0.0019226074218750, -0.0015869140625000, 0.0013732910156250,
+-0.0006713867187500, 0.0010986328125000, -0.0010375976562500, 0.0012207031250000,
+-0.0001831054687500, 0.0004577636718750, 0.0008544921875000, -0.0002136230468750,
+-0.0003051757812500, 0.0003356933593750, 0.0016784667968750, -0.0002136230468750,
+0.0015258789062500, -0.0003356933593750, 0.0015258789062500, -0.0000915527343750,
+0.0027160644531250, -0.0001220703125000, 0.0022888183593750, -0.0000915527343750,
+0.0027160644531250, -0.0003662109375000, 0.0027465820312500, 0.0000610351562500,
+0.0027160644531250, -0.0007019042968750, 0.0029602050781250, -0.0016479492187500,
+0.0026550292968750, -0.0009155273437500, 0.0020751953125000, -0.0019836425781250,
+0.0021972656250000, -0.0030517578125000, 0.0015258789062500, -0.0014953613281250,
+0.0006408691406250, -0.0007019042968750, 0.0009765625000000, -0.0022277832031250,
+-0.0003662109375000, 0.0013427734375000, 0.0000000000000000, 0.0011596679687500,
+-0.0002746582031250, 0.0006408691406250, -0.0021057128906250, 0.0030822753906250,
+-0.0009460449218750, -0.0010681152343750, 0.0000000000000000, 0.0025634765625000,
+0.0003662109375000, 0.0014038085937500, -0.0000305175781250, 0.0017395019531250,
+0.0000000000000000, 0.0030822753906250, 0.0005493164062500, 0.0024719238281250,
+-0.0013427734375000, 0.0026245117187500, -0.0011901855468750, 0.0021057128906250,
+-0.0009765625000000, 0.0024414062500000, -0.0022277832031250, 0.0031127929687500,
+-0.0022888183593750, 0.0025024414062500, -0.0006713867187500, 0.0033569335937500,
+-0.0015563964843750, 0.0037231445312500, -0.0006713867187500, 0.0032958984375000,
+0.0014038085937500, 0.0033874511718750, 0.0000610351562500, 0.0036010742187500,
+0.0009155273437500, 0.0020446777343750, 0.0015563964843750, 0.0023193359375000,
+0.0002136230468750, 0.0023193359375000, -0.0006713867187500, 0.0013427734375000,
+0.0001525878906250, 0.0015869140625000, -0.0016479492187500, 0.0016479492187500,
+-0.0019531250000000, 0.0017089843750000, -0.0011596679687500, 0.0013427734375000,
+-0.0010986328125000, 0.0010986328125000, -0.0010070800781250, 0.0014038085937500,
+0.0000610351562500, 0.0004272460937500, 0.0005187988281250, 0.0006103515625000,
+0.0004882812500000, 0.0006408691406250, 0.0010986328125000, -0.0000305175781250,
+0.0015869140625000, 0.0002746582031250, 0.0000000000000000, 0.0007629394531250,
+0.0010070800781250, 0.0006103515625000, 0.0002746582031250, 0.0004577636718750,
+-0.0016174316406250, 0.0007934570312500, -0.0002136230468750, 0.0007019042968750,
+-0.0013732910156250, -0.0000915527343750, -0.0015869140625000, 0.0001525878906250,
+-0.0008850097656250, -0.0002746582031250, -0.0006408691406250, -0.0009460449218750,
+-0.0003967285156250, -0.0004577636718750, -0.0001525878906250, -0.0010986328125000,
+0.0006408691406250},
+{-0.0006103515625000, 0.0021972656250000, -0.0002136230468750, 0.0028991699218750,
+-0.0004882812500000, 0.0021362304687500, -0.0015258789062500, 0.0024108886718750,
+-0.0007019042968750, 0.0029296875000000, -0.0015869140625000, 0.0004882812500000,
+-0.0018615722656250, 0.0012817382812500, -0.0015258789062500, 0.0009155273437500,
+-0.0015869140625000, -0.0010986328125000, -0.0016174316406250, -0.0001220703125000,
+-0.0015258789062500, -0.0004882812500000, -0.0012817382812500, -0.0013122558593750,
+-0.0011901855468750, -0.0006103515625000, -0.0011291503906250, -0.0000610351562500,
+-0.0007019042968750, -0.0006103515625000, -0.0003051757812500, 0.0010375976562500,
+-0.0001220703125000, 0.0010070800781250, 0.0007324218750000, 0.0008544921875000,
+0.0017395019531250, 0.0018310546875000, 0.0016479492187500, 0.0019226074218750,
+0.0017395019531250, 0.0012512207031250, 0.0024719238281250, 0.0021667480468750,
+0.0023803710937500, 0.0016174316406250, 0.0020141601562500, 0.0006103515625000,
+0.0026245117187500, 0.0016479492187500, 0.0023193359375000, 0.0000610351562500,
+0.0020751953125000, 0.0001831054687500, 0.0018920898437500, 0.0004272460937500,
+0.0017089843750000, -0.0000915527343750, 0.0015563964843750, 0.0003051757812500,
+0.0007629394531250, -0.0004882812500000, 0.0010681152343750, 0.0002746582031250,
+0.0003967285156250, -0.0003662109375000, -0.0003356933593750, -0.0016479492187500,
+0.0005493164062500, -0.0005187988281250, -0.0011596679687500, -0.0013122558593750,
+-0.0012207031250000, -0.0019531250000000, -0.0005187988281250, -0.0008850097656250,
+-0.0008850097656250, 0.0005493164062500, 0.0005798339843750, -0.0004272460937500,
+-0.0011596679687500, -0.0009155273437500, -0.0007324218750000, -0.0015869140625000,
+-0.0009155273437500, -0.0002746582031250, -0.0019531250000000, 0.0007019042968750,
+-0.0018920898437500, -0.0002136230468750, -0.0004272460937500, 0.0023498535156250,
+-0.0012207031250000, 0.0024108886718750, -0.0005493164062500, 0.0023193359375000,
+0.0013732910156250, 0.0033874511718750, 0.0006103515625000, 0.0028686523437500,
+0.0009155273437500, 0.0028991699218750, 0.0017395019531250, 0.0030212402343750,
+0.0007629394531250, 0.0029296875000000, -0.0003662109375000, 0.0027770996093750,
+0.0004272460937500, 0.0028076171875000, -0.0007324218750000, 0.0030517578125000,
+-0.0013427734375000, 0.0032958984375000, -0.0003051757812500, 0.0024414062500000,
+0.0003051757812500, 0.0020751953125000, -0.0002746582031250, 0.0028686523437500,
+0.0014648437500000, 0.0010375976562500, 0.0025024414062500, 0.0010681152343750,
+0.0014953613281250, 0.0012512207031250, 0.0014953613281250, 0.0003356933593750,
+0.0029602050781250, 0.0005493164062500, -0.0001220703125000, 0.0008544921875000,
+0.0009155273437500, 0.0006408691406250, 0.0007324218750000, 0.0005187988281250,
+-0.0014343261718750, 0.0008239746093750, 0.0000915527343750, 0.0008850097656250,
+-0.0004272460937500, 0.0000305175781250, -0.0006103515625000, 0.0000610351562500,
+0.0003356933593750, 0.0004272460937500, 0.0010375976562500, 0.0001220703125000,
+0.0008544921875000, -0.0001831054687500, 0.0010986328125000, 0.0007629394531250,
+0.0019531250000000, 0.0005798339843750, 0.0010681152343750, 0.0002746582031250,
+0.0001220703125000, 0.0006408691406250, 0.0009765625000000, 0.0004272460937500,
+-0.0012207031250000, -0.0004577636718750, -0.0009460449218750, -0.0003051757812500,
+-0.0012512207031250, -0.0004882812500000, -0.0028991699218750, -0.0011596679687500,
+-0.0023498535156250, -0.0008850097656250, -0.0027160644531250, -0.0009460449218750,
+-0.0036926269531250, -0.0008850097656250, -0.0029907226562500, -0.0010986328125000,
+-0.0024108886718750, -0.0011901855468750, -0.0036621093750000, -0.0008850097656250,
+-0.0020751953125000, -0.0017700195312500, -0.0023803710937500, -0.0017700195312500,
+-0.0028686523437500, -0.0016174316406250, -0.0021667480468750, -0.0019531250000000,
+-0.0024414062500000, -0.0020141601562500, -0.0032043457031250, -0.0018310546875000,
+-0.0028686523437500, -0.0018005371093750, -0.0034179687500000, -0.0020751953125000,
+-0.0042114257812500, -0.0021972656250000, -0.0036926269531250, -0.0021972656250000,
+-0.0050964355468750, -0.0025024414062500, -0.0051269531250000, -0.0026550292968750,
+-0.0050659179687500, -0.0025329589843750, -0.0058898925781250, -0.0025329589843750,
+-0.0058288574218750, -0.0026855468750000, -0.0054626464843750, -0.0024414062500000,
+-0.0058593750000000, -0.0024414062500000, -0.0053405761718750, -0.0024108886718750,
+-0.0046691894531250, -0.0023193359375000, -0.0051879882812500, -0.0023498535156250,
+-0.0038452148437500, -0.0018920898437500, -0.0038757324218750, -0.0020751953125000,
+-0.0036315917968750, -0.0018310546875000, -0.0026855468750000, -0.0012512207031250,
+-0.0029296875000000, -0.0014953613281250, -0.0019226074218750, -0.0011291503906250,
+-0.0015563964843750, -0.0010375976562500, -0.0014343261718750, -0.0010986328125000,
+-0.0008850097656250, -0.0010986328125000, -0.0005798339843750, -0.0010375976562500,
+-0.0006103515625000, -0.0009460449218750, -0.0002441406250000, -0.0010070800781250,
+-0.0002746582031250, -0.0009460449218750, -0.0003356933593750, -0.0008239746093750,
+0.0001220703125000, -0.0008544921875000, -0.0003967285156250, -0.0007019042968750,
+-0.0002136230468750, -0.0007324218750000, -0.0000915527343750, -0.0005798339843750,
+-0.0007934570312500, -0.0004272460937500, -0.0010070800781250, -0.0003662109375000,
+-0.0006713867187500, -0.0000610351562500, -0.0008544921875000, 0.0000610351562500,
+-0.0009155273437500, 0.0000305175781250, -0.0007019042968750, 0.0001831054687500,
+-0.0008544921875000, 0.0003356933593750, -0.0008850097656250, -0.0002136230468750,
+-0.0010070800781250, 0.0000305175781250, -0.0010070800781250, -0.0001220703125000,
+-0.0010375976562500, -0.0006408691406250, -0.0012817382812500, -0.0004272460937500,
+-0.0008544921875000, -0.0005798339843750, -0.0008850097656250, -0.0007019042968750,
+-0.0007934570312500, -0.0006103515625000, -0.0003662109375000, -0.0006408691406250,
+-0.0002441406250000, -0.0006713867187500, -0.0002746582031250, -0.0005798339843750,
+0.0000305175781250, -0.0007629394531250, -0.0000915527343750, -0.0005798339843750,
+-0.0002746582031250, -0.0004577636718750, -0.0001525878906250, -0.0007629394531250,
+-0.0009765625000000, -0.0004272460937500, -0.0004272460937500, -0.0002746582031250,
+-0.0009460449218750, -0.0005798339843750, -0.0013732910156250, -0.0005187988281250,
+0.0000305175781250, -0.0001525878906250, -0.0018920898437500, -0.0007019042968750,
+-0.0010986328125000, -0.0006408691406250, -0.0007629394531250, -0.0004577636718750,
+-0.0023193359375000, -0.0006103515625000, -0.0018005371093750, -0.0004577636718750,
+-0.0015869140625000, -0.0005493164062500, -0.0026550292968750, -0.0003356933593750,
+-0.0022277832031250, -0.0003662109375000, -0.0014343261718750, -0.0004577636718750,
+-0.0024108886718750, -0.0002746582031250, -0.0014343261718750, -0.0004882812500000,
+-0.0018005371093750, -0.0003662109375000, -0.0017700195312500, -0.0004272460937500,
+-0.0012512207031250, -0.0007324218750000, -0.0020446777343750, -0.0006713867187500,
+-0.0010375976562500, -0.0005187988281250, -0.0011901855468750, -0.0008239746093750,
+-0.0012512207031250, -0.0005798339843750, -0.0006408691406250, -0.0003356933593750,
+-0.0008239746093750, -0.0007019042968750, -0.0006408691406250, 0.0002136230468750,
+-0.0004882812500000, -0.0000610351562500, -0.0002136230468750, 0.0001831054687500,
+-0.0002136230468750, 0.0009460449218750, -0.0003967285156250, 0.0003967285156250,
+0.0004882812500000, 0.0011901855468750, -0.0002441406250000, 0.0010375976562500,
+0.0003051757812500, 0.0008544921875000, 0.0011901855468750, 0.0011596679687500,
+0.0000610351562500, 0.0009155273437500, 0.0018920898437500, 0.0007019042968750,
+0.0014648437500000, 0.0004882812500000, 0.0015258789062500, 0.0006103515625000,
+0.0028076171875000, 0.0005798339843750, 0.0023193359375000, 0.0003051757812500,
+0.0026855468750000, 0.0007934570312500, 0.0031127929687500, 0.0004882812500000,
+0.0031738281250000, 0.0004272460937500, 0.0031433105468750, 0.0006103515625000,
+0.0032348632812500, 0.0001831054687500, 0.0030212402343750, -0.0001525878906250,
+0.0023803710937500, -0.0003356933593750, 0.0025329589843750, -0.0005187988281250,
+0.0025329589843750, -0.0009765625000000, 0.0013732910156250, -0.0013122558593750,
+0.0023498535156250, -0.0006713867187500, 0.0017089843750000, -0.0011291503906250,
+0.0014953613281250, -0.0008544921875000, 0.0021057128906250, -0.0002441406250000,
+0.0012512207031250, -0.0011596679687500, 0.0018005371093750, -0.0007934570312500,
+0.0014648437500000, -0.0002136230468750, 0.0014648437500000, -0.0006408691406250,
+0.0016479492187500, -0.0010986328125000, 0.0009765625000000, -0.0007019042968750,
+0.0024414062500000, -0.0009765625000000, 0.0025024414062500, -0.0004882812500000,
+0.0022888183593750, -0.0005493164062500, 0.0034790039062500, -0.0010070800781250,
+0.0036926269531250, -0.0003662109375000, 0.0016174316406250, -0.0005798339843750,
+0.0034790039062500, -0.0005798339843750, 0.0027770996093750, -0.0001831054687500,
+-0.0006713867187500, -0.0001220703125000, -0.0000915527343750, -0.0004577636718750,
+-0.0003967285156250, -0.0001525878906250, -0.0006103515625000, -0.0001220703125000,
+-0.0003662109375000, -0.0005187988281250, -0.0001525878906250, -0.0005493164062500,
+-0.0002136230468750, -0.0000305175781250, -0.0005798339843750, -0.0005187988281250,
+0.0000610351562500, -0.0006408691406250, -0.0003662109375000, 0.0003051757812500,
+-0.0008544921875000, 0.0004882812500000, 0.0000305175781250, -0.0000305175781250,
+-0.0011596679687500, 0.0018005371093750, -0.0005493164062500, 0.0010070800781250,
+-0.0006103515625000, 0.0009460449218750, -0.0013732910156250, 0.0018920898437500,
+-0.0004272460937500, 0.0007019042968750, -0.0014343261718750, 0.0014648437500000,
+-0.0009460449218750, 0.0014648437500000, -0.0008850097656250, 0.0013427734375000,
+-0.0017395019531250, 0.0019531250000000, -0.0010375976562500, 0.0021057128906250,
+-0.0009460449218750, 0.0018920898437500, -0.0014343261718750, 0.0027770996093750,
+-0.0007324218750000, 0.0021362304687500, 0.0000000000000000, 0.0015563964843750,
+-0.0008544921875000, 0.0029907226562500, 0.0003662109375000, 0.0004272460937500,
+0.0003051757812500, 0.0013122558593750, 0.0001525878906250, 0.0012207031250000,
+0.0007324218750000, -0.0006408691406250, 0.0004882812500000, 0.0005187988281250,
+0.0002136230468750, -0.0003051757812500, 0.0006103515625000, 0.0001525878906250,
+0.0005493164062500, 0.0002746582031250, 0.0001831054687500, 0.0000305175781250,
+0.0006103515625000, 0.0010375976562500, 0.0001831054687500, 0.0000915527343750,
+0.0006408691406250, 0.0007934570312500, 0.0009155273437500, 0.0005493164062500,
+0.0007324218750000, -0.0002746582031250, 0.0011596679687500, 0.0007934570312500,
+0.0014953613281250, -0.0010681152343750, 0.0016479492187500, -0.0006103515625000,
+0.0015258789062500, -0.0007019042968750, 0.0018920898437500, -0.0017700195312500,
+0.0023498535156250, -0.0008239746093750, 0.0008239746093750, -0.0016784667968750,
+0.0015869140625000, -0.0013732910156250, 0.0011596679687500, -0.0013122558593750,
+-0.0002746582031250, -0.0017089843750000, 0.0007324218750000, -0.0011596679687500,
+-0.0003356933593750, -0.0020141601562500, -0.0007324218750000, -0.0014648437500000,
+-0.0000915527343750, -0.0017395019531250, -0.0000915527343750, -0.0025329589843750,
+-0.0005798339843750, -0.0017089843750000, 0.0002746582031250, -0.0026550292968750,
+0.0003356933593750, -0.0026550292968750, 0.0000000000000000, -0.0024719238281250,
+0.0000610351562500},
+{0.0029602050781250, -0.0004882812500000, 0.0033264160156250, -0.0004882812500000,
+0.0022888183593750, -0.0001831054687500, 0.0029296875000000, -0.0006408691406250,
+0.0020141601562500, -0.0003967285156250, 0.0017700195312500, -0.0006408691406250,
+0.0019531250000000, -0.0009765625000000, 0.0003662109375000, -0.0003356933593750,
+0.0017089843750000, -0.0014038085937500, 0.0017089843750000, -0.0011291503906250,
+0.0018615722656250, -0.0008850097656250, 0.0023803710937500, -0.0010375976562500,
+0.0014648437500000, -0.0003356933593750, 0.0025329589843750, -0.0007019042968750,
+0.0020141601562500, -0.0005187988281250, 0.0022888183593750, -0.0000305175781250,
+0.0032653808593750, -0.0000305175781250, 0.0023498535156250, -0.0000610351562500,
+0.0035400390625000, 0.0003051757812500, 0.0036926269531250, 0.0003051757812500,
+0.0031433105468750, 0.0000305175781250, 0.0027770996093750, -0.0001831054687500,
+0.0015563964843750, -0.0006103515625000, 0.0000305175781250, -0.0008850097656250,
+0.0007934570312500, -0.0008850097656250, 0.0006408691406250, -0.0002441406250000,
+-0.0007629394531250, 0.0000610351562500, -0.0002136230468750, -0.0000305175781250,
+-0.0002136230468750, 0.0019531250000000, -0.0005798339843750, 0.0013427734375000,
+-0.0004577636718750, 0.0018920898437500, -0.0000915527343750, 0.0030212402343750,
+-0.0001525878906250, 0.0013122558593750, -0.0003967285156250, 0.0029296875000000,
+0.0001831054687500, 0.0017395019531250, -0.0002746582031250, 0.0017395019531250,
+-0.0007019042968750, 0.0030822753906250, 0.0003967285156250, 0.0015258789062500,
+-0.0010681152343750, 0.0031433105468750, -0.0002136230468750, 0.0031127929687500,
+-0.0003051757812500, 0.0024719238281250, -0.0010375976562500, 0.0030212402343750,
+0.0003967285156250, 0.0030212402343750, -0.0014038085937500, 0.0017089843750000,
+-0.0007934570312500, 0.0023193359375000, -0.0003967285156250, 0.0018310546875000,
+-0.0016174316406250, 0.0005798339843750, -0.0010375976562500, 0.0014038085937500,
+0.0000610351562500, 0.0006408691406250, -0.0008239746093750, 0.0004577636718750,
+-0.0001525878906250, 0.0005798339843750, 0.0011596679687500, 0.0005187988281250,
+-0.0000305175781250, 0.0006103515625000, 0.0013427734375000, -0.0002136230468750,
+0.0011596679687500, 0.0002441406250000, 0.0006408691406250, -0.0001220703125000,
+0.0009765625000000, -0.0010070800781250, 0.0006103515625000, -0.0004882812500000,
+0.0006103515625000, -0.0013427734375000, 0.0004882812500000, -0.0013122558593750,
+0.0005493164062500, -0.0013122558593750, 0.0007629394531250, -0.0016174316406250,
+0.0009460449218750, -0.0013732910156250, 0.0012512207031250, -0.0014953613281250,
+0.0014343261718750, -0.0012207031250000, 0.0018310546875000, -0.0013732910156250,
+0.0022583007812500, -0.0014953613281250, 0.0022277832031250, -0.0008850097656250,
+0.0022888183593750, -0.0017700195312500, 0.0030212402343750, -0.0016479492187500,
+0.0023193359375000, -0.0016784667968750, 0.0017089843750000, -0.0023803710937500,
+0.0027465820312500, -0.0021972656250000, 0.0007629394531250, -0.0023193359375000,
+0.0013427734375000, -0.0024719238281250, 0.0011901855468750, -0.0020446777343750,
+-0.0002136230468750, -0.0017089843750000, 0.0007019042968750, -0.0018615722656250,
+0.0001220703125000, -0.0010681152343750, 0.0000000000000000, -0.0009460449218750,
+0.0000915527343750, -0.0010986328125000, 0.0000000000000000, -0.0007019042968750,
+0.0002746582031250, -0.0006408691406250, -0.0000610351562500, -0.0014648437500000,
+0.0001220703125000, -0.0008850097656250, 0.0001831054687500, -0.0014648437500000,
+-0.0000610351562500, -0.0023498535156250, 0.0000610351562500, -0.0013427734375000,
+0.0002746582031250, -0.0026855468750000, 0.0000610351562500, -0.0025329589843750,
+0.0002441406250000, -0.0022888183593750, 0.0004577636718750, -0.0029296875000000,
+0.0000915527343750, -0.0026550292968750, 0.0004577636718750, -0.0027465820312500,
+0.0003967285156250, -0.0030517578125000, 0.0003051757812500, -0.0031433105468750,
+0.0003662109375000, -0.0033569335937500, 0.0002136230468750, -0.0036621093750000,
+0.0003967285156250, -0.0034790039062500, 0.0003662109375000, -0.0033264160156250,
+0.0002746582031250, -0.0033264160156250, 0.0005493164062500, -0.0031738281250000,
+0.0006103515625000, -0.0029907226562500, 0.0001220703125000, -0.0030212402343750,
+0.0004882812500000, -0.0027770996093750, 0.0002136230468750, -0.0024719238281250,
+-0.0002746582031250, -0.0024414062500000, 0.0003356933593750, -0.0021972656250000,
+-0.0005493164062500, -0.0018920898437500, -0.0003051757812500, -0.0018310546875000,
+-0.0003051757812500, -0.0017395019531250, -0.0007324218750000, -0.0016479492187500,
+-0.0003662109375000, -0.0016784667968750, -0.0006408691406250, -0.0017089843750000,
+-0.0006408691406250, -0.0018005371093750, -0.0005187988281250, -0.0018005371093750,
+-0.0005798339843750, -0.0018310546875000, -0.0006713867187500, -0.0020446777343750,
+-0.0006408691406250, -0.0020751953125000, -0.0007324218750000, -0.0021667480468750,
+-0.0008239746093750, -0.0024108886718750, -0.0009155273437500, -0.0024414062500000,
+-0.0010681152343750, -0.0024108886718750, -0.0009765625000000, -0.0031738281250000,
+-0.0010986328125000, -0.0027770996093750, -0.0010070800781250, -0.0032653808593750,
+-0.0008544921875000, -0.0040283203125000, -0.0010681152343750, -0.0031433105468750,
+-0.0008544921875000, -0.0045471191406250, -0.0008850097656250, -0.0042114257812500,
+-0.0008544921875000, -0.0043029785156250, -0.0007019042968750, -0.0052185058593750,
+-0.0009155273437500, -0.0046386718750000, -0.0004882812500000, -0.0050964355468750,
+-0.0007629394531250, -0.0054321289062500, -0.0007019042968750, -0.0050659179687500,
+-0.0004272460937500, -0.0050354003906250, -0.0008544921875000, -0.0054931640625000,
+-0.0004272460937500, -0.0042724609375000, -0.0006713867187500, -0.0046081542968750,
+-0.0005187988281250, -0.0041503906250000, -0.0001831054687500, -0.0030517578125000,
+-0.0006713867187500, -0.0036010742187500, 0.0002136230468750, -0.0026245117187500,
+0.0000915527343750, -0.0023193359375000, 0.0000000000000000, -0.0025634765625000,
+0.0004577636718750, -0.0022583007812500, 0.0003051757812500, -0.0019531250000000,
+0.0003356933593750, -0.0027160644531250, 0.0004272460937500, -0.0025939941406250,
+0.0003967285156250, -0.0026855468750000, 0.0004882812500000, -0.0034790039062500,
+0.0005493164062500, -0.0034790039062500, 0.0000305175781250, -0.0032348632812500,
+0.0003662109375000, -0.0042724609375000, -0.0000305175781250, -0.0035095214843750,
+-0.0008544921875000, -0.0028076171875000, -0.0004882812500000, -0.0043945312500000,
+-0.0008544921875000, -0.0016479492187500, -0.0011901855468750, -0.0028686523437500,
+-0.0009155273437500, -0.0025329589843750, -0.0007324218750000, -0.0004272460937500,
+-0.0009155273437500, -0.0023498535156250, -0.0001525878906250, -0.0006103515625000,
+-0.0003662109375000, -0.0003051757812500, -0.0003662109375000, -0.0011291503906250,
+0.0002441406250000, -0.0008850097656250, 0.0000000000000000, -0.0008239746093750,
+0.0000305175781250, -0.0012817382812500, 0.0001831054687500, -0.0015563964843750,
+0.0001220703125000, -0.0010986328125000, 0.0001831054687500, -0.0007629394531250,
+0.0004272460937500, -0.0008239746093750, -0.0000305175781250, 0.0002746582031250,
+0.0004577636718750, 0.0003967285156250, 0.0000305175781250, 0.0007324218750000,
+-0.0005798339843750, 0.0016174316406250, 0.0003662109375000, 0.0017395019531250,
+-0.0009765625000000, 0.0021057128906250, -0.0007934570312500, 0.0027465820312500,
+-0.0001525878906250, 0.0026550292968750, -0.0006713867187500, 0.0025329589843750,
+-0.0008544921875000, 0.0028991699218750, 0.0000915527343750, 0.0014343261718750,
+-0.0006103515625000, 0.0015258789062500, -0.0005493164062500, 0.0011291503906250,
+0.0003356933593750, -0.0000610351562500, -0.0002441406250000, 0.0004272460937500,
+0.0002136230468750, 0.0006103515625000, 0.0003967285156250, 0.0003051757812500,
+0.0003967285156250, 0.0011291503906250, 0.0006408691406250, 0.0021972656250000,
+0.0007019042968750, 0.0018615722656250, 0.0005493164062500, 0.0019836425781250,
+0.0006103515625000, 0.0026245117187500, 0.0004577636718750, 0.0014953613281250,
+0.0002746582031250, 0.0002746582031250, 0.0002136230468750, 0.0010375976562500,
+-0.0005798339843750, -0.0009460449218750, -0.0005187988281250, -0.0012817382812500,
+-0.0005187988281250, -0.0006408691406250, -0.0012817382812500, -0.0009765625000000,
+0.0013427734375000, 0.0003051757812500, 0.0000000000000000, -0.0012817382812500,
+0.0014038085937500, -0.0013122558593750, 0.0006103515625000, -0.0003051757812500,
+-0.0005798339843750, 0.0003356933593750, 0.0008850097656250, 0.0000610351562500,
+0.0001220703125000, 0.0005187988281250, -0.0001831054687500, 0.0010375976562500,
+0.0009155273437500, 0.0005187988281250, 0.0016479492187500, -0.0001831054687500,
+0.0012512207031250, 0.0003967285156250, 0.0017700195312500, 0.0003356933593750,
+0.0028076171875000, -0.0001831054687500, 0.0015258789062500, 0.0008850097656250,
+0.0007934570312500, 0.0018920898437500, 0.0023803710937500, 0.0010375976562500,
+-0.0008239746093750, 0.0023193359375000, -0.0002441406250000, 0.0029602050781250,
+0.0001525878906250, 0.0021057128906250, -0.0015258789062500, 0.0018005371093750,
+-0.0006713867187500, 0.0021667480468750, 0.0002441406250000, 0.0014343261718750,
+0.0000000000000000, 0.0010375976562500, 0.0002746582031250, 0.0010681152343750,
+0.0010070800781250, 0.0010986328125000, 0.0007934570312500, 0.0010681152343750,
+0.0005493164062500, 0.0010375976562500, 0.0004272460937500, 0.0014648437500000,
+0.0000610351562500, 0.0009460449218750, -0.0006103515625000, 0.0002746582031250,
+-0.0009765625000000, 0.0008239746093750, -0.0008850097656250, 0.0000610351562500,
+-0.0020751953125000, -0.0003967285156250, -0.0012512207031250, 0.0000610351562500,
+-0.0005493164062500, 0.0003356933593750, -0.0024108886718750, 0.0000610351562500,
+0.0003051757812500, 0.0005798339843750, -0.0006713867187500, 0.0008850097656250,
+-0.0009765625000000, 0.0007324218750000, 0.0003662109375000, 0.0007629394531250,
+-0.0015258789062500, 0.0010070800781250, -0.0007629394531250, 0.0006713867187500,
+-0.0015258789062500, 0.0005798339843750, -0.0024414062500000, 0.0005798339843750,
+-0.0024414062500000, 0.0004272460937500, -0.0033569335937500, 0.0003051757812500,
+-0.0034179687500000, 0.0000915527343750, -0.0043029785156250, 0.0001525878906250,
+-0.0040893554687500, -0.0002136230468750, -0.0037841796875000, -0.0007629394531250,
+-0.0049438476562500, -0.0007019042968750, -0.0028991699218750, -0.0009765625000000,
+-0.0036621093750000, -0.0011596679687500, -0.0032958984375000, -0.0011596679687500,
+-0.0019226074218750, -0.0009765625000000, -0.0033264160156250, -0.0009765625000000,
+-0.0014343261718750, -0.0014648437500000, -0.0022888183593750, -0.0009155273437500,
+-0.0021362304687500, -0.0013427734375000, -0.0009765625000000, -0.0020446777343750,
+-0.0026245117187500, -0.0013122558593750, -0.0012817382812500, -0.0021667480468750,
+-0.0018615722656250, -0.0021362304687500, -0.0023803710937500, -0.0018615722656250,
+-0.0019226074218750, -0.0021057128906250, -0.0027160644531250, -0.0019531250000000,
+-0.0023193359375000, -0.0018920898437500, -0.0028991699218750, -0.0019226074218750,
+-0.0028686523437500, -0.0017700195312500, -0.0024414062500000, -0.0016784667968750,
+-0.0031127929687500, -0.0018005371093750, -0.0019226074218750, -0.0014038085937500,
+-0.0025939941406250, -0.0014038085937500, -0.0021667480468750, -0.0014343261718750,
+-0.0011596679687500, -0.0012512207031250, -0.0022583007812500, -0.0012512207031250,
+-0.0007934570312500},
+{0.0007629394531250, 0.0010681152343750, 0.0004882812500000, 0.0009765625000000,
+0.0004882812500000, 0.0016784667968750, 0.0007019042968750, 0.0007934570312500,
+0.0008544921875000, -0.0012207031250000, 0.0005187988281250, -0.0003051757812500,
+-0.0016784667968750, -0.0003662109375000, -0.0009460449218750, 0.0003356933593750,
+-0.0000915527343750, -0.0004272460937500, -0.0001831054687500, -0.0014038085937500,
+-0.0001831054687500, -0.0005493164062500, 0.0007629394531250, -0.0002746582031250,
+0.0011291503906250, -0.0006713867187500, 0.0003051757812500, 0.0002136230468750,
+-0.0000305175781250, 0.0013427734375000, 0.0004882812500000, 0.0008239746093750,
+-0.0013122558593750, 0.0012817382812500, -0.0013427734375000, 0.0019226074218750,
+-0.0007629394531250, 0.0013122558593750, -0.0012817382812500, 0.0008544921875000,
+-0.0012512207031250, 0.0015869140625000, 0.0001831054687500, 0.0011901855468750,
+-0.0000915527343750, 0.0009460449218750, -0.0002441406250000, 0.0013427734375000,
+0.0005493164062500, 0.0018615722656250, 0.0001220703125000, 0.0018920898437500,
+-0.0010070800781250, 0.0016479492187500, -0.0013427734375000, 0.0021362304687500,
+-0.0014343261718750, 0.0017395019531250, -0.0024719238281250, 0.0009155273437500,
+-0.0033264160156250, 0.0014038085937500, -0.0016479492187500, 0.0005798339843750,
+-0.0030517578125000, 0.0003967285156250, -0.0024108886718750, 0.0007934570312500,
+-0.0006408691406250, 0.0009460449218750, -0.0025634765625000, 0.0008239746093750,
+-0.0002136230468750, 0.0007324218750000, -0.0006408691406250, 0.0012207031250000,
+-0.0014343261718750, 0.0007019042968750, -0.0006408691406250, 0.0000610351562500,
+-0.0013732910156250, 0.0005187988281250, -0.0021667480468750, -0.0000305175781250,
+-0.0026245117187500, -0.0001831054687500, -0.0028686523437500, 0.0001525878906250,
+-0.0033569335937500, 0.0001831054687500, -0.0039062500000000, 0.0001525878906250,
+-0.0036315917968750, 0.0006408691406250, -0.0039978027343750, 0.0007629394531250,
+-0.0039978027343750, 0.0007019042968750, -0.0035705566406250, 0.0007629394531250,
+-0.0038452148437500, 0.0008544921875000, -0.0034484863281250, 0.0004882812500000,
+-0.0029602050781250, 0.0006408691406250, -0.0032958984375000, 0.0004882812500000,
+-0.0033874511718750, 0.0002136230468750, -0.0027770996093750, 0.0003051757812500,
+-0.0037536621093750, -0.0002441406250000, -0.0037231445312500, -0.0002441406250000,
+-0.0034790039062500, -0.0005493164062500, -0.0040893554687500, -0.0011291503906250,
+-0.0042724609375000, -0.0010681152343750, -0.0035095214843750, -0.0014038085937500,
+-0.0043029785156250, -0.0014953613281250, -0.0039367675781250, -0.0015869140625000,
+-0.0031127929687500, -0.0016784667968750, -0.0042114257812500, -0.0016174316406250,
+-0.0025939941406250, -0.0019531250000000, -0.0029907226562500, -0.0018310546875000,
+-0.0027465820312500, -0.0018005371093750, -0.0015563964843750, -0.0019836425781250,
+-0.0021667480468750, -0.0018615722656250, -0.0009765625000000, -0.0016174316406250,
+-0.0011291503906250, -0.0016784667968750, -0.0009765625000000, -0.0016174316406250,
+-0.0002136230468750, -0.0014343261718750, -0.0007019042968750, -0.0014343261718750,
+-0.0001525878906250, -0.0014953613281250, -0.0002746582031250, -0.0015563964843750,
+-0.0004272460937500, -0.0014343261718750, -0.0002441406250000, -0.0014343261718750,
+-0.0005798339843750, -0.0014648437500000, -0.0006713867187500, -0.0011291503906250,
+-0.0010681152343750, -0.0011596679687500, -0.0008239746093750, -0.0011901855468750,
+-0.0007019042968750, -0.0010681152343750, -0.0015869140625000, -0.0010681152343750,
+-0.0006408691406250, -0.0011291503906250, -0.0015258789062500, -0.0011291503906250,
+-0.0013427734375000, -0.0011596679687500, -0.0007934570312500, -0.0010986328125000,
+-0.0023498535156250, -0.0009765625000000, -0.0004577636718750, -0.0012817382812500,
+-0.0015869140625000, -0.0010681152343750, -0.0017395019531250, -0.0012817382812500,
+-0.0003662109375000, -0.0016479492187500, -0.0016174316406250, -0.0014038085937500,
+-0.0007019042968750, -0.0018005371093750, -0.0010070800781250, -0.0018920898437500,
+-0.0011596679687500, -0.0017700195312500, -0.0007019042968750, -0.0018920898437500,
+-0.0013122558593750, -0.0018615722656250, -0.0006713867187500, -0.0018005371093750,
+-0.0012207031250000, -0.0018005371093750, -0.0011291503906250, -0.0017395019531250,
+-0.0007019042968750, -0.0016479492187500, -0.0017700195312500, -0.0015563964843750,
+-0.0008544921875000, -0.0012817382812500, -0.0015563964843750, -0.0011596679687500,
+-0.0017089843750000, -0.0010070800781250, -0.0012207031250000, -0.0006103515625000,
+-0.0022888183593750, -0.0004577636718750, -0.0013427734375000, -0.0003967285156250,
+-0.0021667480468750, -0.0000610351562500, -0.0021057128906250, 0.0001220703125000,
+-0.0014343261718750, 0.0001525878906250, -0.0027160644531250, 0.0003356933593750,
+-0.0014343261718750, 0.0004577636718750, -0.0020446777343750, 0.0005187988281250,
+-0.0016784667968750, 0.0006713867187500, -0.0004577636718750, 0.0007629394531250,
+-0.0013732910156250, 0.0007324218750000, -0.0000915527343750, 0.0010986328125000,
+-0.0003051757812500, 0.0009765625000000, -0.0007324218750000, 0.0009155273437500,
+-0.0002441406250000, 0.0011291503906250, -0.0006408691406250, 0.0010375976562500,
+-0.0010070800781250, 0.0009765625000000, -0.0004577636718750, 0.0011291503906250,
+-0.0011901855468750, 0.0010681152343750, -0.0016174316406250, 0.0010375976562500,
+-0.0003356933593750, 0.0012207031250000, -0.0023803710937500, 0.0011596679687500,
+-0.0021667480468750, 0.0011291503906250, -0.0018615722656250, 0.0013427734375000,
+-0.0031433105468750, 0.0016174316406250, -0.0031127929687500, 0.0015869140625000,
+-0.0023803710937500, 0.0013122558593750, -0.0032653808593750, 0.0018005371093750,
+-0.0027770996093750, 0.0013732910156250, -0.0020751953125000, 0.0007629394531250,
+-0.0036926269531250, 0.0013427734375000, -0.0015563964843750, 0.0003967285156250,
+-0.0025329589843750, 0.0006713867187500, -0.0026245117187500, 0.0003967285156250,
+-0.0009765625000000, -0.0002746582031250, -0.0022277832031250, 0.0005493164062500,
+-0.0015563964843750, -0.0006103515625000, -0.0010375976562500, -0.0004272460937500,
+-0.0016784667968750, -0.0003662109375000, -0.0019226074218750, -0.0011596679687500,
+-0.0012817382812500, -0.0009155273437500, -0.0021057128906250, -0.0007629394531250,
+-0.0018615722656250, -0.0009765625000000, -0.0013427734375000, -0.0010070800781250,
+-0.0016479492187500, -0.0006713867187500, -0.0016479492187500, -0.0006103515625000,
+-0.0007019042968750, -0.0012512207031250, -0.0006408691406250, -0.0007629394531250,
+-0.0011596679687500, -0.0011291503906250, -0.0005187988281250, -0.0018310546875000,
+0.0007324218750000, -0.0009765625000000, -0.0004882812500000, -0.0017089843750000,
+0.0015563964843750, -0.0014648437500000, 0.0014343261718750, -0.0009765625000000,
+0.0003051757812500, -0.0007629394531250, 0.0029602050781250, -0.0001220703125000,
+0.0011901855468750, -0.0005187988281250, 0.0018005371093750, 0.0001220703125000,
+0.0027770996093750, 0.0007324218750000, 0.0017089843750000, 0.0007934570312500,
+0.0016174316406250, 0.0010986328125000, 0.0025939941406250, 0.0008850097656250,
+0.0025024414062500, 0.0011596679687500, 0.0019531250000000, 0.0009155273437500,
+0.0022277832031250, 0.0003662109375000, 0.0023803710937500, 0.0007629394531250,
+0.0010375976562500, -0.0002746582031250, 0.0007019042968750, -0.0000915527343750,
+0.0011596679687500, -0.0001831054687500, 0.0004272460937500, -0.0007324218750000,
+-0.0004882812500000, -0.0001525878906250, 0.0016174316406250, -0.0001831054687500,
+0.0002746582031250, -0.0002136230468750, 0.0004882812500000, 0.0002746582031250,
+0.0033569335937500, 0.0009460449218750, 0.0028076171875000, 0.0014648437500000,
+0.0018920898437500, 0.0019836425781250, 0.0024719238281250, 0.0017089843750000,
+0.0023803710937500, 0.0019531250000000, 0.0017395019531250, 0.0020446777343750,
+0.0021972656250000, 0.0011596679687500, 0.0017395019531250, 0.0018310546875000,
+0.0018615722656250, 0.0013732910156250, 0.0019226074218750, 0.0011291503906250,
+0.0017700195312500, 0.0017395019531250, 0.0019836425781250, 0.0017700195312500,
+0.0015258789062500, 0.0021362304687500, 0.0015563964843750, 0.0026550292968750,
+0.0016174316406250, 0.0028991699218750, 0.0012207031250000, 0.0031738281250000,
+0.0010681152343750, 0.0034484863281250, 0.0015258789062500, 0.0030517578125000,
+0.0012207031250000, 0.0034484863281250, 0.0009765625000000, 0.0033569335937500,
+0.0013122558593750, 0.0025939941406250, 0.0012207031250000, 0.0027160644531250,
+0.0003356933593750, 0.0021972656250000, 0.0006408691406250, 0.0017089843750000,
+0.0001525878906250, 0.0015258789062500, -0.0005798339843750, 0.0010681152343750,
+0.0000610351562500, 0.0006408691406250, -0.0011596679687500, 0.0011596679687500,
+-0.0006713867187500, 0.0004577636718750, -0.0009155273437500, 0.0006408691406250,
+-0.0019836425781250, 0.0011291503906250, -0.0009460449218750, 0.0000305175781250,
+-0.0014953613281250, 0.0007629394531250, -0.0019531250000000, 0.0001525878906250,
+-0.0010681152343750, -0.0003967285156250, -0.0007019042968750, -0.0005187988281250,
+-0.0014038085937500, -0.0014953613281250, 0.0005187988281250, -0.0013427734375000,
+0.0001220703125000, -0.0020751953125000, 0.0002136230468750, -0.0022888183593750,
+0.0012207031250000, -0.0020751953125000, 0.0004577636718750, -0.0029296875000000,
+0.0010070800781250, -0.0021057128906250, 0.0007934570312500, -0.0025329589843750,
+0.0008239746093750, -0.0023193359375000, 0.0011596679687500, -0.0015869140625000,
+0.0007629394531250, -0.0022888183593750, 0.0016479492187500, -0.0014343261718750,
+0.0014648437500000, -0.0016479492187500, 0.0019226074218750, -0.0015258789062500,
+0.0028076171875000, -0.0011596679687500, 0.0022583007812500, -0.0017395019531250,
+0.0028076171875000, -0.0008239746093750, 0.0032958984375000, -0.0015258789062500,
+0.0026245117187500, -0.0013427734375000, 0.0022888183593750, -0.0005493164062500,
+0.0028991699218750, -0.0016174316406250, 0.0014343261718750, -0.0003967285156250,
+0.0018310546875000, -0.0008544921875000, 0.0017395019531250, -0.0008239746093750,
+0.0007324218750000, -0.0001831054687500, 0.0013122558593750, -0.0011596679687500,
+0.0006408691406250, -0.0001831054687500, 0.0005493164062500, -0.0009765625000000,
+0.0005493164062500, -0.0010070800781250, 0.0002136230468750, -0.0004882812500000,
+0.0003967285156250, -0.0018920898437500, 0.0000915527343750, -0.0010070800781250,
+0.0001220703125000, -0.0017395019531250, 0.0001831054687500, -0.0021667480468750,
+-0.0000915527343750, -0.0017395019531250, -0.0001220703125000, -0.0024108886718750,
+0.0002136230468750, -0.0021362304687500, 0.0000000000000000, -0.0023193359375000,
+0.0001525878906250, -0.0025939941406250, 0.0005798339843750, -0.0024108886718750,
+0.0001525878906250, -0.0023498535156250, 0.0006713867187500, -0.0023193359375000,
+0.0005798339843750, -0.0022277832031250, 0.0003356933593750, -0.0021057128906250,
+0.0005187988281250, -0.0021667480468750, 0.0004272460937500, -0.0022277832031250,
+0.0000610351562500, -0.0016174316406250, 0.0002136230468750, -0.0017089843750000,
+0.0002136230468750, -0.0015869140625000, 0.0000915527343750, -0.0010070800781250,
+0.0003051757812500, -0.0011901855468750, 0.0001525878906250, -0.0011901855468750,
+0.0003051757812500, -0.0007629394531250, 0.0001831054687500, -0.0010375976562500,
+-0.0000610351562500, -0.0012512207031250, 0.0001831054687500, -0.0006103515625000,
+-0.0003662109375000, -0.0015258789062500, -0.0002441406250000, -0.0012207031250000,
+-0.0002746582031250},
+{0.0018005371093750, 0.0027160644531250, 0.0020751953125000, 0.0026855468750000,
+0.0024414062500000, 0.0025939941406250, 0.0022888183593750, 0.0028991699218750,
+0.0021362304687500, 0.0022583007812500, 0.0018920898437500, 0.0016174316406250,
+0.0018615722656250, 0.0018005371093750, 0.0016784667968750, 0.0014953613281250,
+0.0010986328125000, 0.0005187988281250, 0.0010375976562500, 0.0016174316406250,
+0.0011596679687500, 0.0009460449218750, 0.0003967285156250, 0.0008239746093750,
+-0.0000915527343750, 0.0018005371093750, 0.0003662109375000, 0.0009765625000000,
+-0.0012817382812500, 0.0010375976562500, -0.0009155273437500, 0.0011291503906250,
+-0.0010681152343750, 0.0004272460937500, -0.0020446777343750, -0.0001831054687500,
+-0.0012207031250000, -0.0002441406250000, -0.0017089843750000, -0.0013427734375000,
+-0.0016174316406250, -0.0016784667968750, -0.0010986328125000, -0.0017700195312500,
+-0.0010681152343750, -0.0021667480468750, -0.0009765625000000, -0.0023193359375000,
+0.0000305175781250, -0.0023498535156250, -0.0003967285156250, -0.0023193359375000,
+0.0001220703125000, -0.0024719238281250, 0.0010070800781250, -0.0025634765625000,
+0.0002746582031250, -0.0025024414062500, 0.0013427734375000, -0.0028686523437500,
+0.0011596679687500, -0.0027160644531250, 0.0009765625000000, -0.0026550292968750,
+0.0013122558593750, -0.0029907226562500, 0.0009155273437500, -0.0029602050781250,
+0.0012817382812500, -0.0023193359375000, 0.0010681152343750, -0.0026550292968750,
+0.0011291503906250, -0.0022277832031250, 0.0014038085937500, -0.0013732910156250,
+0.0012512207031250, -0.0019531250000000, 0.0021362304687500, -0.0011596679687500,
+0.0021057128906250, -0.0010681152343750, 0.0021667480468750, -0.0011291503906250,
+0.0028686523437500, -0.0010375976562500, 0.0028991699218750, -0.0011596679687500,
+0.0021972656250000, -0.0008850097656250, 0.0025939941406250, -0.0014038085937500,
+0.0021362304687500, -0.0013732910156250, 0.0012817382812500, -0.0013427734375000,
+0.0018615722656250, -0.0023193359375000, 0.0008239746093750, -0.0010070800781250,
+0.0007019042968750, -0.0022888183593750, 0.0007629394531250, -0.0021362304687500,
+0.0002746582031250, -0.0010986328125000, 0.0003356933593750, -0.0028686523437500,
+0.0001831054687500, -0.0014648437500000, 0.0001220703125000, -0.0019226074218750,
+0.0000305175781250, -0.0021362304687500, -0.0001525878906250, -0.0012512207031250,
+-0.0001525878906250, -0.0018615722656250, -0.0001831054687500, -0.0007019042968750,
+-0.0003356933593750, -0.0009765625000000, -0.0003051757812500, -0.0011901855468750,
+-0.0002746582031250, -0.0005493164062500, -0.0003662109375000, -0.0007019042968750,
+-0.0001831054687500, -0.0007629394531250, -0.0005493164062500, -0.0007934570312500,
+-0.0003967285156250, -0.0006713867187500, -0.0000915527343750, -0.0007629394531250,
+-0.0004882812500000, -0.0011291503906250, -0.0000610351562500, -0.0006713867187500,
+-0.0000610351562500, -0.0009155273437500, -0.0001525878906250, -0.0011596679687500,
+-0.0000305175781250, -0.0009460449218750, -0.0000305175781250, -0.0010070800781250,
+-0.0002136230468750, -0.0014038085937500, -0.0001525878906250, -0.0012817382812500,
+-0.0001525878906250, -0.0012817382812500, -0.0002136230468750, -0.0014648437500000,
+-0.0002136230468750, -0.0011596679687500, -0.0003356933593750, -0.0013427734375000,
+-0.0002441406250000, -0.0012512207031250, -0.0003967285156250, -0.0011596679687500,
+-0.0004577636718750, -0.0011901855468750, -0.0002746582031250, -0.0010986328125000,
+-0.0007019042968750, -0.0011596679687500, -0.0004272460937500, -0.0010986328125000,
+-0.0004577636718750, -0.0009765625000000, -0.0007629394531250, -0.0009765625000000,
+-0.0002746582031250, -0.0009460449218750, -0.0005798339843750, -0.0006713867187500,
+-0.0004272460937500, -0.0007934570312500, -0.0002746582031250, -0.0007934570312500,
+-0.0004577636718750, -0.0006103515625000, -0.0003051757812500, -0.0008239746093750,
+-0.0005187988281250, -0.0009765625000000, -0.0006103515625000, -0.0007019042968750,
+-0.0006103515625000, -0.0011596679687500, -0.0007629394531250, -0.0014953613281250,
+-0.0010070800781250, -0.0008239746093750, -0.0008544921875000, -0.0024414062500000,
+-0.0010681152343750, -0.0016174316406250, -0.0010986328125000, -0.0019836425781250,
+-0.0009460449218750, -0.0032653808593750, -0.0010375976562500, -0.0019531250000000,
+-0.0006103515625000, -0.0035095214843750, -0.0007019042968750, -0.0033264160156250,
+-0.0004882812500000, -0.0030822753906250, -0.0002136230468750, -0.0038757324218750,
+-0.0004577636718750, -0.0035400390625000, -0.0000915527343750, -0.0036621093750000,
+-0.0003967285156250, -0.0038757324218750, -0.0002136230468750, -0.0037536621093750,
+0.0000305175781250, -0.0036621093750000, -0.0005493164062500, -0.0038452148437500,
+0.0004577636718750, -0.0038452148437500, 0.0000915527343750, -0.0035400390625000,
+0.0001831054687500, -0.0039062500000000, 0.0008850097656250, -0.0040283203125000,
+0.0003967285156250, -0.0034484863281250, 0.0009765625000000, -0.0051879882812500,
+0.0008544921875000, -0.0044860839843750, 0.0010986328125000, -0.0048828125000000,
+0.0015869140625000, -0.0063781738281250, 0.0010375976562500, -0.0053100585937500,
+0.0015869140625000, -0.0065612792968750, 0.0015869140625000, -0.0066528320312500,
+0.0012512207031250, -0.0060119628906250, 0.0013427734375000, -0.0063781738281250,
+0.0013427734375000, -0.0064086914062500, 0.0010986328125000, -0.0046997070312500,
+0.0011291503906250, -0.0059509277343750, 0.0013427734375000, -0.0050964355468750,
+0.0012512207031250, -0.0032653808593750, 0.0010986328125000, -0.0050048828125000,
+0.0016479492187500, -0.0026550292968750, 0.0016479492187500, -0.0032043457031250,
+0.0015869140625000, -0.0031738281250000, 0.0018005371093750, -0.0019226074218750,
+0.0017700195312500, -0.0034179687500000, 0.0014038085937500, -0.0019836425781250,
+0.0010986328125000, -0.0025024414062500, 0.0009460449218750, -0.0027465820312500,
+0.0006713867187500, -0.0020751953125000, 0.0002441406250000, -0.0027770996093750,
+0.0007324218750000, -0.0020141601562500, 0.0008239746093750, -0.0024719238281250,
+0.0006408691406250, -0.0028076171875000, 0.0008544921875000, -0.0023498535156250,
+0.0012207031250000, -0.0023498535156250, 0.0004882812500000, -0.0014343261718750,
+0.0006713867187500, -0.0009765625000000, 0.0003051757812500, -0.0007019042968750,
+-0.0005493164062500, -0.0000915527343750, -0.0002441406250000, 0.0003967285156250,
+-0.0008239746093750, -0.0000915527343750, -0.0009460449218750, 0.0001831054687500,
+-0.0009765625000000, 0.0003051757812500, -0.0010070800781250, -0.0003051757812500,
+-0.0007324218750000, -0.0002746582031250, -0.0017700195312500, 0.0007324218750000,
+-0.0009765625000000, 0.0001525878906250, -0.0014953613281250, 0.0001525878906250,
+-0.0026550292968750, 0.0012512207031250, -0.0013427734375000, 0.0006408691406250,
+-0.0025939941406250, -0.0005798339843750, -0.0024108886718750, 0.0000305175781250,
+-0.0017089843750000, -0.0004272460937500, -0.0020141601562500, -0.0024719238281250,
+-0.0039062500000000, 0.0003051757812500, -0.0015563964843750, -0.0023193359375000,
+-0.0034484863281250, -0.0011291503906250, -0.0029296875000000, -0.0017395019531250,
+-0.0016174316406250, -0.0037536621093750, -0.0027770996093750, -0.0036621093750000,
+-0.0010986328125000, -0.0029602050781250, -0.0010375976562500, -0.0035705566406250,
+-0.0012512207031250, -0.0035705566406250, -0.0004272460937500, -0.0025024414062500,
+-0.0006713867187500, -0.0025939941406250, -0.0013732910156250, -0.0031738281250000,
+-0.0015258789062500, -0.0024414062500000, -0.0016479492187500, -0.0025329589843750,
+-0.0024414062500000, -0.0035095214843750, -0.0029296875000000, -0.0029907226562500,
+-0.0016174316406250, -0.0018005371093750, -0.0028381347656250, -0.0024719238281250,
+-0.0023498535156250, -0.0016479492187500, -0.0007629394531250, 0.0000000000000000,
+-0.0024108886718750, -0.0006713867187500, -0.0016479492187500, -0.0000610351562500,
+-0.0014038085937500, 0.0004882812500000, -0.0023803710937500, -0.0001525878906250,
+-0.0029602050781250, -0.0005187988281250, -0.0028991699218750, 0.0001220703125000,
+-0.0034790039062500, -0.0003356933593750, -0.0038757324218750, -0.0005187988281250,
+-0.0036621093750000, 0.0001831054687500, -0.0035705566406250, 0.0005493164062500,
+-0.0038452148437500, 0.0000305175781250, -0.0028076171875000, 0.0007934570312500,
+-0.0027770996093750, 0.0010070800781250, -0.0030822753906250, 0.0007324218750000,
+-0.0023193359375000, 0.0007019042968750, -0.0020751953125000, 0.0008239746093750,
+-0.0041198730468750, 0.0006408691406250, -0.0028381347656250, 0.0004272460937500,
+-0.0035705566406250, 0.0006713867187500, -0.0054321289062500, 0.0009765625000000,
+-0.0034790039062500, 0.0006713867187500, -0.0057373046875000, 0.0008544921875000,
+-0.0049133300781250, 0.0010375976562500, -0.0043640136718750, 0.0007324218750000,
+-0.0054626464843750, 0.0005187988281250, -0.0041503906250000, 0.0008239746093750,
+-0.0039062500000000, 0.0006103515625000, -0.0039062500000000, 0.0005187988281250,
+-0.0032653808593750, 0.0007324218750000, -0.0025939941406250, 0.0008850097656250,
+-0.0025939941406250, 0.0007019042968750, -0.0028076171875000, 0.0004577636718750,
+-0.0022583007812500, 0.0006713867187500, -0.0026550292968750, 0.0003356933593750,
+-0.0031433105468750, -0.0002441406250000, -0.0024414062500000, -0.0000610351562500,
+-0.0038452148437500, -0.0002441406250000, -0.0030517578125000, -0.0004882812500000,
+-0.0033569335937500, -0.0004577636718750, -0.0043640136718750, -0.0004577636718750,
+-0.0029296875000000, -0.0007324218750000, -0.0041198730468750, -0.0007019042968750,
+-0.0038452148437500, -0.0008544921875000, -0.0030212402343750, -0.0009765625000000,
+-0.0033569335937500, -0.0010681152343750, -0.0032348632812500, -0.0012207031250000,
+-0.0020446777343750, -0.0011596679687500, -0.0022583007812500, -0.0012207031250000,
+-0.0017700195312500, -0.0013122558593750, -0.0007934570312500, -0.0012817382812500,
+-0.0012512207031250, -0.0012207031250000, -0.0003356933593750, -0.0014648437500000,
+-0.0002746582031250, -0.0014648437500000, -0.0000610351562500, -0.0014343261718750,
+0.0004272460937500, -0.0014953613281250, 0.0001831054687500, -0.0014953613281250,
+0.0010986328125000, -0.0016479492187500, 0.0006408691406250, -0.0015258789062500,
+0.0006713867187500, -0.0016784667968750, 0.0011901855468750, -0.0018920898437500,
+0.0002746582031250, -0.0015869140625000, 0.0002136230468750, -0.0021057128906250,
+0.0000915527343750, -0.0019226074218750, -0.0004577636718750, -0.0019531250000000,
+-0.0007629394531250, -0.0022277832031250, -0.0007629394531250, -0.0019226074218750,
+-0.0015563964843750, -0.0023193359375000, -0.0012512207031250, -0.0022583007812500,
+-0.0015258789062500, -0.0022277832031250, -0.0024719238281250, -0.0024719238281250,
+-0.0021667480468750, -0.0022888183593750, -0.0026855468750000, -0.0022583007812500,
+-0.0030517578125000, -0.0022888183593750, -0.0029907226562500, -0.0020751953125000,
+-0.0033264160156250, -0.0018615722656250, -0.0039672851562500, -0.0018615722656250,
+-0.0032348632812500, -0.0014038085937500, -0.0038757324218750, -0.0013122558593750,
+-0.0036926269531250, -0.0011596679687500, -0.0031127929687500, -0.0008850097656250,
+-0.0039978027343750, -0.0008239746093750, -0.0026550292968750, -0.0007934570312500,
+-0.0031433105468750, -0.0007019042968750, -0.0028686523437500, -0.0006103515625000,
+-0.0019836425781250, -0.0005187988281250, -0.0028991699218750, -0.0003967285156250,
+-0.0013122558593750, -0.0003967285156250, -0.0020751953125000, -0.0002746582031250,
+-0.0018615722656250, -0.0001220703125000, -0.0006713867187500, -0.0000610351562500,
+-0.0019226074218750},
+{-0.0015869140625000, -0.0004577636718750, -0.0008850097656250, 0.0000610351562500,
+-0.0031127929687500, 0.0001831054687500, -0.0021057128906250, -0.0006408691406250,
+-0.0027770996093750, -0.0000915527343750, -0.0047302246093750, 0.0009460449218750,
+-0.0030517578125000, 0.0002441406250000, -0.0042114257812500, 0.0008544921875000,
+-0.0040283203125000, 0.0012207031250000, -0.0034790039062500, 0.0008544921875000,
+-0.0036010742187500, 0.0005187988281250, -0.0029602050781250, 0.0006713867187500,
+-0.0027770996093750, 0.0006103515625000, -0.0028381347656250, 0.0002441406250000,
+-0.0025024414062500, 0.0005187988281250, -0.0023193359375000, 0.0009765625000000,
+-0.0027465820312500, 0.0006408691406250, -0.0026855468750000, 0.0008239746093750,
+-0.0030212402343750, 0.0010375976562500, -0.0032043457031250, 0.0007934570312500,
+-0.0033569335937500, 0.0005493164062500, -0.0038146972656250, 0.0007934570312500,
+-0.0034179687500000, 0.0006408691406250, -0.0036926269531250, 0.0004882812500000,
+-0.0035705566406250, 0.0005493164062500, -0.0028991699218750, 0.0005798339843750,
+-0.0028686523437500, 0.0004882812500000, -0.0023803710937500, 0.0003356933593750,
+-0.0018920898437500, 0.0003356933593750, -0.0015563964843750, 0.0002441406250000,
+-0.0011901855468750, 0.0000915527343750, -0.0007629394531250, 0.0000305175781250,
+-0.0004577636718750, -0.0000610351562500, -0.0001831054687500, -0.0000915527343750,
+-0.0000915527343750, -0.0003051757812500, -0.0000305175781250, -0.0004882812500000,
+0.0000305175781250, -0.0004272460937500, -0.0004882812500000, -0.0008544921875000,
+-0.0002746582031250, -0.0009460449218750, -0.0003967285156250, -0.0009155273437500,
+-0.0009460449218750, -0.0010681152343750, -0.0005493164062500, -0.0011596679687500,
+-0.0009460449218750, -0.0011901855468750, -0.0011291503906250, -0.0011596679687500,
+-0.0008544921875000, -0.0014343261718750, -0.0009460449218750, -0.0014953613281250,
+-0.0013122558593750, -0.0013427734375000, -0.0003051757812500, -0.0019531250000000,
+-0.0009765625000000, -0.0018005371093750, -0.0012817382812500, -0.0020141601562500,
+-0.0007934570312500, -0.0025024414062500, -0.0015869140625000, -0.0021362304687500,
+-0.0014648437500000, -0.0027160644531250, -0.0014953613281250, -0.0026855468750000,
+-0.0018005371093750, -0.0025939941406250, -0.0018310546875000, -0.0028686523437500,
+-0.0016479492187500, -0.0027770996093750, -0.0022888183593750, -0.0026245117187500,
+-0.0021057128906250, -0.0026245117187500, -0.0020751953125000, -0.0025939941406250,
+-0.0028381347656250, -0.0024414062500000, -0.0031738281250000, -0.0022888183593750,
+-0.0027465820312500, -0.0021972656250000, -0.0033569335937500, -0.0021362304687500,
+-0.0033264160156250, -0.0019226074218750, -0.0030212402343750, -0.0017395019531250,
+-0.0038146972656250, -0.0017089843750000, -0.0030517578125000, -0.0014343261718750,
+-0.0036315917968750, -0.0012817382812500, -0.0035400390625000, -0.0012512207031250,
+-0.0030517578125000, -0.0011596679687500, -0.0041503906250000, -0.0010986328125000,
+-0.0028991699218750, -0.0010681152343750, -0.0035095214843750, -0.0010375976562500,
+-0.0035095214843750, -0.0010375976562500, -0.0026855468750000, -0.0010070800781250,
+-0.0036315917968750, -0.0009155273437500, -0.0027160644531250, -0.0008239746093750,
+-0.0032043457031250, -0.0008544921875000, -0.0031738281250000, -0.0007324218750000,
+-0.0027160644531250, -0.0005187988281250, -0.0036315917968750, -0.0004577636718750,
+-0.0028381347656250, -0.0003051757812500, -0.0033569335937500, -0.0002136230468750,
+-0.0036926269531250, -0.0001220703125000, -0.0032958984375000, -0.0000915527343750,
+-0.0039062500000000, -0.0000305175781250, -0.0038452148437500, 0.0001831054687500,
+-0.0037841796875000, 0.0001525878906250, -0.0039062500000000, 0.0004577636718750,
+-0.0039367675781250, 0.0007324218750000, -0.0037536621093750, 0.0005798339843750,
+-0.0035705566406250, 0.0013122558593750, -0.0033569335937500, 0.0015258789062500,
+-0.0031127929687500, 0.0015258789062500, -0.0028076171875000, 0.0018005371093750,
+-0.0023498535156250, 0.0019226074218750, -0.0025329589843750, 0.0017395019531250,
+-0.0024414062500000, 0.0018310546875000, -0.0026550292968750, 0.0018005371093750,
+-0.0023803710937500, 0.0017395019531250, -0.0014038085937500, 0.0020446777343750,
+-0.0032043457031250, 0.0018310546875000, -0.0019226074218750, 0.0018920898437500,
+-0.0025939941406250, 0.0021362304687500, -0.0043029785156250, 0.0021057128906250,
+-0.0024414062500000, 0.0019531250000000, -0.0048217773437500, 0.0019531250000000,
+-0.0045471191406250, 0.0019531250000000, -0.0038146972656250, 0.0015563964843750,
+-0.0048522949218750, 0.0012512207031250, -0.0046386718750000, 0.0013732910156250,
+-0.0031127929687500, 0.0005493164062500, -0.0044860839843750, 0.0006408691406250,
+-0.0035095214843750, 0.0003967285156250, -0.0019531250000000, -0.0002441406250000,
+-0.0042114257812500, -0.0000305175781250, -0.0014343261718750, -0.0003051757812500,
+-0.0019226074218750, -0.0007019042968750, -0.0023498535156250, -0.0005493164062500,
+-0.0009460449218750, -0.0005493164062500, -0.0016479492187500, -0.0009765625000000,
+-0.0014038085937500, -0.0000610351562500, -0.0014953613281250, -0.0006713867187500,
+-0.0010986328125000, -0.0007324218750000, -0.0009460449218750, -0.0001831054687500,
+-0.0018310546875000, -0.0009155273437500, -0.0001525878906250, -0.0003356933593750,
+-0.0010986328125000, -0.0010375976562500, -0.0009765625000000, -0.0012207031250000,
+0.0003356933593750, -0.0006408691406250, -0.0011596679687500, -0.0011596679687500,
+-0.0004882812500000, -0.0008544921875000, -0.0000610351562500, -0.0011901855468750,
+-0.0008544921875000, -0.0008544921875000, -0.0009155273437500, -0.0002136230468750,
+0.0000000000000000, -0.0005187988281250, -0.0011596679687500, 0.0000305175781250,
+-0.0013427734375000, 0.0005187988281250, -0.0006713867187500, 0.0000915527343750,
+-0.0007934570312500, -0.0001220703125000, -0.0008544921875000, 0.0006103515625000,
+0.0002136230468750, -0.0004272460937500, 0.0000915527343750, -0.0002441406250000,
+-0.0001525878906250, -0.0000915527343750, 0.0004882812500000, -0.0006713867187500,
+0.0005187988281250, -0.0004272460937500, -0.0007019042968750, -0.0003662109375000,
+0.0000915527343750, -0.0008850097656250, -0.0000915527343750, -0.0007934570312500,
+-0.0013732910156250, -0.0004882812500000, -0.0004272460937500, -0.0007324218750000,
+0.0000610351562500, -0.0000305175781250, -0.0003967285156250, -0.0004882812500000,
+-0.0005493164062500, -0.0000305175781250, -0.0003662109375000, 0.0009155273437500,
+-0.0004272460937500, 0.0003051757812500, -0.0002746582031250, 0.0015869140625000,
+-0.0002746582031250, 0.0012817382812500, -0.0000915527343750, 0.0008544921875000,
+0.0003051757812500, 0.0011901855468750, 0.0002136230468750, 0.0007324218750000,
+-0.0000610351562500, 0.0006713867187500, 0.0004577636718750, 0.0003356933593750,
+0.0000305175781250, 0.0002441406250000, -0.0004882812500000, 0.0005493164062500,
+0.0003051757812500, 0.0003051757812500, -0.0005798339843750, -0.0001831054687500,
+-0.0002136230468750, 0.0005798339843750, 0.0000610351562500, 0.0000305175781250,
+-0.0003051757812500, -0.0007629394531250, 0.0002746582031250, 0.0003662109375000,
+-0.0001525878906250, -0.0014343261718750, -0.0000915527343750, -0.0012512207031250,
+-0.0000610351562500, -0.0013122558593750, -0.0004882812500000, -0.0023193359375000,
+-0.0005798339843750, -0.0019836425781250, -0.0001220703125000, -0.0030517578125000,
+-0.0008239746093750, -0.0024719238281250, -0.0007324218750000, -0.0030517578125000,
+-0.0000610351562500, -0.0040283203125000, -0.0007934570312500, -0.0026245117187500,
+-0.0004882812500000, -0.0044860839843750, -0.0005187988281250, -0.0039367675781250,
+-0.0007934570312500, -0.0034484863281250, -0.0009460449218750, -0.0042419433593750,
+-0.0009460449218750, -0.0033874511718750, -0.0007019042968750, -0.0030822753906250,
+-0.0012817382812500, -0.0030212402343750, -0.0006103515625000, -0.0025024414062500,
+0.0000610351562500, -0.0020141601562500, -0.0007934570312500, -0.0021057128906250,
+0.0012207031250000, -0.0018615722656250, 0.0009460449218750, -0.0017089843750000,
+0.0008850097656250, -0.0021362304687500, 0.0020446777343750, -0.0023803710937500,
+0.0014953613281250, -0.0020141601562500, 0.0014648437500000, -0.0028381347656250,
+0.0016784667968750, -0.0029907226562500, 0.0014648437500000, -0.0028381347656250,
+0.0010681152343750, -0.0032348632812500, 0.0010681152343750, -0.0033874511718750,
+0.0011596679687500, -0.0027770996093750, 0.0009155273437500, -0.0031433105468750,
+0.0008544921875000, -0.0029602050781250, 0.0009460449218750, -0.0023803710937500,
+0.0007934570312500, -0.0030822753906250, 0.0009460449218750, -0.0028381347656250,
+0.0009155273437500, -0.0029907226562500, 0.0008544921875000, -0.0034179687500000,
+0.0009460449218750, -0.0035400390625000, 0.0009765625000000, -0.0036621093750000,
+0.0009460449218750, -0.0040893554687500, 0.0009765625000000, -0.0041809082031250,
+0.0007324218750000, -0.0039978027343750, 0.0007324218750000, -0.0041809082031250,
+0.0008239746093750, -0.0043029785156250, 0.0001525878906250, -0.0034790039062500,
+0.0002136230468750, -0.0042114257812500, 0.0000305175781250, -0.0038452148437500,
+-0.0005493164062500, -0.0030822753906250, -0.0003967285156250, -0.0041809082031250,
+-0.0006713867187500, -0.0030517578125000, -0.0007629394531250, -0.0031127929687500,
+-0.0010070800781250, -0.0029602050781250, -0.0011291503906250, -0.0020446777343750,
+-0.0009460449218750, -0.0023498535156250, -0.0014953613281250, -0.0019226074218750,
+-0.0011901855468750, -0.0016784667968750, -0.0012817382812500, -0.0016784667968750,
+-0.0017089843750000, -0.0014953613281250, -0.0012207031250000, -0.0013732910156250,
+-0.0014953613281250, -0.0015869140625000, -0.0013732910156250, -0.0014343261718750,
+-0.0013122558593750, -0.0014648437500000, -0.0014038085937500, -0.0017395019531250,
+-0.0011596679687500, -0.0015563964843750, -0.0013732910156250, -0.0014953613281250,
+-0.0012207031250000, -0.0017395019531250, -0.0013122558593750, -0.0014648437500000,
+-0.0015258789062500, -0.0013122558593750, -0.0012817382812500, -0.0017395019531250,
+-0.0015869140625000, -0.0011291503906250, -0.0015869140625000, -0.0013427734375000,
+-0.0014953613281250, -0.0013732910156250, -0.0016479492187500, -0.0009460449218750,
+-0.0014953613281250, -0.0011596679687500, -0.0014343261718750, -0.0009460449218750,
+-0.0016479492187500, -0.0007324218750000, -0.0015258789062500, -0.0008544921875000,
+-0.0014953613281250, -0.0007629394531250, -0.0019836425781250, -0.0003662109375000,
+-0.0016174316406250, -0.0010986328125000, -0.0019226074218750, -0.0002746582031250,
+-0.0021057128906250, -0.0005493164062500, -0.0019531250000000, -0.0011596679687500,
+-0.0021362304687500, 0.0001525878906250, -0.0019836425781250, -0.0014343261718750,
+-0.0019226074218750, -0.0007629394531250, -0.0017700195312500, -0.0009765625000000,
+-0.0015563964843750, -0.0021057128906250, -0.0014648437500000, -0.0010070800781250,
+-0.0011596679687500, -0.0024108886718750, -0.0013122558593750, -0.0021057128906250,
+-0.0010681152343750, -0.0021972656250000, -0.0008850097656250, -0.0029907226562500,
+-0.0015258789062500, -0.0022583007812500, -0.0004882812500000, -0.0032348632812500,
+-0.0010681152343750, -0.0028381347656250, -0.0008239746093750, -0.0027160644531250,
+0.0000610351562500, -0.0031433105468750, -0.0007629394531250, -0.0025024414062500,
+0.0003967285156250, -0.0026855468750000, 0.0003051757812500, -0.0024414062500000,
+0.0002136230468750, -0.0022583007812500, 0.0007934570312500, -0.0021972656250000,
+0.0004577636718750, -0.0018310546875000, 0.0007324218750000, -0.0024414062500000,
+0.0008850097656250},
+{0.0010681152343750, -0.0028991699218750, 0.0009155273437500, -0.0033264160156250,
+0.0006408691406250, -0.0033264160156250, 0.0012817382812500, -0.0028686523437500,
+0.0007629394531250, -0.0036010742187500, 0.0005798339843750, -0.0031738281250000,
+0.0004577636718750, -0.0036926269531250, 0.0003662109375000, -0.0038452148437500,
+0.0001831054687500, -0.0035705566406250, 0.0000305175781250, -0.0042419433593750,
+0.0001831054687500, -0.0035705566406250, 0.0000610351562500, -0.0038146972656250,
+0.0001831054687500, -0.0035705566406250, 0.0004272460937500, -0.0028991699218750,
+0.0002136230468750, -0.0032653808593750, 0.0004577636718750, -0.0023193359375000,
+0.0006103515625000, -0.0023193359375000, 0.0004882812500000, -0.0022583007812500,
+0.0005187988281250, -0.0018310546875000, 0.0005798339843750, -0.0021057128906250,
+0.0001220703125000, -0.0018005371093750, 0.0003662109375000, -0.0019531250000000,
+0.0000610351562500, -0.0021667480468750, -0.0004272460937500, -0.0021667480468750,
+0.0000305175781250, -0.0025024414062500, -0.0006713867187500, -0.0028381347656250,
+-0.0005798339843750, -0.0028076171875000, -0.0006408691406250, -0.0028686523437500,
+-0.0010070800781250, -0.0030212402343750, -0.0008850097656250, -0.0031433105468750,
+-0.0014038085937500, -0.0034484863281250, -0.0010986328125000, -0.0033264160156250,
+-0.0013732910156250, -0.0036315917968750, -0.0020141601562500, -0.0039978027343750,
+-0.0014648437500000, -0.0036315917968750, -0.0021972656250000, -0.0042114257812500,
+-0.0020751953125000, -0.0043334960937500, -0.0020141601562500, -0.0042419433593750,
+-0.0023193359375000, -0.0044250488281250, -0.0020141601562500, -0.0043945312500000,
+-0.0021972656250000, -0.0043029785156250, -0.0020141601562500, -0.0044860839843750,
+-0.0020446777343750, -0.0043334960937500, -0.0021667480468750, -0.0041503906250000,
+-0.0018310546875000, -0.0043945312500000, -0.0022888183593750, -0.0042724609375000,
+-0.0021362304687500, -0.0042724609375000, -0.0020446777343750, -0.0043945312500000,
+-0.0023193359375000, -0.0044250488281250, -0.0022277832031250, -0.0043945312500000,
+-0.0020446777343750, -0.0045776367187500, -0.0022888183593750, -0.0043334960937500,
+-0.0021972656250000, -0.0043945312500000, -0.0019836425781250, -0.0043945312500000,
+-0.0022277832031250, -0.0039367675781250, -0.0020446777343750, -0.0044250488281250,
+-0.0021362304687500, -0.0039672851562500, -0.0021057128906250, -0.0036926269531250,
+-0.0019836425781250, -0.0038757324218750, -0.0021362304687500, -0.0032348632812500,
+-0.0015869140625000, -0.0030517578125000, -0.0018920898437500, -0.0029296875000000,
+-0.0015869140625000, -0.0024108886718750, -0.0010986328125000, -0.0020141601562500,
+-0.0016479492187500, -0.0019836425781250, -0.0007019042968750, -0.0012207031250000,
+-0.0009155273437500, -0.0013122558593750, -0.0008850097656250, -0.0010986328125000,
+-0.0002746582031250, -0.0005493164062500, -0.0008239746093750, -0.0008850097656250,
+-0.0003051757812500, -0.0004577636718750, -0.0005798339843750, -0.0002746582031250,
+-0.0006103515625000, -0.0007019042968750, -0.0004577636718750, -0.0007324218750000,
+-0.0008239746093750, -0.0005798339843750, -0.0002746582031250, -0.0017700195312500,
+-0.0004577636718750, -0.0012512207031250, -0.0003356933593750, -0.0016784667968750,
+0.0001831054687500, -0.0027770996093750, -0.0000915527343750, -0.0018920898437500,
+0.0000915527343750, -0.0030517578125000, 0.0003356933593750, -0.0028686523437500,
+0.0001525878906250, -0.0027160644531250, -0.0000610351562500, -0.0031433105468750,
+0.0002136230468750, -0.0027770996093750, 0.0000000000000000, -0.0029296875000000,
+0.0000000000000000, -0.0027770996093750, 0.0000305175781250, -0.0029602050781250,
+0.0000000000000000, -0.0030822753906250, 0.0000305175781250, -0.0025634765625000,
+-0.0001220703125000, -0.0034790039062500, -0.0002746582031250, -0.0028991699218750,
+-0.0001525878906250, -0.0034790039062500, -0.0002441406250000, -0.0043945312500000,
+-0.0003967285156250, -0.0034179687500000, 0.0000305175781250, -0.0052490234375000,
+-0.0004272460937500, -0.0046386718750000, -0.0002136230468750, -0.0045776367187500,
+0.0000610351562500, -0.0057983398437500, -0.0009765625000000, -0.0048217773437500,
+0.0002441406250000, -0.0047302246093750, -0.0004272460937500, -0.0047912597656250,
+-0.0003967285156250, -0.0042114257812500, 0.0006713867187500, -0.0035400390625000,
+-0.0001831054687500, -0.0033264160156250, 0.0004272460937500, -0.0032043457031250,
+0.0006713867187500, -0.0026550292968750, 0.0002441406250000, -0.0027770996093750,
+0.0000305175781250, -0.0028991699218750, -0.0001525878906250, -0.0022888183593750,
+-0.0011901855468750, -0.0031433105468750, -0.0009765625000000, -0.0029296875000000,
+-0.0019531250000000, -0.0027465820312500, -0.0032653808593750, -0.0030517578125000,
+-0.0023193359375000, -0.0028686523437500, -0.0035400390625000, -0.0030212402343750,
+-0.0034484863281250, -0.0030822753906250, -0.0034790039062500, -0.0027465820312500,
+-0.0040588378906250, -0.0025329589843750, -0.0037231445312500, -0.0026855468750000,
+-0.0043640136718750, -0.0020446777343750, -0.0044555664062500, -0.0013732910156250,
+-0.0042419433593750, -0.0016479492187500, -0.0046997070312500, -0.0016174316406250,
+-0.0049133300781250, -0.0005187988281250, -0.0032958984375000, -0.0010986328125000,
+-0.0049743652343750, -0.0007934570312500, -0.0033874511718750, 0.0002441406250000,
+-0.0012512207031250, 0.0006103515625000, -0.0041809082031250, 0.0006408691406250,
+0.0003051757812500, 0.0017089843750000, -0.0003051757812500, 0.0020751953125000,
+-0.0015258789062500, 0.0018920898437500, 0.0012817382812500, 0.0019531250000000,
+-0.0005798339843750, 0.0008239746093750, 0.0012817382812500, 0.0023803710937500,
+0.0000610351562500, 0.0024719238281250, 0.0000000000000000, 0.0019531250000000,
+0.0009155273437500, 0.0019226074218750, 0.0002441406250000, 0.0021057128906250,
+-0.0007019042968750, 0.0005798339843750, -0.0003662109375000, 0.0004272460937500,
+-0.0009460449218750, 0.0003967285156250, -0.0023498535156250, -0.0004272460937500,
+-0.0020751953125000, -0.0003967285156250, -0.0019226074218750, -0.0002441406250000,
+-0.0028076171875000, -0.0005798339843750, -0.0019531250000000, -0.0011291503906250,
+-0.0006713867187500, -0.0013427734375000, -0.0017395019531250, -0.0014648437500000,
+-0.0007019042968750, -0.0022583007812500, -0.0000915527343750, -0.0026245117187500,
+-0.0011291503906250, -0.0022888183593750, -0.0014953613281250, -0.0023193359375000,
+-0.0005798339843750, -0.0025939941406250, -0.0023803710937500, -0.0010681152343750,
+-0.0025329589843750, -0.0014648437500000, -0.0019531250000000, -0.0010070800781250,
+-0.0023803710937500, 0.0001831054687500, -0.0024719238281250, -0.0006713867187500,
+-0.0016784667968750, 0.0002441406250000, -0.0017089843750000, -0.0000915527343750,
+-0.0018005371093750, 0.0001525878906250, -0.0014648437500000, 0.0006713867187500,
+-0.0016174316406250, -0.0003051757812500, -0.0021667480468750, 0.0009765625000000,
+-0.0021667480468750, 0.0011596679687500, -0.0025024414062500, 0.0007324218750000,
+-0.0033264160156250, 0.0010375976562500, -0.0033569335937500, 0.0011596679687500,
+-0.0024719238281250, 0.0003967285156250, -0.0038146972656250, -0.0000305175781250,
+-0.0027160644531250, 0.0001525878906250, -0.0011596679687500, -0.0000610351562500,
+-0.0030212402343750, -0.0005187988281250, -0.0001831054687500, 0.0003356933593750,
+-0.0008239746093750, 0.0002441406250000, -0.0009460449218750, 0.0000000000000000,
+0.0007629394531250, 0.0003356933593750, -0.0003356933593750, 0.0003356933593750,
+-0.0000305175781250, -0.0002441406250000, 0.0004272460937500, -0.0003356933593750,
+-0.0003356933593750, -0.0001525878906250, -0.0008239746093750, -0.0003967285156250,
+-0.0002746582031250, -0.0006713867187500, -0.0010375976562500, 0.0001220703125000,
+-0.0010986328125000, -0.0000610351562500, -0.0006408691406250, -0.0000610351562500,
+-0.0007934570312500, 0.0004272460937500, -0.0007934570312500, 0.0002441406250000,
+0.0003051757812500, 0.0002136230468750, -0.0000915527343750, 0.0002746582031250,
+0.0003967285156250, 0.0002441406250000, 0.0013732910156250, 0.0002441406250000,
+0.0007629394531250, 0.0003356933593750, 0.0017700195312500, 0.0001831054687500,
+0.0019836425781250, 0.0003051757812500, 0.0016784667968750, 0.0000915527343750,
+0.0021057128906250, -0.0001831054687500, 0.0024719238281250, 0.0000915527343750,
+0.0014343261718750, -0.0004882812500000, 0.0018615722656250, -0.0005493164062500,
+0.0014343261718750, -0.0007019042968750, 0.0003356933593750, -0.0009155273437500,
+0.0008850097656250, -0.0008239746093750, -0.0003967285156250, -0.0013427734375000,
+-0.0005493164062500, -0.0011596679687500, -0.0009155273437500, -0.0015258789062500,
+-0.0017700195312500, -0.0021362304687500, -0.0018310546875000, -0.0017700195312500,
+-0.0028076171875000, -0.0023803710937500, -0.0030212402343750, -0.0024108886718750,
+-0.0033874511718750, -0.0023498535156250, -0.0040283203125000, -0.0025024414062500,
+-0.0038146972656250, -0.0024719238281250, -0.0042724609375000, -0.0025024414062500,
+-0.0044250488281250, -0.0025329589843750, -0.0043334960937500, -0.0025024414062500,
+-0.0046081542968750, -0.0025634765625000, -0.0047302246093750, -0.0025939941406250,
+-0.0043945312500000, -0.0024108886718750, -0.0049438476562500, -0.0024108886718750,
+-0.0048828125000000, -0.0023803710937500, -0.0040588378906250, -0.0022888183593750,
+-0.0043334960937500, -0.0023193359375000, -0.0040893554687500, -0.0023193359375000,
+-0.0036926269531250, -0.0023498535156250, -0.0038146972656250, -0.0023193359375000,
+-0.0036621093750000, -0.0022888183593750, -0.0029907226562500, -0.0023193359375000,
+-0.0036010742187500, -0.0021667480468750, -0.0031433105468750, -0.0021362304687500,
+-0.0031433105468750, -0.0021057128906250, -0.0036010742187500, -0.0020446777343750,
+-0.0032348632812500, -0.0020141601562500, -0.0037841796875000, -0.0019226074218750,
+-0.0039978027343750, -0.0019531250000000, -0.0040588378906250, -0.0018615722656250,
+-0.0044250488281250, -0.0016784667968750, -0.0046691894531250, -0.0017700195312500,
+-0.0047302246093750, -0.0015869140625000, -0.0048828125000000, -0.0015258789062500,
+-0.0049743652343750, -0.0014953613281250, -0.0049438476562500, -0.0014343261718750,
+-0.0049133300781250, -0.0014648437500000, -0.0045166015625000, -0.0012207031250000,
+-0.0046691894531250, -0.0014038085937500, -0.0043945312500000, -0.0010375976562500,
+-0.0041503906250000, -0.0005798339843750, -0.0047302246093750, -0.0009155273437500,
+-0.0037536621093750, -0.0001525878906250, -0.0041503906250000, 0.0000610351562500,
+-0.0042114257812500, -0.0000305175781250, -0.0035705566406250, 0.0001525878906250,
+-0.0040588378906250, 0.0003662109375000, -0.0035400390625000, 0.0002136230468750,
+-0.0033874511718750, 0.0002441406250000, -0.0034484863281250, 0.0002746582031250,
+-0.0031433105468750, 0.0002746582031250, -0.0029296875000000, 0.0003051757812500,
+-0.0031433105468750, 0.0004577636718750, -0.0024719238281250, 0.0003356933593750,
+-0.0025329589843750, 0.0004272460937500, -0.0027770996093750, 0.0005493164062500,
+-0.0018005371093750, 0.0002441406250000, -0.0028076171875000, 0.0002441406250000,
+-0.0023498535156250, 0.0003051757812500, -0.0025024414062500, 0.0000000000000000,
+-0.0031127929687500, -0.0000305175781250, -0.0021972656250000, 0.0003051757812500,
+-0.0034179687500000, -0.0003356933593750, -0.0025939941406250, 0.0003967285156250,
+-0.0028076171875000, 0.0000305175781250, -0.0035400390625000, -0.0007019042968750,
+-0.0021057128906250, 0.0003967285156250, -0.0037841796875000, -0.0010375976562500,
+-0.0029296875000000, -0.0003662109375000, -0.0027160644531250, -0.0003967285156250,
+-0.0039062500000000},
+{-0.0005187988281250, -0.0000915527343750, -0.0016479492187500, -0.0001525878906250,
+-0.0011291503906250, -0.0000610351562500, -0.0013427734375000, -0.0005187988281250,
+-0.0017395019531250, -0.0003051757812500, -0.0014953613281250, -0.0006713867187500,
+-0.0013732910156250, -0.0012512207031250, -0.0017700195312500, -0.0008850097656250,
+-0.0010070800781250, -0.0016174316406250, -0.0012207031250000, -0.0016174316406250,
+-0.0012817382812500, -0.0016784667968750, -0.0008239746093750, -0.0021057128906250,
+-0.0011291503906250, -0.0019226074218750, -0.0012207031250000, -0.0022277832031250,
+-0.0010375976562500, -0.0022888183593750, -0.0011596679687500, -0.0022888183593750,
+-0.0012207031250000, -0.0024414062500000, -0.0009460449218750, -0.0025329589843750,
+-0.0010986328125000, -0.0022277832031250, -0.0010070800781250, -0.0023498535156250,
+-0.0009765625000000, -0.0022888183593750, -0.0010986328125000, -0.0020446777343750,
+-0.0011596679687500, -0.0022277832031250, -0.0010070800781250, -0.0021667480468750,
+-0.0009460449218750, -0.0021667480468750, -0.0011291503906250, -0.0022888183593750,
+-0.0004882812500000, -0.0023193359375000, 0.0005493164062500, -0.0024719238281250,
+-0.0007019042968750, -0.0023803710937500, 0.0003967285156250, -0.0025024414062500,
+0.0002746582031250, -0.0025634765625000, -0.0005798339843750, -0.0024719238281250,
+0.0011291503906250, -0.0025024414062500, -0.0006713867187500, -0.0024108886718750,
+0.0003051757812500, -0.0023803710937500, 0.0002746582031250, -0.0022888183593750,
+-0.0008544921875000, -0.0021667480468750, 0.0007934570312500, -0.0021972656250000,
+-0.0006103515625000, -0.0021362304687500, -0.0000915527343750, -0.0021057128906250,
+0.0001220703125000, -0.0021362304687500, -0.0007324218750000, -0.0022277832031250,
+-0.0000305175781250, -0.0021057128906250, -0.0004882812500000, -0.0020141601562500,
+-0.0006713867187500, -0.0021972656250000, -0.0003967285156250, -0.0018920898437500,
+-0.0005187988281250, -0.0014953613281250, -0.0007629394531250, -0.0018005371093750,
+0.0000610351562500, -0.0010681152343750, -0.0003051757812500, -0.0010375976562500,
+-0.0003356933593750, -0.0011291503906250, 0.0001220703125000, -0.0007934570312500,
+-0.0005493164062500, -0.0008544921875000, -0.0000305175781250, -0.0009765625000000,
+-0.0006103515625000, -0.0010375976562500, -0.0009155273437500, -0.0009460449218750,
+-0.0009765625000000, -0.0009155273437500, -0.0019836425781250, -0.0010375976562500,
+-0.0019226074218750, -0.0007934570312500, -0.0026855468750000, -0.0007324218750000,
+-0.0030212402343750, -0.0006713867187500, -0.0029907226562500, -0.0006103515625000,
+-0.0037231445312500, -0.0006713867187500, -0.0034484863281250, -0.0004882812500000,
+-0.0036621093750000, -0.0005798339843750, -0.0035705566406250, -0.0006713867187500,
+-0.0032043457031250, -0.0005798339843750, -0.0033874511718750, -0.0006103515625000,
+-0.0026550292968750, -0.0007629394531250, -0.0027770996093750, -0.0004882812500000,
+-0.0024414062500000, -0.0007934570312500, -0.0017395019531250, -0.0010070800781250,
+-0.0021972656250000, -0.0004577636718750, -0.0013732910156250, -0.0015258789062500,
+-0.0011596679687500, -0.0010070800781250, -0.0014953613281250, -0.0009765625000000,
+-0.0011291503906250, -0.0015869140625000, -0.0007324218750000, -0.0006713867187500,
+-0.0020141601562500, -0.0015869140625000, -0.0010986328125000, -0.0008850097656250,
+-0.0015869140625000, -0.0008239746093750, -0.0027160644531250, -0.0013122558593750,
+-0.0011901855468750, -0.0003967285156250, -0.0026550292968750, -0.0015563964843750,
+-0.0026245117187500, -0.0007629394531250, -0.0019836425781250, -0.0007324218750000,
+-0.0025634765625000, -0.0018005371093750, -0.0030517578125000, -0.0008850097656250,
+-0.0023803710937500, -0.0013122558593750, -0.0031127929687500, -0.0009765625000000,
+-0.0036621093750000, -0.0009765625000000, -0.0033569335937500, -0.0012817382812500,
+-0.0038146972656250, -0.0006103515625000, -0.0040588378906250, -0.0013732910156250,
+-0.0039672851562500, -0.0013732910156250, -0.0037841796875000, -0.0012817382812500,
+-0.0034179687500000, -0.0017089843750000, -0.0029296875000000, -0.0017395019531250,
+-0.0028076171875000, -0.0015869140625000, -0.0021667480468750, -0.0022583007812500,
+-0.0017700195312500, -0.0019836425781250, -0.0014648437500000, -0.0018615722656250,
+-0.0007629394531250, -0.0030212402343750, -0.0015563964843750, -0.0013427734375000,
+-0.0009155273437500, -0.0024414062500000, -0.0014343261718750, -0.0023498535156250,
+-0.0021972656250000, -0.0011291503906250, -0.0009155273437500, -0.0028076171875000,
+-0.0024414062500000, -0.0014648437500000, -0.0020751953125000, -0.0014953613281250,
+-0.0016174316406250, -0.0018615722656250, -0.0023498535156250, -0.0014038085937500,
+-0.0019836425781250, -0.0014648437500000, -0.0014343261718750, -0.0012817382812500,
+-0.0020751953125000, -0.0013732910156250, -0.0018920898437500, -0.0008239746093750,
+-0.0004882812500000, -0.0003662109375000, 0.0000610351562500, -0.0009460449218750,
+0.0003662109375000, -0.0000915527343750, 0.0002441406250000, -0.0002136230468750,
+0.0003051757812500, -0.0007324218750000, 0.0006713867187500, -0.0006713867187500,
+0.0007324218750000, -0.0008544921875000, 0.0003051757812500, -0.0019226074218750,
+0.0000305175781250, -0.0016479492187500, 0.0000305175781250, -0.0021667480468750,
+-0.0003967285156250, -0.0031127929687500, -0.0009765625000000, -0.0022277832031250,
+-0.0001831054687500, -0.0033264160156250, -0.0008850097656250, -0.0031127929687500,
+-0.0006103515625000, -0.0032958984375000, 0.0000610351562500, -0.0042114257812500,
+-0.0011596679687500, -0.0036621093750000, -0.0000610351562500, -0.0045471191406250,
+-0.0007019042968750, -0.0046081542968750, -0.0009765625000000, -0.0043640136718750,
+-0.0006408691406250, -0.0045166015625000, -0.0016784667968750, -0.0043029785156250,
+-0.0006103515625000, -0.0038757324218750, -0.0010375976562500, -0.0037536621093750,
+-0.0012817382812500, -0.0032348632812500, -0.0004882812500000, -0.0027160644531250,
+-0.0006713867187500, -0.0026550292968750, -0.0011596679687500, -0.0021057128906250,
+-0.0003967285156250, -0.0022888183593750, -0.0003662109375000, -0.0019531250000000,
+-0.0008850097656250, -0.0017700195312500, -0.0001220703125000, -0.0025024414062500,
+-0.0003662109375000, -0.0012817382812500, -0.0001831054687500, -0.0018615722656250,
+-0.0001525878906250, -0.0018310546875000, -0.0002136230468750, -0.0009460449218750,
+0.0000915527343750, -0.0018310546875000, -0.0010681152343750, -0.0011596679687500,
+-0.0005493164062500, -0.0014038085937500, -0.0010375976562500, -0.0015563964843750,
+-0.0022583007812500, -0.0014038085937500, -0.0014953613281250, -0.0020751953125000,
+-0.0028381347656250, -0.0015258789062500, -0.0029907226562500, -0.0021667480468750,
+-0.0027160644531250, -0.0018920898437500, -0.0032348632812500, -0.0013427734375000,
+-0.0032958984375000, -0.0025939941406250, -0.0027770996093750, -0.0011596679687500,
+-0.0032653808593750, -0.0016784667968750, -0.0027770996093750, -0.0017395019531250,
+-0.0021667480468750, -0.0008850097656250, -0.0028076171875000, -0.0017700195312500,
+-0.0013732910156250, -0.0014953613281250, -0.0018615722656250, -0.0016784667968750,
+-0.0014648437500000, -0.0022277832031250, -0.0003967285156250, -0.0023498535156250,
+-0.0013427734375000, -0.0024108886718750, -0.0003967285156250, -0.0027770996093750,
+-0.0004882812500000, -0.0028076171875000, -0.0007324218750000, -0.0025329589843750,
+-0.0003967285156250, -0.0025329589843750, -0.0007629394531250, -0.0027770996093750,
+-0.0008239746093750, -0.0021057128906250, -0.0008544921875000, -0.0023803710937500,
+-0.0010070800781250, -0.0021362304687500, -0.0009460449218750, -0.0013732910156250,
+-0.0009155273437500, -0.0018920898437500, -0.0011291503906250, -0.0012207031250000,
+-0.0007324218750000, -0.0010681152343750, -0.0009460449218750, -0.0013427734375000,
+-0.0012817382812500, -0.0012207031250000, -0.0007324218750000, -0.0010070800781250,
+-0.0016174316406250, -0.0014648437500000, -0.0012817382812500, -0.0011291503906250,
+-0.0014343261718750, -0.0011291503906250, -0.0020751953125000, -0.0014953613281250,
+-0.0014343261718750, -0.0010681152343750, -0.0021667480468750, -0.0012512207031250,
+-0.0021667480468750, -0.0014343261718750, -0.0021362304687500, -0.0014038085937500,
+-0.0025634765625000, -0.0015258789062500, -0.0025939941406250, -0.0016784667968750,
+-0.0024414062500000, -0.0016174316406250, -0.0027465820312500, -0.0018005371093750,
+-0.0026855468750000, -0.0018310546875000, -0.0025634765625000, -0.0017700195312500,
+-0.0030517578125000, -0.0020751953125000, -0.0026245117187500, -0.0019836425781250,
+-0.0029907226562500, -0.0022277832031250, -0.0029602050781250, -0.0022277832031250,
+-0.0026550292968750, -0.0020751953125000, -0.0032043457031250, -0.0023803710937500,
+-0.0026855468750000, -0.0021362304687500, -0.0028381347656250, -0.0021667480468750,
+-0.0028076171875000, -0.0021972656250000, -0.0023803710937500, -0.0021667480468750,
+-0.0025329589843750, -0.0021972656250000, -0.0022888183593750, -0.0019226074218750,
+-0.0020751953125000, -0.0022277832031250, -0.0021057128906250, -0.0018920898437500,
+-0.0019836425781250, -0.0016479492187500, -0.0016479492187500, -0.0022277832031250,
+-0.0018310546875000, -0.0009460449218750, -0.0016174316406250, -0.0018310546875000,
+-0.0014343261718750, -0.0013122558593750, -0.0015258789062500, -0.0001831054687500,
+-0.0013427734375000, -0.0017089843750000, -0.0013122558593750, 0.0003662109375000,
+-0.0012207031250000, -0.0004577636718750, -0.0011901855468750, -0.0003051757812500,
+-0.0012817382812500, 0.0011291503906250, -0.0012207031250000, -0.0003356933593750,
+-0.0014038085937500, 0.0012512207031250, -0.0015258789062500, 0.0008239746093750,
+-0.0014953613281250, 0.0006713867187500, -0.0014953613281250, 0.0014343261718750,
+-0.0016174316406250, 0.0006103515625000, -0.0014648437500000, 0.0009765625000000,
+-0.0013732910156250, 0.0005798339843750, -0.0014038085937500, 0.0002746582031250,
+-0.0013122558593750, 0.0000915527343750, -0.0012512207031250, -0.0004882812500000,
+-0.0014648437500000, -0.0003356933593750, -0.0015563964843750, -0.0009765625000000,
+-0.0015563964843750, -0.0008850097656250, -0.0016784667968750, -0.0005798339843750,
+-0.0017700195312500, -0.0015563964843750, -0.0014648437500000, -0.0000915527343750,
+-0.0016174316406250, -0.0006408691406250, -0.0013427734375000, -0.0002136230468750,
+-0.0009765625000000, 0.0011901855468750, -0.0013732910156250, 0.0003356933593750,
+-0.0005187988281250, 0.0016479492187500, -0.0008239746093750, 0.0019836425781250,
+-0.0006713867187500, 0.0014953613281250, -0.0000305175781250, 0.0018920898437500,
+-0.0005798339843750, 0.0023193359375000, -0.0002441406250000, 0.0010375976562500,
+-0.0006713867187500, 0.0018310546875000, -0.0007019042968750, 0.0011901855468750,
+-0.0007019042968750, -0.0001525878906250, -0.0016479492187500, 0.0010986328125000,
+-0.0004577636718750, -0.0007629394531250, -0.0009155273437500, -0.0006408691406250,
+-0.0009155273437500, -0.0004577636718750, 0.0001525878906250, -0.0011901855468750,
+-0.0002746582031250, -0.0006408691406250, -0.0002441406250000, -0.0012207031250000,
+-0.0000915527343750, -0.0005798339843750, -0.0003662109375000, -0.0010681152343750,
+-0.0009155273437500, -0.0019531250000000, -0.0011596679687500, -0.0010681152343750,
+-0.0011901855468750, -0.0029907226562500, -0.0016479492187500, -0.0021667480468750,
+-0.0015563964843750, -0.0025024414062500, -0.0014038085937500, -0.0038757324218750,
+-0.0020141601562500, -0.0025024414062500, -0.0012207031250000, -0.0039978027343750,
+-0.0017700195312500, -0.0036621093750000, -0.0021057128906250, -0.0033264160156250,
+-0.0020446777343750, -0.0039367675781250, -0.0028381347656250, -0.0033264160156250,
+-0.0020141601562500},
+{-0.0021057128906250, 0.0000610351562500, -0.0024108886718750, 0.0001525878906250,
+-0.0025939941406250, 0.0004272460937500, -0.0023803710937500, 0.0004272460937500,
+-0.0025024414062500, 0.0002746582031250, -0.0028686523437500, 0.0005187988281250,
+-0.0024719238281250, 0.0003967285156250, -0.0028076171875000, 0.0003662109375000,
+-0.0028991699218750, 0.0004272460937500, -0.0025939941406250, 0.0003356933593750,
+-0.0030212402343750, 0.0001525878906250, -0.0028381347656250, 0.0002441406250000,
+-0.0027465820312500, 0.0000915527343750, -0.0027465820312500, 0.0001220703125000,
+-0.0026245117187500, 0.0001831054687500, -0.0025939941406250, -0.0001220703125000,
+-0.0022583007812500, 0.0003356933593750, -0.0023803710937500, -0.0000610351562500,
+-0.0022583007812500, 0.0001525878906250, -0.0019531250000000, 0.0004882812500000,
+-0.0021057128906250, -0.0004272460937500, -0.0019531250000000, 0.0004272460937500,
+-0.0017700195312500, -0.0004272460937500, -0.0019531250000000, -0.0006713867187500,
+-0.0019531250000000, -0.0003051757812500, -0.0015258789062500, -0.0014953613281250,
+-0.0021667480468750, -0.0008544921875000, -0.0018005371093750, -0.0016174316406250,
+-0.0017700195312500, -0.0019531250000000, -0.0022277832031250, -0.0016174316406250,
+-0.0018005371093750, -0.0025634765625000, -0.0021362304687500, -0.0019836425781250,
+-0.0021667480468750, -0.0024108886718750, -0.0020751953125000, -0.0023193359375000,
+-0.0022277832031250, -0.0018005371093750, -0.0023193359375000, -0.0022888183593750,
+-0.0021057128906250, -0.0012207031250000, -0.0023193359375000, -0.0012817382812500,
+-0.0022888183593750, -0.0009765625000000, -0.0020446777343750, -0.0001831054687500,
+-0.0021667480468750, -0.0003967285156250, -0.0020446777343750, 0.0002441406250000,
+-0.0019531250000000, 0.0003967285156250, -0.0017700195312500, 0.0003051757812500,
+-0.0016784667968750, 0.0004577636718750, -0.0016784667968750, 0.0004577636718750,
+-0.0012207031250000, 0.0000000000000000, -0.0016174316406250, 0.0000305175781250,
+-0.0013122558593750, -0.0002136230468750, -0.0007629394531250, -0.0007324218750000,
+-0.0014038085937500, -0.0006408691406250, -0.0008239746093750, -0.0008239746093750,
+-0.0008850097656250, -0.0011291503906250, -0.0010681152343750, -0.0009155273437500,
+-0.0011291503906250, -0.0007324218750000, -0.0014038085937500, -0.0009765625000000,
+-0.0008544921875000, -0.0001220703125000, -0.0015563964843750, -0.0000915527343750,
+-0.0012207031250000, 0.0000915527343750, -0.0002136230468750, 0.0007934570312500,
+-0.0008239746093750, 0.0006713867187500, -0.0004882812500000, 0.0006713867187500,
+0.0002746582031250, 0.0009155273437500, -0.0002136230468750, 0.0005493164062500,
+-0.0009155273437500, 0.0000000000000000, -0.0004577636718750, 0.0001525878906250,
+-0.0011291503906250, -0.0005187988281250, -0.0015563964843750, -0.0007934570312500,
+-0.0013732910156250, -0.0007324218750000, -0.0013427734375000, -0.0010375976562500,
+-0.0017089843750000, -0.0015258789062500, -0.0012207031250000, -0.0010681152343750,
+-0.0015563964843750, -0.0013732910156250, -0.0016784667968750, -0.0017089843750000,
+-0.0017089843750000, -0.0013427734375000, -0.0023803710937500, -0.0013122558593750,
+-0.0017089843750000, -0.0017700195312500, -0.0028076171875000, -0.0015563964843750,
+-0.0024719238281250, -0.0018920898437500, -0.0015563964843750, -0.0023193359375000,
+-0.0028381347656250, -0.0018920898437500, -0.0006408691406250, -0.0031433105468750,
+-0.0010375976562500, -0.0030212402343750, -0.0007629394531250, -0.0030212402343750,
+0.0006408691406250, -0.0035705566406250, -0.0003967285156250, -0.0030822753906250,
+0.0009155273437500, -0.0029296875000000, 0.0004882812500000, -0.0029296875000000,
+0.0000610351562500, -0.0027160644531250, 0.0007629394531250, -0.0023193359375000,
+0.0002746582031250, -0.0023803710937500, 0.0000915527343750, -0.0030822753906250,
+0.0004577636718750, -0.0026550292968750, 0.0003662109375000, -0.0030212402343750,
+0.0001525878906250, -0.0040283203125000, 0.0003662109375000, -0.0033874511718750,
+-0.0005493164062500, -0.0032043457031250, -0.0002746582031250, -0.0036010742187500,
+-0.0005187988281250, -0.0026855468750000, -0.0020141601562500, -0.0008544921875000,
+-0.0020751953125000, -0.0028686523437500, -0.0026550292968750, -0.0028076171875000,
+-0.0027160644531250, -0.0029907226562500, -0.0031127929687500, -0.0020141601562500,
+-0.0039062500000000, -0.0010681152343750, -0.0042114257812500, -0.0014343261718750,
+-0.0032653808593750, -0.0008239746093750, -0.0043334960937500, -0.0004272460937500,
+-0.0034790039062500, -0.0005798339843750, -0.0018005371093750, -0.0006713867187500,
+-0.0031433105468750, -0.0002441406250000, -0.0016174316406250, -0.0000610351562500,
+-0.0012817382812500, -0.0001831054687500, -0.0022583007812500, 0.0000610351562500,
+-0.0023193359375000, 0.0006713867187500, -0.0021057128906250, 0.0005798339843750,
+-0.0033874511718750, 0.0005187988281250, -0.0036621093750000, 0.0010681152343750,
+-0.0031738281250000, 0.0006713867187500, -0.0034790039062500, 0.0000000000000000,
+-0.0039062500000000, 0.0004272460937500, -0.0018920898437500, -0.0001220703125000,
+-0.0020751953125000, -0.0007629394531250, -0.0020141601562500, -0.0003356933593750,
+-0.0007019042968750, 0.0000305175781250, -0.0008239746093750, -0.0006103515625000,
+-0.0009765625000000, -0.0000915527343750, -0.0007324218750000, 0.0001220703125000,
+-0.0005798339843750, -0.0002746582031250, -0.0008850097656250, -0.0004577636718750,
+-0.0009765625000000, -0.0001831054687500, -0.0002136230468750, -0.0001831054687500,
+-0.0007324218750000, -0.0005493164062500, -0.0003051757812500, 0.0001525878906250,
+0.0006103515625000, 0.0007934570312500, -0.0001220703125000, 0.0001525878906250,
+0.0010070800781250, 0.0014648437500000, 0.0014953613281250, 0.0012817382812500,
+0.0011291503906250, 0.0009460449218750, 0.0014343261718750, 0.0015563964843750,
+0.0019836425781250, 0.0015563964843750, 0.0009460449218750, 0.0008544921875000,
+0.0014343261718750, 0.0011291503906250, 0.0013122558593750, 0.0010375976562500,
+0.0003662109375000, 0.0005493164062500, 0.0009460449218750, 0.0007324218750000,
+0.0007324218750000, 0.0004577636718750, -0.0000305175781250, 0.0005187988281250,
+0.0007629394531250, 0.0004272460937500, 0.0011901855468750, 0.0002441406250000,
+-0.0000305175781250, 0.0003662109375000, 0.0020751953125000, 0.0000610351562500,
+0.0014038085937500, 0.0000305175781250, 0.0013732910156250, 0.0000305175781250,
+0.0025939941406250, -0.0000610351562500, 0.0015563964843750, -0.0000305175781250,
+0.0021362304687500, -0.0002441406250000, 0.0019226074218750, -0.0001220703125000,
+0.0015563964843750, -0.0003051757812500, 0.0016784667968750, -0.0005493164062500,
+0.0010681152343750, -0.0003051757812500, 0.0008544921875000, -0.0007019042968750,
+0.0004272460937500, -0.0005493164062500, 0.0000305175781250, -0.0006408691406250,
+-0.0003662109375000, -0.0008544921875000, -0.0009155273437500, -0.0005187988281250,
+-0.0008239746093750, -0.0011596679687500, -0.0014343261718750, -0.0009765625000000,
+-0.0013427734375000, -0.0010681152343750, -0.0011291503906250, -0.0015258789062500,
+-0.0018920898437500, -0.0012207031250000, -0.0007019042968750, -0.0014648437500000,
+-0.0014038085937500, -0.0014648437500000, -0.0011291503906250, -0.0014953613281250,
+-0.0000305175781250, -0.0016174316406250, -0.0010070800781250, -0.0014953613281250,
+0.0004272460937500, -0.0016784667968750, 0.0002136230468750, -0.0017395019531250,
+0.0003356933593750, -0.0017395019531250, 0.0012817382812500, -0.0018615722656250,
+0.0007019042968750, -0.0018615722656250, 0.0014953613281250, -0.0019226074218750,
+0.0015258789062500, -0.0019531250000000, 0.0014343261718750, -0.0020141601562500,
+0.0018920898437500, -0.0021057128906250, 0.0020446777343750, -0.0021362304687500,
+0.0014953613281250, -0.0021667480468750, 0.0027770996093750, -0.0022583007812500,
+0.0024719238281250, -0.0022277832031250, 0.0014648437500000, -0.0021362304687500,
+0.0027160644531250, -0.0021667480468750, 0.0012207031250000, -0.0021057128906250,
+0.0016479492187500, -0.0020141601562500, 0.0016784667968750, -0.0020446777343750,
+0.0008239746093750, -0.0020751953125000, 0.0014648437500000, -0.0019226074218750,
+0.0006713867187500, -0.0019836425781250, 0.0012817382812500, -0.0020141601562500,
+0.0011596679687500, -0.0019531250000000, 0.0007934570312500, -0.0020751953125000,
+0.0020141601562500, -0.0021362304687500, 0.0007019042968750, -0.0018615722656250,
+0.0019226074218750, -0.0020446777343750, 0.0017700195312500, -0.0020141601562500,
+0.0008239746093750, -0.0017395019531250, 0.0026245117187500, -0.0020141601562500,
+0.0006408691406250, -0.0017089843750000, 0.0014953613281250, -0.0018005371093750,
+0.0014038085937500, -0.0018005371093750, -0.0001220703125000, -0.0016174316406250,
+0.0010681152343750, -0.0017700195312500, -0.0001220703125000, -0.0018920898437500,
+-0.0004272460937500, -0.0018920898437500, -0.0005187988281250, -0.0018920898437500,
+-0.0013427734375000, -0.0019226074218750, -0.0016174316406250, -0.0021057128906250,
+-0.0017089843750000, -0.0019226074218750, -0.0023193359375000, -0.0019836425781250,
+-0.0021057128906250, -0.0020141601562500, -0.0019226074218750, -0.0019531250000000,
+-0.0027770996093750, -0.0020751953125000, -0.0018615722656250, -0.0020751953125000,
+-0.0025329589843750, -0.0020141601562500, -0.0023803710937500, -0.0022277832031250,
+-0.0017395019531250, -0.0022888183593750, -0.0029296875000000, -0.0021057128906250,
+-0.0014343261718750, -0.0026550292968750, -0.0022888183593750, -0.0022888183593750,
+-0.0021667480468750, -0.0025024414062500, -0.0009460449218750, -0.0031127929687500,
+-0.0021057128906250, -0.0025634765625000, -0.0009765625000000, -0.0032958984375000,
+-0.0008544921875000, -0.0033264160156250, -0.0012817382812500, -0.0031127929687500,
+-0.0008239746093750, -0.0032958984375000, -0.0007629394531250, -0.0031433105468750,
+-0.0015563964843750, -0.0030212402343750, -0.0011596679687500, -0.0031738281250000,
+-0.0014343261718750, -0.0027770996093750, -0.0022583007812500, -0.0025939941406250,
+-0.0018005371093750, -0.0032348632812500, -0.0024108886718750, -0.0021667480468750,
+-0.0024108886718750, -0.0026245117187500, -0.0022888183593750, -0.0025634765625000,
+-0.0023803710937500, -0.0019531250000000, -0.0021362304687500, -0.0026550292968750,
+-0.0023803710937500, -0.0017089843750000, -0.0018310546875000, -0.0018920898437500,
+-0.0019226074218750, -0.0020141601562500, -0.0022888183593750, -0.0014953613281250,
+-0.0013427734375000, -0.0016784667968750, -0.0018005371093750, -0.0012512207031250,
+-0.0020446777343750, -0.0016784667968750, -0.0012817382812500, -0.0014038085937500,
+-0.0016479492187500, -0.0009765625000000, -0.0030517578125000, -0.0016174316406250,
+-0.0011291503906250, -0.0007934570312500, -0.0032958984375000, -0.0015258789062500,
+-0.0028991699218750, -0.0015563964843750, -0.0009765625000000, -0.0012512207031250,
+-0.0038146972656250, -0.0026550292968750, -0.0007629394531250, -0.0011291503906250,
+-0.0018310546875000, -0.0021057128906250, -0.0021362304687500, -0.0024108886718750,
+-0.0002441406250000, -0.0014343261718750, -0.0018310546875000, -0.0025329589843750,
+-0.0013732910156250, -0.0017395019531250, -0.0012512207031250, -0.0018310546875000,
+-0.0016174316406250, -0.0018920898437500, -0.0015869140625000, -0.0015563964843750,
+-0.0017395019531250, -0.0018310546875000, -0.0016479492187500, -0.0008544921875000,
+-0.0010375976562500, -0.0012512207031250, -0.0018005371093750, -0.0010681152343750,
+-0.0017089843750000, -0.0002136230468750, 0.0001525878906250, -0.0009460449218750,
+-0.0029602050781250, -0.0006713867187500, -0.0009460449218750, -0.0004882812500000,
+-0.0005798339843750},
+{-0.0002136230468750, -0.0017700195312500, -0.0003967285156250, -0.0016174316406250,
+-0.0006713867187500, -0.0016174316406250, -0.0000915527343750, -0.0017395019531250,
+-0.0009155273437500, -0.0015258789062500, -0.0003356933593750, -0.0017089843750000,
+-0.0006103515625000, -0.0016479492187500, -0.0011291503906250, -0.0014953613281250,
+0.0001525878906250, -0.0017395019531250, -0.0012207031250000, -0.0015869140625000,
+-0.0003662109375000, -0.0017089843750000, -0.0002441406250000, -0.0017700195312500,
+-0.0012512207031250, -0.0017395019531250, 0.0000000000000000, -0.0019531250000000,
+-0.0007629394531250, -0.0019531250000000, -0.0007324218750000, -0.0019531250000000,
+-0.0003662109375000, -0.0019836425781250, -0.0007019042968750, -0.0019836425781250,
+-0.0007324218750000, -0.0020141601562500, -0.0001525878906250, -0.0020141601562500,
+-0.0008239746093750, -0.0020446777343750, -0.0003356933593750, -0.0021667480468750,
+0.0003356933593750, -0.0022277832031250, -0.0007324218750000, -0.0022888183593750,
+0.0006103515625000, -0.0027770996093750, 0.0001220703125000, -0.0025939941406250,
+0.0000305175781250, -0.0029296875000000, 0.0007629394531250, -0.0034179687500000,
+-0.0001831054687500, -0.0030822753906250, 0.0005798339843750, -0.0038146972656250,
+-0.0002136230468750, -0.0038452148437500, -0.0003356933593750, -0.0037841796875000,
+-0.0000305175781250, -0.0041809082031250, -0.0014038085937500, -0.0041198730468750,
+-0.0002441406250000, -0.0039367675781250, -0.0008850097656250, -0.0041503906250000,
+-0.0009155273437500, -0.0039367675781250, 0.0000000000000000, -0.0038146972656250,
+-0.0009155273437500, -0.0042114257812500, 0.0000915527343750, -0.0031433105468750,
+0.0001831054687500, -0.0039978027343750, 0.0001831054687500, -0.0037231445312500,
+0.0007934570312500, -0.0027465820312500, 0.0007019042968750, -0.0038757324218750,
+0.0007629394531250, -0.0027465820312500, 0.0010070800781250, -0.0032958984375000,
+0.0007019042968750, -0.0030517578125000, 0.0004882812500000, -0.0021057128906250,
+0.0007019042968750, -0.0028686523437500, -0.0004272460937500, -0.0016784667968750,
+0.0000915527343750, -0.0017089843750000, -0.0006103515625000, -0.0014038085937500,
+-0.0017395019531250, -0.0008239746093750, -0.0005798339843750, -0.0014953613281250,
+-0.0023498535156250, -0.0004882812500000, -0.0020141601562500, -0.0009460449218750,
+-0.0010986328125000, -0.0010986328125000, -0.0016174316406250, -0.0007934570312500,
+-0.0018920898437500, -0.0015869140625000, -0.0016784667968750, -0.0009460449218750,
+-0.0028381347656250, -0.0015869140625000, -0.0027160644531250, -0.0015869140625000,
+-0.0027770996093750, -0.0009460449218750, -0.0048217773437500, -0.0018310546875000,
+-0.0027770996093750, -0.0016784667968750, -0.0046081542968750, -0.0016174316406250,
+-0.0040588378906250, -0.0017700195312500, -0.0022888183593750, -0.0019226074218750,
+-0.0050048828125000, -0.0021667480468750, -0.0013732910156250, -0.0017700195312500,
+-0.0024719238281250, -0.0017395019531250, -0.0026245117187500, -0.0016174316406250,
+-0.0005798339843750, -0.0012207031250000, -0.0025024414062500, -0.0012207031250000,
+-0.0009765625000000, -0.0012207031250000, -0.0013122558593750, -0.0010681152343750,
+-0.0018615722656250, -0.0011596679687500, -0.0010070800781250, -0.0013427734375000,
+-0.0013122558593750, -0.0011291503906250, -0.0019531250000000, -0.0015869140625000,
+-0.0004882812500000, -0.0016174316406250, -0.0010375976562500, -0.0019836425781250,
+-0.0028076171875000, -0.0023803710937500, -0.0015869140625000, -0.0021362304687500,
+-0.0014038085937500, -0.0034179687500000, -0.0018005371093750, -0.0031433105468750,
+-0.0013732910156250, -0.0028991699218750, -0.0010681152343750, -0.0036315917968750,
+-0.0016174316406250, -0.0033569335937500, -0.0002136230468750, -0.0030822753906250,
+-0.0006103515625000, -0.0035400390625000, -0.0005493164062500, -0.0036315917968750,
+0.0002746582031250, -0.0034790039062500, -0.0003967285156250, -0.0041809082031250,
+-0.0003356933593750, -0.0040588378906250, -0.0001831054687500, -0.0042114257812500,
+-0.0004577636718750, -0.0042114257812500, -0.0006713867187500, -0.0041503906250000,
+-0.0005187988281250, -0.0043640136718750, -0.0003967285156250, -0.0033569335937500,
+-0.0001525878906250, -0.0036926269531250, -0.0002136230468750, -0.0031738281250000,
+-0.0001220703125000, -0.0023193359375000, 0.0003967285156250, -0.0031738281250000,
+0.0000305175781250, -0.0018615722656250, 0.0000305175781250, -0.0024414062500000,
+0.0003051757812500, -0.0025329589843750, 0.0000610351562500, -0.0018005371093750,
+-0.0003356933593750, -0.0026855468750000, 0.0004577636718750, -0.0017089843750000,
+0.0000000000000000, -0.0021362304687500, -0.0000915527343750, -0.0018310546875000,
+0.0004882812500000, -0.0009765625000000, -0.0000915527343750, -0.0017089843750000,
+-0.0002136230468750, -0.0002746582031250, -0.0000305175781250, -0.0005187988281250,
+-0.0005493164062500, -0.0004272460937500, -0.0009155273437500, 0.0002441406250000,
+-0.0004577636718750, -0.0003967285156250, -0.0015563964843750, 0.0003662109375000,
+-0.0009765625000000, -0.0000305175781250, -0.0010986328125000, 0.0000610351562500,
+-0.0017395019531250, 0.0007019042968750, -0.0006713867187500, 0.0000610351562500,
+-0.0019836425781250, 0.0009155273437500, -0.0012817382812500, 0.0006103515625000,
+-0.0014038085937500, 0.0003967285156250, -0.0024414062500000, 0.0010070800781250,
+-0.0015563964843750, 0.0005187988281250, -0.0027770996093750, 0.0002746582031250,
+-0.0024719238281250, 0.0004882812500000, -0.0026550292968750, 0.0001525878906250,
+-0.0033874511718750, -0.0003356933593750, -0.0027160644531250, 0.0000305175781250,
+-0.0037536621093750, -0.0003662109375000, -0.0036315917968750, -0.0005493164062500,
+-0.0033569335937500, -0.0002136230468750, -0.0039062500000000, -0.0001831054687500,
+-0.0036010742187500, -0.0003967285156250, -0.0030212402343750, 0.0002136230468750,
+-0.0031738281250000, 0.0001831054687500, -0.0027160644531250, 0.0001525878906250,
+-0.0020141601562500, 0.0006408691406250, -0.0023498535156250, 0.0006713867187500,
+-0.0015869140625000, 0.0003967285156250, -0.0016479492187500, 0.0009460449218750,
+-0.0016174316406250, 0.0007019042968750, -0.0011901855468750, 0.0003356933593750,
+-0.0012817382812500, 0.0011901855468750, -0.0010375976562500, 0.0003662109375000,
+-0.0009155273437500, 0.0008544921875000, -0.0009460449218750, 0.0009765625000000,
+-0.0008544921875000, 0.0005798339843750, -0.0007629394531250, 0.0011901855468750,
+-0.0011291503906250, 0.0007019042968750, -0.0009155273437500, 0.0007934570312500,
+-0.0010070800781250, 0.0007934570312500, -0.0013122558593750, 0.0005187988281250,
+-0.0010681152343750, 0.0007019042968750, -0.0014038085937500, 0.0005493164062500,
+-0.0010681152343750, 0.0005798339843750, -0.0012817382812500, 0.0005187988281250,
+-0.0015869140625000, 0.0004577636718750, -0.0009765625000000, 0.0007629394531250,
+-0.0020446777343750, 0.0003051757812500, -0.0015258789062500, 0.0006713867187500,
+-0.0016479492187500, 0.0007629394531250, -0.0024108886718750, 0.0005187988281250,
+-0.0014038085937500, 0.0010681152343750, -0.0024414062500000, 0.0008850097656250,
+-0.0020141601562500, 0.0012512207031250, -0.0019226074218750, 0.0014343261718750,
+-0.0025634765625000, 0.0013427734375000, -0.0018310546875000, 0.0015869140625000,
+-0.0025024414062500, 0.0017089843750000, -0.0023193359375000, 0.0015869140625000,
+-0.0022277832031250, 0.0016784667968750, -0.0025634765625000, 0.0016784667968750,
+-0.0022277832031250, 0.0014038085937500, -0.0026245117187500, 0.0017395019531250,
+-0.0024414062500000, 0.0014343261718750, -0.0024414062500000, 0.0013427734375000,
+-0.0027160644531250, 0.0014343261718750, -0.0025329589843750, 0.0009460449218750,
+-0.0026550292968750, 0.0008850097656250, -0.0027770996093750, 0.0006408691406250,
+-0.0026245117187500, 0.0004882812500000, -0.0025939941406250, 0.0003967285156250,
+-0.0026855468750000, 0.0002441406250000, -0.0025634765625000, 0.0003051757812500,
+-0.0025329589843750, 0.0001831054687500, -0.0025939941406250, 0.0001831054687500,
+-0.0025329589843750, 0.0001220703125000, -0.0025329589843750, -0.0000305175781250,
+-0.0026245117187500, 0.0005187988281250, -0.0024414062500000, 0.0002441406250000,
+-0.0025024414062500, 0.0006713867187500, -0.0025634765625000, 0.0011901855468750,
+-0.0021362304687500, 0.0006713867187500, -0.0023498535156250, 0.0017700195312500,
+-0.0021972656250000, 0.0015869140625000, -0.0018615722656250, 0.0015869140625000,
+-0.0018920898437500, 0.0021362304687500, -0.0019226074218750, 0.0018005371093750,
+-0.0013732910156250, 0.0020751953125000, -0.0015563964843750, 0.0020751953125000,
+-0.0016174316406250, 0.0019226074218750, -0.0013122558593750, 0.0018920898437500,
+-0.0015258789062500, 0.0017395019531250, -0.0015869140625000, 0.0016174316406250,
+-0.0015258789062500, 0.0013427734375000, -0.0015258789062500, 0.0014343261718750,
+-0.0016174316406250, 0.0013122558593750, -0.0016479492187500, 0.0007934570312500,
+-0.0013122558593750, 0.0018615722656250, -0.0014038085937500, 0.0010986328125000,
+-0.0012817382812500, 0.0014648437500000, -0.0008239746093750, 0.0025024414062500,
+-0.0007019042968750, 0.0014343261718750, -0.0009765625000000, 0.0027770996093750,
+-0.0006408691406250, 0.0025329589843750, -0.0005798339843750, 0.0022277832031250,
+-0.0010070800781250, 0.0028381347656250, -0.0010375976562500, 0.0022888183593750,
+-0.0008544921875000, 0.0018005371093750, -0.0009460449218750, 0.0026855468750000,
+-0.0012207031250000, 0.0018310546875000, -0.0011596679687500, 0.0005798339843750,
+-0.0010375976562500, 0.0016784667968750, -0.0018310546875000, -0.0000305175781250,
+-0.0017395019531250, 0.0000915527343750, -0.0018310546875000, 0.0001525878906250,
+-0.0024108886718750, -0.0005493164062500, -0.0024414062500000, 0.0001831054687500,
+-0.0022583007812500, -0.0003662109375000, -0.0025939941406250, -0.0008850097656250,
+-0.0025939941406250, -0.0006408691406250, -0.0020446777343750, -0.0006103515625000,
+-0.0019226074218750, -0.0010986328125000, -0.0018920898437500, -0.0010070800781250,
+-0.0016479492187500, -0.0007324218750000, -0.0010986328125000, -0.0009765625000000,
+-0.0009765625000000, -0.0012207031250000, -0.0013427734375000, -0.0012512207031250,
+-0.0002746582031250, -0.0021057128906250, -0.0006408691406250, -0.0026550292968750,
+-0.0009460449218750, -0.0028076171875000, -0.0002746582031250, -0.0032958984375000,
+-0.0005493164062500, -0.0037841796875000, -0.0012817382812500, -0.0028381347656250,
+-0.0008239746093750, -0.0032348632812500, -0.0010986328125000, -0.0028991699218750,
+-0.0019531250000000, -0.0016479492187500, -0.0015258789062500, -0.0020446777343750,
+-0.0019531250000000, -0.0021362304687500, -0.0022583007812500, -0.0016479492187500,
+-0.0023193359375000, -0.0021667480468750, -0.0025329589843750, -0.0031127929687500,
+-0.0027465820312500, -0.0028686523437500, -0.0029296875000000, -0.0029907226562500,
+-0.0031433105468750, -0.0036621093750000, -0.0034790039062500, -0.0033569335937500,
+-0.0038452148437500, -0.0023498535156250, -0.0049438476562500, -0.0046997070312500,
+-0.0029296875000000, -0.0036315917968750, -0.0035400390625000, -0.0047607421875000,
+-0.0035705566406250, -0.0040283203125000, -0.0027160644531250, -0.0026245117187500,
+-0.0025634765625000, -0.0033264160156250, -0.0031433105468750, -0.0031433105468750,
+-0.0027770996093750, -0.0025939941406250, -0.0023803710937500, -0.0029296875000000,
+-0.0026550292968750, -0.0034484863281250, -0.0024719238281250, -0.0029602050781250,
+-0.0014038085937500, -0.0027160644531250, -0.0013427734375000, -0.0031738281250000,
+-0.0012207031250000, -0.0024414062500000, -0.0003356933593750, -0.0014648437500000,
+-0.0001525878906250},
+{-0.0015258789062500, 0.0030822753906250, -0.0014953613281250, 0.0033874511718750,
+-0.0011596679687500, 0.0026245117187500, -0.0010070800781250, 0.0024414062500000,
+-0.0010681152343750, 0.0023498535156250, -0.0010375976562500, 0.0019531250000000,
+-0.0010070800781250, 0.0019226074218750, -0.0013122558593750, 0.0021057128906250,
+-0.0014038085937500, 0.0018615722656250, -0.0014038085937500, 0.0020751953125000,
+-0.0015869140625000, 0.0023498535156250, -0.0016479492187500, 0.0020446777343750,
+-0.0012207031250000, 0.0027160644531250, -0.0014953613281250, 0.0027465820312500,
+-0.0013427734375000, 0.0027160644531250, -0.0007629394531250, 0.0031433105468750,
+-0.0010375976562500, 0.0029907226562500, -0.0007934570312500, 0.0027160644531250,
+-0.0004577636718750, 0.0028076171875000, -0.0006713867187500, 0.0026550292968750,
+-0.0008850097656250, 0.0021972656250000, -0.0007324218750000, 0.0021057128906250,
+-0.0010375976562500, 0.0020446777343750, -0.0012817382812500, 0.0014648437500000,
+-0.0012817382812500, 0.0016479492187500, -0.0012817382812500, 0.0018005371093750,
+-0.0013427734375000, 0.0010986328125000, -0.0015869140625000, 0.0022888183593750,
+-0.0013122558593750, 0.0021972656250000, -0.0014648437500000, 0.0021972656250000,
+-0.0018920898437500, 0.0031127929687500, -0.0017089843750000, 0.0029296875000000,
+-0.0020141601562500, 0.0026245117187500, -0.0020751953125000, 0.0029296875000000,
+-0.0022277832031250, 0.0024414062500000, -0.0024414062500000, 0.0017700195312500,
+-0.0022583007812500, 0.0021362304687500, -0.0027465820312500, 0.0012207031250000,
+-0.0025329589843750, 0.0011596679687500, -0.0024414062500000, 0.0011291503906250,
+-0.0027160644531250, 0.0004272460937500, -0.0021057128906250, 0.0002441406250000,
+-0.0017395019531250, 0.0006408691406250, -0.0020751953125000, 0.0000305175781250,
+-0.0015563964843750, 0.0002441406250000, -0.0009155273437500, 0.0009765625000000,
+-0.0017395019531250, 0.0005493164062500, -0.0016479492187500, 0.0006408691406250,
+-0.0014343261718750, 0.0007629394531250, -0.0020141601562500, -0.0000305175781250,
+-0.0023803710937500, -0.0005493164062500, -0.0018615722656250, -0.0005798339843750,
+-0.0023803710937500, -0.0028076171875000, -0.0025939941406250, -0.0027160644531250,
+-0.0023498535156250, -0.0032653808593750, -0.0024719238281250, -0.0048522949218750,
+-0.0028991699218750, -0.0041198730468750, -0.0028991699218750, -0.0041809082031250,
+-0.0031433105468750, -0.0042114257812500, -0.0035095214843750, -0.0037231445312500,
+-0.0038146972656250, -0.0032653808593750, -0.0042114257812500, -0.0032653808593750,
+-0.0041198730468750, -0.0037841796875000, -0.0048828125000000, -0.0036621093750000,
+-0.0046691894531250, -0.0038452148437500, -0.0033874511718750, -0.0047607421875000,
+-0.0040893554687500, -0.0042114257812500, -0.0043640136718750, -0.0061340332031250,
+-0.0041503906250000, -0.0053405761718750, -0.0038452148437500, -0.0049133300781250,
+-0.0035095214843750, -0.0057067871093750, -0.0032348632812500, -0.0056762695312500,
+-0.0027160644531250, -0.0039367675781250, -0.0021362304687500, -0.0042114257812500,
+-0.0021362304687500, -0.0038452148437500, -0.0019226074218750, -0.0023803710937500,
+-0.0010375976562500, -0.0028381347656250, -0.0019226074218750, -0.0028381347656250,
+-0.0012817382812500, -0.0024108886718750, -0.0007324218750000, -0.0027770996093750,
+-0.0009460449218750, -0.0034179687500000, -0.0001525878906250, -0.0031433105468750,
+0.0006408691406250, -0.0026245117187500, 0.0011596679687500, -0.0028991699218750,
+0.0015563964843750, -0.0025329589843750, 0.0024414062500000, -0.0015869140625000,
+0.0030212402343750, -0.0018310546875000, 0.0016784667968750, -0.0022277832031250,
+0.0029907226562500, -0.0017395019531250, 0.0022583007812500, -0.0019226074218750,
+0.0003662109375000, -0.0026245117187500, 0.0018920898437500, -0.0020751953125000,
+0.0003051757812500, -0.0016479492187500, 0.0000915527343750, -0.0021057128906250,
+0.0007934570312500, -0.0014648437500000, 0.0005187988281250, -0.0006103515625000,
+0.0006408691406250, -0.0013122558593750, 0.0019531250000000, -0.0008239746093750,
+0.0017700195312500, -0.0007019042968750, 0.0020751953125000, -0.0010681152343750,
+0.0030822753906250, -0.0013122558593750, 0.0027160644531250, -0.0012512207031250,
+0.0023803710937500, -0.0010070800781250, 0.0026550292968750, -0.0012817382812500,
+0.0024414062500000, -0.0007629394531250, 0.0018920898437500, 0.0000610351562500,
+0.0020141601562500, -0.0001525878906250, 0.0015563964843750, 0.0003967285156250,
+0.0014648437500000, 0.0007629394531250, 0.0015258789062500, 0.0005798339843750,
+0.0013122558593750, 0.0004882812500000, 0.0013122558593750, 0.0007324218750000,
+0.0018310546875000, 0.0004577636718750, 0.0016174316406250, 0.0007324218750000,
+0.0017395019531250, 0.0008239746093750, 0.0022277832031250, 0.0008850097656250,
+0.0020751953125000, 0.0012817382812500, 0.0019531250000000, 0.0007324218750000,
+0.0021057128906250, 0.0011291503906250, 0.0018005371093750, 0.0007934570312500,
+0.0013122558593750, 0.0001525878906250, 0.0013732910156250, 0.0007629394531250,
+0.0008239746093750, -0.0001831054687500, 0.0006103515625000, 0.0000610351562500,
+0.0004882812500000, 0.0000915527343750, 0.0002136230468750, -0.0003967285156250,
+0.0001831054687500, 0.0001525878906250, 0.0003051757812500, -0.0005493164062500,
+0.0000305175781250, -0.0003356933593750, 0.0004882812500000, -0.0002441406250000,
+0.0009155273437500, -0.0005798339843750, 0.0005187988281250, -0.0003051757812500,
+0.0016784667968750, -0.0003662109375000, 0.0015563964843750, -0.0002136230468750,
+0.0018005371093750, -0.0002441406250000, 0.0027160644531250, -0.0003051757812500,
+0.0023803710937500, -0.0000305175781250, 0.0028686523437500, -0.0003662109375000,
+0.0032348632812500, -0.0002136230468750, 0.0030212402343750, -0.0001525878906250,
+0.0030822753906250, -0.0003051757812500, 0.0035095214843750, -0.0000305175781250,
+0.0027770996093750, -0.0001831054687500, 0.0033569335937500, -0.0000305175781250,
+0.0033874511718750, -0.0000610351562500, 0.0029296875000000, -0.0001831054687500,
+0.0037231445312500, -0.0000305175781250, 0.0029907226562500, -0.0003356933593750,
+0.0034790039062500, -0.0001831054687500, 0.0036010742187500, -0.0002746582031250,
+0.0031127929687500, -0.0004882812500000, 0.0038146972656250, -0.0001525878906250,
+0.0034790039062500, -0.0006103515625000, 0.0032653808593750, -0.0003662109375000,
+0.0034484863281250, -0.0004272460937500, 0.0036010742187500, -0.0008544921875000,
+0.0035095214843750, -0.0005798339843750, 0.0035705566406250, -0.0008850097656250,
+0.0036621093750000, -0.0008239746093750, 0.0036010742187500, -0.0008239746093750,
+0.0035400390625000, -0.0010070800781250, 0.0035400390625000, -0.0008544921875000,
+0.0034484863281250, -0.0010375976562500, 0.0034179687500000, -0.0009765625000000,
+0.0033264160156250, -0.0009765625000000, 0.0032653808593750, -0.0011291503906250,
+0.0032958984375000, -0.0010070800781250, 0.0032043457031250, -0.0011901855468750,
+0.0032348632812500, -0.0011291503906250, 0.0032043457031250, -0.0011901855468750,
+0.0031433105468750, -0.0013427734375000, 0.0032653808593750, -0.0012207031250000,
+0.0032043457031250, -0.0012817382812500, 0.0033569335937500, -0.0013732910156250,
+0.0033569335937500, -0.0013732910156250, 0.0034790039062500, -0.0013427734375000,
+0.0039367675781250, -0.0015258789062500, 0.0034484863281250, -0.0015869140625000,
+0.0042724609375000, -0.0017089843750000, 0.0043029785156250, -0.0018615722656250,
+0.0039978027343750, -0.0019226074218750, 0.0051574707031250, -0.0020446777343750,
+0.0041503906250000, -0.0022888183593750, 0.0045166015625000, -0.0022277832031250,
+0.0045471191406250, -0.0025329589843750, 0.0037841796875000, -0.0028991699218750,
+0.0040588378906250, -0.0027770996093750, 0.0037536621093750, -0.0032958984375000,
+0.0030822753906250, -0.0035095214843750, 0.0032043457031250, -0.0035095214843750,
+0.0031433105468750, -0.0037231445312500, 0.0022277832031250, -0.0040588378906250,
+0.0028991699218750, -0.0036926269531250, 0.0023498535156250, -0.0041198730468750,
+0.0022888183593750, -0.0040588378906250, 0.0024719238281250, -0.0038757324218750,
+0.0014343261718750, -0.0045776367187500, 0.0025024414062500, -0.0037536621093750,
+0.0015563964843750, -0.0043334960937500, 0.0017395019531250, -0.0043029785156250,
+0.0026245117187500, -0.0036621093750000, 0.0012512207031250, -0.0043334960937500,
+0.0029296875000000, -0.0034179687500000, 0.0024719238281250, -0.0036315917968750,
+0.0021972656250000, -0.0032348632812500, 0.0032043457031250, -0.0023803710937500,
+0.0025024414062500, -0.0029907226562500, 0.0023193359375000, -0.0021057128906250,
+0.0026550292968750, -0.0021362304687500, 0.0021667480468750, -0.0022888183593750,
+0.0017089843750000, -0.0021362304687500, 0.0022277832031250, -0.0024719238281250,
+0.0012817382812500, -0.0020141601562500, 0.0014648437500000, -0.0022277832031250,
+0.0016174316406250, -0.0022583007812500, 0.0012207031250000, -0.0018920898437500,
+0.0015563964843750, -0.0021362304687500, 0.0010375976562500, -0.0020141601562500,
+0.0016784667968750, -0.0017395019531250, 0.0010375976562500, -0.0021057128906250,
+0.0003967285156250, -0.0021667480468750, 0.0019226074218750, -0.0015563964843750,
+0.0002441406250000, -0.0023193359375000, 0.0008544921875000, -0.0015869140625000,
+0.0007324218750000, -0.0015869140625000, -0.0006713867187500, -0.0019836425781250,
+0.0000305175781250, -0.0008239746093750, -0.0010986328125000, -0.0019226074218750,
+-0.0018310546875000, -0.0007019042968750, -0.0018920898437500, -0.0007934570312500,
+-0.0018615722656250, -0.0019836425781250, -0.0020141601562500, -0.0005493164062500,
+-0.0022888183593750, -0.0018310546875000, -0.0018615722656250, -0.0018920898437500,
+-0.0020751953125000, -0.0014648437500000, -0.0026550292968750, -0.0018005371093750,
+-0.0021362304687500, -0.0017700195312500, -0.0025024414062500, -0.0018005371093750,
+-0.0030517578125000, -0.0014953613281250, -0.0024108886718750, -0.0018920898437500,
+-0.0022583007812500, -0.0021057128906250, -0.0031738281250000, -0.0015563964843750,
+-0.0010681152343750, -0.0026550292968750, -0.0020141601562500, -0.0021972656250000,
+-0.0020446777343750, -0.0020141601562500, -0.0005798339843750, -0.0026550292968750,
+-0.0022583007812500, -0.0021972656250000, -0.0029602050781250, -0.0027770996093750,
+-0.0024414062500000, -0.0025939941406250, -0.0022583007812500, -0.0025024414062500,
+-0.0023498535156250, -0.0030212402343750, -0.0018310546875000, -0.0028991699218750,
+-0.0017395019531250, -0.0027770996093750, -0.0012512207031250, -0.0027770996093750,
+-0.0012817382812500, -0.0024414062500000, -0.0010375976562500, -0.0021057128906250,
+-0.0000915527343750, -0.0021057128906250, -0.0010070800781250, -0.0014648437500000,
+-0.0002441406250000, -0.0018310546875000, 0.0000610351562500, -0.0012207031250000,
+-0.0007019042968750, -0.0007019042968750, -0.0000610351562500, -0.0018920898437500,
+-0.0001525878906250, -0.0001220703125000, -0.0001831054687500, -0.0010681152343750,
+-0.0000305175781250, -0.0011291503906250, -0.0000610351562500, 0.0000915527343750,
+-0.0000915527343750, -0.0012207031250000, 0.0003051757812500, 0.0000305175781250,
+0.0002746582031250, -0.0002441406250000, 0.0002136230468750, -0.0002746582031250,
+0.0006103515625000, 0.0003051757812500, 0.0007019042968750, -0.0003356933593750,
+0.0002136230468750, 0.0011901855468750, 0.0006103515625000, 0.0001525878906250,
+0.0004272460937500, 0.0009765625000000, -0.0004577636718750, 0.0024108886718750,
+-0.0003967285156250, 0.0003662109375000, -0.0004272460937500, 0.0029296875000000,
+-0.0011291503906250},
+{0.0040283203125000, -0.0030822753906250, 0.0031738281250000, -0.0034179687500000,
+0.0029296875000000, -0.0032653808593750, 0.0034179687500000, -0.0025939941406250,
+0.0020141601562500, -0.0031738281250000, 0.0028991699218750, -0.0021667480468750,
+0.0018310546875000, -0.0023498535156250, 0.0018005371093750, -0.0024108886718750,
+0.0026245117187500, -0.0019226074218750, 0.0011901855468750, -0.0023193359375000,
+0.0028686523437500, -0.0019531250000000, 0.0024108886718750, -0.0024414062500000,
+0.0022583007812500, -0.0022277832031250, 0.0033569335937500, -0.0018615722656250,
+0.0027465820312500, -0.0027465820312500, 0.0028991699218750, -0.0017089843750000,
+0.0034179687500000, -0.0021362304687500, 0.0029296875000000, -0.0022888183593750,
+0.0026245117187500, -0.0018310546875000, 0.0031433105468750, -0.0024719238281250,
+0.0017700195312500, -0.0020141601562500, 0.0024414062500000, -0.0021667480468750,
+0.0013732910156250, -0.0023803710937500, -0.0000915527343750, -0.0019531250000000,
+0.0018920898437500, -0.0018310546875000, -0.0002441406250000, -0.0020446777343750,
+0.0007934570312500, -0.0014038085937500, 0.0010681152343750, -0.0015563964843750,
+-0.0000610351562500, -0.0018005371093750, 0.0015258789062500, -0.0007934570312500,
+-0.0000305175781250, -0.0020446777343750, 0.0003051757812500, -0.0015869140625000,
+0.0001525878906250, -0.0013732910156250, -0.0010681152343750, -0.0021362304687500,
+-0.0005798339843750, -0.0015869140625000, -0.0015563964843750, -0.0015258789062500,
+-0.0010070800781250, -0.0015869140625000, -0.0009155273437500, -0.0017089843750000,
+-0.0014648437500000, -0.0015563964843750, -0.0005798339843750, -0.0012817382812500,
+-0.0010375976562500, -0.0021362304687500, -0.0005187988281250, -0.0015869140625000,
+-0.0005493164062500, -0.0019226074218750, -0.0011596679687500, -0.0026855468750000,
+-0.0005493164062500, -0.0016174316406250, -0.0010070800781250, -0.0025634765625000,
+-0.0016174316406250, -0.0023193359375000, -0.0008850097656250, -0.0020446777343750,
+-0.0010681152343750, -0.0026855468750000, -0.0036010742187500, -0.0027160644531250,
+-0.0035095214843750, -0.0025329589843750, -0.0032348632812500, -0.0026550292968750,
+-0.0034179687500000, -0.0026550292968750, -0.0032653808593750, -0.0023498535156250,
+-0.0028076171875000, -0.0021362304687500, -0.0028686523437500, -0.0023803710937500,
+-0.0028076171875000, -0.0018310546875000, -0.0021362304687500, -0.0016479492187500,
+-0.0017089843750000, -0.0018005371093750, -0.0017089843750000, -0.0011901855468750,
+-0.0005493164062500, -0.0006713867187500, 0.0000915527343750, -0.0009765625000000,
+0.0000915527343750, -0.0004272460937500, 0.0005798339843750, 0.0001831054687500,
+0.0012512207031250, -0.0005798339843750, 0.0003967285156250, 0.0002136230468750,
+0.0007324218750000, 0.0000305175781250, 0.0007629394531250, 0.0000610351562500,
+0.0001525878906250, 0.0005187988281250, 0.0004577636718750, 0.0001831054687500,
+0.0001525878906250, 0.0010375976562500, 0.0000610351562500, 0.0006408691406250,
+-0.0000915527343750, 0.0012512207031250, -0.0002746582031250, 0.0021667480468750,
+-0.0002136230468750, 0.0012207031250000, -0.0007629394531250, 0.0028686523437500,
+-0.0005187988281250, 0.0024414062500000, -0.0006103515625000, 0.0022277832031250,
+-0.0009155273437500, 0.0029602050781250, -0.0002746582031250, 0.0021362304687500,
+-0.0007934570312500, 0.0022277832031250, -0.0008544921875000, 0.0019226074218750,
+-0.0006103515625000, 0.0017089843750000, -0.0008544921875000, 0.0016174316406250,
+-0.0010070800781250, 0.0012207031250000, -0.0002136230468750, 0.0015869140625000,
+-0.0008239746093750, 0.0014343261718750, -0.0004272460937500, 0.0015869140625000,
+0.0004272460937500, 0.0019226074218750, -0.0006408691406250, 0.0016174316406250,
+0.0001220703125000, 0.0020141601562500, 0.0000305175781250, 0.0021057128906250,
+-0.0003967285156250, 0.0019531250000000, -0.0003356933593750, 0.0019836425781250,
+-0.0004577636718750, 0.0021057128906250, -0.0004577636718750, 0.0021057128906250,
+-0.0006408691406250, 0.0020751953125000, -0.0005493164062500, 0.0024414062500000,
+-0.0003662109375000, 0.0027770996093750, -0.0004577636718750, 0.0026855468750000,
+-0.0004882812500000, 0.0033874511718750, -0.0002746582031250, 0.0035705566406250,
+-0.0007324218750000, 0.0034484863281250, -0.0009765625000000, 0.0038452148437500,
+-0.0006408691406250, 0.0040893554687500, -0.0017089843750000, 0.0033569335937500,
+-0.0015563964843750, 0.0039367675781250, -0.0016174316406250, 0.0036010742187500,
+-0.0023498535156250, 0.0027770996093750, -0.0020751953125000, 0.0036621093750000,
+-0.0021362304687500, 0.0023193359375000, -0.0024414062500000, 0.0026855468750000,
+-0.0022583007812500, 0.0025634765625000, -0.0021057128906250, 0.0017700195312500,
+-0.0024108886718750, 0.0025024414062500, -0.0018920898437500, 0.0014953613281250,
+-0.0021667480468750, 0.0017700195312500, -0.0019226074218750, 0.0019226074218750,
+-0.0015258789062500, 0.0014343261718750, -0.0020446777343750, 0.0021362304687500,
+-0.0012207031250000, 0.0021057128906250, -0.0014038085937500, 0.0022277832031250,
+-0.0012817382812500, 0.0025939941406250, -0.0006713867187500, 0.0028076171875000,
+-0.0010070800781250, 0.0028381347656250, -0.0004577636718750, 0.0035705566406250,
+-0.0007019042968750, 0.0036010742187500, -0.0005798339843750, 0.0040283203125000,
+-0.0001831054687500, 0.0046081542968750, -0.0007629394531250, 0.0043640136718750,
+-0.0002441406250000, 0.0050964355468750, -0.0004272460937500, 0.0050354003906250,
+-0.0005187988281250, 0.0051269531250000, -0.0002441406250000, 0.0055236816406250,
+-0.0005493164062500, 0.0051879882812500, -0.0003662109375000, 0.0055541992187500,
+-0.0004577636718750, 0.0054626464843750, -0.0004882812500000, 0.0054016113281250,
+-0.0004272460937500, 0.0055541992187500, -0.0006713867187500, 0.0052795410156250,
+-0.0003967285156250, 0.0054626464843750, -0.0006713867187500, 0.0052185058593750,
+-0.0006408691406250, 0.0051879882812500, -0.0005187988281250, 0.0053100585937500,
+-0.0009460449218750, 0.0048522949218750, -0.0007629394531250, 0.0051574707031250,
+-0.0009460449218750, 0.0049133300781250, -0.0010375976562500, 0.0048522949218750,
+-0.0009460449218750, 0.0050048828125000, -0.0011901855468750, 0.0047912597656250,
+-0.0010375976562500, 0.0047912597656250, -0.0011596679687500, 0.0048522949218750,
+-0.0012207031250000, 0.0046386718750000, -0.0010986328125000, 0.0044860839843750,
+-0.0011291503906250, 0.0046081542968750, -0.0008544921875000, 0.0039672851562500,
+-0.0008850097656250, 0.0040588378906250, -0.0005798339843750, 0.0035400390625000,
+-0.0001831054687500, 0.0028991699218750, -0.0003967285156250, 0.0032043457031250,
+0.0000610351562500, 0.0021972656250000, 0.0000610351562500, 0.0023193359375000,
+-0.0001220703125000, 0.0022277832031250, -0.0001220703125000, 0.0015563964843750,
+-0.0002441406250000, 0.0019226074218750, -0.0003967285156250, 0.0015869140625000,
+-0.0004272460937500, 0.0012817382812500, -0.0005493164062500, 0.0016479492187500,
+-0.0006713867187500, 0.0017089843750000, -0.0005798339843750, 0.0013122558593750,
+-0.0006408691406250, 0.0025634765625000, -0.0006103515625000, 0.0020141601562500,
+-0.0005187988281250, 0.0024719238281250, -0.0004577636718750, 0.0035705566406250,
+-0.0003967285156250, 0.0027160644531250, -0.0003662109375000, 0.0036315917968750,
+-0.0002746582031250, 0.0036010742187500, -0.0002746582031250, 0.0032653808593750,
+-0.0004272460937500, 0.0035400390625000, -0.0005493164062500, 0.0033264160156250,
+-0.0003662109375000, 0.0028381347656250, -0.0007934570312500, 0.0027160644531250,
+-0.0007324218750000, 0.0026855468750000, -0.0003967285156250, 0.0022888183593750,
+-0.0009460449218750, 0.0021667480468750, -0.0006103515625000, 0.0021972656250000,
+-0.0007324218750000, 0.0019836425781250, -0.0010375976562500, 0.0021972656250000,
+-0.0012207031250000, 0.0024719238281250, -0.0016174316406250, 0.0022277832031250,
+-0.0017089843750000, 0.0031433105468750, -0.0021667480468750, 0.0026245117187500,
+-0.0022888183593750, 0.0027770996093750, -0.0021667480468750, 0.0036010742187500,
+-0.0024414062500000, 0.0029296875000000, -0.0019531250000000, 0.0029296875000000,
+-0.0018310546875000, 0.0029907226562500, -0.0016479492187500, 0.0024719238281250,
+-0.0014038085937500, 0.0021057128906250, -0.0014038085937500, 0.0020446777343750,
+-0.0009460449218750, 0.0015563964843750, -0.0013427734375000, 0.0017395019531250,
+-0.0014648437500000, 0.0016479492187500, -0.0012207031250000, 0.0011291503906250,
+-0.0016174316406250, 0.0013732910156250, -0.0021667480468750, 0.0014648437500000,
+-0.0016479492187500, 0.0012817382812500, -0.0023803710937500, 0.0015869140625000,
+-0.0031433105468750, 0.0018615722656250, -0.0018005371093750, 0.0015563964843750,
+-0.0034179687500000, 0.0016479492187500, -0.0029296875000000, 0.0020446777343750,
+-0.0029296875000000, 0.0014038085937500, -0.0038146972656250, 0.0008850097656250,
+-0.0028686523437500, 0.0015258789062500, -0.0040588378906250, 0.0003356933593750,
+-0.0034179687500000, -0.0000915527343750, -0.0032958984375000, 0.0002746582031250,
+-0.0039062500000000, -0.0000305175781250, -0.0029602050781250, -0.0007324218750000,
+-0.0039672851562500, -0.0001831054687500, -0.0033264160156250, -0.0002136230468750,
+-0.0035705566406250, -0.0012512207031250, -0.0044250488281250, -0.0014038085937500,
+-0.0033264160156250, -0.0009460449218750, -0.0046691894531250, -0.0035705566406250,
+-0.0042419433593750, -0.0027770996093750, -0.0036315917968750, -0.0028991699218750,
+-0.0043640136718750, -0.0058288574218750, -0.0011291503906250, -0.0020141601562500,
+-0.0039672851562500, -0.0041809082031250, -0.0021972656250000, -0.0031127929687500,
+-0.0022277832031250, -0.0035705566406250, -0.0032653808593750, -0.0045166015625000,
+-0.0020751953125000, -0.0037536621093750, -0.0017395019531250, -0.0045776367187500,
+-0.0013427734375000, -0.0039062500000000, -0.0006408691406250, -0.0042114257812500,
+0.0000915527343750, -0.0048522949218750, 0.0008239746093750, -0.0035400390625000,
+0.0002136230468750, -0.0053710937500000, 0.0011901855468750, -0.0046691894531250,
+0.0010681152343750, -0.0042724609375000, 0.0000000000000000, -0.0052185058593750,
+0.0009460449218750, -0.0042724609375000, 0.0007324218750000, -0.0039672851562500,
+0.0002746582031250, -0.0038452148437500, 0.0012512207031250, -0.0035095214843750,
+0.0019531250000000, -0.0032958984375000, 0.0012207031250000, -0.0031127929687500,
+0.0024108886718750, -0.0029907226562500, 0.0028076171875000, -0.0030212402343750,
+0.0021057128906250, -0.0027770996093750, 0.0019226074218750, -0.0025024414062500,
+0.0023193359375000, -0.0025634765625000, 0.0017089843750000, -0.0020751953125000,
+0.0012817382812500, -0.0017395019531250, 0.0020141601562500, -0.0019531250000000,
+0.0020446777343750, -0.0017395019531250, 0.0012817382812500, -0.0011291503906250,
+0.0032958984375000, -0.0021972656250000, 0.0025329589843750, -0.0017089843750000,
+0.0028381347656250, -0.0014038085937500, 0.0044860839843750, -0.0020446777343750,
+0.0033264160156250, -0.0016479492187500, 0.0037536621093750, -0.0011596679687500,
+0.0040588378906250, -0.0013427734375000, 0.0032958984375000, -0.0010681152343750,
+0.0027465820312500, -0.0006408691406250, 0.0029602050781250, -0.0008239746093750,
+0.0019226074218750, -0.0006713867187500, 0.0018310546875000, -0.0006408691406250,
+0.0019531250000000, -0.0005187988281250, 0.0014648437500000, -0.0002746582031250,
+0.0014038085937500, -0.0002441406250000, 0.0023193359375000, 0.0003356933593750,
+0.0018005371093750, 0.0004882812500000, 0.0022583007812500, 0.0004882812500000,
+0.0032653808593750},
+{-0.0016784667968750, 0.0014953613281250, -0.0017089843750000, 0.0024719238281250,
+-0.0019836425781250, 0.0018615722656250, -0.0016784667968750, 0.0023498535156250,
+-0.0015563964843750, 0.0024108886718750, -0.0021972656250000, 0.0021362304687500,
+-0.0021667480468750, 0.0022583007812500, -0.0013122558593750, 0.0023498535156250,
+-0.0030517578125000, 0.0017395019531250, -0.0020446777343750, 0.0016784667968750,
+-0.0022583007812500, 0.0019836425781250, -0.0035705566406250, 0.0017089843750000,
+-0.0021362304687500, 0.0012207031250000, -0.0035400390625000, 0.0021972656250000,
+-0.0028381347656250, 0.0015563964843750, -0.0028076171875000, 0.0017395019531250,
+-0.0034179687500000, 0.0029602050781250, -0.0018920898437500, 0.0022888183593750,
+-0.0034179687500000, 0.0023193359375000, -0.0025634765625000, 0.0030517578125000,
+-0.0027160644531250, 0.0018005371093750, -0.0038146972656250, 0.0007324218750000,
+-0.0025634765625000, 0.0015258789062500, -0.0041809082031250, -0.0010681152343750,
+-0.0036315917968750, -0.0011291503906250, -0.0034179687500000, -0.0010681152343750,
+-0.0039672851562500, -0.0023803710937500, -0.0027160644531250, -0.0020141601562500,
+-0.0034179687500000, -0.0021362304687500, -0.0022277832031250, -0.0020446777343750,
+-0.0020446777343750, -0.0023193359375000, -0.0035705566406250, -0.0028686523437500,
+-0.0014953613281250, -0.0028381347656250, -0.0018005371093750, -0.0026855468750000,
+-0.0012512207031250, -0.0027465820312500, -0.0009460449218750, -0.0031433105468750,
+-0.0005187988281250, -0.0033569335937500, 0.0001831054687500, -0.0031738281250000,
+-0.0004577636718750, -0.0041198730468750, 0.0004882812500000, -0.0041809082031250,
+0.0005798339843750, -0.0039672851562500, -0.0002136230468750, -0.0042724609375000,
+0.0007324218750000, -0.0042419433593750, 0.0008850097656250, -0.0035400390625000,
+0.0006408691406250, -0.0030517578125000, 0.0013427734375000, -0.0035705566406250,
+0.0021667480468750, -0.0033569335937500, 0.0018920898437500, -0.0023193359375000,
+0.0021667480468750, -0.0043640136718750, 0.0028686523437500, -0.0035400390625000,
+0.0021362304687500, -0.0033874511718750, 0.0014648437500000, -0.0046691894531250,
+0.0023193359375000, -0.0034790039062500, 0.0012817382812500, -0.0039367675781250,
+0.0007629394531250, -0.0038757324218750, 0.0018005371093750, -0.0036010742187500,
+0.0020751953125000, -0.0036010742187500, 0.0012512207031250, -0.0032043457031250,
+0.0032653808593750, -0.0033264160156250, 0.0028686523437500, -0.0030822753906250,
+0.0027160644531250, -0.0028991699218750, 0.0038146972656250, -0.0029602050781250,
+0.0030822753906250, -0.0027770996093750, 0.0028076171875000, -0.0024414062500000,
+0.0027770996093750, -0.0021362304687500, 0.0020141601562500, -0.0020446777343750,
+0.0011901855468750, -0.0018310546875000, 0.0013122558593750, -0.0012512207031250,
+0.0010375976562500, -0.0017700195312500, 0.0005187988281250, -0.0013427734375000,
+0.0011291503906250, -0.0011291503906250, 0.0017089843750000, -0.0015563964843750,
+0.0011901855468750, -0.0013122558593750, 0.0024414062500000, -0.0010375976562500,
+0.0024414062500000, -0.0012512207031250, 0.0025329589843750, -0.0010070800781250,
+0.0032958984375000, -0.0007629394531250, 0.0029602050781250, -0.0009765625000000,
+0.0029296875000000, -0.0005187988281250, 0.0031738281250000, -0.0005493164062500,
+0.0026245117187500, -0.0003662109375000, 0.0020751953125000, 0.0000305175781250,
+0.0023803710937500, -0.0001831054687500, 0.0016174316406250, 0.0002746582031250,
+0.0014038085937500, 0.0003967285156250, 0.0015869140625000, 0.0002136230468750,
+0.0013427734375000, 0.0003356933593750, 0.0011901855468750, 0.0005798339843750,
+0.0020446777343750, 0.0005187988281250, 0.0016479492187500, 0.0005187988281250,
+0.0020446777343750, 0.0008544921875000, 0.0028686523437500, 0.0010681152343750,
+0.0021972656250000, 0.0009765625000000, 0.0034790039062500, 0.0010681152343750,
+0.0033874511718750, 0.0012817382812500, 0.0032348632812500, 0.0008850097656250,
+0.0039672851562500, 0.0004882812500000, 0.0037536621093750, 0.0009155273437500,
+0.0035400390625000, 0.0001220703125000, 0.0038146972656250, 0.0001525878906250,
+0.0034484863281250, 0.0003051757812500, 0.0030517578125000, 0.0000305175781250,
+0.0035400390625000, 0.0002746582031250, 0.0025329589843750, 0.0002136230468750,
+0.0027160644531250, 0.0002441406250000, 0.0026245117187500, 0.0002441406250000,
+0.0018310546875000, 0.0002136230468750, 0.0022583007812500, 0.0002441406250000,
+0.0016784667968750, 0.0002441406250000, 0.0014953613281250, 0.0002136230468750,
+0.0015258789062500, 0.0002441406250000, 0.0011901855468750, 0.0002746582031250,
+0.0009460449218750, 0.0003356933593750, 0.0011291503906250, 0.0002136230468750,
+0.0007019042968750, 0.0003051757812500, 0.0007629394531250, 0.0002746582031250,
+0.0009460449218750, 0.0001831054687500, 0.0003356933593750, 0.0003356933593750,
+0.0009765625000000, 0.0003356933593750, 0.0006103515625000, 0.0002746582031250,
+0.0007629394531250, 0.0003051757812500, 0.0010681152343750, 0.0003662109375000,
+0.0000610351562500, 0.0002746582031250, 0.0011901855468750, 0.0002441406250000,
+0.0005187988281250, 0.0002441406250000, 0.0006103515625000, 0.0001831054687500,
+0.0014038085937500, 0.0000915527343750, 0.0004882812500000, 0.0000915527343750,
+0.0015869140625000, 0.0001220703125000, 0.0010681152343750, 0.0000000000000000,
+0.0011291503906250, -0.0001220703125000, 0.0017395019531250, -0.0000915527343750,
+0.0008239746093750, -0.0002746582031250, 0.0017395019531250, -0.0002441406250000,
+0.0012817382812500, -0.0003967285156250, 0.0012512207031250, -0.0005187988281250,
+0.0018310546875000, -0.0004882812500000, 0.0011291503906250, -0.0006103515625000,
+0.0018005371093750, -0.0007934570312500, 0.0015869140625000, -0.0007324218750000,
+0.0015563964843750, -0.0009765625000000, 0.0019836425781250, -0.0012207031250000,
+0.0014648437500000, -0.0011901855468750, 0.0019226074218750, -0.0015869140625000,
+0.0019226074218750, -0.0017395019531250, 0.0019226074218750, -0.0017700195312500,
+0.0022888183593750, -0.0020446777343750, 0.0022277832031250, -0.0021972656250000,
+0.0025634765625000, -0.0020446777343750, 0.0027465820312500, -0.0022583007812500,
+0.0029602050781250, -0.0022583007812500, 0.0035095214843750, -0.0021362304687500,
+0.0038757324218750, -0.0024108886718750, 0.0040283203125000, -0.0018920898437500,
+0.0048217773437500, -0.0022277832031250, 0.0050048828125000, -0.0020141601562500,
+0.0049133300781250, -0.0014648437500000, 0.0056457519531250, -0.0019836425781250,
+0.0051574707031250, -0.0012512207031250, 0.0050659179687500, -0.0012817382812500,
+0.0050964355468750, -0.0013122558593750, 0.0047912597656250, -0.0009765625000000,
+0.0045776367187500, -0.0011596679687500, 0.0045166015625000, -0.0008544921875000,
+0.0042419433593750, -0.0011596679687500, 0.0041198730468750, -0.0010375976562500,
+0.0039978027343750, -0.0007934570312500, 0.0035705566406250, -0.0013427734375000,
+0.0039062500000000, -0.0005187988281250, 0.0032653808593750, -0.0011901855468750,
+0.0032958984375000, -0.0010986328125000, 0.0036010742187500, -0.0005798339843750,
+0.0026550292968750, -0.0018005371093750, 0.0035095214843750, -0.0007629394531250,
+0.0030212402343750, -0.0017089843750000, 0.0028076171875000, -0.0018310546875000,
+0.0033264160156250, -0.0012512207031250, 0.0025939941406250, -0.0025939941406250,
+0.0025939941406250, -0.0013427734375000, 0.0028076171875000, -0.0019836425781250,
+0.0019836425781250, -0.0021057128906250, 0.0015563964843750, -0.0011901855468750,
+0.0021057128906250, -0.0019836425781250, 0.0006713867187500, -0.0014343261718750,
+0.0012207031250000, -0.0015258789062500, 0.0014343261718750, -0.0014953613281250,
+0.0010375976562500, -0.0012817382812500, 0.0019836425781250, -0.0016784667968750,
+0.0011901855468750, -0.0010375976562500, 0.0024719238281250, -0.0014343261718750,
+0.0021972656250000, -0.0012817382812500, 0.0014953613281250, -0.0007629394531250,
+0.0034179687500000, -0.0013427734375000, 0.0011596679687500, -0.0007019042968750,
+0.0025329589843750, -0.0007019042968750, 0.0027770996093750, -0.0008239746093750,
+0.0008544921875000, -0.0003356933593750, 0.0019226074218750, -0.0002136230468750,
+0.0012817382812500, -0.0010375976562500, 0.0010681152343750, -0.0004577636718750,
+0.0010070800781250, -0.0007324218750000, 0.0008544921875000, -0.0015869140625000,
+0.0008544921875000, -0.0008239746093750, -0.0002136230468750, -0.0014038085937500,
+0.0001831054687500, -0.0020751953125000, -0.0003356933593750, -0.0016784667968750,
+-0.0014953613281250, -0.0015869140625000, -0.0006713867187500, -0.0025024414062500,
+-0.0014648437500000, -0.0015563964843750, -0.0015563964843750, -0.0022277832031250,
+-0.0011596679687500, -0.0024719238281250, -0.0015869140625000, -0.0014343261718750,
+-0.0022888183593750, -0.0017089843750000, -0.0021057128906250, -0.0025939941406250,
+-0.0021057128906250, -0.0022583007812500, -0.0022888183593750, -0.0018310546875000,
+-0.0023498535156250, -0.0020751953125000, -0.0023803710937500, -0.0016479492187500,
+-0.0020446777343750, -0.0004882812500000, -0.0024108886718750, -0.0008850097656250,
+-0.0021667480468750, -0.0003662109375000, -0.0016784667968750, 0.0004882812500000,
+-0.0022888183593750, -0.0004272460937500, -0.0019836425781250, 0.0007629394531250,
+-0.0020141601562500, 0.0005187988281250, -0.0021667480468750, 0.0007324218750000,
+-0.0021667480468750, 0.0017395019531250, -0.0021667480468750, 0.0013732910156250,
+-0.0021057128906250, 0.0022277832031250, -0.0018310546875000, 0.0024719238281250,
+-0.0018615722656250, 0.0024108886718750, -0.0016784667968750, 0.0026550292968750,
+-0.0010986328125000, 0.0025024414062500, -0.0017700195312500, 0.0024414062500000,
+-0.0014648437500000, 0.0022583007812500, -0.0014343261718750, 0.0022583007812500,
+-0.0019836425781250, 0.0022888183593750, -0.0016784667968750, 0.0019531250000000,
+-0.0011901855468750, 0.0021362304687500, -0.0014648437500000, 0.0023498535156250,
+-0.0011901855468750, 0.0018615722656250, -0.0005187988281250, 0.0019531250000000,
+-0.0007629394531250, 0.0027465820312500, -0.0008239746093750, 0.0012512207031250,
+-0.0004882812500000, 0.0023498535156250, -0.0007324218750000, 0.0023498535156250,
+-0.0010375976562500, 0.0013122558593750, -0.0005493164062500, 0.0027160644531250,
+-0.0008544921875000, 0.0019531250000000, -0.0009155273437500, 0.0025329589843750,
+-0.0005798339843750, 0.0028076171875000, -0.0005187988281250, 0.0025329589843750,
+-0.0005798339843750, 0.0034790039062500, 0.0000915527343750, 0.0026245117187500,
+0.0000610351562500, 0.0033264160156250, 0.0002136230468750, 0.0030517578125000,
+0.0005798339843750, 0.0022583007812500, 0.0003967285156250, 0.0034484863281250,
+0.0005493164062500, 0.0017700195312500, 0.0007019042968750, 0.0024108886718750,
+0.0005187988281250, 0.0023498535156250, 0.0005187988281250, 0.0011596679687500,
+0.0007629394531250, 0.0022888183593750, 0.0003051757812500, 0.0015258789062500,
+0.0007019042968750, 0.0017700195312500, 0.0005798339843750, 0.0022888183593750,
+0.0002441406250000, 0.0021667480468750, 0.0009460449218750, 0.0024414062500000,
+0.0000915527343750, 0.0027770996093750, 0.0004272460937500, 0.0028076171875000,
+0.0002136230468750, 0.0027770996093750, -0.0005187988281250, 0.0028686523437500,
+0.0000915527343750, 0.0028076171875000, -0.0009460449218750, 0.0027160644531250,
+-0.0007629394531250, 0.0026245117187500, -0.0008544921875000, 0.0025634765625000,
+-0.0016784667968750, 0.0024414062500000, -0.0013122558593750, 0.0024414062500000,
+-0.0016479492187500},
+{0.0018920898437500, -0.0003662109375000, 0.0010375976562500, -0.0011596679687500,
+0.0008544921875000, -0.0012207031250000, 0.0009765625000000, -0.0009765625000000,
+0.0002746582031250, -0.0013732910156250, 0.0000000000000000, -0.0018005371093750,
+0.0003662109375000, -0.0011596679687500, -0.0004272460937500, -0.0017395019531250,
+-0.0005798339843750, -0.0017700195312500, -0.0001525878906250, -0.0014648437500000,
+-0.0010070800781250, -0.0023193359375000, -0.0009765625000000, -0.0014343261718750,
+-0.0010681152343750, -0.0023193359375000, -0.0017089843750000, -0.0024108886718750,
+-0.0014648437500000, -0.0011596679687500, -0.0006713867187500, -0.0014648437500000,
+-0.0018615722656250, -0.0010070800781250, -0.0013732910156250, -0.0012207031250000,
+-0.0014038085937500, -0.0006713867187500, -0.0021667480468750, 0.0000305175781250,
+-0.0014343261718750, -0.0006103515625000, -0.0021667480468750, 0.0010375976562500,
+-0.0018920898437500, 0.0002441406250000, -0.0017395019531250, 0.0004882812500000,
+-0.0022277832031250, 0.0017700195312500, -0.0019531250000000, 0.0007324218750000,
+-0.0019226074218750, 0.0019836425781250, -0.0022888183593750, 0.0021667480468750,
+-0.0022888183593750, 0.0019836425781250, -0.0020751953125000, 0.0025024414062500,
+-0.0023193359375000, 0.0026245117187500, -0.0026245117187500, 0.0024414062500000,
+-0.0024108886718750, 0.0026245117187500, -0.0026245117187500, 0.0025634765625000,
+-0.0028991699218750, 0.0023803710937500, -0.0024108886718750, 0.0026245117187500,
+-0.0026855468750000, 0.0025329589843750, -0.0024719238281250, 0.0026245117187500,
+-0.0021972656250000, 0.0022277832031250, -0.0023803710937500, 0.0020751953125000,
+-0.0023498535156250, 0.0024719238281250, -0.0020141601562500, 0.0011291503906250,
+-0.0025634765625000, 0.0018920898437500, -0.0019531250000000, 0.0017700195312500,
+-0.0013122558593750, 0.0008239746093750, -0.0022583007812500, 0.0021057128906250,
+-0.0006103515625000, 0.0014953613281250, -0.0007324218750000, 0.0018310546875000,
+-0.0008850097656250, 0.0021057128906250, 0.0000305175781250, 0.0020446777343750,
+-0.0002136230468750, 0.0028076171875000, -0.0004882812500000, 0.0018615722656250,
+-0.0001220703125000, 0.0026550292968750, -0.0002441406250000, 0.0022583007812500,
+-0.0007629394531250, 0.0013122558593750, -0.0003967285156250, 0.0026550292968750,
+-0.0004577636718750, 0.0007934570312500, -0.0007019042968750, 0.0016784667968750,
+-0.0003356933593750, 0.0018005371093750, 0.0000000000000000, 0.0007934570312500,
+-0.0003662109375000, 0.0022277832031250, 0.0004272460937500, 0.0015258789062500,
+0.0003662109375000, 0.0019836425781250, 0.0002441406250000, 0.0024108886718750,
+0.0007019042968750, 0.0021972656250000, 0.0006103515625000, 0.0028076171875000,
+0.0003662109375000, 0.0025634765625000, 0.0006408691406250, 0.0028991699218750,
+0.0004577636718750, 0.0029602050781250, 0.0003051757812500, 0.0027160644531250,
+0.0007629394531250, 0.0029296875000000, 0.0001220703125000, 0.0024719238281250,
+0.0005798339843750, 0.0024414062500000, 0.0003662109375000, 0.0024414062500000,
+-0.0001525878906250, 0.0021057128906250, 0.0006103515625000, 0.0021362304687500,
+-0.0005798339843750, 0.0022277832031250, -0.0001525878906250, 0.0020751953125000,
+-0.0003662109375000, 0.0023193359375000, -0.0012207031250000, 0.0025329589843750,
+-0.0004882812500000, 0.0022583007812500, -0.0012817382812500, 0.0029602050781250,
+-0.0012207031250000, 0.0025024414062500, -0.0009460449218750, 0.0027770996093750,
+-0.0013427734375000, 0.0034484863281250, -0.0012817382812500, 0.0025024414062500,
+-0.0010375976562500, 0.0032348632812500, -0.0013427734375000, 0.0029602050781250,
+-0.0011596679687500, 0.0025634765625000, -0.0009460449218750, 0.0027160644531250,
+-0.0014953613281250, 0.0024414062500000, -0.0008544921875000, 0.0020446777343750,
+-0.0012207031250000, 0.0019836425781250, -0.0010986328125000, 0.0017089843750000,
+-0.0006103515625000, 0.0013122558593750, -0.0012207031250000, 0.0014038085937500,
+-0.0003051757812500, 0.0010375976562500, -0.0005187988281250, 0.0009765625000000,
+-0.0004577636718750, 0.0009155273437500, 0.0000305175781250, 0.0007934570312500,
+-0.0002746582031250, 0.0009155273437500, 0.0001525878906250, 0.0007324218750000,
+0.0000915527343750, 0.0008850097656250, 0.0000610351562500, 0.0009460449218750,
+0.0003051757812500, 0.0009765625000000, 0.0001220703125000, 0.0012207031250000,
+0.0001831054687500, 0.0012512207031250, 0.0001220703125000, 0.0013732910156250,
+0.0001525878906250, 0.0015563964843750, 0.0002746582031250, 0.0016479492187500,
+0.0000610351562500, 0.0017089843750000, 0.0004882812500000, 0.0017700195312500,
+0.0003356933593750, 0.0020141601562500, 0.0004272460937500, 0.0020446777343750,
+0.0006713867187500, 0.0020446777343750, 0.0003662109375000, 0.0024108886718750,
+0.0006408691406250, 0.0021972656250000, 0.0003967285156250, 0.0025634765625000,
+0.0002441406250000, 0.0026855468750000, 0.0003051757812500, 0.0025939941406250,
+-0.0001220703125000, 0.0030517578125000, -0.0001525878906250, 0.0030212402343750,
+-0.0002136230468750, 0.0032653808593750, -0.0004577636718750, 0.0033874511718750,
+-0.0005493164062500, 0.0034179687500000, -0.0004272460937500, 0.0039062500000000,
+-0.0005493164062500, 0.0034484863281250, -0.0004577636718750, 0.0039672851562500,
+-0.0003356933593750, 0.0038757324218750, -0.0002746582031250, 0.0035400390625000,
+-0.0001525878906250, 0.0042114257812500, -0.0001220703125000, 0.0032043457031250,
+0.0000915527343750, 0.0036315917968750, 0.0000305175781250, 0.0035095214843750,
+-0.0000915527343750, 0.0027465820312500, 0.0001220703125000, 0.0034179687500000,
+-0.0000305175781250, 0.0027465820312500, -0.0000305175781250, 0.0028991699218750,
+-0.0000915527343750, 0.0031738281250000, -0.0001220703125000, 0.0029602050781250,
+-0.0001525878906250, 0.0032653808593750, -0.0006713867187500, 0.0036315917968750,
+-0.0003662109375000, 0.0037231445312500, -0.0009765625000000, 0.0041503906250000,
+-0.0017395019531250, 0.0046691894531250, -0.0012207031250000, 0.0046386718750000,
+-0.0025024414062500, 0.0047302246093750, -0.0023498535156250, 0.0052795410156250,
+-0.0025024414062500, 0.0049743652343750, -0.0030822753906250, 0.0046386718750000,
+-0.0023803710937500, 0.0051879882812500, -0.0032653808593750, 0.0038146972656250,
+-0.0024719238281250, 0.0043945312500000, -0.0024414062500000, 0.0038757324218750,
+-0.0029907226562500, 0.0027160644531250, -0.0018615722656250, 0.0037536621093750,
+-0.0023803710937500, 0.0022277832031250, -0.0022277832031250, 0.0024719238281250,
+-0.0018615722656250, 0.0027160644531250, -0.0020141601562500, 0.0021057128906250,
+-0.0020446777343750, 0.0027160644531250, -0.0025634765625000, 0.0027770996093750,
+-0.0019531250000000, 0.0030212402343750, -0.0026550292968750, 0.0031433105468750,
+-0.0034484863281250, 0.0032653808593750, -0.0021362304687500, 0.0035705566406250,
+-0.0034179687500000, 0.0031127929687500, -0.0030517578125000, 0.0033264160156250,
+-0.0025634765625000, 0.0029602050781250, -0.0028686523437500, 0.0024108886718750,
+-0.0024414062500000, 0.0027160644531250, -0.0024108886718750, 0.0016479492187500,
+-0.0019531250000000, 0.0016784667968750, -0.0018615722656250, 0.0017089843750000,
+-0.0021667480468750, 0.0011291503906250, -0.0018005371093750, 0.0012207031250000,
+-0.0018310546875000, 0.0014038085937500, -0.0026855468750000, 0.0007629394531250,
+-0.0023803710937500, 0.0010681152343750, -0.0021667480468750, 0.0015258789062500,
+-0.0033874511718750, 0.0006713867187500, -0.0025024414062500, 0.0014953613281250,
+-0.0029296875000000, 0.0010986328125000, -0.0031433105468750, 0.0010986328125000,
+-0.0025329589843750, 0.0014343261718750, -0.0029296875000000, 0.0008544921875000,
+-0.0025329589843750, 0.0022277832031250, -0.0024108886718750, 0.0014648437500000,
+-0.0021972656250000, 0.0021057128906250, -0.0018920898437500, 0.0034484863281250,
+-0.0017700195312500, 0.0020446777343750, -0.0010986328125000, 0.0032653808593750,
+-0.0011596679687500, 0.0032348632812500, -0.0008850097656250, 0.0020751953125000,
+-0.0000305175781250, 0.0014953613281250, 0.0012512207031250, 0.0019226074218750,
+0.0044555664062500, 0.0037231445312500, 0.0030822753906250, 0.0030517578125000,
+0.0038452148437500, 0.0022277832031250, 0.0056457519531250, 0.0019226074218750,
+0.0046997070312500, 0.0014343261718750, 0.0047302246093750, 0.0006103515625000,
+0.0050659179687500, 0.0000000000000000, 0.0041503906250000, -0.0003967285156250,
+0.0029907226562500, -0.0007324218750000, 0.0031738281250000, -0.0011901855468750,
+0.0023803710937500, -0.0019226074218750, 0.0015258789062500, -0.0017700195312500,
+0.0021057128906250, -0.0025024414062500, 0.0024719238281250, -0.0036010742187500,
+0.0014953613281250, -0.0030517578125000, 0.0029602050781250, -0.0035400390625000,
+0.0028076171875000, -0.0040588378906250, 0.0022583007812500, -0.0032958984375000,
+0.0028076171875000, -0.0026855468750000, 0.0026245117187500, -0.0032348632812500,
+0.0015869140625000, -0.0018615722656250, 0.0016174316406250, -0.0014953613281250,
+0.0015563964843750, -0.0019531250000000, 0.0009155273437500, -0.0016784667968750,
+0.0009765625000000, -0.0011901855468750, 0.0017395019531250, -0.0025939941406250,
+0.0015563964843750, -0.0023498535156250, 0.0018615722656250, -0.0024108886718750,
+0.0027465820312500, -0.0032043457031250, 0.0026245117187500, -0.0027770996093750,
+0.0023803710937500, -0.0032653808593750, 0.0032653808593750, -0.0028991699218750,
+0.0024108886718750, -0.0030822753906250, 0.0014648437500000, -0.0035095214843750,
+0.0028076171875000, -0.0028686523437500, 0.0008544921875000, -0.0037841796875000,
+0.0010070800781250, -0.0037231445312500, 0.0014953613281250, -0.0033264160156250,
+0.0007324218750000, -0.0035095214843750, 0.0011291503906250, -0.0032653808593750,
+0.0012817382812500, -0.0025939941406250, 0.0012817382812500, -0.0025024414062500,
+0.0016174316406250, -0.0022888183593750, 0.0020141601562500, -0.0019226074218750,
+0.0021972656250000, -0.0019836425781250, 0.0020751953125000, -0.0017700195312500,
+0.0029907226562500, -0.0019226074218750, 0.0024108886718750, -0.0016174316406250,
+0.0018005371093750, -0.0013122558593750, 0.0032043457031250, -0.0017395019531250,
+0.0008544921875000, -0.0007629394531250, 0.0017089843750000, -0.0009460449218750,
+0.0015563964843750, -0.0007629394531250, 0.0000610351562500, -0.0001525878906250,
+0.0015869140625000, -0.0005798339843750, 0.0000000000000000, 0.0001525878906250,
+0.0006408691406250, 0.0000305175781250, 0.0007324218750000, 0.0000915527343750,
+-0.0000915527343750, 0.0005798339843750, 0.0011291503906250, 0.0004577636718750,
+0.0004882812500000, 0.0006713867187500, 0.0009155273437500, 0.0006713867187500,
+0.0014343261718750, 0.0009460449218750, 0.0013732910156250, 0.0011291503906250,
+0.0018615722656250, 0.0008850097656250, 0.0020141601562500, 0.0015563964843750,
+0.0023193359375000, 0.0013122558593750, 0.0023803710937500, 0.0011291503906250,
+0.0023803710937500, 0.0015258789062500, 0.0025939941406250, 0.0011901855468750,
+0.0022583007812500, 0.0010375976562500, 0.0023498535156250, 0.0010681152343750,
+0.0021362304687500, 0.0008850097656250, 0.0017089843750000, 0.0007324218750000,
+0.0019836425781250, 0.0007629394531250, 0.0012207031250000, 0.0007324218750000,
+0.0012207031250000, 0.0007629394531250, 0.0010375976562500, 0.0007324218750000,
+0.0005493164062500, 0.0007324218750000, 0.0006408691406250, 0.0007629394531250,
+0.0001220703125000, 0.0006408691406250, 0.0000915527343750, 0.0006408691406250,
+-0.0000305175781250},
+{0.0065002441406250, 0.0025024414062500, 0.0054321289062500, 0.0029907226562500,
+0.0046997070312500, 0.0036926269531250, 0.0051269531250000, 0.0026245117187500,
+0.0039062500000000, 0.0034484863281250, 0.0032958984375000, 0.0032653808593750,
+0.0037231445312500, 0.0025634765625000, 0.0036010742187500, 0.0023193359375000,
+0.0028991699218750, 0.0018310546875000, 0.0036926269531250, 0.0014343261718750,
+0.0035705566406250, 0.0007324218750000, 0.0028381347656250, 0.0004272460937500,
+0.0026855468750000, 0.0003662109375000, 0.0025634765625000, -0.0002746582031250,
+0.0015869140625000, -0.0006408691406250, 0.0009765625000000, -0.0007019042968750,
+0.0013122558593750, -0.0012207031250000, 0.0010681152343750, -0.0017700195312500,
+0.0002746582031250, -0.0016784667968750, 0.0024414062500000, -0.0021057128906250,
+0.0017395019531250, -0.0023803710937500, 0.0018615722656250, -0.0020446777343750,
+0.0035705566406250, -0.0018310546875000, 0.0026855468750000, -0.0019226074218750,
+0.0026550292968750, -0.0012817382812500, 0.0030517578125000, -0.0010375976562500,
+0.0023498535156250, -0.0013122558593750, 0.0015869140625000, -0.0012817382812500,
+0.0020446777343750, -0.0011901855468750, 0.0014038085937500, -0.0019531250000000,
+0.0010375976562500, -0.0019531250000000, 0.0016479492187500, -0.0021362304687500,
+0.0018615722656250, -0.0025329589843750, 0.0014953613281250, -0.0023498535156250,
+0.0027465820312500, -0.0028686523437500, 0.0029907226562500, -0.0026245117187500,
+0.0026855468750000, -0.0029602050781250, 0.0031433105468750, -0.0035400390625000,
+0.0034179687500000, -0.0029907226562500, 0.0023193359375000, -0.0037841796875000,
+0.0028991699218750, -0.0037231445312500, 0.0023803710937500, -0.0032348632812500,
+0.0012817382812500, -0.0032958984375000, 0.0021972656250000, -0.0032958984375000,
+0.0007934570312500, -0.0024414062500000, 0.0007324218750000, -0.0025939941406250,
+0.0010375976562500, -0.0022888183593750, 0.0005798339843750, -0.0016784667968750,
+0.0007629394531250, -0.0021362304687500, 0.0010375976562500, -0.0012207031250000,
+0.0012512207031250, -0.0015258789062500, 0.0013427734375000, -0.0013122558593750,
+0.0015869140625000, -0.0005798339843750, 0.0021362304687500, -0.0011291503906250,
+0.0017089843750000, -0.0003051757812500, 0.0024414062500000, -0.0005187988281250,
+0.0024108886718750, -0.0003967285156250, 0.0021057128906250, 0.0001525878906250,
+0.0031127929687500, -0.0002441406250000, 0.0016784667968750, 0.0003662109375000,
+0.0025939941406250, 0.0003356933593750, 0.0023803710937500, 0.0004272460937500,
+0.0011901855468750, 0.0007934570312500, 0.0025024414062500, 0.0005798339843750,
+0.0010681152343750, 0.0010681152343750, 0.0013427734375000, 0.0009460449218750,
+0.0014648437500000, 0.0010681152343750, 0.0007324218750000, 0.0014343261718750,
+0.0013122558593750, 0.0009155273437500, 0.0007934570312500, 0.0013732910156250,
+0.0008239746093750, 0.0011596679687500, 0.0009765625000000, 0.0009460449218750,
+0.0008239746093750, 0.0010986328125000, 0.0010375976562500, 0.0008850097656250,
+0.0012512207031250, 0.0007629394531250, 0.0012207031250000, 0.0006408691406250,
+0.0013732910156250, 0.0007019042968750, 0.0016784667968750, 0.0007324218750000,
+0.0015258789062500, 0.0005493164062500, 0.0017700195312500, 0.0007934570312500,
+0.0018920898437500, 0.0007934570312500, 0.0018920898437500, 0.0007324218750000,
+0.0020446777343750, 0.0007629394531250, 0.0021362304687500, 0.0006713867187500,
+0.0021362304687500, 0.0007019042968750, 0.0022888183593750, 0.0005798339843750,
+0.0022888183593750, 0.0006103515625000, 0.0023193359375000, 0.0007019042968750,
+0.0026245117187500, 0.0004272460937500, 0.0024719238281250, 0.0007324218750000,
+0.0024414062500000, 0.0005187988281250, 0.0025939941406250, 0.0004272460937500,
+0.0025329589843750, 0.0005493164062500, 0.0024108886718750, 0.0003051757812500,
+0.0028076171875000, 0.0003051757812500, 0.0023803710937500, 0.0002136230468750,
+0.0025024414062500, -0.0000305175781250, 0.0027770996093750, -0.0001220703125000,
+0.0021972656250000, -0.0002441406250000, 0.0029296875000000, -0.0003967285156250,
+0.0024414062500000, -0.0004272460937500, 0.0025634765625000, -0.0007019042968750,
+0.0029907226562500, -0.0009155273437500, 0.0022277832031250, -0.0007934570312500,
+0.0030517578125000, -0.0011901855468750, 0.0024108886718750, -0.0011596679687500,
+0.0024719238281250, -0.0011901855468750, 0.0030212402343750, -0.0013427734375000,
+0.0019226074218750, -0.0012512207031250, 0.0030212402343750, -0.0011291503906250,
+0.0025024414062500, -0.0011596679687500, 0.0022888183593750, -0.0010375976562500,
+0.0029602050781250, -0.0008850097656250, 0.0022583007812500, -0.0009765625000000,
+0.0025024414062500, -0.0006103515625000, 0.0025024414062500, -0.0005187988281250,
+0.0021362304687500, -0.0003967285156250, 0.0020446777343750, -0.0001525878906250,
+0.0020141601562500, -0.0000610351562500, 0.0018005371093750, 0.0000915527343750,
+0.0018920898437500, 0.0001220703125000, 0.0018310546875000, 0.0003356933593750,
+0.0018310546875000, 0.0004577636718750, 0.0022583007812500, 0.0002136230468750,
+0.0019836425781250, 0.0007019042968750, 0.0026550292968750, 0.0003051757812500,
+0.0026245117187500, 0.0002746582031250, 0.0025634765625000, 0.0004577636718750,
+0.0035705566406250, -0.0003051757812500, 0.0025634765625000, 0.0000305175781250,
+0.0029907226562500, -0.0003967285156250, 0.0029907226562500, -0.0007324218750000,
+0.0022277832031250, -0.0004577636718750, 0.0028686523437500, -0.0006408691406250,
+0.0021362304687500, -0.0009460449218750, 0.0021362304687500, -0.0003662109375000,
+0.0023193359375000, -0.0006408691406250, 0.0018920898437500, -0.0008850097656250,
+0.0018920898437500, 0.0000915527343750, 0.0024108886718750, -0.0011291503906250,
+0.0018920898437500, -0.0001831054687500, 0.0024719238281250, -0.0002746582031250,
+0.0031433105468750, -0.0011901855468750, 0.0020751953125000, 0.0002441406250000,
+0.0040588378906250, -0.0012512207031250, 0.0032043457031250, -0.0006713867187500,
+0.0035095214843750, -0.0006408691406250, 0.0049743652343750, -0.0016174316406250,
+0.0033874511718750, -0.0007934570312500, 0.0050964355468750, -0.0012817382812500,
+0.0046081542968750, -0.0013427734375000, 0.0036621093750000, -0.0009460449218750,
+0.0039367675781250, -0.0009155273437500, 0.0035400390625000, -0.0007934570312500,
+0.0037841796875000, -0.0006103515625000, 0.0032653808593750, -0.0004272460937500,
+0.0035705566406250, -0.0005493164062500, 0.0042114257812500, -0.0006713867187500,
+0.0034484863281250, -0.0005187988281250, 0.0041503906250000, -0.0007324218750000,
+0.0041503906250000, -0.0010375976562500, 0.0035400390625000, -0.0010375976562500,
+0.0036926269531250, -0.0010986328125000, 0.0039367675781250, -0.0014038085937500,
+0.0027465820312500, -0.0011901855468750, 0.0024414062500000, -0.0017089843750000,
+0.0024719238281250, -0.0017700195312500, 0.0020751953125000, -0.0016479492187500,
+0.0021362304687500, -0.0021972656250000, 0.0022277832031250, -0.0015258789062500,
+0.0021362304687500, -0.0018310546875000, 0.0018615722656250, -0.0018920898437500,
+0.0016784667968750, -0.0012817382812500, 0.0014648437500000, -0.0015563964843750,
+0.0006103515625000, -0.0012207031250000, 0.0003967285156250, -0.0010986328125000,
+-0.0000915527343750, -0.0006408691406250, -0.0006103515625000, -0.0003051757812500,
+0.0000915527343750, -0.0002136230468750, -0.0003051757812500, 0.0019226074218750,
+-0.0002441406250000, 0.0010070800781250, -0.0001525878906250, 0.0010070800781250,
+-0.0001831054687500, 0.0024108886718750, -0.0000305175781250, 0.0014648437500000,
+-0.0007934570312500, 0.0019226074218750, -0.0002746582031250, 0.0022277832031250,
+-0.0007324218750000, 0.0020751953125000, -0.0014953613281250, 0.0021972656250000,
+-0.0007324218750000, 0.0025024414062500, -0.0019531250000000, 0.0027465820312500,
+-0.0015258789062500, 0.0027160644531250, -0.0014648437500000, 0.0026550292968750,
+-0.0022888183593750, 0.0028686523437500, -0.0016784667968750, 0.0028991699218750,
+-0.0019836425781250, 0.0022583007812500, -0.0024108886718750, 0.0025939941406250,
+-0.0017700195312500, 0.0021972656250000, -0.0016174316406250, 0.0016479492187500,
+-0.0024108886718750, 0.0024108886718750, -0.0006713867187500, 0.0013122558593750,
+-0.0012207031250000, 0.0020446777343750, -0.0012817382812500, 0.0021057128906250,
+0.0000000000000000, 0.0014648437500000, -0.0005798339843750, 0.0025939941406250,
+-0.0007324218750000, 0.0012512207031250, -0.0000915527343750, 0.0018005371093750,
+-0.0005798339843750, 0.0016784667968750, -0.0012207031250000, 0.0005187988281250,
+-0.0007019042968750, 0.0012817382812500, -0.0016479492187500, 0.0007934570312500,
+-0.0018005371093750, 0.0004577636718750, -0.0015563964843750, 0.0007934570312500,
+-0.0018005371093750, 0.0009155273437500, -0.0019836425781250, 0.0007629394531250,
+-0.0010986328125000, 0.0010986328125000, -0.0016174316406250, 0.0013732910156250,
+-0.0012207031250000, 0.0013732910156250, -0.0003967285156250, 0.0015563964843750,
+-0.0013122558593750, 0.0018310546875000, -0.0000915527343750, 0.0014648437500000,
+-0.0002136230468750, 0.0018310546875000, -0.0003662109375000, 0.0019531250000000,
+0.0003051757812500, 0.0017089843750000, 0.0001220703125000, 0.0021972656250000,
+-0.0000915527343750, 0.0024108886718750, -0.0000305175781250, 0.0023498535156250,
+-0.0001220703125000, 0.0027770996093750, -0.0004882812500000, 0.0031433105468750,
+-0.0006103515625000, 0.0028991699218750, -0.0003051757812500, 0.0030822753906250,
+-0.0009155273437500, 0.0032043457031250, -0.0008239746093750, 0.0027770996093750,
+-0.0004272460937500, 0.0025939941406250, -0.0012817382812500, 0.0028686523437500,
+-0.0005493164062500, 0.0019531250000000, -0.0009765625000000, 0.0021667480468750,
+-0.0010070800781250, 0.0022583007812500, -0.0005798339843750, 0.0017700195312500,
+-0.0011291503906250, 0.0020446777343750, -0.0003662109375000, 0.0019836425781250,
+-0.0005798339843750, 0.0018310546875000, -0.0004577636718750, 0.0021667480468750,
+0.0001525878906250, 0.0023498535156250, -0.0003051757812500, 0.0020141601562500,
+-0.0000915527343750, 0.0026550292968750, 0.0000915527343750, 0.0023498535156250,
+-0.0003356933593750, 0.0022888183593750, -0.0005493164062500, 0.0025329589843750,
+-0.0002746582031250, 0.0021667480468750, -0.0008544921875000, 0.0024719238281250,
+-0.0008850097656250, 0.0022888183593750, -0.0007629394531250, 0.0021972656250000,
+-0.0009460449218750, 0.0024108886718750, -0.0008850097656250, 0.0022583007812500,
+-0.0007019042968750, 0.0023498535156250, -0.0012512207031250, 0.0021972656250000,
+-0.0009765625000000, 0.0021667480468750, -0.0005798339843750, 0.0022277832031250,
+-0.0012817382812500, 0.0019836425781250, -0.0003051757812500, 0.0022888183593750,
+-0.0007324218750000, 0.0019836425781250, -0.0006103515625000, 0.0018920898437500,
+0.0000305175781250, 0.0021362304687500, -0.0007629394531250, 0.0017395019531250,
+0.0001831054687500, 0.0020446777343750, -0.0002441406250000, 0.0017089843750000,
+-0.0001525878906250, 0.0017089843750000, 0.0004882812500000, 0.0019226074218750,
+-0.0002441406250000, 0.0014038085937500, 0.0006408691406250, 0.0018920898437500,
+0.0003662109375000, 0.0015869140625000, 0.0003967285156250, 0.0015258789062500,
+0.0008544921875000, 0.0018615722656250, 0.0003051757812500, 0.0014648437500000,
+0.0006713867187500, 0.0017395019531250, 0.0005798339843750, 0.0015563964843750,
+0.0003967285156250, 0.0016174316406250, 0.0006103515625000, 0.0018005371093750,
+0.0004272460937500},
+{-0.0014343261718750, 0.0008239746093750, -0.0014953613281250, 0.0017700195312500,
+-0.0012512207031250, 0.0007019042968750, -0.0017089843750000, 0.0010986328125000,
+-0.0012817382812500, 0.0010681152343750, -0.0007934570312500, 0.0003356933593750,
+-0.0014953613281250, 0.0010681152343750, -0.0001525878906250, 0.0006408691406250,
+-0.0000610351562500, 0.0005798339843750, -0.0002441406250000, 0.0009765625000000,
+0.0005493164062500, 0.0010986328125000, 0.0006103515625000, 0.0010681152343750,
+-0.0000915527343750, 0.0011291503906250, 0.0002441406250000, 0.0014648437500000,
+-0.0000915527343750, 0.0012817382812500, -0.0009460449218750, 0.0010375976562500,
+-0.0005798339843750, 0.0015869140625000, -0.0011291503906250, 0.0012207031250000,
+-0.0012207031250000, 0.0012512207031250, -0.0011596679687500, 0.0017089843750000,
+-0.0013427734375000, 0.0016784667968750, -0.0013427734375000, 0.0016174316406250,
+-0.0013122558593750, 0.0024414062500000, -0.0014038085937500, 0.0022888183593750,
+-0.0011901855468750, 0.0025329589843750, -0.0010070800781250, 0.0032043457031250,
+-0.0012207031250000, 0.0027465820312500, -0.0007629394531250, 0.0032653808593750,
+-0.0006408691406250, 0.0032653808593750, -0.0007019042968750, 0.0027465820312500,
+-0.0005187988281250, 0.0027465820312500, -0.0004577636718750, 0.0026855468750000,
+-0.0006408691406250, 0.0019226074218750, -0.0007019042968750, 0.0020446777343750,
+-0.0005187988281250, 0.0018615722656250, -0.0006713867187500, 0.0014038085937500,
+-0.0010375976562500, 0.0017700195312500, -0.0003051757812500, 0.0018005371093750,
+-0.0008544921875000, 0.0016479492187500, -0.0008239746093750, 0.0022583007812500,
+-0.0002746582031250, 0.0025634765625000, -0.0010070800781250, 0.0021667480468750,
+-0.0003967285156250, 0.0031127929687500, -0.0006713867187500, 0.0026855468750000,
+-0.0006713867187500, 0.0027770996093750, -0.0002746582031250, 0.0033264160156250,
+-0.0007934570312500, 0.0024414062500000, -0.0000915527343750, 0.0027770996093750,
+-0.0003662109375000, 0.0025024414062500, -0.0004272460937500, 0.0020446777343750,
+0.0001525878906250, 0.0019836425781250, -0.0002441406250000, 0.0017089843750000,
+-0.0003356933593750, 0.0013122558593750, -0.0001220703125000, 0.0013122558593750,
+-0.0004272460937500, 0.0011901855468750, -0.0007019042968750, 0.0008850097656250,
+-0.0004882812500000, 0.0010681152343750, -0.0007934570312500, 0.0009460449218750,
+-0.0009765625000000, 0.0009155273437500, -0.0008544921875000, 0.0009460449218750,
+-0.0009765625000000, 0.0009460449218750, -0.0011901855468750, 0.0008850097656250,
+-0.0007324218750000, 0.0008850097656250, -0.0011291503906250, 0.0006713867187500,
+-0.0008544921875000, 0.0007629394531250, -0.0003662109375000, 0.0007019042968750,
+-0.0009765625000000, 0.0003662109375000, 0.0000000000000000, 0.0006408691406250,
+-0.0003662109375000, 0.0003356933593750, -0.0001831054687500, 0.0001525878906250,
+0.0004882812500000, 0.0001831054687500, -0.0002136230468750, -0.0002441406250000,
+0.0006408691406250, -0.0004272460937500, 0.0003662109375000, -0.0005798339843750,
+0.0003662109375000, -0.0009765625000000, 0.0007934570312500, -0.0012512207031250,
+0.0001831054687500, -0.0013427734375000, 0.0006408691406250, -0.0018310546875000,
+0.0005187988281250, -0.0017700195312500, 0.0003356933593750, -0.0020751953125000,
+0.0005187988281250, -0.0024108886718750, 0.0003356933593750, -0.0021057128906250,
+0.0005493164062500, -0.0027465820312500, 0.0004577636718750, -0.0025634765625000,
+0.0005493164062500, -0.0025634765625000, 0.0007019042968750, -0.0029602050781250,
+0.0005187988281250, -0.0025939941406250, 0.0007629394531250, -0.0026855468750000,
+0.0006103515625000, -0.0027770996093750, 0.0005493164062500, -0.0025329589843750,
+0.0006103515625000, -0.0023498535156250, 0.0003051757812500, -0.0023803710937500,
+0.0005798339843750, -0.0017700195312500, 0.0002136230468750, -0.0019226074218750,
+0.0000000000000000, -0.0015563964843750, 0.0001831054687500, -0.0009155273437500,
+-0.0003356933593750, -0.0010681152343750, -0.0004882812500000, -0.0005187988281250,
+-0.0007324218750000, 0.0000305175781250, -0.0011596679687500, 0.0000000000000000,
+-0.0013732910156250, 0.0002136230468750, -0.0015258789062500, 0.0008850097656250,
+-0.0020751953125000, 0.0001525878906250, -0.0018615722656250, 0.0009460449218750,
+-0.0021667480468750, 0.0006408691406250, -0.0025024414062500, -0.0001220703125000,
+-0.0018615722656250, 0.0010681152343750, -0.0025024414062500, -0.0004882812500000,
+-0.0019531250000000, 0.0001831054687500, -0.0017395019531250, 0.0001831054687500,
+-0.0020141601562500, -0.0007629394531250, -0.0010986328125000, 0.0003967285156250,
+-0.0013732910156250, -0.0003051757812500, -0.0011901855468750, 0.0000305175781250,
+-0.0009155273437500, 0.0003967285156250, -0.0010070800781250, 0.0002136230468750,
+-0.0010681152343750, 0.0007629394531250, -0.0011901855468750, 0.0009155273437500,
+-0.0011291503906250, 0.0011596679687500, -0.0015563964843750, 0.0013427734375000,
+-0.0018005371093750, 0.0015563964843750, -0.0012512207031250, 0.0018310546875000,
+-0.0021057128906250, 0.0013732910156250, -0.0017395019531250, 0.0015258789062500,
+-0.0013732910156250, 0.0014038085937500, -0.0016784667968750, 0.0007324218750000,
+-0.0013122558593750, 0.0008239746093750, -0.0010375976562500, 0.0012207031250000,
+-0.0011291503906250, 0.0005798339843750, -0.0007019042968750, 0.0013122558593750,
+-0.0004272460937500, 0.0020751953125000, -0.0008239746093750, 0.0009155273437500,
+0.0004577636718750, 0.0028991699218750, -0.0003967285156250, 0.0023498535156250,
+0.0000610351562500, 0.0024108886718750, 0.0009460449218750, 0.0036010742187500,
+-0.0008239746093750, 0.0026550292968750, 0.0006713867187500, 0.0033874511718750,
+-0.0002136230468750, 0.0032958984375000, -0.0000305175781250, 0.0026855468750000,
+0.0011901855468750, 0.0025939941406250, -0.0001831054687500, 0.0025939941406250,
+0.0017089843750000, 0.0022277832031250, 0.0015563964843750, 0.0021667480468750,
+0.0017700195312500, 0.0024108886718750, 0.0031433105468750, 0.0024108886718750,
+0.0027770996093750, 0.0025024414062500, 0.0040588378906250, 0.0034484863281250,
+0.0038146972656250, 0.0031738281250000, 0.0044860839843750, 0.0037536621093750,
+0.0054321289062500, 0.0046691894531250, 0.0044250488281250, 0.0038452148437500,
+0.0066528320312500, 0.0045166015625000, 0.0057983398437500, 0.0041503906250000,
+0.0062255859375000, 0.0040283203125000, 0.0079345703125000, 0.0040283203125000,
+0.0064086914062500, 0.0032653808593750, 0.0075073242187500, 0.0051879882812500,
+0.0076904296875000, 0.0041503906250000, 0.0067443847656250, 0.0043640136718750,
+0.0063781738281250, 0.0072326660156250, 0.0063171386718750, 0.0046081542968750,
+0.0044860839843750, 0.0070495605468750, 0.0050659179687500, 0.0054321289062500,
+0.0042724609375000, 0.0061340332031250, 0.0029296875000000, 0.0078430175781250,
+0.0032043457031250, 0.0066833496093750, 0.0024108886718750, 0.0069580078125000,
+0.0015563964843750, 0.0072631835937500, 0.0021057128906250, 0.0064697265625000,
+0.0021972656250000, 0.0057067871093750, 0.0011291503906250, 0.0058898925781250,
+0.0031127929687500, 0.0053100585937500, 0.0024108886718750, 0.0050048828125000,
+0.0022277832031250, 0.0048828125000000, 0.0033569335937500, 0.0048522949218750,
+0.0022888183593750, 0.0046997070312500, 0.0020751953125000, 0.0036621093750000,
+0.0018005371093750, 0.0039672851562500, 0.0014343261718750, 0.0032653808593750,
+0.0009460449218750, 0.0019226074218750, 0.0006103515625000, 0.0023498535156250,
+0.0016479492187500, 0.0016784667968750, 0.0009155273437500, 0.0010681152343750,
+0.0014038085937500, 0.0012817382812500, 0.0027465820312500, 0.0014953613281250,
+0.0018005371093750, 0.0010070800781250, 0.0027465820312500, 0.0011596679687500,
+0.0028381347656250, 0.0012817382812500, 0.0024719238281250, 0.0006408691406250,
+0.0025939941406250, 0.0000915527343750, 0.0024414062500000, 0.0003356933593750,
+0.0019226074218750, -0.0004882812500000, 0.0017089843750000, -0.0006103515625000,
+0.0018005371093750, -0.0005798339843750, 0.0013122558593750, -0.0008544921875000,
+0.0008239746093750, -0.0008850097656250, 0.0021667480468750, -0.0010986328125000,
+0.0012817382812500, -0.0012817382812500, 0.0017395019531250, -0.0014343261718750,
+0.0029296875000000, -0.0018005371093750, 0.0016479492187500, -0.0020446777343750,
+0.0033569335937500, -0.0018005371093750, 0.0027465820312500, -0.0021362304687500,
+0.0025329589843750, -0.0021057128906250, 0.0034790039062500, -0.0017395019531250,
+0.0023498535156250, -0.0020141601562500, 0.0026855468750000, -0.0021057128906250,
+0.0023803710937500, -0.0018310546875000, 0.0019836425781250, -0.0020751953125000,
+0.0020141601562500, -0.0025024414062500, 0.0014953613281250, -0.0021057128906250,
+0.0018310546875000, -0.0022583007812500, 0.0014343261718750, -0.0025634765625000,
+0.0014648437500000, -0.0021972656250000, 0.0018920898437500, -0.0019226074218750,
+0.0014343261718750, -0.0024108886718750, 0.0018615722656250, -0.0014343261718750,
+0.0018920898437500, -0.0018920898437500, 0.0019226074218750, -0.0015563964843750,
+0.0022583007812500, -0.0008544921875000, 0.0022277832031250, -0.0017395019531250,
+0.0021667480468750, -0.0004272460937500, 0.0024108886718750, -0.0009155273437500,
+0.0022277832031250, -0.0007629394531250, 0.0020446777343750, 0.0002746582031250,
+0.0023803710937500, -0.0005187988281250, 0.0015869140625000, 0.0003051757812500,
+0.0017395019531250, 0.0001831054687500, 0.0016784667968750, 0.0001525878906250,
+0.0011291503906250, 0.0005187988281250, 0.0014648437500000, 0.0003356933593750,
+0.0011901855468750, 0.0007324218750000, 0.0010375976562500, 0.0007019042968750,
+0.0011291503906250, 0.0007934570312500, 0.0010681152343750, 0.0010681152343750,
+0.0007629394531250, 0.0008850097656250, 0.0010070800781250, 0.0011596679687500,
+0.0007324218750000, 0.0010375976562500, 0.0005187988281250, 0.0010070800781250,
+0.0005798339843750, 0.0011901855468750, 0.0003662109375000, 0.0009155273437500,
+0.0003967285156250, 0.0010681152343750, 0.0002746582031250, 0.0009460449218750,
+0.0002136230468750, 0.0008850097656250, 0.0002746582031250, 0.0008239746093750,
+0.0000610351562500, 0.0006103515625000, 0.0000610351562500, 0.0008239746093750,
+0.0001220703125000, 0.0005187988281250, -0.0000305175781250, 0.0004882812500000,
+-0.0000610351562500, 0.0006103515625000, -0.0000305175781250, 0.0002746582031250,
+-0.0002136230468750, 0.0001831054687500, -0.0001220703125000, 0.0000915527343750,
+-0.0001525878906250, -0.0000915527343750, -0.0001525878906250, -0.0002136230468750,
+0.0001220703125000, -0.0003356933593750, -0.0001831054687500, -0.0004272460937500,
+0.0001220703125000, -0.0005187988281250, 0.0002136230468750, -0.0005798339843750,
+0.0000610351562500, -0.0006408691406250, 0.0005187988281250, -0.0007324218750000,
+0.0001831054687500, -0.0006713867187500, 0.0004882812500000, -0.0007324218750000,
+0.0005187988281250, -0.0006713867187500, 0.0003356933593750, -0.0005187988281250,
+0.0009155273437500, -0.0006408691406250, 0.0003356933593750, -0.0003356933593750,
+0.0009155273437500, -0.0003967285156250, 0.0009155273437500, -0.0002746582031250,
+0.0005187988281250, 0.0000305175781250, 0.0012817382812500, -0.0001220703125000,
+0.0007324218750000, 0.0001525878906250, 0.0009460449218750, 0.0002746582031250,
+0.0010986328125000, 0.0002136230468750, 0.0007324218750000, 0.0002136230468750,
+0.0008544921875000, 0.0003051757812500, 0.0009460449218750, 0.0000915527343750,
+0.0007934570312500},
+{0.0025024414062500, 0.0000000000000000, 0.0024108886718750, -0.0001525878906250,
+0.0023193359375000, -0.0002746582031250, 0.0022277832031250, -0.0006103515625000,
+0.0017700195312500, -0.0008544921875000, 0.0022277832031250, -0.0010681152343750,
+0.0018005371093750, -0.0014648437500000, 0.0018615722656250, -0.0015258789062500,
+0.0021057128906250, -0.0016784667968750, 0.0015258789062500, -0.0021362304687500,
+0.0024108886718750, -0.0016174316406250, 0.0020446777343750, -0.0020446777343750,
+0.0020141601562500, -0.0020751953125000, 0.0026550292968750, -0.0014648437500000,
+0.0022583007812500, -0.0018005371093750, 0.0021972656250000, -0.0016479492187500,
+0.0022277832031250, -0.0015869140625000, 0.0021057128906250, -0.0015869140625000,
+0.0018615722656250, -0.0017089843750000, 0.0017089843750000, -0.0019226074218750,
+0.0018310546875000, -0.0013732910156250, 0.0014343261718750, -0.0017700195312500,
+0.0014343261718750, -0.0015563964843750, 0.0015258789062500, -0.0010681152343750,
+0.0009460449218750, -0.0016479492187500, 0.0013427734375000, -0.0008850097656250,
+0.0008239746093750, -0.0012512207031250, 0.0008850097656250, -0.0010681152343750,
+0.0012817382812500, -0.0005187988281250, 0.0006103515625000, -0.0013122558593750,
+0.0013732910156250, -0.0000305175781250, 0.0010986328125000, -0.0003662109375000,
+0.0010070800781250, -0.0002441406250000, 0.0014343261718750, 0.0005798339843750,
+0.0009460449218750, 0.0000915527343750, 0.0011291503906250, 0.0006408691406250,
+0.0010681152343750, 0.0006103515625000, 0.0006103515625000, 0.0006103515625000,
+0.0005493164062500, 0.0009765625000000, 0.0004882812500000, 0.0007629394531250,
+-0.0002136230468750, 0.0011901855468750, -0.0000915527343750, 0.0011596679687500,
+-0.0003662109375000, 0.0011596679687500, -0.0007934570312500, 0.0013427734375000,
+-0.0003662109375000, 0.0011291503906250, -0.0010375976562500, 0.0012512207031250,
+-0.0007629394531250, 0.0010986328125000, -0.0008239746093750, 0.0009765625000000,
+-0.0012512207031250, 0.0009765625000000, -0.0006713867187500, 0.0007019042968750,
+-0.0012207031250000, 0.0006103515625000, -0.0008850097656250, 0.0004882812500000,
+-0.0007934570312500, 0.0002746582031250, -0.0010986328125000, 0.0001525878906250,
+-0.0003967285156250, 0.0000610351562500, -0.0009765625000000, 0.0000000000000000,
+-0.0005187988281250, -0.0000915527343750, -0.0004272460937500, -0.0002136230468750,
+-0.0006408691406250, -0.0002136230468750, 0.0002441406250000, -0.0001831054687500,
+-0.0005187988281250, -0.0004882812500000, 0.0001525878906250, -0.0003967285156250,
+0.0002441406250000, -0.0004577636718750, -0.0001831054687500, -0.0006408691406250,
+0.0007629394531250, -0.0005798339843750, 0.0000000000000000, -0.0004272460937500,
+0.0005187988281250, -0.0006103515625000, 0.0007019042968750, -0.0004272460937500,
+0.0002746582031250, -0.0002441406250000, 0.0010070800781250, -0.0003967285156250,
+0.0005493164062500, 0.0000000000000000, 0.0006713867187500, 0.0000000000000000,
+0.0009155273437500, -0.0000305175781250, 0.0007324218750000, 0.0002136230468750,
+0.0009155273437500, 0.0002441406250000, 0.0011901855468750, 0.0002746582031250,
+0.0010070800781250, 0.0003051757812500, 0.0012512207031250, 0.0003051757812500,
+0.0015258789062500, 0.0002746582031250, 0.0010070800781250, 0.0001831054687500,
+0.0018615722656250, 0.0001831054687500, 0.0012817382812500, 0.0000305175781250,
+0.0014038085937500, -0.0000915527343750, 0.0019226074218750, -0.0000915527343750,
+0.0009155273437500, -0.0002746582031250, 0.0018920898437500, -0.0005187988281250,
+0.0013427734375000, -0.0003662109375000, 0.0011291503906250, -0.0007019042968750,
+0.0016784667968750, -0.0009155273437500, 0.0010070800781250, -0.0002746582031250,
+0.0013122558593750, -0.0012207031250000, 0.0010375976562500, -0.0004882812500000,
+0.0008850097656250, -0.0004882812500000, 0.0011291503906250, -0.0010681152343750,
+0.0008850097656250, 0.0000610351562500, 0.0010986328125000, -0.0010986328125000,
+0.0012512207031250, -0.0002746582031250, 0.0013427734375000, -0.0003967285156250,
+0.0015563964843750, -0.0012817382812500, 0.0018615722656250, -0.0001525878906250,
+0.0018920898437500, -0.0012817382812500, 0.0023498535156250, -0.0010986328125000,
+0.0023193359375000, -0.0007934570312500, 0.0021667480468750, -0.0014038085937500,
+0.0025939941406250, -0.0012207031250000, 0.0019226074218750, -0.0010070800781250,
+0.0021362304687500, -0.0013122558593750, 0.0021972656250000, -0.0010986328125000,
+0.0015869140625000, -0.0007934570312500, 0.0019226074218750, -0.0010681152343750,
+0.0018005371093750, -0.0003356933593750, 0.0014648437500000, -0.0002441406250000,
+0.0018005371093750, -0.0001831054687500, 0.0019531250000000, 0.0002746582031250,
+0.0014648437500000, 0.0003051757812500, 0.0023498535156250, 0.0001220703125000,
+0.0020446777343750, 0.0004272460937500, 0.0021057128906250, 0.0003967285156250,
+0.0028076171875000, -0.0000610351562500, 0.0022277832031250, 0.0001831054687500,
+0.0020141601562500, 0.0000000000000000, 0.0023193359375000, -0.0003662109375000,
+0.0014343261718750, -0.0000610351562500, 0.0008544921875000, 0.0000305175781250,
+0.0018005371093750, -0.0006713867187500, 0.0002441406250000, 0.0002136230468750,
+0.0008239746093750, -0.0000610351562500, 0.0010375976562500, -0.0000915527343750,
+0.0002136230468750, 0.0003967285156250, 0.0010375976562500, 0.0000305175781250,
+0.0006713867187500, 0.0006103515625000, 0.0007934570312500, 0.0006103515625000,
+0.0008239746093750, 0.0007324218750000, 0.0002746582031250, 0.0011596679687500,
+0.0001525878906250, 0.0010681152343750, 0.0002441406250000, 0.0016174316406250,
+-0.0005187988281250, 0.0015563964843750, 0.0000915527343750, 0.0018615722656250,
+0.0006408691406250, 0.0022583007812500, -0.0003051757812500, 0.0018310546875000,
+0.0020446777343750, 0.0027160644531250, 0.0017089843750000, 0.0025024414062500,
+0.0017395019531250, 0.0025939941406250, 0.0038146972656250, 0.0036010742187500,
+0.0043334960937500, 0.0032958984375000, 0.0048217773437500, 0.0024414062500000,
+0.0046386718750000, 0.0032043457031250, 0.0045166015625000, 0.0028991699218750,
+0.0045166015625000, 0.0021972656250000, 0.0041503906250000, 0.0030822753906250,
+0.0039978027343750, 0.0015258789062500, 0.0037231445312500, 0.0019531250000000,
+0.0034179687500000, 0.0016784667968750, 0.0030822753906250, 0.0006103515625000,
+0.0026245117187500, 0.0015869140625000, 0.0023803710937500, 0.0001220703125000,
+0.0016174316406250, 0.0006408691406250, 0.0011901855468750, 0.0005798339843750,
+0.0010986328125000, -0.0003356933593750, 0.0005187988281250, 0.0007019042968750,
+0.0000915527343750, 0.0002441406250000, 0.0003967285156250, 0.0004882812500000,
+-0.0002441406250000, 0.0006408691406250, -0.0007324218750000, 0.0003967285156250,
+0.0000610351562500, 0.0007324218750000, -0.0008850097656250, 0.0003356933593750,
+-0.0005493164062500, 0.0004577636718750, -0.0002136230468750, 0.0003356933593750,
+-0.0006408691406250, -0.0000915527343750, -0.0002136230468750, -0.0000305175781250,
+0.0000000000000000, -0.0002441406250000, 0.0001220703125000, -0.0005493164062500,
+0.0002136230468750, -0.0002441406250000, 0.0003356933593750, 0.0000000000000000,
+0.0004882812500000, -0.0001525878906250, 0.0003967285156250, 0.0006408691406250,
+0.0007629394531250, 0.0007629394531250, 0.0006408691406250, 0.0007019042968750,
+0.0005187988281250, 0.0010986328125000, 0.0010986328125000, 0.0011291503906250,
+0.0002136230468750, 0.0008850097656250, 0.0007629394531250, 0.0008239746093750,
+0.0004577636718750, 0.0007629394531250, -0.0004882812500000, 0.0006103515625000,
+0.0000915527343750, 0.0005187988281250, -0.0008544921875000, 0.0006713867187500,
+-0.0010681152343750, 0.0004272460937500, -0.0009155273437500, 0.0005493164062500,
+-0.0012817382812500, 0.0007934570312500, -0.0014343261718750, 0.0004577636718750,
+-0.0012512207031250, 0.0007934570312500, -0.0014038085937500, 0.0006713867187500,
+-0.0014343261718750, 0.0005493164062500, -0.0013122558593750, 0.0006408691406250,
+-0.0014038085937500, 0.0003662109375000, -0.0014648437500000, 0.0005798339843750,
+-0.0013122558593750, 0.0001831054687500, -0.0013427734375000, 0.0003662109375000,
+-0.0013427734375000, 0.0006408691406250, -0.0012207031250000, 0.0000000000000000,
+-0.0010681152343750, 0.0010681152343750, -0.0010375976562500, 0.0006103515625000,
+-0.0008239746093750, 0.0007324218750000, -0.0006103515625000, 0.0014648437500000,
+-0.0007324218750000, 0.0007324218750000, -0.0004577636718750, 0.0015258789062500,
+-0.0005493164062500, 0.0013122558593750, -0.0004882812500000, 0.0011291503906250,
+-0.0002441406250000, 0.0015258789062500, -0.0003051757812500, 0.0011596679687500,
+-0.0000915527343750, 0.0013427734375000, 0.0000305175781250, 0.0012512207031250,
+0.0000305175781250, 0.0010681152343750, 0.0001525878906250, 0.0010070800781250,
+0.0003356933593750, 0.0008544921875000, 0.0001220703125000, 0.0006103515625000,
+0.0003051757812500, 0.0003356933593750, 0.0003356933593750, 0.0001525878906250,
+0.0002441406250000, -0.0001220703125000, 0.0004272460937500, -0.0003662109375000,
+0.0000610351562500, -0.0004272460937500, 0.0006713867187500, -0.0008544921875000,
+0.0004272460937500, -0.0009765625000000, -0.0001220703125000, -0.0010375976562500,
+0.0007019042968750, -0.0014648437500000, -0.0002746582031250, -0.0012817382812500,
+-0.0000610351562500, -0.0013732910156250, 0.0000305175781250, -0.0014953613281250,
+-0.0005493164062500, -0.0013122558593750, -0.0001831054687500, -0.0012817382812500,
+-0.0004882812500000, -0.0013122558593750, -0.0004577636718750, -0.0009155273437500,
+-0.0004272460937500, -0.0008850097656250, -0.0005493164062500, -0.0008850097656250,
+-0.0003356933593750, -0.0003051757812500, -0.0005187988281250, -0.0006408691406250,
+-0.0003356933593750, -0.0001831054687500, -0.0000915527343750, -0.0000305175781250,
+-0.0000610351562500, -0.0001525878906250, 0.0001831054687500, 0.0004577636718750,
+0.0003967285156250, 0.0001525878906250, 0.0005493164062500, 0.0004882812500000,
+0.0007629394531250, 0.0006713867187500, 0.0009460449218750, 0.0004577636718750,
+0.0010070800781250, 0.0007629394531250, 0.0011596679687500, 0.0007324218750000,
+0.0010986328125000, 0.0004882812500000, 0.0010681152343750, 0.0007629394531250,
+0.0011596679687500, 0.0007629394531250, 0.0010375976562500, 0.0003662109375000,
+0.0009155273437500, 0.0010986328125000, 0.0008850097656250, 0.0006103515625000,
+0.0005798339843750, 0.0007934570312500, 0.0003356933593750, 0.0013427734375000,
+0.0004272460937500, 0.0005798339843750, -0.0003356933593750, 0.0015563964843750,
+-0.0002136230468750, 0.0011291503906250, -0.0005493164062500, 0.0010986328125000,
+-0.0010375976562500, 0.0016479492187500, -0.0004882812500000, 0.0009460449218750,
+-0.0014343261718750, 0.0014953613281250, -0.0010986328125000, 0.0013732910156250,
+-0.0010375976562500, 0.0010681152343750, -0.0016479492187500, 0.0011901855468750,
+-0.0010986328125000, 0.0009765625000000, -0.0011901855468750, 0.0005798339843750,
+-0.0011901855468750, 0.0007019042968750, -0.0008239746093750, 0.0004882812500000,
+-0.0007629394531250, 0.0000000000000000, -0.0008239746093750, 0.0000610351562500,
+-0.0003356933593750, 0.0000610351562500, -0.0007629394531250, -0.0003967285156250,
+-0.0007324218750000, 0.0000305175781250, -0.0003662109375000, 0.0002441406250000,
+-0.0011596679687500, -0.0005187988281250, -0.0007324218750000, 0.0010375976562500,
+-0.0008850097656250, 0.0003356933593750, -0.0012207031250000, 0.0007629394531250,
+-0.0011596679687500},
+{-0.0012817382812500, 0.0010681152343750, -0.0009460449218750, 0.0018615722656250,
+-0.0010375976562500, 0.0012817382812500, -0.0013732910156250, 0.0015869140625000,
+-0.0010986328125000, 0.0016174316406250, -0.0010681152343750, 0.0011901855468750,
+-0.0012817382812500, 0.0011596679687500, -0.0010681152343750, 0.0011901855468750,
+-0.0009765625000000, 0.0004272460937500, -0.0010375976562500, 0.0005798339843750,
+-0.0007934570312500, 0.0003967285156250, -0.0004882812500000, -0.0001525878906250,
+-0.0005798339843750, 0.0001525878906250, -0.0002136230468750, -0.0004272460937500,
+-0.0001525878906250, -0.0004272460937500, -0.0000610351562500, -0.0004882812500000,
+0.0002136230468750, -0.0009155273437500, 0.0001525878906250, -0.0007019042968750,
+0.0003051757812500, -0.0009155273437500, 0.0005187988281250, -0.0010375976562500,
+0.0006103515625000, -0.0008850097656250, 0.0006408691406250, -0.0007324218750000,
+0.0008239746093750, -0.0007934570312500, 0.0007019042968750, -0.0003356933593750,
+0.0008850097656250, -0.0005493164062500, 0.0007629394531250, -0.0003051757812500,
+0.0007019042968750, 0.0001831054687500, 0.0010681152343750, 0.0000000000000000,
+0.0003356933593750, 0.0006713867187500, 0.0008239746093750, 0.0008544921875000,
+0.0005187988281250, 0.0010375976562500, -0.0001525878906250, 0.0015258789062500,
+0.0007324218750000, 0.0016174316406250, -0.0005493164062500, 0.0019531250000000,
+-0.0000305175781250, 0.0022583007812500, -0.0000305175781250, 0.0023803710937500,
+-0.0007934570312500, 0.0024414062500000, 0.0001525878906250, 0.0026245117187500,
+-0.0005798339843750, 0.0025329589843750, -0.0001525878906250, 0.0025329589843750,
+0.0001525878906250, 0.0025634765625000, -0.0001220703125000, 0.0023803710937500,
+0.0004882812500000, 0.0021972656250000, 0.0003051757812500, 0.0024108886718750,
+0.0005493164062500, 0.0019226074218750, 0.0007934570312500, 0.0018615722656250,
+0.0007324218750000, 0.0019226074218750, 0.0009765625000000, 0.0011291503906250,
+0.0010681152343750, 0.0014953613281250, 0.0011901855468750, 0.0010070800781250,
+0.0010681152343750, 0.0006713867187500, 0.0010375976562500, 0.0007934570312500,
+0.0012207031250000, 0.0002441406250000, 0.0005187988281250, 0.0001831054687500,
+0.0006713867187500, 0.0002136230468750, 0.0003051757812500, -0.0001831054687500,
+-0.0003051757812500, -0.0003356933593750, 0.0000610351562500, -0.0001831054687500,
+-0.0007934570312500, -0.0005798339843750, -0.0005493164062500, -0.0005493164062500,
+-0.0006713867187500, -0.0005493164062500, -0.0012817382812500, -0.0007934570312500,
+-0.0005798339843750, -0.0006713867187500, -0.0010986328125000, -0.0005798339843750,
+-0.0010070800781250, -0.0007629394531250, -0.0006408691406250, -0.0004882812500000,
+-0.0007019042968750, -0.0002441406250000, -0.0006713867187500, -0.0005493164062500,
+-0.0002136230468750, 0.0001831054687500, -0.0005187988281250, 0.0000305175781250,
+-0.0003051757812500, 0.0001525878906250, -0.0000610351562500, 0.0007629394531250,
+-0.0008850097656250, 0.0004577636718750, -0.0001831054687500, 0.0006713867187500,
+-0.0006103515625000, 0.0009765625000000, -0.0009765625000000, 0.0007324218750000,
+-0.0007019042968750, 0.0006103515625000, -0.0010986328125000, 0.0008544921875000,
+-0.0010986328125000, 0.0001525878906250, -0.0012512207031250, 0.0002441406250000,
+-0.0009155273437500, 0.0001220703125000, -0.0006408691406250, -0.0004577636718750,
+-0.0009460449218750, -0.0002136230468750, 0.0002136230468750, -0.0002746582031250,
+-0.0000610351562500, -0.0004882812500000, 0.0002441406250000, -0.0000915527343750,
+0.0010070800781250, 0.0002746582031250, 0.0003662109375000, 0.0000000000000000,
+0.0014343261718750, 0.0010375976562500, 0.0008239746093750, 0.0010986328125000,
+0.0011291503906250, 0.0010681152343750, 0.0019226074218750, 0.0015258789062500,
+0.0006713867187500, 0.0013732910156250, 0.0021667480468750, 0.0012207031250000,
+0.0015258789062500, 0.0012207031250000, 0.0011901855468750, 0.0011291503906250,
+0.0020141601562500, 0.0009155273437500, 0.0013427734375000, 0.0010070800781250,
+0.0017089843750000, 0.0013427734375000, 0.0018310546875000, 0.0009155273437500,
+0.0018615722656250, 0.0013122558593750, 0.0021667480468750, 0.0017089843750000,
+0.0023803710937500, 0.0010375976562500, 0.0030517578125000, 0.0025329589843750,
+0.0029907226562500, 0.0021972656250000, 0.0035400390625000, 0.0022583007812500,
+0.0041809082031250, 0.0032043457031250, 0.0035400390625000, 0.0026245117187500,
+0.0046386718750000, 0.0032958984375000, 0.0040588378906250, 0.0033264160156250,
+0.0040588378906250, 0.0028076171875000, 0.0046386718750000, 0.0028076171875000,
+0.0036926269531250, 0.0028991699218750, 0.0045776367187500, 0.0024719238281250,
+0.0046691894531250, 0.0023498535156250, 0.0042114257812500, 0.0026855468750000,
+0.0047912597656250, 0.0028381347656250, 0.0051879882812500, 0.0027770996093750,
+0.0037841796875000, 0.0032653808593750, 0.0048828125000000, 0.0038452148437500,
+0.0042419433593750, 0.0032043457031250, 0.0025329589843750, 0.0028381347656250,
+0.0039062500000000, 0.0037231445312500, 0.0022277832031250, 0.0017700195312500,
+0.0017395019531250, 0.0019226074218750, 0.0022583007812500, 0.0023193359375000,
+0.0020141601562500, 0.0011291503906250, 0.0004577636718750, 0.0037841796875000,
+0.0018310546875000, 0.0014038085937500, 0.0005493164062500, 0.0025939941406250,
+0.0009460449218750, 0.0025939941406250, 0.0014648437500000, 0.0016479492187500,
+0.0001831054687500, 0.0025024414062500, 0.0021057128906250, 0.0027770996093750,
+0.0012817382812500, 0.0030822753906250, 0.0011901855468750, 0.0031738281250000,
+0.0025634765625000, 0.0031738281250000, 0.0014648437500000, 0.0034484863281250,
+0.0014648437500000, 0.0028381347656250, 0.0018005371093750, 0.0026550292968750,
+0.0011291503906250, 0.0028686523437500, 0.0004882812500000, 0.0026550292968750,
+0.0010070800781250, 0.0024108886718750, 0.0006408691406250, 0.0035705566406250,
+0.0002441406250000, 0.0033264160156250, 0.0010375976562500, 0.0036315917968750,
+0.0014953613281250, 0.0045776367187500, 0.0008239746093750, 0.0039978027343750,
+0.0022277832031250, 0.0044555664062500, 0.0023803710937500, 0.0043945312500000,
+0.0019531250000000, 0.0042114257812500, 0.0022888183593750, 0.0043029785156250,
+0.0023803710937500, 0.0039367675781250, 0.0015869140625000, 0.0039062500000000,
+0.0017700195312500, 0.0038146972656250, 0.0017089843750000, 0.0032348632812500,
+0.0012207031250000, 0.0029296875000000, 0.0016479492187500, 0.0030517578125000,
+0.0016784667968750, 0.0014953613281250, 0.0017089843750000, 0.0015869140625000,
+0.0020141601562500, 0.0012207031250000, 0.0024414062500000, 0.0001525878906250,
+0.0025024414062500, 0.0003967285156250, 0.0021057128906250, 0.0002746582031250,
+0.0028076171875000, 0.0000610351562500, 0.0024108886718750, 0.0002441406250000,
+0.0017089843750000, 0.0004882812500000, 0.0025634765625000, 0.0003967285156250,
+0.0013732910156250, 0.0005187988281250, 0.0017089843750000, 0.0004577636718750,
+0.0017089843750000, 0.0005187988281250, 0.0010375976562500, 0.0004577636718750,
+0.0017089843750000, 0.0000000000000000, 0.0016174316406250, 0.0005187988281250,
+0.0015869140625000, 0.0000000000000000, 0.0020751953125000, -0.0003356933593750,
+0.0023803710937500, -0.0000305175781250, 0.0022277832031250, -0.0007019042968750,
+0.0025634765625000, -0.0008239746093750, 0.0025024414062500, -0.0012207031250000,
+0.0022583007812500, -0.0014648437500000, 0.0021362304687500, -0.0014648437500000,
+0.0018005371093750, -0.0016784667968750, 0.0013427734375000, -0.0014648437500000,
+0.0010070800781250, -0.0015258789062500, 0.0005187988281250, -0.0014343261718750,
+0.0001525878906250, -0.0012512207031250, -0.0001220703125000, -0.0013732910156250,
+-0.0005493164062500, -0.0010375976562500, -0.0008239746093750, -0.0012512207031250,
+-0.0008239746093750, -0.0009765625000000, -0.0009155273437500, -0.0005493164062500,
+-0.0010070800781250, -0.0008850097656250, -0.0007324218750000, -0.0002136230468750,
+-0.0006408691406250, -0.0003967285156250, -0.0004882812500000, -0.0003967285156250,
+-0.0002746582031250, -0.0000610351562500, -0.0001220703125000, -0.0004577636718750,
+-0.0000305175781250, 0.0000000000000000, 0.0002136230468750, -0.0002441406250000,
+0.0001831054687500, -0.0001525878906250, 0.0002441406250000, 0.0002136230468750,
+0.0005187988281250, -0.0000305175781250, 0.0000915527343750, 0.0003356933593750,
+0.0003356933593750, 0.0002136230468750, 0.0003356933593750, 0.0002441406250000,
+0.0000610351562500, 0.0004882812500000, 0.0003356933593750, 0.0001831054687500,
+-0.0002136230468750, 0.0004882812500000, -0.0001220703125000, 0.0003051757812500,
+-0.0003356933593750, 0.0003051757812500, -0.0007629394531250, 0.0005493164062500,
+-0.0004882812500000, 0.0003356933593750, -0.0011291503906250, 0.0003051757812500,
+-0.0007934570312500, 0.0004577636718750, -0.0009460449218750, 0.0002441406250000,
+-0.0013122558593750, 0.0000915527343750, -0.0005798339843750, 0.0004272460937500,
+-0.0014343261718750, -0.0001525878906250, -0.0009155273437500, 0.0000305175781250,
+-0.0008850097656250, 0.0000305175781250, -0.0014038085937500, -0.0002136230468750,
+-0.0006408691406250, 0.0001220703125000, -0.0013732910156250, 0.0000000000000000,
+-0.0008850097656250, 0.0001220703125000, -0.0007934570312500, 0.0002746582031250,
+-0.0011596679687500, 0.0002136230468750, -0.0003662109375000, 0.0003662109375000,
+-0.0009155273437500, 0.0003051757812500, -0.0005493164062500, 0.0004577636718750,
+-0.0002746582031250, 0.0004272460937500, -0.0006103515625000, 0.0002441406250000,
+-0.0001525878906250, 0.0004577636718750, -0.0000915527343750, 0.0002746582031250,
+-0.0001525878906250, 0.0001525878906250, 0.0001220703125000, 0.0002746582031250,
+0.0002746582031250, 0.0003051757812500, 0.0001831054687500, 0.0001525878906250,
+0.0006408691406250, 0.0002136230468750, 0.0003967285156250, 0.0003356933593750,
+0.0004577636718750, 0.0002136230468750, 0.0007324218750000, 0.0002136230468750,
+0.0003051757812500, 0.0003356933593750, 0.0006408691406250, -0.0000915527343750,
+0.0003967285156250, 0.0001525878906250, 0.0002746582031250, 0.0000305175781250,
+0.0003356933593750, -0.0002746582031250, -0.0000610351562500, 0.0002136230468750,
+0.0000000000000000, -0.0004272460937500, -0.0004577636718750, -0.0001220703125000,
+-0.0004882812500000, -0.0001525878906250, -0.0004577636718750, -0.0004577636718750,
+-0.0011901855468750, 0.0001220703125000, -0.0005798339843750, -0.0006103515625000,
+-0.0010375976562500, -0.0002136230468750, -0.0010070800781250, -0.0002746582031250,
+-0.0005493164062500, -0.0008850097656250, -0.0012512207031250, -0.0002441406250000,
+-0.0003662109375000, -0.0007934570312500, -0.0006713867187500, -0.0009460449218750,
+-0.0005798339843750, -0.0007324218750000, 0.0000915527343750, -0.0009155273437500,
+-0.0003662109375000, -0.0011291503906250, 0.0003051757812500, -0.0006408691406250,
+0.0004272460937500, -0.0010070800781250, 0.0003356933593750, -0.0007629394531250,
+0.0007019042968750, -0.0001831054687500, 0.0008544921875000, -0.0005187988281250,
+0.0006713867187500, 0.0002136230468750, 0.0011596679687500, 0.0005187988281250,
+0.0011291503906250, 0.0005493164062500, 0.0010070800781250, 0.0009765625000000,
+0.0016784667968750, 0.0012512207031250, 0.0011291503906250, 0.0007934570312500,
+0.0016784667968750, 0.0013427734375000, 0.0019226074218750, 0.0011901855468750,
+0.0017089843750000, 0.0006103515625000, 0.0024414062500000, 0.0011901855468750,
+0.0023193359375000},
+{0.0006408691406250, -0.0007019042968750, -0.0001831054687500, -0.0002136230468750,
+-0.0003051757812500, -0.0003356933593750, -0.0006103515625000, -0.0003662109375000,
+-0.0011596679687500, -0.0000915527343750, -0.0009765625000000, -0.0003662109375000,
+-0.0015563964843750, -0.0001220703125000, -0.0015258789062500, -0.0002136230468750,
+-0.0014953613281250, -0.0002136230468750, -0.0017395019531250, -0.0000610351562500,
+-0.0016174316406250, -0.0001831054687500, -0.0017395019531250, -0.0001220703125000,
+-0.0016174316406250, -0.0001831054687500, -0.0015563964843750, -0.0001525878906250,
+-0.0014038085937500, -0.0000305175781250, -0.0011291503906250, -0.0001220703125000,
+-0.0011596679687500, 0.0000000000000000, -0.0007934570312500, 0.0001220703125000,
+-0.0006103515625000, 0.0001220703125000, -0.0006713867187500, 0.0002136230468750,
+-0.0003967285156250, 0.0003662109375000, -0.0003662109375000, 0.0001525878906250,
+-0.0003967285156250, 0.0003662109375000, -0.0002441406250000, 0.0003356933593750,
+-0.0002441406250000, 0.0002441406250000, -0.0003967285156250, 0.0006103515625000,
+-0.0000610351562500, 0.0003051757812500, -0.0002746582031250, 0.0006408691406250,
+-0.0002136230468750, 0.0005798339843750, 0.0000915527343750, 0.0003662109375000,
+-0.0002441406250000, 0.0008239746093750, 0.0000915527343750, 0.0003662109375000,
+0.0000000000000000, 0.0005493164062500, -0.0001220703125000, 0.0006408691406250,
+0.0000610351562500, 0.0004272460937500, -0.0001831054687500, 0.0006408691406250,
+-0.0002746582031250, 0.0005493164062500, -0.0003662109375000, 0.0005798339843750,
+-0.0005187988281250, 0.0005798339843750, -0.0007019042968750, 0.0004882812500000,
+-0.0007934570312500, 0.0005798339843750, -0.0009155273437500, 0.0004272460937500,
+-0.0010375976562500, 0.0004577636718750, -0.0010986328125000, 0.0003967285156250,
+-0.0011901855468750, 0.0003051757812500, -0.0013732910156250, 0.0003356933593750,
+-0.0012512207031250, 0.0001525878906250, -0.0014953613281250, 0.0002441406250000,
+-0.0014038085937500, 0.0002441406250000, -0.0011291503906250, 0.0001220703125000,
+-0.0014343261718750, 0.0002746582031250, -0.0010986328125000, -0.0000305175781250,
+-0.0011596679687500, 0.0002746582031250, -0.0011291503906250, 0.0001220703125000,
+-0.0009460449218750, -0.0002746582031250, -0.0011291503906250, 0.0001525878906250,
+-0.0007934570312500, -0.0005798339843750, -0.0010986328125000, -0.0006408691406250,
+-0.0010986328125000, -0.0005798339843750, -0.0008239746093750, -0.0010375976562500,
+-0.0012817382812500, -0.0010986328125000, -0.0010681152343750, -0.0008239746093750,
+-0.0012817382812500, -0.0011596679687500, -0.0014648437500000, -0.0009765625000000,
+-0.0014343261718750, -0.0005493164062500, -0.0017089843750000, -0.0007629394531250,
+-0.0015258789062500, -0.0002441406250000, -0.0016784667968750, 0.0000305175781250,
+-0.0015869140625000, 0.0002136230468750, -0.0012512207031250, 0.0006408691406250,
+-0.0013427734375000, 0.0008850097656250, -0.0008239746093750, 0.0009460449218750,
+-0.0005798339843750, 0.0012512207031250, -0.0003662109375000, 0.0011291503906250,
+0.0000915527343750, 0.0009765625000000, 0.0002746582031250, 0.0013427734375000,
+0.0003662109375000, 0.0006713867187500, 0.0007629394531250, 0.0007934570312500,
+0.0007629394531250, 0.0008850097656250, 0.0006408691406250, 0.0004272460937500,
+0.0009765625000000, 0.0005798339843750, 0.0007324218750000, 0.0004272460937500,
+0.0004882812500000, 0.0003051757812500, 0.0010681152343750, 0.0004272460937500,
+0.0010375976562500, 0.0004882812500000, 0.0002441406250000, 0.0003967285156250,
+0.0015258789062500, 0.0006713867187500, 0.0006103515625000, 0.0006408691406250,
+0.0007324218750000, 0.0008850097656250, 0.0015869140625000, 0.0012817382812500,
+0.0001831054687500, 0.0009765625000000, 0.0015258789062500, 0.0014648437500000,
+0.0010375976562500, 0.0013122558593750, 0.0007019042968750, 0.0016174316406250,
+0.0015869140625000, 0.0020141601562500, 0.0012207031250000, 0.0015258789062500,
+0.0014038085937500, 0.0025329589843750, 0.0018005371093750, 0.0024108886718750,
+0.0020446777343750, 0.0022583007812500, 0.0024108886718750, 0.0028381347656250,
+0.0029907226562500, 0.0029296875000000, 0.0030822753906250, 0.0026245117187500,
+0.0038146972656250, 0.0032043457031250, 0.0039062500000000, 0.0029602050781250,
+0.0037841796875000, 0.0027160644531250, 0.0045776367187500, 0.0036315917968750,
+0.0037231445312500, 0.0025634765625000, 0.0042419433593750, 0.0032348632812500,
+0.0043029785156250, 0.0032653808593750, 0.0032043457031250, 0.0019531250000000,
+0.0028686523437500, 0.0021362304687500, 0.0021972656250000, 0.0018005371093750,
+0.0026550292968750, 0.0017700195312500, 0.0024719238281250, 0.0017700195312500,
+0.0020141601562500, 0.0014648437500000, 0.0026550292968750, 0.0013427734375000,
+0.0018920898437500, 0.0014648437500000, 0.0022277832031250, 0.0013427734375000,
+0.0021667480468750, 0.0014343261718750, 0.0017395019531250, 0.0014648437500000,
+0.0024719238281250, 0.0011291503906250, 0.0014648437500000, 0.0012817382812500,
+0.0017395019531250, 0.0007324218750000, 0.0020141601562500, 0.0008239746093750,
+0.0013122558593750, 0.0010681152343750, 0.0014953613281250, 0.0002136230468750,
+0.0024719238281250, 0.0009765625000000, 0.0015563964843750, 0.0004882812500000,
+0.0022583007812500, 0.0002441406250000, 0.0034484863281250, 0.0006103515625000,
+0.0018920898437500, -0.0000305175781250, 0.0035095214843750, 0.0003051757812500,
+0.0032043457031250, 0.0000000000000000, 0.0024719238281250, 0.0000610351562500,
+0.0029907226562500, 0.0003051757812500, 0.0025939941406250, -0.0000915527343750,
+0.0019531250000000, 0.0007629394531250, 0.0021362304687500, 0.0004272460937500,
+0.0016174316406250, 0.0005798339843750, 0.0008544921875000, 0.0012207031250000,
+0.0012817382812500, 0.0005798339843750, 0.0002746582031250, 0.0009460449218750,
+0.0000305175781250, 0.0007324218750000, -0.0000305175781250, 0.0006408691406250,
+-0.0005493164062500, 0.0007629394531250, -0.0006103515625000, 0.0002746582031250,
+-0.0003662109375000, 0.0008544921875000, -0.0006408691406250, 0.0004272460937500,
+-0.0003967285156250, 0.0006103515625000, 0.0000915527343750, 0.0010986328125000,
+-0.0001525878906250, 0.0003967285156250, 0.0002746582031250, 0.0014343261718750,
+0.0005798339843750, 0.0008850097656250, 0.0004577636718750, 0.0008850097656250,
+0.0005187988281250, 0.0015258789062500, 0.0008544921875000, 0.0005493164062500,
+0.0002441406250000, 0.0011901855468750, 0.0005493164062500, 0.0007629394531250,
+0.0003051757812500, 0.0006103515625000, -0.0002746582031250, 0.0009155273437500,
+0.0001525878906250, 0.0002746582031250, -0.0004272460937500, 0.0007324218750000,
+-0.0006103515625000, 0.0003967285156250, -0.0004882812500000, 0.0002441406250000,
+-0.0007324218750000, 0.0005798339843750, -0.0010070800781250, 0.0001220703125000,
+-0.0002441406250000, 0.0002136230468750, -0.0007934570312500, 0.0002136230468750,
+-0.0005798339843750, -0.0000610351562500, 0.0001220703125000, -0.0001220703125000,
+-0.0005798339843750, 0.0000305175781250, 0.0001831054687500, -0.0004577636718750,
+0.0001525878906250, -0.0002136230468750, -0.0001525878906250, -0.0002441406250000,
+0.0000610351562500, -0.0004577636718750, -0.0000305175781250, -0.0000305175781250,
+-0.0002746582031250, -0.0001220703125000, -0.0003967285156250, 0.0000915527343750,
+-0.0003051757812500, 0.0003662109375000, -0.0003967285156250, 0.0004272460937500,
+-0.0006103515625000, 0.0006713867187500, -0.0002441406250000, 0.0008850097656250,
+-0.0004272460937500, 0.0008544921875000, -0.0003662109375000, 0.0010070800781250,
+-0.0000610351562500, 0.0010986328125000, -0.0002136230468750, 0.0008544921875000,
+0.0000915527343750, 0.0010986328125000, 0.0001831054687500, 0.0008850097656250,
+0.0002136230468750, 0.0007629394531250, 0.0004577636718750, 0.0008850097656250,
+0.0005493164062500, 0.0007019042968750, 0.0005798339843750, 0.0005187988281250,
+0.0007629394531250, 0.0004882812500000, 0.0008239746093750, 0.0003967285156250,
+0.0008544921875000, 0.0001525878906250, 0.0011291503906250, 0.0001220703125000,
+0.0008544921875000, 0.0001220703125000, 0.0011901855468750, 0.0000000000000000,
+0.0012207031250000, 0.0001220703125000, 0.0008544921875000, 0.0001831054687500,
+0.0012207031250000, -0.0000915527343750, 0.0007934570312500, 0.0004272460937500,
+0.0008239746093750, 0.0000305175781250, 0.0007629394531250, 0.0001525878906250,
+0.0003662109375000, 0.0004577636718750, 0.0004577636718750, -0.0001831054687500,
+0.0000000000000000, 0.0006713867187500, 0.0000000000000000, 0.0002441406250000,
+-0.0001831054687500, 0.0002746582031250, -0.0004882812500000, 0.0007629394531250,
+-0.0003051757812500, 0.0001220703125000, -0.0008239746093750, 0.0006713867187500,
+-0.0007019042968750, 0.0004272460937500, -0.0007629394531250, 0.0002441406250000,
+-0.0010375976562500, 0.0005187988281250, -0.0006713867187500, 0.0002746582031250,
+-0.0009155273437500, 0.0002441406250000, -0.0007324218750000, 0.0001220703125000,
+-0.0005187988281250, 0.0000000000000000, -0.0006103515625000, -0.0001525878906250,
+-0.0003051757812500, -0.0003051757812500, -0.0001220703125000, -0.0003662109375000,
+-0.0002136230468750, -0.0004577636718750, 0.0000000000000000, -0.0005493164062500,
+0.0001220703125000, -0.0005798339843750, -0.0001525878906250, -0.0007934570312500,
+0.0003051757812500, -0.0004882812500000, 0.0000000000000000, -0.0007324218750000,
+-0.0000305175781250, -0.0005187988281250, 0.0002136230468750, -0.0001525878906250,
+-0.0003356933593750, -0.0005187988281250, -0.0002746582031250, 0.0003356933593750,
+-0.0002746582031250, 0.0001831054687500, -0.0005493164062500, 0.0001525878906250,
+-0.0007019042968750, 0.0007324218750000, -0.0004882812500000, 0.0004272460937500,
+-0.0005493164062500, 0.0004882812500000, -0.0006103515625000, 0.0005187988281250,
+-0.0002136230468750, 0.0002746582031250, 0.0001525878906250, 0.0000610351562500,
+0.0000915527343750, 0.0000305175781250, 0.0006408691406250, -0.0003051757812500,
+0.0007324218750000, -0.0005798339843750, 0.0010681152343750, -0.0004272460937500,
+0.0013427734375000, -0.0005187988281250, 0.0010375976562500, -0.0009765625000000,
+0.0020141601562500, -0.0002441406250000, 0.0016784667968750, -0.0007934570312500,
+0.0017700195312500, -0.0004882812500000, 0.0024414062500000, 0.0001831054687500,
+0.0017089843750000, -0.0004272460937500, 0.0021057128906250, 0.0006713867187500,
+0.0019531250000000, 0.0005798339843750, 0.0015869140625000, 0.0006408691406250,
+0.0017089843750000, 0.0011901855468750, 0.0016479492187500, 0.0007629394531250,
+0.0014953613281250, 0.0010681152343750, 0.0017395019531250, 0.0008850097656250,
+0.0018005371093750, 0.0008544921875000, 0.0018920898437500, 0.0009155273437500,
+0.0021972656250000, 0.0005187988281250, 0.0021667480468750, 0.0012207031250000,
+0.0022277832031250, 0.0007934570312500, 0.0023498535156250, 0.0010681152343750,
+0.0022583007812500, 0.0017395019531250, 0.0018920898437500, 0.0009765625000000,
+0.0022277832031250, 0.0021362304687500, 0.0017089843750000, 0.0018615722656250,
+0.0013732910156250, 0.0018310546875000, 0.0017700195312500, 0.0025024414062500,
+0.0016479492187500, 0.0020751953125000, 0.0009155273437500, 0.0019531250000000,
+0.0021057128906250, 0.0026245117187500, 0.0013122558593750, 0.0021972656250000,
+0.0003051757812500, 0.0017089843750000, 0.0022583007812500, 0.0024719238281250,
+-0.0003662109375000, 0.0013122558593750, 0.0005187988281250, 0.0017089843750000,
+0.0008239746093750},
+{0.0011596679687500, -0.0009460449218750, 0.0010681152343750, -0.0007934570312500,
+0.0012512207031250, -0.0005493164062500, 0.0012817382812500, -0.0010681152343750,
+0.0008544921875000, -0.0006103515625000, 0.0011596679687500, -0.0009155273437500,
+0.0008544921875000, -0.0009460449218750, 0.0003662109375000, -0.0007019042968750,
+0.0007629394531250, -0.0012512207031250, -0.0002441406250000, -0.0009155273437500,
+0.0000000000000000, -0.0010681152343750, -0.0002746582031250, -0.0010681152343750,
+-0.0010070800781250, -0.0008239746093750, -0.0004577636718750, -0.0009765625000000,
+-0.0013122558593750, -0.0005493164062500, -0.0010681152343750, -0.0006408691406250,
+-0.0010070800781250, -0.0003967285156250, -0.0014343261718750, -0.0000305175781250,
+-0.0008850097656250, -0.0001525878906250, -0.0009155273437500, 0.0004577636718750,
+-0.0010986328125000, 0.0003051757812500, -0.0006103515625000, 0.0005493164062500,
+-0.0003356933593750, 0.0010375976562500, -0.0006103515625000, 0.0006408691406250,
+0.0003356933593750, 0.0012207031250000, 0.0000610351562500, 0.0012512207031250,
+0.0002441406250000, 0.0010375976562500, 0.0007934570312500, 0.0012817382812500,
+0.0001525878906250, 0.0013427734375000, 0.0005798339843750, 0.0008239746093750,
+0.0004272460937500, 0.0012512207031250, -0.0000305175781250, 0.0009155273437500,
+0.0000000000000000, 0.0003051757812500, -0.0000610351562500, 0.0010070800781250,
+-0.0002746582031250, -0.0000305175781250, -0.0003051757812500, 0.0002136230468750,
+-0.0000915527343750, 0.0003051757812500, 0.0001220703125000, -0.0003051757812500,
+0.0001220703125000, 0.0000915527343750, 0.0005187988281250, -0.0000305175781250,
+0.0006713867187500, 0.0000610351562500, 0.0008544921875000, 0.0003051757812500,
+0.0009460449218750, 0.0003967285156250, 0.0010375976562500, 0.0004882812500000,
+0.0015258789062500, 0.0004272460937500, 0.0011596679687500, 0.0007934570312500,
+0.0014953613281250, 0.0005187988281250, 0.0020141601562500, 0.0002441406250000,
+0.0012512207031250, 0.0007629394531250, 0.0019531250000000, -0.0006713867187500,
+0.0015869140625000, -0.0002136230468750, 0.0013122558593750, -0.0006408691406250,
+0.0015563964843750, -0.0018005371093750, 0.0009460449218750, -0.0010070800781250,
+0.0013122558593750, -0.0021362304687500, 0.0010986328125000, -0.0021057128906250,
+0.0010070800781250, -0.0017395019531250, 0.0014038085937500, -0.0021362304687500,
+0.0013427734375000, -0.0019531250000000, 0.0015869140625000, -0.0013427734375000,
+0.0017700195312500, -0.0015869140625000, 0.0019226074218750, -0.0013122558593750,
+0.0019531250000000, -0.0008239746093750, 0.0018615722656250, -0.0012817382812500,
+0.0023498535156250, -0.0006713867187500, 0.0018615722656250, -0.0009460449218750,
+0.0017700195312500, -0.0009460449218750, 0.0021057128906250, -0.0005798339843750,
+0.0015258789062500, -0.0010681152343750, 0.0012512207031250, -0.0003356933593750,
+0.0018615722656250, -0.0006713867187500, 0.0007934570312500, -0.0001525878906250,
+0.0002746582031250, 0.0005798339843750, 0.0018615722656250, -0.0002746582031250,
+-0.0007324218750000, 0.0014648437500000, 0.0005493164062500, 0.0011596679687500,
+0.0004882812500000, 0.0014038085937500, -0.0011596679687500, 0.0027465820312500,
+0.0007324218750000, 0.0022888183593750, -0.0008850097656250, 0.0026245117187500,
+-0.0007629394531250, 0.0034179687500000, -0.0003356933593750, 0.0026245117187500,
+-0.0012817382812500, 0.0020446777343750, -0.0012817382812500, 0.0031433105468750,
+-0.0011291503906250, 0.0013732910156250, -0.0018005371093750, 0.0019226074218750,
+-0.0014648437500000, 0.0019531250000000, -0.0011596679687500, 0.0010986328125000,
+-0.0020446777343750, 0.0022583007812500, -0.0006713867187500, 0.0005187988281250,
+-0.0010070800781250, 0.0015258789062500, -0.0012817382812500, 0.0013122558593750,
+-0.0000915527343750, -0.0015258789062500, -0.0017089843750000, 0.0014038085937500,
+-0.0010375976562500, 0.0002746582031250, -0.0015258789062500, 0.0014953613281250,
+-0.0009155273437500, 0.0004577636718750, 0.0000915527343750, -0.0007629394531250,
+-0.0000915527343750, 0.0004882812500000, 0.0001220703125000, -0.0014038085937500,
+0.0011596679687500, -0.0011291503906250, 0.0006713867187500, -0.0007324218750000,
+0.0001525878906250, -0.0014953613281250, 0.0012817382812500, -0.0008544921875000,
+0.0001220703125000, -0.0006103515625000, 0.0000000000000000, -0.0005798339843750,
+0.0007019042968750, -0.0003662109375000, 0.0006103515625000, -0.0000305175781250,
+0.0005493164062500, 0.0001525878906250, 0.0014343261718750, 0.0003967285156250,
+0.0017395019531250, 0.0003662109375000, 0.0014648437500000, 0.0007934570312500,
+0.0017395019531250, 0.0013427734375000, 0.0021667480468750, 0.0011596679687500,
+0.0007324218750000, 0.0016784667968750, 0.0010070800781250, 0.0021667480468750,
+0.0010986328125000, 0.0019531250000000, 0.0000915527343750, 0.0019836425781250,
+0.0004272460937500, 0.0027770996093750, 0.0007019042968750, 0.0018920898437500,
+0.0002441406250000, 0.0021667480468750, 0.0005493164062500, 0.0026245117187500,
+0.0010375976562500, 0.0023498535156250, 0.0003051757812500, 0.0025939941406250,
+0.0007629394531250, 0.0029296875000000, 0.0005187988281250, 0.0028686523437500,
+-0.0001220703125000, 0.0027770996093750, -0.0002746582031250, 0.0027770996093750,
+-0.0005493164062500, 0.0024414062500000, -0.0010070800781250, 0.0024414062500000,
+-0.0013122558593750, 0.0021057128906250, -0.0010375976562500, 0.0021667480468750,
+-0.0008544921875000, 0.0022888183593750, -0.0011901855468750, 0.0017395019531250,
+-0.0002136230468750, 0.0022583007812500, -0.0003051757812500, 0.0021972656250000,
+-0.0001831054687500, 0.0018615722656250, 0.0004882812500000, 0.0020446777343750,
+0.0002746582031250, 0.0021667480468750, 0.0003051757812500, 0.0015563964843750,
+0.0007019042968750, 0.0018920898437500, 0.0003051757812500, 0.0017395019531250,
+-0.0001831054687500, 0.0013122558593750, 0.0002136230468750, 0.0020141601562500,
+-0.0007934570312500, 0.0010986328125000, -0.0009155273437500, 0.0013122558593750,
+-0.0009765625000000, 0.0013427734375000, -0.0016174316406250, 0.0006408691406250,
+-0.0016174316406250, 0.0007934570312500, -0.0015563964843750, 0.0006713867187500,
+-0.0019531250000000, 0.0002441406250000, -0.0017395019531250, 0.0002746582031250,
+-0.0015258789062500, 0.0002746582031250, -0.0020141601562500, -0.0002746582031250,
+-0.0009460449218750, -0.0002136230468750, -0.0012512207031250, -0.0002441406250000,
+-0.0009155273437500, -0.0006103515625000, -0.0000305175781250, -0.0006408691406250,
+-0.0006408691406250, -0.0004882812500000, 0.0003356933593750, -0.0008544921875000,
+0.0003051757812500, -0.0007934570312500, 0.0003051757812500, -0.0007934570312500,
+0.0008850097656250, -0.0010070800781250, 0.0005493164062500, -0.0008544921875000,
+0.0007629394531250, -0.0011596679687500, 0.0007019042968750, -0.0010375976562500,
+0.0004577636718750, -0.0010986328125000, 0.0003967285156250, -0.0013122558593750,
+0.0001525878906250, -0.0011596679687500, -0.0000305175781250, -0.0012817382812500,
+-0.0003051757812500, -0.0012207031250000, -0.0004577636718750, -0.0011901855468750,
+-0.0006103515625000, -0.0011901855468750, -0.0007934570312500, -0.0009765625000000,
+-0.0007934570312500, -0.0010070800781250, -0.0010375976562500, -0.0007934570312500,
+-0.0010375976562500, -0.0005798339843750, -0.0010070800781250, -0.0005187988281250,
+-0.0013122558593750, -0.0003051757812500, -0.0009460449218750, -0.0000915527343750,
+-0.0013732910156250, 0.0000610351562500, -0.0011596679687500, 0.0001831054687500,
+-0.0009155273437500, 0.0003356933593750, -0.0015869140625000, 0.0004577636718750,
+-0.0006408691406250, 0.0005493164062500, -0.0010375976562500, 0.0006408691406250,
+-0.0008850097656250, 0.0006713867187500, -0.0001525878906250, 0.0006713867187500,
+-0.0008544921875000, 0.0007934570312500, 0.0000915527343750, 0.0007324218750000,
+-0.0001525878906250, 0.0008239746093750, -0.0000610351562500, 0.0008239746093750,
+0.0005187988281250, 0.0007324218750000, 0.0000000000000000, 0.0008544921875000,
+0.0004577636718750, 0.0007629394531250, 0.0004272460937500, 0.0007629394531250,
+0.0003051757812500, 0.0007629394531250, 0.0005798339843750, 0.0007019042968750,
+0.0005493164062500, 0.0007629394531250, 0.0004272460937500, 0.0006713867187500,
+0.0008239746093750, 0.0006713867187500, 0.0007324218750000, 0.0005187988281250,
+0.0006103515625000, 0.0003662109375000, 0.0011901855468750, 0.0004272460937500,
+0.0004882812500000, 0.0002441406250000, 0.0010070800781250, 0.0001525878906250,
+0.0011291503906250, 0.0001831054687500, 0.0007019042968750, 0.0001525878906250,
+0.0013732910156250, 0.0001525878906250, 0.0010070800781250, -0.0000610351562500,
+0.0013122558593750, 0.0000610351562500, 0.0015258789062500, -0.0000915527343750,
+0.0014343261718750, -0.0004272460937500, 0.0017700195312500, -0.0002746582031250,
+0.0018005371093750, -0.0004882812500000, 0.0018920898437500, -0.0007629394531250,
+0.0020141601562500, -0.0005798339843750, 0.0020141601562500, -0.0005187988281250,
+0.0021057128906250, -0.0007934570312500, 0.0021362304687500, -0.0001831054687500,
+0.0019836425781250, -0.0003967285156250, 0.0020141601562500, -0.0002746582031250,
+0.0018615722656250, 0.0002441406250000, 0.0014953613281250, 0.0001220703125000,
+0.0018615722656250, 0.0005493164062500, 0.0014648437500000, 0.0007019042968750,
+0.0014648437500000, 0.0008544921875000, 0.0017700195312500, 0.0011291503906250,
+0.0012512207031250, 0.0012512207031250, 0.0017089843750000, 0.0014343261718750,
+0.0016174316406250, 0.0016174316406250, 0.0015258789062500, 0.0015258789062500,
+0.0017700195312500, 0.0013122558593750, 0.0016479492187500, 0.0012512207031250,
+0.0015563964843750, 0.0011291503906250, 0.0018005371093750, 0.0005798339843750,
+0.0014648437500000, 0.0007019042968750, 0.0012207031250000, 0.0006713867187500,
+0.0015563964843750, -0.0002136230468750, 0.0005187988281250, 0.0006713867187500,
+0.0007629394531250, -0.0001831054687500, 0.0003662109375000, -0.0001220703125000,
+-0.0004577636718750, 0.0006713867187500, 0.0001831054687500, -0.0003662109375000,
+-0.0009460449218750, 0.0007324218750000, -0.0006103515625000, 0.0005187988281250,
+-0.0003967285156250, 0.0005493164062500, -0.0008239746093750, 0.0011901855468750,
+-0.0003662109375000, 0.0007629394531250, -0.0009765625000000, 0.0012817382812500,
+-0.0003662109375000, 0.0013732910156250, -0.0006713867187500, 0.0014648437500000,
+-0.0014343261718750, 0.0017700195312500, -0.0005187988281250, 0.0016174316406250,
+-0.0014648437500000, 0.0019836425781250, -0.0015258789062500, 0.0020751953125000,
+-0.0010986328125000, 0.0020141601562500, -0.0013427734375000, 0.0021972656250000,
+-0.0014038085937500, 0.0024108886718750, -0.0004882812500000, 0.0021362304687500,
+-0.0005187988281250, 0.0025329589843750, -0.0002441406250000, 0.0025329589843750,
+0.0003967285156250, 0.0023498535156250, 0.0002136230468750, 0.0028686523437500,
+0.0004577636718750, 0.0021972656250000, 0.0005187988281250, 0.0023193359375000,
+0.0007934570312500, 0.0023498535156250, 0.0010070800781250, 0.0018615722656250,
+0.0009460449218750, 0.0018615722656250, 0.0022277832031250, 0.0016479492187500,
+0.0021362304687500, 0.0013732910156250, 0.0025329589843750, 0.0013122558593750,
+0.0032958984375000, 0.0010681152343750, 0.0023193359375000, 0.0005798339843750,
+0.0019836425781250, 0.0008544921875000, 0.0023803710937500, 0.0004577636718750,
+0.0021972656250000, 0.0006713867187500, 0.0018005371093750, 0.0008239746093750,
+0.0022277832031250},
+{0.0032348632812500, 0.0002136230468750, 0.0032348632812500, 0.0002441406250000,
+0.0029907226562500, 0.0000305175781250, 0.0031433105468750, -0.0000915527343750,
+0.0028991699218750, -0.0000610351562500, 0.0027770996093750, -0.0000305175781250,
+0.0031127929687500, -0.0000610351562500, 0.0021972656250000, -0.0000610351562500,
+0.0027465820312500, 0.0001220703125000, 0.0024108886718750, 0.0000000000000000,
+0.0016479492187500, -0.0001831054687500, 0.0025024414062500, 0.0000915527343750,
+0.0011901855468750, -0.0003662109375000, 0.0013427734375000, -0.0004577636718750,
+0.0012512207031250, -0.0003051757812500, 0.0003662109375000, -0.0006103515625000,
+0.0006408691406250, -0.0007019042968750, 0.0002746582031250, -0.0002441406250000,
+0.0000000000000000, -0.0006103515625000, 0.0000305175781250, -0.0003356933593750,
+-0.0001220703125000, 0.0002136230468750, -0.0003051757812500, -0.0001220703125000,
+0.0001220703125000, 0.0006103515625000, -0.0002136230468750, 0.0006408691406250,
+0.0000000000000000, 0.0008239746093750, 0.0004882812500000, 0.0013732910156250,
+-0.0000915527343750, 0.0012817382812500, 0.0009460449218750, 0.0017395019531250,
+0.0006103515625000, 0.0018005371093750, 0.0008850097656250, 0.0018005371093750,
+0.0016479492187500, 0.0018920898437500, 0.0008850097656250, 0.0017089843750000,
+0.0021057128906250, 0.0015563964843750, 0.0017395019531250, 0.0011901855468750,
+0.0018310546875000, 0.0010986328125000, 0.0025329589843750, 0.0008544921875000,
+0.0018310546875000, 0.0001831054687500, 0.0025329589843750, 0.0008544921875000,
+0.0023193359375000, 0.0001525878906250, 0.0021972656250000, 0.0001220703125000,
+0.0026245117187500, 0.0006103515625000, 0.0024414062500000, -0.0003051757812500,
+0.0025939941406250, 0.0005187988281250, 0.0028991699218750, 0.0001525878906250,
+0.0028991699218750, 0.0001831054687500, 0.0030822753906250, 0.0007324218750000,
+0.0034484863281250, 0.0002441406250000, 0.0027770996093750, 0.0008239746093750,
+0.0037231445312500, 0.0007934570312500, 0.0031433105468750, 0.0006408691406250,
+0.0022888183593750, 0.0010375976562500, 0.0038452148437500, 0.0010681152343750,
+0.0018005371093750, 0.0011596679687500, 0.0026550292968750, 0.0011596679687500,
+0.0024108886718750, 0.0013122558593750, 0.0010986328125000, 0.0015258789062500,
+0.0025634765625000, 0.0014343261718750, 0.0003662109375000, 0.0018310546875000,
+0.0009460449218750, 0.0019836425781250, 0.0007934570312500, 0.0019836425781250,
+-0.0009155273437500, 0.0022277832031250, -0.0000610351562500, 0.0024414062500000,
+-0.0011596679687500, 0.0022583007812500, -0.0011901855468750, 0.0025024414062500,
+-0.0010986328125000, 0.0024108886718750, -0.0016479492187500, 0.0020141601562500,
+-0.0016174316406250, 0.0022277832031250, -0.0014343261718750, 0.0020141601562500,
+-0.0016174316406250, 0.0015869140625000, -0.0014038085937500, 0.0014648437500000,
+-0.0011596679687500, 0.0012817382812500, -0.0013732910156250, 0.0008239746093750,
+-0.0005187988281250, 0.0008239746093750, -0.0009155273437500, 0.0003662109375000,
+-0.0003967285156250, 0.0003051757812500, 0.0006408691406250, 0.0004272460937500,
+-0.0000610351562500, 0.0000305175781250, 0.0009155273437500, 0.0003967285156250,
+0.0007019042968750, 0.0001220703125000, 0.0005798339843750, 0.0000915527343750,
+0.0013732910156250, 0.0002441406250000, 0.0013732910156250, -0.0002746582031250,
+0.0012207031250000, 0.0000000000000000, 0.0015258789062500, -0.0004577636718750,
+0.0018310546875000, -0.0004882812500000, 0.0016479492187500, -0.0003662109375000,
+0.0017700195312500, -0.0010681152343750, 0.0026855468750000, -0.0002136230468750,
+0.0023498535156250, -0.0008239746093750, 0.0025634765625000, -0.0007629394531250,
+0.0034484863281250, 0.0000000000000000, 0.0029602050781250, -0.0008544921875000,
+0.0028381347656250, 0.0001525878906250, 0.0034790039062500, -0.0002441406250000,
+0.0029296875000000, -0.0001525878906250, 0.0023803710937500, 0.0005187988281250,
+0.0032653808593750, -0.0001525878906250, 0.0018310546875000, 0.0006408691406250,
+0.0024108886718750, 0.0005187988281250, 0.0021972656250000, 0.0003967285156250,
+0.0011291503906250, 0.0007324218750000, 0.0021667480468750, 0.0004272460937500,
+0.0009460449218750, 0.0004882812500000, 0.0007324218750000, 0.0004577636718750,
+0.0010070800781250, 0.0002441406250000, 0.0005798339843750, 0.0001220703125000,
+0.0003967285156250, 0.0001525878906250, 0.0008544921875000, -0.0002136230468750,
+0.0005187988281250, -0.0002746582031250, 0.0006103515625000, -0.0003051757812500,
+0.0010070800781250, -0.0004577636718750, 0.0006103515625000, -0.0004577636718750,
+0.0010375976562500, -0.0007934570312500, 0.0007629394531250, -0.0007324218750000,
+0.0008544921875000, -0.0010375976562500, 0.0012512207031250, -0.0013732910156250,
+0.0007629394531250, -0.0012207031250000, 0.0014343261718750, -0.0017700195312500,
+0.0012512207031250, -0.0017700195312500, 0.0012207031250000, -0.0016479492187500,
+0.0017700195312500, -0.0018310546875000, 0.0014343261718750, -0.0017395019531250,
+0.0017395019531250, -0.0012817382812500, 0.0018310546875000, -0.0013427734375000,
+0.0017700195312500, -0.0009155273437500, 0.0019531250000000, -0.0002746582031250,
+0.0020751953125000, -0.0005187988281250, 0.0018920898437500, 0.0001831054687500,
+0.0022583007812500, 0.0001525878906250, 0.0020446777343750, 0.0000610351562500,
+0.0018310546875000, 0.0002136230468750, 0.0024719238281250, -0.0001220703125000,
+0.0012512207031250, -0.0003051757812500, 0.0018310546875000, -0.0006713867187500,
+0.0015258789062500, -0.0008544921875000, 0.0005798339843750, -0.0010681152343750,
+0.0013732910156250, -0.0015258789062500, 0.0004272460937500, -0.0011291503906250,
+0.0003967285156250, -0.0016174316406250, 0.0006103515625000, -0.0013732910156250,
+0.0001831054687500, -0.0008239746093750, 0.0003356933593750, -0.0014343261718750,
+0.0003662109375000, -0.0001831054687500, 0.0003051757812500, -0.0004272460937500,
+0.0003356933593750, -0.0001525878906250, 0.0003967285156250, 0.0007019042968750,
+0.0004272460937500, 0.0001220703125000, 0.0003967285156250, 0.0010681152343750,
+0.0005493164062500, 0.0008850097656250, 0.0005798339843750, 0.0007629394531250,
+0.0004882812500000, 0.0011291503906250, 0.0006408691406250, 0.0007629394531250,
+0.0005798339843750, 0.0009155273437500, 0.0005187988281250, 0.0009155273437500,
+0.0005798339843750, 0.0008239746093750, 0.0004882812500000, 0.0007934570312500,
+0.0002746582031250, 0.0006408691406250, 0.0004882812500000, 0.0008544921875000,
+0.0002136230468750, 0.0007019042968750, 0.0001525878906250, 0.0007324218750000,
+0.0002746582031250, 0.0009460449218750, -0.0000915527343750, 0.0008239746093750,
+-0.0000610351562500, 0.0008850097656250, -0.0000610351562500, 0.0009765625000000,
+-0.0002746582031250, 0.0009460449218750, -0.0003662109375000, 0.0010375976562500,
+-0.0002441406250000, 0.0011901855468750, -0.0004577636718750, 0.0010681152343750,
+-0.0003662109375000, 0.0012207031250000, -0.0003356933593750, 0.0013427734375000,
+-0.0004882812500000, 0.0012207031250000, -0.0001831054687500, 0.0013732910156250,
+-0.0004577636718750, 0.0013732910156250, -0.0002746582031250, 0.0012817382812500,
+-0.0002136230468750, 0.0014648437500000, -0.0003356933593750, 0.0014648437500000,
+-0.0001831054687500, 0.0012512207031250, -0.0001525878906250, 0.0016479492187500,
+-0.0003051757812500, 0.0014343261718750, -0.0001831054687500, 0.0014648437500000,
+-0.0001525878906250, 0.0017089843750000, -0.0003967285156250, 0.0013427734375000,
+0.0000305175781250, 0.0017395019531250, -0.0001220703125000, 0.0016479492187500,
+-0.0001831054687500, 0.0015258789062500, 0.0000305175781250, 0.0016784667968750,
+-0.0002136230468750, 0.0016174316406250, -0.0001831054687500, 0.0015258789062500,
+-0.0002746582031250, 0.0014648437500000, -0.0003051757812500, 0.0012512207031250,
+-0.0002136230468750, 0.0010070800781250, -0.0002441406250000, 0.0009155273437500,
+0.0002441406250000, 0.0005798339843750, 0.0002441406250000, 0.0001831054687500,
+0.0005187988281250, 0.0001525878906250, 0.0010070800781250, -0.0000610351562500,
+0.0008850097656250, -0.0005187988281250, 0.0013732910156250, 0.0000000000000000,
+0.0013427734375000, -0.0005798339843750, 0.0016479492187500, -0.0002746582031250,
+0.0018920898437500, 0.0003051757812500, 0.0014343261718750, -0.0005187988281250,
+0.0023498535156250, 0.0009155273437500, 0.0017700195312500, 0.0004882812500000,
+0.0016784667968750, 0.0006713867187500, 0.0021972656250000, 0.0017395019531250,
+0.0012207031250000, 0.0009765625000000, 0.0017089843750000, 0.0016174316406250,
+0.0011596679687500, 0.0017395019531250, 0.0009155273437500, 0.0015258789062500,
+0.0011596679687500, 0.0016479492187500, 0.0004272460937500, 0.0016174316406250,
+0.0008544921875000, 0.0013427734375000, 0.0006408691406250, 0.0013732910156250,
+0.0005493164062500, 0.0011901855468750, 0.0008850097656250, 0.0009155273437500,
+0.0006408691406250, 0.0010986328125000, 0.0010986328125000, 0.0010986328125000,
+0.0010070800781250, 0.0008239746093750, 0.0012817382812500, 0.0011291503906250,
+0.0017089843750000, 0.0015563964843750, 0.0013122558593750, 0.0011596679687500,
+0.0019531250000000, 0.0013732910156250, 0.0018920898437500, 0.0016479492187500,
+0.0014953613281250, 0.0012512207031250, 0.0016479492187500, 0.0010375976562500,
+0.0015563964843750, 0.0014648437500000, 0.0007629394531250, 0.0007629394531250,
+0.0015869140625000, 0.0010070800781250, 0.0010681152343750, 0.0009460449218750,
+0.0003967285156250, 0.0004272460937500, 0.0019226074218750, 0.0008850097656250,
+-0.0000915527343750, 0.0002441406250000, 0.0009765625000000, 0.0003967285156250,
+0.0005187988281250, 0.0006103515625000, -0.0012512207031250, 0.0003662109375000,
+0.0003662109375000, 0.0004882812500000, -0.0014953613281250, 0.0005493164062500,
+-0.0019531250000000, 0.0007019042968750, -0.0015258789062500, 0.0004882812500000,
+-0.0024414062500000, 0.0004272460937500, -0.0030517578125000, 0.0007019042968750,
+-0.0019836425781250, -0.0007629394531250, -0.0028991699218750, -0.0001525878906250,
+-0.0028686523437500, -0.0005187988281250, -0.0019226074218750, -0.0017395019531250,
+-0.0029602050781250, -0.0007324218750000, -0.0023803710937500, -0.0011596679687500,
+-0.0023193359375000, -0.0009765625000000, -0.0025939941406250, -0.0006713867187500,
+-0.0025024414062500, -0.0003051757812500, -0.0024719238281250, 0.0002441406250000,
+-0.0019531250000000, -0.0009155273437500, -0.0023193359375000, 0.0000610351562500,
+-0.0019531250000000, -0.0000610351562500, -0.0006408691406250, -0.0022277832031250,
+-0.0007019042968750, -0.0001831054687500, -0.0010986328125000, -0.0024414062500000,
+-0.0005493164062500, -0.0010681152343750, -0.0003356933593750, -0.0016784667968750,
+-0.0002136230468750, -0.0032958984375000, 0.0001525878906250, -0.0024719238281250,
+-0.0000305175781250, -0.0022583007812500, 0.0005493164062500, -0.0031738281250000,
+0.0001220703125000, -0.0022888183593750, -0.0003967285156250, -0.0011596679687500,
+0.0004577636718750, -0.0023803710937500, -0.0009155273437500, -0.0010375976562500,
+-0.0003356933593750, -0.0008544921875000, -0.0001831054687500, -0.0013427734375000,
+-0.0008850097656250, -0.0011596679687500, -0.0000610351562500, -0.0009765625000000,
+-0.0003662109375000, -0.0010070800781250, -0.0000305175781250, -0.0009460449218750,
+-0.0000915527343750, -0.0005493164062500, -0.0002746582031250, -0.0000915527343750,
+0.0001831054687500, 0.0001220703125000, -0.0011291503906250, 0.0000915527343750,
+-0.0008850097656250},
+{0.0019531250000000, 0.0006713867187500, 0.0017089843750000, 0.0005493164062500,
+0.0018920898437500, 0.0000915527343750, 0.0023803710937500, 0.0000305175781250,
+0.0015869140625000, -0.0002441406250000, 0.0021667480468750, -0.0008239746093750,
+0.0016784667968750, -0.0009460449218750, 0.0014648437500000, -0.0012207031250000,
+0.0017089843750000, -0.0017089843750000, 0.0009765625000000, -0.0017700195312500,
+0.0013732910156250, -0.0017395019531250, 0.0010375976562500, -0.0020446777343750,
+0.0007629394531250, -0.0019226074218750, 0.0009460449218750, -0.0017395019531250,
+0.0006408691406250, -0.0020751953125000, 0.0007629394531250, -0.0016784667968750,
+0.0006713867187500, -0.0017700195312500, 0.0008239746093750, -0.0018920898437500,
+0.0010375976562500, -0.0016784667968750, 0.0008239746093750, -0.0018310546875000,
+0.0011901855468750, -0.0020141601562500, 0.0011291503906250, -0.0020751953125000,
+0.0010375976562500, -0.0022277832031250, 0.0010986328125000, -0.0025024414062500,
+0.0007629394531250, -0.0026550292968750, 0.0005187988281250, -0.0023498535156250,
+0.0006713867187500, -0.0027465820312500, 0.0002441406250000, -0.0024414062500000,
+0.0000915527343750, -0.0018920898437500, 0.0008239746093750, -0.0022583007812500,
+-0.0003967285156250, -0.0012207031250000, 0.0007019042968750, -0.0011596679687500,
+0.0003967285156250, -0.0008850097656250, -0.0008239746093750, -0.0002441406250000,
+0.0007934570312500, -0.0003356933593750, -0.0010986328125000, 0.0002136230468750,
+-0.0008850097656250, 0.0004882812500000, -0.0007019042968750, 0.0005187988281250,
+-0.0020751953125000, 0.0007324218750000, -0.0019226074218750, 0.0009155273437500,
+-0.0014038085937500, 0.0007019042968750, -0.0025329589843750, 0.0010375976562500,
+-0.0024108886718750, 0.0011901855468750, -0.0016479492187500, 0.0010681152343750,
+-0.0027160644531250, 0.0013122558593750, -0.0022277832031250, 0.0017089843750000,
+-0.0021972656250000, 0.0019226074218750, -0.0024719238281250, 0.0016174316406250,
+-0.0023803710937500, 0.0018005371093750, -0.0023498535156250, 0.0023193359375000,
+-0.0022888183593750, 0.0003356933593750, -0.0023193359375000, 0.0011901855468750,
+-0.0020446777343750, 0.0007934570312500, -0.0017700195312500, -0.0008850097656250,
+-0.0017700195312500, 0.0002441406250000, -0.0013427734375000, -0.0007324218750000,
+-0.0011596679687500, -0.0005187988281250, -0.0011901855468750, -0.0003356933593750,
+-0.0012207031250000, -0.0012207031250000, 0.0008239746093750, 0.0010986328125000,
+0.0000305175781250, 0.0003051757812500, 0.0012512207031250, 0.0006408691406250,
+0.0008544921875000, 0.0000305175781250, 0.0000915527343750, -0.0003356933593750,
+0.0009460449218750, 0.0003051757812500, -0.0002746582031250, -0.0017395019531250,
+-0.0002136230468750, -0.0011901855468750, 0.0000305175781250, -0.0014648437500000,
+-0.0003051757812500, -0.0032348632812500, 0.0000305175781250, -0.0024719238281250,
+0.0000915527343750, -0.0025024414062500, 0.0005798339843750, -0.0033569335937500,
+0.0002136230468750, -0.0027465820312500, 0.0000610351562500, -0.0018615722656250,
+0.0009155273437500, -0.0026855468750000, -0.0007019042968750, -0.0016174316406250,
+-0.0002136230468750, -0.0017089843750000, -0.0002441406250000, -0.0017089843750000,
+-0.0014648437500000, -0.0011901855468750, -0.0008239746093750, -0.0014038085937500,
+-0.0005798339843750, -0.0004577636718750, -0.0008544921875000, -0.0004577636718750,
+-0.0005187988281250, -0.0001220703125000, 0.0002136230468750, 0.0008239746093750,
+0.0002136230468750, 0.0006103515625000, -0.0001220703125000, 0.0006103515625000,
+0.0004577636718750, 0.0011596679687500, 0.0000305175781250, 0.0007629394531250,
+-0.0006713867187500, 0.0002441406250000, -0.0000305175781250, 0.0008850097656250,
+-0.0010986328125000, 0.0002441406250000, -0.0011901855468750, 0.0002746582031250,
+-0.0010375976562500, 0.0006408691406250, -0.0016479492187500, 0.0006408691406250,
+-0.0015869140625000, 0.0007019042968750, -0.0010070800781250, 0.0010681152343750,
+-0.0011901855468750, 0.0014343261718750, -0.0009765625000000, 0.0014953613281250,
+-0.0003967285156250, 0.0016479492187500, -0.0005798339843750, 0.0019531250000000,
+-0.0002136230468750, 0.0018310546875000, 0.0000610351562500, 0.0019836425781250,
+0.0001220703125000, 0.0021057128906250, 0.0003356933593750, 0.0021667480468750,
+0.0007934570312500, 0.0023193359375000, 0.0006713867187500, 0.0022277832031250,
+0.0008850097656250, 0.0023498535156250, 0.0012512207031250, 0.0021667480468750,
+0.0013122558593750, 0.0019531250000000, 0.0012817382812500, 0.0019836425781250,
+0.0017700195312500, 0.0015869140625000, 0.0017395019531250, 0.0016174316406250,
+0.0015563964843750, 0.0014343261718750, 0.0016784667968750, 0.0012817382812500,
+0.0016174316406250, 0.0015563964843750, 0.0008544921875000, 0.0010070800781250,
+0.0010375976562500, 0.0014343261718750, 0.0007629394531250, 0.0014343261718750,
+0.0000915527343750, 0.0009765625000000, 0.0005798339843750, 0.0016174316406250,
+-0.0001525878906250, 0.0011596679687500, 0.0000305175781250, 0.0013427734375000,
+0.0000915527343750, 0.0013732910156250, -0.0002746582031250, 0.0010070800781250,
+0.0000915527343750, 0.0012207031250000, 0.0000610351562500, 0.0007324218750000,
+0.0001220703125000, 0.0007324218750000, 0.0003967285156250, 0.0005187988281250,
+0.0006103515625000, 0.0001525878906250, 0.0006713867187500, 0.0002746582031250,
+0.0010375976562500, -0.0002441406250000, 0.0010375976562500, -0.0000915527343750,
+0.0012512207031250, -0.0003051757812500, 0.0015563964843750, -0.0007019042968750,
+0.0013122558593750, -0.0003051757812500, 0.0018615722656250, -0.0007629394531250,
+0.0016479492187500, -0.0006408691406250, 0.0017700195312500, -0.0005187988281250,
+0.0020751953125000, -0.0007324218750000, 0.0017089843750000, -0.0004882812500000,
+0.0023193359375000, -0.0005493164062500, 0.0020751953125000, -0.0005493164062500,
+0.0022583007812500, -0.0005798339843750, 0.0026855468750000, -0.0005798339843750,
+0.0021972656250000, -0.0005798339843750, 0.0028686523437500, -0.0004272460937500,
+0.0026855468750000, -0.0005187988281250, 0.0025939941406250, -0.0004577636718750,
+0.0028381347656250, -0.0003356933593750, 0.0025024414062500, -0.0004577636718750,
+0.0025634765625000, -0.0002746582031250, 0.0023498535156250, -0.0003662109375000,
+0.0022277832031250, -0.0002746582031250, 0.0020751953125000, -0.0001831054687500,
+0.0017089843750000, -0.0004272460937500, 0.0018310546875000, -0.0000610351562500,
+0.0014343261718750, -0.0003051757812500, 0.0013732910156250, -0.0001525878906250,
+0.0013732910156250, 0.0001220703125000, 0.0008544921875000, -0.0003051757812500,
+0.0011291503906250, 0.0001525878906250, 0.0006103515625000, 0.0000305175781250,
+0.0004882812500000, -0.0000610351562500, 0.0006713867187500, 0.0002441406250000,
+-0.0001525878906250, 0.0001831054687500, 0.0002746582031250, 0.0000000000000000,
+-0.0003051757812500, 0.0002441406250000, -0.0004272460937500, 0.0000915527343750,
+-0.0002441406250000, -0.0002136230468750, -0.0010681152343750, 0.0001220703125000,
+-0.0005798339843750, -0.0002746582031250, -0.0010681152343750, -0.0002746582031250,
+-0.0013122558593750, -0.0001525878906250, -0.0010375976562500, -0.0003662109375000,
+-0.0016784667968750, -0.0002136230468750, -0.0013732910156250, 0.0001220703125000,
+-0.0014648437500000, -0.0000305175781250, -0.0015563964843750, 0.0002441406250000,
+-0.0013732910156250, 0.0007019042968750, -0.0013427734375000, 0.0004272460937500,
+-0.0013427734375000, 0.0009765625000000, -0.0010681152343750, 0.0009765625000000,
+-0.0010070800781250, 0.0011291503906250, -0.0009460449218750, 0.0015258789062500,
+-0.0006103515625000, 0.0013732910156250, -0.0006103515625000, 0.0018005371093750,
+-0.0003662109375000, 0.0015563964843750, -0.0003051757812500, 0.0015563964843750,
+-0.0003051757812500, 0.0016479492187500, -0.0000305175781250, 0.0011596679687500,
+-0.0002136230468750, 0.0015258789062500, -0.0001220703125000, 0.0008850097656250,
+-0.0002746582031250, 0.0008544921875000, -0.0005798339843750, 0.0011291503906250,
+-0.0003967285156250, 0.0002746582031250, -0.0010070800781250, 0.0010375976562500,
+-0.0012207031250000, 0.0006408691406250, -0.0013122558593750, 0.0003356933593750,
+-0.0017089843750000, 0.0007019042968750, -0.0020141601562500, 0.0002136230468750,
+-0.0017700195312500, 0.0004882812500000, -0.0021972656250000, 0.0005798339843750,
+-0.0021972656250000, 0.0004272460937500, -0.0018310546875000, 0.0006103515625000,
+-0.0022583007812500, 0.0009765625000000, -0.0021057128906250, 0.0005798339843750,
+-0.0019531250000000, 0.0008239746093750, -0.0023193359375000, 0.0009155273437500,
+-0.0024108886718750, 0.0005187988281250, -0.0020446777343750, 0.0007324218750000,
+-0.0027465820312500, 0.0008239746093750, -0.0025024414062500, 0.0005187988281250,
+-0.0024719238281250, 0.0007629394531250, -0.0030517578125000, 0.0009765625000000,
+-0.0026550292968750, 0.0005493164062500, -0.0022888183593750, 0.0012817382812500,
+-0.0026245117187500, 0.0011596679687500, -0.0018920898437500, 0.0010070800781250,
+-0.0011596679687500, 0.0014038085937500, -0.0018615722656250, 0.0013122558593750,
+-0.0000915527343750, 0.0012817382812500, -0.0004882812500000, 0.0012207031250000,
+-0.0002441406250000, 0.0011596679687500, 0.0007019042968750, 0.0010986328125000,
+-0.0003967285156250, 0.0009460449218750, 0.0008850097656250, 0.0009460449218750,
+0.0003051757812500, 0.0004882812500000, 0.0004882812500000, 0.0003356933593750,
+0.0014648437500000, 0.0002441406250000, 0.0006713867187500, -0.0001831054687500,
+0.0016784667968750, 0.0000000000000000, 0.0018005371093750, -0.0004882812500000,
+0.0012512207031250, -0.0004882812500000, 0.0014343261718750, -0.0002746582031250,
+0.0016174316406250, -0.0008850097656250, 0.0001525878906250, -0.0004577636718750,
+0.0004577636718750, -0.0006713867187500, 0.0001525878906250, -0.0008850097656250,
+-0.0010375976562500, -0.0007019042968750, -0.0007324218750000, -0.0007324218750000,
+-0.0013122558593750, -0.0002746582031250, -0.0012207031250000, -0.0006408691406250,
+-0.0011291503906250, -0.0005798339843750, -0.0015563964843750, -0.0002136230468750,
+-0.0013427734375000, -0.0007629394531250, -0.0011901855468750, -0.0002441406250000,
+-0.0014038085937500, -0.0004882812500000, -0.0011291503906250, -0.0004272460937500,
+-0.0007629394531250, 0.0000000000000000, -0.0009155273437500, -0.0003662109375000,
+-0.0003356933593750, 0.0003051757812500, -0.0002746582031250, 0.0002136230468750,
+0.0000000000000000, 0.0001525878906250, 0.0003662109375000, 0.0006103515625000,
+0.0001831054687500, 0.0005187988281250, 0.0010986328125000, 0.0002746582031250,
+0.0007019042968750, 0.0004272460937500, 0.0010986328125000, 0.0002746582031250,
+0.0019836425781250, -0.0001220703125000, 0.0012512207031250, 0.0000305175781250,
+0.0023498535156250, 0.0000000000000000, 0.0024108886718750, -0.0001831054687500,
+0.0021667480468750, -0.0001525878906250, 0.0026245117187500, 0.0000610351562500,
+0.0025634765625000, -0.0000305175781250, 0.0021362304687500, -0.0001831054687500,
+0.0023193359375000, 0.0000305175781250, 0.0021972656250000, -0.0002746582031250,
+0.0018920898437500, -0.0007324218750000, 0.0020141601562500, -0.0004577636718750,
+0.0016784667968750, -0.0008544921875000, 0.0016174316406250, -0.0009155273437500,
+0.0013732910156250, -0.0005798339843750, 0.0011596679687500, -0.0004882812500000,
+0.0014038085937500, -0.0004882812500000, 0.0005798339843750, 0.0000610351562500,
+0.0009155273437500, 0.0003356933593750, 0.0007629394531250, 0.0004272460937500,
+0.0000915527343750},
+{-0.0002136230468750, 0.0003662109375000, -0.0005187988281250, 0.0006103515625000,
+0.0000610351562500, 0.0009765625000000, -0.0007019042968750, 0.0004577636718750,
+-0.0004577636718750, 0.0009765625000000, -0.0004882812500000, 0.0009460449218750,
+-0.0009765625000000, 0.0003967285156250, -0.0006713867187500, 0.0009155273437500,
+-0.0011291503906250, 0.0006408691406250, -0.0010375976562500, 0.0004882812500000,
+-0.0010681152343750, 0.0007934570312500, -0.0013427734375000, 0.0007629394531250,
+-0.0012207031250000, 0.0004882812500000, -0.0012817382812500, 0.0012207031250000,
+-0.0012817382812500, 0.0008850097656250, -0.0014038085937500, 0.0008239746093750,
+-0.0013732910156250, 0.0013122558593750, -0.0013732910156250, 0.0008850097656250,
+-0.0017089843750000, 0.0012817382812500, -0.0018615722656250, 0.0011596679687500,
+-0.0018310546875000, 0.0010681152343750, -0.0021362304687500, 0.0012207031250000,
+-0.0025329589843750, 0.0010070800781250, -0.0019836425781250, 0.0010375976562500,
+-0.0025329589843750, 0.0008850097656250, -0.0023498535156250, 0.0007629394531250,
+-0.0019531250000000, 0.0005798339843750, -0.0028381347656250, 0.0000915527343750,
+-0.0016784667968750, 0.0002441406250000, -0.0025329589843750, -0.0002136230468750,
+-0.0024414062500000, -0.0002746582031250, -0.0016479492187500, -0.0000610351562500,
+-0.0026550292968750, -0.0007324218750000, -0.0012512207031250, -0.0002746582031250,
+-0.0018615722656250, -0.0005493164062500, -0.0014648437500000, -0.0007324218750000,
+-0.0001220703125000, -0.0005493164062500, -0.0011901855468750, -0.0009765625000000,
+0.0002441406250000, -0.0008239746093750, 0.0004577636718750, -0.0010070800781250,
+0.0000305175781250, -0.0010986328125000, 0.0007629394531250, -0.0008239746093750,
+0.0012207031250000, -0.0006408691406250, 0.0006103515625000, -0.0003356933593750,
+0.0008544921875000, -0.0005493164062500, 0.0008850097656250, -0.0006103515625000,
+0.0004272460937500, -0.0004272460937500, 0.0005798339843750, -0.0006713867187500,
+0.0003662109375000, -0.0003356933593750, 0.0003356933593750, -0.0003662109375000,
+0.0000000000000000, -0.0003051757812500, -0.0003051757812500, 0.0000000000000000,
+-0.0001220703125000, -0.0000305175781250, -0.0009765625000000, 0.0000000000000000,
+-0.0007324218750000, 0.0002136230468750, -0.0006713867187500, 0.0000610351562500,
+-0.0011291503906250, -0.0002136230468750, -0.0007629394531250, 0.0000305175781250,
+-0.0005493164062500, -0.0000915527343750, -0.0008239746093750, -0.0001831054687500,
+-0.0003051757812500, -0.0000610351562500, 0.0000610351562500, 0.0000000000000000,
+-0.0005187988281250, -0.0000305175781250, 0.0008239746093750, 0.0000610351562500,
+0.0004882812500000, 0.0002746582031250, 0.0007629394531250, 0.0000305175781250,
+0.0018615722656250, -0.0001831054687500, 0.0013427734375000, 0.0001220703125000,
+0.0019226074218750, -0.0003662109375000, 0.0022583007812500, -0.0002441406250000,
+0.0021667480468750, -0.0000305175781250, 0.0021972656250000, -0.0000915527343750,
+0.0023803710937500, 0.0000610351562500, 0.0022277832031250, 0.0003662109375000,
+0.0021057128906250, 0.0005493164062500, 0.0021362304687500, 0.0005187988281250,
+0.0021057128906250, 0.0005798339843750, 0.0017700195312500, 0.0007934570312500,
+0.0018005371093750, 0.0001525878906250, 0.0018310546875000, 0.0003662109375000,
+0.0013427734375000, 0.0000915527343750, 0.0012207031250000, -0.0005493164062500,
+0.0014343261718750, -0.0001525878906250, 0.0003356933593750, -0.0006713867187500,
+0.0008239746093750, -0.0006713867187500, 0.0005798339843750, -0.0005493164062500,
+-0.0001831054687500, -0.0006408691406250, 0.0005493164062500, -0.0003356933593750,
+-0.0001220703125000, -0.0004272460937500, -0.0001831054687500, -0.0000915527343750,
+0.0001220703125000, -0.0001831054687500, -0.0001525878906250, -0.0004272460937500,
+-0.0002746582031250, 0.0000915527343750, 0.0002746582031250, -0.0008850097656250,
+-0.0002746582031250, -0.0005493164062500, 0.0001525878906250, -0.0006713867187500,
+0.0007934570312500, -0.0013732910156250, -0.0002441406250000, -0.0007019042968750,
+0.0012817382812500, -0.0011596679687500, 0.0008544921875000, -0.0009765625000000,
+0.0008239746093750, -0.0005798339843750, 0.0018310546875000, -0.0005187988281250,
+0.0011596679687500, -0.0001525878906250, 0.0016174316406250, 0.0001525878906250,
+0.0017395019531250, 0.0004577636718750, 0.0015563964843750, 0.0006103515625000,
+0.0016174316406250, 0.0007934570312500, 0.0016784667968750, 0.0009765625000000,
+0.0013732910156250, 0.0007324218750000, 0.0013732910156250, 0.0008544921875000,
+0.0011901855468750, 0.0007324218750000, 0.0007629394531250, 0.0003967285156250,
+0.0007629394531250, 0.0004577636718750, 0.0005493164062500, -0.0001220703125000,
+0.0001525878906250, -0.0001220703125000, 0.0000915527343750, -0.0003967285156250,
+0.0000610351562500, -0.0009155273437500, -0.0003356933593750, -0.0006713867187500,
+-0.0000915527343750, -0.0012207031250000, -0.0003051757812500, -0.0011291503906250,
+-0.0003967285156250, -0.0010070800781250, -0.0002441406250000, -0.0012512207031250,
+-0.0004882812500000, -0.0008850097656250, -0.0002441406250000, -0.0007324218750000,
+-0.0003051757812500, -0.0007934570312500, -0.0003356933593750, -0.0005187988281250,
+-0.0002746582031250, -0.0001831054687500, -0.0003356933593750, -0.0002136230468750,
+-0.0002136230468750, 0.0002441406250000, -0.0003051757812500, 0.0002746582031250,
+-0.0003356933593750, 0.0002746582031250, -0.0002746582031250, 0.0005187988281250,
+-0.0004577636718750, 0.0004577636718750, -0.0002136230468750, 0.0004577636718750,
+-0.0005798339843750, 0.0006103515625000, -0.0004272460937500, 0.0004882812500000,
+-0.0002136230468750, 0.0004272460937500, -0.0007629394531250, 0.0004882812500000,
+-0.0000305175781250, 0.0001220703125000, -0.0002746582031250, 0.0003967285156250,
+-0.0003356933593750, 0.0002136230468750, 0.0000915527343750, -0.0000915527343750,
+-0.0003662109375000, 0.0004577636718750, -0.0001220703125000, -0.0004272460937500,
+-0.0002136230468750, 0.0000000000000000, -0.0003356933593750, -0.0000610351562500,
+-0.0003051757812500, -0.0006103515625000, -0.0003662109375000, 0.0000610351562500,
+-0.0001525878906250, -0.0005187988281250, -0.0002136230468750, -0.0002746582031250,
+-0.0000610351562500, -0.0000915527343750, 0.0000915527343750, -0.0003967285156250,
+-0.0000305175781250, 0.0000305175781250, 0.0003051757812500, -0.0000610351562500,
+0.0002441406250000, 0.0000305175781250, 0.0004577636718750, 0.0001831054687500,
+0.0006713867187500, 0.0003051757812500, 0.0004577636718750, 0.0005187988281250,
+0.0010375976562500, 0.0003356933593750, 0.0006713867187500, 0.0007629394531250,
+0.0007629394531250, 0.0005493164062500, 0.0011596679687500, 0.0002441406250000,
+0.0003967285156250, 0.0007934570312500, 0.0010986328125000, -0.0003051757812500,
+0.0006713867187500, 0.0001220703125000, 0.0005187988281250, -0.0001220703125000,
+0.0008850097656250, -0.0010375976562500, 0.0003051757812500, -0.0003967285156250,
+0.0005187988281250, -0.0011901855468750, 0.0003662109375000, -0.0012512207031250,
+0.0001831054687500, -0.0011291503906250, 0.0003051757812500, -0.0014038085937500,
+0.0002136230468750, -0.0012512207031250, 0.0003051757812500, -0.0009460449218750,
+0.0005187988281250, -0.0010986328125000, 0.0005493164062500, -0.0007629394531250,
+0.0006103515625000, -0.0003356933593750, 0.0009765625000000, -0.0005493164062500,
+0.0006103515625000, 0.0000000000000000, 0.0008239746093750, 0.0000610351562500,
+0.0006713867187500, -0.0001220703125000, 0.0003967285156250, 0.0000610351562500,
+0.0007019042968750, 0.0000610351562500, -0.0000610351562500, -0.0003051757812500,
+0.0005798339843750, -0.0002441406250000, 0.0000610351562500, -0.0002441406250000,
+-0.0005798339843750, -0.0004272460937500, 0.0008544921875000, -0.0002746582031250,
+-0.0012512207031250, -0.0002136230468750, -0.0002746582031250, -0.0000610351562500,
+-0.0001831054687500, -0.0000305175781250, -0.0015869140625000, 0.0001525878906250,
+-0.0001525878906250, 0.0003967285156250, -0.0009765625000000, 0.0000915527343750,
+-0.0011596679687500, 0.0006408691406250, -0.0005187988281250, 0.0003051757812500,
+-0.0007934570312500, -0.0000610351562500, -0.0012512207031250, 0.0007629394531250,
+-0.0001220703125000, -0.0007324218750000, -0.0007934570312500, -0.0001525878906250,
+-0.0007934570312500, -0.0004272460937500, 0.0000915527343750, -0.0016784667968750,
+-0.0005798339843750, -0.0008239746093750, 0.0001831054687500, -0.0017089843750000,
+0.0002441406250000, -0.0019531250000000, 0.0002441406250000, -0.0015563964843750,
+0.0007629394531250, -0.0016784667968750, 0.0007934570312500, -0.0018615722656250,
+0.0007324218750000, -0.0014038085937500, 0.0010986328125000, -0.0013427734375000,
+0.0009765625000000, -0.0013732910156250, 0.0007629394531250, -0.0012512207031250,
+0.0010986328125000, -0.0013427734375000, 0.0008544921875000, -0.0012207031250000,
+0.0009155273437500, -0.0016479492187500, 0.0010681152343750, -0.0010070800781250,
+0.0011901855468750, -0.0004577636718750, 0.0014648437500000, -0.0012817382812500,
+0.0011901855468750, 0.0006103515625000, 0.0017395019531250, 0.0003967285156250,
+0.0016174316406250, 0.0002136230468750, 0.0005493164062500, 0.0017700195312500,
+0.0015258789062500, -0.0007324218750000, 0.0014648437500000, 0.0010986328125000,
+0.0016174316406250, 0.0002136230468750, 0.0015258789062500, 0.0003356933593750,
+0.0016174316406250, 0.0011291503906250, 0.0019531250000000, 0.0007324218750000,
+0.0013122558593750, 0.0010070800781250, 0.0016174316406250, 0.0006408691406250,
+0.0016479492187500, 0.0010375976562500, 0.0010375976562500, 0.0013427734375000,
+0.0012207031250000, 0.0005798339843750, 0.0018005371093750, 0.0021667480468750,
+0.0013427734375000, 0.0018005371093750, 0.0015258789062500, 0.0015258789062500,
+0.0021057128906250, 0.0024108886718750, 0.0014648437500000, 0.0018920898437500,
+0.0012207031250000, 0.0012207031250000, 0.0013732910156250, 0.0014648437500000,
+0.0005798339843750, 0.0009155273437500, -0.0000915527343750, 0.0000915527343750,
+0.0003051757812500, 0.0005187988281250, -0.0004272460937500, -0.0004272460937500,
+-0.0006103515625000, -0.0001831054687500, -0.0002441406250000, -0.0002746582031250,
+-0.0000305175781250, -0.0007629394531250, 0.0001525878906250, -0.0001525878906250,
+0.0004882812500000, -0.0007629394531250, 0.0009765625000000, -0.0007934570312500,
+0.0009460449218750, -0.0003662109375000, 0.0009460449218750, -0.0005798339843750,
+0.0015258789062500, -0.0007934570312500, 0.0006103515625000, 0.0004272460937500,
+0.0010681152343750, -0.0001220703125000, 0.0009460449218750, 0.0000610351562500,
+0.0001525878906250, 0.0012207031250000, 0.0006103515625000, 0.0006408691406250,
+0.0003356933593750, 0.0007934570312500, 0.0002746582031250, 0.0013122558593750,
+0.0002441406250000, 0.0009460449218750, 0.0002441406250000, 0.0006713867187500,
+0.0002441406250000, 0.0013427734375000, -0.0002136230468750, 0.0007019042968750,
+-0.0002136230468750, 0.0012817382812500, -0.0005493164062500, 0.0014038085937500,
+-0.0011291503906250, 0.0011291503906250, -0.0010375976562500, 0.0020446777343750,
+-0.0018920898437500, 0.0011596679687500, -0.0020446777343750, 0.0014343261718750,
+-0.0020446777343750, 0.0016174316406250, -0.0025329589843750, 0.0010986328125000,
+-0.0025939941406250, 0.0013427734375000, -0.0016479492187500, 0.0015869140625000,
+-0.0022583007812500, 0.0013427734375000, -0.0015258789062500, 0.0013732910156250,
+-0.0003356933593750, 0.0016479492187500, -0.0011291503906250, 0.0014343261718750,
+0.0005798339843750},
+{-0.0002746582031250, 0.0001525878906250, 0.0010681152343750, -0.0001831054687500,
+0.0007629394531250, 0.0002441406250000, 0.0007629394531250, 0.0003356933593750,
+0.0017700195312500, 0.0003051757812500, 0.0013732910156250, 0.0010070800781250,
+0.0016784667968750, 0.0002746582031250, 0.0019836425781250, 0.0008239746093750,
+0.0019531250000000, 0.0006713867187500, 0.0018920898437500, -0.0001831054687500,
+0.0020751953125000, 0.0006103515625000, 0.0022888183593750, 0.0000305175781250,
+0.0020751953125000, -0.0003662109375000, 0.0022888183593750, 0.0001220703125000,
+0.0026245117187500, 0.0001831054687500, 0.0022888183593750, -0.0003967285156250,
+0.0025939941406250, 0.0007324218750000, 0.0028076171875000, 0.0003967285156250,
+0.0023193359375000, 0.0002136230468750, 0.0021667480468750, 0.0006408691406250,
+0.0026245117187500, 0.0000610351562500, 0.0016479492187500, 0.0001525878906250,
+0.0017700195312500, -0.0003051757812500, 0.0019531250000000, -0.0003051757812500,
+0.0012817382812500, 0.0003356933593750, 0.0027160644531250, 0.0017395019531250,
+0.0020751953125000, 0.0024108886718750, 0.0022583007812500, 0.0015258789062500,
+0.0021972656250000, 0.0016784667968750, 0.0018615722656250, 0.0019531250000000,
+0.0018310546875000, 0.0010681152343750, 0.0021362304687500, 0.0018005371093750,
+0.0019836425781250, 0.0016479492187500, 0.0019531250000000, 0.0013122558593750,
+0.0021667480468750, 0.0013732910156250, 0.0018615722656250, 0.0011291503906250,
+0.0014953613281250, 0.0011291503906250, 0.0011901855468750, 0.0006713867187500,
+0.0008239746093750, 0.0010375976562500, 0.0002136230468750, 0.0013122558593750,
+-0.0001525878906250, 0.0005187988281250, 0.0000915527343750, 0.0020751953125000,
+-0.0005493164062500, 0.0015563964843750, -0.0003051757812500, 0.0014038085937500,
+0.0004882812500000, 0.0023193359375000, -0.0000305175781250, 0.0015563964843750,
+0.0004577636718750, 0.0015563964843750, 0.0011901855468750, 0.0015258789062500,
+0.0007629394531250, 0.0010681152343750, 0.0004577636718750, 0.0008544921875000,
+0.0014343261718750, 0.0010375976562500, 0.0003051757812500, 0.0005187988281250,
+0.0005187988281250, 0.0007934570312500, 0.0008239746093750, 0.0007019042968750,
+0.0002746582031250, 0.0004272460937500, 0.0005493164062500, 0.0009765625000000,
+0.0006408691406250, 0.0002441406250000, 0.0006713867187500, 0.0002136230468750,
+0.0005493164062500, 0.0004882812500000, 0.0007324218750000, 0.0000305175781250,
+0.0008544921875000, -0.0001831054687500, -0.0001220703125000, 0.0005798339843750,
+0.0003662109375000, 0.0001831054687500, 0.0000000000000000, 0.0003051757812500,
+-0.0010681152343750, 0.0010375976562500, -0.0003662109375000, 0.0006408691406250,
+-0.0010070800781250, 0.0007629394531250, -0.0012512207031250, 0.0010375976562500,
+-0.0007019042968750, 0.0008544921875000, -0.0007629394531250, 0.0008850097656250,
+-0.0010375976562500, 0.0015258789062500, 0.0005798339843750, 0.0009460449218750,
+-0.0000915527343750, 0.0017089843750000, 0.0004882812500000, 0.0016174316406250,
+0.0020141601562500, 0.0010986328125000, 0.0008544921875000, 0.0020141601562500,
+0.0021057128906250, 0.0010986328125000, 0.0023193359375000, 0.0011596679687500,
+0.0018615722656250, 0.0015258789062500, 0.0021667480468750, 0.0010986328125000,
+0.0022583007812500, 0.0010070800781250, 0.0013732910156250, 0.0014038085937500,
+0.0016479492187500, 0.0011901855468750, 0.0014648437500000, 0.0009460449218750,
+0.0008544921875000, 0.0010986328125000, 0.0014648437500000, 0.0008850097656250,
+0.0008850097656250, 0.0004882812500000, 0.0009765625000000, 0.0006103515625000,
+0.0013427734375000, 0.0004882812500000, 0.0011596679687500, 0.0002136230468750,
+0.0012207031250000, 0.0003967285156250, 0.0017700195312500, 0.0002441406250000,
+0.0016479492187500, 0.0003662109375000, 0.0018005371093750, 0.0004272460937500,
+0.0021972656250000, 0.0002746582031250, 0.0019226074218750, 0.0003967285156250,
+0.0024414062500000, 0.0005493164062500, 0.0023498535156250, 0.0002441406250000,
+0.0022583007812500, 0.0003967285156250, 0.0025939941406250, 0.0006408691406250,
+0.0024108886718750, 0.0001220703125000, 0.0022888183593750, 0.0005187988281250,
+0.0024414062500000, 0.0003051757812500, 0.0022583007812500, 0.0001525878906250,
+0.0021057128906250, 0.0002746582031250, 0.0023498535156250, 0.0000000000000000,
+0.0020141601562500, 0.0000915527343750, 0.0021057128906250, -0.0001220703125000,
+0.0022277832031250, -0.0001831054687500, 0.0020751953125000, -0.0001831054687500,
+0.0022277832031250, -0.0004577636718750, 0.0023498535156250, -0.0001220703125000,
+0.0022583007812500, -0.0004577636718750, 0.0024414062500000, -0.0004272460937500,
+0.0026855468750000, -0.0001525878906250, 0.0025634765625000, -0.0007019042968750,
+0.0030212402343750, -0.0002136230468750, 0.0029296875000000, -0.0004272460937500,
+0.0031127929687500, -0.0005798339843750, 0.0034179687500000, -0.0003356933593750,
+0.0032958984375000, -0.0005187988281250, 0.0037536621093750, -0.0006713867187500,
+0.0036315917968750, -0.0006408691406250, 0.0038146972656250, -0.0007629394531250,
+0.0041198730468750, -0.0009765625000000, 0.0038452148437500, -0.0008850097656250,
+0.0043334960937500, -0.0009155273437500, 0.0042724609375000, -0.0010375976562500,
+0.0043945312500000, -0.0008239746093750, 0.0048217773437500, -0.0006103515625000,
+0.0046997070312500, -0.0007934570312500, 0.0050354003906250, -0.0001220703125000,
+0.0051574707031250, -0.0003356933593750, 0.0052185058593750, -0.0000610351562500,
+0.0054626464843750, 0.0006103515625000, 0.0056152343750000, 0.0001831054687500,
+0.0056152343750000, 0.0008239746093750, 0.0058288574218750, 0.0009460449218750,
+0.0056762695312500, 0.0008544921875000, 0.0055541992187500, 0.0010681152343750,
+0.0059509277343750, 0.0011291503906250, 0.0051879882812500, 0.0010681152343750,
+0.0054626464843750, 0.0010986328125000, 0.0050659179687500, 0.0010986328125000,
+0.0043029785156250, 0.0009765625000000, 0.0047302246093750, 0.0009155273437500,
+0.0037231445312500, 0.0006408691406250, 0.0036926269531250, 0.0005187988281250,
+0.0035095214843750, 0.0003051757812500, 0.0029296875000000, 0.0000305175781250,
+0.0031738281250000, -0.0000305175781250, 0.0025634765625000, -0.0001831054687500,
+0.0026550292968750, -0.0001525878906250, 0.0025939941406250, -0.0002136230468750,
+0.0022277832031250, -0.0002441406250000, 0.0024414062500000, 0.0000610351562500,
+0.0023803710937500, -0.0003356933593750, 0.0023803710937500, 0.0001525878906250,
+0.0026245117187500, 0.0001831054687500, 0.0027770996093750, -0.0000915527343750,
+0.0028076171875000, 0.0007324218750000, 0.0030822753906250, 0.0000915527343750,
+0.0031738281250000, 0.0004272460937500, 0.0031127929687500, 0.0007019042968750,
+0.0030822753906250, 0.0003356933593750, 0.0031433105468750, 0.0005187988281250,
+0.0028991699218750, 0.0008239746093750, 0.0026855468750000, 0.0006103515625000,
+0.0027160644531250, 0.0007934570312500, 0.0024414062500000, 0.0010070800781250,
+0.0020751953125000, 0.0006103515625000, 0.0025329589843750, 0.0012512207031250,
+0.0018005371093750, 0.0009765625000000, 0.0020141601562500, 0.0010681152343750,
+0.0025024414062500, 0.0015869140625000, 0.0014038085937500, 0.0011901855468750,
+0.0025329589843750, 0.0015258789062500, 0.0021362304687500, 0.0017089843750000,
+0.0015258789062500, 0.0014343261718750, 0.0018920898437500, 0.0013732910156250,
+0.0015869140625000, 0.0016174316406250, 0.0009765625000000, 0.0011596679687500,
+0.0011291503906250, 0.0011901855468750, 0.0007934570312500, 0.0010375976562500,
+0.0002136230468750, 0.0006408691406250, 0.0007324218750000, 0.0007324218750000,
+0.0001220703125000, 0.0004272460937500, 0.0002441406250000, 0.0003662109375000,
+0.0003662109375000, 0.0002441406250000, 0.0002441406250000, 0.0000305175781250,
+0.0007629394531250, 0.0001831054687500, 0.0003051757812500, -0.0002746582031250,
+0.0003662109375000, -0.0003356933593750, 0.0003662109375000, -0.0002746582031250,
+-0.0001220703125000, -0.0004577636718750, -0.0002441406250000, -0.0004577636718750,
+-0.0003967285156250, -0.0003662109375000, -0.0009155273437500, -0.0003967285156250,
+-0.0010375976562500, -0.0003662109375000, -0.0009460449218750, -0.0003662109375000,
+-0.0013732910156250, -0.0004272460937500, -0.0014038085937500, -0.0003356933593750,
+-0.0013732910156250, -0.0004577636718750, -0.0016784667968750, -0.0004577636718750,
+-0.0018005371093750, -0.0003051757812500, -0.0014343261718750, -0.0002136230468750,
+-0.0013732910156250, 0.0000915527343750, -0.0016174316406250, -0.0000305175781250,
+-0.0013122558593750, 0.0000000000000000, -0.0010681152343750, 0.0002441406250000,
+-0.0013427734375000, 0.0002441406250000, -0.0003662109375000, 0.0002746582031250,
+-0.0005187988281250, 0.0003967285156250, -0.0000305175781250, 0.0003356933593750,
+0.0007629394531250, 0.0003356933593750, 0.0002136230468750, 0.0006103515625000,
+0.0016174316406250, 0.0002441406250000, 0.0012207031250000, 0.0006103515625000,
+0.0013427734375000, 0.0004577636718750, 0.0022583007812500, 0.0001525878906250,
+0.0016479492187500, 0.0007324218750000, 0.0015869140625000, 0.0002746582031250,
+0.0021057128906250, 0.0006713867187500, 0.0014953613281250, 0.0007019042968750,
+0.0009765625000000, 0.0004577636718750, 0.0016784667968750, 0.0009460449218750,
+0.0006103515625000, 0.0003967285156250, 0.0008544921875000, 0.0007019042968750,
+0.0009765625000000, 0.0005187988281250, 0.0004882812500000, 0.0000000000000000,
+0.0009765625000000, 0.0004882812500000, 0.0006713867187500, -0.0003662109375000,
+0.0007629394531250, -0.0004272460937500, 0.0008850097656250, -0.0004577636718750,
+0.0007324218750000, -0.0008544921875000, 0.0008544921875000, -0.0007629394531250,
+0.0009460449218750, -0.0007629394531250, 0.0009765625000000, -0.0007934570312500,
+0.0012207031250000, -0.0007934570312500, 0.0014648437500000, -0.0006713867187500,
+0.0013122558593750, -0.0004272460937500, 0.0018615722656250, -0.0008239746093750,
+0.0015869140625000, -0.0004272460937500, 0.0016784667968750, -0.0005187988281250,
+0.0019531250000000, -0.0009155273437500, 0.0014038085937500, -0.0003356933593750,
+0.0019226074218750, -0.0007324218750000, 0.0015869140625000, -0.0004882812500000,
+0.0014038085937500, -0.0003051757812500, 0.0017700195312500, -0.0003967285156250,
+0.0013427734375000, 0.0000610351562500, 0.0011291503906250, 0.0000915527343750,
+0.0013732910156250, 0.0004577636718750, 0.0008544921875000, 0.0006103515625000,
+0.0003662109375000, 0.0007019042968750, 0.0007934570312500, 0.0010681152343750,
+-0.0003051757812500, 0.0008239746093750, -0.0001220703125000, 0.0011901855468750,
+-0.0001525878906250, 0.0011291503906250, -0.0008850097656250, 0.0008544921875000,
+-0.0004577636718750, 0.0012817382812500, -0.0009155273437500, 0.0007019042968750,
+-0.0009460449218750, 0.0008544921875000, -0.0007934570312500, 0.0009155273437500,
+-0.0010375976562500, 0.0005493164062500, -0.0010681152343750, 0.0007324218750000,
+-0.0004882812500000, 0.0009460449218750, -0.0008850097656250, 0.0007019042968750,
+-0.0004272460937500, 0.0010070800781250, 0.0003051757812500, 0.0013732910156250,
+-0.0003662109375000, 0.0010986328125000, 0.0008544921875000, 0.0016784667968750,
+0.0007324218750000, 0.0017395019531250, 0.0006713867187500, 0.0015869140625000,
+0.0014648437500000, 0.0018920898437500, 0.0012207031250000, 0.0019836425781250,
+0.0010070800781250, 0.0016479492187500, 0.0014343261718750, 0.0020446777343750,
+0.0010986328125000},
+{0.0008544921875000, -0.0001220703125000, 0.0005493164062500, -0.0000915527343750,
+-0.0000610351562500, -0.0002136230468750, 0.0001831054687500, -0.0000915527343750,
+0.0000915527343750, 0.0000000000000000, -0.0001220703125000, 0.0001220703125000,
+0.0000000000000000, 0.0001831054687500, 0.0005798339843750, 0.0003662109375000,
+0.0009155273437500, 0.0006713867187500, 0.0002746582031250, 0.0006713867187500,
+0.0007019042968750, 0.0008239746093750, 0.0002441406250000, 0.0007934570312500,
+-0.0007019042968750, 0.0008850097656250, -0.0000610351562500, 0.0011596679687500,
+-0.0011291503906250, 0.0006408691406250, -0.0014343261718750, 0.0011901855468750,
+-0.0010070800781250, 0.0010681152343750, -0.0013122558593750, 0.0005493164062500,
+-0.0015258789062500, 0.0013732910156250, -0.0002746582031250, 0.0005798339843750,
+-0.0009155273437500, 0.0009765625000000, -0.0002746582031250, 0.0011901855468750,
+0.0009155273437500, 0.0009460449218750, -0.0001831054687500, 0.0016174316406250,
+0.0015258789062500, 0.0010681152343750, 0.0015258789062500, 0.0016174316406250,
+0.0013732910156250, 0.0013427734375000, 0.0022583007812500, 0.0007324218750000,
+0.0022277832031250, 0.0015563964843750, 0.0018920898437500, 0.0002746582031250,
+0.0023803710937500, 0.0005798339843750, 0.0021667480468750, 0.0006408691406250,
+0.0016479492187500, -0.0001831054687500, 0.0021667480468750, 0.0003051757812500,
+0.0014648437500000, 0.0001220703125000, 0.0016479492187500, 0.0002136230468750,
+0.0015563964843750, 0.0003051757812500, 0.0011291503906250, 0.0004272460937500,
+0.0015869140625000, 0.0007019042968750, 0.0007019042968750, 0.0003662109375000,
+0.0010375976562500, 0.0006408691406250, 0.0008544921875000, 0.0006713867187500,
+0.0002441406250000, 0.0004272460937500, 0.0008850097656250, 0.0007629394531250,
+0.0004882812500000, 0.0005798339843750, 0.0003356933593750, 0.0007629394531250,
+0.0007019042968750, 0.0008850097656250, 0.0006408691406250, 0.0009765625000000,
+0.0002136230468750, 0.0013122558593750, 0.0010986328125000, 0.0010681152343750,
+0.0005187988281250, 0.0016479492187500, 0.0007019042968750, 0.0016174316406250,
+0.0014038085937500, 0.0013122558593750, 0.0003356933593750, 0.0021057128906250,
+0.0014648437500000, 0.0013122558593750, 0.0010375976562500, 0.0017395019531250,
+0.0008544921875000, 0.0018615722656250, 0.0014343261718750, 0.0013122558593750,
+0.0008850097656250, 0.0018310546875000, 0.0007324218750000, 0.0014343261718750,
+0.0009765625000000, 0.0016479492187500, 0.0005493164062500, 0.0017395019531250,
+0.0001525878906250, 0.0015869140625000, 0.0005493164062500, 0.0019531250000000,
+-0.0002136230468750, 0.0016784667968750, -0.0000915527343750, 0.0018920898437500,
+-0.0000610351562500, 0.0019531250000000, -0.0005493164062500, 0.0018005371093750,
+-0.0004272460937500, 0.0020446777343750, -0.0005187988281250, 0.0017089843750000,
+-0.0006713867187500, 0.0018005371093750, -0.0005493164062500, 0.0015869140625000,
+-0.0004577636718750, 0.0013122558593750, -0.0005798339843750, 0.0016174316406250,
+-0.0002441406250000, 0.0008544921875000, -0.0003051757812500, 0.0012207031250000,
+-0.0001525878906250, 0.0011291503906250, 0.0001831054687500, 0.0006713867187500,
+0.0002136230468750, 0.0013427734375000, 0.0004272460937500, 0.0009155273437500,
+0.0005798339843750, 0.0012207031250000, 0.0006103515625000, 0.0016174316406250,
+0.0007019042968750, 0.0016174316406250, 0.0007629394531250, 0.0020751953125000,
+0.0004882812500000, 0.0024414062500000, 0.0005798339843750, 0.0026550292968750,
+0.0004272460937500, 0.0030517578125000, 0.0001220703125000, 0.0034484863281250,
+0.0001831054687500, 0.0035400390625000, 0.0000000000000000, 0.0040283203125000,
+-0.0002746582031250, 0.0041809082031250, -0.0002136230468750, 0.0043334960937500,
+-0.0002441406250000, 0.0046386718750000, -0.0007019042968750, 0.0046081542968750,
+-0.0001831054687500, 0.0048828125000000, -0.0005493164062500, 0.0050048828125000,
+-0.0005798339843750, 0.0049133300781250, -0.0002136230468750, 0.0049133300781250,
+-0.0006408691406250, 0.0049438476562500, -0.0003662109375000, 0.0046081542968750,
+-0.0003356933593750, 0.0046997070312500, -0.0003662109375000, 0.0044250488281250,
+-0.0002441406250000, 0.0040588378906250, -0.0002441406250000, 0.0041503906250000,
+-0.0000305175781250, 0.0033569335937500, -0.0000305175781250, 0.0034790039062500,
+0.0000915527343750, 0.0030212402343750, 0.0002746582031250, 0.0022888183593750,
+0.0002136230468750, 0.0026550292968750, 0.0003662109375000, 0.0014038085937500,
+0.0004577636718750, 0.0015258789062500, 0.0005187988281250, 0.0010681152343750,
+0.0005493164062500, 0.0000915527343750, 0.0006103515625000, 0.0005493164062500,
+0.0007934570312500, -0.0006713867187500, 0.0006103515625000, -0.0005187988281250,
+0.0007629394531250, -0.0008239746093750, 0.0009460449218750, -0.0016479492187500,
+0.0005493164062500, -0.0011291503906250, 0.0007629394531250, -0.0019226074218750,
+0.0005493164062500, -0.0018920898437500, 0.0003356933593750, -0.0017700195312500,
+0.0003967285156250, -0.0021362304687500, 0.0001220703125000, -0.0020446777343750,
+0.0000610351562500, -0.0018310546875000, -0.0000305175781250, -0.0020446777343750,
+-0.0000610351562500, -0.0018615722656250, 0.0000000000000000, -0.0016479492187500,
+-0.0000610351562500, -0.0019226074218750, 0.0000305175781250, -0.0015258789062500,
+0.0002746582031250, -0.0015258789062500, 0.0003051757812500, -0.0017089843750000,
+0.0003662109375000, -0.0015563964843750, 0.0007019042968750, -0.0015258789062500,
+0.0005798339843750, -0.0020446777343750, 0.0007324218750000, -0.0019226074218750,
+0.0010070800781250, -0.0022583007812500, 0.0010986328125000, -0.0028381347656250,
+0.0010986328125000, -0.0025939941406250, 0.0011291503906250, -0.0033874511718750,
+0.0013427734375000, -0.0035705566406250, 0.0011291503906250, -0.0034484863281250,
+0.0010681152343750, -0.0037536621093750, 0.0015869140625000, -0.0037231445312500,
+0.0008239746093750, -0.0032653808593750, 0.0015563964843750, -0.0034179687500000,
+0.0015869140625000, -0.0030212402343750, 0.0010681152343750, -0.0024108886718750,
+0.0021362304687500, -0.0027770996093750, 0.0013732910156250, -0.0016479492187500,
+0.0014953613281250, -0.0017395019531250, 0.0018615722656250, -0.0015869140625000,
+0.0013427734375000, -0.0009155273437500, 0.0011291503906250, -0.0012207031250000,
+0.0019836425781250, -0.0006408691406250, 0.0010070800781250, -0.0009155273437500,
+0.0013732910156250, -0.0004577636718750, 0.0023803710937500, 0.0001831054687500,
+0.0009765625000000, -0.0003967285156250, 0.0023498535156250, 0.0010070800781250,
+0.0020751953125000, 0.0008239746093750, 0.0018920898437500, 0.0011901855468750,
+0.0026550292968750, 0.0022277832031250, 0.0022888183593750, 0.0018005371093750,
+0.0026550292968750, 0.0028686523437500, 0.0025939941406250, 0.0030517578125000,
+0.0029296875000000, 0.0028991699218750, 0.0032653808593750, 0.0033874511718750,
+0.0028076171875000, 0.0033874511718750, 0.0035705566406250, 0.0027160644531250,
+0.0032958984375000, 0.0030517578125000, 0.0032348632812500, 0.0027160644531250,
+0.0037231445312500, 0.0018005371093750, 0.0032653808593750, 0.0020751953125000,
+0.0034179687500000, 0.0017089843750000, 0.0036621093750000, 0.0011596679687500,
+0.0035705566406250, 0.0014343261718750, 0.0036926269531250, 0.0016784667968750,
+0.0039978027343750, 0.0010681152343750, 0.0038757324218750, 0.0013732910156250,
+0.0040588378906250, 0.0014038085937500, 0.0039672851562500, 0.0009765625000000,
+0.0036621093750000, 0.0007324218750000, 0.0038146972656250, 0.0007019042968750,
+0.0031127929687500, 0.0005187988281250, 0.0030517578125000, 0.0001220703125000,
+0.0031127929687500, 0.0005798339843750, 0.0025939941406250, 0.0014953613281250,
+0.0028686523437500, 0.0022277832031250, 0.0018310546875000, 0.0011291503906250,
+0.0020446777343750, 0.0010681152343750, 0.0016784667968750, 0.0014038085937500,
+0.0008544921875000, 0.0014343261718750, 0.0008544921875000, 0.0012512207031250,
+0.0010681152343750, 0.0023498535156250, 0.0003967285156250, 0.0022888183593750,
+0.0007019042968750, 0.0024414062500000, 0.0013122558593750, 0.0030212402343750,
+0.0004882812500000, 0.0027770996093750, 0.0011901855468750, 0.0031127929687500,
+0.0012817382812500, 0.0029296875000000, 0.0007629394531250, 0.0029602050781250,
+0.0007019042968750, 0.0031433105468750, 0.0008239746093750, 0.0027160644531250,
+0.0005187988281250, 0.0029602050781250, 0.0000915527343750, 0.0027160644531250,
+0.0006103515625000, 0.0023498535156250, 0.0008239746093750, 0.0021972656250000,
+0.0000915527343750, 0.0018615722656250, 0.0016174316406250, 0.0017395019531250,
+0.0012817382812500, 0.0011901855468750, 0.0008544921875000, 0.0012817382812500,
+0.0015563964843750, 0.0014038085937500, 0.0011901855468750, 0.0006408691406250,
+0.0007324218750000, 0.0015563964843750, 0.0007934570312500, 0.0012817382812500,
+0.0008239746093750, 0.0009155273437500, 0.0005493164062500, 0.0012207031250000,
+0.0006713867187500, 0.0008239746093750, 0.0015869140625000, 0.0007934570312500,
+0.0012817382812500, 0.0006713867187500, 0.0021972656250000, 0.0007019042968750,
+0.0034484863281250, 0.0008544921875000, 0.0028381347656250, 0.0008239746093750,
+0.0041809082031250, 0.0011596679687500, 0.0045471191406250, 0.0012817382812500,
+0.0041809082031250, 0.0012512207031250, 0.0043945312500000, 0.0012817382812500,
+0.0045776367187500, 0.0012207031250000, 0.0036315917968750, 0.0011596679687500,
+0.0038146972656250, 0.0008850097656250, 0.0035400390625000, 0.0009155273437500,
+0.0027465820312500, 0.0008850097656250, 0.0032043457031250, 0.0005493164062500,
+0.0029296875000000, 0.0007019042968750, 0.0026245117187500, 0.0005798339843750,
+0.0031433105468750, 0.0003356933593750, 0.0035095214843750, 0.0004272460937500,
+0.0030822753906250, 0.0004577636718750, 0.0041503906250000, 0.0000610351562500,
+0.0040893554687500, 0.0003662109375000, 0.0040283203125000, 0.0003967285156250,
+0.0045471191406250, 0.0001831054687500, 0.0043334960937500, 0.0007934570312500,
+0.0039978027343750, 0.0003967285156250, 0.0041503906250000, 0.0007324218750000,
+0.0036926269531250, 0.0009155273437500, 0.0032653808593750, 0.0006713867187500,
+0.0035095214843750, 0.0009765625000000, 0.0025939941406250, 0.0010375976562500,
+0.0026245117187500, 0.0009155273437500, 0.0025939941406250, 0.0010375976562500,
+0.0020141601562500, 0.0011291503906250, 0.0023498535156250, 0.0009155273437500,
+0.0025024414062500, 0.0009765625000000, 0.0023498535156250, 0.0009765625000000,
+0.0028076171875000, 0.0007934570312500, 0.0033569335937500, 0.0006408691406250,
+0.0032043457031250, 0.0006408691406250, 0.0042114257812500, 0.0006408691406250,
+0.0041809082031250, 0.0004882812500000, 0.0046386718750000, 0.0005493164062500,
+0.0054321289062500, 0.0006713867187500, 0.0050659179687500, 0.0004577636718750,
+0.0058288574218750, 0.0007019042968750, 0.0059814453125000, 0.0004272460937500,
+0.0058898925781250, 0.0004577636718750, 0.0061950683593750, 0.0005798339843750,
+0.0062255859375000, 0.0000305175781250, 0.0057373046875000, 0.0005187988281250,
+0.0060424804687500, 0.0001220703125000, 0.0056457519531250, -0.0000915527343750,
+0.0050048828125000, 0.0002136230468750, 0.0054931640625000, -0.0002136230468750,
+0.0041809082031250, -0.0001525878906250, 0.0045166015625000, -0.0002746582031250,
+0.0040283203125000, -0.0004577636718750, 0.0028991699218750, -0.0005493164062500,
+0.0036926269531250},
+{0.0003967285156250, 0.0016784667968750, -0.0001525878906250, 0.0020446777343750,
+0.0003051757812500, 0.0021362304687500, -0.0004577636718750, 0.0022888183593750,
+-0.0000610351562500, 0.0023498535156250, 0.0008239746093750, 0.0021667480468750,
+-0.0001220703125000, 0.0024414062500000, 0.0013427734375000, 0.0026245117187500,
+0.0012817382812500, 0.0023193359375000, 0.0011291503906250, 0.0026550292968750,
+0.0016784667968750, 0.0030212402343750, 0.0013122558593750, 0.0023803710937500,
+0.0014343261718750, 0.0030212402343750, 0.0009155273437500, 0.0027465820312500,
+0.0014038085937500, 0.0023193359375000, 0.0018310546875000, 0.0023498535156250,
+0.0008544921875000, 0.0018615722656250, 0.0030517578125000, 0.0016784667968750,
+0.0024108886718750, 0.0012207031250000, 0.0026550292968750, 0.0011291503906250,
+0.0042724609375000, 0.0011596679687500, 0.0033874511718750, 0.0007019042968750,
+0.0039672851562500, 0.0009155273437500, 0.0043334960937500, 0.0008544921875000,
+0.0035095214843750, 0.0005798339843750, 0.0031433105468750, 0.0005187988281250,
+0.0036926269531250, 0.0005187988281250, 0.0021362304687500, 0.0003051757812500,
+0.0021362304687500, 0.0002441406250000, 0.0022888183593750, 0.0004882812500000,
+0.0014953613281250, 0.0005187988281250, 0.0017395019531250, 0.0004882812500000,
+0.0020446777343750, 0.0009155273437500, 0.0016784667968750, 0.0006713867187500,
+0.0019836425781250, 0.0007324218750000, 0.0025024414062500, 0.0009460449218750,
+0.0021057128906250, 0.0004882812500000, 0.0024108886718750, 0.0009460449218750,
+0.0027160644531250, 0.0006408691406250, 0.0020141601562500, 0.0005493164062500,
+0.0017395019531250, 0.0008850097656250, 0.0023498535156250, 0.0004272460937500,
+0.0005493164062500, 0.0004577636718750, 0.0008544921875000, 0.0006103515625000,
+0.0005798339843750, 0.0003356933593750, -0.0007629394531250, 0.0001525878906250,
+-0.0001220703125000, 0.0004882812500000, -0.0007019042968750, 0.0000915527343750,
+-0.0010986328125000, 0.0003051757812500, -0.0006103515625000, 0.0004577636718750,
+-0.0004577636718750, 0.0003967285156250, -0.0007934570312500, 0.0006408691406250,
+0.0007629394531250, 0.0006713867187500, 0.0003662109375000, 0.0007019042968750,
+0.0009765625000000, 0.0008544921875000, 0.0024414062500000, 0.0009460449218750,
+0.0017395019531250, 0.0008544921875000, 0.0030212402343750, 0.0010375976562500,
+0.0033264160156250, 0.0010375976562500, 0.0030822753906250, 0.0009765625000000,
+0.0036315917968750, 0.0009765625000000, 0.0037536621093750, 0.0009460449218750,
+0.0032348632812500, 0.0007629394531250, 0.0037841796875000, 0.0006408691406250,
+0.0031433105468750, 0.0006408691406250, 0.0024108886718750, 0.0005187988281250,
+0.0033569335937500, 0.0004272460937500, 0.0013122558593750, 0.0006713867187500,
+0.0018920898437500, 0.0004882812500000, 0.0015563964843750, 0.0005493164062500,
+0.0001525878906250, 0.0007629394531250, 0.0010986328125000, 0.0004577636718750,
+-0.0004272460937500, 0.0006103515625000, -0.0003662109375000, 0.0004272460937500,
+-0.0003356933593750, 0.0002441406250000, -0.0011901855468750, 0.0002441406250000,
+-0.0008850097656250, 0.0000915527343750, -0.0009460449218750, -0.0000915527343750,
+-0.0010681152343750, -0.0000305175781250, -0.0007324218750000, -0.0001831054687500,
+-0.0005493164062500, -0.0004272460937500, -0.0007629394531250, -0.0002746582031250,
+-0.0000610351562500, -0.0005798339843750, -0.0001525878906250, -0.0005798339843750,
+0.0000915527343750, -0.0004882812500000, 0.0006408691406250, -0.0005798339843750,
+0.0004577636718750, -0.0004882812500000, 0.0010681152343750, -0.0002746582031250,
+0.0010681152343750, -0.0003051757812500, 0.0011596679687500, -0.0001831054687500,
+0.0014953613281250, 0.0000610351562500, 0.0013427734375000, 0.0000000000000000,
+0.0017089843750000, 0.0001220703125000, 0.0016479492187500, 0.0001831054687500,
+0.0017089843750000, 0.0001525878906250, 0.0019226074218750, 0.0001220703125000,
+0.0018005371093750, 0.0002441406250000, 0.0020141601562500, 0.0001525878906250,
+0.0020446777343750, 0.0001220703125000, 0.0020446777343750, 0.0001831054687500,
+0.0020751953125000, 0.0001220703125000, 0.0020141601562500, -0.0000915527343750,
+0.0022583007812500, -0.0000305175781250, 0.0020141601562500, -0.0002746582031250,
+0.0021057128906250, -0.0002746582031250, 0.0023498535156250, -0.0002136230468750,
+0.0019531250000000, -0.0003967285156250, 0.0024108886718750, -0.0001220703125000,
+0.0023498535156250, -0.0001831054687500, 0.0023193359375000, -0.0001220703125000,
+0.0025939941406250, 0.0001220703125000, 0.0024108886718750, 0.0000610351562500,
+0.0025939941406250, 0.0001831054687500, 0.0025634765625000, 0.0003051757812500,
+0.0025024414062500, 0.0002136230468750, 0.0026245117187500, 0.0002746582031250,
+0.0025024414062500, 0.0005187988281250, 0.0025634765625000, 0.0003356933593750,
+0.0025329589843750, 0.0003967285156250, 0.0024414062500000, 0.0004577636718750,
+0.0024719238281250, 0.0002746582031250, 0.0025024414062500, 0.0001831054687500,
+0.0023803710937500, 0.0005187988281250, 0.0024108886718750, 0.0000305175781250,
+0.0024108886718750, 0.0001831054687500, 0.0022888183593750, 0.0005493164062500,
+0.0022888183593750, -0.0001220703125000, 0.0024719238281250, 0.0007324218750000,
+0.0023193359375000, 0.0004577636718750, 0.0024108886718750, 0.0005493164062500,
+0.0026550292968750, 0.0011291503906250, 0.0025329589843750, 0.0007324218750000,
+0.0027160644531250, 0.0012207031250000, 0.0027465820312500, 0.0014343261718750,
+0.0027160644531250, 0.0013427734375000, 0.0027160644531250, 0.0014953613281250,
+0.0026855468750000, 0.0017700195312500, 0.0025024414062500, 0.0014648437500000,
+0.0024108886718750, 0.0017089843750000, 0.0021362304687500, 0.0017700195312500,
+0.0018310546875000, 0.0015258789062500, 0.0018615722656250, 0.0017089843750000,
+0.0016174316406250, 0.0014038085937500, 0.0013122558593750, 0.0014648437500000,
+0.0015563964843750, 0.0013732910156250, 0.0016784667968750, 0.0011291503906250,
+0.0009460449218750, 0.0011596679687500, 0.0011596679687500, 0.0009765625000000,
+0.0009460449218750, 0.0010070800781250, 0.0004577636718750, 0.0008850097656250,
+0.0002441406250000, 0.0007019042968750, 0.0001220703125000, 0.0009460449218750,
+-0.0004272460937500, 0.0005798339843750, -0.0005493164062500, 0.0005798339843750,
+-0.0006713867187500, 0.0006713867187500, -0.0007934570312500, 0.0005187988281250,
+-0.0005493164062500, 0.0005187988281250, -0.0004272460937500, 0.0004882812500000,
+-0.0003051757812500, 0.0005187988281250, 0.0000305175781250, 0.0005798339843750,
+0.0003662109375000, 0.0006408691406250, 0.0006103515625000, 0.0006713867187500,
+0.0010681152343750, 0.0008239746093750, 0.0011901855468750, 0.0008850097656250,
+0.0017089843750000, 0.0010681152343750, 0.0020446777343750, 0.0013732910156250,
+0.0017395019531250, 0.0013122558593750, 0.0025329589843750, 0.0016784667968750,
+0.0022888183593750, 0.0020446777343750, 0.0022583007812500, 0.0018920898437500,
+0.0026550292968750, 0.0018920898437500, 0.0021057128906250, 0.0022888183593750,
+0.0022888183593750, 0.0016479492187500, 0.0021057128906250, 0.0021057128906250,
+0.0020446777343750, 0.0021057128906250, 0.0019531250000000, 0.0017089843750000,
+0.0015258789062500, 0.0022583007812500, 0.0019836425781250, 0.0017700195312500,
+0.0017089843750000, 0.0020446777343750, 0.0015258789062500, 0.0020446777343750,
+0.0019836425781250, 0.0017395019531250, 0.0018005371093750, 0.0021362304687500,
+0.0011596679687500, 0.0014343261718750, 0.0016479492187500, 0.0018310546875000,
+0.0010070800781250, 0.0017395019531250, 0.0001220703125000, 0.0010375976562500,
+0.0008850097656250, 0.0015563964843750, -0.0003051757812500, 0.0011291503906250,
+-0.0004272460937500, 0.0010986328125000, -0.0003356933593750, 0.0012817382812500,
+-0.0010070800781250, 0.0011291503906250, -0.0011596679687500, 0.0010375976562500,
+-0.0008850097656250, 0.0014038085937500, -0.0014038085937500, 0.0012207031250000,
+-0.0010986328125000, 0.0013122558593750, -0.0006103515625000, 0.0015563964843750,
+-0.0012512207031250, 0.0012817382812500, 0.0001831054687500, 0.0017395019531250,
+-0.0000610351562500, 0.0016174316406250, 0.0001831054687500, 0.0016479492187500,
+0.0011901855468750, 0.0019836425781250, 0.0007324218750000, 0.0018310546875000,
+0.0013732910156250, 0.0017395019531250, 0.0014648437500000, 0.0018310546875000,
+0.0013122558593750, 0.0017700195312500, 0.0015258789062500, 0.0015258789062500,
+0.0014343261718750, 0.0015258789062500, 0.0012817382812500, 0.0016174316406250,
+0.0015258789062500, 0.0014038085937500, 0.0012512207031250, 0.0014953613281250,
+0.0010070800781250, 0.0016479492187500, 0.0014648437500000, 0.0013732910156250,
+0.0004882812500000, 0.0015258789062500, 0.0009155273437500, 0.0015869140625000,
+0.0007934570312500, 0.0012817382812500, -0.0000915527343750, 0.0011291503906250,
+0.0003967285156250, 0.0013427734375000, -0.0000915527343750, 0.0005187988281250,
+-0.0003662109375000, 0.0007324218750000, -0.0000915527343750, 0.0004882812500000,
+-0.0003051757812500, -0.0003051757812500, -0.0006408691406250, 0.0000610351562500,
+0.0001525878906250, -0.0004882812500000, -0.0003356933593750, -0.0005493164062500,
+-0.0001220703125000, -0.0004882812500000, 0.0005493164062500, -0.0006103515625000,
+-0.0002136230468750, -0.0005187988281250, 0.0007019042968750, -0.0003967285156250,
+0.0005493164062500, -0.0003051757812500, 0.0004272460937500, -0.0002441406250000,
+0.0010070800781250, -0.0000610351562500, 0.0007324218750000, 0.0001220703125000,
+0.0005187988281250, 0.0001525878906250, 0.0007629394531250, 0.0004882812500000,
+0.0004577636718750, 0.0005493164062500, -0.0000305175781250, 0.0004882812500000,
+0.0003051757812500, 0.0009460449218750, -0.0003662109375000, 0.0004882812500000,
+-0.0003967285156250, 0.0007629394531250, -0.0003051757812500, 0.0007934570312500,
+-0.0006408691406250, 0.0004577636718750, -0.0006103515625000, 0.0008544921875000,
+-0.0005798339843750, 0.0004577636718750, -0.0008239746093750, 0.0005187988281250,
+-0.0006408691406250, 0.0005187988281250, -0.0004882812500000, 0.0003051757812500,
+-0.0007019042968750, 0.0004272460937500, -0.0000610351562500, 0.0003356933593750,
+-0.0002441406250000, 0.0002746582031250, -0.0001220703125000, 0.0002136230468750,
+0.0004272460937500, 0.0001831054687500, 0.0003051757812500, 0.0002441406250000,
+0.0004882812500000, 0.0001220703125000, 0.0008239746093750, 0.0001525878906250,
+0.0006713867187500, 0.0001831054687500, 0.0006103515625000, 0.0001220703125000,
+0.0011291503906250, 0.0001525878906250, 0.0003662109375000, 0.0004577636718750,
+0.0007629394531250, 0.0003051757812500, 0.0007019042968750, 0.0005187988281250,
+0.0001220703125000, 0.0008850097656250, 0.0006713867187500, 0.0006103515625000,
+0.0001831054687500, 0.0014648437500000, 0.0001220703125000, 0.0012207031250000,
+0.0002136230468750, 0.0015563964843750, 0.0000305175781250, 0.0021667480468750,
+0.0000305175781250, 0.0017395019531250, 0.0001525878906250, 0.0028076171875000,
+0.0001220703125000, 0.0026855468750000, 0.0002441406250000, 0.0028381347656250,
+0.0003356933593750, 0.0035400390625000, 0.0003356933593750, 0.0032043457031250,
+0.0006103515625000, 0.0036926269531250, 0.0004272460937500, 0.0038146972656250,
+0.0005798339843750, 0.0036926269531250, 0.0007934570312500, 0.0038757324218750,
+0.0004577636718750, 0.0040588378906250, 0.0006408691406250, 0.0037841796875000,
+0.0005798339843750},
+{-0.0015563964843750, 0.0010681152343750, -0.0019226074218750, 0.0009765625000000,
+-0.0013732910156250, 0.0009460449218750, -0.0012512207031250, 0.0009460449218750,
+-0.0017395019531250, 0.0006713867187500, -0.0001525878906250, 0.0009460449218750,
+-0.0005798339843750, 0.0006408691406250, -0.0001831054687500, 0.0006713867187500,
+0.0010681152343750, 0.0010070800781250, 0.0002746582031250, 0.0005493164062500,
+0.0012817382812500, 0.0008239746093750, 0.0013732910156250, 0.0006103515625000,
+0.0011901855468750, 0.0003662109375000, 0.0016479492187500, 0.0003051757812500,
+0.0016479492187500, 0.0000610351562500, 0.0013732910156250, -0.0001831054687500,
+0.0018005371093750, -0.0004272460937500, 0.0014343261718750, -0.0005493164062500,
+0.0010375976562500, -0.0007019042968750, 0.0017700195312500, -0.0009155273437500,
+0.0005493164062500, -0.0006103515625000, 0.0010070800781250, -0.0008850097656250,
+0.0009765625000000, -0.0007019042968750, 0.0001220703125000, -0.0002136230468750,
+0.0006713867187500, -0.0004882812500000, 0.0001525878906250, 0.0000610351562500,
+-0.0000610351562500, 0.0002136230468750, 0.0001525878906250, 0.0002136230468750,
+0.0000000000000000, 0.0004882812500000, -0.0002136230468750, 0.0005493164062500,
+0.0000610351562500, 0.0006103515625000, -0.0000610351562500, 0.0007934570312500,
+-0.0001831054687500, 0.0009155273437500, 0.0000000000000000, 0.0009765625000000,
+0.0000305175781250, 0.0010375976562500, -0.0000610351562500, 0.0011901855468750,
+0.0001525878906250, 0.0011901855468750, 0.0001220703125000, 0.0012207031250000,
+0.0000610351562500, 0.0013427734375000, 0.0003967285156250, 0.0012817382812500,
+0.0000000000000000, 0.0013122558593750, 0.0001220703125000, 0.0014343261718750,
+0.0001831054687500, 0.0012817382812500, -0.0001220703125000, 0.0011596679687500,
+-0.0000915527343750, 0.0012817382812500, -0.0000610351562500, 0.0008239746093750,
+-0.0003051757812500, 0.0008544921875000, -0.0002746582031250, 0.0005493164062500,
+-0.0001220703125000, 0.0001525878906250, -0.0003356933593750, 0.0002746582031250,
+-0.0001525878906250, -0.0003662109375000, -0.0000915527343750, -0.0003662109375000,
+-0.0000610351562500, -0.0005187988281250, 0.0000610351562500, -0.0009765625000000,
+0.0002136230468750, -0.0009155273437500, 0.0002441406250000, -0.0009155273437500,
+0.0003967285156250, -0.0011596679687500, 0.0004577636718750, -0.0008850097656250,
+0.0005798339843750, -0.0006408691406250, 0.0007629394531250, -0.0008544921875000,
+0.0005798339843750, 0.0000915527343750, 0.0008850097656250, -0.0000305175781250,
+0.0008239746093750, 0.0004882812500000, 0.0006408691406250, 0.0014038085937500,
+0.0010375976562500, 0.0010070800781250, 0.0004882812500000, 0.0025329589843750,
+0.0007324218750000, 0.0023193359375000, 0.0007019042968750, 0.0028076171875000,
+0.0004272460937500, 0.0039672851562500, 0.0007324218750000, 0.0033874511718750,
+0.0005798339843750, 0.0049133300781250, 0.0004577636718750, 0.0047607421875000,
+0.0006408691406250, 0.0049438476562500, 0.0006408691406250, 0.0058593750000000,
+0.0004577636718750, 0.0054016113281250, 0.0007019042968750, 0.0060424804687500,
+0.0005493164062500, 0.0061645507812500, 0.0003662109375000, 0.0059509277343750,
+0.0004272460937500, 0.0061645507812500, 0.0003356933593750, 0.0062561035156250,
+0.0002441406250000, 0.0057067871093750, 0.0002746582031250, 0.0060119628906250,
+0.0002441406250000, 0.0055847167968750, 0.0002746582031250, 0.0049438476562500,
+0.0004272460937500, 0.0053710937500000, 0.0003356933593750, 0.0040893554687500,
+0.0005187988281250, 0.0043945312500000, 0.0006103515625000, 0.0038757324218750,
+0.0006713867187500, 0.0028381347656250, 0.0010070800781250, 0.0036010742187500,
+0.0009155273437500, 0.0021972656250000, 0.0012512207031250, 0.0024414062500000,
+0.0013122558593750, 0.0023193359375000, 0.0011596679687500, 0.0014648437500000,
+0.0015563964843750, 0.0020141601562500, 0.0011291503906250, 0.0014038085937500,
+0.0013732910156250, 0.0014038085937500, 0.0012817382812500, 0.0015563964843750,
+0.0009765625000000, 0.0013732910156250, 0.0014648437500000, 0.0015563964843750,
+0.0005493164062500, 0.0016479492187500, 0.0011901855468750, 0.0017395019531250,
+0.0009765625000000, 0.0017700195312500, 0.0003356933593750, 0.0018920898437500,
+0.0014953613281250, 0.0019531250000000, 0.0003662109375000, 0.0016784667968750,
+0.0007019042968750, 0.0019836425781250, 0.0010986328125000, 0.0017089843750000,
+0.0005187988281250, 0.0013427734375000, 0.0008239746093750, 0.0018005371093750,
+0.0013122558593750, 0.0007629394531250, 0.0005798339843750, 0.0011291503906250,
+0.0010070800781250, 0.0009155273437500, 0.0015563964843750, 0.0001525878906250,
+0.0004272460937500, 0.0007934570312500, 0.0018615722656250, 0.0002746582031250,
+0.0012207031250000, 0.0003967285156250, 0.0012512207031250, 0.0007019042968750,
+0.0023193359375000, 0.0006713867187500, 0.0014648437500000, 0.0009155273437500,
+0.0024414062500000, 0.0009460449218750, 0.0024719238281250, 0.0010375976562500,
+0.0023498535156250, 0.0011596679687500, 0.0027770996093750, 0.0010986328125000,
+0.0025634765625000, 0.0010986328125000, 0.0028686523437500, 0.0014038085937500,
+0.0026550292968750, 0.0009460449218750, 0.0027770996093750, 0.0012817382812500,
+0.0030822753906250, 0.0016784667968750, 0.0026550292968750, 0.0009765625000000,
+0.0030822753906250, 0.0022888183593750, 0.0032043457031250, 0.0016174316406250,
+0.0026855468750000, 0.0019226074218750, 0.0026855468750000, 0.0030517578125000,
+0.0031127929687500, 0.0019226074218750, 0.0022277832031250, 0.0035705566406250,
+0.0025024414062500, 0.0031738281250000, 0.0024108886718750, 0.0028686523437500,
+0.0018005371093750, 0.0037536621093750, 0.0021972656250000, 0.0030822753906250,
+0.0017395019531250, 0.0025024414062500, 0.0017395019531250, 0.0026550292968750,
+0.0016174316406250, 0.0020446777343750, 0.0013732910156250, 0.0011596679687500,
+0.0014953613281250, 0.0014038085937500, 0.0010070800781250, 0.0008544921875000,
+0.0011596679687500, 0.0007324218750000, 0.0011596679687500, 0.0007629394531250,
+0.0007934570312500, 0.0006713867187500, 0.0010681152343750, 0.0007324218750000,
+0.0010986328125000, 0.0003967285156250, 0.0009155273437500, 0.0004577636718750,
+0.0010681152343750, 0.0006103515625000, 0.0013427734375000, 0.0005493164062500,
+-0.0005187988281250, 0.0012207031250000, -0.0002441406250000, -0.0001525878906250,
+-0.0006103515625000, 0.0004272460937500, -0.0002136230468750, 0.0003967285156250,
+0.0002441406250000, -0.0003051757812500, -0.0000915527343750, -0.0000305175781250,
+0.0007019042968750, 0.0009460449218750, 0.0009765625000000, 0.0005187988281250,
+0.0007934570312500, 0.0013122558593750, 0.0009765625000000, 0.0027160644531250,
+0.0012207031250000, 0.0021972656250000, 0.0007629394531250, 0.0032043457031250,
+0.0007934570312500, 0.0038146972656250, 0.0012512207031250, 0.0034484863281250,
+0.0011901855468750, 0.0035095214843750, 0.0009155273437500, 0.0038452148437500,
+0.0020446777343750, 0.0030517578125000, 0.0018615722656250, 0.0032043457031250,
+0.0015869140625000, 0.0028991699218750, 0.0021972656250000, 0.0023193359375000,
+0.0021362304687500, 0.0027160644531250, 0.0007019042968750, 0.0014953613281250,
+0.0013427734375000, 0.0017395019531250, 0.0007019042968750, 0.0015869140625000,
+-0.0007934570312500, 0.0006103515625000, 0.0000305175781250, 0.0010681152343750,
+-0.0007629394531250, 0.0010375976562500, -0.0011901855468750, 0.0005798339843750,
+-0.0006103515625000, 0.0011291503906250, -0.0004577636718750, 0.0016784667968750,
+-0.0009765625000000, 0.0010986328125000, 0.0001525878906250, 0.0020141601562500,
+-0.0001525878906250, 0.0018920898437500, -0.0004272460937500, 0.0015869140625000,
+0.0000305175781250, 0.0017089843750000, -0.0004272460937500, 0.0013427734375000,
+-0.0012207031250000, 0.0010375976562500, -0.0011291503906250, 0.0007934570312500,
+-0.0016174316406250, 0.0006103515625000, -0.0024108886718750, 0.0005493164062500,
+-0.0021057128906250, 0.0003356933593750, -0.0025634765625000, 0.0003356933593750,
+-0.0026245117187500, 0.0003662109375000, -0.0022583007812500, 0.0002136230468750,
+-0.0022277832031250, 0.0001220703125000, -0.0021362304687500, 0.0002441406250000,
+-0.0010070800781250, 0.0001220703125000, -0.0012512207031250, -0.0001220703125000,
+-0.0006103515625000, 0.0000915527343750, 0.0004882812500000, 0.0000915527343750,
+0.0000000000000000, -0.0003967285156250, 0.0012512207031250, 0.0002136230468750,
+0.0014038085937500, -0.0002746582031250, 0.0013732910156250, -0.0002136230468750,
+0.0020751953125000, 0.0002136230468750, 0.0020141601562500, -0.0004577636718750,
+0.0017700195312500, 0.0000610351562500, 0.0022888183593750, 0.0000915527343750,
+0.0019226074218750, -0.0001525878906250, 0.0013427734375000, 0.0002136230468750,
+0.0019226074218750, 0.0002746582031250, 0.0009765625000000, -0.0000610351562500,
+0.0009460449218750, 0.0002441406250000, 0.0011291503906250, 0.0001220703125000,
+0.0007019042968750, -0.0002441406250000, 0.0005798339843750, 0.0001831054687500,
+0.0010681152343750, -0.0000610351562500, 0.0004882812500000, -0.0001220703125000,
+0.0009155273437500, 0.0000915527343750, 0.0014953613281250, 0.0001220703125000,
+0.0007629394531250, 0.0000610351562500, 0.0021057128906250, 0.0003051757812500,
+0.0016479492187500, 0.0002441406250000, 0.0019226074218750, 0.0001831054687500,
+0.0028991699218750, 0.0003967285156250, 0.0021057128906250, 0.0003967285156250,
+0.0033569335937500, 0.0001525878906250, 0.0030822753906250, 0.0004882812500000,
+0.0030822753906250, 0.0003662109375000, 0.0038146972656250, 0.0000610351562500,
+0.0032958984375000, 0.0006103515625000, 0.0035095214843750, 0.0000610351562500,
+0.0036621093750000, 0.0002441406250000, 0.0032043457031250, 0.0004272460937500,
+0.0030517578125000, 0.0002136230468750, 0.0031738281250000, 0.0005187988281250,
+0.0021362304687500, 0.0005187988281250, 0.0023803710937500, 0.0005798339843750,
+0.0020141601562500, 0.0007019042968750, 0.0011901855468750, 0.0007019042968750,
+0.0017395019531250, 0.0007934570312500, 0.0008239746093750, 0.0007934570312500,
+0.0009765625000000, 0.0008544921875000, 0.0009460449218750, 0.0008239746093750,
+0.0003662109375000, 0.0008239746093750, 0.0008239746093750, 0.0008850097656250,
+0.0003051757812500, 0.0007629394531250, 0.0004882812500000, 0.0007934570312500,
+0.0005798339843750, 0.0007324218750000, 0.0003967285156250, 0.0005493164062500,
+0.0007019042968750, 0.0005798339843750, 0.0007629394531250, 0.0004272460937500,
+0.0008239746093750, 0.0002746582031250, 0.0010986328125000, 0.0002136230468750,
+0.0013427734375000, 0.0001525878906250, 0.0014038085937500, 0.0000305175781250,
+0.0019531250000000, -0.0000610351562500, 0.0020141601562500, -0.0000915527343750,
+0.0023193359375000, -0.0002136230468750, 0.0028381347656250, -0.0002441406250000,
+0.0028991699218750, -0.0002136230468750, 0.0032348632812500, -0.0002746582031250,
+0.0035095214843750, -0.0003356933593750, 0.0035400390625000, -0.0002441406250000,
+0.0036621093750000, -0.0002136230468750, 0.0038757324218750, -0.0003662109375000,
+0.0037841796875000, -0.0000915527343750, 0.0039062500000000, -0.0002136230468750,
+0.0038757324218750, -0.0001220703125000, 0.0036621093750000, -0.0000610351562500,
+0.0038452148437500, -0.0002441406250000, 0.0034790039062500, 0.0000915527343750,
+0.0035400390625000, -0.0001831054687500, 0.0034484863281250, -0.0000610351562500,
+0.0031127929687500},
+{0.0007629394531250, 0.0002441406250000, 0.0016479492187500, 0.0002136230468750,
+0.0016784667968750, 0.0001831054687500, 0.0018615722656250, 0.0000305175781250,
+0.0023193359375000, 0.0000915527343750, 0.0022583007812500, 0.0000610351562500,
+0.0021362304687500, -0.0001525878906250, 0.0026245117187500, -0.0001220703125000,
+0.0020446777343750, -0.0000305175781250, 0.0023193359375000, -0.0004272460937500,
+0.0021972656250000, -0.0002441406250000, 0.0017089843750000, 0.0000305175781250,
+0.0021057128906250, -0.0005798339843750, 0.0016174316406250, 0.0000610351562500,
+0.0014648437500000, -0.0002746582031250, 0.0018005371093750, -0.0003356933593750,
+0.0017395019531250, 0.0000305175781250, 0.0015258789062500, -0.0004272460937500,
+0.0024414062500000, -0.0000610351562500, 0.0021972656250000, -0.0000305175781250,
+0.0025329589843750, -0.0001220703125000, 0.0033569335937500, 0.0001220703125000,
+0.0028381347656250, 0.0002746582031250, 0.0037536621093750, -0.0000915527343750,
+0.0036621093750000, 0.0002441406250000, 0.0037536621093750, 0.0002746582031250,
+0.0043029785156250, -0.0000305175781250, 0.0039672851562500, 0.0003356933593750,
+0.0043640136718750, 0.0001831054687500, 0.0042419433593750, 0.0002441406250000,
+0.0041809082031250, 0.0002441406250000, 0.0043945312500000, 0.0002136230468750,
+0.0041198730468750, 0.0003662109375000, 0.0041503906250000, -0.0000305175781250,
+0.0041198730468750, 0.0002136230468750, 0.0039672851562500, 0.0001220703125000,
+0.0038757324218750, -0.0001220703125000, 0.0038146972656250, 0.0003662109375000,
+0.0037536621093750, -0.0001525878906250, 0.0036621093750000, 0.0001525878906250,
+0.0037231445312500, 0.0003051757812500, 0.0037536621093750, 0.0000000000000000,
+0.0037536621093750, 0.0003967285156250, 0.0039062500000000, 0.0004272460937500,
+0.0039672851562500, 0.0004882812500000, 0.0040588378906250, 0.0007324218750000,
+0.0042419433593750, 0.0009155273437500, 0.0042114257812500, 0.0009155273437500,
+0.0043029785156250, 0.0011596679687500, 0.0043945312500000, 0.0012207031250000,
+0.0042724609375000, 0.0010986328125000, 0.0042114257812500, 0.0010681152343750,
+0.0042724609375000, 0.0011596679687500, 0.0038452148437500, 0.0008850097656250,
+0.0040283203125000, 0.0009460449218750, 0.0038146972656250, 0.0007934570312500,
+0.0034179687500000, 0.0004577636718750, 0.0036621093750000, 0.0005187988281250,
+0.0030517578125000, 0.0001525878906250, 0.0031127929687500, 0.0000000000000000,
+0.0030212402343750, -0.0000915527343750, 0.0025939941406250, -0.0003051757812500,
+0.0028076171875000, -0.0003051757812500, 0.0023803710937500, -0.0003356933593750,
+0.0024719238281250, -0.0003662109375000, 0.0025329589843750, -0.0003662109375000,
+0.0022888183593750, -0.0003051757812500, 0.0024414062500000, -0.0004272460937500,
+0.0024719238281250, -0.0003051757812500, 0.0025024414062500, -0.0003356933593750,
+0.0025634765625000, -0.0003356933593750, 0.0026855468750000, -0.0002746582031250,
+0.0027160644531250, -0.0003356933593750, 0.0027465820312500, -0.0003051757812500,
+0.0028381347656250, -0.0003356933593750, 0.0028686523437500, -0.0002136230468750,
+0.0027770996093750, -0.0002136230468750, 0.0028381347656250, -0.0004882812500000,
+0.0028381347656250, 0.0000305175781250, 0.0026550292968750, -0.0003051757812500,
+0.0027770996093750, -0.0001220703125000, 0.0028381347656250, 0.0003356933593750,
+0.0025939941406250, -0.0001525878906250, 0.0030212402343750, 0.0005798339843750,
+0.0028076171875000, 0.0005187988281250, 0.0027465820312500, 0.0004882812500000,
+0.0030212402343750, 0.0009460449218750, 0.0026855468750000, 0.0008544921875000,
+0.0027160644531250, 0.0010375976562500, 0.0026855468750000, 0.0013122558593750,
+0.0024414062500000, 0.0012512207031250, 0.0022888183593750, 0.0013427734375000,
+0.0023498535156250, 0.0016174316406250, 0.0019226074218750, 0.0012512207031250,
+0.0018920898437500, 0.0014648437500000, 0.0018310546875000, 0.0014038085937500,
+0.0016479492187500, 0.0010070800781250, 0.0016174316406250, 0.0012207031250000,
+0.0017700195312500, 0.0009460449218750, 0.0017089843750000, 0.0008239746093750,
+0.0019226074218750, 0.0007629394531250, 0.0021667480468750, 0.0006713867187500,
+0.0021667480468750, 0.0005493164062500, 0.0025634765625000, 0.0003662109375000,
+0.0026550292968750, 0.0003356933593750, 0.0026550292968750, 0.0001831054687500,
+0.0028686523437500, 0.0000305175781250, 0.0029296875000000, 0.0001220703125000,
+0.0026245117187500, -0.0000915527343750, 0.0026855468750000, -0.0000610351562500,
+0.0026550292968750, 0.0000305175781250, 0.0023193359375000, 0.0000610351562500,
+0.0023193359375000, 0.0000915527343750, 0.0025634765625000, 0.0001525878906250,
+0.0023803710937500, 0.0003662109375000, 0.0025329589843750, 0.0004577636718750,
+0.0027770996093750, 0.0005187988281250, 0.0025634765625000, 0.0007629394531250,
+0.0027770996093750, 0.0007629394531250, 0.0028991699218750, 0.0009765625000000,
+0.0025939941406250, 0.0010681152343750, 0.0025024414062500, 0.0011596679687500,
+0.0027465820312500, 0.0015258789062500, 0.0022888183593750, 0.0012207031250000,
+0.0024719238281250, 0.0015258789062500, 0.0025024414062500, 0.0014648437500000,
+0.0023193359375000, 0.0012512207031250, 0.0025634765625000, 0.0016479492187500,
+0.0023498535156250, 0.0012817382812500, 0.0026245117187500, 0.0015258789062500,
+0.0023498535156250, 0.0014038085937500, 0.0019836425781250, 0.0011901855468750,
+0.0025329589843750, 0.0016784667968750, 0.0017395019531250, 0.0010986328125000,
+0.0018005371093750, 0.0013427734375000, 0.0020446777343750, 0.0014343261718750,
+0.0013732910156250, 0.0010375976562500, 0.0007934570312500, 0.0012817382812500,
+0.0006713867187500, 0.0012512207031250, 0.0007629394531250, 0.0013122558593750,
+0.0004577636718750, 0.0014343261718750, 0.0002441406250000, 0.0015258789062500,
+0.0004882812500000, 0.0015869140625000, -0.0001831054687500, 0.0013122558593750,
+-0.0001831054687500, 0.0014038085937500, 0.0000915527343750, 0.0012207031250000,
+-0.0001220703125000, 0.0007934570312500, -0.0000305175781250, 0.0009460449218750,
+0.0006103515625000, 0.0008239746093750, 0.0004882812500000, 0.0005493164062500,
+0.0006713867187500, 0.0007324218750000, 0.0010986328125000, 0.0008239746093750,
+0.0007934570312500, 0.0004882812500000, 0.0008544921875000, 0.0008544921875000,
+0.0009155273437500, 0.0006713867187500, 0.0006713867187500, 0.0006408691406250,
+0.0004272460937500, 0.0008239746093750, 0.0004882812500000, 0.0006713867187500,
+0.0002136230468750, 0.0007629394531250, 0.0003051757812500, 0.0007934570312500,
+0.0002746582031250, 0.0007019042968750, 0.0000305175781250, 0.0007019042968750,
+0.0000000000000000, 0.0007324218750000, 0.0001525878906250, 0.0004577636718750,
+-0.0000915527343750, 0.0005493164062500, -0.0001220703125000, 0.0003967285156250,
+0.0000915527343750, 0.0001220703125000, -0.0000915527343750, 0.0003356933593750,
+-0.0003662109375000, 0.0000915527343750, -0.0001525878906250, -0.0000305175781250,
+-0.0004272460937500, 0.0002746582031250, -0.0008239746093750, 0.0004577636718750,
+-0.0004577636718750, 0.0002746582031250, -0.0008850097656250, 0.0009155273437500,
+-0.0010375976562500, 0.0008850097656250, -0.0007324218750000, 0.0011291503906250,
+-0.0007629394531250, 0.0015869140625000, -0.0009460449218750, 0.0014953613281250,
+0.0000000000000000, 0.0017089843750000, -0.0002746582031250, 0.0019226074218750,
+0.0000305175781250, 0.0020446777343750, 0.0009765625000000, 0.0020446777343750,
+0.0005187988281250, 0.0020751953125000, 0.0012207031250000, 0.0024414062500000,
+0.0014953613281250, 0.0021972656250000, 0.0013732910156250, 0.0024719238281250,
+0.0017395019531250, 0.0028381347656250, 0.0019836425781250, 0.0024108886718750,
+0.0014648437500000, 0.0029907226562500, 0.0019836425781250, 0.0028686523437500,
+0.0017395019531250, 0.0028076171875000, 0.0012207031250000, 0.0031738281250000,
+0.0020141601562500, 0.0028686523437500, 0.0008850097656250, 0.0029296875000000,
+0.0011596679687500, 0.0028686523437500, 0.0011596679687500, 0.0025939941406250,
+0.0004882812500000, 0.0024108886718750, 0.0008544921875000, 0.0022888183593750,
+0.0005187988281250, 0.0022277832031250, 0.0005187988281250, 0.0019226074218750,
+0.0004577636718750, 0.0020141601562500, 0.0003051757812500, 0.0020446777343750,
+0.0004272460937500, 0.0017089843750000, 0.0000610351562500, 0.0025024414062500,
+0.0002441406250000, 0.0021362304687500, 0.0002136230468750, 0.0025024414062500,
+-0.0000610351562500, 0.0032958984375000, 0.0003051757812500, 0.0028076171875000,
+0.0000000000000000, 0.0040588378906250, 0.0003051757812500, 0.0039062500000000,
+0.0002746582031250, 0.0041809082031250, 0.0000610351562500, 0.0051269531250000,
+0.0004272460937500, 0.0047302246093750, 0.0001220703125000, 0.0055541992187500,
+0.0002136230468750, 0.0057067871093750, 0.0002136230468750, 0.0055847167968750,
+0.0000000000000000, 0.0058593750000000, 0.0001220703125000, 0.0057983398437500,
+0.0000915527343750, 0.0054626464843750, 0.0000000000000000, 0.0055541992187500,
+0.0001525878906250, 0.0052795410156250, 0.0002136230468750, 0.0049133300781250,
+0.0001831054687500, 0.0049438476562500, 0.0004882812500000, 0.0044860839843750,
+0.0004577636718750, 0.0043334960937500, 0.0006713867187500, 0.0040893554687500,
+0.0008850097656250, 0.0037231445312500, 0.0008239746093750, 0.0037841796875000,
+0.0010681152343750, 0.0032958984375000, 0.0011901855468750, 0.0032348632812500,
+0.0011291503906250, 0.0031127929687500, 0.0011901855468750, 0.0026855468750000,
+0.0011901855468750, 0.0027465820312500, 0.0010375976562500, 0.0025939941406250,
+0.0009460449218750, 0.0024414062500000, 0.0007934570312500, 0.0025329589843750,
+0.0006713867187500, 0.0025329589843750, 0.0006408691406250, 0.0022888183593750,
+0.0005493164062500, 0.0026550292968750, 0.0004882812500000, 0.0023803710937500,
+0.0004272460937500, 0.0025024414062500, 0.0004577636718750, 0.0028076171875000,
+0.0004882812500000, 0.0023498535156250, 0.0004882812500000, 0.0029907226562500,
+0.0005798339843750, 0.0027770996093750, 0.0006103515625000, 0.0028686523437500,
+0.0007019042968750, 0.0033264160156250, 0.0008850097656250, 0.0030517578125000,
+0.0007324218750000, 0.0036621093750000, 0.0009460449218750, 0.0035400390625000,
+0.0009155273437500, 0.0037231445312500, 0.0007629394531250, 0.0041809082031250,
+0.0010681152343750, 0.0038146972656250, 0.0005798339843750, 0.0042724609375000,
+0.0008544921875000, 0.0041809082031250, 0.0006103515625000, 0.0042114257812500,
+0.0002746582031250, 0.0044250488281250, 0.0008239746093750, 0.0042419433593750,
+-0.0001831054687500, 0.0045471191406250, 0.0003662109375000, 0.0045166015625000,
+0.0003051757812500, 0.0044555664062500, -0.0003051757812500, 0.0046386718750000,
+0.0006408691406250, 0.0045166015625000, 0.0000610351562500, 0.0044860839843750,
+0.0002746582031250, 0.0045471191406250, 0.0007019042968750, 0.0043029785156250,
+0.0005187988281250, 0.0041503906250000, 0.0006103515625000, 0.0043945312500000,
+0.0009460449218750, 0.0036621093750000, 0.0004272460937500, 0.0039367675781250,
+0.0007019042968750, 0.0036926269531250, 0.0010681152343750, 0.0031127929687500,
+0.0002441406250000, 0.0035705566406250, 0.0012817382812500, 0.0028686523437500,
+0.0009460449218750, 0.0028991699218750, 0.0008544921875000, 0.0029602050781250,
+0.0016174316406250, 0.0026550292968750, 0.0012207031250000, 0.0028076171875000,
+0.0014953613281250},
+{0.0015258789062500, 0.0007324218750000, 0.0010986328125000, 0.0014648437500000,
+0.0010070800781250, 0.0013427734375000, 0.0010986328125000, 0.0016479492187500,
+0.0009765625000000, 0.0023193359375000, 0.0008544921875000, 0.0019836425781250,
+0.0007934570312500, 0.0030212402343750, 0.0009460449218750, 0.0029296875000000,
+0.0006103515625000, 0.0031738281250000, 0.0004272460937500, 0.0038757324218750,
+0.0008239746093750, 0.0033874511718750, 0.0000305175781250, 0.0042114257812500,
+0.0003051757812500, 0.0039672851562500, 0.0003662109375000, 0.0038452148437500,
+-0.0001525878906250, 0.0042419433593750, 0.0001831054687500, 0.0037231445312500,
+-0.0000610351562500, 0.0038146972656250, -0.0000610351562500, 0.0036010742187500,
+0.0000305175781250, 0.0034179687500000, -0.0000915527343750, 0.0033874511718750,
+0.0000610351562500, 0.0031738281250000, 0.0000305175781250, 0.0031738281250000,
+0.0000610351562500, 0.0030822753906250, 0.0002136230468750, 0.0030212402343750,
+0.0002746582031250, 0.0029907226562500, 0.0002136230468750, 0.0028991699218750,
+0.0005798339843750, 0.0029296875000000, 0.0003967285156250, 0.0028991699218750,
+0.0005187988281250, 0.0028686523437500, 0.0007629394531250, 0.0029602050781250,
+0.0003662109375000, 0.0028076171875000, 0.0007934570312500, 0.0029296875000000,
+0.0006408691406250, 0.0028381347656250, 0.0005187988281250, 0.0029296875000000,
+0.0006713867187500, 0.0030212402343750, 0.0004882812500000, 0.0028076171875000,
+0.0004882812500000, 0.0032043457031250, 0.0004882812500000, 0.0029907226562500,
+0.0003967285156250, 0.0029602050781250, 0.0003967285156250, 0.0031738281250000,
+0.0005187988281250, 0.0028991699218750, 0.0003967285156250, 0.0028686523437500,
+0.0004577636718750, 0.0028076171875000, 0.0005187988281250, 0.0026245117187500,
+0.0005493164062500, 0.0024719238281250, 0.0006408691406250, 0.0024414062500000,
+0.0006408691406250, 0.0022888183593750, 0.0008239746093750, 0.0021362304687500,
+0.0008239746093750, 0.0021362304687500, 0.0008239746093750, 0.0019836425781250,
+0.0010375976562500, 0.0018615722656250, 0.0007324218750000, 0.0020141601562500,
+0.0008544921875000, 0.0017700195312500, 0.0007324218750000, 0.0017395019531250,
+0.0006103515625000, 0.0019531250000000, 0.0008239746093750, 0.0017089843750000,
+0.0001220703125000, 0.0020751953125000, 0.0005798339843750, 0.0019531250000000,
+0.0002746582031250, 0.0019226074218750, -0.0002746582031250, 0.0021972656250000,
+0.0005798339843750, 0.0020446777343750, -0.0003662109375000, 0.0019836425781250,
+0.0001525878906250, 0.0021362304687500, 0.0003662109375000, 0.0018920898437500,
+-0.0002136230468750, 0.0017700195312500, 0.0003662109375000, 0.0020751953125000,
+0.0002441406250000, 0.0014038085937500, 0.0000305175781250, 0.0017395019531250,
+0.0004272460937500, 0.0015869140625000, 0.0004577636718750, 0.0010375976562500,
+-0.0001220703125000, 0.0015869140625000, 0.0009155273437500, 0.0009460449218750,
+0.0003051757812500, 0.0011596679687500, 0.0003662109375000, 0.0011901855468750,
+0.0011901855468750, 0.0009155273437500, 0.0004577636718750, 0.0014038085937500,
+0.0010375976562500, 0.0009765625000000, 0.0012207031250000, 0.0013122558593750,
+0.0009765625000000, 0.0012512207031250, 0.0012512207031250, 0.0009460449218750,
+0.0014648437500000, 0.0014343261718750, 0.0009765625000000, 0.0007019042968750,
+0.0012817382812500, 0.0009155273437500, 0.0012512207031250, 0.0009765625000000,
+0.0008544921875000, 0.0004272460937500, 0.0012207031250000, 0.0007019042968750,
+0.0006713867187500, 0.0009460449218750, 0.0008850097656250, 0.0006713867187500,
+0.0007019042968750, 0.0011596679687500, 0.0003356933593750, 0.0015869140625000,
+0.0008544921875000, 0.0011596679687500, 0.0000915527343750, 0.0023498535156250,
+0.0005187988281250, 0.0018920898437500, 0.0003967285156250, 0.0021972656250000,
+-0.0002136230468750, 0.0030517578125000, 0.0004577636718750, 0.0022888183593750,
+-0.0004577636718750, 0.0032043457031250, -0.0002136230468750, 0.0030517578125000,
+-0.0003356933593750, 0.0029907226562500, -0.0009155273437500, 0.0034790039062500,
+-0.0005798339843750, 0.0029907226562500, -0.0010986328125000, 0.0034790039062500,
+-0.0009460449218750, 0.0034179687500000, -0.0010986328125000, 0.0032958984375000,
+-0.0014038085937500, 0.0036315917968750, -0.0008544921875000, 0.0035400390625000,
+-0.0015869140625000, 0.0029907226562500, -0.0012512207031250, 0.0035705566406250,
+-0.0011291503906250, 0.0026245117187500, -0.0016174316406250, 0.0016479492187500,
+-0.0011596679687500, 0.0026245117187500, -0.0012817382812500, -0.0000610351562500,
+-0.0014038085937500, 0.0005187988281250, -0.0010681152343750, 0.0000915527343750,
+-0.0008850097656250, -0.0018005371093750, -0.0010070800781250, -0.0006408691406250,
+-0.0007324218750000, -0.0016174316406250, -0.0005798339843750, -0.0017089843750000,
+-0.0007324218750000, -0.0010681152343750, -0.0009460449218750, -0.0011291503906250,
+-0.0009765625000000, -0.0004272460937500, -0.0011901855468750, -0.0021972656250000,
+-0.0005187988281250, -0.0019531250000000, -0.0007934570312500, -0.0018615722656250,
+-0.0010070800781250, -0.0022888183593750, -0.0003356933593750, -0.0021667480468750,
+-0.0016479492187500, -0.0018920898437500, -0.0012512207031250, -0.0021972656250000,
+-0.0011901855468750, -0.0016479492187500, -0.0021057128906250, -0.0011596679687500,
+-0.0016479492187500, -0.0017089843750000, -0.0013732910156250, -0.0002746582031250,
+-0.0015869140625000, -0.0000915527343750, -0.0011291503906250, -0.0003356933593750,
+-0.0003967285156250, 0.0003967285156250, -0.0005187988281250, 0.0005493164062500,
+-0.0004577636718750, -0.0005187988281250, 0.0000915527343750, 0.0000915527343750,
+-0.0002746582031250, -0.0003051757812500, -0.0008544921875000, -0.0014038085937500,
+-0.0003051757812500, -0.0005187988281250, -0.0006408691406250, -0.0013427734375000,
+-0.0012207031250000, -0.0013732910156250, -0.0004272460937500, -0.0009765625000000,
+0.0000000000000000, -0.0010681152343750, -0.0008850097656250, -0.0009155273437500,
+0.0011291503906250, -0.0003662109375000, 0.0005493164062500, -0.0004272460937500,
+0.0007324218750000, 0.0000610351562500, 0.0021972656250000, 0.0006408691406250,
+0.0012817382812500, 0.0003356933593750, 0.0021972656250000, 0.0015258789062500,
+0.0024108886718750, 0.0015563964843750, 0.0021667480468750, 0.0015258789062500,
+0.0022888183593750, 0.0021362304687500, 0.0022583007812500, 0.0019836425781250,
+0.0021057128906250, 0.0013427734375000, 0.0021057128906250, 0.0015563964843750,
+0.0018615722656250, 0.0012207031250000, 0.0015258789062500, 0.0003967285156250,
+0.0015563964843750, 0.0006408691406250, 0.0009765625000000, 0.0001831054687500,
+0.0007324218750000, -0.0000610351562500, 0.0005187988281250, -0.0000915527343750,
+0.0002136230468750, -0.0001831054687500, 0.0000610351562500, -0.0002441406250000,
+-0.0001220703125000, -0.0003051757812500, -0.0002441406250000, -0.0002746582031250,
+-0.0002136230468750, -0.0002441406250000, -0.0002441406250000, -0.0003051757812500,
+-0.0001525878906250, -0.0002441406250000, 0.0002441406250000, -0.0000305175781250,
+0.0003662109375000, 0.0000000000000000, 0.0004882812500000, 0.0000610351562500,
+0.0008850097656250, 0.0001831054687500, 0.0010986328125000, 0.0001525878906250,
+0.0007324218750000, 0.0001831054687500, 0.0012207031250000, -0.0000305175781250,
+0.0008239746093750, 0.0000915527343750, 0.0001831054687500, 0.0001831054687500,
+0.0007629394531250, -0.0002441406250000, -0.0005493164062500, 0.0003967285156250,
+-0.0003356933593750, 0.0001831054687500, -0.0005187988281250, 0.0000000000000000,
+-0.0015258789062500, 0.0003662109375000, -0.0010070800781250, 0.0001831054687500,
+-0.0015563964843750, -0.0000915527343750, -0.0016174316406250, 0.0001831054687500,
+-0.0013122558593750, -0.0000610351562500, -0.0012817382812500, -0.0003662109375000,
+-0.0012817382812500, 0.0001525878906250, -0.0005798339843750, -0.0005187988281250,
+-0.0006408691406250, -0.0001525878906250, -0.0000915527343750, -0.0001525878906250,
+0.0006408691406250, -0.0005798339843750, 0.0004882812500000, 0.0000305175781250,
+0.0017395019531250, -0.0003051757812500, 0.0018005371093750, -0.0000610351562500,
+0.0022277832031250, 0.0001220703125000, 0.0031738281250000, 0.0000915527343750,
+0.0028991699218750, 0.0003662109375000, 0.0038146972656250, 0.0002136230468750,
+0.0039672851562500, 0.0004882812500000, 0.0041198730468750, 0.0004577636718750,
+0.0046081542968750, 0.0002746582031250, 0.0045166015625000, 0.0006408691406250,
+0.0048828125000000, 0.0003051757812500, 0.0049743652343750, 0.0004577636718750,
+0.0049438476562500, 0.0004272460937500, 0.0051269531250000, 0.0001220703125000,
+0.0052185058593750, 0.0002746582031250, 0.0049743652343750, 0.0001525878906250,
+0.0051879882812500, 0.0000610351562500, 0.0049438476562500, 0.0001220703125000,
+0.0046691894531250, 0.0001220703125000, 0.0050048828125000, 0.0000000000000000,
+0.0040588378906250, 0.0000915527343750, 0.0043640136718750, 0.0000915527343750,
+0.0039367675781250, 0.0000915527343750, 0.0030822753906250, 0.0001525878906250,
+0.0036621093750000, 0.0001220703125000, 0.0022888183593750, 0.0001525878906250,
+0.0025329589843750, 0.0000915527343750, 0.0021057128906250, 0.0000915527343750,
+0.0010986328125000, 0.0001220703125000, 0.0017700195312500, 0.0000915527343750,
+0.0003356933593750, 0.0001220703125000, 0.0004882812500000, 0.0001220703125000,
+0.0001525878906250, 0.0001220703125000, -0.0007629394531250, 0.0000610351562500,
+-0.0001525878906250, 0.0000915527343750, -0.0013732910156250, 0.0000610351562500,
+-0.0011901855468750, 0.0000000000000000, -0.0013122558593750, 0.0000915527343750,
+-0.0020141601562500, 0.0001831054687500, -0.0016784667968750, 0.0000000000000000,
+-0.0023193359375000, 0.0003356933593750, -0.0023803710937500, 0.0001831054687500,
+-0.0022583007812500, 0.0003051757812500, -0.0025024414062500, 0.0005798339843750,
+-0.0024719238281250, 0.0003356933593750, -0.0022583007812500, 0.0007324218750000,
+-0.0022583007812500, 0.0007324218750000, -0.0021057128906250, 0.0008239746093750,
+-0.0017395019531250, 0.0011901855468750, -0.0018310546875000, 0.0011596679687500,
+-0.0012817382812500, 0.0014038085937500, -0.0012817382812500, 0.0015258789062500,
+-0.0011901855468750, 0.0014953613281250, -0.0007934570312500, 0.0015869140625000,
+-0.0008850097656250, 0.0016174316406250, -0.0008239746093750, 0.0015869140625000,
+-0.0007324218750000, 0.0015563964843750, -0.0010681152343750, 0.0014343261718750,
+-0.0013732910156250, 0.0013122558593750, -0.0011901855468750, 0.0012512207031250,
+-0.0017700195312500, 0.0010375976562500, -0.0017395019531250, 0.0010070800781250,
+-0.0017700195312500, 0.0008239746093750, -0.0020751953125000, 0.0006408691406250,
+-0.0018920898437500, 0.0006103515625000, -0.0019226074218750, 0.0003356933593750,
+-0.0018615722656250, 0.0002746582031250, -0.0018310546875000, 0.0001831054687500,
+-0.0018005371093750, 0.0000305175781250, -0.0015869140625000, 0.0001525878906250,
+-0.0016479492187500, -0.0001831054687500, -0.0014343261718750, 0.0000915527343750,
+-0.0013122558593750, -0.0000610351562500, -0.0013732910156250, -0.0003967285156250,
+-0.0010070800781250, 0.0000000000000000, -0.0010070800781250, -0.0004272460937500,
+-0.0007934570312500, -0.0003356933593750, -0.0004272460937500, -0.0002441406250000,
+-0.0002136230468750, -0.0004272460937500, 0.0000305175781250, -0.0001525878906250,
+0.0005187988281250, -0.0003356933593750, 0.0007019042968750, -0.0001525878906250,
+0.0009460449218750},
+{0.0025939941406250, 0.0000305175781250, 0.0021362304687500, 0.0001831054687500,
+0.0013732910156250, -0.0000915527343750, 0.0020751953125000, 0.0003051757812500,
+0.0006408691406250, 0.0002746582031250, 0.0009155273437500, 0.0003662109375000,
+0.0006408691406250, 0.0003967285156250, -0.0003662109375000, 0.0003662109375000,
+0.0003051757812500, 0.0005187988281250, -0.0007324218750000, 0.0001831054687500,
+-0.0005798339843750, 0.0003356933593750, -0.0007019042968750, 0.0001831054687500,
+-0.0013732910156250, -0.0001831054687500, -0.0008544921875000, 0.0000000000000000,
+-0.0014343261718750, -0.0002746582031250, -0.0013427734375000, -0.0003356933593750,
+-0.0011901855468750, -0.0002136230468750, -0.0014648437500000, -0.0002746582031250,
+-0.0011291503906250, -0.0003662109375000, -0.0008850097656250, -0.0001525878906250,
+-0.0009155273437500, -0.0002441406250000, -0.0003967285156250, -0.0001220703125000,
+0.0000305175781250, 0.0001525878906250, -0.0001831054687500, -0.0000305175781250,
+0.0009460449218750, 0.0001831054687500, 0.0007324218750000, 0.0002441406250000,
+0.0011596679687500, 0.0001525878906250, 0.0020751953125000, 0.0002441406250000,
+0.0015869140625000, 0.0003356933593750, 0.0029602050781250, 0.0002441406250000,
+0.0027160644531250, 0.0002441406250000, 0.0031127929687500, 0.0003051757812500,
+0.0040283203125000, 0.0002136230468750, 0.0034179687500000, 0.0002441406250000,
+0.0046081542968750, 0.0003967285156250, 0.0045166015625000, 0.0002746582031250,
+0.0046691894531250, 0.0003967285156250, 0.0054321289062500, 0.0006103515625000,
+0.0049743652343750, 0.0003356933593750, 0.0053405761718750, 0.0009155273437500,
+0.0054626464843750, 0.0007629394531250, 0.0052490234375000, 0.0008850097656250,
+0.0052490234375000, 0.0013122558593750, 0.0053100585937500, 0.0011596679687500,
+0.0047302246093750, 0.0014648437500000, 0.0049133300781250, 0.0016174316406250,
+0.0046997070312500, 0.0016174316406250, 0.0041809082031250, 0.0017089843750000,
+0.0045776367187500, 0.0017700195312500, 0.0040588378906250, 0.0016174316406250,
+0.0039672851562500, 0.0016479492187500, 0.0042419433593750, 0.0016479492187500,
+0.0040893554687500, 0.0014343261718750, 0.0039367675781250, 0.0014343261718750,
+0.0045776367187500, 0.0012817382812500, 0.0043640136718750, 0.0012207031250000,
+0.0045166015625000, 0.0010375976562500, 0.0050048828125000, 0.0008544921875000,
+0.0046691894531250, 0.0008850097656250, 0.0049438476562500, 0.0005798339843750,
+0.0050659179687500, 0.0006713867187500, 0.0046081542968750, 0.0005187988281250,
+0.0044860839843750, 0.0003051757812500, 0.0045471191406250, 0.0005187988281250,
+0.0034179687500000, 0.0001220703125000, 0.0035705566406250, 0.0003967285156250,
+0.0030822753906250, 0.0002746582031250, 0.0021667480468750, -0.0000610351562500,
+0.0024414062500000, 0.0003356933593750, 0.0011596679687500, -0.0002136230468750,
+0.0012512207031250, -0.0001525878906250, 0.0008544921875000, -0.0000915527343750,
+-0.0000610351562500, -0.0004577636718750, 0.0004577636718750, -0.0003051757812500,
+-0.0005493164062500, -0.0003662109375000, -0.0003967285156250, -0.0003662109375000,
+-0.0005493164062500, -0.0004577636718750, -0.0011901855468750, -0.0005187988281250,
+-0.0006408691406250, -0.0004577636718750, -0.0014953613281250, -0.0004882812500000,
+-0.0012817382812500, -0.0006103515625000, -0.0011901855468750, -0.0004577636718750,
+-0.0016174316406250, -0.0002441406250000, -0.0013427734375000, -0.0004272460937500,
+-0.0011596679687500, 0.0000305175781250, -0.0012817382812500, 0.0000305175781250,
+-0.0009155273437500, 0.0000610351562500, -0.0005493164062500, 0.0003967285156250,
+-0.0007629394531250, 0.0003356933593750, -0.0002746582031250, 0.0003662109375000,
+-0.0001525878906250, 0.0004882812500000, -0.0001831054687500, 0.0004272460937500,
+-0.0000305175781250, 0.0003967285156250, 0.0000915527343750, 0.0004272460937500,
+0.0001525878906250, 0.0003356933593750, 0.0000610351562500, 0.0003356933593750,
+0.0003356933593750, 0.0001831054687500, 0.0006408691406250, -0.0000305175781250,
+0.0004882812500000, 0.0000305175781250, 0.0009155273437500, -0.0002136230468750,
+0.0009460449218750, -0.0004577636718750, 0.0009765625000000, -0.0003356933593750,
+0.0012207031250000, -0.0001831054687500, 0.0011901855468750, -0.0003051757812500,
+0.0010070800781250, -0.0000610351562500, 0.0010070800781250, -0.0002136230468750,
+0.0011291503906250, -0.0001831054687500, 0.0010375976562500, -0.0000305175781250,
+0.0008544921875000, -0.0002441406250000, 0.0009765625000000, 0.0001525878906250,
+0.0008544921875000, -0.0002136230468750, 0.0005798339843750, -0.0000610351562500,
+0.0005493164062500, 0.0003662109375000, 0.0006103515625000, -0.0002136230468750,
+0.0002441406250000, 0.0004882812500000, 0.0003967285156250, 0.0003662109375000,
+0.0003662109375000, 0.0002136230468750, 0.0000915527343750, 0.0004882812500000,
+0.0004577636718750, 0.0002136230468750, 0.0000305175781250, 0.0003356933593750,
+0.0002441406250000, 0.0002441406250000, -0.0000610351562500, 0.0002746582031250,
+-0.0005798339843750, 0.0004577636718750, -0.0001525878906250, 0.0003662109375000,
+-0.0011596679687500, 0.0007324218750000, -0.0009155273437500, 0.0007019042968750,
+-0.0010375976562500, 0.0006408691406250, -0.0017395019531250, 0.0007629394531250,
+-0.0013732910156250, 0.0008850097656250, -0.0014038085937500, 0.0003967285156250,
+-0.0017395019531250, 0.0007019042968750, -0.0011901855468750, 0.0004577636718750,
+-0.0007934570312500, -0.0000610351562500, -0.0012512207031250, 0.0003356933593750,
+0.0001525878906250, -0.0002746582031250, -0.0000915527343750, -0.0001831054687500,
+0.0000915527343750, -0.0002441406250000, 0.0011596679687500, -0.0006103515625000,
+0.0005187988281250, -0.0003967285156250, 0.0007629394531250, -0.0007934570312500,
+0.0010681152343750, -0.0007934570312500, 0.0006103515625000, -0.0008544921875000,
+0.0002441406250000, -0.0011291503906250, 0.0006713867187500, -0.0010986328125000,
+-0.0000305175781250, -0.0012207031250000, 0.0000000000000000, -0.0014038085937500,
+0.0001831054687500, -0.0013427734375000, 0.0000305175781250, -0.0013427734375000,
+0.0001831054687500, -0.0014953613281250, 0.0002746582031250, -0.0013122558593750,
+0.0003662109375000, -0.0012817382812500, 0.0005493164062500, -0.0012207031250000,
+0.0006408691406250, -0.0010375976562500, 0.0007324218750000, -0.0009765625000000,
+0.0008850097656250, -0.0006408691406250, 0.0008239746093750, -0.0006408691406250,
+0.0009460449218750, -0.0001831054687500, 0.0010986328125000, 0.0002136230468750,
+0.0010681152343750, -0.0000305175781250, 0.0013122558593750, 0.0007934570312500,
+0.0013732910156250, 0.0007324218750000, 0.0013732910156250, 0.0008239746093750,
+0.0015258789062500, 0.0012817382812500, 0.0016174316406250, 0.0009765625000000,
+0.0014343261718750, 0.0011901855468750, 0.0016784667968750, 0.0012207031250000,
+0.0015869140625000, 0.0011901855468750, 0.0013427734375000, 0.0012512207031250,
+0.0016479492187500, 0.0012207031250000, 0.0012207031250000, 0.0013427734375000,
+0.0011901855468750, 0.0012207031250000, 0.0011901855468750, 0.0011596679687500,
+0.0008544921875000, 0.0011291503906250, 0.0007629394531250, 0.0009460449218750,
+0.0007934570312500, 0.0010375976562500, 0.0005187988281250, 0.0007324218750000,
+0.0004577636718750, 0.0007324218750000, 0.0006408691406250, 0.0008239746093750,
+0.0004272460937500, 0.0004272460937500, 0.0005798339843750, 0.0006408691406250,
+0.0007019042968750, 0.0006103515625000, 0.0006103515625000, 0.0003662109375000,
+0.0005493164062500, 0.0003662109375000, 0.0006713867187500, 0.0002136230468750,
+0.0003967285156250, 0.0001220703125000, 0.0005493164062500, -0.0000305175781250,
+0.0004272460937500, -0.0000915527343750, 0.0001220703125000, -0.0000915527343750,
+0.0003051757812500, -0.0002136230468750, 0.0000305175781250, -0.0000915527343750,
+-0.0000305175781250, -0.0002136230468750, -0.0000610351562500, -0.0000305175781250,
+-0.0001831054687500, 0.0002136230468750, -0.0002136230468750, 0.0000000000000000,
+-0.0001831054687500, 0.0006713867187500, -0.0003051757812500, 0.0004882812500000,
+-0.0001831054687500, 0.0008239746093750, -0.0001220703125000, 0.0013732910156250,
+-0.0001831054687500, 0.0010375976562500, 0.0001831054687500, 0.0021057128906250,
+0.0001220703125000, 0.0019226074218750, 0.0002136230468750, 0.0023498535156250,
+0.0004577636718750, 0.0033569335937500, 0.0004272460937500, 0.0028686523437500,
+0.0005493164062500, 0.0041503906250000, 0.0006713867187500, 0.0040588378906250,
+0.0006408691406250, 0.0043945312500000, 0.0006408691406250, 0.0052795410156250,
+0.0006713867187500, 0.0047607421875000, 0.0004882812500000, 0.0057678222656250,
+0.0006103515625000, 0.0056152343750000, 0.0004272460937500, 0.0057678222656250,
+0.0002136230468750, 0.0064086914062500, 0.0005187988281250, 0.0060119628906250,
+-0.0000915527343750, 0.0065917968750000, 0.0001525878906250, 0.0065612792968750,
+0.0001220703125000, 0.0065612792968750, -0.0002136230468750, 0.0069580078125000,
+0.0003662109375000, 0.0068359375000000, -0.0000305175781250, 0.0068359375000000,
+0.0002746582031250, 0.0070800781250000, 0.0005493164062500, 0.0067749023437500,
+0.0003356933593750, 0.0065917968750000, 0.0005187988281250, 0.0069580078125000,
+0.0008239746093750, 0.0060729980468750, 0.0004577636718750, 0.0064086914062500,
+0.0007324218750000, 0.0062561035156250, 0.0009460449218750, 0.0055236816406250,
+0.0001831054687500, 0.0060424804687500, 0.0009155273437500, 0.0054931640625000,
+0.0005187988281250, 0.0054931640625000, 0.0003356933593750, 0.0057373046875000,
+0.0008239746093750, 0.0056457519531250, 0.0002746582031250, 0.0057067871093750,
+0.0004882812500000, 0.0060729980468750, 0.0005798339843750, 0.0062255859375000,
+0.0003051757812500, 0.0062866210937500, 0.0003356933593750, 0.0065917968750000,
+0.0005798339843750, 0.0067443847656250, 0.0000610351562500, 0.0062866210937500,
+0.0002441406250000, 0.0068054199218750, 0.0002441406250000, 0.0061645507812500,
+-0.0001220703125000, 0.0054626464843750, 0.0002136230468750, 0.0062255859375000,
+-0.0000915527343750, 0.0042114257812500, 0.0001525878906250, 0.0046081542968750,
+0.0001220703125000, 0.0042114257812500, -0.0000610351562500, 0.0028381347656250,
+0.0003967285156250, 0.0036010742187500, -0.0002746582031250, 0.0021972656250000,
+0.0001525878906250, 0.0022888183593750, 0.0000915527343750, 0.0019531250000000,
+-0.0003356933593750, 0.0010681152343750, 0.0003051757812500, 0.0016784667968750,
+-0.0003051757812500, 0.0001831054687500, 0.0000305175781250, 0.0004882812500000,
+0.0000000000000000, 0.0000000000000000, -0.0004577636718750, -0.0011291503906250,
+0.0001220703125000, -0.0003662109375000, -0.0007629394531250, -0.0020751953125000,
+-0.0004272460937500, -0.0020446777343750, -0.0005187988281250, -0.0021667480468750,
+-0.0011901855468750, -0.0031433105468750, -0.0006713867187500, -0.0027160644531250,
+-0.0010681152343750, -0.0029602050781250, -0.0012207031250000, -0.0031127929687500,
+-0.0009765625000000, -0.0028686523437500, -0.0010070800781250, -0.0026245117187500,
+-0.0012512207031250, -0.0026550292968750, -0.0008239746093750, -0.0021667480468750,
+-0.0011901855468750, -0.0020446777343750, -0.0011901855468750, -0.0018920898437500,
+-0.0009765625000000, -0.0015869140625000, -0.0014953613281250, -0.0014038085937500,
+-0.0011596679687500, -0.0008239746093750, -0.0013427734375000, -0.0009460449218750,
+-0.0013427734375000, -0.0001525878906250, -0.0010375976562500, 0.0006103515625000,
+-0.0011291503906250},
+{-0.0003662109375000, 0.0025634765625000, -0.0003662109375000, 0.0027160644531250,
+-0.0000610351562500, 0.0033264160156250, -0.0000305175781250, 0.0032653808593750,
+0.0000915527343750, 0.0037536621093750, 0.0003051757812500, 0.0042724609375000,
+0.0002746582031250, 0.0039978027343750, 0.0004577636718750, 0.0051879882812500,
+0.0005798339843750, 0.0050354003906250, 0.0004882812500000, 0.0053405761718750,
+0.0004577636718750, 0.0061950683593750, 0.0005798339843750, 0.0057373046875000,
+0.0000610351562500, 0.0065307617187500, 0.0004272460937500, 0.0065307617187500,
+0.0001831054687500, 0.0064086914062500, -0.0002746582031250, 0.0066528320312500,
+0.0004272460937500, 0.0064697265625000, -0.0003967285156250, 0.0063476562500000,
+0.0000000000000000, 0.0062255859375000, 0.0001220703125000, 0.0059204101562500,
+-0.0003967285156250, 0.0056457519531250, 0.0001525878906250, 0.0056152343750000,
+0.0001220703125000, 0.0052795410156250, -0.0000915527343750, 0.0050048828125000,
+0.0003356933593750, 0.0049438476562500, 0.0004577636718750, 0.0047912597656250,
+-0.0000915527343750, 0.0046386718750000, 0.0008239746093750, 0.0045471191406250,
+0.0002441406250000, 0.0043640136718750, 0.0003356933593750, 0.0042419433593750,
+0.0009765625000000, 0.0043029785156250, 0.0000305175781250, 0.0041809082031250,
+0.0010070800781250, 0.0037536621093750, 0.0007629394531250, 0.0038757324218750,
+0.0005187988281250, 0.0033874511718750, 0.0010681152343750, 0.0028076171875000,
+0.0007934570312500, 0.0031738281250000, 0.0006408691406250, 0.0024414062500000,
+0.0007934570312500, 0.0023193359375000, 0.0006408691406250, 0.0026245117187500,
+0.0003356933593750, 0.0023803710937500, 0.0003967285156250, 0.0023193359375000,
+0.0001831054687500, 0.0032653808593750, 0.0001220703125000, 0.0027770996093750,
+0.0000305175781250, 0.0034790039062500, -0.0000610351562500, 0.0046386718750000,
+0.0000610351562500, 0.0037841796875000, -0.0002746582031250, 0.0053710937500000,
+0.0000915527343750, 0.0053100585937500, -0.0000915527343750, 0.0052185058593750,
+-0.0003356933593750, 0.0059814453125000, 0.0003051757812500, 0.0055541992187500,
+-0.0004272460937500, 0.0054321289062500, -0.0000305175781250, 0.0055541992187500,
+-0.0000305175781250, 0.0052490234375000, -0.0004272460937500, 0.0049438476562500,
+0.0001525878906250, 0.0051269531250000, -0.0005187988281250, 0.0046691894531250,
+-0.0002746582031250, 0.0046997070312500, -0.0003051757812500, 0.0045166015625000,
+-0.0007629394531250, 0.0042114257812500, -0.0003967285156250, 0.0044250488281250,
+-0.0006408691406250, 0.0039367675781250, -0.0007019042968750, 0.0041809082031250,
+-0.0003051757812500, 0.0035400390625000, -0.0002441406250000, 0.0028381347656250,
+-0.0003662109375000, 0.0033874511718750, -0.0000610351562500, 0.0015869140625000,
+-0.0003662109375000, 0.0019226074218750, -0.0003356933593750, 0.0013122558593750,
+-0.0002441406250000, -0.0001525878906250, -0.0007629394531250, 0.0007019042968750,
+-0.0001831054687500, -0.0011596679687500, -0.0007629394531250, -0.0010681152343750,
+-0.0008544921875000, -0.0011901855468750, -0.0003662109375000, -0.0021362304687500,
+-0.0010681152343750, -0.0015869140625000, -0.0007019042968750, -0.0022583007812500,
+-0.0007324218750000, -0.0019836425781250, -0.0008239746093750, -0.0022277832031250,
+-0.0005798339843750, -0.0028991699218750, -0.0005798339843750, -0.0022583007812500,
+-0.0006713867187500, -0.0029296875000000, -0.0006408691406250, -0.0032043457031250,
+-0.0006103515625000, -0.0024108886718750, -0.0007324218750000, -0.0016479492187500,
+0.0003356933593750, -0.0010681152343750, 0.0002746582031250, -0.0032958984375000,
+0.0005187988281250, -0.0034484863281250, 0.0005187988281250, -0.0027160644531250,
+0.0002441406250000, -0.0028076171875000, 0.0004577636718750, -0.0034484863281250,
+0.0009155273437500, -0.0012512207031250, 0.0007934570312500, -0.0018310546875000,
+0.0012207031250000, -0.0014648437500000, 0.0021057128906250, 0.0001525878906250,
+0.0019531250000000, -0.0008850097656250, 0.0020751953125000, 0.0004272460937500,
+0.0027770996093750, 0.0003356933593750, 0.0021362304687500, 0.0003051757812500,
+0.0013732910156250, 0.0011291503906250, 0.0022277832031250, 0.0007324218750000,
+0.0003051757812500, 0.0008850097656250, 0.0003662109375000, 0.0013427734375000,
+0.0001831054687500, 0.0005187988281250, -0.0010375976562500, 0.0000000000000000,
+-0.0006103515625000, 0.0006713867187500, -0.0011291503906250, -0.0011901855468750,
+-0.0010375976562500, -0.0010070800781250, -0.0012207031250000, -0.0010986328125000,
+-0.0015563964843750, -0.0022277832031250, -0.0010375976562500, -0.0016479492187500,
+-0.0015563964843750, -0.0017700195312500, -0.0015563964843750, -0.0018615722656250,
+-0.0010375976562500, -0.0014953613281250, -0.0010681152343750, -0.0011596679687500,
+-0.0010070800781250, -0.0011596679687500, 0.0001831054687500, -0.0006103515625000,
+-0.0000610351562500, -0.0003662109375000, 0.0005493164062500, -0.0002746582031250,
+0.0017395019531250, 0.0000000000000000, 0.0011291503906250, 0.0000610351562500,
+0.0018920898437500, -0.0002746582031250, 0.0023498535156250, 0.0000000000000000,
+0.0017395019531250, -0.0003051757812500, 0.0015869140625000, -0.0008544921875000,
+0.0019836425781250, -0.0004882812500000, 0.0007629394531250, -0.0012817382812500,
+0.0008544921875000, -0.0013122558593750, 0.0011291503906250, -0.0010070800781250,
+0.0004272460937500, -0.0011596679687500, 0.0006103515625000, -0.0011596679687500,
+0.0018920898437500, -0.0004272460937500, 0.0009765625000000, -0.0005187988281250,
+0.0022277832031250, -0.0002441406250000, 0.0039367675781250, 0.0003967285156250,
+0.0023803710937500, 0.0001525878906250, 0.0058288574218750, 0.0006103515625000,
+0.0051879882812500, 0.0006103515625000, 0.0055847167968750, 0.0005187988281250,
+0.0077514648437500, 0.0007629394531250, 0.0063171386718750, 0.0007324218750000,
+0.0077514648437500, 0.0002441406250000, 0.0077819824218750, 0.0005187988281250,
+0.0071716308593750, 0.0002441406250000, 0.0075378417968750, -0.0002441406250000,
+0.0073852539062500, 0.0001831054687500, 0.0063781738281250, -0.0003356933593750,
+0.0067138671875000, -0.0003356933593750, 0.0062255859375000, -0.0001525878906250,
+0.0053100585937500, -0.0003356933593750, 0.0057983398437500, -0.0003356933593750,
+0.0047912597656250, -0.0002136230468750, 0.0048828125000000, -0.0003967285156250,
+0.0047607421875000, -0.0003051757812500, 0.0041809082031250, -0.0001220703125000,
+0.0045166015625000, -0.0004272460937500, 0.0039367675781250, 0.0000610351562500,
+0.0039367675781250, -0.0000915527343750, 0.0041809082031250, -0.0000610351562500,
+0.0038757324218750, 0.0002441406250000, 0.0039062500000000, 0.0000305175781250,
+0.0043945312500000, 0.0001220703125000, 0.0041198730468750, 0.0003051757812500,
+0.0043945312500000, 0.0002136230468750, 0.0049133300781250, 0.0001220703125000,
+0.0044860839843750, 0.0003356933593750, 0.0055541992187500, 0.0000915527343750,
+0.0052490234375000, 0.0002441406250000, 0.0056457519531250, 0.0001525878906250,
+0.0065612792968750, -0.0000610351562500, 0.0060119628906250, 0.0001525878906250,
+0.0071716308593750, -0.0000610351562500, 0.0071105957031250, -0.0000305175781250,
+0.0071105957031250, -0.0000915527343750, 0.0077209472656250, -0.0001525878906250,
+0.0074157714843750, -0.0001220703125000, 0.0073547363281250, -0.0002136230468750,
+0.0075988769531250, -0.0001831054687500, 0.0070800781250000, -0.0001525878906250,
+0.0066833496093750, -0.0002136230468750, 0.0068664550781250, -0.0002441406250000,
+0.0054626464843750, -0.0001831054687500, 0.0057373046875000, -0.0002441406250000,
+0.0050659179687500, -0.0002136230468750, 0.0037841796875000, -0.0000915527343750,
+0.0043945312500000, -0.0002441406250000, 0.0028991699218750, 0.0001220703125000,
+0.0028076171875000, 0.0000000000000000, 0.0026245117187500, 0.0000915527343750,
+0.0017700195312500, 0.0004272460937500, 0.0020141601562500, 0.0002136230468750,
+0.0015869140625000, 0.0006103515625000, 0.0013427734375000, 0.0005798339843750,
+0.0014953613281250, 0.0007019042968750, 0.0014953613281250, 0.0010375976562500,
+0.0013732910156250, 0.0009155273437500, 0.0017700195312500, 0.0012512207031250,
+0.0017395019531250, 0.0013427734375000, 0.0018615722656250, 0.0013122558593750,
+0.0022277832031250, 0.0014648437500000, 0.0021667480468750, 0.0014648437500000,
+0.0022583007812500, 0.0014648437500000, 0.0024108886718750, 0.0015869140625000,
+0.0023193359375000, 0.0014648437500000, 0.0021667480468750, 0.0013427734375000,
+0.0022888183593750, 0.0014038085937500, 0.0020141601562500, 0.0012207031250000,
+0.0018920898437500, 0.0012207031250000, 0.0018615722656250, 0.0011291503906250,
+0.0017395019531250, 0.0009155273437500, 0.0016479492187500, 0.0010375976562500,
+0.0015563964843750, 0.0007324218750000, 0.0014343261718750, 0.0007324218750000,
+0.0013122558593750, 0.0007019042968750, 0.0012207031250000, 0.0004882812500000,
+0.0011596679687500, 0.0006103515625000, 0.0008544921875000, 0.0004577636718750,
+0.0009155273437500, 0.0006103515625000, 0.0005493164062500, 0.0005187988281250,
+0.0001831054687500, 0.0004272460937500, 0.0003967285156250, 0.0007324218750000,
+-0.0003967285156250, 0.0002441406250000, -0.0003051757812500, 0.0005493164062500,
+-0.0003356933593750, 0.0004577636718750, -0.0008544921875000, 0.0000305175781250,
+-0.0007324218750000, 0.0004272460937500, -0.0006408691406250, -0.0000610351562500,
+-0.0007934570312500, 0.0000000000000000, -0.0005187988281250, -0.0000305175781250,
+-0.0002441406250000, -0.0002441406250000, -0.0004272460937500, 0.0000000000000000,
+0.0002746582031250, -0.0002136230468750, 0.0002746582031250, -0.0000610351562500,
+0.0005798339843750, 0.0000000000000000, 0.0011901855468750, -0.0000915527343750,
+0.0010070800781250, 0.0000915527343750, 0.0016479492187500, 0.0001525878906250,
+0.0018310546875000, 0.0002136230468750, 0.0018615722656250, 0.0002441406250000,
+0.0021667480468750, 0.0003051757812500, 0.0023498535156250, 0.0004272460937500,
+0.0028686523437500, 0.0002441406250000, 0.0028991699218750, 0.0003662109375000,
+0.0031738281250000, 0.0002746582031250, 0.0036010742187500, 0.0001220703125000,
+0.0034484863281250, 0.0003356933593750, 0.0038452148437500, -0.0002441406250000,
+0.0039978027343750, 0.0000000000000000, 0.0039367675781250, -0.0002441406250000,
+0.0039367675781250, -0.0007019042968750, 0.0038146972656250, -0.0003356933593750,
+0.0034790039062500, -0.0008544921875000, 0.0032043457031250, -0.0009765625000000,
+0.0028686523437500, -0.0007629394531250, 0.0025329589843750, -0.0009155273437500,
+0.0022277832031250, -0.0010681152343750, 0.0016174316406250, -0.0005187988281250,
+0.0016174316406250, -0.0007019042968750, 0.0009155273437500, -0.0007019042968750,
+0.0002441406250000, -0.0003356933593750, 0.0006713867187500, -0.0005493164062500,
+-0.0008239746093750, -0.0004882812500000, -0.0007019042968750, -0.0005187988281250,
+-0.0009460449218750, -0.0004577636718750, -0.0019226074218750, -0.0006103515625000,
+-0.0014343261718750, -0.0008239746093750, -0.0015563964843750, -0.0003051757812500,
+-0.0016174316406250, -0.0006408691406250, -0.0013732910156250, -0.0007629394531250,
+-0.0011596679687500, -0.0004577636718750, -0.0010681152343750, -0.0007019042968750,
+-0.0005493164062500, -0.0005798339843750, -0.0003662109375000, -0.0005493164062500,
+0.0000000000000000, -0.0006713867187500, 0.0002746582031250, -0.0006103515625000,
+0.0000915527343750, -0.0005187988281250, 0.0013122558593750, -0.0008239746093750,
+0.0009460449218750},
+{0.0023803710937500, 0.0014343261718750, 0.0021972656250000, 0.0015869140625000,
+0.0022583007812500, 0.0014648437500000, 0.0022888183593750, 0.0014343261718750,
+0.0021362304687500, 0.0015563964843750, 0.0029296875000000, 0.0012817382812500,
+0.0026855468750000, 0.0013732910156250, 0.0030822753906250, 0.0012207031250000,
+0.0038452148437500, 0.0008544921875000, 0.0034484863281250, 0.0010681152343750,
+0.0042724609375000, 0.0007324218750000, 0.0043640136718750, 0.0007324218750000,
+0.0043945312500000, 0.0006713867187500, 0.0048217773437500, 0.0004577636718750,
+0.0047302246093750, 0.0006103515625000, 0.0047302246093750, 0.0003967285156250,
+0.0048828125000000, 0.0004882812500000, 0.0046691894531250, 0.0003967285156250,
+0.0044860839843750, 0.0002746582031250, 0.0046386718750000, 0.0004882812500000,
+0.0040283203125000, -0.0000305175781250, 0.0041809082031250, 0.0002136230468750,
+0.0038757324218750, 0.0001220703125000, 0.0033874511718750, -0.0002746582031250,
+0.0036315917968750, 0.0001831054687500, 0.0030212402343750, -0.0003662109375000,
+0.0030822753906250, -0.0002441406250000, 0.0028991699218750, -0.0002441406250000,
+0.0024414062500000, -0.0005187988281250, 0.0026550292968750, -0.0002441406250000,
+0.0019836425781250, -0.0004882812500000, 0.0020751953125000, -0.0003662109375000,
+0.0019226074218750, -0.0003051757812500, 0.0014953613281250, -0.0003662109375000,
+0.0016784667968750, -0.0000915527343750, 0.0010681152343750, -0.0003051757812500,
+0.0011596679687500, -0.0000610351562500, 0.0008850097656250, 0.0000610351562500,
+0.0003356933593750, 0.0000000000000000, 0.0005798339843750, 0.0003662109375000,
+0.0000610351562500, 0.0001220703125000, 0.0000305175781250, 0.0003662109375000,
+-0.0000915527343750, 0.0003051757812500, -0.0003967285156250, 0.0000305175781250,
+-0.0002136230468750, 0.0003967285156250, -0.0004882812500000, -0.0000610351562500,
+-0.0004882812500000, 0.0002136230468750, -0.0003356933593750, 0.0000915527343750,
+-0.0004577636718750, -0.0002746582031250, -0.0005493164062500, 0.0002136230468750,
+0.0001831054687500, -0.0005493164062500, -0.0001220703125000, -0.0003967285156250,
+0.0003051757812500, -0.0002441406250000, 0.0009765625000000, -0.0007019042968750,
+0.0004577636718750, -0.0005798339843750, 0.0015258789062500, -0.0004577636718750,
+0.0010986328125000, -0.0006103515625000, 0.0013732910156250, -0.0004577636718750,
+0.0023193359375000, -0.0001831054687500, 0.0016479492187500, -0.0003662109375000,
+0.0024719238281250, -0.0002746582031250, 0.0024108886718750, -0.0002746582031250,
+0.0021972656250000, -0.0003356933593750, 0.0025329589843750, -0.0003967285156250,
+0.0022888183593750, -0.0003967285156250, 0.0020141601562500, -0.0004272460937500,
+0.0022888183593750, -0.0005187988281250, 0.0018310546875000, -0.0004882812500000,
+0.0012512207031250, -0.0005493164062500, 0.0014038085937500, -0.0007019042968750,
+-0.0001525878906250, -0.0005798339843750, 0.0005187988281250, -0.0007019042968750,
+0.0000305175781250, -0.0007324218750000, -0.0013427734375000, -0.0006103515625000,
+-0.0003967285156250, -0.0007019042968750, -0.0018005371093750, -0.0006713867187500,
+-0.0016174316406250, -0.0005493164062500, -0.0013427734375000, -0.0007629394531250,
+-0.0019836425781250, -0.0008850097656250, -0.0015563964843750, -0.0006408691406250,
+-0.0011901855468750, -0.0011901855468750, -0.0016479492187500, -0.0010375976562500,
+-0.0009765625000000, -0.0008850097656250, -0.0003662109375000, -0.0011291503906250,
+-0.0010070800781250, -0.0009460449218750, 0.0005798339843750, -0.0005187988281250,
+0.0003662109375000, -0.0004882812500000, 0.0005187988281250, -0.0000915527343750,
+0.0015258789062500, 0.0004882812500000, 0.0009765625000000, 0.0003967285156250,
+0.0015258789062500, 0.0008544921875000, 0.0016174316406250, 0.0011596679687500,
+0.0013732910156250, 0.0010375976562500, 0.0015258789062500, 0.0010681152343750,
+0.0015869140625000, 0.0014343261718750, 0.0011596679687500, 0.0008850097656250,
+0.0014953613281250, 0.0010375976562500, 0.0012512207031250, 0.0010681152343750,
+0.0007629394531250, 0.0007629394531250, 0.0012817382812500, 0.0009460449218750,
+0.0002746582031250, 0.0010681152343750, 0.0005493164062500, 0.0008544921875000,
+0.0003356933593750, 0.0013427734375000, -0.0003356933593750, 0.0017395019531250,
+0.0001831054687500, 0.0014038085937500, -0.0007019042968750, 0.0026550292968750,
+-0.0004577636718750, 0.0025329589843750, -0.0005798339843750, 0.0029907226562500,
+-0.0012207031250000, 0.0039672851562500, -0.0008239746093750, 0.0036621093750000,
+-0.0013732910156250, 0.0046691894531250, -0.0015258789062500, 0.0048217773437500,
+-0.0012817382812500, 0.0049743652343750, -0.0014648437500000, 0.0055541992187500,
+-0.0016479492187500, 0.0054016113281250, -0.0009765625000000, 0.0054931640625000,
+-0.0012817382812500, 0.0056152343750000, -0.0009765625000000, 0.0054931640625000,
+-0.0003356933593750, 0.0053710937500000, -0.0008544921875000, 0.0054321289062500,
+-0.0001525878906250, 0.0052795410156250, -0.0002136230468750, 0.0051574707031250,
+-0.0002441406250000, 0.0051879882812500, 0.0001831054687500, 0.0052490234375000,
+-0.0001220703125000, 0.0051574707031250, 0.0001525878906250, 0.0052795410156250,
+-0.0000305175781250, 0.0052795410156250, 0.0000610351562500, 0.0051879882812500,
+0.0002746582031250, 0.0051269531250000, -0.0001220703125000, 0.0050659179687500,
+0.0003051757812500, 0.0050964355468750, 0.0001525878906250, 0.0049438476562500,
+0.0000915527343750, 0.0049743652343750, 0.0004272460937500, 0.0050964355468750,
+0.0002136230468750, 0.0049438476562500, 0.0005493164062500, 0.0050964355468750,
+0.0005493164062500, 0.0050354003906250, 0.0005798339843750, 0.0050659179687500,
+0.0007629394531250, 0.0051879882812500, 0.0008239746093750, 0.0050354003906250,
+0.0007324218750000, 0.0055847167968750, 0.0009155273437500, 0.0053710937500000,
+0.0008850097656250, 0.0058593750000000, 0.0008239746093750, 0.0065612792968750,
+0.0009155273437500, 0.0061035156250000, 0.0009155273437500, 0.0075683593750000,
+0.0008239746093750, 0.0074157714843750, 0.0008544921875000, 0.0078430175781250,
+0.0008544921875000, 0.0089111328125000, 0.0007934570312500, 0.0084228515625000,
+0.0006408691406250, 0.0093078613281250, 0.0006103515625000, 0.0095214843750000,
+0.0005187988281250, 0.0092773437500000, 0.0003356933593750, 0.0095825195312500,
+0.0003662109375000, 0.0097351074218750, 0.0002441406250000, 0.0089721679687500,
+0.0001831054687500, 0.0092468261718750, 0.0001831054687500, 0.0087585449218750,
+0.0002136230468750, 0.0079650878906250, 0.0001525878906250, 0.0084533691406250,
+0.0001525878906250, 0.0068054199218750, 0.0002441406250000, 0.0069885253906250,
+0.0000915527343750, 0.0065307617187500, 0.0000305175781250, 0.0052795410156250,
+0.0001525878906250, 0.0057983398437500, -0.0001525878906250, 0.0044250488281250,
+-0.0000305175781250, 0.0044250488281250, -0.0001831054687500, 0.0041198730468750,
+-0.0004272460937500, 0.0032653808593750, -0.0001220703125000, 0.0036621093750000,
+-0.0006713867187500, 0.0028076171875000, -0.0003967285156250, 0.0028381347656250,
+-0.0004577636718750, 0.0026550292968750, -0.0007019042968750, 0.0021362304687500,
+-0.0003051757812500, 0.0025329589843750, -0.0004882812500000, 0.0018310546875000,
+-0.0003051757812500, 0.0019531250000000, -0.0000305175781250, 0.0019531250000000,
+-0.0000305175781250, 0.0016479492187500, 0.0001220703125000, 0.0019226074218750,
+0.0004272460937500, 0.0018005371093750, 0.0001525878906250, 0.0018005371093750,
+0.0003051757812500, 0.0022583007812500, 0.0005187988281250, 0.0025024414062500,
+-0.0000305175781250, 0.0023803710937500, 0.0005798339843750, 0.0033569335937500,
+0.0000305175781250, 0.0032653808593750, 0.0000305175781250, 0.0037841796875000,
+0.0005187988281250, 0.0046386718750000, -0.0003051757812500, 0.0043029785156250,
+0.0003967285156250, 0.0053710937500000, 0.0002441406250000, 0.0055236816406250,
+0.0000000000000000, 0.0055236816406250, 0.0003967285156250, 0.0061340332031250,
+0.0002136230468750, 0.0062255859375000, 0.0002441406250000, 0.0057678222656250,
+0.0003662109375000, 0.0060729980468750, 0.0003967285156250, 0.0057067871093750,
+0.0003662109375000, 0.0050964355468750, 0.0004882812500000, 0.0055236816406250,
+0.0003051757812500, 0.0047302246093750, 0.0004577636718750, 0.0045471191406250,
+0.0003662109375000, 0.0047302246093750, 0.0001525878906250, 0.0044860839843750,
+0.0004882812500000, 0.0043334960937500, -0.0000305175781250, 0.0048522949218750,
+0.0002441406250000, 0.0044250488281250, 0.0000610351562500, 0.0043945312500000,
+-0.0003051757812500, 0.0048522949218750, 0.0002136230468750, 0.0043640136718750,
+-0.0007324218750000, 0.0043334960937500, -0.0003967285156250, 0.0044860839843750,
+-0.0005798339843750, 0.0039367675781250, -0.0014038085937500, 0.0034790039062500,
+-0.0007629394531250, 0.0036621093750000, -0.0016174316406250, 0.0029296875000000,
+-0.0015563964843750, 0.0028076171875000, -0.0014648437500000, 0.0026855468750000,
+-0.0018005371093750, 0.0023498535156250, -0.0015258789062500, 0.0025329589843750,
+-0.0015869140625000, 0.0021667480468750, -0.0015869140625000, 0.0023803710937500,
+-0.0014343261718750, 0.0021362304687500, -0.0013732910156250, 0.0019226074218750,
+-0.0012817382812500, 0.0023498535156250, -0.0010375976562500, 0.0011291503906250,
+-0.0012512207031250, 0.0016784667968750, -0.0007324218750000, 0.0012817382812500,
+-0.0002746582031250, 0.0001831054687500, -0.0009460449218750, 0.0010070800781250,
+0.0000610351562500, -0.0003967285156250, -0.0000610351562500, -0.0002746582031250,
+-0.0002136230468750, -0.0001220703125000, 0.0003967285156250, -0.0009155273437500,
+0.0002746582031250, -0.0005187988281250, 0.0001831054687500, 0.0002441406250000,
+0.0003662109375000, -0.0002746582031250, 0.0003967285156250, 0.0006713867187500,
+0.0003051757812500, 0.0018920898437500, 0.0004577636718750, 0.0010070800781250,
+0.0007934570312500, 0.0025024414062500, 0.0006408691406250, 0.0025939941406250,
+0.0007019042968750, 0.0023803710937500, 0.0010375976562500, 0.0028381347656250,
+0.0008239746093750, 0.0026245117187500, 0.0007019042968750, 0.0030212402343750,
+0.0007324218750000, 0.0028686523437500, 0.0007324218750000, 0.0029602050781250,
+0.0006408691406250, 0.0036315917968750, 0.0019836425781250, -0.0001220703125000,
+0.0017700195312500, 0.0020446777343750, 0.0021667480468750, 0.0010375976562500,
+0.0015563964843750, 0.0017700195312500, 0.0010681152343750, 0.0034484863281250,
+0.0016784667968750, 0.0028076171875000, 0.0005798339843750, 0.0027465820312500,
+0.0005493164062500, 0.0035095214843750, 0.0007629394531250, 0.0025634765625000,
+0.0003051757812500, 0.0015563964843750, 0.0002746582031250, 0.0023803710937500,
+0.0008239746093750, 0.0004272460937500, 0.0005798339843750, 0.0004882812500000,
+0.0006103515625000, 0.0003051757812500, 0.0010375976562500, -0.0008239746093750,
+0.0008239746093750, -0.0004882812500000, 0.0011596679687500, -0.0013122558593750,
+0.0012512207031250, -0.0015869140625000, 0.0014648437500000, -0.0015563964843750,
+0.0018005371093750, -0.0020446777343750, 0.0018615722656250, -0.0022583007812500,
+0.0024719238281250, -0.0014648437500000, 0.0028076171875000, -0.0021972656250000,
+0.0026855468750000, -0.0015563964843750, 0.0029602050781250, -0.0003662109375000,
+0.0034179687500000, -0.0012207031250000, 0.0024719238281250, 0.0003662109375000,
+0.0028076171875000, 0.0006103515625000, 0.0028076171875000, 0.0002441406250000,
+0.0020446777343750},
+{0.0000610351562500, 0.0016174316406250, -0.0000305175781250, 0.0026245117187500,
+0.0000915527343750, 0.0019531250000000, -0.0001525878906250, 0.0026550292968750,
+0.0000305175781250, 0.0026550292968750, -0.0000305175781250, 0.0023498535156250,
+-0.0001831054687500, 0.0023803710937500, 0.0002441406250000, 0.0022277832031250,
+-0.0004272460937500, 0.0018920898437500, 0.0001220703125000, 0.0014953613281250,
+-0.0001220703125000, 0.0015869140625000, -0.0007629394531250, 0.0014953613281250,
+0.0000000000000000, 0.0009765625000000, -0.0010681152343750, 0.0020446777343750,
+-0.0008850097656250, 0.0014038085937500, -0.0008850097656250, 0.0018005371093750,
+-0.0015258789062500, 0.0029296875000000, -0.0011291503906250, 0.0018920898437500,
+-0.0015563964843750, 0.0031127929687500, -0.0015258789062500, 0.0029907226562500,
+-0.0015258789062500, 0.0029296875000000, -0.0016479492187500, 0.0035095214843750,
+-0.0014038085937500, 0.0031433105468750, -0.0014648437500000, 0.0035705566406250,
+-0.0015869140625000, 0.0035400390625000, -0.0013122558593750, 0.0035400390625000,
+-0.0011901855468750, 0.0038452148437500, -0.0014953613281250, 0.0038452148437500,
+-0.0007629394531250, 0.0041198730468750, -0.0010375976562500, 0.0043334960937500,
+-0.0007934570312500, 0.0042724609375000, -0.0001525878906250, 0.0043945312500000,
+-0.0006713867187500, 0.0045776367187500, -0.0001525878906250, 0.0038757324218750,
+-0.0001525878906250, 0.0043029785156250, -0.0000915527343750, 0.0036010742187500,
+0.0002136230468750, 0.0026550292968750, 0.0000915527343750, 0.0034484863281250,
+0.0005187988281250, 0.0010375976562500, 0.0005798339843750, 0.0013122558593750,
+0.0007629394531250, 0.0007934570312500, 0.0010681152343750, -0.0010986328125000,
+0.0010070800781250, -0.0003051757812500, 0.0013427734375000, -0.0007934570312500,
+0.0013427734375000, -0.0014953613281250, 0.0014038085937500, -0.0008544921875000,
+0.0015869140625000, -0.0003356933593750, 0.0014953613281250, -0.0010070800781250,
+0.0017395019531250, -0.0000305175781250, 0.0017089843750000, -0.0002136230468750,
+0.0017395019531250, -0.0001831054687500, 0.0020141601562500, 0.0007629394531250,
+0.0014648437500000, 0.0002136230468750, 0.0009155273437500, -0.0007934570312500,
+0.0006408691406250, -0.0007324218750000, 0.0010375976562500, -0.0006713867187500,
+0.0010986328125000, -0.0011901855468750, 0.0005187988281250, -0.0013427734375000,
+0.0014953613281250, 0.0000305175781250, 0.0012207031250000, -0.0005493164062500,
+0.0010986328125000, 0.0001220703125000, 0.0014953613281250, 0.0016784667968750,
+0.0010070800781250, 0.0007934570312500, 0.0014648437500000, 0.0016479492187500,
+0.0010375976562500, 0.0020751953125000, 0.0013732910156250, 0.0014343261718750,
+0.0019836425781250, 0.0012512207031250, 0.0013732910156250, 0.0015258789062500,
+0.0028076171875000, 0.0004272460937500, 0.0027770996093750, 0.0004882812500000,
+0.0027465820312500, 0.0001220703125000, 0.0035705566406250, -0.0006408691406250,
+0.0033264160156250, -0.0003356933593750, 0.0032043457031250, -0.0014343261718750,
+0.0033569335937500, -0.0015258789062500, 0.0035095214843750, -0.0017700195312500,
+0.0033264160156250, -0.0026245117187500, 0.0032348632812500, -0.0024719238281250,
+0.0044555664062500, -0.0024414062500000, 0.0040283203125000, -0.0029602050781250,
+0.0047912597656250, -0.0023498535156250, 0.0063476562500000, -0.0017089843750000,
+0.0057067871093750, -0.0022888183593750, 0.0069274902343750, -0.0008850097656250,
+0.0076293945312500, -0.0008239746093750, 0.0069885253906250, -0.0007629394531250,
+0.0072326660156250, -0.0000305175781250, 0.0079040527343750, -0.0001831054687500,
+0.0051574707031250, -0.0002746582031250, 0.0062561035156250, -0.0000610351562500,
+0.0052795410156250, -0.0001220703125000, 0.0027465820312500, -0.0002746582031250,
+0.0043029785156250, -0.0000915527343750, 0.0010986328125000, -0.0004272460937500,
+0.0013732910156250, -0.0003662109375000, 0.0009765625000000, -0.0003356933593750,
+-0.0011291503906250, -0.0006713867187500, -0.0001831054687500, -0.0006408691406250,
+-0.0016479492187500, -0.0002746582031250, -0.0017395019531250, -0.0004882812500000,
+-0.0017700195312500, -0.0002136230468750, -0.0025634765625000, 0.0002441406250000,
+-0.0023193359375000, -0.0000305175781250, -0.0023803710937500, 0.0004272460937500,
+-0.0028076171875000, 0.0005493164062500, -0.0022888183593750, 0.0003967285156250,
+-0.0020141601562500, 0.0004882812500000, -0.0026550292968750, 0.0005187988281250,
+-0.0008850097656250, 0.0002441406250000, -0.0016174316406250, 0.0002136230468750,
+-0.0010986328125000, 0.0001831054687500, 0.0001831054687500, 0.0000305175781250,
+-0.0010375976562500, 0.0000915527343750, 0.0011596679687500, 0.0000915527343750,
+0.0003356933593750, 0.0000305175781250, 0.0007324218750000, 0.0001220703125000,
+0.0023498535156250, 0.0001831054687500, 0.0008544921875000, 0.0000610351562500,
+0.0025939941406250, 0.0003051757812500, 0.0022583007812500, 0.0002136230468750,
+0.0019531250000000, 0.0001831054687500, 0.0027465820312500, 0.0002441406250000,
+0.0021057128906250, 0.0000305175781250, 0.0021667480468750, 0.0000610351562500,
+0.0023193359375000, -0.0002746582031250, 0.0017700195312500, -0.0003051757812500,
+0.0014648437500000, -0.0003051757812500, 0.0017700195312500, -0.0007934570312500,
+0.0008544921875000, -0.0004882812500000, 0.0009155273437500, -0.0008544921875000,
+0.0007934570312500, -0.0008850097656250, 0.0002441406250000, -0.0006408691406250,
+0.0005493164062500, -0.0010681152343750, 0.0003356933593750, -0.0006713867187500,
+0.0000000000000000, -0.0008544921875000, 0.0002136230468750, -0.0008850097656250,
+0.0002746582031250, -0.0005798339843750, -0.0000610351562500, -0.0007324218750000,
+0.0003051757812500, -0.0005798339843750, 0.0001525878906250, -0.0005187988281250,
+0.0000915527343750, -0.0004882812500000, 0.0001525878906250, -0.0004272460937500,
+-0.0000610351562500, -0.0004577636718750, 0.0001220703125000, -0.0001831054687500,
+-0.0000610351562500, -0.0003356933593750, 0.0000610351562500, -0.0002136230468750,
+0.0003356933593750, 0.0000305175781250, 0.0000915527343750, -0.0001220703125000,
+0.0006408691406250, 0.0002441406250000, 0.0004882812500000, 0.0002746582031250,
+0.0007934570312500, 0.0003967285156250, 0.0013427734375000, 0.0006713867187500,
+0.0010070800781250, 0.0006103515625000, 0.0019531250000000, 0.0008239746093750,
+0.0019836425781250, 0.0008850097656250, 0.0022583007812500, 0.0009765625000000,
+0.0029907226562500, 0.0010986328125000, 0.0028991699218750, 0.0010375976562500,
+0.0035400390625000, 0.0009765625000000, 0.0036010742187500, 0.0010375976562500,
+0.0038452148437500, 0.0010070800781250, 0.0043029785156250, 0.0009155273437500,
+0.0041198730468750, 0.0009765625000000, 0.0046081542968750, 0.0009155273437500,
+0.0046386718750000, 0.0009155273437500, 0.0046386718750000, 0.0009155273437500,
+0.0048522949218750, 0.0008850097656250, 0.0046386718750000, 0.0008544921875000,
+0.0044555664062500, 0.0007019042968750, 0.0045471191406250, 0.0007629394531250,
+0.0041809082031250, 0.0005798339843750, 0.0037841796875000, 0.0003356933593750,
+0.0039672851562500, 0.0004882812500000, 0.0029907226562500, 0.0000915527343750,
+0.0031127929687500, 0.0002441406250000, 0.0027770996093750, 0.0000915527343750,
+0.0020141601562500, -0.0002746582031250, 0.0024108886718750, 0.0000305175781250,
+0.0016479492187500, -0.0004272460937500, 0.0015869140625000, -0.0003662109375000,
+0.0017089843750000, -0.0004272460937500, 0.0013732910156250, -0.0006713867187500,
+0.0015258789062500, -0.0005187988281250, 0.0017700195312500, -0.0007934570312500,
+0.0016479492187500, -0.0008239746093750, 0.0017700195312500, -0.0007934570312500,
+0.0020446777343750, -0.0009155273437500, 0.0018310546875000, -0.0009765625000000,
+0.0020751953125000, -0.0007629394531250, 0.0021972656250000, -0.0008239746093750,
+0.0020141601562500, -0.0007019042968750, 0.0019531250000000, -0.0004272460937500,
+0.0019836425781250, -0.0004577636718750, 0.0015563964843750, -0.0002746582031250,
+0.0016174316406250, -0.0000915527343750, 0.0013427734375000, -0.0000610351562500,
+0.0009460449218750, 0.0000000000000000, 0.0010375976562500, 0.0002136230468750,
+0.0005493164062500, 0.0001525878906250, 0.0003967285156250, 0.0003051757812500,
+0.0003051757812500, 0.0003051757812500, -0.0000610351562500, 0.0002746582031250,
+-0.0002746582031250, 0.0004272460937500, -0.0002136230468750, 0.0002441406250000,
+-0.0004272460937500, 0.0003662109375000, -0.0003356933593750, 0.0003356933593750,
+-0.0000610351562500, 0.0000610351562500, -0.0003051757812500, 0.0002746582031250,
+0.0001220703125000, 0.0002441406250000, 0.0001831054687500, 0.0000915527343750,
+0.0001831054687500, 0.0003051757812500, 0.0003967285156250, 0.0005187988281250,
+0.0004882812500000, 0.0001831054687500, 0.0003967285156250, 0.0005798339843750,
+0.0005493164062500, 0.0004272460937500, 0.0005187988281250, 0.0003662109375000,
+0.0004272460937500, 0.0005798339843750, 0.0006103515625000, 0.0001831054687500,
+0.0003662109375000, 0.0003051757812500, 0.0004882812500000, 0.0002136230468750,
+0.0001831054687500, 0.0000610351562500, -0.0001525878906250, 0.0000610351562500,
+0.0000915527343750, -0.0000610351562500, -0.0006713867187500, -0.0001220703125000,
+-0.0008544921875000, -0.0001831054687500, -0.0007324218750000, -0.0002441406250000,
+-0.0010375976562500, -0.0002746582031250, -0.0011291503906250, -0.0003967285156250,
+-0.0007629394531250, -0.0003051757812500, -0.0008544921875000, -0.0003356933593750,
+-0.0008850097656250, -0.0004272460937500, -0.0004882812500000, -0.0003356933593750,
+-0.0002441406250000, -0.0002746582031250, 0.0002136230468750, -0.0003051757812500,
+-0.0001220703125000, -0.0003051757812500, 0.0001525878906250, -0.0001831054687500,
+0.0006408691406250, -0.0002136230468750, 0.0001525878906250, -0.0001525878906250,
+0.0011291503906250, 0.0003662109375000, 0.0010070800781250, 0.0002746582031250,
+0.0007934570312500, 0.0006103515625000, 0.0013732910156250, 0.0012512207031250,
+0.0012207031250000, 0.0010681152343750, 0.0007019042968750, 0.0014648437500000,
+0.0010070800781250, 0.0016784667968750, 0.0006408691406250, 0.0014648437500000,
+0.0000610351562500, 0.0014953613281250, 0.0005493164062500, 0.0016479492187500,
+-0.0001220703125000, 0.0010681152343750, -0.0000610351562500, 0.0011291503906250,
+-0.0001525878906250, 0.0011596679687500, -0.0005798339843750, 0.0006408691406250,
+-0.0002746582031250, 0.0006713867187500, -0.0006713867187500, 0.0010681152343750,
+-0.0007324218750000, 0.0007019042968750, -0.0004577636718750, 0.0011596679687500,
+-0.0005493164062500, 0.0017395019531250, -0.0007324218750000, 0.0011291503906250,
+0.0002136230468750, 0.0026550292968750, -0.0001525878906250, 0.0023803710937500,
+0.0001525878906250, 0.0024414062500000, 0.0009765625000000, 0.0034484863281250,
+0.0003356933593750, 0.0030212402343750, 0.0013122558593750, 0.0026245117187500,
+0.0012817382812500, 0.0030822753906250, 0.0011596679687500, 0.0022888183593750,
+0.0016784667968750, 0.0014343261718750, 0.0014343261718750, 0.0020141601562500,
+0.0013122558593750, 0.0004882812500000, 0.0014953613281250, 0.0004272460937500,
+0.0010986328125000, 0.0003051757812500, 0.0007629394531250, -0.0005493164062500,
+0.0010681152343750, -0.0003662109375000, 0.0002441406250000, -0.0007324218750000,
+0.0003662109375000, -0.0008544921875000, 0.0002746582031250, -0.0008544921875000,
+-0.0003051757812500, -0.0008850097656250, 0.0000000000000000, -0.0008850097656250,
+-0.0006103515625000},
+{0.0037536621093750, 0.0000915527343750, 0.0035400390625000, 0.0001525878906250,
+0.0034790039062500, 0.0000915527343750, 0.0032043457031250, 0.0001525878906250,
+0.0029296875000000, 0.0001220703125000, 0.0030517578125000, 0.0000000000000000,
+0.0021362304687500, 0.0003356933593750, 0.0022583007812500, 0.0000915527343750,
+0.0018920898437500, 0.0002136230468750, 0.0010375976562500, 0.0005493164062500,
+0.0014038085937500, 0.0002136230468750, 0.0005187988281250, 0.0004882812500000,
+0.0005187988281250, 0.0003662109375000, 0.0003662109375000, 0.0003662109375000,
+-0.0000305175781250, 0.0004577636718750, 0.0002441406250000, 0.0001525878906250,
+0.0000000000000000, 0.0003967285156250, 0.0000305175781250, 0.0001220703125000,
+0.0000305175781250, 0.0000305175781250, 0.0000305175781250, 0.0002136230468750,
+0.0003356933593750, -0.0000610351562500, 0.0000000000000000, -0.0000915527343750,
+0.0000000000000000, -0.0001220703125000, 0.0001831054687500, -0.0003356933593750,
+0.0001525878906250, -0.0005187988281250, 0.0000915527343750, -0.0004882812500000,
+0.0000610351562500, -0.0005798339843750, 0.0001525878906250, -0.0008544921875000,
+-0.0000915527343750, -0.0007629394531250, -0.0003356933593750, -0.0007019042968750,
+-0.0001525878906250, -0.0009155273437500, -0.0005187988281250, -0.0006713867187500,
+-0.0005798339843750, -0.0007934570312500, -0.0004577636718750, -0.0005493164062500,
+-0.0005187988281250, -0.0001525878906250, -0.0005493164062500, -0.0002136230468750,
+-0.0001220703125000, 0.0005798339843750, -0.0003967285156250, 0.0003356933593750,
+-0.0003967285156250, 0.0004882812500000, -0.0000915527343750, 0.0012512207031250,
+-0.0003051757812500, 0.0011291503906250, 0.0000915527343750, 0.0012207031250000,
+-0.0001831054687500, 0.0015563964843750, -0.0000610351562500, 0.0013122558593750,
+0.0003967285156250, 0.0010986328125000, 0.0000000000000000, 0.0014648437500000,
+0.0005187988281250, 0.0007629394531250, 0.0005493164062500, 0.0009460449218750,
+0.0004272460937500, 0.0009460449218750, 0.0005493164062500, 0.0005493164062500,
+0.0005798339843750, 0.0008239746093750, 0.0003051757812500, 0.0007324218750000,
+0.0003356933593750, 0.0005493164062500, 0.0003356933593750, 0.0009155273437500,
+0.0000915527343750, 0.0011901855468750, 0.0000915527343750, 0.0008544921875000,
+-0.0001220703125000, 0.0014648437500000, -0.0000915527343750, 0.0014343261718750,
+-0.0002746582031250, 0.0013732910156250, -0.0005187988281250, 0.0017089843750000,
+-0.0003662109375000, 0.0014648437500000, -0.0005493164062500, 0.0012817382812500,
+-0.0007934570312500, 0.0013732910156250, -0.0005493164062500, 0.0009765625000000,
+-0.0003967285156250, 0.0004272460937500, -0.0007324218750000, 0.0005187988281250,
+-0.0000305175781250, 0.0002441406250000, -0.0003051757812500, -0.0001831054687500,
+-0.0001831054687500, 0.0001831054687500, 0.0004272460937500, 0.0003967285156250,
+0.0000000000000000, 0.0000305175781250, 0.0005798339843750, 0.0012207031250000,
+0.0004272460937500, 0.0008239746093750, 0.0004272460937500, 0.0013122558593750,
+0.0007324218750000, 0.0023193359375000, 0.0004272460937500, 0.0015869140625000,
+0.0006103515625000, 0.0032958984375000, 0.0006103515625000, 0.0029907226562500,
+0.0003967285156250, 0.0035095214843750, 0.0004272460937500, 0.0048522949218750,
+0.0005187988281250, 0.0040893554687500, 0.0000000000000000, 0.0056152343750000,
+0.0001525878906250, 0.0057067871093750, 0.0000305175781250, 0.0059814453125000,
+-0.0003967285156250, 0.0068969726562500, -0.0001831054687500, 0.0066833496093750,
+-0.0005798339843750, 0.0074157714843750, -0.0005493164062500, 0.0075988769531250,
+-0.0008239746093750, 0.0077209472656250, -0.0011901855468750, 0.0081176757812500,
+-0.0009765625000000, 0.0081176757812500, -0.0016479492187500, 0.0081787109375000,
+-0.0016784667968750, 0.0083923339843750, -0.0015563964843750, 0.0083007812500000,
+-0.0018920898437500, 0.0083007812500000, -0.0019226074218750, 0.0086059570312500,
+-0.0014038085937500, 0.0080261230468750, -0.0017700195312500, 0.0083007812500000,
+-0.0013732910156250, 0.0080871582031250, -0.0006713867187500, 0.0076293945312500,
+-0.0012817382812500, 0.0081176757812500, -0.0001525878906250, 0.0071411132812500,
+-0.0003356933593750, 0.0073547363281250, -0.0002136230468750, 0.0072631835937500,
+0.0005493164062500, 0.0065612792968750, 0.0000000000000000, 0.0071411132812500,
+0.0004882812500000, 0.0065917968750000, 0.0003662109375000, 0.0067138671875000,
+0.0001831054687500, 0.0069274902343750, 0.0003662109375000, 0.0068359375000000,
+0.0002136230468750, 0.0070800781250000, 0.0001831054687500, 0.0073242187500000,
+0.0002136230468750, 0.0074157714843750, 0.0002136230468750, 0.0076904296875000,
+0.0002136230468750, 0.0080261230468750, 0.0003662109375000, 0.0079956054687500,
+0.0002441406250000, 0.0081176757812500, 0.0005187988281250, 0.0084533691406250,
+0.0003967285156250, 0.0081176757812500, 0.0002441406250000, 0.0079345703125000,
+0.0007324218750000, 0.0083007812500000, 0.0000305175781250, 0.0070800781250000,
+0.0003967285156250, 0.0075073242187500, 0.0003967285156250, 0.0068969726562500,
+-0.0000305175781250, 0.0057373046875000, 0.0004272460937500, 0.0065002441406250,
+-0.0000610351562500, 0.0046691894531250, 0.0001525878906250, 0.0048828125000000,
+0.0001525878906250, 0.0044860839843750, -0.0000915527343750, 0.0031433105468750,
+0.0002746582031250, 0.0038452148437500, 0.0000305175781250, 0.0026550292968750,
+0.0001831054687500, 0.0027465820312500, 0.0002441406250000, 0.0027465820312500,
+0.0001220703125000, 0.0021362304687500, 0.0003051757812500, 0.0025634765625000,
+0.0002441406250000, 0.0027465820312500, 0.0001220703125000, 0.0026550292968750,
+0.0002136230468750, 0.0031738281250000, 0.0001831054687500, 0.0036315917968750,
+-0.0001525878906250, 0.0036315917968750, 0.0002441406250000, 0.0045166015625000,
+-0.0002441406250000, 0.0047302246093750, -0.0001525878906250, 0.0050354003906250,
+0.0002746582031250, 0.0057373046875000, -0.0004577636718750, 0.0058288574218750,
+0.0002746582031250, 0.0061035156250000, -0.0000305175781250, 0.0065307617187500,
+-0.0000915527343750, 0.0065612792968750, 0.0004272460937500, 0.0066223144531250,
+0.0000610351562500, 0.0069885253906250, 0.0004882812500000, 0.0065307617187500,
+0.0005187988281250, 0.0068054199218750, 0.0004577636718750, 0.0067443847656250,
+0.0007019042968750, 0.0062866210937500, 0.0005798339843750, 0.0066528320312500,
+0.0005187988281250, 0.0061340332031250, 0.0005493164062500, 0.0061950683593750,
+0.0004272460937500, 0.0062255859375000, 0.0002746582031250, 0.0059814453125000,
+0.0003662109375000, 0.0061340332031250, -0.0000610351562500, 0.0057678222656250,
+0.0001220703125000, 0.0058288574218750, -0.0000915527343750, 0.0055847167968750,
+-0.0004577636718750, 0.0052185058593750, 0.0000000000000000, 0.0052795410156250,
+-0.0007324218750000, 0.0046386718750000, -0.0005798339843750, 0.0045166015625000,
+-0.0006103515625000, 0.0044250488281250, -0.0010681152343750, 0.0040283203125000,
+-0.0007019042968750, 0.0039672851562500, -0.0010375976562500, 0.0041503906250000,
+-0.0010375976562500, 0.0039062500000000, -0.0009765625000000, 0.0040893554687500,
+-0.0011596679687500, 0.0045166015625000, -0.0011291503906250, 0.0042419433593750,
+-0.0010070800781250, 0.0047607421875000, -0.0011291503906250, 0.0049133300781250,
+-0.0009765625000000, 0.0046691894531250, -0.0007934570312500, 0.0047607421875000,
+-0.0009460449218750, 0.0048522949218750, -0.0003967285156250, 0.0038146972656250,
+-0.0006103515625000, 0.0039978027343750, -0.0005798339843750, 0.0033874511718750,
+-0.0002746582031250, 0.0024108886718750, -0.0007324218750000, 0.0029296875000000,
+-0.0003356933593750, 0.0016174316406250, -0.0005798339843750, 0.0016784667968750,
+-0.0005798339843750, 0.0015258789062500, -0.0003662109375000, 0.0006713867187500,
+-0.0007019042968750, 0.0010375976562500, -0.0000915527343750, 0.0002441406250000,
+-0.0001831054687500, 0.0002441406250000, -0.0000305175781250, 0.0001831054687500,
+0.0004882812500000, -0.0003051757812500, 0.0002441406250000, -0.0000915527343750,
+0.0006103515625000, -0.0002746582031250, 0.0006713867187500, -0.0004272460937500,
+0.0007324218750000, -0.0003051757812500, 0.0009460449218750, -0.0003356933593750,
+0.0008239746093750, -0.0004272460937500, 0.0012207031250000, 0.0002441406250000,
+0.0011291503906250, -0.0001831054687500, 0.0011291503906250, 0.0007629394531250,
+0.0013732910156250, 0.0016784667968750, 0.0012512207031250, 0.0007019042968750,
+0.0010681152343750, 0.0035095214843750, 0.0011901855468750, 0.0030822753906250,
+0.0009155273437500, 0.0032653808593750, 0.0005493164062500, 0.0050964355468750,
+0.0007324218750000, 0.0041809082031250, 0.0006103515625000, 0.0044860839843750,
+0.0003662109375000, 0.0048217773437500, 0.0006713867187500, 0.0043334960937500,
+0.0011901855468750, 0.0037231445312500, 0.0011291503906250, 0.0023803710937500,
+0.0018920898437500, 0.0035400390625000, 0.0012512207031250, 0.0035095214843750,
+0.0017089843750000, 0.0036315917968750, 0.0021057128906250, 0.0039672851562500,
+0.0014038085937500, 0.0038757324218750, 0.0028991699218750, 0.0035400390625000,
+0.0024719238281250, 0.0040588378906250, 0.0026855468750000, 0.0032043457031250,
+0.0038757324218750, 0.0023498535156250, 0.0030517578125000, 0.0032653808593750,
+0.0036621093750000, 0.0006408691406250, 0.0038452148437500, 0.0010070800781250,
+0.0035400390625000, 0.0007934570312500, 0.0035705566406250, -0.0009765625000000,
+0.0036315917968750, -0.0002746582031250, 0.0036621093750000, -0.0003967285156250,
+0.0036315917968750, -0.0009765625000000, 0.0035705566406250, -0.0005798339843750,
+0.0036926269531250, -0.0000915527343750, 0.0036926269531250, -0.0005493164062500,
+0.0027160644531250, 0.0000305175781250, 0.0031433105468750, -0.0000915527343750,
+0.0022888183593750, -0.0001831054687500, 0.0010070800781250, -0.0000305175781250,
+0.0018310546875000, -0.0002746582031250, 0.0001831054687500, 0.0000915527343750,
+0.0002441406250000, -0.0000915527343750, 0.0003051757812500, -0.0001525878906250,
+-0.0005187988281250, 0.0002136230468750, -0.0000305175781250, 0.0000305175781250,
+0.0003662109375000, -0.0005493164062500, 0.0001220703125000, -0.0002441406250000,
+0.0007934570312500, -0.0007019042968750, 0.0015563964843750, -0.0015869140625000,
+0.0011596679687500, -0.0011291503906250, 0.0025329589843750, -0.0015869140625000,
+0.0023498535156250, -0.0018615722656250, 0.0029296875000000, -0.0015563964843750,
+0.0040283203125000, -0.0014038085937500, 0.0033264160156250, -0.0016174316406250,
+0.0053710937500000, -0.0010681152343750, 0.0051269531250000, -0.0010986328125000,
+0.0057373046875000, -0.0010070800781250, 0.0072937011718750, -0.0005187988281250,
+0.0064697265625000, -0.0006713867187500, 0.0084533691406250, -0.0002136230468750,
+0.0083618164062500, -0.0001220703125000, 0.0086059570312500, -0.0002136230468750,
+0.0098571777343750, -0.0001525878906250, 0.0093383789062500, -0.0000305175781250,
+0.0097351074218750, -0.0005493164062500, 0.0100402832031250, -0.0004272460937500,
+0.0097961425781250, -0.0004882812500000, 0.0096435546875000, -0.0009155273437500,
+0.0098266601562500, -0.0007019042968750, 0.0098266601562500, -0.0006713867187500,
+0.0096435546875000, -0.0008239746093750, 0.0098876953125000, -0.0005187988281250,
+0.0099792480468750, -0.0001525878906250, 0.0097351074218750, -0.0003051757812500,
+0.0105590820312500, 0.0000000000000000, 0.0101318359375000, 0.0001220703125000,
+0.0104980468750000},
+{0.0005187988281250, 0.0024108886718750, 0.0004882812500000, 0.0022583007812500,
+0.0004272460937500, 0.0016479492187500, 0.0004882812500000, 0.0017395019531250,
+0.0002441406250000, 0.0012817382812500, 0.0003356933593750, 0.0011596679687500,
+0.0002136230468750, 0.0007934570312500, -0.0001220703125000, 0.0002746582031250,
+0.0000000000000000, 0.0004577636718750, 0.0000000000000000, 0.0000610351562500,
+-0.0002746582031250, -0.0003967285156250, 0.0000610351562500, 0.0001831054687500,
+0.0003967285156250, 0.0003967285156250, 0.0001831054687500, -0.0003051757812500,
+0.0010375976562500, 0.0015869140625000, 0.0009765625000000, 0.0012512207031250,
+0.0009460449218750, 0.0014343261718750, 0.0016174316406250, 0.0034790039062500,
+0.0019226074218750, 0.0000305175781250, 0.0031738281250000, 0.0007324218750000,
+0.0027770996093750, 0.0002746582031250, 0.0030212402343750, 0.0007629394531250,
+0.0036315917968750, 0.0014953613281250, 0.0033874511718750, 0.0012207031250000,
+0.0032043457031250, 0.0021972656250000, 0.0035400390625000, 0.0021362304687500,
+0.0031738281250000, 0.0024719238281250, 0.0026550292968750, 0.0033264160156250,
+0.0030517578125000, 0.0029602050781250, 0.0024414062500000, 0.0033569335937500,
+0.0026245117187500, 0.0038757324218750, 0.0024108886718750, 0.0032348632812500,
+0.0020141601562500, 0.0028686523437500, 0.0025024414062500, 0.0035705566406250,
+0.0014038085937500, 0.0016479492187500, 0.0016174316406250, 0.0018920898437500,
+0.0013122558593750, 0.0015869140625000, 0.0002136230468750, 0.0002136230468750,
+0.0006408691406250, 0.0007629394531250, 0.0006408691406250, 0.0002441406250000,
+-0.0000305175781250, -0.0000915527343750, 0.0007629394531250, 0.0001831054687500,
+0.0016479492187500, 0.0002441406250000, 0.0010681152343750, -0.0000915527343750,
+0.0027465820312500, 0.0001831054687500, 0.0026550292968750, 0.0000000000000000,
+0.0030212402343750, -0.0000610351562500, 0.0041198730468750, 0.0000610351562500,
+0.0036621093750000, -0.0002441406250000, 0.0048828125000000, 0.0003356933593750,
+0.0048522949218750, 0.0001525878906250, 0.0052490234375000, 0.0001220703125000,
+0.0059814453125000, 0.0005798339843750, 0.0055541992187500, 0.0004272460937500,
+0.0072631835937500, -0.0000915527343750, 0.0069885253906250, 0.0001220703125000,
+0.0075683593750000, -0.0001831054687500, 0.0090637207031250, -0.0007629394531250,
+0.0084228515625000, -0.0004272460937500, 0.0094604492187500, -0.0009155273437500,
+0.0099182128906250, -0.0010070800781250, 0.0096130371093750, -0.0009460449218750,
+0.0099487304687500, -0.0010681152343750, 0.0104064941406250, -0.0010375976562500,
+0.0095214843750000, -0.0007934570312500, 0.0100402832031250, -0.0008544921875000,
+0.0099487304687500, -0.0005798339843750, 0.0092163085937500, -0.0003051757812500,
+0.0098876953125000, -0.0004272460937500, 0.0097045898437500, 0.0000000000000000,
+0.0095520019531250, 0.0001525878906250, 0.0101013183593750, 0.0000610351562500,
+0.0103759765625000, 0.0001525878906250, 0.0101623535156250, 0.0002136230468750,
+0.0111389160156250, -0.0003051757812500, 0.0110778808593750, -0.0001525878906250,
+0.0111694335937500, -0.0002746582031250, 0.0117797851562500, -0.0007934570312500,
+0.0114135742187500, -0.0006103515625000, 0.0115051269531250, -0.0007019042968750,
+0.0116271972656250, -0.0008239746093750, 0.0110168457031250, -0.0006713867187500,
+0.0106811523437500, -0.0005187988281250, 0.0109252929687500, -0.0006713867187500,
+0.0095825195312500, -0.0002441406250000, 0.0097045898437500, -0.0003356933593750,
+0.0092468261718750, -0.0002441406250000, 0.0082397460937500, 0.0001525878906250,
+0.0087280273437500, -0.0000305175781250, 0.0076904296875000, 0.0001220703125000,
+0.0076904296875000, 0.0002441406250000, 0.0074768066406250, 0.0000305175781250,
+0.0068969726562500, -0.0000305175781250, 0.0072631835937500, 0.0000915527343750,
+0.0067749023437500, -0.0003662109375000, 0.0067138671875000, -0.0003051757812500,
+0.0069580078125000, -0.0003662109375000, 0.0067749023437500, -0.0006103515625000,
+0.0067138671875000, -0.0005798339843750, 0.0075073242187500, -0.0004272460937500,
+0.0070495605468750, -0.0006103515625000, 0.0075683593750000, -0.0003967285156250,
+0.0084228515625000, -0.0001525878906250, 0.0076293945312500, -0.0003356933593750,
+0.0090942382812500, 0.0002746582031250, 0.0087585449218750, 0.0003051757812500,
+0.0089721679687500, 0.0003967285156250, 0.0099182128906250, 0.0008544921875000,
+0.0090942382812500, 0.0008544921875000, 0.0101013183593750, 0.0009460449218750,
+0.0099487304687500, 0.0011901855468750, 0.0099792480468750, 0.0012207031250000,
+0.0105895996093750, 0.0011901855468750, 0.0101928710937500, 0.0013427734375000,
+0.0107116699218750, 0.0010986328125000, 0.0107116699218750, 0.0012207031250000,
+0.0106506347656250, 0.0010681152343750, 0.0109252929687500, 0.0007629394531250,
+0.0107727050781250, 0.0009460449218750, 0.0106201171875000, 0.0004882812500000,
+0.0106811523437500, 0.0005493164062500, 0.0103454589843750, 0.0004577636718750,
+0.0100708007812500, 0.0001220703125000, 0.0100708007812500, 0.0002136230468750,
+0.0091552734375000, -0.0000915527343750, 0.0092773437500000, -0.0000305175781250,
+0.0086669921875000, -0.0001220703125000, 0.0077514648437500, -0.0003967285156250,
+0.0082092285156250, -0.0002136230468750, 0.0065917968750000, -0.0005798339843750,
+0.0066833496093750, -0.0006103515625000, 0.0061340332031250, -0.0005798339843750,
+0.0049743652343750, -0.0007629394531250, 0.0054016113281250, -0.0007934570312500,
+0.0045471191406250, -0.0008544921875000, 0.0043334960937500, -0.0009765625000000,
+0.0045471191406250, -0.0009765625000000, 0.0042724609375000, -0.0010070800781250,
+0.0042724609375000, -0.0010681152343750, 0.0049438476562500, -0.0009765625000000,
+0.0047912597656250, -0.0010070800781250, 0.0052795410156250, -0.0010375976562500,
+0.0060424804687500, -0.0008850097656250, 0.0057067871093750, -0.0009155273437500,
+0.0067749023437500, -0.0007324218750000, 0.0069274902343750, -0.0007019042968750,
+0.0068969726562500, -0.0007324218750000, 0.0075073242187500, -0.0006408691406250,
+0.0075378417968750, -0.0004882812500000, 0.0073242187500000, -0.0006103515625000,
+0.0077514648437500, -0.0004577636718750, 0.0073242187500000, -0.0003967285156250,
+0.0068969726562500, -0.0004882812500000, 0.0073852539062500, -0.0003356933593750,
+0.0061645507812500, -0.0003662109375000, 0.0062866210937500, -0.0003356933593750,
+0.0061645507812500, -0.0003356933593750, 0.0052795410156250, -0.0003967285156250,
+0.0055541992187500, -0.0003662109375000, 0.0053100585937500, -0.0002746582031250,
+0.0051269531250000, -0.0003967285156250, 0.0054016113281250, -0.0002441406250000,
+0.0055541992187500, -0.0000610351562500, 0.0055236816406250, -0.0002136230468750,
+0.0060424804687500, 0.0000915527343750, 0.0060729980468750, 0.0000610351562500,
+0.0062561035156250, 0.0000610351562500, 0.0066528320312500, 0.0002136230468750,
+0.0065002441406250, 0.0002136230468750, 0.0065002441406250, 0.0000915527343750,
+0.0065917968750000, 0.0000000000000000, 0.0065002441406250, 0.0001220703125000,
+0.0064086914062500, 0.0000610351562500, 0.0063171386718750, -0.0001525878906250,
+0.0062255859375000, 0.0000000000000000, 0.0061950683593750, -0.0000610351562500,
+0.0058898925781250, -0.0003051757812500, 0.0057678222656250, -0.0003662109375000,
+0.0058593750000000, -0.0003356933593750, 0.0048217773437500, -0.0007324218750000,
+0.0052490234375000, -0.0007324218750000, 0.0045471191406250, -0.0006103515625000,
+0.0033874511718750, -0.0008239746093750, 0.0040893554687500, -0.0008239746093750,
+0.0025939941406250, -0.0003662109375000, 0.0027465820312500, -0.0006713867187500,
+0.0026550292968750, -0.0002441406250000, 0.0018005371093750, 0.0003051757812500,
+0.0022583007812500, -0.0001831054687500, 0.0018005371093750, 0.0007629394531250,
+0.0018005371093750, 0.0007324218750000, 0.0018310546875000, 0.0006713867187500,
+0.0016174316406250, 0.0012207031250000, 0.0018310546875000, 0.0011596679687500,
+0.0015258789062500, 0.0010375976562500, 0.0014953613281250, 0.0014038085937500,
+0.0014343261718750, 0.0011901855468750, 0.0013732910156250, 0.0006408691406250,
+0.0012817382812500, 0.0007629394531250, 0.0009765625000000, 0.0006103515625000,
+0.0012817382812500, 0.0004882812500000, 0.0010681152343750, 0.0007324218750000,
+0.0007019042968750, 0.0008239746093750, 0.0011291503906250, 0.0007019042968750,
+0.0004882812500000, 0.0011596679687500, 0.0006103515625000, 0.0013122558593750,
+0.0005493164062500, 0.0013427734375000, 0.0000915527343750, 0.0017089843750000,
+0.0002746582031250, 0.0018005371093750, -0.0000305175781250, 0.0015258789062500,
+-0.0002746582031250, 0.0018615722656250, -0.0002136230468750, 0.0016479492187500,
+-0.0003967285156250, 0.0012207031250000, -0.0005187988281250, 0.0017395019531250,
+-0.0000915527343750, 0.0010375976562500, -0.0003662109375000, 0.0012512207031250,
+-0.0000915527343750, 0.0014038085937500, 0.0003662109375000, 0.0009460449218750,
+-0.0000305175781250, 0.0012207031250000, 0.0007324218750000, 0.0021057128906250,
+0.0006713867187500, 0.0018310546875000, 0.0005798339843750, 0.0025024414062500,
+0.0008850097656250, 0.0034790039062500, 0.0007324218750000, 0.0029602050781250,
+0.0004272460937500, 0.0047912597656250, 0.0007629394531250, 0.0045166015625000,
+0.0003051757812500, 0.0048522949218750, -0.0001831054687500, 0.0061340332031250,
+0.0003967285156250, 0.0055847167968750, -0.0007019042968750, 0.0064392089843750,
+-0.0005493164062500, 0.0065002441406250, -0.0006103515625000, 0.0067138671875000,
+-0.0013732910156250, 0.0073242187500000, -0.0009460449218750, 0.0070190429687500,
+-0.0010681152343750, 0.0074157714843750, -0.0012817382812500, 0.0075988769531250,
+-0.0010070800781250, 0.0074768066406250, -0.0007019042968750, 0.0075378417968750,
+-0.0009765625000000, 0.0076293945312500, -0.0004882812500000, 0.0070495605468750,
+-0.0004882812500000, 0.0071411132812500, -0.0005187988281250, 0.0070190429687500,
+-0.0003051757812500, 0.0065612792968750, -0.0005187988281250, 0.0067443847656250,
+-0.0004272460937500, 0.0068359375000000, -0.0005493164062500, 0.0065917968750000,
+-0.0005493164062500, 0.0068054199218750, -0.0005187988281250, 0.0071105957031250,
+-0.0006713867187500, 0.0068664550781250, -0.0004272460937500, 0.0071716308593750,
+-0.0005187988281250, 0.0072631835937500, -0.0004882812500000, 0.0069580078125000,
+-0.0003356933593750, 0.0068664550781250, -0.0004272460937500, 0.0070800781250000,
+-0.0004272460937500, 0.0063781738281250, -0.0003967285156250, 0.0064392089843750,
+-0.0004882812500000, 0.0064086914062500, -0.0004882812500000, 0.0059204101562500,
+-0.0004577636718750, 0.0061035156250000, -0.0007934570312500, 0.0063171386718750,
+-0.0006408691406250, 0.0060119628906250, -0.0009155273437500, 0.0064697265625000,
+-0.0012817382812500, 0.0069274902343750, -0.0010070800781250, 0.0065612792968750,
+-0.0016784667968750, 0.0076904296875000, -0.0017089843750000, 0.0076293945312500,
+-0.0016174316406250, 0.0079040527343750, -0.0019226074218750, 0.0086669921875000,
+-0.0018615722656250, 0.0083923339843750, -0.0013732910156250, 0.0089721679687500,
+-0.0016784667968750, 0.0091247558593750, -0.0012817382812500, 0.0090026855468750,
+-0.0007019042968750, 0.0092468261718750, -0.0010986328125000, 0.0093078613281250,
+0.0000000000000000, 0.0089721679687500, -0.0001220703125000, 0.0091247558593750,
+0.0000305175781250, 0.0090026855468750, 0.0007934570312500, 0.0086669921875000,
+0.0004882812500000},
+{0.0019531250000000, 0.0004577636718750, 0.0016174316406250, 0.0005798339843750,
+0.0009460449218750, 0.0007629394531250, 0.0011291503906250, 0.0007324218750000,
+0.0010681152343750, 0.0007934570312500, 0.0006713867187500, 0.0010070800781250,
+0.0008850097656250, 0.0010681152343750, 0.0008239746093750, 0.0010681152343750,
+0.0008544921875000, 0.0011596679687500, 0.0009460449218750, 0.0011596679687500,
+0.0010070800781250, 0.0011291503906250, 0.0010375976562500, 0.0013122558593750,
+0.0009765625000000, 0.0012207031250000, 0.0011291503906250, 0.0010681152343750,
+0.0010375976562500, 0.0015258789062500, 0.0008850097656250, 0.0018005371093750,
+0.0009460449218750, 0.0015869140625000, 0.0004882812500000, 0.0030517578125000,
+0.0005493164062500, 0.0026550292968750, 0.0003662109375000, 0.0035400390625000,
+-0.0000915527343750, 0.0050659179687500, 0.0000610351562500, 0.0041809082031250,
+0.0001220703125000, 0.0061340332031250, -0.0001525878906250, 0.0064086914062500,
+0.0001525878906250, 0.0063476562500000, 0.0005493164062500, 0.0072021484375000,
+0.0001220703125000, 0.0070495605468750, 0.0009155273437500, 0.0069274902343750,
+0.0007629394531250, 0.0073547363281250, 0.0007629394531250, 0.0071105957031250,
+0.0012207031250000, 0.0067138671875000, 0.0009765625000000, 0.0071411132812500,
+0.0009460449218750, 0.0063171386718750, 0.0011291503906250, 0.0066528320312500,
+0.0008239746093750, 0.0063781738281250, 0.0005493164062500, 0.0057678222656250,
+0.0008239746093750, 0.0062255859375000, -0.0000305175781250, 0.0050964355468750,
+0.0001525878906250, 0.0052490234375000, -0.0000610351562500, 0.0051574707031250,
+-0.0007324218750000, 0.0044250488281250, -0.0002136230468750, 0.0047912597656250,
+-0.0006713867187500, 0.0045166015625000, -0.0006408691406250, 0.0043334960937500,
+-0.0004272460937500, 0.0044860839843750, -0.0004272460937500, 0.0045776367187500,
+-0.0004577636718750, 0.0045166015625000, -0.0002441406250000, 0.0043029785156250,
+-0.0001831054687500, 0.0043945312500000, -0.0002441406250000, 0.0039062500000000,
+-0.0002136230468750, 0.0035095214843750, -0.0001525878906250, 0.0037841796875000,
+-0.0003051757812500, 0.0025329589843750, -0.0001831054687500, 0.0025634765625000,
+-0.0002136230468750, 0.0023193359375000, -0.0003662109375000, 0.0014648437500000,
+-0.0002136230468750, 0.0018005371093750, -0.0003967285156250, 0.0018310546875000,
+-0.0003356933593750, 0.0014953613281250, -0.0003967285156250, 0.0019226074218750,
+-0.0006103515625000, 0.0023498535156250, -0.0005187988281250, 0.0020751953125000,
+-0.0008239746093750, 0.0029296875000000, -0.0007934570312500, 0.0028381347656250,
+-0.0008544921875000, 0.0028991699218750, -0.0010375976562500, 0.0034484863281250,
+-0.0009765625000000, 0.0032653808593750, -0.0011596679687500, 0.0033569335937500,
+-0.0010681152343750, 0.0034790039062500, -0.0011291503906250, 0.0033264160156250,
+-0.0013122558593750, 0.0031738281250000, -0.0011901855468750, 0.0032958984375000,
+-0.0013732910156250, 0.0029907226562500, -0.0013732910156250, 0.0029602050781250,
+-0.0013427734375000, 0.0031127929687500, -0.0013732910156250, 0.0029907226562500,
+-0.0013427734375000, 0.0028381347656250, -0.0012207031250000, 0.0036010742187500,
+-0.0012207031250000, 0.0031433105468750, -0.0010070800781250, 0.0037231445312500,
+-0.0007934570312500, 0.0045776367187500, -0.0007629394531250, 0.0038146972656250,
+-0.0003356933593750, 0.0055847167968750, -0.0002136230468750, 0.0052490234375000,
+0.0000000000000000, 0.0055541992187500, 0.0003967285156250, 0.0068359375000000,
+0.0004882812500000, 0.0062561035156250, 0.0005798339843750, 0.0072937011718750,
+0.0008850097656250, 0.0073242187500000, 0.0009155273437500, 0.0072937011718750,
+0.0009460449218750, 0.0078430175781250, 0.0011901855468750, 0.0076599121093750,
+0.0008850097656250, 0.0076293945312500, 0.0010681152343750, 0.0079040527343750,
+0.0010986328125000, 0.0075683593750000, 0.0008239746093750, 0.0072631835937500,
+0.0009460449218750, 0.0075378417968750, 0.0008850097656250, 0.0064086914062500,
+0.0007629394531250, 0.0067138671875000, 0.0008544921875000, 0.0061950683593750,
+0.0008544921875000, 0.0052490234375000, 0.0006408691406250, 0.0058898925781250,
+0.0008239746093750, 0.0042724609375000, 0.0007324218750000, 0.0045471191406250,
+0.0006408691406250, 0.0042724609375000, 0.0007934570312500, 0.0032043457031250,
+0.0006713867187500, 0.0038757324218750, 0.0006103515625000, 0.0029296875000000,
+0.0007324218750000, 0.0029907226562500, 0.0005798339843750, 0.0030822753906250,
+0.0004272460937500, 0.0025939941406250, 0.0006408691406250, 0.0028991699218750,
+0.0002136230468750, 0.0026550292968750, 0.0003051757812500, 0.0026550292968750,
+0.0002136230468750, 0.0027770996093750, -0.0000915527343750, 0.0027465820312500,
+0.0000915527343750, 0.0028076171875000, -0.0003662109375000, 0.0025634765625000,
+-0.0001220703125000, 0.0028381347656250, -0.0002441406250000, 0.0024719238281250,
+-0.0005493164062500, 0.0020141601562500, -0.0000610351562500, 0.0023498535156250,
+-0.0006103515625000, 0.0014648437500000, -0.0003051757812500, 0.0015869140625000,
+-0.0002441406250000, 0.0014038085937500, -0.0005493164062500, 0.0007629394531250,
+-0.0000915527343750, 0.0011596679687500, -0.0004882812500000, 0.0009765625000000,
+-0.0003967285156250, 0.0008544921875000, -0.0002746582031250, 0.0012512207031250,
+-0.0005493164062500, 0.0015563964843750, -0.0003967285156250, 0.0014343261718750,
+-0.0003051757812500, 0.0022583007812500, -0.0004272460937500, 0.0023498535156250,
+-0.0002746582031250, 0.0024108886718750, -0.0000610351562500, 0.0029296875000000,
+-0.0002441406250000, 0.0029296875000000, 0.0000915527343750, 0.0027465820312500,
+-0.0000915527343750, 0.0032043457031250, 0.0000000000000000, 0.0030212402343750,
+0.0001220703125000, 0.0026245117187500, -0.0003356933593750, 0.0031127929687500,
+0.0002441406250000, 0.0023193359375000, -0.0000915527343750, 0.0025634765625000,
+-0.0000610351562500, 0.0025024414062500, 0.0002746582031250, 0.0019226074218750,
+-0.0002136230468750, 0.0022277832031250, 0.0003051757812500, 0.0019531250000000,
+0.0003662109375000, 0.0018920898437500, 0.0002136230468750, 0.0021667480468750,
+0.0003967285156250, 0.0021972656250000, 0.0003051757812500, 0.0020751953125000,
+0.0002136230468750, 0.0031433105468750, 0.0002746582031250, 0.0026855468750000,
+0.0003051757812500, 0.0033569335937500, 0.0002746582031250, 0.0045166015625000,
+0.0002746582031250, 0.0037231445312500, 0.0002441406250000, 0.0052185058593750,
+0.0003356933593750, 0.0051574707031250, 0.0001525878906250, 0.0052795410156250,
+-0.0000610351562500, 0.0060119628906250, 0.0002746582031250, 0.0054321289062500,
+0.0001220703125000, 0.0059814453125000, 0.0002746582031250, 0.0057067871093750,
+0.0004577636718750, 0.0056152343750000, 0.0004882812500000, 0.0058898925781250,
+0.0007019042968750, 0.0056457519531250, 0.0008239746093750, 0.0057067871093750,
+0.0009460449218750, 0.0057983398437500, 0.0010986328125000, 0.0054016113281250,
+0.0011901855468750, 0.0050659179687500, 0.0012512207031250, 0.0051879882812500,
+0.0016784667968750, 0.0046081542968750, 0.0015563964843750, 0.0046081542968750,
+0.0020141601562500, 0.0044555664062500, 0.0025939941406250, 0.0042114257812500,
+0.0022277832031250, 0.0043334960937500, 0.0032653808593750, 0.0039367675781250,
+0.0033569335937500, 0.0041503906250000, 0.0032043457031250, 0.0037231445312500,
+0.0036315917968750, 0.0025939941406250, 0.0020446777343750, 0.0042724609375000,
+0.0013122558593750, 0.0045166015625000, 0.0012817382812500, 0.0049438476562500,
+0.0016479492187500, 0.0043945312500000, 0.0017089843750000, 0.0041503906250000,
+0.0015563964843750, 0.0047912597656250, 0.0023498535156250, 0.0029907226562500,
+0.0020751953125000, 0.0035095214843750, 0.0024719238281250, 0.0029907226562500,
+0.0031433105468750, 0.0014648437500000, 0.0025634765625000, 0.0024414062500000,
+0.0040588378906250, 0.0008850097656250, 0.0036926269531250, 0.0009460449218750,
+0.0041503906250000, 0.0009765625000000, 0.0054626464843750, 0.0000610351562500,
+0.0047607421875000, 0.0004577636718750, 0.0057678222656250, 0.0008239746093750,
+0.0058593750000000, 0.0002136230468750, 0.0056762695312500, 0.0010070800781250,
+0.0060729980468750, 0.0019836425781250, 0.0062561035156250, 0.0011596679687500,
+0.0061950683593750, 0.0030212402343750, 0.0061340332031250, 0.0029907226562500,
+0.0067749023437500, 0.0027770996093750, 0.0071411132812500, 0.0036926269531250,
+0.0067749023437500, 0.0034484863281250, 0.0083007812500000, 0.0027465820312500,
+0.0080871582031250, 0.0031433105468750, 0.0081481933593750, 0.0027160644531250,
+0.0090942382812500, 0.0018615722656250, 0.0086975097656250, 0.0021667480468750,
+0.0084533691406250, 0.0012817382812500, 0.0087890625000000, 0.0012512207031250,
+0.0080261230468750, 0.0010681152343750, 0.0073242187500000, 0.0004882812500000,
+0.0079040527343750, 0.0007019042968750, 0.0062866210937500, 0.0002441406250000,
+0.0065917968750000, 0.0001220703125000, 0.0063781738281250, 0.0003051757812500,
+0.0054321289062500, 0.0003356933593750, 0.0061645507812500, 0.0002746582031250,
+0.0050048828125000, 0.0006713867187500, 0.0054626464843750, 0.0007019042968750,
+0.0050659179687500, 0.0006408691406250, 0.0042419433593750, 0.0008239746093750,
+0.0051879882812500, 0.0008544921875000, 0.0029602050781250, 0.0004882812500000,
+0.0038146972656250, 0.0006103515625000, 0.0030517578125000, 0.0003662109375000,
+0.0011291503906250, -0.0000305175781250, 0.0027160644531250, 0.0001220703125000,
+0.0000915527343750, -0.0004272460937500, 0.0003967285156250, -0.0004272460937500,
+0.0003356933593750, -0.0004882812500000, -0.0011901855468750, -0.0009765625000000,
+-0.0002746582031250, -0.0008544921875000, -0.0008850097656250, -0.0008239746093750,
+-0.0010375976562500, -0.0011596679687500, -0.0006408691406250, -0.0008544921875000,
+-0.0007019042968750, -0.0004882812500000, -0.0007324218750000, -0.0009460449218750,
+0.0000610351562500, -0.0001831054687500, -0.0001220703125000, -0.0003356933593750,
+0.0001831054687500, -0.0003967285156250, 0.0008544921875000, 0.0000305175781250,
+0.0003662109375000, -0.0002441406250000, 0.0009155273437500, -0.0000610351562500,
+0.0007324218750000, -0.0001220703125000, 0.0007019042968750, -0.0002441406250000,
+0.0009460449218750, -0.0002136230468750, 0.0003662109375000, -0.0003356933593750,
+0.0013427734375000, -0.0002441406250000, 0.0006103515625000, -0.0005187988281250,
+0.0010070800781250, -0.0004882812500000, 0.0018005371093750, -0.0003356933593750,
+0.0006103515625000, -0.0005798339843750, 0.0024108886718750, -0.0001220703125000,
+0.0017395019531250, -0.0003051757812500, 0.0018005371093750, -0.0001831054687500,
+0.0030212402343750, 0.0002441406250000, 0.0020751953125000, 0.0000305175781250,
+0.0029602050781250, 0.0003662109375000, 0.0027465820312500, 0.0005493164062500,
+0.0025024414062500, 0.0004882812500000, 0.0029296875000000, 0.0005187988281250,
+0.0025024414062500, 0.0006103515625000, 0.0023193359375000, 0.0005187988281250,
+0.0024414062500000, 0.0005187988281250, 0.0022277832031250, 0.0004882812500000,
+0.0018920898437500, 0.0003356933593750, 0.0020751953125000, 0.0003051757812500,
+0.0018005371093750, 0.0002136230468750, 0.0017395019531250, 0.0001220703125000,
+0.0018920898437500, -0.0000305175781250, 0.0018920898437500, -0.0000610351562500,
+0.0018920898437500, -0.0000915527343750, 0.0024414062500000, -0.0001831054687500,
+0.0023803710937500},
+{0.0054931640625000, 0.0010070800781250, 0.0050964355468750, 0.0013732910156250,
+0.0057373046875000, 0.0010070800781250, 0.0062866210937500, -0.0005798339843750,
+0.0058593750000000, 0.0002441406250000, 0.0072326660156250, -0.0003967285156250,
+0.0073852539062500, -0.0009460449218750, 0.0071105957031250, 0.0000915527343750,
+0.0076904296875000, 0.0006103515625000, 0.0075683593750000, -0.0001220703125000,
+0.0066833496093750, 0.0021667480468750, 0.0069580078125000, 0.0019226074218750,
+0.0064697265625000, 0.0020446777343750, 0.0054931640625000, 0.0034179687500000,
+0.0059814453125000, 0.0028076171875000, 0.0049743652343750, 0.0029602050781250,
+0.0048522949218750, 0.0032348632812500, 0.0048828125000000, 0.0027465820312500,
+0.0044860839843750, 0.0025329589843750, 0.0044860839843750, 0.0027160644531250,
+0.0041503906250000, 0.0015869140625000, 0.0043334960937500, 0.0018005371093750,
+0.0039062500000000, 0.0013732910156250, 0.0033569335937500, 0.0004577636718750,
+0.0037841796875000, 0.0010070800781250, 0.0026550292968750, 0.0001831054687500,
+0.0027770996093750, 0.0000305175781250, 0.0026550292968750, 0.0003356933593750,
+0.0019531250000000, 0.0001525878906250, 0.0024108886718750, 0.0001220703125000,
+0.0022888183593750, 0.0005493164062500, 0.0020141601562500, 0.0003967285156250,
+0.0025634765625000, 0.0004882812500000, 0.0029907226562500, 0.0007324218750000,
+0.0026855468750000, 0.0004272460937500, 0.0037841796875000, 0.0004882812500000,
+0.0038757324218750, 0.0004272460937500, 0.0040283203125000, 0.0001220703125000,
+0.0047912597656250, 0.0000000000000000, 0.0048522949218750, -0.0000610351562500,
+0.0048522949218750, -0.0005187988281250, 0.0051879882812500, -0.0005798339843750,
+0.0052795410156250, -0.0006408691406250, 0.0051574707031250, -0.0010375976562500,
+0.0053710937500000, -0.0009460449218750, 0.0053405761718750, -0.0008239746093750,
+0.0053405761718750, -0.0009765625000000, 0.0056457519531250, -0.0007934570312500,
+0.0056762695312500, -0.0004272460937500, 0.0054626464843750, -0.0006103515625000,
+0.0065002441406250, -0.0001525878906250, 0.0058898925781250, -0.0000915527343750,
+0.0065612792968750, -0.0000915527343750, 0.0076599121093750, 0.0001220703125000,
+0.0065612792968750, 0.0001220703125000, 0.0085754394531250, 0.0002441406250000,
+0.0080871582031250, 0.0002441406250000, 0.0081481933593750, 0.0002136230468750,
+0.0094299316406250, 0.0003356933593750, 0.0085144042968750, 0.0003356933593750,
+0.0090332031250000, 0.0003967285156250, 0.0090026855468750, 0.0003967285156250,
+0.0085144042968750, 0.0004577636718750, 0.0084228515625000, 0.0005798339843750,
+0.0083007812500000, 0.0004577636718750, 0.0075378417968750, 0.0006713867187500,
+0.0074768066406250, 0.0007629394531250, 0.0070190429687500, 0.0007324218750000,
+0.0064392089843750, 0.0007934570312500, 0.0065612792968750, 0.0009155273437500,
+0.0056457519531250, 0.0007629394531250, 0.0055236816406250, 0.0008850097656250,
+0.0053405761718750, 0.0007629394531250, 0.0047302246093750, 0.0005798339843750,
+0.0046997070312500, 0.0007934570312500, 0.0043640136718750, 0.0003662109375000,
+0.0041503906250000, 0.0005493164062500, 0.0039367675781250, 0.0004882812500000,
+0.0036621093750000, 0.0001525878906250, 0.0035095214843750, 0.0004882812500000,
+0.0033874511718750, 0.0000305175781250, 0.0033569335937500, 0.0000915527343750,
+0.0033569335937500, 0.0000610351562500, 0.0032653808593750, -0.0002746582031250,
+0.0032043457031250, -0.0002136230468750, 0.0036010742187500, -0.0004577636718750,
+0.0033874511718750, -0.0004272460937500, 0.0037841796875000, -0.0005187988281250,
+0.0043029785156250, -0.0007019042968750, 0.0040588378906250, -0.0006713867187500,
+0.0051879882812500, -0.0007934570312500, 0.0051269531250000, -0.0008239746093750,
+0.0056152343750000, -0.0008239746093750, 0.0065307617187500, -0.0007629394531250,
+0.0062561035156250, -0.0007934570312500, 0.0073547363281250, -0.0007629394531250,
+0.0074768066406250, -0.0006713867187500, 0.0076293945312500, -0.0006103515625000,
+0.0083312988281250, -0.0005187988281250, 0.0081787109375000, -0.0004577636718750,
+0.0083618164062500, -0.0003967285156250, 0.0086364746093750, -0.0003051757812500,
+0.0083618164062500, -0.0003051757812500, 0.0081787109375000, -0.0002746582031250,
+0.0084533691406250, -0.0001525878906250, 0.0075988769531250, -0.0002441406250000,
+0.0078735351562500, -0.0001525878906250, 0.0075683593750000, -0.0000915527343750,
+0.0068969726562500, -0.0001831054687500, 0.0074157714843750, -0.0000915527343750,
+0.0064086914062500, 0.0000000000000000, 0.0065917968750000, -0.0001525878906250,
+0.0064697265625000, -0.0000610351562500, 0.0058288574218750, 0.0000305175781250,
+0.0062866210937500, -0.0002136230468750, 0.0057983398437500, 0.0001525878906250,
+0.0057983398437500, 0.0000305175781250, 0.0058593750000000, 0.0000000000000000,
+0.0057373046875000, 0.0001831054687500, 0.0058593750000000, 0.0000000000000000,
+0.0057067871093750, 0.0001525878906250, 0.0058593750000000, 0.0000610351562500,
+0.0059509277343750, 0.0000915527343750, 0.0058593750000000, 0.0002441406250000,
+0.0061035156250000, 0.0000915527343750, 0.0063171386718750, 0.0002746582031250,
+0.0064086914062500, 0.0001220703125000, 0.0067443847656250, 0.0000610351562500,
+0.0070800781250000, 0.0001220703125000, 0.0071105957031250, 0.0000000000000000,
+0.0077209472656250, -0.0000610351562500, 0.0079040527343750, -0.0001525878906250,
+0.0080261230468750, -0.0001525878906250, 0.0084228515625000, -0.0002136230468750,
+0.0083923339843750, -0.0003051757812500, 0.0079650878906250, -0.0000305175781250,
+0.0083923339843750, -0.0001831054687500, 0.0077819824218750, 0.0000305175781250,
+0.0068969726562500, 0.0003662109375000, 0.0074768066406250, 0.0001525878906250,
+0.0061035156250000, 0.0007324218750000, 0.0060119628906250, 0.0005187988281250,
+0.0058898925781250, 0.0007629394531250, 0.0051574707031250, 0.0012207031250000,
+0.0053100585937500, 0.0007629394531250, 0.0050964355468750, 0.0013122558593750,
+0.0050354003906250, 0.0012817382812500, 0.0047607421875000, 0.0011291503906250,
+0.0046081542968750, 0.0012817382812500, 0.0047607421875000, 0.0012512207031250,
+0.0036315917968750, 0.0010070800781250, 0.0038452148437500, 0.0011291503906250,
+0.0033569335937500, 0.0010986328125000, 0.0023193359375000, 0.0008850097656250,
+0.0027160644531250, 0.0010375976562500, 0.0017395019531250, 0.0012207031250000,
+0.0016479492187500, 0.0011901855468750, 0.0017089843750000, 0.0013732910156250,
+0.0013732910156250, 0.0016479492187500, 0.0014953613281250, 0.0015563964843750,
+0.0014953613281250, 0.0018005371093750, 0.0014648437500000, 0.0017395019531250,
+0.0018310546875000, 0.0017089843750000, 0.0020141601562500, 0.0018920898437500,
+0.0021362304687500, 0.0019531250000000, 0.0034179687500000, 0.0022888183593750,
+0.0028076171875000, 0.0020751953125000, 0.0030517578125000, 0.0024414062500000,
+0.0040893554687500, 0.0028991699218750, 0.0032348632812500, 0.0026550292968750,
+0.0039978027343750, 0.0037231445312500, 0.0040588378906250, 0.0034179687500000,
+0.0033264160156250, 0.0039367675781250, 0.0033569335937500, 0.0048828125000000,
+0.0035095214843750, 0.0041198730468750, 0.0020446777343750, 0.0054321289062500,
+0.0023193359375000, 0.0054321289062500, 0.0018310546875000, 0.0050354003906250,
+0.0007324218750000, 0.0053405761718750, 0.0013427734375000, 0.0051574707031250,
+0.0004272460937500, 0.0046997070312500, 0.0003662109375000, 0.0047607421875000,
+0.0005493164062500, 0.0043029785156250, 0.0001831054687500, 0.0036621093750000,
+0.0003051757812500, 0.0039367675781250, 0.0008544921875000, 0.0032348632812500,
+0.0007324218750000, 0.0031738281250000, 0.0009765625000000, 0.0029602050781250,
+0.0014648437500000, 0.0025329589843750, 0.0012512207031250, 0.0027770996093750,
+0.0014038085937500, 0.0026245117187500, 0.0014648437500000, 0.0025024414062500,
+0.0013427734375000, 0.0027770996093750, 0.0011901855468750, 0.0029602050781250,
+0.0011291503906250, 0.0028076171875000, 0.0010986328125000, 0.0034790039062500,
+0.0008850097656250, 0.0034484863281250, 0.0009155273437500, 0.0036315917968750,
+0.0010375976562500, 0.0041503906250000, 0.0008850097656250, 0.0040588378906250,
+0.0012207031250000, 0.0043029785156250, 0.0010681152343750, 0.0044250488281250,
+0.0012512207031250, 0.0045166015625000, 0.0016784667968750, 0.0045776367187500,
+0.0013732910156250, 0.0046081542968750, 0.0018310546875000, 0.0050964355468750,
+0.0019836425781250, 0.0048522949218750, 0.0017089843750000, 0.0053100585937500,
+0.0019226074218750, 0.0059814453125000, 0.0021362304687500, 0.0054931640625000,
+0.0012512207031250, 0.0067749023437500, 0.0017395019531250, 0.0067749023437500,
+0.0013732910156250, 0.0070495605468750, 0.0005187988281250, 0.0079345703125000,
+0.0012817382812500, 0.0075988769531250, 0.0001525878906250, 0.0081787109375000,
+0.0003967285156250, 0.0084228515625000, 0.0004577636718750, 0.0082397460937500,
+-0.0001220703125000, 0.0083618164062500, 0.0003356933593750, 0.0084838867187500,
+-0.0000305175781250, 0.0077819824218750, 0.0000915527343750, 0.0080261230468750,
+0.0001831054687500, 0.0077819824218750, 0.0000610351562500, 0.0071716308593750,
+0.0003356933593750, 0.0075988769531250, 0.0002136230468750, 0.0070495605468750,
+0.0003051757812500, 0.0070190429687500, 0.0003356933593750, 0.0071105957031250,
+0.0002746582031250, 0.0069274902343750, 0.0005187988281250, 0.0069885253906250,
+0.0000915527343750, 0.0072631835937500, 0.0003967285156250, 0.0071105957031250,
+0.0002441406250000, 0.0072326660156250, -0.0002136230468750, 0.0075378417968750,
+0.0001525878906250, 0.0072021484375000, -0.0004882812500000, 0.0073852539062500,
+-0.0003051757812500, 0.0074768066406250, -0.0003662109375000, 0.0071411132812500,
+-0.0008544921875000, 0.0069580078125000, -0.0004882812500000, 0.0070190429687500,
+-0.0009460449218750, 0.0059204101562500, -0.0008544921875000, 0.0061035156250000,
+-0.0008239746093750, 0.0054931640625000, -0.0010375976562500, 0.0044555664062500,
+-0.0008544921875000, 0.0049743652343750, -0.0008239746093750, 0.0036010742187500,
+-0.0008850097656250, 0.0036315917968750, -0.0007019042968750, 0.0034179687500000,
+-0.0005493164062500, 0.0025634765625000, -0.0005798339843750, 0.0029907226562500,
+-0.0002746582031250, 0.0023803710937500, -0.0004272460937500, 0.0022583007812500,
+-0.0001831054687500, 0.0023193359375000, 0.0001220703125000, 0.0020446777343750,
+-0.0002136230468750, 0.0022888183593750, 0.0005798339843750, 0.0022583007812500,
+0.0003662109375000, 0.0021362304687500, 0.0005493164062500, 0.0023498535156250,
+0.0011596679687500, 0.0025024414062500, 0.0007324218750000, 0.0024108886718750,
+0.0013732910156250, 0.0030822753906250, 0.0013732910156250, 0.0028076171875000,
+0.0012817382812500, 0.0033874511718750, 0.0016479492187500, 0.0040588378906250,
+0.0015258789062500, 0.0035400390625000, 0.0013732910156250, 0.0053405761718750,
+0.0015563964843750, 0.0049133300781250, 0.0013427734375000, 0.0056762695312500,
+0.0010681152343750, 0.0072631835937500, 0.0012817382812500, 0.0062866210937500,
+0.0006713867187500, 0.0083923339843750, 0.0008239746093750, 0.0082702636718750,
+0.0005798339843750, 0.0085449218750000, 0.0001525878906250, 0.0098266601562500,
+0.0004882812500000, 0.0092163085937500, -0.0001220703125000, 0.0100402832031250,
+0.0000610351562500, 0.0102233886718750, 0.0000000000000000, 0.0097351074218750,
+-0.0003356933593750},
+{0.0010070800781250, 0.0065612792968750, 0.0011291503906250, 0.0068969726562500,
+0.0009765625000000, 0.0068054199218750, 0.0008850097656250, 0.0072937011718750,
+0.0008850097656250, 0.0072021484375000, 0.0008850097656250, 0.0074768066406250,
+0.0007019042968750, 0.0079650878906250, 0.0007324218750000, 0.0076904296875000,
+0.0010681152343750, 0.0083312988281250, 0.0008239746093750, 0.0083618164062500,
+0.0011596679687500, 0.0083618164062500, 0.0017395019531250, 0.0087280273437500,
+0.0013122558593750, 0.0085754394531250, 0.0020751953125000, 0.0085449218750000,
+0.0021972656250000, 0.0086364746093750, 0.0020141601562500, 0.0085449218750000,
+0.0023803710937500, 0.0083923339843750, 0.0024719238281250, 0.0084838867187500,
+0.0017700195312500, 0.0083007812500000, 0.0022583007812500, 0.0081787109375000,
+0.0018615722656250, 0.0082397460937500, 0.0010375976562500, 0.0081787109375000,
+0.0018005371093750, 0.0080566406250000, 0.0007019042968750, 0.0083923339843750,
+0.0008544921875000, 0.0082092285156250, 0.0009460449218750, 0.0083007812500000,
+0.0004272460937500, 0.0086059570312500, 0.0007934570312500, 0.0083007812500000,
+0.0005798339843750, 0.0085449218750000, 0.0007019042968750, 0.0085449218750000,
+0.0007324218750000, 0.0083007812500000, 0.0007019042968750, 0.0083618164062500,
+0.0009765625000000, 0.0083312988281250, 0.0006713867187500, 0.0076599121093750,
+0.0010070800781250, 0.0079650878906250, 0.0007934570312500, 0.0074157714843750,
+0.0004577636718750, 0.0066223144531250, 0.0010070800781250, 0.0071105957031250,
+0.0000305175781250, 0.0056762695312500, 0.0003967285156250, 0.0058288574218750,
+0.0002746582031250, 0.0055847167968750, -0.0004577636718750, 0.0045471191406250,
+0.0001220703125000, 0.0050659179687500, -0.0006103515625000, 0.0043334960937500,
+-0.0003967285156250, 0.0043334960937500, -0.0003662109375000, 0.0045166015625000,
+-0.0007934570312500, 0.0042419433593750, -0.0004882812500000, 0.0044250488281250,
+-0.0007629394531250, 0.0046386718750000, -0.0007629394531250, 0.0044860839843750,
+-0.0006103515625000, 0.0047607421875000, -0.0006408691406250, 0.0050354003906250,
+-0.0007019042968750, 0.0048828125000000, -0.0003356933593750, 0.0054626464843750,
+-0.0005493164062500, 0.0054321289062500, -0.0004272460937500, 0.0055847167968750,
+-0.0001525878906250, 0.0059204101562500, -0.0005187988281250, 0.0058593750000000,
+0.0000915527343750, 0.0060424804687500, -0.0001831054687500, 0.0061645507812500,
+-0.0000610351562500, 0.0061645507812500, 0.0003967285156250, 0.0062255859375000,
+-0.0000915527343750, 0.0062561035156250, 0.0005798339843750, 0.0060424804687500,
+0.0005493164062500, 0.0062561035156250, 0.0005187988281250, 0.0059814453125000,
+0.0009765625000000, 0.0055847167968750, 0.0008544921875000, 0.0058593750000000,
+0.0009460449218750, 0.0048828125000000, 0.0010986328125000, 0.0049438476562500,
+0.0009765625000000, 0.0046691894531250, 0.0009155273437500, 0.0039672851562500,
+0.0010681152343750, 0.0041809082031250, 0.0005493164062500, 0.0036926269531250,
+0.0008239746093750, 0.0034179687500000, 0.0006408691406250, 0.0035400390625000,
+0.0001831054687500, 0.0033264160156250, 0.0007019042968750, 0.0031433105468750,
+-0.0000305175781250, 0.0037841796875000, 0.0002746582031250, 0.0033569335937500,
+0.0001831054687500, 0.0038757324218750, -0.0003967285156250, 0.0046386718750000,
+0.0001831054687500, 0.0039978027343750, -0.0005798339843750, 0.0055541992187500,
+-0.0004882812500000, 0.0053710937500000, -0.0004272460937500, 0.0055847167968750,
+-0.0008239746093750, 0.0066528320312500, -0.0006408691406250, 0.0061035156250000,
+-0.0006713867187500, 0.0064392089843750, -0.0007934570312500, 0.0067138671875000,
+-0.0006713867187500, 0.0061645507812500, -0.0006103515625000, 0.0059204101562500,
+-0.0006713867187500, 0.0062561035156250, -0.0003662109375000, 0.0048522949218750,
+-0.0004882812500000, 0.0052185058593750, -0.0001831054687500, 0.0047912597656250,
+0.0001220703125000, 0.0037536621093750, -0.0003356933593750, 0.0044860839843750,
+0.0004577636718750, 0.0036315917968750, 0.0001525878906250, 0.0037536621093750,
+0.0001831054687500, 0.0038452148437500, 0.0006408691406250, 0.0036010742187500,
+0.0000915527343750, 0.0040283203125000, 0.0007324218750000, 0.0038146972656250,
+0.0004882812500000, 0.0042114257812500, 0.0005798339843750, 0.0042419433593750,
+0.0010986328125000, 0.0041503906250000, 0.0007324218750000, 0.0046691894531250,
+0.0013122558593750, 0.0037536621093750, 0.0012512207031250, 0.0043334960937500,
+0.0011901855468750, 0.0039062500000000, 0.0014648437500000, 0.0029602050781250,
+0.0012512207031250, 0.0038452148437500, 0.0013732910156250, 0.0023498535156250,
+0.0012817382812500, 0.0023193359375000, 0.0013122558593750, 0.0024414062500000,
+0.0014038085937500, 0.0017089843750000, 0.0011291503906250, 0.0020446777343750,
+0.0015563964843750, 0.0022583007812500, 0.0013732910156250, 0.0018615722656250,
+0.0014648437500000, 0.0025024414062500, 0.0017700195312500, 0.0031433105468750,
+0.0014038085937500, 0.0025024414062500, 0.0020751953125000, 0.0042114257812500,
+0.0018615722656250, 0.0040588378906250, 0.0020446777343750, 0.0039672851562500,
+0.0025634765625000, 0.0048217773437500, 0.0020446777343750, 0.0045471191406250,
+0.0025329589843750, 0.0043945312500000, 0.0024719238281250, 0.0046386718750000,
+0.0020446777343750, 0.0042724609375000, 0.0020141601562500, 0.0038452148437500,
+0.0019531250000000, 0.0040893554687500, 0.0011291503906250, 0.0041198730468750,
+0.0010986328125000, 0.0039062500000000, 0.0009460449218750, 0.0043334960937500,
+0.0002746582031250, 0.0048828125000000, 0.0003051757812500, 0.0047302246093750,
+0.0006103515625000, 0.0049438476562500, -0.0000610351562500, 0.0054931640625000,
+0.0006408691406250, 0.0051574707031250, 0.0014038085937500, 0.0047302246093750,
+0.0004882812500000, 0.0052490234375000, 0.0025634765625000, 0.0048217773437500,
+0.0022583007812500, 0.0046691894531250, 0.0023498535156250, 0.0048828125000000,
+0.0045166015625000, 0.0048828125000000, 0.0037536621093750, 0.0032958984375000,
+0.0031433105468750, 0.0041198730468750, 0.0032043457031250, 0.0035705566406250,
+0.0033569335937500, 0.0039367675781250, 0.0032348632812500, 0.0047607421875000,
+0.0030212402343750, 0.0043029785156250, 0.0043640136718750, 0.0039978027343750,
+0.0039367675781250, 0.0044860839843750, 0.0045471191406250, 0.0035095214843750,
+0.0060424804687500, 0.0025634765625000, 0.0052795410156250, 0.0032653808593750,
+0.0063171386718750, 0.0015563964843750, 0.0067443847656250, 0.0014343261718750,
+0.0060729980468750, 0.0016174316406250, 0.0060119628906250, 0.0008239746093750,
+0.0065612792968750, 0.0009765625000000, 0.0050048828125000, 0.0016479492187500,
+0.0051269531250000, 0.0012207031250000, 0.0050659179687500, 0.0017395019531250,
+0.0041198730468750, 0.0025024414062500, 0.0043029785156250, 0.0019226074218750,
+0.0045471191406250, 0.0032653808593750, 0.0039672851562500, 0.0033569335937500,
+0.0040283203125000, 0.0031433105468750, 0.0044250488281250, 0.0038757324218750,
+0.0039367675781250, 0.0037536621093750, 0.0040283203125000, 0.0025634765625000,
+0.0039978027343750, 0.0033874511718750, 0.0038452148437500, 0.0023803710937500,
+0.0037231445312500, 0.0006103515625000, 0.0036010742187500, 0.0017089843750000,
+0.0041198730468750, -0.0002441406250000, 0.0036926269531250, -0.0005187988281250,
+0.0043945312500000, -0.0000915527343750, 0.0052185058593750, -0.0007629394531250,
+0.0044555664062500, -0.0007934570312500, 0.0067443847656250, 0.0002136230468750,
+0.0063171386718750, -0.0001525878906250, 0.0068969726562500, 0.0003967285156250,
+0.0086059570312500, 0.0014038085937500, 0.0075988769531250, 0.0007324218750000,
+0.0091857910156250, 0.0019226074218750, 0.0091857910156250, 0.0020141601562500,
+0.0091857910156250, 0.0018920898437500, 0.0099792480468750, 0.0024719238281250,
+0.0095520019531250, 0.0023498535156250, 0.0099792480468750, 0.0018920898437500,
+0.0098876953125000, 0.0022888183593750, 0.0100708007812500, 0.0018310546875000,
+0.0104675292968750, 0.0010681152343750, 0.0101928710937500, 0.0016479492187500,
+0.0108947753906250, 0.0005493164062500, 0.0109252929687500, 0.0006408691406250,
+0.0109863281250000, 0.0007019042968750, 0.0115051269531250, 0.0001525878906250,
+0.0115051269531250, 0.0004882812500000, 0.0113220214843750, 0.0004577636718750,
+0.0118713378906250, 0.0003967285156250, 0.0113830566406250, 0.0005798339843750,
+0.0109252929687500, 0.0007019042968750, 0.0117492675781250, 0.0006103515625000,
+0.0101013183593750, 0.0010070800781250, 0.0106201171875000, 0.0009765625000000,
+0.0102539062500000, 0.0009765625000000, 0.0090332031250000, 0.0011596679687500,
+0.0100402832031250, 0.0010375976562500, 0.0083923339843750, 0.0009460449218750,
+0.0087585449218750, 0.0008850097656250, 0.0085754394531250, 0.0008544921875000,
+0.0075073242187500, 0.0007934570312500, 0.0083007812500000, 0.0007324218750000,
+0.0074157714843750, 0.0008850097656250, 0.0073852539062500, 0.0008239746093750,
+0.0076599121093750, 0.0009155273437500, 0.0073547363281250, 0.0011596679687500,
+0.0074157714843750, 0.0010681152343750, 0.0079345703125000, 0.0013427734375000,
+0.0078125000000000, 0.0013427734375000, 0.0081176757812500, 0.0014953613281250,
+0.0086669921875000, 0.0017089843750000, 0.0082702636718750, 0.0014648437500000,
+0.0089111328125000, 0.0017700195312500, 0.0089416503906250, 0.0017089843750000,
+0.0088806152343750, 0.0015563964843750, 0.0091552734375000, 0.0015258789062500,
+0.0090637207031250, 0.0013732910156250, 0.0087890625000000, 0.0010375976562500,
+0.0089721679687500, 0.0010070800781250, 0.0085449218750000, 0.0006408691406250,
+0.0080871582031250, 0.0003356933593750, 0.0084228515625000, 0.0004272460937500,
+0.0074768066406250, -0.0000915527343750, 0.0075683593750000, -0.0000610351562500,
+0.0074157714843750, -0.0002746582031250, 0.0067443847656250, -0.0006103515625000,
+0.0071105957031250, -0.0003356933593750, 0.0065307617187500, -0.0007629394531250,
+0.0065002441406250, -0.0007324218750000, 0.0065307617187500, -0.0006713867187500,
+0.0063476562500000, -0.0007629394531250, 0.0064086914062500, -0.0006713867187500,
+0.0065002441406250, -0.0005798339843750, 0.0064697265625000, -0.0005798339843750,
+0.0065002441406250, -0.0006103515625000, 0.0065917968750000, -0.0005798339843750,
+0.0066223144531250, -0.0005493164062500, 0.0065002441406250, -0.0005798339843750,
+0.0066223144531250, -0.0004882812500000, 0.0064086914062500, -0.0003356933593750,
+0.0061645507812500, -0.0002746582031250, 0.0062561035156250, -0.0003356933593750,
+0.0055236816406250, -0.0001831054687500, 0.0055541992187500, -0.0001831054687500,
+0.0054016113281250, -0.0001220703125000, 0.0047912597656250, -0.0000610351562500,
+0.0048217773437500, -0.0002441406250000, 0.0046997070312500, -0.0000610351562500,
+0.0043640136718750, -0.0000915527343750, 0.0047607421875000, -0.0000305175781250,
+0.0049743652343750, 0.0001525878906250, 0.0045471191406250, 0.0000305175781250,
+0.0059204101562500, 0.0002441406250000, 0.0055236816406250, 0.0001831054687500,
+0.0061645507812500, 0.0002136230468750, 0.0074768066406250, 0.0003967285156250,
+0.0066528320312500, 0.0003967285156250, 0.0084838867187500, 0.0004577636718750,
+0.0084228515625000, 0.0005798339843750, 0.0085144042968750, 0.0006103515625000,
+0.0095825195312500, 0.0005798339843750, 0.0091857910156250, 0.0007629394531250,
+0.0094909667968750},
+{0.0080871582031250, 0.0014038085937500, 0.0054626464843750, 0.0014953613281250,
+0.0060424804687500, 0.0018005371093750, 0.0053100585937500, 0.0014648437500000,
+0.0032958984375000, 0.0011596679687500, 0.0046997070312500, 0.0015563964843750,
+0.0021667480468750, 0.0008850097656250, 0.0025329589843750, 0.0010070800781250,
+0.0023193359375000, 0.0010681152343750, 0.0008239746093750, 0.0007324218750000,
+0.0020141601562500, 0.0009155273437500, 0.0006713867187500, 0.0008544921875000,
+0.0009460449218750, 0.0007324218750000, 0.0010986328125000, 0.0010070800781250,
+0.0005493164062500, 0.0011596679687500, 0.0012207031250000, 0.0010070800781250,
+0.0006713867187500, 0.0015563964843750, 0.0009460449218750, 0.0014953613281250,
+0.0011901855468750, 0.0015563964843750, 0.0009765625000000, 0.0019836425781250,
+0.0014953613281250, 0.0018310546875000, 0.0016174316406250, 0.0018615722656250,
+0.0018005371093750, 0.0019531250000000, 0.0020446777343750, 0.0018310546875000,
+0.0022583007812500, 0.0017395019531250, 0.0025024414062500, 0.0018615722656250,
+0.0024108886718750, 0.0015563964843750, 0.0028076171875000, 0.0016784667968750,
+0.0027160644531250, 0.0016174316406250, 0.0025634765625000, 0.0013732910156250,
+0.0029907226562500, 0.0015563964843750, 0.0020751953125000, 0.0013122558593750,
+0.0025634765625000, 0.0013427734375000, 0.0023498535156250, 0.0013427734375000,
+0.0015563964843750, 0.0012817382812500, 0.0022277832031250, 0.0012817382812500,
+0.0017395019531250, 0.0013122558593750, 0.0018310546875000, 0.0013122558593750,
+0.0021362304687500, 0.0010681152343750, 0.0021972656250000, 0.0009460449218750,
+0.0023498535156250, 0.0010070800781250, 0.0031127929687500, 0.0004272460937500,
+0.0030212402343750, 0.0005798339843750, 0.0035705566406250, 0.0004577636718750,
+0.0043029785156250, 0.0000305175781250, 0.0040283203125000, 0.0002746582031250,
+0.0052795410156250, -0.0002136230468750, 0.0051269531250000, -0.0002136230468750,
+0.0056152343750000, -0.0002746582031250, 0.0065307617187500, -0.0005798339843750,
+0.0060119628906250, -0.0005493164062500, 0.0072631835937500, -0.0006103515625000,
+0.0070495605468750, -0.0006713867187500, 0.0072937011718750, -0.0006408691406250,
+0.0082397460937500, -0.0006103515625000, 0.0077819824218750, -0.0006103515625000,
+0.0087280273437500, -0.0004882812500000, 0.0087890625000000, -0.0004882812500000,
+0.0088500976562500, -0.0004882812500000, 0.0094604492187500, -0.0003967285156250,
+0.0092468261718750, -0.0003967285156250, 0.0094604492187500, -0.0003356933593750,
+0.0096435546875000, -0.0003051757812500, 0.0093688964843750, -0.0002136230468750,
+0.0092773437500000, -0.0000610351562500, 0.0094604492187500, -0.0000915527343750,
+0.0086669921875000, 0.0001831054687500, 0.0089111328125000, 0.0001525878906250,
+0.0083618164062500, 0.0001831054687500, 0.0075378417968750, 0.0003662109375000,
+0.0079956054687500, 0.0002746582031250, 0.0066223144531250, 0.0003967285156250,
+0.0067443847656250, 0.0004882812500000, 0.0064392089843750, 0.0004882812500000,
+0.0054931640625000, 0.0005187988281250, 0.0059204101562500, 0.0005798339843750,
+0.0053405761718750, 0.0004577636718750, 0.0052185058593750, 0.0004882812500000,
+0.0054016113281250, 0.0004882812500000, 0.0053405761718750, 0.0004272460937500,
+0.0054626464843750, 0.0005187988281250, 0.0058593750000000, 0.0004272460937500,
+0.0058898925781250, 0.0003967285156250, 0.0061950683593750, 0.0003662109375000,
+0.0066833496093750, 0.0003356933593750, 0.0066223144531250, 0.0002746582031250,
+0.0069274902343750, 0.0004272460937500, 0.0071411132812500, 0.0003051757812500,
+0.0070495605468750, 0.0004272460937500, 0.0070190429687500, 0.0006713867187500,
+0.0072021484375000, 0.0004882812500000, 0.0067138671875000, 0.0008850097656250,
+0.0068969726562500, 0.0009155273437500, 0.0066528320312500, 0.0009155273437500,
+0.0062561035156250, 0.0011596679687500, 0.0065612792968750, 0.0011596679687500,
+0.0060729980468750, 0.0010986328125000, 0.0061035156250000, 0.0012512207031250,
+0.0062561035156250, 0.0012207031250000, 0.0061035156250000, 0.0010375976562500,
+0.0061645507812500, 0.0011901855468750, 0.0065612792968750, 0.0010070800781250,
+0.0064086914062500, 0.0010375976562500, 0.0067138671875000, 0.0010681152343750,
+0.0072021484375000, 0.0010070800781250, 0.0069274902343750, 0.0010070800781250,
+0.0075683593750000, 0.0012817382812500, 0.0076293945312500, 0.0012817382812500,
+0.0074768066406250, 0.0012512207031250, 0.0076293945312500, 0.0014648437500000,
+0.0075378417968750, 0.0015869140625000, 0.0072021484375000, 0.0011596679687500,
+0.0072326660156250, 0.0014343261718750, 0.0068359375000000, 0.0014038085937500,
+0.0063781738281250, 0.0009155273437500, 0.0064086914062500, 0.0012207031250000,
+0.0054626464843750, 0.0011596679687500, 0.0057678222656250, 0.0010070800781250,
+0.0050964355468750, 0.0015258789062500, 0.0040893554687500, 0.0018615722656250,
+0.0047607421875000, 0.0014953613281250, 0.0034179687500000, 0.0024719238281250,
+0.0033874511718750, 0.0024108886718750, 0.0035705566406250, 0.0024108886718750,
+0.0029907226562500, 0.0029602050781250, 0.0031433105468750, 0.0028076171875000,
+0.0034790039062500, 0.0028076171875000, 0.0035095214843750, 0.0029602050781250,
+0.0034790039062500, 0.0028076171875000, 0.0038146972656250, 0.0025024414062500,
+0.0041198730468750, 0.0024414062500000, 0.0037231445312500, 0.0023193359375000,
+0.0039672851562500, 0.0022277832031250, 0.0040588378906250, 0.0023498535156250,
+0.0037536621093750, 0.0023193359375000, 0.0039367675781250, 0.0019836425781250,
+0.0037231445312500, 0.0026550292968750, 0.0039367675781250, 0.0024414062500000,
+0.0036926269531250, 0.0025939941406250, 0.0033569335937500, 0.0030517578125000,
+0.0037536621093750, 0.0028381347656250, 0.0025329589843750, 0.0037536621093750,
+0.0026550292968750, 0.0036315917968750, 0.0024108886718750, 0.0040588378906250,
+0.0014648437500000, 0.0047912597656250, 0.0017395019531250, 0.0044860839843750,
+0.0013427734375000, 0.0060729980468750, 0.0011596679687500, 0.0059509277343750,
+0.0013732910156250, 0.0061950683593750, 0.0012512207031250, 0.0072631835937500,
+0.0010681152343750, 0.0069580078125000, 0.0018920898437500, 0.0074157714843750,
+0.0015563964843750, 0.0078125000000000, 0.0017700195312500, 0.0074768066406250,
+0.0025634765625000, 0.0074157714843750, 0.0020141601562500, 0.0077819824218750,
+0.0021972656250000, 0.0072021484375000, 0.0026550292968750, 0.0072631835937500,
+0.0019836425781250, 0.0073852539062500, 0.0016174316406250, 0.0071716308593750,
+0.0022583007812500, 0.0073547363281250, 0.0004577636718750, 0.0071105957031250,
+0.0009155273437500, 0.0073242187500000, 0.0007324218750000, 0.0070800781250000,
+-0.0004882812500000, 0.0067443847656250, 0.0002441406250000, 0.0070495605468750,
+-0.0000610351562500, 0.0061340332031250, -0.0003967285156250, 0.0062866210937500,
+0.0001525878906250, 0.0060424804687500, 0.0004882812500000, 0.0054321289062500,
+0.0002746582031250, 0.0057678222656250, 0.0009460449218750, 0.0048217773437500,
+0.0010070800781250, 0.0049133300781250, 0.0010681152343750, 0.0044860839843750,
+0.0013732910156250, 0.0036926269531250, 0.0013122558593750, 0.0040283203125000,
+0.0014038085937500, 0.0028076171875000, 0.0014648437500000, 0.0026550292968750,
+0.0014343261718750, 0.0023803710937500, 0.0014038085937500, 0.0015258789062500,
+0.0014648437500000, 0.0016174316406250, 0.0015563964843750, 0.0011291503906250,
+0.0015869140625000, 0.0008850097656250, 0.0017089843750000, 0.0007629394531250,
+0.0019226074218750, 0.0005493164062500, 0.0019531250000000, 0.0004882812500000,
+0.0021972656250000, 0.0002441406250000, 0.0024414062500000, 0.0000610351562500,
+0.0023193359375000, -0.0000610351562500, 0.0024108886718750, -0.0003051757812500,
+0.0027160644531250, -0.0004272460937500, 0.0018920898437500, -0.0003967285156250,
+0.0023803710937500, -0.0006713867187500, 0.0020141601562500, -0.0004577636718750,
+0.0011901855468750, -0.0002136230468750, 0.0018920898437500, -0.0004882812500000,
+0.0007324218750000, 0.0004272460937500, 0.0009460449218750, 0.0001220703125000,
+0.0009460449218750, 0.0004882812500000, 0.0003051757812500, 0.0011901855468750,
+0.0007324218750000, 0.0006408691406250, 0.0005187988281250, 0.0018005371093750,
+0.0004882812500000, 0.0016479492187500, 0.0007019042968750, 0.0018615722656250,
+0.0007324218750000, 0.0025939941406250, 0.0007934570312500, 0.0021057128906250,
+0.0009155273437500, 0.0030212402343750, 0.0009155273437500, 0.0028686523437500,
+0.0009155273437500, 0.0030822753906250, 0.0009460449218750, 0.0037841796875000,
+0.0008544921875000, 0.0034179687500000, 0.0007934570312500, 0.0041809082031250,
+0.0006713867187500, 0.0041503906250000, 0.0006408691406250, 0.0042724609375000,
+0.0005493164062500, 0.0047302246093750, 0.0003051757812500, 0.0046081542968750,
+0.0004577636718750, 0.0049133300781250, 0.0002136230468750, 0.0049743652343750,
+0.0001525878906250, 0.0050048828125000, 0.0002746582031250, 0.0052185058593750,
+0.0000305175781250, 0.0052185058593750, 0.0001525878906250, 0.0052185058593750,
+0.0000915527343750, 0.0053405761718750, -0.0000610351562500, 0.0053710937500000,
+-0.0000610351562500, 0.0053710937500000, -0.0000915527343750, 0.0054321289062500,
+-0.0002136230468750, 0.0054016113281250, -0.0001831054687500, 0.0054321289062500,
+-0.0002136230468750, 0.0053710937500000, -0.0002746582031250, 0.0053405761718750,
+-0.0002441406250000, 0.0053710937500000, -0.0002441406250000, 0.0051574707031250,
+-0.0001525878906250, 0.0050964355468750, -0.0001220703125000, 0.0050354003906250,
+-0.0000305175781250, 0.0048522949218750, 0.0000610351562500, 0.0047607421875000,
+0.0001220703125000, 0.0047302246093750, 0.0003051757812500, 0.0044250488281250,
+0.0003051757812500, 0.0045166015625000, 0.0002746582031250, 0.0045166015625000,
+0.0004272460937500, 0.0042114257812500, 0.0002746582031250, 0.0048217773437500,
+0.0003356933593750, 0.0043640136718750, 0.0003662109375000, 0.0045471191406250,
+0.0002441406250000, 0.0051269531250000, 0.0003356933593750, 0.0044860839843750,
+0.0003051757812500, 0.0052490234375000, 0.0002441406250000, 0.0051269531250000,
+0.0003356933593750, 0.0050048828125000, 0.0003356933593750, 0.0053405761718750,
+0.0002441406250000, 0.0051269531250000, 0.0005798339843750, 0.0049743652343750,
+0.0004577636718750, 0.0049743652343750, 0.0005493164062500, 0.0048828125000000,
+0.0007324218750000, 0.0047302246093750, 0.0003662109375000, 0.0047302246093750,
+0.0006713867187500, 0.0047912597656250, 0.0004577636718750, 0.0046081542968750,
+0.0004272460937500, 0.0049438476562500, 0.0006408691406250, 0.0052185058593750,
+0.0002746582031250, 0.0050354003906250, 0.0006103515625000, 0.0058288574218750,
+0.0005187988281250, 0.0057678222656250, 0.0005493164062500, 0.0059509277343750,
+0.0008544921875000, 0.0066223144531250, 0.0007019042968750, 0.0065002441406250,
+0.0010070800781250, 0.0067443847656250, 0.0010070800781250, 0.0068054199218750,
+0.0011596679687500, 0.0067138671875000, 0.0014953613281250, 0.0067138671875000,
+0.0014038085937500, 0.0067443847656250, 0.0018920898437500, 0.0066833496093750,
+0.0018920898437500, 0.0065612792968750, 0.0020141601562500, 0.0067443847656250,
+0.0023803710937500, 0.0066528320312500, 0.0022888183593750, 0.0064086914062500,
+0.0027160644531250, 0.0073852539062500, 0.0028991699218750, 0.0069885253906250,
+0.0029296875000000},
+{0.0018005371093750, 0.0030822753906250, 0.0015869140625000, 0.0029907226562500,
+0.0010986328125000, 0.0030822753906250, 0.0014648437500000, 0.0029907226562500,
+0.0007324218750000, 0.0028686523437500, 0.0008544921875000, 0.0029296875000000,
+0.0008239746093750, 0.0026550292968750, 0.0004272460937500, 0.0025024414062500,
+0.0007629394531250, 0.0026245117187500, 0.0005798339843750, 0.0019226074218750,
+0.0005187988281250, 0.0021057128906250, 0.0007019042968750, 0.0018615722656250,
+0.0007629394531250, 0.0012512207031250, 0.0006408691406250, 0.0015869140625000,
+0.0008544921875000, 0.0010375976562500, 0.0008239746093750, 0.0009765625000000,
+0.0007934570312500, 0.0011596679687500, 0.0008544921875000, 0.0010070800781250,
+0.0007324218750000, 0.0010070800781250, 0.0006103515625000, 0.0016479492187500,
+0.0005798339843750, 0.0014343261718750, 0.0004272460937500, 0.0019226074218750,
+0.0003356933593750, 0.0026550292968750, 0.0002441406250000, 0.0021972656250000,
+0.0000610351562500, 0.0036315917968750, 0.0000000000000000, 0.0034179687500000,
+-0.0001220703125000, 0.0038757324218750, -0.0003051757812500, 0.0049438476562500,
+-0.0002746582031250, 0.0043029785156250, -0.0003967285156250, 0.0057067871093750,
+-0.0003662109375000, 0.0055541992187500, -0.0003967285156250, 0.0056762695312500,
+-0.0004577636718750, 0.0065307617187500, -0.0003051757812500, 0.0060424804687500,
+-0.0004272460937500, 0.0066223144531250, -0.0002441406250000, 0.0066223144531250,
+-0.0002136230468750, 0.0063171386718750, -0.0003051757812500, 0.0065002441406250,
+0.0000000000000000, 0.0064392089843750, -0.0002441406250000, 0.0055847167968750,
+0.0000000000000000, 0.0058593750000000, 0.0000610351562500, 0.0050964355468750,
+-0.0000915527343750, 0.0041503906250000, 0.0001831054687500, 0.0046997070312500,
+0.0001220703125000, 0.0028686523437500, 0.0001220703125000, 0.0030212402343750,
+0.0002746582031250, 0.0025329589843750, 0.0003356933593750, 0.0011291503906250,
+0.0003051757812500, 0.0017700195312500, 0.0006103515625000, 0.0007019042968750,
+0.0005798339843750, 0.0004272460937500, 0.0006713867187500, 0.0007019042968750,
+0.0009155273437500, 0.0004272460937500, 0.0007324218750000, 0.0003356933593750,
+0.0009765625000000, 0.0009155273437500, 0.0009155273437500, 0.0007019042968750,
+0.0008239746093750, 0.0010375976562500, 0.0008850097656250, 0.0015869140625000,
+0.0006408691406250, 0.0012817382812500, 0.0007324218750000, 0.0020446777343750,
+0.0006713867187500, 0.0020446777343750, 0.0005798339843750, 0.0021057128906250,
+0.0007019042968750, 0.0026245117187500, 0.0006713867187500, 0.0023803710937500,
+0.0006408691406250, 0.0027160644531250, 0.0007934570312500, 0.0026855468750000,
+0.0009155273437500, 0.0028381347656250, 0.0009155273437500, 0.0031127929687500,
+0.0010375976562500, 0.0028991699218750, 0.0012512207031250, 0.0034179687500000,
+0.0012512207031250, 0.0034484863281250, 0.0014648437500000, 0.0035705566406250,
+0.0016784667968750, 0.0039062500000000, 0.0016479492187500, 0.0038757324218750,
+0.0021362304687500, 0.0042419433593750, 0.0021667480468750, 0.0042724609375000,
+0.0022583007812500, 0.0043029785156250, 0.0025634765625000, 0.0045471191406250,
+0.0025939941406250, 0.0044250488281250, 0.0025024414062500, 0.0043945312500000,
+0.0028381347656250, 0.0044250488281250, 0.0025939941406250, 0.0043945312500000,
+0.0022888183593750, 0.0042724609375000, 0.0028076171875000, 0.0042724609375000,
+0.0019531250000000, 0.0048217773437500, 0.0020141601562500, 0.0044250488281250,
+0.0023193359375000, 0.0049133300781250, 0.0019836425781250, 0.0056457519531250,
+0.0020751953125000, 0.0051269531250000, 0.0032043457031250, 0.0067138671875000,
+0.0027160644531250, 0.0065002441406250, 0.0034179687500000, 0.0067138671875000,
+0.0047302246093750, 0.0077514648437500, 0.0039367675781250, 0.0073547363281250,
+0.0052490234375000, 0.0078125000000000, 0.0054931640625000, 0.0081481933593750,
+0.0051879882812500, 0.0075683593750000, 0.0055541992187500, 0.0073547363281250,
+0.0055847167968750, 0.0078430175781250, 0.0047302246093750, 0.0063171386718750,
+0.0048828125000000, 0.0064392089843750, 0.0046081542968750, 0.0064697265625000,
+0.0038146972656250, 0.0057067871093750, 0.0039978027343750, 0.0060119628906250,
+0.0039367675781250, 0.0058898925781250, 0.0034179687500000, 0.0060119628906250,
+0.0040893554687500, 0.0057373046875000, 0.0046691894531250, 0.0055236816406250,
+0.0039672851562500, 0.0058898925781250, 0.0056457519531250, 0.0046997070312500,
+0.0053710937500000, 0.0048828125000000, 0.0053100585937500, 0.0048522949218750,
+0.0068359375000000, 0.0036621093750000, 0.0050354003906250, 0.0046997070312500,
+0.0039978027343750, 0.0046081542968750, 0.0039978027343750, 0.0043945312500000,
+0.0042419433593750, 0.0045471191406250, 0.0041198730468750, 0.0046691894531250,
+0.0039367675781250, 0.0043334960937500, 0.0048828125000000, 0.0046997070312500,
+0.0046691894531250, 0.0045776367187500, 0.0047302246093750, 0.0043029785156250,
+0.0054321289062500, 0.0044555664062500, 0.0052490234375000, 0.0044860839843750,
+0.0055541992187500, 0.0035705566406250, 0.0054321289062500, 0.0037841796875000,
+0.0059204101562500, 0.0034790039062500, 0.0063171386718750, 0.0025634765625000,
+0.0058898925781250, 0.0030212402343750, 0.0076293945312500, 0.0027160644531250,
+0.0071716308593750, 0.0023193359375000, 0.0075988769531250, 0.0028381347656250,
+0.0091552734375000, 0.0031127929687500, 0.0085144042968750, 0.0026245117187500,
+0.0089721679687500, 0.0037536621093750, 0.0096740722656250, 0.0036315917968750,
+0.0086975097656250, 0.0037231445312500, 0.0082092285156250, 0.0044555664062500,
+0.0090332031250000, 0.0040893554687500, 0.0065612792968750, 0.0042419433593750,
+0.0072937011718750, 0.0047912597656250, 0.0064392089843750, 0.0040283203125000,
+0.0044250488281250, 0.0034484863281250, 0.0057983398437500, 0.0041809082031250,
+0.0031433105468750, 0.0020751953125000, 0.0033874511718750, 0.0024414062500000,
+0.0031127929687500, 0.0023193359375000, 0.0015869140625000, 0.0008544921875000,
+0.0024414062500000, 0.0014038085937500, 0.0006408691406250, 0.0014953613281250,
+0.0007934570312500, 0.0010375976562500, 0.0005798339843750, 0.0015869140625000,
+-0.0006713867187500, 0.0022583007812500, -0.0000915527343750, 0.0017395019531250,
+-0.0004577636718750, 0.0028076171875000, -0.0009765625000000, 0.0028076171875000,
+-0.0002441406250000, 0.0025939941406250, 0.0000610351562500, 0.0029602050781250,
+-0.0006103515625000, 0.0029296875000000, 0.0015563964843750, 0.0022583007812500,
+0.0009155273437500, 0.0025634765625000, 0.0015563964843750, 0.0019836425781250,
+0.0033264160156250, 0.0012207031250000, 0.0019531250000000, 0.0017395019531250,
+0.0038146972656250, 0.0005493164062500, 0.0037231445312500, 0.0005493164062500,
+0.0036621093750000, 0.0006408691406250, 0.0046386718750000, 0.0000305175781250,
+0.0040283203125000, 0.0003051757812500, 0.0045166015625000, 0.0004882812500000,
+0.0045776367187500, 0.0004272460937500, 0.0042724609375000, 0.0006713867187500,
+0.0043640136718750, 0.0010681152343750, 0.0043334960937500, 0.0010681152343750,
+0.0039062500000000, 0.0013732910156250, 0.0040588378906250, 0.0015869140625000,
+0.0039367675781250, 0.0015869140625000, 0.0035705566406250, 0.0017395019531250,
+0.0037536621093750, 0.0019531250000000, 0.0033874511718750, 0.0016479492187500,
+0.0035400390625000, 0.0019531250000000, 0.0033569335937500, 0.0017700195312500,
+0.0029296875000000, 0.0013122558593750, 0.0032348632812500, 0.0016784667968750,
+0.0028381347656250, 0.0010375976562500, 0.0027160644531250, 0.0010070800781250,
+0.0028991699218750, 0.0009765625000000, 0.0028381347656250, 0.0006408691406250,
+0.0026550292968750, 0.0007324218750000, 0.0036315917968750, 0.0005798339843750,
+0.0031738281250000, 0.0005493164062500, 0.0037841796875000, 0.0005187988281250,
+0.0048217773437500, 0.0004272460937500, 0.0040588378906250, 0.0005798339843750,
+0.0057678222656250, 0.0003051757812500, 0.0056762695312500, 0.0004882812500000,
+0.0058288574218750, 0.0004577636718750, 0.0069885253906250, 0.0002136230468750,
+0.0065307617187500, 0.0004882812500000, 0.0068664550781250, 0.0002746582031250,
+0.0071105957031250, 0.0002136230468750, 0.0066223144531250, 0.0003051757812500,
+0.0065002441406250, 0.0001220703125000, 0.0067443847656250, 0.0001831054687500,
+0.0054626464843750, 0.0003662109375000, 0.0057983398437500, 0.0002136230468750,
+0.0051574707031250, 0.0002136230468750, 0.0039672851562500, 0.0003967285156250,
+0.0046386718750000, 0.0002746582031250, 0.0028991699218750, 0.0002746582031250,
+0.0030517578125000, 0.0003662109375000, 0.0026855468750000, 0.0002136230468750,
+0.0015563964843750, 0.0002136230468750, 0.0021362304687500, 0.0004882812500000,
+0.0013427734375000, -0.0000305175781250, 0.0012207031250000, 0.0003051757812500,
+0.0014343261718750, 0.0002136230468750, 0.0011901855468750, -0.0002136230468750,
+0.0012817382812500, 0.0001525878906250, 0.0017700195312500, -0.0002441406250000,
+0.0015258789062500, -0.0001525878906250, 0.0019226074218750, 0.0000610351562500,
+0.0024719238281250, -0.0000610351562500, 0.0021057128906250, 0.0000000000000000,
+0.0030517578125000, 0.0001525878906250, 0.0030212402343750, 0.0000915527343750,
+0.0032958984375000, 0.0002136230468750, 0.0039672851562500, 0.0003051757812500,
+0.0037841796875000, 0.0002746582031250, 0.0043640136718750, 0.0004272460937500,
+0.0044250488281250, 0.0004882812500000, 0.0044555664062500, 0.0004272460937500,
+0.0046997070312500, 0.0004577636718750, 0.0047302246093750, 0.0005798339843750,
+0.0046386718750000, 0.0004577636718750, 0.0047607421875000, 0.0005187988281250,
+0.0047607421875000, 0.0005493164062500, 0.0046691894531250, 0.0005493164062500,
+0.0047912597656250, 0.0006408691406250, 0.0047912597656250, 0.0005798339843750,
+0.0048217773437500, 0.0006103515625000, 0.0048217773437500, 0.0006713867187500,
+0.0047607421875000, 0.0007324218750000, 0.0047302246093750, 0.0007324218750000,
+0.0046997070312500, 0.0008544921875000, 0.0046691894531250, 0.0009155273437500,
+0.0045166015625000, 0.0009765625000000, 0.0043945312500000, 0.0010070800781250,
+0.0043945312500000, 0.0010681152343750, 0.0042114257812500, 0.0011596679687500,
+0.0040893554687500, 0.0012207031250000, 0.0041198730468750, 0.0012207031250000,
+0.0040588378906250, 0.0012512207031250, 0.0039367675781250, 0.0013427734375000,
+0.0039978027343750, 0.0011596679687500, 0.0039672851562500, 0.0012207031250000,
+0.0039672851562500, 0.0012512207031250, 0.0040588378906250, 0.0010681152343750,
+0.0040588378906250, 0.0010986328125000, 0.0038452148437500, 0.0012512207031250,
+0.0039672851562500, 0.0011291503906250, 0.0036926269531250, 0.0011596679687500,
+0.0033264160156250, 0.0014038085937500, 0.0036621093750000, 0.0012207031250000,
+0.0029296875000000, 0.0012512207031250, 0.0031127929687500, 0.0013732910156250,
+0.0027770996093750, 0.0011291503906250, 0.0021667480468750, 0.0009460449218750,
+0.0026245117187500, 0.0011901855468750, 0.0018920898437500, 0.0008850097656250,
+0.0017395019531250, 0.0007934570312500, 0.0019836425781250, 0.0011901855468750,
+0.0018310546875000, 0.0013732910156250, 0.0018310546875000, 0.0010070800781250,
+0.0026550292968750, 0.0020751953125000, 0.0025329589843750, 0.0020141601562500,
+0.0031738281250000, 0.0022277832031250, 0.0040893554687500, 0.0028991699218750,
+0.0037231445312500},
+{0.0067138671875000, 0.0002441406250000, 0.0075683593750000, 0.0002441406250000,
+0.0057373046875000, 0.0002441406250000, 0.0058898925781250, 0.0002136230468750,
+0.0055541992187500, 0.0002136230468750, 0.0043029785156250, 0.0002136230468750,
+0.0048522949218750, 0.0000915527343750, 0.0038146972656250, 0.0000915527343750,
+0.0037536621093750, 0.0000305175781250, 0.0037231445312500, 0.0000000000000000,
+0.0032043457031250, 0.0000305175781250, 0.0034179687500000, 0.0000610351562500,
+0.0032958984375000, 0.0000610351562500, 0.0031738281250000, 0.0001220703125000,
+0.0032958984375000, 0.0000610351562500, 0.0033569335937500, 0.0000915527343750,
+0.0032958984375000, 0.0003051757812500, 0.0034179687500000, 0.0001220703125000,
+0.0033264160156250, 0.0002441406250000, 0.0032958984375000, 0.0004577636718750,
+0.0032653808593750, 0.0004882812500000, 0.0032043457031250, 0.0004882812500000,
+0.0032043457031250, 0.0007629394531250, 0.0031738281250000, 0.0006713867187500,
+0.0032043457031250, 0.0007324218750000, 0.0032348632812500, 0.0009155273437500,
+0.0030517578125000, 0.0007629394531250, 0.0032348632812500, 0.0009765625000000,
+0.0031127929687500, 0.0009765625000000, 0.0032348632812500, 0.0009155273437500,
+0.0033874511718750, 0.0010070800781250, 0.0031738281250000, 0.0010070800781250,
+0.0037536621093750, 0.0008850097656250, 0.0037231445312500, 0.0009155273437500,
+0.0038757324218750, 0.0008239746093750, 0.0043334960937500, 0.0006103515625000,
+0.0041809082031250, 0.0007019042968750, 0.0045776367187500, 0.0006408691406250,
+0.0047302246093750, 0.0005187988281250, 0.0047912597656250, 0.0006408691406250,
+0.0048828125000000, 0.0006408691406250, 0.0050048828125000, 0.0003662109375000,
+0.0049133300781250, 0.0008850097656250, 0.0049743652343750, 0.0005798339843750,
+0.0049743652343750, 0.0006408691406250, 0.0048828125000000, 0.0010375976562500,
+0.0048828125000000, 0.0007019042968750, 0.0050354003906250, 0.0010986328125000,
+0.0048828125000000, 0.0010070800781250, 0.0048522949218750, 0.0009460449218750,
+0.0050659179687500, 0.0010986328125000, 0.0049133300781250, 0.0009765625000000,
+0.0047912597656250, 0.0009460449218750, 0.0048828125000000, 0.0010375976562500,
+0.0046691894531250, 0.0009460449218750, 0.0043640136718750, 0.0008239746093750,
+0.0043945312500000, 0.0008850097656250, 0.0041503906250000, 0.0010070800781250,
+0.0039672851562500, 0.0008239746093750, 0.0040283203125000, 0.0010070800781250,
+0.0040283203125000, 0.0012207031250000, 0.0039367675781250, 0.0009765625000000,
+0.0042419433593750, 0.0011596679687500, 0.0041503906250000, 0.0012207031250000,
+0.0041809082031250, 0.0010681152343750, 0.0044860839843750, 0.0009765625000000,
+0.0043334960937500, 0.0010375976562500, 0.0041198730468750, 0.0009155273437500,
+0.0042114257812500, 0.0007934570312500, 0.0037536621093750, 0.0010375976562500,
+0.0032348632812500, 0.0010986328125000, 0.0034790039062500, 0.0009155273437500,
+0.0025024414062500, 0.0018310546875000, 0.0025329589843750, 0.0016174316406250,
+0.0020446777343750, 0.0018615722656250, 0.0012207031250000, 0.0026245117187500,
+0.0016479492187500, 0.0022277832031250, 0.0006713867187500, 0.0027465820312500,
+0.0006408691406250, 0.0028076171875000, 0.0005493164062500, 0.0027465820312500,
+-0.0000610351562500, 0.0029296875000000, 0.0001831054687500, 0.0027770996093750,
+0.0005798339843750, 0.0026245117187500, 0.0000000000000000, 0.0026855468750000,
+0.0007324218750000, 0.0024719238281250, 0.0017089843750000, 0.0022888183593750,
+0.0008850097656250, 0.0024108886718750, 0.0027160644531250, 0.0024108886718750,
+0.0025634765625000, 0.0022277832031250, 0.0027160644531250, 0.0025634765625000,
+0.0038146972656250, 0.0028381347656250, 0.0032958984375000, 0.0024719238281250,
+0.0040893554687500, 0.0034790039062500, 0.0041809082031250, 0.0032043457031250,
+0.0041809082031250, 0.0034790039062500, 0.0046691894531250, 0.0043945312500000,
+0.0047302246093750, 0.0041809082031250, 0.0044555664062500, 0.0048522949218750,
+0.0046081542968750, 0.0047607421875000, 0.0043945312500000, 0.0046997070312500,
+0.0040588378906250, 0.0050354003906250, 0.0043640136718750, 0.0049438476562500,
+0.0036621093750000, 0.0046386718750000, 0.0036621093750000, 0.0047912597656250,
+0.0036926269531250, 0.0044250488281250, 0.0034179687500000, 0.0039978027343750,
+0.0035095214843750, 0.0043029785156250, 0.0033569335937500, 0.0033569335937500,
+0.0035095214843750, 0.0034484863281250, 0.0034179687500000, 0.0030212402343750,
+0.0031738281250000, 0.0022888183593750, 0.0034790039062500, 0.0026855468750000,
+0.0029602050781250, 0.0015869140625000, 0.0031738281250000, 0.0016479492187500,
+0.0030517578125000, 0.0014038085937500, 0.0026855468750000, 0.0006103515625000,
+0.0030212402343750, 0.0010070800781250, 0.0026245117187500, 0.0003356933593750,
+0.0027160644531250, 0.0001525878906250, 0.0026550292968750, 0.0004882812500000,
+0.0024414062500000, 0.0003051757812500, 0.0027160644531250, 0.0001831054687500,
+0.0021972656250000, 0.0011901855468750, 0.0023498535156250, 0.0007019042968750,
+0.0022277832031250, 0.0012817382812500, 0.0018920898437500, 0.0021972656250000,
+0.0021667480468750, 0.0014953613281250, 0.0016174316406250, 0.0033569335937500,
+0.0016784667968750, 0.0029602050781250, 0.0016784667968750, 0.0035400390625000,
+0.0013122558593750, 0.0049438476562500, 0.0014953613281250, 0.0040893554687500,
+0.0014953613281250, 0.0059814453125000, 0.0013732910156250, 0.0058898925781250,
+0.0016174316406250, 0.0060729980468750, 0.0017700195312500, 0.0072631835937500,
+0.0016479492187500, 0.0067749023437500, 0.0017089843750000, 0.0072021484375000,
+0.0018615722656250, 0.0074157714843750, 0.0015563964843750, 0.0070190429687500,
+0.0014038085937500, 0.0068054199218750, 0.0014953613281250, 0.0070495605468750,
+0.0007324218750000, 0.0064697265625000, 0.0008544921875000, 0.0063171386718750,
+0.0006103515625000, 0.0064392089843750, 0.0000305175781250, 0.0061645507812500,
+0.0003662109375000, 0.0060729980468750, -0.0002746582031250, 0.0065612792968750,
+-0.0002441406250000, 0.0063476562500000, -0.0001525878906250, 0.0065307617187500,
+-0.0004577636718750, 0.0070495605468750, -0.0004577636718750, 0.0068054199218750,
+-0.0000915527343750, 0.0070800781250000, -0.0004272460937500, 0.0071411132812500,
+-0.0000305175781250, 0.0068664550781250, 0.0004577636718750, 0.0068359375000000,
+-0.0000915527343750, 0.0069885253906250, 0.0009765625000000, 0.0063171386718750,
+0.0007629394531250, 0.0065002441406250, 0.0008544921875000, 0.0062255859375000,
+0.0014648437500000, 0.0056762695312500, 0.0010681152343750, 0.0061035156250000,
+0.0014953613281250, 0.0052185058593750, 0.0014038085937500, 0.0053710937500000,
+0.0014343261718750, 0.0053100585937500, 0.0016784667968750, 0.0046997070312500,
+0.0013732910156250, 0.0050659179687500, 0.0017395019531250, 0.0047607421875000,
+0.0015258789062500, 0.0047302246093750, 0.0014953613281250, 0.0049743652343750,
+0.0017395019531250, 0.0049743652343750, 0.0013427734375000, 0.0050659179687500,
+0.0014953613281250, 0.0053100585937500, 0.0014343261718750, 0.0054321289062500,
+0.0011901855468750, 0.0053710937500000, 0.0011596679687500, 0.0054321289062500,
+0.0011291503906250, 0.0055236816406250, 0.0007934570312500, 0.0050354003906250,
+0.0009155273437500, 0.0053710937500000, 0.0007019042968750, 0.0049133300781250,
+0.0003356933593750, 0.0043029785156250, 0.0006103515625000, 0.0047912597656250,
+0.0002441406250000, 0.0034484863281250, 0.0002746582031250, 0.0035400390625000,
+0.0002441406250000, 0.0031738281250000, 0.0000915527343750, 0.0022583007812500,
+0.0003051757812500, 0.0027770996093750, 0.0002136230468750, 0.0018310546875000,
+0.0002441406250000, 0.0017700195312500, 0.0003356933593750, 0.0019836425781250,
+0.0002746582031250, 0.0015869140625000, 0.0003356933593750, 0.0017089843750000,
+0.0003051757812500, 0.0021057128906250, 0.0003051757812500, 0.0018920898437500,
+0.0003356933593750, 0.0022277832031250, 0.0003356933593750, 0.0027160644531250,
+0.0003051757812500, 0.0023193359375000, 0.0003967285156250, 0.0031738281250000,
+0.0002746582031250, 0.0029602050781250, 0.0003051757812500, 0.0031738281250000,
+0.0003356933593750, 0.0037841796875000, 0.0001525878906250, 0.0033874511718750,
+0.0003662109375000, 0.0040893554687500, 0.0002136230468750, 0.0040283203125000,
+0.0003356933593750, 0.0040588378906250, 0.0004882812500000, 0.0045166015625000,
+0.0002441406250000, 0.0043334960937500, 0.0006408691406250, 0.0047912597656250,
+0.0005493164062500, 0.0047302246093750, 0.0005798339843750, 0.0048522949218750,
+0.0007629394531250, 0.0051574707031250, 0.0005798339843750, 0.0050354003906250,
+0.0008239746093750, 0.0055236816406250, 0.0006713867187500, 0.0055541992187500,
+0.0007324218750000, 0.0057678222656250, 0.0009155273437500, 0.0061950683593750,
+0.0005798339843750, 0.0061035156250000, 0.0009460449218750, 0.0063781738281250,
+0.0008544921875000, 0.0065307617187500, 0.0008544921875000, 0.0063171386718750,
+0.0011596679687500, 0.0062255859375000, 0.0010070800781250, 0.0063476562500000,
+0.0013427734375000, 0.0053710937500000, 0.0013732910156250, 0.0056152343750000,
+0.0014648437500000, 0.0051879882812500, 0.0018005371093750, 0.0042114257812500,
+0.0017089843750000, 0.0046081542968750, 0.0017089843750000, 0.0038452148437500,
+0.0018920898437500, 0.0035705566406250, 0.0016784667968750, 0.0037536621093750,
+0.0014343261718750, 0.0035095214843750, 0.0017089843750000, 0.0033264160156250,
+0.0009765625000000, 0.0038757324218750, 0.0011901855468750, 0.0034484863281250,
+0.0010070800781250, 0.0037841796875000, 0.0004272460937500, 0.0043640136718750,
+0.0008850097656250, 0.0037841796875000, 0.0005187988281250, 0.0049438476562500,
+0.0004272460937500, 0.0047607421875000, 0.0008850097656250, 0.0046997070312500,
+0.0009460449218750, 0.0054016113281250, 0.0007629394531250, 0.0049743652343750,
+0.0022277832031250, 0.0050659179687500, 0.0017395019531250, 0.0050964355468750,
+0.0022888183593750, 0.0047912597656250, 0.0036010742187500, 0.0046386718750000,
+0.0026550292968750, 0.0047302246093750, 0.0037841796875000, 0.0040893554687500,
+0.0039062500000000, 0.0042114257812500, 0.0034484863281250, 0.0039062500000000,
+0.0035705566406250, 0.0033874511718750, 0.0035095214843750, 0.0037841796875000,
+0.0029602050781250, 0.0029602050781250, 0.0027770996093750, 0.0031433105468750,
+0.0028381347656250, 0.0029907226562500, 0.0024108886718750, 0.0024719238281250,
+0.0022888183593750, 0.0028381347656250, 0.0032348632812500, 0.0020446777343750,
+0.0026550292968750, 0.0020446777343750, 0.0032348632812500, 0.0021972656250000,
+0.0043334960937500, 0.0017700195312500, 0.0035095214843750, 0.0018005371093750,
+0.0049438476562500, 0.0028991699218750, 0.0049743652343750, 0.0024414062500000,
+0.0046691894531250, 0.0033569335937500, 0.0053100585937500, 0.0046691894531250,
+0.0052795410156250, 0.0037841796875000, 0.0043640136718750, 0.0056457519531250,
+0.0047607421875000, 0.0056457519531250, 0.0043640136718750, 0.0056457519531250,
+0.0032958984375000, 0.0065917968750000, 0.0037231445312500, 0.0061035156250000,
+0.0031738281250000, 0.0060729980468750, 0.0027465820312500, 0.0062561035156250,
+0.0030822753906250, 0.0058288574218750, 0.0034484863281250, 0.0051574707031250,
+0.0034790039062500, 0.0050659179687500, 0.0038146972656250, 0.0052185058593750,
+0.0033569335937500},
+{-0.0001831054687500, 0.0046691894531250, -0.0000915527343750, 0.0044555664062500,
+0.0000000000000000, 0.0046691894531250, -0.0000305175781250, 0.0049743652343750,
+0.0000000000000000, 0.0045776367187500, 0.0001220703125000, 0.0052490234375000,
+0.0000610351562500, 0.0050964355468750, 0.0001525878906250, 0.0050659179687500,
+0.0002441406250000, 0.0054321289062500, 0.0002136230468750, 0.0050964355468750,
+0.0004272460937500, 0.0053405761718750, 0.0002746582031250, 0.0054016113281250,
+0.0004577636718750, 0.0050964355468750, 0.0006408691406250, 0.0050659179687500,
+0.0003356933593750, 0.0051574707031250, 0.0009460449218750, 0.0046081542968750,
+0.0007324218750000, 0.0046997070312500, 0.0008544921875000, 0.0045776367187500,
+0.0012817382812500, 0.0041198730468750, 0.0009155273437500, 0.0043640136718750,
+0.0016174316406250, 0.0040893554687500, 0.0014953613281250, 0.0039672851562500,
+0.0015563964843750, 0.0041809082031250, 0.0020751953125000, 0.0043029785156250,
+0.0018615722656250, 0.0041809082031250, 0.0019226074218750, 0.0046081542968750,
+0.0021362304687500, 0.0045471191406250, 0.0018920898437500, 0.0046081542968750,
+0.0016174316406250, 0.0048522949218750, 0.0018920898437500, 0.0046691894531250,
+0.0011901855468750, 0.0044250488281250, 0.0012512207031250, 0.0044250488281250,
+0.0010986328125000, 0.0040588378906250, 0.0006408691406250, 0.0037231445312500,
+0.0007934570312500, 0.0037536621093750, 0.0005493164062500, 0.0032653808593750,
+0.0003662109375000, 0.0029296875000000, 0.0007934570312500, 0.0028686523437500,
+0.0008850097656250, 0.0025634765625000, 0.0007019042968750, 0.0022277832031250,
+0.0018005371093750, 0.0026245117187500, 0.0015563964843750, 0.0021667480468750,
+0.0017700195312500, 0.0025024414062500, 0.0026550292968750, 0.0029907226562500,
+0.0022277832031250, 0.0024108886718750, 0.0025939941406250, 0.0036926269531250,
+0.0027160644531250, 0.0034179687500000, 0.0023498535156250, 0.0036621093750000,
+0.0020751953125000, 0.0045166015625000, 0.0021667480468750, 0.0039672851562500,
+0.0016174316406250, 0.0046691894531250, 0.0014343261718750, 0.0047912597656250,
+0.0015258789062500, 0.0046386718750000, 0.0012512207031250, 0.0048522949218750,
+0.0010681152343750, 0.0049438476562500, 0.0020751953125000, 0.0046081542968750,
+0.0015258789062500, 0.0047302246093750, 0.0021057128906250, 0.0045471191406250,
+0.0032348632812500, 0.0043029785156250, 0.0023498535156250, 0.0045471191406250,
+0.0036621093750000, 0.0038757324218750, 0.0037841796875000, 0.0039672851562500,
+0.0035400390625000, 0.0035705566406250, 0.0040893554687500, 0.0028991699218750,
+0.0041198730468750, 0.0032653808593750, 0.0033874511718750, 0.0021362304687500,
+0.0037536621093750, 0.0019531250000000, 0.0034179687500000, 0.0018920898437500,
+0.0025939941406250, 0.0011596679687500, 0.0031433105468750, 0.0012512207031250,
+0.0026550292968750, 0.0016479492187500, 0.0023193359375000, 0.0010681152343750,
+0.0027160644531250, 0.0019531250000000, 0.0028686523437500, 0.0028076171875000,
+0.0024719238281250, 0.0019531250000000, 0.0030822753906250, 0.0042419433593750,
+0.0031127929687500, 0.0039978027343750, 0.0030822753906250, 0.0039978027343750,
+0.0035095214843750, 0.0062561035156250, 0.0036010742187500, 0.0005798339843750,
+0.0046386718750000, 0.0028076171875000, 0.0043334960937500, 0.0020141601562500,
+0.0042419433593750, 0.0024719238281250, 0.0047912597656250, 0.0036010742187500,
+0.0049133300781250, 0.0031433105468750, 0.0032958984375000, 0.0034790039062500,
+0.0040893554687500, 0.0034790039062500, 0.0030212402343750, 0.0034790039062500,
+0.0010986328125000, 0.0036621093750000, 0.0024414062500000, 0.0035095214843750,
+0.0003662109375000, 0.0036926269531250, 0.0000000000000000, 0.0039062500000000,
+0.0004882812500000, 0.0034484863281250, -0.0000610351562500, 0.0033264160156250,
+-0.0001525878906250, 0.0036010742187500, 0.0007324218750000, 0.0023803710937500,
+0.0004272460937500, 0.0025024414062500, 0.0007324218750000, 0.0024414062500000,
+0.0014648437500000, 0.0015869140625000, 0.0010375976562500, 0.0018005371093750,
+0.0020751953125000, 0.0019836425781250, 0.0017395019531250, 0.0017395019531250,
+0.0023193359375000, 0.0019836425781250, 0.0032958984375000, 0.0023498535156250,
+0.0025329589843750, 0.0020751953125000, 0.0043029785156250, 0.0021972656250000,
+0.0041809082031250, 0.0023803710937500, 0.0045471191406250, 0.0019531250000000,
+0.0057373046875000, 0.0016784667968750, 0.0052490234375000, 0.0020141601562500,
+0.0061645507812500, 0.0010375976562500, 0.0063476562500000, 0.0012512207031250,
+0.0063171386718750, 0.0009460449218750, 0.0066223144531250, 0.0002136230468750,
+0.0065307617187500, 0.0007324218750000, 0.0068664550781250, 0.0002441406250000,
+0.0066833496093750, 0.0001525878906250, 0.0067443847656250, 0.0006103515625000,
+0.0069580078125000, 0.0007629394531250, 0.0066528320312500, 0.0006713867187500,
+0.0069580078125000, 0.0015258789062500, 0.0069274902343750, 0.0016174316406250,
+0.0067138671875000, 0.0016174316406250, 0.0067443847656250, 0.0021667480468750,
+0.0066833496093750, 0.0021057128906250, 0.0061340332031250, 0.0016479492187500,
+0.0063476562500000, 0.0018920898437500, 0.0058593750000000, 0.0015869140625000,
+0.0051879882812500, 0.0010375976562500, 0.0056457519531250, 0.0014343261718750,
+0.0048217773437500, 0.0008544921875000, 0.0046691894531250, 0.0007934570312500,
+0.0049743652343750, 0.0009155273437500, 0.0047302246093750, 0.0007019042968750,
+0.0046691894531250, 0.0008239746093750, 0.0055847167968750, 0.0009765625000000,
+0.0053100585937500, 0.0010070800781250, 0.0056762695312500, 0.0010986328125000,
+0.0065612792968750, 0.0013732910156250, 0.0061645507812500, 0.0014953613281250,
+0.0069274902343750, 0.0012207031250000, 0.0072021484375000, 0.0015869140625000,
+0.0069274902343750, 0.0012817382812500, 0.0070495605468750, 0.0008544921875000,
+0.0074157714843750, 0.0013732910156250, 0.0061950683593750, 0.0003051757812500,
+0.0068054199218750, 0.0005493164062500, 0.0061645507812500, 0.0004577636718750,
+0.0050354003906250, -0.0002746582031250, 0.0060729980468750, 0.0001831054687500,
+0.0041198730468750, -0.0001831054687500, 0.0045471191406250, -0.0002746582031250,
+0.0043334960937500, -0.0001525878906250, 0.0030517578125000, -0.0003051757812500,
+0.0040588378906250, -0.0002136230468750, 0.0032958984375000, -0.0001220703125000,
+0.0032653808593750, -0.0001220703125000, 0.0037231445312500, -0.0000915527343750,
+0.0036315917968750, 0.0000000000000000, 0.0036926269531250, -0.0000305175781250,
+0.0043640136718750, -0.0002136230468750, 0.0042724609375000, -0.0000610351562500,
+0.0046386718750000, -0.0001831054687500, 0.0052795410156250, -0.0003967285156250,
+0.0050048828125000, -0.0001831054687500, 0.0057983398437500, -0.0004577636718750,
+0.0057983398437500, -0.0003356933593750, 0.0057983398437500, -0.0002441406250000,
+0.0061950683593750, -0.0003051757812500, 0.0060729980468750, -0.0001831054687500,
+0.0058593750000000, 0.0000915527343750, 0.0059814453125000, -0.0000305175781250,
+0.0057678222656250, 0.0003662109375000, 0.0054626464843750, 0.0007629394531250,
+0.0056152343750000, 0.0004272460937500, 0.0050659179687500, 0.0012817382812500,
+0.0051269531250000, 0.0010986328125000, 0.0049438476562500, 0.0012512207031250,
+0.0045166015625000, 0.0017395019531250, 0.0046386718750000, 0.0012817382812500,
+0.0043029785156250, 0.0017395019531250, 0.0043334960937500, 0.0018310546875000,
+0.0042114257812500, 0.0016174316406250, 0.0039978027343750, 0.0016784667968750,
+0.0040893554687500, 0.0016784667968750, 0.0037231445312500, 0.0014038085937500,
+0.0037536621093750, 0.0014648437500000, 0.0036315917968750, 0.0013122558593750,
+0.0033264160156250, 0.0010681152343750, 0.0035400390625000, 0.0011901855468750,
+0.0031738281250000, 0.0008850097656250, 0.0032653808593750, 0.0008544921875000,
+0.0032348632812500, 0.0009765625000000, 0.0029296875000000, 0.0009460449218750,
+0.0030822753906250, 0.0008239746093750, 0.0029296875000000, 0.0011291503906250,
+0.0028686523437500, 0.0009765625000000, 0.0029296875000000, 0.0009765625000000,
+0.0029602050781250, 0.0011596679687500, 0.0028991699218750, 0.0009765625000000,
+0.0029296875000000, 0.0010681152343750, 0.0028991699218750, 0.0010986328125000,
+0.0029296875000000, 0.0010070800781250, 0.0028686523437500, 0.0010070800781250,
+0.0027770996093750, 0.0011291503906250, 0.0029907226562500, 0.0008850097656250,
+0.0027770996093750, 0.0009155273437500, 0.0028076171875000, 0.0009460449218750,
+0.0030212402343750, 0.0008239746093750, 0.0027770996093750, 0.0008544921875000,
+0.0029296875000000, 0.0009155273437500, 0.0028991699218750, 0.0008239746093750,
+0.0027770996093750, 0.0008850097656250, 0.0027770996093750, 0.0009155273437500,
+0.0026245117187500, 0.0008544921875000, 0.0025634765625000, 0.0009460449218750,
+0.0024108886718750, 0.0009460449218750, 0.0024719238281250, 0.0007629394531250,
+0.0024719238281250, 0.0006713867187500, 0.0021972656250000, 0.0006713867187500,
+0.0027465820312500, 0.0004882812500000, 0.0025024414062500, 0.0003356933593750,
+0.0025024414062500, 0.0005187988281250, 0.0028991699218750, 0.0005493164062500,
+0.0025939941406250, 0.0003356933593750, 0.0027160644531250, 0.0011901855468750,
+0.0028076171875000, 0.0010070800781250, 0.0025939941406250, 0.0012512207031250,
+0.0024108886718750, 0.0019531250000000, 0.0024719238281250, 0.0016479492187500,
+0.0022888183593750, 0.0019531250000000, 0.0021667480468750, 0.0021972656250000,
+0.0024719238281250, 0.0019226074218750, 0.0027160644531250, 0.0018615722656250,
+0.0026245117187500, 0.0020141601562500, 0.0036010742187500, 0.0014953613281250,
+0.0036010742187500, 0.0016479492187500, 0.0038452148437500, 0.0015563964843750,
+0.0045166015625000, 0.0011291503906250, 0.0043334960937500, 0.0013732910156250,
+0.0046691894531250, 0.0012207031250000, 0.0047912597656250, 0.0010681152343750,
+0.0048217773437500, 0.0014648437500000, 0.0048522949218750, 0.0015869140625000,
+0.0047912597656250, 0.0013122558593750, 0.0047302246093750, 0.0023193359375000,
+0.0047912597656250, 0.0020446777343750, 0.0044860839843750, 0.0025329589843750,
+0.0042724609375000, 0.0035095214843750, 0.0044860839843750, 0.0030212402343750,
+0.0034790039062500, 0.0039062500000000, 0.0037231445312500, 0.0040588378906250,
+0.0032653808593750, 0.0038452148437500, 0.0022888183593750, 0.0041503906250000,
+0.0028076171875000, 0.0042724609375000, 0.0016174316406250, 0.0037231445312500,
+0.0015258789062500, 0.0041198730468750, 0.0014953613281250, 0.0036926269531250,
+0.0007629394531250, 0.0028686523437500, 0.0007629394531250, 0.0032653808593750,
+0.0007324218750000, 0.0024414062500000, 0.0006713867187500, 0.0027465820312500,
+0.0008850097656250, 0.0027160644531250, 0.0010070800781250, 0.0021667480468750,
+0.0010375976562500, 0.0024414062500000, 0.0017395019531250, 0.0020141601562500,
+0.0016784667968750, 0.0022277832031250, 0.0020446777343750, 0.0021972656250000,
+0.0027770996093750, 0.0018310546875000, 0.0025329589843750, 0.0021667480468750,
+0.0030212402343750, 0.0021057128906250, 0.0032348632812500, 0.0020141601562500,
+0.0029602050781250, 0.0025024414062500, 0.0030517578125000, 0.0028991699218750,
+0.0033264160156250, 0.0027770996093750, 0.0025939941406250, 0.0037536621093750,
+0.0027465820312500, 0.0038146972656250, 0.0026245117187500, 0.0042114257812500,
+0.0021667480468750},
+{0.0053405761718750, 0.0008850097656250, 0.0054626464843750, 0.0008850097656250,
+0.0051879882812500, 0.0008239746093750, 0.0056152343750000, 0.0007019042968750,
+0.0054626464843750, 0.0007934570312500, 0.0055541992187500, 0.0007019042968750,
+0.0058898925781250, 0.0005187988281250, 0.0056457519531250, 0.0006408691406250,
+0.0059509277343750, 0.0005493164062500, 0.0058288574218750, 0.0005187988281250,
+0.0058288574218750, 0.0004882812500000, 0.0059509277343750, 0.0004882812500000,
+0.0057067871093750, 0.0006103515625000, 0.0059509277343750, 0.0005187988281250,
+0.0057678222656250, 0.0005493164062500, 0.0057678222656250, 0.0005493164062500,
+0.0059509277343750, 0.0004882812500000, 0.0056457519531250, 0.0006103515625000,
+0.0058898925781250, 0.0005187988281250, 0.0058288574218750, 0.0004882812500000,
+0.0056152343750000, 0.0006713867187500, 0.0057067871093750, 0.0007629394531250,
+0.0055236816406250, 0.0006103515625000, 0.0050354003906250, 0.0012512207031250,
+0.0050659179687500, 0.0011596679687500, 0.0046997070312500, 0.0013732910156250,
+0.0042114257812500, 0.0019531250000000, 0.0042724609375000, 0.0018005371093750,
+0.0036010742187500, 0.0018310546875000, 0.0035400390625000, 0.0021972656250000,
+0.0034179687500000, 0.0019531250000000, 0.0030517578125000, 0.0016784667968750,
+0.0030822753906250, 0.0020446777343750, 0.0027160644531250, 0.0014038085937500,
+0.0027465820312500, 0.0014953613281250, 0.0025939941406250, 0.0015563964843750,
+0.0022888183593750, 0.0012512207031250, 0.0024719238281250, 0.0014343261718750,
+0.0022583007812500, 0.0015258789062500, 0.0021667480468750, 0.0014648437500000,
+0.0025024414062500, 0.0017700195312500, 0.0026550292968750, 0.0020141601562500,
+0.0025939941406250, 0.0018310546875000, 0.0034790039062500, 0.0026550292968750,
+0.0034179687500000, 0.0025634765625000, 0.0037841796875000, 0.0028076171875000,
+0.0046081542968750, 0.0034179687500000, 0.0043640136718750, 0.0032043457031250,
+0.0050048828125000, 0.0037231445312500, 0.0051269531250000, 0.0037841796875000,
+0.0052490234375000, 0.0038146972656250, 0.0055541992187500, 0.0040283203125000,
+0.0056152343750000, 0.0040283203125000, 0.0060424804687500, 0.0036621093750000,
+0.0059509277343750, 0.0039672851562500, 0.0060424804687500, 0.0036010742187500,
+0.0064392089843750, 0.0031127929687500, 0.0062561035156250, 0.0036621093750000,
+0.0059204101562500, 0.0025939941406250, 0.0061645507812500, 0.0028686523437500,
+0.0056152343750000, 0.0028686523437500, 0.0045166015625000, 0.0021972656250000,
+0.0041809082031250, 0.0026245117187500, 0.0026550292968750, 0.0022888183593750,
+0.0030822753906250, 0.0023803710937500, 0.0028991699218750, 0.0025634765625000,
+0.0018310546875000, 0.0025024414062500, 0.0022583007812500, 0.0027465820312500,
+0.0012512207031250, 0.0030212402343750, 0.0012207031250000, 0.0030212402343750,
+0.0011291503906250, 0.0033874511718750, 0.0006103515625000, 0.0038452148437500,
+0.0009765625000000, 0.0039367675781250, 0.0008850097656250, 0.0046386718750000,
+0.0006713867187500, 0.0049133300781250, 0.0012207031250000, 0.0050048828125000,
+0.0015869140625000, 0.0054626464843750, 0.0012817382812500, 0.0056762695312500,
+0.0020751953125000, 0.0055847167968750, 0.0021362304687500, 0.0059814453125000,
+0.0021667480468750, 0.0058898925781250, 0.0024414062500000, 0.0057678222656250,
+0.0023193359375000, 0.0061950683593750, 0.0026550292968750, 0.0054626464843750,
+0.0026550292968750, 0.0058593750000000, 0.0026245117187500, 0.0053405761718750,
+0.0027770996093750, 0.0044860839843750, 0.0027465820312500, 0.0052795410156250,
+0.0025634765625000, 0.0033264160156250, 0.0025939941406250, 0.0037536621093750,
+0.0025329589843750, 0.0032958984375000, 0.0024414062500000, 0.0017700195312500,
+0.0025024414062500, 0.0025329589843750, 0.0022888183593750, 0.0017089843750000,
+0.0024108886718750, 0.0014343261718750, 0.0022888183593750, 0.0017089843750000,
+0.0020446777343750, 0.0016174316406250, 0.0022583007812500, 0.0015563964843750,
+0.0018615722656250, 0.0016174316406250, 0.0018920898437500, 0.0016784667968750,
+0.0017089843750000, 0.0015563964843750, 0.0014038085937500, 0.0015258789062500,
+0.0016174316406250, 0.0014953613281250, 0.0009460449218750, 0.0012207031250000,
+0.0011596679687500, 0.0011596679687500, 0.0008850097656250, 0.0010986328125000,
+0.0003967285156250, 0.0009765625000000, 0.0008544921875000, 0.0009460449218750,
+-0.0000610351562500, 0.0008239746093750, 0.0001831054687500, 0.0008239746093750,
+0.0001831054687500, 0.0007019042968750, -0.0003662109375000, 0.0005493164062500,
+-0.0001220703125000, 0.0005493164062500, -0.0000610351562500, 0.0002441406250000,
+-0.0003967285156250, 0.0001831054687500, -0.0000305175781250, 0.0000915527343750,
+0.0002746582031250, -0.0000305175781250, -0.0002136230468750, -0.0000610351562500,
+0.0005798339843750, 0.0001525878906250, 0.0004272460937500, 0.0000305175781250,
+0.0003967285156250, 0.0001525878906250, 0.0008239746093750, 0.0004577636718750,
+0.0004272460937500, 0.0003356933593750, 0.0008239746093750, 0.0003051757812500,
+0.0005493164062500, 0.0004272460937500, 0.0005187988281250, 0.0002746582031250,
+0.0007324218750000, 0.0001220703125000, 0.0002441406250000, 0.0002746582031250,
+0.0006103515625000, -0.0000915527343750, 0.0002441406250000, -0.0000915527343750,
+0.0002441406250000, 0.0000610351562500, 0.0004577636718750, -0.0001220703125000,
+-0.0000610351562500, -0.0001220703125000, 0.0005493164062500, 0.0004882812500000,
+0.0001831054687500, 0.0003051757812500, 0.0002746582031250, 0.0007019042968750,
+0.0007934570312500, 0.0012512207031250, 0.0003356933593750, 0.0009155273437500,
+0.0008239746093750, 0.0020446777343750, 0.0008544921875000, 0.0018005371093750,
+0.0007934570312500, 0.0022277832031250, 0.0010986328125000, 0.0030517578125000,
+0.0011291503906250, 0.0024414062500000, 0.0011291503906250, 0.0038452148437500,
+0.0013122558593750, 0.0036010742187500, 0.0012207031250000, 0.0039367675781250,
+0.0011291503906250, 0.0049743652343750, 0.0014343261718750, 0.0042724609375000,
+0.0009460449218750, 0.0052490234375000, 0.0012207031250000, 0.0051269531250000,
+0.0010986328125000, 0.0051574707031250, 0.0007629394531250, 0.0057067871093750,
+0.0012512207031250, 0.0053100585937500, 0.0006103515625000, 0.0056152343750000,
+0.0008850097656250, 0.0056457519531250, 0.0007934570312500, 0.0054931640625000,
+0.0003051757812500, 0.0054931640625000, 0.0007324218750000, 0.0054016113281250,
+0.0000610351562500, 0.0051269531250000, 0.0001525878906250, 0.0051269531250000,
+0.0000915527343750, 0.0049133300781250, -0.0003662109375000, 0.0046081542968750,
+-0.0000915527343750, 0.0046691894531250, -0.0003356933593750, 0.0043029785156250,
+-0.0003967285156250, 0.0042724609375000, -0.0002441406250000, 0.0042419433593750,
+-0.0002746582031250, 0.0040283203125000, -0.0003967285156250, 0.0041198730468750,
+0.0001220703125000, 0.0039672851562500, -0.0001525878906250, 0.0039978027343750,
+0.0000610351562500, 0.0040588378906250, 0.0004577636718750, 0.0039978027343750,
+-0.0000305175781250, 0.0040893554687500, 0.0007934570312500, 0.0039978027343750,
+0.0005493164062500, 0.0039978027343750, 0.0006103515625000, 0.0041198730468750,
+0.0011596679687500, 0.0041198730468750, 0.0008239746093750, 0.0041503906250000,
+0.0012207031250000, 0.0043640136718750, 0.0013732910156250, 0.0043029785156250,
+0.0011901855468750, 0.0045166015625000, 0.0013427734375000, 0.0047607421875000,
+0.0015563964843750, 0.0045471191406250, 0.0010986328125000, 0.0048828125000000,
+0.0014038085937500, 0.0049438476562500, 0.0012817382812500, 0.0048522949218750,
+0.0008544921875000, 0.0048217773437500, 0.0012512207031250, 0.0047607421875000,
+0.0008239746093750, 0.0044860839843750, 0.0008239746093750, 0.0044555664062500,
+0.0010681152343750, 0.0043945312500000, 0.0010681152343750, 0.0041198730468750,
+0.0011596679687500, 0.0041503906250000, 0.0015869140625000, 0.0043029785156250,
+0.0017395019531250, 0.0040588378906250, 0.0018310546875000, 0.0042724609375000,
+0.0021972656250000, 0.0045166015625000, 0.0024108886718750, 0.0041503906250000,
+0.0020141601562500, 0.0046386718750000, 0.0023803710937500, 0.0045471191406250,
+0.0021057128906250, 0.0044555664062500, 0.0014343261718750, 0.0046997070312500,
+0.0018920898437500, 0.0046386718750000, 0.0010375976562500, 0.0044250488281250,
+0.0009460449218750, 0.0045776367187500, 0.0011901855468750, 0.0043640136718750,
+0.0009155273437500, 0.0041198730468750, 0.0007629394531250, 0.0042724609375000,
+0.0016479492187500, 0.0036621093750000, 0.0014953613281250, 0.0038452148437500,
+0.0017089843750000, 0.0033874511718750, 0.0025024414062500, 0.0027160644531250,
+0.0022888183593750, 0.0032348632812500, 0.0024719238281250, 0.0020446777343750,
+0.0029296875000000, 0.0021362304687500, 0.0025634765625000, 0.0020141601562500,
+0.0023803710937500, 0.0013732910156250, 0.0029602050781250, 0.0017089843750000,
+0.0020141601562500, 0.0012512207031250, 0.0024108886718750, 0.0012512207031250,
+0.0024108886718750, 0.0012512207031250, 0.0018920898437500, 0.0010070800781250,
+0.0024108886718750, 0.0012207031250000, 0.0022888183593750, 0.0012512207031250,
+0.0022888183593750, 0.0010681152343750, 0.0025329589843750, 0.0014343261718750,
+0.0026855468750000, 0.0016479492187500, 0.0028991699218750, 0.0014343261718750,
+0.0028381347656250, 0.0025329589843750, 0.0032348632812500, 0.0022583007812500,
+0.0031433105468750, 0.0028076171875000, 0.0028991699218750, 0.0037231445312500,
+0.0034484863281250, 0.0030517578125000, 0.0030822753906250, 0.0045776367187500,
+0.0032653808593750, 0.0043029785156250, 0.0036621093750000, 0.0045471191406250,
+0.0037841796875000, 0.0056152343750000, 0.0039978027343750, 0.0048522949218750,
+0.0045166015625000, 0.0049133300781250, 0.0048828125000000, 0.0054321289062500,
+0.0046081542968750, 0.0046081542968750, 0.0041503906250000, 0.0031433105468750,
+0.0021362304687500, 0.0039062500000000, 0.0005187988281250, 0.0054321289062500,
+0.0006713867187500, 0.0057678222656250, 0.0011596679687500, 0.0050964355468750,
+0.0010375976562500, 0.0049133300781250, 0.0008239746093750, 0.0054016113281250,
+0.0018310546875000, 0.0040283203125000, 0.0017395019531250, 0.0042114257812500,
+0.0018920898437500, 0.0043334960937500, 0.0025024414062500, 0.0038146972656250,
+0.0022277832031250, 0.0040588378906250, 0.0030517578125000, 0.0036926269531250,
+0.0027770996093750, 0.0039672851562500, 0.0034790039062500, 0.0037231445312500,
+0.0044860839843750, 0.0031433105468750, 0.0038757324218750, 0.0035400390625000,
+0.0056152343750000, 0.0032348632812500, 0.0057373046875000, 0.0029602050781250,
+0.0056457519531250, 0.0033264160156250, 0.0066223144531250, 0.0036926269531250,
+0.0065612792968750, 0.0033569335937500, 0.0061340332031250, 0.0041503906250000,
+0.0065917968750000, 0.0043945312500000, 0.0062866210937500, 0.0041198730468750,
+0.0057678222656250, 0.0043334960937500, 0.0064086914062500, 0.0046081542968750,
+0.0051879882812500, 0.0035705566406250, 0.0058593750000000, 0.0039062500000000,
+0.0052795410156250, 0.0035705566406250, 0.0042724609375000, 0.0026245117187500,
+0.0054016113281250, 0.0030212402343750, 0.0030212402343750, 0.0021667480468750,
+0.0037231445312500, 0.0020446777343750, 0.0030212402343750, 0.0020141601562500,
+0.0011291503906250, 0.0015563964843750, 0.0025939941406250, 0.0016174316406250,
+0.0003356933593750},
+{0.0019226074218750, 0.0037536621093750, 0.0012207031250000, 0.0039062500000000,
+0.0011291503906250, 0.0036621093750000, 0.0014038085937500, 0.0039062500000000,
+0.0013122558593750, 0.0042419433593750, 0.0012207031250000, 0.0038146972656250,
+0.0018920898437500, 0.0046997070312500, 0.0018005371093750, 0.0045471191406250,
+0.0019836425781250, 0.0046997070312500, 0.0025024414062500, 0.0053100585937500,
+0.0024108886718750, 0.0049438476562500, 0.0023803710937500, 0.0055541992187500,
+0.0028076171875000, 0.0055847167968750, 0.0024719238281250, 0.0055847167968750,
+0.0020751953125000, 0.0059509277343750, 0.0026245117187500, 0.0058593750000000,
+0.0017700195312500, 0.0056457519531250, 0.0018920898437500, 0.0059509277343750,
+0.0021057128906250, 0.0056457519531250, 0.0017089843750000, 0.0051574707031250,
+0.0020141601562500, 0.0055847167968750, 0.0022583007812500, 0.0048522949218750,
+0.0022583007812500, 0.0049743652343750, 0.0024719238281250, 0.0048522949218750,
+0.0028076171875000, 0.0043640136718750, 0.0028076171875000, 0.0046386718750000,
+0.0028076171875000, 0.0037231445312500, 0.0031433105468750, 0.0038452148437500,
+0.0029602050781250, 0.0034484863281250, 0.0027465820312500, 0.0025939941406250,
+0.0031738281250000, 0.0028686523437500, 0.0029602050781250, 0.0019531250000000,
+0.0028076171875000, 0.0016174316406250, 0.0031738281250000, 0.0016174316406250,
+0.0034179687500000, 0.0012512207031250, 0.0032653808593750, 0.0010375976562500,
+0.0037841796875000, 0.0013427734375000, 0.0041503906250000, 0.0009460449218750,
+0.0038452148437500, 0.0013122558593750, 0.0040283203125000, 0.0016784667968750,
+0.0045166015625000, 0.0010986328125000, 0.0025634765625000, 0.0029296875000000,
+0.0032653808593750, 0.0023498535156250, 0.0030212402343750, 0.0028076171875000,
+0.0003967285156250, 0.0055236816406250, 0.0010681152343750, 0.0010681152343750,
+0.0024719238281250, 0.0024108886718750, 0.0020446777343750, 0.0013122558593750,
+0.0024414062500000, 0.0025024414062500, 0.0032348632812500, 0.0040893554687500,
+0.0030517578125000, 0.0031738281250000, 0.0036926269531250, 0.0045471191406250,
+0.0034484863281250, 0.0048522949218750, 0.0039672851562500, 0.0045471191406250,
+0.0046997070312500, 0.0048217773437500, 0.0041809082031250, 0.0048522949218750,
+0.0055541992187500, 0.0049133300781250, 0.0057678222656250, 0.0048828125000000,
+0.0054626464843750, 0.0049133300781250, 0.0061035156250000, 0.0051269531250000,
+0.0062561035156250, 0.0052490234375000, 0.0047302246093750, 0.0046386718750000,
+0.0053405761718750, 0.0050659179687500, 0.0046691894531250, 0.0046081542968750,
+0.0031738281250000, 0.0037231445312500, 0.0041198730468750, 0.0043334960937500,
+0.0025329589843750, 0.0033874511718750, 0.0026245117187500, 0.0031127929687500,
+0.0023193359375000, 0.0034790039062500, 0.0013427734375000, 0.0033569335937500,
+0.0020446777343750, 0.0030517578125000, 0.0004882812500000, 0.0037841796875000,
+0.0007019042968750, 0.0037536621093750, 0.0005798339843750, 0.0036010742187500,
+-0.0004882812500000, 0.0039367675781250, 0.0000610351562500, 0.0038757324218750,
+-0.0002136230468750, 0.0034790039062500, -0.0004577636718750, 0.0036926269531250,
+0.0001831054687500, 0.0033264160156250, 0.0004577636718750, 0.0028686523437500,
+0.0001525878906250, 0.0032958984375000, 0.0018310546875000, 0.0022888183593750,
+0.0016479492187500, 0.0025329589843750, 0.0019531250000000, 0.0024719238281250,
+0.0032653808593750, 0.0018310546875000, 0.0028381347656250, 0.0022583007812500,
+0.0034484863281250, 0.0018615722656250, 0.0038452148437500, 0.0018615722656250,
+0.0034790039062500, 0.0019531250000000, 0.0034484863281250, 0.0018615722656250,
+0.0037841796875000, 0.0020141601562500, 0.0030212402343750, 0.0016784667968750,
+0.0031738281250000, 0.0019531250000000, 0.0031433105468750, 0.0017395019531250,
+0.0026245117187500, 0.0013122558593750, 0.0028686523437500, 0.0016479492187500,
+0.0029296875000000, 0.0010375976562500, 0.0027465820312500, 0.0010070800781250,
+0.0032653808593750, 0.0009460449218750, 0.0036010742187500, 0.0005798339843750,
+0.0032958984375000, 0.0007019042968750, 0.0044250488281250, 0.0004577636718750,
+0.0041198730468750, 0.0004272460937500, 0.0044860839843750, 0.0003967285156250,
+0.0054321289062500, 0.0002746582031250, 0.0048522949218750, 0.0002441406250000,
+0.0057373046875000, 0.0001525878906250, 0.0056762695312500, 0.0000915527343750,
+0.0055847167968750, 0.0000305175781250, 0.0058593750000000, 0.0000305175781250,
+0.0055236816406250, 0.0000610351562500, 0.0056457519531250, -0.0003356933593750,
+0.0055236816406250, -0.0001831054687500, 0.0052795410156250, -0.0002746582031250,
+0.0052490234375000, -0.0006103515625000, 0.0052490234375000, -0.0004272460937500,
+0.0046997070312500, -0.0005798339843750, 0.0047607421875000, -0.0007324218750000,
+0.0043334960937500, -0.0006408691406250, 0.0038146972656250, -0.0006103515625000,
+0.0040283203125000, -0.0008239746093750, 0.0031433105468750, -0.0003662109375000,
+0.0031127929687500, -0.0005493164062500, 0.0029907226562500, -0.0003967285156250,
+0.0023498535156250, 0.0000610351562500, 0.0024414062500000, -0.0001831054687500,
+0.0026245117187500, 0.0000915527343750, 0.0020141601562500, 0.0002136230468750,
+0.0025939941406250, 0.0000610351562500, 0.0032653808593750, 0.0000305175781250,
+0.0025024414062500, 0.0002441406250000, 0.0040588378906250, -0.0000305175781250,
+0.0036621093750000, 0.0000610351562500, 0.0039367675781250, 0.0001220703125000,
+0.0050048828125000, -0.0000610351562500, 0.0041503906250000, 0.0000610351562500,
+0.0050964355468750, 0.0001220703125000, 0.0050964355468750, 0.0000305175781250,
+0.0049133300781250, 0.0002441406250000, 0.0053405761718750, 0.0003662109375000,
+0.0051574707031250, 0.0001220703125000, 0.0050354003906250, 0.0005493164062500,
+0.0052795410156250, 0.0005187988281250, 0.0051269531250000, 0.0004272460937500,
+0.0049133300781250, 0.0006713867187500, 0.0051879882812500, 0.0007019042968750,
+0.0047912597656250, 0.0006408691406250, 0.0050048828125000, 0.0007019042968750,
+0.0049743652343750, 0.0007934570312500, 0.0047302246093750, 0.0008239746093750,
+0.0049743652343750, 0.0007934570312500, 0.0044555664062500, 0.0009460449218750,
+0.0046386718750000, 0.0008239746093750, 0.0044555664062500, 0.0008239746093750,
+0.0040283203125000, 0.0009155273437500, 0.0043945312500000, 0.0008239746093750,
+0.0039367675781250, 0.0008850097656250, 0.0039672851562500, 0.0009155273437500,
+0.0039367675781250, 0.0008544921875000, 0.0036926269531250, 0.0008850097656250,
+0.0038757324218750, 0.0008239746093750, 0.0037231445312500, 0.0007324218750000,
+0.0036926269531250, 0.0007629394531250, 0.0036315917968750, 0.0006408691406250,
+0.0035095214843750, 0.0005187988281250, 0.0035400390625000, 0.0007019042968750,
+0.0032653808593750, 0.0004272460937500, 0.0032348632812500, 0.0005798339843750,
+0.0030212402343750, 0.0004882812500000, 0.0026855468750000, 0.0002746582031250,
+0.0027770996093750, 0.0005493164062500, 0.0023193359375000, 0.0003051757812500,
+0.0022277832031250, 0.0002746582031250, 0.0021972656250000, 0.0003967285156250,
+0.0019531250000000, 0.0003662109375000, 0.0020751953125000, 0.0003356933593750,
+0.0022277832031250, 0.0009155273437500, 0.0020751953125000, 0.0007019042968750,
+0.0022583007812500, 0.0008850097656250, 0.0025939941406250, 0.0014648437500000,
+0.0025329589843750, 0.0010986328125000, 0.0025634765625000, 0.0014953613281250,
+0.0027770996093750, 0.0016174316406250, 0.0025939941406250, 0.0014038085937500,
+0.0024108886718750, 0.0013122558593750, 0.0026245117187500, 0.0014648437500000,
+0.0022583007812500, 0.0010681152343750, 0.0022277832031250, 0.0010070800781250,
+0.0023193359375000, 0.0010986328125000, 0.0021667480468750, 0.0009155273437500,
+0.0022583007812500, 0.0007629394531250, 0.0026550292968750, 0.0011901855468750,
+0.0025939941406250, 0.0008850097656250, 0.0028686523437500, 0.0010986328125000,
+0.0033264160156250, 0.0015258789062500, 0.0031738281250000, 0.0010986328125000,
+0.0035705566406250, 0.0020751953125000, 0.0036621093750000, 0.0017700195312500,
+0.0035095214843750, 0.0021362304687500, 0.0034790039062500, 0.0029296875000000,
+0.0036621093750000, 0.0023498535156250, 0.0039367675781250, 0.0032348632812500,
+0.0037841796875000, 0.0031738281250000, 0.0041809082031250, 0.0031738281250000,
+0.0046386718750000, 0.0036010742187500, 0.0043334960937500, 0.0032653808593750,
+0.0050048828125000, 0.0035400390625000, 0.0052490234375000, 0.0035400390625000,
+0.0050964355468750, 0.0034484863281250, 0.0052185058593750, 0.0035400390625000,
+0.0053710937500000, 0.0035705566406250, 0.0051879882812500, 0.0032348632812500,
+0.0051879882812500, 0.0034790039062500, 0.0051879882812500, 0.0031433105468750,
+0.0050048828125000, 0.0026245117187500, 0.0050048828125000, 0.0030517578125000,
+0.0049133300781250, 0.0025939941406250, 0.0048217773437500, 0.0024719238281250,
+0.0046691894531250, 0.0027160644531250, 0.0044555664062500, 0.0026855468750000,
+0.0045166015625000, 0.0027160644531250, 0.0040893554687500, 0.0034179687500000,
+0.0039062500000000, 0.0032958984375000, 0.0039672851562500, 0.0035705566406250,
+0.0038146972656250, 0.0043945312500000, 0.0036315917968750, 0.0044555664062500,
+0.0043029785156250, 0.0049743652343750, 0.0040588378906250, 0.0050659179687500,
+0.0040893554687500, 0.0049438476562500, 0.0046386718750000, 0.0051269531250000,
+0.0045166015625000, 0.0053405761718750, 0.0047912597656250, 0.0044860839843750,
+0.0048522949218750, 0.0048217773437500, 0.0050048828125000, 0.0045166015625000,
+0.0051574707031250, 0.0037231445312500, 0.0049743652343750, 0.0042419433593750,
+0.0054016113281250, 0.0033569335937500, 0.0053405761718750, 0.0033569335937500,
+0.0050659179687500, 0.0031433105468750, 0.0051879882812500, 0.0025939941406250,
+0.0051879882812500, 0.0028686523437500, 0.0045471191406250, 0.0019531250000000,
+0.0046386718750000, 0.0019226074218750, 0.0043334960937500, 0.0017700195312500,
+0.0038146972656250, 0.0011596679687500, 0.0040893554687500, 0.0013732910156250,
+0.0035705566406250, 0.0009765625000000, 0.0035095214843750, 0.0008239746093750,
+0.0034790039062500, 0.0009765625000000, 0.0032348632812500, 0.0008850097656250,
+0.0033569335937500, 0.0008850097656250, 0.0032653808593750, 0.0014953613281250,
+0.0030517578125000, 0.0012817382812500, 0.0032043457031250, 0.0015869140625000,
+0.0032348632812500, 0.0022277832031250, 0.0029296875000000, 0.0019836425781250,
+0.0032958984375000, 0.0028686523437500, 0.0030517578125000, 0.0028381347656250,
+0.0029602050781250, 0.0030822753906250, 0.0032348632812500, 0.0037841796875000,
+0.0029602050781250, 0.0036010742187500, 0.0028076171875000, 0.0041503906250000,
+0.0028991699218750, 0.0043334960937500, 0.0026245117187500, 0.0042724609375000,
+0.0023193359375000, 0.0044555664062500, 0.0024108886718750, 0.0045471191406250,
+0.0022583007812500, 0.0045166015625000, 0.0021057128906250, 0.0046081542968750,
+0.0020141601562500, 0.0048522949218750, 0.0019531250000000, 0.0049133300781250,
+0.0018920898437500, 0.0050354003906250, 0.0017089843750000, 0.0054626464843750,
+0.0016784667968750, 0.0056762695312500, 0.0014953613281250, 0.0059204101562500,
+0.0013122558593750, 0.0063171386718750, 0.0014038085937500, 0.0064697265625000,
+0.0009155273437500, 0.0062255859375000, 0.0010070800781250, 0.0065917968750000,
+0.0007934570312500},
+{0.0034179687500000, 0.0026550292968750, 0.0031738281250000, 0.0027465820312500,
+0.0030822753906250, 0.0032348632812500, 0.0032043457031250, 0.0029907226562500,
+0.0025939941406250, 0.0031738281250000, 0.0027160644531250, 0.0031738281250000,
+0.0025939941406250, 0.0030822753906250, 0.0020751953125000, 0.0032043457031250,
+0.0023498535156250, 0.0031127929687500, 0.0022888183593750, 0.0030212402343750,
+0.0021362304687500, 0.0031433105468750, 0.0025329589843750, 0.0030822753906250,
+0.0027770996093750, 0.0028686523437500, 0.0025939941406250, 0.0030212402343750,
+0.0034484863281250, 0.0026245117187500, 0.0031127929687500, 0.0027160644531250,
+0.0034484863281250, 0.0028381347656250, 0.0042419433593750, 0.0025939941406250,
+0.0037536621093750, 0.0026550292968750, 0.0044860839843750, 0.0030822753906250,
+0.0043640136718750, 0.0029602050781250, 0.0046081542968750, 0.0033264160156250,
+0.0050964355468750, 0.0038757324218750, 0.0046386718750000, 0.0035400390625000,
+0.0054016113281250, 0.0043029785156250, 0.0052795410156250, 0.0043334960937500,
+0.0052795410156250, 0.0043029785156250, 0.0054931640625000, 0.0048217773437500,
+0.0047912597656250, 0.0048217773437500, 0.0048217773437500, 0.0040588378906250,
+0.0047302246093750, 0.0045471191406250, 0.0044860839843750, 0.0041809082031250,
+0.0044860839843750, 0.0033874511718750, 0.0042724609375000, 0.0040588378906250,
+0.0041503906250000, 0.0029602050781250, 0.0041809082031250, 0.0028991699218750,
+0.0041198730468750, 0.0029296875000000, 0.0040588378906250, 0.0024414062500000,
+0.0041503906250000, 0.0026245117187500, 0.0041809082031250, 0.0022888183593750,
+0.0040893554687500, 0.0023498535156250, 0.0043640136718750, 0.0022277832031250,
+0.0046386718750000, 0.0018920898437500, 0.0043640136718750, 0.0019226074218750,
+0.0048217773437500, 0.0019531250000000, 0.0048522949218750, 0.0017700195312500,
+0.0046691894531250, 0.0019531250000000, 0.0047302246093750, 0.0021057128906250,
+0.0047912597656250, 0.0019531250000000, 0.0042419433593750, 0.0027770996093750,
+0.0044250488281250, 0.0024414062500000, 0.0041198730468750, 0.0030212402343750,
+0.0036315917968750, 0.0039062500000000, 0.0039367675781250, 0.0033264160156250,
+0.0032653808593750, 0.0048828125000000, 0.0033569335937500, 0.0047607421875000,
+0.0032348632812500, 0.0050659179687500, 0.0027160644531250, 0.0061035156250000,
+0.0028991699218750, 0.0056152343750000, 0.0026550292968750, 0.0062866210937500,
+0.0023803710937500, 0.0064086914062500, 0.0025024414062500, 0.0061340332031250,
+0.0025329589843750, 0.0061950683593750, 0.0022277832031250, 0.0063171386718750,
+0.0024719238281250, 0.0060119628906250, 0.0025024414062500, 0.0060729980468750,
+0.0022277832031250, 0.0062255859375000, 0.0021972656250000, 0.0061035156250000,
+0.0022888183593750, 0.0061340332031250, 0.0017395019531250, 0.0066528320312500,
+0.0017700195312500, 0.0066528320312500, 0.0017089843750000, 0.0067443847656250,
+0.0012817382812500, 0.0071716308593750, 0.0013732910156250, 0.0071411132812500,
+0.0013122558593750, 0.0068969726562500, 0.0011596679687500, 0.0071411132812500,
+0.0012207031250000, 0.0069274902343750, 0.0012817382812500, 0.0065307617187500,
+0.0010375976562500, 0.0068969726562500, 0.0013732910156250, 0.0063781738281250,
+0.0012817382812500, 0.0064086914062500, 0.0012512207031250, 0.0065612792968750,
+0.0013427734375000, 0.0065307617187500, 0.0011291503906250, 0.0066833496093750,
+0.0011291503906250, 0.0067749023437500, 0.0010681152343750, 0.0069274902343750,
+0.0008239746093750, 0.0068664550781250, 0.0006408691406250, 0.0069580078125000,
+0.0006103515625000, 0.0071716308593750, 0.0001831054687500, 0.0068054199218750,
+0.0000610351562500, 0.0069885253906250, -0.0000305175781250, 0.0069274902343750,
+-0.0002441406250000, 0.0066528320312500, -0.0003356933593750, 0.0069580078125000,
+-0.0002746582031250, 0.0065307617187500, -0.0003051757812500, 0.0065917968750000,
+-0.0002136230468750, 0.0066528320312500, -0.0000305175781250, 0.0064086914062500,
+-0.0000305175781250, 0.0065307617187500, 0.0000610351562500, 0.0064086914062500,
+0.0002136230468750, 0.0064086914062500, 0.0002441406250000, 0.0063171386718750,
+0.0002136230468750, 0.0061340332031250, 0.0003662109375000, 0.0061950683593750,
+0.0002136230468750, 0.0060424804687500, 0.0003356933593750, 0.0059204101562500,
+0.0003356933593750, 0.0060424804687500, 0.0001220703125000, 0.0061340332031250,
+0.0001831054687500, 0.0059814453125000, 0.0000610351562500, 0.0064392089843750,
+0.0000000000000000, 0.0064086914062500, 0.0000305175781250, 0.0063476562500000,
+0.0000305175781250, 0.0066223144531250, -0.0000305175781250, 0.0065917968750000,
+0.0000915527343750, 0.0060729980468750, 0.0000610351562500, 0.0063476562500000,
+0.0001220703125000, 0.0057373046875000, 0.0002441406250000, 0.0050048828125000,
+0.0000610351562500, 0.0054321289062500, 0.0003662109375000, 0.0036621093750000,
+0.0002441406250000, 0.0039978027343750, 0.0004272460937500, 0.0032348632812500,
+0.0006713867187500, 0.0017395019531250, 0.0004272460937500, 0.0024719238281250,
+0.0009460449218750, 0.0005798339843750, 0.0007934570312500, 0.0007324218750000,
+0.0008850097656250, 0.0003967285156250, 0.0012207031250000, -0.0007934570312500,
+0.0008850097656250, -0.0001831054687500, 0.0012817382812500, -0.0008239746093750,
+0.0011596679687500, -0.0010375976562500, 0.0010375976562500, -0.0005493164062500,
+0.0012512207031250, -0.0005187988281250, 0.0011291503906250, -0.0006713867187500,
+0.0010070800781250, 0.0005493164062500, 0.0010681152343750, 0.0003356933593750,
+0.0009765625000000, 0.0010070800781250, 0.0008850097656250, 0.0021972656250000,
+0.0010070800781250, 0.0016479492187500, 0.0009460449218750, 0.0033264160156250,
+0.0009460449218750, 0.0032958984375000, 0.0010070800781250, 0.0036621093750000,
+0.0010681152343750, 0.0048828125000000, 0.0010681152343750, 0.0043640136718750,
+0.0011291503906250, 0.0052490234375000, 0.0011901855468750, 0.0054321289062500,
+0.0011291503906250, 0.0053100585937500, 0.0011291503906250, 0.0055847167968750,
+0.0012207031250000, 0.0054931640625000, 0.0010070800781250, 0.0050048828125000,
+0.0011291503906250, 0.0053100585937500, 0.0010681152343750, 0.0047607421875000,
+0.0008850097656250, 0.0039672851562500, 0.0011291503906250, 0.0044555664062500,
+0.0012207031250000, 0.0032348632812500, 0.0011291503906250, 0.0032348632812500,
+0.0014343261718750, 0.0031738281250000, 0.0018310546875000, 0.0024108886718750,
+0.0016174316406250, 0.0026550292968750, 0.0023803710937500, 0.0027465820312500,
+0.0023498535156250, 0.0025634765625000, 0.0023803710937500, 0.0027465820312500,
+0.0028076171875000, 0.0030212402343750, 0.0026550292968750, 0.0028381347656250,
+0.0025329589843750, 0.0031127929687500, 0.0026855468750000, 0.0032043457031250,
+0.0024414062500000, 0.0029602050781250, 0.0021362304687500, 0.0029907226562500,
+0.0024108886718750, 0.0031433105468750, 0.0019531250000000, 0.0024108886718750,
+0.0020751953125000, 0.0026245117187500, 0.0021362304687500, 0.0023498535156250,
+0.0018920898437500, 0.0016174316406250, 0.0021057128906250, 0.0020446777343750,
+0.0020751953125000, 0.0013122558593750, 0.0020751953125000, 0.0012512207031250,
+0.0021362304687500, 0.0011901855468750, 0.0021362304687500, 0.0007629394531250,
+0.0022583007812500, 0.0009460449218750, 0.0021667480468750, 0.0005187988281250,
+0.0023498535156250, 0.0003967285156250, 0.0022583007812500, 0.0006408691406250,
+0.0021362304687500, 0.0006103515625000, 0.0024108886718750, 0.0004577636718750,
+0.0025024414062500, 0.0007324218750000, 0.0024414062500000, 0.0007324218750000,
+0.0027465820312500, 0.0006713867187500, 0.0031127929687500, 0.0007934570312500,
+0.0029602050781250, 0.0007629394531250, 0.0031738281250000, 0.0007019042968750,
+0.0035705566406250, 0.0005798339843750, 0.0031433105468750, 0.0009765625000000,
+0.0029602050781250, 0.0012207031250000, 0.0035095214843750, 0.0009460449218750,
+0.0019226074218750, 0.0029296875000000, 0.0023193359375000, 0.0024108886718750,
+0.0020446777343750, 0.0034179687500000, 0.0007629394531250, 0.0054626464843750,
+0.0015258789062500, 0.0043945312500000, 0.0011596679687500, 0.0066223144531250,
+0.0007629394531250, 0.0067443847656250, 0.0014648437500000, 0.0066528320312500,
+0.0018310546875000, 0.0076293945312500, 0.0013122558593750, 0.0071716308593750,
+0.0028381347656250, 0.0069885253906250, 0.0026855468750000, 0.0073547363281250,
+0.0026855468750000, 0.0066833496093750, 0.0040893554687500, 0.0052795410156250,
+0.0054016113281250, 0.0053710937500000, 0.0051574707031250, 0.0063171386718750,
+0.0060729980468750, 0.0065002441406250, 0.0051574707031250, 0.0059509277343750,
+0.0043029785156250, 0.0058898925781250, 0.0051879882812500, 0.0062866210937500,
+0.0030212402343750, 0.0043945312500000, 0.0033264160156250, 0.0048828125000000,
+0.0029907226562500, 0.0044860839843750, 0.0015869140625000, 0.0029907226562500,
+0.0023193359375000, 0.0038452148437500, 0.0015563964843750, 0.0028686523437500,
+0.0013427734375000, 0.0027770996093750, 0.0014953613281250, 0.0028381347656250,
+0.0012817382812500, 0.0023803710937500, 0.0011901855468750, 0.0025939941406250,
+0.0013732910156250, 0.0022277832031250, 0.0011901855468750, 0.0022277832031250,
+0.0013427734375000, 0.0023193359375000, 0.0015563964843750, 0.0020751953125000,
+0.0013427734375000, 0.0020446777343750, 0.0022583007812500, 0.0028381347656250,
+0.0019531250000000, 0.0026245117187500, 0.0022888183593750, 0.0028381347656250,
+0.0030517578125000, 0.0035400390625000, 0.0026550292968750, 0.0032348632812500,
+0.0034179687500000, 0.0035095214843750, 0.0032653808593750, 0.0036926269531250,
+0.0032958984375000, 0.0034484863281250, 0.0036621093750000, 0.0033264160156250,
+0.0032653808593750, 0.0034484863281250, 0.0036926269531250, 0.0031127929687500,
+0.0034179687500000, 0.0030517578125000, 0.0034790039062500, 0.0031127929687500,
+0.0037536621093750, 0.0028991699218750, 0.0032958984375000, 0.0028991699218750,
+0.0044860839843750, 0.0030212402343750, 0.0040893554687500, 0.0029907226562500,
+0.0045166015625000, 0.0029907226562500, 0.0055236816406250, 0.0030212402343750,
+0.0047912597656250, 0.0030212402343750, 0.0064697265625000, 0.0030212402343750,
+0.0062866210937500, 0.0029907226562500, 0.0065002441406250, 0.0030212402343750,
+0.0075988769531250, 0.0030822753906250, 0.0070800781250000, 0.0029907226562500,
+0.0074462890625000, 0.0029296875000000, 0.0075988769531250, 0.0028991699218750,
+0.0073242187500000, 0.0027465820312500, 0.0072937011718750, 0.0026550292968750,
+0.0073852539062500, 0.0026855468750000, 0.0068664550781250, 0.0023193359375000,
+0.0068969726562500, 0.0023193359375000, 0.0067749023437500, 0.0021057128906250,
+0.0064392089843750, 0.0018005371093750, 0.0066833496093750, 0.0018920898437500,
+0.0060119628906250, 0.0014038085937500, 0.0063476562500000, 0.0014038085937500,
+0.0059204101562500, 0.0012817382812500, 0.0052490234375000, 0.0009155273437500,
+0.0059204101562500, 0.0010681152343750, 0.0043945312500000, 0.0008850097656250,
+0.0047302246093750, 0.0008544921875000, 0.0045166015625000, 0.0008850097656250,
+0.0032958984375000, 0.0008239746093750, 0.0040283203125000, 0.0008850097656250,
+0.0036010742187500, 0.0009460449218750, 0.0031433105468750, 0.0008544921875000,
+0.0038452148437500, 0.0009765625000000, 0.0041809082031250, 0.0011596679687500,
+0.0036010742187500},
+{0.0023193359375000, 0.0000915527343750, 0.0017700195312500, 0.0003967285156250,
+0.0030822753906250, 0.0012512207031250, 0.0029296875000000, 0.0007019042968750,
+0.0032043457031250, 0.0017089843750000, 0.0039978027343750, 0.0030822753906250,
+0.0035400390625000, 0.0023193359375000, 0.0046081542968750, 0.0044250488281250,
+0.0046997070312500, 0.0043334960937500, 0.0047607421875000, 0.0044860839843750,
+0.0056457519531250, 0.0064392089843750, 0.0046081542968750, 0.0009460449218750,
+0.0019531250000000, 0.0027770996093750, 0.0028076171875000, 0.0021057128906250,
+0.0026245117187500, 0.0027770996093750, 0.0014648437500000, 0.0038146972656250,
+0.0018310546875000, 0.0031738281250000, 0.0021057128906250, 0.0043334960937500,
+0.0019836425781250, 0.0043945312500000, 0.0021362304687500, 0.0042114257812500,
+0.0025634765625000, 0.0047607421875000, 0.0025939941406250, 0.0046691894531250,
+0.0024108886718750, 0.0043029785156250, 0.0026550292968750, 0.0044860839843750,
+0.0026855468750000, 0.0043640136718750, 0.0024108886718750, 0.0039367675781250,
+0.0025634765625000, 0.0041503906250000, 0.0030822753906250, 0.0038452148437500,
+0.0028686523437500, 0.0039672851562500, 0.0032958984375000, 0.0036926269531250,
+0.0039672851562500, 0.0034790039062500, 0.0036315917968750, 0.0037841796875000,
+0.0041809082031250, 0.0029602050781250, 0.0042114257812500, 0.0030517578125000,
+0.0040893554687500, 0.0030212402343750, 0.0040893554687500, 0.0024108886718750,
+0.0039978027343750, 0.0025329589843750, 0.0041809082031250, 0.0027160644531250,
+0.0038452148437500, 0.0023803710937500, 0.0041809082031250, 0.0027160644531250,
+0.0045776367187500, 0.0031738281250000, 0.0040588378906250, 0.0026550292968750,
+0.0053710937500000, 0.0034179687500000, 0.0050354003906250, 0.0032958984375000,
+0.0053405761718750, 0.0032348632812500, 0.0063781738281250, 0.0036010742187500,
+0.0056762695312500, 0.0033264160156250, 0.0063476562500000, 0.0034179687500000,
+0.0065002441406250, 0.0033264160156250, 0.0061950683593750, 0.0034179687500000,
+0.0063476562500000, 0.0034790039062500, 0.0063171386718750, 0.0032653808593750,
+0.0055541992187500, 0.0034790039062500, 0.0059204101562500, 0.0033569335937500,
+0.0053710937500000, 0.0032958984375000, 0.0045166015625000, 0.0033569335937500,
+0.0051574707031250, 0.0031738281250000, 0.0038146972656250, 0.0032348632812500,
+0.0040283203125000, 0.0031738281250000, 0.0037841796875000, 0.0031127929687500,
+0.0029907226562500, 0.0031433105468750, 0.0036010742187500, 0.0031433105468750,
+0.0024108886718750, 0.0030212402343750, 0.0026550292968750, 0.0031738281250000,
+0.0023498535156250, 0.0029907226562500, 0.0014648437500000, 0.0028381347656250,
+0.0020751953125000, 0.0030822753906250, 0.0008544921875000, 0.0026245117187500,
+0.0008850097656250, 0.0028076171875000, 0.0008850097656250, 0.0026245117187500,
+0.0001220703125000, 0.0022583007812500, 0.0003356933593750, 0.0026245117187500,
+0.0007629394531250, 0.0017700195312500, 0.0002136230468750, 0.0019836425781250,
+0.0010986328125000, 0.0018615722656250, 0.0020446777343750, 0.0013427734375000,
+0.0012512207031250, 0.0016174316406250, 0.0033874511718750, 0.0013122558593750,
+0.0032653808593750, 0.0013122558593750, 0.0034484863281250, 0.0014038085937500,
+0.0047912597656250, 0.0013122558593750, 0.0042114257812500, 0.0014343261718750,
+0.0047302246093750, 0.0016174316406250, 0.0050659179687500, 0.0016784667968750,
+0.0045471191406250, 0.0017700195312500, 0.0044555664062500, 0.0019836425781250,
+0.0049743652343750, 0.0020751953125000, 0.0033264160156250, 0.0020141601562500,
+0.0039062500000000, 0.0021057128906250, 0.0035095214843750, 0.0020751953125000,
+0.0022583007812500, 0.0019226074218750, 0.0032348632812500, 0.0019531250000000,
+0.0020751953125000, 0.0017395019531250, 0.0021057128906250, 0.0016479492187500,
+0.0023803710937500, 0.0014953613281250, 0.0019531250000000, 0.0012817382812500,
+0.0021667480468750, 0.0012512207031250, 0.0022583007812500, 0.0009765625000000,
+0.0023498535156250, 0.0008850097656250, 0.0025329589843750, 0.0007934570312500,
+0.0027160644531250, 0.0006713867187500, 0.0027770996093750, 0.0006408691406250,
+0.0030517578125000, 0.0006713867187500, 0.0031433105468750, 0.0006103515625000,
+0.0032653808593750, 0.0006713867187500, 0.0035705566406250, 0.0007019042968750,
+0.0036621093750000, 0.0006713867187500, 0.0038757324218750, 0.0006103515625000,
+0.0039978027343750, 0.0006713867187500, 0.0041809082031250, 0.0006408691406250,
+0.0043945312500000, 0.0004882812500000, 0.0043640136718750, 0.0004272460937500,
+0.0046081542968750, 0.0005493164062500, 0.0046997070312500, 0.0004577636718750,
+0.0047607421875000, 0.0004577636718750, 0.0048217773437500, 0.0005798339843750,
+0.0048217773437500, 0.0006408691406250, 0.0047302246093750, 0.0006713867187500,
+0.0047607421875000, 0.0007324218750000, 0.0046386718750000, 0.0007324218750000,
+0.0044860839843750, 0.0007934570312500, 0.0044860839843750, 0.0008850097656250,
+0.0041198730468750, 0.0007324218750000, 0.0040893554687500, 0.0008544921875000,
+0.0039978027343750, 0.0009460449218750, 0.0036621093750000, 0.0008850097656250,
+0.0036926269531250, 0.0009765625000000, 0.0034484863281250, 0.0013122558593750,
+0.0033874511718750, 0.0012817382812500, 0.0033874511718750, 0.0014953613281250,
+0.0032653808593750, 0.0018920898437500, 0.0032043457031250, 0.0017700195312500,
+0.0034790039062500, 0.0019531250000000, 0.0032653808593750, 0.0021057128906250,
+0.0035095214843750, 0.0019531250000000, 0.0039367675781250, 0.0018920898437500,
+0.0036010742187500, 0.0021362304687500, 0.0043334960937500, 0.0015258789062500,
+0.0041503906250000, 0.0017395019531250, 0.0042724609375000, 0.0015869140625000,
+0.0047912597656250, 0.0012207031250000, 0.0044250488281250, 0.0014343261718750,
+0.0050659179687500, 0.0012207031250000, 0.0049133300781250, 0.0011901855468750,
+0.0051269531250000, 0.0013427734375000, 0.0056152343750000, 0.0012817382812500,
+0.0052795410156250, 0.0012512207031250, 0.0058593750000000, 0.0016479492187500,
+0.0059204101562500, 0.0014953613281250, 0.0058593750000000, 0.0016784667968750,
+0.0061035156250000, 0.0021362304687500, 0.0061035156250000, 0.0018615722656250,
+0.0057373046875000, 0.0024108886718750, 0.0059204101562500, 0.0024414062500000,
+0.0056762695312500, 0.0025024414062500, 0.0051879882812500, 0.0027465820312500,
+0.0054321289062500, 0.0027770996093750, 0.0049438476562500, 0.0026855468750000,
+0.0047607421875000, 0.0029296875000000, 0.0048828125000000, 0.0028686523437500,
+0.0048217773437500, 0.0026550292968750, 0.0046691894531250, 0.0029602050781250,
+0.0050964355468750, 0.0025939941406250, 0.0048828125000000, 0.0027465820312500,
+0.0050354003906250, 0.0026855468750000, 0.0054626464843750, 0.0023498535156250,
+0.0051574707031250, 0.0026855468750000, 0.0055541992187500, 0.0021667480468750,
+0.0054931640625000, 0.0023193359375000, 0.0054016113281250, 0.0024414062500000,
+0.0055236816406250, 0.0021057128906250, 0.0053405761718750, 0.0023193359375000,
+0.0053100585937500, 0.0025024414062500, 0.0052490234375000, 0.0023803710937500,
+0.0052490234375000, 0.0027160644531250, 0.0052490234375000, 0.0031433105468750,
+0.0051574707031250, 0.0029602050781250, 0.0050659179687500, 0.0036926269531250,
+0.0053405761718750, 0.0038146972656250, 0.0050048828125000, 0.0037536621093750,
+0.0045776367187500, 0.0041198730468750, 0.0050048828125000, 0.0042724609375000,
+0.0042419433593750, 0.0039062500000000, 0.0042419433593750, 0.0041809082031250,
+0.0041198730468750, 0.0037841796875000, 0.0036926269531250, 0.0032653808593750,
+0.0040283203125000, 0.0037841796875000, 0.0031738281250000, 0.0030212402343750,
+0.0032653808593750, 0.0032348632812500, 0.0030212402343750, 0.0031738281250000,
+0.0024414062500000, 0.0026550292968750, 0.0028686523437500, 0.0031127929687500,
+0.0028381347656250, 0.0026855468750000, 0.0026855468750000, 0.0028076171875000,
+0.0032348632812500, 0.0028991699218750, 0.0036315917968750, 0.0026245117187500,
+0.0034484863281250, 0.0028381347656250, 0.0045776367187500, 0.0030822753906250,
+0.0045776367187500, 0.0030212402343750, 0.0046691894531250, 0.0033874511718750,
+0.0053710937500000, 0.0037231445312500, 0.0052490234375000, 0.0035705566406250,
+0.0050048828125000, 0.0045471191406250, 0.0054016113281250, 0.0043945312500000,
+0.0050048828125000, 0.0048217773437500, 0.0045471191406250, 0.0057067871093750,
+0.0050964355468750, 0.0053100585937500, 0.0038146972656250, 0.0061645507812500,
+0.0040283203125000, 0.0062561035156250, 0.0038146972656250, 0.0061035156250000,
+0.0028991699218750, 0.0064392089843750, 0.0032653808593750, 0.0064697265625000,
+0.0030822753906250, 0.0059204101562500, 0.0027160644531250, 0.0061950683593750,
+0.0030212402343750, 0.0058288574218750, 0.0032348632812500, 0.0051879882812500,
+0.0028076171875000, 0.0056762695312500, 0.0036315917968750, 0.0049133300781250,
+0.0034484863281250, 0.0049743652343750, 0.0034484863281250, 0.0050964355468750,
+0.0040588378906250, 0.0048217773437500, 0.0038146972656250, 0.0050354003906250,
+0.0037536621093750, 0.0050354003906250, 0.0040283203125000, 0.0051574707031250,
+0.0038452148437500, 0.0048522949218750, 0.0036315917968750, 0.0046386718750000,
+0.0039062500000000, 0.0047912597656250, 0.0034179687500000, 0.0037841796875000,
+0.0035095214843750, 0.0039367675781250, 0.0034179687500000, 0.0034484863281250,
+0.0030822753906250, 0.0025634765625000, 0.0033569335937500, 0.0030517578125000,
+0.0032348632812500, 0.0021057128906250, 0.0032043457031250, 0.0019531250000000,
+0.0033569335937500, 0.0020751953125000, 0.0035400390625000, 0.0018310546875000,
+0.0036621093750000, 0.0018310546875000, 0.0036926269531250, 0.0017700195312500,
+0.0039367675781250, 0.0018310546875000, 0.0038757324218750, 0.0016479492187500,
+0.0038146972656250, 0.0014953613281250, 0.0041198730468750, 0.0016174316406250,
+0.0035095214843750, 0.0012207031250000, 0.0038757324218750, 0.0011291503906250,
+0.0036315917968750, 0.0012817382812500, 0.0030822753906250, 0.0013122558593750,
+0.0036315917968750, 0.0013122558593750, 0.0026550292968750, 0.0016174316406250,
+0.0029296875000000, 0.0016479492187500, 0.0028686523437500, 0.0016479492187500,
+0.0022277832031250, 0.0018005371093750, 0.0027770996093750, 0.0018310546875000,
+0.0023498535156250, 0.0017089843750000, 0.0024108886718750, 0.0017089843750000,
+0.0025329589843750, 0.0017395019531250, 0.0023803710937500, 0.0016174316406250,
+0.0025939941406250, 0.0016479492187500, 0.0024414062500000, 0.0018005371093750,
+0.0026245117187500, 0.0018005371093750, 0.0024414062500000, 0.0019836425781250,
+0.0021057128906250, 0.0022583007812500, 0.0023498535156250, 0.0021362304687500,
+0.0015563964843750, 0.0025024414062500, 0.0016174316406250, 0.0025634765625000,
+0.0014343261718750, 0.0024719238281250, 0.0009155273437500, 0.0025329589843750,
+0.0010986328125000, 0.0024719238281250, 0.0008544921875000, 0.0021362304687500,
+0.0007324218750000, 0.0021362304687500, 0.0007934570312500, 0.0020141601562500,
+0.0008239746093750, 0.0016479492187500, 0.0008239746093750, 0.0017395019531250,
+0.0009155273437500, 0.0018005371093750, 0.0009765625000000, 0.0015258789062500,
+0.0010070800781250, 0.0020141601562500, 0.0010681152343750, 0.0024719238281250,
+0.0010986328125000, 0.0020751953125000, 0.0011291503906250, 0.0035400390625000,
+0.0011291503906250},
+{0.0026245117187500, 0.0048828125000000, 0.0022277832031250, 0.0047607421875000,
+0.0027160644531250, 0.0049438476562500, 0.0027770996093750, 0.0058898925781250,
+0.0024414062500000, 0.0054626464843750, 0.0032958984375000, 0.0058593750000000,
+0.0031738281250000, 0.0060729980468750, 0.0033569335937500, 0.0057983398437500,
+0.0040283203125000, 0.0057373046875000, 0.0037536621093750, 0.0057983398437500,
+0.0041809082031250, 0.0051269531250000, 0.0043640136718750, 0.0052490234375000,
+0.0040893554687500, 0.0049133300781250, 0.0040283203125000, 0.0042724609375000,
+0.0040588378906250, 0.0046081542968750, 0.0035400390625000, 0.0039978027343750,
+0.0035400390625000, 0.0039978027343750, 0.0033569335937500, 0.0039062500000000,
+0.0028991699218750, 0.0036621093750000, 0.0030517578125000, 0.0038146972656250,
+0.0030212402343750, 0.0032043457031250, 0.0026550292968750, 0.0036010742187500,
+0.0031127929687500, 0.0031127929687500, 0.0035705566406250, 0.0024719238281250,
+0.0031127929687500, 0.0030517578125000, 0.0042419433593750, 0.0014953613281250,
+0.0042724609375000, 0.0017089843750000, 0.0043029785156250, 0.0015869140625000,
+0.0049743652343750, 0.0005187988281250, 0.0047607421875000, 0.0009765625000000,
+0.0048828125000000, 0.0008850097656250, 0.0051574707031250, 0.0006408691406250,
+0.0048828125000000, 0.0009460449218750, 0.0046997070312500, 0.0011901855468750,
+0.0050048828125000, 0.0009765625000000, 0.0042114257812500, 0.0015869140625000,
+0.0044250488281250, 0.0016479492187500, 0.0043640136718750, 0.0017089843750000,
+0.0039367675781250, 0.0020446777343750, 0.0042419433593750, 0.0019836425781250,
+0.0041503906250000, 0.0021667480468750, 0.0041809082031250, 0.0021972656250000,
+0.0043640136718750, 0.0023803710937500, 0.0044860839843750, 0.0026855468750000,
+0.0044860839843750, 0.0026245117187500, 0.0046691894531250, 0.0030517578125000,
+0.0048522949218750, 0.0033264160156250, 0.0046386718750000, 0.0031433105468750,
+0.0046386718750000, 0.0032043457031250, 0.0050048828125000, 0.0034790039062500,
+0.0040893554687500, 0.0025329589843750, 0.0046386718750000, 0.0027160644531250,
+0.0042114257812500, 0.0025634765625000, 0.0034179687500000, 0.0019531250000000,
+0.0043640136718750, 0.0022583007812500, 0.0029296875000000, 0.0020141601562500,
+0.0033264160156250, 0.0019226074218750, 0.0032958984375000, 0.0020751953125000,
+0.0024108886718750, 0.0021057128906250, 0.0031433105468750, 0.0020446777343750,
+0.0024719238281250, 0.0020141601562500, 0.0026855468750000, 0.0020751953125000,
+0.0028076171875000, 0.0018920898437500, 0.0025329589843750, 0.0016784667968750,
+0.0029296875000000, 0.0018005371093750, 0.0023498535156250, 0.0012207031250000,
+0.0027465820312500, 0.0012512207031250, 0.0025329589843750, 0.0011596679687500,
+0.0020141601562500, 0.0008544921875000, 0.0025634765625000, 0.0009460449218750,
+0.0016174316406250, 0.0012512207031250, 0.0018310546875000, 0.0010681152343750,
+0.0018005371093750, 0.0015258789062500, 0.0011291503906250, 0.0021362304687500,
+0.0014648437500000, 0.0017395019531250, 0.0011901855468750, 0.0029602050781250,
+0.0010375976562500, 0.0028991699218750, 0.0012817382812500, 0.0032043457031250,
+0.0013732910156250, 0.0041198730468750, 0.0011901855468750, 0.0036926269531250,
+0.0016784667968750, 0.0043640136718750, 0.0015258789062500, 0.0045166015625000,
+0.0015258789062500, 0.0045776367187500, 0.0018005371093750, 0.0048522949218750,
+0.0016174316406250, 0.0047607421875000, 0.0016174316406250, 0.0051879882812500,
+0.0016174316406250, 0.0050048828125000, 0.0014953613281250, 0.0052490234375000,
+0.0014953613281250, 0.0056762695312500, 0.0014038085937500, 0.0054016113281250,
+0.0015563964843750, 0.0064086914062500, 0.0014953613281250, 0.0062561035156250,
+0.0015258789062500, 0.0066528320312500, 0.0016784667968750, 0.0074462890625000,
+0.0016784667968750, 0.0071105957031250, 0.0016479492187500, 0.0079040527343750,
+0.0018615722656250, 0.0079650878906250, 0.0017089843750000, 0.0080261230468750,
+0.0016479492187500, 0.0084838867187500, 0.0019226074218750, 0.0083007812500000,
+0.0014343261718750, 0.0080871582031250, 0.0015869140625000, 0.0084533691406250,
+0.0016174316406250, 0.0079345703125000, 0.0013122558593750, 0.0073242187500000,
+0.0015258789062500, 0.0077819824218750, 0.0015869140625000, 0.0061645507812500,
+0.0015869140625000, 0.0063781738281250, 0.0018615722656250, 0.0058593750000000,
+0.0021057128906250, 0.0045776367187500, 0.0020446777343750, 0.0052490234375000,
+0.0023803710937500, 0.0038757324218750, 0.0024719238281250, 0.0038146972656250,
+0.0024719238281250, 0.0036621093750000, 0.0026245117187500, 0.0028991699218750,
+0.0026550292968750, 0.0031433105468750, 0.0025634765625000, 0.0028381347656250,
+0.0025939941406250, 0.0027160644531250, 0.0025329589843750, 0.0028991699218750,
+0.0023498535156250, 0.0029907226562500, 0.0023193359375000, 0.0030212402343750,
+0.0024108886718750, 0.0034179687500000, 0.0021972656250000, 0.0034484863281250,
+0.0023193359375000, 0.0037231445312500, 0.0025024414062500, 0.0040893554687500,
+0.0021972656250000, 0.0040283203125000, 0.0025329589843750, 0.0045166015625000,
+0.0025024414062500, 0.0046997070312500, 0.0024719238281250, 0.0047302246093750,
+0.0025939941406250, 0.0049743652343750, 0.0025024414062500, 0.0051269531250000,
+0.0028381347656250, 0.0048217773437500, 0.0026550292968750, 0.0050659179687500,
+0.0028686523437500, 0.0048217773437500, 0.0032043457031250, 0.0044250488281250,
+0.0028991699218750, 0.0048217773437500, 0.0035400390625000, 0.0042114257812500,
+0.0034179687500000, 0.0042114257812500, 0.0033874511718750, 0.0042724609375000,
+0.0037841796875000, 0.0040283203125000, 0.0035095214843750, 0.0041198730468750,
+0.0035095214843750, 0.0041809082031250, 0.0037231445312500, 0.0041503906250000,
+0.0032958984375000, 0.0042114257812500, 0.0029907226562500, 0.0043334960937500,
+0.0032653808593750, 0.0043640136718750, 0.0023498535156250, 0.0041809082031250,
+0.0022888183593750, 0.0042724609375000, 0.0023193359375000, 0.0042114257812500,
+0.0018310546875000, 0.0040283203125000, 0.0018310546875000, 0.0041198730468750,
+0.0023803710937500, 0.0039062500000000, 0.0021057128906250, 0.0038757324218750,
+0.0025939941406250, 0.0039978027343750, 0.0032653808593750, 0.0040893554687500,
+0.0027770996093750, 0.0039978027343750, 0.0039062500000000, 0.0039367675781250,
+0.0038146972656250, 0.0040588378906250, 0.0038757324218750, 0.0038146972656250,
+0.0045471191406250, 0.0036010742187500, 0.0043029785156250, 0.0037841796875000,
+0.0046997070312500, 0.0036621093750000, 0.0048522949218750, 0.0033874511718750,
+0.0047302246093750, 0.0038146972656250, 0.0049438476562500, 0.0043029785156250,
+0.0050354003906250, 0.0040283203125000, 0.0043029785156250, 0.0048217773437500,
+0.0048217773437500, 0.0050048828125000, 0.0041198730468750, 0.0045166015625000,
+0.0031127929687500, 0.0046997070312500, 0.0040283203125000, 0.0050354003906250,
+0.0023193359375000, 0.0035705566406250, 0.0022583007812500, 0.0038757324218750,
+0.0025634765625000, 0.0037231445312500, 0.0016479492187500, 0.0020141601562500,
+0.0032958984375000, 0.0048522949218750, 0.0039367675781250, 0.0037841796875000,
+0.0039062500000000, 0.0042114257812500, 0.0036315917968750, 0.0040893554687500,
+0.0036010742187500, 0.0035705566406250, 0.0037841796875000, 0.0037841796875000,
+0.0039978027343750, 0.0038146972656250, 0.0037536621093750, 0.0038146972656250,
+0.0041503906250000, 0.0038757324218750, 0.0046386718750000, 0.0039978027343750,
+0.0041809082031250, 0.0039978027343750, 0.0050048828125000, 0.0040893554687500,
+0.0050659179687500, 0.0040283203125000, 0.0048217773437500, 0.0041198730468750,
+0.0051574707031250, 0.0042724609375000, 0.0050964355468750, 0.0042114257812500,
+0.0046997070312500, 0.0042724609375000, 0.0046997070312500, 0.0043945312500000,
+0.0046081542968750, 0.0043640136718750, 0.0043029785156250, 0.0043640136718750,
+0.0043640136718750, 0.0044860839843750, 0.0040588378906250, 0.0045166015625000,
+0.0041198730468750, 0.0043334960937500, 0.0037841796875000, 0.0046997070312500,
+0.0033569335937500, 0.0050354003906250, 0.0034484863281250, 0.0048522949218750,
+0.0027465820312500, 0.0056152343750000, 0.0027465820312500, 0.0056152343750000,
+0.0025024414062500, 0.0055236816406250, 0.0019531250000000, 0.0059509277343750,
+0.0021667480468750, 0.0058898925781250, 0.0018615722656250, 0.0052185058593750,
+0.0016479492187500, 0.0055236816406250, 0.0019226074218750, 0.0051269531250000,
+0.0020141601562500, 0.0042724609375000, 0.0018615722656250, 0.0047302246093750,
+0.0022583007812500, 0.0042419433593750, 0.0021972656250000, 0.0040893554687500,
+0.0022277832031250, 0.0041503906250000, 0.0023803710937500, 0.0041198730468750,
+0.0022583007812500, 0.0039978027343750, 0.0025024414062500, 0.0039062500000000,
+0.0025024414062500, 0.0039062500000000, 0.0025024414062500, 0.0038757324218750,
+0.0025634765625000, 0.0037841796875000, 0.0025329589843750, 0.0036926269531250,
+0.0031433105468750, 0.0037536621093750, 0.0028076171875000, 0.0036010742187500,
+0.0032958984375000, 0.0035095214843750, 0.0040283203125000, 0.0034790039062500,
+0.0034179687500000, 0.0032958984375000, 0.0050048828125000, 0.0030517578125000,
+0.0048828125000000, 0.0028686523437500, 0.0051269531250000, 0.0028076171875000,
+0.0061645507812500, 0.0025939941406250, 0.0057373046875000, 0.0024719238281250,
+0.0060729980468750, 0.0025634765625000, 0.0063476562500000, 0.0023803710937500,
+0.0058288574218750, 0.0023803710937500, 0.0056457519531250, 0.0025634765625000,
+0.0059509277343750, 0.0024108886718750, 0.0044860839843750, 0.0026550292968750,
+0.0046691894531250, 0.0025939941406250, 0.0043945312500000, 0.0027160644531250,
+0.0033264160156250, 0.0030212402343750, 0.0036010742187500, 0.0029296875000000,
+0.0033569335937500, 0.0031127929687500, 0.0030517578125000, 0.0032958984375000,
+0.0031738281250000, 0.0031433105468750, 0.0032958984375000, 0.0031127929687500,
+0.0030822753906250, 0.0032958984375000, 0.0032958984375000, 0.0027160644531250,
+0.0032653808593750, 0.0029296875000000, 0.0031738281250000, 0.0027160644531250,
+0.0033264160156250, 0.0021972656250000, 0.0031433105468750, 0.0026550292968750,
+0.0028991699218750, 0.0019531250000000, 0.0031127929687500, 0.0020751953125000,
+0.0027160644531250, 0.0021362304687500, 0.0022583007812500, 0.0018005371093750,
+0.0026245117187500, 0.0021667480468750, 0.0019531250000000, 0.0020751953125000,
+0.0019226074218750, 0.0022277832031250, 0.0020751953125000, 0.0023193359375000,
+0.0018920898437500, 0.0023498535156250, 0.0020446777343750, 0.0024719238281250,
+0.0023193359375000, 0.0022888183593750, 0.0023498535156250, 0.0025634765625000,
+0.0025634765625000, 0.0023803710937500, 0.0028076171875000, 0.0021362304687500,
+0.0027770996093750, 0.0023803710937500, 0.0031433105468750, 0.0016784667968750,
+0.0032348632812500, 0.0019531250000000, 0.0033874511718750, 0.0018310546875000,
+0.0036010742187500, 0.0013122558593750, 0.0036315917968750, 0.0016784667968750,
+0.0036010742187500, 0.0010681152343750, 0.0038452148437500, 0.0013122558593750,
+0.0036926269531250, 0.0011291503906250, 0.0035095214843750, 0.0007019042968750,
+0.0038146972656250, 0.0012207031250000, 0.0033569335937500, 0.0004272460937500,
+0.0036010742187500, 0.0006103515625000, 0.0036315917968750, 0.0007019042968750,
+0.0033264160156250},
+{0.0018920898437500, 0.0039062500000000, 0.0019836425781250, 0.0037536621093750,
+0.0014648437500000, 0.0037536621093750, 0.0022888183593750, 0.0037841796875000,
+0.0020751953125000, 0.0036315917968750, 0.0021667480468750, 0.0039978027343750,
+0.0027465820312500, 0.0042419433593750, 0.0023193359375000, 0.0040283203125000,
+0.0029296875000000, 0.0048828125000000, 0.0029907226562500, 0.0049438476562500,
+0.0028686523437500, 0.0048522949218750, 0.0030822753906250, 0.0052795410156250,
+0.0030212402343750, 0.0052490234375000, 0.0031738281250000, 0.0046691894531250,
+0.0030517578125000, 0.0047912597656250, 0.0033264160156250, 0.0046386718750000,
+0.0036010742187500, 0.0041503906250000, 0.0033569335937500, 0.0042114257812500,
+0.0043029785156250, 0.0039062500000000, 0.0039672851562500, 0.0037536621093750,
+0.0046081542968750, 0.0036315917968750, 0.0055541992187500, 0.0034179687500000,
+0.0048522949218750, 0.0033569335937500, 0.0064392089843750, 0.0032348632812500,
+0.0064086914062500, 0.0030517578125000, 0.0064697265625000, 0.0031127929687500,
+0.0074462890625000, 0.0032043457031250, 0.0070495605468750, 0.0029296875000000,
+0.0070800781250000, 0.0033264160156250, 0.0073547363281250, 0.0031738281250000,
+0.0070190429687500, 0.0030822753906250, 0.0066528320312500, 0.0032958984375000,
+0.0068359375000000, 0.0030517578125000, 0.0062866210937500, 0.0030822753906250,
+0.0062561035156250, 0.0028686523437500, 0.0063171386718750, 0.0028686523437500,
+0.0059814453125000, 0.0029907226562500, 0.0059814453125000, 0.0028076171875000,
+0.0064697265625000, 0.0029602050781250, 0.0061340332031250, 0.0029907226562500,
+0.0063171386718750, 0.0029602050781250, 0.0068969726562500, 0.0029602050781250,
+0.0064697265625000, 0.0030212402343750, 0.0065002441406250, 0.0030517578125000,
+0.0067138671875000, 0.0030517578125000, 0.0062866210937500, 0.0030822753906250,
+0.0059814453125000, 0.0031738281250000, 0.0062866210937500, 0.0031738281250000,
+0.0057067871093750, 0.0032653808593750, 0.0057067871093750, 0.0033264160156250,
+0.0056762695312500, 0.0032958984375000, 0.0053100585937500, 0.0033569335937500,
+0.0054931640625000, 0.0033874511718750, 0.0054321289062500, 0.0032348632812500,
+0.0052490234375000, 0.0033264160156250, 0.0054931640625000, 0.0031738281250000,
+0.0056762695312500, 0.0029602050781250, 0.0053710937500000, 0.0031433105468750,
+0.0060119628906250, 0.0027465820312500, 0.0059204101562500, 0.0027160644531250,
+0.0057373046875000, 0.0026855468750000, 0.0060119628906250, 0.0024414062500000,
+0.0058898925781250, 0.0023803710937500, 0.0054321289062500, 0.0024108886718750,
+0.0055847167968750, 0.0022888183593750, 0.0051574707031250, 0.0023193359375000,
+0.0046081542968750, 0.0024108886718750, 0.0049743652343750, 0.0022277832031250,
+0.0040283203125000, 0.0023498535156250, 0.0042114257812500, 0.0023193359375000,
+0.0039672851562500, 0.0021667480468750, 0.0031738281250000, 0.0021057128906250,
+0.0036315917968750, 0.0020446777343750, 0.0028686523437500, 0.0016784667968750,
+0.0028991699218750, 0.0017700195312500, 0.0028381347656250, 0.0015258789062500,
+0.0023498535156250, 0.0011901855468750, 0.0025329589843750, 0.0015258789062500,
+0.0024414062500000, 0.0010070800781250, 0.0022583007812500, 0.0010375976562500,
+0.0023193359375000, 0.0010681152343750, 0.0023193359375000, 0.0009155273437500,
+0.0021057128906250, 0.0011901855468750, 0.0023193359375000, 0.0010070800781250,
+0.0021667480468750, 0.0012207031250000, 0.0022583007812500, 0.0011291503906250,
+0.0024108886718750, 0.0009765625000000, 0.0021972656250000, 0.0013732910156250,
+0.0028381347656250, 0.0009460449218750, 0.0025939941406250, 0.0011291503906250,
+0.0030517578125000, 0.0011596679687500, 0.0037536621093750, 0.0008544921875000,
+0.0033264160156250, 0.0011596679687500, 0.0048828125000000, 0.0010375976562500,
+0.0046691894531250, 0.0010986328125000, 0.0051574707031250, 0.0012512207031250,
+0.0063781738281250, 0.0012512207031250, 0.0058288574218750, 0.0013122558593750,
+0.0068969726562500, 0.0017089843750000, 0.0070190429687500, 0.0015563964843750,
+0.0068664550781250, 0.0018920898437500, 0.0073547363281250, 0.0023498535156250,
+0.0073242187500000, 0.0020141601562500, 0.0067138671875000, 0.0028686523437500,
+0.0071105957031250, 0.0027160644531250, 0.0065917968750000, 0.0027770996093750,
+0.0058288574218750, 0.0033874511718750, 0.0065002441406250, 0.0029602050781250,
+0.0051574707031250, 0.0033569335937500, 0.0053100585937500, 0.0033264160156250,
+0.0052490234375000, 0.0032043457031250, 0.0043945312500000, 0.0034179687500000,
+0.0048217773437500, 0.0033264160156250, 0.0043945312500000, 0.0032653808593750,
+0.0043334960937500, 0.0034179687500000, 0.0044250488281250, 0.0032958984375000,
+0.0042724609375000, 0.0032348632812500, 0.0042724609375000, 0.0035095214843750,
+0.0045776367187500, 0.0029907226562500, 0.0045471191406250, 0.0032043457031250,
+0.0046997070312500, 0.0030822753906250, 0.0049438476562500, 0.0026855468750000,
+0.0049438476562500, 0.0029907226562500, 0.0051574707031250, 0.0027770996093750,
+0.0053405761718750, 0.0026245117187500, 0.0053100585937500, 0.0028991699218750,
+0.0053405761718750, 0.0029907226562500, 0.0054016113281250, 0.0028381347656250,
+0.0050354003906250, 0.0034790039062500, 0.0052185058593750, 0.0034179687500000,
+0.0050048828125000, 0.0034484863281250, 0.0045471191406250, 0.0039062500000000,
+0.0047302246093750, 0.0037231445312500, 0.0040893554687500, 0.0037231445312500,
+0.0040893554687500, 0.0038146972656250, 0.0039672851562500, 0.0035400390625000,
+0.0035400390625000, 0.0033264160156250, 0.0035705566406250, 0.0034790039062500,
+0.0033569335937500, 0.0029907226562500, 0.0031433105468750, 0.0029907226562500,
+0.0033874511718750, 0.0030822753906250, 0.0034179687500000, 0.0029602050781250,
+0.0031738281250000, 0.0029907226562500, 0.0041198730468750, 0.0030822753906250,
+0.0037841796875000, 0.0030212402343750, 0.0040283203125000, 0.0030212402343750,
+0.0048522949218750, 0.0030517578125000, 0.0043334960937500, 0.0031433105468750,
+0.0048828125000000, 0.0034790039062500, 0.0049743652343750, 0.0033264160156250,
+0.0047302246093750, 0.0036621093750000, 0.0047912597656250, 0.0040893554687500,
+0.0047912597656250, 0.0038146972656250, 0.0043945312500000, 0.0045776367187500,
+0.0044250488281250, 0.0045776367187500, 0.0042114257812500, 0.0044860839843750,
+0.0037231445312500, 0.0048522949218750, 0.0037231445312500, 0.0047302246093750,
+0.0034790039062500, 0.0043945312500000, 0.0034790039062500, 0.0046691894531250,
+0.0035095214843750, 0.0044250488281250, 0.0032653808593750, 0.0039367675781250,
+0.0031433105468750, 0.0041809082031250, 0.0038146972656250, 0.0031738281250000,
+0.0036010742187500, 0.0032348632812500, 0.0038757324218750, 0.0030517578125000,
+0.0046386718750000, 0.0022888183593750, 0.0042114257812500, 0.0025024414062500,
+0.0049438476562500, 0.0021057128906250, 0.0049743652343750, 0.0018615722656250,
+0.0047607421875000, 0.0020141601562500, 0.0049743652343750, 0.0020141601562500,
+0.0048217773437500, 0.0018310546875000, 0.0044860839843750, 0.0020141601562500,
+0.0045166015625000, 0.0019531250000000, 0.0043334960937500, 0.0019226074218750,
+0.0040588378906250, 0.0020141601562500, 0.0042724609375000, 0.0018615722656250,
+0.0039062500000000, 0.0022583007812500, 0.0039672851562500, 0.0019836425781250,
+0.0040283203125000, 0.0024108886718750, 0.0038452148437500, 0.0030212402343750,
+0.0040283203125000, 0.0023803710937500, 0.0040588378906250, 0.0039672851562500,
+0.0042419433593750, 0.0036926269531250, 0.0041503906250000, 0.0040893554687500,
+0.0040893554687500, 0.0053405761718750, 0.0043640136718750, 0.0047302246093750,
+0.0036315917968750, 0.0056152343750000, 0.0038757324218750, 0.0058288574218750,
+0.0037231445312500, 0.0056457519531250, 0.0031127929687500, 0.0059509277343750,
+0.0034179687500000, 0.0058898925781250, 0.0032043457031250, 0.0059509277343750,
+0.0029602050781250, 0.0059204101562500, 0.0032958984375000, 0.0060119628906250,
+0.0034484863281250, 0.0060424804687500, 0.0030517578125000, 0.0059814453125000,
+0.0039367675781250, 0.0059814453125000, 0.0036010742187500, 0.0060729980468750,
+0.0036926269531250, 0.0057983398437500, 0.0042724609375000, 0.0056762695312500,
+0.0037231445312500, 0.0058593750000000, 0.0041198730468750, 0.0052490234375000,
+0.0040283203125000, 0.0052490234375000, 0.0037231445312500, 0.0052795410156250,
+0.0038146972656250, 0.0049133300781250, 0.0036010742187500, 0.0050354003906250,
+0.0032958984375000, 0.0053710937500000, 0.0033569335937500, 0.0050964355468750,
+0.0031738281250000, 0.0054931640625000, 0.0028076171875000, 0.0061340332031250,
+0.0029907226562500, 0.0057983398437500, 0.0028381347656250, 0.0060729980468750,
+0.0027160644531250, 0.0063171386718750, 0.0028991699218750, 0.0059509277343750,
+0.0029907226562500, 0.0056457519531250, 0.0028381347656250, 0.0058898925781250,
+0.0032043457031250, 0.0054931640625000, 0.0032958984375000, 0.0053710937500000,
+0.0032043457031250, 0.0057067871093750, 0.0033264160156250, 0.0057067871093750,
+0.0035095214843750, 0.0055541992187500, 0.0028686523437500, 0.0063781738281250,
+0.0032653808593750, 0.0062866210937500, 0.0029907226562500, 0.0064697265625000,
+0.0023498535156250, 0.0071105957031250, 0.0030212402343750, 0.0068054199218750,
+0.0020141601562500, 0.0069274902343750, 0.0022277832031250, 0.0070495605468750,
+0.0022277832031250, 0.0067138671875000, 0.0015869140625000, 0.0064392089843750,
+0.0020141601562500, 0.0066223144531250, 0.0017700195312500, 0.0058288574218750,
+0.0016784667968750, 0.0059204101562500, 0.0018920898437500, 0.0058593750000000,
+0.0018615722656250, 0.0053710937500000, 0.0018005371093750, 0.0057373046875000,
+0.0021362304687500, 0.0052490234375000, 0.0020751953125000, 0.0054931640625000,
+0.0020751953125000, 0.0054321289062500, 0.0023193359375000, 0.0050964355468750,
+0.0021362304687500, 0.0054626464843750, 0.0022277832031250, 0.0047912597656250,
+0.0022277832031250, 0.0051574707031250, 0.0022583007812500, 0.0048217773437500,
+0.0022888183593750, 0.0041503906250000, 0.0021667480468750, 0.0046691894531250,
+0.0023498535156250, 0.0035400390625000, 0.0022888183593750, 0.0036926269531250,
+0.0023193359375000, 0.0034179687500000, 0.0024414062500000, 0.0025329589843750,
+0.0023498535156250, 0.0029907226562500, 0.0025024414062500, 0.0023498535156250,
+0.0024414062500000, 0.0021667480468750, 0.0024719238281250, 0.0022583007812500,
+0.0025634765625000, 0.0020141601562500, 0.0024414062500000, 0.0020141601562500,
+0.0027160644531250, 0.0024108886718750, 0.0026550292968750, 0.0021972656250000,
+0.0026855468750000, 0.0023803710937500, 0.0029907226562500, 0.0028076171875000,
+0.0028686523437500, 0.0025634765625000, 0.0031127929687500, 0.0030212402343750,
+0.0031738281250000, 0.0029907226562500, 0.0031738281250000, 0.0030822753906250,
+0.0032958984375000, 0.0033264160156250, 0.0033874511718750, 0.0032653808593750,
+0.0032043457031250, 0.0035400390625000, 0.0032348632812500, 0.0034790039062500,
+0.0031433105468750, 0.0036010742187500, 0.0030212402343750, 0.0038757324218750,
+0.0030212402343750, 0.0036621093750000, 0.0028381347656250, 0.0041198730468750,
+0.0027465820312500, 0.0040588378906250, 0.0026550292968750, 0.0042419433593750,
+0.0025634765625000, 0.0046081542968750, 0.0024108886718750, 0.0043945312500000,
+0.0024719238281250},
+{0.0028686523437500, 0.0052795410156250, 0.0022888183593750, 0.0048522949218750,
+0.0022277832031250, 0.0046691894531250, 0.0023498535156250, 0.0047607421875000,
+0.0021667480468750, 0.0046386718750000, 0.0021362304687500, 0.0043640136718750,
+0.0025329589843750, 0.0048828125000000, 0.0022583007812500, 0.0046081542968750,
+0.0024108886718750, 0.0046386718750000, 0.0028686523437500, 0.0050964355468750,
+0.0024719238281250, 0.0048217773437500, 0.0030212402343750, 0.0047302246093750,
+0.0029602050781250, 0.0047912597656250, 0.0028991699218750, 0.0043945312500000,
+0.0031433105468750, 0.0039672851562500, 0.0029907226562500, 0.0041198730468750,
+0.0029907226562500, 0.0039062500000000, 0.0029907226562500, 0.0034790039062500,
+0.0028991699218750, 0.0039672851562500, 0.0027160644531250, 0.0043334960937500,
+0.0027160644531250, 0.0037841796875000, 0.0028381347656250, 0.0051879882812500,
+0.0026550292968750, 0.0050048828125000, 0.0028076171875000, 0.0051574707031250,
+0.0030822753906250, 0.0061035156250000, 0.0028686523437500, 0.0055541992187500,
+0.0032348632812500, 0.0061035156250000, 0.0032653808593750, 0.0061035156250000,
+0.0032653808593750, 0.0060424804687500, 0.0034790039062500, 0.0062255859375000,
+0.0034790039062500, 0.0060424804687500, 0.0034790039062500, 0.0064086914062500,
+0.0036621093750000, 0.0061645507812500, 0.0034179687500000, 0.0066223144531250,
+0.0032958984375000, 0.0072021484375000, 0.0035705566406250, 0.0068054199218750,
+0.0028381347656250, 0.0080261230468750, 0.0031127929687500, 0.0080566406250000,
+0.0029602050781250, 0.0080566406250000, 0.0022277832031250, 0.0087585449218750,
+0.0027465820312500, 0.0086059570312500, 0.0022277832031250, 0.0083007812500000,
+0.0021972656250000, 0.0086975097656250, 0.0024719238281250, 0.0080566406250000,
+0.0024414062500000, 0.0074462890625000, 0.0023803710937500, 0.0079956054687500,
+0.0027160644531250, 0.0060424804687500, 0.0027465820312500, 0.0063781738281250,
+0.0026855468750000, 0.0055847167968750, 0.0028686523437500, 0.0039672851562500,
+0.0028991699218750, 0.0049133300781250, 0.0026245117187500, 0.0026550292968750,
+0.0027770996093750, 0.0027770996093750, 0.0026550292968750, 0.0024108886718750,
+0.0023498535156250, 0.0009460449218750, 0.0025329589843750, 0.0016479492187500,
+0.0024108886718750, 0.0004882812500000, 0.0023193359375000, 0.0005493164062500,
+0.0024719238281250, 0.0006408691406250, 0.0026245117187500, 0.0000610351562500,
+0.0025939941406250, 0.0003662109375000, 0.0028686523437500, 0.0004577636718750,
+0.0029296875000000, 0.0002441406250000, 0.0028991699218750, 0.0005798339843750,
+0.0031127929687500, 0.0009460449218750, 0.0031433105468750, 0.0006408691406250,
+0.0030517578125000, 0.0016784667968750, 0.0031433105468750, 0.0014953613281250,
+0.0031127929687500, 0.0018005371093750, 0.0029907226562500, 0.0026245117187500,
+0.0030822753906250, 0.0022888183593750, 0.0029296875000000, 0.0028381347656250,
+0.0028991699218750, 0.0029907226562500, 0.0028686523437500, 0.0027465820312500,
+0.0027770996093750, 0.0028381347656250, 0.0027465820312500, 0.0029602050781250,
+0.0027465820312500, 0.0022888183593750, 0.0026855468750000, 0.0025024414062500,
+0.0026855468750000, 0.0021667480468750, 0.0026550292968750, 0.0015258789062500,
+0.0025329589843750, 0.0018310546875000, 0.0028991699218750, 0.0011291503906250,
+0.0026855468750000, 0.0010375976562500, 0.0028381347656250, 0.0010986328125000,
+0.0032653808593750, 0.0008239746093750, 0.0028686523437500, 0.0009155273437500,
+0.0034790039062500, 0.0009460449218750, 0.0033874511718750, 0.0009765625000000,
+0.0032653808593750, 0.0010375976562500, 0.0036010742187500, 0.0011291503906250,
+0.0034179687500000, 0.0011596679687500, 0.0032653808593750, 0.0010375976562500,
+0.0032958984375000, 0.0011291503906250, 0.0030212402343750, 0.0010681152343750,
+0.0027465820312500, 0.0009155273437500, 0.0027465820312500, 0.0010681152343750,
+0.0023803710937500, 0.0007934570312500, 0.0021972656250000, 0.0008239746093750,
+0.0023498535156250, 0.0007934570312500, 0.0022277832031250, 0.0005493164062500,
+0.0020446777343750, 0.0005493164062500, 0.0028686523437500, 0.0008850097656250,
+0.0026550292968750, 0.0005798339843750, 0.0029907226562500, 0.0010986328125000,
+0.0038452148437500, 0.0017089843750000, 0.0034179687500000, 0.0012207031250000,
+0.0044250488281250, 0.0028381347656250, 0.0045776367187500, 0.0026550292968750,
+0.0044860839843750, 0.0030212402343750, 0.0048522949218750, 0.0041503906250000,
+0.0048828125000000, 0.0036315917968750, 0.0044555664062500, 0.0045166015625000,
+0.0048217773437500, 0.0045166015625000, 0.0043945312500000, 0.0045471191406250,
+0.0038452148437500, 0.0050354003906250, 0.0042419433593750, 0.0048522949218750,
+0.0031433105468750, 0.0050354003906250, 0.0032958984375000, 0.0050964355468750,
+0.0030517578125000, 0.0052185058593750, 0.0021667480468750, 0.0053710937500000,
+0.0026245117187500, 0.0052490234375000, 0.0023803710937500, 0.0056762695312500,
+0.0020751953125000, 0.0056762695312500, 0.0026550292968750, 0.0057373046875000,
+0.0029907226562500, 0.0061035156250000, 0.0026245117187500, 0.0060424804687500,
+0.0038146972656250, 0.0055541992187500, 0.0036315917968750, 0.0060424804687500,
+0.0037231445312500, 0.0052795410156250, 0.0045166015625000, 0.0043029785156250,
+0.0041503906250000, 0.0051269531250000, 0.0043334960937500, 0.0036010742187500,
+0.0044250488281250, 0.0034790039062500, 0.0043029785156250, 0.0034790039062500,
+0.0042419433593750, 0.0028686523437500, 0.0042419433593750, 0.0032348632812500,
+0.0043945312500000, 0.0023803710937500, 0.0042114257812500, 0.0026550292968750,
+0.0043029785156250, 0.0025634765625000, 0.0046997070312500, 0.0015258789062500,
+0.0034484863281250, 0.0046691894531250, 0.0025024414062500, 0.0021057128906250,
+0.0025939941406250, 0.0030517578125000, 0.0025939941406250, 0.0024414062500000,
+0.0021972656250000, 0.0010375976562500, 0.0021667480468750, 0.0016479492187500,
+0.0025939941406250, 0.0014038085937500, 0.0024108886718750, 0.0009765625000000,
+0.0025634765625000, 0.0016784667968750, 0.0029907226562500, 0.0021667480468750,
+0.0025634765625000, 0.0016479492187500, 0.0031127929687500, 0.0030212402343750,
+0.0029602050781250, 0.0029602050781250, 0.0031738281250000, 0.0029907226562500,
+0.0036315917968750, 0.0039062500000000, 0.0032043457031250, 0.0037231445312500,
+0.0039978027343750, 0.0036010742187500, 0.0037841796875000, 0.0039978027343750,
+0.0037841796875000, 0.0035095214843750, 0.0042724609375000, 0.0031127929687500,
+0.0039367675781250, 0.0036621093750000, 0.0040283203125000, 0.0023498535156250,
+0.0039672851562500, 0.0025634765625000, 0.0040588378906250, 0.0025634765625000,
+0.0040588378906250, 0.0017700195312500, 0.0037536621093750, 0.0022583007812500,
+0.0048522949218750, 0.0023193359375000, 0.0044250488281250, 0.0021057128906250,
+0.0048217773437500, 0.0026855468750000, 0.0058288574218750, 0.0033569335937500,
+0.0051269531250000, 0.0030212402343750, 0.0065002441406250, 0.0036926269531250,
+0.0062866210937500, 0.0040588378906250, 0.0064086914062500, 0.0037231445312500,
+0.0073852539062500, 0.0037231445312500, 0.0068359375000000, 0.0039062500000000,
+0.0072021484375000, 0.0033874511718750, 0.0073852539062500, 0.0034790039062500,
+0.0070190429687500, 0.0034179687500000, 0.0068664550781250, 0.0031433105468750,
+0.0070800781250000, 0.0032043457031250, 0.0058898925781250, 0.0032958984375000,
+0.0062255859375000, 0.0031433105468750, 0.0055847167968750, 0.0032043457031250,
+0.0044860839843750, 0.0033569335937500, 0.0050354003906250, 0.0032958984375000,
+0.0036010742187500, 0.0034790039062500, 0.0037536621093750, 0.0036010742187500,
+0.0034179687500000, 0.0035400390625000, 0.0024414062500000, 0.0036315917968750,
+0.0028991699218750, 0.0037536621093750, 0.0019531250000000, 0.0035095214843750,
+0.0020141601562500, 0.0037841796875000, 0.0018005371093750, 0.0037231445312500,
+0.0010986328125000, 0.0035095214843750, 0.0014343261718750, 0.0037536621093750,
+0.0009460449218750, 0.0036010742187500, 0.0008239746093750, 0.0036010742187500,
+0.0009765625000000, 0.0037231445312500, 0.0008239746093750, 0.0037536621093750,
+0.0008544921875000, 0.0038146972656250, 0.0014038085937500, 0.0036315917968750,
+0.0011596679687500, 0.0038146972656250, 0.0016479492187500, 0.0037841796875000,
+0.0024108886718750, 0.0036010742187500, 0.0019836425781250, 0.0036621093750000,
+0.0027160644531250, 0.0036010742187500, 0.0030822753906250, 0.0036010742187500,
+0.0025939941406250, 0.0036010742187500, 0.0025939941406250, 0.0036621093750000,
+0.0030822753906250, 0.0037231445312500, 0.0015869140625000, 0.0036315917968750,
+0.0021057128906250, 0.0037536621093750, 0.0015869140625000, 0.0036315917968750,
+0.0003662109375000, 0.0035095214843750, 0.0012817382812500, 0.0036621093750000,
+-0.0001525878906250, 0.0033264160156250, -0.0000915527343750, 0.0033569335937500,
+0.0003356933593750, 0.0032348632812500, -0.0003356933593750, 0.0030212402343750,
+-0.0001220703125000, 0.0031433105468750, 0.0011596679687500, 0.0029296875000000,
+0.0006713867187500, 0.0028991699218750, 0.0017089843750000, 0.0028381347656250,
+0.0032348632812500, 0.0027160644531250, 0.0023803710937500, 0.0027770996093750,
+0.0045776367187500, 0.0025939941406250, 0.0046081542968750, 0.0026855468750000,
+0.0049438476562500, 0.0025634765625000, 0.0062866210937500, 0.0024108886718750,
+0.0057678222656250, 0.0025939941406250, 0.0064697265625000, 0.0022277832031250,
+0.0068664550781250, 0.0022277832031250, 0.0065002441406250, 0.0022277832031250,
+0.0065612792968750, 0.0020446777343750, 0.0068054199218750, 0.0021057128906250,
+0.0057067871093750, 0.0021972656250000, 0.0061035156250000, 0.0021972656250000,
+0.0054626464843750, 0.0021667480468750, 0.0042724609375000, 0.0021972656250000,
+0.0050964355468750, 0.0021972656250000, 0.0029907226562500, 0.0021362304687500,
+0.0034179687500000, 0.0022583007812500, 0.0028686523437500, 0.0023498535156250,
+0.0012817382812500, 0.0022888183593750, 0.0022277832031250, 0.0023498535156250,
+0.0005798339843750, 0.0023498535156250, 0.0005493164062500, 0.0023803710937500,
+0.0005187988281250, 0.0023803710937500, -0.0003356933593750, 0.0024108886718750,
+0.0000305175781250, 0.0025329589843750, -0.0002136230468750, 0.0024108886718750,
+-0.0004272460937500, 0.0025024414062500, -0.0001831054687500, 0.0026245117187500,
+-0.0000305175781250, 0.0025634765625000, -0.0002441406250000, 0.0025634765625000,
+0.0002136230468750, 0.0027465820312500, 0.0002136230468750, 0.0027160644531250,
+0.0003051757812500, 0.0028076171875000, 0.0006408691406250, 0.0029907226562500,
+0.0005493164062500, 0.0027770996093750, 0.0007629394531250, 0.0029907226562500,
+0.0008544921875000, 0.0029296875000000, 0.0010375976562500, 0.0029602050781250,
+0.0012512207031250, 0.0029907226562500, 0.0011596679687500, 0.0028381347656250,
+0.0019226074218750, 0.0029296875000000, 0.0017089843750000, 0.0028381347656250,
+0.0021667480468750, 0.0028076171875000, 0.0028991699218750, 0.0028076171875000,
+0.0024108886718750, 0.0027770996093750, 0.0034484863281250, 0.0026855468750000,
+0.0033874511718750, 0.0026855468750000, 0.0032348632812500, 0.0025634765625000,
+0.0036621093750000, 0.0023803710937500, 0.0035095214843750, 0.0024108886718750,
+0.0032348632812500, 0.0024108886718750, 0.0034790039062500, 0.0021972656250000,
+0.0032348632812500},
+{0.0019836425781250, 0.0041198730468750, 0.0020141601562500, 0.0039672851562500,
+0.0029296875000000, 0.0038757324218750, 0.0023803710937500, 0.0039367675781250,
+0.0023498535156250, 0.0037536621093750, 0.0027160644531250, 0.0038146972656250,
+0.0018615722656250, 0.0038146972656250, 0.0011291503906250, 0.0036621093750000,
+0.0018005371093750, 0.0038757324218750, -0.0001525878906250, 0.0036621093750000,
+0.0000000000000000, 0.0036621093750000, -0.0001220703125000, 0.0036621093750000,
+-0.0014343261718750, 0.0035400390625000, -0.0010375976562500, 0.0036621093750000,
+-0.0009460449218750, 0.0035400390625000, -0.0015258789062500, 0.0036621093750000,
+-0.0007934570312500, 0.0035705566406250, -0.0001525878906250, 0.0035400390625000,
+-0.0009765625000000, 0.0037231445312500, 0.0012512207031250, 0.0031738281250000,
+0.0007324218750000, 0.0034484863281250, 0.0012207031250000, 0.0032958984375000,
+0.0030212402343750, 0.0027770996093750, 0.0020141601562500, 0.0031433105468750,
+0.0031433105468750, 0.0025634765625000, 0.0033569335937500, 0.0025634765625000,
+0.0027770996093750, 0.0025634765625000, 0.0028991699218750, 0.0022277832031250,
+0.0030517578125000, 0.0023498535156250, 0.0019226074218750, 0.0023193359375000,
+0.0021667480468750, 0.0021972656250000, 0.0016479492187500, 0.0024414062500000,
+0.0006713867187500, 0.0025939941406250, 0.0012817382812500, 0.0024414062500000,
+0.0001831054687500, 0.0028991699218750, 0.0000915527343750, 0.0027770996093750,
+0.0003356933593750, 0.0027770996093750, -0.0000915527343750, 0.0030517578125000,
+0.0000610351562500, 0.0028381347656250, 0.0007629394531250, 0.0029602050781250,
+0.0003356933593750, 0.0028381347656250, 0.0010986328125000, 0.0027465820312500,
+0.0020141601562500, 0.0027160644531250, 0.0014343261718750, 0.0025939941406250,
+0.0032958984375000, 0.0025634765625000, 0.0030822753906250, 0.0025024414062500,
+0.0035095214843750, 0.0024414062500000, 0.0047912597656250, 0.0024108886718750,
+0.0040893554687500, 0.0024414062500000, 0.0050659179687500, 0.0024719238281250,
+0.0051269531250000, 0.0025024414062500, 0.0048828125000000, 0.0026550292968750,
+0.0052185058593750, 0.0027465820312500, 0.0049438476562500, 0.0026245117187500,
+0.0043945312500000, 0.0028686523437500, 0.0045776367187500, 0.0029296875000000,
+0.0040588378906250, 0.0029296875000000, 0.0033874511718750, 0.0031433105468750,
+0.0038146972656250, 0.0030822753906250, 0.0027160644531250, 0.0031738281250000,
+0.0029602050781250, 0.0032043457031250, 0.0027770996093750, 0.0031738281250000,
+0.0020446777343750, 0.0032043457031250, 0.0026245117187500, 0.0031738281250000,
+0.0020141601562500, 0.0030517578125000, 0.0022277832031250, 0.0030517578125000,
+0.0022583007812500, 0.0029907226562500, 0.0018615722656250, 0.0028991699218750,
+0.0022583007812500, 0.0028991699218750, 0.0018920898437500, 0.0026550292968750,
+0.0019836425781250, 0.0027465820312500, 0.0021057128906250, 0.0026855468750000,
+0.0018615722656250, 0.0023803710937500, 0.0019226074218750, 0.0026550292968750,
+0.0022277832031250, 0.0024108886718750, 0.0020141601562500, 0.0023193359375000,
+0.0024414062500000, 0.0025939941406250, 0.0029907226562500, 0.0026550292968750,
+0.0026245117187500, 0.0025024414062500, 0.0037536621093750, 0.0031738281250000,
+0.0036315917968750, 0.0030212402343750, 0.0037536621093750, 0.0031738281250000,
+0.0043945312500000, 0.0036926269531250, 0.0040588378906250, 0.0034484863281250,
+0.0042114257812500, 0.0035400390625000, 0.0043029785156250, 0.0037231445312500,
+0.0041198730468750, 0.0035095214843750, 0.0039978027343750, 0.0033874511718750,
+0.0039978027343750, 0.0035400390625000, 0.0036315917968750, 0.0031738281250000,
+0.0036926269531250, 0.0032653808593750, 0.0036315917968750, 0.0032043457031250,
+0.0033874511718750, 0.0029296875000000, 0.0034484863281250, 0.0031433105468750,
+0.0032958984375000, 0.0029296875000000, 0.0032958984375000, 0.0028991699218750,
+0.0031738281250000, 0.0030212402343750, 0.0030517578125000, 0.0029907226562500,
+0.0032043457031250, 0.0030517578125000, 0.0031433105468750, 0.0032958984375000,
+0.0032043457031250, 0.0033569335937500, 0.0032348632812500, 0.0035400390625000,
+0.0032653808593750, 0.0038146972656250, 0.0033264160156250, 0.0038452148437500,
+0.0028686523437500, 0.0039062500000000, 0.0030212402343750, 0.0040588378906250,
+0.0027770996093750, 0.0038757324218750, 0.0023193359375000, 0.0037536621093750,
+0.0025634765625000, 0.0039978027343750, 0.0018920898437500, 0.0035095214843750,
+0.0018310546875000, 0.0036621093750000, 0.0018615722656250, 0.0035095214843750,
+0.0015258789062500, 0.0031127929687500, 0.0016479492187500, 0.0034179687500000,
+0.0018920898437500, 0.0029602050781250, 0.0018310546875000, 0.0029602050781250,
+0.0020751953125000, 0.0027770996093750, 0.0024108886718750, 0.0023803710937500,
+0.0024414062500000, 0.0026550292968750, 0.0028381347656250, 0.0021972656250000,
+0.0028686523437500, 0.0021667480468750, 0.0030517578125000, 0.0022888183593750,
+0.0034179687500000, 0.0020446777343750, 0.0033874511718750, 0.0020751953125000,
+0.0038452148437500, 0.0029296875000000, 0.0036926269531250, 0.0025024414062500,
+0.0035095214843750, 0.0028991699218750, 0.0036926269531250, 0.0037841796875000,
+0.0035400390625000, 0.0032043457031250, 0.0033874511718750, 0.0042419433593750,
+0.0033874511718750, 0.0043945312500000, 0.0032958984375000, 0.0044555664062500,
+0.0031433105468750, 0.0050354003906250, 0.0032653808593750, 0.0049438476562500,
+0.0036010742187500, 0.0050659179687500, 0.0034484863281250, 0.0053100585937500,
+0.0036315917968750, 0.0050964355468750, 0.0041503906250000, 0.0049133300781250,
+0.0040588378906250, 0.0052490234375000, 0.0040588378906250, 0.0044250488281250,
+0.0044555664062500, 0.0045471191406250, 0.0042114257812500, 0.0044250488281250,
+0.0039367675781250, 0.0039367675781250, 0.0043029785156250, 0.0042724609375000,
+0.0036315917968750, 0.0038757324218750, 0.0037536621093750, 0.0039062500000000,
+0.0037231445312500, 0.0038146972656250, 0.0032348632812500, 0.0037536621093750,
+0.0034790039062500, 0.0039367675781250, 0.0036315917968750, 0.0029296875000000,
+0.0034179687500000, 0.0033264160156250, 0.0036926269531250, 0.0028991699218750,
+0.0041198730468750, 0.0018005371093750, 0.0038452148437500, 0.0022583007812500,
+0.0043029785156250, 0.0013732910156250, 0.0044250488281250, 0.0011291503906250,
+0.0043334960937500, 0.0012817382812500, 0.0044555664062500, 0.0010070800781250,
+0.0045471191406250, 0.0008239746093750, 0.0042419433593750, 0.0010986328125000,
+0.0042419433593750, 0.0010681152343750, 0.0042419433593750, 0.0009155273437500,
+0.0040283203125000, 0.0010375976562500, 0.0039978027343750, 0.0010070800781250,
+0.0042419433593750, 0.0007019042968750, 0.0040588378906250, 0.0007324218750000,
+0.0041503906250000, 0.0007019042968750, 0.0044555664062500, 0.0003662109375000,
+0.0043334960937500, 0.0003051757812500, 0.0043029785156250, 0.0007324218750000,
+0.0044555664062500, 0.0003967285156250, 0.0042114257812500, 0.0008850097656250,
+0.0039367675781250, 0.0015258789062500, 0.0041198730468750, 0.0010375976562500,
+0.0035705566406250, 0.0021667480468750, 0.0036010742187500, 0.0022583007812500,
+0.0036010742187500, 0.0021667480468750, 0.0032653808593750, 0.0027160644531250,
+0.0034179687500000, 0.0025939941406250, 0.0032958984375000, 0.0021667480468750,
+0.0032653808593750, 0.0023193359375000, 0.0033569335937500, 0.0023193359375000,
+0.0034484863281250, 0.0019531250000000, 0.0034790039062500, 0.0020141601562500,
+0.0036315917968750, 0.0023498535156250, 0.0036926269531250, 0.0021362304687500,
+0.0037231445312500, 0.0024108886718750, 0.0038452148437500, 0.0029602050781250,
+0.0039367675781250, 0.0026855468750000, 0.0038146972656250, 0.0031433105468750,
+0.0038452148437500, 0.0032653808593750, 0.0038452148437500, 0.0029602050781250,
+0.0037536621093750, 0.0029296875000000, 0.0037536621093750, 0.0029907226562500,
+0.0037841796875000, 0.0020751953125000, 0.0036926269531250, 0.0022583007812500,
+0.0036926269531250, 0.0017395019531250, 0.0037536621093750, 0.0008239746093750,
+0.0037231445312500, 0.0012207031250000, 0.0038757324218750, 0.0003662109375000,
+0.0037841796875000, 0.0003051757812500, 0.0038452148437500, 0.0003356933593750,
+0.0039978027343750, -0.0000305175781250, 0.0038452148437500, 0.0001220703125000,
+0.0040588378906250, 0.0000000000000000, 0.0040588378906250, 0.0000305175781250,
+0.0039062500000000, 0.0001525878906250, 0.0038757324218750, 0.0000915527343750,
+0.0039978027343750, 0.0001525878906250, 0.0035400390625000, 0.0003051757812500,
+0.0036621093750000, 0.0002746582031250, 0.0035095214843750, 0.0005493164062500,
+0.0032348632812500, 0.0007629394531250, 0.0034484863281250, 0.0006408691406250,
+0.0029602050781250, 0.0015869140625000, 0.0030517578125000, 0.0013427734375000,
+0.0030212402343750, 0.0019531250000000, 0.0026550292968750, 0.0028686523437500,
+0.0028381347656250, 0.0023498535156250, 0.0026550292968750, 0.0040893554687500,
+0.0026245117187500, 0.0038452148437500, 0.0026855468750000, 0.0043640136718750,
+0.0026855468750000, 0.0057067871093750, 0.0026245117187500, 0.0050048828125000,
+0.0027160644531250, 0.0063476562500000, 0.0026550292968750, 0.0065002441406250,
+0.0026245117187500, 0.0063171386718750, 0.0026245117187500, 0.0069274902343750,
+0.0025634765625000, 0.0068359375000000, 0.0025329589843750, 0.0062866210937500,
+0.0024414062500000, 0.0066528320312500, 0.0024414062500000, 0.0060119628906250,
+0.0024414062500000, 0.0052185058593750, 0.0024108886718750, 0.0056762695312500,
+0.0026550292968750, 0.0039978027343750, 0.0025634765625000, 0.0041503906250000,
+0.0027465820312500, 0.0036926269531250, 0.0030517578125000, 0.0025329589843750,
+0.0028991699218750, 0.0030822753906250, 0.0032958984375000, 0.0021362304687500,
+0.0033874511718750, 0.0019226074218750, 0.0032958984375000, 0.0020446777343750,
+0.0035095214843750, 0.0018310546875000, 0.0036010742187500, 0.0018310546875000,
+0.0031738281250000, 0.0022583007812500, 0.0034484863281250, 0.0020751953125000,
+0.0032653808593750, 0.0025329589843750, 0.0028991699218750, 0.0031433105468750,
+0.0032043457031250, 0.0028381347656250, 0.0026550292968750, 0.0035400390625000,
+0.0027770996093750, 0.0036315917968750, 0.0027770996093750, 0.0036926269531250,
+0.0024719238281250, 0.0040893554687500, 0.0027465820312500, 0.0040283203125000,
+0.0028076171875000, 0.0040283203125000, 0.0027770996093750, 0.0041809082031250,
+0.0030822753906250, 0.0039978027343750, 0.0033264160156250, 0.0038452148437500,
+0.0032348632812500, 0.0039978027343750, 0.0036926269531250, 0.0036010742187500,
+0.0038146972656250, 0.0035095214843750, 0.0036621093750000, 0.0035400390625000,
+0.0038146972656250, 0.0033569335937500, 0.0039062500000000, 0.0032348632812500,
+0.0034790039062500, 0.0035705566406250, 0.0035400390625000, 0.0032653808593750,
+0.0034484863281250, 0.0036010742187500, 0.0030212402343750, 0.0040588378906250,
+0.0030822753906250, 0.0036926269531250, 0.0029907226562500, 0.0042114257812500,
+0.0027770996093750, 0.0043334960937500, 0.0029602050781250, 0.0042419433593750,
+0.0031127929687500, 0.0043640136718750, 0.0028686523437500, 0.0043640136718750,
+0.0033569335937500, 0.0042724609375000, 0.0032348632812500, 0.0042419433593750,
+0.0033569335937500, 0.0042419433593750, 0.0036926269531250, 0.0042114257812500,
+0.0034484863281250},
+{0.0037231445312500, 0.0013732910156250, 0.0037536621093750, 0.0015563964843750,
+0.0035400390625000, 0.0015258789062500, 0.0036621093750000, 0.0015563964843750,
+0.0035400390625000, 0.0014648437500000, 0.0034484863281250, 0.0013122558593750,
+0.0036010742187500, 0.0014343261718750, 0.0034790039062500, 0.0010375976562500,
+0.0035095214843750, 0.0010986328125000, 0.0036010742187500, 0.0011291503906250,
+0.0036010742187500, 0.0009460449218750, 0.0036010742187500, 0.0010986328125000,
+0.0036315917968750, 0.0011596679687500, 0.0037841796875000, 0.0011901855468750,
+0.0036010742187500, 0.0013122558593750, 0.0034790039062500, 0.0014038085937500,
+0.0037231445312500, 0.0015258789062500, 0.0031433105468750, 0.0016784667968750,
+0.0033264160156250, 0.0016174316406250, 0.0032043457031250, 0.0017700195312500,
+0.0027160644531250, 0.0019226074218750, 0.0030212402343750, 0.0017700195312500,
+0.0025939941406250, 0.0021057128906250, 0.0026550292968750, 0.0020751953125000,
+0.0026855468750000, 0.0021057128906250, 0.0024414062500000, 0.0023498535156250,
+0.0025939941406250, 0.0022583007812500, 0.0024719238281250, 0.0025024414062500,
+0.0025329589843750, 0.0025024414062500, 0.0025939941406250, 0.0026245117187500,
+0.0025939941406250, 0.0028381347656250, 0.0027160644531250, 0.0028076171875000,
+0.0027770996093750, 0.0031433105468750, 0.0027770996093750, 0.0032348632812500,
+0.0028381347656250, 0.0032348632812500, 0.0029907226562500, 0.0034790039062500,
+0.0029602050781250, 0.0036010742187500, 0.0029602050781250, 0.0033874511718750,
+0.0030517578125000, 0.0035705566406250, 0.0029296875000000, 0.0032653808593750,
+0.0028686523437500, 0.0029296875000000, 0.0029907226562500, 0.0032958984375000,
+0.0026550292968750, 0.0022277832031250, 0.0028381347656250, 0.0024414062500000,
+0.0026855468750000, 0.0019226074218750, 0.0024414062500000, 0.0009460449218750,
+0.0027770996093750, 0.0015258789062500, 0.0024108886718750, 0.0003356933593750,
+0.0025634765625000, 0.0003356933593750, 0.0026855468750000, 0.0003051757812500,
+0.0025329589843750, -0.0003967285156250, 0.0028076171875000, -0.0001831054687500,
+0.0028381347656250, 0.0000305175781250, 0.0029296875000000, -0.0002441406250000,
+0.0031433105468750, 0.0002441406250000, 0.0033874511718750, 0.0007629394531250,
+0.0034790039062500, 0.0004272460937500, 0.0036010742187500, 0.0017395019531250,
+0.0038757324218750, 0.0015869140625000, 0.0037841796875000, 0.0019226074218750,
+0.0036926269531250, 0.0029907226562500, 0.0039978027343750, 0.0025024414062500,
+0.0035095214843750, 0.0033264160156250, 0.0036010742187500, 0.0033874511718750,
+0.0035095214843750, 0.0032958984375000, 0.0030517578125000, 0.0036010742187500,
+0.0031738281250000, 0.0034179687500000, 0.0030517578125000, 0.0034179687500000,
+0.0028076171875000, 0.0034179687500000, 0.0028991699218750, 0.0033569335937500,
+0.0029907226562500, 0.0032348632812500, 0.0027465820312500, 0.0032043457031250,
+0.0032958984375000, 0.0034790039062500, 0.0030822753906250, 0.0032958984375000,
+0.0031738281250000, 0.0032958984375000, 0.0036621093750000, 0.0035400390625000,
+0.0033264160156250, 0.0034790039062500, 0.0037231445312500, 0.0034484863281250,
+0.0036926269531250, 0.0035400390625000, 0.0036010742187500, 0.0035705566406250,
+0.0038146972656250, 0.0035400390625000, 0.0038146972656250, 0.0035095214843750,
+0.0039062500000000, 0.0036010742187500, 0.0038757324218750, 0.0035705566406250,
+0.0040283203125000, 0.0037536621093750, 0.0041503906250000, 0.0038146972656250,
+0.0039367675781250, 0.0037536621093750, 0.0042114257812500, 0.0042724609375000,
+0.0041503906250000, 0.0041198730468750, 0.0038452148437500, 0.0044555664062500,
+0.0039367675781250, 0.0049743652343750, 0.0039672851562500, 0.0046691894531250,
+0.0031738281250000, 0.0049438476562500, 0.0033569335937500, 0.0052490234375000,
+0.0030517578125000, 0.0045776367187500, 0.0022583007812500, 0.0042724609375000,
+0.0026855468750000, 0.0046997070312500, 0.0021972656250000, 0.0027160644531250,
+0.0019226074218750, 0.0029907226562500, 0.0023803710937500, 0.0026245117187500,
+0.0025634765625000, 0.0011596679687500, 0.0022888183593750, 0.0017700195312500,
+0.0032958984375000, 0.0013122558593750, 0.0031738281250000, 0.0008239746093750,
+0.0031433105468750, 0.0014038085937500, 0.0039367675781250, 0.0021667480468750,
+0.0033264160156250, 0.0030212402343750, 0.0035705566406250, 0.0016174316406250,
+0.0032653808593750, 0.0017700195312500, 0.0035705566406250, 0.0017395019531250,
+0.0038757324218750, 0.0014343261718750, 0.0035705566406250, 0.0016784667968750,
+0.0042724609375000, 0.0014953613281250, 0.0039978027343750, 0.0014343261718750,
+0.0042419433593750, 0.0019226074218750, 0.0048522949218750, 0.0020751953125000,
+0.0042724609375000, 0.0017700195312500, 0.0054016113281250, 0.0036926269531250,
+0.0051574707031250, 0.0031433105468750, 0.0051574707031250, 0.0037841796875000,
+0.0058288574218750, 0.0055847167968750, 0.0054931640625000, 0.0047302246093750,
+0.0052490234375000, 0.0052490234375000, 0.0055541992187500, 0.0059509277343750,
+0.0051269531250000, 0.0049438476562500, 0.0046081542968750, 0.0043334960937500,
+0.0049743652343750, 0.0051269531250000, 0.0042114257812500, 0.0028991699218750,
+0.0041809082031250, 0.0031433105468750, 0.0039367675781250, 0.0028991699218750,
+0.0034790039062500, 0.0014343261718750, 0.0037841796875000, 0.0020446777343750,
+0.0028686523437500, 0.0016174316406250, 0.0031738281250000, 0.0010986328125000,
+0.0029907226562500, 0.0017395019531250, 0.0022888183593750, 0.0022277832031250,
+0.0027770996093750, 0.0016479492187500, 0.0021972656250000, 0.0033569335937500,
+0.0022277832031250, 0.0032043457031250, 0.0024719238281250, 0.0034179687500000,
+0.0022583007812500, 0.0046081542968750, 0.0023498535156250, 0.0039978027343750,
+0.0028076171875000, 0.0041503906250000, 0.0026550292968750, 0.0044250488281250,
+0.0029296875000000, 0.0039367675781250, 0.0035095214843750, 0.0034484863281250,
+0.0033264160156250, 0.0037536621093750, 0.0039672851562500, 0.0031738281250000,
+0.0040283203125000, 0.0030822753906250, 0.0040893554687500, 0.0032043457031250,
+0.0045166015625000, 0.0030517578125000, 0.0044555664062500, 0.0031738281250000,
+0.0044250488281250, 0.0032958984375000, 0.0046386718750000, 0.0033569335937500,
+0.0043640136718750, 0.0033569335937500, 0.0041503906250000, 0.0033874511718750,
+0.0044555664062500, 0.0035095214843750, 0.0032958984375000, 0.0034179687500000,
+0.0036010742187500, 0.0034790039062500, 0.0032348632812500, 0.0034790039062500,
+0.0021972656250000, 0.0033874511718750, 0.0026550292968750, 0.0033874511718750,
+0.0021362304687500, 0.0033264160156250, 0.0018005371093750, 0.0032958984375000,
+0.0024719238281250, 0.0033264160156250, 0.0026550292968750, 0.0032958984375000,
+0.0021667480468750, 0.0032348632812500, 0.0041809082031250, 0.0033264160156250,
+0.0037536621093750, 0.0033264160156250, 0.0040893554687500, 0.0032958984375000,
+0.0056457519531250, 0.0032348632812500, 0.0049438476562500, 0.0032043457031250,
+0.0053100585937500, 0.0030517578125000, 0.0058898925781250, 0.0029907226562500,
+0.0050659179687500, 0.0029602050781250, 0.0045166015625000, 0.0028686523437500,
+0.0052490234375000, 0.0027160644531250, 0.0032958984375000, 0.0028076171875000,
+0.0036621093750000, 0.0025634765625000, 0.0032043457031250, 0.0026245117187500,
+0.0016479492187500, 0.0027160644531250, 0.0025329589843750, 0.0024414062500000,
+0.0013732910156250, 0.0027465820312500, 0.0010070800781250, 0.0025939941406250,
+0.0014038085937500, 0.0026855468750000, 0.0010681152343750, 0.0030517578125000,
+0.0007934570312500, 0.0027465820312500, 0.0018005371093750, 0.0032348632812500,
+0.0014343261718750, 0.0031127929687500, 0.0016479492187500, 0.0031738281250000,
+0.0024414062500000, 0.0034484863281250, 0.0020141601562500, 0.0031738281250000,
+0.0025939941406250, 0.0034179687500000, 0.0025329589843750, 0.0034179687500000,
+0.0025939941406250, 0.0032653808593750, 0.0029907226562500, 0.0033264160156250,
+0.0027465820312500, 0.0032348632812500, 0.0028991699218750, 0.0029602050781250,
+0.0030517578125000, 0.0029602050781250, 0.0028076171875000, 0.0029296875000000,
+0.0026245117187500, 0.0027770996093750, 0.0027770996093750, 0.0027465820312500,
+0.0020751953125000, 0.0026855468750000, 0.0021667480468750, 0.0026550292968750,
+0.0018615722656250, 0.0027160644531250, 0.0012512207031250, 0.0027465820312500,
+0.0014038085937500, 0.0026550292968750, 0.0007629394531250, 0.0028381347656250,
+0.0006713867187500, 0.0028076171875000, 0.0005493164062500, 0.0028991699218750,
+0.0000610351562500, 0.0030822753906250, 0.0000610351562500, 0.0029296875000000,
+0.0001525878906250, 0.0030822753906250, -0.0001220703125000, 0.0029907226562500,
+0.0001525878906250, 0.0028381347656250, 0.0004272460937500, 0.0028686523437500,
+0.0002441406250000, 0.0027770996093750, 0.0014038085937500, 0.0027465820312500,
+0.0012207031250000, 0.0026550292968750, 0.0018920898437500, 0.0026550292968750,
+0.0030822753906250, 0.0026550292968750, 0.0025939941406250, 0.0025634765625000,
+0.0043029785156250, 0.0025634765625000, 0.0043640136718750, 0.0026550292968750,
+0.0045776367187500, 0.0026550292968750, 0.0055847167968750, 0.0025634765625000,
+0.0053405761718750, 0.0025634765625000, 0.0054626464843750, 0.0025939941406250,
+0.0057983398437500, 0.0025329589843750, 0.0053100585937500, 0.0027160644531250,
+0.0049133300781250, 0.0028076171875000, 0.0052185058593750, 0.0027770996093750,
+0.0042419433593750, 0.0032043457031250, 0.0042724609375000, 0.0031433105468750,
+0.0042114257812500, 0.0033264160156250, 0.0036621093750000, 0.0037536621093750,
+0.0038146972656250, 0.0035705566406250, 0.0039367675781250, 0.0038757324218750,
+0.0037536621093750, 0.0039672851562500, 0.0039062500000000, 0.0037841796875000,
+0.0041198730468750, 0.0038146972656250, 0.0038452148437500, 0.0039367675781250,
+0.0040893554687500, 0.0036010742187500, 0.0039672851562500, 0.0036621093750000,
+0.0038757324218750, 0.0036010742187500, 0.0038146972656250, 0.0033569335937500,
+0.0036621093750000, 0.0034179687500000, 0.0040283203125000, 0.0032348632812500,
+0.0036621093750000, 0.0031738281250000, 0.0040283203125000, 0.0032348632812500,
+0.0046386718750000, 0.0031433105468750, 0.0042114257812500, 0.0031127929687500,
+0.0052795410156250, 0.0033569335937500, 0.0052795410156250, 0.0032348632812500,
+0.0052185058593750, 0.0033874511718750, 0.0058593750000000, 0.0036621093750000,
+0.0056457519531250, 0.0034179687500000, 0.0052185058593750, 0.0038146972656250,
+0.0055847167968750, 0.0038452148437500, 0.0050964355468750, 0.0037231445312500,
+0.0044250488281250, 0.0038146972656250, 0.0047302246093750, 0.0038146972656250,
+0.0038452148437500, 0.0035400390625000, 0.0037536621093750, 0.0035095214843750,
+0.0036315917968750, 0.0033874511718750, 0.0030822753906250, 0.0031738281250000,
+0.0033264160156250, 0.0032043457031250, 0.0028381347656250, 0.0028991699218750,
+0.0028076171875000, 0.0029296875000000, 0.0027770996093750, 0.0028381347656250,
+0.0026550292968750, 0.0025634765625000, 0.0027465820312500, 0.0027160644531250,
+0.0026855468750000, 0.0023498535156250, 0.0025939941406250, 0.0022277832031250,
+0.0028991699218750, 0.0023498535156250, 0.0031127929687500, 0.0022583007812500,
+0.0028381347656250, 0.0021057128906250, 0.0037536621093750, 0.0027770996093750,
+0.0036926269531250},
+{0.0031738281250000, 0.0030212402343750, 0.0032043457031250, 0.0028686523437500,
+0.0032348632812500, 0.0029296875000000, 0.0030517578125000, 0.0031127929687500,
+0.0029907226562500, 0.0029602050781250, 0.0031127929687500, 0.0031127929687500,
+0.0029907226562500, 0.0030517578125000, 0.0030212402343750, 0.0029907226562500,
+0.0030822753906250, 0.0030517578125000, 0.0029296875000000, 0.0029907226562500,
+0.0031127929687500, 0.0029907226562500, 0.0030517578125000, 0.0030212402343750,
+0.0030212402343750, 0.0029907226562500, 0.0031127929687500, 0.0028991699218750,
+0.0030822753906250, 0.0028686523437500, 0.0027770996093750, 0.0028381347656250,
+0.0030212402343750, 0.0027465820312500, 0.0027465820312500, 0.0027770996093750,
+0.0023498535156250, 0.0028076171875000, 0.0027770996093750, 0.0026855468750000,
+0.0020751953125000, 0.0028381347656250, 0.0021667480468750, 0.0027465820312500,
+0.0021362304687500, 0.0026855468750000, 0.0017089843750000, 0.0027465820312500,
+0.0019531250000000, 0.0025024414062500, 0.0020446777343750, 0.0025634765625000,
+0.0017395019531250, 0.0024719238281250, 0.0021972656250000, 0.0025329589843750,
+0.0026550292968750, 0.0025634765625000, 0.0022888183593750, 0.0025024414062500,
+0.0033874511718750, 0.0027465820312500, 0.0032348632812500, 0.0027770996093750,
+0.0034484863281250, 0.0029296875000000, 0.0042114257812500, 0.0032043457031250,
+0.0039367675781250, 0.0032348632812500, 0.0039062500000000, 0.0032958984375000,
+0.0042724609375000, 0.0034790039062500, 0.0038146972656250, 0.0034484863281250,
+0.0032958984375000, 0.0034179687500000, 0.0036926269531250, 0.0035400390625000,
+0.0024719238281250, 0.0032348632812500, 0.0025939941406250, 0.0033569335937500,
+0.0023498535156250, 0.0032653808593750, 0.0014953613281250, 0.0029907226562500,
+0.0018310546875000, 0.0032043457031250, 0.0014648437500000, 0.0030517578125000,
+0.0013427734375000, 0.0030517578125000, 0.0014038085937500, 0.0031433105468750,
+0.0013427734375000, 0.0030822753906250, 0.0013732910156250, 0.0030212402343750,
+0.0011291503906250, 0.0032958984375000, 0.0011901855468750, 0.0031738281250000,
+0.0010986328125000, 0.0033264160156250, 0.0007934570312500, 0.0035400390625000,
+0.0009765625000000, 0.0033264160156250, 0.0011291503906250, 0.0036926269531250,
+0.0009460449218750, 0.0036926269531250, 0.0014953613281250, 0.0036010742187500,
+0.0019531250000000, 0.0037231445312500, 0.0016784667968750, 0.0036315917968750,
+0.0029907226562500, 0.0034790039062500, 0.0029602050781250, 0.0035095214843750,
+0.0031433105468750, 0.0034179687500000, 0.0039672851562500, 0.0032043457031250,
+0.0037841796875000, 0.0032653808593750, 0.0042419433593750, 0.0030822753906250,
+0.0043640136718750, 0.0030822753906250, 0.0043945312500000, 0.0029296875000000,
+0.0046081542968750, 0.0027770996093750, 0.0045166015625000, 0.0027770996093750,
+0.0044860839843750, 0.0026855468750000, 0.0045776367187500, 0.0024719238281250,
+0.0045471191406250, 0.0027160644531250, 0.0044860839843750, 0.0028381347656250,
+0.0045776367187500, 0.0026245117187500, 0.0041809082031250, 0.0033569335937500,
+0.0045776367187500, 0.0032348632812500, 0.0041503906250000, 0.0033569335937500,
+0.0034790039062500, 0.0039062500000000, 0.0040283203125000, 0.0037536621093750,
+0.0030822753906250, 0.0039367675781250, 0.0030517578125000, 0.0041809082031250,
+0.0032653808593750, 0.0039062500000000, 0.0029907226562500, 0.0037536621093750,
+0.0030517578125000, 0.0039672851562500, 0.0036010742187500, 0.0035095214843750,
+0.0034484863281250, 0.0034790039062500, 0.0035705566406250, 0.0036010742187500,
+0.0041198730468750, 0.0035095214843750, 0.0040283203125000, 0.0034484863281250,
+0.0036010742187500, 0.0036010742187500, 0.0039672851562500, 0.0036926269531250,
+0.0036315917968750, 0.0035400390625000, 0.0031127929687500, 0.0036621093750000,
+0.0036926269531250, 0.0038452148437500, 0.0023498535156250, 0.0034179687500000,
+0.0025329589843750, 0.0035400390625000, 0.0023193359375000, 0.0034484863281250,
+0.0012817382812500, 0.0031433105468750, 0.0016784667968750, 0.0034179687500000,
+0.0011901855468750, 0.0029296875000000, 0.0008850097656250, 0.0030822753906250,
+0.0012512207031250, 0.0029907226562500, 0.0013427734375000, 0.0026550292968750,
+0.0010681152343750, 0.0030212402343750, 0.0021667480468750, 0.0031127929687500,
+0.0019836425781250, 0.0030212402343750, 0.0022583007812500, 0.0035095214843750,
+0.0031127929687500, 0.0039978027343750, 0.0026550292968750, 0.0039367675781250,
+0.0030517578125000, 0.0046386718750000, 0.0031433105468750, 0.0047607421875000,
+0.0030212402343750, 0.0047607421875000, 0.0030822753906250, 0.0051269531250000,
+0.0030212402343750, 0.0050964355468750, 0.0030212402343750, 0.0048522949218750,
+0.0030212402343750, 0.0049438476562500, 0.0029907226562500, 0.0048217773437500,
+0.0029602050781250, 0.0044860839843750, 0.0028686523437500, 0.0046691894531250,
+0.0031433105468750, 0.0044555664062500, 0.0028686523437500, 0.0043334960937500,
+0.0029602050781250, 0.0044860839843750, 0.0032653808593750, 0.0045166015625000,
+0.0030517578125000, 0.0044555664062500, 0.0030517578125000, 0.0043029785156250,
+0.0031738281250000, 0.0044250488281250, 0.0028686523437500, 0.0042419433593750,
+0.0026245117187500, 0.0039367675781250, 0.0027465820312500, 0.0040283203125000,
+0.0021972656250000, 0.0037841796875000, 0.0021667480468750, 0.0036621093750000,
+0.0021362304687500, 0.0038757324218750, 0.0018310546875000, 0.0038757324218750,
+0.0018615722656250, 0.0036621093750000, 0.0018920898437500, 0.0042419433593750,
+0.0017395019531250, 0.0042419433593750, 0.0018005371093750, 0.0043029785156250,
+0.0018920898437500, 0.0046386718750000, 0.0016784667968750, 0.0045776367187500,
+0.0017700195312500, 0.0042419433593750, 0.0017089843750000, 0.0043945312500000,
+0.0017089843750000, 0.0039978027343750, 0.0017395019531250, 0.0033264160156250,
+0.0016479492187500, 0.0036621093750000, 0.0020446777343750, 0.0029296875000000,
+0.0017700195312500, 0.0027770996093750, 0.0020141601562500, 0.0029296875000000,
+0.0024414062500000, 0.0027770996093750, 0.0019226074218750, 0.0027465820312500,
+0.0026550292968750, 0.0029907226562500, 0.0024719238281250, 0.0030517578125000,
+0.0024108886718750, 0.0030822753906250, 0.0027770996093750, 0.0031738281250000,
+0.0024719238281250, 0.0032043457031250, 0.0024414062500000, 0.0031433105468750,
+0.0025024414062500, 0.0031127929687500, 0.0023498535156250, 0.0031433105468750,
+0.0021972656250000, 0.0030822753906250, 0.0021667480468750, 0.0029907226562500,
+0.0021057128906250, 0.0032043457031250, 0.0020446777343750, 0.0030212402343750,
+0.0021667480468750, 0.0031433105468750, 0.0022277832031250, 0.0033569335937500,
+0.0021057128906250, 0.0031738281250000, 0.0025024414062500, 0.0034790039062500,
+0.0024108886718750, 0.0036315917968750, 0.0024108886718750, 0.0034179687500000,
+0.0025939941406250, 0.0034484863281250, 0.0024414062500000, 0.0036621093750000,
+0.0024108886718750, 0.0029602050781250, 0.0023498535156250, 0.0032348632812500,
+0.0022583007812500, 0.0028381347656250, 0.0021972656250000, 0.0021667480468750,
+0.0021972656250000, 0.0026855468750000, 0.0021667480468750, 0.0015563964843750,
+0.0022277832031250, 0.0016784667968750, 0.0022583007812500, 0.0015258789062500,
+0.0023193359375000, 0.0008544921875000, 0.0025024414062500, 0.0012817382812500,
+0.0024719238281250, 0.0007629394531250, 0.0027160644531250, 0.0007019042968750,
+0.0028076171875000, 0.0009460449218750, 0.0027465820312500, 0.0007934570312500,
+0.0029602050781250, 0.0008544921875000, 0.0028991699218750, 0.0011901855468750,
+0.0030212402343750, 0.0011596679687500, 0.0030517578125000, 0.0014038085937500,
+0.0030212402343750, 0.0016784667968750, 0.0031433105468750, 0.0016174316406250,
+0.0030822753906250, 0.0020751953125000, 0.0032043457031250, 0.0021057128906250,
+0.0032043457031250, 0.0022583007812500, 0.0031738281250000, 0.0025634765625000,
+0.0033264160156250, 0.0025024414062500, 0.0031738281250000, 0.0029296875000000,
+0.0033264160156250, 0.0028686523437500, 0.0033264160156250, 0.0029907226562500,
+0.0032348632812500, 0.0033264160156250, 0.0035095214843750, 0.0031127929687500,
+0.0033569335937500, 0.0033874511718750, 0.0034179687500000, 0.0033874511718750,
+0.0035400390625000, 0.0032043457031250, 0.0034790039062500, 0.0032653808593750,
+0.0035400390625000, 0.0032348632812500, 0.0035705566406250, 0.0030212402343750,
+0.0036010742187500, 0.0029296875000000, 0.0036621093750000, 0.0029602050781250,
+0.0036315917968750, 0.0028381347656250, 0.0036315917968750, 0.0026550292968750,
+0.0036315917968750, 0.0028381347656250, 0.0036926269531250, 0.0027465820312500,
+0.0036010742187500, 0.0028076171875000, 0.0035400390625000, 0.0030212402343750,
+0.0036010742187500, 0.0027465820312500, 0.0033264160156250, 0.0028686523437500,
+0.0033569335937500, 0.0029296875000000, 0.0032653808593750, 0.0026245117187500,
+0.0030822753906250, 0.0024108886718750, 0.0031127929687500, 0.0026245117187500,
+0.0031738281250000, 0.0018310546875000, 0.0029602050781250, 0.0019836425781250,
+0.0030822753906250, 0.0016174316406250, 0.0032043457031250, 0.0009155273437500,
+0.0030212402343750, 0.0012512207031250, 0.0032958984375000, 0.0007019042968750,
+0.0032653808593750, 0.0005798339843750, 0.0032653808593750, 0.0008239746093750,
+0.0034179687500000, 0.0008239746093750, 0.0033569335937500, 0.0007019042968750,
+0.0033264160156250, 0.0014038085937500, 0.0034179687500000, 0.0013122558593750,
+0.0034790039062500, 0.0015563964843750, 0.0034484863281250, 0.0021362304687500,
+0.0034484863281250, 0.0018615722656250, 0.0036010742187500, 0.0024414062500000,
+0.0035705566406250, 0.0024719238281250, 0.0036315917968750, 0.0025024414062500,
+0.0038146972656250, 0.0027465820312500, 0.0036926269531250, 0.0025024414062500,
+0.0036926269531250, 0.0029602050781250, 0.0038452148437500, 0.0028076171875000,
+0.0035400390625000, 0.0030517578125000, 0.0033264160156250, 0.0035095214843750,
+0.0035400390625000, 0.0032043457031250, 0.0028686523437500, 0.0040893554687500,
+0.0028686523437500, 0.0039978027343750, 0.0027160644531250, 0.0041198730468750,
+0.0022277832031250, 0.0046997070312500, 0.0023193359375000, 0.0043640136718750,
+0.0024108886718750, 0.0046691894531250, 0.0023498535156250, 0.0047607421875000,
+0.0025634765625000, 0.0046081542968750, 0.0029602050781250, 0.0045776367187500,
+0.0029602050781250, 0.0047302246093750, 0.0031738281250000, 0.0042419433593750,
+0.0033264160156250, 0.0043640136718750, 0.0032043457031250, 0.0042114257812500,
+0.0031738281250000, 0.0037841796875000, 0.0034179687500000, 0.0039672851562500,
+0.0029296875000000, 0.0037536621093750, 0.0031127929687500, 0.0036621093750000,
+0.0031433105468750, 0.0037536621093750, 0.0028686523437500, 0.0038146972656250,
+0.0031127929687500, 0.0036621093750000, 0.0031127929687500, 0.0040588378906250,
+0.0030517578125000, 0.0040283203125000, 0.0032043457031250, 0.0039062500000000,
+0.0035095214843750, 0.0042114257812500, 0.0041809082031250, 0.0040893554687500,
+0.0048217773437500, 0.0041198730468750, 0.0048217773437500, 0.0042724609375000,
+0.0047302246093750, 0.0039672851562500, 0.0047912597656250, 0.0037841796875000,
+0.0048217773437500, 0.0039672851562500, 0.0044860839843750, 0.0033264160156250,
+0.0044860839843750, 0.0033569335937500, 0.0044250488281250, 0.0032653808593750,
+0.0041809082031250},
+{0.0034179687500000, 0.0016479492187500, 0.0035095214843750, 0.0018615722656250,
+0.0033569335937500, 0.0016784667968750, 0.0035400390625000, 0.0023803710937500,
+0.0034790039062500, 0.0022277832031250, 0.0034790039062500, 0.0025939941406250,
+0.0036315917968750, 0.0031738281250000, 0.0034484863281250, 0.0028381347656250,
+0.0036010742187500, 0.0033874511718750, 0.0036010742187500, 0.0034790039062500,
+0.0035095214843750, 0.0033874511718750, 0.0035400390625000, 0.0035705566406250,
+0.0035705566406250, 0.0035095214843750, 0.0033874511718750, 0.0034790039062500,
+0.0033569335937500, 0.0034484863281250, 0.0032348632812500, 0.0033874511718750,
+0.0031433105468750, 0.0032958984375000, 0.0031738281250000, 0.0032043457031250,
+0.0030822753906250, 0.0032958984375000, 0.0030212402343750, 0.0031433105468750,
+0.0030517578125000, 0.0032043457031250, 0.0031127929687500, 0.0033569335937500,
+0.0030517578125000, 0.0032348632812500, 0.0032043457031250, 0.0032653808593750,
+0.0031433105468750, 0.0033569335937500, 0.0030822753906250, 0.0031738281250000,
+0.0031127929687500, 0.0032043457031250, 0.0030822753906250, 0.0031738281250000,
+0.0031127929687500, 0.0025634765625000, 0.0030822753906250, 0.0027160644531250,
+0.0030822753906250, 0.0024108886718750, 0.0031127929687500, 0.0017700195312500,
+0.0031738281250000, 0.0020751953125000, 0.0033569335937500, 0.0016479492187500,
+0.0033874511718750, 0.0014648437500000, 0.0034790039062500, 0.0017089843750000,
+0.0036926269531250, 0.0017089843750000, 0.0036010742187500, 0.0015869140625000,
+0.0035095214843750, 0.0021057128906250, 0.0037841796875000, 0.0020446777343750,
+0.0034179687500000, 0.0020446777343750, 0.0030822753906250, 0.0024108886718750,
+0.0035705566406250, 0.0022888183593750, 0.0028076171875000, 0.0023193359375000,
+0.0029296875000000, 0.0024414062500000, 0.0029602050781250, 0.0021972656250000,
+0.0025939941406250, 0.0020446777343750, 0.0027770996093750, 0.0022583007812500,
+0.0029296875000000, 0.0017089843750000, 0.0028686523437500, 0.0018310546875000,
+0.0031127929687500, 0.0018310546875000, 0.0034790039062500, 0.0014648437500000,
+0.0035095214843750, 0.0016479492187500, 0.0035705566406250, 0.0018005371093750,
+0.0040283203125000, 0.0016174316406250, 0.0037536621093750, 0.0018310546875000,
+0.0035400390625000, 0.0021972656250000, 0.0039978027343750, 0.0019531250000000,
+0.0032958984375000, 0.0025939941406250, 0.0032653808593750, 0.0024414062500000,
+0.0033264160156250, 0.0028076171875000, 0.0029907226562500, 0.0033874511718750,
+0.0031433105468750, 0.0029296875000000, 0.0032348632812500, 0.0040588378906250,
+0.0031127929687500, 0.0039062500000000, 0.0032958984375000, 0.0039062500000000,
+0.0035400390625000, 0.0046691894531250, 0.0034179687500000, 0.0044250488281250,
+0.0038146972656250, 0.0043029785156250, 0.0037841796875000, 0.0044250488281250,
+0.0039062500000000, 0.0043029785156250, 0.0044860839843750, 0.0040283203125000,
+0.0046997070312500, 0.0025329589843750, 0.0042419433593750, 0.0036621093750000,
+0.0044250488281250, 0.0031127929687500, 0.0043640136718750, 0.0035400390625000,
+0.0040893554687500, 0.0043029785156250, 0.0040893554687500, 0.0038146972656250,
+0.0040893554687500, 0.0044860839843750, 0.0041198730468750, 0.0045166015625000,
+0.0039367675781250, 0.0043334960937500, 0.0039367675781250, 0.0045166015625000,
+0.0041198730468750, 0.0045471191406250, 0.0035400390625000, 0.0042419433593750,
+0.0036926269531250, 0.0042114257812500, 0.0036926269531250, 0.0042419433593750,
+0.0032348632812500, 0.0041503906250000, 0.0035705566406250, 0.0040588378906250,
+0.0037536621093750, 0.0041198730468750, 0.0034179687500000, 0.0043640136718750,
+0.0040588378906250, 0.0038757324218750, 0.0047302246093750, 0.0036010742187500,
+0.0043334960937500, 0.0040283203125000, 0.0054626464843750, 0.0024719238281250,
+0.0057678222656250, 0.0028686523437500, 0.0055541992187500, 0.0024719238281250,
+0.0060119628906250, 0.0010681152343750, 0.0062866210937500, 0.0017700195312500,
+0.0056762695312500, 0.0012817382812500, 0.0060729980468750, 0.0007019042968750,
+0.0057067871093750, 0.0015258789062500, 0.0049743652343750, 0.0019531250000000,
+0.0054931640625000, 0.0013732910156250, 0.0041809082031250, 0.0031738281250000,
+0.0044555664062500, 0.0029602050781250, 0.0038757324218750, 0.0032653808593750,
+0.0026855468750000, 0.0045471191406250, 0.0033569335937500, 0.0039367675781250,
+0.0016784667968750, 0.0042724609375000, 0.0017395019531250, 0.0047912597656250,
+0.0016479492187500, 0.0040283203125000, 0.0006103515625000, 0.0035095214843750,
+0.0010681152343750, 0.0041809082031250, 0.0009765625000000, 0.0025329589843750,
+0.0007324218750000, 0.0025939941406250, 0.0009460449218750, 0.0026550292968750,
+0.0012817382812500, 0.0017700195312500, 0.0011901855468750, 0.0019836425781250,
+0.0015563964843750, 0.0024108886718750, 0.0016784667968750, 0.0020751953125000,
+0.0014038085937500, 0.0026550292968750, 0.0015258789062500, 0.0033569335937500,
+0.0017700195312500, 0.0028686523437500, 0.0006408691406250, 0.0038452148437500,
+0.0009765625000000, 0.0037841796875000, 0.0006713867187500, 0.0038452148437500,
+-0.0003356933593750, 0.0043945312500000, 0.0003051757812500, 0.0041503906250000,
+-0.0003662109375000, 0.0042114257812500, -0.0005798339843750, 0.0043029785156250,
+-0.0002136230468750, 0.0040283203125000, -0.0002746582031250, 0.0038146972656250,
+-0.0005187988281250, 0.0038452148437500, 0.0002746582031250, 0.0036315917968750,
+0.0001220703125000, 0.0034484863281250, 0.0003662109375000, 0.0035705566406250,
+0.0009765625000000, 0.0035705566406250, 0.0007324218750000, 0.0033569335937500,
+0.0012512207031250, 0.0037536621093750, 0.0014038085937500, 0.0036926269531250,
+0.0013427734375000, 0.0037231445312500, 0.0015258789062500, 0.0039978027343750,
+0.0014953613281250, 0.0038146972656250, 0.0013732910156250, 0.0039062500000000,
+0.0013427734375000, 0.0039672851562500, 0.0013427734375000, 0.0037841796875000,
+0.0012512207031250, 0.0038146972656250, 0.0011291503906250, 0.0039062500000000,
+0.0013427734375000, 0.0034484863281250, 0.0011291503906250, 0.0036315917968750,
+0.0012207031250000, 0.0034484863281250, 0.0015258789062500, 0.0030212402343750,
+0.0012207031250000, 0.0032653808593750, 0.0014038085937500, 0.0028686523437500,
+0.0014648437500000, 0.0028076171875000, 0.0011901855468750, 0.0028381347656250,
+0.0010681152343750, 0.0026245117187500, 0.0012512207031250, 0.0025939941406250,
+0.0008544921875000, 0.0025634765625000, 0.0008239746093750, 0.0025024414062500,
+0.0009155273437500, 0.0024719238281250, 0.0008239746093750, 0.0025024414062500,
+0.0007934570312500, 0.0024719238281250, 0.0011596679687500, 0.0024719238281250,
+0.0010681152343750, 0.0025329589843750, 0.0012512207031250, 0.0025024414062500,
+0.0016174316406250, 0.0024108886718750, 0.0014648437500000, 0.0024719238281250,
+0.0021667480468750, 0.0024414062500000, 0.0019531250000000, 0.0024108886718750,
+0.0024108886718750, 0.0024414062500000, 0.0031127929687500, 0.0025024414062500,
+0.0026855468750000, 0.0023498535156250, 0.0039062500000000, 0.0025634765625000,
+0.0037536621093750, 0.0025634765625000, 0.0040588378906250, 0.0025634765625000,
+0.0050048828125000, 0.0025939941406250, 0.0046081542968750, 0.0025634765625000,
+0.0054931640625000, 0.0025939941406250, 0.0054321289062500, 0.0025634765625000,
+0.0054626464843750, 0.0025634765625000, 0.0058898925781250, 0.0025939941406250,
+0.0056762695312500, 0.0025939941406250, 0.0056152343750000, 0.0027770996093750,
+0.0057373046875000, 0.0026855468750000, 0.0053405761718750, 0.0027465820312500,
+0.0050354003906250, 0.0029602050781250, 0.0051574707031250, 0.0028991699218750,
+0.0042114257812500, 0.0029602050781250, 0.0044250488281250, 0.0030517578125000,
+0.0038757324218750, 0.0031127929687500, 0.0030212402343750, 0.0031433105468750,
+0.0036010742187500, 0.0031433105468750, 0.0022277832031250, 0.0030212402343750,
+0.0024414062500000, 0.0030517578125000, 0.0022277832031250, 0.0029602050781250,
+0.0014038085937500, 0.0028076171875000, 0.0018920898437500, 0.0029296875000000,
+0.0012207031250000, 0.0026245117187500, 0.0011901855468750, 0.0026855468750000,
+0.0013427734375000, 0.0026550292968750, 0.0011291503906250, 0.0024719238281250,
+0.0013732910156250, 0.0024719238281250, 0.0017700195312500, 0.0026550292968750,
+0.0016784667968750, 0.0025329589843750, 0.0020751953125000, 0.0026550292968750,
+0.0026855468750000, 0.0028991699218750, 0.0025634765625000, 0.0026855468750000,
+0.0029602050781250, 0.0030212402343750, 0.0033569335937500, 0.0030212402343750,
+0.0030212402343750, 0.0029907226562500, 0.0028991699218750, 0.0031433105468750,
+0.0033874511718750, 0.0031433105468750, 0.0022583007812500, 0.0029907226562500,
+0.0024414062500000, 0.0030517578125000, 0.0022888183593750, 0.0029907226562500,
+0.0015563964843750, 0.0027770996093750, 0.0019226074218750, 0.0029296875000000,
+0.0018005371093750, 0.0026550292968750, 0.0015563964843750, 0.0028381347656250,
+0.0019226074218750, 0.0027770996093750, 0.0021972656250000, 0.0025634765625000,
+0.0019226074218750, 0.0028076171875000, 0.0026245117187500, 0.0026550292968750,
+0.0026245117187500, 0.0026550292968750, 0.0025329589843750, 0.0027770996093750,
+0.0028686523437500, 0.0026855468750000, 0.0028381347656250, 0.0026855468750000,
+0.0026550292968750, 0.0031127929687500, 0.0027770996093750, 0.0029907226562500,
+0.0026855468750000, 0.0032958984375000, 0.0024414062500000, 0.0037841796875000,
+0.0026855468750000, 0.0033874511718750, 0.0024719238281250, 0.0039062500000000,
+0.0025024414062500, 0.0039978027343750, 0.0025939941406250, 0.0038452148437500,
+0.0025634765625000, 0.0039367675781250, 0.0025024414062500, 0.0039367675781250,
+0.0021667480468750, 0.0038452148437500, 0.0021362304687500, 0.0038146972656250,
+0.0020751953125000, 0.0039672851562500, 0.0017395019531250, 0.0040283203125000,
+0.0017395019531250, 0.0039062500000000, 0.0020751953125000, 0.0041198730468750,
+0.0016784667968750, 0.0039978027343750, 0.0022888183593750, 0.0040283203125000,
+0.0029602050781250, 0.0042114257812500, 0.0024108886718750, 0.0040893554687500,
+0.0041198730468750, 0.0040588378906250, 0.0039062500000000, 0.0041503906250000,
+0.0041809082031250, 0.0040283203125000, 0.0058593750000000, 0.0038757324218750,
+0.0058593750000000, 0.0039062500000000, 0.0063171386718750, 0.0039062500000000,
+0.0063476562500000, 0.0038146972656250, 0.0061645507812500, 0.0039367675781250,
+0.0063171386718750, 0.0040893554687500, 0.0063171386718750, 0.0039978027343750,
+0.0057067871093750, 0.0045776367187500, 0.0059204101562500, 0.0045166015625000,
+0.0055236816406250, 0.0047607421875000, 0.0050354003906250, 0.0053710937500000,
+0.0054321289062500, 0.0050964355468750, 0.0049133300781250, 0.0055236816406250,
+0.0047302246093750, 0.0057373046875000, 0.0048522949218750, 0.0054321289062500,
+0.0047607421875000, 0.0053710937500000, 0.0046386718750000, 0.0055541992187500,
+0.0043334960937500, 0.0046081542968750, 0.0044250488281250, 0.0047302246093750,
+0.0039978027343750, 0.0045471191406250, 0.0035705566406250, 0.0038452148437500,
+0.0038452148437500, 0.0040588378906250, 0.0030212402343750, 0.0037841796875000,
+0.0029907226562500, 0.0038146972656250, 0.0029602050781250, 0.0037841796875000,
+0.0025329589843750, 0.0036621093750000, 0.0026245117187500, 0.0039062500000000,
+0.0026855468750000},
+{0.0016784667968750, 0.0023803710937500, 0.0015563964843750, 0.0023803710937500,
+0.0016479492187500, 0.0023193359375000, 0.0015258789062500, 0.0025024414062500,
+0.0013427734375000, 0.0026245117187500, 0.0015563964843750, 0.0025939941406250,
+0.0010681152343750, 0.0028686523437500, 0.0011596679687500, 0.0028991699218750,
+0.0011596679687500, 0.0029296875000000, 0.0008239746093750, 0.0031433105468750,
+0.0010681152343750, 0.0030822753906250, 0.0012207031250000, 0.0030822753906250,
+0.0010986328125000, 0.0031738281250000, 0.0015258789062500, 0.0031127929687500,
+0.0019836425781250, 0.0029907226562500, 0.0016784667968750, 0.0030822753906250,
+0.0022888183593750, 0.0029602050781250, 0.0024108886718750, 0.0029296875000000,
+0.0020751953125000, 0.0028381347656250, 0.0021972656250000, 0.0027160644531250,
+0.0022277832031250, 0.0027465820312500, 0.0013427734375000, 0.0026245117187500,
+0.0015258789062500, 0.0026245117187500, 0.0012817382812500, 0.0027465820312500,
+0.0005187988281250, 0.0026550292968750, 0.0007629394531250, 0.0025939941406250,
+0.0006103515625000, 0.0030212402343750, 0.0003356933593750, 0.0027770996093750,
+0.0006103515625000, 0.0030517578125000, 0.0009765625000000, 0.0035705566406250,
+0.0007019042968750, 0.0031738281250000, 0.0014038085937500, 0.0038146972656250,
+0.0015258789062500, 0.0038452148437500, 0.0016174316406250, 0.0037536621093750,
+0.0020751953125000, 0.0039672851562500, 0.0021057128906250, 0.0039062500000000,
+0.0022277832031250, 0.0037841796875000, 0.0023498535156250, 0.0037536621093750,
+0.0027160644531250, 0.0037536621093750, 0.0029296875000000, 0.0037231445312500,
+0.0028991699218750, 0.0037231445312500, 0.0039367675781250, 0.0036926269531250,
+0.0038757324218750, 0.0036926269531250, 0.0039978027343750, 0.0036315917968750,
+0.0047302246093750, 0.0036315917968750, 0.0044555664062500, 0.0036926269531250,
+0.0042114257812500, 0.0036315917968750, 0.0045166015625000, 0.0036010742187500,
+0.0038452148437500, 0.0036010742187500, 0.0031127929687500, 0.0036621093750000,
+0.0035095214843750, 0.0037841796875000, 0.0024719238281250, 0.0037231445312500,
+0.0022583007812500, 0.0038757324218750, 0.0025024414062500, 0.0039367675781250,
+0.0022277832031250, 0.0039062500000000, 0.0024108886718750, 0.0042114257812500,
+0.0035705566406250, 0.0042724609375000, 0.0031738281250000, 0.0042724609375000,
+0.0035705566406250, 0.0044250488281250, 0.0046386718750000, 0.0046997070312500,
+0.0042114257812500, 0.0047607421875000, 0.0053100585937500, 0.0048828125000000,
+0.0053710937500000, 0.0051879882812500, 0.0052185058593750, 0.0048828125000000,
+0.0058288574218750, 0.0046997070312500, 0.0056457519531250, 0.0051879882812500,
+0.0052795410156250, 0.0039978027343750, 0.0055847167968750, 0.0042724609375000,
+0.0053100585937500, 0.0041198730468750, 0.0046691894531250, 0.0032653808593750,
+0.0048828125000000, 0.0037841796875000, 0.0044555664062500, 0.0033264160156250,
+0.0042419433593750, 0.0032348632812500, 0.0043334960937500, 0.0032653808593750,
+0.0041809082031250, 0.0032348632812500, 0.0038146972656250, 0.0033569335937500,
+0.0040283203125000, 0.0028381347656250, 0.0039978027343750, 0.0031738281250000,
+0.0037231445312500, 0.0028991699218750, 0.0036621093750000, 0.0023193359375000,
+0.0036010742187500, 0.0028381347656250, 0.0032958984375000, 0.0022583007812500,
+0.0032043457031250, 0.0022583007812500, 0.0032043457031250, 0.0024719238281250,
+0.0029907226562500, 0.0022888183593750, 0.0030517578125000, 0.0025329589843750,
+0.0032653808593750, 0.0026855468750000, 0.0029602050781250, 0.0026855468750000,
+0.0032653808593750, 0.0028686523437500, 0.0036621093750000, 0.0031738281250000,
+0.0032653808593750, 0.0031433105468750, 0.0038757324218750, 0.0032348632812500,
+0.0038146972656250, 0.0034790039062500, 0.0037231445312500, 0.0033264160156250,
+0.0039672851562500, 0.0032653808593750, 0.0039672851562500, 0.0036010742187500,
+0.0036010742187500, 0.0032348632812500, 0.0037536621093750, 0.0034179687500000,
+0.0036010742187500, 0.0035095214843750, 0.0031738281250000, 0.0033569335937500,
+0.0033569335937500, 0.0036315917968750, 0.0032958984375000, 0.0036010742187500,
+0.0031738281250000, 0.0036926269531250, 0.0033569335937500, 0.0038452148437500,
+0.0036010742187500, 0.0039062500000000, 0.0035095214843750, 0.0039367675781250,
+0.0038452148437500, 0.0039978027343750, 0.0039367675781250, 0.0041198730468750,
+0.0038757324218750, 0.0042724609375000, 0.0039978027343750, 0.0043945312500000,
+0.0042114257812500, 0.0044860839843750, 0.0036010742187500, 0.0045776367187500,
+0.0039367675781250, 0.0048522949218750, 0.0037231445312500, 0.0047302246093750,
+0.0031738281250000, 0.0046691894531250, 0.0036315917968750, 0.0049743652343750,
+0.0027770996093750, 0.0040893554687500, 0.0029907226562500, 0.0044250488281250,
+0.0029602050781250, 0.0040893554687500, 0.0023498535156250, 0.0032653808593750,
+0.0028686523437500, 0.0037536621093750, 0.0024108886718750, 0.0028381347656250,
+0.0025329589843750, 0.0028991699218750, 0.0026550292968750, 0.0028991699218750,
+0.0024108886718750, 0.0024414062500000, 0.0026245117187500, 0.0026855468750000,
+0.0025939941406250, 0.0025634765625000, 0.0025939941406250, 0.0025329589843750,
+0.0027465820312500, 0.0026245117187500, 0.0028076171875000, 0.0026550292968750,
+0.0027465820312500, 0.0026245117187500, 0.0031738281250000, 0.0025634765625000,
+0.0030822753906250, 0.0026855468750000, 0.0031738281250000, 0.0024719238281250,
+0.0035095214843750, 0.0022583007812500, 0.0032958984375000, 0.0024108886718750,
+0.0036010742187500, 0.0017700195312500, 0.0036010742187500, 0.0018615722656250,
+0.0036010742187500, 0.0017089843750000, 0.0037536621093750, 0.0012207031250000,
+0.0036010742187500, 0.0014038085937500, 0.0036926269531250, 0.0012207031250000,
+0.0036926269531250, 0.0010986328125000, 0.0036926269531250, 0.0012512207031250,
+0.0037231445312500, 0.0013427734375000, 0.0036926269531250, 0.0011901855468750,
+0.0036621093750000, 0.0019531250000000, 0.0036621093750000, 0.0017700195312500,
+0.0036010742187500, 0.0021057128906250, 0.0035095214843750, 0.0028076171875000,
+0.0034790039062500, 0.0023498535156250, 0.0034179687500000, 0.0030212402343750,
+0.0032958984375000, 0.0031127929687500, 0.0032653808593750, 0.0028381347656250,
+0.0031738281250000, 0.0031127929687500, 0.0031127929687500, 0.0032653808593750,
+0.0031433105468750, 0.0025634765625000, 0.0031127929687500, 0.0029907226562500,
+0.0031127929687500, 0.0027465820312500, 0.0031738281250000, 0.0020446777343750,
+0.0030822753906250, 0.0025939941406250, 0.0032043457031250, 0.0016174316406250,
+0.0032043457031250, 0.0018005371093750, 0.0031433105468750, 0.0016479492187500,
+0.0031433105468750, 0.0010375976562500, 0.0031738281250000, 0.0014648437500000,
+0.0031738281250000, 0.0010070800781250, 0.0031738281250000, 0.0009460449218750,
+0.0031127929687500, 0.0011291503906250, 0.0030517578125000, 0.0010070800781250,
+0.0031433105468750, 0.0010375976562500, 0.0029602050781250, 0.0013732910156250,
+0.0029602050781250, 0.0012512207031250, 0.0029907226562500, 0.0015258789062500,
+0.0029296875000000, 0.0018920898437500, 0.0029907226562500, 0.0016784667968750,
+0.0028991699218750, 0.0023193359375000, 0.0029602050781250, 0.0022888183593750,
+0.0029907226562500, 0.0024414062500000, 0.0029296875000000, 0.0028991699218750,
+0.0030212402343750, 0.0026245117187500, 0.0030212402343750, 0.0030212402343750,
+0.0030212402343750, 0.0030517578125000, 0.0032043457031250, 0.0029907226562500,
+0.0032958984375000, 0.0031433105468750, 0.0032653808593750, 0.0031127929687500,
+0.0034484863281250, 0.0029602050781250, 0.0035705566406250, 0.0030517578125000,
+0.0035705566406250, 0.0028076171875000, 0.0036315917968750, 0.0025634765625000,
+0.0037841796875000, 0.0026855468750000, 0.0035705566406250, 0.0021972656250000,
+0.0037841796875000, 0.0021667480468750, 0.0036315917968750, 0.0021972656250000,
+0.0032958984375000, 0.0018920898437500, 0.0036315917968750, 0.0019226074218750,
+0.0030212402343750, 0.0021362304687500, 0.0030517578125000, 0.0021362304687500,
+0.0032043457031250, 0.0020751953125000, 0.0028686523437500, 0.0022888183593750,
+0.0029296875000000, 0.0023498535156250, 0.0033264160156250, 0.0017395019531250,
+0.0031433105468750, 0.0021057128906250, 0.0034179687500000, 0.0017395019531250,
+0.0039062500000000, 0.0009765625000000, 0.0036010742187500, 0.0015563964843750,
+0.0039062500000000, 0.0006103515625000, 0.0039978027343750, 0.0006408691406250,
+0.0038452148437500, 0.0009460449218750, 0.0038452148437500, 0.0004882812500000,
+0.0038452148437500, 0.0004882812500000, 0.0038146972656250, 0.0017700195312500,
+0.0037536621093750, 0.0012817382812500, 0.0039978027343750, 0.0020751953125000,
+0.0041503906250000, 0.0034179687500000, 0.0040283203125000, 0.0026550292968750,
+0.0046691894531250, 0.0043640136718750, 0.0044860839843750, 0.0044250488281250,
+0.0045471191406250, 0.0045776367187500, 0.0050659179687500, 0.0054931640625000,
+0.0047607421875000, 0.0051574707031250, 0.0050354003906250, 0.0056457519531250,
+0.0051269531250000, 0.0057373046875000, 0.0049438476562500, 0.0057067871093750,
+0.0049438476562500, 0.0061035156250000, 0.0050354003906250, 0.0060424804687500,
+0.0046691894531250, 0.0053710937500000, 0.0047607421875000, 0.0059814453125000,
+0.0046997070312500, 0.0050354003906250, 0.0043640136718750, 0.0038757324218750,
+0.0044250488281250, 0.0049438476562500, 0.0042724609375000, 0.0024414062500000,
+0.0041503906250000, 0.0024719238281250, 0.0040893554687500, 0.0026245117187500,
+0.0039672851562500, 0.0007629394531250, 0.0039367675781250, 0.0065002441406250,
+0.0035400390625000, 0.0049438476562500, 0.0035705566406250, 0.0057067871093750,
+0.0037536621093750, 0.0048522949218750, 0.0036621093750000, 0.0036926269531250,
+0.0036621093750000, 0.0043945312500000, 0.0046386718750000, 0.0028991699218750,
+0.0045166015625000, 0.0031433105468750, 0.0049133300781250, 0.0028686523437500,
+0.0058593750000000, 0.0017089843750000, 0.0056152343750000, 0.0022277832031250,
+0.0057067871093750, 0.0018005371093750, 0.0063476562500000, 0.0014343261718750,
+0.0057678222656250, 0.0021362304687500, 0.0052490234375000, 0.0025024414062500,
+0.0059814453125000, 0.0020751953125000, 0.0041503906250000, 0.0035400390625000,
+0.0046081542968750, 0.0035095214843750, 0.0041198730468750, 0.0035705566406250,
+0.0027160644531250, 0.0043945312500000, 0.0036315917968750, 0.0041809082031250,
+0.0022583007812500, 0.0044250488281250, 0.0022888183593750, 0.0045166015625000,
+0.0023193359375000, 0.0046386718750000, 0.0015869140625000, 0.0048522949218750,
+0.0019836425781250, 0.0047607421875000, 0.0018005371093750, 0.0050354003906250,
+0.0014648437500000, 0.0051879882812500, 0.0019836425781250, 0.0049133300781250,
+0.0023498535156250, 0.0048522949218750, 0.0019531250000000, 0.0050964355468750,
+0.0034790039062500, 0.0042419433593750, 0.0032043457031250, 0.0044555664062500,
+0.0035400390625000, 0.0041503906250000, 0.0047302246093750, 0.0033264160156250,
+0.0041503906250000, 0.0037231445312500, 0.0046081542968750, 0.0028381347656250,
+0.0049743652343750, 0.0026855468750000, 0.0043640136718750, 0.0028381347656250,
+0.0040283203125000, 0.0024719238281250, 0.0044860839843750, 0.0023803710937500,
+0.0032348632812500, 0.0031738281250000, 0.0033569335937500, 0.0029296875000000,
+0.0033264160156250},
+{0.0034484863281250, 0.0024719238281250, 0.0035705566406250, 0.0026550292968750,
+0.0039367675781250, 0.0025024414062500, 0.0036621093750000, 0.0024414062500000,
+0.0039367675781250, 0.0030517578125000, 0.0039062500000000, 0.0029907226562500,
+0.0038757324218750, 0.0030822753906250, 0.0039062500000000, 0.0035705566406250,
+0.0037536621093750, 0.0034790039062500, 0.0039367675781250, 0.0033569335937500,
+0.0037536621093750, 0.0037231445312500, 0.0039062500000000, 0.0032043457031250,
+0.0041809082031250, 0.0027160644531250, 0.0039062500000000, 0.0032653808593750,
+0.0046997070312500, 0.0016174316406250, 0.0046691894531250, 0.0018310546875000,
+0.0048217773437500, 0.0016784667968750, 0.0053100585937500, 0.0005798339843750,
+0.0049743652343750, 0.0010986328125000, 0.0052795410156250, 0.0007629394531250,
+0.0052795410156250, 0.0005493164062500, 0.0050659179687500, 0.0009460449218750,
+0.0051574707031250, 0.0011596679687500, 0.0050964355468750, 0.0009765625000000,
+0.0049438476562500, 0.0016174316406250, 0.0050048828125000, 0.0015563964843750,
+0.0050048828125000, 0.0018310546875000, 0.0048828125000000, 0.0022583007812500,
+0.0049133300781250, 0.0020141601562500, 0.0046997070312500, 0.0033264160156250,
+0.0046997070312500, 0.0028991699218750, 0.0045776367187500, 0.0037536621093750,
+0.0044250488281250, 0.0052795410156250, 0.0045166015625000, 0.0043640136718750,
+0.0042724609375000, 0.0060119628906250, 0.0043029785156250, 0.0063781738281250,
+0.0041503906250000, 0.0058898925781250, 0.0036621093750000, 0.0062255859375000,
+0.0043334960937500, 0.0019531250000000, 0.0054321289062500, 0.0051269531250000,
+0.0050048828125000, 0.0041503906250000, 0.0051574707031250, 0.0048217773437500,
+0.0059204101562500, 0.0063476562500000, 0.0057678222656250, 0.0056457519531250,
+0.0051879882812500, 0.0061035156250000, 0.0058898925781250, 0.0062255859375000,
+0.0050048828125000, 0.0058288574218750, 0.0039062500000000, 0.0058898925781250,
+0.0049133300781250, 0.0059509277343750, 0.0028991699218750, 0.0046081542968750,
+0.0030212402343750, 0.0051574707031250, 0.0029296875000000, 0.0042724609375000,
+0.0018005371093750, 0.0026245117187500, 0.0024414062500000, 0.0036315917968750,
+0.0018310546875000, 0.0021667480468750, 0.0017395019531250, 0.0018310546875000,
+0.0019836425781250, 0.0024414062500000, 0.0018310546875000, 0.0021667480468750,
+0.0019226074218750, 0.0019531250000000, 0.0023193359375000, 0.0030517578125000,
+0.0021362304687500, 0.0028381347656250, 0.0025939941406250, 0.0029907226562500,
+0.0031127929687500, 0.0036926269531250, 0.0026855468750000, 0.0034179687500000,
+0.0038757324218750, 0.0038757324218750, 0.0037841796875000, 0.0037841796875000,
+0.0037841796875000, 0.0039062500000000, 0.0044860839843750, 0.0041809082031250,
+0.0042419433593750, 0.0038757324218750, 0.0039367675781250, 0.0043334960937500,
+0.0043029785156250, 0.0043029785156250, 0.0037536621093750, 0.0042724609375000,
+0.0030517578125000, 0.0045471191406250, 0.0035705566406250, 0.0044555664062500,
+0.0026245117187500, 0.0043029785156250, 0.0024414062500000, 0.0045776367187500,
+0.0026855468750000, 0.0040588378906250, 0.0023803710937500, 0.0036621093750000,
+0.0023498535156250, 0.0040588378906250, 0.0031127929687500, 0.0026245117187500,
+0.0028381347656250, 0.0029296875000000, 0.0031433105468750, 0.0026245117187500,
+0.0038452148437500, 0.0014953613281250, 0.0034484863281250, 0.0021972656250000,
+0.0042114257812500, 0.0017395019531250, 0.0042419433593750, 0.0015563964843750,
+0.0041198730468750, 0.0021972656250000, 0.0043945312500000, 0.0024108886718750,
+0.0043640136718750, 0.0020751953125000, 0.0041503906250000, 0.0030212402343750,
+0.0041503906250000, 0.0030822753906250, 0.0040893554687500, 0.0030212402343750,
+0.0039978027343750, 0.0034790039062500, 0.0040893554687500, 0.0035095214843750,
+0.0039672851562500, 0.0028686523437500, 0.0040283203125000, 0.0030517578125000,
+0.0039672851562500, 0.0027465820312500, 0.0039367675781250, 0.0020751953125000,
+0.0041503906250000, 0.0022888183593750, 0.0037231445312500, 0.0017395019531250,
+0.0038452148437500, 0.0015563964843750, 0.0037841796875000, 0.0017395019531250,
+0.0034179687500000, 0.0016174316406250, 0.0035705566406250, 0.0014343261718750,
+0.0035095214843750, 0.0021057128906250, 0.0032653808593750, 0.0019531250000000,
+0.0035400390625000, 0.0022277832031250, 0.0037231445312500, 0.0028686523437500,
+0.0033569335937500, 0.0025329589843750, 0.0042419433593750, 0.0030212402343750,
+0.0038757324218750, 0.0030212402343750, 0.0043029785156250, 0.0029296875000000,
+0.0050659179687500, 0.0031127929687500, 0.0044555664062500, 0.0030212402343750,
+0.0057373046875000, 0.0029907226562500, 0.0055847167968750, 0.0030822753906250,
+0.0056762695312500, 0.0030212402343750, 0.0064697265625000, 0.0029602050781250,
+0.0060119628906250, 0.0031127929687500, 0.0065612792968750, 0.0030517578125000,
+0.0066528320312500, 0.0030517578125000, 0.0065307617187500, 0.0031127929687500,
+0.0067443847656250, 0.0031127929687500, 0.0067749023437500, 0.0030517578125000,
+0.0065002441406250, 0.0032958984375000, 0.0067443847656250, 0.0032348632812500,
+0.0065307617187500, 0.0031433105468750, 0.0062561035156250, 0.0032043457031250,
+0.0066833496093750, 0.0031738281250000, 0.0059509277343750, 0.0030517578125000,
+0.0061950683593750, 0.0030822753906250, 0.0059204101562500, 0.0030212402343750,
+0.0053100585937500, 0.0029296875000000, 0.0056762695312500, 0.0029296875000000,
+0.0046997070312500, 0.0029296875000000, 0.0048217773437500, 0.0028381347656250,
+0.0043945312500000, 0.0028991699218750, 0.0035095214843750, 0.0029602050781250,
+0.0039672851562500, 0.0028991699218750, 0.0028381347656250, 0.0029907226562500,
+0.0028686523437500, 0.0029602050781250, 0.0025024414062500, 0.0028991699218750,
+0.0017089843750000, 0.0029602050781250, 0.0019836425781250, 0.0029907226562500,
+0.0010986328125000, 0.0027770996093750, 0.0011291503906250, 0.0028381347656250,
+0.0008850097656250, 0.0027770996093750, 0.0003356933593750, 0.0026550292968750,
+0.0006103515625000, 0.0028076171875000, 0.0000915527343750, 0.0026245117187500,
+0.0001525878906250, 0.0025634765625000, 0.0002441406250000, 0.0026550292968750,
+0.0000305175781250, 0.0025634765625000, 0.0000610351562500, 0.0025634765625000,
+0.0002136230468750, 0.0026245117187500, 0.0001220703125000, 0.0026245117187500,
+0.0002136230468750, 0.0027465820312500, 0.0003967285156250, 0.0028686523437500,
+0.0002136230468750, 0.0027160644531250, 0.0004272460937500, 0.0030517578125000,
+0.0004272460937500, 0.0031433105468750, 0.0004272460937500, 0.0031127929687500,
+0.0005493164062500, 0.0031738281250000, 0.0004577636718750, 0.0031738281250000,
+0.0006408691406250, 0.0032043457031250, 0.0006713867187500, 0.0032043457031250,
+0.0007019042968750, 0.0031738281250000, 0.0007324218750000, 0.0031433105468750,
+0.0006713867187500, 0.0031127929687500, 0.0010375976562500, 0.0029602050781250,
+0.0008850097656250, 0.0030822753906250, 0.0010070800781250, 0.0028991699218750,
+0.0012817382812500, 0.0027465820312500, 0.0010375976562500, 0.0028991699218750,
+0.0016479492187500, 0.0025329589843750, 0.0015258789062500, 0.0026550292968750,
+0.0016784667968750, 0.0025634765625000, 0.0020446777343750, 0.0022277832031250,
+0.0018310546875000, 0.0025329589843750, 0.0021667480468750, 0.0024414062500000,
+0.0021057128906250, 0.0023803710937500, 0.0021972656250000, 0.0026855468750000,
+0.0024108886718750, 0.0029602050781250, 0.0021972656250000, 0.0028686523437500,
+0.0028076171875000, 0.0032958984375000, 0.0026245117187500, 0.0034179687500000,
+0.0028381347656250, 0.0034179687500000, 0.0033569335937500, 0.0035705566406250,
+0.0030517578125000, 0.0035705566406250, 0.0036010742187500, 0.0034790039062500,
+0.0037231445312500, 0.0034790039062500, 0.0036926269531250, 0.0034484863281250,
+0.0039062500000000, 0.0032958984375000, 0.0038146972656250, 0.0032653808593750,
+0.0036010742187500, 0.0032653808593750, 0.0036010742187500, 0.0032043457031250,
+0.0036010742187500, 0.0031433105468750, 0.0034179687500000, 0.0031433105468750,
+0.0033874511718750, 0.0030517578125000, 0.0034179687500000, 0.0030212402343750,
+0.0033264160156250, 0.0029907226562500, 0.0032958984375000, 0.0030822753906250,
+0.0034484863281250, 0.0031127929687500, 0.0033569335937500, 0.0029602050781250,
+0.0027465820312500, 0.0033874511718750, 0.0030822753906250, 0.0032958984375000,
+0.0025329589843750, 0.0034790039062500, 0.0016479492187500, 0.0039062500000000,
+0.0021057128906250, 0.0036621093750000, 0.0009155273437500, 0.0041809082031250,
+0.0008239746093750, 0.0042419433593750, 0.0009765625000000, 0.0042419433593750,
+0.0005187988281250, 0.0045166015625000, 0.0006713867187500, 0.0044250488281250,
+0.0013122558593750, 0.0043334960937500, 0.0011901855468750, 0.0044555664062500,
+0.0016174316406250, 0.0041809082031250, 0.0025329589843750, 0.0039062500000000,
+0.0028076171875000, 0.0039367675781250, 0.0032958984375000, 0.0033264160156250,
+0.0033569335937500, 0.0035095214843750, 0.0032348632812500, 0.0034790039062500,
+0.0034484863281250, 0.0029907226562500, 0.0036621093750000, 0.0031433105468750,
+0.0031127929687500, 0.0030822753906250, 0.0034179687500000, 0.0029296875000000,
+0.0032043457031250, 0.0030517578125000, 0.0027770996093750, 0.0031127929687500,
+0.0031433105468750, 0.0029907226562500, 0.0022583007812500, 0.0029907226562500,
+0.0021972656250000, 0.0029602050781250, 0.0022888183593750, 0.0029907226562500,
+0.0018310546875000, 0.0029602050781250, 0.0019226074218750, 0.0028076171875000,
+0.0023803710937500, 0.0033264160156250, 0.0020751953125000, 0.0030517578125000,
+0.0025329589843750, 0.0034179687500000, 0.0031127929687500, 0.0040283203125000,
+0.0027160644531250, 0.0035705566406250, 0.0033569335937500, 0.0046997070312500,
+0.0032043457031250, 0.0046386718750000, 0.0031127929687500, 0.0045776367187500,
+0.0033569335937500, 0.0051574707031250, 0.0030517578125000, 0.0050048828125000,
+0.0030212402343750, 0.0048217773437500, 0.0029907226562500, 0.0050354003906250,
+0.0028991699218750, 0.0045776367187500, 0.0028381347656250, 0.0041503906250000,
+0.0027770996093750, 0.0044860839843750, 0.0027160644531250, 0.0033569335937500,
+0.0027770996093750, 0.0034484863281250, 0.0026550292968750, 0.0033569335937500,
+0.0025634765625000, 0.0026855468750000, 0.0027160644531250, 0.0029602050781250,
+0.0023803710937500, 0.0028381347656250, 0.0024719238281250, 0.0025329589843750,
+0.0024108886718750, 0.0028686523437500, 0.0021362304687500, 0.0030822753906250,
+0.0023803710937500, 0.0027160644531250, 0.0020141601562500, 0.0033569335937500,
+0.0020141601562500, 0.0032043457031250, 0.0023193359375000, 0.0032348632812500,
+0.0022888183593750, 0.0036926269531250, 0.0022277832031250, 0.0034179687500000,
+0.0028686523437500, 0.0036621093750000, 0.0028381347656250, 0.0036621093750000,
+0.0029296875000000, 0.0035095214843750, 0.0034179687500000, 0.0035400390625000,
+0.0033569335937500, 0.0035095214843750, 0.0031127929687500, 0.0032348632812500,
+0.0034790039062500, 0.0032958984375000, 0.0031738281250000, 0.0032348632812500,
+0.0027465820312500, 0.0029602050781250, 0.0032043457031250, 0.0030822753906250,
+0.0026245117187500, 0.0032653808593750, 0.0026245117187500, 0.0031433105468750,
+0.0028686523437500, 0.0034179687500000, 0.0027160644531250, 0.0037536621093750,
+0.0027465820312500},
+{0.0016784667968750, 0.0031738281250000, 0.0020446777343750, 0.0031738281250000,
+0.0013732910156250, 0.0035400390625000, 0.0014038085937500, 0.0032348632812500,
+0.0015869140625000, 0.0034790039062500, 0.0011901855468750, 0.0039672851562500,
+0.0012817382812500, 0.0035095214843750, 0.0018005371093750, 0.0040283203125000,
+0.0016174316406250, 0.0039672851562500, 0.0019531250000000, 0.0039062500000000,
+0.0025329589843750, 0.0041809082031250, 0.0023803710937500, 0.0040588378906250,
+0.0031127929687500, 0.0038146972656250, 0.0031738281250000, 0.0039672851562500,
+0.0030822753906250, 0.0037231445312500, 0.0033569335937500, 0.0033874511718750,
+0.0033264160156250, 0.0036315917968750, 0.0027465820312500, 0.0032348632812500,
+0.0029602050781250, 0.0030822753906250, 0.0026550292968750, 0.0032043457031250,
+0.0017395019531250, 0.0032348632812500, 0.0015869140625000, 0.0031433105468750,
+0.0013122558593750, 0.0031433105468750, 0.0012512207031250, 0.0032348632812500,
+0.0014953613281250, 0.0031127929687500, 0.0014953613281250, 0.0031127929687500,
+0.0014038085937500, 0.0033874511718750, 0.0020141601562500, 0.0029907226562500,
+0.0020141601562500, 0.0030212402343750, 0.0023803710937500, 0.0031127929687500,
+0.0029296875000000, 0.0028076171875000, 0.0027465820312500, 0.0028991699218750,
+0.0032958984375000, 0.0033264160156250, 0.0036315917968750, 0.0029296875000000,
+0.0033569335937500, 0.0034179687500000, 0.0033264160156250, 0.0041198730468750,
+0.0034790039062500, 0.0036315917968750, 0.0026245117187500, 0.0044555664062500,
+0.0027770996093750, 0.0046081542968750, 0.0027160644531250, 0.0043640136718750,
+0.0021362304687500, 0.0046691894531250, 0.0023193359375000, 0.0046691894531250,
+0.0024414062500000, 0.0040283203125000, 0.0021362304687500, 0.0041198730468750,
+0.0024108886718750, 0.0036926269531250, 0.0026245117187500, 0.0030212402343750,
+0.0021972656250000, 0.0032958984375000, 0.0028381347656250, 0.0025024414062500,
+0.0027465820312500, 0.0024108886718750, 0.0026550292968750, 0.0024108886718750,
+0.0029296875000000, 0.0020141601562500, 0.0026855468750000, 0.0020446777343750,
+0.0026855468750000, 0.0022888183593750, 0.0025939941406250, 0.0022277832031250,
+0.0025939941406250, 0.0023193359375000, 0.0026245117187500, 0.0025634765625000,
+0.0024719238281250, 0.0025939941406250, 0.0026550292968750, 0.0028076171875000,
+0.0023803710937500, 0.0028381347656250, 0.0024414062500000, 0.0028991699218750,
+0.0026855468750000, 0.0030822753906250, 0.0023193359375000, 0.0029602050781250,
+0.0025024414062500, 0.0029296875000000, 0.0025634765625000, 0.0029296875000000,
+0.0022888183593750, 0.0027160644531250, 0.0021362304687500, 0.0025939941406250,
+0.0022277832031250, 0.0026550292968750, 0.0019226074218750, 0.0022888183593750,
+0.0018005371093750, 0.0022888183593750, 0.0019226074218750, 0.0021972656250000,
+0.0018310546875000, 0.0019836425781250, 0.0017395019531250, 0.0020141601562500,
+0.0023498535156250, 0.0020751953125000, 0.0020141601562500, 0.0017700195312500,
+0.0021972656250000, 0.0019531250000000, 0.0028381347656250, 0.0022277832031250,
+0.0023193359375000, 0.0017700195312500, 0.0025634765625000, 0.0025939941406250,
+0.0026245117187500, 0.0024414062500000, 0.0024414062500000, 0.0023803710937500,
+0.0022277832031250, 0.0027465820312500, 0.0021362304687500, 0.0025024414062500,
+0.0022277832031250, 0.0026855468750000, 0.0017700195312500, 0.0026855468750000,
+0.0020446777343750, 0.0026855468750000, 0.0023193359375000, 0.0027160644531250,
+0.0018005371093750, 0.0027770996093750, 0.0028381347656250, 0.0028686523437500,
+0.0025024414062500, 0.0028076171875000, 0.0027160644531250, 0.0029602050781250,
+0.0036010742187500, 0.0031738281250000, 0.0030212402343750, 0.0030517578125000,
+0.0037536621093750, 0.0034484863281250, 0.0037841796875000, 0.0033874511718750,
+0.0036621093750000, 0.0034484863281250, 0.0039367675781250, 0.0036621093750000,
+0.0038452148437500, 0.0035095214843750, 0.0037231445312500, 0.0038146972656250,
+0.0036926269531250, 0.0038757324218750, 0.0035705566406250, 0.0038452148437500,
+0.0033264160156250, 0.0039367675781250, 0.0032653808593750, 0.0039367675781250,
+0.0031433105468750, 0.0038757324218750, 0.0029296875000000, 0.0038757324218750,
+0.0029296875000000, 0.0038146972656250, 0.0028991699218750, 0.0037841796875000,
+0.0027160644531250, 0.0037841796875000, 0.0028991699218750, 0.0038452148437500,
+0.0026550292968750, 0.0036621093750000, 0.0027465820312500, 0.0038146972656250,
+0.0029296875000000, 0.0039978027343750, 0.0026550292968750, 0.0037536621093750,
+0.0030212402343750, 0.0043334960937500, 0.0029296875000000, 0.0042114257812500,
+0.0029907226562500, 0.0043334960937500, 0.0032348632812500, 0.0046691894531250,
+0.0030212402343750, 0.0044250488281250, 0.0032348632812500, 0.0046081542968750,
+0.0031433105468750, 0.0046997070312500, 0.0031738281250000, 0.0044250488281250,
+0.0033264160156250, 0.0043640136718750, 0.0031433105468750, 0.0044555664062500,
+0.0034179687500000, 0.0040283203125000, 0.0032653808593750, 0.0040283203125000,
+0.0033569335937500, 0.0039672851562500, 0.0035400390625000, 0.0036926269531250,
+0.0032958984375000, 0.0037231445312500, 0.0036621093750000, 0.0038146972656250,
+0.0035705566406250, 0.0036315917968750, 0.0036621093750000, 0.0038757324218750,
+0.0039062500000000, 0.0041809082031250, 0.0037231445312500, 0.0039367675781250,
+0.0040893554687500, 0.0046081542968750, 0.0039367675781250, 0.0044555664062500,
+0.0039672851562500, 0.0047607421875000, 0.0043029785156250, 0.0054016113281250,
+0.0040893554687500, 0.0049438476562500, 0.0043640136718750, 0.0057678222656250,
+0.0043029785156250, 0.0058288574218750, 0.0042114257812500, 0.0058288574218750,
+0.0043029785156250, 0.0061950683593750, 0.0041809082031250, 0.0061035156250000,
+0.0040588378906250, 0.0064086914062500, 0.0040283203125000, 0.0064392089843750,
+0.0039367675781250, 0.0065002441406250, 0.0038146972656250, 0.0067138671875000,
+0.0037841796875000, 0.0066528320312500, 0.0037841796875000, 0.0065917968750000,
+0.0036621093750000, 0.0067443847656250, 0.0036926269531250, 0.0065002441406250,
+0.0037536621093750, 0.0063781738281250, 0.0035400390625000, 0.0067443847656250,
+0.0036315917968750, 0.0058288574218750, 0.0035705566406250, 0.0061645507812500,
+0.0034484863281250, 0.0058898925781250, 0.0034790039062500, 0.0050964355468750,
+0.0033874511718750, 0.0056457519531250, 0.0033264160156250, 0.0046081542968750,
+0.0032653808593750, 0.0047607421875000, 0.0032653808593750, 0.0046386718750000,
+0.0032653808593750, 0.0039978027343750, 0.0031738281250000, 0.0043945312500000,
+0.0034179687500000, 0.0042114257812500, 0.0032958984375000, 0.0042114257812500,
+0.0035705566406250, 0.0043945312500000, 0.0038757324218750, 0.0044555664062500,
+0.0036010742187500, 0.0046081542968750, 0.0041809082031250, 0.0045776367187500,
+0.0041809082031250, 0.0048828125000000, 0.0040588378906250, 0.0045776367187500,
+0.0042419433593750, 0.0042724609375000, 0.0041809082031250, 0.0045776367187500,
+0.0039672851562500, 0.0032653808593750, 0.0040588378906250, 0.0035095214843750,
+0.0040893554687500, 0.0028686523437500, 0.0039062500000000, 0.0015563964843750,
+0.0038452148437500, 0.0022277832031250, 0.0039672851562500, 0.0011596679687500,
+0.0037536621093750, 0.0008544921875000, 0.0038452148437500, 0.0012817382812500,
+0.0039672851562500, 0.0011291503906250, 0.0037536621093750, 0.0008544921875000,
+0.0041503906250000, 0.0018920898437500, 0.0040283203125000, 0.0017395019531250,
+0.0041198730468750, 0.0017089843750000, 0.0043640136718750, 0.0023193359375000,
+0.0041503906250000, 0.0021362304687500, 0.0043945312500000, 0.0019836425781250,
+0.0043334960937500, 0.0020446777343750, 0.0042724609375000, 0.0022888183593750,
+0.0042724609375000, 0.0022583007812500, 0.0041198730468750, 0.0021057128906250,
+0.0044555664062500, 0.0032653808593750, 0.0041809082031250, 0.0030822753906250,
+0.0044555664062500, 0.0031127929687500, 0.0048217773437500, 0.0041198730468750,
+0.0043640136718750, 0.0039062500000000, 0.0052185058593750, 0.0030212402343750,
+0.0050659179687500, 0.0036621093750000, 0.0049438476562500, 0.0031127929687500,
+0.0057983398437500, 0.0009460449218750, 0.0034484863281250, 0.0037231445312500,
+0.0019226074218750, 0.0041198730468750, 0.0022583007812500, 0.0045776367187500,
+0.0023803710937500, 0.0040893554687500, 0.0020751953125000, 0.0035400390625000,
+0.0021667480468750, 0.0039672851562500, 0.0025939941406250, 0.0032348632812500,
+0.0027160644531250, 0.0032958984375000, 0.0027465820312500, 0.0030822753906250,
+0.0030212402343750, 0.0024719238281250, 0.0032348632812500, 0.0028076171875000,
+0.0027465820312500, 0.0020751953125000, 0.0030822753906250, 0.0018005371093750,
+0.0030822753906250, 0.0023193359375000, 0.0027770996093750, 0.0021972656250000,
+0.0031127929687500, 0.0016479492187500, 0.0029296875000000, 0.0036010742187500,
+0.0029907226562500, 0.0030822753906250, 0.0029602050781250, 0.0034179687500000,
+0.0027770996093750, 0.0050354003906250, 0.0028381347656250, 0.0041503906250000,
+0.0025024414062500, 0.0047607421875000, 0.0024414062500000, 0.0052795410156250,
+0.0024414062500000, 0.0045471191406250, 0.0021667480468750, 0.0040588378906250,
+0.0021972656250000, 0.0046081542968750, 0.0026550292968750, 0.0030212402343750,
+0.0023803710937500, 0.0031738281250000, 0.0027465820312500, 0.0031127929687500,
+0.0033264160156250, 0.0021972656250000, 0.0028686523437500, 0.0025939941406250,
+0.0036315917968750, 0.0023498535156250, 0.0037231445312500, 0.0023193359375000,
+0.0035400390625000, 0.0024719238281250, 0.0037536621093750, 0.0025939941406250,
+0.0037841796875000, 0.0026245117187500, 0.0032348632812500, 0.0025329589843750,
+0.0033874511718750, 0.0026855468750000, 0.0031433105468750, 0.0027465820312500,
+0.0026855468750000, 0.0027770996093750, 0.0029296875000000, 0.0028381347656250,
+0.0023803710937500, 0.0029602050781250, 0.0024108886718750, 0.0028991699218750,
+0.0024108886718750, 0.0030517578125000, 0.0021667480468750, 0.0031738281250000,
+0.0023498535156250, 0.0030212402343750, 0.0021057128906250, 0.0032958984375000,
+0.0022277832031250, 0.0033569335937500, 0.0021057128906250, 0.0033264160156250,
+0.0018920898437500, 0.0035705566406250, 0.0020751953125000, 0.0035400390625000,
+0.0014953613281250, 0.0031433105468750, 0.0015869140625000, 0.0034179687500000,
+0.0014648437500000, 0.0031738281250000, 0.0010375976562500, 0.0026245117187500,
+0.0011596679687500, 0.0028991699218750, 0.0011596679687500, 0.0026245117187500,
+0.0007934570312500, 0.0023803710937500, 0.0012207031250000, 0.0027160644531250,
+0.0015869140625000, 0.0028686523437500, 0.0010375976562500, 0.0026550292968750,
+0.0024719238281250, 0.0034484863281250, 0.0021972656250000, 0.0032653808593750,
+0.0025634765625000, 0.0034790039062500, 0.0036315917968750, 0.0040283203125000,
+0.0030822753906250, 0.0036621093750000, 0.0038452148437500, 0.0039672851562500,
+0.0040588378906250, 0.0041198730468750, 0.0037536621093750, 0.0038452148437500,
+0.0039062500000000, 0.0038146972656250, 0.0040588378906250, 0.0039367675781250,
+0.0033874511718750, 0.0035705566406250, 0.0036621093750000, 0.0036315917968750,
+0.0032348632812500, 0.0036621093750000, 0.0025634765625000, 0.0034484863281250,
+0.0030517578125000, 0.0035095214843750, 0.0018920898437500, 0.0036010742187500,
+0.0020141601562500},
+{0.0039978027343750, 0.0025329589843750, 0.0043945312500000, 0.0022277832031250,
+0.0039367675781250, 0.0024719238281250, 0.0024108886718750, 0.0046691894531250,
+0.0021972656250000, 0.0010986328125000, 0.0028381347656250, 0.0024108886718750,
+0.0027465820312500, 0.0014343261718750, 0.0026550292968750, 0.0023193359375000,
+0.0030212402343750, 0.0036621093750000, 0.0032348632812500, 0.0029602050781250,
+0.0025634765625000, 0.0041198730468750, 0.0028991699218750, 0.0041198730468750,
+0.0028686523437500, 0.0041503906250000, 0.0023498535156250, 0.0048522949218750,
+0.0027465820312500, 0.0044555664062500, 0.0024719238281250, 0.0044555664062500,
+0.0025024414062500, 0.0048522949218750, 0.0025939941406250, 0.0041809082031250,
+0.0025024414062500, 0.0036315917968750, 0.0025634765625000, 0.0041503906250000,
+0.0025329589843750, 0.0029296875000000, 0.0024108886718750, 0.0028076171875000,
+0.0025634765625000, 0.0030517578125000, 0.0026855468750000, 0.0024414062500000,
+0.0024719238281250, 0.0024414062500000, 0.0031433105468750, 0.0035400390625000,
+0.0029296875000000, 0.0029296875000000, 0.0031433105468750, 0.0035400390625000,
+0.0038146972656250, 0.0049438476562500, 0.0034790039062500, 0.0041809082031250,
+0.0036621093750000, 0.0052490234375000, 0.0039062500000000, 0.0055541992187500,
+0.0034790039062500, 0.0050048828125000, 0.0031738281250000, 0.0049743652343750,
+0.0034484863281250, 0.0050659179687500, 0.0025024414062500, 0.0041198730468750,
+0.0024719238281250, 0.0041198730468750, 0.0025024414062500, 0.0040283203125000,
+0.0020446777343750, 0.0035095214843750, 0.0020141601562500, 0.0036315917968750,
+0.0023193359375000, 0.0035705566406250, 0.0021057128906250, 0.0035095214843750,
+0.0022583007812500, 0.0036010742187500, 0.0025939941406250, 0.0036010742187500,
+0.0022888183593750, 0.0035400390625000, 0.0027160644531250, 0.0036621093750000,
+0.0025939941406250, 0.0036010742187500, 0.0025939941406250, 0.0036315917968750,
+0.0028076171875000, 0.0037231445312500, 0.0026245117187500, 0.0037231445312500,
+0.0027160644531250, 0.0036010742187500, 0.0026855468750000, 0.0036621093750000,
+0.0027160644531250, 0.0037231445312500, 0.0027770996093750, 0.0036315917968750,
+0.0026855468750000, 0.0036315917968750, 0.0032653808593750, 0.0038452148437500,
+0.0029907226562500, 0.0038757324218750, 0.0033874511718750, 0.0039367675781250,
+0.0040588378906250, 0.0041198730468750, 0.0036010742187500, 0.0042419433593750,
+0.0048522949218750, 0.0039367675781250, 0.0047607421875000, 0.0041809082031250,
+0.0048522949218750, 0.0039672851562500, 0.0057067871093750, 0.0036315917968750,
+0.0052795410156250, 0.0039978027343750, 0.0053100585937500, 0.0034179687500000,
+0.0057373046875000, 0.0034484863281250, 0.0050964355468750, 0.0035705566406250,
+0.0045776367187500, 0.0033874511718750, 0.0050354003906250, 0.0034790039062500,
+0.0035400390625000, 0.0037231445312500, 0.0036621093750000, 0.0037841796875000,
+0.0033264160156250, 0.0040588378906250, 0.0021362304687500, 0.0044250488281250,
+0.0026550292968750, 0.0043640136718750, 0.0016479492187500, 0.0044860839843750,
+0.0016174316406250, 0.0048217773437500, 0.0013732910156250, 0.0045471191406250,
+0.0007629394531250, 0.0043640136718750, 0.0010070800781250, 0.0048217773437500,
+0.0005187988281250, 0.0039978027343750, 0.0005187988281250, 0.0041503906250000,
+0.0005187988281250, 0.0041809082031250, 0.0003051757812500, 0.0036010742187500,
+0.0003051757812500, 0.0039367675781250, 0.0007019042968750, 0.0037841796875000,
+0.0004882812500000, 0.0037231445312500, 0.0006713867187500, 0.0039062500000000,
+0.0010986328125000, 0.0039672851562500, 0.0008239746093750, 0.0038757324218750,
+0.0012817382812500, 0.0043029785156250, 0.0011901855468750, 0.0042114257812500,
+0.0013427734375000, 0.0042419433593750, 0.0016479492187500, 0.0046081542968750,
+0.0013732910156250, 0.0045166015625000, 0.0019226074218750, 0.0045471191406250,
+0.0017395019531250, 0.0046386718750000, 0.0019531250000000, 0.0044860839843750,
+0.0023803710937500, 0.0044250488281250, 0.0020446777343750, 0.0044860839843750,
+0.0029296875000000, 0.0041503906250000, 0.0027160644531250, 0.0041503906250000,
+0.0029602050781250, 0.0040893554687500, 0.0035400390625000, 0.0037536621093750,
+0.0031738281250000, 0.0037536621093750, 0.0039978027343750, 0.0035705566406250,
+0.0039367675781250, 0.0034790039062500, 0.0041198730468750, 0.0034179687500000,
+0.0046691894531250, 0.0034179687500000, 0.0044555664062500, 0.0033569335937500,
+0.0046997070312500, 0.0032653808593750, 0.0049133300781250, 0.0033874511718750,
+0.0047302246093750, 0.0033874511718750, 0.0047607421875000, 0.0033874511718750,
+0.0049133300781250, 0.0035400390625000, 0.0045166015625000, 0.0033874511718750,
+0.0046691894531250, 0.0034790039062500, 0.0044250488281250, 0.0035095214843750,
+0.0040283203125000, 0.0034179687500000, 0.0043334960937500, 0.0034790039062500,
+0.0035705566406250, 0.0034790039062500, 0.0037536621093750, 0.0033874511718750,
+0.0034179687500000, 0.0035095214843750, 0.0028076171875000, 0.0036010742187500,
+0.0032043457031250, 0.0034484863281250, 0.0023193359375000, 0.0035705566406250,
+0.0024719238281250, 0.0035400390625000, 0.0023803710937500, 0.0034790039062500,
+0.0018005371093750, 0.0034790039062500, 0.0021362304687500, 0.0033874511718750,
+0.0019836425781250, 0.0034179687500000, 0.0019226074218750, 0.0031738281250000,
+0.0021667480468750, 0.0033569335937500, 0.0024108886718750, 0.0035095214843750,
+0.0023803710937500, 0.0032958984375000, 0.0028076171875000, 0.0038146972656250,
+0.0029296875000000, 0.0037536621093750, 0.0028991699218750, 0.0037231445312500,
+0.0031127929687500, 0.0040588378906250, 0.0032043457031250, 0.0039062500000000,
+0.0029296875000000, 0.0039062500000000, 0.0031738281250000, 0.0040283203125000,
+0.0030517578125000, 0.0038757324218750, 0.0026855468750000, 0.0036926269531250,
+0.0029296875000000, 0.0038452148437500, 0.0025634765625000, 0.0036010742187500,
+0.0025939941406250, 0.0035095214843750, 0.0025024414062500, 0.0035095214843750,
+0.0022277832031250, 0.0034179687500000, 0.0023803710937500, 0.0033569335937500,
+0.0019836425781250, 0.0032348632812500, 0.0020141601562500, 0.0032348632812500,
+0.0019226074218750, 0.0032043457031250, 0.0015869140625000, 0.0030517578125000,
+0.0016479492187500, 0.0030212402343750, 0.0014953613281250, 0.0031433105468750,
+0.0013732910156250, 0.0029907226562500, 0.0014953613281250, 0.0031433105468750,
+0.0014648437500000, 0.0033569335937500, 0.0014343261718750, 0.0031738281250000,
+0.0018920898437500, 0.0035400390625000, 0.0017395019531250, 0.0035400390625000,
+0.0020446777343750, 0.0035400390625000, 0.0025329589843750, 0.0036926269531250,
+0.0021667480468750, 0.0035705566406250, 0.0031433105468750, 0.0035705566406250,
+0.0029907226562500, 0.0036315917968750, 0.0032958984375000, 0.0035705566406250,
+0.0040588378906250, 0.0036010742187500, 0.0036010742187500, 0.0036621093750000,
+0.0043334960937500, 0.0033874511718750, 0.0045166015625000, 0.0036010742187500,
+0.0042724609375000, 0.0035095214843750, 0.0044555664062500, 0.0033264160156250,
+0.0044250488281250, 0.0035705566406250, 0.0034484863281250, 0.0033569335937500,
+0.0037231445312500, 0.0033569335937500, 0.0030212402343750, 0.0034790039062500,
+0.0017700195312500, 0.0034484863281250, 0.0023193359375000, 0.0034484863281250,
+0.0012817382812500, 0.0033569335937500, 0.0008544921875000, 0.0034790039062500,
+0.0011901855468750, 0.0035095214843750, 0.0011291503906250, 0.0034790039062500,
+0.0011901855468750, 0.0035400390625000, 0.0022888183593750, 0.0038146972656250,
+0.0019531250000000, 0.0036315917968750, 0.0021972656250000, 0.0037231445312500,
+0.0031127929687500, 0.0040893554687500, 0.0026855468750000, 0.0039062500000000,
+0.0035400390625000, 0.0039367675781250, 0.0035705566406250, 0.0041809082031250,
+0.0036926269531250, 0.0038146972656250, 0.0041809082031250, 0.0035095214843750,
+0.0038146972656250, 0.0038452148437500, 0.0039367675781250, 0.0027770996093750,
+0.0039978027343750, 0.0029296875000000, 0.0036926269531250, 0.0027465820312500,
+0.0034484863281250, 0.0018615722656250, 0.0036010742187500, 0.0021972656250000,
+0.0029907226562500, 0.0017700195312500, 0.0031433105468750, 0.0015563964843750,
+0.0031127929687500, 0.0017089843750000, 0.0027465820312500, 0.0017089843750000,
+0.0030822753906250, 0.0015869140625000, 0.0028686523437500, 0.0018920898437500,
+0.0030822753906250, 0.0018920898437500, 0.0030822753906250, 0.0018310546875000,
+0.0030212402343750, 0.0020141601562500, 0.0034179687500000, 0.0019531250000000,
+0.0029602050781250, 0.0019531250000000, 0.0031127929687500, 0.0020141601562500,
+0.0032348632812500, 0.0018615722656250, 0.0029907226562500, 0.0016784667968750,
+0.0030822753906250, 0.0018310546875000, 0.0032958984375000, 0.0015563964843750,
+0.0031127929687500, 0.0014038085937500, 0.0033264160156250, 0.0014953613281250,
+0.0036926269531250, 0.0014953613281250, 0.0034790039062500, 0.0014648437500000,
+0.0038146972656250, 0.0017089843750000, 0.0038757324218750, 0.0017089843750000,
+0.0038757324218750, 0.0017700195312500, 0.0040588378906250, 0.0019531250000000,
+0.0041503906250000, 0.0019226074218750, 0.0039978027343750, 0.0022583007812500,
+0.0040588378906250, 0.0021972656250000, 0.0040893554687500, 0.0024719238281250,
+0.0039672851562500, 0.0028381347656250, 0.0038757324218750, 0.0026550292968750,
+0.0042114257812500, 0.0033569335937500, 0.0039978027343750, 0.0033569335937500,
+0.0042114257812500, 0.0034790039062500, 0.0046997070312500, 0.0039978027343750,
+0.0043640136718750, 0.0039367675781250, 0.0048217773437500, 0.0039978027343750,
+0.0049438476562500, 0.0043029785156250, 0.0047607421875000, 0.0041809082031250,
+0.0049133300781250, 0.0039672851562500, 0.0051269531250000, 0.0043029785156250,
+0.0046997070312500, 0.0037841796875000, 0.0049438476562500, 0.0038757324218750,
+0.0048828125000000, 0.0039062500000000, 0.0045776367187500, 0.0035400390625000,
+0.0049438476562500, 0.0036315917968750, 0.0046691894531250, 0.0035095214843750,
+0.0049438476562500, 0.0033874511718750, 0.0049438476562500, 0.0034484863281250,
+0.0048522949218750, 0.0033569335937500, 0.0051879882812500, 0.0032348632812500,
+0.0047912597656250, 0.0031433105468750, 0.0051269531250000, 0.0031433105468750,
+0.0048522949218750, 0.0029907226562500, 0.0043640136718750, 0.0028686523437500,
+0.0047912597656250, 0.0028686523437500, 0.0040893554687500, 0.0027160644531250,
+0.0040588378906250, 0.0026855468750000, 0.0041198730468750, 0.0026550292968750,
+0.0037536621093750, 0.0025329589843750, 0.0037231445312500, 0.0025634765625000,
+0.0040893554687500, 0.0025329589843750, 0.0038146972656250, 0.0025024414062500,
+0.0039978027343750, 0.0025939941406250, 0.0044555664062500, 0.0026550292968750,
+0.0041198730468750, 0.0025939941406250, 0.0046997070312500, 0.0027770996093750,
+0.0047302246093750, 0.0027770996093750, 0.0046386718750000, 0.0028686523437500,
+0.0048828125000000, 0.0030212402343750, 0.0048522949218750, 0.0028686523437500,
+0.0046997070312500, 0.0031738281250000, 0.0048828125000000, 0.0030822753906250,
+0.0048217773437500, 0.0032653808593750, 0.0046691894531250, 0.0035400390625000,
+0.0046997070312500, 0.0032348632812500, 0.0046997070312500, 0.0039672851562500,
+0.0045776367187500, 0.0037231445312500, 0.0046691894531250, 0.0039978027343750,
+0.0046997070312500},
+{0.0025024414062500, 0.0018310546875000, 0.0035095214843750, 0.0017395019531250,
+0.0027770996093750, 0.0016784667968750, 0.0035705566406250, 0.0018005371093750,
+0.0037536621093750, 0.0017700195312500, 0.0033874511718750, 0.0017395019531250,
+0.0034179687500000, 0.0018005371093750, 0.0035095214843750, 0.0016174316406250,
+0.0028381347656250, 0.0017700195312500, 0.0029296875000000, 0.0016784667968750,
+0.0028381347656250, 0.0016174316406250, 0.0024414062500000, 0.0017395019531250,
+0.0027160644531250, 0.0016784667968750, 0.0026550292968750, 0.0017395019531250,
+0.0027160644531250, 0.0017395019531250, 0.0029602050781250, 0.0019226074218750,
+0.0031127929687500, 0.0021057128906250, 0.0031127929687500, 0.0019226074218750,
+0.0031738281250000, 0.0024719238281250, 0.0033569335937500, 0.0025024414062500,
+0.0033264160156250, 0.0025634765625000, 0.0032653808593750, 0.0028991699218750,
+0.0034179687500000, 0.0028076171875000, 0.0033874511718750, 0.0031738281250000,
+0.0033264160156250, 0.0032653808593750, 0.0034484863281250, 0.0033264160156250,
+0.0035095214843750, 0.0036010742187500, 0.0033569335937500, 0.0036010742187500,
+0.0039062500000000, 0.0037536621093750, 0.0036315917968750, 0.0039062500000000,
+0.0038146972656250, 0.0039978027343750, 0.0042724609375000, 0.0040283203125000,
+0.0038452148437500, 0.0040588378906250, 0.0041503906250000, 0.0040283203125000,
+0.0041503906250000, 0.0040588378906250, 0.0039672851562500, 0.0039672851562500,
+0.0039672851562500, 0.0038452148437500, 0.0039062500000000, 0.0039062500000000,
+0.0038146972656250, 0.0036621093750000, 0.0036621093750000, 0.0035705566406250,
+0.0039062500000000, 0.0035095214843750, 0.0040283203125000, 0.0033264160156250,
+0.0038452148437500, 0.0033569335937500, 0.0043640136718750, 0.0032958984375000,
+0.0044555664062500, 0.0032348632812500, 0.0043334960937500, 0.0032653808593750,
+0.0046081542968750, 0.0032958984375000, 0.0047912597656250, 0.0032043457031250,
+0.0042114257812500, 0.0032958984375000, 0.0045776367187500, 0.0032653808593750,
+0.0044250488281250, 0.0032348632812500, 0.0039367675781250, 0.0032043457031250,
+0.0045471191406250, 0.0031433105468750, 0.0038757324218750, 0.0031738281250000,
+0.0041198730468750, 0.0031127929687500, 0.0042419433593750, 0.0032043457031250,
+0.0039367675781250, 0.0032958984375000, 0.0044860839843750, 0.0031738281250000,
+0.0040893554687500, 0.0035095214843750, 0.0044555664062500, 0.0033569335937500,
+0.0044555664062500, 0.0034790039062500, 0.0042114257812500, 0.0037841796875000,
+0.0046691894531250, 0.0035095214843750, 0.0042114257812500, 0.0039062500000000,
+0.0043334960937500, 0.0038146972656250, 0.0044555664062500, 0.0037536621093750,
+0.0041809082031250, 0.0039978027343750, 0.0042419433593750, 0.0038757324218750,
+0.0046386718750000, 0.0039367675781250, 0.0044555664062500, 0.0038757324218750,
+0.0046997070312500, 0.0038757324218750, 0.0051269531250000, 0.0039062500000000,
+0.0048522949218750, 0.0039367675781250, 0.0054016113281250, 0.0040893554687500,
+0.0053710937500000, 0.0040893554687500, 0.0053100585937500, 0.0041809082031250,
+0.0055236816406250, 0.0043334960937500, 0.0054321289062500, 0.0043334960937500,
+0.0052185058593750, 0.0042114257812500, 0.0052185058593750, 0.0043945312500000,
+0.0050354003906250, 0.0041503906250000, 0.0047607421875000, 0.0038757324218750,
+0.0047912597656250, 0.0040283203125000, 0.0046081542968750, 0.0030517578125000,
+0.0044250488281250, 0.0032043457031250, 0.0044250488281250, 0.0027160644531250,
+0.0043640136718750, 0.0017395019531250, 0.0041503906250000, 0.0022277832031250,
+0.0042419433593750, 0.0010681152343750, 0.0041198730468750, 0.0010070800781250,
+0.0040283203125000, 0.0009765625000000, 0.0039978027343750, 0.0003051757812500,
+0.0038452148437500, 0.0004577636718750, 0.0036621093750000, 0.0005187988281250,
+0.0036315917968750, 0.0003356933593750, 0.0035400390625000, 0.0005187988281250,
+0.0033874511718750, 0.0008239746093750, 0.0034179687500000, 0.0005798339843750,
+0.0032348632812500, 0.0010986328125000, 0.0032958984375000, 0.0009460449218750,
+0.0032653808593750, 0.0010986328125000, 0.0031738281250000, 0.0014953613281250,
+0.0032653808593750, 0.0012207031250000, 0.0032043457031250, 0.0016479492187500,
+0.0032043457031250, 0.0017395019531250, 0.0032958984375000, 0.0016479492187500,
+0.0032653808593750, 0.0017700195312500, 0.0032958984375000, 0.0018310546875000,
+0.0035400390625000, 0.0018615722656250, 0.0034179687500000, 0.0018920898437500,
+0.0036926269531250, 0.0021057128906250, 0.0039672851562500, 0.0021972656250000,
+0.0036010742187500, 0.0021362304687500, 0.0043029785156250, 0.0026245117187500,
+0.0041809082031250, 0.0026550292968750, 0.0043334960937500, 0.0028381347656250,
+0.0048522949218750, 0.0031738281250000, 0.0045471191406250, 0.0030517578125000,
+0.0049133300781250, 0.0034790039062500, 0.0049438476562500, 0.0034484863281250,
+0.0048522949218750, 0.0036010742187500, 0.0049133300781250, 0.0038757324218750,
+0.0048828125000000, 0.0037536621093750, 0.0046386718750000, 0.0042724609375000,
+0.0046386718750000, 0.0042419433593750, 0.0045776367187500, 0.0042419433593750,
+0.0043945312500000, 0.0046081542968750, 0.0044860839843750, 0.0045166015625000,
+0.0044250488281250, 0.0044555664062500, 0.0043945312500000, 0.0046081542968750,
+0.0043640136718750, 0.0044860839843750, 0.0043945312500000, 0.0042419433593750,
+0.0042724609375000, 0.0043334960937500, 0.0042419433593750, 0.0043945312500000,
+0.0041503906250000, 0.0042114257812500, 0.0042114257812500, 0.0043029785156250,
+0.0042419433593750, 0.0045166015625000, 0.0040893554687500, 0.0043640136718750,
+0.0042724609375000, 0.0043334960937500, 0.0041809082031250, 0.0044860839843750,
+0.0041198730468750, 0.0042114257812500, 0.0041809082031250, 0.0040283203125000,
+0.0040588378906250, 0.0042419433593750, 0.0042419433593750, 0.0036010742187500,
+0.0041809082031250, 0.0036315917968750, 0.0042419433593750, 0.0036010742187500,
+0.0043945312500000, 0.0033264160156250, 0.0042419433593750, 0.0034484863281250,
+0.0043640136718750, 0.0033569335937500, 0.0043029785156250, 0.0032958984375000,
+0.0041198730468750, 0.0033264160156250, 0.0039978027343750, 0.0033264160156250,
+0.0039672851562500, 0.0033264160156250, 0.0035095214843750, 0.0031127929687500,
+0.0034790039062500, 0.0030517578125000, 0.0033264160156250, 0.0030822753906250,
+0.0029907226562500, 0.0029296875000000, 0.0029602050781250, 0.0028381347656250,
+0.0030212402343750, 0.0033264160156250, 0.0027160644531250, 0.0030212402343750,
+0.0029907226562500, 0.0035400390625000, 0.0033569335937500, 0.0043334960937500,
+0.0030517578125000, 0.0037841796875000, 0.0035400390625000, 0.0048828125000000,
+0.0037231445312500, 0.0050659179687500, 0.0035095214843750, 0.0046997070312500,
+0.0034484863281250, 0.0048828125000000, 0.0034179687500000, 0.0030517578125000,
+0.0031433105468750, 0.0047607421875000, 0.0032653808593750, 0.0043945312500000,
+0.0033874511718750, 0.0045166015625000, 0.0034790039062500, 0.0052490234375000,
+0.0034790039062500, 0.0051574707031250, 0.0032958984375000, 0.0046081542968750,
+0.0033874511718750, 0.0048522949218750, 0.0032653808593750, 0.0043945312500000,
+0.0028686523437500, 0.0037231445312500, 0.0029602050781250, 0.0042419433593750,
+0.0030517578125000, 0.0029602050781250, 0.0027160644531250, 0.0030822753906250,
+0.0028076171875000, 0.0029296875000000, 0.0030822753906250, 0.0021057128906250,
+0.0026855468750000, 0.0024108886718750, 0.0029602050781250, 0.0022277832031250,
+0.0027770996093750, 0.0018615722656250, 0.0027465820312500, 0.0024414062500000,
+0.0028076171875000, 0.0028686523437500, 0.0025634765625000, 0.0024108886718750,
+0.0030212402343750, 0.0036926269531250, 0.0027160644531250, 0.0037231445312500,
+0.0029602050781250, 0.0036926269531250, 0.0035095214843750, 0.0044860839843750,
+0.0030212402343750, 0.0042724609375000, 0.0037231445312500, 0.0038452148437500,
+0.0036315917968750, 0.0042419433593750, 0.0035400390625000, 0.0039978027343750,
+0.0038452148437500, 0.0032958984375000, 0.0035095214843750, 0.0036926269531250,
+0.0032958984375000, 0.0036621093750000, 0.0033264160156250, 0.0034790039062500,
+0.0030822753906250, 0.0039062500000000, 0.0027160644531250, 0.0042114257812500,
+0.0027160644531250, 0.0039367675781250, 0.0025634765625000, 0.0045471191406250,
+0.0023498535156250, 0.0046081542968750, 0.0025329589843750, 0.0045776367187500,
+0.0025634765625000, 0.0048522949218750, 0.0023803710937500, 0.0049133300781250,
+0.0031738281250000, 0.0047912597656250, 0.0028381347656250, 0.0049438476562500,
+0.0032043457031250, 0.0046997070312500, 0.0039062500000000, 0.0043029785156250,
+0.0033874511718750, 0.0045166015625000, 0.0042724609375000, 0.0040893554687500,
+0.0042419433593750, 0.0040893554687500, 0.0040283203125000, 0.0041809082031250,
+0.0043945312500000, 0.0040893554687500, 0.0042724609375000, 0.0040588378906250,
+0.0036010742187500, 0.0040283203125000, 0.0039062500000000, 0.0043029785156250,
+0.0033264160156250, 0.0041198730468750, 0.0024414062500000, 0.0040283203125000,
+0.0030212402343750, 0.0041503906250000, 0.0015563964843750, 0.0036926269531250,
+0.0016174316406250, 0.0037841796875000, 0.0015258789062500, 0.0035400390625000,
+0.0006103515625000, 0.0030822753906250, 0.0009460449218750, 0.0033569335937500,
+0.0007629394531250, 0.0030212402343750, 0.0004272460937500, 0.0029602050781250,
+0.0007019042968750, 0.0029907226562500, 0.0008850097656250, 0.0029296875000000,
+0.0007019042968750, 0.0029602050781250, 0.0012207031250000, 0.0028381347656250,
+0.0011596679687500, 0.0028686523437500, 0.0013732910156250, 0.0027160644531250,
+0.0016784667968750, 0.0025024414062500, 0.0016174316406250, 0.0025939941406250,
+0.0025329589843750, 0.0024414062500000, 0.0022583007812500, 0.0022583007812500,
+0.0028686523437500, 0.0024108886718750, 0.0037841796875000, 0.0023803710937500,
+0.0032653808593750, 0.0021667480468750, 0.0045166015625000, 0.0027465820312500,
+0.0044860839843750, 0.0025329589843750, 0.0046081542968750, 0.0028076171875000,
+0.0053405761718750, 0.0033874511718750, 0.0049743652343750, 0.0030212402343750,
+0.0052795410156250, 0.0037536621093750, 0.0053405761718750, 0.0037536621093750,
+0.0051879882812500, 0.0037536621093750, 0.0052490234375000, 0.0041809082031250,
+0.0052185058593750, 0.0040283203125000, 0.0048522949218750, 0.0040283203125000,
+0.0050048828125000, 0.0041809082031250, 0.0046386718750000, 0.0039367675781250,
+0.0041503906250000, 0.0037841796875000, 0.0044250488281250, 0.0038452148437500,
+0.0037841796875000, 0.0033874511718750, 0.0037231445312500, 0.0033569335937500,
+0.0036010742187500, 0.0033874511718750, 0.0032043457031250, 0.0032348632812500,
+0.0034179687500000, 0.0032653808593750, 0.0030822753906250, 0.0034179687500000,
+0.0030822753906250, 0.0033874511718750, 0.0031127929687500, 0.0034790039062500,
+0.0029296875000000, 0.0036621093750000, 0.0029907226562500, 0.0035400390625000,
+0.0028076171875000, 0.0037231445312500, 0.0027465820312500, 0.0037231445312500,
+0.0026550292968750, 0.0037231445312500, 0.0024414062500000, 0.0037841796875000,
+0.0024719238281250, 0.0037231445312500, 0.0022583007812500, 0.0038146972656250,
+0.0021972656250000, 0.0037231445312500, 0.0021667480468750, 0.0036926269531250,
+0.0020141601562500, 0.0037231445312500, 0.0019531250000000, 0.0037231445312500,
+0.0020141601562500},
+{0.0008850097656250, 0.0030822753906250, 0.0011901855468750, 0.0027770996093750,
+0.0008850097656250, 0.0029602050781250, 0.0012207031250000, 0.0028381347656250,
+0.0017089843750000, 0.0023803710937500, 0.0013122558593750, 0.0025329589843750,
+0.0021972656250000, 0.0025329589843750, 0.0020751953125000, 0.0023803710937500,
+0.0021972656250000, 0.0027770996093750, 0.0028076171875000, 0.0031433105468750,
+0.0024414062500000, 0.0028991699218750, 0.0026855468750000, 0.0036315917968750,
+0.0028381347656250, 0.0036010742187500, 0.0023803710937500, 0.0036010742187500,
+0.0021667480468750, 0.0040283203125000, 0.0024108886718750, 0.0038757324218750,
+0.0010375976562500, 0.0037231445312500, 0.0013427734375000, 0.0039978027343750,
+0.0007629394531250, 0.0036010742187500, -0.0003967285156250, 0.0032653808593750,
+0.0001831054687500, 0.0036621093750000, -0.0012817382812500, 0.0027770996093750,
+-0.0013427734375000, 0.0028686523437500, -0.0013427734375000, 0.0028686523437500,
+-0.0021057128906250, 0.0023498535156250, -0.0019531250000000, 0.0025634765625000,
+-0.0019226074218750, 0.0025329589843750, -0.0022277832031250, 0.0025329589843750,
+-0.0018310546875000, 0.0025329589843750, -0.0014648437500000, 0.0026550292968750,
+-0.0018920898437500, 0.0027465820312500, -0.0010070800781250, 0.0025939941406250,
+-0.0011901855468750, 0.0026245117187500, -0.0008850097656250, 0.0026855468750000,
+-0.0002136230468750, 0.0025939941406250, -0.0007629394531250, 0.0025939941406250,
+0.0004882812500000, 0.0027770996093750, 0.0001220703125000, 0.0025634765625000,
+0.0004882812500000, 0.0027770996093750, 0.0015563964843750, 0.0030212402343750,
+0.0008850097656250, 0.0026855468750000, 0.0019836425781250, 0.0032348632812500,
+0.0021057128906250, 0.0031433105468750, 0.0019226074218750, 0.0031127929687500,
+0.0024719238281250, 0.0033874511718750, 0.0024719238281250, 0.0031127929687500,
+0.0021362304687500, 0.0032958984375000, 0.0024108886718750, 0.0031738281250000,
+0.0023193359375000, 0.0031127929687500, 0.0020141601562500, 0.0031738281250000,
+0.0023498535156250, 0.0029296875000000, 0.0020141601562500, 0.0031433105468750,
+0.0021667480468750, 0.0030517578125000, 0.0022888183593750, 0.0030517578125000,
+0.0020751953125000, 0.0032043457031250, 0.0022583007812500, 0.0030822753906250,
+0.0027465820312500, 0.0033264160156250, 0.0025634765625000, 0.0033264160156250,
+0.0030822753906250, 0.0032958984375000, 0.0036926269531250, 0.0034790039062500,
+0.0032958984375000, 0.0034790039062500, 0.0045166015625000, 0.0033569335937500,
+0.0044555664062500, 0.0035095214843750, 0.0046386718750000, 0.0033874511718750,
+0.0054931640625000, 0.0032653808593750, 0.0051269531250000, 0.0033569335937500,
+0.0055541992187500, 0.0031127929687500, 0.0056762695312500, 0.0031127929687500,
+0.0054016113281250, 0.0031738281250000, 0.0053710937500000, 0.0029907226562500,
+0.0054016113281250, 0.0028991699218750, 0.0046997070312500, 0.0031127929687500,
+0.0049133300781250, 0.0029602050781250, 0.0044555664062500, 0.0029907226562500,
+0.0037536621093750, 0.0031738281250000, 0.0041809082031250, 0.0030212402343750,
+0.0028686523437500, 0.0031433105468750, 0.0030517578125000, 0.0031127929687500,
+0.0027770996093750, 0.0031127929687500, 0.0019226074218750, 0.0031738281250000,
+0.0023803710937500, 0.0030517578125000, 0.0015869140625000, 0.0032043457031250,
+0.0016174316406250, 0.0031127929687500, 0.0015869140625000, 0.0032348632812500,
+0.0012207031250000, 0.0034179687500000, 0.0014648437500000, 0.0032043457031250,
+0.0012512207031250, 0.0037231445312500, 0.0012207031250000, 0.0036926269531250,
+0.0013732910156250, 0.0038452148437500, 0.0014038085937500, 0.0041503906250000,
+0.0014038085937500, 0.0040283203125000, 0.0016174316406250, 0.0042419433593750,
+0.0015563964843750, 0.0042724609375000, 0.0015563964843750, 0.0042419433593750,
+0.0016479492187500, 0.0043334960937500, 0.0015563964843750, 0.0043640136718750,
+0.0015563964843750, 0.0042419433593750, 0.0014648437500000, 0.0043029785156250,
+0.0014648437500000, 0.0042724609375000, 0.0014648437500000, 0.0041809082031250,
+0.0013122558593750, 0.0042419433593750, 0.0015869140625000, 0.0040588378906250,
+0.0014038085937500, 0.0040893554687500, 0.0015563964843750, 0.0039062500000000,
+0.0018310546875000, 0.0036010742187500, 0.0015563964843750, 0.0038146972656250,
+0.0019226074218750, 0.0035400390625000, 0.0019836425781250, 0.0034179687500000,
+0.0018310546875000, 0.0035400390625000, 0.0019226074218750, 0.0035705566406250,
+0.0018615722656250, 0.0033874511718750, 0.0017089843750000, 0.0036010742187500,
+0.0016479492187500, 0.0035400390625000, 0.0017089843750000, 0.0034484863281250,
+0.0017395019531250, 0.0035400390625000, 0.0016479492187500, 0.0035095214843750,
+0.0019531250000000, 0.0033264160156250, 0.0020141601562500, 0.0033569335937500,
+0.0020751953125000, 0.0032653808593750, 0.0022583007812500, 0.0030212402343750,
+0.0023193359375000, 0.0031433105468750, 0.0021972656250000, 0.0029907226562500,
+0.0024414062500000, 0.0029907226562500, 0.0022888183593750, 0.0030822753906250,
+0.0020141601562500, 0.0030822753906250, 0.0022583007812500, 0.0031127929687500,
+0.0016479492187500, 0.0031127929687500, 0.0017395019531250, 0.0031127929687500,
+0.0014953613281250, 0.0030517578125000, 0.0010375976562500, 0.0030212402343750,
+0.0012817382812500, 0.0031127929687500, 0.0006103515625000, 0.0030212402343750,
+0.0006408691406250, 0.0030822753906250, 0.0007019042968750, 0.0031433105468750,
+0.0003356933593750, 0.0031738281250000, 0.0004577636718750, 0.0031433105468750,
+0.0006713867187500, 0.0032043457031250, 0.0004272460937500, 0.0032653808593750,
+0.0007629394531250, 0.0032958984375000, 0.0011291503906250, 0.0033264160156250,
+0.0008239746093750, 0.0032653808593750, 0.0016784667968750, 0.0030212402343750,
+0.0014953613281250, 0.0032348632812500, 0.0016174316406250, 0.0028991699218750,
+0.0021362304687500, 0.0024414062500000, 0.0017089843750000, 0.0027160644531250,
+0.0020141601562500, 0.0019531250000000, 0.0020446777343750, 0.0019226074218750,
+0.0017700195312500, 0.0019836425781250, 0.0016784667968750, 0.0016174316406250,
+0.0017700195312500, 0.0016479492187500, 0.0014343261718750, 0.0019226074218750,
+0.0013732910156250, 0.0017700195312500, 0.0018310546875000, 0.0018005371093750,
+0.0019226074218750, 0.0019836425781250, 0.0015563964843750, 0.0017395019531250,
+0.0028381347656250, 0.0017700195312500, 0.0027160644531250, 0.0017395019531250,
+0.0026855468750000, 0.0017395019531250, 0.0034484863281250, 0.0016784667968750,
+0.0032348632812500, 0.0016174316406250, 0.0032348632812500, 0.0018310546875000,
+0.0036010742187500, 0.0016784667968750, 0.0032958984375000, 0.0017089843750000,
+0.0030212402343750, 0.0018310546875000, 0.0033874511718750, 0.0016784667968750,
+0.0025634765625000, 0.0019836425781250, 0.0026245117187500, 0.0019531250000000,
+0.0025024414062500, 0.0019836425781250, 0.0018310546875000, 0.0021972656250000,
+0.0021362304687500, 0.0020141601562500, 0.0021057128906250, 0.0021362304687500,
+0.0017700195312500, 0.0020141601562500, 0.0021362304687500, 0.0019531250000000,
+0.0025024414062500, 0.0019531250000000, 0.0021362304687500, 0.0017089843750000,
+0.0028686523437500, 0.0018920898437500, 0.0028991699218750, 0.0017089843750000,
+0.0026855468750000, 0.0017395019531250, 0.0029602050781250, 0.0018920898437500,
+0.0028991699218750, 0.0015869140625000, 0.0025329589843750, 0.0018920898437500,
+0.0025634765625000, 0.0018615722656250, 0.0025939941406250, 0.0017700195312500,
+0.0023803710937500, 0.0018920898437500, 0.0023498535156250, 0.0017700195312500,
+0.0025634765625000, 0.0016479492187500, 0.0023803710937500, 0.0015869140625000,
+0.0024719238281250, 0.0014648437500000, 0.0027770996093750, 0.0012207031250000,
+0.0025329589843750, 0.0011901855468750, 0.0027160644531250, 0.0010070800781250,
+0.0028686523437500, 0.0009460449218750, 0.0026855468750000, 0.0007629394531250,
+0.0025939941406250, 0.0006103515625000, 0.0028381347656250, 0.0007019042968750,
+0.0024108886718750, 0.0002746582031250, 0.0025634765625000, 0.0003662109375000,
+0.0026245117187500, 0.0002136230468750, 0.0024414062500000, -0.0001831054687500,
+0.0025634765625000, -0.0000305175781250, 0.0028076171875000, -0.0002441406250000,
+0.0026855468750000, -0.0003662109375000, 0.0028991699218750, -0.0003662109375000,
+0.0032043457031250, -0.0004272460937500, 0.0029602050781250, -0.0004882812500000,
+0.0031433105468750, -0.0002441406250000, 0.0032348632812500, -0.0003967285156250,
+0.0031127929687500, -0.0001831054687500, 0.0029907226562500, 0.0001831054687500,
+0.0029907226562500, -0.0000915527343750, 0.0030212402343750, 0.0003967285156250,
+0.0027160644531250, 0.0004882812500000, 0.0029296875000000, 0.0005187988281250,
+0.0030822753906250, 0.0007934570312500, 0.0027160644531250, 0.0007934570312500,
+0.0034790039062500, 0.0007629394531250, 0.0031738281250000, 0.0008544921875000,
+0.0032958984375000, 0.0008544921875000, 0.0037841796875000, 0.0007324218750000,
+0.0032653808593750, 0.0007934570312500, 0.0038146972656250, 0.0009155273437500,
+0.0035095214843750, 0.0007934570312500, 0.0034790039062500, 0.0009765625000000,
+0.0037536621093750, 0.0011291503906250, 0.0031738281250000, 0.0010681152343750,
+0.0037231445312500, 0.0014953613281250, 0.0033569335937500, 0.0014953613281250,
+0.0033569335937500, 0.0014953613281250, 0.0037231445312500, 0.0018005371093750,
+0.0032043457031250, 0.0016784667968750, 0.0035705566406250, 0.0016784667968750,
+0.0034179687500000, 0.0016784667968750, 0.0031738281250000, 0.0015563964843750,
+0.0032348632812500, 0.0014648437500000, 0.0030822753906250, 0.0014648437500000,
+0.0028381347656250, 0.0014648437500000, 0.0027160644531250, 0.0011596679687500,
+0.0026855468750000, 0.0012512207031250, 0.0025329589843750, 0.0014343261718750,
+0.0024719238281250, 0.0011596679687500, 0.0026550292968750, 0.0015869140625000,
+0.0024414062500000, 0.0014038085937500, 0.0025939941406250, 0.0014953613281250,
+0.0028991699218750, 0.0018005371093750, 0.0026245117187500, 0.0015258789062500,
+0.0030517578125000, 0.0020446777343750, 0.0030212402343750, 0.0019226074218750,
+0.0029907226562500, 0.0021057128906250, 0.0032043457031250, 0.0025024414062500,
+0.0031127929687500, 0.0021972656250000, 0.0030517578125000, 0.0028686523437500,
+0.0031127929687500, 0.0027465820312500, 0.0029907226562500, 0.0028381347656250,
+0.0028991699218750, 0.0032653808593750, 0.0030517578125000, 0.0029907226562500,
+0.0028076171875000, 0.0032653808593750, 0.0028686523437500, 0.0033264160156250,
+0.0029602050781250, 0.0031433105468750, 0.0028076171875000, 0.0031433105468750,
+0.0029907226562500, 0.0031127929687500, 0.0030822753906250, 0.0027160644531250,
+0.0029907226562500, 0.0026855468750000, 0.0031738281250000, 0.0025634765625000,
+0.0033264160156250, 0.0022888183593750, 0.0032348632812500, 0.0023498535156250,
+0.0035400390625000, 0.0020446777343750, 0.0036315917968750, 0.0019531250000000,
+0.0036315917968750, 0.0020141601562500, 0.0037841796875000, 0.0018310546875000,
+0.0038452148437500, 0.0018005371093750, 0.0036621093750000, 0.0019836425781250,
+0.0039367675781250, 0.0019531250000000, 0.0038452148437500, 0.0018920898437500,
+0.0036315917968750, 0.0020141601562500, 0.0039672851562500, 0.0020751953125000,
+0.0035400390625000, 0.0016784667968750, 0.0036621093750000, 0.0018310546875000,
+0.0036926269531250},
+{0.0028686523437500, 0.0007324218750000, 0.0028991699218750, 0.0008544921875000,
+0.0032348632812500, 0.0008850097656250, 0.0029602050781250, 0.0008544921875000,
+0.0031433105468750, 0.0011901855468750, 0.0030212402343750, 0.0011596679687500,
+0.0030212402343750, 0.0013427734375000, 0.0030517578125000, 0.0016174316406250,
+0.0027770996093750, 0.0014343261718750, 0.0031738281250000, 0.0018615722656250,
+0.0028381347656250, 0.0018920898437500, 0.0029296875000000, 0.0018615722656250,
+0.0032958984375000, 0.0021057128906250, 0.0027770996093750, 0.0020141601562500,
+0.0032653808593750, 0.0018005371093750, 0.0030517578125000, 0.0018920898437500,
+0.0029602050781250, 0.0018005371093750, 0.0032043457031250, 0.0015258789062500,
+0.0028686523437500, 0.0017395019531250, 0.0032348632812500, 0.0014343261718750,
+0.0029602050781250, 0.0015258789062500, 0.0031433105468750, 0.0014953613281250,
+0.0034484863281250, 0.0013732910156250, 0.0030212402343750, 0.0014953613281250,
+0.0037536621093750, 0.0010681152343750, 0.0035705566406250, 0.0012817382812500,
+0.0036315917968750, 0.0009155273437500, 0.0040893554687500, 0.0003356933593750,
+0.0037536621093750, 0.0007324218750000, 0.0039978027343750, -0.0004272460937500,
+0.0039672851562500, -0.0003356933593750, 0.0038757324218750, -0.0006103515625000,
+0.0038452148437500, -0.0015563964843750, 0.0036926269531250, -0.0011596679687500,
+0.0037231445312500, -0.0018005371093750, 0.0035705566406250, -0.0020751953125000,
+0.0035400390625000, -0.0017395019531250, 0.0035705566406250, -0.0018005371093750,
+0.0034179687500000, -0.0019531250000000, 0.0035705566406250, -0.0009155273437500,
+0.0034790039062500, -0.0012512207031250, 0.0034179687500000, -0.0007019042968750,
+0.0034790039062500, 0.0002746582031250, 0.0033874511718750, -0.0002746582031250,
+0.0032043457031250, 0.0013427734375000, 0.0033264160156250, 0.0011596679687500,
+0.0031433105468750, 0.0015258789062500, 0.0028381347656250, 0.0026245117187500,
+0.0031127929687500, 0.0020141601562500, 0.0027770996093750, 0.0029602050781250,
+0.0028076171875000, 0.0029907226562500, 0.0028381347656250, 0.0029602050781250,
+0.0027770996093750, 0.0034179687500000, 0.0028991699218750, 0.0031127929687500,
+0.0028686523437500, 0.0033264160156250, 0.0029296875000000, 0.0032043457031250,
+0.0030212402343750, 0.0032348632812500, 0.0030212402343750, 0.0033569335937500,
+0.0030822753906250, 0.0031738281250000, 0.0031738281250000, 0.0034179687500000,
+0.0031738281250000, 0.0032958984375000, 0.0032958984375000, 0.0032958984375000,
+0.0033874511718750, 0.0035095214843750, 0.0033874511718750, 0.0033569335937500,
+0.0037231445312500, 0.0033264160156250, 0.0037231445312500, 0.0033569335937500,
+0.0038146972656250, 0.0032348632812500, 0.0040588378906250, 0.0031433105468750,
+0.0040283203125000, 0.0031433105468750, 0.0041809082031250, 0.0028381347656250,
+0.0042419433593750, 0.0029602050781250, 0.0042419433593750, 0.0027465820312500,
+0.0043334960937500, 0.0023803710937500, 0.0043029785156250, 0.0026245117187500,
+0.0043945312500000, 0.0022888183593750, 0.0043640136718750, 0.0021667480468750,
+0.0043029785156250, 0.0023803710937500, 0.0043334960937500, 0.0024108886718750,
+0.0043029785156250, 0.0023193359375000, 0.0041809082031250, 0.0026855468750000,
+0.0042114257812500, 0.0027770996093750, 0.0042114257812500, 0.0026855468750000,
+0.0041503906250000, 0.0029296875000000, 0.0041503906250000, 0.0030212402343750,
+0.0040893554687500, 0.0023803710937500, 0.0041198730468750, 0.0026550292968750,
+0.0039978027343750, 0.0023803710937500, 0.0039062500000000, 0.0015869140625000,
+0.0040283203125000, 0.0019836425781250, 0.0039367675781250, 0.0014648437500000,
+0.0039062500000000, 0.0011596679687500, 0.0038757324218750, 0.0014648437500000,
+0.0037841796875000, 0.0014343261718750, 0.0038452148437500, 0.0012512207031250,
+0.0037231445312500, 0.0018920898437500, 0.0036621093750000, 0.0017700195312500,
+0.0036010742187500, 0.0018310546875000, 0.0034790039062500, 0.0022277832031250,
+0.0034484863281250, 0.0020141601562500, 0.0032348632812500, 0.0021972656250000,
+0.0032043457031250, 0.0021667480468750, 0.0030212402343750, 0.0021972656250000,
+0.0027465820312500, 0.0023498535156250, 0.0028076171875000, 0.0021362304687500,
+0.0025634765625000, 0.0026245117187500, 0.0024108886718750, 0.0024108886718750,
+0.0025634765625000, 0.0025939941406250, 0.0025329589843750, 0.0031127929687500,
+0.0023803710937500, 0.0027465820312500, 0.0028686523437500, 0.0034179687500000,
+0.0027160644531250, 0.0034179687500000, 0.0028991699218750, 0.0032653808593750,
+0.0033874511718750, 0.0035705566406250, 0.0030212402343750, 0.0035400390625000,
+0.0033264160156250, 0.0031127929687500, 0.0036010742187500, 0.0032653808593750,
+0.0032043457031250, 0.0030517578125000, 0.0029602050781250, 0.0026550292968750,
+0.0032958984375000, 0.0029296875000000, 0.0023803710937500, 0.0021667480468750,
+0.0025024414062500, 0.0021667480468750, 0.0025329589843750, 0.0021362304687500,
+0.0019226074218750, 0.0016784667968750, 0.0021972656250000, 0.0016784667968750,
+0.0025329589843750, 0.0017089843750000, 0.0024108886718750, 0.0014953613281250,
+0.0026245117187500, 0.0017395019531250, 0.0031433105468750, 0.0018615722656250,
+0.0029602050781250, 0.0015563964843750, 0.0030212402343750, 0.0028686523437500,
+0.0032348632812500, 0.0025634765625000, 0.0031738281250000, 0.0028381347656250,
+0.0028686523437500, 0.0045471191406250, 0.0028991699218750, 0.0014953613281250,
+0.0033874511718750, 0.0022583007812500, 0.0033569335937500, 0.0018005371093750,
+0.0029296875000000, 0.0023803710937500, 0.0028381347656250, 0.0031738281250000,
+0.0029907226562500, 0.0028381347656250, 0.0021057128906250, 0.0035400390625000,
+0.0022277832031250, 0.0036315917968750, 0.0018920898437500, 0.0034484863281250,
+0.0012817382812500, 0.0036621093750000, 0.0015563964843750, 0.0036926269531250,
+0.0006408691406250, 0.0031433105468750, 0.0007019042968750, 0.0034790039062500,
+0.0004272460937500, 0.0031127929687500, -0.0002746582031250, 0.0025329589843750,
+0.0001220703125000, 0.0030517578125000, -0.0004577636718750, 0.0018615722656250,
+-0.0005493164062500, 0.0018615722656250, -0.0003356933593750, 0.0018920898437500,
+-0.0004577636718750, 0.0011291503906250, -0.0004577636718750, 0.0011901855468750,
+0.0000305175781250, 0.0015869140625000, 0.0000000000000000, 0.0010986328125000,
+0.0001220703125000, 0.0017700195312500, 0.0005798339843750, 0.0025329589843750,
+0.0004577636718750, 0.0018920898437500, 0.0008239746093750, 0.0033569335937500,
+0.0008239746093750, 0.0035400390625000, 0.0009460449218750, 0.0031738281250000,
+0.0011596679687500, 0.0036010742187500, 0.0010681152343750, 0.0036010742187500,
+0.0011291503906250, 0.0029296875000000, 0.0012817382812500, 0.0032043457031250,
+0.0010986328125000, 0.0029296875000000, 0.0009765625000000, 0.0022277832031250,
+0.0010681152343750, 0.0027160644531250, 0.0002746582031250, 0.0021972656250000,
+0.0004882812500000, 0.0021057128906250, 0.0003051757812500, 0.0022583007812500,
+-0.0004272460937500, 0.0020751953125000, -0.0001525878906250, 0.0020141601562500,
+-0.0002441406250000, 0.0022583007812500, -0.0005798339843750, 0.0022277832031250,
+-0.0000610351562500, 0.0023193359375000, 0.0003051757812500, 0.0025329589843750,
+-0.0001220703125000, 0.0025329589843750, 0.0013427734375000, 0.0027770996093750,
+0.0009765625000000, 0.0027465820312500, 0.0014648437500000, 0.0029602050781250,
+0.0026245117187500, 0.0031433105468750, 0.0019226074218750, 0.0028991699218750,
+0.0032348632812500, 0.0034790039062500, 0.0031738281250000, 0.0035095214843750,
+0.0033264160156250, 0.0036926269531250, 0.0041198730468750, 0.0042114257812500,
+0.0037231445312500, 0.0039978027343750, 0.0042724609375000, 0.0043640136718750,
+0.0044250488281250, 0.0045471191406250, 0.0041198730468750, 0.0043334960937500,
+0.0041503906250000, 0.0042724609375000, 0.0043640136718750, 0.0044555664062500,
+0.0032958984375000, 0.0039672851562500, 0.0036621093750000, 0.0040588378906250,
+0.0031738281250000, 0.0040283203125000, 0.0022583007812500, 0.0037231445312500,
+0.0028686523437500, 0.0038452148437500, 0.0017395019531250, 0.0037841796875000,
+0.0016479492187500, 0.0036010742187500, 0.0018615722656250, 0.0037536621093750,
+0.0013732910156250, 0.0038452148437500, 0.0014038085937500, 0.0036926269531250,
+0.0022277832031250, 0.0039367675781250, 0.0017089843750000, 0.0038146972656250,
+0.0021667480468750, 0.0036926269531250, 0.0031738281250000, 0.0038452148437500,
+0.0023803710937500, 0.0036926269531250, 0.0035705566406250, 0.0035705566406250,
+0.0035095214843750, 0.0036621093750000, 0.0032653808593750, 0.0035705566406250,
+0.0038757324218750, 0.0033874511718750, 0.0035400390625000, 0.0035400390625000,
+0.0030822753906250, 0.0034179687500000, 0.0034179687500000, 0.0034484863281250,
+0.0025329589843750, 0.0034790039062500, 0.0017395019531250, 0.0035095214843750,
+0.0024414062500000, 0.0036315917968750, 0.0001831054687500, 0.0036010742187500,
+0.0005798339843750, 0.0036926269531250, -0.0000305175781250, 0.0036926269531250,
+-0.0017700195312500, 0.0036926269531250, -0.0008239746093750, 0.0037536621093750,
+-0.0023498535156250, 0.0036315917968750, -0.0025024414062500, 0.0036926269531250,
+-0.0021972656250000, 0.0037231445312500, -0.0028076171875000, 0.0036315917968750,
+-0.0026855468750000, 0.0037231445312500, -0.0017089843750000, 0.0037536621093750,
+-0.0022583007812500, 0.0037841796875000, -0.0013122558593750, 0.0038452148437500,
+-0.0000610351562500, 0.0038452148437500, -0.0010070800781250, 0.0038146972656250,
+0.0015869140625000, 0.0039367675781250, 0.0011291503906250, 0.0038452148437500,
+0.0018615722656250, 0.0039062500000000, 0.0038146972656250, 0.0040283203125000,
+0.0027465820312500, 0.0038452148437500, 0.0045776367187500, 0.0042724609375000,
+0.0046386718750000, 0.0041198730468750, 0.0044555664062500, 0.0042419433593750,
+0.0053405761718750, 0.0045776367187500, 0.0049133300781250, 0.0042724609375000,
+0.0048217773437500, 0.0046691894531250, 0.0049743652343750, 0.0045776367187500,
+0.0045471191406250, 0.0045776367187500, 0.0041503906250000, 0.0047607421875000,
+0.0043029785156250, 0.0046386718750000, 0.0035705566406250, 0.0047302246093750,
+0.0035400390625000, 0.0046997070312500, 0.0032958984375000, 0.0046691894531250,
+0.0027465820312500, 0.0046691894531250, 0.0028686523437500, 0.0046081542968750,
+0.0025329589843750, 0.0045776367187500, 0.0023498535156250, 0.0046081542968750,
+0.0022888183593750, 0.0044860839843750, 0.0021057128906250, 0.0043945312500000,
+0.0021362304687500, 0.0043945312500000, 0.0021362304687500, 0.0042724609375000,
+0.0020446777343750, 0.0042114257812500, 0.0021057128906250, 0.0042114257812500,
+0.0022277832031250, 0.0041198730468750, 0.0020751953125000, 0.0039978027343750,
+0.0024719238281250, 0.0041198730468750, 0.0024108886718750, 0.0040893554687500,
+0.0025329589843750, 0.0039978027343750, 0.0028076171875000, 0.0041503906250000,
+0.0026245117187500, 0.0040893554687500, 0.0027160644531250, 0.0039367675781250,
+0.0028076171875000, 0.0039978027343750, 0.0026550292968750, 0.0039062500000000,
+0.0025024414062500, 0.0037536621093750, 0.0025329589843750, 0.0038146972656250,
+0.0022888183593750, 0.0037536621093750, 0.0022583007812500, 0.0036621093750000,
+0.0021057128906250, 0.0036621093750000, 0.0019226074218750, 0.0036315917968750,
+0.0019531250000000},
+{0.0032653808593750, 0.0045471191406250, 0.0030517578125000, 0.0044555664062500,
+0.0043029785156250, 0.0044860839843750, 0.0040893554687500, 0.0044860839843750,
+0.0044250488281250, 0.0045166015625000, 0.0054931640625000, 0.0045471191406250,
+0.0050964355468750, 0.0045776367187500, 0.0054016113281250, 0.0046081542968750,
+0.0057983398437500, 0.0047302246093750, 0.0053405761718750, 0.0046386718750000,
+0.0051269531250000, 0.0045776367187500, 0.0054931640625000, 0.0046997070312500,
+0.0040588378906250, 0.0045471191406250, 0.0045471191406250, 0.0045471191406250,
+0.0039062500000000, 0.0044555664062500, 0.0026245117187500, 0.0043945312500000,
+0.0035095214843750, 0.0044555664062500, 0.0018005371093750, 0.0042419433593750,
+0.0018310546875000, 0.0041809082031250, 0.0018005371093750, 0.0041503906250000,
+0.0008544921875000, 0.0041198730468750, 0.0011596679687500, 0.0040283203125000,
+0.0011596679687500, 0.0040893554687500, 0.0007324218750000, 0.0041198730468750,
+0.0011901855468750, 0.0040588378906250, 0.0015563964843750, 0.0041198730468750,
+0.0009155273437500, 0.0043029785156250, 0.0020141601562500, 0.0039978027343750,
+0.0016784667968750, 0.0042419433593750, 0.0017700195312500, 0.0041503906250000,
+0.0024719238281250, 0.0038757324218750, 0.0018615722656250, 0.0041503906250000,
+0.0023498535156250, 0.0038452148437500, 0.0022888183593750, 0.0038452148437500,
+0.0020751953125000, 0.0039672851562500, 0.0020751953125000, 0.0037841796875000,
+0.0019836425781250, 0.0038452148437500, 0.0018005371093750, 0.0039978027343750,
+0.0017395019531250, 0.0039062500000000, 0.0017395019531250, 0.0039672851562500,
+0.0016479492187500, 0.0041503906250000, 0.0015258789062500, 0.0040283203125000,
+0.0018005371093750, 0.0042419433593750, 0.0016784667968750, 0.0042419433593750,
+0.0019226074218750, 0.0042724609375000, 0.0022888183593750, 0.0043640136718750,
+0.0021362304687500, 0.0043029785156250, 0.0028076171875000, 0.0044250488281250,
+0.0028381347656250, 0.0043640136718750, 0.0028381347656250, 0.0043640136718750,
+0.0032348632812500, 0.0044555664062500, 0.0031127929687500, 0.0043640136718750,
+0.0029907226562500, 0.0043334960937500, 0.0031127929687500, 0.0043334960937500,
+0.0028381347656250, 0.0042114257812500, 0.0025024414062500, 0.0041198730468750,
+0.0026855468750000, 0.0040893554687500, 0.0021057128906250, 0.0041809082031250,
+0.0021362304687500, 0.0040283203125000, 0.0021057128906250, 0.0040893554687500,
+0.0017089843750000, 0.0042419433593750, 0.0018615722656250, 0.0040893554687500,
+0.0017089843750000, 0.0042114257812500, 0.0016479492187500, 0.0041809082031250,
+0.0016174316406250, 0.0040893554687500, 0.0015258789062500, 0.0041198730468750,
+0.0015563964843750, 0.0041198730468750, 0.0014648437500000, 0.0039367675781250,
+0.0013732910156250, 0.0039978027343750, 0.0015258789062500, 0.0038452148437500,
+0.0015563964843750, 0.0036315917968750, 0.0014343261718750, 0.0037231445312500,
+0.0021667480468750, 0.0035095214843750, 0.0018920898437500, 0.0035095214843750,
+0.0022277832031250, 0.0034484863281250, 0.0028686523437500, 0.0032958984375000,
+0.0024719238281250, 0.0033874511718750, 0.0032348632812500, 0.0032348632812500,
+0.0032043457031250, 0.0031738281250000, 0.0031738281250000, 0.0031433105468750,
+0.0035705566406250, 0.0030822753906250, 0.0032958984375000, 0.0029907226562500,
+0.0032348632812500, 0.0030822753906250, 0.0032043457031250, 0.0030212402343750,
+0.0029907226562500, 0.0030517578125000, 0.0027770996093750, 0.0031433105468750,
+0.0027160644531250, 0.0031433105468750, 0.0025024414062500, 0.0032653808593750,
+0.0023803710937500, 0.0032348632812500, 0.0022277832031250, 0.0032958984375000,
+0.0020446777343750, 0.0033874511718750, 0.0019531250000000, 0.0034179687500000,
+0.0018615722656250, 0.0034790039062500, 0.0017700195312500, 0.0035400390625000,
+0.0017395019531250, 0.0034179687500000, 0.0016784667968750, 0.0033569335937500,
+0.0015563964843750, 0.0033264160156250, 0.0015563964843750, 0.0029907226562500,
+0.0014343261718750, 0.0030822753906250, 0.0014953613281250, 0.0029907226562500,
+0.0015869140625000, 0.0027770996093750, 0.0013732910156250, 0.0028991699218750,
+0.0017089843750000, 0.0026855468750000, 0.0018005371093750, 0.0028686523437500,
+0.0018310546875000, 0.0027770996093750, 0.0019836425781250, 0.0025634765625000,
+0.0020751953125000, 0.0027770996093750, 0.0019836425781250, 0.0024414062500000,
+0.0021362304687500, 0.0024719238281250, 0.0021972656250000, 0.0025634765625000,
+0.0021972656250000, 0.0023498535156250, 0.0023498535156250, 0.0024719238281250,
+0.0025024414062500, 0.0026245117187500, 0.0026245117187500, 0.0026550292968750,
+0.0026855468750000, 0.0026245117187500, 0.0027465820312500, 0.0027770996093750,
+0.0025024414062500, 0.0028076171875000, 0.0020446777343750, 0.0025024414062500,
+0.0022277832031250, 0.0027465820312500, 0.0018615722656250, 0.0025634765625000,
+0.0011596679687500, 0.0022583007812500, 0.0013427734375000, 0.0024719238281250,
+0.0007324218750000, 0.0018310546875000, 0.0006103515625000, 0.0018310546875000,
+0.0007019042968750, 0.0017700195312500, 0.0003662109375000, 0.0014038085937500,
+0.0003967285156250, 0.0014953613281250, 0.0013122558593750, 0.0014648437500000,
+0.0007629394531250, 0.0012817382812500, 0.0010070800781250, 0.0014343261718750,
+0.0020141601562500, 0.0016174316406250, 0.0015258789062500, 0.0014648437500000,
+0.0018005371093750, 0.0018920898437500, 0.0019836425781250, 0.0019531250000000,
+0.0017089843750000, 0.0019836425781250, 0.0017700195312500, 0.0021972656250000,
+0.0020446777343750, 0.0021362304687500, 0.0017395019531250, 0.0020141601562500,
+0.0016174316406250, 0.0022277832031250, 0.0020751953125000, 0.0021057128906250,
+0.0021972656250000, 0.0018310546875000, 0.0020751953125000, 0.0019836425781250,
+0.0030212402343750, 0.0019226074218750, 0.0029296875000000, 0.0018005371093750,
+0.0032043457031250, 0.0020751953125000, 0.0040283203125000, 0.0022888183593750,
+0.0037231445312500, 0.0021057128906250, 0.0039367675781250, 0.0026245117187500,
+0.0041809082031250, 0.0027160644531250, 0.0040283203125000, 0.0026855468750000,
+0.0039367675781250, 0.0028686523437500, 0.0041503906250000, 0.0028076171875000,
+0.0038452148437500, 0.0028686523437500, 0.0039062500000000, 0.0028381347656250,
+0.0040588378906250, 0.0029296875000000, 0.0039978027343750, 0.0029602050781250,
+0.0039367675781250, 0.0028381347656250, 0.0042724609375000, 0.0032348632812500,
+0.0041198730468750, 0.0031127929687500, 0.0041809082031250, 0.0032348632812500,
+0.0044250488281250, 0.0034484863281250, 0.0042724609375000, 0.0032653808593750,
+0.0042114257812500, 0.0033569335937500, 0.0043945312500000, 0.0033874511718750,
+0.0042114257812500, 0.0032958984375000, 0.0039978027343750, 0.0032653808593750,
+0.0042724609375000, 0.0032653808593750, 0.0039978027343750, 0.0031738281250000,
+0.0040893554687500, 0.0031738281250000, 0.0043334960937500, 0.0032348632812500,
+0.0043640136718750, 0.0032348632812500, 0.0044555664062500, 0.0033264160156250,
+0.0048217773437500, 0.0035705566406250, 0.0050048828125000, 0.0035095214843750,
+0.0049438476562500, 0.0037536621093750, 0.0052795410156250, 0.0039672851562500,
+0.0054626464843750, 0.0038452148437500, 0.0050659179687500, 0.0038757324218750,
+0.0054626464843750, 0.0039672851562500, 0.0053100585937500, 0.0037841796875000,
+0.0048522949218750, 0.0035400390625000, 0.0053710937500000, 0.0035705566406250,
+0.0046691894531250, 0.0032958984375000, 0.0048828125000000, 0.0031738281250000,
+0.0048217773437500, 0.0031127929687500, 0.0043640136718750, 0.0028991699218750,
+0.0048522949218750, 0.0028991699218750, 0.0041503906250000, 0.0030212402343750,
+0.0044250488281250, 0.0029296875000000, 0.0042724609375000, 0.0030517578125000,
+0.0037231445312500, 0.0032653808593750, 0.0042114257812500, 0.0031433105468750,
+0.0034790039062500, 0.0034179687500000, 0.0035705566406250, 0.0034179687500000,
+0.0036621093750000, 0.0034179687500000, 0.0033569335937500, 0.0035705566406250,
+0.0035705566406250, 0.0035705566406250, 0.0035400390625000, 0.0035095214843750,
+0.0035400390625000, 0.0035400390625000, 0.0036010742187500, 0.0035095214843750,
+0.0036926269531250, 0.0034484863281250, 0.0037536621093750, 0.0034790039062500,
+0.0035400390625000, 0.0034790039062500, 0.0036621093750000, 0.0035095214843750,
+0.0035705566406250, 0.0034484863281250, 0.0034790039062500, 0.0034484863281250,
+0.0036010742187500, 0.0036010742187500, 0.0034484863281250, 0.0031433105468750,
+0.0034179687500000, 0.0032958984375000, 0.0036010742187500, 0.0030212402343750,
+0.0036010742187500, 0.0025634765625000, 0.0035095214843750, 0.0028991699218750,
+0.0038757324218750, 0.0021667480468750, 0.0038452148437500, 0.0022277832031250,
+0.0039672851562500, 0.0021667480468750, 0.0042724609375000, 0.0017089843750000,
+0.0042114257812500, 0.0019531250000000, 0.0044860839843750, 0.0016479492187500,
+0.0044860839843750, 0.0016174316406250, 0.0045776367187500, 0.0018005371093750,
+0.0046997070312500, 0.0018005371093750, 0.0047302246093750, 0.0018310546875000,
+0.0048828125000000, 0.0020446777343750, 0.0048828125000000, 0.0020141601562500,
+0.0049743652343750, 0.0022277832031250, 0.0050659179687500, 0.0024108886718750,
+0.0050048828125000, 0.0022888183593750, 0.0051269531250000, 0.0027770996093750,
+0.0050659179687500, 0.0026855468750000, 0.0050354003906250, 0.0028381347656250,
+0.0050964355468750, 0.0031738281250000, 0.0049438476562500, 0.0029602050781250,
+0.0049743652343750, 0.0033874511718750, 0.0048828125000000, 0.0034179687500000,
+0.0048217773437500, 0.0034790039062500, 0.0048217773437500, 0.0037536621093750,
+0.0046997070312500, 0.0036621093750000, 0.0046386718750000, 0.0039367675781250,
+0.0046691894531250, 0.0040893554687500, 0.0046081542968750, 0.0040588378906250,
+0.0045776367187500, 0.0040893554687500, 0.0045471191406250, 0.0042724609375000,
+0.0044250488281250, 0.0038146972656250, 0.0044250488281250, 0.0039978027343750,
+0.0043640136718750, 0.0038452148437500, 0.0043334960937500, 0.0033569335937500,
+0.0043334960937500, 0.0036010742187500, 0.0041198730468750, 0.0033264160156250,
+0.0041809082031250, 0.0031127929687500, 0.0040283203125000, 0.0033264160156250,
+0.0038146972656250, 0.0034179687500000, 0.0039672851562500, 0.0032653808593750,
+0.0035400390625000, 0.0037231445312500, 0.0036315917968750, 0.0037231445312500,
+0.0034790039062500, 0.0036621093750000, 0.0032043457031250, 0.0040588378906250,
+0.0034790039062500, 0.0040588378906250, 0.0029602050781250, 0.0036315917968750,
+0.0031433105468750, 0.0041198730468750, 0.0030822753906250, 0.0036315917968750,
+0.0026855468750000, 0.0029907226562500, 0.0029907226562500, 0.0034790039062500,
+0.0027465820312500, 0.0024719238281250, 0.0027465820312500, 0.0024108886718750,
+0.0028686523437500, 0.0024719238281250, 0.0028991699218750, 0.0020141601562500,
+0.0028991699218750, 0.0021972656250000, 0.0027465820312500, 0.0022277832031250,
+0.0029907226562500, 0.0021972656250000, 0.0026855468750000, 0.0023498535156250,
+0.0023193359375000, 0.0025024414062500, 0.0026245117187500, 0.0024108886718750,
+0.0020446777343750, 0.0025634765625000, 0.0019836425781250, 0.0026855468750000,
+0.0020141601562500, 0.0025024414062500, 0.0018310546875000, 0.0022888183593750,
+0.0019836425781250, 0.0023803710937500, 0.0021667480468750, 0.0021057128906250,
+0.0022277832031250},
+{0.0035400390625000, 0.0054321289062500, 0.0037231445312500, 0.0054016113281250,
+0.0037841796875000, 0.0053100585937500, 0.0035400390625000, 0.0054321289062500,
+0.0038452148437500, 0.0055236816406250, 0.0037231445312500, 0.0051269531250000,
+0.0038146972656250, 0.0053710937500000, 0.0039978027343750, 0.0050354003906250,
+0.0039978027343750, 0.0046081542968750, 0.0040893554687500, 0.0050659179687500,
+0.0041809082031250, 0.0041198730468750, 0.0042419433593750, 0.0043640136718750,
+0.0042724609375000, 0.0041198730468750, 0.0043334960937500, 0.0033874511718750,
+0.0044250488281250, 0.0039062500000000, 0.0044250488281250, 0.0027770996093750,
+0.0044250488281250, 0.0028991699218750, 0.0044860839843750, 0.0027465820312500,
+0.0045166015625000, 0.0019531250000000, 0.0045776367187500, 0.0023803710937500,
+0.0046386718750000, 0.0017700195312500, 0.0046081542968750, 0.0016784667968750,
+0.0046081542968750, 0.0018615722656250, 0.0045776367187500, 0.0017700195312500,
+0.0045471191406250, 0.0016784667968750, 0.0044860839843750, 0.0020751953125000,
+0.0044860839843750, 0.0020141601562500, 0.0043945312500000, 0.0022583007812500,
+0.0042724609375000, 0.0026855468750000, 0.0043029785156250, 0.0025329589843750,
+0.0040588378906250, 0.0031433105468750, 0.0040283203125000, 0.0031127929687500,
+0.0040588378906250, 0.0034179687500000, 0.0039672851562500, 0.0038757324218750,
+0.0040893554687500, 0.0036315917968750, 0.0040893554687500, 0.0043945312500000,
+0.0040588378906250, 0.0043334960937500, 0.0042114257812500, 0.0044860839843750,
+0.0043029785156250, 0.0049438476562500, 0.0042724609375000, 0.0046997070312500,
+0.0043640136718750, 0.0051269531250000, 0.0044250488281250, 0.0051269531250000,
+0.0043640136718750, 0.0052185058593750, 0.0043945312500000, 0.0054626464843750,
+0.0043334960937500, 0.0052795410156250, 0.0040893554687500, 0.0056152343750000,
+0.0041809082031250, 0.0056457519531250, 0.0039978027343750, 0.0056457519531250,
+0.0037231445312500, 0.0058898925781250, 0.0039367675781250, 0.0058288574218750,
+0.0034790039062500, 0.0055847167968750, 0.0035095214843750, 0.0057373046875000,
+0.0034179687500000, 0.0054626464843750, 0.0031738281250000, 0.0050354003906250,
+0.0033569335937500, 0.0052490234375000, 0.0031738281250000, 0.0045166015625000,
+0.0032348632812500, 0.0044555664062500, 0.0032653808593750, 0.0043334960937500,
+0.0031127929687500, 0.0037841796875000, 0.0033264160156250, 0.0038757324218750,
+0.0032348632812500, 0.0037841796875000, 0.0032043457031250, 0.0036010742187500,
+0.0031433105468750, 0.0036926269531250, 0.0030822753906250, 0.0038452148437500,
+0.0031433105468750, 0.0038146972656250, 0.0027770996093750, 0.0037231445312500,
+0.0029602050781250, 0.0039062500000000, 0.0026855468750000, 0.0037231445312500,
+0.0023193359375000, 0.0034790039062500, 0.0026550292968750, 0.0036926269531250,
+0.0020751953125000, 0.0032958984375000, 0.0020141601562500, 0.0032653808593750,
+0.0021972656250000, 0.0034179687500000, 0.0020141601562500, 0.0032653808593750,
+0.0020446777343750, 0.0032653808593750, 0.0022888183593750, 0.0038146972656250,
+0.0024108886718750, 0.0036315917968750, 0.0023193359375000, 0.0038757324218750,
+0.0024414062500000, 0.0044860839843750, 0.0027770996093750, 0.0041503906250000,
+0.0021362304687500, 0.0044860839843750, 0.0025634765625000, 0.0045776367187500,
+0.0023498535156250, 0.0045166015625000, 0.0017700195312500, 0.0047302246093750,
+0.0023803710937500, 0.0046081542968750, 0.0018310546875000, 0.0043945312500000,
+0.0020141601562500, 0.0046081542968750, 0.0020751953125000, 0.0042419433593750,
+0.0019226074218750, 0.0037841796875000, 0.0022583007812500, 0.0040588378906250,
+0.0017089843750000, 0.0033264160156250, 0.0021362304687500, 0.0034484863281250,
+0.0017700195312500, 0.0031433105468750, 0.0011291503906250, 0.0025634765625000,
+0.0017089843750000, 0.0029907226562500, 0.0005187988281250, 0.0019836425781250,
+0.0005493164062500, 0.0018615722656250, 0.0006713867187500, 0.0021057128906250,
+-0.0001831054687500, 0.0017395019531250, -0.0000915527343750, 0.0042114257812500,
+0.0010070800781250, 0.0030822753906250, 0.0004577636718750, 0.0035400390625000,
+0.0009765625000000, 0.0030517578125000, 0.0016784667968750, 0.0023193359375000,
+0.0011291503906250, 0.0027770996093750, 0.0024719238281250, 0.0020751953125000,
+0.0022888183593750, 0.0020446777343750, 0.0025024414062500, 0.0019836425781250,
+0.0033569335937500, 0.0015258789062500, 0.0028991699218750, 0.0017395019531250,
+0.0034484863281250, 0.0016479492187500, 0.0034484863281250, 0.0017089843750000,
+0.0034790039062500, 0.0015258789062500, 0.0037536621093750, 0.0015258789062500,
+0.0035400390625000, 0.0018310546875000, 0.0036621093750000, 0.0008239746093750,
+0.0037231445312500, 0.0010681152343750, 0.0033569335937500, 0.0010986328125000,
+0.0031433105468750, 0.0003662109375000, 0.0033264160156250, 0.0006408691406250,
+0.0024719238281250, 0.0010070800781250, 0.0025634765625000, 0.0006408691406250,
+0.0023803710937500, 0.0013122558593750, 0.0017089843750000, 0.0020141601562500,
+0.0019836425781250, 0.0015258789062500, 0.0019531250000000, 0.0033264160156250,
+0.0017395019531250, 0.0033874511718750, 0.0019836425781250, 0.0032958984375000,
+0.0022583007812500, 0.0043945312500000, 0.0020446777343750, 0.0042419433593750,
+0.0026245117187500, 0.0034790039062500, 0.0025939941406250, 0.0041198730468750,
+0.0025329589843750, 0.0032653808593750, 0.0027770996093750, 0.0019531250000000,
+0.0027770996093750, 0.0027770996093750, 0.0023803710937500, 0.0012512207031250,
+0.0026550292968750, 0.0010681152343750, 0.0024414062500000, 0.0014343261718750,
+0.0020446777343750, 0.0008850097656250, 0.0023803710937500, 0.0008239746093750,
+0.0013732910156250, 0.0016784667968750, 0.0017700195312500, 0.0014953613281250,
+0.0012207031250000, 0.0020141601562500, 0.0003051757812500, 0.0029602050781250,
+0.0010375976562500, 0.0026550292968750, -0.0004577636718750, 0.0033264160156250,
+-0.0002136230468750, 0.0036926269531250, -0.0004272460937500, 0.0034790039062500,
+-0.0014343261718750, 0.0035400390625000, -0.0007629394531250, 0.0037536621093750,
+-0.0012512207031250, 0.0032958984375000, -0.0014953613281250, 0.0034790039062500,
+-0.0008544921875000, 0.0033264160156250, -0.0006713867187500, 0.0028381347656250,
+-0.0009155273437500, 0.0032043457031250, 0.0003356933593750, 0.0028381347656250,
+0.0002136230468750, 0.0029296875000000, 0.0003662109375000, 0.0028991699218750,
+0.0012817382812500, 0.0026855468750000, 0.0010070800781250, 0.0028076171875000,
+0.0010375976562500, 0.0024414062500000, 0.0013427734375000, 0.0025329589843750,
+0.0009460449218750, 0.0025329589843750, 0.0005493164062500, 0.0023498535156250,
+0.0009155273437500, 0.0025939941406250, 0.0001220703125000, 0.0025634765625000,
+0.0001525878906250, 0.0025329589843750, 0.0003356933593750, 0.0027465820312500,
+-0.0000305175781250, 0.0029296875000000, 0.0000610351562500, 0.0027465820312500,
+0.0006408691406250, 0.0030517578125000, 0.0004272460937500, 0.0030517578125000,
+0.0008544921875000, 0.0030212402343750, 0.0015563964843750, 0.0031433105468750,
+0.0012207031250000, 0.0030212402343750, 0.0024414062500000, 0.0030212402343750,
+0.0023803710937500, 0.0030212402343750, 0.0026245117187500, 0.0029907226562500,
+0.0034790039062500, 0.0029602050781250, 0.0032653808593750, 0.0029296875000000,
+0.0035705566406250, 0.0030212402343750, 0.0039062500000000, 0.0028991699218750,
+0.0036926269531250, 0.0029602050781250, 0.0035705566406250, 0.0030822753906250,
+0.0039672851562500, 0.0029602050781250, 0.0032348632812500, 0.0031738281250000,
+0.0034790039062500, 0.0031127929687500, 0.0033874511718750, 0.0030822753906250,
+0.0029602050781250, 0.0031433105468750, 0.0033874511718750, 0.0030822753906250,
+0.0030212402343750, 0.0029907226562500, 0.0031127929687500, 0.0029907226562500,
+0.0032043457031250, 0.0029296875000000, 0.0030822753906250, 0.0027160644531250,
+0.0033264160156250, 0.0028381347656250, 0.0034179687500000, 0.0026550292968750,
+0.0034790039062500, 0.0026550292968750, 0.0036010742187500, 0.0026550292968750,
+0.0037841796875000, 0.0025329589843750, 0.0037231445312500, 0.0025329589843750,
+0.0039672851562500, 0.0026245117187500, 0.0039672851562500, 0.0025024414062500,
+0.0039367675781250, 0.0025939941406250, 0.0040893554687500, 0.0026245117187500,
+0.0040893554687500, 0.0024414062500000, 0.0037536621093750, 0.0027160644531250,
+0.0039367675781250, 0.0026245117187500, 0.0037536621093750, 0.0025939941406250,
+0.0033874511718750, 0.0027465820312500, 0.0036315917968750, 0.0025939941406250,
+0.0030517578125000, 0.0026550292968750, 0.0030212402343750, 0.0026550292968750,
+0.0029907226562500, 0.0026245117187500, 0.0026855468750000, 0.0026550292968750,
+0.0027465820312500, 0.0025939941406250, 0.0030822753906250, 0.0027770996093750,
+0.0028381347656250, 0.0028076171875000, 0.0032958984375000, 0.0027465820312500,
+0.0038757324218750, 0.0028381347656250, 0.0035705566406250, 0.0028686523437500,
+0.0047912597656250, 0.0026855468750000, 0.0048217773437500, 0.0027770996093750,
+0.0049743652343750, 0.0027465820312500, 0.0058288574218750, 0.0026245117187500,
+0.0055541992187500, 0.0027465820312500, 0.0055236816406250, 0.0025634765625000,
+0.0059509277343750, 0.0026245117187500, 0.0054016113281250, 0.0026550292968750,
+0.0049133300781250, 0.0025939941406250, 0.0054016113281250, 0.0027465820312500,
+0.0040283203125000, 0.0025939941406250, 0.0042114257812500, 0.0027160644531250,
+0.0039367675781250, 0.0026855468750000, 0.0030517578125000, 0.0025329589843750,
+0.0036010742187500, 0.0027160644531250, 0.0026245117187500, 0.0025634765625000,
+0.0027770996093750, 0.0025024414062500, 0.0025939941406250, 0.0026855468750000,
+0.0019531250000000, 0.0027465820312500, 0.0024414062500000, 0.0026245117187500,
+0.0016479492187500, 0.0028686523437500, 0.0016784667968750, 0.0028991699218750,
+0.0016784667968750, 0.0029296875000000, 0.0012207031250000, 0.0031127929687500,
+0.0014038085937500, 0.0032043457031250, 0.0015869140625000, 0.0029602050781250,
+0.0014343261718750, 0.0032348632812500, 0.0017700195312500, 0.0031433105468750,
+0.0021057128906250, 0.0029602050781250, 0.0019531250000000, 0.0033569335937500,
+0.0025329589843750, 0.0028686523437500, 0.0025939941406250, 0.0031127929687500,
+0.0026855468750000, 0.0029907226562500, 0.0030822753906250, 0.0026245117187500,
+0.0030822753906250, 0.0030212402343750, 0.0033264160156250, 0.0025329589843750,
+0.0035095214843750, 0.0026245117187500, 0.0034790039062500, 0.0025939941406250,
+0.0035095214843750, 0.0023193359375000, 0.0037536621093750, 0.0026245117187500,
+0.0036315917968750, 0.0021057128906250, 0.0037231445312500, 0.0022888183593750,
+0.0039367675781250, 0.0022583007812500, 0.0039978027343750, 0.0019226074218750,
+0.0040893554687500, 0.0021362304687500, 0.0044860839843750, 0.0020141601562500,
+0.0042114257812500, 0.0018920898437500, 0.0046691894531250, 0.0019836425781250,
+0.0052185058593750, 0.0019836425781250, 0.0047302246093750, 0.0019531250000000,
+0.0059204101562500, 0.0019226074218750, 0.0057983398437500, 0.0019531250000000,
+0.0057067871093750, 0.0018005371093750, 0.0064086914062500, 0.0016784667968750,
+0.0060424804687500, 0.0017700195312500, 0.0056762695312500, 0.0014038085937500,
+0.0060119628906250, 0.0014038085937500, 0.0054626464843750, 0.0014648437500000,
+0.0044555664062500},
+{0.0026855468750000, 0.0025939941406250, 0.0029602050781250, 0.0026245117187500,
+0.0028686523437500, 0.0025024414062500, 0.0029602050781250, 0.0026855468750000,
+0.0029907226562500, 0.0025939941406250, 0.0030212402343750, 0.0026855468750000,
+0.0030212402343750, 0.0027770996093750, 0.0030517578125000, 0.0027160644531250,
+0.0032653808593750, 0.0028686523437500, 0.0031433105468750, 0.0026855468750000,
+0.0032958984375000, 0.0028991699218750, 0.0035400390625000, 0.0031127929687500,
+0.0033264160156250, 0.0028381347656250, 0.0033569335937500, 0.0032958984375000,
+0.0035400390625000, 0.0031127929687500, 0.0031433105468750, 0.0032348632812500,
+0.0029602050781250, 0.0036010742187500, 0.0032043457031250, 0.0032653808593750,
+0.0022583007812500, 0.0036010742187500, 0.0024719238281250, 0.0036315917968750,
+0.0022277832031250, 0.0036010742187500, 0.0014343261718750, 0.0037231445312500,
+0.0018920898437500, 0.0036010742187500, 0.0014343261718750, 0.0036010742187500,
+0.0014038085937500, 0.0036621093750000, 0.0017700195312500, 0.0036315917968750,
+0.0018005371093750, 0.0036315917968750, 0.0018615722656250, 0.0037231445312500,
+0.0027160644531250, 0.0035400390625000, 0.0026245117187500, 0.0036315917968750,
+0.0029907226562500, 0.0034790039062500, 0.0037536621093750, 0.0032958984375000,
+0.0034790039062500, 0.0034790039062500, 0.0038757324218750, 0.0029907226562500,
+0.0041198730468750, 0.0030822753906250, 0.0038452148437500, 0.0029907226562500,
+0.0037536621093750, 0.0027160644531250, 0.0038452148437500, 0.0028686523437500,
+0.0032958984375000, 0.0028076171875000, 0.0032958984375000, 0.0028076171875000,
+0.0031738281250000, 0.0028076171875000, 0.0027465820312500, 0.0028686523437500,
+0.0028686523437500, 0.0029296875000000, 0.0028076171875000, 0.0028381347656250,
+0.0026245117187500, 0.0029602050781250, 0.0026855468750000, 0.0028381347656250,
+0.0027160644531250, 0.0027160644531250, 0.0025634765625000, 0.0028991699218750,
+0.0025939941406250, 0.0026245117187500, 0.0025024414062500, 0.0026245117187500,
+0.0024719238281250, 0.0026550292968750, 0.0024719238281250, 0.0025329589843750,
+0.0023498535156250, 0.0025939941406250, 0.0023498535156250, 0.0026855468750000,
+0.0023193359375000, 0.0025634765625000, 0.0022277832031250, 0.0027160644531250,
+0.0021972656250000, 0.0028686523437500, 0.0022277832031250, 0.0025634765625000,
+0.0021667480468750, 0.0028686523437500, 0.0022583007812500, 0.0028076171875000,
+0.0020751953125000, 0.0027465820312500, 0.0019226074218750, 0.0028076171875000,
+0.0021362304687500, 0.0027465820312500, 0.0014648437500000, 0.0026550292968750,
+0.0017700195312500, 0.0027160644531250, 0.0016174316406250, 0.0026550292968750,
+0.0010986328125000, 0.0025939941406250, 0.0015869140625000, 0.0025939941406250,
+0.0009460449218750, 0.0022888183593750, 0.0007934570312500, 0.0023498535156250,
+0.0010375976562500, 0.0022277832031250, 0.0008239746093750, 0.0019836425781250,
+0.0007324218750000, 0.0021057128906250, 0.0015563964843750, 0.0017700195312500,
+0.0010986328125000, 0.0017089843750000, 0.0017700195312500, 0.0017395019531250,
+0.0028381347656250, 0.0015563964843750, 0.0020751953125000, 0.0015258789062500,
+0.0039367675781250, 0.0016174316406250, 0.0038452148437500, 0.0014343261718750,
+0.0038757324218750, 0.0015869140625000, 0.0051269531250000, 0.0018005371093750,
+0.0049743652343750, 0.0015563964843750, 0.0046997070312500, 0.0020446777343750,
+0.0050048828125000, 0.0018310546875000, 0.0048522949218750, 0.0018615722656250,
+0.0046081542968750, 0.0022277832031250, 0.0049743652343750, 0.0019531250000000,
+0.0041809082031250, 0.0020751953125000, 0.0043945312500000, 0.0020751953125000,
+0.0040588378906250, 0.0019836425781250, 0.0035095214843750, 0.0018920898437500,
+0.0040283203125000, 0.0019226074218750, 0.0028076171875000, 0.0019531250000000,
+0.0030212402343750, 0.0018310546875000, 0.0028381347656250, 0.0018615722656250,
+0.0020751953125000, 0.0020141601562500, 0.0026245117187500, 0.0018615722656250,
+0.0020141601562500, 0.0018310546875000, 0.0018920898437500, 0.0018005371093750,
+0.0020446777343750, 0.0017700195312500, 0.0018920898437500, 0.0016174316406250,
+0.0018005371093750, 0.0015563964843750, 0.0016174316406250, 0.0016174316406250,
+0.0014953613281250, 0.0014953613281250, 0.0013427734375000, 0.0014953613281250,
+0.0010681152343750, 0.0015869140625000, 0.0009765625000000, 0.0013732910156250,
+0.0008239746093750, 0.0014953613281250, 0.0006408691406250, 0.0014343261718750,
+0.0006713867187500, 0.0013427734375000, 0.0006713867187500, 0.0013427734375000,
+0.0003662109375000, 0.0012817382812500, 0.0005493164062500, 0.0010070800781250,
+0.0005798339843750, 0.0009155273437500, 0.0003967285156250, 0.0009765625000000,
+0.0003356933593750, 0.0007629394531250, 0.0002746582031250, 0.0007019042968750,
+0.0001831054687500, 0.0010681152343750, -0.0000305175781250, 0.0007019042968750,
+0.0002441406250000, 0.0011291503906250, 0.0003356933593750, 0.0016174316406250,
+-0.0000610351562500, 0.0010375976562500, 0.0010070800781250, 0.0021667480468750,
+0.0006713867187500, 0.0018920898437500, 0.0010070800781250, 0.0019531250000000,
+0.0018920898437500, 0.0025939941406250, 0.0013122558593750, 0.0021057128906250,
+0.0023193359375000, 0.0025024414062500, 0.0023193359375000, 0.0024414062500000,
+0.0022583007812500, 0.0023498535156250, 0.0026855468750000, 0.0025329589843750,
+0.0025024414062500, 0.0024108886718750, 0.0023803710937500, 0.0024414062500000,
+0.0024108886718750, 0.0025329589843750, 0.0022583007812500, 0.0024414062500000,
+0.0020141601562500, 0.0024108886718750, 0.0020141601562500, 0.0025329589843750,
+0.0020141601562500, 0.0022583007812500, 0.0017395019531250, 0.0025024414062500,
+0.0018920898437500, 0.0023193359375000, 0.0021057128906250, 0.0021057128906250,
+0.0016174316406250, 0.0024719238281250, 0.0022888183593750, 0.0017089843750000,
+0.0019836425781250, 0.0018615722656250, 0.0019531250000000, 0.0018310546875000,
+0.0023498535156250, 0.0012817382812500, 0.0018615722656250, 0.0016174316406250,
+0.0023803710937500, 0.0013122558593750, 0.0019836425781250, 0.0012207031250000,
+0.0020446777343750, 0.0012817382812500, 0.0023498535156250, 0.0012817382812500,
+0.0017089843750000, 0.0012512207031250, 0.0024414062500000, 0.0012207031250000,
+0.0020141601562500, 0.0013122558593750, 0.0020141601562500, 0.0010070800781250,
+0.0024719238281250, 0.0007629394531250, 0.0018005371093750, 0.0009765625000000,
+0.0025024414062500, 0.0001525878906250, 0.0021667480468750, 0.0002136230468750,
+0.0021972656250000, 0.0000000000000000, 0.0026245117187500, -0.0007019042968750,
+0.0019836425781250, -0.0004577636718750, 0.0025939941406250, -0.0007934570312500,
+0.0023193359375000, -0.0010681152343750, 0.0023193359375000, -0.0007324218750000,
+0.0027160644531250, -0.0006408691406250, 0.0022888183593750, -0.0009155273437500,
+0.0028381347656250, 0.0001525878906250, 0.0025939941406250, -0.0001831054687500,
+0.0026550292968750, 0.0003051757812500, 0.0030822753906250, 0.0012817382812500,
+0.0026550292968750, 0.0006408691406250, 0.0030212402343750, 0.0022888183593750,
+0.0029907226562500, 0.0020751953125000, 0.0029602050781250, 0.0024719238281250,
+0.0031433105468750, 0.0036315917968750, 0.0029907226562500, 0.0029907226562500,
+0.0031738281250000, 0.0041503906250000, 0.0031738281250000, 0.0041503906250000,
+0.0030822753906250, 0.0041503906250000, 0.0032348632812500, 0.0046997070312500,
+0.0032043457031250, 0.0044860839843750, 0.0032043457031250, 0.0042419433593750,
+0.0032043457031250, 0.0044860839843750, 0.0032348632812500, 0.0040283203125000,
+0.0031433105468750, 0.0034790039062500, 0.0030822753906250, 0.0039672851562500,
+0.0031738281250000, 0.0026550292968750, 0.0031127929687500, 0.0028381347656250,
+0.0031433105468750, 0.0024414062500000, 0.0032653808593750, 0.0014648437500000,
+0.0031127929687500, 0.0019226074218750, 0.0032958984375000, 0.0010986328125000,
+0.0032043457031250, 0.0010375976562500, 0.0031433105468750, 0.0010070800781250,
+0.0032348632812500, 0.0005798339843750, 0.0031738281250000, 0.0007019042968750,
+0.0028991699218750, 0.0005798339843750, 0.0030212402343750, 0.0003662109375000,
+0.0026855468750000, 0.0006103515625000, 0.0024108886718750, 0.0007324218750000,
+0.0026550292968750, 0.0004882812500000, 0.0018615722656250, 0.0011596679687500,
+0.0020446777343750, 0.0009765625000000, 0.0018920898437500, 0.0011901855468750,
+0.0014038085937500, 0.0017395019531250, 0.0017089843750000, 0.0014038085937500,
+0.0013732910156250, 0.0021057128906250, 0.0013732910156250, 0.0019531250000000,
+0.0014953613281250, 0.0020751953125000, 0.0014953613281250, 0.0025939941406250,
+0.0014953613281250, 0.0022583007812500, 0.0016479492187500, 0.0027160644531250,
+0.0016479492187500, 0.0026855468750000, 0.0017089843750000, 0.0026855468750000,
+0.0018005371093750, 0.0030212402343750, 0.0017700195312500, 0.0028076171875000,
+0.0018310546875000, 0.0028686523437500, 0.0018310546875000, 0.0029907226562500,
+0.0019836425781250, 0.0029602050781250, 0.0020446777343750, 0.0029602050781250,
+0.0020141601562500, 0.0030517578125000, 0.0023498535156250, 0.0029602050781250,
+0.0022277832031250, 0.0032043457031250, 0.0024719238281250, 0.0030517578125000,
+0.0028686523437500, 0.0028381347656250, 0.0025634765625000, 0.0031127929687500,
+0.0031738281250000, 0.0025024414062500, 0.0032043457031250, 0.0025634765625000,
+0.0030212402343750, 0.0024108886718750, 0.0032653808593750, 0.0019531250000000,
+0.0032043457031250, 0.0021057128906250, 0.0027465820312500, 0.0017089843750000,
+0.0028991699218750, 0.0016479492187500, 0.0026550292968750, 0.0014953613281250,
+0.0021667480468750, 0.0012207031250000, 0.0025329589843750, 0.0013122558593750,
+0.0021667480468750, 0.0010070800781250, 0.0021972656250000, 0.0009460449218750,
+0.0022583007812500, 0.0008850097656250, 0.0021362304687500, 0.0007019042968750,
+0.0024108886718750, 0.0007934570312500, 0.0019836425781250, 0.0005798339843750,
+0.0023193359375000, 0.0006713867187500, 0.0019836425781250, 0.0004882812500000,
+0.0014648437500000, 0.0002441406250000, 0.0019836425781250, 0.0005798339843750,
+0.0010375976562500, -0.0003662109375000, 0.0011596679687500, -0.0002746582031250,
+0.0011291503906250, 0.0000000000000000, 0.0004882812500000, -0.0004577636718750,
+0.0007324218750000, -0.0003662109375000, 0.0008239746093750, 0.0005187988281250,
+0.0006103515625000, 0.0003662109375000, 0.0008239746093750, 0.0008544921875000,
+0.0010375976562500, 0.0016784667968750, 0.0008239746093750, 0.0012817382812500,
+0.0013732910156250, 0.0025329589843750, 0.0012207031250000, 0.0024719238281250,
+0.0013122558593750, 0.0024719238281250, 0.0019226074218750, 0.0035095214843750,
+0.0030212402343750, 0.0010070800781250, 0.0032653808593750, 0.0015258789062500,
+0.0032653808593750, 0.0014953613281250, 0.0029296875000000, 0.0019531250000000,
+0.0028991699218750, 0.0022277832031250, 0.0030517578125000, 0.0018310546875000,
+0.0022583007812500, 0.0027770996093750, 0.0025329589843750, 0.0026550292968750,
+0.0021362304687500, 0.0029602050781250, 0.0014953613281250, 0.0036621093750000,
+0.0020141601562500, 0.0031433105468750, 0.0008544921875000, 0.0037536621093750,
+0.0010375976562500, 0.0041503906250000, 0.0009460449218750, 0.0035095214843750,
+0.0001525878906250, 0.0031433105468750, 0.0005187988281250, 0.0035400390625000,
+0.0003967285156250},
+{0.0031433105468750, 0.0005187988281250, 0.0032653808593750, 0.0000915527343750,
+0.0031127929687500, -0.0000915527343750, 0.0031738281250000, -0.0000915527343750,
+0.0032958984375000, -0.0002441406250000, 0.0030822753906250, -0.0003662109375000,
+0.0035095214843750, 0.0000305175781250, 0.0033264160156250, -0.0002746582031250,
+0.0033264160156250, 0.0000305175781250, 0.0036621093750000, 0.0005493164062500,
+0.0034179687500000, 0.0001220703125000, 0.0035400390625000, 0.0008850097656250,
+0.0035400390625000, 0.0008239746093750, 0.0032958984375000, 0.0008850097656250,
+0.0031433105468750, 0.0014648437500000, 0.0031738281250000, 0.0012512207031250,
+0.0027770996093750, 0.0015869140625000, 0.0026855468750000, 0.0015869140625000,
+0.0025634765625000, 0.0018310546875000, 0.0023498535156250, 0.0021362304687500,
+0.0023498535156250, 0.0019531250000000, 0.0024414062500000, 0.0026245117187500,
+0.0021667480468750, 0.0025024414062500, 0.0023803710937500, 0.0026550292968750,
+0.0026550292968750, 0.0031127929687500, 0.0023193359375000, 0.0028686523437500,
+0.0028686523437500, 0.0033569335937500, 0.0027465820312500, 0.0032348632812500,
+0.0027465820312500, 0.0032958984375000, 0.0030822753906250, 0.0036621093750000,
+0.0028686523437500, 0.0034179687500000, 0.0028991699218750, 0.0039672851562500,
+0.0029602050781250, 0.0039062500000000, 0.0027770996093750, 0.0040283203125000,
+0.0025634765625000, 0.0043640136718750, 0.0027160644531250, 0.0041198730468750,
+0.0024414062500000, 0.0043640136718750, 0.0024719238281250, 0.0043029785156250,
+0.0024108886718750, 0.0041503906250000, 0.0022888183593750, 0.0042724609375000,
+0.0024719238281250, 0.0040893554687500, 0.0018920898437500, 0.0036621093750000,
+0.0021667480468750, 0.0037231445312500, 0.0018005371093750, 0.0035095214843750,
+0.0011901855468750, 0.0031127929687500, 0.0017089843750000, 0.0032958984375000,
+0.0010681152343750, 0.0028686523437500, 0.0009460449218750, 0.0028991699218750,
+0.0011596679687500, 0.0027465820312500, 0.0010070800781250, 0.0025634765625000,
+0.0008850097656250, 0.0027160644531250, 0.0014953613281250, 0.0019531250000000,
+0.0012512207031250, 0.0023803710937500, 0.0015258789062500, 0.0017395019531250,
+0.0021362304687500, 0.0007629394531250, 0.0017089843750000, 0.0015563964843750,
+0.0024719238281250, 0.0003967285156250, 0.0023193359375000, 0.0002746582031250,
+0.0024108886718750, 0.0005493164062500, 0.0028991699218750, 0.0002746582031250,
+0.0025024414062500, 0.0003967285156250, 0.0030822753906250, 0.0006103515625000,
+0.0029602050781250, 0.0005187988281250, 0.0028686523437500, 0.0007934570312500,
+0.0033569335937500, 0.0013732910156250, 0.0034790039062500, 0.0023498535156250,
+0.0026245117187500, 0.0019836425781250, 0.0032653808593750, 0.0020751953125000,
+0.0024719238281250, 0.0019836425781250, 0.0016784667968750, 0.0018615722656250,
+0.0023498535156250, 0.0021362304687500, 0.0009155273437500, 0.0022583007812500,
+0.0010986328125000, 0.0022277832031250, 0.0010070800781250, 0.0021667480468750,
+0.0000610351562500, 0.0021057128906250, 0.0005187988281250, 0.0021972656250000,
+0.0001220703125000, 0.0027465820312500, 0.0001220703125000, 0.0022888183593750,
+0.0002441406250000, 0.0027160644531250, 0.0002136230468750, 0.0035400390625000,
+0.0004272460937500, 0.0029907226562500, 0.0003967285156250, 0.0035400390625000,
+0.0005187988281250, 0.0036010742187500, 0.0005493164062500, 0.0034484863281250,
+0.0004577636718750, 0.0036010742187500, 0.0006103515625000, 0.0035095214843750,
+0.0006713867187500, 0.0031127929687500, 0.0006103515625000, 0.0031127929687500,
+0.0007629394531250, 0.0028686523437500, 0.0009155273437500, 0.0024719238281250,
+0.0008850097656250, 0.0025329589843750, 0.0011291503906250, 0.0024414062500000,
+0.0011291503906250, 0.0021972656250000, 0.0011291503906250, 0.0023193359375000,
+0.0012512207031250, 0.0025329589843750, 0.0013122558593750, 0.0023498535156250,
+0.0015563964843750, 0.0023498535156250, 0.0014038085937500, 0.0025939941406250,
+0.0020446777343750, 0.0020141601562500, 0.0025634765625000, 0.0014953613281250,
+0.0020446777343750, 0.0018615722656250, 0.0040283203125000, 0.0005187988281250,
+0.0036010742187500, 0.0004272460937500, 0.0040893554687500, 0.0005187988281250,
+0.0057373046875000, -0.0002136230468750, 0.0049133300781250, -0.0002136230468750,
+0.0058898925781250, 0.0005493164062500, 0.0062866210937500, -0.0000610351562500,
+0.0056762695312500, 0.0007629394531250, 0.0056762695312500, 0.0020141601562500,
+0.0060119628906250, 0.0010986328125000, 0.0045166015625000, 0.0026855468750000,
+0.0049133300781250, 0.0027770996093750, 0.0044250488281250, 0.0025329589843750,
+0.0032348632812500, 0.0031127929687500, 0.0039672851562500, 0.0028991699218750,
+0.0034790039062500, 0.0028076171875000, 0.0031433105468750, 0.0028076171875000,
+0.0037536621093750, 0.0026245117187500, 0.0040283203125000, 0.0024108886718750,
+0.0037841796875000, 0.0025024414062500, 0.0048828125000000, 0.0024719238281250,
+0.0048522949218750, 0.0024108886718750, 0.0050354003906250, 0.0025939941406250,
+0.0057373046875000, 0.0027465820312500, 0.0054931640625000, 0.0026245117187500,
+0.0057067871093750, 0.0031433105468750, 0.0060119628906250, 0.0031738281250000,
+0.0056152343750000, 0.0032043457031250, 0.0054321289062500, 0.0035705566406250,
+0.0057373046875000, 0.0035095214843750, 0.0046691894531250, 0.0035095214843750,
+0.0049743652343750, 0.0035705566406250, 0.0046386718750000, 0.0034484863281250,
+0.0038146972656250, 0.0033569335937500, 0.0043334960937500, 0.0034179687500000,
+0.0034484863281250, 0.0031738281250000, 0.0034484863281250, 0.0032653808593750,
+0.0035095214843750, 0.0031127929687500, 0.0031433105468750, 0.0028991699218750,
+0.0032958984375000, 0.0031433105468750, 0.0031433105468750, 0.0027160644531250,
+0.0032348632812500, 0.0026855468750000, 0.0030517578125000, 0.0027465820312500,
+0.0028686523437500, 0.0025939941406250, 0.0030822753906250, 0.0025939941406250,
+0.0024414062500000, 0.0028076171875000, 0.0024719238281250, 0.0026245117187500,
+0.0023498535156250, 0.0028076171875000, 0.0018920898437500, 0.0030822753906250,
+0.0020446777343750, 0.0027160644531250, 0.0018005371093750, 0.0036010742187500,
+0.0015869140625000, 0.0034179687500000, 0.0017395019531250, 0.0035095214843750,
+0.0017395019531250, 0.0041198730468750, 0.0013427734375000, 0.0037841796875000,
+0.0017395019531250, 0.0041198730468750, 0.0014953613281250, 0.0042419433593750,
+0.0013732910156250, 0.0040588378906250, 0.0015869140625000, 0.0041503906250000,
+0.0012512207031250, 0.0042419433593750, 0.0012207031250000, 0.0040283203125000,
+0.0012207031250000, 0.0040893554687500, 0.0008850097656250, 0.0040893554687500,
+0.0006713867187500, 0.0039978027343750, 0.0007019042968750, 0.0039978027343750,
+0.0003051757812500, 0.0040588378906250, 0.0003356933593750, 0.0039672851562500,
+0.0002441406250000, 0.0041198730468750, 0.0000305175781250, 0.0042114257812500,
+0.0000305175781250, 0.0039672851562500, -0.0000915527343750, 0.0043640136718750,
+-0.0001220703125000, 0.0042114257812500, -0.0001831054687500, 0.0042724609375000,
+-0.0002136230468750, 0.0045166015625000, -0.0001831054687500, 0.0042724609375000,
+-0.0004272460937500, 0.0046081542968750, -0.0002746582031250, 0.0045471191406250,
+-0.0003051757812500, 0.0044555664062500, -0.0004272460937500, 0.0046386718750000,
+-0.0003662109375000, 0.0045166015625000, -0.0005798339843750, 0.0043334960937500,
+-0.0005187988281250, 0.0044250488281250, -0.0004577636718750, 0.0041503906250000,
+-0.0005187988281250, 0.0039672851562500, -0.0005493164062500, 0.0041503906250000,
+-0.0004577636718750, 0.0036926269531250, -0.0004577636718750, 0.0036926269531250,
+-0.0004272460937500, 0.0035705566406250, -0.0003051757812500, 0.0033264160156250,
+-0.0003051757812500, 0.0034790039062500, -0.0003356933593750, 0.0031738281250000,
+-0.0002746582031250, 0.0031738281250000, -0.0003967285156250, 0.0031433105468750,
+-0.0005798339843750, 0.0030212402343750, -0.0005798339843750, 0.0030517578125000,
+-0.0005798339843750, 0.0030822753906250, -0.0008239746093750, 0.0030212402343750,
+-0.0005493164062500, 0.0030822753906250, -0.0003662109375000, 0.0031433105468750,
+-0.0007019042968750, 0.0030517578125000, 0.0004577636718750, 0.0031433105468750,
+0.0001525878906250, 0.0031738281250000, 0.0006713867187500, 0.0031433105468750,
+0.0017700195312500, 0.0030822753906250, 0.0011596679687500, 0.0031738281250000,
+0.0022888183593750, 0.0031738281250000, 0.0024108886718750, 0.0030822753906250,
+0.0022888183593750, 0.0031433105468750, 0.0026855468750000, 0.0031738281250000,
+0.0026550292968750, 0.0030822753906250, 0.0024108886718750, 0.0031738281250000,
+0.0025024414062500, 0.0032043457031250, 0.0024414062500000, 0.0031738281250000,
+0.0022583007812500, 0.0031127929687500, 0.0023193359375000, 0.0030822753906250,
+0.0023193359375000, 0.0031433105468750, 0.0022888183593750, 0.0030517578125000,
+0.0023498535156250, 0.0029602050781250, 0.0024108886718750, 0.0029602050781250,
+0.0025024414062500, 0.0029907226562500, 0.0023498535156250, 0.0028381347656250,
+0.0025024414062500, 0.0028991699218750, 0.0023803710937500, 0.0027770996093750,
+0.0021667480468750, 0.0025939941406250, 0.0023803710937500, 0.0025939941406250,
+0.0021362304687500, 0.0022888183593750, 0.0019836425781250, 0.0021972656250000,
+0.0021362304687500, 0.0021667480468750, 0.0020446777343750, 0.0019531250000000,
+0.0019836425781250, 0.0018310546875000, 0.0021057128906250, 0.0019531250000000,
+0.0021667480468750, 0.0018615722656250, 0.0020141601562500, 0.0018310546875000,
+0.0020141601562500, 0.0018310546875000, 0.0021972656250000, 0.0017395019531250,
+0.0015869140625000, 0.0020141601562500, 0.0018005371093750, 0.0019531250000000,
+0.0015258789062500, 0.0019836425781250, 0.0008850097656250, 0.0022277832031250,
+0.0013122558593750, 0.0021057128906250, 0.0007324218750000, 0.0022583007812500,
+0.0005187988281250, 0.0023193359375000, 0.0007019042968750, 0.0022277832031250,
+0.0004272460937500, 0.0022277832031250, 0.0000000000000000, 0.0021057128906250,
+0.0000915527343750, 0.0019836425781250, 0.0000000000000000, 0.0020446777343750,
+0.0001525878906250, 0.0020751953125000, 0.0002441406250000, 0.0020751953125000,
+0.0000305175781250, 0.0021362304687500, 0.0014038085937500, 0.0019836425781250,
+0.0009765625000000, 0.0020446777343750, 0.0016479492187500, 0.0020446777343750,
+0.0030822753906250, 0.0019531250000000, 0.0023803710937500, 0.0021057128906250,
+0.0037536621093750, 0.0018310546875000, 0.0041198730468750, 0.0019226074218750,
+0.0037841796875000, 0.0019531250000000, 0.0042419433593750, 0.0018310546875000,
+0.0045776367187500, 0.0019226074218750, 0.0036315917968750, 0.0019836425781250,
+0.0040588378906250, 0.0020446777343750, 0.0037231445312500, 0.0020446777343750,
+0.0028991699218750, 0.0021057128906250, 0.0036621093750000, 0.0021972656250000,
+0.0029907226562500, 0.0021667480468750, 0.0030212402343750, 0.0022888183593750,
+0.0031127929687500, 0.0024414062500000, 0.0029296875000000, 0.0024414062500000,
+0.0031738281250000, 0.0025024414062500, 0.0030822753906250, 0.0030212402343750,
+0.0031127929687500, 0.0028686523437500, 0.0031738281250000, 0.0033264160156250,
+0.0032653808593750, 0.0039367675781250, 0.0034484863281250, 0.0035705566406250,
+0.0028991699218750, 0.0042419433593750, 0.0031738281250000, 0.0042724609375000,
+0.0029907226562500},
+{0.0014953613281250, 0.0035095214843750, 0.0014038085937500, 0.0033874511718750,
+0.0011596679687500, 0.0032043457031250, 0.0012207031250000, 0.0033264160156250,
+0.0011901855468750, 0.0031127929687500, 0.0008850097656250, 0.0031738281250000,
+0.0012512207031250, 0.0031433105468750, 0.0014953613281250, 0.0030517578125000,
+0.0009765625000000, 0.0031738281250000, 0.0024108886718750, 0.0030212402343750,
+0.0019836425781250, 0.0030517578125000, 0.0024414062500000, 0.0030517578125000,
+0.0036621093750000, 0.0029602050781250, 0.0029907226562500, 0.0029602050781250,
+0.0040283203125000, 0.0029602050781250, 0.0041809082031250, 0.0029296875000000,
+0.0040283203125000, 0.0029907226562500, 0.0043945312500000, 0.0030212402343750,
+0.0043029785156250, 0.0029602050781250, 0.0042114257812500, 0.0029907226562500,
+0.0042419433593750, 0.0030212402343750, 0.0042114257812500, 0.0028686523437500,
+0.0040588378906250, 0.0028076171875000, 0.0040588378906250, 0.0028381347656250,
+0.0039367675781250, 0.0025024414062500, 0.0039062500000000, 0.0025329589843750,
+0.0038146972656250, 0.0024719238281250, 0.0037841796875000, 0.0021667480468750,
+0.0037536621093750, 0.0021972656250000, 0.0035095214843750, 0.0021057128906250,
+0.0035095214843750, 0.0019836425781250, 0.0033569335937500, 0.0020751953125000,
+0.0030822753906250, 0.0021057128906250, 0.0030822753906250, 0.0019531250000000,
+0.0028991699218750, 0.0020751953125000, 0.0028381347656250, 0.0019531250000000,
+0.0027770996093750, 0.0020141601562500, 0.0026245117187500, 0.0020751953125000,
+0.0026550292968750, 0.0018310546875000, 0.0024414062500000, 0.0020751953125000,
+0.0024719238281250, 0.0020141601562500, 0.0022583007812500, 0.0019531250000000,
+0.0019226074218750, 0.0019531250000000, 0.0020446777343750, 0.0018005371093750,
+0.0017395019531250, 0.0020141601562500, 0.0015869140625000, 0.0018310546875000,
+0.0016174316406250, 0.0018005371093750, 0.0016479492187500, 0.0020751953125000,
+0.0014953613281250, 0.0018310546875000, 0.0013427734375000, 0.0021057128906250,
+0.0014648437500000, 0.0020141601562500, 0.0013732910156250, 0.0020446777343750,
+0.0011596679687500, 0.0021667480468750, 0.0013122558593750, 0.0020751953125000,
+0.0009460449218750, 0.0023498535156250, 0.0006408691406250, 0.0021667480468750,
+0.0011901855468750, 0.0023498535156250, 0.0012817382812500, 0.0027160644531250,
+0.0007019042968750, 0.0025329589843750, 0.0024719238281250, 0.0029907226562500,
+0.0019226074218750, 0.0029907226562500, 0.0023803710937500, 0.0030822753906250,
+0.0039062500000000, 0.0033874511718750, 0.0030517578125000, 0.0033264160156250,
+0.0040283203125000, 0.0033569335937500, 0.0043029785156250, 0.0034484863281250,
+0.0040893554687500, 0.0034484863281250, 0.0043334960937500, 0.0033874511718750,
+0.0043640136718750, 0.0035400390625000, 0.0037841796875000, 0.0035095214843750,
+0.0040588378906250, 0.0034790039062500, 0.0038146972656250, 0.0036010742187500,
+0.0033264160156250, 0.0036315917968750, 0.0036926269531250, 0.0036010742187500,
+0.0029907226562500, 0.0037231445312500, 0.0030822753906250, 0.0037231445312500,
+0.0031738281250000, 0.0036315917968750, 0.0028076171875000, 0.0036315917968750,
+0.0029602050781250, 0.0036315917968750, 0.0031433105468750, 0.0032653808593750,
+0.0031738281250000, 0.0033264160156250, 0.0032043457031250, 0.0031433105468750,
+0.0035400390625000, 0.0028381347656250, 0.0036315917968750, 0.0030822753906250,
+0.0032043457031250, 0.0024414062500000, 0.0036010742187500, 0.0024719238281250,
+0.0034179687500000, 0.0023498535156250, 0.0028991699218750, 0.0018920898437500,
+0.0034790039062500, 0.0021057128906250, 0.0027465820312500, 0.0018615722656250,
+0.0028076171875000, 0.0017089843750000, 0.0030517578125000, 0.0018005371093750,
+0.0027160644531250, 0.0017700195312500, 0.0028381347656250, 0.0015869140625000,
+0.0030517578125000, 0.0018920898437500, 0.0031433105468750, 0.0017395019531250,
+0.0030822753906250, 0.0016784667968750, 0.0032653808593750, 0.0019531250000000,
+0.0034179687500000, 0.0016784667968750, 0.0029907226562500, 0.0016784667968750,
+0.0032348632812500, 0.0017089843750000, 0.0031738281250000, 0.0015563964843750,
+0.0027770996093750, 0.0014038085937500, 0.0030822753906250, 0.0014648437500000,
+0.0030822753906250, 0.0011901855468750, 0.0029296875000000, 0.0011291503906250,
+0.0032653808593750, 0.0012207031250000, 0.0035705566406250, 0.0010375976562500,
+0.0033874511718750, 0.0010070800781250, 0.0039978027343750, 0.0012817382812500,
+0.0040893554687500, 0.0009460449218750, 0.0041809082031250, 0.0012207031250000,
+0.0046691894531250, 0.0015563964843750, 0.0046997070312500, 0.0010375976562500,
+0.0046997070312500, 0.0020751953125000, 0.0049743652343750, 0.0017700195312500,
+0.0048522949218750, 0.0020751953125000, 0.0046997070312500, 0.0028991699218750,
+0.0049438476562500, 0.0023498535156250, 0.0045471191406250, 0.0034179687500000,
+0.0046081542968750, 0.0032958984375000, 0.0046081542968750, 0.0035095214843750,
+0.0043029785156250, 0.0042114257812500, 0.0045166015625000, 0.0038146972656250,
+0.0043945312500000, 0.0045166015625000, 0.0043945312500000, 0.0046081542968750,
+0.0043945312500000, 0.0044860839843750, 0.0043029785156250, 0.0047302246093750,
+0.0044555664062500, 0.0045776367187500, 0.0042724609375000, 0.0043640136718750,
+0.0042724609375000, 0.0044250488281250, 0.0042724609375000, 0.0041198730468750,
+0.0041503906250000, 0.0037841796875000, 0.0041809082031250, 0.0039978027343750,
+0.0040283203125000, 0.0032043457031250, 0.0040893554687500, 0.0032958984375000,
+0.0040588378906250, 0.0029296875000000, 0.0039062500000000, 0.0022583007812500,
+0.0039978027343750, 0.0026855468750000, 0.0039062500000000, 0.0015869140625000,
+0.0039062500000000, 0.0016784667968750, 0.0039367675781250, 0.0013732910156250,
+0.0039062500000000, 0.0005798339843750, 0.0039062500000000, 0.0010375976562500,
+0.0039672851562500, 0.0001525878906250, 0.0039978027343750, 0.0001525878906250,
+0.0039367675781250, 0.0000610351562500, 0.0039367675781250, -0.0004272460937500,
+0.0039062500000000, -0.0002136230468750, 0.0038452148437500, -0.0004882812500000,
+0.0038146972656250, -0.0006408691406250, 0.0038146972656250, -0.0005187988281250,
+0.0037536621093750, -0.0005493164062500, 0.0037231445312500, -0.0005798339843750,
+0.0038452148437500, -0.0000915527343750, 0.0037841796875000, -0.0002746582031250,
+0.0039062500000000, 0.0001220703125000, 0.0039978027343750, 0.0006103515625000,
+0.0038757324218750, 0.0002746582031250, 0.0041503906250000, 0.0014038085937500,
+0.0041198730468750, 0.0012512207031250, 0.0042114257812500, 0.0016174316406250,
+0.0043945312500000, 0.0025329589843750, 0.0043029785156250, 0.0020446777343750,
+0.0044250488281250, 0.0029602050781250, 0.0044860839843750, 0.0030517578125000,
+0.0043640136718750, 0.0030212402343750, 0.0043945312500000, 0.0034790039062500,
+0.0044555664062500, 0.0033874511718750, 0.0042114257812500, 0.0030212402343750,
+0.0042419433593750, 0.0033569335937500, 0.0041809082031250, 0.0027465820312500,
+0.0039672851562500, 0.0021057128906250, 0.0040588378906250, 0.0025939941406250,
+0.0038757324218750, 0.0011901855468750, 0.0038146972656250, 0.0012512207031250,
+0.0037841796875000, 0.0012207031250000, 0.0036621093750000, 0.0003662109375000,
+0.0036926269531250, 0.0007324218750000, 0.0037536621093750, 0.0008850097656250,
+0.0037231445312500, 0.0006103515625000, 0.0037231445312500, 0.0010681152343750,
+0.0037841796875000, 0.0015869140625000, 0.0037536621093750, 0.0012817382812500,
+0.0035400390625000, 0.0020446777343750, 0.0036315917968750, 0.0021667480468750,
+0.0034790039062500, 0.0021667480468750, 0.0032653808593750, 0.0025634765625000,
+0.0034179687500000, 0.0024719238281250, 0.0030212402343750, 0.0025634765625000,
+0.0031738281250000, 0.0025634765625000, 0.0029907226562500, 0.0026855468750000,
+0.0026855468750000, 0.0027465820312500, 0.0030212402343750, 0.0026855468750000,
+0.0023498535156250, 0.0031738281250000, 0.0024719238281250, 0.0030517578125000,
+0.0022277832031250, 0.0032043457031250, 0.0017700195312500, 0.0036621093750000,
+0.0021667480468750, 0.0034179687500000, 0.0016784667968750, 0.0036621093750000,
+0.0015869140625000, 0.0037841796875000, 0.0017700195312500, 0.0036926269531250,
+0.0016174316406250, 0.0036926269531250, 0.0015563964843750, 0.0038146972656250,
+0.0020446777343750, 0.0034179687500000, 0.0018005371093750, 0.0034790039062500,
+0.0019226074218750, 0.0034179687500000, 0.0023498535156250, 0.0032043457031250,
+0.0019836425781250, 0.0031738281250000, 0.0023498535156250, 0.0029296875000000,
+0.0022583007812500, 0.0028991699218750, 0.0022277832031250, 0.0028381347656250,
+0.0023803710937500, 0.0026550292968750, 0.0021667480468750, 0.0026245117187500,
+0.0025634765625000, 0.0025329589843750, 0.0023498535156250, 0.0024719238281250,
+0.0025634765625000, 0.0024719238281250, 0.0029296875000000, 0.0025024414062500,
+0.0025329589843750, 0.0023498535156250, 0.0031738281250000, 0.0024414062500000,
+0.0031127929687500, 0.0023803710937500, 0.0029907226562500, 0.0023193359375000,
+0.0033569335937500, 0.0022888183593750, 0.0031433105468750, 0.0021667480468750,
+0.0028381347656250, 0.0024719238281250, 0.0030212402343750, 0.0022583007812500,
+0.0026245117187500, 0.0023498535156250, 0.0017395019531250, 0.0029907226562500,
+0.0002746582031250, 0.0016784667968750, 0.0006713867187500, 0.0030822753906250,
+0.0002136230468750, 0.0026550292968750, 0.0008544921875000, 0.0027465820312500,
+0.0014953613281250, 0.0033569335937500, 0.0009765625000000, 0.0031433105468750,
+0.0021057128906250, 0.0027770996093750, 0.0019531250000000, 0.0029907226562500,
+0.0021667480468750, 0.0029907226562500, 0.0028381347656250, 0.0026245117187500,
+0.0023803710937500, 0.0026245117187500, 0.0034179687500000, 0.0034484863281250,
+0.0032348632812500, 0.0034790039062500, 0.0034790039062500, 0.0034179687500000,
+0.0042724609375000, 0.0038146972656250, 0.0037536621093750, 0.0039672851562500,
+0.0045166015625000, 0.0035705566406250, 0.0045166015625000, 0.0037231445312500,
+0.0045166015625000, 0.0034790039062500, 0.0048522949218750, 0.0029907226562500,
+0.0046997070312500, 0.0032348632812500, 0.0051879882812500, 0.0030212402343750,
+0.0048828125000000, 0.0026855468750000, 0.0052795410156250, 0.0030212402343750,
+0.0058898925781250, 0.0032653808593750, 0.0054626464843750, 0.0029602050781250,
+0.0062255859375000, 0.0034484863281250, 0.0063476562500000, 0.0034179687500000,
+0.0059814453125000, 0.0033264160156250, 0.0062561035156250, 0.0035095214843750,
+0.0063476562500000, 0.0032958984375000, 0.0051574707031250, 0.0030212402343750,
+0.0056762695312500, 0.0030212402343750, 0.0048828125000000, 0.0028381347656250,
+0.0035095214843750, 0.0025634765625000, 0.0044860839843750, 0.0025329589843750,
+0.0024108886718750, 0.0026245117187500, 0.0025939941406250, 0.0024108886718750,
+0.0025024414062500, 0.0026245117187500, 0.0013427734375000, 0.0027770996093750,
+0.0019531250000000, 0.0024108886718750, 0.0014648437500000, 0.0032653808593750,
+0.0014038085937500, 0.0032653808593750, 0.0015563964843750, 0.0029602050781250,
+0.0015563964843750, 0.0032043457031250, 0.0017395019531250, 0.0030822753906250,
+0.0012817382812500, 0.0022583007812500, 0.0016784667968750, 0.0023193359375000,
+0.0013122558593750, 0.0020141601562500, 0.0007629394531250, 0.0012512207031250,
+0.0014038085937500},
+{0.0027770996093750, 0.0022277832031250, 0.0025939941406250, 0.0018005371093750,
+0.0032958984375000, 0.0028076171875000, 0.0032653808593750, 0.0026550292968750,
+0.0032653808593750, 0.0029602050781250, 0.0036621093750000, 0.0038146972656250,
+0.0034179687500000, 0.0034484863281250, 0.0033569335937500, 0.0042419433593750,
+0.0034179687500000, 0.0043640136718750, 0.0031433105468750, 0.0043640136718750,
+0.0029296875000000, 0.0047302246093750, 0.0030212402343750, 0.0045471191406250,
+0.0026550292968750, 0.0044250488281250, 0.0025634765625000, 0.0046081542968750,
+0.0025634765625000, 0.0042114257812500, 0.0023803710937500, 0.0037841796875000,
+0.0023803710937500, 0.0040283203125000, 0.0025024414062500, 0.0032348632812500,
+0.0023803710937500, 0.0031433105468750, 0.0023498535156250, 0.0030517578125000,
+0.0024414062500000, 0.0027465820312500, 0.0022888183593750, 0.0028076171875000,
+0.0020141601562500, 0.0025024414062500, 0.0021362304687500, 0.0025939941406250,
+0.0017089843750000, 0.0024414062500000, 0.0012817382812500, 0.0021057128906250,
+0.0016174316406250, 0.0022888183593750, 0.0005798339843750, 0.0018615722656250,
+0.0005493164062500, 0.0016784667968750, 0.0006408691406250, 0.0019836425781250,
+0.0000000000000000, 0.0022583007812500, 0.0006713867187500, 0.0026550292968750,
+0.0019836425781250, 0.0019226074218750, 0.0015258789062500, 0.0017700195312500,
+0.0019226074218750, 0.0020141601562500, 0.0026245117187500, 0.0020141601562500,
+0.0022583007812500, 0.0019226074218750, 0.0031127929687500, 0.0025939941406250,
+0.0028991699218750, 0.0025634765625000, 0.0031738281250000, 0.0021667480468750,
+0.0038146972656250, 0.0022888183593750, 0.0032958984375000, 0.0024414062500000,
+0.0042419433593750, 0.0018005371093750, 0.0040588378906250, 0.0016174316406250,
+0.0042419433593750, 0.0019531250000000, 0.0047912597656250, 0.0020141601562500,
+0.0043334960937500, 0.0018005371093750, 0.0054626464843750, 0.0026550292968750,
+0.0050354003906250, 0.0025329589843750, 0.0053405761718750, 0.0027160644531250,
+0.0063476562500000, 0.0034484863281250, 0.0055847167968750, 0.0032043457031250,
+0.0065002441406250, 0.0033569335937500, 0.0065917968750000, 0.0035705566406250,
+0.0059509277343750, 0.0034790039062500, 0.0060424804687500, 0.0033264160156250,
+0.0062255859375000, 0.0033264160156250, 0.0047912597656250, 0.0032043457031250,
+0.0050964355468750, 0.0031127929687500, 0.0043640136718750, 0.0031433105468750,
+0.0030822753906250, 0.0031127929687500, 0.0037231445312500, 0.0030212402343750,
+0.0021057128906250, 0.0034179687500000, 0.0019836425781250, 0.0033264160156250,
+0.0019226074218750, 0.0035400390625000, 0.0011901855468750, 0.0040283203125000,
+0.0014038085937500, 0.0038146972656250, 0.0008850097656250, 0.0038146972656250,
+0.0009155273437500, 0.0039672851562500, 0.0006713867187500, 0.0034790039062500,
+0.0002746582031250, 0.0031127929687500, 0.0004882812500000, 0.0032958984375000,
+-0.0003356933593750, 0.0025329589843750, -0.0001525878906250, 0.0022583007812500,
+-0.0004577636718750, 0.0025634765625000, -0.0011291503906250, 0.0024719238281250,
+-0.0006713867187500, 0.0021667480468750, -0.0014038085937500, 0.0031738281250000,
+-0.0013732910156250, 0.0028381347656250, -0.0012817382812500, 0.0028991699218750,
+-0.0015869140625000, 0.0035705566406250, -0.0013427734375000, 0.0030517578125000,
+-0.0014648437500000, 0.0032043457031250, -0.0014648437500000, 0.0032653808593750,
+-0.0014953613281250, 0.0028076171875000, -0.0015258789062500, 0.0024414062500000,
+-0.0014038085937500, 0.0025329589843750, -0.0020141601562500, 0.0021667480468750,
+-0.0017700195312500, 0.0018920898437500, -0.0021362304687500, 0.0020446777343750,
+-0.0027465820312500, 0.0019836425781250, -0.0023803710937500, 0.0018005371093750,
+-0.0032653808593750, 0.0025634765625000, -0.0034179687500000, 0.0023193359375000,
+-0.0031433105468750, 0.0025939941406250, -0.0034484863281250, 0.0033264160156250,
+-0.0036010742187500, 0.0030212402343750, -0.0025939941406250, 0.0033569335937500,
+-0.0031127929687500, 0.0036010742187500, -0.0024108886718750, 0.0034179687500000,
+-0.0013732910156250, 0.0033874511718750, -0.0021057128906250, 0.0035400390625000,
+-0.0003051757812500, 0.0031127929687500, -0.0005493164062500, 0.0032043457031250,
+-0.0003356933593750, 0.0031127929687500, 0.0009155273437500, 0.0027770996093750,
+0.0003662109375000, 0.0031738281250000, 0.0007324218750000, 0.0027770996093750,
+0.0011596679687500, 0.0028076171875000, 0.0006103515625000, 0.0030212402343750,
+0.0004272460937500, 0.0028991699218750, 0.0007629394531250, 0.0030822753906250,
+-0.0005798339843750, 0.0032958984375000, -0.0002746582031250, 0.0032958984375000,
+-0.0006103515625000, 0.0035095214843750, -0.0016784667968750, 0.0036621093750000,
+-0.0011291503906250, 0.0035705566406250, -0.0018310546875000, 0.0038146972656250,
+-0.0021057128906250, 0.0037536621093750, -0.0017089843750000, 0.0037536621093750,
+-0.0017395019531250, 0.0037841796875000, -0.0019531250000000, 0.0036621093750000,
+-0.0006408691406250, 0.0035095214843750, -0.0009765625000000, 0.0034484863281250,
+-0.0003662109375000, 0.0032653808593750, 0.0009155273437500, 0.0030822753906250,
+0.0002746582031250, 0.0030212402343750, 0.0018920898437500, 0.0027160644531250,
+0.0018615722656250, 0.0027770996093750, 0.0021667480468750, 0.0026245117187500,
+0.0031738281250000, 0.0023803710937500, 0.0026855468750000, 0.0025634765625000,
+0.0035400390625000, 0.0022888183593750, 0.0036315917968750, 0.0023803710937500,
+0.0036010742187500, 0.0021667480468750, 0.0040893554687500, 0.0019531250000000,
+0.0040283203125000, 0.0021667480468750, 0.0039672851562500, 0.0017089843750000,
+0.0041198730468750, 0.0016784667968750, 0.0038452148437500, 0.0015869140625000,
+0.0036315917968750, 0.0012817382812500, 0.0038146972656250, 0.0013732910156250,
+0.0031433105468750, 0.0013122558593750, 0.0031433105468750, 0.0012207031250000,
+0.0029907226562500, 0.0012817382812500, 0.0024414062500000, 0.0013122558593750,
+0.0025024414062500, 0.0012817382812500, 0.0021972656250000, 0.0014648437500000,
+0.0019836425781250, 0.0014648437500000, 0.0021057128906250, 0.0015258789062500,
+0.0020446777343750, 0.0016784667968750, 0.0018615722656250, 0.0016174316406250,
+0.0023498535156250, 0.0018920898437500, 0.0021667480468750, 0.0018310546875000,
+0.0024108886718750, 0.0018615722656250, 0.0028991699218750, 0.0020751953125000,
+0.0025939941406250, 0.0020141601562500, 0.0031738281250000, 0.0021972656250000,
+0.0031433105468750, 0.0021972656250000, 0.0031127929687500, 0.0022583007812500,
+0.0034179687500000, 0.0024414062500000, 0.0032653808593750, 0.0023803710937500,
+0.0031738281250000, 0.0025024414062500, 0.0032653808593750, 0.0025329589843750,
+0.0029907226562500, 0.0024719238281250, 0.0027770996093750, 0.0024719238281250,
+0.0028686523437500, 0.0025024414062500, 0.0023498535156250, 0.0024719238281250,
+0.0024414062500000, 0.0025024414062500, 0.0021362304687500, 0.0024719238281250,
+0.0017089843750000, 0.0024108886718750, 0.0019836425781250, 0.0024414062500000,
+0.0012207031250000, 0.0024108886718750, 0.0013427734375000, 0.0024108886718750,
+0.0010986328125000, 0.0023498535156250, 0.0005187988281250, 0.0022888183593750,
+0.0007934570312500, 0.0024108886718750, 0.0003356933593750, 0.0020446777343750,
+0.0002136230468750, 0.0021362304687500, 0.0004272460937500, 0.0020446777343750,
+0.0002746582031250, 0.0017395019531250, 0.0002441406250000, 0.0019531250000000,
+0.0008850097656250, 0.0016784667968750, 0.0006408691406250, 0.0017089843750000,
+0.0010375976562500, 0.0016784667968750, 0.0017395019531250, 0.0015258789062500,
+0.0012512207031250, 0.0016784667968750, 0.0021057128906250, 0.0016784667968750,
+0.0019531250000000, 0.0016479492187500, 0.0020446777343750, 0.0017395019531250,
+0.0025024414062500, 0.0018005371093750, 0.0021667480468750, 0.0018005371093750,
+0.0027160644531250, 0.0018920898437500, 0.0025634765625000, 0.0018920898437500,
+0.0027160644531250, 0.0019531250000000, 0.0031433105468750, 0.0019531250000000,
+0.0028686523437500, 0.0018615722656250, 0.0032348632812500, 0.0018615722656250,
+0.0032653808593750, 0.0018005371093750, 0.0032348632812500, 0.0018310546875000,
+0.0034179687500000, 0.0018920898437500, 0.0033569335937500, 0.0018310546875000,
+0.0036621093750000, 0.0019531250000000, 0.0034790039062500, 0.0018920898437500,
+0.0036621093750000, 0.0018615722656250, 0.0040893554687500, 0.0019226074218750,
+0.0038452148437500, 0.0018310546875000, 0.0038146972656250, 0.0019531250000000,
+0.0040893554687500, 0.0018005371093750, 0.0036621093750000, 0.0019531250000000,
+0.0031738281250000, 0.0021972656250000, 0.0033264160156250, 0.0021057128906250,
+0.0026855468750000, 0.0027770996093750, 0.0028076171875000, 0.0025024414062500,
+0.0028686523437500, 0.0025024414062500, 0.0024719238281250, 0.0029907226562500,
+0.0026550292968750, 0.0027160644531250, 0.0024414062500000, 0.0028686523437500,
+0.0025329589843750, 0.0028686523437500, 0.0025024414062500, 0.0027770996093750,
+0.0023803710937500, 0.0026855468750000, 0.0025939941406250, 0.0025939941406250,
+0.0023193359375000, 0.0026855468750000, 0.0020141601562500, 0.0024719238281250,
+0.0024108886718750, 0.0024719238281250, 0.0025634765625000, 0.0026245117187500,
+0.0020141601562500, 0.0023193359375000, 0.0029907226562500, 0.0026855468750000,
+0.0026855468750000, 0.0025024414062500, 0.0025024414062500, 0.0023803710937500,
+0.0030517578125000, 0.0025024414062500, 0.0027160644531250, 0.0022888183593750,
+0.0028076171875000, 0.0020751953125000, 0.0028076171875000, 0.0020446777343750,
+0.0028381347656250, 0.0017700195312500, 0.0028991699218750, 0.0014648437500000,
+0.0028686523437500, 0.0014648437500000, 0.0032958984375000, 0.0010070800781250,
+0.0030822753906250, 0.0008239746093750, 0.0032348632812500, 0.0007324218750000,
+0.0038146972656250, 0.0004882812500000, 0.0036621093750000, 0.0003967285156250,
+0.0033874511718750, 0.0004882812500000, 0.0037841796875000, 0.0002746582031250,
+0.0033264160156250, 0.0003967285156250, 0.0028076171875000, 0.0006103515625000,
+0.0033569335937500, 0.0003967285156250, 0.0022583007812500, 0.0006408691406250,
+0.0024108886718750, 0.0007324218750000, 0.0023498535156250, 0.0006408691406250,
+0.0017089843750000, 0.0007324218750000, 0.0020751953125000, 0.0008850097656250,
+0.0020141601562500, 0.0006408691406250, 0.0018310546875000, 0.0007934570312500,
+0.0021362304687500, 0.0007629394531250, 0.0024414062500000, 0.0005798339843750,
+0.0023193359375000, 0.0007934570312500, 0.0024414062500000, 0.0006713867187500,
+0.0025939941406250, 0.0007324218750000, 0.0025329589843750, 0.0007324218750000,
+0.0024414062500000, 0.0006713867187500, 0.0024719238281250, 0.0008544921875000,
+0.0023193359375000, 0.0005798339843750, 0.0022888183593750, 0.0007324218750000,
+0.0021667480468750, 0.0007324218750000, 0.0019531250000000, 0.0006103515625000,
+0.0019531250000000, 0.0006408691406250, 0.0017089843750000, 0.0007019042968750,
+0.0015869140625000, 0.0006713867187500, 0.0015563964843750, 0.0008239746093750,
+0.0014343261718750, 0.0009460449218750, 0.0012512207031250, 0.0009765625000000,
+0.0012207031250000, 0.0011291503906250, 0.0011291503906250, 0.0012207031250000,
+0.0010070800781250, 0.0012512207031250, 0.0009155273437500, 0.0013732910156250,
+0.0007629394531250, 0.0014648437500000, 0.0006713867187500, 0.0011901855468750,
+0.0004882812500000},
+{0.0023803710937500, 0.0019226074218750, 0.0022583007812500, 0.0018005371093750,
+0.0022583007812500, 0.0018615722656250, 0.0021972656250000, 0.0018920898437500,
+0.0021057128906250, 0.0017395019531250, 0.0021362304687500, 0.0019226074218750,
+0.0019531250000000, 0.0017700195312500, 0.0020751953125000, 0.0018310546875000,
+0.0021362304687500, 0.0019836425781250, 0.0019226074218750, 0.0018310546875000,
+0.0025024414062500, 0.0021362304687500, 0.0023498535156250, 0.0020751953125000,
+0.0023193359375000, 0.0019836425781250, 0.0027770996093750, 0.0020446777343750,
+0.0028686523437500, 0.0019836425781250, 0.0028381347656250, 0.0018005371093750,
+0.0028686523437500, 0.0018005371093750, 0.0028076171875000, 0.0017395019531250,
+0.0025939941406250, 0.0015258789062500, 0.0025939941406250, 0.0015869140625000,
+0.0026245117187500, 0.0014343261718750, 0.0026245117187500, 0.0013732910156250,
+0.0024719238281250, 0.0013732910156250, 0.0025634765625000, 0.0013427734375000,
+0.0027465820312500, 0.0013732910156250, 0.0019836425781250, 0.0014038085937500,
+0.0024719238281250, 0.0013732910156250, 0.0024108886718750, 0.0013427734375000,
+0.0016479492187500, 0.0012512207031250, 0.0019836425781250, 0.0011901855468750,
+0.0022583007812500, 0.0010375976562500, 0.0020141601562500, 0.0010986328125000,
+0.0023193359375000, 0.0010070800781250, 0.0027465820312500, 0.0007629394531250,
+0.0023498535156250, 0.0007934570312500, 0.0027465820312500, 0.0007629394531250,
+0.0028991699218750, 0.0006408691406250, 0.0025634765625000, 0.0007934570312500,
+0.0025024414062500, 0.0009155273437500, 0.0025634765625000, 0.0006713867187500,
+0.0022583007812500, 0.0013427734375000, 0.0022277832031250, 0.0011596679687500,
+0.0022888183593750, 0.0013427734375000, 0.0021057128906250, 0.0019226074218750,
+0.0020446777343750, 0.0015563964843750, 0.0025024414062500, 0.0020751953125000,
+0.0022277832031250, 0.0022277832031250, 0.0023498535156250, 0.0021362304687500,
+0.0028381347656250, 0.0023193359375000, 0.0024719238281250, 0.0023498535156250,
+0.0025329589843750, 0.0025024414062500, 0.0025939941406250, 0.0025024414062500,
+0.0022583007812500, 0.0025634765625000, 0.0021057128906250, 0.0026550292968750,
+0.0021972656250000, 0.0026550292968750, 0.0012817382812500, 0.0028381347656250,
+0.0014038085937500, 0.0029602050781250, 0.0013427734375000, 0.0028991699218750,
+0.0006713867187500, 0.0030212402343750, 0.0008850097656250, 0.0030517578125000,
+0.0010681152343750, 0.0028991699218750, 0.0008850097656250, 0.0029907226562500,
+0.0012817382812500, 0.0029602050781250, 0.0017089843750000, 0.0027465820312500,
+0.0014953613281250, 0.0028381347656250, 0.0020446777343750, 0.0030517578125000,
+0.0021667480468750, 0.0029907226562500, 0.0021362304687500, 0.0033874511718750,
+0.0023193359375000, 0.0037841796875000, 0.0022888183593750, 0.0036010742187500,
+0.0020141601562500, 0.0042419433593750, 0.0021057128906250, 0.0043945312500000,
+0.0019531250000000, 0.0043029785156250, 0.0016174316406250, 0.0046081542968750,
+0.0016479492187500, 0.0047302246093750, 0.0015258789062500, 0.0041198730468750,
+0.0012207031250000, 0.0044860839843750, 0.0013427734375000, 0.0040893554687500,
+0.0014038085937500, 0.0033569335937500, 0.0010681152343750, 0.0039062500000000,
+0.0013732910156250, 0.0025939941406250, 0.0013122558593750, 0.0028076171875000,
+0.0012207031250000, 0.0025024414062500, 0.0012817382812500, 0.0015258789062500,
+0.0010681152343750, 0.0020751953125000, 0.0012817382812500, 0.0011291503906250,
+0.0010375976562500, 0.0012207031250000, 0.0011291503906250, 0.0010986328125000,
+0.0012512207031250, 0.0005493164062500, 0.0008850097656250, 0.0008239746093750,
+0.0016784667968750, 0.0001525878906250, 0.0013732910156250, 0.0003662109375000,
+0.0015258789062500, 0.0001525878906250, 0.0021362304687500, -0.0003662109375000,
+0.0016174316406250, 0.0000305175781250, 0.0024414062500000, -0.0005798339843750,
+0.0021972656250000, -0.0005493164062500, 0.0021667480468750, -0.0003051757812500,
+0.0026855468750000, -0.0005187988281250, 0.0021972656250000, -0.0003662109375000,
+0.0025939941406250, 0.0002136230468750, 0.0025024414062500, 0.0000610351562500,
+0.0024719238281250, 0.0004882812500000, 0.0026550292968750, 0.0010986328125000,
+0.0023803710937500, 0.0007324218750000, 0.0025634765625000, 0.0016784667968750,
+0.0024719238281250, 0.0015869140625000, 0.0025024414062500, 0.0018005371093750,
+0.0026855468750000, 0.0023498535156250, 0.0024719238281250, 0.0020751953125000,
+0.0026550292968750, 0.0026550292968750, 0.0027160644531250, 0.0025634765625000,
+0.0025634765625000, 0.0026855468750000, 0.0025634765625000, 0.0031127929687500,
+0.0026550292968750, 0.0029296875000000, 0.0023498535156250, 0.0033569335937500,
+0.0024414062500000, 0.0033264160156250, 0.0022888183593750, 0.0034790039062500,
+0.0019836425781250, 0.0037536621093750, 0.0022277832031250, 0.0036010742187500,
+0.0016784667968750, 0.0039367675781250, 0.0017395019531250, 0.0039367675781250,
+0.0015869140625000, 0.0039062500000000, 0.0012817382812500, 0.0041198730468750,
+0.0014343261718750, 0.0040893554687500, 0.0011291503906250, 0.0038757324218750,
+0.0011291503906250, 0.0039367675781250, 0.0011291503906250, 0.0038452148437500,
+0.0009460449218750, 0.0036621093750000, 0.0010375976562500, 0.0036926269531250,
+0.0008850097656250, 0.0035095214843750, 0.0009765625000000, 0.0035095214843750,
+0.0009765625000000, 0.0033569335937500, 0.0009155273437500, 0.0031738281250000,
+0.0010070800781250, 0.0032043457031250, 0.0009460449218750, 0.0028686523437500,
+0.0009155273437500, 0.0028381347656250, 0.0010375976562500, 0.0027160644531250,
+0.0010375976562500, 0.0025634765625000, 0.0010070800781250, 0.0025939941406250,
+0.0011596679687500, 0.0023803710937500, 0.0010986328125000, 0.0023803710937500,
+0.0010986328125000, 0.0024108886718750, 0.0011291503906250, 0.0023498535156250,
+0.0010681152343750, 0.0022888183593750, 0.0010986328125000, 0.0027160644531250,
+0.0009765625000000, 0.0025329589843750, 0.0009460449218750, 0.0028381347656250,
+0.0009155273437500, 0.0033569335937500, 0.0007934570312500, 0.0030517578125000,
+0.0011901855468750, 0.0037536621093750, 0.0009460449218750, 0.0038146972656250,
+0.0013122558593750, 0.0036010742187500, 0.0018005371093750, 0.0038757324218750,
+0.0014648437500000, 0.0038757324218750, 0.0023193359375000, 0.0032043457031250,
+0.0023803710937500, 0.0033874511718750, 0.0024108886718750, 0.0030517578125000,
+0.0028686523437500, 0.0022277832031250, 0.0027160644531250, 0.0025329589843750,
+0.0028381347656250, 0.0021972656250000, 0.0029602050781250, 0.0019226074218750,
+0.0028686523437500, 0.0021667480468750, 0.0027160644531250, 0.0021667480468750,
+0.0028686523437500, 0.0020446777343750, 0.0027465820312500, 0.0024108886718750,
+0.0026245117187500, 0.0022888183593750, 0.0027160644531250, 0.0022583007812500,
+0.0027160644531250, 0.0025634765625000, 0.0025634765625000, 0.0024414062500000,
+0.0028076171875000, 0.0022888183593750, 0.0027465820312500, 0.0023498535156250,
+0.0026550292968750, 0.0021057128906250, 0.0028076171875000, 0.0018310546875000,
+0.0026550292968750, 0.0019836425781250, 0.0023193359375000, 0.0018615722656250,
+0.0024108886718750, 0.0018310546875000, 0.0020751953125000, 0.0018310546875000,
+0.0016174316406250, 0.0018310546875000, 0.0018005371093750, 0.0018310546875000,
+0.0011901855468750, 0.0021057128906250, 0.0010070800781250, 0.0020751953125000,
+0.0010375976562500, 0.0021362304687500, 0.0007934570312500, 0.0023498535156250,
+0.0008239746093750, 0.0023193359375000, 0.0013122558593750, 0.0024108886718750,
+0.0010681152343750, 0.0023803710937500, 0.0013427734375000, 0.0025024414062500,
+0.0018920898437500, 0.0025634765625000, 0.0014953613281250, 0.0024719238281250,
+0.0021667480468750, 0.0028686523437500, 0.0022277832031250, 0.0027770996093750,
+0.0021362304687500, 0.0028381347656250, 0.0025329589843750, 0.0033569335937500,
+0.0033874511718750, 0.0023803710937500, 0.0038452148437500, 0.0030517578125000,
+0.0036926269531250, 0.0024414062500000, 0.0035095214843750, 0.0028381347656250,
+0.0036926269531250, 0.0034790039062500, 0.0036010742187500, 0.0030517578125000,
+0.0032958984375000, 0.0033264160156250, 0.0034179687500000, 0.0035095214843750,
+0.0031127929687500, 0.0030517578125000, 0.0028381347656250, 0.0027160644531250,
+0.0031127929687500, 0.0028686523437500, 0.0020141601562500, 0.0022888183593750,
+0.0025024414062500, 0.0022277832031250, 0.0017700195312500, 0.0021057128906250,
+0.0006103515625000, 0.0017395019531250, 0.0015563964843750, 0.0019531250000000,
+-0.0002441406250000, 0.0015258789062500, -0.0001831054687500, 0.0016784667968750,
+-0.0001525878906250, 0.0014648437500000, -0.0010681152343750, 0.0010070800781250,
+-0.0006408691406250, 0.0014343261718750, -0.0004577636718750, 0.0011596679687500,
+-0.0006713867187500, 0.0007934570312500, -0.0002746582031250, 0.0014343261718750,
+0.0003051757812500, 0.0018920898437500, 0.0002136230468750, 0.0014343261718750,
+0.0004882812500000, 0.0029602050781250, 0.0007629394531250, 0.0027770996093750,
+0.0007629394531250, 0.0033264160156250, 0.0007934570312500, 0.0046081542968750,
+0.0010375976562500, 0.0040283203125000, 0.0007324218750000, 0.0047302246093750,
+0.0008544921875000, 0.0050659179687500, 0.0009765625000000, 0.0046386718750000,
+0.0007324218750000, 0.0045471191406250, 0.0008239746093750, 0.0046997070312500,
+0.0014648437500000, 0.0039062500000000, 0.0012817382812500, 0.0040588378906250,
+0.0015869140625000, 0.0039978027343750, 0.0021667480468750, 0.0033874511718750,
+0.0018005371093750, 0.0035400390625000, 0.0026245117187500, 0.0039062500000000,
+0.0025329589843750, 0.0035705566406250, 0.0025939941406250, 0.0037841796875000,
+0.0029907226562500, 0.0043334960937500, 0.0027160644531250, 0.0041503906250000,
+0.0028991699218750, 0.0042724609375000, 0.0028076171875000, 0.0045166015625000,
+0.0028381347656250, 0.0039672851562500, 0.0028686523437500, 0.0036621093750000,
+0.0025634765625000, 0.0040588378906250, 0.0031738281250000, 0.0029907226562500,
+0.0028076171875000, 0.0028381347656250, 0.0033264160156250, 0.0028381347656250,
+0.0039978027343750, 0.0022888183593750, 0.0033874511718750, 0.0021667480468750,
+0.0053405761718750, 0.0023803710937500, 0.0048522949218750, 0.0019836425781250,
+0.0054321289062500, 0.0022888183593750, 0.0070800781250000, 0.0026550292968750,
+0.0059814453125000, 0.0021362304687500, 0.0073242187500000, 0.0028076171875000,
+0.0075683593750000, 0.0027465820312500, 0.0070190429687500, 0.0024108886718750,
+0.0073242187500000, 0.0025939941406250, 0.0075073242187500, 0.0025024414062500,
+0.0060729980468750, 0.0021362304687500, 0.0064392089843750, 0.0021667480468750,
+0.0058593750000000, 0.0020751953125000, 0.0045776367187500, 0.0017395019531250,
+0.0054016113281250, 0.0016784667968750, 0.0042724609375000, 0.0020751953125000,
+0.0040588378906250, 0.0018005371093750, 0.0045471191406250, 0.0021362304687500,
+0.0042724609375000, 0.0026245117187500, 0.0040283203125000, 0.0022277832031250,
+0.0050354003906250, 0.0030822753906250, 0.0046386718750000, 0.0029296875000000,
+0.0047607421875000, 0.0029296875000000, 0.0054931640625000, 0.0034484863281250,
+0.0049133300781250, 0.0032958984375000, 0.0054931640625000, 0.0031738281250000,
+0.0055236816406250, 0.0034179687500000, 0.0051574707031250, 0.0032348632812500,
+0.0052795410156250},
+{0.0031738281250000, 0.0034179687500000, 0.0026245117187500, 0.0027465820312500,
+0.0028991699218750, 0.0030822753906250, 0.0018005371093750, 0.0032348632812500,
+0.0018920898437500, 0.0029907226562500, 0.0014953613281250, 0.0032958984375000,
+0.0005493164062500, 0.0037536621093750, 0.0010070800781250, 0.0034790039062500,
+-0.0003051757812500, 0.0036010742187500, -0.0003051757812500, 0.0038452148437500,
+-0.0003051757812500, 0.0034790039062500, -0.0010375976562500, 0.0031433105468750,
+-0.0007629394531250, 0.0033264160156250, -0.0007019042968750, 0.0029602050781250,
+-0.0009155273437500, 0.0028381347656250, -0.0006713867187500, 0.0028381347656250,
+-0.0003967285156250, 0.0027770996093750, -0.0004577636718750, 0.0028381347656250,
+-0.0001831054687500, 0.0024108886718750, -0.0001525878906250, 0.0025939941406250,
+0.0000000000000000, 0.0023803710937500, 0.0001220703125000, 0.0018920898437500,
+0.0001220703125000, 0.0022277832031250, 0.0007629394531250, 0.0018615722656250,
+0.0006713867187500, 0.0017089843750000, 0.0010986328125000, 0.0021057128906250,
+0.0017089843750000, 0.0019531250000000, 0.0014648437500000, 0.0017089843750000,
+0.0026855468750000, 0.0029296875000000, 0.0026245117187500, 0.0025939941406250,
+0.0029296875000000, 0.0029602050781250, 0.0038146972656250, 0.0040893554687500,
+0.0034179687500000, 0.0035705566406250, 0.0041198730468750, 0.0043029785156250,
+0.0042419433593750, 0.0046081542968750, 0.0041503906250000, 0.0043334960937500,
+0.0043945312500000, 0.0043945312500000, 0.0043029785156250, 0.0046997070312500,
+0.0042724609375000, 0.0038452148437500, 0.0041809082031250, 0.0038757324218750,
+0.0041809082031250, 0.0037231445312500, 0.0040283203125000, 0.0032348632812500,
+0.0039978027343750, 0.0035705566406250, 0.0044555664062500, 0.0034179687500000,
+0.0040893554687500, 0.0032958984375000, 0.0045776367187500, 0.0038146972656250,
+0.0052490234375000, 0.0041503906250000, 0.0047302246093750, 0.0038757324218750,
+0.0060729980468750, 0.0045471191406250, 0.0058898925781250, 0.0046997070312500,
+0.0060424804687500, 0.0045471191406250, 0.0071105957031250, 0.0046386718750000,
+0.0066223144531250, 0.0045166015625000, 0.0069580078125000, 0.0043945312500000,
+0.0073547363281250, 0.0042419433593750, 0.0066833496093750, 0.0040283203125000,
+0.0062561035156250, 0.0037841796875000, 0.0067749023437500, 0.0036926269531250,
+0.0050048828125000, 0.0036926269531250, 0.0052490234375000, 0.0034484863281250,
+0.0050964355468750, 0.0035095214843750, 0.0039062500000000, 0.0036926269531250,
+0.0044860839843750, 0.0034790039062500, 0.0043334960937500, 0.0036926269531250,
+0.0039062500000000, 0.0036621093750000, 0.0045471191406250, 0.0035705566406250,
+0.0049743652343750, 0.0036010742187500, 0.0045166015625000, 0.0035705566406250,
+0.0058593750000000, 0.0031433105468750, 0.0057983398437500, 0.0032653808593750,
+0.0058593750000000, 0.0029602050781250, 0.0067138671875000, 0.0025939941406250,
+0.0064697265625000, 0.0029296875000000, 0.0065307617187500, 0.0023803710937500,
+0.0068054199218750, 0.0024108886718750, 0.0065002441406250, 0.0024719238281250,
+0.0062255859375000, 0.0022888183593750, 0.0065612792968750, 0.0024414062500000,
+0.0058593750000000, 0.0025329589843750, 0.0060424804687500, 0.0025939941406250,
+0.0058898925781250, 0.0027770996093750, 0.0053710937500000, 0.0029296875000000,
+0.0056457519531250, 0.0029602050781250, 0.0053100585937500, 0.0032958984375000,
+0.0052795410156250, 0.0032653808593750, 0.0052490234375000, 0.0033264160156250,
+0.0051574707031250, 0.0035400390625000, 0.0052185058593750, 0.0034790039062500,
+0.0046386718750000, 0.0036926269531250, 0.0048828125000000, 0.0037231445312500,
+0.0043640136718750, 0.0037536621093750, 0.0036926269531250, 0.0039062500000000,
+0.0041198730468750, 0.0039062500000000, 0.0025024414062500, 0.0039672851562500,
+0.0028686523437500, 0.0040588378906250, 0.0021667480468750, 0.0040893554687500,
+0.0008239746093750, 0.0040283203125000, 0.0016784667968750, 0.0040893554687500,
+-0.0002136230468750, 0.0040588378906250, 0.0000915527343750, 0.0041503906250000,
+-0.0003356933593750, 0.0040588378906250, -0.0016174316406250, 0.0039062500000000,
+-0.0008544921875000, 0.0040588378906250, -0.0021057128906250, 0.0038757324218750,
+-0.0021057128906250, 0.0037841796875000, -0.0020446777343750, 0.0037536621093750,
+-0.0027465820312500, 0.0036010742187500, -0.0024414062500000, 0.0035705566406250,
+-0.0025329589843750, 0.0036621093750000, -0.0027160644531250, 0.0035400390625000,
+-0.0023803710937500, 0.0035705566406250, -0.0021972656250000, 0.0036926269531250,
+-0.0022277832031250, 0.0036010742187500, -0.0014953613281250, 0.0036315917968750,
+-0.0016479492187500, 0.0036315917968750, -0.0012817382812500, 0.0035705566406250,
+-0.0005493164062500, 0.0035705566406250, -0.0008850097656250, 0.0033874511718750,
+0.0000915527343750, 0.0034484863281250, 0.0000610351562500, 0.0033264160156250,
+0.0002136230468750, 0.0032653808593750, 0.0008239746093750, 0.0032653808593750,
+0.0006408691406250, 0.0031433105468750, 0.0008239746093750, 0.0031127929687500,
+0.0009765625000000, 0.0030517578125000, 0.0008239746093750, 0.0029602050781250,
+0.0007629394531250, 0.0030212402343750, 0.0008850097656250, 0.0028991699218750,
+0.0005187988281250, 0.0028686523437500, 0.0005493164062500, 0.0028686523437500,
+0.0006103515625000, 0.0028076171875000, 0.0004882812500000, 0.0027770996093750,
+0.0004577636718750, 0.0028381347656250, 0.0009460449218750, 0.0026550292968750,
+0.0007019042968750, 0.0026550292968750, 0.0010375976562500, 0.0026550292968750,
+0.0016174316406250, 0.0025329589843750, 0.0012207031250000, 0.0025634765625000,
+0.0019836425781250, 0.0025634765625000, 0.0019531250000000, 0.0025024414062500,
+0.0019836425781250, 0.0025329589843750, 0.0024108886718750, 0.0025634765625000,
+0.0022888183593750, 0.0024108886718750, 0.0023803710937500, 0.0024719238281250,
+0.0024414062500000, 0.0023803710937500, 0.0025024414062500, 0.0023803710937500,
+0.0025024414062500, 0.0024414062500000, 0.0024108886718750, 0.0022888183593750,
+0.0027465820312500, 0.0023803710937500, 0.0026855468750000, 0.0023193359375000,
+0.0026855468750000, 0.0022888183593750, 0.0029602050781250, 0.0023498535156250,
+0.0029296875000000, 0.0022277832031250, 0.0027160644531250, 0.0023498535156250,
+0.0029296875000000, 0.0023193359375000, 0.0026855468750000, 0.0023193359375000,
+0.0022888183593750, 0.0023803710937500, 0.0026550292968750, 0.0022277832031250,
+0.0021362304687500, 0.0024414062500000, 0.0020141601562500, 0.0023803710937500,
+0.0022583007812500, 0.0024108886718750, 0.0021057128906250, 0.0025634765625000,
+0.0020141601562500, 0.0024108886718750, 0.0028076171875000, 0.0025634765625000,
+0.0025329589843750, 0.0025939941406250, 0.0027770996093750, 0.0023803710937500,
+0.0036010742187500, 0.0024108886718750, 0.0032958984375000, 0.0025634765625000,
+0.0034790039062500, 0.0020141601562500, 0.0039062500000000, 0.0022583007812500,
+0.0034484863281250, 0.0019836425781250, 0.0031738281250000, 0.0014343261718750,
+0.0037231445312500, 0.0017700195312500, 0.0025329589843750, 0.0013122558593750,
+0.0027160644531250, 0.0012512207031250, 0.0026550292968750, 0.0012512207031250,
+0.0015563964843750, 0.0011291503906250, 0.0013122558593750, 0.0013122558593750,
+0.0010375976562500, 0.0010070800781250, 0.0009765625000000, 0.0010681152343750,
+0.0008850097656250, 0.0010681152343750, 0.0006408691406250, 0.0007934570312500,
+0.0004577636718750, 0.0009460449218750, 0.0003967285156250, 0.0009460449218750,
+0.0000610351562500, 0.0008239746093750, 0.0004577636718750, 0.0010681152343750,
+0.0008239746093750, 0.0013122558593750, 0.0002441406250000, 0.0011901855468750,
+0.0016784667968750, 0.0018310546875000, 0.0015258789062500, 0.0018005371093750,
+0.0017089843750000, 0.0020751953125000, 0.0028686523437500, 0.0025634765625000,
+0.0026855468750000, 0.0023498535156250, 0.0027465820312500, 0.0030822753906250,
+0.0032653808593750, 0.0031433105468750, 0.0031127929687500, 0.0032653808593750,
+0.0028381347656250, 0.0036315917968750, 0.0032653808593750, 0.0035400390625000,
+0.0027465820312500, 0.0036926269531250, 0.0028076171875000, 0.0038146972656250,
+0.0028076171875000, 0.0038146972656250, 0.0025634765625000, 0.0038452148437500,
+0.0027465820312500, 0.0039062500000000, 0.0026550292968750, 0.0039978027343750,
+0.0026855468750000, 0.0039978027343750, 0.0028076171875000, 0.0041198730468750,
+0.0029296875000000, 0.0042114257812500, 0.0030212402343750, 0.0041503906250000,
+0.0032653808593750, 0.0045776367187500, 0.0033264160156250, 0.0044860839843750,
+0.0035095214843750, 0.0044860839843750, 0.0037841796875000, 0.0047912597656250,
+0.0038452148437500, 0.0047607421875000, 0.0039672851562500, 0.0046081542968750,
+0.0041503906250000, 0.0046386718750000, 0.0041198730468750, 0.0045776367187500,
+0.0040283203125000, 0.0044250488281250, 0.0042724609375000, 0.0044860839843750,
+0.0041809082031250, 0.0045166015625000, 0.0041503906250000, 0.0043945312500000,
+0.0042724609375000, 0.0046081542968750, 0.0044555664062500, 0.0047912597656250,
+0.0044250488281250, 0.0047607421875000, 0.0043640136718750, 0.0049438476562500,
+0.0046386718750000, 0.0050354003906250, 0.0043640136718750, 0.0048522949218750,
+0.0041198730468750, 0.0048522949218750, 0.0045471191406250, 0.0050354003906250,
+0.0034484863281250, 0.0042419433593750, 0.0037841796875000, 0.0043945312500000,
+0.0035400390625000, 0.0041503906250000, 0.0026245117187500, 0.0034179687500000,
+0.0032958984375000, 0.0037536621093750, 0.0027465820312500, 0.0032653808593750,
+0.0027160644531250, 0.0030212402343750, 0.0031127929687500, 0.0032043457031250,
+0.0031433105468750, 0.0031738281250000, 0.0032958984375000, 0.0029907226562500,
+0.0036315917968750, 0.0034179687500000, 0.0038146972656250, 0.0032958984375000,
+0.0039367675781250, 0.0032958984375000, 0.0041198730468750, 0.0036010742187500,
+0.0043640136718750, 0.0033264160156250, 0.0040283203125000, 0.0035705566406250,
+0.0042724609375000, 0.0035095214843750, 0.0042724609375000, 0.0034484863281250,
+0.0040588378906250, 0.0035400390625000, 0.0043029785156250, 0.0034484863281250,
+0.0039672851562500, 0.0036010742187500, 0.0041503906250000, 0.0035705566406250,
+0.0040588378906250, 0.0036621093750000, 0.0038146972656250, 0.0038452148437500,
+0.0040893554687500, 0.0038146972656250, 0.0036621093750000, 0.0040588378906250,
+0.0038757324218750, 0.0041809082031250, 0.0037536621093750, 0.0039978027343750,
+0.0034484863281250, 0.0040588378906250, 0.0038452148437500, 0.0042419433593750,
+0.0032653808593750, 0.0033569335937500, 0.0034790039062500, 0.0036926269531250,
+0.0034179687500000, 0.0032348632812500, 0.0029602050781250, 0.0023498535156250,
+0.0033264160156250, 0.0029907226562500, 0.0030517578125000, 0.0017395019531250,
+0.0030212402343750, 0.0017700195312500, 0.0031433105468750, 0.0016784667968750,
+0.0030822753906250, 0.0010070800781250, 0.0030822753906250, 0.0012817382812500,
+0.0031127929687500, 0.0007629394531250, 0.0031738281250000, 0.0008544921875000,
+0.0031738281250000, 0.0007019042968750, 0.0032043457031250, 0.0003051757812500,
+0.0031738281250000, 0.0004882812500000, 0.0030212402343750, -0.0000610351562500,
+0.0030822753906250, 0.0000610351562500, 0.0029602050781250, -0.0002441406250000,
+0.0028076171875000, -0.0007934570312500, 0.0028686523437500, -0.0005187988281250,
+0.0027465820312500},
+{0.0034484863281250, 0.0032348632812500, 0.0040283203125000, 0.0032653808593750,
+0.0041198730468750, 0.0031433105468750, 0.0040893554687500, 0.0032653808593750,
+0.0043029785156250, 0.0034790039062500, 0.0042419433593750, 0.0032958984375000,
+0.0039978027343750, 0.0035705566406250, 0.0040283203125000, 0.0035400390625000,
+0.0039062500000000, 0.0034484863281250, 0.0035095214843750, 0.0035400390625000,
+0.0035095214843750, 0.0034484863281250, 0.0033264160156250, 0.0031738281250000,
+0.0031738281250000, 0.0032348632812500, 0.0033264160156250, 0.0030212402343750,
+0.0034179687500000, 0.0026855468750000, 0.0032043457031250, 0.0028686523437500,
+0.0039062500000000, 0.0024414062500000, 0.0036926269531250, 0.0023803710937500,
+0.0038757324218750, 0.0024108886718750, 0.0044555664062500, 0.0022583007812500,
+0.0039978027343750, 0.0022583007812500, 0.0043640136718750, 0.0020446777343750,
+0.0043334960937500, 0.0021057128906250, 0.0042419433593750, 0.0018615722656250,
+0.0043945312500000, 0.0016479492187500, 0.0041198730468750, 0.0018005371093750,
+0.0040893554687500, 0.0010375976562500, 0.0040893554687500, 0.0010681152343750,
+0.0039978027343750, 0.0007324218750000, 0.0039062500000000, 0.0001220703125000,
+0.0038757324218750, 0.0003662109375000, 0.0038452148437500, -0.0004272460937500,
+0.0039367675781250, -0.0003967285156250, 0.0037841796875000, -0.0004882812500000,
+0.0038146972656250, -0.0010681152343750, 0.0040588378906250, -0.0008850097656250,
+0.0034179687500000, -0.0010070800781250, 0.0037841796875000, -0.0010986328125000,
+0.0035705566406250, -0.0011291503906250, 0.0029296875000000, -0.0011291503906250,
+0.0034179687500000, -0.0011901855468750, 0.0028991699218750, -0.0011291503906250,
+0.0028686523437500, -0.0012512207031250, 0.0031127929687500, -0.0011291503906250,
+0.0030517578125000, -0.0010070800781250, 0.0031127929687500, -0.0011596679687500,
+0.0032348632812500, -0.0006408691406250, 0.0033874511718750, -0.0007934570312500,
+0.0033569335937500, -0.0005798339843750, 0.0033874511718750, -0.0001220703125000,
+0.0035400390625000, -0.0003662109375000, 0.0032958984375000, 0.0001831054687500,
+0.0035400390625000, 0.0001525878906250, 0.0034484863281250, 0.0002746582031250,
+0.0033264160156250, 0.0006713867187500, 0.0036315917968750, 0.0004577636718750,
+0.0032043457031250, 0.0009460449218750, 0.0034790039062500, 0.0008239746093750,
+0.0034484863281250, 0.0009765625000000, 0.0031433105468750, 0.0013427734375000,
+0.0035400390625000, 0.0010986328125000, 0.0031433105468750, 0.0018310546875000,
+0.0033874511718750, 0.0016784667968750, 0.0032958984375000, 0.0019531250000000,
+0.0029907226562500, 0.0026245117187500, 0.0035095214843750, 0.0023498535156250,
+0.0028686523437500, 0.0029602050781250, 0.0031738281250000, 0.0032043457031250,
+0.0031738281250000, 0.0029602050781250, 0.0027160644531250, 0.0030822753906250,
+0.0031738281250000, 0.0032653808593750, 0.0028381347656250, 0.0024108886718750,
+0.0028686523437500, 0.0027465820312500, 0.0030822753906250, 0.0022583007812500,
+0.0029296875000000, 0.0013427734375000, 0.0029907226562500, 0.0018005371093750,
+0.0032348632812500, 0.0002441406250000, 0.0030822753906250, 0.0004577636718750,
+0.0032348632812500, -0.0000305175781250, 0.0034179687500000, -0.0011901855468750,
+0.0032348632812500, -0.0005798339843750, 0.0033874511718750, -0.0018005371093750,
+0.0033569335937500, -0.0018615722656250, 0.0033264160156250, -0.0020141601562500,
+0.0034484863281250, -0.0027160644531250, 0.0033264160156250, -0.0024414062500000,
+0.0035705566406250, -0.0028076171875000, 0.0034179687500000, -0.0029907226562500,
+0.0034790039062500, -0.0027465820312500, 0.0036926269531250, -0.0028381347656250,
+0.0035095214843750, -0.0028686523437500, 0.0037841796875000, -0.0021972656250000,
+0.0036926269531250, -0.0025024414062500, 0.0035400390625000, -0.0019226074218750,
+0.0037231445312500, -0.0010681152343750, 0.0036010742187500, -0.0016174316406250,
+0.0034484863281250, 0.0000305175781250, 0.0034484863281250, 0.0000000000000000,
+0.0033569335937500, 0.0003051757812500, 0.0032043457031250, 0.0014343261718750,
+0.0032653808593750, 0.0010375976562500, 0.0031433105468750, 0.0017700195312500,
+0.0032043457031250, 0.0018615722656250, 0.0032043457031250, 0.0018920898437500,
+0.0031127929687500, 0.0022583007812500, 0.0031738281250000, 0.0021667480468750,
+0.0029296875000000, 0.0021362304687500, 0.0031127929687500, 0.0021972656250000,
+0.0029296875000000, 0.0021057128906250, 0.0027160644531250, 0.0020141601562500,
+0.0029907226562500, 0.0020751953125000, 0.0022583007812500, 0.0018005371093750,
+0.0025024414062500, 0.0018615722656250, 0.0023498535156250, 0.0017700195312500,
+0.0018310546875000, 0.0015869140625000, 0.0021972656250000, 0.0017089843750000,
+0.0015869140625000, 0.0014648437500000, 0.0016784667968750, 0.0014648437500000,
+0.0016479492187500, 0.0014038085937500, 0.0014343261718750, 0.0013122558593750,
+0.0015869140625000, 0.0014038085937500, 0.0011596679687500, 0.0012207031250000,
+0.0013732910156250, 0.0013122558593750, 0.0011596679687500, 0.0012817382812500,
+0.0007019042968750, 0.0010681152343750, 0.0010681152343750, 0.0012512207031250,
+0.0001525878906250, 0.0009460449218750, 0.0003051757812500, 0.0010070800781250,
+0.0002441406250000, 0.0009155273437500, -0.0003967285156250, 0.0007629394531250,
+-0.0000610351562500, 0.0009155273437500, -0.0000610351562500, 0.0006713867187500,
+-0.0002746582031250, 0.0007629394531250, 0.0001220703125000, 0.0007019042968750,
+0.0004882812500000, 0.0005187988281250, 0.0002441406250000, 0.0007324218750000,
+0.0012207031250000, 0.0003967285156250, 0.0011596679687500, 0.0004882812500000,
+0.0014343261718750, 0.0004882812500000, 0.0021362304687500, 0.0003051757812500,
+0.0018310546875000, 0.0003967285156250, 0.0023193359375000, 0.0004577636718750,
+0.0023193359375000, 0.0004272460937500, 0.0024108886718750, 0.0003967285156250,
+0.0026245117187500, 0.0003662109375000, 0.0024108886718750, 0.0004882812500000,
+0.0031738281250000, 0.0008239746093750, 0.0029907226562500, 0.0004882812500000,
+0.0031738281250000, 0.0010375976562500, 0.0036926269531250, 0.0017700195312500,
+0.0034179687500000, 0.0012817382812500, 0.0037231445312500, 0.0024414062500000,
+0.0038146972656250, 0.0023803710937500, 0.0035095214843750, 0.0024414062500000,
+0.0034790039062500, 0.0031127929687500, 0.0034790039062500, 0.0027160644531250,
+0.0028076171875000, 0.0030517578125000, 0.0028381347656250, 0.0030822753906250,
+0.0027160644531250, 0.0031433105468750, 0.0019836425781250, 0.0035095214843750,
+0.0003967285156250, 0.0009765625000000, 0.0007629394531250, 0.0014038085937500,
+0.0004272460937500, 0.0008850097656250, 0.0007629394531250, 0.0013427734375000,
+0.0011291503906250, 0.0018615722656250, 0.0007934570312500, 0.0014953613281250,
+0.0015869140625000, 0.0025329589843750, 0.0014953613281250, 0.0023498535156250,
+0.0016784667968750, 0.0025939941406250, 0.0022888183593750, 0.0035400390625000,
+0.0020141601562500, 0.0031433105468750, 0.0026855468750000, 0.0035705566406250,
+0.0025329589843750, 0.0037231445312500, 0.0028381347656250, 0.0038146972656250,
+0.0032958984375000, 0.0040588378906250, 0.0029296875000000, 0.0039978027343750,
+0.0039978027343750, 0.0040893554687500, 0.0037536621093750, 0.0043334960937500,
+0.0039062500000000, 0.0040588378906250, 0.0047302246093750, 0.0039062500000000,
+0.0042419433593750, 0.0042724609375000, 0.0044250488281250, 0.0035095214843750,
+0.0045471191406250, 0.0037231445312500, 0.0040893554687500, 0.0035400390625000,
+0.0038146972656250, 0.0029602050781250, 0.0039672851562500, 0.0033569335937500,
+0.0031127929687500, 0.0027160644531250, 0.0030822753906250, 0.0027465820312500,
+0.0028076171875000, 0.0028991699218750, 0.0021362304687500, 0.0027160644531250,
+0.0023803710937500, 0.0028076171875000, 0.0016784667968750, 0.0027160644531250,
+0.0015563964843750, 0.0029296875000000, 0.0015258789062500, 0.0028991699218750,
+0.0011596679687500, 0.0026245117187500, 0.0012207031250000, 0.0027160644531250,
+0.0012207031250000, 0.0025024414062500, 0.0010681152343750, 0.0026245117187500,
+0.0012207031250000, 0.0023803710937500, 0.0014038085937500, 0.0020446777343750,
+0.0011901855468750, 0.0024719238281250, 0.0014648437500000, 0.0024719238281250,
+0.0014953613281250, 0.0024108886718750, 0.0012512207031250, 0.0028686523437500,
+0.0013732910156250, 0.0032043457031250, 0.0014953613281250, 0.0031738281250000,
+0.0004272460937500, 0.0042724609375000, 0.0007629394531250, 0.0043029785156250,
+0.0002136230468750, 0.0041503906250000, -0.0007934570312500, 0.0048522949218750,
+-0.0002441406250000, 0.0050354003906250, -0.0014648437500000, 0.0039367675781250,
+-0.0015563964843750, 0.0043945312500000, -0.0014343261718750, 0.0039367675781250,
+-0.0020446777343750, 0.0028076171875000, -0.0020141601562500, 0.0036010742187500,
+-0.0017089843750000, 0.0026855468750000, -0.0019226074218750, 0.0025024414062500,
+-0.0017089843750000, 0.0029602050781250, -0.0013122558593750, 0.0028076171875000,
+-0.0014953613281250, 0.0026855468750000, -0.0012512207031250, 0.0036010742187500,
+-0.0010681152343750, 0.0033264160156250, -0.0012817382812500, 0.0035400390625000,
+-0.0013122558593750, 0.0042419433593750, -0.0010681152343750, 0.0036315917968750,
+-0.0019836425781250, 0.0040893554687500, -0.0016784667968750, 0.0040893554687500,
+-0.0018920898437500, 0.0038452148437500, -0.0025634765625000, 0.0037841796875000,
+-0.0021057128906250, 0.0036926269531250, -0.0024719238281250, 0.0036926269531250,
+-0.0025329589843750, 0.0035400390625000, -0.0023498535156250, 0.0036621093750000,
+-0.0023498535156250, 0.0038757324218750, -0.0023193359375000, 0.0036315917968750,
+-0.0022583007812500, 0.0042114257812500, -0.0021362304687500, 0.0041503906250000,
+-0.0022888183593750, 0.0040893554687500, -0.0022888183593750, 0.0045166015625000,
+-0.0021057128906250, 0.0045166015625000, -0.0025939941406250, 0.0041809082031250,
+-0.0023803710937500, 0.0044860839843750, -0.0026245117187500, 0.0042114257812500,
+-0.0031433105468750, 0.0037841796875000, -0.0027465820312500, 0.0042114257812500,
+-0.0032958984375000, 0.0035095214843750, -0.0033264160156250, 0.0036621093750000,
+-0.0030822753906250, 0.0036621093750000, -0.0032653808593750, 0.0031738281250000,
+-0.0032348632812500, 0.0035095214843750, -0.0023803710937500, 0.0032348632812500,
+-0.0026550292968750, 0.0032958984375000, -0.0021362304687500, 0.0032958984375000,
+-0.0012817382812500, 0.0032348632812500, -0.0017089843750000, 0.0033264160156250,
+-0.0003967285156250, 0.0032043457031250, -0.0004882812500000, 0.0032348632812500,
+-0.0002136230468750, 0.0032043457031250, 0.0007019042968750, 0.0031127929687500,
+0.0002441406250000, 0.0031127929687500, 0.0010375976562500, 0.0030822753906250,
+0.0010681152343750, 0.0030517578125000, 0.0009155273437500, 0.0029602050781250,
+0.0012207031250000, 0.0029296875000000, 0.0011291503906250, 0.0029602050781250,
+0.0008239746093750, 0.0028381347656250, 0.0009155273437500, 0.0027770996093750,
+0.0005187988281250, 0.0028076171875000, -0.0000305175781250, 0.0027770996093750,
+0.0001831054687500, 0.0026855468750000, -0.0009155273437500, 0.0027160644531250,
+-0.0009155273437500, 0.0026550292968750, -0.0012207031250000, 0.0025634765625000,
+-0.0019836425781250, 0.0025329589843750, -0.0018005371093750, 0.0024719238281250,
+-0.0022277832031250, 0.0025024414062500, -0.0024414062500000, 0.0023803710937500,
+-0.0023193359375000},
+{0.0000305175781250, 0.0033874511718750, -0.0000915527343750, 0.0032043457031250,
+0.0003051757812500, 0.0031433105468750, 0.0000915527343750, 0.0034790039062500,
+-0.0002441406250000, 0.0025024414062500, -0.0002136230468750, 0.0028381347656250,
+-0.0004577636718750, 0.0025024414062500, -0.0008544921875000, 0.0017700195312500,
+-0.0007934570312500, 0.0023803710937500, -0.0007934570312500, 0.0016479492187500,
+-0.0012207031250000, 0.0015563964843750, -0.0008850097656250, 0.0020141601562500,
+-0.0005187988281250, 0.0020141601562500, -0.0010681152343750, 0.0019531250000000,
+0.0001831054687500, 0.0024719238281250, -0.0002136230468750, 0.0027160644531250,
+-0.0000610351562500, 0.0027465820312500, 0.0008544921875000, 0.0029602050781250,
+0.0003356933593750, 0.0030517578125000, 0.0005187988281250, 0.0027160644531250,
+0.0007324218750000, 0.0029296875000000, 0.0004272460937500, 0.0027160644531250,
+0.0003051757812500, 0.0021972656250000, 0.0004272460937500, 0.0025634765625000,
+-0.0002746582031250, 0.0020446777343750, -0.0001525878906250, 0.0018005371093750,
+-0.0003051757812500, 0.0021667480468750, -0.0009155273437500, 0.0021362304687500,
+-0.0006408691406250, 0.0017089843750000, -0.0009765625000000, 0.0028076171875000,
+-0.0011596679687500, 0.0022277832031250, -0.0011596679687500, 0.0024414062500000,
+-0.0012512207031250, 0.0034484863281250, -0.0013732910156250, 0.0028381347656250,
+-0.0017395019531250, 0.0033569335937500, -0.0015563964843750, 0.0035095214843750,
+-0.0021057128906250, 0.0032348632812500, -0.0027160644531250, 0.0033264160156250,
+-0.0023193359375000, 0.0034179687500000, -0.0036315917968750, 0.0029296875000000,
+-0.0035400390625000, 0.0030517578125000, -0.0037841796875000, 0.0030517578125000,
+-0.0046691894531250, 0.0027770996093750, -0.0043029785156250, 0.0029296875000000,
+-0.0048522949218750, 0.0027465820312500, -0.0050964355468750, 0.0028686523437500,
+-0.0048217773437500, 0.0028991699218750, -0.0049438476562500, 0.0027770996093750,
+-0.0051269531250000, 0.0028381347656250, -0.0041503906250000, 0.0028381347656250,
+-0.0045776367187500, 0.0028076171875000, -0.0040588378906250, 0.0028381347656250,
+-0.0031127929687500, 0.0028076171875000, -0.0037536621093750, 0.0028686523437500,
+-0.0024108886718750, 0.0027770996093750, -0.0024719238281250, 0.0027160644531250,
+-0.0022888183593750, 0.0027770996093750, -0.0014953613281250, 0.0028076171875000,
+-0.0017700195312500, 0.0027770996093750, -0.0010070800781250, 0.0028381347656250,
+-0.0010375976562500, 0.0028686523437500, -0.0008239746093750, 0.0028076171875000,
+-0.0003662109375000, 0.0027465820312500, -0.0006103515625000, 0.0027160644531250,
+0.0000610351562500, 0.0025329589843750, -0.0000915527343750, 0.0025329589843750,
+0.0000915527343750, 0.0024108886718750, 0.0006408691406250, 0.0021972656250000,
+0.0003662109375000, 0.0021972656250000, 0.0009765625000000, 0.0020446777343750,
+0.0010070800781250, 0.0019531250000000, 0.0010070800781250, 0.0018615722656250,
+0.0014038085937500, 0.0017700195312500, 0.0012817382812500, 0.0016784667968750,
+0.0013427734375000, 0.0017089843750000, 0.0014648437500000, 0.0015258789062500,
+0.0011901855468750, 0.0015869140625000, 0.0009155273437500, 0.0016479492187500,
+0.0009765625000000, 0.0014648437500000, 0.0001525878906250, 0.0017089843750000,
+0.0002746582031250, 0.0015563964843750, -0.0001525878906250, 0.0015563964843750,
+-0.0009155273437500, 0.0016784667968750, -0.0006713867187500, 0.0014648437500000,
+-0.0013122558593750, 0.0016784667968750, -0.0014343261718750, 0.0016174316406250,
+-0.0014343261718750, 0.0016479492187500, -0.0016784667968750, 0.0018920898437500,
+-0.0016784667968750, 0.0018005371093750, -0.0014953613281250, 0.0020141601562500,
+-0.0016174316406250, 0.0020141601562500, -0.0014038085937500, 0.0020141601562500,
+-0.0011596679687500, 0.0021972656250000, -0.0014038085937500, 0.0021972656250000,
+-0.0007324218750000, 0.0021362304687500, -0.0009765625000000, 0.0022583007812500,
+-0.0007629394531250, 0.0021972656250000, -0.0003051757812500, 0.0021362304687500,
+-0.0006713867187500, 0.0022888183593750, 0.0000305175781250, 0.0022277832031250,
+-0.0001220703125000, 0.0022583007812500, -0.0000915527343750, 0.0022888183593750,
+0.0004272460937500, 0.0022583007812500, 0.0001831054687500, 0.0023193359375000,
+0.0003356933593750, 0.0021362304687500, 0.0004577636718750, 0.0022888183593750,
+0.0002441406250000, 0.0022583007812500, 0.0002136230468750, 0.0020751953125000,
+0.0003356933593750, 0.0022583007812500, -0.0001525878906250, 0.0019226074218750,
+-0.0001220703125000, 0.0020751953125000, -0.0001831054687500, 0.0020751953125000,
+-0.0005187988281250, 0.0018615722656250, -0.0003662109375000, 0.0020141601562500,
+-0.0003051757812500, 0.0019226074218750, -0.0004882812500000, 0.0020141601562500,
+-0.0001220703125000, 0.0020141601562500, 0.0001831054687500, 0.0019531250000000,
+-0.0000915527343750, 0.0020141601562500, 0.0009460449218750, 0.0018310546875000,
+0.0007324218750000, 0.0018615722656250, 0.0009155273437500, 0.0018005371093750,
+0.0015869140625000, 0.0016784667968750, 0.0011901855468750, 0.0017700195312500,
+0.0016784667968750, 0.0016174316406250, 0.0018005371093750, 0.0016174316406250,
+0.0015563964843750, 0.0016784667968750, 0.0017395019531250, 0.0016174316406250,
+0.0018615722656250, 0.0016784667968750, 0.0012207031250000, 0.0017089843750000,
+0.0014953613281250, 0.0016174316406250, 0.0012512207031250, 0.0017395019531250,
+0.0006103515625000, 0.0018615722656250, 0.0009155273437500, 0.0017395019531250,
+0.0000915527343750, 0.0023193359375000, 0.0000610351562500, 0.0020751953125000,
+-0.0000610351562500, 0.0023193359375000, -0.0006408691406250, 0.0029602050781250,
+-0.0006408691406250, 0.0025024414062500, -0.0004882812500000, 0.0032348632812500,
+-0.0007934570312500, 0.0032653808593750, -0.0005798339843750, 0.0031433105468750,
+0.0000610351562500, 0.0034179687500000, 0.0002136230468750, 0.0032043457031250,
+0.0010070800781250, 0.0029296875000000, 0.0008544921875000, 0.0030822753906250,
+0.0008850097656250, 0.0028991699218750, 0.0013732910156250, 0.0026550292968750,
+0.0011901855468750, 0.0028381347656250, 0.0018005371093750, 0.0022888183593750,
+0.0017395019531250, 0.0024414062500000, 0.0019836425781250, 0.0022888183593750,
+0.0025634765625000, 0.0018615722656250, 0.0023498535156250, 0.0020141601562500,
+0.0027465820312500, 0.0016479492187500, 0.0028076171875000, 0.0017089843750000,
+0.0028076171875000, 0.0015869140625000, 0.0028686523437500, 0.0012817382812500,
+0.0026245117187500, 0.0014038085937500, 0.0030822753906250, 0.0010070800781250,
+0.0028686523437500, 0.0010681152343750, 0.0027770996093750, 0.0007934570312500,
+0.0030212402343750, 0.0003662109375000, 0.0028686523437500, 0.0005798339843750,
+0.0024719238281250, -0.0000915527343750, 0.0028076171875000, -0.0002136230468750,
+0.0025024414062500, -0.0002746582031250, 0.0019531250000000, -0.0006103515625000,
+0.0023498535156250, -0.0006103515625000, 0.0019531250000000, -0.0004272460937500,
+0.0019531250000000, -0.0006103515625000, 0.0021362304687500, -0.0004882812500000,
+0.0020141601562500, -0.0001525878906250, 0.0021667480468750, -0.0003662109375000,
+0.0024414062500000, 0.0001525878906250, 0.0022888183593750, 0.0001525878906250,
+0.0025024414062500, 0.0002746582031250, 0.0028381347656250, 0.0005493164062500,
+0.0025634765625000, 0.0003967285156250, 0.0027160644531250, 0.0006408691406250,
+0.0027770996093750, 0.0006713867187500, 0.0026855468750000, 0.0006713867187500,
+0.0027770996093750, 0.0007629394531250, 0.0026855468750000, 0.0007629394531250,
+0.0024719238281250, 0.0007019042968750, 0.0025024414062500, 0.0008239746093750,
+0.0025024414062500, 0.0007324218750000, 0.0023803710937500, 0.0006103515625000,
+0.0022277832031250, 0.0007629394531250, 0.0027160644531250, 0.0004882812500000,
+0.0023498535156250, 0.0006103515625000, 0.0026550292968750, 0.0005798339843750,
+0.0032653808593750, 0.0004272460937500, 0.0026855468750000, 0.0006713867187500,
+0.0032958984375000, 0.0004882812500000, 0.0032348632812500, 0.0006408691406250,
+0.0030822753906250, 0.0006103515625000, 0.0032043457031250, 0.0004882812500000,
+0.0030212402343750, 0.0006713867187500, 0.0030517578125000, 0.0004272460937500,
+0.0029296875000000, 0.0005798339843750, 0.0029296875000000, 0.0004577636718750,
+0.0030212402343750, 0.0002746582031250, 0.0028686523437500, 0.0005187988281250,
+0.0030212402343750, -0.0001220703125000, 0.0030212402343750, 0.0000915527343750,
+0.0028686523437500, -0.0001831054687500, 0.0029296875000000, -0.0008239746093750,
+0.0029907226562500, -0.0003967285156250, 0.0024719238281250, -0.0012512207031250,
+0.0026550292968750, -0.0012512207031250, 0.0024414062500000, -0.0012207031250000,
+0.0018920898437500, -0.0016784667968750, 0.0021972656250000, -0.0014953613281250,
+0.0015563964843750, -0.0014953613281250, 0.0015869140625000, -0.0015563964843750,
+0.0015563964843750, -0.0014343261718750, 0.0012512207031250, -0.0012207031250000,
+0.0014038085937500, -0.0014038085937500, 0.0014038085937500, -0.0013122558593750,
+0.0013732910156250, -0.0011596679687500, 0.0014648437500000, -0.0013427734375000,
+0.0016479492187500, -0.0015563964843750, 0.0016479492187500, -0.0013732910156250,
+0.0018310546875000, -0.0018920898437500, 0.0018615722656250, -0.0018310546875000,
+0.0018920898437500, -0.0018310546875000, 0.0019836425781250, -0.0021667480468750,
+0.0019531250000000, -0.0020751953125000, 0.0019836425781250, -0.0015258789062500,
+0.0019531250000000, -0.0018615722656250, 0.0020446777343750, -0.0013122558593750,
+0.0020751953125000, -0.0005493164062500, 0.0020446777343750, -0.0009765625000000,
+0.0020751953125000, 0.0003356933593750, 0.0021667480468750, 0.0003051757812500,
+0.0021667480468750, 0.0005798339843750, 0.0021057128906250, 0.0014648437500000,
+0.0021667480468750, 0.0010986328125000, 0.0021667480468750, 0.0017089843750000,
+0.0021362304687500, 0.0019226074218750, 0.0022277832031250, 0.0018310546875000,
+0.0022888183593750, 0.0020446777343750, 0.0021972656250000, 0.0021362304687500,
+0.0024108886718750, 0.0016174316406250, 0.0024108886718750, 0.0018920898437500,
+0.0023803710937500, 0.0015869140625000, 0.0025329589843750, 0.0010070800781250,
+0.0024108886718750, 0.0014343261718750, 0.0024108886718750, 0.0006103515625000,
+0.0024108886718750, 0.0007019042968750, 0.0023193359375000, 0.0005493164062500,
+0.0022277832031250, 0.0000305175781250, 0.0022277832031250, 0.0002441406250000,
+0.0021362304687500, -0.0001831054687500, 0.0021057128906250, -0.0001220703125000,
+0.0020751953125000, -0.0004577636718750, 0.0020141601562500, -0.0008544921875000,
+0.0020141601562500, -0.0005798339843750, 0.0018310546875000, -0.0016479492187500,
+0.0019226074218750, -0.0015869140625000, 0.0018005371093750, -0.0018920898437500,
+0.0016174316406250, -0.0027770996093750, 0.0017395019531250, -0.0025024414062500,
+0.0013732910156250, -0.0028686523437500, 0.0014038085937500, -0.0031738281250000,
+0.0014648437500000, -0.0029602050781250, 0.0013732910156250, -0.0028991699218750,
+0.0013732910156250, -0.0031433105468750, 0.0016174316406250, -0.0022888183593750,
+0.0014953613281250, -0.0025634765625000, 0.0016784667968750, -0.0021362304687500,
+0.0019531250000000, -0.0013122558593750, 0.0017700195312500, -0.0017395019531250,
+0.0020751953125000, -0.0008544921875000, 0.0021057128906250, -0.0008239746093750,
+0.0019836425781250, -0.0007934570312500, 0.0020141601562500, -0.0003051757812500,
+0.0020446777343750},
+{0.0013122558593750, 0.0015869140625000, 0.0010375976562500, 0.0015563964843750,
+0.0012207031250000, 0.0015563964843750, 0.0013122558593750, 0.0016174316406250,
+0.0010986328125000, 0.0013732910156250, 0.0010070800781250, 0.0012207031250000,
+0.0011596679687500, 0.0013427734375000, 0.0007324218750000, 0.0010375976562500,
+0.0008239746093750, 0.0009765625000000, 0.0007934570312500, 0.0010681152343750,
+0.0006103515625000, 0.0009765625000000, 0.0008239746093750, 0.0009155273437500,
+0.0007629394531250, 0.0011901855468750, 0.0008544921875000, 0.0011596679687500,
+0.0008850097656250, 0.0011901855468750, 0.0008850097656250, 0.0014343261718750,
+0.0009765625000000, 0.0013122558593750, 0.0005798339843750, 0.0012207031250000,
+0.0007324218750000, 0.0013732910156250, 0.0006408691406250, 0.0010375976562500,
+0.0002746582031250, 0.0006408691406250, 0.0004882812500000, 0.0008850097656250,
+0.0002441406250000, -0.0001831054687500, 0.0002136230468750, -0.0000305175781250,
+0.0002746582031250, -0.0003967285156250, 0.0001831054687500, -0.0013427734375000,
+0.0001831054687500, -0.0009155273437500, 0.0003967285156250, -0.0017700195312500,
+0.0003356933593750, -0.0017395019531250, 0.0004882812500000, -0.0018310546875000,
+0.0007934570312500, -0.0023193359375000, 0.0007019042968750, -0.0021057128906250,
+0.0008850097656250, -0.0024719238281250, 0.0009765625000000, -0.0025634765625000,
+0.0009460449218750, -0.0024414062500000, 0.0009460449218750, -0.0026855468750000,
+0.0009765625000000, -0.0026855468750000, 0.0010070800781250, -0.0021057128906250,
+0.0009765625000000, -0.0024719238281250, 0.0010375976562500, -0.0019226074218750,
+0.0010375976562500, -0.0011596679687500, 0.0010375976562500, -0.0017395019531250,
+0.0010681152343750, -0.0000610351562500, 0.0011291503906250, -0.0003967285156250,
+0.0010986328125000, 0.0000305175781250, 0.0011291503906250, 0.0013122558593750,
+0.0012207031250000, 0.0006713867187500, 0.0010986328125000, 0.0016784667968750,
+0.0011291503906250, 0.0018920898437500, 0.0010681152343750, 0.0015869140625000,
+0.0009460449218750, 0.0019226074218750, 0.0010375976562500, 0.0019836425781250,
+0.0010681152343750, 0.0011596679687500, 0.0010681152343750, 0.0015563964843750,
+0.0011596679687500, 0.0010070800781250, 0.0012512207031250, 0.0000000000000000,
+0.0012512207031250, 0.0005798339843750, 0.0014343261718750, -0.0006103515625000,
+0.0014648437500000, -0.0007324218750000, 0.0014648437500000, -0.0006103515625000,
+0.0015869140625000, -0.0010375976562500, 0.0016784667968750, -0.0009765625000000,
+0.0015563964843750, -0.0007324218750000, 0.0016479492187500, -0.0008239746093750,
+0.0016784667968750, -0.0007019042968750, 0.0015563964843750, -0.0004882812500000,
+0.0016479492187500, -0.0005798339843750, 0.0017089843750000, -0.0006103515625000,
+0.0017089843750000, -0.0005493164062500, 0.0018005371093750, -0.0006408691406250,
+0.0019531250000000, -0.0007934570312500, 0.0018920898437500, -0.0008544921875000,
+0.0020141601562500, -0.0008544921875000, 0.0020446777343750, -0.0010681152343750,
+0.0019836425781250, -0.0008850097656250, 0.0019836425781250, -0.0007324218750000,
+0.0020446777343750, -0.0010681152343750, 0.0018310546875000, -0.0002136230468750,
+0.0018920898437500, -0.0004882812500000, 0.0018615722656250, -0.0000610351562500,
+0.0017700195312500, 0.0007629394531250, 0.0017700195312500, 0.0002746582031250,
+0.0019226074218750, 0.0011901855468750, 0.0018005371093750, 0.0012512207031250,
+0.0020141601562500, 0.0012512207031250, 0.0022888183593750, 0.0017700195312500,
+0.0020141601562500, 0.0017089843750000, 0.0024414062500000, 0.0015869140625000,
+0.0023803710937500, 0.0018310546875000, 0.0023498535156250, 0.0016479492187500,
+0.0025939941406250, 0.0013732910156250, 0.0024414062500000, 0.0016479492187500,
+0.0024719238281250, 0.0013122558593750, 0.0024108886718750, 0.0013732910156250,
+0.0024108886718750, 0.0013427734375000, 0.0024108886718750, 0.0011596679687500,
+0.0022277832031250, 0.0013427734375000, 0.0024719238281250, 0.0010681152343750,
+0.0023193359375000, 0.0011901855468750, 0.0023193359375000, 0.0010375976562500,
+0.0025634765625000, 0.0007934570312500, 0.0024414062500000, 0.0010986328125000,
+0.0022583007812500, 0.0006408691406250, 0.0025329589843750, 0.0006713867187500,
+0.0022583007812500, 0.0006408691406250, 0.0018310546875000, 0.0002441406250000,
+0.0021972656250000, 0.0003967285156250, 0.0013427734375000, 0.0005187988281250,
+0.0015258789062500, 0.0002441406250000, 0.0013732910156250, 0.0004272460937500,
+0.0007324218750000, 0.0008544921875000, 0.0011596679687500, 0.0005798339843750,
+0.0005798339843750, 0.0008239746093750, 0.0005798339843750, 0.0009155273437500,
+0.0005187988281250, 0.0006713867187500, 0.0001220703125000, 0.0006408691406250,
+0.0002441406250000, 0.0007019042968750, 0.0000610351562500, 0.0002746582031250,
+0.0000000000000000, 0.0002441406250000, 0.0001220703125000, 0.0004577636718750,
+0.0001831054687500, 0.0003051757812500, 0.0000915527343750, 0.0002136230468750,
+0.0006713867187500, 0.0006103515625000, 0.0005798339843750, 0.0005798339843750,
+0.0008239746093750, 0.0004577636718750, 0.0017089843750000, 0.0004882812500000,
+0.0032348632812500, 0.0010681152343750, 0.0024719238281250, -0.0003662109375000,
+0.0031127929687500, 0.0001831054687500, 0.0025329589843750, -0.0000610351562500,
+0.0018005371093750, -0.0008850097656250, 0.0024108886718750, -0.0007019042968750,
+0.0009460449218750, -0.0003662109375000, 0.0011901855468750, -0.0008850097656250,
+0.0008239746093750, -0.0002441406250000, -0.0001525878906250, 0.0005798339843750,
+0.0004577636718750, -0.0000915527343750, -0.0004882812500000, 0.0009460449218750,
+-0.0005798339843750, 0.0008850097656250, -0.0003662109375000, 0.0009460449218750,
+-0.0007324218750000, 0.0016479492187500, -0.0007324218750000, 0.0012207031250000,
+-0.0002441406250000, 0.0016784667968750, -0.0004577636718750, 0.0017700195312500,
+-0.0001831054687500, 0.0017089843750000, 0.0002746582031250, 0.0018920898437500,
+0.0000000000000000, 0.0018920898437500, 0.0007324218750000, 0.0017395019531250,
+0.0006713867187500, 0.0020446777343750, 0.0008544921875000, 0.0014953613281250,
+0.0014343261718750, 0.0009765625000000, 0.0011901855468750, 0.0015563964843750,
+0.0015258789062500, 0.0005493164062500, 0.0016784667968750, 0.0004577636718750,
+0.0014343261718750, 0.0007934570312500, 0.0013427734375000, 0.0004577636718750,
+0.0014343261718750, 0.0003051757812500, 0.0009765625000000, 0.0011901855468750,
+0.0009765625000000, 0.0011291503906250, 0.0008850097656250, 0.0015258789062500,
+0.0005187988281250, 0.0023193359375000, 0.0005798339843750, 0.0021362304687500,
+0.0008544921875000, 0.0025939941406250, 0.0005493164062500, 0.0028381347656250,
+0.0009765625000000, 0.0026550292968750, 0.0015258789062500, 0.0025024414062500,
+0.0010986328125000, 0.0025939941406250, 0.0018310546875000, 0.0023803710937500,
+0.0018920898437500, 0.0022277832031250, 0.0017395019531250, 0.0020751953125000,
+0.0020446777343750, 0.0019226074218750, 0.0020446777343750, 0.0019836425781250,
+0.0016174316406250, 0.0021972656250000, 0.0017700195312500, 0.0020141601562500,
+0.0015563964843750, 0.0021057128906250, 0.0010681152343750, 0.0025634765625000,
+0.0013122558593750, 0.0024719238281250, 0.0010070800781250, 0.0025024414062500,
+0.0008850097656250, 0.0026550292968750, 0.0010375976562500, 0.0023193359375000,
+0.0010986328125000, 0.0021362304687500, 0.0009460449218750, 0.0025024414062500,
+0.0011291503906250, 0.0016784667968750, 0.0011596679687500, 0.0017089843750000,
+0.0008239746093750, 0.0018310546875000, 0.0006408691406250, 0.0015258789062500,
+0.0007629394531250, 0.0016174316406250, -0.0000610351562500, 0.0017700195312500,
+0.0000305175781250, 0.0018920898437500, -0.0002136230468750, 0.0018310546875000,
+-0.0008544921875000, 0.0019531250000000, -0.0006408691406250, 0.0021057128906250,
+-0.0009460449218750, 0.0015563964843750, -0.0012207031250000, 0.0018310546875000,
+-0.0008850097656250, 0.0015869140625000, -0.0007629394531250, 0.0008850097656250,
+-0.0009155273437500, 0.0010681152343750, 0.0001525878906250, 0.0005187988281250,
+-0.0001831054687500, 0.0002441406250000, 0.0003051757812500, 0.0003051757812500,
+0.0013122558593750, 0.0000610351562500, 0.0006713867187500, -0.0000610351562500,
+0.0019531250000000, 0.0004882812500000, 0.0018920898437500, 0.0001831054687500,
+0.0019836425781250, 0.0003662109375000, 0.0027465820312500, 0.0009460449218750,
+0.0023803710937500, 0.0005187988281250, 0.0025329589843750, 0.0009155273437500,
+0.0026855468750000, 0.0009155273437500, 0.0023803710937500, 0.0007019042968750,
+0.0021362304687500, 0.0007934570312500, 0.0022583007812500, 0.0006713867187500,
+0.0014648437500000, 0.0004577636718750, 0.0016174316406250, 0.0004577636718750,
+0.0012817382812500, 0.0003356933593750, 0.0006713867187500, 0.0000915527343750,
+0.0009460449218750, 0.0000305175781250, -0.0000610351562500, 0.0001525878906250,
+0.0000000000000000, -0.0000305175781250, -0.0003356933593750, 0.0001220703125000,
+-0.0011901855468750, 0.0003967285156250, -0.0008544921875000, 0.0001831054687500,
+-0.0016479492187500, 0.0005493164062500, -0.0017700195312500, 0.0004882812500000,
+-0.0017395019531250, 0.0005493164062500, -0.0021362304687500, 0.0007629394531250,
+-0.0021972656250000, 0.0006103515625000, -0.0019226074218750, 0.0009460449218750,
+-0.0022277832031250, 0.0008239746093750, -0.0018615722656250, 0.0009155273437500,
+-0.0014038085937500, 0.0011596679687500, -0.0018005371093750, 0.0010986328125000,
+-0.0006408691406250, 0.0012512207031250, -0.0008239746093750, 0.0012817382812500,
+-0.0005187988281250, 0.0013122558593750, 0.0003662109375000, 0.0014343261718750,
+-0.0000610351562500, 0.0014038085937500, 0.0008850097656250, 0.0015869140625000,
+0.0009155273437500, 0.0015563964843750, 0.0008850097656250, 0.0016784667968750,
+0.0013732910156250, 0.0018615722656250, 0.0013122558593750, 0.0018005371093750,
+0.0014343261718750, 0.0019531250000000, 0.0015869140625000, 0.0020446777343750,
+0.0015258789062500, 0.0019836425781250, 0.0014953613281250, 0.0019836425781250,
+0.0017700195312500, 0.0021667480468750, 0.0014038085937500, 0.0019531250000000,
+0.0016479492187500, 0.0020751953125000, 0.0014953613281250, 0.0021057128906250,
+0.0011901855468750, 0.0019836425781250, 0.0015563964843750, 0.0021667480468750,
+0.0006103515625000, 0.0019226074218750, 0.0008544921875000, 0.0019836425781250,
+0.0004272460937500, 0.0019836425781250, -0.0003662109375000, 0.0018920898437500,
+0.0000915527343750, 0.0019836425781250, -0.0009765625000000, 0.0019226074218750,
+-0.0010375976562500, 0.0019226074218750, -0.0010070800781250, 0.0018920898437500,
+-0.0015563964843750, 0.0018615722656250, -0.0013427734375000, 0.0018615722656250,
+-0.0013122558593750, 0.0018005371093750, -0.0014038085937500, 0.0018310546875000,
+-0.0010681152343750, 0.0018615722656250, -0.0007629394531250, 0.0018005371093750,
+-0.0008850097656250, 0.0017089843750000, -0.0002746582031250, 0.0018920898437500,
+-0.0002136230468750, 0.0017700195312500, -0.0000305175781250, 0.0017395019531250,
+0.0003967285156250, 0.0018005371093750, 0.0003662109375000, 0.0017089843750000,
+0.0006713867187500, 0.0017089843750000, 0.0008544921875000, 0.0017089843750000,
+0.0008544921875000, 0.0016784667968750, 0.0010070800781250, 0.0015869140625000,
+0.0010375976562500, 0.0015258789062500, 0.0008239746093750, 0.0016174316406250,
+0.0010681152343750},
+{0.0012207031250000, 0.0029602050781250, 0.0014343261718750, 0.0026550292968750,
+0.0011596679687500, 0.0025939941406250, 0.0011901855468750, 0.0030212402343750,
+0.0013732910156250, 0.0025329589843750, 0.0006713867187500, 0.0025329589843750,
+0.0009155273437500, 0.0024108886718750, 0.0006713867187500, 0.0023193359375000,
+0.0000915527343750, 0.0022888183593750, 0.0004577636718750, 0.0019836425781250,
+-0.0001220703125000, 0.0022583007812500, -0.0000915527343750, 0.0020446777343750,
+0.0000915527343750, 0.0021057128906250, -0.0001831054687500, 0.0023498535156250,
+-0.0001525878906250, 0.0020446777343750, 0.0004272460937500, 0.0023803710937500,
+0.0002136230468750, 0.0023498535156250, 0.0006103515625000, 0.0022277832031250,
+0.0012512207031250, 0.0023498535156250, 0.0008239746093750, 0.0023498535156250,
+0.0018615722656250, 0.0020751953125000, 0.0018310546875000, 0.0021362304687500,
+0.0019531250000000, 0.0021972656250000, 0.0026245117187500, 0.0020446777343750,
+0.0023498535156250, 0.0021362304687500, 0.0026550292968750, 0.0023498535156250,
+0.0028076171875000, 0.0022888183593750, 0.0023803710937500, 0.0024108886718750,
+0.0022888183593750, 0.0026550292968750, 0.0024719238281250, 0.0025634765625000,
+0.0013732910156250, 0.0028076171875000, 0.0015869140625000, 0.0027770996093750,
+0.0010375976562500, 0.0028686523437500, 0.0000610351562500, 0.0030212402343750,
+0.0005187988281250, 0.0028381347656250, -0.0009460449218750, 0.0032043457031250,
+-0.0008850097656250, 0.0030517578125000, -0.0010986328125000, 0.0031433105468750,
+-0.0020141601562500, 0.0034790039062500, -0.0015258789062500, 0.0031738281250000,
+-0.0021057128906250, 0.0035400390625000, -0.0020446777343750, 0.0035705566406250,
+-0.0019531250000000, 0.0034179687500000, -0.0021362304687500, 0.0034790039062500,
+-0.0018615722656250, 0.0034790039062500, -0.0017395019531250, 0.0032348632812500,
+-0.0017395019531250, 0.0033264160156250, -0.0014953613281250, 0.0031433105468750,
+-0.0012817382812500, 0.0028076171875000, -0.0014038085937500, 0.0029602050781250,
+-0.0009460449218750, 0.0024108886718750, -0.0011291503906250, 0.0024719238281250,
+-0.0008239746093750, 0.0023193359375000, -0.0003051757812500, 0.0018615722656250,
+-0.0006713867187500, 0.0020446777343750, 0.0003051757812500, 0.0016784667968750,
+0.0000610351562500, 0.0015869140625000, 0.0003051757812500, 0.0015869140625000,
+0.0010375976562500, 0.0013732910156250, 0.0005187988281250, 0.0013122558593750,
+0.0010375976562500, 0.0014648437500000, 0.0011291503906250, 0.0011901855468750,
+0.0008850097656250, 0.0013732910156250, 0.0009155273437500, 0.0015869140625000,
+0.0009155273437500, 0.0012512207031250, 0.0005187988281250, 0.0018615722656250,
+0.0006408691406250, 0.0017700195312500, 0.0005798339843750, 0.0018005371093750,
+0.0002136230468750, 0.0022277832031250, 0.0003967285156250, 0.0019531250000000,
+0.0003356933593750, 0.0022888183593750, 0.0002746582031250, 0.0022888183593750,
+0.0004272460937500, 0.0022277832031250, 0.0005493164062500, 0.0023803710937500,
+0.0004577636718750, 0.0022277832031250, 0.0007629394531250, 0.0023193359375000,
+0.0006103515625000, 0.0022583007812500, 0.0007934570312500, 0.0022277832031250,
+0.0010681152343750, 0.0022583007812500, 0.0008239746093750, 0.0020751953125000,
+0.0012207031250000, 0.0021057128906250, 0.0011291503906250, 0.0020446777343750,
+0.0010070800781250, 0.0019836425781250, 0.0011291503906250, 0.0020141601562500,
+0.0011291503906250, 0.0019531250000000, 0.0008544921875000, 0.0022888183593750,
+0.0009460449218750, 0.0021057128906250, 0.0007324218750000, 0.0023498535156250,
+0.0003967285156250, 0.0028076171875000, 0.0005493164062500, 0.0024414062500000,
+0.0001831054687500, 0.0031127929687500, -0.0001220703125000, 0.0030822753906250,
+0.0000000000000000, 0.0029907226562500, -0.0000305175781250, 0.0032958984375000,
+-0.0003356933593750, 0.0032348632812500, 0.0004272460937500, 0.0030822753906250,
+0.0000000000000000, 0.0032043457031250, 0.0004882812500000, 0.0030822753906250,
+0.0013732910156250, 0.0028381347656250, 0.0007019042968750, 0.0028991699218750,
+0.0021972656250000, 0.0028381347656250, 0.0021972656250000, 0.0027465820312500,
+0.0022888183593750, 0.0027465820312500, 0.0031127929687500, 0.0026855468750000,
+0.0028991699218750, 0.0026855468750000, 0.0032348632812500, 0.0024108886718750,
+0.0034179687500000, 0.0024719238281250, 0.0033569335937500, 0.0022888183593750,
+0.0034484863281250, 0.0018920898437500, 0.0036315917968750, 0.0018005371093750,
+0.0038146972656250, 0.0017089843750000, 0.0037536621093750, 0.0015869140625000,
+0.0037841796875000, 0.0016479492187500, 0.0039672851562500, 0.0016784667968750,
+0.0039367675781250, 0.0014648437500000, 0.0038757324218750, 0.0019531250000000,
+0.0039062500000000, 0.0017089843750000, 0.0037536621093750, 0.0019531250000000,
+0.0036010742187500, 0.0025634765625000, 0.0037231445312500, 0.0021362304687500,
+0.0033264160156250, 0.0029907226562500, 0.0034790039062500, 0.0029907226562500,
+0.0032348632812500, 0.0029907226562500, 0.0030822753906250, 0.0035095214843750,
+0.0034790039062500, 0.0033569335937500, 0.0024719238281250, 0.0034790039062500,
+0.0028991699218750, 0.0036315917968750, 0.0024108886718750, 0.0034179687500000,
+0.0014343261718750, 0.0032348632812500, 0.0022277832031250, 0.0034179687500000,
+0.0010375976562500, 0.0028076171875000, 0.0010070800781250, 0.0029296875000000,
+0.0010375976562500, 0.0028076171875000, 0.0004272460937500, 0.0024108886718750,
+0.0005798339843750, 0.0026245117187500, 0.0005798339843750, 0.0022583007812500,
+0.0003356933593750, 0.0022583007812500, 0.0006103515625000, 0.0022277832031250,
+0.0008239746093750, 0.0019836425781250, 0.0004882812500000, 0.0020751953125000,
+0.0014038085937500, 0.0019226074218750, 0.0011901855468750, 0.0018615722656250,
+0.0014343261718750, 0.0018310546875000, 0.0022583007812500, 0.0017089843750000,
+0.0017700195312500, 0.0017089843750000, 0.0021057128906250, 0.0014953613281250,
+0.0022583007812500, 0.0014343261718750, 0.0020446777343750, 0.0014953613281250,
+0.0020141601562500, 0.0014343261718750, 0.0020751953125000, 0.0013427734375000,
+0.0016784667968750, 0.0014343261718750, 0.0017700195312500, 0.0013427734375000,
+0.0017089843750000, 0.0013122558593750, 0.0014343261718750, 0.0013427734375000,
+0.0016784667968750, 0.0012817382812500, 0.0013427734375000, 0.0011901855468750,
+0.0014648437500000, 0.0011291503906250, 0.0015258789062500, 0.0010375976562500,
+0.0014343261718750, 0.0009765625000000, 0.0015258789062500, 0.0009765625000000,
+0.0014038085937500, 0.0007019042968750, 0.0014648437500000, 0.0006713867187500,
+0.0015563964843750, 0.0006103515625000, 0.0015258789062500, 0.0003051757812500,
+0.0015563964843750, 0.0003356933593750, 0.0016479492187500, 0.0005798339843750,
+0.0016784667968750, 0.0003356933593750, 0.0017089843750000, 0.0006408691406250,
+0.0017089843750000, 0.0010986328125000, 0.0017700195312500, 0.0007934570312500,
+0.0021057128906250, 0.0013122558593750, 0.0019226074218750, 0.0014343261718750,
+0.0022583007812500, 0.0013122558593750, 0.0026550292968750, 0.0015563964843750,
+0.0022583007812500, 0.0015258789062500, 0.0031127929687500, 0.0014038085937500,
+0.0029602050781250, 0.0014343261718750, 0.0030212402343750, 0.0013732910156250,
+0.0034484863281250, 0.0012817382812500, 0.0030212402343750, 0.0013427734375000,
+0.0035705566406250, 0.0011596679687500, 0.0032958984375000, 0.0011291503906250,
+0.0033874511718750, 0.0010986328125000, 0.0038146972656250, 0.0010070800781250,
+0.0034790039062500, 0.0010070800781250, 0.0038146972656250, 0.0009765625000000,
+0.0038146972656250, 0.0009155273437500, 0.0036010742187500, 0.0008239746093750,
+0.0036621093750000, 0.0007934570312500, 0.0036621093750000, 0.0007324218750000,
+0.0032348632812500, 0.0005187988281250, 0.0033264160156250, 0.0005798339843750,
+0.0030517578125000, 0.0004577636718750, 0.0025939941406250, 0.0002136230468750,
+0.0027465820312500, 0.0001831054687500, 0.0022583007812500, 0.0000000000000000,
+0.0021362304687500, -0.0000915527343750, 0.0021057128906250, -0.0002441406250000,
+0.0018005371093750, -0.0004272460937500, 0.0018005371093750, -0.0004272460937500,
+0.0018005371093750, -0.0006408691406250, 0.0016174316406250, -0.0006408691406250,
+0.0015869140625000, -0.0007934570312500, 0.0015563964843750, -0.0010070800781250,
+0.0014038085937500, -0.0008850097656250, 0.0014953613281250, -0.0012817382812500,
+0.0013427734375000, -0.0011291503906250, 0.0012817382812500, -0.0012817382812500,
+0.0013732910156250, -0.0015563964843750, 0.0011596679687500, -0.0012512207031250,
+0.0013122558593750, -0.0018005371093750, 0.0012817382812500, -0.0016479492187500,
+0.0012817382812500, -0.0017089843750000, 0.0013427734375000, -0.0020751953125000,
+0.0012512207031250, -0.0018310546875000, 0.0013732910156250, -0.0019836425781250,
+0.0014038085937500, -0.0021667480468750, 0.0014343261718750, -0.0018005371093750,
+0.0014343261718750, -0.0016784667968750, 0.0014953613281250, -0.0018615722656250,
+0.0016479492187500, -0.0007934570312500, 0.0016479492187500, -0.0009460449218750,
+0.0017089843750000, -0.0006713867187500, 0.0018310546875000, 0.0002136230468750,
+0.0018615722656250, -0.0001525878906250, 0.0018615722656250, 0.0003967285156250,
+0.0019531250000000, 0.0006103515625000, 0.0018920898437500, 0.0005187988281250,
+0.0018615722656250, 0.0005798339843750, 0.0020446777343750, 0.0007019042968750,
+0.0017700195312500, 0.0003356933593750, 0.0018920898437500, 0.0005187988281250,
+0.0018920898437500, 0.0002746582031250, 0.0017700195312500, -0.0001220703125000,
+0.0019531250000000, 0.0001525878906250, 0.0018615722656250, -0.0002136230468750,
+0.0019226074218750, -0.0003051757812500, 0.0020141601562500, -0.0000915527343750,
+0.0020751953125000, -0.0000305175781250, 0.0020446777343750, -0.0000610351562500,
+0.0021362304687500, 0.0002136230468750, 0.0021972656250000, 0.0003662109375000,
+0.0022888183593750, 0.0003051757812500, 0.0023498535156250, 0.0003662109375000,
+0.0023498535156250, 0.0005187988281250, 0.0025329589843750, 0.0002136230468750,
+0.0024719238281250, 0.0003967285156250, 0.0025939941406250, 0.0003356933593750,
+0.0026855468750000, 0.0000305175781250, 0.0025634765625000, 0.0002746582031250,
+0.0028686523437500, 0.0002136230468750, 0.0028381347656250, 0.0001220703125000,
+0.0028686523437500, 0.0003356933593750, 0.0030822753906250, 0.0004882812500000,
+0.0029907226562500, 0.0003662109375000, 0.0028381347656250, 0.0010070800781250,
+0.0029907226562500, 0.0008850097656250, 0.0026855468750000, 0.0010986328125000,
+0.0023193359375000, 0.0016174316406250, 0.0025329589843750, 0.0015258789062500,
+0.0020141601562500, 0.0018615722656250, 0.0018920898437500, 0.0019836425781250,
+0.0018920898437500, 0.0020751953125000, 0.0017089843750000, 0.0021667480468750,
+0.0017089843750000, 0.0021972656250000, 0.0016479492187500, 0.0021362304687500,
+0.0016174316406250, 0.0021667480468750, 0.0015869140625000, 0.0021972656250000,
+0.0015563964843750, 0.0021362304687500, 0.0015258789062500, 0.0019531250000000,
+0.0015869140625000, 0.0019226074218750, 0.0014343261718750, 0.0019836425781250,
+0.0017395019531250, 0.0016479492187500, 0.0018920898437500, 0.0014038085937500,
+0.0016784667968750, 0.0015563964843750, 0.0026855468750000, 0.0008544921875000,
+0.0024108886718750, 0.0008850097656250, 0.0027770996093750, 0.0007324218750000,
+0.0036621093750000},
+{0.0031738281250000, 0.0009155273437500, 0.0028076171875000, 0.0007629394531250,
+0.0031127929687500, 0.0009460449218750, 0.0024414062500000, 0.0003356933593750,
+0.0025634765625000, 0.0004882812500000, 0.0025024414062500, 0.0002441406250000,
+0.0020751953125000, -0.0002746582031250, 0.0024108886718750, 0.0000000000000000,
+0.0019226074218750, -0.0007019042968750, 0.0019836425781250, -0.0006103515625000,
+0.0018615722656250, -0.0007934570312500, 0.0015258789062500, -0.0013122558593750,
+0.0017395019531250, -0.0011291503906250, 0.0012817382812500, -0.0016174316406250,
+0.0012817382812500, -0.0017089843750000, 0.0011901855468750, -0.0016174316406250,
+0.0009460449218750, -0.0017700195312500, 0.0011291503906250, -0.0017700195312500,
+0.0010070800781250, -0.0015258789062500, 0.0010070800781250, -0.0016174316406250,
+0.0011291503906250, -0.0014038085937500, 0.0012207031250000, -0.0010986328125000,
+0.0013122558593750, -0.0013122558593750, 0.0014038085937500, -0.0006408691406250,
+0.0015563964843750, -0.0008544921875000, 0.0015563964843750, -0.0003051757812500,
+0.0016174316406250, 0.0004882812500000, 0.0018005371093750, -0.0000305175781250,
+0.0016784667968750, 0.0015258789062500, 0.0018005371093750, 0.0014648437500000,
+0.0018615722656250, 0.0018310546875000, 0.0017700195312500, 0.0028686523437500,
+0.0019531250000000, 0.0025939941406250, 0.0019226074218750, 0.0032958984375000,
+0.0020141601562500, 0.0036010742187500, 0.0020446777343750, 0.0034484863281250,
+0.0020141601562500, 0.0037536621093750, 0.0021362304687500, 0.0039062500000000,
+0.0020446777343750, 0.0031127929687500, 0.0020751953125000, 0.0035095214843750,
+0.0021667480468750, 0.0030212402343750, 0.0021667480468750, 0.0021362304687500,
+0.0021972656250000, 0.0027465820312500, 0.0023193359375000, 0.0014343261718750,
+0.0022583007812500, 0.0014343261718750, 0.0023193359375000, 0.0014343261718750,
+0.0024414062500000, 0.0007629394531250, 0.0023803710937500, 0.0010070800781250,
+0.0025634765625000, 0.0009460449218750, 0.0025634765625000, 0.0008544921875000,
+0.0025939941406250, 0.0009765625000000, 0.0027160644531250, 0.0010986328125000,
+0.0026245117187500, 0.0010986328125000, 0.0026245117187500, 0.0012512207031250,
+0.0026245117187500, 0.0012817382812500, 0.0025329589843750, 0.0011901855468750,
+0.0024719238281250, 0.0012207031250000, 0.0024719238281250, 0.0011901855468750,
+0.0024719238281250, 0.0010681152343750, 0.0023803710937500, 0.0010070800781250,
+0.0024108886718750, 0.0010986328125000, 0.0025024414062500, 0.0010375976562500,
+0.0024108886718750, 0.0009765625000000, 0.0026550292968750, 0.0013427734375000,
+0.0026550292968750, 0.0012817382812500, 0.0026855468750000, 0.0015258789062500,
+0.0028686523437500, 0.0019531250000000, 0.0028076171875000, 0.0018005371093750,
+0.0029602050781250, 0.0021972656250000, 0.0028991699218750, 0.0023803710937500,
+0.0030212402343750, 0.0023803710937500, 0.0030822753906250, 0.0025329589843750,
+0.0029907226562500, 0.0025634765625000, 0.0035400390625000, 0.0025329589843750,
+0.0034484863281250, 0.0025939941406250, 0.0035400390625000, 0.0025634765625000,
+0.0039367675781250, 0.0025329589843750, 0.0038146972656250, 0.0026245117187500,
+0.0039672851562500, 0.0023803710937500, 0.0041503906250000, 0.0024719238281250,
+0.0038757324218750, 0.0024108886718750, 0.0037536621093750, 0.0022277832031250,
+0.0040283203125000, 0.0022888183593750, 0.0032958984375000, 0.0017089843750000,
+0.0034179687500000, 0.0018310546875000, 0.0031433105468750, 0.0016784667968750,
+0.0025634765625000, 0.0011596679687500, 0.0028991699218750, 0.0013122558593750,
+0.0019836425781250, 0.0012207031250000, 0.0019836425781250, 0.0010375976562500,
+0.0019226074218750, 0.0011596679687500, 0.0010986328125000, 0.0014953613281250,
+0.0016174316406250, 0.0026550292968750, 0.0012817382812500, 0.0022277832031250,
+0.0014953613281250, 0.0023803710937500, 0.0014343261718750, 0.0022583007812500,
+0.0012512207031250, 0.0020141601562500, 0.0013732910156250, 0.0020751953125000,
+0.0014038085937500, 0.0017700195312500, 0.0013427734375000, 0.0019226074218750,
+0.0015258789062500, 0.0019531250000000, 0.0015869140625000, 0.0017700195312500,
+0.0014953613281250, 0.0019531250000000, 0.0019531250000000, 0.0020446777343750,
+0.0018310546875000, 0.0019531250000000, 0.0019836425781250, 0.0021362304687500,
+0.0023193359375000, 0.0025024414062500, 0.0021057128906250, 0.0022583007812500,
+0.0022888183593750, 0.0023193359375000, 0.0023803710937500, 0.0024108886718750,
+0.0022583007812500, 0.0022277832031250, 0.0022888183593750, 0.0019836425781250,
+0.0023498535156250, 0.0021057128906250, 0.0020751953125000, 0.0018005371093750,
+0.0023498535156250, 0.0018920898437500, 0.0020446777343750, 0.0016479492187500,
+0.0017089843750000, 0.0013122558593750, 0.0021362304687500, 0.0017089843750000,
+0.0010070800781250, 0.0015258789062500, 0.0012512207031250, 0.0013427734375000,
+0.0008850097656250, 0.0016784667968750, -0.0000610351562500, 0.0018615722656250,
+0.0004577636718750, 0.0016174316406250, -0.0004882812500000, 0.0025024414062500,
+-0.0005493164062500, 0.0022277832031250, -0.0005798339843750, 0.0025634765625000,
+-0.0010375976562500, 0.0034790039062500, -0.0008239746093750, 0.0028381347656250,
+-0.0012207031250000, 0.0033569335937500, -0.0010681152343750, 0.0036010742187500,
+-0.0013122558593750, 0.0030212402343750, -0.0016479492187500, 0.0028686523437500,
+-0.0013122558593750, 0.0032043457031250, -0.0024414062500000, 0.0019531250000000,
+-0.0021972656250000, 0.0020141601562500, -0.0024414062500000, 0.0017395019531250,
+-0.0033874511718750, 0.0008239746093750, -0.0029602050781250, 0.0010070800781250,
+-0.0034484863281250, 0.0009765625000000, -0.0037231445312500, 0.0003967285156250,
+-0.0032348632812500, 0.0009155273437500, -0.0031738281250000, 0.0014343261718750,
+-0.0035705566406250, 0.0007629394531250, -0.0020141601562500, 0.0020751953125000,
+-0.0025024414062500, 0.0018920898437500, -0.0018005371093750, 0.0018920898437500,
+-0.0003051757812500, 0.0027770996093750, -0.0012207031250000, 0.0025939941406250,
+0.0006103515625000, 0.0024719238281250, 0.0006103515625000, 0.0028076171875000,
+0.0004882812500000, 0.0025329589843750, 0.0014343261718750, 0.0022583007812500,
+0.0010681152343750, 0.0026855468750000, 0.0004882812500000, 0.0020141601562500,
+0.0009155273437500, 0.0022583007812500, 0.0001831054687500, 0.0022583007812500,
+-0.0009460449218750, 0.0019531250000000, -0.0003662109375000, 0.0023498535156250,
+-0.0018310546875000, 0.0019226074218750, -0.0020141601562500, 0.0021057128906250,
+-0.0019836425781250, 0.0021057128906250, -0.0028076171875000, 0.0018615722656250,
+-0.0028686523437500, 0.0022277832031250, -0.0022888183593750, 0.0017395019531250,
+-0.0029602050781250, 0.0018310546875000, -0.0024414062500000, 0.0018310546875000,
+-0.0015869140625000, 0.0015869140625000, -0.0024719238281250, 0.0017395019531250,
+-0.0007934570312500, 0.0018005371093750, -0.0010681152343750, 0.0015563964843750,
+-0.0009765625000000, 0.0017700195312500, 0.0000915527343750, 0.0020446777343750,
+-0.0005187988281250, 0.0018310546875000, 0.0000000000000000, 0.0020446777343750,
+0.0001220703125000, 0.0021972656250000, -0.0001525878906250, 0.0020446777343750,
+-0.0000305175781250, 0.0020141601562500, 0.0000610351562500, 0.0021362304687500,
+-0.0006408691406250, 0.0016784667968750, -0.0003356933593750, 0.0018310546875000,
+-0.0006408691406250, 0.0016784667968750, -0.0013122558593750, 0.0012512207031250,
+-0.0007629394531250, 0.0014648437500000, -0.0015258789062500, 0.0013427734375000,
+-0.0015869140625000, 0.0012207031250000, -0.0014038085937500, 0.0013427734375000,
+-0.0017395019531250, 0.0014343261718750, -0.0016479492187500, 0.0014038085937500,
+-0.0011901855468750, 0.0015258789062500, -0.0014648437500000, 0.0015869140625000,
+-0.0009155273437500, 0.0015869140625000, -0.0002136230468750, 0.0016174316406250,
+-0.0007934570312500, 0.0017089843750000, 0.0008239746093750, 0.0015563964843750,
+0.0003967285156250, 0.0016174316406250, 0.0010375976562500, 0.0016479492187500,
+0.0023803710937500, 0.0015869140625000, 0.0015258789062500, 0.0017700195312500,
+0.0032958984375000, 0.0015869140625000, 0.0032348632812500, 0.0017395019531250,
+0.0033264160156250, 0.0016784667968750, 0.0043640136718750, 0.0015258789062500,
+0.0038757324218750, 0.0017700195312500, 0.0041503906250000, 0.0014038085937500,
+0.0043945312500000, 0.0015563964843750, 0.0038757324218750, 0.0016174316406250,
+0.0035705566406250, 0.0013732910156250, 0.0037536621093750, 0.0016174316406250,
+0.0025634765625000, 0.0013732910156250, 0.0027465820312500, 0.0015869140625000,
+0.0022277832031250, 0.0014953613281250, 0.0011291503906250, 0.0012512207031250,
+0.0016784667968750, 0.0016784667968750, 0.0005493164062500, 0.0013732910156250,
+0.0005798339843750, 0.0014343261718750, 0.0004882812500000, 0.0015563964843750,
+-0.0000610351562500, 0.0014648437500000, 0.0003051757812500, 0.0016479492187500,
+-0.0002746582031250, 0.0016174316406250, -0.0000915527343750, 0.0016784667968750,
+-0.0001220703125000, 0.0016784667968750, -0.0004577636718750, 0.0017089843750000,
+-0.0000915527343750, 0.0018920898437500, -0.0004882812500000, 0.0016174316406250,
+-0.0003662109375000, 0.0018005371093750, -0.0003356933593750, 0.0017089843750000,
+-0.0005187988281250, 0.0014648437500000, -0.0003356933593750, 0.0017700195312500,
+-0.0006103515625000, 0.0013427734375000, -0.0005187988281250, 0.0014953613281250,
+-0.0005798339843750, 0.0015258789062500, -0.0007934570312500, 0.0012817382812500,
+-0.0007324218750000, 0.0015258789062500, -0.0009460449218750, 0.0013732910156250,
+-0.0009155273437500, 0.0014953613281250, -0.0010375976562500, 0.0015869140625000,
+-0.0013427734375000, 0.0015258789062500, -0.0011901855468750, 0.0017089843750000,
+-0.0014953613281250, 0.0018005371093750, -0.0015869140625000, 0.0018920898437500,
+-0.0014648437500000, 0.0020446777343750, -0.0015258789062500, 0.0021667480468750,
+-0.0014648437500000, 0.0021972656250000, -0.0011291503906250, 0.0024414062500000,
+-0.0011596679687500, 0.0025024414062500, -0.0007629394531250, 0.0025024414062500,
+-0.0003051757812500, 0.0025939941406250, -0.0005187988281250, 0.0026550292968750,
+0.0002136230468750, 0.0025939941406250, 0.0002441406250000, 0.0025634765625000,
+0.0005187988281250, 0.0025939941406250, 0.0011291503906250, 0.0025329589843750,
+0.0008544921875000, 0.0024414062500000, 0.0015563964843750, 0.0024719238281250,
+0.0016174316406250, 0.0023803710937500, 0.0016174316406250, 0.0023193359375000,
+0.0020446777343750, 0.0023498535156250, 0.0020141601562500, 0.0022888183593750,
+0.0017395019531250, 0.0021972656250000, 0.0021057128906250, 0.0021667480468750,
+0.0018005371093750, 0.0021972656250000, 0.0012817382812500, 0.0020751953125000,
+0.0017395019531250, 0.0020751953125000, 0.0011596679687500, 0.0023193359375000,
+0.0011291503906250, 0.0021057128906250, 0.0014038085937500, 0.0023803710937500,
+0.0012817382812500, 0.0027465820312500, 0.0013427734375000, 0.0024414062500000,
+0.0016174316406250, 0.0031738281250000, 0.0017089843750000, 0.0031433105468750,
+0.0015869140625000, 0.0032653808593750, 0.0015258789062500, 0.0038146972656250,
+0.0013732910156250, 0.0037536621093750, 0.0007324218750000, 0.0039062500000000,
+0.0009155273437500, 0.0039978027343750, 0.0007934570312500, 0.0039978027343750,
+0.0003051757812500, 0.0041809082031250, 0.0005187988281250, 0.0043334960937500,
+0.0002441406250000},
+{0.0024719238281250, 0.0018310546875000, 0.0023193359375000, 0.0017395019531250,
+0.0025939941406250, 0.0016174316406250, 0.0024414062500000, 0.0015869140625000,
+0.0022583007812500, 0.0015869140625000, 0.0025329589843750, 0.0014648437500000,
+0.0019531250000000, 0.0015258789062500, 0.0021362304687500, 0.0014648437500000,
+0.0018615722656250, 0.0014648437500000, 0.0013732910156250, 0.0014953613281250,
+0.0017089843750000, 0.0013732910156250, 0.0007019042968750, 0.0016479492187500,
+0.0009155273437500, 0.0014343261718750, 0.0004272460937500, 0.0016479492187500,
+-0.0004882812500000, 0.0019226074218750, 0.0000305175781250, 0.0016174316406250,
+-0.0013122558593750, 0.0021972656250000, -0.0011596679687500, 0.0021362304687500,
+-0.0014343261718750, 0.0021667480468750, -0.0023803710937500, 0.0025634765625000,
+-0.0018310546875000, 0.0023803710937500, -0.0025024414062500, 0.0025634765625000,
+-0.0026245117187500, 0.0025939941406250, -0.0023803710937500, 0.0024108886718750,
+-0.0025024414062500, 0.0023193359375000, -0.0024414062500000, 0.0024108886718750,
+-0.0019226074218750, 0.0020141601562500, -0.0020446777343750, 0.0020141601562500,
+-0.0017395019531250, 0.0018615722656250, -0.0012512207031250, 0.0015258789062500,
+-0.0015563964843750, 0.0015869140625000, -0.0007324218750000, 0.0013122558593750,
+-0.0007324218750000, 0.0012817382812500, -0.0006713867187500, 0.0013122558593750,
+-0.0002441406250000, 0.0012512207031250, -0.0004272460937500, 0.0012207031250000,
+-0.0000610351562500, 0.0014038085937500, -0.0000305175781250, 0.0012817382812500,
+-0.0000915527343750, 0.0015258789062500, 0.0001831054687500, 0.0018005371093750,
+0.0000305175781250, 0.0015869140625000, -0.0003051757812500, 0.0021972656250000,
+-0.0001220703125000, 0.0020751953125000, -0.0005493164062500, 0.0022277832031250,
+-0.0010986328125000, 0.0026245117187500, -0.0008239746093750, 0.0024108886718750,
+-0.0017700195312500, 0.0026855468750000, -0.0018310546875000, 0.0028076171875000,
+-0.0017395019531250, 0.0025634765625000, -0.0021972656250000, 0.0026245117187500,
+-0.0022583007812500, 0.0027160644531250, -0.0019836425781250, 0.0024414062500000,
+-0.0022888183593750, 0.0025634765625000, -0.0020141601562500, 0.0024414062500000,
+-0.0016174316406250, 0.0022277832031250, -0.0020141601562500, 0.0024108886718750,
+-0.0010986328125000, 0.0020751953125000, -0.0012512207031250, 0.0021057128906250,
+-0.0010375976562500, 0.0021057128906250, -0.0003356933593750, 0.0018920898437500,
+-0.0006713867187500, 0.0018920898437500, -0.0000610351562500, 0.0020751953125000,
+0.0001525878906250, 0.0018310546875000, -0.0000610351562500, 0.0020751953125000,
+0.0000000000000000, 0.0024414062500000, 0.0001525878906250, 0.0020751953125000,
+-0.0001831054687500, 0.0028381347656250, -0.0002746582031250, 0.0027770996093750,
+-0.0000610351562500, 0.0028381347656250, -0.0001220703125000, 0.0033264160156250,
+-0.0002441406250000, 0.0031127929687500, 0.0007019042968750, 0.0034179687500000,
+0.0004882812500000, 0.0034790039062500, 0.0007324218750000, 0.0033874511718750,
+0.0016479492187500, 0.0036010742187500, 0.0014343261718750, 0.0036621093750000,
+0.0017089843750000, 0.0033264160156250, 0.0017089843750000, 0.0035705566406250,
+0.0016784667968750, 0.0034179687500000, 0.0018005371093750, 0.0031127929687500,
+0.0018310546875000, 0.0034179687500000, 0.0017089843750000, 0.0027770996093750,
+0.0018920898437500, 0.0029907226562500, 0.0017395019531250, 0.0027770996093750,
+0.0014648437500000, 0.0023498535156250, 0.0017395019531250, 0.0027160644531250,
+0.0014343261718750, 0.0019226074218750, 0.0012817382812500, 0.0021362304687500,
+0.0014648437500000, 0.0018310546875000, 0.0014648437500000, 0.0011901855468750,
+0.0012512207031250, 0.0016784667968750, 0.0018920898437500, 0.0006408691406250,
+0.0017089843750000, 0.0008239746093750, 0.0018005371093750, 0.0005493164062500,
+0.0023193359375000, -0.0001831054687500, 0.0020446777343750, 0.0002746582031250,
+0.0024719238281250, -0.0004577636718750, 0.0025329589843750, -0.0003662109375000,
+0.0023193359375000, -0.0003662109375000, 0.0024719238281250, -0.0007324218750000,
+0.0025939941406250, -0.0005187988281250, 0.0022888183593750, -0.0005798339843750,
+0.0022583007812500, -0.0005187988281250, 0.0022888183593750, -0.0003967285156250,
+0.0021972656250000, -0.0003051757812500, 0.0022277832031250, -0.0001831054687500,
+0.0021362304687500, 0.0000610351562500, 0.0022277832031250, 0.0000915527343750,
+0.0020751953125000, 0.0002136230468750, 0.0019531250000000, 0.0003662109375000,
+0.0021362304687500, 0.0003356933593750, 0.0016784667968750, 0.0007629394531250,
+0.0017700195312500, 0.0006408691406250, 0.0017395019531250, 0.0008850097656250,
+0.0014648437500000, 0.0013122558593750, 0.0015869140625000, 0.0011596679687500,
+0.0013427734375000, 0.0016479492187500, 0.0013427734375000, 0.0017089843750000,
+0.0014953613281250, 0.0018005371093750, 0.0014343261718750, 0.0021667480468750,
+0.0013427734375000, 0.0020446777343750, 0.0020141601562500, 0.0020141601562500,
+0.0017395019531250, 0.0021972656250000, 0.0021362304687500, 0.0020141601562500,
+0.0027770996093750, 0.0017395019531250, 0.0023193359375000, 0.0019226074218750,
+0.0033569335937500, 0.0012817382812500, 0.0032043457031250, 0.0013122558593750,
+0.0032043457031250, 0.0011901855468750, 0.0038452148437500, 0.0007324218750000,
+0.0035705566406250, 0.0009460449218750, 0.0036010742187500, 0.0007934570312500,
+0.0037841796875000, 0.0007019042968750, 0.0034179687500000, 0.0007629394531250,
+0.0031127929687500, 0.0008544921875000, 0.0033569335937500, 0.0007934570312500,
+0.0029907226562500, 0.0008239746093750, 0.0028991699218750, 0.0009155273437500,
+0.0028991699218750, 0.0007629394531250, 0.0028076171875000, 0.0006103515625000,
+0.0028381347656250, 0.0007629394531250, 0.0028991699218750, 0.0003356933593750,
+0.0029296875000000, 0.0004272460937500, 0.0028076171875000, 0.0003051757812500,
+0.0027465820312500, -0.0000610351562500, 0.0028381347656250, 0.0001220703125000,
+0.0023498535156250, -0.0003662109375000, 0.0023803710937500, -0.0003051757812500,
+0.0022583007812500, -0.0005187988281250, 0.0018615722656250, -0.0008544921875000,
+0.0020141601562500, -0.0007324218750000, 0.0017395019531250, -0.0012512207031250,
+0.0017089843750000, -0.0011596679687500, 0.0017395019531250, -0.0012817382812500,
+0.0015869140625000, -0.0015869140625000, 0.0015563964843750, -0.0013732910156250,
+0.0017089843750000, -0.0016784667968750, 0.0015563964843750, -0.0016784667968750,
+0.0015258789062500, -0.0016174316406250, 0.0016479492187500, -0.0017089843750000,
+0.0015258789062500, -0.0016479492187500, 0.0015258789062500, -0.0014648437500000,
+0.0014343261718750, -0.0015563964843750, 0.0013732910156250, -0.0012817382812500,
+0.0013122558593750, -0.0009765625000000, 0.0012817382812500, -0.0012207031250000,
+0.0013122558593750, -0.0000610351562500, 0.0011901855468750, -0.0003967285156250,
+0.0012207031250000, 0.0001831054687500, 0.0013122558593750, 0.0011901855468750,
+0.0012207031250000, 0.0005493164062500, 0.0012512207031250, 0.0021057128906250,
+0.0012512207031250, 0.0019531250000000, 0.0012207031250000, 0.0020751953125000,
+0.0012512207031250, 0.0030517578125000, 0.0011901855468750, 0.0026245117187500,
+0.0011901855468750, 0.0026550292968750, 0.0011596679687500, 0.0029907226562500,
+0.0011596679687500, 0.0023803710937500, 0.0011596679687500, 0.0019226074218750,
+0.0010681152343750, 0.0022888183593750, 0.0012512207031250, 0.0006713867187500,
+0.0011291503906250, 0.0008544921875000, 0.0012207031250000, 0.0003967285156250,
+0.0013732910156250, -0.0009155273437500, 0.0012817382812500, -0.0004272460937500,
+0.0014343261718750, -0.0011901855468750, 0.0013732910156250, -0.0014038085937500,
+0.0013122558593750, -0.0012817382812500, 0.0013732910156250, -0.0015563964843750,
+0.0013427734375000, -0.0015258789062500, 0.0012512207031250, -0.0015869140625000,
+0.0012817382812500, -0.0015258789062500, 0.0012207031250000, -0.0014953613281250,
+0.0010986328125000, -0.0014953613281250, 0.0011901855468750, -0.0014648437500000,
+0.0011596679687500, -0.0013427734375000, 0.0011291503906250, -0.0013122558593750,
+0.0011901855468750, -0.0014038085937500, 0.0012817382812500, -0.0014953613281250,
+0.0012207031250000, -0.0014038085937500, 0.0014038085937500, -0.0013122558593750,
+0.0013427734375000, -0.0015258789062500, 0.0013732910156250, -0.0012512207031250,
+0.0014648437500000, -0.0010375976562500, 0.0013732910156250, -0.0012512207031250,
+0.0014648437500000, -0.0004272460937500, 0.0014343261718750, -0.0005493164062500,
+0.0014038085937500, -0.0003356933593750, 0.0013427734375000, 0.0003662109375000,
+0.0013427734375000, 0.0000610351562500, 0.0014648437500000, 0.0004882812500000,
+0.0013427734375000, 0.0007019042968750, 0.0015563964843750, 0.0005187988281250,
+0.0017700195312500, 0.0005798339843750, 0.0015563964843750, 0.0007019042968750,
+0.0020751953125000, 0.0000000000000000, 0.0021057128906250, 0.0002136230468750,
+0.0022583007812500, -0.0000305175781250, 0.0025939941406250, -0.0006713867187500,
+0.0024719238281250, -0.0003967285156250, 0.0030822753906250, -0.0006408691406250,
+0.0030212402343750, -0.0008850097656250, 0.0032043457031250, -0.0006408691406250,
+0.0036010742187500, -0.0004882812500000, 0.0033874511718750, -0.0007019042968750,
+0.0039672851562500, -0.0003051757812500, 0.0039367675781250, -0.0004272460937500,
+0.0037841796875000, -0.0005798339843750, 0.0040588378906250, -0.0003662109375000,
+0.0039672851562500, -0.0005187988281250, 0.0034790039062500, -0.0007629394531250,
+0.0036926269531250, -0.0008239746093750, 0.0033264160156250, -0.0009460449218750,
+0.0027770996093750, -0.0012512207031250, 0.0031127929687500, -0.0012207031250000,
+0.0025024414062500, -0.0010070800781250, 0.0024108886718750, -0.0012512207031250,
+0.0024414062500000, -0.0009765625000000, 0.0022888183593750, -0.0006408691406250,
+0.0023498535156250, -0.0009155273437500, 0.0022277832031250, -0.0003356933593750,
+0.0021362304687500, -0.0002441406250000, 0.0021057128906250, -0.0003967285156250,
+0.0020446777343750, -0.0002441406250000, 0.0021057128906250, -0.0002136230468750,
+0.0018920898437500, -0.0006103515625000, 0.0019836425781250, -0.0006408691406250,
+0.0019226074218750, -0.0005493164062500, 0.0017700195312500, -0.0007629394531250,
+0.0019531250000000, -0.0008239746093750, 0.0016479492187500, -0.0000915527343750,
+0.0017395019531250, -0.0004882812500000, 0.0017395019531250, -0.0002746582031250,
+0.0014343261718750, 0.0010070800781250, 0.0021362304687500, -0.0008544921875000,
+0.0023498535156250, 0.0003051757812500, 0.0025024414062500, -0.0003662109375000,
+0.0023193359375000, 0.0003051757812500, 0.0022277832031250, 0.0012512207031250,
+0.0024108886718750, 0.0008239746093750, 0.0019836425781250, 0.0014343261718750,
+0.0020141601562500, 0.0015563964843750, 0.0018615722656250, 0.0014038085937500,
+0.0014953613281250, 0.0015563964843750, 0.0016174316406250, 0.0014953613281250,
+0.0009460449218750, 0.0014343261718750, 0.0010681152343750, 0.0015258789062500,
+0.0007629394531250, 0.0015258789062500, 0.0001220703125000, 0.0015869140625000,
+0.0004577636718750, 0.0017700195312500, -0.0004577636718750, 0.0015258789062500,
+-0.0004577636718750, 0.0016784667968750, -0.0006103515625000, 0.0014953613281250,
+-0.0011291503906250, 0.0012512207031250, -0.0009155273437500, 0.0016479492187500,
+-0.0009155273437500, 0.0010986328125000, -0.0010375976562500, 0.0010986328125000,
+-0.0008544921875000},
+{0.0010986328125000, -0.0003967285156250, 0.0011596679687500, -0.0003051757812500,
+0.0014343261718750, 0.0000000000000000, 0.0011291503906250, -0.0001220703125000,
+0.0013122558593750, 0.0001525878906250, 0.0011901855468750, 0.0002441406250000,
+0.0011291503906250, 0.0001831054687500, 0.0011291503906250, 0.0003051757812500,
+0.0009765625000000, 0.0002746582031250, 0.0010681152343750, 0.0000305175781250,
+0.0009155273437500, 0.0000305175781250, 0.0010070800781250, 0.0000000000000000,
+0.0012207031250000, -0.0002136230468750, 0.0009460449218750, -0.0001831054687500,
+0.0014953613281250, -0.0000610351562500, 0.0013732910156250, -0.0002441406250000,
+0.0015258789062500, 0.0000305175781250, 0.0020141601562500, 0.0003356933593750,
+0.0017395019531250, 0.0002136230468750, 0.0022888183593750, 0.0006713867187500,
+0.0022888183593750, 0.0007324218750000, 0.0024719238281250, 0.0007324218750000,
+0.0028991699218750, 0.0010070800781250, 0.0027465820312500, 0.0009460449218750,
+0.0032348632812500, 0.0009765625000000, 0.0033264160156250, 0.0008850097656250,
+0.0032043457031250, 0.0008850097656250, 0.0033569335937500, 0.0008239746093750,
+0.0033264160156250, 0.0007019042968750, 0.0029907226562500, 0.0010070800781250,
+0.0031433105468750, 0.0007934570312500, 0.0028076171875000, 0.0009155273437500,
+0.0023193359375000, 0.0012817382812500, 0.0025329589843750, 0.0010070800781250,
+0.0021057128906250, 0.0013427734375000, 0.0018920898437500, 0.0014343261718750,
+0.0018310546875000, 0.0011596679687500, 0.0016784667968750, 0.0010986328125000,
+0.0016174316406250, 0.0011596679687500, 0.0016174316406250, 0.0006103515625000,
+0.0015258789062500, 0.0006103515625000, 0.0014648437500000, 0.0003662109375000,
+0.0014648437500000, -0.0000610351562500, 0.0014038085937500, 0.0001220703125000,
+0.0013732910156250, -0.0003356933593750, 0.0013732910156250, -0.0003356933593750,
+0.0013122558593750, -0.0003662109375000, 0.0012817382812500, -0.0005798339843750,
+0.0013122558593750, -0.0004272460937500, 0.0013122558593750, -0.0007324218750000,
+0.0013732910156250, -0.0005798339843750, 0.0013427734375000, -0.0006713867187500,
+0.0013427734375000, -0.0009155273437500, 0.0014343261718750, -0.0006713867187500,
+0.0016784667968750, -0.0009765625000000, 0.0016784667968750, -0.0010375976562500,
+0.0018310546875000, -0.0007934570312500, 0.0022277832031250, -0.0006408691406250,
+0.0021362304687500, 0.0008239746093750, 0.0011596679687500, 0.0000305175781250,
+0.0016479492187500, 0.0004272460937500, 0.0013427734375000, 0.0003967285156250,
+0.0007019042968750, -0.0000305175781250, 0.0009765625000000, 0.0000915527343750,
+0.0004882812500000, 0.0003967285156250, 0.0004577636718750, 0.0003356933593750,
+0.0002746582031250, 0.0004577636718750, -0.0001220703125000, 0.0006713867187500,
+-0.0000305175781250, 0.0005798339843750, -0.0005493164062500, 0.0003662109375000,
+-0.0005798339843750, 0.0006408691406250, -0.0006713867187500, 0.0003356933593750,
+-0.0010375976562500, -0.0001831054687500, -0.0010681152343750, 0.0001831054687500,
+-0.0008850097656250, -0.0000610351562500, -0.0012512207031250, -0.0003051757812500,
+-0.0010375976562500, 0.0000610351562500, -0.0006103515625000, 0.0003356933593750,
+-0.0010681152343750, 0.0001220703125000, -0.0003967285156250, 0.0006713867187500,
+-0.0005187988281250, 0.0005798339843750, -0.0005493164062500, 0.0005493164062500,
+-0.0001831054687500, 0.0007934570312500, -0.0004882812500000, 0.0006408691406250,
+-0.0000610351562500, 0.0007629394531250, -0.0002136230468750, 0.0006103515625000,
+-0.0001831054687500, 0.0006103515625000, 0.0001831054687500, 0.0007324218750000,
+-0.0000610351562500, 0.0006408691406250, 0.0006408691406250, 0.0007324218750000,
+0.0005187988281250, 0.0006713867187500, 0.0007629394531250, 0.0008544921875000,
+0.0013732910156250, 0.0009765625000000, 0.0011901855468750, 0.0008544921875000,
+0.0018920898437500, 0.0010986328125000, 0.0021057128906250, 0.0011291503906250,
+0.0020446777343750, 0.0010070800781250, 0.0022888183593750, 0.0010681152343750,
+0.0023803710937500, 0.0009765625000000, 0.0018310546875000, 0.0004272460937500,
+0.0022583007812500, 0.0004882812500000, 0.0016784667968750, 0.0003662109375000,
+0.0009765625000000, -0.0000610351562500, 0.0015869140625000, 0.0000610351562500,
+-0.0000305175781250, 0.0000305175781250, 0.0002441406250000, -0.0000610351562500,
+-0.0000305175781250, 0.0003356933593750, -0.0011901855468750, 0.0006103515625000,
+-0.0005187988281250, 0.0004577636718750, -0.0012512207031250, 0.0013122558593750,
+-0.0014648437500000, 0.0012817382812500, -0.0010681152343750, 0.0013732910156250,
+-0.0010681152343750, 0.0020446777343750, -0.0011901855468750, 0.0017700195312500,
+-0.0002746582031250, 0.0016479492187500, -0.0004272460937500, 0.0018005371093750,
+-0.0002441406250000, 0.0013732910156250, 0.0004577636718750, 0.0009765625000000,
+0.0002136230468750, 0.0011291503906250, 0.0003051757812500, 0.0003662109375000,
+0.0006103515625000, 0.0003662109375000, 0.0002441406250000, 0.0003051757812500,
+-0.0000305175781250, -0.0000915527343750, 0.0004272460937500, 0.0001525878906250,
+-0.0006408691406250, 0.0003051757812500, -0.0004882812500000, 0.0001220703125000,
+-0.0005187988281250, 0.0003662109375000, -0.0012817382812500, 0.0006408691406250,
+-0.0009460449218750, 0.0005187988281250, -0.0008239746093750, 0.0008239746093750,
+-0.0011596679687500, 0.0007934570312500, -0.0004882812500000, 0.0005798339843750,
+0.0000610351562500, 0.0006408691406250, -0.0004882812500000, 0.0007629394531250,
+0.0010986328125000, 0.0003356933593750, 0.0008850097656250, 0.0003356933593750,
+0.0011596679687500, 0.0003051757812500, 0.0022888183593750, 0.0000610351562500,
+0.0017089843750000, 0.0001525878906250, 0.0025329589843750, 0.0000915527343750,
+0.0026550292968750, 0.0000610351562500, 0.0024719238281250, 0.0000305175781250,
+0.0027770996093750, 0.0000610351562500, 0.0028076171875000, 0.0000610351562500,
+0.0023193359375000, 0.0000000000000000, 0.0025634765625000, -0.0000305175781250,
+0.0022277832031250, -0.0001220703125000, 0.0016479492187500, -0.0002441406250000,
+0.0020446777343750, -0.0002136230468750, 0.0012817382812500, -0.0002441406250000,
+0.0013732910156250, -0.0003662109375000, 0.0011901855468750, -0.0003051757812500,
+0.0006713867187500, -0.0001831054687500, 0.0009765625000000, -0.0002441406250000,
+0.0002441406250000, -0.0000915527343750, 0.0003967285156250, 0.0000000000000000,
+0.0001525878906250, -0.0000305175781250, -0.0004882812500000, 0.0000000000000000,
+-0.0000915527343750, 0.0001525878906250, -0.0009155273437500, -0.0001220703125000,
+-0.0008239746093750, 0.0001220703125000, -0.0008544921875000, 0.0000305175781250,
+-0.0013732910156250, -0.0002746582031250, -0.0010375976562500, 0.0001831054687500,
+-0.0012207031250000, -0.0002441406250000, -0.0012817382812500, -0.0000915527343750,
+-0.0011291503906250, 0.0000915527343750, -0.0010681152343750, -0.0000305175781250,
+-0.0010681152343750, 0.0003356933593750, -0.0007324218750000, 0.0003356933593750,
+-0.0008544921875000, 0.0005493164062500, -0.0005798339843750, 0.0006713867187500,
+-0.0002136230468750, 0.0007629394531250, -0.0003967285156250, 0.0010070800781250,
+0.0002136230468750, 0.0009765625000000, 0.0001220703125000, 0.0013122558593750,
+0.0003051757812500, 0.0012512207031250, 0.0007324218750000, 0.0011901855468750,
+0.0004882812500000, 0.0017395019531250, 0.0008239746093750, 0.0010681152343750,
+0.0008544921875000, 0.0014648437500000, 0.0007934570312500, 0.0013732910156250,
+0.0009155273437500, 0.0008850097656250, 0.0007629394531250, 0.0014343261718750,
+0.0005493164062500, 0.0007934570312500, 0.0005187988281250, 0.0009155273437500,
+0.0002441406250000, 0.0009765625000000, -0.0000610351562500, 0.0006713867187500,
+-0.0000915527343750, 0.0010070800781250, -0.0007324218750000, 0.0009155273437500,
+-0.0007324218750000, 0.0009765625000000, -0.0010375976562500, 0.0011291503906250,
+-0.0016174316406250, 0.0011596679687500, -0.0014038085937500, 0.0012207031250000,
+-0.0018920898437500, 0.0015563964843750, -0.0020141601562500, 0.0015563964843750,
+-0.0018310546875000, 0.0017089843750000, -0.0018615722656250, 0.0019836425781250,
+-0.0019531250000000, 0.0018310546875000, -0.0014648437500000, 0.0021972656250000,
+-0.0015563964843750, 0.0021362304687500, -0.0013732910156250, 0.0021667480468750,
+-0.0009765625000000, 0.0023498535156250, -0.0010986328125000, 0.0021972656250000,
+-0.0006103515625000, 0.0024719238281250, -0.0006408691406250, 0.0023498535156250,
+-0.0004577636718750, 0.0023193359375000, 0.0000000000000000, 0.0024719238281250,
+-0.0002746582031250, 0.0022277832031250, 0.0004577636718750, 0.0023498535156250,
+0.0004272460937500, 0.0021667480468750, 0.0003967285156250, 0.0020141601562500,
+0.0008850097656250, 0.0019836425781250, 0.0007934570312500, 0.0018310546875000,
+0.0010681152343750, 0.0020141601562500, 0.0010986328125000, 0.0017700195312500,
+0.0010070800781250, 0.0019531250000000, 0.0010070800781250, 0.0022583007812500,
+0.0010070800781250, 0.0018310546875000, 0.0008850097656250, 0.0024719238281250,
+0.0007629394531250, 0.0022888183593750, 0.0008544921875000, 0.0023498535156250,
+0.0008850097656250, 0.0027465820312500, 0.0007629394531250, 0.0024414062500000,
+0.0010681152343750, 0.0027465820312500, 0.0009460449218750, 0.0028381347656250,
+0.0010681152343750, 0.0027160644531250, 0.0013427734375000, 0.0027465820312500,
+0.0010986328125000, 0.0028686523437500, 0.0013427734375000, 0.0024414062500000,
+0.0013427734375000, 0.0026245117187500, 0.0010375976562500, 0.0024108886718750,
+0.0009765625000000, 0.0019531250000000, 0.0009460449218750, 0.0023193359375000,
+0.0003967285156250, 0.0016479492187500, 0.0003356933593750, 0.0016784667968750,
+0.0001831054687500, 0.0016784667968750, -0.0001525878906250, 0.0012512207031250,
+0.0000915527343750, 0.0013427734375000, 0.0003662109375000, 0.0011596679687500,
+0.0002136230468750, 0.0011291503906250, 0.0003356933593750, 0.0010070800781250,
+0.0006103515625000, 0.0007934570312500, 0.0004577636718750, 0.0008544921875000,
+0.0006713867187500, 0.0006713867187500, 0.0005798339843750, 0.0005493164062500,
+0.0006408691406250, 0.0007019042968750, 0.0008544921875000, 0.0007934570312500,
+0.0006713867187500, 0.0006408691406250, 0.0011596679687500, 0.0012207031250000,
+0.0010375976562500, 0.0010681152343750, 0.0012207031250000, 0.0013427734375000,
+0.0016174316406250, 0.0019226074218750, 0.0013732910156250, 0.0016174316406250,
+0.0017395019531250, 0.0022583007812500, 0.0015563964843750, 0.0023498535156250,
+0.0014343261718750, 0.0023193359375000, 0.0015869140625000, 0.0025634765625000,
+0.0014038085937500, 0.0025939941406250, 0.0012817382812500, 0.0023193359375000,
+0.0011291503906250, 0.0024108886718750, 0.0010070800781250, 0.0022277832031250,
+0.0008544921875000, 0.0019226074218750, 0.0007324218750000, 0.0020446777343750,
+0.0008544921875000, 0.0017395019531250, 0.0006408691406250, 0.0016479492187500,
+0.0007324218750000, 0.0016174316406250, 0.0008239746093750, 0.0014343261718750,
+0.0004882812500000, 0.0013732910156250, 0.0010986328125000, 0.0011901855468750,
+0.0007629394531250, 0.0011291503906250, 0.0010375976562500, 0.0010070800781250,
+0.0016784667968750, 0.0008850097656250, 0.0010375976562500, 0.0008850097656250,
+0.0018615722656250, 0.0003967285156250, 0.0017089843750000, 0.0004272460937500,
+0.0017089843750000, 0.0002136230468750, 0.0021057128906250, -0.0001831054687500,
+0.0018005371093750},
+{0.0000915527343750, 0.0021667480468750, 0.0005187988281250, 0.0024414062500000,
+0.0000610351562500, 0.0017700195312500, -0.0000915527343750, 0.0019531250000000,
+0.0001525878906250, 0.0018310546875000, 0.0001220703125000, 0.0014038085937500,
+0.0000915527343750, 0.0017395019531250, 0.0006103515625000, 0.0013732910156250,
+0.0004272460937500, 0.0014648437500000, 0.0006408691406250, 0.0015258789062500,
+0.0011291503906250, 0.0013122558593750, 0.0008239746093750, 0.0016174316406250,
+0.0013427734375000, 0.0015258789062500, 0.0013122558593750, 0.0015869140625000,
+0.0012512207031250, 0.0017700195312500, 0.0014953613281250, 0.0018920898437500,
+0.0012817382812500, 0.0019836425781250, 0.0013427734375000, 0.0022583007812500,
+0.0013427734375000, 0.0022583007812500, 0.0011291503906250, 0.0024414062500000,
+0.0010681152343750, 0.0026550292968750, 0.0011596679687500, 0.0026245117187500,
+0.0004882812500000, 0.0027160644531250, 0.0006408691406250, 0.0028686523437500,
+0.0003356933593750, 0.0028076171875000, -0.0003356933593750, 0.0027160644531250,
+-0.0000305175781250, 0.0028991699218750, -0.0006408691406250, 0.0024719238281250,
+-0.0007934570312500, 0.0026245117187500, -0.0006408691406250, 0.0023803710937500,
+-0.0007324218750000, 0.0019836425781250, -0.0007934570312500, 0.0022583007812500,
+-0.0002136230468750, 0.0016174316406250, -0.0003356933593750, 0.0016479492187500,
+-0.0001831054687500, 0.0015563964843750, 0.0003356933593750, 0.0011291503906250,
+0.0000610351562500, 0.0013427734375000, 0.0003051757812500, 0.0010681152343750,
+0.0003356933593750, 0.0010375976562500, 0.0003051757812500, 0.0009765625000000,
+0.0003356933593750, 0.0007934570312500, 0.0003356933593750, 0.0008850097656250,
+0.0001831054687500, 0.0008850097656250, 0.0001220703125000, 0.0008239746093750,
+0.0002441406250000, 0.0008850097656250, 0.0002441406250000, 0.0009155273437500,
+0.0001220703125000, 0.0008544921875000, 0.0003662109375000, 0.0012817382812500,
+0.0004577636718750, 0.0011901855468750, 0.0003967285156250, 0.0013427734375000,
+0.0004272460937500, 0.0017089843750000, 0.0004882812500000, 0.0015869140625000,
+0.0004882812500000, 0.0018615722656250, 0.0003662109375000, 0.0020141601562500,
+0.0006408691406250, 0.0018310546875000, 0.0009765625000000, 0.0018920898437500,
+0.0007019042968750, 0.0021057128906250, 0.0014648437500000, 0.0014953613281250,
+0.0013732910156250, 0.0016784667968750, 0.0016174316406250, 0.0014648437500000,
+0.0021972656250000, 0.0009765625000000, 0.0018005371093750, 0.0013122558593750,
+0.0024719238281250, 0.0007629394531250, 0.0024414062500000, 0.0007934570312500,
+0.0023498535156250, 0.0007629394531250, 0.0027160644531250, 0.0005187988281250,
+0.0026245117187500, 0.0007629394531250, 0.0024414062500000, 0.0005187988281250,
+0.0025024414062500, 0.0006713867187500, 0.0022583007812500, 0.0006103515625000,
+0.0019836425781250, 0.0003967285156250, 0.0021667480468750, 0.0005493164062500,
+0.0015563964843750, 0.0001220703125000, 0.0015258789062500, 0.0001525878906250,
+0.0015563964843750, 0.0000305175781250, 0.0012207031250000, -0.0003662109375000,
+0.0013122558593750, -0.0001525878906250, 0.0015258789062500, -0.0005493164062500,
+0.0014953613281250, -0.0005798339843750, 0.0014953613281250, -0.0006103515625000,
+0.0016174316406250, -0.0007324218750000, 0.0016784667968750, -0.0007019042968750,
+0.0015869140625000, -0.0005493164062500, 0.0016174316406250, -0.0007019042968750,
+0.0015563964843750, -0.0005187988281250, 0.0014038085937500, -0.0002746582031250,
+0.0013732910156250, -0.0004272460937500, 0.0011901855468750, 0.0000610351562500,
+0.0011901855468750, -0.0000305175781250, 0.0009765625000000, 0.0000915527343750,
+0.0007019042968750, 0.0005187988281250, 0.0007324218750000, 0.0003356933593750,
+0.0003051757812500, 0.0005798339843750, 0.0003662109375000, 0.0006713867187500,
+0.0001220703125000, 0.0006713867187500, -0.0002441406250000, 0.0007629394531250,
+-0.0000305175781250, 0.0007934570312500, -0.0006408691406250, 0.0006408691406250,
+-0.0005493164062500, 0.0007629394531250, -0.0006408691406250, 0.0007324218750000,
+-0.0010070800781250, 0.0006408691406250, -0.0007019042968750, 0.0007019042968750,
+-0.0009460449218750, 0.0004882812500000, -0.0008850097656250, 0.0005798339843750,
+-0.0007934570312500, 0.0006103515625000, -0.0008544921875000, 0.0006408691406250,
+-0.0007324218750000, 0.0007019042968750, -0.0005187988281250, 0.0008239746093750,
+-0.0006103515625000, 0.0008239746093750, -0.0003051757812500, 0.0009460449218750,
+0.0000610351562500, 0.0010986328125000, 0.0000000000000000, 0.0010986328125000,
+0.0003356933593750, 0.0013427734375000, 0.0005798339843750, 0.0013427734375000,
+0.0004882812500000, 0.0014343261718750, 0.0004882812500000, 0.0016479492187500,
+0.0007019042968750, 0.0015563964843750, 0.0003967285156250, 0.0018005371093750,
+0.0006103515625000, 0.0018615722656250, 0.0006103515625000, 0.0017395019531250,
+0.0003967285156250, 0.0018310546875000, 0.0007629394531250, 0.0018920898437500,
+0.0005187988281250, 0.0013732910156250, 0.0007019042968750, 0.0014343261718750,
+0.0007934570312500, 0.0012512207031250, 0.0007324218750000, 0.0007934570312500,
+0.0009460449218750, 0.0009765625000000, 0.0008239746093750, 0.0007324218750000,
+0.0008850097656250, 0.0005187988281250, 0.0009155273437500, 0.0006713867187500,
+0.0007934570312500, 0.0007019042968750, 0.0009460449218750, 0.0005493164062500,
+0.0008239746093750, 0.0008544921875000, 0.0008239746093750, 0.0008544921875000,
+0.0008850097656250, 0.0008239746093750, 0.0008544921875000, 0.0009155273437500,
+0.0008544921875000, 0.0008239746093750, 0.0008850097656250, 0.0007324218750000,
+0.0009155273437500, 0.0008544921875000, 0.0008239746093750, 0.0007019042968750,
+0.0007629394531250, 0.0005187988281250, 0.0007629394531250, 0.0005798339843750,
+0.0006713867187500, 0.0004272460937500, 0.0006408691406250, 0.0002441406250000,
+0.0007019042968750, 0.0004577636718750, 0.0006713867187500, 0.0004577636718750,
+0.0006408691406250, 0.0002441406250000, 0.0008850097656250, 0.0010375976562500,
+0.0007324218750000, 0.0007019042968750, 0.0009155273437500, 0.0011901855468750,
+0.0011291503906250, 0.0020446777343750, 0.0010375976562500, 0.0014343261718750,
+0.0012817382812500, 0.0028686523437500, 0.0013732910156250, 0.0027465820312500,
+0.0012512207031250, 0.0029907226562500, 0.0012207031250000, 0.0039062500000000,
+0.0013732910156250, 0.0034484863281250, 0.0010986328125000, 0.0038452148437500,
+0.0011901855468750, 0.0041198730468750, 0.0010986328125000, 0.0038452148437500,
+0.0008544921875000, 0.0037231445312500, 0.0010986328125000, 0.0039367675781250,
+0.0008850097656250, 0.0034790039062500, 0.0009460449218750, 0.0035705566406250,
+0.0010070800781250, 0.0033874511718750, 0.0009765625000000, 0.0029907226562500,
+0.0010986328125000, 0.0031433105468750, 0.0010681152343750, 0.0026245117187500,
+0.0012207031250000, 0.0026245117187500, 0.0012817382812500, 0.0023803710937500,
+0.0014038085937500, 0.0019226074218750, 0.0016174316406250, 0.0020751953125000,
+0.0014648437500000, 0.0016174316406250, 0.0016174316406250, 0.0015258789062500,
+0.0016784667968750, 0.0015563964843750, 0.0016479492187500, 0.0013427734375000,
+0.0018005371093750, 0.0013122558593750, 0.0016174316406250, 0.0013732910156250,
+0.0016784667968750, 0.0012512207031250, 0.0016479492187500, 0.0012817382812500,
+0.0014343261718750, 0.0014953613281250, 0.0015563964843750, 0.0013122558593750,
+0.0011901855468750, 0.0014648437500000, 0.0012817382812500, 0.0014648437500000,
+0.0012207031250000, 0.0013427734375000, 0.0008544921875000, 0.0013122558593750,
+0.0009155273437500, 0.0012512207031250, 0.0008544921875000, 0.0008850097656250,
+0.0007629394531250, 0.0009460449218750, 0.0007934570312500, 0.0008850097656250,
+0.0007934570312500, 0.0006408691406250, 0.0006408691406250, 0.0007324218750000,
+0.0009155273437500, 0.0006103515625000, 0.0008544921875000, 0.0006103515625000,
+0.0007934570312500, 0.0004577636718750, 0.0008850097656250, 0.0003662109375000,
+0.0008239746093750, 0.0004577636718750, 0.0009155273437500, -0.0002136230468750,
+0.0008850097656250, -0.0000305175781250, 0.0008850097656250, -0.0003051757812500,
+0.0008850097656250, -0.0009155273437500, 0.0009155273437500, -0.0004882812500000,
+0.0010070800781250, -0.0014038085937500, 0.0009765625000000, -0.0012512207031250,
+0.0009460449218750, -0.0013427734375000, 0.0010375976562500, -0.0019531250000000,
+0.0011596679687500, -0.0016784667968750, 0.0011901855468750, -0.0019226074218750,
+0.0013122558593750, -0.0021362304687500, 0.0013427734375000, -0.0017395019531250,
+0.0014038085937500, -0.0015869140625000, 0.0016479492187500, -0.0018920898437500,
+0.0013732910156250, -0.0010070800781250, 0.0016479492187500, -0.0010681152343750,
+0.0015563964843750, -0.0010375976562500, 0.0012817382812500, -0.0004882812500000,
+0.0016479492187500, -0.0006103515625000, 0.0009765625000000, -0.0005187988281250,
+0.0011901855468750, -0.0004577636718750, 0.0011291503906250, -0.0004577636718750,
+0.0002746582031250, -0.0004577636718750, 0.0003051757812500, -0.0023193359375000,
+-0.0004882812500000, -0.0019531250000000, -0.0000610351562500, -0.0023498535156250,
+-0.0003967285156250, -0.0017700195312500, -0.0009155273437500, -0.0012207031250000,
+-0.0005187988281250, -0.0015563964843750, -0.0010986328125000, -0.0006408691406250,
+-0.0010375976562500, -0.0005493164062500, -0.0009460449218750, -0.0005493164062500,
+-0.0011596679687500, 0.0000305175781250, -0.0009765625000000, -0.0000305175781250,
+-0.0008850097656250, -0.0002136230468750, -0.0009460449218750, -0.0001220703125000,
+-0.0007324218750000, 0.0000305175781250, -0.0006408691406250, 0.0000305175781250,
+-0.0006713867187500, 0.0000000000000000, -0.0001525878906250, 0.0002136230468750,
+-0.0002441406250000, 0.0002746582031250, -0.0000305175781250, 0.0001525878906250,
+0.0004882812500000, 0.0001831054687500, 0.0003356933593750, 0.0002746582031250,
+0.0007629394531250, 0.0003356933593750, 0.0009155273437500, 0.0001831054687500,
+0.0008239746093750, 0.0002441406250000, 0.0009765625000000, 0.0003662109375000,
+0.0011901855468750, 0.0003051757812500, 0.0008850097656250, 0.0007324218750000,
+0.0010681152343750, 0.0004882812500000, 0.0009765625000000, 0.0007629394531250,
+0.0007629394531250, 0.0012207031250000, 0.0010070800781250, 0.0008239746093750,
+0.0004882812500000, 0.0017395019531250, 0.0006713867187500, 0.0015869140625000,
+0.0005187988281250, 0.0016174316406250, 0.0000610351562500, 0.0022888183593750,
+0.0004882812500000, 0.0020141601562500, -0.0001831054687500, 0.0020751953125000,
+-0.0002136230468750, 0.0023803710937500, 0.0000000000000000, 0.0021362304687500,
+-0.0002746582031250, 0.0019226074218750, -0.0001525878906250, 0.0023498535156250,
+0.0005493164062500, 0.0020446777343750, 0.0002136230468750, 0.0019531250000000,
+0.0007019042968750, 0.0020446777343750, 0.0015258789062500, 0.0019226074218750,
+0.0009765625000000, 0.0020141601562500, 0.0020751953125000, 0.0021972656250000,
+0.0021972656250000, 0.0020446777343750, 0.0019531250000000, 0.0019836425781250,
+0.0024414062500000, 0.0022277832031250, 0.0024414062500000, 0.0022583007812500,
+0.0016784667968750, 0.0018005371093750, 0.0020751953125000, 0.0018920898437500,
+0.0017089843750000, 0.0018005371093750, 0.0008239746093750, 0.0013122558593750,
+0.0014038085937500, 0.0014038085937500, 0.0007629394531250, 0.0014343261718750,
+0.0005493164062500},
+{0.0002136230468750, 0.0002441406250000, 0.0003356933593750, 0.0002136230468750,
+0.0005187988281250, 0.0002136230468750, 0.0004882812500000, 0.0005798339843750,
+0.0007019042968750, 0.0004882812500000, 0.0007934570312500, 0.0003356933593750,
+0.0009155273437500, 0.0006713867187500, 0.0009765625000000, 0.0002136230468750,
+0.0010375976562500, -0.0002136230468750, 0.0011596679687500, 0.0002746582031250,
+0.0011291503906250, -0.0010070800781250, 0.0012817382812500, -0.0007629394531250,
+0.0011901855468750, -0.0010375976562500, 0.0010375976562500, -0.0020446777343750,
+0.0011901855468750, -0.0015258789062500, 0.0008239746093750, -0.0021667480468750,
+0.0009155273437500, -0.0023498535156250, 0.0008239746093750, -0.0022583007812500,
+0.0006103515625000, -0.0023803710937500, 0.0007629394531250, -0.0024719238281250,
+0.0004577636718750, -0.0023803710937500, 0.0005493164062500, -0.0024414062500000,
+0.0004882812500000, -0.0024108886718750, -0.0000305175781250, -0.0023193359375000,
+-0.0009155273437500, -0.0001831054687500, -0.0004882812500000, -0.0012817382812500,
+-0.0008544921875000, -0.0008239746093750, -0.0004577636718750, -0.0013427734375000,
+0.0000915527343750, -0.0022583007812500, -0.0002746582031250, -0.0018005371093750,
+0.0003662109375000, -0.0020141601562500, 0.0004577636718750, -0.0024108886718750,
+0.0004882812500000, -0.0016784667968750, 0.0008239746093750, -0.0011291503906250,
+0.0007629394531250, -0.0017089843750000, 0.0008850097656250, -0.0000305175781250,
+0.0010986328125000, -0.0000305175781250, 0.0010070800781250, 0.0000305175781250,
+0.0010375976562500, 0.0009155273437500, 0.0012817382812500, 0.0006713867187500,
+0.0008239746093750, 0.0010986328125000, 0.0011596679687500, 0.0010986328125000,
+0.0008850097656250, 0.0010070800781250, 0.0003051757812500, 0.0011596679687500,
+0.0008850097656250, 0.0011596679687500, -0.0000915527343750, 0.0008850097656250,
+0.0000000000000000, 0.0009460449218750, 0.0000000000000000, 0.0009460449218750,
+-0.0005187988281250, 0.0008239746093750, -0.0003051757812500, 0.0009155273437500,
+-0.0004272460937500, 0.0007324218750000, -0.0004882812500000, 0.0008544921875000,
+-0.0004882812500000, 0.0008544921875000, -0.0004882812500000, 0.0007324218750000,
+-0.0004882812500000, 0.0008544921875000, -0.0004882812500000, 0.0009765625000000,
+-0.0004577636718750, 0.0008544921875000, -0.0002136230468750, 0.0010375976562500,
+-0.0001525878906250, 0.0012207031250000, -0.0001831054687500, 0.0010070800781250,
+0.0007019042968750, 0.0016784667968750, 0.0004882812500000, 0.0014953613281250,
+0.0010070800781250, 0.0017089843750000, 0.0019531250000000, 0.0021972656250000,
+0.0014953613281250, 0.0018310546875000, 0.0023803710937500, 0.0021362304687500,
+0.0026245117187500, 0.0022583007812500, 0.0023193359375000, 0.0021667480468750,
+0.0025634765625000, 0.0021667480468750, 0.0027160644531250, 0.0020446777343750,
+0.0017089843750000, 0.0016479492187500, 0.0020751953125000, 0.0017089843750000,
+0.0016784667968750, 0.0016784667968750, 0.0007934570312500, 0.0014038085937500,
+0.0013732910156250, 0.0013427734375000, 0.0008850097656250, 0.0017089843750000,
+0.0006103515625000, 0.0016174316406250, 0.0010681152343750, 0.0018310546875000,
+0.0012512207031250, 0.0022277832031250, 0.0010375976562500, 0.0021667480468750,
+0.0019531250000000, 0.0023498535156250, 0.0019531250000000, 0.0025024414062500,
+0.0021057128906250, 0.0025329589843750, 0.0026550292968750, 0.0025939941406250,
+0.0025024414062500, 0.0025939941406250, 0.0028076171875000, 0.0026550292968750,
+0.0028686523437500, 0.0024108886718750, 0.0028686523437500, 0.0025329589843750,
+0.0030212402343750, 0.0026550292968750, 0.0030212402343750, 0.0023498535156250,
+0.0031738281250000, 0.0032348632812500, 0.0031433105468750, 0.0028381347656250,
+0.0032958984375000, 0.0029907226562500, 0.0034484863281250, 0.0037841796875000,
+0.0034179687500000, 0.0033264160156250, 0.0037231445312500, 0.0036926269531250,
+0.0037841796875000, 0.0038452148437500, 0.0036621093750000, 0.0035705566406250,
+0.0038452148437500, 0.0035095214843750, 0.0039672851562500, 0.0036926269531250,
+0.0033874511718750, 0.0031127929687500, 0.0036926269531250, 0.0032043457031250,
+0.0034179687500000, 0.0031433105468750, 0.0028076171875000, 0.0028381347656250,
+0.0031738281250000, 0.0031738281250000, 0.0024414062500000, 0.0029296875000000,
+0.0025024414062500, 0.0029907226562500, 0.0022583007812500, 0.0031433105468750,
+0.0016784667968750, 0.0031127929687500, 0.0019531250000000, 0.0031433105468750,
+0.0014038085937500, 0.0033569335937500, 0.0012817382812500, 0.0032958984375000,
+0.0014343261718750, 0.0034179687500000, 0.0012207031250000, 0.0035705566406250,
+0.0009765625000000, 0.0034484863281250, 0.0014648437500000, 0.0036010742187500,
+0.0012512207031250, 0.0035705566406250, 0.0012817382812500, 0.0035095214843750,
+0.0016479492187500, 0.0034790039062500, 0.0013427734375000, 0.0034484863281250,
+0.0013427734375000, 0.0031738281250000, 0.0014648437500000, 0.0032348632812500,
+0.0011901855468750, 0.0030517578125000, 0.0009155273437500, 0.0028381347656250,
+0.0011291503906250, 0.0029907226562500, 0.0005493164062500, 0.0025329589843750,
+0.0006408691406250, 0.0025939941406250, 0.0006713867187500, 0.0024414062500000,
+0.0003967285156250, 0.0020446777343750, 0.0006408691406250, 0.0022583007812500,
+0.0006103515625000, 0.0018005371093750, 0.0006103515625000, 0.0018310546875000,
+0.0007019042968750, 0.0018005371093750, 0.0007629394531250, 0.0015563964843750,
+0.0008544921875000, 0.0017089843750000, 0.0006408691406250, 0.0015869140625000,
+0.0007934570312500, 0.0016479492187500, 0.0006103515625000, 0.0016174316406250,
+0.0003356933593750, 0.0015563964843750, 0.0004882812500000, 0.0016784667968750,
+0.0001220703125000, 0.0015258789062500, 0.0000915527343750, 0.0016784667968750,
+0.0001525878906250, 0.0017089843750000, 0.0000610351562500, 0.0015869140625000,
+-0.0000305175781250, 0.0017700195312500, 0.0004272460937500, 0.0017395019531250,
+0.0002136230468750, 0.0018920898437500, 0.0004577636718750, 0.0018615722656250,
+0.0009460449218750, 0.0019226074218750, 0.0007629394531250, 0.0021667480468750,
+0.0013122558593750, 0.0018310546875000, 0.0012817382812500, 0.0020751953125000,
+0.0012817382812500, 0.0019531250000000, 0.0016174316406250, 0.0017089843750000,
+0.0014343261718750, 0.0021667480468750, 0.0016174316406250, 0.0014648437500000,
+0.0015563964843750, 0.0018005371093750, 0.0015563964843750, 0.0016479492187500,
+0.0016174316406250, 0.0010986328125000, 0.0015258789062500, 0.0017089843750000,
+0.0015563964843750, 0.0010986328125000, 0.0015258789062500, 0.0012817382812500,
+0.0014343261718750, 0.0014953613281250, 0.0014343261718750, 0.0012207031250000,
+0.0014038085937500, 0.0014953613281250, 0.0010986328125000, 0.0017089843750000,
+0.0010681152343750, 0.0016479492187500, 0.0007934570312500, 0.0020141601562500,
+0.0004272460937500, 0.0022888183593750, 0.0004577636718750, 0.0022277832031250,
+-0.0003356933593750, 0.0029296875000000, -0.0003662109375000, 0.0027465820312500,
+-0.0006103515625000, 0.0030517578125000, -0.0012207031250000, 0.0036010742187500,
+-0.0011596679687500, 0.0032348632812500, -0.0014648437500000, 0.0037536621093750,
+-0.0016784667968750, 0.0036926269531250, -0.0014343261718750, 0.0036315917968750,
+-0.0013122558593750, 0.0038146972656250, -0.0014953613281250, 0.0035400390625000,
+-0.0007324218750000, 0.0033264160156250, -0.0008239746093750, 0.0034179687500000,
+-0.0006408691406250, 0.0029296875000000, -0.0000915527343750, 0.0025634765625000,
+-0.0003051757812500, 0.0027160644531250, 0.0002136230468750, 0.0017395019531250,
+0.0003051757812500, 0.0018005371093750, 0.0004272460937500, 0.0016479492187500,
+0.0007934570312500, 0.0010070800781250, 0.0006713867187500, 0.0011901855468750,
+0.0010070800781250, 0.0008239746093750, 0.0011596679687500, 0.0008239746093750,
+0.0010681152343750, 0.0008544921875000, 0.0011596679687500, 0.0007324218750000,
+0.0013122558593750, 0.0008239746093750, 0.0010986328125000, 0.0008850097656250,
+0.0012207031250000, 0.0009155273437500, 0.0012207031250000, 0.0009765625000000,
+0.0010986328125000, 0.0010986328125000, 0.0011901855468750, 0.0010986328125000,
+0.0012207031250000, 0.0012512207031250, 0.0011901855468750, 0.0013427734375000,
+0.0012512207031250, 0.0013427734375000, 0.0013122558593750, 0.0014343261718750,
+0.0010986328125000, 0.0016174316406250, 0.0009460449218750, 0.0014648437500000,
+0.0010070800781250, 0.0014953613281250, 0.0009765625000000, 0.0014038085937500,
+0.0007629394531250, 0.0012207031250000, 0.0007934570312500, 0.0011901855468750,
+0.0009155273437500, 0.0008239746093750, 0.0008239746093750, 0.0007934570312500,
+0.0008850097656250, 0.0006408691406250, 0.0010681152343750, 0.0004272460937500,
+0.0009460449218750, 0.0003967285156250, 0.0006408691406250, 0.0002746582031250,
+0.0008850097656250, 0.0001220703125000, 0.0004577636718750, 0.0002441406250000,
+-0.0000915527343750, 0.0002136230468750, 0.0003051757812500, 0.0001220703125000,
+-0.0004882812500000, 0.0003356933593750, -0.0005493164062500, 0.0003662109375000,
+-0.0003051757812500, 0.0003967285156250, -0.0005798339843750, 0.0005187988281250,
+-0.0005493164062500, 0.0004882812500000, 0.0000610351562500, 0.0005798339843750,
+-0.0002746582031250, 0.0005493164062500, 0.0002136230468750, 0.0005187988281250,
+0.0008850097656250, 0.0005187988281250, 0.0002136230468750, 0.0005187988281250,
+0.0017089843750000, 0.0006408691406250, 0.0012817382812500, 0.0006408691406250,
+0.0016174316406250, 0.0007019042968750, 0.0026855468750000, 0.0007934570312500,
+0.0018615722656250, 0.0007629394531250, 0.0029296875000000, 0.0009460449218750,
+0.0028991699218750, 0.0009155273437500, 0.0026245117187500, 0.0010375976562500,
+0.0030212402343750, 0.0012207031250000, 0.0028686523437500, 0.0011596679687500,
+0.0025329589843750, 0.0013122558593750, 0.0026855468750000, 0.0013427734375000,
+0.0025024414062500, 0.0012512207031250, 0.0021972656250000, 0.0012512207031250,
+0.0024719238281250, 0.0012817382812500, 0.0020751953125000, 0.0010375976562500,
+0.0023803710937500, 0.0010681152343750, 0.0024108886718750, 0.0009765625000000,
+0.0021972656250000, 0.0007934570312500, 0.0025939941406250, 0.0009155273437500,
+0.0025634765625000, 0.0005798339843750, 0.0026855468750000, 0.0005493164062500,
+0.0029296875000000, 0.0006103515625000, 0.0030822753906250, 0.0004882812500000,
+0.0032348632812500, 0.0004882812500000, 0.0034179687500000, 0.0006408691406250,
+0.0035705566406250, 0.0005187988281250, 0.0036926269531250, 0.0007019042968750,
+0.0038146972656250, 0.0010070800781250, 0.0039978027343750, 0.0008544921875000,
+0.0039978027343750, 0.0011291503906250, 0.0039978027343750, 0.0012207031250000,
+0.0040283203125000, 0.0012207031250000, 0.0041198730468750, 0.0013732910156250,
+0.0039978027343750, 0.0013427734375000, 0.0041809082031250, 0.0014648437500000,
+0.0041503906250000, 0.0015258789062500, 0.0040588378906250, 0.0014953613281250,
+0.0042114257812500, 0.0014648437500000, 0.0041198730468750, 0.0014648437500000,
+0.0040588378906250, 0.0012817382812500, 0.0041198730468750, 0.0013732910156250,
+0.0037536621093750, 0.0011901855468750, 0.0035095214843750, 0.0009155273437500,
+0.0037536621093750, 0.0010375976562500, 0.0030517578125000, 0.0006103515625000,
+0.0031127929687500, 0.0006103515625000, 0.0030822753906250, 0.0005187988281250,
+0.0026550292968750},
+{-0.0005798339843750, 0.0014953613281250, -0.0006713867187500, 0.0013122558593750,
+-0.0007019042968750, 0.0014953613281250, -0.0001831054687500, 0.0007934570312500,
+-0.0002746582031250, 0.0009765625000000, 0.0000610351562500, 0.0007629394531250,
+0.0006713867187500, 0.0001525878906250, 0.0006408691406250, 0.0004272460937500,
+0.0010986328125000, -0.0001220703125000, 0.0010070800781250, 0.0000305175781250,
+0.0007934570312500, 0.0000915527343750, 0.0009155273437500, -0.0002746582031250,
+0.0007629394531250, 0.0000000000000000, 0.0006408691406250, -0.0001220703125000,
+0.0004577636718750, -0.0001831054687500, 0.0004882812500000, 0.0000000000000000,
+0.0005187988281250, 0.0000610351562500, 0.0002746582031250, -0.0000610351562500,
+0.0007629394531250, 0.0003356933593750, 0.0005798339843750, 0.0001525878906250,
+0.0007019042968750, 0.0003662109375000, 0.0011291503906250, 0.0007019042968750,
+0.0008239746093750, 0.0003967285156250, 0.0012817382812500, 0.0009765625000000,
+0.0012512207031250, 0.0008544921875000, 0.0012207031250000, 0.0009460449218750,
+0.0013427734375000, 0.0013732910156250, 0.0011291503906250, 0.0010375976562500,
+0.0015258789062500, 0.0015869140625000, 0.0013427734375000, 0.0014648437500000,
+0.0014343261718750, 0.0014953613281250, 0.0017700195312500, 0.0018615722656250,
+0.0014648437500000, 0.0015869140625000, 0.0020751953125000, 0.0018615722656250,
+0.0020141601562500, 0.0018005371093750, 0.0021057128906250, 0.0018920898437500,
+0.0025634765625000, 0.0020141601562500, 0.0025329589843750, 0.0018310546875000,
+0.0028991699218750, 0.0019531250000000, 0.0029907226562500, 0.0020141601562500,
+0.0031738281250000, 0.0019226074218750, 0.0035400390625000, 0.0018615722656250,
+0.0035705566406250, 0.0018310546875000, 0.0038146972656250, 0.0017089843750000,
+0.0041503906250000, 0.0017089843750000, 0.0040283203125000, 0.0015869140625000,
+0.0039367675781250, 0.0014953613281250, 0.0042724609375000, 0.0014953613281250,
+0.0037231445312500, 0.0013732910156250, 0.0038757324218750, 0.0014343261718750,
+0.0037231445312500, 0.0014343261718750, 0.0032043457031250, 0.0013122558593750,
+0.0034790039062500, 0.0014343261718750, 0.0029907226562500, 0.0014343261718750,
+0.0028076171875000, 0.0014038085937500, 0.0029296875000000, 0.0015258789062500,
+0.0028076171875000, 0.0016174316406250, 0.0025634765625000, 0.0015869140625000,
+0.0032653808593750, 0.0017700195312500, 0.0029296875000000, 0.0017089843750000,
+0.0030517578125000, 0.0017395019531250, 0.0036010742187500, 0.0018310546875000,
+0.0031433105468750, 0.0017700195312500, 0.0034790039062500, 0.0019226074218750,
+0.0035095214843750, 0.0018920898437500, 0.0033874511718750, 0.0019226074218750,
+0.0034790039062500, 0.0020751953125000, 0.0035400390625000, 0.0019836425781250,
+0.0031738281250000, 0.0018920898437500, 0.0033264160156250, 0.0020446777343750,
+0.0032348632812500, 0.0018310546875000, 0.0031127929687500, 0.0015563964843750,
+0.0033874511718750, 0.0018005371093750, 0.0029602050781250, 0.0010681152343750,
+0.0031738281250000, 0.0011291503906250, 0.0032043457031250, 0.0010375976562500,
+0.0029907226562500, 0.0005493164062500, 0.0033264160156250, 0.0006713867187500,
+0.0028076171875000, 0.0004882812500000, 0.0031433105468750, 0.0003356933593750,
+0.0029296875000000, 0.0004272460937500, 0.0025634765625000, 0.0004272460937500,
+0.0030517578125000, 0.0002746582031250, 0.0023193359375000, 0.0006408691406250,
+0.0024719238281250, 0.0005798339843750, 0.0026550292968750, 0.0006408691406250,
+0.0022583007812500, 0.0010070800781250, 0.0025329589843750, 0.0007629394531250,
+0.0026245117187500, 0.0012207031250000, 0.0026550292968750, 0.0011291503906250,
+0.0028076171875000, 0.0014038085937500, 0.0029907226562500, 0.0018005371093750,
+0.0030517578125000, 0.0015563964843750, 0.0029907226562500, 0.0022888183593750,
+0.0031127929687500, 0.0023193359375000, 0.0031433105468750, 0.0024414062500000,
+0.0030517578125000, 0.0029296875000000, 0.0031127929687500, 0.0027770996093750,
+0.0029602050781250, 0.0029907226562500, 0.0029602050781250, 0.0031433105468750,
+0.0029907226562500, 0.0029907226562500, 0.0028991699218750, 0.0028686523437500,
+0.0028686523437500, 0.0029296875000000, 0.0030212402343750, 0.0024414062500000,
+0.0029602050781250, 0.0025024414062500, 0.0030212402343750, 0.0023498535156250,
+0.0031433105468750, 0.0019531250000000, 0.0030212402343750, 0.0021362304687500,
+0.0032348632812500, 0.0017089843750000, 0.0031127929687500, 0.0016784667968750,
+0.0031127929687500, 0.0015869140625000, 0.0032348632812500, 0.0012817382812500,
+0.0031433105468750, 0.0012817382812500, 0.0030822753906250, 0.0013732910156250,
+0.0030212402343750, 0.0011291503906250, 0.0028991699218750, 0.0014648437500000,
+0.0028686523437500, 0.0018005371093750, 0.0027465820312500, 0.0014038085937500,
+0.0028076171875000, 0.0023498535156250, 0.0027160644531250, 0.0022583007812500,
+0.0027770996093750, 0.0024414062500000, 0.0028991699218750, 0.0030822753906250,
+0.0027465820312500, 0.0027465820312500, 0.0028991699218750, 0.0030517578125000,
+0.0029296875000000, 0.0032348632812500, 0.0028991699218750, 0.0029602050781250,
+0.0029602050781250, 0.0028076171875000, 0.0029602050781250, 0.0029907226562500,
+0.0028991699218750, 0.0021362304687500, 0.0030212402343750, 0.0022888183593750,
+0.0028991699218750, 0.0021972656250000, 0.0027160644531250, 0.0015869140625000,
+0.0028381347656250, 0.0017700195312500, 0.0023498535156250, 0.0018615722656250,
+0.0024719238281250, 0.0015563964843750, 0.0022888183593750, 0.0019836425781250,
+0.0019226074218750, 0.0024719238281250, 0.0020751953125000, 0.0021362304687500,
+0.0016479492187500, 0.0031127929687500, 0.0016174316406250, 0.0030517578125000,
+0.0015258789062500, 0.0031433105468750, 0.0013122558593750, 0.0037841796875000,
+0.0013732910156250, 0.0035400390625000, 0.0009460449218750, 0.0036926269531250,
+0.0010375976562500, 0.0037536621093750, 0.0008544921875000, 0.0036010742187500,
+0.0004882812500000, 0.0035400390625000, 0.0007934570312500, 0.0036010742187500,
+0.0003356933593750, 0.0031127929687500, 0.0004272460937500, 0.0032348632812500,
+0.0005187988281250, 0.0030822753906250, 0.0003051757812500, 0.0026550292968750,
+0.0003967285156250, 0.0028076171875000, 0.0006408691406250, 0.0025329589843750,
+0.0005187988281250, 0.0024414062500000, 0.0007019042968750, 0.0024414062500000,
+0.0009155273437500, 0.0023498535156250, 0.0007934570312500, 0.0023803710937500,
+0.0012207031250000, 0.0021362304687500, 0.0012207031250000, 0.0022277832031250,
+0.0011291503906250, 0.0019531250000000, 0.0013122558593750, 0.0016174316406250,
+0.0012512207031250, 0.0017700195312500, 0.0013122558593750, 0.0008850097656250,
+0.0012817382812500, 0.0010070800781250, 0.0012207031250000, 0.0007019042968750,
+0.0011596679687500, -0.0000305175781250, 0.0010681152343750, 0.0002136230468750,
+0.0010375976562500, -0.0003967285156250, 0.0009460449218750, -0.0004882812500000,
+0.0008850097656250, -0.0003967285156250, 0.0008850097656250, -0.0005493164062500,
+0.0008850097656250, -0.0005493164062500, 0.0006408691406250, -0.0005493164062500,
+0.0007934570312500, -0.0005187988281250, 0.0004577636718750, -0.0005798339843750,
+0.0000610351562500, -0.0006103515625000, 0.0003051757812500, -0.0005493164062500,
+-0.0006713867187500, -0.0006103515625000, -0.0005493164062500, -0.0007019042968750,
+-0.0006713867187500, -0.0003662109375000, -0.0013427734375000, -0.0002441406250000,
+-0.0010070800781250, -0.0004882812500000, -0.0009155273437500, 0.0004577636718750,
+-0.0012207031250000, 0.0002441406250000, -0.0008544921875000, 0.0003967285156250,
+0.0000000000000000, 0.0016174316406250, 0.0010986328125000, -0.0011596679687500,
+0.0012512207031250, -0.0012817382812500, 0.0014343261718750, -0.0015869140625000,
+0.0012817382812500, -0.0010681152343750, 0.0011596679687500, -0.0007019042968750,
+0.0013122558593750, -0.0010375976562500, 0.0010375976562500, -0.0000610351562500,
+0.0010375976562500, -0.0001525878906250, 0.0010070800781250, -0.0000305175781250,
+0.0008239746093750, 0.0007019042968750, 0.0008544921875000, 0.0004882812500000,
+0.0009765625000000, 0.0005187988281250, 0.0008239746093750, 0.0007934570312500,
+0.0008850097656250, 0.0007019042968750, 0.0010070800781250, 0.0005493164062500,
+0.0008850097656250, 0.0007019042968750, 0.0008239746093750, 0.0003662109375000,
+0.0008544921875000, 0.0004577636718750, 0.0007019042968750, 0.0003356933593750,
+0.0006408691406250, -0.0000305175781250, 0.0006103515625000, 0.0001525878906250,
+0.0005493164062500, 0.0001831054687500, 0.0003967285156250, 0.0000000000000000,
+0.0006713867187500, 0.0000305175781250, 0.0008544921875000, -0.0000305175781250,
+0.0006103515625000, -0.0000610351562500, 0.0015869140625000, 0.0003356933593750,
+0.0013122558593750, 0.0000000000000000, 0.0015258789062500, 0.0002746582031250,
+0.0022583007812500, 0.0009155273437500, 0.0017395019531250, 0.0005187988281250,
+0.0021362304687500, 0.0012512207031250, 0.0021362304687500, 0.0013122558593750,
+0.0018005371093750, 0.0014038085937500, 0.0017700195312500, 0.0017700195312500,
+0.0017395019531250, 0.0015869140625000, 0.0014648437500000, 0.0017395019531250,
+0.0013427734375000, 0.0019531250000000, 0.0014648437500000, 0.0019531250000000,
+0.0014038085937500, 0.0019836425781250, 0.0012817382812500, 0.0020141601562500,
+0.0018310546875000, 0.0018615722656250, 0.0016479492187500, 0.0019836425781250,
+0.0018310546875000, 0.0019226074218750, 0.0023803710937500, 0.0017089843750000,
+0.0021057128906250, 0.0019226074218750, 0.0023498535156250, 0.0021972656250000,
+0.0025329589843750, 0.0019531250000000, 0.0022583007812500, 0.0023498535156250,
+0.0021362304687500, 0.0029907226562500, 0.0023498535156250, 0.0026245117187500,
+0.0017700195312500, 0.0030212402343750, 0.0018615722656250, 0.0031433105468750,
+0.0019531250000000, 0.0028381347656250, 0.0016479492187500, 0.0028991699218750,
+0.0016784667968750, 0.0030212402343750, 0.0023498535156250, 0.0022583007812500,
+0.0020751953125000, 0.0024719238281250, 0.0024108886718750, 0.0023193359375000,
+0.0031433105468750, 0.0017395019531250, 0.0028076171875000, 0.0022888183593750,
+0.0032348632812500, 0.0020141601562500, 0.0035400390625000, 0.0019226074218750,
+0.0032043457031250, 0.0021972656250000, 0.0031127929687500, 0.0022583007812500,
+0.0035400390625000, 0.0022277832031250, 0.0022583007812500, 0.0024414062500000,
+0.0026245117187500, 0.0023193359375000, 0.0021667480468750, 0.0023803710937500,
+0.0010375976562500, 0.0025634765625000, 0.0017395019531250, 0.0024108886718750,
+0.0006713867187500, 0.0025634765625000, 0.0006408691406250, 0.0024719238281250,
+0.0008239746093750, 0.0025329589843750, 0.0004272460937500, 0.0026855468750000,
+0.0006103515625000, 0.0025329589843750, 0.0008239746093750, 0.0028991699218750,
+0.0007934570312500, 0.0028076171875000, 0.0009460449218750, 0.0028991699218750,
+0.0011596679687500, 0.0031738281250000, 0.0010986328125000, 0.0029907226562500,
+0.0013427734375000, 0.0032958984375000, 0.0013122558593750, 0.0032043457031250,
+0.0014038085937500, 0.0031433105468750, 0.0015869140625000, 0.0032653808593750,
+0.0014953613281250, 0.0031738281250000, 0.0015563964843750, 0.0031433105468750,
+0.0015869140625000, 0.0031433105468750, 0.0015258789062500, 0.0030212402343750,
+0.0014648437500000, 0.0029602050781250, 0.0014343261718750, 0.0028381347656250,
+0.0016479492187500},
+{0.0010986328125000, -0.0009460449218750, 0.0009765625000000, 0.0002136230468750,
+0.0011596679687500, -0.0003356933593750, 0.0009155273437500, 0.0002441406250000,
+0.0006103515625000, 0.0014648437500000, 0.0008544921875000, 0.0006713867187500,
+0.0007019042968750, 0.0021667480468750, 0.0004577636718750, 0.0022277832031250,
+0.0009155273437500, 0.0019226074218750, 0.0012207031250000, 0.0025329589843750,
+0.0008239746093750, 0.0025634765625000, 0.0022277832031250, 0.0017700195312500,
+0.0019836425781250, 0.0020446777343750, 0.0021972656250000, 0.0016174316406250,
+0.0031738281250000, 0.0008544921875000, 0.0026245117187500, 0.0012207031250000,
+0.0027465820312500, 0.0001831054687500, 0.0029296875000000, 0.0001220703125000,
+0.0025329589843750, 0.0002136230468750, 0.0021057128906250, -0.0002746582031250,
+0.0023498535156250, -0.0002441406250000, 0.0016784667968750, 0.0003967285156250,
+0.0014038085937500, 0.0001525878906250, 0.0014953613281250, 0.0005187988281250,
+0.0013732910156250, 0.0011901855468750, 0.0011291503906250, 0.0008850097656250,
+0.0013427734375000, 0.0018920898437500, 0.0012207031250000, 0.0020446777343750,
+0.0011596679687500, 0.0019531250000000, 0.0013427734375000, 0.0023193359375000,
+0.0011901855468750, 0.0022583007812500, 0.0008850097656250, 0.0019226074218750,
+0.0009460449218750, 0.0020446777343750, 0.0007324218750000, 0.0018615722656250,
+0.0003967285156250, 0.0014343261718750, 0.0005798339843750, 0.0014953613281250,
+0.0004272460937500, 0.0010375976562500, 0.0001525878906250, 0.0010070800781250,
+0.0005493164062500, 0.0011901855468750, 0.0007019042968750, 0.0010375976562500,
+0.0003967285156250, 0.0010070800781250, 0.0015563964843750, 0.0013122558593750,
+0.0012512207031250, 0.0012817382812500, 0.0014648437500000, 0.0014343261718750,
+0.0024414062500000, 0.0018615722656250, 0.0019226074218750, 0.0017395019531250,
+0.0021667480468750, 0.0019226074218750, 0.0025634765625000, 0.0021057128906250,
+0.0018615722656250, 0.0020141601562500, 0.0014648437500000, 0.0019531250000000,
+0.0019531250000000, 0.0022583007812500, 0.0002441406250000, 0.0018310546875000,
+0.0006103515625000, 0.0019226074218750, 0.0002136230468750, 0.0020446777343750,
+-0.0010986328125000, 0.0018310546875000, -0.0003356933593750, 0.0018310546875000,
+-0.0010681152343750, 0.0021667480468750, -0.0013427734375000, 0.0017700195312500,
+-0.0008544921875000, 0.0018920898437500, -0.0008239746093750, 0.0022277832031250,
+-0.0009765625000000, 0.0017089843750000, 0.0000610351562500, 0.0023193359375000,
+-0.0000610351562500, 0.0020141601562500, 0.0001831054687500, 0.0018615722656250,
+0.0010375976562500, 0.0021972656250000, 0.0007934570312500, 0.0018615722656250,
+0.0012817382812500, 0.0018920898437500, 0.0014648437500000, 0.0018005371093750,
+0.0014953613281250, 0.0017089843750000, 0.0017089843750000, 0.0017089843750000,
+0.0018005371093750, 0.0016784667968750, 0.0018310546875000, 0.0016479492187500,
+0.0019226074218750, 0.0016174316406250, 0.0019531250000000, 0.0016479492187500,
+0.0019531250000000, 0.0016784667968750, 0.0019836425781250, 0.0016174316406250,
+0.0021057128906250, 0.0018005371093750, 0.0020446777343750, 0.0017089843750000,
+0.0021667480468750, 0.0018310546875000, 0.0023498535156250, 0.0019836425781250,
+0.0021057128906250, 0.0018310546875000, 0.0024719238281250, 0.0023193359375000,
+0.0024414062500000, 0.0021667480468750, 0.0023803710937500, 0.0021972656250000,
+0.0025634765625000, 0.0025329589843750, 0.0024719238281250, 0.0022583007812500,
+0.0022888183593750, 0.0025329589843750, 0.0022888183593750, 0.0025024414062500,
+0.0021362304687500, 0.0024719238281250, 0.0018920898437500, 0.0027160644531250,
+0.0018920898437500, 0.0025634765625000, 0.0018615722656250, 0.0027465820312500,
+0.0017395019531250, 0.0028991699218750, 0.0018310546875000, 0.0028686523437500,
+0.0019226074218750, 0.0029602050781250, 0.0018615722656250, 0.0030822753906250,
+0.0021667480468750, 0.0028991699218750, 0.0021667480468750, 0.0030517578125000,
+0.0021362304687500, 0.0029907226562500, 0.0022888183593750, 0.0028686523437500,
+0.0022888183593750, 0.0029907226562500, 0.0019531250000000, 0.0026855468750000,
+0.0020446777343750, 0.0027465820312500, 0.0018005371093750, 0.0026550292968750,
+0.0014038085937500, 0.0023803710937500, 0.0015869140625000, 0.0025329589843750,
+0.0008850097656250, 0.0022888183593750, 0.0009155273437500, 0.0022888183593750,
+0.0008850097656250, 0.0022277832031250, 0.0004272460937500, 0.0020141601562500,
+0.0006408691406250, 0.0021667480468750, 0.0007324218750000, 0.0019226074218750,
+0.0005798339843750, 0.0020751953125000, 0.0009155273437500, 0.0020141601562500,
+0.0012207031250000, 0.0018920898437500, 0.0009765625000000, 0.0021057128906250,
+0.0017089843750000, 0.0019226074218750, 0.0016479492187500, 0.0019226074218750,
+0.0015869140625000, 0.0020446777343750, 0.0019836425781250, 0.0020751953125000,
+0.0018310546875000, 0.0021972656250000, 0.0015869140625000, 0.0022888183593750,
+0.0017089843750000, 0.0023498535156250, 0.0013427734375000, 0.0023498535156250,
+0.0010070800781250, 0.0024719238281250, 0.0012512207031250, 0.0025024414062500,
+0.0007324218750000, 0.0023803710937500, 0.0006713867187500, 0.0025024414062500,
+0.0006713867187500, 0.0024108886718750, 0.0004272460937500, 0.0022277832031250,
+0.0005798339843750, 0.0023498535156250, 0.0004882812500000, 0.0020141601562500,
+0.0005493164062500, 0.0021057128906250, 0.0005493164062500, 0.0019226074218750,
+0.0005493164062500, 0.0015869140625000, 0.0007019042968750, 0.0018005371093750,
+0.0006103515625000, 0.0013427734375000, 0.0007324218750000, 0.0013122558593750,
+0.0008239746093750, 0.0013427734375000, 0.0007934570312500, 0.0011291503906250,
+0.0009155273437500, 0.0010986328125000, 0.0010681152343750, 0.0012207031250000,
+0.0011291503906250, 0.0009765625000000, 0.0011596679687500, 0.0011596679687500,
+0.0012207031250000, 0.0014343261718750, 0.0014038085937500, 0.0010375976562500,
+0.0013427734375000, 0.0014343261718750, 0.0014343261718750, 0.0013122558593750,
+0.0014648437500000, 0.0011291503906250, 0.0014953613281250, 0.0012817382812500,
+0.0015869140625000, 0.0011596679687500, 0.0014648437500000, 0.0010681152343750,
+0.0015869140625000, 0.0011596679687500, 0.0017395019531250, 0.0009765625000000,
+0.0016784667968750, 0.0008239746093750, 0.0018615722656250, 0.0009765625000000,
+0.0021057128906250, 0.0004577636718750, 0.0020751953125000, 0.0006103515625000,
+0.0025939941406250, 0.0004272460937500, 0.0030822753906250, -0.0000305175781250,
+0.0028381347656250, 0.0002746582031250, 0.0042114257812500, -0.0001220703125000,
+0.0040588378906250, -0.0002136230468750, 0.0043334960937500, -0.0000610351562500,
+0.0053100585937500, -0.0002136230468750, 0.0049133300781250, -0.0002136230468750,
+0.0049438476562500, 0.0001525878906250, 0.0051269531250000, 0.0000000000000000,
+0.0046081542968750, 0.0001831054687500, 0.0039367675781250, 0.0005187988281250,
+0.0035095214843750, 0.0002136230468750, 0.0025634765625000, 0.0010986328125000,
+0.0027770996093750, 0.0007019042968750, 0.0025024414062500, 0.0007019042968750,
+0.0017700195312500, 0.0013732910156250, 0.0020446777343750, 0.0008239746093750,
+0.0011901855468750, 0.0014953613281250, 0.0011596679687500, 0.0013427734375000,
+0.0011901855468750, 0.0013732910156250, 0.0007934570312500, 0.0017395019531250,
+0.0009460449218750, 0.0013732910156250, 0.0013732910156250, 0.0017089843750000,
+0.0011901855468750, 0.0015563964843750, 0.0016174316406250, 0.0014648437500000,
+0.0023803710937500, 0.0016479492187500, 0.0021362304687500, 0.0014343261718750,
+0.0025939941406250, 0.0015563964843750, 0.0029296875000000, 0.0014953613281250,
+0.0028381347656250, 0.0015258789062500, 0.0028686523437500, 0.0015563964843750,
+0.0030212402343750, 0.0014953613281250, 0.0024719238281250, 0.0016174316406250,
+0.0026855468750000, 0.0016479492187500, 0.0025329589843750, 0.0014953613281250,
+0.0019836425781250, 0.0014953613281250, 0.0022888183593750, 0.0015258789062500,
+0.0017700195312500, 0.0013732910156250, 0.0017700195312500, 0.0013732910156250,
+0.0017395019531250, 0.0013122558593750, 0.0014343261718750, 0.0011291503906250,
+0.0014038085937500, 0.0011596679687500, 0.0011596679687500, 0.0010986328125000,
+0.0011901855468750, 0.0010070800781250, 0.0010375976562500, 0.0010375976562500,
+0.0007019042968750, 0.0010375976562500, 0.0007629394531250, 0.0009765625000000,
+0.0006103515625000, 0.0009155273437500, 0.0005798339843750, 0.0008850097656250,
+0.0006408691406250, 0.0007629394531250, 0.0006713867187500, 0.0006408691406250,
+0.0007629394531250, 0.0006103515625000, 0.0009765625000000, 0.0006713867187500,
+0.0009460449218750, 0.0005187988281250, 0.0013427734375000, 0.0006408691406250,
+0.0017395019531250, 0.0007324218750000, 0.0015869140625000, 0.0004882812500000,
+0.0024108886718750, 0.0010375976562500, 0.0024414062500000, 0.0008239746093750,
+0.0025329589843750, 0.0009765625000000, 0.0030517578125000, 0.0014648437500000,
+0.0029296875000000, 0.0010986328125000, 0.0029296875000000, 0.0015258789062500,
+0.0029907226562500, 0.0015563964843750, 0.0027160644531250, 0.0013122558593750,
+0.0024719238281250, 0.0014343261718750, 0.0025634765625000, 0.0013732910156250,
+0.0019226074218750, 0.0006713867187500, 0.0018310546875000, 0.0007629394531250,
+0.0016174316406250, 0.0005187988281250, 0.0011596679687500, -0.0000610351562500,
+0.0011596679687500, 0.0001525878906250, 0.0010681152343750, 0.0000305175781250,
+0.0007934570312500, -0.0002441406250000, 0.0008850097656250, 0.0001525878906250,
+0.0010070800781250, 0.0003662109375000, 0.0007629394531250, 0.0000915527343750,
+0.0011901855468750, 0.0010375976562500, 0.0010070800781250, 0.0008544921875000,
+0.0011291503906250, 0.0012207031250000, 0.0015258789062500, 0.0019531250000000,
+0.0012207031250000, 0.0014953613281250, 0.0017700195312500, 0.0025634765625000,
+0.0017700195312500, 0.0024108886718750, 0.0018310546875000, 0.0025329589843750,
+0.0021972656250000, 0.0032348632812500, 0.0020751953125000, 0.0028991699218750,
+0.0022888183593750, 0.0033264160156250, 0.0023803710937500, 0.0034179687500000,
+0.0023193359375000, 0.0032958984375000, 0.0024414062500000, 0.0033874511718750,
+0.0025024414062500, 0.0033874511718750, 0.0022888183593750, 0.0029296875000000,
+0.0024719238281250, 0.0031127929687500, 0.0024108886718750, 0.0028381347656250,
+0.0021972656250000, 0.0023193359375000, 0.0024414062500000, 0.0025329589843750,
+0.0021667480468750, 0.0018920898437500, 0.0022888183593750, 0.0017700195312500,
+0.0022888183593750, 0.0017700195312500, 0.0020141601562500, 0.0014953613281250,
+0.0022888183593750, 0.0014648437500000, 0.0019836425781250, 0.0014343261718750,
+0.0019836425781250, 0.0014038085937500, 0.0021057128906250, 0.0014343261718750,
+0.0019836425781250, 0.0014648437500000, 0.0020446777343750, 0.0014038085937500,
+0.0021667480468750, 0.0015258789062500, 0.0021057128906250, 0.0015563964843750,
+0.0021667480468750, 0.0015563964843750, 0.0023193359375000, 0.0014953613281250,
+0.0022583007812500, 0.0015258789062500, 0.0023193359375000, 0.0015563964843750,
+0.0024414062500000, 0.0015258789062500, 0.0023498535156250, 0.0017700195312500,
+0.0023498535156250, 0.0019531250000000, 0.0024414062500000, 0.0018005371093750,
+0.0024108886718750, 0.0022277832031250, 0.0023803710937500, 0.0022583007812500,
+0.0024414062500000},
+{0.0024414062500000, 0.0005798339843750, 0.0024108886718750, 0.0006408691406250,
+0.0025634765625000, 0.0007629394531250, 0.0024414062500000, 0.0005798339843750,
+0.0024414062500000, 0.0007019042968750, 0.0025939941406250, 0.0007324218750000,
+0.0023803710937500, 0.0006103515625000, 0.0023498535156250, 0.0006103515625000,
+0.0025939941406250, 0.0005798339843750, 0.0021667480468750, 0.0006103515625000,
+0.0023193359375000, 0.0004882812500000, 0.0022583007812500, 0.0006408691406250,
+0.0020141601562500, 0.0008544921875000, 0.0022583007812500, 0.0006408691406250,
+0.0020446777343750, 0.0010986328125000, 0.0020141601562500, 0.0010375976562500,
+0.0021972656250000, 0.0011901855468750, 0.0021362304687500, 0.0014953613281250,
+0.0020751953125000, 0.0011901855468750, 0.0024719238281250, 0.0014953613281250,
+0.0022583007812500, 0.0016479492187500, 0.0024719238281250, 0.0013732910156250,
+0.0028686523437500, 0.0013427734375000, 0.0024719238281250, 0.0014343261718750,
+0.0029907226562500, 0.0007629394531250, 0.0028991699218750, 0.0008239746093750,
+0.0029602050781250, 0.0005798339843750, 0.0032348632812500, -0.0000305175781250,
+0.0029907226562500, 0.0001525878906250, 0.0031433105468750, 0.0000305175781250,
+0.0032348632812500, -0.0002746582031250, 0.0030517578125000, 0.0000610351562500,
+0.0030212402343750, 0.0003356933593750, 0.0030822753906250, 0.0000610351562500,
+0.0026245117187500, 0.0007934570312500, 0.0027160644531250, 0.0007324218750000,
+0.0026855468750000, 0.0008239746093750, 0.0023193359375000, 0.0012512207031250,
+0.0024414062500000, 0.0010681152343750, 0.0024414062500000, 0.0013427734375000,
+0.0023498535156250, 0.0014038085937500, 0.0024414062500000, 0.0014038085937500,
+0.0025939941406250, 0.0014953613281250, 0.0024108886718750, 0.0014953613281250,
+0.0025329589843750, 0.0014953613281250, 0.0025634765625000, 0.0015258789062500,
+0.0025634765625000, 0.0014953613281250, 0.0025939941406250, 0.0014038085937500,
+0.0025329589843750, 0.0014648437500000, 0.0025634765625000, 0.0013427734375000,
+0.0025634765625000, 0.0012817382812500, 0.0024719238281250, 0.0013122558593750,
+0.0024414062500000, 0.0012817382812500, 0.0024414062500000, 0.0013122558593750,
+0.0024108886718750, 0.0015563964843750, 0.0024414062500000, 0.0014038085937500,
+0.0023803710937500, 0.0016174316406250, 0.0023498535156250, 0.0020141601562500,
+0.0023803710937500, 0.0018615722656250, 0.0022888183593750, 0.0024719238281250,
+0.0022583007812500, 0.0025634765625000, 0.0022583007812500, 0.0027160644531250,
+0.0022888183593750, 0.0030822753906250, 0.0021972656250000, 0.0030212402343750,
+0.0022277832031250, 0.0030822753906250, 0.0022277832031250, 0.0032653808593750,
+0.0021362304687500, 0.0030212402343750, 0.0021667480468750, 0.0028076171875000,
+0.0021057128906250, 0.0029907226562500, 0.0021972656250000, 0.0022583007812500,
+0.0021057128906250, 0.0024414062500000, 0.0022583007812500, 0.0021972656250000,
+0.0024108886718750, 0.0015869140625000, 0.0022277832031250, 0.0019836425781250,
+0.0026245117187500, 0.0011291503906250, 0.0025634765625000, 0.0012207031250000,
+0.0026550292968750, 0.0010681152343750, 0.0028991699218750, 0.0004882812500000,
+0.0028381347656250, 0.0007934570312500, 0.0028381347656250, 0.0004577636718750,
+0.0030212402343750, 0.0003662109375000, 0.0029602050781250, 0.0004272460937500,
+0.0028076171875000, 0.0003356933593750, 0.0029602050781250, 0.0003356933593750,
+0.0027160644531250, 0.0003967285156250, 0.0026855468750000, 0.0003051757812500,
+0.0026550292968750, 0.0004272460937500, 0.0025024414062500, 0.0006103515625000,
+0.0026245117187500, 0.0004272460937500, 0.0023498535156250, 0.0008850097656250,
+0.0024719238281250, 0.0009155273437500, 0.0024108886718750, 0.0008850097656250,
+0.0021972656250000, 0.0011596679687500, 0.0023498535156250, 0.0010375976562500,
+0.0021057128906250, 0.0008544921875000, 0.0021972656250000, 0.0010375976562500,
+0.0021972656250000, 0.0006103515625000, 0.0020141601562500, 0.0000610351562500,
+0.0021057128906250, 0.0003356933593750, 0.0021057128906250, -0.0004272460937500,
+0.0020141601562500, -0.0004577636718750, 0.0021972656250000, -0.0004272460937500,
+0.0023498535156250, -0.0007934570312500, 0.0022277832031250, -0.0006713867187500,
+0.0024719238281250, -0.0003662109375000, 0.0024719238281250, -0.0004882812500000,
+0.0023803710937500, -0.0002746582031250, 0.0025024414062500, 0.0000915527343750,
+0.0025329589843750, -0.0000305175781250, 0.0022277832031250, 0.0001831054687500,
+0.0022277832031250, 0.0003051757812500, 0.0021057128906250, 0.0002746582031250,
+0.0018005371093750, 0.0003356933593750, 0.0019531250000000, 0.0003662109375000,
+0.0015563964843750, 0.0002746582031250, 0.0015258789062500, 0.0002746582031250,
+0.0014343261718750, 0.0003051757812500, 0.0012207031250000, 0.0003051757812500,
+0.0012512207031250, 0.0003051757812500, 0.0009765625000000, 0.0006408691406250,
+0.0010681152343750, 0.0006408691406250, 0.0008850097656250, 0.0007934570312500,
+0.0006713867187500, 0.0011596679687500, 0.0008544921875000, 0.0012512207031250,
+0.0003662109375000, 0.0013427734375000, 0.0005493164062500, 0.0015258789062500,
+0.0004577636718750, 0.0015563964843750, 0.0000305175781250, 0.0016174316406250,
+0.0003051757812500, 0.0017700195312500, -0.0000915527343750, 0.0014648437500000,
+-0.0001220703125000, 0.0017089843750000, 0.0000000000000000, 0.0015258789062500,
+-0.0002441406250000, 0.0010375976562500, -0.0002441406250000, 0.0013732910156250,
+0.0000305175781250, 0.0007019042968750, -0.0001831054687500, 0.0007629394531250,
+0.0000305175781250, 0.0006713867187500, 0.0003356933593750, 0.0003662109375000,
+0.0000610351562500, 0.0005187988281250, 0.0005187988281250, 0.0002441406250000,
+0.0005493164062500, 0.0001831054687500, 0.0004577636718750, 0.0002441406250000,
+0.0006408691406250, 0.0001220703125000, 0.0006103515625000, 0.0000305175781250,
+0.0004577636718750, 0.0005798339843750, 0.0005493164062500, 0.0002441406250000,
+0.0005798339843750, 0.0007019042968750, 0.0004882812500000, 0.0013122558593750,
+0.0005493164062500, 0.0008239746093750, 0.0007019042968750, 0.0023498535156250,
+0.0006713867187500, 0.0021362304687500, 0.0007629394531250, 0.0024719238281250,
+0.0009460449218750, 0.0044555664062500, 0.0010986328125000, 0.0005187988281250,
+0.0012207031250000, 0.0009155273437500, 0.0012817382812500, 0.0002136230468750,
+0.0012817382812500, 0.0010375976562500, 0.0012817382812500, 0.0018920898437500,
+0.0014343261718750, 0.0012817382812500, 0.0017395019531250, 0.0030822753906250,
+0.0016174316406250, 0.0028991699218750, 0.0020446777343750, 0.0030822753906250,
+0.0025939941406250, 0.0043029785156250, 0.0023193359375000, 0.0038146972656250,
+0.0032043457031250, 0.0039978027343750, 0.0032958984375000, 0.0043945312500000,
+0.0031738281250000, 0.0037536621093750, 0.0036010742187500, 0.0033569335937500,
+0.0036010742187500, 0.0038757324218750, 0.0029602050781250, 0.0020751953125000,
+0.0032043457031250, 0.0025024414062500, 0.0028991699218750, 0.0021057128906250,
+0.0021972656250000, 0.0006408691406250, 0.0025939941406250, 0.0013732910156250,
+0.0019226074218750, 0.0007019042968750, 0.0017395019531250, 0.0005187988281250,
+0.0018920898437500, 0.0009155273437500, 0.0018005371093750, 0.0009155273437500,
+0.0017395019531250, 0.0007934570312500, 0.0019226074218750, 0.0016479492187500,
+0.0018615722656250, 0.0014343261718750, 0.0017089843750000, 0.0015563964843750,
+0.0017700195312500, 0.0021667480468750, 0.0016174316406250, 0.0018920898437500,
+0.0012817382812500, 0.0022277832031250, 0.0013122558593750, 0.0022888183593750,
+0.0012817382812500, 0.0023498535156250, 0.0010070800781250, 0.0025024414062500,
+0.0008850097656250, 0.0023498535156250, 0.0013427734375000, 0.0022583007812500,
+0.0009460449218750, 0.0022583007812500, 0.0012207031250000, 0.0024108886718750,
+0.0017395019531250, 0.0023193359375000, 0.0012512207031250, 0.0019836425781250,
+0.0020141601562500, 0.0026550292968750, 0.0019836425781250, 0.0023498535156250,
+0.0017395019531250, 0.0026550292968750, 0.0021057128906250, 0.0032653808593750,
+0.0019836425781250, 0.0027465820312500, 0.0012817382812500, 0.0036010742187500,
+0.0016174316406250, 0.0033874511718750, 0.0011291503906250, 0.0035400390625000,
+0.0002746582031250, 0.0040588378906250, 0.0007629394531250, 0.0035400390625000,
+-0.0001220703125000, 0.0040893554687500, -0.0002136230468750, 0.0039367675781250,
+-0.0001220703125000, 0.0038146972656250, -0.0005187988281250, 0.0041198730468750,
+-0.0004882812500000, 0.0038757324218750, -0.0000305175781250, 0.0037536621093750,
+-0.0003662109375000, 0.0036010742187500, 0.0001220703125000, 0.0035400390625000,
+0.0007629394531250, 0.0035705566406250, 0.0001831054687500, 0.0035705566406250,
+0.0014953613281250, 0.0036010742187500, 0.0012512207031250, 0.0036621093750000,
+0.0014038085937500, 0.0036315917968750, 0.0022583007812500, 0.0038146972656250,
+0.0017089843750000, 0.0041503906250000, 0.0020141601562500, 0.0038757324218750,
+0.0021667480468750, 0.0041503906250000, 0.0018310546875000, 0.0041503906250000,
+0.0016784667968750, 0.0039672851562500, 0.0018615722656250, 0.0043945312500000,
+0.0014648437500000, 0.0039367675781250, 0.0014038085937500, 0.0039978027343750,
+0.0014343261718750, 0.0039672851562500, 0.0011901855468750, 0.0037231445312500,
+0.0012817382812500, 0.0038146972656250, 0.0012207031250000, 0.0037841796875000,
+0.0011901855468750, 0.0036926269531250, 0.0012512207031250, 0.0037536621093750,
+0.0012512207031250, 0.0038146972656250, 0.0011596679687500, 0.0037536621093750,
+0.0012817382812500, 0.0036926269531250, 0.0012512207031250, 0.0036621093750000,
+0.0013122558593750, 0.0034790039062500, 0.0014953613281250, 0.0033569335937500,
+0.0014343261718750, 0.0033874511718750, 0.0015869140625000, 0.0028991699218750,
+0.0016784667968750, 0.0029602050781250, 0.0016174316406250, 0.0028076171875000,
+0.0016174316406250, 0.0024414062500000, 0.0017089843750000, 0.0026855468750000,
+0.0015563964843750, 0.0022277832031250, 0.0015869140625000, 0.0022888183593750,
+0.0015869140625000, 0.0022583007812500, 0.0015563964843750, 0.0019531250000000,
+0.0016174316406250, 0.0020446777343750, 0.0015563964843750, 0.0018920898437500,
+0.0016174316406250, 0.0017700195312500, 0.0016174316406250, 0.0017395019531250,
+0.0015563964843750, 0.0017089843750000, 0.0015563964843750, 0.0016174316406250,
+0.0016479492187500, 0.0015563964843750, 0.0014953613281250, 0.0014343261718750,
+0.0017089843750000, 0.0014038085937500, 0.0018615722656250, 0.0013427734375000,
+0.0017089843750000, 0.0012512207031250, 0.0021972656250000, 0.0013732910156250,
+0.0020141601562500, 0.0012512207031250, 0.0021972656250000, 0.0013122558593750,
+0.0025939941406250, 0.0014343261718750, 0.0022888183593750, 0.0013122558593750,
+0.0026855468750000, 0.0016174316406250, 0.0026855468750000, 0.0015563964843750,
+0.0025939941406250, 0.0016174316406250, 0.0027465820312500, 0.0018615722656250,
+0.0027160644531250, 0.0016784667968750, 0.0027465820312500, 0.0018920898437500,
+0.0027160644531250, 0.0018005371093750, 0.0027160644531250, 0.0018920898437500,
+0.0027160644531250, 0.0020141601562500, 0.0027160644531250, 0.0018005371093750,
+0.0026855468750000, 0.0021057128906250, 0.0027465820312500, 0.0019836425781250,
+0.0025939941406250, 0.0021362304687500, 0.0024414062500000, 0.0023193359375000,
+0.0025329589843750},
+{-0.0006408691406250, 0.0027465820312500, -0.0003356933593750, 0.0028686523437500,
+-0.0007934570312500, 0.0023803710937500, -0.0010986328125000, 0.0023498535156250,
+-0.0008239746093750, 0.0023193359375000, -0.0007934570312500, 0.0020446777343750,
+-0.0010375976562500, 0.0020141601562500, -0.0002746582031250, 0.0018920898437500,
+-0.0005493164062500, 0.0018615722656250, -0.0001831054687500, 0.0020446777343750,
+0.0006103515625000, 0.0020751953125000, 0.0001525878906250, 0.0020141601562500,
+0.0010681152343750, 0.0021667480468750, 0.0010681152343750, 0.0023193359375000,
+0.0011596679687500, 0.0021972656250000, 0.0016784667968750, 0.0021362304687500,
+0.0014038085937500, 0.0021362304687500, 0.0017089843750000, 0.0017395019531250,
+0.0018005371093750, 0.0018615722656250, 0.0016784667968750, 0.0018005371093750,
+0.0016784667968750, 0.0014953613281250, 0.0017089843750000, 0.0017089843750000,
+0.0015563964843750, 0.0016174316406250, 0.0015258789062500, 0.0016479492187500,
+0.0016174316406250, 0.0018920898437500, 0.0016479492187500, 0.0018920898437500,
+0.0015563964843750, 0.0018615722656250, 0.0019226074218750, 0.0023803710937500,
+0.0018615722656250, 0.0021362304687500, 0.0018920898437500, 0.0023193359375000,
+0.0021057128906250, 0.0027770996093750, 0.0019836425781250, 0.0024108886718750,
+0.0020751953125000, 0.0030212402343750, 0.0020446777343750, 0.0028381347656250,
+0.0019836425781250, 0.0029907226562500, 0.0020141601562500, 0.0034484863281250,
+0.0019836425781250, 0.0030517578125000, 0.0018615722656250, 0.0036010742187500,
+0.0019531250000000, 0.0035705566406250, 0.0018005371093750, 0.0035705566406250,
+0.0016479492187500, 0.0039367675781250, 0.0017395019531250, 0.0037231445312500,
+0.0013122558593750, 0.0038757324218750, 0.0013427734375000, 0.0040588378906250,
+0.0013122558593750, 0.0039672851562500, 0.0010681152343750, 0.0039367675781250,
+0.0011596679687500, 0.0040893554687500, 0.0010681152343750, 0.0041198730468750,
+0.0009765625000000, 0.0041198730468750, 0.0010681152343750, 0.0042419433593750,
+0.0011291503906250, 0.0043334960937500, 0.0010681152343750, 0.0042724609375000,
+0.0014343261718750, 0.0045166015625000, 0.0013122558593750, 0.0044860839843750,
+0.0015869140625000, 0.0044860839843750, 0.0019531250000000, 0.0046081542968750,
+0.0017395019531250, 0.0045166015625000, 0.0023803710937500, 0.0046997070312500,
+0.0023498535156250, 0.0046997070312500, 0.0025024414062500, 0.0046081542968750,
+0.0029907226562500, 0.0046997070312500, 0.0027465820312500, 0.0047607421875000,
+0.0031433105468750, 0.0044250488281250, 0.0031433105468750, 0.0045776367187500,
+0.0030822753906250, 0.0043640136718750, 0.0032348632812500, 0.0040283203125000,
+0.0032043457031250, 0.0041809082031250, 0.0030212402343750, 0.0036926269531250,
+0.0030822753906250, 0.0036621093750000, 0.0029296875000000, 0.0035095214843750,
+0.0026550292968750, 0.0031433105468750, 0.0028381347656250, 0.0032043457031250,
+0.0023193359375000, 0.0030212402343750, 0.0023803710937500, 0.0028991699218750,
+0.0022583007812500, 0.0028991699218750, 0.0018310546875000, 0.0027160644531250,
+0.0020751953125000, 0.0026245117187500, 0.0016479492187500, 0.0028076171875000,
+0.0017089843750000, 0.0026550292968750, 0.0017395019531250, 0.0027160644531250,
+0.0015869140625000, 0.0028686523437500, 0.0017395019531250, 0.0027160644531250,
+0.0017395019531250, 0.0028686523437500, 0.0017700195312500, 0.0028381347656250,
+0.0017700195312500, 0.0028076171875000, 0.0018005371093750, 0.0028686523437500,
+0.0018615722656250, 0.0027770996093750, 0.0018005371093750, 0.0026855468750000,
+0.0017700195312500, 0.0027160644531250, 0.0018920898437500, 0.0025329589843750,
+0.0018615722656250, 0.0023803710937500, 0.0018005371093750, 0.0024719238281250,
+0.0024414062500000, 0.0022888183593750, 0.0021667480468750, 0.0022277832031250,
+0.0024414062500000, 0.0022277832031250, 0.0030517578125000, 0.0021057128906250,
+0.0026550292968750, 0.0021667480468750, 0.0032348632812500, 0.0020141601562500,
+0.0033264160156250, 0.0020141601562500, 0.0031127929687500, 0.0018615722656250,
+0.0033264160156250, 0.0017089843750000, 0.0032958984375000, 0.0017700195312500,
+0.0028686523437500, 0.0014953613281250, 0.0029296875000000, 0.0014953613281250,
+0.0027770996093750, 0.0014038085937500, 0.0023803710937500, 0.0011596679687500,
+0.0025329589843750, 0.0012512207031250, 0.0022277832031250, 0.0009460449218750,
+0.0021057128906250, 0.0009460449218750, 0.0022583007812500, 0.0009155273437500,
+0.0022277832031250, 0.0007629394531250, 0.0020446777343750, 0.0007019042968750,
+0.0023193359375000, 0.0007629394531250, 0.0022277832031250, 0.0005493164062500,
+0.0021972656250000, 0.0005798339843750, 0.0022277832031250, 0.0006713867187500,
+0.0021057128906250, 0.0003967285156250, 0.0022583007812500, 0.0006408691406250,
+0.0021057128906250, 0.0005187988281250, 0.0022583007812500, 0.0006713867187500,
+0.0024108886718750, 0.0008239746093750, 0.0021362304687500, 0.0006103515625000,
+0.0026245117187500, 0.0013122558593750, 0.0025634765625000, 0.0010375976562500,
+0.0024108886718750, 0.0012512207031250, 0.0027160644531250, 0.0018310546875000,
+0.0025634765625000, 0.0014343261718750, 0.0022583007812500, 0.0019531250000000,
+0.0024108886718750, 0.0020141601562500, 0.0021972656250000, 0.0018005371093750,
+0.0018920898437500, 0.0018920898437500, 0.0022277832031250, 0.0019531250000000,
+0.0016174316406250, 0.0016784667968750, 0.0017089843750000, 0.0016479492187500,
+0.0017700195312500, 0.0016784667968750, 0.0011291503906250, 0.0015563964843750,
+0.0007934570312500, 0.0016174316406250, 0.0003356933593750, 0.0016479492187500,
+0.0004577636718750, 0.0016174316406250, 0.0003356933593750, 0.0016174316406250,
+-0.0001525878906250, 0.0017089843750000, -0.0000305175781250, 0.0017395019531250,
+-0.0001831054687500, 0.0015563964843750, -0.0004882812500000, 0.0016174316406250,
+-0.0000610351562500, 0.0015258789062500, 0.0002746582031250, 0.0013122558593750,
+-0.0001220703125000, 0.0013732910156250, 0.0011291503906250, 0.0008850097656250,
+0.0010070800781250, 0.0009155273437500, 0.0013427734375000, 0.0009155273437500,
+0.0022888183593750, 0.0006103515625000, 0.0019836425781250, 0.0007629394531250,
+0.0028991699218750, 0.0008239746093750, 0.0031433105468750, 0.0007629394531250,
+0.0030212402343750, 0.0008850097656250, 0.0033874511718750, 0.0011291503906250,
+0.0035400390625000, 0.0010070800781250, 0.0028991699218750, 0.0010375976562500,
+0.0033569335937500, 0.0011901855468750, 0.0029907226562500, 0.0011596679687500,
+0.0022277832031250, 0.0010681152343750, 0.0027770996093750, 0.0011596679687500,
+0.0018310546875000, 0.0010986328125000, 0.0019226074218750, 0.0010986328125000,
+0.0018615722656250, 0.0011901855468750, 0.0012207031250000, 0.0011901855468750,
+0.0015563964843750, 0.0011291503906250, 0.0013732910156250, 0.0015258789062500,
+0.0010986328125000, 0.0014038085937500, 0.0014343261718750, 0.0015258789062500,
+0.0015258789062500, 0.0018920898437500, 0.0011596679687500, 0.0016479492187500,
+0.0018310546875000, 0.0020751953125000, 0.0016479492187500, 0.0020446777343750,
+0.0016174316406250, 0.0018920898437500, 0.0019531250000000, 0.0019531250000000,
+0.0016174316406250, 0.0018615722656250, 0.0018005371093750, 0.0016479492187500,
+0.0017700195312500, 0.0016784667968750, 0.0015563964843750, 0.0014343261718750,
+0.0015563964843750, 0.0011291503906250, 0.0016174316406250, 0.0011596679687500,
+0.0010070800781250, 0.0009460449218750, 0.0010986328125000, 0.0007629394531250,
+0.0010070800781250, 0.0008850097656250, 0.0005187988281250, 0.0009155273437500,
+0.0007019042968750, 0.0008239746093750, 0.0006103515625000, 0.0010986328125000,
+0.0003967285156250, 0.0010681152343750, 0.0006713867187500, 0.0010986328125000,
+0.0008544921875000, 0.0012817382812500, 0.0004577636718750, 0.0012207031250000,
+0.0014953613281250, 0.0012207031250000, 0.0010375976562500, 0.0012207031250000,
+0.0015258789062500, 0.0012207031250000, 0.0025024414062500, 0.0012512207031250,
+0.0016479492187500, 0.0011596679687500, 0.0032653808593750, 0.0012817382812500,
+0.0029602050781250, 0.0012512207031250, 0.0031738281250000, 0.0011596679687500,
+0.0043640136718750, 0.0012207031250000, 0.0037231445312500, 0.0011901855468750,
+0.0043945312500000, 0.0010986328125000, 0.0045471191406250, 0.0010070800781250,
+0.0043640136718750, 0.0010986328125000, 0.0045166015625000, 0.0010681152343750,
+0.0045776367187500, 0.0008850097656250, 0.0044250488281250, 0.0013427734375000,
+0.0044860839843750, 0.0010986328125000, 0.0043640136718750, 0.0013122558593750,
+0.0042724609375000, 0.0018005371093750, 0.0044250488281250, 0.0014343261718750,
+0.0041503906250000, 0.0021057128906250, 0.0042114257812500, 0.0021057128906250,
+0.0042724609375000, 0.0021057128906250, 0.0041809082031250, 0.0024414062500000,
+0.0042114257812500, 0.0022888183593750, 0.0043029785156250, 0.0023193359375000,
+0.0042419433593750, 0.0024719238281250, 0.0043334960937500, 0.0021972656250000,
+0.0043640136718750, 0.0020141601562500, 0.0042724609375000, 0.0022277832031250,
+0.0044860839843750, 0.0014953613281250, 0.0043640136718750, 0.0015563964843750,
+0.0044250488281250, 0.0013732910156250, 0.0046691894531250, 0.0008850097656250,
+0.0044860839843750, 0.0010986328125000, 0.0048217773437500, 0.0007629394531250,
+0.0047912597656250, 0.0007324218750000, 0.0048217773437500, 0.0008544921875000,
+0.0050048828125000, 0.0007629394531250, 0.0048522949218750, 0.0007934570312500,
+0.0050659179687500, 0.0011901855468750, 0.0051269531250000, 0.0010986328125000,
+0.0049438476562500, 0.0013122558593750, 0.0049743652343750, 0.0016479492187500,
+0.0050659179687500, 0.0014953613281250, 0.0046386718750000, 0.0018005371093750,
+0.0047607421875000, 0.0018615722656250, 0.0045166015625000, 0.0018310546875000,
+0.0041198730468750, 0.0019836425781250, 0.0043640136718750, 0.0019836425781250,
+0.0036621093750000, 0.0017089843750000, 0.0036621093750000, 0.0019226074218750,
+0.0035095214843750, 0.0017700195312500, 0.0030212402343750, 0.0014343261718750,
+0.0032653808593750, 0.0016479492187500, 0.0027770996093750, 0.0012817382812500,
+0.0028076171875000, 0.0013122558593750, 0.0027465820312500, 0.0012817382812500,
+0.0024719238281250, 0.0010986328125000, 0.0026245117187500, 0.0011901855468750,
+0.0023498535156250, 0.0010070800781250, 0.0023803710937500, 0.0010375976562500,
+0.0023803710937500, 0.0009460449218750, 0.0022583007812500, 0.0007324218750000,
+0.0023498535156250, 0.0008239746093750, 0.0021972656250000, 0.0006103515625000,
+0.0022888183593750, 0.0005187988281250, 0.0022888183593750, 0.0007019042968750,
+0.0021972656250000, 0.0007324218750000, 0.0023803710937500, 0.0005187988281250,
+0.0021057128906250, 0.0012512207031250, 0.0022277832031250, 0.0010681152343750,
+0.0021972656250000, 0.0013122558593750, 0.0020141601562500, 0.0019226074218750,
+0.0021972656250000, 0.0015869140625000, 0.0019836425781250, 0.0020141601562500,
+0.0020446777343750, 0.0021362304687500, 0.0020446777343750, 0.0018615722656250,
+0.0019531250000000, 0.0018310546875000, 0.0020751953125000, 0.0019531250000000,
+0.0018005371093750, 0.0014038085937500, 0.0019226074218750, 0.0014648437500000,
+0.0018005371093750, 0.0013427734375000, 0.0015563964843750, 0.0009765625000000,
+0.0018005371093750, 0.0011291503906250, 0.0013122558593750, 0.0009460449218750,
+0.0014343261718750},
+{0.0031127929687500, 0.0011596679687500, 0.0029602050781250, 0.0013122558593750,
+0.0031738281250000, 0.0010681152343750, 0.0038146972656250, 0.0005187988281250,
+0.0034484863281250, 0.0008239746093750, 0.0042114257812500, 0.0003662109375000,
+0.0041809082031250, 0.0002746582031250, 0.0042114257812500, 0.0003967285156250,
+0.0046691894531250, 0.0002746582031250, 0.0044250488281250, 0.0002746582031250,
+0.0044555664062500, 0.0005187988281250, 0.0044860839843750, 0.0004577636718750,
+0.0042724609375000, 0.0004882812500000, 0.0040893554687500, 0.0005798339843750,
+0.0041198730468750, 0.0005493164062500, 0.0038757324218750, 0.0006408691406250,
+0.0036926269531250, 0.0005798339843750, 0.0036621093750000, 0.0006103515625000,
+0.0035400390625000, 0.0007324218750000, 0.0032653808593750, 0.0005798339843750,
+0.0035400390625000, 0.0009155273437500, 0.0032348632812500, 0.0008239746093750,
+0.0033264160156250, 0.0009460449218750, 0.0035705566406250, 0.0012512207031250,
+0.0031738281250000, 0.0010375976562500, 0.0035400390625000, 0.0013732910156250,
+0.0033264160156250, 0.0014953613281250, 0.0032958984375000, 0.0014953613281250,
+0.0035705566406250, 0.0016479492187500, 0.0032348632812500, 0.0016784667968750,
+0.0036315917968750, 0.0015563964843750, 0.0034484863281250, 0.0016784667968750,
+0.0034790039062500, 0.0015869140625000, 0.0037231445312500, 0.0013732910156250,
+0.0034179687500000, 0.0015563964843750, 0.0038452148437500, 0.0013427734375000,
+0.0036926269531250, 0.0013732910156250, 0.0036926269531250, 0.0014343261718750,
+0.0039978027343750, 0.0013732910156250, 0.0036926269531250, 0.0013732910156250,
+0.0040283203125000, 0.0014038085937500, 0.0039062500000000, 0.0014343261718750,
+0.0038452148437500, 0.0013732910156250, 0.0039978027343750, 0.0013732910156250,
+0.0038146972656250, 0.0014648437500000, 0.0038757324218750, 0.0013427734375000,
+0.0038452148437500, 0.0014343261718750, 0.0038146972656250, 0.0013732910156250,
+0.0038146972656250, 0.0011901855468750, 0.0037536621093750, 0.0013427734375000,
+0.0037841796875000, 0.0011291503906250, 0.0037841796875000, 0.0010681152343750,
+0.0036926269531250, 0.0012207031250000, 0.0036926269531250, 0.0012207031250000,
+0.0037841796875000, 0.0010986328125000, 0.0035400390625000, 0.0016479492187500,
+0.0035400390625000, 0.0014648437500000, 0.0033874511718750, 0.0016784667968750,
+0.0031738281250000, 0.0021667480468750, 0.0032043457031250, 0.0018920898437500,
+0.0028076171875000, 0.0024108886718750, 0.0027770996093750, 0.0024719238281250,
+0.0026245117187500, 0.0023803710937500, 0.0022888183593750, 0.0025634765625000,
+0.0023193359375000, 0.0025329589843750, 0.0021057128906250, 0.0025939941406250,
+0.0019836425781250, 0.0025329589843750, 0.0019226074218750, 0.0027160644531250,
+0.0018920898437500, 0.0028686523437500, 0.0018310546875000, 0.0027465820312500,
+0.0018920898437500, 0.0030822753906250, 0.0018615722656250, 0.0031127929687500,
+0.0018005371093750, 0.0031433105468750, 0.0018615722656250, 0.0033264160156250,
+0.0018005371093750, 0.0032653808593750, 0.0017700195312500, 0.0032043457031250,
+0.0017395019531250, 0.0033264160156250, 0.0017700195312500, 0.0031127929687500,
+0.0018310546875000, 0.0028381347656250, 0.0017395019531250, 0.0029602050781250,
+0.0019226074218750, 0.0024719238281250, 0.0018615722656250, 0.0025024414062500,
+0.0020751953125000, 0.0023498535156250, 0.0022888183593750, 0.0019531250000000,
+0.0021057128906250, 0.0020751953125000, 0.0026550292968750, 0.0019836425781250,
+0.0025939941406250, 0.0017700195312500, 0.0025634765625000, 0.0020446777343750,
+0.0028381347656250, 0.0022888183593750, 0.0026855468750000, 0.0020446777343750,
+0.0024414062500000, 0.0026855468750000, 0.0025634765625000, 0.0025634765625000,
+0.0021972656250000, 0.0025634765625000, 0.0018615722656250, 0.0028686523437500,
+0.0020751953125000, 0.0027160644531250, 0.0015563964843750, 0.0029296875000000,
+0.0014038085937500, 0.0027770996093750, 0.0016174316406250, 0.0029296875000000,
+0.0014343261718750, 0.0031738281250000, 0.0012512207031250, 0.0028991699218750,
+0.0017089843750000, 0.0035095214843750, 0.0014343261718750, 0.0032958984375000,
+0.0016174316406250, 0.0034484863281250, 0.0020446777343750, 0.0040283203125000,
+0.0016174316406250, 0.0037536621093750, 0.0021667480468750, 0.0042724609375000,
+0.0020751953125000, 0.0043029785156250, 0.0019226074218750, 0.0043029785156250,
+0.0021667480468750, 0.0045471191406250, 0.0019836425781250, 0.0044860839843750,
+0.0021362304687500, 0.0040283203125000, 0.0019226074218750, 0.0043640136718750,
+0.0021362304687500, 0.0037841796875000, 0.0024108886718750, 0.0030212402343750,
+0.0021362304687500, 0.0035095214843750, 0.0026855468750000, 0.0024414062500000,
+0.0025634765625000, 0.0023193359375000, 0.0025329589843750, 0.0025634765625000,
+0.0030822753906250, 0.0022277832031250, 0.0022277832031250, 0.0033264160156250,
+0.0021972656250000, 0.0026550292968750, 0.0021667480468750, 0.0027465820312500,
+0.0021667480468750, 0.0026550292968750, 0.0020446777343750, 0.0024108886718750,
+0.0019226074218750, 0.0024719238281250, 0.0022583007812500, 0.0024719238281250,
+0.0020751953125000, 0.0023498535156250, 0.0019836425781250, 0.0023498535156250,
+0.0022583007812500, 0.0023803710937500, 0.0021057128906250, 0.0023193359375000,
+0.0017395019531250, 0.0025024414062500, 0.0018615722656250, 0.0023803710937500,
+0.0014953613281250, 0.0023193359375000, 0.0009155273437500, 0.0026245117187500,
+0.0011596679687500, 0.0025939941406250, 0.0007019042968750, 0.0020141601562500,
+0.0003356933593750, 0.0023498535156250, 0.0006408691406250, 0.0019836425781250,
+0.0007019042968750, 0.0013732910156250, 0.0003662109375000, 0.0017700195312500,
+0.0012207031250000, 0.0008239746093750, 0.0009765625000000, 0.0009765625000000,
+0.0010681152343750, 0.0007934570312500, 0.0016479492187500, -0.0000305175781250,
+0.0013732910156250, 0.0003051757812500, 0.0016784667968750, 0.0002746582031250,
+0.0017089843750000, -0.0002441406250000, 0.0015258789062500, 0.0001525878906250,
+0.0015563964843750, 0.0005187988281250, 0.0015563964843750, 0.0000000000000000,
+0.0011596679687500, 0.0008544921875000, 0.0012207031250000, 0.0007324218750000,
+0.0010375976562500, 0.0007324218750000, 0.0006103515625000, 0.0012207031250000,
+0.0007019042968750, 0.0009155273437500, 0.0006408691406250, 0.0009155273437500,
+0.0005187988281250, 0.0010070800781250, 0.0006408691406250, 0.0009765625000000,
+0.0007019042968750, 0.0009155273437500, 0.0005493164062500, 0.0009460449218750,
+0.0010681152343750, 0.0007629394531250, 0.0009155273437500, 0.0009155273437500,
+0.0011291503906250, 0.0010070800781250, 0.0015563964843750, 0.0007629394531250,
+0.0012207031250000, 0.0008544921875000, 0.0018310546875000, 0.0012817382812500,
+0.0017089843750000, 0.0010681152343750, 0.0016784667968750, 0.0014648437500000,
+0.0020141601562500, 0.0020141601562500, 0.0017700195312500, 0.0015563964843750,
+0.0017700195312500, 0.0022888183593750, 0.0017700195312500, 0.0022583007812500,
+0.0016479492187500, 0.0021362304687500, 0.0015563964843750, 0.0025329589843750,
+0.0015563964843750, 0.0023498535156250, 0.0016479492187500, 0.0021972656250000,
+0.0014953613281250, 0.0021972656250000, 0.0015563964843750, 0.0021667480468750,
+0.0016784667968750, 0.0021057128906250, 0.0015869140625000, 0.0021667480468750,
+0.0016479492187500, 0.0022888183593750, 0.0016784667968750, 0.0021057128906250,
+0.0016479492187500, 0.0023193359375000, 0.0016174316406250, 0.0026550292968750,
+0.0016174316406250, 0.0024719238281250, 0.0015563964843750, 0.0028991699218750,
+0.0015258789062500, 0.0030517578125000, 0.0014953613281250, 0.0028381347656250,
+0.0013732910156250, 0.0029602050781250, 0.0013427734375000, 0.0033264160156250,
+0.0015258789062500, 0.0027465820312500, 0.0013427734375000, 0.0030517578125000,
+0.0014648437500000, 0.0029296875000000, 0.0017089843750000, 0.0023803710937500,
+0.0013732910156250, 0.0027770996093750, 0.0020446777343750, 0.0024108886718750,
+0.0018310546875000, 0.0022583007812500, 0.0018615722656250, 0.0024414062500000,
+0.0022888183593750, 0.0023498535156250, 0.0020141601562500, 0.0022583007812500,
+0.0023193359375000, 0.0025024414062500, 0.0022277832031250, 0.0022888183593750,
+0.0021667480468750, 0.0024108886718750, 0.0024108886718750, 0.0026245117187500,
+0.0022888183593750, 0.0022888183593750, 0.0021972656250000, 0.0025634765625000,
+0.0023498535156250, 0.0024108886718750, 0.0021362304687500, 0.0023803710937500,
+0.0019226074218750, 0.0025329589843750, 0.0022277832031250, 0.0022888183593750,
+0.0017700195312500, 0.0023498535156250, 0.0018615722656250, 0.0023498535156250,
+0.0018310546875000, 0.0022583007812500, 0.0015563964843750, 0.0023193359375000,
+0.0017395019531250, 0.0022583007812500, 0.0014343261718750, 0.0021667480468750,
+0.0014648437500000, 0.0021972656250000, 0.0014343261718750, 0.0021362304687500,
+0.0012817382812500, 0.0021362304687500, 0.0013427734375000, 0.0021362304687500,
+0.0013122558593750, 0.0020141601562500, 0.0012512207031250, 0.0020141601562500,
+0.0013122558593750, 0.0019836425781250, 0.0013427734375000, 0.0018310546875000,
+0.0011901855468750, 0.0018920898437500, 0.0015563964843750, 0.0019531250000000,
+0.0013732910156250, 0.0017700195312500, 0.0016479492187500, 0.0019836425781250,
+0.0020446777343750, 0.0021667480468750, 0.0017395019531250, 0.0019531250000000,
+0.0025939941406250, 0.0024719238281250, 0.0025329589843750, 0.0023803710937500,
+0.0027160644531250, 0.0025634765625000, 0.0033264160156250, 0.0028991699218750,
+0.0030822753906250, 0.0026855468750000, 0.0034790039062500, 0.0031127929687500,
+0.0036315917968750, 0.0031127929687500, 0.0034790039062500, 0.0030517578125000,
+0.0036010742187500, 0.0032348632812500, 0.0036315917968750, 0.0031738281250000,
+0.0032653808593750, 0.0031433105468750, 0.0034790039062500, 0.0032958984375000,
+0.0032653808593750, 0.0032043457031250, 0.0028686523437500, 0.0030212402343750,
+0.0032043457031250, 0.0032043457031250, 0.0025939941406250, 0.0028381347656250,
+0.0027160644531250, 0.0029602050781250, 0.0025024414062500, 0.0028381347656250,
+0.0020446777343750, 0.0025939941406250, 0.0024414062500000, 0.0027465820312500,
+0.0017395019531250, 0.0024719238281250, 0.0018920898437500, 0.0023803710937500,
+0.0017700195312500, 0.0022888183593750, 0.0012817382812500, 0.0021362304687500,
+0.0016174316406250, 0.0021362304687500, 0.0011596679687500, 0.0018310546875000,
+0.0012512207031250, 0.0018005371093750, 0.0011901855468750, 0.0016784667968750,
+0.0008850097656250, 0.0014038085937500, 0.0010681152343750, 0.0014038085937500,
+0.0008850097656250, 0.0011291503906250, 0.0007629394531250, 0.0011291503906250,
+0.0009765625000000, 0.0010070800781250, 0.0010375976562500, 0.0007019042968750,
+0.0009460449218750, 0.0008239746093750, 0.0016479492187500, 0.0007324218750000,
+0.0014343261718750, 0.0006713867187500, 0.0018920898437500, 0.0008850097656250,
+0.0026245117187500, 0.0010070800781250, 0.0021667480468750, 0.0009460449218750,
+0.0032043457031250, 0.0014953613281250, 0.0032653808593750, 0.0013732910156250,
+0.0031127929687500, 0.0015869140625000, 0.0035400390625000, 0.0020446777343750,
+0.0035095214843750, 0.0017700195312500, 0.0032653808593750, 0.0022277832031250,
+0.0033874511718750, 0.0022277832031250, 0.0033264160156250, 0.0022583007812500,
+0.0031127929687500},
+{0.0032348632812500, 0.0011596679687500, 0.0030212402343750, 0.0010681152343750,
+0.0033264160156250, 0.0010375976562500, 0.0022888183593750, 0.0010681152343750,
+0.0025939941406250, 0.0010681152343750, 0.0021057128906250, 0.0011291503906250,
+0.0012512207031250, 0.0011596679687500, 0.0018615722656250, 0.0010375976562500,
+0.0005187988281250, 0.0012817382812500, 0.0006408691406250, 0.0013732910156250,
+0.0005798339843750, 0.0014038085937500, -0.0001831054687500, 0.0015563964843750,
+0.0001525878906250, 0.0015563964843750, 0.0001831054687500, 0.0016479492187500,
+-0.0000305175781250, 0.0017089843750000, 0.0004577636718750, 0.0018005371093750,
+0.0008239746093750, 0.0018310546875000, 0.0006103515625000, 0.0018310546875000,
+0.0018920898437500, 0.0018615722656250, 0.0016479492187500, 0.0019226074218750,
+0.0021667480468750, 0.0019531250000000, 0.0032958984375000, 0.0019226074218750,
+0.0027465820312500, 0.0019226074218750, 0.0037536621093750, 0.0020446777343750,
+0.0038146972656250, 0.0021057128906250, 0.0036315917968750, 0.0021362304687500,
+0.0040893554687500, 0.0022583007812500, 0.0040283203125000, 0.0023498535156250,
+0.0032958984375000, 0.0021972656250000, 0.0036010742187500, 0.0023803710937500,
+0.0029602050781250, 0.0023498535156250, 0.0020446777343750, 0.0021667480468750,
+0.0025634765625000, 0.0023498535156250, 0.0009765625000000, 0.0020751953125000,
+0.0011291503906250, 0.0021972656250000, 0.0007934570312500, 0.0022583007812500,
+-0.0003356933593750, 0.0020446777343750, 0.0001831054687500, 0.0022277832031250,
+-0.0005187988281250, 0.0020446777343750, -0.0006713867187500, 0.0021972656250000,
+-0.0004272460937500, 0.0021972656250000, -0.0005493164062500, 0.0019836425781250,
+-0.0006103515625000, 0.0021972656250000, -0.0002441406250000, 0.0019531250000000,
+-0.0003051757812500, 0.0021057128906250, -0.0001220703125000, 0.0021362304687500,
+0.0002136230468750, 0.0019836425781250, -0.0000305175781250, 0.0021972656250000,
+0.0005493164062500, 0.0021972656250000, 0.0003967285156250, 0.0021362304687500,
+0.0005798339843750, 0.0022888183593750, 0.0010375976562500, 0.0024108886718750,
+0.0006713867187500, 0.0023193359375000, 0.0014648437500000, 0.0026550292968750,
+0.0012207031250000, 0.0026550292968750, 0.0014343261718750, 0.0026550292968750,
+0.0020446777343750, 0.0028076171875000, 0.0014953613281250, 0.0026855468750000,
+0.0023498535156250, 0.0027160644531250, 0.0021972656250000, 0.0027160644531250,
+0.0022888183593750, 0.0025329589843750, 0.0027770996093750, 0.0024108886718750,
+0.0024414062500000, 0.0023803710937500, 0.0028991699218750, 0.0020751953125000,
+0.0027465820312500, 0.0018920898437500, 0.0027160644531250, 0.0018005371093750,
+0.0030517578125000, 0.0015258789062500, 0.0027770996093750, 0.0014648437500000,
+0.0028991699218750, 0.0013427734375000, 0.0028076171875000, 0.0011901855468750,
+0.0027770996093750, 0.0010986328125000, 0.0027465820312500, 0.0010070800781250,
+0.0025939941406250, 0.0009460449218750, 0.0029602050781250, 0.0008850097656250,
+0.0026550292968750, 0.0008239746093750, 0.0028381347656250, 0.0008850097656250,
+0.0032958984375000, 0.0008544921875000, 0.0028381347656250, 0.0008850097656250,
+0.0033569335937500, 0.0011901855468750, 0.0033874511718750, 0.0010986328125000,
+0.0031433105468750, 0.0012512207031250, 0.0032653808593750, 0.0014648437500000,
+0.0032958984375000, 0.0013122558593750, 0.0026550292968750, 0.0016174316406250,
+0.0029296875000000, 0.0015258789062500, 0.0025634765625000, 0.0017089843750000,
+0.0019226074218750, 0.0020446777343750, 0.0023193359375000, 0.0018005371093750,
+0.0013427734375000, 0.0021362304687500, 0.0014953613281250, 0.0021972656250000,
+0.0013427734375000, 0.0021057128906250, 0.0006713867187500, 0.0021972656250000,
+0.0011596679687500, 0.0022277832031250, 0.0005187988281250, 0.0018310546875000,
+0.0004882812500000, 0.0019531250000000, 0.0004882812500000, 0.0017700195312500,
+0.0001220703125000, 0.0013427734375000, 0.0002746582031250, 0.0016784667968750,
+0.0000610351562500, 0.0013732910156250, -0.0000610351562500, 0.0012817382812500,
+-0.0000610351562500, 0.0013732910156250, -0.0000915527343750, 0.0013732910156250,
+0.0002746582031250, 0.0013427734375000, 0.0005493164062500, 0.0014343261718750,
+0.0003662109375000, 0.0014648437500000, 0.0005493164062500, 0.0013732910156250,
+0.0008850097656250, 0.0014343261718750, 0.0006103515625000, 0.0014953613281250,
+0.0014038085937500, 0.0013122558593750, 0.0013427734375000, 0.0014038085937500,
+0.0013122558593750, 0.0014038085937500, 0.0016784667968750, 0.0012512207031250,
+0.0015563964843750, 0.0013732910156250, 0.0015258789062500, 0.0013732910156250,
+0.0013732910156250, 0.0012512207031250, 0.0014953613281250, 0.0014648437500000,
+0.0015869140625000, 0.0017089843750000, 0.0013122558593750, 0.0016174316406250,
+0.0019226074218750, 0.0018920898437500, 0.0019531250000000, 0.0020141601562500,
+0.0018615722656250, 0.0019836425781250, 0.0021972656250000, 0.0021362304687500,
+0.0022583007812500, 0.0021972656250000, 0.0019531250000000, 0.0019226074218750,
+0.0022277832031250, 0.0021972656250000, 0.0019836425781250, 0.0020141601562500,
+0.0016174316406250, 0.0017089843750000, 0.0020751953125000, 0.0020446777343750,
+0.0013122558593750, 0.0014038085937500, 0.0014648437500000, 0.0015258789062500,
+0.0015563964843750, 0.0014343261718750, 0.0010375976562500, 0.0009765625000000,
+0.0013122558593750, 0.0012207031250000, 0.0014648437500000, 0.0009460449218750,
+0.0013122558593750, 0.0008544921875000, 0.0015563964843750, 0.0010681152343750,
+0.0019226074218750, 0.0010986328125000, 0.0016479492187500, 0.0010070800781250,
+0.0021667480468750, 0.0013427734375000, 0.0022583007812500, 0.0012512207031250,
+0.0020751953125000, 0.0013732910156250, 0.0022583007812500, 0.0016784667968750,
+0.0023498535156250, 0.0014343261718750, 0.0020446777343750, 0.0018005371093750,
+0.0022277832031250, 0.0018005371093750, 0.0021667480468750, 0.0017700195312500,
+0.0019226074218750, 0.0019226074218750, 0.0021667480468750, 0.0018310546875000,
+0.0019531250000000, 0.0018615722656250, 0.0019531250000000, 0.0019226074218750,
+0.0020446777343750, 0.0018310546875000, 0.0020141601562500, 0.0017395019531250,
+0.0020751953125000, 0.0018615722656250, 0.0019531250000000, 0.0015563964843750,
+0.0019836425781250, 0.0016174316406250, 0.0019836425781250, 0.0016174316406250,
+0.0018920898437500, 0.0014648437500000, 0.0018310546875000, 0.0016174316406250,
+0.0018005371093750, 0.0017395019531250, 0.0017700195312500, 0.0015258789062500,
+0.0017395019531250, 0.0018615722656250, 0.0017700195312500, 0.0021362304687500,
+0.0017089843750000, 0.0017700195312500, 0.0015258789062500, 0.0024719238281250,
+0.0015869140625000, 0.0023193359375000, 0.0013732910156250, 0.0023193359375000,
+0.0010070800781250, 0.0027160644531250, 0.0011596679687500, 0.0024414062500000,
+0.0007934570312500, 0.0026245117187500, 0.0007019042968750, 0.0025329589843750,
+0.0007324218750000, 0.0025024414062500, 0.0005187988281250, 0.0025329589843750,
+0.0005187988281250, 0.0024108886718750, 0.0005798339843750, 0.0025634765625000,
+0.0004272460937500, 0.0025634765625000, 0.0006713867187500, 0.0025024414062500,
+0.0009155273437500, 0.0026245117187500, 0.0006103515625000, 0.0026245117187500,
+0.0013122558593750, 0.0026550292968750, 0.0012817382812500, 0.0027770996093750,
+0.0014038085937500, 0.0026550292968750, 0.0018920898437500, 0.0025939941406250,
+0.0016784667968750, 0.0028076171875000, 0.0018920898437500, 0.0021667480468750,
+0.0020751953125000, 0.0024719238281250, 0.0018920898437500, 0.0021667480468750,
+0.0018005371093750, 0.0014648437500000, 0.0019836425781250, 0.0018920898437500,
+0.0014343261718750, 0.0009460449218750, 0.0014953613281250, 0.0010070800781250,
+0.0013732910156250, 0.0008239746093750, 0.0010070800781250, 0.0001525878906250,
+0.0012207031250000, 0.0003662109375000, 0.0008239746093750, 0.0000000000000000,
+0.0008239746093750, -0.0001831054687500, 0.0008239746093750, 0.0000000000000000,
+0.0006103515625000, -0.0000305175781250, 0.0007629394531250, -0.0002136230468750,
+0.0004882812500000, 0.0004882812500000, 0.0006103515625000, 0.0002136230468750,
+0.0005798339843750, 0.0005493164062500, 0.0003967285156250, 0.0012512207031250,
+0.0006103515625000, 0.0008850097656250, 0.0004272460937500, 0.0018920898437500,
+0.0004882812500000, 0.0018615722656250, 0.0005187988281250, 0.0019836425781250,
+0.0003967285156250, 0.0027160644531250, 0.0005798339843750, 0.0024719238281250,
+0.0005493164062500, 0.0025634765625000, 0.0005493164062500, 0.0028076171875000,
+0.0006408691406250, 0.0025024414062500, 0.0006408691406250, 0.0021972656250000,
+0.0006713867187500, 0.0024108886718750, 0.0007324218750000, 0.0017089843750000,
+0.0007019042968750, 0.0017700195312500, 0.0007324218750000, 0.0015869140625000,
+0.0007934570312500, 0.0009765625000000, 0.0007324218750000, 0.0013122558593750,
+0.0009460449218750, 0.0008544921875000, 0.0008544921875000, 0.0007629394531250,
+0.0010070800781250, 0.0009460449218750, 0.0011596679687500, 0.0007934570312500,
+0.0010986328125000, 0.0008239746093750, 0.0014648437500000, 0.0011291503906250,
+0.0014953613281250, 0.0010070800781250, 0.0016174316406250, 0.0011291503906250,
+0.0018005371093750, 0.0014343261718750, 0.0017700195312500, 0.0011596679687500,
+0.0018920898437500, 0.0014953613281250, 0.0019836425781250, 0.0014953613281250,
+0.0020446777343750, 0.0014648437500000, 0.0021362304687500, 0.0015869140625000,
+0.0021667480468750, 0.0015563964843750, 0.0022888183593750, 0.0016784667968750,
+0.0023498535156250, 0.0016479492187500, 0.0024414062500000, 0.0017395019531250,
+0.0025329589843750, 0.0018310546875000, 0.0026245117187500, 0.0017395019531250,
+0.0026550292968750, 0.0020751953125000, 0.0027770996093750, 0.0019836425781250,
+0.0026855468750000, 0.0021972656250000, 0.0026245117187500, 0.0025329589843750,
+0.0027465820312500, 0.0023193359375000, 0.0022583007812500, 0.0028991699218750,
+0.0023803710937500, 0.0028381347656250, 0.0022277832031250, 0.0028991699218750,
+0.0017089843750000, 0.0032958984375000, 0.0020141601562500, 0.0031127929687500,
+0.0015563964843750, 0.0032958984375000, 0.0014343261718750, 0.0033874511718750,
+0.0015563964843750, 0.0031433105468750, 0.0014648437500000, 0.0030517578125000,
+0.0013427734375000, 0.0032043457031250, 0.0016784667968750, 0.0027465820312500,
+0.0015563964843750, 0.0027770996093750, 0.0016479492187500, 0.0027465820312500,
+0.0019226074218750, 0.0024414062500000, 0.0018310546875000, 0.0026245117187500,
+0.0021057128906250, 0.0025329589843750, 0.0021057128906250, 0.0025024414062500,
+0.0021972656250000, 0.0025634765625000, 0.0023498535156250, 0.0026245117187500,
+0.0022583007812500, 0.0026245117187500, 0.0023803710937500, 0.0024108886718750,
+0.0023803710937500, 0.0025634765625000, 0.0023498535156250, 0.0024414062500000,
+0.0024108886718750, 0.0021362304687500, 0.0023803710937500, 0.0022277832031250,
+0.0023193359375000, 0.0019836425781250, 0.0022583007812500, 0.0018615722656250,
+0.0022583007812500, 0.0019226074218750, 0.0022583007812500, 0.0018310546875000,
+0.0022277832031250, 0.0016479492187500, 0.0021057128906250, 0.0021667480468750,
+0.0021057128906250, 0.0019226074218750, 0.0020141601562500, 0.0021667480468750,
+0.0018920898437500, 0.0027160644531250, 0.0018615722656250, 0.0022888183593750,
+0.0017700195312500},
+{0.0017700195312500, 0.0018005371093750, 0.0015563964843750, 0.0011596679687500,
+0.0016479492187500, 0.0012207031250000, 0.0015869140625000, 0.0011291503906250,
+0.0014038085937500, 0.0006103515625000, 0.0016479492187500, 0.0008544921875000,
+0.0012817382812500, 0.0004577636718750, 0.0014343261718750, 0.0003051757812500,
+0.0013427734375000, 0.0004577636718750, 0.0011291503906250, 0.0003662109375000,
+0.0014648437500000, 0.0002746582031250, 0.0010070800781250, 0.0008239746093750,
+0.0011596679687500, 0.0006713867187500, 0.0011596679687500, 0.0009765625000000,
+0.0008850097656250, 0.0015869140625000, 0.0011291503906250, 0.0013427734375000,
+0.0008544921875000, 0.0020751953125000, 0.0009155273437500, 0.0020446777343750,
+0.0009460449218750, 0.0023193359375000, 0.0008239746093750, 0.0028381347656250,
+0.0010070800781250, 0.0026245117187500, 0.0007324218750000, 0.0031127929687500,
+0.0008850097656250, 0.0031433105468750, 0.0007629394531250, 0.0032348632812500,
+0.0005187988281250, 0.0034179687500000, 0.0007934570312500, 0.0033874511718750,
+0.0004577636718750, 0.0035400390625000, 0.0005493164062500, 0.0035705566406250,
+0.0007324218750000, 0.0036926269531250, 0.0006408691406250, 0.0037841796875000,
+0.0006713867187500, 0.0036621093750000, 0.0009460449218750, 0.0040893554687500,
+0.0009460449218750, 0.0039978027343750, 0.0012207031250000, 0.0041503906250000,
+0.0015869140625000, 0.0045166015625000, 0.0015869140625000, 0.0042724609375000,
+0.0018005371093750, 0.0045776367187500, 0.0019836425781250, 0.0045471191406250,
+0.0019531250000000, 0.0044860839843750, 0.0020446777343750, 0.0046386718750000,
+0.0022583007812500, 0.0045471191406250, 0.0017700195312500, 0.0044860839843750,
+0.0019531250000000, 0.0044555664062500, 0.0018920898437500, 0.0043640136718750,
+0.0014953613281250, 0.0042114257812500, 0.0017700195312500, 0.0042419433593750,
+0.0016479492187500, 0.0041503906250000, 0.0015869140625000, 0.0040588378906250,
+0.0017700195312500, 0.0041503906250000, 0.0018920898437500, 0.0040893554687500,
+0.0019226074218750, 0.0039672851562500, 0.0020751953125000, 0.0042419433593750,
+0.0022583007812500, 0.0041809082031250, 0.0021667480468750, 0.0041198730468750,
+0.0021362304687500, 0.0042724609375000, 0.0021972656250000, 0.0041809082031250,
+0.0020141601562500, 0.0038452148437500, 0.0018920898437500, 0.0039672851562500,
+0.0018920898437500, 0.0036315917968750, 0.0018310546875000, 0.0031738281250000,
+0.0017700195312500, 0.0033569335937500, 0.0018005371093750, 0.0026550292968750,
+0.0017700195312500, 0.0026245117187500, 0.0018615722656250, 0.0023803710937500,
+0.0019836425781250, 0.0018615722656250, 0.0018615722656250, 0.0020141601562500,
+0.0021057128906250, 0.0018310546875000, 0.0021057128906250, 0.0016174316406250,
+0.0021362304687500, 0.0017395019531250, 0.0022888183593750, 0.0018310546875000,
+0.0023193359375000, 0.0017395019531250, 0.0022888183593750, 0.0019531250000000,
+0.0023803710937500, 0.0019531250000000, 0.0024108886718750, 0.0019531250000000,
+0.0023193359375000, 0.0019836425781250, 0.0024414062500000, 0.0019836425781250,
+0.0022888183593750, 0.0021667480468750, 0.0022888183593750, 0.0019836425781250,
+0.0021667480468750, 0.0021057128906250, 0.0020751953125000, 0.0024719238281250,
+0.0021057128906250, 0.0021667480468750, 0.0016784667968750, 0.0025939941406250,
+0.0016479492187500, 0.0026550292968750, 0.0015258789062500, 0.0023803710937500,
+0.0010070800781250, 0.0024108886718750, 0.0010681152343750, 0.0018920898437500,
+0.0010986328125000, 0.0023193359375000, 0.0010070800781250, 0.0022888183593750,
+0.0012207031250000, 0.0022583007812500, 0.0013732910156250, 0.0022888183593750,
+0.0012512207031250, 0.0022583007812500, 0.0014038085937500, 0.0024719238281250,
+0.0015258789062500, 0.0023803710937500, 0.0014038085937500, 0.0024414062500000,
+0.0013122558593750, 0.0026550292968750, 0.0014953613281250, 0.0025024414062500,
+0.0012207031250000, 0.0024719238281250, 0.0012817382812500, 0.0024719238281250,
+0.0012817382812500, 0.0022277832031250, 0.0010986328125000, 0.0020751953125000,
+0.0011901855468750, 0.0022888183593750, 0.0011596679687500, 0.0018310546875000,
+0.0011291503906250, 0.0018615722656250, 0.0011291503906250, 0.0018615722656250,
+0.0010986328125000, 0.0017395019531250, 0.0011901855468750, 0.0018615722656250,
+0.0010986328125000, 0.0014343261718750, 0.0010681152343750, 0.0016784667968750,
+0.0011596679687500, 0.0014038085937500, 0.0012512207031250, 0.0008239746093750,
+0.0012817382812500, 0.0010681152343750, 0.0014038085937500, 0.0006713867187500,
+0.0015258789062500, 0.0004272460937500, 0.0015258789062500, 0.0006103515625000,
+0.0016784667968750, 0.0005187988281250, 0.0018310546875000, 0.0003967285156250,
+0.0017395019531250, 0.0010375976562500, 0.0020141601562500, 0.0008850097656250,
+0.0019226074218750, 0.0011291503906250, 0.0016784667968750, 0.0018615722656250,
+0.0020141601562500, 0.0016174316406250, 0.0018920898437500, 0.0015563964843750,
+0.0018615722656250, 0.0019226074218750, 0.0020446777343750, 0.0014953613281250,
+0.0020446777343750, 0.0010070800781250, 0.0019836425781250, 0.0014343261718750,
+0.0022277832031250, 0.0003356933593750, 0.0023193359375000, 0.0003662109375000,
+0.0022888183593750, 0.0003662109375000, 0.0024719238281250, -0.0003662109375000,
+0.0024719238281250, -0.0002441406250000, 0.0021667480468750, -0.0002136230468750,
+0.0023498535156250, -0.0004272460937500, 0.0021057128906250, -0.0003051757812500,
+0.0016174316406250, -0.0001220703125000, 0.0019226074218750, -0.0003662109375000,
+0.0015258789062500, 0.0001220703125000, 0.0013122558593750, 0.0000915527343750,
+0.0015869140625000, 0.0001525878906250, 0.0016174316406250, 0.0005187988281250,
+0.0013122558593750, 0.0003662109375000, 0.0023193359375000, 0.0005493164062500,
+0.0019836425781250, 0.0006103515625000, 0.0023193359375000, 0.0005798339843750,
+0.0032653808593750, 0.0005798339843750, 0.0026855468750000, 0.0005493164062500,
+0.0033874511718750, 0.0005798339843750, 0.0036621093750000, 0.0005798339843750,
+0.0032043457031250, 0.0007019042968750, 0.0032958984375000, 0.0008239746093750,
+0.0035705566406250, 0.0007934570312500, 0.0021667480468750, 0.0010681152343750,
+0.0027160644531250, 0.0010375976562500, 0.0021972656250000, 0.0010986328125000,
+0.0008544921875000, 0.0012512207031250, 0.0017089843750000, 0.0012512207031250,
+0.0003967285156250, 0.0012207031250000, 0.0002746582031250, 0.0012207031250000,
+0.0005798339843750, 0.0012512207031250, 0.0000305175781250, 0.0012817382812500,
+0.0000610351562500, 0.0013427734375000, 0.0007019042968750, 0.0014648437500000,
+0.0003356933593750, 0.0013732910156250, 0.0007629394531250, 0.0015563964843750,
+0.0014648437500000, 0.0018615722656250, 0.0009460449218750, 0.0016174316406250,
+0.0021362304687500, 0.0021667480468750, 0.0019836425781250, 0.0019836425781250,
+0.0020751953125000, 0.0020446777343750, 0.0028686523437500, 0.0024108886718750,
+0.0025024414062500, 0.0021362304687500, 0.0026550292968750, 0.0024108886718750,
+0.0029296875000000, 0.0023803710937500, 0.0024414062500000, 0.0022888183593750,
+0.0021972656250000, 0.0023193359375000, 0.0024719238281250, 0.0022277832031250,
+0.0014953613281250, 0.0020751953125000, 0.0016479492187500, 0.0021362304687500,
+0.0013732910156250, 0.0021057128906250, 0.0007019042968750, 0.0019836425781250,
+0.0010986328125000, 0.0020446777343750, 0.0003662109375000, 0.0020446777343750,
+0.0003356933593750, 0.0020141601562500, 0.0004272460937500, 0.0021057128906250,
+0.0000305175781250, 0.0021362304687500, 0.0001220703125000, 0.0021057128906250,
+0.0003356933593750, 0.0023193359375000, 0.0001525878906250, 0.0022583007812500,
+0.0004272460937500, 0.0022888183593750, 0.0007629394531250, 0.0024108886718750,
+0.0004577636718750, 0.0023498535156250, 0.0015258789062500, 0.0023803710937500,
+0.0011901855468750, 0.0023193359375000, 0.0017395019531250, 0.0023193359375000,
+0.0027160644531250, 0.0024108886718750, 0.0021362304687500, 0.0022583007812500,
+0.0036926269531250, 0.0025329589843750, 0.0035095214843750, 0.0024108886718750,
+0.0038757324218750, 0.0024414062500000, 0.0050048828125000, 0.0026245117187500,
+0.0044250488281250, 0.0023803710937500, 0.0054016113281250, 0.0025939941406250,
+0.0054321289062500, 0.0025024414062500, 0.0052795410156250, 0.0025329589843750,
+0.0056762695312500, 0.0025939941406250, 0.0055847167968750, 0.0024414062500000,
+0.0052490234375000, 0.0026550292968750, 0.0054931640625000, 0.0025634765625000,
+0.0051269531250000, 0.0025939941406250, 0.0046997070312500, 0.0027465820312500,
+0.0050354003906250, 0.0026550292968750, 0.0042724609375000, 0.0027770996093750,
+0.0043640136718750, 0.0028686523437500, 0.0042419433593750, 0.0028076171875000,
+0.0036621093750000, 0.0028076171875000, 0.0040283203125000, 0.0028381347656250,
+0.0036010742187500, 0.0026855468750000, 0.0035705566406250, 0.0027465820312500,
+0.0036315917968750, 0.0027160644531250, 0.0034790039062500, 0.0025634765625000,
+0.0035705566406250, 0.0026245117187500, 0.0034790039062500, 0.0025329589843750,
+0.0035095214843750, 0.0025634765625000, 0.0033264160156250, 0.0025634765625000,
+0.0031738281250000, 0.0024414062500000, 0.0033874511718750, 0.0025024414062500,
+0.0028686523437500, 0.0024414062500000, 0.0030212402343750, 0.0025024414062500,
+0.0028686523437500, 0.0024108886718750, 0.0024414062500000, 0.0023498535156250,
+0.0028076171875000, 0.0024414062500000, 0.0022277832031250, 0.0021972656250000,
+0.0024108886718750, 0.0023803710937500, 0.0023193359375000, 0.0023498535156250,
+0.0018920898437500, 0.0021667480468750, 0.0021972656250000, 0.0023498535156250,
+0.0016784667968750, 0.0021972656250000, 0.0017395019531250, 0.0021667480468750,
+0.0018005371093750, 0.0022888183593750, 0.0014953613281250, 0.0022888183593750,
+0.0016174316406250, 0.0022583007812500, 0.0015563964843750, 0.0024719238281250,
+0.0014648437500000, 0.0024108886718750, 0.0015258789062500, 0.0023803710937500,
+0.0014343261718750, 0.0025634765625000, 0.0013122558593750, 0.0024414062500000,
+0.0014953613281250, 0.0021972656250000, 0.0012207031250000, 0.0024108886718750,
+0.0013122558593750, 0.0022277832031250, 0.0015869140625000, 0.0018615722656250,
+0.0012207031250000, 0.0020446777343750, 0.0016174316406250, 0.0018005371093750,
+0.0014648437500000, 0.0017395019531250, 0.0012817382812500, 0.0018005371093750,
+0.0013732910156250, 0.0017395019531250, 0.0012512207031250, 0.0018005371093750,
+0.0009155273437500, 0.0018615722656250, 0.0009765625000000, 0.0018310546875000,
+0.0007934570312500, 0.0018310546875000, 0.0005493164062500, 0.0018615722656250,
+0.0007019042968750, 0.0018615722656250, 0.0007934570312500, 0.0017395019531250,
+0.0007934570312500, 0.0017089843750000, 0.0008850097656250, 0.0017395019531250,
+0.0011596679687500, 0.0016784667968750, 0.0012207031250000, 0.0017395019531250,
+0.0012207031250000, 0.0018920898437500, 0.0013122558593750, 0.0017700195312500,
+0.0011291503906250, 0.0019226074218750, 0.0009155273437500, 0.0021972656250000,
+0.0010070800781250, 0.0019836425781250, 0.0004577636718750, 0.0022583007812500,
+0.0003662109375000, 0.0023498535156250, 0.0003662109375000, 0.0022583007812500,
+0.0000915527343750, 0.0023193359375000, 0.0000610351562500, 0.0023803710937500,
+0.0003356933593750, 0.0018615722656250, 0.0002136230468750, 0.0021057128906250,
+0.0003967285156250},
+{0.0048522949218750, 0.0003967285156250, 0.0045166015625000, 0.0004272460937500,
+0.0039367675781250, 0.0001831054687500, 0.0042724609375000, 0.0003051757812500,
+0.0034790039062500, 0.0003051757812500, 0.0035400390625000, 0.0003356933593750,
+0.0033874511718750, 0.0004577636718750, 0.0028686523437500, 0.0005493164062500,
+0.0031738281250000, 0.0005493164062500, 0.0025634765625000, 0.0007324218750000,
+0.0026855468750000, 0.0008239746093750, 0.0024719238281250, 0.0008239746093750,
+0.0019531250000000, 0.0009155273437500, 0.0022583007812500, 0.0009765625000000,
+0.0014953613281250, 0.0009460449218750, 0.0015869140625000, 0.0010681152343750,
+0.0012512207031250, 0.0011291503906250, 0.0005798339843750, 0.0011291503906250,
+0.0009765625000000, 0.0011901855468750, 0.0000610351562500, 0.0014343261718750,
+0.0000305175781250, 0.0013732910156250, -0.0000610351562500, 0.0015869140625000,
+-0.0005798339843750, 0.0017700195312500, -0.0004272460937500, 0.0017395019531250,
+-0.0006408691406250, 0.0021972656250000, -0.0006408691406250, 0.0022277832031250,
+-0.0007324218750000, 0.0024719238281250, -0.0008239746093750, 0.0028076171875000,
+-0.0007934570312500, 0.0025939941406250, -0.0010070800781250, 0.0031738281250000,
+-0.0010070800781250, 0.0030822753906250, -0.0010681152343750, 0.0030822753906250,
+-0.0012207031250000, 0.0034179687500000, -0.0011291503906250, 0.0032043457031250,
+-0.0012512207031250, 0.0032653808593750, -0.0012817382812500, 0.0032348632812500,
+-0.0011596679687500, 0.0030822753906250, -0.0011291503906250, 0.0029907226562500,
+-0.0011596679687500, 0.0029296875000000, -0.0009155273437500, 0.0026550292968750,
+-0.0009155273437500, 0.0027160644531250, -0.0008850097656250, 0.0024108886718750,
+-0.0007629394531250, 0.0021667480468750, -0.0007019042968750, 0.0024414062500000,
+-0.0003662109375000, 0.0019226074218750, -0.0004272460937500, 0.0020141601562500,
+-0.0001831054687500, 0.0019226074218750, 0.0001220703125000, 0.0015563964843750,
+0.0000610351562500, 0.0018310546875000, 0.0005798339843750, 0.0017089843750000,
+0.0005187988281250, 0.0016174316406250, 0.0009460449218750, 0.0019226074218750,
+0.0014343261718750, 0.0021057128906250, 0.0011291503906250, 0.0020446777343750,
+0.0021667480468750, 0.0023803710937500, 0.0019836425781250, 0.0025329589843750,
+0.0022583007812500, 0.0025329589843750, 0.0031127929687500, 0.0025939941406250,
+0.0026550292968750, 0.0026855468750000, 0.0032653808593750, 0.0025939941406250,
+0.0033874511718750, 0.0027160644531250, 0.0030822753906250, 0.0025939941406250,
+0.0031738281250000, 0.0023498535156250, 0.0033264160156250, 0.0025939941406250,
+0.0027160644531250, 0.0021972656250000, 0.0028076171875000, 0.0022888183593750,
+0.0026550292968750, 0.0022888183593750, 0.0019226074218750, 0.0020141601562500,
+0.0016479492187500, 0.0019836425781250, 0.0013427734375000, 0.0018920898437500,
+0.0014343261718750, 0.0018920898437500, 0.0014343261718750, 0.0017700195312500,
+0.0012817382812500, 0.0016784667968750, 0.0014038085937500, 0.0016174316406250,
+0.0013427734375000, 0.0014038085937500, 0.0012512207031250, 0.0012817382812500,
+0.0012512207031250, 0.0011901855468750, 0.0013732910156250, 0.0010681152343750,
+0.0013122558593750, 0.0010070800781250, 0.0014038085937500, 0.0008850097656250,
+0.0015258789062500, 0.0007629394531250, 0.0013732910156250, 0.0007629394531250,
+0.0014038085937500, 0.0007629394531250, 0.0015258789062500, 0.0007019042968750,
+0.0010070800781250, 0.0009460449218750, 0.0010986328125000, 0.0008850097656250,
+0.0010375976562500, 0.0010681152343750, 0.0006103515625000, 0.0013122558593750,
+0.0007019042968750, 0.0010986328125000, 0.0007934570312500, 0.0016784667968750,
+0.0006103515625000, 0.0015869140625000, 0.0005493164062500, 0.0016784667968750,
+0.0005493164062500, 0.0019836425781250, 0.0004272460937500, 0.0018310546875000,
+0.0003967285156250, 0.0020751953125000, 0.0001220703125000, 0.0021057128906250,
+0.0003356933593750, 0.0020141601562500, 0.0005493164062500, 0.0020751953125000,
+0.0001220703125000, 0.0021057128906250, 0.0009155273437500, 0.0019531250000000,
+0.0007019042968750, 0.0019836425781250, 0.0007324218750000, 0.0018920898437500,
+0.0012817382812500, 0.0017700195312500, 0.0009155273437500, 0.0018615722656250,
+0.0012207031250000, 0.0015869140625000, 0.0011291503906250, 0.0015869140625000,
+0.0010375976562500, 0.0015258789062500, 0.0010986328125000, 0.0013122558593750,
+0.0009765625000000, 0.0013732910156250, 0.0010681152343750, 0.0013732910156250,
+0.0010070800781250, 0.0012207031250000, 0.0009460449218750, 0.0013732910156250,
+0.0009765625000000, 0.0015258789062500, 0.0009155273437500, 0.0014648437500000,
+0.0010070800781250, 0.0017089843750000, 0.0008850097656250, 0.0017089843750000,
+0.0009765625000000, 0.0017395019531250, 0.0012207031250000, 0.0019226074218750,
+0.0010070800781250, 0.0019531250000000, 0.0014038085937500, 0.0018005371093750,
+0.0013427734375000, 0.0018615722656250, 0.0013427734375000, 0.0018310546875000,
+0.0015563964843750, 0.0016784667968750, 0.0015869140625000, 0.0018005371093750,
+0.0015869140625000, 0.0015258789062500, 0.0016174316406250, 0.0015869140625000,
+0.0015563964843750, 0.0015869140625000, 0.0014648437500000, 0.0014343261718750,
+0.0014953613281250, 0.0014343261718750, 0.0014038085937500, 0.0014648437500000,
+0.0014648437500000, 0.0014038085937500, 0.0013732910156250, 0.0014953613281250,
+0.0012512207031250, 0.0016479492187500, 0.0012512207031250, 0.0014953613281250,
+0.0014343261718750, 0.0018615722656250, 0.0013732910156250, 0.0017395019531250,
+0.0013427734375000, 0.0018310546875000, 0.0015258789062500, 0.0021057128906250,
+0.0014343261718750, 0.0019531250000000, 0.0013732910156250, 0.0021972656250000,
+0.0014648437500000, 0.0022277832031250, 0.0013122558593750, 0.0021972656250000,
+0.0011901855468750, 0.0023193359375000, 0.0013427734375000, 0.0022277832031250,
+0.0009765625000000, 0.0022277832031250, 0.0010681152343750, 0.0022583007812500,
+0.0009460449218750, 0.0021972656250000, 0.0007019042968750, 0.0021057128906250,
+0.0009155273437500, 0.0022277832031250, 0.0006103515625000, 0.0020141601562500,
+0.0005798339843750, 0.0020751953125000, 0.0006103515625000, 0.0020141601562500,
+0.0004882812500000, 0.0018615722656250, 0.0005798339843750, 0.0019531250000000,
+0.0005493164062500, 0.0018615722656250, 0.0005798339843750, 0.0017700195312500,
+0.0006408691406250, 0.0018615722656250, 0.0006713867187500, 0.0018310546875000,
+0.0006408691406250, 0.0018310546875000, 0.0008544921875000, 0.0020446777343750,
+0.0008239746093750, 0.0019531250000000, 0.0009460449218750, 0.0021362304687500,
+0.0011596679687500, 0.0024414062500000, 0.0010375976562500, 0.0022583007812500,
+0.0013732910156250, 0.0029296875000000, 0.0013427734375000, 0.0028381347656250,
+0.0013427734375000, 0.0030517578125000, 0.0015869140625000, 0.0035400390625000,
+0.0014648437500000, 0.0033569335937500, 0.0014953613281250, 0.0037841796875000,
+0.0015258789062500, 0.0039062500000000, 0.0014648437500000, 0.0037841796875000,
+0.0013427734375000, 0.0039672851562500, 0.0014038085937500, 0.0040588378906250,
+0.0012817382812500, 0.0033874511718750, 0.0012207031250000, 0.0036315917968750,
+0.0012817382812500, 0.0032653808593750, 0.0012817382812500, 0.0025024414062500,
+0.0012817382812500, 0.0028686523437500, 0.0014343261718750, 0.0019836425781250,
+0.0013732910156250, 0.0020446777343750, 0.0015869140625000, 0.0017089843750000,
+0.0018310546875000, 0.0010986328125000, 0.0016174316406250, 0.0014648437500000,
+0.0021667480468750, 0.0005798339843750, 0.0021362304687500, 0.0006103515625000,
+0.0022277832031250, 0.0005187988281250, 0.0026245117187500, 0.0000000000000000,
+0.0024108886718750, 0.0001831054687500, 0.0025329589843750, 0.0001220703125000,
+0.0027465820312500, 0.0000000000000000, 0.0024414062500000, 0.0002746582031250,
+0.0022888183593750, 0.0004272460937500, 0.0025024414062500, 0.0002441406250000,
+0.0017700195312500, 0.0008239746093750, 0.0019226074218750, 0.0007629394531250,
+0.0018005371093750, 0.0008544921875000, 0.0012207031250000, 0.0012512207031250,
+0.0014648437500000, 0.0010681152343750, 0.0012512207031250, 0.0009765625000000,
+0.0010681152343750, 0.0011291503906250, 0.0013122558593750, 0.0008544921875000,
+0.0014038085937500, 0.0005493164062500, 0.0011596679687500, 0.0007629394531250,
+0.0016784667968750, 0.0002441406250000, 0.0015869140625000, 0.0001831054687500,
+0.0016479492187500, 0.0002746582031250, 0.0020141601562500, 0.0001220703125000,
+0.0018005371093750, 0.0000305175781250, 0.0020446777343750, 0.0002746582031250,
+0.0021057128906250, 0.0001831054687500, 0.0021362304687500, 0.0002136230468750,
+0.0022277832031250, 0.0003662109375000, 0.0021362304687500, 0.0001831054687500,
+0.0024108886718750, 0.0002746582031250, 0.0023498535156250, 0.0002746582031250,
+0.0024108886718750, 0.0000610351562500, 0.0026245117187500, -0.0000915527343750,
+0.0025329589843750, -0.0001220703125000, 0.0026245117187500, -0.0002136230468750,
+0.0027160644531250, -0.0003356933593750, 0.0025939941406250, -0.0003356933593750,
+0.0025634765625000, -0.0004577636718750, 0.0027465820312500, -0.0005187988281250,
+0.0024414062500000, -0.0003356933593750, 0.0026550292968750, -0.0006408691406250,
+0.0025634765625000, -0.0002746582031250, 0.0022888183593750, 0.0000915527343750,
+0.0025329589843750, -0.0003662109375000, 0.0020751953125000, 0.0005493164062500,
+0.0022277832031250, 0.0003967285156250, 0.0021057128906250, 0.0005493164062500,
+0.0017700195312500, 0.0011901855468750, 0.0020141601562500, 0.0007019042968750,
+0.0016174316406250, 0.0015563964843750, 0.0016174316406250, 0.0015869140625000,
+0.0015258789062500, 0.0015563964843750, 0.0013122558593750, 0.0019836425781250,
+0.0013732910156250, 0.0018005371093750, 0.0011901855468750, 0.0018005371093750,
+0.0011291503906250, 0.0018310546875000, 0.0011596679687500, 0.0018310546875000,
+0.0011291503906250, 0.0017395019531250, 0.0010681152343750, 0.0017700195312500,
+0.0012207031250000, 0.0017395019531250, 0.0011901855468750, 0.0016479492187500,
+0.0012512207031250, 0.0017395019531250, 0.0013732910156250, 0.0018005371093750,
+0.0009460449218750, 0.0011291503906250, 0.0013122558593750, 0.0019836425781250,
+0.0010681152343750, 0.0019531250000000, 0.0012207031250000, 0.0019226074218750,
+0.0014953613281250, 0.0021972656250000, 0.0013427734375000, 0.0021362304687500,
+0.0017089843750000, 0.0018310546875000, 0.0017089843750000, 0.0020141601562500,
+0.0017395019531250, 0.0017700195312500, 0.0019226074218750, 0.0013122558593750,
+0.0018920898437500, 0.0017089843750000, 0.0018005371093750, 0.0011596679687500,
+0.0019531250000000, 0.0012207031250000, 0.0018920898437500, 0.0012512207031250,
+0.0018310546875000, 0.0010375976562500, 0.0019836425781250, 0.0011291503906250,
+0.0018615722656250, 0.0012817382812500, 0.0019836425781250, 0.0012512207031250,
+0.0019226074218750, 0.0012817382812500, 0.0018920898437500, 0.0014648437500000,
+0.0020446777343750, 0.0014648437500000, 0.0016479492187500, 0.0016479492187500,
+0.0017089843750000, 0.0015563964843750, 0.0016479492187500, 0.0016174316406250,
+0.0013122558593750, 0.0018920898437500, 0.0015258789062500, 0.0018615722656250,
+0.0013122558593750, 0.0018920898437500, 0.0013122558593750, 0.0020141601562500,
+0.0013427734375000, 0.0019531250000000, 0.0013732910156250, 0.0018920898437500,
+0.0014038085937500},
+{0.0019531250000000, 0.0011596679687500, 0.0020751953125000, 0.0007629394531250,
+0.0018920898437500, 0.0018310546875000, 0.0020446777343750, 0.0018005371093750,
+0.0019226074218750, 0.0018920898437500, 0.0016479492187500, 0.0025634765625000,
+0.0018920898437500, 0.0023803710937500, 0.0014038085937500, 0.0024719238281250,
+0.0014953613281250, 0.0027160644531250, 0.0014343261718750, 0.0025024414062500,
+0.0011291503906250, 0.0022888183593750, 0.0013427734375000, 0.0025024414062500,
+0.0012207031250000, 0.0019226074218750, 0.0012207031250000, 0.0020751953125000,
+0.0013732910156250, 0.0019531250000000, 0.0014648437500000, 0.0015869140625000,
+0.0014038085937500, 0.0018005371093750, 0.0017700195312500, 0.0016174316406250,
+0.0017089843750000, 0.0014648437500000, 0.0018005371093750, 0.0015563964843750,
+0.0020141601562500, 0.0015258789062500, 0.0018920898437500, 0.0014648437500000,
+0.0021362304687500, 0.0014953613281250, 0.0021057128906250, 0.0014343261718750,
+0.0020446777343750, 0.0012817382812500, 0.0022277832031250, 0.0011596679687500,
+0.0021057128906250, 0.0011596679687500, 0.0020751953125000, 0.0006713867187500,
+0.0021667480468750, 0.0007934570312500, 0.0020751953125000, 0.0006408691406250,
+0.0019226074218750, 0.0002136230468750, 0.0020446777343750, 0.0003662109375000,
+0.0018615722656250, 0.0000305175781250, 0.0018615722656250, 0.0000000000000000,
+0.0018310546875000, -0.0000610351562500, 0.0017700195312500, -0.0003051757812500,
+0.0018920898437500, -0.0002746582031250, 0.0016174316406250, -0.0002136230468750,
+0.0017700195312500, -0.0003662109375000, 0.0015869140625000, -0.0001831054687500,
+0.0013122558593750, 0.0000305175781250, 0.0015869140625000, -0.0001220703125000,
+0.0010375976562500, 0.0002441406250000, 0.0011291503906250, 0.0002746582031250,
+0.0011596679687500, 0.0002746582031250, 0.0008544921875000, 0.0004272460937500,
+0.0010681152343750, 0.0003662109375000, 0.0009460449218750, 0.0007324218750000,
+0.0008850097656250, 0.0005798339843750, 0.0009460449218750, 0.0008239746093750,
+0.0010070800781250, 0.0016479492187500, 0.0016174316406250, -0.0008544921875000,
+0.0016479492187500, -0.0004272460937500, 0.0018615722656250, -0.0009765625000000,
+0.0017089843750000, -0.0002746582031250, 0.0015869140625000, 0.0004272460937500,
+0.0017395019531250, -0.0001220703125000, 0.0013732910156250, 0.0013122558593750,
+0.0014648437500000, 0.0010986328125000, 0.0013122558593750, 0.0013732910156250,
+0.0009765625000000, 0.0024414062500000, 0.0012512207031250, 0.0019226074218750,
+0.0008239746093750, 0.0026855468750000, 0.0009460449218750, 0.0028686523437500,
+0.0008544921875000, 0.0026855468750000, 0.0005798339843750, 0.0028686523437500,
+0.0008544921875000, 0.0029602050781250, 0.0003662109375000, 0.0026245117187500,
+0.0004882812500000, 0.0025634765625000, 0.0005187988281250, 0.0024414062500000,
+0.0001831054687500, 0.0022277832031250, 0.0004577636718750, 0.0021972656250000,
+0.0004577636718750, 0.0021057128906250, 0.0004272460937500, 0.0019226074218750,
+0.0005798339843750, 0.0020141601562500, 0.0007934570312500, 0.0020141601562500,
+0.0008544921875000, 0.0018005371093750, 0.0008239746093750, 0.0021362304687500,
+0.0009460449218750, 0.0021362304687500, 0.0008850097656250, 0.0020751953125000,
+0.0007019042968750, 0.0022583007812500, 0.0008850097656250, 0.0021667480468750,
+0.0005798339843750, 0.0021057128906250, 0.0005493164062500, 0.0021057128906250,
+0.0007019042968750, 0.0019531250000000, 0.0005493164062500, 0.0018310546875000,
+0.0005493164062500, 0.0019531250000000, 0.0012817382812500, 0.0013732910156250,
+0.0010375976562500, 0.0014648437500000, 0.0014648437500000, 0.0013427734375000,
+0.0022888183593750, 0.0009155273437500, 0.0018920898437500, 0.0011291503906250,
+0.0027160644531250, 0.0004272460937500, 0.0028686523437500, 0.0005493164062500,
+0.0026550292968750, 0.0003662109375000, 0.0028991699218750, -0.0001220703125000,
+0.0029602050781250, 0.0000305175781250, 0.0022277832031250, -0.0003051757812500,
+0.0024414062500000, -0.0004272460937500, 0.0020446777343750, -0.0002136230468750,
+0.0012817382812500, -0.0002136230468750, 0.0016479492187500, -0.0003356933593750,
+0.0008239746093750, 0.0001525878906250, 0.0006408691406250, 0.0002136230468750,
+0.0007629394531250, 0.0001525878906250, 0.0003662109375000, 0.0003967285156250,
+0.0002746582031250, 0.0004272460937500, 0.0010986328125000, 0.0002441406250000,
+0.0006103515625000, 0.0003356933593750, 0.0010681152343750, 0.0002746582031250,
+0.0020751953125000, 0.0000000000000000, 0.0013122558593750, 0.0000305175781250,
+0.0027160644531250, -0.0000610351562500, 0.0025939941406250, -0.0002441406250000,
+0.0025634765625000, -0.0001525878906250, 0.0032958984375000, -0.0001220703125000,
+0.0029602050781250, -0.0004272460937500, 0.0025634765625000, 0.0000610351562500,
+0.0029296875000000, -0.0002746582031250, 0.0022277832031250, -0.0000915527343750,
+0.0014343261718750, 0.0004577636718750, 0.0019531250000000, -0.0000610351562500,
+0.0006103515625000, 0.0008239746093750, 0.0005187988281250, 0.0005187988281250,
+0.0004882812500000, 0.0006408691406250, -0.0002136230468750, 0.0013122558593750,
+-0.0000305175781250, 0.0007324218750000, 0.0001525878906250, 0.0014038085937500,
+-0.0001831054687500, 0.0012512207031250, 0.0003967285156250, 0.0013122558593750,
+0.0009765625000000, 0.0017089843750000, 0.0004272460937500, 0.0014038085937500,
+0.0022277832031250, 0.0018615722656250, 0.0018310546875000, 0.0017700195312500,
+0.0023803710937500, 0.0018310546875000, 0.0037841796875000, 0.0021057128906250,
+0.0029907226562500, 0.0018615722656250, 0.0043334960937500, 0.0024108886718750,
+0.0043640136718750, 0.0021972656250000, 0.0042724609375000, 0.0023498535156250,
+0.0047912597656250, 0.0027160644531250, 0.0045776367187500, 0.0023803710937500,
+0.0043945312500000, 0.0028076171875000, 0.0045471191406250, 0.0028076171875000,
+0.0042114257812500, 0.0027770996093750, 0.0038146972656250, 0.0029907226562500,
+0.0040283203125000, 0.0027160644531250, 0.0031127929687500, 0.0029296875000000,
+0.0032653808593750, 0.0028991699218750, 0.0028991699218750, 0.0028686523437500,
+0.0021057128906250, 0.0029907226562500, 0.0025634765625000, 0.0028686523437500,
+0.0013732910156250, 0.0030212402343750, 0.0015258789062500, 0.0029907226562500,
+0.0012512207031250, 0.0029602050781250, 0.0003967285156250, 0.0030822753906250,
+0.0007934570312500, 0.0029907226562500, 0.0001220703125000, 0.0029296875000000,
+-0.0000305175781250, 0.0029296875000000, 0.0001220703125000, 0.0029296875000000,
+-0.0000610351562500, 0.0028381347656250, -0.0001525878906250, 0.0027465820312500,
+0.0002441406250000, 0.0027465820312500, 0.0000610351562500, 0.0027160644531250,
+0.0003051757812500, 0.0026855468750000, 0.0006713867187500, 0.0026245117187500,
+0.0003662109375000, 0.0025939941406250, 0.0008544921875000, 0.0025024414062500,
+0.0008544921875000, 0.0025634765625000, 0.0008239746093750, 0.0025939941406250,
+0.0010375976562500, 0.0024719238281250, 0.0008544921875000, 0.0025024414062500,
+0.0007934570312500, 0.0024719238281250, 0.0008239746093750, 0.0024108886718750,
+0.0007629394531250, 0.0023498535156250, 0.0006103515625000, 0.0023193359375000,
+0.0007324218750000, 0.0022583007812500, 0.0006713867187500, 0.0021972656250000,
+0.0005798339843750, 0.0021667480468750, 0.0007629394531250, 0.0021972656250000,
+0.0008544921875000, 0.0021972656250000, 0.0007019042968750, 0.0021057128906250,
+0.0012817382812500, 0.0022583007812500, 0.0011291503906250, 0.0021972656250000,
+0.0012817382812500, 0.0022583007812500, 0.0016784667968750, 0.0023193359375000,
+0.0014648437500000, 0.0021667480468750, 0.0015258789062500, 0.0023803710937500,
+0.0016174316406250, 0.0022583007812500, 0.0015258789062500, 0.0022583007812500,
+0.0014343261718750, 0.0023498535156250, 0.0014648437500000, 0.0021667480468750,
+0.0014343261718750, 0.0022583007812500, 0.0013122558593750, 0.0021667480468750,
+0.0014038085937500, 0.0020141601562500, 0.0015258789062500, 0.0019531250000000,
+0.0013427734375000, 0.0018615722656250, 0.0018005371093750, 0.0015563964843750,
+0.0016784667968750, 0.0016479492187500, 0.0017395019531250, 0.0014038085937500,
+0.0021057128906250, 0.0010986328125000, 0.0018920898437500, 0.0012817382812500,
+0.0020141601562500, 0.0010070800781250, 0.0021057128906250, 0.0010070800781250,
+0.0019836425781250, 0.0010681152343750, 0.0019836425781250, 0.0010070800781250,
+0.0020141601562500, 0.0010375976562500, 0.0018920898437500, 0.0011901855468750,
+0.0019226074218750, 0.0011596679687500, 0.0019531250000000, 0.0013732910156250,
+0.0019226074218750, 0.0014953613281250, 0.0019226074218750, 0.0014343261718750,
+0.0018615722656250, 0.0016784667968750, 0.0019531250000000, 0.0017089843750000,
+0.0017700195312500, 0.0017395019531250, 0.0016784667968750, 0.0018920898437500,
+0.0018005371093750, 0.0018310546875000, 0.0011291503906250, 0.0018310546875000,
+0.0012817382812500, 0.0018615722656250, 0.0010986328125000, 0.0017700195312500,
+0.0005187988281250, 0.0016784667968750, 0.0007324218750000, 0.0017700195312500,
+0.0004577636718750, 0.0014953613281250, 0.0002746582031250, 0.0014648437500000,
+0.0003051757812500, 0.0013732910156250, 0.0003051757812500, 0.0011901855468750,
+0.0002136230468750, 0.0012207031250000, 0.0003967285156250, 0.0010070800781250,
+0.0002441406250000, 0.0010070800781250, 0.0004272460937500, 0.0009155273437500,
+0.0006713867187500, 0.0006713867187500, 0.0005187988281250, 0.0006408691406250,
+0.0015258789062500, 0.0005798339843750, 0.0010986328125000, 0.0005798339843750,
+0.0013122558593750, 0.0006103515625000, 0.0021972656250000, 0.0005798339843750,
+0.0018310546875000, 0.0005493164062500, 0.0025939941406250, 0.0006103515625000,
+0.0027465820312500, 0.0005493164062500, 0.0028076171875000, 0.0006103515625000,
+0.0031433105468750, 0.0007324218750000, 0.0030822753906250, 0.0005798339843750,
+0.0030517578125000, 0.0010070800781250, 0.0030822753906250, 0.0009155273437500,
+0.0028991699218750, 0.0010070800781250, 0.0026855468750000, 0.0013122558593750,
+0.0027770996093750, 0.0010375976562500, 0.0025329589843750, 0.0014343261718750,
+0.0027160644531250, 0.0013732910156250, 0.0026245117187500, 0.0013427734375000,
+0.0023803710937500, 0.0014648437500000, 0.0027160644531250, 0.0012817382812500,
+0.0019531250000000, 0.0015563964843750, 0.0022277832031250, 0.0015258789062500,
+0.0018920898437500, 0.0014343261718750, 0.0013122558593750, 0.0015869140625000,
+0.0018005371093750, 0.0015563964843750, 0.0010986328125000, 0.0015869140625000,
+0.0011291503906250, 0.0015258789062500, 0.0011596679687500, 0.0016174316406250,
+0.0007324218750000, 0.0017700195312500, 0.0009460449218750, 0.0016479492187500,
+0.0007934570312500, 0.0020446777343750, 0.0006713867187500, 0.0019226074218750,
+0.0007934570312500, 0.0021362304687500, 0.0008544921875000, 0.0025024414062500,
+0.0007019042968750, 0.0023193359375000, 0.0007019042968750, 0.0026245117187500,
+0.0008239746093750, 0.0028686523437500, 0.0005187988281250, 0.0027770996093750,
+0.0003967285156250, 0.0028076171875000, 0.0005493164062500, 0.0028686523437500,
+-0.0000305175781250, 0.0026855468750000, 0.0001220703125000, 0.0027160644531250,
+-0.0000305175781250, 0.0026855468750000, -0.0003967285156250, 0.0024719238281250,
+-0.0001220703125000, 0.0025329589843750, -0.0005187988281250, 0.0023498535156250,
+-0.0004577636718750},
+{0.0018310546875000, 0.0020141601562500, 0.0017089843750000, 0.0018920898437500,
+0.0017700195312500, 0.0021057128906250, 0.0016174316406250, 0.0025634765625000,
+0.0015563964843750, 0.0021972656250000, 0.0018920898437500, 0.0027160644531250,
+0.0018005371093750, 0.0026855468750000, 0.0018310546875000, 0.0025024414062500,
+0.0021362304687500, 0.0027160644531250, 0.0020446777343750, 0.0026855468750000,
+0.0021972656250000, 0.0024108886718750, 0.0023498535156250, 0.0025329589843750,
+0.0023193359375000, 0.0023803710937500, 0.0022888183593750, 0.0021057128906250,
+0.0024414062500000, 0.0022277832031250, 0.0023803710937500, 0.0018920898437500,
+0.0023803710937500, 0.0019836425781250, 0.0025329589843750, 0.0018310546875000,
+0.0025939941406250, 0.0015258789062500, 0.0025634765625000, 0.0017395019531250,
+0.0026245117187500, 0.0012207031250000, 0.0027770996093750, 0.0013122558593750,
+0.0027160644531250, 0.0013122558593750, 0.0025329589843750, 0.0009765625000000,
+0.0028076171875000, 0.0011901855468750, 0.0025634765625000, 0.0010375976562500,
+0.0025024414062500, 0.0011596679687500, 0.0024414062500000, 0.0011596679687500,
+0.0022277832031250, 0.0011291503906250, 0.0021362304687500, 0.0012512207031250,
+0.0016784667968750, 0.0011291503906250, 0.0017395019531250, 0.0011901855468750,
+0.0013427734375000, 0.0012207031250000, 0.0005187988281250, 0.0011291503906250,
+0.0002136230468750, 0.0010681152343750, -0.0004882812500000, 0.0009765625000000,
+-0.0004577636718750, 0.0010070800781250, -0.0003967285156250, 0.0010070800781250,
+-0.0009155273437500, 0.0009460449218750, -0.0008239746093750, 0.0009155273437500,
+-0.0006408691406250, 0.0010986328125000, -0.0006103515625000, 0.0010375976562500,
+-0.0003662109375000, 0.0011596679687500, -0.0001525878906250, 0.0012817382812500,
+-0.0000610351562500, 0.0011596679687500, 0.0005493164062500, 0.0014648437500000,
+0.0004882812500000, 0.0013427734375000, 0.0008544921875000, 0.0013427734375000,
+0.0014343261718750, 0.0015258789062500, 0.0011901855468750, 0.0012207031250000,
+0.0016784667968750, 0.0016479492187500, 0.0018005371093750, 0.0015869140625000,
+0.0016784667968750, 0.0016174316406250, 0.0017700195312500, 0.0018005371093750,
+0.0017089843750000, 0.0016479492187500, 0.0014648437500000, 0.0018920898437500,
+0.0014648437500000, 0.0018310546875000, 0.0013427734375000, 0.0018615722656250,
+0.0010681152343750, 0.0019531250000000, 0.0010375976562500, 0.0018005371093750,
+0.0008850097656250, 0.0022583007812500, 0.0008850097656250, 0.0021667480468750,
+0.0007629394531250, 0.0022888183593750, 0.0005798339843750, 0.0026550292968750,
+0.0006713867187500, 0.0025329589843750, 0.0003356933593750, 0.0028686523437500,
+0.0003356933593750, 0.0029907226562500, 0.0003662109375000, 0.0029296875000000,
+0.0002441406250000, 0.0031127929687500, 0.0002136230468750, 0.0032043457031250,
+0.0003051757812500, 0.0028991699218750, 0.0003051757812500, 0.0030212402343750,
+0.0002746582031250, 0.0029907226562500, 0.0003967285156250, 0.0027770996093750,
+0.0004882812500000, 0.0029296875000000, 0.0003967285156250, 0.0027770996093750,
+0.0004577636718750, 0.0027465820312500, 0.0005798339843750, 0.0028991699218750,
+0.0006713867187500, 0.0029602050781250, 0.0007019042968750, 0.0029296875000000,
+0.0011291503906250, 0.0031433105468750, 0.0010375976562500, 0.0030822753906250,
+0.0013122558593750, 0.0032348632812500, 0.0017089843750000, 0.0034484863281250,
+0.0014648437500000, 0.0033569335937500, 0.0021362304687500, 0.0037231445312500,
+0.0020141601562500, 0.0036315917968750, 0.0020446777343750, 0.0037231445312500,
+0.0024719238281250, 0.0040283203125000, 0.0022583007812500, 0.0038452148437500,
+0.0023193359375000, 0.0040588378906250, 0.0023803710937500, 0.0040283203125000,
+0.0021362304687500, 0.0040588378906250, 0.0020141601562500, 0.0041503906250000,
+0.0021057128906250, 0.0039978027343750, 0.0019226074218750, 0.0042724609375000,
+0.0018005371093750, 0.0041809082031250, 0.0018920898437500, 0.0042724609375000,
+0.0018615722656250, 0.0045166015625000, 0.0017700195312500, 0.0043334960937500,
+0.0020141601562500, 0.0045776367187500, 0.0018615722656250, 0.0046691894531250,
+0.0017700195312500, 0.0045166015625000, 0.0019226074218750, 0.0045776367187500,
+0.0018005371093750, 0.0046691894531250, 0.0017395019531250, 0.0042724609375000,
+0.0017089843750000, 0.0045166015625000, 0.0016174316406250, 0.0043334960937500,
+0.0015563964843750, 0.0039062500000000, 0.0015563964843750, 0.0042419433593750,
+0.0014343261718750, 0.0036621093750000, 0.0014038085937500, 0.0037841796875000,
+0.0014343261718750, 0.0036315917968750, 0.0013427734375000, 0.0032043457031250,
+0.0012207031250000, 0.0034790039062500, 0.0014038085937500, 0.0026855468750000,
+0.0012817382812500, 0.0028686523437500, 0.0013732910156250, 0.0024719238281250,
+0.0015258789062500, 0.0017395019531250, 0.0013427734375000, 0.0021667480468750,
+0.0018005371093750, 0.0010375976562500, 0.0017700195312500, 0.0009765625000000,
+0.0018005371093750, 0.0007629394531250, 0.0020446777343750, 0.0000610351562500,
+0.0019531250000000, 0.0002136230468750, 0.0018920898437500, 0.0000305175781250,
+0.0019531250000000, -0.0002136230468750, 0.0018310546875000, 0.0000305175781250,
+0.0017089843750000, 0.0001220703125000, 0.0017089843750000, -0.0000610351562500,
+0.0016174316406250, 0.0006713867187500, 0.0015869140625000, 0.0004272460937500,
+0.0016174316406250, 0.0007019042968750, 0.0015869140625000, 0.0012817382812500,
+0.0015563964843750, 0.0008850097656250, 0.0018005371093750, 0.0017089843750000,
+0.0016784667968750, 0.0015869140625000, 0.0017395019531250, 0.0017395019531250,
+0.0020446777343750, 0.0023193359375000, 0.0018310546875000, 0.0020141601562500,
+0.0021057128906250, 0.0025634765625000, 0.0021057128906250, 0.0024414062500000,
+0.0020446777343750, 0.0026550292968750, 0.0021972656250000, 0.0030517578125000,
+0.0021972656250000, 0.0027465820312500, 0.0021057128906250, 0.0034484863281250,
+0.0021362304687500, 0.0034790039062500, 0.0021362304687500, 0.0034484863281250,
+0.0020141601562500, 0.0038146972656250, 0.0020446777343750, 0.0037536621093750,
+0.0020751953125000, 0.0035095214843750, 0.0020141601562500, 0.0036621093750000,
+0.0020751953125000, 0.0034790039062500, 0.0021362304687500, 0.0031433105468750,
+0.0020751953125000, 0.0033264160156250, 0.0021667480468750, 0.0028991699218750,
+0.0021972656250000, 0.0028991699218750, 0.0021057128906250, 0.0028991699218750,
+0.0020446777343750, 0.0025939941406250, 0.0020446777343750, 0.0027160644531250,
+0.0018920898437500, 0.0023803710937500, 0.0018615722656250, 0.0024108886718750,
+0.0018005371093750, 0.0023193359375000, 0.0016784667968750, 0.0020141601562500,
+0.0016174316406250, 0.0020751953125000, 0.0016784667968750, 0.0015563964843750,
+0.0014648437500000, 0.0015563964843750, 0.0015258789062500, 0.0013732910156250,
+0.0016479492187500, 0.0009765625000000, 0.0014343261718750, 0.0011596679687500,
+0.0016174316406250, 0.0009460449218750, 0.0014953613281250, 0.0008239746093750,
+0.0015563964843750, 0.0011291503906250, 0.0016784667968750, 0.0012207031250000,
+0.0014648437500000, 0.0011596679687500, 0.0018615722656250, 0.0017089843750000,
+0.0017700195312500, 0.0018005371093750, 0.0018005371093750, 0.0018310546875000,
+0.0020751953125000, 0.0021667480468750, 0.0019226074218750, 0.0021667480468750,
+0.0019531250000000, 0.0019531250000000, 0.0020751953125000, 0.0022888183593750,
+0.0018615722656250, 0.0018920898437500, 0.0017089843750000, 0.0013732910156250,
+0.0017700195312500, 0.0017700195312500, 0.0013427734375000, 0.0008239746093750,
+0.0014038085937500, 0.0009155273437500, 0.0012512207031250, 0.0007934570312500,
+0.0009460449218750, 0.0002136230468750, 0.0010375976562500, 0.0005798339843750,
+0.0005798339843750, 0.0003051757812500, 0.0005493164062500, 0.0002441406250000,
+0.0004272460937500, 0.0003051757812500, 0.0001220703125000, 0.0002136230468750,
+0.0003051757812500, 0.0002136230468750, 0.0002441406250000, -0.0000305175781250,
+0.0000915527343750, 0.0000915527343750, 0.0002441406250000, 0.0000305175781250,
+0.0003356933593750, -0.0002136230468750, 0.0002441406250000, -0.0000305175781250,
+0.0007324218750000, 0.0000610351562500, 0.0005493164062500, 0.0000305175781250,
+0.0008544921875000, 0.0002136230468750, 0.0013122558593750, 0.0002746582031250,
+0.0009765625000000, 0.0001525878906250, 0.0018615722656250, 0.0007934570312500,
+0.0017700195312500, 0.0005187988281250, 0.0018615722656250, 0.0008850097656250,
+0.0025024414062500, 0.0015869140625000, 0.0021972656250000, 0.0010070800781250,
+0.0023498535156250, 0.0017395019531250, 0.0025329589843750, 0.0018310546875000,
+0.0023498535156250, 0.0015258789062500, 0.0021057128906250, 0.0014953613281250,
+0.0016174316406250, -0.0000610351562500, 0.0015869140625000, 0.0007019042968750,
+0.0015869140625000, 0.0004882812500000, 0.0016174316406250, 0.0005798339843750,
+0.0016784667968750, 0.0010070800781250, 0.0017089843750000, 0.0008850097656250,
+0.0017395019531250, 0.0007629394531250, 0.0017395019531250, 0.0010070800781250,
+0.0018310546875000, 0.0007629394531250, 0.0019226074218750, 0.0003967285156250,
+0.0019226074218750, 0.0007934570312500, 0.0019531250000000, -0.0000305175781250,
+0.0021057128906250, -0.0000305175781250, 0.0020446777343750, -0.0000305175781250,
+0.0019836425781250, -0.0004272460937500, 0.0021362304687500, -0.0003662109375000,
+0.0016784667968750, 0.0000000000000000, 0.0018615722656250, -0.0001831054687500,
+0.0016174316406250, 0.0000915527343750, 0.0011596679687500, 0.0005187988281250,
+0.0014648437500000, 0.0003051757812500, 0.0010375976562500, 0.0013122558593750,
+0.0008850097656250, 0.0011291503906250, 0.0011291503906250, 0.0014343261718750,
+0.0010986328125000, 0.0022583007812500, 0.0010375976562500, 0.0018310546875000,
+0.0016479492187500, 0.0023193359375000, 0.0014953613281250, 0.0025024414062500,
+0.0018310546875000, 0.0022277832031250, 0.0024414062500000, 0.0021972656250000,
+0.0020751953125000, 0.0021667480468750, 0.0026855468750000, 0.0018920898437500,
+0.0027160644531250, 0.0017395019531250, 0.0025634765625000, 0.0016784667968750,
+0.0027770996093750, 0.0014953613281250, 0.0026550292968750, 0.0013732910156250,
+0.0024719238281250, 0.0012512207031250, 0.0025329589843750, 0.0012817382812500,
+0.0024108886718750, 0.0011901855468750, 0.0022277832031250, 0.0010681152343750,
+0.0023498535156250, 0.0011901855468750, 0.0023498535156250, 0.0011596679687500,
+0.0021972656250000, 0.0011291503906250, 0.0025939941406250, 0.0013732910156250,
+0.0029602050781250, 0.0015258789062500, 0.0026855468750000, 0.0014343261718750,
+0.0036621093750000, 0.0018615722656250, 0.0035705566406250, 0.0018920898437500,
+0.0037536621093750, 0.0018920898437500, 0.0044555664062500, 0.0021362304687500,
+0.0042114257812500, 0.0020751953125000, 0.0043945312500000, 0.0017089843750000,
+0.0046997070312500, 0.0019226074218750, 0.0042114257812500, 0.0016784667968750,
+0.0038757324218750, 0.0012207031250000, 0.0043029785156250, 0.0014648437500000,
+0.0028991699218750, 0.0009765625000000, 0.0032348632812500, 0.0010070800781250,
+0.0028381347656250, 0.0010986328125000, 0.0017089843750000, 0.0009765625000000,
+0.0024108886718750, 0.0011291503906250, 0.0015258789062500, 0.0011901855468750,
+0.0014038085937500, 0.0012817382812500, 0.0017395019531250, 0.0013732910156250,
+0.0015258789062500},
+{0.0012817382812500, 0.0010070800781250, 0.0013732910156250, 0.0008544921875000,
+0.0011596679687500, 0.0008850097656250, 0.0015258789062500, 0.0004272460937500,
+0.0014038085937500, 0.0005187988281250, 0.0014953613281250, 0.0003051757812500,
+0.0017700195312500, -0.0001220703125000, 0.0015563964843750, 0.0000000000000000,
+0.0018615722656250, -0.0003356933593750, 0.0018920898437500, -0.0004272460937500,
+0.0017700195312500, -0.0003356933593750, 0.0018005371093750, -0.0003967285156250,
+0.0018615722656250, -0.0004882812500000, 0.0016479492187500, -0.0001525878906250,
+0.0017089843750000, -0.0003051757812500, 0.0017089843750000, -0.0002136230468750,
+0.0014953613281250, 0.0003356933593750, 0.0013122558593750, 0.0001831054687500,
+0.0015869140625000, -0.0009460449218750, 0.0015258789062500, -0.0006408691406250,
+0.0017700195312500, -0.0006408691406250, 0.0020751953125000, -0.0010375976562500,
+0.0019836425781250, -0.0008850097656250, 0.0025024414062500, -0.0002136230468750,
+0.0024414062500000, -0.0004272460937500, 0.0024719238281250, -0.0000610351562500,
+0.0027770996093750, 0.0005493164062500, 0.0026550292968750, 0.0002746582031250,
+0.0026855468750000, 0.0008544921875000, 0.0027465820312500, 0.0009765625000000,
+0.0025329589843750, 0.0008850097656250, 0.0024108886718750, 0.0010681152343750,
+0.0024108886718750, 0.0011901855468750, 0.0022888183593750, 0.0008850097656250,
+0.0020141601562500, 0.0009765625000000, 0.0022583007812500, 0.0008239746093750,
+0.0024719238281250, 0.0005798339843750, 0.0021057128906250, 0.0007934570312500,
+0.0028686523437500, 0.0004577636718750, 0.0027465820312500, 0.0003356933593750,
+0.0027770996093750, 0.0007019042968750, 0.0032653808593750, 0.0006408691406250,
+0.0030517578125000, 0.0004272460937500, 0.0031433105468750, 0.0013732910156250,
+0.0031433105468750, 0.0010375976562500, 0.0029296875000000, 0.0014038085937500,
+0.0028381347656250, 0.0021972656250000, 0.0027465820312500, 0.0015869140625000,
+0.0023193359375000, 0.0025939941406250, 0.0022277832031250, 0.0025329589843750,
+0.0020751953125000, 0.0025634765625000, 0.0017089843750000, 0.0031738281250000,
+0.0017089843750000, 0.0028076171875000, 0.0018310546875000, 0.0029296875000000,
+0.0015563964843750, 0.0029296875000000, 0.0017700195312500, 0.0025939941406250,
+0.0019531250000000, 0.0023498535156250, 0.0017089843750000, 0.0022277832031250,
+0.0024719238281250, 0.0019531250000000, 0.0022888183593750, 0.0017089843750000,
+0.0024414062500000, 0.0018005371093750, 0.0030822753906250, 0.0018005371093750,
+0.0028381347656250, 0.0015563964843750, 0.0028076171875000, 0.0019226074218750,
+0.0031127929687500, 0.0017089843750000, 0.0027160644531250, 0.0016784667968750,
+0.0023498535156250, 0.0019226074218750, 0.0027770996093750, 0.0016479492187500,
+0.0015869140625000, 0.0016479492187500, 0.0017395019531250, 0.0017089843750000,
+0.0015563964843750, 0.0013732910156250, 0.0007324218750000, 0.0010986328125000,
+0.0010681152343750, 0.0011596679687500, 0.0007019042968750, 0.0005493164062500,
+0.0004577636718750, 0.0005798339843750, 0.0007019042968750, 0.0002136230468750,
+0.0007324218750000, -0.0003662109375000, 0.0006103515625000, -0.0001525878906250,
+0.0012207031250000, -0.0008239746093750, 0.0010375976562500, -0.0009155273437500,
+0.0012817382812500, -0.0006713867187500, 0.0018615722656250, -0.0008850097656250,
+0.0014343261718750, -0.0009765625000000, 0.0018310546875000, -0.0005187988281250,
+0.0019226074218750, -0.0005493164062500, 0.0016479492187500, -0.0004882812500000,
+0.0016174316406250, -0.0000915527343750, 0.0016174316406250, -0.0002136230468750,
+0.0011291503906250, -0.0002441406250000, 0.0012207031250000, -0.0001220703125000,
+0.0009460449218750, -0.0002441406250000, 0.0004882812500000, -0.0004577636718750,
+0.0007629394531250, -0.0003051757812500, 0.0002746582031250, -0.0005798339843750,
+0.0002136230468750, -0.0006408691406250, 0.0003662109375000, -0.0005493164062500,
+0.0002746582031250, -0.0007019042968750, 0.0003356933593750, -0.0007019042968750,
+0.0006408691406250, -0.0004577636718750, 0.0007324218750000, -0.0005187988281250,
+0.0009460449218750, -0.0004272460937500, 0.0012817382812500, -0.0001525878906250,
+0.0012817382812500, -0.0001220703125000, 0.0015869140625000, 0.0001831054687500,
+0.0017089843750000, 0.0002441406250000, 0.0017089843750000, 0.0003662109375000,
+0.0018615722656250, 0.0005493164062500, 0.0019836425781250, 0.0006103515625000,
+0.0017700195312500, 0.0007629394531250, 0.0018310546875000, 0.0007934570312500,
+0.0018310546875000, 0.0008544921875000, 0.0016479492187500, 0.0009460449218750,
+0.0017395019531250, 0.0008544921875000, 0.0016479492187500, 0.0010681152343750,
+0.0016784667968750, 0.0009765625000000, 0.0015258789062500, 0.0010070800781250,
+0.0014343261718750, 0.0011596679687500, 0.0014648437500000, 0.0009765625000000,
+0.0013122558593750, 0.0010986328125000, 0.0012817382812500, 0.0010375976562500,
+0.0013732910156250, 0.0010375976562500, 0.0013427734375000, 0.0011291503906250,
+0.0013122558593750, 0.0010375976562500, 0.0016479492187500, 0.0010375976562500,
+0.0015869140625000, 0.0010681152343750, 0.0018005371093750, 0.0010375976562500,
+0.0021972656250000, 0.0010681152343750, 0.0020141601562500, 0.0010681152343750,
+0.0025024414062500, 0.0011291503906250, 0.0026245117187500, 0.0011596679687500,
+0.0025024414062500, 0.0010681152343750, 0.0026855468750000, 0.0009765625000000,
+0.0027160644531250, 0.0010070800781250, 0.0023193359375000, 0.0008544921875000,
+0.0024719238281250, 0.0007934570312500, 0.0021667480468750, 0.0007629394531250,
+0.0017089843750000, 0.0006408691406250, 0.0019836425781250, 0.0005493164062500,
+0.0012207031250000, 0.0007324218750000, 0.0012512207031250, 0.0006713867187500,
+0.0012512207031250, 0.0006408691406250, 0.0007934570312500, 0.0007629394531250,
+0.0009155273437500, 0.0007019042968750, 0.0008850097656250, 0.0008239746093750,
+0.0006713867187500, 0.0007934570312500, 0.0008544921875000, 0.0009155273437500,
+0.0009460449218750, 0.0010681152343750, 0.0007629394531250, 0.0008850097656250,
+0.0010681152343750, 0.0012817382812500, 0.0009765625000000, 0.0011901855468750,
+0.0010070800781250, 0.0013122558593750, 0.0012207031250000, 0.0015258789062500,
+0.0010681152343750, 0.0014648437500000, 0.0014038085937500, 0.0019226074218750,
+0.0012817382812500, 0.0018310546875000, 0.0013732910156250, 0.0019531250000000,
+0.0016479492187500, 0.0022277832031250, 0.0014038085937500, 0.0019836425781250,
+0.0018005371093750, 0.0023193359375000, 0.0017395019531250, 0.0022583007812500,
+0.0017700195312500, 0.0020751953125000, 0.0020446777343750, 0.0021362304687500,
+0.0019226074218750, 0.0019531250000000, 0.0021057128906250, 0.0015869140625000,
+0.0021972656250000, 0.0015563964843750, 0.0021362304687500, 0.0013122558593750,
+0.0022277832031250, 0.0008544921875000, 0.0022277832031250, 0.0009765625000000,
+0.0018615722656250, 0.0005798339843750, 0.0020751953125000, 0.0005493164062500,
+0.0018310546875000, 0.0004882812500000, 0.0013732910156250, 0.0002136230468750,
+0.0017395019531250, 0.0003356933593750, 0.0011901855468750, 0.0003051757812500,
+0.0011901855468750, 0.0002136230468750, 0.0012512207031250, 0.0004272460937500,
+0.0010986328125000, 0.0005187988281250, 0.0011901855468750, 0.0002746582031250,
+0.0012207031250000, 0.0008850097656250, 0.0013427734375000, 0.0007019042968750,
+0.0015869140625000, 0.0009460449218750, 0.0016479492187500, 0.0014648437500000,
+0.0018005371093750, 0.0011901855468750, 0.0020751953125000, 0.0016174316406250,
+0.0020751953125000, 0.0016174316406250, 0.0020751953125000, 0.0017395019531250,
+0.0022277832031250, 0.0020141601562500, 0.0022277832031250, 0.0018615722656250,
+0.0020446777343750, 0.0022277832031250, 0.0020446777343750, 0.0021667480468750,
+0.0020141601562500, 0.0021362304687500, 0.0018310546875000, 0.0023498535156250,
+0.0018920898437500, 0.0022888183593750, 0.0018615722656250, 0.0022277832031250,
+0.0017395019531250, 0.0022277832031250, 0.0018005371093750, 0.0021972656250000,
+0.0019226074218750, 0.0021362304687500, 0.0017089843750000, 0.0020751953125000,
+0.0017089843750000, 0.0023193359375000, 0.0017395019531250, 0.0021972656250000,
+0.0018005371093750, 0.0022277832031250, 0.0018005371093750, 0.0024414062500000,
+0.0018920898437500, 0.0023193359375000, 0.0020141601562500, 0.0024719238281250,
+0.0019531250000000, 0.0024414062500000, 0.0019531250000000, 0.0023193359375000,
+0.0022583007812500, 0.0024108886718750, 0.0022277832031250, 0.0023498535156250,
+0.0016479492187500, 0.0021972656250000, 0.0020751953125000, 0.0021362304687500,
+0.0015258789062500, 0.0020751953125000, 0.0006713867187500, 0.0019226074218750,
+0.0011901855468750, 0.0018920898437500, -0.0001220703125000, 0.0019531250000000,
+-0.0001525878906250, 0.0018310546875000, -0.0001831054687500, 0.0019226074218750,
+-0.0009155273437500, 0.0019836425781250, -0.0006713867187500, 0.0018310546875000,
+-0.0004577636718750, 0.0022277832031250, -0.0007019042968750, 0.0020751953125000,
+-0.0002136230468750, 0.0020141601562500, 0.0003662109375000, 0.0021972656250000,
+0.0000610351562500, 0.0020751953125000, 0.0012512207031250, 0.0019836425781250,
+0.0011901855468750, 0.0019226074218750, 0.0014953613281250, 0.0018310546875000,
+0.0023193359375000, 0.0017089843750000, 0.0019836425781250, 0.0015869140625000,
+0.0026245117187500, 0.0014953613281250, 0.0027770996093750, 0.0013732910156250,
+0.0026245117187500, 0.0013427734375000, 0.0028076171875000, 0.0013122558593750,
+0.0028686523437500, 0.0011596679687500, 0.0023803710937500, 0.0013427734375000,
+0.0025634765625000, 0.0012512207031250, 0.0023193359375000, 0.0011901855468750,
+0.0018005371093750, 0.0013427734375000, 0.0020751953125000, 0.0011901855468750,
+0.0014343261718750, 0.0013122558593750, 0.0014343261718750, 0.0012512207031250,
+0.0012817382812500, 0.0012512207031250, 0.0009155273437500, 0.0013427734375000,
+0.0009460449218750, 0.0012207031250000, 0.0006408691406250, 0.0013122558593750,
+0.0006103515625000, 0.0013732910156250, 0.0004882812500000, 0.0012817382812500,
+0.0003662109375000, 0.0013732910156250, 0.0004272460937500, 0.0014343261718750,
+0.0000610351562500, 0.0012817382812500, 0.0002136230468750, 0.0013732910156250,
+0.0001525878906250, 0.0014648437500000, -0.0002136230468750, 0.0014038085937500,
+-0.0000305175781250, 0.0015258789062500, -0.0001525878906250, 0.0014648437500000,
+-0.0002136230468750, 0.0015869140625000, -0.0001220703125000, 0.0014953613281250,
+-0.0000915527343750, 0.0013732910156250, -0.0001525878906250, 0.0016174316406250,
+-0.0001525878906250, 0.0011901855468750, -0.0000305175781250, 0.0013122558593750,
+-0.0002136230468750, 0.0013122558593750, -0.0003662109375000, 0.0010986328125000,
+-0.0003051757812500, 0.0012817382812500, -0.0006408691406250, 0.0010681152343750,
+-0.0006408691406250, 0.0010986328125000, -0.0006408691406250, 0.0011596679687500,
+-0.0009155273437500, 0.0011291503906250, -0.0008850097656250, 0.0011901855468750,
+-0.0008544921875000, 0.0011596679687500, -0.0008239746093750, 0.0012207031250000,
+-0.0007324218750000, 0.0010375976562500, -0.0007019042968750, 0.0008850097656250,
+-0.0006408691406250, 0.0009460449218750, -0.0007019042968750, 0.0005187988281250,
+-0.0004882812500000, 0.0004882812500000, -0.0004882812500000, 0.0004882812500000,
+-0.0005187988281250, 0.0001525878906250, -0.0002746582031250, 0.0001831054687500,
+-0.0005187988281250},
+{0.0026550292968750, 0.0028381347656250, 0.0025634765625000, 0.0028381347656250,
+0.0027160644531250, 0.0029296875000000, 0.0025329589843750, 0.0028076171875000,
+0.0022888183593750, 0.0026550292968750, 0.0026245117187500, 0.0028076171875000,
+0.0018310546875000, 0.0023803710937500, 0.0019531250000000, 0.0024719238281250,
+0.0019226074218750, 0.0022888183593750, 0.0013732910156250, 0.0019531250000000,
+0.0017089843750000, 0.0022277832031250, 0.0013427734375000, 0.0019226074218750,
+0.0014343261718750, 0.0019226074218750, 0.0013427734375000, 0.0020141601562500,
+0.0012512207031250, 0.0020141601562500, 0.0015869140625000, 0.0019836425781250,
+0.0011291503906250, 0.0021667480468750, 0.0014648437500000, 0.0021972656250000,
+0.0011901855468750, 0.0022277832031250, 0.0007019042968750, 0.0023193359375000,
+0.0012817382812500, 0.0022583007812500, 0.0002441406250000, 0.0022583007812500,
+0.0003967285156250, 0.0022888183593750, 0.0003051757812500, 0.0021667480468750,
+-0.0004882812500000, 0.0019836425781250, -0.0001220703125000, 0.0019836425781250,
+-0.0003662109375000, 0.0018615722656250, -0.0006713867187500, 0.0018005371093750,
+-0.0002441406250000, 0.0017089843750000, -0.0001220703125000, 0.0016174316406250,
+-0.0004577636718750, 0.0015563964843750, 0.0007934570312500, 0.0015563964843750,
+0.0005187988281250, 0.0013732910156250, 0.0007934570312500, 0.0014038085937500,
+0.0017089843750000, 0.0014648437500000, 0.0012207031250000, 0.0012817382812500,
+0.0018005371093750, 0.0015563964843750, 0.0019531250000000, 0.0014343261718750,
+0.0016479492187500, 0.0013732910156250, 0.0016784667968750, 0.0015258789062500,
+0.0016784667968750, 0.0013122558593750, 0.0010070800781250, 0.0014038085937500,
+0.0010986328125000, 0.0014343261718750, 0.0007934570312500, 0.0014038085937500,
+0.0001831054687500, 0.0014038085937500, 0.0004272460937500, 0.0013122558593750,
+-0.0001525878906250, 0.0015563964843750, -0.0002441406250000, 0.0014343261718750,
+-0.0002441406250000, 0.0015869140625000, -0.0005187988281250, 0.0018615722656250,
+-0.0005187988281250, 0.0015869140625000, -0.0002746582031250, 0.0021667480468750,
+-0.0004577636718750, 0.0020751953125000, -0.0003967285156250, 0.0021057128906250,
+-0.0001525878906250, 0.0024414062500000, -0.0003356933593750, 0.0022583007812500,
+-0.0002441406250000, 0.0025024414062500, -0.0003662109375000, 0.0025024414062500,
+-0.0004577636718750, 0.0025329589843750, -0.0005187988281250, 0.0026245117187500,
+-0.0007019042968750, 0.0025024414062500, -0.0006408691406250, 0.0027465820312500,
+-0.0008850097656250, 0.0027770996093750, -0.0009155273437500, 0.0027465820312500,
+-0.0007629394531250, 0.0029602050781250, -0.0010070800781250, 0.0028991699218750,
+-0.0006713867187500, 0.0027770996093750, -0.0007934570312500, 0.0029602050781250,
+-0.0007019042968750, 0.0027160644531250, -0.0004272460937500, 0.0024719238281250,
+-0.0005798339843750, 0.0027160644531250, -0.0002746582031250, 0.0020446777343750,
+-0.0002746582031250, 0.0021362304687500, -0.0002136230468750, 0.0020141601562500,
+0.0000000000000000, 0.0014953613281250, -0.0000610351562500, 0.0017089843750000,
+0.0001831054687500, 0.0013122558593750, 0.0002136230468750, 0.0012512207031250,
+0.0002136230468750, 0.0014038085937500, 0.0003967285156250, 0.0012817382812500,
+0.0003356933593750, 0.0012512207031250, 0.0005187988281250, 0.0014343261718750,
+0.0005187988281250, 0.0014038085937500, 0.0006103515625000, 0.0014953613281250,
+0.0007324218750000, 0.0016174316406250, 0.0007019042968750, 0.0015258789062500,
+0.0007934570312500, 0.0018920898437500, 0.0008850097656250, 0.0018310546875000,
+0.0009155273437500, 0.0020751953125000, 0.0009765625000000, 0.0025024414062500,
+0.0010375976562500, 0.0023193359375000, 0.0010070800781250, 0.0030822753906250,
+0.0010986328125000, 0.0030822753906250, 0.0011596679687500, 0.0032348632812500,
+0.0012512207031250, 0.0038146972656250, 0.0013122558593750, 0.0036010742187500,
+0.0013427734375000, 0.0037536621093750, 0.0013427734375000, 0.0038757324218750,
+0.0014038085937500, 0.0035705566406250, 0.0014343261718750, 0.0034179687500000,
+0.0014953613281250, 0.0035705566406250, 0.0015869140625000, 0.0026855468750000,
+0.0015869140625000, 0.0029296875000000, 0.0015563964843750, 0.0024414062500000,
+0.0015869140625000, 0.0015869140625000, 0.0015563964843750, 0.0020141601562500,
+0.0014953613281250, 0.0008544921875000, 0.0014953613281250, 0.0008239746093750,
+0.0013732910156250, 0.0008544921875000, 0.0012817382812500, 0.0002441406250000,
+0.0013122558593750, 0.0004272460937500, 0.0010375976562500, 0.0007934570312500,
+0.0010070800781250, 0.0004882812500000, 0.0010070800781250, 0.0008544921875000,
+0.0007629394531250, 0.0014648437500000, 0.0007934570312500, 0.0011596679687500,
+0.0007324218750000, 0.0019226074218750, 0.0007324218750000, 0.0019531250000000,
+0.0007324218750000, 0.0019531250000000, 0.0007019042968750, 0.0024108886718750,
+0.0006713867187500, 0.0022277832031250, 0.0005798339843750, 0.0025024414062500,
+0.0006408691406250, 0.0025024414062500, 0.0006103515625000, 0.0024719238281250,
+0.0005187988281250, 0.0025634765625000, 0.0006408691406250, 0.0024719238281250,
+0.0005798339843750, 0.0025939941406250, 0.0005493164062500, 0.0025329589843750,
+0.0006713867187500, 0.0025329589843750, 0.0007629394531250, 0.0026550292968750,
+0.0006713867187500, 0.0026245117187500, 0.0008850097656250, 0.0024719238281250,
+0.0008544921875000, 0.0026855468750000, 0.0008850097656250, 0.0023803710937500,
+0.0010375976562500, 0.0020751953125000, 0.0009155273437500, 0.0024108886718750,
+0.0010681152343750, 0.0016784667968750, 0.0010070800781250, 0.0018310546875000,
+0.0010070800781250, 0.0018005371093750, 0.0011596679687500, 0.0014343261718750,
+0.0010070800781250, 0.0016479492187500, 0.0010986328125000, 0.0016174316406250,
+0.0011291503906250, 0.0016479492187500, 0.0010375976562500, 0.0015869140625000,
+0.0010681152343750, 0.0015869140625000, 0.0010375976562500, 0.0017089843750000,
+0.0011901855468750, 0.0014648437500000, 0.0011291503906250, 0.0014038085937500,
+0.0013122558593750, 0.0013732910156250, 0.0015258789062500, 0.0012207031250000,
+0.0014038085937500, 0.0012512207031250, 0.0017089843750000, 0.0012512207031250,
+0.0017089843750000, 0.0011596679687500, 0.0017395019531250, 0.0012207031250000,
+0.0019226074218750, 0.0012512207031250, 0.0018920898437500, 0.0012817382812500,
+0.0018310546875000, 0.0012817382812500, 0.0019836425781250, 0.0013732910156250,
+0.0017700195312500, 0.0012512207031250, 0.0016174316406250, 0.0012817382812500,
+0.0018615722656250, 0.0013122558593750, 0.0015258789062500, 0.0007934570312500,
+0.0016174316406250, 0.0010375976562500, 0.0016174316406250, 0.0007934570312500,
+0.0014953613281250, 0.0002136230468750, 0.0017089843750000, 0.0006408691406250,
+0.0017089843750000, 0.0001525878906250, 0.0018005371093750, 0.0000000000000000,
+0.0019531250000000, 0.0002441406250000, 0.0019836425781250, 0.0002441406250000,
+0.0021057128906250, 0.0001525878906250, 0.0021057128906250, 0.0007324218750000,
+0.0022888183593750, 0.0007629394531250, 0.0022888183593750, 0.0010070800781250,
+0.0021972656250000, 0.0014038085937500, 0.0024719238281250, 0.0013122558593750,
+0.0021667480468750, 0.0020751953125000, 0.0022583007812500, 0.0019531250000000,
+0.0022888183593750, 0.0022277832031250, 0.0020141601562500, 0.0032653808593750,
+0.0022277832031250, -0.0000305175781250, 0.0021972656250000, 0.0007629394531250,
+0.0021362304687500, 0.0001220703125000, 0.0020446777343750, 0.0009155273437500,
+0.0018005371093750, 0.0019226074218750, 0.0016784667968750, 0.0012817382812500,
+0.0018920898437500, 0.0024719238281250, 0.0015869140625000, 0.0024719238281250,
+0.0017700195312500, 0.0024108886718750, 0.0020446777343750, 0.0031433105468750,
+0.0016174316406250, 0.0028686523437500, 0.0023193359375000, 0.0023803710937500,
+0.0022277832031250, 0.0028686523437500, 0.0021972656250000, 0.0022277832031250,
+0.0025939941406250, 0.0014038085937500, 0.0023803710937500, 0.0020751953125000,
+0.0025329589843750, 0.0006103515625000, 0.0026245117187500, 0.0007934570312500,
+0.0024108886718750, 0.0006408691406250, 0.0023193359375000, -0.0003662109375000,
+0.0025329589843750, 0.0002136230468750, 0.0021972656250000, -0.0000305175781250,
+0.0023193359375000, -0.0002746582031250, 0.0022888183593750, 0.0002136230468750,
+0.0019836425781250, 0.0004577636718750, 0.0022277832031250, 0.0002746582031250,
+0.0019226074218750, 0.0013732910156250, 0.0019531250000000, 0.0013427734375000,
+0.0020446777343750, 0.0014343261718750, 0.0020141601562500, 0.0022277832031250,
+0.0019226074218750, 0.0020751953125000, 0.0020446777343750, 0.0018615722656250,
+0.0020751953125000, 0.0021972656250000, 0.0018615722656250, 0.0019226074218750,
+0.0018310546875000, 0.0013122558593750, 0.0018920898437500, 0.0017700195312500,
+0.0011291503906250, 0.0014343261718750, 0.0012817382812500, 0.0013427734375000,
+0.0008850097656250, 0.0017089843750000, 0.0001831054687500, 0.0018310546875000,
+0.0005187988281250, 0.0016479492187500, -0.0002441406250000, 0.0023193359375000,
+-0.0003662109375000, 0.0022888183593750, -0.0002746582031250, 0.0023193359375000,
+-0.0006103515625000, 0.0027770996093750, -0.0006713867187500, 0.0025939941406250,
+-0.0001525878906250, 0.0026245117187500, -0.0005187988281250, 0.0028686523437500,
+-0.0001831054687500, 0.0025939941406250, 0.0005187988281250, 0.0022583007812500,
+0.0000305175781250, 0.0025634765625000, 0.0009765625000000, 0.0017395019531250,
+0.0009460449218750, 0.0018005371093750, 0.0008544921875000, 0.0016784667968750,
+0.0012512207031250, 0.0010681152343750, 0.0012207031250000, 0.0012207031250000,
+0.0010070800781250, 0.0012207031250000, 0.0010986328125000, 0.0009155273437500,
+0.0010681152343750, 0.0012817382812500, 0.0008544921875000, 0.0015869140625000,
+0.0009765625000000, 0.0011901855468750, 0.0008239746093750, 0.0021057128906250,
+0.0008239746093750, 0.0019226074218750, 0.0008544921875000, 0.0020751953125000,
+0.0007324218750000, 0.0026855468750000, 0.0007324218750000, 0.0023193359375000,
+0.0009155273437500, 0.0025939941406250, 0.0007934570312500, 0.0026855468750000,
+0.0008850097656250, 0.0025634765625000, 0.0010681152343750, 0.0025634765625000,
+0.0009765625000000, 0.0025939941406250, 0.0009765625000000, 0.0023803710937500,
+0.0009765625000000, 0.0022583007812500, 0.0008850097656250, 0.0023498535156250,
+0.0008544921875000, 0.0023193359375000, 0.0008850097656250, 0.0022888183593750,
+0.0007629394531250, 0.0025939941406250, 0.0007629394531250, 0.0025024414062500,
+0.0008850097656250, 0.0026245117187500, 0.0007934570312500, 0.0030212402343750,
+0.0007629394531250, 0.0027770996093750, 0.0009765625000000, 0.0031738281250000,
+0.0009155273437500, 0.0030822753906250, 0.0010986328125000, 0.0029296875000000,
+0.0013122558593750, 0.0029602050781250, 0.0011596679687500, 0.0028686523437500,
+0.0015563964843750, 0.0027160644531250, 0.0015563964843750, 0.0027465820312500,
+0.0015563964843750, 0.0026245117187500, 0.0017700195312500, 0.0025024414062500,
+0.0017395019531250, 0.0026245117187500, 0.0015258789062500, 0.0023803710937500,
+0.0016479492187500, 0.0024108886718750, 0.0014648437500000, 0.0024108886718750,
+0.0010986328125000, 0.0022888183593750, 0.0012207031250000, 0.0024108886718750,
+0.0010070800781250, 0.0022583007812500, 0.0008850097656250, 0.0023498535156250,
+0.0010375976562500}};
+float hrirs_right[HRIRS_IMPULSE_SIZE][HRIRS_IMPULSE_LEN] = {
+{-0.0016174316406250, -0.0000305175781250, -0.0017700195312500, 0.0000000000000000,
+-0.0016784667968750, 0.0000000000000000, -0.0018310546875000, 0.0000000000000000,
+-0.0018310546875000, 0.0000000000000000, -0.0017700195312500, 0.0000000000000000,
+-0.0018310546875000, 0.0000000000000000, -0.0018615722656250, 0.0000000000000000,
+-0.0015869140625000, 0.0000000000000000, -0.0017700195312500, 0.0000000000000000,
+-0.0014343261718750, 0.0000000000000000, -0.0010986328125000, 0.0000000000000000,
+-0.0014038085937500, 0.0000000000000000, -0.0003051757812500, 0.0000000000000000,
+-0.0006103515625000, 0.0000000000000000, -0.0002441406250000, 0.0000000000000000,
+0.0006713867187500, 0.0000305175781250, 0.0000915527343750, 0.0000000000000000,
+0.0011291503906250, 0.0000000000000000, 0.0012512207031250, 0.0000000000000000,
+0.0010681152343750, 0.0000000000000000, 0.0015258789062500, 0.0000000000000000,
+0.0014648437500000, 0.0000000000000000, 0.0009155273437500, 0.0000305175781250,
+0.0012207031250000, 0.0000000000000000, 0.0007019042968750, -0.0000305175781250,
+-0.0000610351562500, 0.0000000000000000, 0.0004272460937500, 0.0000000000000000,
+-0.0007019042968750, 0.0000000000000000, -0.0007324218750000, 0.0000000000000000,
+-0.0006713867187500, 0.0000305175781250, -0.0012817382812500, 0.0000000000000000,
+-0.0011291503906250, 0.0000305175781250, -0.0008239746093750, 0.0000305175781250,
+-0.0011291503906250, -0.0000305175781250, -0.0006713867187500, 0.0000000000000000,
+-0.0000610351562500, 0.0000000000000000, -0.0004272460937500, -0.0000305175781250,
+0.0005187988281250, -0.0000305175781250, 0.0006103515625000, 0.0000000000000000,
+0.0005187988281250, 0.0000000000000000, 0.0009765625000000, 0.0000000000000000,
+0.0008850097656250, 0.0000000000000000, 0.0003967285156250, 0.0000305175781250,
+0.0006103515625000, 0.0000000000000000, 0.0002746582031250, 0.0000000000000000,
+-0.0003967285156250, 0.0000000000000000, -0.0000915527343750, -0.0000305175781250,
+-0.0005493164062500, -0.0000305175781250, -0.0007324218750000, 0.0000000000000000,
+-0.0004577636718750, 0.0000305175781250, -0.0003967285156250, 0.0000000000000000,
+-0.0005493164062500, 0.0000305175781250, 0.0000915527343750, 0.0000610351562500,
+0.0000610351562500, 0.0000305175781250, 0.0000915527343750, 0.0000000000000000,
+0.0005798339843750, 0.0000305175781250, 0.0003967285156250, -0.0000610351562500,
+0.0002746582031250, -0.0000915527343750, 0.0005187988281250, -0.0000305175781250,
+0.0002136230468750, -0.0000305175781250, -0.0002441406250000, -0.0000610351562500,
+0.0000610351562500, 0.0000610351562500, -0.0003967285156250, 0.0000610351562500,
+-0.0006103515625000, 0.0000305175781250, -0.0002746582031250, 0.0000610351562500,
+-0.0002746582031250, 0.0000915527343750, -0.0004272460937500, -0.0000610351562500,
+0.0002746582031250, -0.0000610351562500, 0.0003051757812500, -0.0000305175781250,
+0.0001831054687500, -0.0001220703125000, 0.0005187988281250, -0.0001220703125000,
+0.0005187988281250, 0.0000000000000000, -0.0001525878906250, -0.0000610351562500,
+0.0000000000000000, 0.0000000000000000, -0.0000610351562500, 0.0002136230468750,
+-0.0005798339843750, 0.0001525878906250, -0.0004577636718750, 0.0000305175781250,
+-0.0000915527343750, 0.0001831054687500, -0.0002746582031250, 0.0000610351562500,
+-0.0001525878906250, -0.0002746582031250, 0.0005493164062500, -0.0000305175781250,
+-0.0004577636718750, 0.0002441406250000, 0.0000610351562500, 0.0003662109375000,
+-0.0003051757812500, 0.0000915527343750, -0.0000610351562500, -0.0000915527343750,
+0.0004577636718750, 0.0001525878906250, 0.0003051757812500, -0.0003356933593750,
+0.0000610351562500, -0.0004577636718750, 0.0002746582031250, -0.0001831054687500,
+0.0000610351562500, -0.0003051757812500, -0.0003967285156250, -0.0004577636718750,
+-0.0002136230468750, 0.0003356933593750, -0.0000915527343750, 0.0002441406250000,
+-0.0002746582031250, 0.0002136230468750, -0.0000915527343750, 0.0006713867187500,
+0.0002441406250000, 0.0006103515625000, 0.0001220703125000, -0.0000305175781250,
+0.0000915527343750, 0.0002441406250000, 0.0002746582031250, 0.0000000000000000,
+0.0000915527343750, -0.0008239746093750, -0.0001831054687500, -0.0004882812500000,
+0.0000000000000000, -0.0003662109375000, -0.0000610351562500, -0.0007934570312500,
+-0.0002136230468750, -0.0002136230468750, -0.0000610351562500, 0.0005493164062500,
+0.0001220703125000, 0.0000000000000000, 0.0000000000000000, 0.0006408691406250,
+0.0000915527343750, 0.0009765625000000, 0.0001831054687500, 0.0004272460937500,
+0.0000610351562500, 0.0002746582031250, -0.0000915527343750, 0.0006103515625000,
+0.0000000000000000, -0.0005798339843750, -0.0000305175781250, -0.0005798339843750,
+-0.0001220703125000, -0.0003662109375000, -0.0000305175781250, -0.0009460449218750,
+0.0000610351562500, -0.0009155273437500, 0.0000000000000000, -0.0000305175781250,
+0.0000305175781250, -0.0004272460937500, 0.0001220703125000, -0.0000305175781250,
+0.0000305175781250, 0.0008544921875000, -0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, 0.0007019042968750, -0.0000305175781250, 0.0010986328125000,
+-0.0000915527343750, 0.0005493164062500, -0.0000305175781250, 0.0002746582031250,
+0.0000305175781250, 0.0006713867187500, 0.0000000000000000, -0.0005187988281250,
+0.0000305175781250, -0.0004272460937500, 0.0000610351562500, -0.0004577636718750,
+0.0000000000000000, -0.0011901855468750, -0.0000305175781250, -0.0009765625000000,
+0.0000000000000000, -0.0007019042968750, 0.0000000000000000, -0.0010375976562500,
+-0.0000305175781250, -0.0006103515625000, -0.0000305175781250, 0.0000000000000000,
+0.0000000000000000, -0.0004272460937500, 0.0000000000000000, 0.0006408691406250,
+0.0000305175781250, 0.0005798339843750, 0.0000000000000000, 0.0006713867187500,
+0.0000000000000000, 0.0013732910156250, -0.0000305175781250, 0.0010681152343750,
+-0.0000305175781250, 0.0011596679687500, 0.0000000000000000, 0.0014343261718750,
+0.0000000000000000, 0.0010070800781250, 0.0000305175781250, 0.0007019042968750,
+0.0000305175781250, 0.0010681152343750, 0.0000305175781250, -0.0001525878906250,
+0.0000000000000000, 0.0001831054687500, 0.0000000000000000, -0.0002136230468750,
+0.0000000000000000, -0.0011901855468750, 0.0000000000000000, -0.0005798339843750,
+0.0000000000000000, -0.0015563964843750, 0.0000000000000000, -0.0016174316406250,
+0.0000305175781250, -0.0015563964843750, 0.0000000000000000, -0.0020751953125000,
+0.0000305175781250, -0.0019226074218750, 0.0000000000000000, -0.0017089843750000,
+0.0000000000000000, -0.0019836425781250, 0.0000000000000000, -0.0016479492187500,
+0.0000305175781250, -0.0013122558593750, 0.0000000000000000, -0.0016784667968750,
+0.0000000000000000, -0.0006713867187500, -0.0000305175781250, -0.0010070800781250,
+0.0000000000000000, -0.0007019042968750, 0.0000000000000000, 0.0001220703125000,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, 0.0005187988281250,
+0.0000000000000000, 0.0003051757812500, 0.0000000000000000, 0.0004577636718750,
+0.0000000000000000, 0.0010070800781250, 0.0000000000000000, 0.0005187988281250,
+0.0000000000000000, 0.0010986328125000, 0.0000000000000000, 0.0009765625000000,
+-0.0000305175781250, 0.0009765625000000, 0.0000000000000000, 0.0012512207031250,
+0.0000000000000000, 0.0009460449218750, 0.0000305175781250, 0.0012207031250000,
+0.0000000000000000, 0.0011291503906250, -0.0000305175781250, 0.0010375976562500,
+-0.0000305175781250, 0.0011596679687500, 0.0000000000000000, 0.0009765625000000,
+0.0000000000000000, 0.0008850097656250, 0.0000000000000000, 0.0008239746093750,
+0.0000000000000000, 0.0006408691406250, -0.0000305175781250, 0.0004882812500000,
+0.0000305175781250, 0.0003662109375000, -0.0000305175781250, 0.0000305175781250,
+-0.0000305175781250, 0.0000305175781250, 0.0000000000000000, -0.0002136230468750,
+0.0000000000000000, -0.0005493164062500, -0.0000305175781250, -0.0004577636718750,
+0.0000000000000000, -0.0010070800781250, 0.0000000000000000, -0.0009765625000000,
+0.0000000000000000, -0.0010681152343750, 0.0000000000000000, -0.0014343261718750,
+0.0000000000000000, -0.0013732910156250, -0.0000305175781250, -0.0014343261718750,
+0.0000305175781250, -0.0016174316406250, 0.0000000000000000, -0.0014038085937500,
+0.0000000000000000, -0.0012817382812500, -0.0000305175781250, -0.0014038085937500,
+-0.0000305175781250, -0.0007324218750000, 0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0005187988281250, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, -0.0000915527343750, 0.0000000000000000, 0.0006713867187500,
+0.0000000000000000, 0.0007324218750000, 0.0000000000000000, 0.0007324218750000,
+-0.0000305175781250, 0.0010986328125000, -0.0000305175781250, 0.0010375976562500,
+0.0000000000000000, 0.0007629394531250, 0.0000000000000000, 0.0010070800781250,
+0.0000000000000000, 0.0006408691406250, 0.0000000000000000, 0.0001831054687500,
+0.0000000000000000, 0.0004272460937500, 0.0000000000000000, -0.0003967285156250,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, -0.0004272460937500,
+0.0000000000000000, -0.0008239746093750, -0.0000305175781250, -0.0007629394531250,
+0.0000000000000000, -0.0004272460937500, 0.0000000000000000, -0.0006103515625000,
+-0.0000305175781250, -0.0003051757812500, 0.0000305175781250, 0.0002136230468750,
+0.0000305175781250, -0.0000305175781250, 0.0000000000000000, 0.0004272460937500,
+0.0000000000000000, 0.0005798339843750, 0.0000000000000000, 0.0003662109375000,
+0.0000000000000000, 0.0003662109375000, -0.0000305175781250, 0.0004577636718750,
+0.0000000000000000, -0.0000915527343750, 0.0000000000000000, -0.0000610351562500,
+-0.0000305175781250, -0.0000915527343750, -0.0000305175781250, -0.0004577636718750,
+0.0000305175781250, -0.0003662109375000, 0.0000000000000000, -0.0001220703125000,
+0.0000000000000000, -0.0002746582031250, 0.0000305175781250, -0.0000915527343750,
+0.0000000000000000, 0.0003051757812500, -0.0000305175781250, 0.0001220703125000,
+0.0000000000000000, 0.0001525878906250, 0.0000000000000000, 0.0003051757812500,
+-0.0000305175781250, 0.0001220703125000, -0.0000305175781250, -0.0001525878906250,
+0.0000305175781250, 0.0000000000000000, 0.0000305175781250, -0.0001831054687500,
+0.0000305175781250, -0.0003051757812500, 0.0000610351562500, -0.0001220703125000,
+0.0000305175781250, 0.0000000000000000, -0.0000610351562500, -0.0001220703125000,
+0.0000000000000000, 0.0001525878906250, -0.0000305175781250, 0.0002441406250000,
+-0.0000610351562500, 0.0000915527343750, -0.0000305175781250, 0.0000610351562500,
+0.0000305175781250, 0.0001525878906250, 0.0000000000000000, -0.0000915527343750,
+0.0000000000000000, -0.0001220703125000, 0.0000915527343750, -0.0000610351562500,
+0.0000000000000000, -0.0000610351562500, -0.0000610351562500, -0.0000305175781250,
+-0.0000305175781250, 0.0000610351562500, -0.0000305175781250, 0.0001220703125000,
+-0.0000610351562500, 0.0000305175781250, -0.0000305175781250, 0.0000305175781250,
+0.0000610351562500, 0.0000915527343750, 0.0000610351562500, -0.0000610351562500,
+0.0000305175781250, -0.0000915527343750, 0.0000915527343750, -0.0000305175781250,
+0.0000305175781250, -0.0000305175781250, -0.0000610351562500, -0.0000915527343750,
+-0.0000305175781250, 0.0000610351562500, 0.0000000000000000, 0.0001220703125000,
+-0.0000610351562500, 0.0000305175781250, -0.0000305175781250, 0.0000305175781250,
+0.0001220703125000, 0.0000915527343750, 0.0000305175781250, -0.0000610351562500,
+0.0000305175781250},
+{0.0000000000000000, 0.0014648437500000, 0.0000000000000000, 0.0015869140625000,
+-0.0000305175781250, 0.0012817382812500, 0.0000000000000000, 0.0009460449218750,
+0.0000000000000000, 0.0011596679687500, 0.0000000000000000, 0.0003356933593750,
+0.0000000000000000, 0.0003967285156250, 0.0000000000000000, 0.0000610351562500,
+0.0000305175781250, -0.0006103515625000, 0.0000000000000000, -0.0003356933593750,
+-0.0000305175781250, -0.0009460449218750, 0.0000000000000000, -0.0010986328125000,
+-0.0000305175781250, -0.0009460449218750, -0.0000305175781250, -0.0011291503906250,
+0.0000000000000000, -0.0011901855468750, 0.0000305175781250, -0.0006408691406250,
+0.0000000000000000, -0.0007934570312500, 0.0000000000000000, -0.0003967285156250,
+0.0000000000000000, 0.0001525878906250, -0.0000305175781250, -0.0001220703125000,
+0.0000610351562500, 0.0006103515625000, 0.0000000000000000, 0.0007324218750000,
+-0.0000305175781250, 0.0005798339843750, 0.0000000000000000, 0.0007934570312500,
+0.0000305175781250, 0.0008239746093750, 0.0000000000000000, 0.0002746582031250,
+0.0000000000000000, 0.0003967285156250, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, -0.0004272460937500, -0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, -0.0006713867187500,
+-0.0000305175781250, -0.0003662109375000, 0.0000000000000000, -0.0001831054687500,
+0.0000000000000000, -0.0003967285156250, 0.0000305175781250, 0.0002136230468750,
+0.0000000000000000, 0.0002441406250000, 0.0000305175781250, 0.0001831054687500,
+0.0000000000000000, 0.0004577636718750, -0.0000305175781250, 0.0004272460937500,
+-0.0000305175781250, 0.0000000000000000, -0.0000305175781250, 0.0000915527343750,
+-0.0000610351562500, 0.0000000000000000, -0.0000305175781250, -0.0003662109375000,
+0.0000305175781250, -0.0002441406250000, 0.0000000000000000, -0.0000915527343750,
+0.0000305175781250, -0.0002441406250000, 0.0000610351562500, -0.0000610351562500,
+0.0000305175781250, 0.0002746582031250, -0.0000305175781250, 0.0000915527343750,
+0.0000000000000000, 0.0001831054687500, -0.0000610351562500, 0.0003662109375000,
+-0.0000915527343750, 0.0000915527343750, -0.0000305175781250, -0.0000915527343750,
+0.0000610351562500, 0.0000915527343750, -0.0000305175781250, -0.0001220703125000,
+0.0000305175781250, -0.0002746582031250, 0.0000915527343750, -0.0001220703125000,
+0.0000305175781250, 0.0000000000000000, 0.0000000000000000, -0.0001525878906250,
+0.0000305175781250, 0.0001220703125000, -0.0000305175781250, 0.0001831054687500,
+-0.0000915527343750, 0.0000610351562500, -0.0000305175781250, 0.0000000000000000,
+0.0000915527343750, 0.0000000000000000, 0.0000305175781250, -0.0000610351562500,
+0.0000305175781250, -0.0001220703125000, 0.0000610351562500, -0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, -0.0000305175781250, -0.0000610351562500,
+-0.0000305175781250, 0.0000610351562500, 0.0000000000000000, 0.0001220703125000,
+-0.0000915527343750, 0.0000305175781250, 0.0000000000000000, -0.0000305175781250,
+0.0001220703125000, 0.0000610351562500, 0.0000000000000000, -0.0000610351562500,
+0.0000610351562500, -0.0001220703125000, 0.0001525878906250, -0.0000610351562500,
+0.0000000000000000, 0.0000305175781250, -0.0000915527343750, -0.0000305175781250,
+-0.0000305175781250, 0.0000915527343750, -0.0000915527343750, 0.0001525878906250,
+-0.0001831054687500, 0.0000305175781250, -0.0000305175781250, 0.0000000000000000,
+0.0001525878906250, 0.0000305175781250, 0.0000000000000000, -0.0000610351562500,
+0.0001220703125000, -0.0000915527343750, 0.0002746582031250, -0.0000305175781250,
+0.0000305175781250, 0.0000000000000000, -0.0000915527343750, -0.0000610351562500,
+0.0000915527343750, 0.0000305175781250, -0.0002441406250000, 0.0000610351562500,
+-0.0003356933593750, 0.0000305175781250, -0.0001220703125000, 0.0000305175781250,
+-0.0001220703125000, 0.0000305175781250, -0.0002441406250000, -0.0000610351562500,
+0.0002441406250000, -0.0000915527343750, 0.0002746582031250, -0.0000305175781250,
+0.0001525878906250, -0.0000305175781250, 0.0003662109375000, -0.0000305175781250,
+0.0003967285156250, 0.0000305175781250, -0.0000610351562500, 0.0000305175781250,
+0.0000610351562500, 0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0003967285156250, -0.0000305175781250,
+-0.0002136230468750, -0.0000305175781250, -0.0004272460937500, 0.0000000000000000,
+-0.0000915527343750, -0.0000305175781250, 0.0003051757812500, -0.0000305175781250,
+0.0000305175781250, 0.0000305175781250, 0.0004882812500000, 0.0000305175781250,
+0.0006408691406250, 0.0000305175781250, 0.0003662109375000, 0.0000305175781250,
+0.0003967285156250, 0.0000305175781250, 0.0005187988281250, -0.0000305175781250,
+-0.0001831054687500, 0.0000000000000000, -0.0001525878906250, 0.0000000000000000,
+-0.0003051757812500, 0.0000000000000000, -0.0008544921875000, -0.0000305175781250,
+-0.0006408691406250, 0.0000305175781250, -0.0006408691406250, 0.0000000000000000,
+-0.0008544921875000, 0.0000000000000000, -0.0005187988281250, 0.0000305175781250,
+-0.0002441406250000, 0.0000305175781250, -0.0004272460937500, 0.0000305175781250,
+0.0003662109375000, 0.0000000000000000, 0.0003356933593750, 0.0000000000000000,
+0.0004882812500000, 0.0000000000000000, 0.0010986328125000, 0.0000000000000000,
+0.0008544921875000, 0.0000000000000000, 0.0010681152343750, 0.0000000000000000,
+0.0012512207031250, 0.0000000000000000, 0.0010375976562500, 0.0000305175781250,
+0.0009460449218750, 0.0000000000000000, 0.0010375976562500, 0.0000000000000000,
+0.0005493164062500, -0.0000305175781250, 0.0005798339843750, 0.0000000000000000,
+0.0002746582031250, 0.0000000000000000, -0.0001831054687500, 0.0000000000000000,
+-0.0000305175781250, 0.0000000000000000, -0.0006713867187500, 0.0000000000000000,
+-0.0007324218750000, 0.0000000000000000, -0.0008850097656250, 0.0000000000000000,
+-0.0012512207031250, 0.0000000000000000, -0.0011596679687500, 0.0000000000000000,
+-0.0014953613281250, 0.0000000000000000, -0.0015258789062500, 0.0000305175781250,
+-0.0015563964843750, 0.0000000000000000, -0.0017089843750000, 0.0000000000000000,
+-0.0016784667968750, -0.0000305175781250, -0.0017700195312500, 0.0000000000000000,
+-0.0018310546875000, 0.0000000000000000, -0.0018615722656250, 0.0000000000000000,
+-0.0018920898437500, 0.0000305175781250, -0.0018920898437500, 0.0000305175781250,
+-0.0019226074218750, 0.0000305175781250, -0.0019226074218750, 0.0000000000000000,
+-0.0018920898437500, 0.0000305175781250, -0.0019226074218750, 0.0000000000000000,
+-0.0019531250000000, -0.0000305175781250, -0.0018920898437500, 0.0000000000000000,
+-0.0019531250000000, 0.0000000000000000, -0.0019226074218750, 0.0000000000000000,
+-0.0018920898437500, 0.0000000000000000, -0.0019531250000000, 0.0000000000000000,
+-0.0016479492187500, -0.0000305175781250, -0.0018005371093750, -0.0000305175781250,
+-0.0015258789062500, -0.0000305175781250, -0.0011901855468750, -0.0000305175781250,
+-0.0014953613281250, 0.0000000000000000, -0.0006408691406250, 0.0000000000000000,
+-0.0009460449218750, -0.0000305175781250, -0.0004577636718750, -0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, -0.0002746582031250, -0.0000305175781250,
+0.0010070800781250, 0.0000000000000000, 0.0008544921875000, 0.0000000000000000,
+0.0009460449218750, 0.0000305175781250, 0.0018310546875000, 0.0000305175781250,
+0.0014038085937500, 0.0000000000000000, 0.0015258789062500, 0.0000000000000000,
+0.0019226074218750, 0.0000000000000000, 0.0013427734375000, 0.0000000000000000,
+0.0009460449218750, 0.0000000000000000, 0.0013732910156250, 0.0000000000000000,
+-0.0000305175781250, 0.0000000000000000, 0.0002136230468750, 0.0000000000000000,
+-0.0001220703125000, 0.0000000000000000, -0.0012817382812500, 0.0000000000000000,
+-0.0007324218750000, 0.0000000000000000, -0.0013427734375000, 0.0000000000000000,
+-0.0016174316406250, -0.0000305175781250, -0.0011596679687500, 0.0000000000000000,
+-0.0010986328125000, 0.0000000000000000, -0.0013427734375000, 0.0000000000000000,
+-0.0001525878906250, 0.0000305175781250, -0.0003662109375000, 0.0000000000000000,
+-0.0000915527343750, 0.0000000000000000, 0.0008850097656250, 0.0000000000000000,
+0.0005187988281250, -0.0000305175781250, 0.0009155273437500, -0.0000305175781250,
+0.0012512207031250, 0.0000000000000000, 0.0007934570312500, -0.0000305175781250,
+0.0006103515625000, 0.0000000000000000, 0.0008544921875000, 0.0000610351562500,
+-0.0001831054687500, 0.0000610351562500, -0.0001220703125000, 0.0000000000000000,
+-0.0002136230468750, 0.0000000000000000, -0.0009155273437500, 0.0000305175781250,
+-0.0007019042968750, -0.0000305175781250, -0.0004882812500000, -0.0000610351562500,
+-0.0007629394531250, 0.0000000000000000, -0.0003967285156250, -0.0000305175781250,
+0.0001220703125000, -0.0000305175781250, -0.0001525878906250, 0.0000610351562500,
+0.0004577636718750, 0.0000610351562500, 0.0005493164062500, 0.0000000000000000,
+0.0003662109375000, 0.0000610351562500, 0.0006103515625000, 0.0000915527343750,
+0.0005798339843750, -0.0000305175781250, 0.0000000000000000, -0.0000305175781250,
+0.0001525878906250, -0.0000305175781250, -0.0000305175781250, -0.0001220703125000,
+-0.0006713867187500, -0.0000915527343750, -0.0003967285156250, 0.0000305175781250,
+-0.0003356933593750, 0.0000000000000000, -0.0006103515625000, 0.0000305175781250,
+-0.0002136230468750, 0.0001220703125000, 0.0002746582031250, 0.0000915527343750,
+-0.0000610351562500, 0.0000610351562500, 0.0004577636718750, 0.0000915527343750,
+0.0007629394531250, -0.0000305175781250, 0.0002746582031250, -0.0001831054687500,
+0.0001220703125000, -0.0000305175781250, 0.0004272460937500, -0.0000915527343750,
+-0.0004272460937500, -0.0001831054687500, -0.0005187988281250, -0.0000610351562500,
+-0.0002136230468750, 0.0001525878906250, -0.0003967285156250, 0.0000305175781250,
+-0.0005798339843750, 0.0001525878906250, 0.0002441406250000, 0.0002746582031250,
+0.0002441406250000, 0.0001220703125000, 0.0000915527343750, -0.0001220703125000,
+0.0006103515625000, -0.0002136230468750, -0.0005798339843750, 0.0003356933593750,
+0.0003662109375000, 0.0002441406250000, 0.0001220703125000, 0.0001220703125000,
+0.0000610351562500, 0.0002746582031250, 0.0004272460937500, 0.0003662109375000,
+0.0004882812500000, -0.0002441406250000, -0.0001525878906250, -0.0001831054687500,
+-0.0000915527343750, -0.0001831054687500, -0.0000610351562500, -0.0006408691406250,
+-0.0004272460937500, -0.0005187988281250, -0.0003967285156250, 0.0000000000000000,
+0.0001220703125000, -0.0003051757812500, 0.0000000000000000, 0.0000000000000000,
+-0.0000305175781250, 0.0007629394531250, 0.0003356933593750, 0.0004272460937500,
+0.0003051757812500, 0.0003967285156250, 0.0000000000000000, 0.0007934570312500,
+0.0001220703125000, 0.0002441406250000, 0.0000610351562500, -0.0004882812500000,
+-0.0002746582031250, 0.0000610351562500, -0.0001525878906250, -0.0006408691406250,
+-0.0000305175781250, -0.0010375976562500, -0.0000915527343750, -0.0003967285156250,
+-0.0000610351562500, -0.0001525878906250, 0.0002136230468750, -0.0006103515625000,
+0.0001220703125000, 0.0006408691406250, 0.0000305175781250, 0.0006713867187500,
+0.0000915527343750, 0.0003967285156250, 0.0000000000000000, 0.0009460449218750,
+-0.0001831054687500, 0.0009460449218750, -0.0000915527343750, -0.0001220703125000,
+-0.0000305175781250, 0.0002136230468750, -0.0000915527343750, -0.0000305175781250,
+-0.0000305175781250},
+{-0.0002136230468750, 0.0000000000000000, -0.0003356933593750, 0.0000000000000000,
+-0.0012817382812500, -0.0000305175781250, -0.0009765625000000, 0.0000000000000000,
+-0.0009765625000000, 0.0000000000000000, -0.0013122558593750, 0.0000000000000000,
+-0.0007934570312500, 0.0000305175781250, -0.0003051757812500, 0.0000305175781250,
+-0.0006713867187500, -0.0000305175781250, 0.0004577636718750, -0.0000305175781250,
+0.0005187988281250, -0.0000305175781250, 0.0004272460937500, -0.0000610351562500,
+0.0009765625000000, -0.0000610351562500, 0.0009155273437500, 0.0000305175781250,
+0.0003356933593750, 0.0000305175781250, 0.0005798339843750, -0.0000305175781250,
+0.0002441406250000, 0.0000610351562500, -0.0004577636718750, 0.0000305175781250,
+-0.0000610351562500, -0.0000305175781250, -0.0005798339843750, -0.0000305175781250,
+-0.0007629394531250, -0.0000305175781250, -0.0004577636718750, -0.0000610351562500,
+-0.0004272460937500, -0.0000915527343750, -0.0006103515625000, 0.0000305175781250,
+0.0002441406250000, -0.0000305175781250, 0.0001525878906250, 0.0000305175781250,
+0.0001525878906250, 0.0001220703125000, 0.0007019042968750, 0.0000610351562500,
+0.0005493164062500, 0.0000305175781250, 0.0002136230468750, 0.0000610351562500,
+0.0004882812500000, 0.0000000000000000, 0.0001220703125000, -0.0000915527343750,
+-0.0005493164062500, -0.0000305175781250, -0.0001831054687500, -0.0000610351562500,
+-0.0004577636718750, -0.0001525878906250, -0.0007934570312500, 0.0000000000000000,
+-0.0002746582031250, 0.0001831054687500, 0.0001525878906250, 0.0000305175781250,
+-0.0002746582031250, 0.0001220703125000, 0.0004577636718750, 0.0002136230468750,
+0.0007934570312500, 0.0000610351562500, 0.0002746582031250, -0.0000305175781250,
+0.0001831054687500, 0.0000610351562500, 0.0004882812500000, -0.0001831054687500,
+-0.0003662109375000, -0.0003051757812500, -0.0004577636718750, -0.0001220703125000,
+-0.0002136230468750, -0.0000610351562500, -0.0004272460937500, 0.0002746582031250,
+0.0005493164062500, -0.0002746582031250, -0.0004272460937500, -0.0000610351562500,
+-0.0003662109375000, -0.0000915527343750, -0.0001220703125000, -0.0004577636718750,
+-0.0002746582031250, -0.0003967285156250, -0.0004577636718750, 0.0001220703125000,
+0.0002441406250000, -0.0000305175781250, 0.0003051757812500, 0.0001220703125000,
+0.0000915527343750, 0.0006713867187500, 0.0003662109375000, 0.0004882812500000,
+0.0003967285156250, 0.0001831054687500, -0.0001525878906250, 0.0005798339843750,
+-0.0001525878906250, 0.0001220703125000, -0.0000305175781250, -0.0006103515625000,
+-0.0003051757812500, -0.0002136230468750, -0.0003356933593750, -0.0005493164062500,
+0.0000915527343750, -0.0009460449218750, 0.0000610351562500, -0.0003051757812500,
+0.0000305175781250, 0.0001220703125000, 0.0002441406250000, -0.0003967285156250,
+0.0002746582031250, 0.0007019042968750, -0.0000305175781250, 0.0009460449218750,
+0.0000305175781250, 0.0003967285156250, 0.0000000000000000, 0.0005187988281250,
+-0.0002136230468750, 0.0008544921875000, -0.0001525878906250, -0.0004272460937500,
+0.0000305175781250, -0.0003051757812500, -0.0000305175781250, -0.0003051757812500,
+0.0000000000000000, -0.0011596679687500, 0.0001831054687500, -0.0009765625000000,
+0.0001525878906250, -0.0002136230468750, -0.0000305175781250, -0.0006713867187500,
+0.0000610351562500, -0.0001525878906250, 0.0000000000000000, 0.0008850097656250,
+-0.0001220703125000, 0.0003051757812500, -0.0000915527343750, 0.0007629394531250,
+0.0000305175781250, 0.0011596679687500, -0.0000305175781250, 0.0005493164062500,
+0.0000305175781250, 0.0001525878906250, 0.0001525878906250, 0.0006713867187500,
+0.0000610351562500, -0.0006713867187500, 0.0000000000000000, -0.0006408691406250,
+0.0000305175781250, -0.0005187988281250, 0.0000305175781250, -0.0012512207031250,
+-0.0001220703125000, -0.0010986328125000, -0.0000610351562500, -0.0004577636718750,
+0.0000000000000000, -0.0009155273437500, -0.0000305175781250, -0.0003356933593750,
+0.0000000000000000, 0.0005798339843750, 0.0000305175781250, -0.0000610351562500,
+0.0000610351562500, 0.0010070800781250, -0.0000305175781250, 0.0011901855468750,
+0.0000000000000000, 0.0009155273437500, 0.0000305175781250, 0.0012512207031250,
+0.0000000000000000, 0.0013122558593750, -0.0000610351562500, 0.0004882812500000,
+0.0000000000000000, 0.0008239746093750, 0.0000000000000000, 0.0003967285156250,
+0.0000000000000000, -0.0004577636718750, 0.0000000000000000, 0.0000610351562500,
+0.0000000000000000, -0.0011901855468750, -0.0000305175781250, -0.0011596679687500,
+0.0000000000000000, -0.0011291503906250, -0.0000305175781250, -0.0018005371093750,
+0.0000000000000000, -0.0015869140625000, 0.0000000000000000, -0.0013732910156250,
+0.0000000000000000, -0.0017700195312500, 0.0000305175781250, -0.0012512207031250,
+0.0000000000000000, -0.0007629394531250, 0.0000305175781250, -0.0012817382812500,
+0.0000000000000000, 0.0001525878906250, 0.0000000000000000, -0.0002136230468750,
+0.0000000000000000, 0.0002746582031250, 0.0000305175781250, 0.0013732910156250,
+0.0000000000000000, 0.0006408691406250, -0.0000305175781250, 0.0019531250000000,
+0.0000000000000000, 0.0018615722656250, 0.0000000000000000, 0.0018920898437500,
+0.0000305175781250, 0.0026245117187500, 0.0000000000000000, 0.0023193359375000,
+0.0000000000000000, 0.0025024414062500, 0.0000000000000000, 0.0027160644531250,
+0.0000000000000000, 0.0024719238281250, 0.0000000000000000, 0.0024108886718750,
+0.0000000000000000, 0.0025939941406250, -0.0000305175781250, 0.0021057128906250,
+0.0000305175781250, 0.0022888183593750, 0.0000305175781250, 0.0021362304687500,
+0.0000305175781250, 0.0017700195312500, 0.0000000000000000, 0.0021362304687500,
+0.0000000000000000, 0.0016174316406250, 0.0000000000000000, 0.0018005371093750,
+-0.0000305175781250, 0.0017395019531250, 0.0000000000000000, 0.0014038085937500,
+0.0000305175781250, 0.0017395019531250, 0.0000000000000000, 0.0013732910156250,
+0.0000305175781250, 0.0014343261718750, 0.0000305175781250, 0.0015258789062500,
+-0.0000305175781250, 0.0013732910156250, 0.0000000000000000, 0.0015869140625000,
+0.0000610351562500, 0.0015258789062500, 0.0000610351562500, 0.0016174316406250,
+0.0000000000000000, 0.0016784667968750, 0.0000000000000000, 0.0017700195312500,
+0.0000305175781250, 0.0018920898437500, 0.0000000000000000, 0.0019531250000000,
+0.0000000000000000, 0.0020141601562500, 0.0000000000000000, 0.0020446777343750,
+0.0000000000000000, 0.0021362304687500, 0.0000305175781250, 0.0021362304687500,
+-0.0000305175781250, 0.0019531250000000, 0.0000000000000000, 0.0020751953125000,
+0.0000000000000000, 0.0018310546875000, 0.0000000000000000, 0.0016174316406250,
+0.0000000000000000, 0.0017395019531250, 0.0000305175781250, 0.0009460449218750,
+0.0000000000000000, 0.0010681152343750, 0.0000000000000000, 0.0007019042968750,
+0.0000000000000000, -0.0000610351562500, 0.0000000000000000, 0.0002136230468750,
+-0.0000305175781250, -0.0007019042968750, 0.0000000000000000, -0.0007019042968750,
+0.0000000000000000, -0.0007934570312500, 0.0000305175781250, -0.0013732910156250,
+0.0000000000000000, -0.0012207031250000, 0.0000000000000000, -0.0012207031250000,
+0.0000000000000000, -0.0014343261718750, -0.0000305175781250, -0.0010375976562500,
+0.0000000000000000, -0.0007629394531250, 0.0000305175781250, -0.0009460449218750,
+0.0000305175781250, -0.0000305175781250, 0.0000610351562500, -0.0000610351562500,
+0.0000305175781250, 0.0001525878906250, 0.0000000000000000, 0.0008544921875000,
+0.0000305175781250, 0.0006103515625000, 0.0000305175781250, 0.0008239746093750,
+-0.0000305175781250, 0.0010681152343750, -0.0000305175781250, 0.0007019042968750,
+0.0000000000000000, 0.0003662109375000, 0.0000000000000000, 0.0006408691406250,
+-0.0000305175781250, -0.0001831054687500, 0.0000305175781250, -0.0002441406250000,
+0.0000305175781250, -0.0002441406250000, -0.0000305175781250, -0.0008239746093750,
+0.0000000000000000, -0.0006713867187500, 0.0000000000000000, -0.0003967285156250,
+0.0000000000000000, -0.0006103515625000, -0.0000305175781250, -0.0002746582031250,
+0.0000305175781250, 0.0002441406250000, 0.0000000000000000, -0.0000610351562500,
+-0.0000610351562500, 0.0003967285156250, 0.0000305175781250, 0.0006103515625000,
+0.0000305175781250, 0.0002746582031250, -0.0000305175781250, 0.0002136230468750,
+0.0000305175781250, 0.0003662109375000, 0.0000000000000000, -0.0001831054687500,
+-0.0000305175781250, -0.0002136230468750, -0.0000305175781250, -0.0000915527343750,
+0.0000000000000000, -0.0003356933593750, 0.0000000000000000, -0.0003356933593750,
+-0.0000305175781250, 0.0000610351562500, 0.0000610351562500, 0.0000000000000000,
+0.0000610351562500, 0.0000610351562500, 0.0000000000000000, 0.0003662109375000,
+0.0000000000000000, 0.0003356933593750, 0.0000305175781250, 0.0000610351562500,
+-0.0000610351562500, 0.0001831054687500, -0.0000305175781250, 0.0000305175781250,
+0.0000000000000000, -0.0002746582031250, -0.0000305175781250, -0.0001525878906250,
+-0.0000610351562500, -0.0000610351562500, 0.0000610351562500, -0.0002136230468750,
+0.0000915527343750, -0.0000915527343750, 0.0000000000000000, 0.0001831054687500,
+0.0000305175781250, 0.0000305175781250, 0.0000915527343750, 0.0000915527343750,
+-0.0000305175781250, 0.0002136230468750, -0.0000915527343750, 0.0000915527343750,
+0.0000000000000000, -0.0001220703125000, 0.0000000000000000, -0.0001220703125000,
+0.0000000000000000, 0.0000000000000000, 0.0000305175781250, -0.0001220703125000,
+0.0000610351562500, -0.0000915527343750, 0.0000610351562500, 0.0001220703125000,
+0.0000305175781250, 0.0000305175781250, 0.0000610351562500, 0.0000305175781250,
+-0.0000915527343750, 0.0001220703125000, -0.0001220703125000, 0.0000305175781250,
+-0.0000610351562500, -0.0001525878906250, -0.0000305175781250, 0.0000000000000000,
+-0.0000305175781250, -0.0000305175781250, 0.0001220703125000, -0.0001220703125000,
+0.0001220703125000, -0.0000305175781250, 0.0000305175781250, 0.0000915527343750,
+0.0000000000000000, 0.0000305175781250, 0.0000610351562500, 0.0000305175781250,
+-0.0001220703125000, 0.0001525878906250, -0.0001831054687500, 0.0000915527343750,
+-0.0000610351562500, -0.0000915527343750, -0.0000305175781250, -0.0000305175781250,
+-0.0001525878906250, 0.0000000000000000, 0.0001831054687500, -0.0001220703125000,
+0.0002441406250000, -0.0000610351562500, 0.0000915527343750, 0.0000610351562500,
+0.0001220703125000, 0.0000000000000000, 0.0002441406250000, 0.0000305175781250,
+-0.0002136230468750, 0.0001220703125000, -0.0001831054687500, 0.0000305175781250,
+-0.0000610351562500, -0.0000610351562500, -0.0003662109375000, -0.0000305175781250,
+-0.0003967285156250, -0.0000305175781250, 0.0000915527343750, -0.0000610351562500,
+-0.0000305175781250, 0.0000000000000000, 0.0000610351562500, 0.0000610351562500,
+0.0004882812500000, 0.0000000000000000, 0.0003356933593750, 0.0000610351562500,
+0.0001525878906250, 0.0000915527343750, 0.0004272460937500, 0.0000305175781250,
+0.0000915527343750, -0.0000305175781250, -0.0003662109375000, 0.0000305175781250,
+-0.0001220703125000, -0.0000305175781250, -0.0004882812500000, 0.0000000000000000,
+-0.0006408691406250, 0.0000305175781250, -0.0003356933593750, 0.0000000000000000,
+-0.0001831054687500, -0.0000305175781250, -0.0003967285156250, 0.0000305175781250,
+0.0003356933593750, 0.0000305175781250, 0.0003356933593750, 0.0000305175781250,
+0.0003662109375000, 0.0000000000000000, 0.0007324218750000, 0.0000305175781250,
+0.0006408691406250},
+{0.0000305175781250, -0.0002441406250000, 0.0000610351562500, -0.0004272460937500,
+0.0000305175781250, -0.0006713867187500, -0.0000305175781250, -0.0003356933593750,
+0.0000305175781250, 0.0000000000000000, -0.0000610351562500, -0.0003051757812500,
+-0.0000305175781250, 0.0003051757812500, 0.0000305175781250, 0.0004272460937500,
+0.0000610351562500, 0.0001525878906250, 0.0000305175781250, 0.0002441406250000,
+0.0000610351562500, 0.0003662109375000, 0.0000305175781250, -0.0001525878906250,
+-0.0000305175781250, -0.0001831054687500, -0.0000305175781250, -0.0000915527343750,
+0.0000000000000000, -0.0003356933593750, -0.0000610351562500, -0.0003356933593750,
+-0.0000610351562500, 0.0000610351562500, 0.0000610351562500, -0.0000305175781250,
+0.0000610351562500, 0.0000000000000000, 0.0000305175781250, 0.0003356933593750,
+0.0000610351562500, 0.0002746582031250, 0.0001220703125000, 0.0000610351562500,
+0.0000000000000000, 0.0001831054687500, -0.0000305175781250, 0.0000305175781250,
+-0.0000305175781250, -0.0002441406250000, -0.0000610351562500, -0.0001220703125000,
+-0.0000915527343750, -0.0000610351562500, 0.0000305175781250, -0.0002136230468750,
+0.0000610351562500, -0.0000915527343750, 0.0000305175781250, 0.0001831054687500,
+0.0000305175781250, 0.0001525878906250, 0.0000305175781250, 0.0000000000000000,
+-0.0000610351562500, 0.0000610351562500, -0.0000610351562500, 0.0000305175781250,
+0.0000000000000000, -0.0001525878906250, -0.0000305175781250, -0.0000610351562500,
+-0.0000305175781250, 0.0000000000000000, 0.0000915527343750, -0.0000915527343750,
+0.0000610351562500, -0.0000305175781250, 0.0000000000000000, 0.0001220703125000,
+0.0000610351562500, 0.0000915527343750, 0.0001220703125000, 0.0000305175781250,
+-0.0000915527343750, 0.0000915527343750, -0.0000915527343750, 0.0000305175781250,
+0.0000000000000000, -0.0001220703125000, -0.0000610351562500, -0.0000915527343750,
+-0.0000915527343750, -0.0000305175781250, 0.0001525878906250, -0.0000610351562500,
+0.0001525878906250, -0.0000305175781250, 0.0000610351562500, 0.0000915527343750,
+0.0001220703125000, 0.0000610351562500, 0.0001831054687500, 0.0000305175781250,
+-0.0002136230468750, 0.0000915527343750, -0.0001831054687500, 0.0000000000000000,
+-0.0000610351562500, -0.0000915527343750, -0.0003051757812500, -0.0000305175781250,
+-0.0003051757812500, -0.0000610351562500, 0.0001220703125000, -0.0000915527343750,
+0.0000000000000000, -0.0000305175781250, 0.0000610351562500, 0.0000915527343750,
+0.0004577636718750, 0.0000305175781250, 0.0003662109375000, 0.0000305175781250,
+0.0000305175781250, 0.0000915527343750, 0.0002136230468750, 0.0000305175781250,
+0.0000000000000000, -0.0000610351562500, -0.0005187988281250, -0.0000305175781250,
+-0.0002441406250000, -0.0000305175781250, -0.0003662109375000, -0.0000610351562500,
+-0.0005798339843750, 0.0000000000000000, -0.0002136230468750, 0.0000610351562500,
+0.0001220703125000, 0.0000000000000000, -0.0001525878906250, 0.0000000000000000,
+0.0004272460937500, 0.0000610351562500, 0.0005493164062500, 0.0000305175781250,
+0.0002746582031250, -0.0000305175781250, 0.0003662109375000, 0.0000000000000000,
+0.0005187988281250, -0.0000305175781250, -0.0002746582031250, -0.0000610351562500,
+-0.0001525878906250, -0.0000305175781250, -0.0002746582031250, 0.0000000000000000,
+-0.0008850097656250, -0.0000305175781250, -0.0006713867187500, 0.0000305175781250,
+-0.0005493164062500, 0.0000305175781250, -0.0007629394531250, 0.0000305175781250,
+-0.0003356933593750, 0.0000000000000000, 0.0001831054687500, 0.0000000000000000,
+-0.0001220703125000, 0.0000000000000000, 0.0007019042968750, 0.0000000000000000,
+0.0007934570312500, 0.0000000000000000, 0.0007019042968750, 0.0000000000000000,
+0.0010681152343750, -0.0000305175781250, 0.0010375976562500, -0.0000305175781250,
+0.0006103515625000, 0.0000305175781250, 0.0008239746093750, 0.0000000000000000,
+0.0003967285156250, 0.0000000000000000, -0.0001831054687500, 0.0000305175781250,
+0.0000915527343750, 0.0000000000000000, -0.0008239746093750, -0.0000305175781250,
+-0.0008239746093750, 0.0000000000000000, -0.0009765625000000, 0.0000000000000000,
+-0.0015563964843750, 0.0000000000000000, -0.0013732910156250, 0.0000000000000000,
+-0.0016174316406250, 0.0000305175781250, -0.0017395019531250, -0.0000305175781250,
+-0.0015869140625000, 0.0000000000000000, -0.0015869140625000, 0.0000000000000000,
+-0.0015869140625000, 0.0000000000000000, -0.0012207031250000, 0.0000000000000000,
+-0.0012207031250000, 0.0000000000000000, -0.0010375976562500, 0.0000000000000000,
+-0.0006713867187500, -0.0000305175781250, -0.0007019042968750, 0.0000000000000000,
+-0.0003662109375000, 0.0000000000000000, -0.0002441406250000, 0.0000000000000000,
+-0.0000915527343750, 0.0000305175781250, 0.0000915527343750, 0.0000000000000000,
+0.0001220703125000, -0.0000305175781250, 0.0003356933593750, 0.0000000000000000,
+0.0004882812500000, 0.0000000000000000, 0.0004882812500000, 0.0000000000000000,
+0.0005187988281250, 0.0000610351562500, 0.0006103515625000, 0.0000000000000000,
+0.0004577636718750, 0.0000000000000000, 0.0005493164062500, -0.0000305175781250,
+0.0005493164062500, 0.0000305175781250, 0.0004272460937500, 0.0000000000000000,
+0.0004882812500000, 0.0000000000000000, 0.0001525878906250, 0.0000000000000000,
+0.0003356933593750, 0.0000000000000000, 0.0001831054687500, 0.0000000000000000,
+-0.0001220703125000, 0.0000000000000000, 0.0001220703125000, 0.0000000000000000,
+-0.0005798339843750, -0.0000305175781250, -0.0004577636718750, -0.0000305175781250,
+-0.0006408691406250, 0.0000000000000000, -0.0012207031250000, 0.0000610351562500,
+-0.0008239746093750, 0.0000000000000000, -0.0017089843750000, 0.0000000000000000,
+-0.0015869140625000, 0.0000000000000000, -0.0016784667968750, 0.0000305175781250,
+-0.0022888183593750, 0.0000000000000000, -0.0020141601562500, 0.0000305175781250,
+-0.0020751953125000, 0.0000000000000000, -0.0023498535156250, -0.0000305175781250,
+-0.0018920898437500, -0.0000305175781250, -0.0016174316406250, -0.0000305175781250,
+-0.0019836425781250, 0.0000000000000000, -0.0005187988281250, 0.0000305175781250,
+-0.0008850097656250, 0.0000000000000000, -0.0003051757812500, 0.0000000000000000,
+0.0009460449218750, 0.0000305175781250, 0.0002441406250000, 0.0000000000000000,
+0.0014648437500000, 0.0000305175781250, 0.0015869140625000, 0.0000000000000000,
+0.0012817382812500, -0.0000305175781250, 0.0017395019531250, -0.0000305175781250,
+0.0018005371093750, 0.0000000000000000, 0.0007324218750000, 0.0000000000000000,
+0.0010986328125000, 0.0000000000000000, 0.0004882812500000, 0.0000305175781250,
+-0.0006408691406250, 0.0000305175781250, 0.0000305175781250, 0.0000000000000000,
+-0.0011291503906250, 0.0000000000000000, -0.0013427734375000, 0.0000000000000000,
+-0.0010986328125000, -0.0000305175781250, -0.0014648437500000, -0.0000305175781250,
+-0.0014953613281250, 0.0000000000000000, -0.0005798339843750, -0.0000305175781250,
+-0.0008544921875000, 0.0000000000000000, -0.0003356933593750, 0.0000610351562500,
+0.0006103515625000, 0.0000305175781250, 0.0001525878906250, 0.0000000000000000,
+0.0008544921875000, 0.0000000000000000, 0.0011596679687500, 0.0000000000000000,
+0.0007324218750000, -0.0000610351562500, 0.0006103515625000, -0.0000305175781250,
+0.0008544921875000, 0.0000305175781250, -0.0002136230468750, -0.0000305175781250,
+-0.0000915527343750, -0.0000305175781250, -0.0002136230468750, 0.0001220703125000,
+-0.0009460449218750, 0.0000305175781250, -0.0006408691406250, 0.0000610351562500,
+-0.0004882812500000, 0.0000610351562500, -0.0007934570312500, 0.0000000000000000,
+-0.0004272460937500, -0.0001220703125000, 0.0001525878906250, -0.0000610351562500,
+-0.0002441406250000, -0.0000305175781250, 0.0004882812500000, -0.0000915527343750,
+0.0006408691406250, 0.0000305175781250, 0.0003662109375000, 0.0001220703125000,
+0.0005187988281250, 0.0000305175781250, 0.0006713867187500, 0.0000610351562500,
+-0.0002136230468750, 0.0001525878906250, -0.0000915527343750, 0.0000610351562500,
+-0.0001525878906250, -0.0000610351562500, -0.0008544921875000, 0.0000305175781250,
+-0.0006713867187500, -0.0001220703125000, -0.0001831054687500, -0.0001831054687500,
+-0.0005798339843750, -0.0000610351562500, -0.0001525878906250, 0.0000000000000000,
+0.0007019042968750, -0.0001220703125000, 0.0002746582031250, 0.0001831054687500,
+0.0003662109375000, 0.0001831054687500, 0.0008239746093750, 0.0000915527343750,
+0.0002441406250000, 0.0001831054687500, -0.0004272460937500, 0.0002136230468750,
+0.0000610351562500, -0.0001831054687500, -0.0004882812500000, -0.0001525878906250,
+-0.0007934570312500, -0.0000915527343750, -0.0003662109375000, -0.0003662109375000,
+0.0001525878906250, 0.0003051757812500, 0.0001525878906250, -0.0000305175781250,
+-0.0004272460937500, 0.0002441406250000, -0.0007019042968750, 0.0000000000000000,
+-0.0002136230468750, -0.0005798339843750, 0.0001831054687500, -0.0003051757812500,
+-0.0001831054687500, -0.0001831054687500, 0.0003662109375000, -0.0004577636718750,
+0.0005798339843750, -0.0000915527343750, 0.0001831054687500, 0.0005187988281250,
+0.0000000000000000, 0.0000610351562500, 0.0003051757812500, 0.0005493164062500,
+-0.0003051757812500, 0.0009155273437500, -0.0004577636718750, 0.0003051757812500,
+-0.0001525878906250, -0.0000305175781250, -0.0001220703125000, 0.0004577636718750,
+-0.0002746582031250, -0.0006713867187500, 0.0002136230468750, -0.0008850097656250,
+0.0003051757812500, -0.0004272460937500, 0.0001220703125000, -0.0006408691406250,
+0.0001525878906250, -0.0009155273437500, 0.0002136230468750, 0.0003967285156250,
+-0.0001220703125000, 0.0002746582031250, -0.0001525878906250, 0.0002441406250000,
+-0.0000305175781250, 0.0010986328125000, -0.0001220703125000, 0.0009765625000000,
+-0.0001831054687500, 0.0000915527343750, 0.0001220703125000, 0.0006103515625000,
+0.0001525878906250, 0.0000610351562500, 0.0000305175781250, -0.0010986328125000,
+0.0001220703125000, -0.0005187988281250, 0.0001525878906250, -0.0007934570312500,
+-0.0000610351562500, -0.0013732910156250, -0.0000915527343750, -0.0005798339843750,
+-0.0000305175781250, 0.0000915527343750, -0.0000915527343750, -0.0005187988281250,
+-0.0001220703125000, 0.0007934570312500, 0.0000610351562500, 0.0009765625000000,
+0.0000610351562500, 0.0005798339843750, 0.0000305175781250, 0.0010375976562500,
+0.0000915527343750, 0.0011291503906250, 0.0001220703125000, -0.0000915527343750,
+-0.0000305175781250, 0.0003051757812500, -0.0000305175781250, -0.0001220703125000,
+0.0000000000000000, -0.0012817382812500, -0.0000610351562500, -0.0006408691406250,
+-0.0000610351562500, -0.0010681152343750, 0.0000305175781250, -0.0015563964843750,
+0.0000305175781250, -0.0008850097656250, 0.0000000000000000, -0.0004882812500000,
+0.0000305175781250, -0.0009765625000000, 0.0000610351562500, 0.0005187988281250,
+-0.0000305175781250, 0.0003967285156250, -0.0000305175781250, 0.0004882812500000,
+0.0000000000000000, 0.0014648437500000, -0.0000610351562500, 0.0010986328125000,
+-0.0000305175781250, 0.0011901855468750, 0.0000305175781250, 0.0014953613281250,
+0.0000305175781250, 0.0010375976562500, 0.0000305175781250, 0.0007324218750000,
+0.0000610351562500, 0.0010986328125000, 0.0000305175781250, -0.0002746582031250,
+0.0000000000000000, -0.0000610351562500, -0.0000305175781250, -0.0004882812500000,
+0.0000000000000000, -0.0015258789062500, 0.0000000000000000, -0.0009155273437500,
+0.0000000000000000, -0.0018310546875000, 0.0000000000000000, -0.0019836425781250,
+0.0000305175781250},
+{0.0000000000000000, 0.0001525878906250, 0.0002441406250000, 0.0000610351562500,
+-0.0000305175781250, 0.0001220703125000, -0.0008544921875000, 0.0001525878906250,
+-0.0005798339843750, -0.0001831054687500, -0.0002136230468750, -0.0001220703125000,
+-0.0006103515625000, -0.0000915527343750, -0.0001831054687500, -0.0002746582031250,
+0.0007324218750000, -0.0002441406250000, 0.0002441406250000, 0.0000915527343750,
+0.0004577636718750, 0.0000305175781250, 0.0008239746093750, 0.0000305175781250,
+0.0003967285156250, 0.0004577636718750, -0.0004577636718750, -0.0002441406250000,
+0.0001220703125000, -0.0000915527343750, 0.0003356933593750, -0.0003967285156250,
+0.0007629394531250, -0.0000305175781250, 0.0002136230468750, 0.0004882812500000,
+-0.0003662109375000, 0.0001831054687500, 0.0000000000000000, 0.0003662109375000,
+-0.0003051757812500, 0.0006713867187500, -0.0006408691406250, 0.0002136230468750,
+-0.0002136230468750, -0.0001831054687500, 0.0002746582031250, 0.0001525878906250,
+-0.0001220703125000, -0.0006713867187500, 0.0002746582031250, -0.0008850097656250,
+0.0005493164062500, -0.0003662109375000, 0.0001831054687500, -0.0003967285156250,
+-0.0000610351562500, -0.0007019042968750, 0.0002136230468750, 0.0005798339843750,
+-0.0001831054687500, 0.0005493164062500, -0.0003662109375000, 0.0003356933593750,
+-0.0001220703125000, 0.0010070800781250, 0.0000000000000000, 0.0009765625000000,
+-0.0001831054687500, -0.0002136230468750, 0.0001831054687500, 0.0001525878906250,
+0.0002746582031250, -0.0000915527343750, 0.0000610351562500, -0.0012817382812500,
+0.0000305175781250, -0.0008239746093750, 0.0001525878906250, -0.0004882812500000,
+-0.0001525878906250, -0.0010375976562500, -0.0002136230468750, -0.0003356933593750,
+-0.0000610351562500, 0.0007019042968750, -0.0000305175781250, 0.0000305175781250,
+-0.0001220703125000, 0.0009155273437500, 0.0000915527343750, 0.0013732910156250,
+0.0001831054687500, 0.0006103515625000, 0.0000610351562500, 0.0003356933593750,
+0.0000305175781250, 0.0008544921875000, 0.0001220703125000, -0.0007629394531250,
+-0.0000610351562500, -0.0007324218750000, -0.0001220703125000, -0.0005493164062500,
+-0.0000305175781250, -0.0013427734375000, -0.0000305175781250, -0.0012512207031250,
+-0.0001220703125000, -0.0003967285156250, 0.0000610351562500, -0.0008544921875000,
+0.0000610351562500, -0.0002746582031250, 0.0000000000000000, 0.0008544921875000,
+0.0000000000000000, 0.0001525878906250, 0.0000610351562500, 0.0011291503906250,
+-0.0000305175781250, 0.0014343261718750, -0.0000915527343750, 0.0008850097656250,
+-0.0000305175781250, 0.0008544921875000, -0.0000305175781250, 0.0011901855468750,
+-0.0000610351562500, -0.0001831054687500, 0.0000610351562500, 0.0000915527343750,
+0.0000610351562500, -0.0002746582031250, 0.0000000000000000, -0.0014038085937500,
+0.0000000000000000, -0.0008239746093750, 0.0000305175781250, -0.0014953613281250,
+0.0000000000000000, -0.0017700195312500, -0.0000305175781250, -0.0013427734375000,
+-0.0000305175781250, -0.0013732910156250, 0.0000305175781250, -0.0015563964843750,
+-0.0000305175781250, -0.0003662109375000, 0.0000000000000000, -0.0007934570312500,
+0.0000610351562500, -0.0001525878906250, 0.0000000000000000, 0.0009460449218750,
+-0.0000305175781250, 0.0001831054687500, 0.0000305175781250, 0.0015563964843750,
+-0.0000305175781250, 0.0015258789062500, -0.0000305175781250, 0.0014038085937500,
+0.0000000000000000, 0.0021667480468750, 0.0000305175781250, 0.0019531250000000,
+0.0000000000000000, 0.0017395019531250, 0.0000000000000000, 0.0021362304687500,
+0.0000000000000000, 0.0015563964843750, 0.0000000000000000, 0.0009765625000000,
+0.0000000000000000, 0.0015869140625000, 0.0000000000000000, -0.0000305175781250,
+0.0000000000000000, 0.0003662109375000, 0.0000000000000000, -0.0000610351562500,
+0.0000305175781250, -0.0012817382812500, 0.0000305175781250, -0.0004577636718750,
+0.0000000000000000, -0.0019836425781250, 0.0000000000000000, -0.0017700195312500,
+0.0000305175781250, -0.0019226074218750, 0.0000305175781250, -0.0028686523437500,
+-0.0000305175781250, -0.0023498535156250, 0.0000000000000000, -0.0029602050781250,
+0.0000000000000000, -0.0029907226562500, 0.0000000000000000, -0.0029907226562500,
+0.0000305175781250, -0.0032348632812500, 0.0000305175781250, -0.0030822753906250,
+0.0000000000000000, -0.0031738281250000, 0.0000000000000000, -0.0032043457031250,
+0.0000000000000000, -0.0032043457031250, 0.0000000000000000, -0.0031738281250000,
+0.0000000000000000, -0.0031738281250000, 0.0000000000000000, -0.0030822753906250,
+0.0000000000000000, -0.0030822753906250, 0.0000305175781250, -0.0030822753906250,
+0.0000305175781250, -0.0029602050781250, -0.0000305175781250, -0.0029907226562500,
+0.0000000000000000, -0.0028991699218750, 0.0000000000000000, -0.0029296875000000,
+0.0000000000000000, -0.0028381347656250, 0.0000000000000000, -0.0026855468750000,
+0.0000000000000000, -0.0026550292968750, 0.0000000000000000, -0.0023193359375000,
+0.0000000000000000, -0.0023803710937500, 0.0000000000000000, -0.0021057128906250,
+0.0000000000000000, -0.0017089843750000, 0.0000305175781250, -0.0018615722656250,
+-0.0000305175781250, -0.0009765625000000, -0.0000305175781250, -0.0010986328125000,
+0.0000000000000000, -0.0007324218750000, 0.0000000000000000, 0.0000305175781250,
+0.0000000000000000, -0.0002746582031250, 0.0000000000000000, 0.0006713867187500,
+-0.0000305175781250, 0.0007324218750000, 0.0000305175781250, 0.0008544921875000,
+0.0000000000000000, 0.0015258789062500, 0.0000000000000000, 0.0014038085937500,
+0.0000000000000000, 0.0015563964843750, -0.0000305175781250, 0.0017089843750000,
+0.0000000000000000, 0.0014038085937500, 0.0000000000000000, 0.0011901855468750,
+0.0000000000000000, 0.0013427734375000, 0.0000305175781250, 0.0003662109375000,
+0.0000610351562500, 0.0004882812500000, 0.0000000000000000, 0.0001525878906250,
+0.0000000000000000, -0.0007324218750000, 0.0000000000000000, -0.0003967285156250,
+0.0000610351562500, -0.0010986328125000, 0.0000305175781250, -0.0012207031250000,
+-0.0000305175781250, -0.0010375976562500, 0.0000000000000000, -0.0012207031250000,
+0.0000000000000000, -0.0012817382812500, -0.0000305175781250, -0.0004272460937500,
+-0.0000305175781250, -0.0006713867187500, 0.0000000000000000, -0.0002441406250000,
+0.0000305175781250, 0.0006103515625000, 0.0000305175781250, 0.0002441406250000,
+0.0000305175781250, 0.0007324218750000, -0.0000305175781250, 0.0009765625000000,
+0.0000000000000000, 0.0005798339843750, 0.0000305175781250, 0.0005187988281250,
+-0.0000305175781250, 0.0007019042968750, -0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, -0.0001831054687500, 0.0000000000000000, -0.0001831054687500,
+0.0000305175781250, -0.0008239746093750, 0.0000610351562500, -0.0006408691406250,
+0.0000610351562500, -0.0003051757812500, 0.0000305175781250, -0.0005493164062500,
+0.0000000000000000, -0.0002441406250000, -0.0000305175781250, 0.0003356933593750,
+-0.0000610351562500, 0.0000610351562500, 0.0000000000000000, 0.0003051757812500,
+0.0000610351562500, 0.0005798339843750, -0.0000305175781250, 0.0001831054687500,
+-0.0000305175781250, -0.0000915527343750, 0.0000915527343750, 0.0001831054687500,
+0.0000915527343750, -0.0003356933593750, -0.0000305175781250, -0.0004882812500000,
+0.0000000000000000, -0.0002136230468750, 0.0000305175781250, -0.0001831054687500,
+-0.0000915527343750, -0.0003051757812500, -0.0000610351562500, 0.0002136230468750,
+0.0000305175781250, 0.0002441406250000, 0.0000000000000000, 0.0000915527343750,
+0.0000000000000000, 0.0002746582031250, 0.0001220703125000, 0.0003662109375000,
+0.0000610351562500, -0.0001220703125000, -0.0000305175781250, -0.0000915527343750,
+0.0000610351562500, -0.0000305175781250, 0.0000000000000000, -0.0002746582031250,
+-0.0001220703125000, -0.0003051757812500, -0.0000610351562500, 0.0000305175781250,
+0.0000610351562500, -0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+0.0000000000000000, 0.0002136230468750, 0.0001220703125000, 0.0001525878906250,
+0.0000610351562500, -0.0001220703125000, -0.0000610351562500, -0.0000610351562500,
+0.0000305175781250, 0.0000000000000000, 0.0000305175781250, -0.0001220703125000,
+-0.0001220703125000, -0.0001220703125000, -0.0000610351562500, 0.0000915527343750,
+0.0000610351562500, 0.0000000000000000, 0.0000000000000000, 0.0000000000000000,
+0.0000000000000000, 0.0001220703125000, 0.0001525878906250, 0.0001525878906250,
+0.0000915527343750, -0.0000305175781250, -0.0000305175781250, 0.0000000000000000,
+-0.0000305175781250, 0.0000305175781250, 0.0000000000000000, -0.0000915527343750,
+-0.0001831054687500, -0.0001220703125000, -0.0001831054687500, 0.0000610351562500,
+0.0000915527343750, 0.0000305175781250, 0.0000000000000000, -0.0000305175781250,
+0.0000610351562500, 0.0001525878906250, 0.0002746582031250, 0.0001525878906250,
+0.0001831054687500, -0.0000305175781250, -0.0000915527343750, 0.0000000000000000,
+0.0000915527343750, 0.0000000000000000, -0.0000305175781250, -0.0001525878906250,
+-0.0004272460937500, -0.0000915527343750, -0.0003051757812500, 0.0000000000000000,
+-0.0000915527343750, 0.0000000000000000, -0.0003356933593750, 0.0000305175781250,
+-0.0000610351562500, 0.0001220703125000, 0.0003967285156250, 0.0001525878906250,
+0.0001525878906250, 0.0000000000000000, 0.0002746582031250, 0.0000305175781250,
+0.0005187988281250, 0.0000305175781250, 0.0001831054687500, -0.0000915527343750,
+-0.0001525878906250, -0.0000305175781250, 0.0001220703125000, -0.0000305175781250,
+-0.0004272460937500, -0.0000305175781250, -0.0006103515625000, 0.0000000000000000,
+-0.0003051757812500, 0.0000915527343750, -0.0003051757812500, 0.0000610351562500,
+-0.0004882812500000, 0.0000000000000000, 0.0002441406250000, 0.0000305175781250,
+0.0001831054687500, 0.0000610351562500, 0.0001831054687500, -0.0000305175781250,
+0.0006408691406250, 0.0000000000000000, 0.0005798339843750, 0.0000000000000000,
+0.0001831054687500, -0.0000305175781250, 0.0004577636718750, 0.0000000000000000,
+0.0000305175781250, 0.0000610351562500, -0.0006103515625000, 0.0000305175781250,
+-0.0003051757812500, 0.0000305175781250, -0.0008239746093750, 0.0000000000000000,
+-0.0010681152343750, 0.0000305175781250, -0.0006408691406250, -0.0000305175781250,
+-0.0006408691406250, 0.0000000000000000, -0.0007934570312500, -0.0000305175781250,
+0.0001220703125000, -0.0000610351562500, 0.0000610351562500, -0.0000305175781250,
+0.0003051757812500, 0.0000305175781250, 0.0010375976562500, 0.0000000000000000,
+0.0007934570312500, 0.0000305175781250, 0.0010375976562500, 0.0000000000000000,
+0.0012512207031250, 0.0000000000000000, 0.0009155273437500, -0.0000305175781250,
+0.0007629394531250, 0.0000000000000000, 0.0009155273437500, 0.0000000000000000,
+0.0000000000000000, -0.0000305175781250, 0.0001525878906250, 0.0000000000000000,
+-0.0001525878906250, 0.0000000000000000, -0.0009155273437500, 0.0000305175781250,
+-0.0006713867187500, 0.0000000000000000, -0.0013732910156250, 0.0000000000000000,
+-0.0014648437500000, 0.0000000000000000, -0.0015258789062500, 0.0000000000000000,
+-0.0019226074218750, -0.0000305175781250, -0.0018310546875000, 0.0000000000000000,
+-0.0019836425781250, -0.0000305175781250, -0.0020141601562500, 0.0000000000000000,
+-0.0019226074218750, 0.0000610351562500, -0.0018310546875000, 0.0000000000000000,
+-0.0018310546875000, 0.0000305175781250, -0.0016174316406250, 0.0000000000000000,
+-0.0015563964843750, 0.0000000000000000, -0.0014648437500000, -0.0000305175781250,
+-0.0013122558593750},
+{-0.0000305175781250, 0.0001220703125000, 0.0001525878906250, 0.0001831054687500,
+0.0000915527343750, -0.0000915527343750, 0.0000000000000000, -0.0001220703125000,
+0.0000610351562500, -0.0000305175781250, 0.0000000000000000, -0.0000915527343750,
+-0.0001525878906250, -0.0001220703125000, -0.0000915527343750, 0.0000915527343750,
+0.0000305175781250, 0.0001220703125000, -0.0000915527343750, 0.0000305175781250,
+-0.0000305175781250, 0.0000915527343750, 0.0002136230468750, 0.0001525878906250,
+0.0001525878906250, -0.0000610351562500, 0.0000000000000000, -0.0000915527343750,
+0.0000915527343750, -0.0000610351562500, -0.0000305175781250, -0.0000915527343750,
+-0.0003051757812500, -0.0001220703125000, -0.0002136230468750, 0.0000610351562500,
+-0.0000610351562500, 0.0000610351562500, -0.0002746582031250, 0.0000000000000000,
+-0.0000305175781250, 0.0001220703125000, 0.0003662109375000, 0.0001220703125000,
+0.0001220703125000, -0.0000610351562500, 0.0001525878906250, 0.0000000000000000,
+0.0004272460937500, 0.0000000000000000, 0.0000610351562500, -0.0000915527343750,
+-0.0003356933593750, -0.0000610351562500, -0.0000305175781250, 0.0000305175781250,
+-0.0004272460937500, 0.0000000000000000, -0.0006103515625000, 0.0000000000000000,
+-0.0003051757812500, 0.0000915527343750, -0.0000915527343750, 0.0000915527343750,
+-0.0003051757812500, 0.0000000000000000, 0.0003662109375000, -0.0000305175781250,
+0.0004272460937500, 0.0000305175781250, 0.0002441406250000, -0.0000305175781250,
+0.0004577636718750, -0.0000610351562500, 0.0005493164062500, 0.0000610351562500,
+-0.0001525878906250, -0.0000305175781250, 0.0000000000000000, -0.0000305175781250,
+-0.0001831054687500, 0.0000610351562500, -0.0008239746093750, 0.0000915527343750,
+-0.0005798339843750, 0.0000000000000000, -0.0004882812500000, 0.0000305175781250,
+-0.0007629394531250, 0.0000305175781250, -0.0003051757812500, -0.0000305175781250,
+0.0002441406250000, -0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+0.0006713867187500, -0.0000305175781250, 0.0008239746093750, -0.0000305175781250,
+0.0006408691406250, 0.0000305175781250, 0.0008239746093750, 0.0000305175781250,
+0.0008544921875000, 0.0000000000000000, 0.0000000000000000, 0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, -0.0001220703125000, -0.0000305175781250,
+-0.0009460449218750, 0.0000000000000000, -0.0005493164062500, 0.0000000000000000,
+-0.0011901855468750, 0.0000000000000000, -0.0013427734375000, 0.0000000000000000,
+-0.0011596679687500, 0.0000000000000000, -0.0012512207031250, 0.0000000000000000,
+-0.0012512207031250, 0.0000000000000000, -0.0006103515625000, 0.0000305175781250,
+-0.0007629394531250, 0.0000610351562500, -0.0004272460937500, -0.0000305175781250,
+0.0001831054687500, -0.0000305175781250, -0.0000305175781250, 0.0000305175781250,
+0.0008544921875000, 0.0000000000000000, 0.0008544921875000, 0.0000000000000000,
+0.0010375976562500, 0.0000000000000000, 0.0016174316406250, 0.0000305175781250,
+0.0014343261718750, 0.0000000000000000, 0.0017700195312500, 0.0000000000000000,
+0.0019226074218750, 0.0000305175781250, 0.0018615722656250, 0.0000000000000000,
+0.0019836425781250, 0.0000000000000000, 0.0019531250000000, 0.0000000000000000,
+0.0018615722656250, 0.0000000000000000, 0.0018615722656250, 0.0000000000000000,
+0.0017395019531250, 0.0000000000000000, 0.0016784667968750, 0.0000000000000000,
+0.0016479492187500, 0.0000000000000000, 0.0015563964843750, 0.0000000000000000,
+0.0014038085937500, 0.0000000000000000, 0.0014038085937500, 0.0000000000000000,
+0.0014038085937500, 0.0000000000000000, 0.0012817382812500, 0.0000305175781250,
+0.0014343261718750, -0.0000305175781250, 0.0013732910156250, 0.0000000000000000,
+0.0014343261718750, 0.0000000000000000, 0.0016174316406250, 0.0000000000000000,
+0.0014953613281250, 0.0000000000000000, 0.0017700195312500, 0.0000305175781250,
+0.0016784667968750, 0.0000305175781250, 0.0018615722656250, 0.0000000000000000,
+0.0020751953125000, 0.0000000000000000, 0.0018615722656250, 0.0000000000000000,
+0.0022888183593750, 0.0000000000000000, 0.0022888183593750, 0.0000000000000000,
+0.0022277832031250, -0.0000305175781250, 0.0025024414062500, 0.0000000000000000,
+0.0023498535156250, 0.0000000000000000, 0.0022888183593750, 0.0000000000000000,
+0.0025024414062500, 0.0000000000000000, 0.0021057128906250, -0.0000305175781250,
+0.0017700195312500, 0.0000305175781250, 0.0021667480468750, 0.0000000000000000,
+0.0007629394531250, 0.0000000000000000, 0.0011901855468750, 0.0000000000000000,
+0.0006408691406250, 0.0000000000000000, -0.0006408691406250, -0.0000305175781250,
+0.0001831054687500, 0.0000000000000000, -0.0014343261718750, 0.0000305175781250,
+-0.0015258789062500, 0.0000000000000000, -0.0014038085937500, -0.0000305175781250,
+-0.0021972656250000, 0.0000305175781250, -0.0020141601562500, 0.0000000000000000,
+-0.0014648437500000, -0.0000305175781250, -0.0019226074218750, 0.0000000000000000,
+-0.0011596679687500, 0.0000000000000000, -0.0002136230468750, 0.0000000000000000,
+-0.0009155273437500, 0.0000305175781250, 0.0007934570312500, 0.0000305175781250,
+0.0007934570312500, 0.0000305175781250, 0.0008544921875000, 0.0000305175781250,
+0.0018920898437500, 0.0000000000000000, 0.0015258789062500, 0.0000000000000000,
+0.0011901855468750, -0.0000305175781250, 0.0016479492187500, 0.0000000000000000,
+0.0010070800781250, 0.0000305175781250, 0.0001525878906250, 0.0000000000000000,
+0.0006713867187500, 0.0000305175781250, -0.0007934570312500, 0.0000610351562500,
+-0.0009155273437500, 0.0000305175781250, -0.0007934570312500, -0.0000305175781250,
+-0.0014953613281250, 0.0000305175781250, -0.0013427734375000, 0.0000000000000000,
+-0.0007019042968750, -0.0000915527343750, -0.0010986328125000, 0.0000000000000000,
+-0.0004882812500000, 0.0000000000000000, 0.0004882812500000, 0.0000000000000000,
+0.0000000000000000, 0.0000305175781250, 0.0007629394531250, 0.0000610351562500,
+0.0010681152343750, 0.0000305175781250, 0.0006103515625000, 0.0000305175781250,
+0.0005798339843750, 0.0000305175781250, 0.0007934570312500, -0.0000305175781250,
+-0.0002136230468750, -0.0000915527343750, -0.0000610351562500, -0.0000305175781250,
+-0.0001831054687500, 0.0000000000000000, -0.0009460449218750, -0.0000610351562500,
+-0.0007019042968750, 0.0000915527343750, -0.0005187988281250, 0.0001220703125000,
+-0.0009155273437500, 0.0000305175781250, -0.0003356933593750, 0.0000610351562500,
+0.0003051757812500, 0.0000915527343750, -0.0001831054687500, -0.0000610351562500,
+0.0006713867187500, -0.0000915527343750, 0.0009155273437500, -0.0000610351562500,
+0.0004272460937500, -0.0001220703125000, 0.0003967285156250, -0.0001220703125000,
+0.0006713867187500, 0.0001220703125000, -0.0005187988281250, 0.0000915527343750,
+-0.0005493164062500, 0.0000610351562500, -0.0003051757812500, 0.0002746582031250,
+-0.0008850097656250, 0.0002136230468750, -0.0009460449218750, -0.0000610351562500,
+0.0002441406250000, 0.0000610351562500, 0.0000000000000000, -0.0000305175781250,
+0.0000610351562500, -0.0003356933593750, 0.0010070800781250, -0.0002441406250000,
+0.0007629394531250, -0.0000915527343750, 0.0000915527343750, -0.0002441406250000,
+0.0004272460937500, -0.0001220703125000, 0.0002441406250000, 0.0004882812500000,
+-0.0009460449218750, 0.0000915527343750, 0.0006713867187500, -0.0003967285156250,
+-0.0001220703125000, -0.0005493164062500, 0.0004882812500000, -0.0001525878906250,
+0.0000915527343750, 0.0000915527343750, -0.0007629394531250, -0.0002441406250000,
+-0.0005187988281250, 0.0004882812500000, -0.0001220703125000, 0.0007324218750000,
+-0.0004272460937500, 0.0003356933593750, -0.0001220703125000, 0.0004577636718750,
+0.0006103515625000, 0.0006408691406250, 0.0003051757812500, -0.0005493164062500,
+0.0001220703125000, -0.0004882812500000, 0.0004882812500000, -0.0003356933593750,
+0.0002136230468750, -0.0010986328125000, -0.0003662109375000, -0.0010375976562500,
+-0.0001525878906250, 0.0000915527343750, -0.0001525878906250, -0.0003356933593750,
+-0.0004272460937500, 0.0000305175781250, -0.0001220703125000, 0.0012817382812500,
+0.0002746582031250, 0.0007629394531250, 0.0000610351562500, 0.0003662109375000,
+0.0001525878906250, 0.0010681152343750, 0.0003967285156250, 0.0002746582031250,
+0.0001525878906250, -0.0009460449218750, -0.0001831054687500, -0.0001525878906250,
+0.0000000000000000, -0.0009460449218750, -0.0001220703125000, -0.0015258789062500,
+-0.0002441406250000, -0.0006408691406250, -0.0000915527343750, -0.0002441406250000,
+0.0001220703125000, -0.0008850097656250, -0.0000305175781250, 0.0008544921875000,
+0.0000915527343750, 0.0009155273437500, 0.0002441406250000, 0.0005493164062500,
+0.0000915527343750, 0.0012512207031250, -0.0000610351562500, 0.0012817382812500,
+0.0000305175781250, -0.0001220703125000, -0.0000610351562500, 0.0002441406250000,
+-0.0001220703125000, -0.0000915527343750, 0.0000000000000000, -0.0013732910156250,
+0.0000915527343750, -0.0007934570312500, 0.0000305175781250, -0.0010070800781250,
+0.0000305175781250, -0.0015563964843750, 0.0001220703125000, -0.0007629394531250,
+0.0000305175781250, -0.0001525878906250, -0.0000610351562500, -0.0007934570312500,
+0.0000305175781250, 0.0008544921875000, -0.0000305175781250, 0.0007934570312500,
+-0.0000915527343750, 0.0006713867187500, 0.0000000000000000, 0.0015563964843750,
+0.0000610351562500, 0.0013732910156250, 0.0000000000000000, 0.0006408691406250,
+0.0000305175781250, 0.0012207031250000, 0.0000915527343750, 0.0005798339843750,
+0.0000305175781250, -0.0004577636718750, -0.0000305175781250, 0.0001831054687500,
+-0.0000305175781250, -0.0011596679687500, -0.0000305175781250, -0.0011901855468750,
+-0.0000610351562500, -0.0011291503906250, 0.0000000000000000, -0.0019226074218750,
+0.0000305175781250, -0.0016479492187500, 0.0000000000000000, -0.0013732910156250,
+0.0000305175781250, -0.0017700195312500, 0.0000000000000000, -0.0011291503906250,
+0.0000305175781250, -0.0004577636718750, -0.0000305175781250, -0.0010681152343750,
+-0.0000305175781250, 0.0006103515625000, 0.0000000000000000, 0.0003051757812500,
+-0.0000305175781250, 0.0006103515625000, 0.0000000000000000, 0.0018615722656250,
+0.0000305175781250, 0.0011596679687500, 0.0000000000000000, 0.0020751953125000,
+0.0000000000000000, 0.0022888183593750, 0.0000305175781250, 0.0019226074218750,
+0.0000305175781250, 0.0022277832031250, 0.0000305175781250, 0.0023803710937500,
+0.0000000000000000, 0.0015258789062500, 0.0000305175781250, 0.0018615722656250,
+0.0000000000000000, 0.0014343261718750, -0.0000305175781250, 0.0006408691406250,
+0.0000000000000000, 0.0013427734375000, -0.0000305175781250, -0.0002441406250000,
+-0.0000305175781250, 0.0001525878906250, 0.0000305175781250, -0.0001220703125000,
+0.0000305175781250, -0.0012817382812500, 0.0000000000000000, -0.0006103515625000,
+0.0000000000000000, -0.0017089843750000, 0.0000000000000000, -0.0015869140625000,
+0.0000305175781250, -0.0016174316406250, 0.0000000000000000, -0.0022583007812500,
+0.0000000000000000, -0.0018005371093750, 0.0000000000000000, -0.0023803710937500,
+0.0000305175781250, -0.0022277832031250, 0.0000000000000000, -0.0021972656250000,
+0.0000000000000000, -0.0025024414062500, 0.0000000000000000, -0.0021972656250000,
+0.0000000000000000, -0.0024414062500000, 0.0000000000000000, -0.0023193359375000,
+0.0000305175781250, -0.0022583007812500, -0.0000305175781250, -0.0023803710937500,
+0.0000000000000000, -0.0022277832031250, 0.0000000000000000, -0.0020751953125000,
+0.0000610351562500},
+{-0.0001831054687500, -0.0007934570312500, -0.0000915527343750, -0.0014648437500000,
+-0.0002746582031250, -0.0005187988281250, -0.0001220703125000, 0.0004577636718750,
+0.0002746582031250, -0.0003967285156250, 0.0001220703125000, 0.0009765625000000,
+0.0000610351562500, 0.0013427734375000, 0.0002441406250000, 0.0006103515625000,
+0.0000915527343750, 0.0007019042968750, -0.0001831054687500, 0.0010986328125000,
+-0.0000305175781250, -0.0007019042968750, -0.0000610351562500, -0.0005493164062500,
+-0.0002136230468750, -0.0004272460937500, -0.0000610351562500, -0.0015563964843750,
+0.0001525878906250, -0.0013732910156250, 0.0000610351562500, -0.0003662109375000,
+0.0000305175781250, -0.0009460449218750, 0.0001525878906250, -0.0003051757812500,
+0.0000610351562500, 0.0009765625000000, -0.0001220703125000, 0.0002136230468750,
+-0.0000305175781250, 0.0010070800781250, -0.0000610351562500, 0.0014343261718750,
+-0.0001220703125000, 0.0007629394531250, -0.0000305175781250, 0.0005798339843750,
+0.0000610351562500, 0.0010681152343750, 0.0000610351562500, -0.0006103515625000,
+0.0000305175781250, -0.0003967285156250, 0.0000915527343750, -0.0005187988281250,
+0.0000305175781250, -0.0016784667968750, -0.0000610351562500, -0.0012817382812500,
+-0.0000610351562500, -0.0011291503906250, 0.0000000000000000, -0.0016174316406250,
+-0.0000305175781250, -0.0010070800781250, 0.0000000000000000, -0.0002746582031250,
+0.0000305175781250, -0.0009155273437500, 0.0000305175781250, 0.0006103515625000,
+-0.0000305175781250, 0.0005187988281250, 0.0000000000000000, 0.0006713867187500,
+0.0000305175781250, 0.0016784667968750, 0.0000000000000000, 0.0012512207031250,
+0.0000000000000000, 0.0014953613281250, 0.0000305175781250, 0.0018615722656250,
+-0.0000305175781250, 0.0012817382812500, 0.0000000000000000, 0.0010070800781250,
+0.0000305175781250, 0.0014343261718750, 0.0000000000000000, -0.0001525878906250,
+0.0000000000000000, 0.0002746582031250, 0.0000000000000000, -0.0001831054687500,
+0.0000305175781250, -0.0014648437500000, -0.0000305175781250, -0.0006713867187500,
+-0.0000305175781250, -0.0019836425781250, 0.0000000000000000, -0.0020141601562500,
+0.0000305175781250, -0.0018920898437500, 0.0000305175781250, -0.0025329589843750,
+0.0000305175781250, -0.0024414062500000, 0.0000000000000000, -0.0021057128906250,
+0.0000000000000000, -0.0024414062500000, -0.0000305175781250, -0.0020141601562500,
+0.0000000000000000, -0.0014953613281250, 0.0000305175781250, -0.0020141601562500,
+0.0000000000000000, -0.0007019042968750, -0.0000305175781250, -0.0011291503906250,
+0.0000305175781250, -0.0007629394531250, 0.0000305175781250, 0.0002746582031250,
+-0.0000305175781250, -0.0004577636718750, 0.0000000000000000, 0.0007019042968750,
+-0.0000305175781250, 0.0005798339843750, 0.0000000000000000, 0.0006408691406250,
+-0.0000305175781250, 0.0013732910156250, 0.0000305175781250, 0.0007934570312500,
+0.0000000000000000, 0.0014343261718750, 0.0000000000000000, 0.0013122558593750,
+0.0000000000000000, 0.0013427734375000, 0.0000000000000000, 0.0016174316406250,
+0.0000000000000000, 0.0013427734375000, 0.0000000000000000, 0.0016479492187500,
+0.0000305175781250, 0.0015563964843750, -0.0000305175781250, 0.0014648437500000,
+0.0000305175781250, 0.0016174316406250, 0.0000000000000000, 0.0014343261718750,
+0.0000000000000000, 0.0013122558593750, 0.0000000000000000, 0.0012512207031250,
+0.0000000000000000, 0.0010375976562500, 0.0000000000000000, 0.0008850097656250,
+-0.0000305175781250, 0.0007324218750000, 0.0000000000000000, 0.0003356933593750,
+0.0000000000000000, 0.0003662109375000, 0.0000000000000000, 0.0000000000000000,
+-0.0000305175781250, -0.0004272460937500, 0.0000000000000000, -0.0003051757812500,
+0.0000000000000000, -0.0010070800781250, 0.0000000000000000, -0.0010070800781250,
+0.0000000000000000, -0.0011901855468750, 0.0000000000000000, -0.0017700195312500,
+0.0000000000000000, -0.0015869140625000, 0.0000305175781250, -0.0018005371093750,
+0.0000000000000000, -0.0019836425781250, 0.0000000000000000, -0.0017395019531250,
+0.0000305175781250, -0.0016479492187500, -0.0000305175781250, -0.0018005371093750,
+0.0000000000000000, -0.0010070800781250, -0.0000305175781250, -0.0011596679687500,
+0.0000000000000000, -0.0006713867187500, 0.0000305175781250, 0.0001220703125000,
+0.0000305175781250, -0.0002441406250000, 0.0000000000000000, 0.0008239746093750,
+0.0000000000000000, 0.0008544921875000, 0.0000305175781250, 0.0007934570312500,
+0.0000000000000000, 0.0013427734375000, 0.0000000000000000, 0.0012207031250000,
+0.0000000000000000, 0.0009765625000000, 0.0000305175781250, 0.0012512207031250,
+0.0000000000000000, 0.0007324218750000, 0.0000000000000000, 0.0000610351562500,
+0.0000305175781250, 0.0004577636718750, 0.0000000000000000, -0.0006713867187500,
+0.0000000000000000, -0.0007629394531250, 0.0000305175781250, -0.0006713867187500,
+0.0000000000000000, -0.0012207031250000, -0.0000305175781250, -0.0011291503906250,
+-0.0000305175781250, -0.0005493164062500, 0.0000000000000000, -0.0008544921875000,
+-0.0000610351562500, -0.0004272460937500, 0.0000000000000000, 0.0003662109375000,
+0.0000610351562500, -0.0000305175781250, 0.0000000000000000, 0.0006103515625000,
+0.0000305175781250, 0.0008544921875000, 0.0000915527343750, 0.0004882812500000,
+0.0000305175781250, 0.0004272460937500, -0.0000305175781250, 0.0006408691406250,
+0.0000000000000000, -0.0002746582031250, 0.0000305175781250, -0.0002441406250000,
+-0.0000305175781250, -0.0002136230468750, -0.0000305175781250, -0.0007324218750000,
+0.0000305175781250, -0.0006408691406250, 0.0000000000000000, -0.0000915527343750,
+0.0000305175781250, -0.0003051757812500, 0.0001220703125000, -0.0001525878906250,
+0.0000000000000000, 0.0004882812500000, -0.0000610351562500, 0.0002136230468750,
+0.0000610351562500, 0.0001220703125000, -0.0000610351562500, 0.0003662109375000,
+-0.0000915527343750, 0.0001220703125000, 0.0000000000000000, -0.0003662109375000,
+0.0000610351562500, -0.0001220703125000, -0.0000305175781250, -0.0003051757812500,
+0.0000305175781250, -0.0005187988281250, 0.0001525878906250, -0.0001831054687500,
+0.0000305175781250, 0.0000915527343750, -0.0000610351562500, -0.0001220703125000,
+0.0000610351562500, 0.0002441406250000, -0.0000610351562500, 0.0003967285156250,
+-0.0001525878906250, 0.0001220703125000, -0.0000610351562500, 0.0000000000000000,
+0.0000000000000000, 0.0002136230468750, -0.0000305175781250, -0.0001831054687500,
+0.0000610351562500, -0.0003051757812500, 0.0001220703125000, -0.0001220703125000,
+0.0000305175781250, -0.0000305175781250, -0.0000915527343750, -0.0000305175781250,
+-0.0000915527343750, 0.0001220703125000, -0.0000305175781250, 0.0001525878906250,
+-0.0001220703125000, 0.0000305175781250, -0.0000610351562500, -0.0000305175781250,
+0.0000915527343750, 0.0001220703125000, 0.0000305175781250, -0.0001220703125000,
+0.0000610351562500, -0.0002136230468750, 0.0001831054687500, -0.0000610351562500,
+0.0000305175781250, 0.0000000000000000, -0.0001525878906250, -0.0000915527343750,
+-0.0000305175781250, 0.0001525878906250, -0.0000610351562500, 0.0002136230468750,
+-0.0001525878906250, 0.0000305175781250, -0.0000305175781250, -0.0000305175781250,
+0.0001831054687500, 0.0000915527343750, 0.0000610351562500, -0.0000915527343750,
+0.0000915527343750, -0.0001831054687500, 0.0002441406250000, -0.0000305175781250,
+0.0000000000000000, 0.0000305175781250, -0.0003051757812500, -0.0000915527343750,
+-0.0000915527343750, 0.0000915527343750, -0.0002136230468750, 0.0001831054687500,
+-0.0004272460937500, 0.0000305175781250, -0.0001220703125000, 0.0000000000000000,
+0.0001220703125000, 0.0000915527343750, -0.0001220703125000, -0.0000915527343750,
+0.0003356933593750, -0.0001525878906250, 0.0005187988281250, -0.0000305175781250,
+0.0001525878906250, 0.0000000000000000, 0.0000305175781250, -0.0000915527343750,
+0.0003051757812500, 0.0000915527343750, -0.0004272460937500, 0.0001220703125000,
+-0.0004882812500000, 0.0000305175781250, -0.0003051757812500, 0.0000305175781250,
+-0.0005187988281250, 0.0000915527343750, -0.0005798339843750, -0.0000610351562500,
+0.0001220703125000, -0.0000915527343750, -0.0000305175781250, 0.0000000000000000,
+0.0000610351562500, 0.0000000000000000, 0.0006408691406250, -0.0000610351562500,
+0.0004882812500000, 0.0000000000000000, 0.0001831054687500, 0.0000610351562500,
+0.0004882812500000, 0.0000000000000000, 0.0000610351562500, 0.0000305175781250,
+-0.0005493164062500, 0.0000610351562500, -0.0001831054687500, -0.0000610351562500,
+-0.0006713867187500, -0.0000305175781250, -0.0009460449218750, 0.0000000000000000,
+-0.0005493164062500, -0.0000305175781250, -0.0003967285156250, -0.0000610351562500,
+-0.0006103515625000, 0.0000000000000000, 0.0003051757812500, 0.0000305175781250,
+0.0003051757812500, 0.0000000000000000, 0.0003967285156250, 0.0000305175781250,
+0.0009765625000000, 0.0000305175781250, 0.0008239746093750, 0.0000000000000000,
+0.0006408691406250, 0.0000000000000000, 0.0008850097656250, 0.0000305175781250,
+0.0004577636718750, -0.0000305175781250, -0.0001831054687500, -0.0000305175781250,
+0.0001831054687500, 0.0000000000000000, -0.0007629394531250, -0.0000305175781250,
+-0.0007629394531250, 0.0000000000000000, -0.0008544921875000, 0.0000305175781250,
+-0.0013732910156250, 0.0000305175781250, -0.0012207031250000, -0.0000305175781250,
+-0.0011901855468750, 0.0000305175781250, -0.0013427734375000, 0.0000305175781250,
+-0.0010375976562500, -0.0000305175781250, -0.0007934570312500, -0.0000305175781250,
+-0.0008850097656250, 0.0000305175781250, -0.0001525878906250, 0.0000000000000000,
+-0.0002441406250000, 0.0000000000000000, 0.0000610351562500, 0.0000000000000000,
+0.0006713867187500, 0.0000305175781250, 0.0005187988281250, -0.0000305175781250,
+0.0011596679687500, 0.0000000000000000, 0.0012512207031250, 0.0000305175781250,
+0.0013427734375000, 0.0000305175781250, 0.0016784667968750, -0.0000305175781250,
+0.0015869140625000, 0.0000000000000000, 0.0017395019531250, 0.0000000000000000,
+0.0018005371093750, 0.0000305175781250, 0.0018005371093750, 0.0000000000000000,
+0.0018615722656250, -0.0000305175781250, 0.0018005371093750, 0.0000000000000000,
+0.0017700195312500, 0.0000000000000000, 0.0017395019531250, 0.0000305175781250,
+0.0017395019531250, 0.0000305175781250, 0.0017089843750000, 0.0000610351562500,
+0.0017395019531250, 0.0000000000000000, 0.0018005371093750, 0.0000000000000000,
+0.0017089843750000, 0.0000000000000000, 0.0017395019531250, 0.0000000000000000,
+0.0018920898437500, 0.0000305175781250, 0.0018005371093750, -0.0000305175781250,
+0.0019531250000000, 0.0000000000000000, 0.0019531250000000, 0.0000000000000000,
+0.0019836425781250, -0.0000305175781250, 0.0021057128906250, 0.0000000000000000,
+0.0019836425781250, 0.0000000000000000, 0.0020751953125000, 0.0000000000000000,
+0.0021057128906250, 0.0000000000000000, 0.0019531250000000, 0.0000000000000000,
+0.0018920898437500, 0.0000000000000000, 0.0020141601562500, 0.0000000000000000,
+0.0012817382812500, 0.0000000000000000, 0.0016784667968750, 0.0000305175781250,
+0.0011596679687500, 0.0000305175781250, 0.0003967285156250, 0.0000305175781250,
+0.0010375976562500, -0.0000305175781250, -0.0005798339843750, 0.0000000000000000,
+-0.0002746582031250, 0.0000305175781250, -0.0005493164062500, -0.0000305175781250,
+-0.0017089843750000, 0.0000000000000000, -0.0010986328125000, 0.0000305175781250,
+-0.0018310546875000, 0.0000000000000000, -0.0021362304687500, 0.0000000000000000,
+-0.0016479492187500},
+{0.0001831054687500, 0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+-0.0007019042968750, 0.0000305175781250, -0.0004577636718750, -0.0000610351562500,
+-0.0004882812500000, -0.0000610351562500, -0.0007629394531250, 0.0000000000000000,
+-0.0003051757812500, 0.0000000000000000, 0.0001831054687500, -0.0000305175781250,
+-0.0001831054687500, 0.0000305175781250, 0.0005493164062500, 0.0000305175781250,
+0.0007324218750000, 0.0000305175781250, 0.0004272460937500, 0.0000000000000000,
+0.0005493164062500, 0.0000305175781250, 0.0006408691406250, 0.0000305175781250,
+-0.0002136230468750, -0.0000305175781250, -0.0001220703125000, -0.0000305175781250,
+-0.0003051757812500, 0.0000000000000000, -0.0010070800781250, -0.0000305175781250,
+-0.0007934570312500, 0.0000305175781250, -0.0008544921875000, 0.0000000000000000,
+-0.0010986328125000, 0.0000305175781250, -0.0006713867187500, 0.0000000000000000,
+-0.0003662109375000, 0.0000305175781250, -0.0006103515625000, 0.0000000000000000,
+0.0002746582031250, -0.0000305175781250, 0.0002746582031250, -0.0000305175781250,
+0.0004577636718750, -0.0000305175781250, 0.0010375976562500, 0.0000000000000000,
+0.0008544921875000, -0.0000305175781250, 0.0010375976562500, 0.0000000000000000,
+0.0011901855468750, 0.0000305175781250, 0.0010070800781250, 0.0000000000000000,
+0.0009155273437500, 0.0000000000000000, 0.0009460449218750, -0.0000305175781250,
+0.0004882812500000, -0.0000305175781250, 0.0004882812500000, 0.0000000000000000,
+0.0001831054687500, 0.0000000000000000, -0.0003051757812500, -0.0000305175781250,
+-0.0001525878906250, -0.0000305175781250, -0.0007934570312500, 0.0000000000000000,
+-0.0008239746093750, 0.0000305175781250, -0.0009765625000000, 0.0000000000000000,
+-0.0013122558593750, 0.0000000000000000, -0.0012512207031250, 0.0000000000000000,
+-0.0014648437500000, 0.0000305175781250, -0.0015258789062500, 0.0000305175781250,
+-0.0015258789062500, 0.0000305175781250, -0.0015869140625000, -0.0000305175781250,
+-0.0015869140625000, 0.0000000000000000, -0.0015869140625000, 0.0000305175781250,
+-0.0015869140625000, -0.0000305175781250, -0.0015869140625000, -0.0000305175781250,
+-0.0015563964843750, -0.0000305175781250, -0.0015258789062500, 0.0000000000000000,
+-0.0015869140625000, 0.0000000000000000, -0.0015563964843750, 0.0000000000000000,
+-0.0015563964843750, -0.0000305175781250, -0.0016174316406250, 0.0000000000000000,
+-0.0016174316406250, 0.0000000000000000, -0.0017089843750000, 0.0000305175781250,
+-0.0017089843750000, 0.0000000000000000, -0.0017089843750000, 0.0000000000000000,
+-0.0017700195312500, 0.0000000000000000, -0.0017089843750000, 0.0000000000000000,
+-0.0016784667968750, 0.0000000000000000, -0.0018005371093750, 0.0000000000000000,
+-0.0015563964843750, 0.0000000000000000, -0.0014648437500000, 0.0000305175781250,
+-0.0016784667968750, 0.0000000000000000, -0.0008544921875000, 0.0000000000000000,
+-0.0011901855468750, 0.0000305175781250, -0.0007019042968750, 0.0000000000000000,
+0.0000000000000000, 0.0000000000000000, -0.0005493164062500, 0.0000000000000000,
+0.0008239746093750, 0.0000000000000000, 0.0006713867187500, 0.0000000000000000,
+0.0007324218750000, 0.0000000000000000, 0.0016784667968750, 0.0000000000000000,
+0.0012512207031250, 0.0000000000000000, 0.0014648437500000, -0.0000305175781250,
+0.0018615722656250, 0.0000000000000000, 0.0012817382812500, -0.0000305175781250,
+0.0008544921875000, -0.0000305175781250, 0.0013122558593750, 0.0000000000000000,
+-0.0002441406250000, 0.0000305175781250, 0.0001220703125000, 0.0000305175781250,
+-0.0003051757812500, 0.0000915527343750, -0.0016174316406250, 0.0000610351562500,
+-0.0010070800781250, 0.0000000000000000, -0.0015869140625000, 0.0000000000000000,
+-0.0019226074218750, 0.0000000000000000, -0.0013732910156250, -0.0000305175781250,
+-0.0011596679687500, 0.0000000000000000, -0.0015258789062500, 0.0000305175781250,
+-0.0000305175781250, 0.0000305175781250, -0.0002441406250000, 0.0000000000000000,
+0.0000305175781250, 0.0000305175781250, 0.0012512207031250, 0.0000305175781250,
+0.0008544921875000, -0.0000305175781250, 0.0011901855468750, -0.0000305175781250,
+0.0015563964843750, 0.0000000000000000, 0.0009155273437500, -0.0000915527343750,
+0.0005187988281250, -0.0000305175781250, 0.0009460449218750, 0.0000305175781250,
+-0.0005187988281250, 0.0000610351562500, -0.0005187988281250, 0.0000305175781250,
+-0.0004882812500000, 0.0000305175781250, -0.0013732910156250, 0.0000915527343750,
+-0.0011901855468750, -0.0000305175781250, -0.0006103515625000, -0.0000305175781250,
+-0.0009765625000000, 0.0000000000000000, -0.0004882812500000, -0.0000915527343750,
+0.0003662109375000, -0.0000305175781250, -0.0000915527343750, 0.0000305175781250,
+0.0006713867187500, 0.0000000000000000, 0.0008850097656250, 0.0000305175781250,
+0.0004882812500000, 0.0001220703125000, 0.0005798339843750, 0.0000915527343750,
+0.0007934570312500, -0.0000305175781250, -0.0002746582031250, 0.0000305175781250,
+-0.0000915527343750, 0.0000000000000000, -0.0001831054687500, -0.0001831054687500,
+-0.0010375976562500, -0.0001220703125000, -0.0007629394531250, 0.0000000000000000,
+-0.0004272460937500, -0.0001220703125000, -0.0008239746093750, -0.0000305175781250,
+-0.0002441406250000, 0.0002136230468750, 0.0006103515625000, 0.0000915527343750,
+0.0000610351562500, 0.0000915527343750, 0.0007324218750000, 0.0002441406250000,
+0.0012207031250000, 0.0000305175781250, 0.0003967285156250, -0.0001831054687500,
+-0.0001525878906250, 0.0000000000000000, 0.0004272460937500, -0.0002441406250000,
+-0.0007629394531250, -0.0003967285156250, -0.0011596679687500, -0.0001220703125000,
+-0.0004577636718750, 0.0000610351562500, -0.0003662109375000, -0.0001220703125000,
+-0.0008239746093750, 0.0002746582031250, 0.0005187988281250, 0.0003967285156250,
+0.0006408691406250, 0.0001831054687500, 0.0002441406250000, 0.0000915527343750,
+0.0006408691406250, -0.0004272460937500, -0.0008850097656250, 0.0003967285156250,
+0.0006103515625000, 0.0001220703125000, 0.0004882812500000, 0.0000915527343750,
+0.0001525878906250, 0.0005798339843750, 0.0004577636718750, 0.0006103515625000,
+0.0007019042968750, -0.0002746582031250, -0.0004272460937500, 0.0000305175781250,
+-0.0005187988281250, -0.0001831054687500, -0.0001525878906250, -0.0010986328125000,
+-0.0005493164062500, -0.0007629394531250, -0.0006713867187500, -0.0002746582031250,
+0.0002746582031250, -0.0008850097656250, 0.0002136230468750, -0.0001831054687500,
+0.0000610351562500, 0.0009460449218750, 0.0005187988281250, 0.0003051757812500,
+0.0005187988281250, 0.0007324218750000, -0.0001831054687500, 0.0013732910156250,
+-0.0000305175781250, 0.0003967285156250, 0.0000000000000000, -0.0003967285156250,
+-0.0004577636718750, 0.0004272460937500, -0.0003967285156250, -0.0010375976562500,
+0.0000915527343750, -0.0015258789062500, -0.0000305175781250, -0.0006408691406250,
+0.0000000000000000, -0.0006103515625000, 0.0003356933593750, -0.0011901855468750,
+0.0002746582031250, 0.0006713867187500, -0.0000610351562500, 0.0006103515625000,
+0.0000915527343750, 0.0004272460937500, 0.0000305175781250, 0.0014953613281250,
+-0.0003051757812500, 0.0012817382812500, -0.0002441406250000, 0.0000000000000000,
+0.0000305175781250, 0.0006408691406250, -0.0000610351562500, 0.0000610351562500,
+-0.0000305175781250, -0.0014648437500000, 0.0002441406250000, -0.0007324218750000,
+0.0001831054687500, -0.0009460449218750, -0.0000305175781250, -0.0016174316406250,
+0.0000610351562500, -0.0007324218750000, 0.0000305175781250, 0.0000305175781250,
+-0.0001525878906250, -0.0007324218750000, -0.0001220703125000, 0.0008544921875000,
+0.0000000000000000, 0.0009460449218750, 0.0000000000000000, 0.0006408691406250,
+-0.0000305175781250, 0.0013427734375000, 0.0001220703125000, 0.0013122558593750,
+0.0001220703125000, 0.0001831054687500, -0.0000305175781250, 0.0007324218750000,
+0.0000305175781250, 0.0001220703125000, 0.0000305175781250, -0.0011596679687500,
+-0.0000610351562500, -0.0004272460937500, -0.0000915527343750, -0.0012817382812500,
+0.0000305175781250, -0.0016174316406250, 0.0000305175781250, -0.0011596679687500,
+0.0000305175781250, -0.0012512207031250, 0.0000610351562500, -0.0014953613281250,
+0.0000610351562500, -0.0002441406250000, -0.0000305175781250, -0.0005187988281250,
+0.0000000000000000, -0.0001525878906250, 0.0000000000000000, 0.0009460449218750,
+0.0000000000000000, 0.0003356933593750, 0.0000000000000000, 0.0013122558593750,
+0.0000305175781250, 0.0014953613281250, 0.0000305175781250, 0.0012207031250000,
+0.0000000000000000, 0.0015563964843750, 0.0000305175781250, 0.0016479492187500,
+0.0000305175781250, 0.0007934570312500, -0.0000305175781250, 0.0011901855468750,
+-0.0000305175781250, 0.0006408691406250, 0.0000305175781250, -0.0003356933593750,
+0.0000000000000000, 0.0003662109375000, -0.0000305175781250, -0.0010681152343750,
+0.0000000000000000, -0.0009460449218750, 0.0000305175781250, -0.0010986328125000,
+0.0000000000000000, -0.0020141601562500, 0.0000305175781250, -0.0015563964843750,
+0.0000305175781250, -0.0019531250000000, 0.0000305175781250, -0.0021972656250000,
+0.0000000000000000, -0.0018920898437500, -0.0000305175781250, -0.0018920898437500,
+0.0000305175781250, -0.0021362304687500, 0.0000305175781250, -0.0013122558593750,
+-0.0000305175781250, -0.0016784667968750, 0.0000610351562500, -0.0013732910156250,
+0.0000305175781250, -0.0006408691406250, 0.0000000000000000, -0.0011596679687500,
+0.0000000000000000, -0.0001831054687500, -0.0000610351562500, -0.0003967285156250,
+0.0000000000000000, -0.0002136230468750, 0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, -0.0001220703125000, 0.0000000000000000, 0.0005798339843750,
+-0.0000305175781250, 0.0003967285156250, -0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, 0.0007934570312500, 0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, 0.0008239746093750, -0.0000305175781250, 0.0007324218750000,
+0.0000305175781250, 0.0007019042968750, -0.0000305175781250, 0.0008544921875000,
+0.0000000000000000, 0.0006408691406250, 0.0000000000000000, 0.0006408691406250,
+0.0000305175781250, 0.0004882812500000, 0.0000000000000000, 0.0002746582031250,
+0.0000000000000000, 0.0001525878906250, 0.0000000000000000, -0.0000305175781250,
+-0.0000305175781250, -0.0003051757812500, 0.0000000000000000, -0.0003967285156250,
+0.0000000000000000, -0.0006713867187500, -0.0000305175781250, -0.0010375976562500,
+0.0000305175781250, -0.0009460449218750, 0.0000000000000000, -0.0014343261718750,
+-0.0000305175781250, -0.0014343261718750, 0.0000000000000000, -0.0014648437500000,
+0.0000000000000000, -0.0018310546875000, 0.0000305175781250, -0.0017700195312500,
+0.0000000000000000, -0.0015869140625000, 0.0000000000000000, -0.0017700195312500,
+0.0000305175781250, -0.0014343261718750, 0.0000000000000000, -0.0010070800781250,
+0.0000000000000000, -0.0012512207031250, -0.0000305175781250, -0.0003662109375000,
+-0.0000305175781250, -0.0004272460937500, 0.0000000000000000, -0.0000915527343750,
+0.0000000000000000, 0.0006713867187500, 0.0000305175781250, 0.0004272460937500,
+0.0000000000000000, 0.0010986328125000, 0.0000000000000000, 0.0011901855468750,
+0.0000305175781250, 0.0009460449218750, 0.0000000000000000, 0.0011291503906250,
+0.0000305175781250, 0.0012207031250000, 0.0000305175781250, 0.0004577636718750,
+0.0000305175781250, 0.0006408691406250, -0.0000305175781250, 0.0001525878906250,
+0.0000305175781250, -0.0006408691406250, 0.0000610351562500, -0.0002136230468750,
+-0.0000305175781250},
+{-0.0000610351562500, 0.0013122558593750, -0.0000610351562500, 0.0004577636718750,
+0.0000000000000000, 0.0007629394531250, 0.0000305175781250, 0.0003051757812500,
+0.0000000000000000, -0.0004882812500000, 0.0000305175781250, 0.0000000000000000,
+0.0000305175781250, -0.0010681152343750, -0.0000305175781250, -0.0011291503906250,
+-0.0000305175781250, -0.0009460449218750, 0.0000305175781250, -0.0014648437500000,
+0.0000000000000000, -0.0014038085937500, -0.0000305175781250, -0.0008239746093750,
+0.0000610351562500, -0.0011901855468750, 0.0000305175781250, -0.0007019042968750,
+0.0000000000000000, 0.0000305175781250, 0.0000000000000000, -0.0005187988281250,
+0.0000305175781250, 0.0007629394531250, -0.0000305175781250, 0.0006103515625000,
+-0.0000305175781250, 0.0007629394531250, 0.0000000000000000, 0.0016174316406250,
+0.0000000000000000, 0.0012207031250000, 0.0000305175781250, 0.0015869140625000,
+0.0000305175781250, 0.0018005371093750, 0.0000610351562500, 0.0014953613281250,
+0.0000305175781250, 0.0014953613281250, -0.0000305175781250, 0.0016784667968750,
+0.0000305175781250, 0.0008239746093750, -0.0000610351562500, 0.0011901855468750,
+0.0000305175781250, 0.0008544921875000, 0.0000305175781250, 0.0000610351562500,
+-0.0000305175781250, 0.0006408691406250, 0.0000305175781250, -0.0003967285156250,
+0.0000000000000000, -0.0001831054687500, 0.0000000000000000, -0.0003051757812500,
+0.0000305175781250, -0.0009155273437500, -0.0000305175781250, -0.0004272460937500,
+0.0000000000000000, -0.0010986328125000, 0.0000000000000000, -0.0009460449218750,
+0.0000305175781250, -0.0010070800781250, 0.0000305175781250, -0.0013427734375000,
+0.0000000000000000, -0.0010070800781250, 0.0000000000000000, -0.0014038085937500,
+0.0000000000000000, -0.0012207031250000, 0.0000000000000000, -0.0011901855468750,
+0.0000000000000000, -0.0014038085937500, 0.0000000000000000, -0.0011596679687500,
+0.0000000000000000, -0.0010681152343750, 0.0000305175781250, -0.0009765625000000,
+0.0000000000000000, -0.0007629394531250, 0.0000000000000000, -0.0005798339843750,
+0.0000000000000000, -0.0003967285156250, 0.0000305175781250, -0.0000305175781250,
+0.0000000000000000, 0.0000000000000000, 0.0000000000000000, 0.0003356933593750,
+0.0000000000000000, 0.0007324218750000, 0.0000000000000000, 0.0006103515625000,
+-0.0000305175781250, 0.0012207031250000, 0.0000000000000000, 0.0012207031250000,
+0.0000305175781250, 0.0012512207031250, 0.0000305175781250, 0.0017089843750000,
+0.0000000000000000, 0.0016479492187500, 0.0000000000000000, 0.0014648437500000,
+0.0000305175781250, 0.0016784667968750, 0.0000000000000000, 0.0013427734375000,
+0.0000000000000000, 0.0009460449218750, -0.0000305175781250, 0.0011901855468750,
+-0.0000305175781250, 0.0003051757812500, 0.0000000000000000, 0.0003662109375000,
+0.0000305175781250, 0.0000305175781250, 0.0000305175781250, -0.0007019042968750,
+0.0000000000000000, -0.0004272460937500, 0.0000000000000000, -0.0010070800781250,
+-0.0000305175781250, -0.0011901855468750, 0.0000000000000000, -0.0009155273437500,
+0.0000000000000000, -0.0010681152343750, 0.0000000000000000, -0.0010986328125000,
+0.0000000000000000, -0.0003051757812500, 0.0000305175781250, -0.0004882812500000,
+0.0000000000000000, -0.0001220703125000, 0.0000000000000000, 0.0006713867187500,
+0.0000305175781250, 0.0003356933593750, 0.0000305175781250, 0.0008544921875000,
+-0.0000610351562500, 0.0010681152343750, 0.0000000000000000, 0.0006713867187500,
+0.0000000000000000, 0.0005187988281250, 0.0000000000000000, 0.0007629394531250,
+-0.0000305175781250, -0.0002746582031250, 0.0000305175781250, -0.0002136230468750,
+0.0000305175781250, -0.0003051757812500, 0.0000305175781250, -0.0010375976562500,
+0.0000305175781250, -0.0008544921875000, 0.0000305175781250, -0.0005187988281250,
+-0.0000305175781250, -0.0008544921875000, 0.0000000000000000, -0.0003967285156250,
+0.0000000000000000, 0.0002746582031250, -0.0000305175781250, -0.0000915527343750,
+-0.0000610351562500, 0.0005493164062500, 0.0000915527343750, 0.0007629394531250,
+0.0000915527343750, 0.0003662109375000, 0.0000000000000000, 0.0002136230468750,
+0.0000000000000000, 0.0004882812500000, 0.0000610351562500, -0.0003662109375000,
+-0.0000305175781250, -0.0004272460937500, -0.0000305175781250, -0.0002441406250000,
+0.0000000000000000, -0.0004882812500000, 0.0000000000000000, -0.0005798339843750,
+-0.0000915527343750, 0.0001220703125000, 0.0000305175781250, 0.0000305175781250,
+0.0000610351562500, 0.0000305175781250, 0.0000000000000000, 0.0005187988281250,
+0.0000915527343750, 0.0004272460937500, 0.0000915527343750, -0.0000305175781250,
+-0.0000610351562500, 0.0001525878906250, -0.0000610351562500, 0.0000000000000000,
+0.0000000000000000, -0.0005187988281250, -0.0000915527343750, -0.0003967285156250,
+-0.0000915527343750, -0.0001220703125000, 0.0001220703125000, -0.0003356933593750,
+0.0000610351562500, -0.0001525878906250, 0.0000000000000000, 0.0003356933593750,
+0.0000915527343750, 0.0001220703125000, 0.0000915527343750, 0.0000610351562500,
+-0.0000610351562500, 0.0002441406250000, -0.0000915527343750, 0.0000915527343750,
+-0.0000305175781250, -0.0003051757812500, -0.0000610351562500, -0.0003051757812500,
+-0.0000915527343750, 0.0000000000000000, 0.0000610351562500, -0.0001831054687500,
+0.0000915527343750, -0.0001525878906250, -0.0000305175781250, 0.0001831054687500,
+0.0000305175781250, 0.0000610351562500, 0.0001220703125000, 0.0000000000000000,
+-0.0000915527343750, 0.0001831054687500, -0.0001220703125000, 0.0000000000000000,
+-0.0000305175781250, -0.0002136230468750, -0.0000915527343750, -0.0000915527343750,
+-0.0001220703125000, 0.0000000000000000, 0.0000915527343750, -0.0001831054687500,
+0.0001525878906250, -0.0000915527343750, -0.0000305175781250, 0.0002136230468750,
+-0.0000305175781250, 0.0000915527343750, 0.0000915527343750, 0.0000305175781250,
+-0.0002441406250000, 0.0001525878906250, -0.0003051757812500, 0.0000305175781250,
+-0.0000915527343750, -0.0001831054687500, -0.0002136230468750, -0.0000915527343750,
+-0.0002746582031250, -0.0000305175781250, 0.0002441406250000, -0.0001525878906250,
+0.0002441406250000, -0.0000610351562500, 0.0000305175781250, 0.0001220703125000,
+0.0003051757812500, 0.0000915527343750, 0.0003662109375000, 0.0000305175781250,
+-0.0003051757812500, 0.0000915527343750, -0.0002136230468750, 0.0000610351562500,
+-0.0001525878906250, -0.0001220703125000, -0.0006713867187500, -0.0000610351562500,
+-0.0005798339843750, 0.0000000000000000, -0.0000610351562500, -0.0001220703125000,
+-0.0002746582031250, -0.0000305175781250, -0.0000610351562500, 0.0000915527343750,
+0.0005493164062500, 0.0000000000000000, 0.0003051757812500, 0.0000000000000000,
+0.0002136230468750, 0.0001220703125000, 0.0005493164062500, 0.0000000000000000,
+0.0001525878906250, -0.0000610351562500, -0.0003967285156250, 0.0000000000000000,
+-0.0000305175781250, 0.0000000000000000, -0.0005493164062500, -0.0000915527343750,
+-0.0007629394531250, -0.0000305175781250, -0.0003967285156250, 0.0000610351562500,
+-0.0003051757812500, 0.0000305175781250, -0.0005187988281250, 0.0000305175781250,
+0.0003051757812500, 0.0000610351562500, 0.0002746582031250, 0.0000305175781250,
+0.0002746582031250, -0.0000305175781250, 0.0007629394531250, 0.0000000000000000,
+0.0006408691406250, -0.0000305175781250, 0.0002441406250000, -0.0000610351562500,
+0.0003967285156250, -0.0000305175781250, 0.0000610351562500, 0.0000305175781250,
+-0.0004882812500000, 0.0000000000000000, -0.0002441406250000, 0.0000000000000000,
+-0.0006713867187500, 0.0000610351562500, -0.0008544921875000, 0.0000000000000000,
+-0.0005798339843750, 0.0000000000000000, -0.0005493164062500, 0.0000305175781250,
+-0.0006713867187500, -0.0000305175781250, -0.0000915527343750, -0.0000610351562500,
+-0.0001525878906250, -0.0000610351562500, -0.0000305175781250, -0.0000305175781250,
+0.0004577636718750, 0.0000000000000000, 0.0003051757812500, 0.0000305175781250,
+0.0005187988281250, 0.0000305175781250, 0.0005798339843750, 0.0000000000000000,
+0.0004882812500000, 0.0000000000000000, 0.0004272460937500, 0.0000000000000000,
+0.0004577636718750, 0.0000000000000000, 0.0002136230468750, -0.0000305175781250,
+0.0002136230468750, 0.0000000000000000, 0.0000305175781250, -0.0000305175781250,
+-0.0003051757812500, 0.0000000000000000, -0.0000915527343750, 0.0000305175781250,
+-0.0005187988281250, 0.0000000000000000, -0.0005798339843750, 0.0000305175781250,
+-0.0006103515625000, 0.0000000000000000, -0.0008239746093750, 0.0000000000000000,
+-0.0007629394531250, 0.0000000000000000, -0.0008544921875000, 0.0000305175781250,
+-0.0008850097656250, 0.0000305175781250, -0.0008544921875000, 0.0000305175781250,
+-0.0008850097656250, 0.0000000000000000, -0.0008239746093750, 0.0000305175781250,
+-0.0007629394531250, 0.0000305175781250, -0.0006713867187500, 0.0000000000000000,
+-0.0006713867187500, 0.0000000000000000, -0.0005798339843750, 0.0000305175781250,
+-0.0005493164062500, -0.0000305175781250, -0.0006408691406250, 0.0000000000000000,
+-0.0005187988281250, 0.0000000000000000, -0.0005493164062500, -0.0000305175781250,
+-0.0006103515625000, -0.0000305175781250, -0.0006103515625000, -0.0000305175781250,
+-0.0008544921875000, 0.0000000000000000, -0.0007324218750000, 0.0000000000000000,
+-0.0007934570312500, 0.0000000000000000, -0.0010375976562500, 0.0000305175781250,
+-0.0009155273437500, 0.0000000000000000, -0.0012207031250000, -0.0000305175781250,
+-0.0011901855468750, 0.0000000000000000, -0.0010986328125000, 0.0000000000000000,
+-0.0011901855468750, 0.0000000000000000, -0.0012512207031250, 0.0000000000000000,
+-0.0007934570312500, 0.0000000000000000, -0.0010681152343750, 0.0000000000000000,
+-0.0007324218750000, 0.0000000000000000, -0.0002746582031250, 0.0000000000000000,
+-0.0006713867187500, 0.0000000000000000, 0.0003356933593750, -0.0000610351562500,
+0.0001831054687500, 0.0000305175781250, 0.0001831054687500, 0.0000000000000000,
+0.0008850097656250, 0.0000000000000000, 0.0006103515625000, 0.0000000000000000,
+0.0006408691406250, 0.0000305175781250, 0.0008850097656250, 0.0000000000000000,
+0.0004882812500000, -0.0000305175781250, 0.0001831054687500, 0.0000000000000000,
+0.0005187988281250, 0.0000000000000000, -0.0005493164062500, 0.0000305175781250,
+-0.0003662109375000, 0.0000305175781250, -0.0004577636718750, 0.0000305175781250,
+-0.0013427734375000, 0.0000000000000000, -0.0010070800781250, 0.0000000000000000,
+-0.0008850097656250, 0.0000305175781250, -0.0013122558593750, 0.0000305175781250,
+-0.0006713867187500, 0.0000000000000000, -0.0000610351562500, 0.0000000000000000,
+-0.0006408691406250, 0.0000000000000000, 0.0007019042968750, 0.0000000000000000,
+0.0007019042968750, 0.0000000000000000, 0.0006408691406250, 0.0000305175781250,
+0.0013427734375000, 0.0000305175781250, 0.0011901855468750, 0.0000000000000000,
+0.0006713867187500, 0.0000305175781250, 0.0010375976562500, 0.0000000000000000,
+0.0004577636718750, -0.0000610351562500, -0.0004272460937500, -0.0000610351562500,
+0.0001220703125000, 0.0000305175781250, -0.0009155273437500, -0.0000305175781250,
+-0.0011596679687500, 0.0000000000000000, -0.0007629394531250, 0.0001220703125000,
+-0.0010070800781250, 0.0000915527343750, -0.0011596679687500, 0.0000000000000000,
+-0.0000305175781250, 0.0000610351562500, -0.0001831054687500, 0.0000000000000000,
+-0.0000305175781250, -0.0001220703125000, 0.0007934570312500, -0.0000610351562500,
+0.0005798339843750, 0.0000000000000000, 0.0005798339843750, -0.0000915527343750,
+0.0009155273437500},
+{0.0007629394531250, 0.0000000000000000, 0.0007324218750000, 0.0000000000000000,
+0.0009155273437500, 0.0000000000000000, 0.0013122558593750, -0.0000305175781250,
+0.0012207031250000, 0.0000000000000000, 0.0015563964843750, 0.0000000000000000,
+0.0015869140625000, 0.0000305175781250, 0.0015869140625000, -0.0000305175781250,
+0.0016784667968750, 0.0000000000000000, 0.0015563964843750, 0.0000000000000000,
+0.0014953613281250, 0.0000305175781250, 0.0014648437500000, 0.0000000000000000,
+0.0013732910156250, 0.0000000000000000, 0.0012512207031250, 0.0000000000000000,
+0.0011901855468750, 0.0000305175781250, 0.0009765625000000, 0.0000305175781250,
+0.0007629394531250, 0.0000000000000000, 0.0007019042968750, 0.0000305175781250,
+0.0005798339843750, -0.0000305175781250, 0.0004272460937500, 0.0000305175781250,
+0.0007324218750000, 0.0000000000000000, 0.0005493164062500, 0.0000000000000000,
+0.0006103515625000, 0.0000000000000000, 0.0007629394531250, 0.0000000000000000,
+0.0006103515625000, -0.0000305175781250, 0.0012512207031250, 0.0000305175781250,
+0.0010070800781250, 0.0000000000000000, 0.0012512207031250, -0.0000305175781250,
+0.0018310546875000, -0.0000305175781250, 0.0013427734375000, 0.0000000000000000,
+0.0022277832031250, 0.0000000000000000, 0.0021667480468750, 0.0000610351562500,
+0.0021972656250000, 0.0000305175781250, 0.0027465820312500, 0.0000000000000000,
+0.0025024414062500, -0.0000305175781250, 0.0023803710937500, -0.0000305175781250,
+0.0026550292968750, 0.0000000000000000, 0.0022277832031250, 0.0000000000000000,
+0.0018615722656250, -0.0000305175781250, 0.0022888183593750, 0.0000000000000000,
+0.0008850097656250, 0.0000305175781250, 0.0012512207031250, -0.0000305175781250,
+0.0009155273437500, 0.0000000000000000, -0.0001525878906250, 0.0000000000000000,
+0.0004577636718750, 0.0000000000000000, -0.0004272460937500, -0.0000305175781250,
+-0.0006713867187500, 0.0000000000000000, -0.0003967285156250, 0.0000305175781250,
+-0.0006713867187500, 0.0000305175781250, -0.0007324218750000, 0.0000000000000000,
+-0.0000305175781250, 0.0000610351562500, -0.0001831054687500, 0.0000000000000000,
+-0.0000305175781250, -0.0000305175781250, 0.0006408691406250, -0.0000305175781250,
+0.0003356933593750, 0.0000000000000000, 0.0003967285156250, -0.0000305175781250,
+0.0006713867187500, 0.0000000000000000, 0.0002746582031250, 0.0000610351562500,
+-0.0001220703125000, 0.0000305175781250, 0.0002136230468750, 0.0000305175781250,
+-0.0006713867187500, 0.0000305175781250, -0.0007324218750000, 0.0000000000000000,
+-0.0006713867187500, -0.0000305175781250, -0.0010681152343750, -0.0000305175781250,
+-0.0010070800781250, 0.0000000000000000, -0.0004577636718750, 0.0000000000000000,
+-0.0007019042968750, 0.0000000000000000, -0.0002441406250000, 0.0000610351562500,
+0.0005187988281250, 0.0000305175781250, 0.0000915527343750, 0.0000305175781250,
+0.0007934570312500, 0.0000610351562500, 0.0010986328125000, 0.0000000000000000,
+0.0006408691406250, -0.0001220703125000, 0.0005187988281250, -0.0000610351562500,
+0.0007629394531250, -0.0000610351562500, -0.0003356933593750, -0.0000610351562500,
+-0.0002746582031250, -0.0000305175781250, -0.0003356933593750, 0.0001220703125000,
+-0.0009765625000000, 0.0000305175781250, -0.0007629394531250, 0.0000915527343750,
+-0.0005187988281250, 0.0001525878906250, -0.0007934570312500, 0.0000000000000000,
+-0.0003967285156250, -0.0001525878906250, 0.0002136230468750, -0.0000305175781250,
+-0.0002441406250000, -0.0000915527343750, 0.0006408691406250, -0.0001831054687500,
+0.0008544921875000, -0.0000305175781250, 0.0004272460937500, 0.0000610351562500,
+0.0005798339843750, -0.0000305175781250, 0.0007629394531250, 0.0001525878906250,
+-0.0004577636718750, 0.0002136230468750, -0.0003051757812500, 0.0000305175781250,
+-0.0003662109375000, -0.0000305175781250, -0.0012817382812500, 0.0000915527343750,
+-0.0010070800781250, -0.0001831054687500, -0.0001220703125000, -0.0002746582031250,
+-0.0006408691406250, -0.0001525878906250, -0.0001220703125000, -0.0001525878906250,
+0.0011596679687500, -0.0002441406250000, 0.0005798339843750, 0.0001831054687500,
+0.0003356933593750, 0.0001831054687500, 0.0010070800781250, 0.0001220703125000,
+0.0002136230468750, 0.0003356933593750, -0.0009765625000000, 0.0003662109375000,
+-0.0002136230468750, -0.0002136230468750, -0.0007324218750000, -0.0000915527343750,
+-0.0012817382812500, -0.0000915527343750, -0.0006408691406250, -0.0006408691406250,
+0.0005187988281250, 0.0003051757812500, -0.0000610351562500, 0.0000610351562500,
+-0.0005798339843750, 0.0005493164062500, -0.0011901855468750, 0.0000000000000000,
+-0.0003967285156250, -0.0007934570312500, 0.0004882812500000, -0.0003051757812500,
+-0.0000610351562500, -0.0005187988281250, 0.0004272460937500, -0.0009765625000000,
+0.0009460449218750, -0.0002746582031250, 0.0002746582031250, 0.0003967285156250,
+-0.0002746582031250, -0.0002136230468750, 0.0001831054687500, 0.0008544921875000,
+-0.0004577636718750, 0.0012512207031250, -0.0008850097656250, 0.0004577636718750,
+-0.0003356933593750, 0.0003356933593750, 0.0000610351562500, 0.0008850097656250,
+-0.0003356933593750, -0.0009155273437500, 0.0003356933593750, -0.0010375976562500,
+0.0006103515625000, -0.0005187988281250, 0.0001831054687500, -0.0012817382812500,
+0.0000305175781250, -0.0014343261718750, 0.0003051757812500, 0.0003967285156250,
+-0.0002746582031250, 0.0000000000000000, -0.0004577636718750, 0.0000915527343750,
+-0.0001525878906250, 0.0015258789062500, -0.0000610351562500, 0.0011596679687500,
+-0.0002441406250000, 0.0002136230468750, 0.0002136230468750, 0.0009460449218750,
+0.0003051757812500, 0.0002136230468750, 0.0000610351562500, -0.0012207031250000,
+0.0000610351562500, -0.0004882812500000, 0.0002136230468750, -0.0010070800781250,
+-0.0001525878906250, -0.0016784667968750, -0.0002441406250000, -0.0007324218750000,
+-0.0000915527343750, 0.0000610351562500, -0.0000610351562500, -0.0007324218750000,
+-0.0001831054687500, 0.0007324218750000, 0.0001525878906250, 0.0010375976562500,
+0.0001831054687500, 0.0005493164062500, 0.0000610351562500, 0.0008850097656250,
+0.0000915527343750, 0.0010375976562500, 0.0001831054687500, -0.0001220703125000,
+-0.0000610351562500, 0.0001525878906250, -0.0001220703125000, -0.0000610351562500,
+-0.0000305175781250, -0.0010070800781250, -0.0000305175781250, -0.0006408691406250,
+-0.0000915527343750, -0.0006713867187500, 0.0000610351562500, -0.0010070800781250,
+0.0000915527343750, -0.0005493164062500, 0.0000305175781250, 0.0000000000000000,
+0.0000000000000000, -0.0004577636718750, 0.0000305175781250, 0.0004577636718750,
+0.0000000000000000, 0.0005798339843750, -0.0000915527343750, 0.0003051757812500,
+-0.0000305175781250, 0.0006103515625000, 0.0000000000000000, 0.0006713867187500,
+-0.0000305175781250, 0.0000305175781250, 0.0000305175781250, 0.0002441406250000,
+0.0000915527343750, 0.0000305175781250, 0.0000305175781250, -0.0005187988281250,
+0.0000000000000000, -0.0002136230468750, 0.0000610351562500, -0.0005798339843750,
+-0.0000305175781250, -0.0007629394531250, -0.0000305175781250, -0.0003356933593750,
+0.0000000000000000, -0.0002746582031250, 0.0000305175781250, -0.0005798339843750,
+0.0000000000000000, 0.0007629394531250, 0.0000000000000000, 0.0003356933593750,
+0.0000610351562500, 0.0008239746093750, 0.0000610351562500, 0.0020141601562500,
+0.0000000000000000, 0.0012512207031250, 0.0000000000000000, 0.0024719238281250,
+-0.0000305175781250, 0.0026855468750000, -0.0000305175781250, 0.0022888183593750,
+-0.0000305175781250, 0.0027465820312500, 0.0000305175781250, 0.0028991699218750,
+0.0000000000000000, 0.0015258789062500, 0.0000000000000000, 0.0023803710937500,
+0.0000305175781250, 0.0014343261718750, 0.0000000000000000, 0.0000610351562500,
+0.0000305175781250, 0.0013427734375000, 0.0000000000000000, -0.0018920898437500,
+0.0000305175781250, -0.0008850097656250, 0.0000610351562500, -0.0017089843750000,
+0.0000610351562500, -0.0042724609375000, 0.0000000000000000, -0.0024719238281250,
+0.0000305175781250, -0.0053405761718750, -0.0000305175781250, -0.0049133300781250,
+0.0000000000000000, -0.0052185058593750, 0.0000000000000000, -0.0069274902343750,
+0.0000000000000000, -0.0056762695312500, -0.0000610351562500, -0.0072631835937500,
+0.0000000000000000, -0.0069274902343750, 0.0000000000000000, -0.0068664550781250,
+0.0000000000000000, -0.0076904296875000, 0.0000000000000000, -0.0068054199218750,
+0.0000000000000000, -0.0074462890625000, 0.0000000000000000, -0.0072326660156250,
+0.0000000000000000, -0.0068359375000000, 0.0000000000000000, -0.0070800781250000,
+0.0000000000000000, -0.0067138671875000, 0.0000000000000000, -0.0060729980468750,
+0.0000305175781250, -0.0058593750000000, 0.0000610351562500, -0.0052185058593750,
+0.0000305175781250, -0.0043640136718750, 0.0000305175781250, -0.0040588378906250,
+0.0000000000000000, -0.0027770996093750, -0.0000305175781250, -0.0027160644531250,
+0.0000000000000000, -0.0017700195312500, 0.0000000000000000, -0.0004577636718750,
+0.0000000000000000, -0.0007934570312500, -0.0000305175781250, 0.0012207031250000,
+0.0000305175781250, 0.0011901855468750, 0.0000000000000000, 0.0015563964843750,
+0.0000000000000000, 0.0029907226562500, 0.0000000000000000, 0.0026855468750000,
+0.0000305175781250, 0.0032958984375000, 0.0000000000000000, 0.0036315917968750,
+0.0000000000000000, 0.0031738281250000, 0.0000000000000000, 0.0031433105468750,
+0.0000000000000000, 0.0033264160156250, -0.0000305175781250, 0.0019226074218750,
+0.0000000000000000, 0.0021667480468750, 0.0000000000000000, 0.0014648437500000,
+0.0000000000000000, 0.0000915527343750, 0.0000000000000000, 0.0006713867187500,
+0.0000305175781250, -0.0007934570312500, 0.0000305175781250, -0.0010070800781250,
+0.0000000000000000, -0.0008239746093750, 0.0000000000000000, -0.0014343261718750,
+-0.0000305175781250, -0.0013427734375000, -0.0000305175781250, -0.0006103515625000,
+0.0000305175781250, -0.0008850097656250, 0.0000305175781250, -0.0004272460937500,
+0.0000000000000000, 0.0004577636718750, 0.0000305175781250, 0.0000915527343750,
+0.0000000000000000, 0.0006713867187500, -0.0000305175781250, 0.0009765625000000,
+0.0000000000000000, 0.0005187988281250, -0.0000305175781250, 0.0003967285156250,
+-0.0000305175781250, 0.0005798339843750, -0.0000610351562500, -0.0003662109375000,
+0.0000305175781250, -0.0003967285156250, 0.0000000000000000, -0.0003662109375000,
+0.0000000000000000, -0.0009155273437500, 0.0000610351562500, -0.0007934570312500,
+0.0000305175781250, -0.0002746582031250, 0.0000000000000000, -0.0005187988281250,
+0.0000000000000000, -0.0001220703125000, 0.0000000000000000, 0.0006408691406250,
+-0.0000305175781250, 0.0002746582031250, -0.0000915527343750, 0.0004577636718750,
+0.0000000000000000, 0.0008239746093750, 0.0000000000000000, 0.0003356933593750,
+-0.0000305175781250, -0.0002136230468750, 0.0000305175781250, 0.0001525878906250,
+0.0000610351562500, -0.0004577636718750, 0.0000000000000000, -0.0007324218750000,
+0.0000305175781250, -0.0003356933593750, 0.0000305175781250, -0.0001831054687500,
+-0.0000610351562500, -0.0005187988281250, -0.0000915527343750, 0.0002746582031250,
+0.0000305175781250, 0.0003967285156250, -0.0000305175781250, 0.0001220703125000,
+-0.0000305175781250, 0.0003051757812500, 0.0001525878906250, 0.0004577636718750,
+0.0000915527343750, -0.0002746582031250, -0.0000305175781250, -0.0002746582031250,
+0.0000610351562500, -0.0001525878906250, 0.0000000000000000, -0.0004882812500000,
+-0.0001525878906250},
+{0.0000305175781250, 0.0169067382812500, -0.0000305175781250, 0.0136718750000000,
+0.0000305175781250, 0.0174255371093750, 0.0000000000000000, 0.0166015625000000,
+-0.0000305175781250, 0.0163574218750000, -0.0000305175781250, 0.0181884765625000,
+0.0000000000000000, 0.0160522460937500, 0.0000305175781250, 0.0176086425781250,
+0.0000000000000000, 0.0171203613281250, -0.0000305175781250, 0.0161743164062500,
+0.0000000000000000, 0.0166625976562500, 0.0000000000000000, 0.0159606933593750,
+0.0000305175781250, 0.0142211914062500, 0.0000305175781250, 0.0137329101562500,
+0.0000000000000000, 0.0121459960937500, 0.0000000000000000, 0.0101623535156250,
+0.0000000000000000, 0.0096130371093750, -0.0000305175781250, 0.0065612792968750,
+0.0000000000000000, 0.0064697265625000, 0.0000000000000000, 0.0043640136718750,
+0.0000000000000000, 0.0014038085937500, -0.0000305175781250, 0.0021057128906250,
+0.0000000000000000, -0.0022277832031250, 0.0000305175781250, -0.0021057128906250,
+0.0000000000000000, -0.0030517578125000, 0.0000000000000000, -0.0062255859375000,
+0.0000000000000000, -0.0054321289062500, 0.0000000000000000, -0.0068664550781250,
+-0.0000305175781250, -0.0075073242187500, -0.0000305175781250, -0.0065917968750000,
+0.0000000000000000, -0.0065612792968750, 0.0000000000000000, -0.0069885253906250,
+0.0000305175781250, -0.0042114257812500, 0.0000000000000000, -0.0047302246093750,
+0.0000000000000000, -0.0033264160156250, 0.0000000000000000, -0.0005798339843750,
+-0.0000305175781250, -0.0017089843750000, 0.0000305175781250, 0.0013427734375000,
+0.0000305175781250, 0.0016174316406250, 0.0000305175781250, 0.0014038085937500,
+-0.0000305175781250, 0.0026855468750000, 0.0000305175781250, 0.0024719238281250,
+0.0000000000000000, 0.0015563964843750, -0.0000305175781250, 0.0020141601562500,
+0.0000305175781250, 0.0011596679687500, 0.0000000000000000, -0.0002441406250000,
+-0.0000610351562500, 0.0004272460937500, -0.0000610351562500, -0.0007934570312500,
+0.0000000000000000, -0.0011596679687500, -0.0000305175781250, -0.0006713867187500,
+0.0000305175781250, -0.0007629394531250, 0.0000610351562500, -0.0009765625000000,
+0.0000610351562500, 0.0001220703125000, -0.0000305175781250, 0.0000915527343750,
+0.0000305175781250, 0.0000915527343750, 0.0000000000000000, 0.0008239746093750,
+-0.0000610351562500, 0.0006713867187500, -0.0000305175781250, 0.0001220703125000,
+0.0000000000000000, 0.0004272460937500, -0.0000305175781250, 0.0000915527343750,
+0.0000000000000000, -0.0007324218750000, 0.0000915527343750, -0.0003967285156250,
+0.0000305175781250, -0.0004882812500000, -0.0000305175781250, -0.0008544921875000,
+0.0000305175781250, -0.0003356933593750, 0.0000000000000000, 0.0002441406250000,
+-0.0001220703125000, -0.0001831054687500, -0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, 0.0006713867187500, -0.0000305175781250, 0.0001831054687500,
+0.0000305175781250, -0.0000305175781250, 0.0001525878906250, 0.0003356933593750,
+0.0000610351562500, -0.0003967285156250, -0.0000305175781250, -0.0006103515625000,
+0.0000610351562500, -0.0002441406250000, -0.0000610351562500, -0.0003051757812500,
+-0.0001831054687500, -0.0004577636718750, -0.0000610351562500, 0.0003051757812500,
+0.0000000000000000, 0.0003356933593750, -0.0001525878906250, 0.0001220703125000,
+-0.0000305175781250, 0.0003662109375000, 0.0001525878906250, 0.0004272460937500,
+0.0000610351562500, -0.0002441406250000, 0.0000000000000000, -0.0002136230468750,
+0.0000915527343750, -0.0000915527343750, 0.0000610351562500, -0.0004272460937500,
+-0.0001525878906250, -0.0004272460937500, -0.0001220703125000, 0.0000915527343750,
+-0.0000305175781250, 0.0000000000000000, -0.0000915527343750, -0.0000305175781250,
+-0.0000305175781250, 0.0003051757812500, 0.0001525878906250, 0.0002136230468750,
+0.0000915527343750, -0.0002441406250000, -0.0000610351562500, -0.0001525878906250,
+0.0000305175781250, -0.0000305175781250, 0.0000000000000000, -0.0002746582031250,
+-0.0002136230468750, -0.0002746582031250, -0.0001831054687500, 0.0001220703125000,
+-0.0000305175781250, 0.0000305175781250, -0.0000915527343750, -0.0000305175781250,
+-0.0000305175781250, 0.0001525878906250, 0.0001525878906250, 0.0002136230468750,
+0.0001220703125000, -0.0001525878906250, -0.0000915527343750, -0.0001220703125000,
+-0.0000305175781250, -0.0000305175781250, -0.0000915527343750, -0.0002136230468750,
+-0.0003356933593750, -0.0002746582031250, -0.0002746582031250, 0.0000915527343750,
+0.0000610351562500, 0.0000915527343750, -0.0000610351562500, 0.0000000000000000,
+0.0000000000000000, 0.0001525878906250, 0.0003662109375000, 0.0002136230468750,
+0.0002441406250000, -0.0001220703125000, -0.0001525878906250, -0.0001220703125000,
+0.0000915527343750, -0.0000305175781250, -0.0000915527343750, -0.0002136230468750,
+-0.0006103515625000, -0.0002441406250000, -0.0003967285156250, 0.0000610351562500,
+-0.0001525878906250, 0.0000305175781250, -0.0004577636718750, 0.0000305175781250,
+-0.0001525878906250, 0.0001525878906250, 0.0004272460937500, 0.0001525878906250,
+0.0001220703125000, -0.0000610351562500, 0.0002746582031250, -0.0000305175781250,
+0.0005187988281250, 0.0000000000000000, 0.0001831054687500, -0.0001220703125000,
+-0.0001525878906250, -0.0001220703125000, 0.0000915527343750, 0.0000610351562500,
+-0.0003051757812500, 0.0000000000000000, -0.0004577636718750, 0.0000000000000000,
+-0.0002136230468750, 0.0000915527343750, -0.0001831054687500, 0.0000915527343750,
+-0.0003967285156250, -0.0000305175781250, 0.0000000000000000, 0.0000000000000000,
+0.0000915527343750, 0.0000305175781250, 0.0000000000000000, -0.0000915527343750,
+0.0001220703125000, -0.0000610351562500, 0.0001525878906250, 0.0000305175781250,
+-0.0001525878906250, 0.0000000000000000, -0.0001831054687500, -0.0000305175781250,
+-0.0000305175781250, 0.0000610351562500, -0.0000305175781250, 0.0000305175781250,
+-0.0001525878906250, 0.0000000000000000, 0.0004577636718750, 0.0000305175781250,
+0.0004272460937500, 0.0000305175781250, 0.0003967285156250, -0.0000610351562500,
+0.0007934570312500, 0.0000000000000000, 0.0007629394531250, 0.0000000000000000,
+-0.0002136230468750, 0.0000000000000000, 0.0002136230468750, -0.0000305175781250,
+-0.0006103515625000, 0.0000610351562500, -0.0020141601562500, 0.0000305175781250,
+-0.0013122558593750, 0.0000000000000000, -0.0026245117187500, 0.0000305175781250,
+-0.0030822753906250, 0.0000000000000000, -0.0022583007812500, 0.0000305175781250,
+-0.0023498535156250, 0.0000305175781250, -0.0026245117187500, 0.0000000000000000,
+0.0004272460937500, -0.0000305175781250, -0.0003051757812500, 0.0000000000000000,
+0.0013732910156250, 0.0000000000000000, 0.0044555664062500, -0.0000305175781250,
+0.0032653808593750, 0.0000305175781250, 0.0070800781250000, 0.0000000000000000,
+0.0072326660156250, 0.0000000000000000, 0.0077514648437500, 0.0000000000000000,
+0.0100097656250000, 0.0000000000000000, 0.0094604492187500, 0.0000000000000000,
+0.0102539062500000, 0.0000000000000000, 0.0105895996093750, -0.0000305175781250,
+0.0097961425781250, 0.0000000000000000, 0.0092163085937500, 0.0000000000000000,
+0.0089111328125000, 0.0000000000000000, 0.0073242187500000, 0.0000000000000000,
+0.0063476562500000, 0.0000000000000000, 0.0050048828125000, -0.0000305175781250,
+0.0037536621093750, 0.0000305175781250, 0.0035095214843750, -0.0000305175781250,
+0.0007629394531250, 0.0000000000000000, -0.0016479492187500, 0.0000000000000000,
+-0.0013732910156250, -0.0000305175781250, -0.0018615722656250, 0.0000000000000000,
+-0.0040893554687500, 0.0000000000000000, -0.0009765625000000, 0.0000305175781250,
+-0.0026245117187500, 0.0000000000000000, -0.0017700195312500, 0.0000000000000000,
+0.0010375976562500, 0.0000000000000000, -0.0006408691406250, 0.0000000000000000,
+0.0040283203125000, 0.0000305175781250, 0.0021667480468750, 0.0000000000000000,
+0.0046081542968750, 0.0000000000000000, 0.0092468261718750, 0.0000305175781250,
+0.0057983398437500, -0.0000305175781250, 0.0127868652343750, 0.0000305175781250,
+0.0124816894531250, -0.0000305175781250, 0.0124816894531250, 0.0000305175781250,
+0.0163574218750000, 0.0000000000000000, 0.0148315429687500, 0.0000000000000000,
+0.0155334472656250, 0.0000305175781250, 0.0170288085937500, 0.0000000000000000,
+0.0145568847656250, 0.0000000000000000, 0.0131835937500000, -0.0000305175781250,
+0.0153198242187500, 0.0000305175781250, 0.0076293945312500, 0.0000305175781250,
+0.0098571777343750, 0.0000000000000000, 0.0073242187500000, 0.0000000000000000,
+0.0008239746093750, 0.0000305175781250, 0.0046386718750000, -0.0000305175781250,
+-0.0027770996093750, -0.0000305175781250, -0.0032348632812500, 0.0000305175781250,
+-0.0026855468750000, -0.0000305175781250, -0.0061645507812500, 0.0000000000000000,
+-0.0052795410156250, -0.0000305175781250, -0.0040283203125000, 0.0000610351562500,
+-0.0052795410156250, 0.0000610351562500, -0.0035705566406250, 0.0000000000000000,
+-0.0011291503906250, 0.0000305175781250, -0.0027465820312500, -0.0000305175781250,
+0.0004272460937500, -0.0000610351562500, 0.0006408691406250, 0.0000000000000000,
+0.0003662109375000, 0.0000610351562500, 0.0018005371093750, 0.0000000000000000,
+0.0015258789062500, 0.0000305175781250, 0.0008850097656250, 0.0000915527343750,
+0.0012512207031250, 0.0000305175781250, 0.0008544921875000, -0.0000610351562500,
+0.0000305175781250, -0.0000305175781250, 0.0004272460937500, -0.0000305175781250,
+-0.0001525878906250, -0.0000915527343750, -0.0003967285156250, -0.0000305175781250,
+-0.0001220703125000, 0.0000610351562500, 0.0000305175781250, 0.0000000000000000,
+-0.0000915527343750, 0.0000610351562500, 0.0001525878906250, 0.0001220703125000,
+0.0003051757812500, 0.0000305175781250, 0.0000915527343750, -0.0000305175781250,
+0.0000000000000000, 0.0000305175781250, 0.0000915527343750, -0.0000915527343750,
+-0.0003051757812500, -0.0001220703125000, -0.0003356933593750, 0.0000000000000000,
+-0.0003051757812500, 0.0000305175781250, -0.0004882812500000, -0.0000610351562500,
+-0.0004577636718750, 0.0001220703125000, -0.0001220703125000, 0.0001525878906250,
+-0.0003051757812500, 0.0000305175781250, -0.0000610351562500, 0.0000000000000000,
+0.0004272460937500, 0.0000915527343750, 0.0001525878906250, -0.0001220703125000,
+0.0005187988281250, -0.0001831054687500, 0.0007934570312500, -0.0000915527343750,
+0.0002746582031250, -0.0000915527343750, 0.0000305175781250, -0.0001831054687500,
+0.0004272460937500, 0.0001220703125000, -0.0007629394531250, 0.0001525878906250,
+-0.0008239746093750, 0.0000305175781250, -0.0004577636718750, 0.0002441406250000,
+-0.0009460449218750, 0.0002136230468750, -0.0010375976562500, -0.0001831054687500,
+0.0004272460937500, -0.0000915527343750, 0.0002136230468750, -0.0000915527343750,
+0.0001525878906250, -0.0003967285156250, 0.0012512207031250, -0.0003356933593750,
+0.0010681152343750, 0.0000000000000000, -0.0002136230468750, -0.0001220703125000,
+0.0002136230468750, 0.0000000000000000, -0.0000610351562500, 0.0004882812500000,
+-0.0014343261718750, 0.0003051757812500, -0.0010070800781250, 0.0000610351562500,
+-0.0003356933593750, 0.0003051757812500, -0.0009155273437500, 0.0000915527343750,
+-0.0004272460937500, -0.0007324218750000, 0.0012817382812500, -0.0000305175781250,
+-0.0008239746093750, 0.0004577636718750, -0.0000610351562500, 0.0007019042968750,
+-0.0009155273437500, 0.0001220703125000, -0.0002746582031250, -0.0002746582031250,
+0.0010375976562500, 0.0002441406250000, 0.0005493164062500, -0.0007629394531250,
+0.0001525878906250},
+{-0.1732177734375000, 0.0000000000000000, -0.1270751953125000, 0.0000305175781250,
+-0.1393432617187500, 0.0000305175781250, -0.1225891113281250, 0.0000305175781250,
+-0.0855102539062500, 0.0000000000000000, -0.1110839843750000, -0.0000305175781250,
+-0.0522766113281250, 0.0000000000000000, -0.0614929199218750, 0.0000000000000000,
+-0.0523986816406250, -0.0000610351562500, -0.0119628906250000, -0.0000305175781250,
+-0.0327453613281250, 0.0000305175781250, 0.0000000000000000, 0.0000305175781250,
+0.0033569335937500, 0.0000000000000000, 0.0012512207031250, 0.0000305175781250,
+0.0157470703125000, 0.0000305175781250, 0.0118103027343750, -0.0000305175781250,
+0.0097351074218750, 0.0000000000000000, 0.0136108398437500, 0.0000305175781250,
+0.0088195800781250, 0.0000000000000000, 0.0026245117187500, 0.0000305175781250,
+0.0067443847656250, 0.0000305175781250, -0.0015563964843750, 0.0000610351562500,
+-0.0020446777343750, 0.0000305175781250, -0.0013427734375000, 0.0000000000000000,
+-0.0050964355468750, 0.0000000000000000, -0.0042419433593750, -0.0000305175781250,
+-0.0028076171875000, -0.0000610351562500, -0.0039672851562500, -0.0000305175781250,
+-0.0025634765625000, 0.0000305175781250, -0.0002136230468750, -0.0000305175781250,
+-0.0013732910156250, 0.0000305175781250, 0.0008544921875000, 0.0000610351562500,
+0.0013122558593750, 0.0000000000000000, 0.0007019042968750, 0.0000000000000000,
+0.0012207031250000, 0.0000000000000000, 0.0013732910156250, -0.0000610351562500,
+0.0003967285156250, -0.0000915527343750, 0.0004577636718750, 0.0000000000000000,
+0.0004272460937500, 0.0000000000000000, -0.0002441406250000, -0.0000305175781250,
+-0.0000915527343750, 0.0000610351562500, -0.0001220703125000, 0.0001220703125000,
+-0.0002746582031250, 0.0000000000000000, -0.0000915527343750, 0.0000305175781250,
+0.0000915527343750, 0.0000915527343750, -0.0000610351562500, -0.0001220703125000,
+0.0000915527343750, -0.0001525878906250, 0.0002136230468750, -0.0000915527343750,
+-0.0000610351562500, -0.0000915527343750, -0.0003662109375000, -0.0001525878906250,
+-0.0001220703125000, 0.0001220703125000, -0.0004882812500000, 0.0001220703125000,
+-0.0007324218750000, 0.0000610351562500, -0.0002441406250000, 0.0001525878906250,
+0.0000000000000000, 0.0001831054687500, -0.0003051757812500, -0.0001831054687500,
+0.0006408691406250, -0.0000915527343750, 0.0007934570312500, -0.0000610351562500,
+0.0003356933593750, -0.0003356933593750, 0.0005798339843750, -0.0002746582031250,
+0.0008544921875000, 0.0000305175781250, -0.0007019042968750, -0.0000915527343750,
+-0.0006408691406250, 0.0000000000000000, -0.0003662109375000, 0.0003356933593750,
+-0.0011291503906250, 0.0002441406250000, -0.0011901855468750, 0.0000305175781250,
+0.0003967285156250, 0.0002136230468750, 0.0000305175781250, 0.0000000000000000,
+0.0000915527343750, -0.0004882812500000, 0.0013122558593750, -0.0002441406250000,
+0.0010681152343750, -0.0003356933593750, -0.0000610351562500, -0.0006103515625000,
+0.0003967285156250, -0.0002746582031250, 0.0002136230468750, 0.0004577636718750,
+-0.0014953613281250, 0.0002441406250000, 0.0010070800781250, -0.0006408691406250,
+-0.0002746582031250, -0.0007324218750000, 0.0004882812500000, -0.0002441406250000,
+0.0000610351562500, -0.0002136230468750, -0.0011596679687500, -0.0005798339843750,
+-0.0008544921875000, 0.0006103515625000, 0.0000000000000000, 0.0007324218750000,
+-0.0004577636718750, 0.0003356933593750, -0.0001525878906250, 0.0007629394531250,
+0.0009460449218750, 0.0009155273437500, 0.0005798339843750, -0.0004882812500000,
+0.0000610351562500, -0.0002746582031250, 0.0005187988281250, -0.0002746582031250,
+0.0001525878906250, -0.0014038085937500, -0.0007934570312500, -0.0011291503906250,
+-0.0004272460937500, -0.0000610351562500, -0.0001831054687500, -0.0006408691406250,
+-0.0005798339843750, -0.0001525878906250, -0.0002136230468750, 0.0013122558593750,
+0.0005187988281250, 0.0006713867187500, 0.0001831054687500, 0.0003967285156250,
+0.0001220703125000, 0.0011596679687500, 0.0004272460937500, 0.0003662109375000,
+0.0001220703125000, -0.0008544921875000, -0.0004272460937500, -0.0000915527343750,
+-0.0001220703125000, -0.0006103515625000, -0.0001220703125000, -0.0011596679687500,
+-0.0003967285156250, -0.0004577636718750, -0.0001220703125000, 0.0000915527343750,
+0.0002746582031250, -0.0003967285156250, 0.0000610351562500, 0.0003356933593750,
+0.0000610351562500, 0.0006713867187500, 0.0002441406250000, 0.0001220703125000,
+0.0001220703125000, -0.0001220703125000, -0.0002136230468750, 0.0003051757812500,
+-0.0000915527343750, -0.0003051757812500, -0.0001525878906250, -0.0007324218750000,
+-0.0002441406250000, 0.0000610351562500, -0.0000610351562500, 0.0005493164062500,
+0.0001525878906250, -0.0001525878906250, 0.0000610351562500, 0.0012512207031250,
+0.0000915527343750, 0.0016479492187500, 0.0001831054687500, 0.0006103515625000,
+0.0000305175781250, 0.0007934570312500, -0.0001220703125000, 0.0014648437500000,
+-0.0000610351562500, -0.0018005371093750, -0.0000305175781250, -0.0009765625000000,
+-0.0001525878906250, -0.0016784667968750, -0.0000305175781250, -0.0046386718750000,
+0.0001220703125000, -0.0033569335937500, 0.0000305175781250, -0.0024108886718750,
+0.0000000000000000, -0.0044250488281250, 0.0000915527343750, -0.0011291503906250,
+0.0000305175781250, 0.0019836425781250, -0.0000915527343750, -0.0010986328125000,
+-0.0000305175781250, 0.0079956054687500, 0.0000305175781250, 0.0073547363281250,
+-0.0000610351562500, 0.0071105957031250, -0.0000610351562500, 0.0133666992187500,
+0.0000305175781250, 0.0115356445312500, 0.0000610351562500, 0.0046386718750000,
+0.0000000000000000, 0.0118103027343750, 0.0000305175781250, -0.0021667480468750,
+0.0000000000000000, -0.0142211914062500, -0.0000610351562500, -0.0013122558593750,
+0.0000000000000000, -0.0486755371093750, 0.0000915527343750, -0.0323791503906250,
+0.0000305175781250, -0.0565185546875000, 0.0000305175781250, -0.0997009277343750,
+0.0000610351562500, -0.0673522949218750, 0.0000915527343750, -0.1422119140625000,
+0.0000610351562500, -0.1279602050781250, 0.0000610351562500, -0.1482849121093750,
+0.0000305175781250, -0.2034912109375000, 0.0000305175781250, -0.1672058105468750,
+0.0000305175781250, -0.2322082519531250, 0.0000610351562500, -0.2271728515625000,
+0.0000610351562500, -0.2299194335937500, 0.0000610351562500, -0.2670593261718750,
+0.0000610351562500, -0.2479858398437500, 0.0000610351562500, -0.2685241699218750,
+0.0000305175781250, -0.2733459472656250, 0.0000305175781250, -0.2658081054687500,
+0.0000000000000000, -0.2714538574218750, 0.0000000000000000, -0.2742309570312500,
+0.0000000000000000, -0.2580871582031250, 0.0000305175781250, -0.2659606933593750,
+0.0000305175781250, -0.2578735351562500, 0.0000000000000000, -0.2431945800781250,
+-0.0000305175781250, -0.2589111328125000, 0.0000000000000000, -0.2380371093750000,
+-0.0000610351562500, -0.2438354492187500, -0.0000610351562500, -0.2415771484375000,
+-0.0000305175781250, -0.2284240722656250, -0.0000610351562500, -0.2418518066406250,
+-0.0000610351562500, -0.2284240722656250, -0.0000305175781250, -0.2324218750000000,
+-0.0000305175781250, -0.2352905273437500, -0.0000305175781250, -0.2295227050781250,
+-0.0000305175781250, -0.2380065917968750, -0.0000305175781250, -0.2383422851562500,
+-0.0000305175781250, -0.2427062988281250, 0.0000000000000000, -0.2452392578125000,
+-0.0000305175781250, -0.2455139160156250, 0.0000000000000000, -0.2496643066406250,
+-0.0000610351562500, -0.2462158203125000, 0.0000000000000000, -0.2483825683593750,
+-0.0000610351562500, -0.2437744140625000, 0.0000000000000000, -0.2384948730468750,
+0.0000305175781250, -0.2407226562500000, 0.0000305175781250, -0.2190551757812500,
+0.0000305175781250, -0.2237854003906250, 0.0000610351562500, -0.2085876464843750,
+0.0000305175781250, -0.1863708496093750, 0.0000305175781250, -0.1955566406250000,
+0.0000000000000000, -0.1480712890625000, 0.0000305175781250, -0.1536865234375000,
+0.0000610351562500, -0.1338500976562500, 0.0000610351562500, -0.0936584472656250,
+0.0000610351562500, -0.1094665527343750, 0.0000305175781250, -0.0586547851562500,
+0.0000305175781250, -0.0590820312500000, 0.0000305175781250, -0.0473632812500000,
+0.0000305175781250, -0.0124206542968750, 0.0000000000000000, -0.0244750976562500,
+0.0000000000000000, 0.0011596679687500, 0.0000305175781250, 0.0049133300781250,
+-0.0000305175781250, 0.0029296875000000, 0.0000000000000000, 0.0130615234375000,
+0.0000305175781250, 0.0110778808593750, -0.0000305175781250, 0.0072937011718750,
+-0.0000305175781250, 0.0097045898437500, 0.0000305175781250, 0.0059204101562500,
+0.0000000000000000, -0.0000610351562500, -0.0000305175781250, 0.0028381347656250,
+-0.0000305175781250, -0.0026550292968750, 0.0000000000000000, -0.0041503906250000,
+-0.0000610351562500, -0.0025939941406250, -0.0000305175781250, -0.0035705566406250,
+0.0000915527343750, -0.0039978027343750, 0.0000305175781250, -0.0006713867187500,
+0.0000000000000000, -0.0009460449218750, 0.0000610351562500, -0.0004882812500000,
+0.0000305175781250, 0.0018615722656250, -0.0000305175781250, 0.0011596679687500,
+-0.0000305175781250, 0.0007629394531250, -0.0000610351562500, 0.0014343261718750,
+-0.0000610351562500, 0.0006408691406250, 0.0000000000000000, -0.0007019042968750,
+0.0000610351562500, -0.0000305175781250, -0.0000610351562500, -0.0005187988281250,
+0.0000305175781250, -0.0010681152343750, 0.0001220703125000, -0.0004272460937500,
+-0.0000305175781250, 0.0002136230468750, -0.0000915527343750, -0.0002441406250000,
+0.0000000000000000, 0.0003662109375000, -0.0000305175781250, 0.0007324218750000,
+-0.0001525878906250, 0.0002441406250000, -0.0000305175781250, -0.0000610351562500,
+0.0000915527343750, 0.0002441406250000, 0.0000000000000000, -0.0003356933593750,
+0.0000610351562500, -0.0005798339843750, 0.0001220703125000, -0.0002136230468750,
+0.0000305175781250, -0.0000915527343750, -0.0001220703125000, -0.0003051757812500,
+0.0000000000000000, 0.0003051757812500, -0.0000915527343750, 0.0004577636718750,
+-0.0001831054687500, 0.0000915527343750, -0.0000610351562500, 0.0001220703125000,
+0.0000610351562500, 0.0003356933593750, -0.0000610351562500, -0.0003051757812500,
+0.0000610351562500, -0.0004577636718750, 0.0001525878906250, -0.0001831054687500,
+0.0000305175781250, -0.0002441406250000, -0.0000610351562500, -0.0004577636718750,
+0.0000305175781250, 0.0001220703125000, -0.0001525878906250, 0.0001831054687500,
+-0.0001831054687500, 0.0000305175781250, -0.0000610351562500, 0.0001220703125000,
+0.0001220703125000, 0.0000305175781250, 0.0000305175781250, -0.0002441406250000,
+-0.0000610351562500, -0.0003051757812500, 0.0000915527343750, -0.0000610351562500,
+0.0000000000000000, -0.0000915527343750, -0.0002136230468750, -0.0002441406250000,
+-0.0000915527343750, 0.0001220703125000, -0.0000915527343750, 0.0002441406250000,
+-0.0002441406250000, 0.0000000000000000, -0.0000610351562500, 0.0000000000000000,
+0.0001525878906250, 0.0001525878906250, 0.0000305175781250, -0.0001831054687500,
+-0.0000610351562500, -0.0003051757812500, 0.0000915527343750, -0.0000915527343750,
+-0.0000915527343750, -0.0000610351562500, -0.0003051757812500, -0.0002746582031250,
+-0.0001831054687500, 0.0001220703125000, -0.0000305175781250, 0.0002441406250000,
+-0.0002136230468750, 0.0000305175781250, -0.0000610351562500, -0.0000305175781250,
+0.0003356933593750, 0.0001220703125000, 0.0001220703125000, -0.0001831054687500,
+0.0000610351562500, -0.0002441406250000, 0.0002441406250000, -0.0000915527343750,
+0.0000000000000000},
+{-0.0000610351562500, -0.1788635253906250, 0.0000000000000000, -0.1888122558593750,
+0.0000305175781250, -0.1915283203125000, 0.0000000000000000, -0.1538391113281250,
+0.0000610351562500, -0.1652832031250000, 0.0000305175781250, -0.1402587890625000,
+0.0000000000000000, -0.0991516113281250, 0.0000000000000000, -0.1190490722656250,
+0.0000000000000000, -0.0572814941406250, 0.0000000000000000, -0.0572204589843750,
+-0.0000305175781250, -0.0456848144531250, 0.0000305175781250, -0.0059814453125000,
+0.0000610351562500, -0.0207824707031250, 0.0000000000000000, 0.0022277832031250,
+0.0000305175781250, 0.0083312988281250, 0.0000305175781250, 0.0032958984375000,
+0.0000000000000000, 0.0082397460937500, -0.0000305175781250, 0.0086364746093750,
+0.0000000000000000, 0.0014953613281250, -0.0000305175781250, 0.0021667480468750,
+-0.0000610351562500, 0.0011901855468750, 0.0000000000000000, -0.0038146972656250,
+0.0000305175781250, -0.0024108886718750, -0.0000305175781250, -0.0016479492187500,
+0.0000305175781250, -0.0030517578125000, 0.0000915527343750, -0.0014953613281250,
+0.0000000000000000, 0.0010070800781250, -0.0000305175781250, -0.0002136230468750,
+0.0000000000000000, 0.0008239746093750, -0.0000610351562500, 0.0017089843750000,
+-0.0001220703125000, 0.0006713867187500, -0.0000305175781250, -0.0001220703125000,
+0.0000000000000000, 0.0005187988281250, -0.0000305175781250, -0.0004882812500000,
+0.0000610351562500, -0.0009765625000000, 0.0001525878906250, -0.0003356933593750,
+0.0000305175781250, -0.0000915527343750, -0.0000610351562500, -0.0005187988281250,
+0.0000610351562500, 0.0002136230468750, -0.0000915527343750, 0.0005187988281250,
+-0.0001831054687500, 0.0000915527343750, -0.0000610351562500, 0.0000000000000000,
+0.0000000000000000, 0.0003051757812500, -0.0000915527343750, -0.0003662109375000,
+0.0000915527343750, -0.0005493164062500, 0.0001831054687500, -0.0001525878906250,
+0.0000000000000000, -0.0001220703125000, -0.0000610351562500, -0.0003662109375000,
+0.0000610351562500, 0.0002441406250000, -0.0001525878906250, 0.0004272460937500,
+-0.0002441406250000, 0.0000610351562500, -0.0000610351562500, 0.0000610351562500,
+0.0000610351562500, 0.0002441406250000, -0.0001220703125000, -0.0003051757812500,
+0.0000305175781250, -0.0003662109375000, 0.0001525878906250, -0.0001525878906250,
+0.0000305175781250, -0.0001220703125000, -0.0000915527343750, -0.0000915527343750,
+-0.0001220703125000, 0.0001220703125000, -0.0001220703125000, 0.0002136230468750,
+-0.0001525878906250, 0.0000000000000000, -0.0000610351562500, -0.0000610351562500,
+0.0000915527343750, 0.0001525878906250, -0.0000305175781250, -0.0002136230468750,
+0.0000305175781250, -0.0003356933593750, 0.0001525878906250, -0.0000915527343750,
+0.0000000000000000, 0.0000305175781250, -0.0002136230468750, -0.0001525878906250,
+-0.0000915527343750, 0.0001220703125000, -0.0000915527343750, 0.0002746582031250,
+-0.0002136230468750, 0.0000305175781250, -0.0000610351562500, -0.0001220703125000,
+0.0001525878906250, 0.0000305175781250, 0.0000305175781250, -0.0002136230468750,
+0.0000305175781250, -0.0003662109375000, 0.0001831054687500, -0.0001220703125000,
+-0.0000305175781250, 0.0000610351562500, -0.0003051757812500, -0.0001220703125000,
+-0.0001525878906250, 0.0001220703125000, -0.0001525878906250, 0.0002441406250000,
+-0.0003967285156250, 0.0000610351562500, -0.0001525878906250, -0.0001220703125000,
+0.0002136230468750, 0.0000610351562500, 0.0000305175781250, -0.0001525878906250,
+0.0000610351562500, -0.0002441406250000, 0.0002441406250000, -0.0000915527343750,
+0.0000915527343750, 0.0000610351562500, -0.0001525878906250, -0.0000915527343750,
+-0.0000305175781250, 0.0000610351562500, 0.0000915527343750, 0.0002136230468750,
+-0.0000915527343750, 0.0000610351562500, 0.0000305175781250, 0.0000000000000000,
+0.0003967285156250, 0.0000915527343750, 0.0002746582031250, -0.0000610351562500,
+-0.0004882812500000, -0.0001525878906250, -0.0001220703125000, -0.0000305175781250,
+-0.0004577636718750, 0.0000305175781250, -0.0015869140625000, -0.0000915527343750,
+-0.0012207031250000, 0.0000610351562500, -0.0000610351562500, 0.0001220703125000,
+-0.0011596679687500, 0.0000305175781250, 0.0006408691406250, 0.0000305175781250,
+0.0036010742187500, 0.0000610351562500, 0.0016784667968750, -0.0000610351562500,
+0.0018310546875000, -0.0000915527343750, 0.0059204101562500, 0.0000000000000000,
+-0.0036010742187500, -0.0000305175781250, -0.0094909667968750, -0.0000610351562500,
+-0.0038452148437500, 0.0000305175781250, -0.0398559570312500, 0.0000305175781250,
+-0.0322875976562500, 0.0000000000000000, -0.0533142089843750, 0.0000305175781250,
+-0.0901794433593750, 0.0000610351562500, -0.0724487304687500, -0.0000305175781250,
+-0.1173400878906250, 0.0000000000000000, -0.1232299804687500, 0.0000000000000000,
+-0.1164550781250000, -0.0000305175781250, -0.1358947753906250, -0.0000610351562500,
+-0.1337280273437500, 0.0000305175781250, -0.0971679687500000, 0.0000305175781250,
+-0.1155700683593750, 0.0000000000000000, -0.0749511718750000, 0.0000000000000000,
+-0.0227050781250000, 0.0000610351562500, -0.0484619140625000, 0.0000000000000000,
+0.0701599121093750, 0.0000000000000000, 0.0583190917968750, 0.0000000000000000,
+0.1020507812500000, -0.0000305175781250, 0.1978759765625000, -0.0000305175781250,
+0.1622009277343750, 0.0000000000000000, 0.2756347656250000, 0.0000610351562500,
+0.2764587402343750, 0.0000305175781250, 0.3076477050781250, 0.0000305175781250,
+0.3871154785156250, 0.0000305175781250, 0.3618774414062500, 0.0000000000000000,
+0.4421997070312500, 0.0000000000000000, 0.4496765136718750, 0.0000610351562500,
+0.4650268554687500, 0.0000000000000000, 0.5104980468750000, 0.0000000000000000,
+0.4947509765625000, 0.0000610351562500, 0.5336914062500000, 0.0000000000000000,
+0.5345764160156250, 0.0000305175781250, 0.5450439453125000, 0.0000305175781250,
+0.5698547363281250, 0.0000305175781250, 0.5559997558593750, 0.0000305175781250,
+0.5819396972656250, 0.0000000000000000, 0.5802612304687500, 0.0000305175781250,
+0.5871276855468750, 0.0000305175781250, 0.6038513183593750, 0.0000305175781250,
+0.5926208496093750, 0.0000000000000000, 0.6171875000000000, -0.0000305175781250,
+0.6148986816406250, 0.0000305175781250, 0.6169738769531250, 0.0000305175781250,
+0.6322631835937500, 0.0000305175781250, 0.6273498535156250, 0.0000000000000000,
+0.6412353515625000, 0.0000000000000000, 0.6433105468750000, 0.0000305175781250,
+0.6436462402343750, 0.0000305175781250, 0.6510620117187500, 0.0000000000000000,
+0.6474304199218750, 0.0000000000000000, 0.6364135742187500, 0.0000305175781250,
+0.6489868164062500, 0.0000000000000000, 0.6247253417968750, 0.0000000000000000,
+0.6054382324218750, 0.0000305175781250, 0.6287841796875000, 0.0000305175781250,
+0.5438537597656250, 0.0000000000000000, 0.5796508789062500, 0.0000000000000000,
+0.5264892578125000, 0.0000610351562500, 0.4465332031250000, 0.0000305175781250,
+0.5121765136718750, 0.0000305175781250, 0.3138122558593750, 0.0000000000000000,
+0.3676757812500000, 0.0000000000000000, 0.2991943359375000, -0.0000305175781250,
+0.1372375488281250, 0.0000305175781250, 0.2400817871093750, 0.0000000000000000,
+0.0099792480468750, 0.0000305175781250, 0.0283508300781250, 0.0000305175781250,
+-0.0129699707031250, 0.0000610351562500, -0.1678161621093750, 0.0000305175781250,
+-0.0939941406250000, 0.0000000000000000, -0.2091979980468750, 0.0000000000000000,
+-0.2213134765625000, 0.0000305175781250, -0.2009582519531250, -0.0000305175781250,
+-0.2481079101562500, -0.0000305175781250, -0.2426757812500000, 0.0000610351562500,
+-0.1946411132812500, 0.0000000000000000, -0.2182617187500000, 0.0000305175781250,
+-0.1806030273437500, 0.0000610351562500, -0.1220703125000000, 0.0000915527343750,
+-0.1584167480468750, -0.0000610351562500, -0.0707397460937500, -0.0000305175781250,
+-0.0733642578125000, 0.0000000000000000, -0.0624694824218750, -0.0000305175781250,
+-0.0075378417968750, -0.0000610351562500, -0.0274963378906250, 0.0000305175781250,
+0.0014343261718750, 0.0000610351562500, 0.0091247558593750, 0.0000000000000000,
+0.0029602050781250, 0.0000610351562500, 0.0096435546875000, 0.0000915527343750,
+0.0099487304687500, -0.0000610351562500, 0.0018920898437500, -0.0000305175781250,
+0.0031433105468750, 0.0000000000000000, 0.0018310546875000, -0.0001525878906250,
+-0.0040893554687500, -0.0001220703125000, -0.0022277832031250, 0.0000610351562500,
+-0.0023193359375000, 0.0000610351562500, -0.0039978027343750, 0.0000000000000000,
+-0.0021057128906250, 0.0000915527343750, 0.0001525878906250, 0.0001525878906250,
+-0.0013427734375000, -0.0000305175781250, 0.0010375976562500, -0.0000305175781250,
+0.0018005371093750, -0.0000305175781250, 0.0007324218750000, -0.0002136230468750,
+0.0007629394531250, -0.0002136230468750, 0.0013122558593750, 0.0000000000000000,
+-0.0003662109375000, -0.0000610351562500, -0.0005798339843750, 0.0000000000000000,
+-0.0000610351562500, 0.0002441406250000, -0.0004577636718750, 0.0001525878906250,
+-0.0006713867187500, 0.0000000000000000, 0.0004272460937500, 0.0001220703125000,
+0.0005187988281250, 0.0000000000000000, 0.0000610351562500, -0.0003356933593750,
+0.0003051757812500, -0.0001831054687500, 0.0006103515625000, -0.0001525878906250,
+-0.0007019042968750, -0.0003051757812500, -0.0008544921875000, -0.0000915527343750,
+-0.0003051757812500, 0.0002441406250000, -0.0006103515625000, 0.0000305175781250,
+-0.0009155273437500, 0.0001831054687500, 0.0007019042968750, 0.0004577636718750,
+0.0007629394531250, 0.0001220703125000, 0.0002746582031250, -0.0001831054687500,
+0.0008544921875000, 0.0000915527343750, 0.0010681152343750, -0.0004577636718750,
+-0.0006103515625000, -0.0006103515625000, -0.0004882812500000, -0.0002746582031250,
+-0.0001831054687500, -0.0000610351562500, -0.0012207031250000, 0.0004577636718750,
+0.0010070800781250, -0.0005187988281250, -0.0007019042968750, -0.0002746582031250,
+-0.0002746582031250, -0.0001831054687500, -0.0001220703125000, -0.0007019042968750,
+-0.0008544921875000, -0.0007019042968750, -0.0009765625000000, 0.0002441406250000,
+0.0003967285156250, 0.0000305175781250, 0.0002746582031250, 0.0001220703125000,
+0.0000610351562500, 0.0009765625000000, 0.0008239746093750, 0.0007629394531250,
+0.0008850097656250, 0.0000915527343750, -0.0003051757812500, 0.0005798339843750,
+-0.0001525878906250, 0.0000915527343750, -0.0000610351562500, -0.0009155273437500,
+-0.0008544921875000, -0.0004577636718750, -0.0007629394531250, -0.0004272460937500,
+0.0000915527343750, -0.0009765625000000, -0.0000610351562500, -0.0003662109375000,
+-0.0000915527343750, 0.0004272460937500, 0.0005493164062500, -0.0000915527343750,
+0.0004577636718750, 0.0001831054687500, -0.0000915527343750, 0.0005493164062500,
+0.0000610351562500, 0.0002441406250000, 0.0000000000000000, -0.0002441406250000,
+-0.0005493164062500, 0.0000305175781250, -0.0004272460937500, 0.0003967285156250,
+0.0000305175781250, 0.0000000000000000, -0.0001525878906250, 0.0003051757812500,
+-0.0000610351562500, 0.0012512207031250, 0.0003967285156250, 0.0008239746093750,
+0.0002746582031250, -0.0004272460937500, -0.0000915527343750, 0.0008239746093750,
+0.0000915527343750, -0.0006713867187500, 0.0000915527343750, -0.0034484863281250,
+-0.0003662109375000, -0.0017395019531250, -0.0002746582031250, -0.0021057128906250,
+-0.0000305175781250, -0.0047607421875000, -0.0001525878906250, -0.0001220703125000,
+-0.0000610351562500, 0.0039672851562500, 0.0002441406250000, -0.0004577636718750,
+0.0001831054687500},
+{-0.0602416992187500, -0.0000915527343750, -0.0938110351562500, 0.0000610351562500,
+-0.0245056152343750, 0.0000305175781250, -0.0187377929687500, 0.0000000000000000,
+-0.0201416015625000, 0.0000915527343750, 0.0121765136718750, 0.0000610351562500,
+0.0035095214843750, -0.0000915527343750, 0.0048217773437500, 0.0000000000000000,
+0.0102539062500000, -0.0000305175781250, 0.0049743652343750, -0.0001831054687500,
+-0.0010070800781250, -0.0000915527343750, 0.0025329589843750, 0.0000305175781250,
+-0.0024108886718750, -0.0000915527343750, -0.0045166015625000, 0.0000000000000000,
+-0.0020446777343750, 0.0002136230468750, -0.0016784667968750, 0.0000915527343750,
+-0.0029602050781250, 0.0000000000000000, 0.0007629394531250, 0.0001220703125000,
+0.0012207031250000, -0.0000305175781250, 0.0003051757812500, -0.0002746582031250,
+0.0013427734375000, -0.0001525878906250, 0.0016479492187500, -0.0001220703125000,
+-0.0003051757812500, -0.0002746582031250, -0.0003356933593750, -0.0000915527343750,
+0.0001525878906250, 0.0002441406250000, -0.0006408691406250, 0.0000305175781250,
+-0.0008239746093750, 0.0001831054687500, 0.0005493164062500, 0.0003356933593750,
+0.0006408691406250, 0.0000305175781250, 0.0001220703125000, -0.0001831054687500,
+0.0004577636718750, 0.0000305175781250, 0.0007324218750000, -0.0003662109375000,
+-0.0006713867187500, -0.0005187988281250, -0.0007934570312500, -0.0001525878906250,
+-0.0001831054687500, -0.0000915527343750, -0.0005798339843750, -0.0003356933593750,
+-0.0009460449218750, 0.0003356933593750, 0.0006103515625000, 0.0004577636718750,
+0.0006713867187500, 0.0001831054687500, 0.0002746582031250, 0.0003356933593750,
+0.0008239746093750, -0.0005493164062500, -0.0009460449218750, 0.0002746582031250,
+0.0006713867187500, -0.0001831054687500, 0.0004577636718750, 0.0000305175781250,
+0.0001220703125000, 0.0006713867187500, 0.0004882812500000, 0.0005187988281250,
+0.0007629394531250, -0.0000915527343750, -0.0006103515625000, 0.0002746582031250,
+-0.0007019042968750, 0.0000000000000000, -0.0002136230468750, -0.0008239746093750,
+-0.0006408691406250, -0.0004882812500000, -0.0008544921875000, -0.0003662109375000,
+0.0003967285156250, -0.0007629394531250, 0.0003967285156250, -0.0003356933593750,
+0.0000915527343750, 0.0003662109375000, 0.0006103515625000, -0.0000305175781250,
+0.0007629394531250, 0.0002441406250000, -0.0002746582031250, 0.0005187988281250,
+-0.0002746582031250, 0.0003051757812500, -0.0000610351562500, 0.0000915527343750,
+-0.0006103515625000, 0.0002136230468750, -0.0006103515625000, 0.0005187988281250,
+0.0001220703125000, 0.0003662109375000, 0.0000915527343750, 0.0001220703125000,
+0.0000000000000000, 0.0005798339843750, 0.0003967285156250, 0.0007324218750000,
+0.0004272460937500, -0.0014648437500000, -0.0001220703125000, -0.0007019042968750,
+-0.0000610351562500, -0.0010375976562500, -0.0000305175781250, -0.0036926269531250,
+-0.0004272460937500, -0.0028381347656250, -0.0003967285156250, 0.0010070800781250,
+0.0000915527343750, -0.0018615722656250, -0.0000305175781250, 0.0014953613281250,
+0.0000000000000000, 0.0088195800781250, 0.0003051757812500, 0.0043945312500000,
+0.0002746582031250, 0.0014038085937500, -0.0000915527343750, 0.0126037597656250,
+-0.0000610351562500, -0.0104980468750000, 0.0000000000000000, -0.0263977050781250,
+-0.0002136230468750, -0.0069885253906250, -0.0002441406250000, -0.0854797363281250,
+0.0000610351562500, -0.0653381347656250, 0.0000000000000000, -0.0930480957031250,
+0.0000000000000000, -0.1628723144531250, 0.0001831054687500, -0.1232299804687500,
+0.0002136230468750, -0.1783142089843750, -0.0000305175781250, -0.1967163085937500,
+-0.0000305175781250, -0.1602478027343750, 0.0000305175781250, -0.1687927246093750,
+-0.0001525878906250, -0.1882019042968750, -0.0001525878906250, -0.0605468750000000,
+-0.0000305175781250, -0.1136169433593750, -0.0000305175781250, -0.0315551757812500,
+-0.0000610351562500, 0.1007995605468750, 0.0000000000000000, 0.0096130371093750,
+0.0000000000000000, 0.2584838867187500, -0.0001525878906250, 0.2217102050781250,
+-0.0001525878906250, 0.2854309082031250, -0.0000915527343750, 0.4679260253906250,
+-0.0001220703125000, 0.3681945800781250, -0.0001220703125000, 0.5502014160156250,
+0.0000305175781250, 0.5587158203125000, 0.0000305175781250, 0.5479125976562500,
+-0.0000305175781250, 0.6390686035156250, 0.0000610351562500, 0.6114196777343750,
+0.0000305175781250, 0.5705566406250000, 0.0000610351562500, 0.6248168945312500,
+-0.0000305175781250, 0.5467224121093750, 0.0000305175781250, 0.4653015136718750,
+0.0001220703125000, 0.5466918945312500, 0.0000610351562500, 0.3243103027343750,
+0.0001831054687500, 0.3807067871093750, 0.0001831054687500, 0.2969055175781250,
+0.0001525878906250, 0.1177673339843750, 0.0001525878906250, 0.2469787597656250,
+0.0001831054687500, -0.0136108398437500, 0.0001525878906250, 0.0231323242187500,
+0.0000915527343750, -0.0147399902343750, 0.0001525878906250, -0.1794738769531250,
+0.0001831054687500, -0.0779724121093750, 0.0002136230468750, -0.2492065429687500,
+0.0000915527343750, -0.2300109863281250, 0.0001525878906250, -0.2435913085937500,
+0.0001220703125000, -0.3493652343750000, 0.0000610351562500, -0.2927551269531250,
+0.0000915527343750, -0.3686218261718750, 0.0000915527343750, -0.3653564453125000,
+0.0000915527343750, -0.3666381835937500, 0.0000610351562500, -0.4049987792968750,
+0.0000610351562500, -0.3789062500000000, 0.0001220703125000, -0.4119262695312500,
+0.0000305175781250, -0.4085388183593750, 0.0000610351562500, -0.4041748046875000,
+0.0000610351562500, -0.4184265136718750, 0.0000610351562500, -0.4075927734375000,
+0.0000610351562500, -0.4136352539062500, 0.0000305175781250, -0.4086303710937500,
+0.0000915527343750, -0.4034423828125000, 0.0000610351562500, -0.4063720703125000,
+0.0000610351562500, -0.3986511230468750, 0.0000610351562500, -0.3829345703125000,
+0.0000610351562500, -0.3847961425781250, 0.0000915527343750, -0.3695068359375000,
+0.0000305175781250, -0.3464965820312500, 0.0000915527343750, -0.3403930664062500,
+0.0000915527343750, -0.2973937988281250, 0.0000915527343750, -0.3014221191406250,
+0.0000915527343750, -0.2619628906250000, 0.0000610351562500, -0.2141113281250000,
+0.0001220703125000, -0.2284545898437500, 0.0001220703125000, -0.1188659667968750,
+0.0001220703125000, -0.1353759765625000, 0.0001220703125000, -0.0817871093750000,
+0.0001525878906250, 0.0155639648437500, 0.0001220703125000, -0.0202636718750000,
+0.0001525878906250, 0.1386413574218750, 0.0001220703125000, 0.1271057128906250,
+0.0001525878906250, 0.1801147460937500, 0.0001525878906250, 0.3051757812500000,
+0.0001220703125000, 0.2618408203125000, 0.0000610351562500, 0.3903808593750000,
+0.0000610351562500, 0.3998107910156250, 0.0000610351562500, 0.4124450683593750,
+0.0000610351562500, 0.4889526367187500, 0.0000000000000000, 0.4703674316406250,
+0.0000610351562500, 0.4694519042968750, 0.0000000000000000, 0.4983215332031250,
+0.0000000000000000, 0.4441833496093750, -0.0000305175781250, 0.3994750976562500,
+-0.0000610351562500, 0.4323425292968750, -0.0000610351562500, 0.2718505859375000,
+-0.0000915527343750, 0.2962036132812500, -0.0000915527343750, 0.2237854003906250,
+-0.0000915527343750, 0.0832824707031250, -0.0000915527343750, 0.1499938964843750,
+-0.0000915527343750, -0.0229187011718750, -0.0000305175781250, -0.0306396484375000,
+-0.0000305175781250, -0.0401306152343750, -0.0000305175781250, -0.1332702636718750,
+0.0000000000000000, -0.1034545898437500, 0.0000305175781250, -0.1183776855468750,
+-0.0000610351562500, -0.1409912109375000, -0.0000915527343750, -0.1105651855468750,
+0.0000000000000000, -0.0869445800781250, -0.0000305175781250, -0.1051940917968750,
+-0.0000915527343750, -0.0419311523437500, 0.0000610351562500, -0.0424194335937500,
+0.0000915527343750, -0.0351562500000000, 0.0000000000000000, 0.0028381347656250,
+0.0000000000000000, -0.0100402832031250, 0.0000915527343750, 0.0014343261718750,
+-0.0000610351562500, 0.0077209472656250, -0.0000610351562500, 0.0022277832031250,
+-0.0000610351562500, 0.0003356933593750, -0.0000915527343750, 0.0025634765625000,
+-0.0000915527343750, -0.0015258789062500, 0.0000915527343750, -0.0029907226562500,
+0.0000915527343750, -0.0011291503906250, 0.0000000000000000, -0.0007324218750000,
+0.0000610351562500, -0.0016784667968750, 0.0001220703125000, 0.0006103515625000,
+-0.0001220703125000, 0.0011901855468750, -0.0001525878906250, 0.0003051757812500,
+-0.0000610351562500, 0.0003967285156250, -0.0001220703125000, 0.0007934570312500,
+-0.0001525878906250, -0.0003662109375000, 0.0000915527343750, -0.0006103515625000,
+0.0000915527343750, -0.0001220703125000, -0.0000305175781250, -0.0001831054687500,
+0.0000610351562500, -0.0004577636718750, 0.0000915527343750, 0.0002746582031250,
+-0.0001525878906250, 0.0004882812500000, -0.0001525878906250, 0.0000610351562500,
+-0.0000305175781250, 0.0000000000000000, -0.0000915527343750, 0.0002441406250000,
+-0.0001831054687500, -0.0002136230468750, 0.0000305175781250, -0.0003967285156250,
+0.0000915527343750, -0.0001525878906250, -0.0000305175781250, 0.0000305175781250,
+0.0000000000000000, 0.0000305175781250, 0.0000000000000000, 0.0000610351562500,
+-0.0001220703125000, 0.0002441406250000, -0.0001831054687500, 0.0000305175781250,
+-0.0000915527343750, -0.0001831054687500, 0.0000000000000000, 0.0000610351562500,
+-0.0000915527343750, -0.0000915527343750, 0.0000610351562500, -0.0003662109375000,
+0.0001220703125000, -0.0000915527343750, -0.0000305175781250, 0.0002136230468750,
+-0.0001525878906250, 0.0000000000000000, -0.0000610351562500, 0.0000305175781250,
+-0.0000610351562500, 0.0002441406250000, -0.0001525878906250, 0.0000000000000000,
+-0.0000305175781250, -0.0003051757812500, 0.0001831054687500, -0.0001220703125000,
+0.0000610351562500, -0.0001220703125000, -0.0000305175781250, -0.0003051757812500,
+0.0001220703125000, -0.0000915527343750, 0.0000000000000000, 0.0002441406250000,
+-0.0003356933593750, 0.0000000000000000, -0.0002441406250000, 0.0000915527343750,
+0.0000915527343750, 0.0002441406250000, -0.0000610351562500, 0.0000000000000000,
+-0.0000610351562500, -0.0002746582031250, 0.0004272460937500, -0.0000915527343750,
+0.0003356933593750, -0.0000610351562500, -0.0005798339843750, -0.0002746582031250,
+-0.0002746582031250, -0.0001525878906250, -0.0002136230468750, 0.0001525878906250,
+-0.0012207031250000, 0.0000305175781250, -0.0010986328125000, 0.0000305175781250,
+0.0011596679687500, 0.0001831054687500, 0.0007934570312500, 0.0000610351562500,
+-0.0002746582031250, -0.0001831054687500, 0.0026855468750000, -0.0000305175781250,
+0.0023193359375000, -0.0000610351562500, -0.0130920410156250, -0.0002136230468750,
+-0.0029296875000000, -0.0000610351562500, -0.0221252441406250, 0.0000915527343750,
+-0.0466308593750000, -0.0000305175781250, -0.0293579101562500, 0.0000610351562500,
+-0.0769348144531250, 0.0000915527343750, -0.0787963867187500, 0.0000305175781250,
+-0.0742797851562500, -0.0000305175781250, -0.1016540527343750, 0.0000000000000000,
+-0.0967712402343750, -0.0000305175781250, -0.0523376464843750, -0.0000915527343750,
+-0.0806579589843750, 0.0000000000000000, -0.0219726562500000, 0.0000305175781250,
+0.0503845214843750, -0.0000305175781250, 0.0073547363281250, 0.0000305175781250,
+0.1644897460937500, 0.0000915527343750, 0.1576232910156250, 0.0000305175781250,
+0.1967163085937500, 0.0000000000000000, 0.3108520507812500, 0.0000000000000000,
+0.2666320800781250, -0.0000305175781250, 0.3291625976562500, -0.0000610351562500,
+0.3556518554687500},
+{-0.0000915527343750, 0.0062255859375000, -0.0000610351562500, -0.0000915527343750,
+0.0000000000000000, 0.0017089843750000, -0.0001525878906250, 0.0034179687500000,
+-0.0001525878906250, -0.0012817382812500, 0.0000610351562500, -0.0024414062500000,
+0.0000305175781250, -0.0007019042968750, -0.0000305175781250, -0.0009155273437500,
+0.0001220703125000, -0.0017395019531250, 0.0001525878906250, 0.0004882812500000,
+-0.0001220703125000, 0.0009765625000000, -0.0000610351562500, 0.0001831054687500,
+0.0000000000000000, 0.0003662109375000, -0.0002136230468750, 0.0007629394531250,
+-0.0002136230468750, -0.0003356933593750, 0.0000305175781250, -0.0005798339843750,
+0.0000000000000000, -0.0001220703125000, -0.0000610351562500, -0.0001525878906250,
+0.0001220703125000, -0.0003967285156250, 0.0000915527343750, 0.0001525878906250,
+-0.0001220703125000, 0.0003662109375000, -0.0000915527343750, 0.0000610351562500,
+0.0000000000000000, -0.0001831054687500, -0.0000610351562500, -0.0001220703125000,
+-0.0000610351562500, -0.0000915527343750, 0.0000305175781250, -0.0002746582031250,
+0.0000610351562500, -0.0001220703125000, -0.0000305175781250, 0.0001831054687500,
+-0.0000305175781250, -0.0000610351562500, 0.0000610351562500, 0.0000305175781250,
+-0.0000915527343750, 0.0002136230468750, -0.0001831054687500, 0.0000305175781250,
+-0.0000610351562500, -0.0003051757812500, 0.0000915527343750, -0.0001220703125000,
+-0.0000305175781250, -0.0000610351562500, -0.0000610351562500, -0.0002441406250000,
+0.0000915527343750, -0.0000915527343750, -0.0000305175781250, 0.0002441406250000,
+-0.0003356933593750, 0.0001220703125000, -0.0001831054687500, 0.0000000000000000,
+0.0001220703125000, 0.0001525878906250, -0.0000915527343750, 0.0000000000000000,
+0.0000000000000000, -0.0003356933593750, 0.0004882812500000, -0.0001831054687500,
+0.0003662109375000, -0.0000305175781250, -0.0005187988281250, -0.0002441406250000,
+-0.0002746582031250, -0.0000915527343750, -0.0001831054687500, 0.0002136230468750,
+-0.0011596679687500, 0.0001220703125000, -0.0010681152343750, -0.0000305175781250,
+0.0010375976562500, 0.0001525878906250, 0.0008239746093750, 0.0000000000000000,
+-0.0009765625000000, -0.0002746582031250, 0.0017700195312500, -0.0001220703125000,
+0.0019836425781250, -0.0000610351562500, -0.0170593261718750, -0.0001525878906250,
+-0.0062255859375000, -0.0000610351562500, -0.0255432128906250, 0.0001220703125000,
+-0.0532836914062500, 0.0000305175781250, -0.0340270996093750, 0.0000000000000000,
+-0.0753784179687500, 0.0001220703125000, -0.0839538574218750, 0.0000610351562500,
+-0.0637512207031250, -0.0001220703125000, -0.0758972167968750, -0.0000915527343750,
+-0.0834045410156250, -0.0000305175781250, 0.0018005371093750, -0.0001220703125000,
+-0.0303344726562500, -0.0000305175781250, 0.0346374511718750, 0.0000915527343750,
+0.1353454589843750, 0.0000305175781250, 0.0736694335937500, 0.0000610351562500,
+0.2203063964843750, 0.0001220703125000, 0.2291259765625000, 0.0000305175781250,
+0.2276000976562500, -0.0000610351562500, 0.3042602539062500, -0.0000305175781250,
+0.2836914062500000, -0.0000610351562500, 0.2469177246093750, -0.0000915527343750,
+0.2793579101562500, -0.0000305175781250, 0.2104187011718750, 0.0000305175781250,
+0.1311645507812500, -0.0000610351562500, 0.1754455566406250, 0.0000305175781250,
+0.0122985839843750, 0.0000610351562500, 0.0175781250000000, 0.0000000000000000,
+-0.0180969238281250, 0.0000000000000000, -0.1295166015625000, 0.0000000000000000,
+-0.0873718261718750, 0.0000000000000000, -0.1574096679687500, -0.0000610351562500,
+-0.1731872558593750, 0.0000000000000000, -0.1615905761718750, 0.0000000000000000,
+-0.1830749511718750, 0.0000305175781250, -0.1819458007812500, 0.0000610351562500,
+-0.1560974121093750, 0.0000305175781250, -0.1632995605468750, 0.0000000000000000,
+-0.1505432128906250, 0.0000000000000000, -0.1236572265625000, 0.0000305175781250,
+-0.1334228515625000, 0.0000305175781250, -0.1114807128906250, -0.0000305175781250,
+-0.1065368652343750, 0.0000000000000000, -0.1076049804687500, 0.0000000000000000,
+-0.0986938476562500, -0.0000305175781250, -0.0987548828125000, 0.0000610351562500,
+-0.1041259765625000, 0.0000305175781250, -0.1011352539062500, 0.0000305175781250,
+-0.1094970703125000, 0.0000000000000000, -0.1206054687500000, 0.0000305175781250,
+-0.1172180175781250, 0.0000000000000000, -0.1343383789062500, -0.0000305175781250,
+-0.1370239257812500, 0.0000000000000000, -0.1453247070312500, 0.0000000000000000,
+-0.1594238281250000, -0.0000305175781250, -0.1578063964843750, 0.0000305175781250,
+-0.1737670898437500, 0.0000000000000000, -0.1806335449218750, 0.0000305175781250,
+-0.1841735839843750, 0.0000305175781250, -0.1932067871093750, 0.0000000000000000,
+-0.1974487304687500, 0.0000000000000000, -0.1971740722656250, -0.0000305175781250,
+-0.2016906738281250, 0.0000000000000000, -0.2017517089843750, 0.0000000000000000,
+-0.1996459960937500, 0.0000000000000000, -0.2028198242187500, 0.0000305175781250,
+-0.1966552734375000, 0.0000000000000000, -0.2013244628906250, -0.0000305175781250,
+-0.1932067871093750, 0.0000000000000000, -0.1835327148437500, 0.0000000000000000,
+-0.1948547363281250, -0.0000305175781250, -0.1774902343750000, -0.0000305175781250,
+-0.1785583496093750, 0.0000000000000000, -0.1792602539062500, 0.0000305175781250,
+-0.1694335937500000, 0.0000305175781250, -0.1757507324218750, 0.0000000000000000,
+-0.1827087402343750, 0.0000305175781250, -0.1708374023437500, 0.0000000000000000,
+-0.1903381347656250, 0.0000000000000000, -0.2056274414062500, 0.0000305175781250,
+-0.1878967285156250, 0.0000000000000000, -0.2621765136718750, -0.0000305175781250,
+-0.2353210449218750, 0.0000000000000000, -0.2696838378906250, 0.0000000000000000,
+-0.3405761718750000, -0.0000305175781250, -0.2938537597656250, 0.0000000000000000,
+-0.3898010253906250, 0.0000610351562500, -0.3955383300781250, 0.0000000000000000,
+-0.3806152343750000, -0.0000610351562500, -0.4309997558593750, 0.0000000000000000,
+-0.4267578125000000, 0.0000000000000000, -0.3458251953125000, -0.0000610351562500,
+-0.4047241210937500, 0.0000000000000000, -0.2902221679687500, 0.0000305175781250,
+-0.1623840332031250, 0.0000000000000000, -0.2633361816406250, -0.0000305175781250,
+0.0700988769531250, 0.0000000000000000, 0.0109863281250000, 0.0000000000000000,
+0.1206970214843750, -0.0000915527343750, 0.3903503417968750, -0.0000610351562500,
+0.2450866699218750, -0.0000610351562500, 0.5116577148437500, -0.0000610351562500,
+0.5362854003906250, 0.0000000000000000, 0.5067443847656250, 0.0000305175781250,
+0.6291809082031250, 0.0000305175781250, 0.6092834472656250, 0.0000000000000000,
+0.5044860839843750, 0.0000000000000000, 0.5722961425781250, -0.0000305175781250,
+0.4512634277343750, -0.0000915527343750, 0.2902526855468750, -0.0000610351562500,
+0.3901062011718750, 0.0000000000000000, 0.0949707031250000, -0.0000610351562500,
+0.1124572753906250, 0.0000000000000000, 0.0639953613281250, 0.0000915527343750,
+-0.1348571777343750, 0.0000610351562500, -0.0592346191406250, 0.0000000000000000,
+-0.1584472656250000, 0.0000915527343750, -0.1935424804687500, -0.0000305175781250,
+-0.1521606445312500, -0.0001525878906250, -0.1613464355468750, -0.0000610351562500,
+-0.1775817871093750, -0.0000610351562500, -0.0918273925781250, -0.0002136230468750,
+-0.1075134277343750, -0.0000610351562500, -0.0841369628906250, 0.0001220703125000,
+-0.0153808593750000, 0.0000000000000000, -0.0460205078125000, 0.0000915527343750,
+-0.0025634765625000, 0.0002136230468750, 0.0086669921875000, 0.0000305175781250,
+-0.0014343261718750, -0.0001831054687500, 0.0075378417968750, 0.0000000000000000,
+0.0086975097656250, -0.0002136230468750, -0.0010681152343750, -0.0003356933593750,
+-0.0011291503906250, -0.0001220703125000, 0.0003356933593750, -0.0000305175781250,
+-0.0036010742187500, -0.0002136230468750, -0.0036010742187500, 0.0002136230468750,
+0.0003051757812500, 0.0002746582031250, 0.0000000000000000, 0.0001220703125000,
+-0.0003967285156250, 0.0001525878906250, 0.0015869140625000, 0.0002441406250000,
+0.0015258789062500, -0.0002746582031250, -0.0003356933593750, -0.0002746582031250,
+-0.0001831054687500, -0.0001831054687500, 0.0002746582031250, -0.0004577636718750,
+-0.0004882812500000, -0.0004882812500000, -0.0007324218750000, 0.0001220703125000,
+0.0004882812500000, -0.0000305175781250, 0.0006103515625000, 0.0000000000000000,
+0.0001220703125000, 0.0005798339843750, 0.0003356933593750, -0.0001831054687500,
+-0.0006713867187500, -0.0002746582031250, 0.0005187988281250, -0.0004882812500000,
+0.0004577636718750, -0.0001525878906250, 0.0000000000000000, 0.0001831054687500,
+0.0000610351562500, -0.0000610351562500, 0.0003967285156250, 0.0003967285156250,
+-0.0004882812500000, 0.0004882812500000, -0.0007934570312500, 0.0003967285156250,
+-0.0002136230468750, 0.0004882812500000, -0.0000915527343750, 0.0005187988281250,
+-0.0004882812500000, 0.0000305175781250, 0.0004882812500000, 0.0003356933593750,
+0.0007629394531250, -0.0002746582031250, 0.0001831054687500, -0.0011291503906250,
+0.0001525878906250, -0.0003967285156250, 0.0005493164062500, -0.0015563964843750,
+-0.0003967285156250, -0.0023803710937500, -0.0006713867187500, -0.0002746582031250,
+-0.0002136230468750, 0.0004882812500000, -0.0002441406250000, -0.0013427734375000,
+-0.0005187988281250, 0.0044250488281250, 0.0003051757812500, 0.0061645507812500,
+0.0004272460937500, -0.0014038085937500, 0.0000610351562500, 0.0011901855468750,
+0.0001831054687500, 0.0053405761718750, 0.0003967285156250, -0.0375976562500000,
+-0.0002746582031250, -0.0163879394531250, -0.0003967285156250, -0.0508728027343750,
+-0.0000610351562500, -0.1047668457031250, -0.0001525878906250, -0.0677185058593750,
+-0.0003356933593750, -0.1493225097656250, 0.0001525878906250, -0.1591186523437500,
+0.0003051757812500, -0.1310119628906250, 0.0000305175781250, -0.1663208007812500,
+0.0000915527343750, -0.1718139648437500, 0.0002441406250000, -0.0350952148437500,
+-0.0002746582031250, -0.1026306152343750, -0.0003356933593750, 0.0031433105468750,
+-0.0001525878906250, 0.1657104492187500, -0.0002136230468750, 0.0512084960937500,
+-0.0003356933593750, 0.3208312988281250, 0.0000305175781250, 0.3054199218750000,
+0.0000610351562500, 0.3223266601562500, -0.0000305175781250, 0.4883117675781250,
+0.0000000000000000, 0.4227294921875000, 0.0000610351562500, 0.4466247558593750,
+-0.0000610351562500, 0.5031433105468750, -0.0001525878906250, 0.4203796386718750,
+-0.0000305175781250, 0.3656005859375000, 0.0000305175781250, 0.4327392578125000,
+-0.0000915527343750, 0.2131347656250000, 0.0001525878906250, 0.2613525390625000,
+0.0002136230468750, 0.1885681152343750, 0.0001220703125000, 0.0113830566406250,
+0.0000610351562500, 0.1111450195312500, 0.0001220703125000, -0.1012268066406250,
+0.0000305175781250, -0.0922241210937500, -0.0000610351562500, -0.1169433593750000,
+0.0000305175781250, -0.2468872070312500, 0.0000610351562500, -0.1871643066406250,
+0.0000000000000000, -0.2630004882812500, 0.0000610351562500, -0.2810974121093750,
+0.0000915527343750, -0.2612304687500000, 0.0000305175781250, -0.2826232910156250,
+-0.0000915527343750, -0.2863464355468750, 0.0000305175781250, -0.2443847656250000,
+0.0000000000000000, -0.2626342773437500, -0.0000915527343750, -0.2483825683593750,
+-0.0000305175781250, -0.2086181640625000, 0.0000305175781250, -0.2345275878906250,
+0.0000000000000000, -0.2070007324218750, -0.0000305175781250, -0.1974792480468750,
+0.0000305175781250, -0.2130737304687500, 0.0000000000000000, -0.2087402343750000,
+0.0000000000000000},
+{-0.0006103515625000, 0.0004577636718750, 0.0013427734375000, 0.0003356933593750,
+0.0012512207031250, 0.0000305175781250, -0.0003051757812500, 0.0002441406250000,
+-0.0000915527343750, 0.0000915527343750, 0.0002136230468750, -0.0005493164062500,
+-0.0005187988281250, -0.0001220703125000, 0.0010070800781250, 0.0005187988281250,
+-0.0002746582031250, 0.0004272460937500, 0.0000610351562500, 0.0003051757812500,
+0.0002441406250000, 0.0004882812500000, -0.0001831054687500, 0.0005798339843750,
+-0.0004272460937500, -0.0003662109375000, 0.0003662109375000, 0.0000000000000000,
+0.0005493164062500, -0.0004272460937500, 0.0000305175781250, -0.0016174316406250,
+-0.0001220703125000, -0.0009460449218750, 0.0002746582031250, -0.0009460449218750,
+-0.0004577636718750, -0.0021057128906250, -0.0007324218750000, 0.0001525878906250,
+-0.0002136230468750, 0.0025939941406250, 0.0000305175781250, 0.0003051757812500,
+-0.0003662109375000, 0.0026245117187500, 0.0003662109375000, 0.0078430175781250,
+0.0006713867187500, -0.0059509277343750, 0.0001525878906250, -0.0101623535156250,
+0.0000000000000000, -0.0001525878906250, 0.0003662109375000, -0.0609130859375000,
+-0.0003051757812500, -0.0397949218750000, -0.0005493164062500, -0.0712585449218750,
+-0.0001525878906250, -0.1373596191406250, -0.0000305175781250, -0.0979003906250000,
+-0.0002746582031250, -0.1443176269531250, 0.0002746582031250, -0.1734924316406250,
+0.0004882812500000, -0.1144104003906250, 0.0000915527343750, -0.1036071777343750,
+-0.0000610351562500, -0.1397399902343750, 0.0002136230468750, 0.0746154785156250,
+-0.0003356933593750, 0.0101928710937500, -0.0005493164062500, 0.1067504882812500,
+-0.0002136230468750, 0.3122558593750000, -0.0001220703125000, 0.1931152343750000,
+-0.0003356933593750, 0.4007568359375000, 0.0001220703125000, 0.4322814941406250,
+0.0002441406250000, 0.3775634765625000, 0.0000305175781250, 0.4487304687500000,
+0.0000610351562500, 0.4570312500000000, 0.0001525878906250, 0.2973327636718750,
+-0.0000915527343750, 0.3659973144531250, -0.0002441406250000, 0.2707519531250000,
+-0.0000305175781250, 0.1003723144531250, 0.0000610351562500, 0.2116394042968750,
+-0.0000305175781250, -0.0206604003906250, 0.0001831054687500, -0.0113220214843750,
+0.0002441406250000, -0.0277404785156250, 0.0000915527343750, -0.1629333496093750,
+0.0000000000000000, -0.1050415039062500, 0.0000915527343750, -0.1687927246093750,
+-0.0000915527343750, -0.1861572265625000, -0.0001831054687500, -0.1756286621093750,
+-0.0000610351562500, -0.1934814453125000, 0.0000000000000000, -0.1937561035156250,
+-0.0000915527343750, -0.1913146972656250, 0.0000610351562500, -0.1891479492187500,
+0.0000915527343750, -0.1994323730468750, 0.0000000000000000, -0.2012023925781250,
+-0.0000305175781250, -0.1934814453125000, 0.0000000000000000, -0.2383422851562500,
+0.0000000000000000, -0.2196960449218750, -0.0000610351562500, -0.2556457519531250,
+0.0000000000000000, -0.3038024902343750, 0.0001220703125000, -0.2633056640625000,
+0.0000610351562500, -0.3670349121093750, 0.0000610351562500, -0.3466186523437500,
+0.0000915527343750, -0.3691406250000000, 0.0000610351562500, -0.4454650878906250,
+0.0000000000000000, -0.3998107910156250, 0.0000305175781250, -0.4647521972656250,
+0.0000305175781250, -0.4701843261718750, -0.0000305175781250, -0.4540100097656250,
+0.0000000000000000, -0.4806518554687500, 0.0000305175781250, -0.4784851074218750,
+0.0000610351562500, -0.4330749511718750, -0.0000610351562500, -0.4600219726562500,
+-0.0000305175781250, -0.4227600097656250, -0.0000305175781250, -0.3711853027343750,
+-0.0000915527343750, -0.4151000976562500, -0.0000610351562500, -0.2999267578125000,
+-0.0000610351562500, -0.3309936523437500, 0.0000000000000000, -0.2987976074218750,
+-0.0000915527343750, -0.2084350585937500, -0.0000915527343750, -0.2677307128906250,
+-0.0000610351562500, -0.1611022949218750, -0.0001220703125000, -0.1781616210937500,
+-0.0001220703125000, -0.1651306152343750, -0.0001220703125000, -0.0999755859375000,
+-0.0000915527343750, -0.1506652832031250, -0.0000915527343750, -0.0920410156250000,
+-0.0000915527343750, -0.1023864746093750, -0.0001220703125000, -0.1080627441406250,
+-0.0001525878906250, -0.0810241699218750, -0.0000915527343750, -0.1103515625000000,
+-0.0000915527343750, -0.0968627929687500, -0.0000915527343750, -0.1033935546875000,
+-0.0001220703125000, -0.1210021972656250, -0.0001220703125000, -0.1231689453125000,
+-0.0001220703125000, -0.1343078613281250, -0.0001220703125000, -0.1687622070312500,
+-0.0000915527343750, -0.1755676269531250, -0.0000610351562500, -0.1989746093750000,
+-0.0000915527343750, -0.2328491210937500, -0.0001220703125000, -0.2375183105468750,
+-0.0000610351562500, -0.2781982421875000, -0.0000915527343750, -0.2805786132812500,
+-0.0000915527343750, -0.3013610839843750, -0.0000610351562500, -0.3349914550781250,
+-0.0000610351562500, -0.3263549804687500, -0.0000305175781250, -0.3655395507812500,
+-0.0000305175781250, -0.3664855957031250, -0.0000305175781250, -0.3680725097656250,
+0.0000000000000000, -0.3916931152343750, 0.0000305175781250, -0.3879699707031250,
+-0.0000305175781250, -0.3820190429687500, 0.0000000000000000, -0.3925781250000000,
+0.0000000000000000, -0.3741149902343750, 0.0000305175781250, -0.3554382324218750,
+0.0000000000000000, -0.3671569824218750, 0.0000000000000000, -0.3097839355468750,
+0.0000610351562500, -0.3173828125000000, 0.0000610351562500, -0.2924499511718750,
+0.0000305175781250, -0.2423706054687500, 0.0000610351562500, -0.2633361816406250,
+0.0000610351562500, -0.2049255371093750, 0.0000000000000000, -0.1999816894531250,
+0.0000000000000000, -0.2020874023437500, 0.0000305175781250, -0.1745605468750000,
+-0.0000305175781250, -0.1803283691406250, -0.0000305175781250, -0.1916503906250000,
+0.0000305175781250, -0.1842346191406250, -0.0000610351562500, -0.1968994140625000,
+0.0000000000000000, -0.2193603515625000, 0.0000610351562500, -0.2097473144531250,
+0.0000000000000000, -0.2123107910156250, -0.0000305175781250, -0.2350158691406250,
+0.0000000000000000, -0.1920166015625000, 0.0000000000000000, -0.1632385253906250,
+-0.0000610351562500, -0.1933288574218750, 0.0000000000000000, -0.0401611328125000,
+0.0000610351562500, -0.0721740722656250, 0.0000000000000000, 0.0021972656250000,
+0.0000305175781250, 0.1486816406250000, 0.0001220703125000, 0.0788574218750000,
+0.0000610351562500, 0.2412719726562500, 0.0000000000000000, 0.2653198242187500,
+0.0000610351562500, 0.2403869628906250, 0.0000000000000000, 0.3052978515625000,
+-0.0001220703125000, 0.3017272949218750, -0.0001220703125000, 0.1968383789062500,
+0.0000000000000000, 0.2411193847656250, -0.0001220703125000, 0.1690063476562500,
+-0.0000305175781250, 0.0431518554687500, 0.0000915527343750, 0.1099243164062500,
+0.0000305175781250, -0.0361022949218750, -0.0000610351562500, -0.0507202148437500,
+-0.0000305175781250, -0.0375366210937500, -0.0000305175781250, -0.0989074707031250,
+-0.0001831054687500, -0.0877990722656250, -0.0001525878906250, -0.0556030273437500,
+0.0000610351562500, -0.0718383789062500, -0.0000915527343750, -0.0517883300781250,
+-0.0000610351562500, -0.0115356445312500, 0.0001525878906250, -0.0297546386718750,
+0.0000610351562500, -0.0035705566406250, -0.0000610351562500, 0.0032958984375000,
+0.0000305175781250, -0.0027770996093750, 0.0000305175781250, 0.0019226074218750,
+-0.0002441406250000, 0.0027465820312500, -0.0001220703125000, -0.0011596679687500,
+-0.0000305175781250, -0.0020141601562500, -0.0001525878906250, -0.0005493164062500,
+-0.0000915527343750, -0.0006408691406250, 0.0000915527343750, -0.0014648437500000,
+0.0000610351562500, 0.0003356933593750, -0.0000915527343750, 0.0007629394531250,
+-0.0000305175781250, 0.0001220703125000, 0.0000000000000000, -0.0000610351562500,
+-0.0000305175781250, -0.0000915527343750, -0.0000610351562500, -0.0000610351562500,
+0.0000000000000000, -0.0003967285156250, 0.0000915527343750, -0.0001525878906250,
+-0.0000305175781250, 0.0002136230468750, -0.0001831054687500, -0.0000610351562500,
+0.0000000000000000, -0.0000305175781250, -0.0000305175781250, 0.0001831054687500,
+-0.0001831054687500, 0.0000305175781250, 0.0000000000000000, -0.0003051757812500,
+0.0003662109375000, -0.0001525878906250, 0.0001831054687500, 0.0000610351562500,
+-0.0003051757812500, -0.0001220703125000, 0.0000915527343750, -0.0000610351562500,
+-0.0002746582031250, 0.0002746582031250, -0.0013427734375000, 0.0001831054687500,
+-0.0009460449218750, -0.0001220703125000, 0.0004272460937500, -0.0000610351562500,
+0.0000610351562500, -0.0000305175781250, -0.0010375976562500, -0.0003356933593750,
+0.0017089843750000, -0.0003356933593750, 0.0014648437500000, 0.0000305175781250,
+-0.0151062011718750, -0.0000915527343750, -0.0050048828125000, -0.0000915527343750,
+-0.0227661132812500, 0.0002441406250000, -0.0491333007812500, 0.0002136230468750,
+-0.0308532714843750, -0.0000915527343750, -0.0607910156250000, -0.0000610351562500,
+-0.0745544433593750, 0.0000000000000000, -0.0429992675781250, -0.0002441406250000,
+-0.0408325195312500, -0.0002441406250000, -0.0589904785156250, 0.0000610351562500,
+0.0583190917968750, -0.0000305175781250, 0.0282592773437500, -0.0000610351562500,
+0.0741577148437500, 0.0001831054687500, 0.1844482421875000, 0.0001525878906250,
+0.1276245117187500, -0.0000610351562500, 0.1999511718750000, 0.0000000000000000,
+0.2334899902343750, 0.0000000000000000, 0.1798400878906250, -0.0001831054687500,
+0.1707153320312500, -0.0001525878906250, 0.1996459960937500, 0.0000000000000000,
+0.0626525878906250, -0.0000610351562500, 0.0840148925781250, -0.0000610351562500,
+0.0394287109375000, 0.0001220703125000, -0.0739135742187500, 0.0000915527343750,
+-0.0218505859375000, -0.0000305175781250, -0.0999145507812500, 0.0000305175781250,
+-0.1212463378906250, 0.0000305175781250, -0.1003417968750000, -0.0000915527343750,
+-0.1129455566406250, -0.0000915527343750, -0.1164550781250000, 0.0000000000000000,
+-0.0897827148437500, -0.0000915527343750, -0.0850830078125000, 0.0000000000000000,
+-0.0967407226562500, 0.0000915527343750, -0.0880432128906250, 0.0000305175781250,
+-0.0844726562500000, -0.0000305175781250, -0.1486511230468750, 0.0000305175781250,
+-0.1286621093750000, 0.0000000000000000, -0.1735839843750000, -0.0000915527343750,
+-0.2462158203125000, -0.0000305175781250, -0.2090759277343750, 0.0000000000000000,
+-0.3103332519531250, -0.0000305175781250, -0.3142395019531250, 0.0000000000000000,
+-0.3230895996093750, 0.0000305175781250, -0.3815917968750000, 0.0000305175781250,
+-0.3625488281250000, 0.0000305175781250, -0.3675842285156250, 0.0000915527343750,
+-0.3828125000000000, 0.0000305175781250, -0.3587341308593750, 0.0000000000000000,
+-0.3362121582031250, 0.0000000000000000, -0.3448486328125000, 0.0000305175781250,
+-0.2865905761718750, 0.0000000000000000, -0.2882995605468750, 0.0000000000000000,
+-0.2570495605468750, 0.0000305175781250, -0.2027893066406250, 0.0000305175781250,
+-0.2171325683593750, 0.0000610351562500, -0.1494750976562500, 0.0000305175781250,
+-0.1375732421875000, 0.0000305175781250, -0.1194152832031250, 0.0000305175781250,
+-0.0771789550781250, 0.0000305175781250, -0.0817565917968750, 0.0000000000000000,
+-0.0472412109375000, 0.0000000000000000, -0.0355224609375000, 0.0000305175781250,
+-0.0216369628906250, 0.0000610351562500, -0.0003356933593750, 0.0000305175781250,
+-0.0019836425781250, 0.0000610351562500, 0.0206604003906250, 0.0000610351562500,
+0.0346984863281250, 0.0000305175781250, 0.0368652343750000, 0.0000610351562500,
+0.0441589355468750, 0.0000305175781250, 0.0524597167968750, 0.0000000000000000,
+0.0453491210937500},
+{-0.0006408691406250, -0.0000915527343750, -0.0000305175781250, 0.0000915527343750,
+-0.0006713867187500, 0.0000000000000000, -0.0007324218750000, -0.0002746582031250,
+0.0019226074218750, -0.0002441406250000, 0.0011596679687500, 0.0000915527343750,
+-0.0109252929687500, 0.0000000000000000, -0.0021972656250000, -0.0000610351562500,
+-0.0185546875000000, 0.0002136230468750, -0.0410156250000000, 0.0002136230468750,
+-0.0255126953125000, -0.0001220703125000, -0.0546569824218750, -0.0001220703125000,
+-0.0675354003906250, -0.0000610351562500, -0.0370788574218750, -0.0002746582031250,
+-0.0353393554687500, -0.0002746582031250, -0.0541687011718750, 0.0001220703125000,
+0.0569763183593750, 0.0000305175781250, 0.0273742675781250, 0.0000000000000000,
+0.0666503906250000, 0.0001220703125000, 0.1731262207031250, 0.0001525878906250,
+0.1228637695312500, -0.0001525878906250, 0.1707763671875000, -0.0001220703125000,
+0.2087402343750000, -0.0000610351562500, 0.1488037109375000, -0.0002441406250000,
+0.1203002929687500, -0.0002441406250000, 0.1597290039062500, 0.0000305175781250,
+0.0133972167968750, 0.0000000000000000, 0.0273437500000000, -0.0000305175781250,
+0.0054931640625000, 0.0000915527343750, -0.0950317382812500, 0.0001525878906250,
+-0.0571289062500000, -0.0000915527343750, -0.0894470214843750, -0.0000305175781250,
+-0.1093750000000000, 0.0000000000000000, -0.0892028808593750, -0.0001525878906250,
+-0.0773315429687500, -0.0001525878906250, -0.0882263183593750, 0.0000305175781250,
+-0.0804443359375000, -0.0000305175781250, -0.0635375976562500, -0.0000305175781250,
+-0.0983581542968750, 0.0001220703125000, -0.1232910156250000, 0.0000610351562500,
+-0.1011352539062500, 0.0000000000000000, -0.1986389160156250, -0.0000305175781250,
+-0.1914367675781250, -0.0000305175781250, -0.2163085937500000, -0.0000915527343750,
+-0.2919006347656250, -0.0000610351562500, -0.2625732421875000, 0.0000000000000000,
+-0.2834777832031250, -0.0000610351562500, -0.3080139160156250, -0.0000610351562500,
+-0.2620239257812500, 0.0000915527343750, -0.2329101562500000, 0.0000610351562500,
+-0.2574157714843750, 0.0000000000000000, -0.1390686035156250, 0.0000305175781250,
+-0.1531677246093750, 0.0000000000000000, -0.1072998046875000, -0.0000305175781250,
+-0.0065307617187500, 0.0000000000000000, -0.0426940917968750, -0.0000305175781250,
+0.0642395019531250, -0.0000305175781250, 0.0725708007812500, 0.0000000000000000,
+0.0829772949218750, 0.0000610351562500, 0.1411132812500000, 0.0000610351562500,
+0.1268310546875000, 0.0000000000000000, 0.1528015136718750, 0.0000000000000000,
+0.1601257324218750, -0.0000305175781250, 0.1538391113281250, -0.0000610351562500,
+0.1596374511718750, 0.0000000000000000, 0.1617431640625000, 0.0000305175781250,
+0.1513061523437500, -0.0000305175781250, 0.1523437500000000, 0.0000000000000000,
+0.1463928222656250, 0.0000305175781250, 0.1354980468750000, 0.0000000000000000,
+0.1372680664062500, -0.0000305175781250, 0.1239318847656250, 0.0000305175781250,
+0.1219787597656250, 0.0000305175781250, 0.1158447265625000, 0.0000000000000000,
+0.1071166992187500, -0.0000305175781250, 0.1095275878906250, 0.0000000000000000,
+0.0902709960937500, 0.0000000000000000, 0.0889587402343750, 0.0000000000000000,
+0.0854492187500000, 0.0000000000000000, 0.0754394531250000, 0.0000000000000000,
+0.0780029296875000, 0.0000000000000000, 0.0641174316406250, -0.0000305175781250,
+0.0654907226562500, 0.0000000000000000, 0.0607604980468750, 0.0000000000000000,
+0.0501708984375000, 0.0000000000000000, 0.0565490722656250, -0.0000305175781250,
+0.0480346679687500, 0.0000305175781250, 0.0444335937500000, 0.0000610351562500,
+0.0481567382812500, 0.0000000000000000, 0.0461120605468750, 0.0000000000000000,
+0.0438537597656250, 0.0000000000000000, 0.0603027343750000, 0.0000000000000000,
+0.0527343750000000, 0.0000000000000000, 0.0622558593750000, 0.0000000000000000,
+0.0797424316406250, 0.0000305175781250, 0.0681152343750000, 0.0000305175781250,
+0.0950317382812500, 0.0000000000000000, 0.0936279296875000, 0.0000305175781250,
+0.0944519042968750, 0.0000000000000000, 0.1144714355468750, -0.0000305175781250,
+0.1077270507812500, 0.0000610351562500, 0.0870361328125000, 0.0000000000000000,
+0.1135864257812500, 0.0000000000000000, 0.0704650878906250, 0.0000610351562500,
+0.0333251953125000, 0.0000610351562500, 0.0765075683593750, -0.0000305175781250,
+-0.0815734863281250, -0.0000305175781250, -0.0341491699218750, 0.0000000000000000,
+-0.1152954101562500, -0.0000305175781250, -0.2669067382812500, 0.0000000000000000,
+-0.1726074218750000, 0.0000610351562500, -0.3903808593750000, 0.0000915527343750,
+-0.3747863769531250, 0.0000305175781250, -0.3897094726562500, 0.0000305175781250,
+-0.5310058593750000, 0.0000610351562500, -0.4781188964843750, -0.0000915527343750,
+-0.4888000488281250, -0.0000915527343750, -0.5339050292968750, 0.0000000000000000,
+-0.4636230468750000, 0.0000610351562500, -0.4058227539062500, -0.0000305175781250,
+-0.4553222656250000, 0.0000915527343750, -0.2912902832031250, 0.0000915527343750,
+-0.3075866699218750, 0.0000305175781250, -0.2795104980468750, -0.0000305175781250,
+-0.1611938476562500, 0.0000305175781250, -0.2052307128906250, -0.0000305175781250,
+-0.1661682128906250, -0.0001220703125000, -0.1387634277343750, -0.0000610351562500,
+-0.1756591796875000, 0.0000000000000000, -0.1967773437500000, -0.0000305175781250,
+-0.1766967773437500, 0.0000915527343750, -0.2356567382812500, 0.0001525878906250,
+-0.2520446777343750, 0.0000610351562500, -0.2178649902343750, 0.0000000000000000,
+-0.2344665527343750, 0.0000305175781250, -0.2517395019531250, -0.0001220703125000,
+-0.0840454101562500, -0.0001525878906250, -0.1432189941406250, -0.0000915527343750,
+-0.0336914062500000, -0.0000610351562500, 0.1465759277343750, -0.0001220703125000,
+0.0347595214843750, 0.0000610351562500, 0.3022155761718750, 0.0000915527343750,
+0.3053588867187500, 0.0000305175781250, 0.3038024902343750, 0.0000610351562500,
+0.4550476074218750, 0.0000915527343750, 0.4118652343750000, -0.0001220703125000,
+0.3473510742187500, -0.0001525878906250, 0.4221496582031250, -0.0000915527343750,
+0.3047180175781250, -0.0002136230468750, 0.1602478027343750, -0.0002441406250000,
+0.2597351074218750, 0.0000610351562500, 0.0010070800781250, 0.0000000000000000,
+0.0050659179687500, 0.0000000000000000, -0.0057678222656250, 0.0002746582031250,
+-0.1555175781250000, 0.0002136230468750, -0.1069946289062500, 0.0000000000000000,
+-0.1122741699218750, 0.0000915527343750, -0.1511535644531250, -0.0000610351562500,
+-0.1043090820312500, -0.0003356933593750, -0.0532836914062500, -0.0002136230468750,
+-0.0872802734375000, -0.0002441406250000, -0.0165405273437500, -0.0004272460937500,
+-0.0087585449218750, -0.0001220703125000, -0.0162353515625000, 0.0000915527343750,
+0.0100097656250000, -0.0001220703125000, 0.0056762695312500, 0.0004577636718750,
+-0.0001525878906250, 0.0004882812500000, 0.0022888183593750, 0.0003356933593750,
+0.0021057128906250, 0.0004577636718750, -0.0035095214843750, -0.0012207031250000,
+0.0039062500000000, 0.0010986328125000, -0.0003967285156250, -0.0007324218750000,
+0.0027465820312500, 0.0003662109375000, 0.0017089843750000, 0.0043945312500000,
+-0.0024108886718750, 0.0027160644531250, -0.0018920898437500, -0.0069580078125000,
+-0.0000305175781250, 0.0036621093750000, -0.0004272460937500, -0.0176086425781250,
+-0.0006103515625000, -0.0372924804687500, 0.0005798339843750, -0.0190429687500000,
+0.0006408691406250, -0.0885620117187500, -0.0001525878906250, -0.0801696777343750,
+-0.0002136230468750, -0.0868835449218750, 0.0000915527343750, -0.1429443359375000,
+0.0000000000000000, -0.1222839355468750, -0.0001525878906250, -0.0770263671875000,
+0.0001831054687500, -0.1347045898437500, 0.0004272460937500, -0.0307922363281250,
+0.0000305175781250, 0.0747375488281250, -0.0003051757812500, -0.0204162597656250,
+0.0000000000000000, 0.2587280273437500, -0.0003051757812500, 0.2368469238281250,
+-0.0006103515625000, 0.2540588378906250, -0.0002136230468750, 0.4395446777343750,
+0.0001525878906250, 0.3749694824218750, -0.0000915527343750, 0.3348083496093750,
+0.0002441406250000, 0.4243774414062500, 0.0004882812500000, 0.2942810058593750,
+0.0001220703125000, 0.1514587402343750, -0.0001525878906250, 0.2632751464843750,
+0.0000915527343750, -0.0409851074218750, -0.0001220703125000, -0.0304260253906250,
+-0.0003356933593750, -0.0339050292968750, -0.0000610351562500, -0.2094421386718750,
+0.0002136230468750, -0.1537170410156250, 0.0000000000000000, -0.1254272460937500,
+0.0001220703125000, -0.1799621582031250, 0.0003051757812500, -0.1230468750000000,
+0.0000915527343750, -0.0356140136718750, -0.0002136230468750, -0.0928955078125000,
+-0.0000610351562500, -0.0426635742187500, -0.0000915527343750, 0.0031127929687500,
+-0.0003356933593750, -0.0781860351562500, -0.0000610351562500, -0.1147766113281250,
+0.0002136230468750, -0.0566101074218750, 0.0000610351562500, -0.2801513671875000,
+0.0001525878906250, -0.2333068847656250, 0.0003051757812500, -0.2895507812500000,
+0.0001525878906250, -0.4680175781250000, -0.0000610351562500, -0.3750000000000000,
+0.0000610351562500, -0.4841613769531250, -0.0000305175781250, -0.5242004394531250,
+-0.0001220703125000, -0.4643249511718750, -0.0000305175781250, -0.4760437011718750,
+0.0000610351562500, -0.5036010742187500, 0.0000305175781250, -0.3280639648437500,
+-0.0000915527343750, -0.3854370117187500, -0.0000305175781250, -0.2917785644531250,
+-0.0000305175781250, -0.1244201660156250, -0.0002441406250000, -0.2310180664062500,
+-0.0001525878906250, 0.0328063964843750, -0.0000915527343750, 0.0108947753906250,
+-0.0001831054687500, 0.0527343750000000, -0.0001220703125000, 0.2251281738281250,
+0.0000000000000000, 0.1384582519531250, -0.0000610351562500, 0.2473144531250000,
+-0.0000610351562500, 0.2721862792968750, -0.0000305175781250, 0.2324829101562500,
+-0.0000305175781250, 0.2603759765625000, -0.0000305175781250, 0.2725830078125000,
+-0.0000915527343750, 0.1771240234375000, 0.0000915527343750, 0.2175598144531250,
+0.0000305175781250, 0.1583557128906250, 0.0000610351562500, 0.0618286132812500,
+0.0000915527343750, 0.1380615234375000, 0.0001220703125000, -0.0220336914062500,
+0.0000610351562500, -0.0027160644531250, 0.0000305175781250, -0.0210266113281250,
+0.0000610351562500, -0.1211242675781250, 0.0000610351562500, -0.0663452148437500,
+0.0000610351562500, -0.1444702148437500, 0.0000610351562500, -0.1477355957031250,
+0.0001220703125000, -0.1393432617187500, 0.0000305175781250, -0.1759948730468750,
+-0.0000305175781250, -0.1669006347656250, 0.0000000000000000, -0.1611633300781250,
+-0.0000305175781250, -0.1735839843750000, -0.0000610351562500, -0.1606140136718750,
+0.0000000000000000, -0.1468505859375000, -0.0000305175781250, -0.1655273437500000,
+-0.0000915527343750, -0.1379699707031250, -0.0000305175781250, -0.1475524902343750,
+0.0000000000000000, -0.1407165527343750, -0.0000305175781250, -0.1206359863281250,
+-0.0000915527343750, -0.1410522460937500, 0.0000000000000000, -0.1121826171875000,
+-0.0000305175781250, -0.1185913085937500, -0.0000305175781250, -0.1162719726562500,
+-0.0000305175781250, -0.0991210937500000, -0.0000305175781250, -0.1137084960937500,
+0.0000000000000000, -0.0929565429687500, -0.0000305175781250, -0.0987243652343750,
+-0.0000305175781250, -0.0926513671875000, -0.0000305175781250, -0.0748901367187500,
+0.0000000000000000, -0.0831604003906250, 0.0000000000000000, -0.0541992187500000,
+0.0000000000000000, -0.0565795898437500, 0.0000000000000000, -0.0428161621093750,
+0.0000610351562500},
+{-0.0003051757812500, -0.1249389648437500, -0.0000915527343750, -0.2243652343750000,
+0.0002441406250000, -0.2242431640625000, 0.0000305175781250, -0.0652770996093750,
+0.0000000000000000, -0.1141967773437500, 0.0002441406250000, -0.0751953125000000,
+0.0000305175781250, 0.0755004882812500, -0.0003967285156250, 0.0110778808593750,
+-0.0002136230468750, -0.0132141113281250, -0.0001220703125000, 0.0736999511718750,
+-0.0003356933593750, -0.0614013671875000, -0.0001220703125000, -0.1955871582031250,
+0.0003051757812500, -0.0763549804687500, 0.0001220703125000, -0.3868713378906250,
+0.0000915527343750, -0.3716125488281250, 0.0002746582031250, -0.3756713867187500,
+0.0000610351562500, -0.5619812011718750, -0.0002136230468750, -0.4996032714843750,
+-0.0000305175781250, -0.4526062011718750, -0.0000610351562500, -0.5405883789062500,
+-0.0002136230468750, -0.4107360839843750, -0.0001220703125000, -0.2736206054687500,
+0.0000610351562500, -0.3926696777343750, 0.0000000000000000, -0.0711975097656250,
+-0.0000915527343750, -0.1112060546875000, -0.0000305175781250, -0.0523986816406250,
+-0.0000305175781250, 0.1732177734375000, -0.0002136230468750, 0.0679931640625000,
+-0.0001831054687500, 0.2198181152343750, 0.0000000000000000, 0.2501831054687500,
+-0.0001220703125000, 0.2120971679687500, -0.0000610351562500, 0.2607116699218750,
+0.0001831054687500, 0.2629699707031250, 0.0001220703125000, 0.1566467285156250,
+0.0000305175781250, 0.2008972167968750, 0.0000610351562500, 0.1348266601562500,
+0.0000610351562500, 0.0228881835937500, 0.0000000000000000, 0.1008911132812500,
+0.0000000000000000, -0.0582275390625000, 0.0001220703125000, -0.0526428222656250,
+0.0000915527343750, -0.0474243164062500, 0.0000610351562500, -0.1367492675781250,
+0.0000610351562500, -0.1051635742187500, 0.0000610351562500, -0.0786132812500000,
+-0.0000915527343750, -0.1271667480468750, -0.0000610351562500, -0.0533142089843750,
+-0.0000610351562500, 0.0140991210937500, -0.0001220703125000, -0.0591735839843750,
+-0.0000915527343750, 0.1584777832031250, -0.0000915527343750, 0.1027832031250000,
+-0.0000915527343750, 0.1751403808593750, -0.0000915527343750, 0.3425903320312500,
+-0.0001525878906250, 0.2242431640625000, -0.0001220703125000, 0.4725952148437500,
+-0.0001220703125000, 0.4266967773437500, -0.0001525878906250, 0.4716186523437500,
+-0.0001831054687500, 0.6446228027343750, -0.0001831054687500, 0.5410766601562500,
+-0.0001831054687500, 0.7119750976562500, -0.0001525878906250, 0.6938476562500000,
+-0.0000915527343750, 0.7097473144531250, -0.0001525878906250, 0.8084411621093750,
+-0.0001525878906250, 0.7380981445312500, -0.0001525878906250, 0.8244934082031250,
+-0.0001831054687500, 0.8116455078125000, -0.0001831054687500, 0.8027343750000000,
+-0.0001525878906250, 0.8423156738281250, -0.0001525878906250, 0.8046569824218750,
+-0.0001525878906250, 0.8206481933593750, -0.0001220703125000, 0.8116760253906250,
+-0.0001220703125000, 0.7872009277343750, -0.0001525878906250, 0.7835388183593750,
+-0.0001220703125000, 0.7691040039062500, -0.0001525878906250, 0.7163085937500000,
+-0.0001220703125000, 0.7120361328125000, -0.0001525878906250, 0.6754455566406250,
+-0.0001525878906250, 0.6191101074218750, -0.0001525878906250, 0.6120300292968750,
+-0.0001525878906250, 0.5339355468750000, -0.0001525878906250, 0.5338745117187500,
+-0.0001220703125000, 0.4855651855468750, -0.0001220703125000, 0.4146118164062500,
+-0.0001525878906250, 0.4328918457031250, -0.0001220703125000, 0.3154602050781250,
+-0.0001220703125000, 0.3254699707031250, -0.0001220703125000, 0.2801513671875000,
+-0.0001220703125000, 0.1854248046875000, -0.0001525878906250, 0.2179870605468750,
+-0.0001525878906250, 0.0966491699218750, -0.0000915527343750, 0.1003112792968750,
+-0.0000915527343750, 0.0716857910156250, -0.0000915527343750, -0.0145568847656250,
+-0.0000610351562500, 0.0127258300781250, -0.0000305175781250, -0.0510559082031250,
+-0.0000305175781250, -0.0605773925781250, 0.0000000000000000, -0.0487060546875000,
+0.0000000000000000, -0.0713806152343750, -0.0000305175781250, -0.0672302246093750,
+0.0000000000000000, -0.0230407714843750, 0.0000000000000000, -0.0385131835937500,
+0.0000000000000000, -0.0035095214843750, 0.0000610351562500, 0.0524902343750000,
+0.0001220703125000, 0.0281066894531250, 0.0000610351562500, 0.0983886718750000,
+0.0000610351562500, 0.1104736328125000, 0.0000915527343750, 0.1035156250000000,
+0.0000305175781250, 0.1355590820312500, -0.0000305175781250, 0.1344604492187500,
+-0.0000305175781250, 0.0795898437500000, -0.0000610351562500, 0.1065979003906250,
+-0.0001525878906250, 0.0598144531250000, -0.0000610351562500, -0.0128479003906250,
+-0.0000305175781250, 0.0278625488281250, -0.0000915527343750, -0.0789489746093750,
+0.0000000000000000, -0.0842590332031250, 0.0000305175781250, -0.0866088867187500,
+-0.0000305175781250, -0.1433715820312500, -0.0000915527343750, -0.1287841796875000,
+-0.0000305175781250, -0.1227722167968750, -0.0000610351562500, -0.1361999511718750,
+-0.0001220703125000, -0.1231384277343750, -0.0000305175781250, -0.1004333496093750,
+0.0000915527343750, -0.1113891601562500, -0.0000305175781250, -0.1098632812500000,
+0.0000305175781250, -0.1003723144531250, 0.0001220703125000, -0.1120300292968750,
+0.0000000000000000, -0.1309814453125000, -0.0001525878906250, -0.1211547851562500,
+-0.0000305175781250, -0.1073303222656250, -0.0000305175781250, -0.1378784179687500,
+-0.0001831054687500, -0.0779418945312500, -0.0000305175781250, -0.0223999023437500,
+0.0001220703125000, -0.0714416503906250, 0.0000000000000000, 0.0914916992187500,
+0.0000915527343750, 0.0831298828125000, 0.0001831054687500, 0.0988464355468750,
+0.0000305175781250, 0.2088317871093750, -0.0000610351562500, 0.1732482910156250,
+0.0000305175781250, 0.1520080566406250, -0.0000610351562500, 0.1977539062500000,
+-0.0000915527343750, 0.1312866210937500, -0.0000610351562500, 0.0516662597656250,
+-0.0000610351562500, 0.1056823730468750, -0.0000305175781250, -0.0210571289062500,
+-0.0000610351562500, -0.0319824218750000, -0.0000610351562500, -0.0202941894531250,
+0.0000305175781250, -0.0737304687500000, 0.0001220703125000, -0.0645446777343750,
+-0.0000305175781250, -0.0358886718750000, 0.0000915527343750, -0.0494079589843750,
+0.0002746582031250, -0.0377807617187500, -0.0001525878906250, -0.0007934570312500,
+-0.0007934570312500, -0.0045776367187500, -0.0005798339843750, -0.0007324218750000,
+-0.0003967285156250, 0.0021057128906250, -0.0009765625000000, -0.0001525878906250,
+-0.0004272460937500, -0.0011596679687500, 0.0012207031250000, 0.0003051757812500,
+0.0000915527343750, -0.0004882812500000, -0.0033569335937500, -0.0016479492187500,
+0.0022583007812500, -0.0007629394531250, -0.0101928710937500, 0.0005187988281250,
+-0.0217895507812500, -0.0002136230468750, -0.0110778808593750, -0.0000305175781250,
+-0.0432739257812500, 0.0003967285156250, -0.0476989746093750, 0.0001525878906250,
+-0.0325622558593750, -0.0003662109375000, -0.0458984375000000, -0.0002136230468750,
+-0.0514221191406250, 0.0001220703125000, 0.0264282226562500, -0.0000305175781250,
+0.0005187988281250, -0.0000610351562500, 0.0403442382812500, 0.0001831054687500,
+0.1271972656250000, 0.0002441406250000, 0.0835876464843750, -0.0001525878906250,
+0.1298828125000000, -0.0001220703125000, 0.1635742187500000, 0.0000305175781250,
+0.1093750000000000, -0.0001525878906250, 0.0838623046875000, -0.0002136230468750,
+0.1210327148437500, 0.0001220703125000, -0.0003356933593750, 0.0001525878906250,
+0.0072326660156250, 0.0000305175781250, 0.0038146972656250, 0.0000610351562500,
+-0.0693969726562500, 0.0001831054687500, -0.0488586425781250, -0.0001220703125000,
+-0.0426025390625000, -0.0001831054687500, -0.0554199218750000, -0.0000305175781250,
+-0.0505371093750000, -0.0000915527343750, -0.0296936035156250, -0.0001525878906250,
+-0.0355529785156250, 0.0001220703125000, -0.0756530761718750, 0.0001831054687500,
+-0.0506591796875000, 0.0000610351562500, -0.0878906250000000, 0.0000305175781250,
+-0.1503601074218750, 0.0000915527343750, -0.1148071289062500, -0.0000915527343750,
+-0.1725158691406250, -0.0001220703125000, -0.1935729980468750, -0.0000305175781250,
+-0.1557006835937500, -0.0000305175781250, -0.1562805175781250, -0.0001220703125000,
+-0.1766357421875000, 0.0000305175781250, -0.0680847167968750, 0.0000915527343750,
+-0.0861511230468750, 0.0000305175781250, -0.0465698242187500, 0.0000610351562500,
+0.0489807128906250, 0.0000915527343750, 0.0092163085937500, -0.0000610351562500,
+0.0760803222656250, -0.0000305175781250, 0.0956115722656250, 0.0000000000000000,
+0.0720214843750000, -0.0000305175781250, 0.0805358886718750, -0.0001220703125000,
+0.0871582031250000, 0.0000305175781250, 0.0365600585937500, 0.0000000000000000,
+0.0379943847656250, -0.0000305175781250, 0.0345458984375000, 0.0000305175781250,
+-0.0007019042968750, 0.0000610351562500, 0.0075988769531250, -0.0000305175781250,
+0.0403442382812500, -0.0000610351562500, 0.0156860351562500, -0.0000305175781250,
+0.0680541992187500, -0.0000305175781250, 0.1279296875000000, -0.0000610351562500,
+0.0937805175781250, 0.0000305175781250, 0.2423706054687500, 0.0000305175781250,
+0.2277526855468750, 0.0000305175781250, 0.2803039550781250, 0.0000305175781250,
+0.3992614746093750, 0.0000610351562500, 0.3543701171875000, 0.0000000000000000,
+0.4866638183593750, 0.0000000000000000, 0.4905395507812500, 0.0000000000000000,
+0.5219421386718750, -0.0000305175781250, 0.6093750000000000, -0.0000305175781250,
+0.5816040039062500, 0.0000305175781250, 0.6629943847656250, 0.0000000000000000,
+0.6705932617187500, 0.0000000000000000, 0.6853637695312500, 0.0000305175781250,
+0.7311401367187500, 0.0000305175781250, 0.7156677246093750, -0.0000305175781250,
+0.7562866210937500, 0.0000000000000000, 0.7561645507812500, 0.0000000000000000,
+0.7693176269531250, 0.0000000000000000, 0.7965087890625000, 0.0000000000000000,
+0.7805175781250000, 0.0000000000000000, 0.8152465820312500, 0.0000305175781250,
+0.8128356933593750, -0.0000305175781250, 0.8207092285156250, 0.0000305175781250,
+0.8433837890625000, 0.0000610351562500, 0.8304443359375000, 0.0000000000000000,
+0.8570251464843750, -0.0000305175781250, 0.8562927246093750, 0.0000000000000000,
+0.8583984375000000, 0.0000000000000000, 0.8747558593750000, 0.0000000000000000,
+0.8709106445312500, 0.0000305175781250, 0.8816223144531250, 0.0000000000000000,
+0.8833007812500000, 0.0000305175781250, 0.8848266601562500, 0.0000000000000000,
+0.8915710449218750, 0.0000000000000000, 0.8871459960937500, 0.0000000000000000,
+0.8769836425781250, 0.0000000000000000, 0.8873901367187500, 0.0000000000000000,
+0.8661499023437500, 0.0000000000000000, 0.8491821289062500, 0.0000000000000000,
+0.8687438964843750, 0.0000000000000000, 0.7958984375000000, 0.0000305175781250,
+0.8279113769531250, -0.0000305175781250, 0.7763366699218750, 0.0000000000000000,
+0.7050170898437500, 0.0000305175781250, 0.7658081054687500, 0.0000305175781250,
+0.5754089355468750, 0.0000305175781250, 0.6284790039062500, 0.0000305175781250,
+0.5559997558593750, 0.0000610351562500, 0.3955383300781250, 0.0000610351562500,
+0.4996643066406250, 0.0000000000000000, 0.2691040039062500, 0.0000305175781250,
+0.2798767089843750, 0.0000000000000000, 0.2511291503906250, -0.0000305175781250,
+0.1046447753906250, -0.0000305175781250, 0.1681213378906250, 0.0000305175781250,
+0.1016540527343750, 0.0000305175781250, 0.0730285644531250, 0.0000305175781250,
+0.1139526367187500, 0.0000305175781250, 0.1180419921875000, 0.0000610351562500,
+0.0971984863281250},
+{0.0612792968750000, 0.0000915527343750, 0.0360717773437500, -0.0000915527343750,
+0.0411071777343750, -0.0001220703125000, 0.0611572265625000, -0.0000305175781250,
+0.0351867675781250, 0.0000000000000000, 0.0046081542968750, -0.0000915527343750,
+0.0217285156250000, 0.0000305175781250, -0.0025024414062500, 0.0000915527343750,
+-0.0238647460937500, 0.0000305175781250, 0.0213317871093750, 0.0000610351562500,
+0.0420837402343750, 0.0000915527343750, 0.0184631347656250, 0.0000000000000000,
+0.1710815429687500, -0.0000610351562500, 0.1402282714843750, 0.0000305175781250,
+0.2196350097656250, -0.0000305175781250, 0.3646545410156250, -0.0000305175781250,
+0.2949218750000000, 0.0000305175781250, 0.4815368652343750, 0.0000305175781250,
+0.4855651855468750, 0.0000305175781250, 0.5118408203125000, 0.0000305175781250,
+0.6260070800781250, 0.0000305175781250, 0.5884094238281250, 0.0000000000000000,
+0.6441040039062500, -0.0000305175781250, 0.6614379882812500, -0.0000305175781250,
+0.6482238769531250, -0.0000610351562500, 0.6572875976562500, -0.0000915527343750,
+0.6555480957031250, -0.0000305175781250, 0.6307373046875000, 0.0000000000000000,
+0.6321105957031250, 0.0000000000000000, 0.6109008789062500, -0.0000305175781250,
+0.5806579589843750, 0.0000000000000000, 0.5874023437500000, -0.0000610351562500,
+0.5492858886718750, -0.0000305175781250, 0.5383911132812500, -0.0000305175781250,
+0.5296020507812500, -0.0000610351562500, 0.5070800781250000, -0.0000610351562500,
+0.5030822753906250, 0.0000000000000000, 0.4913635253906250, 0.0000000000000000,
+0.4794921875000000, 0.0000000000000000, 0.4700012207031250, 0.0000000000000000,
+0.4624938964843750, -0.0000305175781250, 0.4570312500000000, 0.0000000000000000,
+0.4461669921875000, 0.0000000000000000, 0.4331359863281250, -0.0000305175781250,
+0.4324645996093750, -0.0000305175781250, 0.4306640625000000, 0.0000000000000000,
+0.4191589355468750, 0.0000305175781250, 0.4353332519531250, 0.0000000000000000,
+0.4271240234375000, 0.0000000000000000, 0.4329833984375000, -0.0000305175781250,
+0.4479064941406250, 0.0000305175781250, 0.4371337890625000, 0.0000000000000000,
+0.4647216796875000, 0.0000000000000000, 0.4511718750000000, -0.0000305175781250,
+0.4696960449218750, -0.0000305175781250, 0.4992065429687500, 0.0000000000000000,
+0.4768981933593750, 0.0000000000000000, 0.5435180664062500, 0.0000305175781250,
+0.5283203125000000, 0.0000000000000000, 0.5558776855468750, 0.0000305175781250,
+0.6076660156250000, -0.0000305175781250, 0.5704650878906250, 0.0000000000000000,
+0.6680908203125000, -0.0000305175781250, 0.6454467773437500, 0.0000000000000000,
+0.6796569824218750, 0.0000000000000000, 0.7582092285156250, 0.0000000000000000,
+0.7077026367187500, 0.0000000000000000, 0.8167114257812500, 0.0000000000000000,
+0.8072509765625000, 0.0000000000000000, 0.8085021972656250, -0.0000305175781250,
+0.8770141601562500, -0.0000305175781250, 0.8497924804687500, -0.0000305175781250,
+0.8392333984375000, -0.0000610351562500, 0.8822631835937500, 0.0000305175781250,
+0.8073425292968750, 0.0000305175781250, 0.7472229003906250, 0.0000305175781250,
+0.8092041015625000, -0.0000305175781250, 0.5795593261718750, 0.0000000000000000,
+0.6348571777343750, -0.0000305175781250, 0.5415649414062500, -0.0000915527343750,
+0.3369445800781250, -0.0001220703125000, 0.4514160156250000, -0.0000305175781250,
+0.2242431640625000, -0.0000305175781250, 0.2157287597656250, -0.0000305175781250,
+0.2162170410156250, 0.0000305175781250, 0.0987243652343750, 0.0000305175781250,
+0.1357727050781250, -0.0000610351562500, 0.1344604492187500, -0.0000305175781250,
+0.1085510253906250, -0.0000305175781250, 0.1350402832031250, -0.0000915527343750,
+0.1717529296875000, -0.0001220703125000, 0.1546020507812500, 0.0000610351562500,
+0.1732482910156250, 0.0000305175781250, 0.2035522460937500, 0.0000000000000000,
+0.1457824707031250, 0.0000610351562500, 0.1126098632812500, 0.0001220703125000,
+0.1512756347656250, -0.0000305175781250, -0.0388183593750000, 0.0000000000000000,
+0.0071411132812500, -0.0000305175781250, -0.0765380859375000, -0.0001220703125000,
+-0.2553405761718750, -0.0000915527343750, -0.1682434082031250, 0.0000305175781250,
+-0.3586730957031250, 0.0000305175781250, -0.3778076171875000, 0.0000305175781250,
+-0.3563232421875000, 0.0001525878906250, -0.4393310546875000, 0.0000915527343750,
+-0.4256896972656250, 0.0000305175781250, -0.3467102050781250, 0.0000915527343750,
+-0.3928527832031250, 0.0000000000000000, -0.3243408203125000, -0.0001831054687500,
+-0.2153320312500000, -0.0000610351562500, -0.2808227539062500, -0.0000610351562500,
+-0.1400146484375000, -0.0002136230468750, -0.1276245117187500, -0.0000305175781250,
+-0.1395874023437500, 0.0000915527343750, -0.0834655761718750, 0.0000000000000000,
+-0.0946655273437500, 0.0002746582031250, -0.0997924804687500, 0.0003356933593750,
+-0.1054992675781250, 0.0001525878906250, -0.0717773437500000, 0.0002136230468750,
+-0.0650634765625000, 0.0002746582031250, -0.0887451171875000, -0.0005493164062500,
+0.0722656250000000, -0.0004272460937500, 0.0286254882812500, -0.0002441406250000,
+0.1020202636718750, -0.0009460449218750, 0.2663269042968750, -0.0008850097656250,
+0.1826477050781250, 0.0009765625000000, 0.2802124023437500, 0.0008239746093750,
+0.3364562988281250, -0.0002441406250000, 0.2459106445312500, 0.0024719238281250,
+0.2161560058593750, 0.0021972656250000, 0.2734985351562500, -0.0133666992187500,
+0.0598754882812500, -0.0041809082031250, 0.0792236328125000, -0.0187988281250000,
+0.0675964355468750, -0.0598449707031250, -0.1351013183593750, -0.0898437500000000,
+0.2546997070312500, 0.0638122558593750, 0.0196228027343750, -0.0412292480468750,
+0.1033020019531250, 0.0741271972656250, 0.0414428710937500, 0.2287292480468750,
+-0.0982971191406250, 0.1336669921875000, -0.0494079589843750, 0.2879333496093750,
+-0.0728759765625000, 0.3320007324218750, -0.1058044433593750, 0.2538452148437500,
+-0.0688781738281250, 0.2811889648437500, -0.0363159179687500, 0.3157958984375000,
+-0.0604248046875000, 0.0527343750000000, -0.0112915039062500, 0.1222839355468750,
+-0.0047912597656250, 0.0285949707031250, -0.0108947753906250, -0.2213134765625000,
+0.0054321289062500, -0.1006774902343750, 0.0032653808593750, -0.2064208984375000,
+-0.0005493164062500, -0.2970581054687500, 0.0002136230468750, -0.1597900390625000,
+0.0005493164062500, -0.0856933593750000, -0.0012207031250000, -0.1886596679687500,
+-0.0013122558593750, 0.0924987792968750, 0.0000000000000000, 0.1017456054687500,
+0.0000915527343750, 0.0462646484375000, -0.0001831054687500, 0.1817932128906250,
+0.0001831054687500, 0.1798706054687500, 0.0002441406250000, -0.0692749023437500,
+-0.0000305175781250, 0.0472717285156250, -0.0001220703125000, -0.1054077148437500,
+0.0000305175781250, -0.4015197753906250, 0.0002441406250000, -0.2299804687500000,
+0.0000915527343750, -0.5060424804687500, 0.0000000000000000, -0.5767822265625000,
+0.0001831054687500, -0.4562683105468750, 0.0000305175781250, -0.5091857910156250,
+-0.0003356933593750, -0.5589599609375000, -0.0002441406250000, -0.2142028808593750,
+-0.0001220703125000, -0.3088378906250000, -0.0002746582031250, -0.1871337890625000,
+-0.0001220703125000, 0.1196594238281250, 0.0001525878906250, -0.0436096191406250,
+0.0000000000000000, 0.1982421875000000, -0.0000915527343750, 0.2453308105468750,
+0.0000000000000000, 0.1843261718750000, -0.0000915527343750, 0.2539062500000000,
+-0.0002136230468750, 0.2639160156250000, -0.0001831054687500, 0.1428833007812500,
+0.0000000000000000, 0.1810302734375000, -0.0000305175781250, 0.1407165527343750,
+-0.0000610351562500, 0.0288085937500000, 0.0001525878906250, 0.0871276855468750,
+0.0001220703125000, 0.0312805175781250, -0.0001220703125000, -0.0067749023437500,
+-0.0000610351562500, 0.0509338378906250, -0.0000610351562500, 0.0638427734375000,
+-0.0002441406250000, 0.0266723632812500, -0.0002136230468750, 0.2119445800781250,
+-0.0000915527343750, 0.1535339355468750, -0.0000915527343750, 0.2504272460937500,
+-0.0001525878906250, 0.4244079589843750, -0.0000610351562500, 0.3130798339843750,
+-0.0000305175781250, 0.5958557128906250, -0.0001525878906250, 0.5692749023437500,
+-0.0001831054687500, 0.6175537109375000, -0.0000915527343750, 0.8046569824218750,
+-0.0000610351562500, 0.7086181640625000, -0.0001220703125000, 0.8397216796875000,
+0.0000610351562500, 0.8638610839843750, 0.0000915527343750, 0.8336181640625000,
+0.0000000000000000, 0.8776550292968750, 0.0000000000000000, 0.8807373046875000,
+0.0000610351562500, 0.8151550292968750, 0.0000305175781250, 0.8493652343750000,
+0.0000000000000000, 0.8034057617187500, 0.0000610351562500, 0.7321472167968750,
+0.0001220703125000, 0.7888183593750000, 0.0000610351562500, 0.6574401855468750,
+0.0000915527343750, 0.6788330078125000, 0.0001525878906250, 0.6506042480468750,
+0.0000915527343750, 0.5623168945312500, 0.0000305175781250, 0.6175842285156250,
+0.0000305175781250, 0.5054931640625000, 0.0000915527343750, 0.5225219726562500,
+0.0000915527343750, 0.4986877441406250, 0.0000610351562500, 0.4208374023437500,
+0.0000610351562500, 0.4657897949218750, 0.0000915527343750, 0.3744812011718750,
+0.0000610351562500, 0.3867187500000000, 0.0000915527343750, 0.3622436523437500,
+0.0000610351562500, 0.2990722656250000, 0.0000305175781250, 0.3403625488281250,
+0.0000305175781250, 0.2516784667968750, 0.0000610351562500, 0.2659301757812500,
+0.0000610351562500, 0.2476806640625000, 0.0000305175781250, 0.1883850097656250,
+0.0000915527343750, 0.2273864746093750, 0.0000610351562500, 0.1565246582031250,
+0.0000610351562500, 0.1712646484375000, 0.0000915527343750, 0.1631774902343750,
+0.0000610351562500, 0.1168518066406250, 0.0000915527343750, 0.1529846191406250,
+0.0000915527343750, 0.1235961914062500, 0.0001525878906250, 0.1274108886718750,
+0.0000915527343750, 0.1351928710937500, 0.0001220703125000, 0.1305541992187500,
+0.0000610351562500, 0.1572570800781250, 0.0000915527343750, 0.1652832031250000,
+0.0000915527343750, 0.1683349609375000, 0.0000610351562500, 0.1965026855468750,
+0.0000915527343750, 0.2186279296875000, 0.0001220703125000, 0.2208557128906250,
+0.0000915527343750, 0.2784729003906250, 0.0001525878906250, 0.2717285156250000,
+0.0001525878906250, 0.3027648925781250, 0.0001220703125000, 0.3553466796875000,
+0.0001525878906250, 0.3422241210937500, 0.0001220703125000, 0.4338073730468750,
+0.0000915527343750, 0.4260253906250000, 0.0000915527343750, 0.4660644531250000,
+0.0001220703125000, 0.5448913574218750, 0.0001220703125000, 0.5209350585937500,
+0.0000915527343750, 0.6239929199218750, 0.0001220703125000, 0.6246032714843750,
+0.0001220703125000, 0.6495971679687500, 0.0000610351562500, 0.7254638671875000,
+0.0000610351562500, 0.7044067382812500, 0.0001220703125000, 0.7341003417968750,
+-0.0000305175781250, 0.7616577148437500, -0.0000305175781250, 0.7097167968750000,
+-0.0000305175781250, 0.6866760253906250, -0.0000305175781250, 0.7137145996093750,
+-0.0000305175781250, 0.5329284667968750, -0.0000305175781250, 0.5761108398437500,
+-0.0000305175781250, 0.4655456542968750, -0.0000915527343750, 0.2832641601562500,
+-0.0000915527343750, 0.3778381347656250, -0.0000915527343750, 0.1001586914062500,
+-0.0001525878906250, 0.1016540527343750, -0.0002136230468750, 0.0676269531250000,
+-0.0000915527343750, -0.1023254394531250, -0.0000610351562500, -0.0407714843750000,
+-0.0000915527343750, -0.1000061035156250, 0.0000610351562500, -0.1361694335937500,
+0.0000610351562500},
+{0.0000000000000000, 0.7776184082031250, 0.0000610351562500, 0.7325134277343750,
+0.0000000000000000, 0.7031555175781250, 0.0000305175781250, 0.7392272949218750,
+0.0000610351562500, 0.6222534179687500, -0.0000305175781250, 0.6517028808593750,
+-0.0000610351562500, 0.6116638183593750, 0.0000000000000000, 0.5172729492187500,
+0.0000610351562500, 0.5762634277343750, 0.0000305175781250, 0.4517822265625000,
+0.0000305175781250, 0.4699096679687500, 0.0001220703125000, 0.4344177246093750,
+0.0000915527343750, 0.3464050292968750, 0.0000305175781250, 0.4016418457031250,
+0.0000000000000000, 0.2712707519531250, 0.0000610351562500, 0.2967529296875000,
+0.0000000000000000, 0.2587890625000000, 0.0000915527343750, 0.1645507812500000,
+0.0001525878906250, 0.2276000976562500, 0.0001220703125000, 0.0842895507812500,
+0.0000915527343750, 0.1138610839843750, 0.0001220703125000, 0.0762939453125000,
+0.0000610351562500, -0.0307006835937500, 0.0000610351562500, 0.0366516113281250,
+0.0000610351562500, -0.0905456542968750, 0.0001525878906250, -0.0709533691406250,
+0.0001831054687500, -0.0952453613281250, 0.0001220703125000, -0.1793518066406250,
+0.0001220703125000, -0.1199951171875000, 0.0001525878906250, -0.2105407714843750,
+0.0000915527343750, -0.1965637207031250, 0.0000610351562500, -0.2053833007812500,
+0.0001220703125000, -0.2601928710937500, 0.0000915527343750, -0.2206115722656250,
+0.0000610351562500, -0.2760314941406250, 0.0001220703125000, -0.2665405273437500,
+0.0001220703125000, -0.2681274414062500, 0.0000610351562500, -0.3010559082031250,
+0.0000610351562500, -0.2770996093750000, 0.0000610351562500, -0.2980041503906250,
+0.0000000000000000, -0.2968139648437500, -0.0000305175781250, -0.2931213378906250,
+0.0000000000000000, -0.2998352050781250, 0.0000305175781250, -0.2883605957031250,
+0.0000305175781250, -0.2887573242187500, 0.0000305175781250, -0.2907714843750000,
+0.0000305175781250, -0.2737121582031250, 0.0000305175781250, -0.2623901367187500,
+0.0000305175781250, -0.2658691406250000, 0.0000610351562500, -0.2238159179687500,
+0.0000610351562500, -0.2312927246093750, 0.0000305175781250, -0.2110900878906250,
+0.0000610351562500, -0.1734008789062500, 0.0000610351562500, -0.1870727539062500,
+0.0000305175781250, -0.1272277832031250, 0.0000610351562500, -0.1327819824218750,
+0.0000305175781250, -0.1099548339843750, 0.0000610351562500, -0.0647583007812500,
+0.0000915527343750, -0.0854492187500000, 0.0000610351562500, -0.0147399902343750,
+0.0000305175781250, -0.0250244140625000, 0.0000305175781250, 0.0166015625000000,
+0.0000610351562500, 0.0807800292968750, 0.0000915527343750, 0.0517578125000000,
+0.0000610351562500, 0.1881103515625000, 0.0001525878906250, 0.1660461425781250,
+0.0001220703125000, 0.2359313964843750, 0.0000915527343750, 0.3612365722656250,
+0.0001525878906250, 0.3025512695312500, 0.0001831054687500, 0.4830017089843750,
+0.0000610351562500, 0.4883117675781250, 0.0000305175781250, 0.5132141113281250,
+0.0000915527343750, 0.6307373046875000, 0.0000305175781250, 0.5940856933593750,
+0.0000305175781250, 0.5906372070312500, 0.0000610351562500, 0.6450805664062500,
+0.0000305175781250, 0.5508117675781250, 0.0000305175781250, 0.4700012207031250,
+-0.0000305175781250, 0.5366516113281250, 0.0000000000000000, 0.2720642089843750,
+-0.0001525878906250, 0.3038940429687500, -0.0001831054687500, 0.2207336425781250,
+-0.0000915527343750, 0.0084228515625000, -0.0000915527343750, 0.0989074707031250,
+-0.0001525878906250, -0.0675964355468750, 0.0000000000000000, -0.1024780273437500,
+-0.0000305175781250, -0.0689697265625000, -0.0000305175781250, -0.1146850585937500,
+0.0000305175781250, -0.1169433593750000, 0.0000305175781250, -0.0537719726562500,
+-0.0000610351562500, -0.0616149902343750, -0.0000610351562500, -0.0574340820312500,
+0.0000305175781250, -0.0118103027343750, -0.0000610351562500, -0.0224914550781250,
+-0.0000610351562500, -0.0539550781250000, 0.0000000000000000, -0.0323181152343750,
+0.0000305175781250, -0.0585937500000000, -0.0000305175781250, -0.1088562011718750,
+-0.0000915527343750, -0.0825500488281250, -0.0000915527343750, -0.1050415039062500,
+-0.0000305175781250, -0.1245422363281250, -0.0000610351562500, -0.0969238281250000,
+-0.0001525878906250, -0.0777587890625000, -0.0000915527343750, -0.0951843261718750,
+0.0000000000000000, -0.0519714355468750, -0.0006713867187500, -0.0479736328125000,
+-0.0007019042968750, -0.0525512695312500, -0.0001831054687500, -0.0396728515625000,
+-0.0004272460937500, -0.0404357910156250, -0.0008544921875000, -0.0248413085937500,
+0.0007019042968750, -0.0434875488281250, 0.0021362304687500, -0.0028381347656250,
+-0.0031738281250000, 0.0371093750000000, -0.0039062500000000, 0.0014038085937500,
+-0.0001220703125000, 0.1106567382812500, -0.0245666503906250, 0.1091003417968750,
+-0.0190124511718750, 0.0993652343750000, -0.0232238769531250, 0.1653747558593750,
+-0.0513000488281250, 0.1504821777343750, -0.0454711914062500, 0.0468750000000000,
+-0.0111999511718750, 0.0990905761718750, -0.0384216308593750, 0.0699462890625000,
+-0.0077209472656250, -0.0292053222656250, 0.0422668457031250, 0.0242919921875000,
+0.0096740722656250, -0.0412597656250000, 0.0940246582031250, -0.0629577636718750,
+0.1047058105468750, -0.0367736816406250, 0.0845642089843750, -0.0421142578125000,
+0.1154174804687500, -0.0533447265625000, 0.1192626953125000, -0.0141296386718750,
+0.0480346679687500, -0.0151977539062500, 0.0649108886718750, -0.0158996582031250,
+0.0469665527343750, 0.0041809082031250, -0.0162048339843750, -0.0006408691406250,
+0.0059204101562500, -0.0011596679687500, -0.0037231445312500, 0.0010375976562500,
+-0.0158081054687500, 0.0003356933593750, -0.0120239257812500, -0.0016784667968750,
+0.0007934570312500, -0.0014038085937500, -0.0017395019531250, -0.0001525878906250,
+-0.0449523925781250, -0.0004882812500000, -0.0231933593750000, -0.0005187988281250,
+-0.0533447265625000, 0.0003356933593750, -0.1131896972656250, 0.0002441406250000,
+-0.0807495117187500, -0.0001831054687500, -0.1094665527343750, -0.0001831054687500,
+-0.1349487304687500, 0.0000000000000000, -0.0960998535156250, -0.0001220703125000,
+-0.0713195800781250, -0.0002136230468750, -0.0982971191406250, 0.0000305175781250,
+-0.0270996093750000, 0.0001525878906250, -0.0198059082031250, -0.0000305175781250,
+-0.0341796875000000, -0.0000610351562500, -0.0087890625000000, 0.0000610351562500,
+-0.0063476562500000, -0.0000915527343750, -0.0439453125000000, -0.0002441406250000,
+-0.0432434082031250, -0.0000305175781250, -0.0283203125000000, 0.0000915527343750,
+-0.0498657226562500, -0.0001220703125000, -0.0534362792968750, 0.0000305175781250,
+0.0421142578125000, 0.0001220703125000, -0.0008239746093750, 0.0000305175781250,
+0.0842895507812500, -0.0000915527343750, 0.2077026367187500, 0.0000000000000000,
+0.1299438476562500, -0.0000610351562500, 0.3274230957031250, -0.0001525878906250,
+0.3304138183593750, -0.0000305175781250, 0.3473815917968750, 0.0000305175781250,
+0.4639892578125000, -0.0000610351562500, 0.4249572753906250, 0.0000305175781250,
+0.4411926269531250, 0.0000915527343750, 0.4710693359375000, 0.0000000000000000,
+0.4176940917968750, 0.0000000000000000, 0.3775939941406250, 0.0000305175781250,
+0.4046936035156250, -0.0000305175781250, 0.2766113281250000, -0.0001220703125000,
+0.2877807617187500, -0.0000305175781250, 0.2457580566406250, 0.0000000000000000,
+0.1490173339843750, -0.0000305175781250, 0.1897583007812500, 0.0000305175781250,
+0.0855407714843750, 0.0000610351562500, 0.0827636718750000, 0.0000305175781250,
+0.0599975585937500, 0.0000610351562500, -0.0059814453125000, 0.0000610351562500,
+0.0161437988281250, 0.0000000000000000, -0.0556945800781250, -0.0000305175781250,
+-0.0556945800781250, 0.0000305175781250, -0.0783081054687500, 0.0000305175781250,
+-0.1294860839843750, 0.0000000000000000, -0.1123352050781250, 0.0000915527343750,
+-0.1779479980468750, 0.0000915527343750, -0.1767883300781250, 0.0000610351562500,
+-0.1963195800781250, 0.0000610351562500, -0.2449035644531250, 0.0000915527343750,
+-0.2300109863281250, -0.0000305175781250, -0.2774047851562500, 0.0000305175781250,
+-0.2839355468750000, 0.0000610351562500, -0.2936401367187500, 0.0000610351562500,
+-0.3207702636718750, 0.0000915527343750, -0.3118896484375000, 0.0000610351562500,
+-0.3290100097656250, 0.0000915527343750, -0.3343200683593750, 0.0001220703125000,
+-0.3364562988281250, 0.0000915527343750, -0.3441467285156250, 0.0000915527343750,
+-0.3421630859375000, 0.0000610351562500, -0.3434753417968750, 0.0000305175781250,
+-0.3490600585937500, 0.0000610351562500, -0.3477172851562500, 0.0000610351562500,
+-0.3463745117187500, 0.0000610351562500, -0.3519287109375000, 0.0000610351562500,
+-0.3428039550781250, 0.0000610351562500, -0.3490295410156250, 0.0000305175781250,
+-0.3425292968750000, 0.0000305175781250, -0.3334350585937500, 0.0000305175781250,
+-0.3441772460937500, 0.0000305175781250, -0.3162536621093750, 0.0000000000000000,
+-0.3299560546875000, 0.0000000000000000, -0.3132324218750000, -0.0000305175781250,
+-0.2858276367187500, -0.0000305175781250, -0.3068542480468750, 0.0000000000000000,
+-0.2380065917968750, 0.0000000000000000, -0.2598571777343750, -0.0000305175781250,
+-0.2234497070312500, 0.0000000000000000, -0.1620178222656250, 0.0000305175781250,
+-0.2084045410156250, -0.0000305175781250, -0.0893249511718750, -0.0000610351562500,
+-0.1203613281250000, 0.0000000000000000, -0.0773010253906250, 0.0000305175781250,
+0.0196533203125000, -0.0000305175781250, -0.0447998046875000, 0.0000610351562500,
+0.1012573242187500, 0.0000305175781250, 0.0787658691406250, -0.0000305175781250,
+0.1051330566406250, -0.0000305175781250, 0.2055358886718750, 0.0000000000000000,
+0.1523437500000000, 0.0000000000000000, 0.2526245117187500, -0.0000610351562500,
+0.2462158203125000, 0.0000000000000000, 0.2692565917968750, 0.0000000000000000,
+0.3319702148437500, -0.0000305175781250, 0.2944946289062500, 0.0000000000000000,
+0.3912658691406250, 0.0000610351562500, 0.3677978515625000, 0.0000000000000000,
+0.4180908203125000, -0.0000610351562500, 0.5052185058593750, 0.0000000000000000,
+0.4475402832031250, 0.0000000000000000, 0.6027832031250000, -0.0000305175781250,
+0.5871276855468750, -0.0000305175781250, 0.6146240234375000, 0.0000000000000000,
+0.7244262695312500, 0.0000305175781250, 0.6721801757812500, -0.0000610351562500,
+0.7174682617187500, 0.0000000000000000, 0.7516784667968750, -0.0000610351562500,
+0.6925048828125000, -0.0001525878906250, 0.6714477539062500, -0.0001220703125000,
+0.7080078125000000, -0.0000915527343750, 0.5372009277343750, -0.0001220703125000,
+0.5722656250000000, -0.0000610351562500, 0.4974365234375000, 0.0000305175781250,
+0.3458557128906250, -0.0000305175781250, 0.4251098632812500, -0.0000610351562500,
+0.2461853027343750, 0.0000000000000000, 0.2440795898437500, -0.0000305175781250,
+0.2333374023437500, -0.0001525878906250, 0.1335449218750000, -0.0000915527343750,
+0.1684570312500000, -0.0001525878906250, 0.1361999511718750, -0.0002136230468750,
+0.1248474121093750, -0.0001220703125000, 0.1287231445312500, -0.0000610351562500,
+0.1283569335937500, -0.0001220703125000, 0.1294860839843750, 0.0001220703125000,
+0.0845031738281250, 0.0001220703125000, 0.1208801269531250, 0.0000000000000000,
+0.0482177734375000, 0.0001220703125000, -0.0288085937500000, 0.0001831054687500,
+0.0342407226562500, -0.0004272460937500, -0.1912231445312500, -0.0003967285156250,
+-0.1619567871093750, -0.0001525878906250, -0.2121582031250000, -0.0005187988281250,
+-0.3875427246093750},
+{-0.2116394042968750, 0.0000610351562500, -0.1924133300781250, 0.0000915527343750,
+-0.1909179687500000, 0.0000915527343750, -0.1836853027343750, 0.0000305175781250,
+-0.1732482910156250, 0.0000915527343750, -0.1737060546875000, 0.0000915527343750,
+-0.1745300292968750, 0.0000610351562500, -0.1674499511718750, 0.0000915527343750,
+-0.1781005859375000, 0.0000915527343750, -0.1657104492187500, 0.0000610351562500,
+-0.1748352050781250, 0.0000610351562500, -0.1908569335937500, 0.0000915527343750,
+-0.1758422851562500, 0.0000305175781250, -0.2123718261718750, 0.0000305175781250,
+-0.2051391601562500, 0.0000305175781250, -0.2159729003906250, 0.0000000000000000,
+-0.2436828613281250, 0.0000000000000000, -0.2282714843750000, 0.0000610351562500,
+-0.2674865722656250, 0.0000610351562500, -0.2588195800781250, 0.0000305175781250,
+-0.2736511230468750, 0.0000915527343750, -0.3065490722656250, 0.0000305175781250,
+-0.2875671386718750, 0.0000305175781250, -0.3344726562500000, 0.0000000000000000,
+-0.3294067382812500, 0.0000305175781250, -0.3366088867187500, 0.0000305175781250,
+-0.3684692382812500, 0.0000610351562500, -0.3540039062500000, 0.0000305175781250,
+-0.3755493164062500, 0.0000610351562500, -0.3833007812500000, 0.0000610351562500,
+-0.3706054687500000, 0.0000305175781250, -0.3765563964843750, 0.0000610351562500,
+-0.3835144042968750, -0.0000610351562500, -0.3267822265625000, -0.0000610351562500,
+-0.3571166992187500, -0.0000305175781250, -0.3096923828125000, -0.0000610351562500,
+-0.2459411621093750, -0.0000610351562500, -0.2991027832031250, -0.0000305175781250,
+-0.1432189941406250, 0.0000000000000000, -0.1760559082031250, -0.0000305175781250,
+-0.1174316406250000, -0.0000915527343750, 0.0109558105468750, -0.0000915527343750,
+-0.0654296875000000, -0.0000610351562500, 0.1036071777343750, -0.0001220703125000,
+0.0874023437500000, -0.0000610351562500, 0.1236267089843750, 0.0000000000000000,
+0.2384338378906250, -0.0000305175781250, 0.1807861328125000, -0.0000305175781250,
+0.3045654296875000, 0.0000000000000000, 0.2911071777343750, -0.0000610351562500,
+0.3302307128906250, -0.0001525878906250, 0.4199523925781250, -0.0000610351562500,
+0.3678894042968750, -0.0000915527343750, 0.5006713867187500, -0.0001525878906250,
+0.4874877929687500, -0.0000610351562500, 0.5169982910156250, 0.0000000000000000,
+0.6150207519531250, -0.0000305175781250, 0.5720214843750000, 0.0000000000000000,
+0.6376647949218750, 0.0000000000000000, 0.6598510742187500, -0.0000610351562500,
+0.6285400390625000, -0.0000305175781250, 0.6381835937500000, 0.0000000000000000,
+0.6513977050781250, -0.0000610351562500, 0.5512695312500000, -0.0000915527343750,
+0.5873413085937500, -0.0000610351562500, 0.5252075195312500, -0.0000305175781250,
+0.4201965332031250, -0.0000305175781250, 0.4805297851562500, 0.0001831054687500,
+0.3073120117187500, 0.0002136230468750, 0.3219909667968750, 0.0000915527343750,
+0.2853088378906250, 0.0002441406250000, 0.1652526855468750, 0.0002441406250000,
+0.2211608886718750, -0.0002746582031250, 0.1305847167968750, -0.0002441406250000,
+0.1202087402343750, -0.0000915527343750, 0.1282958984375000, -0.0004577636718750,
+0.0966796875000000, -0.0004882812500000, 0.1040954589843750, 0.0005798339843750,
+0.0798034667968750, 0.0008544921875000, 0.1024169921875000, -0.0008850097656250,
+0.0473327636718750, 0.0004577636718750, 0.0013122558593750, 0.0009765625000000,
+0.0469055175781250, -0.0137634277343750, -0.1514282226562500, -0.0052795410156250,
+-0.1105041503906250, -0.0208740234375000, -0.1728210449218750, -0.0428771972656250,
+-0.3471679687500000, -0.0272216796875000, -0.2649536132812500, -0.0603027343750000,
+-0.3566589355468750, -0.0698242187500000, -0.4157104492187500, -0.0445251464843750,
+-0.3238525390625000, -0.0516662597656250, -0.2862243652343750, -0.0635375976562500,
+-0.3431701660156250, 0.0503845214843750, -0.1302185058593750, 0.0135498046875000,
+-0.1455993652343750, 0.0757751464843750, -0.1295471191406250, 0.2013549804687500,
+0.0048522949218750, 0.1334533691406250, -0.0396728515625000, 0.2241821289062500,
+-0.0342712402343750, 0.2681884765625000, -0.0090942382812500, 0.1992797851562500,
+-0.0206298828125000, 0.1357727050781250, -0.0522155761718750, -0.1597595214843750,
+-0.0311279296875000, -0.0859680175781250, -0.0210571289062500, -0.2081604003906250,
+0.0012512207031250, -0.0658569335937500, -0.0085754394531250, 0.1084289550781250,
+-0.0441894531250000, -0.0061340332031250, -0.0367736816406250, 0.1486816406250000,
+0.0388183593750000, 0.2248229980468750, -0.0033874511718750, 0.0931396484375000,
+0.0560302734375000, 0.0711059570312500, 0.1681213378906250, 0.1524963378906250,
+0.1075439453125000, -0.2294311523437500, 0.1695556640625000, -0.1683044433593750,
+0.2153625488281250, -0.2363891601562500, 0.1450805664062500, -0.5372314453125000,
+0.1090087890625000, -0.4170837402343750, 0.1553039550781250, -0.4218444824218750,
+0.0059509277343750, -0.5513000488281250, 0.0084228515625000, -0.3617553710937500,
+0.0054931640625000, -0.1838073730468750, -0.0778198242187500, -0.3432617187500000,
+-0.0542297363281250, 0.0571594238281250, -0.0444946289062500, 0.0534973144531250,
+-0.0642089843750000, 0.0516967773437500, -0.0435485839843750, 0.2678527832031250,
+-0.0122680664062500, 0.2064208984375000, -0.0282897949218750, 0.1932983398437500,
+-0.0041809082031250, 0.2458190917968750, 0.0019531250000000, 0.1962280273437500,
+-0.0029907226562500, 0.1234741210937500, 0.0016479492187500, 0.1662902832031250,
+0.0019531250000000, 0.1149597167968750, -0.0003967285156250, 0.0890502929687500,
+-0.0008850097656250, 0.1289367675781250, -0.0001525878906250, 0.1372985839843750,
+-0.0001220703125000, 0.1145935058593750, -0.0004882812500000, 0.2157592773437500,
+0.0000000000000000, 0.1920471191406250, 0.0002441406250000, 0.2360839843750000,
+0.0000305175781250, 0.3189697265625000, -0.0001220703125000, 0.2682189941406250,
+0.0000000000000000, 0.4113464355468750, -0.0000610351562500, 0.3825988769531250,
+-0.0000915527343750, 0.4371032714843750, -0.0000915527343750, 0.5561523437500000,
+-0.0000305175781250, 0.4847717285156250, 0.0000000000000000, 0.6365661621093750,
+-0.0001525878906250, 0.6381225585937500, -0.0001831054687500, 0.6283874511718750,
+-0.0000915527343750, 0.7108154296875000, -0.0000915527343750, 0.6886901855468750,
+-0.0001831054687500, 0.6308288574218750, 0.0000915527343750, 0.6850280761718750,
+0.0001220703125000, 0.6015319824218750, 0.0000305175781250, 0.5023803710937500,
+0.0000305175781250, 0.5777893066406250, 0.0000915527343750, 0.3395080566406250,
+-0.0000305175781250, 0.3789978027343750, -0.0001220703125000, 0.3032531738281250,
+0.0000305175781250, 0.1229553222656250, 0.0000915527343750, 0.2239990234375000,
+-0.0000305175781250, -0.0086059570312500, 0.0000610351562500, 0.0099792480468750,
+0.0001831054687500, -0.0408325195312500, 0.0000915527343750, -0.1943359375000000,
+0.0000000000000000, -0.1087646484375000, 0.0000610351562500, -0.2705383300781250,
+0.0000305175781250, -0.2647705078125000, -0.0000305175781250, -0.2802429199218750,
+0.0000000000000000, -0.3771362304687500, 0.0000915527343750, -0.3341979980468750,
+0.0000915527343750, -0.3938598632812500, 0.0000305175781250, -0.4058227539062500,
+0.0000610351562500, -0.3904724121093750, 0.0000305175781250, -0.4119567871093750,
+-0.0000610351562500, -0.4180297851562500, -0.0000305175781250, -0.3795776367187500,
+0.0000000000000000, -0.4003601074218750, -0.0000305175781250, -0.3776855468750000,
+-0.0000610351562500, -0.3388366699218750, 0.0000000000000000, -0.3708496093750000,
+0.0000000000000000, -0.2952880859375000, -0.0000915527343750, -0.3127746582031250,
+-0.0000610351562500, -0.2931823730468750, 0.0000000000000000, -0.2350463867187500,
+-0.0001220703125000, -0.2698974609375000, -0.0000915527343750, -0.2042541503906250,
+-0.0000610351562500, -0.2114868164062500, 0.0000000000000000, -0.2003784179687500,
+-0.0000305175781250, -0.1584167480468750, -0.0000305175781250, -0.1831970214843750,
+0.0000305175781250, -0.1367492675781250, -0.0000610351562500, -0.1412048339843750,
+-0.0000915527343750, -0.1368103027343750, -0.0000305175781250, -0.1102600097656250,
+-0.0000305175781250, -0.1264648437500000, -0.0000305175781250, -0.0999145507812500,
+0.0000305175781250, -0.1057434082031250, 0.0000305175781250, -0.1026306152343750,
+0.0000000000000000, -0.0850830078125000, 0.0000000000000000, -0.0983581542968750,
+0.0000305175781250, -0.0837707519531250, 0.0000000000000000, -0.0877380371093750,
+0.0000000000000000, -0.0856628417968750, -0.0000305175781250, -0.0787048339843750,
+0.0000305175781250, -0.0935974121093750, 0.0000305175781250, -0.0807800292968750,
+0.0000305175781250, -0.0837402343750000, 0.0000305175781250, -0.0932617187500000,
+0.0000305175781250, -0.0914001464843750, 0.0000000000000000, -0.0971374511718750,
+0.0000305175781250, -0.1196899414062500, 0.0000000000000000, -0.1117553710937500,
+-0.0000305175781250, -0.1314086914062500, -0.0000305175781250, -0.1596679687500000,
+0.0000000000000000, -0.1503906250000000, -0.0000305175781250, -0.2065734863281250,
+-0.0000305175781250, -0.2032165527343750, -0.0000305175781250, -0.2197875976562500,
+0.0000000000000000, -0.2644042968750000, -0.0000305175781250, -0.2508544921875000,
+-0.0000305175781250, -0.2771301269531250, -0.0000610351562500, -0.2894897460937500,
+-0.0000610351562500, -0.2747192382812500, -0.0000305175781250, -0.2766723632812500,
+-0.0000610351562500, -0.2857360839843750, -0.0000305175781250, -0.2294006347656250,
+0.0000000000000000, -0.2440795898437500, 0.0000000000000000, -0.2165222167968750,
+0.0000000000000000, -0.1624755859375000, 0.0000610351562500, -0.1886901855468750,
+0.0000305175781250, -0.1196289062500000, -0.0000305175781250, -0.1187133789062500,
+-0.0000305175781250, -0.1099243164062500, -0.0000305175781250, -0.0723876953125000,
+-0.0000610351562500, -0.0875244140625000, -0.0000915527343750, -0.0501098632812500,
+0.0000000000000000, -0.0610656738281250, -0.0000610351562500, -0.0297546386718750,
+-0.0000305175781250, 0.0049438476562500, 0.0000610351562500, -0.0220642089843750,
+0.0000610351562500, 0.1109313964843750, -0.0000610351562500, 0.0787048339843750,
+-0.0000610351562500, 0.1546936035156250, 0.0000000000000000, 0.2896423339843750,
+0.0000000000000000, 0.2206726074218750, 0.0000000000000000, 0.4014892578125000,
+0.0000000000000000, 0.4204406738281250, 0.0000915527343750, 0.3998413085937500,
+-0.0000305175781250, 0.4854736328125000, -0.0000305175781250, 0.4758605957031250,
+-0.0000305175781250, 0.3500671386718750, -0.0001220703125000, 0.4158630371093750,
+-0.0001525878906250, 0.3055114746093750, -0.0001220703125000, 0.1343078613281250,
+-0.0001525878906250, 0.2324829101562500, -0.0001525878906250, -0.0072021484375000,
+-0.0004272460937500, -0.0195617675781250, -0.0003356933593750, -0.0124206542968750,
+-0.0003356933593750, -0.1291809082031250, -0.0007324218750000, -0.1003112792968750,
+-0.0006408691406250, -0.0754394531250000, 0.0004882812500000, -0.0982055664062500,
+0.0003967285156250, -0.0791320800781250, -0.0008544921875000, -0.0342102050781250,
+0.0009765625000000, -0.0504150390625000, 0.0011596679687500, -0.0627441406250000,
+-0.0125732421875000, -0.0413818359375000, -0.0050048828125000, -0.0672607421875000,
+-0.0161743164062500, -0.1076660156250000, -0.0362548828125000, -0.0850219726562500,
+-0.0224304199218750, -0.1081237792968750, -0.0328369140625000, -0.1258239746093750,
+-0.0495300292968750, -0.1010742187500000, -0.0165710449218750, -0.0859375000000000,
+0.0031433105468750, -0.1025085449218750, -0.0227050781250000, -0.0608215332031250,
+0.0667114257812500},
+{0.0000610351562500, -0.1401672363281250, 0.0000305175781250, -0.1469726562500000,
+0.0000305175781250, -0.1307678222656250, 0.0000305175781250, -0.0943603515625000,
+0.0000000000000000, -0.1144104003906250, 0.0000305175781250, -0.0639648437500000,
+0.0000305175781250, -0.0674438476562500, 0.0000610351562500, -0.0525207519531250,
+0.0000000000000000, -0.0173034667968750, -0.0000305175781250, -0.0355529785156250,
+0.0000305175781250, 0.0109252929687500, -0.0000305175781250, 0.0050964355468750,
+-0.0000305175781250, 0.0177612304687500, 0.0000305175781250, 0.0507507324218750,
+0.0000000000000000, 0.0328369140625000, 0.0000000000000000, 0.0770263671875000,
+0.0000305175781250, 0.0721130371093750, 0.0000305175781250, 0.0826416015625000,
+0.0000000000000000, 0.1146240234375000, 0.0000305175781250, 0.0964660644531250,
+-0.0000305175781250, 0.1196899414062500, -0.0000610351562500, 0.1253662109375000,
+-0.0000305175781250, 0.1116333007812500, -0.0000915527343750, 0.1146545410156250,
+-0.0001525878906250, 0.1163330078125000, -0.0000915527343750, 0.0707092285156250,
+-0.0000610351562500, 0.0847167968750000, -0.0000915527343750, 0.0469665527343750,
+-0.0000915527343750, -0.0089416503906250, -0.0000610351562500, 0.0153503417968750,
+-0.0000610351562500, -0.0773620605468750, -0.0000915527343750, -0.0768737792968750,
+-0.0000915527343750, -0.0953369140625000, -0.0000915527343750, -0.1590270996093750,
+-0.0001220703125000, -0.1387939453125000, -0.0001220703125000, -0.1637573242187500,
+-0.0000305175781250, -0.1826782226562500, -0.0000610351562500, -0.1618041992187500,
+-0.0000610351562500, -0.1539306640625000, 0.0000305175781250, -0.1695251464843750,
+0.0000000000000000, -0.1132202148437500, -0.0000610351562500, -0.1215209960937500,
+-0.0000305175781250, -0.1074218750000000, -0.0000305175781250, -0.0654296875000000,
+-0.0000610351562500, -0.0843505859375000, -0.0000915527343750, -0.0492553710937500,
+-0.0000305175781250, -0.0519104003906250, -0.0000305175781250, -0.0357360839843750,
+-0.0000305175781250, -0.0151367187500000, -0.0000305175781250, -0.0297546386718750,
+-0.0000610351562500, 0.0636901855468750, -0.0000915527343750, 0.0307617187500000,
+-0.0000305175781250, 0.1048889160156250, -0.0000305175781250, 0.2140197753906250,
+0.0000305175781250, 0.1473083496093750, -0.0000610351562500, 0.3371582031250000,
+-0.0003051757812500, 0.3388061523437500, -0.0001525878906250, 0.3389282226562500,
+-0.0002441406250000, 0.4520874023437500, -0.0006713867187500, 0.4225158691406250,
+-0.0004882812500000, 0.3462524414062500, 0.0001831054687500, 0.4154968261718750,
+-0.0001525878906250, 0.3016662597656250, -0.0004272460937500, 0.1542053222656250,
+0.0013427734375000, 0.2530822753906250, 0.0008239746093750, 0.0083923339843750,
+-0.0075683593750000, -0.0023803710937500, -0.0011901855468750, -0.0007019042968750,
+-0.0120239257812500, -0.1245727539062500, -0.0275268554687500, -0.0923156738281250,
+-0.0162048339843750, -0.0752563476562500, -0.0335998535156250, -0.0992736816406250,
+-0.0442810058593750, -0.0805969238281250, -0.0190734863281250, -0.0399169921875000,
+-0.0132446289062500, -0.0556335449218750, -0.0320129394531250, -0.0663452148437500,
+0.0488891601562500, -0.0468139648437500, 0.0350036621093750, -0.0695190429687500,
+0.0484313964843750, -0.1057434082031250, 0.1140747070312500, -0.0853576660156250,
+0.0890502929687500, -0.1014099121093750, 0.0812683105468750, -0.1171875000000000,
+0.1081848144531250, -0.0978088378906250, 0.0749816894531250, -0.0759277343750000,
+0.0197143554687500, -0.0766601562500000, 0.0289611816406250, -0.0530700683593750,
+-0.0031738281250000, -0.0592651367187500, -0.0042114257812500, -0.0504150390625000,
+0.0026245117187500, -0.0379028320312500, -0.0065612792968750, -0.0481872558593750,
+-0.0085754394531250, 0.0179748535156250, -0.0191955566406250, -0.0098876953125000,
+-0.0037231445312500, 0.0298461914062500, -0.0356445312500000, 0.1073913574218750,
+-0.0711975097656250, 0.0590515136718750, -0.0435791015625000, 0.1274719238281250,
+-0.1066894531250000, 0.1545410156250000, -0.1184692382812500, 0.1083984375000000,
+-0.0973510742187500, 0.1045837402343750, -0.1117248535156250, 0.1315917968750000,
+-0.1200866699218750, 0.0212707519531250, -0.0686035156250000, 0.0327758789062500,
+-0.0712280273437500, 0.0203247070312500, -0.0724792480468750, -0.0544738769531250,
+-0.0444335937500000, -0.0277404785156250, -0.0483398437500000, -0.0349731445312500,
+-0.0557861328125000, -0.0532836914062500, -0.0625610351562500, -0.0344543457031250,
+-0.0323486328125000, -0.0146789550781250, -0.0259399414062500, -0.0281066894531250,
+-0.0473937988281250, -0.0048217773437500, 0.0853271484375000, -0.0004882812500000,
+0.0457458496093750, -0.0043029785156250, 0.1048889160156250, 0.0019836425781250,
+0.2372741699218750, 0.0017089843750000, 0.1702575683593750, -0.0006408691406250,
+0.2833862304687500, -0.0006408691406250, 0.3075256347656250, -0.0000915527343750,
+0.2770080566406250, -0.0007019042968750, 0.3072204589843750, -0.0009155273437500,
+0.3128967285156250, -0.0000610351562500, 0.2372131347656250, 0.0000610351562500,
+0.2611083984375000, -0.0001525878906250, 0.2201538085937500, -0.0000305175781250,
+0.1464538574218750, 0.0000915527343750, 0.1869812011718750, -0.0000915527343750,
+0.0784912109375000, -0.0002136230468750, 0.0867614746093750, -0.0000610351562500,
+0.0542907714843750, 0.0000305175781250, -0.0246887207031250, -0.0000305175781250,
+0.0148925781250000, 0.0000000000000000, -0.0721130371093750, 0.0000305175781250,
+-0.0766906738281250, -0.0000305175781250, -0.0802917480468750, -0.0001525878906250,
+-0.1275634765625000, -0.0000610351562500, -0.1112976074218750, -0.0000915527343750,
+-0.1087036132812500, -0.0001525878906250, -0.1224670410156250, -0.0000915527343750,
+-0.0924987792968750, 0.0000305175781250, -0.0652160644531250, -0.0000305175781250,
+-0.0803527832031250, -0.0000610351562500, -0.0079345703125000, -0.0000305175781250,
+-0.0100097656250000, -0.0000915527343750, 0.0097045898437500, -0.0001831054687500,
+0.0621948242187500, -0.0001220703125000, 0.0429992675781250, -0.0001220703125000,
+0.0757751464843750, -0.0001831054687500, 0.0873413085937500, -0.0001220703125000,
+0.0760498046875000, -0.0000610351562500, 0.0817871093750000, -0.0000610351562500,
+0.0869445800781250, -0.0000915527343750, 0.0520629882812500, -0.0000610351562500,
+0.0625000000000000, -0.0001220703125000, 0.0427551269531250, -0.0001831054687500,
+0.0072326660156250, -0.0001220703125000, 0.0256347656250000, -0.0000610351562500,
+-0.0237121582031250, -0.0001220703125000, -0.0215759277343750, -0.0001220703125000,
+-0.0324707031250000, -0.0000305175781250, -0.0641479492187500, -0.0000610351562500,
+-0.0482177734375000, -0.0000915527343750, -0.0803527832031250, -0.0000610351562500,
+-0.0772705078125000, -0.0000610351562500, -0.0813598632812500, -0.0000915527343750,
+-0.0997619628906250, -0.0000915527343750, -0.0880126953125000, -0.0000610351562500,
+-0.1109008789062500, -0.0000915527343750, -0.1034240722656250, -0.0000305175781250,
+-0.1093139648437500, -0.0000305175781250, -0.1265258789062500, -0.0000610351562500,
+-0.1130371093750000, -0.0000305175781250, -0.1399230957031250, -0.0000305175781250,
+-0.1296386718750000, -0.0000305175781250, -0.1376037597656250, 0.0000000000000000,
+-0.1592102050781250, 0.0000000000000000, -0.1421508789062500, -0.0000305175781250,
+-0.1782531738281250, -0.0000305175781250, -0.1720581054687500, -0.0000305175781250,
+-0.1798400878906250, -0.0000305175781250, -0.2054443359375000, -0.0000305175781250,
+-0.1917724609375000, -0.0000610351562500, -0.2225341796875000, -0.0000610351562500,
+-0.2167663574218750, -0.0000915527343750, -0.2281799316406250, -0.0000610351562500,
+-0.2526550292968750, -0.0000305175781250, -0.2364807128906250, -0.0001220703125000,
+-0.2715148925781250, -0.0000915527343750, -0.2678833007812500, -0.0000915527343750,
+-0.2761840820312500, -0.0001220703125000, -0.3000488281250000, -0.0000915527343750,
+-0.2865600585937500, -0.0000610351562500, -0.3142089843750000, -0.0000915527343750,
+-0.3124084472656250, -0.0000610351562500, -0.3154907226562500, 0.0000000000000000,
+-0.3340759277343750, -0.0000305175781250, -0.3279724121093750, -0.0000610351562500,
+-0.3368530273437500, -0.0000305175781250, -0.3399963378906250, -0.0000610351562500,
+-0.3377075195312500, -0.0000610351562500, -0.3405151367187500, -0.0000915527343750,
+-0.3432922363281250, 0.0000000000000000, -0.3376159667968750, 0.0000000000000000,
+-0.3376770019531250, -0.0000610351562500, -0.3393249511718750, -0.0000610351562500,
+-0.3375549316406250, 0.0000000000000000, -0.3368835449218750, -0.0000610351562500,
+-0.3374023437500000, -0.0000610351562500, -0.3385314941406250, -0.0000305175781250,
+-0.3340759277343750, -0.0000305175781250, -0.3338317871093750, -0.0000610351562500,
+-0.3365478515625000, 0.0000000000000000, -0.3116455078125000, 0.0000305175781250,
+-0.3222656250000000, 0.0000305175781250, -0.2979431152343750, 0.0000305175781250,
+-0.2685546875000000, 0.0000610351562500, -0.2878417968750000, -0.0000305175781250,
+-0.2081909179687500, -0.0000305175781250, -0.2308044433593750, 0.0000305175781250,
+-0.1806640625000000, -0.0000305175781250, -0.1056213378906250, -0.0000610351562500,
+-0.1515808105468750, 0.0000305175781250, 0.0225830078125000, 0.0000610351562500,
+-0.0138854980468750, 0.0000305175781250, 0.0736694335937500, 0.0001525878906250,
+0.2312927246093750, 0.0001525878906250, 0.1423034667968750, -0.0001525878906250,
+0.3826904296875000, 0.0000000000000000, 0.3702697753906250, -0.0000915527343750,
+0.4047241210937500, -0.0004577636718750, 0.5628967285156250, -0.0003356933593750,
+0.5027160644531250, 0.0003662109375000, 0.5771789550781250, 0.0003662109375000,
+0.6065673828125000, -0.0006103515625000, 0.5727233886718750, 0.0006103515625000,
+0.5751342773437500, 0.0007629394531250, 0.5891418457031250, -0.0096130371093750,
+0.5247497558593750, -0.0030822753906250, 0.5396423339843750, -0.0153198242187500,
+0.5104370117187500, -0.0321655273437500, 0.4576416015625000, -0.0202331542968750,
+0.4870605468750000, -0.0454406738281250, 0.3830261230468750, -0.0537719726562500,
+0.4068908691406250, -0.0323791503906250, 0.3547363281250000, -0.0364990234375000,
+0.2588806152343750, -0.0478820800781250, 0.3156738281250000, 0.0448913574218750,
+0.1773681640625000, 0.0151367187500000, 0.1818847656250000, 0.0600585937500000,
+0.1693725585937500, 0.1588745117187500, 0.0885314941406250, 0.1060180664062500,
+0.1203918457031250, 0.1689758300781250, 0.0762329101562500, 0.2107238769531250,
+0.0797729492187500, 0.1352539062500000, 0.0521240234375000, 0.1063232421875000,
+0.0236511230468750, 0.1547851562500000, 0.0496520996093750, -0.0480041503906250,
+-0.0978088378906250, -0.0314941406250000, -0.0542907714843750, -0.0472717285156250,
+-0.1217956542968750, -0.1900024414062500, -0.2718505859375000, -0.1453857421875000,
+-0.1892395019531250, -0.0741882324218750, -0.2921752929687500, -0.1417541503906250,
+-0.3360900878906250, -0.0700073242187500, -0.2707214355468750, 0.1466369628906250,
+-0.2232055664062500, 0.0979919433593750, -0.1776733398437500, -0.2965393066406250,
+-0.2817993164062500, -0.1639099121093750, -0.3138122558593750, -0.2261657714843750,
+-0.2363891601562500, -0.4462280273437500, -0.2046203613281250, -0.3847351074218750,
+-0.2570495605468750, -0.2768554687500000, -0.0695190429687500, -0.3813781738281250,
+-0.0782165527343750, -0.2401123046875000, -0.0733642578125000, -0.0373840332031250,
+0.0371093750000000, -0.1683044433593750, 0.0079345703125000, 0.1078796386718750,
+-0.0056457519531250, 0.1342468261718750, 0.0102539062500000, 0.1021728515625000,
+0.0125732421875000},
+{-0.1389160156250000, 0.0000305175781250, -0.1673889160156250, 0.0000000000000000,
+-0.2387084960937500, 0.0000305175781250, -0.1954345703125000, -0.0000305175781250,
+-0.2885131835937500, 0.0000000000000000, -0.2835693359375000, 0.0000000000000000,
+-0.2991333007812500, -0.0000305175781250, -0.3603210449218750, 0.0000000000000000,
+-0.3320007324218750, 0.0000915527343750, -0.3716125488281250, 0.0000610351562500,
+-0.3787231445312500, 0.0000610351562500, -0.3675842285156250, 0.0000915527343750,
+-0.3807983398437500, 0.0000915527343750, -0.3828430175781250, 0.0000305175781250,
+-0.3543395996093750, 0.0000305175781250, -0.3661804199218750, 0.0000610351562500,
+-0.3440551757812500, 0.0000000000000000, -0.3099060058593750, 0.0000305175781250,
+-0.3292541503906250, 0.0001525878906250, -0.2744750976562500, 0.0000915527343750,
+-0.2794494628906250, 0.0001220703125000, -0.2683105468750000, 0.0002136230468750,
+-0.2311401367187500, 0.0002136230468750, -0.2484741210937500, 0.0000305175781250,
+-0.2179565429687500, 0.0001525878906250, -0.2250976562500000, 0.0000610351562500,
+-0.2027282714843750, -0.0003051757812500, -0.1810302734375000, -0.0001831054687500,
+-0.2011413574218750, 0.0003051757812500, -0.0799255371093750, 0.0002136230468750,
+-0.1314086914062500, -0.0002746582031250, -0.0322265625000000, 0.0008544921875000,
+0.1110534667968750, 0.0006713867187500, 0.0202941894531250, -0.0068664550781250,
+0.3103027343750000, -0.0013732910156250, 0.2864685058593750, -0.0120849609375000,
+0.3398742675781250, -0.0256042480468750, 0.5391845703125000, -0.0154113769531250,
+0.4531250000000000, -0.0388488769531250, 0.5622863769531250, -0.0452575683593750,
+0.5970153808593750, -0.0281066894531250, 0.5563354492187500, -0.0338439941406250,
+0.5709838867187500, -0.0423278808593750, 0.5849304199218750, 0.0362243652343750,
+0.5076599121093750, 0.0096740722656250, 0.5218505859375000, 0.0521240234375000,
+0.4951782226562500, 0.1400756835937500, 0.4367370605468750, 0.0914306640625000,
+0.4662780761718750, 0.1457824707031250, 0.3798828125000000, 0.1837158203125000,
+0.3994445800781250, 0.1176452636718750, 0.3539123535156250, 0.0922546386718750,
+0.2763366699218750, 0.1366271972656250, 0.3257446289062500, -0.0412902832031250,
+0.1857299804687500, -0.0234680175781250, 0.1957702636718750, -0.0350341796875000,
+0.1704101562500000, -0.1615905761718750, 0.0759277343750000, -0.1224060058593750,
+0.1155090332031250, -0.0582275390625000, 0.0541992187500000, -0.1199035644531250,
+0.0531616210937500, -0.0327453613281250, 0.0329284667968750, 0.1027221679687500,
+0.0014953613281250, 0.0214233398437500, 0.0258789062500000, 0.1159973144531250,
+-0.0943908691406250, 0.1781616210937500, -0.0642700195312500, 0.0833435058593750,
+-0.1022949218750000, -0.0317382812500000, -0.2817077636718750, -0.3423767089843750,
+0.0069885253906250, -0.2142028808593750, -0.1273193359375000, -0.3583679199218750,
+-0.0548400878906250, -0.1980590820312500, -0.1316223144531250, 0.0033874511718750,
+-0.2510681152343750, -0.1211853027343750, -0.1907958984375000, 0.0964965820312500,
+-0.2532653808593750, 0.1308288574218750, -0.3025207519531250, 0.0950317382812500,
+-0.2212524414062500, 0.1651916503906250, -0.1772460937500000, 0.1624755859375000,
+-0.2317810058593750, 0.1142883300781250, -0.0575256347656250, 0.1197204589843750,
+-0.0595703125000000, 0.1320800781250000, -0.0625305175781250, 0.1062927246093750,
+0.0297241210937500, 0.1024780273437500, 0.0087280273437500, 0.1645507812500000,
+-0.0062866210937500, 0.1476440429687500, 0.0014953613281250, 0.1638183593750000,
+0.0122070312500000, 0.2166442871093750, -0.0020141601562500, 0.1958312988281250,
+-0.0093688964843750, 0.2255859375000000, 0.0707397460937500, 0.2176818847656250,
+0.0507507324218750, 0.2539367675781250, 0.0708923339843750, 0.2793579101562500,
+0.1476440429687500, 0.2468566894531250, 0.1194152832031250, 0.4160156250000000,
+0.1047668457031250, 0.3613891601562500, 0.1408386230468750, 0.4422607421875000,
+0.0877685546875000, 0.6155090332031250, 0.0233764648437500, 0.5154113769531250,
+0.0667114257812500, 0.6648254394531250, -0.0222473144531250, 0.7167968750000000,
+-0.0354309082031250, 0.6207580566406250, -0.0196228027343750, 0.6337280273437500,
+-0.0477905273437500, 0.6828002929687500, -0.0476379394531250, 0.3718261718750000,
+-0.0199584960937500, 0.4620666503906250, -0.0250549316406250, 0.3326110839843750,
+-0.0198364257812500, 0.0450439453125000, 0.0007629394531250, 0.2060852050781250,
+-0.0061950683593750, -0.0831298828125000, -0.0006713867187500, -0.1039428710937500,
+0.0024414062500000, -0.0856018066406250, 0.0003967285156250, -0.2138366699218750,
+-0.0002746582031250, -0.1824340820312500, 0.0002441406250000, -0.1866149902343750,
+0.0000610351562500, -0.1996459960937500, -0.0003051757812500, -0.2030944824218750,
+-0.0000915527343750, -0.1923217773437500, 0.0003356933593750, -0.1902465820312500,
+0.0001525878906250, -0.2573242187500000, 0.0001220703125000, -0.2225036621093750,
+0.0001831054687500, -0.2735595703125000, 0.0002441406250000, -0.3555603027343750,
+0.0001220703125000, -0.3004455566406250, 0.0001220703125000, -0.4284362792968750,
+0.0001220703125000, -0.4248046875000000, 0.0001831054687500, -0.4283752441406250,
+0.0000915527343750, -0.5027465820312500, 0.0000000000000000, -0.4754943847656250,
+0.0000305175781250, -0.4732360839843750, 0.0000000000000000, -0.5027770996093750,
+0.0000000000000000, -0.4547729492187500, -0.0000305175781250, -0.4167175292968750,
+0.0000610351562500, -0.4572143554687500, 0.0000610351562500, -0.3266906738281250,
+-0.0000305175781250, -0.3574218750000000, 0.0000000000000000, -0.3123779296875000,
+0.0000000000000000, -0.2070922851562500, -0.0001220703125000, -0.2728576660156250,
+-0.0000915527343750, -0.1407775878906250, 0.0000000000000000, -0.1525268554687500,
+-0.0000305175781250, -0.1317749023437500, -0.0000305175781250, -0.0472412109375000,
+0.0000000000000000, -0.0916137695312500, 0.0000610351562500, -0.0141296386718750,
+-0.0000610351562500, -0.0156860351562500, -0.0000915527343750, -0.0098266601562500,
+-0.0000305175781250, 0.0339965820312500, -0.0000305175781250, 0.0138549804687500,
+-0.0000915527343750, 0.0455932617187500, 0.0000000000000000, 0.0496520996093750,
+0.0000000000000000, 0.0501708984375000, -0.0000305175781250, 0.0661010742187500,
+-0.0000915527343750, 0.0631103515625000, -0.0000305175781250, 0.0682373046875000,
+-0.0000915527343750, 0.0725097656250000, -0.0000915527343750, 0.0710144042968750,
+-0.0000915527343750, 0.0721130371093750, -0.0000610351562500, 0.0763244628906250,
+-0.0000610351562500, 0.0698852539062500, -0.0000610351562500, 0.0749206542968750,
+0.0000000000000000, 0.0720520019531250, -0.0000305175781250, 0.0668334960937500,
+-0.0000610351562500, 0.0753784179687500, -0.0000610351562500, 0.0594787597656250,
+-0.0000610351562500, 0.0650329589843750, -0.0000915527343750, 0.0613403320312500,
+-0.0000610351562500, 0.0491027832031250, -0.0000610351562500, 0.0589599609375000,
+-0.0000610351562500, 0.0437927246093750, -0.0000610351562500, 0.0481872558593750,
+-0.0000610351562500, 0.0465393066406250, -0.0000915527343750, 0.0369873046875000,
+-0.0000915527343750, 0.0471801757812500, -0.0000915527343750, 0.0386657714843750,
+-0.0000915527343750, 0.0389404296875000, -0.0001220703125000, 0.0442504882812500,
+-0.0000915527343750, 0.0436706542968750, -0.0001220703125000, 0.0456237792968750,
+-0.0001220703125000, 0.0535888671875000, -0.0000915527343750, 0.0516052246093750,
+-0.0000915527343750, 0.0535583496093750, -0.0000915527343750, 0.0610961914062500,
+-0.0000915527343750, 0.0602111816406250, -0.0000915527343750, 0.0621337890625000,
+-0.0001220703125000, 0.0640563964843750, -0.0000915527343750, 0.0615234375000000,
+-0.0000915527343750, 0.0596313476562500, -0.0000915527343750, 0.0615234375000000,
+-0.0001220703125000, 0.0565795898437500, -0.0001220703125000, 0.0543212890625000,
+-0.0001220703125000, 0.0601806640625000, -0.0000915527343750, 0.0604248046875000,
+-0.0000305175781250, 0.0572509765625000, -0.0000610351562500, 0.0820312500000000,
+-0.0000305175781250, 0.0736694335937500, -0.0000305175781250, 0.0930175781250000,
+-0.0000915527343750, 0.1223754882812500, -0.0000915527343750, 0.1064453125000000,
+-0.0000915527343750, 0.1537475585937500, 0.0000000000000000, 0.1557312011718750,
+-0.0000915527343750, 0.1600036621093750, -0.0000610351562500, 0.1896667480468750,
+0.0000000000000000, 0.1808471679687500, -0.0000305175781250, 0.1708679199218750,
+-0.0000610351562500, 0.1871337890625000, -0.0000915527343750, 0.1605529785156250,
+-0.0000610351562500, 0.1309814453125000, -0.0000610351562500, 0.1508483886718750,
+-0.0000915527343750, 0.0827026367187500, -0.0001525878906250, 0.0852661132812500,
+-0.0000915527343750, 0.0721130371093750, -0.0002136230468750, 0.0249328613281250,
+-0.0004272460937500, 0.0406494140625000, -0.0002746582031250, 0.0187377929687500,
+-0.0001220703125000, 0.0088500976562500, -0.0003967285156250, 0.0171203613281250,
+-0.0000610351562500, 0.0170288085937500, 0.0010070800781250, 0.0115661621093750,
+0.0003051757812500, 0.0204162597656250, -0.0023193359375000, 0.0199584960937500,
+0.0014648437500000, 0.0207824707031250, -0.0062866210937500, 0.0205688476562500,
+-0.0134277343750000, 0.0188903808593750, -0.0061035156250000, 0.0507507324218750,
+-0.0265502929687500, 0.0260314941406250, -0.0281066894531250, 0.0775756835937500,
+-0.0195312500000000, 0.1335449218750000, -0.0299377441406250, 0.0887451171875000,
+-0.0323486328125000, 0.2380065917968750, 0.0161437988281250, 0.2300415039062500,
+-0.0045166015625000, 0.2433471679687500, 0.0224609375000000, 0.3482666015625000,
+0.0805053710937500, 0.3152770996093750, 0.0491027832031250, 0.2698669433593750,
+0.0827941894531250, 0.3302307128906250, 0.1049194335937500, 0.2355041503906250,
+0.0698852539062500, 0.1200866699218750, 0.0548706054687500, 0.2005004882812500,
+0.0810241699218750, -0.0023193359375000, 0.0118408203125000, -0.0112609863281250,
+0.0109252929687500, -0.0037841796875000, 0.0148620605468750, -0.1008605957031250,
+-0.0170593261718750, -0.0772094726562500, -0.0115356445312500, -0.0573730468750000,
+-0.0134582519531250, -0.0737304687500000, -0.0079650878906250, -0.0675354003906250,
+-0.0268249511718750, -0.0359802246093750, -0.0470886230468750, -0.0419616699218750,
+-0.0437927246093750, -0.0824890136718750, -0.0986022949218750, -0.0653686523437500,
+-0.0870666503906250, -0.0669250488281250, -0.0801391601562500, -0.1013793945312500,
+-0.1169128417968750, -0.0892333984375000, -0.1110839843750000, -0.0779113769531250,
+-0.0742492675781250, -0.0912475585937500, -0.0872497558593750, -0.0783081054687500,
+-0.0737609863281250, -0.0552062988281250, -0.0346374511718750, -0.0673217773437500,
+-0.0484313964843750, -0.0525207519531250, -0.0450134277343750, -0.0541687011718750,
+-0.0387268066406250, -0.0389709472656250, -0.0324707031250000, -0.0299377441406250,
+-0.0419006347656250, -0.0439453125000000, -0.0446777343750000, 0.0355834960937500,
+0.0336303710937500, 0.0147399902343750, -0.0007324218750000, 0.0432128906250000,
+0.0586547851562500, 0.1211547851562500, 0.1578674316406250, 0.0833129882812500,
+0.1013488769531250, 0.1057434082031250, 0.2203063964843750, 0.1373901367187500,
+0.2330627441406250, 0.0912780761718750, 0.2192993164062500, 0.0570983886718750,
+0.2681274414062500, 0.0914001464843750, 0.2614135742187500, 0.0000000000000000,
+0.2260437011718750, -0.0014038085937500, 0.2477111816406250, 0.0005187988281250,
+0.2149963378906250, -0.0468750000000000, 0.1686706542968750, -0.0353393554687500,
+0.1976318359375000},
+{0.0000000000000000, 0.0369262695312500, -0.0000610351562500, 0.0490722656250000,
+0.0000305175781250, 0.0450134277343750, 0.0000305175781250, 0.0558166503906250,
+0.0000305175781250, 0.0690917968750000, 0.0000000000000000, 0.0603637695312500,
+-0.0000610351562500, 0.0989074707031250, -0.0000305175781250, 0.0932312011718750,
+0.0000000000000000, 0.1056823730468750, -0.0001220703125000, 0.1376647949218750,
+-0.0002441406250000, 0.1232910156250000, -0.0001220703125000, 0.1465759277343750,
+-0.0001220703125000, 0.1544189453125000, -0.0003967285156250, 0.1427612304687500,
+0.0000000000000000, 0.1452636718750000, 0.0007324218750000, 0.1503906250000000,
+0.0001525878906250, 0.1152038574218750, -0.0011291503906250, 0.1216430664062500,
+0.0015563964843750, 0.1098327636718750, -0.0044555664062500, 0.0790710449218750,
+-0.0090637207031250, 0.0921630859375000, -0.0035400390625000, 0.0755004882812500,
+-0.0217590332031250, 0.0676269531250000, -0.0209655761718750, 0.0766906738281250,
+-0.0174560546875000, 0.0785217285156250, -0.0301208496093750, 0.0723876953125000,
+-0.0286865234375000, 0.0840759277343750, 0.0040283203125000, 0.0847778320312500,
+-0.0151367187500000, 0.0825195312500000, 0.0127258300781250, 0.0834960937500000,
+0.0593872070312500, 0.0837707519531250, 0.0292053222656250, 0.0972900390625000,
+0.0766601562500000, 0.0802612304687500, 0.0917663574218750, 0.1161804199218750,
+0.0647888183593750, 0.1502685546875000, 0.0672302246093750, 0.1206665039062500,
+0.0821228027343750, 0.2293701171875000, 0.0167846679687500, 0.2218322753906250,
+0.0240478515625000, 0.2250976562500000, 0.0211486816406250, 0.3021545410156250,
+-0.0190429687500000, 0.2815856933593750, -0.0068054199218750, 0.2173767089843750,
+-0.0118103027343750, 0.2718811035156250, -0.0123596191406250, 0.1858520507812500,
+-0.0252685546875000, 0.0679626464843750, -0.0306091308593750, 0.1464233398437500,
+-0.0183715820312500, -0.0266113281250000, -0.0733642578125000, -0.0384521484375000,
+-0.0673217773437500, -0.0198669433593750, -0.0674743652343750, -0.0928955078125000,
+-0.1087646484375000, -0.0795593261718750, -0.1021423339843750, -0.0406799316406250,
+-0.0631103515625000, -0.0505371093750000, -0.0832824707031250, -0.0603332519531250,
+-0.0724182128906250, -0.0384826660156250, -0.0304565429687500, -0.0354614257812500,
+-0.0467529296875000, -0.0729980468750000, -0.0377502441406250, -0.0651855468750000,
+-0.0268554687500000, -0.0642700195312500, -0.0317993164062500, -0.0917968750000000,
+-0.0463867187500000, -0.0867004394531250, -0.0406188964843750, -0.0657653808593750,
+0.0065612792968750, -0.0757141113281250, -0.0270996093750000, -0.0684509277343750,
+0.0331420898437500, -0.0459899902343750, 0.1149291992187500, -0.0538635253906250,
+0.0632629394531250, -0.0454711914062500, 0.1949768066406250, -0.0495605468750000,
+0.2001953125000000, -0.0305786132812500, 0.1988525390625000, -0.0198974609375000,
+0.2656860351562500, -0.0368347167968750, 0.2473754882812500, 0.0437927246093750,
+0.2350158691406250, 0.0275878906250000, 0.2571716308593750, 0.0451965332031250,
+0.2250976562500000, 0.1144714355468750, 0.1912231445312500, 0.0865478515625000,
+0.2174377441406250, 0.0900573730468750, 0.1311950683593750, 0.1199340820312500,
+0.1447143554687500, 0.0784912109375000, 0.1222229003906250, 0.0366821289062500,
+0.0527648925781250, 0.0692749023437500, 0.0848083496093750, -0.0084533691406250,
+0.0468444824218750, -0.0125427246093750, 0.0272521972656250, -0.0056762695312500,
+0.0568542480468750, -0.0402832031250000, 0.0642700195312500, -0.0337524414062500,
+0.0471801757812500, -0.0189514160156250, 0.1167602539062500, -0.0275268554687500,
+0.1116333007812500, -0.0198364257812500, 0.1166381835937500, -0.0018310546875000,
+0.1622314453125000, -0.0100708007812500, 0.1472167968750000, -0.0012817382812500,
+0.1371459960937500, 0.0017700195312500, 0.1538696289062500, -0.0006408691406250,
+0.1271057128906250, 0.0002746582031250, 0.0943603515625000, 0.0007629394531250,
+0.1152954101562500, -0.0002136230468750, 0.0707092285156250, -0.0005798339843750,
+0.0626525878906250, -0.0001525878906250, 0.0688476562500000, -0.0000610351562500,
+0.0534362792968750, -0.0003051757812500, 0.0531921386718750, -0.0000305175781250,
+0.0702209472656250, 0.0000610351562500, 0.0664672851562500, -0.0000305175781250,
+0.0697021484375000, -0.0000610351562500, 0.0854797363281250, 0.0000000000000000,
+0.0791931152343750, -0.0000305175781250, 0.0645446777343750, -0.0000915527343750,
+0.0755004882812500, 0.0000000000000000, 0.0490722656250000, 0.0000305175781250,
+0.0198059082031250, 0.0000000000000000, 0.0368041992187500, -0.0000305175781250,
+-0.0291137695312500, 0.0000000000000000, -0.0254821777343750, 0.0000305175781250,
+-0.0439147949218750, -0.0000305175781250, -0.0939025878906250, -0.0000610351562500,
+-0.0775451660156250, 0.0000305175781250, -0.1157531738281250, 0.0000000000000000,
+-0.1236572265625000, 0.0000305175781250, -0.1199645996093750, 0.0000915527343750,
+-0.1342468261718750, 0.0001220703125000, -0.1338195800781250, 0.0000000000000000,
+-0.1206359863281250, 0.0000000000000000, -0.1268310546875000, 0.0000305175781250,
+-0.1127624511718750, -0.0000305175781250, -0.0939331054687500, -0.0000305175781250,
+-0.1045532226562500, 0.0000000000000000, -0.0717163085937500, 0.0000000000000000,
+-0.0718078613281250, 0.0000000000000000, -0.0632019042968750, 0.0000305175781250,
+-0.0400695800781250, 0.0000305175781250, -0.0469665527343750, 0.0000000000000000,
+-0.0257873535156250, 0.0000000000000000, -0.0241394042968750, 0.0000000000000000,
+-0.0189514160156250, 0.0000305175781250, -0.0057373046875000, 0.0000000000000000,
+-0.0097351074218750, 0.0000305175781250, 0.0046997070312500, 0.0000305175781250,
+0.0057678222656250, 0.0000610351562500, 0.0079650878906250, 0.0000610351562500,
+0.0166625976562500, 0.0000610351562500, 0.0157165527343750, 0.0000305175781250,
+0.0226745605468750, 0.0000305175781250, 0.0238037109375000, 0.0000305175781250,
+0.0245971679687500, 0.0000305175781250, 0.0289001464843750, -0.0000305175781250,
+0.0283508300781250, 0.0000000000000000, 0.0262145996093750, 0.0000305175781250,
+0.0312500000000000, 0.0000305175781250, 0.0249938964843750, 0.0000000000000000,
+0.0199890136718750, 0.0000610351562500, 0.0263977050781250, 0.0000000000000000,
+-0.0016784667968750, 0.0000000000000000, 0.0096740722656250, 0.0000000000000000,
+-0.0069274902343750, 0.0000305175781250, -0.0338745117187500, 0.0000305175781250,
+-0.0123291015625000, 0.0000610351562500, -0.0655822753906250, 0.0000305175781250,
+-0.0538940429687500, 0.0000305175781250, -0.0700073242187500, 0.0000000000000000,
+-0.1117553710937500, 0.0000305175781250, -0.0840454101562500, 0.0000305175781250,
+-0.1301574707031250, 0.0000305175781250, -0.1304931640625000, 0.0000305175781250,
+-0.1279296875000000, 0.0000610351562500, -0.1510620117187500, 0.0000305175781250,
+-0.1426696777343750, 0.0000000000000000, -0.1405944824218750, 0.0000915527343750,
+-0.1504211425781250, 0.0000610351562500, -0.1347045898437500, -0.0000305175781250,
+-0.1197814941406250, 0.0000000000000000, -0.1324462890625000, -0.0000305175781250,
+-0.0951232910156250, -0.0000610351562500, -0.0993041992187500, 0.0000305175781250,
+-0.0992736816406250, 0.0000610351562500, -0.0724792480468750, 0.0000305175781250,
+-0.0805664062500000, 0.0000305175781250, -0.1023254394531250, 0.0001220703125000,
+-0.0798034667968750, -0.0000305175781250, -0.1219177246093750, -0.0001525878906250,
+-0.1642456054687500, -0.0000305175781250, -0.1275329589843750, -0.0000305175781250,
+-0.2518615722656250, -0.0002441406250000, -0.2317504882812500, -0.0000305175781250,
+-0.2704772949218750, 0.0006713867187500, -0.3747558593750000, 0.0002441406250000,
+-0.3290710449218750, -0.0025939941406250, -0.4073486328125000, 0.0009460449218750,
+-0.4318542480468750, -0.0067749023437500, -0.3963623046875000, -0.0141296386718750,
+-0.4084167480468750, -0.0072631835937500, -0.4237976074218750, -0.0277099609375000,
+-0.3128967285156250, -0.0290222167968750, -0.3412475585937500, -0.0215454101562500,
+-0.3015747070312500, -0.0329284667968750, -0.1976928710937500, -0.0338745117187500,
+-0.2448425292968750, 0.0151977539062500, -0.1879272460937500, -0.0070495605468750,
+-0.1679077148437500, 0.0283203125000000, -0.1882019042968750, 0.0923767089843750,
+-0.1909790039062500, 0.0556030273437500, -0.1821289062500000, 0.1084594726562500,
+-0.1705627441406250, 0.1352539062500000, -0.2109375000000000, 0.0873413085937500,
+-0.1275024414062500, 0.0763549804687500, -0.0643615722656250, 0.1063537597656250,
+-0.1311340332031250, -0.0251159667968750, 0.1464843750000000, -0.0114135742187500,
+0.0954589843750000, -0.0223388671875000, 0.1861267089843750, -0.1175537109375000,
+0.4192810058593750, -0.0869750976562500, 0.3058776855468750, -0.0408630371093750,
+0.4848327636718750, -0.0887451171875000, 0.5204162597656250, -0.0285949707031250,
+0.4790344238281250, 0.0697937011718750, 0.5299987792968750, 0.0109252929687500,
+0.5332946777343750, 0.0736999511718750, 0.4500122070312500, 0.1278991699218750,
+0.4716491699218750, 0.0307006835937500, 0.4407043457031250, -0.0204162597656250,
+0.3677978515625000, 0.0466003417968750, 0.4006347656250000, -0.1944580078125000,
+0.3315734863281250, -0.1852416992187500, 0.3373718261718750, -0.1872253417968750,
+0.3078002929687500, -0.3369750976562500, 0.2595825195312500, -0.2964782714843750,
+0.2931823730468750, -0.1948852539062500, 0.1766357421875000, -0.2647399902343750,
+0.1939392089843750, -0.1902770996093750, 0.1752929687500000, 0.0545349121093750,
+0.0383605957031250, 0.1257324218750000, 0.2663269042968750, 0.0963134765625000,
+0.1318664550781250, 0.0927124023437500, 0.1929931640625000, 0.1187744140625000,
+0.1350708007812500, 0.1233215332031250, 0.0391845703125000, 0.1035156250000000,
+0.0834655761718750, 0.1723937988281250, 0.0138854980468750, 0.1701965332031250,
+0.0099182128906250, 0.1705932617187500, -0.0056152343750000, 0.2062988281250000,
+-0.0386047363281250, 0.1976928710937500, -0.0171813964843750, 0.2061462402343750,
+-0.1169128417968750, 0.1908569335937500, -0.0934448242187500, 0.2391052246093750,
+-0.1242980957031250, 0.2647705078125000, -0.2197265625000000, 0.2261047363281250,
+-0.1766662597656250, 0.4245605468750000, -0.1898193359375000, 0.3746337890625000,
+-0.2346496582031250, 0.4427185058593750, -0.1673583984375000, 0.6289978027343750,
+-0.1045837402343750, 0.5332336425781250, -0.1559448242187500, 0.6039428710937500,
+-0.0321960449218750, 0.6885375976562500, -0.0219116210937500, 0.5475769042968750,
+-0.0353698730468750, 0.4686584472656250, 0.0127258300781250, 0.5737304687500000,
+0.0073852539062500, 0.2088623046875000, -0.0064392089843750, 0.2745666503906250,
+-0.0117187500000000, 0.1831054687500000, 0.0120849609375000, -0.1010131835937500,
+0.0189514160156250, 0.0356445312500000, 0.0005187988281250, -0.1720581054687500,
+0.0778198242187500, -0.2084350585937500, 0.0749206542968750, -0.1780090332031250,
+0.0695495605468750, -0.2444458007812500, 0.1152648925781250, -0.2377319335937500,
+0.1079406738281250, -0.2222290039062500, 0.0576782226562500, -0.2183227539062500,
+0.0805358886718750, -0.2465515136718750, 0.0499267578125000, -0.2504272460937500,
+-0.0121765136718750, -0.2299499511718750, 0.0172119140625000, -0.3367919921875000,
+-0.0253601074218750, -0.3096008300781250, -0.0395507812500000, -0.3443298339843750,
+-0.0235595703125000, -0.4404296875000000, -0.0260314941406250, -0.3915405273437500,
+-0.0320129394531250, -0.4444580078125000, -0.0090332031250000, -0.4748229980468750,
+-0.0086669921875000},
+{-0.3432617187500000, 0.1123657226562500, -0.3796081542968750, 0.0753784179687500,
+-0.3294677734375000, 0.0775146484375000, -0.2298889160156250, 0.0982971191406250,
+-0.2838439941406250, -0.0132446289062500, -0.2010803222656250, 0.0017700195312500,
+-0.1828918457031250, -0.0108947753906250, -0.2046203613281250, -0.0960998535156250,
+-0.1926879882812500, -0.0689697265625000, -0.1875610351562500, -0.0372619628906250,
+-0.1853027343750000, -0.0783691406250000, -0.2172851562500000, -0.0236511230468750,
+-0.1469116210937500, 0.0596618652343750, -0.1000976562500000, 0.0075988769531250,
+-0.1545104980468750, 0.0599060058593750, 0.0913696289062500, 0.1074218750000000,
+0.0352783203125000, 0.0250244140625000, 0.1284179687500000, -0.0185852050781250,
+0.3468933105468750, 0.0400695800781250, 0.2343139648437500, -0.1667785644531250,
+0.4345703125000000, -0.1546325683593750, 0.4700927734375000, -0.1546936035156250,
+0.4322814941406250, -0.2856445312500000, 0.4963073730468750, -0.2506103515625000,
+0.4949951171875000, -0.1640930175781250, 0.4093627929687500, -0.2278137207031250,
+0.4348144531250000, -0.1336669921875000, 0.4026489257812500, 0.0112609863281250,
+0.3243103027343750, -0.0740966796875000, 0.3597717285156250, 0.0795593261718750,
+0.2975158691406250, 0.1078491210937500, 0.2976074218750000, 0.0754394531250000,
+0.2865600585937500, 0.1261901855468750, 0.2199707031250000, 0.0883483886718750,
+0.3411865234375000, 0.1689453125000000, 0.2638549804687500, 0.1455688476562500,
+0.2892456054687500, 0.1549072265625000, 0.2516784667968750, 0.1893310546875000,
+0.2135925292968750, 0.1836853027343750, 0.2423095703125000, 0.1975708007812500,
+0.1296997070312500, 0.1785888671875000, 0.1506958007812500, 0.2298583984375000,
+0.1173400878906250, 0.2636413574218750, 0.0234985351562500, 0.2205810546875000,
+0.0655212402343750, 0.4176330566406250, -0.0009765625000000, 0.3784790039062500,
+-0.0043945312500000, 0.4298095703125000, -0.0190429687500000, 0.6021728515625000,
+-0.0512695312500000, 0.5237426757812500, -0.0297546386718750, 0.5477600097656250,
+-0.1208496093750000, 0.6345520019531250, -0.1052551269531250, 0.4901733398437500,
+-0.1244506835937500, 0.3834838867187500, -0.2031250000000000, 0.4978942871093750,
+-0.1715698242187500, 0.1206054687500000, -0.1596679687500000, 0.1716308593750000,
+-0.1997985839843750, 0.0929260253906250, -0.1412963867187500, -0.1816406250000000,
+-0.0716857910156250, -0.0558471679687500, -0.1185302734375000, -0.2279968261718750,
+-0.0196228027343750, -0.2613830566406250, -0.0046691894531250, -0.2299194335937500,
+-0.0214233398437500, -0.2793579101562500, 0.0065307617187500, -0.2747802734375000,
+0.0073852539062500, -0.2540283203125000, -0.0018005371093750, -0.2518310546875000,
+-0.0130310058593750, -0.2684326171875000, 0.0147705078125000, -0.2640380859375000,
+0.0332336425781250, -0.2544250488281250, 0.0102539062500000, -0.3291015625000000,
+0.0753173828125000, -0.3080139160156250, 0.0811157226562500, -0.3372192382812500,
+0.0671997070312500, -0.4090881347656250, 0.0941772460937500, -0.3716735839843750,
+0.0957946777343750, -0.4129638671875000, 0.0386047363281250, -0.4364013671875000,
+0.0552062988281250, -0.4016723632812500, 0.0335388183593750, -0.3952331542968750,
+-0.0237121582031250, -0.4178161621093750, -0.0006103515625000, -0.3207397460937500,
+-0.0234985351562500, -0.3453674316406250, -0.0366821289062500, -0.3099975585937500,
+-0.0223083496093750, -0.2267761230468750, -0.0161437988281250, -0.2731323242187500,
+-0.0234985351562500, -0.1826477050781250, -0.0051574707031250, -0.1824645996093750,
+-0.0032043457031250, -0.1752624511718750, -0.0050048828125000, -0.1245727539062500,
+0.0015258789062500, -0.1437072753906250, 0.0005187988281250, -0.1087646484375000,
+-0.0002441406250000, -0.1093444824218750, -0.0000305175781250, -0.0983581542968750,
+0.0000915527343750, -0.0778808593750000, -0.0001831054687500, -0.0915832519531250,
+-0.0002441406250000, -0.0375671386718750, -0.0000305175781250, -0.0528564453125000,
+0.0000915527343750, -0.0240783691406250, 0.0000000000000000, 0.0279541015625000,
+-0.0000610351562500, -0.0061645507812500, 0.0000305175781250, 0.0651245117187500,
+0.0000000000000000, 0.0643920898437500, -0.0000305175781250, 0.0612792968750000,
+-0.0000305175781250, 0.1001892089843750, 0.0000305175781250, 0.0894775390625000,
+0.0000000000000000, 0.0772399902343750, 0.0000000000000000, 0.0951232910156250,
+0.0000000000000000, 0.0704345703125000, 0.0000305175781250, 0.0432434082031250,
+0.0000915527343750, 0.0684814453125000, 0.0000305175781250, 0.0053710937500000,
+0.0000610351562500, 0.0161743164062500, 0.0000915527343750, 0.0036010742187500,
+0.0000915527343750, -0.0402526855468750, 0.0000305175781250, -0.0157470703125000,
+0.0000610351562500, -0.0612182617187500, 0.0000610351562500, -0.0583801269531250,
+0.0000610351562500, -0.0632324218750000, 0.0000915527343750, -0.0907592773437500,
+0.0000305175781250, -0.0773315429687500, 0.0000610351562500, -0.0988769531250000,
+0.0000305175781250, -0.0990295410156250, 0.0000305175781250, -0.1008300781250000,
+0.0000305175781250, -0.1133117675781250, 0.0000000000000000, -0.1091918945312500,
+-0.0000305175781250, -0.1200866699218750, 0.0000000000000000, -0.1206665039062500,
+0.0000305175781250, -0.1221008300781250, 0.0000610351562500, -0.1282653808593750,
+0.0000610351562500, -0.1266784667968750, 0.0000305175781250, -0.1330261230468750,
+0.0000305175781250, -0.1328430175781250, 0.0000610351562500, -0.1356201171875000,
+0.0000610351562500, -0.1410217285156250, 0.0000610351562500, -0.1398925781250000,
+0.0000610351562500, -0.1479797363281250, 0.0000610351562500, -0.1490173339843750,
+0.0001220703125000, -0.1525268554687500, 0.0000610351562500, -0.1585693359375000,
+0.0000610351562500, -0.1566467285156250, 0.0000610351562500, -0.1614379882812500,
+0.0000305175781250, -0.1617431640625000, 0.0000610351562500, -0.1607360839843750,
+0.0000610351562500, -0.1626281738281250, 0.0000610351562500, -0.1616821289062500,
+0.0000305175781250, -0.1559753417968750, 0.0000915527343750, -0.1572570800781250,
+0.0000915527343750, -0.1509399414062500, 0.0001220703125000, -0.1435852050781250,
+0.0000915527343750, -0.1466064453125000, 0.0001220703125000, -0.1300048828125000,
+0.0000915527343750, -0.1304626464843750, 0.0000915527343750, -0.1247558593750000,
+0.0000915527343750, -0.1112060546875000, 0.0001220703125000, -0.1147155761718750,
+0.0000915527343750, -0.1048889160156250, 0.0000915527343750, -0.1013488769531250,
+0.0001220703125000, -0.1065673828125000, 0.0000915527343750, -0.1048889160156250,
+0.0000915527343750, -0.1015014648437500, 0.0000915527343750, -0.1223449707031250,
+0.0001220703125000, -0.1168823242187500, 0.0001220703125000, -0.1269836425781250,
+0.0000305175781250, -0.1480712890625000, -0.0000610351562500, -0.1375732421875000,
+0.0000915527343750, -0.1551208496093750, -0.0000610351562500, -0.1607360839843750,
+-0.0001220703125000, -0.1495971679687500, 0.0001525878906250, -0.1507568359375000,
+0.0003356933593750, -0.1548767089843750, 0.0000000000000000, -0.1171875000000000,
+0.0001525878906250, -0.1263732910156250, 0.0012512207031250, -0.1064758300781250,
+-0.0018920898437500, -0.0705566406250000, -0.0032043457031250, -0.0887451171875000,
+-0.0006713867187500, -0.0382385253906250, -0.0132446289062500, -0.0402526855468750,
+-0.0104370117187500, -0.0257873535156250, -0.0123291015625000, 0.0091552734375000,
+-0.0245056152343750, -0.0062561035156250, -0.0197753906250000, 0.0375366210937500,
+-0.0086364746093750, 0.0349731445312500, -0.0223999023437500, 0.0466003417968750,
+0.0002136230468750, 0.0774536132812500, 0.0262451171875000, 0.0627746582031250,
+0.0039672851562500, 0.0934753417968750, 0.0549621582031250, 0.0957641601562500,
+0.0578002929687500, 0.0942077636718750, 0.0476684570312500, 0.1090393066406250,
+0.0708923339843750, 0.1052856445312500, 0.0706176757812500, 0.0979919433593750,
+0.0310668945312500, 0.1029663085937500, 0.0462036132812500, 0.0972595214843750,
+0.0307922363281250, 0.0847473144531250, -0.0103759765625000, 0.0905761718750000,
+0.0089416503906250, 0.0931396484375000, -0.0097045898437500, 0.0830078125000000,
+-0.0174255371093750, 0.1014099121093750, -0.0151977539062500, 0.1189270019531250,
+-0.0139465332031250, 0.1049804687500000, -0.0136718750000000, 0.1500244140625000,
+-0.0403747558593750, 0.1457214355468750, -0.0281372070312500, 0.1537780761718750,
+-0.0410766601562500, 0.1824951171875000, -0.0745239257812500, 0.1703491210937500,
+-0.0573425292968750, 0.2020568847656250, -0.0599365234375000, 0.1942138671875000,
+-0.0776672363281250, 0.2144470214843750, -0.0565185546875000, 0.2474670410156250,
+-0.0302734375000000, 0.2252807617187500, -0.0484619140625000, 0.2733764648437500,
+-0.0303955078125000, 0.2850036621093750, -0.0172729492187500, 0.2529602050781250,
+-0.0328674316406250, 0.2585144042968750, -0.0526123046875000, 0.2462158203125000,
+-0.0466613769531250, 0.0684204101562500, -0.0292358398437500, 0.1464538574218750,
+-0.0551147460937500, 0.1062316894531250, -0.0160827636718750, -0.0535278320312500,
+0.0241088867187500, 0.0291442871093750, -0.0100708007812500, -0.0682067871093750,
+0.1240234375000000, -0.1026306152343750, 0.1127014160156250, -0.0629577636718750,
+0.1370544433593750, -0.0669860839843750, 0.2310485839843750, -0.0849304199218750,
+0.1945190429687500, -0.0385742187500000, 0.2320251464843750, -0.0288085937500000,
+0.2531738281250000, -0.0483398437500000, 0.2270202636718750, -0.0432739257812500,
+0.2207946777343750, -0.0320129394531250, 0.2334899902343750, -0.0672302246093750,
+0.1691284179687500, -0.0715637207031250, 0.1816406250000000, -0.0589294433593750,
+0.1570434570312500, -0.0677185058593750, 0.0993652343750000, -0.0740966796875000,
+0.1275024414062500, -0.0441589355468750, 0.0859375000000000, -0.0467529296875000,
+0.0708007812500000, -0.0468444824218750, 0.0918579101562500, -0.0308837890625000,
+0.0915527343750000, -0.0340881347656250, 0.0795593261718750, -0.0278930664062500,
+0.1189880371093750, -0.0366821289062500, 0.1218566894531250, -0.0149841308593750,
+0.1084594726562500, 0.0030212402343750, 0.1235351562500000, -0.0159301757812500,
+0.1271362304687500, 0.0528564453125000, 0.0717163085937500, 0.0469665527343750,
+0.0859375000000000, 0.0483703613281250, 0.0659179687500000, 0.0941467285156250,
+0.0138854980468750, 0.0812377929687500, 0.0375366210937500, 0.0566406250000000,
+0.0133972167968750, 0.0796203613281250, -0.0003662109375000, 0.0501098632812500,
+0.0132446289062500, 0.0044860839843750, 0.0205383300781250, 0.0331420898437500,
+0.0124206542968750, -0.0151062011718750, 0.0186462402343750, -0.0225830078125000,
+0.0311279296875000, -0.0122070312500000, 0.0037231445312500, -0.0268554687500000,
+-0.0134277343750000, -0.0274963378906250, 0.0061340332031250, -0.0109252929687500,
+-0.0730590820312500, -0.0151977539062500, -0.0640869140625000, -0.0123596191406250,
+-0.0879516601562500, 0.0003967285156250, -0.1522216796875000, -0.0047607421875000,
+-0.1270446777343750, -0.0006408691406250, -0.1669921875000000, 0.0011901855468750,
+-0.1806335449218750, -0.0001220703125000, -0.1655883789062500, -0.0000305175781250,
+-0.1694335937500000, 0.0003051757812500, -0.1741943359375000, 0.0000000000000000,
+-0.1383666992187500, -0.0003051757812500, -0.1424255371093750, -0.0000610351562500,
+-0.1316223144531250, 0.0001831054687500, -0.1029663085937500, 0.0000305175781250,
+-0.1147766113281250, 0.0000915527343750, -0.1011657714843750, 0.0001525878906250,
+-0.0946655273437500, 0.0000915527343750, -0.1035156250000000, 0.0000000000000000,
+-0.1053466796875000},
+{-0.0296630859375000, 0.0650329589843750, -0.0553283691406250, 0.0690307617187500,
+-0.0396423339843750, 0.0751647949218750, -0.0549316406250000, 0.0644531250000000,
+-0.0673828125000000, 0.0845642089843750, -0.0496215820312500, 0.1041564941406250,
+-0.0374450683593750, 0.0879516601562500, -0.0523071289062500, 0.1435241699218750,
+-0.0329589843750000, 0.1368713378906250, -0.0212097167968750, 0.1528625488281250,
+-0.0394592285156250, 0.1922912597656250, -0.0506286621093750, 0.1725769042968750,
+-0.0357360839843750, 0.2251586914062500, -0.0529479980468750, 0.2176513671875000,
+-0.0719299316406250, 0.2324218750000000, -0.0316162109375000, 0.2886962890625000,
+0.0007934570312500, 0.2821350097656250, -0.0023498535156250, 0.2781372070312500,
+0.1221923828125000, 0.3072814941406250, 0.0791931152343750, 0.2548522949218750,
+0.1009826660156250, 0.2221679687500000, 0.2064819335937500, 0.2718200683593750,
+0.1648254394531250, 0.0795898437500000, 0.2233581542968750, 0.1170959472656250,
+0.2415466308593750, 0.0735473632812500, 0.2218933105468750, -0.0805053710937500,
+0.2321472167968750, -0.0085754394531250, 0.2369384765625000, -0.0730895996093750,
+0.1878356933593750, -0.1109313964843750, 0.2025756835937500, -0.0701293945312500,
+0.1759338378906250, -0.0512084960937500, 0.1244506835937500, -0.0742187500000000,
+0.1507873535156250, -0.0356445312500000, 0.0988769531250000, -0.0207519531250000,
+0.0890502929687500, -0.0421752929687500, 0.1015625000000000, -0.0469970703125000,
+0.0881958007812500, -0.0332336425781250, 0.0849914550781250, -0.0615539550781250,
+0.1072998046875000, -0.0692443847656250, 0.1099548339843750, -0.0547180175781250,
+0.0953369140625000, -0.0557861328125000, 0.1011352539062500, -0.0644226074218750,
+0.1091918945312500, -0.0361022949218750, 0.0513000488281250, -0.0356750488281250,
+0.0653381347656250, -0.0373535156250000, 0.0490722656250000, -0.0261535644531250,
+-0.0033264160156250, -0.0275573730468750, 0.0177612304687500, -0.0190734863281250,
+0.0022583007812500, -0.0289306640625000, -0.0110473632812500, -0.0069885253906250,
+0.0001831054687500, 0.0144348144531250, 0.0112609863281250, -0.0055541992187500,
+0.0025939941406250, 0.0530395507812500, -0.0039978027343750, 0.0513610839843750,
+0.0136413574218750, 0.0462036132812500, -0.0202941894531250, 0.0795593261718750,
+-0.0525512695312500, 0.0737609863281250, -0.0263061523437500, 0.0411376953125000,
+-0.1130371093750000, 0.0597229003906250, -0.1072082519531250, 0.0383605957031250,
+-0.1233825683593750, -0.0032043457031250, -0.1845397949218750, 0.0206298828125000,
+-0.1604309082031250, -0.0153198242187500, -0.1808166503906250, -0.0224609375000000,
+-0.1963500976562500, -0.0124511718750000, -0.1771545410156250, -0.0210571289062500,
+-0.1658935546875000, -0.0234375000000000, -0.1758117675781250, -0.0084533691406250,
+-0.1406555175781250, -0.0112915039062500, -0.1387939453125000, -0.0097351074218750,
+-0.1391906738281250, 0.0004577636718750, -0.1215820312500000, -0.0034790039062500,
+-0.1254882812500000, -0.0006103515625000, -0.1337585449218750, 0.0008850097656250,
+-0.1290893554687500, -0.0000915527343750, -0.1373596191406250, -0.0002441406250000,
+-0.1509094238281250, 0.0000305175781250, -0.1449584960937500, -0.0001220703125000,
+-0.1538085937500000, -0.0003967285156250, -0.1587524414062500, -0.0001525878906250,
+-0.1503601074218750, 0.0000915527343750, -0.1477661132812500, -0.0000305175781250,
+-0.1511230468750000, 0.0000610351562500, -0.1238403320312500, 0.0001525878906250,
+-0.1291198730468750, 0.0000915527343750, -0.1156311035156250, 0.0000915527343750,
+-0.0907592773437500, 0.0000915527343750, -0.1015625000000000, 0.0000915527343750,
+-0.0659484863281250, 0.0000610351562500, -0.0682067871093750, 0.0000305175781250,
+-0.0584411621093750, 0.0000305175781250, -0.0331420898437500, 0.0000610351562500,
+-0.0440063476562500, 0.0000305175781250, -0.0172424316406250, 0.0000305175781250,
+-0.0178833007812500, 0.0000610351562500, -0.0145874023437500, 0.0000305175781250,
+0.0004577636718750, 0.0000610351562500, -0.0079345703125000, 0.0000610351562500,
+0.0039367675781250, 0.0000610351562500, 0.0026855468750000, 0.0000610351562500,
+0.0032348632812500, 0.0000610351562500, 0.0100402832031250, 0.0000610351562500,
+0.0053405761718750, 0.0000305175781250, 0.0096130371093750, 0.0000305175781250,
+0.0062866210937500, 0.0000915527343750, 0.0079345703125000, 0.0000610351562500,
+0.0123596191406250, 0.0000915527343750, 0.0075073242187500, 0.0001220703125000,
+0.0173034667968750, 0.0001525878906250, 0.0146179199218750, 0.0000915527343750,
+0.0184631347656250, 0.0001220703125000, 0.0261535644531250, 0.0001220703125000,
+0.0212097167968750, 0.0001220703125000, 0.0389709472656250, 0.0001220703125000,
+0.0328063964843750, 0.0001220703125000, 0.0412597656250000, 0.0000915527343750,
+0.0577697753906250, 0.0000915527343750, 0.0464782714843750, 0.0001220703125000,
+0.0722045898437500, 0.0000915527343750, 0.0707092285156250, 0.0000915527343750,
+0.0729675292968750, 0.0001220703125000, 0.0888061523437500, 0.0001525878906250,
+0.0823974609375000, 0.0000305175781250, 0.0890808105468750, 0.0000610351562500,
+0.0915222167968750, 0.0000305175781250, 0.0918884277343750, -0.0000305175781250,
+0.0936584472656250, 0.0000000000000000, 0.0925903320312500, 0.0000000000000000,
+0.0974121093750000, 0.0000610351562500, 0.0976867675781250, 0.0000305175781250,
+0.0979003906250000, 0.0000305175781250, 0.1035156250000000, 0.0000610351562500,
+0.1025390625000000, -0.0001220703125000, 0.0858764648437500, -0.0000305175781250,
+0.0983276367187500, 0.0000000000000000, 0.0763244628906250, -0.0001831054687500,
+0.0484924316406250, -0.0002136230468750, 0.0694274902343750, 0.0000915527343750,
+0.0162048339843750, 0.0006408691406250, 0.0184631347656250, -0.0012817382812500,
+0.0185546875000000, -0.0014038085937500, -0.0114746093750000, -0.0002441406250000,
+-0.0023803710937500, -0.0110473632812500, -0.0017395019531250, -0.0075073242187500,
+-0.0042419433593750, -0.0118103027343750, -0.0071716308593750, -0.0249633789062500,
+-0.0029296875000000, -0.0187072753906250, -0.0021057128906250, -0.0114746093750000,
+-0.0388793945312500, -0.0259094238281250, -0.0192871093750000, 0.0006103515625000,
+-0.0541687011718750, 0.0276794433593750, -0.1031494140625000, 0.0040283203125000,
+-0.0720520019531250, 0.0628051757812500, -0.1680908203125000, 0.0691833496093750,
+-0.1617431640625000, 0.0523681640625000, -0.1794738769531250, 0.0759277343750000,
+-0.2456665039062500, 0.0803527832031250, -0.2173767089843750, 0.0116271972656250,
+-0.2464904785156250, 0.0325622558593750, -0.2611999511718750, 0.0110778808593750,
+-0.2441101074218750, -0.0579528808593750, -0.2362365722656250, -0.0292053222656250,
+-0.2442321777343750, -0.0320129394531250, -0.2324829101562500, -0.0634155273437500,
+-0.2194213867187500, -0.0211791992187500, -0.2456359863281250, 0.0279541015625000,
+-0.2605590820312500, -0.0076293945312500, -0.2413940429687500, 0.0411987304687500,
+-0.3144836425781250, 0.0729980468750000, -0.3083496093750000, 0.0145874023437500,
+-0.3110046386718750, -0.0093078613281250, -0.3616333007812500, 0.0303039550781250,
+-0.3479309082031250, -0.1187744140625000, -0.3169555664062500, -0.1102294921875000,
+-0.3464355468750000, -0.1130065917968750, -0.3043518066406250, -0.2093200683593750,
+-0.2449340820312500, -0.1834106445312500, -0.2840881347656250, -0.1224365234375000,
+-0.2040405273437500, -0.1712036132812500, -0.2019042968750000, -0.1066284179687500,
+-0.2010192871093750, -0.0043640136718750, -0.1646423339843750, -0.0649108886718750,
+-0.1777038574218750, 0.0481872558593750, -0.1457824707031250, 0.0653381347656250,
+-0.1645202636718750, 0.0478210449218750, -0.1182556152343750, 0.0814208984375000,
+-0.0766906738281250, 0.0798339843750000, -0.1147460937500000, 0.0761718750000000,
+0.0744323730468750, 0.0666503906250000, 0.0220031738281250, 0.0901794433593750,
+0.1138000488281250, 0.1075439453125000, 0.2979431152343750, 0.0892333984375000,
+0.1912841796875000, 0.1536254882812500, 0.3865661621093750, 0.1532592773437500,
+0.4109497070312500, 0.1498107910156250, 0.3739929199218750, 0.1950988769531250,
+0.4593811035156250, 0.2201843261718750, 0.1722717285156250, 0.4230651855468750,
+0.3914184570312500, 0.3414611816406250, 0.3665771484375000, 0.3936462402343750,
+0.3535766601562500, 0.5282592773437500, 0.4115905761718750, 0.4785461425781250,
+0.4156188964843750, 0.4349365234375000, 0.3278503417968750, 0.5139160156250000,
+0.3482360839843750, 0.3896484375000000, 0.3253784179687500, 0.2583923339843750,
+0.2547302246093750, 0.3628540039062500, 0.2818298339843750, 0.0412597656250000,
+0.2337646484375000, 0.0708923339843750, 0.2398071289062500, 0.0138549804687500,
+0.2103271484375000, -0.2048645019531250, 0.1708374023437500, -0.1098327636718750,
+0.2015075683593750, -0.2442932128906250, 0.0938110351562500, -0.2712402343750000,
+0.1063842773437500, -0.2510986328125000, 0.0841369628906250, -0.2946472167968750,
+0.0068664550781250, -0.2902832031250000, 0.0388793945312500, -0.2696228027343750,
+-0.0132446289062500, -0.2772827148437500, -0.0120849609375000, -0.2735900878906250,
+-0.0307006835937500, -0.2527465820312500, -0.0650024414062500, -0.2601623535156250,
+-0.0419006347656250, -0.2781677246093750, -0.1174316406250000, -0.2581787109375000,
+-0.1155700683593750, -0.2932434082031250, -0.1129760742187500, -0.3331604003906250,
+-0.1610717773437500, -0.3013916015625000, -0.1509094238281250, -0.3838500976562500,
+-0.0983886718750000, -0.3840026855468750, -0.1270141601562500, -0.3779296875000000,
+-0.0893554687500000, -0.4232177734375000, -0.0171508789062500, -0.4130859375000000,
+-0.0538635253906250, -0.3713684082031250, -0.0052490234375000, -0.4017028808593750,
+0.0107116699218750, -0.3554992675781250, -0.0021972656250000, -0.2922363281250000,
+-0.0001831054687500, -0.3364868164062500, 0.0032348632812500, -0.2247619628906250,
+0.0116882324218750, -0.2347106933593750, -0.0019531250000000, -0.2147521972656250,
+0.0206909179687500, -0.1415100097656250, 0.0515747070312500, -0.1756286621093750,
+0.0323486328125000, -0.1090087890625000, 0.0630493164062500, -0.1109619140625000,
+0.0776977539062500, -0.0993957519531250, 0.0531005859375000, -0.0592346191406250,
+0.0462646484375000, -0.0791015625000000, 0.0613403320312500, -0.0291442871093750,
+0.0036621093750000, -0.0362243652343750, 0.0058288574218750, -0.0180053710937500,
+0.0045471191406250, 0.0217590332031250, -0.0290527343750000, -0.0009765625000000,
+-0.0204772949218750, 0.0506286621093750, -0.0146789550781250, 0.0520629882812500,
+-0.0219726562500000, 0.0481872558593750, -0.0148620605468750, 0.0744934082031250,
+-0.0022888183593750, 0.0691833496093750, -0.0079345703125000, 0.0529479980468750,
+-0.0012207031250000, 0.0642395019531250, 0.0010681152343750, 0.0530700683593750,
+-0.0005493164062500, 0.0303039550781250, 0.0001220703125000, 0.0429077148437500,
+0.0003051757812500, 0.0277709960937500, 0.0000305175781250, 0.0222167968750000,
+-0.0002441406250000, 0.0286560058593750, -0.0000610351562500, 0.0289306640625000,
+0.0001220703125000, 0.0253295898437500, 0.0000305175781250, 0.0304565429687500,
+0.0000305175781250, 0.0344543457031250, 0.0000305175781250, 0.0260925292968750,
+0.0000610351562500, 0.0235595703125000, 0.0000610351562500, 0.0303955078125000,
+0.0000610351562500, 0.0034484863281250, 0.0000915527343750, 0.0108947753906250,
+0.0000610351562500, 0.0009765625000000, 0.0000915527343750, -0.0215759277343750,
+0.0001220703125000, -0.0070190429687500, 0.0000610351562500, -0.0355224609375000,
+0.0001220703125000},
+{-0.0658874511718750, 0.3717346191406250, 0.1075744628906250, 0.4755859375000000,
+0.0102844238281250, 0.3322143554687500, 0.1212158203125000, 0.1960144042968750,
+0.2739257812500000, 0.3036193847656250, 0.1895141601562500, 0.0127258300781250,
+0.3439331054687500, 0.0324707031250000, 0.3726501464843750, -0.0133666992187500,
+0.3319702148437500, -0.2019958496093750, 0.3734741210937500, -0.1208190917968750,
+0.3821105957031250, -0.2428894042968750, 0.2944641113281250, -0.2648010253906250,
+0.3096618652343750, -0.2501220703125000, 0.2915039062500000, -0.2929382324218750,
+0.2285461425781250, -0.2834777832031250, 0.2523498535156250, -0.2642211914062500,
+0.2066650390625000, -0.2783508300781250, 0.2148437500000000, -0.2647094726562500,
+0.1843872070312500, -0.2356567382812500, 0.1422729492187500, -0.2531127929687500,
+0.1733093261718750, -0.2492065429687500, 0.0742187500000000, -0.2260437011718750,
+0.0831298828125000, -0.2672119140625000, 0.0666198730468750, -0.2998962402343750,
+0.0001831054687500, -0.2659606933593750, 0.0260620117187500, -0.3664855957031250,
+-0.0189514160156250, -0.3602905273437500, -0.0160522460937500, -0.3601684570312500,
+-0.0366516113281250, -0.4244079589843750, -0.0711975097656250, -0.4055480957031250,
+-0.0469970703125000, -0.3643798828125000, -0.1131591796875000, -0.4024963378906250,
+-0.1171875000000000, -0.3447265625000000, -0.1033630371093750, -0.2694091796875000,
+-0.1338500976562500, -0.3215026855468750, -0.1343994140625000, -0.1886901855468750,
+-0.0722351074218750, -0.1943969726562500, -0.0911560058593750, -0.1805419921875000,
+-0.0665588378906250, -0.1022033691406250, -0.0041503906250000, -0.1341247558593750,
+-0.0308227539062500, -0.0829772949218750, -0.0018615722656250, -0.0831298828125000,
+0.0104064941406250, -0.0732116699218750, 0.0024414062500000, -0.0437622070312500,
+-0.0003051757812500, -0.0606384277343750, 0.0017700195312500, -0.0111694335937500,
+0.0200500488281250, -0.0219726562500000, 0.0082702636718750, -0.0018920898437500,
+0.0254211425781250, 0.0386962890625000, 0.0576171875000000, 0.0124511718750000,
+0.0411071777343750, 0.0759277343750000, 0.0521850585937500, 0.0732116699218750,
+0.0691833496093750, 0.0758361816406250, 0.0437316894531250, 0.1163635253906250,
+0.0225219726562500, 0.1037292480468750, 0.0407104492187500, 0.0945434570312500,
+-0.0066223144531250, 0.1106262207031250, -0.0105285644531250, 0.0886840820312500,
+-0.0050964355468750, 0.0615539550781250, -0.0243530273437500, 0.0816040039062500,
+-0.0222778320312500, 0.0417175292968750, -0.0097351074218750, 0.0396728515625000,
+-0.0134277343750000, 0.0405883789062500, -0.0103454589843750, 0.0229797363281250,
+0.0003356933593750, 0.0288391113281250, -0.0035705566406250, 0.0211791992187500,
+-0.0006103515625000, 0.0231628417968750, 0.0010375976562500, 0.0182189941406250,
+0.0000915527343750, 0.0113830566406250, -0.0000915527343750, 0.0181274414062500,
+0.0000305175781250, -0.0011596679687500, 0.0001220703125000, 0.0007934570312500,
+-0.0000305175781250, 0.0009155273437500, 0.0000305175781250, -0.0118103027343750,
+0.0001831054687500, -0.0082397460937500, 0.0001831054687500, -0.0024414062500000,
+0.0000610351562500, -0.0111694335937500, 0.0000610351562500, -0.0024414062500000,
+0.0000610351562500, 0.0086975097656250, 0.0001220703125000, -0.0018615722656250,
+0.0000915527343750, 0.0246582031250000, 0.0000610351562500, 0.0182189941406250,
+0.0000915527343750, 0.0244750976562500, 0.0000915527343750, 0.0447998046875000,
+0.0000610351562500, 0.0323791503906250, 0.0000610351562500, 0.0513305664062500,
+0.0001525878906250, 0.0494689941406250, 0.0001220703125000, 0.0492553710937500,
+0.0001220703125000, 0.0590820312500000, 0.0000915527343750, 0.0520935058593750,
+0.0001220703125000, 0.0561828613281250, 0.0000915527343750, 0.0558166503906250,
+0.0001220703125000, 0.0530395507812500, 0.0000915527343750, 0.0528869628906250,
+0.0001220703125000, 0.0508728027343750, 0.0000915527343750, 0.0489807128906250,
+0.0001525878906250, 0.0483398437500000, 0.0001220703125000, 0.0459594726562500,
+0.0001220703125000, 0.0440979003906250, 0.0001220703125000, 0.0442810058593750,
+0.0001220703125000, 0.0394592285156250, 0.0001220703125000, 0.0383911132812500,
+0.0001220703125000, 0.0361633300781250, 0.0001220703125000, 0.0321655273437500,
+0.0001525878906250, 0.0306701660156250, 0.0001525878906250, 0.0245971679687500,
+0.0001831054687500, 0.0252075195312500, 0.0002136230468750, 0.0184936523437500,
+0.0001831054687500, 0.0114440917968750, 0.0001831054687500, 0.0143432617187500,
+0.0002136230468750, -0.0049133300781250, 0.0001220703125000, -0.0018615722656250,
+0.0001220703125000, -0.0107421875000000, 0.0001220703125000, -0.0281066894531250,
+0.0001220703125000, -0.0213012695312500, 0.0001220703125000, -0.0437927246093750,
+-0.0000610351562500, -0.0433654785156250, -0.0000610351562500, -0.0466918945312500,
+0.0000305175781250, -0.0607604980468750, -0.0000915527343750, -0.0560607910156250,
+-0.0001220703125000, -0.0648803710937500, 0.0001220703125000, -0.0648193359375000,
+0.0003356933593750, -0.0652770996093750, -0.0004882812500000, -0.0695800781250000,
+-0.0002746582031250, -0.0678100585937500, 0.0001831054687500, -0.0747070312500000,
+-0.0050048828125000, -0.0714721679687500, -0.0024414062500000, -0.0788574218750000,
+-0.0061340332031250, -0.0886230468750000, -0.0130615234375000, -0.0821838378906250,
+-0.0083923339843750, -0.1010742187500000, -0.0125427246093750, -0.1018066406250000,
+-0.0177917480468750, -0.1056518554687500, -0.0069274902343750, -0.1174926757812500,
+-0.0020141601562500, -0.1147155761718750, -0.0107116699218750, -0.1260681152343750,
+0.0232238769531250, -0.1236572265625000, 0.0169982910156250, -0.1338500976562500,
+0.0224609375000000, -0.1444702148437500, 0.0498962402343750, -0.1357421875000000,
+0.0388183593750000, -0.1726989746093750, 0.0366210937500000, -0.1683349609375000,
+0.0492248535156250, -0.1794433593750000, 0.0324401855468750, -0.2095336914062500,
+0.0132751464843750, -0.1965332031250000, 0.0281677246093750, -0.2114257812500000,
+0.0001525878906250, -0.2176208496093750, -0.0030822753906250, -0.2098693847656250,
+0.0014038085937500, -0.2084350585937500, -0.0072937011718750, -0.2109069824218750,
+-0.0075073242187500, -0.1991271972656250, -0.0088806152343750, -0.2007751464843750,
+-0.0035400390625000, -0.1943664550781250, -0.0154418945312500, -0.1859130859375000,
+-0.0259704589843750, -0.1908874511718750, -0.0149230957031250, -0.1633605957031250,
+-0.0428161621093750, -0.1716918945312500, -0.0447082519531250, -0.1531066894531250,
+-0.0391235351562500, -0.1245422363281250, -0.0495605468750000, -0.1417846679687500,
+-0.0503234863281250, -0.0887756347656250, -0.0386962890625000, -0.0947875976562500,
+-0.0374450683593750, -0.0768127441406250, -0.0455322265625000, -0.0362854003906250,
+-0.0441894531250000, -0.0561523437500000, -0.0378417968750000, -0.0057373046875000,
+-0.0590515136718750, -0.0083618164062500, -0.0653381347656250, -0.0006713867187500,
+-0.0414428710937500, 0.0319824218750000, -0.0440368652343750, 0.0181274414062500,
+-0.0596923828125000, 0.0377502441406250, 0.0385742187500000, 0.0406188964843750,
+0.0086975097656250, 0.0399780273437500, 0.0544738769531250, 0.0456237792968750,
+0.1519470214843750, 0.0431213378906250, 0.0994262695312500, 0.0593566894531250,
+0.1915283203125000, 0.0501098632812500, 0.2022094726562500, 0.0665283203125000,
+0.1928100585937500, 0.0940551757812500, 0.2324218750000000, 0.0938415527343750,
+0.2261352539062500, 0.1529235839843750, 0.2021179199218750, 0.1281738281250000,
+0.2184753417968750, 0.1473693847656250, 0.1999206542968750, 0.1968994140625000,
+0.1703186035156250, 0.1651611328125000, 0.1885681152343750, 0.2456054687500000,
+0.1282348632812500, 0.2361755371093750, 0.1292419433593750, 0.2447204589843750,
+0.1238403320312500, 0.3057861328125000, 0.0866394042968750, 0.2801208496093750,
+0.0975952148437500, 0.2593383789062500, 0.0943298339843750, 0.3078002929687500,
+0.0889892578125000, 0.2264099121093750, 0.0856018066406250, 0.1449279785156250,
+0.0898437500000000, 0.2152404785156250, 0.0915222167968750, 0.0130004882812500,
+0.0534057617187500, 0.0214538574218750, 0.0656127929687500, 0.0113220214843750,
+0.0473022460937500, -0.1109313964843750, 0.0058288574218750, -0.0707397460937500,
+0.0245666503906250, -0.0674133300781250, 0.0021667480468750, -0.0974426269531250,
+-0.0065002441406250, -0.0691223144531250, -0.0018920898437500, -0.0277099609375000,
+-0.0001220703125000, -0.0501708984375000, -0.0025634765625000, -0.0312500000000000,
+-0.0270080566406250, -0.0155334472656250, -0.0075683593750000, -0.0323486328125000,
+-0.0427551269531250, -0.0463562011718750, -0.0871582031250000, -0.0352172851562500,
+-0.0562133789062500, -0.0465087890625000, -0.1411132812500000, -0.0560913085937500,
+-0.1398010253906250, -0.0424499511718750, -0.1441955566406250, -0.0330505371093750,
+-0.1928100585937500, -0.0430908203125000, -0.1756591796875000, -0.0220947265625000,
+-0.1803283691406250, -0.0202941894531250, -0.1920166015625000, -0.0213623046875000,
+-0.1758422851562500, -0.0150756835937500, -0.1621093750000000, -0.0162658691406250,
+-0.1728515625000000, -0.0013732910156250, -0.1416931152343750, -0.0115051269531250,
+-0.1420288085937500, 0.0054321289062500, -0.1387329101562500, 0.0292053222656250,
+-0.1202392578125000, 0.0113525390625000, -0.1269531250000000, 0.0479125976562500,
+-0.1205749511718750, 0.0520935058593750, -0.1180725097656250, 0.0400390625000000,
+-0.1160278320312500, 0.0522460937500000, -0.1141357421875000, 0.0557250976562500,
+-0.1155090332031250, 0.0208435058593750, -0.1019897460937500, 0.0323791503906250,
+-0.1046142578125000, 0.0214233398437500, -0.0972595214843750, -0.0105590820312500,
+-0.0830383300781250, 0.0058898925781250, -0.0887145996093750, -0.0126647949218750,
+-0.0765686035156250, -0.0193481445312500, -0.0729675292968750, -0.0115356445312500,
+-0.0765991210937500, -0.0126037597656250, -0.0740966796875000, -0.0160217285156250,
+-0.0716247558593750, -0.0048522949218750, -0.0806579589843750, -0.0054016113281250,
+-0.0799255371093750, -0.0053100585937500, -0.0789489746093750, 0.0007019042968750,
+-0.0845031738281250, -0.0010986328125000, -0.0834350585937500, -0.0003662109375000,
+-0.0755310058593750, 0.0003967285156250, -0.0789184570312500, 0.0000000000000000,
+-0.0714416503906250, -0.0002746582031250, -0.0600280761718750, -0.0001831054687500,
+-0.0642395019531250, -0.0000305175781250, -0.0482177734375000, -0.0001525878906250,
+-0.0477905273437500, -0.0001220703125000, -0.0448303222656250, 0.0001220703125000,
+-0.0343017578125000, 0.0000305175781250, -0.0367431640625000, 0.0000915527343750,
+-0.0310058593750000, 0.0001220703125000, -0.0286865234375000, 0.0001220703125000,
+-0.0289306640625000, 0.0001525878906250, -0.0273742675781250, 0.0001220703125000,
+-0.0272216796875000, 0.0001525878906250, -0.0291442871093750, 0.0001525878906250,
+-0.0296936035156250, 0.0001220703125000, -0.0300598144531250, 0.0000915527343750,
+-0.0315551757812500, 0.0001220703125000, -0.0319519042968750, 0.0000610351562500,
+-0.0307006835937500, 0.0000915527343750, -0.0339355468750000, 0.0000915527343750,
+-0.0306396484375000, 0.0000610351562500, -0.0273742675781250, 0.0000305175781250,
+-0.0322875976562500, 0.0000305175781250, -0.0205688476562500, -0.0000305175781250,
+-0.0231018066406250, -0.0000305175781250, -0.0210266113281250, -0.0000305175781250,
+-0.0123901367187500, -0.0000610351562500, -0.0168762207031250, -0.0000610351562500,
+-0.0098266601562500, -0.0000610351562500, -0.0101623535156250, -0.0000610351562500,
+-0.0105590820312500},
+{-0.0087280273437500, 0.1958618164062500, -0.0060424804687500, 0.0945739746093750,
+-0.0147094726562500, 0.1726379394531250, -0.0133361816406250, -0.0152587890625000,
+-0.0377807617187500, -0.0160827636718750, -0.0216979980468750, -0.0105285644531250,
+-0.0542907714843750, -0.1093444824218750, -0.0944824218750000, -0.0840148925781250,
+-0.0641784667968750, -0.0617675781250000, -0.1427307128906250, -0.0848083496093750,
+-0.1412963867187500, -0.0636596679687500, -0.1430664062500000, -0.0213623046875000,
+-0.1872558593750000, -0.0400695800781250, -0.1737670898437500, -0.0307922363281250,
+-0.1758422851562500, -0.0171203613281250, -0.1853332519531250, -0.0296936035156250,
+-0.1726379394531250, -0.0443115234375000, -0.1627197265625000, -0.0350646972656250,
+-0.1717224121093750, -0.0388488769531250, -0.1406555175781250, -0.0479736328125000,
+-0.1468200683593750, -0.0357055664062500, -0.1361083984375000, -0.0234985351562500,
+-0.1101074218750000, -0.0335083007812500, -0.1248168945312500, -0.0166931152343750,
+-0.1019897460937500, -0.0147399902343750, -0.0986938476562500, -0.0147705078125000,
+-0.1014709472656250, -0.0102539062500000, -0.0938415527343750, -0.0122680664062500,
+-0.0953674316406250, 0.0047302246093750, -0.0944824218750000, -0.0045471191406250,
+-0.0952148437500000, 0.0096740722656250, -0.0929870605468750, 0.0328063964843750,
+-0.0912475585937500, 0.0162353515625000, -0.0938720703125000, 0.0427246093750000,
+-0.0864257812500000, 0.0480651855468750, -0.0882873535156250, 0.0357971191406250,
+-0.0858764648437500, 0.0416870117187500, -0.0796203613281250, 0.0472106933593750,
+-0.0829772949218750, 0.0146179199218750, -0.0759887695312500, 0.0226745605468750,
+-0.0773925781250000, 0.0148620605468750, -0.0724182128906250, -0.0121765136718750,
+-0.0658569335937500, 0.0004272460937500, -0.0698852539062500, -0.0112915039062500,
+-0.0507507324218750, -0.0172729492187500, -0.0531311035156250, -0.0107421875000000,
+-0.0468444824218750, -0.0092773437500000, -0.0305175781250000, -0.0128173828125000,
+-0.0373229980468750, -0.0035400390625000, -0.0295104980468750, -0.0034484863281250,
+-0.0238037109375000, -0.0037536621093750, -0.0310058593750000, 0.0006103515625000,
+-0.0341186523437500, -0.0004882812500000, -0.0299377441406250, -0.0003662109375000,
+-0.0496826171875000, 0.0000610351562500, -0.0466308593750000, -0.0001220703125000,
+-0.0552978515625000, -0.0003967285156250, -0.0725708007812500, -0.0003356933593750,
+-0.0652160644531250, -0.0001525878906250, -0.0870056152343750, -0.0002746582031250,
+-0.0873107910156250, -0.0002136230468750, -0.0928039550781250, -0.0000610351562500,
+-0.1069030761718750, -0.0000915527343750, -0.1010131835937500, -0.0000610351562500,
+-0.1152038574218750, -0.0000610351562500, -0.1152038574218750, 0.0000000000000000,
+-0.1191711425781250, 0.0000000000000000, -0.1284790039062500, 0.0000000000000000,
+-0.1233825683593750, 0.0000305175781250, -0.1343078613281250, 0.0000610351562500,
+-0.1343383789062500, 0.0000305175781250, -0.1360473632812500, 0.0000305175781250,
+-0.1429748535156250, 0.0000305175781250, -0.1409606933593750, 0.0000000000000000,
+-0.1480712890625000, 0.0000000000000000, -0.1491088867187500, -0.0000305175781250,
+-0.1484069824218750, -0.0000305175781250, -0.1521606445312500, 0.0000000000000000,
+-0.1534118652343750, -0.0000610351562500, -0.1496582031250000, -0.0000610351562500,
+-0.1535339355468750, -0.0000305175781250, -0.1504821777343750, -0.0000610351562500,
+-0.1447143554687500, -0.0000610351562500, -0.1489868164062500, 0.0000000000000000,
+-0.1389465332031250, 0.0000305175781250, -0.1396179199218750, 0.0000000000000000,
+-0.1398315429687500, 0.0000000000000000, -0.1354370117187500, 0.0000305175781250,
+-0.1380004882812500, -0.0000915527343750, -0.1341552734375000, -0.0000915527343750,
+-0.1401367187500000, -0.0000915527343750, -0.1295471191406250, 0.0001220703125000,
+-0.1233520507812500, 0.0000000000000000, -0.1346740722656250, -0.0011901855468750,
+-0.0833435058593750, 0.0003051757812500, -0.1047668457031250, -0.0036926269531250,
+-0.0776062011718750, -0.0072326660156250, -0.0244445800781250, -0.0038452148437500,
+-0.0606689453125000, -0.0145568847656250, 0.0088500976562500, -0.0162658691406250,
+0.0122375488281250, -0.0100402832031250, 0.0075988769531250, -0.0144042968750000,
+0.0397644042968750, -0.0168762207031250, 0.0325012207031250, 0.0167846679687500,
+0.0284729003906250, 0.0067138671875000, 0.0289306640625000, 0.0197753906250000,
+0.0373535156250000, 0.0555725097656250, 0.0357360839843750, 0.0387573242187500,
+0.0296630859375000, 0.0405273437500000, 0.0773925781250000, 0.0603942871093750,
+0.0638427734375000, 0.0300903320312500, 0.0818481445312500, -0.0014953613281250,
+0.1277160644531250, 0.0236206054687500, 0.1053771972656250, -0.0234985351562500,
+0.1315002441406250, -0.0366821289062500, 0.1398010253906250, -0.0122680664062500,
+0.1325683593750000, -0.0146484375000000, 0.1355895996093750, -0.0292358398437500,
+0.1377258300781250, 0.0199584960937500, 0.1373291015625000, 0.0256347656250000,
+0.1390075683593750, 0.0046081542968750, 0.1322326660156250, 0.0210876464843750,
+0.1315002441406250, 0.0310668945312500, 0.1365661621093750, -0.0500793457031250,
+0.0985107421875000, -0.0310974121093750, 0.1112976074218750, -0.0480346679687500,
+0.0918273925781250, -0.1198730468750000, 0.0528259277343750, -0.0930175781250000,
+0.0730285644531250, -0.0862121582031250, 0.0270385742187500, -0.1183776855468750,
+0.0304870605468750, -0.0740356445312500, 0.0184326171875000, -0.0222167968750000,
+-0.0095214843750000, -0.0581665039062500, 0.0072937011718750, 0.0187988281250000,
+-0.0488891601562500, 0.0274047851562500, -0.0319824218750000, 0.0202941894531250,
+-0.0653381347656250, 0.0479431152343750, -0.1213073730468750, 0.0442504882812500,
+-0.0862426757812500, 0.0497436523437500, -0.1829223632812500, 0.0439147949218750,
+-0.1754455566406250, 0.0615539550781250, -0.1915283203125000, 0.0774230957031250,
+-0.2574157714843750, 0.0623779296875000, -0.2285766601562500, 0.1102294921875000,
+-0.2601013183593750, 0.1076965332031250, -0.2746276855468750, 0.1132507324218750,
+-0.2545166015625000, 0.1402893066406250, -0.2524108886718750, 0.1310729980468750,
+-0.2624206542968750, 0.1603393554687500, -0.2145690917968750, 0.1452331542968750,
+-0.2193603515625000, 0.1846313476562500, -0.2161865234375000, 0.2245178222656250,
+-0.1838378906250000, 0.1894226074218750, -0.1935424804687500, 0.3216552734375000,
+-0.2028808593750000, 0.3080444335937500, -0.1947631835937500, 0.3230285644531250,
+-0.1968383789062500, 0.4222106933593750, -0.2146301269531250, 0.3867492675781250,
+-0.2108154296875000, 0.3457031250000000, -0.1752014160156250, 0.3999023437500000,
+-0.1949768066406250, 0.3270568847656250, -0.1744079589843750, 0.1448059082031250,
+-0.0983886718750000, -0.0956726074218750, -0.1848449707031250, -0.2249755859375000,
+-0.1419372558593750, -0.2082519531250000, -0.1823730468750000, -0.2105102539062500,
+-0.1393737792968750, -0.2485656738281250, -0.0824584960937500, -0.2425231933593750,
+-0.1137695312500000, -0.2174987792968750, -0.0534667968750000, -0.2332153320312500,
+-0.0575561523437500, -0.2136535644531250, -0.0314941406250000, -0.1761169433593750,
+0.0053100585937500, -0.1965942382812500, -0.0215148925781250, -0.1771240234375000,
+0.1121215820312500, -0.1536254882812500, 0.0761413574218750, -0.1990661621093750,
+0.1327209472656250, -0.2243652343750000, 0.2640686035156250, -0.1899108886718750,
+0.1967468261718750, -0.3123168945312500, 0.2916564941406250, -0.3009338378906250,
+0.3241577148437500, -0.3059387207031250, 0.2821350097656250, -0.3891906738281250,
+0.2861938476562500, -0.3626403808593750, 0.3048095703125000, -0.3209228515625000,
+0.2288818359375000, -0.3677062988281250, 0.2345275878906250, -0.3035888671875000,
+0.2231750488281250, -0.2164001464843750, 0.1780090332031250, -0.2758483886718750,
+0.1949157714843750, -0.1473388671875000, 0.1485595703125000, -0.1475219726562500,
+0.1602172851562500, -0.1386108398437500, 0.1298828125000000, -0.0707092285156250,
+0.0821838378906250, -0.0962219238281250, 0.1134948730468750, -0.0391235351562500,
+0.0342407226562500, -0.0504150390625000, 0.0342712402343750, -0.0167541503906250,
+0.0278320312500000, 0.0346984863281250, -0.0144042968750000, -0.0011901855468750,
+0.0011596679687500, 0.1080017089843750, -0.0329895019531250, 0.0983276367187500,
+-0.0274658203125000, 0.1090087890625000, -0.0437622070312500, 0.1816406250000000,
+-0.0766906738281250, 0.1537475585937500, -0.0581359863281250, 0.1614685058593750,
+-0.0906372070312500, 0.1803894042968750, -0.1034240722656250, 0.1610717773437500,
+-0.0799255371093750, 0.1427001953125000, -0.0773315429687500, 0.1572570800781250,
+-0.0909423828125000, 0.1260070800781250, -0.0292663574218750, 0.1302795410156250,
+-0.0335693359375000, 0.1195373535156250, -0.0288696289062500, 0.0966491699218750,
+0.0097961425781250, 0.1124572753906250, -0.0012817382812500, 0.0788269042968750,
+0.0013122558593750, 0.0779113769531250, 0.0041809082031250, 0.0795593261718750,
+0.0092773437500000, 0.0628662109375000, 0.0091552734375000, 0.0657348632812500,
+0.0039367675781250, 0.0662841796875000, 0.0346679687500000, 0.0689392089843750,
+0.0316772460937500, 0.0578613281250000, 0.0310058593750000, 0.0525817871093750,
+0.0529479980468750, 0.0611267089843750, 0.0491027832031250, 0.0202026367187500,
+0.0238647460937500, 0.0301513671875000, 0.0361938476562500, 0.0145263671875000,
+0.0210266113281250, -0.0197448730468750, -0.0107727050781250, 0.0015258789062500,
+0.0050659179687500, -0.0381774902343750, -0.0131225585937500, -0.0348815917968750,
+-0.0213928222656250, -0.0392150878906250, -0.0123901367187500, -0.0619812011718750,
+-0.0102844238281250, -0.0486755371093750, -0.0144653320312500, -0.0736694335937500,
+-0.0034179687500000, -0.0682373046875000, -0.0023498535156250, -0.0771484375000000,
+-0.0033569335937500, -0.0980529785156250, 0.0007324218750000, -0.0841674804687500,
+0.0000305175781250, -0.1105346679687500, -0.0002441406250000, -0.1110534667968750,
+-0.0000610351562500, -0.1085815429687500, -0.0000305175781250, -0.1212158203125000,
+-0.0001525878906250, -0.1179809570312500, -0.0001831054687500, -0.1131896972656250,
+-0.0000915527343750, -0.1195678710937500, 0.0000000000000000, -0.1138610839843750,
+-0.0000610351562500, -0.1053466796875000, -0.0001525878906250, -0.1127319335937500,
+-0.0000915527343750, -0.0981750488281250, -0.0000610351562500, -0.1019897460937500,
+-0.0000610351562500, -0.0989379882812500, -0.0000915527343750, -0.0899658203125000,
+-0.0000610351562500, -0.0997924804687500, -0.0000610351562500, -0.0858764648437500,
+-0.0000915527343750, -0.0910034179687500, -0.0000610351562500, -0.0884399414062500,
+-0.0000610351562500, -0.0782470703125000, -0.0000305175781250, -0.0878295898437500,
+-0.0000305175781250, -0.0744018554687500, 0.0000000000000000, -0.0778503417968750,
+0.0000305175781250, -0.0774841308593750, -0.0000305175781250, -0.0693359375000000,
+-0.0000610351562500, -0.0762329101562500, 0.0000305175781250, -0.0701599121093750,
+0.0000000000000000, -0.0728759765625000, -0.0000305175781250, -0.0725708007812500,
+-0.0000915527343750, -0.0687255859375000, -0.0001220703125000, -0.0743103027343750,
+-0.0000610351562500, -0.0690002441406250, -0.0001525878906250, -0.0702514648437500,
+-0.0001831054687500, -0.0707397460937500, -0.0001525878906250, -0.0680847167968750,
+-0.0002441406250000, -0.0702819824218750, -0.0002746582031250, -0.0659484863281250,
+-0.0000915527343750, -0.0674743652343750, -0.0001525878906250, -0.0630798339843750,
+-0.0002746582031250, -0.0582885742187500, 0.0000610351562500, -0.0613098144531250,
+0.0000610351562500},
+{0.0538024902343750, 0.2014465332031250, 0.0830383300781250, 0.1761169433593750,
+0.0185546875000000, 0.2054748535156250, 0.0160827636718750, 0.1738281250000000,
+0.0119323730468750, 0.1206665039062500, -0.0193481445312500, 0.1532592773437500,
+-0.0076599121093750, 0.1187133789062500, -0.0388488769531250, 0.1029357910156250,
+-0.0327453613281250, 0.1208801269531250, -0.0461120605468750, 0.1282348632812500,
+-0.0779724121093750, 0.1184082031250000, -0.0624084472656250, 0.1215515136718750,
+-0.0766296386718750, 0.1351013183593750, -0.0921020507812500, 0.1159973144531250,
+-0.0665588378906250, 0.0986022949218750, -0.0494689941406250, 0.1146850585937500,
+-0.0675659179687500, 0.0711364746093750, -0.0134277343750000, 0.0792236328125000,
+-0.0117187500000000, 0.0615539550781250, -0.0148315429687500, 0.0295715332031250,
+0.0096435546875000, 0.0519104003906250, 0.0054626464843750, -0.0171508789062500,
+0.0038452148437500, -0.0056457519531250, 0.0014343261718750, -0.0230712890625000,
+0.0117187500000000, -0.0757446289062500, 0.0182495117187500, -0.0494689941406250,
+0.0094299316406250, -0.0951232910156250, 0.0371704101562500, -0.0991516113281250,
+0.0400390625000000, -0.0968627929687500, 0.0307617187500000, -0.1182861328125000,
+0.0405273437500000, -0.1120605468750000, 0.0438842773437500, -0.1065368652343750,
+0.0102539062500000, -0.1162719726562500, 0.0172119140625000, -0.1047973632812500,
+0.0100402832031250, -0.0917663574218750, -0.0180664062500000, -0.1040039062500000,
+-0.0081176757812500, -0.0725708007812500, -0.0120544433593750, -0.0796508789062500,
+-0.0186767578125000, -0.0694885253906250, -0.0119934082031250, -0.0453186035156250,
+-0.0051269531250000, -0.0623779296875000, -0.0093383789062500, -0.0308227539062500,
+-0.0020446777343750, -0.0332641601562500, -0.0003967285156250, -0.0321655273437500,
+-0.0015563964843750, -0.0146789550781250, 0.0000610351562500, -0.0238342285156250,
+-0.0000305175781250, -0.0133666992187500, -0.0002441406250000, -0.0132446289062500,
+-0.0002746582031250, -0.0137023925781250, -0.0002136230468750, -0.0082397460937500,
+-0.0000610351562500, -0.0100708007812500, -0.0001525878906250, -0.0065307617187500,
+-0.0000305175781250, -0.0054016113281250, -0.0000305175781250, -0.0057983398437500,
+-0.0000610351562500, -0.0040588378906250, -0.0000610351562500, -0.0026245117187500,
+-0.0000610351562500, -0.0040893554687500, -0.0000610351562500, -0.0021667480468750,
+0.0000000000000000, -0.0034484863281250, -0.0000305175781250, -0.0056762695312500,
+-0.0000610351562500, -0.0024108886718750, 0.0000000000000000, -0.0052795410156250,
+-0.0000305175781250, -0.0054016113281250, -0.0000610351562500, -0.0037841796875000,
+-0.0000610351562500, -0.0037536621093750, -0.0000915527343750, -0.0032653808593750,
+-0.0001220703125000, -0.0017700195312500, -0.0000915527343750, -0.0006408691406250,
+-0.0000610351562500, 0.0004272460937500, -0.0001220703125000, 0.0011596679687500,
+-0.0001831054687500, 0.0014343261718750, -0.0001525878906250, 0.0028686523437500,
+0.0000305175781250, 0.0025329589843750, -0.0000610351562500, 0.0033264160156250,
+-0.0000915527343750, 0.0047607421875000, 0.0002136230468750, 0.0032653808593750,
+0.0001525878906250, 0.0025634765625000, -0.0014038085937500, 0.0036010742187500,
+-0.0000305175781250, 0.0001831054687500, -0.0025329589843750, -0.0026855468750000,
+-0.0054016113281250, -0.0017089843750000, -0.0027770996093750, -0.0126037597656250,
+-0.0086364746093750, -0.0111999511718750, -0.0095520019531250, -0.0164184570312500,
+-0.0064086914062500, -0.0268554687500000, -0.0087890625000000, -0.0240173339843750,
+-0.0102233886718750, -0.0354614257812500, 0.0045471191406250, -0.0374450683593750,
+-0.0016784667968750, -0.0366210937500000, 0.0068054199218750, -0.0425415039062500,
+0.0235900878906250, -0.0421142578125000, 0.0130615234375000, -0.0303649902343750,
+0.0272521972656250, -0.0360412597656250, 0.0324707031250000, -0.0245971679687500,
+0.0239868164062500, -0.0079040527343750, 0.0242614746093750, -0.0167541503906250,
+0.0293273925781250, 0.0086975097656250, 0.0097961425781250, 0.0101623535156250,
+0.0126953125000000, 0.0147094726562500, 0.0110168457031250, 0.0309448242187500,
+-0.0020446777343750, 0.0257873535156250, 0.0023193359375000, 0.0310668945312500,
+-0.0010986328125000, 0.0380249023437500, -0.0012207031250000, 0.0256652832031250,
+-0.0052185058593750, 0.0184936523437500, -0.0075073242187500, 0.0269165039062500,
+-0.0034790039062500, -0.0101013183593750, -0.0227050781250000, -0.0058288574218750,
+-0.0183715820312500, -0.0136108398437500, -0.0226440429687500, -0.0438232421875000,
+-0.0389099121093750, -0.0327148437500000, -0.0319824218750000, -0.0290222167968750,
+-0.0353088378906250, -0.0415954589843750, -0.0394287109375000, -0.0266723632812500,
+-0.0369567871093750, -0.0033569335937500, -0.0332336425781250, -0.0155334472656250,
+-0.0348815917968750, -0.0099792480468750, -0.0396423339843750, 0.0057067871093750,
+-0.0368347167968750, -0.0191040039062500, -0.0364685058593750, -0.0398559570312500,
+-0.0451049804687500, -0.0223999023437500, -0.0427856445312500, -0.0829162597656250,
+-0.0152282714843750, -0.0764465332031250, -0.0357360839843750, -0.0915832519531250,
+-0.0003662109375000, -0.1354370117187500, 0.0436096191406250, -0.1153564453125000,
+0.0104675292968750, -0.1513671875000000, 0.1043395996093750, -0.1558227539062500,
+0.0943908691406250, -0.1549072265625000, 0.1121826171875000, -0.1699829101562500,
+0.1771545410156250, -0.1655578613281250, 0.1481628417968750, -0.1698608398437500,
+0.1961059570312500, -0.1714782714843750, 0.2030639648437500, -0.1687927246093750,
+0.1912231445312500, -0.1705017089843750, 0.2100524902343750, -0.1711730957031250,
+0.2104797363281250, -0.1573486328125000, 0.1681518554687500, -0.1648254394531250,
+0.1873779296875000, -0.1500549316406250, 0.1587219238281250, -0.1315917968750000,
+0.1068420410156250, -0.1450195312500000, 0.1370544433593750, -0.1022033691406250,
+0.0859985351562500, -0.1085510253906250, 0.0780639648437500, -0.0963745117187500,
+0.0842285156250000, -0.0580139160156250, 0.0700073242187500, -0.0599975585937500,
+0.0705566406250000, -0.0205688476562500, 0.0526428222656250, -0.0340576171875000,
+0.0666809082031250, -0.0273437500000000, 0.0490112304687500, 0.0028991699218750,
+0.0219116210937500, -0.0127258300781250, 0.0397949218750000, 0.0187072753906250,
+-0.0144653320312500, 0.0147399902343750, -0.0156555175781250, 0.0255737304687500,
+-0.0172729492187500, 0.0477294921875000, -0.0440979003906250, 0.0324096679687500,
+-0.0369262695312500, 0.0739746093750000, -0.0509948730468750, 0.0654296875000000,
+-0.0450439453125000, 0.0805053710937500, -0.0647888183593750, 0.1136779785156250,
+-0.0850219726562500, 0.0948486328125000, -0.0684204101562500, 0.1462402343750000,
+-0.1315002441406250, 0.1368408203125000, -0.1255798339843750, 0.1593933105468750,
+-0.1362609863281250, 0.2032470703125000, -0.1810302734375000, 0.1751098632812500,
+-0.1640014648437500, 0.2345886230468750, -0.1776428222656250, 0.2420349121093750,
+-0.1872863769531250, 0.2217712402343750, -0.1773376464843750, 0.2474060058593750,
+-0.1732177734375000, 0.2528076171875000, -0.1786193847656250, 0.1578063964843750,
+-0.1597595214843750, 0.1996765136718750, -0.1664428710937500, 0.1339416503906250,
+-0.1519470214843750, 0.0184936523437500, -0.1319885253906250, 0.0861816406250000,
+-0.1468505859375000, -0.0451965332031250, -0.1018676757812500, -0.0606689453125000,
+-0.1019287109375000, -0.0393371582031250, -0.1040954589843750, -0.0868225097656250,
+-0.0790100097656250, -0.0830993652343750, -0.0823669433593750, -0.0442199707031250,
+-0.0967712402343750, -0.0555114746093750, -0.0892639160156250, -0.0484313964843750,
+-0.0909118652343750, -0.0151062011718750, -0.1065979003906250, -0.0257568359375000,
+-0.1007080078125000, -0.0283203125000000, -0.0849914550781250, -0.0203247070312500,
+-0.0942077636718750, -0.0244750976562500, -0.0769348144531250, -0.0353393554687500,
+-0.0513916015625000, -0.0313720703125000, -0.0650939941406250, -0.0247802734375000,
+-0.0381164550781250, -0.0317993164062500, -0.0318603515625000, -0.0243530273437500,
+-0.0343322753906250, -0.0117187500000000, -0.0249328613281250, -0.0195922851562500,
+-0.0238342285156250, -0.0083007812500000, -0.0288391113281250, -0.0074768066406250,
+-0.0272827148437500, -0.0057373046875000, -0.0247497558593750, -0.0014648437500000,
+-0.0273437500000000, -0.0046691894531250, -0.0263671875000000, 0.0119628906250000,
+-0.0142517089843750, 0.0060424804687500, -0.0158996582031250, 0.0135192871093750,
+-0.0099487304687500, 0.0311279296875000, 0.0018310546875000, 0.0207214355468750,
+-0.0015258789062500, 0.0311279296875000, 0.0079040527343750, 0.0380859375000000,
+0.0095825195312500, 0.0268249511718750, 0.0115356445312500, 0.0225830078125000,
+0.0168457031250000, 0.0306396484375000, 0.0159606933593750, 0.0041503906250000,
+0.0209960937500000, 0.0067749023437500, 0.0220031738281250, 0.0045166015625000,
+0.0213012695312500, -0.0128784179687500, 0.0230407714843750, -0.0066833496093750,
+0.0226745605468750, -0.0081787109375000, 0.0177917480468750, -0.0123901367187500,
+0.0190124511718750, -0.0082092285156250, 0.0140686035156250, -0.0039672851562500,
+0.0073852539062500, -0.0071105957031250, 0.0100097656250000, -0.0017395019531250,
+0.0003967285156250, -0.0010375976562500, -0.0010375976562500, -0.0016784667968750,
+-0.0041198730468750, 0.0001525878906250, -0.0108337402343750, -0.0001220703125000,
+-0.0101623535156250, -0.0003662109375000, -0.0164184570312500, -0.0002441406250000,
+-0.0175476074218750, -0.0002136230468750, -0.0199279785156250, -0.0003662109375000,
+-0.0242614746093750, -0.0003662109375000, -0.0235290527343750, -0.0003051757812500,
+-0.0280761718750000, -0.0003051757812500, -0.0300292968750000, -0.0002746582031250,
+-0.0294189453125000, -0.0002441406250000, -0.0308227539062500, -0.0002746582031250,
+-0.0325927734375000, -0.0001831054687500, -0.0301818847656250, -0.0001831054687500,
+-0.0306091308593750, -0.0001831054687500, -0.0315856933593750, -0.0001831054687500,
+-0.0306091308593750, -0.0001525878906250, -0.0308227539062500, -0.0001525878906250,
+-0.0327148437500000, -0.0001831054687500, -0.0334472656250000, -0.0001525878906250,
+-0.0328674316406250, -0.0001831054687500, -0.0339050292968750, -0.0001831054687500,
+-0.0350952148437500, -0.0002441406250000, -0.0307006835937500, 0.0001220703125000,
+-0.0340270996093750, -0.0010681152343750, -0.0313415527343750, -0.0013427734375000,
+-0.0272827148437500, -0.0006408691406250, -0.0316772460937500, -0.0068969726562500,
+-0.0171203613281250, -0.0050659179687500, -0.0227966308593750, -0.0075988769531250,
+-0.0168762207031250, -0.0154113769531250, -0.0022583007812500, -0.0120239257812500,
+-0.0132141113281250, -0.0071716308593750, -0.0048217773437500, -0.0154418945312500,
+0.0043334960937500, 0.0014038085937500, -0.0151062011718750, 0.0195617675781250,
+-0.0227661132812500, 0.0053710937500000, -0.0079650878906250, 0.0372009277343750,
+-0.0679931640625000, 0.0447692871093750, -0.0539245605468750, 0.0292358398437500,
+-0.0744628906250000, 0.0342407226562500, -0.1273193359375000, 0.0420227050781250,
+-0.1004333496093750, -0.0038757324218750, -0.1404724121093750, -0.0005187988281250,
+-0.1470031738281250, 0.0010681152343750, -0.1348266601562500, -0.0280151367187500,
+-0.1469726562500000, -0.0228271484375000, -0.1489257812500000, 0.0046081542968750,
+-0.1203613281250000, -0.0061645507812500, -0.1323852539062500, -0.0008239746093750,
+-0.1064453125000000, 0.0305480957031250, -0.0777282714843750, 0.0214233398437500,
+-0.1005249023437500, -0.0105895996093750, -0.0001220703125000, 0.0125427246093750,
+-0.0281677246093750},
+{0.0074157714843750, -0.0252075195312500, 0.0033874511718750, -0.0194702148437500,
+0.0140991210937500, -0.0083312988281250, 0.0374450683593750, -0.0148925781250000,
+0.0285949707031250, -0.0050048828125000, 0.0417785644531250, -0.0047607421875000,
+0.0491333007812500, -0.0024108886718750, 0.0404663085937500, 0.0022277832031250,
+0.0380554199218750, -0.0015258789062500, 0.0433654785156250, 0.0140075683593750,
+0.0228576660156250, 0.0097961425781250, 0.0262756347656250, 0.0147094726562500,
+0.0187072753906250, 0.0295715332031250, 0.0020141601562500, 0.0220947265625000,
+0.0100097656250000, 0.0260314941406250, -0.0109252929687500, 0.0330810546875000,
+-0.0108337402343750, 0.0223999023437500, -0.0130310058593750, 0.0143737792968750,
+-0.0256347656250000, 0.0228881835937500, -0.0212402343750000, -0.0000915527343750,
+-0.0264587402343750, 0.0003662109375000, -0.0286865234375000, 0.0006103515625000,
+-0.0252075195312500, -0.0116577148437500, -0.0245361328125000, -0.0079345703125000,
+-0.0256042480468750, -0.0067138671875000, -0.0173339843750000, -0.0097656250000000,
+-0.0176086425781250, -0.0068054199218750, -0.0133666992187500, -0.0024414062500000,
+-0.0057373046875000, -0.0051269531250000, -0.0081176757812500, -0.0012207031250000,
+-0.0010070800781250, -0.0005798339843750, 0.0007934570312500, -0.0011596679687500,
+0.0015869140625000, -0.0000305175781250, 0.0055847167968750, -0.0001525878906250,
+0.0053100585937500, -0.0003662109375000, 0.0031433105468750, -0.0003356933593750,
+0.0086059570312500, -0.0003356933593750, 0.0029602050781250, -0.0003967285156250,
+-0.0034790039062500, -0.0004272460937500, 0.0036315917968750, -0.0002746582031250,
+-0.0145568847656250, -0.0003356933593750, -0.0110473632812500, -0.0002746582031250,
+-0.0155944824218750, -0.0001831054687500, -0.0294494628906250, -0.0002441406250000,
+-0.0222778320312500, -0.0000610351562500, -0.0361633300781250, 0.0000610351562500,
+-0.0350036621093750, -0.0003967285156250, -0.0389099121093750, -0.0002136230468750,
+-0.0478515625000000, -0.0000305175781250, -0.0420227050781250, -0.0036621093750000,
+-0.0583496093750000, -0.0018920898437500, -0.0551757812500000, -0.0050964355468750,
+-0.0611267089843750, -0.0108032226562500, -0.0741882324218750, -0.0073547363281250,
+-0.0657653808593750, -0.0102539062500000, -0.0822753906250000, -0.0156555175781250,
+-0.0821838378906250, -0.0039978027343750, -0.0815124511718750, 0.0037231445312500,
+-0.0919799804687500, -0.0044555664062500, -0.0892333984375000, 0.0278320312500000,
+-0.0793457031250000, 0.0286865234375000, -0.0897521972656250, 0.0249938964843750,
+-0.0747985839843750, 0.0430908203125000, -0.0571594238281250, 0.0415649414062500,
+-0.0730285644531250, 0.0108032226562500, -0.0319519042968750, 0.0209960937500000,
+-0.0355529785156250, 0.0111999511718750, -0.0317993164062500, -0.0218811035156250,
+-0.0048217773437500, -0.0096130371093750, -0.0156250000000000, -0.0015563964843750,
+-0.0117187500000000, -0.0155029296875000, -0.0033569335937500, -0.0018310546875000,
+-0.0180358886718750, 0.0250549316406250, -0.0287475585937500, 0.0114746093750000,
+-0.0187377929687500, 0.0038146972656250, -0.0508728027343750, 0.0249633789062500,
+-0.0451660156250000, -0.0062561035156250, -0.0596008300781250, -0.0453796386718750,
+-0.0852966308593750, -0.0179138183593750, -0.0671997070312500, -0.0623168945312500,
+-0.1113281250000000, -0.0778503417968750, -0.1163635253906250, -0.0517272949218750,
+-0.0981140136718750, -0.0506286621093750, -0.1203308105468750, -0.0657043457031250,
+-0.1269836425781250, -0.0102844238281250, -0.0226440429687500, -0.0137023925781250,
+-0.0591430664062500, -0.0094604492187500, -0.0009460449218750, 0.0235290527343750,
+0.1172180175781250, 0.0135498046875000, 0.0585327148437500, 0.0255432128906250,
+0.1476745605468750, 0.0247497558593750, 0.1719055175781250, 0.0330505371093750,
+0.1470642089843750, 0.0427551269531250, 0.1625976562500000, 0.0343933105468750,
+0.1687927246093750, 0.0679931640625000, 0.1313476562500000, 0.0644226074218750,
+0.1358337402343750, 0.0711364746093750, 0.1288146972656250, 0.0933837890625000,
+0.1026000976562500, 0.0848083496093750, 0.1141357421875000, 0.1090087890625000,
+0.1064758300781250, 0.0993957519531250, 0.0986328125000000, 0.1258850097656250,
+0.1089477539062500, 0.1545715332031250, 0.1184692382812500, 0.1303405761718750,
+0.1118774414062500, 0.2192993164062500, 0.1189270019531250, 0.2093200683593750,
+0.1300964355468750, 0.2224731445312500, 0.1098632812500000, 0.2911682128906250,
+0.0985717773437500, 0.2648620605468750, 0.1143188476562500, 0.2468261718750000,
+0.0516662597656250, 0.2876281738281250, 0.0666198730468750, 0.2209167480468750,
+0.0407104492187500, 0.1538391113281250, -0.0133056640625000, 0.2076110839843750,
+0.0169677734375000, 0.0340270996093750, -0.0564575195312500, 0.0534973144531250,
+-0.0493164062500000, 0.0149841308593750, -0.0711669921875000, -0.1090087890625000,
+-0.1204528808593750, -0.0533447265625000, -0.0929870605468750, -0.1499633789062500,
+-0.1819763183593750, -0.1605529785156250, -0.1653442382812500, -0.1542663574218750,
+-0.1944274902343750, -0.1969909667968750, -0.2721557617187500, -0.1864929199218750,
+-0.2317199707031250, -0.1756286621093750, -0.2748413085937500, -0.1884765625000000,
+-0.2990112304687500, -0.1765136718750000, -0.2625122070312500, -0.1406860351562500,
+-0.2252197265625000, -0.1572570800781250, -0.2272338867187500, -0.2667541503906250,
+-0.2598876953125000, -0.2298278808593750, -0.2879028320312500, -0.2474365234375000,
+-0.2376708984375000, -0.3086547851562500, -0.2023925781250000, -0.2915649414062500,
+-0.2383422851562500, -0.2627258300781250, -0.1412658691406250, -0.2910461425781250,
+-0.1349182128906250, -0.2511596679687500, -0.1497497558593750, -0.1971740722656250,
+-0.1109924316406250, -0.2324218750000000, -0.1120300292968750, -0.1445007324218750,
+-0.1462097167968750, -0.1487426757812500, -0.1443786621093750, -0.1263732910156250,
+-0.1323852539062500, -0.0736389160156250, -0.1495056152343750, -0.1015319824218750,
+-0.1547241210937500, -0.0009155273437500, -0.0947570800781250, -0.0269165039062500,
+-0.1109924316406250, 0.0237426757812500, -0.0882873535156250, 0.1212768554687500,
+-0.0346069335937500, 0.0617370605468750, -0.0592346191406250, 0.1755371093750000,
+-0.0063171386718750, 0.1912841796875000, -0.0150146484375000, 0.1645812988281250,
+0.0162353515625000, 0.2049865722656250, 0.0606994628906250, 0.2097473144531250,
+0.0287170410156250, 0.1356811523437500, 0.1411132812500000, 0.1618041992187500,
+0.1303710937500000, 0.1342773437500000, 0.1484680175781250, 0.0605163574218750,
+0.2306823730468750, 0.0975036621093750, 0.1995239257812500, 0.0610656738281250,
+0.2069702148437500, 0.0362243652343750, 0.2323913574218750, 0.0759582519531250,
+0.2008361816406250, 0.0942687988281250, 0.1689758300781250, 0.0659790039062500,
+0.1911315917968750, 0.1435241699218750, 0.1393127441406250, 0.1532592773437500,
+0.1392822265625000, 0.1217651367187500, 0.1301879882812500, 0.1455383300781250,
+0.1028747558593750, 0.1604003906250000, 0.1166381835937500, 0.0568237304687500,
+0.0683288574218750, 0.0813903808593750, 0.0778503417968750, 0.0606994628906250,
+0.0590209960937500, -0.0236206054687500, 0.0153503417968750, 0.0100097656250000,
+0.0361633300781250, -0.0169372558593750, -0.0034484863281250, -0.0317077636718750,
+-0.0056457519531250, -0.0230712890625000, -0.0102844238281250, -0.0227661132812500,
+-0.0296630859375000, -0.0257568359375000, -0.0204162597656250, -0.0210876464843750,
+-0.0493469238281250, -0.0235290527343750, -0.0474853515625000, -0.0188598632812500,
+-0.0479431152343750, -0.0113220214843750, -0.0691223144531250, -0.0173034667968750,
+-0.0637817382812500, -0.0133361816406250, -0.0426025390625000, -0.0093994140625000,
+-0.0564575195312500, -0.0149230957031250, -0.0373229980468750, -0.0193176269531250,
+-0.0034790039062500, -0.0155944824218750, -0.0211486816406250, -0.0223388671875000,
+0.0034790039062500, -0.0233154296875000, 0.0105895996093750, -0.0251770019531250,
+0.0048828125000000, -0.0276489257812500, 0.0067443847656250, -0.0256042480468750,
+0.0080566406250000, -0.0369567871093750, 0.0164794921875000, -0.0338134765625000,
+0.0095520019531250, -0.0394287109375000, 0.0187072753906250, -0.0501403808593750,
+0.0361938476562500, -0.0446166992187500, 0.0271301269531250, -0.0605773925781250,
+0.0303344726562500, -0.0594482421875000, 0.0407104492187500, -0.0637512207031250,
+0.0249023437500000, -0.0767517089843750, 0.0096740722656250, -0.0724182128906250,
+0.0213928222656250, -0.0786437988281250, -0.0065002441406250, -0.0840148925781250,
+-0.0103759765625000, -0.0781555175781250, -0.0057678222656250, -0.0762939453125000,
+-0.0148620605468750, -0.0821228027343750, -0.0148315429687500, -0.0633239746093750,
+-0.0060119628906250, -0.0675354003906250, -0.0072937011718750, -0.0631408691406250,
+-0.0060119628906250, -0.0485229492187500, -0.0000305175781250, -0.0565490722656250,
+-0.0018310546875000, -0.0463562011718750, -0.0005493164062500, -0.0454406738281250,
+0.0002746582031250, -0.0474548339843750, -0.0002136230468750, -0.0446472167968750,
+-0.0003662109375000, -0.0465087890625000, -0.0002746582031250, -0.0461425781250000,
+-0.0003051757812500, -0.0467834472656250, -0.0003051757812500, -0.0469055175781250,
+-0.0003662109375000, -0.0464477539062500, -0.0003662109375000, -0.0475158691406250,
+-0.0003356933593750, -0.0468444824218750, -0.0003662109375000, -0.0461730957031250,
+-0.0003662109375000, -0.0474548339843750, -0.0003356933593750, -0.0484924316406250,
+-0.0002441406250000, -0.0479125976562500, -0.0002746582031250, -0.0496215820312500,
+-0.0008544921875000, -0.0506591796875000, -0.0001525878906250, -0.0498352050781250,
+-0.0016784667968750, -0.0500488281250000, -0.0029296875000000, -0.0509948730468750,
+-0.0014648437500000, -0.0465393066406250, -0.0060729980468750, -0.0477905273437500,
+-0.0056762695312500, -0.0466308593750000, -0.0050964355468750, -0.0429687500000000,
+-0.0083923339843750, -0.0455017089843750, -0.0077209472656250, -0.0441894531250000,
+-0.0013732910156250, -0.0436706542968750, -0.0059509277343750, -0.0457763671875000,
+0.0011596679687500, -0.0473022460937500, 0.0111389160156250, -0.0471496582031250,
+0.0035095214843750, -0.0487060546875000, 0.0187988281250000, -0.0504760742187500,
+0.0202636718750000, -0.0476074218750000, 0.0168457031250000, -0.0471191406250000,
+0.0227966308593750, -0.0492858886718750, 0.0232543945312500, -0.0343322753906250,
+0.0128173828125000, -0.0405883789062500, 0.0164489746093750, -0.0275878906250000,
+0.0127563476562500, -0.0085449218750000, 0.0026855468750000, -0.0192565917968750,
+0.0077514648437500, 0.0144348144531250, 0.0018005371093750, 0.0133056640625000,
+0.0009155273437500, 0.0172424316406250, -0.0002746582031250, 0.0383300781250000,
+-0.0020751953125000, 0.0312500000000000, -0.0003356933593750, 0.0399169921875000,
+-0.0115356445312500, 0.0434875488281250, -0.0073852539062500, 0.0376281738281250,
+-0.0129089355468750, 0.0379333496093750, -0.0250549316406250, 0.0394287109375000,
+-0.0180664062500000, 0.0176696777343750, -0.0270080566406250, 0.0239868164062500,
+-0.0297851562500000, 0.0104370117187500, -0.0270996093750000, -0.0132141113281250,
+-0.0277709960937500, -0.0009765625000000, -0.0289916992187500, -0.0287780761718750,
+-0.0294799804687500, -0.0327453613281250, -0.0273132324218750, -0.0276489257812500,
+-0.0297851562500000, -0.0379943847656250, -0.0348510742187500, -0.0381164550781250,
+-0.0315246582031250, -0.0206298828125000, -0.0249633789062500, -0.0224304199218750,
+-0.0361328125000000, -0.0199890136718750, -0.0141296386718750, -0.0072631835937500,
+0.0046386718750000},
+{0.0094909667968750, -0.0625000000000000, 0.0086669921875000, -0.0515136718750000,
+0.0075988769531250, -0.0728759765625000, 0.0234985351562500, -0.0729980468750000,
+0.0193786621093750, -0.0745544433593750, 0.0223083496093750, -0.0858764648437500,
+0.0384826660156250, -0.0801696777343750, 0.0335998535156250, -0.0867919921875000,
+0.0216064453125000, -0.0872497558593750, 0.0312194824218750, -0.0855712890625000,
+0.0178527832031250, -0.0886230468750000, -0.0043945312500000, -0.0880432128906250,
+0.0072326660156250, -0.0836486816406250, -0.0101928710937500, -0.0870056152343750,
+-0.0158996582031250, -0.0799865722656250, -0.0097045898437500, -0.0727844238281250,
+-0.0108032226562500, -0.0792236328125000, -0.0130004882812500, -0.0596313476562500,
+-0.0042114257812500, -0.0623474121093750, -0.0040283203125000, -0.0579528808593750,
+-0.0039062500000000, -0.0437927246093750, 0.0003662109375000, -0.0513305664062500,
+-0.0006713867187500, -0.0419616699218750, -0.0002441406250000, -0.0421142578125000,
+0.0001220703125000, -0.0426940917968750, -0.0000305175781250, -0.0393981933593750,
+0.0000000000000000, -0.0431213378906250, -0.0000610351562500, -0.0383300781250000,
+-0.0003967285156250, -0.0407714843750000, 0.0001525878906250, -0.0398559570312500,
+-0.0010681152343750, -0.0360107421875000, -0.0019531250000000, -0.0404357910156250,
+-0.0008544921875000, -0.0372619628906250, -0.0046997070312500, -0.0386657714843750,
+-0.0041809082031250, -0.0405883789062500, -0.0043945312500000, -0.0406494140625000,
+-0.0075988769531250, -0.0437316894531250, -0.0064392089843750, -0.0438537597656250,
+-0.0029907226562500, -0.0457458496093750, -0.0067749023437500, -0.0451660156250000,
+-0.0007324218750000, -0.0446777343750000, 0.0063476562500000, -0.0471801757812500,
+0.0002441406250000, -0.0399780273437500, 0.0146484375000000, -0.0421752929687500,
+0.0147399902343750, -0.0392150878906250, 0.0133056640625000, -0.0325317382812500,
+0.0206909179687500, -0.0365905761718750, 0.0193481445312500, -0.0293273925781250,
+0.0132446289062500, -0.0284729003906250, 0.0168762207031250, -0.0299377441406250,
+0.0127563476562500, -0.0273132324218750, 0.0047912597656250, -0.0281066894531250,
+0.0092773437500000, -0.0327453613281250, 0.0030517578125000, -0.0317687988281250,
+0.0022583007812500, -0.0347595214843750, 0.0017089843750000, -0.0408020019531250,
+-0.0002441406250000, -0.0389099121093750, 0.0010070800781250, -0.0390625000000000,
+-0.0073242187500000, -0.0446166992187500, -0.0036926269531250, -0.0360717773437500,
+-0.0090026855468750, -0.0286560058593750, -0.0187988281250000, -0.0353088378906250,
+-0.0126037597656250, -0.0098876953125000, -0.0224609375000000, -0.0148010253906250,
+-0.0242614746093750, -0.0026550292968750, -0.0223388671875000, 0.0193481445312500,
+-0.0247192382812500, 0.0069885253906250, -0.0250549316406250, 0.0420532226562500,
+-0.0254516601562500, 0.0411682128906250, -0.0239257812500000, 0.0434570312500000,
+-0.0264892578125000, 0.0655212402343750, -0.0302124023437500, 0.0579528808593750,
+-0.0269775390625000, 0.0534362792968750, -0.0258483886718750, 0.0633239746093750,
+-0.0333862304687500, 0.0445251464843750, -0.0170898437500000, 0.0252990722656250,
+-0.0063476562500000, 0.0378112792968750, -0.0193481445312500, -0.0068664550781250,
+0.0352172851562500, -0.0080566406250000, 0.0196533203125000, -0.0072937011718750,
+0.0447387695312500, -0.0296936035156250, 0.0944519042968750, -0.0252685546875000,
+0.0640258789062500, -0.0223388671875000, 0.1328735351562500, -0.0253601074218750,
+0.1288146972656250, -0.0219116210937500, 0.1360168457031250, -0.0183105468750000,
+0.1812744140625000, -0.0203857421875000, 0.1618957519531250, -0.0048522949218750,
+0.1688232421875000, -0.0073547363281250, 0.1876525878906250, -0.0066528320312500,
+0.1589660644531250, 0.0062866210937500, 0.1368408203125000, 0.0037841796875000,
+0.1597595214843750, -0.0079650878906250, 0.0918884277343750, -0.0034179687500000,
+0.0957031250000000, -0.0061035156250000, 0.0937500000000000, -0.0174255371093750,
+0.0540771484375000, -0.0145874023437500, 0.0652770996093750, -0.0165710449218750,
+0.0604858398437500, -0.0132446289062500, 0.0595397949218750, -0.0241088867187500,
+0.0490722656250000, -0.0306701660156250, 0.0458984375000000, -0.0216064453125000,
+0.0558166503906250, -0.0625000000000000, 0.0044555664062500, -0.0546569824218750,
+0.0165710449218750, -0.0668945312500000, 0.0011901855468750, -0.1002197265625000,
+-0.0441589355468750, -0.0844116210937500, -0.0239257812500000, -0.1111450195312500,
+-0.0469665527343750, -0.1132202148437500, -0.0523376464843750, -0.1136474609375000,
+-0.0547790527343750, -0.1278686523437500, -0.0611877441406250, -0.1272888183593750,
+-0.0616149902343750, -0.1372070312500000, -0.1064453125000000, -0.1350402832031250,
+-0.0851745605468750, -0.1325988769531250, -0.1013793945312500, -0.1394653320312500,
+-0.1478576660156250, -0.1381225585937500, -0.1253356933593750, -0.1244812011718750,
+-0.1662902832031250, -0.1331176757812500, -0.1744995117187500, -0.1192321777343750,
+-0.1646423339843750, -0.1006469726562500, -0.1755981445312500, -0.1142883300781250,
+-0.1757812500000000, -0.0755920410156250, -0.1515502929687500, -0.0808715820312500,
+-0.1590881347656250, -0.0698852539062500, -0.1444702148437500, -0.0418090820312500,
+-0.1190490722656250, -0.0569458007812500, -0.1323242187500000, -0.0225219726562500,
+-0.1002807617187500, -0.0263671875000000, -0.1011047363281250, -0.0161132812500000,
+-0.0935058593750000, 0.0080871582031250, -0.0744934082031250, -0.0057983398437500,
+-0.0842895507812500, 0.0308227539062500, -0.0505371093750000, 0.0242004394531250,
+-0.0547180175781250, 0.0376586914062500, -0.0475463867187500, 0.0671997070312500,
+-0.0217590332031250, 0.0502624511718750, -0.0312805175781250, 0.0912475585937500,
+-0.0187072753906250, 0.0868835449218750, -0.0173034667968750, 0.0982055664062500,
+-0.0145874023437500, 0.1272277832031250, -0.0076904296875000, 0.1125793457031250,
+-0.0125122070312500, 0.1523742675781250, -0.0026855468750000, 0.1479492187500000,
+-0.0003662109375000, 0.1557617187500000, -0.0027160644531250, 0.1868896484375000,
+-0.0016784667968750, 0.1726684570312500, 0.0003662109375000, 0.1761169433593750,
+0.0049133300781250, 0.1960144042968750, -0.0024719238281250, 0.1566467285156250,
+0.0127868652343750, 0.1299133300781250, 0.0280151367187500, 0.1609191894531250,
+0.0155029296875000, 0.0473327636718750, 0.0522766113281250, 0.0682678222656250,
+0.0528869628906250, 0.0432739257812500, 0.0526428222656250, -0.0448913574218750,
+0.0728454589843750, -0.0030822753906250, 0.0679321289062500, -0.0499877929687500,
+0.0593566894531250, -0.0674743652343750, 0.0667114257812500, -0.0451049804687500,
+0.0538330078125000, -0.0458374023437500, 0.0371398925781250, -0.0569152832031250,
+0.0463256835937500, -0.0249938964843750, 0.0183410644531250, -0.0259399414062500,
+0.0166625976562500, -0.0283203125000000, 0.0162658691406250, -0.0126342773437500,
+0.0016174316406250, -0.0155334472656250, 0.0053100585937500, -0.0195617675781250,
+0.0103759765625000, -0.0178527832031250, 0.0065002441406250, -0.0162963867187500,
+0.0118408203125000, -0.0193786621093750, 0.0205688476562500, -0.0201721191406250,
+0.0163269042968750, -0.0102844238281250, 0.0229187011718750, -0.0134887695312500,
+0.0240783691406250, -0.0103149414062500, 0.0226745605468750, -0.0014038085937500,
+0.0242614746093750, -0.0056762695312500, 0.0232238769531250, 0.0011596679687500,
+0.0204162597656250, 0.0005798339843750, 0.0204162597656250, 0.0035400390625000,
+0.0181884765625000, 0.0081787109375000, 0.0151062011718750, 0.0043945312500000,
+0.0155944824218750, 0.0168151855468750, 0.0119934082031250, 0.0155639648437500,
+0.0112915039062500, 0.0157470703125000, 0.0119323730468750, 0.0242309570312500,
+0.0106506347656250, 0.0215454101562500, 0.0101623535156250, 0.0156250000000000,
+0.0125427246093750, 0.0213623046875000, 0.0125122070312500, 0.0139770507812500,
+0.0140380859375000, 0.0031433105468750, 0.0170898437500000, 0.0108337402343750,
+0.0173339843750000, -0.0029907226562500, 0.0200500488281250, -0.0041503906250000,
+0.0219726562500000, -0.0021362304687500, 0.0231323242187500, -0.0077209472656250,
+0.0240478515625000, -0.0070495605468750, 0.0249328613281250, -0.0037231445312500,
+0.0286254882812500, -0.0050659179687500, 0.0294494628906250, -0.0040893554687500,
+0.0315246582031250, -0.0008544921875000, 0.0348510742187500, -0.0023193359375000,
+0.0344848632812500, -0.0023193359375000, 0.0361328125000000, -0.0007324218750000,
+0.0374145507812500, -0.0033874511718750, 0.0355224609375000, -0.0057678222656250,
+0.0351257324218750, -0.0037231445312500, 0.0365295410156250, -0.0100708007812500,
+0.0284423828125000, -0.0108642578125000, 0.0326843261718750, -0.0081176757812500,
+0.0267944335937500, -0.0112609863281250, 0.0176391601562500, -0.0119628906250000,
+0.0242004394531250, 0.0056152343750000, 0.0054931640625000, -0.0013122558593750,
+0.0091247558593750, 0.0094299316406250, 0.0040893554687500, 0.0317077636718750,
+-0.0106201171875000, 0.0204772949218750, -0.0021667480468750, 0.0298767089843750,
+-0.0145263671875000, 0.0409545898437500, -0.0166320800781250, 0.0234680175781250,
+-0.0135498046875000, 0.0092468261718750, -0.0173950195312500, 0.0214233398437500,
+-0.0169067382812500, -0.0071105957031250, -0.0089721679687500, -0.0146179199218750,
+-0.0128784179687500, -0.0016174316406250, -0.0078735351562500, -0.0030212402343750,
+0.0003662109375000, -0.0107116699218750, -0.0058898925781250, 0.0114440917968750,
+0.0089721679687500, 0.0184020996093750, 0.0087585449218750, 0.0011291503906250,
+0.0079650878906250, -0.0003662109375000, 0.0175170898437500, 0.0105285644531250,
+0.0150756835937500, -0.0381469726562500, 0.0027770996093750, -0.0371704101562500,
+0.0115051269531250, -0.0328063964843750, -0.0022888183593750, -0.0595397949218750,
+-0.0213928222656250, -0.0560607910156250, -0.0091247558593750, -0.0260314941406250,
+-0.0424194335937500, -0.0382690429687500, -0.0391540527343750, -0.0244140625000000,
+-0.0477600097656250, 0.0083923339843750, -0.0737304687500000, -0.0062866210937500,
+-0.0603637695312500, 0.0126953125000000, -0.0692138671875000, 0.0158081054687500,
+-0.0822448730468750, 0.0187988281250000, -0.0643005371093750, 0.0259399414062500,
+-0.0476684570312500, 0.0205993652343750, -0.0622253417968750, 0.0440063476562500,
+-0.0398254394531250, 0.0392456054687500, -0.0251770019531250, 0.0468139648437500,
+-0.0507202148437500, 0.0666198730468750, -0.0671997070312500, 0.0581054687500000,
+-0.0498352050781250, 0.0765075683593750, -0.0871582031250000, 0.0718994140625000,
+-0.1000671386718750, 0.0880126953125000, -0.0781860351562500, 0.1050109863281250,
+-0.0780029296875000, 0.0908508300781250, -0.0918884277343750, 0.1572570800781250,
+-0.0397644042968750, 0.1453247070312500, -0.0476074218750000, 0.1631469726562500,
+-0.0295104980468750, 0.2213439941406250, 0.0066528320312500, 0.1958923339843750,
+-0.0151367187500000, 0.2031555175781250, 0.0545654296875000, 0.2318420410156250,
+0.0447998046875000, 0.1837463378906250, 0.0577087402343750, 0.1460266113281250,
+0.1124572753906250, 0.1807556152343750, 0.0911560058593750, 0.0560913085937500,
+0.1011352539062500, 0.0729675292968750, 0.1159667968750000, 0.0370178222656250,
+0.0995788574218750, -0.0581054687500000, 0.0838928222656250, -0.0120239257812500,
+0.0949401855468750, -0.0971069335937500, 0.0765380859375000, -0.1035461425781250,
+0.0724487304687500, -0.1040344238281250, 0.0791625976562500, -0.1463623046875000,
+0.0767517089843750, -0.1333312988281250, 0.0749511718750000, -0.1390075683593750,
+0.0861816406250000},
+{0.0683288574218750, -0.0061340332031250, 0.0687866210937500, -0.0069580078125000,
+0.0661621093750000, -0.0064697265625000, 0.0655212402343750, -0.0113830566406250,
+0.0659790039062500, -0.0100402832031250, 0.0632324218750000, -0.0112304687500000,
+0.0651245117187500, -0.0121459960937500, 0.0627746582031250, -0.0103454589843750,
+0.0646972656250000, -0.0108032226562500, 0.0675354003906250, -0.0115356445312500,
+0.0644226074218750, 0.0001831054687500, 0.0738830566406250, -0.0055541992187500,
+0.0718994140625000, 0.0051879882812500, 0.0728759765625000, 0.0218505859375000,
+0.0797119140625000, 0.0113525390625000, 0.0758666992187500, 0.0309143066406250,
+0.0760192871093750, 0.0373840332031250, 0.0790710449218750, 0.0252075195312500,
+0.0740356445312500, 0.0254516601562500, 0.0699157714843750, 0.0323181152343750,
+0.0733337402343750, 0.0004272460937500, 0.0601806640625000, 0.0009765625000000,
+0.0649414062500000, 0.0042114257812500, 0.0570373535156250, -0.0129699707031250,
+0.0445556640625000, -0.0118408203125000, 0.0538940429687500, 0.0066223144531250,
+0.0274963378906250, 0.0052185058593750, 0.0337219238281250, 0.0008239746093750,
+0.0240783691406250, 0.0127868652343750, 0.0023193359375000, 0.0139465332031250,
+0.0158386230468750, -0.0215759277343750, -0.0140686035156250, -0.0127868652343750,
+-0.0146789550781250, -0.0201110839843750, -0.0137634277343750, -0.0528869628906250,
+-0.0303955078125000, -0.0419311523437500, -0.0270080566406250, -0.0312500000000000,
+-0.0199584960937500, -0.0451965332031250, -0.0270996093750000, -0.0285644531250000,
+-0.0203857421875000, -0.0029602050781250, -0.0082092285156250, -0.0174865722656250,
+-0.0157470703125000, 0.0056762695312500, -0.0042114257812500, 0.0091552734375000,
+-0.0019226074218750, 0.0098876953125000, -0.0103149414062500, 0.0176391601562500,
+-0.0061950683593750, 0.0139770507812500, -0.0030212402343750, 0.0315856933593750,
+-0.0425415039062500, 0.0265808105468750, -0.0282897949218750, 0.0347595214843750,
+-0.0494995117187500, 0.0523376464843750, -0.0961303710937500, 0.0436401367187500,
+-0.0748901367187500, 0.0608825683593750, -0.1000366210937500, 0.0583496093750000,
+-0.1141662597656250, 0.0709533691406250, -0.1004638671875000, 0.0842895507812500,
+-0.0926818847656250, 0.0728454589843750, -0.1004638671875000, 0.1275634765625000,
+-0.0943908691406250, 0.1144104003906250, -0.0903930664062500, 0.1334838867187500,
+-0.0903015136718750, 0.1855163574218750, -0.0952758789062500, 0.1610412597656250,
+-0.0963745117187500, 0.1821289062500000, -0.0669555664062500, 0.2046203613281250,
+-0.0766296386718750, 0.1664428710937500, -0.0694274902343750, 0.1456298828125000,
+-0.0417175292968750, 0.1708679199218750, -0.0507812500000000, 0.0663146972656250,
+-0.0438537597656250, 0.0830078125000000, -0.0487670898437500, 0.0505065917968750,
+-0.0285644531250000, -0.0318908691406250, -0.0153503417968750, 0.0072937011718750,
+-0.0331726074218750, -0.0727233886718750, 0.0425109863281250, -0.0769958496093750,
+0.0326538085937500, -0.0801696777343750, 0.0449218750000000, -0.1223449707031250,
+0.0993041992187500, -0.1083374023437500, 0.0780944824218750, -0.1181335449218750,
+0.0951232910156250, -0.1265563964843750, 0.1055603027343750, -0.1199645996093750,
+0.0954589843750000, -0.1144104003906250, 0.0921325683593750, -0.1197814941406250,
+0.0972900390625000, -0.1215209960937500, 0.0825805664062500, -0.1103515625000000,
+0.0820617675781250, -0.1329650878906250, 0.0845031738281250, -0.1514892578125000,
+0.0782775878906250, -0.1336364746093750, 0.0787963867187500, -0.1971740722656250,
+0.0872192382812500, -0.1925964355468750, 0.0875549316406250, -0.1983337402343750,
+0.0804748535156250, -0.2412719726562500, 0.0859985351562500, -0.2267456054687500,
+0.0897216796875000, -0.2182617187500000, 0.0440063476562500, -0.2341613769531250,
+0.0608825683593750, -0.2160339355468750, 0.0409851074218750, -0.1702270507812500,
+-0.0391540527343750, -0.0952148437500000, 0.0798339843750000, 0.0464782714843750,
+0.0242004394531250, -0.0177917480468750, 0.0615539550781250, 0.0410461425781250,
+0.0202026367187500, 0.1416931152343750, -0.0340270996093750, 0.0922546386718750,
+-0.0036926269531250, 0.1491699218750000, -0.0728149414062500, 0.1781005859375000,
+-0.0687866210937500, 0.1383972167968750, -0.0851745605468750, 0.1300659179687500,
+-0.1295776367187500, 0.1537780761718750, -0.1064453125000000, 0.0634155273437500,
+-0.1695251464843750, 0.0696411132812500, -0.1655578613281250, 0.0679931640625000,
+-0.1680603027343750, 0.0124206542968750, -0.2116088867187500, 0.0277099609375000,
+-0.1980285644531250, 0.0453796386718750, -0.1732482910156250, 0.0282287597656250,
+-0.1977233886718750, 0.0454406738281250, -0.1634216308593750, 0.0775756835937500,
+-0.1119384765625000, 0.0602111816406250, -0.1428222656250000, 0.0731811523437500,
+-0.0917663574218750, 0.0834045410156250, -0.0786743164062500, 0.0753173828125000,
+-0.0941467285156250, 0.0682373046875000, -0.0884399414062500, 0.0721740722656250,
+-0.0827026367187500, 0.0714416503906250, -0.0927124023437500, 0.0739746093750000,
+-0.1038513183593750, 0.0573730468750000, -0.0816040039062500, 0.0540771484375000,
+-0.0679626464843750, 0.0662231445312500, -0.0848388671875000, -0.0146484375000000,
+-0.0271301269531250, 0.0065612792968750, -0.0319519042968750, -0.0224304199218750,
+-0.0202026367187500, -0.0957641601562500, 0.0158996582031250, -0.0604248046875000,
+0.0001220703125000, -0.1188964843750000, 0.0474853515625000, -0.1259460449218750,
+0.0369262695312500, -0.1199645996093750, 0.0622253417968750, -0.1459655761718750,
+0.1084899902343750, -0.1390686035156250, 0.0817260742187500, -0.1142883300781250,
+0.1423645019531250, -0.1366577148437500, 0.1500549316406250, -0.1021728515625000,
+0.1381225585937500, -0.0568237304687500, 0.1609497070312500, -0.0920715332031250,
+0.1619873046875000, -0.0202636718750000, 0.1248474121093750, -0.0134582519531250,
+0.1352844238281250, -0.0295410156250000, 0.1206665039062500, -0.0029907226562500,
+0.0873413085937500, -0.0013427734375000, 0.1028442382812500, -0.0404357910156250,
+0.0687561035156250, -0.0266723632812500, 0.0722656250000000, -0.0425720214843750,
+0.0598754882812500, -0.0779724121093750, 0.0339050292968750, -0.0576477050781250,
+0.0487060546875000, -0.0997619628906250, 0.0104064941406250, -0.1006469726562500,
+0.0113220214843750, -0.0957946777343750, 0.0074462890625000, -0.1182861328125000,
+-0.0144348144531250, -0.1141052246093750, -0.0063781738281250, -0.0889282226562500,
+-0.0235290527343750, -0.1049499511718750, -0.0210876464843750, -0.0805053710937500,
+-0.0274353027343750, -0.0485839843750000, -0.0431518554687500, -0.0740661621093750,
+-0.0342407226562500, -0.0041503906250000, -0.0418701171875000, -0.0193786621093750,
+-0.0509033203125000, 0.0023498535156250, -0.0354309082031250, 0.0581054687500000,
+-0.0244445800781250, 0.0242309570312500, -0.0356140136718750, 0.0858764648437500,
+-0.0043640136718750, 0.0846862792968750, -0.0028991699218750, 0.0862731933593750,
+-0.0046997070312500, 0.1208496093750000, 0.0083312988281250, 0.1063537597656250,
+0.0061035156250000, 0.1130676269531250, 0.0090637207031250, 0.1215820312500000,
+0.0056152343750000, 0.1092834472656250, 0.0142822265625000, 0.1029052734375000,
+0.0240478515625000, 0.1113891601562500, 0.0162048339843750, 0.0809936523437500,
+0.0323486328125000, 0.0903625488281250, 0.0376892089843750, 0.0784912109375000,
+0.0276489257812500, 0.0527954101562500, 0.0274658203125000, 0.0699157714843750,
+0.0327758789062500, 0.0310974121093750, 0.0061035156250000, 0.0380249023437500,
+0.0083007812500000, 0.0297546386718750, 0.0045471191406250, 0.0025634765625000,
+-0.0135498046875000, 0.0208740234375000, -0.0079650878906250, -0.0049133300781250,
+-0.0114440917968750, -0.0033264160156250, -0.0143432617187500, -0.0009155273437500,
+-0.0113830566406250, -0.0114440917968750, -0.0095825195312500, -0.0028076171875000,
+-0.0106201171875000, -0.0080261230468750, -0.0060424804687500, -0.0047912597656250,
+-0.0074157714843750, -0.0035095214843750, -0.0039062500000000, -0.0063171386718750,
+0.0005798339843750, -0.0000915527343750, -0.0031127929687500, -0.0009155273437500,
+0.0078735351562500, 0.0017395019531250, 0.0064392089843750, 0.0065002441406250,
+0.0078735351562500, 0.0086975097656250, 0.0158081054687500, 0.0135498046875000,
+0.0124206542968750, 0.0249633789062500, 0.0130310058593750, 0.0231933593750000,
+0.0160217285156250, 0.0330505371093750, 0.0126037597656250, 0.0466613769531250,
+0.0090942382812500, 0.0422973632812500, 0.0120239257812500, 0.0611572265625000,
+0.0059204101562500, 0.0613098144531250, 0.0057678222656250, 0.0637207031250000,
+0.0053710937500000, 0.0757141113281250, 0.0027770996093750, 0.0727844238281250,
+0.0039367675781250, 0.0760803222656250, -0.0012817382812500, 0.0789489746093750,
+0.0010681152343750, 0.0754089355468750, -0.0033264160156250, 0.0740661621093750,
+-0.0097045898437500, 0.0761108398437500, -0.0048828125000000, 0.0634765625000000,
+-0.0154113769531250, 0.0661621093750000, -0.0154418945312500, 0.0582580566406250,
+-0.0152587890625000, 0.0451660156250000, -0.0205383300781250, 0.0505676269531250,
+-0.0191040039062500, 0.0309143066406250, -0.0206604003906250, 0.0319519042968750,
+-0.0203247070312500, 0.0247192382812500, -0.0219421386718750, 0.0097961425781250,
+-0.0240478515625000, 0.0162353515625000, -0.0219726562500000, -0.0032958984375000,
+-0.0248718261718750, -0.0032958984375000, -0.0274658203125000, -0.0059204101562500,
+-0.0202026367187500, -0.0181579589843750, -0.0187683105468750, -0.0142517089843750,
+-0.0236816406250000, -0.0195617675781250, 0.0065002441406250, -0.0225524902343750,
+-0.0056152343750000, -0.0172424316406250, 0.0152893066406250, -0.0166625976562500,
+0.0475463867187500, -0.0200805664062500, 0.0252990722656250, -0.0019836425781250,
+0.0851745605468750, -0.0063781738281250, 0.0772094726562500, 0.0027770996093750,
+0.0914306640625000, 0.0184020996093750, 0.1362915039062500, 0.0096130371093750,
+0.1133117675781250, 0.0406188964843750, 0.1439514160156250, 0.0366821289062500,
+0.1546630859375000, 0.0430297851562500, 0.1367187500000000, 0.0684509277343750,
+0.1392822265625000, 0.0585632324218750, 0.1495666503906250, 0.0569152832031250,
+0.1015930175781250, 0.0650024414062500, 0.1115417480468750, 0.0559997558593750,
+0.1004638671875000, 0.0446166992187500, 0.0615234375000000, 0.0500183105468750,
+0.0785217285156250, 0.0314636230468750, 0.0621643066406250, 0.0354614257812500,
+0.0563964843750000, 0.0266723632812500, 0.0561828613281250, 0.0063476562500000,
+0.0557861328125000, 0.0165100097656250, 0.0576171875000000, 0.0136718750000000,
+0.0264892578125000, 0.0030212402343750, 0.0420532226562500, 0.0148620605468750,
+0.0198974609375000, 0.0315551757812500, -0.0205078125000000, 0.0227050781250000,
+0.0028381347656250, 0.0252990722656250, -0.0385131835937500, 0.0358276367187500,
+-0.0438537597656250, 0.0232238769531250, -0.0408630371093750, 0.0090942382812500,
+-0.0535888671875000, 0.0194396972656250, -0.0513610839843750, 0.0009155273437500,
+-0.0719909667968750, -0.0010986328125000, -0.0588684082031250, -0.0015563964843750,
+-0.0838623046875000, -0.0077514648437500, -0.1172485351562500, -0.0046081542968750,
+-0.0924377441406250, -0.0193176269531250, -0.1463928222656250, -0.0130310058593750,
+-0.1508178710937500, -0.0239562988281250, -0.1437683105468750, -0.0456237792968750,
+-0.1660461425781250, -0.0448608398437500, -0.1626586914062500, -0.0786437988281250,
+-0.1435852050781250, -0.0679016113281250, -0.1524047851562500, -0.0714721679687500,
+-0.1456298828125000},
+{0.0130920410156250, -0.0973815917968750, 0.0098571777343750, -0.0945434570312500,
+0.0088500976562500, -0.0953063964843750, 0.0110168457031250, -0.0955810546875000,
+0.0056762695312500, -0.0962829589843750, 0.0059509277343750, -0.0934448242187500,
+0.0054626464843750, -0.0919494628906250, 0.0028381347656250, -0.0927124023437500,
+0.0039367675781250, -0.0800476074218750, -0.0001831054687500, -0.0836486816406250,
+0.0016784667968750, -0.0747375488281250, -0.0019531250000000, -0.0613708496093750,
+-0.0068969726562500, -0.0675354003906250, -0.0028686523437500, -0.0406188964843750,
+-0.0127868652343750, -0.0442199707031250, -0.0123291015625000, -0.0328979492187500,
+-0.0128173828125000, -0.0106201171875000, -0.0184936523437500, -0.0205688476562500,
+-0.0164184570312500, 0.0118408203125000, -0.0185241699218750, 0.0104675292968750,
+-0.0186767578125000, 0.0192260742187500, -0.0196533203125000, 0.0430603027343750,
+-0.0212097167968750, 0.0340270996093750, -0.0198059082031250, 0.0525817871093750,
+-0.0226745605468750, 0.0555725097656250, -0.0241394042968750, 0.0525207519531250,
+-0.0195312500000000, 0.0592956542968750, -0.0197143554687500, 0.0578613281250000,
+-0.0225524902343750, 0.0466613769531250, -0.0010681152343750, 0.0522155761718750,
+-0.0110778808593750, 0.0396728515625000, 0.0061645507812500, 0.0238037109375000,
+0.0304565429687500, 0.0330200195312500, 0.0124511718750000, -0.0003967285156250,
+0.0646972656250000, 0.0024108886718750, 0.0553283691406250, -0.0061035156250000,
+0.0715332031250000, -0.0313415527343750, 0.1137390136718750, -0.0203857421875000,
+0.0904541015625000, -0.0335693359375000, 0.1287231445312500, -0.0409851074218750,
+0.1354980468750000, -0.0302734375000000, 0.1234741210937500, -0.0259094238281250,
+0.1355590820312500, -0.0314636230468750, 0.1388244628906250, -0.0097351074218750,
+0.1032714843750000, -0.0132141113281250, 0.1153869628906250, -0.0024414062500000,
+0.1020202636718750, 0.0154724121093750, 0.0673217773437500, 0.0035705566406250,
+0.0849304199218750, 0.0335083007812500, 0.0635070800781250, 0.0353393554687500,
+0.0582275390625000, 0.0314331054687500, 0.0603637695312500, 0.0430297851562500,
+0.0575866699218750, 0.0422973632812500, 0.0582580566406250, 0.0406799316406250,
+0.0367126464843750, 0.0394897460937500, 0.0502929687500000, 0.0389404296875000,
+0.0277099609375000, 0.0419921875000000, -0.0048828125000000, 0.0416564941406250,
+0.0180969238281250, 0.0251464843750000, -0.0306396484375000, 0.0287780761718750,
+-0.0330810546875000, 0.0278015136718750, -0.0313720703125000, 0.0135192871093750,
+-0.0521850585937500, 0.0160217285156250, -0.0479736328125000, 0.0287780761718750,
+-0.0625915527343750, 0.0249023437500000, -0.0533752441406250, 0.0256042480468750,
+-0.0744934082031250, 0.0371398925781250, -0.0977172851562500, 0.0355529785156250,
+-0.0760803222656250, 0.0261535644531250, -0.1331176757812500, 0.0310974121093750,
+-0.1323242187500000, 0.0257568359375000, -0.1311645507812500, 0.0150451660156250,
+-0.1628112792968750, 0.0207824707031250, -0.1553039550781250, 0.0084533691406250,
+-0.1458435058593750, 0.0106201171875000, -0.1545715332031250, 0.0056152343750000,
+-0.1457519531250000, -0.0055541992187500, -0.1282653808593750, -0.0054321289062500,
+-0.1296691894531250, -0.0312194824218750, -0.1086730957031250, -0.0190429687500000,
+-0.1183471679687500, -0.0289306640625000, -0.1072692871093750, -0.0527648925781250,
+-0.0857849121093750, -0.0367431640625000, -0.0983581542968750, -0.0717468261718750,
+-0.0576782226562500, -0.0693664550781250, -0.0597839355468750, -0.0727233886718750,
+-0.0503540039062500, -0.0937194824218750, -0.0242614746093750, -0.0842285156250000,
+-0.0349426269531250, -0.0976562500000000, 0.0010375976562500, -0.0979309082031250,
+-0.0027770996093750, -0.0996093750000000, 0.0062255859375000, -0.1075134277343750,
+0.0378417968750000, -0.1032409667968750, 0.0259094238281250, -0.1094970703125000,
+0.0207519531250000, -0.1119689941406250, 0.0340576171875000, -0.1073303222656250,
+0.0209655761718750, -0.1083374023437500, -0.0030517578125000, -0.1109313964843750,
+0.0083312988281250, -0.0917663574218750, 0.0102233886718750, -0.0989685058593750,
+-0.0016479492187500, -0.0874328613281250, 0.0119628906250000, -0.0679016113281250,
+0.0297851562500000, -0.0806579589843750, 0.0187988281250000, -0.0493164062500000,
+0.0389099121093750, -0.0532531738281250, 0.0414123535156250, -0.0429992675781250,
+0.0370178222656250, -0.0203247070312500, 0.0454101562500000, -0.0335083007812500,
+0.0447692871093750, 0.0015869140625000, 0.0243225097656250, -0.0040893554687500,
+0.0324096679687500, 0.0076599121093750, 0.0215454101562500, 0.0345764160156250,
+-0.0008544921875000, 0.0190734863281250, 0.0107727050781250, 0.0561828613281250,
+-0.0090026855468750, 0.0520324707031250, -0.0114746093750000, 0.0616760253906250,
+-0.0091552734375000, 0.0882568359375000, -0.0162353515625000, 0.0742492675781250,
+-0.0146179199218750, 0.1040649414062500, -0.0139465332031250, 0.1016235351562500,
+-0.0148010253906250, 0.1069946289062500, -0.0128784179687500, 0.1268615722656250,
+-0.0127563476562500, 0.1168823242187500, -0.0134582519531250, 0.1345520019531250,
+-0.0004272460937500, 0.1377258300781250, -0.0067443847656250, 0.1288452148437500,
+0.0052795410156250, 0.1352844238281250, 0.0236206054687500, 0.1382141113281250,
+0.0135192871093750, 0.0997314453125000, 0.0401916503906250, 0.1190185546875000,
+0.0420227050781250, 0.0885314941406250, 0.0422668457031250, 0.0427551269531250,
+0.0559997558593750, 0.0766601562500000, 0.0516052246093750, -0.0028076171875000,
+0.0454101562500000, 0.0013427734375000, 0.0514831542968750, -0.0016784667968750,
+0.0368957519531250, -0.0490722656250000, 0.0220642089843750, -0.0317382812500000,
+0.0304870605468750, -0.0366516113281250, -0.0083312988281250, -0.0482177734375000,
+-0.0045776367187500, -0.0360412597656250, -0.0198364257812500, -0.0241088867187500,
+-0.0523681640625000, -0.0332946777343750, -0.0398559570312500, -0.0173034667968750,
+-0.0742187500000000, -0.0143737792968750, -0.0779418945312500, -0.0177612304687500,
+-0.0803222656250000, -0.0140075683593750, -0.0982971191406250, -0.0133056640625000,
+-0.0946655273437500, -0.0144958496093750, -0.0986938476562500, -0.0164794921875000,
+-0.1026611328125000, -0.0123901367187500, -0.0982055664062500, -0.0095825195312500,
+-0.0953979492187500, -0.0130615234375000, -0.0982971191406250, -0.0029907226562500,
+-0.0875244140625000, -0.0040588378906250, -0.0879821777343750, -0.0030517578125000,
+-0.0842895507812500, 0.0031738281250000, -0.0755310058593750, 0.0005493164062500,
+-0.0780029296875000, 0.0045471191406250, -0.0714111328125000, 0.0039367675781250,
+-0.0701904296875000, 0.0059814453125000, -0.0682678222656250, 0.0091857910156250,
+-0.0645141601562500, 0.0068054199218750, -0.0654907226562500, 0.0144653320312500,
+-0.0592346191406250, 0.0131835937500000, -0.0584106445312500, 0.0160522460937500,
+-0.0563049316406250, 0.0225524902343750, -0.0521850585937500, 0.0195312500000000,
+-0.0525207519531250, 0.0263977050781250, -0.0468444824218750, 0.0281677246093750,
+-0.0480651855468750, 0.0247497558593750, -0.0436096191406250, 0.0272216796875000,
+-0.0388183593750000, 0.0286560058593750, -0.0425720214843750, 0.0104370117187500,
+-0.0258178710937500, 0.0164794921875000, -0.0299072265625000, 0.0077514648437500,
+-0.0219116210937500, -0.0130310058593750, -0.0066528320312500, -0.0032043457031250,
+-0.0153198242187500, -0.0090942382812500, 0.0057678222656250, -0.0178222656250000,
+0.0029296875000000, -0.0070800781250000, 0.0097961425781250, 0.0046081542968750,
+0.0255737304687500, -0.0036926269531250, 0.0163574218750000, 0.0009460449218750,
+0.0384216308593750, 0.0117187500000000, 0.0374450683593750, -0.0060729980468750,
+0.0379028320312500, -0.0233154296875000, 0.0523986816406250, -0.0103149414062500,
+0.0485839843750000, -0.0379943847656250, 0.0421142578125000, -0.0461730957031250,
+0.0498657226562500, -0.0328979492187500, 0.0421447753906250, -0.0341186523437500,
+0.0292968750000000, -0.0410461425781250, 0.0379943847656250, -0.0131225585937500,
+0.0245666503906250, -0.0140991210937500, 0.0212707519531250, -0.0120849609375000,
+0.0247192382812500, 0.0026855468750000, 0.0228271484375000, -0.0017700195312500,
+0.0211181640625000, 0.0103759765625000, 0.0195922851562500, 0.0070495605468750,
+0.0232238769531250, 0.0154418945312500, 0.0148010253906250, 0.0288391113281250,
+0.0079650878906250, 0.0202941894531250, 0.0141906738281250, 0.0400695800781250,
+-0.0108337402343750, 0.0395202636718750, -0.0053100585937500, 0.0443420410156250,
+-0.0167236328125000, 0.0542907714843750, -0.0390319824218750, 0.0484924316406250,
+-0.0259399414062500, 0.0801086425781250, -0.0542907714843750, 0.0680236816406250,
+-0.0557556152343750, 0.0903625488281250, -0.0556945800781250, 0.1282958984375000,
+-0.0693664550781250, 0.1042175292968750, -0.0665588378906250, 0.1469726562500000,
+-0.0722961425781250, 0.1582336425781250, -0.0697326660156250, 0.1383361816406250,
+-0.0781860351562500, 0.1463012695312500, -0.0879821777343750, 0.1545715332031250,
+-0.0799560546875000, 0.0916442871093750, -0.0979309082031250, 0.1101379394531250,
+-0.1020507812500000, 0.0821228027343750, -0.0959472656250000, 0.0231933593750000,
+-0.0974426269531250, 0.0532226562500000, -0.1016845703125000, -0.0196838378906250,
+-0.1018676757812500, -0.0197753906250000, -0.0959777832031250, -0.0277099609375000,
+-0.1038208007812500, -0.0705566406250000, -0.1192321777343750, -0.0541992187500000,
+-0.1125183105468750, -0.0756835937500000, -0.0941467285156250, -0.0812683105468750,
+-0.1133422851562500, -0.0786132812500000, -0.0893554687500000, -0.0833129882812500,
+-0.0484619140625000, -0.0832519531250000, -0.0718383789062500, -0.0924072265625000,
+-0.0476379394531250, -0.0850524902343750, -0.0368347167968750, -0.1019592285156250,
+-0.0424499511718750, -0.1196289062500000, -0.0468750000000000, -0.1055908203125000,
+-0.0463867187500000, -0.1523742675781250, -0.0201721191406250, -0.1490173339843750,
+-0.0344543457031250, -0.1538696289062500, -0.0147094726562500, -0.1848754882812500,
+0.0200195312500000, -0.1734924316406250, 0.0000000000000000, -0.1751708984375000,
+0.0405273437500000, -0.1861572265625000, 0.0411682128906250, -0.1704406738281250,
+0.0468750000000000, -0.1573486328125000, 0.0696105957031250, -0.1676025390625000,
+0.0586853027343750, -0.1236877441406250, 0.0811767578125000, -0.1365051269531250,
+0.0832824707031250, -0.1048278808593750, 0.0790710449218750, -0.0617675781250000,
+0.0888671875000000, -0.0904235839843750, 0.0888977050781250, 0.0172119140625000,
+0.0744018554687500, 0.0038146972656250, 0.0784301757812500, 0.0208435058593750,
+0.0772094726562500, 0.1432495117187500, 0.0607299804687500, 0.1198120117187500,
+0.0916137695312500, 0.0338134765625000, 0.0726623535156250, 0.0552368164062500,
+0.0826110839843750, 0.0531005859375000, 0.0770568847656250, 0.0175476074218750,
+0.0628662109375000, 0.0221252441406250, 0.0672302246093750, 0.0545654296875000,
+0.0681457519531250, 0.0426940917968750, 0.0699768066406250, 0.0521545410156250,
+0.0598144531250000, 0.0844421386718750, 0.0585937500000000, 0.0715026855468750,
+0.0661926269531250, 0.0709228515625000, 0.0122985839843750, 0.0812072753906250,
+0.0288696289062500, 0.0699462890625000, 0.0028686523437500, 0.0544128417968750,
+-0.0500488281250000, 0.0630798339843750, -0.0216369628906250, 0.0546264648437500,
+-0.0820312500000000, 0.0493774414062500, -0.0805358886718750, 0.0514831542968750,
+-0.0920410156250000, 0.0579833984375000, -0.1294555664062500, 0.0543212890625000,
+-0.1104431152343750},
+{0.0291748046875000, 0.0158691406250000, 0.0409240722656250, 0.0352172851562500,
+0.0106506347656250, 0.0349121093750000, 0.0127563476562500, 0.0376892089843750,
+0.0093078613281250, 0.0471801757812500, -0.0088806152343750, 0.0433349609375000,
+-0.0011291503906250, 0.0637207031250000, -0.0149536132812500, 0.0536499023437500,
+-0.0149536132812500, 0.0741271972656250, -0.0158081054687500, 0.1026000976562500,
+-0.0242919921875000, 0.0825195312500000, -0.0214843750000000, 0.1295471191406250,
+-0.0236206054687500, 0.1338500976562500, -0.0260009765625000, 0.1235656738281250,
+-0.0230407714843750, 0.1437072753906250, -0.0205688476562500, 0.1442871093750000,
+-0.0231628417968750, 0.1029357910156250, -0.0198364257812500, 0.1208801269531250,
+-0.0162963867187500, 0.0910644531250000, -0.0249023437500000, 0.0421447753906250,
+-0.0276184082031250, 0.0706787109375000, -0.0230102539062500, 0.0014343261718750,
+-0.0597534179687500, 0.0042724609375000, -0.0494689941406250, -0.0042419433593750,
+-0.0663146972656250, -0.0468139648437500, -0.1050109863281250, -0.0300598144531250,
+-0.0872802734375000, -0.0575561523437500, -0.1079101562500000, -0.0623474121093750,
+-0.1180725097656250, -0.0610351562500000, -0.1100158691406250, -0.0699462890625000,
+-0.1067199707031250, -0.0676574707031250, -0.1103820800781250, -0.0789489746093750,
+-0.1119079589843750, -0.0723266601562500, -0.1135253906250000, -0.0867309570312500,
+-0.1024475097656250, -0.1037292480468750, -0.1014099121093750, -0.0909423828125000,
+-0.1116027832031250, -0.1320190429687500, -0.0614929199218750, -0.1294555664062500,
+-0.0661315917968750, -0.1344299316406250, -0.0667724609375000, -0.1618652343750000,
+-0.0367431640625000, -0.1517333984375000, -0.0425109863281250, -0.1548461914062500,
+-0.0496215820312500, -0.1633300781250000, -0.0520019531250000, -0.1513977050781250,
+-0.0386962890625000, -0.1430053710937500, -0.0362854003906250, -0.1502380371093750,
+-0.0471801757812500, -0.1141662597656250, 0.0024719238281250, -0.1268615722656250,
+-0.0050048828125000, -0.0975341796875000, 0.0054931640625000, -0.0599060058593750,
+0.0419311523437500, -0.0862426757812500, 0.0266723632812500, 0.0144042968750000,
+0.0500488281250000, -0.0012207031250000, 0.0506591796875000, 0.0282287597656250,
+0.0558471679687500, 0.1120300292968750, 0.0692443847656250, 0.0736083984375000,
+0.0603637695312500, 0.1167907714843750, 0.0827636718750000, 0.1404113769531250,
+0.0833435058593750, 0.1120605468750000, 0.0795593261718750, 0.0860900878906250,
+0.0923156738281250, 0.0251770019531250, 0.0606079101562500, 0.0625000000000000,
+0.0836486816406250, 0.0404968261718750, 0.0797729492187500, 0.0559082031250000,
+0.0776367187500000, 0.0890808105468750, 0.0860900878906250, 0.0771789550781250,
+0.0879516601562500, 0.0769348144531250, 0.0699768066406250, 0.0862731933593750,
+0.0732116699218750, 0.0773620605468750, 0.0719299316406250, 0.0643920898437500,
+0.0594787597656250, 0.0711975097656250, 0.0626525878906250, 0.0656127929687500,
+0.0621643066406250, 0.0625915527343750, 0.0649719238281250, 0.0601196289062500,
+0.0523986816406250, 0.0625305175781250, 0.0477905273437500, 0.0627746582031250,
+0.0577697753906250, 0.0270690917968750, 0.0046997070312500, 0.0472412109375000,
+0.0183410644531250, 0.0097656250000000, -0.0044860839843750, -0.0404968261718750,
+-0.0534362792968750, -0.0033264160156250, -0.0280761718750000, -0.0940856933593750,
+-0.0812072753906250, -0.0987548828125000, -0.0799560546875000, -0.0855102539062500,
+-0.0894470214843750, -0.1271667480468750, -0.1229858398437500, -0.1261596679687500,
+-0.1068725585937500, -0.0838928222656250, -0.1356506347656250, -0.0945739746093750,
+-0.1427001953125000, -0.0949401855468750, -0.1286621093750000, -0.0619812011718750,
+-0.1351013183593750, -0.0685424804687500, -0.1413574218750000, -0.0969848632812500,
+-0.0931396484375000, -0.0826416015625000, -0.1033935546875000, -0.0985107421875000,
+-0.0901489257812500, -0.1321411132812500, -0.0486145019531250, -0.1142883300781250,
+-0.0656433105468750, -0.1380310058593750, -0.0512084960937500, -0.1495056152343750,
+-0.0442810058593750, -0.1275024414062500, -0.0474853515625000, -0.1244201660156250,
+-0.0503234863281250, -0.1398925781250000, -0.0502624511718750, -0.0793151855468750,
+-0.0304870605468750, -0.0921630859375000, -0.0422363281250000, -0.0766906738281250,
+-0.0234985351562500, -0.0301513671875000, 0.0057373046875000, -0.0549011230468750,
+-0.0112609863281250, -0.0147094726562500, 0.0289611816406250, -0.0121765136718750,
+0.0270690917968750, -0.0145263671875000, 0.0370788574218750, 0.0030517578125000,
+0.0618896484375000, -0.0021057128906250, 0.0485534667968750, 0.0045166015625000,
+0.0906982421875000, 0.0036621093750000, 0.0862731933593750, 0.0060424804687500,
+0.0931396484375000, 0.0111389160156250, 0.1254577636718750, 0.0078430175781250,
+0.1134948730468750, 0.0188293457031250, 0.1127319335937500, 0.0165405273437500,
+0.1254882812500000, 0.0198669433593750, 0.1075439453125000, 0.0298461914062500,
+0.0882263183593750, 0.0245056152343750, 0.1015625000000000, 0.0263977050781250,
+0.0675964355468750, 0.0319213867187500, 0.0695190429687500, 0.0237121582031250,
+0.0612792968750000, 0.0179138183593750, 0.0401916503906250, 0.0246582031250000,
+0.0515136718750000, 0.0017089843750000, 0.0184020996093750, 0.0088195800781250,
+0.0223999023437500, -0.0016174316406250, 0.0144042968750000, -0.0219116210937500,
+-0.0105285644531250, -0.0074768066406250, 0.0002441406250000, -0.0385742187500000,
+-0.0185546875000000, -0.0337219238281250, -0.0190734863281250, -0.0405578613281250,
+-0.0231018066406250, -0.0620727539062500, -0.0345153808593750, -0.0483398437500000,
+-0.0282592773437500, -0.0712280273437500, -0.0419311523437500, -0.0711975097656250,
+-0.0454406738281250, -0.0699462890625000, -0.0377502441406250, -0.0819091796875000,
+-0.0397033691406250, -0.0775146484375000, -0.0436096191406250, -0.0745544433593750,
+-0.0192565917968750, -0.0796203613281250, -0.0221862792968750, -0.0728149414062500,
+-0.0177917480468750, -0.0649414062500000, -0.0006103515625000, -0.0712280273437500,
+-0.0065307617187500, -0.0558776855468750, 0.0006713867187500, -0.0581359863281250,
+-0.0001220703125000, -0.0565490722656250, 0.0048522949218750, -0.0475158691406250,
+0.0109252929687500, -0.0540161132812500, 0.0054626464843750, -0.0475769042968750,
+0.0248718261718750, -0.0489807128906250, 0.0249633789062500, -0.0491027832031250,
+0.0241699218750000, -0.0459899902343750, 0.0354309082031250, -0.0504150390625000,
+0.0338134765625000, -0.0462341308593750, 0.0237426757812500, -0.0474853515625000,
+0.0289001464843750, -0.0498657226562500, 0.0217895507812500, -0.0485534667968750,
+0.0088195800781250, -0.0507812500000000, 0.0154113769531250, -0.0575256347656250,
+0.0018920898437500, -0.0576171875000000, 0.0010986328125000, -0.0617980957031250,
+0.0000610351562500, -0.0689697265625000, -0.0064392089843750, -0.0689392089843750,
+-0.0033264160156250, -0.0747375488281250, -0.0094299316406250, -0.0754699707031250,
+-0.0087280273437500, -0.0781860351562500, -0.0096740722656250, -0.0822753906250000,
+-0.0139160156250000, -0.0813598632812500, -0.0118408203125000, -0.0884399414062500,
+-0.0147094726562500, -0.0870361328125000, -0.0150146484375000, -0.0898437500000000,
+-0.0152587890625000, -0.0964660644531250, -0.0165100097656250, -0.0938110351562500,
+-0.0157470703125000, -0.0994567871093750, -0.0177917480468750, -0.1007690429687500,
+-0.0179748535156250, -0.0997619628906250, -0.0165100097656250, -0.1011962890625000,
+-0.0178527832031250, -0.1010437011718750, -0.0182495117187500, -0.0983886718750000,
+-0.0085144042968750, -0.0986633300781250, -0.0145568847656250, -0.0987548828125000,
+-0.0033569335937500, -0.0976562500000000, 0.0094299316406250, -0.0977478027343750,
+-0.0013122558593750, -0.0959777832031250, 0.0349731445312500, -0.0982055664062500,
+0.0259399414062500, -0.0922546386718750, 0.0416564941406250, -0.0876464843750000,
+0.0735168457031250, -0.0916748046875000, 0.0535888671875000, -0.0709533691406250,
+0.0964050292968750, -0.0758666992187500, 0.0962524414062500, -0.0648498535156250,
+0.0949096679687500, -0.0454101562500000, 0.1188659667968750, -0.0559692382812500,
+0.1120300292968750, -0.0254821777343750, 0.1020507812500000, -0.0274353027343750,
+0.1135559082031250, -0.0199890136718750, 0.0989074707031250, 0.0014648437500000,
+0.0777893066406250, -0.0083923339843750, 0.0922241210937500, 0.0098571777343750,
+0.0669860839843750, 0.0147399902343750, 0.0641784667968750, 0.0057067871093750,
+0.0663757324218750, 0.0083923339843750, 0.0584716796875000, 0.0124816894531250,
+0.0600280761718750, -0.0136718750000000, 0.0494079589843750, -0.0114440917968750,
+0.0582275390625000, -0.0160827636718750, 0.0402526855468750, -0.0332946777343750,
+0.0211486816406250, -0.0274963378906250, 0.0388183593750000, -0.0395812988281250,
+-0.0112609863281250, -0.0417175292968750, -0.0065307617187500, -0.0365600585937500,
+-0.0123291015625000, -0.0413513183593750, -0.0449829101562500, -0.0443420410156250,
+-0.0325927734375000, -0.0223693847656250, -0.0474548339843750, -0.0251464843750000,
+-0.0481872558593750, -0.0252380371093750, -0.0568237304687500, -0.0126647949218750,
+-0.0652770996093750, -0.0145263671875000, -0.0561523437500000, -0.0097961425781250,
+-0.0989074707031250, -0.0140075683593750, -0.0880737304687500, -0.0057983398437500,
+-0.1015930175781250, 0.0058288574218750, -0.1410827636718750, -0.0017395019531250,
+-0.1227416992187500, 0.0102844238281250, -0.1403503417968750, 0.0109558105468750,
+-0.1522216796875000, 0.0128173828125000, -0.1368713378906250, 0.0184020996093750,
+-0.1308898925781250, 0.0145568847656250, -0.1417846679687500, 0.0258483886718750,
+-0.1110534667968750, 0.0256042480468750, -0.1152038574218750, 0.0242309570312500,
+-0.1035156250000000, 0.0300598144531250, -0.0816650390625000, 0.0294799804687500,
+-0.0942077636718750, 0.0242004394531250, -0.0479125976562500, 0.0250244140625000,
+-0.0575256347656250, 0.0262756347656250, -0.0409851074218750, 0.0251159667968750,
+0.0080566406250000, 0.0262145996093750, 0.0003356933593750, 0.0270690917968750,
+0.0244445800781250, 0.0300903320312500, 0.0232238769531250, 0.0246582031250000,
+0.0213928222656250, 0.0217285156250000, 0.0337219238281250, 0.0274658203125000,
+0.0319824218750000, 0.0081176757812500, 0.0389709472656250, 0.0128784179687500,
+0.0384826660156250, 0.0056762695312500, 0.0361328125000000, -0.0094604492187500,
+0.0418701171875000, 0.0003356933593750, 0.0418395996093750, -0.0253601074218750,
+0.0207214355468750, -0.0198059082031250, 0.0246887207031250, -0.0298767089843750,
+0.0255737304687500, -0.0515136718750000, 0.0100708007812500, -0.0383300781250000,
+0.0110473632812500, -0.0674133300781250, 0.0240478515625000, -0.0663757324218750,
+0.0238647460937500, -0.0694274902343750, 0.0173950195312500, -0.0867004394531250,
+0.0199584960937500, -0.0792541503906250, 0.0241394042968750, -0.0900878906250000,
+0.0114440917968750, -0.0910949707031250, 0.0076904296875000, -0.0910339355468750,
+0.0140380859375000, -0.0967712402343750, 0.0154418945312500, -0.0950622558593750,
+0.0108642578125000, -0.0944824218750000, 0.0148925781250000, -0.0988159179687500,
+0.0196533203125000, -0.0921936035156250, 0.0140075683593750, -0.0871582031250000,
+0.0095825195312500, -0.0929260253906250, 0.0137023925781250, -0.0700683593750000,
+0.0008850097656250, -0.0757446289062500, 0.0007934570312500, -0.0669250488281250,
+0.0048522949218750, -0.0479431152343750, -0.0024719238281250, -0.0590209960937500,
+-0.0036926269531250, -0.0315551757812500, 0.0198669433593750, -0.0366821289062500,
+0.0121154785156250},
+{0.0671081542968750, -0.0541687011718750, 0.0661621093750000, -0.0478515625000000,
+0.0671386718750000, -0.0263977050781250, 0.0572814941406250, -0.0359191894531250,
+0.0595397949218750, -0.0190429687500000, 0.0520019531250000, -0.0190124511718750,
+0.0582885742187500, -0.0162963867187500, 0.0435791015625000, -0.0051879882812500,
+0.0301513671875000, -0.0104370117187500, 0.0440673828125000, -0.0054931640625000,
+-0.0021362304687500, -0.0018920898437500, 0.0055541992187500, -0.0052490234375000,
+-0.0035705566406250, -0.0078735351562500, -0.0384826660156250, -0.0059204101562500,
+-0.0232543945312500, -0.0075683593750000, -0.0422058105468750, -0.0074157714843750,
+-0.0451965332031250, -0.0116882324218750, -0.0483703613281250, -0.0150451660156250,
+-0.0549011230468750, -0.0113525390625000, -0.0500793457031250, -0.0208129882812500,
+-0.0800476074218750, -0.0244140625000000, -0.0689086914062500, -0.0203247070312500,
+-0.0862121582031250, -0.0188903808593750, -0.1196899414062500, -0.0218811035156250,
+-0.0993957519531250, -0.0221557617187500, -0.1313171386718750, -0.0209350585937500,
+-0.1398010253906250, -0.0180053710937500, -0.1265869140625000, -0.0194702148437500,
+-0.1325073242187500, -0.0223388671875000, -0.1385498046875000, -0.0071411132812500,
+-0.1038513183593750, -0.0100402832031250, -0.1127014160156250, -0.0066528320312500,
+-0.0997314453125000, 0.0036621093750000, -0.0713500976562500, -0.0012512207031250,
+-0.0870056152343750, 0.0131225585937500, -0.0460510253906250, 0.0100708007812500,
+-0.0531005859375000, 0.0153503417968750, -0.0346679687500000, 0.0286865234375000,
+0.0008850097656250, 0.0215148925781250, -0.0209655761718750, 0.0292663574218750,
+0.0253295898437500, 0.0332031250000000, 0.0281677246093750, 0.0278015136718750,
+0.0239562988281250, 0.0226745605468750, 0.0451049804687500, 0.0234680175781250,
+0.0427856445312500, 0.0257873535156250, 0.0361022949218750, 0.0219116210937500,
+0.0379638671875000, 0.0252990722656250, 0.0393981933593750, 0.0319213867187500,
+0.0366210937500000, 0.0275268554687500, 0.0353393554687500, 0.0289611816406250,
+0.0336303710937500, 0.0347595214843750, 0.0378723144531250, 0.0264282226562500,
+0.0283813476562500, 0.0198059082031250, 0.0169372558593750, 0.0270080566406250,
+0.0249633789062500, 0.0050354003906250, 0.0167541503906250, 0.0084838867187500,
+0.0105895996093750, 0.0027770996093750, 0.0151977539062500, -0.0125122070312500,
+0.0208740234375000, -0.0042114257812500, 0.0181579589843750, -0.0251159667968750,
+0.0114746093750000, -0.0205993652343750, 0.0135498046875000, -0.0299682617187500,
+0.0176696777343750, -0.0480346679687500, 0.0127868652343750, -0.0363464355468750,
+0.0097351074218750, -0.0632629394531250, 0.0304870605468750, -0.0614624023437500,
+0.0308532714843750, -0.0644226074218750, 0.0243835449218750, -0.0812988281250000,
+0.0321655273437500, -0.0740966796875000, 0.0354003906250000, -0.0832824707031250,
+0.0170898437500000, -0.0849304199218750, 0.0188293457031250, -0.0847167968750000,
+0.0186157226562500, -0.0893554687500000, 0.0064697265625000, -0.0880432128906250,
+0.0099182128906250, -0.0852355957031250, 0.0201721191406250, -0.0895996093750000,
+0.0107727050781250, -0.0819702148437500, 0.0224304199218750, -0.0748901367187500,
+0.0429382324218750, -0.0818481445312500, 0.0302429199218750, -0.0589904785156250,
+0.0382995605468750, -0.0641174316406250, 0.0487365722656250, -0.0552673339843750,
+0.0323181152343750, -0.0363769531250000, 0.0205383300781250, -0.0475769042968750,
+0.0307312011718750, -0.0215148925781250, -0.0056152343750000, -0.0264587402343750,
+-0.0016784667968750, -0.0164184570312500, -0.0130004882812500, 0.0036010742187500,
+-0.0410461425781250, -0.0099182128906250, -0.0282287597656250, 0.0256652832031250,
+-0.0539550781250000, 0.0190124511718750, -0.0567321777343750, 0.0287780761718750,
+-0.0567626953125000, 0.0557556152343750, -0.0677185058593750, 0.0402526855468750,
+-0.0650024414062500, 0.0703735351562500, -0.0728759765625000, 0.0680847167968750,
+-0.0717163085937500, 0.0723266601562500, -0.0775146484375000, 0.0910949707031250,
+-0.0863647460937500, 0.0812683105468750, -0.0816955566406250, 0.0984497070312500,
+-0.0936279296875000, 0.0988464355468750, -0.0951538085937500, 0.0982360839843750,
+-0.0923156738281250, 0.1077575683593750, -0.0964355468750000, 0.1044006347656250,
+-0.0961608886718750, 0.0993041992187500, -0.0860290527343750, 0.1075744628906250,
+-0.0895996093750000, 0.0928039550781250, -0.0798950195312500, 0.0802307128906250,
+-0.0669250488281250, 0.0932617187500000, -0.0732421875000000, 0.0440979003906250,
+-0.0485229492187500, 0.0556945800781250, -0.0489501953125000, 0.0372619628906250,
+-0.0436096191406250, -0.0062866210937500, -0.0267028808593750, 0.0177001953125000,
+-0.0318908691406250, -0.0220336914062500, -0.0206604003906250, -0.0282592773437500,
+-0.0184326171875000, -0.0197753906250000, -0.0190429687500000, -0.0325012207031250,
+-0.0146789550781250, -0.0324401855468750, -0.0150756835937500, -0.0153503417968750,
+-0.0162963867187500, -0.0202941894531250, -0.0151367187500000, -0.0151367187500000,
+-0.0176391601562500, 0.0004272460937500, -0.0211486816406250, -0.0062255859375000,
+-0.0201416015625000, -0.0012207031250000, -0.0247192382812500, 0.0028686523437500,
+-0.0268249511718750, -0.0021667480468750, -0.0252380371093750, -0.0054931640625000,
+-0.0263061523437500, -0.0023803710937500, -0.0281677246093750, -0.0093078613281250,
+-0.0202026367187500, -0.0116271972656250, -0.0245056152343750, -0.0078735351562500,
+-0.0201416015625000, -0.0086059570312500, -0.0115051269531250, -0.0109252929687500,
+-0.0180969238281250, 0.0018920898437500, -0.0063781738281250, -0.0001220703125000,
+-0.0078125000000000, 0.0003967285156250, -0.0057678222656250, 0.0107421875000000,
+0.0018615722656250, 0.0078735351562500, -0.0029602050781250, -0.0028686523437500,
+0.0047912597656250, 0.0040588378906250, 0.0054931640625000, -0.0055847167968750,
+0.0029907226562500, -0.0231933593750000, 0.0061035156250000, -0.0138549804687500,
+0.0061035156250000, -0.0217895507812500, -0.0031433105468750, -0.0292358398437500,
+0.0010375976562500, -0.0187988281250000, -0.0055236816406250, -0.0110168457031250,
+-0.0163574218750000, -0.0174255371093750, -0.0097961425781250, -0.0005187988281250,
+-0.0248107910156250, 0.0003662109375000, -0.0242919921875000, 0.0025634765625000,
+-0.0267028808593750, 0.0107421875000000, -0.0358276367187500, 0.0077819824218750,
+-0.0323791503906250, 0.0207214355468750, -0.0428771972656250, 0.0186157226562500,
+-0.0411071777343750, 0.0238037109375000, -0.0460205078125000, 0.0350952148437500,
+-0.0545959472656250, 0.0300598144531250, -0.0485839843750000, 0.0416564941406250,
+-0.0633239746093750, 0.0388183593750000, -0.0635070800781250, 0.0492553710937500,
+-0.0611572265625000, 0.0602111816406250, -0.0699462890625000, 0.0510559082031250,
+-0.0687866210937500, 0.0925598144531250, -0.0501708984375000, 0.0864562988281250,
+-0.0601501464843750, 0.0945739746093750, -0.0431518554687500, 0.1277770996093750,
+-0.0182189941406250, 0.1143493652343750, -0.0340881347656250, 0.1149902343750000,
+0.0062255859375000, 0.1300048828125000, 0.0046386718750000, 0.1060791015625000,
+0.0105285644531250, 0.0847167968750000, 0.0361022949218750, 0.1028137207031250,
+0.0246582031250000, 0.0457763671875000, 0.0370178222656250, 0.0522766113281250,
+0.0447082519531250, 0.0382995605468750, 0.0307006835937500, -0.0027770996093750,
+0.0267944335937500, 0.0169067382812500, 0.0361328125000000, -0.0188598632812500,
+-0.0038757324218750, -0.0217590332031250, -0.0015869140625000, -0.0234985351562500,
+-0.0030212402343750, -0.0405273437500000, -0.0279541015625000, -0.0344543457031250,
+-0.0213623046875000, -0.0482788085937500, -0.0136108398437500, -0.0447998046875000,
+-0.0227966308593750, -0.0554809570312500, -0.0083007812500000, -0.0698547363281250,
+0.0098266601562500, -0.0597839355468750, -0.0022888183593750, -0.0934448242187500,
+0.0272521972656250, -0.0904235839843750, 0.0352783203125000, -0.0956420898437500,
+0.0164184570312500, -0.1192016601562500, 0.0191345214843750, -0.1101074218750000,
+0.0304260253906250, -0.1170654296875000, -0.0256042480468750, -0.1227111816406250,
+-0.0172424316406250, -0.1151733398437500, -0.0302124023437500, -0.1122131347656250,
+-0.0698547363281250, -0.1156921386718750, -0.0534667968750000, -0.0933837890625000,
+-0.0960083007812500, -0.1030883789062500, -0.0955505371093750, -0.0789489746093750,
+-0.0937805175781250, -0.0521545410156250, -0.1224670410156250, -0.0732116699218750,
+-0.1168518066406250, 0.0055236816406250, -0.0803833007812500, -0.0077209472656250,
+-0.0995788574218750, 0.0156250000000000, -0.0793457031250000, 0.0812377929687500,
+-0.0321655273437500, 0.0503540039062500, -0.0541687011718750, 0.0894470214843750,
+-0.0355834960937500, 0.1077575683593750, -0.0239257812500000, 0.0827026367187500,
+-0.0345458984375000, 0.0770874023437500, -0.0421142578125000, 0.0916748046875000,
+-0.0366210937500000, 0.0443725585937500, -0.0346374511718750, 0.0423583984375000,
+-0.0465087890625000, 0.0492248535156250, -0.0256347656250000, 0.0273132324218750,
+-0.0065307617187500, 0.0287475585937500, -0.0247802734375000, 0.0595397949218750,
+0.0256042480468750, 0.0512084960937500, 0.0244445800781250, 0.0568237304687500,
+0.0226745605468750, 0.0957031250000000, 0.0613098144531250, 0.0748901367187500,
+-0.0232238769531250, 0.0598449707031250, 0.0326538085937500, 0.0661010742187500,
+0.0181579589843750, 0.0553894042968750, 0.0253601074218750, 0.0479125976562500,
+0.0492553710937500, 0.0551147460937500, 0.0436096191406250, 0.0129089355468750,
+0.0488281250000000, 0.0263977050781250, 0.0474853515625000, 0.0040283203125000,
+0.0537719726562500, -0.0409545898437500, 0.0612182617187500, -0.0162353515625000,
+0.0546569824218750, -0.0577087402343750, 0.0703735351562500, -0.0680236816406250,
+0.0735473632812500, -0.0573425292968750, 0.0656738281250000, -0.0638732910156250,
+0.0679321289062500, -0.0675354003906250, 0.0723876953125000, -0.0618896484375000,
+0.0532836914062500, -0.0563659667968750, 0.0543212890625000, -0.0701904296875000,
+0.0538635253906250, -0.0767211914062500, 0.0437011718750000, -0.0673217773437500,
+0.0458374023437500, -0.1109008789062500, 0.0409851074218750, -0.1024169921875000,
+0.0462951660156250, -0.1101989746093750, 0.0325622558593750, -0.1494750976562500,
+0.0215454101562500, -0.1335754394531250, 0.0329589843750000, -0.1118469238281250,
+-0.0150146484375000, -0.1418762207031250, -0.0067749023437500, -0.1007995605468750,
+-0.0218505859375000, -0.0459899902343750, -0.0601196289062500, -0.0836486816406250,
+-0.0416259765625000, -0.0042419433593750, -0.0788269042968750, -0.0019836425781250,
+-0.0787658691406250, -0.0007934570312500, -0.0837097167968750, 0.0400085449218750,
+-0.1070861816406250, 0.0249328613281250, -0.0960998535156250, 0.0287170410156250,
+-0.1036071777343750, 0.0393981933593750, -0.1135864257812500, 0.0288085937500000,
+-0.0961303710937500, 0.0148010253906250, -0.0851745605468750, 0.0228576660156250,
+-0.0972595214843750, 0.0234680175781250, -0.0534667968750000, 0.0142822265625000,
+-0.0562133789062500, 0.0223999023437500, -0.0524597167968750, 0.0347595214843750,
+-0.0251159667968750, 0.0271301269531250, -0.0335388183593750, 0.0355834960937500,
+-0.0279541015625000, 0.0394287109375000, -0.0278930664062500, 0.0333251953125000,
+-0.0218505859375000, 0.0338745117187500, -0.0183715820312500, 0.0372314453125000,
+-0.0246582031250000, 0.0160522460937500, 0.0077209472656250, 0.0244445800781250,
+-0.0000915527343750, 0.0104370117187500, 0.0119323730468750, -0.0121765136718750,
+0.0402832031250000},
+{0.0227966308593750, -0.0523376464843750, 0.0068054199218750, -0.0683898925781250,
+0.0198364257812500, -0.0020141601562500, -0.0145874023437500, -0.0140686035156250,
+-0.0096740722656250, 0.0088806152343750, -0.0271301269531250, 0.0671997070312500,
+-0.0552673339843750, 0.0388488769531250, -0.0359802246093750, 0.0742492675781250,
+-0.0892333984375000, 0.0903015136718750, -0.0922546386718750, 0.0701293945312500,
+-0.0829772949218750, 0.0673828125000000, -0.1067810058593750, 0.0794677734375000,
+-0.1068725585937500, 0.0404052734375000, -0.0730285644531250, 0.0401306152343750,
+-0.0828552246093750, 0.0468444824218750, -0.0714111328125000, 0.0285339355468750,
+-0.0406188964843750, 0.0296630859375000, -0.0545349121093750, 0.0568847656250000,
+-0.0362243652343750, 0.0491638183593750, -0.0307922363281250, 0.0580749511718750,
+-0.0372619628906250, 0.0840454101562500, -0.0369567871093750, 0.0729980468750000,
+-0.0341796875000000, 0.0774841308593750, -0.0325927734375000, 0.0848999023437500,
+-0.0408630371093750, 0.0785522460937500, -0.0273742675781250, 0.0668029785156250,
+0.0011901855468750, 0.0436096191406250, -0.0274658203125000, -0.0031127929687500,
+-0.0258789062500000, 0.0218505859375000, -0.0385437011718750, -0.0034179687500000,
+-0.0186157226562500, -0.0418395996093750, -0.0016784667968750, -0.0217895507812500,
+-0.0164489746093750, -0.0522460937500000, 0.0231628417968750, -0.0610046386718750,
+0.0236816406250000, -0.0514221191406250, 0.0226745605468750, -0.0548706054687500,
+0.0413513183593750, -0.0600891113281250, 0.0369262695312500, -0.0539550781250000,
+0.0419311523437500, -0.0436401367187500, 0.0397644042968750, -0.0665893554687500,
+0.0460205078125000, -0.0821838378906250, 0.0544433593750000, -0.0637512207031250,
+0.0478515625000000, -0.1180419921875000, 0.0602416992187500, -0.1179809570312500,
+0.0649414062500000, -0.1163024902343750, 0.0565185546875000, -0.1468505859375000,
+0.0552368164062500, -0.1384582519531250, 0.0603942871093750, -0.1163635253906250,
+0.0411376953125000, -0.1403808593750000, 0.0418701171875000, -0.1011657714843750,
+0.0414123535156250, -0.0544433593750000, 0.0319519042968750, -0.0917968750000000,
+0.0343017578125000, -0.0030822753906250, 0.0263977050781250, -0.0045471191406250,
+0.0322875976562500, -0.0025939941406250, 0.0180969238281250, 0.0441894531250000,
+0.0042114257812500, 0.0284423828125000, 0.0168457031250000, 0.0478515625000000,
+-0.0267639160156250, 0.0515441894531250, -0.0214538574218750, 0.0462036132812500,
+-0.0321655273437500, 0.0527648925781250, -0.0636291503906250, 0.0541687011718750,
+-0.0489196777343750, 0.0445251464843750, -0.0772094726562500, 0.0439147949218750,
+-0.0780029296875000, 0.0472717285156250, -0.0798339843750000, 0.0457458496093750,
+-0.0972595214843750, 0.0434570312500000, -0.0909729003906250, 0.0434265136718750,
+-0.0885620117187500, 0.0473022460937500, -0.0988464355468750, 0.0444030761718750,
+-0.0810852050781250, 0.0387573242187500, -0.0632934570312500, 0.0411682128906250,
+-0.0772094726562500, 0.0401000976562500, -0.0379638671875000, 0.0406494140625000,
+-0.0376892089843750, 0.0346069335937500, -0.0371093750000000, 0.0324096679687500,
+-0.0173645019531250, 0.0366516113281250, -0.0230407714843750, 0.0102233886718750,
+-0.0167541503906250, 0.0186462402343750, -0.0197753906250000, 0.0051574707031250,
+-0.0100402832031250, -0.0209350585937500, -0.0008239746093750, -0.0063171386718750,
+-0.0097045898437500, -0.0399780273437500, 0.0242614746093750, -0.0369873046875000,
+0.0192565917968750, -0.0462036132812500, 0.0287170410156250, -0.0698547363281250,
+0.0539855957031250, -0.0558471679687500, 0.0422973632812500, -0.0822143554687500,
+0.0699768066406250, -0.0854187011718750, 0.0669860839843750, -0.0797424316406250,
+0.0747680664062500, -0.0891113281250000, 0.0966491699218750, -0.0889587402343750,
+0.0853271484375000, -0.0745849609375000, 0.0992126464843750, -0.0806579589843750,
+0.1067810058593750, -0.0707397460937500, 0.0947875976562500, -0.0551757812500000,
+0.0907592773437500, -0.0652160644531250, 0.0979919433593750, -0.0386047363281250,
+0.0715026855468750, -0.0419311523437500, 0.0736999511718750, -0.0357971191406250,
+0.0683898925781250, -0.0173950195312500, 0.0514831542968750, -0.0274353027343750,
+0.0583190917968750, -0.0081176757812500, 0.0385742187500000, -0.0098876953125000,
+0.0415039062500000, -0.0057373046875000, 0.0335083007812500, 0.0067749023437500,
+0.0166015625000000, -0.0006713867187500, 0.0252990722656250, 0.0168151855468750,
+0.0055847167968750, 0.0142211914062500, 0.0058898925781250, 0.0186462402343750,
+0.0012817382812500, 0.0315856933593750, -0.0106811523437500, 0.0246276855468750,
+-0.0046081542968750, 0.0380554199218750, -0.0229492187500000, 0.0384216308593750,
+-0.0224304199218750, 0.0385437011718750, -0.0232238769531250, 0.0455932617187500,
+-0.0354309082031250, 0.0428466796875000, -0.0321960449218750, 0.0428466796875000,
+-0.0252990722656250, 0.0443420410156250, -0.0298767089843750, 0.0417175292968750,
+-0.0233154296875000, 0.0393066406250000, -0.0118713378906250, 0.0410766601562500,
+-0.0168151855468750, 0.0368652343750000, -0.0088806152343750, 0.0369262695312500,
+-0.0066223144531250, 0.0368957519531250, -0.0083312988281250, 0.0352172851562500,
+-0.0071105957031250, 0.0361633300781250, -0.0068359375000000, 0.0333557128906250,
+-0.0075683593750000, 0.0347290039062500, -0.0081481933593750, 0.0327148437500000,
+-0.0068969726562500, 0.0293273925781250, -0.0060729980468750, 0.0316162109375000,
+-0.0065612792968750, 0.0260009765625000, -0.0030822753906250, 0.0264282226562500,
+-0.0040588378906250, 0.0251464843750000, -0.0014343261718750, 0.0219421386718750,
+0.0012207031250000, 0.0238342285156250, -0.0013427734375000, 0.0173339843750000,
+0.0104064941406250, 0.0189514160156250, 0.0049438476562500, 0.0153503417968750,
+0.0154418945312500, 0.0088500976562500, 0.0287780761718750, 0.0123291015625000,
+0.0174865722656250, 0.0025329589843750, 0.0505065917968750, 0.0025634765625000,
+0.0453796386718750, 0.0022583007812500, 0.0531616210937500, -0.0032653808593750,
+0.0785522460937500, -0.0017700195312500, 0.0658264160156250, -0.0019531250000000,
+0.0804748535156250, -0.0024414062500000, 0.0866088867187500, -0.0024108886718750,
+0.0783386230468750, -0.0012512207031250, 0.0782470703125000, -0.0009765625000000,
+0.0832824707031250, -0.0060424804687500, 0.0667724609375000, -0.0028991699218750,
+0.0688171386718750, -0.0087890625000000, 0.0673828125000000, -0.0157165527343750,
+0.0572509765625000, -0.0113830566406250, 0.0609741210937500, -0.0323181152343750,
+0.0550842285156250, -0.0270385742187500, 0.0575256347656250, -0.0411376953125000,
+0.0494689941406250, -0.0630187988281250, 0.0431213378906250, -0.0503234863281250,
+0.0508422851562500, -0.0864868164062500, 0.0178222656250000, -0.0871582031250000,
+0.0267333984375000, -0.0877380371093750, 0.0134887695312500, -0.1093139648437500,
+-0.0171203613281250, -0.1036682128906250, -0.0003051757812500, -0.0909423828125000,
+-0.0285644531250000, -0.1044006347656250, -0.0314025878906250, -0.0812988281250000,
+-0.0307006835937500, -0.0528869628906250, -0.0417480468750000, -0.0713195800781250,
+-0.0390625000000000, -0.0198669433593750, -0.0516357421875000, -0.0191040039062500,
+-0.0446472167968750, -0.0180358886718750, -0.0596008300781250, 0.0095825195312500,
+-0.0772399902343750, 0.0006103515625000, -0.0618591308593750, -0.0025634765625000,
+-0.1031494140625000, 0.0045471191406250, -0.1005859375000000, -0.0045166015625000,
+-0.1002197265625000, -0.0180358886718750, -0.1263427734375000, -0.0115661621093750,
+-0.1188354492187500, -0.0163879394531250, -0.1036987304687500, -0.0227355957031250,
+-0.1190795898437500, -0.0149230957031250, -0.0979919433593750, -0.0091552734375000,
+-0.0693359375000000, -0.0145568847656250, -0.0892028808593750, 0.0048522949218750,
+-0.0422973632812500, 0.0037841796875000, -0.0431518554687500, 0.0013122558593750,
+-0.0375671386718750, 0.0140075683593750, -0.0113220214843750, 0.0128479003906250,
+-0.0212402343750000, -0.0088195800781250, 0.0081176757812500, -0.0024108886718750,
+0.0018005371093750, -0.0077819824218750, 0.0127258300781250, -0.0275878906250000,
+0.0390625000000000, -0.0212707519531250, 0.0231933593750000, -0.0263366699218750,
+0.0422058105468750, -0.0314331054687500, 0.0512695312500000, -0.0237731933593750,
+0.0388488769531250, -0.0204162597656250, 0.0357666015625000, -0.0260314941406250,
+0.0439147949218750, -0.0041809082031250, 0.0228576660156250, -0.0080871582031250,
+0.0214233398437500, -0.0026855468750000, 0.0241394042968750, 0.0170898437500000,
+0.0174255371093750, 0.0148315429687500, 0.0183410644531250, 0.0266418457031250,
+0.0161743164062500, 0.0260925292968750, 0.0190734863281250, 0.0216979980468750,
+0.0163574218750000, 0.0258789062500000, 0.0102539062500000, 0.0270385742187500,
+0.0145874023437500, 0.0200500488281250, 0.0167846679687500, 0.0183410644531250,
+0.0113220214843750, 0.0222167968750000, 0.0175476074218750, 0.0212402343750000,
+0.0277709960937500, 0.0184326171875000, 0.0228271484375000, 0.0310058593750000,
+0.0257873535156250, 0.0289916992187500, 0.0275268554687500, 0.0295715332031250,
+0.0281372070312500, 0.0396728515625000, 0.0289916992187500, 0.0366516113281250,
+0.0270080566406250, 0.0266113281250000, 0.0272827148437500, 0.0347290039062500,
+0.0317993164062500, 0.0228576660156250, 0.0234375000000000, 0.0057678222656250,
+0.0145568847656250, 0.0173339843750000, 0.0218505859375000, -0.0065002441406250,
+0.0112304687500000, -0.0064697265625000, 0.0068359375000000, -0.0075378417968750,
+0.0100402832031250, -0.0187988281250000, 0.0113220214843750, -0.0136718750000000,
+0.0097045898437500, -0.0268554687500000, 0.0054321289062500, -0.0225524902343750,
+0.0090637207031250, -0.0316467285156250, 0.0029907226562500, -0.0463256835937500,
+-0.0037231445312500, -0.0368347167968750, 0.0005493164062500, -0.0599670410156250,
+-0.0224304199218750, -0.0594482421875000, -0.0190734863281250, -0.0604553222656250,
+-0.0248718261718750, -0.0732116699218750, -0.0421447753906250, -0.0684814453125000,
+-0.0365295410156250, -0.0734863281250000, -0.0538940429687500, -0.0748291015625000,
+-0.0498657226562500, -0.0728759765625000, -0.0606079101562500, -0.0747375488281250,
+-0.0790100097656250, -0.0748901367187500, -0.0678100585937500, -0.0665893554687500,
+-0.0903320312500000, -0.0718383789062500, -0.0941467285156250, -0.0633544921875000,
+-0.0890197753906250, -0.0522155761718750, -0.0962219238281250, -0.0610961914062500,
+-0.0971679687500000, -0.0370788574218750, -0.0834350585937500, -0.0411071777343750,
+-0.0868225097656250, -0.0347290039062500, -0.0796203613281250, -0.0174865722656250,
+-0.0669555664062500, -0.0270996093750000, -0.0732421875000000, -0.0038452148437500,
+-0.0539245605468750, -0.0084838867187500, -0.0549621582031250, 0.0003051757812500,
+-0.0501708984375000, 0.0184936523437500, -0.0364990234375000, 0.0061340332031250,
+-0.0423889160156250, 0.0355529785156250, -0.0316467285156250, 0.0312500000000000,
+-0.0310058593750000, 0.0377502441406250, -0.0305175781250000, 0.0584716796875000,
+-0.0251464843750000, 0.0470275878906250, -0.0272827148437500, 0.0684509277343750,
+-0.0239868164062500, 0.0673217773437500, -0.0234069824218750, 0.0702209472656250,
+-0.0233459472656250, 0.0833129882812500, -0.0221862792968750, 0.0766906738281250,
+-0.0221862792968750, 0.0884704589843750, -0.0190429687500000, 0.0899658203125000,
+-0.0198364257812500, 0.0873107910156250, -0.0170898437500000, 0.0932922363281250,
+-0.0128479003906250, 0.0931396484375000, -0.0141601562500000, 0.0773925781250000,
+-0.0089721679687500},
+{0.0491638183593750, -0.0131835937500000, 0.0239562988281250, -0.0109252929687500,
+0.0301513671875000, -0.0167541503906250, 0.0262756347656250, -0.0346984863281250,
+0.0052185058593750, -0.0263671875000000, 0.0123596191406250, -0.0289001464843750,
+0.0142517089843750, -0.0380554199218750, 0.0087890625000000, -0.0269470214843750,
+0.0124206542968750, -0.0121765136718750, 0.0207519531250000, -0.0131530761718750,
+0.0195617675781250, 0.0099182128906250, 0.0198974609375000, 0.0023803710937500,
+0.0192871093750000, 0.0048828125000000, 0.0229492187500000, 0.0230407714843750,
+0.0255432128906250, 0.0146179199218750, 0.0220336914062500, 0.0206909179687500,
+0.0305175781250000, 0.0248718261718750, 0.0336914062500000, 0.0207214355468750,
+0.0263977050781250, 0.0182495117187500, 0.0234375000000000, 0.0205993652343750,
+0.0278930664062500, 0.0181274414062500, 0.0166320800781250, 0.0155639648437500,
+0.0149536132812500, 0.0201721191406250, 0.0146484375000000, 0.0224914550781250,
+0.0106201171875000, 0.0188903808593750, 0.0118103027343750, 0.0322875976562500,
+0.0071105957031250, 0.0315856933593750, 0.0082092285156250, 0.0303344726562500,
+0.0059814453125000, 0.0395202636718750, 0.0017700195312500, 0.0383300781250000,
+0.0031738281250000, 0.0223083496093750, -0.0060119628906250, 0.0310363769531250,
+-0.0019531250000000, 0.0196838378906250, -0.0111389160156250, -0.0010986328125000,
+-0.0245666503906250, 0.0112609863281250, -0.0163879394531250, -0.0088195800781250,
+-0.0330810546875000, -0.0108947753906250, -0.0339660644531250, -0.0104370117187500,
+-0.0371398925781250, -0.0173339843750000, -0.0464172363281250, -0.0147705078125000,
+-0.0419006347656250, -0.0272216796875000, -0.0530700683593750, -0.0216064453125000,
+-0.0512084960937500, -0.0306701660156250, -0.0596618652343750, -0.0461120605468750,
+-0.0694885253906250, -0.0361328125000000, -0.0605773925781250, -0.0564270019531250,
+-0.0878601074218750, -0.0570373535156250, -0.0873413085937500, -0.0568847656250000,
+-0.0837402343750000, -0.0669250488281250, -0.0976867675781250, -0.0634460449218750,
+-0.0956115722656250, -0.0656127929687500, -0.0800476074218750, -0.0671386718750000,
+-0.0853881835937500, -0.0644531250000000, -0.0773315429687500, -0.0643920898437500,
+-0.0604553222656250, -0.0657958984375000, -0.0676879882812500, -0.0552673339843750,
+-0.0558776855468750, -0.0604553222656250, -0.0538024902343750, -0.0521850585937500,
+-0.0527648925781250, -0.0397644042968750, -0.0478210449218750, -0.0488586425781250,
+-0.0494079589843750, -0.0266723632812500, -0.0420532226562500, -0.0299072265625000,
+-0.0425415039062500, -0.0244445800781250, -0.0378112792968750, -0.0093688964843750,
+-0.0308532714843750, -0.0183105468750000, -0.0337829589843750, 0.0031738281250000,
+-0.0213928222656250, -0.0012817382812500, -0.0210571289062500, 0.0072021484375000,
+-0.0186157226562500, 0.0245056152343750, -0.0100402832031250, 0.0133056640625000,
+-0.0120849609375000, 0.0412292480468750, -0.0099487304687500, 0.0375976562500000,
+-0.0071105957031250, 0.0439147949218750, -0.0099182128906250, 0.0637207031250000,
+-0.0114746093750000, 0.0534667968750000, -0.0094909667968750, 0.0725097656250000,
+-0.0203552246093750, 0.0727844238281250, -0.0177917480468750, 0.0733947753906250,
+-0.0235290527343750, 0.0836486816406250, -0.0346069335937500, 0.0792541503906250,
+-0.0301513671875000, 0.0816955566406250, -0.0437316894531250, 0.0846252441406250,
+-0.0450744628906250, 0.0788269042968750, -0.0469360351562500, 0.0766601562500000,
+-0.0548400878906250, 0.0797424316406250, -0.0532531738281250, 0.0592651367187500,
+-0.0567016601562500, 0.0680541992187500, -0.0582275390625000, 0.0532836914062500,
+-0.0585632324218750, 0.0311889648437500, -0.0596923828125000, 0.0478210449218750,
+-0.0594482421875000, 0.0046997070312500, -0.0598144531250000, 0.0104675292968750,
+-0.0602416992187500, 0.0016174316406250, -0.0593566894531250, -0.0302429199218750,
+-0.0582580566406250, -0.0157165527343750, -0.0584716796875000, -0.0362548828125000,
+-0.0575561523437500, -0.0418395996093750, -0.0551757812500000, -0.0345153808593750,
+-0.0579528808593750, -0.0388183593750000, -0.0595703125000000, -0.0411376953125000,
+-0.0559997558593750, -0.0222778320312500, -0.0651550292968750, -0.0274353027343750,
+-0.0625610351562500, -0.0197143554687500, -0.0658264160156250, -0.0022583007812500,
+-0.0733947753906250, -0.0106201171875000, -0.0688171386718750, 0.0065002441406250,
+-0.0805969238281250, 0.0068054199218750, -0.0797729492187500, 0.0101928710937500,
+-0.0807800292968750, 0.0199279785156250, -0.0890502929687500, 0.0149230957031250,
+-0.0860290527343750, 0.0296020507812500, -0.0842590332031250, 0.0280761718750000,
+-0.0890502929687500, 0.0321044921875000, -0.0830383300781250, 0.0421142578125000,
+-0.0765075683593750, 0.0375366210937500, -0.0822143554687500, 0.0535888671875000,
+-0.0664672851562500, 0.0482177734375000, -0.0709533691406250, 0.0609741210937500,
+-0.0634460449218750, 0.0796813964843750, -0.0505371093750000, 0.0676879882812500,
+-0.0603637695312500, 0.1005859375000000, -0.0321350097656250, 0.1029968261718750,
+-0.0366516113281250, 0.0990600585937500, -0.0319824218750000, 0.1151428222656250,
+-0.0105895996093750, 0.1130065917968750, -0.0201721191406250, 0.0888671875000000,
+-0.0138244628906250, 0.1005554199218750, -0.0060424804687500, 0.0814514160156250,
+-0.0156250000000000, 0.0511169433593750, -0.0222473144531250, 0.0683288574218750,
+-0.0144958496093750, 0.0235290527343750, -0.0327758789062500, 0.0244140625000000,
+-0.0321960449218750, 0.0183410644531250, -0.0331420898437500, -0.0083618164062500,
+-0.0422973632812500, 0.0024414062500000, -0.0383300781250000, -0.0170288085937500,
+-0.0481262207031250, -0.0166015625000000, -0.0473022460937500, -0.0221557617187500,
+-0.0452880859375000, -0.0343627929687500, -0.0540771484375000, -0.0272521972656250,
+-0.0529479980468750, -0.0514221191406250, -0.0266723632812500, -0.0472412109375000,
+-0.0419921875000000, -0.0545654296875000, -0.0158386230468750, -0.0747070312500000,
+0.0238037109375000, -0.0648803710937500, 0.0005187988281250, -0.0793762207031250,
+0.0538940429687500, -0.0830078125000000, 0.0578918457031250, -0.0794372558593750,
+0.0591430664062500, -0.0830383300781250, 0.0860290527343750, -0.0831909179687500,
+0.0768432617187500, -0.0752258300781250, 0.0742492675781250, -0.0802612304687500,
+0.0871582031250000, -0.0692749023437500, 0.0672302246093750, -0.0591430664062500,
+0.0451660156250000, -0.0675964355468750, 0.0617370605468750, -0.0223388671875000,
+0.0276184082031250, -0.0332336425781250, 0.0228576660156250, -0.0133361816406250,
+0.0276794433593750, 0.0304565429687500, 0.0170898437500000, 0.0082702636718750,
+0.0191040039062500, 0.0428771972656250, 0.0238952636718750, 0.0538940429687500,
+0.0198059082031250, 0.0401000976562500, 0.0283203125000000, 0.0434875488281250,
+0.0398864746093750, 0.0503234863281250, 0.0314941406250000, 0.0237121582031250,
+0.0367736816406250, 0.0244445800781250, 0.0482788085937500, 0.0279235839843750,
+0.0308227539062500, 0.0142211914062500, 0.0145874023437500, 0.0163574218750000,
+0.0286254882812500, 0.0351257324218750, -0.0008239746093750, 0.0284729003906250,
+-0.0028076171875000, 0.0351257324218750, -0.0030822753906250, 0.0540466308593750,
+-0.0148925781250000, 0.0458984375000000, -0.0104980468750000, 0.0509033203125000,
+-0.0247497558593750, 0.0562133789062500, -0.0220947265625000, 0.0506286621093750,
+-0.0274963378906250, 0.0460510253906250, -0.0378723144531250, 0.0489807128906250,
+-0.0332336425781250, 0.0407714843750000, -0.0615234375000000, 0.0429382324218750,
+-0.0527038574218750, 0.0346069335937500, -0.0642700195312500, 0.0272216796875000,
+-0.0966796875000000, 0.0341796875000000, -0.0820617675781250, -0.0014648437500000,
+-0.0791625976562500, 0.0048217773437500, -0.0957641601562500, -0.0025634765625000,
+-0.0779724121093750, -0.0472106933593750, -0.0350646972656250, -0.0455322265625000,
+-0.0857238769531250, -0.0610656738281250, -0.0712280273437500, -0.0450134277343750,
+-0.0956420898437500, -0.0645446777343750, -0.0688171386718750, -0.0869750976562500,
+-0.0341491699218750, -0.0724792480468750, -0.0538940429687500, -0.1063537597656250,
+-0.0249328613281250, -0.1083374023437500, -0.0171203613281250, -0.1053466796875000,
+-0.0253906250000000, -0.1239929199218750, -0.0234985351562500, -0.1204223632812500,
+-0.0211486816406250, -0.0932617187500000, -0.0172729492187500, -0.1092834472656250,
+-0.0258789062500000, -0.0880737304687500, -0.0119323730468750, -0.0519409179687500,
+0.0045776367187500, -0.0720825195312500, -0.0080566406250000, -0.0237731933593750,
+0.0179748535156250, -0.0312194824218750, 0.0205993652343750, -0.0102233886718750,
+0.0187072753906250, 0.0287780761718750, 0.0278930664062500, 0.0031127929687500,
+0.0258789062500000, 0.0654602050781250, 0.0321655273437500, 0.0654602050781250,
+0.0290527343750000, 0.0642089843750000, 0.0347595214843750, 0.0985107421875000,
+0.0450744628906250, 0.0889892578125000, 0.0393066406250000, 0.0772094726562500,
+0.0442199707031250, 0.0923156738281250, 0.0503845214843750, 0.0691833496093750,
+0.0405883789062500, 0.0406799316406250, 0.0333251953125000, 0.0625915527343750,
+0.0405578613281250, 0.0196228027343750, 0.0209045410156250, 0.0115051269531250,
+0.0209960937500000, 0.0275878906250000, 0.0189514160156250, 0.0177917480468750,
+0.0094299316406250, 0.0123596191406250, 0.0130615234375000, 0.0547790527343750,
+-0.0025939941406250, 0.0436706542968750, 0.0033264160156250, 0.0587158203125000,
+-0.0083007812500000, 0.0995483398437500, -0.0263671875000000, 0.0781555175781250,
+-0.0148620605468750, 0.0857849121093750, -0.0449829101562500, 0.1107482910156250,
+-0.0436706542968750, 0.0695800781250000, -0.0483093261718750, 0.0365295410156250,
+-0.0671997070312500, 0.0711364746093750, -0.0594482421875000, -0.0190429687500000,
+-0.0736083984375000, -0.0161132812500000, -0.0755004882812500, -0.0155639648437500,
+-0.0723876953125000, -0.0656433105468750, -0.0794677734375000, -0.0500183105468750,
+-0.0792846679687500, -0.0436096191406250, -0.0601806640625000, -0.0584716796875000,
+-0.0690002441406250, -0.0421447753906250, -0.0557556152343750, -0.0213012695312500,
+-0.0314636230468750, -0.0352478027343750, -0.0442504882812500, -0.0007934570312500,
+-0.0203857421875000, -0.0028686523437500, -0.0177612304687500, 0.0000610351562500,
+-0.0177307128906250, 0.0223693847656250, -0.0095214843750000, 0.0137329101562500,
+-0.0126037597656250, 0.0151672363281250, 0.0037841796875000, 0.0218811035156250,
+-0.0034179687500000, 0.0139770507812500, 0.0097351074218750, 0.0076293945312500,
+0.0300292968750000, 0.0138244628906250, 0.0168457031250000, -0.0079650878906250,
+0.0489196777343750, -0.0009155273437500, 0.0472412109375000, -0.0118408203125000,
+0.0538024902343750, -0.0322570800781250, 0.0742492675781250, -0.0179748535156250,
+0.0644531250000000, -0.0486450195312500, 0.0876159667968750, -0.0463562011718750,
+0.0880737304687500, -0.0503540039062500, 0.0874633789062500, -0.0699462890625000,
+0.1011962890625000, -0.0610351562500000, 0.0982360839843750, -0.0731506347656250,
+0.0875549316406250, -0.0760498046875000, 0.0950317382812500, -0.0725708007812500,
+0.0839233398437500, -0.0762939453125000, 0.0673522949218750, -0.0773010253906250,
+0.0769653320312500, -0.0677185058593750, 0.0518188476562500, -0.0727844238281250,
+0.0542602539062500, -0.0668029785156250, 0.0458984375000000, -0.0562744140625000,
+0.0274353027343750, -0.0637817382812500, 0.0373535156250000, -0.0466003417968750,
+0.0116882324218750, -0.0498352050781250, 0.0125732421875000, -0.0446472167968750,
+0.0084533691406250},
+{-0.0821533203125000, -0.0753784179687500, -0.0623779296875000, -0.0541992187500000,
+-0.0287170410156250, -0.0679626464843750, -0.0476074218750000, -0.0294494628906250,
+-0.0231323242187500, -0.0348815917968750, -0.0152587890625000, -0.0188903808593750,
+-0.0220642089843750, 0.0092468261718750, -0.0216369628906250, -0.0090332031250000,
+-0.0198974609375000, 0.0515136718750000, -0.0140686035156250, 0.0438842773437500,
+-0.0216064453125000, 0.0523071289062500, -0.0104064941406250, 0.0985412597656250,
+0.0053405761718750, 0.0812683105468750, -0.0049743652343750, 0.0764465332031250,
+0.0132446289062500, 0.0964355468750000, 0.0161132812500000, 0.0703430175781250,
+0.0144042968750000, 0.0395507812500000, 0.0193481445312500, 0.0625305175781250,
+0.0184936523437500, 0.0180053710937500, 0.0260009765625000, 0.0120544433593750,
+0.0227966308593750, 0.0255126953125000, 0.0280151367187500, 0.0108032226562500,
+0.0391235351562500, 0.0077209472656250, 0.0336608886718750, 0.0492248535156250,
+0.0352172851562500, 0.0385742187500000, 0.0417480468750000, 0.0516662597656250,
+0.0322570800781250, 0.0886535644531250, 0.0231018066406250, 0.0710754394531250,
+0.0302734375000000, 0.0876464843750000, 0.0116271972656250, 0.1034240722656250,
+0.0124511718750000, 0.0727233886718750, 0.0087280273437500, 0.0596008300781250,
+-0.0015258789062500, 0.0831909179687500, 0.0039367675781250, -0.0030822753906250,
+-0.0157165527343750, 0.0067749023437500, -0.0104370117187500, 0.0020751953125000,
+-0.0207214355468750, -0.0552978515625000, -0.0405273437500000, -0.0372314453125000,
+-0.0296936035156250, -0.0321960449218750, -0.0538024902343750, -0.0488586425781250,
+-0.0541381835937500, -0.0303649902343750, -0.0565795898437500, -0.0061645507812500,
+-0.0702514648437500, -0.0226440429687500, -0.0648498535156250, 0.0041198730468750,
+-0.0733032226562500, 0.0101318359375000, -0.0765686035156250, 0.0037536621093750,
+-0.0699462890625000, 0.0114746093750000, -0.0708923339843750, 0.0132751464843750,
+-0.0742797851562500, -0.0031127929687500, -0.0476684570312500, 0.0028686523437500,
+-0.0549011230468750, -0.0040283203125000, -0.0438842773437500, -0.0205383300781250,
+-0.0180664062500000, -0.0115966796875000, -0.0301513671875000, -0.0227966308593750,
+-0.0110168457031250, -0.0256042480468750, -0.0091247558593750, -0.0241394042968750,
+-0.0068969726562500, -0.0267028808593750, 0.0008239746093750, -0.0261230468750000,
+-0.0037536621093750, -0.0289916992187500, 0.0165405273437500, -0.0297241210937500,
+0.0109252929687500, -0.0287780761718750, 0.0207519531250000, -0.0300903320312500,
+0.0407714843750000, -0.0316772460937500, 0.0304565429687500, -0.0277709960937500,
+0.0540771484375000, -0.0292053222656250, 0.0530395507812500, -0.0283203125000000,
+0.0585327148437500, -0.0254211425781250, 0.0744934082031250, -0.0280761718750000,
+0.0668029785156250, -0.0249328613281250, 0.0824279785156250, -0.0252990722656250,
+0.0859985351562500, -0.0263366699218750, 0.0802001953125000, -0.0250244140625000,
+0.0841674804687500, -0.0252990722656250, 0.0867614746093750, -0.0271911621093750,
+0.0679626464843750, -0.0260925292968750, 0.0721130371093750, -0.0264892578125000,
+0.0650939941406250, -0.0280151367187500, 0.0492553710937500, -0.0264892578125000,
+0.0565795898437500, -0.0270385742187500, 0.0377502441406250, -0.0266418457031250,
+0.0399169921875000, -0.0249023437500000, 0.0326232910156250, -0.0238342285156250,
+0.0173034667968750, -0.0227966308593750, 0.0249633789062500, -0.0191040039062500,
+0.0051269531250000, -0.0185546875000000, 0.0045166015625000, -0.0161743164062500,
+0.0028381347656250, -0.0128784179687500, -0.0083618164062500, -0.0128479003906250,
+-0.0047302246093750, -0.0078735351562500, -0.0090942382812500, -0.0077209472656250,
+-0.0121765136718750, -0.0050354003906250, -0.0077514648437500, -0.0006713867187500,
+-0.0077514648437500, -0.0016784667968750, -0.0113525390625000, 0.0044555664062500,
+0.0066833496093750, 0.0059204101562500, -0.0010375976562500, 0.0067749023437500,
+0.0114135742187500, 0.0101013183593750, 0.0317687988281250, 0.0111083984375000,
+0.0177307128906250, 0.0129699707031250, 0.0496826171875000, 0.0130004882812500,
+0.0495605468750000, 0.0155639648437500, 0.0500793457031250, 0.0180053710937500,
+0.0678100585937500, 0.0167541503906250, 0.0621948242187500, 0.0225219726562500,
+0.0622253417968750, 0.0226745605468750, 0.0664672851562500, 0.0226440429687500,
+0.0625000000000000, 0.0260925292968750, 0.0573120117187500, 0.0254821777343750,
+0.0605773925781250, 0.0222778320312500, 0.0563659667968750, 0.0241699218750000,
+0.0559387207031250, 0.0211181640625000, 0.0543823242187500, 0.0165100097656250,
+0.0535583496093750, 0.0188293457031250, 0.0552062988281250, 0.0115661621093750,
+0.0412292480468750, 0.0120239257812500, 0.0484008789062500, 0.0095214843750000,
+0.0362854003906250, 0.0043640136718750, 0.0185546875000000, 0.0067443847656250,
+0.0313720703125000, -0.0021667480468750, 0.0005798339843750, -0.0009155273437500,
+0.0022277832031250, -0.0030517578125000, -0.0014343261718750, -0.0103149414062500,
+-0.0199890136718750, -0.0067443847656250, -0.0120849609375000, -0.0074157714843750,
+-0.0257873535156250, -0.0105285644531250, -0.0248107910156250, -0.0038146972656250,
+-0.0302124023437500, 0.0015869140625000, -0.0394592285156250, -0.0023193359375000,
+-0.0330810546875000, 0.0132141113281250, -0.0562744140625000, 0.0134887695312500,
+-0.0505371093750000, 0.0129089355468750, -0.0597534179687500, 0.0227050781250000,
+-0.0809936523437500, 0.0211791992187500, -0.0681762695312500, 0.0082702636718750,
+-0.0860290527343750, 0.0169677734375000, -0.0925292968750000, 0.0021667480468750,
+-0.0794982910156250, -0.0175781250000000, -0.0803222656250000, -0.0054321289062500,
+-0.0882568359375000, -0.0455932617187500, -0.0498352050781250, -0.0413208007812500,
+-0.0601501464843750, -0.0504455566406250, -0.0479125976562500, -0.0815734863281250,
+-0.0143737792968750, -0.0676269531250000, -0.0314636230468750, -0.0748901367187500,
+-0.0054931640625000, -0.0868835449218750, -0.0024414062500000, -0.0692443847656250,
+-0.0019531250000000, -0.0538635253906250, 0.0082702636718750, -0.0655212402343750,
+0.0044555664062500, -0.0357360839843750, 0.0202026367187500, -0.0337829589843750,
+0.0151367187500000, -0.0337219238281250, 0.0224304199218750, -0.0201721191406250,
+0.0401611328125000, -0.0232238769531250, 0.0299682617187500, -0.0180358886718750,
+0.0339660644531250, -0.0178222656250000, 0.0444335937500000, -0.0169982910156250,
+0.0296020507812500, -0.0131225585937500, 0.0157165527343750, -0.0142517089843750,
+0.0283508300781250, -0.0145874023437500, 0.0051269531250000, -0.0176086425781250,
+-0.0003662109375000, -0.0098876953125000, 0.0104980468750000, -0.0036621093750000,
+0.0073547363281250, -0.0115051269531250, 0.0015563964843750, 0.0022277832031250,
+0.0256347656250000, 0.0045776367187500, 0.0238342285156250, 0.0042724609375000,
+0.0234069824218750, 0.0084838867187500, 0.0379333496093750, 0.0054626464843750,
+0.0349121093750000, 0.0078735351562500, 0.0278320312500000, 0.0128784179687500,
+0.0322875976562500, 0.0018920898437500, 0.0274658203125000, -0.0121765136718750,
+0.0168457031250000, -0.0123596191406250, 0.0172119140625000, -0.0280456542968750,
+0.0043945312500000, -0.0283203125000000, 0.0094604492187500, -0.0230712890625000,
+0.0054626464843750, -0.0291748046875000, -0.0083618164062500, -0.0308227539062500,
+-0.0025939941406250, -0.0128784179687500, -0.0054321289062500, -0.0199279785156250,
+-0.0121459960937500, -0.0107727050781250, -0.0053710937500000, 0.0084533691406250,
+0.0032348632812500, -0.0027465820312500, -0.0032043457031250, 0.0136108398437500,
+-0.0004882812500000, 0.0161132812500000, 0.0072021484375000, 0.0160522460937500,
+-0.0083007812500000, 0.0222473144531250, -0.0213623046875000, 0.0194702148437500,
+-0.0099487304687500, 0.0253295898437500, -0.0388183593750000, 0.0267333984375000,
+-0.0413208007812500, 0.0240173339843750, -0.0403442382812500, 0.0245971679687500,
+-0.0531311035156250, 0.0266418457031250, -0.0498657226562500, 0.0231628417968750,
+-0.0466003417968750, 0.0208435058593750, -0.0507812500000000, 0.0265502929687500,
+-0.0475158691406250, 0.0296630859375000, -0.0402221679687500, 0.0253601074218750,
+-0.0433349609375000, 0.0368041992187500, -0.0419921875000000, 0.0403137207031250,
+-0.0379638671875000, 0.0321960449218750, -0.0448608398437500, 0.0333862304687500,
+-0.0503845214843750, 0.0386352539062500, -0.0438842773437500, 0.0095825195312500,
+-0.0570678710937500, 0.0154724121093750, -0.0606994628906250, 0.0097351074218750,
+-0.0536193847656250, -0.0128479003906250, -0.0544128417968750, -0.0032348632812500,
+-0.0582580566406250, -0.0116271972656250, -0.0420837402343750, -0.0140075683593750,
+-0.0453186035156250, -0.0155944824218750, -0.0394897460937500, -0.0171813964843750,
+-0.0266723632812500, -0.0140991210937500, -0.0333251953125000, -0.0305175781250000,
+-0.0151672363281250, -0.0249633789062500, -0.0166625976562500, -0.0310974121093750,
+-0.0115356445312500, -0.0470581054687500, 0.0030212402343750, -0.0386352539062500,
+-0.0035400390625000, -0.0510864257812500, 0.0032043457031250, -0.0532226562500000,
+0.0080871582031250, -0.0509948730468750, 0.0013427734375000, -0.0548095703125000,
+-0.0019531250000000, -0.0543823242187500, 0.0015563964843750, -0.0519714355468750,
+-0.0144348144531250, -0.0535888671875000, -0.0126647949218750, -0.0496520996093750,
+-0.0183105468750000, -0.0463867187500000, -0.0313415527343750, -0.0497131347656250,
+-0.0252380371093750, -0.0351867675781250, -0.0344848632812500, -0.0399169921875000,
+-0.0369873046875000, -0.0328369140625000, -0.0324707031250000, -0.0191040039062500,
+-0.0330200195312500, -0.0278930664062500, -0.0334777832031250, -0.0084533691406250,
+-0.0224609375000000, -0.0110168457031250, -0.0242919921875000, -0.0061340332031250,
+-0.0159301757812500, 0.0068969726562500, -0.0038146972656250, -0.0008544921875000,
+-0.0083618164062500, 0.0195922851562500, 0.0122375488281250, 0.0157775878906250,
+0.0127868652343750, 0.0232849121093750, 0.0174865722656250, 0.0404052734375000,
+0.0315856933593750, 0.0304870605468750, 0.0281372070312500, 0.0496826171875000,
+0.0361022949218750, 0.0502929687500000, 0.0382995605468750, 0.0497131347656250,
+0.0373535156250000, 0.0586547851562500, 0.0395812988281250, 0.0552062988281250,
+0.0399780273437500, 0.0568542480468750, 0.0392761230468750, 0.0566711425781250,
+0.0398559570312500, 0.0567626953125000, 0.0379333496093750, 0.0579833984375000,
+0.0363769531250000, 0.0567016601562500, 0.0371704101562500, 0.0574035644531250,
+0.0297851562500000, 0.0587463378906250, 0.0314025878906250, 0.0561218261718750,
+0.0271911621093750, 0.0555419921875000, 0.0198669433593750, 0.0579528808593750,
+0.0233764648437500, 0.0467529296875000, 0.0132751464843750, 0.0520324707031250,
+0.0134277343750000, 0.0456237792968750, 0.0104980468750000, 0.0336303710937500,
+0.0040588378906250, 0.0424804687500000, 0.0074462890625000, 0.0256042480468750,
+-0.0027465820312500, 0.0254211425781250, -0.0010070800781250, 0.0270690917968750,
+-0.0036621093750000, 0.0187988281250000, -0.0108032226562500, 0.0209350585937500,
+-0.0063171386718750, 0.0258178710937500, -0.0153808593750000, 0.0216369628906250,
+-0.0140075683593750, 0.0298461914062500, -0.0159912109375000, 0.0378112792968750,
+-0.0219116210937500, 0.0312194824218750, -0.0179138183593750, 0.0576477050781250,
+-0.0267333984375000, 0.0532226562500000, -0.0256652832031250, 0.0611267089843750,
+-0.0264892578125000, 0.0837402343750000, -0.0326843261718750, 0.0734863281250000,
+-0.0294189453125000},
+{-0.0243225097656250, 0.0236206054687500, -0.0245666503906250, 0.0216369628906250,
+-0.0391235351562500, 0.0176696777343750, -0.0337524414062500, 0.0248413085937500,
+-0.0380554199218750, 0.0317382812500000, -0.0544738769531250, 0.0257873535156250,
+-0.0491943359375000, 0.0348205566406250, -0.0435180664062500, 0.0407714843750000,
+-0.0518798828125000, 0.0290222167968750, -0.0405273437500000, 0.0245361328125000,
+-0.0268249511718750, 0.0332641601562500, -0.0367126464843750, 0.0025634765625000,
+-0.0076293945312500, 0.0057983398437500, -0.0092773437500000, 0.0032958984375000,
+-0.0075073242187500, -0.0164489746093750, 0.0108337402343750, -0.0098876953125000,
+0.0046691894531250, -0.0148315429687500, 0.0040588378906250, -0.0152893066406250,
+0.0097351074218750, -0.0185546875000000, 0.0004272460937500, -0.0212097167968750,
+-0.0092163085937500, -0.0172424316406250, -0.0022583007812500, -0.0345458984375000,
+-0.0172729492187500, -0.0303955078125000, -0.0203552246093750, -0.0340881347656250,
+-0.0172729492187500, -0.0482788085937500, -0.0216674804687500, -0.0417785644531250,
+-0.0218811035156250, -0.0481872558593750, -0.0117492675781250, -0.0503845214843750,
+-0.0146179199218750, -0.0484924316406250, -0.0066528320312500, -0.0493469238281250,
+0.0051879882812500, -0.0496215820312500, -0.0002136230468750, -0.0475769042968750,
+0.0187377929687500, -0.0492248535156250, 0.0187988281250000, -0.0453796386718750,
+0.0225830078125000, -0.0426025390625000, 0.0344848632812500, -0.0458679199218750,
+0.0301208496093750, -0.0306091308593750, 0.0399169921875000, -0.0359802246093750,
+0.0403137207031250, -0.0278930664062500, 0.0407409667968750, -0.0128173828125000,
+0.0463256835937500, -0.0224304199218750, 0.0442810058593750, -0.0007019042968750,
+0.0431213378906250, -0.0023803710937500, 0.0453491210937500, 0.0010070800781250,
+0.0423278808593750, 0.0141906738281250, 0.0386657714843750, 0.0072326660156250,
+0.0411376953125000, 0.0218200683593750, 0.0354003906250000, 0.0191650390625000,
+0.0361328125000000, 0.0244140625000000, 0.0352172851562500, 0.0356750488281250,
+0.0320739746093750, 0.0285644531250000, 0.0344543457031250, 0.0473632812500000,
+0.0289001464843750, 0.0449829101562500, 0.0311279296875000, 0.0500488281250000,
+0.0285034179687500, 0.0641174316406250, 0.0229797363281250, 0.0574035644531250,
+0.0269470214843750, 0.0720520019531250, 0.0189819335937500, 0.0715637207031250,
+0.0201721191406250, 0.0746765136718750, 0.0186462402343750, 0.0850524902343750,
+0.0131835937500000, 0.0804443359375000, 0.0163574218750000, 0.0861206054687500,
+0.0106201171875000, 0.0900268554687500, 0.0105590820312500, 0.0840148925781250,
+0.0111999511718750, 0.0827331542968750, 0.0083312988281250, 0.0870971679687500,
+0.0091247558593750, 0.0672607421875000, 0.0114135742187500, 0.0723876953125000,
+0.0099487304687500, 0.0662841796875000, 0.0119934082031250, 0.0483093261718750,
+0.0156860351562500, 0.0574645996093750, 0.0137023925781250, 0.0497131347656250,
+0.0151062011718750, 0.0439758300781250, 0.0173950195312500, 0.0543823242187500,
+0.0152282714843750, 0.0599670410156250, 0.0143737792968750, 0.0534362792968750,
+0.0172119140625000, 0.0730895996093750, 0.0098266601562500, 0.0750122070312500,
+0.0134582519531250, 0.0699768066406250, 0.0086669921875000, 0.0776977539062500,
+0.0018615722656250, 0.0781555175781250, 0.0087585449218750, 0.0560302734375000,
+-0.0106506347656250, 0.0639648437500000, -0.0050354003906250, 0.0497741699218750,
+-0.0103149414062500, 0.0245971679687500, -0.0267028808593750, 0.0371398925781250,
+-0.0162963867187500, 0.0068969726562500, -0.0277404785156250, 0.0057067871093750,
+-0.0315551757812500, 0.0020446777343750, -0.0235290527343750, -0.0145568847656250,
+-0.0239868164062500, -0.0085449218750000, -0.0285034179687500, -0.0229797363281250,
+-0.0093688964843750, -0.0213928222656250, -0.0127563476562500, -0.0275878906250000,
+-0.0088806152343750, -0.0394592285156250, 0.0062255859375000, -0.0328063964843750,
+0.0006408691406250, -0.0506286621093750, 0.0065612792968750, -0.0508728027343750,
+0.0082702636718750, -0.0513610839843750, 0.0068054199218750, -0.0606689453125000,
+0.0092468261718750, -0.0577392578125000, 0.0086059570312500, -0.0597229003906250,
+0.0003662109375000, -0.0610656738281250, 0.0080566406250000, -0.0586547851562500,
+-0.0079345703125000, -0.0592041015625000, -0.0245666503906250, -0.0601196289062500,
+-0.0098876953125000, -0.0464782714843750, -0.0512695312500000, -0.0540771484375000,
+-0.0492553710937500, -0.0395507812500000, -0.0514831542968750, -0.0198364257812500,
+-0.0779418945312500, -0.0342407226562500, -0.0699768066406250, -0.0010070800781250,
+-0.0654602050781250, 0.0015258789062500, -0.0747680664062500, -0.0020446777343750,
+-0.0588989257812500, 0.0127258300781250, -0.0447692871093750, 0.0110778808593750,
+-0.0561218261718750, -0.0005493164062500, -0.0058288574218750, 0.0034179687500000,
+-0.0152587890625000, 0.0024108886718750, -0.0003662109375000, -0.0082092285156250,
+0.0437927246093750, -0.0052490234375000, 0.0247497558593750, 0.0027770996093750,
+0.0409240722656250, -0.0025329589843750, 0.0532226562500000, 0.0048217773437500,
+0.0401000976562500, 0.0174560546875000, 0.0310974121093750, 0.0108642578125000,
+0.0391540527343750, 0.0207519531250000, 0.0261230468750000, 0.0234680175781250,
+0.0217895507812500, 0.0208435058593750, 0.0278320312500000, 0.0220947265625000,
+0.0271301269531250, 0.0224609375000000, 0.0242614746093750, 0.0209655761718750,
+0.0385437011718750, 0.0215454101562500, 0.0369262695312500, 0.0184936523437500,
+0.0385742187500000, 0.0176086425781250, 0.0500488281250000, 0.0193786621093750,
+0.0455932617187500, 0.0017395019531250, 0.0341796875000000, 0.0083923339843750,
+0.0465393066406250, -0.0025329589843750, 0.0287170410156250, -0.0229492187500000,
+0.0027160644531250, -0.0116577148437500, 0.0191345214843750, -0.0320129394531250,
+-0.0089721679687500, -0.0364074707031250, -0.0169982910156250, -0.0325317382812500,
+-0.0038757324218750, -0.0359497070312500, -0.0055236816406250, -0.0373535156250000,
+-0.0126037597656250, -0.0435485839843750, 0.0105285644531250, -0.0347900390625000,
+0.0163879394531250, -0.0529479980468750, 0.0017395019531250, -0.0726318359375000,
+0.0009155273437500, -0.0564880371093750, 0.0112304687500000, -0.0941467285156250,
+-0.0155029296875000, -0.1007690429687500, -0.0184020996093750, -0.0884094238281250,
+-0.0125427246093750, -0.0943603515625000, -0.0217285156250000, -0.0470581054687500,
+0.0038452148437500, -0.0283813476562500, -0.0159912109375000, -0.0386047363281250,
+-0.0086975097656250, -0.0182495117187500, -0.0124206542968750, -0.0032348632812500,
+-0.0245056152343750, -0.0201721191406250, -0.0220947265625000, 0.0340270996093750,
+-0.0175476074218750, 0.0301818847656250, -0.0174255371093750, 0.0339355468750000,
+-0.0256042480468750, 0.0683898925781250, -0.0253295898437500, 0.0565490722656250,
+-0.0194091796875000, 0.0560913085937500, -0.0530090332031250, 0.0675964355468750,
+-0.0502319335937500, 0.0530090332031250, -0.0483093261718750, 0.0366516113281250,
+-0.0703430175781250, 0.0493469238281250, -0.0675354003906250, 0.0275268554687500,
+-0.0433044433593750, 0.0229797363281250, -0.0540466308593750, 0.0304260253906250,
+-0.0427551269531250, 0.0263061523437500, -0.0149230957031250, 0.0237121582031250,
+-0.0274963378906250, 0.0375976562500000, -0.0164184570312500, 0.0377502441406250,
+-0.0115356445312500, 0.0348815917968750, -0.0141601562500000, 0.0410461425781250,
+-0.0156555175781250, 0.0425109863281250, -0.0158996582031250, 0.0334472656250000,
+-0.0056457519531250, 0.0337829589843750, -0.0103759765625000, 0.0385437011718750,
+-0.0051879882812500, 0.0359497070312500, 0.0075378417968750, 0.0325927734375000,
+0.0015869140625000, 0.0435485839843750, 0.0058898925781250, 0.0500793457031250,
+0.0084838867187500, 0.0302429199218750, 0.0079040527343750, 0.0240173339843750,
+0.0066833496093750, 0.0389099121093750, 0.0064697265625000, -0.0217590332031250,
+0.0144348144531250, -0.0135498046875000, 0.0120849609375000, -0.0235900878906250,
+0.0147094726562500, -0.0648803710937500, 0.0236511230468750, -0.0469970703125000,
+0.0206604003906250, -0.0741577148437500, 0.0182495117187500, -0.0771484375000000,
+0.0231628417968750, -0.0705871582031250, 0.0164184570312500, -0.0812377929687500,
+0.0082397460937500, -0.0805664062500000, 0.0137939453125000, -0.0619812011718750,
+-0.0008239746093750, -0.0684509277343750, 0.0014648437500000, -0.0593872070312500,
+-0.0046386718750000, -0.0423278808593750, -0.0156250000000000, -0.0525207519531250,
+-0.0086364746093750, -0.0268554687500000, -0.0312805175781250, -0.0298156738281250,
+-0.0283203125000000, -0.0241088867187500, -0.0337829589843750, -0.0057373046875000,
+-0.0509033203125000, -0.0150756835937500, -0.0434570312500000, -0.0000305175781250,
+-0.0566101074218750, 0.0039978027343750, -0.0576477050781250, -0.0034790039062500,
+-0.0575866699218750, -0.0014038085937500, -0.0646972656250000, 0.0028076171875000,
+-0.0620422363281250, -0.0145568847656250, -0.0588989257812500, -0.0129394531250000,
+-0.0644531250000000, -0.0145874023437500, -0.0542297363281250, -0.0251464843750000,
+-0.0440979003906250, -0.0207519531250000, -0.0529174804687500, -0.0249328613281250,
+-0.0272216796875000, -0.0270690917968750, -0.0279235839843750, -0.0229492187500000,
+-0.0262756347656250, -0.0226135253906250, -0.0121765136718750, -0.0255737304687500,
+-0.0166931152343750, -0.0137634277343750, -0.0082702636718750, -0.0175170898437500,
+-0.0107116699218750, -0.0129699707031250, -0.0032653808593750, -0.0028076171875000,
+0.0059814453125000, -0.0097656250000000, -0.0010375976562500, 0.0028686523437500,
+0.0236816406250000, 0.0020751953125000, 0.0206909179687500, 0.0021972656250000,
+0.0278625488281250, 0.0090026855468750, 0.0460510253906250, 0.0054321289062500,
+0.0376586914062500, 0.0078430175781250, 0.0585021972656250, 0.0092163085937500,
+0.0580749511718750, 0.0064697265625000, 0.0611877441406250, 0.0055847167968750,
+0.0750732421875000, 0.0068664550781250, 0.0698547363281250, 0.0010681152343750,
+0.0738220214843750, 0.0025024414062500, 0.0768127441406250, 0.0001220703125000,
+0.0739135742187500, -0.0048828125000000, 0.0709838867187500, -0.0015563964843750,
+0.0718994140625000, -0.0064697265625000, 0.0725097656250000, -0.0065002441406250,
+0.0691833496093750, -0.0055847167968750, 0.0727844238281250, -0.0076293945312500,
+0.0773010253906250, -0.0064086914062500, 0.0731201171875000, -0.0036010742187500,
+0.0797424316406250, -0.0049133300781250, 0.0806579589843750, -0.0014038085937500,
+0.0772705078125000, 0.0028686523437500, 0.0786437988281250, 0.0006408691406250,
+0.0792236328125000, 0.0087585449218750, 0.0694885253906250, 0.0083618164062500,
+0.0722351074218750, 0.0106811523437500, 0.0672302246093750, 0.0164184570312500,
+0.0578308105468750, 0.0141906738281250, 0.0629272460937500, 0.0204772949218750,
+0.0516662597656250, 0.0204467773437500, 0.0520629882812500, 0.0208740234375000,
+0.0494995117187500, 0.0249328613281250, 0.0426025390625000, 0.0234375000000000,
+0.0461730957031250, 0.0219116210937500, 0.0355529785156250, 0.0240783691406250,
+0.0380859375000000, 0.0203247070312500, 0.0329589843750000, 0.0161132812500000,
+0.0241394042968750, 0.0188293457031250, 0.0305786132812500, 0.0096740722656250,
+0.0142211914062500, 0.0100097656250000, 0.0168762207031250, 0.0086975097656250,
+0.0121154785156250, 0.0026245117187500, -0.0001220703125000, 0.0043640136718750,
+0.0068054199218750, 0.0020751953125000, -0.0080871582031250, 0.0006103515625000,
+-0.0064086914062500},
+{0.0097656250000000, -0.1048889160156250, 0.0078125000000000, -0.0867614746093750,
+0.0094909667968750, -0.0467834472656250, 0.0159912109375000, -0.0674438476562500,
+0.0141601562500000, -0.0396118164062500, 0.0108642578125000, -0.0339355468750000,
+0.0145263671875000, -0.0368652343750000, 0.0093994140625000, -0.0284118652343750,
+0.0029602050781250, -0.0303039550781250, 0.0073852539062500, -0.0265197753906250,
+-0.0064086914062500, -0.0268249511718750, -0.0031433105468750, -0.0248413085937500,
+-0.0104675292968750, -0.0209045410156250, -0.0230712890625000, -0.0228271484375000,
+-0.0153198242187500, -0.0209045410156250, -0.0374755859375000, -0.0200500488281250,
+-0.0358581542968750, -0.0187988281250000, -0.0393066406250000, -0.0187377929687500,
+-0.0538024902343750, -0.0200500488281250, -0.0478210449218750, -0.0126037597656250,
+-0.0569152832031250, -0.0144348144531250, -0.0580139160156250, -0.0112915039062500,
+-0.0568847656250000, -0.0056457519531250, -0.0617980957031250, -0.0092163085937500,
+-0.0606384277343750, 0.0030517578125000, -0.0516357421875000, -0.0001525878906250,
+-0.0579833984375000, 0.0042114257812500, -0.0473327636718750, 0.0156860351562500,
+-0.0323791503906250, 0.0095825195312500, -0.0421447753906250, 0.0174255371093750,
+-0.0200195312500000, 0.0202636718750000, -0.0191040039062500, 0.0159912109375000,
+-0.0187377929687500, 0.0169372558593750, -0.0087585449218750, 0.0196533203125000,
+-0.0119628906250000, 0.0080261230468750, -0.0028686523437500, 0.0117492675781250,
+-0.0060424804687500, 0.0070800781250000, 0.0016784667968750, -0.0035095214843750,
+0.0125732421875000, 0.0025939941406250, 0.0057983398437500, -0.0099182128906250,
+0.0278015136718750, -0.0095520019531250, 0.0249328613281250, -0.0109863281250000,
+0.0338439941406250, -0.0186462402343750, 0.0509948730468750, -0.0155639648437500,
+0.0417480468750000, -0.0207824707031250, 0.0693054199218750, -0.0216369628906250,
+0.0672607421875000, -0.0216979980468750, 0.0723876953125000, -0.0245361328125000,
+0.0917663574218750, -0.0246887207031250, 0.0837707519531250, -0.0250854492187500,
+0.0924682617187500, -0.0267639160156250, 0.0957946777343750, -0.0252990722656250,
+0.0919799804687500, -0.0242004394531250, 0.0924682617187500, -0.0263061523437500,
+0.0937194824218750, -0.0212097167968750, 0.0860900878906250, -0.0230407714843750,
+0.0884704589843750, -0.0205688476562500, 0.0845336914062500, -0.0160522460937500,
+0.0776977539062500, -0.0195312500000000, 0.0823974609375000, -0.0110168457031250,
+0.0708923339843750, -0.0128479003906250, 0.0727233886718750, -0.0103759765625000,
+0.0685119628906250, -0.0034484863281250, 0.0599365234375000, -0.0072021484375000,
+0.0652770996093750, 0.0003662109375000, 0.0501098632812500, -0.0001831054687500,
+0.0537414550781250, 0.0013427734375000, 0.0460205078125000, 0.0059814453125000,
+0.0331115722656250, 0.0035705566406250, 0.0422363281250000, 0.0104370117187500,
+0.0165100097656250, 0.0096740722656250, 0.0204772949218750, 0.0111999511718750,
+0.0143127441406250, 0.0163574218750000, -0.0049743652343750, 0.0143127441406250,
+0.0048522949218750, 0.0177612304687500, -0.0104370117187500, 0.0187377929687500,
+-0.0117797851562500, 0.0183410644531250, -0.0121765136718750, 0.0196228027343750,
+-0.0181579589843750, 0.0199584960937500, -0.0153808593750000, 0.0190429687500000,
+-0.0263671875000000, 0.0199584960937500, -0.0220031738281250, 0.0185241699218750,
+-0.0300292968750000, 0.0171203613281250, -0.0429382324218750, 0.0184326171875000,
+-0.0341186523437500, 0.0126037597656250, -0.0530700683593750, 0.0141601562500000,
+-0.0538024902343750, 0.0113830566406250, -0.0519104003906250, 0.0057373046875000,
+-0.0608520507812500, 0.0088195800781250, -0.0586547851562500, 0.0025024414062500,
+-0.0518798828125000, 0.0018920898437500, -0.0578002929687500, 0.0017089843750000,
+-0.0485839843750000, -0.0009155273437500, -0.0378112792968750, 0.0001525878906250,
+-0.0469665527343750, -0.0021362304687500, -0.0216064453125000, -0.0016784667968750,
+-0.0253601074218750, -0.0013122558593750, -0.0194396972656250, -0.0026855468750000,
+-0.0009155273437500, -0.0018615722656250, -0.0098571777343750, -0.0007324218750000,
+0.0082397460937500, -0.0008850097656250, 0.0075683593750000, 0.0007934570312500,
+0.0105895996093750, 0.0021057128906250, 0.0225830078125000, 0.0014953613281250,
+0.0169982910156250, 0.0068664550781250, 0.0238647460937500, 0.0072937011718750,
+0.0263061523437500, 0.0050354003906250, 0.0238037109375000, 0.0072937011718750,
+0.0247802734375000, 0.0086364746093750, 0.0258483886718750, -0.0010986328125000,
+0.0207519531250000, 0.0011291503906250, 0.0224304199218750, -0.0027160644531250,
+0.0202026367187500, -0.0094909667968750, 0.0147705078125000, -0.0056457519531250,
+0.0180664062500000, -0.0245971679687500, 0.0166015625000000, -0.0205993652343750,
+0.0134887695312500, -0.0245666503906250, 0.0179443359375000, -0.0408935546875000,
+0.0222167968750000, -0.0350341796875000, 0.0186462402343750, -0.0379943847656250,
+0.0246582031250000, -0.0411071777343750, 0.0266418457031250, -0.0375061035156250,
+0.0231018066406250, -0.0342102050781250, 0.0243225097656250, -0.0361328125000000,
+0.0258789062500000, -0.0346679687500000, 0.0098266601562500, -0.0317993164062500,
+0.0162353515625000, -0.0334777832031250, 0.0069885253906250, -0.0354919433593750,
+-0.0108642578125000, -0.0341796875000000, -0.0003662109375000, -0.0400390625000000,
+-0.0191955566406250, -0.0386352539062500, -0.0222778320312500, -0.0379638671875000,
+-0.0207519531250000, -0.0438842773437500, -0.0278930664062500, -0.0437011718750000,
+-0.0269165039062500, -0.0321655273437500, -0.0252075195312500, -0.0352172851562500,
+-0.0265197753906250, -0.0334167480468750, -0.0256347656250000, -0.0241394042968750,
+-0.0226135253906250, -0.0248718261718750, -0.0239868164062500, -0.0173034667968750,
+-0.0273742675781250, -0.0197448730468750, -0.0239562988281250, -0.0202636718750000,
+-0.0280151367187500, -0.0148620605468750, -0.0343627929687500, -0.0161743164062500,
+-0.0302429199218750, -0.0166320800781250, -0.0379028320312500, -0.0178527832031250,
+-0.0404052734375000, -0.0157470703125000, -0.0358886718750000, -0.0128173828125000,
+-0.0364379882812500, -0.0160827636718750, -0.0390930175781250, -0.0188903808593750,
+-0.0259704589843750, -0.0136413574218750, -0.0259704589843750, -0.0205688476562500,
+-0.0264587402343750, -0.0315551757812500, -0.0208129882812500, -0.0256042480468750,
+-0.0216064453125000, -0.0297241210937500, -0.0174255371093750, -0.0352478027343750,
+-0.0144042968750000, -0.0293273925781250, -0.0239868164062500, -0.0248718261718750,
+-0.0276489257812500, -0.0294189453125000, -0.0196533203125000, -0.0156555175781250,
+-0.0458374023437500, -0.0193481445312500, -0.0473327636718750, -0.0122070312500000,
+-0.0410766601562500, 0.0003051757812500, -0.0519409179687500, -0.0083618164062500,
+-0.0533447265625000, 0.0122985839843750, -0.0303649902343750, 0.0117797851562500,
+-0.0362854003906250, 0.0125732421875000, -0.0314636230468750, 0.0254211425781250,
+-0.0122985839843750, 0.0215759277343750, -0.0196533203125000, 0.0188598632812500,
+-0.0141906738281250, 0.0227355957031250, -0.0086669921875000, 0.0198059082031250,
+-0.0180664062500000, 0.0130310058593750, -0.0243530273437500, 0.0163879394531250,
+-0.0165100097656250, 0.0193176269531250, -0.0298767089843750, 0.0156860351562500,
+-0.0349121093750000, 0.0191040039062500, -0.0245056152343750, 0.0274353027343750,
+-0.0221252441406250, 0.0241394042968750, -0.0275268554687500, 0.0175781250000000,
+-0.0036315917968750, 0.0262145996093750, -0.0059509277343750, 0.0149841308593750,
+-0.0001220703125000, -0.0003356933593750, 0.0174560546875000, 0.0111083984375000,
+0.0106506347656250, -0.0103759765625000, 0.0217285156250000, -0.0116271972656250,
+0.0236206054687500, -0.0097351074218750, 0.0242919921875000, -0.0180053710937500,
+0.0290832519531250, -0.0158996582031250, 0.0273742675781250, -0.0201416015625000,
+0.0322875976562500, -0.0174865722656250, 0.0317993164062500, -0.0223388671875000,
+0.0346984863281250, -0.0282287597656250, 0.0388183593750000, -0.0223388671875000,
+0.0362548828125000, -0.0367126464843750, 0.0453796386718750, -0.0354614257812500,
+0.0444030761718750, -0.0363769531250000, 0.0457458496093750, -0.0451049804687500,
+0.0518798828125000, -0.0415039062500000, 0.0492248535156250, -0.0451354980468750,
+0.0519714355468750, -0.0465087890625000, 0.0524291992187500, -0.0449218750000000,
+0.0505065917968750, -0.0459289550781250, 0.0506896972656250, -0.0465393066406250,
+0.0503540039062500, -0.0397949218750000, 0.0445861816406250, -0.0435791015625000,
+0.0462341308593750, -0.0360107421875000, 0.0412292480468750, -0.0267333984375000,
+0.0343627929687500, -0.0335388183593750, 0.0374755859375000, -0.0109252929687500,
+0.0247497558593750, -0.0146789550781250, 0.0255432128906250, -0.0072937011718750,
+0.0227661132812500, 0.0096435546875000, 0.0140380859375000, -0.0000915527343750,
+0.0171508789062500, 0.0235290527343750, 0.0095520019531250, 0.0199279785156250,
+0.0096740722656250, 0.0267639160156250, 0.0078430175781250, 0.0442504882812500,
+0.0032043457031250, 0.0340576171875000, 0.0060729980468750, 0.0554809570312500,
+0.0004882812500000, 0.0540161132812500, 0.0014953613281250, 0.0568847656250000,
+0.0006713867187500, 0.0699157714843750, -0.0028991699218750, 0.0637512207031250,
+-0.0003662109375000, 0.0762023925781250, -0.0044555664062500, 0.0736694335937500,
+-0.0024414062500000, 0.0790405273437500, -0.0037231445312500, 0.0891418457031250,
+-0.0077209472656250, 0.0820312500000000, -0.0045471191406250, 0.1015625000000000,
+-0.0083312988281250, 0.1006164550781250, -0.0087585449218750, 0.1008911132812500,
+-0.0074157714843750, 0.1141357421875000, -0.0083007812500000, 0.1103515625000000,
+-0.0082702636718750, 0.0998840332031250, -0.0058898925781250, 0.1100463867187500,
+-0.0063781738281250, 0.0914916992187500, -0.0040893554687500, 0.0706787109375000,
+-0.0018310546875000, 0.0862426757812500, -0.0034790039062500, 0.0374145507812500,
+0.0046691894531250, 0.0419006347656250, 0.0028076171875000, 0.0314941406250000,
+0.0057067871093750, -0.0025634765625000, 0.0131225585937500, 0.0120239257812500,
+0.0097045898437500, -0.0143127441406250, 0.0144042968750000, -0.0166931152343750,
+0.0157165527343750, -0.0188903808593750, 0.0156250000000000, -0.0320129394531250,
+0.0169067382812500, -0.0273132324218750, 0.0170898437500000, -0.0400390625000000,
+0.0210876464843750, -0.0400085449218750, 0.0197753906250000, -0.0437316894531250,
+0.0212097167968750, -0.0530090332031250, 0.0250549316406250, -0.0490417480468750,
+0.0228881835937500, -0.0563354492187500, 0.0239257812500000, -0.0572814941406250,
+0.0261535644531250, -0.0570983886718750, 0.0231933593750000, -0.0605468750000000,
+0.0210876464843750, -0.0592651367187500, 0.0234680175781250, -0.0570373535156250,
+0.0154724121093750, -0.0612487792968750, 0.0176391601562500, -0.0513916015625000,
+0.0111083984375000, -0.0436401367187500, 0.0044860839843750, -0.0513916015625000,
+0.0106201171875000, -0.0206909179687500, -0.0176696777343750, -0.0248718261718750,
+-0.0114440917968750, -0.0194396972656250, -0.0194091796875000, 0.0039367675781250,
+-0.0451049804687500, -0.0047912597656250, -0.0331420898437500, -0.0004882812500000,
+-0.0367126464843750, 0.0050048828125000, -0.0462646484375000, 0.0008850097656250,
+-0.0339965820312500, -0.0040588378906250, -0.0198669433593750, -0.0009155273437500,
+-0.0289306640625000, 0.0009765625000000, -0.0208129882812500, -0.0036010742187500,
+-0.0103759765625000, 0.0030822753906250, -0.0261535644531250, 0.0116271972656250,
+-0.0386047363281250},
+{0.0256958007812500, -0.0319824218750000, 0.0235595703125000, -0.0244750976562500,
+0.0234680175781250, -0.0382080078125000, 0.0233459472656250, -0.0392456054687500,
+0.0226440429687500, -0.0368652343750000, 0.0217895507812500, -0.0420227050781250,
+0.0215454101562500, -0.0412902832031250, 0.0211181640625000, -0.0368957519531250,
+0.0187072753906250, -0.0383300781250000, 0.0185241699218750, -0.0357360839843750,
+0.0171813964843750, -0.0316467285156250, 0.0148925781250000, -0.0336914062500000,
+0.0148315429687500, -0.0253295898437500, 0.0125732421875000, -0.0282897949218750,
+0.0121765136718750, -0.0218505859375000, 0.0102844238281250, -0.0130310058593750,
+0.0079345703125000, -0.0197753906250000, 0.0082702636718750, -0.0009765625000000,
+0.0036315917968750, -0.0037841796875000, 0.0025024414062500, -0.0002441406250000,
+0.0017089843750000, 0.0121154785156250, -0.0010986328125000, 0.0051879882812500,
+-0.0017395019531250, 0.0191650390625000, -0.0015869140625000, 0.0158691406250000,
+-0.0034484863281250, 0.0227966308593750, -0.0042114257812500, 0.0343322753906250,
+-0.0034484863281250, 0.0259704589843750, -0.0044555664062500, 0.0527038574218750,
+-0.0059814453125000, 0.0479125976562500, -0.0065917968750000, 0.0556640625000000,
+-0.0069885253906250, 0.0772705078125000, -0.0076599121093750, 0.0656127929687500,
+-0.0079345703125000, 0.0807495117187500, -0.0069274902343750, 0.0862426757812500,
+-0.0074462890625000, 0.0767822265625000, -0.0064392089843750, 0.0771179199218750,
+-0.0054931640625000, 0.0814819335937500, -0.0061340332031250, 0.0556030273437500,
+-0.0006408691406250, 0.0610046386718750, -0.0029296875000000, 0.0500488281250000,
+0.0007629394531250, 0.0277404785156250, 0.0064697265625000, 0.0391845703125000,
+0.0020751953125000, 0.0113220214843750, 0.0121459960937500, 0.0130310058593750,
+0.0101318359375000, 0.0052795410156250, 0.0136413574218750, -0.0133056640625000,
+0.0209350585937500, -0.0038146972656250, 0.0153808593750000, -0.0319213867187500,
+0.0284729003906250, -0.0286254882812500, 0.0271301269531250, -0.0362854003906250,
+0.0271911621093750, -0.0583496093750000, 0.0356750488281250, -0.0485534667968750,
+0.0326843261718750, -0.0638732910156250, 0.0302429199218750, -0.0680236816406250,
+0.0343017578125000, -0.0642089843750000, 0.0296630859375000, -0.0675048828125000,
+0.0236511230468750, -0.0681152343750000, 0.0283508300781250, -0.0603942871093750,
+0.0181884765625000, -0.0636596679687500, 0.0185852050781250, -0.0559692382812500,
+0.0173339843750000, -0.0473022460937500, 0.0114440917968750, -0.0535583496093750,
+0.0140075683593750, -0.0267639160156250, 0.0075378417968750, -0.0307617187500000,
+0.0081787109375000, -0.0237731933593750, 0.0080871582031250, -0.0021057128906250,
+0.0038146972656250, -0.0114746093750000, 0.0046997070312500, -0.0024719238281250,
+0.0049743652343750, 0.0028076171875000, 0.0062561035156250, -0.0025634765625000,
+-0.0006408691406250, -0.0053405761718750, -0.0036621093750000, -0.0019226074218750,
+0.0017700195312500, -0.0045776367187500, -0.0222473144531250, -0.0081481933593750,
+-0.0190124511718750, -0.0018920898437500, -0.0220947265625000, 0.0028991699218750,
+-0.0406494140625000, -0.0015869140625000, -0.0348815917968750, 0.0108642578125000,
+-0.0304260253906250, 0.0109252929687500, -0.0346374511718750, 0.0117492675781250,
+-0.0343627929687500, 0.0184020996093750, -0.0260620117187500, 0.0161437988281250,
+-0.0259704589843750, 0.0195312500000000, -0.0418090820312500, 0.0200195312500000,
+-0.0376586914062500, 0.0192565917968750, -0.0421142578125000, 0.0212097167968750,
+-0.0544433593750000, 0.0206909179687500, -0.0497741699218750, 0.0148925781250000,
+-0.0669555664062500, 0.0184936523437500, -0.0640258789062500, 0.0110473632812500,
+-0.0667114257812500, 0.0019836425781250, -0.0842285156250000, 0.0081787109375000,
+-0.0786132812500000, -0.0107421875000000, -0.0580749511718750, -0.0094299316406250,
+-0.0724182128906250, -0.0140075683593750, -0.0552673339843750, -0.0268249511718750,
+-0.0235595703125000, -0.0211791992187500, -0.0390625000000000, -0.0348815917968750,
+-0.0101318359375000, -0.0334777832031250, -0.0095520019531250, -0.0389099121093750,
+-0.0043640136718750, -0.0494079589843750, 0.0131835937500000, -0.0432128906250000,
+0.0040283203125000, -0.0599975585937500, 0.0185546875000000, -0.0596618652343750,
+0.0227966308593750, -0.0599365234375000, 0.0183105468750000, -0.0706787109375000,
+0.0177917480468750, -0.0678100585937500, 0.0199279785156250, -0.0612182617187500,
+0.0285034179687500, -0.0691833496093750, 0.0236511230468750, -0.0569152832031250,
+0.0265502929687500, -0.0420227050781250, 0.0365600585937500, -0.0528869628906250,
+0.0338745117187500, -0.0240478515625000, 0.0409851074218750, -0.0248413085937500,
+0.0421752929687500, -0.0213317871093750, 0.0352478027343750, -0.0059814453125000,
+0.0379943847656250, -0.0122070312500000, 0.0426330566406250, 0.0097351074218750,
+0.0121459960937500, 0.0057067871093750, 0.0195007324218750, 0.0106811523437500,
+0.0107727050781250, 0.0398864746093750, -0.0320129394531250, 0.0200805664062500,
+0.0315856933593750, 0.0125427246093750, 0.0014648437500000, 0.0070190429687500,
+0.0220642089843750, 0.0177612304687500, 0.0007934570312500, 0.0261535644531250,
+-0.0331420898437500, 0.0182189941406250, -0.0158996582031250, 0.0323486328125000,
+-0.0224914550781250, 0.0381469726562500, -0.0389709472656250, 0.0296020507812500,
+-0.0173950195312500, 0.0281982421875000, 0.0072937011718750, 0.0337524414062500,
+-0.0093383789062500, 0.0158081054687500, 0.0067138671875000, 0.0155334472656250,
+0.0206604003906250, 0.0182800292968750, 0.0022888183593750, 0.0097045898437500,
+-0.0116271972656250, 0.0094909667968750, 0.0013122558593750, 0.0191650390625000,
+-0.0222167968750000, 0.0186462402343750, -0.0285644531250000, 0.0147094726562500,
+-0.0209045410156250, 0.0203552246093750, -0.0225524902343750, 0.0210266113281250,
+-0.0260620117187500, -0.0038146972656250, -0.0206604003906250, 0.0113220214843750,
+-0.0158081054687500, -0.0171508789062500, -0.0252075195312500, -0.0539245605468750,
+-0.0324096679687500, -0.0261535644531250, -0.0255126953125000, -0.0949096679687500,
+-0.0394897460937500, -0.0936584472656250, -0.0454711914062500, -0.0949707031250000,
+-0.0346679687500000, -0.1334228515625000, -0.0324096679687500, -0.1212768554687500,
+-0.0398254394531250, -0.1236572265625000, -0.0140380859375000, -0.1337890625000000,
+-0.0146789550781250, -0.1169738769531250, -0.0148315429687500, -0.1060180664062500,
+-0.0015869140625000, -0.1188354492187500, -0.0047912597656250, -0.0702514648437500,
+-0.0035095214843750, -0.0772399902343750, -0.0043029785156250, -0.0690917968750000,
+-0.0006103515625000, -0.0340270996093750, 0.0025329589843750, -0.0482177734375000,
+-0.0010986328125000, -0.0325927734375000, 0.0105590820312500, -0.0303344726562500,
+0.0110778808593750, -0.0278930664062500, 0.0092468261718750, -0.0199584960937500,
+0.0144348144531250, -0.0258483886718750, 0.0145874023437500, -0.0171813964843750,
+0.0083007812500000, -0.0150451660156250, 0.0094299316406250, -0.0169982910156250,
+0.0091552734375000, -0.0145568847656250, 0.0050964355468750, -0.0141296386718750,
+0.0061645507812500, -0.0196533203125000, 0.0066833496093750, -0.0174255371093750,
+0.0072021484375000, -0.0179748535156250, 0.0056762695312500, -0.0237426757812500,
+0.0050964355468750, -0.0217285156250000, 0.0065002441406250, -0.0169982910156250,
+0.0000610351562500, -0.0216674804687500, 0.0018310546875000, -0.0121459960937500,
+-0.0016784667968750, -0.0031738281250000, -0.0070800781250000, -0.0115051269531250,
+-0.0036926269531250, 0.0174255371093750, -0.0177612304687500, 0.0140075683593750,
+-0.0138244628906250, 0.0182800292968750, -0.0212097167968750, 0.0380859375000000,
+-0.0349731445312500, 0.0298156738281250, -0.0268249511718750, 0.0387268066406250,
+-0.0450744628906250, 0.0414123535156250, -0.0449829101562500, 0.0387268066406250,
+-0.0467529296875000, 0.0407714843750000, -0.0572204589843750, 0.0413818359375000,
+-0.0527648925781250, 0.0360717773437500, -0.0581665039062500, 0.0392150878906250,
+-0.0610046386718750, 0.0346069335937500, -0.0549621582031250, 0.0287170410156250,
+-0.0542907714843750, 0.0343322753906250, -0.0577697753906250, 0.0189208984375000,
+-0.0361938476562500, 0.0230407714843750, -0.0413513183593750, 0.0175781250000000,
+-0.0322570800781250, 0.0054931640625000, -0.0126647949218750, 0.0142211914062500,
+-0.0226745605468750, -0.0049438476562500, -0.0021972656250000, -0.0017395019531250,
+-0.0031127929687500, -0.0056152343750000, 0.0018310546875000, -0.0191955566406250,
+0.0149536132812500, -0.0117797851562500, 0.0078430175781250, -0.0258483886718750,
+0.0267333984375000, -0.0249633789062500, 0.0252380371093750, -0.0266418457031250,
+0.0302124023437500, -0.0352478027343750, 0.0438537597656250, -0.0308532714843750,
+0.0382385253906250, -0.0376281738281250, 0.0531616210937500, -0.0378723144531250,
+0.0528869628906250, -0.0376586914062500, 0.0556640625000000, -0.0407409667968750,
+0.0663452148437500, -0.0393371582031250, 0.0622253417968750, -0.0410461425781250,
+0.0642700195312500, -0.0412597656250000, 0.0694274902343750, -0.0408020019531250,
+0.0608825683593750, -0.0415954589843750, 0.0552673339843750, -0.0416259765625000,
+0.0617980957031250, -0.0398864746093750, 0.0368652343750000, -0.0405273437500000,
+0.0417480468750000, -0.0390014648437500, 0.0332641601562500, -0.0368652343750000,
+0.0126647949218750, -0.0373229980468750, 0.0234680175781250, -0.0320129394531250,
+0.0034790039062500, -0.0334777832031250, 0.0032653808593750, -0.0284118652343750,
+0.0011596679687500, -0.0222167968750000, -0.0090332031250000, -0.0257263183593750,
+-0.0037231445312500, -0.0104064941406250, -0.0172119140625000, -0.0123901367187500,
+-0.0141601562500000, -0.0079040527343750, -0.0202941894531250, 0.0042724609375000,
+-0.0323181152343750, -0.0001831054687500, -0.0245971679687500, 0.0107116699218750,
+-0.0419006347656250, 0.0119628906250000, -0.0418395996093750, 0.0122985839843750,
+-0.0418395996093750, 0.0179443359375000, -0.0516357421875000, 0.0170288085937500,
+-0.0484008789062500, 0.0170593261718750, -0.0454101562500000, 0.0187072753906250,
+-0.0493164062500000, 0.0165710449218750, -0.0439758300781250, 0.0143432617187500,
+-0.0374145507812500, 0.0159912109375000, -0.0420837402343750, 0.0099792480468750,
+-0.0305480957031250, 0.0102844238281250, -0.0334167480468750, 0.0095825195312500,
+-0.0273742675781250, 0.0051269531250000, -0.0177612304687500, 0.0062561035156250,
+-0.0251770019531250, 0.0074462890625000, -0.0036010742187500, 0.0047302246093750,
+-0.0071716308593750, 0.0090942382812500, -0.0013427734375000, 0.0135803222656250,
+0.0155639648437500, 0.0100402832031250, 0.0069580078125000, 0.0215454101562500,
+0.0210266113281250, 0.0208129882812500, 0.0239257812500000, 0.0212402343750000,
+0.0195007324218750, 0.0294799804687500, 0.0234069824218750, 0.0267944335937500,
+0.0253601074218750, 0.0195007324218750, 0.0155639648437500, 0.0255737304687500,
+0.0166625976562500, 0.0153198242187500, 0.0179748535156250, 0.0013427734375000,
+0.0124816894531250, 0.0095214843750000, 0.0128479003906250, -0.0117492675781250,
+0.0173950195312500, -0.0136413574218750, 0.0165710449218750, -0.0115661621093750,
+0.0159912109375000, -0.0218505859375000, 0.0186767578125000, -0.0204467773437500,
+0.0186767578125000, -0.0069274902343750, 0.0168762207031250, -0.0123291015625000,
+0.0177307128906250, -0.0063476562500000, 0.0155639648437500, 0.0097961425781250,
+0.0137329101562500, 0.0041503906250000, 0.0154418945312500, 0.0066833496093750,
+0.0061340332031250},
+{-0.0537414550781250, 0.0000000000000000, -0.0418090820312500, -0.0028076171875000,
+-0.0481872558593750, 0.0004577636718750, -0.0365905761718750, 0.0056762695312500,
+-0.0208129882812500, 0.0005798339843750, -0.0321350097656250, 0.0119323730468750,
+-0.0015869140625000, 0.0085754394531250, -0.0028991699218750, 0.0109252929687500,
+0.0008544921875000, 0.0197753906250000, 0.0191650390625000, 0.0139465332031250,
+0.0118103027343750, 0.0219421386718750, 0.0242614746093750, 0.0209960937500000,
+0.0236511230468750, 0.0203247070312500, 0.0273437500000000, 0.0239868164062500,
+0.0358581542968750, 0.0204467773437500, 0.0307617187500000, 0.0205383300781250,
+0.0439758300781250, 0.0204467773437500, 0.0433349609375000, 0.0178222656250000,
+0.0444641113281250, 0.0158691406250000, 0.0528869628906250, 0.0155639648437500,
+0.0496826171875000, 0.0122070312500000, 0.0510559082031250, 0.0116271972656250,
+0.0541992187500000, 0.0098571777343750, 0.0493774414062500, 0.0074462890625000,
+0.0463562011718750, 0.0075988769531250, 0.0500183105468750, 0.0025634765625000,
+0.0357055664062500, 0.0025634765625000, 0.0395202636718750, 0.0001525878906250,
+0.0317382812500000, -0.0044555664062500, 0.0183410644531250, -0.0040283203125000,
+0.0260314941406250, -0.0092468261718750, 0.0027160644531250, -0.0095825195312500,
+0.0058898925781250, -0.0118713378906250, -0.0019226074218750, -0.0160217285156250,
+-0.0197448730468750, -0.0149841308593750, -0.0099487304687500, -0.0192871093750000,
+-0.0325622558593750, -0.0198059082031250, -0.0318298339843750, -0.0196838378906250,
+-0.0338134765625000, -0.0220642089843750, -0.0481872558593750, -0.0221252441406250,
+-0.0424194335937500, -0.0202026367187500, -0.0430908203125000, -0.0219726562500000,
+-0.0485229492187500, -0.0190429687500000, -0.0406799316406250, -0.0161437988281250,
+-0.0328063964843750, -0.0192565917968750, -0.0390625000000000, -0.0092773437500000,
+-0.0257873535156250, -0.0112304687500000, -0.0254211425781250, -0.0075683593750000,
+-0.0248413085937500, 0.0009765625000000, -0.0196533203125000, -0.0032043457031250,
+-0.0224609375000000, 0.0061340332031250, -0.0118713378906250, 0.0065002441406250,
+-0.0169677734375000, 0.0069274902343750, -0.0089721679687500, 0.0118103027343750,
+0.0042724609375000, 0.0101928710937500, -0.0046997070312500, 0.0115356445312500,
+0.0148925781250000, 0.0119323730468750, 0.0154113769531250, 0.0121459960937500,
+0.0142822265625000, 0.0123596191406250, 0.0245361328125000, 0.0118103027343750,
+0.0224609375000000, 0.0137634277343750, 0.0170898437500000, 0.0130310058593750,
+0.0212707519531250, 0.0144653320312500, 0.0188293457031250, 0.0166320800781250,
+0.0108032226562500, 0.0150756835937500, 0.0141906738281250, 0.0200195312500000,
+0.0165710449218750, 0.0204772949218750, 0.0128784179687500, 0.0190429687500000,
+0.0149230957031250, 0.0216369628906250, 0.0208740234375000, 0.0219116210937500,
+0.0185546875000000, 0.0127563476562500, 0.0157470703125000, 0.0168762207031250,
+0.0187377929687500, 0.0090332031250000, 0.0155029296875000, -0.0024108886718750,
+0.0106201171875000, 0.0049743652343750, 0.0132751464843750, -0.0140991210937500,
+0.0045471191406250, -0.0155639648437500, 0.0071105957031250, -0.0114440917968750,
+0.0014038085937500, -0.0190734863281250, -0.0078125000000000, -0.0196228027343750,
+-0.0019226074218750, -0.0064392089843750, -0.0165100097656250, -0.0096130371093750,
+-0.0160217285156250, -0.0054321289062500, -0.0184020996093750, 0.0068054199218750,
+-0.0283508300781250, 0.0005798339843750, -0.0245056152343750, -0.0000610351562500,
+-0.0275878906250000, 0.0087280273437500, -0.0306701660156250, -0.0044250488281250,
+-0.0289001464843750, -0.0196228027343750, -0.0269775390625000, -0.0095520019531250,
+-0.0282592773437500, -0.0295715332031250, -0.0286560058593750, -0.0329589843750000,
+-0.0289611816406250, -0.0282287597656250, -0.0264282226562500, -0.0350952148437500,
+-0.0262145996093750, -0.0358581542968750, -0.0291748046875000, -0.0183715820312500,
+-0.0197448730468750, -0.0206604003906250, -0.0211791992187500, -0.0204772949218750,
+-0.0194396972656250, -0.0077209472656250, -0.0130615234375000, -0.0095214843750000,
+-0.0163879394531250, -0.0204162597656250, -0.0114746093750000, -0.0144042968750000,
+-0.0111694335937500, -0.0185241699218750, -0.0103454589843750, -0.0338134765625000,
+-0.0057983398437500, -0.0303955078125000, -0.0072021484375000, -0.0252990722656250,
+-0.0128173828125000, -0.0283508300781250, -0.0078125000000000, -0.0268554687500000,
+-0.0109252929687500, -0.0218811035156250, -0.0193481445312500, -0.0236511230468750,
+-0.0149230957031250, -0.0198059082031250, -0.0192260742187500, -0.0192260742187500,
+-0.0210266113281250, -0.0200195312500000, -0.0181274414062500, -0.0179443359375000,
+-0.0162353515625000, -0.0176696777343750, -0.0177307128906250, -0.0231933593750000,
+-0.0208740234375000, -0.0206298828125000, -0.0159301757812500, -0.0231323242187500,
+-0.0203552246093750, -0.0296630859375000, -0.0289611816406250, -0.0263061523437500,
+-0.0250549316406250, -0.0278015136718750, -0.0313415527343750, -0.0306091308593750,
+-0.0315856933593750, -0.0260925292968750, -0.0310363769531250, -0.0227661132812500,
+-0.0363159179687500, -0.0260314941406250, -0.0354614257812500, -0.0159606933593750,
+-0.0224609375000000, -0.0162353515625000, -0.0289611816406250, -0.0175170898437500,
+-0.0187988281250000, -0.0116577148437500, -0.0009155273437500, -0.0127868652343750,
+-0.0110168457031250, -0.0224914550781250, 0.0033569335937500, -0.0182189941406250,
+0.0077819824218750, -0.0238342285156250, 0.0061950683593750, -0.0367126464843750,
+0.0084533691406250, -0.0310363769531250, 0.0080261230468750, -0.0325317382812500,
+0.0131835937500000, -0.0392150878906250, 0.0128784179687500, -0.0303344726562500,
+0.0148620605468750, -0.0224914550781250, 0.0181884765625000, -0.0296936035156250,
+0.0166015625000000, -0.0077209472656250, 0.0260009765625000, -0.0116577148437500,
+0.0250854492187500, -0.0047607421875000, 0.0242614746093750, 0.0128479003906250,
+0.0306396484375000, 0.0028076171875000, 0.0301208496093750, 0.0187072753906250,
+0.0219421386718750, 0.0205078125000000, 0.0253601074218750, 0.0178222656250000,
+0.0209960937500000, 0.0233154296875000, 0.0114135742187500, 0.0229187011718750,
+0.0163269042968750, 0.0217590332031250, 0.0120544433593750, 0.0214538574218750,
+0.0096130371093750, 0.0228271484375000, 0.0129394531250000, 0.0236511230468750,
+0.0154113769531250, 0.0222778320312500, 0.0140686035156250, 0.0234069824218750,
+0.0146789550781250, 0.0255432128906250, 0.0173950195312500, 0.0210266113281250,
+0.0125427246093750, 0.0186462402343750, 0.0085449218750000, 0.0229187011718750,
+0.0117187500000000, 0.0106201171875000, -0.0024719238281250, 0.0128784179687500,
+-0.0005798339843750, 0.0107421875000000, -0.0052490234375000, 0.0022888183593750,
+-0.0168762207031250, 0.0066528320312500, -0.0118408203125000, -0.0009765625000000,
+-0.0205383300781250, 0.0000305175781250, -0.0223999023437500, -0.0024414062500000,
+-0.0205688476562500, -0.0077819824218750, -0.0231323242187500, -0.0040588378906250,
+-0.0228271484375000, -0.0144042968750000, -0.0171813964843750, -0.0120544433593750,
+-0.0190429687500000, -0.0157165527343750, -0.0148315429687500, -0.0244140625000000,
+-0.0084838867187500, -0.0191955566406250, -0.0112915039062500, -0.0294494628906250,
+0.0004272460937500, -0.0305480957031250, -0.0007629394531250, -0.0285949707031250,
+0.0034790039062500, -0.0329895019531250, 0.0128784179687500, -0.0322265625000000,
+0.0087890625000000, -0.0232238769531250, 0.0186157226562500, -0.0271911621093750,
+0.0189514160156250, -0.0186462402343750, 0.0200500488281250, -0.0067443847656250,
+0.0252075195312500, -0.0131835937500000, 0.0227050781250000, 0.0075683593750000,
+0.0275573730468750, 0.0061035156250000, 0.0268554687500000, 0.0101318359375000,
+0.0280456542968750, 0.0244750976562500, 0.0311279296875000, 0.0180664062500000,
+0.0286865234375000, 0.0252990722656250, 0.0338439941406250, 0.0277099609375000,
+0.0319824218750000, 0.0239257812500000, 0.0342102050781250, 0.0239257812500000,
+0.0385131835937500, 0.0253906250000000, 0.0346374511718750, 0.0194702148437500,
+0.0433044433593750, 0.0190124511718750, 0.0421142578125000, 0.0192260742187500,
+0.0428466796875000, 0.0170593261718750, 0.0489196777343750, 0.0169677734375000,
+0.0467224121093750, 0.0150146484375000, 0.0484924316406250, 0.0171813964843750,
+0.0507507324218750, 0.0126953125000000, 0.0483703613281250, 0.0083618164062500,
+0.0469055175781250, 0.0121459960937500, 0.0488891601562500, -0.0021362304687500,
+0.0422363281250000, -0.0000305175781250, 0.0440979003906250, -0.0044860839843750,
+0.0417175292968750, -0.0154418945312500, 0.0363159179687500, -0.0100708007812500,
+0.0395507812500000, -0.0222778320312500, 0.0312805175781250, -0.0206298828125000,
+0.0339050292968750, -0.0230407714843750, 0.0294494628906250, -0.0314636230468750,
+0.0220031738281250, -0.0263061523437500, 0.0277099609375000, -0.0333862304687500,
+0.0119628906250000, -0.0346679687500000, 0.0143432617187500, -0.0295715332031250,
+0.0108032226562500, -0.0286865234375000, -0.0006103515625000, -0.0301208496093750,
+0.0052490234375000, -0.0227050781250000, -0.0050964355468750, -0.0212097167968750,
+-0.0064392089843750, -0.0230407714843750, -0.0052490234375000, -0.0218505859375000,
+-0.0097961425781250, -0.0205078125000000, -0.0092468261718750, -0.0220336914062500,
+-0.0047912597656250, -0.0245971679687500, -0.0080871582031250, -0.0182495117187500,
+-0.0022888183593750, -0.0144958496093750, 0.0050354003906250, -0.0191345214843750,
+-0.0007629394531250, -0.0007629394531250, 0.0117492675781250, -0.0033264160156250,
+0.0127258300781250, 0.0011291503906250, 0.0103759765625000, 0.0148010253906250,
+0.0154418945312500, 0.0089416503906250, 0.0149841308593750, 0.0196533203125000,
+0.0078430175781250, 0.0200195312500000, 0.0115661621093750, 0.0206298828125000,
+0.0048522949218750, 0.0266723632812500, -0.0039062500000000, 0.0242004394531250,
+0.0020751953125000, 0.0249633789062500, -0.0181579589843750, 0.0276489257812500,
+-0.0153198242187500, 0.0223388671875000, -0.0211486816406250, 0.0184326171875000,
+-0.0392456054687500, 0.0221252441406250, -0.0312194824218750, 0.0085449218750000,
+-0.0304260253906250, 0.0111083984375000, -0.0377807617187500, 0.0059814453125000,
+-0.0316162109375000, -0.0050048828125000, -0.0188903808593750, 0.0004882812500000,
+-0.0236816406250000, -0.0155944824218750, -0.0325927734375000, -0.0142211914062500,
+-0.0245056152343750, -0.0186462402343750, -0.0324401855468750, -0.0303955078125000,
+-0.0481262207031250, -0.0247192382812500, -0.0411071777343750, -0.0355529785156250,
+-0.0530700683593750, -0.0366516113281250, -0.0539245605468750, -0.0360412597656250,
+-0.0531921386718750, -0.0409240722656250, -0.0617980957031250, -0.0398254394531250,
+-0.0596923828125000, -0.0392150878906250, -0.0409851074218750, -0.0414733886718750,
+-0.0500488281250000, -0.0382080078125000, -0.0392761230468750, -0.0359802246093750,
+-0.0156250000000000, -0.0388183593750000, -0.0269165039062500, -0.0248413085937500,
+-0.0150451660156250, -0.0304565429687500, -0.0099792480468750, -0.0195007324218750,
+-0.0120544433593750, -0.0031738281250000, -0.0119323730468750, -0.0143737792968750,
+-0.0126953125000000, 0.0138244628906250, -0.0090026855468750, 0.0147705078125000,
+-0.0099182128906250, 0.0130615234375000, -0.0037841796875000, 0.0266723632812500,
+-0.0016174316406250, 0.0238647460937500, -0.0069580078125000, 0.0196228027343750,
+0.0248413085937500, 0.0223083496093750, 0.0215454101562500, 0.0185852050781250,
+0.0226745605468750},
+{-0.0027160644531250, 0.0175476074218750, -0.0015869140625000, 0.0175476074218750,
+0.0029602050781250, 0.0179443359375000, -0.0005187988281250, 0.0114135742187500,
+0.0055236816406250, 0.0127258300781250, 0.0043945312500000, 0.0109558105468750,
+0.0047607421875000, 0.0055236816406250, 0.0091247558593750, 0.0076293945312500,
+0.0071716308593750, 0.0051269531250000, 0.0087280273437500, 0.0045776367187500,
+0.0097045898437500, 0.0041809082031250, 0.0083923339843750, 0.0033264160156250,
+0.0074768066406250, 0.0045471191406250, 0.0083618164062500, 0.0031433105468750,
+0.0081787109375000, 0.0024414062500000, 0.0063781738281250, 0.0066223144531250,
+0.0093383789062500, 0.0093078613281250, 0.0113220214843750, 0.0075683593750000,
+0.0082092285156250, 0.0151672363281250, 0.0170898437500000, 0.0181884765625000,
+0.0146789550781250, 0.0146789550781250, 0.0163879394531250, 0.0165100097656250,
+0.0234680175781250, 0.0200195312500000, 0.0195007324218750, 0.0035705566406250,
+0.0233459472656250, 0.0069885253906250, 0.0264587402343750, 0.0034790039062500,
+0.0215454101562500, -0.0100402832031250, 0.0196533203125000, -0.0040588378906250,
+0.0234069824218750, -0.0068664550781250, 0.0097045898437500, -0.0103454589843750,
+0.0130004882812500, -0.0053405761718750, 0.0079956054687500, -0.0021362304687500,
+-0.0043640136718750, -0.0050354003906250, 0.0023498535156250, 0.0063476562500000,
+-0.0070495605468750, 0.0043029785156250, -0.0096435546875000, 0.0084533691406250,
+-0.0065917968750000, 0.0183105468750000, -0.0082397460937500, 0.0132446289062500,
+-0.0092163085937500, 0.0211486816406250, -0.0046081542968750, 0.0233154296875000,
+-0.0059814453125000, 0.0197448730468750, -0.0047302246093750, 0.0211791992187500,
+-0.0004577636718750, 0.0228271484375000, -0.0020446777343750, 0.0129394531250000,
+0.0005493164062500, 0.0155639648437500, 0.0021972656250000, 0.0103149414062500,
+-0.0011901855468750, 0.0011901855468750, -0.0009765625000000, 0.0063781738281250,
+0.0012817382812500, -0.0083618164062500, -0.0163269042968750, -0.0064697265625000,
+-0.0130004882812500, -0.0109558105468750, -0.0145874023437500, -0.0226745605468750,
+-0.0281066894531250, -0.0172119140625000, -0.0251464843750000, -0.0281677246093750,
+-0.0245056152343750, -0.0296020507812500, -0.0233154296875000, -0.0291748046875000,
+-0.0292358398437500, -0.0338134765625000, -0.0324707031250000, -0.0328369140625000,
+-0.0265808105468750, -0.0328979492187500, -0.0429687500000000, -0.0339660644531250,
+-0.0437622070312500, -0.0322265625000000, -0.0435485839843750, -0.0318298339843750,
+-0.0507507324218750, -0.0329895019531250, -0.0477600097656250, -0.0235595703125000,
+-0.0479125976562500, -0.0285339355468750, -0.0529479980468750, -0.0183410644531250,
+-0.0433349609375000, -0.0056457519531250, -0.0350646972656250, -0.0152282714843750,
+-0.0425415039062500, 0.0133972167968750, -0.0199279785156250, 0.0118408203125000,
+-0.0205688476562500, 0.0133666992187500, -0.0210876464843750, 0.0317687988281250,
+-0.0090942382812500, 0.0263061523437500, -0.0108337402343750, 0.0222473144531250,
+-0.0079650878906250, 0.0281066894531250, -0.0086669921875000, 0.0231323242187500,
+-0.0093994140625000, 0.0129089355468750, -0.0064697265625000, 0.0175170898437500,
+-0.0045166015625000, 0.0168151855468750, -0.0051269531250000, 0.0126037597656250,
+-0.0107116699218750, 0.0186462402343750, -0.0007324218750000, 0.0245056152343750,
+0.0088195800781250, 0.0198059082031250, 0.0014953613281250, 0.0309448242187500,
+0.0239562988281250, 0.0332031250000000, 0.0217285156250000, 0.0294494628906250,
+0.0235290527343750, 0.0308532714843750, 0.0467529296875000, 0.0330200195312500,
+0.0018310546875000, 0.0334472656250000, 0.0314025878906250, 0.0432739257812500,
+0.0219116210937500, 0.0253601074218750, 0.0247802734375000, 0.0132446289062500,
+0.0407104492187500, 0.0264282226562500, 0.0384826660156250, -0.0187072753906250,
+0.0265502929687500, -0.0140075683593750, 0.0310974121093750, -0.0188598632812500,
+0.0281982421875000, -0.0501098632812500, 0.0169372558593750, -0.0391540527343750,
+0.0203857421875000, -0.0406799316406250, 0.0178833007812500, -0.0473937988281250,
+0.0183715820312500, -0.0435485839843750, 0.0149536132812500, -0.0334167480468750,
+0.0123291015625000, -0.0364379882812500, 0.0158996582031250, -0.0565795898437500,
+0.0019836425781250, -0.0445861816406250, 0.0041503906250000, -0.0572204589843750,
+0.0015869140625000, -0.0859375000000000, -0.0097351074218750, -0.0722045898437500,
+-0.0051269531250000, -0.0827941894531250, -0.0033569335937500, -0.0941772460937500,
+-0.0075683593750000, -0.0744934082031250, -0.0032958984375000, -0.0647888183593750,
+0.0046081542968750, -0.0789184570312500, 0.0011596679687500, -0.0241088867187500,
+-0.0005187988281250, -0.0306396484375000, 0.0033874511718750, -0.0267639160156250,
+-0.0007629394531250, 0.0129394531250000, -0.0081787109375000, 0.0017700195312500,
+-0.0046997070312500, -0.0119018554687500, -0.0045471191406250, 0.0002441406250000,
+-0.0078430175781250, -0.0087890625000000, -0.0057983398437500, -0.0304565429687500,
+-0.0009765625000000, -0.0205078125000000, -0.0029296875000000, -0.0252685546875000,
+-0.0073242187500000, -0.0345153808593750, -0.0048828125000000, -0.0200500488281250,
+-0.0071716308593750, -0.0100402832031250, -0.0144653320312500, -0.0213317871093750,
+-0.0124206542968750, 0.0144958496093750, -0.0062866210937500, 0.0117797851562500,
+-0.0119323730468750, 0.0108337402343750, -0.0046386718750000, 0.0334472656250000,
+0.0077209472656250, 0.0285949707031250, 0.0010681152343750, 0.0150756835937500,
+0.0098876953125000, 0.0233459472656250, 0.0132751464843750, 0.0148620605468750,
+0.0093078613281250, -0.0024719238281250, 0.0085144042968750, 0.0066223144531250,
+0.0103454589843750, -0.0072326660156250, 0.0078430175781250, -0.0110778808593750,
+0.0062561035156250, -0.0054016113281250, 0.0079956054687500, -0.0087585449218750,
+0.0097045898437500, -0.0116577148437500, 0.0085449218750000, 0.0066528320312500,
+0.0075988769531250, 0.0004577636718750, 0.0099182128906250, 0.0084533691406250,
+0.0063781738281250, 0.0268859863281250, 0.0019531250000000, 0.0162353515625000,
+0.0049133300781250, 0.0293884277343750, -0.0008239746093750, 0.0345764160156250,
+-0.0015869140625000, 0.0275573730468750, -0.0006408691406250, 0.0271301269531250,
+-0.0026245117187500, 0.0307006835937500, -0.0024414062500000, 0.0181274414062500,
+-0.0003356933593750, 0.0208129882812500, -0.0022583007812500, 0.0169677734375000,
+0.0003662109375000, 0.0077819824218750, 0.0037231445312500, 0.0139160156250000,
+0.0005493164062500, 0.0018310546875000, 0.0071716308593750, 0.0033874511718750,
+0.0070190429687500, 0.0013122558593750, 0.0061340332031250, -0.0067443847656250,
+0.0090637207031250, -0.0018615722656250, 0.0081481933593750, -0.0090942382812500,
+0.0068664550781250, -0.0092773437500000, 0.0068054199218750, -0.0080566406250000,
+0.0065917968750000, -0.0111999511718750, 0.0055541992187500, -0.0101623535156250,
+0.0052795410156250, -0.0075988769531250, 0.0067443847656250, -0.0093994140625000,
+0.0054626464843750, -0.0070190429687500, 0.0074157714843750, -0.0036010742187500,
+0.0092163085937500, -0.0061645507812500, 0.0071716308593750, -0.0004577636718750,
+0.0163574218750000, -0.0014953613281250, 0.0142211914062500, -0.0005798339843750,
+0.0177001953125000, 0.0031738281250000, 0.0264282226562500, 0.0003051757812500,
+0.0229187011718750, 0.0025024414062500, 0.0294799804687500, 0.0025634765625000,
+0.0305786132812500, 0.0005187988281250, 0.0307922363281250, 0.0002136230468750,
+0.0333251953125000, -0.0003662109375000, 0.0325317382812500, -0.0045471191406250,
+0.0359191894531250, -0.0039062500000000, 0.0357666015625000, -0.0065612792968750,
+0.0359497070312500, -0.0105895996093750, 0.0386352539062500, -0.0091247558593750,
+0.0380554199218750, -0.0158386230468750, 0.0350036621093750, -0.0156860351562500,
+0.0379943847656250, -0.0190124511718750, 0.0335388183593750, -0.0245361328125000,
+0.0275878906250000, -0.0227355957031250, 0.0319213867187500, -0.0327148437500000,
+0.0216674804687500, -0.0317687988281250, 0.0209960937500000, -0.0350341796875000,
+0.0214233398437500, -0.0429077148437500, 0.0166320800781250, -0.0396728515625000,
+0.0168457031250000, -0.0458068847656250, 0.0160522460937500, -0.0468139648437500,
+0.0172119140625000, -0.0456237792968750, 0.0133361816406250, -0.0477905273437500,
+0.0104675292968750, -0.0475463867187500, 0.0133666992187500, -0.0435791015625000,
+0.0011901855468750, -0.0456237792968750, 0.0006408691406250, -0.0409545898437500,
+0.0028381347656250, -0.0354309082031250, -0.0037231445312500, -0.0386962890625000,
+-0.0047302246093750, -0.0245056152343750, 0.0080261230468750, -0.0263671875000000,
+0.0036926269531250, -0.0212097167968750, 0.0077209472656250, -0.0094604492187500,
+0.0201110839843750, -0.0145568847656250, 0.0144348144531250, -0.0030212402343750,
+0.0184326171875000, -0.0019836425781250, 0.0220031738281250, -0.0024108886718750,
+0.0184326171875000, 0.0028686523437500, 0.0164184570312500, 0.0016479492187500,
+0.0195312500000000, 0.0009155273437500, 0.0138854980468750, 0.0022277832031250,
+0.0130004882812500, -0.0008544921875000, 0.0164489746093750, -0.0037231445312500,
+0.0154724121093750, -0.0019531250000000, 0.0131530761718750, -0.0099792480468750,
+0.0215454101562500, -0.0104675292968750, 0.0218505859375000, -0.0095520019531250,
+0.0179138183593750, -0.0140991210937500, 0.0205078125000000, -0.0142517089843750,
+0.0223388671875000, -0.0059204101562500, 0.0107421875000000, -0.0093078613281250,
+0.0131835937500000, -0.0037841796875000, 0.0093078613281250, 0.0065612792968750,
+0.0006713867187500, 0.0016174316406250, 0.0050354003906250, 0.0115661621093750,
+-0.0094604492187500, 0.0137939453125000, -0.0066833496093750, 0.0106811523437500,
+-0.0101318359375000, 0.0129699707031250, -0.0223999023437500, 0.0136413574218750,
+-0.0170593261718750, 0.0051574707031250, -0.0207214355468750, 0.0066223144531250,
+-0.0236816406250000, 0.0040588378906250, -0.0216369628906250, -0.0029907226562500,
+-0.0202636718750000, -0.0002441406250000, -0.0216674804687500, -0.0044250488281250,
+-0.0200195312500000, -0.0068359375000000, -0.0195922851562500, -0.0032653808593750,
+-0.0211486816406250, -0.0011596679687500, -0.0217895507812500, -0.0043029785156250,
+-0.0206604003906250, -0.0026855468750000, -0.0209960937500000, -0.0004882812500000,
+-0.0246276855468750, -0.0019836425781250, -0.0178222656250000, -0.0037231445312500,
+-0.0115051269531250, -0.0037536621093750, -0.0178222656250000, -0.0086669921875000,
+-0.0057373046875000, -0.0063476562500000, -0.0027770996093750, -0.0080566406250000,
+-0.0065917968750000, -0.0135192871093750, -0.0050048828125000, -0.0122985839843750,
+-0.0034790039062500, -0.0180053710937500, -0.0071105957031250, -0.0160827636718750,
+-0.0063476562500000, -0.0191955566406250, -0.0083618164062500, -0.0274353027343750,
+-0.0113830566406250, -0.0243530273437500, -0.0089721679687500, -0.0216979980468750,
+-0.0152893066406250, -0.0263977050781250, -0.0133666992187500, -0.0214538574218750,
+-0.0158081054687500, -0.0100402832031250, -0.0242309570312500, -0.0113220214843750,
+-0.0220031738281250, -0.0144958496093750, -0.0171813964843750, -0.0102539062500000,
+-0.0192871093750000, -0.0130004882812500, -0.0206604003906250, -0.0183410644531250,
+-0.0162963867187500, -0.0151062011718750, -0.0156250000000000, -0.0245361328125000,
+-0.0254516601562500, -0.0237731933593750, -0.0221252441406250, -0.0212402343750000,
+-0.0255126953125000, -0.0269775390625000, -0.0357360839843750, -0.0275573730468750,
+-0.0313720703125000},
+{0.0484924316406250, -0.0103149414062500, 0.0442199707031250, -0.0172119140625000,
+0.0425720214843750, -0.0156555175781250, 0.0465087890625000, -0.0180053710937500,
+0.0414428710937500, -0.0236511230468750, 0.0339965820312500, -0.0204162597656250,
+0.0374755859375000, -0.0284118652343750, 0.0331726074218750, -0.0278625488281250,
+0.0298461914062500, -0.0300903320312500, 0.0324096679687500, -0.0360717773437500,
+0.0346069335937500, -0.0335693359375000, 0.0317687988281250, -0.0390930175781250,
+0.0315551757812500, -0.0399780273437500, 0.0349426269531250, -0.0391540527343750,
+0.0293884277343750, -0.0413208007812500, 0.0240478515625000, -0.0412292480468750,
+0.0289001464843750, -0.0372314453125000, 0.0170288085937500, -0.0392456054687500,
+0.0176086425781250, -0.0350036621093750, 0.0182495117187500, -0.0294189453125000,
+0.0114135742187500, -0.0324401855468750, 0.0133361816406250, -0.0205078125000000,
+0.0179443359375000, -0.0220336914062500, 0.0147705078125000, -0.0170288085937500,
+0.0166931152343750, -0.0070800781250000, 0.0234985351562500, -0.0124206542968750,
+0.0205383300781250, -0.0008544921875000, 0.0158996582031250, 0.0009155273437500,
+0.0208129882812500, -0.0022277832031250, 0.0144042968750000, 0.0014343261718750,
+0.0050048828125000, 0.0022888183593750, 0.0104675292968750, -0.0066223144531250,
+-0.0024108886718750, -0.0060424804687500, -0.0010681152343750, -0.0055236816406250,
+-0.0054321289062500, -0.0114746093750000, -0.0157165527343750, -0.0111694335937500,
+-0.0097351074218750, -0.0032348632812500, -0.0180358886718750, -0.0068054199218750,
+-0.0213012695312500, -0.0027465820312500, -0.0171508789062500, 0.0066833496093750,
+-0.0167541503906250, 0.0021667480468750, -0.0195617675781250, 0.0072326660156250,
+-0.0150756835937500, 0.0101623535156250, -0.0142517089843750, 0.0061035156250000,
+-0.0153503417968750, 0.0045471191406250, -0.0146484375000000, 0.0064086914062500,
+-0.0148620605468750, -0.0033569335937500, -0.0156860351562500, -0.0020141601562500,
+-0.0161743164062500, -0.0039672851562500, -0.0146484375000000, -0.0126037597656250,
+-0.0150451660156250, -0.0097351074218750, -0.0164489746093750, -0.0044250488281250,
+-0.0072326660156250, -0.0080261230468750, -0.0089416503906250, -0.0061035156250000,
+-0.0090026855468750, 0.0009460449218750, -0.0021057128906250, -0.0011901855468750,
+-0.0030517578125000, -0.0014648437500000, -0.0072021484375000, 0.0018310546875000,
+-0.0059509277343750, -0.0052795410156250, -0.0060119628906250, -0.0118713378906250,
+-0.0081481933593750, -0.0057067871093750, -0.0079345703125000, -0.0164489746093750,
+-0.0128173828125000, -0.0192260742187500, -0.0107116699218750, -0.0185241699218750,
+-0.0118408203125000, -0.0210876464843750, -0.0162353515625000, -0.0200195312500000,
+-0.0144958496093750, -0.0213012695312500, -0.0197448730468750, -0.0228881835937500,
+-0.0193481445312500, -0.0210571289062500, -0.0172119140625000, -0.0193481445312500,
+-0.0172424316406250, -0.0199584960937500, -0.0170898437500000, -0.0148010253906250,
+-0.0209655761718750, -0.0151062011718750, -0.0167541503906250, -0.0166931152343750,
+-0.0200805664062500, -0.0150451660156250, -0.0271301269531250, -0.0144348144531250,
+-0.0223999023437500, -0.0148620605468750, -0.0278930664062500, -0.0139465332031250,
+-0.0306396484375000, -0.0182189941406250, -0.0256347656250000, -0.0213623046875000,
+-0.0243530273437500, -0.0160522460937500, -0.0264892578125000, -0.0200195312500000,
+-0.0128784179687500, -0.0258178710937500, -0.0177307128906250, -0.0169982910156250,
+-0.0075988769531250, -0.0082092285156250, 0.0104675292968750, -0.0154113769531250,
+0.0006408691406250, -0.0083618164062500, 0.0109558105468750, -0.0023803710937500,
+0.0176391601562500, -0.0100708007812500, 0.0117797851562500, -0.0158386230468750,
+0.0090637207031250, -0.0103759765625000, 0.0121154785156250, -0.0202636718750000,
+0.0016479492187500, -0.0225524902343750, 0.0032348632812500, -0.0185241699218750,
+0.0021362304687500, -0.0200805664062500, -0.0057983398437500, -0.0217895507812500,
+-0.0024719238281250, -0.0133056640625000, 0.0002441406250000, -0.0143432617187500,
+-0.0050964355468750, -0.0132446289062500, 0.0043640136718750, -0.0082092285156250,
+0.0159606933593750, -0.0101928710937500, 0.0078430175781250, -0.0041809082031250,
+0.0187988281250000, -0.0058898925781250, 0.0246582031250000, -0.0037536621093750,
+0.0174865722656250, 0.0028076171875000, 0.0156860351562500, -0.0004882812500000,
+0.0195922851562500, -0.0017395019531250, 0.0004272460937500, 0.0028076171875000,
+0.0043029785156250, -0.0039672851562500, -0.0013427734375000, -0.0119628906250000,
+-0.0184020996093750, -0.0061645507812500, -0.0111389160156250, -0.0170593261718750,
+-0.0166625976562500, -0.0193786621093750, -0.0220947265625000, -0.0153503417968750,
+-0.0140075683593750, -0.0182495117187500, -0.0087280273437500, -0.0198974609375000,
+-0.0140991210937500, -0.0073547363281250, 0.0005187988281250, -0.0124206542968750,
+0.0024414062500000, -0.0051879882812500, -0.0007019042968750, 0.0080566406250000,
+0.0046691894531250, -0.0005798339843750, 0.0049438476562500, 0.0162658691406250,
+-0.0076293945312500, 0.0159606933593750, -0.0038757324218750, 0.0161132812500000,
+-0.0123901367187500, 0.0249633789062500, -0.0265808105468750, 0.0214538574218750,
+-0.0197753906250000, 0.0246582031250000, -0.0370178222656250, 0.0251464843750000,
+-0.0387878417968750, 0.0252380371093750, -0.0381469726562500, 0.0265197753906250,
+-0.0458984375000000, 0.0256652832031250, -0.0443725585937500, 0.0287475585937500,
+-0.0422668457031250, 0.0278625488281250, -0.0441284179687500, 0.0306091308593750,
+-0.0414123535156250, 0.0342407226562500, -0.0375976562500000, 0.0328979492187500,
+-0.0393371582031250, 0.0419616699218750, -0.0330505371093750, 0.0412597656250000,
+-0.0332946777343750, 0.0440368652343750, -0.0314941406250000, 0.0515747070312500,
+-0.0268249511718750, 0.0489501953125000, -0.0285644531250000, 0.0517272949218750,
+-0.0245361328125000, 0.0546264648437500, -0.0241088867187500, 0.0514221191406250,
+-0.0239868164062500, 0.0495605468750000, -0.0222778320312500, 0.0519409179687500,
+-0.0227661132812500, 0.0431213378906250, -0.0215454101562500, 0.0456542968750000,
+-0.0221862792968750, 0.0402832031250000, -0.0213928222656250, 0.0317382812500000,
+-0.0200195312500000, 0.0369567871093750, -0.0210876464843750, 0.0200500488281250,
+-0.0183410644531250, 0.0226745605468750, -0.0186462402343750, 0.0166015625000000,
+-0.0178222656250000, 0.0026855468750000, -0.0157165527343750, 0.0096740722656250,
+-0.0166931152343750, -0.0059509277343750, -0.0141601562500000, -0.0068664550781250,
+-0.0145874023437500, -0.0087585449218750, -0.0142211914062500, -0.0176086425781250,
+-0.0121459960937500, -0.0145874023437500, -0.0130310058593750, -0.0199279785156250,
+-0.0129089355468750, -0.0214538574218750, -0.0121765136718750, -0.0202331542968750,
+-0.0131835937500000, -0.0219726562500000, -0.0143432617187500, -0.0224304199218750,
+-0.0137939453125000, -0.0174560546875000, -0.0152587890625000, -0.0210876464843750,
+-0.0162048339843750, -0.0149230957031250, -0.0161437988281250, -0.0081481933593750,
+-0.0168762207031250, -0.0140991210937500, -0.0178527832031250, 0.0081787109375000,
+-0.0161132812500000, 0.0031433105468750, -0.0182800292968750, 0.0123901367187500,
+-0.0160827636718750, 0.0328979492187500, -0.0137023925781250, 0.0225524902343750,
+-0.0172119140625000, 0.0430603027343750, -0.0077209472656250, 0.0447998046875000,
+-0.0103759765625000, 0.0444335937500000, -0.0081787109375000, 0.0531005859375000,
+-0.0002136230468750, 0.0505065917968750, -0.0047912597656250, 0.0558166503906250,
+-0.0004272460937500, 0.0553283691406250, 0.0015869140625000, 0.0577392578125000,
+-0.0021667480468750, 0.0626525878906250, -0.0025024414062500, 0.0599975585937500,
+-0.0000915527343750, 0.0642395019531250, -0.0101013183593750, 0.0656433105468750,
+-0.0077819824218750, 0.0635681152343750, -0.0109252929687500, 0.0645751953125000,
+-0.0198364257812500, 0.0657653808593750, -0.0159912109375000, 0.0581665039062500,
+-0.0213317871093750, 0.0621643066406250, -0.0227050781250000, 0.0540466308593750,
+-0.0208740234375000, 0.0442810058593750, -0.0228576660156250, 0.0515747070312500,
+-0.0233764648437500, 0.0261535644531250, -0.0147094726562500, 0.0302429199218750,
+-0.0198669433593750, 0.0221252441406250, -0.0099487304687500, 0.0020446777343750,
+0.0020751953125000, 0.0121154785156250, -0.0073547363281250, -0.0087890625000000,
+0.0215148925781250, -0.0084533691406250, 0.0187377929687500, -0.0119934082031250,
+0.0226440429687500, -0.0251464843750000, 0.0443725585937500, -0.0191345214843750,
+0.0370483398437500, -0.0292053222656250, 0.0352478027343750, -0.0312500000000000,
+0.0450744628906250, -0.0292968750000000, 0.0275573730468750, -0.0325012207031250,
+0.0110168457031250, -0.0328369140625000, 0.0243835449218750, -0.0285644531250000,
+-0.0155639648437500, -0.0297546386718750, -0.0162048339843750, -0.0283508300781250,
+-0.0146789550781250, -0.0253295898437500, -0.0352172851562500, -0.0268859863281250,
+-0.0308227539062500, -0.0200500488281250, -0.0228271484375000, -0.0232849121093750,
+-0.0285034179687500, -0.0160827636718750, -0.0206298828125000, -0.0072631835937500,
+-0.0087280273437500, -0.0140380859375000, -0.0158386230468750, 0.0068664550781250,
+0.0003356933593750, 0.0052490234375000, 0.0047302246093750, 0.0084838867187500,
+-0.0069885253906250, 0.0225830078125000, -0.0050048828125000, 0.0169372558593750,
+0.0023498535156250, 0.0229797363281250, -0.0368652343750000, 0.0263671875000000,
+-0.0310668945312500, 0.0220642089843750, -0.0382080078125000, 0.0205993652343750,
+-0.0693969726562500, 0.0237121582031250, -0.0573730468750000, 0.0179138183593750,
+-0.0542297363281250, 0.0158081054687500, -0.0668334960937500, 0.0204467773437500,
+-0.0513000488281250, 0.0214233398437500, -0.0334167480468750, 0.0182800292968750,
+-0.0454711914062500, 0.0277404785156250, -0.0095214843750000, 0.0284423828125000,
+-0.0079956054687500, 0.0276489257812500, -0.0141296386718750, 0.0307922363281250,
+0.0003051757812500, 0.0303039550781250, 0.0010375976562500, 0.0330810546875000,
+-0.0089416503906250, 0.0320434570312500, -0.0097961425781250, 0.0332031250000000,
+-0.0064697265625000, 0.0367126464843750, -0.0101013183593750, 0.0350341796875000,
+-0.0115356445312500, 0.0349731445312500, 0.0051574707031250, 0.0388183593750000,
+0.0029907226562500, 0.0298767089843750, -0.0001831054687500, 0.0140991210937500,
+0.0107727050781250, -0.0285034179687500, -0.0048217773437500, -0.0267639160156250,
+0.0088500976562500, -0.0343627929687500, 0.0094604492187500, -0.0292663574218750,
+0.0067443847656250, -0.0196533203125000, 0.0071716308593750, -0.0231323242187500,
+0.0092773437500000, -0.0232238769531250, 0.0061340332031250, -0.0187988281250000,
+0.0020141601562500, -0.0251770019531250, 0.0113220214843750, -0.0310668945312500,
+0.0176696777343750, -0.0255126953125000, 0.0103454589843750, -0.0411071777343750,
+0.0297851562500000, -0.0401000976562500, 0.0314636230468750, -0.0399169921875000,
+0.0277709960937500, -0.0487670898437500, 0.0353088378906250, -0.0469360351562500,
+0.0353698730468750, -0.0485534667968750, 0.0232543945312500, -0.0466918945312500,
+0.0256958007812500, -0.0489501953125000, 0.0250244140625000, -0.0548706054687500,
+0.0158691406250000, -0.0512695312500000, 0.0177001953125000, -0.0430908203125000,
+0.0226135253906250, -0.0502624511718750, 0.0222167968750000, -0.0394592285156250,
+0.0181274414062500, -0.0251159667968750, 0.0202026367187500, -0.0346069335937500,
+0.0230407714843750, -0.0085754394531250, 0.0047302246093750, -0.0129699707031250,
+0.0066833496093750},
+{-0.0094604492187500, 0.0354919433593750, -0.0057983398437500, 0.0268554687500000,
+-0.0106506347656250, 0.0087280273437500, -0.0195312500000000, 0.0186767578125000,
+-0.0145263671875000, -0.0064086914062500, -0.0187072753906250, -0.0052490234375000,
+-0.0249633789062500, -0.0087280273437500, -0.0125427246093750, -0.0250854492187500,
+-0.0052795410156250, -0.0189208984375000, -0.0145568847656250, -0.0270996093750000,
+0.0202941894531250, -0.0296020507812500, 0.0177917480468750, -0.0278625488281250,
+0.0228271484375000, -0.0300292968750000, 0.0479431152343750, -0.0298767089843750,
+0.0387268066406250, -0.0249328613281250, 0.0351257324218750, -0.0285949707031250,
+0.0480346679687500, -0.0219726562500000, 0.0286254882812500, -0.0144348144531250,
+0.0067443847656250, -0.0205993652343750, 0.0218811035156250, -0.0003051757812500,
+-0.0163269042968750, -0.0022583007812500, -0.0185852050781250, 0.0008239746093750,
+-0.0170288085937500, 0.0147705078125000, -0.0348510742187500, 0.0092468261718750,
+-0.0301818847656250, 0.0152587890625000, -0.0256652832031250, 0.0168457031250000,
+-0.0330200195312500, 0.0162658691406250, -0.0208129882812500, 0.0179443359375000,
+-0.0051269531250000, 0.0173339843750000, -0.0162963867187500, 0.0184631347656250,
+-0.0031127929687500, 0.0185241699218750, 0.0075988769531250, 0.0196228027343750,
+-0.0100708007812500, 0.0202941894531250, -0.0199890136718750, 0.0196228027343750,
+-0.0072937011718750, 0.0250244140625000, -0.0468750000000000, 0.0240478515625000,
+-0.0476379394531250, 0.0247497558593750, -0.0438842773437500, 0.0288696289062500,
+-0.0628356933593750, 0.0277404785156250, -0.0597229003906250, 0.0283813476562500,
+-0.0461730957031250, 0.0280151367187500, -0.0570983886718750, 0.0300903320312500,
+-0.0377807617187500, 0.0328063964843750, -0.0108642578125000, 0.0302124023437500,
+-0.0287780761718750, 0.0284118652343750, -0.0012817382812500, 0.0339965820312500,
+0.0064086914062500, 0.0230407714843750, -0.0025634765625000, 0.0137329101562500,
+0.0006713867187500, 0.0219116210937500, 0.0044860839843750, -0.0112304687500000,
+-0.0065612792968750, -0.0074157714843750, -0.0101623535156250, -0.0094909667968750,
+-0.0024719238281250, -0.0426940917968750, 0.0040283203125000, -0.0264587402343750,
+0.0040893554687500, -0.0204467773437500, 0.0005798339843750, -0.0151672363281250,
+-0.0001220703125000, -0.0193176269531250, 0.0021362304687500, -0.0248107910156250,
+0.0022888183593750, -0.0224914550781250, 0.0000000000000000, -0.0257873535156250,
+0.0073547363281250, -0.0243530273437500, 0.0089721679687500, -0.0292663574218750,
+0.0051269531250000, -0.0328063964843750, 0.0043945312500000, -0.0279541015625000,
+0.0069885253906250, -0.0489196777343750, 0.0054321289062500, -0.0446777343750000,
+0.0015563964843750, -0.0505676269531250, 0.0086364746093750, -0.0684204101562500,
+0.0143737792968750, -0.0591735839843750, 0.0088500976562500, -0.0659790039062500,
+0.0251770019531250, -0.0766601562500000, 0.0268554687500000, -0.0557556152343750,
+0.0224304199218750, -0.0416564941406250, 0.0277099609375000, -0.0577697753906250,
+0.0294799804687500, -0.0106811523437500, 0.0185852050781250, -0.0136108398437500,
+0.0183715820312500, -0.0087585449218750, 0.0212707519531250, 0.0205688476562500,
+0.0180358886718750, 0.0090637207031250, 0.0166931152343750, 0.0187988281250000,
+0.0203247070312500, 0.0254211425781250, 0.0236511230468750, 0.0154418945312500,
+0.0157775878906250, 0.0094299316406250, 0.0111083984375000, 0.0169677734375000,
+0.0172424316406250, 0.0099792480468750, -0.0009765625000000, 0.0022583007812500,
+-0.0020446777343750, 0.0124511718750000, 0.0001831054687500, 0.0204772949218750,
+-0.0072326660156250, 0.0134582519531250, -0.0069274902343750, 0.0334167480468750,
+-0.0029296875000000, 0.0317077636718750, -0.0026550292968750, 0.0335693359375000,
+-0.0046386718750000, 0.0473632812500000, -0.0037536621093750, 0.0417480468750000,
+-0.0022888183593750, 0.0385437011718750, -0.0086669921875000, 0.0457153320312500,
+-0.0089416503906250, 0.0361022949218750, -0.0074157714843750, 0.0230407714843750,
+-0.0101013183593750, 0.0321044921875000, -0.0107421875000000, 0.0221862792968750,
+-0.0053710937500000, 0.0174255371093750, -0.0062561035156250, 0.0209350585937500,
+-0.0058898925781250, 0.0226440429687500, -0.0030822753906250, 0.0210571289062500,
+-0.0042114257812500, 0.0181884765625000, -0.0002136230468750, 0.0215148925781250,
+-0.0042114257812500, 0.0165100097656250, 0.0040893554687500, 0.0102844238281250,
+0.0121765136718750, 0.0146179199218750, 0.0046081542968750, 0.0024108886718750,
+0.0304260253906250, 0.0044860839843750, 0.0274047851562500, -0.0007934570312500,
+0.0318603515625000, -0.0099487304687500, 0.0497741699218750, -0.0034179687500000,
+0.0423278808593750, -0.0229492187500000, 0.0534057617187500, -0.0213317871093750,
+0.0538330078125000, -0.0223388671875000, 0.0552673339843750, -0.0347900390625000,
+0.0614929199218750, -0.0304870605468750, 0.0580139160156250, -0.0300292968750000,
+0.0636291503906250, -0.0340881347656250, 0.0651855468750000, -0.0287475585937500,
+0.0621337890625000, -0.0239562988281250, 0.0629577636718750, -0.0287170410156250,
+0.0643920898437500, -0.0149536132812500, 0.0550231933593750, -0.0184631347656250,
+0.0578918457031250, -0.0134277343750000, 0.0528564453125000, -0.0023498535156250,
+0.0445251464843750, -0.0101318359375000, 0.0498657226562500, 0.0054626464843750,
+0.0337524414062500, 0.0036010742187500, 0.0368957519531250, 0.0057983398437500,
+0.0306701660156250, 0.0158386230468750, 0.0176086425781250, 0.0101013183593750,
+0.0252990722656250, 0.0183105468750000, 0.0060424804687500, 0.0188293457031250,
+0.0086975097656250, 0.0179443359375000, 0.0021362304687500, 0.0213623046875000,
+-0.0122680664062500, 0.0204467773437500, -0.0040283203125000, 0.0213928222656250,
+-0.0259704589843750, 0.0215148925781250, -0.0235290527343750, 0.0227355957031250,
+-0.0293579101562500, 0.0242309570312500, -0.0456237792968750, 0.0239562988281250,
+-0.0375976562500000, 0.0271911621093750, -0.0527648925781250, 0.0280151367187500,
+-0.0541076660156250, 0.0281982421875000, -0.0522766113281250, 0.0296325683593750,
+-0.0593566894531250, 0.0303649902343750, -0.0579223632812500, 0.0306396484375000,
+-0.0498657226562500, 0.0308837890625000, -0.0551452636718750, 0.0322570800781250,
+-0.0456237792968750, 0.0332946777343750, -0.0333862304687500, 0.0332336425781250,
+-0.0412902832031250, 0.0359497070312500, -0.0169067382812500, 0.0368652343750000,
+-0.0185852050781250, 0.0369567871093750, -0.0133972167968750, 0.0380859375000000,
+0.0032958984375000, 0.0384521484375000, -0.0033569335937500, 0.0369567871093750,
+0.0106506347656250, 0.0378723144531250, 0.0087890625000000, 0.0359191894531250,
+0.0159301757812500, 0.0336914062500000, 0.0267639160156250, 0.0348815917968750,
+0.0187377929687500, 0.0280151367187500, 0.0450439453125000, 0.0293579101562500,
+0.0417785644531250, 0.0252075195312500, 0.0459594726562500, 0.0187988281250000,
+0.0652160644531250, 0.0218200683593750, 0.0576477050781250, 0.0087585449218750,
+0.0630493164062500, 0.0105285644531250, 0.0676269531250000, 0.0057678222656250,
+0.0629272460937500, -0.0049133300781250, 0.0607604980468750, -0.0003356933593750,
+0.0640869140625000, -0.0121765136718750, 0.0547180175781250, -0.0124511718750000,
+0.0566711425781250, -0.0140991210937500, 0.0535583496093750, -0.0214233398437500,
+0.0458068847656250, -0.0191650390625000, 0.0493774414062500, -0.0230712890625000,
+0.0393981933593750, -0.0242004394531250, 0.0412902832031250, -0.0237426757812500,
+0.0340881347656250, -0.0251464843750000, 0.0249633789062500, -0.0252990722656250,
+0.0317687988281250, -0.0237731933593750, 0.0088500976562500, -0.0242614746093750,
+0.0112915039062500, -0.0236511230468750, 0.0077209472656250, -0.0216064453125000,
+-0.0077514648437500, -0.0221252441406250, -0.0015869140625000, -0.0233154296875000,
+-0.0112915039062500, -0.0220947265625000, -0.0118713378906250, -0.0231018066406250,
+-0.0124206542968750, -0.0259399414062500, -0.0183715820312500, -0.0249938964843750,
+-0.0160522460937500, -0.0238037109375000, -0.0146179199218750, -0.0265197753906250,
+-0.0176696777343750, -0.0217590332031250, -0.0146789550781250, -0.0170898437500000,
+-0.0098266601562500, -0.0207214355468750, -0.0126953125000000, -0.0071716308593750,
+-0.0094299316406250, -0.0090637207031250, -0.0093383789062500, -0.0046386718750000,
+-0.0084533691406250, 0.0061340332031250, -0.0067749023437500, 0.0006713867187500,
+-0.0089721679687500, 0.0101318359375000, -0.0057983398437500, 0.0120544433593750,
+-0.0052490234375000, 0.0093688964843750, -0.0061340332031250, 0.0119018554687500,
+-0.0059509277343750, 0.0122070312500000, -0.0057983398437500, 0.0048217773437500,
+-0.0033264160156250, 0.0064697265625000, -0.0044555664062500, 0.0043029785156250,
+-0.0034790039062500, -0.0020141601562500, -0.0004577636718750, 0.0000305175781250,
+-0.0016479492187500, -0.0048217773437500, -0.0016479492187500, -0.0037536621093750,
+0.0000000000000000, -0.0056762695312500, -0.0031433105468750, -0.0114746093750000,
+-0.0050964355468750, -0.0092468261718750, -0.0026855468750000, -0.0074768066406250,
+-0.0142211914062500, -0.0101013183593750, -0.0125122070312500, -0.0064697265625000,
+-0.0112915039062500, -0.0020446777343750, -0.0183410644531250, -0.0055236816406250,
+-0.0181884765625000, 0.0021972656250000, -0.0147705078125000, 0.0049133300781250,
+-0.0129089355468750, 0.0001831054687500, -0.0152587890625000, -0.0000305175781250,
+-0.0170898437500000, 0.0027465820312500, -0.0152587890625000, -0.0089416503906250,
+-0.0178222656250000, -0.0061950683593750, -0.0166931152343750, -0.0091857910156250,
+-0.0192871093750000, -0.0228881835937500, -0.0247192382812500, -0.0206604003906250,
+-0.0234985351562500, -0.0112915039062500, -0.0239257812500000, -0.0183410644531250,
+-0.0253601074218750, -0.0119018554687500, -0.0226745605468750, 0.0010070800781250,
+-0.0213012695312500, -0.0078735351562500, -0.0235900878906250, 0.0021667480468750,
+-0.0148010253906250, 0.0083312988281250, -0.0169677734375000, 0.0018310546875000,
+-0.0123291015625000, -0.0023803710937500, -0.0028076171875000, 0.0016479492187500,
+-0.0085144042968750, -0.0026855468750000, -0.0053405761718750, -0.0040283203125000,
+-0.0010375976562500, -0.0041503906250000, -0.0035400390625000, -0.0058593750000000,
+-0.0072326660156250, -0.0044860839843750, -0.0064392089843750, -0.0021362304687500,
+-0.0081787109375000, -0.0048217773437500, -0.0077819824218750, -0.0012207031250000,
+-0.0066223144531250, 0.0049133300781250, -0.0089416503906250, 0.0016479492187500,
+-0.0095825195312500, 0.0032958984375000, 0.0007934570312500, 0.0062561035156250,
+-0.0009765625000000, 0.0035705566406250, -0.0010375976562500, 0.0014343261718750,
+0.0070190429687500, 0.0034484863281250, 0.0054016113281250, -0.0050964355468750,
+-0.0042419433593750, -0.0025329589843750, 0.0001831054687500, -0.0060119628906250,
+-0.0023803710937500, -0.0151977539062500, -0.0130920410156250, -0.0107727050781250,
+-0.0098876953125000, -0.0130004882812500, -0.0094604492187500, -0.0169372558593750,
+-0.0115966796875000, -0.0117187500000000, -0.0085144042968750, -0.0073242187500000,
+-0.0057983398437500, -0.0113220214843750, -0.0086364746093750, -0.0014953613281250,
+-0.0020751953125000, -0.0022583007812500, -0.0004272460937500, -0.0011291503906250,
+-0.0052795410156250, 0.0051574707031250, -0.0052185058593750, 0.0023498535156250,
+-0.0029602050781250, 0.0047912597656250, -0.0166931152343750, 0.0065612792968750,
+-0.0147094726562500, 0.0031433105468750, -0.0194091796875000, 0.0018920898437500,
+-0.0309753417968750},
+{-0.0113220214843750, -0.0250549316406250, -0.0107727050781250, -0.0249938964843750,
+-0.0116882324218750, -0.0234680175781250, -0.0079650878906250, -0.0254516601562500,
+-0.0090332031250000, -0.0220031738281250, -0.0090637207031250, -0.0190734863281250,
+-0.0061645507812500, -0.0218811035156250, -0.0065917968750000, -0.0075683593750000,
+-0.0063781738281250, -0.0115661621093750, -0.0072326660156250, -0.0050048828125000,
+-0.0040283203125000, 0.0092163085937500, -0.0013732910156250, 0.0019836425781250,
+-0.0040588378906250, 0.0145874023437500, 0.0023193359375000, 0.0173950195312500,
+0.0035705566406250, 0.0122680664062500, 0.0013122558593750, 0.0142517089843750,
+0.0031433105468750, 0.0163574218750000, 0.0040588378906250, 0.0062866210937500,
+-0.0041198730468750, 0.0066528320312500, -0.0013122558593750, 0.0039978027343750,
+-0.0046997070312500, -0.0025329589843750, -0.0132446289062500, 0.0009155273437500,
+-0.0083618164062500, -0.0045776367187500, -0.0113830566406250, -0.0066528320312500,
+-0.0143432617187500, -0.0039367675781250, -0.0120239257812500, -0.0034790039062500,
+-0.0079650878906250, -0.0045471191406250, -0.0086669921875000, 0.0003051757812500,
+-0.0160217285156250, 0.0000610351562500, -0.0132446289062500, -0.0004577636718750,
+-0.0140075683593750, 0.0027465820312500, -0.0194091796875000, 0.0032653808593750,
+-0.0179748535156250, -0.0000305175781250, -0.0247497558593750, 0.0013732910156250,
+-0.0233154296875000, -0.0022888183593750, -0.0228271484375000, -0.0078125000000000,
+-0.0286560058593750, -0.0074768066406250, -0.0278320312500000, -0.0191955566406250,
+-0.0199584960937500, -0.0186767578125000, -0.0242309570312500, -0.0141601562500000,
+-0.0214233398437500, -0.0198364257812500, -0.0133972167968750, -0.0213928222656250,
+-0.0173645019531250, -0.0019836425781250, -0.0084838867187500, -0.0074462890625000,
+-0.0078430175781250, -0.0044555664062500, -0.0093383789062500, 0.0126953125000000,
+-0.0057373046875000, 0.0073547363281250, -0.0051269531250000, 0.0049743652343750,
+-0.0064392089843750, 0.0096435546875000, -0.0072937011718750, 0.0045166015625000,
+-0.0071105957031250, -0.0025634765625000, -0.0057373046875000, 0.0021972656250000,
+-0.0049133300781250, -0.0054931640625000, -0.0081787109375000, -0.0086059570312500,
+-0.0093994140625000, -0.0017395019531250, -0.0062561035156250, 0.0002441406250000,
+-0.0044555664062500, -0.0046691894531250, -0.0067138671875000, 0.0068054199218750,
+-0.0046691894531250, 0.0090026855468750, -0.0033874511718750, 0.0050659179687500,
+-0.0042114257812500, 0.0069885253906250, -0.0057983398437500, 0.0091247558593750,
+-0.0052490234375000, 0.0038452148437500, 0.0001525878906250, 0.0043334960937500,
+-0.0025634765625000, 0.0023193359375000, -0.0018615722656250, -0.0005493164062500,
+0.0048828125000000, 0.0018615722656250, 0.0034790039062500, -0.0091552734375000,
+-0.0028991699218750, -0.0074462890625000, -0.0001525878906250, -0.0083312988281250,
+-0.0044250488281250, -0.0168151855468750, -0.0113220214843750, -0.0143127441406250,
+-0.0077209472656250, -0.0115356445312500, -0.0201110839843750, -0.0149841308593750,
+-0.0197448730468750, -0.0111083984375000, -0.0210266113281250, -0.0051879882812500,
+-0.0285644531250000, -0.0090026855468750, -0.0263366699218750, -0.0023498535156250,
+-0.0297851562500000, -0.0019226074218750, -0.0303039550781250, -0.0030517578125000,
+-0.0303649902343750, -0.0005798339843750, -0.0318603515625000, -0.0011596679687500,
+-0.0312194824218750, -0.0023498535156250, -0.0330810546875000, -0.0020751953125000,
+-0.0335388183593750, -0.0018920898437500, -0.0325927734375000, -0.0025939941406250,
+-0.0332641601562500, -0.0024414062500000, -0.0339050292968750, -0.0008850097656250,
+-0.0292358398437500, -0.0010681152343750, -0.0302429199218750, 0.0006713867187500,
+-0.0275268554687500, 0.0021667480468750, -0.0230407714843750, 0.0014343261718750,
+-0.0251159667968750, 0.0083618164062500, -0.0171203613281250, 0.0049133300781250,
+-0.0176086425781250, 0.0115356445312500, -0.0151672363281250, 0.0199279785156250,
+-0.0086669921875000, 0.0129089355468750, -0.0108642578125000, 0.0318298339843750,
+-0.0075988769531250, 0.0296020507812500, -0.0056152343750000, 0.0316467285156250,
+-0.0070190429687500, 0.0442504882812500, -0.0069885253906250, 0.0382690429687500,
+-0.0057983398437500, 0.0418395996093750, -0.0099792480468750, 0.0446472167968750,
+-0.0087585449218750, 0.0390319824218750, -0.0101013183593750, 0.0362243652343750,
+-0.0135498046875000, 0.0381164550781250, -0.0115051269531250, 0.0234985351562500,
+-0.0162353515625000, 0.0263977050781250, -0.0159301757812500, 0.0184936523437500,
+-0.0162048339843750, 0.0046997070312500, -0.0190734863281250, 0.0117492675781250,
+-0.0181274414062500, -0.0072326660156250, -0.0186767578125000, -0.0072937011718750,
+-0.0187683105468750, -0.0099487304687500, -0.0184020996093750, -0.0208740234375000,
+-0.0180053710937500, -0.0167236328125000, -0.0173339843750000, -0.0269775390625000,
+-0.0181274414062500, -0.0259094238281250, -0.0166931152343750, -0.0293884277343750,
+-0.0169067382812500, -0.0376586914062500, -0.0181274414062500, -0.0332336425781250,
+-0.0168457031250000, -0.0408020019531250, -0.0181884765625000, -0.0423278808593750,
+-0.0168762207031250, -0.0387878417968750, -0.0174560546875000, -0.0400085449218750,
+-0.0188293457031250, -0.0410461425781250, -0.0171203613281250, -0.0317077636718750,
+-0.0203857421875000, -0.0331420898437500, -0.0195007324218750, -0.0313720703125000,
+-0.0201721191406250, -0.0242919921875000, -0.0226440429687500, -0.0266723632812500,
+-0.0213012695312500, -0.0249633789062500, -0.0243225097656250, -0.0238037109375000,
+-0.0233764648437500, -0.0240783691406250, -0.0243530273437500, -0.0252380371093750,
+-0.0265197753906250, -0.0251770019531250, -0.0244445800781250, -0.0193481445312500,
+-0.0289611816406250, -0.0227050781250000, -0.0275878906250000, -0.0167236328125000,
+-0.0292358398437500, -0.0086364746093750, -0.0327758789062500, -0.0141601562500000,
+-0.0297241210937500, 0.0014343261718750, -0.0358276367187500, -0.0005187988281250,
+-0.0358276367187500, 0.0030822753906250, -0.0341491699218750, 0.0140380859375000,
+-0.0372314453125000, 0.0081787109375000, -0.0371704101562500, 0.0180358886718750,
+-0.0299072265625000, 0.0182189941406250, -0.0334777832031250, 0.0166931152343750,
+-0.0288391113281250, 0.0201110839843750, -0.0205688476562500, 0.0185546875000000,
+-0.0260620117187500, 0.0199584960937500, -0.0156250000000000, 0.0190734863281250,
+-0.0150756835937500, 0.0203552246093750, -0.0153198242187500, 0.0225219726562500,
+-0.0104064941406250, 0.0205078125000000, -0.0117187500000000, 0.0228271484375000,
+-0.0098571777343750, 0.0235290527343750, -0.0093078613281250, 0.0225524902343750,
+-0.0108947753906250, 0.0226135253906250, -0.0103149414062500, 0.0229187011718750,
+-0.0091552734375000, 0.0215759277343750, -0.0155944824218750, 0.0214843750000000,
+-0.0144958496093750, 0.0209045410156250, -0.0146789550781250, 0.0199890136718750,
+-0.0193786621093750, 0.0201110839843750, -0.0180358886718750, 0.0172729492187500,
+-0.0153198242187500, 0.0177917480468750, -0.0177001953125000, 0.0170593261718750,
+-0.0146484375000000, 0.0142822265625000, -0.0094299316406250, 0.0148925781250000,
+-0.0119628906250000, 0.0157165527343750, -0.0084838867187500, 0.0149841308593750,
+-0.0087280273437500, 0.0145568847656250, -0.0043640136718750, 0.0162048339843750,
+-0.0014648437500000, 0.0160522460937500, -0.0054016113281250, 0.0094909667968750,
+0.0121459960937500, 0.0122375488281250, 0.0093383789062500, 0.0091552734375000,
+0.0136413574218750, 0.0012817382812500, 0.0278930664062500, 0.0048828125000000,
+0.0216064453125000, 0.0029907226562500, 0.0258178710937500, 0.0003356933593750,
+0.0311889648437500, 0.0032043457031250, 0.0234069824218750, 0.0063781738281250,
+0.0175170898437500, 0.0046691894531250, 0.0229797363281250, 0.0061035156250000,
+0.0090026855468750, 0.0071411132812500, 0.0085144042968750, 0.0070800781250000,
+0.0061340332031250, 0.0061035156250000, -0.0002746582031250, 0.0060119628906250,
+0.0033874511718750, 0.0111999511718750, -0.0114440917968750, 0.0097961425781250,
+-0.0108642578125000, 0.0109252929687500, -0.0111694335937500, 0.0162048339843750,
+-0.0191955566406250, 0.0145263671875000, -0.0172729492187500, 0.0114440917968750,
+-0.0199890136718750, 0.0148620605468750, -0.0177612304687500, 0.0088806152343750,
+-0.0256347656250000, 0.0029907226562500, -0.0326538085937500, 0.0078430175781250,
+-0.0244750976562500, -0.0124511718750000, -0.0411376953125000, -0.0084228515625000,
+-0.0458068847656250, -0.0141601562500000, -0.0404663085937500, -0.0318603515625000,
+-0.0404968261718750, -0.0240783691406250, -0.0418395996093750, -0.0303344726562500,
+-0.0407409667968750, -0.0350646972656250, -0.0411376953125000, -0.0283508300781250,
+-0.0386047363281250, -0.0247192382812500, -0.0375671386718750, -0.0288391113281250,
+-0.0389099121093750, -0.0164794921875000, -0.0253906250000000, -0.0144653320312500,
+-0.0300903320312500, -0.0197448730468750, -0.0249023437500000, -0.0196228027343750,
+-0.0020446777343750, -0.0199890136718750, -0.0299377441406250, -0.0409545898437500,
+-0.0203857421875000, -0.0235900878906250, -0.0305480957031250, -0.0438842773437500,
+-0.0153503417968750, -0.0700073242187500, 0.0025024414062500, -0.0534057617187500,
+-0.0095825195312500, -0.0775146484375000, 0.0018920898437500, -0.0874328613281250,
+0.0102539062500000, -0.0699768066406250, 0.0032043457031250, -0.0682373046875000,
+-0.0029296875000000, -0.0787048339843750, 0.0008850097656250, -0.0341186523437500,
+-0.0014953613281250, -0.0413208007812500, -0.0044860839843750, -0.0284118652343750,
+-0.0003051757812500, 0.0062561035156250, 0.0036621093750000, -0.0108947753906250,
+0.0001525878906250, 0.0174255371093750, 0.0031127929687500, 0.0205078125000000,
+0.0070495605468750, 0.0207824707031250, 0.0015869140625000, 0.0339660644531250,
+-0.0039367675781250, 0.0286560058593750, 0.0006408691406250, 0.0307922363281250,
+-0.0027770996093750, 0.0342407226562500, -0.0067138671875000, 0.0310974121093750,
+-0.0016174316406250, 0.0279541015625000, 0.0033569335937500, 0.0303344726562500,
+-0.0000305175781250, 0.0270385742187500, 0.0048217773437500, 0.0256042480468750,
+0.0071411132812500, 0.0282592773437500, 0.0047302246093750, 0.0284729003906250,
+0.0037841796875000, 0.0276794433593750, 0.0051574707031250, 0.0375671386718750,
+0.0031433105468750, 0.0329284667968750, 0.0026245117187500, 0.0379028320312500,
+0.0024414062500000, 0.0521545410156250, 0.0025939941406250, 0.0457153320312500,
+0.0028381347656250, 0.0389404296875000, -0.0025024414062500, 0.0479125976562500,
+-0.0003662109375000, 0.0370483398437500, -0.0033264160156250, 0.0214233398437500,
+-0.0101318359375000, 0.0312805175781250, -0.0068969726562500, 0.0105285644531250,
+-0.0086364746093750, 0.0123596191406250, -0.0124206542968750, 0.0061035156250000,
+-0.0070190429687500, -0.0080566406250000, -0.0023498535156250, 0.0016784667968750,
+-0.0065917968750000, -0.0175170898437500, 0.0044555664062500, -0.0193786621093750,
+0.0043029785156250, -0.0153808593750000, 0.0050354003906250, -0.0220642089843750,
+0.0104064941406250, -0.0223083496093750, 0.0080566406250000, -0.0149841308593750,
+0.0143737792968750, -0.0177917480468750, 0.0123291015625000, -0.0143432617187500,
+0.0158386230468750, -0.0077514648437500, 0.0229187011718750, -0.0118103027343750,
+0.0188598632812500, -0.0011596679687500, 0.0253906250000000, -0.0025939941406250,
+0.0266723632812500, -0.0003051757812500, 0.0253906250000000, 0.0083312988281250,
+0.0264892578125000, 0.0043640136718750, 0.0265502929687500, 0.0055541992187500,
+0.0282287597656250},
+{-0.0053100585937500, -0.0172424316406250, -0.0079956054687500, -0.0167236328125000,
+-0.0074768066406250, -0.0190124511718750, -0.0095520019531250, -0.0275878906250000,
+-0.0099487304687500, -0.0234680175781250, -0.0094299316406250, -0.0294189453125000,
+-0.0320434570312500, -0.0317077636718750, -0.0238342285156250, -0.0278625488281250,
+-0.0308837890625000, -0.0273437500000000, -0.0560607910156250, -0.0290832519531250,
+-0.0480346679687500, -0.0207519531250000, -0.0468750000000000, -0.0216369628906250,
+-0.0532226562500000, -0.0198059082031250, -0.0453796386718750, -0.0127868652343750,
+-0.0360717773437500, -0.0154113769531250, -0.0427246093750000, -0.0178222656250000,
+-0.0324401855468750, -0.0142517089843750, -0.0298156738281250, -0.0159912109375000,
+-0.0316162109375000, -0.0238342285156250, -0.0288085937500000, -0.0400085449218750,
+-0.0368957519531250, -0.0735778808593750, -0.0241394042968750, -0.0708007812500000,
+-0.0224914550781250, -0.0628356933593750, -0.0243530273437500, -0.0699462890625000,
+-0.0277099609375000, -0.0747070312500000, -0.0268249511718750, -0.0415649414062500,
+-0.0124206542968750, -0.0502319335937500, -0.0173950195312500, -0.0383300781250000,
+-0.0143737792968750, -0.0090942382812500, -0.0005493164062500, -0.0228576660156250,
+-0.0043029785156250, 0.0053405761718750, -0.0023803710937500, 0.0039978027343750,
+-0.0002136230468750, 0.0104675292968750, -0.0029296875000000, 0.0299987792968750,
+-0.0051879882812500, 0.0202331542968750, -0.0028686523437500, 0.0371704101562500,
+-0.0037536621093750, 0.0413208007812500, -0.0063171386718750, 0.0331115722656250,
+-0.0041503906250000, 0.0354614257812500, 0.0002441406250000, 0.0404052734375000,
+-0.0015258789062500, 0.0239562988281250, -0.0065307617187500, 0.0240173339843750,
+-0.0029907226562500, 0.0248718261718750, -0.0060729980468750, 0.0158691406250000,
+-0.0138854980468750, 0.0180969238281250, -0.0104980468750000, 0.0265197753906250,
+-0.0106811523437500, 0.0203552246093750, -0.0135192871093750, 0.0289916992187500,
+-0.0095214843750000, 0.0430297851562500, -0.0053405761718750, 0.0334472656250000,
+-0.0080871582031250, 0.0444335937500000, -0.0011596679687500, 0.0545349121093750,
+-0.0007629394531250, 0.0350952148437500, -0.0005187988281250, 0.0235595703125000,
+0.0028076171875000, 0.0384521484375000, 0.0018310546875000, -0.0015258789062500,
+0.0034484863281250, -0.0009765625000000, 0.0034484863281250, -0.0025329589843750,
+0.0036621093750000, -0.0237731933593750, 0.0047912597656250, -0.0159606933593750,
+0.0043640136718750, -0.0222778320312500, 0.0043334960937500, -0.0263671875000000,
+0.0043640136718750, -0.0201416015625000, 0.0046386718750000, -0.0186157226562500,
+0.0037231445312500, -0.0231933593750000, 0.0032958984375000, -0.0085754394531250,
+0.0083923339843750, -0.0088195800781250, 0.0061340332031250, -0.0113830566406250,
+0.0090332031250000, -0.0047302246093750, 0.0155639648437500, -0.0047912597656250,
+0.0122680664062500, -0.0105590820312500, 0.0149230957031250, -0.0086669921875000,
+0.0173645019531250, -0.0099487304687500, 0.0141601562500000, -0.0144958496093750,
+0.0117492675781250, -0.0122070312500000, 0.0139465332031250, -0.0153503417968750,
+0.0099792480468750, -0.0154113769531250, 0.0092773437500000, -0.0150756835937500,
+0.0096740722656250, -0.0170288085937500, 0.0091247558593750, -0.0161437988281250,
+0.0093994140625000, -0.0120239257812500, 0.0058593750000000, -0.0150146484375000,
+0.0070800781250000, -0.0105895996093750, 0.0055541992187500, -0.0039672851562500,
+0.0014343261718750, -0.0085449218750000, 0.0032653808593750, -0.0003662109375000,
+0.0036621093750000, -0.0001525878906250, 0.0022277832031250, -0.0006103515625000,
+0.0037231445312500, 0.0032958984375000, 0.0065612792968750, 0.0018005371093750,
+0.0055847167968750, 0.0011291503906250, 0.0044555664062500, 0.0029602050781250,
+0.0063476562500000, -0.0001831054687500, 0.0041809082031250, -0.0029907226562500,
+0.0010681152343750, -0.0001220703125000, 0.0031738281250000, -0.0074768066406250,
+-0.0004882812500000, -0.0062561035156250, -0.0002136230468750, -0.0078430175781250,
+-0.0011291503906250, -0.0124206542968750, -0.0030822753906250, -0.0090332031250000,
+-0.0015258789062500, -0.0163879394531250, -0.0070190429687500, -0.0139465332031250,
+-0.0061645507812500, -0.0161743164062500, -0.0078125000000000, -0.0225219726562500,
+-0.0122070312500000, -0.0183105468750000, -0.0101928710937500, -0.0244445800781250,
+-0.0145263671875000, -0.0242919921875000, -0.0146789550781250, -0.0235900878906250,
+-0.0148010253906250, -0.0259094238281250, -0.0168457031250000, -0.0240783691406250,
+-0.0162353515625000, -0.0241394042968750, -0.0177612304687500, -0.0237121582031250,
+-0.0167541503906250, -0.0228881835937500, -0.0191955566406250, -0.0224609375000000,
+-0.0216674804687500, -0.0217590332031250, -0.0196838378906250, -0.0206604003906250,
+-0.0281982421875000, -0.0206298828125000, -0.0275573730468750, -0.0190429687500000,
+-0.0286865234375000, -0.0173645019531250, -0.0350036621093750, -0.0175476074218750,
+-0.0333251953125000, -0.0140380859375000, -0.0323486328125000, -0.0134277343750000,
+-0.0346374511718750, -0.0122985839843750, -0.0325927734375000, -0.0097656250000000,
+-0.0290832519531250, -0.0094604492187500, -0.0308837890625000, -0.0070190429687500,
+-0.0288696289062500, -0.0070190429687500, -0.0281677246093750, -0.0050659179687500,
+-0.0274658203125000, -0.0024719238281250, -0.0269165039062500, -0.0033264160156250,
+-0.0267944335937500, 0.0010681152343750, -0.0203857421875000, 0.0009460449218750,
+-0.0227355957031250, 0.0021362304687500, -0.0179443359375000, 0.0054931640625000,
+-0.0107116699218750, 0.0045776367187500, -0.0151977539062500, 0.0070800781250000,
+-0.0039672851562500, 0.0075073242187500, -0.0041503906250000, 0.0063476562500000,
+-0.0042419433593750, 0.0071411132812500, 0.0020141601562500, 0.0075683593750000,
+0.0007934570312500, 0.0032958984375000, 0.0013122558593750, 0.0047607421875000,
+0.0010375976562500, 0.0014953613281250, 0.0040283203125000, -0.0032958984375000,
+0.0058898925781250, -0.0004882812500000, 0.0032958984375000, -0.0092468261718750,
+0.0115356445312500, -0.0083618164062500, 0.0122070312500000, -0.0098266601562500,
+0.0101928710937500, -0.0158996582031250, 0.0130615234375000, -0.0129394531250000,
+0.0141906738281250, -0.0153808593750000, 0.0102844238281250, -0.0169372558593750,
+0.0101928710937500, -0.0139160156250000, 0.0110168457031250, -0.0122070312500000,
+0.0101318359375000, -0.0133056640625000, 0.0095520019531250, -0.0076293945312500,
+0.0080261230468750, -0.0072326660156250, 0.0093078613281250, -0.0075073242187500,
+0.0064086914062500, -0.0042419433593750, 0.0027160644531250, -0.0048522949218750,
+0.0046691894531250, -0.0108032226562500, -0.0007629394531250, -0.0077209472656250,
+-0.0007019042968750, -0.0128784179687500, -0.0016479492187500, -0.0215759277343750,
+-0.0050659179687500, -0.0175476074218750, -0.0033264160156250, -0.0261840820312500,
+-0.0043640136718750, -0.0274353027343750, -0.0057983398437500, -0.0265502929687500,
+-0.0039367675781250, -0.0301513671875000, -0.0019531250000000, -0.0294494628906250,
+-0.0032348632812500, -0.0251464843750000, -0.0027465820312500, -0.0277709960937500,
+-0.0012512207031250, -0.0231933593750000, -0.0026550292968750, -0.0172119140625000,
+-0.0039978027343750, -0.0213012695312500, -0.0027160644531250, -0.0092163085937500,
+-0.0072021484375000, -0.0097351074218750, -0.0059204101562500, -0.0087280273437500,
+-0.0067138671875000, -0.0020446777343750, -0.0101013183593750, -0.0040893554687500,
+-0.0083618164062500, 0.0032043457031250, -0.0117187500000000, 0.0023803710937500,
+-0.0107116699218750, 0.0031127929687500, -0.0106201171875000, 0.0089111328125000,
+-0.0122070312500000, 0.0073852539062500, -0.0116882324218750, 0.0061340332031250,
+-0.0177001953125000, 0.0087890625000000, -0.0152587890625000, 0.0036926269531250,
+-0.0154113769531250, -0.0014648437500000, -0.0206298828125000, 0.0029602050781250,
+-0.0200805664062500, -0.0068054199218750, -0.0218505859375000, -0.0085449218750000,
+-0.0209350585937500, -0.0067749023437500, -0.0199279785156250, -0.0101928710937500,
+-0.0227050781250000, -0.0107116699218750, -0.0234680175781250, -0.0044555664062500,
+-0.0149841308593750, -0.0052185058593750, -0.0168151855468750, -0.0065307617187500,
+-0.0160522460937500, -0.0035705566406250, -0.0090637207031250, -0.0040588378906250,
+-0.0097961425781250, -0.0071411132812500, -0.0102844238281250, -0.0039978027343750,
+-0.0111694335937500, -0.0095214843750000, -0.0086059570312500, -0.0155334472656250,
+-0.0065917968750000, -0.0092468261718750, -0.0090332031250000, -0.0224304199218750,
+-0.0042114257812500, -0.0240478515625000, -0.0024108886718750, -0.0214538574218750,
+-0.0060729980468750, -0.0270080566406250, -0.0064086914062500, -0.0267333984375000,
+-0.0039672851562500, -0.0142517089843750, -0.0137634277343750, -0.0197448730468750,
+-0.0157165527343750, -0.0131530761718750, -0.0106201171875000, 0.0010986328125000,
+-0.0116271972656250, -0.0066223144531250, -0.0150146484375000, 0.0045776367187500,
+-0.0037231445312500, 0.0094299316406250, -0.0049438476562500, 0.0001525878906250,
+-0.0021362304687500, -0.0021972656250000, 0.0067443847656250, 0.0040283203125000,
+0.0022888183593750, -0.0140991210937500, 0.0022583007812500, -0.0158386230468750,
+0.0076904296875000, -0.0101318359375000, -0.0020141601562500, -0.0158996582031250,
+-0.0109863281250000, -0.0185546875000000, -0.0032958984375000, -0.0048522949218750,
+-0.0228576660156250, -0.0056457519531250, -0.0248718261718750, -0.0065002441406250,
+-0.0219726562500000, 0.0010375976562500, -0.0288696289062500, 0.0001220703125000,
+-0.0290527343750000, -0.0048828125000000, -0.0262451171875000, -0.0025329589843750,
+-0.0272521972656250, -0.0052185058593750, -0.0261535644531250, -0.0102844238281250,
+-0.0235290527343750, -0.0074462890625000, -0.0252990722656250, -0.0146484375000000,
+-0.0249328613281250, -0.0136108398437500, -0.0245056152343750, -0.0150146484375000,
+-0.0229797363281250, -0.0205993652343750, -0.0233154296875000, -0.0178222656250000,
+-0.0248413085937500, -0.0190734863281250, -0.0161743164062500, -0.0214538574218750,
+-0.0178833007812500, -0.0162353515625000, -0.0154724121093750, -0.0133056640625000,
+-0.0079650878906250, -0.0170593261718750, -0.0114440917968750, -0.0023803710937500,
+-0.0097656250000000, -0.0050964355468750, -0.0073852539062500, -0.0012512207031250,
+-0.0110473632812500, 0.0105590820312500, -0.0146789550781250, 0.0050354003906250,
+-0.0126647949218750, 0.0123291015625000, -0.0159912109375000, 0.0142211914062500,
+-0.0181579589843750, 0.0120849609375000, -0.0151977539062500, 0.0137634277343750,
+-0.0139160156250000, 0.0141296386718750, -0.0155639648437500, 0.0087585449218750,
+-0.0070800781250000, 0.0110778808593750, -0.0085754394531250, 0.0059509277343750,
+-0.0045776367187500, -0.0005187988281250, 0.0032043457031250, 0.0037231445312500,
+-0.0004272460937500, -0.0121459960937500, 0.0084228515625000, -0.0100402832031250,
+0.0090026855468750, -0.0145568847656250, 0.0092163085937500, -0.0276184082031250,
+0.0136718750000000, -0.0224914550781250, 0.0122985839843750, -0.0303649902343750,
+0.0120849609375000, -0.0338134765625000, 0.0133666992187500, -0.0311584472656250,
+0.0112915039062500, -0.0318298339843750, 0.0093994140625000, -0.0339660644531250,
+0.0108032226562500, -0.0297546386718750, 0.0049133300781250, -0.0305175781250000,
+0.0060424804687500, -0.0307006835937500, 0.0028076171875000, -0.0286865234375000,
+-0.0028076171875000, -0.0301818847656250, 0.0000000000000000, -0.0276794433593750,
+-0.0082397460937500, -0.0289001464843750, -0.0082397460937500, -0.0276489257812500,
+-0.0097045898437500},
+{-0.0076904296875000, -0.0063171386718750, -0.0082397460937500, -0.0053100585937500,
+-0.0080261230468750, -0.0057983398437500, -0.0079650878906250, -0.0026550292968750,
+0.0003356933593750, -0.0041198730468750, -0.0023498535156250, -0.0053405761718750,
+-0.0038146972656250, -0.0014038085937500, 0.0025634765625000, -0.0016784667968750,
+0.0019226074218750, -0.0146179199218750, -0.0054321289062500, -0.0095214843750000,
+-0.0033264160156250, -0.0137329101562500, -0.0055236816406250, -0.0270996093750000,
+-0.0136413574218750, -0.0212402343750000, -0.0109252929687500, -0.0245666503906250,
+-0.0045166015625000, -0.0291442871093750, -0.0090942382812500, -0.0234375000000000,
+-0.0056457519531250, -0.0191345214843750, 0.0033264160156250, -0.0228576660156250,
+-0.0002441406250000, -0.0104064941406250, 0.0043945312500000, -0.0121154785156250,
+0.0067138671875000, -0.0104675292968750, 0.0010681152343750, -0.0005493164062500,
+-0.0005798339843750, -0.0043334960937500, 0.0027160644531250, -0.0083312988281250,
+-0.0125732421875000, -0.0030822753906250, -0.0111389160156250, -0.0093383789062500,
+-0.0134582519531250, -0.0193786621093750, -0.0244750976562500, -0.0133056640625000,
+-0.0208740234375000, -0.0195007324218750, -0.0218200683593750, -0.0237731933593750,
+-0.0239868164062500, -0.0192565917968750, -0.0234985351562500, -0.0167541503906250,
+-0.0227355957031250, -0.0195007324218750, -0.0228576660156250, -0.0114440917968750,
+-0.0197448730468750, -0.0116882324218750, -0.0212707519531250, -0.0109863281250000,
+-0.0202636718750000, -0.0061035156250000, -0.0172424316406250, -0.0075378417968750,
+-0.0183410644531250, -0.0060119628906250, -0.0137329101562500, -0.0047912597656250,
+-0.0149841308593750, -0.0062255859375000, -0.0142517089843750, -0.0067749023437500,
+-0.0097961425781250, -0.0053405761718750, -0.0113830566406250, -0.0087585449218750,
+-0.0137634277343750, -0.0079345703125000, -0.0120849609375000, -0.0079956054687500,
+-0.0147399902343750, -0.0105285644531250, -0.0194396972656250, -0.0093078613281250,
+-0.0174560546875000, -0.0081787109375000, -0.0180358886718750, -0.0101928710937500,
+-0.0215454101562500, -0.0062561035156250, -0.0157470703125000, -0.0024719238281250,
+-0.0104980468750000, -0.0059814453125000, -0.0149230957031250, 0.0030517578125000,
+-0.0010070800781250, 0.0027465820312500, -0.0014038085937500, 0.0011596679687500,
+0.0007629394531250, 0.0057067871093750, 0.0095520019531250, 0.0043334960937500,
+0.0060119628906250, -0.0032653808593750, 0.0101928710937500, 0.0003967285156250,
+0.0115966796875000, -0.0064697265625000, 0.0095825195312500, -0.0164489746093750,
+0.0095520019531250, -0.0107421875000000, 0.0099487304687500, -0.0266113281250000,
+0.0062255859375000, -0.0255432128906250, 0.0062561035156250, -0.0295410156250000,
+0.0053405761718750, -0.0404968261718750, 0.0026550292968750, -0.0355224609375000,
+0.0034790039062500, -0.0475769042968750, 0.0028686523437500, -0.0480651855468750,
+0.0023498535156250, -0.0480651855468750, 0.0028381347656250, -0.0551147460937500,
+0.0032043457031250, -0.0534667968750000, 0.0029296875000000, -0.0497741699218750,
+0.0035400390625000, -0.0539245605468750, 0.0034484863281250, -0.0483093261718750,
+0.0038452148437500, -0.0411376953125000, 0.0046081542968750, -0.0466003417968750,
+0.0044250488281250, -0.0323486328125000, 0.0046081542968750, -0.0339660644531250,
+0.0054931640625000, -0.0309753417968750, 0.0047607421875000, -0.0209350585937500,
+0.0043029785156250, -0.0258178710937500, 0.0055847167968750, -0.0177917480468750,
+0.0025634765625000, -0.0169677734375000, 0.0035095214843750, -0.0176696777343750,
+0.0029296875000000, -0.0143737792968750, 0.0006103515625000, -0.0150756835937500,
+0.0020751953125000, -0.0152587890625000, 0.0000305175781250, -0.0148925781250000,
+0.0003356933593750, -0.0138854980468750, 0.0003662109375000, -0.0138244628906250,
+-0.0007934570312500, -0.0142822265625000, 0.0001525878906250, -0.0100402832031250,
+-0.0004882812500000, -0.0106811523437500, -0.0004577636718750, -0.0096130371093750,
+-0.0003051757812500, -0.0060119628906250, -0.0004272460937500, -0.0073242187500000,
+0.0000000000000000, -0.0075683593750000, -0.0000305175781250, -0.0054016113281250,
+0.0000915527343750, -0.0085754394531250, 0.0009155273437500, -0.0115356445312500,
+0.0011596679687500, -0.0088500976562500, 0.0007324218750000, -0.0172119140625000,
+0.0025634765625000, -0.0166320800781250, 0.0026550292968750, -0.0178527832031250,
+0.0025634765625000, -0.0235290527343750, 0.0040893554687500, -0.0216979980468750,
+0.0040588378906250, -0.0243225097656250, 0.0003662109375000, -0.0245971679687500,
+0.0036010742187500, -0.0258789062500000, -0.0007019042968750, -0.0274963378906250,
+-0.0062255859375000, -0.0263977050781250, -0.0008544921875000, -0.0327453613281250,
+-0.0126953125000000, -0.0322875976562500, -0.0106811523437500, -0.0343017578125000,
+-0.0118713378906250, -0.0398864746093750, -0.0199279785156250, -0.0384521484375000,
+-0.0158386230468750, -0.0397033691406250, -0.0196228027343750, -0.0418090820312500,
+-0.0221557617187500, -0.0395812988281250, -0.0184326171875000, -0.0384826660156250,
+-0.0174865722656250, -0.0405578613281250, -0.0200195312500000, -0.0320129394531250,
+-0.0093994140625000, -0.0354003906250000, -0.0127258300781250, -0.0295715332031250,
+-0.0077819824218750, -0.0202331542968750, 0.0030822753906250, -0.0267333984375000,
+-0.0033569335937500, -0.0109558105468750, 0.0046691894531250, -0.0115051269531250,
+0.0076293945312500, -0.0103149414062500, 0.0030517578125000, -0.0012207031250000,
+0.0032043457031250, -0.0043640136718750, 0.0051574707031250, 0.0006408691406250,
+-0.0063476562500000, -0.0000305175781250, -0.0032958984375000, 0.0030822753906250,
+-0.0080871582031250, 0.0077819824218750, -0.0186767578125000, 0.0044860839843750,
+-0.0137329101562500, 0.0133666992187500, -0.0260314941406250, 0.0133361816406250,
+-0.0261840820312500, 0.0132446289062500, -0.0257568359375000, 0.0180358886718750,
+-0.0338134765625000, 0.0170898437500000, -0.0320739746093750, 0.0170593261718750,
+-0.0218200683593750, 0.0175170898437500, -0.0268554687500000, 0.0180969238281250,
+-0.0202941894531250, 0.0186462402343750, -0.0072937011718750, 0.0180664062500000,
+-0.0131530761718750, 0.0171203613281250, -0.0025329589843750, 0.0203857421875000,
+-0.0019836425781250, 0.0131225585937500, -0.0003967285156250, 0.0075073242187500,
+0.0052490234375000, 0.0134277343750000, 0.0019531250000000, -0.0086059570312500,
+0.0080871582031250, -0.0060729980468750, 0.0094604492187500, -0.0094299316406250,
+0.0073852539062500, -0.0247497558593750, 0.0097045898437500, -0.0187072753906250,
+0.0099182128906250, -0.0245971679687500, -0.0022888183593750, -0.0261840820312500,
+0.0003967285156250, -0.0258789062500000, 0.0004272460937500, -0.0274658203125000,
+-0.0083923339843750, -0.0263977050781250, -0.0072326660156250, -0.0289916992187500,
+-0.0054016113281250, -0.0296630859375000, -0.0057983398437500, -0.0268859863281250,
+-0.0057678222656250, -0.0264587402343750, -0.0035095214843750, -0.0280761718750000,
+-0.0041503906250000, -0.0205688476562500, -0.0137939453125000, -0.0201416015625000,
+-0.0114746093750000, -0.0218811035156250, -0.0120544433593750, -0.0177612304687500,
+-0.0182189941406250, -0.0176086425781250, -0.0177001953125000, -0.0304565429687500,
+-0.0262451171875000, -0.0242614746093750, -0.0223388671875000, -0.0336608886718750,
+-0.0283203125000000, -0.0495910644531250, -0.0381164550781250, -0.0402832031250000,
+-0.0322570800781250, -0.0613403320312500, -0.0505065917968750, -0.0623168945312500,
+-0.0499572753906250, -0.0592651367187500, -0.0477905273437500, -0.0713806152343750,
+-0.0601501464843750, -0.0496826171875000, -0.0425720214843750, -0.0186767578125000,
+-0.0569458007812500, -0.0314636230468750, -0.0542602539062500, -0.0167846679687500,
+-0.0506591796875000, 0.0037231445312500, -0.0559082031250000, -0.0086364746093750,
+-0.0589904785156250, 0.0139465332031250, -0.0393676757812500, 0.0161132812500000,
+-0.0427246093750000, 0.0153503417968750, -0.0368347167968750, 0.0261840820312500,
+-0.0209960937500000, 0.0228881835937500, -0.0288696289062500, 0.0173339843750000,
+-0.0202941894531250, 0.0219421386718750, -0.0147094726562500, 0.0167541503906250,
+-0.0208129882812500, 0.0060729980468750, -0.0259399414062500, 0.0114440917968750,
+-0.0218505859375000, 0.0128479003906250, -0.0202941894531250, 0.0058898925781250,
+-0.0251464843750000, 0.0112915039062500, -0.0199584960937500, 0.0223083496093750,
+-0.0120239257812500, 0.0171203613281250, -0.0161437988281250, 0.0145874023437500,
+-0.0093078613281250, 0.0204772949218750, -0.0079650878906250, 0.0119934082031250,
+-0.0096740722656250, 0.0031127929687500, -0.0081176757812500, 0.0105895996093750,
+-0.0077514648437500, -0.0101623535156250, -0.0095520019531250, -0.0088500976562500,
+-0.0101623535156250, -0.0098876953125000, -0.0091552734375000, -0.0241394042968750,
+-0.0088500976562500, -0.0196533203125000, -0.0096435546875000, -0.0115356445312500,
+-0.0083312988281250, -0.0169982910156250, -0.0082702636718750, -0.0164184570312500,
+-0.0070495605468750, -0.0069274902343750, -0.0068359375000000, -0.0086059570312500,
+-0.0075683593750000, -0.0144348144531250, 0.0010986328125000, -0.0120544433593750,
+-0.0018615722656250, -0.0157165527343750, 0.0030822753906250, -0.0212402343750000,
+0.0131225585937500, -0.0175170898437500, 0.0082702636718750, -0.0281982421875000,
+0.0157470703125000, -0.0270690917968750, 0.0184631347656250, -0.0278625488281250,
+0.0149230957031250, -0.0364379882812500, 0.0151672363281250, -0.0335388183593750,
+0.0169372558593750, -0.0260009765625000, 0.0091552734375000, -0.0324096679687500,
+0.0109252929687500, -0.0259399414062500, 0.0075073242187500, -0.0135803222656250,
+0.0010986328125000, -0.0206604003906250, 0.0050964355468750, -0.0109863281250000,
+-0.0061645507812500, -0.0083923339843750, -0.0050354003906250, -0.0115661621093750,
+-0.0068969726562500, -0.0092468261718750, -0.0145874023437500, -0.0081176757812500,
+-0.0114440917968750, -0.0163269042968750, -0.0162048339843750, -0.0132141113281250,
+-0.0160522460937500, -0.0178222656250000, -0.0179138183593750, -0.0264892578125000,
+-0.0208129882812500, -0.0207214355468750, -0.0184631347656250, -0.0302429199218750,
+-0.0274658203125000, -0.0320434570312500, -0.0260009765625000, -0.0286560058593750,
+-0.0281372070312500, -0.0308837890625000, -0.0350341796875000, -0.0319824218750000,
+-0.0319519042968750, -0.0237426757812500, -0.0364074707031250, -0.0265502929687500,
+-0.0365905761718750, -0.0226745605468750, -0.0369262695312500, -0.0157470703125000,
+-0.0395507812500000, -0.0208435058593750, -0.0380859375000000, -0.0080871582031250,
+-0.0382995605468750, -0.0104675292968750, -0.0402221679687500, -0.0069274902343750,
+-0.0362243652343750, 0.0030517578125000, -0.0330200195312500, -0.0026550292968750,
+-0.0360107421875000, 0.0063171386718750, -0.0262451171875000, 0.0070495605468750,
+-0.0265502929687500, 0.0059509277343750, -0.0253295898437500, 0.0096740722656250,
+-0.0197753906250000, 0.0086975097656250, -0.0217285156250000, 0.0070495605468750,
+-0.0162048339843750, 0.0089111328125000, -0.0175781250000000, 0.0067443847656250,
+-0.0140075683593750, 0.0039978027343750, -0.0078735351562500, 0.0063781738281250,
+-0.0113525390625000, 0.0002746582031250, -0.0037841796875000, 0.0016479492187500,
+-0.0023803710937500, 0.0003051757812500, -0.0043640136718750, -0.0039672851562500,
+-0.0025024414062500, -0.0007629394531250, -0.0021362304687500, -0.0046997070312500,
+-0.0067443847656250, -0.0039062500000000, -0.0066833496093750, -0.0036926269531250,
+-0.0070495605468750, -0.0055847167968750, -0.0096130371093750, -0.0034484863281250,
+-0.0091857910156250},
+{-0.0155639648437500, 0.0105590820312500, -0.0213012695312500, 0.0049438476562500,
+-0.0242309570312500, 0.0120239257812500, -0.0209655761718750, 0.0013122558593750,
+-0.0210876464843750, -0.0141296386718750, -0.0259704589843750, -0.0043029785156250,
+-0.0255737304687500, -0.0167541503906250, -0.0160827636718750, -0.0213012695312500,
+-0.0205383300781250, -0.0163574218750000, -0.0139465332031250, -0.0177917480468750,
+-0.0015258789062500, -0.0203552246093750, -0.0082702636718750, -0.0083618164062500,
+0.0007019042968750, -0.0106811523437500, 0.0036621093750000, -0.0103149414062500,
+0.0007629394531250, 0.0001220703125000, 0.0009155273437500, -0.0034484863281250,
+0.0016784667968750, -0.0181274414062500, 0.0016479492187500, -0.0096130371093750,
+0.0007934570312500, -0.0163879394531250, 0.0009765625000000, -0.0359191894531250,
+0.0025939941406250, -0.0288085937500000, 0.0025329589843750, -0.0321044921875000,
+-0.0013732910156250, -0.0360107421875000, -0.0000610351562500, -0.0327758789062500,
+-0.0007019042968750, -0.0308532714843750, -0.0047607421875000, -0.0329284667968750,
+-0.0033264160156250, -0.0259704589843750, 0.0000305175781250, -0.0281066894531250,
+-0.0018005371093750, -0.0241394042968750, 0.0006103515625000, -0.0165405273437500,
+0.0061035156250000, -0.0213928222656250, 0.0042724609375000, -0.0152282714843750,
+0.0032348632812500, -0.0123901367187500, 0.0064392089843750, -0.0156555175781250,
+0.0019836425781250, -0.0163879394531250, -0.0028076171875000, -0.0144653320312500,
+0.0009155273437500, -0.0192871093750000, -0.0097656250000000, -0.0190429687500000,
+-0.0079956054687500, -0.0189208984375000, -0.0117797851562500, -0.0220642089843750,
+-0.0201110839843750, -0.0211791992187500, -0.0153198242187500, -0.0182800292968750,
+-0.0268554687500000, -0.0210571289062500, -0.0266113281250000, -0.0167846679687500,
+-0.0272216796875000, -0.0115356445312500, -0.0335388183593750, -0.0159606933593750,
+-0.0314331054687500, -0.0050048828125000, -0.0348815917968750, -0.0050659179687500,
+-0.0340270996093750, -0.0054016113281250, -0.0369873046875000, 0.0000915527343750,
+-0.0407104492187500, -0.0014953613281250, -0.0375671386718750, -0.0017089843750000,
+-0.0464782714843750, -0.0007934570312500, -0.0462341308593750, -0.0016784667968750,
+-0.0462951660156250, -0.0023803710937500, -0.0513000488281250, -0.0016784667968750,
+-0.0498962402343750, -0.0042114257812500, -0.0495605468750000, -0.0025329589843750,
+-0.0505981445312500, -0.0050354003906250, -0.0488281250000000, -0.0083923339843750,
+-0.0478210449218750, -0.0052795410156250, -0.0488586425781250, -0.0130004882812500,
+-0.0409240722656250, -0.0120239257812500, -0.0441284179687500, -0.0129699707031250,
+-0.0377502441406250, -0.0178833007812500, -0.0278320312500000, -0.0154418945312500,
+-0.0337524414062500, -0.0189819335937500, -0.0196533203125000, -0.0195312500000000,
+-0.0190734863281250, -0.0191650390625000, -0.0186767578125000, -0.0207214355468750,
+-0.0118713378906250, -0.0206909179687500, -0.0142822265625000, -0.0203247070312500,
+-0.0104980468750000, -0.0210876464843750, -0.0114135742187500, -0.0206298828125000,
+-0.0090637207031250, -0.0200805664062500, -0.0053710937500000, -0.0213317871093750,
+-0.0080566406250000, -0.0197448730468750, -0.0006103515625000, -0.0208740234375000,
+-0.0004882812500000, -0.0201110839843750, -0.0011901855468750, -0.0186462402343750,
+0.0028686523437500, -0.0205688476562500, 0.0023498535156250, -0.0174560546875000,
+-0.0016784667968750, -0.0185241699218750, 0.0006408691406250, -0.0178527832031250,
+-0.0021362304687500, -0.0156555175781250, -0.0069274902343750, -0.0177612304687500,
+-0.0039062500000000, -0.0146789550781250, -0.0107421875000000, -0.0159301757812500,
+-0.0093688964843750, -0.0152587890625000, -0.0129699707031250, -0.0129394531250000,
+-0.0188293457031250, -0.0151977539062500, -0.0152587890625000, -0.0119934082031250,
+-0.0280761718750000, -0.0125122070312500, -0.0258789062500000, -0.0124816894531250,
+-0.0307006835937500, -0.0106506347656250, -0.0413208007812500, -0.0115661621093750,
+-0.0356140136718750, -0.0093994140625000, -0.0476379394531250, -0.0096130371093750,
+-0.0484313964843750, -0.0086059570312500, -0.0474243164062500, -0.0068054199218750,
+-0.0527954101562500, -0.0075683593750000, -0.0520019531250000, -0.0044860839843750,
+-0.0503845214843750, -0.0046997070312500, -0.0505371093750000, -0.0029907226562500,
+-0.0512390136718750, -0.0004272460937500, -0.0508117675781250, -0.0015563964843750,
+-0.0496520996093750, 0.0038757324218750, -0.0502319335937500, 0.0030212402343750,
+-0.0520935058593750, 0.0044555664062500, -0.0479431152343750, 0.0086975097656250,
+-0.0451354980468750, 0.0064392089843750, -0.0486450195312500, 0.0081787109375000,
+-0.0380554199218750, 0.0089111328125000, -0.0400695800781250, 0.0064392089843750,
+-0.0363464355468750, 0.0053100585937500, -0.0287475585937500, 0.0054626464843750,
+-0.0336914062500000, -0.0011901855468750, -0.0183105468750000, -0.0005187988281250,
+-0.0209350585937500, -0.0037231445312500, -0.0158996582031250, -0.0098876953125000,
+-0.0030517578125000, -0.0075073242187500, -0.0091552734375000, -0.0122070312500000,
+0.0023193359375000, -0.0142822265625000, 0.0038757324218750, -0.0107727050781250,
+0.0035705566406250, -0.0102233886718750, 0.0083618164062500, -0.0125427246093750,
+0.0075988769531250, -0.0026245117187500, 0.0090942382812500, -0.0032348632812500,
+0.0090332031250000, -0.0018310546875000, 0.0101623535156250, 0.0046386718750000,
+0.0119628906250000, 0.0022583007812500, 0.0104370117187500, 0.0042419433593750,
+0.0107421875000000, 0.0067749023437500, 0.0140380859375000, 0.0017700195312500,
+0.0065307617187500, -0.0012512207031250, 0.0015563964843750, 0.0023498535156250,
+0.0083007812500000, -0.0093383789062500, -0.0122985839843750, -0.0088806152343750,
+-0.0102844238281250, -0.0097656250000000, -0.0126647949218750, -0.0167846679687500,
+-0.0260314941406250, -0.0141296386718750, -0.0203552246093750, -0.0145263671875000,
+-0.0253295898437500, -0.0156860351562500, -0.0293273925781250, -0.0154418945312500,
+-0.0236206054687500, -0.0144958496093750, -0.0190429687500000, -0.0139160156250000,
+-0.0234985351562500, -0.0150451660156250, -0.0219116210937500, -0.0169982910156250,
+-0.0190734863281250, -0.0137329101562500, -0.0195922851562500, -0.0106811523437500,
+-0.0216369628906250, -0.0130615234375000, -0.0221862792968750, -0.0071105957031250,
+-0.0218200683593750, -0.0078735351562500, -0.0217895507812500, -0.0054016113281250,
+-0.0194396972656250, -0.0000915527343750, -0.0190429687500000, -0.0028076171875000,
+-0.0209655761718750, 0.0017089843750000, -0.0111389160156250, 0.0034484863281250,
+-0.0132751464843750, 0.0013732910156250, -0.0107421875000000, 0.0012817382812500,
+-0.0023803710937500, 0.0032348632812500, -0.0056457519531250, 0.0014953613281250,
+-0.0012512207031250, 0.0002441406250000, -0.0004577636718750, 0.0024719238281250,
+-0.0022888183593750, 0.0054321289062500, -0.0018005371093750, 0.0052185058593750,
+-0.0016784667968750, 0.0041198730468750, -0.0037536621093750, 0.0062561035156250,
+-0.0051879882812500, 0.0028076171875000, -0.0026245117187500, -0.0013122558593750,
+-0.0014038085937500, 0.0026855468750000, -0.0034484863281250, -0.0015258789062500,
+0.0041503906250000, -0.0039367675781250, 0.0036621093750000, -0.0048217773437500,
+0.0032653808593750, -0.0032653808593750, 0.0072937011718750, -0.0021667480468750,
+0.0067138671875000, -0.0163574218750000, 0.0067138671875000, -0.0112609863281250,
+0.0070495605468750, -0.0174865722656250, 0.0046386718750000, -0.0325927734375000,
+0.0040283203125000, -0.0249938964843750, 0.0058898925781250, -0.0333251953125000,
+-0.0019531250000000, -0.0374450683593750, -0.0011291503906250, -0.0324401855468750,
+-0.0036315917968750, -0.0317382812500000, -0.0095520019531250, -0.0343017578125000,
+-0.0061645507812500, -0.0225830078125000, -0.0109252929687500, -0.0238037109375000,
+-0.0130310058593750, -0.0241088867187500, -0.0108337402343750, -0.0169372558593750,
+-0.0106201171875000, -0.0180969238281250, -0.0116882324218750, -0.0198974609375000,
+-0.0072631835937500, -0.0194702148437500, -0.0069885253906250, -0.0177001953125000,
+-0.0099487304687500, -0.0182800292968750, -0.0091247558593750, -0.0194702148437500,
+-0.0070190429687500, -0.0129394531250000, -0.0135498046875000, -0.0134582519531250,
+-0.0146484375000000, -0.0132141113281250, -0.0132446289062500, -0.0099182128906250,
+-0.0150146484375000, -0.0105285644531250, -0.0148925781250000, -0.0064697265625000,
+-0.0105285644531250, -0.0086364746093750, -0.0129394531250000, -0.0041503906250000,
+-0.0084228515625000, 0.0021057128906250, -0.0014953613281250, -0.0023193359375000,
+-0.0056762695312500, 0.0068054199218750, 0.0010070800781250, 0.0081787109375000,
+0.0030517578125000, 0.0046997070312500, 0.0003967285156250, 0.0075683593750000,
+0.0008850097656250, 0.0088500976562500, 0.0019531250000000, -0.0025939941406250,
+-0.0030517578125000, 0.0015258789062500, -0.0033874511718750, -0.0049438476562500,
+-0.0025329589843750, -0.0166625976562500, -0.0049133300781250, -0.0098266601562500,
+-0.0050659179687500, -0.0262756347656250, 0.0002746582031250, -0.0256958007812500,
+-0.0016479492187500, -0.0281066894531250, 0.0021362304687500, -0.0385742187500000,
+0.0083312988281250, -0.0344543457031250, 0.0049133300781250, -0.0412597656250000,
+0.0127258300781250, -0.0426025390625000, 0.0134277343750000, -0.0422363281250000,
+0.0126037597656250, -0.0449523925781250, 0.0159606933593750, -0.0447387695312500,
+0.0152893066406250, -0.0444946289062500, 0.0123596191406250, -0.0451965332031250,
+0.0132751464843750, -0.0452575683593750, 0.0105895996093750, -0.0444946289062500,
+0.0064392089843750, -0.0446166992187500, 0.0075683593750000, -0.0456542968750000,
+0.0029296875000000, -0.0450744628906250, 0.0021667480468750, -0.0451660156250000,
+0.0018615722656250, -0.0462341308593750, -0.0003662109375000, -0.0457153320312500,
+0.0000610351562500, -0.0452880859375000, 0.0000305175781250, -0.0455627441406250,
+-0.0007019042968750, -0.0447387695312500, 0.0004577636718750, -0.0440673828125000,
+0.0012817382812500, -0.0440979003906250, 0.0000915527343750, -0.0404052734375000,
+0.0031738281250000, -0.0422058105468750, 0.0028076171875000, -0.0378417968750000,
+0.0033874511718750, -0.0330505371093750, 0.0056152343750000, -0.0368347167968750,
+0.0048522949218750, -0.0240478515625000, 0.0064392089843750, -0.0263061523437500,
+0.0068054199218750, -0.0223083496093750, 0.0065917968750000, -0.0122375488281250,
+0.0072326660156250, -0.0175170898437500, 0.0072631835937500, -0.0069580078125000,
+0.0063476562500000, -0.0068664550781250, 0.0071411132812500, -0.0063781738281250,
+0.0065917968750000, -0.0004577636718750, 0.0052490234375000, -0.0027770996093750,
+0.0060424804687500, -0.0019226074218750, 0.0050659179687500, -0.0001525878906250,
+0.0049438476562500, -0.0025024414062500, 0.0046997070312500, -0.0042419433593750,
+0.0043945312500000, -0.0026550292968750, 0.0048522949218750, -0.0078430175781250,
+0.0026855468750000, -0.0070495605468750, 0.0036010742187500, -0.0087585449218750,
+0.0027770996093750, -0.0122375488281250, 0.0007019042968750, -0.0103454589843750,
+0.0021362304687500, -0.0185546875000000, 0.0000000000000000, -0.0158081054687500,
+0.0006408691406250, -0.0215454101562500, -0.0003356933593750, -0.0305786132812500,
+-0.0018920898437500, -0.0250549316406250, -0.0003356933593750, -0.0400085449218750,
+-0.0052795410156250, -0.0396118164062500, -0.0042114257812500, -0.0410156250000000,
+-0.0047607421875000, -0.0498962402343750, -0.0086364746093750, -0.0465698242187500,
+-0.0068359375000000, -0.0504455566406250, -0.0066528320312500, -0.0519409179687500,
+-0.0081787109375000},
+{-0.0034179687500000, -0.0137939453125000, -0.0044250488281250, -0.0097656250000000,
+-0.0033264160156250, -0.0042114257812500, 0.0022888183593750, -0.0078125000000000,
+0.0002441406250000, -0.0042419433593750, 0.0024719238281250, -0.0022277832031250,
+0.0045776367187500, -0.0039978027343750, 0.0009460449218750, -0.0050354003906250,
+-0.0013732910156250, -0.0039672851562500, 0.0016174316406250, -0.0047912597656250,
+-0.0042419433593750, -0.0057678222656250, -0.0054626464843750, -0.0050659179687500,
+-0.0027160644531250, -0.0039062500000000, -0.0034484863281250, -0.0048522949218750,
+-0.0045471191406250, -0.0068359375000000, 0.0024414062500000, -0.0041503906250000,
+0.0018310546875000, -0.0104370117187500, 0.0036621093750000, -0.0159606933593750,
+0.0092163085937500, -0.0111083984375000, 0.0075378417968750, -0.0285949707031250,
+0.0096435546875000, -0.0255126953125000, 0.0109252929687500, -0.0304870605468750,
+0.0092773437500000, -0.0443725585937500, 0.0086669921875000, -0.0372314453125000,
+0.0094909667968750, -0.0477600097656250, 0.0054626464843750, -0.0496826171875000,
+0.0057983398437500, -0.0475158691406250, 0.0050964355468750, -0.0505065917968750,
+0.0020751953125000, -0.0503234863281250, 0.0028381347656250, -0.0470581054687500,
+0.0020446777343750, -0.0484619140625000, 0.0010375976562500, -0.0461425781250000,
+0.0018615722656250, -0.0424804687500000, 0.0025939941406250, -0.0447387695312500,
+0.0020751953125000, -0.0389404296875000, 0.0041503906250000, -0.0389404296875000,
+0.0041198730468750, -0.0385131835937500, 0.0043334960937500, -0.0350341796875000,
+0.0053405761718750, -0.0358886718750000, 0.0044860839843750, -0.0350646972656250,
+0.0050048828125000, -0.0346069335937500, 0.0043640136718750, -0.0342102050781250,
+0.0036010742187500, -0.0337829589843750, 0.0033264160156250, -0.0338134765625000,
+0.0023193359375000, -0.0316467285156250, 0.0021667480468750, -0.0317687988281250,
+0.0010986328125000, -0.0305786132812500, 0.0004882812500000, -0.0289611816406250,
+0.0002441406250000, -0.0296020507812500, -0.0009155273437500, -0.0247802734375000,
+-0.0011291503906250, -0.0265502929687500, -0.0021362304687500, -0.0227355957031250,
+-0.0032348632812500, -0.0172119140625000, -0.0038757324218750, -0.0210266113281250,
+-0.0045166015625000, -0.0111694335937500, -0.0048828125000000, -0.0119323730468750,
+-0.0065307617187500, -0.0109252929687500, -0.0062561035156250, -0.0047302246093750,
+-0.0054931640625000, -0.0075073242187500, -0.0070495605468750, -0.0054016113281250,
+-0.0051574707031250, -0.0042419433593750, -0.0057067871093750, -0.0060424804687500,
+-0.0055236816406250, -0.0062561035156250, -0.0043945312500000, -0.0056762695312500,
+-0.0053405761718750, -0.0111083984375000, -0.0030517578125000, -0.0089721679687500,
+-0.0039062500000000, -0.0130615234375000, -0.0032043457031250, -0.0191040039062500,
+-0.0012207031250000, -0.0151672363281250, -0.0024719238281250, -0.0262451171875000,
+0.0005187988281250, -0.0256652832031250, -0.0001525878906250, -0.0271911621093750,
+0.0015869140625000, -0.0341491699218750, 0.0041198730468750, -0.0314025878906250,
+0.0020141601562500, -0.0361022949218750, 0.0083923339843750, -0.0359497070312500,
+0.0075378417968750, -0.0368041992187500, 0.0085449218750000, -0.0397033691406250,
+0.0133056640625000, -0.0378112792968750, 0.0113525390625000, -0.0407104492187500,
+0.0126342773437500, -0.0410766601562500, 0.0140380859375000, -0.0401611328125000,
+0.0122375488281250, -0.0405578613281250, 0.0115966796875000, -0.0403137207031250,
+0.0130615234375000, -0.0400390625000000, 0.0066223144531250, -0.0397033691406250,
+0.0086059570312500, -0.0400390625000000, 0.0059814453125000, -0.0405578613281250,
+0.0000000000000000, -0.0399780273437500, 0.0034484863281250, -0.0390625000000000,
+-0.0029296875000000, -0.0398864746093750, -0.0026855468750000, -0.0382385253906250,
+-0.0016479492187500, -0.0363159179687500, -0.0051879882812500, -0.0375671386718750,
+-0.0043334960937500, -0.0339660644531250, 0.0002136230468750, -0.0344543457031250,
+-0.0019531250000000, -0.0322570800781250, 0.0012207031250000, -0.0292053222656250,
+0.0066833496093750, -0.0310668945312500, 0.0032958984375000, -0.0234069824218750,
+0.0089721679687500, -0.0245971679687500, 0.0107727050781250, -0.0229797363281250,
+0.0070190429687500, -0.0169372558593750, 0.0071411132812500, -0.0193176269531250,
+0.0092163085937500, -0.0174560546875000, -0.0009765625000000, -0.0162658691406250,
+0.0006103515625000, -0.0167846679687500, -0.0037841796875000, -0.0174255371093750,
+-0.0133056640625000, -0.0172119140625000, -0.0085754394531250, -0.0145874023437500,
+-0.0133361816406250, -0.0158386230468750, -0.0169067382812500, -0.0141906738281250,
+-0.0129394531250000, -0.0107116699218750, -0.0099487304687500, -0.0122070312500000,
+-0.0122070312500000, -0.0096740722656250, -0.0077819824218750, -0.0095825195312500,
+-0.0060424804687500, -0.0082397460937500, -0.0085449218750000, -0.0066223144531250,
+-0.0090637207031250, -0.0079650878906250, -0.0074768066406250, -0.0024108886718750,
+-0.0118408203125000, -0.0022277832031250, -0.0113525390625000, -0.0037536621093750,
+-0.0117797851562500, -0.0006713867187500, -0.0152282714843750, -0.0002441406250000,
+-0.0141601562500000, -0.0097045898437500, -0.0120544433593750, -0.0062561035156250,
+-0.0147094726562500, -0.0117492675781250, -0.0079650878906250, -0.0216979980468750,
+-0.0023193359375000, -0.0159912109375000, -0.0086059570312500, -0.0289001464843750,
+0.0090637207031250, -0.0291442871093750, 0.0112304687500000, -0.0289916992187500,
+0.0070495605468750, -0.0356140136718750, 0.0133666992187500, -0.0341186523437500,
+0.0133361816406250, -0.0360107421875000, -0.0035400390625000, -0.0354309082031250,
+0.0032043457031250, -0.0371704101562500, -0.0039062500000000, -0.0409851074218750,
+-0.0239562988281250, -0.0387268066406250, -0.0165100097656250, -0.0353088378906250,
+-0.0189208984375000, -0.0401306152343750, -0.0230712890625000, -0.0327758789062500,
+-0.0187072753906250, -0.0234069824218750, -0.0141601562500000, -0.0297241210937500,
+-0.0178527832031250, -0.0155334472656250, -0.0173034667968750, -0.0155639648437500,
+-0.0118713378906250, -0.0134277343750000, -0.0196533203125000, 0.0007019042968750,
+-0.0356140136718750, 0.0013122558593750, -0.0221862792968750, 0.0028076171875000,
+-0.0263366699218750, 0.0003662109375000, -0.0196228027343750, 0.0024719238281250,
+-0.0314331054687500, 0.0077209472656250, -0.0419006347656250, 0.0057983398437500,
+-0.0328063964843750, -0.0039978027343750, -0.0535888671875000, 0.0019226074218750,
+-0.0558776855468750, -0.0057983398437500, -0.0522155761718750, -0.0209350585937500,
+-0.0602722167968750, -0.0128479003906250, -0.0600891113281250, -0.0193176269531250,
+-0.0479736328125000, -0.0265197753906250, -0.0509948730468750, -0.0176696777343750,
+-0.0487976074218750, -0.0110778808593750, -0.0398559570312500, -0.0176086425781250,
+-0.0426330566406250, -0.0003662109375000, -0.0352478027343750, 0.0014343261718750,
+-0.0375061035156250, -0.0048828125000000, -0.0330200195312500, -0.0007934570312500,
+-0.0255126953125000, 0.0022888183593750, -0.0303649902343750, -0.0071411132812500,
+-0.0169372558593750, -0.0065002441406250, -0.0171508789062500, -0.0106811523437500,
+-0.0174560546875000, -0.0168762207031250, -0.0100708007812500, -0.0120239257812500,
+-0.0111694335937500, -0.0261230468750000, -0.0114440917968750, -0.0267639160156250,
+-0.0127258300781250, -0.0237426757812500, -0.0077514648437500, -0.0294494628906250,
+-0.0043945312500000, -0.0307617187500000, -0.0086059570312500, -0.0248107910156250,
+0.0042724609375000, -0.0225830078125000, 0.0051269531250000, -0.0265502929687500,
+0.0030212402343750, -0.0292053222656250, 0.0085144042968750, -0.0260620117187500,
+0.0085449218750000, -0.0247497558593750, 0.0010070800781250, -0.0291137695312500,
+0.0032043457031250, -0.0222778320312500, 0.0004272460937500, -0.0151977539062500,
+-0.0061340332031250, -0.0209350585937500, -0.0031127929687500, -0.0046386718750000,
+-0.0099182128906250, -0.0056762695312500, -0.0086669921875000, -0.0050659179687500,
+-0.0119628906250000, 0.0051269531250000, -0.0179138183593750, 0.0015563964843750,
+-0.0140075683593750, -0.0005493164062500, -0.0234680175781250, 0.0035705566406250,
+-0.0237121582031250, -0.0007019042968750, -0.0230407714843750, -0.0074157714843750,
+-0.0272827148437500, -0.0031127929687500, -0.0261535644531250, -0.0070190429687500,
+-0.0254821777343750, -0.0092468261718750, -0.0249023437500000, -0.0064697265625000,
+-0.0267333984375000, -0.0053405761718750, -0.0273742675781250, -0.0067443847656250,
+-0.0255737304687500, -0.0017395019531250, -0.0331420898437500, -0.0031738281250000,
+-0.0317382812500000, 0.0002746582031250, -0.0336608886718750, 0.0048828125000000,
+-0.0394897460937500, 0.0010986328125000, -0.0368041992187500, 0.0126647949218750,
+-0.0422668457031250, 0.0115051269531250, -0.0418395996093750, 0.0122680664062500,
+-0.0427856445312500, 0.0199279785156250, -0.0467224121093750, 0.0169372558593750,
+-0.0447692871093750, 0.0162048339843750, -0.0458984375000000, 0.0191650390625000,
+-0.0477600097656250, 0.0146179199218750, -0.0449829101562500, 0.0101623535156250,
+-0.0426940917968750, 0.0139160156250000, -0.0447692871093750, 0.0036926269531250,
+-0.0388488769531250, 0.0052185058593750, -0.0391845703125000, 0.0024108886718750,
+-0.0383300781250000, -0.0047607421875000, -0.0347900390625000, -0.0002746582031250,
+-0.0365600585937500, -0.0092773437500000, -0.0336303710937500, -0.0083923339843750,
+-0.0336914062500000, -0.0096130371093750, -0.0339660644531250, -0.0152587890625000,
+-0.0330810546875000, -0.0120239257812500, -0.0338439941406250, -0.0169372558593750,
+-0.0316467285156250, -0.0169372558593750, -0.0339660644531250, -0.0166015625000000,
+-0.0298767089843750, -0.0190429687500000, -0.0255737304687500, -0.0181579589843750,
+-0.0293579101562500, -0.0184936523437500, -0.0166625976562500, -0.0190429687500000,
+-0.0175781250000000, -0.0188293457031250, -0.0153808593750000, -0.0186462402343750,
+-0.0066223144531250, -0.0191040039062500, -0.0095214843750000, -0.0194396972656250,
+-0.0046691894531250, -0.0195617675781250, -0.0040283203125000, -0.0200805664062500,
+-0.0033569335937500, -0.0206604003906250, -0.0007324218750000, -0.0210266113281250,
+-0.0018920898437500, -0.0221557617187500, -0.0008239746093750, -0.0222167968750000,
+0.0004882812500000, -0.0234069824218750, -0.0021667480468750, -0.0246887207031250,
+-0.0035705566406250, -0.0245361328125000, -0.0018005371093750, -0.0277709960937500,
+-0.0097656250000000, -0.0276489257812500, -0.0083923339843750, -0.0293273925781250,
+-0.0111389160156250, -0.0322570800781250, -0.0179138183593750, -0.0311889648437500,
+-0.0149841308593750, -0.0338745117187500, -0.0212097167968750, -0.0346679687500000,
+-0.0214538574218750, -0.0339355468750000, -0.0227966308593750, -0.0346374511718750,
+-0.0261230468750000, -0.0348815917968750, -0.0245361328125000, -0.0323486328125000,
+-0.0314636230468750, -0.0336914062500000, -0.0294799804687500, -0.0306396484375000,
+-0.0324096679687500, -0.0275573730468750, -0.0391540527343750, -0.0300598144531250,
+-0.0354003906250000, -0.0191955566406250, -0.0415649414062500, -0.0216979980468750,
+-0.0423889160156250, -0.0178527832031250, -0.0422058105468750, -0.0084533691406250,
+-0.0450744628906250, -0.0131835937500000, -0.0445556640625000, -0.0037231445312500,
+-0.0441589355468750, -0.0037841796875000, -0.0463256835937500, -0.0028686523437500,
+-0.0427246093750000, 0.0025329589843750, -0.0395202636718750, 0.0000000000000000,
+-0.0430297851562500, 0.0030517578125000, -0.0334167480468750, 0.0036926269531250,
+-0.0335998535156250, 0.0027770996093750, -0.0328979492187500, 0.0034484863281250,
+-0.0273132324218750},
+{-0.0229187011718750, -0.0007934570312500, -0.0294799804687500, -0.0076293945312500,
+-0.0254516601562500, 0.0092163085937500, -0.0334777832031250, 0.0088195800781250,
+-0.0341796875000000, 0.0093078613281250, -0.0331115722656250, 0.0181579589843750,
+-0.0359802246093750, 0.0146484375000000, -0.0354003906250000, 0.0184631347656250,
+-0.0344848632812500, 0.0198059082031250, -0.0341796875000000, 0.0169677734375000,
+-0.0348205566406250, 0.0176696777343750, -0.0349426269531250, 0.0189514160156250,
+-0.0342407226562500, 0.0083007812500000, -0.0355224609375000, 0.0114135742187500,
+-0.0357055664062500, 0.0069580078125000, -0.0350952148437500, -0.0029907226562500,
+-0.0351257324218750, 0.0026855468750000, -0.0353393554687500, -0.0066528320312500,
+-0.0346679687500000, -0.0075683593750000, -0.0348510742187500, -0.0063781738281250,
+-0.0339965820312500, -0.0103759765625000, -0.0335388183593750, -0.0097351074218750,
+-0.0342407226562500, -0.0072631835937500, -0.0298767089843750, -0.0093688964843750,
+-0.0311279296875000, -0.0066528320312500, -0.0298461914062500, -0.0033874511718750,
+-0.0260620117187500, -0.0066528320312500, -0.0280456542968750, 0.0017395019531250,
+-0.0263671875000000, -0.0004577636718750, -0.0263977050781250, 0.0020446777343750,
+-0.0260009765625000, 0.0084838867187500, -0.0254516601562500, 0.0039978027343750,
+-0.0266723632812500, 0.0129699707031250, -0.0223693847656250, 0.0116271972656250,
+-0.0230407714843750, 0.0124816894531250, -0.0223693847656250, 0.0182800292968750,
+-0.0193481445312500, 0.0151367187500000, -0.0204162597656250, 0.0184020996093750,
+-0.0184020996093750, 0.0186462402343750, -0.0189819335937500, 0.0176696777343750,
+-0.0170288085937500, 0.0184631347656250, -0.0144348144531250, 0.0178833007812500,
+-0.0161437988281250, 0.0168151855468750, -0.0104064941406250, 0.0170593261718750,
+-0.0101623535156250, 0.0156555175781250, -0.0105285644531250, 0.0143432617187500,
+-0.0077819824218750, 0.0149230957031250, -0.0081481933593750, 0.0115356445312500,
+-0.0102233886718750, 0.0123596191406250, -0.0096435546875000, 0.0102233886718750,
+-0.0100402832031250, 0.0068969726562500, -0.0119323730468750, 0.0089111328125000,
+-0.0115966796875000, 0.0030212402343750, -0.0109863281250000, 0.0036621093750000,
+-0.0116577148437500, 0.0021057128906250, -0.0113525390625000, -0.0024108886718750,
+-0.0101013183593750, -0.0009765625000000, -0.0107116699218750, -0.0056152343750000,
+-0.0124206542968750, -0.0057373046875000, -0.0104675292968750, -0.0072326660156250,
+-0.0138244628906250, -0.0109252929687500, -0.0175781250000000, -0.0099487304687500,
+-0.0146484375000000, -0.0123901367187500, -0.0228576660156250, -0.0137329101562500,
+-0.0224914550781250, -0.0132751464843750, -0.0231933593750000, -0.0137023925781250,
+-0.0280151367187500, -0.0150756835937500, -0.0264282226562500, -0.0147399902343750,
+-0.0292968750000000, -0.0151062011718750, -0.0301208496093750, -0.0151672363281250,
+-0.0295715332031250, -0.0154113769531250, -0.0307006835937500, -0.0162048339843750,
+-0.0314025878906250, -0.0135192871093750, -0.0287170410156250, -0.0146484375000000,
+-0.0290832519531250, -0.0132446289062500, -0.0292053222656250, -0.0100402832031250,
+-0.0273742675781250, -0.0115051269531250, -0.0273742675781250, -0.0080261230468750,
+-0.0287780761718750, -0.0079956054687500, -0.0279235839843750, -0.0064392089843750,
+-0.0278930664062500, -0.0039367675781250, -0.0287170410156250, -0.0050354003906250,
+-0.0280456542968750, 0.0001525878906250, -0.0289306640625000, -0.0003356933593750,
+-0.0290527343750000, 0.0010681152343750, -0.0282287597656250, 0.0050354003906250,
+-0.0284729003906250, 0.0031127929687500, -0.0291137695312500, 0.0050964355468750,
+-0.0267028808593750, 0.0062866210937500, -0.0260009765625000, 0.0045471191406250,
+-0.0286560058593750, 0.0037841796875000, -0.0287170410156250, 0.0050048828125000,
+-0.0266418457031250, 0.0019531250000000, -0.0346984863281250, 0.0025634765625000,
+-0.0333862304687500, 0.0015869140625000, -0.0334777832031250, -0.0006103515625000,
+-0.0395507812500000, 0.0009460449218750, -0.0371093750000000, -0.0022583007812500,
+-0.0306701660156250, -0.0013732910156250, -0.0359191894531250, -0.0036010742187500,
+-0.0273132324218750, -0.0071105957031250, -0.0162048339843750, -0.0045166015625000,
+-0.0238647460937500, -0.0088806152343750, -0.0033264160156250, -0.0100097656250000,
+-0.0032958984375000, -0.0089111328125000, -0.0070190429687500, -0.0096435546875000,
+0.0030517578125000, -0.0095214843750000, 0.0035705566406250, -0.0078125000000000,
+-0.0072326660156250, -0.0086059570312500, -0.0072631835937500, -0.0070495605468750,
+-0.0044555664062500, -0.0039978027343750, -0.0081176757812500, -0.0054931640625000,
+-0.0092773437500000, -0.0065307617187500, -0.0032043457031250, -0.0050048828125000,
+-0.0052490234375000, -0.0058288574218750, -0.0028381347656250, -0.0079956054687500,
+0.0040283203125000, -0.0075073242187500, 0.0006713867187500, -0.0093994140625000,
+0.0000610351562500, -0.0101318359375000, 0.0032043457031250, -0.0075683593750000,
+0.0001525878906250, -0.0073242187500000, -0.0057067871093750, -0.0096130371093750,
+-0.0027160644531250, -0.0018615722656250, 0.0020141601562500, -0.0026550292968750,
+-0.0028076171875000, -0.0022583007812500, 0.0003051757812500, 0.0028991699218750,
+0.0086975097656250, 0.0007019042968750, 0.0057373046875000, -0.0028381347656250,
+0.0088195800781250, -0.0004577636718750, 0.0102233886718750, -0.0033874511718750,
+0.0076904296875000, -0.0105895996093750, 0.0062561035156250, -0.0103149414062500,
+0.0063781738281250, -0.0107421875000000, 0.0031127929687500, -0.0120239257812500,
+0.0039367675781250, -0.0105285644531250, 0.0031127929687500, -0.0088806152343750,
+0.0006103515625000, -0.0103759765625000, 0.0016784667968750, -0.0089111328125000,
+-0.0011596679687500, -0.0070495605468750, -0.0013732910156250, -0.0085754394531250,
+-0.0015563964843750, -0.0108947753906250, -0.0032958984375000, -0.0087585449218750,
+-0.0031433105468750, -0.0048217773437500, -0.0033569335937500, -0.0094299316406250,
+-0.0024108886718750, -0.0037536621093750, -0.0047607421875000, 0.0070190429687500,
+-0.0073242187500000, 0.0015869140625000, -0.0055847167968750, 0.0011291503906250,
+-0.0082397460937500, 0.0072937011718750, -0.0090942382812500, -0.0002746582031250,
+-0.0080871582031250, -0.0100097656250000, -0.0091552734375000, -0.0030517578125000,
+-0.0093994140625000, -0.0107727050781250, -0.0026855468750000, -0.0144958496093750,
+-0.0042419433593750, -0.0105895996093750, -0.0030212402343750, -0.0096435546875000,
+0.0028076171875000, -0.0113525390625000, 0.0008850097656250, -0.0041198730468750,
+0.0003662109375000, -0.0053710937500000, 0.0030822753906250, -0.0044860839843750,
+0.0003967285156250, 0.0012512207031250, -0.0040893554687500, -0.0004882812500000,
+-0.0012207031250000, -0.0009460449218750, -0.0008239746093750, 0.0014648437500000,
+-0.0030212402343750, -0.0027770996093750, -0.0001525878906250, -0.0065917968750000,
+0.0033264160156250, -0.0028381347656250, 0.0017395019531250, -0.0119934082031250,
+0.0086669921875000, -0.0119934082031250, 0.0084533691406250, -0.0131835937500000,
+0.0086364746093750, -0.0180969238281250, 0.0131225585937500, -0.0160217285156250,
+0.0123901367187500, -0.0213317871093750, 0.0101928710937500, -0.0208435058593750,
+0.0115356445312500, -0.0226440429687500, 0.0106506347656250, -0.0267333984375000,
+0.0080566406250000, -0.0246276855468750, 0.0094299316406250, -0.0297851562500000,
+0.0097656250000000, -0.0301818847656250, 0.0095214843750000, -0.0294494628906250,
+0.0093994140625000, -0.0318908691406250, 0.0104064941406250, -0.0317687988281250,
+0.0109252929687500, -0.0282897949218750, 0.0061645507812500, -0.0299987792968750,
+0.0080261230468750, -0.0277099609375000, 0.0047912597656250, -0.0231628417968750,
+-0.0007934570312500, -0.0257873535156250, 0.0020141601562500, -0.0230102539062500,
+-0.0064392089843750, -0.0205078125000000, -0.0062561035156250, -0.0242919921875000,
+-0.0076293945312500, -0.0262451171875000, -0.0131225585937500, -0.0227050781250000,
+-0.0110168457031250, -0.0311279296875000, -0.0129089355468750, -0.0307922363281250,
+-0.0143432617187500, -0.0294799804687500, -0.0123291015625000, -0.0333862304687500,
+-0.0115966796875000, -0.0321655273437500, -0.0126342773437500, -0.0285949707031250,
+-0.0063171386718750, -0.0299072265625000, -0.0075073242187500, -0.0273437500000000,
+-0.0046081542968750, -0.0231933593750000, 0.0013427734375000, -0.0248107910156250,
+-0.0010070800781250, -0.0191040039062500, 0.0052185058593750, -0.0195312500000000,
+0.0056762695312500, -0.0175476074218750, 0.0060729980468750, -0.0130920410156250,
+0.0091552734375000, -0.0151062011718750, 0.0083312988281250, -0.0105895996093750,
+0.0101318359375000, -0.0096435546875000, 0.0099792480468750, -0.0103149414062500,
+0.0104370117187500, -0.0088195800781250, 0.0114135742187500, -0.0085754394531250,
+0.0102844238281250, -0.0106201171875000, 0.0124816894531250, -0.0098876953125000,
+0.0114746093750000, -0.0108947753906250, 0.0119018554687500, -0.0129394531250000,
+0.0136108398437500, -0.0119018554687500, 0.0119323730468750, -0.0149230957031250,
+0.0132141113281250, -0.0146179199218750, 0.0122985839843750, -0.0160522460937500,
+0.0120239257812500, -0.0186157226562500, 0.0126037597656250, -0.0172424316406250,
+0.0112304687500000, -0.0210571289062500, 0.0125427246093750, -0.0213623046875000,
+0.0118713378906250, -0.0213623046875000, 0.0119323730468750, -0.0233459472656250,
+0.0128479003906250, -0.0233459472656250, 0.0119934082031250, -0.0230102539062500,
+0.0132751464843750, -0.0236816406250000, 0.0125732421875000, -0.0235290527343750,
+0.0130920410156250, -0.0228576660156250, 0.0141906738281250, -0.0232849121093750,
+0.0130004882812500, -0.0225830078125000, 0.0154418945312500, -0.0230712890625000,
+0.0149536132812500, -0.0217285156250000, 0.0153198242187500, -0.0203552246093750,
+0.0170288085937500, -0.0213012695312500, 0.0158386230468750, -0.0169372558593750,
+0.0169372558593750, -0.0167846679687500, 0.0177917480468750, -0.0165100097656250,
+0.0158386230468750, -0.0136108398437500, 0.0152282714843750, -0.0135498046875000,
+0.0164489746093750, -0.0146789550781250, 0.0099182128906250, -0.0134582519531250,
+0.0115661621093750, -0.0137329101562500, 0.0089111328125000, -0.0148315429687500,
+0.0032043457031250, -0.0137634277343750, 0.0066223144531250, -0.0159606933593750,
+0.0004577636718750, -0.0150146484375000, -0.0003356933593750, -0.0164184570312500,
+0.0005187988281250, -0.0189208984375000, -0.0017700195312500, -0.0168762207031250,
+-0.0015869140625000, -0.0199890136718750, -0.0004577636718750, -0.0201721191406250,
+-0.0014953613281250, -0.0198364257812500, 0.0008850097656250, -0.0206604003906250,
+0.0030212402343750, -0.0202636718750000, 0.0008239746093750, -0.0223083496093750,
+0.0080261230468750, -0.0207824707031250, 0.0075073242187500, -0.0233764648437500,
+0.0083007812500000, -0.0269775390625000, 0.0133666992187500, -0.0243530273437500,
+0.0113525390625000, -0.0304260253906250, 0.0098266601562500, -0.0304565429687500,
+0.0126037597656250, -0.0304260253906250, 0.0086669921875000, -0.0333251953125000,
+0.0034484863281250, -0.0322875976562500, 0.0063476562500000, -0.0335998535156250,
+-0.0002136230468750, -0.0335388183593750, -0.0007324218750000, -0.0333251953125000,
+-0.0013122558593750, -0.0343017578125000, -0.0049438476562500, -0.0337524414062500,
+-0.0034790039062500, -0.0308532714843750, -0.0036926269531250, -0.0325012207031250,
+-0.0044250488281250, -0.0299682617187500, -0.0042114257812500, -0.0251770019531250,
+-0.0032348632812500, -0.0276489257812500, -0.0031433105468750, -0.0264587402343750,
+-0.0052795410156250},
+{0.0012512207031250, -0.0181579589843750, -0.0003967285156250, -0.0191955566406250,
+0.0003967285156250, -0.0162353515625000, -0.0012207031250000, -0.0157165527343750,
+-0.0037536621093750, -0.0167236328125000, -0.0028991699218750, -0.0114440917968750,
+-0.0069274902343750, -0.0113525390625000, -0.0073242187500000, -0.0106506347656250,
+-0.0083923339843750, -0.0075378417968750, -0.0112609863281250, -0.0083312988281250,
+-0.0107727050781250, -0.0070800781250000, -0.0120544433593750, -0.0064697265625000,
+-0.0128479003906250, -0.0072631835937500, -0.0125732421875000, -0.0072021484375000,
+-0.0126342773437500, -0.0068054199218750, -0.0131530761718750, -0.0100708007812500,
+-0.0122070312500000, -0.0088195800781250, -0.0125732421875000, -0.0113220214843750,
+-0.0120544433593750, -0.0151062011718750, -0.0113525390625000, -0.0127258300781250,
+-0.0120239257812500, -0.0191040039062500, -0.0091247558593750, -0.0188598632812500,
+-0.0098266601562500, -0.0201110839843750, -0.0088806152343750, -0.0242309570312500,
+-0.0062561035156250, -0.0224609375000000, -0.0075988769531250, -0.0258483886718750,
+-0.0056762695312500, -0.0263977050781250, -0.0052490234375000, -0.0260314941406250,
+-0.0057373046875000, -0.0275573730468750, -0.0053100585937500, -0.0275268554687500,
+-0.0055847167968750, -0.0257263183593750, -0.0065307617187500, -0.0272216796875000,
+-0.0062866210937500, -0.0246276855468750, -0.0070495605468750, -0.0217895507812500,
+-0.0081481933593750, -0.0242614746093750, -0.0074768066406250, -0.0158996582031250,
+-0.0089416503906250, -0.0171813964843750, -0.0091552734375000, -0.0149841308593750,
+-0.0092163085937500, -0.0083618164062500, -0.0100708007812500, -0.0111083984375000,
+-0.0099487304687500, -0.0066528320312500, -0.0091552734375000, -0.0056152343750000,
+-0.0103454589843750, -0.0061645507812500, -0.0085449218750000, -0.0046081542968750,
+-0.0069274902343750, -0.0044860839843750, -0.0089111328125000, -0.0055541992187500,
+-0.0024719238281250, -0.0043029785156250, -0.0044250488281250, -0.0053710937500000,
+-0.0024108886718750, -0.0071411132812500, 0.0027465820312500, -0.0055541992187500,
+-0.0009460449218750, -0.0083618164062500, 0.0050354003906250, -0.0083923339843750,
+0.0051574707031250, -0.0081481933593750, 0.0050964355468750, -0.0089416503906250,
+0.0080871582031250, -0.0084533691406250, 0.0068359375000000, -0.0107727050781250,
+0.0076599121093750, -0.0089721679687500, 0.0086975097656250, -0.0121765136718750,
+0.0056762695312500, -0.0160217285156250, 0.0040283203125000, -0.0129394531250000,
+0.0061950683593750, -0.0216979980468750, -0.0011901855468750, -0.0209045410156250,
+-0.0007324218750000, -0.0225219726562500, -0.0011901855468750, -0.0282287597656250,
+-0.0057983398437500, -0.0255737304687500, -0.0039367675781250, -0.0313720703125000,
+-0.0035400390625000, -0.0305480957031250, -0.0051879882812500, -0.0323486328125000,
+-0.0031738281250000, -0.0371398925781250, -0.0003356933593750, -0.0344238281250000,
+-0.0014038085937500, -0.0379333496093750, 0.0017395019531250, -0.0393371582031250,
+0.0014953613281250, -0.0370788574218750, 0.0022583007812500, -0.0364990234375000,
+0.0048522949218750, -0.0377502441406250, 0.0036010742187500, -0.0351257324218750,
+0.0028076171875000, -0.0337219238281250, 0.0042114257812500, -0.0364990234375000,
+0.0016174316406250, -0.0376281738281250, -0.0011901855468750, -0.0354919433593750,
+0.0004882812500000, -0.0411376953125000, -0.0056762695312500, -0.0424804687500000,
+-0.0054931640625000, -0.0389404296875000, -0.0063476562500000, -0.0395507812500000,
+-0.0102233886718750, -0.0415344238281250, -0.0083923339843750, -0.0306396484375000,
+-0.0088195800781250, -0.0322570800781250, -0.0103454589843750, -0.0297546386718750,
+-0.0094299316406250, -0.0224609375000000, -0.0076904296875000, -0.0255432128906250,
+-0.0080871582031250, -0.0157775878906250, -0.0082702636718750, -0.0185852050781250,
+-0.0080871582031250, -0.0128784179687500, -0.0079650878906250, -0.0017700195312500,
+-0.0092163085937500, -0.0081176757812500, -0.0089721679687500, 0.0003967285156250,
+-0.0015563964843750, 0.0028076171875000, -0.0034179687500000, 0.0015563964843750,
+-0.0027160644531250, 0.0029907226562500, 0.0018615722656250, 0.0025634765625000,
+0.0011291503906250, 0.0041503906250000, 0.0093383789062500, 0.0028991699218750,
+0.0078735351562500, 0.0061645507812500, 0.0072937011718750, 0.0088500976562500,
+0.0134582519531250, 0.0056457519531250, 0.0128173828125000, 0.0165710449218750,
+0.0069580078125000, 0.0169677734375000, 0.0103454589843750, 0.0136108398437500,
+0.0058898925781250, 0.0183715820312500, -0.0044250488281250, 0.0194702148437500,
+0.0003662109375000, 0.0059814453125000, 0.0065917968750000, 0.0076293945312500,
+0.0009155273437500, 0.0083007812500000, 0.0040283203125000, -0.0026550292968750,
+0.0187683105468750, 0.0159912109375000, 0.0062866210937500, 0.0227661132812500,
+0.0164794921875000, 0.0279235839843750, 0.0126953125000000, 0.0184020996093750,
+0.0126037597656250, 0.0112609863281250, 0.0191955566406250, 0.0183410644531250,
+0.0193786621093750, 0.0044555664062500, 0.0074462890625000, 0.0028686523437500,
+0.0115051269531250, 0.0035095214843750, 0.0049438476562500, -0.0021057128906250,
+-0.0076599121093750, -0.0006408691406250, -0.0006103515625000, -0.0007324218750000,
+-0.0123291015625000, -0.0004272460937500, -0.0151367187500000, -0.0028991699218750,
+-0.0130920410156250, -0.0033569335937500, -0.0143432617187500, -0.0016784667968750,
+-0.0144348144531250, -0.0155334472656250, -0.0213623046875000, -0.0103149414062500,
+-0.0181274414062500, -0.0156555175781250, -0.0207214355468750, -0.0315856933593750,
+-0.0300903320312500, -0.0248107910156250, -0.0267944335937500, -0.0251770019531250,
+-0.0213623046875000, -0.0303955078125000, -0.0264892578125000, -0.0245361328125000,
+-0.0215454101562500, -0.0184326171875000, -0.0109558105468750, -0.0229187011718750,
+-0.0155944824218750, -0.0103759765625000, -0.0142822265625000, -0.0118103027343750,
+-0.0106506347656250, -0.0089416503906250, -0.0148620605468750, 0.0008239746093750,
+-0.0193176269531250, -0.0037231445312500, -0.0162353515625000, -0.0017089843750000,
+-0.0223388671875000, 0.0000610351562500, -0.0236511230468750, 0.0021057128906250,
+-0.0211181640625000, 0.0024108886718750, -0.0220031738281250, -0.0000610351562500,
+-0.0232543945312500, 0.0071716308593750, -0.0178527832031250, 0.0074462890625000,
+-0.0177612304687500, 0.0080261230468750, -0.0181579589843750, 0.0111999511718750,
+-0.0158081054687500, 0.0091857910156250, -0.0160217285156250, 0.0124511718750000,
+-0.0164184570312500, 0.0138549804687500, -0.0166320800781250, 0.0102539062500000,
+-0.0155334472656250, 0.0099487304687500, -0.0146484375000000, 0.0126342773437500,
+-0.0153198242187500, 0.0013427734375000, -0.0139465332031250, 0.0031738281250000,
+-0.0130310058593750, 0.0024719238281250, -0.0144653320312500, -0.0057373046875000,
+-0.0151367187500000, -0.0032043457031250, -0.0141296386718750, -0.0013427734375000,
+-0.0170898437500000, -0.0043640136718750, -0.0171508789062500, -0.0016784667968750,
+-0.0172424316406250, 0.0022277832031250, -0.0187072753906250, -0.0005187988281250,
+-0.0183410644531250, 0.0064086914062500, -0.0190429687500000, 0.0051269531250000,
+-0.0193176269531250, 0.0066833496093750, -0.0184020996093750, 0.0126647949218750,
+-0.0186157226562500, 0.0096130371093750, -0.0191345214843750, 0.0098266601562500,
+-0.0141601562500000, 0.0128173828125000, -0.0153198242187500, 0.0083923339843750,
+-0.0137634277343750, 0.0043334960937500, -0.0091552734375000, 0.0081481933593750,
+-0.0111389160156250, -0.0008850097656250, -0.0105285644531250, -0.0000305175781250,
+-0.0092468261718750, -0.0006103515625000, -0.0109252929687500, -0.0061340332031250,
+-0.0128479003906250, -0.0035400390625000, -0.0119323730468750, -0.0053710937500000,
+-0.0133361816406250, -0.0063781738281250, -0.0138854980468750, -0.0048522949218750,
+-0.0135803222656250, -0.0044860839843750, -0.0134887695312500, -0.0052185058593750,
+-0.0134582519531250, -0.0023498535156250, -0.0139465332031250, -0.0028686523437500,
+-0.0134277343750000, -0.0021667480468750, -0.0137634277343750, -0.0003662109375000,
+-0.0143432617187500, -0.0016784667968750, -0.0138244628906250, 0.0010070800781250,
+-0.0151977539062500, 0.0000000000000000, -0.0144958496093750, 0.0004577636718750,
+-0.0156555175781250, 0.0025329589843750, -0.0173645019531250, 0.0009765625000000,
+-0.0161743164062500, 0.0029907226562500, -0.0198059082031250, 0.0026855468750000,
+-0.0193786621093750, 0.0024108886718750, -0.0208129882812500, 0.0032043457031250,
+-0.0239257812500000, 0.0021667480468750, -0.0225830078125000, 0.0023193359375000,
+-0.0256958007812500, 0.0019226074218750, -0.0268859863281250, 0.0013122558593750,
+-0.0256042480468750, 0.0010070800781250, -0.0264282226562500, 0.0004577636718750,
+-0.0274658203125000, -0.0001831054687500, -0.0221252441406250, -0.0004882812500000,
+-0.0237121582031250, -0.0014648437500000, -0.0216369628906250, -0.0021972656250000,
+-0.0164489746093750, -0.0024108886718750, -0.0189208984375000, -0.0044250488281250,
+-0.0151672363281250, -0.0049743652343750, -0.0142517089843750, -0.0058288574218750,
+-0.0146484375000000, -0.0074462890625000, -0.0138854980468750, -0.0080566406250000,
+-0.0138244628906250, -0.0089416503906250, -0.0120849609375000, -0.0093383789062500,
+-0.0132446289062500, -0.0104675292968750, -0.0111999511718750, -0.0113830566406250,
+-0.0085449218750000, -0.0115356445312500, -0.0108337402343750, -0.0144348144531250,
+-0.0055541992187500, -0.0140686035156250, -0.0065917968750000, -0.0150146484375000,
+-0.0052795410156250, -0.0176086425781250, -0.0011291503906250, -0.0169677734375000,
+-0.0035705566406250, -0.0182800292968750, -0.0010681152343750, -0.0187377929687500,
+0.0007019042968750, -0.0183105468750000, -0.0018005371093750, -0.0186157226562500,
+-0.0026855468750000, -0.0186462402343750, -0.0008239746093750, -0.0157775878906250,
+-0.0069580078125000, -0.0169372558593750, -0.0049743652343750, -0.0138244628906250,
+-0.0081787109375000, -0.0098266601562500, -0.0137939453125000, -0.0121154785156250,
+-0.0096740722656250, -0.0046386718750000, -0.0199584960937500, -0.0051574707031250,
+-0.0196533203125000, -0.0036926269531250, -0.0202941894531250, 0.0015258789062500,
+-0.0259704589843750, -0.0007629394531250, -0.0245056152343750, 0.0015869140625000,
+-0.0292663574218750, 0.0031433105468750, -0.0275878906250000, 0.0007324218750000,
+-0.0316467285156250, -0.0003662109375000, -0.0377502441406250, 0.0010986328125000,
+-0.0335388183593750, -0.0043029785156250, -0.0411987304687500, -0.0042724609375000,
+-0.0424194335937500, -0.0036926269531250, -0.0404052734375000, -0.0068054199218750,
+-0.0428161621093750, -0.0065002441406250, -0.0426330566406250, -0.0015258789062500,
+-0.0361328125000000, -0.0038452148437500, -0.0394897460937500, -0.0003051757812500,
+-0.0347595214843750, 0.0060424804687500, -0.0265502929687500, 0.0025939941406250,
+-0.0315246582031250, 0.0090332031250000, -0.0222167968750000, 0.0098571777343750,
+-0.0222473144531250, 0.0095825195312500, -0.0220336914062500, 0.0123291015625000,
+-0.0166931152343750, 0.0112304687500000, -0.0186462402343750, 0.0099487304687500,
+-0.0190429687500000, 0.0122070312500000, -0.0191040039062500, 0.0082702636718750,
+-0.0161743164062500, 0.0035095214843750, -0.0150756835937500, 0.0069580078125000,
+-0.0181579589843750, 0.0014038085937500, -0.0111083984375000, -0.0000915527343750,
+-0.0119934082031250, 0.0009460449218750, -0.0081176757812500, 0.0005187988281250,
+-0.0025634765625000, 0.0003662109375000, -0.0069580078125000, -0.0020446777343750,
+0.0043334960937500, -0.0005798339843750, 0.0029602050781250, -0.0033264160156250,
+0.0062561035156250},
+{-0.0165710449218750, 0.0035400390625000, -0.0169982910156250, 0.0000305175781250,
+-0.0187377929687500, 0.0025939941406250, -0.0180358886718750, -0.0016174316406250,
+-0.0209960937500000, -0.0010070800781250, -0.0206909179687500, -0.0011291503906250,
+-0.0217895507812500, -0.0033874511718750, -0.0246276855468750, -0.0014648437500000,
+-0.0239562988281250, -0.0033264160156250, -0.0245361328125000, -0.0025329589843750,
+-0.0258178710937500, -0.0025939941406250, -0.0248107910156250, -0.0039367675781250,
+-0.0236816406250000, -0.0025329589843750, -0.0246582031250000, -0.0041198730468750,
+-0.0224609375000000, -0.0040283203125000, -0.0225830078125000, -0.0033874511718750,
+-0.0219116210937500, -0.0038757324218750, -0.0206298828125000, -0.0034790039062500,
+-0.0211791992187500, -0.0018615722656250, -0.0176086425781250, -0.0020751953125000,
+-0.0188903808593750, -0.0006103515625000, -0.0164184570312500, 0.0014953613281250,
+-0.0124206542968750, 0.0009460449218750, -0.0148925781250000, 0.0029602050781250,
+-0.0095520019531250, 0.0035095214843750, -0.0099792480468750, 0.0033569335937500,
+-0.0090026855468750, 0.0040893554687500, -0.0059814453125000, 0.0041503906250000,
+-0.0080566406250000, 0.0032958984375000, -0.0023193359375000, 0.0038146972656250,
+-0.0032348632812500, 0.0025024414062500, -0.0018615722656250, 0.0013427734375000,
+0.0029602050781250, 0.0024108886718750, 0.0010681152343750, -0.0022277832031250,
+0.0033569335937500, -0.0013122558593750, 0.0049133300781250, -0.0027770996093750,
+0.0033264160156250, -0.0066833496093750, 0.0035400390625000, -0.0051879882812500,
+0.0048828125000000, -0.0087890625000000, -0.0026245117187500, -0.0090942382812500,
+0.0002746582031250, -0.0100402832031250, -0.0035705566406250, -0.0125427246093750,
+-0.0119323730468750, -0.0119018554687500, -0.0077209472656250, -0.0139770507812500,
+-0.0151977539062500, -0.0142822265625000, -0.0156860351562500, -0.0140991210937500,
+-0.0178833007812500, -0.0150146484375000, -0.0219726562500000, -0.0146789550781250,
+-0.0192565917968750, -0.0123901367187500, -0.0283813476562500, -0.0134582519531250,
+-0.0279541015625000, -0.0109863281250000, -0.0282897949218750, -0.0075073242187500,
+-0.0335083007812500, -0.0091552734375000, -0.0319519042968750, -0.0036010742187500,
+-0.0333862304687500, -0.0036315917968750, -0.0337829589843750, -0.0033569335937500,
+-0.0323486328125000, -0.0002136230468750, -0.0328674316406250, -0.0012512207031250,
+-0.0338439941406250, -0.0012512207031250, -0.0281066894531250, -0.0013122558593750,
+-0.0295715332031250, -0.0008544921875000, -0.0281066894531250, -0.0010681152343750,
+-0.0237731933593750, -0.0017089843750000, -0.0260314941406250, 0.0019836425781250,
+-0.0199890136718750, 0.0005187988281250, -0.0201416015625000, 0.0022277832031250,
+-0.0220947265625000, 0.0058288574218750, -0.0197753906250000, 0.0037841796875000,
+-0.0187377929687500, 0.0096130371093750, -0.0211791992187500, 0.0089721679687500,
+-0.0225830078125000, 0.0090942382812500, -0.0204162597656250, 0.0133666992187500,
+-0.0202331542968750, 0.0119934082031250, -0.0211181640625000, 0.0086669921875000,
+-0.0102233886718750, 0.0105590820312500, -0.0146789550781250, 0.0083007812500000,
+-0.0082702636718750, 0.0036315917968750, 0.0039062500000000, 0.0054626464843750,
+-0.0029602050781250, 0.0029602050781250, 0.0115661621093750, 0.0028381347656250,
+0.0122680664062500, 0.0019531250000000, 0.0109252929687500, 0.0005798339843750,
+0.0174865722656250, 0.0015869140625000, 0.0163574218750000, -0.0039367675781250,
+0.0155639648437500, -0.0018310546875000, 0.0173645019531250, -0.0044555664062500,
+0.0142211914062500, -0.0108337402343750, 0.0112304687500000, -0.0077819824218750,
+0.0140991210937500, -0.0116577148437500, 0.0065002441406250, -0.0133056640625000,
+0.0079956054687500, -0.0100097656250000, 0.0055541992187500, -0.0090637207031250,
+-0.0017089843750000, -0.0110778808593750, 0.0014038085937500, -0.0046081542968750,
+0.0003051757812500, -0.0039062500000000, -0.0013427734375000, -0.0039978027343750,
+-0.0006408691406250, -0.0015258789062500, 0.0011291503906250, -0.0014038085937500,
+0.0008239746093750, 0.0013427734375000, -0.0032653808593750, 0.0008239746093750,
+-0.0006713867187500, 0.0013122558593750, -0.0020446777343750, 0.0028991699218750,
+-0.0065917968750000, 0.0021057128906250, -0.0044555664062500, 0.0058288574218750,
+-0.0086059570312500, 0.0044860839843750, -0.0086975097656250, 0.0067749023437500,
+-0.0076904296875000, 0.0106506347656250, -0.0093383789062500, 0.0077819824218750,
+-0.0092468261718750, 0.0128479003906250, -0.0076904296875000, 0.0137634277343750,
+-0.0081176757812500, 0.0118713378906250, -0.0055847167968750, 0.0128479003906250,
+-0.0031433105468750, 0.0136413574218750, -0.0051879882812500, 0.0101318359375000,
+-0.0011596679687500, 0.0088806152343750, -0.0000915527343750, 0.0118103027343750,
+0.0003662109375000, 0.0128784179687500, 0.0017700195312500, 0.0103759765625000,
+0.0008850097656250, 0.0135803222656250, 0.0026855468750000, 0.0153503417968750,
+0.0038452148437500, 0.0122375488281250, 0.0030517578125000, 0.0115051269531250,
+0.0023803710937500, 0.0137023925781250, 0.0032043457031250, 0.0045166015625000,
+0.0027770996093750, 0.0069885253906250, 0.0021362304687500, 0.0027465820312500,
+0.0042724609375000, -0.0056457519531250, 0.0058898925781250, -0.0004272460937500,
+0.0041503906250000, -0.0116882324218750, 0.0068664550781250, -0.0134277343750000,
+0.0088500976562500, -0.0104980468750000, 0.0060424804687500, -0.0133361816406250,
+0.0044555664062500, -0.0142517089843750, 0.0064086914062500, -0.0102539062500000,
+0.0000305175781250, -0.0102539062500000, 0.0003356933593750, -0.0111694335937500,
+-0.0000305175781250, -0.0090332031250000, -0.0040588378906250, -0.0087890625000000,
+-0.0027465820312500, -0.0140686035156250, -0.0016784667968750, -0.0124816894531250,
+-0.0031433105468750, -0.0131835937500000, -0.0007019042968750, -0.0174255371093750,
+0.0025634765625000, -0.0160217285156250, 0.0005493164062500, -0.0172119140625000,
+0.0036315917968750, -0.0174255371093750, 0.0054016113281250, -0.0171508789062500,
+0.0021972656250000, -0.0173339843750000, 0.0006408691406250, -0.0170288085937500,
+0.0024414062500000, -0.0173034667968750, -0.0049438476562500, -0.0171508789062500,
+-0.0049438476562500, -0.0159606933593750, -0.0059509277343750, -0.0158081054687500,
+-0.0104370117187500, -0.0161743164062500, -0.0090637207031250, -0.0111083984375000,
+-0.0106811523437500, -0.0125122070312500, -0.0116577148437500, -0.0102539062500000,
+-0.0102233886718750, -0.0053710937500000, -0.0093688964843750, -0.0074157714843750,
+-0.0100708007812500, -0.0027160644531250, -0.0074768066406250, -0.0029907226562500,
+-0.0070495605468750, -0.0012817382812500, -0.0072021484375000, 0.0021057128906250,
+-0.0061950683593750, -0.0001525878906250, -0.0061340332031250, 0.0043334960937500,
+-0.0065612792968750, 0.0053405761718750, -0.0063171386718750, 0.0028381347656250,
+-0.0066528320312500, 0.0038757324218750, -0.0070495605468750, 0.0050964355468750,
+-0.0069274902343750, -0.0032348632812500, -0.0080871582031250, -0.0012512207031250,
+-0.0079345703125000, -0.0034790039062500, -0.0088500976562500, -0.0104370117187500,
+-0.0103149414062500, -0.0072326660156250, -0.0097961425781250, -0.0113830566406250,
+-0.0110473632812500, -0.0121154785156250, -0.0115661621093750, -0.0120849609375000,
+-0.0111999511718750, -0.0136413574218750, -0.0111999511718750, -0.0130004882812500,
+-0.0114746093750000, -0.0154113769531250, -0.0105895996093750, -0.0146789550781250,
+-0.0104370117187500, -0.0160827636718750, -0.0106201171875000, -0.0184631347656250,
+-0.0102844238281250, -0.0170593261718750, -0.0100097656250000, -0.0205993652343750,
+-0.0107421875000000, -0.0207824707031250, -0.0104370117187500, -0.0209045410156250,
+-0.0107421875000000, -0.0229797363281250, -0.0114440917968750, -0.0224609375000000,
+-0.0109558105468750, -0.0214233398437500, -0.0118408203125000, -0.0227050781250000,
+-0.0119934082031250, -0.0205383300781250, -0.0115966796875000, -0.0181884765625000,
+-0.0118103027343750, -0.0199279785156250, -0.0123291015625000, -0.0141906738281250,
+-0.0113830566406250, -0.0146484375000000, -0.0117187500000000, -0.0137939453125000,
+-0.0121459960937500, -0.0101623535156250, -0.0118103027343750, -0.0117492675781250,
+-0.0118408203125000, -0.0091247558593750, -0.0123596191406250, -0.0092773437500000,
+-0.0129089355468750, -0.0086059570312500, -0.0123901367187500, -0.0068664550781250,
+-0.0126953125000000, -0.0079956054687500, -0.0133972167968750, -0.0054931640625000,
+-0.0093383789062500, -0.0060119628906250, -0.0116271972656250, -0.0053405761718750,
+-0.0089721679687500, -0.0035095214843750, -0.0044250488281250, -0.0047302246093750,
+-0.0081787109375000, -0.0018310546875000, -0.0008239746093750, -0.0016784667968750,
+-0.0019836425781250, -0.0020446777343750, -0.0018310546875000, -0.0004577636718750,
+0.0028686523437500, -0.0007019042968750, 0.0006408691406250, -0.0035705566406250,
+0.0017700195312500, -0.0019836425781250, 0.0036621093750000, -0.0043334960937500,
+0.0010375976562500, -0.0078430175781250, -0.0007629394531250, -0.0054626464843750,
+0.0010375976562500, -0.0116271972656250, -0.0062561035156250, -0.0106811523437500,
+-0.0047912597656250, -0.0123291015625000, -0.0074157714843750, -0.0169677734375000,
+-0.0141601562500000, -0.0142517089843750, -0.0108947753906250, -0.0194702148437500,
+-0.0139160156250000, -0.0196533203125000, -0.0160522460937500, -0.0191955566406250,
+-0.0123901367187500, -0.0213623046875000, -0.0111694335937500, -0.0208129882812500,
+-0.0134887695312500, -0.0209045410156250, -0.0032653808593750, -0.0202026367187500,
+-0.0053710937500000, -0.0220336914062500, -0.0028076171875000, -0.0231933593750000,
+0.0057678222656250, -0.0215454101562500, 0.0024719238281250, -0.0275878906250000,
+0.0069885253906250, -0.0269165039062500, 0.0086059570312500, -0.0275878906250000,
+0.0056152343750000, -0.0318603515625000, 0.0058593750000000, -0.0303649902343750,
+0.0071716308593750, -0.0306701660156250, -0.0021362304687500, -0.0319213867187500,
+-0.0005493164062500, -0.0308227539062500, -0.0032958984375000, -0.0298461914062500,
+-0.0117187500000000, -0.0305480957031250, -0.0084838867187500, -0.0270690917968750,
+-0.0090332031250000, -0.0289001464843750, -0.0126953125000000, -0.0249633789062500,
+-0.0082702636718750, -0.0203247070312500, -0.0035095214843750, -0.0240173339843750,
+-0.0074157714843750, -0.0120849609375000, -0.0003051757812500, -0.0130310058593750,
+0.0017700195312500, -0.0117797851562500, -0.0030822753906250, -0.0032958984375000,
+-0.0036926269531250, -0.0061645507812500, -0.0009460449218750, -0.0087585449218750,
+-0.0093994140625000, -0.0060729980468750, -0.0098266601562500, -0.0073852539062500,
+-0.0099182128906250, -0.0126953125000000, -0.0143127441406250, -0.0115966796875000,
+-0.0133361816406250, -0.0064392089843750, -0.0120849609375000, -0.0089721679687500,
+-0.0137634277343750, -0.0065917968750000, -0.0094909667968750, -0.0007629394531250,
+-0.0070800781250000, -0.0034484863281250, -0.0112304687500000, 0.0000305175781250,
+0.0042419433593750, 0.0012817382812500, 0.0052185058593750, 0.0005187988281250,
+0.0022583007812500, -0.0001525878906250, 0.0074462890625000, 0.0003662109375000,
+0.0075683593750000, 0.0067138671875000, 0.0028381347656250, 0.0026245117187500,
+0.0030517578125000, 0.0093994140625000, 0.0063476562500000, 0.0196533203125000,
+0.0044250488281250, 0.0132446289062500, 0.0018005371093750, 0.0255432128906250,
+0.0140380859375000, 0.0279846191406250, 0.0136718750000000, 0.0235900878906250,
+0.0138244628906250, 0.0251159667968750, 0.0233459472656250, 0.0169677734375000,
+0.0102844238281250},
+{-0.0012512207031250, -0.0158691406250000, -0.0005798339843750, -0.0146484375000000,
+-0.0024108886718750, -0.0151672363281250, -0.0015563964843750, -0.0142822265625000,
+-0.0032348632812500, -0.0130920410156250, -0.0050964355468750, -0.0141601562500000,
+-0.0033874511718750, -0.0104980468750000, -0.0098571777343750, -0.0115661621093750,
+-0.0079956054687500, -0.0102233886718750, -0.0102233886718750, -0.0067443847656250,
+-0.0158996582031250, -0.0087280273437500, -0.0123901367187500, -0.0059204101562500,
+-0.0176086425781250, -0.0054321289062500, -0.0178833007812500, -0.0062561035156250,
+-0.0172729492187500, -0.0057373046875000, -0.0198974609375000, -0.0057678222656250,
+-0.0194091796875000, -0.0066528320312500, -0.0171813964843750, -0.0063171386718750,
+-0.0191955566406250, -0.0072021484375000, -0.0162353515625000, -0.0080261230468750,
+-0.0122070312500000, -0.0070495605468750, -0.0147705078125000, -0.0099487304687500,
+-0.0070800781250000, -0.0094604492187500, -0.0081787109375000, -0.0092468261718750,
+-0.0069580078125000, -0.0108642578125000, -0.0011901855468750, -0.0101623535156250,
+-0.0038146972656250, -0.0099182128906250, -0.0034179687500000, -0.0096740722656250,
+-0.0015869140625000, -0.0104675292968750, -0.0044860839843750, -0.0106506347656250,
+-0.0068664550781250, -0.0098266601562500, -0.0046691894531250, -0.0137939453125000,
+-0.0104064941406250, -0.0128784179687500, -0.0109863281250000, -0.0143432617187500,
+-0.0092773437500000, -0.0178833007812500, -0.0115356445312500, -0.0161743164062500,
+-0.0124206542968750, -0.0190734863281250, -0.0074157714843750, -0.0194702148437500,
+-0.0089721679687500, -0.0194396972656250, -0.0054321289062500, -0.0206604003906250,
+-0.0000915527343750, -0.0202026367187500, -0.0036010742187500, -0.0207214355468750,
+0.0054016113281250, -0.0214233398437500, 0.0058288574218750, -0.0199279785156250,
+0.0048217773437500, -0.0194396972656250, 0.0096130371093750, -0.0206298828125000,
+0.0086364746093750, -0.0145568847656250, 0.0013427734375000, -0.0157470703125000,
+0.0041809082031250, -0.0144958496093750, 0.0003967285156250, -0.0093994140625000,
+-0.0079956054687500, -0.0114746093750000, -0.0043029785156250, -0.0105285644531250,
+-0.0086364746093750, -0.0089416503906250, -0.0112609863281250, -0.0117187500000000,
+-0.0089111328125000, -0.0141601562500000, -0.0072326660156250, -0.0121154785156250,
+-0.0086364746093750, -0.0154418945312500, -0.0095214843750000, -0.0173034667968750,
+-0.0089111328125000, -0.0147399902343750, -0.0088500976562500, -0.0133972167968750,
+-0.0099182128906250, -0.0150451660156250, -0.0101623535156250, -0.0115966796875000,
+-0.0086059570312500, -0.0119934082031250, -0.0090026855468750, -0.0099792480468750,
+-0.0102539062500000, -0.0075378417968750, -0.0099792480468750, -0.0096130371093750,
+-0.0084228515625000, -0.0006408691406250, -0.0080261230468750, -0.0034179687500000,
+-0.0130920410156250, 0.0021362304687500, -0.0054931640625000, 0.0119934082031250,
+0.0034790039062500, 0.0060119628906250, -0.0025024414062500, 0.0166931152343750,
+0.0111694335937500, 0.0182800292968750, 0.0122985839843750, 0.0173339843750000,
+0.0090026855468750, 0.0209655761718750, 0.0136718750000000, 0.0197753906250000,
+0.0151672363281250, 0.0190124511718750, 0.0105895996093750, 0.0204162597656250,
+0.0095214843750000, 0.0185546875000000, 0.0115051269531250, 0.0140380859375000,
+0.0121459960937500, 0.0094909667968750, 0.0166625976562500, -0.0038452148437500,
+0.0108337402343750, 0.0031738281250000, 0.0109863281250000, -0.0012512207031250,
+0.0164184570312500, -0.0125122070312500, 0.0170288085937500, -0.0089416503906250,
+0.0126647949218750, -0.0046081542968750, 0.0257263183593750, -0.0085754394531250,
+0.0272827148437500, -0.0048217773437500, 0.0227966308593750, 0.0030822753906250,
+0.0274353027343750, -0.0007324218750000, 0.0291137695312500, -0.0000305175781250,
+0.0111083984375000, 0.0039672851562500, 0.0131225585937500, -0.0005187988281250,
+0.0145263671875000, -0.0064392089843750, 0.0046997070312500, -0.0023803710937500,
+0.0050354003906250, -0.0054626464843750, 0.0071411132812500, -0.0110778808593750,
+0.0099182128906250, -0.0001831054687500, 0.0028381347656250, 0.0086059570312500,
+-0.0011901855468750, -0.0010375976562500, 0.0040893554687500, 0.0166320800781250,
+-0.0185852050781250, 0.0212707519531250, -0.0147094726562500, 0.0134277343750000,
+-0.0201721191406250, 0.0158386230468750, -0.0389404296875000, 0.0197448730468750,
+-0.0310058593750000, -0.0002746582031250, -0.0352478027343750, 0.0036315917968750,
+-0.0406799316406250, -0.0003662109375000, -0.0346374511718750, -0.0166320800781250,
+-0.0286560058593750, -0.0097045898437500, -0.0326843261718750, -0.0109252929687500,
+-0.0281982421875000, -0.0179748535156250, -0.0261840820312500, -0.0098876953125000,
+-0.0270996093750000, -0.0003356933593750, -0.0278015136718750, -0.0068359375000000,
+-0.0280761718750000, 0.0034484863281250, -0.0240783691406250, 0.0045471191406250,
+-0.0249023437500000, 0.0043029785156250, -0.0250854492187500, 0.0091857910156250,
+-0.0220642089843750, 0.0071411132812500, -0.0223693847656250, 0.0046386718750000,
+-0.0255126953125000, 0.0080261230468750, -0.0254516601562500, 0.0036621093750000,
+-0.0239562988281250, -0.0027465820312500, -0.0258483886718750, 0.0013732910156250,
+-0.0266418457031250, -0.0042724609375000, -0.0168457031250000, -0.0056457519531250,
+-0.0199890136718750, -0.0047607421875000, -0.0153198242187500, -0.0057373046875000,
+-0.0053710937500000, -0.0054321289062500, -0.0103454589843750, -0.0071411132812500,
+-0.0018920898437500, -0.0062255859375000, -0.0009765625000000, -0.0070495605468750,
+-0.0007934570312500, -0.0091552734375000, 0.0026550292968750, -0.0077819824218750,
+0.0011901855468750, -0.0086669921875000, 0.0052185058593750, -0.0089111328125000,
+0.0046386718750000, -0.0089721679687500, 0.0053100585937500, -0.0089111328125000,
+0.0083312988281250, -0.0082397460937500, 0.0071105957031250, -0.0094299316406250,
+0.0079650878906250, -0.0097656250000000, 0.0083007812500000, -0.0083312988281250,
+0.0081787109375000, -0.0080871582031250, 0.0084228515625000, -0.0088806152343750,
+0.0083007812500000, -0.0051574707031250, 0.0076904296875000, -0.0060424804687500,
+0.0086364746093750, -0.0046081542968750, 0.0068969726562500, -0.0016174316406250,
+0.0050659179687500, -0.0035705566406250, 0.0065612792968750, 0.0009155273437500,
+0.0025329589843750, 0.0002441406250000, 0.0026550292968750, 0.0012512207031250,
+0.0022888183593750, 0.0044250488281250, 0.0003662109375000, 0.0024414062500000,
+0.0011596679687500, 0.0056457519531250, -0.0025939941406250, 0.0054931640625000,
+-0.0010986328125000, 0.0054321289062500, -0.0044860839843750, 0.0071105957031250,
+-0.0093688964843750, 0.0062561035156250, -0.0063476562500000, 0.0070495605468750,
+-0.0146179199218750, 0.0075073242187500, -0.0146789550781250, 0.0071105957031250,
+-0.0155639648437500, 0.0073547363281250, -0.0202636718750000, 0.0078430175781250,
+-0.0186157226562500, 0.0065917968750000, -0.0213928222656250, 0.0072021484375000,
+-0.0216369628906250, 0.0068969726562500, -0.0219116210937500, 0.0058593750000000,
+-0.0232238769531250, 0.0069580078125000, -0.0225524902343750, 0.0065307617187500,
+-0.0238647460937500, 0.0065612792968750, -0.0238037109375000, 0.0072631835937500,
+-0.0235900878906250, 0.0076293945312500, -0.0242614746093750, 0.0080261230468750,
+-0.0241394042968750, 0.0092163085937500, -0.0237426757812500, 0.0093688964843750,
+-0.0241088867187500, 0.0102844238281250, -0.0238647460937500, 0.0115356445312500,
+-0.0237426757812500, 0.0113220214843750, -0.0242309570312500, 0.0126953125000000,
+-0.0222778320312500, 0.0134582519531250, -0.0238952636718750, 0.0136718750000000,
+-0.0211181640625000, 0.0143127441406250, -0.0178833007812500, 0.0149230957031250,
+-0.0206298828125000, 0.0151367187500000, -0.0123596191406250, 0.0154113769531250,
+-0.0131225585937500, 0.0153808593750000, -0.0113830566406250, 0.0155029296875000,
+-0.0055541992187500, 0.0158996582031250, -0.0078125000000000, 0.0148620605468750,
+-0.0031433105468750, 0.0151672363281250, -0.0020446777343750, 0.0144653320312500,
+-0.0032958984375000, 0.0136108398437500, -0.0017089843750000, 0.0139770507812500,
+-0.0009155273437500, 0.0104064941406250, -0.0053405761718750, 0.0116577148437500,
+-0.0048217773437500, 0.0087890625000000, -0.0047302246093750, 0.0046386718750000,
+-0.0078430175781250, 0.0070190429687500, -0.0078125000000000, -0.0008850097656250,
+-0.0061645507812500, 0.0001831054687500, -0.0066528320312500, -0.0021057128906250,
+-0.0074462890625000, -0.0080871582031250, -0.0067443847656250, -0.0049743652343750,
+-0.0063171386718750, -0.0105590820312500, -0.0075073242187500, -0.0107116699218750,
+-0.0075683593750000, -0.0098266601562500, -0.0080566406250000, -0.0123901367187500,
+-0.0076904296875000, -0.0115661621093750, -0.0070800781250000, -0.0089111328125000,
+-0.0135498046875000, -0.0099792480468750, -0.0110778808593750, -0.0082397460937500,
+-0.0127563476562500, -0.0051269531250000, -0.0198364257812500, -0.0066833496093750,
+-0.0175781250000000, -0.0037536621093750, -0.0156555175781250, -0.0031127929687500,
+-0.0182189941406250, -0.0047607421875000, -0.0169982910156250, -0.0045166015625000,
+-0.0134887695312500, -0.0041198730468750, -0.0148315429687500, -0.0077514648437500,
+-0.0133666992187500, -0.0079650878906250, -0.0122680664062500, -0.0082702636718750,
+-0.0162963867187500, -0.0107421875000000, -0.0175476074218750, -0.0103759765625000,
+-0.0138854980468750, -0.0075073242187500, -0.0251464843750000, -0.0100708007812500,
+-0.0268249511718750, -0.0061950683593750, -0.0208740234375000, -0.0012207031250000,
+-0.0231933593750000, -0.0050354003906250, -0.0267333984375000, 0.0048828125000000,
+-0.0106811523437500, 0.0043640136718750, -0.0133056640625000, 0.0052185058593750,
+-0.0101318359375000, 0.0113220214843750, 0.0019226074218750, 0.0088195800781250,
+-0.0030517578125000, 0.0101318359375000, 0.0022888183593750, 0.0116577148437500,
+0.0050048828125000, 0.0096435546875000, 0.0003051757812500, 0.0082092285156250,
+-0.0009765625000000, 0.0094299316406250, 0.0027465820312500, 0.0059814453125000,
+-0.0046386718750000, 0.0066833496093750, -0.0063476562500000, 0.0059204101562500,
+-0.0043945312500000, 0.0034790039062500, -0.0050964355468750, 0.0048217773437500,
+-0.0049133300781250, 0.0020446777343750, -0.0035705566406250, 0.0028381347656250,
+-0.0037231445312500, 0.0012207031250000, -0.0043029785156250, -0.0013427734375000,
+-0.0039062500000000, 0.0006713867187500, -0.0030517578125000, -0.0050659179687500,
+-0.0025329589843750, -0.0049438476562500, -0.0031433105468750, -0.0041809082031250,
+-0.0033874511718750, -0.0068359375000000, -0.0028076171875000, -0.0066223144531250,
+-0.0020446777343750, -0.0058593750000000, -0.0004882812500000, -0.0064392089843750,
+-0.0019226074218750, -0.0048522949218750, 0.0000305175781250, -0.0028686523437500,
+0.0041503906250000, -0.0028076171875000, 0.0034179687500000, 0.0015869140625000,
+0.0036621093750000, -0.0000610351562500, 0.0055847167968750, 0.0018615722656250,
+0.0032958984375000, 0.0061645507812500, 0.0009765625000000, 0.0030822753906250,
+0.0033264160156250, 0.0059509277343750, -0.0014648437500000, 0.0087280273437500,
+-0.0018310546875000, 0.0055847167968750, -0.0008239746093750, 0.0025634765625000,
+-0.0022888183593750, 0.0052795410156250, -0.0023193359375000, 0.0036926269531250,
+-0.0017395019531250, 0.0000305175781250, -0.0010375976562500, 0.0058898925781250,
+-0.0014038085937500, 0.0119934082031250, -0.0022277832031250, 0.0065917968750000,
+-0.0019836425781250, 0.0112609863281250, -0.0014038085937500, 0.0164184570312500,
+-0.0010070800781250},
+{-0.0109252929687500, 0.0092468261718750, -0.0131835937500000, 0.0093688964843750,
+-0.0113220214843750, 0.0114135742187500, -0.0068664550781250, 0.0116577148437500,
+-0.0091857910156250, 0.0101928710937500, -0.0073242187500000, 0.0119628906250000,
+-0.0065002441406250, 0.0100708007812500, -0.0070190429687500, 0.0073852539062500,
+-0.0067138671875000, 0.0097045898437500, -0.0068054199218750, 0.0042114257812500,
+-0.0085449218750000, 0.0047302246093750, -0.0065307617187500, 0.0030212402343750,
+-0.0092163085937500, -0.0010986328125000, -0.0135192871093750, 0.0007019042968750,
+-0.0101318359375000, -0.0045776367187500, -0.0114135742187500, -0.0049438476562500,
+-0.0150756835937500, -0.0053100585937500, -0.0102539062500000, -0.0084228515625000,
+-0.0050354003906250, -0.0076293945312500, -0.0094604492187500, -0.0068359375000000,
+-0.0049743652343750, -0.0086669921875000, -0.0025024414062500, -0.0055847167968750,
+-0.0051269531250000, -0.0021057128906250, -0.0059204101562500, -0.0048522949218750,
+-0.0051879882812500, 0.0011291503906250, -0.0118713378906250, 0.0017089843750000,
+-0.0090942382812500, 0.0007629394531250, -0.0122070312500000, 0.0030517578125000,
+-0.0213623046875000, 0.0024414062500000, -0.0171508789062500, -0.0008239746093750,
+-0.0129089355468750, 0.0000610351562500, -0.0192565917968750, -0.0016784667968750,
+-0.0131225585937500, -0.0053100585937500, -0.0025634765625000, -0.0038452148437500,
+-0.0086669921875000, -0.0053710937500000, -0.0000610351562500, -0.0073242187500000,
+0.0020446777343750, -0.0041198730468750, -0.0007934570312500, -0.0022888183593750,
+0.0011901855468750, -0.0046081542968750, 0.0015869140625000, 0.0052795410156250,
+-0.0065307617187500, 0.0044250488281250, -0.0036621093750000, 0.0044250488281250,
+-0.0055847167968750, 0.0108032226562500, -0.0142822265625000, 0.0095214843750000,
+-0.0115661621093750, 0.0064392089843750, -0.0089111328125000, 0.0080261230468750,
+-0.0108337402343750, 0.0059204101562500, -0.0096130371093750, 0.0021057128906250,
+-0.0061035156250000, 0.0041503906250000, -0.0075378417968750, 0.0010375976562500,
+-0.0080566406250000, 0.0003662109375000, -0.0061950683593750, 0.0013732910156250,
+-0.0069580078125000, 0.0012207031250000, -0.0096435546875000, 0.0010375976562500,
+-0.0089721679687500, 0.0010681152343750, -0.0073242187500000, 0.0023193359375000,
+-0.0089416503906250, 0.0004272460937500, -0.0049438476562500, -0.0018920898437500,
+-0.0010375976562500, -0.0000610351562500, -0.0047607421875000, -0.0015869140625000,
+0.0035400390625000, -0.0026245117187500, 0.0044860839843750, -0.0014953613281250,
+0.0032348632812500, -0.0007934570312500, 0.0066528320312500, -0.0007019042968750,
+0.0061035156250000, 0.0025939941406250, 0.0003356933593750, 0.0016784667968750,
+0.0021972656250000, 0.0017089843750000, 0.0021057128906250, 0.0037841796875000,
+-0.0021057128906250, 0.0032958984375000, -0.0009765625000000, 0.0053710937500000,
+-0.0014038085937500, 0.0051879882812500, -0.0021667480468750, 0.0037841796875000,
+-0.0013732910156250, 0.0050048828125000, -0.0007019042968750, 0.0057678222656250,
+-0.0010986328125000, -0.0009155273437500, 0.0008544921875000, 0.0005798339843750,
+0.0002746582031250, 0.0002136230468750, 0.0011901855468750, -0.0059204101562500,
+0.0038757324218750, -0.0047912597656250, 0.0029907226562500, 0.0022888183593750,
+0.0021972656250000, 0.0001220703125000, 0.0030517578125000, 0.0000000000000000,
+0.0023498535156250, 0.0064086914062500, 0.0013122558593750, 0.0053405761718750,
+0.0018615722656250, -0.0033569335937500, -0.0013732910156250, 0.0000305175781250,
+-0.0010681152343750, -0.0014038085937500, -0.0007324218750000, -0.0097961425781250,
+-0.0027160644531250, -0.0078430175781250, -0.0026550292968750, -0.0071411132812500,
+-0.0007934570312500, -0.0085754394531250, -0.0013122558593750, -0.0079345703125000,
+-0.0014953613281250, -0.0052795410156250, 0.0000610351562500, -0.0059509277343750,
+-0.0002441406250000, -0.0110778808593750, -0.0027160644531250, -0.0085144042968750,
+-0.0018310546875000, -0.0097351074218750, -0.0032958984375000, -0.0153808593750000,
+-0.0060424804687500, -0.0133972167968750, -0.0046386718750000, -0.0132446289062500,
+-0.0075073242187500, -0.0150451660156250, -0.0088806152343750, -0.0116882324218750,
+-0.0068969726562500, -0.0083923339843750, -0.0062866210937500, -0.0114440917968750,
+-0.0079040527343750, -0.0070190429687500, -0.0047912597656250, -0.0057983398437500,
+-0.0042419433593750, -0.0059814453125000, -0.0056152343750000, -0.0052490234375000,
+-0.0052185058593750, -0.0055541992187500, -0.0048217773437500, -0.0024719238281250,
+-0.0080261230468750, -0.0032348632812500, -0.0077514648437500, -0.0015563964843750,
+-0.0082092285156250, 0.0017089843750000, -0.0107421875000000, -0.0000305175781250,
+-0.0101318359375000, 0.0027465820312500, -0.0083923339843750, 0.0039367675781250,
+-0.0092163085937500, 0.0015869140625000, -0.0076293945312500, 0.0010070800781250,
+-0.0049438476562500, 0.0025939941406250, -0.0057983398437500, -0.0018005371093750,
+-0.0034790039062500, -0.0017395019531250, -0.0030517578125000, -0.0021667480468750,
+-0.0028686523437500, -0.0043029785156250, -0.0016479492187500, -0.0034179687500000,
+-0.0018005371093750, -0.0061645507812500, -0.0023498535156250, -0.0055236816406250,
+-0.0015563964843750, -0.0066528320312500, -0.0024414062500000, -0.0091857910156250,
+-0.0038146972656250, -0.0079650878906250, -0.0031433105468750, -0.0104980468750000,
+-0.0048217773437500, -0.0108032226562500, -0.0053100585937500, -0.0111999511718750,
+-0.0055541992187500, -0.0126647949218750, -0.0061950683593750, -0.0122985839843750,
+-0.0062866210937500, -0.0137939453125000, -0.0072326660156250, -0.0139465332031250,
+-0.0069580078125000, -0.0144958496093750, -0.0075988769531250, -0.0155639648437500,
+-0.0086059570312500, -0.0154113769531250, -0.0081787109375000, -0.0164184570312500,
+-0.0102233886718750, -0.0166625976562500, -0.0102233886718750, -0.0169372558593750,
+-0.0109558105468750, -0.0173034667968750, -0.0126342773437500, -0.0171203613281250,
+-0.0122070312500000, -0.0185241699218750, -0.0138244628906250, -0.0178833007812500,
+-0.0143737792968750, -0.0187072753906250, -0.0148315429687500, -0.0202636718750000,
+-0.0158081054687500, -0.0192565917968750, -0.0162658691406250, -0.0210876464843750,
+-0.0171813964843750, -0.0211181640625000, -0.0178222656250000, -0.0209350585937500,
+-0.0183410644531250, -0.0218200683593750, -0.0191345214843750, -0.0214843750000000,
+-0.0198059082031250, -0.0212097167968750, -0.0197753906250000, -0.0211791992187500,
+-0.0208129882812500, -0.0213623046875000, -0.0209655761718750, -0.0213012695312500,
+-0.0208740234375000, -0.0209655761718750, -0.0220031738281250, -0.0212097167968750,
+-0.0213012695312500, -0.0214233398437500, -0.0222473144531250, -0.0205688476562500,
+-0.0224914550781250, -0.0201110839843750, -0.0221557617187500, -0.0205993652343750,
+-0.0235290527343750, -0.0185852050781250, -0.0220642089843750, -0.0184631347656250,
+-0.0224914550781250, -0.0187988281250000, -0.0223083496093750, -0.0179748535156250,
+-0.0209655761718750, -0.0180053710937500, -0.0213623046875000, -0.0190734863281250,
+-0.0209960937500000, -0.0190124511718750, -0.0209350585937500, -0.0187988281250000,
+-0.0204467773437500, -0.0194702148437500, -0.0204467773437500, -0.0196228027343750,
+-0.0209045410156250, -0.0180053710937500, -0.0179748535156250, -0.0188598632812500,
+-0.0193481445312500, -0.0173034667968750, -0.0176086425781250, -0.0154418945312500,
+-0.0148925781250000, -0.0169067382812500, -0.0170898437500000, -0.0111694335937500,
+-0.0103759765625000, -0.0122680664062500, -0.0117797851562500, -0.0104980468750000,
+-0.0096740722656250, -0.0056152343750000, -0.0042419433593750, -0.0080566406250000,
+-0.0072937011718750, -0.0049743652343750, -0.0019836425781250, -0.0040893554687500,
+-0.0020141601562500, -0.0050354003906250, -0.0031738281250000, -0.0046691894531250,
+-0.0004882812500000, -0.0045471191406250, -0.0005493164062500, -0.0050659179687500,
+-0.0045471191406250, -0.0052490234375000, -0.0027160644531250, -0.0053405761718750,
+-0.0049133300781250, -0.0053710937500000, -0.0091552734375000, -0.0054321289062500,
+-0.0065002441406250, -0.0066223144531250, -0.0119934082031250, -0.0063476562500000,
+-0.0122680664062500, -0.0067443847656250, -0.0122070312500000, -0.0082397460937500,
+-0.0148925781250000, -0.0078735351562500, -0.0138549804687500, -0.0061645507812500,
+-0.0140380859375000, -0.0077209472656250, -0.0155334472656250, -0.0054016113281250,
+-0.0120849609375000, -0.0019836425781250, -0.0092773437500000, -0.0042724609375000,
+-0.0120239257812500, -0.0010681152343750, -0.0050964355468750, 0.0002746582031250,
+-0.0041198730468750, -0.0016479492187500, -0.0057983398437500, -0.0020141601562500,
+-0.0039062500000000, -0.0009155273437500, -0.0033264160156250, -0.0046386718750000,
+-0.0061645507812500, -0.0038757324218750, -0.0069580078125000, -0.0064697265625000,
+-0.0048828125000000, -0.0093994140625000, -0.0045776367187500, -0.0068359375000000,
+-0.0059509277343750, -0.0167541503906250, -0.0010375976562500, -0.0164184570312500,
+-0.0014343261718750, -0.0158691406250000, -0.0016784667968750, -0.0219726562500000,
+0.0010375976562500, -0.0210876464843750, 0.0007324218750000, -0.0149841308593750,
+0.0013122558593750, -0.0182800292968750, 0.0012207031250000, -0.0134582519531250,
+-0.0012512207031250, -0.0050048828125000, -0.0007019042968750, -0.0099182128906250,
+0.0014038085937500, -0.0028381347656250, -0.0100402832031250, -0.0007934570312500,
+-0.0085449218750000, -0.0029602050781250, -0.0106201171875000, -0.0021972656250000,
+-0.0198974609375000, -0.0014038085937500, -0.0160522460937500, -0.0029907226562500,
+-0.0122375488281250, -0.0041503906250000, -0.0173339843750000, -0.0009460449218750,
+-0.0124206542968750, 0.0006103515625000, -0.0037841796875000, -0.0020446777343750,
+-0.0079345703125000, 0.0057983398437500, 0.0004272460937500, 0.0077514648437500,
+0.0012817382812500, 0.0022583007812500, -0.0011901855468750, 0.0020446777343750,
+0.0007324218750000, 0.0057067871093750, 0.0015258789062500, -0.0060119628906250,
+0.0008850097656250, -0.0063171386718750, -0.0003051757812500, -0.0064392089843750,
+0.0017089843750000, -0.0149230957031250, 0.0052795410156250, -0.0179443359375000,
+0.0028076171875000, -0.0133666992187500, -0.0036315917968750, -0.0158691406250000,
+-0.0056762695312500, -0.0133361816406250, -0.0003356933593750, -0.0107727050781250,
+0.0026245117187500, -0.0124816894531250, -0.0008239746093750, -0.0016479492187500,
+0.0087585449218750, -0.0042724609375000, 0.0092163085937500, -0.0018920898437500,
+0.0075378417968750, 0.0077514648437500, 0.0119628906250000, 0.0038452148437500,
+0.0124206542968750, 0.0044555664062500, 0.0071411132812500, 0.0075073242187500,
+0.0072631835937500, 0.0047912597656250, 0.0075683593750000, 0.0008850097656250,
+0.0060729980468750, 0.0026245117187500, 0.0067138671875000, 0.0007019042968750,
+0.0032348632812500, -0.0004882812500000, 0.0041809082031250, 0.0022277832031250,
+0.0027465820312500, 0.0036315917968750, 0.0001220703125000, 0.0010375976562500,
+0.0016784667968750, 0.0036315917968750, -0.0057983398437500, 0.0053710937500000,
+-0.0043640136718750, 0.0037841796875000, -0.0061035156250000, 0.0021667480468750,
+-0.0115356445312500, 0.0026855468750000, -0.0095825195312500, 0.0019836425781250,
+-0.0172424316406250, 0.0018005371093750, -0.0144653320312500, 0.0019226074218750,
+-0.0199584960937500, 0.0016479492187500, -0.0295715332031250, 0.0016784667968750,
+-0.0237121582031250, 0.0030517578125000, -0.0321655273437500, 0.0016174316406250,
+-0.0357360839843750, 0.0037536621093750, -0.0303955078125000, 0.0082702636718750,
+-0.0291442871093750},
+{-0.0012817382812500, -0.0022277832031250, 0.0019226074218750, -0.0048828125000000,
+0.0016479492187500, 0.0001220703125000, -0.0028686523437500, 0.0006713867187500,
+-0.0003662109375000, -0.0001220703125000, -0.0041198730468750, 0.0018310546875000,
+-0.0108032226562500, 0.0015258789062500, -0.0069274902343750, -0.0001220703125000,
+-0.0116882324218750, 0.0006408691406250, -0.0140991210937500, -0.0002746582031250,
+-0.0108337402343750, -0.0015258789062500, -0.0094299316406250, -0.0009155273437500,
+-0.0113220214843750, -0.0064697265625000, -0.0067443847656250, -0.0040893554687500,
+-0.0058898925781250, -0.0078430175781250, -0.0062255859375000, -0.0154418945312500,
+-0.0051879882812500, -0.0114135742187500, -0.0054931640625000, -0.0134277343750000,
+-0.0039672851562500, -0.0172119140625000, -0.0043945312500000, -0.0126342773437500,
+-0.0033569335937500, -0.0077819824218750, -0.0014038085937500, -0.0109863281250000,
+-0.0027465820312500, -0.0047912597656250, -0.0030517578125000, -0.0032958984375000,
+-0.0025024414062500, -0.0053100585937500, -0.0010375976562500, -0.0042114257812500,
+-0.0009460449218750, -0.0032653808593750, -0.0027770996093750, -0.0072021484375000,
+-0.0005798339843750, -0.0072937011718750, 0.0013732910156250, -0.0063476562500000,
+-0.0014953613281250, -0.0078735351562500, -0.0031127929687500, -0.0080261230468750,
+-0.0015563964843750, -0.0051269531250000, -0.0111083984375000, -0.0057678222656250,
+-0.0101928710937500, -0.0060119628906250, -0.0100402832031250, -0.0031127929687500,
+-0.0162048339843750, -0.0037231445312500, -0.0151672363281250, -0.0115661621093750,
+-0.0123596191406250, -0.0086364746093750, -0.0157775878906250, -0.0107421875000000,
+-0.0099182128906250, -0.0186157226562500, -0.0026245117187500, -0.0161437988281250,
+-0.0081481933593750, -0.0172729492187500, 0.0010070800781250, -0.0180664062500000,
+0.0027465820312500, -0.0175170898437500, -0.0005798339843750, -0.0174255371093750,
+0.0006103515625000, -0.0211181640625000, -0.0098876953125000, -0.0098571777343750,
+-0.0025024414062500, -0.0156250000000000, 0.0005187988281250, -0.0091247558593750,
+-0.0039672851562500, 0.0007019042968750, -0.0082397460937500, -0.0050659179687500,
+-0.0054321289062500, -0.0016174316406250, -0.0071716308593750, 0.0032348632812500,
+-0.0107727050781250, -0.0026245117187500, -0.0037231445312500, -0.0084533691406250,
+0.0023193359375000, -0.0038146972656250, -0.0036621093750000, -0.0089111328125000,
+0.0062866210937500, -0.0135803222656250, 0.0105895996093750, -0.0061035156250000,
+0.0050964355468750, -0.0011291503906250, 0.0035400390625000, -0.0067138671875000,
+0.0064392089843750, 0.0074157714843750, -0.0027160644531250, 0.0086669921875000,
+-0.0026245117187500, 0.0044250488281250, -0.0018310546875000, 0.0088195800781250,
+-0.0066223144531250, 0.0106201171875000, -0.0061035156250000, 0.0027465820312500,
+-0.0023193359375000, 0.0013122558593750, -0.0019531250000000, 0.0043640136718750,
+-0.0045776367187500, 0.0033874511718750, -0.0042114257812500, 0.0018310546875000,
+-0.0024719238281250, 0.0081787109375000, -0.0098266601562500, 0.0072021484375000,
+-0.0087585449218750, 0.0080871582031250, -0.0115661621093750, 0.0132446289062500,
+-0.0171203613281250, 0.0111389160156250, -0.0137329101562500, 0.0085449218750000,
+-0.0215148925781250, 0.0137329101562500, -0.0216979980468750, 0.0040588378906250,
+-0.0215148925781250, -0.0061035156250000, -0.0256958007812500, 0.0028381347656250,
+-0.0245971679687500, -0.0152282714843750, -0.0225830078125000, -0.0182495117187500,
+-0.0242004394531250, -0.0134277343750000, -0.0220947265625000, -0.0183105468750000,
+-0.0188598632812500, -0.0194396972656250, -0.0206298828125000, -0.0090026855468750,
+-0.0163574218750000, -0.0118408203125000, -0.0162353515625000, -0.0086669921875000,
+-0.0165100097656250, 0.0008239746093750, -0.0143127441406250, -0.0038757324218750,
+-0.0147705078125000, -0.0013427734375000, -0.0161743164062500, 0.0036315917968750,
+-0.0158081054687500, -0.0040283203125000, -0.0158996582031250, -0.0093994140625000,
+-0.0171508789062500, -0.0029296875000000, -0.0171813964843750, -0.0216674804687500,
+-0.0163269042968750, -0.0187988281250000, -0.0162048339843750, -0.0221252441406250,
+-0.0170288085937500, -0.0350646972656250, -0.0170593261718750, -0.0286560058593750,
+-0.0165100097656250, -0.0390625000000000, -0.0180664062500000, -0.0389709472656250,
+-0.0187988281250000, -0.0386657714843750, -0.0172729492187500, -0.0443115234375000,
+-0.0167846679687500, -0.0418395996093750, -0.0178527832031250, -0.0404968261718750,
+-0.0142211914062500, -0.0424194335937500, -0.0144958496093750, -0.0395812988281250,
+-0.0139465332031250, -0.0360717773437500, -0.0116577148437500, -0.0384826660156250,
+-0.0125732421875000, -0.0337829589843750, -0.0101318359375000, -0.0340270996093750,
+-0.0105285644531250, -0.0334777832031250, -0.0099182128906250, -0.0309448242187500,
+-0.0079040527343750, -0.0322265625000000, -0.0089111328125000, -0.0285644531250000,
+-0.0082702636718750, -0.0293579101562500, -0.0078735351562500, -0.0279846191406250,
+-0.0080871582031250, -0.0252075195312500, -0.0084228515625000, -0.0272521972656250,
+-0.0083312988281250, -0.0232543945312500, -0.0071411132812500, -0.0244140625000000,
+-0.0075988769531250, -0.0234069824218750, -0.0070800781250000, -0.0207519531250000,
+-0.0057373046875000, -0.0233459472656250, -0.0060729980468750, -0.0180969238281250,
+-0.0055236816406250, -0.0202026367187500, -0.0055541992187500, -0.0185241699218750,
+-0.0049743652343750, -0.0140991210937500, -0.0043945312500000, -0.0174865722656250,
+-0.0049438476562500, -0.0113525390625000, -0.0039672851562500, -0.0131530761718750,
+-0.0034790039062500, -0.0119628906250000, -0.0047912597656250, -0.0077209472656250,
+-0.0054931640625000, -0.0117187500000000, -0.0048522949218750, -0.0062561035156250,
+-0.0080871582031250, -0.0077209472656250, -0.0083618164062500, -0.0075073242187500,
+-0.0083007812500000, -0.0043640136718750, -0.0099182128906250, -0.0075683593750000,
+-0.0097351074218750, -0.0041198730468750, -0.0093383789062500, -0.0055236816406250,
+-0.0091247558593750, -0.0057983398437500, -0.0098571777343750, -0.0039367675781250,
+-0.0099182128906250, -0.0068359375000000, -0.0091247558593750, -0.0058288574218750,
+-0.0122375488281250, -0.0068054199218750, -0.0108642578125000, -0.0083312988281250,
+-0.0126342773437500, -0.0084533691406250, -0.0157470703125000, -0.0094909667968750,
+-0.0134582519531250, -0.0110473632812500, -0.0188293457031250, -0.0111083984375000,
+-0.0191345214843750, -0.0125427246093750, -0.0187683105468750, -0.0142517089843750,
+-0.0217590332031250, -0.0139465332031250, -0.0216979980468750, -0.0171508789062500,
+-0.0193481445312500, -0.0171203613281250, -0.0205383300781250, -0.0183105468750000,
+-0.0201416015625000, -0.0209350585937500, -0.0179138183593750, -0.0203552246093750,
+-0.0188293457031250, -0.0230102539062500, -0.0187072753906250, -0.0231628417968750,
+-0.0182189941406250, -0.0229492187500000, -0.0187377929687500, -0.0242309570312500,
+-0.0188293457031250, -0.0240478515625000, -0.0181884765625000, -0.0225524902343750,
+-0.0199584960937500, -0.0234375000000000, -0.0199890136718750, -0.0220642089843750,
+-0.0182800292968750, -0.0204162597656250, -0.0193176269531250, -0.0216674804687500,
+-0.0202331542968750, -0.0165100097656250, -0.0113525390625000, -0.0174865722656250,
+-0.0143432617187500, -0.0152282714843750, -0.0115966796875000, -0.0108032226562500,
+-0.0030517578125000, -0.0128784179687500, -0.0068969726562500, -0.0058898925781250,
+-0.0037536621093750, -0.0067443847656250, -0.0025024414062500, -0.0043334960937500,
+-0.0018615722656250, 0.0011901855468750, -0.0011901855468750, -0.0013427734375000,
+-0.0026245117187500, 0.0049438476562500, 0.0013732910156250, 0.0049438476562500,
+0.0021057128906250, 0.0052490234375000, 0.0005187988281250, 0.0091247558593750,
+0.0001525878906250, 0.0079040527343750, 0.0015258789062500, 0.0068359375000000,
+0.0051574707031250, 0.0083923339843750, 0.0015258789062500, 0.0062561035156250,
+0.0044860839843750, 0.0036315917968750, 0.0116577148437500, 0.0053710937500000,
+0.0083007812500000, 0.0007324218750000, 0.0085754394531250, 0.0010681152343750,
+0.0115051269531250, 0.0001831054687500, 0.0075988769531250, -0.0028381347656250,
+0.0040283203125000, -0.0011901855468750, 0.0069274902343750, -0.0036621093750000,
+-0.0033264160156250, -0.0039672851562500, -0.0015258789062500, -0.0031127929687500,
+-0.0034484863281250, -0.0034179687500000, -0.0121765136718750, -0.0036315917968750,
+-0.0087890625000000, -0.0039367675781250, -0.0067443847656250, -0.0027770996093750,
+-0.0087890625000000, -0.0040588378906250, -0.0097351074218750, -0.0068664550781250,
+-0.0071716308593750, -0.0055541992187500, -0.0060729980468750, -0.0022583007812500,
+-0.0114135742187500, -0.0048828125000000, -0.0102233886718750, -0.0021972656250000,
+-0.0115966796875000, 0.0028991699218750, -0.0169677734375000, 0.0004577636718750,
+-0.0147705078125000, 0.0061645507812500, -0.0111999511718750, 0.0060119628906250,
+-0.0142822265625000, 0.0057983398437500, -0.0112609863281250, 0.0091247558593750,
+-0.0053710937500000, 0.0080566406250000, -0.0086059570312500, 0.0056457519531250,
+-0.0048217773437500, 0.0069274902343750, -0.0028076171875000, 0.0056152343750000,
+-0.0045166015625000, 0.0023193359375000, -0.0051879882812500, 0.0034484863281250,
+-0.0046386718750000, 0.0037231445312500, -0.0072326660156250, 0.0029602050781250,
+-0.0066528320312500, 0.0032958984375000, -0.0067138671875000, 0.0049743652343750,
+-0.0089416503906250, 0.0048217773437500, -0.0083007812500000, 0.0016174316406250,
+-0.0057373046875000, 0.0034179687500000, -0.0076293945312500, 0.0021667480468750,
+-0.0051269531250000, -0.0015258789062500, -0.0015869140625000, 0.0001525878906250,
+-0.0042419433593750, -0.0017700195312500, 0.0028381347656250, -0.0025024414062500,
+0.0032348632812500, -0.0026245117187500, 0.0025634765625000, -0.0027465820312500,
+0.0058288574218750, -0.0028076171875000, 0.0054016113281250, -0.0069885253906250,
+0.0028991699218750, -0.0048828125000000, 0.0029907226562500, -0.0076904296875000,
+0.0039978027343750, -0.0132751464843750, 0.0030822753906250, -0.0105590820312500,
+0.0022583007812500, -0.0151977539062500, 0.0053100585937500, -0.0166931152343750,
+0.0052185058593750, -0.0133666992187500, 0.0053710937500000, -0.0131835937500000,
+0.0073242187500000, -0.0155029296875000, 0.0065002441406250, -0.0093078613281250,
+0.0038757324218750, -0.0083923339843750, 0.0055541992187500, -0.0105285644531250,
+0.0034790039062500, -0.0091552734375000, -0.0007019042968750, -0.0076904296875000,
+0.0011291503906250, -0.0110473632812500, -0.0004272460937500, -0.0126953125000000,
+-0.0018310546875000, -0.0091247558593750, -0.0013122558593750, -0.0070190429687500,
+-0.0002746582031250, -0.0097656250000000, -0.0006713867187500, -0.0060729980468750,
+-0.0033264160156250, -0.0049133300781250, -0.0024719238281250, -0.0059814453125000,
+-0.0036315917968750, -0.0055847167968750, -0.0062561035156250, -0.0050659179687500,
+-0.0053710937500000, -0.0068664550781250, -0.0081481933593750, -0.0068054199218750,
+-0.0081481933593750, -0.0059509277343750, -0.0076599121093750, -0.0062255859375000,
+-0.0092468261718750, -0.0061950683593750, -0.0092468261718750, -0.0058288574218750,
+-0.0063171386718750, -0.0062561035156250, -0.0068969726562500, -0.0046081542968750,
+-0.0061340332031250, -0.0031433105468750, -0.0036926269531250, -0.0047912597656250,
+-0.0047302246093750, -0.0026550292968750, -0.0045471191406250, -0.0018615722656250,
+-0.0039672851562500, -0.0025329589843750, -0.0054016113281250, -0.0023803710937500,
+-0.0070190429687500, -0.0022583007812500, -0.0068664550781250, -0.0034484863281250,
+-0.0087585449218750},
+{0.0049133300781250, -0.0026245117187500, -0.0026855468750000, -0.0014343261718750,
+-0.0028076171875000, -0.0031127929687500, -0.0012512207031250, -0.0068054199218750,
+-0.0037231445312500, -0.0047607421875000, -0.0042724609375000, -0.0068359375000000,
+-0.0061645507812500, -0.0079345703125000, -0.0036010742187500, -0.0066223144531250,
+-0.0056762695312500, -0.0059814453125000, -0.0098571777343750, -0.0063476562500000,
+-0.0078125000000000, -0.0046997070312500, -0.0126953125000000, -0.0059509277343750,
+-0.0130920410156250, -0.0033874511718750, -0.0110168457031250, 0.0000000000000000,
+-0.0120849609375000, -0.0024719238281250, -0.0132751464843750, 0.0017395019531250,
+-0.0107116699218750, 0.0017089843750000, -0.0097656250000000, 0.0025634765625000,
+-0.0104064941406250, 0.0049133300781250, -0.0100708007812500, 0.0033569335937500,
+-0.0093688964843750, 0.0072021484375000, -0.0111083984375000, 0.0075378417968750,
+-0.0111389160156250, 0.0058898925781250, -0.0113830566406250, 0.0072326660156250,
+-0.0119934082031250, 0.0078430175781250, -0.0110473632812500, 0.0025329589843750,
+-0.0120544433593750, 0.0033569335937500, -0.0133056640625000, 0.0026245117187500,
+-0.0109252929687500, -0.0016174316406250, -0.0086364746093750, -0.0010681152343750,
+-0.0085754394531250, -0.0013427734375000, -0.0023193359375000, -0.0013732910156250,
+-0.0044860839843750, -0.0023803710937500, -0.0040893554687500, -0.0027160644531250,
+0.0009460449218750, -0.0018920898437500, -0.0005798339843750, -0.0064697265625000,
+-0.0001525878906250, -0.0056457519531250, 0.0004272460937500, -0.0066223144531250,
+0.0003662109375000, -0.0102844238281250, -0.0003967285156250, -0.0086975097656250,
+-0.0003967285156250, -0.0090332031250000, 0.0036010742187500, -0.0092468261718750,
+0.0021972656250000, -0.0114440917968750, 0.0031433105468750, -0.0120544433593750,
+0.0070800781250000, -0.0099182128906250, 0.0057373046875000, -0.0178833007812500,
+0.0063171386718750, -0.0174255371093750, 0.0074768066406250, -0.0175476074218750,
+0.0051574707031250, -0.0231933593750000, 0.0031738281250000, -0.0217285156250000,
+0.0049438476562500, -0.0152587890625000, 0.0013122558593750, -0.0185241699218750,
+0.0003967285156250, -0.0159912109375000, 0.0011901855468750, -0.0079345703125000,
+0.0005798339843750, -0.0111083984375000, 0.0001831054687500, -0.0121459960937500,
+-0.0000305175781250, -0.0093078613281250, 0.0007629394531250, -0.0098571777343750,
+-0.0009460449218750, -0.0143737792968750, -0.0032043457031250, -0.0139160156250000,
+-0.0018310546875000, -0.0057678222656250, -0.0025024414062500, -0.0082397460937500,
+-0.0037231445312500, -0.0067138671875000, -0.0031738281250000, 0.0003967285156250,
+-0.0020446777343750, -0.0022277832031250, -0.0021972656250000, -0.0024108886718750,
+-0.0023803710937500, 0.0001220703125000, -0.0019226074218750, -0.0024719238281250,
+-0.0029296875000000, -0.0061950683593750, -0.0034484863281250, -0.0039978027343750,
+-0.0025634765625000, -0.0063171386718750, -0.0072326660156250, -0.0070495605468750,
+-0.0064697265625000, -0.0063476562500000, -0.0081176757812500, -0.0069885253906250,
+-0.0122680664062500, -0.0071105957031250, -0.0109558105468750, -0.0046997070312500,
+-0.0142517089843750, -0.0052795410156250, -0.0151367187500000, -0.0055541992187500,
+-0.0154418945312500, -0.0039367675781250, -0.0168762207031250, -0.0040588378906250,
+-0.0170288085937500, -0.0046997070312500, -0.0180664062500000, -0.0045166015625000,
+-0.0181579589843750, -0.0055541992187500, -0.0186462402343750, -0.0064392089843750,
+-0.0192871093750000, -0.0056457519531250, -0.0184326171875000, -0.0089416503906250,
+-0.0179748535156250, -0.0087585449218750, -0.0183715820312500, -0.0092468261718750,
+-0.0158081054687500, -0.0112304687500000, -0.0138854980468750, -0.0106811523437500,
+-0.0150146484375000, -0.0129394531250000, -0.0088500976562500, -0.0126037597656250,
+-0.0093383789062500, -0.0132751464843750, -0.0058898925781250, -0.0151367187500000,
+0.0001220703125000, -0.0144348144531250, -0.0016479492187500, -0.0155334472656250,
+0.0063476562500000, -0.0156860351562500, 0.0065612792968750, -0.0160217285156250,
+0.0090026855468750, -0.0165710449218750, 0.0148315429687500, -0.0160217285156250,
+0.0134887695312500, -0.0172119140625000, 0.0193176269531250, -0.0173034667968750,
+0.0196838378906250, -0.0169677734375000, 0.0209350585937500, -0.0173950195312500,
+0.0247802734375000, -0.0172729492187500, 0.0236511230468750, -0.0159606933593750,
+0.0254516601562500, -0.0164489746093750, 0.0255432128906250, -0.0153503417968750,
+0.0249328613281250, -0.0139770507812500, 0.0253601074218750, -0.0148925781250000,
+0.0249023437500000, -0.0113830566406250, 0.0235900878906250, -0.0121459960937500,
+0.0232543945312500, -0.0107727050781250, 0.0223693847656250, -0.0077209472656250,
+0.0210876464843750, -0.0093688964843750, 0.0208129882812500, -0.0064697265625000,
+0.0191040039062500, -0.0058898925781250, 0.0177612304687500, -0.0064392089843750,
+0.0178833007812500, -0.0054016113281250, 0.0168762207031250, -0.0053710937500000,
+0.0149536132812500, -0.0068359375000000, 0.0175476074218750, -0.0061340332031250,
+0.0152282714843750, -0.0068664550781250, 0.0161437988281250, -0.0083007812500000,
+0.0187072753906250, -0.0072937011718750, 0.0159301757812500, -0.0092163085937500,
+0.0216674804687500, -0.0090332031250000, 0.0188903808593750, -0.0092468261718750,
+0.0201721191406250, -0.0103149414062500, 0.0246582031250000, -0.0096130371093750,
+0.0199890136718750, -0.0109863281250000, 0.0261840820312500, -0.0107421875000000,
+0.0252075195312500, -0.0108947753906250, 0.0263061523437500, -0.0117492675781250,
+0.0308532714843750, -0.0111999511718750, 0.0285644531250000, -0.0121765136718750,
+0.0331115722656250, -0.0115661621093750, 0.0331115722656250, -0.0128479003906250,
+0.0327148437500000, -0.0141296386718750, 0.0350036621093750, -0.0129394531250000,
+0.0339660644531250, -0.0176086425781250, 0.0323181152343750, -0.0172729492187500,
+0.0342407226562500, -0.0182800292968750, 0.0307312011718750, -0.0216979980468750,
+0.0277099609375000, -0.0205078125000000, 0.0317077636718750, -0.0221557617187500,
+0.0216064453125000, -0.0227661132812500, 0.0231628417968750, -0.0226745605468750,
+0.0195312500000000, -0.0233459472656250, 0.0121765136718750, -0.0234069824218750,
+0.0170593261718750, -0.0224609375000000, 0.0048828125000000, -0.0237426757812500,
+0.0075683593750000, -0.0217285156250000, 0.0044250488281250, -0.0197143554687500,
+-0.0048522949218750, -0.0216674804687500, 0.0007324218750000, -0.0155944824218750,
+-0.0099792480468750, -0.0166931152343750, -0.0081176757812500, -0.0149841308593750,
+-0.0098876953125000, -0.0102539062500000, -0.0168762207031250, -0.0126342773437500,
+-0.0126342773437500, -0.0079345703125000, -0.0220336914062500, -0.0079345703125000,
+-0.0204467773437500, -0.0076599121093750, -0.0231323242187500, -0.0048522949218750,
+-0.0312500000000000, -0.0058593750000000, -0.0275573730468750, -0.0056457519531250,
+-0.0326538085937500, -0.0048522949218750, -0.0346374511718750, -0.0058593750000000,
+-0.0318603515625000, -0.0068664550781250, -0.0325012207031250, -0.0062561035156250,
+-0.0338134765625000, -0.0086669921875000, -0.0231933593750000, -0.0083923339843750,
+-0.0263977050781250, -0.0084838867187500, -0.0203857421875000, -0.0106506347656250,
+-0.0094299316406250, -0.0101928710937500, -0.0152587890625000, -0.0068664550781250,
+-0.0012817382812500, -0.0083618164062500, -0.0012207031250000, -0.0072021484375000,
+-0.0003356933593750, -0.0033874511718750, 0.0079040527343750, -0.0043640136718750,
+0.0051269531250000, -0.0039062500000000, 0.0053405761718750, -0.0036010742187500,
+0.0083007812500000, -0.0030212402343750, 0.0051574707031250, -0.0022277832031250,
+0.0016479492187500, -0.0028076171875000, 0.0044555664062500, -0.0036315917968750,
+-0.0012207031250000, -0.0016784667968750, -0.0028991699218750, -0.0048828125000000,
+0.0003662109375000, -0.0085449218750000, 0.0010070800781250, -0.0055847167968750,
+-0.0009460449218750, -0.0117797851562500, 0.0025939941406250, -0.0124206542968750,
+0.0025024414062500, -0.0112304687500000, 0.0025024414062500, -0.0135192871093750,
+0.0055541992187500, -0.0137329101562500, 0.0047607421875000, -0.0111694335937500,
+-0.0018920898437500, -0.0103454589843750, 0.0008239746093750, -0.0134277343750000,
+-0.0021972656250000, -0.0141601562500000, -0.0097961425781250, -0.0118713378906250,
+-0.0062255859375000, -0.0197448730468750, -0.0080261230468750, -0.0198059082031250,
+-0.0110778808593750, -0.0185546875000000, -0.0084228515625000, -0.0224609375000000,
+-0.0043640136718750, -0.0221862792968750, -0.0062866210937500, -0.0154724121093750,
+-0.0087280273437500, -0.0173645019531250, -0.0061645507812500, -0.0162963867187500,
+-0.0094909667968750, -0.0079040527343750, -0.0185852050781250, -0.0141296386718750,
+-0.0088195800781250, -0.0113830566406250, -0.0145874023437500, -0.0140686035156250,
+-0.0092468261718750, -0.0103759765625000, -0.0134887695312500, -0.0065612792968750,
+-0.0212402343750000, -0.0095214843750000, -0.0184631347656250, -0.0026245117187500,
+-0.0202941894531250, -0.0024719238281250, -0.0230102539062500, -0.0025634765625000,
+-0.0185546875000000, 0.0009460449218750, -0.0149841308593750, -0.0001220703125000,
+-0.0183105468750000, -0.0005798339843750, -0.0114135742187500, -0.0002746582031250,
+-0.0116271972656250, 0.0001220703125000, -0.0087585449218750, -0.0005187988281250,
+-0.0037231445312500, -0.0007934570312500, -0.0074157714843750, 0.0039672851562500,
+-0.0004882812500000, 0.0020141601562500, 0.0003051757812500, 0.0032653808593750,
+0.0001220703125000, 0.0093383789062500, 0.0035705566406250, 0.0075683593750000,
+0.0022583007812500, 0.0022277832031250, -0.0010986328125000, 0.0051269531250000,
+0.0011596679687500, 0.0033874511718750, -0.0003356933593750, -0.0023803710937500,
+-0.0053405761718750, -0.0002441406250000, -0.0038452148437500, -0.0030212402343750,
+-0.0025939941406250, -0.0030822753906250, -0.0038452148437500, -0.0031738281250000,
+-0.0022888183593750, -0.0048828125000000, 0.0005187988281250, -0.0042419433593750,
+-0.0010681152343750, -0.0035095214843750, -0.0011901855468750, -0.0027160644531250,
+0.0019531250000000, -0.0069580078125000, -0.0040588378906250, -0.0081481933593750,
+-0.0096435546875000, -0.0045166015625000, -0.0046081542968750, -0.0202941894531250,
+-0.0163269042968750, -0.0170898437500000, -0.0175170898437500, -0.0213623046875000,
+-0.0159912109375000, -0.0338134765625000, -0.0201416015625000, -0.0271301269531250,
+-0.0198059082031250, -0.0354919433593750, -0.0184020996093750, -0.0384826660156250,
+-0.0179443359375000, -0.0332336425781250, -0.0193786621093750, -0.0340881347656250,
+-0.0201110839843750, -0.0372009277343750, -0.0188903808593750, -0.0222473144531250,
+-0.0209350585937500, -0.0260314941406250, -0.0220642089843750, -0.0214538574218750,
+-0.0202026367187500, -0.0090942382812500, -0.0189208984375000, -0.0156250000000000,
+-0.0202026367187500, -0.0037841796875000, -0.0189514160156250, -0.0043334960937500,
+-0.0173339843750000, -0.0032958984375000, -0.0196838378906250, 0.0034484863281250,
+-0.0218505859375000, 0.0001220703125000, -0.0201416015625000, 0.0057678222656250,
+-0.0233154296875000, 0.0055236816406250, -0.0243530273437500, 0.0053405761718750,
+-0.0232238769531250, 0.0079345703125000, -0.0233154296875000, 0.0066223144531250,
+-0.0240478515625000, 0.0094909667968750, -0.0233459472656250, 0.0084838867187500,
+-0.0231018066406250, 0.0095520019531250, -0.0235900878906250, 0.0121154785156250,
+-0.0240783691406250, 0.0101928710937500, -0.0238647460937500, 0.0133972167968750,
+-0.0230407714843750, 0.0135498046875000, -0.0238952636718750, 0.0130004882812500,
+-0.0226745605468750},
+{0.0040893554687500, -0.0098876953125000, 0.0016784667968750, -0.0177001953125000,
+0.0030212402343750, -0.0146789550781250, 0.0039367675781250, -0.0143432617187500,
+-0.0065612792968750, -0.0169067382812500, -0.0036926269531250, -0.0147094726562500,
+-0.0057373046875000, -0.0107116699218750, -0.0155029296875000, -0.0125122070312500,
+-0.0123901367187500, -0.0134277343750000, -0.0108337402343750, -0.0119018554687500,
+-0.0134582519531250, -0.0123596191406250, -0.0118408203125000, -0.0151062011718750,
+-0.0060729980468750, -0.0104980468750000, -0.0151062011718750, -0.0015869140625000,
+-0.0120849609375000, -0.0015563964843750, -0.0149536132812500, -0.0038452148437500,
+-0.0143432617187500, -0.0035400390625000, -0.0104064941406250, -0.0018310546875000,
+-0.0110778808593750, -0.0050048828125000, -0.0171813964843750, -0.0069580078125000,
+-0.0141906738281250, -0.0035400390625000, -0.0179443359375000, -0.0019226074218750,
+-0.0269470214843750, -0.0044860839843750, -0.0228576660156250, 0.0031433105468750,
+-0.0213928222656250, 0.0034484863281250, -0.0265502929687500, 0.0018310546875000,
+-0.0192871093750000, 0.0051269531250000, -0.0112609863281250, 0.0050964355468750,
+-0.0172729492187500, -0.0008544921875000, -0.0010681152343750, 0.0016479492187500,
+-0.0004577636718750, -0.0013732910156250, -0.0016784667968750, -0.0089721679687500,
+0.0053405761718750, -0.0052490234375000, 0.0040283203125000, -0.0041809082031250,
+0.0036621093750000, -0.0069274902343750, 0.0044555664062500, -0.0065307617187500,
+0.0028686523437500, -0.0025024414062500, 0.0014953613281250, -0.0028686523437500,
+0.0028076171875000, -0.0095520019531250, 0.0004577636718750, -0.0060729980468750,
+0.0001831054687500, -0.0109558105468750, -0.0004577636718750, -0.0186462402343750,
+-0.0016784667968750, -0.0137634277343750, -0.0006408691406250, -0.0286254882812500,
+-0.0015258789062500, -0.0275573730468750, -0.0025634765625000, -0.0264892578125000,
+-0.0020446777343750, -0.0358886718750000, -0.0003051757812500, -0.0339965820312500,
+-0.0009460449218750, -0.0246276855468750, -0.0068054199218750, -0.0295715332031250,
+-0.0034790039062500, -0.0223388671875000, -0.0079956054687500, -0.0112304687500000,
+-0.0165100097656250, -0.0184020996093750, -0.0120849609375000, 0.0004272460937500,
+-0.0184936523437500, -0.0010681152343750, -0.0203857421875000, 0.0009460449218750,
+-0.0184936523437500, 0.0127868652343750, -0.0187377929687500, 0.0077209472656250,
+-0.0192565917968750, 0.0137939453125000, -0.0201416015625000, 0.0148315429687500,
+-0.0187072753906250, 0.0146789550781250, -0.0204162597656250, 0.0168457031250000,
+-0.0235900878906250, 0.0153808593750000, -0.0220336914062500, 0.0178527832031250,
+-0.0219116210937500, 0.0180969238281250, -0.0235290527343750, 0.0176696777343750,
+-0.0220336914062500, 0.0187683105468750, -0.0193176269531250, 0.0186767578125000,
+-0.0205993652343750, 0.0174255371093750, -0.0214538574218750, 0.0181884765625000,
+-0.0198364257812500, 0.0173339843750000, -0.0218200683593750, 0.0160827636718750,
+-0.0248718261718750, 0.0168762207031250, -0.0233459472656250, 0.0127563476562500,
+-0.0252685546875000, 0.0146789550781250, -0.0261230468750000, 0.0119628906250000,
+-0.0254211425781250, 0.0072021484375000, -0.0252990722656250, 0.0103759765625000,
+-0.0255432128906250, 0.0038146972656250, -0.0249023437500000, 0.0042419433593750,
+-0.0252075195312500, 0.0041809082031250, -0.0241394042968750, 0.0002441406250000,
+-0.0233154296875000, 0.0014953613281250, -0.0241699218750000, 0.0005493164062500,
+-0.0205993652343750, -0.0004577636718750, -0.0210876464843750, 0.0010986328125000,
+-0.0202636718750000, 0.0016784667968750, -0.0177001953125000, 0.0002136230468750,
+-0.0188598632812500, 0.0040893554687500, -0.0166931152343750, 0.0028381347656250,
+-0.0170288085937500, 0.0043029785156250, -0.0161437988281250, 0.0072326660156250,
+-0.0145568847656250, 0.0044250488281250, -0.0156860351562500, 0.0097961425781250,
+-0.0124206542968750, 0.0087890625000000, -0.0128784179687500, 0.0093994140625000,
+-0.0116577148437500, 0.0128784179687500, -0.0089721679687500, 0.0108032226562500,
+-0.0101928710937500, 0.0139770507812500, -0.0066223144531250, 0.0138549804687500,
+-0.0063476562500000, 0.0140686035156250, -0.0060119628906250, 0.0156555175781250,
+-0.0037231445312500, 0.0142517089843750, -0.0039672851562500, 0.0158691406250000,
+-0.0042419433593750, 0.0155639648437500, -0.0032043457031250, 0.0151062011718750,
+-0.0040588378906250, 0.0158386230468750, -0.0054626464843750, 0.0152893066406250,
+-0.0044250488281250, 0.0145568847656250, -0.0065612792968750, 0.0148620605468750,
+-0.0070190429687500, 0.0139465332031250, -0.0066528320312500, 0.0130004882812500,
+-0.0074462890625000, 0.0133361816406250, -0.0079040527343750, 0.0100708007812500,
+-0.0075683593750000, 0.0102844238281250, -0.0072631835937500, 0.0088195800781250,
+-0.0088500976562500, 0.0059509277343750, -0.0097045898437500, 0.0067138671875000,
+-0.0086975097656250, 0.0032348632812500, -0.0125732421875000, 0.0027770996093750,
+-0.0121765136718750, 0.0018005371093750, -0.0126953125000000, -0.0005493164062500,
+-0.0148010253906250, -0.0002136230468750, -0.0136413574218750, -0.0017700195312500,
+-0.0165100097656250, -0.0027770996093750, -0.0158996582031250, -0.0023498535156250,
+-0.0160522460937500, -0.0027160644531250, -0.0185241699218750, -0.0036926269531250,
+-0.0178833007812500, -0.0018310546875000, -0.0166625976562500, -0.0029602050781250,
+-0.0179443359375000, -0.0012207031250000, -0.0170288085937500, 0.0011291503906250,
+-0.0151672363281250, -0.0010986328125000, -0.0162658691406250, 0.0042724609375000,
+-0.0134277343750000, 0.0037536621093750, -0.0144958496093750, 0.0058593750000000,
+-0.0123901367187500, 0.0101623535156250, -0.0091857910156250, 0.0085754394531250,
+-0.0114746093750000, 0.0156250000000000, -0.0056762695312500, 0.0151977539062500,
+-0.0057678222656250, 0.0170898437500000, -0.0057373046875000, 0.0222778320312500,
+-0.0022583007812500, 0.0200805664062500, -0.0031738281250000, 0.0234069824218750,
+-0.0050964355468750, 0.0247497558593750, -0.0040283203125000, 0.0228881835937500,
+-0.0035095214843750, 0.0232238769531250, -0.0058593750000000, 0.0241088867187500,
+-0.0060729980468750, 0.0170288085937500, -0.0008544921875000, 0.0187377929687500,
+-0.0023803710937500, 0.0147399902343750, -0.0016479492187500, 0.0077819824218750,
+0.0024719238281250, 0.0112915039062500, 0.0012817382812500, 0.0014038085937500,
+0.0032653808593750, 0.0021667480468750, 0.0038757324218750, -0.0009155273437500,
+0.0024719238281250, -0.0080871582031250, 0.0030517578125000, -0.0048217773437500,
+0.0039367675781250, -0.0147399902343750, -0.0020446777343750, -0.0145263671875000,
+-0.0007019042968750, -0.0147094726562500, -0.0003967285156250, -0.0203857421875000,
+-0.0049743652343750, -0.0186157226562500, -0.0043029785156250, -0.0181274414062500,
+-0.0006713867187500, -0.0197143554687500, -0.0014038085937500, -0.0172424316406250,
+-0.0012512207031250, -0.0150146484375000, 0.0014953613281250, -0.0166625976562500,
+0.0005798339843750, -0.0095520019531250, -0.0033264160156250, -0.0105590820312500,
+-0.0003662109375000, -0.0090026855468750, -0.0036010742187500, -0.0033874511718750,
+-0.0102539062500000, -0.0055847167968750, -0.0073242187500000, -0.0036315917968750,
+-0.0105590820312500, -0.0024719238281250, -0.0114440917968750, -0.0043334960937500,
+-0.0103454589843750, -0.0054321289062500, -0.0109863281250000, -0.0040588378906250,
+-0.0116271972656250, -0.0049438476562500, -0.0106811523437500, -0.0067443847656250,
+-0.0101013183593750, -0.0042724609375000, -0.0103759765625000, -0.0018920898437500,
+-0.0104980468750000, -0.0036926269531250, -0.0105285644531250, 0.0004272460937500,
+-0.0121154785156250, 0.0010986328125000, -0.0121459960937500, -0.0004577636718750,
+-0.0108642578125000, 0.0004272460937500, -0.0106811523437500, 0.0010681152343750,
+-0.0116882324218750, -0.0021667480468750, -0.0104370117187500, -0.0018310546875000,
+-0.0102539062500000, -0.0025939941406250, -0.0094909667968750, -0.0052490234375000,
+-0.0088195800781250, -0.0050048828125000, -0.0087585449218750, -0.0066528320312500,
+-0.0041198730468750, -0.0064086914062500, -0.0063171386718750, -0.0068359375000000,
+-0.0050354003906250, -0.0076904296875000, -0.0004882812500000, -0.0069885253906250,
+-0.0026855468750000, -0.0100097656250000, 0.0001220703125000, -0.0092468261718750,
+0.0008544921875000, -0.0099182128906250, -0.0005798339843750, -0.0125732421875000,
+-0.0004272460937500, -0.0116271972656250, -0.0000915527343750, -0.0120849609375000,
+-0.0023498535156250, -0.0121765136718750, -0.0020751953125000, -0.0127258300781250,
+-0.0035400390625000, -0.0133361816406250, -0.0057678222656250, -0.0127563476562500,
+-0.0042724609375000, -0.0131530761718750, -0.0060119628906250, -0.0126953125000000,
+-0.0074157714843750, -0.0132446289062500, -0.0057373046875000, -0.0140686035156250,
+-0.0050048828125000, -0.0133361816406250, -0.0061950683593750, -0.0148010253906250,
+-0.0014953613281250, -0.0147705078125000, -0.0026550292968750, -0.0134277343750000,
+-0.0013732910156250, -0.0140075683593750, 0.0029296875000000, -0.0149841308593750,
+0.0010070800781250, -0.0104675292968750, 0.0018615722656250, -0.0113830566406250,
+0.0040283203125000, -0.0098266601562500, 0.0000000000000000, -0.0053100585937500,
+-0.0027770996093750, -0.0074462890625000, 0.0003356933593750, -0.0090026855468750,
+-0.0096435546875000, -0.0063476562500000, -0.0097351074218750, -0.0070190429687500,
+-0.0101623535156250, -0.0109863281250000, -0.0156250000000000, -0.0103149414062500,
+-0.0140075683593750, -0.0079040527343750, -0.0148010253906250, -0.0092773437500000,
+-0.0162963867187500, -0.0079650878906250, -0.0141296386718750, -0.0049133300781250,
+-0.0122985839843750, -0.0069580078125000, -0.0136718750000000, -0.0068664550781250,
+-0.0095214843750000, -0.0049133300781250, -0.0092773437500000, -0.0072326660156250,
+-0.0079956054687500, -0.0100097656250000, -0.0053405761718750, -0.0083312988281250,
+-0.0056152343750000, -0.0119934082031250, 0.0000915527343750, -0.0120849609375000,
+-0.0004882812500000, -0.0120849609375000, 0.0033264160156250, -0.0144348144531250,
+0.0094909667968750, -0.0141906738281250, 0.0072631835937500, -0.0128173828125000,
+0.0141296386718750, -0.0136108398437500, 0.0152893066406250, -0.0139160156250000,
+0.0144348144531250, -0.0128784179687500, 0.0166625976562500, -0.0130920410156250,
+0.0158996582031250, -0.0135803222656250, 0.0122680664062500, -0.0137939453125000,
+0.0131530761718750, -0.0136108398437500, 0.0100708007812500, -0.0136413574218750,
+0.0057067871093750, -0.0137939453125000, 0.0076599121093750, -0.0128479003906250,
+0.0017089843750000, -0.0134582519531250, 0.0011291503906250, -0.0122375488281250,
+0.0003967285156250, -0.0105590820312500, -0.0030212402343750, -0.0117187500000000,
+-0.0021972656250000, -0.0097351074218750, -0.0024719238281250, -0.0091857910156250,
+-0.0032043457031250, -0.0094299316406250, -0.0024108886718750, -0.0090942382812500,
+-0.0015869140625000, -0.0090332031250000, -0.0018310546875000, -0.0077819824218750,
+0.0003356933593750, -0.0080871582031250, 0.0002746582031250, -0.0077819824218750,
+0.0016784667968750, -0.0064392089843750, 0.0038757324218750, -0.0068054199218750,
+0.0033569335937500, -0.0071105957031250, 0.0060424804687500, -0.0064086914062500,
+0.0067749023437500, -0.0070190429687500, 0.0071411132812500, -0.0079040527343750,
+0.0081787109375000, -0.0072326660156250, 0.0080566406250000, -0.0082092285156250,
+0.0091247558593750, -0.0082397460937500, 0.0092163085937500, -0.0080871582031250,
+0.0099487304687500, -0.0085144042968750, 0.0110168457031250, -0.0084533691406250,
+0.0105590820312500},
+{-0.0039978027343750, -0.0112304687500000, -0.0041198730468750, -0.0078430175781250,
+-0.0055236816406250, -0.0078430175781250, -0.0057067871093750, -0.0090332031250000,
+-0.0058593750000000, -0.0077819824218750, -0.0064086914062500, -0.0070190429687500,
+-0.0067443847656250, -0.0097961425781250, -0.0078430175781250, -0.0103454589843750,
+-0.0078125000000000, -0.0091247558593750, -0.0070495605468750, -0.0092773437500000,
+-0.0079345703125000, -0.0097656250000000, -0.0087280273437500, -0.0076293945312500,
+-0.0043640136718750, -0.0076293945312500, -0.0054016113281250, -0.0081787109375000,
+-0.0041503906250000, -0.0067749023437500, -0.0003051757812500, -0.0061645507812500,
+-0.0025329589843750, -0.0090942382812500, -0.0016174316406250, -0.0085144042968750,
+0.0007629394531250, -0.0086364746093750, -0.0032958984375000, -0.0111389160156250,
+-0.0064392089843750, -0.0103454589843750, -0.0035095214843750, -0.0067749023437500,
+-0.0128173828125000, -0.0082397460937500, -0.0122680664062500, -0.0080566406250000,
+-0.0123291015625000, -0.0048522949218750, -0.0183410644531250, -0.0054016113281250,
+-0.0169677734375000, -0.0062255859375000, -0.0141906738281250, -0.0048828125000000,
+-0.0166015625000000, -0.0073852539062500, -0.0134582519531250, -0.0103454589843750,
+-0.0090637207031250, -0.0076293945312500, -0.0117797851562500, -0.0095214843750000,
+-0.0032958984375000, -0.0112609863281250, -0.0040283203125000, -0.0101928710937500,
+-0.0017089843750000, -0.0091552734375000, 0.0045166015625000, -0.0097656250000000,
+0.0019226074218750, -0.0092773437500000, 0.0083007812500000, -0.0088806152343750,
+0.0083618164062500, -0.0107421875000000, 0.0098266601562500, -0.0115661621093750,
+0.0141601562500000, -0.0101928710937500, 0.0120239257812500, -0.0135498046875000,
+0.0138244628906250, -0.0139465332031250, 0.0150146484375000, -0.0143127441406250,
+0.0121765136718750, -0.0158386230468750, 0.0105590820312500, -0.0151977539062500,
+0.0115051269531250, -0.0169067382812500, 0.0051879882812500, -0.0174865722656250,
+0.0048522949218750, -0.0164184570312500, 0.0033874511718750, -0.0163879394531250,
+-0.0006408691406250, -0.0172729492187500, 0.0007629394531250, -0.0150451660156250,
+-0.0011291503906250, -0.0155029296875000, -0.0016174316406250, -0.0144958496093750,
+-0.0003662109375000, -0.0127258300781250, -0.0001831054687500, -0.0140380859375000,
+-0.0002746582031250, -0.0111999511718750, 0.0022583007812500, -0.0112304687500000,
+0.0020751953125000, -0.0108337402343750, 0.0031433105468750, -0.0093383789062500,
+0.0054626464843750, -0.0099487304687500, 0.0047912597656250, -0.0074462890625000,
+0.0061645507812500, -0.0077514648437500, 0.0068054199218750, -0.0075073242187500,
+0.0066833496093750, -0.0055541992187500, 0.0070190429687500, -0.0060729980468750,
+0.0072021484375000, -0.0064392089843750, 0.0062255859375000, -0.0059814453125000,
+0.0063171386718750, -0.0059509277343750, 0.0050048828125000, -0.0065612792968750,
+0.0036926269531250, -0.0064086914062500, 0.0042114257812500, -0.0052795410156250,
+0.0007019042968750, -0.0053710937500000, 0.0005187988281250, -0.0054016113281250,
+-0.0005798339843750, -0.0045166015625000, -0.0030212402343750, -0.0044555664062500,
+-0.0021667480468750, -0.0057678222656250, -0.0043640136718750, -0.0051574707031250,
+-0.0045776367187500, -0.0057983398437500, -0.0046081542968750, -0.0071716308593750,
+-0.0057067871093750, -0.0064392089843750, -0.0053100585937500, -0.0078125000000000,
+-0.0053100585937500, -0.0079345703125000, -0.0056152343750000, -0.0077819824218750,
+-0.0056762695312500, -0.0084533691406250, -0.0056152343750000, -0.0083923339843750,
+-0.0057983398437500, -0.0074462890625000, -0.0059814453125000, -0.0081787109375000,
+-0.0065612792968750, -0.0072937011718750, -0.0064392089843750, -0.0057373046875000,
+-0.0062866210937500, -0.0068054199218750, -0.0068359375000000, -0.0065002441406250,
+-0.0059509277343750, -0.0053100585937500, -0.0067749023437500, -0.0076293945312500,
+-0.0065002441406250, -0.0093383789062500, -0.0055236816406250, -0.0076599121093750,
+-0.0064392089843750, -0.0132446289062500, -0.0044250488281250, -0.0131225585937500,
+-0.0053100585937500, -0.0132446289062500, -0.0033264160156250, -0.0165100097656250,
+-0.0010375976562500, -0.0155944824218750, -0.0030517578125000, -0.0148925781250000,
+0.0042419433593750, -0.0158081054687500, 0.0020751953125000, -0.0143432617187500,
+0.0050354003906250, -0.0124206542968750, 0.0112304687500000, -0.0131835937500000,
+0.0070800781250000, -0.0103759765625000, 0.0166625976562500, -0.0101013183593750,
+0.0153808593750000, -0.0094299316406250, 0.0165100097656250, -0.0074768066406250,
+0.0232849121093750, -0.0079040527343750, 0.0207214355468750, -0.0069274902343750,
+0.0236206054687500, -0.0064697265625000, 0.0252380371093750, -0.0066223144531250,
+0.0230102539062500, -0.0064086914062500, 0.0228881835937500, -0.0062561035156250,
+0.0244140625000000, -0.0069274902343750, 0.0173339843750000, -0.0063476562500000,
+0.0204162597656250, -0.0074768066406250, 0.0168151855468750, -0.0086975097656250,
+0.0090637207031250, -0.0076599121093750, 0.0138549804687500, -0.0112915039062500,
+0.0063476562500000, -0.0110168457031250, 0.0058593750000000, -0.0113220214843750,
+0.0074462890625000, -0.0136108398437500, 0.0046691894531250, -0.0129089355468750,
+0.0053100585937500, -0.0134582519531250, 0.0080871582031250, -0.0135192871093750,
+0.0079956054687500, -0.0142822265625000, 0.0075378417968750, -0.0148315429687500,
+0.0084533691406250, -0.0140686035156250, 0.0088195800781250, -0.0163879394531250,
+0.0086975097656250, -0.0170288085937500, 0.0078125000000000, -0.0153808593750000,
+0.0090026855468750, -0.0155029296875000, 0.0098876953125000, -0.0166320800781250,
+0.0087890625000000, -0.0119018554687500, 0.0133666992187500, -0.0121765136718750,
+0.0120849609375000, -0.0118103027343750, 0.0142517089843750, -0.0088500976562500,
+0.0189819335937500, -0.0097961425781250, 0.0166320800781250, -0.0094909667968750,
+0.0204467773437500, -0.0086059570312500, 0.0223083496093750, -0.0097656250000000,
+0.0184020996093750, -0.0114135742187500, 0.0183715820312500, -0.0102539062500000,
+0.0210876464843750, -0.0095825195312500, 0.0065917968750000, -0.0120544433593750,
+0.0114746093750000, -0.0075073242187500, 0.0035095214843750, -0.0026855468750000,
+-0.0118408203125000, -0.0066528320312500, -0.0033264160156250, 0.0018310546875000,
+-0.0208435058593750, 0.0027465820312500, -0.0229797363281250, 0.0025329589843750,
+-0.0195312500000000, 0.0061035156250000, -0.0260620117187500, 0.0044860839843750,
+-0.0263366699218750, 0.0033264160156250, -0.0172424316406250, 0.0058898925781250,
+-0.0199890136718750, 0.0021362304687500, -0.0135192871093750, -0.0028686523437500,
+-0.0036315917968750, 0.0000610351562500, -0.0101623535156250, -0.0050659179687500,
+0.0024414062500000, -0.0050964355468750, 0.0060729980468750, -0.0073242187500000,
+0.0022277832031250, -0.0114440917968750, 0.0041503906250000, -0.0086059570312500,
+0.0052185058593750, -0.0122680664062500, -0.0039978027343750, -0.0151672363281250,
+-0.0027770996093750, -0.0101013183593750, -0.0019531250000000, -0.0069885253906250,
+-0.0075073242187500, -0.0107727050781250, -0.0075073242187500, -0.0018310546875000,
+-0.0046691894531250, -0.0007629394531250, -0.0046997070312500, -0.0026550292968750,
+-0.0060729980468750, 0.0005187988281250, -0.0058898925781250, 0.0007629394531250,
+-0.0100402832031250, 0.0002441406250000, -0.0051574707031250, -0.0004577636718750,
+-0.0055847167968750, -0.0013732910156250, -0.0074462890625000, -0.0003662109375000,
+-0.0065917968750000, 0.0002746582031250, -0.0054016113281250, -0.0072326660156250,
+-0.0126037597656250, -0.0047912597656250, -0.0119934082031250, -0.0078735351562500,
+-0.0108337402343750, -0.0154418945312500, -0.0151062011718750, -0.0116577148437500,
+-0.0154724121093750, -0.0157165527343750, -0.0101013183593750, -0.0174560546875000,
+-0.0099182128906250, -0.0164489746093750, -0.0121765136718750, -0.0160522460937500,
+-0.0107116699218750, -0.0161743164062500, -0.0096740722656250, -0.0180358886718750,
+-0.0158691406250000, -0.0157165527343750, -0.0167541503906250, -0.0202026367187500,
+-0.0123901367187500, -0.0260314941406250, -0.0141906738281250, -0.0208740234375000,
+-0.0172424316406250, -0.0237731933593750, 0.0004577636718750, -0.0297546386718750,
+-0.0020141601562500, -0.0197448730468750, -0.0003662109375000, -0.0119934082031250,
+0.0122070312500000, -0.0195617675781250, 0.0081787109375000, 0.0025634765625000,
+0.0067138671875000, 0.0004272460937500, 0.0104675292968750, 0.0041503906250000,
+0.0061950683593750, 0.0193481445312500, -0.0004577636718750, 0.0124206542968750,
+0.0029602050781250, 0.0197753906250000, 0.0011901855468750, 0.0240478515625000,
+-0.0005798339843750, 0.0185852050781250, -0.0006103515625000, 0.0161132812500000,
+0.0004272460937500, 0.0193786621093750, 0.0009155273437500, 0.0117492675781250,
+-0.0032348632812500, 0.0119628906250000, -0.0025329589843750, 0.0112304687500000,
+-0.0036315917968750, 0.0065307617187500, -0.0066223144531250, 0.0086364746093750,
+-0.0053405761718750, 0.0087890625000000, -0.0086669921875000, 0.0069274902343750,
+-0.0081481933593750, 0.0092163085937500, -0.0093994140625000, 0.0119628906250000,
+-0.0119628906250000, 0.0099487304687500, -0.0105285644531250, 0.0122680664062500,
+-0.0149536132812500, 0.0130920410156250, -0.0149230957031250, 0.0130615234375000,
+-0.0149536132812500, 0.0131835937500000, -0.0174560546875000, 0.0128479003906250,
+-0.0170593261718750, 0.0156555175781250, -0.0168457031250000, 0.0144042968750000,
+-0.0166015625000000, 0.0150146484375000, -0.0178527832031250, 0.0177307128906250,
+-0.0184631347656250, 0.0163879394531250, -0.0174255371093750, 0.0164794921875000,
+-0.0213012695312500, 0.0173645019531250, -0.0214233398437500, 0.0159912109375000,
+-0.0206298828125000, 0.0148315429687500, -0.0221862792968750, 0.0156250000000000,
+-0.0222167968750000, 0.0121154785156250, -0.0197448730468750, 0.0135498046875000,
+-0.0198974609375000, 0.0103149414062500, -0.0200500488281250, 0.0063476562500000,
+-0.0186157226562500, 0.0098266601562500, -0.0186462402343750, 0.0000610351562500,
+-0.0199584960937500, 0.0021972656250000, -0.0198669433593750, -0.0000915527343750,
+-0.0196533203125000, -0.0073852539062500, -0.0203247070312500, -0.0028686523437500,
+-0.0205078125000000, -0.0094299316406250, -0.0198974609375000, -0.0098876953125000,
+-0.0198669433593750, -0.0094604492187500, -0.0202331542968750, -0.0126037597656250,
+-0.0203857421875000, -0.0118103027343750, -0.0202331542968750, -0.0112915039062500,
+-0.0195617675781250, -0.0126953125000000, -0.0203857421875000, -0.0113525390625000,
+-0.0186767578125000, -0.0100402832031250, -0.0166320800781250, -0.0116271972656250,
+-0.0178527832031250, -0.0069274902343750, -0.0146179199218750, -0.0094604492187500,
+-0.0141601562500000, -0.0075988769531250, -0.0140991210937500, -0.0030822753906250,
+-0.0127258300781250, -0.0072326660156250, -0.0129089355468750, -0.0016784667968750,
+-0.0122985839843750, -0.0031433105468750, -0.0122680664062500, -0.0028076171875000,
+-0.0120849609375000, 0.0003967285156250, -0.0117187500000000, -0.0033264160156250,
+-0.0120239257812500, -0.0003662109375000, -0.0114440917968750, -0.0016784667968750,
+-0.0117492675781250, -0.0022888183593750, -0.0116271972656250, -0.0009155273437500,
+-0.0110168457031250, -0.0032348632812500, -0.0115661621093750, -0.0021667480468750,
+-0.0111999511718750, -0.0032653808593750, -0.0110473632812500, -0.0042114257812500,
+-0.0111999511718750, -0.0041809082031250, -0.0112304687500000, -0.0058593750000000,
+-0.0111389160156250, -0.0065612792968750, -0.0103759765625000, -0.0079345703125000,
+-0.0104370117187500},
+{-0.0086059570312500, 0.0114746093750000, -0.0117797851562500, 0.0089416503906250,
+-0.0067749023437500, 0.0172424316406250, -0.0004577636718750, 0.0164489746093750,
+-0.0048828125000000, 0.0112304687500000, 0.0049438476562500, 0.0138244628906250,
+0.0061340332031250, 0.0112915039062500, 0.0036315917968750, 0.0057983398437500,
+0.0071716308593750, 0.0089416503906250, 0.0078125000000000, 0.0035400390625000,
+0.0001831054687500, 0.0019531250000000, 0.0007629394531250, 0.0049438476562500,
+0.0014343261718750, 0.0050048828125000, -0.0023193359375000, 0.0034790039062500,
+-0.0021667480468750, 0.0086975097656250, -0.0011596679687500, 0.0079956054687500,
+-0.0008544921875000, 0.0106811523437500, -0.0015258789062500, 0.0147399902343750,
+-0.0015869140625000, 0.0117187500000000, -0.0013427734375000, 0.0198059082031250,
+-0.0044860839843750, 0.0192565917968750, -0.0034179687500000, 0.0195007324218750,
+-0.0052185058593750, 0.0235595703125000, -0.0087890625000000, 0.0213623046875000,
+-0.0070800781250000, 0.0251159667968750, -0.0099182128906250, 0.0249023437500000,
+-0.0101623535156250, 0.0238952636718750, -0.0108337402343750, 0.0263366699218750,
+-0.0123596191406250, 0.0258789062500000, -0.0114135742187500, 0.0197753906250000,
+-0.0141296386718750, 0.0235595703125000, -0.0142517089843750, 0.0183105468750000,
+-0.0140991210937500, 0.0101623535156250, -0.0152282714843750, 0.0157165527343750,
+-0.0150451660156250, 0.0029907226562500, -0.0155944824218750, 0.0036010742187500,
+-0.0150756835937500, 0.0024108886718750, -0.0160217285156250, -0.0053100585937500,
+-0.0173950195312500, -0.0018615722656250, -0.0164489746093750, -0.0055847167968750,
+-0.0177612304687500, -0.0077819824218750, -0.0183715820312500, -0.0045776367187500,
+-0.0173950195312500, -0.0042419433593750, -0.0170288085937500, -0.0066833496093750,
+-0.0176696777343750, 0.0027465820312500, -0.0166931152343750, -0.0005493164062500,
+-0.0162048339843750, 0.0033874511718750, -0.0172119140625000, 0.0116882324218750,
+-0.0177307128906250, 0.0060119628906250, -0.0170898437500000, 0.0190429687500000,
+-0.0183410644531250, 0.0171508789062500, -0.0187988281250000, 0.0198364257812500,
+-0.0178833007812500, 0.0288391113281250, -0.0174560546875000, 0.0236816406250000,
+-0.0178833007812500, 0.0325012207031250, -0.0164184570312500, 0.0320434570312500,
+-0.0160217285156250, 0.0323791503906250, -0.0160522460937500, 0.0372009277343750,
+-0.0157470703125000, 0.0345153808593750, -0.0155944824218750, 0.0375671386718750,
+-0.0152282714843750, 0.0382690429687500, -0.0153808593750000, 0.0373840332031250,
+-0.0149536132812500, 0.0382690429687500, -0.0144042968750000, 0.0386657714843750,
+-0.0147705078125000, 0.0371093750000000, -0.0143127441406250, 0.0377197265625000,
+-0.0145263671875000, 0.0366210937500000, -0.0144348144531250, 0.0352172851562500,
+-0.0141906738281250, 0.0367431640625000, -0.0146789550781250, 0.0336914062500000,
+-0.0138854980468750, 0.0343627929687500, -0.0140686035156250, 0.0335998535156250,
+-0.0140686035156250, 0.0313110351562500, -0.0135498046875000, 0.0328979492187500,
+-0.0137023925781250, 0.0308227539062500, -0.0129089355468750, 0.0311279296875000,
+-0.0131225585937500, 0.0310974121093750, -0.0124511718750000, 0.0300292968750000,
+-0.0111999511718750, 0.0307617187500000, -0.0116577148437500, 0.0296020507812500,
+-0.0108642578125000, 0.0301818847656250, -0.0104064941406250, 0.0302734375000000,
+-0.0104370117187500, 0.0294799804687500, -0.0105895996093750, 0.0300598144531250,
+-0.0104064941406250, 0.0294494628906250, -0.0088195800781250, 0.0293884277343750,
+-0.0097351074218750, 0.0291442871093750, -0.0084533691406250, 0.0288391113281250,
+-0.0064392089843750, 0.0286254882812500, -0.0077209472656250, 0.0267639160156250,
+-0.0044555664062500, 0.0271911621093750, -0.0051574707031250, 0.0259704589843750,
+-0.0037536621093750, 0.0238647460937500, -0.0006408691406250, 0.0240783691406250,
+-0.0025024414062500, 0.0196838378906250, -0.0005493164062500, 0.0199279785156250,
+0.0009460449218750, 0.0168151855468750, -0.0007324218750000, 0.0121154785156250,
+-0.0017395019531250, 0.0135192871093750, -0.0004272460937500, 0.0070190429687500,
+-0.0033264160156250, 0.0065307617187500, -0.0030517578125000, 0.0044860839843750,
+-0.0040283203125000, -0.0000305175781250, -0.0063781738281250, 0.0009765625000000,
+-0.0048217773437500, -0.0032043457031250, -0.0064392089843750, -0.0035705566406250,
+-0.0082702636718750, -0.0034790039062500, -0.0055541992187500, -0.0056457519531250,
+-0.0033874511718750, -0.0051269531250000, -0.0060729980468750, -0.0029296875000000,
+-0.0037536621093750, -0.0039672851562500, -0.0016174316406250, -0.0019226074218750,
+-0.0045471191406250, 0.0011596679687500, -0.0068359375000000, -0.0002746582031250,
+-0.0047607421875000, 0.0045776367187500, -0.0090942382812500, 0.0053100585937500,
+-0.0093688964843750, 0.0041503906250000, -0.0097961425781250, 0.0055847167968750,
+-0.0122375488281250, 0.0058898925781250, -0.0109863281250000, 0.0023803710937500,
+-0.0110473632812500, 0.0032958984375000, -0.0131225585937500, 0.0014648437500000,
+-0.0099487304687500, -0.0024414062500000, -0.0063781738281250, -0.0008850097656250,
+-0.0094909667968750, -0.0030212402343750, -0.0062255859375000, -0.0043640136718750,
+-0.0040283203125000, -0.0015258789062500, -0.0066833496093750, -0.0004272460937500,
+-0.0080566406250000, -0.0023498535156250, -0.0064392089843750, 0.0054321289062500,
+-0.0117492675781250, 0.0049438476562500, -0.0123291015625000, 0.0052795410156250,
+-0.0096435546875000, 0.0100402832031250, -0.0110778808593750, 0.0086975097656250,
+-0.0127258300781250, 0.0084533691406250, -0.0052795410156250, 0.0097351074218750,
+-0.0067749023437500, 0.0075683593750000, -0.0042419433593750, 0.0059204101562500,
+0.0022888183593750, 0.0071716308593750, -0.0010375976562500, -0.0011596679687500,
+0.0018920898437500, 0.0008544921875000, 0.0032653808593750, -0.0028686523437500,
+0.0020751953125000, -0.0107727050781250, 0.0022583007812500, -0.0070495605468750,
+0.0027160644531250, -0.0149536132812500, -0.0004577636718750, -0.0154418945312500,
+-0.0007019042968750, -0.0152282714843750, -0.0003051757812500, -0.0195007324218750,
+-0.0011596679687500, -0.0180664062500000, -0.0010681152343750, -0.0150451660156250,
+-0.0014038085937500, -0.0179138183593750, -0.0016784667968750, -0.0135192871093750,
+-0.0018005371093750, -0.0073242187500000, -0.0015869140625000, -0.0110778808593750,
+-0.0014343261718750, -0.0041503906250000, -0.0028686523437500, -0.0032653808593750,
+-0.0027160644531250, -0.0042724609375000, -0.0033264160156250, -0.0017395019531250,
+-0.0048217773437500, -0.0024719238281250, -0.0049743652343750, -0.0052795410156250,
+-0.0059509277343750, -0.0037231445312500, -0.0056152343750000, -0.0047607421875000,
+-0.0056457519531250, -0.0075378417968750, -0.0065002441406250, -0.0059204101562500,
+-0.0062866210937500, -0.0096130371093750, -0.0060119628906250, -0.0101013183593750,
+-0.0060424804687500, -0.0083007812500000, -0.0059204101562500, -0.0096435546875000,
+-0.0054626464843750, -0.0103759765625000, -0.0057983398437500, -0.0030212402343750,
+-0.0072631835937500, -0.0050048828125000, -0.0056762695312500, -0.0034790039062500,
+-0.0087890625000000, 0.0029296875000000, -0.0126342773437500, 0.0006408691406250,
+-0.0101928710937500, 0.0017700195312500, -0.0162353515625000, 0.0033569335937500,
+-0.0162963867187500, 0.0009155273437500, -0.0162658691406250, -0.0010070800781250,
+-0.0200195312500000, 0.0008239746093750, -0.0189514160156250, -0.0016784667968750,
+-0.0153808593750000, -0.0023803710937500, -0.0181274414062500, -0.0021362304687500,
+-0.0145874023437500, -0.0022277832031250, -0.0092163085937500, -0.0022583007812500,
+-0.0126342773437500, -0.0053100585937500, -0.0046386718750000, -0.0037841796875000,
+-0.0050964355468750, -0.0042724609375000, -0.0035400390625000, -0.0084838867187500,
+0.0018005371093750, -0.0078735351562500, -0.0007934570312500, -0.0032348632812500,
+0.0029907226562500, -0.0047607421875000, 0.0039672851562500, -0.0044555664062500,
+0.0028076171875000, -0.0002746582031250, 0.0035400390625000, -0.0016479492187500,
+0.0036315917968750, -0.0056457519531250, 0.0010375976562500, -0.0021057128906250,
+0.0016174316406250, -0.0063171386718750, 0.0009460449218750, -0.0140380859375000,
+-0.0014953613281250, -0.0096435546875000, -0.0003967285156250, -0.0133972167968750,
+0.0003662109375000, -0.0153808593750000, -0.0008544921875000, -0.0140686035156250,
+0.0018310546875000, -0.0139160156250000, 0.0048828125000000, -0.0144958496093750,
+0.0034179687500000, -0.0133972167968750, 0.0087890625000000, -0.0128784179687500,
+0.0094909667968750, -0.0146789550781250, 0.0093383789062500, -0.0155029296875000,
+0.0120849609375000, -0.0138854980468750, 0.0119628906250000, -0.0161437988281250,
+0.0094299316406250, -0.0170593261718750, 0.0104980468750000, -0.0156250000000000,
+0.0092163085937500, -0.0152587890625000, 0.0061645507812500, -0.0158996582031250,
+0.0076599121093750, -0.0131835937500000, 0.0072021484375000, -0.0138549804687500,
+0.0060729980468750, -0.0127563476562500, 0.0086975097656250, -0.0102539062500000,
+0.0107421875000000, -0.0116271972656250, 0.0092468261718750, -0.0100402832031250,
+0.0150451660156250, -0.0096130371093750, 0.0149230957031250, -0.0094909667968750,
+0.0169372558593750, -0.0090637207031250, 0.0216979980468750, -0.0093688964843750,
+0.0205383300781250, -0.0071411132812500, 0.0238037109375000, -0.0075073242187500,
+0.0247802734375000, -0.0069274902343750, 0.0244445800781250, -0.0051574707031250,
+0.0252685546875000, -0.0057983398437500, 0.0254821777343750, -0.0043640136718750,
+0.0252685546875000, -0.0045166015625000, 0.0252380371093750, -0.0039978027343750,
+0.0250549316406250, -0.0027770996093750, 0.0247497558593750, -0.0035095214843750,
+0.0245056152343750, -0.0025634765625000, 0.0232543945312500, -0.0019226074218750,
+0.0230102539062500, -0.0028076171875000, 0.0225219726562500, -0.0030517578125000,
+0.0215148925781250, -0.0023498535156250, 0.0212707519531250, -0.0042114257812500,
+0.0203857421875000, -0.0040283203125000, 0.0197143554687500, -0.0041809082031250,
+0.0193481445312500, -0.0052185058593750, 0.0188903808593750, -0.0047912597656250,
+0.0184936523437500, -0.0055847167968750, 0.0185241699218750, -0.0052185058593750,
+0.0176696777343750, -0.0061645507812500, 0.0177612304687500, -0.0072021484375000,
+0.0180358886718750, -0.0062561035156250, 0.0169372558593750, -0.0097351074218750,
+0.0184936523437500, -0.0093383789062500, 0.0180053710937500, -0.0098266601562500,
+0.0178527832031250, -0.0123901367187500, 0.0185546875000000, -0.0115356445312500,
+0.0180358886718750, -0.0111083984375000, 0.0195922851562500, -0.0123291015625000,
+0.0184631347656250, -0.0104980468750000, 0.0187377929687500, -0.0085449218750000,
+0.0199584960937500, -0.0100708007812500, 0.0185852050781250, -0.0059509277343750,
+0.0210266113281250, -0.0061645507812500, 0.0204162597656250, -0.0057067871093750,
+0.0211181640625000, -0.0030212402343750, 0.0227661132812500, -0.0040588378906250,
+0.0215148925781250, -0.0031738281250000, 0.0251159667968750, -0.0026550292968750,
+0.0245056152343750, -0.0030822753906250, 0.0245361328125000, -0.0032348632812500,
+0.0270080566406250, -0.0028991699218750, 0.0258789062500000, -0.0031127929687500,
+0.0250854492187500, -0.0031127929687500, 0.0269165039062500, -0.0032348632812500,
+0.0237121582031250, -0.0030822753906250, 0.0211791992187500, -0.0029907226562500,
+0.0242309570312500, -0.0053100585937500, 0.0143432617187500, -0.0042419433593750,
+0.0164184570312500, -0.0059509277343750, 0.0122680664062500, -0.0088806152343750,
+0.0039062500000000},
+{0.0053405761718750, -0.0053710937500000, 0.0094604492187500, -0.0026550292968750,
+0.0071716308593750, -0.0111694335937500, 0.0115051269531250, -0.0109863281250000,
+0.0125427246093750, -0.0102233886718750, 0.0111999511718750, -0.0141296386718750,
+0.0120544433593750, -0.0132751464843750, 0.0126037597656250, -0.0127868652343750,
+0.0096740722656250, -0.0136108398437500, 0.0099182128906250, -0.0124206542968750,
+0.0101013183593750, -0.0106811523437500, 0.0081481933593750, -0.0118103027343750,
+0.0085754394531250, -0.0121459960937500, 0.0115661621093750, -0.0108032226562500,
+0.0103759765625000, -0.0114135742187500, 0.0138244628906250, -0.0130310058593750,
+0.0184326171875000, -0.0123291015625000, 0.0162353515625000, -0.0119934082031250,
+0.0227355957031250, -0.0125732421875000, 0.0231933593750000, -0.0115356445312500,
+0.0226745605468750, -0.0107116699218750, 0.0256042480468750, -0.0114440917968750,
+0.0251464843750000, -0.0083007812500000, 0.0227966308593750, -0.0086364746093750,
+0.0242614746093750, -0.0087280273437500, 0.0213623046875000, -0.0068359375000000,
+0.0173034667968750, -0.0071105957031250, 0.0190429687500000, -0.0073852539062500,
+0.0122375488281250, -0.0073547363281250, 0.0121459960937500, -0.0064392089843750,
+0.0109252929687500, -0.0059814453125000, 0.0066223144531250, -0.0066528320312500,
+0.0077819824218750, -0.0043640136718750, 0.0055541992187500, -0.0043945312500000,
+0.0049133300781250, -0.0043945312500000, 0.0050354003906250, -0.0032653808593750,
+0.0042114257812500, -0.0036315917968750, 0.0041809082031250, -0.0035095214843750,
+0.0047912597656250, -0.0033874511718750, 0.0045166015625000, -0.0036621093750000,
+0.0051269531250000, -0.0037231445312500, 0.0058898925781250, -0.0036621093750000,
+0.0053405761718750, -0.0041503906250000, 0.0075073242187500, -0.0038452148437500,
+0.0069580078125000, -0.0043945312500000, 0.0072326660156250, -0.0050659179687500,
+0.0085754394531250, -0.0044555664062500, 0.0076599121093750, -0.0064392089843750,
+0.0086669921875000, -0.0060729980468750, 0.0085449218750000, -0.0066833496093750,
+0.0084533691406250, -0.0083007812500000, 0.0086059570312500, -0.0075073242187500,
+0.0080261230468750, -0.0088500976562500, 0.0086059570312500, -0.0090332031250000,
+0.0076599121093750, -0.0087890625000000, 0.0080566406250000, -0.0093078613281250,
+0.0086975097656250, -0.0092163085937500, 0.0070800781250000, -0.0083923339843750,
+0.0086669921875000, -0.0088500976562500, 0.0079956054687500, -0.0079345703125000,
+0.0076599121093750, -0.0068664550781250, 0.0085754394531250, -0.0077819824218750,
+0.0075988769531250, -0.0050659179687500, 0.0075378417968750, -0.0054931640625000,
+0.0077514648437500, -0.0049438476562500, 0.0069580078125000, -0.0030212402343750,
+0.0064392089843750, -0.0039672851562500, 0.0068054199218750, -0.0022583007812500,
+0.0053405761718750, -0.0023193359375000, 0.0049743652343750, -0.0024108886718750,
+0.0057067871093750, -0.0013122558593750, 0.0050048828125000, -0.0016784667968750,
+0.0043029785156250, -0.0027465820312500, 0.0081481933593750, -0.0018615722656250,
+0.0058898925781250, -0.0034484863281250, 0.0088500976562500, -0.0056152343750000,
+0.0134887695312500, -0.0043334960937500, 0.0101013183593750, -0.0070800781250000,
+0.0181884765625000, -0.0074768066406250, 0.0175781250000000, -0.0072326660156250,
+0.0186767578125000, -0.0079956054687500, 0.0239562988281250, -0.0077514648437500,
+0.0216064453125000, -0.0084228515625000, 0.0242919921875000, -0.0078125000000000,
+0.0257263183593750, -0.0086364746093750, 0.0225830078125000, -0.0097961425781250,
+0.0221252441406250, -0.0087890625000000, 0.0245971679687500, -0.0112915039062500,
+0.0151062011718750, -0.0111389160156250, 0.0177917480468750, -0.0111083984375000,
+0.0136718750000000, -0.0124511718750000, 0.0050354003906250, -0.0119323730468750,
+0.0103454589843750, -0.0119018554687500, 0.0004272460937500, -0.0118103027343750,
+0.0003356933593750, -0.0122070312500000, -0.0001525878906250, -0.0123901367187500,
+-0.0060424804687500, -0.0118713378906250, -0.0039672851562500, -0.0129699707031250,
+-0.0040283203125000, -0.0132446289062500, -0.0071105957031250, -0.0126342773437500,
+-0.0018615722656250, -0.0125427246093750, 0.0026855468750000, -0.0129699707031250,
+-0.0014648437500000, -0.0119323730468750, 0.0114135742187500, -0.0121459960937500,
+0.0103759765625000, -0.0116882324218750, 0.0119323730468750, -0.0112609863281250,
+0.0200500488281250, -0.0115661621093750, 0.0166931152343750, -0.0071716308593750,
+0.0208129882812500, -0.0088500976562500, 0.0209350585937500, -0.0068359375000000,
+0.0205383300781250, -0.0015563964843750, 0.0228576660156250, -0.0040283203125000,
+0.0219116210937500, -0.0040283203125000, 0.0202026367187500, -0.0018920898437500,
+0.0225219726562500, -0.0045166015625000, 0.0175476074218750, -0.0071411132812500,
+0.0123596191406250, -0.0054931640625000, 0.0164184570312500, -0.0125732421875000,
+0.0057983398437500, -0.0124816894531250, 0.0054016113281250, -0.0108032226562500,
+0.0043945312500000, -0.0142822265625000, -0.0017700195312500, -0.0150451660156250,
+0.0002746582031250, -0.0097961425781250, -0.0003051757812500, -0.0102844238281250,
+-0.0019836425781250, -0.0091552734375000, 0.0008850097656250, -0.0062561035156250,
+0.0025329589843750, -0.0078125000000000, 0.0003356933593750, -0.0019226074218750,
+0.0098571777343750, -0.0031738281250000, 0.0100402832031250, -0.0004577636718750,
+0.0085754394531250, 0.0047912597656250, 0.0135498046875000, 0.0018005371093750,
+0.0129394531250000, 0.0087585449218750, 0.0035400390625000, 0.0103149414062500,
+0.0090026855468750, 0.0058593750000000, 0.0010375976562500, 0.0071105957031250,
+-0.0126953125000000, 0.0097961425781250, -0.0049743652343750, -0.0050964355468750,
+-0.0180969238281250, -0.0032043457031250, -0.0201110839843750, -0.0047607421875000,
+-0.0163879394531250, -0.0200500488281250, -0.0203247070312500, -0.0210571289062500,
+0.0045471191406250, -0.0076293945312500, -0.0151062011718750, -0.0137939453125000,
+-0.0147399902343750, -0.0083312988281250, -0.0117797851562500, -0.0002136230468750,
+-0.0142517089843750, -0.0046081542968750, -0.0160827636718750, 0.0081787109375000,
+-0.0090332031250000, 0.0067138671875000, -0.0086364746093750, 0.0086669921875000,
+-0.0096130371093750, 0.0188293457031250, -0.0076904296875000, 0.0148315429687500,
+-0.0072937011718750, 0.0124206542968750, -0.0048828125000000, 0.0191955566406250,
+-0.0068054199218750, 0.0096740722656250, -0.0046386718750000, -0.0015258789062500,
+-0.0000305175781250, 0.0068359375000000, -0.0023193359375000, -0.0120544433593750,
+-0.0013732910156250, -0.0134887695312500, 0.0015563964843750, -0.0098571777343750,
+-0.0037841796875000, -0.0178833007812500, -0.0084533691406250, -0.0179138183593750,
+-0.0041198730468750, -0.0051269531250000, -0.0138244628906250, -0.0092468261718750,
+-0.0153198242187500, -0.0036010742187500, -0.0129089355468750, 0.0087280273437500,
+-0.0162658691406250, 0.0024414062500000, -0.0165710449218750, 0.0139160156250000,
+-0.0077209472656250, 0.0147094726562500, -0.0101318359375000, 0.0146179199218750,
+-0.0076904296875000, 0.0198669433593750, 0.0001525878906250, 0.0179138183593750,
+-0.0029907226562500, 0.0194091796875000, 0.0003356933593750, 0.0196838378906250,
+0.0012512207031250, 0.0200805664062500, 0.0003051757812500, 0.0214233398437500,
+0.0009460449218750, 0.0200500488281250, 0.0010681152343750, 0.0177307128906250,
+0.0000305175781250, 0.0209350585937500, -0.0003051757812500, 0.0162963867187500,
+0.0001220703125000, 0.0104370117187500, 0.0003967285156250, 0.0147094726562500,
+0.0000305175781250, 0.0042724609375000, -0.0010070800781250, 0.0053100585937500,
+-0.0005187988281250, 0.0035095214843750, -0.0007019042968750, -0.0039978027343750,
+-0.0017395019531250, -0.0005493164062500, -0.0016784667968750, -0.0032958984375000,
+-0.0022583007812500, -0.0065307617187500, -0.0011291503906250, -0.0016174316406250,
+-0.0033264160156250, 0.0010986328125000, -0.0057067871093750, -0.0027465820312500,
+-0.0036010742187500, 0.0099792480468750, -0.0081481933593750, 0.0074157714843750,
+-0.0087280273437500, 0.0103759765625000, -0.0079040527343750, 0.0200195312500000,
+-0.0092773437500000, 0.0151367187500000, -0.0093994140625000, 0.0235900878906250,
+-0.0082397460937500, 0.0238342285156250, -0.0082702636718750, 0.0235290527343750,
+-0.0085144042968750, 0.0278930664062500, -0.0080871582031250, 0.0263366699218750,
+-0.0080871582031250, 0.0254211425781250, -0.0088806152343750, 0.0269775390625000,
+-0.0086364746093750, 0.0255737304687500, -0.0091857910156250, 0.0233154296875000,
+-0.0098876953125000, 0.0247802734375000, -0.0093688964843750, 0.0229797363281250,
+-0.0112304687500000, 0.0227050781250000, -0.0108947753906250, 0.0233154296875000,
+-0.0109252929687500, 0.0228881835937500, -0.0122375488281250, 0.0227661132812500,
+-0.0117187500000000, 0.0231628417968750, -0.0110473632812500, 0.0228271484375000,
+-0.0113525390625000, 0.0230712890625000, -0.0111389160156250, 0.0237731933593750,
+-0.0101928710937500, 0.0232543945312500, -0.0103149414062500, 0.0229492187500000,
+-0.0117797851562500, 0.0238647460937500, -0.0111083984375000, 0.0224914550781250,
+-0.0121154785156250, 0.0213012695312500, -0.0140991210937500, 0.0224304199218750,
+-0.0133056640625000, 0.0176086425781250, -0.0150146484375000, 0.0195007324218750,
+-0.0151367187500000, 0.0173950195312500, -0.0155639648437500, 0.0129699707031250,
+-0.0166320800781250, 0.0161743164062500, -0.0161437988281250, 0.0098266601562500,
+-0.0173339843750000, 0.0112915039062500, -0.0176086425781250, 0.0101623535156250,
+-0.0171203613281250, 0.0058593750000000, -0.0173339843750000, 0.0091247558593750,
+-0.0175781250000000, 0.0043640136718750, -0.0156555175781250, 0.0054321289062500,
+-0.0158691406250000, 0.0049438476562500, -0.0153503417968750, 0.0021667480468750,
+-0.0138854980468750, 0.0047912597656250, -0.0142822265625000, 0.0011901855468750,
+-0.0134582519531250, 0.0021057128906250, -0.0136108398437500, 0.0023498535156250,
+-0.0126647949218750, 0.0005493164062500, -0.0117797851562500, 0.0025939941406250,
+-0.0126342773437500, 0.0016174316406250, -0.0091552734375000, 0.0021362304687500,
+-0.0098571777343750, 0.0028381347656250, -0.0088195800781250, 0.0027770996093750,
+-0.0062255859375000, 0.0037231445312500, -0.0076293945312500, 0.0038452148437500,
+-0.0041503906250000, 0.0040588378906250, -0.0045471191406250, 0.0044555664062500,
+-0.0039062500000000, 0.0048522949218750, -0.0010375976562500, 0.0051574707031250,
+-0.0020446777343750, 0.0049133300781250, -0.0020751953125000, 0.0053405761718750,
+-0.0008850097656250, 0.0049438476562500, -0.0015869140625000, 0.0044555664062500,
+-0.0030517578125000, 0.0052795410156250, -0.0022888183593750, 0.0042724609375000,
+-0.0020751953125000, 0.0044250488281250, -0.0025634765625000, 0.0047607421875000,
+-0.0025024414062500, 0.0044250488281250, -0.0019836425781250, 0.0046386718750000,
+-0.0020141601562500, 0.0052795410156250, -0.0020141601562500, 0.0051879882812500,
+-0.0025329589843750, 0.0068054199218750, -0.0019531250000000, 0.0083312988281250,
+-0.0008544921875000, 0.0081787109375000, -0.0016174316406250, 0.0127563476562500,
+-0.0031433105468750, 0.0120544433593750, -0.0016174316406250, 0.0147399902343750,
+-0.0030822753906250, 0.0192260742187500, -0.0060119628906250, 0.0173034667968750,
+-0.0046386718750000, 0.0229187011718750, -0.0060424804687500, 0.0235900878906250,
+-0.0061645507812500, 0.0236206054687500, -0.0077209472656250, 0.0266113281250000,
+-0.0090637207031250, 0.0257568359375000, -0.0075073242187500, 0.0223693847656250,
+-0.0115966796875000},
+{-0.0038146972656250, 0.0262145996093750, -0.0069885253906250, 0.0264892578125000,
+-0.0074157714843750, 0.0258483886718750, -0.0071411132812500, 0.0257873535156250,
+-0.0082702636718750, 0.0259399414062500, -0.0080871582031250, 0.0246582031250000,
+-0.0078125000000000, 0.0255737304687500, -0.0077819824218750, 0.0237426757812500,
+-0.0077209472656250, 0.0223999023437500, -0.0075378417968750, 0.0246582031250000,
+-0.0074462890625000, 0.0169067382812500, -0.0073852539062500, 0.0200195312500000,
+-0.0072021484375000, 0.0160522460937500, -0.0074462890625000, 0.0086975097656250,
+-0.0075683593750000, 0.0140686035156250, -0.0072631835937500, 0.0009460449218750,
+-0.0087890625000000, 0.0038757324218750, -0.0085449218750000, -0.0004272460937500,
+-0.0088806152343750, -0.0103454589843750, -0.0101318359375000, -0.0032348632812500,
+-0.0098266601562500, -0.0177612304687500, -0.0106201171875000, -0.0151977539062500,
+-0.0104980468750000, -0.0173950195312500, -0.0114440917968750, -0.0269165039062500,
+-0.0123901367187500, -0.0208129882812500, -0.0116577148437500, -0.0301818847656250,
+-0.0144348144531250, -0.0292358398437500, -0.0144042968750000, -0.0293884277343750,
+-0.0145263671875000, -0.0348815917968750, -0.0161132812500000, -0.0322265625000000,
+-0.0156555175781250, -0.0355529785156250, -0.0157165527343750, -0.0350646972656250,
+-0.0158081054687500, -0.0346984863281250, -0.0158386230468750, -0.0363464355468750,
+-0.0157470703125000, -0.0351562500000000, -0.0156250000000000, -0.0356445312500000,
+-0.0162658691406250, -0.0356445312500000, -0.0162658691406250, -0.0351562500000000,
+-0.0158996582031250, -0.0351257324218750, -0.0162963867187500, -0.0350646972656250,
+-0.0163879394531250, -0.0344238281250000, -0.0142211914062500, -0.0346374511718750,
+-0.0151672363281250, -0.0341796875000000, -0.0136108398437500, -0.0335693359375000,
+-0.0111694335937500, -0.0340576171875000, -0.0127258300781250, -0.0331420898437500,
+-0.0086975097656250, -0.0333557128906250, -0.0093078613281250, -0.0332031250000000,
+-0.0079345703125000, -0.0325012207031250, -0.0045166015625000, -0.0325622558593750,
+-0.0061645507812500, -0.0310974121093750, -0.0035400390625000, -0.0313110351562500,
+-0.0024108886718750, -0.0301208496093750, -0.0034179687500000, -0.0284423828125000,
+-0.0034790039062500, -0.0288391113281250, -0.0026245117187500, -0.0254516601562500,
+-0.0032958984375000, -0.0259094238281250, -0.0040283203125000, -0.0244750976562500,
+-0.0028686523437500, -0.0213623046875000, -0.0018005371093750, -0.0220947265625000,
+-0.0026855468750000, -0.0181884765625000, -0.0018310546875000, -0.0179443359375000,
+-0.0014038085937500, -0.0165710449218750, -0.0017089843750000, -0.0135803222656250,
+-0.0019226074218750, -0.0139465332031250, -0.0019531250000000, -0.0108337402343750,
+-0.0016479492187500, -0.0106201171875000, -0.0014648437500000, -0.0099487304687500,
+-0.0024108886718750, -0.0079040527343750, -0.0028076171875000, -0.0081176757812500,
+-0.0018310546875000, -0.0065002441406250, -0.0039062500000000, -0.0065612792968750,
+-0.0040893554687500, -0.0056762695312500, -0.0041809082031250, -0.0047607421875000,
+-0.0046386718750000, -0.0054626464843750, -0.0044860839843750, -0.0015563964843750,
+-0.0079956054687500, -0.0029296875000000, -0.0066223144531250, -0.0000610351562500,
+-0.0077514648437500, 0.0039978027343750, -0.0118103027343750, 0.0014343261718750,
+-0.0102233886718750, 0.0109252929687500, -0.0089416503906250, 0.0095520019531250,
+-0.0106201171875000, 0.0124511718750000, -0.0096435546875000, 0.0201721191406250,
+-0.0066223144531250, 0.0164184570312500, -0.0075073242187500, 0.0216064453125000,
+-0.0088806152343750, 0.0231018066406250, -0.0075683593750000, 0.0205688476562500,
+-0.0092163085937500, 0.0210876464843750, -0.0119323730468750, 0.0216064453125000,
+-0.0103149414062500, 0.0144042968750000, -0.0120544433593750, 0.0158081054687500,
+-0.0140380859375000, 0.0124511718750000, -0.0112915039062500, 0.0054321289062500,
+-0.0090942382812500, 0.0087890625000000, -0.0113220214843750, 0.0043945312500000,
+-0.0069885253906250, 0.0021972656250000, -0.0067138671875000, 0.0049133300781250,
+-0.0068969726562500, 0.0056457519531250, -0.0051574707031250, 0.0039672851562500,
+-0.0054321289062500, 0.0090637207031250, -0.0037841796875000, 0.0093383789062500,
+-0.0045776367187500, 0.0085754394531250, -0.0049743652343750, 0.0105285644531250,
+-0.0032043457031250, 0.0104064941406250, -0.0029296875000000, 0.0090942382812500,
+-0.0051269531250000, 0.0098266601562500, -0.0048217773437500, 0.0088806152343750,
+-0.0060424804687500, 0.0074462890625000, -0.0080261230468750, 0.0085449218750000,
+-0.0061340332031250, 0.0056152343750000, -0.0064697265625000, 0.0060424804687500,
+-0.0090026855468750, 0.0057983398437500, -0.0060424804687500, 0.0039062500000000,
+-0.0025634765625000, 0.0046997070312500, -0.0050354003906250, 0.0034790039062500,
+0.0000610351562500, 0.0035400390625000, 0.0005493164062500, 0.0031738281250000,
+-0.0003662109375000, 0.0025939941406250, 0.0016784667968750, 0.0029907226562500,
+0.0013427734375000, -0.0001525878906250, -0.0018920898437500, 0.0015563964843750,
+0.0002441406250000, -0.0016784667968750, -0.0034484863281250, -0.0069274902343750,
+-0.0098571777343750, -0.0060424804687500, -0.0093688964843750, -0.0162963867187500,
+-0.0191955566406250, -0.0135803222656250, -0.0167541503906250, -0.0122680664062500,
+-0.0158691406250000, -0.0195312500000000, -0.0229187011718750, -0.0178527832031250,
+-0.0215454101562500, -0.0108337402343750, -0.0158386230468750, -0.0148925781250000,
+-0.0192565917968750, -0.0115356445312500, -0.0141296386718750, -0.0028076171875000,
+-0.0057067871093750, -0.0072326660156250, -0.0105285644531250, -0.0044250488281250,
+-0.0024414062500000, -0.0029907226562500, -0.0006713867187500, -0.0035400390625000,
+-0.0014343261718750, -0.0036926269531250, 0.0009765625000000, -0.0041809082031250,
+0.0004882812500000, -0.0018920898437500, 0.0000305175781250, -0.0023803710937500,
+0.0004577636718750, -0.0022888183593750, 0.0010070800781250, -0.0004577636718750,
+0.0008239746093750, -0.0008239746093750, 0.0007019042968750, -0.0014343261718750,
+0.0027770996093750, -0.0014343261718750, 0.0023498535156250, -0.0010986328125000,
+0.0036621093750000, -0.0012512207031250, 0.0057678222656250, -0.0016479492187500,
+0.0046997070312500, -0.0005493164062500, 0.0073242187500000, 0.0002441406250000,
+0.0079040527343750, -0.0023803710937500, 0.0075073242187500, -0.0033874511718750,
+0.0086975097656250, -0.0014343261718750, 0.0086975097656250, -0.0090942382812500,
+0.0048522949218750, -0.0083618164062500, 0.0056457519531250, -0.0088500976562500,
+0.0060119628906250, -0.0144958496093750, 0.0031127929687500, -0.0128479003906250,
+0.0034179687500000, -0.0088195800781250, 0.0074462890625000, -0.0119323730468750,
+0.0054626464843750, -0.0088806152343750, 0.0089721679687500, -0.0028076171875000,
+0.0144042968750000, -0.0060119628906250, 0.0112609863281250, -0.0011596679687500,
+0.0184020996093750, 0.0003356933593750, 0.0191650390625000, -0.0023803710937500,
+0.0184631347656250, -0.0021667480468750, 0.0216979980468750, -0.0000915527343750,
+0.0210266113281250, -0.0044250488281250, 0.0174255371093750, -0.0047912597656250,
+0.0187683105468750, -0.0039978027343750, 0.0156555175781250, -0.0048522949218750,
+0.0107421875000000, -0.0045166015625000, 0.0126342773437500, -0.0054931640625000,
+0.0070495605468750, -0.0049438476562500, 0.0058593750000000, -0.0051879882812500,
+0.0053710937500000, -0.0059814453125000, 0.0028381347656250, -0.0054016113281250,
+0.0031127929687500, -0.0072937011718750, 0.0024108886718750, -0.0068664550781250,
+0.0019226074218750, -0.0066528320312500, 0.0018005371093750, -0.0078430175781250,
+0.0016174316406250, -0.0076904296875000, 0.0011596679687500, -0.0073852539062500,
+-0.0003356933593750, -0.0071105957031250, -0.0000610351562500, -0.0076904296875000,
+-0.0018615722656250, -0.0082092285156250, -0.0039978027343750, -0.0075683593750000,
+-0.0037231445312500, -0.0077209472656250, -0.0088195800781250, -0.0083618164062500,
+-0.0085754394531250, -0.0079040527343750, -0.0107116699218750, -0.0071411132812500,
+-0.0150451660156250, -0.0075988769531250, -0.0138244628906250, -0.0081481933593750,
+-0.0181579589843750, -0.0076904296875000, -0.0185546875000000, -0.0079650878906250,
+-0.0192871093750000, -0.0087890625000000, -0.0217895507812500, -0.0084533691406250,
+-0.0211181640625000, -0.0082397460937500, -0.0229187011718750, -0.0085754394531250,
+-0.0230407714843750, -0.0083007812500000, -0.0229187011718750, -0.0078735351562500,
+-0.0234680175781250, -0.0083007812500000, -0.0228881835937500, -0.0080566406250000,
+-0.0231933593750000, -0.0080566406250000, -0.0228576660156250, -0.0080871582031250,
+-0.0223999023437500, -0.0080871582031250, -0.0223083496093750, -0.0081787109375000,
+-0.0220336914062500, -0.0075378417968750, -0.0213317871093750, -0.0078735351562500,
+-0.0202026367187500, -0.0074462890625000, -0.0208129882812500, -0.0066528320312500,
+-0.0208435058593750, -0.0070800781250000, -0.0191040039062500, -0.0060424804687500,
+-0.0217590332031250, -0.0058593750000000, -0.0203552246093750, -0.0061035156250000,
+-0.0211791992187500, -0.0057678222656250, -0.0233459472656250, -0.0056457519531250,
+-0.0212707519531250, -0.0061340332031250, -0.0256652832031250, -0.0060119628906250,
+-0.0238952636718750, -0.0060424804687500, -0.0249938964843750, -0.0064697265625000,
+-0.0285339355468750, -0.0063171386718750, -0.0254516601562500, -0.0063781738281250,
+-0.0297851562500000, -0.0064086914062500, -0.0293579101562500, -0.0066833496093750,
+-0.0295715332031250, -0.0068969726562500, -0.0323181152343750, -0.0067138671875000,
+-0.0311584472656250, -0.0075378417968750, -0.0322875976562500, -0.0079040527343750,
+-0.0331115722656250, -0.0075073242187500, -0.0316772460937500, -0.0077514648437500,
+-0.0315856933593750, -0.0082397460937500, -0.0323181152343750, -0.0062866210937500,
+-0.0263977050781250, -0.0066833496093750, -0.0289611816406250, -0.0063781738281250,
+-0.0252380371093750, -0.0048522949218750, -0.0190124511718750, -0.0055847167968750,
+-0.0234985351562500, -0.0052490234375000, -0.0129699707031250, -0.0050354003906250,
+-0.0137329101562500, -0.0052490234375000, -0.0114746093750000, -0.0053405761718750,
+-0.0045776367187500, -0.0052490234375000, -0.0082397460937500, -0.0058288574218750,
+-0.0012207031250000, -0.0054931640625000, -0.0015258789062500, -0.0058593750000000,
+-0.0012207031250000, -0.0065612792968750, 0.0025939941406250, -0.0060119628906250,
+0.0006103515625000, -0.0067749023437500, 0.0028076171875000, -0.0067443847656250,
+0.0025024414062500, -0.0067749023437500, 0.0024108886718750, -0.0071411132812500,
+0.0033264160156250, -0.0068664550781250, 0.0025024414062500, -0.0072326660156250,
+0.0050659179687500, -0.0073242187500000, 0.0039062500000000, -0.0069885253906250,
+0.0056457519531250, -0.0069274902343750, 0.0082397460937500, -0.0070800781250000,
+0.0066223144531250, -0.0061035156250000, 0.0141296386718750, -0.0064697265625000,
+0.0127868652343750, -0.0053710937500000, 0.0151977539062500, -0.0041809082031250,
+0.0221557617187500, -0.0049438476562500, 0.0187377929687500, -0.0010070800781250,
+0.0190734863281250, -0.0017089843750000, 0.0229492187500000, -0.0010986328125000,
+0.0173034667968750, 0.0025024414062500, 0.0119934082031250, 0.0011291503906250,
+0.0163574218750000, -0.0010070800781250, 0.0035705566406250, 0.0006408691406250,
+0.0040893554687500, -0.0007019042968750, 0.0025024414062500, -0.0037536621093750,
+-0.0055847167968750, -0.0029602050781250, -0.0023803710937500, -0.0064392089843750,
+-0.0052490234375000, -0.0052185058593750, -0.0078125000000000, -0.0072631835937500,
+-0.0040588378906250},
+{-0.0024719238281250, -0.0093688964843750, -0.0001525878906250, -0.0089111328125000,
+0.0043640136718750, -0.0088500976562500, 0.0041198730468750, -0.0103149414062500,
+0.0081787109375000, -0.0101623535156250, 0.0099182128906250, -0.0101623535156250,
+0.0110473632812500, -0.0111389160156250, 0.0134277343750000, -0.0110168457031250,
+0.0141296386718750, -0.0101623535156250, 0.0147705078125000, -0.0107727050781250,
+0.0166320800781250, -0.0099792480468750, 0.0174255371093750, -0.0087585449218750,
+0.0177612304687500, -0.0095825195312500, 0.0193786621093750, -0.0079956054687500,
+0.0190429687500000, -0.0079956054687500, 0.0207214355468750, -0.0078735351562500,
+0.0207519531250000, -0.0069885253906250, 0.0202941894531250, -0.0073242187500000,
+0.0224914550781250, -0.0065307617187500, 0.0198974609375000, -0.0065002441406250,
+0.0217895507812500, -0.0064086914062500, 0.0212402343750000, -0.0059204101562500,
+0.0192565917968750, -0.0060424804687500, 0.0222473144531250, -0.0056457519531250,
+0.0179748535156250, -0.0054321289062500, 0.0204162597656250, -0.0057678222656250,
+0.0189514160156250, -0.0057373046875000, 0.0150451660156250, -0.0054931640625000,
+0.0188903808593750, -0.0072326660156250, 0.0123596191406250, -0.0067443847656250,
+0.0136718750000000, -0.0074157714843750, 0.0118713378906250, -0.0092163085937500,
+0.0073547363281250, -0.0084228515625000, 0.0105285644531250, -0.0089111328125000,
+0.0015258789062500, -0.0096435546875000, 0.0043640136718750, -0.0088500976562500,
+0.0004577636718750, -0.0081787109375000, -0.0072631835937500, -0.0088500976562500,
+-0.0016479492187500, -0.0075683593750000, -0.0155334472656250, -0.0076599121093750,
+-0.0129699707031250, -0.0075988769531250, -0.0159912109375000, -0.0069274902343750,
+-0.0264587402343750, -0.0074462890625000, -0.0209350585937500, -0.0067138671875000,
+-0.0293884277343750, -0.0066528320312500, -0.0304260253906250, -0.0068969726562500,
+-0.0293273925781250, -0.0066223144531250, -0.0328979492187500, -0.0066528320312500,
+-0.0321044921875000, -0.0066528320312500, -0.0291748046875000, -0.0067443847656250,
+-0.0317077636718750, -0.0063476562500000, -0.0270080566406250, -0.0060729980468750,
+-0.0220336914062500, -0.0062866210937500, -0.0263977050781250, -0.0052490234375000,
+-0.0133361816406250, -0.0054626464843750, -0.0158996582031250, -0.0048217773437500,
+-0.0119934082031250, -0.0039062500000000, -0.0014038085937500, -0.0046386718750000,
+-0.0070800781250000, -0.0022888183593750, 0.0017089843750000, -0.0025939941406250,
+0.0027465820312500, -0.0020141601562500, 0.0014343261718750, -0.0003051757812500,
+0.0049133300781250, -0.0010375976562500, 0.0041809082031250, 0.0006103515625000,
+0.0005493164062500, 0.0003662109375000, 0.0014648437500000, 0.0014343261718750,
+0.0014038085937500, 0.0030822753906250, -0.0019531250000000, 0.0019226074218750,
+-0.0010070800781250, 0.0041809082031250, 0.0047912597656250, 0.0050964355468750,
+0.0015563964843750, 0.0031127929687500, 0.0058593750000000, 0.0025939941406250,
+0.0142822265625000, 0.0040283203125000, 0.0100402832031250, -0.0001525878906250,
+0.0151367187500000, -0.0000915527343750, 0.0183715820312500, -0.0016174316406250,
+0.0142517089843750, -0.0041198730468750, 0.0127868652343750, -0.0021972656250000,
+0.0155029296875000, -0.0078735351562500, 0.0056457519531250, -0.0088195800781250,
+0.0070495605468750, -0.0068359375000000, 0.0042114257812500, -0.0076599121093750,
+-0.0037536621093750, -0.0088806152343750, 0.0000000000000000, -0.0071716308593750,
+-0.0047302246093750, -0.0056152343750000, -0.0075378417968750, -0.0083312988281250,
+-0.0047607421875000, -0.0095214843750000, -0.0038452148437500, -0.0073852539062500,
+-0.0055847167968750, -0.0161437988281250, 0.0002441406250000, -0.0151062011718750,
+-0.0006408691406250, -0.0157470703125000, 0.0008544921875000, -0.0216674804687500,
+0.0054016113281250, -0.0197143554687500, 0.0037231445312500, -0.0184631347656250,
+0.0074462890625000, -0.0206604003906250, 0.0069885253906250, -0.0178833007812500,
+0.0085144042968750, -0.0146484375000000, 0.0115661621093750, -0.0167236328125000,
+0.0098876953125000, -0.0088806152343750, 0.0148315429687500, -0.0108947753906250,
+0.0158691406250000, -0.0068969726562500, 0.0119323730468750, 0.0003051757812500,
+0.0122070312500000, -0.0038146972656250, 0.0145874023437500, 0.0087585449218750,
+0.0037841796875000, 0.0093078613281250, 0.0050659179687500, 0.0070190429687500,
+0.0044860839843750, 0.0134582519531250, -0.0061950683593750, -0.0085754394531250,
+0.0184631347656250, -0.0101928710937500, 0.0036010742187500, -0.0156555175781250,
+0.0093688964843750, -0.0060424804687500, 0.0054321289062500, 0.0011291503906250,
+-0.0041198730468750, -0.0065307617187500, -0.0010070800781250, 0.0127868652343750,
+-0.0000915527343750, 0.0133056640625000, -0.0036621093750000, 0.0125122070312500,
+0.0007019042968750, 0.0223693847656250, 0.0072631835937500, 0.0201721191406250,
+0.0036926269531250, 0.0154724121093750, 0.0056457519531250, 0.0187377929687500,
+0.0092163085937500, 0.0150756835937500, 0.0035095214843750, 0.0083007812500000,
+-0.0007629394531250, 0.0117797851562500, 0.0037231445312500, 0.0058593750000000,
+-0.0079650878906250, 0.0046081542968750, -0.0082702636718750, 0.0052185058593750,
+-0.0082092285156250, 0.0039367675781250, -0.0133361816406250, 0.0039367675781250,
+-0.0114135742187500, 0.0007019042968750, -0.0146179199218750, 0.0020141601562500,
+-0.0148620605468750, 0.0009765625000000, -0.0148315429687500, -0.0025939941406250,
+-0.0165710449218750, -0.0012817382812500, -0.0159301757812500, -0.0020751953125000,
+-0.0152587890625000, -0.0030517578125000, -0.0165100097656250, -0.0014953613281250,
+-0.0141906738281250, -0.0011596679687500, -0.0114746093750000, -0.0022583007812500,
+-0.0133666992187500, 0.0039978027343750, -0.0097045898437500, 0.0024719238281250,
+-0.0097351074218750, 0.0034790039062500, -0.0083618164062500, 0.0084838867187500,
+-0.0061340332031250, 0.0067138671875000, -0.0077514648437500, 0.0083618164062500,
+-0.0022888183593750, 0.0080566406250000, -0.0032958984375000, 0.0104370117187500,
+-0.0014648437500000, 0.0127868652343750, 0.0035095214843750, 0.0101928710937500,
+0.0010375976562500, 0.0160522460937500, 0.0023498535156250, 0.0169067382812500,
+0.0044555664062500, 0.0151367187500000, 0.0018615722656250, 0.0173339843750000,
+-0.0010986328125000, 0.0179748535156250, 0.0006103515625000, 0.0111999511718750,
+-0.0007629394531250, 0.0146484375000000, -0.0020141601562500, 0.0088195800781250,
+-0.0009155273437500, 0.0011596679687500, 0.0001525878906250, 0.0076599121093750,
+-0.0004882812500000, -0.0094604492187500, -0.0000915527343750, -0.0065002441406250,
+0.0003967285156250, -0.0098876953125000, -0.0002441406250000, -0.0221862792968750,
+-0.0007934570312500, -0.0156555175781250, -0.0003356933593750, -0.0265808105468750,
+-0.0013122558593750, -0.0269775390625000, -0.0012512207031250, -0.0260009765625000,
+-0.0013122558593750, -0.0316162109375000, -0.0016479492187500, -0.0296936035156250,
+-0.0012817382812500, -0.0254821777343750, -0.0026550292968750, -0.0299987792968750,
+-0.0022583007812500, -0.0232238769531250, -0.0026550292968750, -0.0152893066406250,
+-0.0039978027343750, -0.0223083496093750, -0.0033569335937500, -0.0053710937500000,
+-0.0039367675781250, -0.0086059570312500, -0.0041503906250000, -0.0055541992187500,
+-0.0043029785156250, 0.0062561035156250, -0.0044860839843750, -0.0000915527343750,
+-0.0043640136718750, 0.0116882324218750, -0.0054931640625000, 0.0107727050781250,
+-0.0053100585937500, 0.0120849609375000, -0.0057983398437500, 0.0191955566406250,
+-0.0066833496093750, 0.0152893066406250, -0.0062866210937500, 0.0215148925781250,
+-0.0079650878906250, 0.0210266113281250, -0.0076293945312500, 0.0211486816406250,
+-0.0084228515625000, 0.0246582031250000, -0.0100708007812500, 0.0228271484375000,
+-0.0091552734375000, 0.0250244140625000, -0.0104370117187500, 0.0252075195312500,
+-0.0108947753906250, 0.0256042480468750, -0.0104675292968750, 0.0269470214843750,
+-0.0104980468750000, 0.0265808105468750, -0.0105895996093750, 0.0288085937500000,
+-0.0105590820312500, 0.0279541015625000, -0.0103759765625000, 0.0289611816406250,
+-0.0104980468750000, 0.0312500000000000, -0.0108947753906250, 0.0300903320312500,
+-0.0108642578125000, 0.0327453613281250, -0.0102539062500000, 0.0325622558593750,
+-0.0107421875000000, 0.0329895019531250, -0.0102539062500000, 0.0346679687500000,
+-0.0093688964843750, 0.0339660644531250, -0.0099487304687500, 0.0359191894531250,
+-0.0089721679687500, 0.0357666015625000, -0.0086975097656250, 0.0363159179687500,
+-0.0089111328125000, 0.0375671386718750, -0.0085449218750000, 0.0368957519531250,
+-0.0083618164062500, 0.0388793945312500, -0.0087890625000000, 0.0391235351562500,
+-0.0087890625000000, 0.0397033691406250, -0.0082397460937500, 0.0409851074218750,
+-0.0083312988281250, 0.0406799316406250, -0.0085449218750000, 0.0418090820312500,
+-0.0068054199218750, 0.0419006347656250, -0.0071411132812500, 0.0419311523437500,
+-0.0070190429687500, 0.0426025390625000, -0.0057678222656250, 0.0423278808593750,
+-0.0061340332031250, 0.0413818359375000, -0.0062255859375000, 0.0422363281250000,
+-0.0061645507812500, 0.0411071777343750, -0.0060424804687500, 0.0396118164062500,
+-0.0061035156250000, 0.0401306152343750, -0.0063171386718750, 0.0354919433593750,
+-0.0053100585937500, 0.0365295410156250, -0.0054321289062500, 0.0323181152343750,
+-0.0052795410156250, 0.0268859863281250, -0.0044555664062500, 0.0295104980468750,
+-0.0046691894531250, 0.0181274414062500, -0.0046997070312500, 0.0191955566406250,
+-0.0041809082031250, 0.0142822265625000, -0.0050354003906250, 0.0046081542968750,
+-0.0057067871093750, 0.0079040527343750, -0.0049133300781250, -0.0055847167968750,
+-0.0080261230468750, -0.0049438476562500, -0.0076599121093750, -0.0081176757812500,
+-0.0075378417968750, -0.0174865722656250, -0.0097656250000000, -0.0136413574218750,
+-0.0090942382812500, -0.0200805664062500, -0.0066528320312500, -0.0216064453125000,
+-0.0078125000000000, -0.0203857421875000, -0.0065307617187500, -0.0221862792968750,
+-0.0033264160156250, -0.0219421386718750, -0.0046386718750000, -0.0179138183593750,
+-0.0039062500000000, -0.0185241699218750, -0.0026550292968750, -0.0174560546875000,
+-0.0042419433593750, -0.0141906738281250, -0.0053710937500000, -0.0148620605468750,
+-0.0041809082031250, -0.0131225585937500, -0.0083007812500000, -0.0130310058593750,
+-0.0085754394531250, -0.0118408203125000, -0.0072631835937500, -0.0108947753906250,
+-0.0087585449218750, -0.0115966796875000, -0.0094604492187500, -0.0051269531250000,
+-0.0059509277343750, -0.0069274902343750, -0.0067138671875000, -0.0025024414062500,
+-0.0055847167968750, 0.0045166015625000, -0.0024414062500000, 0.0003967285156250,
+-0.0040893554687500, 0.0109863281250000, -0.0034790039062500, 0.0113220214843750,
+-0.0026855468750000, 0.0114746093750000, -0.0037841796875000, 0.0171813964843750,
+-0.0044555664062500, 0.0155944824218750, -0.0037841796875000, 0.0144958496093750,
+-0.0066528320312500, 0.0164489746093750, -0.0070190429687500, 0.0138854980468750,
+-0.0062561035156250, 0.0105895996093750, -0.0067138671875000, 0.0124511718750000,
+-0.0068054199218750, 0.0066223144531250, -0.0070190429687500, 0.0068054199218750,
+-0.0076293945312500, 0.0059204101562500, -0.0064086914062500, 0.0017700195312500,
+-0.0047912597656250, 0.0031433105468750, -0.0058593750000000, 0.0029907226562500,
+-0.0069885253906250, 0.0019226074218750, -0.0051574707031250, 0.0026245117187500,
+-0.0082702636718750, 0.0039367675781250, -0.0112915039062500, 0.0034179687500000,
+-0.0082702636718750},
+{-0.0065307617187500, 0.0390625000000000, -0.0070495605468750, 0.0423889160156250,
+-0.0066223144531250, 0.0419006347656250, -0.0063171386718750, 0.0423583984375000,
+-0.0067138671875000, 0.0445556640625000, -0.0068969726562500, 0.0430297851562500,
+-0.0065917968750000, 0.0448608398437500, -0.0073547363281250, 0.0448303222656250,
+-0.0071105957031250, 0.0441894531250000, -0.0071411132812500, 0.0448913574218750,
+-0.0077209472656250, 0.0448913574218750, -0.0074157714843750, 0.0438232421875000,
+-0.0073547363281250, 0.0439758300781250, -0.0075988769531250, 0.0431823730468750,
+-0.0074157714843750, 0.0419921875000000, -0.0070800781250000, 0.0424194335937500,
+-0.0072326660156250, 0.0406494140625000, -0.0077209472656250, 0.0408325195312500,
+-0.0074462890625000, 0.0396728515625000, -0.0078430175781250, 0.0378112792968750,
+-0.0085144042968750, 0.0389404296875000, -0.0081787109375000, 0.0365905761718750,
+-0.0085144042968750, 0.0367431640625000, -0.0087890625000000, 0.0367431640625000,
+-0.0083312988281250, 0.0354614257812500, -0.0081176757812500, 0.0363464355468750,
+-0.0084533691406250, 0.0365905761718750, -0.0073547363281250, 0.0361328125000000,
+-0.0072326660156250, 0.0375671386718750, -0.0074768066406250, 0.0390014648437500,
+-0.0069580078125000, 0.0379943847656250, -0.0067749023437500, 0.0400695800781250,
+-0.0081481933593750, 0.0407409667968750, -0.0076904296875000, 0.0403442382812500,
+-0.0083312988281250, 0.0411376953125000, -0.0097961425781250, 0.0414733886718750,
+-0.0088806152343750, 0.0385742187500000, -0.0092773437500000, 0.0400390625000000,
+-0.0101928710937500, 0.0361022949218750, -0.0082092285156250, 0.0318908691406250,
+-0.0067443847656250, 0.0345458984375000, -0.0081176757812500, 0.0221252441406250,
+-0.0037536621093750, 0.0243530273437500, -0.0034790039062500, 0.0184936523437500,
+-0.0038757324218750, 0.0075683593750000, -0.0017395019531250, 0.0134277343750000,
+-0.0019836425781250, -0.0014343261718750, -0.0040893554687500, -0.0014953613281250,
+-0.0029296875000000, -0.0029907226562500, -0.0045471191406250, -0.0112304687500000,
+-0.0077514648437500, -0.0076599121093750, -0.0058593750000000, -0.0127563476562500,
+-0.0061645507812500, -0.0130920410156250, -0.0080261230468750, -0.0126953125000000,
+-0.0058898925781250, -0.0148315429687500, -0.0033874511718750, -0.0137634277343750,
+-0.0050659179687500, -0.0142211914062500, -0.0015563964843750, -0.0148010253906250,
+-0.0009460449218750, -0.0128479003906250, -0.0022583007812500, -0.0117187500000000,
+-0.0017395019531250, -0.0125427246093750, -0.0011901855468750, -0.0075988769531250,
+-0.0024414062500000, -0.0083923339843750, -0.0025634765625000, -0.0057373046875000,
+-0.0034179687500000, -0.0012817382812500, -0.0044250488281250, -0.0033264160156250,
+-0.0036010742187500, 0.0044250488281250, -0.0046386718750000, 0.0039367675781250,
+-0.0042724609375000, 0.0056152343750000, -0.0062255859375000, 0.0113830566406250,
+-0.0082092285156250, 0.0093688964843750, -0.0062561035156250, 0.0117492675781250,
+-0.0094299316406250, 0.0132751464843750, -0.0093383789062500, 0.0107727050781250,
+-0.0110778808593750, 0.0096740722656250, -0.0135498046875000, 0.0110473632812500,
+-0.0115356445312500, 0.0051269531250000, -0.0162048339843750, 0.0053710937500000,
+-0.0159606933593750, 0.0051879882812500, -0.0156250000000000, 0.0017700195312500,
+-0.0185852050781250, 0.0024108886718750, -0.0177612304687500, 0.0018615722656250,
+-0.0137329101562500, 0.0017395019531250, -0.0171203613281250, 0.0016174316406250,
+-0.0114746093750000, 0.0010375976562500, -0.0039367675781250, 0.0011596679687500,
+-0.0093688964843750, 0.0021362304687500, 0.0006408691406250, 0.0015258789062500,
+0.0018920898437500, 0.0020751953125000, 0.0010375976562500, 0.0036315917968750,
+0.0054016113281250, 0.0034179687500000, 0.0047912597656250, 0.0044860839843750,
+0.0020446777343750, 0.0041198730468750, 0.0032958984375000, 0.0040893554687500,
+0.0028076171875000, 0.0047302246093750, 0.0003662109375000, 0.0045166015625000,
+0.0014953613281250, 0.0057373046875000, 0.0007629394531250, 0.0049133300781250,
+0.0000915527343750, 0.0055541992187500, 0.0008239746093750, 0.0073852539062500,
+0.0014343261718750, 0.0061340332031250, 0.0009765625000000, 0.0055236816406250,
+0.0014648437500000, 0.0079040527343750, 0.0015563964843750, 0.0035400390625000,
+0.0016784667968750, 0.0002441406250000, 0.0017395019531250, 0.0041503906250000,
+0.0017089843750000, -0.0096130371093750, 0.0034179687500000, -0.0076599121093750,
+0.0025329589843750, -0.0092468261718750, 0.0039062500000000, -0.0193786621093750,
+0.0060119628906250, -0.0159301757812500, 0.0047302246093750, -0.0156860351562500,
+0.0085754394531250, -0.0189514160156250, 0.0075378417968750, -0.0150756835937500,
+0.0091857910156250, -0.0111694335937500, 0.0131530761718750, -0.0143737792968750,
+0.0109558105468750, -0.0056457519531250, 0.0125122070312500, -0.0075683593750000,
+0.0149536132812500, -0.0052185058593750, 0.0107116699218750, 0.0021972656250000,
+0.0079650878906250, -0.0017395019531250, 0.0112609863281250, 0.0024719238281250,
+-0.0001220703125000, 0.0042114257812500, 0.0005798339843750, 0.0021667480468750,
+-0.0003662109375000, 0.0021362304687500, -0.0072631835937500, 0.0031127929687500,
+-0.0049133300781250, -0.0006713867187500, -0.0067138671875000, 0.0000915527343750,
+-0.0075683593750000, -0.0008239746093750, -0.0075073242187500, -0.0042419433593750,
+-0.0073852539062500, -0.0026550292968750, -0.0072021484375000, -0.0017700195312500,
+-0.0102539062500000, -0.0037536621093750, -0.0091247558593750, -0.0016784667968750,
+-0.0103759765625000, 0.0012817382812500, -0.0137939453125000, -0.0006713867187500,
+-0.0119628906250000, 0.0027770996093750, -0.0115356445312500, 0.0027770996093750,
+-0.0132446289062500, 0.0030517578125000, -0.0083923339843750, 0.0053710937500000,
+-0.0039978027343750, 0.0043640136718750, -0.0061035156250000, 0.0029296875000000,
+0.0057067871093750, 0.0045166015625000, 0.0059509277343750, 0.0030822753906250,
+0.0106811523437500, 0.0003967285156250, 0.0197448730468750, 0.0018615722656250,
+0.0166625976562500, -0.0007629394531250, 0.0261230468750000, -0.0004272460937500,
+0.0267028808593750, -0.0012207031250000, 0.0274353027343750, -0.0031433105468750,
+0.0325012207031250, -0.0019226074218750, 0.0311584472656250, -0.0046691894531250,
+0.0333557128906250, -0.0039672851562500, 0.0341186523437500, -0.0054931640625000,
+0.0336608886718750, -0.0079650878906250, 0.0337524414062500, -0.0059814453125000,
+0.0334167480468750, -0.0101013183593750, 0.0324401855468750, -0.0102539062500000,
+0.0320434570312500, -0.0098876953125000, 0.0318298339843750, -0.0117187500000000,
+0.0310363769531250, -0.0113220214843750, 0.0306701660156250, -0.0115356445312500,
+0.0317687988281250, -0.0116271972656250, 0.0309143066406250, -0.0115661621093750,
+0.0314941406250000, -0.0116271972656250, 0.0325927734375000, -0.0116577148437500,
+0.0314636230468750, -0.0113525390625000, 0.0338134765625000, -0.0115051269531250,
+0.0332946777343750, -0.0113220214843750, 0.0338439941406250, -0.0109558105468750,
+0.0352172851562500, -0.0111999511718750, 0.0337829589843750, -0.0103454589843750,
+0.0363769531250000, -0.0105590820312500, 0.0351257324218750, -0.0101928710937500,
+0.0352783203125000, -0.0094909667968750, 0.0370788574218750, -0.0098876953125000,
+0.0352783203125000, -0.0088195800781250, 0.0369567871093750, -0.0089721679687500,
+0.0362243652343750, -0.0084838867187500, 0.0365905761718750, -0.0075073242187500,
+0.0377502441406250, -0.0081176757812500, 0.0360717773437500, -0.0070495605468750,
+0.0386962890625000, -0.0068054199218750, 0.0378417968750000, -0.0072021484375000,
+0.0381469726562500, -0.0069885253906250, 0.0398864746093750, -0.0068664550781250,
+0.0383300781250000, -0.0075073242187500, 0.0400085449218750, -0.0076599121093750,
+0.0394897460937500, -0.0073852539062500, 0.0396423339843750, -0.0073242187500000,
+0.0408630371093750, -0.0075073242187500, 0.0394592285156250, -0.0069885253906250,
+0.0399169921875000, -0.0066223144531250, 0.0404968261718750, -0.0072937011718750,
+0.0392150878906250, -0.0075683593750000, 0.0388488769531250, -0.0069580078125000,
+0.0395202636718750, -0.0090332031250000, 0.0353088378906250, -0.0086975097656250,
+0.0365905761718750, -0.0091247558593750, 0.0347290039062500, -0.0106506347656250,
+0.0308837890625000, -0.0099487304687500, 0.0330200195312500, -0.0109558105468750,
+0.0270690917968750, -0.0112304687500000, 0.0275268554687500, -0.0107421875000000,
+0.0266418457031250, -0.0108642578125000, 0.0227050781250000, -0.0110473632812500,
+0.0242919921875000, -0.0091552734375000, 0.0220642089843750, -0.0098571777343750,
+0.0214843750000000, -0.0087280273437500, 0.0216674804687500, -0.0068664550781250,
+0.0208740234375000, -0.0079345703125000, 0.0209350585937500, -0.0047607421875000,
+0.0213623046875000, -0.0054016113281250, 0.0211791992187500, -0.0039978027343750,
+0.0200500488281250, -0.0013427734375000, 0.0205993652343750, -0.0030517578125000,
+0.0213012695312500, 0.0010070800781250, 0.0143432617187500, 0.0007934570312500,
+0.0179443359375000, 0.0012817382812500, 0.0114746093750000, 0.0036621093750000,
+0.0025329589843750, 0.0026550292968750, 0.0088806152343750, 0.0048522949218750,
+-0.0090942382812500, 0.0044860839843750, -0.0072937011718750, 0.0056457519531250,
+-0.0109863281250000, 0.0077819824218750, -0.0240478515625000, 0.0065002441406250,
+-0.0184020996093750, 0.0085449218750000, -0.0257263183593750, 0.0096435546875000,
+-0.0288391113281250, 0.0074768066406250, -0.0240173339843750, 0.0067749023437500,
+-0.0231628417968750, 0.0083618164062500, -0.0256958007812500, 0.0034484863281250,
+-0.0155334472656250, 0.0038452148437500, -0.0160522460937500, 0.0025024414062500,
+-0.0151672363281250, -0.0003051757812500, -0.0089721679687500, 0.0011901855468750,
+-0.0107421875000000, -0.0058288574218750, -0.0091247558593750, -0.0048522949218750,
+-0.0095214843750000, -0.0060424804687500, -0.0075378417968750, -0.0107116699218750,
+-0.0057067871093750, -0.0090026855468750, -0.0077514648437500, -0.0141296386718750,
+-0.0000915527343750, -0.0131835937500000, -0.0010070800781250, -0.0139160156250000,
+0.0007934570312500, -0.0185546875000000, 0.0065612792968750, -0.0169982910156250,
+0.0044860839843750, -0.0144348144531250, 0.0078125000000000, -0.0176391601562500,
+0.0086364746093750, -0.0123596191406250, 0.0073242187500000, -0.0059814453125000,
+0.0078430175781250, -0.0108337402343750, 0.0085144042968750, -0.0006103515625000,
+0.0067138671875000, 0.0005798339843750, 0.0065307617187500, -0.0012512207031250,
+0.0054016113281250, 0.0027465820312500, 0.0042419433593750, 0.0028686523437500,
+0.0053100585937500, -0.0033264160156250, 0.0024414062500000, -0.0014038085937500,
+0.0024108886718750, -0.0030822753906250, 0.0022277832031250, -0.0082702636718750,
+0.0005798339843750, -0.0058288574218750, 0.0010375976562500, -0.0095825195312500,
+0.0010375976562500, -0.0097351074218750, 0.0007019042968750, -0.0100402832031250,
+0.0012207031250000, -0.0127868652343750, 0.0012512207031250, -0.0112609863281250,
+0.0007019042968750, -0.0074462890625000, 0.0034179687500000, -0.0111694335937500,
+0.0026855468750000, -0.0067138671875000, 0.0049438476562500, 0.0062866210937500,
+0.0105895996093750, 0.0130004882812500, 0.0033874511718750, 0.0007934570312500,
+0.0005798339843750, 0.0067749023437500, -0.0014953613281250, 0.0028381347656250,
+0.0028381347656250, -0.0061645507812500, 0.0052795410156250, -0.0026245117187500,
+0.0022888183593750, -0.0023193359375000, 0.0131835937500000, -0.0051879882812500,
+0.0139160156250000},
+{0.0280151367187500, -0.0085144042968750, 0.0266723632812500, -0.0085449218750000,
+0.0278320312500000, -0.0087280273437500, 0.0324707031250000, -0.0083312988281250,
+0.0296630859375000, -0.0085754394531250, 0.0339050292968750, -0.0086669921875000,
+0.0342102050781250, -0.0082702636718750, 0.0330505371093750, -0.0081176757812500,
+0.0346984863281250, -0.0082092285156250, 0.0345153808593750, -0.0073547363281250,
+0.0315246582031250, -0.0077819824218750, 0.0328369140625000, -0.0068969726562500,
+0.0305175781250000, -0.0058898925781250, 0.0270690917968750, -0.0067749023437500,
+0.0292663574218750, -0.0036315917968750, 0.0227966308593750, -0.0042419433593750,
+0.0232849121093750, -0.0033569335937500, 0.0222778320312500, -0.0009460449218750,
+0.0177001953125000, -0.0022888183593750, 0.0194396972656250, 0.0004272460937500,
+0.0186767578125000, 0.0000305175781250, 0.0176086425781250, 0.0012207031250000,
+0.0183410644531250, 0.0034484863281250, 0.0192565917968750, 0.0020751953125000,
+0.0184326171875000, 0.0055541992187500, 0.0181579589843750, 0.0056457519531250,
+0.0197448730468750, 0.0052795410156250, 0.0165405273437500, 0.0072937011718750,
+0.0144042968750000, 0.0069885253906250, 0.0169067382812500, 0.0042724609375000,
+0.0057067871093750, 0.0055847167968750, 0.0086364746093750, 0.0039367675781250,
+0.0027770996093750, 0.0006713867187500, -0.0083312988281250, 0.0021057128906250,
+-0.0019531250000000, -0.0020751953125000, -0.0140380859375000, -0.0009155273437500,
+-0.0155334472656250, -0.0039062500000000, -0.0141906738281250, -0.0087890625000000,
+-0.0186157226562500, -0.0056762695312500, -0.0180969238281250, -0.0111999511718750,
+-0.0173034667968750, -0.0123596191406250, -0.0170898437500000, -0.0114440917968750,
+-0.0168762207031250, -0.0130004882812500, -0.0171813964843750, -0.0125122070312500,
+-0.0170593261718750, -0.0095214843750000, -0.0132446289062500, -0.0116882324218750,
+-0.0157470703125000, -0.0081481933593750, -0.0111389160156250, -0.0032043457031250,
+-0.0048522949218750, -0.0063476562500000, -0.0087890625000000, 0.0013732910156250,
+0.0022583007812500, 0.0018005371093750, 0.0022277832031250, 0.0009765625000000,
+0.0033874511718750, 0.0050354003906250, 0.0098266601562500, 0.0047302246093750,
+0.0072021484375000, -0.0001525878906250, 0.0085754394531250, 0.0028991699218750,
+0.0097961425781250, -0.0015869140625000, 0.0092468261718750, -0.0088195800781250,
+0.0087890625000000, -0.0039062500000000, 0.0087585449218750, -0.0111389160156250,
+0.0079956054687500, -0.0131225585937500, 0.0084838867187500, -0.0102233886718750,
+0.0074768066406250, -0.0115051269531250, 0.0065002441406250, -0.0123291015625000,
+0.0075073242187500, -0.0037841796875000, 0.0038757324218750, -0.0064697265625000,
+0.0038757324218750, -0.0015563964843750, 0.0041809082031250, 0.0076293945312500,
+0.0023498535156250, 0.0023498535156250, 0.0026855468750000, 0.0111083984375000,
+0.0036926269531250, 0.0140686035156250, 0.0034790039062500, 0.0106811523437500,
+0.0022888183593750, 0.0101013183593750, 0.0016174316406250, -0.0068359375000000,
+0.0031127929687500, -0.0048522949218750, 0.0046691894531250, -0.0077819824218750,
+0.0066223144531250, -0.0045166015625000, 0.0032043457031250, -0.0003356933593750,
+0.0002441406250000, -0.0027160644531250, 0.0026550292968750, 0.0008544921875000,
+-0.0020446777343750, 0.0011291503906250, -0.0044555664062500, 0.0016174316406250,
+0.0008850097656250, 0.0031127929687500, 0.0032958984375000, 0.0023193359375000,
+-0.0004882812500000, 0.0066833496093750, 0.0096130371093750, 0.0047912597656250,
+0.0105590820312500, 0.0075683593750000, 0.0081787109375000, 0.0127563476562500,
+0.0119628906250000, 0.0096130371093750, 0.0125122070312500, 0.0158996582031250,
+0.0045471191406250, 0.0169982910156250, 0.0060119628906250, 0.0141601562500000,
+0.0051574707031250, 0.0154418945312500, -0.0003967285156250, 0.0171508789062500,
+0.0012207031250000, 0.0109863281250000, -0.0003356933593750, 0.0115051269531250,
+-0.0007629394531250, 0.0102539062500000, 0.0002136230468750, 0.0067443847656250,
+0.0004272460937500, 0.0083007812500000, 0.0001525878906250, 0.0024108886718750,
+-0.0002441406250000, 0.0041198730468750, 0.0008544921875000, 0.0011596679687500,
+-0.0002441406250000, -0.0040893554687500, -0.0016784667968750, -0.0010375976562500,
+-0.0010070800781250, -0.0117797851562500, -0.0052490234375000, -0.0091857910156250,
+-0.0033264160156250, -0.0131225585937500, -0.0069274902343750, -0.0231323242187500,
+-0.0126647949218750, -0.0172424316406250, -0.0090637207031250, -0.0219726562500000,
+-0.0160217285156250, -0.0258178710937500, -0.0173645019531250, -0.0198059082031250,
+-0.0146789550781250, -0.0169677734375000, -0.0165405273437500, -0.0207214355468750,
+-0.0174560546875000, -0.0058288574218750, -0.0082092285156250, -0.0086364746093750,
+-0.0102539062500000, -0.0049438476562500, -0.0076293945312500, 0.0068359375000000,
+0.0002746582031250, 0.0011901855468750, -0.0027770996093750, 0.0083618164062500,
+0.0017700195312500, 0.0101623535156250, 0.0024719238281250, 0.0091247558593750,
+0.0023803710937500, 0.0106506347656250, 0.0043334960937500, 0.0102844238281250,
+0.0037841796875000, 0.0121459960937500, 0.0042419433593750, 0.0104675292968750,
+0.0046997070312500, 0.0133056640625000, 0.0039672851562500, 0.0165405273437500,
+0.0036010742187500, 0.0137634277343750, 0.0039978027343750, 0.0223083496093750,
+0.0016479492187500, 0.0207519531250000, 0.0018920898437500, 0.0237731933593750,
+0.0015563964843750, 0.0302734375000000, -0.0001525878906250, 0.0260009765625000,
+0.0003051757812500, 0.0353393554687500, -0.0002441406250000, 0.0352478027343750,
+-0.0001831054687500, 0.0355224609375000, -0.0008850097656250, 0.0408325195312500,
+-0.0014343261718750, 0.0389099121093750, -0.0007324218750000, 0.0387268066406250,
+-0.0030822753906250, 0.0408020019531250, -0.0029907226562500, 0.0379028320312500,
+-0.0028686523437500, 0.0355224609375000, -0.0040893554687500, 0.0380249023437500,
+-0.0037841796875000, 0.0290222167968750, -0.0036010742187500, 0.0313415527343750,
+-0.0036010742187500, 0.0278930664062500, -0.0038146972656250, 0.0205688476562500,
+-0.0039367675781250, 0.0257263183593750, -0.0037231445312500, 0.0152587890625000,
+-0.0041809082031250, 0.0167541503906250, -0.0043029785156250, 0.0151977539062500,
+-0.0041198730468750, 0.0084228515625000, -0.0041198730468750, 0.0124816894531250,
+-0.0042419433593750, 0.0056457519531250, -0.0039367675781250, 0.0063171386718750,
+-0.0039367675781250, 0.0059204101562500, -0.0037841796875000, 0.0017089843750000,
+-0.0035400390625000, 0.0035705566406250, -0.0036926269531250, 0.0006103515625000,
+-0.0032043457031250, 0.0011901855468750, -0.0031127929687500, 0.0008544921875000,
+-0.0031127929687500, -0.0011901855468750, -0.0028076171875000, -0.0000610351562500,
+-0.0027465820312500, -0.0020446777343750, -0.0029296875000000, -0.0017700195312500,
+-0.0028991699218750, -0.0021362304687500, -0.0028991699218750, -0.0033569335937500,
+-0.0028991699218750, -0.0025024414062500, -0.0030517578125000, -0.0047302246093750,
+-0.0033264160156250, -0.0042724609375000, -0.0030212402343750, -0.0046081542968750,
+-0.0037841796875000, -0.0061035156250000, -0.0046081542968750, -0.0052185058593750,
+-0.0040893554687500, -0.0066833496093750, -0.0059814453125000, -0.0065917968750000,
+-0.0059814453125000, -0.0068664550781250, -0.0063781738281250, -0.0078125000000000,
+-0.0076293945312500, -0.0071716308593750, -0.0071411132812500, -0.0082397460937500,
+-0.0082092285156250, -0.0079956054687500, -0.0082702636718750, -0.0074768066406250,
+-0.0082702636718750, -0.0077514648437500, -0.0088195800781250, -0.0073242187500000,
+-0.0087280273437500, -0.0055236816406250, -0.0092773437500000, -0.0059509277343750,
+-0.0087890625000000, -0.0043640136718750, -0.0097045898437500, -0.0021667480468750,
+-0.0108032226562500, -0.0026245117187500, -0.0097351074218750, 0.0021667480468750,
+-0.0115966796875000, 0.0017089843750000, -0.0119018554687500, 0.0041198730468750,
+-0.0115356445312500, 0.0085144042968750, -0.0121459960937500, 0.0072326660156250,
+-0.0119323730468750, 0.0128784179687500, -0.0111083984375000, 0.0129089355468750,
+-0.0117797851562500, 0.0141296386718750, -0.0100402832031250, 0.0180969238281250,
+-0.0082702636718750, 0.0171813964843750, -0.0096740722656250, 0.0198974609375000,
+-0.0059814453125000, 0.0202941894531250, -0.0056457519531250, 0.0203857421875000,
+-0.0061645507812500, 0.0216064453125000, -0.0046997070312500, 0.0213012695312500,
+-0.0047607421875000, 0.0212707519531250, -0.0057983398437500, 0.0211486816406250,
+-0.0053710937500000, 0.0216064453125000, -0.0061340332031250, 0.0216674804687500,
+-0.0075378417968750, 0.0213317871093750, -0.0065307617187500, 0.0242004394531250,
+-0.0068054199218750, 0.0231018066406250, -0.0081176757812500, 0.0255126953125000,
+-0.0059509277343750, 0.0291748046875000, -0.0040283203125000, 0.0266113281250000,
+-0.0057067871093750, 0.0309753417968750, -0.0009765625000000, 0.0319213867187500,
+-0.0010070800781250, 0.0299377441406250, -0.0011291503906250, 0.0312500000000000,
+0.0014953613281250, 0.0318603515625000, 0.0009765625000000, 0.0238647460937500,
+0.0005493164062500, 0.0267944335937500, 0.0014343261718750, 0.0205688476562500,
+-0.0010375976562500, 0.0116271972656250, -0.0025329589843750, 0.0173950195312500,
+-0.0001220703125000, 0.0012817382812500, -0.0072937011718750, 0.0023193359375000,
+-0.0063781738281250, 0.0002746582031250, -0.0083923339843750, -0.0102539062500000,
+-0.0135803222656250, -0.0063171386718750, -0.0101623535156250, -0.0102844238281250,
+-0.0145568847656250, -0.0116577148437500, -0.0163879394531250, -0.0111083984375000,
+-0.0143127441406250, -0.0122985839843750, -0.0145874023437500, -0.0118408203125000,
+-0.0153198242187500, -0.0105590820312500, -0.0089111328125000, -0.0118103027343750,
+-0.0113525390625000, -0.0090026855468750, -0.0083007812500000, -0.0061950683593750,
+-0.0014648437500000, -0.0081481933593750, -0.0051269531250000, -0.0002136230468750,
+0.0007324218750000, -0.0017089843750000, 0.0016174316406250, 0.0012817382812500,
+0.0000915527343750, 0.0080566406250000, 0.0019226074218750, 0.0045776367187500,
+0.0020141601562500, 0.0104980468750000, -0.0015258789062500, 0.0118408203125000,
+-0.0002746582031250, 0.0097351074218750, -0.0010070800781250, 0.0109252929687500,
+-0.0043334960937500, 0.0115356445312500, -0.0034179687500000, 0.0062866210937500,
+-0.0035095214843750, 0.0074768066406250, -0.0039672851562500, 0.0057373046875000,
+-0.0032958984375000, 0.0006103515625000, -0.0027770996093750, 0.0012512207031250,
+-0.0028076171875000, -0.0025939941406250, -0.0009460449218750, -0.0019836425781250,
+-0.0016784667968750, -0.0014953613281250, -0.0013122558593750, -0.0041198730468750,
+0.0001831054687500, -0.0036621093750000, -0.0003662109375000, -0.0008544921875000,
+0.0013122558593750, -0.0027160644531250, 0.0004882812500000, -0.0010375976562500,
+0.0023498535156250, 0.0030212402343750, 0.0046997070312500, 0.0007324218750000,
+0.0029602050781250, 0.0010681152343750, 0.0081176757812500, 0.0031738281250000,
+0.0082702636718750, 0.0016174316406250, 0.0075378417968750, -0.0011596679687500,
+0.0101928710937500, 0.0002441406250000, 0.0101928710937500, 0.0013122558593750,
+0.0063781738281250, -0.0003967285156250, 0.0073547363281250, 0.0014953613281250,
+0.0062561035156250, 0.0050964355468750, 0.0035705566406250, 0.0033569335937500,
+0.0049133300781250, 0.0023803710937500, 0.0009765625000000, 0.0051879882812500,
+0.0018310546875000, 0.0010986328125000, 0.0006103515625000, -0.0034790039062500,
+-0.0027160644531250},
+{-0.0090332031250000, -0.0123901367187500, -0.0072326660156250, -0.0136718750000000,
+-0.0079956054687500, -0.0078735351562500, -0.0069274902343750, -0.0090026855468750,
+-0.0067138671875000, -0.0061035156250000, -0.0069580078125000, -0.0005798339843750,
+-0.0065612792968750, -0.0031738281250000, -0.0067138671875000, 0.0043334960937500,
+-0.0079956054687500, 0.0043029785156250, -0.0075073242187500, 0.0055236816406250,
+-0.0074462890625000, 0.0100097656250000, -0.0089721679687500, 0.0079040527343750,
+-0.0086669921875000, 0.0105285644531250, -0.0054626464843750, 0.0113830566406250,
+-0.0069885253906250, 0.0110778808593750, -0.0049743652343750, 0.0114135742187500,
+-0.0008544921875000, 0.0115051269531250, -0.0030212402343750, 0.0133056640625000,
+-0.0007324218750000, 0.0120239257812500, 0.0007324218750000, 0.0144958496093750,
+-0.0012817382812500, 0.0176696777343750, -0.0020751953125000, 0.0154724121093750,
+-0.0009765625000000, 0.0213928222656250, -0.0061950683593750, 0.0217285156250000,
+-0.0043640136718750, 0.0220031738281250, -0.0064697265625000, 0.0251159667968750,
+-0.0117492675781250, 0.0237426757812500, -0.0090332031250000, 0.0223693847656250,
+-0.0124511718750000, 0.0246276855468750, -0.0129089355468750, 0.0195922851562500,
+-0.0116271972656250, 0.0149841308593750, -0.0132446289062500, 0.0187988281250000,
+-0.0135498046875000, 0.0061035156250000, -0.0074768066406250, 0.0066528320312500,
+-0.0096740722656250, 0.0043945312500000, -0.0064392089843750, -0.0039062500000000,
+-0.0000610351562500, -0.0007019042968750, -0.0039367675781250, -0.0064392089843750,
+0.0021972656250000, -0.0073242187500000, 0.0036926269531250, -0.0069885253906250,
+0.0014343261718750, -0.0094299316406250, 0.0023498535156250, -0.0088500976562500,
+0.0031127929687500, -0.0077819824218750, -0.0017700195312500, -0.0086669921875000,
+-0.0011596679687500, -0.0069274902343750, -0.0013427734375000, -0.0054321289062500,
+-0.0043334960937500, -0.0067443847656250, -0.0032653808593750, 0.0003967285156250,
+-0.0032043457031250, -0.0012817382812500, -0.0043334960937500, 0.0017089843750000,
+-0.0032348632812500, 0.0085449218750000, -0.0015258789062500, 0.0049743652343750,
+-0.0023803710937500, 0.0097351074218750, -0.0016174316406250, 0.0113525390625000,
+-0.0015869140625000, 0.0092163085937500, -0.0010986328125000, 0.0093078613281250,
+-0.0003356933593750, 0.0089721679687500, -0.0003967285156250, 0.0031127929687500,
+0.0017089843750000, 0.0056152343750000, 0.0010070800781250, 0.0043640136718750,
+0.0016479492187500, -0.0003967285156250, 0.0030517578125000, 0.0022583007812500,
+0.0021972656250000, -0.0026855468750000, 0.0064697265625000, -0.0031738281250000,
+0.0057983398437500, -0.0021972656250000, 0.0063171386718750, -0.0044555664062500,
+0.0098876953125000, -0.0044555664062500, 0.0087890625000000, -0.0003051757812500,
+0.0067443847656250, -0.0017089843750000, 0.0088500976562500, -0.0014648437500000,
+0.0057983398437500, 0.0020751953125000, 0.0012512207031250, 0.0013122558593750,
+0.0038146972656250, 0.0003662109375000, 0.0000305175781250, 0.0009765625000000,
+-0.0004272460937500, 0.0009765625000000, -0.0009155273437500, -0.0001525878906250,
+-0.0025329589843750, -0.0000305175781250, -0.0014953613281250, 0.0020751953125000,
+-0.0042419433593750, 0.0011291503906250, -0.0041198730468750, 0.0023193359375000,
+-0.0046691894531250, 0.0049438476562500, -0.0068969726562500, 0.0034179687500000,
+-0.0060729980468750, 0.0033264160156250, -0.0052490234375000, 0.0050964355468750,
+-0.0072021484375000, 0.0022888183593750, -0.0038757324218750, -0.0001220703125000,
+-0.0009460449218750, 0.0025634765625000, -0.0032348632812500, -0.0040588378906250,
+0.0077819824218750, -0.0028381347656250, 0.0059509277343750, -0.0032348632812500,
+0.0090026855468750, -0.0076293945312500, 0.0178222656250000, -0.0055541992187500,
+0.0139770507812500, -0.0066833496093750, 0.0203552246093750, -0.0075073242187500,
+0.0220031738281250, -0.0065612792968750, 0.0195617675781250, -0.0063781738281250,
+0.0204772949218750, -0.0071105957031250, 0.0212097167968750, -0.0050964355468750,
+0.0167541503906250, -0.0052795410156250, 0.0167846679687500, -0.0066223144531250,
+0.0158691406250000, -0.0063781738281250, 0.0126647949218750, -0.0058593750000000,
+0.0131225585937500, -0.0076904296875000, 0.0118408203125000, -0.0084228515625000,
+0.0111083984375000, -0.0073547363281250, 0.0106201171875000, -0.0074462890625000,
+0.0103149414062500, -0.0083007812500000, 0.0101318359375000, -0.0047302246093750,
+0.0075378417968750, -0.0055847167968750, 0.0079956054687500, -0.0046386718750000,
+0.0054321289062500, -0.0017089843750000, 0.0021667480468750, -0.0033874511718750,
+0.0037231445312500, -0.0010070800781250, -0.0015563964843750, -0.0006713867187500,
+-0.0016784667968750, -0.0016784667968750, -0.0027160644531250, -0.0011901855468750,
+-0.0062561035156250, -0.0007934570312500, -0.0053405761718750, -0.0024414062500000,
+-0.0065307617187500, -0.0025939941406250, -0.0077514648437500, -0.0021972656250000,
+-0.0067138671875000, -0.0027160644531250, -0.0060119628906250, -0.0029602050781250,
+-0.0068969726562500, -0.0026245117187500, -0.0045471191406250, -0.0022888183593750,
+-0.0048217773437500, -0.0032348632812500, -0.0043029785156250, -0.0036621093750000,
+-0.0025634765625000, -0.0030822753906250, -0.0033874511718750, -0.0056152343750000,
+-0.0021057128906250, -0.0052185058593750, -0.0020751953125000, -0.0054626464843750,
+-0.0023498535156250, -0.0071716308593750, -0.0019531250000000, -0.0064697265625000,
+-0.0022583007812500, -0.0068054199218750, -0.0028381347656250, -0.0068054199218750,
+-0.0030517578125000, -0.0068664550781250, -0.0035400390625000, -0.0069580078125000,
+-0.0041198730468750, -0.0066223144531250, -0.0042419433593750, -0.0071105957031250,
+-0.0047607421875000, -0.0072021484375000, -0.0051269531250000, -0.0068664550781250,
+-0.0049743652343750, -0.0068359375000000, -0.0049743652343750, -0.0070495605468750,
+-0.0052490234375000, -0.0069580078125000, -0.0044555664062500, -0.0066528320312500,
+-0.0049438476562500, -0.0071716308593750, -0.0047912597656250, -0.0075683593750000,
+-0.0040893554687500, -0.0071105957031250, -0.0046386718750000, -0.0083312988281250,
+-0.0037536621093750, -0.0083312988281250, -0.0037841796875000, -0.0080871582031250,
+-0.0037536621093750, -0.0086364746093750, -0.0032653808593750, -0.0085144042968750,
+-0.0033264160156250, -0.0075683593750000, -0.0025634765625000, -0.0079345703125000,
+-0.0027465820312500, -0.0073852539062500, -0.0021667480468750, -0.0062561035156250,
+-0.0014343261718750, -0.0066528320312500, -0.0018920898437500, -0.0058593750000000,
+0.0002746582031250, -0.0056457519531250, -0.0006103515625000, -0.0056152343750000,
+0.0009155273437500, -0.0053100585937500, 0.0031127929687500, -0.0053405761718750,
+0.0011596679687500, -0.0052490234375000, 0.0055847167968750, -0.0052185058593750,
+0.0047302246093750, -0.0050659179687500, 0.0054931640625000, -0.0050048828125000,
+0.0087280273437500, -0.0050964355468750, 0.0071105957031250, -0.0040588378906250,
+0.0094299316406250, -0.0043945312500000, 0.0097045898437500, -0.0036010742187500,
+0.0090942382812500, -0.0022277832031250, 0.0096740722656250, -0.0028686523437500,
+0.0097351074218750, -0.0016479492187500, 0.0095214843750000, -0.0012817382812500,
+0.0094604492187500, -0.0015258789062500, 0.0094909667968750, -0.0013427734375000,
+0.0093994140625000, -0.0011596679687500, 0.0092468261718750, -0.0015563964843750,
+0.0092468261718750, -0.0013732910156250, 0.0087280273437500, -0.0017089843750000,
+0.0104675292968750, -0.0020141601562500, 0.0114440917968750, -0.0017395019531250,
+0.0098571777343750, -0.0033264160156250, 0.0149230957031250, -0.0028991699218750,
+0.0139160156250000, -0.0036315917968750, 0.0156860351562500, -0.0050964355468750,
+0.0198364257812500, -0.0044250488281250, 0.0173645019531250, -0.0064392089843750,
+0.0216064453125000, -0.0063171386718750, 0.0222167968750000, -0.0067443847656250,
+0.0209350585937500, -0.0080871582031250, 0.0223388671875000, -0.0075988769531250,
+0.0227050781250000, -0.0086669921875000, 0.0186462402343750, -0.0087280273437500,
+0.0195617675781250, -0.0087585449218750, 0.0184326171875000, -0.0097961425781250,
+0.0148620605468750, -0.0093078613281250, 0.0161132812500000, -0.0066833496093750,
+0.0145568847656250, -0.0084533691406250, 0.0140380859375000, -0.0059814453125000,
+0.0137634277343750, -0.0016479492187500, 0.0131530761718750, -0.0037536621093750,
+0.0137939453125000, -0.0000305175781250, 0.0124511718750000, 0.0008850097656250,
+0.0125122070312500, 0.0000610351562500, 0.0125732421875000, 0.0009765625000000,
+0.0124206542968750, 0.0012207031250000, 0.0125732421875000, 0.0001525878906250,
+0.0102233886718750, -0.0000305175781250, 0.0130004882812500, 0.0007019042968750,
+0.0076599121093750, 0.0008544921875000, 0.0022583007812500, 0.0005798339843750,
+0.0067749023437500, 0.0020751953125000, -0.0087890625000000, 0.0020751953125000,
+-0.0076599121093750, 0.0020751953125000, -0.0097045898437500, 0.0023193359375000,
+-0.0203552246093750, 0.0023498535156250, -0.0161743164062500, 0.0053405761718750,
+-0.0166320800781250, 0.0032348632812500, -0.0209655761718750, 0.0056152343750000,
+-0.0143432617187500, 0.0105285644531250, -0.0079040527343750, 0.0079650878906250,
+-0.0128173828125000, 0.0106811523437500, 0.0001220703125000, 0.0130004882812500,
+0.0006713867187500, 0.0086364746093750, 0.0004272460937500, 0.0066528320312500,
+0.0065307617187500, 0.0099182128906250, 0.0051574707031250, -0.0022277832031250,
+0.0059509277343750, -0.0014953613281250, 0.0064086914062500, -0.0018005371093750,
+0.0062561035156250, -0.0097961425781250, 0.0066528320312500, -0.0078125000000000,
+0.0065307617187500, -0.0034790039062500, 0.0054016113281250, -0.0058898925781250,
+0.0062561035156250, -0.0048217773437500, 0.0049743652343750, 0.0020141601562500,
+0.0014038085937500, 0.0014343261718750, 0.0033874511718750, 0.0043945312500000,
+0.0028381347656250, 0.0007934570312500, 0.0043640136718750, 0.0049743652343750,
+0.0024414062500000, 0.0104675292968750, 0.0000000000000000, 0.0068054199218750,
+0.0015869140625000, 0.0094909667968750, 0.0009765625000000, 0.0122985839843750,
+-0.0011291503906250, 0.0086059570312500, 0.0019836425781250, 0.0059509277343750,
+0.0052795410156250, 0.0086364746093750, 0.0025939941406250, 0.0018920898437500,
+0.0061950683593750, 0.0021057128906250, 0.0084228515625000, 0.0008544921875000,
+0.0047302246093750, -0.0026245117187500, 0.0027465820312500, -0.0007324218750000,
+0.0056457519531250, -0.0081481933593750, 0.0003662109375000, -0.0063781738281250,
+-0.0021667480468750, -0.0100402832031250, 0.0016784667968750, -0.0174255371093750,
+0.0029296875000000, -0.0129394531250000, 0.0006103515625000, -0.0185852050781250,
+0.0088500976562500, -0.0220336914062500, 0.0083618164062500, -0.0158081054687500,
+0.0084228515625000, -0.0134277343750000, 0.0140686035156250, -0.0174560546875000,
+0.0128479003906250, -0.0026855468750000, 0.0082092285156250, -0.0047607421875000,
+0.0103149414062500, -0.0018920898437500, 0.0082702636718750, 0.0085144042968750,
+0.0035095214843750, 0.0039672851562500, 0.0056762695312500, 0.0121459960937500,
+0.0008850097656250, 0.0132751464843750, 0.0012817382812500, 0.0114440917968750,
+0.0003356933593750, 0.0140075683593750, -0.0028076171875000, 0.0142517089843750,
+-0.0012817382812500, 0.0123596191406250, -0.0049743652343750, 0.0117187500000000,
+-0.0047302246093750, 0.0135803222656250, -0.0051269531250000, 0.0139465332031250,
+-0.0074462890625000, 0.0126342773437500, -0.0065612792968750, 0.0181579589843750,
+-0.0079650878906250},
+{0.0157165527343750, -0.0041198730468750, 0.0165100097656250, -0.0037536621093750,
+0.0164794921875000, -0.0041809082031250, 0.0161437988281250, -0.0035095214843750,
+0.0164794921875000, -0.0034179687500000, 0.0161743164062500, -0.0036621093750000,
+0.0144958496093750, -0.0040893554687500, 0.0150756835937500, -0.0025634765625000,
+0.0143737792968750, -0.0020141601562500, 0.0123901367187500, -0.0031127929687500,
+0.0132446289062500, 0.0029602050781250, 0.0145263671875000, 0.0018615722656250,
+0.0130920410156250, 0.0036926269531250, 0.0145568847656250, 0.0086975097656250,
+0.0176696777343750, 0.0064086914062500, 0.0162963867187500, 0.0094604492187500,
+0.0159301757812500, 0.0104980468750000, 0.0189819335937500, 0.0097656250000000,
+0.0130004882812500, 0.0101318359375000, 0.0080871582031250, 0.0099792480468750,
+0.0126342773437500, 0.0087585449218750, -0.0048217773437500, 0.0101928710937500,
+-0.0023498535156250, 0.0080261230468750, -0.0061035156250000, 0.0053405761718750,
+-0.0198364257812500, 0.0073547363281250, -0.0140380859375000, 0.0018615722656250,
+-0.0173034667968750, 0.0025634765625000, -0.0217590332031250, 0.0021057128906250,
+-0.0154724121093750, -0.0021057128906250, -0.0104064941406250, -0.0007324218750000,
+-0.0146484375000000, 0.0001831054687500, -0.0038452148437500, -0.0015258789062500,
+-0.0030517578125000, 0.0004577636718750, -0.0030517578125000, 0.0038146972656250,
+0.0019531250000000, 0.0019531250000000, 0.0008544921875000, 0.0020751953125000,
+0.0020446777343750, 0.0035400390625000, 0.0022277832031250, 0.0029602050781250,
+0.0026550292968750, 0.0005493164062500, 0.0043334960937500, 0.0065917968750000,
+-0.0051574707031250, 0.0099182128906250, -0.0006103515625000, 0.0112304687500000,
+-0.0027770996093750, 0.0067443847656250, -0.0001831054687500, 0.0048217773437500,
+0.0039367675781250, 0.0082397460937500, 0.0019836425781250, -0.0014953613281250,
+0.0025329589843750, -0.0009155273437500, 0.0044555664062500, -0.0020141601562500,
+0.0024108886718750, -0.0075988769531250, -0.0007324218750000, -0.0053405761718750,
+0.0009765625000000, -0.0104370117187500, 0.0020446777343750, -0.0095214843750000,
+-0.0009155273437500, -0.0121765136718750, 0.0028381347656250, -0.0175170898437500,
+0.0083923339843750, -0.0144653320312500, 0.0049133300781250, -0.0173645019531250,
+0.0075378417968750, -0.0206298828125000, 0.0104980468750000, -0.0137329101562500,
+0.0071716308593750, -0.0105285644531250, 0.0039978027343750, -0.0156250000000000,
+0.0060424804687500, 0.0046997070312500, 0.0024108886718750, 0.0022888183593750,
+0.0009155273437500, 0.0044555664062500, 0.0036926269531250, 0.0180969238281250,
+0.0040588378906250, 0.0131835937500000, 0.0021057128906250, 0.0165710449218750,
+0.0091857910156250, 0.0192565917968750, 0.0097351074218750, 0.0167236328125000,
+0.0082397460937500, 0.0153808593750000, 0.0105895996093750, 0.0167846679687500,
+0.0108642578125000, 0.0126037597656250, 0.0073852539062500, 0.0123596191406250,
+0.0086059570312500, 0.0135803222656250, 0.0063781738281250, 0.0116577148437500,
+0.0026855468750000, 0.0110168457031250, 0.0048522949218750, 0.0159301757812500,
+-0.0006408691406250, 0.0147399902343750, 0.0000610351562500, 0.0159301757812500,
+-0.0017395019531250, 0.0196533203125000, -0.0059509277343750, 0.0174255371093750,
+-0.0034790039062500, 0.0203857421875000, -0.0076599121093750, 0.0210876464843750,
+-0.0085449218750000, 0.0194702148437500, -0.0073242187500000, 0.0195617675781250,
+-0.0079956054687500, 0.0201721191406250, -0.0082397460937500, 0.0173339843750000,
+-0.0068359375000000, 0.0177917480468750, -0.0067443847656250, 0.0167236328125000,
+-0.0067138671875000, 0.0147705078125000, -0.0062255859375000, 0.0159606933593750,
+-0.0063781738281250, 0.0113830566406250, -0.0061340332031250, 0.0128479003906250,
+-0.0063781738281250, 0.0105285644531250, -0.0057983398437500, 0.0062561035156250,
+-0.0051574707031250, 0.0090026855468750, -0.0057067871093750, 0.0012207031250000,
+-0.0041503906250000, 0.0026245117187500, -0.0040283203125000, 0.0002746582031250,
+-0.0041503906250000, -0.0054931640625000, -0.0036315917968750, -0.0019836425781250,
+-0.0035705566406250, -0.0097351074218750, -0.0027160644531250, -0.0091857910156250,
+-0.0032043457031250, -0.0103759765625000, -0.0023803710937500, -0.0152893066406250,
+-0.0008850097656250, -0.0126647949218750, -0.0016784667968750, -0.0159301757812500,
+-0.0007629394531250, -0.0164184570312500, -0.0003051757812500, -0.0154418945312500,
+-0.0006713867187500, -0.0164184570312500, -0.0008544921875000, -0.0161743164062500,
+-0.0007019042968750, -0.0136413574218750, -0.0005798339843750, -0.0147705078125000,
+-0.0007019042968750, -0.0127563476562500, -0.0007324218750000, -0.0099487304687500,
+-0.0003967285156250, -0.0119628906250000, -0.0003967285156250, -0.0061950683593750,
+-0.0012512207031250, -0.0074157714843750, -0.0010375976562500, -0.0056152343750000,
+-0.0012817382812500, -0.0012207031250000, -0.0021972656250000, -0.0039062500000000,
+-0.0018920898437500, 0.0021057128906250, -0.0021972656250000, 0.0010375976562500,
+-0.0022583007812500, 0.0020751953125000, -0.0024108886718750, 0.0063171386718750,
+-0.0026245117187500, 0.0041503906250000, -0.0024414062500000, 0.0079650878906250,
+-0.0030517578125000, 0.0071411132812500, -0.0030517578125000, 0.0076904296875000,
+-0.0029907226562500, 0.0103759765625000, -0.0032348632812500, 0.0086364746093750,
+-0.0031127929687500, 0.0111999511718750, -0.0030517578125000, 0.0107116699218750,
+-0.0029602050781250, 0.0107421875000000, -0.0031738281250000, 0.0123291015625000,
+-0.0032348632812500, 0.0114135742187500, -0.0030212402343750, 0.0124511718750000,
+-0.0042114257812500, 0.0123901367187500, -0.0038146972656250, 0.0122070312500000,
+-0.0042724609375000, 0.0126342773437500, -0.0053710937500000, 0.0124206542968750,
+-0.0048217773437500, 0.0124816894531250, -0.0058898925781250, 0.0123291015625000,
+-0.0057373046875000, 0.0120849609375000, -0.0063171386718750, 0.0120849609375000,
+-0.0073242187500000, 0.0119934082031250, -0.0065307617187500, 0.0114135742187500,
+-0.0075683593750000, 0.0116271972656250, -0.0080566406250000, 0.0107116699218750,
+-0.0071716308593750, 0.0096740722656250, -0.0067749023437500, 0.0101623535156250,
+-0.0073852539062500, 0.0079345703125000, -0.0066528320312500, 0.0083618164062500,
+-0.0060729980468750, 0.0073852539062500, -0.0066223144531250, 0.0052795410156250,
+-0.0072631835937500, 0.0058898925781250, -0.0067443847656250, 0.0032348632812500,
+-0.0065307617187500, 0.0032958984375000, -0.0072326660156250, 0.0025634765625000,
+-0.0064697265625000, 0.0006408691406250, -0.0054626464843750, 0.0009765625000000,
+-0.0061950683593750, -0.0009765625000000, -0.0047607421875000, -0.0009155273437500,
+-0.0048217773437500, -0.0013427734375000, -0.0052490234375000, -0.0027465820312500,
+-0.0044860839843750, -0.0024414062500000, -0.0046081542968750, -0.0036315917968750,
+-0.0069274902343750, -0.0036010742187500, -0.0062255859375000, -0.0041198730468750,
+-0.0065307617187500, -0.0048217773437500, -0.0087890625000000, -0.0043945312500000,
+-0.0082702636718750, -0.0062255859375000, -0.0063476562500000, -0.0057678222656250,
+-0.0073547363281250, -0.0067749023437500, -0.0068359375000000, -0.0086364746093750,
+-0.0045166015625000, -0.0074768066406250, -0.0052795410156250, -0.0099182128906250,
+-0.0064392089843750, -0.0101318359375000, -0.0050354003906250, -0.0098876953125000,
+-0.0069274902343750, -0.0109863281250000, -0.0098266601562500, -0.0106201171875000,
+-0.0080566406250000, -0.0091247558593750, -0.0106506347656250, -0.0099182128906250,
+-0.0111694335937500, -0.0076904296875000, -0.0107116699218750, -0.0051269531250000,
+-0.0118103027343750, -0.0064086914062500, -0.0116271972656250, -0.0012512207031250,
+-0.0085754394531250, -0.0019836425781250, -0.0098571777343750, -0.0000305175781250,
+-0.0086059570312500, 0.0045471191406250, -0.0058593750000000, 0.0023193359375000,
+-0.0070800781250000, 0.0052490234375000, -0.0021667480468750, 0.0063171386718750,
+-0.0030517578125000, 0.0057678222656250, -0.0027770996093750, 0.0058593750000000,
+0.0009155273437500, 0.0060729980468750, -0.0000305175781250, 0.0075988769531250,
+0.0002441406250000, 0.0065612792968750, 0.0011901855468750, 0.0083007812500000,
+0.0000000000000000, 0.0109863281250000, -0.0012817382812500, 0.0095825195312500,
+-0.0004272460937500, 0.0126342773437500, -0.0016479492187500, 0.0130310058593750,
+-0.0020141601562500, 0.0124206542968750, -0.0010375976562500, 0.0137023925781250,
+-0.0008850097656250, 0.0135192871093750, -0.0015258789062500, 0.0098571777343750,
+-0.0003662109375000, 0.0116882324218750, -0.0000610351562500, 0.0088806152343750,
+0.0003967285156250, 0.0041503906250000, 0.0006408691406250, 0.0068359375000000,
+0.0000610351562500, 0.0008544921875000, 0.0014648437500000, 0.0012512207031250,
+0.0011291503906250, -0.0000915527343750, 0.0017700195312500, -0.0042419433593750,
+0.0029907226562500, -0.0021057128906250, 0.0022888183593750, -0.0050354003906250,
+0.0038452148437500, -0.0062561035156250, 0.0034790039062500, -0.0040893554687500,
+0.0043640136718750, -0.0032958984375000, 0.0061950683593750, -0.0031127929687500,
+0.0057983398437500, 0.0027770996093750, 0.0076293945312500, 0.0004577636718750,
+0.0078125000000000, 0.0012512207031250, 0.0061035156250000, 0.0062255859375000,
+0.0066833496093750, 0.0039367675781250, 0.0078430175781250, 0.0058288574218750,
+0.0018310546875000, 0.0067749023437500, 0.0026245117187500, 0.0063781738281250,
+0.0012817382812500, 0.0064392089843750, -0.0031738281250000, 0.0063476562500000,
+-0.0012512207031250, 0.0064392089843750, -0.0029907226562500, 0.0071716308593750,
+-0.0036010742187500, 0.0057678222656250, -0.0033874511718750, 0.0048217773437500,
+-0.0042114257812500, 0.0061035156250000, -0.0038146972656250, 0.0018005371093750,
+-0.0019531250000000, 0.0027770996093750, -0.0040283203125000, 0.0012817382812500,
+-0.0008544921875000, -0.0028381347656250, 0.0029296875000000, -0.0007629394531250,
+-0.0000305175781250, -0.0014648437500000, 0.0085754394531250, -0.0028686523437500,
+0.0079040527343750, -0.0020751953125000, 0.0087585449218750, -0.0007324218750000,
+0.0146789550781250, -0.0012512207031250, 0.0127563476562500, -0.0021972656250000,
+0.0131225585937500, -0.0017395019531250, 0.0147399902343750, -0.0014953613281250,
+0.0130004882812500, -0.0024719238281250, 0.0109863281250000, -0.0025634765625000,
+0.0120544433593750, -0.0007324218750000, 0.0094909667968750, -0.0014038085937500,
+0.0091857910156250, -0.0010986328125000, 0.0090332031250000, 0.0004882812500000,
+0.0077819824218750, -0.0002746582031250, 0.0079345703125000, -0.0001220703125000,
+0.0066833496093750, 0.0005798339843750, 0.0070800781250000, -0.0001831054687500,
+0.0049133300781250, -0.0007629394531250, 0.0029907226562500, -0.0002746582031250,
+0.0044250488281250, -0.0033874511718750, -0.0016479492187500, -0.0014343261718750,
+-0.0012817382812500, -0.0035400390625000, -0.0022888183593750, -0.0072326660156250,
+-0.0064086914062500, -0.0044860839843750, -0.0047302246093750, -0.0107727050781250,
+-0.0058898925781250, -0.0095520019531250, -0.0066223144531250, -0.0105285644531250,
+-0.0050964355468750, -0.0158386230468750, -0.0041503906250000, -0.0136108398437500,
+-0.0047607421875000, -0.0137023925781250, -0.0030517578125000, -0.0161743164062500,
+-0.0024108886718750, -0.0132751464843750, -0.0025329589843750, -0.0105285644531250,
+-0.0021972656250000, -0.0133361816406250, -0.0020141601562500, -0.0062561035156250,
+-0.0023193359375000, -0.0070800781250000, -0.0024414062500000, -0.0065612792968750,
+-0.0023803710937500},
+{-0.0013732910156250, 0.0049438476562500, -0.0016784667968750, 0.0062255859375000,
+-0.0019531250000000, 0.0061035156250000, -0.0015869140625000, 0.0066528320312500,
+-0.0011596679687500, 0.0062866210937500, -0.0020446777343750, 0.0074768066406250,
+-0.0008239746093750, 0.0085754394531250, 0.0008239746093750, 0.0078125000000000,
+-0.0000305175781250, 0.0118103027343750, 0.0027160644531250, 0.0114135742187500,
+0.0022277832031250, 0.0111083984375000, 0.0025634765625000, 0.0139160156250000,
+0.0046691894531250, 0.0135192871093750, 0.0039367675781250, 0.0102539062500000,
+0.0043640136718750, 0.0118408203125000, 0.0043640136718750, 0.0089111328125000,
+0.0048522949218750, 0.0048217773437500, 0.0049743652343750, 0.0074768066406250,
+0.0045471191406250, 0.0003356933593750, 0.0075683593750000, 0.0007934570312500,
+0.0071716308593750, 0.0002746582031250, 0.0068359375000000, -0.0049133300781250,
+0.0093078613281250, -0.0040588378906250, 0.0085754394531250, -0.0035095214843750,
+0.0032348632812500, -0.0047302246093750, 0.0062255859375000, -0.0028076171875000,
+0.0045166015625000, -0.0013122558593750, -0.0012512207031250, -0.0033264160156250,
+0.0014648437500000, 0.0028686523437500, -0.0020141601562500, 0.0022888183593750,
+-0.0031127929687500, 0.0024719238281250, -0.0019836425781250, 0.0060424804687500,
+-0.0025329589843750, 0.0049743652343750, -0.0027770996093750, 0.0063781738281250,
+-0.0002746582031250, 0.0059814453125000, -0.0016479492187500, 0.0066528320312500,
+0.0013122558593750, 0.0081787109375000, 0.0048828125000000, 0.0073242187500000,
+0.0019836425781250, 0.0088806152343750, 0.0093994140625000, 0.0095825195312500,
+0.0096130371093750, 0.0073242187500000, 0.0094604492187500, 0.0071716308593750,
+0.0133056640625000, 0.0088500976562500, 0.0122985839843750, 0.0009155273437500,
+0.0108032226562500, 0.0022888183593750, 0.0125427246093750, 0.0015258789062500,
+0.0108337402343750, -0.0039672851562500, 0.0077514648437500, -0.0019226074218750,
+0.0093688964843750, -0.0032348632812500, 0.0075683593750000, -0.0036926269531250,
+0.0068664550781250, -0.0037536621093750, 0.0070800781250000, -0.0042419433593750,
+0.0071105957031250, -0.0039062500000000, 0.0070190429687500, -0.0045471191406250,
+0.0043334960937500, -0.0049438476562500, 0.0055541992187500, -0.0044555664062500,
+0.0035705566406250, -0.0042724609375000, 0.0003356933593750, -0.0045776367187500,
+0.0019836425781250, -0.0032958984375000, -0.0031127929687500, -0.0038757324218750,
+-0.0032958984375000, -0.0031433105468750, -0.0032958984375000, -0.0018920898437500,
+-0.0061645507812500, -0.0028381347656250, -0.0054321289062500, -0.0006408691406250,
+-0.0036926269531250, -0.0006713867187500, -0.0046081542968750, -0.0018310546875000,
+-0.0030212402343750, -0.0008544921875000, -0.0003356933593750, -0.0001831054687500,
+-0.0012207031250000, -0.0037841796875000, 0.0005187988281250, -0.0021972656250000,
+0.0014648437500000, -0.0050354003906250, 0.0008850097656250, -0.0087280273437500,
+0.0009460449218750, -0.0049438476562500, 0.0015563964843750, -0.0133361816406250,
+0.0007934570312500, -0.0130004882812500, 0.0007934570312500, -0.0121154785156250,
+0.0024108886718750, -0.0166320800781250, 0.0029602050781250, -0.0155639648437500,
+0.0025939941406250, -0.0126342773437500, 0.0063781738281250, -0.0153808593750000,
+0.0061035156250000, -0.0118408203125000, 0.0077209472656250, -0.0068664550781250,
+0.0109558105468750, -0.0107727050781250, 0.0097961425781250, -0.0034790039062500,
+0.0130920410156250, -0.0035095214843750, 0.0135803222656250, -0.0035705566406250,
+0.0133972167968750, 0.0001220703125000, 0.0147705078125000, -0.0013427734375000,
+0.0144042968750000, -0.0001525878906250, 0.0131835937500000, 0.0005187988281250,
+0.0136718750000000, -0.0002136230468750, 0.0124511718750000, -0.0003662109375000,
+0.0107116699218750, 0.0001220703125000, 0.0113525390625000, -0.0012207031250000,
+0.0089416503906250, -0.0011596679687500, 0.0088500976562500, -0.0012817382812500,
+0.0077514648437500, -0.0019531250000000, 0.0056762695312500, -0.0018005371093750,
+0.0062255859375000, -0.0025329589843750, 0.0038452148437500, -0.0021667480468750,
+0.0032653808593750, -0.0030822753906250, 0.0027770996093750, -0.0042114257812500,
+0.0013122558593750, -0.0032958984375000, 0.0012512207031250, -0.0054626464843750,
+0.0010681152343750, -0.0054931640625000, 0.0003356933593750, -0.0053405761718750,
+0.0002136230468750, -0.0063171386718750, 0.0002136230468750, -0.0060424804687500,
+-0.0003967285156250, -0.0061340332031250, -0.0001220703125000, -0.0060119628906250,
+-0.0006408691406250, -0.0060424804687500, -0.0006103515625000, -0.0061645507812500,
+-0.0003356933593750, -0.0059814453125000, -0.0011291503906250, -0.0059814453125000,
+-0.0005798339843750, -0.0061340332031250, -0.0012207031250000, -0.0059204101562500,
+-0.0012207031250000, -0.0057678222656250, -0.0009765625000000, -0.0059509277343750,
+-0.0019226074218750, -0.0054321289062500, -0.0009155273437500, -0.0056152343750000,
+-0.0020446777343750, -0.0052795410156250, -0.0017395019531250, -0.0047912597656250,
+-0.0005798339843750, -0.0051879882812500, -0.0021057128906250, -0.0040588378906250,
+-0.0004272460937500, -0.0041503906250000, -0.0009460449218750, -0.0040283203125000,
+-0.0007019042968750, -0.0032653808593750, 0.0003356933593750, -0.0036010742187500,
+-0.0006103515625000, -0.0038146972656250, 0.0021057128906250, -0.0035095214843750,
+0.0008850097656250, -0.0039672851562500, 0.0020446777343750, -0.0046081542968750,
+0.0044250488281250, -0.0043029785156250, 0.0023803710937500, -0.0048217773437500,
+0.0069580078125000, -0.0048828125000000, 0.0060729980468750, -0.0049133300781250,
+0.0065917968750000, -0.0052185058593750, 0.0098876953125000, -0.0049438476562500,
+0.0083007812500000, -0.0043640136718750, 0.0098571777343750, -0.0050048828125000,
+0.0107727050781250, -0.0034790039062500, 0.0091552734375000, -0.0018920898437500,
+0.0089416503906250, -0.0030822753906250, 0.0099792480468750, 0.0003356933593750,
+0.0045471191406250, 0.0006408691406250, 0.0068359375000000, 0.0000915527343750,
+0.0033569335937500, 0.0015258789062500, -0.0024414062500000, 0.0015563964843750,
+0.0017089843750000, -0.0006103515625000, -0.0072937011718750, 0.0003051757812500,
+-0.0064392089843750, -0.0013427734375000, -0.0072021484375000, -0.0037841796875000,
+-0.0133056640625000, -0.0021972656250000, -0.0105895996093750, -0.0074768066406250,
+-0.0111999511718750, -0.0067138671875000, -0.0134277343750000, -0.0079345703125000,
+-0.0100097656250000, -0.0122985839843750, -0.0074157714843750, -0.0104370117187500,
+-0.0097045898437500, -0.0108947753906250, -0.0004577636718750, -0.0127258300781250,
+-0.0022277832031250, -0.0100708007812500, 0.0006103515625000, -0.0078125000000000,
+0.0082397460937500, -0.0097656250000000, 0.0041503906250000, -0.0035095214843750,
+0.0095825195312500, -0.0045166015625000, 0.0110778808593750, -0.0023803710937500,
+0.0094604492187500, 0.0025329589843750, 0.0103149414062500, -0.0000610351562500,
+0.0105590820312500, 0.0061950683593750, 0.0090026855468750, 0.0066528320312500,
+0.0085449218750000, 0.0058898925781250, 0.0092773437500000, 0.0087890625000000,
+0.0091857910156250, 0.0085449218750000, 0.0086669921875000, 0.0051574707031250,
+0.0106811523437500, 0.0071411132812500, 0.0105285644531250, 0.0041503906250000,
+0.0105590820312500, -0.0011291503906250, 0.0114440917968750, 0.0018920898437500,
+0.0109863281250000, -0.0007934570312500, 0.0120544433593750, -0.0031738281250000,
+0.0124206542968750, -0.0002136230468750, 0.0117492675781250, 0.0014038085937500,
+0.0114440917968750, -0.0005798339843750, 0.0118408203125000, 0.0062561035156250,
+0.0119018554687500, 0.0062866210937500, 0.0110778808593750, 0.0049743652343750,
+0.0129394531250000, 0.0078430175781250, 0.0148620605468750, 0.0079345703125000,
+0.0132446289062500, 0.0055541992187500, 0.0158996582031250, 0.0057067871093750,
+0.0177001953125000, 0.0057983398437500, 0.0138854980468750, 0.0040283203125000,
+0.0126953125000000, 0.0042419433593750, 0.0150756835937500, 0.0070495605468750,
+0.0013732910156250, 0.0060119628906250, 0.0042419433593750, 0.0073242187500000,
+-0.0005493164062500, 0.0106506347656250, -0.0126953125000000, 0.0088806152343750,
+-0.0069274902343750, 0.0085144042968750, -0.0148010253906250, 0.0104675292968750,
+-0.0172119140625000, 0.0083007812500000, -0.0146179199218750, 0.0031738281250000,
+-0.0150756835937500, -0.0047302246093750, -0.0017089843750000, -0.0105590820312500,
+-0.0111694335937500, -0.0097351074218750, -0.0111389160156250, -0.0093383789062500,
+-0.0104370117187500, -0.0114135742187500, -0.0117492675781250, -0.0111694335937500,
+-0.0121154785156250, -0.0063476562500000, -0.0104064941406250, -0.0094909667968750,
+-0.0111083984375000, -0.0046081542968750, -0.0089721679687500, 0.0023193359375000,
+-0.0071105957031250, -0.0023803710937500, -0.0089111328125000, 0.0107116699218750,
+-0.0013427734375000, 0.0102539062500000, -0.0020446777343750, 0.0109252929687500,
+-0.0012817382812500, 0.0190124511718750, 0.0039062500000000, 0.0165100097656250,
+0.0023498535156250, 0.0162658691406250, 0.0023803710937500, 0.0188293457031250,
+0.0035095214843750, 0.0155334472656250, 0.0023498535156250, 0.0113830566406250,
+0.0008850097656250, 0.0142211914062500, 0.0019531250000000, 0.0108947753906250,
+0.0009155273437500, 0.0092773437500000, -0.0005493164062500, 0.0109863281250000,
+0.0022888183593750, 0.0113220214843750, 0.0046997070312500, 0.0104980468750000,
+0.0023498535156250, 0.0130615234375000, 0.0057678222656250, 0.0124511718750000,
+0.0072937011718750, 0.0133056640625000, 0.0057983398437500, 0.0154113769531250,
+0.0052490234375000, 0.0140686035156250, 0.0059509277343750, 0.0162963867187500,
+0.0043945312500000, 0.0163269042968750, 0.0040893554687500, 0.0169982910156250,
+0.0048217773437500, 0.0186767578125000, 0.0045471191406250, 0.0175476074218750,
+0.0041809082031250, 0.0183410644531250, 0.0060424804687500, 0.0200805664062500,
+0.0064392089843750, 0.0155334472656250, 0.0054321289062500, 0.0128479003906250,
+0.0056457519531250, 0.0162963867187500, 0.0061035156250000, 0.0031738281250000,
+0.0032653808593750, 0.0056152343750000, 0.0043640136718750, 0.0016479492187500,
+0.0017700195312500, -0.0086059570312500, -0.0013427734375000, -0.0030212402343750,
+0.0011291503906250, -0.0130920410156250, -0.0068969726562500, -0.0143127441406250,
+-0.0061645507812500, -0.0126037597656250, -0.0073242187500000, -0.0162658691406250,
+-0.0130004882812500, -0.0160522460937500, -0.0107116699218750, -0.0127868652343750,
+-0.0114746093750000, -0.0143127441406250, -0.0134582519531250, -0.0128479003906250,
+-0.0105590820312500, -0.0097351074218750, -0.0079650878906250, -0.0115661621093750,
+-0.0100402832031250, -0.0072631835937500, -0.0048828125000000, -0.0076293945312500,
+-0.0046691894531250, -0.0065307617187500, -0.0046997070312500, -0.0034790039062500,
+-0.0022583007812500, -0.0051269531250000, -0.0030822753906250, -0.0026550292968750,
+-0.0023193359375000, -0.0018310546875000, -0.0020141601562500, -0.0026550292968750,
+-0.0027770996093750, -0.0022583007812500, -0.0028381347656250, -0.0016174316406250,
+-0.0024719238281250, -0.0030822753906250, -0.0047302246093750, -0.0026245117187500,
+-0.0045471191406250, -0.0028991699218750, -0.0044555664062500, -0.0038452148437500,
+-0.0058288574218750, -0.0031127929687500, -0.0055847167968750, -0.0046386718750000,
+-0.0044860839843750, -0.0044860839843750, -0.0048828125000000, -0.0046081542968750,
+-0.0046081542968750, -0.0054321289062500, -0.0036010742187500, -0.0048522949218750,
+-0.0039062500000000},
+{0.0138244628906250, 0.0069274902343750, 0.0141906738281250, -0.0010986328125000,
+0.0054016113281250, -0.0014648437500000, 0.0083923339843750, -0.0019836425781250,
+0.0049743652343750, -0.0084228515625000, -0.0096130371093750, -0.0046386718750000,
+0.0143737792968750, 0.0008544921875000, 0.0062255859375000, -0.0033874511718750,
+0.0118713378906250, 0.0020446777343750, 0.0063781738281250, 0.0079650878906250,
+-0.0021362304687500, 0.0033264160156250, 0.0018920898437500, 0.0134582519531250,
+-0.0032958984375000, 0.0146179199218750, -0.0045471191406250, 0.0130310058593750,
+-0.0042114257812500, 0.0169067382812500, -0.0055847167968750, 0.0165710449218750,
+-0.0049133300781250, 0.0122375488281250, -0.0065917968750000, 0.0136718750000000,
+-0.0068969726562500, 0.0123291015625000, -0.0058898925781250, 0.0080261230468750,
+-0.0063781738281250, 0.0095825195312500, -0.0068359375000000, 0.0087585449218750,
+-0.0026855468750000, 0.0073242187500000, -0.0037536621093750, 0.0094299316406250,
+-0.0022888183593750, 0.0112915039062500, 0.0018615722656250, 0.0093078613281250,
+-0.0000305175781250, 0.0111999511718750, 0.0005493164062500, 0.0120849609375000,
+0.0024414062500000, 0.0119323730468750, 0.0002746582031250, 0.0118713378906250,
+-0.0030212402343750, 0.0116882324218750, -0.0012512207031250, 0.0129699707031250,
+0.0000305175781250, 0.0128479003906250, -0.0022888183593750, 0.0128784179687500,
+0.0002136230468750, 0.0137023925781250, 0.0043945312500000, 0.0131835937500000,
+0.0022888183593750, 0.0121459960937500, 0.0050964355468750, 0.0133361816406250,
+0.0063476562500000, 0.0099182128906250, 0.0043945312500000, 0.0076904296875000,
+0.0039062500000000, 0.0102844238281250, 0.0050354003906250, -0.0025634765625000,
+0.0017089843750000, 0.0001220703125000, 0.0013427734375000, -0.0026245117187500,
+0.0020141601562500, -0.0128173828125000, 0.0013122558593750, -0.0080261230468750,
+0.0013427734375000, -0.0121154785156250, 0.0011291503906250, -0.0145263671875000,
+0.0015869140625000, -0.0112915039062500, 0.0008544921875000, -0.0104370117187500,
+0.0001525878906250, -0.0125732421875000, 0.0006713867187500, -0.0056152343750000,
+-0.0018005371093750, -0.0060729980468750, -0.0007934570312500, -0.0063171386718750,
+-0.0025634765625000, -0.0023193359375000, -0.0054321289062500, -0.0032348632812500,
+-0.0036315917968750, -0.0046691894531250, -0.0087280273437500, -0.0035705566406250,
+-0.0085144042968750, -0.0047912597656250, -0.0088806152343750, -0.0068054199218750,
+-0.0122985839843750, -0.0054626464843750, -0.0112304687500000, -0.0073852539062500,
+-0.0100402832031250, -0.0075988769531250, -0.0115661621093750, -0.0073242187500000,
+-0.0096740722656250, -0.0080261230468750, -0.0068054199218750, -0.0077209472656250,
+-0.0084228515625000, -0.0068054199218750, -0.0052490234375000, -0.0073547363281250,
+-0.0051269531250000, -0.0058288574218750, -0.0046997070312500, -0.0043334960937500,
+-0.0029907226562500, -0.0057373046875000, -0.0038146972656250, -0.0020141601562500,
+-0.0021057128906250, -0.0024414062500000, -0.0018310546875000, -0.0016174316406250,
+-0.0024719238281250, 0.0009155273437500, -0.0020446777343750, -0.0004882812500000,
+-0.0018615722656250, 0.0019531250000000, -0.0035400390625000, 0.0020141601562500,
+-0.0033874511718750, 0.0021667480468750, -0.0036315917968750, 0.0034484863281250,
+-0.0047302246093750, 0.0027770996093750, -0.0044555664062500, 0.0035400390625000,
+-0.0048828125000000, 0.0036010742187500, -0.0048217773437500, 0.0036315917968750,
+-0.0050048828125000, 0.0040588378906250, -0.0052490234375000, 0.0038146972656250,
+-0.0049743652343750, 0.0037231445312500, -0.0050964355468750, 0.0039978027343750,
+-0.0052185058593750, 0.0036621093750000, -0.0049133300781250, 0.0032348632812500,
+-0.0045166015625000, 0.0033569335937500, -0.0046691894531250, 0.0027770996093750,
+-0.0047607421875000, 0.0031127929687500, -0.0042724609375000, 0.0026855468750000,
+-0.0047912597656250, 0.0019226074218750, -0.0055236816406250, 0.0022277832031250,
+-0.0049743652343750, 0.0012817382812500, -0.0058288574218750, 0.0014953613281250,
+-0.0059814453125000, 0.0010375976562500, -0.0058898925781250, 0.0000915527343750,
+-0.0061340332031250, 0.0003967285156250, -0.0061035156250000, -0.0007934570312500,
+-0.0061340332031250, -0.0007629394531250, -0.0061340332031250, -0.0008544921875000,
+-0.0059814453125000, -0.0016174316406250, -0.0058898925781250, -0.0015563964843750,
+-0.0059814453125000, -0.0016784667968750, -0.0058593750000000, -0.0017089843750000,
+-0.0057373046875000, -0.0016479492187500, -0.0059204101562500, -0.0015563964843750,
+-0.0061645507812500, -0.0015563964843750, -0.0059204101562500, -0.0015869140625000,
+-0.0054321289062500, -0.0015869140625000, -0.0061035156250000, -0.0013732910156250,
+-0.0050659179687500, -0.0012512207031250, -0.0038757324218750, -0.0014038085937500,
+-0.0049743652343750, -0.0014953613281250, -0.0025634765625000, -0.0016174316406250,
+-0.0026855468750000, -0.0019531250000000, -0.0026855468750000, -0.0021362304687500,
+-0.0010681152343750, -0.0020751953125000, -0.0017089843750000, -0.0029296875000000,
+-0.0031738281250000, -0.0028686523437500, -0.0018005371093750, -0.0031127929687500,
+-0.0039978027343750, -0.0037841796875000, -0.0069580078125000, -0.0035400390625000,
+-0.0047912597656250, -0.0035705566406250, -0.0088806152343750, -0.0042114257812500,
+-0.0092773437500000, -0.0035400390625000, -0.0085449218750000, -0.0028686523437500,
+-0.0101928710937500, -0.0034790039062500, -0.0100097656250000, -0.0010681152343750,
+-0.0078125000000000, -0.0016784667968750, -0.0090332031250000, -0.0000915527343750,
+-0.0075683593750000, 0.0024108886718750, -0.0046997070312500, 0.0008850097656250,
+-0.0065917968750000, 0.0051269531250000, -0.0047912597656250, 0.0049743652343750,
+-0.0040893554687500, 0.0058898925781250, -0.0050048828125000, 0.0087585449218750,
+-0.0051269531250000, 0.0074768066406250, -0.0049438476562500, 0.0097961425781250,
+-0.0059814453125000, 0.0103454589843750, -0.0058593750000000, 0.0093688964843750,
+-0.0054931640625000, 0.0097045898437500, -0.0060119628906250, 0.0099487304687500,
+-0.0061950683593750, 0.0074462890625000, -0.0048217773437500, 0.0076904296875000,
+-0.0050354003906250, 0.0066833496093750, -0.0045166015625000, 0.0046081542968750,
+-0.0038146972656250, 0.0053710937500000, -0.0046691894531250, 0.0033264160156250,
+-0.0023803710937500, 0.0032348632812500, -0.0024719238281250, 0.0033569335937500,
+-0.0025329589843750, 0.0022583007812500, -0.0013732910156250, 0.0023803710937500,
+-0.0018920898437500, 0.0023803710937500, -0.0016479492187500, 0.0023498535156250,
+-0.0013732910156250, 0.0019226074218750, -0.0013732910156250, 0.0017395019531250,
+-0.0018920898437500, 0.0018615722656250, -0.0018310546875000, 0.0002746582031250,
+0.0005798339843750, 0.0003051757812500, -0.0007324218750000, 0.0001525878906250,
+0.0010986328125000, -0.0011291503906250, 0.0046386718750000, -0.0010681152343750,
+0.0026855468750000, 0.0001220703125000, 0.0039672851562500, -0.0007629394531250,
+0.0052185058593750, 0.0008239746093750, 0.0046386718750000, 0.0029602050781250,
+0.0039367675781250, 0.0016174316406250, 0.0043640136718750, 0.0046997070312500,
+0.0047302246093750, 0.0052795410156250, 0.0036621093750000, 0.0044250488281250,
+0.0053710937500000, 0.0048522949218750, 0.0074768066406250, 0.0051574707031250,
+0.0058898925781250, 0.0057373046875000, 0.0078430175781250, 0.0046997070312500,
+0.0096130371093750, 0.0065612792968750, 0.0060424804687500, 0.0085754394531250,
+0.0040893554687500, 0.0070190429687500, 0.0070495605468750, 0.0113830566406250,
+-0.0003051757812500, 0.0115051269531250, -0.0006713867187500, 0.0109863281250000,
+0.0003356933593750, 0.0135498046875000, -0.0031127929687500, 0.0130615234375000,
+-0.0025024414062500, 0.0084838867187500, 0.0026550292968750, 0.0111389160156250,
+0.0003662109375000, 0.0093383789062500, 0.0016174316406250, 0.0048522949218750,
+0.0060729980468750, 0.0078125000000000, 0.0039367675781250, 0.0011596679687500,
+0.0086059570312500, 0.0017395019531250, 0.0083923339843750, 0.0010681152343750,
+0.0090332031250000, -0.0033874511718750, 0.0122985839843750, -0.0014038085937500,
+0.0109558105468750, -0.0023803710937500, 0.0107727050781250, -0.0040893554687500,
+0.0122070312500000, -0.0020446777343750, 0.0104675292968750, -0.0006408691406250,
+0.0082092285156250, -0.0023193359375000, 0.0095214843750000, 0.0031127929687500,
+0.0077209472656250, 0.0024719238281250, 0.0074157714843750, 0.0028686523437500,
+0.0072021484375000, 0.0063476562500000, 0.0065002441406250, 0.0051269531250000,
+0.0067749023437500, 0.0069274902343750, 0.0046691894531250, 0.0069885253906250,
+0.0055541992187500, 0.0066833496093750, 0.0032653808593750, 0.0078430175781250,
+0.0002746582031250, 0.0074768066406250, 0.0023193359375000, 0.0052795410156250,
+-0.0027160644531250, 0.0068969726562500, -0.0035705566406250, 0.0042419433593750,
+-0.0018615722656250, 0.0007629394531250, -0.0030822753906250, 0.0036010742187500,
+-0.0039672851562500, -0.0025634765625000, 0.0000000000000000, -0.0023193359375000,
+-0.0002136230468750, -0.0025329589843750, -0.0000915527343750, -0.0060424804687500,
+0.0019531250000000, -0.0045471191406250, 0.0013122558593750, -0.0053710937500000,
+0.0021972656250000, -0.0057983398437500, 0.0023803710937500, -0.0057373046875000,
+0.0028991699218750, -0.0058898925781250, 0.0031738281250000, -0.0056457519531250,
+0.0027770996093750, -0.0059814453125000, 0.0059814453125000, -0.0062561035156250,
+0.0049133300781250, -0.0055541992187500, 0.0069580078125000, -0.0055847167968750,
+0.0106811523437500, -0.0061950683593750, 0.0085144042968750, -0.0028381347656250,
+0.0115966796875000, -0.0033874511718750, 0.0122375488281250, -0.0034484863281250,
+0.0106811523437500, -0.0014038085937500, 0.0108642578125000, -0.0018615722656250,
+0.0109252929687500, -0.0013732910156250, 0.0071716308593750, -0.0010070800781250,
+0.0072631835937500, -0.0027160644531250, 0.0049438476562500, -0.0032348632812500,
+0.0013122558593750, -0.0017089843750000, 0.0023803710937500, -0.0066528320312500,
+-0.0018920898437500, -0.0061035156250000, -0.0021057128906250, -0.0067749023437500,
+-0.0026855468750000, -0.0097045898437500, -0.0050354003906250, -0.0080871582031250,
+-0.0042419433593750, -0.0105590820312500, -0.0052490234375000, -0.0110168457031250,
+-0.0057067871093750, -0.0095825195312500, -0.0052795410156250, -0.0100402832031250,
+-0.0051574707031250, -0.0107116699218750, -0.0050659179687500, -0.0068359375000000,
+-0.0046081542968750, -0.0079040527343750, -0.0043334960937500, -0.0066833496093750,
+-0.0042419433593750, -0.0034179687500000, -0.0040893554687500, -0.0052795410156250,
+-0.0038757324218750, -0.0025939941406250, -0.0040893554687500, -0.0023803710937500,
+-0.0039978027343750, -0.0026245117187500, -0.0039978027343750, -0.0015563964843750,
+-0.0040588378906250, -0.0019836425781250, -0.0037841796875000, -0.0013427734375000,
+-0.0041198730468750, -0.0013732910156250, -0.0037841796875000, -0.0013122558593750,
+-0.0038146972656250, -0.0007629394531250, -0.0039978027343750, -0.0009460449218750,
+-0.0034179687500000, -0.0014038085937500, -0.0042724609375000, -0.0009155273437500,
+-0.0036621093750000, -0.0018310546875000, -0.0033874511718750, -0.0028686523437500,
+-0.0039978027343750, -0.0021057128906250, -0.0035400390625000, -0.0041809082031250,
+-0.0035095214843750, -0.0041198730468750, -0.0030822753906250, -0.0042724609375000,
+-0.0032958984375000, -0.0054931640625000, -0.0034790039062500, -0.0051269531250000,
+-0.0027160644531250, -0.0058593750000000, -0.0039978027343750, -0.0059814453125000,
+-0.0036315917968750},
+{0.0076293945312500, 0.0027160644531250, 0.0075073242187500, 0.0010681152343750,
+0.0072937011718750, -0.0042419433593750, 0.0060729980468750, -0.0016784667968750,
+0.0066223144531250, -0.0043640136718750, 0.0036926269531250, -0.0063476562500000,
+0.0043945312500000, -0.0032653808593750, 0.0033569335937500, -0.0018615722656250,
+0.0006103515625000, -0.0042114257812500, 0.0017089843750000, 0.0021667480468750,
+-0.0006408691406250, 0.0016479492187500, -0.0010986328125000, 0.0020446777343750,
+-0.0006103515625000, 0.0057983398437500, -0.0015869140625000, 0.0041503906250000,
+-0.0018005371093750, 0.0051879882812500, 0.0006103515625000, 0.0057373046875000,
+-0.0001831054687500, 0.0050659179687500, -0.0000305175781250, 0.0052490234375000,
+0.0019836425781250, 0.0054321289062500, 0.0015563964843750, 0.0027160644531250,
+0.0016174316406250, 0.0039672851562500, 0.0011901855468750, 0.0024414062500000,
+0.0024414062500000, -0.0004882812500000, 0.0032653808593750, 0.0014038085937500,
+0.0022888183593750, -0.0026855468750000, 0.0065917968750000, -0.0023193359375000,
+0.0061035156250000, -0.0025634765625000, 0.0071105957031250, -0.0048522949218750,
+0.0105590820312500, -0.0037536621093750, 0.0091552734375000, -0.0052490234375000,
+0.0094909667968750, -0.0053100585937500, 0.0109252929687500, -0.0050964355468750,
+0.0082702636718750, -0.0057983398437500, 0.0057983398437500, -0.0055847167968750,
+0.0071716308593750, -0.0046691894531250, 0.0014953613281250, -0.0058288574218750,
+0.0011596679687500, -0.0041809082031250, 0.0003662109375000, -0.0018005371093750,
+-0.0029907226562500, -0.0034790039062500, -0.0021972656250000, -0.0011291503906250,
+-0.0035095214843750, -0.0006713867187500, -0.0038452148437500, -0.0009460449218750,
+-0.0037536621093750, 0.0002136230468750, -0.0039367675781250, -0.0002441406250000,
+-0.0034790039062500, -0.0029907226562500, -0.0036621093750000, -0.0013427734375000,
+-0.0032348632812500, -0.0029907226562500, -0.0030822753906250, -0.0067443847656250,
+-0.0032653808593750, -0.0050659179687500, -0.0028686523437500, -0.0075988769531250,
+-0.0030517578125000, -0.0076904296875000, -0.0029907226562500, -0.0080871582031250,
+-0.0025634765625000, -0.0095214843750000, -0.0023193359375000, -0.0084533691406250,
+-0.0021362304687500, -0.0092163085937500, -0.0015869140625000, -0.0098266601562500,
+-0.0014343261718750, -0.0085449218750000, -0.0006408691406250, -0.0079956054687500,
+0.0002441406250000, -0.0086975097656250, 0.0003051757812500, -0.0054016113281250,
+0.0021667480468750, -0.0061645507812500, 0.0020446777343750, -0.0051879882812500,
+0.0033569335937500, -0.0025634765625000, 0.0052795410156250, -0.0039672851562500,
+0.0045471191406250, -0.0021362304687500, 0.0068054199218750, -0.0018920898437500,
+0.0070800781250000, -0.0020141601562500, 0.0076904296875000, -0.0014343261718750,
+0.0091857910156250, -0.0017395019531250, 0.0089111328125000, -0.0011596679687500,
+0.0097351074218750, -0.0012207031250000, 0.0099792480468750, -0.0014343261718750,
+0.0103454589843750, -0.0010986328125000, 0.0108947753906250, -0.0010986328125000,
+0.0108032226562500, -0.0022583007812500, 0.0113830566406250, -0.0018005371093750,
+0.0115966796875000, -0.0023498535156250, 0.0114135742187500, -0.0035705566406250,
+0.0114440917968750, -0.0029602050781250, 0.0115661621093750, -0.0038452148437500,
+0.0112609863281250, -0.0039367675781250, 0.0113525390625000, -0.0041503906250000,
+0.0110778808593750, -0.0045776367187500, 0.0107421875000000, -0.0042114257812500,
+0.0111389160156250, -0.0050048828125000, 0.0100097656250000, -0.0050964355468750,
+0.0105590820312500, -0.0045166015625000, 0.0099792480468750, -0.0047302246093750,
+0.0088195800781250, -0.0048828125000000, 0.0095520019531250, -0.0032043457031250,
+0.0075378417968750, -0.0034790039062500, 0.0081481933593750, -0.0030212402343750,
+0.0071411132812500, -0.0016479492187500, 0.0054626464843750, -0.0021667480468750,
+0.0067138671875000, -0.0008850097656250, 0.0030212402343750, -0.0010070800781250,
+0.0037536621093750, -0.0007324218750000, 0.0031433105468750, 0.0002441406250000,
+0.0004882812500000, -0.0003051757812500, 0.0019531250000000, -0.0005187988281250,
+-0.0005493164062500, 0.0000305175781250, -0.0009155273437500, -0.0014953613281250,
+-0.0003356933593750, -0.0028381347656250, -0.0011291503906250, -0.0019226074218750,
+-0.0011901855468750, -0.0054321289062500, -0.0003051757812500, -0.0053405761718750,
+-0.0007324218750000, -0.0058898925781250, -0.0002441406250000, -0.0081481933593750,
+0.0007019042968750, -0.0072937011718750, 0.0001525878906250, -0.0083618164062500,
+0.0013427734375000, -0.0087890625000000, 0.0016479492187500, -0.0080566406250000,
+0.0010681152343750, -0.0080566406250000, 0.0014648437500000, -0.0082397460937500,
+0.0017700195312500, -0.0058288574218750, -0.0008239746093750, -0.0064697265625000,
+0.0003356933593750, -0.0051574707031250, -0.0011901855468750, -0.0025024414062500,
+-0.0039672851562500, -0.0037536621093750, -0.0021362304687500, -0.0016784667968750,
+-0.0060119628906250, -0.0011291503906250, -0.0062866210937500, -0.0015258789062500,
+-0.0056457519531250, -0.0010986328125000, -0.0076293945312500, -0.0009460449218750,
+-0.0075073242187500, -0.0009765625000000, -0.0040893554687500, -0.0010375976562500,
+-0.0058898925781250, -0.0010986328125000, -0.0028686523437500, -0.0008239746093750,
+0.0018005371093750, -0.0006408691406250, -0.0011596679687500, -0.0014648437500000,
+0.0045166015625000, -0.0012207031250000, 0.0053405761718750, -0.0012512207031250,
+0.0046997070312500, -0.0018615722656250, 0.0071716308593750, -0.0016479492187500,
+0.0068969726562500, -0.0025939941406250, 0.0037231445312500, -0.0027465820312500,
+0.0057983398437500, -0.0018005371093750, 0.0024108886718750, -0.0021362304687500,
+-0.0020446777343750, -0.0029296875000000, 0.0014953613281250, 0.0008850097656250,
+-0.0073242187500000, 0.0002136230468750, -0.0075378417968750, 0.0014953613281250,
+-0.0069885253906250, 0.0043640136718750, -0.0121154785156250, 0.0025634765625000,
+-0.0114135742187500, 0.0062255859375000, -0.0062866210937500, 0.0066833496093750,
+-0.0089111328125000, 0.0059204101562500, -0.0061035156250000, 0.0070190429687500,
+0.0000305175781250, 0.0068664550781250, -0.0025939941406250, 0.0049743652343750,
+0.0033874511718750, 0.0062255859375000, 0.0032348632812500, 0.0043945312500000,
+0.0037231445312500, 0.0016174316406250, 0.0075378417968750, 0.0034484863281250,
+0.0062561035156250, -0.0004577636718750, 0.0063476562500000, -0.0002136230468750,
+0.0071411132812500, -0.0001220703125000, 0.0071411132812500, -0.0025329589843750,
+0.0065917968750000, -0.0018920898437500, 0.0065612792968750, -0.0007934570312500,
+0.0074462890625000, -0.0018920898437500, 0.0078125000000000, -0.0003662109375000,
+0.0062561035156250, 0.0015563964843750, 0.0055541992187500, 0.0004577636718750,
+0.0068054199218750, 0.0050964355468750, 0.0033264160156250, 0.0038757324218750,
+0.0023803710937500, 0.0059509277343750, 0.0048217773437500, 0.0138244628906250,
+0.0065307617187500, 0.0168457031250000, 0.0081176757812500, 0.0107421875000000,
+0.0063781738281250, 0.0129699707031250, 0.0052185058593750, 0.0118103027343750,
+0.0073547363281250, 0.0079956054687500, 0.0087280273437500, 0.0090942382812500,
+0.0071105957031250, 0.0116577148437500, 0.0110778808593750, 0.0100402832031250,
+0.0118713378906250, 0.0109863281250000, 0.0097961425781250, 0.0143432617187500,
+0.0103454589843750, 0.0130004882812500, 0.0115661621093750, 0.0116577148437500,
+0.0062255859375000, 0.0134582519531250, 0.0069885253906250, 0.0109252929687500,
+0.0061035156250000, 0.0075988769531250, 0.0025024414062500, 0.0096740722656250,
+0.0040588378906250, 0.0043945312500000, 0.0017700195312500, 0.0041503906250000,
+0.0021972656250000, 0.0045776367187500, 0.0007934570312500, 0.0025634765625000,
+-0.0014038085937500, 0.0025634765625000, 0.0003356933593750, 0.0001220703125000,
+-0.0034179687500000, 0.0021362304687500, -0.0042114257812500, -0.0011291503906250,
+-0.0021667480468750, -0.0060729980468750, -0.0026245117187500, -0.0030212402343750,
+-0.0038757324218750, -0.0078430175781250, 0.0018005371093750, -0.0087890625000000,
+0.0019226074218750, -0.0079345703125000, 0.0015563964843750, -0.0092468261718750,
+0.0042114257812500, -0.0087890625000000, 0.0037536621093750, -0.0071105957031250,
+0.0022888183593750, -0.0080566406250000, 0.0029296875000000, -0.0067138671875000,
+0.0022277832031250, -0.0042114257812500, 0.0002136230468750, -0.0057678222656250,
+0.0009765625000000, -0.0050048828125000, 0.0021057128906250, -0.0034484863281250,
+0.0010986328125000, -0.0052795410156250, 0.0018005371093750, -0.0068359375000000,
+0.0038146972656250, -0.0052185058593750, 0.0031433105468750, -0.0088195800781250,
+0.0031127929687500, -0.0089721679687500, 0.0036315917968750, -0.0078430175781250,
+0.0031127929687500, -0.0094299316406250, 0.0025329589843750, -0.0093078613281250,
+0.0028686523437500, -0.0042114257812500, 0.0016174316406250, -0.0062255859375000,
+0.0020446777343750, -0.0036315917968750, 0.0010986328125000, 0.0019531250000000,
+-0.0000610351562500, -0.0010986328125000, 0.0007934570312500, 0.0032348632812500,
+-0.0026855468750000, 0.0040588378906250, -0.0020446777343750, 0.0032653808593750,
+-0.0029602050781250, 0.0045776367187500, -0.0058593750000000, 0.0042724609375000,
+-0.0046997070312500, 0.0028076171875000, -0.0062866210937500, 0.0030822753906250,
+-0.0065002441406250, 0.0019836425781250, -0.0067749023437500, 0.0003967285156250,
+-0.0075683593750000, 0.0012207031250000, -0.0069885253906250, -0.0010375976562500,
+-0.0079650878906250, -0.0018005371093750, -0.0083923339843750, -0.0004882812500000,
+-0.0075683593750000, -0.0008239746093750, -0.0073852539062500, -0.0018005371093750,
+-0.0080566406250000, 0.0023803710937500, -0.0060424804687500, 0.0010375976562500,
+-0.0062866210937500, 0.0025939941406250, -0.0060729980468750, 0.0061340332031250,
+-0.0048522949218750, 0.0037536621093750, -0.0052795410156250, 0.0081176757812500,
+-0.0042114257812500, 0.0079650878906250, -0.0043640136718750, 0.0081481933593750,
+-0.0040893554687500, 0.0106201171875000, -0.0030517578125000, 0.0094604492187500,
+-0.0034790039062500, 0.0106506347656250, -0.0033874511718750, 0.0109252929687500,
+-0.0028381347656250, 0.0104370117187500, -0.0036010742187500, 0.0105590820312500,
+-0.0043640136718750, 0.0107116699218750, -0.0037536621093750, 0.0098266601562500,
+-0.0052490234375000, 0.0103149414062500, -0.0053100585937500, 0.0096130371093750,
+-0.0053710937500000, 0.0086364746093750, -0.0061950683593750, 0.0095214843750000,
+-0.0059204101562500, 0.0075378417968750, -0.0061950683593750, 0.0077514648437500,
+-0.0063781738281250, 0.0073242187500000, -0.0062255859375000, 0.0061645507812500,
+-0.0061950683593750, 0.0072326660156250, -0.0063781738281250, 0.0056457519531250,
+-0.0059814453125000, 0.0061340332031250, -0.0059509277343750, 0.0059814453125000,
+-0.0059204101562500, 0.0050964355468750, -0.0056457519531250, 0.0061645507812500,
+-0.0057067871093750, 0.0048217773437500, -0.0057373046875000, 0.0054321289062500,
+-0.0057678222656250, 0.0054321289062500, -0.0054321289062500, 0.0045471191406250,
+-0.0053710937500000, 0.0055541992187500, -0.0056457519531250, 0.0048522949218750,
+-0.0039062500000000, 0.0052795410156250, -0.0043945312500000, 0.0052490234375000,
+-0.0037536621093750, 0.0049133300781250, -0.0021362304687500, 0.0057983398437500,
+-0.0030517578125000, 0.0051269531250000, -0.0019531250000000, 0.0053710937500000,
+-0.0014343261718750, 0.0052795410156250, -0.0024414062500000, 0.0048828125000000,
+-0.0025024414062500},
+{-0.0030822753906250, -0.0031738281250000, -0.0031127929687500, -0.0025939941406250,
+-0.0036621093750000, -0.0065002441406250, 0.0000000000000000, -0.0055236816406250,
+-0.0006713867187500, -0.0071716308593750, 0.0003051757812500, -0.0111694335937500,
+0.0034790039062500, -0.0091552734375000, 0.0021667480468750, -0.0101013183593750,
+0.0023803710937500, -0.0120544433593750, 0.0029907226562500, -0.0090026855468750,
+0.0030212402343750, -0.0068969726562500, 0.0025634765625000, -0.0092773437500000,
+0.0024108886718750, -0.0017700195312500, 0.0037536621093750, -0.0027160644531250,
+0.0036010742187500, -0.0014343261718750, 0.0036315917968750, 0.0040588378906250,
+0.0046691894531250, 0.0016174316406250, 0.0044860839843750, 0.0030517578125000,
+0.0037231445312500, 0.0046386718750000, 0.0043334960937500, 0.0030212402343750,
+0.0034790039062500, 0.0014953613281250, 0.0022277832031250, 0.0026245117187500,
+0.0030517578125000, 0.0014648437500000, 0.0009765625000000, 0.0005798339843750,
+0.0014038085937500, 0.0019226074218750, 0.0001525878906250, 0.0024719238281250,
+-0.0016479492187500, 0.0013427734375000, -0.0003662109375000, 0.0043334960937500,
+-0.0041809082031250, 0.0038146972656250, -0.0041198730468750, 0.0045776367187500,
+-0.0041809082031250, 0.0066223144531250, -0.0062561035156250, 0.0052490234375000,
+-0.0056152343750000, 0.0080871582031250, -0.0060729980468750, 0.0079650878906250,
+-0.0060729980468750, 0.0077209472656250, -0.0061340332031250, 0.0093994140625000,
+-0.0065307617187500, 0.0090942382812500, -0.0062561035156250, 0.0079040527343750,
+-0.0059814453125000, 0.0088195800781250, -0.0064392089843750, 0.0075683593750000,
+-0.0057373046875000, 0.0059509277343750, -0.0047607421875000, 0.0073852539062500,
+-0.0053710937500000, 0.0039367675781250, -0.0043945312500000, 0.0046386718750000,
+-0.0040283203125000, 0.0036621093750000, -0.0043334960937500, 0.0011901855468750,
+-0.0042419433593750, 0.0029907226562500, -0.0042114257812500, -0.0000610351562500,
+-0.0043334960937500, 0.0002136230468750, -0.0044555664062500, -0.0002136230468750,
+-0.0043945312500000, -0.0021667480468750, -0.0043334960937500, -0.0012207031250000,
+-0.0043334960937500, -0.0030212402343750, -0.0045471191406250, -0.0029907226562500,
+-0.0045471191406250, -0.0029296875000000, -0.0046081542968750, -0.0037841796875000,
+-0.0048217773437500, -0.0034179687500000, -0.0048828125000000, -0.0036621093750000,
+-0.0047302246093750, -0.0040893554687500, -0.0046997070312500, -0.0038757324218750,
+-0.0048217773437500, -0.0035400390625000, -0.0048217773437500, -0.0038146972656250,
+-0.0046997070312500, -0.0036010742187500, -0.0050354003906250, -0.0036315917968750,
+-0.0050659179687500, -0.0036621093750000, -0.0050964355468750, -0.0035705566406250,
+-0.0053710937500000, -0.0035400390625000, -0.0053100585937500, -0.0036926269531250,
+-0.0051269531250000, -0.0037231445312500, -0.0054321289062500, -0.0037536621093750,
+-0.0050354003906250, -0.0037841796875000, -0.0046386718750000, -0.0036621093750000,
+-0.0050354003906250, -0.0038146972656250, -0.0040283203125000, -0.0039367675781250,
+-0.0040893554687500, -0.0039367675781250, -0.0041809082031250, -0.0039367675781250,
+-0.0034179687500000, -0.0039062500000000, -0.0035705566406250, -0.0040588378906250,
+-0.0045776367187500, -0.0039978027343750, -0.0037231445312500, -0.0039672851562500,
+-0.0048522949218750, -0.0041198730468750, -0.0066223144531250, -0.0039978027343750,
+-0.0054016113281250, -0.0035705566406250, -0.0071105957031250, -0.0036010742187500,
+-0.0078735351562500, -0.0029296875000000, -0.0068664550781250, -0.0022277832031250,
+-0.0068664550781250, -0.0024719238281250, -0.0076904296875000, -0.0006103515625000,
+-0.0047912597656250, -0.0006713867187500, -0.0056457519531250, 0.0000610351562500,
+-0.0046386718750000, 0.0016784667968750, -0.0019836425781250, 0.0013122558593750,
+-0.0035095214843750, 0.0021667480468750, -0.0019226074218750, 0.0028076171875000,
+-0.0010681152343750, 0.0025939941406250, -0.0023193359375000, 0.0025939941406250,
+-0.0027160644531250, 0.0030822753906250, -0.0020141601562500, 0.0025024414062500,
+-0.0039367675781250, 0.0028381347656250, -0.0039978027343750, 0.0021362304687500,
+-0.0039367675781250, 0.0012512207031250, -0.0048217773437500, 0.0019531250000000,
+-0.0044250488281250, 0.0003356933593750, -0.0047302246093750, 0.0003967285156250,
+-0.0050659179687500, 0.0002746582031250, -0.0042114257812500, -0.0006408691406250,
+-0.0036010742187500, -0.0003356933593750, -0.0043334960937500, -0.0008850097656250,
+-0.0031127929687500, -0.0009460449218750, -0.0032348632812500, -0.0012512207031250,
+-0.0022888183593750, -0.0016784667968750, -0.0013732910156250, -0.0014648437500000,
+-0.0024414062500000, -0.0023803710937500, 0.0007019042968750, -0.0022277832031250,
+0.0001831054687500, -0.0025024414062500, 0.0011596679687500, -0.0033569335937500,
+0.0036926269531250, -0.0031433105468750, 0.0022277832031250, -0.0037536621093750,
+0.0035095214843750, -0.0039367675781250, 0.0043334960937500, -0.0041809082031250,
+0.0035095214843750, -0.0045776367187500, 0.0030212402343750, -0.0044860839843750,
+0.0035400390625000, -0.0047912597656250, 0.0030822753906250, -0.0051574707031250,
+0.0023193359375000, -0.0043640136718750, 0.0033569335937500, -0.0039672851562500,
+0.0044250488281250, -0.0047912597656250, 0.0035705566406250, -0.0024108886718750,
+0.0046081542968750, -0.0024108886718750, 0.0052490234375000, -0.0021972656250000,
+0.0035705566406250, -0.0008544921875000, 0.0030517578125000, -0.0011901855468750,
+0.0043945312500000, -0.0004882812500000, -0.0003356933593750, -0.0003356933593750,
+-0.0001220703125000, 0.0000000000000000, 0.0002136230468750, 0.0004882812500000,
+-0.0023498535156250, 0.0004577636718750, -0.0016174316406250, 0.0012817382812500,
+0.0008544921875000, 0.0012207031250000, -0.0007324218750000, 0.0014953613281250,
+0.0014038085937500, 0.0024719238281250, 0.0046081542968750, 0.0023498535156250,
+0.0022277832031250, 0.0016784667968750, 0.0076599121093750, 0.0024414062500000,
+0.0078430175781250, 0.0013427734375000, 0.0075073242187500, -0.0002746582031250,
+0.0098876953125000, 0.0009460449218750, 0.0092163085937500, 0.0003662109375000,
+0.0086669921875000, -0.0008850097656250, 0.0093994140625000, 0.0008544921875000,
+0.0085449218750000, 0.0029296875000000, 0.0067749023437500, 0.0028076171875000,
+0.0068664550781250, 0.0059204101562500, 0.0061950683593750, 0.0052185058593750,
+0.0063781738281250, 0.0051574707031250, 0.0059204101562500, 0.0068359375000000,
+0.0054626464843750, 0.0061645507812500, 0.0060119628906250, 0.0079040527343750,
+0.0029907226562500, 0.0074462890625000, 0.0036010742187500, 0.0075683593750000,
+0.0029296875000000, 0.0091857910156250, 0.0006408691406250, 0.0086364746093750,
+0.0015258789062500, 0.0075378417968750, -0.0002441406250000, 0.0081481933593750,
+-0.0006103515625000, 0.0078125000000000, 0.0000915527343750, 0.0066833496093750,
+-0.0001525878906250, 0.0070800781250000, -0.0004882812500000, 0.0061645507812500,
+0.0006408691406250, 0.0067443847656250, 0.0005493164062500, 0.0051269531250000,
+0.0005798339843750, 0.0040588378906250, 0.0009460449218750, 0.0056457519531250,
+0.0007934570312500, -0.0014038085937500, 0.0021057128906250, 0.0003051757812500,
+0.0011596679687500, -0.0011901855468750, 0.0027160644531250, -0.0072326660156250,
+0.0050964355468750, -0.0043945312500000, 0.0035400390625000, -0.0046997070312500,
+0.0062561035156250, -0.0070800781250000, 0.0069274902343750, -0.0042419433593750,
+0.0057373046875000, -0.0013122558593750, 0.0065917968750000, -0.0038757324218750,
+0.0068359375000000, 0.0006103515625000, 0.0011901855468750, 0.0004272460937500,
+0.0029296875000000, 0.0000610351562500, 0.0005798339843750, 0.0022277832031250,
+-0.0050354003906250, 0.0015258789062500, -0.0025024414062500, 0.0017089843750000,
+-0.0063171386718750, 0.0014038085937500, -0.0077819824218750, 0.0020751953125000,
+-0.0060119628906250, 0.0028991699218750, -0.0061035156250000, 0.0021972656250000,
+-0.0070800781250000, 0.0033874511718750, -0.0036926269531250, 0.0037536621093750,
+-0.0034484863281250, 0.0029602050781250, -0.0033569335937500, 0.0032043457031250,
+-0.0015258789062500, 0.0039367675781250, -0.0014343261718750, 0.0009765625000000,
+-0.0018005371093750, 0.0018310546875000, -0.0014038085937500, 0.0009460449218750,
+-0.0011291503906250, -0.0014038085937500, -0.0014648437500000, 0.0003356933593750,
+-0.0013427734375000, -0.0016479492187500, 0.0000305175781250, -0.0022888183593750,
+-0.0001831054687500, -0.0014038085937500, 0.0011596679687500, -0.0013732910156250,
+0.0028381347656250, -0.0016784667968750, 0.0018920898437500, -0.0010681152343750,
+0.0042114257812500, -0.0011596679687500, 0.0043029785156250, -0.0008850097656250,
+0.0042724609375000, 0.0000000000000000, 0.0054626464843750, -0.0004272460937500,
+0.0052185058593750, -0.0023803710937500, 0.0049133300781250, -0.0009155273437500,
+0.0054321289062500, -0.0024719238281250, 0.0046386718750000, -0.0054931640625000,
+0.0037231445312500, -0.0037536621093750, 0.0042724609375000, -0.0061950683593750,
+0.0022583007812500, -0.0065612792968750, 0.0023498535156250, -0.0061340332031250,
+0.0017700195312500, -0.0070190429687500, 0.0002136230468750, -0.0068969726562500,
+0.0008544921875000, -0.0060424804687500, 0.0001220703125000, -0.0063476562500000,
+-0.0000305175781250, -0.0061035156250000, 0.0003356933593750, -0.0051574707031250,
+0.0003662109375000, -0.0053710937500000, 0.0003051757812500, -0.0050048828125000,
+0.0010375976562500, -0.0050048828125000, 0.0010986328125000, -0.0046386718750000,
+0.0014038085937500, -0.0044250488281250, 0.0019531250000000, -0.0047912597656250,
+0.0017395019531250, -0.0036926269531250, 0.0025329589843750, -0.0039062500000000,
+0.0024108886718750, -0.0035095214843750, 0.0027465820312500, -0.0027465820312500,
+0.0034484863281250, -0.0032348632812500, 0.0032043457031250, -0.0016479492187500,
+0.0040893554687500, -0.0021667480468750, 0.0041198730468750, -0.0013732910156250,
+0.0044250488281250, 0.0002441406250000, 0.0051269531250000, -0.0007934570312500,
+0.0048522949218750, 0.0008239746093750, 0.0053405761718750, 0.0011596679687500,
+0.0054321289062500, 0.0006713867187500, 0.0054931640625000, 0.0009765625000000,
+0.0056762695312500, 0.0011596679687500, 0.0055847167968750, 0.0003356933593750,
+0.0056152343750000, 0.0005493164062500, 0.0056762695312500, 0.0003967285156250,
+0.0056762695312500, -0.0001831054687500, 0.0057983398437500, 0.0000915527343750,
+0.0058593750000000, -0.0008544921875000, 0.0055847167968750, -0.0005187988281250,
+0.0061340332031250, -0.0011291503906250, 0.0056457519531250, -0.0021362304687500,
+0.0050659179687500, -0.0015563964843750, 0.0058898925781250, -0.0032958984375000,
+0.0042419433593750, -0.0032043457031250, 0.0046081542968750, -0.0037536621093750,
+0.0045776367187500, -0.0051269531250000, 0.0034790039062500, -0.0046081542968750,
+0.0041809082031250, -0.0060119628906250, 0.0039672851562500, -0.0061340332031250,
+0.0037536621093750, -0.0062561035156250, 0.0044250488281250, -0.0070800781250000,
+0.0047912597656250, -0.0068969726562500, 0.0045166015625000, -0.0066223144531250,
+0.0061035156250000, -0.0070800781250000, 0.0057678222656250, -0.0062255859375000,
+0.0066528320312500, -0.0053100585937500, 0.0081176757812500, -0.0058593750000000,
+0.0074157714843750, -0.0035705566406250, 0.0097351074218750, -0.0039367675781250,
+0.0097961425781250, -0.0030822753906250, 0.0093078613281250, -0.0012817382812500,
+0.0105285644531250, -0.0022277832031250, 0.0106506347656250, 0.0000305175781250,
+0.0084838867187500},
+{-0.0002136230468750, 0.0040588378906250, 0.0012512207031250, 0.0051269531250000,
+0.0011291503906250, 0.0038757324218750, 0.0017395019531250, 0.0029296875000000,
+0.0032043457031250, 0.0038146972656250, 0.0027465820312500, 0.0008239746093750,
+0.0028686523437500, 0.0015563964843750, 0.0033569335937500, 0.0011596679687500,
+0.0025024414062500, -0.0011596679687500, 0.0016784667968750, -0.0002746582031250,
+0.0019531250000000, -0.0010375976562500, 0.0003356933593750, -0.0010681152343750,
+0.0003662109375000, -0.0008850097656250, -0.0000610351562500, -0.0013427734375000,
+-0.0011291503906250, -0.0011901855468750, -0.0005493164062500, -0.0003662109375000,
+-0.0013122558593750, -0.0002746582031250, -0.0013427734375000, -0.0011596679687500,
+-0.0014343261718750, -0.0010986328125000, -0.0018310546875000, -0.0003662109375000,
+-0.0014953613281250, -0.0034179687500000, -0.0017089843750000, -0.0028381347656250,
+-0.0013732910156250, -0.0035095214843750, -0.0012817382812500, -0.0057678222656250,
+-0.0013427734375000, -0.0045471191406250, -0.0008239746093750, -0.0063476562500000,
+-0.0009765625000000, -0.0067443847656250, -0.0007324218750000, -0.0058898925781250,
+-0.0009155273437500, -0.0061950683593750, -0.0012207031250000, -0.0065917968750000,
+-0.0007324218750000, -0.0047607421875000, -0.0014038085937500, -0.0050964355468750,
+-0.0015258789062500, -0.0047302246093750, -0.0015563964843750, -0.0034790039062500,
+-0.0021972656250000, -0.0040283203125000, -0.0025329589843750, -0.0031127929687500,
+-0.0019226074218750, -0.0031433105468750, -0.0024414062500000, -0.0028076171875000,
+-0.0022277832031250, -0.0021667480468750, -0.0016479492187500, -0.0024719238281250,
+-0.0022888183593750, -0.0014953613281250, -0.0014038085937500, -0.0016784667968750,
+-0.0019836425781250, -0.0010986328125000, -0.0017089843750000, -0.0003356933593750,
+-0.0008239746093750, -0.0008850097656250, -0.0016479492187500, 0.0008544921875000,
+-0.0003662109375000, 0.0007019042968750, -0.0005798339843750, 0.0008239746093750,
+-0.0002746582031250, 0.0020141601562500, 0.0005493164062500, 0.0015869140625000,
+0.0000000000000000, 0.0011901855468750, 0.0014343261718750, 0.0017395019531250,
+0.0009155273437500, 0.0010070800781250, 0.0016174316406250, 0.0000915527343750,
+0.0029602050781250, 0.0006713867187500, 0.0019531250000000, -0.0013427734375000,
+0.0042114257812500, -0.0009460449218750, 0.0039062500000000, -0.0019226074218750,
+0.0045471191406250, -0.0037536621093750, 0.0061950683593750, -0.0027160644531250,
+0.0051879882812500, -0.0053405761718750, 0.0072631835937500, -0.0053100585937500,
+0.0072326660156250, -0.0056762695312500, 0.0074157714843750, -0.0073242187500000,
+0.0086669921875000, -0.0066833496093750, 0.0082702636718750, -0.0074462890625000,
+0.0084533691406250, -0.0079040527343750, 0.0090942382812500, -0.0072326660156250,
+0.0087280273437500, -0.0069885253906250, 0.0083923339843750, -0.0072937011718750,
+0.0089721679687500, -0.0054931640625000, 0.0079956054687500, -0.0057678222656250,
+0.0083618164062500, -0.0049438476562500, 0.0077209472656250, -0.0035705566406250,
+0.0066528320312500, -0.0042724609375000, 0.0074768066406250, -0.0017089843750000,
+0.0057373046875000, -0.0020751953125000, 0.0055847167968750, -0.0015563964843750,
+0.0061340332031250, 0.0002746582031250, 0.0055236816406250, -0.0005798339843750,
+0.0053405761718750, 0.0007324218750000, 0.0067749023437500, 0.0006408691406250,
+0.0065002441406250, 0.0012512207031250, 0.0070190429687500, 0.0021667480468750,
+0.0084533691406250, 0.0015258789062500, 0.0079040527343750, 0.0039062500000000,
+0.0077514648437500, 0.0036926269531250, 0.0088806152343750, 0.0040588378906250,
+0.0072021484375000, 0.0058288574218750, 0.0052490234375000, 0.0052795410156250,
+0.0066833496093750, 0.0054626464843750, 0.0037231445312500, 0.0061645507812500,
+0.0031738281250000, 0.0051574707031250, 0.0034179687500000, 0.0040283203125000,
+0.0024108886718750, 0.0048828125000000, 0.0024108886718750, 0.0041809082031250,
+0.0027160644531250, 0.0034790039062500, 0.0025329589843750, 0.0040893554687500,
+0.0027160644531250, 0.0050659179687500, 0.0030822753906250, 0.0045471191406250,
+0.0027160644531250, 0.0029907226562500, 0.0024719238281250, 0.0042419433593750,
+0.0034790039062500, 0.0026245117187500, 0.0011291503906250, 0.0001831054687500,
+-0.0004882812500000, 0.0017089843750000, 0.0014648437500000, -0.0021972656250000,
+-0.0057678222656250, -0.0026855468750000, -0.0052185058593750, -0.0009765625000000,
+-0.0054931640625000, -0.0021667480468750, -0.0103454589843750, -0.0029907226562500,
+-0.0089416503906250, 0.0019226074218750, -0.0066833496093750, 0.0010070800781250,
+-0.0089416503906250, 0.0028076171875000, -0.0059509277343750, 0.0069274902343750,
+-0.0013122558593750, 0.0048828125000000, -0.0039367675781250, 0.0082702636718750,
+0.0008544921875000, 0.0086059570312500, 0.0009765625000000, 0.0087890625000000,
+0.0005187988281250, 0.0105895996093750, 0.0033874511718750, 0.0098876953125000,
+0.0025634765625000, 0.0110778808593750, -0.0025939941406250, 0.0116271972656250,
+-0.0001220703125000, 0.0105895996093750, -0.0026245117187500, 0.0107727050781250,
+-0.0092468261718750, 0.0117492675781250, -0.0064697265625000, 0.0075378417968750,
+-0.0063476562500000, 0.0079650878906250, -0.0097351074218750, 0.0080261230468750,
+-0.0048217773437500, 0.0050354003906250, 0.0003967285156250, 0.0058288574218750,
+-0.0038146972656250, 0.0096130371093750, 0.0042724609375000, 0.0078125000000000,
+0.0054321289062500, 0.0086975097656250, 0.0039672851562500, 0.0145874023437500,
+0.0072326660156250, 0.0073547363281250, -0.0020141601562500, 0.0034179687500000,
+0.0046081542968750, 0.0039367675781250, 0.0039978027343750, 0.0047607421875000,
+0.0032958984375000, 0.0041809082031250, 0.0042419433593750, 0.0035095214843750,
+0.0046997070312500, 0.0046997070312500, 0.0047302246093750, 0.0057983398437500,
+0.0033264160156250, 0.0034179687500000, 0.0054626464843750, 0.0017700195312500,
+0.0081787109375000, 0.0036315917968750, 0.0062866210937500, -0.0008239746093750,
+0.0085754394531250, -0.0018310546875000, 0.0101318359375000, -0.0005798339843750,
+0.0075378417968750, -0.0013122558593750, 0.0060729980468750, -0.0019226074218750,
+0.0079650878906250, 0.0001525878906250, 0.0041198730468750, 0.0002441406250000,
+0.0028991699218750, -0.0010681152343750, 0.0050048828125000, -0.0002441406250000,
+0.0052490234375000, 0.0004577636718750, 0.0039978027343750, -0.0054016113281250,
+0.0079040527343750, -0.0044250488281250, 0.0085144042968750, -0.0048828125000000,
+0.0071716308593750, -0.0090637207031250, 0.0081176757812500, -0.0077209472656250,
+0.0089111328125000, -0.0070495605468750, 0.0055541992187500, -0.0086059570312500,
+0.0058288574218750, -0.0066223144531250, 0.0056152343750000, -0.0043945312500000,
+0.0038757324218750, -0.0059814453125000, 0.0044860839843750, -0.0013732910156250,
+0.0028991699218750, -0.0014343261718750, 0.0037841796875000, -0.0015869140625000,
+0.0018920898437500, 0.0007934570312500, -0.0006103515625000, 0.0004272460937500,
+0.0014953613281250, -0.0001220703125000, -0.0021057128906250, 0.0001831054687500,
+-0.0032348632812500, -0.0002136230468750, -0.0012207031250000, -0.0010681152343750,
+-0.0011596679687500, -0.0006408691406250, -0.0022583007812500, 0.0000610351562500,
+0.0016784667968750, -0.0009765625000000, 0.0016784667968750, 0.0006408691406250,
+0.0017395019531250, 0.0026550292968750, 0.0038757324218750, 0.0010070800781250,
+0.0032958984375000, 0.0038452148437500, 0.0033874511718750, 0.0046691894531250,
+0.0035705566406250, 0.0033569335937500, 0.0036315917968750, 0.0035095214843750,
+0.0036010742187500, 0.0044250488281250, 0.0034179687500000, 0.0024108886718750,
+0.0041503906250000, 0.0021667480468750, 0.0043640136718750, 0.0028381347656250,
+0.0037231445312500, 0.0021057128906250, 0.0035705566406250, 0.0018920898437500,
+0.0039978027343750, 0.0045776367187500, 0.0025939941406250, 0.0036315917968750,
+0.0026245117187500, 0.0050048828125000, 0.0023498535156250, 0.0074768066406250,
+0.0018005371093750, 0.0058898925781250, 0.0021057128906250, 0.0098876953125000,
+0.0000000000000000, 0.0096435546875000, 0.0008239746093750, 0.0095520019531250,
+-0.0004882812500000, 0.0119934082031250, -0.0029602050781250, 0.0111999511718750,
+-0.0014953613281250, 0.0099182128906250, -0.0041503906250000, 0.0114135742187500,
+-0.0043640136718750, 0.0090332031250000, -0.0044860839843750, 0.0065002441406250,
+-0.0057373046875000, 0.0086669921875000, -0.0052185058593750, 0.0018615722656250,
+-0.0064392089843750, 0.0032653808593750, -0.0065917968750000, 0.0008544921875000,
+-0.0062866210937500, -0.0044555664062500, -0.0068969726562500, -0.0009155273437500,
+-0.0069885253906250, -0.0081787109375000, -0.0053710937500000, -0.0074157714843750,
+-0.0059204101562500, -0.0081176757812500, -0.0053405761718750, -0.0123901367187500,
+-0.0037841796875000, -0.0099792480468750, -0.0044250488281250, -0.0134582519531250,
+-0.0034484863281250, -0.0133056640625000, -0.0033874511718750, -0.0133972167968750,
+-0.0031738281250000, -0.0152893066406250, -0.0025939941406250, -0.0143737792968750,
+-0.0029602050781250, -0.0156860351562500, -0.0021057128906250, -0.0153808593750000,
+-0.0020751953125000, -0.0155944824218750, -0.0022583007812500, -0.0165710449218750,
+-0.0019836425781250, -0.0160522460937500, -0.0021057128906250, -0.0170288085937500,
+-0.0021972656250000, -0.0170898437500000, -0.0023193359375000, -0.0170288085937500,
+-0.0023193359375000, -0.0174560546875000, -0.0023498535156250, -0.0174255371093750,
+-0.0025329589843750, -0.0172424316406250, -0.0026550292968750, -0.0176391601562500,
+-0.0025329589843750, -0.0174255371093750, -0.0028686523437500, -0.0171203613281250,
+-0.0032043457031250, -0.0175781250000000, -0.0029907226562500, -0.0169677734375000,
+-0.0037231445312500, -0.0170898437500000, -0.0038452148437500, -0.0168762207031250,
+-0.0038757324218750, -0.0164489746093750, -0.0043029785156250, -0.0166625976562500,
+-0.0043945312500000, -0.0155639648437500, -0.0044860839843750, -0.0157775878906250,
+-0.0044860839843750, -0.0150451660156250, -0.0048522949218750, -0.0138549804687500,
+-0.0052795410156250, -0.0144348144531250, -0.0050354003906250, -0.0121154785156250,
+-0.0048522949218750, -0.0123901367187500, -0.0055847167968750, -0.0113525390625000,
+-0.0046081542968750, -0.0094604492187500, -0.0034484863281250, -0.0102539062500000,
+-0.0043945312500000, -0.0072937011718750, -0.0028076171875000, -0.0075378417968750,
+-0.0023803710937500, -0.0067138671875000, -0.0029602050781250, -0.0045471191406250,
+-0.0025939941406250, -0.0053405761718750, -0.0023803710937500, -0.0034179687500000,
+-0.0037231445312500, -0.0032043457031250, -0.0034179687500000, -0.0031433105468750,
+-0.0036621093750000, -0.0019531250000000, -0.0047302246093750, -0.0020141601562500,
+-0.0041809082031250, -0.0021972656250000, -0.0038452148437500, -0.0017395019531250,
+-0.0046081542968750, -0.0021057128906250, -0.0030822753906250, -0.0027770996093750,
+-0.0016174316406250, -0.0024414062500000, -0.0029602050781250, -0.0032348632812500,
+0.0004577636718750, -0.0032958984375000, 0.0002136230468750, -0.0034179687500000,
+0.0008544921875000, -0.0039367675781250, 0.0029602050781250, -0.0037841796875000,
+0.0017700195312500, -0.0034484863281250, 0.0036315917968750, -0.0040893554687500,
+0.0039978027343750, -0.0028076171875000, 0.0037536621093750, -0.0014038085937500,
+0.0043029785156250, -0.0024108886718750, 0.0041198730468750, 0.0002136230468750,
+0.0038757324218750, 0.0005187988281250, 0.0041198730468750, 0.0001525878906250,
+0.0038452148437500},
+{0.0025329589843750, 0.0000915527343750, 0.0019531250000000, -0.0065002441406250,
+0.0010375976562500, -0.0026550292968750, 0.0017700195312500, -0.0093383789062500,
+-0.0010070800781250, -0.0094604492187500, -0.0005493164062500, -0.0089416503906250,
+-0.0012512207031250, -0.0123291015625000, -0.0034790039062500, -0.0114440917968750,
+-0.0025939941406250, -0.0109558105468750, -0.0038452148437500, -0.0120239257812500,
+-0.0040588378906250, -0.0108642578125000, -0.0041809082031250, -0.0096130371093750,
+-0.0047302246093750, -0.0111389160156250, -0.0044860839843750, -0.0081176757812500,
+-0.0051879882812500, -0.0088195800781250, -0.0054016113281250, -0.0082702636718750,
+-0.0049743652343750, -0.0062561035156250, -0.0050964355468750, -0.0076599121093750,
+-0.0052795410156250, -0.0048828125000000, -0.0039062500000000, -0.0057373046875000,
+-0.0041198730468750, -0.0051574707031250, -0.0038452148437500, -0.0031433105468750,
+-0.0029296875000000, -0.0045776367187500, -0.0033264160156250, -0.0020446777343750,
+-0.0027770996093750, -0.0022888183593750, -0.0028381347656250, -0.0020446777343750,
+-0.0026855468750000, -0.0005798339843750, -0.0023193359375000, -0.0017395019531250,
+-0.0026245117187500, -0.0003051757812500, -0.0020141601562500, -0.0005798339843750,
+-0.0020141601562500, -0.0006713867187500, -0.0020446777343750, -0.0000610351562500,
+-0.0018310546875000, -0.0007019042968750, -0.0019531250000000, -0.0000305175781250,
+-0.0019836425781250, -0.0003967285156250, -0.0020141601562500, -0.0002441406250000,
+-0.0021362304687500, 0.0001525878906250, -0.0021972656250000, -0.0007324218750000,
+-0.0023193359375000, 0.0001525878906250, -0.0025329589843750, -0.0003051757812500,
+-0.0025329589843750, -0.0003967285156250, -0.0029602050781250, 0.0000305175781250,
+-0.0034179687500000, -0.0007934570312500, -0.0031738281250000, -0.0004882812500000,
+-0.0038757324218750, -0.0005493164062500, -0.0042114257812500, -0.0013122558593750,
+-0.0035095214843750, -0.0015563964843750, -0.0033874511718750, -0.0016174316406250,
+-0.0039978027343750, -0.0033264160156250, -0.0020751953125000, -0.0031738281250000,
+-0.0021667480468750, -0.0041809082031250, -0.0023803710937500, -0.0058288574218750,
+-0.0013427734375000, -0.0053710937500000, -0.0015869140625000, -0.0077819824218750,
+-0.0027160644531250, -0.0078735351562500, -0.0021057128906250, -0.0082397460937500,
+-0.0028381347656250, -0.0099182128906250, -0.0043334960937500, -0.0098876953125000,
+-0.0033569335937500, -0.0097351074218750, -0.0040588378906250, -0.0105590820312500,
+-0.0047607421875000, -0.0097961425781250, -0.0035095214843750, -0.0089416503906250,
+-0.0028076171875000, -0.0098571777343750, -0.0036315917968750, -0.0073547363281250,
+-0.0001831054687500, -0.0076599121093750, -0.0007934570312500, -0.0069274902343750,
+0.0000000000000000, -0.0049438476562500, 0.0025634765625000, -0.0058288574218750,
+0.0013732910156250, -0.0040283203125000, 0.0028076171875000, -0.0039672851562500,
+0.0030822753906250, -0.0038452148437500, 0.0027770996093750, -0.0028381347656250,
+0.0031738281250000, -0.0031127929687500, 0.0032653808593750, -0.0033264160156250,
+0.0030212402343750, -0.0031127929687500, 0.0029602050781250, -0.0032958984375000,
+0.0030822753906250, -0.0038452148437500, 0.0032653808593750, -0.0036926269531250,
+0.0030212402343750, -0.0028076171875000, 0.0026245117187500, -0.0033874511718750,
+0.0030212402343750, -0.0024719238281250, 0.0025024414062500, -0.0009460449218750,
+0.0016479492187500, -0.0016479492187500, 0.0022277832031250, -0.0000610351562500,
+0.0013122558593750, 0.0001220703125000, 0.0013732910156250, 0.0000915527343750,
+0.0010070800781250, 0.0010986328125000, 0.0001525878906250, 0.0010375976562500,
+0.0008544921875000, -0.0005493164062500, 0.0014038085937500, 0.0002441406250000,
+0.0003967285156250, -0.0007629394531250, 0.0018310546875000, -0.0030212402343750,
+0.0035400390625000, -0.0018920898437500, 0.0023803710937500, -0.0021667480468750,
+0.0059814453125000, -0.0029907226562500, 0.0055847167968750, -0.0018920898437500,
+0.0061645507812500, -0.0005798339843750, 0.0086059570312500, -0.0014343261718750,
+0.0075378417968750, -0.0004577636718750, 0.0088806152343750, 0.0002136230468750,
+0.0093383789062500, -0.0009765625000000, 0.0086364746093750, -0.0014343261718750,
+0.0087890625000000, -0.0008544921875000, 0.0090026855468750, -0.0048522949218750,
+0.0064697265625000, -0.0043945312500000, 0.0074768066406250, -0.0046081542968750,
+0.0066528320312500, -0.0075988769531250, 0.0041198730468750, -0.0069274902343750,
+0.0053100585937500, -0.0054626464843750, 0.0032043457031250, -0.0066223144531250,
+0.0033569335937500, -0.0053710937500000, 0.0030822753906250, -0.0030517578125000,
+0.0018005371093750, -0.0041503906250000, 0.0022888183593750, -0.0022277832031250,
+0.0005493164062500, -0.0021362304687500, 0.0010375976562500, -0.0023193359375000,
+0.0003356933593750, -0.0015563964843750, -0.0018920898437500, -0.0018920898437500,
+-0.0014038085937500, -0.0018005371093750, -0.0005798339843750, -0.0021362304687500,
+-0.0014953613281250, -0.0019531250000000, -0.0010986328125000, -0.0014038085937500,
+0.0002441406250000, -0.0016479492187500, -0.0001525878906250, -0.0013122558593750,
+0.0005798339843750, -0.0016174316406250, 0.0002441406250000, -0.0006713867187500,
+0.0009155273437500, 0.0000000000000000, 0.0020141601562500, -0.0009155273437500,
+0.0012512207031250, 0.0018920898437500, 0.0022277832031250, 0.0013427734375000,
+0.0030212402343750, 0.0021057128906250, 0.0011291503906250, 0.0043640136718750,
+0.0000915527343750, 0.0030212402343750, 0.0014038085937500, 0.0042724609375000,
+-0.0030212402343750, 0.0045166015625000, -0.0030517578125000, 0.0048522949218750,
+-0.0029296875000000, 0.0052795410156250, -0.0052795410156250, 0.0047607421875000,
+-0.0046997070312500, 0.0065002441406250, -0.0031738281250000, 0.0063171386718750,
+-0.0041503906250000, 0.0063476562500000, -0.0032958984375000, 0.0073547363281250,
+-0.0012817382812500, 0.0070190429687500, -0.0021362304687500, 0.0072937011718750,
+-0.0013122558593750, 0.0072937011718750, -0.0010681152343750, 0.0072631835937500,
+-0.0008850097656250, 0.0075683593750000, -0.0004882812500000, 0.0074768066406250,
+-0.0007934570312500, 0.0070800781250000, 0.0009155273437500, 0.0079345703125000,
+0.0008850097656250, 0.0065307617187500, 0.0012207031250000, 0.0054626464843750,
+0.0024108886718750, 0.0069885253906250, 0.0022277832031250, 0.0015258789062500,
+0.0028991699218750, 0.0027160644531250, 0.0032958984375000, 0.0018920898437500,
+0.0027465820312500, -0.0021972656250000, 0.0028991699218750, -0.0004272460937500,
+0.0034484863281250, -0.0025329589843750, 0.0004272460937500, -0.0032653808593750,
+0.0012207031250000, -0.0018005371093750, 0.0002746582031250, -0.0019836425781250,
+-0.0025024414062500, -0.0030517578125000, -0.0014038085937500, 0.0001525878906250,
+-0.0028381347656250, -0.0007629394531250, -0.0032348632812500, 0.0003051757812500,
+-0.0029296875000000, 0.0030517578125000, -0.0029602050781250, 0.0011596679687500,
+-0.0028076171875000, 0.0036315917968750, -0.0036010742187500, 0.0039367675781250,
+-0.0026550292968750, 0.0034484863281250, -0.0040893554687500, 0.0043945312500000,
+-0.0057067871093750, 0.0041503906250000, -0.0041503906250000, 0.0028381347656250,
+-0.0078125000000000, 0.0036621093750000, -0.0073242187500000, 0.0029602050781250,
+-0.0078430175781250, 0.0014038085937500, -0.0101318359375000, 0.0021057128906250,
+-0.0087890625000000, 0.0004577636718750, -0.0106506347656250, 0.0008239746093750,
+-0.0104370117187500, 0.0005798339843750, -0.0101623535156250, -0.0006103515625000,
+-0.0109558105468750, 0.0000305175781250, -0.0101928710937500, -0.0006408691406250,
+-0.0099182128906250, -0.0003662109375000, -0.0099792480468750, -0.0010070800781250,
+-0.0092163085937500, -0.0019531250000000, -0.0086059570312500, -0.0010070800781250,
+-0.0087585449218750, -0.0023193359375000, -0.0074157714843750, -0.0025939941406250,
+-0.0074768066406250, -0.0023193359375000, -0.0068664550781250, -0.0025329589843750,
+-0.0057373046875000, -0.0024719238281250, -0.0061340332031250, -0.0028686523437500,
+-0.0049743652343750, -0.0026855468750000, -0.0048828125000000, -0.0025024414062500,
+-0.0049743652343750, -0.0028991699218750, -0.0042724609375000, -0.0028076171875000,
+-0.0043945312500000, -0.0018310546875000, -0.0047302246093750, -0.0021667480468750,
+-0.0044250488281250, -0.0017700195312500, -0.0046081542968750, -0.0007629394531250,
+-0.0049743652343750, -0.0012512207031250, -0.0045166015625000, -0.0008239746093750,
+-0.0050354003906250, -0.0006103515625000, -0.0048522949218750, -0.0009765625000000,
+-0.0048522949218750, -0.0010681152343750, -0.0050354003906250, -0.0008239746093750,
+-0.0043945312500000, -0.0020751953125000, -0.0046691894531250, -0.0017700195312500,
+-0.0043640136718750, -0.0022277832031250, -0.0042419433593750, -0.0033874511718750,
+-0.0043640136718750, -0.0028381347656250, -0.0038146972656250, -0.0035705566406250,
+-0.0043945312500000, -0.0038452148437500, -0.0037231445312500, -0.0036621093750000,
+-0.0035095214843750, -0.0037536621093750, -0.0038757324218750, -0.0038757324218750,
+-0.0032653808593750, -0.0040588378906250, -0.0031738281250000, -0.0039978027343750,
+-0.0032653808593750, -0.0039367675781250, -0.0028381347656250, -0.0040893554687500,
+-0.0024414062500000, -0.0040893554687500, -0.0027465820312500, -0.0032043457031250,
+-0.0024108886718750, -0.0033264160156250, -0.0021667480468750, -0.0030822753906250,
+-0.0025329589843750, -0.0022277832031250, -0.0025634765625000, -0.0022888183593750,
+-0.0022583007812500, -0.0021972656250000, -0.0037841796875000, -0.0021057128906250,
+-0.0033874511718750, -0.0016784667968750, -0.0039672851562500, -0.0014953613281250,
+-0.0052795410156250, -0.0017395019531250, -0.0046997070312500, -0.0006103515625000,
+-0.0059204101562500, -0.0007324218750000, -0.0063171386718750, -0.0004577636718750,
+-0.0058593750000000, 0.0001525878906250, -0.0063781738281250, -0.0000915527343750,
+-0.0067138671875000, 0.0010986328125000, -0.0043945312500000, 0.0007324218750000,
+-0.0058288574218750, 0.0016479492187500, -0.0035400390625000, 0.0030517578125000,
+-0.0004882812500000, 0.0021057128906250, -0.0029296875000000, 0.0039062500000000,
+0.0032348632812500, 0.0042419433593750, 0.0023498535156250, 0.0037231445312500,
+0.0037231445312500, 0.0042114257812500, 0.0083618164062500, 0.0043029785156250,
+0.0059814453125000, 0.0032043457031250, 0.0090026855468750, 0.0034179687500000,
+0.0097961425781250, 0.0029296875000000, 0.0090026855468750, 0.0019226074218750,
+0.0095520019531250, 0.0023803710937500, 0.0095825195312500, 0.0013427734375000,
+0.0086669921875000, 0.0010681152343750, 0.0090637207031250, 0.0010375976562500,
+0.0080871582031250, 0.0005187988281250, 0.0068664550781250, 0.0005187988281250,
+0.0075378417968750, 0.0006408691406250, 0.0052795410156250, 0.0000610351562500,
+0.0052490234375000, 0.0011596679687500, 0.0053405761718750, 0.0021667480468750,
+0.0043029785156250, 0.0012207031250000, 0.0044860839843750, 0.0036926269531250,
+0.0040588378906250, 0.0036621093750000, 0.0042114257812500, 0.0044250488281250,
+0.0032653808593750, 0.0059814453125000, 0.0024108886718750, 0.0050354003906250,
+0.0029296875000000, 0.0075378417968750, 0.0010070800781250, 0.0077209472656250,
+0.0008239746093750, 0.0078125000000000, 0.0007629394531250, 0.0090026855468750,
+-0.0001525878906250, 0.0085449218750000, 0.0000000000000000, 0.0085754394531250,
+-0.0003356933593750, 0.0090942382812500, -0.0003662109375000, 0.0083923339843750,
+-0.0004272460937500, 0.0075378417968750, -0.0006103515625000, 0.0079650878906250,
+-0.0002746582031250},
+{-0.0030517578125000, -0.0020751953125000, -0.0026550292968750, -0.0030212402343750,
+-0.0029296875000000, -0.0030822753906250, -0.0027160644531250, -0.0028686523437500,
+-0.0022277832031250, -0.0031433105468750, -0.0022277832031250, -0.0032653808593750,
+-0.0021972656250000, -0.0026855468750000, -0.0014038085937500, -0.0028076171875000,
+-0.0014953613281250, -0.0028076171875000, -0.0012512207031250, -0.0024108886718750,
+-0.0007019042968750, -0.0025024414062500, -0.0009460449218750, -0.0028991699218750,
+-0.0007019042968750, -0.0026550292968750, -0.0007324218750000, -0.0030517578125000,
+-0.0008544921875000, -0.0035705566406250, -0.0009155273437500, -0.0031433105468750,
+-0.0009765625000000, -0.0043334960937500, -0.0007629394531250, -0.0043029785156250,
+-0.0008850097656250, -0.0040283203125000, -0.0012512207031250, -0.0046386718750000,
+-0.0013732910156250, -0.0046081542968750, -0.0014343261718750, -0.0033874511718750,
+-0.0019531250000000, -0.0038757324218750, -0.0019531250000000, -0.0032348632812500,
+-0.0021972656250000, -0.0018310546875000, -0.0025634765625000, -0.0023803710937500,
+-0.0022888183593750, -0.0013732910156250, -0.0023803710937500, -0.0011291503906250,
+-0.0025939941406250, -0.0010375976562500, -0.0023193359375000, -0.0005798339843750,
+-0.0020751953125000, -0.0006713867187500, -0.0021972656250000, -0.0002441406250000,
+-0.0012207031250000, -0.0003356933593750, -0.0014648437500000, 0.0000915527343750,
+-0.0012512207031250, 0.0005187988281250, -0.0003356933593750, 0.0003356933593750,
+-0.0006408691406250, 0.0015258789062500, -0.0003051757812500, 0.0013122558593750,
+0.0003356933593750, 0.0018005371093750, -0.0004272460937500, 0.0029296875000000,
+-0.0006713867187500, 0.0024414062500000, 0.0001525878906250, 0.0033874511718750,
+-0.0027770996093750, 0.0038452148437500, -0.0017089843750000, 0.0031433105468750,
+-0.0027465820312500, 0.0031433105468750, -0.0056152343750000, 0.0036315917968750,
+-0.0040893554687500, 0.0015563964843750, -0.0064086914062500, 0.0018920898437500,
+-0.0068969726562500, 0.0012817382812500, -0.0060424804687500, -0.0003967285156250,
+-0.0069274902343750, 0.0003356933593750, -0.0071716308593750, -0.0011596679687500,
+-0.0036621093750000, -0.0014648437500000, -0.0050354003906250, -0.0012512207031250,
+-0.0031433105468750, -0.0019531250000000, 0.0007019042968750, -0.0020446777343750,
+-0.0012207031250000, -0.0007324218750000, 0.0038757324218750, -0.0015563964843750,
+0.0035400390625000, -0.0003356933593750, 0.0048522949218750, 0.0014953613281250,
+0.0085449218750000, 0.0003662109375000, 0.0066528320312500, 0.0037841796875000,
+0.0094299316406250, 0.0036621093750000, 0.0102539062500000, 0.0036621093750000,
+0.0090637207031250, 0.0055236816406250, 0.0093383789062500, 0.0052490234375000,
+0.0098876953125000, 0.0058593750000000, 0.0078125000000000, 0.0056152343750000,
+0.0079040527343750, 0.0064086914062500, 0.0077819824218750, 0.0074462890625000,
+0.0068359375000000, 0.0067749023437500, 0.0072326660156250, 0.0081176757812500,
+0.0058593750000000, 0.0081176757812500, 0.0063476562500000, 0.0080566406250000,
+0.0050354003906250, 0.0086669921875000, 0.0031127929687500, 0.0085144042968750,
+0.0040283203125000, 0.0084838867187500, 0.0007934570312500, 0.0082397460937500,
+0.0008850097656250, 0.0084838867187500, 0.0006408691406250, 0.0087280273437500,
+-0.0013427734375000, 0.0083618164062500, -0.0006713867187500, 0.0096435546875000,
+-0.0011596679687500, 0.0098876953125000, -0.0016784667968750, 0.0083618164062500,
+-0.0004577636718750, 0.0081787109375000, 0.0001525878906250, 0.0093994140625000,
+-0.0009460449218750, 0.0053405761718750, 0.0010070800781250, 0.0056762695312500,
+0.0016174316406250, 0.0050354003906250, 0.0008544921875000, 0.0021667480468750,
+0.0010681152343750, 0.0033569335937500, 0.0012207031250000, 0.0036926269531250,
+-0.0009765625000000, 0.0025634765625000, -0.0003662109375000, 0.0035705566406250,
+-0.0006713867187500, 0.0054931640625000, -0.0029296875000000, 0.0044555664062500,
+-0.0023498535156250, 0.0035095214843750, -0.0004882812500000, 0.0050354003906250,
+-0.0015869140625000, 0.0031738281250000, -0.0004577636718750, -0.0016784667968750,
+0.0035095214843750, -0.0051574707031250, -0.0024414062500000, -0.0020141601562500,
+-0.0002441406250000, -0.0032348632812500, -0.0026550292968750, -0.0028381347656250,
+-0.0010681152343750, -0.0012817382812500, 0.0026245117187500, -0.0018005371093750,
+0.0015563964843750, -0.0016784667968750, -0.0002441406250000, -0.0015869140625000,
+0.0009460449218750, -0.0014038085937500, 0.0000000000000000, -0.0012207031250000,
+-0.0027465820312500, -0.0012207031250000, -0.0018615722656250, -0.0004577636718750,
+-0.0005493164062500, 0.0002441406250000, -0.0018920898437500, -0.0014648437500000,
+-0.0004272460937500, -0.0022583007812500, 0.0022277832031250, -0.0006713867187500,
+0.0008850097656250, -0.0053405761718750, 0.0022583007812500, -0.0054931640625000,
+0.0026855468750000, -0.0052185058593750, 0.0028381347656250, -0.0080566406250000,
+0.0029602050781250, -0.0073547363281250, 0.0025634765625000, -0.0027160644531250,
+0.0050659179687500, -0.0056457519531250, 0.0044860839843750, -0.0023498535156250,
+0.0048522949218750, 0.0042724609375000, 0.0070800781250000, 0.0007629394531250,
+0.0063781738281250, 0.0054626464843750, 0.0058288574218750, 0.0067749023437500,
+0.0065307617187500, 0.0056152343750000, 0.0057373046875000, 0.0064697265625000,
+0.0044860839843750, 0.0067138671875000, 0.0051574707031250, 0.0053100585937500,
+0.0042419433593750, 0.0052185058593750, 0.0037536621093750, 0.0059814453125000,
+0.0045471191406250, 0.0054321289062500, 0.0050048828125000, 0.0048828125000000,
+0.0045166015625000, 0.0081787109375000, 0.0052490234375000, 0.0075378417968750,
+0.0057067871093750, 0.0077514648437500, 0.0051269531250000, 0.0103149414062500,
+0.0047302246093750, 0.0092163085937500, 0.0050354003906250, 0.0072021484375000,
+0.0044555664062500, 0.0094604492187500, 0.0044250488281250, 0.0058593750000000,
+0.0043640136718750, 0.0018005371093750, 0.0043640136718750, 0.0053100585937500,
+0.0043945312500000, -0.0041198730468750, 0.0032653808593750, -0.0035095214843750,
+0.0037231445312500, -0.0034179687500000, 0.0027770996093750, -0.0091857910156250,
+0.0010986328125000, -0.0076904296875000, 0.0020446777343750, -0.0063476562500000,
+0.0009460449218750, -0.0086364746093750, 0.0003356933593750, -0.0061340332031250,
+0.0010070800781250, -0.0029296875000000, 0.0012817382812500, -0.0055236816406250,
+0.0009155273437500, 0.0001220703125000, 0.0018615722656250, -0.0005187988281250,
+0.0017700195312500, 0.0001220703125000, 0.0021667480468750, 0.0038146972656250,
+0.0029296875000000, 0.0018310546875000, 0.0024414062500000, 0.0034484863281250,
+0.0037841796875000, 0.0043334960937500, 0.0039062500000000, 0.0027160644531250,
+0.0036010742187500, 0.0022583007812500, 0.0040893554687500, 0.0030822753906250,
+0.0042114257812500, -0.0000610351562500, 0.0031738281250000, 0.0003967285156250,
+0.0033874511718750, -0.0002136230468750, 0.0032043457031250, -0.0025024414062500,
+0.0023803710937500, -0.0013732910156250, 0.0027160644531250, -0.0028686523437500,
+0.0018005371093750, -0.0033569335937500, 0.0021362304687500, -0.0030212402343750,
+0.0013732910156250, -0.0035095214843750, 0.0002441406250000, -0.0038146972656250,
+0.0010681152343750, -0.0027160644531250, -0.0008544921875000, -0.0035095214843750,
+-0.0009460449218750, -0.0028686523437500, -0.0011291503906250, -0.0017089843750000,
+-0.0021362304687500, -0.0029907226562500, -0.0017700195312500, -0.0007324218750000,
+-0.0027160644531250, -0.0012817382812500, -0.0026245117187500, -0.0009460449218750,
+-0.0028381347656250, 0.0007324218750000, -0.0036621093750000, -0.0003662109375000,
+-0.0033569335937500, 0.0009765625000000, -0.0032653808593750, 0.0007324218750000,
+-0.0036926269531250, 0.0007629394531250, -0.0030822753906250, 0.0014038085937500,
+-0.0023193359375000, 0.0007324218750000, -0.0028991699218750, 0.0015258789062500,
+-0.0018005371093750, 0.0012207031250000, -0.0016784667968750, 0.0014953613281250,
+-0.0018615722656250, 0.0021667480468750, -0.0015258789062500, 0.0014953613281250,
+-0.0015563964843750, 0.0028076171875000, -0.0019836425781250, 0.0026550292968750,
+-0.0019836425781250, 0.0026855468750000, -0.0020446777343750, 0.0034790039062500,
+-0.0025329589843750, 0.0031127929687500, -0.0024719238281250, 0.0032348632812500,
+-0.0020446777343750, 0.0031738281250000, -0.0025024414062500, 0.0027770996093750,
+-0.0018310546875000, 0.0025329589843750, -0.0009155273437500, 0.0024414062500000,
+-0.0016784667968750, 0.0018615722656250, -0.0003967285156250, 0.0017700195312500,
+-0.0001525878906250, 0.0014038085937500, -0.0005493164062500, 0.0008850097656250,
+-0.0001831054687500, 0.0009155273437500, -0.0000305175781250, 0.0008850097656250,
+-0.0013732910156250, 0.0005798339843750, -0.0010375976562500, 0.0009155273437500,
+-0.0011901855468750, 0.0012817382812500, -0.0024719238281250, 0.0010070800781250,
+-0.0020446777343750, 0.0021057128906250, -0.0008850097656250, 0.0019226074218750,
+-0.0018615722656250, 0.0024414062500000, -0.0006713867187500, 0.0034790039062500,
+0.0013122558593750, 0.0029907226562500, 0.0001220703125000, 0.0043029785156250,
+0.0024719238281250, 0.0044860839843750, 0.0025329589843750, 0.0048522949218750,
+0.0024414062500000, 0.0057067871093750, 0.0036926269531250, 0.0057067871093750,
+0.0033874511718750, 0.0067138671875000, 0.0024108886718750, 0.0066833496093750,
+0.0030517578125000, 0.0066223144531250, 0.0025634765625000, 0.0071716308593750,
+0.0011901855468750, 0.0070800781250000, 0.0018920898437500, 0.0062866210937500,
+0.0016174316406250, 0.0066833496093750, 0.0012817382812500, 0.0055236816406250,
+0.0014953613281250, 0.0043029785156250, 0.0018615722656250, 0.0050354003906250,
+0.0017700195312500, 0.0017700195312500, 0.0009460449218750, 0.0019836425781250,
+0.0014038085937500, 0.0014038085937500, 0.0010986328125000, -0.0007629394531250,
+0.0000000000000000, 0.0000915527343750, 0.0005187988281250, -0.0010070800781250,
+0.0007019042968750, -0.0011901855468750, 0.0000305175781250, -0.0009765625000000,
+0.0009460449218750, -0.0014648437500000, 0.0020751953125000, -0.0014038085937500,
+0.0014343261718750, -0.0005187988281250, 0.0032653808593750, -0.0006103515625000,
+0.0030212402343750, -0.0007934570312500, 0.0034790039062500, -0.0000915527343750,
+0.0048828125000000, -0.0000610351562500, 0.0043029785156250, -0.0013427734375000,
+0.0051574707031250, -0.0009765625000000, 0.0050964355468750, -0.0013427734375000,
+0.0050964355468750, -0.0024719238281250, 0.0057983398437500, -0.0019226074218750,
+0.0056762695312500, -0.0026550292968750, 0.0048217773437500, -0.0028381347656250,
+0.0054321289062500, -0.0023193359375000, 0.0044555664062500, -0.0023498535156250,
+0.0031433105468750, -0.0025939941406250, 0.0041503906250000, -0.0012207031250000,
+0.0026855468750000, -0.0014648437500000, 0.0023803710937500, -0.0010986328125000,
+0.0027160644531250, 0.0000915527343750, 0.0024719238281250, -0.0003051757812500,
+0.0022888183593750, 0.0001525878906250, 0.0023193359375000, 0.0001220703125000,
+0.0030517578125000, 0.0002746582031250, 0.0019836425781250, 0.0008544921875000,
+0.0007324218750000, 0.0006103515625000, 0.0017395019531250, -0.0006103515625000,
+0.0005493164062500, 0.0002441406250000, 0.0001525878906250, -0.0011901855468750,
+0.0007019042968750, -0.0031433105468750, 0.0005798339843750, -0.0018310546875000,
+0.0005187988281250, -0.0049133300781250, 0.0023498535156250, -0.0051269531250000,
+0.0017395019531250},
+{-0.0022888183593750, -0.0016784667968750, -0.0024414062500000, -0.0013427734375000,
+-0.0021362304687500, -0.0014953613281250, -0.0019531250000000, -0.0011291503906250,
+-0.0022583007812500, -0.0007019042968750, -0.0011596679687500, -0.0006408691406250,
+-0.0013122558593750, -0.0002746582031250, -0.0011901855468750, 0.0003051757812500,
+-0.0003967285156250, 0.0005493164062500, -0.0006408691406250, 0.0007934570312500,
+-0.0005187988281250, 0.0009155273437500, -0.0003051757812500, 0.0011901855468750,
+-0.0005493164062500, 0.0014343261718750, -0.0008544921875000, 0.0014648437500000,
+-0.0006408691406250, 0.0018005371093750, -0.0005187988281250, 0.0017089843750000,
+-0.0011291503906250, 0.0019531250000000, 0.0000305175781250, 0.0024414062500000,
+0.0010986328125000, 0.0023193359375000, 0.0000000000000000, 0.0028076171875000,
+0.0024719238281250, 0.0029296875000000, 0.0025024414062500, 0.0028381347656250,
+0.0025634765625000, 0.0030517578125000, 0.0038146972656250, 0.0031738281250000,
+0.0033569335937500, 0.0025939941406250, 0.0034484863281250, 0.0027465820312500,
+0.0041198730468750, 0.0024108886718750, 0.0029602050781250, 0.0018920898437500,
+0.0019226074218750, 0.0023803710937500, 0.0030517578125000, 0.0018920898437500,
+0.0009460449218750, 0.0016174316406250, 0.0006408691406250, 0.0023498535156250,
+0.0011291503906250, 0.0025939941406250, 0.0006103515625000, 0.0022277832031250,
+0.0005798339843750, 0.0047607421875000, 0.0011596679687500, 0.0043029785156250,
+0.0011291503906250, 0.0055236816406250, 0.0010375976562500, 0.0078735351562500,
+0.0011291503906250, 0.0065307617187500, 0.0012817382812500, 0.0090026855468750,
+0.0015258789062500, 0.0096130371093750, 0.0012817382812500, 0.0085754394531250,
+0.0017395019531250, 0.0089111328125000, 0.0021972656250000, 0.0093078613281250,
+0.0018310546875000, 0.0069580078125000, 0.0032043457031250, 0.0075073242187500,
+0.0029907226562500, 0.0060119628906250, 0.0031127929687500, 0.0036621093750000,
+0.0040893554687500, 0.0050048828125000, 0.0037231445312500, 0.0012512207031250,
+0.0041503906250000, 0.0009460449218750, 0.0045166015625000, 0.0014038085937500,
+0.0038452148437500, -0.0000915527343750, 0.0034179687500000, 0.0000305175781250,
+0.0038757324218750, 0.0010070800781250, 0.0026855468750000, 0.0009155273437500,
+0.0027770996093750, 0.0009155273437500, 0.0024719238281250, 0.0016174316406250,
+0.0014343261718750, 0.0014648437500000, 0.0018310546875000, 0.0003662109375000,
+0.0017700195312500, 0.0009460449218750, 0.0013122558593750, -0.0000610351562500,
+0.0017395019531250, -0.0017395019531250, 0.0021057128906250, -0.0008850097656250,
+0.0016174316406250, -0.0019836425781250, 0.0020141601562500, -0.0027160644531250,
+0.0024719238281250, -0.0016174316406250, 0.0018310546875000, -0.0010070800781250,
+0.0014038085937500, -0.0017089843750000, 0.0019836425781250, 0.0002441406250000,
+0.0001525878906250, 0.0005187988281250, -0.0001831054687500, 0.0001525878906250,
+0.0009460449218750, 0.0007324218750000, 0.0008850097656250, 0.0006408691406250,
+0.0001831054687500, -0.0001525878906250, 0.0021667480468750, 0.0002136230468750,
+0.0020141601562500, -0.0003967285156250, 0.0024719238281250, -0.0014953613281250,
+0.0038452148437500, -0.0010070800781250, 0.0030517578125000, -0.0024414062500000,
+0.0037536621093750, -0.0022583007812500, 0.0045471191406250, -0.0025329589843750,
+0.0027160644531250, -0.0036926269531250, 0.0006408691406250, -0.0036315917968750,
+0.0006713867187500, -0.0053100585937500, -0.0020751953125000, -0.0050354003906250,
+-0.0013427734375000, -0.0046386718750000, -0.0017700195312500, -0.0054626464843750,
+-0.0040283203125000, -0.0054626464843750, -0.0030517578125000, -0.0046386718750000,
+-0.0032348632812500, -0.0048828125000000, -0.0044250488281250, -0.0043029785156250,
+-0.0026245117187500, -0.0036315917968750, -0.0007019042968750, -0.0042419433593750,
+-0.0021972656250000, -0.0018920898437500, 0.0007629394531250, -0.0024719238281250,
+0.0016479492187500, -0.0016479492187500, 0.0006713867187500, 0.0004882812500000,
+0.0009460449218750, -0.0006408691406250, 0.0014953613281250, 0.0007019042968750,
+0.0003051757812500, 0.0012207031250000, 0.0000915527343750, 0.0006713867187500,
+0.0006408691406250, 0.0005493164062500, 0.0005187988281250, 0.0007629394531250,
+0.0003967285156250, 0.0001525878906250, 0.0017089843750000, -0.0000915527343750,
+0.0018310546875000, 0.0004577636718750, 0.0018920898437500, 0.0006713867187500,
+0.0026245117187500, 0.0001831054687500, 0.0025939941406250, 0.0013427734375000,
+0.0020446777343750, 0.0013122558593750, 0.0024414062500000, 0.0013427734375000,
+0.0021362304687500, 0.0016784667968750, 0.0014343261718750, 0.0013122558593750,
+0.0017700195312500, 0.0030517578125000, 0.0008239746093750, 0.0020141601562500,
+0.0014038085937500, 0.0031127929687500, 0.0004577636718750, 0.0052490234375000,
+-0.0009460449218750, 0.0036926269531250, 0.0000305175781250, 0.0061645507812500,
+-0.0019531250000000, 0.0062255859375000, -0.0022583007812500, 0.0059814453125000,
+-0.0021362304687500, 0.0070495605468750, -0.0030822753906250, 0.0065612792968750,
+-0.0029296875000000, 0.0063171386718750, -0.0016784667968750, 0.0068969726562500,
+-0.0023803710937500, 0.0063781738281250, -0.0008544921875000, 0.0057983398437500,
+0.0013732910156250, 0.0060729980468750, 0.0003967285156250, 0.0040588378906250,
+0.0028991699218750, 0.0052185058593750, 0.0038452148437500, 0.0033264160156250,
+0.0033874511718750, 0.0003662109375000, 0.0040588378906250, 0.0024414062500000,
+0.0045166015625000, -0.0012207031250000, 0.0034179687500000, -0.0011291503906250,
+0.0033569335937500, -0.0007324218750000, 0.0037536621093750, -0.0025329589843750,
+0.0029907226562500, -0.0018615722656250, 0.0023498535156250, -0.0007629394531250,
+0.0040588378906250, -0.0014343261718750, 0.0027160644531250, -0.0005493164062500,
+0.0035705566406250, 0.0009155273437500, 0.0054626464843750, 0.0000610351562500,
+0.0035095214843750, 0.0021057128906250, 0.0065002441406250, 0.0018005371093750,
+0.0059509277343750, 0.0021057128906250, 0.0056457519531250, 0.0033569335937500,
+0.0072021484375000, 0.0024719238281250, 0.0061035156250000, 0.0042419433593750,
+0.0063781738281250, 0.0041503906250000, 0.0065307617187500, 0.0036315917968750,
+0.0055236816406250, 0.0045471191406250, 0.0049133300781250, 0.0045166015625000,
+0.0051574707031250, 0.0030517578125000, 0.0031127929687500, 0.0033874511718750,
+0.0034484863281250, 0.0029602050781250, 0.0023193359375000, 0.0018920898437500,
+0.0003356933593750, 0.0023803710937500, 0.0012817382812500, 0.0009155273437500,
+-0.0009155273437500, 0.0013732910156250, -0.0012207031250000, 0.0006103515625000,
+-0.0016479492187500, -0.0008850097656250, -0.0029296875000000, 0.0000610351562500,
+-0.0026550292968750, -0.0018615722656250, -0.0033569335937500, -0.0017395019531250,
+-0.0036315917968750, -0.0019226074218750, -0.0038757324218750, -0.0030212402343750,
+-0.0042724609375000, -0.0024414062500000, -0.0041503906250000, -0.0033264160156250,
+-0.0044860839843750, -0.0033264160156250, -0.0047607421875000, -0.0032043457031250,
+-0.0046997070312500, -0.0036926269531250, -0.0046691894531250, -0.0035400390625000,
+-0.0048828125000000, -0.0029907226562500, -0.0048828125000000, -0.0034179687500000,
+-0.0048828125000000, -0.0026550292968750, -0.0048217773437500, -0.0017089843750000,
+-0.0048217773437500, -0.0023803710937500, -0.0048522949218750, -0.0005798339843750,
+-0.0047607421875000, -0.0004882812500000, -0.0047607421875000, -0.0004882812500000,
+-0.0048522949218750, 0.0004577636718750, -0.0047302246093750, 0.0003356933593750,
+-0.0044250488281250, 0.0000610351562500, -0.0047302246093750, 0.0002746582031250,
+-0.0048828125000000, 0.0005187988281250, -0.0045166015625000, 0.0003356933593750,
+-0.0046081542968750, 0.0003051757812500, -0.0049438476562500, 0.0012817382812500,
+-0.0036315917968750, 0.0010375976562500, -0.0041503906250000, 0.0014038085937500,
+-0.0036010742187500, 0.0022583007812500, -0.0024414062500000, 0.0019531250000000,
+-0.0032958984375000, 0.0026245117187500, -0.0012817382812500, 0.0027160644531250,
+-0.0016784667968750, 0.0026245117187500, -0.0010986328125000, 0.0030212402343750,
+0.0003662109375000, 0.0029907226562500, -0.0006103515625000, 0.0018920898437500,
+0.0008239746093750, 0.0023193359375000, 0.0011596679687500, 0.0016479492187500,
+0.0006408691406250, 0.0002746582031250, 0.0009460449218750, 0.0010070800781250,
+0.0009460449218750, -0.0002136230468750, -0.0004882812500000, -0.0003967285156250,
+-0.0001831054687500, -0.0001525878906250, -0.0006713867187500, -0.0006408691406250,
+-0.0018615722656250, -0.0005493164062500, -0.0013427734375000, 0.0001831054687500,
+-0.0017089843750000, -0.0003356933593750, -0.0021972656250000, 0.0006103515625000,
+-0.0011291503906250, 0.0018920898437500, -0.0006408691406250, 0.0009460449218750,
+-0.0014038085937500, 0.0028381347656250, 0.0014038085937500, 0.0029602050781250,
+0.0010681152343750, 0.0027770996093750, 0.0014648437500000, 0.0036315917968750,
+0.0034179687500000, 0.0035400390625000, 0.0026550292968750, 0.0036010742187500,
+0.0037231445312500, 0.0035400390625000, 0.0041809082031250, 0.0037841796875000,
+0.0029296875000000, 0.0037536621093750, 0.0029296875000000, 0.0036010742187500,
+0.0038146972656250, 0.0053710937500000, -0.0012817382812500, 0.0046386718750000,
+0.0000305175781250, 0.0057678222656250, -0.0013732910156250, 0.0079040527343750,
+-0.0064086914062500, 0.0067443847656250, -0.0043029785156250, 0.0086669921875000,
+-0.0038757324218750, 0.0089721679687500, -0.0061950683593750, 0.0089416503906250,
+-0.0024719238281250, 0.0097656250000000, 0.0012817382812500, 0.0095214843750000,
+-0.0015258789062500, 0.0090026855468750, 0.0068054199218750, 0.0095520019531250,
+0.0062866210937500, 0.0083923339843750, 0.0079956054687500, 0.0072937011718750,
+0.0138244628906250, 0.0083923339843750, 0.0112915039062500, 0.0052490234375000,
+0.0146789550781250, 0.0052490234375000, 0.0161437988281250, 0.0053100585937500,
+0.0136108398437500, 0.0038146972656250, 0.0133666992187500, 0.0043945312500000,
+0.0145568847656250, 0.0046081542968750, 0.0078125000000000, 0.0038452148437500,
+0.0087890625000000, 0.0043029785156250, 0.0072631835937500, 0.0055541992187500,
+0.0021667480468750, 0.0050659179687500, 0.0043029785156250, 0.0034790039062500,
+0.0014038085937500, 0.0043334960937500, 0.0008850097656250, 0.0025634765625000,
+0.0009460449218750, 0.0003662109375000, 0.0001831054687500, 0.0018310546875000,
+0.0007019042968750, -0.0025939941406250, -0.0014343261718750, -0.0027160644531250,
+-0.0017089843750000, -0.0022888183593750, -0.0003662109375000, -0.0046997070312500,
+-0.0007324218750000, -0.0046081542968750, -0.0015869140625000, -0.0014648437500000,
+0.0019836425781250, -0.0021667480468750, 0.0020446777343750, -0.0022583007812500,
+0.0011901855468750, 0.0006103515625000, 0.0027770996093750, -0.0009155273437500,
+-0.0030517578125000, -0.0007019042968750, 0.0007324218750000, -0.0003967285156250,
+0.0001831054687500, -0.0013427734375000, -0.0002441406250000, -0.0017089843750000,
+0.0008239746093750, -0.0009460449218750, 0.0010986328125000, -0.0040588378906250,
+-0.0012512207031250, -0.0039978027343750, -0.0009460449218750, -0.0039367675781250,
+-0.0009460449218750, -0.0060119628906250, -0.0025329589843750, -0.0054321289062500,
+-0.0021972656250000, -0.0024414062500000, -0.0011901855468750, -0.0047607421875000,
+-0.0018310546875000, -0.0015563964843750, -0.0012817382812500, 0.0035400390625000,
+0.0002441406250000},
+{-0.0046386718750000, 0.0008544921875000, -0.0063171386718750, 0.0012207031250000,
+-0.0054931640625000, 0.0007629394531250, -0.0061950683593750, 0.0005187988281250,
+-0.0068054199218750, 0.0007934570312500, -0.0056457519531250, 0.0008544921875000,
+-0.0051269531250000, 0.0007629394531250, -0.0058288574218750, 0.0014953613281250,
+-0.0026855468750000, 0.0012207031250000, -0.0034179687500000, 0.0018920898437500,
+-0.0026550292968750, 0.0027770996093750, 0.0000305175781250, 0.0021667480468750,
+-0.0011291503906250, 0.0038757324218750, 0.0002136230468750, 0.0038146972656250,
+0.0008544921875000, 0.0038757324218750, -0.0000610351562500, 0.0049438476562500,
+-0.0002136230468750, 0.0045776367187500, 0.0004577636718750, 0.0045471191406250,
+-0.0014648437500000, 0.0047607421875000, -0.0013122558593750, 0.0047607421875000,
+-0.0011291503906250, 0.0045776367187500, -0.0022277832031250, 0.0045776367187500,
+-0.0020751953125000, 0.0051574707031250, -0.0011291503906250, 0.0048217773437500,
+-0.0022583007812500, 0.0054016113281250, -0.0002746582031250, 0.0062866210937500,
+0.0017395019531250, 0.0057373046875000, -0.0003662109375000, 0.0068359375000000,
+0.0049743652343750, 0.0070800781250000, 0.0053710937500000, 0.0067138671875000,
+0.0044555664062500, 0.0069274902343750, 0.0071716308593750, 0.0071105957031250,
+0.0072937011718750, 0.0064392089843750, 0.0017395019531250, 0.0066833496093750,
+0.0035705566406250, 0.0060119628906250, 0.0017395019531250, 0.0052795410156250,
+-0.0036315917968750, 0.0058593750000000, -0.0010375976562500, 0.0038757324218750,
+-0.0027465820312500, 0.0039978027343750, -0.0043945312500000, 0.0039367675781250,
+-0.0014648437500000, 0.0025024414062500, 0.0000305175781250, 0.0028991699218750,
+-0.0015258789062500, 0.0043945312500000, 0.0064392089843750, 0.0039367675781250,
+0.0050048828125000, 0.0039978027343750, 0.0078735351562500, 0.0053100585937500,
+0.0149230957031250, 0.0050048828125000, 0.0114440917968750, 0.0032653808593750,
+0.0160522460937500, 0.0045776367187500, 0.0181579589843750, 0.0025024414062500,
+0.0144653320312500, -0.0007934570312500, 0.0134887695312500, 0.0012207031250000,
+0.0155639648437500, -0.0016479492187500, 0.0073547363281250, -0.0024108886718750,
+0.0076599121093750, -0.0018310546875000, 0.0070495605468750, -0.0024719238281250,
+0.0022277832031250, -0.0024719238281250, 0.0035705566406250, -0.0014038085937500,
+0.0026245117187500, -0.0019226074218750, 0.0027770996093750, -0.0011901855468750,
+0.0012512207031250, 0.0002441406250000, -0.0003356933593750, -0.0007629394531250,
+0.0011596679687500, -0.0005493164062500, -0.0023193359375000, -0.0000305175781250,
+-0.0027465820312500, -0.0002441406250000, -0.0015869140625000, -0.0009460449218750,
+-0.0023498535156250, -0.0014038085937500, 0.0020141601562500, -0.0032348632812500,
+-0.0023498535156250, -0.0026245117187500, -0.0020751953125000, -0.0025024414062500,
+-0.0013427734375000, -0.0038146972656250, -0.0021972656250000, -0.0038452148437500,
+-0.0026550292968750, -0.0005798339843750, 0.0005187988281250, -0.0018310546875000,
+0.0002136230468750, 0.0001525878906250, -0.0002136230468750, 0.0035400390625000,
+0.0017700195312500, 0.0014343261718750, 0.0016479492187500, 0.0067749023437500,
+-0.0013427734375000, 0.0065917968750000, -0.0004272460937500, 0.0074462890625000,
+-0.0008544921875000, 0.0105590820312500, -0.0036315917968750, 0.0090637207031250,
+-0.0029602050781250, 0.0127868652343750, -0.0018615722656250, 0.0129394531250000,
+-0.0025634765625000, 0.0126037597656250, -0.0021972656250000, 0.0147094726562500,
+-0.0009765625000000, 0.0140380859375000, -0.0013427734375000, 0.0105590820312500,
+-0.0010375976562500, 0.0136413574218750, -0.0009460449218750, 0.0087280273437500,
+-0.0007934570312500, 0.0021667480468750, -0.0008239746093750, 0.0069885253906250,
+-0.0010681152343750, -0.0029602050781250, 0.0006103515625000, -0.0038757324218750,
+0.0002441406250000, -0.0024414062500000, 0.0003356933593750, -0.0067138671875000,
+0.0017395019531250, -0.0062866210937500, 0.0012817382812500, -0.0020141601562500,
+0.0006408691406250, -0.0040893554687500, 0.0009460449218750, -0.0015869140625000,
+0.0011596679687500, 0.0034790039062500, 0.0005493164062500, 0.0005493164062500,
+0.0003356933593750, 0.0042114257812500, 0.0022277832031250, 0.0054321289062500,
+0.0019836425781250, 0.0038757324218750, 0.0018005371093750, 0.0043029785156250,
+0.0028076171875000, 0.0048522949218750, 0.0026550292968750, 0.0013122558593750,
+0.0020141601562500, 0.0020141601562500, 0.0019836425781250, 0.0011291503906250,
+0.0021972656250000, -0.0018310546875000, 0.0019836425781250, -0.0003967285156250,
+0.0018615722656250, -0.0011901855468750, 0.0027770996093750, -0.0024108886718750,
+0.0026550292968750, -0.0010681152343750, 0.0026550292968750, 0.0001525878906250,
+0.0032958984375000, -0.0008850097656250, 0.0032348632812500, 0.0007629394531250,
+0.0025329589843750, 0.0012207031250000, 0.0026550292968750, 0.0002746582031250,
+0.0028991699218750, 0.0004577636718750, 0.0025329589843750, 0.0009765625000000,
+0.0023803710937500, -0.0017089843750000, 0.0035095214843750, -0.0007324218750000,
+0.0034790039062500, -0.0022277832031250, 0.0032043457031250, -0.0046691894531250,
+0.0036926269531250, -0.0027770996093750, 0.0037536621093750, -0.0075073242187500,
+0.0028076171875000, -0.0070800781250000, 0.0029602050781250, -0.0073242187500000,
+0.0027160644531250, -0.0102233886718750, 0.0020141601562500, -0.0090332031250000,
+0.0021972656250000, -0.0091247558593750, 0.0015563964843750, -0.0100402832031250,
+0.0015869140625000, -0.0084533691406250, 0.0014343261718750, -0.0072937011718750,
+0.0008239746093750, -0.0084533691406250, 0.0010375976562500, -0.0045776367187500,
+0.0010070800781250, -0.0053100585937500, 0.0007629394531250, -0.0041503906250000,
+0.0010681152343750, -0.0012207031250000, 0.0014343261718750, -0.0029602050781250,
+0.0012207031250000, 0.0003967285156250, 0.0016784667968750, 0.0001831054687500,
+0.0017395019531250, 0.0005493164062500, 0.0016784667968750, 0.0027465820312500,
+0.0019226074218750, 0.0018920898437500, 0.0018615722656250, 0.0035400390625000,
+0.0018920898437500, 0.0038757324218750, 0.0021057128906250, 0.0038452148437500,
+0.0017395019531250, 0.0046386718750000, 0.0015258789062500, 0.0048217773437500,
+0.0018615722656250, 0.0046997070312500, 0.0007019042968750, 0.0052185058593750,
+0.0008544921875000, 0.0052185058593750, 0.0005798339843750, 0.0050048828125000,
+-0.0002746582031250, 0.0056457519531250, 0.0000610351562500, 0.0051574707031250,
+-0.0005798339843750, 0.0054931640625000, -0.0006408691406250, 0.0054931640625000,
+-0.0006408691406250, 0.0051879882812500, -0.0009460449218750, 0.0057373046875000,
+-0.0008850097656250, 0.0052795410156250, -0.0007019042968750, 0.0054626464843750,
+-0.0008850097656250, 0.0056152343750000, -0.0006103515625000, 0.0053405761718750,
+-0.0002441406250000, 0.0056457519531250, -0.0004882812500000, 0.0055847167968750,
+0.0000610351562500, 0.0057373046875000, 0.0001831054687500, 0.0059204101562500,
+0.0000915527343750, 0.0059204101562500, 0.0001525878906250, 0.0060424804687500,
+0.0001831054687500, 0.0061645507812500, 0.0008544921875000, 0.0062255859375000,
+0.0002441406250000, 0.0063171386718750, 0.0010070800781250, 0.0064086914062500,
+0.0021972656250000, 0.0062866210937500, 0.0013122558593750, 0.0061645507812500,
+0.0024719238281250, 0.0063476562500000, 0.0028991699218750, 0.0056762695312500,
+0.0025329589843750, 0.0051269531250000, 0.0026245117187500, 0.0053710937500000,
+0.0026550292968750, 0.0030212402343750, 0.0021057128906250, 0.0035705566406250,
+0.0025024414062500, 0.0021057128906250, 0.0018920898437500, -0.0002746582031250,
+0.0008850097656250, 0.0008239746093750, 0.0017395019531250, -0.0022277832031250,
+0.0010986328125000, -0.0023193359375000, 0.0004577636718750, -0.0027160644531250,
+0.0013122558593750, -0.0045166015625000, 0.0018310546875000, -0.0039978027343750,
+0.0012512207031250, -0.0048828125000000, 0.0028991699218750, -0.0051879882812500,
+0.0025634765625000, -0.0046691894531250, 0.0030212402343750, -0.0046691894531250,
+0.0043334960937500, -0.0047607421875000, 0.0036010742187500, -0.0037231445312500,
+0.0044555664062500, -0.0039062500000000, 0.0048217773437500, -0.0032348632812500,
+0.0043945312500000, -0.0022888183593750, 0.0042419433593750, -0.0028381347656250,
+0.0044860839843750, -0.0010681152343750, 0.0047912597656250, -0.0011596679687500,
+0.0044555664062500, -0.0007019042968750, 0.0044250488281250, 0.0003051757812500,
+0.0050354003906250, -0.0002136230468750, 0.0049743652343750, 0.0014343261718750,
+0.0037536621093750, 0.0008544921875000, 0.0044555664062500, 0.0018005371093750,
+0.0033264160156250, 0.0033569335937500, 0.0015563964843750, 0.0022583007812500,
+0.0026855468750000, 0.0052185058593750, 0.0012817382812500, 0.0050964355468750,
+0.0005187988281250, 0.0054626464843750, 0.0015563964843750, 0.0074157714843750,
+0.0018920898437500, 0.0068054199218750, 0.0012207031250000, 0.0076293945312500,
+0.0029296875000000, 0.0079345703125000, 0.0032958984375000, 0.0072937011718750,
+0.0022583007812500, 0.0074157714843750, 0.0023498535156250, 0.0077819824218750,
+0.0029907226562500, 0.0053710937500000, 0.0010070800781250, 0.0057678222656250,
+0.0009765625000000, 0.0052490234375000, 0.0010681152343750, 0.0034179687500000,
+0.0001220703125000, 0.0040588378906250, 0.0002746582031250, 0.0024414062500000,
+0.0009155273437500, 0.0026855468750000, 0.0004272460937500, 0.0018920898437500,
+0.0011291503906250, 0.0004272460937500, 0.0023803710937500, 0.0010375976562500,
+0.0014953613281250, -0.0016479492187500, 0.0005798339843750, -0.0016174316406250,
+0.0018310546875000, -0.0016784667968750, 0.0008544921875000, -0.0035095214843750,
+-0.0012512207031250, -0.0032348632812500, -0.0002441406250000, -0.0017395019531250,
+-0.0017089843750000, -0.0028076171875000, -0.0016479492187500, -0.0011291503906250,
+-0.0025024414062500, 0.0015869140625000, -0.0039367675781250, -0.0001220703125000,
+-0.0027770996093750, 0.0014038085937500, -0.0032043457031250, 0.0025634765625000,
+-0.0042114257812500, 0.0010681152343750, -0.0030212402343750, -0.0004882812500000,
+-0.0014038085937500, -0.0004577636718750, -0.0012512207031250, -0.0021362304687500,
+0.0011901855468750, -0.0021972656250000, 0.0003967285156250, -0.0016174316406250,
+0.0006103515625000, -0.0019226074218750, 0.0025939941406250, -0.0022277832031250,
+0.0019226074218750, -0.0022888183593750, 0.0024414062500000, -0.0017700195312500,
+0.0029907226562500, -0.0025634765625000, 0.0024108886718750, -0.0035705566406250,
+0.0019226074218750, -0.0029907226562500, 0.0023803710937500, -0.0046997070312500,
+0.0019531250000000, -0.0047302246093750, 0.0019226074218750, -0.0047912597656250,
+0.0016174316406250, -0.0057983398437500, 0.0013427734375000, -0.0055847167968750,
+0.0017700195312500, -0.0052185058593750, 0.0012817382812500, -0.0056457519531250,
+0.0009765625000000, -0.0049438476562500, 0.0016784667968750, -0.0043640136718750,
+0.0020446777343750, -0.0049743652343750, 0.0014648437500000, -0.0022888183593750,
+0.0022277832031250, -0.0031738281250000, 0.0031127929687500, -0.0021057128906250,
+0.0014953613281250, 0.0005798339843750, 0.0002746582031250, -0.0009765625000000,
+0.0017700195312500, 0.0009155273437500, -0.0011291503906250, 0.0012817382812500,
+-0.0014343261718750, 0.0006713867187500, -0.0005493164062500, 0.0010375976562500,
+-0.0014648437500000, 0.0012512207031250, -0.0015258789062500, 0.0005798339843750,
+0.0018615722656250},
+{0.0056762695312500, -0.0014953613281250, 0.0049438476562500, -0.0014343261718750,
+0.0049743652343750, -0.0021972656250000, 0.0050659179687500, -0.0042114257812500,
+0.0047912597656250, -0.0033264160156250, 0.0046081542968750, -0.0046081542968750,
+0.0042419433593750, -0.0050659179687500, 0.0047607421875000, -0.0043640136718750,
+0.0037536621093750, -0.0041809082031250, 0.0025634765625000, -0.0044250488281250,
+0.0034179687500000, -0.0028991699218750, 0.0013122558593750, -0.0030822753906250,
+0.0011596679687500, -0.0024108886718750, 0.0017089843750000, -0.0011596679687500,
+0.0008850097656250, -0.0017700195312500, 0.0007324218750000, 0.0000305175781250,
+0.0024108886718750, -0.0001831054687500, 0.0021057128906250, 0.0002441406250000,
+0.0022583007812500, 0.0015563964843750, 0.0035400390625000, 0.0009460449218750,
+0.0031738281250000, 0.0017395019531250, 0.0021667480468750, 0.0015563964843750,
+0.0027465820312500, 0.0025634765625000, 0.0024108886718750, 0.0034790039062500,
+0.0013427734375000, 0.0025024414062500, 0.0017395019531250, 0.0058898925781250,
+0.0013122558593750, 0.0061035156250000, 0.0013427734375000, 0.0057678222656250,
+0.0010681152343750, 0.0073852539062500, 0.0007324218750000, 0.0071105957031250,
+0.0010681152343750, 0.0058593750000000, 0.0001220703125000, 0.0066223144531250,
+0.0004272460937500, 0.0054016113281250, -0.0009765625000000, 0.0035705566406250,
+-0.0023498535156250, 0.0045776367187500, -0.0009155273437500, 0.0020751953125000,
+-0.0037536621093750, 0.0021057128906250, -0.0042724609375000, 0.0013122558593750,
+-0.0036621093750000, -0.0004882812500000, -0.0041198730468750, 0.0003356933593750,
+-0.0042114257812500, -0.0017395019531250, -0.0042724609375000, -0.0019531250000000,
+-0.0044555664062500, -0.0014648437500000, -0.0034484863281250, -0.0025024414062500,
+-0.0030822753906250, -0.0025939941406250, -0.0040283203125000, 0.0003051757812500,
+-0.0012512207031250, -0.0003662109375000, -0.0009460449218750, -0.0001525878906250,
+-0.0007629394531250, 0.0024719238281250, 0.0005187988281250, 0.0019836425781250,
+0.0006103515625000, -0.0001831054687500, 0.0017089843750000, 0.0011901855468750,
+0.0015258789062500, 0.0004272460937500, 0.0014648437500000, -0.0019531250000000,
+0.0023193359375000, -0.0006408691406250, 0.0022583007812500, -0.0019531250000000,
+0.0015258789062500, -0.0024414062500000, 0.0018920898437500, -0.0022277832031250,
+0.0017089843750000, -0.0023498535156250, 0.0009765625000000, -0.0022888183593750,
+0.0012207031250000, -0.0031127929687500, 0.0008239746093750, -0.0027160644531250,
+0.0007324218750000, -0.0036926269531250, 0.0008544921875000, -0.0050048828125000,
+0.0006713867187500, -0.0040893554687500, 0.0005798339843750, -0.0058593750000000,
+0.0011291503906250, -0.0061950683593750, 0.0011291503906250, -0.0058898925781250,
+0.0007934570312500, -0.0065002441406250, 0.0011291503906250, -0.0065917968750000,
+0.0014038085937500, -0.0061035156250000, -0.0003662109375000, -0.0064697265625000,
+-0.0003662109375000, -0.0056762695312500, 0.0005187988281250, -0.0051269531250000,
+-0.0000915527343750, -0.0059509277343750, -0.0006103515625000, -0.0028076171875000,
+0.0024414062500000, -0.0036926269531250, 0.0018310546875000, -0.0025939941406250,
+0.0023498535156250, 0.0002746582031250, 0.0047302246093750, -0.0013732910156250,
+0.0037231445312500, 0.0007324218750000, 0.0041503906250000, 0.0012512207031250,
+0.0046386718750000, 0.0005493164062500, 0.0039672851562500, 0.0009460449218750,
+0.0034179687500000, 0.0012207031250000, 0.0036010742187500, 0.0003967285156250,
+0.0021972656250000, 0.0003662109375000, 0.0021057128906250, 0.0004577636718750,
+0.0017700195312500, 0.0001525878906250, 0.0004272460937500, 0.0002136230468750,
+0.0003356933593750, 0.0006408691406250, 0.0009155273437500, 0.0002746582031250,
+-0.0000305175781250, 0.0007019042968750, 0.0001525878906250, 0.0012207031250000,
+0.0010070800781250, 0.0007324218750000, 0.0001220703125000, 0.0028076171875000,
+0.0001525878906250, 0.0022583007812500, 0.0002746582031250, 0.0025939941406250,
+-0.0000610351562500, 0.0042114257812500, -0.0004882812500000, 0.0035095214843750,
+-0.0003967285156250, 0.0047302246093750, 0.0003051757812500, 0.0048217773437500,
+-0.0002136230468750, 0.0042419433593750, 0.0007019042968750, 0.0045776367187500,
+0.0019836425781250, 0.0049133300781250, 0.0012512207031250, 0.0037231445312500,
+0.0032043457031250, 0.0037231445312500, 0.0034179687500000, 0.0036315917968750,
+0.0034179687500000, 0.0030822753906250, 0.0043334960937500, 0.0032653808593750,
+0.0042419433593750, 0.0027160644531250, 0.0044860839843750, 0.0029296875000000,
+0.0046997070312500, 0.0025634765625000, 0.0047302246093750, 0.0020751953125000,
+0.0046997070312500, 0.0026855468750000, 0.0046691894531250, 0.0015258789062500,
+0.0043334960937500, 0.0017089843750000, 0.0042724609375000, 0.0017395019531250,
+0.0039367675781250, 0.0010681152343750, 0.0034790039062500, 0.0015258789062500,
+0.0034179687500000, 0.0008544921875000, 0.0027770996093750, 0.0008850097656250,
+0.0026245117187500, 0.0010681152343750, 0.0025024414062500, 0.0007629394531250,
+0.0021057128906250, 0.0006713867187500, 0.0019836425781250, 0.0010681152343750,
+0.0019531250000000, 0.0008544921875000, 0.0020141601562500, 0.0009765625000000,
+0.0018920898437500, 0.0012512207031250, 0.0019836425781250, 0.0009155273437500,
+0.0021667480468750, 0.0014648437500000, 0.0015869140625000, 0.0014343261718750,
+0.0018310546875000, 0.0012817382812500, 0.0013122558593750, 0.0015258789062500,
+0.0006103515625000, 0.0015563964843750, 0.0009765625000000, 0.0010681152343750,
+-0.0003967285156250, 0.0012512207031250, -0.0001525878906250, 0.0011291503906250,
+-0.0006408691406250, 0.0007629394531250, -0.0018310546875000, 0.0010375976562500,
+-0.0011596679687500, 0.0004882812500000, -0.0024719238281250, 0.0006408691406250,
+-0.0024719238281250, 0.0005187988281250, -0.0026550292968750, 0.0000915527343750,
+-0.0033569335937500, 0.0005187988281250, -0.0029907226562500, 0.0003051757812500,
+-0.0037841796875000, 0.0001831054687500, -0.0034484863281250, 0.0005798339843750,
+-0.0037231445312500, 0.0007629394531250, -0.0043640136718750, 0.0005493164062500,
+-0.0040588378906250, 0.0014343261718750, -0.0050964355468750, 0.0013122558593750,
+-0.0048217773437500, 0.0014953613281250, -0.0050354003906250, 0.0021972656250000,
+-0.0057983398437500, 0.0019226074218750, -0.0052490234375000, 0.0021972656250000,
+-0.0060424804687500, 0.0021972656250000, -0.0060424804687500, 0.0023803710937500,
+-0.0059204101562500, 0.0026245117187500, -0.0063476562500000, 0.0025024414062500,
+-0.0061645507812500, 0.0030822753906250, -0.0054626464843750, 0.0030517578125000,
+-0.0058898925781250, 0.0033874511718750, -0.0048522949218750, 0.0038452148437500,
+-0.0036926269531250, 0.0035705566406250, -0.0044555664062500, 0.0043334960937500,
+-0.0015563964843750, 0.0042724609375000, -0.0020751953125000, 0.0043640136718750,
+-0.0010375976562500, 0.0048522949218750, 0.0014343261718750, 0.0046081542968750,
+0.0000610351562500, 0.0045776367187500, 0.0025634765625000, 0.0048217773437500,
+0.0030212402343750, 0.0045776367187500, 0.0023193359375000, 0.0041809082031250,
+0.0034484863281250, 0.0043640136718750, 0.0038757324218750, 0.0047912597656250,
+0.0012512207031250, 0.0044555664062500, 0.0023498535156250, 0.0048217773437500,
+0.0008544921875000, 0.0056457519531250, -0.0020751953125000, 0.0053710937500000,
+-0.0002746582031250, 0.0053100585937500, -0.0032348632812500, 0.0057373046875000,
+-0.0037536621093750, 0.0053100585937500, -0.0030517578125000, 0.0047302246093750,
+-0.0037536621093750, 0.0051879882812500, -0.0035095214843750, 0.0041503906250000,
+-0.0020141601562500, 0.0042724609375000, -0.0020751953125000, 0.0036926269531250,
+-0.0008544921875000, 0.0027770996093750, 0.0011596679687500, 0.0032958984375000,
+0.0008239746093750, 0.0014953613281250, 0.0016479492187500, 0.0016784667968750,
+0.0024108886718750, 0.0011291503906250, 0.0014038085937500, -0.0002441406250000,
+0.0004577636718750, 0.0003662109375000, 0.0010681152343750, -0.0012207031250000,
+-0.0002441406250000, -0.0010681152343750, -0.0012207031250000, -0.0015563964843750,
+-0.0002441406250000, -0.0027465820312500, 0.0002746582031250, -0.0020751953125000,
+-0.0007019042968750, -0.0030212402343750, 0.0012512207031250, -0.0028076171875000,
+0.0011596679687500, -0.0037231445312500, 0.0012512207031250, -0.0049133300781250,
+0.0022888183593750, -0.0038757324218750, 0.0018615722656250, -0.0050048828125000,
+0.0027160644531250, -0.0059814453125000, 0.0030822753906250, -0.0046691894531250,
+0.0031127929687500, -0.0040283203125000, 0.0035095214843750, -0.0049133300781250,
+0.0036621093750000, -0.0013122558593750, 0.0032043457031250, -0.0016479492187500,
+0.0030517578125000, -0.0019226074218750, 0.0039672851562500, 0.0002441406250000,
+0.0038146972656250, -0.0000915527343750, 0.0032043457031250, -0.0014953613281250,
+0.0082092285156250, -0.0012207031250000, 0.0070495605468750, -0.0010986328125000,
+0.0086059570312500, -0.0019836425781250, 0.0133972167968750, -0.0021667480468750,
+0.0112915039062500, -0.0015563964843750, 0.0113220214843750, -0.0014648437500000,
+0.0136108398437500, -0.0016479492187500, 0.0105895996093750, -0.0016174316406250,
+0.0070800781250000, -0.0017395019531250, 0.0095214843750000, -0.0020141601562500,
+0.0048522949218750, -0.0020446777343750, 0.0040283203125000, -0.0018310546875000,
+0.0052185058593750, -0.0018615722656250, 0.0040588378906250, 0.0048828125000000,
+0.0086669921875000, 0.0086059570312500, 0.0054931640625000, 0.0083923339843750,
+0.0051879882812500, 0.0077514648437500, 0.0061950683593750, 0.0080261230468750,
+0.0066833496093750, 0.0085449218750000, 0.0059204101562500, 0.0088500976562500,
+0.0058898925781250, 0.0082397460937500, 0.0077819824218750, 0.0086059570312500,
+0.0047302246093750, 0.0100097656250000, 0.0013427734375000, 0.0095520019531250,
+0.0039978027343750, 0.0074157714843750, -0.0007324218750000, 0.0097351074218750,
+-0.0018615722656250, 0.0059509277343750, -0.0001831054687500, 0.0012207031250000,
+-0.0005798339843750, 0.0049438476562500, -0.0014038085937500, -0.0025024414062500,
+0.0000915527343750, -0.0032958984375000, 0.0008239746093750, -0.0019226074218750,
+-0.0003967285156250, -0.0049743652343750, -0.0010375976562500, -0.0049438476562500,
+-0.0003356933593750, -0.0007934570312500, -0.0024108886718750, -0.0020141601562500,
+-0.0026550292968750, -0.0014648437500000, -0.0022888183593750, 0.0020446777343750,
+-0.0030517578125000, 0.0010070800781250, -0.0030822753906250, 0.0010375976562500,
+-0.0020751953125000, 0.0019226074218750, -0.0019531250000000, 0.0014343261718750,
+-0.0024719238281250, 0.0005798339843750, -0.0023193359375000, 0.0011596679687500,
+-0.0020141601562500, 0.0007019042968750, -0.0029296875000000, 0.0003967285156250,
+-0.0032348632812500, 0.0009460449218750, -0.0026245117187500, 0.0013122558593750,
+-0.0025329589843750, 0.0009155273437500, -0.0030517578125000, 0.0011596679687500,
+-0.0018920898437500, 0.0018310546875000, -0.0017395019531250, 0.0011901855468750,
+-0.0018920898437500, 0.0006408691406250, -0.0017395019531250, 0.0012817382812500,
+-0.0018310546875000, -0.0010681152343750, -0.0012817382812500, -0.0002136230468750,
+-0.0018310546875000, -0.0014953613281250, -0.0011291503906250, -0.0041198730468750,
+-0.0000610351562500, -0.0024414062500000, -0.0007934570312500, -0.0044250488281250,
+0.0008850097656250, -0.0052185058593750, 0.0009765625000000, -0.0038146972656250,
+0.0006713867187500},
+{0.0014343261718750, -0.0032653808593750, 0.0009765625000000, -0.0042724609375000,
+0.0009765625000000, -0.0041809082031250, 0.0010070800781250, -0.0042724609375000,
+0.0002441406250000, -0.0043640136718750, 0.0004577636718750, -0.0039978027343750,
+-0.0001525878906250, -0.0039672851562500, -0.0010986328125000, -0.0044860839843750,
+-0.0007019042968750, -0.0030517578125000, -0.0022583007812500, -0.0030822753906250,
+-0.0026855468750000, -0.0030517578125000, -0.0017700195312500, -0.0025024414062500,
+-0.0019226074218750, -0.0028686523437500, -0.0022277832031250, -0.0028076171875000,
+0.0012512207031250, -0.0025024414062500, 0.0004272460937500, -0.0027160644531250,
+0.0016174316406250, -0.0032958984375000, 0.0047302246093750, -0.0031127929687500,
+0.0033569335937500, -0.0022888183593750, 0.0051574707031250, -0.0028686523437500,
+0.0054626464843750, -0.0024414062500000, 0.0056152343750000, -0.0012207031250000,
+0.0061340332031250, -0.0017089843750000, 0.0057678222656250, -0.0013427734375000,
+0.0082397460937500, -0.0018310546875000, 0.0074768066406250, -0.0007019042968750,
+0.0083007812500000, 0.0002746582031250, 0.0108337402343750, -0.0006408691406250,
+0.0096130371093750, 0.0027770996093750, 0.0098571777343750, 0.0021972656250000,
+0.0109558105468750, 0.0030822753906250, 0.0093994140625000, 0.0076599121093750,
+0.0063781738281250, 0.0087890625000000, 0.0080871582031250, 0.0057067871093750,
+0.0090026855468750, 0.0067749023437500, 0.0100708007812500, 0.0063476562500000,
+0.0084533691406250, 0.0051879882812500, 0.0068054199218750, 0.0055236816406250,
+0.0080566406250000, 0.0037231445312500, 0.0069580078125000, 0.0049438476562500,
+0.0060729980468750, 0.0032653808593750, 0.0067749023437500, 0.0008239746093750,
+0.0079040527343750, 0.0026550292968750, 0.0074157714843750, -0.0019836425781250,
+0.0054321289062500, -0.0021362304687500, 0.0070800781250000, -0.0012512207031250,
+0.0047607421875000, -0.0031738281250000, 0.0010986328125000, -0.0032043457031250,
+0.0033874511718750, -0.0014953613281250, -0.0000610351562500, -0.0018920898437500,
+-0.0008544921875000, -0.0017089843750000, -0.0003662109375000, -0.0003356933593750,
+-0.0012207031250000, -0.0005798339843750, -0.0010375976562500, -0.0015869140625000,
+-0.0007629394531250, -0.0012512207031250, -0.0006408691406250, -0.0006713867187500,
+-0.0011901855468750, -0.0014038085937500, -0.0015563964843750, -0.0016784667968750,
+-0.0010986328125000, 0.0010986328125000, -0.0024414062500000, 0.0006103515625000,
+-0.0025939941406250, 0.0008239746093750, -0.0022888183593750, 0.0029907226562500,
+-0.0027160644531250, 0.0025329589843750, -0.0028076171875000, 0.0012817382812500,
+-0.0025024414062500, 0.0024108886718750, -0.0024414062500000, 0.0006103515625000,
+-0.0027160644531250, -0.0017395019531250, -0.0028686523437500, 0.0000305175781250,
+-0.0027160644531250, -0.0034179687500000, -0.0027160644531250, -0.0038146972656250,
+-0.0031433105468750, -0.0032348632812500, -0.0025634765625000, -0.0045471191406250,
+-0.0018920898437500, -0.0043945312500000, -0.0023803710937500, -0.0031127929687500,
+-0.0012817382812500, -0.0040283203125000, -0.0013122558593750, -0.0026245117187500,
+-0.0013122558593750, -0.0006713867187500, -0.0008544921875000, -0.0023803710937500,
+-0.0009765625000000, 0.0003051757812500, -0.0007934570312500, 0.0010986328125000,
+-0.0010681152343750, -0.0003051757812500, -0.0004577636718750, -0.0000610351562500,
+0.0001525878906250, 0.0007324218750000, -0.0004577636718750, -0.0021362304687500,
+0.0006713867187500, -0.0016784667968750, 0.0008850097656250, -0.0018310546875000,
+0.0005187988281250, -0.0036621093750000, 0.0005798339843750, -0.0028686523437500,
+0.0006408691406250, -0.0032348632812500, 0.0007629394531250, -0.0035705566406250,
+0.0003662109375000, -0.0029602050781250, 0.0009460449218750, -0.0028686523437500,
+0.0016479492187500, -0.0032348632812500, 0.0011901855468750, -0.0010070800781250,
+0.0021972656250000, -0.0018615722656250, 0.0021972656250000, -0.0005798339843750,
+0.0023193359375000, 0.0017089843750000, 0.0028686523437500, 0.0000915527343750,
+0.0025329589843750, 0.0028076171875000, 0.0032653808593750, 0.0028076171875000,
+0.0032043457031250, 0.0025939941406250, 0.0032653808593750, 0.0039672851562500,
+0.0037841796875000, 0.0035400390625000, 0.0036010742187500, 0.0032348632812500,
+0.0034179687500000, 0.0040588378906250, 0.0035095214843750, 0.0031738281250000,
+0.0033264160156250, 0.0024414062500000, 0.0030822753906250, 0.0035705566406250,
+0.0032043457031250, 0.0009155273437500, 0.0026855468750000, 0.0019531250000000,
+0.0028991699218750, 0.0013122558593750, 0.0024719238281250, -0.0007629394531250,
+0.0018615722656250, 0.0009765625000000, 0.0022583007812500, -0.0018310546875000,
+0.0010986328125000, -0.0011901855468750, 0.0010070800781250, -0.0016784667968750,
+0.0011291503906250, -0.0037841796875000, 0.0006103515625000, -0.0024108886718750,
+0.0007019042968750, -0.0043029785156250, 0.0011291503906250, -0.0040588378906250,
+0.0010070800781250, -0.0040588378906250, 0.0011901855468750, -0.0050048828125000,
+0.0014648437500000, -0.0041198730468750, 0.0013732910156250, -0.0050659179687500,
+0.0017089843750000, -0.0046691894531250, 0.0016784667968750, -0.0046081542968750,
+0.0017395019531250, -0.0051879882812500, 0.0020141601562500, -0.0044555664062500,
+0.0018615722656250, -0.0052490234375000, 0.0017700195312500, -0.0049133300781250,
+0.0019226074218750, -0.0047607421875000, 0.0017089843750000, -0.0052795410156250,
+0.0014648437500000, -0.0048217773437500, 0.0016479492187500, -0.0048522949218750,
+0.0009765625000000, -0.0047607421875000, 0.0011291503906250, -0.0046081542968750,
+0.0010070800781250, -0.0045471191406250, 0.0003662109375000, -0.0042419433593750,
+0.0007324218750000, -0.0042114257812500, 0.0008239746093750, -0.0043029785156250,
+0.0005798339843750, -0.0037536621093750, 0.0008239746093750, -0.0033874511718750,
+0.0011901855468750, -0.0034790039062500, 0.0010986328125000, -0.0018615722656250,
+0.0015563964843750, -0.0021972656250000, 0.0011901855468750, -0.0015258789062500,
+0.0020141601562500, -0.0001220703125000, 0.0027770996093750, -0.0006713867187500,
+0.0018920898437500, 0.0009460449218750, 0.0039367675781250, 0.0011291503906250,
+0.0036315917968750, 0.0015258789062500, 0.0039978027343750, 0.0026245117187500,
+0.0052795410156250, 0.0024414062500000, 0.0044555664062500, 0.0031127929687500,
+0.0056152343750000, 0.0032348632812500, 0.0059204101562500, 0.0029602050781250,
+0.0051269531250000, 0.0029907226562500, 0.0051879882812500, 0.0030212402343750,
+0.0056152343750000, 0.0022583007812500, 0.0037231445312500, 0.0022888183593750,
+0.0039062500000000, 0.0020751953125000, 0.0036621093750000, 0.0014038085937500,
+0.0025024414062500, 0.0015258789062500, 0.0029296875000000, 0.0015563964843750,
+0.0021362304687500, 0.0014038085937500, 0.0021362304687500, 0.0016479492187500,
+0.0020751953125000, 0.0019226074218750, 0.0014648437500000, 0.0018920898437500,
+0.0015869140625000, 0.0023498535156250, 0.0017700195312500, 0.0026855468750000,
+0.0015563964843750, 0.0024108886718750, 0.0017089843750000, 0.0025329589843750,
+0.0018920898437500, 0.0028381347656250, 0.0018005371093750, 0.0014038085937500,
+0.0027770996093750, 0.0019531250000000, 0.0024108886718750, 0.0010375976562500,
+0.0026550292968750, -0.0005798339843750, 0.0036010742187500, 0.0003356933593750,
+0.0032348632812500, -0.0019836425781250, 0.0032958984375000, -0.0019836425781250,
+0.0036621093750000, -0.0021362304687500, 0.0029907226562500, -0.0036926269531250,
+0.0026245117187500, -0.0031433105468750, 0.0029907226562500, -0.0021667480468750,
+0.0006103515625000, -0.0032348632812500, 0.0015258789062500, -0.0020141601562500,
+0.0001831054687500, 0.0001525878906250, -0.0025939941406250, -0.0011596679687500,
+-0.0009765625000000, -0.0000915527343750, -0.0035095214843750, 0.0005187988281250,
+-0.0036926269531250, 0.0000610351562500, -0.0038146972656250, -0.0002441406250000,
+-0.0049743652343750, 0.0000000000000000, -0.0042114257812500, 0.0002746582031250,
+-0.0047912597656250, -0.0002136230468750, -0.0057983398437500, 0.0009460449218750,
+-0.0042114257812500, 0.0019836425781250, -0.0027160644531250, 0.0011901855468750,
+-0.0040893554687500, 0.0046386718750000, -0.0022888183593750, 0.0046386718750000,
+-0.0018615722656250, 0.0045776367187500, -0.0023498535156250, 0.0066833496093750,
+-0.0021362304687500, 0.0062866210937500, -0.0021972656250000, 0.0046081542968750,
+-0.0026245117187500, 0.0056762695312500, -0.0025329589843750, 0.0043334960937500,
+-0.0026550292968750, 0.0018920898437500, -0.0032653808593750, 0.0030517578125000,
+-0.0032348632812500, 0.0010986328125000, -0.0015258789062500, 0.0009460449218750,
+-0.0023193359375000, 0.0013122558593750, -0.0018310546875000, 0.0010070800781250,
+0.0003051757812500, 0.0012817382812500, -0.0000915527343750, 0.0006713867187500,
+-0.0003662109375000, 0.0013122558593750, 0.0000915527343750, 0.0007629394531250,
+-0.0000915527343750, -0.0005187988281250, -0.0007324218750000, 0.0002136230468750,
+-0.0006408691406250, -0.0007019042968750, -0.0006103515625000, -0.0008239746093750,
+-0.0005798339843750, -0.0010681152343750, -0.0005798339843750, -0.0014953613281250,
+-0.0008239746093750, -0.0009155273437500, -0.0007629394531250, -0.0024414062500000,
+0.0001525878906250, -0.0021972656250000, -0.0004272460937500, -0.0022277832031250,
+0.0007019042968750, -0.0033874511718750, 0.0023803710937500, -0.0030517578125000,
+0.0014038085937500, -0.0018005371093750, 0.0035095214843750, -0.0022888183593750,
+0.0039062500000000, -0.0023193359375000, 0.0036621093750000, -0.0012207031250000,
+0.0045166015625000, -0.0014038085937500, 0.0043029785156250, -0.0032653808593750,
+0.0026245117187500, -0.0025939941406250, 0.0034790039062500, -0.0033264160156250,
+0.0026550292968750, -0.0052185058593750, 0.0004577636718750, -0.0044250488281250,
+0.0014648437500000, -0.0054931640625000, 0.0008850097656250, -0.0057373046875000,
+0.0003662109375000, -0.0056457519531250, 0.0003967285156250, -0.0060424804687500,
+0.0005798339843750, -0.0058288574218750, 0.0004272460937500, -0.0056152343750000,
+-0.0008850097656250, -0.0063476562500000, -0.0005187988281250, -0.0050354003906250,
+-0.0011596679687500, -0.0039672851562500, -0.0028076171875000, -0.0051269531250000,
+-0.0025329589843750, -0.0012512207031250, -0.0020141601562500, -0.0018920898437500,
+-0.0032348632812500, -0.0013122558593750, -0.0017089843750000, 0.0014343261718750,
+0.0001525878906250, 0.0002441406250000, -0.0012817382812500, 0.0017089843750000,
+0.0020751953125000, 0.0020751953125000, 0.0019836425781250, 0.0017395019531250,
+0.0021362304687500, 0.0021362304687500, 0.0039978027343750, 0.0020751953125000,
+0.0032653808593750, 0.0018920898437500, 0.0032958984375000, 0.0020751953125000,
+0.0037536621093750, 0.0018310546875000, 0.0028686523437500, 0.0014038085937500,
+0.0020446777343750, 0.0016174316406250, 0.0026245117187500, 0.0014038085937500,
+0.0013732910156250, 0.0012817382812500, 0.0010986328125000, 0.0013732910156250,
+0.0016784667968750, 0.0013427734375000, 0.0015563964843750, 0.0012512207031250,
+0.0012817382812500, 0.0016174316406250, 0.0026550292968750, 0.0015869140625000,
+0.0023803710937500, 0.0015258789062500, 0.0026855468750000, 0.0017089843750000,
+0.0036621093750000, 0.0018005371093750, 0.0029602050781250, 0.0017395019531250,
+0.0032043457031250, 0.0016174316406250, 0.0031127929687500, 0.0018310546875000,
+0.0026550292968750, 0.0020141601562500, 0.0024414062500000, 0.0017700195312500,
+0.0022277832031250},
+{-0.0008850097656250, 0.0005798339843750, -0.0006408691406250, 0.0003967285156250,
+-0.0029296875000000, -0.0001525878906250, -0.0026550292968750, 0.0011596679687500,
+-0.0027770996093750, 0.0021667480468750, -0.0043029785156250, 0.0013122558593750,
+-0.0038757324218750, 0.0043640136718750, -0.0034484863281250, 0.0047302246093750,
+-0.0044250488281250, 0.0038452148437500, -0.0029602050781250, 0.0049133300781250,
+-0.0012207031250000, 0.0053100585937500, -0.0028076171875000, 0.0024108886718750,
+-0.0002746582031250, 0.0030212402343750, -0.0000610351562500, 0.0023193359375000,
+-0.0006408691406250, -0.0003051757812500, 0.0000610351562500, 0.0004882812500000,
+-0.0002746582031250, 0.0008544921875000, -0.0015563964843750, 0.0003967285156250,
+-0.0014648437500000, 0.0000305175781250, -0.0012207031250000, 0.0004577636718750,
+-0.0017395019531250, 0.0007324218750000, -0.0018310546875000, -0.0013122558593750,
+-0.0011291503906250, -0.0009765625000000, -0.0018005371093750, -0.0009765625000000,
+-0.0005187988281250, -0.0023193359375000, 0.0010070800781250, -0.0019836425781250,
+-0.0001831054687500, -0.0017700195312500, 0.0030212402343750, -0.0017700195312500,
+0.0032958984375000, -0.0021667480468750, 0.0028686523437500, -0.0021972656250000,
+0.0042724609375000, -0.0015869140625000, 0.0043945312500000, -0.0025329589843750,
+0.0024719238281250, -0.0028991699218750, 0.0030212402343750, -0.0019226074218750,
+0.0022277832031250, -0.0015563964843750, 0.0004882812500000, -0.0022583007812500,
+0.0014038085937500, -0.0006408691406250, 0.0005798339843750, -0.0003356933593750,
+-0.0002136230468750, -0.0009765625000000, 0.0005493164062500, -0.0006103515625000,
+0.0012207031250000, -0.0004272460937500, 0.0003051757812500, -0.0025634765625000,
+0.0003051757812500, -0.0019226074218750, 0.0008239746093750, -0.0025939941406250,
+-0.0003356933593750, -0.0046081542968750, -0.0016784667968750, -0.0038146972656250,
+-0.0010681152343750, -0.0052185058593750, -0.0019836425781250, -0.0052490234375000,
+-0.0030822753906250, -0.0049743652343750, -0.0019531250000000, -0.0056762695312500,
+-0.0010681152343750, -0.0056152343750000, -0.0020446777343750, -0.0044250488281250,
+0.0004882812500000, -0.0051879882812500, 0.0005798339843750, -0.0038757324218750,
+0.0004882812500000, -0.0021667480468750, 0.0017089843750000, -0.0034484863281250,
+0.0013122558593750, -0.0003356933593750, 0.0007324218750000, -0.0006713867187500,
+0.0010986328125000, -0.0002441406250000, 0.0005798339843750, 0.0017700195312500,
+-0.0003051757812500, 0.0007934570312500, 0.0002136230468750, 0.0017395019531250,
+-0.0002746582031250, 0.0019531250000000, -0.0005493164062500, 0.0016479492187500,
+0.0001525878906250, 0.0019531250000000, 0.0005798339843750, 0.0020141601562500,
+0.0003356933593750, 0.0013732910156250, 0.0015869140625000, 0.0014038085937500,
+0.0015258789062500, 0.0014953613281250, 0.0015258789062500, 0.0012512207031250,
+0.0021362304687500, 0.0012207031250000, 0.0015563964843750, 0.0011291503906250,
+0.0012512207031250, 0.0012512207031250, 0.0012207031250000, 0.0011291503906250,
+0.0005187988281250, 0.0010375976562500, -0.0000915527343750, 0.0012512207031250,
+0.0000915527343750, 0.0004272460937500, -0.0009460449218750, 0.0006103515625000,
+-0.0009460449218750, 0.0006103515625000, -0.0011596679687500, 0.0000915527343750,
+-0.0017089843750000, 0.0002441406250000, -0.0013427734375000, 0.0002746582031250,
+-0.0018920898437500, 0.0000610351562500, -0.0014648437500000, 0.0006408691406250,
+-0.0016174316406250, 0.0010375976562500, -0.0021057128906250, 0.0006103515625000,
+-0.0014953613281250, 0.0021667480468750, -0.0021667480468750, 0.0021057128906250,
+-0.0017700195312500, 0.0023193359375000, -0.0015563964843750, 0.0033264160156250,
+-0.0019836425781250, 0.0030822753906250, -0.0015869140625000, 0.0030517578125000,
+-0.0015258789062500, 0.0035400390625000, -0.0017700195312500, 0.0031433105468750,
+-0.0017395019531250, 0.0025024414062500, -0.0018005371093750, 0.0030212402343750,
+-0.0023498535156250, 0.0024719238281250, -0.0018310546875000, 0.0024414062500000,
+-0.0021362304687500, 0.0025939941406250, -0.0021057128906250, 0.0025024414062500,
+-0.0017395019531250, 0.0026245117187500, -0.0021972656250000, 0.0024719238281250,
+-0.0017700195312500, 0.0025634765625000, -0.0021972656250000, 0.0026855468750000,
+-0.0020141601562500, 0.0026245117187500, -0.0015258789062500, 0.0025939941406250,
+-0.0021362304687500, 0.0027160644531250, -0.0015869140625000, 0.0028076171875000,
+-0.0015869140625000, 0.0027770996093750, -0.0016174316406250, 0.0027160644531250,
+-0.0013122558593750, 0.0028076171875000, -0.0014343261718750, 0.0031127929687500,
+-0.0017089843750000, 0.0028381347656250, -0.0014038085937500, 0.0032043457031250,
+-0.0013732910156250, 0.0037536621093750, -0.0015258789062500, 0.0034790039062500,
+-0.0012817382812500, 0.0040893554687500, -0.0012512207031250, 0.0042724609375000,
+-0.0012512207031250, 0.0042419433593750, -0.0011901855468750, 0.0043945312500000,
+-0.0011596679687500, 0.0044555664062500, -0.0013122558593750, 0.0043945312500000,
+-0.0016174316406250, 0.0044250488281250, -0.0014343261718750, 0.0042724609375000,
+-0.0018310546875000, 0.0041198730468750, -0.0023498535156250, 0.0041809082031250,
+-0.0020751953125000, 0.0035705566406250, -0.0032958984375000, 0.0035400390625000,
+-0.0032043457031250, 0.0034484863281250, -0.0033569335937500, 0.0030212402343750,
+-0.0042114257812500, 0.0030212402343750, -0.0039062500000000, 0.0028381347656250,
+-0.0039062500000000, 0.0028686523437500, -0.0045471191406250, 0.0026245117187500,
+-0.0039978027343750, 0.0023498535156250, -0.0035400390625000, 0.0024414062500000,
+-0.0044555664062500, 0.0019226074218750, -0.0024108886718750, 0.0018615722656250,
+-0.0031433105468750, 0.0019226074218750, -0.0024414062500000, 0.0017700195312500,
+-0.0006103515625000, 0.0016479492187500, -0.0018920898437500, 0.0015869140625000,
+0.0000915527343750, 0.0018310546875000, 0.0000610351562500, 0.0011901855468750,
+-0.0003051757812500, 0.0007934570312500, 0.0006103515625000, 0.0012207031250000,
+0.0002136230468750, -0.0003356933593750, -0.0011596679687500, -0.0002136230468750,
+-0.0005493164062500, -0.0004577636718750, -0.0014953613281250, -0.0015563964843750,
+-0.0033264160156250, -0.0011596679687500, -0.0025329589843750, -0.0018310546875000,
+-0.0040283203125000, -0.0020141601562500, -0.0043945312500000, -0.0019226074218750,
+-0.0039978027343750, -0.0022277832031250, -0.0044555664062500, -0.0020446777343750,
+-0.0045776367187500, -0.0017395019531250, -0.0028381347656250, -0.0020141601562500,
+-0.0034179687500000, -0.0014038085937500, -0.0026855468750000, -0.0006408691406250,
+-0.0009460449218750, -0.0011901855468750, -0.0015869140625000, -0.0009765625000000,
+-0.0004577636718750, -0.0003356933593750, 0.0001220703125000, -0.0012512207031250,
+0.0000000000000000, -0.0022277832031250, 0.0001831054687500, -0.0017395019531250,
+0.0004272460937500, -0.0028991699218750, -0.0000610351562500, -0.0032348632812500,
+0.0000000000000000, -0.0029602050781250, -0.0000305175781250, -0.0034484863281250,
+-0.0004882812500000, -0.0037536621093750, -0.0003967285156250, -0.0018920898437500,
+0.0001831054687500, -0.0021362304687500, -0.0002136230468750, -0.0022277832031250,
+0.0000915527343750, -0.0011596679687500, 0.0010681152343750, -0.0014038085937500,
+0.0006713867187500, -0.0010375976562500, 0.0003051757812500, -0.0016174316406250,
+0.0011901855468750, -0.0002746582031250, -0.0007324218750000, 0.0008239746093750,
+-0.0022888183593750, -0.0003356933593750, -0.0004882812500000, 0.0037536621093750,
+-0.0050048828125000, 0.0029296875000000, -0.0053405761718750, 0.0045471191406250,
+-0.0044250488281250, 0.0083923339843750, -0.0065917968750000, 0.0063476562500000,
+-0.0064086914062500, 0.0075683593750000, -0.0011596679687500, 0.0093383789062500,
+-0.0030212402343750, 0.0075378417968750, -0.0007019042968750, 0.0058593750000000,
+0.0050048828125000, 0.0072021484375000, 0.0025939941406250, 0.0039062500000000,
+0.0049743652343750, 0.0042114257812500, 0.0064086914062500, 0.0041503906250000,
+0.0052795410156250, 0.0012512207031250, 0.0044250488281250, -0.0013427734375000,
+0.0020141601562500, -0.0021057128906250, 0.0036315917968750, -0.0020751953125000,
+0.0046691894531250, -0.0016479492187500, 0.0037231445312500, -0.0016479492187500,
+0.0022583007812500, -0.0020751953125000, 0.0028686523437500, -0.0018920898437500,
+0.0044250488281250, -0.0010070800781250, 0.0035400390625000, -0.0021972656250000,
+0.0044250488281250, -0.0035400390625000, 0.0067749023437500, -0.0025024414062500,
+0.0059204101562500, -0.0044250488281250, 0.0056152343750000, -0.0051574707031250,
+0.0068359375000000, -0.0034790039062500, 0.0056457519531250, -0.0034484863281250,
+0.0036315917968750, -0.0047302246093750, 0.0046997070312500, -0.0002746582031250,
+0.0042724609375000, -0.0006713867187500, 0.0033874511718750, -0.0006408691406250,
+0.0042419433593750, 0.0022583007812500, 0.0053710937500000, 0.0013122558593750,
+0.0046691894531250, -0.0005493164062500, 0.0047912597656250, 0.0010070800781250,
+0.0058593750000000, -0.0007629394531250, 0.0038757324218750, -0.0040283203125000,
+0.0019531250000000, -0.0022277832031250, 0.0034790039062500, -0.0037231445312500,
+0.0002441406250000, -0.0050048828125000, -0.0003051757812500, -0.0034790039062500,
+0.0001831054687500, -0.0025634765625000, -0.0007934570312500, -0.0035705566406250,
+-0.0008850097656250, -0.0001831054687500, -0.0005187988281250, -0.0003967285156250,
+-0.0002441406250000, -0.0004272460937500, -0.0005187988281250, 0.0016174316406250,
+-0.0007629394531250, 0.0011596679687500, -0.0005493164062500, 0.0000915527343750,
+-0.0006408691406250, 0.0007019042968750, -0.0008850097656250, 0.0000305175781250,
+-0.0007324218750000, -0.0011291503906250, -0.0003662109375000, -0.0003356933593750,
+-0.0005798339843750, -0.0016479492187500, -0.0008850097656250, -0.0018005371093750,
+-0.0004882812500000, -0.0009155273437500, -0.0010986328125000, -0.0011596679687500,
+-0.0019531250000000, -0.0013732910156250, -0.0013122558593750, 0.0010681152343750,
+-0.0018310546875000, 0.0007934570312500, -0.0023803710937500, 0.0006103515625000,
+-0.0016784667968750, 0.0022277832031250, -0.0010070800781250, 0.0021972656250000,
+-0.0016174316406250, 0.0003356933593750, -0.0009765625000000, 0.0014343261718750,
+-0.0007019042968750, 0.0003051757812500, -0.0009460449218750, -0.0018310546875000,
+-0.0011291503906250, -0.0004577636718750, -0.0010681152343750, -0.0033874511718750,
+-0.0007019042968750, -0.0031433105468750, -0.0010070800781250, -0.0035400390625000,
+-0.0006713867187500, -0.0053100585937500, -0.0001220703125000, -0.0040588378906250,
+-0.0003662109375000, -0.0053405761718750, 0.0000000000000000, -0.0056457519531250,
+0.0000305175781250, -0.0048217773437500, 0.0001220703125000, -0.0048522949218750,
+0.0003356933593750, -0.0049133300781250, 0.0002746582031250, -0.0036315917968750,
+0.0009765625000000, -0.0037536621093750, 0.0008239746093750, -0.0034790039062500,
+0.0010681152343750, -0.0025634765625000, 0.0017700195312500, -0.0028381347656250,
+0.0014648437500000, -0.0024108886718750, 0.0017700195312500, -0.0021667480468750,
+0.0018615722656250, -0.0023803710937500, 0.0018615722656250, -0.0024414062500000,
+0.0019226074218750, -0.0022888183593750, 0.0018920898437500, -0.0029296875000000,
+0.0022277832031250, -0.0027160644531250, 0.0020751953125000, -0.0029602050781250,
+0.0023803710937500, -0.0035095214843750, 0.0027465820312500, -0.0031433105468750,
+0.0025939941406250, -0.0038452148437500, 0.0032958984375000, -0.0036010742187500,
+0.0032348632812500},
+{-0.0010681152343750, -0.0014648437500000, -0.0034484863281250, -0.0023193359375000,
+-0.0018615722656250, -0.0026855468750000, 0.0017700195312500, -0.0017395019531250,
+0.0007324218750000, -0.0038452148437500, 0.0003967285156250, -0.0043029785156250,
+0.0010375976562500, -0.0033264160156250, 0.0011291503906250, -0.0038146972656250,
+0.0003356933593750, -0.0042419433593750, 0.0003967285156250, -0.0010986328125000,
+0.0028076171875000, -0.0018310546875000, 0.0021057128906250, -0.0019226074218750,
+0.0026245117187500, 0.0004272460937500, 0.0051574707031250, -0.0002136230468750,
+0.0045471191406250, -0.0020446777343750, 0.0032653808593750, -0.0011291503906250,
+0.0040893554687500, -0.0020141601562500, 0.0032653808593750, -0.0042419433593750,
+0.0014343261718750, -0.0032043457031250, 0.0022888183593750, -0.0026855468750000,
+0.0021362304687500, -0.0040283203125000, 0.0013427734375000, -0.0025329589843750,
+0.0023803710937500, -0.0003662109375000, 0.0037536621093750, -0.0017395019531250,
+0.0028686523437500, 0.0001220703125000, 0.0028991699218750, 0.0007934570312500,
+0.0039062500000000, -0.0000305175781250, 0.0025329589843750, 0.0000000000000000,
+0.0008850097656250, 0.0003662109375000, 0.0019226074218750, -0.0006103515625000,
+0.0000915527343750, -0.0006103515625000, -0.0005187988281250, -0.0002746582031250,
+0.0001525878906250, -0.0007629394531250, 0.0002136230468750, -0.0009460449218750,
+0.0000000000000000, 0.0003967285156250, 0.0001220703125000, 0.0003051757812500,
+0.0004272460937500, 0.0002746582031250, 0.0001831054687500, 0.0010070800781250,
+-0.0002746582031250, 0.0009155273437500, -0.0000915527343750, 0.0008544921875000,
+-0.0000610351562500, 0.0012512207031250, -0.0003051757812500, 0.0004577636718750,
+-0.0001525878906250, 0.0001831054687500, 0.0003662109375000, 0.0010375976562500,
+0.0002136230468750, -0.0023803710937500, -0.0006103515625000, -0.0015869140625000,
+-0.0001220703125000, -0.0023803710937500, -0.0005798339843750, -0.0052490234375000,
+-0.0018005371093750, -0.0038452148437500, -0.0013122558593750, -0.0048522949218750,
+-0.0010375976562500, -0.0053710937500000, -0.0015869140625000, -0.0046997070312500,
+-0.0011291503906250, -0.0045166015625000, -0.0003356933593750, -0.0047912597656250,
+-0.0006713867187500, -0.0038452148437500, -0.0005493164062500, -0.0037231445312500,
+-0.0004272460937500, -0.0036621093750000, -0.0004577636718750, -0.0031433105468750,
+-0.0005798339843750, -0.0031433105468750, -0.0005798339843750, -0.0032348632812500,
+-0.0001831054687500, -0.0028991699218750, -0.0003051757812500, -0.0034179687500000,
+-0.0001831054687500, -0.0039367675781250, 0.0002136230468750, -0.0033264160156250,
+0.0001220703125000, -0.0046997070312500, 0.0002136230468750, -0.0044555664062500,
+0.0002136230468750, -0.0046997070312500, 0.0004272460937500, -0.0055847167968750,
+0.0006103515625000, -0.0050659179687500, 0.0004272460937500, -0.0054626464843750,
+0.0012512207031250, -0.0058288574218750, 0.0011596679687500, -0.0053100585937500,
+0.0013122558593750, -0.0052490234375000, 0.0018920898437500, -0.0056762695312500,
+0.0017395019531250, -0.0043029785156250, 0.0020751953125000, -0.0049743652343750,
+0.0020141601562500, -0.0044250488281250, 0.0022583007812500, -0.0032653808593750,
+0.0025939941406250, -0.0044250488281250, 0.0024414062500000, -0.0024414062500000,
+0.0029602050781250, -0.0030212402343750, 0.0029602050781250, -0.0025634765625000,
+0.0030212402343750, -0.0010986328125000, 0.0033569335937500, -0.0020446777343750,
+0.0032348632812500, -0.0005187988281250, 0.0034484863281250, -0.0008850097656250,
+0.0035705566406250, -0.0008544921875000, 0.0033874511718750, 0.0000000000000000,
+0.0034179687500000, -0.0008544921875000, 0.0035400390625000, -0.0002441406250000,
+0.0031127929687500, -0.0004577636718750, 0.0030822753906250, -0.0006103515625000,
+0.0030822753906250, -0.0003356933593750, 0.0028076171875000, -0.0008544921875000,
+0.0029296875000000, -0.0005187988281250, 0.0029602050781250, -0.0007934570312500,
+0.0029296875000000, -0.0009460449218750, 0.0029602050781250, -0.0007629394531250,
+0.0031127929687500, -0.0011596679687500, 0.0029907226562500, -0.0010681152343750,
+0.0028991699218750, -0.0012512207031250, 0.0028991699218750, -0.0014953613281250,
+0.0029602050781250, -0.0015869140625000, 0.0027160644531250, -0.0018615722656250,
+0.0026550292968750, -0.0021972656250000, 0.0031738281250000, -0.0021667480468750,
+0.0028991699218750, -0.0026245117187500, 0.0031127929687500, -0.0031433105468750,
+0.0036621093750000, -0.0028991699218750, 0.0033874511718750, -0.0032653808593750,
+0.0036621093750000, -0.0033874511718750, 0.0039367675781250, -0.0033874511718750,
+0.0032653808593750, -0.0036315917968750, 0.0029907226562500, -0.0038146972656250,
+0.0035705566406250, -0.0034484863281250, 0.0014038085937500, -0.0039062500000000,
+0.0018920898437500, -0.0035400390625000, 0.0014343261718750, -0.0030212402343750,
+-0.0002441406250000, -0.0037231445312500, 0.0005493164062500, -0.0023498535156250,
+-0.0001831054687500, -0.0024414062500000, -0.0006713867187500, -0.0021667480468750,
+0.0000915527343750, -0.0011596679687500, 0.0003662109375000, -0.0016174316406250,
+-0.0000915527343750, -0.0011596679687500, 0.0012207031250000, -0.0005798339843750,
+0.0011901855468750, -0.0012512207031250, 0.0012817382812500, -0.0015258789062500,
+0.0019836425781250, -0.0009155273437500, 0.0017089843750000, -0.0028076171875000,
+0.0021057128906250, -0.0024719238281250, 0.0021667480468750, -0.0031127929687500,
+0.0021667480468750, -0.0046691894531250, 0.0025024414062500, -0.0039062500000000,
+0.0023498535156250, -0.0053405761718750, 0.0017395019531250, -0.0059204101562500,
+0.0023498535156250, -0.0053100585937500, 0.0014953613281250, -0.0053710937500000,
+0.0002746582031250, -0.0057373046875000, 0.0010375976562500, -0.0041503906250000,
+-0.0010070800781250, -0.0041809082031250, -0.0007629394531250, -0.0036010742187500,
+-0.0012512207031250, -0.0022583007812500, -0.0028076171875000, -0.0026550292968750,
+-0.0020446777343750, -0.0017089843750000, -0.0028686523437500, -0.0012512207031250,
+-0.0034484863281250, -0.0017395019531250, -0.0023498535156250, -0.0017395019531250,
+-0.0017395019531250, -0.0013427734375000, -0.0025634765625000, -0.0028381347656250,
+-0.0005187988281250, -0.0027160644531250, -0.0003662109375000, -0.0025024414062500,
+-0.0007324218750000, -0.0034484863281250, -0.0000305175781250, -0.0032348632812500,
+-0.0000305175781250, -0.0015258789062500, -0.0005798339843750, -0.0021972656250000,
+-0.0002136230468750, -0.0010681152343750, -0.0013122558593750, 0.0009765625000000,
+-0.0023498535156250, -0.0001525878906250, -0.0011901855468750, 0.0007934570312500,
+-0.0033874511718750, 0.0015258789062500, -0.0041198730468750, 0.0005493164062500,
+-0.0029602050781250, -0.0002746582031250, -0.0028991699218750, 0.0002441406250000,
+-0.0035705566406250, -0.0005493164062500, -0.0014953613281250, -0.0008239746093750,
+-0.0014953613281250, -0.0006408691406250, -0.0014648437500000, -0.0006408691406250,
+-0.0003967285156250, -0.0006713867187500, -0.0007629394531250, -0.0006713867187500,
+-0.0008850097656250, -0.0004577636718750, -0.0004272460937500, -0.0009155273437500,
+-0.0008850097656250, -0.0013122558593750, -0.0014648437500000, -0.0008239746093750,
+-0.0011291503906250, -0.0016174316406250, -0.0027770996093750, -0.0017089843750000,
+-0.0028381347656250, -0.0016784667968750, -0.0018920898437500, -0.0021362304687500,
+-0.0019836425781250, -0.0019836425781250, -0.0018005371093750, -0.0008544921875000,
+0.0007629394531250, -0.0012207031250000, -0.0002136230468750, -0.0012207031250000,
+0.0003356933593750, -0.0004272460937500, 0.0025634765625000, -0.0006103515625000,
+0.0014343261718750, -0.0001525878906250, 0.0020446777343750, -0.0001831054687500,
+0.0028381347656250, -0.0005187988281250, 0.0018615722656250, -0.0001831054687500,
+0.0009765625000000, 0.0000305175781250, 0.0016784667968750, -0.0010986328125000,
+0.0002441406250000, -0.0007324218750000, -0.0001525878906250, -0.0012207031250000,
+0.0002441406250000, -0.0022888183593750, -0.0001220703125000, -0.0016784667968750,
+-0.0003662109375000, -0.0026855468750000, 0.0005187988281250, -0.0028381347656250,
+0.0003051757812500, -0.0025634765625000, 0.0002441406250000, -0.0028991699218750,
+0.0008239746093750, -0.0028381347656250, 0.0004882812500000, -0.0015869140625000,
+-0.0000610351562500, -0.0018920898437500, 0.0001220703125000, -0.0016479492187500,
+-0.0002746582031250, -0.0006103515625000, -0.0007934570312500, -0.0008850097656250,
+-0.0004882812500000, -0.0007629394531250, -0.0013732910156250, -0.0007629394531250,
+-0.0011291503906250, -0.0004272460937500, -0.0013732910156250, -0.0000305175781250,
+-0.0023803710937500, -0.0003662109375000, -0.0021362304687500, -0.0003662109375000,
+-0.0021667480468750, 0.0002136230468750, -0.0030822753906250, -0.0005493164062500,
+-0.0024108886718750, -0.0012817382812500, -0.0018005371093750, -0.0006103515625000,
+-0.0030212402343750, -0.0029602050781250, -0.0008239746093750, -0.0026245117187500,
+-0.0011291503906250, -0.0028076171875000, -0.0010986328125000, -0.0043029785156250,
+0.0003967285156250, -0.0037231445312500, -0.0001525878906250, -0.0039672851562500,
+-0.0007629394531250, -0.0043334960937500, -0.0002746582031250, -0.0038452148437500,
+-0.0011291503906250, -0.0034790039062500, -0.0025939941406250, -0.0037231445312500,
+-0.0021972656250000, -0.0025634765625000, -0.0033569335937500, -0.0029296875000000,
+-0.0042114257812500, -0.0022277832031250, -0.0037841796875000, -0.0010375976562500,
+-0.0036926269531250, -0.0017089843750000, -0.0044860839843750, 0.0000000000000000,
+-0.0034179687500000, -0.0002136230468750, -0.0031433105468750, -0.0001220703125000,
+-0.0036315917968750, 0.0010986328125000, -0.0032043457031250, 0.0006408691406250,
+-0.0026855468750000, 0.0004882812500000, -0.0044555664062500, 0.0007324218750000,
+-0.0034484863281250, 0.0005187988281250, -0.0042724609375000, 0.0000305175781250,
+-0.0058898925781250, 0.0002136230468750, -0.0042419433593750, 0.0003662109375000,
+-0.0069580078125000, 0.0002441406250000, -0.0062866210937500, 0.0001831054687500,
+-0.0062255859375000, 0.0005798339843750, -0.0077514648437500, 0.0007934570312500,
+-0.0064392089843750, -0.0004272460937500, -0.0070800781250000, -0.0000305175781250,
+-0.0070495605468750, -0.0000305175781250, -0.0064697265625000, -0.0010681152343750,
+-0.0060729980468750, -0.0007019042968750, -0.0054016113281250, -0.0000610351562500,
+-0.0049438476562500, -0.0005798339843750, -0.0046081542968750, 0.0001831054687500,
+-0.0039672851562500, 0.0013122558593750, -0.0032958984375000, 0.0005493164062500,
+-0.0029602050781250, 0.0022583007812500, -0.0024414062500000, 0.0021972656250000,
+-0.0022277832031250, 0.0025634765625000, -0.0017089843750000, 0.0036315917968750,
+-0.0010375976562500, 0.0031433105468750, -0.0010681152343750, 0.0039672851562500,
+-0.0008544921875000, 0.0041198730468750, -0.0003662109375000, 0.0038757324218750,
+-0.0003662109375000, 0.0039978027343750, -0.0004272460937500, 0.0040588378906250,
+-0.0000915527343750, 0.0038146972656250, -0.0006408691406250, 0.0037231445312500,
+-0.0001220703125000, 0.0037841796875000, -0.0000915527343750, 0.0038452148437500,
+-0.0005187988281250, 0.0037536621093750, 0.0002136230468750, 0.0036926269531250,
+0.0001831054687500, 0.0038452148437500, 0.0004272460937500, 0.0036926269531250,
+0.0005798339843750, 0.0036010742187500, 0.0007934570312500, 0.0037231445312500,
+0.0013122558593750, 0.0032043457031250, 0.0010375976562500, 0.0034484863281250,
+0.0014038085937500, 0.0031127929687500, 0.0014648437500000, 0.0025329589843750,
+0.0012207031250000},
+{-0.0003051757812500, -0.0018005371093750, -0.0002746582031250, -0.0008544921875000,
+-0.0001525878906250, -0.0026855468750000, -0.0015869140625000, -0.0031433105468750,
+-0.0012817382812500, -0.0020751953125000, -0.0020141601562500, -0.0021362304687500,
+-0.0035095214843750, -0.0027465820312500, -0.0028381347656250, -0.0006713867187500,
+-0.0033874511718750, -0.0005187988281250, -0.0046386718750000, -0.0008544921875000,
+-0.0032653808593750, -0.0001220703125000, -0.0022888183593750, -0.0000610351562500,
+-0.0041809082031250, -0.0000915527343750, -0.0006408691406250, 0.0001220703125000,
+-0.0008239746093750, 0.0000000000000000, -0.0013427734375000, -0.0001220703125000,
+0.0004882812500000, 0.0001831054687500, 0.0002136230468750, -0.0004882812500000,
+-0.0008239746093750, 0.0000000000000000, 0.0002441406250000, -0.0007324218750000,
+-0.0007629394531250, -0.0018005371093750, -0.0026245117187500, -0.0010375976562500,
+-0.0015258789062500, -0.0025939941406250, -0.0029907226562500, -0.0025024414062500,
+-0.0036010742187500, -0.0024414062500000, -0.0029296875000000, -0.0032043457031250,
+-0.0028686523437500, -0.0028381347656250, -0.0032043457031250, -0.0031433105468750,
+-0.0022583007812500, -0.0030212402343750, -0.0017395019531250, -0.0028991699218750,
+-0.0022888183593750, -0.0030822753906250, -0.0019226074218750, -0.0029602050781250,
+-0.0010681152343750, -0.0029296875000000, -0.0036621093750000, -0.0029296875000000,
+-0.0023193359375000, -0.0024719238281250, -0.0030212402343750, -0.0022888183593750,
+-0.0052490234375000, -0.0026245117187500, -0.0031433105468750, -0.0011291503906250,
+-0.0052490234375000, -0.0014343261718750, -0.0045776367187500, -0.0011596679687500,
+-0.0039978027343750, -0.0000915527343750, -0.0047607421875000, -0.0006408691406250,
+-0.0034179687500000, 0.0000000000000000, -0.0035095214843750, -0.0000915527343750,
+-0.0029907226562500, 0.0001220703125000, -0.0024414062500000, 0.0006408691406250,
+-0.0021972656250000, 0.0000915527343750, -0.0016784667968750, 0.0007629394531250,
+-0.0016479492187500, 0.0010681152343750, -0.0014343261718750, 0.0005187988281250,
+-0.0014343261718750, 0.0003356933593750, -0.0016174316406250, 0.0008544921875000,
+-0.0013427734375000, 0.0000305175781250, -0.0017700195312500, -0.0000915527343750,
+-0.0019226074218750, 0.0003662109375000, -0.0020751953125000, 0.0002441406250000,
+-0.0025329589843750, 0.0000610351562500, -0.0025939941406250, 0.0012817382812500,
+-0.0026245117187500, 0.0009765625000000, -0.0031127929687500, 0.0013122558593750,
+-0.0032958984375000, 0.0022888183593750, -0.0032348632812500, 0.0018005371093750,
+-0.0034790039062500, 0.0026245117187500, -0.0036010742187500, 0.0025024414062500,
+-0.0036315917968750, 0.0026550292968750, -0.0036315917968750, 0.0032348632812500,
+-0.0038146972656250, 0.0028381347656250, -0.0040893554687500, 0.0034179687500000,
+-0.0039367675781250, 0.0033874511718750, -0.0039978027343750, 0.0034790039062500,
+-0.0037231445312500, 0.0037536621093750, -0.0034179687500000, 0.0035400390625000,
+-0.0037231445312500, 0.0036926269531250, -0.0032958984375000, 0.0038452148437500,
+-0.0034790039062500, 0.0034790039062500, -0.0035400390625000, 0.0032653808593750,
+-0.0034484863281250, 0.0034790039062500, -0.0038452148437500, 0.0024719238281250,
+-0.0034484863281250, 0.0026245117187500, -0.0039367675781250, 0.0021667480468750,
+-0.0039978027343750, 0.0013122558593750, -0.0036621093750000, 0.0017395019531250,
+-0.0042724609375000, 0.0005798339843750, -0.0035400390625000, 0.0004272460937500,
+-0.0042114257812500, 0.0004882812500000, -0.0038452148437500, -0.0000915527343750,
+-0.0030517578125000, -0.0000610351562500, -0.0041198730468750, -0.0000610351562500,
+-0.0025024414062500, -0.0000610351562500, -0.0025024414062500, -0.0003356933593750,
+-0.0021972656250000, -0.0005493164062500, -0.0010681152343750, -0.0003967285156250,
+-0.0012207031250000, -0.0007934570312500, -0.0004272460937500, -0.0010375976562500,
+-0.0001525878906250, -0.0009155273437500, 0.0001525878906250, -0.0009765625000000,
+0.0007934570312500, -0.0010681152343750, 0.0007934570312500, -0.0007019042968750,
+0.0007629394531250, -0.0008544921875000, 0.0014038085937500, -0.0006103515625000,
+0.0003967285156250, -0.0002441406250000, -0.0003356933593750, -0.0005187988281250,
+0.0006103515625000, 0.0006103515625000, -0.0026855468750000, 0.0004577636718750,
+-0.0016784667968750, 0.0005493164062500, -0.0025024414062500, 0.0013732910156250,
+-0.0051574707031250, 0.0010986328125000, -0.0033874511718750, 0.0009155273437500,
+-0.0054931640625000, 0.0013122558593750, -0.0056762695312500, 0.0008850097656250,
+-0.0049743652343750, 0.0004577636718750, -0.0056152343750000, 0.0009460449218750,
+-0.0054626464843750, -0.0002136230468750, -0.0046691894531250, 0.0001220703125000,
+-0.0045471191406250, -0.0002441406250000, -0.0043945312500000, -0.0013122558593750,
+-0.0038146972656250, -0.0006408691406250, -0.0034484863281250, -0.0013122558593750,
+-0.0035400390625000, -0.0015869140625000, -0.0028686523437500, -0.0012207031250000,
+-0.0035095214843750, -0.0011901855468750, -0.0041198730468750, -0.0013122558593750,
+-0.0030822753906250, -0.0009155273437500, -0.0050354003906250, -0.0011596679687500,
+-0.0051879882812500, -0.0009460449218750, -0.0043029785156250, -0.0006408691406250,
+-0.0052490234375000, -0.0009155273437500, -0.0055541992187500, 0.0001831054687500,
+-0.0017089843750000, -0.0002746582031250, -0.0026855468750000, 0.0006408691406250,
+-0.0012512207031250, 0.0021972656250000, 0.0023193359375000, 0.0012207031250000,
+0.0003967285156250, 0.0024414062500000, 0.0017700195312500, 0.0031127929687500,
+0.0028381347656250, 0.0022277832031250, 0.0005798339843750, 0.0019226074218750,
+-0.0004882812500000, 0.0023803710937500, 0.0008850097656250, 0.0002136230468750,
+-0.0040893554687500, 0.0007934570312500, -0.0035400390625000, -0.0002441406250000,
+-0.0034790039062500, -0.0024414062500000, -0.0066223144531250, -0.0011291503906250,
+-0.0055541992187500, -0.0025329589843750, -0.0043029785156250, -0.0033264160156250,
+-0.0056762695312500, -0.0028381347656250, -0.0037231445312500, -0.0023193359375000,
+-0.0009460449218750, -0.0025329589843750, -0.0026245117187500, -0.0039367675781250,
+0.0003051757812500, -0.0033874511718750, 0.0009765625000000, -0.0036926269531250,
+-0.0001831054687500, -0.0049133300781250, 0.0002746582031250, -0.0044555664062500,
+0.0006408691406250, -0.0046691894531250, -0.0018615722656250, -0.0050659179687500,
+-0.0017395019531250, -0.0042419433593750, -0.0013732910156250, -0.0035705566406250,
+-0.0025024414062500, -0.0041809082031250, -0.0022888183593750, -0.0030212402343750,
+-0.0014038085937500, -0.0025939941406250, -0.0014953613281250, -0.0030212402343750,
+-0.0019836425781250, -0.0030517578125000, -0.0017700195312500, -0.0041809082031250,
+-0.0040283203125000, -0.0035095214843750, -0.0012207031250000, -0.0038146972656250,
+-0.0013732910156250, -0.0038146972656250, -0.0026245117187500, -0.0034179687500000,
+-0.0023803710937500, -0.0036315917968750, -0.0014953613281250, -0.0043640136718750,
+-0.0049743652343750, -0.0046691894531250, -0.0051574707031250, -0.0039978027343750,
+-0.0044555664062500, -0.0040893554687500, -0.0059509277343750, -0.0046691894531250,
+-0.0060729980468750, -0.0015869140625000, -0.0030822753906250, -0.0023498535156250,
+-0.0037231445312500, -0.0012817382812500, -0.0033569335937500, 0.0019226074218750,
+-0.0011291503906250, 0.0006103515625000, -0.0017700195312500, 0.0005493164062500,
+-0.0009765625000000, 0.0021057128906250, -0.0010070800781250, 0.0001220703125000,
+-0.0007934570312500, -0.0021362304687500, -0.0003967285156250, -0.0003051757812500,
+-0.0006408691406250, -0.0030517578125000, 0.0006408691406250, -0.0040893554687500,
+0.0003051757812500, -0.0019836425781250, 0.0003051757812500, -0.0015258789062500,
+0.0015258789062500, -0.0027465820312500, 0.0013122558593750, 0.0014343261718750,
+-0.0000305175781250, 0.0014953613281250, 0.0003051757812500, 0.0008239746093750,
+0.0004882812500000, 0.0028991699218750, -0.0006103515625000, 0.0028076171875000,
+-0.0006103515625000, -0.0004577636718750, 0.0012512207031250, 0.0009460449218750,
+0.0007324218750000, -0.0003051757812500, 0.0011291503906250, -0.0036621093750000,
+0.0027770996093750, -0.0018005371093750, 0.0022277832031250, -0.0027160644531250,
+0.0020751953125000, -0.0035400390625000, 0.0024414062500000, -0.0028381347656250,
+0.0021362304687500, -0.0021972656250000, 0.0016784667968750, -0.0024414062500000,
+0.0018920898437500, -0.0020141601562500, 0.0016174316406250, -0.0017395019531250,
+0.0016174316406250, -0.0021972656250000, 0.0012817382812500, -0.0021057128906250,
+0.0010681152343750, -0.0016784667968750, 0.0013122558593750, -0.0038452148437500,
+0.0001220703125000, -0.0029602050781250, 0.0001220703125000, -0.0037231445312500,
+0.0002136230468750, -0.0060424804687500, -0.0004577636718750, -0.0046997070312500,
+-0.0004272460937500, -0.0053100585937500, -0.0000305175781250, -0.0061645507812500,
+-0.0000305175781250, -0.0049133300781250, -0.0003051757812500, -0.0041809082031250,
+-0.0002136230468750, -0.0050354003906250, -0.0000305175781250, -0.0014343261718750,
+-0.0007019042968750, -0.0024108886718750, -0.0007629394531250, -0.0009460449218750,
+-0.0005798339843750, 0.0020446777343750, -0.0007629394531250, -0.0000610351562500,
+-0.0008850097656250, 0.0041809082031250, -0.0002136230468750, 0.0039367675781250,
+-0.0003356933593750, 0.0036621093750000, -0.0003662109375000, 0.0058593750000000,
+0.0000610351562500, 0.0047912597656250, 0.0000305175781250, 0.0043029785156250,
+-0.0002441406250000, 0.0049133300781250, -0.0002746582031250, 0.0036315917968750,
+-0.0001220703125000, 0.0023803710937500, -0.0001525878906250, 0.0032043457031250,
+-0.0003051757812500, 0.0001220703125000, -0.0000610351562500, 0.0005493164062500,
+0.0000305175781250, -0.0003967285156250, -0.0000610351562500, -0.0026550292968750,
+-0.0001220703125000, -0.0012817382812500, -0.0000305175781250, -0.0037231445312500,
+0.0003967285156250, -0.0035095214843750, 0.0000610351562500, -0.0035705566406250,
+0.0005187988281250, -0.0050048828125000, 0.0013122558593750, -0.0043334960937500,
+0.0009155273437500, -0.0052490234375000, 0.0013732910156250, -0.0054626464843750,
+0.0016479492187500, -0.0053710937500000, 0.0015869140625000, -0.0057983398437500,
+0.0015869140625000, -0.0059509277343750, 0.0016479492187500, -0.0057067871093750,
+0.0020751953125000, -0.0060424804687500, 0.0019226074218750, -0.0058288574218750,
+0.0022277832031250, -0.0056457519531250, 0.0027465820312500, -0.0062561035156250,
+0.0025634765625000, -0.0054016113281250, 0.0031433105468750, -0.0058593750000000,
+0.0032043457031250, -0.0057373046875000, 0.0033569335937500, -0.0051269531250000,
+0.0037536621093750, -0.0058898925781250, 0.0036315917968750, -0.0050964355468750,
+0.0038146972656250, -0.0053710937500000, 0.0039062500000000, -0.0055236816406250,
+0.0039367675781250, -0.0051574707031250, 0.0039062500000000, -0.0057373046875000,
+0.0038452148437500, -0.0055847167968750, 0.0040283203125000, -0.0058898925781250,
+0.0039978027343750, -0.0061950683593750, 0.0038452148437500, -0.0063171386718750,
+0.0039672851562500, -0.0066833496093750, 0.0040588378906250, -0.0067749023437500,
+0.0032653808593750, -0.0069885253906250, 0.0036926269531250, -0.0071411132812500,
+0.0030212402343750, -0.0071105957031250, 0.0020446777343750, -0.0073547363281250,
+0.0027770996093750, -0.0076904296875000, 0.0012207031250000, -0.0074768066406250,
+0.0013122558593750, -0.0070800781250000, 0.0012512207031250, -0.0069885253906250,
+0.0003967285156250, -0.0068054199218750, 0.0007324218750000, -0.0056762695312500,
+0.0003356933593750},
+{0.0009460449218750, -0.0018310546875000, 0.0013427734375000, -0.0007324218750000,
+0.0012817382812500, -0.0020446777343750, 0.0012512207031250, -0.0039978027343750,
+0.0014648437500000, -0.0026550292968750, 0.0015258789062500, -0.0053100585937500,
+0.0015258789062500, -0.0053710937500000, 0.0015869140625000, -0.0050048828125000,
+0.0016479492187500, -0.0063781738281250, 0.0016784667968750, -0.0059509277343750,
+0.0016174316406250, -0.0039978027343750, 0.0014648437500000, -0.0054016113281250,
+0.0017395019531250, -0.0032958984375000, 0.0011901855468750, -0.0003662109375000,
+0.0005798339843750, -0.0025939941406250, 0.0010375976562500, 0.0016174316406250,
+-0.0000915527343750, 0.0015563964843750, -0.0002746582031250, 0.0015563964843750,
+-0.0000610351562500, 0.0037231445312500, -0.0003662109375000, 0.0027770996093750,
+-0.0004577636718750, 0.0035400390625000, -0.0001525878906250, 0.0037231445312500,
+-0.0001220703125000, 0.0030212402343750, -0.0003662109375000, 0.0030212402343750,
+-0.0004272460937500, 0.0031433105468750, -0.0002746582031250, 0.0010070800781250,
+-0.0009460449218750, 0.0014648437500000, -0.0010070800781250, 0.0007934570312500,
+-0.0008850097656250, -0.0009765625000000, -0.0011901855468750, -0.0001525878906250,
+-0.0011291503906250, -0.0013732910156250, -0.0008544921875000, -0.0016174316406250,
+-0.0008850097656250, -0.0013427734375000, -0.0008544921875000, -0.0017700195312500,
+-0.0007324218750000, -0.0018615722656250, -0.0007324218750000, -0.0011596679687500,
+-0.0006408691406250, -0.0014343261718750, -0.0007629394531250, -0.0011901855468750,
+-0.0005798339843750, -0.0006103515625000, -0.0003051757812500, -0.0009765625000000,
+-0.0004577636718750, -0.0003051757812500, -0.0003967285156250, -0.0003967285156250,
+-0.0002746582031250, -0.0003662109375000, -0.0003051757812500, -0.0000610351562500,
+-0.0004272460937500, -0.0003356933593750, -0.0004272460937500, 0.0000000000000000,
+0.0001831054687500, -0.0000610351562500, -0.0000610351562500, -0.0003051757812500,
+0.0003356933593750, -0.0003051757812500, 0.0010986328125000, -0.0004272460937500,
+0.0007934570312500, -0.0008239746093750, 0.0013732910156250, -0.0006713867187500,
+0.0014648437500000, -0.0009765625000000, 0.0015869140625000, -0.0014038085937500,
+0.0018920898437500, -0.0010681152343750, 0.0018005371093750, -0.0018310546875000,
+0.0021667480468750, -0.0016784667968750, 0.0021667480468750, -0.0017089843750000,
+0.0023193359375000, -0.0021362304687500, 0.0025329589843750, -0.0016784667968750,
+0.0024719238281250, -0.0019531250000000, 0.0030517578125000, -0.0017700195312500,
+0.0029602050781250, -0.0015869140625000, 0.0031127929687500, -0.0017089843750000,
+0.0036926269531250, -0.0013732910156250, 0.0034790039062500, -0.0015258789062500,
+0.0034790039062500, -0.0013732910156250, 0.0039062500000000, -0.0012817382812500,
+0.0033264160156250, -0.0013427734375000, 0.0028381347656250, -0.0011291503906250,
+0.0033874511718750, -0.0013732910156250, 0.0019226074218750, -0.0012207031250000,
+0.0020751953125000, -0.0014038085937500, 0.0020141601562500, -0.0016784667968750,
+0.0011901855468750, -0.0014953613281250, 0.0015869140625000, -0.0022888183593750,
+0.0009765625000000, -0.0020751953125000, 0.0009460449218750, -0.0027160644531250,
+0.0010986328125000, -0.0034179687500000, 0.0007324218750000, -0.0028991699218750,
+0.0007934570312500, -0.0046997070312500, 0.0010070800781250, -0.0042419433593750,
+0.0010681152343750, -0.0046997070312500, 0.0007324218750000, -0.0062255859375000,
+0.0008239746093750, -0.0054626464843750, 0.0011291503906250, -0.0067138671875000,
+-0.0001220703125000, -0.0070190429687500, 0.0002441406250000, -0.0068054199218750,
+-0.0004272460937500, -0.0072326660156250, -0.0015563964843750, -0.0072937011718750,
+-0.0006408691406250, -0.0062255859375000, -0.0025329589843750, -0.0068054199218750,
+-0.0024719238281250, -0.0058593750000000, -0.0024414062500000, -0.0044555664062500,
+-0.0033569335937500, -0.0054321289062500, -0.0030212402343750, -0.0026855468750000,
+-0.0030822753906250, -0.0030517578125000, -0.0030822753906250, -0.0020141601562500,
+-0.0030822753906250, 0.0001831054687500, -0.0030517578125000, -0.0010375976562500,
+-0.0029907226562500, 0.0013427734375000, -0.0030822753906250, 0.0014953613281250,
+-0.0032043457031250, 0.0012512207031250, -0.0027770996093750, 0.0022277832031250,
+-0.0024719238281250, 0.0018615722656250, -0.0028076171875000, 0.0012817382812500,
+-0.0015869140625000, 0.0018310546875000, -0.0015258789062500, 0.0011596679687500,
+-0.0018005371093750, 0.0001831054687500, -0.0012512207031250, 0.0009460449218750,
+-0.0012207031250000, -0.0002441406250000, -0.0024108886718750, -0.0003662109375000,
+-0.0021362304687500, -0.0001831054687500, -0.0021667480468750, -0.0005798339843750,
+-0.0031433105468750, -0.0005493164062500, -0.0029602050781250, -0.0003967285156250,
+-0.0024108886718750, -0.0000610351562500, -0.0024108886718750, -0.0008239746093750,
+-0.0029296875000000, -0.0012207031250000, -0.0030517578125000, -0.0003051757812500,
+-0.0026245117187500, -0.0025024414062500, -0.0026550292968750, -0.0024108886718750,
+-0.0031127929687500, -0.0024108886718750, -0.0025634765625000, -0.0037841796875000,
+-0.0019836425781250, -0.0035705566406250, -0.0023803710937500, -0.0033874511718750,
+-0.0011901855468750, -0.0036926269531250, -0.0011596679687500, -0.0029602050781250,
+-0.0007324218750000, -0.0021667480468750, 0.0001831054687500, -0.0026855468750000,
+-0.0002136230468750, -0.0014953613281250, 0.0006408691406250, -0.0010681152343750,
+0.0009155273437500, -0.0015563964843750, 0.0005493164062500, -0.0015869140625000,
+0.0006408691406250, -0.0012512207031250, 0.0010375976562500, -0.0015869140625000,
+0.0000915527343750, -0.0019531250000000, 0.0001525878906250, -0.0015563964843750,
+-0.0001220703125000, -0.0011596679687500, -0.0004882812500000, -0.0014343261718750,
+0.0002136230468750, -0.0008239746093750, -0.0007629394531250, -0.0007629394531250,
+-0.0011596679687500, -0.0007934570312500, -0.0006408691406250, -0.0006103515625000,
+-0.0003356933593750, -0.0006713867187500, -0.0005187988281250, -0.0002746582031250,
+-0.0006408691406250, -0.0001525878906250, -0.0006713867187500, -0.0008544921875000,
+-0.0006103515625000, -0.0007629394531250, -0.0006103515625000, -0.0002441406250000,
+-0.0007629394531250, -0.0027160644531250, -0.0005493164062500, -0.0023193359375000,
+-0.0010070800781250, -0.0024719238281250, -0.0010375976562500, -0.0042419433593750,
+-0.0006713867187500, -0.0034179687500000, -0.0011596679687500, -0.0028686523437500,
+-0.0015563964843750, -0.0036926269531250, -0.0012512207031250, -0.0025939941406250,
+-0.0018920898437500, -0.0009155273437500, -0.0028381347656250, -0.0017700195312500,
+-0.0023498535156250, -0.0008850097656250, -0.0033874511718750, -0.0005493164062500,
+-0.0037841796875000, -0.0005493164062500, -0.0033569335937500, -0.0004882812500000,
+-0.0037231445312500, -0.0005187988281250, -0.0043029785156250, 0.0006103515625000,
+-0.0022583007812500, 0.0005493164062500, -0.0029602050781250, 0.0001831054687500,
+-0.0029296875000000, 0.0010986328125000, -0.0012817382812500, 0.0014648437500000,
+-0.0018615722656250, -0.0007934570312500, -0.0021057128906250, 0.0001220703125000,
+-0.0008850097656250, -0.0003356933593750, -0.0014343261718750, -0.0024414062500000,
+-0.0024108886718750, -0.0014953613281250, -0.0008850097656250, -0.0018615722656250,
+-0.0013427734375000, -0.0023498535156250, -0.0013122558593750, -0.0014343261718750,
+-0.0008239746093750, -0.0007629394531250, -0.0006713867187500, -0.0013732910156250,
+-0.0004577636718750, -0.0001831054687500, -0.0002441406250000, 0.0004577636718750,
+0.0002746582031250, -0.0001220703125000, -0.0000305175781250, -0.0001220703125000,
+-0.0000915527343750, 0.0003356933593750, 0.0009460449218750, -0.0011291503906250,
+-0.0007324218750000, -0.0007324218750000, 0.0003051757812500, -0.0008850097656250,
+0.0001220703125000, -0.0019531250000000, -0.0009765625000000, -0.0016174316406250,
+0.0007019042968750, -0.0022583007812500, -0.0009460449218750, -0.0022277832031250,
+0.0000000000000000, -0.0022888183593750, 0.0000305175781250, -0.0027160644531250,
+-0.0010681152343750, -0.0025939941406250, 0.0003662109375000, -0.0029602050781250,
+-0.0015563964843750, -0.0029602050781250, -0.0011291503906250, -0.0028991699218750,
+-0.0014648437500000, -0.0031433105468750, -0.0029907226562500, -0.0031127929687500,
+-0.0023193359375000, -0.0029907226562500, -0.0034790039062500, -0.0031127929687500,
+-0.0039367675781250, -0.0028991699218750, -0.0039062500000000, -0.0026855468750000,
+-0.0044250488281250, -0.0028381347656250, -0.0046997070312500, -0.0020446777343750,
+-0.0043334960937500, -0.0023193359375000, -0.0049438476562500, -0.0018005371093750,
+-0.0046081542968750, -0.0009460449218750, -0.0041198730468750, -0.0016174316406250,
+-0.0047607421875000, -0.0003662109375000, -0.0033874511718750, -0.0005493164062500,
+-0.0037536621093750, -0.0003662109375000, -0.0037841796875000, 0.0003662109375000,
+-0.0029907226562500, -0.0003662109375000, -0.0034179687500000, 0.0007324218750000,
+-0.0028991699218750, 0.0006408691406250, -0.0030822753906250, 0.0004577636718750,
+-0.0031738281250000, 0.0009155273437500, -0.0031433105468750, 0.0007019042968750,
+-0.0036010742187500, 0.0005798339843750, -0.0031433105468750, 0.0007019042968750,
+-0.0032958984375000, 0.0005798339843750, -0.0032958984375000, 0.0003662109375000,
+-0.0029907226562500, 0.0005798339843750, -0.0033569335937500, 0.0007629394531250,
+-0.0031127929687500, 0.0005493164062500, -0.0029907226562500, 0.0008544921875000,
+-0.0030212402343750, 0.0013732910156250, -0.0029907226562500, 0.0010986328125000,
+-0.0029907226562500, 0.0014648437500000, -0.0027465820312500, 0.0016174316406250,
+-0.0032958984375000, 0.0014038085937500, -0.0030212402343750, 0.0014343261718750,
+-0.0025939941406250, 0.0015563964843750, -0.0033264160156250, 0.0007934570312500,
+-0.0022888183593750, 0.0009460449218750, -0.0026550292968750, 0.0007629394531250,
+-0.0025939941406250, 0.0002136230468750, -0.0020141601562500, 0.0003967285156250,
+-0.0027160644531250, 0.0001525878906250, -0.0019226074218750, 0.0001220703125000,
+-0.0022583007812500, -0.0001220703125000, -0.0022277832031250, -0.0002746582031250,
+-0.0017089843750000, -0.0001220703125000, -0.0021972656250000, -0.0010681152343750,
+-0.0016479492187500, -0.0009765625000000, -0.0017395019531250, -0.0011291503906250,
+-0.0017395019531250, -0.0017700195312500, -0.0014343261718750, -0.0014343261718750,
+-0.0016479492187500, -0.0018005371093750, -0.0010375976562500, -0.0017395019531250,
+-0.0014953613281250, -0.0017395019531250, -0.0013122558593750, -0.0019836425781250,
+-0.0006713867187500, -0.0018005371093750, -0.0013427734375000, -0.0016784667968750,
+-0.0009155273437500, -0.0019531250000000, -0.0006408691406250, -0.0016174316406250,
+-0.0013122558593750, -0.0012512207031250, -0.0014038085937500, -0.0016174316406250,
+-0.0008544921875000, -0.0011901855468750, -0.0029602050781250, -0.0011596679687500,
+-0.0020751953125000, -0.0014038085937500, -0.0025634765625000, -0.0014648437500000,
+-0.0043640136718750, -0.0014953613281250, -0.0028991699218750, -0.0018310546875000,
+-0.0042114257812500, -0.0018920898437500, -0.0043945312500000, -0.0020446777343750,
+-0.0033874511718750, -0.0022277832031250, -0.0036315917968750, -0.0022277832031250,
+-0.0040893554687500, -0.0028686523437500, -0.0016479492187500, -0.0028381347656250,
+-0.0024108886718750, -0.0028076171875000, -0.0014343261718750, -0.0033569335937500,
+0.0008544921875000, -0.0034484863281250, -0.0003662109375000, -0.0021057128906250,
+0.0014648437500000, -0.0029296875000000, 0.0019531250000000, -0.0023498535156250,
+0.0012817382812500},
+{-0.0059204101562500, -0.0022583007812500, -0.0052185058593750, -0.0015869140625000,
+-0.0046386718750000, -0.0021362304687500, -0.0059204101562500, -0.0014038085937500,
+-0.0038452148437500, -0.0017089843750000, -0.0044250488281250, -0.0015563964843750,
+-0.0043334960937500, -0.0009155273437500, -0.0029907226562500, -0.0014343261718750,
+-0.0040283203125000, -0.0010070800781250, -0.0030212402343750, -0.0008850097656250,
+-0.0031433105468750, -0.0012207031250000, -0.0031738281250000, -0.0014038085937500,
+-0.0025329589843750, -0.0013732910156250, -0.0028991699218750, -0.0015563964843750,
+-0.0031127929687500, -0.0018310546875000, -0.0029602050781250, -0.0015869140625000,
+-0.0031433105468750, -0.0014953613281250, -0.0036315917968750, -0.0018920898437500,
+-0.0036621093750000, -0.0005493164062500, -0.0037841796875000, -0.0010375976562500,
+-0.0035095214843750, -0.0007324218750000, -0.0034179687500000, 0.0004882812500000,
+-0.0034179687500000, -0.0002441406250000, -0.0028686523437500, 0.0005493164062500,
+-0.0028991699218750, 0.0007019042968750, -0.0026550292968750, 0.0004272460937500,
+-0.0023193359375000, 0.0006103515625000, -0.0021972656250000, 0.0007324218750000,
+-0.0022888183593750, 0.0000610351562500, -0.0020751953125000, 0.0002441406250000,
+-0.0017700195312500, 0.0000305175781250, -0.0013732910156250, -0.0005798339843750,
+-0.0014343261718750, -0.0002136230468750, -0.0017089843750000, -0.0006713867187500,
+-0.0008239746093750, -0.0007629394531250, -0.0013427734375000, -0.0005798339843750,
+-0.0010681152343750, -0.0006408691406250, -0.0001831054687500, -0.0006713867187500,
+-0.0011596679687500, -0.0003356933593750, -0.0003356933593750, -0.0003967285156250,
+-0.0003662109375000, -0.0003967285156250, -0.0005493164062500, -0.0001525878906250,
+-0.0001525878906250, -0.0002136230468750, -0.0001220703125000, -0.0004882812500000,
+-0.0001831054687500, -0.0003662109375000, 0.0001525878906250, -0.0003967285156250,
+-0.0000305175781250, -0.0006713867187500, 0.0000000000000000, -0.0005798339843750,
+0.0006103515625000, -0.0006408691406250, -0.0004577636718750, -0.0006713867187500,
+0.0003356933593750, -0.0006713867187500, 0.0001525878906250, -0.0007019042968750,
+-0.0007324218750000, -0.0006408691406250, 0.0004882812500000, -0.0009155273437500,
+-0.0008239746093750, -0.0008850097656250, -0.0006408691406250, -0.0010986328125000,
+-0.0004577636718750, -0.0014648437500000, -0.0013427734375000, -0.0013122558593750,
+-0.0011596679687500, -0.0018005371093750, -0.0010070800781250, -0.0018005371093750,
+-0.0015869140625000, -0.0019836425781250, -0.0011596679687500, -0.0023193359375000,
+-0.0006408691406250, -0.0021362304687500, -0.0012512207031250, -0.0028381347656250,
+-0.0000915527343750, -0.0028076171875000, -0.0000915527343750, -0.0030212402343750,
+-0.0002746582031250, -0.0034179687500000, 0.0003967285156250, -0.0032348632812500,
+0.0004272460937500, -0.0037841796875000, -0.0001831054687500, -0.0039062500000000,
+0.0005493164062500, -0.0037536621093750, 0.0000305175781250, -0.0041503906250000,
+-0.0008850097656250, -0.0041198730468750, 0.0000305175781250, -0.0029602050781250,
+-0.0018310546875000, -0.0035705566406250, -0.0018005371093750, -0.0024719238281250,
+-0.0018920898437500, -0.0006713867187500, -0.0030822753906250, -0.0016784667968750,
+-0.0027465820312500, 0.0002441406250000, -0.0026245117187500, 0.0004272460937500,
+-0.0032958984375000, 0.0001220703125000, -0.0024719238281250, 0.0006713867187500,
+-0.0017700195312500, 0.0003662109375000, -0.0026855468750000, 0.0003356933593750,
+-0.0002441406250000, 0.0000610351562500, -0.0007629394531250, 0.0001220703125000,
+-0.0004882812500000, 0.0003356933593750, 0.0012512207031250, 0.0000000000000000,
+0.0000610351562500, 0.0000305175781250, -0.0000915527343750, 0.0002746582031250,
+0.0007324218750000, -0.0001220703125000, -0.0011901855468750, -0.0007629394531250,
+-0.0028686523437500, -0.0004272460937500, -0.0015563964843750, -0.0002746582031250,
+-0.0053710937500000, -0.0009765625000000, -0.0048522949218750, -0.0003051757812500,
+-0.0045471191406250, 0.0006713867187500, -0.0066223144531250, -0.0000305175781250,
+-0.0057067871093750, 0.0009155273437500, -0.0049133300781250, 0.0010986328125000,
+-0.0054626464843750, 0.0008239746093750, -0.0041809082031250, 0.0009460449218750,
+-0.0027160644531250, 0.0008544921875000, -0.0033569335937500, 0.0005187988281250,
+-0.0005798339843750, 0.0007934570312500, -0.0005798339843750, 0.0000305175781250,
+-0.0007629394531250, -0.0008544921875000, 0.0005187988281250, -0.0002136230468750,
+0.0002746582031250, -0.0009155273437500, 0.0001525878906250, -0.0012512207031250,
+0.0008544921875000, -0.0014648437500000, -0.0001831054687500, -0.0014038085937500,
+-0.0010986328125000, -0.0010986328125000, 0.0000000000000000, -0.0023193359375000,
+-0.0027160644531250, -0.0022583007812500, -0.0025634765625000, -0.0026245117187500,
+-0.0024719238281250, -0.0033569335937500, -0.0036621093750000, -0.0029296875000000,
+-0.0032348632812500, -0.0041198730468750, -0.0044250488281250, -0.0040588378906250,
+-0.0039672851562500, -0.0042419433593750, -0.0045166015625000, -0.0051574707031250,
+-0.0059814453125000, -0.0048217773437500, -0.0051879882812500, -0.0051879882812500,
+-0.0050048828125000, -0.0055541992187500, -0.0057983398437500, -0.0047912597656250,
+-0.0046081542968750, -0.0037231445312500, -0.0020141601562500, -0.0010681152343750,
+-0.0045166015625000, -0.0012817382812500, -0.0049133300781250, -0.0002746582031250,
+-0.0059204101562500, -0.0016784667968750, -0.0045471191406250, -0.0031433105468750,
+-0.0030517578125000, -0.0017089843750000, -0.0039367675781250, -0.0025329589843750,
+-0.0028686523437500, -0.0036621093750000, -0.0021667480468750, -0.0023498535156250,
+-0.0031127929687500, -0.0010986328125000, -0.0037231445312500, -0.0019836425781250,
+-0.0030517578125000, -0.0005798339843750, -0.0039062500000000, 0.0000610351562500,
+-0.0046691894531250, -0.0009155273437500, -0.0034484863281250, -0.0011596679687500,
+-0.0026245117187500, -0.0002746582031250, -0.0036010742187500, -0.0017089843750000,
+-0.0016174316406250, -0.0021362304687500, -0.0013122558593750, -0.0007629394531250,
+-0.0019226074218750, -0.0004882812500000, -0.0014343261718750, -0.0012512207031250,
+-0.0010681152343750, 0.0014953613281250, -0.0016784667968750, 0.0020141601562500,
+-0.0023803710937500, 0.0005187988281250, -0.0007019042968750, 0.0009460449218750,
+0.0005187988281250, 0.0018005371093750, -0.0007629394531250, -0.0028991699218750,
+0.0019226074218750, -0.0021972656250000, 0.0024719238281250, -0.0030822753906250,
+0.0018310546875000, -0.0069274902343750, 0.0023498535156250, -0.0057678222656250,
+0.0025024414062500, -0.0055847167968750, 0.0015258789062500, -0.0075378417968750,
+0.0015563964843750, -0.0052490234375000, 0.0017700195312500, -0.0027770996093750,
+0.0014038085937500, -0.0050964355468750, 0.0013427734375000, -0.0005187988281250,
+0.0020751953125000, -0.0003967285156250, 0.0019531250000000, -0.0005187988281250,
+0.0018615722656250, 0.0015869140625000, 0.0024414062500000, 0.0006103515625000,
+0.0024414062500000, 0.0008850097656250, 0.0012207031250000, 0.0012512207031250,
+0.0014343261718750, 0.0004577636718750, 0.0013122558593750, -0.0001831054687500,
+0.0003662109375000, 0.0003356933593750, 0.0005798339843750, -0.0004577636718750,
+0.0010375976562500, -0.0012817382812500, 0.0008239746093750, -0.0000610351562500,
+0.0010681152343750, 0.0005493164062500, 0.0015869140625000, -0.0007629394531250,
+0.0013427734375000, 0.0021972656250000, 0.0016174316406250, 0.0017395019531250,
+0.0016174316406250, 0.0018920898437500, 0.0015258789062500, 0.0036621093750000,
+0.0016479492187500, 0.0026245117187500, 0.0016479492187500, 0.0030822753906250,
+0.0008850097656250, 0.0033569335937500, 0.0012512207031250, 0.0025634765625000,
+0.0004577636718750, 0.0022888183593750, -0.0006103515625000, 0.0025634765625000,
+0.0000000000000000, 0.0001220703125000, -0.0013427734375000, 0.0007934570312500,
+-0.0014953613281250, -0.0002441406250000, -0.0014038085937500, -0.0024719238281250,
+-0.0018920898437500, -0.0011291503906250, -0.0018005371093750, -0.0032958984375000,
+-0.0018920898437500, -0.0032348632812500, -0.0019226074218750, -0.0031433105468750,
+-0.0018920898437500, -0.0041198730468750, -0.0020141601562500, -0.0034790039062500,
+-0.0020751953125000, -0.0039062500000000, -0.0016784667968750, -0.0038146972656250,
+-0.0018005371093750, -0.0033874511718750, -0.0018005371093750, -0.0032653808593750,
+-0.0015869140625000, -0.0029907226562500, -0.0016174316406250, -0.0026550292968750,
+-0.0016479492187500, -0.0024108886718750, -0.0017700195312500, -0.0021057128906250,
+-0.0015563964843750, -0.0018615722656250, -0.0013427734375000, -0.0016174316406250,
+-0.0015563964843750, -0.0010375976562500, -0.0010375976562500, -0.0009765625000000,
+-0.0009155273437500, -0.0007324218750000, -0.0009155273437500, -0.0003051757812500,
+-0.0007019042968750, -0.0003356933593750, -0.0006408691406250, -0.0000610351562500,
+-0.0003967285156250, 0.0000915527343750, -0.0005798339843750, 0.0000610351562500,
+-0.0002746582031250, 0.0000610351562500, 0.0002136230468750, 0.0002746582031250,
+-0.0000610351562500, 0.0000610351562500, 0.0005493164062500, 0.0001831054687500,
+0.0007019042968750, 0.0001525878906250, 0.0005493164062500, -0.0000610351562500,
+0.0007324218750000, 0.0001831054687500, 0.0008239746093750, -0.0000915527343750,
+0.0005798339843750, 0.0000305175781250, 0.0006713867187500, 0.0000610351562500,
+0.0007629394531250, -0.0000610351562500, 0.0006408691406250, 0.0002136230468750,
+0.0007019042968750, -0.0000305175781250, 0.0010681152343750, 0.0001831054687500,
+0.0010681152343750, 0.0001831054687500, 0.0010986328125000, 0.0000610351562500,
+0.0014648437500000, 0.0003051757812500, 0.0014038085937500, 0.0001220703125000,
+0.0011291503906250, 0.0002746582031250, 0.0013122558593750, 0.0000915527343750,
+0.0011596679687500, -0.0000915527343750, 0.0008239746093750, 0.0000915527343750,
+0.0010070800781250, -0.0012512207031250, 0.0005493164062500, -0.0007324218750000,
+0.0008239746093750, -0.0013427734375000, 0.0002746582031250, -0.0024414062500000,
+-0.0003967285156250, -0.0015563964843750, 0.0002746582031250, -0.0036926269531250,
+-0.0011901855468750, -0.0033569335937500, -0.0010070800781250, -0.0036621093750000,
+-0.0013427734375000, -0.0051269531250000, -0.0022888183593750, -0.0043334960937500,
+-0.0016174316406250, -0.0052490234375000, -0.0025939941406250, -0.0052490234375000,
+-0.0027770996093750, -0.0049743652343750, -0.0023498535156250, -0.0054016113281250,
+-0.0024414062500000, -0.0052795410156250, -0.0025939941406250, -0.0049133300781250,
+-0.0020751953125000, -0.0052795410156250, -0.0019531250000000, -0.0049743652343750,
+-0.0019531250000000, -0.0045166015625000, -0.0017395019531250, -0.0051269531250000,
+-0.0018310546875000, -0.0039978027343750, -0.0019226074218750, -0.0046691894531250,
+-0.0017700195312500, -0.0045776367187500, -0.0018310546875000, -0.0036926269531250,
+-0.0020446777343750, -0.0046691894531250, -0.0018615722656250, -0.0040283203125000,
+-0.0016479492187500, -0.0039978027343750, -0.0017395019531250, -0.0043029785156250,
+-0.0016479492187500, -0.0040588378906250, -0.0012817382812500, -0.0038757324218750,
+-0.0014038085937500, -0.0039062500000000, -0.0019531250000000, -0.0040588378906250,
+-0.0014953613281250, -0.0031738281250000, -0.0020751953125000, -0.0026855468750000,
+-0.0030822753906250, -0.0034790039062500, -0.0024719238281250, -0.0015258789062500,
+-0.0032958984375000, -0.0018310546875000, -0.0035400390625000, -0.0017089843750000,
+-0.0032958984375000, -0.0003662109375000, -0.0036621093750000, -0.0010986328125000,
+-0.0036315917968750},
+{-0.0013732910156250, -0.0011596679687500, -0.0014343261718750, -0.0018310546875000,
+-0.0017089843750000, -0.0011901855468750, -0.0016174316406250, -0.0014038085937500,
+-0.0015563964843750, -0.0019836425781250, -0.0018005371093750, -0.0009765625000000,
+-0.0017395019531250, -0.0025939941406250, -0.0015563964843750, -0.0021972656250000,
+-0.0014953613281250, -0.0023193359375000, -0.0015563964843750, -0.0034484863281250,
+-0.0015258789062500, -0.0027465820312500, -0.0014953613281250, -0.0037231445312500,
+-0.0014953613281250, -0.0037536621093750, -0.0015869140625000, -0.0038146972656250,
+-0.0013122558593750, -0.0043334960937500, -0.0010681152343750, -0.0042419433593750,
+-0.0012512207031250, -0.0044860839843750, -0.0007324218750000, -0.0046691894531250,
+-0.0006408691406250, -0.0046386718750000, -0.0006408691406250, -0.0046997070312500,
+-0.0003967285156250, -0.0049438476562500, -0.0003967285156250, -0.0048217773437500,
+-0.0003356933593750, -0.0049743652343750, -0.0002746582031250, -0.0049438476562500,
+-0.0002136230468750, -0.0049133300781250, -0.0001831054687500, -0.0051269531250000,
+-0.0001525878906250, -0.0048522949218750, 0.0000000000000000, -0.0050659179687500,
+0.0000000000000000, -0.0049743652343750, 0.0000305175781250, -0.0047607421875000,
+0.0002441406250000, -0.0051879882812500, 0.0002136230468750, -0.0046386718750000,
+0.0001525878906250, -0.0049438476562500, 0.0003356933593750, -0.0047912597656250,
+0.0001831054687500, -0.0042724609375000, 0.0000610351562500, -0.0045776367187500,
+0.0002746582031250, -0.0039672851562500, 0.0000915527343750, -0.0039978027343750,
+0.0001831054687500, -0.0038452148437500, 0.0000305175781250, -0.0033874511718750,
+0.0000000000000000, -0.0036010742187500, 0.0001831054687500, -0.0030822753906250,
+-0.0006408691406250, -0.0031127929687500, -0.0003967285156250, -0.0029907226562500,
+-0.0006103515625000, -0.0025634765625000, -0.0012512207031250, -0.0026550292968750,
+-0.0008239746093750, -0.0025024414062500, -0.0017089843750000, -0.0022583007812500,
+-0.0015258789062500, -0.0025939941406250, -0.0015563964843750, -0.0026245117187500,
+-0.0022277832031250, -0.0021362304687500, -0.0019226074218750, -0.0031433105468750,
+-0.0016174316406250, -0.0025329589843750, -0.0018005371093750, -0.0028991699218750,
+-0.0016784667968750, -0.0037841796875000, -0.0013122558593750, -0.0028686523437500,
+-0.0013732910156250, -0.0043640136718750, -0.0010375976562500, -0.0040893554687500,
+-0.0010681152343750, -0.0040588378906250, -0.0012207031250000, -0.0051269531250000,
+-0.0010681152343750, -0.0045776367187500, -0.0009155273437500, -0.0049438476562500,
+-0.0014038085937500, -0.0054626464843750, -0.0014038085937500, -0.0049743652343750,
+-0.0012512207031250, -0.0048828125000000, -0.0013732910156250, -0.0057373046875000,
+-0.0014343261718750, -0.0038146972656250, -0.0013732910156250, -0.0044555664062500,
+-0.0010986328125000, -0.0043640136718750, -0.0018615722656250, -0.0029602050781250,
+-0.0024108886718750, -0.0038146972656250, -0.0016784667968750, -0.0032653808593750,
+-0.0034179687500000, -0.0025634765625000, -0.0035400390625000, -0.0029602050781250,
+-0.0030517578125000, -0.0031127929687500, -0.0035705566406250, -0.0022888183593750,
+-0.0037536621093750, -0.0029907226562500, -0.0026245117187500, -0.0031127929687500,
+-0.0030517578125000, -0.0029296875000000, -0.0021057128906250, -0.0032653808593750,
+-0.0008239746093750, -0.0033874511718750, -0.0018615722656250, -0.0028076171875000,
+-0.0000610351562500, -0.0032653808593750, 0.0004577636718750, -0.0027160644531250,
+0.0000000000000000, -0.0017700195312500, 0.0003662109375000, -0.0024414062500000,
+0.0006103515625000, -0.0015258789062500, 0.0001525878906250, -0.0016174316406250,
+0.0003356933593750, -0.0017700195312500, 0.0000610351562500, -0.0011291503906250,
+-0.0003967285156250, -0.0011901855468750, -0.0001220703125000, -0.0015258789062500,
+-0.0015869140625000, -0.0007019042968750, -0.0012817382812500, -0.0010070800781250,
+-0.0016479492187500, -0.0021057128906250, -0.0029602050781250, -0.0018615722656250,
+-0.0023803710937500, -0.0027465820312500, -0.0020446777343750, -0.0025939941406250,
+-0.0029296875000000, -0.0028686523437500, -0.0018005371093750, -0.0036621093750000,
+-0.0003967285156250, -0.0033569335937500, -0.0012207031250000, -0.0034790039062500,
+0.0009155273437500, -0.0036926269531250, 0.0012817382812500, -0.0032348632812500,
+0.0002136230468750, -0.0029296875000000, 0.0005493164062500, -0.0031127929687500,
+0.0013122558593750, -0.0021362304687500, -0.0003051757812500, -0.0023498535156250,
+-0.0000610351562500, -0.0021667480468750, -0.0005798339843750, -0.0013427734375000,
+-0.0019836425781250, -0.0015563964843750, -0.0013427734375000, -0.0013427734375000,
+-0.0016784667968750, -0.0015563964843750, -0.0017700195312500, -0.0009460449218750,
+-0.0013427734375000, -0.0005493164062500, -0.0011901855468750, -0.0012207031250000,
+-0.0011596679687500, 0.0012207031250000, -0.0003356933593750, 0.0006408691406250,
+-0.0003051757812500, 0.0007019042968750, -0.0010681152343750, 0.0025329589843750,
+-0.0011291503906250, 0.0017700195312500, -0.0009765625000000, 0.0007629394531250,
+-0.0030212402343750, 0.0013732910156250, -0.0030517578125000, 0.0009460449218750,
+-0.0028381347656250, -0.0003662109375000, -0.0037536621093750, -0.0000915527343750,
+-0.0034484863281250, -0.0006103515625000, -0.0024414062500000, -0.0005493164062500,
+-0.0023803710937500, -0.0010681152343750, -0.0017395019531250, -0.0014038085937500,
+-0.0006103515625000, -0.0009155273437500, -0.0007629394531250, -0.0031433105468750,
+-0.0007934570312500, -0.0026855468750000, -0.0003967285156250, -0.0026855468750000,
+-0.0006713867187500, -0.0042724609375000, -0.0013122558593750, -0.0036315917968750,
+-0.0010986328125000, -0.0024414062500000, -0.0009155273437500, -0.0032348632812500,
+-0.0006103515625000, -0.0022888183593750, -0.0004272460937500, -0.0006408691406250,
+-0.0000610351562500, -0.0014953613281250, 0.0004882812500000, -0.0003051757812500,
+-0.0007629394531250, -0.0002136230468750, 0.0003051757812500, -0.0004577636718750,
+-0.0005187988281250, -0.0001220703125000, -0.0021972656250000, -0.0003967285156250,
+-0.0008850097656250, -0.0001220703125000, -0.0033874511718750, -0.0001831054687500,
+-0.0033569335937500, -0.0006103515625000, -0.0033569335937500, -0.0007629394531250,
+-0.0050048828125000, -0.0007934570312500, -0.0048217773437500, -0.0009765625000000,
+-0.0042724609375000, -0.0010681152343750, -0.0054626464843750, -0.0012207031250000,
+-0.0048522949218750, -0.0013427734375000, -0.0038146972656250, -0.0012207031250000,
+-0.0052795410156250, -0.0011901855468750, -0.0035400390625000, -0.0012817382812500,
+-0.0040893554687500, -0.0012207031250000, -0.0043640136718750, -0.0011596679687500,
+-0.0035400390625000, -0.0012817382812500, -0.0045166015625000, -0.0009155273437500,
+-0.0039062500000000, -0.0010986328125000, -0.0042419433593750, -0.0010681152343750,
+-0.0041198730468750, -0.0008239746093750, -0.0034484863281250, -0.0010986328125000,
+-0.0039978027343750, -0.0003967285156250, -0.0032653808593750, -0.0008544921875000,
+-0.0033874511718750, -0.0005798339843750, -0.0030822753906250, 0.0002441406250000,
+-0.0025024414062500, -0.0004272460937500, -0.0030517578125000, 0.0002441406250000,
+-0.0025939941406250, 0.0002746582031250, -0.0027465820312500, 0.0002136230468750,
+-0.0031433105468750, 0.0003967285156250, -0.0032653808593750, 0.0001831054687500,
+-0.0035400390625000, 0.0005187988281250, -0.0034484863281250, 0.0004272460937500,
+-0.0039367675781250, 0.0002441406250000, -0.0041809082031250, 0.0003967285156250,
+-0.0039367675781250, 0.0004272460937500, -0.0041503906250000, -0.0003662109375000,
+-0.0043334960937500, -0.0001525878906250, -0.0044860839843750, -0.0005187988281250,
+-0.0046691894531250, -0.0013427734375000, -0.0048522949218750, -0.0009765625000000,
+-0.0050048828125000, -0.0014343261718750, -0.0050659179687500, -0.0016479492187500,
+-0.0051574707031250, -0.0016479492187500, -0.0049743652343750, -0.0016174316406250,
+-0.0048522949218750, -0.0016479492187500, -0.0050048828125000, -0.0018005371093750,
+-0.0045776367187500, -0.0017089843750000, -0.0048828125000000, -0.0017089843750000,
+-0.0048522949218750, -0.0018310546875000, -0.0043640136718750, -0.0017700195312500,
+-0.0047912597656250, -0.0015869140625000, -0.0046386718750000, -0.0017089843750000,
+-0.0043334960937500, -0.0015258789062500, -0.0047302246093750, -0.0013427734375000,
+-0.0049133300781250, -0.0015563964843750, -0.0042419433593750, -0.0010681152343750,
+-0.0048522949218750, -0.0011596679687500, -0.0045166015625000, -0.0010681152343750,
+-0.0044250488281250, -0.0007019042968750, -0.0047607421875000, -0.0008544921875000,
+-0.0042419433593750, -0.0004882812500000, -0.0044860839843750, -0.0004577636718750,
+-0.0040588378906250, -0.0005493164062500, -0.0040283203125000, -0.0004272460937500,
+-0.0042419433593750, -0.0004272460937500, -0.0036010742187500, -0.0005798339843750,
+-0.0038757324218750, -0.0006408691406250, -0.0038757324218750, -0.0005798339843750,
+-0.0036010742187500, -0.0005798339843750, -0.0036926269531250, -0.0007629394531250,
+-0.0036926269531250, -0.0007324218750000, -0.0032348632812500, -0.0006713867187500,
+-0.0035705566406250, -0.0008544921875000, -0.0034179687500000, -0.0010070800781250,
+-0.0031127929687500, -0.0008850097656250, -0.0035400390625000, -0.0014038085937500,
+-0.0025024414062500, -0.0012817382812500, -0.0030517578125000, -0.0015258789062500,
+-0.0024414062500000, -0.0020141601562500, -0.0015869140625000, -0.0017395019531250,
+-0.0027160644531250, -0.0019531250000000, -0.0006713867187500, -0.0022277832031250,
+-0.0013427734375000, -0.0016479492187500, -0.0010681152343750, -0.0012512207031250,
+0.0003356933593750, -0.0014953613281250, -0.0007019042968750, -0.0001525878906250,
+0.0005493164062500, 0.0000000000000000, 0.0004577636718750, 0.0000610351562500,
+0.0000610351562500, 0.0007934570312500, 0.0005493164062500, 0.0006713867187500,
+0.0003662109375000, 0.0003051757812500, -0.0002441406250000, 0.0005798339843750,
+0.0001220703125000, 0.0003051757812500, -0.0003051757812500, -0.0001831054687500,
+-0.0009460449218750, 0.0001525878906250, -0.0001525878906250, -0.0001220703125000,
+-0.0007934570312500, -0.0002746582031250, -0.0007629394531250, 0.0001831054687500,
+-0.0003051757812500, 0.0004882812500000, -0.0000915527343750, 0.0004272460937500,
+0.0001525878906250, 0.0010375976562500, 0.0000000000000000, 0.0012817382812500,
+0.0005187988281250, 0.0013427734375000, 0.0002441406250000, 0.0015563964843750,
+-0.0001220703125000, 0.0016479492187500, 0.0007019042968750, 0.0018005371093750,
+-0.0010681152343750, 0.0019836425781250, -0.0008239746093750, 0.0016784667968750,
+-0.0012817382812500, 0.0014953613281250, -0.0027770996093750, 0.0018310546875000,
+-0.0024108886718750, 0.0008239746093750, -0.0030517578125000, 0.0007934570312500,
+-0.0038452148437500, 0.0009765625000000, -0.0037841796875000, 0.0006713867187500,
+-0.0035095214843750, 0.0007019042968750, -0.0037231445312500, 0.0003356933593750,
+-0.0036010742187500, 0.0007019042968750, -0.0036010742187500, -0.0001831054687500,
+-0.0035400390625000, -0.0010681152343750, -0.0034179687500000, -0.0003051757812500,
+-0.0033569335937500, -0.0022277832031250, -0.0031127929687500, -0.0024108886718750,
+-0.0031738281250000, -0.0022583007812500, -0.0030517578125000, -0.0028381347656250,
+-0.0028381347656250, -0.0025329589843750, -0.0030517578125000, -0.0025634765625000,
+-0.0027465820312500, -0.0025939941406250, -0.0027465820312500, -0.0026245117187500,
+-0.0025329589843750, -0.0023803710937500, -0.0026245117187500, -0.0021972656250000,
+-0.0028686523437500, -0.0034179687500000, -0.0007019042968750, -0.0031738281250000,
+-0.0011901855468750},
+{-0.0018615722656250, -0.0013427734375000, -0.0013732910156250, -0.0013427734375000,
+-0.0016784667968750, -0.0014648437500000, -0.0023193359375000, -0.0014038085937500,
+-0.0013122558593750, -0.0012817382812500, -0.0020751953125000, -0.0014343261718750,
+-0.0017395019531250, -0.0011901855468750, -0.0015258789062500, -0.0008544921875000,
+-0.0018920898437500, -0.0011291503906250, -0.0013427734375000, -0.0006103515625000,
+-0.0010986328125000, -0.0006103515625000, -0.0014038085937500, -0.0005493164062500,
+-0.0008850097656250, -0.0002441406250000, -0.0005493164062500, -0.0004882812500000,
+-0.0013732910156250, -0.0002136230468750, 0.0000915527343750, -0.0001831054687500,
+-0.0004882812500000, -0.0003051757812500, -0.0005187988281250, -0.0002746582031250,
+0.0005493164062500, -0.0002746582031250, -0.0001220703125000, -0.0003051757812500,
+0.0002441406250000, -0.0003356933593750, 0.0006713867187500, -0.0003356933593750,
+0.0003662109375000, -0.0002746582031250, 0.0003356933593750, -0.0002441406250000,
+0.0008850097656250, -0.0003051757812500, -0.0003662109375000, -0.0003967285156250,
+0.0003051757812500, -0.0001525878906250, -0.0002136230468750, -0.0000305175781250,
+-0.0014648437500000, -0.0002136230468750, -0.0004577636718750, 0.0003967285156250,
+-0.0022277832031250, 0.0003967285156250, -0.0025024414062500, 0.0004577636718750,
+-0.0025329589843750, 0.0008544921875000, -0.0035400390625000, 0.0007324218750000,
+-0.0036621093750000, 0.0007324218750000, -0.0032043457031250, 0.0009460449218750,
+-0.0038757324218750, 0.0007019042968750, -0.0033569335937500, 0.0005187988281250,
+-0.0025634765625000, 0.0006713867187500, -0.0032348632812500, 0.0000610351562500,
+-0.0017395019531250, 0.0002136230468750, -0.0018615722656250, 0.0002136230468750,
+-0.0019226074218750, -0.0002441406250000, -0.0009460449218750, 0.0001525878906250,
+-0.0011901855468750, 0.0005493164062500, -0.0016174316406250, 0.0005187988281250,
+-0.0009765625000000, 0.0007019042968750, -0.0010986328125000, 0.0012512207031250,
+-0.0017395019531250, 0.0012817382812500, -0.0014343261718750, 0.0013122558593750,
+-0.0017395019531250, 0.0014648437500000, -0.0018310546875000, 0.0014953613281250,
+-0.0018310546875000, 0.0015869140625000, -0.0020446777343750, 0.0016784667968750,
+-0.0020751953125000, 0.0009765625000000, -0.0017395019531250, 0.0013122558593750,
+-0.0017089843750000, 0.0009460449218750, -0.0014648437500000, 0.0001220703125000,
+-0.0010681152343750, 0.0005187988281250, -0.0011901855468750, -0.0003356933593750,
+-0.0014648437500000, -0.0000305175781250, -0.0010986328125000, -0.0006103515625000,
+-0.0018615722656250, -0.0014648437500000, -0.0028381347656250, -0.0006713867187500,
+-0.0024719238281250, -0.0025329589843750, -0.0042724609375000, -0.0023803710937500,
+-0.0039367675781250, -0.0022888183593750, -0.0041809082031250, -0.0035705566406250,
+-0.0055236816406250, -0.0031433105468750, -0.0047607421875000, -0.0024108886718750,
+-0.0048828125000000, -0.0025634765625000, -0.0051574707031250, -0.0021667480468750,
+-0.0040893554687500, -0.0014343261718750, -0.0032958984375000, -0.0016784667968750,
+-0.0038146972656250, -0.0011596679687500, -0.0015258789062500, -0.0007934570312500,
+-0.0014953613281250, -0.0013427734375000, -0.0016784667968750, -0.0017395019531250,
+-0.0004577636718750, -0.0013732910156250, -0.0005493164062500, -0.0018920898437500,
+-0.0021057128906250, -0.0023803710937500, -0.0018920898437500, -0.0019531250000000,
+-0.0019531250000000, -0.0014953613281250, -0.0031127929687500, -0.0019836425781250,
+-0.0029296875000000, -0.0021972656250000, -0.0013427734375000, -0.0016174316406250,
+-0.0017700195312500, -0.0023803710937500, -0.0015869140625000, -0.0034790039062500,
+-0.0001831054687500, -0.0028686523437500, -0.0006103515625000, -0.0030822753906250,
+-0.0011596679687500, -0.0036621093750000, -0.0005493164062500, -0.0031127929687500,
+-0.0010070800781250, -0.0018920898437500, -0.0027160644531250, -0.0000610351562500,
+-0.0011291503906250, -0.0009765625000000, -0.0014953613281250, -0.0008239746093750,
+-0.0007019042968750, -0.0007324218750000, -0.0010375976562500, -0.0010070800781250,
+-0.0019531250000000, -0.0012207031250000, -0.0017395019531250, -0.0003356933593750,
+-0.0009765625000000, -0.0002136230468750, -0.0010986328125000, -0.0006408691406250,
+-0.0014343261718750, -0.0006408691406250, -0.0008239746093750, -0.0004272460937500,
+-0.0005187988281250, -0.0012512207031250, -0.0034484863281250, -0.0017089843750000,
+-0.0025939941406250, -0.0010681152343750, -0.0031433105468750, -0.0009765625000000,
+-0.0057983398437500, -0.0015869140625000, -0.0048217773437500, 0.0001831054687500,
+-0.0043334960937500, 0.0005493164062500, -0.0051879882812500, -0.0002441406250000,
+-0.0040283203125000, 0.0004882812500000, -0.0023193359375000, 0.0011291503906250,
+-0.0032653808593750, -0.0022277832031250, -0.0025939941406250, -0.0007629394531250,
+-0.0018310546875000, -0.0015563964843750, -0.0025939941406250, -0.0044555664062500,
+-0.0034179687500000, -0.0023193359375000, -0.0028381347656250, -0.0036621093750000,
+-0.0030822753906250, -0.0041198730468750, -0.0034484863281250, -0.0026550292968750,
+-0.0025634765625000, -0.0022277832031250, -0.0018615722656250, -0.0027160644531250,
+-0.0023498535156250, -0.0001831054687500, -0.0011291503906250, -0.0003356933593750,
+-0.0007629394531250, -0.0005493164062500, -0.0011901855468750, 0.0008544921875000,
+-0.0011596679687500, 0.0006713867187500, -0.0008544921875000, -0.0007324218750000,
+-0.0010681152343750, -0.0002136230468750, -0.0015258789062500, -0.0006713867187500,
+-0.0009765625000000, -0.0019226074218750, -0.0002136230468750, -0.0012207031250000,
+-0.0007019042968750, -0.0026855468750000, -0.0002136230468750, -0.0025939941406250,
+-0.0000305175781250, -0.0025634765625000, -0.0002136230468750, -0.0034484863281250,
+-0.0005187988281250, -0.0031433105468750, -0.0004882812500000, -0.0027465820312500,
+0.0002746582031250, -0.0028076171875000, 0.0000305175781250, -0.0030822753906250,
+0.0001831054687500, -0.0027465820312500, 0.0009765625000000, -0.0023803710937500,
+0.0007629394531250, -0.0038146972656250, 0.0002746582031250, -0.0035095214843750,
+0.0005798339843750, -0.0037536621093750, 0.0003967285156250, -0.0050354003906250,
+-0.0003051757812500, -0.0047607421875000, -0.0000305175781250, -0.0048828125000000,
+0.0002746582031250, -0.0056152343750000, -0.0000610351562500, -0.0050964355468750,
+0.0003051757812500, -0.0046081542968750, 0.0008850097656250, -0.0054931640625000,
+0.0006103515625000, -0.0037841796875000, 0.0009460449218750, -0.0045166015625000,
+0.0010070800781250, -0.0038146972656250, 0.0009155273437500, -0.0023193359375000,
+0.0010070800781250, -0.0036926269531250, 0.0008850097656250, -0.0012512207031250,
+0.0008544921875000, -0.0017089843750000, 0.0009155273437500, -0.0014038085937500,
+0.0007019042968750, 0.0000305175781250, 0.0005493164062500, -0.0010681152343750,
+0.0007019042968750, 0.0004882812500000, 0.0001831054687500, 0.0001831054687500,
+0.0002746582031250, 0.0000305175781250, 0.0002441406250000, 0.0006103515625000,
+-0.0001220703125000, -0.0000915527343750, 0.0000305175781250, 0.0004272460937500,
+0.0000000000000000, 0.0001525878906250, -0.0000915527343750, -0.0000915527343750,
+-0.0001525878906250, 0.0001831054687500, -0.0002136230468750, -0.0002746582031250,
+-0.0001220703125000, -0.0008850097656250, -0.0003662109375000, -0.0004882812500000,
+-0.0003051757812500, -0.0010986328125000, -0.0003051757812500, -0.0017700195312500,
+-0.0004577636718750, -0.0010986328125000, -0.0003356933593750, -0.0022583007812500,
+-0.0006408691406250, -0.0019531250000000, -0.0006408691406250, -0.0019531250000000,
+-0.0005798339843750, -0.0025329589843750, -0.0007934570312500, -0.0021362304687500,
+-0.0007629394531250, -0.0026550292968750, -0.0007629394531250, -0.0025634765625000,
+-0.0007019042968750, -0.0026245117187500, -0.0007629394531250, -0.0028991699218750,
+-0.0007934570312500, -0.0026245117187500, -0.0006103515625000, -0.0027770996093750,
+-0.0010070800781250, -0.0026550292968750, -0.0008544921875000, -0.0025329589843750,
+-0.0009765625000000, -0.0025939941406250, -0.0013122558593750, -0.0025939941406250,
+-0.0010070800781250, -0.0025329589843750, -0.0014038085937500, -0.0025634765625000,
+-0.0013122558593750, -0.0024719238281250, -0.0014343261718750, -0.0024414062500000,
+-0.0016479492187500, -0.0025024414062500, -0.0014038085937500, -0.0022277832031250,
+-0.0017395019531250, -0.0024108886718750, -0.0017395019531250, -0.0023498535156250,
+-0.0016784667968750, -0.0020446777343750, -0.0018005371093750, -0.0022888183593750,
+-0.0018005371093750, -0.0025329589843750, -0.0017700195312500, -0.0021972656250000,
+-0.0019226074218750, -0.0022583007812500, -0.0014953613281250, -0.0023498535156250,
+-0.0013122558593750, -0.0017700195312500, -0.0016174316406250, -0.0023193359375000,
+-0.0003051757812500, -0.0017395019531250, -0.0005187988281250, -0.0018615722656250,
+-0.0002746582031250, -0.0023193359375000, 0.0006103515625000, -0.0016784667968750,
+0.0001525878906250, -0.0029602050781250, 0.0009155273437500, -0.0025634765625000,
+0.0009155273437500, -0.0030517578125000, 0.0004882812500000, -0.0040893554687500,
+0.0007324218750000, -0.0030822753906250, 0.0008850097656250, -0.0047302246093750,
+0.0000610351562500, -0.0043029785156250, 0.0000610351562500, -0.0042724609375000,
+0.0000915527343750, -0.0050964355468750, -0.0002136230468750, -0.0042724609375000,
+-0.0001220703125000, -0.0048828125000000, -0.0008544921875000, -0.0045471191406250,
+-0.0004882812500000, -0.0042724609375000, -0.0010986328125000, -0.0045166015625000,
+-0.0021667480468750, -0.0040893554687500, -0.0015563964843750, -0.0040588378906250,
+-0.0022888183593750, -0.0038452148437500, -0.0027465820312500, -0.0038757324218750,
+-0.0021667480468750, -0.0040283203125000, -0.0018920898437500, -0.0038757324218750,
+-0.0022277832031250, -0.0042419433593750, -0.0014038085937500, -0.0043945312500000,
+-0.0014038085937500, -0.0042419433593750, -0.0011596679687500, -0.0044250488281250,
+-0.0007019042968750, -0.0048217773437500, -0.0010986328125000, -0.0042724609375000,
+-0.0002136230468750, -0.0042724609375000, 0.0000000000000000, -0.0043334960937500,
+-0.0005187988281250, -0.0039062500000000, -0.0005187988281250, -0.0034790039062500,
+-0.0002441406250000, -0.0025939941406250, -0.0013732910156250, -0.0024108886718750,
+-0.0010986328125000, -0.0018615722656250, -0.0015258789062500, -0.0009155273437500,
+-0.0027160644531250, -0.0010986328125000, -0.0021057128906250, -0.0017700195312500,
+-0.0022583007812500, -0.0008239746093750, -0.0027465820312500, -0.0018615722656250,
+-0.0021362304687500, -0.0034179687500000, -0.0014953613281250, -0.0020751953125000,
+-0.0018310546875000, -0.0043640136718750, -0.0008239746093750, -0.0049438476562500,
+-0.0009155273437500, -0.0043640136718750, -0.0005798339843750, -0.0045471191406250,
+0.0002746582031250, -0.0038757324218750, -0.0000610351562500, -0.0029907226562500,
+0.0003051757812500, -0.0035095214843750, 0.0003967285156250, -0.0032348632812500,
+0.0000000000000000, -0.0021362304687500, -0.0000305175781250, -0.0024414062500000,
+0.0003051757812500, -0.0025634765625000, -0.0007019042968750, -0.0025329589843750,
+-0.0007324218750000, -0.0021057128906250, -0.0010070800781250, -0.0020751953125000,
+-0.0015258789062500, -0.0024108886718750, -0.0009460449218750, -0.0006408691406250,
+-0.0010375976562500, -0.0006103515625000, -0.0013122558593750, -0.0007934570312500,
+-0.0011901855468750, 0.0003967285156250, -0.0008544921875000, 0.0007019042968750,
+-0.0008544921875000, -0.0007324218750000, -0.0016784667968750, -0.0000610351562500,
+-0.0014343261718750, -0.0000915527343750, -0.0021362304687500, -0.0012207031250000,
+-0.0031738281250000},
+{-0.0019226074218750, 0.0009155273437500, -0.0019531250000000, 0.0012207031250000,
+-0.0018310546875000, 0.0003662109375000, -0.0018920898437500, 0.0005493164062500,
+-0.0018920898437500, 0.0001831054687500, -0.0018005371093750, -0.0005493164062500,
+-0.0018615722656250, -0.0003051757812500, -0.0018005371093750, -0.0014648437500000,
+-0.0012207031250000, -0.0012817382812500, -0.0015563964843750, -0.0017395019531250,
+-0.0010375976562500, -0.0025939941406250, -0.0003051757812500, -0.0018920898437500,
+-0.0008239746093750, -0.0031127929687500, 0.0002441406250000, -0.0027770996093750,
+0.0001220703125000, -0.0025329589843750, 0.0003051757812500, -0.0031127929687500,
+0.0011291503906250, -0.0027160644531250, 0.0005798339843750, -0.0025939941406250,
+0.0007629394531250, -0.0026245117187500, 0.0010681152343750, -0.0024108886718750,
+0.0007019042968750, -0.0021667480468750, 0.0004272460937500, -0.0022277832031250,
+0.0005798339843750, -0.0022277832031250, 0.0004272460937500, -0.0018615722656250,
+0.0004577636718750, -0.0021972656250000, 0.0001220703125000, -0.0024414062500000,
+-0.0002441406250000, -0.0018920898437500, 0.0000915527343750, -0.0029296875000000,
+-0.0007934570312500, -0.0025024414062500, -0.0008239746093750, -0.0027770996093750,
+-0.0007934570312500, -0.0037231445312500, -0.0012207031250000, -0.0030517578125000,
+-0.0012207031250000, -0.0039672851562500, -0.0010375976562500, -0.0042114257812500,
+-0.0010986328125000, -0.0039978027343750, -0.0010986328125000, -0.0042724609375000,
+-0.0010070800781250, -0.0044860839843750, -0.0010070800781250, -0.0041503906250000,
+-0.0006713867187500, -0.0043640136718750, -0.0008544921875000, -0.0043640136718750,
+-0.0007629394531250, -0.0042114257812500, -0.0000915527343750, -0.0045166015625000,
+-0.0003356933593750, -0.0035400390625000, -0.0012512207031250, -0.0042419433593750,
+-0.0007629394531250, -0.0041503906250000, -0.0011596679687500, -0.0030822753906250,
+-0.0022583007812500, -0.0038452148437500, -0.0017395019531250, -0.0040893554687500,
+-0.0022888183593750, -0.0036621093750000, -0.0026855468750000, -0.0033874511718750,
+-0.0018615722656250, -0.0036315917968750, -0.0013732910156250, -0.0035095214843750,
+-0.0020446777343750, -0.0022888183593750, -0.0007324218750000, -0.0027160644531250,
+-0.0005187988281250, -0.0025634765625000, -0.0007324218750000, -0.0012207031250000,
+-0.0004577636718750, -0.0015258789062500, -0.0003967285156250, -0.0024414062500000,
+-0.0001831054687500, -0.0012817382812500, -0.0002746582031250, -0.0024108886718750,
+-0.0002441406250000, -0.0041503906250000, 0.0000000000000000, -0.0023803710937500,
+-0.0001525878906250, -0.0044860839843750, -0.0001525878906250, -0.0049133300781250,
+0.0001220703125000, -0.0038146972656250, -0.0003356933593750, -0.0039367675781250,
+-0.0007324218750000, -0.0043029785156250, -0.0003662109375000, -0.0027160644531250,
+-0.0014038085937500, -0.0026550292968750, -0.0010986328125000, -0.0028381347656250,
+-0.0010681152343750, -0.0025024414062500, -0.0017395019531250, -0.0024719238281250,
+-0.0015563964843750, -0.0019531250000000, -0.0018615722656250, -0.0022583007812500,
+-0.0013122558593750, -0.0016784667968750, -0.0016174316406250, -0.0003662109375000,
+-0.0028991699218750, -0.0003051757812500, -0.0033874511718750, -0.0003662109375000,
+-0.0038452148437500, 0.0002746582031250, -0.0035400390625000, -0.0003356933593750,
+-0.0035705566406250, -0.0007324218750000, -0.0038757324218750, 0.0003662109375000,
+-0.0035400390625000, -0.0018615722656250, -0.0033874511718750, -0.0015258789062500,
+-0.0031738281250000, -0.0011596679687500, -0.0030822753906250, -0.0021362304687500,
+-0.0028381347656250, -0.0015258789062500, -0.0023193359375000, -0.0010375976562500,
+-0.0027770996093750, -0.0009765625000000, -0.0021057128906250, -0.0004882812500000,
+-0.0025939941406250, 0.0000000000000000, -0.0033264160156250, -0.0002136230468750,
+-0.0024414062500000, 0.0005493164062500, -0.0040588378906250, 0.0006713867187500,
+-0.0041503906250000, 0.0007934570312500, -0.0037841796875000, 0.0012207031250000,
+-0.0040283203125000, 0.0009460449218750, -0.0037231445312500, 0.0008544921875000,
+-0.0039672851562500, 0.0012817382812500, -0.0037231445312500, 0.0008850097656250,
+-0.0033874511718750, 0.0002136230468750, -0.0032958984375000, 0.0005187988281250,
+-0.0031738281250000, -0.0002441406250000, -0.0032043457031250, -0.0003051757812500,
+-0.0027160644531250, -0.0004882812500000, -0.0031738281250000, -0.0008850097656250,
+-0.0037841796875000, -0.0006713867187500, -0.0033874511718750, -0.0018005371093750,
+-0.0047302246093750, -0.0016174316406250, -0.0051269531250000, -0.0018920898437500,
+-0.0050354003906250, -0.0028686523437500, -0.0056762695312500, -0.0023803710937500,
+-0.0061340332031250, -0.0024108886718750, -0.0052795410156250, -0.0029602050781250,
+-0.0058288574218750, -0.0023803710937500, -0.0054321289062500, -0.0016784667968750,
+-0.0044555664062500, -0.0019836425781250, -0.0050354003906250, -0.0008850097656250,
+-0.0039672851562500, -0.0010070800781250, -0.0038452148437500, -0.0010986328125000,
+-0.0038757324218750, -0.0004882812500000, -0.0034484863281250, -0.0006103515625000,
+-0.0035705566406250, -0.0004882812500000, -0.0030822753906250, -0.0008850097656250,
+-0.0035095214843750, -0.0002746582031250, -0.0029907226562500, 0.0002746582031250,
+-0.0023193359375000, -0.0004577636718750, -0.0029296875000000, 0.0011596679687500,
+-0.0010986328125000, 0.0010070800781250, -0.0013122558593750, 0.0008850097656250,
+-0.0010681152343750, 0.0019836425781250, 0.0000915527343750, 0.0017089843750000,
+-0.0005187988281250, 0.0010986328125000, -0.0000305175781250, 0.0016479492187500,
+-0.0001525878906250, 0.0011596679687500, -0.0003662109375000, 0.0002746582031250,
+-0.0003967285156250, 0.0009765625000000, -0.0006713867187500, -0.0002746582031250,
+-0.0006713867187500, -0.0000610351562500, -0.0010986328125000, -0.0002136230468750,
+-0.0011596679687500, -0.0010681152343750, -0.0010681152343750, -0.0005798339843750,
+-0.0016174316406250, -0.0008544921875000, -0.0011291503906250, -0.0010986328125000,
+-0.0016174316406250, -0.0005798339843750, -0.0014343261718750, -0.0002746582031250,
+-0.0009155273437500, -0.0005493164062500, -0.0014648437500000, 0.0004577636718750,
+-0.0004882812500000, 0.0005493164062500, -0.0006713867187500, 0.0004577636718750,
+-0.0005798339843750, 0.0010070800781250, 0.0000305175781250, 0.0010986328125000,
+-0.0004882812500000, 0.0005187988281250, 0.0002136230468750, 0.0008239746093750,
+0.0000000000000000, 0.0005798339843750, -0.0001525878906250, 0.0000000000000000,
+0.0004577636718750, 0.0003662109375000, 0.0004577636718750, -0.0003662109375000,
+-0.0001525878906250, -0.0002746582031250, 0.0004882812500000, -0.0003356933593750,
+0.0004272460937500, -0.0008239746093750, 0.0000610351562500, -0.0005493164062500,
+0.0010375976562500, -0.0007629394531250, 0.0007934570312500, -0.0008544921875000,
+0.0008544921875000, -0.0008239746093750, 0.0011901855468750, -0.0008544921875000,
+0.0011291503906250, -0.0008850097656250, 0.0010681152343750, -0.0008239746093750,
+0.0014038085937500, -0.0007934570312500, 0.0013732910156250, -0.0007629394531250,
+0.0012207031250000, -0.0007019042968750, 0.0014038085937500, -0.0006713867187500,
+0.0014038085937500, -0.0004577636718750, 0.0009765625000000, -0.0005187988281250,
+0.0012512207031250, -0.0002441406250000, 0.0011291503906250, 0.0001220703125000,
+0.0008239746093750, -0.0000305175781250, 0.0013122558593750, 0.0005798339843750,
+0.0007324218750000, 0.0006713867187500, 0.0010986328125000, 0.0008544921875000,
+0.0007629394531250, 0.0012512207031250, 0.0002746582031250, 0.0012817382812500,
+0.0008850097656250, 0.0018005371093750, -0.0007934570312500, 0.0016174316406250,
+-0.0001525878906250, 0.0018615722656250, -0.0008239746093750, 0.0024414062500000,
+-0.0021972656250000, 0.0020751953125000, -0.0009460449218750, 0.0025024414062500,
+-0.0032958984375000, 0.0026245117187500, -0.0025634765625000, 0.0023193359375000,
+-0.0027160644531250, 0.0022583007812500, -0.0043029785156250, 0.0023498535156250,
+-0.0029296875000000, 0.0018615722656250, -0.0039062500000000, 0.0019226074218750,
+-0.0039672851562500, 0.0016174316406250, -0.0032348632812500, 0.0012512207031250,
+-0.0034790039062500, 0.0014953613281250, -0.0033264160156250, 0.0007934570312500,
+-0.0018310546875000, 0.0007934570312500, -0.0024719238281250, 0.0010375976562500,
+-0.0014953613281250, 0.0008544921875000, 0.0000610351562500, 0.0008239746093750,
+-0.0012817382812500, 0.0011901855468750, 0.0010375976562500, 0.0013427734375000,
+0.0007019042968750, 0.0010070800781250, 0.0004882812500000, 0.0009460449218750,
+0.0016784667968750, 0.0011901855468750, 0.0008239746093750, 0.0005187988281250,
+0.0005493164062500, 0.0004272460937500, 0.0004272460937500, 0.0005798339843750,
+-0.0002746582031250, 0.0006103515625000, -0.0009460449218750, 0.0005493164062500,
+-0.0011291503906250, -0.0000305175781250, -0.0022277832031250, 0.0004577636718750,
+-0.0025024414062500, -0.0002441406250000, -0.0024108886718750, -0.0012512207031250,
+-0.0028076171875000, -0.0004882812500000, -0.0028991699218750, -0.0017700195312500,
+-0.0019226074218750, -0.0020141601562500, -0.0021667480468750, -0.0017700195312500,
+-0.0021057128906250, -0.0021667480468750, -0.0012817382812500, -0.0021057128906250,
+-0.0013732910156250, -0.0017395019531250, -0.0020446777343750, -0.0018615722656250,
+-0.0012512207031250, -0.0017700195312500, -0.0015869140625000, -0.0014038085937500,
+-0.0023193359375000, -0.0015258789062500, -0.0011596679687500, -0.0015563964843750,
+-0.0017700195312500, -0.0014648437500000, -0.0020141601562500, -0.0014953613281250,
+-0.0016784667968750, -0.0015563964843750, -0.0013427734375000, -0.0014953613281250,
+-0.0010986328125000, -0.0011596679687500, -0.0014648437500000, -0.0010681152343750,
+-0.0010681152343750, -0.0013732910156250, -0.0007629394531250, -0.0013427734375000,
+-0.0008239746093750, -0.0010375976562500, -0.0007019042968750, -0.0018005371093750,
+-0.0017700195312500, -0.0017395019531250, -0.0014953613281250, -0.0015869140625000,
+-0.0021667480468750, -0.0019836425781250, -0.0032043457031250, -0.0017395019531250,
+-0.0023193359375000, -0.0009155273437500, -0.0032043457031250, -0.0011901855468750,
+-0.0033264160156250, -0.0004272460937500, -0.0025634765625000, 0.0006103515625000,
+-0.0022888183593750, 0.0000610351562500, -0.0024108886718750, 0.0013122558593750,
+-0.0014038085937500, 0.0018615722656250, -0.0010681152343750, 0.0010986328125000,
+-0.0016174316406250, 0.0010681152343750, -0.0012817382812500, 0.0018310546875000,
+-0.0005187988281250, 0.0002746582031250, -0.0029296875000000, 0.0002746582031250,
+-0.0021667480468750, 0.0006103515625000, -0.0018920898437500, -0.0000915527343750,
+-0.0041809082031250, 0.0001831054687500, -0.0007629394531250, -0.0003662109375000,
+-0.0031433105468750, -0.0004272460937500, -0.0026245117187500, -0.0004577636718750,
+-0.0021057128906250, -0.0007324218750000, -0.0028381347656250, -0.0007934570312500,
+-0.0031127929687500, 0.0000000000000000, -0.0007324218750000, 0.0002441406250000,
+-0.0009460449218750, -0.0003967285156250, -0.0007934570312500, 0.0000000000000000,
+0.0008239746093750, 0.0007324218750000, 0.0003356933593750, -0.0023803710937500,
+-0.0003356933593750, -0.0010681152343750, -0.0000305175781250, -0.0020751953125000,
+-0.0001831054687500, -0.0053100585937500, -0.0008544921875000, -0.0035400390625000,
+-0.0006408691406250, -0.0037841796875000, -0.0005493164062500, -0.0053100585937500,
+-0.0004272460937500, -0.0032653808593750, -0.0007629394531250, -0.0011596679687500,
+-0.0007019042968750, -0.0030822753906250, -0.0002441406250000, -0.0002746582031250,
+-0.0027465820312500},
+{-0.0006713867187500, 0.0021057128906250, -0.0031433105468750, 0.0019836425781250,
+-0.0026245117187500, 0.0018005371093750, -0.0028686523437500, 0.0019226074218750,
+-0.0048828125000000, 0.0018310546875000, -0.0040893554687500, 0.0012512207031250,
+-0.0042724609375000, 0.0011596679687500, -0.0052795410156250, 0.0011596679687500,
+-0.0039672851562500, 0.0008544921875000, -0.0030212402343750, 0.0006408691406250,
+-0.0045776367187500, 0.0009460449218750, -0.0008544921875000, 0.0010375976562500,
+-0.0016784667968750, 0.0002441406250000, -0.0009765625000000, -0.0001220703125000,
+0.0015869140625000, 0.0003051757812500, -0.0000915527343750, -0.0013427734375000,
+0.0019836425781250, -0.0016479492187500, 0.0017395019531250, -0.0013122558593750,
+0.0008544921875000, -0.0018615722656250, 0.0013122558593750, -0.0020141601562500,
+0.0007324218750000, -0.0009155273437500, -0.0004272460937500, -0.0011596679687500,
+-0.0003662109375000, -0.0011596679687500, -0.0009155273437500, -0.0003967285156250,
+-0.0020751953125000, -0.0006408691406250, -0.0021362304687500, -0.0012512207031250,
+-0.0021667480468750, -0.0007934570312500, -0.0027160644531250, -0.0010070800781250,
+-0.0026855468750000, -0.0017089843750000, -0.0023498535156250, -0.0012512207031250,
+-0.0024414062500000, -0.0013122558593750, -0.0022583007812500, -0.0014038085937500,
+-0.0021057128906250, -0.0014038085937500, -0.0018005371093750, -0.0011291503906250,
+-0.0016479492187500, -0.0010681152343750, -0.0018615722656250, -0.0019226074218750,
+-0.0014648437500000, -0.0014343261718750, -0.0015258789062500, -0.0016479492187500,
+-0.0012512207031250, -0.0026245117187500, -0.0010681152343750, -0.0020141601562500,
+-0.0017089843750000, -0.0014038085937500, -0.0008544921875000, -0.0018920898437500,
+-0.0022583007812500, -0.0006408691406250, -0.0018920898437500, 0.0008239746093750,
+-0.0006103515625000, 0.0000305175781250, -0.0023498535156250, 0.0019836425781250,
+-0.0010681152343750, 0.0023498535156250, -0.0012207031250000, 0.0020446777343750,
+-0.0020751953125000, 0.0027465820312500, -0.0019226074218750, 0.0027770996093750,
+-0.0020446777343750, 0.0016174316406250, -0.0027465820312500, 0.0019531250000000,
+-0.0032653808593750, 0.0018920898437500, -0.0028381347656250, 0.0010375976562500,
+-0.0029296875000000, 0.0011596679687500, -0.0037231445312500, 0.0008850097656250,
+-0.0015258789062500, 0.0009155273437500, -0.0017089843750000, 0.0005798339843750,
+-0.0023193359375000, -0.0001525878906250, -0.0010375976562500, -0.0006713867187500,
+-0.0048522949218750, -0.0009155273437500, -0.0012512207031250, -0.0003051757812500,
+-0.0019836425781250, -0.0008850097656250, -0.0026245117187500, -0.0012207031250000,
+-0.0020446777343750, -0.0005798339843750, -0.0020751953125000, -0.0025939941406250,
+-0.0028991699218750, -0.0021362304687500, -0.0036010742187500, -0.0020446777343750,
+-0.0024719238281250, -0.0036926269531250, -0.0020141601562500, -0.0032958984375000,
+-0.0032043457031250, -0.0025024414062500, 0.0000000000000000, -0.0036010742187500,
+-0.0000610351562500, -0.0028076171875000, -0.0006103515625000, -0.0014038085937500,
+0.0007934570312500, -0.0028991699218750, 0.0005798339843750, -0.0018310546875000,
+-0.0008239746093750, -0.0016174316406250, -0.0004882812500000, -0.0028076171875000,
+-0.0006713867187500, -0.0032653808593750, -0.0017395019531250, -0.0029296875000000,
+-0.0013427734375000, -0.0041809082031250, -0.0007934570312500, -0.0046691894531250,
+-0.0006713867187500, -0.0040588378906250, -0.0012207031250000, -0.0039978027343750,
+-0.0011901855468750, -0.0042419433593750, -0.0004272460937500, -0.0028076171875000,
+-0.0026550292968750, -0.0031127929687500, -0.0023803710937500, -0.0028381347656250,
+-0.0025024414062500, -0.0018615722656250, -0.0038757324218750, -0.0027160644531250,
+-0.0032043457031250, -0.0024414062500000, -0.0028381347656250, -0.0030212402343750,
+-0.0030517578125000, -0.0032043457031250, -0.0025024414062500, -0.0036621093750000,
+-0.0016784667968750, -0.0046997070312500, -0.0017089843750000, -0.0021667480468750,
+-0.0013427734375000, -0.0037841796875000, -0.0010986328125000, -0.0028686523437500,
+-0.0008850097656250, -0.0000610351562500, -0.0006103515625000, -0.0020751953125000,
+-0.0006103515625000, 0.0002136230468750, -0.0003662109375000, 0.0005493164062500,
+0.0000305175781250, -0.0001525878906250, -0.0003051757812500, 0.0006103515625000,
+-0.0007019042968750, 0.0005493164062500, -0.0002441406250000, -0.0012512207031250,
+-0.0009155273437500, -0.0007019042968750, -0.0010986328125000, -0.0015563964843750,
+-0.0007324218750000, -0.0036926269531250, -0.0008544921875000, -0.0029907226562500,
+-0.0010986328125000, -0.0036926269531250, -0.0000915527343750, -0.0048522949218750,
+-0.0003662109375000, -0.0038757324218750, -0.0000610351562500, -0.0032958984375000,
+0.0008544921875000, -0.0046691894531250, 0.0003967285156250, -0.0017089843750000,
+0.0010070800781250, -0.0027770996093750, 0.0010070800781250, -0.0020141601562500,
+0.0009765625000000, 0.0003051757812500, 0.0012207031250000, -0.0015869140625000,
+0.0010681152343750, 0.0015258789062500, 0.0013732910156250, 0.0012512207031250,
+0.0013732910156250, 0.0011596679687500, 0.0014343261718750, 0.0029296875000000,
+0.0015869140625000, 0.0021362304687500, 0.0014648437500000, 0.0019836425781250,
+0.0016784667968750, 0.0026550292968750, 0.0014648437500000, 0.0019226074218750,
+0.0017395019531250, 0.0012207031250000, 0.0021667480468750, 0.0021362304687500,
+0.0017700195312500, 0.0005187988281250, 0.0024719238281250, 0.0008850097656250,
+0.0024719238281250, 0.0008239746093750, 0.0022888183593750, -0.0001525878906250,
+0.0025024414062500, 0.0005798339843750, 0.0025024414062500, -0.0005187988281250,
+0.0023193359375000, -0.0005187988281250, 0.0023193359375000, 0.0000305175781250,
+0.0021972656250000, -0.0001831054687500, 0.0020446777343750, -0.0004577636718750,
+0.0021362304687500, -0.0000610351562500, 0.0016174316406250, -0.0002441406250000,
+0.0017700195312500, -0.0003051757812500, 0.0015869140625000, 0.0000305175781250,
+0.0010681152343750, -0.0000915527343750, 0.0013732910156250, 0.0005493164062500,
+0.0007934570312500, 0.0005187988281250, 0.0008239746093750, 0.0006408691406250,
+0.0006103515625000, 0.0011291503906250, 0.0002136230468750, 0.0010986328125000,
+0.0004577636718750, 0.0010986328125000, -0.0000915527343750, 0.0013427734375000,
+-0.0001220703125000, 0.0012512207031250, -0.0002746582031250, 0.0012512207031250,
+-0.0006408691406250, 0.0015563964843750, -0.0005493164062500, 0.0008239746093750,
+-0.0010375976562500, 0.0012207031250000, -0.0010375976562500, 0.0010681152343750,
+-0.0011291503906250, 0.0004882812500000, -0.0014953613281250, 0.0010070800781250,
+-0.0014343261718750, 0.0002441406250000, -0.0014648437500000, 0.0002441406250000,
+-0.0017089843750000, 0.0002136230468750, -0.0013732910156250, -0.0002136230468750,
+-0.0011291503906250, -0.0000915527343750, -0.0015258789062500, -0.0003967285156250,
+-0.0002136230468750, -0.0005187988281250, -0.0005493164062500, -0.0004882812500000,
+-0.0003356933593750, -0.0007324218750000, 0.0006408691406250, -0.0006713867187500,
+0.0000610351562500, 0.0003967285156250, 0.0005187988281250, -0.0002441406250000,
+0.0005798339843750, 0.0000915527343750, 0.0004577636718750, 0.0009460449218750,
+0.0006103515625000, -0.0000915527343750, 0.0005493164062500, 0.0013427734375000,
+0.0001525878906250, 0.0011901855468750, 0.0003051757812500, 0.0012207031250000,
+0.0003662109375000, 0.0022583007812500, 0.0000915527343750, 0.0021057128906250,
+0.0000305175781250, 0.0028381347656250, 0.0005493164062500, 0.0033569335937500,
+0.0005187988281250, 0.0034484863281250, 0.0003356933593750, 0.0038452148437500,
+0.0005187988281250, 0.0041503906250000, 0.0005187988281250, 0.0039062500000000,
+0.0001525878906250, 0.0042724609375000, 0.0001525878906250, 0.0039672851562500,
+0.0000000000000000, 0.0034179687500000, -0.0003662109375000, 0.0036010742187500,
+-0.0002746582031250, 0.0021362304687500, -0.0000915527343750, 0.0023193359375000,
+-0.0004577636718750, 0.0010681152343750, -0.0002441406250000, -0.0006408691406250,
+0.0001831054687500, 0.0001220703125000, -0.0001831054687500, -0.0023803710937500,
+0.0001831054687500, -0.0022583007812500, 0.0002746582031250, -0.0024414062500000,
+-0.0000915527343750, -0.0037231445312500, -0.0001220703125000, -0.0029602050781250,
+0.0001525878906250, -0.0035095214843750, -0.0010681152343750, -0.0036315917968750,
+-0.0009155273437500, -0.0032958984375000, -0.0008239746093750, -0.0033264160156250,
+-0.0015869140625000, -0.0035095214843750, -0.0015258789062500, -0.0034790039062500,
+-0.0014038085937500, -0.0033874511718750, -0.0013427734375000, -0.0041503906250000,
+-0.0010070800781250, -0.0048217773437500, -0.0007324218750000, -0.0045471191406250,
+-0.0008850097656250, -0.0055236816406250, -0.0007324218750000, -0.0061340332031250,
+-0.0004272460937500, -0.0061950683593750, -0.0005493164062500, -0.0063171386718750,
+-0.0008850097656250, -0.0063171386718750, -0.0007629394531250, -0.0053405761718750,
+-0.0003356933593750, -0.0050048828125000, -0.0004272460937500, -0.0046691894531250,
+-0.0005493164062500, -0.0040283203125000, -0.0003967285156250, -0.0038146972656250,
+-0.0003662109375000, -0.0032958984375000, -0.0003967285156250, -0.0039672851562500,
+-0.0003967285156250, -0.0037841796875000, -0.0006408691406250, -0.0028381347656250,
+-0.0007324218750000, -0.0032958984375000, -0.0005798339843750, -0.0030822753906250,
+-0.0014038085937500, -0.0025024414062500, -0.0012512207031250, -0.0028076171875000,
+-0.0015258789062500, -0.0030212402343750, -0.0023193359375000, -0.0023803710937500,
+-0.0018005371093750, -0.0033264160156250, -0.0020446777343750, -0.0032043457031250,
+-0.0021057128906250, -0.0030822753906250, -0.0020446777343750, -0.0035400390625000,
+-0.0023193359375000, -0.0032348632812500, -0.0022888183593750, -0.0033264160156250,
+-0.0017395019531250, -0.0036315917968750, -0.0015563964843750, -0.0030822753906250,
+-0.0022583007812500, -0.0030517578125000, -0.0025329589843750, -0.0037231445312500,
+-0.0020751953125000, -0.0010070800781250, -0.0031127929687500, -0.0022583007812500,
+-0.0027770996093750, -0.0021057128906250, -0.0028686523437500, -0.0001525878906250,
+-0.0034790039062500, -0.0015258789062500, -0.0026855468750000, 0.0003356933593750,
+-0.0036621093750000, -0.0000915527343750, -0.0034179687500000, -0.0003662109375000,
+-0.0034179687500000, 0.0007019042968750, -0.0043334960937500, 0.0000000000000000,
+-0.0041198730468750, -0.0004577636718750, -0.0037841796875000, -0.0002746582031250,
+-0.0043334960937500, -0.0005798339843750, -0.0036010742187500, -0.0011596679687500,
+-0.0020141601562500, -0.0010375976562500, -0.0021667480468750, -0.0011291503906250,
+-0.0024108886718750, -0.0010681152343750, -0.0007019042968750, -0.0008239746093750,
+-0.0014953613281250, -0.0005798339843750, -0.0028381347656250, -0.0005798339843750,
+-0.0010681152343750, -0.0012817382812500, -0.0039062500000000, -0.0008544921875000,
+-0.0036315917968750, -0.0010681152343750, -0.0033874511718750, -0.0019836425781250,
+-0.0052185058593750, -0.0015563964843750, -0.0050048828125000, -0.0014953613281250,
+-0.0047302246093750, -0.0019836425781250, -0.0054321289062500, -0.0015258789062500,
+-0.0055236816406250, -0.0008239746093750, -0.0052795410156250, -0.0013732910156250,
+-0.0059509277343750, -0.0011291503906250, -0.0056762695312500, -0.0008850097656250,
+-0.0063171386718750, -0.0015258789062500, -0.0057067871093750, -0.0019836425781250,
+-0.0051269531250000, -0.0017089843750000, -0.0061645507812500, -0.0021362304687500,
+-0.0035095214843750, -0.0026855468750000, -0.0045166015625000, -0.0021362304687500,
+-0.0037231445312500},
+{-0.0002136230468750, 0.0016479492187500, -0.0005187988281250, 0.0002441406250000,
+-0.0007629394531250, 0.0012817382812500, -0.0005493164062500, -0.0013427734375000,
+-0.0009460449218750, -0.0009765625000000, -0.0011596679687500, -0.0013122558593750,
+-0.0010986328125000, -0.0030212402343750, -0.0012512207031250, -0.0021667480468750,
+-0.0012207031250000, -0.0029907226562500, -0.0009460449218750, -0.0032348632812500,
+-0.0010681152343750, -0.0027770996093750, -0.0009765625000000, -0.0027465820312500,
+-0.0005187988281250, -0.0030822753906250, -0.0004577636718750, -0.0031433105468750,
+-0.0006103515625000, -0.0028686523437500, -0.0004272460937500, -0.0037536621093750,
+-0.0006103515625000, -0.0046386718750000, -0.0007934570312500, -0.0040893554687500,
+-0.0004577636718750, -0.0057067871093750, -0.0010681152343750, -0.0056762695312500,
+-0.0010681152343750, -0.0054016113281250, -0.0008544921875000, -0.0058898925781250,
+-0.0010681152343750, -0.0052490234375000, -0.0011596679687500, -0.0044555664062500,
+-0.0010070800781250, -0.0046997070312500, -0.0007629394531250, -0.0036010742187500,
+-0.0010986328125000, -0.0027160644531250, -0.0014038085937500, -0.0041503906250000,
+-0.0011596679687500, -0.0023498535156250, -0.0019836425781250, -0.0018005371093750,
+-0.0018920898437500, -0.0026245117187500, -0.0014343261718750, -0.0024414062500000,
+-0.0015869140625000, -0.0019226074218750, -0.0017395019531250, -0.0031127929687500,
+-0.0012207031250000, -0.0029602050781250, -0.0013122558593750, -0.0027465820312500,
+-0.0009765625000000, -0.0032653808593750, -0.0003662109375000, -0.0030822753906250,
+-0.0008239746093750, -0.0030822753906250, -0.0017700195312500, -0.0028076171875000,
+-0.0012817382812500, -0.0031127929687500, -0.0015869140625000, -0.0037231445312500,
+-0.0029296875000000, -0.0031738281250000, -0.0034179687500000, -0.0027770996093750,
+-0.0043640136718750, -0.0036926269531250, -0.0041503906250000, -0.0025939941406250,
+-0.0040283203125000, -0.0014038085937500, -0.0045166015625000, -0.0027770996093750,
+-0.0042724609375000, -0.0001525878906250, -0.0040893554687500, -0.0005187988281250,
+-0.0044250488281250, -0.0004272460937500, -0.0038757324218750, 0.0008850097656250,
+-0.0032348632812500, -0.0003051757812500, -0.0036315917968750, 0.0008850097656250,
+-0.0024414062500000, 0.0005493164062500, -0.0011901855468750, 0.0002136230468750,
+-0.0019836425781250, 0.0006408691406250, -0.0020751953125000, 0.0000305175781250,
+-0.0003051757812500, 0.0002441406250000, -0.0029602050781250, 0.0001831054687500,
+-0.0022583007812500, -0.0000915527343750, -0.0021972656250000, -0.0001220703125000,
+-0.0039672851562500, -0.0001831054687500, -0.0030822753906250, 0.0000305175781250,
+-0.0034790039062500, -0.0000305175781250, -0.0045166015625000, -0.0005187988281250,
+-0.0043640136718750, -0.0006713867187500, -0.0042114257812500, -0.0005798339843750,
+-0.0053405761718750, -0.0017395019531250, -0.0046997070312500, -0.0017089843750000,
+-0.0057983398437500, -0.0016174316406250, -0.0054321289062500, -0.0024414062500000,
+-0.0048522949218750, -0.0025024414062500, -0.0064697265625000, -0.0014953613281250,
+-0.0032958984375000, -0.0019226074218750, -0.0047607421875000, -0.0016479492187500,
+-0.0040893554687500, -0.0006408691406250, -0.0015563964843750, -0.0012207031250000,
+-0.0040893554687500, -0.0012512207031250, -0.0014953613281250, -0.0010070800781250,
+-0.0021667480468750, -0.0012512207031250, -0.0026855468750000, -0.0016479492187500,
+-0.0016174316406250, -0.0015563964843750, -0.0028381347656250, -0.0014038085937500,
+-0.0020751953125000, -0.0018920898437500, -0.0028381347656250, -0.0011596679687500,
+-0.0020751953125000, -0.0003356933593750, -0.0010681152343750, -0.0010986328125000,
+-0.0024108886718750, -0.0002746582031250, 0.0005798339843750, 0.0001220703125000,
+0.0001525878906250, -0.0002441406250000, 0.0003662109375000, -0.0001525878906250,
+0.0023498535156250, 0.0000915527343750, 0.0015258789062500, -0.0009765625000000,
+0.0017089843750000, -0.0005798339843750, 0.0025329589843750, -0.0005493164062500,
+0.0016784667968750, -0.0013122558593750, 0.0010070800781250, -0.0008850097656250,
+0.0020141601562500, -0.0008544921875000, 0.0000915527343750, -0.0009460449218750,
+0.0005187988281250, -0.0004882812500000, 0.0005493164062500, -0.0001525878906250,
+-0.0004882812500000, -0.0003967285156250, 0.0005187988281250, 0.0002746582031250,
+0.0000000000000000, 0.0001220703125000, 0.0003051757812500, 0.0002746582031250,
+0.0003051757812500, 0.0007934570312500, 0.0000000000000000, 0.0004882812500000,
+0.0007934570312500, 0.0007019042968750, 0.0006103515625000, 0.0009155273437500,
+0.0008544921875000, 0.0007019042968750, 0.0010070800781250, 0.0006103515625000,
+0.0013122558593750, 0.0009460449218750, 0.0021667480468750, 0.0004272460937500,
+0.0011291503906250, 0.0004882812500000, 0.0022583007812500, 0.0006713867187500,
+0.0018615722656250, 0.0004882812500000, 0.0009155273437500, 0.0005187988281250,
+0.0027465820312500, 0.0008850097656250, 0.0006713867187500, 0.0010375976562500,
+0.0016479492187500, 0.0011901855468750, 0.0016784667968750, 0.0014038085937500,
+0.0003356933593750, 0.0014038085937500, 0.0018005371093750, 0.0016784667968750,
+0.0006103515625000, 0.0017395019531250, 0.0006713867187500, 0.0018005371093750,
+0.0004882812500000, 0.0020141601562500, 0.0000915527343750, 0.0020751953125000,
+0.0010375976562500, 0.0020141601562500, -0.0003967285156250, 0.0022583007812500,
+0.0006103515625000, 0.0020751953125000, 0.0003967285156250, 0.0018005371093750,
+-0.0007324218750000, 0.0020446777343750, 0.0010070800781250, 0.0015869140625000,
+-0.0003051757812500, 0.0014953613281250, 0.0002441406250000, 0.0015869140625000,
+0.0004882812500000, 0.0015258789062500, -0.0001831054687500, 0.0015258789062500,
+0.0006408691406250, 0.0016784667968750, -0.0000305175781250, 0.0018615722656250,
+0.0003967285156250, 0.0016784667968750, 0.0003967285156250, 0.0015869140625000,
+0.0000000000000000, 0.0017700195312500, 0.0008544921875000, 0.0012512207031250,
+0.0003356933593750, 0.0011901855468750, 0.0007019042968750, 0.0011596679687500,
+0.0010986328125000, 0.0007324218750000, 0.0010986328125000, 0.0005798339843750,
+0.0014953613281250, 0.0009765625000000, 0.0018310546875000, 0.0006103515625000,
+0.0023498535156250, 0.0007629394531250, 0.0025329589843750, 0.0013122558593750,
+0.0028381347656250, 0.0010986328125000, 0.0034790039062500, 0.0016784667968750,
+0.0029602050781250, 0.0017700195312500, 0.0035400390625000, 0.0019226074218750,
+0.0033569335937500, 0.0023193359375000, 0.0026855468750000, 0.0022277832031250,
+0.0033264160156250, 0.0023193359375000, 0.0017395019531250, 0.0024414062500000,
+0.0022277832031250, 0.0021362304687500, 0.0014953613281250, 0.0018615722656250,
+0.0001525878906250, 0.0019226074218750, 0.0011596679687500, 0.0015258789062500,
+-0.0010681152343750, 0.0012512207031250, -0.0004272460937500, 0.0013732910156250,
+-0.0005187988281250, 0.0013427734375000, -0.0017700195312500, 0.0010986328125000,
+-0.0004882812500000, 0.0007934570312500, -0.0012512207031250, 0.0010375976562500,
+-0.0006408691406250, 0.0004577636718750, -0.0003662109375000, -0.0004272460937500,
+-0.0004882812500000, -0.0001831054687500, 0.0006713867187500, -0.0004272460937500,
+-0.0006103515625000, -0.0010681152343750, 0.0004577636718750, -0.0006103515625000,
+0.0002441406250000, 0.0000000000000000, -0.0008544921875000, -0.0006103515625000,
+0.0004272460937500, -0.0000915527343750, -0.0016174316406250, 0.0002441406250000,
+-0.0010375976562500, -0.0002136230468750, -0.0014343261718750, -0.0005187988281250,
+-0.0031433105468750, -0.0001831054687500, -0.0020141601562500, -0.0006408691406250,
+-0.0030212402343750, -0.0007019042968750, -0.0042114257812500, -0.0004272460937500,
+-0.0029602050781250, -0.0004577636718750, -0.0025634765625000, -0.0004272460937500,
+-0.0040588378906250, -0.0005798339843750, -0.0002746582031250, -0.0004272460937500,
+-0.0013732910156250, -0.0006713867187500, -0.0010070800781250, -0.0011291503906250,
+0.0015258789062500, -0.0009460449218750, -0.0003967285156250, -0.0010375976562500,
+0.0011291503906250, -0.0015869140625000, 0.0011291503906250, -0.0010986328125000,
+0.0000610351562500, -0.0006103515625000, 0.0002136230468750, -0.0013732910156250,
+0.0001220703125000, -0.0002441406250000, -0.0016174316406250, -0.0004882812500000,
+-0.0012817382812500, -0.0004272460937500, -0.0018310546875000, 0.0001525878906250,
+-0.0032348632812500, -0.0005187988281250, -0.0025329589843750, -0.0000915527343750,
+-0.0032043457031250, -0.0002136230468750, -0.0031433105468750, -0.0002136230468750,
+-0.0029296875000000, -0.0000915527343750, -0.0030212402343750, -0.0007019042968750,
+-0.0027160644531250, 0.0001831054687500, -0.0025634765625000, 0.0000000000000000,
+-0.0021972656250000, -0.0003051757812500, -0.0022888183593750, 0.0001220703125000,
+-0.0024719238281250, -0.0000610351562500, -0.0019531250000000, -0.0008544921875000,
+-0.0019836425781250, -0.0004272460937500, -0.0024108886718750, -0.0007324218750000,
+-0.0016784667968750, -0.0014648437500000, -0.0007019042968750, -0.0007324218750000,
+-0.0012817382812500, -0.0012512207031250, -0.0007019042968750, -0.0007019042968750,
+0.0000000000000000, -0.0010070800781250, -0.0011901855468750, -0.0025024414062500,
+-0.0031738281250000, -0.0019836425781250, -0.0012207031250000, 0.0002441406250000,
+-0.0027160644531250, -0.0008850097656250, -0.0012817382812500, -0.0004272460937500,
+-0.0023193359375000, 0.0011291503906250, -0.0040893554687500, 0.0002136230468750,
+-0.0030517578125000, -0.0015869140625000, -0.0041503906250000, -0.0002136230468750,
+-0.0044250488281250, -0.0021667480468750, -0.0035705566406250, -0.0052490234375000,
+-0.0032653808593750, -0.0033264160156250, -0.0034179687500000, -0.0057067871093750,
+-0.0027465820312500, -0.0066223144531250, -0.0021972656250000, -0.0047607421875000,
+-0.0025024414062500, -0.0041809082031250, -0.0028381347656250, -0.0050659179687500,
+-0.0024414062500000, -0.0019531250000000, -0.0028381347656250, -0.0017700195312500,
+-0.0031738281250000, -0.0024108886718750, -0.0025024414062500, -0.0014343261718750,
+-0.0021057128906250, -0.0013122558593750, -0.0027465820312500, -0.0025329589843750,
+-0.0018005371093750, -0.0030212402343750, -0.0013427734375000, -0.0022583007812500,
+-0.0021057128906250, -0.0022888183593750, -0.0025024414062500, -0.0031127929687500,
+-0.0020446777343750, -0.0008850097656250, -0.0030212402343750, -0.0007934570312500,
+-0.0033874511718750, -0.0010986328125000, -0.0029907226562500, 0.0003967285156250,
+-0.0031433105468750, 0.0007934570312500, -0.0035095214843750, -0.0012817382812500,
+-0.0022888183593750, 0.0005798339843750, -0.0025329589843750, -0.0008850097656250,
+-0.0027465820312500, -0.0037231445312500, -0.0020446777343750, -0.0010681152343750,
+-0.0021972656250000, -0.0041198730468750, -0.0029296875000000, -0.0046691894531250,
+-0.0029296875000000, -0.0034179687500000, -0.0025939941406250, -0.0039367675781250,
+-0.0028686523437500, -0.0042114257812500, -0.0029907226562500, -0.0020751953125000,
+-0.0016174316406250, -0.0028686523437500, -0.0018005371093750, -0.0020751953125000,
+-0.0015869140625000, -0.0001525878906250, -0.0004272460937500, -0.0014038085937500,
+-0.0006408691406250, 0.0003051757812500, -0.0003967285156250, 0.0009765625000000,
+-0.0000610351562500, 0.0000000000000000, -0.0000610351562500, 0.0000610351562500,
+-0.0000610351562500, 0.0008239746093750, 0.0002136230468750, -0.0011596679687500,
+0.0001831054687500, -0.0007629394531250, 0.0004882812500000, -0.0007934570312500,
+0.0003051757812500, -0.0019531250000000, 0.0000000000000000, -0.0011596679687500,
+0.0004272460937500},
+{-0.0003662109375000, -0.0004577636718750, -0.0022277832031250, -0.0008544921875000,
+0.0000305175781250, -0.0008544921875000, 0.0000000000000000, -0.0008239746093750,
+-0.0009155273437500, -0.0007324218750000, -0.0000915527343750, -0.0003356933593750,
+0.0000000000000000, -0.0005493164062500, -0.0014648437500000, 0.0000000000000000,
+-0.0009155273437500, -0.0004272460937500, -0.0017395019531250, -0.0007934570312500,
+-0.0034179687500000, 0.0003051757812500, -0.0025329589843750, -0.0019836425781250,
+-0.0038757324218750, -0.0010681152343750, -0.0045471191406250, -0.0012512207031250,
+-0.0036926269531250, -0.0028076171875000, -0.0032043457031250, -0.0013122558593750,
+-0.0036315917968750, -0.0022277832031250, -0.0025634765625000, -0.0018310546875000,
+-0.0018005371093750, -0.0015258789062500, -0.0023498535156250, -0.0018310546875000,
+-0.0024414062500000, -0.0010986328125000, -0.0015258789062500, -0.0021362304687500,
+-0.0025939941406250, -0.0019226074218750, -0.0023498535156250, -0.0013427734375000,
+-0.0017089843750000, -0.0016174316406250, -0.0018310546875000, -0.0000610351562500,
+-0.0013122558593750, -0.0014648437500000, -0.0021972656250000, -0.0000915527343750,
+-0.0019836425781250, -0.0021057128906250, -0.0016784667968750, -0.0042724609375000,
+-0.0016174316406250, -0.0025329589843750, -0.0016784667968750, -0.0048522949218750,
+-0.0018005371093750, -0.0056152343750000, -0.0013427734375000, -0.0040893554687500,
+-0.0021362304687500, -0.0039062500000000, -0.0031738281250000, -0.0046081542968750,
+-0.0024108886718750, -0.0016784667968750, -0.0032348632812500, -0.0019836425781250,
+-0.0037841796875000, -0.0020446777343750, -0.0030822753906250, -0.0004882812500000,
+-0.0026550292968750, -0.0011596679687500, -0.0031433105468750, -0.0016479492187500,
+-0.0019531250000000, -0.0017395019531250, -0.0018310546875000, -0.0016784667968750,
+-0.0023498535156250, -0.0020446777343750, -0.0021667480468750, -0.0025634765625000,
+-0.0020446777343750, -0.0009155273437500, -0.0025634765625000, -0.0010070800781250,
+-0.0028991699218750, -0.0016479492187500, -0.0025634765625000, -0.0007934570312500,
+-0.0022583007812500, -0.0004272460937500, -0.0025329589843750, -0.0026855468750000,
+-0.0021667480468750, -0.0018005371093750, -0.0018310546875000, -0.0023498535156250,
+-0.0021972656250000, -0.0045166015625000, -0.0023498535156250, -0.0034484863281250,
+-0.0019836425781250, -0.0036926269531250, -0.0030822753906250, -0.0048217773437500,
+-0.0030212402343750, -0.0030212402343750, -0.0026855468750000, -0.0017089843750000,
+-0.0030822753906250, -0.0034790039062500, -0.0031738281250000, 0.0003662109375000,
+-0.0025939941406250, 0.0001220703125000, -0.0024108886718750, -0.0002746582031250,
+-0.0026245117187500, 0.0014953613281250, -0.0026245117187500, 0.0006713867187500,
+-0.0024414062500000, 0.0003051757812500, -0.0025939941406250, 0.0006713867187500,
+-0.0028381347656250, -0.0000610351562500, -0.0023193359375000, -0.0008544921875000,
+-0.0018920898437500, -0.0002441406250000, -0.0021972656250000, -0.0011291503906250,
+-0.0010681152343750, -0.0016479492187500, -0.0009155273437500, -0.0009460449218750,
+-0.0010070800781250, -0.0007934570312500, -0.0004272460937500, -0.0013732910156250,
+-0.0004272460937500, 0.0003356933593750, -0.0006408691406250, 0.0003662109375000,
+-0.0006408691406250, 0.0004272460937500, -0.0003356933593750, 0.0015258789062500,
+-0.0002136230468750, 0.0015869140625000, -0.0003356933593750, 0.0017089843750000,
+0.0002136230468750, 0.0023803710937500, 0.0002136230468750, 0.0021972656250000,
+0.0001525878906250, 0.0021362304687500, 0.0002746582031250, 0.0029296875000000,
+0.0002746582031250, 0.0013122558593750, 0.0007934570312500, 0.0021972656250000,
+0.0005187988281250, 0.0017700195312500, 0.0009460449218750, 0.0003356933593750,
+0.0017700195312500, 0.0015258789062500, 0.0013427734375000, 0.0001525878906250,
+0.0018005371093750, 0.0001831054687500, 0.0021057128906250, 0.0005187988281250,
+0.0018920898437500, -0.0000915527343750, 0.0018310546875000, 0.0000915527343750,
+0.0020446777343750, 0.0007019042968750, 0.0019531250000000, 0.0003051757812500,
+0.0019836425781250, 0.0008544921875000, 0.0020751953125000, 0.0016174316406250,
+0.0021667480468750, 0.0009765625000000, 0.0022583007812500, 0.0024414062500000,
+0.0022277832031250, 0.0022888183593750, 0.0023498535156250, 0.0026245117187500,
+0.0023803710937500, 0.0036621093750000, 0.0023803710937500, 0.0031127929687500,
+0.0026550292968750, 0.0037841796875000, 0.0023498535156250, 0.0040893554687500,
+0.0026245117187500, 0.0037536621093750, 0.0024414062500000, 0.0039367675781250,
+0.0020751953125000, 0.0044860839843750, 0.0024108886718750, 0.0035705566406250,
+0.0017700195312500, 0.0043029785156250, 0.0018005371093750, 0.0045166015625000,
+0.0018310546875000, 0.0034790039062500, 0.0014648437500000, 0.0037231445312500,
+0.0015869140625000, 0.0039672851562500, 0.0018920898437500, 0.0036621093750000,
+0.0016479492187500, 0.0038146972656250, 0.0018005371093750, 0.0041198730468750,
+0.0021972656250000, 0.0036926269531250, 0.0019226074218750, 0.0042724609375000,
+0.0022277832031250, 0.0040283203125000, 0.0022583007812500, 0.0040283203125000,
+0.0021667480468750, 0.0044250488281250, 0.0023193359375000, 0.0039978027343750,
+0.0021972656250000, 0.0043334960937500, 0.0021057128906250, 0.0047607421875000,
+0.0022277832031250, 0.0045166015625000, 0.0020446777343750, 0.0046081542968750,
+0.0017700195312500, 0.0052490234375000, 0.0019226074218750, 0.0043945312500000,
+0.0014953613281250, 0.0049133300781250, 0.0015563964843750, 0.0047607421875000,
+0.0014343261718750, 0.0039978027343750, 0.0011901855468750, 0.0046081542968750,
+0.0013732910156250, 0.0036010742187500, 0.0010070800781250, 0.0038146972656250,
+0.0010681152343750, 0.0036621093750000, 0.0007934570312500, 0.0029602050781250,
+0.0004272460937500, 0.0034179687500000, 0.0006408691406250, 0.0023803710937500,
+-0.0002441406250000, 0.0025024414062500, -0.0002746582031250, 0.0024108886718750,
+-0.0004272460937500, 0.0015869140625000, -0.0010070800781250, 0.0016479492187500,
+-0.0008544921875000, 0.0015258789062500, -0.0010681152343750, 0.0011291503906250,
+-0.0013122558593750, 0.0010681152343750, -0.0011291503906250, 0.0011291503906250,
+-0.0008544921875000, 0.0008544921875000, -0.0012207031250000, 0.0007934570312500,
+-0.0009765625000000, 0.0010681152343750, -0.0008850097656250, 0.0007934570312500,
+-0.0009460449218750, 0.0008850097656250, -0.0010070800781250, 0.0018310546875000,
+-0.0010375976562500, 0.0008544921875000, -0.0006713867187500, 0.0016479492187500,
+-0.0006408691406250, 0.0017089843750000, -0.0008544921875000, 0.0010375976562500,
+-0.0008239746093750, 0.0020141601562500, -0.0005798339843750, 0.0012207031250000,
+-0.0010070800781250, 0.0014343261718750, -0.0010986328125000, 0.0016174316406250,
+-0.0008850097656250, 0.0011596679687500, -0.0009155273437500, 0.0014648437500000,
+-0.0009460449218750, 0.0008239746093750, -0.0003356933593750, 0.0008239746093750,
+-0.0003967285156250, 0.0006713867187500, -0.0003967285156250, 0.0003051757812500,
+0.0000915527343750, 0.0006103515625000, -0.0000305175781250, 0.0004577636718750,
+-0.0008544921875000, 0.0007019042968750, -0.0005798339843750, 0.0008850097656250,
+-0.0007629394531250, 0.0009460449218750, -0.0015869140625000, 0.0013122558593750,
+-0.0014038085937500, 0.0005798339843750, -0.0011596679687500, 0.0011596679687500,
+-0.0016174316406250, 0.0006103515625000, -0.0011596679687500, -0.0004272460937500,
+-0.0004577636718750, 0.0001831054687500, -0.0009460449218750, -0.0018005371093750,
+-0.0003356933593750, -0.0017700195312500, -0.0000915527343750, -0.0022583007812500,
+-0.0003051757812500, -0.0035095214843750, -0.0003662109375000, -0.0028686523437500,
+-0.0002441406250000, -0.0042114257812500, -0.0002746582031250, -0.0036315917968750,
+-0.0003662109375000, -0.0039367675781250, -0.0001525878906250, -0.0045471191406250,
+0.0001220703125000, -0.0027160644531250, -0.0002136230468750, -0.0044860839843750,
+-0.0002136230468750, -0.0051879882812500, 0.0000610351562500, -0.0041503906250000,
+-0.0003662109375000, -0.0040588378906250, -0.0006713867187500, -0.0045776367187500,
+-0.0006408691406250, -0.0025329589843750, -0.0025024414062500, -0.0026855468750000,
+-0.0017089843750000, -0.0028076171875000, -0.0022888183593750, -0.0012817382812500,
+-0.0040893554687500, -0.0012512207031250, -0.0031127929687500, -0.0026855468750000,
+-0.0041809082031250, -0.0016479492187500, -0.0039672851562500, -0.0021972656250000,
+-0.0034484863281250, -0.0038146972656250, -0.0033874511718750, -0.0024719238281250,
+-0.0024414062500000, -0.0032653808593750, -0.0016784667968750, -0.0028381347656250,
+-0.0017700195312500, -0.0029602050781250, -0.0021362304687500, -0.0035095214843750,
+-0.0020141601562500, -0.0024108886718750, -0.0017089843750000, -0.0036621093750000,
+-0.0025024414062500, -0.0032958984375000, -0.0020141601562500, -0.0026245117187500,
+-0.0022277832031250, -0.0029602050781250, -0.0028686523437500, -0.0025634765625000,
+-0.0021667480468750, -0.0015258789062500, -0.0028381347656250, -0.0014648437500000,
+-0.0031738281250000, -0.0010681152343750, -0.0031738281250000, -0.0003356933593750,
+-0.0036010742187500, -0.0005493164062500, -0.0041809082031250, -0.0001220703125000,
+-0.0038757324218750, -0.0005187988281250, -0.0050659179687500, -0.0000915527343750,
+-0.0049133300781250, 0.0001220703125000, -0.0043334960937500, -0.0009765625000000,
+-0.0057678222656250, 0.0012207031250000, -0.0039367675781250, 0.0006408691406250,
+-0.0046081542968750, 0.0005798339843750, -0.0043029785156250, 0.0021362304687500,
+-0.0027465820312500, 0.0013427734375000, -0.0037841796875000, 0.0009765625000000,
+-0.0026245117187500, 0.0015869140625000, -0.0025939941406250, 0.0007934570312500,
+-0.0026245117187500, -0.0003356933593750, -0.0020446777343750, 0.0002441406250000,
+-0.0023498535156250, -0.0009155273437500, -0.0014343261718750, -0.0009765625000000,
+-0.0014953613281250, -0.0008544921875000, -0.0010986328125000, -0.0012207031250000,
+-0.0001525878906250, -0.0010375976562500, -0.0002136230468750, -0.0012207031250000,
+0.0004882812500000, -0.0010375976562500, 0.0011901855468750, -0.0011596679687500,
+0.0011596679687500, -0.0012512207031250, 0.0015258789062500, -0.0007629394531250,
+0.0022277832031250, -0.0013732910156250, 0.0011596679687500, -0.0012512207031250,
+0.0022888183593750, -0.0013427734375000, 0.0016784667968750, -0.0015869140625000,
+0.0004272460937500, -0.0010681152343750, 0.0021972656250000, -0.0017395019531250,
+0.0000000000000000, -0.0015869140625000, 0.0012817382812500, -0.0012207031250000,
+0.0013732910156250, -0.0013122558593750, 0.0001525878906250, -0.0010070800781250,
+0.0022888183593750, -0.0008850097656250, 0.0002746582031250, -0.0005187988281250,
+0.0013732910156250, -0.0004272460937500, 0.0013122558593750, -0.0003051757812500,
+-0.0001525878906250, 0.0000610351562500, 0.0014343261718750, -0.0005798339843750,
+-0.0003662109375000, -0.0000305175781250, 0.0002136230468750, -0.0003051757812500,
+-0.0000610351562500, -0.0010986328125000, -0.0012207031250000, -0.0003967285156250,
+0.0003356933593750, -0.0011596679687500, -0.0008850097656250, -0.0011596679687500,
+-0.0001831054687500, -0.0008239746093750, 0.0004272460937500, -0.0009765625000000,
+0.0001525878906250, -0.0009155273437500, 0.0013427734375000, -0.0005187988281250,
+0.0010070800781250, -0.0006103515625000, 0.0018615722656250, -0.0003967285156250,
+0.0021667480468750, -0.0001220703125000, 0.0018920898437500, -0.0003051757812500,
+0.0027160644531250, 0.0005187988281250, 0.0028381347656250, 0.0003662109375000,
+0.0029602050781250},
+{-0.0038146972656250, -0.0034484863281250, -0.0037841796875000, -0.0030517578125000,
+-0.0031738281250000, -0.0028686523437500, -0.0036010742187500, -0.0031127929687500,
+-0.0030822753906250, -0.0021972656250000, -0.0025024414062500, -0.0016784667968750,
+-0.0027160644531250, -0.0021057128906250, -0.0024414062500000, -0.0021667480468750,
+-0.0019836425781250, -0.0017700195312500, -0.0024719238281250, -0.0029907226562500,
+-0.0024719238281250, -0.0023193359375000, -0.0026855468750000, -0.0026245117187500,
+-0.0026245117187500, -0.0036926269531250, -0.0022888183593750, -0.0026245117187500,
+-0.0024414062500000, -0.0041503906250000, -0.0021667480468750, -0.0036315917968750,
+-0.0022583007812500, -0.0032653808593750, -0.0022888183593750, -0.0038757324218750,
+-0.0022277832031250, -0.0029296875000000, -0.0026855468750000, -0.0028076171875000,
+-0.0028076171875000, -0.0025939941406250, -0.0032043457031250, -0.0021057128906250,
+-0.0034179687500000, -0.0016479492187500, -0.0037841796875000, -0.0013122558593750,
+-0.0045471191406250, -0.0011291503906250, -0.0037841796875000, -0.0006408691406250,
+-0.0043945312500000, -0.0007934570312500, -0.0043334960937500, -0.0010375976562500,
+-0.0036621093750000, -0.0005798339843750, -0.0042724609375000, -0.0006103515625000,
+-0.0032958984375000, -0.0014648437500000, -0.0033874511718750, -0.0005798339843750,
+-0.0032653808593750, 0.0002136230468750, -0.0026550292968750, -0.0009765625000000,
+-0.0029296875000000, 0.0014648437500000, -0.0018920898437500, 0.0011901855468750,
+-0.0019836425781250, 0.0007019042968750, -0.0015869140625000, 0.0019836425781250,
+-0.0004272460937500, 0.0018005371093750, -0.0004272460937500, 0.0005187988281250,
+-0.0003662109375000, 0.0010375976562500, 0.0006103515625000, 0.0004882812500000,
+0.0002136230468750, -0.0008239746093750, -0.0002136230468750, -0.0001831054687500,
+0.0012207031250000, -0.0007934570312500, -0.0003356933593750, -0.0010375976562500,
+0.0009765625000000, -0.0007629394531250, 0.0007324218750000, -0.0007324218750000,
+-0.0002746582031250, -0.0008239746093750, 0.0018615722656250, -0.0011901855468750,
+-0.0003356933593750, -0.0007324218750000, 0.0010070800781250, -0.0010070800781250,
+0.0009765625000000, -0.0017700195312500, -0.0002746582031250, -0.0013732910156250,
+0.0019531250000000, -0.0016174316406250, -0.0005798339843750, -0.0017700195312500,
+0.0009765625000000, -0.0015869140625000, 0.0006408691406250, -0.0017395019531250,
+-0.0014038085937500, -0.0018310546875000, 0.0010070800781250, -0.0012207031250000,
+-0.0014038085937500, -0.0012817382812500, -0.0007324218750000, -0.0012817382812500,
+-0.0001831054687500, -0.0008850097656250, -0.0011291503906250, -0.0008544921875000,
+0.0003356933593750, -0.0006103515625000, -0.0004272460937500, -0.0004882812500000,
+0.0005798339843750, -0.0004577636718750, 0.0008239746093750, -0.0002441406250000,
+0.0004882812500000, -0.0000915527343750, 0.0019531250000000, -0.0004272460937500,
+0.0009765625000000, -0.0002136230468750, 0.0019226074218750, -0.0002746582031250,
+0.0020141601562500, -0.0006103515625000, 0.0013427734375000, -0.0003967285156250,
+0.0025024414062500, -0.0003051757812500, 0.0010375976562500, -0.0003662109375000,
+0.0015869140625000, -0.0001525878906250, 0.0015258789062500, 0.0000000000000000,
+0.0003967285156250, -0.0001220703125000, 0.0008239746093750, 0.0003051757812500,
+0.0000305175781250, 0.0000915527343750, 0.0000915527343750, 0.0002441406250000,
+0.0000610351562500, 0.0005187988281250, -0.0006103515625000, 0.0002746582031250,
+-0.0003967285156250, 0.0009765625000000, -0.0002746582031250, 0.0009155273437500,
+-0.0007324218750000, 0.0009765625000000, -0.0004272460937500, 0.0013427734375000,
+0.0001525878906250, 0.0011901855468750, -0.0001525878906250, 0.0016479492187500,
+0.0003356933593750, 0.0014038085937500, 0.0004272460937500, 0.0017089843750000,
+0.0004272460937500, 0.0021667480468750, 0.0004272460937500, 0.0017089843750000,
+0.0002746582031250, 0.0026550292968750, 0.0010681152343750, 0.0025634765625000,
+0.0002441406250000, 0.0025634765625000, 0.0004577636718750, 0.0031127929687500,
+0.0011596679687500, 0.0027770996093750, 0.0000000000000000, 0.0028991699218750,
+0.0006408691406250, 0.0029602050781250, 0.0004272460937500, 0.0028076171875000,
+0.0001525878906250, 0.0026855468750000, 0.0004272460937500, 0.0026245117187500,
+0.0000915527343750, 0.0024414062500000, 0.0003356933593750, 0.0024414062500000,
+0.0002136230468750, 0.0022583007812500, 0.0000000000000000, 0.0021057128906250,
+0.0000305175781250, 0.0021362304687500, -0.0000305175781250, 0.0021057128906250,
+-0.0006103515625000, 0.0020751953125000, -0.0003662109375000, 0.0021057128906250,
+-0.0006103515625000, 0.0022277832031250, -0.0012207031250000, 0.0023193359375000,
+-0.0007324218750000, 0.0017700195312500, -0.0010375976562500, 0.0021972656250000,
+-0.0014038085937500, 0.0018920898437500, -0.0009155273437500, 0.0012512207031250,
+-0.0007629394531250, 0.0018310546875000, -0.0014343261718750, 0.0010681152343750,
+0.0001220703125000, 0.0010681152343750, -0.0005187988281250, 0.0011596679687500,
+0.0000610351562500, 0.0007629394531250, 0.0015869140625000, 0.0007629394531250,
+0.0006103515625000, 0.0007324218750000, 0.0026550292968750, 0.0006713867187500,
+0.0028991699218750, 0.0004882812500000, 0.0031127929687500, 0.0005493164062500,
+0.0043334960937500, 0.0007629394531250, 0.0042419433593750, 0.0000915527343750,
+0.0044250488281250, 0.0004882812500000, 0.0048217773437500, 0.0003051757812500,
+0.0045166015625000, -0.0003051757812500, 0.0041809082031250, 0.0001525878906250,
+0.0043334960937500, -0.0003662109375000, 0.0033569335937500, -0.0003051757812500,
+0.0032348632812500, -0.0002746582031250, 0.0027770996093750, -0.0005798339843750,
+0.0018920898437500, -0.0003356933593750, 0.0018920898437500, -0.0010070800781250,
+0.0016174316406250, -0.0007019042968750, 0.0012512207031250, -0.0009765625000000,
+0.0010681152343750, -0.0016479492187500, 0.0011291503906250, -0.0011596679687500,
+0.0010681152343750, -0.0018005371093750, 0.0006713867187500, -0.0017700195312500,
+0.0010681152343750, -0.0017395019531250, 0.0006713867187500, -0.0019531250000000,
+0.0002746582031250, -0.0017089843750000, 0.0011596679687500, -0.0018920898437500,
+-0.0007019042968750, -0.0017089843750000, 0.0001220703125000, -0.0020446777343750,
+-0.0005187988281250, -0.0023193359375000, -0.0021972656250000, -0.0018005371093750,
+-0.0006408691406250, -0.0026245117187500, -0.0019836425781250, -0.0025634765625000,
+-0.0015258789062500, -0.0020751953125000, -0.0006713867187500, -0.0021972656250000,
+-0.0008239746093750, -0.0020446777343750, -0.0000610351562500, -0.0009460449218750,
+0.0007324218750000, -0.0009460449218750, 0.0009460449218750, -0.0008239746093750,
+0.0011596679687500, 0.0000305175781250, 0.0016784667968750, -0.0000610351562500,
+0.0017089843750000, -0.0008544921875000, 0.0007934570312500, -0.0007019042968750,
+0.0013122558593750, -0.0008544921875000, 0.0004272460937500, -0.0015563964843750,
+-0.0010986328125000, -0.0015563964843750, -0.0005798339843750, -0.0018005371093750,
+-0.0027160644531250, -0.0023803710937500, -0.0031433105468750, -0.0021362304687500,
+-0.0028991699218750, -0.0020141601562500, -0.0037536621093750, -0.0028381347656250,
+-0.0036315917968750, -0.0015563964843750, -0.0015869140625000, -0.0021972656250000,
+-0.0015563964843750, -0.0020446777343750, -0.0010070800781250, -0.0011596679687500,
+0.0009155273437500, -0.0021362304687500, 0.0009155273437500, -0.0008544921875000,
+-0.0003356933593750, -0.0010986328125000, 0.0006408691406250, -0.0013732910156250,
+-0.0000915527343750, -0.0005798339843750, -0.0021972656250000, -0.0010375976562500,
+-0.0012512207031250, -0.0018920898437500, -0.0010986328125000, -0.0014038085937500,
+-0.0021667480468750, -0.0018615722656250, -0.0014648437500000, -0.0034484863281250,
+0.0005493164062500, 0.0000305175781250, 0.0007629394531250, 0.0018920898437500,
+0.0006408691406250, 0.0017700195312500, -0.0000305175781250, 0.0014343261718750,
+0.0003356933593750, 0.0017395019531250, 0.0009460449218750, 0.0018615722656250,
+0.0001220703125000, 0.0011596679687500, -0.0005493164062500, 0.0009155273437500,
+0.0001525878906250, 0.0012512207031250, -0.0008544921875000, 0.0015563964843750,
+-0.0024414062500000, 0.0014343261718750, -0.0015563964843750, 0.0008850097656250,
+-0.0018310546875000, 0.0021972656250000, -0.0023498535156250, 0.0005493164062500,
+-0.0018310546875000, -0.0012512207031250, -0.0010681152343750, 0.0008850097656250,
+-0.0012817382812500, -0.0032958984375000, -0.0013427734375000, -0.0029907226562500,
+-0.0006408691406250, -0.0023498535156250, -0.0012207031250000, -0.0044555664062500,
+-0.0021972656250000, -0.0035705566406250, -0.0013427734375000, -0.0015869140625000,
+-0.0015869140625000, -0.0021362304687500, -0.0021972656250000, -0.0011291503906250,
+-0.0014343261718750, 0.0012817382812500, -0.0006408691406250, 0.0008239746093750,
+-0.0011901855468750, 0.0012207031250000, -0.0006408691406250, 0.0024108886718750,
+-0.0003662109375000, 0.0016479492187500, -0.0011596679687500, 0.0007019042968750,
+-0.0016479492187500, 0.0018310546875000, -0.0012512207031250, -0.0000610351562500,
+-0.0023193359375000, -0.0001525878906250, -0.0027770996093750, 0.0003051757812500,
+-0.0023193359375000, -0.0005493164062500, -0.0022583007812500, -0.0007324218750000,
+-0.0026550292968750, 0.0007324218750000, -0.0025634765625000, 0.0002136230468750,
+-0.0022583007812500, 0.0004882812500000, -0.0027160644531250, 0.0015869140625000,
+-0.0032348632812500, 0.0008850097656250, -0.0029296875000000, 0.0018920898437500,
+-0.0032043457031250, 0.0018310546875000, -0.0033569335937500, 0.0014343261718750,
+-0.0030212402343750, 0.0017395019531250, -0.0027770996093750, 0.0016479492187500,
+-0.0028686523437500, 0.0009765625000000, -0.0023803710937500, 0.0008239746093750,
+-0.0023803710937500, 0.0012512207031250, -0.0021057128906250, 0.0009155273437500,
+-0.0018310546875000, 0.0004272460937500, -0.0020446777343750, 0.0024719238281250,
+-0.0009460449218750, 0.0015563964843750, -0.0010986328125000, 0.0021057128906250,
+-0.0008239746093750, 0.0038146972656250, 0.0001525878906250, 0.0025329589843750,
+-0.0001220703125000, 0.0040893554687500, -0.0000915527343750, 0.0038757324218750,
+0.0002746582031250, 0.0036926269531250, 0.0000915527343750, 0.0044860839843750,
+-0.0002136230468750, 0.0039062500000000, 0.0001525878906250, 0.0037841796875000,
+-0.0000305175781250, 0.0041198730468750, 0.0000000000000000, 0.0032653808593750,
+0.0002136230468750, 0.0025939941406250, 0.0003051757812500, 0.0031433105468750,
+0.0003662109375000, 0.0013122558593750, 0.0005187988281250, 0.0016784667968750,
+0.0006103515625000, 0.0013122558593750, 0.0007934570312500, 0.0000305175781250,
+0.0009155273437500, 0.0008850097656250, 0.0009155273437500, -0.0003356933593750,
+0.0013732910156250, -0.0002136230468750, 0.0014343261718750, 0.0000000000000000,
+0.0013427734375000, -0.0005493164062500, 0.0015869140625000, -0.0002136230468750,
+0.0016784667968750, -0.0001220703125000, 0.0013732910156250, -0.0001525878906250,
+0.0013732910156250, -0.0001525878906250, 0.0014953613281250, -0.0001220703125000,
+0.0013122558593750, 0.0000915527343750, 0.0012512207031250, 0.0000305175781250,
+0.0015869140625000, 0.0002746582031250, 0.0014343261718750, 0.0005798339843750,
+0.0015563964843750, 0.0006408691406250, 0.0018615722656250, 0.0007629394531250,
+0.0016479492187500, 0.0009765625000000, 0.0020751953125000, 0.0009460449218750,
+0.0021057128906250, 0.0009460449218750, 0.0021972656250000, 0.0014648437500000,
+0.0024414062500000},
+{0.0015869140625000, -0.0003967285156250, 0.0020446777343750, 0.0008544921875000,
+0.0018615722656250, -0.0027160644531250, 0.0005798339843750, -0.0024719238281250,
+0.0008544921875000, -0.0018920898437500, 0.0005187988281250, -0.0037536621093750,
+-0.0007629394531250, -0.0033569335937500, -0.0004577636718750, -0.0010681152343750,
+-0.0001831054687500, -0.0017700195312500, -0.0008850097656250, -0.0008239746093750,
+-0.0003356933593750, 0.0017089843750000, 0.0008850097656250, 0.0008850097656250,
+0.0003051757812500, 0.0006408691406250, 0.0002136230468750, 0.0019226074218750,
+0.0010070800781250, 0.0009460449218750, 0.0004882812500000, -0.0007934570312500,
+-0.0003356933593750, 0.0003356933593750, 0.0002746582031250, -0.0005493164062500,
+-0.0004577636718750, -0.0016784667968750, -0.0005798339843750, -0.0005493164062500,
+-0.0004272460937500, 0.0000000000000000, -0.0004272460937500, -0.0015258789062500,
+-0.0004882812500000, 0.0013427734375000, -0.0012512207031250, 0.0005798339843750,
+-0.0007019042968750, 0.0001831054687500, -0.0011901855468750, 0.0018920898437500,
+-0.0024719238281250, 0.0009460449218750, -0.0018920898437500, 0.0005798339843750,
+-0.0023498535156250, 0.0005798339843750, -0.0026855468750000, 0.0006713867187500,
+-0.0025939941406250, 0.0003356933593750, -0.0025024414062500, -0.0000610351562500,
+-0.0025939941406250, 0.0009765625000000, -0.0028991699218750, 0.0003356933593750,
+-0.0028381347656250, 0.0011291503906250, -0.0029602050781250, 0.0021667480468750,
+-0.0032348632812500, 0.0009460449218750, -0.0030212402343750, 0.0039978027343750,
+-0.0030212402343750, 0.0031738281250000, -0.0031738281250000, 0.0034179687500000,
+-0.0028076171875000, 0.0055541992187500, -0.0025024414062500, 0.0040893554687500,
+-0.0026550292968750, 0.0045776367187500, -0.0020751953125000, 0.0047912597656250,
+-0.0020446777343750, 0.0039367675781250, -0.0017089843750000, 0.0034484863281250,
+-0.0012817382812500, 0.0034790039062500, -0.0014953613281250, 0.0026245117187500,
+-0.0003356933593750, 0.0022583007812500, -0.0003051757812500, 0.0020141601562500,
+-0.0003051757812500, 0.0014038085937500, 0.0003356933593750, 0.0010986328125000,
+0.0003662109375000, 0.0014038085937500, -0.0000610351562500, 0.0006103515625000,
+0.0000915527343750, 0.0009765625000000, 0.0001525878906250, 0.0017395019531250,
+-0.0002136230468750, 0.0007934570312500, -0.0002746582031250, 0.0022583007812500,
+0.0001831054687500, 0.0020751953125000, -0.0000610351562500, 0.0021667480468750,
+0.0000610351562500, 0.0030822753906250, 0.0005187988281250, 0.0026550292968750,
+0.0002746582031250, 0.0030822753906250, 0.0004882812500000, 0.0033874511718750,
+0.0004882812500000, 0.0031433105468750, 0.0004882812500000, 0.0032653808593750,
+0.0005493164062500, 0.0036315917968750, 0.0004882812500000, 0.0022888183593750,
+0.0007019042968750, 0.0029602050781250, 0.0007324218750000, 0.0023498535156250,
+0.0007019042968750, 0.0012817382812500, 0.0008239746093750, 0.0025024414062500,
+0.0008850097656250, 0.0003662109375000, 0.0010070800781250, 0.0008544921875000,
+0.0009460449218750, 0.0006408691406250, 0.0012512207031250, -0.0008850097656250,
+0.0015563964843750, 0.0000305175781250, 0.0014343261718750, -0.0012817382812500,
+0.0019836425781250, -0.0008850097656250, 0.0019226074218750, -0.0009155273437500,
+0.0020751953125000, -0.0016174316406250, 0.0023193359375000, -0.0006713867187500,
+0.0020446777343750, -0.0016174316406250, 0.0025024414062500, -0.0013732910156250,
+0.0023803710937500, -0.0011901855468750, 0.0023498535156250, -0.0017089843750000,
+0.0025329589843750, -0.0013732910156250, 0.0022583007812500, -0.0015563964843750,
+0.0022583007812500, -0.0014648437500000, 0.0023193359375000, -0.0013732910156250,
+0.0020141601562500, -0.0013122558593750, 0.0019531250000000, -0.0009765625000000,
+0.0020141601562500, -0.0010986328125000, 0.0015869140625000, -0.0005798339843750,
+0.0016479492187500, -0.0003662109375000, 0.0015869140625000, -0.0005798339843750,
+0.0013427734375000, -0.0000610351562500, 0.0014343261718750, 0.0002746582031250,
+0.0010681152343750, -0.0004272460937500, 0.0012207031250000, 0.0001831054687500,
+0.0009460449218750, 0.0007934570312500, 0.0005798339843750, -0.0000610351562500,
+0.0008850097656250, 0.0017395019531250, 0.0000305175781250, 0.0016479492187500,
+0.0001525878906250, 0.0016784667968750, 0.0000305175781250, 0.0028076171875000,
+-0.0005187988281250, 0.0026245117187500, -0.0002136230468750, 0.0025329589843750,
+-0.0007934570312500, 0.0030212402343750, -0.0005493164062500, 0.0028381347656250,
+-0.0006713867187500, 0.0025024414062500, -0.0012512207031250, 0.0030517578125000,
+-0.0007324218750000, 0.0020141601562500, -0.0008850097656250, 0.0023498535156250,
+-0.0010986328125000, 0.0021057128906250, -0.0007629394531250, 0.0013427734375000,
+-0.0003967285156250, 0.0018920898437500, -0.0005187988281250, 0.0013122558593750,
+-0.0006713867187500, 0.0009765625000000, -0.0004882812500000, 0.0014953613281250,
+-0.0004882812500000, 0.0014038085937500, -0.0007324218750000, 0.0007324218750000,
+-0.0007324218750000, 0.0022583007812500, -0.0006408691406250, 0.0014343261718750,
+-0.0005798339843750, 0.0017395019531250, -0.0006408691406250, 0.0032043457031250,
+-0.0007324218750000, 0.0020751953125000, -0.0006408691406250, 0.0032958984375000,
+-0.0005187988281250, 0.0037841796875000, -0.0005493164062500, 0.0032958984375000,
+-0.0006103515625000, 0.0036621093750000, -0.0004882812500000, 0.0043640136718750,
+-0.0004272460937500, 0.0026855468750000, -0.0004577636718750, 0.0036315917968750,
+-0.0005187988281250, 0.0032043457031250, -0.0001831054687500, 0.0015869140625000,
+0.0000610351562500, 0.0026855468750000, -0.0000915527343750, 0.0012207031250000,
+0.0008239746093750, 0.0008239746093750, 0.0008544921875000, 0.0008239746093750,
+0.0010070800781250, 0.0002136230468750, 0.0016174316406250, 0.0000305175781250,
+0.0015563964843750, 0.0002441406250000, 0.0013732910156250, -0.0000305175781250,
+0.0018310546875000, 0.0000915527343750, 0.0011596679687500, 0.0003662109375000,
+0.0003967285156250, -0.0001220703125000, 0.0010681152343750, 0.0001831054687500,
+-0.0009460449218750, -0.0003051757812500, -0.0007934570312500, -0.0002441406250000,
+-0.0009765625000000, 0.0000610351562500, -0.0021667480468750, -0.0003356933593750,
+-0.0016479492187500, 0.0009155273437500, -0.0021667480468750, 0.0001220703125000,
+-0.0020446777343750, 0.0007934570312500, -0.0018920898437500, 0.0020446777343750,
+-0.0018615722656250, 0.0002746582031250, -0.0013732910156250, 0.0015563964843750,
+-0.0022583007812500, 0.0021057128906250, -0.0021667480468750, 0.0010681152343750,
+-0.0018615722656250, 0.0008239746093750, -0.0022583007812500, 0.0012512207031250,
+-0.0022583007812500, -0.0005798339843750, -0.0022888183593750, -0.0010986328125000,
+-0.0025939941406250, -0.0008544921875000, -0.0024719238281250, -0.0016479492187500,
+-0.0022277832031250, -0.0021667480468750, -0.0025329589843750, -0.0008544921875000,
+-0.0022888183593750, -0.0015258789062500, -0.0025634765625000, -0.0013427734375000,
+-0.0022583007812500, -0.0003356933593750, -0.0009460449218750, -0.0014953613281250,
+-0.0003967285156250, -0.0012512207031250, -0.0019226074218750, -0.0015563964843750,
+-0.0007324218750000, -0.0014038085937500, -0.0008544921875000, -0.0013732910156250,
+-0.0021362304687500, -0.0024414062500000, -0.0008239746093750, -0.0009765625000000,
+-0.0020751953125000, -0.0017089843750000, -0.0021667480468750, -0.0022888183593750,
+-0.0020751953125000, -0.0016174316406250, -0.0028381347656250, -0.0023803710937500,
+-0.0028991699218750, -0.0031433105468750, -0.0030517578125000, -0.0033264160156250,
+-0.0032958984375000, -0.0034179687500000, -0.0031433105468750, -0.0036926269531250,
+-0.0028686523437500, -0.0039062500000000, -0.0027770996093750, -0.0031433105468750,
+-0.0021362304687500, -0.0029907226562500, -0.0012207031250000, -0.0031738281250000,
+-0.0011596679687500, -0.0026855468750000, -0.0007629394531250, -0.0019836425781250,
+0.0005187988281250, -0.0026550292968750, -0.0004882812500000, -0.0019836425781250,
+0.0002441406250000, -0.0015869140625000, 0.0002441406250000, -0.0019531250000000,
+-0.0007629394531250, -0.0011596679687500, 0.0000610351562500, -0.0004882812500000,
+-0.0001831054687500, -0.0004272460937500, -0.0006408691406250, 0.0000305175781250,
+0.0000915527343750, 0.0004577636718750, 0.0008544921875000, 0.0003051757812500,
+0.0004882812500000, 0.0013122558593750, 0.0014343261718750, 0.0010681152343750,
+0.0021362304687500, 0.0012207031250000, 0.0015563964843750, 0.0020751953125000,
+0.0015563964843750, 0.0017089843750000, 0.0026550292968750, 0.0014343261718750,
+0.0008544921875000, 0.0018920898437500, 0.0015563964843750, 0.0012512207031250,
+0.0017089843750000, 0.0004272460937500, 0.0005493164062500, 0.0008850097656250,
+0.0013732910156250, -0.0003356933593750, 0.0011291503906250, -0.0003967285156250,
+0.0009155273437500, -0.0001831054687500, 0.0010681152343750, -0.0004882812500000,
+0.0010681152343750, -0.0006103515625000, 0.0009460449218750, -0.0007629394531250,
+0.0009155273437500, -0.0005493164062500, 0.0008544921875000, -0.0007629394531250,
+0.0004882812500000, -0.0011901855468750, 0.0002746582031250, -0.0010070800781250,
+0.0002746582031250, -0.0015258789062500, -0.0004882812500000, -0.0015258789062500,
+-0.0007019042968750, -0.0014648437500000, -0.0007629394531250, -0.0020141601562500,
+-0.0010681152343750, -0.0019836425781250, -0.0009765625000000, -0.0011596679687500,
+-0.0007324218750000, -0.0016784667968750, -0.0007019042968750, -0.0012207031250000,
+-0.0002136230468750, -0.0002441406250000, 0.0003051757812500, -0.0009155273437500,
+0.0002136230468750, 0.0001831054687500, 0.0008850097656250, 0.0000000000000000,
+0.0008850097656250, 0.0001831054687500, 0.0010070800781250, 0.0008850097656250,
+0.0012817382812500, 0.0003662109375000, 0.0007934570312500, 0.0009155273437500,
+0.0013732910156250, 0.0009460449218750, 0.0007324218750000, 0.0007934570312500,
+0.0006408691406250, 0.0010070800781250, 0.0007934570312500, 0.0009765625000000,
+-0.0002441406250000, 0.0007934570312500, 0.0003051757812500, 0.0007629394531250,
+-0.0005187988281250, 0.0007629394531250, -0.0008850097656250, 0.0007934570312500,
+-0.0003967285156250, 0.0006713867187500, -0.0010681152343750, 0.0009765625000000,
+-0.0008544921875000, 0.0010070800781250, -0.0007629394531250, 0.0008239746093750,
+-0.0012512207031250, 0.0008850097656250, -0.0014038085937500, 0.0009155273437500,
+-0.0008544921875000, 0.0006103515625000, -0.0014648437500000, 0.0005493164062500,
+-0.0015563964843750, 0.0004577636718750, -0.0012512207031250, 0.0002746582031250,
+-0.0013427734375000, 0.0002136230468750, -0.0015258789062500, 0.0003356933593750,
+-0.0010375976562500, 0.0001525878906250, -0.0012512207031250, 0.0003051757812500,
+-0.0015258789062500, 0.0005187988281250, -0.0014038085937500, 0.0001831054687500,
+-0.0014648437500000, 0.0009765625000000, -0.0018920898437500, 0.0006713867187500,
+-0.0015258789062500, 0.0009155273437500, -0.0018615722656250, 0.0015869140625000,
+-0.0023193359375000, 0.0010986328125000, -0.0017089843750000, 0.0017089843750000,
+-0.0025024414062500, 0.0017395019531250, -0.0020751953125000, 0.0015258789062500,
+-0.0020751953125000, 0.0016784667968750, -0.0025024414062500, 0.0016174316406250,
+-0.0015563964843750, 0.0011596679687500, -0.0019226074218750, 0.0011901855468750,
+-0.0017395019531250, 0.0010681152343750, -0.0010681152343750, 0.0007019042968750,
+-0.0009155273437500, 0.0007324218750000, -0.0009155273437500, 0.0006713867187500,
+0.0004272460937500},
+{-0.0004577636718750, -0.0014648437500000, 0.0015563964843750, -0.0012817382812500,
+0.0017395019531250, -0.0003967285156250, 0.0015258789062500, -0.0000305175781250,
+0.0024108886718750, 0.0000610351562500, 0.0026245117187500, 0.0008850097656250,
+0.0016174316406250, 0.0011291503906250, 0.0021667480468750, 0.0011291503906250,
+0.0021972656250000, 0.0015869140625000, 0.0015258789062500, 0.0017089843750000,
+0.0022888183593750, 0.0009765625000000, 0.0020446777343750, 0.0015563964843750,
+0.0019531250000000, 0.0010375976562500, 0.0020141601562500, 0.0001525878906250,
+0.0018920898437500, 0.0011596679687500, 0.0016479492187500, -0.0000610351562500,
+0.0009155273437500, 0.0001525878906250, 0.0010681152343750, 0.0000610351562500,
+0.0006103515625000, -0.0006713867187500, -0.0001831054687500, 0.0000915527343750,
+0.0001220703125000, -0.0005187988281250, -0.0004577636718750, -0.0003967285156250,
+-0.0006713867187500, -0.0004577636718750, -0.0004882812500000, -0.0007324218750000,
+-0.0004882812500000, -0.0002441406250000, -0.0003662109375000, -0.0007324218750000,
+0.0002441406250000, -0.0006103515625000, 0.0006103515625000, -0.0005798339843750,
+0.0007934570312500, -0.0008239746093750, 0.0012207031250000, -0.0003662109375000,
+0.0016174316406250, -0.0009765625000000, 0.0011291503906250, -0.0009155273437500,
+0.0014038085937500, -0.0006103515625000, 0.0012512207031250, -0.0008239746093750,
+0.0005493164062500, -0.0008239746093750, 0.0006103515625000, -0.0003662109375000,
+0.0002746582031250, -0.0004272460937500, -0.0003051757812500, -0.0003051757812500,
+-0.0003051757812500, -0.0000305175781250, -0.0004577636718750, -0.0002136230468750,
+-0.0011596679687500, 0.0000915527343750, -0.0003356933593750, 0.0000000000000000,
+-0.0011291503906250, 0.0001525878906250, -0.0010681152343750, 0.0003967285156250,
+-0.0004272460937500, 0.0003662109375000, -0.0014648437500000, 0.0005798339843750,
+-0.0001525878906250, 0.0007019042968750, -0.0007934570312500, 0.0006103515625000,
+-0.0007629394531250, 0.0005798339843750, 0.0001220703125000, 0.0007324218750000,
+-0.0005798339843750, 0.0005187988281250, 0.0003662109375000, 0.0004882812500000,
+0.0002136230468750, 0.0005187988281250, 0.0002136230468750, 0.0003356933593750,
+0.0004882812500000, 0.0001220703125000, -0.0000305175781250, 0.0004577636718750,
+0.0006408691406250, 0.0002441406250000, 0.0001220703125000, 0.0000915527343750,
+0.0000610351562500, 0.0002746582031250, 0.0006408691406250, 0.0000915527343750,
+-0.0001220703125000, 0.0000305175781250, -0.0002441406250000, 0.0000305175781250,
+-0.0000305175781250, -0.0000305175781250, -0.0006713867187500, -0.0000915527343750,
+-0.0010986328125000, -0.0000610351562500, -0.0005493164062500, 0.0000915527343750,
+-0.0013732910156250, -0.0000610351562500, -0.0012512207031250, 0.0002136230468750,
+-0.0011291503906250, 0.0005187988281250, -0.0014648437500000, 0.0002746582031250,
+-0.0012817382812500, 0.0007629394531250, -0.0011901855468750, 0.0008239746093750,
+-0.0013122558593750, 0.0007324218750000, -0.0011291503906250, 0.0009460449218750,
+-0.0009765625000000, 0.0010070800781250, -0.0011901855468750, 0.0007324218750000,
+-0.0007629394531250, 0.0008850097656250, -0.0009460449218750, 0.0007019042968750,
+-0.0007324218750000, 0.0003967285156250, -0.0003662109375000, 0.0006713867187500,
+-0.0008239746093750, 0.0000000000000000, -0.0003356933593750, 0.0001220703125000,
+-0.0005798339843750, -0.0000305175781250, -0.0006103515625000, -0.0004577636718750,
+-0.0005493164062500, -0.0000305175781250, -0.0010375976562500, -0.0008544921875000,
+-0.0003662109375000, -0.0005187988281250, -0.0012207031250000, -0.0007629394531250,
+-0.0009155273437500, -0.0013427734375000, -0.0003662109375000, -0.0006713867187500,
+-0.0018920898437500, -0.0013427734375000, 0.0001831054687500, -0.0010681152343750,
+-0.0013122558593750, -0.0010986328125000, -0.0007629394531250, -0.0014038085937500,
+0.0009155273437500, -0.0007934570312500, -0.0014038085937500, -0.0019226074218750,
+0.0020751953125000, -0.0015563964843750, 0.0007019042968750, -0.0016479492187500,
+0.0009460449218750, -0.0025329589843750, 0.0035705566406250, -0.0020751953125000,
+0.0014648437500000, -0.0025024414062500, 0.0033874511718750, -0.0024108886718750,
+0.0036621093750000, -0.0025329589843750, 0.0029296875000000, -0.0027160644531250,
+0.0035705566406250, -0.0021362304687500, 0.0037841796875000, -0.0028991699218750,
+0.0024719238281250, -0.0026550292968750, 0.0028991699218750, -0.0024108886718750,
+0.0024414062500000, -0.0027160644531250, 0.0012817382812500, -0.0025024414062500,
+0.0020751953125000, -0.0022277832031250, 0.0011901855468750, -0.0021667480468750,
+0.0011596679687500, -0.0020751953125000, 0.0014953613281250, -0.0018310546875000,
+0.0012817382812500, -0.0017395019531250, 0.0014648437500000, -0.0020446777343750,
+0.0022888183593750, -0.0015869140625000, 0.0021057128906250, -0.0015869140625000,
+0.0024719238281250, -0.0019226074218750, 0.0032653808593750, -0.0013427734375000,
+0.0028991699218750, -0.0015563964843750, 0.0028991699218750, -0.0010375976562500,
+0.0028991699218750, -0.0010070800781250, 0.0023803710937500, -0.0011901855468750,
+0.0018005371093750, -0.0003051757812500, 0.0015869140625000, -0.0010375976562500,
+0.0010375976562500, -0.0008850097656250, 0.0009765625000000, -0.0007324218750000,
+0.0006713867187500, -0.0011901855468750, 0.0001831054687500, -0.0008239746093750,
+0.0003356933593750, -0.0006103515625000, 0.0004272460937500, -0.0010375976562500,
+0.0000610351562500, -0.0003356933593750, -0.0002746582031250, 0.0004272460937500,
+-0.0001525878906250, -0.0003967285156250, -0.0000915527343750, 0.0006713867187500,
+-0.0004577636718750, 0.0003662109375000, -0.0006408691406250, 0.0003967285156250,
+-0.0009460449218750, 0.0010375976562500, -0.0013122558593750, 0.0003662109375000,
+-0.0012512207031250, 0.0010681152343750, -0.0014343261718750, 0.0009765625000000,
+-0.0021667480468750, 0.0013427734375000, -0.0013122558593750, 0.0020446777343750,
+-0.0008239746093750, 0.0016784667968750, -0.0020446777343750, 0.0029907226562500,
+0.0011596679687500, 0.0030822753906250, 0.0004577636718750, 0.0027465820312500,
+0.0003356933593750, 0.0031433105468750, 0.0022583007812500, 0.0030517578125000,
+0.0012207031250000, 0.0018310546875000, 0.0010375976562500, 0.0020141601562500,
+0.0010375976562500, 0.0017089843750000, 0.0010070800781250, 0.0002441406250000,
+0.0011291503906250, -0.0007629394531250, 0.0020141601562500, -0.0019531250000000,
+0.0019836425781250, -0.0018310546875000, 0.0013427734375000, -0.0015563964843750,
+0.0020141601562500, -0.0016479492187500, 0.0021972656250000, -0.0015258789062500,
+0.0010986328125000, -0.0006408691406250, 0.0036315917968750, -0.0006713867187500,
+0.0039367675781250, -0.0005493164062500, 0.0032348632812500, 0.0000000000000000,
+0.0042419433593750, -0.0001220703125000, 0.0046691894531250, -0.0007629394531250,
+0.0027770996093750, -0.0005493164062500, 0.0030517578125000, -0.0008239746093750,
+0.0028991699218750, -0.0017700195312500, 0.0015869140625000, -0.0016784667968750,
+0.0020751953125000, -0.0006103515625000, 0.0023498535156250, -0.0017089843750000,
+0.0019836425781250, -0.0006103515625000, 0.0019836425781250, 0.0010681152343750,
+0.0024108886718750, -0.0007324218750000, 0.0022277832031250, 0.0010375976562500,
+0.0010375976562500, 0.0010986328125000, 0.0010681152343750, -0.0001525878906250,
+0.0008544921875000, -0.0002746582031250, 0.0000610351562500, -0.0003356933593750,
+0.0002441406250000, -0.0017395019531250, 0.0005187988281250, -0.0018615722656250,
+0.0003662109375000, -0.0017395019531250, 0.0005493164062500, -0.0024719238281250,
+0.0010681152343750, -0.0022888183593750, 0.0009460449218750, -0.0007629394531250,
+0.0001220703125000, -0.0013122558593750, 0.0007324218750000, -0.0004882812500000,
+0.0000915527343750, 0.0014648437500000, -0.0010375976562500, 0.0007934570312500,
+-0.0003967285156250, 0.0007934570312500, -0.0013122558593750, 0.0020141601562500,
+-0.0011596679687500, 0.0011291503906250, -0.0010681152343750, 0.0001220703125000,
+-0.0014953613281250, 0.0015258789062500, -0.0011291503906250, -0.0003662109375000,
+-0.0016174316406250, 0.0003356933593750, -0.0014648437500000, 0.0004272460937500,
+-0.0016174316406250, -0.0006408691406250, -0.0020751953125000, 0.0007324218750000,
+-0.0017395019531250, 0.0001831054687500, -0.0021057128906250, 0.0007934570312500,
+-0.0021057128906250, 0.0012512207031250, -0.0021057128906250, 0.0014953613281250,
+-0.0021667480468750, 0.0026245117187500, -0.0020446777343750, 0.0016174316406250,
+-0.0025024414062500, 0.0028991699218750, -0.0023803710937500, 0.0023193359375000,
+-0.0023498535156250, 0.0010375976562500, -0.0027160644531250, 0.0027160644531250,
+-0.0026245117187500, -0.0002746582031250, -0.0021972656250000, 0.0007019042968750,
+-0.0024414062500000, 0.0000305175781250, -0.0022277832031250, -0.0020446777343750,
+-0.0017089843750000, 0.0000305175781250, -0.0019226074218750, -0.0031738281250000,
+-0.0015869140625000, -0.0023193359375000, -0.0015258789062500, -0.0022277832031250,
+-0.0014343261718750, -0.0041503906250000, -0.0011596679687500, -0.0025024414062500,
+-0.0013122558593750, -0.0036010742187500, -0.0009765625000000, -0.0033264160156250,
+-0.0007934570312500, -0.0029907226562500, -0.0009460449218750, -0.0034484863281250,
+-0.0009765625000000, -0.0028991699218750, -0.0008239746093750, -0.0028991699218750,
+-0.0010681152343750, -0.0027770996093750, -0.0013122558593750, -0.0027770996093750,
+-0.0010681152343750, -0.0028381347656250, -0.0010070800781250, -0.0026550292968750,
+-0.0012817382812500, -0.0029907226562500, -0.0005187988281250, -0.0027770996093750,
+-0.0007629394531250, -0.0024719238281250, -0.0005798339843750, -0.0025939941406250,
+-0.0000610351562500, -0.0024108886718750, -0.0005493164062500, -0.0018615722656250,
+0.0003356933593750, -0.0020446777343750, 0.0000610351562500, -0.0017395019531250,
+0.0002136230468750, -0.0014038085937500, 0.0008850097656250, -0.0018310546875000,
+0.0003051757812500, -0.0013122558593750, 0.0009155273437500, -0.0013732910156250,
+0.0007629394531250, -0.0015869140625000, 0.0006713867187500, -0.0012512207031250,
+0.0009155273437500, -0.0010681152343750, 0.0006408691406250, -0.0014038085937500,
+0.0008544921875000, -0.0012817382812500, 0.0007324218750000, -0.0012817382812500,
+0.0007019042968750, -0.0015258789062500, 0.0008850097656250, -0.0012512207031250,
+0.0007629394531250, -0.0013732910156250, 0.0009460449218750, -0.0010681152343750,
+0.0008544921875000, -0.0010681152343750, 0.0009460449218750, -0.0010986328125000,
+0.0010681152343750, -0.0006103515625000, 0.0008544921875000, -0.0011291503906250,
+0.0008239746093750, -0.0008544921875000, 0.0009460449218750, -0.0008239746093750,
+0.0004882812500000, -0.0010986328125000, 0.0001220703125000, -0.0007019042968750,
+0.0003967285156250, -0.0008850097656250, -0.0005187988281250, -0.0004882812500000,
+-0.0004577636718750, -0.0003662109375000, -0.0004577636718750, -0.0003967285156250,
+-0.0009460449218750, 0.0001831054687500, -0.0007019042968750, -0.0003356933593750,
+-0.0010070800781250, -0.0006103515625000, -0.0009765625000000, -0.0000305175781250,
+-0.0008544921875000, -0.0002746582031250, -0.0010681152343750, -0.0011596679687500,
+-0.0010681152343750, 0.0001831054687500, -0.0005493164062500, -0.0004272460937500,
+-0.0006713867187500, -0.0006408691406250, -0.0007019042968750, 0.0003051757812500,
+-0.0003051757812500, -0.0003356933593750, -0.0003662109375000, -0.0004577636718750,
+-0.0007629394531250, -0.0002441406250000, -0.0006408691406250, -0.0003662109375000,
+-0.0006103515625000},
+{-0.0011291503906250, 0.0004882812500000, -0.0010681152343750, 0.0006103515625000,
+-0.0012207031250000, 0.0016479492187500, -0.0010375976562500, -0.0004882812500000,
+-0.0015563964843750, 0.0009155273437500, -0.0015258789062500, 0.0000305175781250,
+-0.0015869140625000, -0.0022277832031250, -0.0018615722656250, -0.0002746582031250,
+-0.0018615722656250, -0.0029296875000000, -0.0018615722656250, -0.0029296875000000,
+-0.0017700195312500, -0.0027770996093750, -0.0020446777343750, -0.0040283203125000,
+-0.0023498535156250, -0.0033264160156250, -0.0020446777343750, -0.0040283203125000,
+-0.0027770996093750, -0.0036926269531250, -0.0029296875000000, -0.0036315917968750,
+-0.0026855468750000, -0.0039062500000000, -0.0028076171875000, -0.0031433105468750,
+-0.0029296875000000, -0.0036315917968750, -0.0021972656250000, -0.0031127929687500,
+-0.0021972656250000, -0.0032348632812500, -0.0021972656250000, -0.0036621093750000,
+-0.0018005371093750, -0.0028686523437500, -0.0018005371093750, -0.0042114257812500,
+-0.0018615722656250, -0.0035400390625000, -0.0017700195312500, -0.0039367675781250,
+-0.0016479492187500, -0.0050354003906250, -0.0016784667968750, -0.0038452148437500,
+-0.0015869140625000, -0.0054626464843750, -0.0015563964843750, -0.0049743652343750,
+-0.0014648437500000, -0.0052185058593750, -0.0013732910156250, -0.0063781738281250,
+-0.0014343261718750, -0.0052795410156250, -0.0014038085937500, -0.0063171386718750,
+-0.0010375976562500, -0.0059509277343750, -0.0012207031250000, -0.0054931640625000,
+-0.0009765625000000, -0.0058593750000000, -0.0004577636718750, -0.0054321289062500,
+-0.0007019042968750, -0.0048828125000000, -0.0002441406250000, -0.0050354003906250,
+-0.0002746582031250, -0.0046081542968750, -0.0002136230468750, -0.0040588378906250,
+0.0000000000000000, -0.0043029785156250, -0.0001525878906250, -0.0036010742187500,
+0.0001831054687500, -0.0032653808593750, 0.0001831054687500, -0.0031433105468750,
+0.0001831054687500, -0.0030212402343750, 0.0003662109375000, -0.0030212402343750,
+0.0003967285156250, -0.0028991699218750, 0.0003051757812500, -0.0029296875000000,
+0.0003051757812500, -0.0028381347656250, 0.0004272460937500, -0.0028076171875000,
+0.0003662109375000, -0.0029907226562500, 0.0003356933593750, -0.0025939941406250,
+0.0005493164062500, -0.0026855468750000, 0.0006713867187500, -0.0024719238281250,
+0.0004577636718750, -0.0019836425781250, 0.0003967285156250, -0.0021362304687500,
+0.0005187988281250, -0.0018005371093750, -0.0000915527343750, -0.0014343261718750,
+-0.0000610351562500, -0.0015563964843750, -0.0001525878906250, -0.0015563964843750,
+-0.0006408691406250, -0.0013427734375000, -0.0005493164062500, -0.0019836425781250,
+-0.0006103515625000, -0.0018615722656250, -0.0007324218750000, -0.0021667480468750,
+-0.0007629394531250, -0.0024414062500000, -0.0007324218750000, -0.0019226074218750,
+-0.0007629394531250, -0.0031738281250000, -0.0007629394531250, -0.0022277832031250,
+-0.0008239746093750, -0.0020751953125000, -0.0007019042968750, -0.0030822753906250,
+-0.0004577636718750, -0.0021057128906250, -0.0006103515625000, -0.0021362304687500,
+-0.0007324218750000, -0.0030822753906250, -0.0004577636718750, -0.0020446777343750,
+-0.0006713867187500, -0.0012207031250000, -0.0010986328125000, -0.0027465820312500,
+-0.0007324218750000, -0.0005187988281250, -0.0007324218750000, -0.0007324218750000,
+-0.0009155273437500, -0.0009765625000000, -0.0007019042968750, 0.0001220703125000,
+-0.0004882812500000, -0.0002441406250000, -0.0004882812500000, -0.0000305175781250,
+-0.0002441406250000, -0.0000610351562500, -0.0001831054687500, -0.0003662109375000,
+-0.0002441406250000, -0.0001831054687500, -0.0000610351562500, -0.0000610351562500,
+-0.0000305175781250, -0.0007019042968750, -0.0003662109375000, -0.0002746582031250,
+-0.0003356933593750, -0.0002746582031250, -0.0001525878906250, -0.0008850097656250,
+-0.0003967285156250, -0.0002746582031250, -0.0004577636718750, -0.0001220703125000,
+0.0001831054687500, -0.0004577636718750, -0.0000915527343750, 0.0002746582031250,
+0.0002136230468750, 0.0009460449218750, 0.0007934570312500, 0.0002746582031250,
+0.0003051757812500, 0.0018615722656250, 0.0010986328125000, 0.0017395019531250,
+0.0009155273437500, 0.0018005371093750, 0.0009765625000000, 0.0028076171875000,
+0.0013122558593750, 0.0026245117187500, 0.0007324218750000, 0.0021362304687500,
+0.0009765625000000, 0.0029907226562500, 0.0009460449218750, 0.0022277832031250,
+0.0005187988281250, 0.0014648437500000, 0.0003051757812500, 0.0030517578125000,
+0.0003051757812500, 0.0011901855468750, -0.0000305175781250, 0.0017089843750000,
+-0.0001831054687500, 0.0024414062500000, -0.0000915527343750, 0.0016479492187500,
+-0.0001220703125000, 0.0020446777343750, -0.0001831054687500, 0.0023498535156250,
+0.0002746582031250, 0.0021972656250000, 0.0001220703125000, 0.0020751953125000,
+-0.0002746582031250, 0.0020751953125000, -0.0004882812500000, 0.0018005371093750,
+-0.0007629394531250, 0.0012512207031250, -0.0010070800781250, 0.0006713867187500,
+-0.0018615722656250, 0.0006408691406250, -0.0016174316406250, 0.0007629394531250,
+-0.0012512207031250, 0.0008850097656250, -0.0020141601562500, 0.0014343261718750,
+-0.0003967285156250, 0.0007934570312500, -0.0007934570312500, 0.0013732910156250,
+-0.0008850097656250, 0.0027770996093750, 0.0001220703125000, 0.0021972656250000,
+-0.0003356933593750, 0.0022888183593750, 0.0001220703125000, 0.0033264160156250,
+-0.0001220703125000, 0.0018920898437500, -0.0003967285156250, 0.0007324218750000,
+0.0000915527343750, 0.0025939941406250, 0.0001525878906250, -0.0002136230468750,
+-0.0004882812500000, 0.0006103515625000, -0.0002441406250000, 0.0010070800781250,
+-0.0001831054687500, -0.0005798339843750, -0.0009155273437500, 0.0004577636718750,
+-0.0011291503906250, 0.0003967285156250, -0.0002441406250000, 0.0008850097656250,
+-0.0007629394531250, 0.0003662109375000, -0.0009155273437500, -0.0001220703125000,
+-0.0002136230468750, 0.0008850097656250, -0.0006408691406250, -0.0005187988281250,
+-0.0008239746093750, -0.0000610351562500, -0.0000915527343750, -0.0001525878906250,
+-0.0005187988281250, -0.0009460449218750, -0.0009460449218750, 0.0001220703125000,
+0.0002441406250000, -0.0012207031250000, -0.0006103515625000, -0.0010375976562500,
+-0.0005798339843750, -0.0007324218750000, 0.0000915527343750, -0.0014648437500000,
+-0.0000305175781250, -0.0013427734375000, -0.0004272460937500, -0.0011596679687500,
+0.0005493164062500, -0.0015869140625000, 0.0001525878906250, -0.0012512207031250,
+0.0002136230468750, -0.0010681152343750, 0.0010070800781250, -0.0019226074218750,
+0.0008850097656250, -0.0005493164062500, 0.0010681152343750, -0.0012817382812500,
+0.0016479492187500, -0.0011291503906250, 0.0015563964843750, -0.0000915527343750,
+0.0014648437500000, -0.0010375976562500, 0.0021362304687500, 0.0000305175781250,
+0.0012512207031250, 0.0001220703125000, 0.0021972656250000, -0.0004272460937500,
+0.0017700195312500, 0.0000305175781250, 0.0009460449218750, 0.0007019042968750,
+0.0024719238281250, -0.0007934570312500, 0.0000610351562500, 0.0000305175781250,
+0.0007629394531250, 0.0000915527343750, 0.0006103515625000, -0.0008850097656250,
+-0.0009155273437500, 0.0003967285156250, 0.0004272460937500, 0.0002441406250000,
+-0.0006103515625000, 0.0001831054687500, -0.0003356933593750, 0.0008850097656250,
+0.0002136230468750, 0.0013732910156250, 0.0000915527343750, 0.0012207031250000,
+0.0005798339843750, 0.0010375976562500, 0.0005493164062500, 0.0018310546875000,
+0.0012207031250000, 0.0009765625000000, 0.0007629394531250, -0.0001525878906250,
+0.0003051757812500, 0.0008850097656250, 0.0012512207031250, -0.0004882812500000,
+-0.0006713867187500, -0.0007019042968750, -0.0001220703125000, -0.0002746582031250,
+-0.0003051757812500, -0.0007019042968750, -0.0016479492187500, -0.0008850097656250,
+-0.0007019042968750, 0.0002746582031250, -0.0014648437500000, -0.0001220703125000,
+-0.0015869140625000, -0.0002136230468750, -0.0013122558593750, 0.0004577636718750,
+-0.0014343261718750, 0.0001831054687500, -0.0013427734375000, 0.0000305175781250,
+-0.0013732910156250, -0.0000305175781250, -0.0012207031250000, -0.0000305175781250,
+-0.0015258789062500, -0.0001220703125000, -0.0017089843750000, -0.0002136230468750,
+-0.0012512207031250, -0.0000610351562500, -0.0022277832031250, -0.0000915527343750,
+-0.0019226074218750, -0.0002136230468750, -0.0019226074218750, -0.0001831054687500,
+-0.0023803710937500, -0.0000610351562500, -0.0018310546875000, -0.0004272460937500,
+-0.0031127929687500, -0.0002441406250000, -0.0023193359375000, -0.0003356933593750,
+-0.0024414062500000, -0.0007324218750000, -0.0034179687500000, -0.0005187988281250,
+-0.0021057128906250, -0.0006713867187500, -0.0027160644531250, -0.0007019042968750,
+-0.0027160644531250, -0.0007934570312500, -0.0022888183593750, -0.0008850097656250,
+-0.0026245117187500, -0.0006103515625000, -0.0028686523437500, -0.0009765625000000,
+-0.0022888183593750, -0.0007629394531250, -0.0028381347656250, -0.0006408691406250,
+-0.0026245117187500, -0.0007629394531250, -0.0020446777343750, -0.0004882812500000,
+-0.0029602050781250, -0.0005187988281250, -0.0023193359375000, -0.0003662109375000,
+-0.0022583007812500, -0.0003051757812500, -0.0025939941406250, -0.0003662109375000,
+-0.0026550292968750, -0.0001831054687500, -0.0025634765625000, -0.0002136230468750,
+-0.0024414062500000, -0.0002136230468750, -0.0024719238281250, -0.0001220703125000,
+-0.0021362304687500, 0.0000915527343750, -0.0019531250000000, 0.0000000000000000,
+-0.0023193359375000, -0.0001525878906250, -0.0017089843750000, 0.0001831054687500,
+-0.0022277832031250, -0.0000610351562500, -0.0021667480468750, -0.0003662109375000,
+-0.0018005371093750, 0.0001525878906250, -0.0024414062500000, -0.0005798339843750,
+-0.0017395019531250, -0.0003662109375000, -0.0023498535156250, -0.0002746582031250,
+-0.0021972656250000, -0.0006103515625000, -0.0017700195312500, -0.0002441406250000,
+-0.0030212402343750, -0.0003662109375000, -0.0017395019531250, -0.0002136230468750,
+-0.0027160644531250, -0.0001831054687500, -0.0032043457031250, -0.0002746582031250,
+-0.0025024414062500, -0.0000305175781250, -0.0036621093750000, -0.0003967285156250,
+-0.0036621093750000, -0.0002746582031250, -0.0037536621093750, -0.0003356933593750,
+-0.0044555664062500, -0.0005187988281250, -0.0046997070312500, -0.0003051757812500,
+-0.0045166015625000, -0.0007934570312500, -0.0053405761718750, -0.0005187988281250,
+-0.0049438476562500, -0.0009155273437500, -0.0049133300781250, -0.0014343261718750,
+-0.0053710937500000, -0.0007629394531250, -0.0046997070312500, -0.0018920898437500,
+-0.0047912597656250, -0.0015563964843750, -0.0048217773437500, -0.0016174316406250,
+-0.0041198730468750, -0.0021362304687500, -0.0039062500000000, -0.0014343261718750,
+-0.0041809082031250, -0.0023193359375000, -0.0025329589843750, -0.0017395019531250,
+-0.0032348632812500, -0.0017395019531250, -0.0026550292968750, -0.0023803710937500,
+-0.0014343261718750, -0.0014038085937500, -0.0029296875000000, -0.0020446777343750,
+-0.0010375976562500, -0.0017395019531250, -0.0020141601562500, -0.0013732910156250,
+-0.0020751953125000, -0.0014038085937500, -0.0011596679687500, -0.0009765625000000,
+-0.0028991699218750, -0.0012207031250000, -0.0013427734375000, -0.0007629394531250,
+-0.0021667480468750, -0.0006408691406250, -0.0019226074218750, -0.0007934570312500,
+-0.0007629394531250, -0.0000610351562500, -0.0021057128906250, -0.0004272460937500,
+-0.0001525878906250, 0.0003051757812500, -0.0007629394531250, 0.0002136230468750,
+-0.0001525878906250, -0.0002441406250000, 0.0015869140625000, 0.0006713867187500,
+0.0005493164062500},
+{-0.0027465820312500, 0.0001831054687500, -0.0023803710937500, -0.0000915527343750,
+-0.0023498535156250, -0.0001220703125000, -0.0019531250000000, -0.0000915527343750,
+-0.0016479492187500, -0.0001525878906250, -0.0013122558593750, -0.0000610351562500,
+-0.0006713867187500, -0.0001220703125000, -0.0010375976562500, -0.0000915527343750,
+-0.0002746582031250, -0.0000610351562500, -0.0000610351562500, -0.0001831054687500,
+-0.0003662109375000, -0.0003356933593750, 0.0003967285156250, -0.0003051757812500,
+-0.0000610351562500, -0.0003967285156250, -0.0001525878906250, -0.0004577636718750,
+-0.0004882812500000, -0.0002746582031250, -0.0012817382812500, -0.0003662109375000,
+-0.0015258789062500, -0.0004577636718750, -0.0019836425781250, -0.0001220703125000,
+-0.0023803710937500, -0.0003051757812500, -0.0026855468750000, -0.0006103515625000,
+-0.0028686523437500, -0.0001220703125000, -0.0029296875000000, -0.0004882812500000,
+-0.0033569335937500, -0.0002441406250000, -0.0034179687500000, -0.0002136230468750,
+-0.0034790039062500, -0.0003967285156250, -0.0037536621093750, 0.0000305175781250,
+-0.0035705566406250, -0.0003967285156250, -0.0034179687500000, -0.0001831054687500,
+-0.0036926269531250, -0.0002441406250000, -0.0035705566406250, -0.0005798339843750,
+-0.0033264160156250, -0.0001525878906250, -0.0039367675781250, -0.0007934570312500,
+-0.0033874511718750, -0.0003967285156250, -0.0038452148437500, -0.0003662109375000,
+-0.0036621093750000, -0.0007629394531250, -0.0032653808593750, -0.0003051757812500,
+-0.0039672851562500, -0.0006103515625000, -0.0030212402343750, -0.0006103515625000,
+-0.0037231445312500, -0.0005493164062500, -0.0035400390625000, -0.0008239746093750,
+-0.0028381347656250, -0.0007019042968750, -0.0040588378906250, -0.0008239746093750,
+-0.0028381347656250, -0.0009155273437500, -0.0033874511718750, -0.0007629394531250,
+-0.0035095214843750, -0.0007019042968750, -0.0028076171875000, -0.0007324218750000,
+-0.0035400390625000, -0.0004272460937500, -0.0028686523437500, -0.0003051757812500,
+-0.0032043457031250, -0.0003967285156250, -0.0032043457031250, -0.0003051757812500,
+-0.0027465820312500, -0.0002136230468750, -0.0032958984375000, -0.0007324218750000,
+-0.0028076171875000, -0.0007629394531250, -0.0028381347656250, -0.0008850097656250,
+-0.0031127929687500, -0.0012512207031250, -0.0027465820312500, -0.0012207031250000,
+-0.0025939941406250, -0.0012512207031250, -0.0033264160156250, -0.0012817382812500,
+-0.0025634765625000, -0.0012207031250000, -0.0030212402343750, -0.0011596679687500,
+-0.0038757324218750, -0.0009765625000000, -0.0028076171875000, -0.0009765625000000,
+-0.0041503906250000, -0.0008239746093750, -0.0040588378906250, -0.0006713867187500,
+-0.0036315917968750, -0.0005187988281250, -0.0043640136718750, -0.0004577636718750,
+-0.0045166015625000, -0.0004577636718750, -0.0032653808593750, -0.0001220703125000,
+-0.0043029785156250, -0.0002441406250000, -0.0034484863281250, -0.0004272460937500,
+-0.0021972656250000, 0.0002441406250000, -0.0041198730468750, -0.0002441406250000,
+-0.0011901855468750, 0.0001220703125000, -0.0023193359375000, 0.0000305175781250,
+-0.0019836425781250, -0.0001831054687500, -0.0000610351562500, 0.0005187988281250,
+-0.0023498535156250, -0.0005493164062500, -0.0006103515625000, -0.0000915527343750,
+-0.0014038085937500, -0.0001220703125000, -0.0019226074218750, -0.0008850097656250,
+-0.0012512207031250, -0.0000610351562500, -0.0024414062500000, -0.0007629394531250,
+-0.0014038085937500, -0.0008239746093750, -0.0014953613281250, -0.0006103515625000,
+-0.0010681152343750, -0.0007019042968750, -0.0000915527343750, -0.0005798339843750,
+-0.0006408691406250, -0.0007324218750000, 0.0009155273437500, -0.0003356933593750,
+0.0013122558593750, -0.0005187988281250, 0.0010375976562500, -0.0008544921875000,
+0.0018005371093750, -0.0002136230468750, 0.0023803710937500, -0.0003967285156250,
+0.0012207031250000, -0.0003051757812500, 0.0016784667968750, 0.0000305175781250,
+0.0014038085937500, 0.0005187988281250, 0.0005493164062500, 0.0008239746093750,
+0.0014038085937500, 0.0003356933593750, 0.0001220703125000, 0.0012512207031250,
+0.0001220703125000, 0.0010375976562500, 0.0007324218750000, 0.0001525878906250,
+0.0004272460937500, 0.0010986328125000, 0.0003662109375000, 0.0003051757812500,
+0.0010681152343750, 0.0000610351562500, 0.0009765625000000, 0.0007019042968750,
+0.0013427734375000, 0.0007934570312500, 0.0023193359375000, 0.0002136230468750,
+0.0021667480468750, 0.0008850097656250, 0.0019226074218750, 0.0010986328125000,
+0.0028076171875000, 0.0005798339843750, 0.0018615722656250, 0.0006713867187500,
+0.0006713867187500, 0.0011596679687500, 0.0018615722656250, 0.0000000000000000,
+-0.0001525878906250, 0.0003967285156250, -0.0001220703125000, 0.0004577636718750,
+0.0000610351562500, -0.0003051757812500, -0.0006408691406250, 0.0000305175781250,
+-0.0001220703125000, 0.0005187988281250, -0.0005187988281250, 0.0003967285156250,
+-0.0001831054687500, 0.0000915527343750, -0.0003662109375000, 0.0001220703125000,
+-0.0012817382812500, -0.0013122558593750, -0.0009460449218750, 0.0000000000000000,
+-0.0002136230468750, -0.0010681152343750, 0.0000915527343750, -0.0001831054687500,
+-0.0003967285156250, 0.0012207031250000, -0.0008544921875000, 0.0003967285156250,
+-0.0005187988281250, 0.0007324218750000, -0.0005798339843750, 0.0017700195312500,
+-0.0014343261718750, 0.0010070800781250, -0.0000610351562500, 0.0000305175781250,
+0.0012207031250000, 0.0010681152343750, 0.0000610351562500, 0.0001525878906250,
+0.0027770996093750, -0.0002746582031250, 0.0028076171875000, 0.0006408691406250,
+0.0021972656250000, 0.0010986328125000, 0.0032348632812500, 0.0007019042968750,
+0.0032348632812500, 0.0018005371093750, 0.0020141601562500, 0.0027465820312500,
+0.0019531250000000, 0.0017700195312500, 0.0021667480468750, 0.0016784667968750,
+0.0017700195312500, 0.0031738281250000, 0.0014648437500000, -0.0001220703125000,
+0.0016174316406250, 0.0005798339843750, 0.0019226074218750, 0.0008850097656250,
+0.0014038085937500, -0.0009155273437500, 0.0010681152343750, 0.0001525878906250,
+0.0015258789062500, 0.0005187988281250, 0.0002136230468750, 0.0002136230468750,
+0.0000610351562500, 0.0009765625000000, 0.0003356933593750, 0.0018310546875000,
+-0.0003662109375000, 0.0012817382812500, -0.0004577636718750, 0.0014648437500000,
+0.0003662109375000, 0.0019836425781250, 0.0004272460937500, 0.0009155273437500,
+0.0003051757812500, -0.0000915527343750, 0.0007934570312500, 0.0006408691406250,
+0.0008850097656250, -0.0011291503906250, 0.0005187988281250, -0.0018920898437500,
+0.0004882812500000, -0.0013732910156250, 0.0006103515625000, -0.0018615722656250,
+0.0004272460937500, -0.0026855468750000, 0.0002746582031250, -0.0009155273437500,
+0.0003967285156250, -0.0022888183593750, 0.0004577636718750, -0.0017700195312500,
+0.0000000000000000, -0.0003051757812500, -0.0002746582031250, -0.0026550292968750,
+-0.0001220703125000, -0.0005798339843750, -0.0009155273437500, -0.0012207031250000,
+-0.0010070800781250, -0.0022583007812500, -0.0009460449218750, -0.0017395019531250,
+-0.0012512207031250, -0.0026550292968750, -0.0011291503906250, -0.0033569335937500,
+-0.0011596679687500, -0.0035705566406250, -0.0008850097656250, -0.0039978027343750,
+-0.0011596679687500, -0.0045776367187500, -0.0015258789062500, -0.0047302246093750,
+-0.0010375976562500, -0.0043945312500000, -0.0015869140625000, -0.0050964355468750,
+-0.0015563964843750, -0.0044555664062500, -0.0014648437500000, -0.0039672851562500,
+-0.0015563964843750, -0.0050964355468750, -0.0013732910156250, -0.0025634765625000,
+-0.0019836425781250, -0.0036926269531250, -0.0017089843750000, -0.0031127929687500,
+-0.0019531250000000, -0.0010986328125000, -0.0026550292968750, -0.0030822753906250,
+-0.0022583007812500, -0.0010986328125000, -0.0025634765625000, -0.0014953613281250,
+-0.0028381347656250, -0.0018310546875000, -0.0024414062500000, -0.0008850097656250,
+-0.0021667480468750, -0.0016784667968750, -0.0024108886718750, -0.0014038085937500,
+-0.0018920898437500, -0.0013732910156250, -0.0018615722656250, -0.0017700195312500,
+-0.0018310546875000, -0.0017395019531250, -0.0014953613281250, -0.0016479492187500,
+-0.0015869140625000, -0.0021667480468750, -0.0013427734375000, -0.0018615722656250,
+-0.0012512207031250, -0.0022888183593750, -0.0012512207031250, -0.0027770996093750,
+-0.0010986328125000, -0.0021667480468750, -0.0010375976562500, -0.0029602050781250,
+-0.0012512207031250, -0.0027160644531250, -0.0011901855468750, -0.0026245117187500,
+-0.0010986328125000, -0.0031127929687500, -0.0012207031250000, -0.0026550292968750,
+-0.0011291503906250, -0.0028076171875000, -0.0009765625000000, -0.0033569335937500,
+-0.0008544921875000, -0.0030212402343750, -0.0007934570312500, -0.0028381347656250,
+-0.0007324218750000, -0.0037536621093750, -0.0006103515625000, -0.0025634765625000,
+-0.0009155273437500, -0.0031127929687500, -0.0007019042968750, -0.0030822753906250,
+-0.0007934570312500, -0.0021972656250000, -0.0010681152343750, -0.0030822753906250,
+-0.0007934570312500, -0.0023498535156250, -0.0011596679687500, -0.0026550292968750,
+-0.0011291503906250, -0.0027770996093750, -0.0010681152343750, -0.0024414062500000,
+-0.0012207031250000, -0.0031433105468750, -0.0011291503906250, -0.0028076171875000,
+-0.0010681152343750, -0.0032348632812500, -0.0010986328125000, -0.0033874511718750,
+-0.0009765625000000, -0.0031433105468750, -0.0008544921875000, -0.0036315917968750,
+-0.0008544921875000, -0.0032653808593750, -0.0004882812500000, -0.0031738281250000,
+-0.0005798339843750, -0.0032653808593750, -0.0004882812500000, -0.0029602050781250,
+-0.0001525878906250, -0.0028991699218750, -0.0003662109375000, -0.0031433105468750,
+-0.0003967285156250, -0.0029907226562500, -0.0002746582031250, -0.0030822753906250,
+-0.0002136230468750, -0.0032348632812500, -0.0001831054687500, -0.0029296875000000,
+-0.0000915527343750, -0.0032653808593750, 0.0000915527343750, -0.0024414062500000,
+0.0003662109375000, -0.0023193359375000, 0.0002136230468750, -0.0024719238281250,
+0.0001220703125000, -0.0011596679687500, 0.0004882812500000, -0.0016479492187500,
+-0.0000610351562500, -0.0018310546875000, 0.0000915527343750, -0.0009765625000000,
+0.0000000000000000, -0.0009765625000000, -0.0003967285156250, -0.0018310546875000,
+0.0000305175781250, -0.0001525878906250, -0.0006103515625000, -0.0006408691406250,
+-0.0006713867187500, -0.0007934570312500, -0.0005493164062500, -0.0002746582031250,
+-0.0008850097656250, -0.0014953613281250, -0.0008239746093750, -0.0008239746093750,
+-0.0004577636718750, -0.0013732910156250, -0.0007324218750000, -0.0018615722656250,
+-0.0003356933593750, -0.0014343261718750, 0.0001831054687500, -0.0018005371093750,
+-0.0003356933593750, -0.0018310546875000, 0.0005187988281250, -0.0016784667968750,
+0.0006408691406250, -0.0015258789062500, 0.0003967285156250, -0.0015258789062500,
+0.0006103515625000, -0.0013427734375000, 0.0006713867187500, -0.0003967285156250,
+0.0002746582031250, -0.0008544921875000, 0.0006103515625000, -0.0001831054687500,
+0.0003356933593750, 0.0007324218750000, -0.0000915527343750, -0.0003967285156250,
+0.0004577636718750, 0.0014343261718750, 0.0000305175781250, 0.0006408691406250,
+0.0002441406250000, 0.0007629394531250, 0.0002746582031250, 0.0019836425781250,
+0.0000305175781250, 0.0007324218750000, 0.0003356933593750, 0.0018615722656250,
+0.0001525878906250, 0.0017089843750000, 0.0001220703125000, 0.0015869140625000,
+-0.0000305175781250, 0.0021667480468750, -0.0002441406250000, 0.0020446777343750,
+-0.0001220703125000, 0.0032348632812500, -0.0003967285156250, 0.0029296875000000,
+-0.0003967285156250},
+{-0.0014343261718750, -0.0013122558593750, -0.0014648437500000, -0.0011291503906250,
+-0.0013122558593750, -0.0002136230468750, -0.0013427734375000, -0.0012512207031250,
+-0.0014038085937500, 0.0003662109375000, -0.0010375976562500, -0.0003356933593750,
+-0.0009460449218750, -0.0000915527343750, -0.0009765625000000, 0.0009460449218750,
+-0.0007934570312500, -0.0003967285156250, -0.0006103515625000, 0.0015563964843750,
+-0.0009460449218750, 0.0004882812500000, -0.0007019042968750, 0.0005493164062500,
+-0.0008239746093750, 0.0020446777343750, -0.0011901855468750, 0.0004882812500000,
+-0.0008850097656250, 0.0018615722656250, -0.0014648437500000, 0.0013427734375000,
+-0.0013732910156250, 0.0011596679687500, -0.0014343261718750, 0.0019226074218750,
+-0.0018005371093750, 0.0010375976562500, -0.0015869140625000, 0.0017700195312500,
+-0.0017089843750000, 0.0013122558593750, -0.0017089843750000, 0.0012207031250000,
+-0.0016174316406250, 0.0015258789062500, -0.0015563964843750, 0.0007629394531250,
+-0.0014038085937500, 0.0012817382812500, -0.0013732910156250, 0.0006713867187500,
+-0.0012817382812500, 0.0004882812500000, -0.0011291503906250, 0.0007019042968750,
+-0.0010375976562500, -0.0001525878906250, -0.0009460449218750, 0.0003051757812500,
+-0.0006408691406250, -0.0001831054687500, -0.0005493164062500, -0.0003967285156250,
+-0.0003967285156250, 0.0000000000000000, -0.0001525878906250, -0.0006103515625000,
+-0.0000610351562500, -0.0003967285156250, 0.0002746582031250, -0.0002746582031250,
+0.0003051757812500, -0.0006713867187500, 0.0003662109375000, -0.0008544921875000,
+0.0006103515625000, -0.0007324218750000, 0.0005493164062500, -0.0018615722656250,
+0.0005187988281250, -0.0018615722656250, 0.0006408691406250, -0.0023193359375000,
+0.0006713867187500, -0.0027770996093750, 0.0006408691406250, -0.0020141601562500,
+0.0008239746093750, -0.0030822753906250, 0.0005187988281250, -0.0022583007812500,
+0.0007629394531250, -0.0019531250000000, 0.0006713867187500, -0.0029296875000000,
+0.0001831054687500, -0.0022277832031250, 0.0005493164062500, -0.0016174316406250,
+0.0001220703125000, -0.0025024414062500, 0.0000305175781250, -0.0017089843750000,
+0.0002136230468750, -0.0006713867187500, 0.0000305175781250, -0.0020446777343750,
+0.0000000000000000, -0.0001525878906250, 0.0006103515625000, -0.0008850097656250,
+0.0003967285156250, -0.0011291503906250, 0.0004272460937500, -0.0001220703125000,
+0.0009155273437500, -0.0013122558593750, 0.0008544921875000, -0.0009155273437500,
+0.0003967285156250, -0.0008544921875000, 0.0007324218750000, -0.0015869140625000,
+0.0004577636718750, -0.0017089843750000, -0.0000610351562500, -0.0013122558593750,
+0.0004577636718750, -0.0021362304687500, -0.0005798339843750, -0.0021362304687500,
+-0.0001220703125000, -0.0015869140625000, -0.0001525878906250, -0.0018615722656250,
+-0.0007934570312500, -0.0019531250000000, -0.0001220703125000, -0.0003662109375000,
+-0.0008239746093750, -0.0011291503906250, -0.0004882812500000, -0.0005187988281250,
+-0.0002746582031250, 0.0009155273437500, -0.0006713867187500, -0.0004272460937500,
+-0.0003662109375000, 0.0011291503906250, -0.0005187988281250, 0.0014038085937500,
+-0.0002746582031250, 0.0001831054687500, -0.0005187988281250, 0.0003356933593750,
+-0.0007019042968750, 0.0015869140625000, -0.0001831054687500, 0.0004882812500000,
+-0.0010070800781250, 0.0010070800781250, -0.0003662109375000, 0.0019836425781250,
+-0.0005187988281250, 0.0016784667968750, -0.0012207031250000, 0.0019531250000000,
+-0.0003356933593750, 0.0030822753906250, -0.0010986328125000, 0.0021667480468750,
+-0.0002136230468750, 0.0025024414062500, -0.0002441406250000, 0.0033264160156250,
+-0.0010986328125000, 0.0016174316406250, 0.0002136230468750, 0.0021057128906250,
+0.0000305175781250, 0.0025329589843750, 0.0000610351562500, 0.0020141601562500,
+0.0003051757812500, 0.0010986328125000, 0.0002441406250000, 0.0010375976562500,
+0.0005187988281250, 0.0020751953125000, 0.0003356933593750, 0.0007019042968750,
+0.0005187988281250, 0.0014038085937500, 0.0003051757812500, 0.0027770996093750,
+-0.0000305175781250, 0.0009155273437500, 0.0001831054687500, 0.0035400390625000,
+-0.0005493164062500, 0.0027465820312500, -0.0006713867187500, 0.0024719238281250,
+-0.0005798339843750, 0.0039978027343750, -0.0006103515625000, 0.0025024414062500,
+0.0000305175781250, 0.0022277832031250, 0.0006408691406250, 0.0024108886718750,
+0.0004882812500000, 0.0019226074218750, 0.0012512207031250, 0.0016784667968750,
+0.0020446777343750, 0.0021057128906250, 0.0016479492187500, 0.0011901855468750,
+0.0030517578125000, 0.0010681152343750, 0.0020446777343750, 0.0010986328125000,
+0.0021972656250000, 0.0006103515625000, 0.0028686523437500, 0.0005798339843750,
+0.0011596679687500, 0.0006408691406250, 0.0027770996093750, 0.0005493164062500,
+0.0023193359375000, 0.0003662109375000, 0.0021057128906250, 0.0005493164062500,
+0.0032958984375000, 0.0007019042968750, 0.0029602050781250, -0.0000610351562500,
+0.0029296875000000, 0.0006103515625000, 0.0034484863281250, 0.0003662109375000,
+0.0029602050781250, -0.0004577636718750, 0.0023803710937500, 0.0005187988281250,
+0.0030822753906250, -0.0004577636718750, 0.0021362304687500, 0.0002441406250000,
+0.0020141601562500, -0.0000610351562500, 0.0019226074218750, -0.0009460449218750,
+0.0013427734375000, 0.0002441406250000, 0.0011901855468750, -0.0011901855468750,
+0.0007324218750000, -0.0014038085937500, 0.0007019042968750, -0.0004577636718750,
+0.0003662109375000, -0.0008544921875000, -0.0002441406250000, -0.0015563964843750,
+-0.0002441406250000, 0.0003662109375000, -0.0003356933593750, -0.0004882812500000,
+-0.0009765625000000, -0.0003051757812500, -0.0008544921875000, 0.0012512207031250,
+-0.0004882812500000, 0.0000610351562500, -0.0011291503906250, 0.0006713867187500,
+-0.0008239746093750, 0.0005798339843750, -0.0010986328125000, 0.0004577636718750,
+-0.0012512207031250, 0.0005187988281250, -0.0010986328125000, 0.0002136230468750,
+-0.0014648437500000, 0.0010986328125000, -0.0016479492187500, 0.0007019042968750,
+-0.0017395019531250, 0.0011291503906250, -0.0016784667968750, 0.0021057128906250,
+-0.0018005371093750, 0.0016174316406250, -0.0021057128906250, 0.0027465820312500,
+-0.0013122558593750, 0.0028381347656250, -0.0018920898437500, 0.0025939941406250,
+-0.0015563964843750, 0.0031738281250000, -0.0007019042968750, 0.0032653808593750,
+-0.0015563964843750, 0.0022583007812500, -0.0003051757812500, 0.0026855468750000,
+-0.0007934570312500, 0.0021972656250000, -0.0010070800781250, 0.0012817382812500,
+-0.0003967285156250, 0.0021362304687500, -0.0011901855468750, 0.0005493164062500,
+-0.0011901855468750, 0.0008544921875000, -0.0013732910156250, 0.0009155273437500,
+-0.0016174316406250, 0.0001220703125000, -0.0018005371093750, 0.0006408691406250,
+-0.0021057128906250, 0.0002441406250000, -0.0021972656250000, 0.0004272460937500,
+-0.0023498535156250, 0.0003967285156250, -0.0025024414062500, 0.0000915527343750,
+-0.0025939941406250, 0.0004272460937500, -0.0025939941406250, 0.0002136230468750,
+-0.0023193359375000, 0.0000915527343750, -0.0025939941406250, 0.0002746582031250,
+-0.0021362304687500, 0.0003356933593750, -0.0016174316406250, 0.0001220703125000,
+-0.0022277832031250, 0.0006713867187500, -0.0014648437500000, 0.0004272460937500,
+-0.0015563964843750, 0.0004882812500000, -0.0016174316406250, 0.0008850097656250,
+-0.0013122558593750, 0.0004577636718750, -0.0016174316406250, 0.0006408691406250,
+-0.0012512207031250, 0.0006713867187500, -0.0011596679687500, 0.0005798339843750,
+-0.0012207031250000, 0.0006103515625000, -0.0011596679687500, 0.0006408691406250,
+-0.0011291503906250, 0.0004272460937500, -0.0010375976562500, 0.0006408691406250,
+-0.0013732910156250, 0.0004272460937500, -0.0010986328125000, 0.0001831054687500,
+-0.0006713867187500, 0.0006713867187500, -0.0010681152343750, -0.0000305175781250,
+-0.0007019042968750, 0.0000915527343750, -0.0007934570312500, 0.0001525878906250,
+-0.0010070800781250, -0.0001831054687500, -0.0008239746093750, 0.0000915527343750,
+-0.0007629394531250, -0.0002441406250000, -0.0010070800781250, -0.0002746582031250,
+-0.0007934570312500, -0.0003356933593750, -0.0008544921875000, -0.0005493164062500,
+-0.0010681152343750, -0.0006713867187500, -0.0008850097656250, -0.0006408691406250,
+-0.0011596679687500, -0.0007934570312500, -0.0011596679687500, -0.0008239746093750,
+-0.0011596679687500, -0.0008239746093750, -0.0012817382812500, -0.0010070800781250,
+-0.0010986328125000, -0.0009460449218750, -0.0012207031250000, -0.0011291503906250,
+-0.0010375976562500, -0.0010681152343750, -0.0009765625000000, -0.0010986328125000,
+-0.0010070800781250, -0.0014648437500000, -0.0006103515625000, -0.0009460449218750,
+-0.0008850097656250, -0.0013122558593750, -0.0005493164062500, -0.0010986328125000,
+-0.0006103515625000, -0.0005493164062500, -0.0008850097656250, -0.0009765625000000,
+-0.0005798339843750, -0.0001831054687500, -0.0013427734375000, -0.0002746582031250,
+-0.0012817382812500, -0.0001220703125000, -0.0017395019531250, 0.0003662109375000,
+-0.0026245117187500, 0.0000000000000000, -0.0024719238281250, 0.0005493164062500,
+-0.0038146972656250, 0.0003051757812500, -0.0041503906250000, 0.0002441406250000,
+-0.0045776367187500, 0.0004577636718750, -0.0055236816406250, 0.0000610351562500,
+-0.0054321289062500, 0.0002746582031250, -0.0057678222656250, 0.0001220703125000,
+-0.0059814453125000, 0.0001831054687500, -0.0055541992187500, 0.0003662109375000,
+-0.0053405761718750, 0.0000915527343750, -0.0055236816406250, 0.0003967285156250,
+-0.0043945312500000, 0.0001220703125000, -0.0045166015625000, 0.0000915527343750,
+-0.0040588378906250, 0.0000305175781250, -0.0031127929687500, -0.0005798339843750,
+-0.0034179687500000, 0.0000610351562500, -0.0027770996093750, -0.0007324218750000,
+-0.0026855468750000, -0.0005798339843750, -0.0026855468750000, 0.0000915527343750,
+-0.0025329589843750, -0.0009155273437500, -0.0027160644531250, 0.0001525878906250,
+-0.0025939941406250, -0.0001831054687500, -0.0031433105468750, -0.0002136230468750,
+-0.0029296875000000, 0.0003967285156250, -0.0028381347656250, -0.0003051757812500,
+-0.0037536621093750, 0.0004577636718750, -0.0021057128906250, 0.0001525878906250,
+-0.0032348632812500, 0.0000305175781250, -0.0025329589843750, 0.0004882812500000,
+-0.0007629394531250, 0.0001525878906250, -0.0023803710937500, 0.0002441406250000,
+-0.0006408691406250, 0.0003967285156250, -0.0006713867187500, -0.0000305175781250,
+-0.0011901855468750, -0.0002746582031250, -0.0006408691406250, 0.0000610351562500,
+-0.0008239746093750, -0.0007324218750000, -0.0010070800781250, -0.0006408691406250,
+-0.0008239746093750, -0.0003967285156250, -0.0006713867187500, -0.0008544921875000,
+-0.0008544921875000, -0.0006713867187500, -0.0008850097656250, 0.0000000000000000,
+-0.0000305175781250, -0.0003662109375000, -0.0001525878906250, -0.0000610351562500,
+0.0005798339843750, 0.0005187988281250, 0.0017395019531250, -0.0001831054687500,
+0.0014953613281250, 0.0004272460937500, 0.0027770996093750, 0.0003967285156250,
+0.0027160644531250, 0.0004272460937500, 0.0024719238281250, 0.0009765625000000,
+0.0030212402343750, 0.0010681152343750, 0.0026550292968750, 0.0013122558593750,
+0.0014648437500000, 0.0015869140625000, 0.0018005371093750, 0.0014343261718750,
+0.0011291503906250, 0.0013427734375000, -0.0002136230468750, 0.0014953613281250,
+0.0003662109375000, 0.0007629394531250, 0.0003662109375000, 0.0009155273437500,
+-0.0000610351562500, 0.0007019042968750, 0.0006408691406250, 0.0002136230468750,
+0.0015869140625000},
+{-0.0011901855468750, -0.0006103515625000, -0.0018005371093750, -0.0003662109375000,
+-0.0014038085937500, -0.0006408691406250, -0.0019531250000000, -0.0004882812500000,
+-0.0017089843750000, -0.0005493164062500, -0.0019531250000000, -0.0007629394531250,
+-0.0023498535156250, -0.0005493164062500, -0.0018005371093750, -0.0007324218750000,
+-0.0026855468750000, -0.0008544921875000, -0.0024108886718750, -0.0007629394531250,
+-0.0023193359375000, -0.0008850097656250, -0.0028686523437500, -0.0010070800781250,
+-0.0025024414062500, -0.0006408691406250, -0.0025939941406250, -0.0009155273437500,
+-0.0027465820312500, -0.0007629394531250, -0.0025634765625000, -0.0003356933593750,
+-0.0025634765625000, -0.0007629394531250, -0.0026245117187500, 0.0000000000000000,
+-0.0022277832031250, -0.0001525878906250, -0.0025634765625000, -0.0001220703125000,
+-0.0023193359375000, 0.0003662109375000, -0.0018005371093750, 0.0000915527343750,
+-0.0023193359375000, 0.0001831054687500, -0.0017395019531250, 0.0002441406250000,
+-0.0018310546875000, 0.0001220703125000, -0.0021972656250000, -0.0001220703125000,
+-0.0021057128906250, -0.0001220703125000, -0.0024108886718750, -0.0001220703125000,
+-0.0032653808593750, -0.0004577636718750, -0.0030517578125000, -0.0003356933593750,
+-0.0039062500000000, -0.0001831054687500, -0.0047912597656250, -0.0007324218750000,
+-0.0040588378906250, -0.0001220703125000, -0.0059814453125000, -0.0007019042968750,
+-0.0054016113281250, -0.0003967285156250, -0.0056762695312500, 0.0002136230468750,
+-0.0068969726562500, -0.0007629394531250, -0.0056152343750000, 0.0007324218750000,
+-0.0064697265625000, 0.0002136230468750, -0.0063781738281250, 0.0002746582031250,
+-0.0054321289062500, 0.0012512207031250, -0.0053405761718750, 0.0002441406250000,
+-0.0052490234375000, 0.0012817382812500, -0.0038146972656250, 0.0009765625000000,
+-0.0040283203125000, 0.0007324218750000, -0.0035095214843750, 0.0012512207031250,
+-0.0025329589843750, 0.0007934570312500, -0.0032653808593750, 0.0007324218750000,
+-0.0023498535156250, 0.0006103515625000, -0.0032653808593750, 0.0003662109375000,
+-0.0031738281250000, 0.0002136230468750, -0.0025939941406250, 0.0001525878906250,
+-0.0039062500000000, 0.0002136230468750, -0.0020446777343750, 0.0001831054687500,
+-0.0031433105468750, 0.0002746582031250, -0.0028686523437500, 0.0003967285156250,
+-0.0013427734375000, 0.0003662109375000, -0.0029296875000000, 0.0007629394531250,
+-0.0011596679687500, 0.0006103515625000, -0.0012817382812500, 0.0006103515625000,
+-0.0013122558593750, 0.0007629394531250, -0.0006103515625000, 0.0003356933593750,
+-0.0014038085937500, 0.0005493164062500, -0.0004272460937500, 0.0000610351562500,
+-0.0006103515625000, 0.0001831054687500, -0.0010375976562500, 0.0007629394531250,
+-0.0007324218750000, 0.0005493164062500, -0.0006408691406250, 0.0011596679687500,
+-0.0000610351562500, 0.0018005371093750, 0.0000000000000000, 0.0016784667968750,
+0.0005493164062500, 0.0017395019531250, 0.0010375976562500, 0.0022888183593750,
+0.0005493164062500, 0.0011901855468750, 0.0019836425781250, 0.0014648437500000,
+0.0014953613281250, 0.0014343261718750, 0.0013122558593750, 0.0007629394531250,
+0.0021972656250000, 0.0013427734375000, 0.0014953613281250, 0.0012207031250000,
+0.0004272460937500, 0.0014953613281250, 0.0008850097656250, 0.0014648437500000,
+0.0000610351562500, 0.0014648437500000, -0.0013732910156250, 0.0021362304687500,
+-0.0005493164062500, 0.0003662109375000, -0.0005798339843750, 0.0013427734375000,
+-0.0012207031250000, 0.0009765625000000, -0.0006408691406250, -0.0020141601562500,
+0.0006408691406250, -0.0015563964843750, -0.0016479492187500, -0.0009460449218750,
+-0.0013122558593750, -0.0012817382812500, -0.0026245117187500, -0.0012512207031250,
+-0.0020446777343750, -0.0012512207031250, -0.0007629394531250, -0.0018005371093750,
+-0.0016479492187500, -0.0021972656250000, -0.0015869140625000, -0.0028381347656250,
+-0.0009460449218750, -0.0022583007812500, -0.0011596679687500, -0.0019836425781250,
+-0.0016784667968750, -0.0027160644531250, -0.0012512207031250, -0.0003356933593750,
+-0.0004882812500000, -0.0007324218750000, -0.0010681152343750, -0.0008544921875000,
+-0.0003662109375000, 0.0005493164062500, 0.0010070800781250, -0.0003051757812500,
+0.0003967285156250, -0.0010986328125000, 0.0007934570312500, -0.0012207031250000,
+0.0016479492187500, -0.0016784667968750, 0.0010070800781250, -0.0029602050781250,
+0.0003662109375000, -0.0034179687500000, 0.0012207031250000, -0.0018615722656250,
+0.0000000000000000, -0.0034484863281250, 0.0002136230468750, -0.0029296875000000,
+0.0006713867187500, -0.0009155273437500, 0.0003051757812500, -0.0026855468750000,
+0.0004882812500000, -0.0015869140625000, 0.0010070800781250, -0.0015563964843750,
+0.0010681152343750, -0.0024719238281250, 0.0007019042968750, -0.0026245117187500,
+0.0007019042968750, -0.0028076171875000, 0.0010986328125000, -0.0039978027343750,
+0.0004577636718750, -0.0040588378906250, 0.0005798339843750, -0.0039978027343750,
+0.0007934570312500, -0.0046386718750000, 0.0005798339843750, -0.0047302246093750,
+0.0007019042968750, -0.0038146972656250, 0.0005493164062500, -0.0037536621093750,
+0.0006103515625000, -0.0039978027343750, 0.0005187988281250, -0.0034484863281250,
+0.0003662109375000, -0.0028991699218750, 0.0004882812500000, -0.0043945312500000,
+0.0002746582031250, -0.0031127929687500, 0.0002746582031250, -0.0039062500000000,
+0.0004577636718750, -0.0052795410156250, 0.0006103515625000, -0.0031433105468750,
+0.0005493164062500, -0.0061035156250000, 0.0007019042968750, -0.0051574707031250,
+0.0008850097656250, -0.0047302246093750, 0.0006103515625000, -0.0063171386718750,
+0.0004577636718750, -0.0049743652343750, 0.0006103515625000, -0.0048828125000000,
+0.0002136230468750, -0.0049743652343750, 0.0002441406250000, -0.0041503906250000,
+0.0000305175781250, -0.0034790039062500, -0.0003662109375000, -0.0036621093750000,
+-0.0003051757812500, -0.0027465820312500, -0.0010375976562500, -0.0024719238281250,
+-0.0011901855468750, -0.0025329589843750, -0.0011901855468750, -0.0020446777343750,
+-0.0016174316406250, -0.0018310546875000, -0.0018310546875000, -0.0025024414062500,
+-0.0013122558593750, -0.0017395019531250, -0.0015869140625000, -0.0021972656250000,
+-0.0015869140625000, -0.0031433105468750, -0.0010986328125000, -0.0021362304687500,
+-0.0012817382812500, -0.0031127929687500, -0.0014953613281250, -0.0031738281250000,
+-0.0013122558593750, -0.0029296875000000, -0.0014343261718750, -0.0033264160156250,
+-0.0016784667968750, -0.0032653808593750, -0.0015563964843750, -0.0029602050781250,
+-0.0016479492187500, -0.0032653808593750, -0.0016174316406250, -0.0028381347656250,
+-0.0015563964843750, -0.0024414062500000, -0.0015869140625000, -0.0028991699218750,
+-0.0015258789062500, -0.0014038085937500, -0.0014343261718750, -0.0020141601562500,
+-0.0014343261718750, -0.0012512207031250, -0.0013732910156250, 0.0000305175781250,
+-0.0011596679687500, -0.0014953613281250, -0.0011596679687500, 0.0006713867187500,
+-0.0013427734375000, -0.0001831054687500, -0.0010375976562500, -0.0001220703125000,
+-0.0013427734375000, 0.0012512207031250, -0.0018005371093750, -0.0002136230468750,
+-0.0013427734375000, 0.0012817382812500, -0.0020446777343750, 0.0009765625000000,
+-0.0021057128906250, 0.0007629394531250, -0.0018920898437500, 0.0015258789062500,
+-0.0021057128906250, 0.0010375976562500, -0.0020751953125000, 0.0013732910156250,
+-0.0019226074218750, 0.0012207031250000, -0.0019226074218750, 0.0011901855468750,
+-0.0018920898437500, 0.0012512207031250, -0.0017395019531250, 0.0007324218750000,
+-0.0017395019531250, 0.0013732910156250, -0.0016479492187500, 0.0008544921875000,
+-0.0016174316406250, 0.0009460449218750, -0.0014343261718750, 0.0013732910156250,
+-0.0012207031250000, 0.0005187988281250, -0.0012817382812500, 0.0014343261718750,
+-0.0008544921875000, 0.0007629394531250, -0.0006713867187500, 0.0007324218750000,
+-0.0006408691406250, 0.0012207031250000, -0.0003662109375000, 0.0001220703125000,
+-0.0000915527343750, 0.0009460449218750, -0.0000915527343750, 0.0003051757812500,
+0.0001220703125000, 0.0000305175781250, 0.0002136230468750, 0.0003967285156250,
+0.0003356933593750, -0.0005187988281250, 0.0005798339843750, -0.0006713867187500,
+0.0006713867187500, -0.0009460449218750, 0.0007019042968750, -0.0010681152343750,
+0.0008239746093750, -0.0011901855468750, 0.0009155273437500, -0.0016174316406250,
+0.0009155273437500, -0.0010986328125000, 0.0010375976562500, -0.0014953613281250,
+0.0010375976562500, -0.0016174316406250, 0.0011596679687500, -0.0013122558593750,
+0.0011291503906250, -0.0018005371093750, 0.0009155273437500, -0.0017395019531250,
+0.0013122558593750, -0.0018920898437500, 0.0011901855468750, -0.0022583007812500,
+0.0013122558593750, -0.0021972656250000, 0.0017089843750000, -0.0020141601562500,
+0.0015258789062500, -0.0027770996093750, 0.0020141601562500, -0.0018310546875000,
+0.0023193359375000, -0.0020141601562500, 0.0020141601562500, -0.0031738281250000,
+0.0021057128906250, -0.0024719238281250, 0.0026550292968750, -0.0029296875000000,
+0.0015563964843750, -0.0031738281250000, 0.0020141601562500, -0.0027465820312500,
+0.0017395019531250, -0.0027770996093750, 0.0010070800781250, -0.0032348632812500,
+0.0017700195312500, -0.0023193359375000, 0.0002746582031250, -0.0033569335937500,
+0.0007324218750000, -0.0032653808593750, 0.0007324218750000, -0.0026855468750000,
+-0.0003356933593750, -0.0042419433593750, 0.0001220703125000, -0.0027465820312500,
+-0.0000915527343750, -0.0032043457031250, -0.0005187988281250, -0.0034179687500000,
+-0.0003051757812500, -0.0022888183593750, -0.0001831054687500, -0.0025024414062500,
+-0.0007324218750000, -0.0023803710937500, -0.0008850097656250, -0.0017089843750000,
+-0.0007019042968750, -0.0015258789062500, -0.0012207031250000, -0.0014343261718750,
+-0.0018920898437500, -0.0009155273437500, -0.0015563964843750, -0.0009155273437500,
+-0.0021057128906250, -0.0005187988281250, -0.0021667480468750, -0.0007629394531250,
+-0.0017700195312500, -0.0010375976562500, -0.0016174316406250, -0.0001831054687500,
+-0.0015869140625000, -0.0009765625000000, -0.0005798339843750, 0.0002746582031250,
+-0.0003662109375000, 0.0001220703125000, -0.0003051757812500, -0.0000610351562500,
+0.0000610351562500, 0.0022888183593750, 0.0002746582031250, 0.0001525878906250,
+0.0009460449218750, 0.0022277832031250, 0.0006713867187500, 0.0025939941406250,
+0.0004577636718750, 0.0006408691406250, 0.0009460449218750, 0.0027465820312500,
+0.0010681152343750, 0.0014648437500000, 0.0001831054687500, 0.0011291503906250,
+0.0002746582031250, 0.0016174316406250, 0.0003051757812500, 0.0016479492187500,
+-0.0002136230468750, 0.0017700195312500, 0.0000305175781250, 0.0014038085937500,
+0.0002136230468750, 0.0019531250000000, 0.0001220703125000, 0.0013427734375000,
+0.0002746582031250, 0.0003051757812500, 0.0006408691406250, 0.0010070800781250,
+0.0006408691406250, 0.0001831054687500, 0.0006713867187500, -0.0005493164062500,
+0.0011901855468750, -0.0002136230468750, 0.0010986328125000, 0.0002441406250000,
+0.0006103515625000, 0.0001525878906250, 0.0011901855468750, 0.0008544921875000,
+0.0020446777343750, 0.0008850097656250, 0.0013427734375000, 0.0007629394531250,
+0.0010986328125000, 0.0012817382812500, 0.0014343261718750, 0.0016174316406250,
+0.0007629394531250, 0.0008544921875000, 0.0004272460937500, 0.0011901855468750,
+0.0010681152343750, 0.0013122558593750, 0.0012207031250000, 0.0007934570312500,
+0.0012207031250000, 0.0014038085937500, 0.0022583007812500, 0.0017089843750000,
+0.0024414062500000},
+{0.0001831054687500, -0.0007324218750000, 0.0004882812500000, -0.0010375976562500,
+0.0005187988281250, -0.0012207031250000, 0.0005493164062500, -0.0006103515625000,
+0.0007019042968750, -0.0010681152343750, 0.0006713867187500, -0.0008239746093750,
+0.0007934570312500, -0.0008544921875000, 0.0008239746093750, -0.0010375976562500,
+0.0008850097656250, -0.0009460449218750, 0.0009460449218750, -0.0008850097656250,
+0.0009765625000000, -0.0013427734375000, 0.0013122558593750, -0.0009460449218750,
+0.0010986328125000, -0.0014343261718750, 0.0013732910156250, -0.0022583007812500,
+0.0019836425781250, -0.0017395019531250, 0.0016784667968750, -0.0032043457031250,
+0.0018920898437500, -0.0027160644531250, 0.0024108886718750, -0.0033569335937500,
+0.0021972656250000, -0.0042419433593750, 0.0020446777343750, -0.0026245117187500,
+0.0025329589843750, -0.0046997070312500, 0.0016479492187500, -0.0043334960937500,
+0.0021972656250000, -0.0038452148437500, 0.0019226074218750, -0.0048522949218750,
+0.0010681152343750, -0.0042114257812500, 0.0018615722656250, -0.0033569335937500,
+0.0012207031250000, -0.0039978027343750, 0.0010070800781250, -0.0032043457031250,
+0.0011596679687500, -0.0023193359375000, 0.0010375976562500, -0.0037841796875000,
+0.0007629394531250, -0.0018920898437500, 0.0006713867187500, -0.0029296875000000,
+0.0005493164062500, -0.0031127929687500, 0.0000610351562500, -0.0018310546875000,
+-0.0002441406250000, -0.0031738281250000, -0.0002136230468750, -0.0023193359375000,
+-0.0009460449218750, -0.0020751953125000, -0.0012817382812500, -0.0022277832031250,
+-0.0011901855468750, -0.0019836425781250, -0.0014038085937500, -0.0018615722656250,
+-0.0016784667968750, -0.0012512207031250, -0.0009460449218750, -0.0014343261718750,
+-0.0014648437500000, -0.0007629394531250, -0.0009155273437500, 0.0001525878906250,
+0.0000000000000000, -0.0005187988281250, -0.0008544921875000, -0.0001220703125000,
+0.0001525878906250, 0.0003051757812500, -0.0000915527343750, -0.0003356933593750,
+0.0000915527343750, -0.0004272460937500, 0.0005798339843750, 0.0005187988281250,
+-0.0004577636718750, -0.0010681152343750, -0.0000305175781250, 0.0006103515625000,
+0.0000915527343750, 0.0006408691406250, -0.0003051757812500, -0.0003967285156250,
+-0.0005187988281250, 0.0020751953125000, -0.0005493164062500, 0.0007324218750000,
+-0.0001831054687500, 0.0006408691406250, -0.0002136230468750, 0.0011901855468750,
+0.0000000000000000, 0.0009460449218750, 0.0004882812500000, 0.0012817382812500,
+0.0004272460937500, 0.0009155273437500, 0.0003356933593750, 0.0014648437500000,
+0.0007324218750000, 0.0009155273437500, 0.0006408691406250, 0.0002746582031250,
+0.0003356933593750, 0.0013427734375000, 0.0007019042968750, -0.0002136230468750,
+0.0000610351562500, 0.0000915527343750, 0.0002441406250000, 0.0004272460937500,
+0.0002441406250000, -0.0007019042968750, -0.0001831054687500, -0.0007934570312500,
+0.0003662109375000, -0.0003356933593750, 0.0005798339843750, -0.0006408691406250,
+0.0002746582031250, -0.0003967285156250, 0.0000000000000000, 0.0002136230468750,
+0.0000915527343750, 0.0000000000000000, 0.0003662109375000, 0.0006713867187500,
+0.0010070800781250, 0.0011901855468750, 0.0010986328125000, 0.0009765625000000,
+0.0015563964843750, 0.0009765625000000, 0.0022888183593750, 0.0015258789062500,
+0.0020446777343750, 0.0009155273437500, 0.0015563964843750, 0.0011291503906250,
+0.0019531250000000, 0.0014038085937500, 0.0016479492187500, 0.0013427734375000,
+0.0005798339843750, 0.0016784667968750, 0.0007324218750000, 0.0016479492187500,
+0.0008544921875000, 0.0019836425781250, 0.0003662109375000, 0.0014953613281250,
+0.0004272460937500, 0.0008850097656250, 0.0008850097656250, 0.0013122558593750,
+0.0007629394531250, 0.0004882812500000, 0.0006713867187500, 0.0001220703125000,
+0.0011901855468750, 0.0001220703125000, 0.0005187988281250, -0.0000915527343750,
+-0.0002746582031250, -0.0002136230468750, 0.0003662109375000, -0.0003967285156250,
+-0.0011291503906250, -0.0001525878906250, -0.0011291503906250, -0.0005187988281250,
+-0.0014648437500000, -0.0009765625000000, -0.0022277832031250, -0.0005798339843750,
+-0.0016784667968750, -0.0012817382812500, -0.0028991699218750, -0.0013427734375000,
+-0.0019836425781250, -0.0009765625000000, -0.0023498535156250, -0.0011291503906250,
+-0.0034179687500000, -0.0013122558593750, -0.0018310546875000, -0.0001831054687500,
+-0.0033874511718750, -0.0003356933593750, -0.0028991699218750, 0.0001220703125000,
+-0.0025024414062500, 0.0010375976562500, -0.0032348632812500, 0.0003967285156250,
+-0.0025939941406250, 0.0013427734375000, -0.0022583007812500, 0.0012817382812500,
+-0.0025634765625000, 0.0012207031250000, -0.0021057128906250, 0.0016479492187500,
+-0.0014648437500000, 0.0011291503906250, -0.0019226074218750, 0.0015869140625000,
+-0.0014343261718750, 0.0014648437500000, -0.0011596679687500, 0.0013732910156250,
+-0.0015869140625000, 0.0015258789062500, -0.0017700195312500, 0.0013122558593750,
+-0.0016479492187500, 0.0013427734375000, -0.0024108886718750, 0.0013427734375000,
+-0.0025634765625000, 0.0015258789062500, -0.0024719238281250, 0.0017089843750000,
+-0.0029602050781250, 0.0014953613281250, -0.0032348632812500, 0.0020751953125000,
+-0.0025024414062500, 0.0020751953125000, -0.0033569335937500, 0.0016479492187500,
+-0.0029907226562500, 0.0017395019531250, -0.0021057128906250, 0.0018615722656250,
+-0.0032958984375000, 0.0009460449218750, -0.0019836425781250, 0.0011291503906250,
+-0.0023803710937500, 0.0008850097656250, -0.0025634765625000, 0.0001831054687500,
+-0.0019226074218750, 0.0004272460937500, -0.0025634765625000, 0.0001831054687500,
+-0.0022277832031250, -0.0000305175781250, -0.0024108886718750, -0.0001220703125000,
+-0.0024414062500000, -0.0000610351562500, -0.0023193359375000, -0.0001525878906250,
+-0.0028686523437500, -0.0006408691406250, -0.0026550292968750, -0.0006408691406250,
+-0.0028686523437500, -0.0007934570312500, -0.0032348632812500, -0.0012817382812500,
+-0.0032653808593750, -0.0011596679687500, -0.0033264160156250, -0.0010375976562500,
+-0.0038146972656250, -0.0012207031250000, -0.0039978027343750, -0.0010986328125000,
+-0.0038452148437500, -0.0007934570312500, -0.0039367675781250, -0.0008850097656250,
+-0.0041503906250000, -0.0006408691406250, -0.0039062500000000, -0.0006713867187500,
+-0.0040588378906250, -0.0006713867187500, -0.0041809082031250, -0.0006103515625000,
+-0.0041503906250000, -0.0007019042968750, -0.0043029785156250, -0.0007324218750000,
+-0.0040283203125000, -0.0007324218750000, -0.0042114257812500, -0.0007019042968750,
+-0.0042114257812500, -0.0007019042968750, -0.0039672851562500, -0.0007019042968750,
+-0.0040893554687500, -0.0006713867187500, -0.0039367675781250, -0.0007324218750000,
+-0.0038757324218750, -0.0007934570312500, -0.0037841796875000, -0.0007629394531250,
+-0.0036315917968750, -0.0008239746093750, -0.0035705566406250, -0.0010070800781250,
+-0.0036315917968750, -0.0009460449218750, -0.0033569335937500, -0.0009155273437500,
+-0.0034790039062500, -0.0010986328125000, -0.0036315917968750, -0.0010375976562500,
+-0.0032958984375000, -0.0009460449218750, -0.0038452148437500, -0.0010070800781250,
+-0.0035705566406250, -0.0010375976562500, -0.0036315917968750, -0.0008544921875000,
+-0.0040588378906250, -0.0010070800781250, -0.0036315917968750, -0.0011291503906250,
+-0.0038452148437500, -0.0010070800781250, -0.0037231445312500, -0.0012512207031250,
+-0.0036315917968750, -0.0016174316406250, -0.0036315917968750, -0.0013732910156250,
+-0.0033264160156250, -0.0016784667968750, -0.0033264160156250, -0.0018615722656250,
+-0.0029602050781250, -0.0014953613281250, -0.0028991699218750, -0.0014343261718750,
+-0.0027465820312500, -0.0017395019531250, -0.0021057128906250, -0.0006713867187500,
+-0.0027465820312500, -0.0010375976562500, -0.0017700195312500, -0.0007324218750000,
+-0.0019836425781250, 0.0000610351562500, -0.0026245117187500, -0.0006103515625000,
+-0.0013427734375000, 0.0004577636718750, -0.0031433105468750, 0.0003662109375000,
+-0.0024414062500000, 0.0003356933593750, -0.0026245117187500, 0.0008850097656250,
+-0.0041503906250000, 0.0005493164062500, -0.0031433105468750, 0.0008544921875000,
+-0.0046081542968750, 0.0007019042968750, -0.0047607421875000, 0.0007629394531250,
+-0.0046997070312500, 0.0009765625000000, -0.0054626464843750, 0.0007324218750000,
+-0.0053405761718750, 0.0010070800781250, -0.0053710937500000, 0.0011596679687500,
+-0.0054016113281250, 0.0010070800781250, -0.0052795410156250, 0.0010986328125000,
+-0.0050659179687500, 0.0013122558593750, -0.0050048828125000, 0.0010986328125000,
+-0.0050659179687500, 0.0011291503906250, -0.0047302246093750, 0.0014343261718750,
+-0.0049133300781250, 0.0016174316406250, -0.0051879882812500, 0.0016174316406250,
+-0.0047912597656250, 0.0019836425781250, -0.0054321289062500, 0.0021057128906250,
+-0.0057373046875000, 0.0021057128906250, -0.0054931640625000, 0.0022277832031250,
+-0.0057678222656250, 0.0022888183593750, -0.0062255859375000, 0.0019226074218750,
+-0.0048828125000000, 0.0016784667968750, -0.0055236816406250, 0.0016174316406250,
+-0.0048828125000000, 0.0012817382812500, -0.0036621093750000, 0.0007629394531250,
+-0.0046997070312500, 0.0013732910156250, -0.0027465820312500, 0.0007324218750000,
+-0.0031738281250000, 0.0007629394531250, -0.0031738281250000, 0.0015869140625000,
+-0.0019226074218750, 0.0010375976562500, -0.0026855468750000, 0.0007629394531250,
+-0.0026855468750000, 0.0011901855468750, -0.0022583007812500, 0.0004577636718750,
+-0.0030822753906250, -0.0005493164062500, -0.0038452148437500, -0.0002136230468750,
+-0.0031738281250000, -0.0015258789062500, -0.0037841796875000, -0.0017395019531250,
+-0.0033569335937500, -0.0021362304687500, -0.0027770996093750, -0.0029296875000000,
+-0.0026855468750000, -0.0027770996093750, -0.0018615722656250, -0.0037536621093750,
+-0.0009765625000000, -0.0035705566406250, -0.0001831054687500, -0.0039978027343750,
+-0.0000305175781250, -0.0048522949218750, 0.0004577636718750, -0.0043640136718750,
+0.0013732910156250, -0.0050659179687500, -0.0003662109375000, -0.0051574707031250,
+0.0001220703125000, -0.0047302246093750, 0.0003662109375000, -0.0046691894531250,
+-0.0007934570312500, -0.0045471191406250, -0.0004577636718750, -0.0037536621093750,
+0.0000610351562500, -0.0038146972656250, -0.0000610351562500, -0.0035095214843750,
+-0.0003051757812500, -0.0026550292968750, -0.0001831054687500, -0.0038452148437500,
+-0.0003356933593750, -0.0004272460937500, -0.0005798339843750, -0.0010070800781250,
+0.0000000000000000, -0.0010986328125000, -0.0009460449218750, -0.0001220703125000,
+-0.0014648437500000, -0.0000915527343750, -0.0004272460937500, -0.0007324218750000,
+-0.0032348632812500, -0.0007629394531250, -0.0034179687500000, -0.0006408691406250,
+-0.0033874511718750, -0.0012207031250000, -0.0050048828125000, -0.0018005371093750,
+-0.0049743652343750, -0.0007019042968750, -0.0043945312500000, -0.0014038085937500,
+-0.0047302246093750, -0.0014343261718750, -0.0044555664062500, -0.0005493164062500,
+-0.0036010742187500, -0.0013732910156250, -0.0036926269531250, -0.0014038085937500,
+-0.0035400390625000, -0.0015258789062500, -0.0038452148437500, -0.0021667480468750,
+-0.0031433105468750, -0.0027770996093750, -0.0028076171875000, -0.0028686523437500,
+-0.0036926269531250, -0.0025329589843750, -0.0008850097656250, -0.0032958984375000,
+-0.0014343261718750, -0.0026550292968750, -0.0010070800781250, -0.0015563964843750,
+0.0012817382812500, -0.0022583007812500, 0.0003051757812500, -0.0014038085937500,
+0.0003051757812500, -0.0008544921875000, 0.0010375976562500, -0.0015563964843750,
+0.0005187988281250},
+{-0.0019836425781250, 0.0007324218750000, -0.0025024414062500, 0.0008239746093750,
+-0.0033264160156250, 0.0010375976562500, -0.0026855468750000, 0.0004577636718750,
+-0.0043945312500000, 0.0007019042968750, -0.0042114257812500, 0.0006713867187500,
+-0.0044555664062500, 0.0003051757812500, -0.0056457519531250, 0.0007019042968750,
+-0.0051574707031250, 0.0006408691406250, -0.0058593750000000, 0.0007324218750000,
+-0.0060729980468750, 0.0010070800781250, -0.0057678222656250, 0.0010681152343750,
+-0.0058593750000000, 0.0010986328125000, -0.0057983398437500, 0.0012817382812500,
+-0.0050354003906250, 0.0012207031250000, -0.0049743652343750, 0.0011901855468750,
+-0.0047607421875000, 0.0010986328125000, -0.0040283203125000, 0.0007324218750000,
+-0.0039367675781250, 0.0010986328125000, -0.0039978027343750, 0.0001525878906250,
+-0.0035705566406250, 0.0003967285156250, -0.0037536621093750, 0.0010070800781250,
+-0.0040893554687500, -0.0001831054687500, -0.0034484863281250, 0.0010681152343750,
+-0.0039978027343750, 0.0011596679687500, -0.0041503906250000, 0.0007629394531250,
+-0.0037841796875000, 0.0010070800781250, -0.0036621093750000, 0.0008239746093750,
+-0.0036621093750000, 0.0000915527343750, -0.0028686523437500, 0.0000305175781250,
+-0.0029602050781250, -0.0009460449218750, -0.0029907226562500, -0.0018615722656250,
+-0.0024414062500000, -0.0013122558593750, -0.0025634765625000, -0.0032653808593750,
+-0.0029907226562500, -0.0029602050781250, -0.0021667480468750, -0.0033874511718750,
+-0.0026550292968750, -0.0047302246093750, -0.0032958984375000, -0.0039672851562500,
+-0.0017395019531250, -0.0051574707031250, -0.0031127929687500, -0.0051879882812500,
+-0.0025634765625000, -0.0049743652343750, -0.0013732910156250, -0.0054626464843750,
+-0.0012817382812500, -0.0054626464843750, -0.0006713867187500, -0.0050048828125000,
+0.0006713867187500, -0.0055236816406250, 0.0014343261718750, -0.0050048828125000,
+0.0011291503906250, -0.0046386718750000, 0.0012817382812500, -0.0057373046875000,
+0.0021362304687500, -0.0032348632812500, 0.0002441406250000, -0.0047607421875000,
+0.0004272460937500, -0.0044250488281250, 0.0006408691406250, -0.0002746582031250,
+-0.0008239746093750, 0.0002136230468750, 0.0023193359375000, 0.0000305175781250,
+0.0007019042968750, -0.0001220703125000, 0.0011291503906250, 0.0002136230468750,
+0.0006103515625000, 0.0000915527343750, -0.0002136230468750, -0.0005798339843750,
+0.0000000000000000, 0.0009155273437500, -0.0008239746093750, 0.0003662109375000,
+-0.0003356933593750, -0.0000610351562500, -0.0009765625000000, 0.0008239746093750,
+-0.0022277832031250, 0.0002441406250000, -0.0017089843750000, -0.0010681152343750,
+-0.0032043457031250, -0.0009155273437500, -0.0039978027343750, -0.0014953613281250,
+-0.0036621093750000, -0.0028991699218750, -0.0041198730468750, -0.0028381347656250,
+-0.0046997070312500, -0.0021667480468750, -0.0031738281250000, -0.0030212402343750,
+-0.0035705566406250, -0.0021667480468750, -0.0032348632812500, -0.0006103515625000,
+-0.0021057128906250, -0.0014953613281250, -0.0027465820312500, -0.0007324218750000,
+-0.0013732910156250, 0.0002746582031250, -0.0019226074218750, -0.0008850097656250,
+-0.0013427734375000, -0.0017089843750000, -0.0002746582031250, -0.0004882812500000,
+-0.0014953613281250, -0.0027160644531250, 0.0008239746093750, -0.0025329589843750,
+0.0005187988281250, -0.0023498535156250, 0.0001220703125000, -0.0034484863281250,
+0.0015258789062500, -0.0028076171875000, 0.0012512207031250, -0.0028076171875000,
+0.0003356933593750, -0.0027465820312500, 0.0006408691406250, -0.0024108886718750,
+0.0006408691406250, -0.0021057128906250, -0.0003356933593750, -0.0019226074218750,
+-0.0004882812500000, -0.0025024414062500, 0.0002746582031250, -0.0021972656250000,
+-0.0001220703125000, -0.0021972656250000, -0.0000305175781250, -0.0029907226562500,
+0.0008239746093750, -0.0031738281250000, 0.0003967285156250, -0.0024108886718750,
+0.0003356933593750, -0.0037231445312500, 0.0004272460937500, -0.0031738281250000,
+0.0003051757812500, -0.0022888183593750, 0.0000305175781250, -0.0041809082031250,
+-0.0000915527343750, -0.0013427734375000, 0.0004272460937500, -0.0025329589843750,
+0.0001831054687500, -0.0023193359375000, 0.0001220703125000, -0.0000305175781250,
+0.0004272460937500, -0.0016479492187500, 0.0003356933593750, -0.0004882812500000,
+0.0003051757812500, 0.0000305175781250, 0.0002441406250000, -0.0005798339843750,
+0.0004577636718750, -0.0004577636718750, 0.0004577636718750, 0.0000915527343750,
+0.0002441406250000, -0.0007324218750000, 0.0006408691406250, -0.0004882812500000,
+0.0007324218750000, -0.0003967285156250, 0.0004882812500000, -0.0008544921875000,
+0.0005798339843750, -0.0005493164062500, 0.0007934570312500, -0.0006408691406250,
+0.0000915527343750, -0.0007934570312500, 0.0000915527343750, -0.0003967285156250,
+0.0000915527343750, -0.0001525878906250, -0.0003662109375000, -0.0003051757812500,
+-0.0003051757812500, 0.0004882812500000, -0.0004577636718750, 0.0003967285156250,
+-0.0004882812500000, 0.0006408691406250, -0.0006713867187500, 0.0011901855468750,
+-0.0008239746093750, 0.0008544921875000, -0.0008239746093750, 0.0014953613281250,
+-0.0012817382812500, 0.0012512207031250, -0.0012817382812500, 0.0012817382812500,
+-0.0014343261718750, 0.0015258789062500, -0.0017395019531250, 0.0009155273437500,
+-0.0014953613281250, 0.0011596679687500, -0.0021972656250000, 0.0008850097656250,
+-0.0019531250000000, 0.0001220703125000, -0.0021362304687500, -0.0001220703125000,
+-0.0027465820312500, 0.0000305175781250, -0.0023498535156250, -0.0011901855468750,
+-0.0027465820312500, -0.0007934570312500, -0.0028381347656250, -0.0013732910156250,
+-0.0028076171875000, -0.0024108886718750, -0.0029296875000000, -0.0016479492187500,
+-0.0028686523437500, -0.0032653808593750, -0.0028686523437500, -0.0033569335937500,
+-0.0028686523437500, -0.0033569335937500, -0.0026245117187500, -0.0042114257812500,
+-0.0024414062500000, -0.0041198730468750, -0.0025024414062500, -0.0042419433593750,
+-0.0020446777343750, -0.0043029785156250, -0.0019226074218750, -0.0043945312500000,
+-0.0018920898437500, -0.0043334960937500, -0.0017395019531250, -0.0040283203125000,
+-0.0016479492187500, -0.0043945312500000, -0.0017089843750000, -0.0043640136718750,
+-0.0017089843750000, -0.0043029785156250, -0.0016174316406250, -0.0046691894531250,
+-0.0015869140625000, -0.0048522949218750, -0.0016784667968750, -0.0045471191406250,
+-0.0014038085937500, -0.0050048828125000, -0.0014343261718750, -0.0049438476562500,
+-0.0014038085937500, -0.0045166015625000, -0.0011291503906250, -0.0049438476562500,
+-0.0012207031250000, -0.0043029785156250, -0.0011596679687500, -0.0043640136718750,
+-0.0010986328125000, -0.0041809082031250, -0.0011596679687500, -0.0037231445312500,
+-0.0011901855468750, -0.0039672851562500, -0.0011291503906250, -0.0030822753906250,
+-0.0009765625000000, -0.0034179687500000, -0.0010681152343750, -0.0034484863281250,
+-0.0008850097656250, -0.0029907226562500, -0.0006103515625000, -0.0034484863281250,
+-0.0006713867187500, -0.0033569335937500, -0.0000305175781250, -0.0030822753906250,
+-0.0000305175781250, -0.0035705566406250, 0.0002441406250000, -0.0038146972656250,
+0.0008239746093750, -0.0032348632812500, 0.0007629394531250, -0.0042114257812500,
+0.0009765625000000, -0.0038452148437500, 0.0012207031250000, -0.0037536621093750,
+0.0012207031250000, -0.0041809082031250, 0.0012207031250000, -0.0035705566406250,
+0.0013122558593750, -0.0039367675781250, 0.0013427734375000, -0.0035400390625000,
+0.0013122558593750, -0.0034790039062500, 0.0014038085937500, -0.0036621093750000,
+0.0014343261718750, -0.0029602050781250, 0.0013427734375000, -0.0035400390625000,
+0.0016784667968750, -0.0027160644531250, 0.0015869140625000, -0.0028381347656250,
+0.0013427734375000, -0.0031127929687500, 0.0014648437500000, -0.0017089843750000,
+0.0015258789062500, -0.0034484863281250, 0.0008544921875000, -0.0022277832031250,
+0.0012207031250000, -0.0025329589843750, 0.0008850097656250, -0.0040893554687500,
+0.0003967285156250, -0.0022888183593750, 0.0010986328125000, -0.0040283203125000,
+-0.0001220703125000, -0.0041809082031250, 0.0003051757812500, -0.0033874511718750,
+0.0001525878906250, -0.0042114257812500, -0.0006408691406250, -0.0049133300781250,
+0.0002441406250000, -0.0031738281250000, -0.0007324218750000, -0.0047607421875000,
+-0.0006713867187500, -0.0042724609375000, -0.0003356933593750, -0.0025634765625000,
+-0.0006408691406250, -0.0045471191406250, -0.0006103515625000, -0.0018920898437500,
+-0.0008544921875000, -0.0022583007812500, -0.0007324218750000, -0.0020446777343750,
+-0.0008544921875000, -0.0004882812500000, -0.0013427734375000, -0.0017700195312500,
+-0.0014648437500000, -0.0007019042968750, -0.0012207031250000, -0.0014648437500000,
+-0.0014648437500000, -0.0013732910156250, -0.0013427734375000, -0.0008850097656250,
+-0.0010681152343750, -0.0024719238281250, -0.0012817382812500, -0.0007629394531250,
+-0.0004577636718750, -0.0018920898437500, -0.0003662109375000, -0.0021057128906250,
+-0.0004577636718750, -0.0007324218750000, -0.0000915527343750, -0.0019226074218750,
+0.0000915527343750, -0.0015869140625000, -0.0006408691406250, -0.0014038085937500,
+-0.0003051757812500, -0.0017700195312500, -0.0001831054687500, -0.0021057128906250,
+-0.0007934570312500, -0.0021057128906250, -0.0006713867187500, -0.0014953613281250,
+-0.0004577636718750, -0.0024719238281250, -0.0009460449218750, -0.0015869140625000,
+-0.0006713867187500, -0.0004272460937500, -0.0003967285156250, -0.0020446777343750,
+-0.0012817382812500, 0.0000000000000000, -0.0002746582031250, -0.0000610351562500,
+-0.0007629394531250, -0.0010070800781250, -0.0008239746093750, -0.0001220703125000,
+-0.0000610351562500, 0.0009155273437500, -0.0005187988281250, 0.0003051757812500,
+0.0000305175781250, -0.0000305175781250, -0.0001220703125000, 0.0000915527343750,
+0.0000610351562500, -0.0002746582031250, 0.0005493164062500, -0.0010070800781250,
+0.0000610351562500, -0.0004272460937500, 0.0003051757812500, -0.0007019042968750,
+0.0000915527343750, -0.0010070800781250, 0.0000000000000000, -0.0007019042968750,
+0.0000000000000000, -0.0010070800781250, -0.0005493164062500, -0.0012512207031250,
+-0.0002441406250000, -0.0013122558593750, -0.0002136230468750, -0.0016479492187500,
+-0.0009155273437500, -0.0018615722656250, -0.0010070800781250, -0.0015563964843750,
+-0.0003967285156250, -0.0020751953125000, -0.0016174316406250, -0.0017089843750000,
+-0.0011901855468750, -0.0017395019531250, -0.0015563964843750, -0.0019531250000000,
+-0.0025939941406250, -0.0012512207031250, -0.0018310546875000, -0.0014038085937500,
+-0.0030212402343750, -0.0011901855468750, -0.0027770996093750, -0.0006408691406250,
+-0.0033569335937500, -0.0005493164062500, -0.0042114257812500, -0.0006713867187500,
+-0.0030822753906250, -0.0000305175781250, -0.0046997070312500, -0.0003356933593750,
+-0.0043334960937500, -0.0003662109375000, -0.0036926269531250, -0.0002136230468750,
+-0.0043029785156250, -0.0010681152343750, -0.0039062500000000, -0.0004272460937500,
+-0.0027770996093750, -0.0008544921875000, -0.0032348632812500, -0.0009155273437500,
+-0.0029296875000000, -0.0004882812500000, -0.0020446777343750, -0.0011596679687500,
+-0.0028076171875000, -0.0007934570312500, -0.0023193359375000, -0.0011291503906250,
+-0.0026855468750000, -0.0011901855468750, -0.0030517578125000, -0.0011291503906250,
+-0.0029602050781250, -0.0016479492187500, -0.0033874511718750, -0.0006408691406250,
+-0.0032653808593750, -0.0012207031250000, -0.0035095214843750, -0.0005493164062500,
+-0.0033874511718750, 0.0006713867187500, -0.0032043457031250, -0.0004577636718750,
+-0.0035705566406250},
+{-0.0013122558593750, -0.0029907226562500, -0.0000305175781250, -0.0032043457031250,
+-0.0009765625000000, -0.0041809082031250, -0.0003967285156250, -0.0038146972656250,
+-0.0005493164062500, -0.0037841796875000, -0.0012817382812500, -0.0052490234375000,
+-0.0000305175781250, -0.0038757324218750, -0.0015258789062500, -0.0048217773437500,
+-0.0013732910156250, -0.0046691894531250, -0.0008239746093750, -0.0033874511718750,
+-0.0011291503906250, -0.0043945312500000, -0.0008850097656250, -0.0024414062500000,
+-0.0010375976562500, -0.0025634765625000, -0.0008850097656250, -0.0026550292968750,
+-0.0007934570312500, -0.0015258789062500, -0.0008239746093750, -0.0018005371093750,
+-0.0007324218750000, -0.0016784667968750, -0.0010986328125000, -0.0018005371093750,
+-0.0009460449218750, -0.0019226074218750, -0.0010681152343750, -0.0018615722656250,
+-0.0014953613281250, -0.0019531250000000, -0.0013427734375000, -0.0021057128906250,
+-0.0010681152343750, -0.0021057128906250, -0.0019226074218750, -0.0021057128906250,
+-0.0015563964843750, -0.0020751953125000, -0.0010070800781250, -0.0018615722656250,
+-0.0021362304687500, -0.0019226074218750, -0.0007019042968750, -0.0023193359375000,
+-0.0014343261718750, -0.0018005371093750, -0.0015258789062500, -0.0018920898437500,
+-0.0004882812500000, -0.0029296875000000, -0.0013427734375000, -0.0005798339843750,
+-0.0008544921875000, -0.0020141601562500, -0.0007629394531250, -0.0021362304687500,
+-0.0007934570312500, 0.0010070800781250, -0.0005493164062500, 0.0014953613281250,
+-0.0007019042968750, 0.0006103515625000, -0.0002136230468750, 0.0010986328125000,
+-0.0003662109375000, 0.0009765625000000, -0.0003356933593750, 0.0000000000000000,
+0.0000000000000000, 0.0000305175781250, -0.0003356933593750, 0.0000305175781250,
+-0.0002441406250000, -0.0009460449218750, -0.0000915527343750, -0.0008239746093750,
+-0.0006408691406250, -0.0004272460937500, -0.0008850097656250, -0.0014343261718750,
+-0.0006103515625000, -0.0007934570312500, -0.0020141601562500, -0.0012207031250000,
+-0.0016479492187500, -0.0016479492187500, -0.0018310546875000, -0.0014953613281250,
+-0.0026855468750000, -0.0020751953125000, -0.0020751953125000, -0.0019226074218750,
+-0.0031127929687500, -0.0022277832031250, -0.0029296875000000, -0.0021362304687500,
+-0.0033264160156250, -0.0016784667968750, -0.0041198730468750, -0.0019531250000000,
+-0.0034484863281250, -0.0016174316406250, -0.0044860839843750, -0.0012207031250000,
+-0.0039978027343750, -0.0013122558593750, -0.0037231445312500, -0.0012817382812500,
+-0.0043640136718750, -0.0007324218750000, -0.0035705566406250, -0.0007629394531250,
+-0.0027465820312500, -0.0007934570312500, -0.0030822753906250, -0.0003051757812500,
+-0.0023803710937500, 0.0000610351562500, -0.0015563964843750, -0.0002136230468750,
+-0.0024719238281250, 0.0002136230468750, -0.0014038085937500, 0.0000610351562500,
+-0.0020751953125000, -0.0003662109375000, -0.0022277832031250, -0.0004577636718750,
+-0.0018310546875000, -0.0006408691406250, -0.0030212402343750, -0.0011291503906250,
+-0.0020141601562500, -0.0013122558593750, -0.0028076171875000, -0.0015869140625000,
+-0.0028686523437500, -0.0019226074218750, -0.0021362304687500, -0.0018615722656250,
+-0.0031738281250000, -0.0020751953125000, -0.0023193359375000, -0.0023498535156250,
+-0.0025024414062500, -0.0020141601562500, -0.0027770996093750, -0.0019836425781250,
+-0.0025939941406250, -0.0024414062500000, -0.0031433105468750, -0.0010070800781250,
+-0.0027160644531250, -0.0017089843750000, -0.0034484863281250, -0.0011901855468750,
+-0.0031738281250000, 0.0001831054687500, -0.0026550292968750, -0.0010681152343750,
+-0.0038452148437500, 0.0003051757812500, -0.0021667480468750, 0.0002441406250000,
+-0.0030212402343750, -0.0001525878906250, -0.0029907226562500, 0.0002136230468750,
+-0.0017089843750000, -0.0000610351562500, -0.0028381347656250, -0.0000305175781250,
+-0.0021362304687500, -0.0001525878906250, -0.0019226074218750, -0.0002136230468750,
+-0.0020446777343750, -0.0000305175781250, -0.0018005371093750, -0.0002746582031250,
+-0.0018310546875000, -0.0001220703125000, -0.0022583007812500, -0.0000305175781250,
+-0.0022888183593750, -0.0001525878906250, -0.0025024414062500, -0.0001220703125000,
+-0.0029602050781250, -0.0000610351562500, -0.0030212402343750, -0.0003662109375000,
+-0.0032958984375000, -0.0000610351562500, -0.0034179687500000, -0.0002746582031250,
+-0.0036315917968750, -0.0005798339843750, -0.0038757324218750, -0.0000305175781250,
+-0.0037231445312500, -0.0008850097656250, -0.0040283203125000, -0.0005493164062500,
+-0.0039062500000000, -0.0007324218750000, -0.0033874511718750, -0.0014648437500000,
+-0.0031738281250000, -0.0009155273437500, -0.0032348632812500, -0.0016174316406250,
+-0.0025024414062500, -0.0017700195312500, -0.0025024414062500, -0.0017700195312500,
+-0.0023498535156250, -0.0021057128906250, -0.0020141601562500, -0.0021362304687500,
+-0.0021362304687500, -0.0023803710937500, -0.0016174316406250, -0.0023803710937500,
+-0.0018005371093750, -0.0025634765625000, -0.0016174316406250, -0.0028076171875000,
+-0.0011901855468750, -0.0027465820312500, -0.0017395019531250, -0.0029602050781250,
+-0.0012512207031250, -0.0029907226562500, -0.0012207031250000, -0.0027770996093750,
+-0.0015258789062500, -0.0027160644531250, -0.0014038085937500, -0.0027770996093750,
+-0.0013427734375000, -0.0023498535156250, -0.0017700195312500, -0.0024108886718750,
+-0.0015563964843750, -0.0024414062500000, -0.0015869140625000, -0.0021972656250000,
+-0.0018615722656250, -0.0023498535156250, -0.0015563964843750, -0.0024414062500000,
+-0.0020751953125000, -0.0022583007812500, -0.0016784667968750, -0.0023498535156250,
+-0.0018615722656250, -0.0023498535156250, -0.0021972656250000, -0.0020141601562500,
+-0.0014953613281250, -0.0019836425781250, -0.0021057128906250, -0.0018310546875000,
+-0.0017395019531250, -0.0014953613281250, -0.0015258789062500, -0.0013732910156250,
+-0.0018920898437500, -0.0015258789062500, -0.0015563964843750, -0.0010070800781250,
+-0.0017395019531250, -0.0014038085937500, -0.0017089843750000, -0.0012817382812500,
+-0.0015869140625000, -0.0008544921875000, -0.0017089843750000, -0.0014038085937500,
+-0.0016479492187500, -0.0008239746093750, -0.0015869140625000, -0.0008544921875000,
+-0.0019531250000000, -0.0009155273437500, -0.0017395019531250, -0.0005798339843750,
+-0.0016784667968750, -0.0007019042968750, -0.0023193359375000, -0.0007019042968750,
+-0.0013122558593750, -0.0006408691406250, -0.0019531250000000, -0.0007324218750000,
+-0.0015869140625000, -0.0008239746093750, -0.0006408691406250, -0.0008239746093750,
+-0.0017089843750000, -0.0008850097656250, -0.0001525878906250, -0.0009765625000000,
+-0.0003051757812500, -0.0009765625000000, -0.0003967285156250, -0.0010375976562500,
+0.0005798339843750, -0.0011291503906250, 0.0003662109375000, -0.0009765625000000,
+0.0001831054687500, -0.0010375976562500, 0.0008239746093750, -0.0008544921875000,
+0.0002441406250000, -0.0006103515625000, -0.0003051757812500, -0.0007629394531250,
+0.0006713867187500, -0.0004882812500000, -0.0011291503906250, -0.0006408691406250,
+-0.0005493164062500, -0.0004882812500000, -0.0009765625000000, -0.0003356933593750,
+-0.0022277832031250, -0.0008239746093750, -0.0009460449218750, -0.0001220703125000,
+-0.0024414062500000, -0.0003967285156250, -0.0018920898437500, -0.0005798339843750,
+-0.0017395019531250, -0.0003356933593750, -0.0023803710937500, -0.0009155273437500,
+-0.0012817382812500, -0.0005187988281250, -0.0024108886718750, -0.0014343261718750,
+-0.0014953613281250, -0.0011291503906250, -0.0019836425781250, -0.0003356933593750,
+-0.0030517578125000, -0.0014038085937500, -0.0014648437500000, 0.0001525878906250,
+-0.0041809082031250, 0.0001220703125000, -0.0034484863281250, -0.0001525878906250,
+-0.0035095214843750, 0.0004577636718750, -0.0052490234375000, 0.0000305175781250,
+-0.0039367675781250, -0.0003662109375000, -0.0047302246093750, -0.0004577636718750,
+-0.0042724609375000, -0.0008544921875000, -0.0038146972656250, -0.0013427734375000,
+-0.0040893554687500, -0.0014343261718750, -0.0034790039062500, -0.0016784667968750,
+-0.0031127929687500, -0.0019836425781250, -0.0027770996093750, -0.0021667480468750,
+-0.0025024414062500, -0.0023498535156250, -0.0021972656250000, -0.0023498535156250,
+-0.0020141601562500, -0.0026245117187500, -0.0023193359375000, -0.0027160644531250,
+-0.0020751953125000, -0.0028381347656250, -0.0025024414062500, -0.0032958984375000,
+-0.0034484863281250, -0.0034790039062500, -0.0032348632812500, -0.0026550292968750,
+-0.0033569335937500, -0.0036926269531250, -0.0043029785156250, -0.0025939941406250,
+-0.0033569335937500, -0.0013427734375000, -0.0022583007812500, -0.0031738281250000,
+-0.0032653808593750, 0.0003967285156250, -0.0012817382812500, -0.0007934570312500,
+-0.0008850097656250, -0.0008239746093750, -0.0010375976562500, 0.0014648437500000,
+-0.0001525878906250, -0.0002746582031250, 0.0002136230468750, 0.0001525878906250,
+-0.0000305175781250, 0.0002136230468750, 0.0001220703125000, 0.0000610351562500,
+0.0007629394531250, 0.0003356933593750, 0.0012207031250000, -0.0001220703125000,
+0.0008239746093750, -0.0006408691406250, 0.0003356933593750, -0.0004882812500000,
+0.0001220703125000, -0.0011291503906250, 0.0010070800781250, -0.0019836425781250,
+0.0016174316406250, -0.0018920898437500, 0.0011596679687500, -0.0018615722656250,
+0.0024414062500000, -0.0028686523437500, 0.0025329589843750, -0.0023193359375000,
+0.0018310546875000, -0.0015563964843750, 0.0018920898437500, -0.0027465820312500,
+0.0018920898437500, -0.0015258789062500, 0.0004577636718750, -0.0014343261718750,
+0.0003662109375000, -0.0021057128906250, 0.0006103515625000, -0.0019531250000000,
+-0.0001220703125000, -0.0018615722656250, -0.0003967285156250, -0.0025634765625000,
+0.0003051757812500, -0.0028991699218750, -0.0000610351562500, -0.0023803710937500,
+-0.0006103515625000, -0.0023803710937500, -0.0003662109375000, -0.0029907226562500,
+-0.0005187988281250, -0.0015563964843750, -0.0014343261718750, -0.0015869140625000,
+-0.0014648437500000, -0.0020751953125000, -0.0011596679687500, -0.0015869140625000,
+-0.0014953613281250, -0.0017395019531250, -0.0015258789062500, -0.0027160644531250,
+0.0000000000000000, -0.0026855468750000, -0.0003051757812500, -0.0028991699218750,
+0.0000000000000000, -0.0037231445312500, 0.0011291503906250, -0.0035400390625000,
+0.0004882812500000, -0.0028076171875000, 0.0008544921875000, -0.0036315917968750,
+0.0006713867187500, -0.0023803710937500, 0.0004882812500000, -0.0008544921875000,
+0.0007934570312500, -0.0022583007812500, 0.0007324218750000, 0.0001831054687500,
+0.0007324218750000, 0.0000305175781250, 0.0009765625000000, -0.0004577636718750,
+0.0008239746093750, 0.0004882812500000, 0.0004577636718750, 0.0000000000000000,
+0.0005187988281250, -0.0003662109375000, 0.0001525878906250, 0.0000000000000000,
+-0.0000610351562500, -0.0004272460937500, -0.0000305175781250, -0.0008850097656250,
+-0.0000610351562500, -0.0000610351562500, -0.0001525878906250, -0.0006713867187500,
+-0.0000915527343750, -0.0003662109375000, -0.0000305175781250, 0.0002136230468750,
+-0.0001525878906250, 0.0002746582031250, -0.0002746582031250, 0.0007324218750000,
+-0.0001220703125000, 0.0013732910156250, -0.0003051757812500, 0.0015869140625000,
+-0.0003356933593750, 0.0017395019531250, -0.0001831054687500, 0.0019836425781250,
+-0.0000610351562500, 0.0019836425781250, -0.0001220703125000, 0.0014953613281250,
+0.0000610351562500, 0.0016784667968750, 0.0001831054687500, 0.0011291503906250,
+0.0000610351562500, 0.0005187988281250, 0.0000000000000000, 0.0010070800781250,
+0.0001525878906250, -0.0005798339843750, -0.0000915527343750, -0.0003356933593750,
+-0.0001220703125000},
+{-0.0030517578125000, -0.0027770996093750, -0.0031738281250000, -0.0021972656250000,
+-0.0026855468750000, -0.0013122558593750, -0.0019531250000000, -0.0025329589843750,
+-0.0021972656250000, 0.0000305175781250, -0.0024719238281250, -0.0009460449218750,
+-0.0023193359375000, -0.0005187988281250, -0.0029296875000000, 0.0014343261718750,
+-0.0039367675781250, -0.0005187988281250, -0.0038452148437500, 0.0008850097656250,
+-0.0033264160156250, 0.0008850097656250, -0.0043945312500000, 0.0000610351562500,
+-0.0031433105468750, 0.0001831054687500, -0.0014648437500000, 0.0003356933593750,
+-0.0027465820312500, 0.0002746582031250, -0.0001220703125000, -0.0000610351562500,
+0.0003051757812500, 0.0000305175781250, -0.0002136230468750, 0.0004882812500000,
+0.0006408691406250, -0.0014038085937500, -0.0016174316406250, -0.0015869140625000,
+-0.0000305175781250, -0.0022277832031250, 0.0001220703125000, -0.0020751953125000,
+-0.0000305175781250, -0.0019531250000000, -0.0000305175781250, -0.0028381347656250,
+0.0000610351562500, -0.0025634765625000, 0.0003662109375000, -0.0028686523437500,
+0.0001831054687500, -0.0032653808593750, 0.0007324218750000, -0.0034484863281250,
+0.0015563964843750, -0.0038452148437500, 0.0012207031250000, -0.0031433105468750,
+0.0016479492187500, -0.0037841796875000, 0.0020446777343750, -0.0032043457031250,
+0.0015258789062500, -0.0021362304687500, 0.0011596679687500, -0.0031127929687500,
+0.0014953613281250, -0.0017089843750000, 0.0006408691406250, -0.0016784667968750,
+0.0003051757812500, -0.0023193359375000, 0.0004882812500000, -0.0021667480468750,
+0.0002746582031250, -0.0022888183593750, -0.0001831054687500, -0.0034484863281250,
+0.0001220703125000, -0.0038757324218750, 0.0002136230468750, -0.0035095214843750,
+-0.0004577636718750, -0.0037841796875000, -0.0007629394531250, -0.0042114257812500,
+-0.0004272460937500, -0.0025024414062500, -0.0010986328125000, -0.0029296875000000,
+-0.0013122558593750, -0.0023193359375000, -0.0005187988281250, -0.0008850097656250,
+-0.0001525878906250, -0.0016479492187500, -0.0005187988281250, -0.0004272460937500,
+0.0010681152343750, -0.0004577636718750, 0.0009765625000000, -0.0007019042968750,
+0.0008544921875000, -0.0004577636718750, 0.0014648437500000, -0.0007629394531250,
+0.0010070800781250, -0.0004882812500000, 0.0008850097656250, -0.0010070800781250,
+0.0011901855468750, -0.0006713867187500, 0.0009460449218750, 0.0001220703125000,
+0.0008239746093750, -0.0003051757812500, 0.0011291503906250, 0.0006713867187500,
+0.0005798339843750, 0.0010681152343750, 0.0005798339843750, 0.0011596679687500,
+0.0004272460937500, 0.0018005371093750, -0.0000610351562500, 0.0021667480468750,
+0.0000915527343750, 0.0018310546875000, -0.0000305175781250, 0.0025939941406250,
+-0.0001831054687500, 0.0022583007812500, -0.0001220703125000, 0.0014648437500000,
+0.0000000000000000, 0.0023193359375000, 0.0000000000000000, 0.0010375976562500,
+-0.0001220703125000, 0.0008544921875000, 0.0001525878906250, 0.0008544921875000,
+-0.0000915527343750, 0.0001220703125000, -0.0004577636718750, -0.0000305175781250,
+-0.0000305175781250, 0.0000000000000000, -0.0004882812500000, -0.0004577636718750,
+-0.0005187988281250, -0.0003051757812500, -0.0003356933593750, -0.0001831054687500,
+-0.0003051757812500, -0.0006408691406250, -0.0002136230468750, 0.0004272460937500,
+-0.0002441406250000, -0.0000915527343750, -0.0000610351562500, 0.0001525878906250,
+-0.0002746582031250, 0.0010375976562500, -0.0005187988281250, 0.0001220703125000,
+-0.0002746582031250, 0.0007019042968750, -0.0008239746093750, 0.0007934570312500,
+-0.0007934570312500, 0.0002441406250000, -0.0009460449218750, 0.0001525878906250,
+-0.0013122558593750, 0.0003356933593750, -0.0010070800781250, -0.0006713867187500,
+-0.0017089843750000, -0.0002746582031250, -0.0016479492187500, -0.0007324218750000,
+-0.0017089843750000, -0.0015563964843750, -0.0021667480468750, -0.0007934570312500,
+-0.0020141601562500, -0.0020446777343750, -0.0018920898437500, -0.0018005371093750,
+-0.0019836425781250, -0.0016479492187500, -0.0019531250000000, -0.0022277832031250,
+-0.0018310546875000, -0.0018310546875000, -0.0019531250000000, -0.0021362304687500,
+-0.0020141601562500, -0.0022277832031250, -0.0020446777343750, -0.0020751953125000,
+-0.0021057128906250, -0.0021667480468750, -0.0022277832031250, -0.0022888183593750,
+-0.0022888183593750, -0.0021972656250000, -0.0022583007812500, -0.0016479492187500,
+-0.0023803710937500, -0.0018615722656250, -0.0023803710937500, -0.0017089843750000,
+-0.0023193359375000, -0.0004882812500000, -0.0024719238281250, -0.0018005371093750,
+-0.0023803710937500, -0.0011291503906250, -0.0023803710937500, -0.0010070800781250,
+-0.0024108886718750, -0.0018310546875000, -0.0023193359375000, -0.0010070800781250,
+-0.0023193359375000, -0.0019226074218750, -0.0023193359375000, -0.0012817382812500,
+-0.0022583007812500, -0.0014343261718750, -0.0022583007812500, -0.0020141601562500,
+-0.0023193359375000, -0.0006103515625000, -0.0022888183593750, -0.0021972656250000,
+-0.0022277832031250, -0.0012207031250000, -0.0023193359375000, -0.0012512207031250,
+-0.0021972656250000, -0.0023193359375000, -0.0020141601562500, -0.0009765625000000,
+-0.0022277832031250, -0.0023498535156250, -0.0018615722656250, -0.0021362304687500,
+-0.0019226074218750, -0.0019531250000000, -0.0018005371093750, -0.0028076171875000,
+-0.0015869140625000, -0.0024719238281250, -0.0017395019531250, -0.0023498535156250,
+-0.0015563964843750, -0.0029602050781250, -0.0015563964843750, -0.0027160644531250,
+-0.0015563964843750, -0.0025024414062500, -0.0015258789062500, -0.0031433105468750,
+-0.0016479492187500, -0.0020141601562500, -0.0015563964843750, -0.0024719238281250,
+-0.0017089843750000, -0.0021667480468750, -0.0017395019531250, -0.0014038085937500,
+-0.0016479492187500, -0.0024108886718750, -0.0017395019531250, -0.0013427734375000,
+-0.0014953613281250, -0.0021057128906250, -0.0016174316406250, -0.0021667480468750,
+-0.0015869140625000, -0.0015869140625000, -0.0012512207031250, -0.0027770996093750,
+-0.0013732910156250, -0.0019531250000000, -0.0014343261718750, -0.0023498535156250,
+-0.0011901855468750, -0.0028076171875000, -0.0013732910156250, -0.0024414062500000,
+-0.0014648437500000, -0.0028991699218750, -0.0010681152343750, -0.0029296875000000,
+-0.0019531250000000, -0.0029907226562500, -0.0014953613281250, -0.0031127929687500,
+-0.0017089843750000, -0.0032653808593750, -0.0024719238281250, -0.0032653808593750,
+-0.0018310546875000, -0.0027465820312500, -0.0023498535156250, -0.0031738281250000,
+-0.0021667480468750, -0.0026550292968750, -0.0020446777343750, -0.0018310546875000,
+-0.0022277832031250, -0.0024719238281250, -0.0017395019531250, -0.0013427734375000,
+-0.0018920898437500, -0.0009460449218750, -0.0017395019531250, -0.0012512207031250,
+-0.0016479492187500, -0.0007934570312500, -0.0016784667968750, -0.0002746582031250,
+-0.0014038085937500, -0.0014343261718750, -0.0016174316406250, -0.0005798339843750,
+-0.0017089843750000, -0.0007019042968750, -0.0015563964843750, -0.0018920898437500,
+-0.0013427734375000, -0.0010986328125000, -0.0013732910156250, -0.0014343261718750,
+-0.0017395019531250, -0.0025939941406250, -0.0012817382812500, -0.0020141601562500,
+-0.0012512207031250, -0.0017089843750000, -0.0018310546875000, -0.0035095214843750,
+-0.0016479492187500, -0.0016784667968750, -0.0014648437500000, -0.0024414062500000,
+-0.0018920898437500, -0.0028991699218750, -0.0017089843750000, -0.0017395019531250,
+-0.0016174316406250, -0.0023498535156250, -0.0025329589843750, -0.0022888183593750,
+-0.0013732910156250, -0.0020141601562500, -0.0027160644531250, -0.0023193359375000,
+-0.0024108886718750, -0.0023193359375000, -0.0012512207031250, -0.0015563964843750,
+-0.0031738281250000, -0.0018615722656250, -0.0014038085937500, -0.0015563964843750,
+-0.0017395019531250, -0.0014648437500000, -0.0020141601562500, -0.0012207031250000,
+-0.0012207031250000, -0.0005187988281250, -0.0018005371093750, -0.0022277832031250,
+-0.0014343261718750, -0.0005493164062500, -0.0016479492187500, -0.0012207031250000,
+-0.0013732910156250, -0.0030212402343750, -0.0010070800781250, -0.0004577636718750,
+-0.0014648437500000, -0.0027160644531250, -0.0003356933593750, -0.0022888183593750,
+-0.0003356933593750, -0.0011901855468750, -0.0004272460937500, -0.0018310546875000,
+-0.0003051757812500, -0.0011596679687500, -0.0010986328125000, 0.0003356933593750,
+-0.0012817382812500, -0.0001525878906250, -0.0011291503906250, -0.0002136230468750,
+-0.0017700195312500, 0.0005187988281250, -0.0023193359375000, -0.0000305175781250,
+-0.0021362304687500, 0.0002746582031250, -0.0036315917968750, 0.0003356933593750,
+-0.0032653808593750, 0.0003662109375000, -0.0035400390625000, 0.0004272460937500,
+-0.0045166015625000, 0.0001220703125000, -0.0038757324218750, 0.0005187988281250,
+-0.0046997070312500, 0.0000000000000000, -0.0047607421875000, 0.0001831054687500,
+-0.0046691894531250, 0.0005187988281250, -0.0050659179687500, -0.0002441406250000,
+-0.0048522949218750, 0.0010070800781250, -0.0046691894531250, 0.0001525878906250,
+-0.0046386718750000, 0.0000305175781250, -0.0043334960937500, 0.0007934570312500,
+-0.0041198730468750, -0.0004577636718750, -0.0043334960937500, 0.0001220703125000,
+-0.0036926269531250, -0.0004882812500000, -0.0040588378906250, -0.0006408691406250,
+-0.0035705566406250, -0.0001831054687500, -0.0029296875000000, -0.0008544921875000,
+-0.0036010742187500, -0.0002441406250000, -0.0020141601562500, 0.0000610351562500,
+-0.0024719238281250, -0.0005493164062500, -0.0023803710937500, -0.0004272460937500,
+-0.0011291503906250, 0.0004577636718750, -0.0016784667968750, -0.0011596679687500,
+-0.0016174316406250, -0.0004272460937500, -0.0012817382812500, -0.0003356933593750,
+-0.0014038085937500, -0.0013427734375000, -0.0016784667968750, -0.0003051757812500,
+-0.0015869140625000, -0.0010070800781250, -0.0011596679687500, -0.0007324218750000,
+-0.0016174316406250, -0.0008850097656250, -0.0010986328125000, -0.0014343261718750,
+-0.0006408691406250, -0.0010681152343750, -0.0015258789062500, -0.0023498535156250,
+-0.0000610351562500, -0.0020446777343750, -0.0007934570312500, -0.0023498535156250,
+-0.0007934570312500, -0.0034484863281250, 0.0000000000000000, -0.0028381347656250,
+-0.0012817382812500, -0.0035705566406250, -0.0005798339843750, -0.0038452148437500,
+-0.0011291503906250, -0.0033569335937500, -0.0014953613281250, -0.0034790039062500,
+-0.0013122558593750, -0.0039062500000000, -0.0020141601562500, -0.0026550292968750,
+-0.0017395019531250, -0.0031433105468750, -0.0020446777343750, -0.0027770996093750,
+-0.0019531250000000, -0.0018310546875000, -0.0014953613281250, -0.0027465820312500,
+-0.0018005371093750, -0.0015258789062500, -0.0010070800781250, -0.0018615722656250,
+-0.0011596679687500, -0.0018920898437500, -0.0011291503906250, -0.0011901855468750,
+-0.0006713867187500, -0.0018615722656250, -0.0010375976562500, -0.0013427734375000,
+-0.0006408691406250, -0.0014038085937500, -0.0007934570312500, -0.0016174316406250,
+-0.0010070800781250, -0.0014343261718750, -0.0009155273437500, -0.0015258789062500,
+-0.0011596679687500, -0.0018615722656250, -0.0014038085937500, -0.0016479492187500,
+-0.0013122558593750, -0.0018310546875000, -0.0015869140625000, -0.0019836425781250,
+-0.0018005371093750, -0.0015563964843750, -0.0016174316406250, -0.0021362304687500,
+-0.0016174316406250, -0.0017089843750000, -0.0016784667968750, -0.0018615722656250,
+-0.0013732910156250, -0.0023803710937500, -0.0011901855468750, -0.0017395019531250,
+-0.0012512207031250, -0.0023498535156250, -0.0004882812500000, -0.0020141601562500,
+-0.0004882812500000, -0.0020141601562500, -0.0003967285156250, -0.0025329589843750,
+0.0000915527343750},
+{-0.0009765625000000, -0.0038757324218750, -0.0013122558593750, -0.0025024414062500,
+-0.0014953613281250, -0.0001220703125000, -0.0023803710937500, -0.0013427734375000,
+-0.0020751953125000, -0.0010070800781250, -0.0025024414062500, 0.0007019042968750,
+-0.0033874511718750, -0.0006103515625000, -0.0028381347656250, 0.0005798339843750,
+-0.0038757324218750, 0.0003051757812500, -0.0040893554687500, 0.0001525878906250,
+-0.0039978027343750, 0.0005798339843750, -0.0044555664062500, 0.0000000000000000,
+-0.0045776367187500, 0.0007934570312500, -0.0045471191406250, 0.0002746582031250,
+-0.0047912597656250, 0.0002441406250000, -0.0049133300781250, 0.0009460449218750,
+-0.0048217773437500, 0.0001831054687500, -0.0047607421875000, 0.0003051757812500,
+-0.0046081542968750, 0.0003662109375000, -0.0047607421875000, 0.0003356933593750,
+-0.0042419433593750, 0.0001220703125000, -0.0039367675781250, -0.0002136230468750,
+-0.0043334960937500, 0.0004577636718750, -0.0027770996093750, 0.0000305175781250,
+-0.0034790039062500, 0.0000610351562500, -0.0030517578125000, 0.0005798339843750,
+-0.0018615722656250, -0.0000305175781250, -0.0033874511718750, 0.0001831054687500,
+-0.0021057128906250, 0.0000915527343750, -0.0022888183593750, -0.0001525878906250,
+-0.0026855468750000, 0.0000915527343750, -0.0021057128906250, 0.0000915527343750,
+-0.0022583007812500, -0.0006408691406250, -0.0020141601562500, 0.0001525878906250,
+-0.0017700195312500, -0.0002746582031250, -0.0013427734375000, -0.0010986328125000,
+-0.0009155273437500, 0.0001220703125000, -0.0007019042968750, -0.0013732910156250,
+-0.0001220703125000, -0.0009155273437500, -0.0000305175781250, -0.0008239746093750,
+0.0000610351562500, -0.0016784667968750, 0.0002441406250000, -0.0009765625000000,
+0.0000000000000000, -0.0019531250000000, 0.0001220703125000, -0.0015563964843750,
+-0.0003662109375000, -0.0018615722656250, -0.0003051757812500, -0.0026245117187500,
+-0.0002746582031250, -0.0019836425781250, -0.0010375976562500, -0.0032348632812500,
+-0.0001525878906250, -0.0031738281250000, -0.0006103515625000, -0.0031738281250000,
+-0.0006713867187500, -0.0039672851562500, 0.0000610351562500, -0.0037231445312500,
+-0.0003967285156250, -0.0034790039062500, -0.0003967285156250, -0.0039367675781250,
+-0.0000610351562500, -0.0034484863281250, -0.0002746582031250, -0.0028686523437500,
+-0.0006713867187500, -0.0035095214843750, -0.0003967285156250, -0.0022277832031250,
+-0.0008544921875000, -0.0024108886718750, -0.0011291503906250, -0.0023498535156250,
+-0.0011901855468750, -0.0016174316406250, -0.0013732910156250, -0.0020446777343750,
+-0.0015563964843750, -0.0015869140625000, -0.0012512207031250, -0.0017089843750000,
+-0.0014648437500000, -0.0017700195312500, -0.0012207031250000, -0.0014343261718750,
+-0.0008544921875000, -0.0017089843750000, -0.0011291503906250, -0.0016784667968750,
+-0.0002441406250000, -0.0016174316406250, -0.0003662109375000, -0.0017089843750000,
+-0.0000915527343750, -0.0017089843750000, 0.0006103515625000, -0.0015563964843750,
+0.0004272460937500, -0.0017700195312500, 0.0010986328125000, -0.0014953613281250,
+0.0011901855468750, -0.0017395019531250, 0.0013732910156250, -0.0018615722656250,
+0.0017700195312500, -0.0013427734375000, 0.0016174316406250, -0.0021972656250000,
+0.0020141601562500, -0.0015869140625000, 0.0021667480468750, -0.0016784667968750,
+0.0019531250000000, -0.0024108886718750, 0.0019531250000000, -0.0015869140625000,
+0.0020141601562500, -0.0022583007812500, 0.0014648437500000, -0.0020446777343750,
+0.0016479492187500, -0.0021362304687500, 0.0013732910156250, -0.0025024414062500,
+0.0009155273437500, -0.0020141601562500, 0.0012817382812500, -0.0026855468750000,
+0.0005798339843750, -0.0024108886718750, 0.0006713867187500, -0.0023803710937500,
+0.0007019042968750, -0.0027465820312500, 0.0003356933593750, -0.0022888183593750,
+0.0004882812500000, -0.0026245117187500, 0.0005493164062500, -0.0025329589843750,
+0.0003662109375000, -0.0023803710937500, 0.0005187988281250, -0.0025634765625000,
+0.0007629394531250, -0.0025024414062500, 0.0004577636718750, -0.0024414062500000,
+0.0008239746093750, -0.0025024414062500, 0.0005187988281250, -0.0024414062500000,
+0.0006103515625000, -0.0023193359375000, 0.0009155273437500, -0.0023498535156250,
+0.0004882812500000, -0.0023803710937500, 0.0012512207031250, -0.0022888183593750,
+0.0008850097656250, -0.0023803710937500, 0.0009765625000000, -0.0024414062500000,
+0.0014648437500000, -0.0022888183593750, 0.0008544921875000, -0.0025939941406250,
+0.0013732910156250, -0.0024414062500000, 0.0011901855468750, -0.0025024414062500,
+0.0010375976562500, -0.0026245117187500, 0.0013122558593750, -0.0023193359375000,
+0.0010375976562500, -0.0026245117187500, 0.0008850097656250, -0.0024414062500000,
+0.0010375976562500, -0.0024108886718750, 0.0004577636718750, -0.0026245117187500,
+0.0000000000000000, -0.0025024414062500, 0.0004272460937500, -0.0026245117187500,
+-0.0007629394531250, -0.0026855468750000, -0.0004577636718750, -0.0025024414062500,
+-0.0005798339843750, -0.0024719238281250, -0.0013427734375000, -0.0025024414062500,
+-0.0005798339843750, -0.0018920898437500, -0.0010070800781250, -0.0019836425781250,
+-0.0008850097656250, -0.0017395019531250, -0.0004577636718750, -0.0012207031250000,
+-0.0004272460937500, -0.0015563964843750, -0.0003662109375000, -0.0012207031250000,
+0.0004577636718750, -0.0012207031250000, 0.0003356933593750, -0.0015258789062500,
+0.0007629394531250, -0.0015563964843750, 0.0014648437500000, -0.0016174316406250,
+0.0010070800781250, -0.0017089843750000, 0.0014343261718750, -0.0021362304687500,
+0.0015563964843750, -0.0017089843750000, 0.0011291503906250, -0.0013122558593750,
+0.0009460449218750, -0.0018615722656250, 0.0010070800781250, -0.0008239746093750,
+-0.0000610351562500, -0.0004272460937500, 0.0000610351562500, -0.0006408691406250,
+-0.0003967285156250, -0.0003662109375000, -0.0011901855468750, 0.0000610351562500,
+-0.0007019042968750, -0.0007019042968750, -0.0013732910156250, -0.0005798339843750,
+-0.0011596679687500, -0.0005187988281250, -0.0008850097656250, -0.0009460449218750,
+-0.0011596679687500, -0.0010375976562500, -0.0010375976562500, -0.0012207031250000,
+-0.0008544921875000, -0.0011291503906250, -0.0006408691406250, -0.0013732910156250,
+-0.0007019042968750, -0.0017395019531250, -0.0007934570312500, -0.0016479492187500,
+-0.0005187988281250, -0.0018310546875000, -0.0008239746093750, -0.0024719238281250,
+-0.0016174316406250, -0.0021667480468750, -0.0017395019531250, -0.0019836425781250,
+-0.0015869140625000, -0.0027465820312500, -0.0019836425781250, -0.0013732910156250,
+-0.0019531250000000, -0.0017395019531250, -0.0024108886718750, -0.0016479492187500,
+-0.0019226074218750, -0.0007019042968750, -0.0016174316406250, -0.0011291503906250,
+-0.0025939941406250, -0.0002136230468750, -0.0015563964843750, -0.0004882812500000,
+-0.0021972656250000, -0.0002136230468750, -0.0023803710937500, 0.0005187988281250,
+-0.0014038085937500, -0.0004272460937500, -0.0018615722656250, 0.0003356933593750,
+-0.0020751953125000, 0.0006103515625000, -0.0016784667968750, -0.0001831054687500,
+-0.0020141601562500, -0.0002441406250000, -0.0026550292968750, 0.0003967285156250,
+-0.0022583007812500, -0.0010070800781250, -0.0025939941406250, -0.0007934570312500,
+-0.0028076171875000, -0.0008239746093750, -0.0027770996093750, -0.0017089843750000,
+-0.0025329589843750, -0.0014343261718750, -0.0025024414062500, -0.0016479492187500,
+-0.0032348632812500, -0.0019531250000000, -0.0026855468750000, -0.0018920898437500,
+-0.0028381347656250, -0.0017395019531250, -0.0040588378906250, -0.0041198730468750,
+-0.0033874511718750, -0.0034484863281250, -0.0030212402343750, -0.0046691894531250,
+-0.0025329589843750, -0.0035705566406250, -0.0031433105468750, -0.0023498535156250,
+-0.0036010742187500, -0.0035095214843750, -0.0027465820312500, -0.0015563964843750,
+-0.0027465820312500, -0.0012512207031250, -0.0031433105468750, -0.0019836425781250,
+-0.0022583007812500, -0.0016174316406250, -0.0014343261718750, -0.0014648437500000,
+-0.0019836425781250, -0.0027465820312500, -0.0010070800781250, -0.0028381347656250,
+-0.0005493164062500, -0.0025329589843750, -0.0008544921875000, -0.0029602050781250,
+-0.0008850097656250, -0.0029602050781250, -0.0005798339843750, -0.0016174316406250,
+-0.0007934570312500, -0.0015563964843750, -0.0010375976562500, -0.0014343261718750,
+-0.0006713867187500, -0.0003051757812500, -0.0003356933593750, -0.0000610351562500,
+-0.0005798339843750, -0.0008544921875000, -0.0001525878906250, -0.0001525878906250,
+-0.0001220703125000, 0.0000610351562500, -0.0003967285156250, -0.0007629394531250,
+-0.0005493164062500, -0.0003051757812500, -0.0007324218750000, 0.0005187988281250,
+-0.0007019042968750, 0.0001525878906250, -0.0011291503906250, 0.0005493164062500,
+-0.0009765625000000, 0.0015563964843750, -0.0005798339843750, 0.0013427734375000,
+-0.0009765625000000, 0.0014343261718750, -0.0004272460937500, 0.0024414062500000,
+-0.0002746582031250, 0.0015258789062500, -0.0002441406250000, 0.0008544921875000,
+0.0000305175781250, 0.0023498535156250, 0.0000305175781250, -0.0004272460937500,
+0.0003662109375000, 0.0000305175781250, 0.0002441406250000, 0.0003967285156250,
+0.0005187988281250, -0.0011901855468750, 0.0007934570312500, -0.0005798339843750,
+0.0003662109375000, -0.0000305175781250, 0.0009765625000000, -0.0008544921875000,
+0.0012207031250000, 0.0000915527343750, 0.0009155273437500, 0.0011901855468750,
+0.0010681152343750, -0.0002136230468750, 0.0014343261718750, 0.0021667480468750,
+0.0008239746093750, 0.0017395019531250, 0.0009155273437500, 0.0014343261718750,
+0.0008544921875000, 0.0025634765625000, 0.0004272460937500, 0.0018310546875000,
+0.0007324218750000, 0.0016174316406250, 0.0002441406250000, 0.0016479492187500,
+0.0003662109375000, 0.0013427734375000, 0.0003356933593750, 0.0008544921875000,
+-0.0000305175781250, 0.0008544921875000, 0.0001525878906250, 0.0007934570312500,
+-0.0000305175781250, 0.0002746582031250, -0.0000305175781250, 0.0004577636718750,
+-0.0001525878906250, 0.0004577636718750, -0.0001831054687500, -0.0003051757812500,
+-0.0000915527343750, 0.0009765625000000, -0.0007324218750000, -0.0000305175781250,
+-0.0004577636718750, 0.0003967285156250, -0.0006713867187500, 0.0015869140625000,
+-0.0012207031250000, 0.0000000000000000, -0.0007324218750000, 0.0018310546875000,
+-0.0011901855468750, 0.0013122558593750, -0.0010986328125000, 0.0011901855468750,
+-0.0010375976562500, 0.0021362304687500, -0.0011901855468750, 0.0011901855468750,
+-0.0008239746093750, 0.0018920898437500, -0.0011901855468750, 0.0016174316406250,
+-0.0009460449218750, 0.0011291503906250, -0.0009765625000000, 0.0013732910156250,
+-0.0011596679687500, 0.0012817382812500, -0.0009460449218750, 0.0005187988281250,
+-0.0012817382812500, 0.0008544921875000, -0.0011596679687500, 0.0005493164062500,
+-0.0013122558593750, -0.0000305175781250, -0.0017089843750000, 0.0007019042968750,
+-0.0014343261718750, -0.0001220703125000, -0.0017395019531250, 0.0003051757812500,
+-0.0018615722656250, 0.0002441406250000, -0.0017395019531250, -0.0000915527343750,
+-0.0017395019531250, 0.0008544921875000, -0.0018310546875000, -0.0003662109375000,
+-0.0016174316406250, 0.0004882812500000, -0.0015563964843750, 0.0007019042968750,
+-0.0016784667968750, -0.0005187988281250, -0.0016479492187500, -0.0001220703125000,
+-0.0015258789062500, -0.0003967285156250, -0.0018005371093750, -0.0003662109375000,
+-0.0016479492187500, -0.0003051757812500, -0.0017089843750000, -0.0003662109375000,
+-0.0018920898437500, -0.0002746582031250, -0.0016784667968750, -0.0005187988281250,
+-0.0017089843750000},
+{-0.0019531250000000, -0.0007019042968750, -0.0014648437500000, -0.0001525878906250,
+-0.0010070800781250, -0.0000610351562500, -0.0013732910156250, -0.0006103515625000,
+-0.0018310546875000, 0.0001220703125000, -0.0016174316406250, 0.0009765625000000,
+-0.0013427734375000, 0.0009155273437500, -0.0017395019531250, 0.0014038085937500,
+-0.0012512207031250, 0.0025939941406250, -0.0005798339843750, 0.0026855468750000,
+-0.0010986328125000, 0.0021362304687500, -0.0007019042968750, 0.0031127929687500,
+-0.0007324218750000, 0.0021972656250000, -0.0010070800781250, 0.0009460449218750,
+-0.0011596679687500, 0.0020751953125000, -0.0014038085937500, -0.0002746582031250,
+-0.0010070800781250, -0.0000610351562500, -0.0013732910156250, 0.0002746582031250,
+-0.0010681152343750, -0.0009155273437500, -0.0003051757812500, -0.0003051757812500,
+-0.0007019042968750, 0.0006103515625000, -0.0001831054687500, 0.0003051757812500,
+0.0002746582031250, 0.0007934570312500, 0.0001831054687500, 0.0016479492187500,
+0.0001525878906250, 0.0011901855468750, 0.0003662109375000, 0.0016479492187500,
+0.0004577636718750, 0.0016174316406250, 0.0002136230468750, 0.0011901855468750,
+0.0003967285156250, 0.0011596679687500, 0.0009155273437500, 0.0010375976562500,
+0.0008544921875000, 0.0003967285156250, 0.0009155273437500, 0.0002441406250000,
+0.0013732910156250, 0.0002441406250000, 0.0010986328125000, -0.0001220703125000,
+0.0006408691406250, -0.0002441406250000, 0.0010070800781250, 0.0005493164062500,
+0.0002746582031250, 0.0001220703125000, 0.0003967285156250, 0.0005798339843750,
+0.0003356933593750, 0.0014038085937500, 0.0000915527343750, 0.0007324218750000,
+0.0004882812500000, 0.0018920898437500, -0.0000915527343750, 0.0015869140625000,
+0.0001525878906250, 0.0016784667968750, -0.0000305175781250, 0.0024414062500000,
+-0.0004882812500000, 0.0016174316406250, -0.0000915527343750, 0.0020446777343750,
+-0.0006713867187500, 0.0019531250000000, -0.0005493164062500, 0.0014648437500000,
+-0.0005493164062500, 0.0013427734375000, -0.0007324218750000, 0.0012512207031250,
+-0.0004577636718750, 0.0005798339843750, -0.0007324218750000, 0.0005798339843750,
+-0.0005798339843750, 0.0003356933593750, -0.0005493164062500, -0.0001525878906250,
+-0.0007324218750000, 0.0000915527343750, -0.0003967285156250, -0.0000915527343750,
+-0.0006713867187500, -0.0001220703125000, -0.0004577636718750, 0.0000000000000000,
+-0.0005493164062500, 0.0001525878906250, -0.0007934570312500, 0.0002136230468750,
+-0.0003662109375000, 0.0003662109375000, -0.0008850097656250, 0.0005798339843750,
+-0.0007934570312500, 0.0009460449218750, -0.0006713867187500, 0.0012817382812500,
+-0.0008850097656250, 0.0014343261718750, -0.0007019042968750, 0.0021972656250000,
+-0.0010070800781250, 0.0023498535156250, -0.0008850097656250, 0.0026855468750000,
+-0.0009155273437500, 0.0033264160156250, -0.0011901855468750, 0.0036010742187500,
+-0.0010070800781250, 0.0039672851562500, -0.0011901855468750, 0.0039367675781250,
+-0.0010681152343750, 0.0042724609375000, -0.0010986328125000, 0.0043945312500000,
+-0.0011901855468750, 0.0038146972656250, -0.0009765625000000, 0.0046386718750000,
+-0.0012512207031250, 0.0044860839843750, -0.0011596679687500, 0.0043640136718750,
+-0.0011596679687500, 0.0047607421875000, -0.0013732910156250, 0.0043945312500000,
+-0.0011596679687500, 0.0045776367187500, -0.0013122558593750, 0.0044860839843750,
+-0.0013122558593750, 0.0042419433593750, -0.0012512207031250, 0.0041503906250000,
+-0.0013427734375000, 0.0040588378906250, -0.0012817382812500, 0.0035705566406250,
+-0.0013122558593750, 0.0036926269531250, -0.0013732910156250, 0.0032653808593750,
+-0.0014648437500000, 0.0028381347656250, -0.0014648437500000, 0.0032043457031250,
+-0.0015869140625000, 0.0022277832031250, -0.0015563964843750, 0.0028991699218750,
+-0.0018005371093750, 0.0024719238281250, -0.0017089843750000, 0.0017395019531250,
+-0.0016479492187500, 0.0030212402343750, -0.0018615722656250, 0.0010070800781250,
+-0.0018615722656250, 0.0018920898437500, -0.0017700195312500, 0.0018005371093750,
+-0.0018615722656250, 0.0003356933593750, -0.0019836425781250, 0.0015258789062500,
+-0.0020141601562500, 0.0002746582031250, -0.0023193359375000, 0.0000610351562500,
+-0.0023193359375000, 0.0004272460937500, -0.0025024414062500, -0.0000915527343750,
+-0.0027465820312500, -0.0003967285156250, -0.0026550292968750, 0.0004577636718750,
+-0.0031127929687500, -0.0002441406250000, -0.0031127929687500, 0.0000915527343750,
+-0.0033569335937500, 0.0009765625000000, -0.0039367675781250, 0.0001525878906250,
+-0.0038146972656250, 0.0013122558593750, -0.0041503906250000, 0.0009460449218750,
+-0.0044555664062500, 0.0010681152343750, -0.0042724609375000, 0.0014953613281250,
+-0.0042724609375000, 0.0003356933593750, -0.0046691894531250, 0.0016174316406250,
+-0.0039978027343750, 0.0009765625000000, -0.0045471191406250, 0.0007934570312500,
+-0.0042419433593750, 0.0015869140625000, -0.0036621093750000, 0.0007324218750000,
+-0.0046386718750000, 0.0010986328125000, -0.0035705566406250, 0.0010070800781250,
+-0.0039672851562500, 0.0008239746093750, -0.0040283203125000, 0.0009460449218750,
+-0.0034179687500000, 0.0008850097656250, -0.0039672851562500, 0.0008850097656250,
+-0.0026245117187500, 0.0009460449218750, -0.0029296875000000, 0.0010986328125000,
+-0.0026855468750000, 0.0011291503906250, -0.0016479492187500, 0.0012207031250000,
+-0.0021667480468750, 0.0015869140625000, -0.0014953613281250, 0.0016479492187500,
+-0.0015563964843750, 0.0014953613281250, -0.0015258789062500, 0.0017700195312500,
+-0.0012207031250000, 0.0019531250000000, -0.0016174316406250, 0.0003967285156250,
+-0.0012817382812500, 0.0012817382812500, -0.0014648437500000, 0.0008239746093750,
+-0.0016479492187500, -0.0006103515625000, -0.0014953613281250, 0.0004577636718750,
+-0.0016174316406250, -0.0008850097656250, -0.0017700195312500, -0.0012207031250000,
+-0.0013427734375000, -0.0012817382812500, -0.0017395019531250, -0.0023193359375000,
+-0.0019836425781250, -0.0028686523437500, -0.0010681152343750, -0.0028381347656250,
+-0.0018615722656250, -0.0038757324218750, -0.0013732910156250, -0.0040893554687500,
+-0.0011901855468750, -0.0038452148437500, -0.0014953613281250, -0.0045166015625000,
+-0.0007019042968750, -0.0037841796875000, -0.0014953613281250, -0.0045776367187500,
+-0.0009765625000000, -0.0039367675781250, -0.0010070800781250, -0.0030517578125000,
+-0.0016784667968750, -0.0046081542968750, -0.0010070800781250, -0.0018005371093750,
+-0.0015258789062500, -0.0028381347656250, -0.0013122558593750, -0.0025329589843750,
+-0.0014648437500000, -0.0005493164062500, -0.0017700195312500, -0.0024719238281250,
+-0.0014343261718750, -0.0009460449218750, -0.0022583007812500, -0.0010070800781250,
+-0.0020446777343750, -0.0016784667968750, -0.0022888183593750, -0.0011901855468750,
+-0.0030517578125000, -0.0013122558593750, -0.0025634765625000, -0.0018310546875000,
+-0.0030212402343750, -0.0014343261718750, -0.0029907226562500, -0.0014038085937500,
+-0.0026855468750000, -0.0015869140625000, -0.0026550292968750, -0.0011596679687500,
+-0.0027770996093750, -0.0016174316406250, -0.0028686523437500, -0.0011291503906250,
+-0.0032043457031250, -0.0010681152343750, -0.0032348632812500, -0.0013427734375000,
+-0.0032958984375000, -0.0007324218750000, -0.0036010742187500, -0.0011901855468750,
+-0.0028076171875000, -0.0009460449218750, -0.0033569335937500, -0.0009765625000000,
+-0.0028991699218750, -0.0013122558593750, -0.0021972656250000, -0.0008239746093750,
+-0.0030517578125000, -0.0008544921875000, -0.0015258789062500, -0.0008239746093750,
+-0.0024108886718750, -0.0003662109375000, -0.0021972656250000, 0.0000305175781250,
+-0.0012512207031250, -0.0000915527343750, -0.0028686523437500, 0.0001831054687500,
+-0.0010070800781250, 0.0007324218750000, -0.0017700195312500, 0.0004882812500000,
+-0.0016784667968750, 0.0002441406250000, -0.0003356933593750, 0.0006408691406250,
+-0.0014343261718750, 0.0001831054687500, -0.0000915527343750, -0.0003356933593750,
+-0.0005187988281250, 0.0000000000000000, -0.0003356933593750, -0.0000305175781250,
+0.0003662109375000, -0.0009460449218750, -0.0008239746093750, 0.0000305175781250,
+0.0010986328125000, -0.0004882812500000, -0.0004577636718750, -0.0008239746093750,
+0.0000305175781250, -0.0002136230468750, 0.0018310546875000, -0.0007629394531250,
+-0.0004882812500000, -0.0012207031250000, 0.0019836425781250, -0.0009765625000000,
+0.0013122558593750, -0.0014953613281250, 0.0007934570312500, -0.0020141601562500,
+0.0020141601562500, -0.0012512207031250, 0.0010070800781250, -0.0018310546875000,
+0.0010986328125000, -0.0015563964843750, 0.0010986328125000, -0.0015258789062500,
+0.0006408691406250, -0.0016784667968750, 0.0004577636718750, -0.0010070800781250,
+0.0005187988281250, -0.0021972656250000, 0.0003967285156250, -0.0015563964843750,
+0.0001220703125000, -0.0018310546875000, 0.0005493164062500, -0.0028686523437500,
+0.0008239746093750, -0.0018005371093750, 0.0004272460937500, -0.0031433105468750,
+0.0013427734375000, -0.0027770996093750, 0.0013732910156250, -0.0028076171875000,
+0.0012817382812500, -0.0035400390625000, 0.0017700195312500, -0.0027465820312500,
+0.0016479492187500, -0.0034179687500000, 0.0014038085937500, -0.0032958984375000,
+0.0016479492187500, -0.0028991699218750, 0.0012512207031250, -0.0030212402343750,
+0.0008850097656250, -0.0028076171875000, 0.0014343261718750, -0.0021667480468750,
+0.0006408691406250, -0.0021362304687500, 0.0010681152343750, -0.0018005371093750,
+0.0011901855468750, -0.0011901855468750, 0.0009155273437500, -0.0011901855468750,
+0.0017395019531250, -0.0009765625000000, 0.0016174316406250, -0.0007629394531250,
+0.0020751953125000, -0.0007324218750000, 0.0024108886718750, -0.0007324218750000,
+0.0024719238281250, -0.0006103515625000, 0.0031127929687500, -0.0004882812500000,
+0.0029907226562500, -0.0003967285156250, 0.0033264160156250, -0.0004272460937500,
+0.0033874511718750, -0.0003662109375000, 0.0032653808593750, -0.0003662109375000,
+0.0036621093750000, -0.0006713867187500, 0.0033569335937500, -0.0006408691406250,
+0.0036010742187500, -0.0007324218750000, 0.0035095214843750, -0.0008850097656250,
+0.0032653808593750, -0.0007019042968750, 0.0037536621093750, -0.0010681152343750,
+0.0035095214843750, -0.0008239746093750, 0.0037231445312500, -0.0010070800781250,
+0.0039978027343750, -0.0013427734375000, 0.0040283203125000, -0.0008850097656250,
+0.0043029785156250, -0.0015563964843750, 0.0044250488281250, -0.0014343261718750,
+0.0044555664062500, -0.0014648437500000, 0.0045776367187500, -0.0018310546875000,
+0.0046691894531250, -0.0015563964843750, 0.0046081542968750, -0.0021362304687500,
+0.0049133300781250, -0.0018615722656250, 0.0046997070312500, -0.0019226074218750,
+0.0048217773437500, -0.0024108886718750, 0.0050354003906250, -0.0018920898437500,
+0.0047302246093750, -0.0023803710937500, 0.0052490234375000, -0.0022888183593750,
+0.0049438476562500, -0.0022583007812500, 0.0051269531250000, -0.0024719238281250,
+0.0054931640625000, -0.0022583007812500, 0.0048217773437500, -0.0024108886718750,
+0.0055847167968750, -0.0024108886718750, 0.0052490234375000, -0.0022583007812500,
+0.0050354003906250, -0.0022888183593750, 0.0054626464843750, -0.0023193359375000,
+0.0051269531250000, -0.0020141601562500, 0.0050354003906250, -0.0021362304687500,
+0.0052490234375000, -0.0020751953125000, 0.0049438476562500, -0.0018920898437500,
+0.0047302246093750, -0.0019531250000000, 0.0049438476562500, -0.0018615722656250,
+0.0040283203125000, -0.0018310546875000, 0.0041809082031250, -0.0017089843750000,
+0.0038757324218750},
+{0.0019836425781250, -0.0007324218750000, 0.0016479492187500, -0.0002746582031250,
+0.0028381347656250, -0.0011596679687500, 0.0012817382812500, -0.0001831054687500,
+0.0019836425781250, -0.0004577636718750, 0.0014648437500000, -0.0008544921875000,
+0.0010681152343750, -0.0005493164062500, 0.0013732910156250, -0.0009460449218750,
+0.0007019042968750, -0.0012817382812500, 0.0014343261718750, -0.0013427734375000,
+0.0010986328125000, -0.0012817382812500, 0.0012817382812500, -0.0012817382812500,
+0.0021362304687500, -0.0012817382812500, 0.0016479492187500, -0.0011596679687500,
+0.0021057128906250, -0.0008544921875000, 0.0023498535156250, -0.0010986328125000,
+0.0021972656250000, -0.0012207031250000, 0.0023193359375000, -0.0008239746093750,
+0.0024719238281250, -0.0019836425781250, 0.0021667480468750, -0.0015563964843750,
+0.0022277832031250, -0.0017700195312500, 0.0023193359375000, -0.0027465820312500,
+0.0021972656250000, -0.0020141601562500, 0.0022888183593750, -0.0028076171875000,
+0.0025024414062500, -0.0027465820312500, 0.0026855468750000, -0.0025024414062500,
+0.0027160644531250, -0.0026855468750000, 0.0030212402343750, -0.0024108886718750,
+0.0032653808593750, -0.0023803710937500, 0.0029907226562500, -0.0022583007812500,
+0.0035400390625000, -0.0020141601562500, 0.0034484863281250, -0.0019531250000000,
+0.0031738281250000, -0.0019531250000000, 0.0039978027343750, -0.0013732910156250,
+0.0028381347656250, -0.0014648437500000, 0.0036315917968750, -0.0012512207031250,
+0.0032958984375000, -0.0007934570312500, 0.0022888183593750, -0.0011291503906250,
+0.0035095214843750, -0.0007019042968750, 0.0019531250000000, -0.0007019042968750,
+0.0023803710937500, -0.0006713867187500, 0.0021972656250000, -0.0003967285156250,
+0.0010681152343750, -0.0004882812500000, 0.0018615722656250, -0.0004577636718750,
+0.0009460449218750, -0.0004882812500000, 0.0009765625000000, -0.0005493164062500,
+0.0012817382812500, -0.0005187988281250, 0.0009765625000000, -0.0006103515625000,
+0.0011291503906250, -0.0004272460937500, 0.0017700195312500, -0.0006408691406250,
+0.0015258789062500, -0.0006103515625000, 0.0018310546875000, -0.0004577636718750,
+0.0023803710937500, -0.0007324218750000, 0.0020141601562500, -0.0007019042968750,
+0.0027465820312500, -0.0006408691406250, 0.0026550292968750, -0.0008544921875000,
+0.0028076171875000, -0.0010070800781250, 0.0032348632812500, -0.0009155273437500,
+0.0029296875000000, -0.0013122558593750, 0.0033874511718750, -0.0012817382812500,
+0.0032043457031250, -0.0012512207031250, 0.0031738281250000, -0.0016174316406250,
+0.0034484863281250, -0.0017089843750000, 0.0030517578125000, -0.0015869140625000,
+0.0033569335937500, -0.0017700195312500, 0.0032043457031250, -0.0018005371093750,
+0.0030822753906250, -0.0017089843750000, 0.0032653808593750, -0.0016784667968750,
+0.0030212402343750, -0.0017700195312500, 0.0032958984375000, -0.0017395019531250,
+0.0030212402343750, -0.0016174316406250, 0.0031127929687500, -0.0015869140625000,
+0.0032958984375000, -0.0016174316406250, 0.0030212402343750, -0.0013732910156250,
+0.0034484863281250, -0.0014343261718750, 0.0032958984375000, -0.0013427734375000,
+0.0035705566406250, -0.0010986328125000, 0.0040283203125000, -0.0011291503906250,
+0.0037536621093750, -0.0005798339843750, 0.0047607421875000, -0.0007019042968750,
+0.0044555664062500, -0.0005187988281250, 0.0046691894531250, -0.0000915527343750,
+0.0053405761718750, -0.0004882812500000, 0.0046997070312500, -0.0003967285156250,
+0.0055847167968750, -0.0003967285156250, 0.0053100585937500, -0.0007324218750000,
+0.0051574707031250, -0.0008239746093750, 0.0055541992187500, -0.0006713867187500,
+0.0050354003906250, -0.0010986328125000, 0.0047607421875000, -0.0010375976562500,
+0.0046997070312500, -0.0009765625000000, 0.0041198730468750, -0.0012207031250000,
+0.0036010742187500, -0.0012512207031250, 0.0036010742187500, -0.0009460449218750,
+0.0027465820312500, -0.0011901855468750, 0.0026855468750000, -0.0010070800781250,
+0.0025939941406250, -0.0006103515625000, 0.0021362304687500, -0.0008850097656250,
+0.0023193359375000, -0.0005187988281250, 0.0024108886718750, -0.0006103515625000,
+0.0023498535156250, -0.0007019042968750, 0.0025939941406250, -0.0006713867187500,
+0.0028381347656250, -0.0009460449218750, 0.0028381347656250, -0.0008544921875000,
+0.0032043457031250, -0.0008850097656250, 0.0032348632812500, -0.0010986328125000,
+0.0030822753906250, -0.0011596679687500, 0.0031433105468750, -0.0010681152343750,
+0.0032043457031250, -0.0013732910156250, 0.0025939941406250, -0.0015869140625000,
+0.0026855468750000, -0.0016174316406250, 0.0023498535156250, -0.0017700195312500,
+0.0016784667968750, -0.0020446777343750, 0.0017395019531250, -0.0020446777343750,
+0.0015563964843750, -0.0019836425781250, 0.0013122558593750, -0.0021057128906250,
+0.0014038085937500, -0.0020751953125000, 0.0015563964843750, -0.0017395019531250,
+0.0014038085937500, -0.0019226074218750, 0.0016479492187500, -0.0018310546875000,
+0.0019531250000000, -0.0014343261718750, 0.0019531250000000, -0.0014648437500000,
+0.0017089843750000, -0.0018920898437500, 0.0016174316406250, -0.0014953613281250,
+0.0016174316406250, -0.0016479492187500, 0.0012817382812500, -0.0021972656250000,
+0.0013122558593750, -0.0022888183593750, 0.0012512207031250, -0.0021057128906250,
+0.0007019042968750, -0.0028686523437500, 0.0010681152343750, -0.0025634765625000,
+0.0009460449218750, -0.0025634765625000, 0.0010986328125000, -0.0031127929687500,
+0.0012512207031250, -0.0027770996093750, 0.0006408691406250, -0.0031127929687500,
+0.0011596679687500, -0.0031127929687500, 0.0010681152343750, -0.0033569335937500,
+-0.0000915527343750, -0.0036621093750000, -0.0005187988281250, -0.0034179687500000,
+-0.0003051757812500, -0.0036315917968750, -0.0031738281250000, -0.0039062500000000,
+-0.0028686523437500, -0.0037536621093750, -0.0034179687500000, -0.0036926269531250,
+-0.0052795410156250, -0.0040893554687500, -0.0043029785156250, -0.0038146972656250,
+-0.0051879882812500, -0.0039062500000000, -0.0049438476562500, -0.0042724609375000,
+-0.0047912597656250, -0.0043334960937500, -0.0055541992187500, -0.0032348632812500,
+-0.0038452148437500, -0.0038452148437500, -0.0058898925781250, -0.0025024414062500,
+-0.0045166015625000, -0.0028381347656250, -0.0050659179687500, -0.0032958984375000,
+-0.0059204101562500, -0.0021057128906250, -0.0043640136718750, -0.0033569335937500,
+-0.0059509277343750, -0.0029296875000000, -0.0055236816406250, -0.0022583007812500,
+-0.0049133300781250, -0.0024719238281250, -0.0054931640625000, -0.0016479492187500,
+-0.0050048828125000, -0.0006713867187500, -0.0040283203125000, 0.0002136230468750,
+-0.0040893554687500, 0.0006713867187500, -0.0036315917968750, 0.0014648437500000,
+-0.0027770996093750, 0.0024414062500000, -0.0029296875000000, 0.0010986328125000,
+-0.0028686523437500, 0.0022277832031250, -0.0027770996093750, 0.0018005371093750,
+-0.0027465820312500, 0.0001220703125000, -0.0028076171875000, 0.0015258789062500,
+-0.0027465820312500, 0.0006103515625000, -0.0023193359375000, -0.0000305175781250,
+-0.0021362304687500, 0.0009765625000000, -0.0021667480468750, 0.0014648437500000,
+-0.0019531250000000, 0.0010070800781250, -0.0015869140625000, 0.0021667480468750,
+-0.0022583007812500, 0.0024108886718750, -0.0018310546875000, 0.0022277832031250,
+-0.0017089843750000, 0.0027770996093750, -0.0022277832031250, 0.0029907226562500,
+-0.0018005371093750, 0.0025329589843750, -0.0014343261718750, 0.0027770996093750,
+-0.0015869140625000, 0.0030212402343750, -0.0013122558593750, 0.0024108886718750,
+-0.0008544921875000, 0.0023803710937500, -0.0012817382812500, 0.0033264160156250,
+-0.0009460449218750, 0.0027160644531250, -0.0010070800781250, 0.0028991699218750,
+-0.0009765625000000, 0.0039062500000000, -0.0008239746093750, 0.0032958984375000,
+-0.0009460449218750, 0.0035705566406250, -0.0002441406250000, 0.0039062500000000,
+-0.0001220703125000, 0.0033569335937500, 0.0000305175781250, 0.0032348632812500,
+0.0007324218750000, 0.0036926269531250, 0.0008544921875000, 0.0019226074218750,
+0.0005798339843750, 0.0024108886718750, 0.0009460449218750, 0.0020751953125000,
+0.0007324218750000, 0.0006103515625000, 0.0004577636718750, 0.0011291503906250,
+0.0010681152343750, 0.0010681152343750, 0.0006713867187500, 0.0003356933593750,
+0.0010986328125000, 0.0008239746093750, 0.0010375976562500, 0.0013122558593750,
+0.0007019042968750, 0.0005187988281250, 0.0013122558593750, 0.0020141601562500,
+0.0000305175781250, 0.0016784667968750, 0.0004272460937500, 0.0018920898437500,
+0.0002746582031250, 0.0028381347656250, -0.0006408691406250, 0.0022277832031250,
+0.0000915527343750, 0.0031738281250000, -0.0003662109375000, 0.0032958984375000,
+-0.0001525878906250, 0.0030212402343750, -0.0000610351562500, 0.0033874511718750,
+-0.0002136230468750, 0.0034179687500000, 0.0000610351562500, 0.0025939941406250,
+-0.0002746582031250, 0.0029602050781250, -0.0001220703125000, 0.0026550292968750,
+-0.0000915527343750, 0.0020141601562500, -0.0002746582031250, 0.0026245117187500,
+-0.0000915527343750, 0.0017395019531250, -0.0001220703125000, 0.0019531250000000,
+-0.0000610351562500, 0.0020446777343750, 0.0000000000000000, 0.0015258789062500,
+-0.0000305175781250, 0.0019531250000000, 0.0001220703125000, 0.0020141601562500,
+0.0000610351562500, 0.0020141601562500, 0.0000915527343750, 0.0023803710937500,
+0.0000915527343750, 0.0026245117187500, 0.0001525878906250, 0.0025329589843750,
+0.0002136230468750, 0.0032958984375000, 0.0000610351562500, 0.0031433105468750,
+0.0002441406250000, 0.0034484863281250, 0.0001525878906250, 0.0039672851562500,
+0.0000305175781250, 0.0034790039062500, 0.0003662109375000, 0.0045471191406250,
+0.0000000000000000, 0.0039367675781250, 0.0001220703125000, 0.0040588378906250,
+0.0000915527343750, 0.0048522949218750, -0.0002136230468750, 0.0040588378906250,
+-0.0001831054687500, 0.0049743652343750, -0.0003662109375000, 0.0045471191406250,
+-0.0004577636718750, 0.0046081542968750, -0.0004882812500000, 0.0050964355468750,
+-0.0006408691406250, 0.0041809082031250, -0.0006408691406250, 0.0050659179687500,
+-0.0007019042968750, 0.0046081542968750, -0.0007324218750000, 0.0044860839843750,
+-0.0008239746093750, 0.0049743652343750, -0.0009460449218750, 0.0042114257812500,
+-0.0007934570312500, 0.0046691894531250, -0.0009765625000000, 0.0043334960937500,
+-0.0008850097656250, 0.0039978027343750, -0.0008544921875000, 0.0041809082031250,
+-0.0009765625000000, 0.0036926269531250, -0.0007629394531250, 0.0035095214843750,
+-0.0009155273437500, 0.0034179687500000, -0.0009155273437500, 0.0030517578125000,
+-0.0009155273437500, 0.0028991699218750, -0.0010375976562500, 0.0028686523437500,
+-0.0011291503906250, 0.0020446777343750, -0.0012512207031250, 0.0025024414062500,
+-0.0014343261718750, 0.0022583007812500, -0.0015563964843750, 0.0017395019531250,
+-0.0017395019531250, 0.0028381347656250, -0.0018920898437500, 0.0016784667968750,
+-0.0020751953125000, 0.0023498535156250, -0.0022277832031250, 0.0025024414062500,
+-0.0024719238281250, 0.0018005371093750, -0.0027465820312500, 0.0026550292968750,
+-0.0030212402343750, 0.0022888183593750, -0.0030212402343750, 0.0021667480468750,
+-0.0032653808593750, 0.0026245117187500, -0.0033569335937500, 0.0025329589843750,
+-0.0033264160156250, 0.0021972656250000, -0.0036621093750000, 0.0034484863281250,
+-0.0032043457031250, 0.0027770996093750, -0.0037536621093750, 0.0030517578125000,
+-0.0036315917968750, 0.0039672851562500, -0.0032348632812500, 0.0028686523437500,
+-0.0039672851562500},
+{0.0008239746093750, 0.0024108886718750, 0.0010070800781250, 0.0033264160156250,
+0.0002746582031250, 0.0032348632812500, 0.0007324218750000, 0.0028686523437500,
+0.0007629394531250, 0.0031127929687500, 0.0002441406250000, 0.0026855468750000,
+0.0008239746093750, 0.0025939941406250, 0.0006408691406250, 0.0024719238281250,
+0.0008239746093750, 0.0022583007812500, 0.0006713867187500, 0.0020751953125000,
+0.0004577636718750, 0.0019531250000000, 0.0008544921875000, 0.0017089843750000,
+-0.0000915527343750, 0.0015563964843750, 0.0001831054687500, 0.0016174316406250,
+0.0001831054687500, 0.0016479492187500, -0.0003356933593750, 0.0013732910156250,
+0.0000915527343750, 0.0021057128906250, -0.0000915527343750, 0.0018005371093750,
+0.0000000000000000, 0.0021972656250000, 0.0000610351562500, 0.0029907226562500,
+-0.0001220703125000, 0.0025024414062500, 0.0001525878906250, 0.0037536621093750,
+-0.0000610351562500, 0.0036315917968750, -0.0000915527343750, 0.0039367675781250,
+0.0000610351562500, 0.0048217773437500, 0.0000305175781250, 0.0043334960937500,
+-0.0000305175781250, 0.0052795410156250, 0.0001220703125000, 0.0051879882812500,
+0.0000915527343750, 0.0050964355468750, 0.0000915527343750, 0.0054321289062500,
+0.0001831054687500, 0.0051574707031250, 0.0001831054687500, 0.0049743652343750,
+0.0002136230468750, 0.0049438476562500, 0.0003967285156250, 0.0046691894531250,
+0.0003662109375000, 0.0043029785156250, 0.0003662109375000, 0.0041198730468750,
+0.0005493164062500, 0.0037841796875000, 0.0003051757812500, 0.0036010742187500,
+0.0003662109375000, 0.0034179687500000, 0.0003967285156250, 0.0031127929687500,
+0.0001831054687500, 0.0029602050781250, 0.0001220703125000, 0.0029296875000000,
+0.0000305175781250, 0.0025329589843750, -0.0000305175781250, 0.0025329589843750,
+-0.0001220703125000, 0.0026245117187500, -0.0002746582031250, 0.0021057128906250,
+-0.0002441406250000, 0.0027160644531250, -0.0003967285156250, 0.0022277832031250,
+-0.0003967285156250, 0.0024108886718750, -0.0004882812500000, 0.0029602050781250,
+-0.0004882812500000, 0.0021972656250000, -0.0004577636718750, 0.0033264160156250,
+-0.0005493164062500, 0.0029296875000000, -0.0004272460937500, 0.0030822753906250,
+-0.0005187988281250, 0.0038146972656250, -0.0006713867187500, 0.0032348632812500,
+-0.0005187988281250, 0.0041198730468750, -0.0007324218750000, 0.0038146972656250,
+-0.0007934570312500, 0.0038452148437500, -0.0008239746093750, 0.0044250488281250,
+-0.0009460449218750, 0.0038757324218750, -0.0010070800781250, 0.0040893554687500,
+-0.0011291503906250, 0.0041809082031250, -0.0012512207031250, 0.0039062500000000,
+-0.0014648437500000, 0.0039978027343750, -0.0017089843750000, 0.0043334960937500,
+-0.0019531250000000, 0.0035095214843750, -0.0020446777343750, 0.0043334960937500,
+-0.0024414062500000, 0.0040588378906250, -0.0024719238281250, 0.0034484863281250,
+-0.0024719238281250, 0.0046081542968750, -0.0028991699218750, 0.0030517578125000,
+-0.0025329589843750, 0.0035705566406250, -0.0027770996093750, 0.0036926269531250,
+-0.0027770996093750, 0.0025634765625000, -0.0025939941406250, 0.0032043457031250,
+-0.0030212402343750, 0.0028991699218750, -0.0023803710937500, 0.0022888183593750,
+-0.0027465820312500, 0.0027770996093750, -0.0025634765625000, 0.0029907226562500,
+-0.0019836425781250, 0.0022888183593750, -0.0025024414062500, 0.0036621093750000,
+-0.0017700195312500, 0.0028686523437500, -0.0019226074218750, 0.0030822753906250,
+-0.0019531250000000, 0.0040283203125000, -0.0016174316406250, 0.0026550292968750,
+-0.0020751953125000, 0.0038757324218750, -0.0016479492187500, 0.0032043457031250,
+-0.0020446777343750, 0.0028076171875000, -0.0018615722656250, 0.0032348632812500,
+-0.0015869140625000, 0.0021972656250000, -0.0024108886718750, 0.0025939941406250,
+-0.0011291503906250, 0.0020751953125000, -0.0019226074218750, 0.0017395019531250,
+-0.0018920898437500, 0.0019836425781250, -0.0011596679687500, 0.0014038085937500,
+-0.0025634765625000, 0.0015563964843750, -0.0014648437500000, 0.0015869140625000,
+-0.0023498535156250, 0.0014953613281250, -0.0024414062500000, 0.0017089843750000,
+-0.0017395019531250, 0.0019226074218750, -0.0029907226562500, 0.0022583007812500,
+-0.0017395019531250, 0.0028381347656250, -0.0019226074218750, 0.0023498535156250,
+-0.0022888183593750, 0.0021972656250000, -0.0016479492187500, 0.0033264160156250,
+-0.0016784667968750, 0.0009765625000000, -0.0018615722656250, 0.0024414062500000,
+-0.0014953613281250, 0.0021667480468750, -0.0014038085937500, 0.0004577636718750,
+-0.0016784667968750, 0.0026245117187500, -0.0015563964843750, 0.0007934570312500,
+-0.0012817382812500, 0.0016784667968750, -0.0014953613281250, 0.0019531250000000,
+-0.0012817382812500, 0.0005187988281250, -0.0008239746093750, 0.0012512207031250,
+-0.0010070800781250, -0.0001525878906250, -0.0011291503906250, 0.0007324218750000,
+-0.0004882812500000, 0.0000915527343750, -0.0009155273437500, -0.0013122558593750,
+-0.0014343261718750, 0.0002441406250000, -0.0007019042968750, -0.0014648437500000,
+-0.0019836425781250, -0.0011596679687500, -0.0016784667968750, -0.0006103515625000,
+-0.0015258789062500, -0.0010375976562500, -0.0024108886718750, -0.0003356933593750,
+-0.0023193359375000, -0.0004272460937500, -0.0021057128906250, -0.0002746582031250,
+-0.0025634765625000, 0.0002746582031250, -0.0024719238281250, -0.0007934570312500,
+-0.0018310546875000, -0.0032348632812500, -0.0020751953125000, -0.0029602050781250,
+-0.0024108886718750, -0.0029907226562500, -0.0020446777343750, -0.0031127929687500,
+-0.0022888183593750, -0.0028076171875000, -0.0026550292968750, -0.0031127929687500,
+-0.0018310546875000, -0.0028686523437500, -0.0022888183593750, -0.0028076171875000,
+-0.0021972656250000, -0.0028076171875000, -0.0013427734375000, -0.0027465820312500,
+-0.0010986328125000, -0.0026855468750000, -0.0011901855468750, -0.0018310546875000,
+-0.0000915527343750, -0.0021362304687500, -0.0003967285156250, -0.0015869140625000,
+-0.0002746582031250, -0.0003356933593750, 0.0003356933593750, -0.0006713867187500,
+-0.0000610351562500, 0.0000305175781250, 0.0008850097656250, 0.0007934570312500,
+0.0006713867187500, 0.0003356933593750, 0.0011901855468750, 0.0000000000000000,
+0.0020446777343750, 0.0005798339843750, 0.0013427734375000, -0.0005798339843750,
+0.0027465820312500, -0.0003967285156250, 0.0024719238281250, -0.0004272460937500,
+0.0023803710937500, -0.0009765625000000, 0.0030517578125000, -0.0006103515625000,
+0.0024414062500000, -0.0011291503906250, 0.0023803710937500, -0.0007629394531250,
+0.0022277832031250, -0.0009460449218750, 0.0019836425781250, -0.0014648437500000,
+0.0019226074218750, -0.0007934570312500, 0.0018615722656250, -0.0013732910156250,
+0.0015563964843750, -0.0012207031250000, 0.0021667480468750, -0.0008544921875000,
+0.0019836425781250, -0.0010375976562500, 0.0017395019531250, -0.0010375976562500,
+0.0027160644531250, -0.0004577636718750, 0.0017089843750000, -0.0006713867187500,
+0.0022583007812500, -0.0003967285156250, 0.0023193359375000, 0.0001220703125000,
+0.0016479492187500, -0.0002441406250000, 0.0023803710937500, 0.0003967285156250,
+0.0020446777343750, 0.0001525878906250, 0.0019836425781250, 0.0001525878906250,
+0.0024414062500000, 0.0006103515625000, 0.0026245117187500, 0.0001831054687500,
+0.0025634765625000, 0.0001831054687500, 0.0029602050781250, 0.0003662109375000,
+0.0033874511718750, 0.0001525878906250, 0.0030212402343750, 0.0000305175781250,
+0.0029907226562500, 0.0003051757812500, 0.0036621093750000, 0.0000000000000000,
+0.0020751953125000, 0.0001525878906250, 0.0028381347656250, 0.0001525878906250,
+0.0025329589843750, 0.0000915527343750, 0.0012817382812500, 0.0003356933593750,
+0.0024719238281250, -0.0003356933593750, 0.0016479492187500, -0.0000915527343750,
+0.0016479492187500, -0.0004577636718750, 0.0021362304687500, -0.0012207031250000,
+0.0020141601562500, -0.0008239746093750, 0.0020751953125000, -0.0016174316406250,
+0.0029296875000000, -0.0018005371093750, 0.0027160644531250, -0.0017089843750000,
+0.0029296875000000, -0.0020446777343750, 0.0035400390625000, -0.0022888183593750,
+0.0032653808593750, -0.0019531250000000, 0.0036621093750000, -0.0023193359375000,
+0.0037231445312500, -0.0021972656250000, 0.0037536621093750, -0.0018920898437500,
+0.0039672851562500, -0.0023193359375000, 0.0038757324218750, -0.0014648437500000,
+0.0040283203125000, -0.0019226074218750, 0.0039978027343750, -0.0015563964843750,
+0.0038146972656250, -0.0007934570312500, 0.0037231445312500, -0.0016479492187500,
+0.0036010742187500, -0.0004272460937500, 0.0035400390625000, -0.0006408691406250,
+0.0035705566406250, -0.0007019042968750, 0.0034179687500000, -0.0001525878906250,
+0.0032653808593750, -0.0006713867187500, 0.0033264160156250, -0.0001831054687500,
+0.0029296875000000, -0.0004272460937500, 0.0028686523437500, -0.0005187988281250,
+0.0027465820312500, -0.0002441406250000, 0.0024108886718750, -0.0005493164062500,
+0.0025329589843750, -0.0002441406250000, 0.0020446777343750, -0.0004577636718750,
+0.0020141601562500, -0.0003662109375000, 0.0017700195312500, -0.0001525878906250,
+0.0014953613281250, -0.0005493164062500, 0.0016479492187500, -0.0001831054687500,
+0.0010986328125000, -0.0004272460937500, 0.0013427734375000, -0.0004577636718750,
+0.0012817382812500, -0.0002136230468750, 0.0008239746093750, -0.0005187988281250,
+0.0013427734375000, -0.0002441406250000, 0.0007934570312500, -0.0004272460937500,
+0.0010375976562500, -0.0005187988281250, 0.0010986328125000, -0.0003967285156250,
+0.0008850097656250, -0.0006408691406250, 0.0014343261718750, -0.0005798339843750,
+0.0012512207031250, -0.0007324218750000, 0.0014953613281250, -0.0006408691406250,
+0.0017089843750000, -0.0004577636718750, 0.0017089843750000, -0.0006713867187500,
+0.0020141601562500, -0.0003356933593750, 0.0022583007812500, -0.0002441406250000,
+0.0023498535156250, -0.0003356933593750, 0.0027465820312500, -0.0002441406250000,
+0.0031127929687500, -0.0002746582031250, 0.0029602050781250, -0.0004272460937500,
+0.0035095214843750, -0.0004577636718750, 0.0035095214843750, -0.0005187988281250,
+0.0035400390625000, -0.0005798339843750, 0.0038452148437500, -0.0004577636718750,
+0.0038757324218750, -0.0004577636718750, 0.0035095214843750, -0.0004272460937500,
+0.0040283203125000, -0.0001831054687500, 0.0035400390625000, -0.0001220703125000,
+0.0028991699218750, -0.0001831054687500, 0.0036621093750000, 0.0001831054687500,
+0.0021667480468750, 0.0002441406250000, 0.0025024414062500, 0.0002136230468750,
+0.0023193359375000, 0.0003662109375000, 0.0011291503906250, 0.0004577636718750,
+0.0016784667968750, 0.0000610351562500, 0.0012817382812500, 0.0000610351562500,
+0.0009765625000000, 0.0000305175781250, 0.0014953613281250, -0.0003662109375000,
+0.0016479492187500, -0.0004882812500000, 0.0012817382812500, -0.0003662109375000,
+0.0023498535156250, -0.0006713867187500, 0.0021667480468750, -0.0009155273437500,
+0.0023498535156250, -0.0009765625000000, 0.0031127929687500, -0.0012817382812500,
+0.0027465820312500, -0.0015563964843750, 0.0029907226562500, -0.0021362304687500,
+0.0031127929687500, -0.0022888183593750, 0.0026550292968750, -0.0023803710937500,
+0.0023193359375000, -0.0028991699218750, 0.0024108886718750, -0.0025329589843750,
+0.0017395019531250, -0.0025329589843750, 0.0012512207031250, -0.0025634765625000,
+0.0014343261718750, -0.0022888183593750, 0.0013122558593750, -0.0020446777343750,
+0.0007324218750000, -0.0020751953125000, 0.0018920898437500, -0.0019531250000000,
+0.0013732910156250},
+{0.0025329589843750, -0.0019226074218750, 0.0022583007812500, -0.0016479492187500,
+0.0024719238281250, -0.0017089843750000, 0.0026855468750000, -0.0019531250000000,
+0.0021972656250000, -0.0012817382812500, 0.0029602050781250, -0.0017700195312500,
+0.0025634765625000, -0.0015258789062500, 0.0024108886718750, -0.0009460449218750,
+0.0028991699218750, -0.0018615722656250, 0.0024108886718750, -0.0007324218750000,
+0.0024414062500000, -0.0010375976562500, 0.0025634765625000, -0.0010375976562500,
+0.0023803710937500, -0.0003356933593750, 0.0022888183593750, -0.0009155273437500,
+0.0024414062500000, -0.0003051757812500, 0.0022277832031250, -0.0004882812500000,
+0.0022277832031250, -0.0005187988281250, 0.0022888183593750, -0.0001220703125000,
+0.0021362304687500, -0.0004577636718750, 0.0021362304687500, -0.0001525878906250,
+0.0021972656250000, -0.0001831054687500, 0.0022888183593750, -0.0002136230468750,
+0.0022888183593750, -0.0000915527343750, 0.0023803710937500, -0.0001831054687500,
+0.0025939941406250, -0.0000305175781250, 0.0025939941406250, -0.0001220703125000,
+0.0028686523437500, -0.0001220703125000, 0.0028686523437500, -0.0000610351562500,
+0.0028381347656250, -0.0002136230468750, 0.0032653808593750, -0.0001220703125000,
+0.0029296875000000, -0.0003051757812500, 0.0033874511718750, -0.0002136230468750,
+0.0033264160156250, -0.0000915527343750, 0.0030517578125000, -0.0003356933593750,
+0.0036621093750000, 0.0001831054687500, 0.0027770996093750, -0.0000610351562500,
+0.0033569335937500, -0.0000915527343750, 0.0031738281250000, 0.0002136230468750,
+0.0025329589843750, -0.0001525878906250, 0.0033874511718750, -0.0002136230468750,
+0.0022583007812500, -0.0002441406250000, 0.0027465820312500, -0.0005187988281250,
+0.0027160644531250, -0.0006103515625000, 0.0019226074218750, -0.0005798339843750,
+0.0027465820312500, -0.0008850097656250, 0.0019531250000000, -0.0007629394531250,
+0.0021667480468750, -0.0006103515625000, 0.0023803710937500, -0.0007019042968750,
+0.0020751953125000, -0.0004272460937500, 0.0024414062500000, -0.0001525878906250,
+0.0026855468750000, -0.0001220703125000, 0.0027465820312500, 0.0001220703125000,
+0.0031738281250000, 0.0003967285156250, 0.0036926269531250, 0.0003967285156250,
+0.0036621093750000, 0.0003967285156250, 0.0042724609375000, 0.0005187988281250,
+0.0044555664062500, 0.0003967285156250, 0.0044860839843750, 0.0002136230468750,
+0.0048828125000000, 0.0001831054687500, 0.0049438476562500, 0.0000305175781250,
+0.0044555664062500, -0.0000610351562500, 0.0047607421875000, -0.0003051757812500,
+0.0042419433593750, -0.0004272460937500, 0.0035095214843750, -0.0003967285156250,
+0.0040283203125000, -0.0013732910156250, 0.0027160644531250, -0.0013732910156250,
+0.0026550292968750, -0.0017395019531250, 0.0025939941406250, -0.0026550292968750,
+0.0018310546875000, -0.0025329589843750, 0.0020446777343750, -0.0028686523437500,
+0.0019226074218750, -0.0030517578125000, 0.0017395019531250, -0.0030517578125000,
+0.0020141601562500, -0.0028686523437500, 0.0022277832031250, -0.0025634765625000,
+0.0019531250000000, -0.0028076171875000, 0.0024108886718750, -0.0021667480468750,
+0.0025634765625000, -0.0020751953125000, 0.0023498535156250, -0.0024414062500000,
+0.0023803710937500, -0.0019531250000000, 0.0025329589843750, -0.0020446777343750,
+0.0018005371093750, -0.0020751953125000, 0.0017395019531250, -0.0022888183593750,
+0.0016174316406250, -0.0021667480468750, 0.0012207031250000, -0.0017395019531250,
+0.0014648437500000, -0.0030822753906250, 0.0012817382812500, -0.0020751953125000,
+0.0010986328125000, -0.0023193359375000, 0.0011291503906250, -0.0035095214843750,
+0.0011596679687500, -0.0019836425781250, 0.0012207031250000, -0.0030822753906250,
+0.0013122558593750, -0.0027465820312500, 0.0014343261718750, -0.0022888183593750,
+0.0018920898437500, -0.0024719238281250, 0.0019531250000000, -0.0018005371093750,
+0.0013122558593750, -0.0024719238281250, 0.0021972656250000, -0.0020446777343750,
+0.0014648437500000, -0.0019531250000000, 0.0015258789062500, -0.0025329589843750,
+0.0020751953125000, -0.0021667480468750, 0.0009155273437500, -0.0024719238281250,
+0.0024108886718750, -0.0027160644531250, 0.0017700195312500, -0.0028076171875000,
+0.0018615722656250, -0.0029296875000000, 0.0033264160156250, -0.0028686523437500,
+0.0025939941406250, -0.0029602050781250, 0.0025939941406250, -0.0024108886718750,
+0.0032348632812500, -0.0025634765625000, 0.0018615722656250, -0.0026550292968750,
+0.0005798339843750, -0.0015869140625000, 0.0013122558593750, -0.0028076171875000,
+-0.0015563964843750, -0.0019531250000000, -0.0018310546875000, -0.0015869140625000,
+-0.0016174316406250, -0.0029296875000000, -0.0033264160156250, -0.0002746582031250,
+0.0014343261718750, 0.0003051757812500, -0.0005187988281250, 0.0004272460937500,
+-0.0006408691406250, 0.0009460449218750, -0.0011596679687500, 0.0013732910156250,
+-0.0019226074218750, 0.0013427734375000, -0.0021972656250000, 0.0024414062500000,
+-0.0025329589843750, 0.0023498535156250, -0.0025329589843750, 0.0025024414062500,
+-0.0031127929687500, 0.0034179687500000, -0.0037536621093750, 0.0030822753906250,
+-0.0034790039062500, 0.0025939941406250, -0.0042114257812500, 0.0030822753906250,
+-0.0046386718750000, 0.0024108886718750, -0.0039367675781250, 0.0013427734375000,
+-0.0035705566406250, 0.0019226074218750, -0.0039672851562500, 0.0013427734375000,
+-0.0027770996093750, 0.0008239746093750, -0.0022888183593750, 0.0016479492187500,
+-0.0028076171875000, 0.0020446777343750, -0.0025024414062500, 0.0013427734375000,
+-0.0016784667968750, 0.0031127929687500, -0.0032653808593750, 0.0031738281250000,
+-0.0025634765625000, 0.0026245117187500, -0.0026550292968750, 0.0032653808593750,
+-0.0038452148437500, 0.0032348632812500, -0.0028686523437500, 0.0019531250000000,
+-0.0037536621093750, 0.0023498535156250, -0.0034790039062500, 0.0018615722656250,
+-0.0034484863281250, 0.0007324218750000, -0.0039367675781250, 0.0013732910156250,
+-0.0032348632812500, 0.0010070800781250, -0.0036621093750000, 0.0008544921875000,
+-0.0035705566406250, 0.0014038085937500, -0.0031738281250000, 0.0018005371093750,
+-0.0030517578125000, 0.0016784667968750, -0.0029296875000000, 0.0018920898437500,
+-0.0025939941406250, 0.0025939941406250, -0.0021972656250000, 0.0021057128906250,
+-0.0020751953125000, 0.0016784667968750, -0.0018615722656250, 0.0024719238281250,
+-0.0014648437500000, 0.0012817382812500, -0.0017395019531250, 0.0015869140625000,
+-0.0017089843750000, 0.0014953613281250, -0.0014648437500000, 0.0007019042968750,
+-0.0015258789062500, 0.0013732910156250, -0.0016479492187500, 0.0008239746093750,
+-0.0011291503906250, 0.0007324218750000, -0.0013427734375000, 0.0012817382812500,
+-0.0012512207031250, 0.0014953613281250, -0.0008850097656250, 0.0015563964843750,
+-0.0012512207031250, 0.0022583007812500, -0.0004577636718750, 0.0026245117187500,
+-0.0007019042968750, 0.0026245117187500, -0.0003662109375000, 0.0029602050781250,
+0.0003967285156250, 0.0033874511718750, -0.0001525878906250, 0.0025634765625000,
+0.0005187988281250, 0.0031433105468750, 0.0007324218750000, 0.0027160644531250,
+0.0006408691406250, 0.0018310546875000, 0.0007629394531250, 0.0028076171875000,
+0.0008850097656250, 0.0010070800781250, 0.0010986328125000, 0.0014648437500000,
+0.0010070800781250, 0.0013427734375000, 0.0011596679687500, 0.0000915527343750,
+0.0014648437500000, 0.0010070800781250, 0.0011901855468750, 0.0004272460937500,
+0.0010986328125000, 0.0003967285156250, 0.0011596679687500, 0.0007629394531250,
+0.0008850097656250, 0.0007324218750000, 0.0006103515625000, 0.0008239746093750,
+0.0007934570312500, 0.0014343261718750, 0.0006713867187500, 0.0011901855468750,
+0.0006103515625000, 0.0016784667968750, 0.0006103515625000, 0.0023193359375000,
+0.0006713867187500, 0.0017700195312500, 0.0007324218750000, 0.0029907226562500,
+0.0005187988281250, 0.0025024414062500, 0.0006103515625000, 0.0026855468750000,
+0.0005798339843750, 0.0034179687500000, 0.0003356933593750, 0.0024719238281250,
+0.0004577636718750, 0.0034179687500000, 0.0003356933593750, 0.0030212402343750,
+0.0003662109375000, 0.0029296875000000, 0.0003967285156250, 0.0033569335937500,
+0.0003356933593750, 0.0026855468750000, 0.0003967285156250, 0.0032653808593750,
+0.0003051757812500, 0.0028076171875000, 0.0002746582031250, 0.0026855468750000,
+0.0003051757812500, 0.0030517578125000, 0.0003356933593750, 0.0024108886718750,
+0.0002746582031250, 0.0028076171875000, 0.0004272460937500, 0.0026855468750000,
+0.0003356933593750, 0.0025024414062500, 0.0003967285156250, 0.0027160644531250,
+0.0004577636718750, 0.0026550292968750, 0.0003051757812500, 0.0026550292968750,
+0.0003967285156250, 0.0026245117187500, 0.0002441406250000, 0.0026550292968750,
+0.0001831054687500, 0.0026550292968750, 0.0001831054687500, 0.0025024414062500,
+-0.0001220703125000, 0.0028076171875000, -0.0001220703125000, 0.0024414062500000,
+-0.0002441406250000, 0.0026550292968750, -0.0005493164062500, 0.0030517578125000,
+-0.0007019042968750, 0.0024719238281250, -0.0008239746093750, 0.0033874511718750,
+-0.0011596679687500, 0.0032043457031250, -0.0012817382812500, 0.0032653808593750,
+-0.0014343261718750, 0.0039062500000000, -0.0016479492187500, 0.0035705566406250,
+-0.0016479492187500, 0.0038452148437500, -0.0019226074218750, 0.0039978027343750,
+-0.0019836425781250, 0.0039672851562500, -0.0019531250000000, 0.0040893554687500,
+-0.0021057128906250, 0.0041809082031250, -0.0019836425781250, 0.0039672851562500,
+-0.0019836425781250, 0.0042419433593750, -0.0020751953125000, 0.0040283203125000,
+-0.0017700195312500, 0.0038757324218750, -0.0016174316406250, 0.0044860839843750,
+-0.0019226074218750, 0.0037231445312500, -0.0011596679687500, 0.0045471191406250,
+-0.0014343261718750, 0.0043640136718750, -0.0012817382812500, 0.0038757324218750,
+-0.0007324218750000, 0.0053710937500000, -0.0010681152343750, 0.0038757324218750,
+-0.0004882812500000, 0.0046081542968750, -0.0006408691406250, 0.0048522949218750,
+-0.0004882812500000, 0.0039367675781250, -0.0000915527343750, 0.0048217773437500,
+-0.0005187988281250, 0.0045471191406250, 0.0002136230468750, 0.0048828125000000,
+-0.0000915527343750, 0.0052490234375000, 0.0001220703125000, 0.0052185058593750,
+0.0005493164062500, 0.0055236816406250, -0.0003051757812500, 0.0054321289062500,
+0.0003967285156250, 0.0055541992187500, -0.0004272460937500, 0.0054016113281250,
+-0.0007019042968750, 0.0049743652343750, -0.0003967285156250, 0.0049438476562500,
+-0.0015563964843750, 0.0043029785156250, -0.0008239746093750, 0.0040588378906250,
+-0.0014038085937500, 0.0037536621093750, -0.0016174316406250, 0.0032653808593750,
+-0.0010681152343750, 0.0031738281250000, -0.0016174316406250, 0.0030517578125000,
+-0.0013122558593750, 0.0028991699218750, -0.0012207031250000, 0.0028381347656250,
+-0.0012817382812500, 0.0030212402343750, -0.0012817382812500, 0.0030517578125000,
+-0.0012817382812500, 0.0024414062500000, -0.0010375976562500, 0.0031433105468750,
+-0.0013122558593750, 0.0025329589843750, -0.0012512207031250, 0.0018615722656250,
+-0.0011291503906250, 0.0033264160156250, -0.0016174316406250, 0.0014343261718750,
+-0.0009765625000000, 0.0024414062500000, -0.0014343261718750, 0.0021972656250000,
+-0.0013427734375000, 0.0008239746093750, -0.0007019042968750, 0.0024414062500000,
+-0.0010375976562500, 0.0007019042968750, -0.0007629394531250, 0.0017395019531250,
+-0.0002441406250000, 0.0019531250000000, -0.0006103515625000, 0.0007934570312500,
+-0.0008850097656250},
+{0.0006103515625000, -0.0003967285156250, 0.0005493164062500, -0.0003967285156250,
+0.0005187988281250, -0.0003967285156250, 0.0005798339843750, -0.0005493164062500,
+0.0005493164062500, -0.0005187988281250, 0.0005493164062500, -0.0002746582031250,
+0.0005493164062500, -0.0000915527343750, 0.0004577636718750, -0.0001525878906250,
+0.0005187988281250, 0.0000915527343750, 0.0003967285156250, 0.0002441406250000,
+0.0004272460937500, 0.0002136230468750, 0.0004577636718750, 0.0003967285156250,
+0.0002136230468750, 0.0005798339843750, 0.0004272460937500, 0.0006103515625000,
+0.0002746582031250, 0.0008850097656250, 0.0001525878906250, 0.0010681152343750,
+0.0001220703125000, 0.0010070800781250, -0.0000610351562500, 0.0015869140625000,
+-0.0003662109375000, 0.0012817382812500, -0.0005187988281250, 0.0016479492187500,
+-0.0007324218750000, 0.0020751953125000, -0.0009765625000000, 0.0014343261718750,
+-0.0011291503906250, 0.0026550292968750, -0.0013732910156250, 0.0020141601562500,
+-0.0014343261718750, 0.0023498535156250, -0.0016174316406250, 0.0033569335937500,
+-0.0018310546875000, 0.0023498535156250, -0.0017395019531250, 0.0038452148437500,
+-0.0018615722656250, 0.0035095214843750, -0.0018005371093750, 0.0037841796875000,
+-0.0016784667968750, 0.0048217773437500, -0.0016784667968750, 0.0040893554687500,
+-0.0015563964843750, 0.0052795410156250, -0.0013122558593750, 0.0051879882812500,
+-0.0013427734375000, 0.0050964355468750, -0.0011596679687500, 0.0057678222656250,
+-0.0008850097656250, 0.0056152343750000, -0.0010375976562500, 0.0057067871093750,
+-0.0005187988281250, 0.0061035156250000, -0.0006408691406250, 0.0058593750000000,
+-0.0003967285156250, 0.0058593750000000, -0.0000610351562500, 0.0066223144531250,
+-0.0004577636718750, 0.0055236816406250, 0.0003662109375000, 0.0065307617187500,
+0.0000305175781250, 0.0062561035156250, 0.0002136230468750, 0.0050964355468750,
+0.0008544921875000, 0.0063476562500000, 0.0001831054687500, 0.0048217773437500,
+0.0009155273437500, 0.0051879882812500, 0.0004272460937500, 0.0053710937500000,
+0.0002441406250000, 0.0044555664062500, 0.0004577636718750, 0.0052490234375000,
+-0.0004272460937500, 0.0050354003906250, -0.0000305175781250, 0.0050048828125000,
+-0.0003967285156250, 0.0054016113281250, -0.0007934570312500, 0.0055236816406250,
+-0.0004882812500000, 0.0054626464843750, -0.0007324218750000, 0.0056762695312500,
+-0.0010681152343750, 0.0056457519531250, -0.0006713867187500, 0.0054626464843750,
+-0.0009155273437500, 0.0054016113281250, -0.0011901855468750, 0.0054016113281250,
+-0.0004882812500000, 0.0046997070312500, -0.0013122558593750, 0.0047607421875000,
+-0.0008850097656250, 0.0045166015625000, -0.0007934570312500, 0.0039978027343750,
+-0.0013732910156250, 0.0043334960937500, -0.0008544921875000, 0.0037231445312500,
+-0.0013427734375000, 0.0039062500000000, -0.0013732910156250, 0.0036315917968750,
+-0.0013122558593750, 0.0031738281250000, -0.0016174316406250, 0.0038757324218750,
+-0.0016174316406250, 0.0036010742187500, -0.0012512207031250, 0.0039062500000000,
+-0.0012817382812500, 0.0035705566406250, -0.0010986328125000, 0.0031127929687500,
+-0.0005798339843750, 0.0035095214843750, -0.0005493164062500, 0.0020141601562500,
+-0.0009460449218750, 0.0023193359375000, -0.0005798339843750, 0.0019836425781250,
+-0.0005493164062500, 0.0009155273437500, -0.0010070800781250, 0.0017700195312500,
+-0.0007934570312500, 0.0011901855468750, -0.0008239746093750, 0.0008544921875000,
+-0.0014648437500000, 0.0011291503906250, -0.0012817382812500, 0.0012207031250000,
+-0.0010681152343750, 0.0008850097656250, -0.0018615722656250, 0.0007324218750000,
+-0.0013427734375000, 0.0003051757812500, -0.0018310546875000, -0.0000915527343750,
+-0.0017700195312500, -0.0005798339843750, -0.0014343261718750, -0.0011291503906250,
+-0.0022277832031250, -0.0007934570312500, -0.0011901855468750, -0.0014953613281250,
+-0.0016479492187500, -0.0017395019531250, -0.0016784667968750, -0.0004577636718750,
+-0.0008850097656250, 0.0006103515625000, -0.0009460449218750, -0.0006713867187500,
+0.0005493164062500, 0.0000305175781250, -0.0002746582031250, -0.0002136230468750,
+0.0001525878906250, -0.0013122558593750, 0.0012817382812500, -0.0003356933593750,
+-0.0000610351562500, -0.0018615722656250, 0.0017089843750000, -0.0013427734375000,
+0.0014953613281250, -0.0015563964843750, 0.0013122558593750, -0.0024108886718750,
+0.0022888183593750, -0.0012207031250000, 0.0019531250000000, -0.0025329589843750,
+0.0021362304687500, -0.0023193359375000, 0.0022583007812500, -0.0020141601562500,
+0.0024414062500000, -0.0027770996093750, 0.0026855468750000, -0.0027160644531250,
+0.0023803710937500, -0.0019836425781250, 0.0027465820312500, -0.0025024414062500,
+0.0029602050781250, -0.0022277832031250, 0.0025939941406250, -0.0013732910156250,
+0.0025939941406250, -0.0020141601562500, 0.0029907226562500, -0.0015258789062500,
+0.0018005371093750, -0.0013122558593750, 0.0025634765625000, -0.0016784667968750,
+0.0021057128906250, -0.0016479492187500, 0.0010986328125000, -0.0013732910156250,
+0.0025024414062500, -0.0018310546875000, 0.0013122558593750, -0.0018615722656250,
+0.0018005371093750, -0.0018310546875000, 0.0020751953125000, -0.0023193359375000,
+0.0015869140625000, -0.0023803710937500, 0.0022583007812500, -0.0016784667968750,
+0.0014038085937500, -0.0022277832031250, 0.0016174316406250, -0.0016784667968750,
+0.0014343261718750, -0.0007629394531250, 0.0006713867187500, -0.0017089843750000,
+0.0010681152343750, -0.0001831054687500, 0.0004272460937500, -0.0003967285156250,
+0.0004272460937500, -0.0004882812500000, 0.0005798339843750, 0.0003662109375000,
+0.0005187988281250, -0.0000915527343750, 0.0009460449218750, 0.0000000000000000,
+0.0009460449218750, 0.0000610351562500, 0.0015563964843750, -0.0000610351562500,
+0.0016174316406250, -0.0003051757812500, 0.0016784667968750, -0.0003967285156250,
+0.0025024414062500, -0.0001220703125000, 0.0016174316406250, -0.0005493164062500,
+0.0021362304687500, -0.0002441406250000, 0.0023803710937500, 0.0001525878906250,
+0.0017089843750000, -0.0005493164062500, 0.0022888183593750, 0.0002441406250000,
+0.0024414062500000, 0.0000305175781250, 0.0022583007812500, -0.0002441406250000,
+0.0026245117187500, 0.0000610351562500, 0.0030517578125000, -0.0001831054687500,
+0.0027770996093750, -0.0002746582031250, 0.0031433105468750, -0.0002441406250000,
+0.0033874511718750, -0.0002136230468750, 0.0032348632812500, -0.0000610351562500,
+0.0032958984375000, -0.0000610351562500, 0.0034484863281250, -0.0001831054687500,
+0.0028381347656250, 0.0002136230468750, 0.0029602050781250, 0.0000000000000000,
+0.0027160644531250, -0.0003051757812500, 0.0021362304687500, 0.0001525878906250,
+0.0022888183593750, -0.0007324218750000, 0.0018005371093750, -0.0004882812500000,
+0.0015563964843750, -0.0005493164062500, 0.0014953613281250, -0.0011596679687500,
+0.0012512207031250, -0.0007324218750000, 0.0008850097656250, -0.0011596679687500,
+0.0010375976562500, -0.0012817382812500, 0.0007324218750000, -0.0010375976562500,
+0.0006713867187500, -0.0011901855468750, 0.0009155273437500, -0.0013427734375000,
+0.0005798339843750, -0.0007934570312500, 0.0009460449218750, -0.0012207031250000,
+0.0007324218750000, -0.0010375976562500, 0.0007934570312500, -0.0004882812500000,
+0.0011901855468750, -0.0010681152343750, 0.0009155273437500, -0.0003356933593750,
+0.0012817382812500, -0.0005798339843750, 0.0013427734375000, -0.0004882812500000,
+0.0012512207031250, 0.0000000000000000, 0.0013732910156250, -0.0005187988281250,
+0.0014953613281250, 0.0002441406250000, 0.0012817382812500, -0.0001525878906250,
+0.0014648437500000, -0.0000305175781250, 0.0015258789062500, 0.0004577636718750,
+0.0014343261718750, -0.0001525878906250, 0.0016784667968750, 0.0005187988281250,
+0.0015258789062500, 0.0003051757812500, 0.0016479492187500, 0.0002746582031250,
+0.0017089843750000, 0.0007324218750000, 0.0016479492187500, 0.0003967285156250,
+0.0018005371093750, 0.0007324218750000, 0.0018920898437500, 0.0006713867187500,
+0.0020446777343750, 0.0006103515625000, 0.0020141601562500, 0.0007629394531250,
+0.0019836425781250, 0.0005187988281250, 0.0022277832031250, 0.0005493164062500,
+0.0018310546875000, 0.0003967285156250, 0.0021362304687500, 0.0002441406250000,
+0.0020446777343750, 0.0001831054687500, 0.0017700195312500, 0.0001220703125000,
+0.0022888183593750, 0.0000610351562500, 0.0017395019531250, 0.0000915527343750,
+0.0020751953125000, 0.0001220703125000, 0.0021362304687500, 0.0000915527343750,
+0.0018005371093750, 0.0001525878906250, 0.0023193359375000, 0.0002136230468750,
+0.0021057128906250, 0.0001831054687500, 0.0021972656250000, 0.0000915527343750,
+0.0025024414062500, 0.0000610351562500, 0.0025329589843750, 0.0001220703125000,
+0.0025939941406250, -0.0002136230468750, 0.0032653808593750, -0.0001831054687500,
+0.0029907226562500, -0.0002136230468750, 0.0033569335937500, -0.0003967285156250,
+0.0039367675781250, -0.0002746582031250, 0.0034484863281250, -0.0005493164062500,
+0.0040588378906250, -0.0004272460937500, 0.0039672851562500, -0.0007629394531250,
+0.0037231445312500, -0.0011901855468750, 0.0040588378906250, -0.0008544921875000,
+0.0039367675781250, -0.0018615722656250, 0.0035400390625000, -0.0017395019531250,
+0.0036621093750000, -0.0019531250000000, 0.0034179687500000, -0.0025024414062500,
+0.0029907226562500, -0.0020141601562500, 0.0032958984375000, -0.0027465820312500,
+0.0030822753906250, -0.0021057128906250, 0.0029907226562500, -0.0023193359375000,
+0.0033264160156250, -0.0027465820312500, 0.0035095214843750, -0.0014648437500000,
+0.0036010742187500, -0.0027770996093750, 0.0043029785156250, -0.0021667480468750,
+0.0043945312500000, -0.0018005371093750, 0.0046691894531250, -0.0025329589843750,
+0.0053405761718750, -0.0019226074218750, 0.0052490234375000, -0.0022583007812500,
+0.0054931640625000, -0.0021667480468750, 0.0058288574218750, -0.0025024414062500,
+0.0053710937500000, -0.0028686523437500, 0.0051269531250000, -0.0022583007812500,
+0.0056762695312500, -0.0034179687500000, 0.0043945312500000, -0.0030212402343750,
+0.0046081542968750, -0.0027160644531250, 0.0045471191406250, -0.0031433105468750,
+0.0038146972656250, -0.0025329589843750, 0.0044250488281250, -0.0025329589843750,
+0.0040283203125000, -0.0021667480468750, 0.0041809082031250, -0.0021972656250000,
+0.0044860839843750, -0.0022277832031250, 0.0043029785156250, -0.0016784667968750,
+0.0043334960937500, -0.0020446777343750, 0.0044250488281250, -0.0021057128906250,
+0.0045471191406250, -0.0013122558593750, 0.0038146972656250, -0.0012512207031250,
+0.0034484863281250, -0.0023193359375000, 0.0039367675781250, -0.0011901855468750,
+0.0023803710937500, -0.0018615722656250, 0.0027770996093750, -0.0022277832031250,
+0.0024414062500000, -0.0017700195312500, 0.0008850097656250, -0.0025634765625000,
+0.0013732910156250, -0.0020446777343750, 0.0014648437500000, -0.0022888183593750,
+0.0005493164062500, -0.0020751953125000, 0.0012207031250000, -0.0015258789062500,
+0.0019531250000000, -0.0018920898437500, 0.0006713867187500, -0.0006713867187500,
+0.0020751953125000, -0.0009765625000000, 0.0017089843750000, -0.0005493164062500,
+0.0015563964843750, 0.0004272460937500, 0.0020141601562500, -0.0003051757812500,
+0.0013732910156250, 0.0010070800781250, 0.0026855468750000, 0.0007324218750000,
+0.0016479492187500, 0.0009765625000000, 0.0018615722656250, 0.0026855468750000,
+0.0043029785156250, 0.0052795410156250, 0.0030212402343750, 0.0062561035156250,
+0.0036315917968750},
+{0.0014343261718750, 0.0003662109375000, 0.0018615722656250, 0.0004882812500000,
+0.0016479492187500, 0.0004577636718750, 0.0016174316406250, 0.0003356933593750,
+0.0018615722656250, 0.0004272460937500, 0.0015258789062500, 0.0006103515625000,
+0.0018005371093750, 0.0005187988281250, 0.0017700195312500, 0.0006408691406250,
+0.0016174316406250, 0.0007629394531250, 0.0016479492187500, 0.0005798339843750,
+0.0015869140625000, 0.0005798339843750, 0.0013427734375000, 0.0003356933593750,
+0.0014038085937500, 0.0002136230468750, 0.0010375976562500, 0.0001525878906250,
+0.0007019042968750, -0.0001220703125000, 0.0009460449218750, -0.0000610351562500,
+0.0003051757812500, -0.0002441406250000, 0.0002441406250000, -0.0006103515625000,
+0.0002441406250000, -0.0007629394531250, -0.0001831054687500, -0.0008239746093750,
+-0.0002441406250000, -0.0014648437500000, -0.0002746582031250, -0.0014648437500000,
+-0.0004882812500000, -0.0017700195312500, -0.0003051757812500, -0.0022277832031250,
+-0.0001220703125000, -0.0019226074218750, -0.0003662109375000, -0.0025634765625000,
+0.0003356933593750, -0.0022583007812500, 0.0001525878906250, -0.0022888183593750,
+0.0003967285156250, -0.0025329589843750, 0.0010681152343750, -0.0017089843750000,
+0.0006408691406250, -0.0023803710937500, 0.0011291503906250, -0.0017089843750000,
+0.0010986328125000, -0.0015563964843750, 0.0010375976562500, -0.0019531250000000,
+0.0012207031250000, -0.0011901855468750, 0.0010375976562500, -0.0017395019531250,
+0.0013427734375000, -0.0015869140625000, 0.0009460449218750, -0.0017395019531250,
+0.0013122558593750, -0.0021057128906250, 0.0016174316406250, -0.0018005371093750,
+0.0009765625000000, -0.0028381347656250, 0.0026550292968750, -0.0023498535156250,
+0.0016479492187500, -0.0024414062500000, 0.0025634765625000, -0.0031738281250000,
+0.0043640136718750, -0.0022277832031250, 0.0028076171875000, -0.0024108886718750,
+0.0052185058593750, -0.0023498535156250, 0.0049133300781250, -0.0018615722656250,
+0.0046691894531250, -0.0017395019531250, 0.0057067871093750, -0.0016174316406250,
+0.0047607421875000, -0.0014038085937500, 0.0051269531250000, -0.0014038085937500,
+0.0050659179687500, -0.0010375976562500, 0.0047302246093750, -0.0009155273437500,
+0.0047302246093750, -0.0013732910156250, 0.0045471191406250, -0.0001525878906250,
+0.0043640136718750, -0.0010681152343750, 0.0041809082031250, -0.0006408691406250,
+0.0037536621093750, 0.0003356933593750, 0.0035705566406250, -0.0013122558593750,
+0.0039062500000000, -0.0003051757812500, 0.0037231445312500, -0.0007019042968750,
+0.0043640136718750, -0.0007629394531250, 0.0043945312500000, 0.0000000000000000,
+0.0040893554687500, -0.0005187988281250, 0.0046081542968750, 0.0006103515625000,
+0.0035400390625000, 0.0006103515625000, 0.0033569335937500, 0.0010070800781250,
+0.0032958984375000, 0.0018005371093750, 0.0023193359375000, 0.0014038085937500,
+0.0019226074218750, 0.0025939941406250, 0.0031738281250000, 0.0020141601562500,
+0.0016479492187500, 0.0026855468750000, 0.0025634765625000, 0.0036315917968750,
+0.0044250488281250, 0.0023803710937500, 0.0023193359375000, 0.0049133300781250,
+0.0043029785156250, 0.0042419433593750, 0.0041503906250000, 0.0042724609375000,
+0.0032348632812500, 0.0071716308593750, 0.0038452148437500, 0.0068664550781250,
+0.0049133300781250, 0.0050354003906250, 0.0064086914062500, 0.0054626464843750,
+0.0060119628906250, 0.0052185058593750, 0.0052795410156250, 0.0043945312500000,
+0.0049133300781250, 0.0043334960937500, 0.0047607421875000, 0.0041809082031250,
+0.0043640136718750, 0.0039978027343750, 0.0039978027343750, 0.0034484863281250,
+0.0041198730468750, 0.0028991699218750, 0.0042114257812500, 0.0029296875000000,
+0.0036926269531250, 0.0021362304687500, 0.0037841796875000, 0.0013732910156250,
+0.0036010742187500, 0.0019226074218750, 0.0031738281250000, 0.0018920898437500,
+0.0026855468750000, 0.0007324218750000, 0.0022583007812500, 0.0030212402343750,
+0.0023498535156250, 0.0021057128906250, 0.0016784667968750, 0.0020141601562500,
+0.0014038085937500, 0.0035095214843750, 0.0015869140625000, 0.0022277832031250,
+0.0011291503906250, 0.0021972656250000, 0.0006408691406250, 0.0020751953125000,
+0.0007629394531250, 0.0018005371093750, 0.0001831054687500, 0.0013732910156250,
+-0.0005493164062500, 0.0008850097656250, -0.0002746582031250, 0.0019531250000000,
+-0.0009460449218750, 0.0011596679687500, -0.0010986328125000, 0.0016174316406250,
+-0.0009460449218750, 0.0027770996093750, -0.0010375976562500, 0.0016479492187500,
+-0.0008850097656250, 0.0033569335937500, -0.0010986328125000, 0.0032653808593750,
+-0.0010070800781250, 0.0029296875000000, -0.0012817382812500, 0.0036926269531250,
+-0.0016479492187500, 0.0033569335937500, -0.0017089843750000, 0.0024719238281250,
+-0.0019531250000000, 0.0025329589843750, -0.0022277832031250, 0.0022888183593750,
+-0.0022277832031250, 0.0014953613281250, -0.0021362304687500, 0.0015563964843750,
+-0.0022583007812500, 0.0018615722656250, -0.0026550292968750, 0.0011901855468750,
+-0.0023498535156250, 0.0017395019531250, -0.0027160644531250, 0.0023498535156250,
+-0.0032653808593750, 0.0013122558593750, -0.0026245117187500, 0.0028686523437500,
+-0.0031433105468750, 0.0024108886718750, -0.0032348632812500, 0.0022583007812500,
+-0.0027160644531250, 0.0031738281250000, -0.0026855468750000, 0.0024719238281250,
+-0.0029296875000000, 0.0025329589843750, -0.0020751953125000, 0.0027160644531250,
+-0.0023193359375000, 0.0021972656250000, -0.0020751953125000, 0.0018920898437500,
+-0.0014648437500000, 0.0023803710937500, -0.0020751953125000, 0.0013122558593750,
+-0.0009155273437500, 0.0017700195312500, -0.0013732910156250, 0.0016784667968750,
+-0.0010681152343750, 0.0010070800781250, -0.0001220703125000, 0.0019226074218750,
+-0.0010070800781250, 0.0012207031250000, 0.0001220703125000, 0.0015869140625000,
+-0.0000915527343750, 0.0019531250000000, -0.0001525878906250, 0.0018005371093750,
+0.0004577636718750, 0.0022277832031250, 0.0000305175781250, 0.0021362304687500,
+0.0004882812500000, 0.0023803710937500, 0.0003967285156250, 0.0024414062500000,
+0.0004882812500000, 0.0022888183593750, 0.0008239746093750, 0.0025939941406250,
+0.0005798339843750, 0.0023193359375000, 0.0011596679687500, 0.0025024414062500,
+0.0009155273437500, 0.0025329589843750, 0.0009765625000000, 0.0023193359375000,
+0.0013122558593750, 0.0025939941406250, 0.0008544921875000, 0.0023803710937500,
+0.0011291503906250, 0.0024108886718750, 0.0010375976562500, 0.0021972656250000,
+0.0008544921875000, 0.0019531250000000, 0.0008544921875000, 0.0021362304687500,
+0.0007019042968750, 0.0013427734375000, 0.0007324218750000, 0.0015258789062500,
+0.0005493164062500, 0.0013427734375000, 0.0006713867187500, 0.0006713867187500,
+0.0007324218750000, 0.0009155273437500, 0.0004882812500000, 0.0002441406250000,
+0.0007934570312500, 0.0001831054687500, 0.0006103515625000, 0.0000305175781250,
+0.0005187988281250, -0.0004577636718750, 0.0006408691406250, -0.0003356933593750,
+0.0003356933593750, -0.0007019042968750, 0.0004577636718750, -0.0010070800781250,
+0.0002441406250000, -0.0010681152343750, 0.0001220703125000, -0.0012512207031250,
+0.0001525878906250, -0.0014953613281250, -0.0001525878906250, -0.0014648437500000,
+-0.0000915527343750, -0.0018005371093750, -0.0002441406250000, -0.0018310546875000,
+-0.0003356933593750, -0.0017700195312500, -0.0003356933593750, -0.0022888183593750,
+-0.0004882812500000, -0.0017395019531250, -0.0004882812500000, -0.0020751953125000,
+-0.0004882812500000, -0.0020446777343750, -0.0005798339843750, -0.0014953613281250,
+-0.0005798339843750, -0.0018920898437500, -0.0005187988281250, -0.0012512207031250,
+-0.0006408691406250, -0.0013122558593750, -0.0005493164062500, -0.0011291503906250,
+-0.0005493164062500, -0.0007019042968750, -0.0006103515625000, -0.0009460449218750,
+-0.0006103515625000, -0.0001831054687500, -0.0003051757812500, -0.0003967285156250,
+-0.0004577636718750, -0.0002136230468750, -0.0003967285156250, 0.0003967285156250,
+-0.0000610351562500, 0.0000305175781250, -0.0002746582031250, 0.0006713867187500,
+-0.0000915527343750, 0.0010681152343750, 0.0001220703125000, 0.0009460449218750,
+0.0001220703125000, 0.0011596679687500, 0.0000915527343750, 0.0016174316406250,
+0.0002746582031250, 0.0010375976562500, 0.0001831054687500, 0.0016174316406250,
+0.0001831054687500, 0.0014038085937500, 0.0002136230468750, 0.0009460449218750,
+0.0001525878906250, 0.0018005371093750, 0.0000610351562500, 0.0004577636718750,
+0.0000000000000000, 0.0014343261718750, -0.0003356933593750, 0.0010986328125000,
+-0.0005798339843750, 0.0002746582031250, -0.0005493164062500, 0.0021362304687500,
+-0.0008850097656250, 0.0000610351562500, -0.0010986328125000, 0.0010375976562500,
+-0.0007629394531250, 0.0009765625000000, -0.0010070800781250, -0.0003356933593750,
+-0.0011291503906250, 0.0012817382812500, -0.0002746582031250, -0.0002746582031250,
+-0.0011901855468750, 0.0003356933593750, -0.0002441406250000, 0.0004882812500000,
+-0.0003356933593750, -0.0004577636718750, -0.0009155273437500, 0.0004882812500000,
+0.0005187988281250, -0.0003051757812500, -0.0010375976562500, -0.0002136230468750,
+-0.0001525878906250, -0.0000305175781250, -0.0001525878906250, -0.0005187988281250,
+-0.0013122558593750, -0.0006408691406250, -0.0001220703125000, -0.0002136230468750,
+-0.0011596679687500, -0.0013122558593750, -0.0010681152343750, -0.0007934570312500,
+-0.0007629394531250, -0.0003356933593750, -0.0011291503906250, -0.0020141601562500,
+-0.0009460449218750, 0.0006713867187500, -0.0006408691406250, -0.0008850097656250,
+-0.0005798339843750, -0.0005187988281250, -0.0003662109375000, 0.0015563964843750,
+-0.0001525878906250, -0.0006408691406250, -0.0003051757812500, 0.0017700195312500,
+-0.0002746582031250, 0.0011596679687500, -0.0004272460937500, 0.0013427734375000,
+-0.0005187988281250, 0.0027770996093750, -0.0006713867187500, 0.0015563964843750,
+-0.0010681152343750, 0.0034790039062500, -0.0011291503906250, 0.0030822753906250,
+-0.0016174316406250, 0.0032348632812500, -0.0016784667968750, 0.0045776367187500,
+-0.0016174316406250, 0.0037841796875000, -0.0022277832031250, 0.0043640136718750,
+-0.0017395019531250, 0.0043029785156250, -0.0019531250000000, 0.0038146972656250,
+-0.0019226074218750, 0.0039978027343750, -0.0014953613281250, 0.0042114257812500,
+-0.0018615722656250, 0.0034484863281250, -0.0013732910156250, 0.0037536621093750,
+-0.0013122558593750, 0.0037231445312500, -0.0011291503906250, 0.0032348632812500,
+-0.0008544921875000, 0.0035705566406250, -0.0007934570312500, 0.0030822753906250,
+-0.0000305175781250, 0.0031127929687500, 0.0000305175781250, 0.0027160644531250,
+0.0003662109375000, 0.0020141601562500, 0.0009765625000000, 0.0018920898437500,
+0.0008544921875000, 0.0008850097656250, 0.0017700195312500, 0.0011596679687500,
+0.0015869140625000, 0.0010681152343750, 0.0014648437500000, 0.0002441406250000,
+0.0019836425781250, 0.0004272460937500, 0.0019531250000000, 0.0003967285156250,
+0.0021667480468750, 0.0000305175781250, 0.0025329589843750, -0.0001220703125000,
+0.0026550292968750, -0.0000610351562500, 0.0028076171875000, -0.0003051757812500,
+0.0031127929687500, -0.0008239746093750, 0.0023498535156250, -0.0005798339843750,
+0.0028686523437500, -0.0008239746093750, 0.0025329589843750, -0.0012817382812500,
+0.0017395019531250, -0.0007934570312500, 0.0025024414062500, -0.0011596679687500,
+0.0012512207031250, -0.0011901855468750, 0.0017700195312500, -0.0011596679687500,
+0.0016174316406250},
+{0.0001220703125000, 0.0010070800781250, 0.0001525878906250, 0.0012512207031250,
+0.0000000000000000, 0.0011901855468750, -0.0000610351562500, 0.0015563964843750,
+-0.0000610351562500, 0.0017089843750000, -0.0001525878906250, 0.0018615722656250,
+-0.0003662109375000, 0.0021362304687500, -0.0004882812500000, 0.0021667480468750,
+-0.0004882812500000, 0.0023498535156250, -0.0010070800781250, 0.0026550292968750,
+-0.0006713867187500, 0.0025024414062500, -0.0009460449218750, 0.0027770996093750,
+-0.0012817382812500, 0.0035095214843750, -0.0004577636718750, 0.0025024414062500,
+-0.0012512207031250, 0.0035400390625000, -0.0005798339843750, 0.0031127929687500,
+-0.0006408691406250, 0.0022277832031250, -0.0010070800781250, 0.0038452148437500,
+0.0000915527343750, 0.0016174316406250, -0.0011291503906250, 0.0025024414062500,
+-0.0002441406250000, 0.0022583007812500, -0.0002746582031250, 0.0007324218750000,
+-0.0013427734375000, 0.0022888183593750, -0.0000610351562500, 0.0007324218750000,
+-0.0011291503906250, 0.0010681152343750, -0.0009765625000000, 0.0013122558593750,
+-0.0006713867187500, 0.0004882812500000, -0.0011291503906250, 0.0009460449218750,
+-0.0009460449218750, 0.0006408691406250, -0.0007934570312500, 0.0003662109375000,
+-0.0009765625000000, 0.0002136230468750, -0.0008239746093750, -0.0001220703125000,
+-0.0004882812500000, -0.0004577636718750, -0.0006408691406250, -0.0006713867187500,
+-0.0002441406250000, -0.0012512207031250, -0.0002136230468750, -0.0010681152343750,
+-0.0001220703125000, -0.0010375976562500, 0.0001220703125000, -0.0019531250000000,
+-0.0001220703125000, -0.0010681152343750, -0.0003051757812500, -0.0015869140625000,
+-0.0005493164062500, -0.0012512207031250, -0.0006713867187500, -0.0002441406250000,
+-0.0009765625000000, -0.0007934570312500, -0.0014038085937500, 0.0007934570312500,
+-0.0008239746093750, 0.0005798339843750, -0.0015869140625000, 0.0008850097656250,
+-0.0014953613281250, 0.0020141601562500, -0.0007934570312500, 0.0014038085937500,
+-0.0015258789062500, 0.0023193359375000, -0.0006713867187500, 0.0019226074218750,
+-0.0007324218750000, 0.0021362304687500, -0.0007629394531250, 0.0028076171875000,
+-0.0002746582031250, 0.0019836425781250, -0.0003051757812500, 0.0032348632812500,
+0.0001525878906250, 0.0029907226562500, 0.0000610351562500, 0.0030212402343750,
+0.0004882812500000, 0.0039367675781250, 0.0009460449218750, 0.0037231445312500,
+0.0006103515625000, 0.0037841796875000, 0.0013122558593750, 0.0040893554687500,
+0.0012207031250000, 0.0039672851562500, 0.0013122558593750, 0.0037231445312500,
+0.0018615722656250, 0.0039672851562500, 0.0016174316406250, 0.0038757324218750,
+0.0019531250000000, 0.0037841796875000, 0.0021057128906250, 0.0033569335937500,
+0.0020446777343750, 0.0028991699218750, 0.0024108886718750, 0.0028076171875000,
+0.0028686523437500, 0.0019531250000000, 0.0022583007812500, 0.0014648437500000,
+0.0027465820312500, 0.0013732910156250, 0.0023803710937500, 0.0009460449218750,
+0.0017700195312500, 0.0002441406250000, 0.0025329589843750, 0.0006103515625000,
+0.0010986328125000, 0.0003051757812500, 0.0017089843750000, -0.0002136230468750,
+0.0014343261718750, -0.0000915527343750, 0.0003967285156250, -0.0000610351562500,
+0.0015258789062500, -0.0008850097656250, 0.0002136230468750, -0.0005187988281250,
+0.0006103515625000, -0.0006103515625000, 0.0007934570312500, -0.0010986328125000,
+0.0000305175781250, -0.0006713867187500, 0.0006713867187500, -0.0009155273437500,
+0.0001831054687500, -0.0008239746093750, 0.0001220703125000, -0.0007019042968750,
+0.0004577636718750, -0.0008544921875000, 0.0004882812500000, -0.0006408691406250,
+0.0003662109375000, -0.0005187988281250, 0.0006713867187500, -0.0007324218750000,
+0.0007019042968750, -0.0002746582031250, 0.0003662109375000, 0.0003051757812500,
+0.0003356933593750, 0.0000610351562500, 0.0006408691406250, 0.0005798339843750,
+0.0001525878906250, 0.0009765625000000, 0.0002441406250000, 0.0006103515625000,
+0.0006103515625000, 0.0005798339843750, 0.0006408691406250, 0.0009460449218750,
+0.0007019042968750, -0.0002441406250000, 0.0013122558593750, -0.0000610351562500,
+0.0011901855468750, -0.0003356933593750, 0.0014953613281250, -0.0011596679687500,
+0.0019531250000000, -0.0007324218750000, 0.0014953613281250, -0.0012512207031250,
+0.0019226074218750, -0.0013122558593750, 0.0017089843750000, -0.0013732910156250,
+0.0014953613281250, -0.0016479492187500, 0.0015563964843750, -0.0015258789062500,
+0.0010681152343750, -0.0017395019531250, 0.0013732910156250, -0.0017395019531250,
+0.0009155273437500, -0.0015869140625000, 0.0010070800781250, -0.0015563964843750,
+0.0013122558593750, -0.0014953613281250, 0.0008239746093750, -0.0011291503906250,
+0.0015869140625000, -0.0011596679687500, 0.0014343261718750, -0.0009155273437500,
+0.0014038085937500, -0.0005493164062500, 0.0018005371093750, -0.0007019042968750,
+0.0014038085937500, -0.0002441406250000, 0.0017089843750000, -0.0004272460937500,
+0.0014953613281250, -0.0003051757812500, 0.0014953613281250, -0.0001220703125000,
+0.0015869140625000, -0.0004882812500000, 0.0010375976562500, -0.0001220703125000,
+0.0015869140625000, -0.0003356933593750, 0.0009765625000000, -0.0003967285156250,
+0.0010375976562500, -0.0000915527343750, 0.0014343261718750, -0.0003662109375000,
+0.0005493164062500, -0.0001831054687500, 0.0015258789062500, -0.0003051757812500,
+0.0010681152343750, -0.0003051757812500, 0.0010375976562500, -0.0001525878906250,
+0.0017395019531250, -0.0002746582031250, 0.0011901855468750, -0.0001220703125000,
+0.0019531250000000, -0.0000305175781250, 0.0018615722656250, -0.0002746582031250,
+0.0019836425781250, -0.0002746582031250, 0.0025024414062500, -0.0000610351562500,
+0.0021667480468750, -0.0007324218750000, 0.0026245117187500, -0.0002746582031250,
+0.0024719238281250, -0.0003356933593750, 0.0023193359375000, -0.0010070800781250,
+0.0022583007812500, -0.0004272460937500, 0.0019226074218750, -0.0007629394531250,
+0.0016479492187500, -0.0008544921875000, 0.0014648437500000, -0.0005798339843750,
+0.0011901855468750, -0.0005798339843750, 0.0008544921875000, -0.0006713867187500,
+0.0007324218750000, -0.0001831054687500, 0.0002746582031250, -0.0004577636718750,
+0.0000305175781250, -0.0002441406250000, -0.0001831054687500, 0.0001831054687500,
+-0.0004577636718750, -0.0003662109375000, -0.0005187988281250, 0.0003051757812500,
+-0.0006713867187500, 0.0001525878906250, -0.0005493164062500, 0.0001220703125000,
+-0.0006408691406250, 0.0004882812500000, -0.0006408691406250, 0.0001831054687500,
+-0.0002441406250000, 0.0004577636718750, -0.0006103515625000, 0.0003967285156250,
+-0.0001525878906250, 0.0005187988281250, -0.0000305175781250, 0.0006713867187500,
+-0.0002136230468750, 0.0004577636718750, 0.0005187988281250, 0.0008544921875000,
+0.0002441406250000, 0.0007019042968750, 0.0007019042968750, 0.0006408691406250,
+0.0009460449218750, 0.0007934570312500, 0.0008544921875000, 0.0005493164062500,
+0.0013427734375000, 0.0007629394531250, 0.0010986328125000, 0.0005187988281250,
+0.0013732910156250, 0.0004272460937500, 0.0014038085937500, 0.0006408691406250,
+0.0011901855468750, 0.0003662109375000, 0.0014648437500000, 0.0002441406250000,
+0.0008850097656250, 0.0001525878906250, 0.0011901855468750, -0.0001831054687500,
+0.0009460449218750, -0.0003967285156250, 0.0004577636718750, -0.0004882812500000,
+0.0010986328125000, -0.0010681152343750, 0.0002136230468750, -0.0010070800781250,
+0.0003967285156250, -0.0013427734375000, 0.0005187988281250, -0.0017395019531250,
+0.0001831054687500, -0.0013122558593750, 0.0006713867187500, -0.0020751953125000,
+0.0007629394531250, -0.0017089843750000, 0.0008850097656250, -0.0015869140625000,
+0.0014343261718750, -0.0019226074218750, 0.0018005371093750, -0.0013122558593750,
+0.0018310546875000, -0.0014343261718750, 0.0025634765625000, -0.0011901855468750,
+0.0027160644531250, -0.0008850097656250, 0.0028686523437500, -0.0009155273437500,
+0.0033874511718750, -0.0008239746093750, 0.0034484863281250, -0.0006408691406250,
+0.0033569335937500, -0.0008850097656250, 0.0036010742187500, -0.0009765625000000,
+0.0034484863281250, -0.0007934570312500, 0.0032348632812500, -0.0010681152343750,
+0.0036010742187500, -0.0012512207031250, 0.0031433105468750, -0.0011291503906250,
+0.0032958984375000, -0.0010375976562500, 0.0034179687500000, -0.0012207031250000,
+0.0032043457031250, -0.0010375976562500, 0.0034179687500000, -0.0006408691406250,
+0.0036926269531250, -0.0007324218750000, 0.0037841796875000, -0.0005187988281250,
+0.0039062500000000, -0.0001525878906250, 0.0042724609375000, -0.0004882812500000,
+0.0043945312500000, 0.0003051757812500, 0.0037536621093750, -0.0001220703125000,
+0.0042724609375000, 0.0002441406250000, 0.0038452148437500, 0.0008239746093750,
+0.0030517578125000, -0.0002441406250000, 0.0037841796875000, 0.0013427734375000,
+0.0024414062500000, 0.0002746582031250, 0.0026245117187500, 0.0002746582031250,
+0.0024414062500000, 0.0013122558593750, 0.0017395019531250, -0.0002136230468750,
+0.0023803710937500, 0.0014953613281250, 0.0018005371093750, 0.0010986328125000,
+0.0019226074218750, 0.0014953613281250, 0.0021057128906250, 0.0029602050781250,
+0.0018310546875000, 0.0022277832031250, 0.0020751953125000, 0.0039978027343750,
+0.0020446777343750, 0.0037841796875000, 0.0017089843750000, 0.0045471191406250,
+0.0015258789062500, 0.0057373046875000, 0.0015563964843750, 0.0047607421875000,
+0.0013732910156250, 0.0071411132812500, 0.0010986328125000, 0.0062866210937500,
+0.0008544921875000, 0.0065917968750000, 0.0013732910156250, 0.0081787109375000,
+0.0013732910156250, 0.0066223144531250, 0.0007019042968750, 0.0079956054687500,
+0.0031738281250000, 0.0079040527343750, 0.0023193359375000, 0.0073242187500000,
+0.0028381347656250, 0.0077209472656250, 0.0048828125000000, 0.0075683593750000,
+0.0036315917968750, 0.0067138671875000, 0.0047302246093750, 0.0070495605468750,
+0.0047912597656250, 0.0068054199218750, 0.0046997070312500, 0.0054931640625000,
+0.0055541992187500, 0.0042724609375000, 0.0034179687500000, 0.0025939941406250,
+0.0050964355468750, 0.0022888183593750, 0.0043945312500000, 0.0025634765625000,
+0.0049438476562500, 0.0022277832031250, 0.0054626464843750, 0.0015563964843750,
+0.0045471191406250, 0.0028076171875000, 0.0066223144531250, 0.0024108886718750,
+0.0061035156250000, 0.0021057128906250, 0.0061950683593750, 0.0027465820312500,
+0.0078430175781250, 0.0021972656250000, 0.0071411132812500, 0.0016784667968750,
+0.0070190429687500, 0.0013732910156250, 0.0075073242187500, 0.0012207031250000,
+0.0068054199218750, 0.0007324218750000, 0.0060119628906250, 0.0003356933593750,
+0.0064392089843750, 0.0014343261718750, 0.0053100585937500, 0.0009460449218750,
+0.0052795410156250, 0.0012817382812500, 0.0047912597656250, 0.0025939941406250,
+0.0040283203125000, 0.0020751953125000, 0.0044555664062500, 0.0018005371093750,
+0.0026245117187500, 0.0026855468750000, 0.0027770996093750, 0.0018615722656250,
+0.0022888183593750, 0.0006713867187500, 0.0007629394531250, 0.0017395019531250,
+0.0012817382812500, 0.0003662109375000, 0.0008850097656250, 0.0003356933593750,
+0.0003662109375000, 0.0009155273437500, 0.0007934570312500, 0.0005493164062500,
+0.0011901855468750, 0.0005493164062500, 0.0005798339843750, 0.0018005371093750,
+0.0009765625000000, 0.0014648437500000, 0.0010681152343750, 0.0017700195312500,
+0.0007019042968750, 0.0027770996093750, 0.0004577636718750, 0.0021667480468750,
+0.0005798339843750},
+{0.0020446777343750, -0.0011596679687500, 0.0009765625000000, -0.0008850097656250,
+0.0027160644531250, -0.0011291503906250, 0.0023803710937500, -0.0006408691406250,
+0.0023193359375000, -0.0000915527343750, 0.0032043457031250, -0.0005798339843750,
+0.0024414062500000, 0.0005187988281250, 0.0028991699218750, 0.0001220703125000,
+0.0026855468750000, 0.0003662109375000, 0.0027770996093750, 0.0010070800781250,
+0.0029907226562500, 0.0001525878906250, 0.0023193359375000, 0.0014343261718750,
+0.0036315917968750, 0.0006103515625000, 0.0032043457031250, 0.0007629394531250,
+0.0033569335937500, 0.0016174316406250, 0.0043029785156250, 0.0001220703125000,
+0.0037231445312500, 0.0016784667968750, 0.0043945312500000, 0.0010070800781250,
+0.0042724609375000, 0.0010070800781250, 0.0040893554687500, 0.0021667480468750,
+0.0045166015625000, 0.0014953613281250, 0.0044555664062500, 0.0025634765625000,
+0.0038146972656250, 0.0026855468750000, 0.0043029785156250, 0.0031738281250000,
+0.0037841796875000, 0.0038452148437500, 0.0030212402343750, 0.0035095214843750,
+0.0037536621093750, 0.0053405761718750, 0.0025024414062500, 0.0046386718750000,
+0.0025329589843750, 0.0052490234375000, 0.0028076171875000, 0.0065307617187500,
+0.0022888183593750, 0.0049743652343750, 0.0021972656250000, 0.0068359375000000,
+0.0024414062500000, 0.0062866210937500, 0.0025024414062500, 0.0058898925781250,
+0.0020446777343750, 0.0069885253906250, 0.0019836425781250, 0.0062561035156250,
+0.0024719238281250, 0.0063781738281250, 0.0012207031250000, 0.0068969726562500,
+0.0010986328125000, 0.0063476562500000, 0.0016479492187500, 0.0061035156250000,
+0.0010986328125000, 0.0069885253906250, 0.0006408691406250, 0.0050964355468750,
+0.0028991699218750, 0.0057067871093750, 0.0021667480468750, 0.0054931640625000,
+0.0022888183593750, 0.0029907226562500, 0.0050354003906250, 0.0011596679687500,
+0.0014953613281250, 0.0031127929687500, 0.0028991699218750, 0.0018310546875000,
+0.0021362304687500, 0.0019531250000000, 0.0028076171875000, 0.0027770996093750,
+0.0038757324218750, 0.0018310546875000, 0.0032958984375000, 0.0017700195312500,
+0.0040283203125000, 0.0014038085937500, 0.0036315917968750, 0.0012512207031250,
+0.0039367675781250, 0.0010070800781250, 0.0044860839843750, 0.0002746582031250,
+0.0037231445312500, 0.0014953613281250, 0.0052185058593750, 0.0009460449218750,
+0.0052185058593750, 0.0009765625000000, 0.0051574707031250, 0.0021057128906250,
+0.0061035156250000, 0.0016784667968750, 0.0060729980468750, 0.0013732910156250,
+0.0056457519531250, 0.0021667480468750, 0.0057983398437500, 0.0014038085937500,
+0.0055541992187500, 0.0003662109375000, 0.0050048828125000, 0.0013427734375000,
+0.0050659179687500, 0.0001525878906250, 0.0047302246093750, -0.0001831054687500,
+0.0046691894531250, 0.0005187988281250, 0.0040893554687500, 0.0006713867187500,
+0.0036621093750000, 0.0007019042968750, 0.0037536621093750, 0.0014953613281250,
+0.0019836425781250, 0.0018920898437500, 0.0021362304687500, 0.0018005371093750,
+0.0016784667968750, 0.0020751953125000, 0.0002746582031250, 0.0025024414062500,
+0.0007019042968750, 0.0014648437500000, 0.0005798339843750, 0.0020446777343750,
+0.0000610351562500, 0.0019836425781250, 0.0003967285156250, 0.0010986328125000,
+0.0008850097656250, 0.0018615722656250, 0.0005187988281250, 0.0018310546875000,
+0.0006713867187500, 0.0017089843750000, 0.0008239746093750, 0.0020751953125000,
+0.0006408691406250, 0.0024108886718750, 0.0003967285156250, 0.0023193359375000,
+0.0003356933593750, 0.0024414062500000, 0.0003662109375000, 0.0027465820312500,
+-0.0000610351562500, 0.0024414062500000, -0.0002441406250000, 0.0023803710937500,
+-0.0002136230468750, 0.0028686523437500, -0.0007934570312500, 0.0017395019531250,
+-0.0009765625000000, 0.0023193359375000, -0.0013427734375000, 0.0021667480468750,
+-0.0015258789062500, 0.0011901855468750, -0.0017395019531250, 0.0019226074218750,
+-0.0022277832031250, 0.0016479492187500, -0.0015869140625000, 0.0014953613281250,
+-0.0019531250000000, 0.0019226074218750, -0.0018920898437500, 0.0019836425781250,
+-0.0013122558593750, 0.0016479492187500, -0.0016174316406250, 0.0022888183593750,
+-0.0013427734375000, 0.0017089843750000, -0.0014953613281250, 0.0017395019531250,
+-0.0013732910156250, 0.0021362304687500, -0.0011291503906250, 0.0010681152343750,
+-0.0015563964843750, 0.0017700195312500, -0.0008544921875000, 0.0013122558593750,
+-0.0010681152343750, 0.0007629394531250, -0.0010070800781250, 0.0009765625000000,
+-0.0004577636718750, 0.0005493164062500, -0.0008850097656250, -0.0001525878906250,
+-0.0003967285156250, -0.0000915527343750, -0.0007324218750000, -0.0006103515625000,
+-0.0005187988281250, -0.0012817382812500, -0.0000305175781250, -0.0009460449218750,
+-0.0005493164062500, -0.0018310546875000, 0.0003967285156250, -0.0015869140625000,
+0.0002136230468750, -0.0018310546875000, 0.0002441406250000, -0.0023498535156250,
+0.0007629394531250, -0.0016784667968750, 0.0004272460937500, -0.0024414062500000,
+0.0009765625000000, -0.0021057128906250, 0.0008544921875000, -0.0018310546875000,
+0.0009155273437500, -0.0020141601562500, 0.0012512207031250, -0.0015563964843750,
+0.0009460449218750, -0.0014648437500000, 0.0011596679687500, -0.0013732910156250,
+0.0010375976562500, -0.0010070800781250, 0.0007934570312500, -0.0008239746093750,
+0.0007324218750000, -0.0007934570312500, 0.0005798339843750, -0.0003967285156250,
+0.0003967285156250, -0.0003051757812500, 0.0003356933593750, -0.0001831054687500,
+0.0001525878906250, 0.0001831054687500, -0.0000305175781250, 0.0002136230468750,
+0.0001525878906250, 0.0002136230468750, -0.0003051757812500, 0.0005493164062500,
+-0.0001525878906250, 0.0005493164062500, -0.0001831054687500, 0.0005187988281250,
+-0.0004272460937500, 0.0009460449218750, -0.0001525878906250, 0.0003356933593750,
+-0.0003051757812500, 0.0008850097656250, -0.0001831054687500, 0.0007324218750000,
+-0.0001831054687500, 0.0003051757812500, -0.0002746582031250, 0.0010681152343750,
+-0.0001525878906250, 0.0000610351562500, -0.0000610351562500, 0.0005187988281250,
+-0.0001220703125000, 0.0005187988281250, 0.0000000000000000, -0.0002136230468750,
+0.0001525878906250, 0.0004272460937500, 0.0000915527343750, -0.0003356933593750,
+0.0001525878906250, -0.0002441406250000, 0.0002746582031250, -0.0001525878906250,
+0.0002136230468750, -0.0006103515625000, 0.0002136230468750, -0.0004577636718750,
+0.0001831054687500, -0.0003967285156250, 0.0002136230468750, -0.0006713867187500,
+0.0001525878906250, -0.0005493164062500, 0.0001220703125000, -0.0004882812500000,
+0.0001831054687500, -0.0008850097656250, 0.0000610351562500, -0.0002441406250000,
+-0.0000305175781250, -0.0006713867187500, 0.0000610351562500, -0.0004577636718750,
+-0.0001831054687500, 0.0000000000000000, -0.0003662109375000, -0.0009460449218750,
+-0.0000610351562500, 0.0004272460937500, -0.0007629394531250, -0.0002746582031250,
+-0.0003662109375000, -0.0002136230468750, -0.0005187988281250, 0.0007629394531250,
+-0.0008544921875000, -0.0002136230468750, -0.0000305175781250, 0.0008239746093750,
+-0.0007629394531250, 0.0007019042968750, -0.0001525878906250, 0.0005798339843750,
+-0.0002746582031250, 0.0010375976562500, -0.0007934570312500, 0.0006713867187500,
+0.0003356933593750, 0.0007324218750000, -0.0010070800781250, 0.0008850097656250,
+-0.0005187988281250, 0.0007019042968750, -0.0003662109375000, 0.0006103515625000,
+-0.0012512207031250, 0.0009765625000000, -0.0007934570312500, 0.0005798339843750,
+-0.0009765625000000, 0.0008544921875000, -0.0013732910156250, 0.0010375976562500,
+-0.0011291503906250, 0.0009155273437500, -0.0009765625000000, 0.0013122558593750,
+-0.0014953613281250, 0.0012817382812500, -0.0006713867187500, 0.0015869140625000,
+-0.0008239746093750, 0.0016174316406250, -0.0006408691406250, 0.0015563964843750,
+0.0001220703125000, 0.0021362304687500, 0.0000610351562500, 0.0016174316406250,
+0.0005493164062500, 0.0019836425781250, 0.0009765625000000, 0.0021972656250000,
+0.0008850097656250, 0.0018615722656250, 0.0008544921875000, 0.0021972656250000,
+0.0013427734375000, 0.0021972656250000, 0.0005187988281250, 0.0019836425781250,
+0.0007019042968750, 0.0023803710937500, 0.0008544921875000, 0.0025939941406250,
+0.0003356933593750, 0.0021057128906250, 0.0004272460937500, 0.0034179687500000,
+0.0006103515625000, 0.0029602050781250, 0.0003967285156250, 0.0027465820312500,
+0.0003967285156250, 0.0036315917968750, 0.0006408691406250, 0.0031738281250000,
+0.0004882812500000, 0.0025634765625000, 0.0005187988281250, 0.0029907226562500,
+0.0005187988281250, 0.0025329589843750, 0.0007324218750000, 0.0018005371093750,
+0.0010070800781250, 0.0026550292968750, 0.0008850097656250, 0.0017089843750000,
+0.0013122558593750, 0.0022277832031250, 0.0013732910156250, 0.0023193359375000,
+0.0016479492187500, 0.0017700195312500, 0.0019531250000000, 0.0023193359375000,
+0.0018310546875000, 0.0013122558593750, 0.0025939941406250, 0.0016784667968750,
+0.0025329589843750, 0.0012817382812500, 0.0027160644531250, 0.0001220703125000,
+0.0033874511718750, 0.0008850097656250, 0.0033264160156250, 0.0001220703125000,
+0.0034484863281250, -0.0001831054687500, 0.0041198730468750, 0.0004272460937500,
+0.0038146972656250, 0.0010070800781250, 0.0031738281250000, 0.0017700195312500,
+0.0033874511718750, 0.0034790039062500, 0.0020751953125000, 0.0027465820312500,
+0.0027160644531250, 0.0028686523437500, 0.0025634765625000, 0.0038757324218750,
+0.0016479492187500, 0.0031127929687500, 0.0025329589843750, 0.0039062500000000,
+0.0011291503906250, 0.0035095214843750, 0.0015869140625000, 0.0035095214843750,
+0.0012817382812500, 0.0040588378906250, 0.0002746582031250, 0.0033569335937500,
+0.0011596679687500, 0.0039672851562500, 0.0002441406250000, 0.0036621093750000,
+0.0004577636718750, 0.0033264160156250, 0.0007324218750000, 0.0032653808593750,
+0.0005187988281250, 0.0026245117187500, 0.0010070800781250, 0.0023193359375000,
+0.0004272460937500, 0.0021972656250000, 0.0007019042968750, 0.0015563964843750,
+0.0006408691406250, 0.0012512207031250, 0.0000915527343750, 0.0014343261718750,
+0.0004882812500000, 0.0003662109375000, 0.0002441406250000, 0.0007629394531250,
+0.0000000000000000, 0.0004882812500000, 0.0002136230468750, -0.0003662109375000,
+0.0003356933593750, 0.0003662109375000, 0.0000915527343750, -0.0001220703125000,
+0.0007934570312500, -0.0002441406250000, 0.0007324218750000, 0.0000000000000000,
+0.0007629394531250, 0.0001220703125000, 0.0011291503906250, 0.0002441406250000,
+0.0008544921875000, 0.0003662109375000, 0.0008850097656250, 0.0006713867187500,
+0.0006103515625000, 0.0006103515625000, 0.0006713867187500, 0.0005798339843750,
+0.0006713867187500, 0.0010681152343750, 0.0002441406250000, 0.0005493164062500,
+0.0009460449218750, 0.0009155273437500, 0.0004882812500000, 0.0007629394531250,
+0.0005493164062500, 0.0002441406250000, 0.0011901855468750, 0.0007934570312500,
+0.0006408691406250, -0.0000915527343750, 0.0008544921875000, 0.0000610351562500,
+0.0009155273437500, -0.0001525878906250, 0.0006713867187500, -0.0008850097656250,
+0.0005187988281250, -0.0007019042968750, 0.0005798339843750, -0.0009155273437500,
+0.0003356933593750, -0.0014343261718750, 0.0000915527343750, -0.0011596679687500,
+0.0004272460937500, -0.0009765625000000, 0.0005493164062500, -0.0015563964843750,
+0.0002441406250000, -0.0009155273437500, 0.0012207031250000, -0.0010375976562500,
+0.0009460449218750},
+{0.0007324218750000, 0.0025024414062500, 0.0007629394531250, 0.0025939941406250,
+0.0007934570312500, 0.0022888183593750, 0.0005493164062500, 0.0022277832031250,
+0.0007019042968750, 0.0020751953125000, 0.0004882812500000, 0.0014648437500000,
+0.0005798339843750, 0.0017395019531250, 0.0005798339843750, 0.0018005371093750,
+0.0004272460937500, 0.0007629394531250, 0.0006408691406250, 0.0020751953125000,
+0.0010070800781250, 0.0014343261718750, 0.0009765625000000, 0.0013122558593750,
+0.0011901855468750, 0.0024108886718750, 0.0013732910156250, 0.0018920898437500,
+0.0012207031250000, 0.0023498535156250, 0.0019836425781250, 0.0027160644531250,
+0.0017700195312500, 0.0028381347656250, 0.0021667480468750, 0.0031127929687500,
+0.0029296875000000, 0.0035095214843750, 0.0022888183593750, 0.0034484863281250,
+0.0029602050781250, 0.0039367675781250, 0.0033874511718750, 0.0036926269531250,
+0.0030212402343750, 0.0035400390625000, 0.0032043457031250, 0.0043945312500000,
+0.0039978027343750, 0.0025634765625000, 0.0027160644531250, 0.0033874511718750,
+0.0036621093750000, 0.0032043457031250, 0.0036621093750000, 0.0013122558593750,
+0.0021972656250000, 0.0019226074218750, 0.0025634765625000, 0.0015563964843750,
+0.0021057128906250, 0.0013732910156250, 0.0023193359375000, 0.0018005371093750,
+0.0021362304687500, 0.0019226074218750, 0.0016479492187500, 0.0016784667968750,
+0.0019836425781250, 0.0023803710937500, 0.0011901855468750, 0.0023193359375000,
+0.0011291503906250, 0.0023193359375000, 0.0011596679687500, 0.0025024414062500,
+0.0008850097656250, 0.0023498535156250, 0.0011901855468750, 0.0029296875000000,
+0.0010375976562500, 0.0024719238281250, 0.0010986328125000, 0.0029296875000000,
+0.0009765625000000, 0.0036621093750000, 0.0005798339843750, 0.0028076171875000,
+0.0004577636718750, 0.0039062500000000, 0.0005187988281250, 0.0036010742187500,
+0.0001220703125000, 0.0031738281250000, 0.0003051757812500, 0.0034790039062500,
+0.0005493164062500, 0.0028686523437500, 0.0000610351562500, 0.0025939941406250,
+0.0008239746093750, 0.0025329589843750, 0.0006713867187500, 0.0019531250000000,
+0.0005798339843750, 0.0015258789062500, 0.0010681152343750, 0.0017700195312500,
+0.0008544921875000, 0.0004577636718750, 0.0008544921875000, 0.0007324218750000,
+0.0007629394531250, 0.0005187988281250, 0.0008850097656250, -0.0004882812500000,
+0.0007629394531250, -0.0001525878906250, 0.0003967285156250, -0.0003967285156250,
+0.0011901855468750, -0.0005798339843750, 0.0004882812500000, -0.0002746582031250,
+0.0006408691406250, -0.0000610351562500, 0.0013122558593750, -0.0001831054687500,
+0.0003967285156250, 0.0003356933593750, 0.0010375976562500, 0.0004272460937500,
+0.0007324218750000, 0.0003967285156250, 0.0005187988281250, 0.0007019042968750,
+0.0007019042968750, 0.0008239746093750, 0.0001831054687500, 0.0004272460937500,
+0.0005493164062500, 0.0006713867187500, 0.0001220703125000, 0.0004882812500000,
+0.0004272460937500, 0.0000610351562500, 0.0009460449218750, 0.0003662109375000,
+0.0002136230468750, -0.0003356933593750, 0.0014038085937500, -0.0004577636718750,
+0.0010375976562500, -0.0004577636718750, 0.0011291503906250, -0.0009460449218750,
+0.0018615722656250, -0.0010375976562500, 0.0012207031250000, -0.0008544921875000,
+0.0015869140625000, -0.0014343261718750, 0.0015258789062500, -0.0011596679687500,
+0.0012207031250000, -0.0007019042968750, 0.0012207031250000, -0.0014648437500000,
+0.0010986328125000, -0.0005798339843750, 0.0007629394531250, -0.0006713867187500,
+0.0007934570312500, -0.0008544921875000, 0.0006103515625000, -0.0003662109375000,
+0.0003967285156250, -0.0004577636718750, 0.0006408691406250, -0.0006408691406250,
+0.0000610351562500, -0.0006103515625000, 0.0003967285156250, -0.0005493164062500,
+0.0003051757812500, -0.0007019042968750, 0.0000000000000000, -0.0006713867187500,
+0.0005798339843750, -0.0004272460937500, -0.0002136230468750, -0.0005798339843750,
+0.0002746582031250, -0.0004272460937500, 0.0002746582031250, -0.0001525878906250,
+-0.0002746582031250, -0.0003051757812500, 0.0004882812500000, 0.0000305175781250,
+0.0000305175781250, 0.0000610351562500, 0.0002441406250000, 0.0001525878906250,
+0.0004272460937500, 0.0004272460937500, 0.0002136230468750, 0.0004577636718750,
+0.0003967285156250, 0.0006103515625000, 0.0004882812500000, 0.0006408691406250,
+0.0002136230468750, 0.0006713867187500, 0.0003051757812500, 0.0008544921875000,
+0.0004577636718750, 0.0009155273437500, 0.0001220703125000, 0.0006408691406250,
+0.0003051757812500, 0.0010986328125000, 0.0001525878906250, 0.0008544921875000,
+-0.0000610351562500, 0.0005493164062500, -0.0000610351562500, 0.0013122558593750,
+-0.0002441406250000, 0.0001831054687500, -0.0004272460937500, 0.0007934570312500,
+-0.0007019042968750, 0.0007934570312500, -0.0007629394531250, -0.0000305175781250,
+-0.0009155273437500, 0.0008850097656250, -0.0012207031250000, 0.0000305175781250,
+-0.0010681152343750, 0.0002746582031250, -0.0014038085937500, 0.0004577636718750,
+-0.0013122558593750, 0.0000000000000000, -0.0011901855468750, 0.0003662109375000,
+-0.0017395019531250, 0.0002441406250000, -0.0010986328125000, 0.0002441406250000,
+-0.0015869140625000, 0.0003051757812500, -0.0015258789062500, 0.0003356933593750,
+-0.0010375976562500, 0.0003967285156250, -0.0018615722656250, 0.0002746582031250,
+-0.0009460449218750, 0.0003662109375000, -0.0013427734375000, 0.0002441406250000,
+-0.0013427734375000, 0.0000610351562500, -0.0007629394531250, 0.0002746582031250,
+-0.0013427734375000, -0.0002441406250000, -0.0007324218750000, 0.0000305175781250,
+-0.0007934570312500, -0.0001831054687500, -0.0006713867187500, -0.0005493164062500,
+-0.0001220703125000, 0.0000000000000000, -0.0003662109375000, -0.0007324218750000,
+0.0002136230468750, -0.0005798339843750, 0.0001831054687500, -0.0003967285156250,
+0.0003356933593750, -0.0007629394531250, 0.0007324218750000, -0.0004882812500000,
+0.0006408691406250, -0.0003051757812500, 0.0008850097656250, -0.0004272460937500,
+0.0009460449218750, -0.0001831054687500, 0.0009155273437500, -0.0000305175781250,
+0.0009155273437500, -0.0003051757812500, 0.0007629394531250, 0.0001220703125000,
+0.0005798339843750, -0.0001831054687500, 0.0005493164062500, -0.0004577636718750,
+0.0004272460937500, -0.0003051757812500, 0.0001220703125000, -0.0007629394531250,
+-0.0000305175781250, -0.0008850097656250, 0.0002136230468750, -0.0009765625000000,
+-0.0001220703125000, -0.0009155273437500, 0.0001831054687500, -0.0009155273437500,
+0.0006103515625000, -0.0009765625000000, 0.0001831054687500, -0.0002441406250000,
+0.0010986328125000, -0.0003967285156250, 0.0009155273437500, -0.0000305175781250,
+0.0010070800781250, 0.0006713867187500, 0.0016174316406250, 0.0003356933593750,
+0.0012512207031250, 0.0010681152343750, 0.0015258789062500, 0.0008544921875000,
+0.0015258789062500, 0.0013122558593750, 0.0012817382812500, 0.0018615722656250,
+0.0012207031250000, 0.0011291503906250, 0.0013122558593750, 0.0023803710937500,
+0.0008544921875000, 0.0018920898437500, 0.0009765625000000, 0.0017700195312500,
+0.0010681152343750, 0.0024719238281250, 0.0008239746093750, 0.0017089843750000,
+0.0009460449218750, 0.0018310546875000, 0.0012512207031250, 0.0019226074218750,
+0.0013122558593750, 0.0015869140625000, 0.0013732910156250, 0.0016479492187500,
+0.0016174316406250, 0.0019836425781250, 0.0017395019531250, 0.0017700195312500,
+0.0014038085937500, 0.0020751953125000, 0.0016784667968750, 0.0023803710937500,
+0.0015563964843750, 0.0023193359375000, 0.0011596679687500, 0.0025329589843750,
+0.0012817382812500, 0.0030212402343750, 0.0009460449218750, 0.0027160644531250,
+0.0008239746093750, 0.0027465820312500, 0.0007629394531250, 0.0029296875000000,
+0.0006408691406250, 0.0022888183593750, 0.0007324218750000, 0.0028381347656250,
+0.0010375976562500, 0.0026855468750000, 0.0007934570312500, 0.0023803710937500,
+0.0011596679687500, 0.0029296875000000, 0.0017700195312500, 0.0031127929687500,
+0.0014953613281250, 0.0019531250000000, 0.0018005371093750, 0.0031127929687500,
+0.0021972656250000, 0.0024414062500000, 0.0018615722656250, 0.0010681152343750,
+0.0014953613281250, 0.0027465820312500, 0.0018615722656250, 0.0007019042968750,
+0.0013122558593750, 0.0006713867187500, 0.0010986328125000, 0.0013122558593750,
+0.0017089843750000, 0.0004272460937500, 0.0020751953125000, 0.0001831054687500,
+0.0018310546875000, 0.0013122558593750, 0.0027160644531250, 0.0005798339843750,
+0.0032043457031250, 0.0004882812500000, 0.0028381347656250, 0.0019836425781250,
+0.0025634765625000, -0.0009460449218750, 0.0000915527343750, 0.0009765625000000,
+0.0011901855468750, -0.0000610351562500, 0.0017395019531250, 0.0003051757812500,
+0.0014038085937500, 0.0013122558593750, 0.0015869140625000, 0.0005798339843750,
+0.0025939941406250, 0.0009460449218750, 0.0013427734375000, 0.0009460449218750,
+0.0020141601562500, 0.0006713867187500, 0.0024108886718750, 0.0005493164062500,
+0.0019531250000000, 0.0005187988281250, 0.0028686523437500, 0.0011291503906250,
+0.0026550292968750, 0.0008239746093750, 0.0031127929687500, 0.0014038085937500,
+0.0031127929687500, 0.0022888183593750, 0.0026245117187500, 0.0018005371093750,
+0.0030822753906250, 0.0025634765625000, 0.0024414062500000, 0.0026855468750000,
+0.0021972656250000, 0.0021972656250000, 0.0025329589843750, 0.0021362304687500,
+0.0024108886718750, 0.0022888183593750, 0.0021362304687500, 0.0017089843750000,
+0.0031127929687500, 0.0013122558593750, 0.0028076171875000, 0.0017089843750000,
+0.0028991699218750, 0.0018615722656250, 0.0035400390625000, 0.0011596679687500,
+0.0030517578125000, 0.0024108886718750, 0.0033569335937500, 0.0019836425781250,
+0.0031127929687500, 0.0019531250000000, 0.0030517578125000, 0.0026245117187500,
+0.0031433105468750, 0.0020446777343750, 0.0026550292968750, 0.0021057128906250,
+0.0029602050781250, 0.0022277832031250, 0.0029602050781250, 0.0017700195312500,
+0.0025634765625000, 0.0013732910156250, 0.0023803710937500, 0.0015563964843750,
+0.0023193359375000, 0.0011901855468750, 0.0014648437500000, 0.0009155273437500,
+0.0014343261718750, 0.0010986328125000, 0.0011596679687500, 0.0012817382812500,
+0.0006103515625000, 0.0011901855468750, 0.0008239746093750, 0.0014648437500000,
+0.0004272460937500, 0.0015563964843750, 0.0005187988281250, 0.0017089843750000,
+0.0005798339843750, 0.0017700195312500, 0.0003662109375000, 0.0016174316406250,
+0.0005187988281250, 0.0015258789062500, 0.0005798339843750, 0.0015258789062500,
+0.0000915527343750, 0.0012817382812500, 0.0003967285156250, 0.0008850097656250,
+0.0005798339843750, 0.0007324218750000, -0.0002136230468750, 0.0004882812500000,
+0.0006103515625000, 0.0000000000000000, 0.0001831054687500, 0.0000915527343750,
+-0.0002441406250000, 0.0001831054687500, -0.0001220703125000, -0.0003356933593750,
+-0.0006103515625000, 0.0004882812500000, -0.0009155273437500, 0.0001831054687500,
+-0.0010375976562500, 0.0003356933593750, -0.0011596679687500, 0.0009765625000000,
+-0.0012512207031250, 0.0005187988281250, -0.0011596679687500, 0.0013122558593750,
+-0.0011901855468750, 0.0012207031250000, -0.0011901855468750, 0.0011596679687500,
+-0.0010986328125000, 0.0016784667968750, -0.0011291503906250, 0.0013732910156250,
+-0.0011291503906250, 0.0012207031250000, -0.0007934570312500, 0.0013732910156250,
+-0.0009460449218750, 0.0009765625000000, -0.0007629394531250, 0.0004882812500000,
+-0.0003967285156250},
+{0.0017700195312500, -0.0008544921875000, 0.0012512207031250, 0.0002136230468750,
+0.0018920898437500, 0.0000000000000000, 0.0007324218750000, -0.0006713867187500,
+0.0014648437500000, -0.0003356933593750, 0.0008544921875000, -0.0001220703125000,
+-0.0003662109375000, -0.0009155273437500, 0.0009155273437500, -0.0001525878906250,
+-0.0003967285156250, -0.0006713867187500, -0.0005798339843750, -0.0007324218750000,
+0.0001525878906250, 0.0004272460937500, 0.0001831054687500, -0.0007629394531250,
+0.0007324218750000, -0.0008544921875000, -0.0017089843750000, -0.0006103515625000,
+-0.0010375976562500, -0.0007934570312500, -0.0006408691406250, -0.0010986328125000,
+-0.0008544921875000, -0.0008239746093750, -0.0004272460937500, -0.0001525878906250,
+-0.0001525878906250, -0.0003662109375000, 0.0003356933593750, 0.0003967285156250,
+0.0003662109375000, 0.0015563964843750, 0.0003662109375000, 0.0012512207031250,
+0.0008544921875000, 0.0018920898437500, 0.0007324218750000, 0.0025939941406250,
+0.0009765625000000, 0.0021667480468750, 0.0015258789062500, 0.0016174316406250,
+0.0018310546875000, 0.0021667480468750, 0.0020446777343750, 0.0013732910156250,
+0.0024414062500000, 0.0010986328125000, 0.0029602050781250, 0.0016174316406250,
+0.0025634765625000, 0.0016174316406250, 0.0023803710937500, 0.0012512207031250,
+0.0030517578125000, 0.0023498535156250, 0.0019531250000000, 0.0023498535156250,
+0.0021362304687500, 0.0021057128906250, 0.0025939941406250, 0.0023803710937500,
+0.0021057128906250, 0.0023498535156250, 0.0020446777343750, 0.0018615722656250,
+0.0028991699218750, 0.0015869140625000, 0.0026855468750000, 0.0014953613281250,
+0.0026245117187500, 0.0010681152343750, 0.0030212402343750, 0.0006103515625000,
+0.0026855468750000, 0.0012207031250000, 0.0026855468750000, 0.0004882812500000,
+0.0023498535156250, 0.0007934570312500, 0.0024414062500000, 0.0015563964843750,
+0.0025939941406250, 0.0005493164062500, 0.0020446777343750, 0.0012512207031250,
+0.0025329589843750, 0.0013122558593750, 0.0025024414062500, 0.0007629394531250,
+0.0021362304687500, 0.0007019042968750, 0.0022583007812500, 0.0005798339843750,
+0.0021362304687500, 0.0000305175781250, 0.0015563964843750, -0.0000305175781250,
+0.0016784667968750, -0.0001220703125000, 0.0014343261718750, -0.0005187988281250,
+0.0010070800781250, -0.0004577636718750, 0.0013732910156250, -0.0002441406250000,
+0.0008544921875000, -0.0006713867187500, 0.0010375976562500, -0.0002746582031250,
+0.0010681152343750, 0.0002136230468750, 0.0005493164062500, -0.0003967285156250,
+0.0007324218750000, 0.0005493164062500, 0.0006408691406250, 0.0005493164062500,
+0.0001525878906250, 0.0004882812500000, 0.0003051757812500, 0.0008544921875000,
+0.0004272460937500, 0.0007324218750000, -0.0001831054687500, 0.0006408691406250,
+0.0002136230468750, 0.0004882812500000, -0.0000610351562500, 0.0004272460937500,
+-0.0004882812500000, 0.0002746582031250, -0.0003967285156250, -0.0001220703125000,
+-0.0006103515625000, -0.0003051757812500, -0.0009460449218750, -0.0007019042968750,
+-0.0008850097656250, -0.0009155273437500, -0.0009155273437500, -0.0010986328125000,
+-0.0010986328125000, -0.0016174316406250, -0.0009155273437500, -0.0012512207031250,
+-0.0010986328125000, -0.0018310546875000, -0.0010375976562500, -0.0016174316406250,
+-0.0010681152343750, -0.0011291503906250, -0.0011901855468750, -0.0017089843750000,
+-0.0010681152343750, -0.0005798339843750, -0.0008544921875000, -0.0007629394531250,
+-0.0010375976562500, -0.0004577636718750, -0.0008850097656250, 0.0004272460937500,
+-0.0006408691406250, -0.0000305175781250, -0.0009460449218750, 0.0009460449218750,
+-0.0007019042968750, 0.0009155273437500, -0.0007019042968750, 0.0010375976562500,
+-0.0007324218750000, 0.0016174316406250, -0.0005798339843750, 0.0014038085937500,
+-0.0005493164062500, 0.0018005371093750, -0.0004577636718750, 0.0019531250000000,
+-0.0003356933593750, 0.0019531250000000, -0.0003051757812500, 0.0021362304687500,
+-0.0003051757812500, 0.0022277832031250, -0.0000610351562500, 0.0020446777343750,
+-0.0001525878906250, 0.0021057128906250, -0.0000610351562500, 0.0020751953125000,
+0.0001220703125000, 0.0017395019531250, 0.0002441406250000, 0.0016174316406250,
+0.0003356933593750, 0.0016174316406250, 0.0004577636718750, 0.0013427734375000,
+0.0006408691406250, 0.0013427734375000, 0.0006408691406250, 0.0013427734375000,
+0.0006103515625000, 0.0009460449218750, 0.0010070800781250, 0.0012817382812500,
+0.0005493164062500, 0.0009460449218750, 0.0008239746093750, 0.0009155273437500,
+0.0008239746093750, 0.0010986328125000, 0.0005187988281250, 0.0006713867187500,
+0.0008850097656250, 0.0009155273437500, 0.0007019042968750, 0.0009765625000000,
+0.0008239746093750, 0.0007324218750000, 0.0008544921875000, 0.0008544921875000,
+0.0007934570312500, 0.0011291503906250, 0.0010070800781250, 0.0005493164062500,
+0.0008239746093750, 0.0011596679687500, 0.0008239746093750, 0.0009460449218750,
+0.0008544921875000, 0.0004272460937500, 0.0006713867187500, 0.0013122558593750,
+0.0007324218750000, 0.0001525878906250, 0.0004882812500000, 0.0007019042968750,
+0.0004882812500000, 0.0005493164062500, 0.0003662109375000, -0.0001525878906250,
+0.0001831054687500, 0.0007629394531250, 0.0001220703125000, -0.0004577636718750,
+0.0001525878906250, 0.0000000000000000, 0.0000915527343750, -0.0001220703125000,
+0.0000915527343750, -0.0009460449218750, 0.0002136230468750, -0.0001831054687500,
+0.0001525878906250, -0.0011596679687500, -0.0001525878906250, -0.0008239746093750,
+0.0000305175781250, -0.0007934570312500, -0.0001220703125000, -0.0013732910156250,
+-0.0005493164062500, -0.0007324218750000, -0.0004577636718750, -0.0012207031250000,
+-0.0006103515625000, -0.0012207031250000, -0.0009460449218750, -0.0009765625000000,
+-0.0007934570312500, -0.0010986328125000, -0.0007019042968750, -0.0011291503906250,
+-0.0010681152343750, -0.0007629394531250, -0.0004882812500000, -0.0007934570312500,
+-0.0004577636718750, -0.0007934570312500, -0.0003662109375000, -0.0005798339843750,
+0.0001831054687500, -0.0005187988281250, 0.0003051757812500, -0.0007934570312500,
+0.0004577636718750, -0.0004272460937500, 0.0009765625000000, -0.0007629394531250,
+0.0011291503906250, -0.0011901855468750, 0.0011291503906250, -0.0004882812500000,
+0.0014648437500000, -0.0014953613281250, 0.0011596679687500, -0.0011291503906250,
+0.0011901855468750, -0.0010681152343750, 0.0011291503906250, -0.0016174316406250,
+0.0007324218750000, -0.0010070800781250, 0.0006103515625000, -0.0012817382812500,
+0.0007629394531250, -0.0010986328125000, 0.0001220703125000, -0.0009155273437500,
+0.0002746582031250, -0.0010375976562500, 0.0005493164062500, -0.0008239746093750,
+-0.0002136230468750, -0.0009460449218750, 0.0005798339843750, -0.0010070800781250,
+0.0001831054687500, -0.0006103515625000, 0.0002441406250000, -0.0006103515625000,
+0.0008239746093750, -0.0010375976562500, 0.0004272460937500, -0.0005187988281250,
+0.0012207031250000, -0.0011291503906250, 0.0010986328125000, -0.0007324218750000,
+0.0011291503906250, -0.0001525878906250, 0.0015869140625000, -0.0013122558593750,
+0.0013427734375000, 0.0006103515625000, 0.0018920898437500, 0.0000000000000000,
+0.0018005371093750, 0.0001831054687500, 0.0019226074218750, 0.0017395019531250,
+0.0023803710937500, 0.0008850097656250, 0.0021667480468750, 0.0013732910156250,
+0.0021362304687500, 0.0020446777343750, 0.0025634765625000, 0.0016784667968750,
+0.0025024414062500, 0.0014953613281250, 0.0023498535156250, 0.0023193359375000,
+0.0029907226562500, 0.0021057128906250, 0.0024414062500000, 0.0024414062500000,
+0.0029602050781250, 0.0029907226562500, 0.0028991699218750, 0.0033264160156250,
+0.0024414062500000, 0.0037231445312500, 0.0031127929687500, 0.0034790039062500,
+0.0021972656250000, 0.0041198730468750, 0.0024414062500000, 0.0041503906250000,
+0.0024108886718750, 0.0029296875000000, 0.0014648437500000, 0.0021667480468750,
+0.0014038085937500, 0.0023803710937500, 0.0014343261718750, 0.0022583007812500,
+0.0011291503906250, 0.0023193359375000, 0.0011901855468750, 0.0027160644531250,
+0.0012817382812500, 0.0025024414062500, 0.0008544921875000, 0.0026245117187500,
+0.0015258789062500, 0.0029602050781250, 0.0008239746093750, 0.0026245117187500,
+0.0008850097656250, 0.0025634765625000, 0.0013732910156250, 0.0031127929687500,
+0.0003356933593750, 0.0018920898437500, 0.0013427734375000, 0.0028076171875000,
+0.0007629394531250, 0.0024414062500000, 0.0005187988281250, 0.0012817382812500,
+0.0009765625000000, 0.0026550292968750, 0.0000915527343750, 0.0011596679687500,
+0.0004577636718750, 0.0012512207031250, 0.0000305175781250, 0.0017700195312500,
+0.0000000000000000, 0.0010375976562500, 0.0002746582031250, 0.0008544921875000,
+-0.0004577636718750, 0.0020446777343750, 0.0005798339843750, 0.0011291503906250,
+0.0000305175781250, 0.0014953613281250, 0.0000610351562500, 0.0025939941406250,
+0.0008544921875000, 0.0012817382812500, 0.0001220703125000, 0.0021972656250000,
+0.0006103515625000, 0.0020751953125000, 0.0004577636718750, 0.0016174316406250,
+0.0003356933593750, 0.0018920898437500, 0.0004882812500000, 0.0015869140625000,
+0.0001525878906250, 0.0013122558593750, 0.0007324218750000, 0.0012207031250000,
+0.0005493164062500, 0.0010681152343750, 0.0006408691406250, 0.0008850097656250,
+0.0011596679687500, 0.0007019042968750, 0.0008239746093750, 0.0004272460937500,
+0.0012207031250000, 0.0002746582031250, 0.0009765625000000, 0.0001525878906250,
+0.0009155273437500, 0.0000305175781250, 0.0010070800781250, -0.0000305175781250,
+0.0004272460937500, 0.0000915527343750, 0.0007934570312500, 0.0001525878906250,
+0.0000610351562500, 0.0001831054687500, 0.0000305175781250, 0.0004272460937500,
+0.0003051757812500, 0.0006713867187500, -0.0007324218750000, 0.0004272460937500,
+0.0000915527343750, 0.0008850097656250, -0.0004577636718750, 0.0007019042968750,
+-0.0007324218750000, 0.0003967285156250, -0.0003356933593750, 0.0010681152343750,
+-0.0010681152343750, 0.0001525878906250, -0.0010070800781250, 0.0004272460937500,
+-0.0012207031250000, 0.0004272460937500, -0.0014953613281250, -0.0001220703125000,
+-0.0015869140625000, 0.0002136230468750, -0.0018615722656250, 0.0000305175781250,
+-0.0013732910156250, -0.0002746582031250, -0.0018005371093750, -0.0000610351562500,
+-0.0014038085937500, -0.0000610351562500, -0.0007324218750000, -0.0003967285156250,
+-0.0013427734375000, 0.0001831054687500, 0.0000305175781250, -0.0001220703125000,
+-0.0001831054687500, -0.0000305175781250, 0.0001220703125000, 0.0005187988281250,
+0.0010986328125000, 0.0001525878906250, 0.0004577636718750, 0.0004272460937500,
+0.0013122558593750, 0.0003967285156250, 0.0010681152343750, 0.0002746582031250,
+0.0007934570312500, 0.0002746582031250, 0.0010681152343750, 0.0000915527343750,
+0.0007019042968750, 0.0000915527343750, 0.0004577636718750, -0.0001525878906250,
+0.0004272460937500, -0.0001831054687500, 0.0000000000000000, -0.0000915527343750,
+-0.0003051757812500, -0.0002746582031250, -0.0002746582031250, -0.0002441406250000,
+-0.0008239746093750, -0.0000610351562500, -0.0006713867187500, -0.0000305175781250,
+-0.0007324218750000, -0.0000305175781250, -0.0011901855468750, 0.0001220703125000,
+-0.0008544921875000, 0.0003051757812500, -0.0011596679687500, 0.0003356933593750,
+-0.0011901855468750, 0.0005798339843750, -0.0009460449218750, 0.0008544921875000,
+-0.0009765625000000, 0.0007629394531250, -0.0010375976562500, 0.0011596679687500,
+-0.0006713867187500},
+{-0.0000610351562500, 0.0023193359375000, 0.0010070800781250, 0.0021362304687500,
+0.0002746582031250, 0.0024108886718750, 0.0002136230468750, 0.0028686523437500,
+0.0007934570312500, 0.0026855468750000, -0.0003356933593750, 0.0028076171875000,
+0.0004577636718750, 0.0031738281250000, -0.0003356933593750, 0.0027465820312500,
+-0.0004272460937500, 0.0026245117187500, 0.0001220703125000, 0.0032958984375000,
+-0.0010070800781250, 0.0019836425781250, 0.0003051757812500, 0.0028991699218750,
+-0.0002441406250000, 0.0024108886718750, -0.0002136230468750, 0.0012207031250000,
+0.0006103515625000, 0.0026245117187500, -0.0003051757812500, 0.0009460449218750,
+0.0006408691406250, 0.0010070800781250, 0.0002746582031250, 0.0014648437500000,
+0.0001831054687500, 0.0004882812500000, 0.0006103515625000, 0.0004272460937500,
+-0.0000305175781250, 0.0013427734375000, 0.0006408691406250, 0.0005493164062500,
+0.0003051757812500, 0.0007934570312500, 0.0003662109375000, 0.0016479492187500,
+0.0009765625000000, 0.0007019042968750, 0.0004272460937500, 0.0015563964843750,
+0.0007629394531250, 0.0013427734375000, 0.0008544921875000, 0.0011291503906250,
+0.0005798339843750, 0.0014648437500000, 0.0004577636718750, 0.0010070800781250,
+0.0004577636718750, 0.0012512207031250, 0.0001831054687500, 0.0011596679687500,
+-0.0000915527343750, 0.0009765625000000, -0.0000610351562500, 0.0011291503906250,
+-0.0002136230468750, 0.0010681152343750, -0.0006713867187500, 0.0008850097656250,
+-0.0003967285156250, 0.0009765625000000, -0.0009155273437500, 0.0009765625000000,
+-0.0011291503906250, 0.0008239746093750, -0.0010070800781250, 0.0009460449218750,
+-0.0017089843750000, 0.0008850097656250, -0.0015563964843750, 0.0012207031250000,
+-0.0019226074218750, 0.0011291503906250, -0.0021057128906250, 0.0011291503906250,
+-0.0020141601562500, 0.0016479492187500, -0.0024108886718750, 0.0010070800781250,
+-0.0017395019531250, 0.0016174316406250, -0.0020446777343750, 0.0014343261718750,
+-0.0015258789062500, 0.0007019042968750, -0.0007324218750000, 0.0015869140625000,
+-0.0012817382812500, 0.0004577636718750, 0.0002136230468750, 0.0005798339843750,
+-0.0001525878906250, 0.0007324218750000, -0.0001220703125000, 0.0000610351562500,
+0.0006103515625000, 0.0003356933593750, -0.0002441406250000, 0.0001831054687500,
+0.0002746582031250, 0.0000305175781250, -0.0001220703125000, 0.0002136230468750,
+-0.0004882812500000, 0.0003051757812500, -0.0004577636718750, 0.0001525878906250,
+-0.0010681152343750, 0.0005187988281250, -0.0011901855468750, 0.0004272460937500,
+-0.0014953613281250, 0.0005187988281250, -0.0016174316406250, 0.0008239746093750,
+-0.0016479492187500, 0.0006408691406250, -0.0019531250000000, 0.0005187988281250,
+-0.0016479492187500, 0.0006408691406250, -0.0017700195312500, 0.0004272460937500,
+-0.0017700195312500, 0.0001831054687500, -0.0015563964843750, 0.0003356933593750,
+-0.0016784667968750, 0.0001220703125000, -0.0013427734375000, 0.0003356933593750,
+-0.0011291503906250, 0.0003051757812500, -0.0010375976562500, 0.0000915527343750,
+-0.0009460449218750, 0.0002136230468750, -0.0009460449218750, 0.0002441406250000,
+-0.0006713867187500, 0.0000610351562500, -0.0007629394531250, 0.0002136230468750,
+-0.0006103515625000, 0.0003356933593750, -0.0003967285156250, 0.0000610351562500,
+-0.0006408691406250, 0.0005493164062500, -0.0002746582031250, 0.0004272460937500,
+-0.0003051757812500, 0.0003051757812500, -0.0003051757812500, 0.0005798339843750,
+-0.0000610351562500, 0.0005187988281250, -0.0000915527343750, 0.0003051757812500,
+-0.0000610351562500, 0.0005798339843750, 0.0001525878906250, 0.0002441406250000,
+0.0001831054687500, -0.0000610351562500, 0.0002136230468750, 0.0004882812500000,
+0.0004882812500000, -0.0004272460937500, 0.0004577636718750, -0.0001220703125000,
+0.0007934570312500, -0.0002441406250000, 0.0008544921875000, -0.0008544921875000,
+0.0008850097656250, -0.0002441406250000, 0.0012817382812500, -0.0007629394531250,
+0.0010986328125000, -0.0007324218750000, 0.0012512207031250, -0.0004882812500000,
+0.0013122558593750, -0.0007324218750000, 0.0011901855468750, -0.0006713867187500,
+0.0013732910156250, -0.0003967285156250, 0.0011291503906250, -0.0007324218750000,
+0.0011291503906250, -0.0004272460937500, 0.0011291503906250, -0.0000305175781250,
+0.0008544921875000, -0.0005798339843750, 0.0007324218750000, 0.0003356933593750,
+0.0009155273437500, 0.0000305175781250, 0.0004882812500000, -0.0000305175781250,
+0.0007629394531250, 0.0004882812500000, 0.0010986328125000, 0.0000610351562500,
+0.0004882812500000, 0.0002136230468750, 0.0015869140625000, 0.0002136230468750,
+0.0011901855468750, 0.0001831054687500, 0.0014648437500000, 0.0002441406250000,
+0.0024108886718750, 0.0003051757812500, 0.0017395019531250, 0.0005493164062500,
+0.0027160644531250, 0.0005187988281250, 0.0026855468750000, 0.0007324218750000,
+0.0026855468750000, 0.0009765625000000, 0.0031738281250000, 0.0009155273437500,
+0.0029602050781250, 0.0014343261718750, 0.0029602050781250, 0.0011596679687500,
+0.0030212402343750, 0.0014953613281250, 0.0026245117187500, 0.0019531250000000,
+0.0024108886718750, 0.0011901855468750, 0.0026245117187500, 0.0020751953125000,
+0.0020446777343750, 0.0015563964843750, 0.0021057128906250, 0.0013122558593750,
+0.0020141601562500, 0.0015869140625000, 0.0017700195312500, 0.0007934570312500,
+0.0020446777343750, 0.0012207031250000, 0.0017089843750000, 0.0007019042968750,
+0.0019836425781250, 0.0005798339843750, 0.0018920898437500, 0.0010070800781250,
+0.0016479492187500, 0.0004577636718750, 0.0020751953125000, 0.0010375976562500,
+0.0007934570312500, 0.0009765625000000, 0.0014648437500000, 0.0010375976562500,
+0.0007019042968750, 0.0014343261718750, -0.0007629394531250, 0.0012817382812500,
+0.0004577636718750, 0.0019531250000000, -0.0014343261718750, 0.0017089843750000,
+-0.0013732910156250, 0.0019531250000000, -0.0011901855468750, 0.0023803710937500,
+-0.0020446777343750, 0.0017089843750000, -0.0016784667968750, 0.0020446777343750,
+-0.0017089843750000, 0.0021057128906250, -0.0018920898437500, 0.0014038085937500,
+-0.0017089843750000, 0.0014343261718750, -0.0015563964843750, 0.0019836425781250,
+-0.0017089843750000, 0.0002441406250000, -0.0013122558593750, 0.0014343261718750,
+-0.0012512207031250, 0.0008239746093750, -0.0011596679687500, -0.0007324218750000,
+-0.0009155273437500, 0.0013122558593750, -0.0009765625000000, -0.0010681152343750,
+-0.0011291503906250, -0.0002746582031250, -0.0012512207031250, -0.0002746582031250,
+-0.0012207031250000, -0.0020141601562500, -0.0013427734375000, -0.0009765625000000,
+-0.0013427734375000, -0.0016784667968750, -0.0004272460937500, -0.0026245117187500,
+-0.0005493164062500, -0.0021667480468750, 0.0000000000000000, -0.0020751953125000,
+0.0012817382812500, -0.0030822753906250, 0.0010986328125000, -0.0015869140625000,
+0.0010986328125000, -0.0021057128906250, 0.0022277832031250, -0.0021362304687500,
+0.0014953613281250, -0.0010986328125000, 0.0005493164062500, -0.0018310546875000,
+0.0019226074218750, -0.0016174316406250, 0.0003967285156250, -0.0014953613281250,
+0.0006408691406250, -0.0016174316406250, 0.0012512207031250, -0.0017089843750000,
+0.0005798339843750, -0.0014953613281250, -0.0004577636718750, -0.0005187988281250,
+-0.0011901855468750, -0.0003051757812500, -0.0013732910156250, -0.0007324218750000,
+-0.0006103515625000, -0.0005798339843750, -0.0000915527343750, 0.0001220703125000,
+-0.0006408691406250, -0.0008544921875000, -0.0000915527343750, -0.0004577636718750,
+0.0001831054687500, -0.0000610351562500, -0.0004882812500000, -0.0004272460937500,
+-0.0007019042968750, 0.0000610351562500, -0.0001831054687500, 0.0001220703125000,
+-0.0012207031250000, 0.0006713867187500, -0.0009460449218750, 0.0003967285156250,
+-0.0006408691406250, 0.0002746582031250, -0.0009765625000000, 0.0010986328125000,
+-0.0004577636718750, -0.0003662109375000, -0.0005493164062500, 0.0001220703125000,
+-0.0001525878906250, 0.0001831054687500, -0.0000610351562500, -0.0007324218750000,
+-0.0003662109375000, -0.0001220703125000, -0.0000915527343750, -0.0000610351562500,
+0.0001525878906250, -0.0001525878906250, -0.0001220703125000, -0.0001831054687500,
+0.0002441406250000, -0.0000305175781250, 0.0008850097656250, -0.0000305175781250,
+0.0007324218750000, -0.0007629394531250, 0.0013122558593750, -0.0005798339843750,
+0.0019226074218750, -0.0009155273437500, 0.0018310546875000, -0.0016479492187500,
+0.0018310546875000, -0.0013732910156250, 0.0023803710937500, -0.0017089843750000,
+0.0021057128906250, -0.0019836425781250, 0.0024414062500000, -0.0014953613281250,
+0.0028381347656250, -0.0011596679687500, 0.0028381347656250, -0.0015258789062500,
+0.0029602050781250, -0.0001831054687500, 0.0031127929687500, -0.0002441406250000,
+0.0029602050781250, -0.0002441406250000, 0.0028381347656250, 0.0004882812500000,
+0.0027160644531250, 0.0003356933593750, 0.0024414062500000, 0.0000610351562500,
+0.0024719238281250, 0.0002746582031250, 0.0020446777343750, -0.0000915527343750,
+0.0019836425781250, -0.0006103515625000, 0.0021362304687500, -0.0004577636718750,
+0.0018005371093750, -0.0012207031250000, 0.0018005371093750, -0.0012817382812500,
+0.0020446777343750, -0.0012817382812500, 0.0016784667968750, -0.0016479492187500,
+0.0015869140625000, -0.0014953613281250, 0.0021057128906250, -0.0013427734375000,
+0.0013427734375000, -0.0013122558593750, 0.0019226074218750, -0.0010070800781250,
+0.0019836425781250, -0.0006713867187500, 0.0014343261718750, -0.0004882812500000,
+0.0021057128906250, -0.0000610351562500, 0.0013732910156250, 0.0001831054687500,
+0.0014343261718750, 0.0003662109375000, 0.0014343261718750, 0.0006713867187500,
+0.0008850097656250, 0.0008239746093750, 0.0008850097656250, 0.0008544921875000,
+0.0006713867187500, 0.0010375976562500, 0.0004272460937500, 0.0009460449218750,
+0.0001831054687500, 0.0007019042968750, 0.0000915527343750, 0.0007629394531250,
+0.0000000000000000, 0.0005493164062500, -0.0004882812500000, 0.0001220703125000,
+-0.0003051757812500, 0.0000305175781250, -0.0005493164062500, -0.0001525878906250,
+-0.0009765625000000, -0.0007629394531250, -0.0006103515625000, -0.0003356933593750,
+-0.0010070800781250, -0.0009765625000000, -0.0009460449218750, -0.0009460449218750,
+-0.0009460449218750, -0.0005187988281250, -0.0011596679687500, -0.0012512207031250,
+-0.0008239746093750, -0.0003967285156250, -0.0012207031250000, -0.0007324218750000,
+-0.0011291503906250, -0.0004882812500000, -0.0010070800781250, 0.0001831054687500,
+-0.0012207031250000, -0.0003356933593750, -0.0010375976562500, 0.0006713867187500,
+-0.0009765625000000, 0.0004882812500000, -0.0008544921875000, 0.0006713867187500,
+-0.0007629394531250, 0.0013427734375000, -0.0007324218750000, 0.0008544921875000,
+-0.0005187988281250, 0.0017700195312500, -0.0004882812500000, 0.0014038085937500,
+-0.0003356933593750, 0.0016479492187500, -0.0001525878906250, 0.0023498535156250,
+-0.0000915527343750, 0.0017089843750000, 0.0000610351562500, 0.0026550292968750,
+0.0003051757812500, 0.0025024414062500, 0.0003662109375000, 0.0024719238281250,
+0.0005493164062500, 0.0031127929687500, 0.0007324218750000, 0.0026855468750000,
+0.0005798339843750, 0.0032043457031250, 0.0008239746093750, 0.0031433105468750,
+0.0008850097656250, 0.0030822753906250, 0.0007629394531250, 0.0033569335937500,
+0.0008239746093750, 0.0032043457031250, 0.0008544921875000, 0.0032653808593750,
+0.0005798339843750, 0.0033264160156250, 0.0006103515625000, 0.0032653808593750,
+0.0004882812500000},
+{0.0006103515625000, -0.0010681152343750, 0.0004577636718750, -0.0021362304687500,
+-0.0000305175781250, -0.0016479492187500, 0.0003662109375000, -0.0018005371093750,
+0.0001525878906250, -0.0018920898437500, -0.0000915527343750, -0.0015869140625000,
+0.0003662109375000, -0.0013122558593750, 0.0008239746093750, -0.0010681152343750,
+0.0006408691406250, -0.0007629394531250, 0.0012817382812500, -0.0003662109375000,
+0.0016174316406250, -0.0003967285156250, 0.0016174316406250, -0.0003967285156250,
+0.0019531250000000, -0.0001220703125000, 0.0023498535156250, -0.0008239746093750,
+0.0022583007812500, -0.0009460449218750, 0.0025634765625000, -0.0009460449218750,
+0.0027770996093750, -0.0013732910156250, 0.0027770996093750, -0.0016479492187500,
+0.0028381347656250, -0.0014343261718750, 0.0027770996093750, -0.0017700195312500,
+0.0028381347656250, -0.0015563964843750, 0.0025939941406250, -0.0010986328125000,
+0.0022583007812500, -0.0013122558593750, 0.0022888183593750, -0.0005187988281250,
+0.0020141601562500, -0.0002441406250000, 0.0017700195312500, -0.0002136230468750,
+0.0017089843750000, 0.0003356933593750, 0.0016784667968750, 0.0007934570312500,
+0.0016784667968750, 0.0006103515625000, 0.0013427734375000, 0.0011901855468750,
+0.0017395019531250, 0.0013427734375000, 0.0015563964843750, 0.0010986328125000,
+0.0012512207031250, 0.0016174316406250, 0.0020141601562500, 0.0013732910156250,
+0.0013732910156250, 0.0014953613281250, 0.0017700195312500, 0.0014343261718750,
+0.0018920898437500, 0.0011901855468750, 0.0014648437500000, 0.0012207031250000,
+0.0018615722656250, 0.0009155273437500, 0.0013427734375000, 0.0007629394531250,
+0.0013427734375000, 0.0007019042968750, 0.0012207031250000, 0.0004882812500000,
+0.0007019042968750, 0.0003051757812500, 0.0007629394531250, 0.0004577636718750,
+0.0002136230468750, 0.0001525878906250, 0.0002136230468750, 0.0001831054687500,
+-0.0001831054687500, 0.0004272460937500, -0.0005493164062500, 0.0002136230468750,
+-0.0001525878906250, 0.0005798339843750, -0.0009765625000000, 0.0005798339843750,
+-0.0007629394531250, 0.0006713867187500, -0.0007019042968750, 0.0010070800781250,
+-0.0011291503906250, 0.0009765625000000, -0.0007324218750000, 0.0011596679687500,
+-0.0008850097656250, 0.0011901855468750, -0.0007934570312500, 0.0012207031250000,
+-0.0007324218750000, 0.0013427734375000, -0.0007629394531250, 0.0012207031250000,
+-0.0007019042968750, 0.0012207031250000, -0.0007324218750000, 0.0012207031250000,
+-0.0007019042968750, 0.0010986328125000, -0.0006713867187500, 0.0009765625000000,
+-0.0007019042968750, 0.0008239746093750, -0.0006103515625000, 0.0009155273437500,
+-0.0006713867187500, 0.0006408691406250, -0.0005187988281250, 0.0005798339843750,
+-0.0003967285156250, 0.0007629394531250, -0.0004577636718750, 0.0005798339843750,
+-0.0002441406250000, 0.0005493164062500, -0.0000915527343750, 0.0006103515625000,
+-0.0000610351562500, 0.0004272460937500, 0.0001525878906250, 0.0002746582031250,
+0.0003662109375000, 0.0002746582031250, 0.0001831054687500, 0.0000000000000000,
+0.0004882812500000, -0.0000305175781250, 0.0003662109375000, -0.0001831054687500,
+0.0003662109375000, -0.0004272460937500, 0.0005187988281250, -0.0004272460937500,
+0.0002746582031250, -0.0005798339843750, 0.0003662109375000, -0.0007324218750000,
+0.0002746582031250, -0.0007629394531250, 0.0001831054687500, -0.0008544921875000,
+0.0001831054687500, -0.0010375976562500, 0.0000610351562500, -0.0008544921875000,
+0.0000610351562500, -0.0011901855468750, 0.0000305175781250, -0.0010986328125000,
+-0.0000610351562500, -0.0008544921875000, -0.0000915527343750, -0.0013122558593750,
+0.0000610351562500, -0.0008544921875000, -0.0003662109375000, -0.0009765625000000,
+-0.0001831054687500, -0.0009765625000000, -0.0001831054687500, -0.0005493164062500,
+-0.0005493164062500, -0.0007019042968750, -0.0002746582031250, -0.0004577636718750,
+-0.0003662109375000, -0.0004272460937500, -0.0004882812500000, -0.0005493164062500,
+-0.0002441406250000, -0.0004882812500000, -0.0000915527343750, -0.0006103515625000,
+-0.0003662109375000, -0.0007629394531250, 0.0002746582031250, -0.0010375976562500,
+0.0000915527343750, -0.0012207031250000, 0.0003662109375000, -0.0014038085937500,
+0.0009765625000000, -0.0017089843750000, 0.0007019042968750, -0.0018615722656250,
+0.0014648437500000, -0.0021362304687500, 0.0014343261718750, -0.0023498535156250,
+0.0015869140625000, -0.0025329589843750, 0.0021057128906250, -0.0028076171875000,
+0.0017395019531250, -0.0027465820312500, 0.0020751953125000, -0.0031433105468750,
+0.0018615722656250, -0.0030212402343750, 0.0016479492187500, -0.0029907226562500,
+0.0014648437500000, -0.0035400390625000, 0.0009765625000000, -0.0023193359375000,
+0.0012512207031250, -0.0030212402343750, 0.0004882812500000, -0.0026550292968750,
+0.0004882812500000, -0.0016479492187500, 0.0007629394531250, -0.0028076171875000,
+-0.0001831054687500, -0.0013427734375000, 0.0005493164062500, -0.0017089843750000,
+0.0001220703125000, -0.0017395019531250, 0.0000000000000000, -0.0008544921875000,
+0.0005187988281250, -0.0014038085937500, 0.0002136230468750, -0.0008544921875000,
+0.0004577636718750, -0.0004882812500000, 0.0007324218750000, -0.0004577636718750,
+0.0007019042968750, -0.0001525878906250, 0.0007019042968750, 0.0003662109375000,
+0.0008850097656250, 0.0005798339843750, 0.0006713867187500, 0.0011596679687500,
+0.0008239746093750, 0.0014038085937500, 0.0007629394531250, 0.0017700195312500,
+0.0005187988281250, 0.0027465820312500, 0.0006408691406250, 0.0024414062500000,
+0.0010375976562500, 0.0033264160156250, 0.0009155273437500, 0.0033874511718750,
+0.0010986328125000, 0.0029296875000000, 0.0014953613281250, 0.0037231445312500,
+0.0013732910156250, 0.0027770996093750, 0.0016174316406250, 0.0029602050781250,
+0.0018310546875000, 0.0029296875000000, 0.0018310546875000, 0.0021667480468750,
+0.0018615722656250, 0.0020141601562500, 0.0018920898437500, 0.0017700195312500,
+0.0017089843750000, 0.0017089843750000, 0.0016479492187500, 0.0012207031250000,
+0.0016479492187500, 0.0008544921875000, 0.0013732910156250, 0.0010681152343750,
+0.0011291503906250, -0.0001831054687500, 0.0008239746093750, 0.0000610351562500,
+0.0004272460937500, -0.0003662109375000, 0.0003051757812500, -0.0014038085937500,
+0.0000610351562500, -0.0009765625000000, -0.0004272460937500, -0.0015869140625000,
+0.0000915527343750, -0.0018615722656250, -0.0003967285156250, -0.0015869140625000,
+-0.0005187988281250, -0.0014648437500000, -0.0000305175781250, -0.0016174316406250,
+-0.0002746582031250, -0.0013122558593750, -0.0001220703125000, -0.0015258789062500,
+-0.0003051757812500, -0.0012512207031250, -0.0003662109375000, -0.0008239746093750,
+-0.0003356933593750, -0.0011596679687500, -0.0007019042968750, -0.0002136230468750,
+-0.0002746582031250, -0.0001220703125000, -0.0006713867187500, 0.0001220703125000,
+-0.0005798339843750, 0.0007324218750000, -0.0001525878906250, 0.0006103515625000,
+-0.0006713867187500, 0.0014038085937500, 0.0000305175781250, 0.0012512207031250,
+-0.0002136230468750, 0.0015869140625000, -0.0001220703125000, 0.0022277832031250,
+0.0003967285156250, 0.0018005371093750, -0.0001220703125000, 0.0027770996093750,
+0.0005187988281250, 0.0028076171875000, 0.0006408691406250, 0.0025634765625000,
+0.0004577636718750, 0.0030212402343750, 0.0006713867187500, 0.0030212402343750,
+0.0006408691406250, 0.0025024414062500, 0.0003051757812500, 0.0026550292968750,
+0.0003967285156250, 0.0025024414062500, 0.0002136230468750, 0.0020446777343750,
+-0.0001220703125000, 0.0023193359375000, -0.0000610351562500, 0.0017700195312500,
+-0.0003051757812500, 0.0019836425781250, -0.0002441406250000, 0.0017089843750000,
+-0.0003051757812500, 0.0011596679687500, -0.0004882812500000, 0.0014648437500000,
+-0.0004577636718750, 0.0006103515625000, -0.0009155273437500, 0.0007629394531250,
+-0.0007629394531250, 0.0006713867187500, -0.0010070800781250, 0.0001525878906250,
+-0.0014038085937500, 0.0005798339843750, -0.0011596679687500, 0.0003662109375000,
+-0.0014648437500000, 0.0003356933593750, -0.0014953613281250, 0.0006408691406250,
+-0.0012817382812500, 0.0006103515625000, -0.0011596679687500, 0.0003356933593750,
+-0.0009155273437500, 0.0010375976562500, -0.0006713867187500, 0.0005187988281250,
+-0.0002441406250000, 0.0008850097656250, 0.0000000000000000, 0.0015258789062500,
+0.0002746582031250, 0.0006713867187500, 0.0006103515625000, 0.0019836425781250,
+0.0003356933593750, 0.0015869140625000, 0.0006713867187500, 0.0016174316406250,
+0.0005187988281250, 0.0025024414062500, -0.0000305175781250, 0.0017700195312500,
+0.0003356933593750, 0.0021972656250000, -0.0003051757812500, 0.0023193359375000,
+-0.0005493164062500, 0.0020751953125000, -0.0004272460937500, 0.0022277832031250,
+-0.0005798339843750, 0.0023498535156250, -0.0007324218750000, 0.0019531250000000,
+-0.0002441406250000, 0.0021972656250000, -0.0003967285156250, 0.0019531250000000,
+-0.0002441406250000, 0.0015869140625000, 0.0002441406250000, 0.0020141601562500,
+0.0000610351562500, 0.0009155273437500, 0.0004272460937500, 0.0012512207031250,
+0.0005798339843750, 0.0010070800781250, 0.0005493164062500, 0.0000610351562500,
+0.0006713867187500, 0.0006103515625000, 0.0008239746093750, 0.0000305175781250,
+0.0006103515625000, -0.0000610351562500, 0.0006103515625000, 0.0001220703125000,
+0.0006103515625000, 0.0000610351562500, 0.0004272460937500, 0.0000610351562500,
+0.0003356933593750, 0.0002441406250000, 0.0005798339843750, 0.0001831054687500,
+0.0003356933593750, 0.0003356933593750, 0.0004577636718750, 0.0004882812500000,
+0.0006713867187500, 0.0002746582031250, 0.0003662109375000, 0.0007019042968750,
+0.0010070800781250, 0.0004272460937500, 0.0008544921875000, 0.0003967285156250,
+0.0009460449218750, 0.0007019042968750, 0.0013732910156250, 0.0002746582031250,
+0.0011596679687500, 0.0003967285156250, 0.0014038085937500, 0.0003051757812500,
+0.0014648437500000, 0.0000610351562500, 0.0013122558593750, 0.0000000000000000,
+0.0013122558593750, -0.0000610351562500, 0.0013122558593750, -0.0003356933593750,
+0.0009765625000000, -0.0003662109375000, 0.0010070800781250, -0.0004882812500000,
+0.0007934570312500, -0.0007324218750000, 0.0004882812500000, -0.0006408691406250,
+0.0005493164062500, -0.0007934570312500, 0.0000915527343750, -0.0008544921875000,
+0.0000305175781250, -0.0006713867187500, -0.0001220703125000, -0.0007019042968750,
+-0.0005493164062500, -0.0007629394531250, -0.0005493164062500, -0.0003662109375000,
+-0.0008850097656250, -0.0005493164062500, -0.0010070800781250, -0.0004272460937500,
+-0.0012207031250000, -0.0001220703125000, -0.0014953613281250, -0.0005187988281250,
+-0.0015258789062500, 0.0000305175781250, -0.0017700195312500, -0.0000915527343750,
+-0.0019531250000000, -0.0001220703125000, -0.0020446777343750, 0.0002746582031250,
+-0.0022583007812500, -0.0000305175781250, -0.0024108886718750, 0.0000000000000000,
+-0.0022277832031250, 0.0001525878906250, -0.0024719238281250, -0.0001220703125000,
+-0.0022888183593750, -0.0002441406250000, -0.0019226074218750, 0.0000000000000000,
+-0.0021972656250000, -0.0002136230468750, -0.0013122558593750, -0.0001831054687500,
+-0.0016479492187500, 0.0000610351562500, -0.0011596679687500, 0.0000610351562500,
+-0.0003356933593750, 0.0002136230468750, -0.0010681152343750, 0.0006103515625000,
+0.0003051757812500, 0.0005798339843750, 0.0000610351562500, 0.0009155273437500,
+0.0001220703125000, 0.0012817382812500, 0.0009155273437500, 0.0011596679687500,
+0.0001220703125000},
+{0.0007629394531250, 0.0007934570312500, 0.0010070800781250, 0.0003662109375000,
+0.0006103515625000, 0.0003356933593750, 0.0009765625000000, 0.0006713867187500,
+0.0007019042968750, 0.0005493164062500, 0.0001220703125000, 0.0003051757812500,
+0.0006103515625000, 0.0010070800781250, -0.0002136230468750, 0.0004272460937500,
+-0.0001220703125000, 0.0007019042968750, -0.0000305175781250, 0.0013427734375000,
+-0.0003662109375000, 0.0003051757812500, -0.0000915527343750, 0.0017089843750000,
+-0.0001220703125000, 0.0011901855468750, 0.0000305175781250, 0.0011901855468750,
+-0.0000305175781250, 0.0021972656250000, 0.0000000000000000, 0.0014343261718750,
+0.0002441406250000, 0.0020141601562500, -0.0002136230468750, 0.0020141601562500,
+0.0000305175781250, 0.0018310546875000, -0.0002136230468750, 0.0020446777343750,
+-0.0006408691406250, 0.0019531250000000, -0.0001220703125000, 0.0016784667968750,
+-0.0007324218750000, 0.0018920898437500, -0.0005187988281250, 0.0014953613281250,
+-0.0003356933593750, 0.0010070800781250, -0.0005187988281250, 0.0014343261718750,
+-0.0001220703125000, 0.0003356933593750, 0.0001525878906250, 0.0003967285156250,
+0.0001525878906250, 0.0003967285156250, 0.0007629394531250, -0.0002441406250000,
+0.0011901855468750, -0.0001525878906250, 0.0009765625000000, 0.0001831054687500,
+0.0021667480468750, -0.0002441406250000, 0.0019836425781250, 0.0000000000000000,
+0.0021972656250000, 0.0003356933593750, 0.0029602050781250, -0.0002441406250000,
+0.0024414062500000, 0.0004882812500000, 0.0028991699218750, 0.0001831054687500,
+0.0029602050781250, 0.0000610351562500, 0.0026550292968750, 0.0004882812500000,
+0.0027465820312500, 0.0001525878906250, 0.0027160644531250, 0.0000915527343750,
+0.0022888183593750, 0.0002441406250000, 0.0023498535156250, 0.0000000000000000,
+0.0021667480468750, -0.0002746582031250, 0.0018310546875000, -0.0000915527343750,
+0.0020141601562500, -0.0004577636718750, 0.0016784667968750, -0.0005493164062500,
+0.0015563964843750, -0.0004882812500000, 0.0015869140625000, -0.0007019042968750,
+0.0014648437500000, -0.0007934570312500, 0.0014038085937500, -0.0004882812500000,
+0.0016479492187500, -0.0008544921875000, 0.0014648437500000, -0.0007019042968750,
+0.0014648437500000, -0.0004577636718750, 0.0016479492187500, -0.0009765625000000,
+0.0014038085937500, -0.0003051757812500, 0.0015563964843750, -0.0006713867187500,
+0.0014648437500000, -0.0006103515625000, 0.0012817382812500, -0.0001220703125000,
+0.0012817382812500, -0.0006408691406250, 0.0011901855468750, 0.0000305175781250,
+0.0010375976562500, -0.0000610351562500, 0.0010375976562500, -0.0002441406250000,
+0.0008850097656250, 0.0000000000000000, 0.0007019042968750, -0.0001220703125000,
+0.0007629394531250, -0.0002441406250000, 0.0003967285156250, -0.0000915527343750,
+0.0004882812500000, -0.0000610351562500, 0.0002441406250000, -0.0000915527343750,
+-0.0000915527343750, 0.0001831054687500, 0.0001220703125000, 0.0001220703125000,
+-0.0003356933593750, 0.0002136230468750, -0.0002746582031250, 0.0004272460937500,
+-0.0003967285156250, 0.0004577636718750, -0.0007019042968750, 0.0006103515625000,
+-0.0004272460937500, 0.0009155273437500, -0.0006408691406250, 0.0007934570312500,
+-0.0008544921875000, 0.0011596679687500, -0.0005798339843750, 0.0014953613281250,
+-0.0005187988281250, 0.0010375976562500, -0.0008239746093750, 0.0018615722656250,
+-0.0000915527343750, 0.0014648437500000, -0.0003662109375000, 0.0013427734375000,
+-0.0000915527343750, 0.0017395019531250, 0.0006103515625000, 0.0010375976562500,
+0.0001525878906250, 0.0014343261718750, 0.0009765625000000, 0.0010681152343750,
+0.0009155273437500, 0.0008239746093750, 0.0008239746093750, 0.0009765625000000,
+0.0012817382812500, 0.0004272460937500, 0.0010681152343750, 0.0006713867187500,
+0.0008850097656250, 0.0003967285156250, 0.0009155273437500, 0.0004272460937500,
+0.0005493164062500, 0.0007019042968750, 0.0000915527343750, 0.0004882812500000,
+0.0001831054687500, 0.0008850097656250, -0.0004272460937500, 0.0009765625000000,
+-0.0006103515625000, 0.0008544921875000, -0.0006103515625000, 0.0009460449218750,
+-0.0009155273437500, 0.0008850097656250, -0.0010986328125000, 0.0006103515625000,
+-0.0009155273437500, 0.0005493164062500, -0.0010986328125000, 0.0002746582031250,
+-0.0013122558593750, 0.0000610351562500, -0.0011291503906250, 0.0002136230468750,
+-0.0010681152343750, -0.0007324218750000, -0.0016784667968750, 0.0000915527343750,
+-0.0014038085937500, -0.0004272460937500, -0.0018615722656250, -0.0011291503906250,
+-0.0025329589843750, 0.0004272460937500, -0.0020141601562500, -0.0015258789062500,
+-0.0030822753906250, -0.0008544921875000, -0.0028991699218750, -0.0006408691406250,
+-0.0029296875000000, -0.0019836425781250, -0.0035705566406250, -0.0011901855468750,
+-0.0030822753906250, -0.0012512207031250, -0.0031433105468750, -0.0019836425781250,
+-0.0033874511718750, -0.0015258789062500, -0.0029296875000000, -0.0011596679687500,
+-0.0025329589843750, -0.0021667480468750, -0.0027770996093750, -0.0010375976562500,
+-0.0017395019531250, -0.0013427734375000, -0.0016174316406250, -0.0016479492187500,
+-0.0016174316406250, -0.0010681152343750, -0.0010070800781250, -0.0013122558593750,
+-0.0008239746093750, -0.0011901855468750, -0.0008544921875000, -0.0009155273437500,
+-0.0007934570312500, -0.0009460449218750, -0.0002136230468750, -0.0008850097656250,
+-0.0000305175781250, -0.0005493164062500, -0.0001525878906250, -0.0008544921875000,
+0.0011596679687500, -0.0005798339843750, 0.0012512207031250, -0.0005187988281250,
+0.0012512207031250, -0.0007019042968750, 0.0020446777343750, -0.0003356933593750,
+0.0021667480468750, -0.0005493164062500, 0.0013732910156250, -0.0001831054687500,
+0.0017700195312500, 0.0001220703125000, 0.0016174316406250, -0.0000915527343750,
+0.0004882812500000, 0.0017395019531250, 0.0010681152343750, 0.0001525878906250,
+0.0012207031250000, 0.0009155273437500, 0.0012512207031250, 0.0008544921875000,
+0.0013122558593750, 0.0002441406250000, 0.0011291503906250, 0.0007324218750000,
+0.0009155273437500, 0.0008850097656250, 0.0014343261718750, 0.0010681152343750,
+0.0014038085937500, 0.0010070800781250, 0.0010070800781250, 0.0011291503906250,
+0.0011901855468750, 0.0015258789062500, 0.0013122558593750, 0.0003967285156250,
+-0.0001525878906250, 0.0007019042968750, 0.0002136230468750, 0.0007019042968750,
+-0.0001831054687500, -0.0001525878906250, -0.0015563964843750, 0.0002136230468750,
+-0.0011291503906250, 0.0006103515625000, -0.0015563964843750, 0.0003662109375000,
+-0.0017395019531250, 0.0007019042968750, -0.0016174316406250, 0.0014343261718750,
+-0.0016174316406250, 0.0011901855468750, -0.0016174316406250, 0.0009460449218750,
+-0.0014953613281250, 0.0015258789062500, -0.0015563964843750, 0.0009155273437500,
+-0.0012207031250000, -0.0000610351562500, -0.0010070800781250, 0.0006408691406250,
+-0.0013427734375000, -0.0003356933593750, 0.0000305175781250, -0.0004882812500000,
+-0.0002746582031250, -0.0001831054687500, 0.0000000000000000, -0.0003967285156250,
+0.0011291503906250, -0.0003356933593750, 0.0006713867187500, 0.0000610351562500,
+0.0007629394531250, 0.0001525878906250, 0.0013122558593750, 0.0001525878906250,
+0.0008850097656250, 0.0005493164062500, 0.0004577636718750, 0.0007934570312500,
+0.0010986328125000, 0.0001525878906250, 0.0004577636718750, 0.0006408691406250,
+0.0006713867187500, 0.0006103515625000, 0.0007934570312500, 0.0000305175781250,
+0.0007019042968750, 0.0005798339843750, 0.0013122558593750, 0.0004882812500000,
+0.0008850097656250, 0.0003967285156250, 0.0013427734375000, 0.0006408691406250,
+0.0014648437500000, 0.0007324218750000, 0.0010986328125000, 0.0004272460937500,
+0.0014953613281250, 0.0008239746093750, 0.0016174316406250, 0.0007019042968750,
+0.0014953613281250, 0.0002746582031250, 0.0017700195312500, 0.0003967285156250,
+0.0020446777343750, 0.0003662109375000, 0.0018310546875000, -0.0007019042968750,
+0.0018005371093750, -0.0004577636718750, 0.0019226074218750, -0.0007629394531250,
+0.0016479492187500, -0.0015563964843750, 0.0013427734375000, -0.0010681152343750,
+0.0015258789062500, -0.0015258789062500, 0.0010375976562500, -0.0014038085937500,
+0.0012207031250000, -0.0011291503906250, 0.0010375976562500, -0.0012207031250000,
+0.0008239746093750, -0.0009765625000000, 0.0013122558593750, -0.0006103515625000,
+0.0006103515625000, -0.0005798339843750, 0.0008850097656250, -0.0001220703125000,
+0.0010070800781250, 0.0003967285156250, 0.0005187988281250, 0.0002136230468750,
+0.0007934570312500, 0.0010070800781250, 0.0007934570312500, 0.0010986328125000,
+0.0005798339843750, 0.0011596679687500, 0.0006103515625000, 0.0016479492187500,
+0.0006103515625000, 0.0015563964843750, 0.0004882812500000, 0.0016784667968750,
+0.0003662109375000, 0.0018920898437500, 0.0003051757812500, 0.0018615722656250,
+0.0001220703125000, 0.0018005371093750, 0.0000000000000000, 0.0019836425781250,
+0.0000610351562500, 0.0019531250000000, -0.0002746582031250, 0.0018920898437500,
+-0.0001831054687500, 0.0019836425781250, -0.0002746582031250, 0.0019836425781250,
+-0.0004882812500000, 0.0018005371093750, -0.0003051757812500, 0.0018920898437500,
+-0.0003356933593750, 0.0018005371093750, -0.0004272460937500, 0.0016174316406250,
+-0.0003051757812500, 0.0015869140625000, -0.0003051757812500, 0.0013732910156250,
+-0.0004272460937500, 0.0012512207031250, -0.0002441406250000, 0.0011596679687500,
+-0.0003662109375000, 0.0009460449218750, -0.0004577636718750, 0.0007629394531250,
+-0.0004577636718750, 0.0007324218750000, -0.0006713867187500, 0.0004272460937500,
+-0.0005493164062500, 0.0003356933593750, -0.0007629394531250, 0.0002136230468750,
+-0.0007019042968750, 0.0000915527343750, -0.0005187988281250, 0.0000610351562500,
+-0.0009460449218750, -0.0000915527343750, -0.0002746582031250, -0.0000915527343750,
+-0.0005798339843750, -0.0000915527343750, -0.0005187988281250, -0.0002136230468750,
+0.0000305175781250, -0.0003051757812500, -0.0004272460937500, -0.0001831054687500,
+0.0000610351562500, -0.0002746582031250, 0.0001220703125000, -0.0002441406250000,
+-0.0000610351562500, -0.0000915527343750, 0.0000610351562500, -0.0002136230468750,
+0.0001220703125000, 0.0000305175781250, -0.0002441406250000, -0.0000610351562500,
+-0.0000305175781250, -0.0000305175781250, 0.0000000000000000, 0.0001525878906250,
+-0.0002441406250000, 0.0000305175781250, 0.0000000000000000, 0.0001525878906250,
+0.0000305175781250, 0.0002136230468750, -0.0000610351562500, 0.0001220703125000,
+0.0001525878906250, 0.0001831054687500, 0.0003662109375000, 0.0002746582031250,
+0.0002441406250000, -0.0000305175781250, 0.0007629394531250, 0.0003051757812500,
+0.0007324218750000, 0.0001220703125000, 0.0008850097656250, -0.0000915527343750,
+0.0012207031250000, 0.0006713867187500, 0.0012207031250000, -0.0003967285156250,
+0.0014038085937500, 0.0005798339843750, 0.0014648437500000, 0.0003662109375000,
+0.0015563964843750, -0.0005493164062500, 0.0016174316406250, 0.0009460449218750,
+0.0014343261718750, -0.0007324218750000, 0.0014648437500000, -0.0001525878906250,
+0.0010681152343750, -0.0000610351562500, 0.0010070800781250, -0.0010375976562500,
+0.0009460449218750, 0.0000305175781250, 0.0003662109375000, -0.0008544921875000,
+0.0008544921875000, -0.0004577636718750, 0.0003356933593750, -0.0003051757812500,
+0.0002136230468750, -0.0007629394531250, 0.0006103515625000, -0.0001220703125000,
+0.0000915527343750, -0.0005493164062500, 0.0004272460937500, -0.0001831054687500,
+0.0005187988281250},
+{0.0011596679687500, 0.0007019042968750, 0.0013427734375000, 0.0006713867187500,
+0.0010070800781250, 0.0015869140625000, 0.0006408691406250, 0.0015563964843750,
+0.0009765625000000, 0.0012512207031250, 0.0004577636718750, 0.0019836425781250,
+0.0006713867187500, 0.0015563964843750, 0.0006713867187500, 0.0008850097656250,
+0.0002441406250000, 0.0018005371093750, 0.0006103515625000, 0.0004272460937500,
+0.0003662109375000, 0.0007629394531250, 0.0002136230468750, 0.0007324218750000,
+0.0004272460937500, -0.0001220703125000, 0.0004272460937500, 0.0003662109375000,
+0.0001220703125000, -0.0000610351562500, 0.0005493164062500, -0.0001525878906250,
+0.0003356933593750, 0.0001220703125000, 0.0002441406250000, 0.0000305175781250,
+0.0004577636718750, -0.0001220703125000, 0.0001831054687500, 0.0003051757812500,
+0.0001831054687500, 0.0000000000000000, 0.0001831054687500, 0.0000000000000000,
+0.0000915527343750, 0.0003051757812500, -0.0000305175781250, -0.0002441406250000,
+0.0000000000000000, 0.0001831054687500, -0.0000915527343750, -0.0000305175781250,
+-0.0001525878906250, -0.0001220703125000, -0.0001525878906250, 0.0001525878906250,
+-0.0002136230468750, -0.0002136230468750, -0.0003967285156250, 0.0001525878906250,
+-0.0001525878906250, -0.0000915527343750, -0.0004272460937500, 0.0000915527343750,
+-0.0003967285156250, 0.0004272460937500, -0.0002441406250000, 0.0000305175781250,
+-0.0007019042968750, 0.0008239746093750, -0.0003051757812500, 0.0006408691406250,
+-0.0006103515625000, 0.0008544921875000, -0.0006408691406250, 0.0014953613281250,
+-0.0003967285156250, 0.0010986328125000, -0.0008239746093750, 0.0018615722656250,
+-0.0004272460937500, 0.0017089843750000, -0.0006408691406250, 0.0019226074218750,
+-0.0007019042968750, 0.0024108886718750, -0.0004272460937500, 0.0019226074218750,
+-0.0006103515625000, 0.0026855468750000, -0.0004882812500000, 0.0024108886718750,
+-0.0005187988281250, 0.0024414062500000, -0.0004882812500000, 0.0029602050781250,
+-0.0004272460937500, 0.0024414062500000, -0.0004272460937500, 0.0031127929687500,
+-0.0001831054687500, 0.0028991699218750, -0.0002746582031250, 0.0028991699218750,
+-0.0001220703125000, 0.0032043457031250, 0.0001525878906250, 0.0028076171875000,
+0.0000305175781250, 0.0030517578125000, 0.0005493164062500, 0.0030517578125000,
+0.0005493164062500, 0.0030517578125000, 0.0006408691406250, 0.0032348632812500,
+0.0009155273437500, 0.0030212402343750, 0.0009155273437500, 0.0032043457031250,
+0.0009765625000000, 0.0032653808593750, 0.0011901855468750, 0.0032348632812500,
+0.0012512207031250, 0.0034179687500000, 0.0012512207031250, 0.0035705566406250,
+0.0013427734375000, 0.0035400390625000, 0.0013732910156250, 0.0038757324218750,
+0.0012512207031250, 0.0039062500000000, 0.0012207031250000, 0.0038452148437500,
+0.0010986328125000, 0.0043334960937500, 0.0007934570312500, 0.0039978027343750,
+0.0007629394531250, 0.0042114257812500, 0.0003662109375000, 0.0041503906250000,
+0.0002136230468750, 0.0037841796875000, 0.0002441406250000, 0.0039978027343750,
+-0.0000610351562500, 0.0033569335937500, 0.0000305175781250, 0.0033874511718750,
+0.0000610351562500, 0.0030212402343750, -0.0000915527343750, 0.0023803710937500,
+0.0000305175781250, 0.0026550292968750, 0.0003662109375000, 0.0016784667968750,
+-0.0000305175781250, 0.0017700195312500, 0.0006408691406250, 0.0015563964843750,
+0.0006103515625000, 0.0009460449218750, 0.0002441406250000, 0.0012512207031250,
+0.0011291503906250, 0.0008239746093750, 0.0004577636718750, 0.0007019042968750,
+0.0006408691406250, 0.0007324218750000, 0.0009155273437500, 0.0004272460937500,
+0.0006103515625000, 0.0002746582031250, 0.0007324218750000, 0.0002746582031250,
+0.0009155273437500, -0.0000305175781250, 0.0007019042968750, -0.0002136230468750,
+0.0008544921875000, -0.0002136230468750, 0.0010681152343750, -0.0007019042968750,
+0.0007629394531250, -0.0009460449218750, 0.0012207031250000, -0.0010681152343750,
+0.0011901855468750, -0.0012207031250000, 0.0011901855468750, -0.0013427734375000,
+0.0014343261718750, -0.0012817382812500, 0.0012817382812500, -0.0009155273437500,
+0.0013427734375000, -0.0008544921875000, 0.0012817382812500, -0.0007934570312500,
+0.0012207031250000, -0.0005187988281250, 0.0010986328125000, -0.0003356933593750,
+0.0009155273437500, -0.0006408691406250, 0.0007629394531250, -0.0002746582031250,
+0.0003967285156250, -0.0004272460937500, 0.0003051757812500, -0.0007934570312500,
+0.0003356933593750, -0.0004272460937500, 0.0000000000000000, -0.0009155273437500,
+-0.0000305175781250, -0.0015258789062500, -0.0002441406250000, -0.0014343261718750,
+-0.0004577636718750, -0.0016174316406250, -0.0005187988281250, -0.0024414062500000,
+-0.0006103515625000, -0.0019836425781250, -0.0005493164062500, -0.0026550292968750,
+-0.0007324218750000, -0.0028076171875000, -0.0007019042968750, -0.0025329589843750,
+-0.0005493164062500, -0.0033264160156250, -0.0007324218750000, -0.0023803710937500,
+-0.0006103515625000, -0.0028381347656250, -0.0007019042968750, -0.0027770996093750,
+-0.0007019042968750, -0.0014953613281250, -0.0006713867187500, -0.0007019042968750,
+-0.0005798339843750, 0.0006103515625000, -0.0001831054687500, 0.0002136230468750,
+-0.0003967285156250, 0.0003967285156250, -0.0003662109375000, 0.0014038085937500,
+0.0000000000000000, 0.0010375976562500, -0.0001525878906250, 0.0015258789062500,
+0.0000305175781250, 0.0016479492187500, 0.0002441406250000, 0.0015563964843750,
+0.0001525878906250, 0.0015563964843750, 0.0001220703125000, 0.0014343261718750,
+0.0003356933593750, 0.0010681152343750, -0.0000305175781250, 0.0012207031250000,
+0.0001220703125000, 0.0006408691406250, 0.0000915527343750, 0.0000915527343750,
+-0.0000305175781250, 0.0006713867187500, 0.0001831054687500, -0.0002441406250000,
+-0.0001220703125000, -0.0002441406250000, 0.0001525878906250, -0.0000305175781250,
+0.0000915527343750, -0.0003356933593750, -0.0001220703125000, -0.0002441406250000,
+0.0003051757812500, -0.0001220703125000, -0.0002441406250000, -0.0001220703125000,
+-0.0000610351562500, 0.0000305175781250, 0.0000000000000000, 0.0000915527343750,
+-0.0002136230468750, 0.0000610351562500, 0.0000305175781250, 0.0008239746093750,
+0.0000000000000000, 0.0006103515625000, 0.0000915527343750, 0.0009765625000000,
+0.0000915527343750, 0.0018005371093750, 0.0000000000000000, 0.0015258789062500,
+0.0000305175781250, 0.0021667480468750, -0.0004272460937500, 0.0022277832031250,
+-0.0003967285156250, 0.0021972656250000, -0.0006103515625000, 0.0024108886718750,
+-0.0010375976562500, 0.0022277832031250, -0.0008850097656250, 0.0023193359375000,
+-0.0012207031250000, 0.0021362304687500, -0.0011901855468750, 0.0019836425781250,
+-0.0012512207031250, 0.0019836425781250, -0.0013427734375000, 0.0017395019531250,
+-0.0010986328125000, 0.0014953613281250, -0.0012817382812500, 0.0016784667968750,
+-0.0009765625000000, 0.0010375976562500, -0.0010070800781250, 0.0006408691406250,
+-0.0011901855468750, 0.0011901855468750, -0.0007934570312500, -0.0001525878906250,
+-0.0011291503906250, 0.0002746582031250, -0.0009155273437500, 0.0001525878906250,
+-0.0007934570312500, -0.0007324218750000, -0.0008544921875000, -0.0000610351562500,
+-0.0005493164062500, -0.0006103515625000, -0.0004272460937500, -0.0006408691406250,
+-0.0002136230468750, -0.0003967285156250, 0.0001220703125000, -0.0007019042968750,
+0.0004272460937500, -0.0007934570312500, 0.0007629394531250, -0.0002441406250000,
+0.0006103515625000, -0.0007629394531250, 0.0011596679687500, -0.0003967285156250,
+0.0010070800781250, 0.0001525878906250, 0.0007019042968750, -0.0005798339843750,
+0.0012817382812500, 0.0007629394531250, 0.0003356933593750, 0.0004577636718750,
+0.0005187988281250, 0.0006408691406250, 0.0003356933593750, 0.0015258789062500,
+-0.0003051757812500, 0.0009155273437500, -0.0000305175781250, 0.0016479492187500,
+-0.0004882812500000, 0.0015258789062500, -0.0006713867187500, 0.0014038085937500,
+-0.0006408691406250, 0.0016784667968750, -0.0006713867187500, 0.0013732910156250,
+-0.0007019042968750, 0.0010681152343750, -0.0006103515625000, 0.0011901855468750,
+-0.0004577636718750, 0.0007324218750000, -0.0003051757812500, 0.0001831054687500,
+-0.0001220703125000, 0.0005187988281250, 0.0002136230468750, -0.0002746582031250,
+0.0004272460937500, -0.0003356933593750, 0.0008850097656250, -0.0003051757812500,
+0.0009460449218750, -0.0007324218750000, 0.0009765625000000, -0.0007324218750000,
+0.0014038085937500, -0.0005493164062500, 0.0012207031250000, -0.0008239746093750,
+0.0014648437500000, -0.0006408691406250, 0.0015258789062500, -0.0004577636718750,
+0.0013732910156250, -0.0008239746093750, 0.0017395019531250, -0.0001220703125000,
+0.0013732910156250, -0.0005187988281250, 0.0016479492187500, -0.0004272460937500,
+0.0017089843750000, 0.0001220703125000, 0.0014953613281250, -0.0004577636718750,
+0.0019836425781250, 0.0001220703125000, 0.0017089843750000, -0.0000915527343750,
+0.0020446777343750, -0.0001220703125000, 0.0021972656250000, 0.0001831054687500,
+0.0020751953125000, -0.0001525878906250, 0.0025634765625000, 0.0000305175781250,
+0.0025939941406250, -0.0000915527343750, 0.0028381347656250, -0.0001831054687500,
+0.0031738281250000, -0.0001831054687500, 0.0032958984375000, -0.0003967285156250,
+0.0036315917968750, -0.0001831054687500, 0.0037536621093750, -0.0003662109375000,
+0.0039367675781250, -0.0002746582031250, 0.0040893554687500, -0.0000610351562500,
+0.0042419433593750, -0.0003662109375000, 0.0043640136718750, 0.0000305175781250,
+0.0042114257812500, -0.0002136230468750, 0.0043945312500000, -0.0001220703125000,
+0.0042419433593750, 0.0001220703125000, 0.0039367675781250, -0.0001831054687500,
+0.0040588378906250, 0.0003662109375000, 0.0035095214843750, 0.0001220703125000,
+0.0034484863281250, 0.0002746582031250, 0.0032653808593750, 0.0006408691406250,
+0.0028381347656250, 0.0000000000000000, 0.0028686523437500, 0.0005187988281250,
+0.0025939941406250, 0.0002441406250000, 0.0024108886718750, -0.0000305175781250,
+0.0025024414062500, 0.0002136230468750, 0.0024719238281250, -0.0002136230468750,
+0.0024414062500000, -0.0000610351562500, 0.0028686523437500, -0.0002746582031250,
+0.0027160644531250, -0.0002746582031250, 0.0028686523437500, -0.0001220703125000,
+0.0032958984375000, -0.0002746582031250, 0.0029907226562500, 0.0000305175781250,
+0.0031738281250000, 0.0002441406250000, 0.0032348632812500, 0.0002746582031250,
+0.0028381347656250, 0.0006408691406250, 0.0027160644531250, 0.0011291503906250,
+0.0027465820312500, 0.0006103515625000, 0.0021667480468750, 0.0012817382812500,
+0.0021362304687500, 0.0012512207031250, 0.0018920898437500, 0.0007019042968750,
+0.0014038085937500, 0.0014648437500000, 0.0013122558593750, 0.0006408691406250,
+0.0010681152343750, 0.0010681152343750, 0.0008239746093750, 0.0007324218750000,
+0.0009155273437500, 0.0001525878906250, 0.0009765625000000, 0.0011901855468750,
+0.0007934570312500, -0.0004882812500000, 0.0011596679687500, 0.0006408691406250,
+0.0011596679687500, 0.0005187988281250, 0.0010375976562500, -0.0006713867187500,
+0.0012207031250000, 0.0010681152343750, 0.0013427734375000, -0.0003051757812500,
+0.0007019042968750, 0.0000610351562500, 0.0009765625000000, 0.0006103515625000,
+0.0006713867187500, -0.0001220703125000, 0.0000000000000000, 0.0001831054687500,
+0.0005493164062500, 0.0006408691406250, -0.0001525878906250, -0.0001525878906250,
+-0.0000305175781250, 0.0002441406250000, -0.0000610351562500, 0.0008544921875000,
+-0.0003967285156250},
+{0.0017395019531250, 0.0006103515625000, 0.0014343261718750, 0.0011596679687500,
+0.0021667480468750, 0.0000915527343750, 0.0011901855468750, 0.0002746582031250,
+0.0017395019531250, 0.0002746582031250, 0.0016784667968750, -0.0004577636718750,
+0.0009765625000000, -0.0002136230468750, 0.0017395019531250, -0.0002746582031250,
+0.0009765625000000, -0.0007629394531250, 0.0013732910156250, -0.0005187988281250,
+0.0014648437500000, -0.0003662109375000, 0.0010986328125000, -0.0011291503906250,
+0.0017700195312500, -0.0001525878906250, 0.0013122558593750, -0.0007324218750000,
+0.0016174316406250, -0.0006713867187500, 0.0018005371093750, 0.0001220703125000,
+0.0016174316406250, -0.0007019042968750, 0.0020751953125000, 0.0000915527343750,
+0.0019226074218750, -0.0000610351562500, 0.0021667480468750, -0.0002441406250000,
+0.0022888183593750, 0.0001525878906250, 0.0022888183593750, -0.0001220703125000,
+0.0024414062500000, 0.0000305175781250, 0.0024414062500000, 0.0000305175781250,
+0.0025329589843750, -0.0000305175781250, 0.0027465820312500, -0.0000305175781250,
+0.0027770996093750, -0.0000305175781250, 0.0028076171875000, 0.0000305175781250,
+0.0031433105468750, -0.0000305175781250, 0.0029907226562500, 0.0000305175781250,
+0.0032348632812500, 0.0001525878906250, 0.0035095214843750, -0.0000915527343750,
+0.0032043457031250, 0.0002441406250000, 0.0038757324218750, 0.0001831054687500,
+0.0037231445312500, 0.0002136230468750, 0.0037536621093750, 0.0004272460937500,
+0.0041503906250000, 0.0002746582031250, 0.0038452148437500, 0.0005798339843750,
+0.0042114257812500, 0.0002746582031250, 0.0040893554687500, 0.0002136230468750,
+0.0038146972656250, 0.0003967285156250, 0.0038146972656250, -0.0000915527343750,
+0.0036926269531250, 0.0000000000000000, 0.0031433105468750, -0.0001220703125000,
+0.0031127929687500, -0.0002746582031250, 0.0026855468750000, -0.0003356933593750,
+0.0021057128906250, -0.0004577636718750, 0.0022583007812500, -0.0003662109375000,
+0.0012817382812500, -0.0003967285156250, 0.0012207031250000, -0.0002746582031250,
+0.0010986328125000, -0.0000610351562500, 0.0005493164062500, 0.0000610351562500,
+0.0005493164062500, 0.0002441406250000, 0.0006103515625000, 0.0005798339843750,
+0.0002441406250000, 0.0007934570312500, 0.0007019042968750, 0.0009155273437500,
+0.0010986328125000, 0.0013427734375000, 0.0005187988281250, 0.0011901855468750,
+0.0017700195312500, 0.0014953613281250, 0.0014953613281250, 0.0014343261718750,
+0.0016784667968750, 0.0012817382812500, 0.0026245117187500, 0.0017089843750000,
+0.0021057128906250, 0.0006103515625000, 0.0030212402343750, 0.0013427734375000,
+0.0028686523437500, 0.0009765625000000, 0.0028686523437500, 0.0001525878906250,
+0.0033264160156250, 0.0015258789062500, 0.0028686523437500, 0.0002746582031250,
+0.0031738281250000, 0.0008850097656250, 0.0029296875000000, 0.0012207031250000,
+0.0030212402343750, 0.0006408691406250, 0.0032958984375000, 0.0013427734375000,
+0.0028076171875000, 0.0011291503906250, 0.0037536621093750, 0.0008239746093750,
+0.0035705566406250, 0.0012512207031250, 0.0034484863281250, 0.0011901855468750,
+0.0039978027343750, 0.0003967285156250, 0.0037536621093750, 0.0018005371093750,
+0.0033569335937500, 0.0009460449218750, 0.0036926269531250, 0.0010681152343750,
+0.0031127929687500, 0.0022583007812500, 0.0025329589843750, 0.0013427734375000,
+0.0030822753906250, 0.0021667480468750, 0.0014953613281250, 0.0022888183593750,
+0.0018920898437500, 0.0022583007812500, 0.0014648437500000, 0.0026245117187500,
+0.0003356933593750, 0.0024108886718750, 0.0012512207031250, 0.0027465820312500,
+0.0000915527343750, 0.0026245117187500, 0.0001831054687500, 0.0028381347656250,
+0.0004577636718750, 0.0031433105468750, 0.0000915527343750, 0.0027465820312500,
+0.0004272460937500, 0.0034790039062500, 0.0004272460937500, 0.0031738281250000,
+0.0006408691406250, 0.0030517578125000, 0.0005798339843750, 0.0034790039062500,
+0.0003051757812500, 0.0031738281250000, 0.0003051757812500, 0.0030212402343750,
+0.0005798339843750, 0.0031127929687500, -0.0000610351562500, 0.0030822753906250,
+0.0004577636718750, 0.0029296875000000, 0.0011596679687500, 0.0029907226562500,
+0.0002746582031250, 0.0029296875000000, 0.0018920898437500, 0.0029296875000000,
+0.0016479492187500, 0.0027465820312500, 0.0014038085937500, 0.0024719238281250,
+0.0026550292968750, 0.0026855468750000, 0.0005187988281250, 0.0027770996093750,
+0.0027465820312500, 0.0025939941406250, 0.0018920898437500, 0.0024108886718750,
+0.0022888183593750, 0.0023803710937500, 0.0032348632812500, 0.0023193359375000,
+0.0025634765625000, 0.0014648437500000, 0.0032653808593750, 0.0014038085937500,
+0.0028076171875000, 0.0011291503906250, 0.0026855468750000, 0.0003051757812500,
+0.0031127929687500, 0.0002746582031250, 0.0024414062500000, 0.0004882812500000,
+0.0026245117187500, -0.0001220703125000, 0.0025329589843750, 0.0002746582031250,
+0.0020446777343750, 0.0010070800781250, 0.0018005371093750, 0.0003662109375000,
+0.0016784667968750, 0.0011291503906250, 0.0012207031250000, 0.0013732910156250,
+0.0007324218750000, 0.0009155273437500, 0.0010375976562500, 0.0008850097656250,
+0.0010986328125000, 0.0011596679687500, 0.0004882812500000, 0.0005187988281250,
+0.0018005371093750, 0.0003051757812500, 0.0011901855468750, 0.0006713867187500,
+0.0010986328125000, 0.0006713867187500, 0.0019836425781250, 0.0003051757812500,
+0.0011291503906250, 0.0009460449218750, 0.0013427734375000, 0.0009460449218750,
+0.0011901855468750, 0.0006408691406250, 0.0010070800781250, 0.0009155273437500,
+0.0011291503906250, 0.0011291503906250, 0.0009765625000000, 0.0001525878906250,
+0.0010375976562500, 0.0006408691406250, 0.0014038085937500, 0.0005187988281250,
+0.0012817382812500, -0.0003356933593750, 0.0010986328125000, 0.0004882812500000,
+0.0015869140625000, 0.0006713867187500, 0.0009155273437500, 0.0002136230468750,
+0.0007629394531250, 0.0011901855468750, 0.0009155273437500, 0.0021057128906250,
+0.0006103515625000, 0.0015258789062500, 0.0003662109375000, 0.0031433105468750,
+0.0007629394531250, 0.0032043457031250, 0.0005187988281250, 0.0032043457031250,
+0.0004272460937500, 0.0040893554687500, 0.0007629394531250, 0.0039062500000000,
+0.0005187988281250, 0.0036010742187500, 0.0003662109375000, 0.0040283203125000,
+0.0007324218750000, 0.0034790039062500, 0.0006408691406250, 0.0027160644531250,
+0.0006103515625000, 0.0032958984375000, 0.0011901855468750, 0.0022888183593750,
+0.0006408691406250, 0.0021667480468750, 0.0012512207031250, 0.0024719238281250,
+0.0011901855468750, 0.0022277832031250, 0.0006103515625000, 0.0021667480468750,
+0.0013427734375000, 0.0029602050781250, 0.0007934570312500, 0.0026855468750000,
+0.0007019042968750, 0.0031433105468750, 0.0009765625000000, 0.0038757324218750,
+0.0009155273437500, 0.0032043457031250, 0.0007019042968750, 0.0042419433593750,
+0.0009155273437500, 0.0041503906250000, 0.0008239746093750, 0.0039672851562500,
+0.0007019042968750, 0.0044860839843750, 0.0007019042968750, 0.0043334960937500,
+0.0005798339843750, 0.0039062500000000, 0.0004882812500000, 0.0042724609375000,
+0.0004882812500000, 0.0038146972656250, 0.0004882812500000, 0.0032653808593750,
+0.0004882812500000, 0.0039672851562500, 0.0006103515625000, 0.0027160644531250,
+0.0005798339843750, 0.0030212402343750, 0.0005798339843750, 0.0029602050781250,
+0.0006713867187500, 0.0021667480468750, 0.0005798339843750, 0.0027160644531250,
+0.0003662109375000, 0.0023803710937500, 0.0007934570312500, 0.0023498535156250,
+0.0002441406250000, 0.0025939941406250, 0.0002136230468750, 0.0026245117187500,
+0.0005493164062500, 0.0026245117187500, -0.0001525878906250, 0.0031127929687500,
+0.0003051757812500, 0.0029602050781250, -0.0000305175781250, 0.0032348632812500,
+-0.0001220703125000, 0.0036315917968750, 0.0000000000000000, 0.0034179687500000,
+-0.0004882812500000, 0.0038452148437500, -0.0001525878906250, 0.0038757324218750,
+-0.0003967285156250, 0.0037231445312500, -0.0004882812500000, 0.0039062500000000,
+-0.0003051757812500, 0.0039367675781250, -0.0006408691406250, 0.0034484863281250,
+-0.0006103515625000, 0.0036315917968750, -0.0006103515625000, 0.0033264160156250,
+-0.0008239746093750, 0.0028686523437500, -0.0008239746093750, 0.0031433105468750,
+-0.0007629394531250, 0.0021972656250000, -0.0009765625000000, 0.0024414062500000,
+-0.0009460449218750, 0.0020446777343750, -0.0008544921875000, 0.0012817382812500,
+-0.0010070800781250, 0.0019226074218750, -0.0009460449218750, 0.0006103515625000,
+-0.0007324218750000, 0.0009765625000000, -0.0009155273437500, 0.0006408691406250,
+-0.0005798339843750, -0.0003662109375000, -0.0002136230468750, 0.0004272460937500,
+-0.0005798339843750, -0.0008850097656250, 0.0001220703125000, -0.0007019042968750,
+0.0000915527343750, -0.0008850097656250, 0.0000915527343750, -0.0015869140625000,
+0.0005187988281250, -0.0010681152343750, 0.0003051757812500, -0.0018310546875000,
+0.0006408691406250, -0.0017395019531250, 0.0006713867187500, -0.0016784667968750,
+0.0007324218750000, -0.0020751953125000, 0.0009460449218750, -0.0017700195312500,
+0.0007324218750000, -0.0018310546875000, 0.0007019042968750, -0.0021057128906250,
+0.0007324218750000, -0.0018615722656250, 0.0003662109375000, -0.0017395019531250,
+0.0001525878906250, -0.0021362304687500, 0.0002441406250000, -0.0015258789062500,
+-0.0002441406250000, -0.0019531250000000, -0.0002441406250000, -0.0019836425781250,
+-0.0002136230468750, -0.0015869140625000, -0.0003662109375000, -0.0021972656250000,
+-0.0000610351562500, -0.0019531250000000, -0.0002441406250000, -0.0020446777343750,
+0.0000915527343750, -0.0024108886718750, 0.0000610351562500, -0.0024414062500000,
+-0.0000610351562500, -0.0024108886718750, 0.0004882812500000, -0.0031127929687500,
+0.0000915527343750, -0.0030212402343750, 0.0003967285156250, -0.0032958984375000,
+0.0005187988281250, -0.0038757324218750, 0.0001831054687500, -0.0035705566406250,
+0.0004272460937500, -0.0039062500000000, 0.0004882812500000, -0.0041198730468750,
+0.0001220703125000, -0.0038146972656250, 0.0003662109375000, -0.0038146972656250,
+0.0006408691406250, -0.0040893554687500, 0.0000305175781250, -0.0030212402343750,
+0.0008544921875000, -0.0033874511718750, 0.0003967285156250, -0.0030517578125000,
+0.0005798339843750, -0.0021057128906250, 0.0012512207031250, -0.0027160644531250,
+0.0005493164062500, -0.0014953613281250, 0.0014343261718750, -0.0017395019531250,
+0.0015563964843750, -0.0013427734375000, 0.0014038085937500, -0.0005187988281250,
+0.0017700195312500, -0.0011901855468750, 0.0018615722656250, 0.0002746582031250,
+0.0014038085937500, -0.0002441406250000, 0.0017089843750000, -0.0000305175781250,
+0.0016479492187500, 0.0009765625000000, 0.0011901855468750, 0.0003662109375000,
+0.0016174316406250, 0.0015869140625000, 0.0010070800781250, 0.0016174316406250,
+0.0011596679687500, 0.0015563964843750, 0.0010375976562500, 0.0022888183593750,
+0.0006103515625000, 0.0021667480468750, 0.0008544921875000, 0.0019836425781250,
+0.0003356933593750, 0.0020751953125000, 0.0003662109375000, 0.0018920898437500,
+0.0003051757812500, 0.0014953613281250, 0.0000000000000000, 0.0015563964843750,
+0.0002746582031250, 0.0014038085937500, 0.0000000000000000, 0.0013732910156250,
+0.0000610351562500, 0.0012817382812500, 0.0000610351562500, 0.0012512207031250,
+-0.0000915527343750, 0.0012512207031250, 0.0001525878906250, 0.0007324218750000,
+-0.0001831054687500},
+{-0.0005798339843750, 0.0022277832031250, -0.0007019042968750, 0.0023803710937500,
+-0.0007629394531250, 0.0022277832031250, -0.0007629394531250, 0.0013427734375000,
+-0.0008239746093750, 0.0018615722656250, -0.0007934570312500, 0.0010375976562500,
+-0.0007629394531250, 0.0010986328125000, -0.0007324218750000, 0.0010070800781250,
+-0.0006103515625000, 0.0005187988281250, -0.0005187988281250, 0.0007934570312500,
+-0.0005187988281250, 0.0004272460937500, -0.0002441406250000, 0.0004272460937500,
+-0.0002746582031250, 0.0003967285156250, -0.0002136230468750, 0.0000915527343750,
+0.0000000000000000, 0.0002136230468750, -0.0000610351562500, 0.0000915527343750,
+0.0000610351562500, -0.0000915527343750, 0.0000305175781250, 0.0000000000000000,
+0.0001220703125000, -0.0000915527343750, 0.0003051757812500, -0.0004577636718750,
+0.0001525878906250, -0.0000305175781250, 0.0003967285156250, -0.0005187988281250,
+0.0003356933593750, -0.0003662109375000, 0.0002746582031250, 0.0000000000000000,
+0.0002441406250000, -0.0008544921875000, 0.0000610351562500, 0.0001831054687500,
+0.0000000000000000, -0.0003967285156250, -0.0000915527343750, -0.0002441406250000,
+-0.0002136230468750, 0.0005187988281250, -0.0001831054687500, -0.0005798339843750,
+-0.0000915527343750, 0.0005493164062500, -0.0001831054687500, 0.0000915527343750,
+0.0000000000000000, 0.0000915527343750, 0.0000305175781250, 0.0007324218750000,
+0.0000000000000000, -0.0000610351562500, 0.0003051757812500, 0.0007324218750000,
+0.0000915527343750, 0.0004882812500000, 0.0003356933593750, 0.0004577636718750,
+0.0004577636718750, 0.0010070800781250, 0.0002746582031250, 0.0006408691406250,
+0.0004882812500000, 0.0009460449218750, 0.0005187988281250, 0.0008850097656250,
+0.0003051757812500, 0.0006408691406250, 0.0004577636718750, 0.0006103515625000,
+0.0004577636718750, 0.0004577636718750, -0.0000915527343750, 0.0002136230468750,
+0.0005798339843750, 0.0000305175781250, 0.0000915527343750, -0.0002136230468750,
+0.0001831054687500, -0.0005493164062500, 0.0007019042968750, -0.0008850097656250,
+-0.0000610351562500, -0.0010986328125000, 0.0009155273437500, -0.0014343261718750,
+0.0007629394531250, -0.0016174316406250, 0.0007324218750000, -0.0017395019531250,
+0.0014343261718750, -0.0020446777343750, 0.0013122558593750, -0.0019226074218750,
+0.0012512207031250, -0.0021667480468750, 0.0018005371093750, -0.0020446777343750,
+0.0016479492187500, -0.0018615722656250, 0.0014038085937500, -0.0022277832031250,
+0.0020446777343750, -0.0018005371093750, 0.0014648437500000, -0.0019836425781250,
+0.0016784667968750, -0.0021972656250000, 0.0016479492187500, -0.0020751953125000,
+0.0012207031250000, -0.0023193359375000, 0.0015563964843750, -0.0020751953125000,
+0.0010986328125000, -0.0024414062500000, 0.0010986328125000, -0.0020141601562500,
+0.0009460449218750, -0.0014343261718750, 0.0006408691406250, -0.0018920898437500,
+0.0008850097656250, -0.0005187988281250, 0.0004577636718750, -0.0005798339843750,
+0.0005187988281250, -0.0001831054687500, 0.0005798339843750, 0.0008544921875000,
+0.0003356933593750, 0.0003967285156250, 0.0004882812500000, 0.0010070800781250,
+0.0002746582031250, 0.0012512207031250, 0.0004577636718750, 0.0012207031250000,
+0.0003662109375000, 0.0014648437500000, 0.0001525878906250, 0.0015563964843750,
+0.0004882812500000, 0.0019226074218750, 0.0002746582031250, 0.0019226074218750,
+0.0004272460937500, 0.0019836425781250, 0.0005493164062500, 0.0021972656250000,
+0.0005493164062500, 0.0019531250000000, 0.0007629394531250, 0.0018005371093750,
+0.0008850097656250, 0.0017395019531250, 0.0010375976562500, 0.0014343261718750,
+0.0012512207031250, 0.0011291503906250, 0.0016479492187500, 0.0012512207031250,
+0.0019531250000000, 0.0015869140625000, 0.0017089843750000, 0.0013122558593750,
+0.0021667480468750, 0.0013122558593750, 0.0019226074218750, 0.0016479492187500,
+0.0017089843750000, 0.0014038085937500, 0.0024719238281250, 0.0009460449218750,
+0.0014343261718750, 0.0013122558593750, 0.0018005371093750, 0.0007629394531250,
+0.0018310546875000, -0.0000305175781250, 0.0013122558593750, 0.0003662109375000,
+0.0019836425781250, -0.0004272460937500, 0.0014038085937500, -0.0007019042968750,
+0.0019226074218750, -0.0005798339843750, 0.0016784667968750, -0.0009155273437500,
+0.0010986328125000, -0.0010375976562500, 0.0019531250000000, -0.0003967285156250,
+0.0007629394531250, -0.0009460449218750, 0.0010070800781250, -0.0004577636718750,
+0.0010681152343750, 0.0002746582031250, 0.0004272460937500, -0.0004577636718750,
+0.0009460449218750, 0.0011291503906250, 0.0007934570312500, 0.0009460449218750,
+0.0008544921875000, 0.0010070800781250, 0.0010375976562500, 0.0020751953125000,
+0.0011596679687500, 0.0016784667968750, 0.0014038085937500, 0.0018005371093750,
+0.0013427734375000, 0.0021057128906250, 0.0015563964843750, 0.0018615722656250,
+0.0015563964843750, 0.0016479492187500, 0.0014038085937500, 0.0019531250000000,
+0.0015869140625000, 0.0013122558593750, 0.0016174316406250, 0.0014648437500000,
+0.0016479492187500, 0.0013122558593750, 0.0017089843750000, 0.0009460449218750,
+0.0017700195312500, 0.0013732910156250, 0.0019226074218750, 0.0004577636718750,
+0.0018310546875000, 0.0008544921875000, 0.0020446777343750, 0.0006408691406250,
+0.0020141601562500, -0.0000915527343750, 0.0019226074218750, 0.0004577636718750,
+0.0022277832031250, 0.0000915527343750, 0.0018920898437500, -0.0002441406250000,
+0.0021667480468750, 0.0001525878906250, 0.0021667480468750, 0.0000305175781250,
+0.0019531250000000, -0.0005493164062500, 0.0022583007812500, 0.0004272460937500,
+0.0018920898437500, -0.0002441406250000, 0.0021667480468750, -0.0001220703125000,
+0.0021057128906250, 0.0007019042968750, 0.0018005371093750, -0.0002746582031250,
+0.0021667480468750, 0.0008239746093750, 0.0015258789062500, 0.0005493164062500,
+0.0017395019531250, 0.0004272460937500, 0.0015869140625000, 0.0010681152343750,
+0.0011291503906250, 0.0007324218750000, 0.0014953613281250, 0.0008544921875000,
+0.0008544921875000, 0.0010070800781250, 0.0010070800781250, 0.0007934570312500,
+0.0009460449218750, 0.0006408691406250, 0.0005798339843750, 0.0006408691406250,
+0.0010375976562500, 0.0003051757812500, 0.0004882812500000, 0.0001525878906250,
+0.0008850097656250, 0.0000000000000000, 0.0009765625000000, -0.0001525878906250,
+0.0007629394531250, -0.0001525878906250, 0.0014953613281250, -0.0005798339843750,
+0.0013427734375000, -0.0005187988281250, 0.0017700195312500, -0.0005798339843750,
+0.0021362304687500, -0.0009155273437500, 0.0021972656250000, -0.0006713867187500,
+0.0027465820312500, -0.0005187988281250, 0.0029907226562500, -0.0006713867187500,
+0.0031738281250000, -0.0003356933593750, 0.0034790039062500, 0.0000305175781250,
+0.0037536621093750, -0.0002746582031250, 0.0037536621093750, 0.0003967285156250,
+0.0040893554687500, 0.0002441406250000, 0.0038757324218750, 0.0002441406250000,
+0.0039062500000000, 0.0006713867187500, 0.0040893554687500, 0.0004272460937500,
+0.0038757324218750, 0.0003967285156250, 0.0038452148437500, 0.0006713867187500,
+0.0037841796875000, 0.0003967285156250, 0.0036010742187500, 0.0000915527343750,
+0.0034484863281250, 0.0005187988281250, 0.0035095214843750, -0.0001220703125000,
+0.0028991699218750, 0.0000305175781250, 0.0029602050781250, 0.0000610351562500,
+0.0027160644531250, -0.0003051757812500, 0.0022583007812500, -0.0000610351562500,
+0.0023803710937500, -0.0002441406250000, 0.0018615722656250, -0.0003051757812500,
+0.0018310546875000, -0.0001831054687500, 0.0016479492187500, -0.0002136230468750,
+0.0012817382812500, -0.0003051757812500, 0.0014038085937500, 0.0000000000000000,
+0.0010375976562500, -0.0000305175781250, 0.0008850097656250, 0.0000305175781250,
+0.0008850097656250, 0.0002746582031250, 0.0007324218750000, 0.0002441406250000,
+0.0006408691406250, 0.0004577636718750, 0.0007934570312500, 0.0004882812500000,
+0.0005798339843750, 0.0007019042968750, 0.0007629394531250, 0.0008850097656250,
+0.0010681152343750, 0.0006713867187500, 0.0006713867187500, 0.0010681152343750,
+0.0014038085937500, 0.0008850097656250, 0.0011901855468750, 0.0007934570312500,
+0.0013122558593750, 0.0009460449218750, 0.0019531250000000, 0.0005798339843750,
+0.0016479492187500, 0.0006103515625000, 0.0021362304687500, 0.0004882812500000,
+0.0022277832031250, 0.0003967285156250, 0.0021667480468750, 0.0004577636718750,
+0.0023803710937500, 0.0003356933593750, 0.0023498535156250, 0.0003967285156250,
+0.0022277832031250, 0.0005493164062500, 0.0022888183593750, 0.0004577636718750,
+0.0020751953125000, 0.0005187988281250, 0.0018615722656250, 0.0007934570312500,
+0.0020446777343750, 0.0005798339843750, 0.0018005371093750, 0.0009765625000000,
+0.0017700195312500, 0.0010375976562500, 0.0019531250000000, 0.0008239746093750,
+0.0018310546875000, 0.0012207031250000, 0.0017089843750000, 0.0008850097656250,
+0.0022583007812500, 0.0011291503906250, 0.0020446777343750, 0.0010986328125000,
+0.0022583007812500, 0.0008544921875000, 0.0028076171875000, 0.0011596679687500,
+0.0024414062500000, 0.0007934570312500, 0.0028991699218750, 0.0010986328125000,
+0.0029602050781250, 0.0011291503906250, 0.0026245117187500, 0.0010681152343750,
+0.0025939941406250, 0.0017089843750000, 0.0024719238281250, 0.0013427734375000,
+0.0017700195312500, 0.0017395019531250, 0.0018920898437500, 0.0019836425781250,
+0.0014953613281250, 0.0015869140625000, 0.0007934570312500, 0.0018615722656250,
+0.0012207031250000, 0.0021057128906250, 0.0003356933593750, 0.0014648437500000,
+0.0003051757812500, 0.0018920898437500, 0.0003051757812500, 0.0022277832031250,
+-0.0000305175781250, 0.0010070800781250, 0.0001525878906250, 0.0024108886718750,
+-0.0001220703125000, 0.0016784667968750, 0.0001220703125000, 0.0015563964843750,
+-0.0001525878906250, 0.0024414062500000, -0.0005798339843750, 0.0016479492187500,
+-0.0001220703125000, 0.0023193359375000, -0.0009765625000000, 0.0021972656250000,
+-0.0008544921875000, 0.0021667480468750, -0.0009765625000000, 0.0026245117187500,
+-0.0015258789062500, 0.0023193359375000, -0.0010986328125000, 0.0028686523437500,
+-0.0014953613281250, 0.0027465820312500, -0.0016174316406250, 0.0029907226562500,
+-0.0011291503906250, 0.0033874511718750, -0.0010681152343750, 0.0028991699218750,
+-0.0011901855468750, 0.0035705566406250, 0.0002136230468750, 0.0034790039062500,
+-0.0000610351562500, 0.0034179687500000, 0.0004577636718750, 0.0039062500000000,
+0.0015869140625000, 0.0039367675781250, 0.0010375976562500, 0.0038146972656250,
+0.0021057128906250, 0.0044250488281250, 0.0020141601562500, 0.0043334960937500,
+0.0018615722656250, 0.0040283203125000, 0.0021057128906250, 0.0047302246093750,
+0.0016479492187500, 0.0037841796875000, 0.0024108886718750, 0.0039978027343750,
+0.0018310546875000, 0.0039367675781250, 0.0022277832031250, 0.0032043457031250,
+0.0031433105468750, 0.0036010742187500, 0.0022277832031250, 0.0031433105468750,
+0.0029296875000000, 0.0031433105468750, 0.0030212402343750, 0.0031127929687500,
+0.0023193359375000, 0.0026245117187500, 0.0018920898437500, 0.0016174316406250,
+0.0025939941406250, 0.0015258789062500, 0.0036926269531250, 0.0009460449218750,
+0.0039978027343750, 0.0012817382812500, 0.0031127929687500, 0.0014648437500000,
+0.0026550292968750, 0.0007629394531250, 0.0032348632812500, 0.0012207031250000,
+0.0018310546875000, 0.0009155273437500, 0.0020141601562500, 0.0005493164062500,
+0.0022583007812500},
+{0.0039978027343750, 0.0004272460937500, 0.0039672851562500, 0.0004272460937500,
+0.0037231445312500, 0.0004272460937500, 0.0039978027343750, 0.0003967285156250,
+0.0037231445312500, 0.0004882812500000, 0.0037841796875000, 0.0004577636718750,
+0.0038146972656250, 0.0005187988281250, 0.0036926269531250, 0.0007324218750000,
+0.0038452148437500, 0.0005798339843750, 0.0038452148437500, 0.0009460449218750,
+0.0039672851562500, 0.0010070800781250, 0.0039978027343750, 0.0009765625000000,
+0.0040588378906250, 0.0014648437500000, 0.0041809082031250, 0.0011901855468750,
+0.0040893554687500, 0.0015258789062500, 0.0041503906250000, 0.0015258789062500,
+0.0039978027343750, 0.0012817382812500, 0.0037536621093750, 0.0017089843750000,
+0.0039062500000000, 0.0009460449218750, 0.0033264160156250, 0.0012817382812500,
+0.0034179687500000, 0.0011596679687500, 0.0032653808593750, 0.0006713867187500,
+0.0027465820312500, 0.0014343261718750, 0.0029907226562500, 0.0005493164062500,
+0.0027770996093750, 0.0010681152343750, 0.0025634765625000, 0.0012207031250000,
+0.0028076171875000, 0.0006713867187500, 0.0029296875000000, 0.0013427734375000,
+0.0028381347656250, 0.0013427734375000, 0.0035705566406250, 0.0009765625000000,
+0.0033264160156250, 0.0015563964843750, 0.0036315917968750, 0.0017700195312500,
+0.0043029785156250, 0.0009155273437500, 0.0038452148437500, 0.0022888183593750,
+0.0044250488281250, 0.0014038085937500, 0.0045166015625000, 0.0014648437500000,
+0.0043334960937500, 0.0023498535156250, 0.0043029785156250, 0.0010986328125000,
+0.0041503906250000, 0.0021972656250000, 0.0040283203125000, 0.0018310546875000,
+0.0038146972656250, 0.0018310546875000, 0.0037841796875000, 0.0025329589843750,
+0.0037536621093750, 0.0020446777343750, 0.0035095214843750, 0.0026855468750000,
+0.0036010742187500, 0.0025634765625000, 0.0034179687500000, 0.0025329589843750,
+0.0032348632812500, 0.0028381347656250, 0.0032043457031250, 0.0025329589843750,
+0.0029907226562500, 0.0029907226562500, 0.0027770996093750, 0.0026855468750000,
+0.0027160644531250, 0.0028991699218750, 0.0021362304687500, 0.0033874511718750,
+0.0016479492187500, 0.0029907226562500, 0.0018310546875000, 0.0035095214843750,
+0.0007629394531250, 0.0037841796875000, 0.0005493164062500, 0.0034790039062500,
+0.0002746582031250, 0.0035705566406250, -0.0004577636718750, 0.0040588378906250,
+-0.0003051757812500, 0.0033569335937500, -0.0003967285156250, 0.0035400390625000,
+-0.0005798339843750, 0.0034790039062500, -0.0001525878906250, 0.0029907226562500,
+0.0002746582031250, 0.0032653808593750, 0.0000000000000000, 0.0027160644531250,
+0.0006713867187500, 0.0028076171875000, 0.0004577636718750, 0.0028076171875000,
+0.0007629394531250, 0.0025634765625000, 0.0010986328125000, 0.0028381347656250,
+0.0005187988281250, 0.0021057128906250, 0.0022277832031250, 0.0022888183593750,
+0.0015563964843750, 0.0022277832031250, 0.0018310546875000, 0.0011901855468750,
+0.0041809082031250, 0.0008850097656250, 0.0003662109375000, 0.0010070800781250,
+0.0020446777343750, 0.0007629394531250, 0.0010375976562500, 0.0005187988281250,
+0.0019531250000000, 0.0001831054687500, 0.0035705566406250, -0.0001525878906250,
+0.0029296875000000, 0.0002136230468750, 0.0036010742187500, -0.0005187988281250,
+0.0041809082031250, -0.0001220703125000, 0.0034484863281250, 0.0006408691406250,
+0.0031127929687500, -0.0003051757812500, 0.0038146972656250, 0.0009460449218750,
+0.0024108886718750, 0.0009765625000000, 0.0025634765625000, 0.0006408691406250,
+0.0025634765625000, 0.0010681152343750, 0.0019836425781250, 0.0009460449218750,
+0.0023803710937500, 0.0010070800781250, 0.0017395019531250, 0.0006713867187500,
+0.0019531250000000, 0.0012512207031250, 0.0018615722656250, 0.0015258789062500,
+0.0013427734375000, 0.0008239746093750, 0.0017395019531250, 0.0028076171875000,
+0.0017395019531250, 0.0023498535156250, 0.0012817382812500, 0.0023498535156250,
+0.0018005371093750, 0.0036621093750000, 0.0021972656250000, 0.0028991699218750,
+0.0016174316406250, 0.0027465820312500, 0.0024108886718750, 0.0029602050781250,
+0.0021972656250000, 0.0022583007812500, 0.0018920898437500, 0.0014648437500000,
+0.0020446777343750, 0.0016479492187500, 0.0016174316406250, 0.0007629394531250,
+0.0014648437500000, 0.0003356933593750, 0.0010681152343750, 0.0005187988281250,
+0.0009460449218750, 0.0003051757812500, 0.0009155273437500, -0.0001831054687500,
+0.0005493164062500, 0.0005798339843750, 0.0005798339843750, 0.0003967285156250,
+0.0005798339843750, 0.0001525878906250, 0.0003356933593750, 0.0005493164062500,
+0.0002746582031250, 0.0004577636718750, 0.0004272460937500, -0.0002746582031250,
+0.0002136230468750, 0.0001525878906250, 0.0000915527343750, -0.0004882812500000,
+0.0003662109375000, -0.0014038085937500, 0.0003967285156250, -0.0006408691406250,
+0.0000610351562500, -0.0023193359375000, 0.0005493164062500, -0.0021972656250000,
+0.0001831054687500, -0.0020141601562500, 0.0001831054687500, -0.0029296875000000,
+0.0005187988281250, -0.0025939941406250, -0.0001220703125000, -0.0020446777343750,
+0.0005493164062500, -0.0024108886718750, 0.0002136230468750, -0.0017089843750000,
+0.0000610351562500, -0.0007934570312500, 0.0005493164062500, -0.0013122558593750,
+0.0002746582031250, 0.0003356933593750, 0.0000915527343750, 0.0003051757812500,
+0.0004272460937500, 0.0005187988281250, 0.0002136230468750, 0.0015563964843750,
+-0.0000915527343750, 0.0011291503906250, 0.0003662109375000, 0.0014038085937500,
+-0.0000610351562500, 0.0017395019531250, 0.0000305175781250, 0.0013122558593750,
+0.0002441406250000, 0.0010070800781250, 0.0001220703125000, 0.0014038085937500,
+0.0002441406250000, 0.0003967285156250, 0.0004882812500000, 0.0004882812500000,
+0.0003662109375000, 0.0002441406250000, 0.0004882812500000, -0.0005798339843750,
+0.0007324218750000, -0.0002441406250000, 0.0005187988281250, -0.0008544921875000,
+0.0007019042968750, -0.0010681152343750, 0.0007629394531250, -0.0010375976562500,
+0.0005798339843750, -0.0014038085937500, 0.0006103515625000, -0.0014648437500000,
+0.0007324218750000, -0.0011901855468750, 0.0004272460937500, -0.0015258789062500,
+0.0004577636718750, -0.0012512207031250, 0.0005493164062500, -0.0008544921875000,
+0.0004577636718750, -0.0013427734375000, 0.0004882812500000, -0.0001525878906250,
+0.0004577636718750, -0.0005187988281250, 0.0005798339843750, -0.0001220703125000,
+0.0004577636718750, 0.0009155273437500, 0.0004272460937500, 0.0002746582031250,
+0.0005187988281250, 0.0014343261718750, 0.0003051757812500, 0.0013732910156250,
+0.0003051757812500, 0.0015869140625000, 0.0003051757812500, 0.0023498535156250,
+0.0001525878906250, 0.0018920898437500, 0.0003356933593750, 0.0026550292968750,
+0.0000610351562500, 0.0027160644531250, 0.0001525878906250, 0.0027770996093750,
+0.0002136230468750, 0.0032653808593750, 0.0000000000000000, 0.0031433105468750,
+0.0002136230468750, 0.0035095214843750, 0.0001831054687500, 0.0035705566406250,
+0.0001525878906250, 0.0036926269531250, 0.0002441406250000, 0.0039367675781250,
+0.0003356933593750, 0.0039672851562500, 0.0002746582031250, 0.0040588378906250,
+0.0003967285156250, 0.0042724609375000, 0.0003967285156250, 0.0042114257812500,
+0.0003662109375000, 0.0042724609375000, 0.0004272460937500, 0.0043945312500000,
+0.0003967285156250, 0.0041198730468750, 0.0002136230468750, 0.0042724609375000,
+0.0001525878906250, 0.0041198730468750, 0.0000305175781250, 0.0038452148437500,
+-0.0001220703125000, 0.0041198730468750, -0.0001831054687500, 0.0035095214843750,
+-0.0002136230468750, 0.0036926269531250, -0.0003356933593750, 0.0033874511718750,
+-0.0002746582031250, 0.0028686523437500, -0.0001831054687500, 0.0032043457031250,
+-0.0003356933593750, 0.0022888183593750, -0.0000305175781250, 0.0023498535156250,
+-0.0001220703125000, 0.0022277832031250, -0.0000305175781250, 0.0015869140625000,
+0.0001831054687500, 0.0018615722656250, 0.0000000000000000, 0.0015258789062500,
+0.0002441406250000, 0.0014648437500000, 0.0002746582031250, 0.0015258789062500,
+0.0002136230468750, 0.0013732910156250, 0.0003356933593750, 0.0014648437500000,
+0.0002136230468750, 0.0015869140625000, 0.0003356933593750, 0.0013122558593750,
+0.0000305175781250, 0.0015563964843750, 0.0001525878906250, 0.0018615722656250,
+0.0002441406250000, 0.0015869140625000, -0.0002441406250000, 0.0022888183593750,
+0.0005493164062500, 0.0021362304687500, 0.0001525878906250, 0.0021667480468750,
+0.0002441406250000, 0.0027160644531250, 0.0008850097656250, 0.0024414062500000,
+0.0003967285156250, 0.0025024414062500, 0.0010375976562500, 0.0027160644531250,
+0.0011291503906250, 0.0024414062500000, 0.0010986328125000, 0.0022888183593750,
+0.0014953613281250, 0.0025939941406250, 0.0014953613281250, 0.0021057128906250,
+0.0015258789062500, 0.0021972656250000, 0.0016784667968750, 0.0021972656250000,
+0.0016784667968750, 0.0019531250000000, 0.0015258789062500, 0.0021057128906250,
+0.0016174316406250, 0.0019531250000000, 0.0014953613281250, 0.0019226074218750,
+0.0013427734375000, 0.0019226074218750, 0.0012817382812500, 0.0018920898437500,
+0.0011901855468750, 0.0018615722656250, 0.0009765625000000, 0.0017700195312500,
+0.0011291503906250, 0.0016784667968750, 0.0008850097656250, 0.0018310546875000,
+0.0007934570312500, 0.0017700195312500, 0.0009765625000000, 0.0014648437500000,
+0.0008850097656250, 0.0018615722656250, 0.0007019042968750, 0.0014953613281250,
+0.0006713867187500, 0.0014343261718750, 0.0006408691406250, 0.0018310546875000,
+0.0004882812500000, 0.0015563964843750, 0.0004882812500000, 0.0015563964843750,
+0.0004272460937500, 0.0017395019531250, 0.0004882812500000, 0.0014953613281250,
+0.0004577636718750, 0.0014038085937500, 0.0004577636718750, 0.0017395019531250,
+0.0006103515625000, 0.0010070800781250, 0.0006103515625000, 0.0013732910156250,
+0.0006408691406250, 0.0013427734375000, 0.0007934570312500, 0.0007324218750000,
+0.0009155273437500, 0.0011291503906250, 0.0010375976562500, 0.0009460449218750,
+0.0013427734375000, 0.0009765625000000, 0.0015869140625000, 0.0011596679687500,
+0.0015869140625000, 0.0012207031250000, 0.0018005371093750, 0.0013427734375000,
+0.0021972656250000, 0.0012512207031250, 0.0016784667968750, 0.0012207031250000,
+0.0021057128906250, 0.0014038085937500, 0.0020446777343750, 0.0014343261718750,
+0.0015563964843750, 0.0013122558593750, 0.0019226074218750, 0.0022583007812500,
+0.0015258789062500, 0.0018005371093750, 0.0017700195312500, 0.0018615722656250,
+0.0017089843750000, 0.0024719238281250, 0.0014953613281250, 0.0018920898437500,
+0.0019226074218750, 0.0022583007812500, 0.0013732910156250, 0.0020446777343750,
+0.0017089843750000, 0.0017089843750000, 0.0018005371093750, 0.0017395019531250,
+0.0014648437500000, 0.0014953613281250, 0.0018920898437500, 0.0014038085937500,
+0.0015869140625000, 0.0014038085937500, 0.0016784667968750, 0.0012512207031250,
+0.0017089843750000, 0.0012817382812500, 0.0014648437500000, 0.0013427734375000,
+0.0017395019531250, 0.0008239746093750, 0.0013732910156250, 0.0010681152343750,
+0.0013122558593750, 0.0007934570312500, 0.0013122558593750, 0.0002746582031250,
+0.0010681152343750, 0.0005187988281250, 0.0010070800781250, -0.0002441406250000,
+0.0011291503906250, -0.0002136230468750, 0.0009155273437500, -0.0006408691406250,
+0.0010070800781250, -0.0013732910156250, 0.0011596679687500, -0.0010375976562500,
+0.0008850097656250},
+{0.0001831054687500, 0.0032043457031250, -0.0003356933593750, 0.0028381347656250,
+0.0003662109375000, 0.0027160644531250, -0.0000305175781250, 0.0026245117187500,
+0.0000610351562500, 0.0024414062500000, 0.0006408691406250, 0.0022888183593750,
+0.0000915527343750, 0.0024414062500000, 0.0006713867187500, 0.0022888183593750,
+0.0006713867187500, 0.0023803710937500, 0.0006713867187500, 0.0026245117187500,
+0.0010375976562500, 0.0025329589843750, 0.0010375976562500, 0.0030517578125000,
+0.0011901855468750, 0.0032043457031250, 0.0014343261718750, 0.0033264160156250,
+0.0014343261718750, 0.0038146972656250, 0.0014648437500000, 0.0038146972656250,
+0.0016784667968750, 0.0037841796875000, 0.0013122558593750, 0.0039978027343750,
+0.0013122558593750, 0.0037536621093750, 0.0012817382812500, 0.0035095214843750,
+0.0009765625000000, 0.0038452148437500, 0.0008544921875000, 0.0030822753906250,
+0.0008850097656250, 0.0032348632812500, 0.0006713867187500, 0.0031738281250000,
+0.0006103515625000, 0.0026245117187500, 0.0006713867187500, 0.0028381347656250,
+0.0003967285156250, 0.0025634765625000, 0.0004882812500000, 0.0024108886718750,
+0.0005187988281250, 0.0025024414062500, 0.0003356933593750, 0.0025329589843750,
+0.0002441406250000, 0.0024108886718750, 0.0003051757812500, 0.0022888183593750,
+0.0002441406250000, 0.0023498535156250, 0.0001831054687500, 0.0020141601562500,
+0.0002441406250000, 0.0018005371093750, 0.0003051757812500, 0.0019531250000000,
+0.0003356933593750, 0.0013732910156250, 0.0003051757812500, 0.0015563964843750,
+0.0006103515625000, 0.0016174316406250, 0.0007019042968750, 0.0013427734375000,
+0.0007019042968750, 0.0016784667968750, 0.0010375976562500, 0.0018005371093750,
+0.0009765625000000, 0.0020141601562500, 0.0012512207031250, 0.0021362304687500,
+0.0014038085937500, 0.0021057128906250, 0.0014343261718750, 0.0022277832031250,
+0.0017395019531250, 0.0017700195312500, 0.0014648437500000, 0.0018920898437500,
+0.0018920898437500, 0.0017395019531250, 0.0017700195312500, 0.0013122558593750,
+0.0014343261718750, 0.0015563964843750, 0.0021057128906250, 0.0013732910156250,
+0.0014343261718750, 0.0014343261718750, 0.0016479492187500, 0.0014648437500000,
+0.0016784667968750, 0.0010375976562500, 0.0013122558593750, 0.0006713867187500,
+0.0015563964843750, 0.0006713867187500, 0.0011291503906250, 0.0006103515625000,
+0.0013122558593750, 0.0007324218750000, 0.0014343261718750, 0.0008544921875000,
+0.0012512207031250, 0.0006713867187500, 0.0014343261718750, 0.0012512207031250,
+0.0016174316406250, 0.0011901855468750, 0.0018005371093750, 0.0011901855468750,
+0.0016784667968750, 0.0014953613281250, 0.0017089843750000, 0.0013732910156250,
+0.0019531250000000, 0.0011596679687500, 0.0014953613281250, 0.0011596679687500,
+0.0015563964843750, 0.0010375976562500, 0.0015258789062500, 0.0008239746093750,
+0.0010986328125000, 0.0006713867187500, 0.0010375976562500, 0.0007934570312500,
+0.0012207031250000, 0.0006713867187500, 0.0011291503906250, 0.0005187988281250,
+0.0010986328125000, 0.0006103515625000, 0.0011596679687500, 0.0006103515625000,
+0.0011901855468750, 0.0002746582031250, 0.0009460449218750, 0.0003662109375000,
+0.0009765625000000, 0.0003051757812500, 0.0008850097656250, 0.0000610351562500,
+0.0005798339843750, 0.0001220703125000, 0.0006103515625000, -0.0003051757812500,
+0.0004882812500000, -0.0000915527343750, 0.0002441406250000, -0.0004882812500000,
+0.0003356933593750, -0.0010681152343750, 0.0004577636718750, -0.0006713867187500,
+0.0001831054687500, -0.0013122558593750, 0.0003662109375000, -0.0015258789062500,
+0.0003967285156250, -0.0010681152343750, 0.0002136230468750, -0.0011291503906250,
+0.0001220703125000, -0.0014953613281250, 0.0001831054687500, -0.0001831054687500,
+0.0000000000000000, -0.0006103515625000, -0.0001525878906250, -0.0002136230468750,
+-0.0000915527343750, 0.0008850097656250, -0.0001525878906250, 0.0001525878906250,
+-0.0003662109375000, 0.0010375976562500, 0.0001220703125000, 0.0012512207031250,
+-0.0001525878906250, 0.0010986328125000, 0.0001525878906250, 0.0015563964843750,
+0.0007324218750000, 0.0017089843750000, 0.0002441406250000, 0.0013122558593750,
+0.0011291503906250, 0.0018310546875000, 0.0010070800781250, 0.0016479492187500,
+0.0011291503906250, 0.0012512207031250, 0.0016784667968750, 0.0019226074218750,
+0.0013427734375000, 0.0009155273437500, 0.0020141601562500, 0.0012207031250000,
+0.0018005371093750, 0.0011596679687500, 0.0020446777343750, 0.0004882812500000,
+0.0025329589843750, 0.0009765625000000, 0.0021057128906250, 0.0002441406250000,
+0.0028991699218750, 0.0003967285156250, 0.0027160644531250, 0.0002746582031250,
+0.0028076171875000, -0.0000915527343750, 0.0032653808593750, 0.0002441406250000,
+0.0028381347656250, -0.0002746582031250, 0.0032043457031250, -0.0001525878906250,
+0.0031738281250000, -0.0000610351562500, 0.0029602050781250, -0.0003051757812500,
+0.0029907226562500, 0.0000000000000000, 0.0028076171875000, 0.0000000000000000,
+0.0025024414062500, 0.0000610351562500, 0.0025024414062500, 0.0002136230468750,
+0.0021972656250000, 0.0002746582031250, 0.0018615722656250, 0.0003662109375000,
+0.0019531250000000, 0.0002136230468750, 0.0012817382812500, 0.0003662109375000,
+0.0012817382812500, 0.0003051757812500, 0.0011901855468750, 0.0001220703125000,
+0.0007324218750000, 0.0002441406250000, 0.0009155273437500, 0.0000000000000000,
+0.0010070800781250, 0.0001220703125000, 0.0009155273437500, 0.0001831054687500,
+0.0012512207031250, 0.0000000000000000, 0.0015563964843750, 0.0001831054687500,
+0.0014038085937500, 0.0001525878906250, 0.0020446777343750, 0.0001220703125000,
+0.0021362304687500, 0.0002136230468750, 0.0022277832031250, 0.0003356933593750,
+0.0026855468750000, 0.0003662109375000, 0.0025634765625000, 0.0004577636718750,
+0.0027465820312500, 0.0006408691406250, 0.0029907226562500, 0.0006713867187500,
+0.0027770996093750, 0.0007629394531250, 0.0026245117187500, 0.0010681152343750,
+0.0028991699218750, 0.0008544921875000, 0.0022583007812500, 0.0011596679687500,
+0.0023803710937500, 0.0011596679687500, 0.0021667480468750, 0.0010070800781250,
+0.0016479492187500, 0.0012207031250000, 0.0018920898437500, 0.0010681152343750,
+0.0013122558593750, 0.0010375976562500, 0.0013732910156250, 0.0010070800781250,
+0.0014038085937500, 0.0008850097656250, 0.0010375976562500, 0.0007019042968750,
+0.0012817382812500, 0.0005798339843750, 0.0013732910156250, 0.0004577636718750,
+0.0011901855468750, 0.0003356933593750, 0.0015258789062500, 0.0002746582031250,
+0.0018310546875000, 0.0003051757812500, 0.0015563964843750, 0.0002746582031250,
+0.0023803710937500, 0.0003356933593750, 0.0022888183593750, 0.0003662109375000,
+0.0024719238281250, 0.0004272460937500, 0.0030517578125000, 0.0006408691406250,
+0.0028381347656250, 0.0005798339843750, 0.0034484863281250, 0.0007934570312500,
+0.0033874511718750, 0.0008544921875000, 0.0035095214843750, 0.0008239746093750,
+0.0039367675781250, 0.0012207031250000, 0.0036621093750000, 0.0008850097656250,
+0.0039672851562500, 0.0010986328125000, 0.0040283203125000, 0.0010681152343750,
+0.0038757324218750, 0.0008239746093750, 0.0038452148437500, 0.0011596679687500,
+0.0039672851562500, 0.0005493164062500, 0.0034790039062500, 0.0010375976562500,
+0.0036621093750000, 0.0007934570312500, 0.0034179687500000, 0.0003051757812500,
+0.0029602050781250, 0.0011901855468750, 0.0032653808593750, 0.0000915527343750,
+0.0026245117187500, 0.0007019042968750, 0.0026550292968750, 0.0008850097656250,
+0.0026550292968750, 0.0002441406250000, 0.0023193359375000, 0.0011291503906250,
+0.0025329589843750, 0.0007934570312500, 0.0025329589843750, 0.0006713867187500,
+0.0026245117187500, 0.0010681152343750, 0.0027770996093750, 0.0008850097656250,
+0.0029296875000000, 0.0005798339843750, 0.0030822753906250, 0.0012207031250000,
+0.0030517578125000, 0.0005187988281250, 0.0033264160156250, 0.0007629394531250,
+0.0031433105468750, 0.0014648437500000, 0.0030517578125000, 0.0004882812500000,
+0.0035095214843750, 0.0017700195312500, 0.0024108886718750, 0.0015563964843750,
+0.0029907226562500, 0.0015563964843750, 0.0026550292968750, 0.0023498535156250,
+0.0017089843750000, 0.0020141601562500, 0.0026245117187500, 0.0024108886718750,
+0.0014953613281250, 0.0024414062500000, 0.0017700195312500, 0.0024719238281250,
+0.0017700195312500, 0.0025329589843750, 0.0011291503906250, 0.0023803710937500,
+0.0018005371093750, 0.0026550292968750, 0.0010070800781250, 0.0025634765625000,
+0.0011596679687500, 0.0023498535156250, 0.0011291503906250, 0.0024719238281250,
+0.0005798339843750, 0.0024108886718750, 0.0007629394531250, 0.0018920898437500,
+0.0002746582031250, 0.0022888183593750, 0.0002746582031250, 0.0016784667968750,
+0.0000305175781250, 0.0010681152343750, -0.0005187988281250, 0.0019836425781250,
+-0.0004272460937500, 0.0008239746093750, -0.0004882812500000, 0.0011291503906250,
+-0.0009155273437500, 0.0011901855468750, -0.0003662109375000, 0.0005493164062500,
+0.0000305175781250, 0.0010375976562500, -0.0006103515625000, 0.0002441406250000,
+0.0014038085937500, 0.0003662109375000, 0.0009155273437500, 0.0002746582031250,
+0.0011901855468750, -0.0003051757812500, 0.0025939941406250, 0.0001525878906250,
+0.0016784667968750, -0.0001525878906250, 0.0021362304687500, -0.0001525878906250,
+0.0023498535156250, -0.0000305175781250, 0.0018920898437500, -0.0000305175781250,
+0.0017089843750000, 0.0000915527343750, 0.0018920898437500, -0.0000610351562500,
+0.0012817382812500, -0.0000915527343750, 0.0014343261718750, -0.0001220703125000,
+0.0012817382812500, -0.0003662109375000, 0.0005493164062500, -0.0007019042968750,
+0.0008850097656250, 0.0000305175781250, 0.0008544921875000, 0.0002746582031250,
+0.0010986328125000, 0.0001831054687500, 0.0007019042968750, 0.0003051757812500,
+0.0005493164062500, 0.0008544921875000, 0.0010070800781250, 0.0002136230468750,
+-0.0003051757812500, 0.0005493164062500, 0.0000610351562500, 0.0008850097656250,
+0.0000000000000000, 0.0006103515625000, -0.0009765625000000, 0.0009460449218750,
+-0.0005798339843750, 0.0011901855468750, -0.0000610351562500, 0.0015258789062500,
+-0.0004272460937500, 0.0011901855468750, 0.0003967285156250, 0.0010681152343750,
+0.0016174316406250, 0.0014953613281250, 0.0011901855468750, -0.0003967285156250,
+0.0025939941406250, -0.0000610351562500, 0.0028076171875000, -0.0004882812500000,
+0.0028381347656250, -0.0020141601562500, 0.0034790039062500, -0.0012207031250000,
+0.0033874511718750, -0.0015869140625000, 0.0032958984375000, -0.0019836425781250,
+0.0035705566406250, -0.0014038085937500, 0.0031738281250000, -0.0007934570312500,
+0.0028686523437500, -0.0010986328125000, 0.0032348632812500, -0.0004882812500000,
+0.0018920898437500, -0.0003662109375000, 0.0021057128906250, -0.0006408691406250,
+0.0017395019531250, -0.0006103515625000, 0.0006103515625000, -0.0007019042968750,
+0.0009765625000000, -0.0012512207031250, 0.0007324218750000, -0.0014648437500000,
+0.0003662109375000, -0.0014648437500000, 0.0008544921875000, -0.0019226074218750,
+0.0011901855468750, -0.0021362304687500, 0.0006713867187500, -0.0011596679687500,
+0.0013122558593750, -0.0017395019531250, 0.0012512207031250, -0.0012817382812500,
+0.0009765625000000, -0.0001220703125000, 0.0009460449218750, -0.0008544921875000,
+0.0007934570312500, 0.0005493164062500, 0.0005493164062500, 0.0006713867187500,
+0.0003662109375000},
+{0.0029907226562500, 0.0016784667968750, 0.0029296875000000, 0.0016174316406250,
+0.0031738281250000, 0.0016174316406250, 0.0034484863281250, 0.0017089843750000,
+0.0033874511718750, 0.0017395019531250, 0.0036926269531250, 0.0017089843750000,
+0.0036621093750000, 0.0017395019531250, 0.0037231445312500, 0.0017395019531250,
+0.0038146972656250, 0.0016479492187500, 0.0036621093750000, 0.0014038085937500,
+0.0038146972656250, 0.0015869140625000, 0.0037231445312500, 0.0012817382812500,
+0.0036010742187500, 0.0008850097656250, 0.0036010742187500, 0.0013122558593750,
+0.0034484863281250, 0.0006103515625000, 0.0032958984375000, 0.0008544921875000,
+0.0032958984375000, 0.0006408691406250, 0.0029907226562500, 0.0001220703125000,
+0.0028686523437500, 0.0007629394531250, 0.0031127929687500, -0.0002441406250000,
+0.0019531250000000, 0.0001831054687500, 0.0023498535156250, 0.0000305175781250,
+0.0016784667968750, -0.0006713867187500, 0.0005187988281250, 0.0000610351562500,
+0.0012512207031250, -0.0007629394531250, -0.0003967285156250, -0.0004577636718750,
+-0.0004882812500000, -0.0004577636718750, -0.0003662109375000, -0.0009460449218750,
+-0.0011901855468750, -0.0003967285156250, -0.0009765625000000, -0.0010681152343750,
+-0.0004272460937500, -0.0009460449218750, -0.0007324218750000, -0.0008850097656250,
+-0.0003051757812500, -0.0013427734375000, 0.0004272460937500, -0.0011596679687500,
+0.0000000000000000, -0.0010986328125000, 0.0006103515625000, -0.0012207031250000,
+0.0008239746093750, -0.0009765625000000, 0.0007019042968750, -0.0003356933593750,
+0.0008544921875000, -0.0003662109375000, -0.0011291503906250, -0.0009460449218750,
+-0.0004882812500000, -0.0005798339843750, -0.0007324218750000, -0.0003662109375000,
+-0.0004577636718750, -0.0003051757812500, 0.0000305175781250, -0.0000610351562500,
+-0.0001525878906250, -0.0001525878906250, 0.0004577636718750, 0.0005493164062500,
+0.0006103515625000, 0.0000915527343750, 0.0003356933593750, -0.0003662109375000,
+0.0006103515625000, 0.0006408691406250, 0.0008850097656250, -0.0013122558593750,
+-0.0005187988281250, -0.0008850097656250, -0.0000610351562500, -0.0009155273437500,
+-0.0003356933593750, -0.0022583007812500, -0.0015563964843750, -0.0016174316406250,
+-0.0009155273437500, -0.0015869140625000, -0.0010375976562500, -0.0018615722656250,
+-0.0013122558593750, -0.0012817382812500, -0.0005493164062500, -0.0005493164062500,
+0.0001220703125000, -0.0009155273437500, -0.0001525878906250, -0.0005493164062500,
+0.0011596679687500, -0.0001220703125000, 0.0012207031250000, -0.0005493164062500,
+0.0014953613281250, -0.0009155273437500, 0.0023803710937500, -0.0007629394531250,
+0.0021057128906250, -0.0010986328125000, 0.0027770996093750, -0.0015258789062500,
+0.0030212402343750, -0.0012207031250000, 0.0026550292968750, -0.0011901855468750,
+0.0027465820312500, -0.0017395019531250, 0.0028991699218750, -0.0002441406250000,
+0.0015869140625000, -0.0008239746093750, 0.0018310546875000, -0.0002441406250000,
+0.0014648437500000, 0.0011596679687500, 0.0003051757812500, 0.0002136230468750,
+0.0006713867187500, 0.0018310546875000, 0.0004577636718750, 0.0018005371093750,
+0.0000000000000000, 0.0018615722656250, 0.0002441406250000, 0.0027770996093750,
+0.0005187988281250, 0.0022583007812500, 0.0001525878906250, 0.0025939941406250,
+0.0003356933593750, 0.0028076171875000, 0.0003356933593750, 0.0025329589843750,
+0.0002136230468750, 0.0025024414062500, 0.0001220703125000, 0.0025634765625000,
+0.0000305175781250, 0.0022277832031250, 0.0000305175781250, 0.0024414062500000,
+-0.0000610351562500, 0.0022277832031250, -0.0000305175781250, 0.0019531250000000,
+0.0001220703125000, 0.0024108886718750, 0.0001220703125000, 0.0016174316406250,
+0.0000000000000000, 0.0018615722656250, 0.0000305175781250, 0.0020751953125000,
+0.0000610351562500, 0.0015563964843750, -0.0000915527343750, 0.0019836425781250,
+-0.0003051757812500, 0.0023193359375000, 0.0001525878906250, 0.0021972656250000,
+-0.0002746582031250, 0.0026550292968750, -0.0002136230468750, 0.0031738281250000,
+0.0001831054687500, 0.0029602050781250, -0.0003967285156250, 0.0035705566406250,
+0.0000000000000000, 0.0037231445312500, -0.0001831054687500, 0.0037536621093750,
+-0.0002746582031250, 0.0039672851562500, -0.0000610351562500, 0.0039062500000000,
+-0.0001831054687500, 0.0037536621093750, -0.0001525878906250, 0.0039367675781250,
+0.0000915527343750, 0.0038146972656250, 0.0000305175781250, 0.0035705566406250,
+0.0000000000000000, 0.0036621093750000, 0.0004272460937500, 0.0035095214843750,
+-0.0000305175781250, 0.0033264160156250, 0.0002136230468750, 0.0033874511718750,
+0.0002746582031250, 0.0033874511718750, -0.0000305175781250, 0.0032653808593750,
+0.0004272460937500, 0.0036315917968750, 0.0000000000000000, 0.0035705566406250,
+0.0002441406250000, 0.0038146972656250, 0.0001831054687500, 0.0043029785156250,
+-0.0001525878906250, 0.0042419433593750, 0.0002441406250000, 0.0048522949218750,
+-0.0001525878906250, 0.0049743652343750, 0.0001220703125000, 0.0051879882812500,
+0.0002136230468750, 0.0055847167968750, 0.0000305175781250, 0.0055847167968750,
+0.0003662109375000, 0.0057983398437500, 0.0004272460937500, 0.0058593750000000,
+0.0004882812500000, 0.0058288574218750, 0.0005798339843750, 0.0058288574218750,
+0.0006103515625000, 0.0057373046875000, 0.0007019042968750, 0.0056457519531250,
+0.0008239746093750, 0.0055847167968750, 0.0008850097656250, 0.0055847167968750,
+0.0008544921875000, 0.0054626464843750, 0.0007934570312500, 0.0054016113281250,
+0.0008850097656250, 0.0053405761718750, 0.0006103515625000, 0.0052795410156250,
+0.0006408691406250, 0.0051269531250000, 0.0004577636718750, 0.0050354003906250,
+0.0002136230468750, 0.0050964355468750, 0.0002746582031250, 0.0045166015625000,
+0.0000305175781250, 0.0047607421875000, -0.0000305175781250, 0.0043029785156250,
+0.0000000000000000, 0.0037231445312500, -0.0001525878906250, 0.0042419433593750,
+-0.0002136230468750, 0.0028381347656250, -0.0002136230468750, 0.0031127929687500,
+-0.0003356933593750, 0.0027160644531250, -0.0003051757812500, 0.0016479492187500,
+-0.0001525878906250, 0.0022583007812500, -0.0003662109375000, 0.0010375976562500,
+-0.0001831054687500, 0.0010681152343750, -0.0001831054687500, 0.0009460449218750,
+-0.0001831054687500, 0.0001831054687500, -0.0001525878906250, 0.0006103515625000,
+-0.0001525878906250, 0.0002136230468750, -0.0000610351562500, 0.0001525878906250,
+-0.0001831054687500, 0.0002441406250000, -0.0000915527343750, 0.0002136230468750,
+0.0000305175781250, 0.0002746582031250, -0.0002136230468750, 0.0003356933593750,
+0.0002136230468750, 0.0003967285156250, 0.0000915527343750, 0.0003356933593750,
+0.0001831054687500, 0.0004272460937500, 0.0006103515625000, 0.0004882812500000,
+0.0004272460937500, 0.0002746582031250, 0.0008544921875000, 0.0004882812500000,
+0.0009765625000000, 0.0003662109375000, 0.0010986328125000, 0.0001525878906250,
+0.0014343261718750, 0.0004882812500000, 0.0014343261718750, 0.0003662109375000,
+0.0014648437500000, 0.0003051757812500, 0.0017089843750000, 0.0005187988281250,
+0.0015563964843750, 0.0006103515625000, 0.0014038085937500, 0.0005798339843750,
+0.0016174316406250, 0.0009155273437500, 0.0011901855468750, 0.0008544921875000,
+0.0011596679687500, 0.0008544921875000, 0.0011291503906250, 0.0010070800781250,
+0.0009765625000000, 0.0010375976562500, 0.0008544921875000, 0.0009460449218750,
+0.0006713867187500, 0.0010986328125000, 0.0007019042968750, 0.0010681152343750,
+0.0005187988281250, 0.0009765625000000, 0.0002746582031250, 0.0011901855468750,
+0.0003356933593750, 0.0010986328125000, 0.0000305175781250, 0.0012512207031250,
+0.0000610351562500, 0.0014038085937500, -0.0000305175781250, 0.0014343261718750,
+-0.0001831054687500, 0.0015258789062500, 0.0000610351562500, 0.0017700195312500,
+-0.0001525878906250, 0.0018920898437500, -0.0000915527343750, 0.0018310546875000,
+-0.0000305175781250, 0.0018920898437500, -0.0001525878906250, 0.0019531250000000,
+0.0001525878906250, 0.0016174316406250, 0.0001220703125000, 0.0015563964843750,
+0.0003051757812500, 0.0016479492187500, 0.0004272460937500, 0.0014038085937500,
+0.0004577636718750, 0.0012817382812500, 0.0006408691406250, 0.0017395019531250,
+0.0005493164062500, 0.0013427734375000, 0.0008239746093750, 0.0015563964843750,
+0.0008544921875000, 0.0021057128906250, 0.0007019042968750, 0.0015869140625000,
+0.0009765625000000, 0.0020141601562500, 0.0007629394531250, 0.0021667480468750,
+0.0010375976562500, 0.0018615722656250, 0.0010070800781250, 0.0018615722656250,
+0.0009155273437500, 0.0021667480468750, 0.0013427734375000, 0.0018920898437500,
+0.0008239746093750, 0.0020141601562500, 0.0011901855468750, 0.0021972656250000,
+0.0011901855468750, 0.0021972656250000, 0.0007629394531250, 0.0024719238281250,
+0.0012207031250000, 0.0021667480468750, 0.0010375976562500, 0.0024414062500000,
+0.0010986328125000, 0.0023803710937500, 0.0010986328125000, 0.0017089843750000,
+0.0011901855468750, 0.0012817382812500, 0.0015869140625000, 0.0007019042968750,
+0.0007324218750000, 0.0009765625000000, 0.0011901855468750, 0.0008544921875000,
+0.0010375976562500, 0.0005187988281250, 0.0003051757812500, 0.0007324218750000,
+0.0008544921875000, 0.0002136230468750, 0.0001831054687500, 0.0003356933593750,
+0.0000000000000000, 0.0001220703125000, 0.0002441406250000, -0.0003356933593750,
+0.0000000000000000, -0.0000305175781250, -0.0001831054687500, -0.0006103515625000,
+0.0002746582031250, -0.0007629394531250, 0.0000610351562500, -0.0005187988281250,
+0.0000915527343750, -0.0005798339843750, 0.0004272460937500, -0.0006103515625000,
+0.0001525878906250, -0.0002136230468750, 0.0006103515625000, -0.0001831054687500,
+0.0004882812500000, -0.0000305175781250, 0.0004882812500000, 0.0002136230468750,
+0.0007934570312500, 0.0001220703125000, 0.0006103515625000, 0.0006713867187500,
+0.0006713867187500, 0.0005187988281250, 0.0009155273437500, 0.0004577636718750,
+0.0007019042968750, 0.0008544921875000, 0.0005493164062500, 0.0006103515625000,
+0.0009155273437500, 0.0004882812500000, 0.0004577636718750, 0.0005187988281250,
+0.0006103515625000, 0.0003967285156250, 0.0008850097656250, 0.0000305175781250,
+0.0008239746093750, -0.0000915527343750, 0.0010681152343750, -0.0002136230468750,
+0.0012512207031250, -0.0004272460937500, 0.0015563964843750, -0.0003967285156250,
+0.0016174316406250, -0.0003356933593750, 0.0016479492187500, -0.0003967285156250,
+0.0020446777343750, -0.0002441406250000, 0.0017395019531250, -0.0001525878906250,
+0.0019836425781250, -0.0001831054687500, 0.0021362304687500, -0.0000915527343750,
+0.0019226074218750, 0.0000915527343750, 0.0020751953125000, -0.0000610351562500,
+0.0021362304687500, -0.0000610351562500, 0.0019836425781250, 0.0003051757812500,
+0.0021667480468750, 0.0003051757812500, 0.0023803710937500, 0.0001525878906250,
+0.0020141601562500, 0.0008239746093750, 0.0024108886718750, 0.0007629394531250,
+0.0021972656250000, 0.0008544921875000, 0.0019836425781250, 0.0014648437500000,
+0.0020446777343750, 0.0014038085937500, 0.0017700195312500, 0.0013732910156250,
+0.0015563964843750, 0.0018005371093750, 0.0014648437500000, 0.0016784667968750,
+0.0012512207031250, 0.0014953613281250, 0.0009460449218750, 0.0020446777343750,
+0.0010070800781250, 0.0013732910156250, 0.0007934570312500, 0.0017395019531250,
+0.0006713867187500, 0.0016784667968750, 0.0008544921875000, 0.0011596679687500,
+0.0008544921875000},
+{-0.0001220703125000, 0.0012817382812500, -0.0001525878906250, 0.0011596679687500,
+0.0000305175781250, 0.0014953613281250, -0.0000915527343750, 0.0013427734375000,
+-0.0000305175781250, 0.0012512207031250, 0.0001831054687500, 0.0014038085937500,
+0.0002746582031250, 0.0011901855468750, 0.0003662109375000, 0.0010375976562500,
+0.0004577636718750, 0.0011596679687500, 0.0007629394531250, 0.0010681152343750,
+0.0006713867187500, 0.0009155273437500, 0.0005798339843750, 0.0011291503906250,
+0.0010986328125000, 0.0010986328125000, 0.0006103515625000, 0.0011901855468750,
+0.0006713867187500, 0.0013732910156250, 0.0008544921875000, 0.0013732910156250,
+0.0006103515625000, 0.0014038085937500, 0.0006408691406250, 0.0016479492187500,
+0.0005187988281250, 0.0015563964843750, 0.0006408691406250, 0.0016174316406250,
+0.0003967285156250, 0.0016479492187500, 0.0001831054687500, 0.0011901855468750,
+0.0005187988281250, 0.0010070800781250, -0.0001220703125000, 0.0010986328125000,
+0.0000915527343750, 0.0009765625000000, 0.0001525878906250, 0.0007934570312500,
+-0.0003051757812500, 0.0009765625000000, 0.0000305175781250, 0.0006713867187500,
+-0.0000305175781250, 0.0006103515625000, -0.0003051757812500, 0.0005493164062500,
+-0.0000305175781250, 0.0005187988281250, 0.0000915527343750, 0.0006103515625000,
+-0.0003356933593750, 0.0000610351562500, 0.0003967285156250, 0.0003967285156250,
+0.0000915527343750, -0.0001525878906250, 0.0001525878906250, -0.0008850097656250,
+0.0007629394531250, -0.0002136230468750, 0.0003967285156250, -0.0012512207031250,
+0.0006103515625000, -0.0012207031250000, 0.0008239746093750, -0.0009460449218750,
+0.0006408691406250, -0.0013732910156250, 0.0005798339843750, -0.0011901855468750,
+0.0006408691406250, -0.0007934570312500, 0.0006103515625000, -0.0010986328125000,
+0.0005493164062500, -0.0006103515625000, 0.0006713867187500, -0.0000915527343750,
+0.0009155273437500, -0.0006103515625000, 0.0009460449218750, 0.0003967285156250,
+0.0011596679687500, 0.0002441406250000, 0.0015258789062500, 0.0002746582031250,
+0.0015258789062500, 0.0009460449218750, 0.0015869140625000, 0.0004882812500000,
+0.0021362304687500, 0.0006103515625000, 0.0011596679687500, 0.0006408691406250,
+0.0017089843750000, 0.0002136230468750, 0.0016174316406250, 0.0000610351562500,
+0.0008850097656250, 0.0000915527343750, 0.0015258789062500, -0.0003662109375000,
+0.0010070800781250, -0.0003967285156250, 0.0009765625000000, -0.0002441406250000,
+0.0010986328125000, -0.0002441406250000, 0.0007934570312500, -0.0002136230468750,
+0.0007934570312500, 0.0001525878906250, 0.0005493164062500, 0.0002746582031250,
+0.0004577636718750, 0.0004272460937500, 0.0002136230468750, 0.0007019042968750,
+-0.0000305175781250, 0.0007629394531250, -0.0000915527343750, 0.0007934570312500,
+-0.0006713867187500, 0.0009460449218750, -0.0008239746093750, 0.0010070800781250,
+-0.0010070800781250, 0.0010070800781250, -0.0013732910156250, 0.0010986328125000,
+-0.0013732910156250, 0.0012512207031250, -0.0014343261718750, 0.0013732910156250,
+-0.0016784667968750, 0.0014648437500000, -0.0014953613281250, 0.0016174316406250,
+-0.0014648437500000, 0.0017700195312500, -0.0017089843750000, 0.0016174316406250,
+-0.0010681152343750, 0.0018310546875000, -0.0012512207031250, 0.0018005371093750,
+-0.0010681152343750, 0.0016784667968750, -0.0005187988281250, 0.0019226074218750,
+-0.0008544921875000, 0.0015563964843750, -0.0003662109375000, 0.0016479492187500,
+-0.0002746582031250, 0.0015258789062500, -0.0003967285156250, 0.0011901855468750,
+-0.0002441406250000, 0.0013732910156250, -0.0002136230468750, 0.0009765625000000,
+-0.0003051757812500, 0.0009460449218750, -0.0003662109375000, 0.0009765625000000,
+-0.0003356933593750, 0.0008544921875000, -0.0004272460937500, 0.0007019042968750,
+-0.0004882812500000, 0.0007324218750000, -0.0002746582031250, 0.0007019042968750,
+-0.0005493164062500, 0.0004882812500000, -0.0002746582031250, 0.0003662109375000,
+0.0000305175781250, 0.0004882812500000, -0.0003967285156250, 0.0000305175781250,
+0.0006713867187500, 0.0002441406250000, 0.0003051757812500, 0.0002136230468750,
+0.0007629394531250, -0.0000915527343750, 0.0017395019531250, 0.0001220703125000,
+0.0010375976562500, 0.0000915527343750, 0.0024719238281250, 0.0000915527343750,
+0.0022277832031250, 0.0002136230468750, 0.0025024414062500, 0.0002441406250000,
+0.0034790039062500, 0.0002136230468750, 0.0027770996093750, 0.0003356933593750,
+0.0040283203125000, 0.0002441406250000, 0.0037841796875000, 0.0003051757812500,
+0.0039672851562500, 0.0003662109375000, 0.0048217773437500, 0.0001525878906250,
+0.0041198730468750, 0.0003356933593750, 0.0050964355468750, 0.0001831054687500,
+0.0049438476562500, 0.0000915527343750, 0.0048522949218750, 0.0002136230468750,
+0.0053100585937500, 0.0000915527343750, 0.0049743652343750, 0.0001525878906250,
+0.0048522949218750, 0.0002136230468750, 0.0049438476562500, 0.0002136230468750,
+0.0045471191406250, 0.0003051757812500, 0.0041809082031250, 0.0003662109375000,
+0.0044555664062500, 0.0003356933593750, 0.0035095214843750, 0.0005187988281250,
+0.0037231445312500, 0.0005493164062500, 0.0033569335937500, 0.0005187988281250,
+0.0026550292968750, 0.0007324218750000, 0.0031738281250000, 0.0005798339843750,
+0.0021057128906250, 0.0006713867187500, 0.0022583007812500, 0.0007019042968750,
+0.0020751953125000, 0.0006103515625000, 0.0013122558593750, 0.0006713867187500,
+0.0018005371093750, 0.0004882812500000, 0.0007934570312500, 0.0006103515625000,
+0.0009155273437500, 0.0003967285156250, 0.0007934570312500, 0.0002136230468750,
+0.0001831054687500, 0.0006408691406250, 0.0005493164062500, -0.0000610351562500,
+-0.0000305175781250, 0.0003662109375000, -0.0000305175781250, 0.0003967285156250,
+0.0000000000000000, -0.0000305175781250, -0.0001525878906250, 0.0007324218750000,
+0.0000000000000000, 0.0002746582031250, 0.0002136230468750, 0.0004577636718750,
+0.0002441406250000, 0.0007324218750000, 0.0004272460937500, 0.0004577636718750,
+0.0007324218750000, 0.0003662109375000, 0.0007934570312500, 0.0007324218750000,
+0.0009460449218750, 0.0001525878906250, 0.0013732910156250, 0.0003051757812500,
+0.0012512207031250, 0.0006103515625000, 0.0013122558593750, -0.0001831054687500,
+0.0017089843750000, 0.0006408691406250, 0.0006103515625000, 0.0003662109375000,
+0.0012207031250000, 0.0001831054687500, 0.0007629394531250, 0.0006408691406250,
+-0.0002441406250000, 0.0005493164062500, 0.0006408691406250, 0.0004577636718750,
+-0.0010070800781250, 0.0008544921875000, -0.0005493164062500, 0.0007324218750000,
+-0.0005798339843750, 0.0004882812500000, -0.0015563964843750, 0.0009765625000000,
+-0.0007629394531250, 0.0002746582031250, -0.0014953613281250, 0.0006103515625000,
+-0.0012207031250000, 0.0005187988281250, -0.0012207031250000, 0.0000915527343750,
+-0.0016479492187500, 0.0005798339843750, -0.0011596679687500, 0.0000610351562500,
+-0.0018615722656250, 0.0003967285156250, -0.0017700195312500, 0.0002441406250000,
+-0.0018005371093750, -0.0001220703125000, -0.0022277832031250, 0.0006408691406250,
+-0.0020141601562500, -0.0003051757812500, -0.0020446777343750, 0.0001525878906250,
+-0.0022583007812500, 0.0000305175781250, -0.0019226074218750, -0.0007019042968750,
+-0.0018310546875000, 0.0000305175781250, -0.0020141601562500, -0.0009155273437500,
+-0.0009155273437500, -0.0006408691406250, -0.0013122558593750, -0.0007324218750000,
+-0.0008544921875000, -0.0012817382812500, 0.0000915527343750, -0.0008544921875000,
+-0.0006103515625000, -0.0015258789062500, 0.0004882812500000, -0.0013732910156250,
+0.0003662109375000, -0.0014343261718750, 0.0003967285156250, -0.0018310546875000,
+0.0010681152343750, -0.0015563964843750, 0.0008850097656250, -0.0017089843750000,
+0.0014343261718750, -0.0018005371093750, 0.0014343261718750, -0.0014343261718750,
+0.0017395019531250, -0.0012817382812500, 0.0023803710937500, -0.0014953613281250,
+0.0022583007812500, -0.0009155273437500, 0.0025024414062500, -0.0009155273437500,
+0.0030822753906250, -0.0012207031250000, 0.0023498535156250, -0.0010681152343750,
+0.0020446777343750, -0.0009460449218750, 0.0028686523437500, -0.0014343261718750,
+0.0003356933593750, -0.0013122558593750, 0.0010070800781250, -0.0010986328125000,
+0.0008239746093750, -0.0013122558593750, -0.0021972656250000, -0.0005187988281250,
+0.0024719238281250, -0.0014953613281250, 0.0006713867187500, -0.0009765625000000,
+0.0017089843750000, -0.0010375976562500, 0.0007629394531250, -0.0016174316406250,
+-0.0006408691406250, -0.0012817382812500, -0.0000305175781250, -0.0008544921875000,
+-0.0013122558593750, -0.0011291503906250, -0.0012817382812500, -0.0008239746093750,
+-0.0014953613281250, -0.0001831054687500, -0.0024108886718750, -0.0005187988281250,
+-0.0020751953125000, -0.0006103515625000, -0.0025939941406250, -0.0001831054687500,
+-0.0029907226562500, -0.0004882812500000, -0.0024414062500000, -0.0009765625000000,
+-0.0021667480468750, -0.0003662109375000, -0.0025024414062500, -0.0005187988281250,
+-0.0011596679687500, -0.0008239746093750, -0.0010375976562500, 0.0000915527343750,
+-0.0009765625000000, 0.0006713867187500, -0.0001220703125000, 0.0000915527343750,
+0.0000000000000000, 0.0017395019531250, -0.0004882812500000, 0.0015869140625000,
+0.0001831054687500, 0.0014038085937500, -0.0001525878906250, 0.0021667480468750,
+-0.0007934570312500, 0.0018310546875000, -0.0000610351562500, 0.0016784667968750,
+-0.0010986328125000, 0.0017700195312500, -0.0008544921875000, 0.0014343261718750,
+-0.0007629394531250, 0.0011291503906250, -0.0014648437500000, 0.0013122558593750,
+-0.0010986328125000, 0.0006103515625000, -0.0010986328125000, 0.0006103515625000,
+-0.0014038085937500, 0.0002746582031250, -0.0007934570312500, -0.0003051757812500,
+-0.0003051757812500, -0.0001220703125000, -0.0007019042968750, -0.0012512207031250,
+0.0005798339843750, -0.0012207031250000, 0.0006408691406250, -0.0012817382812500,
+0.0006408691406250, -0.0021057128906250, 0.0013732910156250, -0.0021362304687500,
+0.0012512207031250, -0.0014953613281250, 0.0010986328125000, -0.0021972656250000,
+0.0011901855468750, -0.0017089843750000, 0.0009460449218750, -0.0008239746093750,
+0.0005187988281250, -0.0017089843750000, 0.0006408691406250, -0.0003662109375000,
+0.0001525878906250, -0.0006103515625000, 0.0001831054687500, -0.0007324218750000,
+-0.0000305175781250, 0.0001525878906250, -0.0003356933593750, -0.0003051757812500,
+-0.0001525878906250, -0.0005493164062500, -0.0004577636718750, -0.0001831054687500,
+-0.0005187988281250, -0.0008239746093750, -0.0003051757812500, -0.0015258789062500,
+-0.0003051757812500, -0.0010375976562500, -0.0003356933593750, -0.0023193359375000,
+0.0000915527343750, -0.0023803710937500, -0.0000305175781250, -0.0022888183593750,
+0.0000000000000000, -0.0029907226562500, 0.0002136230468750, -0.0028381347656250,
+-0.0000610351562500, -0.0022583007812500, 0.0002441406250000, -0.0026550292968750,
+-0.0000305175781250, -0.0018615722656250, 0.0000305175781250, -0.0009155273437500,
+0.0003051757812500, -0.0016479492187500, -0.0000305175781250, 0.0003051757812500,
+0.0001525878906250, 0.0001220703125000, 0.0003356933593750, 0.0004882812500000,
+0.0000610351562500, 0.0018615722656250, -0.0000305175781250, 0.0012207031250000,
+0.0003356933593750, 0.0022888183593750, -0.0004272460937500, 0.0025329589843750,
+0.0000305175781250, 0.0023193359375000, 0.0000000000000000, 0.0028381347656250,
+-0.0004272460937500, 0.0028991699218750, 0.0003356933593750, 0.0021362304687500,
+-0.0003051757812500},
+{0.0000915527343750, -0.0008850097656250, 0.0017395019531250, -0.0006713867187500,
+0.0019226074218750, -0.0007629394531250, 0.0016784667968750, -0.0009460449218750,
+0.0024108886718750, -0.0007324218750000, -0.0026245117187500, -0.0001831054687500,
+0.0005798339843750, -0.0004577636718750, -0.0005493164062500, -0.0003051757812500,
+0.0000000000000000, 0.0001525878906250, 0.0015258789062500, 0.0001220703125000,
+0.0007934570312500, -0.0002136230468750, 0.0010986328125000, 0.0000610351562500,
+0.0011596679687500, -0.0000610351562500, 0.0007019042968750, -0.0005493164062500,
+0.0005798339843750, -0.0002136230468750, 0.0006408691406250, 0.0001525878906250,
+-0.0002136230468750, 0.0000000000000000, 0.0001220703125000, 0.0005493164062500,
+-0.0004882812500000, 0.0012817382812500, -0.0015258789062500, 0.0011596679687500,
+-0.0008544921875000, 0.0018005371093750, -0.0023193359375000, 0.0021667480468750,
+-0.0023803710937500, 0.0017700195312500, -0.0021057128906250, 0.0017089843750000,
+-0.0026855468750000, 0.0021362304687500, -0.0025329589843750, 0.0007629394531250,
+-0.0019531250000000, 0.0009155273437500, -0.0019226074218750, 0.0006103515625000,
+-0.0016174316406250, -0.0003356933593750, -0.0009155273437500, 0.0002136230468750,
+-0.0008850097656250, -0.0008850097656250, -0.0006103515625000, -0.0005798339843750,
+-0.0001831054687500, -0.0008239746093750, -0.0002746582031250, -0.0016479492187500,
+-0.0002441406250000, -0.0010681152343750, 0.0001831054687500, -0.0018615722656250,
+-0.0005187988281250, -0.0020141601562500, -0.0002746582031250, -0.0018615722656250,
+-0.0005493164062500, -0.0022583007812500, -0.0012817382812500, -0.0022583007812500,
+-0.0008850097656250, -0.0011291503906250, -0.0014648437500000, -0.0017089843750000,
+-0.0016784667968750, -0.0009460449218750, -0.0012512207031250, 0.0003967285156250,
+-0.0011596679687500, -0.0005187988281250, -0.0013732910156250, 0.0009765625000000,
+-0.0002136230468750, 0.0010986328125000, -0.0002746582031250, 0.0008544921875000,
+0.0000000000000000, 0.0013427734375000, 0.0008850097656250, 0.0010986328125000,
+0.0006103515625000, 0.0005798339843750, 0.0008850097656250, 0.0005798339843750,
+0.0009765625000000, 0.0002136230468750, 0.0007934570312500, -0.0003967285156250,
+0.0007629394531250, -0.0003967285156250, 0.0007629394531250, -0.0001525878906250,
+0.0003051757812500, -0.0009155273437500, 0.0003967285156250, -0.0001831054687500,
+0.0002136230468750, 0.0006408691406250, -0.0002136230468750, -0.0003967285156250,
+0.0000305175781250, 0.0020751953125000, -0.0002746582031250, 0.0016479492187500,
+-0.0003356933593750, 0.0022583007812500, -0.0001831054687500, 0.0041503906250000,
+-0.0001831054687500, 0.0032348632812500, -0.0001525878906250, 0.0048828125000000,
+-0.0000610351562500, 0.0051574707031250, -0.0000915527343750, 0.0049743652343750,
+-0.0001525878906250, 0.0055541992187500, -0.0001831054687500, 0.0054931640625000,
+-0.0002746582031250, 0.0048828125000000, -0.0001831054687500, 0.0052490234375000,
+-0.0003051757812500, 0.0047607421875000, -0.0002136230468750, 0.0040283203125000,
+0.0000610351562500, 0.0045776367187500, -0.0000915527343750, 0.0031127929687500,
+0.0000610351562500, 0.0034484863281250, 0.0003051757812500, 0.0028686523437500,
+0.0001831054687500, 0.0016479492187500, 0.0001831054687500, 0.0024414062500000,
+0.0006103515625000, 0.0006713867187500, 0.0001831054687500, 0.0008544921875000,
+0.0005187988281250, 0.0006103515625000, 0.0005493164062500, -0.0004882812500000,
+0.0001831054687500, 0.0001831054687500, 0.0006408691406250, -0.0009155273437500,
+0.0001525878906250, -0.0008850097656250, 0.0002136230468750, -0.0008850097656250,
+0.0002136230468750, -0.0015258789062500, -0.0000915527343750, -0.0012207031250000,
+0.0000000000000000, -0.0014648437500000, -0.0001525878906250, -0.0016479492187500,
+-0.0001525878906250, -0.0013732910156250, -0.0002441406250000, -0.0013122558593750,
+-0.0003967285156250, -0.0015563964843750, -0.0002441406250000, -0.0006408691406250,
+-0.0005187988281250, -0.0009765625000000, -0.0003967285156250, -0.0004882812500000,
+-0.0002441406250000, 0.0003356933593750, -0.0003662109375000, -0.0002746582031250,
+-0.0003662109375000, 0.0012207031250000, -0.0001220703125000, 0.0009765625000000,
+-0.0002136230468750, 0.0014038085937500, -0.0001525878906250, 0.0025024414062500,
+0.0000610351562500, 0.0018005371093750, -0.0000305175781250, 0.0032348632812500,
+0.0001220703125000, 0.0029602050781250, 0.0001831054687500, 0.0033264160156250,
+0.0001831054687500, 0.0044250488281250, 0.0003051757812500, 0.0037536621093750,
+0.0003356933593750, 0.0049438476562500, 0.0003967285156250, 0.0049133300781250,
+0.0003662109375000, 0.0049743652343750, 0.0004577636718750, 0.0056152343750000,
+0.0005493164062500, 0.0053405761718750, 0.0004272460937500, 0.0055236816406250,
+0.0007324218750000, 0.0056457519531250, 0.0006103515625000, 0.0054321289062500,
+0.0008239746093750, 0.0053405761718750, 0.0011901855468750, 0.0054626464843750,
+0.0008850097656250, 0.0051574707031250, 0.0014038085937500, 0.0051879882812500,
+0.0014648437500000, 0.0050659179687500, 0.0014648437500000, 0.0048828125000000,
+0.0017395019531250, 0.0050659179687500, 0.0016784667968750, 0.0046081542968750,
+0.0017089843750000, 0.0048828125000000, 0.0018005371093750, 0.0046386718750000,
+0.0016479492187500, 0.0042114257812500, 0.0015258789062500, 0.0046997070312500,
+0.0016174316406250, 0.0037536621093750, 0.0012512207031250, 0.0040283203125000,
+0.0013122558593750, 0.0037841796875000, 0.0011596679687500, 0.0029907226562500,
+0.0008850097656250, 0.0034484863281250, 0.0010070800781250, 0.0026855468750000,
+0.0007019042968750, 0.0026855468750000, 0.0007324218750000, 0.0028991699218750,
+0.0006713867187500, 0.0026550292968750, 0.0005798339843750, 0.0028381347656250,
+0.0007324218750000, 0.0033264160156250, 0.0003967285156250, 0.0031433105468750,
+0.0007019042968750, 0.0035705566406250, 0.0005798339843750, 0.0042114257812500,
+0.0003356933593750, 0.0039672851562500, 0.0008239746093750, 0.0048522949218750,
+0.0001220703125000, 0.0049133300781250, 0.0003356933593750, 0.0050048828125000,
+0.0003662109375000, 0.0055847167968750, -0.0000915527343750, 0.0054016113281250,
+0.0000915527343750, 0.0054626464843750, -0.0000305175781250, 0.0057373046875000,
+-0.0001525878906250, 0.0054931640625000, -0.0001220703125000, 0.0053100585937500,
+-0.0001525878906250, 0.0055847167968750, -0.0002136230468750, 0.0048217773437500,
+-0.0002441406250000, 0.0050048828125000, -0.0003051757812500, 0.0047607421875000,
+-0.0002746582031250, 0.0040893554687500, -0.0001831054687500, 0.0044860839843750,
+-0.0002746582031250, 0.0036010742187500, 0.0000000000000000, 0.0039062500000000,
+-0.0000610351562500, 0.0032653808593750, -0.0000305175781250, 0.0023803710937500,
+0.0002136230468750, 0.0030517578125000, 0.0000610351562500, 0.0010986328125000,
+0.0000915527343750, 0.0013427734375000, 0.0001831054687500, 0.0009460449218750,
+0.0000915527343750, -0.0005493164062500, 0.0001220703125000, 0.0000000000000000,
+0.0002136230468750, -0.0005187988281250, -0.0001220703125000, -0.0009155273437500,
+-0.0000610351562500, -0.0007629394531250, -0.0001220703125000, -0.0007934570312500,
+-0.0003662109375000, -0.0010375976562500, -0.0003662109375000, -0.0010986328125000,
+-0.0004577636718750, -0.0010681152343750, -0.0006713867187500, -0.0013122558593750,
+-0.0005493164062500, -0.0015869140625000, -0.0004882812500000, -0.0014038085937500,
+-0.0008239746093750, -0.0013122558593750, -0.0003356933593750, -0.0014648437500000,
+-0.0005798339843750, -0.0011901855468750, -0.0006103515625000, -0.0003967285156250,
+-0.0002746582031250, 0.0001831054687500, -0.0005493164062500, 0.0005493164062500,
+-0.0001220703125000, 0.0006408691406250, -0.0004882812500000, 0.0007629394531250,
+-0.0003662109375000, 0.0009765625000000, -0.0000610351562500, 0.0010681152343750,
+-0.0007324218750000, 0.0014038085937500, 0.0000000000000000, 0.0012817382812500,
+-0.0001220703125000, 0.0012512207031250, -0.0002136230468750, 0.0014648437500000,
+0.0000610351562500, 0.0013732910156250, -0.0002441406250000, 0.0010681152343750,
+-0.0000915527343750, 0.0011901855468750, -0.0001831054687500, 0.0009765625000000,
+-0.0001525878906250, 0.0006408691406250, -0.0000305175781250, 0.0008239746093750,
+-0.0002136230468750, 0.0005798339843750, -0.0000305175781250, 0.0004882812500000,
+0.0000610351562500, 0.0004882812500000, -0.0001831054687500, 0.0004882812500000,
+-0.0001525878906250, 0.0005493164062500, 0.0000610351562500, 0.0000915527343750,
+-0.0005187988281250, 0.0004577636718750, -0.0003967285156250, 0.0000000000000000,
+-0.0003662109375000, -0.0006103515625000, -0.0007629394531250, 0.0000610351562500,
+-0.0006408691406250, -0.0010375976562500, -0.0006408691406250, -0.0008239746093750,
+-0.0008239746093750, -0.0008850097656250, -0.0007324218750000, -0.0014953613281250,
+-0.0005798339843750, -0.0010070800781250, -0.0007019042968750, -0.0013427734375000,
+-0.0005493164062500, -0.0013732910156250, -0.0005187988281250, -0.0008850097656250,
+-0.0004882812500000, -0.0007934570312500, -0.0004577636718750, -0.0007934570312500,
+-0.0004882812500000, 0.0002441406250000, -0.0002136230468750, 0.0000305175781250,
+-0.0002136230468750, 0.0002746582031250, -0.0001831054687500, 0.0011291503906250,
+-0.0000305175781250, 0.0006713867187500, -0.0000610351562500, 0.0007019042968750,
+0.0001831054687500, 0.0010681152343750, 0.0001831054687500, 0.0006713867187500,
+0.0004272460937500, 0.0002136230468750, 0.0006713867187500, 0.0005798339843750,
+0.0005798339843750, -0.0000305175781250, 0.0012817382812500, -0.0000915527343750,
+0.0011901855468750, 0.0000915527343750, 0.0016174316406250, -0.0001220703125000,
+0.0023498535156250, -0.0001831054687500, 0.0020141601562500, 0.0002746582031250,
+0.0031127929687500, 0.0000610351562500, 0.0031433105468750, 0.0001525878906250,
+0.0032653808593750, 0.0005493164062500, 0.0038757324218750, 0.0001831054687500,
+0.0036315917968750, 0.0005798339843750, 0.0037536621093750, 0.0006408691406250,
+0.0038452148437500, 0.0005798339843750, 0.0036926269531250, 0.0007934570312500,
+0.0036315917968750, 0.0009155273437500, 0.0036010742187500, 0.0008239746093750,
+0.0034790039062500, 0.0010681152343750, 0.0033874511718750, 0.0010986328125000,
+0.0033569335937500, 0.0009460449218750, 0.0033264160156250, 0.0012207031250000,
+0.0031738281250000, 0.0011291503906250, 0.0032958984375000, 0.0011596679687500,
+0.0030517578125000, 0.0011596679687500, 0.0031433105468750, 0.0011901855468750,
+0.0032958984375000, 0.0012207031250000, 0.0029907226562500, 0.0010681152343750,
+0.0034179687500000, 0.0010681152343750, 0.0033264160156250, 0.0010986328125000,
+0.0032043457031250, 0.0009155273437500, 0.0034179687500000, 0.0008544921875000,
+0.0032653808593750, 0.0009460449218750, 0.0032043457031250, 0.0007629394531250,
+0.0032958984375000, 0.0007324218750000, 0.0032043457031250, 0.0007934570312500,
+0.0030517578125000, 0.0006408691406250, 0.0031433105468750, 0.0006103515625000,
+0.0028381347656250, 0.0005493164062500, 0.0029296875000000, 0.0003967285156250,
+0.0028686523437500, 0.0003051757812500, 0.0026245117187500, 0.0002746582031250,
+0.0027770996093750, -0.0001220703125000, 0.0024719238281250, -0.0001525878906250,
+0.0025939941406250, -0.0002746582031250, 0.0025024414062500, -0.0005493164062500,
+0.0022583007812500, -0.0004272460937500, 0.0025634765625000, -0.0004577636718750,
+0.0023193359375000, -0.0005493164062500, 0.0023498535156250, -0.0004272460937500,
+0.0025939941406250},
+{-0.0006408691406250, 0.0010986328125000, -0.0008239746093750, 0.0020141601562500,
+-0.0010986328125000, 0.0014038085937500, -0.0007019042968750, 0.0015258789062500,
+-0.0013122558593750, 0.0017395019531250, -0.0012207031250000, 0.0013427734375000,
+-0.0010070800781250, 0.0009155273437500, -0.0012817382812500, 0.0011596679687500,
+-0.0011901855468750, 0.0006713867187500, -0.0007324218750000, 0.0007629394531250,
+-0.0008544921875000, 0.0006713867187500, -0.0006103515625000, 0.0004577636718750,
+-0.0002136230468750, 0.0007934570312500, -0.0002746582031250, 0.0001831054687500,
+0.0001525878906250, 0.0005493164062500, 0.0000610351562500, 0.0003356933593750,
+0.0001831054687500, -0.0001831054687500, 0.0005187988281250, 0.0003356933593750,
+0.0001525878906250, -0.0006713867187500, 0.0007629394531250, -0.0003662109375000,
+0.0005187988281250, -0.0004882812500000, 0.0006713867187500, -0.0011901855468750,
+0.0010681152343750, -0.0005493164062500, 0.0007324218750000, -0.0010681152343750,
+0.0011901855468750, -0.0011596679687500, 0.0011596679687500, -0.0008239746093750,
+0.0012512207031250, -0.0009460449218750, 0.0015869140625000, -0.0010375976562500,
+0.0015258789062500, -0.0003051757812500, 0.0020751953125000, -0.0005493164062500,
+0.0021667480468750, -0.0003967285156250, 0.0026550292968750, 0.0001831054687500,
+0.0032653808593750, -0.0001831054687500, 0.0031127929687500, 0.0001831054687500,
+0.0039978027343750, 0.0001831054687500, 0.0041198730468750, 0.0000305175781250,
+0.0043640136718750, 0.0000915527343750, 0.0049438476562500, -0.0000915527343750,
+0.0048522949218750, 0.0000000000000000, 0.0049438476562500, -0.0001831054687500,
+0.0050964355468750, -0.0000610351562500, 0.0051269531250000, 0.0000915527343750,
+0.0050354003906250, -0.0001220703125000, 0.0050354003906250, 0.0001831054687500,
+0.0053405761718750, 0.0001831054687500, 0.0050964355468750, 0.0001831054687500,
+0.0053405761718750, 0.0003662109375000, 0.0056762695312500, 0.0003356933593750,
+0.0053405761718750, 0.0004882812500000, 0.0059509277343750, 0.0005798339843750,
+0.0056762695312500, 0.0006103515625000, 0.0059204101562500, 0.0006713867187500,
+0.0064392089843750, 0.0007324218750000, 0.0059814453125000, 0.0006713867187500,
+0.0068359375000000, 0.0007934570312500, 0.0067443847656250, 0.0008544921875000,
+0.0067443847656250, 0.0008239746093750, 0.0072937011718750, 0.0009765625000000,
+0.0071105957031250, 0.0011596679687500, 0.0071411132812500, 0.0011596679687500,
+0.0073547363281250, 0.0012512207031250, 0.0071105957031250, 0.0013427734375000,
+0.0068359375000000, 0.0012207031250000, 0.0071411132812500, 0.0013122558593750,
+0.0065917968750000, 0.0011596679687500, 0.0066833496093750, 0.0010070800781250,
+0.0067749023437500, 0.0009155273437500, 0.0064086914062500, 0.0007324218750000,
+0.0065002441406250, 0.0005798339843750, 0.0065002441406250, 0.0004272460937500,
+0.0063476562500000, 0.0002136230468750, 0.0064392089843750, 0.0000305175781250,
+0.0065002441406250, -0.0000610351562500, 0.0063781738281250, -0.0001525878906250,
+0.0066223144531250, -0.0001831054687500, 0.0065612792968750, -0.0002136230468750,
+0.0064697265625000, -0.0002136230468750, 0.0065002441406250, -0.0001831054687500,
+0.0064392089843750, -0.0003051757812500, 0.0060424804687500, -0.0001831054687500,
+0.0061035156250000, -0.0002441406250000, 0.0057067871093750, -0.0003356933593750,
+0.0052185058593750, -0.0001220703125000, 0.0053710937500000, -0.0002746582031250,
+0.0041198730468750, -0.0001831054687500, 0.0043029785156250, -0.0001525878906250,
+0.0037231445312500, -0.0001831054687500, 0.0026245117187500, -0.0000915527343750,
+0.0031127929687500, -0.0002746582031250, 0.0016174316406250, -0.0000305175781250,
+0.0015869140625000, -0.0001220703125000, 0.0014648437500000, -0.0002746582031250,
+0.0004577636718750, 0.0000915527343750, 0.0007019042968750, -0.0003356933593750,
+0.0006103515625000, 0.0000000000000000, 0.0003051757812500, -0.0000305175781250,
+0.0007019042968750, -0.0003051757812500, 0.0009460449218750, 0.0003356933593750,
+0.0006103515625000, -0.0001525878906250, 0.0017395019531250, 0.0000915527343750,
+0.0015869140625000, 0.0003662109375000, 0.0019226074218750, 0.0000305175781250,
+0.0027770996093750, 0.0001220703125000, 0.0024108886718750, 0.0004272460937500,
+0.0029907226562500, -0.0000610351562500, 0.0032348632812500, 0.0001220703125000,
+0.0030822753906250, 0.0004577636718750, 0.0031738281250000, -0.0004577636718750,
+0.0032348632812500, 0.0006408691406250, 0.0027465820312500, 0.0000610351562500,
+0.0029602050781250, 0.0000610351562500, 0.0026245117187500, 0.0009155273437500,
+0.0020751953125000, 0.0001831054687500, 0.0024108886718750, 0.0007019042968750,
+0.0019226074218750, 0.0007934570312500, 0.0018615722656250, 0.0005798339843750,
+0.0020141601562500, 0.0007324218750000, 0.0019531250000000, 0.0007934570312500,
+0.0019531250000000, 0.0005798339843750, 0.0020141601562500, 0.0006408691406250,
+0.0020141601562500, 0.0005798339843750, 0.0017700195312500, 0.0004577636718750,
+0.0015869140625000, 0.0004577636718750, 0.0016174316406250, 0.0000915527343750,
+0.0008239746093750, 0.0002746582031250, 0.0008544921875000, 0.0000305175781250,
+0.0005493164062500, -0.0002441406250000, -0.0001525878906250, 0.0001220703125000,
+-0.0000305175781250, -0.0004882812500000, -0.0002441406250000, -0.0001831054687500,
+-0.0006713867187500, -0.0003051757812500, -0.0003356933593750, -0.0006408691406250,
+-0.0000915527343750, -0.0000305175781250, -0.0005187988281250, -0.0008850097656250,
+0.0006408691406250, -0.0005187988281250, 0.0005798339843750, -0.0006103515625000,
+0.0006408691406250, -0.0012512207031250, 0.0012512207031250, -0.0007019042968750,
+0.0009460449218750, -0.0010986328125000, 0.0014038085937500, -0.0012207031250000,
+0.0014038085937500, -0.0009460449218750, 0.0013732910156250, -0.0009460449218750,
+0.0014953613281250, -0.0009460449218750, 0.0015563964843750, -0.0005493164062500,
+0.0018615722656250, -0.0006408691406250, 0.0017700195312500, -0.0003662109375000,
+0.0022888183593750, -0.0001831054687500, 0.0028076171875000, -0.0005798339843750,
+0.0025634765625000, -0.0000610351562500, 0.0035400390625000, -0.0005187988281250,
+0.0034790039062500, -0.0003356933593750, 0.0036926269531250, 0.0001220703125000,
+0.0044555664062500, -0.0005187988281250, 0.0041503906250000, 0.0001220703125000,
+0.0040893554687500, -0.0000305175781250, 0.0045471191406250, -0.0002136230468750,
+0.0038146972656250, 0.0000610351562500, 0.0030822753906250, -0.0001220703125000,
+0.0036010742187500, -0.0002746582031250, 0.0018920898437500, -0.0003051757812500,
+0.0020446777343750, -0.0002441406250000, 0.0018005371093750, -0.0003662109375000,
+0.0008239746093750, -0.0004882812500000, 0.0014648437500000, 0.0001831054687500,
+-0.0001220703125000, -0.0000915527343750, 0.0003662109375000, -0.0000305175781250,
+-0.0002136230468750, 0.0007934570312500, -0.0028686523437500, 0.0007019042968750,
+0.0034179687500000, 0.0011901855468750, 0.0021362304687500, 0.0009460449218750,
+0.0034484863281250, 0.0013427734375000, 0.0017700195312500, 0.0019226074218750,
+-0.0000610351562500, 0.0018310546875000, 0.0013122558593750, 0.0017395019531250,
+-0.0012512207031250, 0.0024414062500000, -0.0014648437500000, 0.0018310546875000,
+-0.0010681152343750, 0.0011596679687500, -0.0020751953125000, 0.0021057128906250,
+-0.0020446777343750, 0.0003356933593750, -0.0014648437500000, 0.0003662109375000,
+-0.0019226074218750, 0.0005493164062500, -0.0015563964843750, -0.0003356933593750,
+-0.0010375976562500, -0.0001525878906250, -0.0017395019531250, -0.0001831054687500,
+-0.0003051757812500, -0.0003051757812500, -0.0008239746093750, -0.0003967285156250,
+-0.0003662109375000, -0.0001831054687500, 0.0010375976562500, 0.0000000000000000,
+0.0000610351562500, -0.0002746582031250, 0.0012512207031250, 0.0000610351562500,
+0.0016479492187500, 0.0004272460937500, 0.0010070800781250, 0.0005187988281250,
+0.0011291503906250, 0.0008239746093750, 0.0014953613281250, 0.0015563964843750,
+-0.0000915527343750, 0.0017089843750000, 0.0002746582031250, 0.0018310546875000,
+-0.0000305175781250, 0.0022583007812500, -0.0012207031250000, 0.0023803710937500,
+-0.0006103515625000, 0.0020141601562500, -0.0012512207031250, 0.0022888183593750,
+-0.0013732910156250, 0.0020446777343750, -0.0010681152343750, 0.0014038085937500,
+-0.0010375976562500, 0.0018310546875000, -0.0009765625000000, 0.0017700195312500,
+-0.0004272460937500, 0.0011596679687500, -0.0004882812500000, 0.0021667480468750,
+-0.0002136230468750, 0.0029602050781250, 0.0002136230468750, 0.0020751953125000,
+0.0001220703125000, 0.0052185058593750, 0.0001831054687500, 0.0043945312500000,
+0.0004272460937500, 0.0055541992187500, 0.0001220703125000, 0.0082092285156250,
+-0.0002746582031250, 0.0065917968750000, 0.0000000000000000, 0.0095825195312500,
+-0.0006103515625000, 0.0093688964843750, -0.0007019042968750, 0.0092773437500000,
+-0.0005493164062500, 0.0107421875000000, -0.0007629394531250, 0.0098876953125000,
+-0.0007934570312500, 0.0099182128906250, -0.0003662109375000, 0.0099792480468750,
+-0.0004577636718750, 0.0093383789062500, -0.0002746582031250, 0.0088500976562500,
+0.0000915527343750, 0.0089111328125000, -0.0000915527343750, 0.0077819824218750,
+0.0002136230468750, 0.0077514648437500, 0.0003356933593750, 0.0074157714843750,
+0.0001525878906250, 0.0065917968750000, 0.0001831054687500, 0.0068969726562500,
+0.0003662109375000, 0.0063476562500000, -0.0001525878906250, 0.0064697265625000,
+0.0000000000000000, 0.0064086914062500, 0.0000000000000000, 0.0062561035156250,
+-0.0004577636718750, 0.0067443847656250, -0.0002746582031250, 0.0060119628906250,
+-0.0002441406250000, 0.0064392089843750, -0.0004882812500000, 0.0065917968750000,
+-0.0003051757812500, 0.0061645507812500, -0.0001525878906250, 0.0067138671875000,
+-0.0005187988281250, 0.0066528320312500, -0.0000915527343750, 0.0066833496093750,
+-0.0003051757812500, 0.0071716308593750, -0.0002746582031250, 0.0074462890625000,
+-0.0000305175781250, 0.0074462890625000, -0.0004272460937500, 0.0082397460937500,
+-0.0001220703125000, 0.0082397460937500, -0.0002441406250000, 0.0084533691406250,
+-0.0003662109375000, 0.0090026855468750, -0.0002441406250000, 0.0089416503906250,
+-0.0003662109375000, 0.0092163085937500, -0.0003356933593750, 0.0094909667968750,
+-0.0003662109375000, 0.0093078613281250, -0.0002746582031250, 0.0092773437500000,
+-0.0002441406250000, 0.0095214843750000, -0.0002136230468750, 0.0088500976562500,
+0.0000610351562500, 0.0089721679687500, 0.0000000000000000, 0.0085449218750000,
+0.0000610351562500, 0.0078430175781250, 0.0002136230468750, 0.0082397460937500,
+0.0000305175781250, 0.0070800781250000, 0.0001220703125000, 0.0070800781250000,
+0.0000915527343750, 0.0068664550781250, 0.0000000000000000, 0.0061035156250000,
+0.0000305175781250, 0.0062255859375000, 0.0000610351562500, 0.0057373046875000,
+-0.0000915527343750, 0.0055236816406250, -0.0000305175781250, 0.0054321289062500,
+0.0000305175781250, 0.0051574707031250, -0.0000915527343750, 0.0050354003906250,
+-0.0000915527343750, 0.0048522949218750, 0.0001220703125000, 0.0045776367187500,
+-0.0000610351562500, 0.0044250488281250, 0.0001831054687500, 0.0041503906250000,
+0.0005493164062500, 0.0038757324218750, 0.0002136230468750, 0.0036315917968750,
+0.0010070800781250, 0.0033264160156250, 0.0009155273437500, 0.0030822753906250,
+0.0009765625000000, 0.0028686523437500, 0.0014343261718750, 0.0025634765625000,
+0.0012817382812500},
+{0.0008239746093750, 0.0024719238281250, 0.0009155273437500, 0.0031433105468750,
+0.0000915527343750, 0.0027160644531250, 0.0003967285156250, 0.0038757324218750,
+0.0001220703125000, 0.0051574707031250, -0.0005798339843750, 0.0043640136718750,
+-0.0001525878906250, 0.0072326660156250, -0.0006103515625000, 0.0071411132812500,
+-0.0006408691406250, 0.0074768066406250, -0.0004882812500000, 0.0093078613281250,
+-0.0007019042968750, 0.0086364746093750, -0.0005798339843750, 0.0091552734375000,
+-0.0001831054687500, 0.0096435546875000, -0.0004272460937500, 0.0088195800781250,
+-0.0000915527343750, 0.0083618164062500, 0.0004272460937500, 0.0088806152343750,
+0.0000915527343750, 0.0065612792968750, 0.0005798339843750, 0.0070190429687500,
+0.0007019042968750, 0.0062255859375000, 0.0004577636718750, 0.0043945312500000,
+0.0003967285156250, 0.0053710937500000, 0.0004882812500000, 0.0032348632812500,
+-0.0000305175781250, 0.0033264160156250, 0.0000000000000000, 0.0029296875000000,
+-0.0000915527343750, 0.0014953613281250, -0.0003967285156250, 0.0021057128906250,
+-0.0003051757812500, 0.0009460449218750, -0.0003051757812500, 0.0008850097656250,
+-0.0004272460937500, 0.0008239746093750, -0.0002746582031250, 0.0003051757812500,
+-0.0001525878906250, 0.0005493164062500, -0.0002136230468750, -0.0000610351562500,
+-0.0000305175781250, 0.0000610351562500, 0.0000915527343750, -0.0001220703125000,
+0.0000915527343750, -0.0004882812500000, 0.0000610351562500, -0.0000915527343750,
+0.0001220703125000, -0.0003967285156250, 0.0000610351562500, -0.0002136230468750,
+-0.0000305175781250, 0.0001525878906250, 0.0000305175781250, 0.0002136230468750,
+-0.0000915527343750, 0.0004272460937500, -0.0003662109375000, 0.0008544921875000,
+-0.0001220703125000, 0.0010375976562500, -0.0005187988281250, 0.0012207031250000,
+-0.0005493164062500, 0.0015563964843750, -0.0003051757812500, 0.0018005371093750,
+-0.0008544921875000, 0.0018920898437500, -0.0004882812500000, 0.0022888183593750,
+-0.0007934570312500, 0.0023803710937500, -0.0008544921875000, 0.0024414062500000,
+-0.0005798339843750, 0.0028076171875000, -0.0008850097656250, 0.0027160644531250,
+-0.0007324218750000, 0.0031433105468750, -0.0007629394531250, 0.0034179687500000,
+-0.0007629394531250, 0.0033264160156250, -0.0006408691406250, 0.0037536621093750,
+-0.0006713867187500, 0.0043334960937500, -0.0004882812500000, 0.0043640136718750,
+-0.0005798339843750, 0.0050048828125000, -0.0004577636718750, 0.0057678222656250,
+-0.0001831054687500, 0.0056457519531250, -0.0003967285156250, 0.0068359375000000,
+-0.0000610351562500, 0.0069274902343750, -0.0000610351562500, 0.0074462890625000,
+-0.0000305175781250, 0.0083618164062500, 0.0001220703125000, 0.0080566406250000,
+0.0000915527343750, 0.0090332031250000, 0.0001831054687500, 0.0090332031250000,
+0.0002746582031250, 0.0090637207031250, 0.0003662109375000, 0.0095520019531250,
+0.0003967285156250, 0.0092468261718750, 0.0004577636718750, 0.0091247558593750,
+0.0006408691406250, 0.0092773437500000, 0.0005187988281250, 0.0087890625000000,
+0.0006713867187500, 0.0084228515625000, 0.0009155273437500, 0.0085449218750000,
+0.0007324218750000, 0.0073547363281250, 0.0011291503906250, 0.0074462890625000,
+0.0010986328125000, 0.0068664550781250, 0.0011596679687500, 0.0057983398437500,
+0.0014038085937500, 0.0061950683593750, 0.0012512207031250, 0.0044860839843750,
+0.0013732910156250, 0.0045776367187500, 0.0014038085937500, 0.0039367675781250,
+0.0013122558593750, 0.0025634765625000, 0.0012512207031250, 0.0029907226562500,
+0.0012207031250000, 0.0015258789062500, 0.0009765625000000, 0.0012817382812500,
+0.0010375976562500, 0.0010070800781250, 0.0009460449218750, 0.0000915527343750,
+0.0006713867187500, 0.0002746582031250, 0.0008544921875000, -0.0000915527343750,
+0.0004882812500000, -0.0002746582031250, 0.0005798339843750, -0.0000610351562500,
+0.0004577636718750, 0.0000305175781250, 0.0002441406250000, 0.0000000000000000,
+0.0005187988281250, 0.0005187988281250, 0.0000305175781250, 0.0005187988281250,
+0.0002441406250000, 0.0006713867187500, 0.0000610351562500, 0.0010986328125000,
+-0.0003662109375000, 0.0010375976562500, 0.0000610351562500, 0.0012207031250000,
+-0.0005798339843750, 0.0013427734375000, -0.0003967285156250, 0.0012512207031250,
+-0.0004882812500000, 0.0012512207031250, -0.0009460449218750, 0.0012512207031250,
+-0.0005187988281250, 0.0010375976562500, -0.0010070800781250, 0.0010375976562500,
+-0.0010070800781250, 0.0009155273437500, -0.0008544921875000, 0.0006713867187500,
+-0.0009155273437500, 0.0007324218750000, -0.0007934570312500, 0.0004882812500000,
+-0.0006713867187500, 0.0004577636718750, -0.0005187988281250, 0.0004577636718750,
+-0.0004272460937500, 0.0003051757812500, -0.0002441406250000, 0.0003051757812500,
+-0.0000610351562500, 0.0005798339843750, -0.0001831054687500, 0.0003662109375000,
+0.0002136230468750, 0.0006713867187500, 0.0001220703125000, 0.0010681152343750,
+-0.0000305175781250, 0.0007019042968750, 0.0004577636718750, 0.0012817382812500,
+-0.0000305175781250, 0.0013122558593750, 0.0002136230468750, 0.0010681152343750,
+0.0001831054687500, 0.0012512207031250, -0.0001220703125000, 0.0013122558593750,
+0.0003967285156250, 0.0009460449218750, -0.0001831054687500, 0.0010070800781250,
+0.0000305175781250, 0.0008850097656250, 0.0001831054687500, 0.0007019042968750,
+-0.0001525878906250, 0.0007629394531250, 0.0000305175781250, 0.0004272460937500,
+0.0001220703125000, 0.0004577636718750, -0.0000305175781250, 0.0002441406250000,
+0.0001220703125000, 0.0000000000000000, 0.0003051757812500, 0.0001220703125000,
+0.0000610351562500, -0.0005798339843750, 0.0003051757812500, -0.0005187988281250,
+0.0003051757812500, -0.0006103515625000, 0.0001220703125000, -0.0011596679687500,
+0.0000915527343750, -0.0010375976562500, 0.0000610351562500, -0.0006103515625000,
+0.0000000000000000, -0.0010375976562500, -0.0001220703125000, -0.0006103515625000,
+-0.0001220703125000, 0.0000915527343750, -0.0001525878906250, -0.0004577636718750,
+-0.0002441406250000, 0.0004577636718750, -0.0001525878906250, 0.0004577636718750,
+-0.0001831054687500, 0.0004272460937500, -0.0002136230468750, 0.0010375976562500,
+-0.0001831054687500, 0.0014343261718750, -0.0002441406250000, 0.0018920898437500,
+-0.0002746582031250, 0.0018005371093750, -0.0002441406250000, 0.0014953613281250,
+-0.0003051757812500, 0.0017395019531250, -0.0003662109375000, 0.0018310546875000,
+-0.0003356933593750, 0.0007629394531250, -0.0005798339843750, 0.0011901855468750,
+-0.0006713867187500, 0.0007934570312500, -0.0005187988281250, -0.0000915527343750,
+-0.0004882812500000, 0.0006103515625000, -0.0005798339843750, -0.0005798339843750,
+0.0002441406250000, -0.0004272460937500, 0.0000610351562500, -0.0004882812500000,
+0.0004272460937500, -0.0013122558593750, 0.0011901855468750, -0.0008239746093750,
+0.0009155273437500, -0.0011596679687500, 0.0014953613281250, -0.0013427734375000,
+0.0016784667968750, -0.0008850097656250, 0.0014953613281250, -0.0007019042968750,
+0.0016174316406250, -0.0009460449218750, 0.0017395019531250, 0.0003051757812500,
+0.0011901855468750, 0.0000000000000000, 0.0012817382812500, 0.0003356933593750,
+0.0012512207031250, 0.0013427734375000, 0.0009460449218750, 0.0007629394531250,
+0.0010070800781250, 0.0015563964843750, 0.0011596679687500, 0.0016174316406250,
+0.0010070800781250, 0.0014953613281250, 0.0014343261718750, 0.0018615722656250,
+0.0019226074218750, 0.0017395019531250, 0.0017700195312500, 0.0014648437500000,
+0.0028381347656250, 0.0016784667968750, 0.0028381347656250, 0.0013427734375000,
+0.0030517578125000, 0.0009460449218750, 0.0038146972656250, 0.0013732910156250,
+0.0035400390625000, 0.0003051757812500, 0.0037231445312500, 0.0005187988281250,
+0.0041503906250000, 0.0003051757812500, 0.0037536621093750, -0.0004577636718750,
+0.0034484863281250, 0.0000915527343750, 0.0039062500000000, -0.0006408691406250,
+0.0027160644531250, -0.0006713867187500, 0.0030212402343750, -0.0007019042968750,
+0.0026245117187500, -0.0011901855468750, 0.0016784667968750, -0.0010375976562500,
+0.0022277832031250, -0.0014343261718750, 0.0011596679687500, -0.0015258789062500,
+0.0010375976562500, -0.0015258789062500, 0.0009765625000000, -0.0017700195312500,
+0.0004577636718750, -0.0017700195312500, 0.0004882812500000, -0.0014343261718750,
+0.0000305175781250, -0.0017395019531250, -0.0000305175781250, -0.0013427734375000,
+-0.0003356933593750, -0.0008239746093750, -0.0007324218750000, -0.0013732910156250,
+-0.0005798339843750, -0.0003356933593750, -0.0010681152343750, -0.0006408691406250,
+-0.0011596679687500, -0.0005493164062500, -0.0011596679687500, 0.0000915527343750,
+-0.0013122558593750, -0.0005798339843750, -0.0013122558593750, 0.0001525878906250,
+-0.0012207031250000, -0.0000915527343750, -0.0012512207031250, -0.0000915527343750,
+-0.0011901855468750, 0.0003967285156250, -0.0010375976562500, 0.0000000000000000,
+-0.0011291503906250, 0.0004272460937500, -0.0007019042968750, 0.0003662109375000,
+-0.0008544921875000, 0.0003356933593750, -0.0005798339843750, 0.0006103515625000,
+-0.0001525878906250, 0.0004577636718750, -0.0004577636718750, 0.0006408691406250,
+0.0004882812500000, 0.0007934570312500, 0.0003967285156250, 0.0006408691406250,
+0.0006103515625000, 0.0006408691406250, 0.0012817382812500, 0.0008850097656250,
+0.0010375976562500, 0.0003967285156250, 0.0015869140625000, 0.0006103515625000,
+0.0016174316406250, 0.0006408691406250, 0.0015258789062500, 0.0003051757812500,
+0.0016479492187500, 0.0005798339843750, 0.0016174316406250, 0.0005493164062500,
+0.0017089843750000, 0.0004882812500000, 0.0016174316406250, 0.0007324218750000,
+0.0017089843750000, 0.0008239746093750, 0.0018005371093750, 0.0006713867187500,
+0.0018310546875000, 0.0008544921875000, 0.0023803710937500, 0.0007934570312500,
+0.0021667480468750, 0.0007324218750000, 0.0026245117187500, 0.0007934570312500,
+0.0033874511718750, 0.0007019042968750, 0.0029907226562500, 0.0004577636718750,
+0.0043945312500000, 0.0005187988281250, 0.0044250488281250, 0.0003051757812500,
+0.0048522949218750, 0.0000915527343750, 0.0058898925781250, 0.0003051757812500,
+0.0056457519531250, -0.0002441406250000, 0.0066528320312500, -0.0000305175781250,
+0.0067138671875000, -0.0001831054687500, 0.0068054199218750, -0.0005493164062500,
+0.0073852539062500, -0.0000915527343750, 0.0072326660156250, -0.0006713867187500,
+0.0075378417968750, -0.0003356933593750, 0.0076904296875000, -0.0002746582031250,
+0.0074462890625000, -0.0006408691406250, 0.0074768066406250, -0.0001220703125000,
+0.0076293945312500, -0.0002441406250000, 0.0068359375000000, -0.0003356933593750,
+0.0071716308593750, 0.0000000000000000, 0.0065917968750000, 0.0000000000000000,
+0.0057983398437500, -0.0002746582031250, 0.0063476562500000, 0.0004577636718750,
+0.0044860839843750, -0.0001831054687500, 0.0048217773437500, 0.0000305175781250,
+0.0040588378906250, 0.0005798339843750, 0.0025939941406250, -0.0003967285156250,
+0.0035400390625000, 0.0005798339843750, 0.0015258789062500, 0.0002136230468750,
+0.0015869140625000, 0.0000305175781250, 0.0013732910156250, 0.0005493164062500,
+0.0001831054687500, 0.0001220703125000, 0.0007324218750000, 0.0002136230468750,
+0.0002441406250000, 0.0003967285156250, 0.0000000000000000, 0.0002441406250000,
+0.0005798339843750, 0.0001525878906250, 0.0007019042968750, 0.0002441406250000,
+0.0003051757812500, 0.0001525878906250, 0.0017089843750000, 0.0001220703125000,
+0.0013122558593750},
+{-0.0007629394531250, -0.0005187988281250, -0.0007324218750000, -0.0007934570312500,
+-0.0005798339843750, -0.0013122558593750, -0.0004882812500000, -0.0009765625000000,
+-0.0002746582031250, -0.0017395019531250, 0.0000000000000000, -0.0017395019531250,
+0.0001831054687500, -0.0017700195312500, 0.0007019042968750, -0.0022277832031250,
+0.0010375976562500, -0.0021667480468750, 0.0010375976562500, -0.0018005371093750,
+0.0018615722656250, -0.0021667480468750, 0.0019226074218750, -0.0017089843750000,
+0.0023803710937500, -0.0010681152343750, 0.0031433105468750, -0.0016479492187500,
+0.0030517578125000, -0.0003662109375000, 0.0035095214843750, -0.0006408691406250,
+0.0036926269531250, -0.0004577636718750, 0.0038452148437500, 0.0004882812500000,
+0.0041809082031250, -0.0001831054687500, 0.0042114257812500, 0.0006103515625000,
+0.0042114257812500, 0.0005187988281250, 0.0044555664062500, 0.0003967285156250,
+0.0041503906250000, 0.0007324218750000, 0.0039062500000000, 0.0004577636718750,
+0.0042114257812500, 0.0004882812500000, 0.0029907226562500, 0.0003967285156250,
+0.0032958984375000, 0.0003356933593750, 0.0026550292968750, 0.0003662109375000,
+0.0015563964843750, 0.0003356933593750, 0.0022888183593750, 0.0002441406250000,
+0.0003662109375000, 0.0003967285156250, 0.0007019042968750, 0.0003967285156250,
+0.0003662109375000, 0.0002746582031250, -0.0010070800781250, 0.0005493164062500,
+-0.0002136230468750, 0.0003356933593750, -0.0012207031250000, 0.0004882812500000,
+-0.0011596679687500, 0.0005493164062500, -0.0008544921875000, 0.0003967285156250,
+-0.0012207031250000, 0.0005493164062500, -0.0010070800781250, 0.0004577636718750,
+-0.0004882812500000, 0.0004882812500000, -0.0005493164062500, 0.0005187988281250,
+-0.0000915527343750, 0.0004882812500000, 0.0004882812500000, 0.0005187988281250,
+0.0001831054687500, 0.0001525878906250, 0.0014953613281250, 0.0003967285156250,
+0.0011291503906250, 0.0002441406250000, 0.0016784667968750, -0.0000305175781250,
+0.0027465820312500, 0.0003356933593750, 0.0021057128906250, -0.0002136230468750,
+0.0034790039062500, 0.0000915527343750, 0.0033874511718750, 0.0000000000000000,
+0.0035400390625000, -0.0004272460937500, 0.0044555664062500, 0.0001525878906250,
+0.0041503906250000, -0.0003051757812500, 0.0045776367187500, -0.0001831054687500,
+0.0048217773437500, 0.0000610351562500, 0.0047302246093750, -0.0002136230468750,
+0.0048217773437500, -0.0000610351562500, 0.0050964355468750, 0.0000610351562500,
+0.0047607421875000, -0.0002441406250000, 0.0050354003906250, -0.0000610351562500,
+0.0050048828125000, 0.0001220703125000, 0.0047302246093750, -0.0005187988281250,
+0.0051574707031250, 0.0003967285156250, 0.0047302246093750, -0.0001831054687500,
+0.0050048828125000, -0.0000915527343750, 0.0048828125000000, 0.0006713867187500,
+0.0045776367187500, -0.0001831054687500, 0.0050659179687500, 0.0005493164062500,
+0.0042724609375000, 0.0004577636718750, 0.0047607421875000, 0.0003356933593750,
+0.0044860839843750, 0.0006713867187500, 0.0038757324218750, 0.0004882812500000,
+0.0046386718750000, 0.0004882812500000, 0.0033264160156250, 0.0006713867187500,
+0.0038452148437500, 0.0004577636718750, 0.0035095214843750, 0.0003051757812500,
+0.0023803710937500, 0.0003967285156250, 0.0031433105468750, -0.0000915527343750,
+0.0021362304687500, -0.0000305175781250, 0.0020751953125000, -0.0001220703125000,
+0.0024108886718750, -0.0004577636718750, 0.0020446777343750, -0.0002441406250000,
+0.0018310546875000, -0.0006103515625000, 0.0026855468750000, -0.0003356933593750,
+0.0022583007812500, -0.0004882812500000, 0.0025329589843750, -0.0007934570312500,
+0.0033264160156250, -0.0001831054687500, 0.0027465820312500, -0.0010070800781250,
+0.0034790039062500, -0.0007019042968750, 0.0036315917968750, -0.0007019042968750,
+0.0032653808593750, -0.0012207031250000, 0.0033569335937500, -0.0007324218750000,
+0.0035400390625000, -0.0011291503906250, 0.0024414062500000, -0.0010681152343750,
+0.0025634765625000, -0.0010070800781250, 0.0021667480468750, -0.0012207031250000,
+0.0011901855468750, -0.0011291503906250, 0.0014343261718750, -0.0010986328125000,
+0.0007629394531250, -0.0012207031250000, 0.0005798339843750, -0.0010375976562500,
+0.0005493164062500, -0.0008850097656250, 0.0002746582031250, -0.0011291503906250,
+0.0003051757812500, -0.0006103515625000, 0.0002746582031250, -0.0008239746093750,
+0.0002136230468750, -0.0008239746093750, 0.0001525878906250, -0.0005798339843750,
+0.0000610351562500, -0.0008850097656250, 0.0000915527343750, -0.0007019042968750,
+0.0004577636718750, -0.0008544921875000, 0.0002746582031250, -0.0007019042968750,
+0.0008239746093750, -0.0004272460937500, 0.0014038085937500, -0.0007019042968750,
+0.0010986328125000, -0.0000305175781250, 0.0024414062500000, -0.0001220703125000,
+0.0022277832031250, 0.0000915527343750, 0.0027160644531250, 0.0006103515625000,
+0.0038452148437500, 0.0003662109375000, 0.0032653808593750, 0.0009460449218750,
+0.0046081542968750, 0.0009460449218750, 0.0047302246093750, 0.0010986328125000,
+0.0044555664062500, 0.0015563964843750, 0.0050354003906250, 0.0014343261718750,
+0.0049133300781250, 0.0019531250000000, 0.0034484863281250, 0.0020141601562500,
+0.0038757324218750, 0.0020141601562500, 0.0031433105468750, 0.0022583007812500,
+0.0016174316406250, 0.0022277832031250, 0.0021667480468750, 0.0019531250000000,
+0.0010681152343750, 0.0021057128906250, 0.0007629394531250, 0.0018310546875000,
+0.0007324218750000, 0.0011291503906250, 0.0000000000000000, 0.0005493164062500,
+0.0041809082031250, 0.0018005371093750, 0.0028686523437500, 0.0011901855468750,
+0.0035095214843750, 0.0013122558593750, 0.0024719238281250, 0.0016784667968750,
+0.0014038085937500, 0.0011596679687500, 0.0021667480468750, 0.0016174316406250,
+-0.0000305175781250, 0.0010986328125000, 0.0000915527343750, 0.0014953613281250,
+-0.0000915527343750, 0.0020751953125000, -0.0016479492187500, 0.0011596679687500,
+-0.0011596679687500, 0.0028381347656250, -0.0009765625000000, 0.0024719238281250,
+-0.0015258789062500, 0.0025634765625000, -0.0008850097656250, 0.0037536621093750,
+-0.0000305175781250, 0.0030822753906250, -0.0006713867187500, 0.0036010742187500,
+0.0002136230468750, 0.0036621093750000, 0.0003356933593750, 0.0036315917968750,
+0.0001220703125000, 0.0038757324218750, 0.0003356933593750, 0.0037231445312500,
+0.0002136230468750, 0.0046691894531250, 0.0000915527343750, 0.0044250488281250,
+0.0000915527343750, 0.0046081542968750, -0.0001831054687500, 0.0055236816406250,
+-0.0004272460937500, 0.0052490234375000, -0.0003356933593750, 0.0049133300781250,
+-0.0012512207031250, 0.0057373046875000, -0.0010986328125000, 0.0047302246093750,
+-0.0014953613281250, 0.0037841796875000, -0.0023803710937500, 0.0050048828125000,
+-0.0020751953125000, 0.0018615722656250, -0.0024719238281250, 0.0026550292968750,
+-0.0028381347656250, 0.0020446777343750, -0.0023193359375000, -0.0003051757812500,
+-0.0020141601562500, 0.0011596679687500, -0.0023498535156250, -0.0006103515625000,
+-0.0013122558593750, -0.0007324218750000, -0.0013122558593750, -0.0004272460937500,
+-0.0013122558593750, -0.0011901855468750, -0.0007629394531250, -0.0009765625000000,
+-0.0009155273437500, -0.0004882812500000, -0.0005187988281250, -0.0010681152343750,
+-0.0003662109375000, -0.0003967285156250, -0.0004882812500000, 0.0003051757812500,
+-0.0003051757812500, -0.0004577636718750, -0.0001831054687500, 0.0016784667968750,
+-0.0006103515625000, 0.0009155273437500, -0.0004577636718750, 0.0018920898437500,
+-0.0005798339843750, 0.0036621093750000, -0.0010986328125000, 0.0020751953125000,
+-0.0008850097656250, 0.0054016113281250, -0.0007019042968750, 0.0048522949218750,
+-0.0009155273437500, 0.0054016113281250, -0.0006103515625000, 0.0077209472656250,
+-0.0001220703125000, 0.0063781738281250, -0.0003662109375000, 0.0082092285156250,
+0.0001831054687500, 0.0081481933593750, 0.0003051757812500, 0.0081481933593750,
+0.0002746582031250, 0.0090026855468750, 0.0004882812500000, 0.0083618164062500,
+0.0004882812500000, 0.0090332031250000, 0.0004272460937500, 0.0088806152343750,
+0.0004577636718750, 0.0088195800781250, 0.0003051757812500, 0.0090637207031250,
+0.0001525878906250, 0.0086364746093750, 0.0002746582031250, 0.0090332031250000,
+-0.0001525878906250, 0.0087890625000000, -0.0001220703125000, 0.0087890625000000,
+-0.0001831054687500, 0.0090942382812500, -0.0004882812500000, 0.0086669921875000,
+-0.0004577636718750, 0.0090637207031250, -0.0003967285156250, 0.0089416503906250,
+-0.0005187988281250, 0.0088500976562500, -0.0003967285156250, 0.0090637207031250,
+-0.0003051757812500, 0.0088806152343750, -0.0004882812500000, 0.0088195800781250,
+-0.0000915527343750, 0.0088195800781250, -0.0003967285156250, 0.0085144042968750,
+-0.0002746582031250, 0.0082702636718750, 0.0000915527343750, 0.0082702636718750,
+-0.0005187988281250, 0.0074768066406250, 0.0000000000000000, 0.0075073242187500,
+-0.0003051757812500, 0.0067749023437500, -0.0004577636718750, 0.0058898925781250,
+-0.0001831054687500, 0.0063171386718750, -0.0005493164062500, 0.0044250488281250,
+-0.0003662109375000, 0.0044860839843750, -0.0005187988281250, 0.0039062500000000,
+-0.0004272460937500, 0.0024719238281250, -0.0003051757812500, 0.0029296875000000,
+-0.0005187988281250, 0.0014953613281250, -0.0001220703125000, 0.0012817382812500,
+-0.0003356933593750, 0.0009765625000000, -0.0001831054687500, 0.0000915527343750,
+0.0001220703125000, 0.0002441406250000, -0.0000915527343750, -0.0002746582031250,
+0.0004577636718750, -0.0005187988281250, 0.0003967285156250, -0.0004577636718750,
+0.0004272460937500, -0.0006103515625000, 0.0007629394531250, -0.0007324218750000,
+0.0006408691406250, 0.0000000000000000, 0.0007629394531250, -0.0003356933593750,
+0.0009460449218750, 0.0002441406250000, 0.0008544921875000, 0.0011291503906250,
+0.0008239746093750, 0.0006103515625000, 0.0009765625000000, 0.0023803710937500,
+0.0005798339843750, 0.0021972656250000, 0.0007324218750000, 0.0027160644531250,
+0.0006713867187500, 0.0040588378906250, 0.0003967285156250, 0.0033874511718750,
+0.0006103515625000, 0.0050354003906250, 0.0002746582031250, 0.0050354003906250,
+0.0003662109375000, 0.0052795410156250, 0.0002746582031250, 0.0063171386718750,
+0.0000915527343750, 0.0058593750000000, 0.0003662109375000, 0.0066223144531250,
+-0.0000610351562500, 0.0067749023437500, 0.0000915527343750, 0.0067443847656250,
+0.0000610351562500, 0.0071411132812500, -0.0001831054687500, 0.0070190429687500,
+0.0000000000000000, 0.0067138671875000, -0.0003967285156250, 0.0069274902343750,
+-0.0002746582031250, 0.0064697265625000, -0.0003967285156250, 0.0058898925781250,
+-0.0007324218750000, 0.0062255859375000, -0.0005187988281250, 0.0052185058593750,
+-0.0007934570312500, 0.0052490234375000, -0.0008544921875000, 0.0050354003906250,
+-0.0007934570312500, 0.0043334960937500, -0.0009155273437500, 0.0045166015625000,
+-0.0009460449218750, 0.0041198730468750, -0.0006408691406250, 0.0039367675781250,
+-0.0008544921875000, 0.0039367675781250, -0.0007019042968750, 0.0037841796875000,
+-0.0003051757812500, 0.0037536621093750, -0.0005493164062500, 0.0039062500000000,
+-0.0002441406250000, 0.0038452148437500, -0.0001525878906250, 0.0039062500000000,
+-0.0001220703125000, 0.0040283203125000, -0.0000305175781250, 0.0039672851562500,
+0.0000610351562500, 0.0039978027343750, 0.0000915527343750, 0.0039672851562500,
+0.0000915527343750, 0.0039062500000000, 0.0001220703125000, 0.0039062500000000,
+0.0001525878906250},
+{0.0000610351562500, 0.0112304687500000, 0.0001220703125000, 0.0104675292968750,
+0.0001220703125000, 0.0113830566406250, -0.0000610351562500, 0.0112609863281250,
+0.0000610351562500, 0.0109863281250000, -0.0001525878906250, 0.0113220214843750,
+-0.0004577636718750, 0.0110168457031250, -0.0001831054687500, 0.0106506347656250,
+-0.0006713867187500, 0.0107727050781250, -0.0006713867187500, 0.0102233886718750,
+-0.0005187988281250, 0.0097045898437500, -0.0006713867187500, 0.0100097656250000,
+-0.0006103515625000, 0.0086364746093750, -0.0003662109375000, 0.0089721679687500,
+-0.0005187988281250, 0.0083923339843750, -0.0003051757812500, 0.0071716308593750,
+-0.0000915527343750, 0.0079650878906250, -0.0003967285156250, 0.0061645507812500,
+0.0000915527343750, 0.0064697265625000, -0.0001831054687500, 0.0061950683593750,
+-0.0001831054687500, 0.0050048828125000, 0.0001220703125000, 0.0057067871093750,
+-0.0002441406250000, 0.0050048828125000, 0.0000610351562500, 0.0049133300781250,
+-0.0000915527343750, 0.0051269531250000, -0.0000610351562500, 0.0049743652343750,
+0.0000915527343750, 0.0051269531250000, -0.0000610351562500, 0.0049133300781250,
+0.0002441406250000, 0.0050659179687500, 0.0001831054687500, 0.0049438476562500,
+0.0003051757812500, 0.0048217773437500, 0.0006408691406250, 0.0050354003906250,
+0.0004882812500000, 0.0046386718750000, 0.0007934570312500, 0.0047912597656250,
+0.0007629394531250, 0.0046081542968750, 0.0008239746093750, 0.0042419433593750,
+0.0009765625000000, 0.0043640136718750, 0.0008544921875000, 0.0037841796875000,
+0.0009460449218750, 0.0037841796875000, 0.0009155273437500, 0.0036621093750000,
+0.0005798339843750, 0.0032043457031250, 0.0004882812500000, 0.0031738281250000,
+0.0007629394531250, 0.0032653808593750, 0.0003051757812500, 0.0029907226562500,
+0.0003662109375000, 0.0033264160156250, 0.0003356933593750, 0.0036010742187500,
+-0.0000305175781250, 0.0032043457031250, 0.0001831054687500, 0.0044860839843750,
+-0.0002746582031250, 0.0041809082031250, -0.0001525878906250, 0.0046691894531250,
+-0.0002746582031250, 0.0057373046875000, -0.0006408691406250, 0.0052490234375000,
+-0.0003967285156250, 0.0066528320312500, -0.0007629394531250, 0.0065307617187500,
+-0.0007019042968750, 0.0068664550781250, -0.0006103515625000, 0.0078430175781250,
+-0.0007324218750000, 0.0073242187500000, -0.0006408691406250, 0.0083923339843750,
+-0.0007629394531250, 0.0082397460937500, -0.0008239746093750, 0.0082702636718750,
+-0.0008850097656250, 0.0088195800781250, -0.0009155273437500, 0.0083923339843750,
+-0.0007629394531250, 0.0085144042968750, -0.0009155273437500, 0.0085449218750000,
+-0.0009460449218750, 0.0081176757812500, -0.0008239746093750, 0.0079040527343750,
+-0.0007324218750000, 0.0079345703125000, -0.0007324218750000, 0.0071716308593750,
+-0.0004577636718750, 0.0071105957031250, -0.0003662109375000, 0.0068664550781250,
+-0.0004577636718750, 0.0062561035156250, -0.0003356933593750, 0.0063476562500000,
+-0.0002441406250000, 0.0061340332031250, -0.0002136230468750, 0.0058898925781250,
+-0.0001220703125000, 0.0061035156250000, -0.0001220703125000, 0.0062255859375000,
+-0.0001220703125000, 0.0058288574218750, -0.0001220703125000, 0.0065307617187500,
+-0.0001831054687500, 0.0062255859375000, -0.0001831054687500, 0.0061645507812500,
+-0.0002136230468750, 0.0065917968750000, -0.0002746582031250, 0.0061340332031250,
+-0.0003356933593750, 0.0059814453125000, -0.0000915527343750, 0.0060119628906250,
+-0.0002746582031250, 0.0057067871093750, -0.0001220703125000, 0.0054016113281250,
+0.0001220703125000, 0.0054016113281250, -0.0001525878906250, 0.0049438476562500,
+0.0002746582031250, 0.0049133300781250, 0.0001831054687500, 0.0046081542968750,
+0.0001220703125000, 0.0042419433593750, 0.0003356933593750, 0.0043640136718750,
+0.0001525878906250, 0.0032043457031250, 0.0002746582031250, 0.0033874511718750,
+0.0000610351562500, 0.0027770996093750, 0.0000610351562500, 0.0017700195312500,
+0.0001525878906250, 0.0023803710937500, -0.0001525878906250, 0.0008239746093750,
+-0.0002136230468750, 0.0009765625000000, -0.0001831054687500, 0.0010070800781250,
+-0.0003356933593750, 0.0002746582031250, -0.0004882812500000, 0.0007019042968750,
+-0.0005493164062500, 0.0006103515625000, -0.0007019042968750, 0.0005798339843750,
+-0.0007934570312500, 0.0007629394531250, -0.0006408691406250, 0.0008239746093750,
+-0.0006103515625000, 0.0008850097656250, -0.0009460449218750, 0.0009460449218750,
+-0.0003051757812500, 0.0010375976562500, -0.0004882812500000, 0.0008850097656250,
+-0.0003967285156250, 0.0007629394531250, 0.0000305175781250, 0.0008850097656250,
+-0.0002441406250000, 0.0006103515625000, 0.0004882812500000, 0.0005798339843750,
+0.0003967285156250, 0.0004577636718750, 0.0006103515625000, 0.0003051757812500,
+0.0012207031250000, 0.0002136230468750, 0.0011291503906250, -0.0001220703125000,
+0.0011291503906250, -0.0000915527343750, 0.0012512207031250, -0.0000305175781250,
+0.0011291503906250, -0.0001220703125000, 0.0010681152343750, -0.0002136230468750,
+0.0012817382812500, -0.0001220703125000, 0.0007629394531250, -0.0001831054687500,
+0.0008850097656250, -0.0001220703125000, 0.0009155273437500, -0.0001831054687500,
+0.0006408691406250, -0.0003356933593750, 0.0009765625000000, 0.0002136230468750,
+0.0009765625000000, -0.0000610351562500, 0.0010681152343750, 0.0001831054687500,
+0.0011291503906250, 0.0007934570312500, 0.0011291503906250, 0.0003356933593750,
+0.0013122558593750, 0.0007629394531250, 0.0012207031250000, 0.0008850097656250,
+0.0014038085937500, 0.0006713867187500, 0.0011901855468750, 0.0006103515625000,
+0.0009460449218750, 0.0006408691406250, 0.0012817382812500, 0.0003051757812500,
+0.0004577636718750, 0.0002441406250000, 0.0003662109375000, 0.0000915527343750,
+0.0003967285156250, -0.0002136230468750, -0.0000305175781250, -0.0000915527343750,
+0.0000000000000000, -0.0005187988281250, 0.0004577636718750, -0.0003662109375000,
+-0.0002136230468750, -0.0005187988281250, 0.0006408691406250, -0.0009155273437500,
+0.0015563964843750, -0.0007324218750000, 0.0005187988281250, -0.0010070800781250,
+0.0031433105468750, -0.0011901855468750, 0.0025939941406250, -0.0009155273437500,
+0.0031738281250000, -0.0009155273437500, 0.0050354003906250, -0.0011596679687500,
+0.0039672851562500, -0.0005493164062500, 0.0059204101562500, -0.0007019042968750,
+0.0058593750000000, -0.0005798339843750, 0.0061340332031250, -0.0001525878906250,
+0.0072937011718750, -0.0005493164062500, 0.0067749023437500, -0.0001831054687500,
+0.0077209472656250, -0.0003051757812500, 0.0079345703125000, -0.0003051757812500,
+0.0078735351562500, -0.0000915527343750, 0.0082092285156250, -0.0004272460937500,
+0.0082702636718750, -0.0001220703125000, 0.0080871582031250, -0.0002441406250000,
+0.0081787109375000, -0.0002441406250000, 0.0082092285156250, -0.0000610351562500,
+0.0080261230468750, -0.0003356933593750, 0.0080871582031250, -0.0002746582031250,
+0.0081176757812500, -0.0002746582031250, 0.0079650878906250, -0.0003051757812500,
+0.0080871582031250, -0.0003356933593750, 0.0082702636718750, -0.0003967285156250,
+0.0081481933593750, -0.0005187988281250, 0.0082702636718750, -0.0003967285156250,
+0.0083007812500000, -0.0006103515625000, 0.0081176757812500, -0.0008850097656250,
+0.0080871582031250, -0.0007019042968750, 0.0081787109375000, -0.0014038085937500,
+0.0077209472656250, -0.0013732910156250, 0.0078735351562500, -0.0014038085937500,
+0.0077209472656250, -0.0018615722656250, 0.0073852539062500, -0.0017089843750000,
+0.0076904296875000, -0.0016174316406250, 0.0072937011718750, -0.0018615722656250,
+0.0074768066406250, -0.0015258789062500, 0.0075378417968750, -0.0011596679687500,
+0.0073852539062500, -0.0014648437500000, 0.0076904296875000, -0.0004882812500000,
+0.0075683593750000, -0.0006408691406250, 0.0078125000000000, -0.0003662109375000,
+0.0078125000000000, 0.0003662109375000, 0.0077514648437500, -0.0000305175781250,
+0.0080261230468750, 0.0007019042968750, 0.0077514648437500, 0.0007629394531250,
+0.0080261230468750, 0.0006408691406250, 0.0078125000000000, 0.0009155273437500,
+0.0074462890625000, 0.0008850097656250, 0.0078125000000000, 0.0006408691406250,
+0.0069580078125000, 0.0007934570312500, 0.0071105957031250, 0.0006103515625000,
+0.0068664550781250, 0.0003967285156250, 0.0062561035156250, 0.0007629394531250,
+0.0065917968750000, 0.0002136230468750, 0.0059814453125000, 0.0005798339843750,
+0.0060119628906250, 0.0005187988281250, 0.0059204101562500, 0.0001220703125000,
+0.0056762695312500, 0.0007019042968750, 0.0059814453125000, 0.0000610351562500,
+0.0054931640625000, 0.0003051757812500, 0.0057678222656250, 0.0003662109375000,
+0.0057373046875000, 0.0000000000000000, 0.0054321289062500, 0.0003051757812500,
+0.0059204101562500, 0.0001525878906250, 0.0053710937500000, 0.0001220703125000,
+0.0057373046875000, 0.0001831054687500, 0.0056152343750000, 0.0000915527343750,
+0.0051879882812500, 0.0000000000000000, 0.0056457519531250, 0.0002441406250000,
+0.0049133300781250, -0.0000915527343750, 0.0051269531250000, 0.0000305175781250,
+0.0049743652343750, 0.0002746582031250, 0.0043029785156250, -0.0002441406250000,
+0.0046386718750000, 0.0003051757812500, 0.0042419433593750, 0.0000610351562500,
+0.0041503906250000, -0.0000305175781250, 0.0042419433593750, 0.0003356933593750,
+0.0041809082031250, -0.0000610351562500, 0.0042114257812500, 0.0001220703125000,
+0.0047912597656250, 0.0002136230468750, 0.0046081542968750, 0.0001220703125000,
+0.0050048828125000, 0.0001831054687500, 0.0057067871093750, 0.0002746582031250,
+0.0053710937500000, 0.0001831054687500, 0.0066223144531250, 0.0002136230468750,
+0.0065612792968750, 0.0002136230468750, 0.0068359375000000, 0.0001220703125000,
+0.0077514648437500, 0.0001831054687500, 0.0073852539062500, 0.0000915527343750,
+0.0078430175781250, 0.0001831054687500, 0.0079345703125000, 0.0001220703125000,
+0.0078430175781250, 0.0000610351562500, 0.0078430175781250, 0.0002136230468750,
+0.0076293945312500, -0.0001220703125000, 0.0075378417968750, 0.0000915527343750,
+0.0073547363281250, 0.0000305175781250, 0.0072326660156250, -0.0002136230468750,
+0.0071716308593750, 0.0001220703125000, 0.0069580078125000, -0.0003662109375000,
+0.0069885253906250, -0.0002441406250000, 0.0068969726562500, -0.0003356933593750,
+0.0067138671875000, -0.0007324218750000, 0.0065612792968750, -0.0005798339843750,
+0.0065612792968750, -0.0007629394531250, 0.0060424804687500, -0.0009460449218750,
+0.0060119628906250, -0.0007629394531250, 0.0057678222656250, -0.0007019042968750,
+0.0053405761718750, -0.0008544921875000, 0.0053710937500000, -0.0003967285156250,
+0.0050964355468750, -0.0005493164062500, 0.0050048828125000, -0.0004577636718750,
+0.0051269531250000, -0.0000915527343750, 0.0051574707031250, -0.0004882812500000,
+0.0050964355468750, -0.0000305175781250, 0.0055236816406250, -0.0003967285156250,
+0.0055541992187500, -0.0003662109375000, 0.0054626464843750, -0.0001525878906250,
+0.0057067871093750, -0.0006713867187500, 0.0056152343750000, 0.0001525878906250,
+0.0053100585937500, -0.0001831054687500, 0.0054626464843750, -0.0001220703125000,
+0.0050354003906250, 0.0005798339843750, 0.0044860839843750, 0.0001220703125000,
+0.0047302246093750, 0.0005187988281250, 0.0038452148437500, 0.0007019042968750,
+0.0039062500000000, 0.0005493164062500, 0.0035095214843750, 0.0006103515625000,
+0.0028381347656250, 0.0006408691406250, 0.0032043457031250, 0.0004882812500000,
+0.0023803710937500},
+{0.0089416503906250, 0.0007629394531250, 0.0086059570312500, 0.0009765625000000,
+0.0086669921875000, 0.0007629394531250, 0.0087890625000000, 0.0007629394531250,
+0.0086364746093750, 0.0010070800781250, 0.0087890625000000, 0.0004272460937500,
+0.0089111328125000, 0.0008850097656250, 0.0089416503906250, 0.0007324218750000,
+0.0090332031250000, 0.0003051757812500, 0.0091552734375000, 0.0010070800781250,
+0.0091552734375000, 0.0003356933593750, 0.0090332031250000, 0.0006408691406250,
+0.0090332031250000, 0.0007629394531250, 0.0089416503906250, 0.0003051757812500,
+0.0087585449218750, 0.0007324218750000, 0.0088195800781250, 0.0003967285156250,
+0.0083618164062500, 0.0003662109375000, 0.0084838867187500, 0.0004272460937500,
+0.0083007812500000, 0.0002441406250000, 0.0079345703125000, 0.0000915527343750,
+0.0081481933593750, 0.0003662109375000, 0.0074157714843750, 0.0000915527343750,
+0.0076904296875000, 0.0002136230468750, 0.0073242187500000, 0.0004882812500000,
+0.0067443847656250, 0.0001220703125000, 0.0072326660156250, 0.0004882812500000,
+0.0060729980468750, 0.0004577636718750, 0.0064392089843750, 0.0003662109375000,
+0.0060119628906250, 0.0006408691406250, 0.0049743652343750, 0.0005798339843750,
+0.0056762695312500, 0.0004577636718750, 0.0044250488281250, 0.0007019042968750,
+0.0046386718750000, 0.0004577636718750, 0.0043945312500000, 0.0001525878906250,
+0.0035095214843750, 0.0005493164062500, 0.0040893554687500, -0.0001220703125000,
+0.0033569335937500, -0.0000305175781250, 0.0030822753906250, -0.0000305175781250,
+0.0034790039062500, -0.0004577636718750, 0.0033569335937500, -0.0001220703125000,
+0.0032043457031250, -0.0004577636718750, 0.0040893554687500, -0.0003051757812500,
+0.0036926269531250, -0.0002136230468750, 0.0041809082031250, -0.0003051757812500,
+0.0051574707031250, 0.0000305175781250, 0.0046081542968750, -0.0001831054687500,
+0.0055541992187500, 0.0000000000000000, 0.0056152343750000, 0.0000305175781250,
+0.0055541992187500, -0.0001220703125000, 0.0059509277343750, 0.0000610351562500,
+0.0057373046875000, -0.0001525878906250, 0.0054626464843750, -0.0002746582031250,
+0.0055236816406250, -0.0001831054687500, 0.0053710937500000, -0.0002746582031250,
+0.0050659179687500, -0.0004272460937500, 0.0051269531250000, -0.0001220703125000,
+0.0050048828125000, -0.0003051757812500, 0.0047607421875000, -0.0002746582031250,
+0.0048522949218750, -0.0000305175781250, 0.0048522949218750, -0.0003356933593750,
+0.0046997070312500, 0.0000305175781250, 0.0049743652343750, -0.0002746582031250,
+0.0048828125000000, -0.0001831054687500, 0.0050354003906250, 0.0000000000000000,
+0.0052490234375000, -0.0006103515625000, 0.0050659179687500, 0.0002441406250000,
+0.0057373046875000, -0.0001525878906250, 0.0056152343750000, -0.0001220703125000,
+0.0059204101562500, 0.0004882812500000, 0.0065307617187500, 0.0000000000000000,
+0.0061035156250000, 0.0004272460937500, 0.0069580078125000, 0.0004272460937500,
+0.0069580078125000, 0.0002441406250000, 0.0067749023437500, 0.0004272460937500,
+0.0070190429687500, 0.0004577636718750, 0.0069580078125000, 0.0002746582031250,
+0.0067138671875000, 0.0004577636718750, 0.0067443847656250, 0.0003051757812500,
+0.0063781738281250, 0.0000610351562500, 0.0059814453125000, 0.0004272460937500,
+0.0060729980468750, -0.0000305175781250, 0.0054016113281250, 0.0001525878906250,
+0.0054931640625000, 0.0001220703125000, 0.0052185058593750, -0.0001831054687500,
+0.0046997070312500, 0.0002136230468750, 0.0049438476562500, 0.0000610351562500,
+0.0040893554687500, 0.0000610351562500, 0.0039672851562500, 0.0003356933593750,
+0.0038452148437500, 0.0004577636718750, 0.0034484863281250, 0.0003967285156250,
+0.0034790039062500, 0.0008239746093750, 0.0027160644531250, 0.0008239746093750,
+0.0030212402343750, 0.0009765625000000, 0.0023803710937500, 0.0012512207031250,
+0.0014648437500000, 0.0011291503906250, 0.0019836425781250, 0.0015869140625000,
+0.0005798339843750, 0.0014953613281250, 0.0004882812500000, 0.0017089843750000,
+0.0006103515625000, 0.0025329589843750, -0.0002136230468750, 0.0036926269531250,
+0.0036621093750000, 0.0025329589843750, 0.0012512207031250, 0.0034484863281250,
+0.0022277832031250, 0.0028076171875000, 0.0015869140625000, 0.0018310546875000,
+0.0002441406250000, 0.0025939941406250, 0.0010070800781250, 0.0017395019531250,
+0.0003051757812500, 0.0018005371093750, 0.0002441406250000, 0.0018310546875000,
+0.0006408691406250, 0.0014648437500000, 0.0005187988281250, 0.0017395019531250,
+0.0004882812500000, 0.0012512207031250, 0.0014648437500000, 0.0011596679687500,
+0.0011596679687500, 0.0014648437500000, 0.0017089843750000, 0.0012512207031250,
+0.0027770996093750, 0.0010681152343750, 0.0022583007812500, 0.0021667480468750,
+0.0032348632812500, 0.0017395019531250, 0.0035705566406250, 0.0024719238281250,
+0.0031433105468750, 0.0037536621093750, 0.0033569335937500, 0.0029907226562500,
+0.0036010742187500, 0.0048217773437500, 0.0022888183593750, 0.0050964355468750,
+0.0026245117187500, 0.0050354003906250, 0.0022277832031250, 0.0058898925781250,
+0.0010681152343750, 0.0060119628906250, 0.0016174316406250, 0.0063781738281250,
+0.0008544921875000, 0.0063171386718750, 0.0006408691406250, 0.0068359375000000,
+0.0007324218750000, 0.0072631835937500, 0.0003967285156250, 0.0068969726562500,
+0.0003356933593750, 0.0082702636718750, 0.0002136230468750, 0.0080566406250000,
+-0.0000305175781250, 0.0083618164062500, 0.0001525878906250, 0.0094299316406250,
+0.0002441406250000, 0.0090637207031250, -0.0001220703125000, 0.0092468261718750,
+0.0006103515625000, 0.0099182128906250, 0.0006103515625000, 0.0093383789062500,
+0.0003967285156250, 0.0089111328125000, 0.0007629394531250, 0.0097045898437500,
+0.0007324218750000, 0.0082397460937500, 0.0001525878906250, 0.0086975097656250,
+0.0003967285156250, 0.0085449218750000, 0.0001220703125000, 0.0076599121093750,
+-0.0004577636718750, 0.0086059570312500, -0.0001220703125000, 0.0075988769531250,
+-0.0007324218750000, 0.0081176757812500, -0.0007019042968750, 0.0081176757812500,
+-0.0006408691406250, 0.0075683593750000, -0.0009765625000000, 0.0084533691406250,
+-0.0007629394531250, 0.0070800781250000, -0.0007324218750000, 0.0078125000000000,
+-0.0008850097656250, 0.0071716308593750, -0.0005798339843750, 0.0057678222656250,
+-0.0003051757812500, 0.0069580078125000, -0.0005187988281250, 0.0043945312500000,
+0.0000000000000000, 0.0049133300781250, 0.0000305175781250, 0.0043029785156250,
+-0.0001220703125000, 0.0025024414062500, -0.0000305175781250, 0.0036926269531250,
+-0.0001220703125000, 0.0014953613281250, -0.0004577636718750, 0.0017395019531250,
+-0.0005493164062500, 0.0016479492187500, -0.0005798339843750, 0.0004272460937500,
+-0.0007629394531250, 0.0011291503906250, -0.0008239746093750, 0.0007934570312500,
+-0.0006103515625000, 0.0006408691406250, -0.0007934570312500, 0.0009765625000000,
+-0.0006408691406250, 0.0010986328125000, -0.0003967285156250, 0.0010986328125000,
+-0.0007019042968750, 0.0016784667968750, -0.0001220703125000, 0.0014648437500000,
+-0.0002441406250000, 0.0019226074218750, -0.0002441406250000, 0.0024719238281250,
+0.0001831054687500, 0.0019836425781250, -0.0000305175781250, 0.0035095214843750,
+0.0000305175781250, 0.0030212402343750, 0.0001525878906250, 0.0036926269531250,
+-0.0000305175781250, 0.0050659179687500, -0.0001220703125000, 0.0040893554687500,
+0.0000610351562500, 0.0061035156250000, -0.0001525878906250, 0.0060424804687500,
+-0.0000915527343750, 0.0063781738281250, -0.0000305175781250, 0.0076293945312500,
+-0.0000915527343750, 0.0071105957031250, -0.0000305175781250, 0.0082702636718750,
+0.0001525878906250, 0.0084838867187500, 0.0001525878906250, 0.0087890625000000,
+0.0002746582031250, 0.0095825195312500, 0.0004882812500000, 0.0096130371093750,
+0.0003967285156250, 0.0101318359375000, 0.0006713867187500, 0.0104980468750000,
+0.0007019042968750, 0.0105895996093750, 0.0006713867187500, 0.0108337402343750,
+0.0007934570312500, 0.0111389160156250, 0.0007019042968750, 0.0107116699218750,
+0.0004577636718750, 0.0110473632812500, 0.0007019042968750, 0.0107421875000000,
+0.0004882812500000, 0.0102539062500000, 0.0003051757812500, 0.0107421875000000,
+0.0006103515625000, 0.0097351074218750, 0.0000610351562500, 0.0100402832031250,
+0.0003051757812500, 0.0097045898437500, 0.0003051757812500, 0.0089721679687500,
+0.0000305175781250, 0.0096130371093750, 0.0003051757812500, 0.0083618164062500,
+-0.0001525878906250, 0.0085449218750000, -0.0000915527343750, 0.0081787109375000,
+-0.0001525878906250, 0.0072937011718750, -0.0004272460937500, 0.0079345703125000,
+-0.0001831054687500, 0.0065307617187500, -0.0005798339843750, 0.0067749023437500,
+-0.0004882812500000, 0.0062255859375000, -0.0005798339843750, 0.0051574707031250,
+-0.0008544921875000, 0.0057678222656250, -0.0005798339843750, 0.0041503906250000,
+-0.0009765625000000, 0.0043029785156250, -0.0009765625000000, 0.0038452148437500,
+-0.0008850097656250, 0.0026550292968750, -0.0010375976562500, 0.0032653808593750,
+-0.0010681152343750, 0.0018615722656250, -0.0010375976562500, 0.0018920898437500,
+-0.0009460449218750, 0.0018310546875000, -0.0008850097656250, 0.0010375976562500,
+-0.0008544921875000, 0.0014038085937500, -0.0007934570312500, 0.0013122558593750,
+-0.0007324218750000, 0.0010986328125000, -0.0006408691406250, 0.0015563964843750,
+-0.0006408691406250, 0.0018615722656250, -0.0006408691406250, 0.0017089843750000,
+-0.0005187988281250, 0.0025939941406250, -0.0005798339843750, 0.0026550292968750,
+-0.0004882812500000, 0.0028686523437500, -0.0004577636718750, 0.0036010742187500,
+-0.0004882812500000, 0.0033874511718750, -0.0004577636718750, 0.0037841796875000,
+-0.0003356933593750, 0.0040588378906250, -0.0004882812500000, 0.0036315917968750,
+-0.0003662109375000, 0.0034484863281250, -0.0001525878906250, 0.0037841796875000,
+-0.0003051757812500, 0.0024108886718750, 0.0000305175781250, 0.0027160644531250,
+0.0000305175781250, 0.0022583007812500, -0.0000305175781250, 0.0011291503906250,
+0.0001831054687500, 0.0018615722656250, 0.0000915527343750, 0.0009765625000000,
+-0.0000305175781250, 0.0007934570312500, 0.0000915527343750, 0.0012817382812500,
+0.0000610351562500, 0.0011901855468750, -0.0000610351562500, 0.0011596679687500,
+0.0000000000000000, 0.0022277832031250, -0.0000610351562500, 0.0020751953125000,
+-0.0001525878906250, 0.0025329589843750, -0.0001831054687500, 0.0034484863281250,
+-0.0001831054687500, 0.0030517578125000, -0.0002746582031250, 0.0040588378906250,
+-0.0003662109375000, 0.0039978027343750, -0.0003662109375000, 0.0042419433593750,
+-0.0003662109375000, 0.0048828125000000, -0.0004577636718750, 0.0044555664062500,
+-0.0003662109375000, 0.0053405761718750, -0.0003356933593750, 0.0053405761718750,
+-0.0003662109375000, 0.0054016113281250, -0.0001220703125000, 0.0060424804687500,
+0.0000915527343750, 0.0059204101562500, -0.0001525878906250, 0.0056762695312500,
+0.0005493164062500, 0.0062561035156250, 0.0004272460937500, 0.0057983398437500,
+0.0005493164062500, 0.0051879882812500, 0.0010681152343750, 0.0058898925781250,
+0.0008239746093750, 0.0046081542968750, 0.0009155273437500, 0.0046997070312500,
+0.0011291503906250, 0.0045471191406250, 0.0008544921875000, 0.0037841796875000,
+0.0005798339843750, 0.0040893554687500, 0.0009765625000000, 0.0032348632812500,
+0.0003662109375000, 0.0033569335937500, 0.0004882812500000, 0.0030517578125000,
+0.0007019042968750},
+{-0.0001525878906250, 0.0028381347656250, -0.0001525878906250, 0.0034790039062500,
+-0.0001220703125000, 0.0032653808593750, 0.0000915527343750, 0.0042114257812500,
+-0.0000915527343750, 0.0043945312500000, -0.0000305175781250, 0.0045776367187500,
+-0.0000915527343750, 0.0052185058593750, -0.0001831054687500, 0.0052795410156250,
+0.0000000000000000, 0.0056152343750000, -0.0002441406250000, 0.0057373046875000,
+-0.0001525878906250, 0.0058288574218750, -0.0002441406250000, 0.0060729980468750,
+-0.0003967285156250, 0.0060424804687500, -0.0002136230468750, 0.0063476562500000,
+-0.0005493164062500, 0.0063171386718750, -0.0004882812500000, 0.0065307617187500,
+-0.0005493164062500, 0.0068359375000000, -0.0007324218750000, 0.0066833496093750,
+-0.0006103515625000, 0.0069885253906250, -0.0009155273437500, 0.0071105957031250,
+-0.0007629394531250, 0.0071105957031250, -0.0007324218750000, 0.0072631835937500,
+-0.0009460449218750, 0.0072937011718750, -0.0006713867187500, 0.0072021484375000,
+-0.0007629394531250, 0.0073242187500000, -0.0007324218750000, 0.0072631835937500,
+-0.0007019042968750, 0.0070800781250000, -0.0005798339843750, 0.0072937011718750,
+-0.0004882812500000, 0.0069580078125000, -0.0004882812500000, 0.0069885253906250,
+-0.0004272460937500, 0.0070190429687500, -0.0004272460937500, 0.0068664550781250,
+-0.0003967285156250, 0.0068359375000000, -0.0003662109375000, 0.0069274902343750,
+-0.0003662109375000, 0.0067749023437500, -0.0004882812500000, 0.0068664550781250,
+-0.0003967285156250, 0.0070495605468750, -0.0003662109375000, 0.0068054199218750,
+-0.0005493164062500, 0.0073547363281250, -0.0001220703125000, 0.0072937011718750,
+-0.0003356933593750, 0.0074157714843750, -0.0003051757812500, 0.0078735351562500,
+0.0000305175781250, 0.0077514648437500, -0.0002441406250000, 0.0078430175781250,
+0.0000915527343750, 0.0081176757812500, -0.0000610351562500, 0.0077514648437500,
+-0.0000610351562500, 0.0075073242187500, 0.0000915527343750, 0.0078430175781250,
+-0.0000915527343750, 0.0062866210937500, -0.0000610351562500, 0.0066528320312500,
+-0.0000610351562500, 0.0059509277343750, -0.0001831054687500, 0.0043640136718750,
+-0.0002441406250000, 0.0051574707031250, -0.0003356933593750, 0.0034179687500000,
+-0.0004577636718750, 0.0032348632812500, -0.0004882812500000, 0.0032958984375000,
+-0.0004577636718750, 0.0024719238281250, -0.0004882812500000, 0.0025939941406250,
+-0.0004577636718750, 0.0027465820312500, -0.0002441406250000, 0.0025329589843750,
+-0.0003356933593750, 0.0026550292968750, 0.0000000000000000, 0.0029296875000000,
+0.0003051757812500, 0.0027160644531250, 0.0000610351562500, 0.0028686523437500,
+0.0007324218750000, 0.0028686523437500, 0.0006103515625000, 0.0028381347656250,
+0.0007324218750000, 0.0029907226562500, 0.0012512207031250, 0.0029907226562500,
+0.0008850097656250, 0.0033264160156250, 0.0010681152343750, 0.0033264160156250,
+0.0012512207031250, 0.0035400390625000, 0.0009155273437500, 0.0039367675781250,
+0.0007019042968750, 0.0039672851562500, 0.0010375976562500, 0.0043640136718750,
+0.0004272460937500, 0.0043945312500000, 0.0004577636718750, 0.0043945312500000,
+0.0007019042968750, 0.0044860839843750, 0.0005493164062500, 0.0044250488281250,
+0.0006713867187500, 0.0045471191406250, 0.0010070800781250, 0.0044555664062500,
+0.0010986328125000, 0.0043945312500000, 0.0010681152343750, 0.0043640136718750,
+0.0013427734375000, 0.0038757324218750, 0.0016174316406250, 0.0031738281250000,
+0.0009765625000000, 0.0034790039062500, 0.0013427734375000, 0.0030822753906250,
+0.0014038085937500, 0.0022888183593750, 0.0009155273437500, 0.0025939941406250,
+0.0013427734375000, 0.0014343261718750, 0.0014038085937500, 0.0013732910156250,
+0.0011901855468750, 0.0011596679687500, 0.0017395019531250, 0.0004882812500000,
+0.0020446777343750, 0.0007934570312500, 0.0017089843750000, 0.0003967285156250,
+0.0033874511718750, 0.0003051757812500, 0.0028991699218750, 0.0005798339843750,
+0.0035705566406250, 0.0005798339843750, 0.0051574707031250, 0.0005493164062500,
+0.0042419433593750, 0.0011291503906250, 0.0058593750000000, 0.0010681152343750,
+0.0058898925781250, 0.0010375976562500, 0.0057678222656250, 0.0014953613281250,
+0.0065917968750000, 0.0013732910156250, 0.0063476562500000, 0.0010986328125000,
+0.0060119628906250, 0.0013427734375000, 0.0062561035156250, 0.0009765625000000,
+0.0059204101562500, 0.0003967285156250, 0.0054931640625000, 0.0007324218750000,
+0.0058593750000000, 0.0004577636718750, 0.0047912597656250, 0.0000610351562500,
+0.0050659179687500, 0.0004272460937500, 0.0047912597656250, 0.0007019042968750,
+0.0039978027343750, 0.0001525878906250, 0.0046386718750000, 0.0012512207031250,
+0.0033569335937500, 0.0009765625000000, 0.0034790039062500, 0.0010681152343750,
+0.0034484863281250, 0.0018005371093750, 0.0027160644531250, 0.0013427734375000,
+0.0030517578125000, 0.0017089843750000, 0.0028686523437500, 0.0018615722656250,
+0.0027160644531250, 0.0014953613281250, 0.0028686523437500, 0.0014343261718750,
+0.0030212402343750, 0.0017089843750000, 0.0029602050781250, 0.0006408691406250,
+0.0029602050781250, 0.0010681152343750, 0.0031433105468750, 0.0007629394531250,
+0.0028991699218750, -0.0001220703125000, 0.0026245117187500, 0.0006408691406250,
+0.0029907226562500, -0.0002136230468750, 0.0023803710937500, -0.0000915527343750,
+0.0024108886718750, 0.0000915527343750, 0.0025939941406250, -0.0001525878906250,
+0.0024414062500000, 0.0000305175781250, 0.0025939941406250, -0.0000915527343750,
+0.0032043457031250, 0.0000610351562500, 0.0030822753906250, 0.0000305175781250,
+0.0036315917968750, -0.0000610351562500, 0.0043945312500000, 0.0001220703125000,
+0.0040588378906250, -0.0002441406250000, 0.0048522949218750, 0.0000000000000000,
+0.0051269531250000, -0.0001525878906250, 0.0050048828125000, -0.0004272460937500,
+0.0051879882812500, -0.0001525878906250, 0.0053405761718750, -0.0006408691406250,
+0.0047912597656250, -0.0005187988281250, 0.0050659179687500, -0.0005493164062500,
+0.0047607421875000, -0.0008850097656250, 0.0041198730468750, -0.0005493164062500,
+0.0045776367187500, -0.0008544921875000, 0.0037231445312500, -0.0007324218750000,
+0.0037231445312500, -0.0007019042968750, 0.0036926269531250, -0.0008850097656250,
+0.0031738281250000, -0.0007019042968750, 0.0033264160156250, -0.0010681152343750,
+0.0033569335937500, -0.0008544921875000, 0.0031433105468750, -0.0009460449218750,
+0.0033874511718750, -0.0012512207031250, 0.0036621093750000, -0.0009155273437500,
+0.0033264160156250, -0.0012512207031250, 0.0039062500000000, -0.0011596679687500,
+0.0038757324218750, -0.0010986328125000, 0.0037536621093750, -0.0012817382812500,
+0.0039062500000000, -0.0010681152343750, 0.0037536621093750, -0.0010375976562500,
+0.0034790039062500, -0.0010375976562500, 0.0034790039062500, -0.0007629394531250,
+0.0031738281250000, -0.0004882812500000, 0.0028076171875000, -0.0005493164062500,
+0.0029296875000000, 0.0000915527343750, 0.0023498535156250, -0.0000305175781250,
+0.0021972656250000, 0.0002136230468750, 0.0020751953125000, 0.0007934570312500,
+0.0018005371093750, 0.0004577636718750, 0.0018920898437500, 0.0010681152343750,
+0.0020446777343750, 0.0010375976562500, 0.0018615722656250, 0.0010986328125000,
+0.0022277832031250, 0.0014343261718750, 0.0025939941406250, 0.0011291503906250,
+0.0023498535156250, 0.0014648437500000, 0.0035400390625000, 0.0013732910156250,
+0.0033569335937500, 0.0012817382812500, 0.0038146972656250, 0.0014038085937500,
+0.0048217773437500, 0.0012207031250000, 0.0043640136718750, 0.0011901855468750,
+0.0054931640625000, 0.0011596679687500, 0.0056457519531250, 0.0009765625000000,
+0.0056152343750000, 0.0009155273437500, 0.0061950683593750, 0.0009765625000000,
+0.0062561035156250, 0.0005493164062500, 0.0060424804687500, 0.0006713867187500,
+0.0062866210937500, 0.0004882812500000, 0.0061340332031250, 0.0001525878906250,
+0.0058898925781250, 0.0003967285156250, 0.0061645507812500, -0.0000305175781250,
+0.0054626464843750, 0.0000000000000000, 0.0055236816406250, -0.0000915527343750,
+0.0055541992187500, -0.0003356933593750, 0.0051269531250000, -0.0000915527343750,
+0.0051574707031250, -0.0003662109375000, 0.0054931640625000, -0.0001525878906250,
+0.0052185058593750, -0.0002136230468750, 0.0056457519531250, -0.0003662109375000,
+0.0061645507812500, 0.0000610351562500, 0.0057983398437500, -0.0005493164062500,
+0.0069274902343750, -0.0001525878906250, 0.0068969726562500, -0.0002746582031250,
+0.0069885253906250, -0.0008544921875000, 0.0075988769531250, -0.0003967285156250,
+0.0073852539062500, -0.0007934570312500, 0.0073852539062500, -0.0008544921875000,
+0.0076293945312500, -0.0006713867187500, 0.0073242187500000, -0.0006408691406250,
+0.0070800781250000, -0.0006408691406250, 0.0074768066406250, -0.0004882812500000,
+0.0069885253906250, -0.0003662109375000, 0.0070495605468750, -0.0002136230468750,
+0.0071716308593750, -0.0001525878906250, 0.0069885253906250, -0.0002136230468750,
+0.0071716308593750, 0.0002136230468750, 0.0072021484375000, 0.0000000000000000,
+0.0073852539062500, 0.0001220703125000, 0.0074768066406250, 0.0004272460937500,
+0.0076293945312500, 0.0000610351562500, 0.0079345703125000, 0.0004577636718750,
+0.0076599121093750, 0.0004272460937500, 0.0080871582031250, 0.0003356933593750,
+0.0078430175781250, 0.0004882812500000, 0.0075378417968750, 0.0004577636718750,
+0.0081176757812500, 0.0004882812500000, 0.0068664550781250, 0.0005493164062500,
+0.0073547363281250, 0.0005798339843750, 0.0068054199218750, 0.0006408691406250,
+0.0056762695312500, 0.0006713867187500, 0.0065612792968750, 0.0006408691406250,
+0.0049133300781250, 0.0007019042968750, 0.0050048828125000, 0.0007019042968750,
+0.0048217773437500, 0.0005493164062500, 0.0036926269531250, 0.0006408691406250,
+0.0042114257812500, 0.0005493164062500, 0.0035705566406250, 0.0003967285156250,
+0.0034484863281250, 0.0003662109375000, 0.0038757324218750, 0.0004577636718750,
+0.0039367675781250, 0.0005493164062500, 0.0037536621093750, 0.0007019042968750,
+0.0044555664062500, 0.0008239746093750, 0.0042419433593750, 0.0009460449218750,
+0.0042114257812500, 0.0010375976562500, 0.0045776367187500, 0.0011596679687500,
+0.0043029785156250, 0.0015563964843750, 0.0047302246093750, 0.0014648437500000,
+0.0045166015625000, 0.0018310546875000, 0.0046386718750000, 0.0023193359375000,
+0.0049743652343750, 0.0021362304687500, 0.0046691894531250, 0.0030212402343750,
+0.0050048828125000, 0.0031127929687500, 0.0050964355468750, 0.0029907226562500,
+0.0047302246093750, 0.0033874511718750, 0.0046386718750000, 0.0034484863281250,
+0.0048828125000000, 0.0025939941406250, 0.0041503906250000, 0.0028686523437500,
+0.0041809082031250, 0.0025634765625000, 0.0044250488281250, 0.0012817382812500,
+0.0042419433593750, 0.0011291503906250, 0.0042724609375000, 0.0034179687500000,
+0.0032653808593750, 0.0023498535156250, 0.0031127929687500, 0.0032043457031250,
+0.0035400390625000, 0.0046691894531250, 0.0036315917968750, 0.0038146972656250,
+0.0034484863281250, 0.0051269531250000, 0.0042419433593750, 0.0051879882812500,
+0.0043945312500000, 0.0051269531250000, 0.0041809082031250, 0.0057678222656250,
+0.0044555664062500, 0.0056152343750000, 0.0046081542968750, 0.0055541992187500,
+0.0035095214843750, 0.0058898925781250, 0.0040283203125000, 0.0056152343750000,
+0.0033569335937500, 0.0052185058593750, 0.0022277832031250, 0.0055541992187500,
+0.0031738281250000},
+{0.0098876953125000, 0.0000915527343750, 0.0098876953125000, -0.0003967285156250,
+0.0086975097656250, -0.0001831054687500, 0.0088500976562500, -0.0002746582031250,
+0.0081481933593750, -0.0006408691406250, 0.0069885253906250, -0.0002441406250000,
+0.0074768066406250, -0.0006713867187500, 0.0059814453125000, -0.0006713867187500,
+0.0058898925781250, -0.0005798339843750, 0.0058288574218750, -0.0007934570312500,
+0.0050659179687500, -0.0007324218750000, 0.0053100585937500, -0.0006713867187500,
+0.0051574707031250, -0.0007019042968750, 0.0050659179687500, -0.0005493164062500,
+0.0054016113281250, -0.0003662109375000, 0.0055847167968750, -0.0004882812500000,
+0.0055236816406250, 0.0000000000000000, 0.0056457519531250, -0.0002441406250000,
+0.0059204101562500, 0.0000000000000000, 0.0057678222656250, 0.0004272460937500,
+0.0057373046875000, -0.0000610351562500, 0.0060729980468750, 0.0005493164062500,
+0.0056457519531250, 0.0003967285156250, 0.0057067871093750, 0.0003356933593750,
+0.0057067871093750, 0.0006713867187500, 0.0054321289062500, 0.0004272460937500,
+0.0056457519531250, 0.0007934570312500, 0.0056762695312500, 0.0007934570312500,
+0.0055541992187500, 0.0008544921875000, 0.0059509277343750, 0.0011291503906250,
+0.0062255859375000, 0.0009765625000000, 0.0060424804687500, 0.0012207031250000,
+0.0069580078125000, 0.0011901855468750, 0.0069274902343750, 0.0011596679687500,
+0.0071105957031250, 0.0011291503906250, 0.0077819824218750, 0.0010986328125000,
+0.0074462890625000, 0.0010986328125000, 0.0076293945312500, 0.0010681152343750,
+0.0078430175781250, 0.0010681152343750, 0.0075073242187500, 0.0009765625000000,
+0.0073547363281250, 0.0008544921875000, 0.0075378417968750, 0.0011596679687500,
+0.0065612792968750, 0.0009460449218750, 0.0066528320312500, 0.0009765625000000,
+0.0063781738281250, 0.0012207031250000, 0.0055236816406250, 0.0010681152343750,
+0.0057373046875000, 0.0016784667968750, 0.0054321289062500, 0.0014953613281250,
+0.0052185058593750, 0.0018310546875000, 0.0053405761718750, 0.0024108886718750,
+0.0054626464843750, 0.0019531250000000, 0.0053710937500000, 0.0025634765625000,
+0.0053405761718750, 0.0026550292968750, 0.0056457519531250, 0.0023498535156250,
+0.0052795410156250, 0.0025024414062500, 0.0048828125000000, 0.0025329589843750,
+0.0053405761718750, 0.0016479492187500, 0.0045166015625000, 0.0018310546875000,
+0.0044860839843750, 0.0015258789062500, 0.0047912597656250, 0.0007629394531250,
+0.0046081542968750, 0.0009765625000000, 0.0046691894531250, 0.0009460449218750,
+0.0045471191406250, 0.0005798339843750, 0.0048217773437500, 0.0010070800781250,
+0.0044250488281250, 0.0019531250000000, 0.0034790039062500, 0.0031738281250000,
+0.0048217773437500, 0.0046691894531250, 0.0039672851562500, 0.0043640136718750,
+0.0046997070312500, 0.0043029785156250, 0.0037536621093750, 0.0047302246093750,
+0.0025024414062500, 0.0046691894531250, 0.0031433105468750, 0.0042419433593750,
+0.0019226074218750, 0.0044555664062500, 0.0016174316406250, 0.0044860839843750,
+0.0019836425781250, 0.0040893554687500, 0.0017089843750000, 0.0042419433593750,
+0.0015258789062500, 0.0047302246093750, 0.0025939941406250, 0.0042724609375000,
+0.0023803710937500, 0.0050659179687500, 0.0025634765625000, 0.0060729980468750,
+0.0034484863281250, 0.0051879882812500, 0.0032043457031250, 0.0067138671875000,
+0.0033874511718750, 0.0069274902343750, 0.0038452148437500, 0.0064392089843750,
+0.0032653808593750, 0.0068664550781250, 0.0029602050781250, 0.0069274902343750,
+0.0037231445312500, 0.0057678222656250, 0.0016784667968750, 0.0058288574218750,
+0.0023803710937500, 0.0055236816406250, 0.0016174316406250, 0.0046386718750000,
+-0.0003356933593750, 0.0047302246093750, 0.0008544921875000, 0.0044860839843750,
+-0.0006713867187500, 0.0041809082031250, -0.0012207031250000, 0.0042419433593750,
+-0.0003356933593750, 0.0042724609375000, -0.0002746582031250, 0.0041198730468750,
+-0.0007324218750000, 0.0040893554687500, 0.0011596679687500, 0.0040893554687500,
+0.0008544921875000, 0.0038757324218750, 0.0011596679687500, 0.0036315917968750,
+0.0025634765625000, 0.0036621093750000, 0.0019531250000000, 0.0034179687500000,
+0.0026855468750000, 0.0031433105468750, 0.0028381347656250, 0.0033569335937500,
+0.0025329589843750, 0.0035400390625000, 0.0027465820312500, 0.0032348632812500,
+0.0026855468750000, 0.0043029785156250, 0.0018920898437500, 0.0040283203125000,
+0.0021972656250000, 0.0046081542968750, 0.0016479492187500, 0.0056457519531250,
+0.0008239746093750, 0.0050659179687500, 0.0014343261718750, 0.0064086914062500,
+0.0003356933593750, 0.0065002441406250, 0.0003356933593750, 0.0065002441406250,
+0.0004882812500000, 0.0072937011718750, 0.0000610351562500, 0.0071411132812500,
+0.0002441406250000, 0.0073547363281250, 0.0003662109375000, 0.0075683593750000,
+0.0002441406250000, 0.0076293945312500, 0.0003662109375000, 0.0077819824218750,
+0.0005187988281250, 0.0079345703125000, 0.0003356933593750, 0.0083007812500000,
+0.0004882812500000, 0.0083312988281250, 0.0005187988281250, 0.0087280273437500,
+0.0003051757812500, 0.0091247558593750, 0.0003356933593750, 0.0089416503906250,
+0.0003662109375000, 0.0097656250000000, -0.0000610351562500, 0.0096435546875000,
+0.0000915527343750, 0.0099182128906250, -0.0000610351562500, 0.0105285644531250,
+-0.0004577636718750, 0.0101623535156250, -0.0002441406250000, 0.0107421875000000,
+-0.0004577636718750, 0.0108032226562500, -0.0005187988281250, 0.0104980468750000,
+-0.0003051757812500, 0.0106811523437500, -0.0001831054687500, 0.0107421875000000,
+-0.0000915527343750, 0.0098876953125000, 0.0002441406250000, 0.0101318359375000,
+0.0003356933593750, 0.0095214843750000, 0.0005798339843750, 0.0085754394531250,
+0.0009460449218750, 0.0091552734375000, 0.0009765625000000, 0.0077819824218750,
+0.0012207031250000, 0.0077209472656250, 0.0013732910156250, 0.0075988769531250,
+0.0014038085937500, 0.0067749023437500, 0.0014953613281250, 0.0070190429687500,
+0.0015563964843750, 0.0066833496093750, 0.0013732910156250, 0.0064697265625000,
+0.0014038085937500, 0.0066833496093750, 0.0012512207031250, 0.0067138671875000,
+0.0010375976562500, 0.0065002441406250, 0.0010681152343750, 0.0070495605468750,
+0.0006103515625000, 0.0068359375000000, 0.0007019042968750, 0.0068969726562500,
+0.0005187988281250, 0.0074157714843750, 0.0001525878906250, 0.0071411132812500,
+0.0004272460937500, 0.0073242187500000, 0.0000000000000000, 0.0074462890625000,
+0.0000000000000000, 0.0072631835937500, 0.0000305175781250, 0.0072937011718750,
+-0.0001525878906250, 0.0072937011718750, -0.0001525878906250, 0.0066528320312500,
+-0.0003356933593750, 0.0068969726562500, -0.0002746582031250, 0.0063781738281250,
+-0.0003662109375000, 0.0056152343750000, -0.0006103515625000, 0.0060424804687500,
+-0.0004882812500000, 0.0045471191406250, -0.0007629394531250, 0.0047302246093750,
+-0.0007019042968750, 0.0041503906250000, -0.0007019042968750, 0.0029296875000000,
+-0.0008239746093750, 0.0035400390625000, -0.0006408691406250, 0.0021362304687500,
+-0.0007629394531250, 0.0021667480468750, -0.0007629394531250, 0.0021667480468750,
+-0.0007019042968750, 0.0014343261718750, -0.0007324218750000, 0.0018310546875000,
+-0.0006408691406250, 0.0017395019531250, -0.0006103515625000, 0.0016479492187500,
+-0.0005493164062500, 0.0020751953125000, -0.0004577636718750, 0.0023193359375000,
+-0.0003356933593750, 0.0022888183593750, -0.0002441406250000, 0.0033264160156250,
+-0.0001220703125000, 0.0032348632812500, -0.0000610351562500, 0.0037536621093750,
+-0.0000305175781250, 0.0046386718750000, 0.0000305175781250, 0.0043029785156250,
+0.0000610351562500, 0.0053100585937500, 0.0000610351562500, 0.0053100585937500,
+0.0000000000000000, 0.0053405761718750, 0.0001220703125000, 0.0058593750000000,
+0.0001525878906250, 0.0056762695312500, 0.0000610351562500, 0.0055236816406250,
+0.0003662109375000, 0.0055847167968750, 0.0002746582031250, 0.0052795410156250,
+0.0003967285156250, 0.0049438476562500, 0.0006408691406250, 0.0051574707031250,
+0.0004882812500000, 0.0044250488281250, 0.0007324218750000, 0.0044555664062500,
+0.0007629394531250, 0.0042724609375000, 0.0007019042968750, 0.0037536621093750,
+0.0007934570312500, 0.0040588378906250, 0.0008544921875000, 0.0036926269531250,
+0.0006713867187500, 0.0037231445312500, 0.0008544921875000, 0.0038757324218750,
+0.0007019042968750, 0.0037841796875000, 0.0005187988281250, 0.0038452148437500,
+0.0007324218750000, 0.0040588378906250, 0.0002746582031250, 0.0039978027343750,
+0.0002746582031250, 0.0041809082031250, 0.0002746582031250, 0.0044250488281250,
+0.0000000000000000, 0.0042724609375000, 0.0000000000000000, 0.0047912597656250,
+0.0002441406250000, 0.0046691894531250, 0.0000000000000000, 0.0048828125000000,
+0.0001831054687500, 0.0052795410156250, 0.0005187988281250, 0.0051269531250000,
+0.0002136230468750, 0.0056762695312500, 0.0007324218750000, 0.0056762695312500,
+0.0006713867187500, 0.0057983398437500, 0.0006408691406250, 0.0062561035156250,
+0.0009460449218750, 0.0061035156250000, 0.0007324218750000, 0.0059814453125000,
+0.0009460449218750, 0.0061950683593750, 0.0009155273437500, 0.0057678222656250,
+0.0008544921875000, 0.0053100585937500, 0.0009460449218750, 0.0055847167968750,
+0.0007934570312500, 0.0047302246093750, 0.0007324218750000, 0.0046691894531250,
+0.0007019042968750, 0.0048217773437500, 0.0007629394531250, 0.0043945312500000,
+0.0008239746093750, 0.0043945312500000, 0.0007629394531250, 0.0047912597656250,
+0.0010070800781250, 0.0045776367187500, 0.0010070800781250, 0.0045776367187500,
+0.0010681152343750, 0.0050354003906250, 0.0012512207031250, 0.0048217773437500,
+0.0013122558593750, 0.0043029785156250, 0.0012512207031250, 0.0047302246093750,
+0.0013122558593750, 0.0039978027343750, 0.0014343261718750, 0.0031127929687500,
+0.0012817382812500, 0.0038452148437500, 0.0011901855468750, 0.0023498535156250,
+0.0018005371093750, 0.0024719238281250, 0.0013122558593750, 0.0023803710937500,
+0.0017395019531250, 0.0012817382812500, 0.0026245117187500, 0.0013122558593750,
+0.0021972656250000, 0.0008544921875000, 0.0034484863281250, 0.0008544921875000,
+0.0029907226562500, 0.0010375976562500, 0.0030822753906250, 0.0007934570312500,
+0.0039978027343750, 0.0006713867187500, 0.0034179687500000, 0.0018920898437500,
+0.0039978027343750, 0.0011901855468750, 0.0039367675781250, 0.0018615722656250,
+0.0037841796875000, 0.0032348632812500, 0.0039672851562500, 0.0022277832031250,
+0.0036315917968750, 0.0037536621093750, 0.0033264160156250, 0.0039367675781250,
+0.0030517578125000, 0.0035400390625000, 0.0028991699218750, 0.0039978027343750,
+0.0026245117187500, 0.0039978027343750, 0.0022583007812500, 0.0030517578125000,
+0.0024108886718750, 0.0035095214843750, 0.0021362304687500, 0.0027160644531250,
+0.0021667480468750, 0.0015869140625000, 0.0022277832031250, 0.0025329589843750,
+0.0018310546875000, 0.0006713867187500, 0.0026550292968750, 0.0007324218750000,
+0.0022583007812500, 0.0008850097656250, 0.0028381347656250, -0.0000915527343750,
+0.0036926269531250, 0.0002746582031250, 0.0028991699218750, 0.0004882812500000,
+0.0048522949218750, 0.0002441406250000, 0.0045166015625000, 0.0004882812500000,
+0.0048828125000000, 0.0009460449218750, 0.0063171386718750, 0.0006103515625000,
+0.0054931640625000, 0.0010986328125000, 0.0064697265625000, 0.0010681152343750,
+0.0065917968750000},
+{0.0007324218750000, 0.0097351074218750, 0.0008544921875000, 0.0094604492187500,
+0.0008239746093750, 0.0093383789062500, 0.0007934570312500, 0.0094909667968750,
+0.0009460449218750, 0.0087585449218750, 0.0006103515625000, 0.0088500976562500,
+0.0007019042968750, 0.0086975097656250, 0.0006103515625000, 0.0081481933593750,
+0.0002746582031250, 0.0083007812500000, 0.0003967285156250, 0.0079956054687500,
+0.0003051757812500, 0.0078430175781250, 0.0000610351562500, 0.0078735351562500,
+0.0002746582031250, 0.0078430175781250, 0.0003662109375000, 0.0076904296875000,
+0.0001220703125000, 0.0077209472656250, 0.0006103515625000, 0.0076904296875000,
+0.0005187988281250, 0.0076293945312500, 0.0004882812500000, 0.0075988769531250,
+0.0007934570312500, 0.0076293945312500, 0.0005798339843750, 0.0073242187500000,
+0.0007629394531250, 0.0073852539062500, 0.0007324218750000, 0.0072631835937500,
+0.0006408691406250, 0.0070190429687500, 0.0007629394531250, 0.0070190429687500,
+0.0007629394531250, 0.0065612792968750, 0.0005493164062500, 0.0065612792968750,
+0.0006103515625000, 0.0062866210937500, 0.0006408691406250, 0.0059204101562500,
+0.0004882812500000, 0.0060729980468750, 0.0005798339843750, 0.0051269531250000,
+0.0006103515625000, 0.0052185058593750, 0.0006408691406250, 0.0047912597656250,
+0.0006713867187500, 0.0039062500000000, 0.0007324218750000, 0.0040893554687500,
+0.0009155273437500, 0.0032653808593750, 0.0007019042968750, 0.0030822753906250,
+0.0008544921875000, 0.0032348632812500, 0.0008850097656250, 0.0029907226562500,
+0.0006713867187500, 0.0029296875000000, 0.0009155273437500, 0.0037231445312500,
+0.0010681152343750, 0.0036315917968750, 0.0009155273437500, 0.0038452148437500,
+0.0012817382812500, 0.0045471191406250, 0.0016479492187500, 0.0044250488281250,
+0.0013732910156250, 0.0045166015625000, 0.0021667480468750, 0.0046997070312500,
+0.0020751953125000, 0.0044555664062500, 0.0022583007812500, 0.0042114257812500,
+0.0028076171875000, 0.0042419433593750, 0.0025634765625000, 0.0033264160156250,
+0.0028686523437500, 0.0037536621093750, 0.0028381347656250, 0.0033874511718750,
+0.0026855468750000, 0.0027160644531250, 0.0027160644531250, 0.0034179687500000,
+0.0026550292968750, 0.0018005371093750, 0.0024719238281250, 0.0019531250000000,
+0.0024108886718750, 0.0019531250000000, 0.0022888183593750, 0.0008544921875000,
+0.0020141601562500, 0.0011901855468750, 0.0019531250000000, 0.0014648437500000,
+0.0021057128906250, 0.0009765625000000, 0.0017700195312500, 0.0013732910156250,
+0.0021057128906250, 0.0019836425781250, 0.0025634765625000, 0.0015563964843750,
+0.0022583007812500, 0.0023803710937500, 0.0032653808593750, 0.0023803710937500,
+0.0028991699218750, 0.0024108886718750, 0.0034484863281250, 0.0028381347656250,
+0.0045166015625000, 0.0026245117187500, 0.0039062500000000, 0.0024719238281250,
+0.0057067871093750, 0.0028076171875000, 0.0055847167968750, 0.0020751953125000,
+0.0058898925781250, 0.0014038085937500, 0.0071716308593750, 0.0020446777343750,
+0.0066223144531250, 0.0003051757812500, 0.0072326660156250, 0.0006103515625000,
+0.0074157714843750, 0.0004882812500000, 0.0073242187500000, -0.0005493164062500,
+0.0074768066406250, 0.0000610351562500, 0.0074157714843750, -0.0002441406250000,
+0.0078125000000000, -0.0004272460937500, 0.0076293945312500, 0.0000610351562500,
+0.0080261230468750, 0.0002441406250000, 0.0086059570312500, 0.0000000000000000,
+0.0081176757812500, 0.0008544921875000, 0.0088500976562500, 0.0007324218750000,
+0.0089111328125000, 0.0007934570312500, 0.0087280273437500, 0.0013122558593750,
+0.0090637207031250, 0.0010986328125000, 0.0089416503906250, 0.0013122558593750,
+0.0086669921875000, 0.0013732910156250, 0.0087585449218750, 0.0013427734375000,
+0.0086364746093750, 0.0013732910156250, 0.0083618164062500, 0.0013427734375000,
+0.0083923339843750, 0.0016174316406250, 0.0082092285156250, 0.0014648437500000,
+0.0081176757812500, 0.0017089843750000, 0.0079345703125000, 0.0020446777343750,
+0.0078125000000000, 0.0018005371093750, 0.0077819824218750, 0.0022888183593750,
+0.0073547363281250, 0.0024108886718750, 0.0072631835937500, 0.0022583007812500,
+0.0070800781250000, 0.0025634765625000, 0.0067749023437500, 0.0027160644531250,
+0.0067443847656250, 0.0020751953125000, 0.0065917968750000, 0.0025329589843750,
+0.0065002441406250, 0.0021362304687500, 0.0064086914062500, 0.0014038085937500,
+0.0061950683593750, 0.0021362304687500, 0.0061035156250000, 0.0008850097656250,
+0.0059204101562500, 0.0011901855468750, 0.0057983398437500, 0.0010375976562500,
+0.0056152343750000, 0.0003051757812500, 0.0054016113281250, 0.0010986328125000,
+0.0054016113281250, 0.0002136230468750, 0.0054321289062500, 0.0005493164062500,
+0.0052490234375000, 0.0006408691406250, 0.0054321289062500, 0.0001525878906250,
+0.0056762695312500, 0.0008239746093750, 0.0054016113281250, 0.0003356933593750,
+0.0062866210937500, 0.0006103515625000, 0.0060424804687500, 0.0005798339843750,
+0.0064086914062500, 0.0002136230468750, 0.0070800781250000, 0.0007019042968750,
+0.0066833496093750, 0.0000915527343750, 0.0077209472656250, 0.0002441406250000,
+0.0076599121093750, 0.0003051757812500, 0.0078430175781250, -0.0001525878906250,
+0.0085754394531250, 0.0000000000000000, 0.0082702636718750, -0.0001831054687500,
+0.0087280273437500, -0.0004272460937500, 0.0088195800781250, -0.0003051757812500,
+0.0087280273437500, -0.0003051757812500, 0.0088806152343750, -0.0006408691406250,
+0.0088806152343750, -0.0001831054687500, 0.0084228515625000, -0.0005798339843750,
+0.0085449218750000, -0.0004272460937500, 0.0081787109375000, -0.0000305175781250,
+0.0076904296875000, -0.0005798339843750, 0.0079345703125000, 0.0000915527343750,
+0.0070190429687500, -0.0001220703125000, 0.0071411132812500, -0.0001220703125000,
+0.0067443847656250, 0.0003051757812500, 0.0059204101562500, -0.0001525878906250,
+0.0063171386718750, 0.0002441406250000, 0.0052795410156250, 0.0001525878906250,
+0.0052490234375000, 0.0000305175781250, 0.0050048828125000, 0.0002136230468750,
+0.0043640136718750, 0.0001220703125000, 0.0046081542968750, 0.0001831054687500,
+0.0042419433593750, 0.0002136230468750, 0.0040283203125000, 0.0001831054687500,
+0.0041198730468750, 0.0002746582031250, 0.0039978027343750, 0.0003051757812500,
+0.0038452148437500, 0.0003356933593750, 0.0042114257812500, 0.0004882812500000,
+0.0038757324218750, 0.0004577636718750, 0.0040588378906250, 0.0004882812500000,
+0.0044860839843750, 0.0006713867187500, 0.0039367675781250, 0.0002746582031250,
+0.0046691894531250, 0.0005187988281250, 0.0044250488281250, 0.0004272460937500,
+0.0045471191406250, 0.0000000000000000, 0.0050354003906250, 0.0003662109375000,
+0.0045166015625000, -0.0001831054687500, 0.0052795410156250, -0.0001831054687500,
+0.0049438476562500, -0.0001831054687500, 0.0053405761718750, -0.0003967285156250,
+0.0059814453125000, -0.0003051757812500, 0.0053405761718750, -0.0002441406250000,
+0.0069580078125000, -0.0004272460937500, 0.0065917968750000, -0.0002136230468750,
+0.0071411132812500, -0.0000305175781250, 0.0084838867187500, -0.0003051757812500,
+0.0076599121093750, 0.0002441406250000, 0.0090942382812500, 0.0000000000000000,
+0.0091552734375000, 0.0001220703125000, 0.0090332031250000, 0.0004577636718750,
+0.0097045898437500, -0.0000305175781250, 0.0096740722656250, 0.0005493164062500,
+0.0093078613281250, 0.0001831054687500, 0.0097045898437500, 0.0002746582031250,
+0.0093383789062500, 0.0008544921875000, 0.0087890625000000, 0.0003051757812500,
+0.0092163085937500, 0.0010986328125000, 0.0084228515625000, 0.0010070800781250,
+0.0084838867187500, 0.0009765625000000, 0.0084533691406250, 0.0014343261718750,
+0.0081176757812500, 0.0011901855468750, 0.0084228515625000, 0.0014343261718750,
+0.0082702636718750, 0.0014038085937500, 0.0083618164062500, 0.0014343261718750,
+0.0084838867187500, 0.0015258789062500, 0.0085449218750000, 0.0013732910156250,
+0.0087280273437500, 0.0017395019531250, 0.0084533691406250, 0.0016174316406250,
+0.0087890625000000, 0.0018005371093750, 0.0082702636718750, 0.0020751953125000,
+0.0077514648437500, 0.0018920898437500, 0.0082702636718750, 0.0025329589843750,
+0.0066528320312500, 0.0024719238281250, 0.0069885253906250, 0.0025024414062500,
+0.0063781738281250, 0.0029296875000000, 0.0050048828125000, 0.0028381347656250,
+0.0057983398437500, 0.0026855468750000, 0.0042114257812500, 0.0029907226562500,
+0.0042419433593750, 0.0026245117187500, 0.0042724609375000, 0.0021362304687500,
+0.0036010742187500, 0.0024108886718750, 0.0039978027343750, 0.0012207031250000,
+0.0039062500000000, 0.0010986328125000, 0.0039672851562500, 0.0010375976562500,
+0.0039367675781250, 0.0003051757812500, 0.0039062500000000, 0.0003051757812500,
+0.0043029785156250, 0.0010070800781250, 0.0031127929687500, 0.0003967285156250,
+0.0035095214843750, 0.0011596679687500, 0.0031738281250000, 0.0022277832031250,
+0.0020751953125000, 0.0012207031250000, 0.0026855468750000, 0.0031738281250000,
+0.0025939941406250, 0.0029907226562500, 0.0023193359375000, 0.0029296875000000,
+0.0026855468750000, 0.0040283203125000, 0.0032348632812500, 0.0035095214843750,
+0.0031127929687500, 0.0036621093750000, 0.0031127929687500, 0.0038757324218750,
+0.0033569335937500, 0.0036621093750000, 0.0034484863281250, 0.0033874511718750,
+0.0032958984375000, 0.0034790039062500, 0.0023803710937500, 0.0049133300781250,
+0.0021057128906250, 0.0040893554687500, 0.0020446777343750, 0.0047302246093750,
+0.0023498535156250, 0.0059814453125000, 0.0023498535156250, 0.0054931640625000,
+0.0021057128906250, 0.0058288574218750, 0.0034179687500000, 0.0065307617187500,
+0.0029602050781250, 0.0058288574218750, 0.0034484863281250, 0.0051879882812500,
+0.0047302246093750, 0.0059509277343750, 0.0039367675781250, 0.0042419433593750,
+0.0045166015625000, 0.0044555664062500, 0.0048828125000000, 0.0045166015625000,
+0.0041198730468750, 0.0035400390625000, 0.0037231445312500, 0.0040283203125000,
+0.0040893554687500, 0.0042724609375000, 0.0025329589843750, 0.0040588378906250,
+0.0026245117187500, 0.0043334960937500, 0.0023498535156250, 0.0047302246093750,
+0.0014038085937500, 0.0043945312500000, 0.0018310546875000, 0.0044860839843750,
+0.0014038085937500, 0.0044555664062500, 0.0011596679687500, 0.0045166015625000,
+0.0016479492187500, 0.0043029785156250, 0.0017395019531250, 0.0041809082031250,
+0.0015869140625000, 0.0052795410156250, 0.0029907226562500, 0.0046691894531250,
+0.0028076171875000, 0.0055847167968750, 0.0030212402343750, 0.0069274902343750,
+0.0040893554687500, 0.0059204101562500, 0.0036926269531250, 0.0085144042968750,
+0.0034179687500000, 0.0082702636718750, 0.0040283203125000, 0.0085449218750000,
+0.0030822753906250, 0.0101623535156250, 0.0020141601562500, 0.0094604492187500,
+0.0028381347656250, 0.0101318359375000, 0.0007934570312500, 0.0104370117187500,
+0.0008239746093750, 0.0102233886718750, 0.0007629394531250, 0.0102539062500000,
+-0.0003967285156250, 0.0103149414062500, 0.0000305175781250, 0.0098876953125000,
+-0.0001831054687500, 0.0100402832031250, -0.0005187988281250, 0.0097351074218750,
+0.0000305175781250, 0.0092773437500000, 0.0003051757812500, 0.0096130371093750,
+-0.0001220703125000, 0.0083312988281250, 0.0009765625000000, 0.0088195800781250,
+0.0007934570312500, 0.0080871582031250, 0.0009765625000000, 0.0069580078125000,
+0.0018005371093750},
+{0.0075378417968750, 0.0032043457031250, 0.0085144042968750, 0.0033874511718750,
+0.0077514648437500, 0.0029602050781250, 0.0090942382812500, 0.0034484863281250,
+0.0090942382812500, 0.0030212402343750, 0.0089416503906250, 0.0025024414062500,
+0.0096740722656250, 0.0031433105468750, 0.0094299316406250, 0.0018615722656250,
+0.0090637207031250, 0.0020141601562500, 0.0094299316406250, 0.0021362304687500,
+0.0087585449218750, 0.0013427734375000, 0.0080261230468750, 0.0015258789062500,
+0.0085144042968750, 0.0021362304687500, 0.0069274902343750, 0.0016174316406250,
+0.0071716308593750, 0.0023498535156250, 0.0069580078125000, 0.0033264160156250,
+0.0059814453125000, 0.0025939941406250, 0.0065307617187500, 0.0042114257812500,
+0.0058593750000000, 0.0040893554687500, 0.0061950683593750, 0.0040588378906250,
+0.0057678222656250, 0.0049133300781250, 0.0052490234375000, 0.0045166015625000,
+0.0059204101562500, 0.0042114257812500, 0.0037841796875000, 0.0045776367187500,
+0.0042724609375000, 0.0041809082031250, 0.0039062500000000, 0.0035705566406250,
+0.0023193359375000, 0.0041503906250000, 0.0031433105468750, 0.0033874511718750,
+0.0024719238281250, 0.0033264160156250, 0.0022277832031250, 0.0036315917968750,
+0.0023803710937500, 0.0036315917968750, 0.0021972656250000, 0.0059814453125000,
+0.0041809082031250, 0.0057067871093750, 0.0035400390625000, 0.0063476562500000,
+0.0036926269531250, 0.0055541992187500, 0.0034179687500000, 0.0047302246093750,
+0.0030822753906250, 0.0055236816406250, 0.0031433105468750, 0.0040588378906250,
+0.0026550292968750, 0.0039062500000000, 0.0024719238281250, 0.0043029785156250,
+0.0026550292968750, 0.0037536621093750, 0.0022888183593750, 0.0037536621093750,
+0.0020446777343750, 0.0045776367187500, 0.0031738281250000, 0.0044860839843750,
+0.0025024414062500, 0.0046691894531250, 0.0030517578125000, 0.0053710937500000,
+0.0044250488281250, 0.0052795410156250, 0.0034790039062500, 0.0053100585937500,
+0.0046081542968750, 0.0054016113281250, 0.0048828125000000, 0.0056152343750000,
+0.0043945312500000, 0.0055847167968750, 0.0046691894531250, 0.0054321289062500,
+0.0050354003906250, 0.0069274902343750, 0.0038757324218750, 0.0063476562500000,
+0.0043640136718750, 0.0072021484375000, 0.0038146972656250, 0.0089111328125000,
+0.0028076171875000, 0.0080566406250000, 0.0038452148437500, 0.0097656250000000,
+0.0020446777343750, 0.0099487304687500, 0.0021057128906250, 0.0095520019531250,
+0.0022583007812500, 0.0102539062500000, 0.0014038085937500, 0.0102844238281250,
+0.0018005371093750, 0.0090637207031250, 0.0022888183593750, 0.0097351074218750,
+0.0019226074218750, 0.0089111328125000, 0.0025329589843750, 0.0076293945312500,
+0.0034484863281250, 0.0086059570312500, 0.0030822753906250, 0.0064697265625000,
+0.0034484863281250, 0.0069274902343750, 0.0039367675781250, 0.0062561035156250,
+0.0033264160156250, 0.0046691894531250, 0.0029602050781250, 0.0058898925781250,
+0.0033874511718750, 0.0032043457031250, 0.0018310546875000, 0.0036621093750000,
+0.0019531250000000, 0.0028686523437500, 0.0014953613281250, 0.0008239746093750,
+0.0003051757812500, 0.0021667480468750, 0.0008544921875000, -0.0005493164062500,
+0.0000000000000000, -0.0004272460937500, -0.0002746582031250, -0.0006103515625000,
+0.0000610351562500, -0.0023193359375000, -0.0000610351562500, -0.0015869140625000,
+-0.0002136230468750, -0.0022277832031250, 0.0008850097656250, -0.0026550292968750,
+0.0006103515625000, -0.0021972656250000, 0.0008850097656250, -0.0021972656250000,
+0.0018920898437500, -0.0025024414062500, 0.0014343261718750, -0.0014343261718750,
+0.0019226074218750, -0.0018615722656250, 0.0021972656250000, -0.0015258789062500,
+0.0018920898437500, -0.0005187988281250, 0.0018005371093750, -0.0012817382812500,
+0.0020751953125000, -0.0002746582031250, 0.0013427734375000, -0.0001831054687500,
+0.0015869140625000, -0.0003967285156250, 0.0014343261718750, -0.0000305175781250,
+0.0009765625000000, -0.0000610351562500, 0.0013732910156250, -0.0003051757812500,
+0.0010681152343750, -0.0001525878906250, 0.0010375976562500, -0.0003967285156250,
+0.0012207031250000, -0.0007019042968750, 0.0012207031250000, -0.0004272460937500,
+0.0013427734375000, -0.0010681152343750, 0.0015258789062500, -0.0009460449218750,
+0.0016174316406250, -0.0011901855468750, 0.0015258789062500, -0.0017700195312500,
+0.0015869140625000, -0.0014038085937500, 0.0017089843750000, -0.0021362304687500,
+0.0013122558593750, -0.0022583007812500, 0.0015258789062500, -0.0021362304687500,
+0.0012817382812500, -0.0025024414062500, 0.0009155273437500, -0.0024719238281250,
+0.0013122558593750, -0.0018005371093750, 0.0007324218750000, -0.0024108886718750,
+0.0008544921875000, -0.0017395019531250, 0.0009155273437500, -0.0007324218750000,
+0.0005798339843750, -0.0016174316406250, 0.0007629394531250, 0.0005798339843750,
+0.0007019042968750, 0.0001220703125000, 0.0006408691406250, 0.0007934570312500,
+0.0007629394531250, 0.0024719238281250, 0.0008239746093750, 0.0014343261718750,
+0.0006713867187500, 0.0036010742187500, 0.0007934570312500, 0.0033874511718750,
+0.0006713867187500, 0.0038757324218750, 0.0006103515625000, 0.0053710937500000,
+0.0006103515625000, 0.0046081542968750, 0.0003356933593750, 0.0060729980468750,
+0.0002441406250000, 0.0061035156250000, 0.0002136230468750, 0.0062255859375000,
+0.0000305175781250, 0.0071411132812500, -0.0001220703125000, 0.0068359375000000,
+-0.0001831054687500, 0.0072021484375000, -0.0003662109375000, 0.0073547363281250,
+-0.0004577636718750, 0.0072021484375000, -0.0005187988281250, 0.0072631835937500,
+-0.0005187988281250, 0.0073547363281250, -0.0004272460937500, 0.0067749023437500,
+-0.0005187988281250, 0.0068969726562500, -0.0004272460937500, 0.0066833496093750,
+-0.0004272460937500, 0.0062561035156250, -0.0004882812500000, 0.0065612792968750,
+-0.0002746582031250, 0.0058593750000000, -0.0003662109375000, 0.0059814453125000,
+-0.0002441406250000, 0.0058593750000000, -0.0001525878906250, 0.0054626464843750,
+-0.0001831054687500, 0.0058288574218750, 0.0000000000000000, 0.0052490234375000,
+0.0000000000000000, 0.0053710937500000, 0.0001220703125000, 0.0052795410156250,
+0.0002441406250000, 0.0048828125000000, 0.0003356933593750, 0.0052185058593750,
+0.0003967285156250, 0.0045776367187500, 0.0005493164062500, 0.0047607421875000,
+0.0005798339843750, 0.0045471191406250, 0.0006103515625000, 0.0041198730468750,
+0.0007019042968750, 0.0044860839843750, 0.0007934570312500, 0.0037231445312500,
+0.0006408691406250, 0.0037841796875000, 0.0008239746093750, 0.0036926269531250,
+0.0007324218750000, 0.0032348632812500, 0.0005493164062500, 0.0032958984375000,
+0.0007629394531250, 0.0031433105468750, 0.0005493164062500, 0.0030212402343750,
+0.0004577636718750, 0.0031127929687500, 0.0006103515625000, 0.0030517578125000,
+0.0005798339843750, 0.0030212402343750, 0.0006103515625000, 0.0034790039062500,
+0.0008239746093750, 0.0033874511718750, 0.0008544921875000, 0.0035705566406250,
+0.0009765625000000, 0.0039978027343750, 0.0012512207031250, 0.0038452148437500,
+0.0012207031250000, 0.0039978027343750, 0.0013732910156250, 0.0042724609375000,
+0.0016174316406250, 0.0041198730468750, 0.0015258789062500, 0.0040283203125000,
+0.0015258789062500, 0.0042419433593750, 0.0017395019531250, 0.0037841796875000,
+0.0013732910156250, 0.0038452148437500, 0.0015258789062500, 0.0037231445312500,
+0.0014953613281250, 0.0033569335937500, 0.0012817382812500, 0.0035705566406250,
+0.0014648437500000, 0.0033264160156250, 0.0014038085937500, 0.0033264160156250,
+0.0013427734375000, 0.0035095214843750, 0.0014648437500000, 0.0035095214843750,
+0.0015258789062500, 0.0035400390625000, 0.0014953613281250, 0.0043945312500000,
+0.0014038085937500, 0.0042419433593750, 0.0016479492187500, 0.0047912597656250,
+0.0014953613281250, 0.0056762695312500, 0.0012207031250000, 0.0052795410156250,
+0.0015563964843750, 0.0063476562500000, 0.0014038085937500, 0.0064086914062500,
+0.0012512207031250, 0.0065002441406250, 0.0016174316406250, 0.0070800781250000,
+0.0018310546875000, 0.0068664550781250, 0.0017089843750000, 0.0068054199218750,
+0.0024719238281250, 0.0070495605468750, 0.0023803710937500, 0.0067138671875000,
+0.0025939941406250, 0.0064086914062500, 0.0031433105468750, 0.0067138671875000,
+0.0027770996093750, 0.0057067871093750, 0.0031127929687500, 0.0059204101562500,
+0.0031127929687500, 0.0054016113281250, 0.0029602050781250, 0.0045471191406250,
+0.0030517578125000, 0.0050964355468750, 0.0030517578125000, 0.0039367675781250,
+0.0028381347656250, 0.0039978027343750, 0.0028381347656250, 0.0038757324218750,
+0.0028686523437500, 0.0032348632812500, 0.0025939941406250, 0.0035095214843750,
+0.0023803710937500, 0.0034484863281250, 0.0030212402343750, 0.0035095214843750,
+0.0025329589843750, 0.0032958984375000, 0.0028076171875000, 0.0030822753906250,
+0.0033874511718750, 0.0033569335937500, 0.0027160644531250, 0.0028686523437500,
+0.0039672851562500, 0.0028381347656250, 0.0036010742187500, 0.0029602050781250,
+0.0040283203125000, 0.0027770996093750, 0.0049743652343750, 0.0029907226562500,
+0.0043029785156250, 0.0030822753906250, 0.0057373046875000, 0.0030212402343750,
+0.0056762695312500, 0.0030517578125000, 0.0058593750000000, 0.0030822753906250,
+0.0067443847656250, 0.0029602050781250, 0.0062866210937500, 0.0028076171875000,
+0.0067443847656250, 0.0028686523437500, 0.0069580078125000, 0.0025329589843750,
+0.0067443847656250, 0.0022277832031250, 0.0066223144531250, 0.0024719238281250,
+0.0066528320312500, 0.0017395019531250, 0.0063476562500000, 0.0017700195312500,
+0.0063476562500000, 0.0018310546875000, 0.0061035156250000, 0.0014648437500000,
+0.0058288574218750, 0.0016479492187500, 0.0059814453125000, 0.0018920898437500,
+0.0051574707031250, 0.0015869140625000, 0.0054931640625000, 0.0018615722656250,
+0.0049438476562500, 0.0023803710937500, 0.0041198730468750, 0.0020446777343750,
+0.0047912597656250, 0.0022888183593750, 0.0030212402343750, 0.0026245117187500,
+0.0033264160156250, 0.0021362304687500, 0.0029907226562500, 0.0019226074218750,
+0.0018310546875000, 0.0024719238281250, 0.0024719238281250, 0.0010375976562500,
+0.0012207031250000, 0.0013427734375000, 0.0013122558593750, 0.0011901855468750,
+0.0010070800781250, 0.0001220703125000, 0.0000610351562500, 0.0005798339843750,
+0.0005187988281250, 0.0002136230468750, -0.0004882812500000, 0.0000000000000000,
+-0.0005187988281250, 0.0002441406250000, -0.0005187988281250, 0.0002746582031250,
+-0.0010681152343750, 0.0001525878906250, -0.0008544921875000, 0.0006408691406250,
+-0.0006408691406250, 0.0004882812500000, -0.0007324218750000, 0.0007324218750000,
+-0.0002746582031250, 0.0011596679687500, 0.0002136230468750, 0.0007934570312500,
+0.0001220703125000, 0.0014343261718750, 0.0007324218750000, 0.0012512207031250,
+0.0009155273437500, 0.0012817382812500, 0.0009765625000000, 0.0016784667968750,
+0.0011291503906250, 0.0013732910156250, 0.0010986328125000, 0.0016479492187500,
+0.0011596679687500, 0.0016784667968750, 0.0011901855468750, 0.0016479492187500,
+0.0014953613281250, 0.0017395019531250, 0.0017395019531250, 0.0018005371093750,
+0.0015258789062500, 0.0018615722656250, 0.0022888183593750, 0.0019836425781250,
+0.0021667480468750, 0.0018920898437500, 0.0023498535156250, 0.0019226074218750,
+0.0029602050781250, 0.0021057128906250, 0.0026245117187500, 0.0015869140625000,
+0.0030822753906250},
+{0.0026245117187500, 0.0051574707031250, 0.0029602050781250, 0.0050659179687500,
+0.0031127929687500, 0.0052185058593750, 0.0029296875000000, 0.0060729980468750,
+0.0028991699218750, 0.0056457519531250, 0.0028991699218750, 0.0058593750000000,
+0.0025939941406250, 0.0057983398437500, 0.0026550292968750, 0.0054931640625000,
+0.0027160644531250, 0.0054931640625000, 0.0025939941406250, 0.0054321289062500,
+0.0026550292968750, 0.0047912597656250, 0.0029602050781250, 0.0049438476562500,
+0.0028076171875000, 0.0046691894531250, 0.0029602050781250, 0.0039367675781250,
+0.0032653808593750, 0.0039672851562500, 0.0032348632812500, 0.0037536621093750,
+0.0039367675781250, 0.0036621093750000, 0.0036010742187500, 0.0036315917968750,
+0.0039062500000000, 0.0036315917968750, 0.0044555664062500, 0.0036315917968750,
+0.0040283203125000, 0.0033874511718750, 0.0050048828125000, 0.0036315917968750,
+0.0048217773437500, 0.0032653808593750, 0.0049438476562500, 0.0029296875000000,
+0.0057067871093750, 0.0034179687500000, 0.0053405761718750, 0.0025329589843750,
+0.0054321289062500, 0.0026855468750000, 0.0057067871093750, 0.0027160644531250,
+0.0054016113281250, 0.0022277832031250, 0.0050964355468750, 0.0025329589843750,
+0.0052490234375000, 0.0024108886718750, 0.0043945312500000, 0.0023498535156250,
+0.0044555664062500, 0.0023193359375000, 0.0042114257812500, 0.0023803710937500,
+0.0035400390625000, 0.0024414062500000, 0.0036926269531250, 0.0021057128906250,
+0.0028381347656250, 0.0024719238281250, 0.0028076171875000, 0.0022277832031250,
+0.0024719238281250, 0.0018615722656250, 0.0017700195312500, 0.0023803710937500,
+0.0020141601562500, 0.0017089843750000, 0.0011596679687500, 0.0018005371093750,
+0.0011901855468750, 0.0019531250000000, 0.0010681152343750, 0.0016479492187500,
+0.0005187988281250, 0.0017395019531250, 0.0007324218750000, 0.0021667480468750,
+0.0005187988281250, 0.0019226074218750, 0.0003662109375000, 0.0022277832031250,
+0.0004882812500000, 0.0028076171875000, 0.0004577636718750, 0.0024414062500000,
+0.0004272460937500, 0.0027770996093750, 0.0009155273437500, 0.0030822753906250,
+0.0005798339843750, 0.0025939941406250, 0.0011596679687500, 0.0023498535156250,
+0.0018310546875000, 0.0026855468750000, 0.0012817382812500, 0.0014038085937500,
+0.0031127929687500, 0.0015563964843750, 0.0029907226562500, 0.0013427734375000,
+0.0034790039062500, 0.0003967285156250, 0.0049438476562500, 0.0007324218750000,
+0.0043334960937500, 0.0002441406250000, 0.0055236816406250, 0.0000610351562500,
+0.0058898925781250, 0.0001831054687500, 0.0056762695312500, 0.0000305175781250,
+0.0060729980468750, -0.0000915527343750, 0.0062255859375000, 0.0003051757812500,
+0.0057678222656250, -0.0000305175781250, 0.0059509277343750, 0.0002136230468750,
+0.0061035156250000, 0.0006713867187500, 0.0058288574218750, 0.0001831054687500,
+0.0059814453125000, 0.0009460449218750, 0.0062255859375000, 0.0008544921875000,
+0.0063476562500000, 0.0007324218750000, 0.0065002441406250, 0.0011901855468750,
+0.0067138671875000, 0.0010375976562500, 0.0068359375000000, 0.0010986328125000,
+0.0068054199218750, 0.0012512207031250, 0.0069580078125000, 0.0012207031250000,
+0.0068054199218750, 0.0011901855468750, 0.0065917968750000, 0.0014038085937500,
+0.0068054199218750, 0.0012817382812500, 0.0062255859375000, 0.0014343261718750,
+0.0062866210937500, 0.0014343261718750, 0.0059814453125000, 0.0013732910156250,
+0.0054321289062500, 0.0015258789062500, 0.0056152343750000, 0.0012512207031250,
+0.0047912597656250, 0.0012512207031250, 0.0047607421875000, 0.0011596679687500,
+0.0044860839843750, 0.0008544921875000, 0.0038146972656250, 0.0009460449218750,
+0.0040588378906250, 0.0006408691406250, 0.0033264160156250, 0.0004882812500000,
+0.0033569335937500, 0.0005493164062500, 0.0032348632812500, 0.0003662109375000,
+0.0028076171875000, 0.0001831054687500, 0.0030822753906250, 0.0004882812500000,
+0.0026550292968750, 0.0002441406250000, 0.0028076171875000, 0.0003356933593750,
+0.0027465820312500, 0.0006713867187500, 0.0025024414062500, 0.0003051757812500,
+0.0028381347656250, 0.0006713867187500, 0.0023803710937500, 0.0005493164062500,
+0.0026245117187500, 0.0004882812500000, 0.0025024414062500, 0.0006713867187500,
+0.0020141601562500, 0.0005187988281250, 0.0024108886718750, 0.0004882812500000,
+0.0018310546875000, 0.0004272460937500, 0.0018310546875000, 0.0003662109375000,
+0.0017089843750000, 0.0003051757812500, 0.0012817382812500, 0.0001831054687500,
+0.0013732910156250, 0.0003051757812500, 0.0010375976562500, 0.0002136230468750,
+0.0008850097656250, 0.0001525878906250, 0.0008544921875000, 0.0001831054687500,
+0.0007019042968750, 0.0000915527343750, 0.0006103515625000, 0.0000305175781250,
+0.0009460449218750, 0.0000305175781250, 0.0006713867187500, 0.0000000000000000,
+0.0009460449218750, -0.0000610351562500, 0.0014038085937500, -0.0000610351562500,
+0.0010375976562500, 0.0000610351562500, 0.0020446777343750, 0.0000000000000000,
+0.0018310546875000, 0.0000915527343750, 0.0021667480468750, 0.0001831054687500,
+0.0030517578125000, 0.0001220703125000, 0.0026855468750000, 0.0004272460937500,
+0.0035705566406250, 0.0003356933593750, 0.0035705566406250, 0.0004882812500000,
+0.0035705566406250, 0.0007934570312500, 0.0041503906250000, 0.0006408691406250,
+0.0039672851562500, 0.0008850097656250, 0.0037536621093750, 0.0008850097656250,
+0.0041198730468750, 0.0008239746093750, 0.0036315917968750, 0.0008850097656250,
+0.0030822753906250, 0.0007629394531250, 0.0035400390625000, 0.0008239746093750,
+0.0025634765625000, 0.0006713867187500, 0.0027160644531250, 0.0006408691406250,
+0.0026245117187500, 0.0007019042968750, 0.0019226074218750, 0.0005187988281250,
+0.0022277832031250, 0.0007629394531250, 0.0020446777343750, 0.0007324218750000,
+0.0018310546875000, 0.0008239746093750, 0.0021362304687500, 0.0010681152343750,
+0.0024108886718750, 0.0010681152343750, 0.0022277832031250, 0.0012817382812500,
+0.0030212402343750, 0.0013732910156250, 0.0028686523437500, 0.0014343261718750,
+0.0030822753906250, 0.0015563964843750, 0.0036621093750000, 0.0016479492187500,
+0.0032653808593750, 0.0015258789062500, 0.0040893554687500, 0.0017089843750000,
+0.0039062500000000, 0.0016174316406250, 0.0040893554687500, 0.0014953613281250,
+0.0046386718750000, 0.0018005371093750, 0.0042114257812500, 0.0012817382812500,
+0.0046691894531250, 0.0015563964843750, 0.0045776367187500, 0.0015258789062500,
+0.0045471191406250, 0.0010986328125000, 0.0047607421875000, 0.0014343261718750,
+0.0045166015625000, 0.0011596679687500, 0.0048217773437500, 0.0010681152343750,
+0.0047912597656250, 0.0014648437500000, 0.0045471191406250, 0.0015563964843750,
+0.0046386718750000, 0.0013427734375000, 0.0046997070312500, 0.0028991699218750,
+0.0041503906250000, 0.0024108886718750, 0.0043029785156250, 0.0030822753906250,
+0.0041809082031250, 0.0045471191406250, 0.0037841796875000, 0.0035705566406250,
+0.0039978027343750, 0.0050354003906250, 0.0036010742187500, 0.0051879882812500,
+0.0036010742187500, 0.0047302246093750, 0.0038452148437500, 0.0051269531250000,
+0.0037536621093750, 0.0052185058593750, 0.0037536621093750, 0.0044555664062500,
+0.0045471191406250, 0.0045166015625000, 0.0042419433593750, 0.0043945312500000,
+0.0049133300781250, 0.0037536621093750, 0.0060119628906250, 0.0037841796875000,
+0.0055236816406250, 0.0040283203125000, 0.0065002441406250, 0.0034484863281250,
+0.0068359375000000, 0.0040283203125000, 0.0063781738281250, 0.0047302246093750,
+0.0063781738281250, 0.0038757324218750, 0.0066223144531250, 0.0055541992187500,
+0.0056762695312500, 0.0053100585937500, 0.0056457519531250, 0.0053405761718750,
+0.0054931640625000, 0.0063781738281250, 0.0049743652343750, 0.0058288574218750,
+0.0052185058593750, 0.0057373046875000, 0.0048522949218750, 0.0059509277343750,
+0.0049133300781250, 0.0054321289062500, 0.0048828125000000, 0.0044860839843750,
+0.0045166015625000, 0.0029602050781250, 0.0055236816406250, 0.0042724609375000,
+0.0045166015625000, 0.0038452148437500, 0.0046081542968750, 0.0040893554687500,
+0.0044250488281250, 0.0046691894531250, 0.0039978027343750, 0.0044860839843750,
+0.0041198730468750, 0.0048522949218750, 0.0039062500000000, 0.0046386718750000,
+0.0037841796875000, 0.0051269531250000, 0.0038452148437500, 0.0055541992187500,
+0.0037536621093750, 0.0049743652343750, 0.0036621093750000, 0.0068664550781250,
+0.0040893554687500, 0.0065917968750000, 0.0039978027343750, 0.0068054199218750,
+0.0039367675781250, 0.0081481933593750, 0.0041198730468750, 0.0074768066406250,
+0.0040588378906250, 0.0071411132812500, 0.0033264160156250, 0.0079345703125000,
+0.0036010742187500, 0.0065917968750000, 0.0030822753906250, 0.0053405761718750,
+0.0022277832031250, 0.0065917968750000, 0.0025329589843750, 0.0035705566406250,
+0.0017395019531250, 0.0039672851562500, 0.0014953613281250, 0.0035400390625000,
+0.0015563964843750, 0.0016479492187500, 0.0011901855468750, 0.0027465820312500,
+0.0011596679687500, 0.0011901855468750, 0.0019226074218750, 0.0011291503906250,
+0.0015869140625000, 0.0011596679687500, 0.0021362304687500, 0.0002746582031250,
+0.0030822753906250, 0.0005187988281250, 0.0025024414062500, 0.0003967285156250,
+0.0034790039062500, -0.0000305175781250, 0.0038452148437500, 0.0003051757812500,
+0.0033264160156250, 0.0003967285156250, 0.0033874511718750, -0.0001220703125000,
+0.0036926269531250, 0.0013427734375000, 0.0020141601562500, 0.0006713867187500,
+0.0022888183593750, 0.0013427734375000, 0.0019531250000000, 0.0027160644531250,
+0.0006408691406250, 0.0015869140625000, 0.0010681152343750, 0.0037841796875000,
+0.0007019042968750, 0.0035095214843750, 0.0003051757812500, 0.0038452148437500,
+0.0007934570312500, 0.0053405761718750, 0.0011291503906250, 0.0044250488281250,
+0.0007629394531250, 0.0054321289062500, 0.0017089843750000, 0.0055847167968750,
+0.0018005371093750, 0.0054016113281250, 0.0017700195312500, 0.0056762695312500,
+0.0023193359375000, 0.0054626464843750, 0.0022583007812500, 0.0056152343750000,
+0.0018615722656250, 0.0054931640625000, 0.0023498535156250, 0.0054321289062500,
+0.0018615722656250, 0.0055236816406250, 0.0011291503906250, 0.0054321289062500,
+0.0018005371093750, 0.0052490234375000, 0.0007019042968750, 0.0054016113281250,
+0.0007019042968750, 0.0052490234375000, 0.0008850097656250, 0.0050048828125000,
+0.0003967285156250, 0.0051269531250000, 0.0005798339843750, 0.0047912597656250,
+0.0009765625000000, 0.0047302246093750, 0.0007629394531250, 0.0047302246093750,
+0.0010375976562500, 0.0044860839843750, 0.0015869140625000, 0.0045166015625000,
+0.0012512207031250, 0.0045776367187500, 0.0017700195312500, 0.0043945312500000,
+0.0018310546875000, 0.0048217773437500, 0.0017089843750000, 0.0051574707031250,
+0.0018615722656250, 0.0048522949218750, 0.0018920898437500, 0.0061035156250000,
+0.0014038085937500, 0.0058898925781250, 0.0015869140625000, 0.0063781738281250,
+0.0013122558593750, 0.0075378417968750, 0.0007934570312500, 0.0069274902343750,
+0.0010986328125000, 0.0080871582031250, 0.0005493164062500, 0.0083923339843750,
+0.0004882812500000, 0.0081481933593750, 0.0005493164062500, 0.0086059570312500,
+0.0002746582031250, 0.0087280273437500, 0.0003662109375000, 0.0078735351562500,
+0.0003051757812500, 0.0084533691406250, 0.0002746582031250, 0.0077819824218750,
+0.0002136230468750},
+{0.0057373046875000, 0.0039978027343750, 0.0050659179687500, 0.0044555664062500,
+0.0044250488281250, 0.0037841796875000, 0.0048217773437500, 0.0054321289062500,
+0.0038757324218750, 0.0052795410156250, 0.0039672851562500, 0.0051879882812500,
+0.0037231445312500, 0.0063171386718750, 0.0030517578125000, 0.0060119628906250,
+0.0033569335937500, 0.0049743652343750, 0.0024719238281250, 0.0056762695312500,
+0.0022583007812500, 0.0045166015625000, 0.0024108886718750, 0.0028381347656250,
+0.0020141601562500, 0.0039672851562500, 0.0019226074218750, 0.0015563964843750,
+0.0025329589843750, 0.0015869140625000, 0.0024108886718750, 0.0016479492187500,
+0.0024719238281250, 0.0004882812500000, 0.0029907226562500, 0.0010070800781250,
+0.0029296875000000, 0.0007934570312500, 0.0025329589843750, 0.0007324218750000,
+0.0029602050781250, 0.0007934570312500, 0.0024414062500000, 0.0006713867187500,
+0.0016784667968750, 0.0007934570312500, 0.0023498535156250, 0.0011291503906250,
+0.0009460449218750, 0.0007019042968750, 0.0010375976562500, 0.0012817382812500,
+0.0010681152343750, 0.0018920898437500, 0.0002746582031250, 0.0012207031250000,
+0.0006408691406250, 0.0030822753906250, 0.0006713867187500, 0.0026855468750000,
+0.0003967285156250, 0.0031127929687500, 0.0009460449218750, 0.0044250488281250,
+0.0014648437500000, 0.0036010742187500, 0.0011901855468750, 0.0050354003906250,
+0.0020751953125000, 0.0048828125000000, 0.0022888183593750, 0.0049133300781250,
+0.0020446777343750, 0.0057678222656250, 0.0023803710937500, 0.0052795410156250,
+0.0025329589843750, 0.0057678222656250, 0.0014343261718750, 0.0057373046875000,
+0.0017700195312500, 0.0056457519531250, 0.0014038085937500, 0.0057983398437500,
+0.0002441406250000, 0.0055541992187500, 0.0008850097656250, 0.0059204101562500,
+0.0002746582031250, 0.0055847167968750, 0.0000305175781250, 0.0057373046875000,
+0.0004882812500000, 0.0061950683593750, 0.0005493164062500, 0.0056762695312500,
+0.0003662109375000, 0.0061950683593750, 0.0013122558593750, 0.0061340332031250,
+0.0009765625000000, 0.0059204101562500, 0.0013427734375000, 0.0060119628906250,
+0.0021972656250000, 0.0058898925781250, 0.0016784667968750, 0.0057678222656250,
+0.0022888183593750, 0.0056152343750000, 0.0024414062500000, 0.0057983398437500,
+0.0021057128906250, 0.0058593750000000, 0.0022277832031250, 0.0056457519531250,
+0.0023803710937500, 0.0066223144531250, 0.0015869140625000, 0.0061950683593750,
+0.0018005371093750, 0.0068359375000000, 0.0014648437500000, 0.0079040527343750,
+0.0008544921875000, 0.0071105957031250, 0.0012207031250000, 0.0090637207031250,
+0.0006408691406250, 0.0088195800781250, 0.0005493164062500, 0.0090637207031250,
+0.0006713867187500, 0.0104064941406250, 0.0004272460937500, 0.0098571777343750,
+0.0005187988281250, 0.0103149414062500, 0.0006408691406250, 0.0108032226562500,
+0.0006713867187500, 0.0101318359375000, 0.0007019042968750, 0.0099487304687500,
+0.0007934570312500, 0.0104980468750000, 0.0008850097656250, 0.0085144042968750,
+0.0007934570312500, 0.0091247558593750, 0.0008239746093750, 0.0085144042968750,
+0.0007324218750000, 0.0068664550781250, 0.0004577636718750, 0.0079040527343750,
+0.0004882812500000, 0.0060729980468750, 0.0003356933593750, 0.0061035156250000,
+0.0001525878906250, 0.0061035156250000, 0.0001220703125000, 0.0050964355468750,
+0.0001220703125000, 0.0054931640625000, -0.0000915527343750, 0.0052490234375000,
+0.0000000000000000, 0.0050048828125000, -0.0000915527343750, 0.0052795410156250,
+-0.0000610351562500, 0.0054626464843750, 0.0000915527343750, 0.0051574707031250,
+-0.0000610351562500, 0.0057983398437500, 0.0001831054687500, 0.0056152343750000,
+0.0001525878906250, 0.0057373046875000, 0.0002746582031250, 0.0061035156250000,
+0.0003967285156250, 0.0058288574218750, 0.0002136230468750, 0.0061340332031250,
+0.0006713867187500, 0.0061340332031250, 0.0003967285156250, 0.0060119628906250,
+0.0006713867187500, 0.0060729980468750, 0.0009155273437500, 0.0060119628906250,
+0.0003662109375000, 0.0058288574218750, 0.0012512207031250, 0.0058898925781250,
+0.0009460449218750, 0.0056762695312500, 0.0008544921875000, 0.0054321289062500,
+0.0014648437500000, 0.0056457519531250, 0.0010070800781250, 0.0051269531250000,
+0.0013427734375000, 0.0051574707031250, 0.0013732910156250, 0.0050659179687500,
+0.0012207031250000, 0.0046081542968750, 0.0014038085937500, 0.0047607421875000,
+0.0014038085937500, 0.0043640136718750, 0.0012207031250000, 0.0042419433593750,
+0.0013122558593750, 0.0041198730468750, 0.0011901855468750, 0.0038757324218750,
+0.0009460449218750, 0.0038146972656250, 0.0010986328125000, 0.0035400390625000,
+0.0009765625000000, 0.0034790039062500, 0.0008239746093750, 0.0035095214843750,
+0.0010070800781250, 0.0033264160156250, 0.0010681152343750, 0.0032958984375000,
+0.0008544921875000, 0.0033874511718750, 0.0011291503906250, 0.0033569335937500,
+0.0009155273437500, 0.0035095214843750, 0.0009765625000000, 0.0036926269531250,
+0.0012512207031250, 0.0036315917968750, 0.0009765625000000, 0.0038757324218750,
+0.0012207031250000, 0.0039062500000000, 0.0011901855468750, 0.0038757324218750,
+0.0011291503906250, 0.0039367675781250, 0.0012207031250000, 0.0038452148437500,
+0.0010070800781250, 0.0037231445312500, 0.0010986328125000, 0.0036621093750000,
+0.0010986328125000, 0.0036621093750000, 0.0009460449218750, 0.0035400390625000,
+0.0008850097656250, 0.0034179687500000, 0.0008239746093750, 0.0037231445312500,
+0.0007934570312500, 0.0035095214843750, 0.0007324218750000, 0.0036315917968750,
+0.0008239746093750, 0.0038757324218750, 0.0008239746093750, 0.0035400390625000,
+0.0007324218750000, 0.0039062500000000, 0.0008850097656250, 0.0038146972656250,
+0.0008239746093750, 0.0037841796875000, 0.0007324218750000, 0.0039062500000000,
+0.0007629394531250, 0.0036926269531250, 0.0007019042968750, 0.0039367675781250,
+0.0006713867187500, 0.0037536621093750, 0.0005187988281250, 0.0039367675781250,
+0.0007629394531250, 0.0042419433593750, 0.0009460449218750, 0.0038757324218750,
+0.0007019042968750, 0.0046386718750000, 0.0016174316406250, 0.0045471191406250,
+0.0014038085937500, 0.0043945312500000, 0.0016479492187500, 0.0048522949218750,
+0.0023498535156250, 0.0047302246093750, 0.0019836425781250, 0.0043640136718750,
+0.0023193359375000, 0.0046081542968750, 0.0025024414062500, 0.0041809082031250,
+0.0022277832031250, 0.0036621093750000, 0.0021057128906250, 0.0039367675781250,
+0.0023193359375000, 0.0028686523437500, 0.0018310546875000, 0.0028991699218750,
+0.0018615722656250, 0.0024719238281250, 0.0018615722656250, 0.0017395019531250,
+0.0015563964843750, 0.0019531250000000, 0.0016174316406250, 0.0006713867187500,
+0.0018310546875000, 0.0007629394531250, 0.0015563964843750, 0.0004577636718750,
+0.0018615722656250, -0.0005187988281250, 0.0022277832031250, -0.0001831054687500,
+0.0018615722656250, -0.0004577636718750, 0.0030517578125000, -0.0006408691406250,
+0.0027160644531250, -0.0002136230468750, 0.0030517578125000, -0.0000305175781250,
+0.0039672851562500, 0.0000000000000000, 0.0032958984375000, 0.0008850097656250,
+0.0043334960937500, 0.0008239746093750, 0.0041503906250000, 0.0012207031250000,
+0.0040893554687500, 0.0022888183593750, 0.0047302246093750, 0.0028076171875000,
+0.0045471191406250, 0.0038146972656250, 0.0043029785156250, 0.0035705566406250,
+0.0044860839843750, 0.0036621093750000, 0.0040893554687500, 0.0043945312500000,
+0.0036010742187500, 0.0041503906250000, 0.0039978027343750, 0.0042724609375000,
+0.0030517578125000, 0.0045471191406250, 0.0031127929687500, 0.0042419433593750,
+0.0028686523437500, 0.0039672851562500, 0.0021667480468750, 0.0042114257812500,
+0.0024414062500000, 0.0036926269531250, 0.0018005371093750, 0.0036926269531250,
+0.0018310546875000, 0.0035400390625000, 0.0016784667968750, 0.0032348632812500,
+0.0012207031250000, 0.0035095214843750, 0.0014648437500000, 0.0030517578125000,
+0.0013427734375000, 0.0032653808593750, 0.0012512207031250, 0.0031127929687500,
+0.0015869140625000, 0.0027465820312500, 0.0017395019531250, 0.0030822753906250,
+0.0015563964843750, 0.0025634765625000, 0.0027770996093750, 0.0025634765625000,
+0.0024414062500000, 0.0025024414062500, 0.0030212402343750, 0.0022583007812500,
+0.0040283203125000, 0.0023803710937500, 0.0033569335937500, 0.0019836425781250,
+0.0050354003906250, 0.0021362304687500, 0.0047302246093750, 0.0019226074218750,
+0.0052490234375000, 0.0014343261718750, 0.0066223144531250, 0.0018005371093750,
+0.0057983398437500, 0.0010375976562500, 0.0071716308593750, 0.0011291503906250,
+0.0073242187500000, 0.0010375976562500, 0.0070190429687500, 0.0005798339843750,
+0.0075073242187500, 0.0008239746093750, 0.0073852539062500, 0.0001831054687500,
+0.0067138671875000, 0.0002441406250000, 0.0069274902343750, 0.0003356933593750,
+0.0065307617187500, -0.0001220703125000, 0.0057983398437500, 0.0000000000000000,
+0.0061340332031250, 0.0003051757812500, 0.0055541992187500, -0.0000305175781250,
+0.0053710937500000, 0.0003662109375000, 0.0055847167968750, 0.0008239746093750,
+0.0054321289062500, 0.0003051757812500, 0.0053405761718750, 0.0010986328125000,
+0.0058898925781250, 0.0009765625000000, 0.0056457519531250, 0.0008850097656250,
+0.0058593750000000, 0.0012817382812500, 0.0063781738281250, 0.0010070800781250,
+0.0060424804687500, 0.0008239746093750, 0.0065307617187500, 0.0010375976562500,
+0.0065002441406250, 0.0007629394531250, 0.0065002441406250, 0.0003356933593750,
+0.0067443847656250, 0.0006713867187500, 0.0066223144531250, 0.0001525878906250,
+0.0067138671875000, 0.0000610351562500, 0.0066833496093750, 0.0001831054687500,
+0.0065917968750000, -0.0000610351562500, 0.0065917968750000, -0.0001831054687500,
+0.0065612792968750, 0.0003662109375000, 0.0066223144531250, -0.0000610351562500,
+0.0065307617187500, 0.0002746582031250, 0.0065612792968750, 0.0009765625000000,
+0.0066833496093750, 0.0003051757812500, 0.0065917968750000, 0.0014343261718750,
+0.0068969726562500, 0.0011596679687500, 0.0068969726562500, 0.0012817382812500,
+0.0070190429687500, 0.0020141601562500, 0.0072937011718750, 0.0014343261718750,
+0.0072937011718750, 0.0019531250000000, 0.0073852539062500, 0.0019226074218750,
+0.0076599121093750, 0.0017395019531250, 0.0071716308593750, 0.0019836425781250,
+0.0069274902343750, 0.0018310546875000, 0.0073242187500000, 0.0017395019531250,
+0.0059814453125000, 0.0018615722656250, 0.0063171386718750, 0.0016174316406250,
+0.0058898925781250, 0.0014038085937500, 0.0047912597656250, 0.0017700195312500,
+0.0054321289062500, 0.0009765625000000, 0.0042724609375000, 0.0013122558593750,
+0.0042724609375000, 0.0010070800781250, 0.0043029785156250, 0.0003967285156250,
+0.0036315917968750, 0.0009765625000000, 0.0038757324218750, -0.0000305175781250,
+0.0038146972656250, 0.0002136230468750, 0.0036010742187500, 0.0000610351562500,
+0.0038146972656250, -0.0005798339843750, 0.0039672851562500, -0.0000915527343750,
+0.0037536621093750, -0.0007324218750000, 0.0041809082031250, -0.0006103515625000,
+0.0041503906250000, -0.0005187988281250, 0.0041809082031250, -0.0007934570312500,
+0.0043334960937500, -0.0004882812500000, 0.0042114257812500, -0.0005493164062500,
+0.0043945312500000, -0.0004882812500000, 0.0043945312500000, -0.0003967285156250,
+0.0043945312500000, -0.0003356933593750, 0.0045166015625000, -0.0001831054687500,
+0.0043945312500000},
+{0.0048522949218750, 0.0024719238281250, 0.0046386718750000, 0.0020751953125000,
+0.0055236816406250, 0.0021362304687500, 0.0054626464843750, 0.0020751953125000,
+0.0056152343750000, 0.0017089843750000, 0.0062255859375000, 0.0018310546875000,
+0.0059509277343750, 0.0017395019531250, 0.0062866210937500, 0.0017089843750000,
+0.0064697265625000, 0.0016784667968750, 0.0061035156250000, 0.0016174316406250,
+0.0059509277343750, 0.0017700195312500, 0.0060729980468750, 0.0012817382812500,
+0.0050659179687500, 0.0014953613281250, 0.0050964355468750, 0.0013732910156250,
+0.0048828125000000, 0.0009155273437500, 0.0040893554687500, 0.0011596679687500,
+0.0042724609375000, 0.0006713867187500, 0.0041198730468750, 0.0007019042968750,
+0.0038452148437500, 0.0005493164062500, 0.0041198730468750, 0.0002441406250000,
+0.0043945312500000, 0.0003662109375000, 0.0040893554687500, -0.0000305175781250,
+0.0046081542968750, 0.0000000000000000, 0.0045776367187500, 0.0000305175781250,
+0.0045776367187500, -0.0001831054687500, 0.0048522949218750, -0.0001220703125000,
+0.0046386718750000, 0.0001525878906250, 0.0047302246093750, -0.0000915527343750,
+0.0046386718750000, 0.0001525878906250, 0.0045776367187500, 0.0005187988281250,
+0.0045776367187500, 0.0001525878906250, 0.0044860839843750, 0.0007629394531250,
+0.0043334960937500, 0.0006408691406250, 0.0043029785156250, 0.0005798339843750,
+0.0040893554687500, 0.0009460449218750, 0.0038452148437500, 0.0007934570312500,
+0.0038757324218750, 0.0006103515625000, 0.0036315917968750, 0.0006713867187500,
+0.0034484863281250, 0.0006103515625000, 0.0034484863281250, 0.0003662109375000,
+0.0033264160156250, 0.0004272460937500, 0.0031738281250000, 0.0002441406250000,
+0.0034484863281250, 0.0001220703125000, 0.0032653808593750, 0.0002441406250000,
+0.0032958984375000, 0.0001831054687500, 0.0035705566406250, 0.0000000000000000,
+0.0033874511718750, 0.0007019042968750, 0.0035400390625000, 0.0002746582031250,
+0.0036315917968750, 0.0006408691406250, 0.0035400390625000, 0.0013427734375000,
+0.0034790039062500, 0.0006713867187500, 0.0036010742187500, 0.0017395019531250,
+0.0034484863281250, 0.0015258789062500, 0.0033569335937500, 0.0014953613281250,
+0.0035705566406250, 0.0020751953125000, 0.0035400390625000, 0.0017089843750000,
+0.0034484863281250, 0.0018005371093750, 0.0042419433593750, 0.0018615722656250,
+0.0041198730468750, 0.0016174316406250, 0.0045471191406250, 0.0014648437500000,
+0.0052490234375000, 0.0016174316406250, 0.0049133300781250, 0.0011901855468750,
+0.0058898925781250, 0.0012512207031250, 0.0057983398437500, 0.0011596679687500,
+0.0061950683593750, 0.0008544921875000, 0.0068664550781250, 0.0010681152343750,
+0.0064697265625000, 0.0006408691406250, 0.0073242187500000, 0.0007629394531250,
+0.0073547363281250, 0.0006103515625000, 0.0073547363281250, 0.0002441406250000,
+0.0077514648437500, 0.0006103515625000, 0.0075378417968750, -0.0000610351562500,
+0.0076599121093750, 0.0001831054687500, 0.0076599121093750, -0.0000305175781250,
+0.0073852539062500, -0.0005187988281250, 0.0072631835937500, -0.0000610351562500,
+0.0072326660156250, -0.0007019042968750, 0.0065612792968750, -0.0006103515625000,
+0.0066833496093750, -0.0005798339843750, 0.0061950683593750, -0.0009155273437500,
+0.0055541992187500, -0.0006713867187500, 0.0059509277343750, -0.0007019042968750,
+0.0046081542968750, -0.0007324218750000, 0.0048522949218750, -0.0005187988281250,
+0.0044250488281250, -0.0003967285156250, 0.0033264160156250, -0.0005187988281250,
+0.0039978027343750, -0.0001220703125000, 0.0027465820312500, -0.0001525878906250,
+0.0027770996093750, -0.0000305175781250, 0.0027160644531250, 0.0002746582031250,
+0.0020141601562500, 0.0000305175781250, 0.0023803710937500, 0.0005187988281250,
+0.0022888183593750, 0.0003356933593750, 0.0021972656250000, 0.0004272460937500,
+0.0024719238281250, 0.0007934570312500, 0.0026855468750000, 0.0005187988281250,
+0.0025329589843750, 0.0010375976562500, 0.0028991699218750, 0.0010681152343750,
+0.0030212402343750, 0.0011901855468750, 0.0028991699218750, 0.0015869140625000,
+0.0030212402343750, 0.0015258789062500, 0.0030517578125000, 0.0016784667968750,
+0.0026855468750000, 0.0019531250000000, 0.0028381347656250, 0.0018005371093750,
+0.0027465820312500, 0.0017089843750000, 0.0022888183593750, 0.0019226074218750,
+0.0025024414062500, 0.0012207031250000, 0.0022583007812500, 0.0013732910156250,
+0.0021057128906250, 0.0012817382812500, 0.0022583007812500, 0.0007324218750000,
+0.0023193359375000, 0.0010070800781250, 0.0022277832031250, 0.0007629394531250,
+0.0025024414062500, 0.0006103515625000, 0.0024108886718750, 0.0009155273437500,
+0.0023193359375000, 0.0010681152343750, 0.0024414062500000, 0.0009155273437500,
+0.0023498535156250, 0.0017700195312500, 0.0021057128906250, 0.0016784667968750,
+0.0021667480468750, 0.0018920898437500, 0.0019531250000000, 0.0025329589843750,
+0.0016174316406250, 0.0023498535156250, 0.0015869140625000, 0.0022277832031250,
+0.0010375976562500, 0.0024719238281250, 0.0010375976562500, 0.0020141601562500,
+0.0008850097656250, 0.0014953613281250, 0.0003356933593750, 0.0018005371093750,
+0.0003967285156250, 0.0009765625000000, 0.0003356933593750, 0.0008544921875000,
+0.0000305175781250, 0.0009765625000000, 0.0004577636718750, 0.0007324218750000,
+0.0007629394531250, 0.0007629394531250, 0.0003356933593750, 0.0016174316406250,
+0.0017089843750000, 0.0012817382812500, 0.0014648437500000, 0.0016784667968750,
+0.0018920898437500, 0.0025939941406250, 0.0028991699218750, 0.0021972656250000,
+0.0023498535156250, 0.0027770996093750, 0.0035705566406250, 0.0032043457031250,
+0.0034790039062500, 0.0028686523437500, 0.0036621093750000, 0.0028686523437500,
+0.0044555664062500, 0.0032653808593750, 0.0039978027343750, 0.0022888183593750,
+0.0048217773437500, 0.0025634765625000, 0.0048217773437500, 0.0024719238281250,
+0.0047912597656250, 0.0018005371093750, 0.0052795410156250, 0.0022583007812500,
+0.0050659179687500, 0.0022583007812500, 0.0050659179687500, 0.0022277832031250,
+0.0052185058593750, 0.0025939941406250, 0.0047912597656250, 0.0029907226562500,
+0.0044860839843750, 0.0029907226562500, 0.0046691894531250, 0.0032958984375000,
+0.0032958984375000, 0.0035095214843750, 0.0036010742187500, 0.0032653808593750,
+0.0028381347656250, 0.0032348632812500, 0.0014038085937500, 0.0036926269531250,
+0.0022888183593750, 0.0032653808593750, 0.0008850097656250, 0.0033264160156250,
+0.0005798339843750, 0.0036926269531250, 0.0011596679687500, 0.0038146972656250,
+0.0010375976562500, 0.0050048828125000, 0.0043029785156250, 0.0023193359375000,
+0.0019531250000000, 0.0039978027343750, 0.0022277832031250, 0.0027465820312500,
+0.0023498535156250, 0.0005798339843750, 0.0017089843750000, 0.0019531250000000,
+0.0017395019531250, 0.0005798339843750, 0.0024414062500000, 0.0001831054687500,
+0.0023498535156250, 0.0007934570312500, 0.0023803710937500, 0.0006408691406250,
+0.0028076171875000, 0.0004272460937500, 0.0026855468750000, 0.0014038085937500,
+0.0028686523437500, 0.0013427734375000, 0.0027465820312500, 0.0014648437500000,
+0.0031433105468750, 0.0018920898437500, 0.0035095214843750, 0.0017700195312500,
+0.0031433105468750, 0.0025939941406250, 0.0039978027343750, 0.0022888183593750,
+0.0040588378906250, 0.0030212402343750, 0.0036621093750000, 0.0040283203125000,
+0.0038757324218750, 0.0032958984375000, 0.0039367675781250, 0.0053710937500000,
+0.0029907226562500, 0.0053100585937500, 0.0031433105468750, 0.0054931640625000,
+0.0028381347656250, 0.0068054199218750, 0.0021057128906250, 0.0065917968750000,
+0.0023803710937500, 0.0069580078125000, 0.0019531250000000, 0.0074157714843750,
+0.0018310546875000, 0.0071411132812500, 0.0018920898437500, 0.0070495605468750,
+0.0017700195312500, 0.0075378417968750, 0.0017089843750000, 0.0067443847656250,
+0.0018005371093750, 0.0070495605468750, 0.0017395019531250, 0.0068664550781250,
+0.0017700195312500, 0.0064392089843750, 0.0018920898437500, 0.0068664550781250,
+0.0017395019531250, 0.0054626464843750, 0.0015258789062500, 0.0060729980468750,
+0.0017089843750000, 0.0053710937500000, 0.0014648437500000, 0.0039062500000000,
+0.0009765625000000, 0.0046997070312500, 0.0012512207031250, 0.0027160644531250,
+0.0008544921875000, 0.0028686523437500, 0.0007019042968750, 0.0025634765625000,
+0.0009765625000000, 0.0012817382812500, 0.0010375976562500, 0.0018920898437500,
+0.0009460449218750, 0.0011291503906250, 0.0014953613281250, 0.0010070800781250,
+0.0015869140625000, 0.0014038085937500, 0.0015869140625000, 0.0012817382812500,
+0.0019226074218750, 0.0012207031250000, 0.0019531250000000, 0.0021972656250000,
+0.0017700195312500, 0.0020751953125000, 0.0020751953125000, 0.0022277832031250,
+0.0018310546875000, 0.0029296875000000, 0.0015258789062500, 0.0027160644531250,
+0.0019226074218750, 0.0026855468750000, 0.0012512207031250, 0.0029907226562500,
+0.0013427734375000, 0.0024108886718750, 0.0012817382812500, 0.0019226074218750,
+0.0007934570312500, 0.0023193359375000, 0.0010681152343750, 0.0007324218750000,
+0.0007629394531250, 0.0010070800781250, 0.0006713867187500, 0.0006103515625000,
+0.0007629394531250, -0.0005187988281250, 0.0006713867187500, 0.0001831054687500,
+0.0006408691406250, -0.0007934570312500, 0.0005798339843750, -0.0007629394531250,
+0.0006713867187500, -0.0005798339843750, 0.0004577636718750, -0.0009765625000000,
+0.0003356933593750, -0.0007629394531250, 0.0005187988281250, -0.0004272460937500,
+-0.0000305175781250, -0.0005187988281250, 0.0000915527343750, -0.0000915527343750,
+-0.0001220703125000, 0.0003967285156250, -0.0005187988281250, 0.0002441406250000,
+-0.0001831054687500, 0.0011291503906250, -0.0008544921875000, 0.0011901855468750,
+-0.0007934570312500, 0.0014343261718750, -0.0007934570312500, 0.0020446777343750,
+-0.0011291503906250, 0.0018310546875000, -0.0008850097656250, 0.0021057128906250,
+-0.0008544921875000, 0.0023193359375000, -0.0009460449218750, 0.0021057128906250,
+-0.0006713867187500, 0.0020446777343750, -0.0004882812500000, 0.0022277832031250,
+-0.0005493164062500, 0.0015258789062500, -0.0001220703125000, 0.0018005371093750,
+-0.0001831054687500, 0.0015869140625000, 0.0000610351562500, 0.0009155273437500,
+0.0004882812500000, 0.0013427734375000, 0.0002136230468750, 0.0008850097656250,
+0.0006713867187500, 0.0008239746093750, 0.0006408691406250, 0.0010681152343750,
+0.0007019042968750, 0.0010375976562500, 0.0009460449218750, 0.0010681152343750,
+0.0009460449218750, 0.0016174316406250, 0.0010375976562500, 0.0013732910156250,
+0.0011596679687500, 0.0017700195312500, 0.0011901855468750, 0.0023498535156250,
+0.0011901855468750, 0.0019531250000000, 0.0012207031250000, 0.0029296875000000,
+0.0013122558593750, 0.0028076171875000, 0.0012817382812500, 0.0030517578125000,
+0.0012817382812500, 0.0037231445312500, 0.0013732910156250, 0.0033874511718750,
+0.0012817382812500, 0.0041809082031250, 0.0014038085937500, 0.0041809082031250,
+0.0012207031250000, 0.0043945312500000, 0.0012207031250000, 0.0049438476562500,
+0.0013122558593750, 0.0046691894531250, 0.0010986328125000, 0.0050964355468750,
+0.0011901855468750, 0.0050964355468750, 0.0011291503906250, 0.0051574707031250,
+0.0010375976562500, 0.0054016113281250, 0.0010070800781250, 0.0051574707031250,
+0.0009765625000000, 0.0054016113281250, 0.0009155273437500, 0.0052795410156250,
+0.0008850097656250},
+{0.0015869140625000, 0.0003662109375000, 0.0015258789062500, 0.0007019042968750,
+0.0015563964843750, -0.0002746582031250, 0.0016479492187500, 0.0001220703125000,
+0.0016479492187500, 0.0005493164062500, 0.0014343261718750, 0.0001525878906250,
+0.0016174316406250, 0.0007324218750000, 0.0014343261718750, 0.0014343261718750,
+0.0010681152343750, 0.0010070800781250, 0.0013427734375000, 0.0021057128906250,
+0.0010375976562500, 0.0021057128906250, 0.0008544921875000, 0.0020141601562500,
+0.0010681152343750, 0.0025634765625000, 0.0010681152343750, 0.0024108886718750,
+0.0008544921875000, 0.0019226074218750, 0.0013427734375000, 0.0022583007812500,
+0.0011596679687500, 0.0019531250000000, 0.0012207031250000, 0.0011901855468750,
+0.0016174316406250, 0.0015869140625000, 0.0013122558593750, 0.0010070800781250,
+0.0015258789062500, 0.0007019042968750, 0.0014953613281250, 0.0009460449218750,
+0.0012817382812500, 0.0007324218750000, 0.0013122558593750, 0.0005493164062500,
+0.0012817382812500, 0.0013732910156250, 0.0007019042968750, 0.0009460449218750,
+0.0009765625000000, 0.0012817382812500, 0.0007019042968750, 0.0021667480468750,
+0.0001831054687500, 0.0015869140625000, 0.0006713867187500, 0.0023498535156250,
+0.0000610351562500, 0.0024108886718750, 0.0001525878906250, 0.0022277832031250,
+0.0001525878906250, 0.0024719238281250, -0.0001525878906250, 0.0024108886718750,
+0.0001220703125000, 0.0021362304687500, -0.0003662109375000, 0.0022888183593750,
+-0.0002136230468750, 0.0018310546875000, -0.0003662109375000, 0.0014648437500000,
+-0.0007324218750000, 0.0016479492187500, -0.0004272460937500, 0.0004882812500000,
+-0.0007934570312500, 0.0006103515625000, -0.0007324218750000, 0.0001525878906250,
+-0.0007019042968750, -0.0007629394531250, -0.0008850097656250, -0.0003051757812500,
+-0.0006713867187500, -0.0017089843750000, -0.0006713867187500, -0.0017089843750000,
+-0.0007629394531250, -0.0018920898437500, -0.0005493164062500, -0.0027770996093750,
+-0.0003662109375000, -0.0024719238281250, -0.0004272460937500, -0.0027465820312500,
+-0.0000610351562500, -0.0030822753906250, 0.0000305175781250, -0.0025939941406250,
+0.0000915527343750, -0.0022583007812500, 0.0003662109375000, -0.0026245117187500,
+0.0004577636718750, -0.0012207031250000, 0.0005187988281250, -0.0015258789062500,
+0.0006713867187500, -0.0009155273437500, 0.0007324218750000, 0.0002441406250000,
+0.0007324218750000, -0.0004272460937500, 0.0009155273437500, 0.0013427734375000,
+0.0008850097656250, 0.0011291503906250, 0.0009460449218750, 0.0015563964843750,
+0.0010070800781250, 0.0026855468750000, 0.0009155273437500, 0.0019836425781250,
+0.0009460449218750, 0.0033569335937500, 0.0010070800781250, 0.0031127929687500,
+0.0008239746093750, 0.0033874511718750, 0.0009155273437500, 0.0043029785156250,
+0.0010681152343750, 0.0037231445312500, 0.0008850097656250, 0.0048828125000000,
+0.0011596679687500, 0.0047607421875000, 0.0010375976562500, 0.0050659179687500,
+0.0010681152343750, 0.0058593750000000, 0.0012817382812500, 0.0053405761718750,
+0.0010986328125000, 0.0064086914062500, 0.0011596679687500, 0.0063476562500000,
+0.0012207031250000, 0.0064086914062500, 0.0010986328125000, 0.0070800781250000,
+0.0010070800781250, 0.0067443847656250, 0.0010986328125000, 0.0071105957031250,
+0.0007934570312500, 0.0071716308593750, 0.0009155273437500, 0.0069885253906250,
+0.0008544921875000, 0.0070190429687500, 0.0005798339843750, 0.0070495605468750,
+0.0007324218750000, 0.0065917968750000, 0.0005493164062500, 0.0066528320312500,
+0.0005187988281250, 0.0065002441406250, 0.0005187988281250, 0.0061950683593750,
+0.0005187988281250, 0.0063781738281250, 0.0006103515625000, 0.0061645507812500,
+0.0004272460937500, 0.0061645507812500, 0.0005493164062500, 0.0061035156250000,
+0.0004272460937500, 0.0060119628906250, 0.0001831054687500, 0.0061950683593750,
+0.0004577636718750, 0.0058593750000000, 0.0001525878906250, 0.0060119628906250,
+0.0001220703125000, 0.0058898925781250, 0.0003051757812500, 0.0055541992187500,
+0.0002441406250000, 0.0057983398437500, 0.0001525878906250, 0.0054626464843750,
+0.0009765625000000, 0.0053710937500000, 0.0007629394531250, 0.0054321289062500,
+0.0010681152343750, 0.0053710937500000, 0.0018005371093750, 0.0053100585937500,
+0.0014343261718750, 0.0054321289062500, 0.0018005371093750, 0.0053710937500000,
+0.0021057128906250, 0.0053100585937500, 0.0018615722656250, 0.0053100585937500,
+0.0017700195312500, 0.0052795410156250, 0.0020141601562500, 0.0050964355468750,
+0.0014953613281250, 0.0051269531250000, 0.0014953613281250, 0.0049438476562500,
+0.0015869140625000, 0.0046997070312500, 0.0013732910156250, 0.0048217773437500,
+0.0013122558593750, 0.0046386718750000, 0.0016174316406250, 0.0045471191406250,
+0.0014343261718750, 0.0046081542968750, 0.0017395019531250, 0.0046386718750000,
+0.0020751953125000, 0.0045776367187500, 0.0017395019531250, 0.0046081542968750,
+0.0028076171875000, 0.0046691894531250, 0.0025634765625000, 0.0044250488281250,
+0.0028381347656250, 0.0043334960937500, 0.0036010742187500, 0.0043945312500000,
+0.0031127929687500, 0.0039062500000000, 0.0038452148437500, 0.0039367675781250,
+0.0039978027343750, 0.0039672851562500, 0.0038452148437500, 0.0037536621093750,
+0.0040893554687500, 0.0038452148437500, 0.0040588378906250, 0.0039367675781250,
+0.0038146972656250, 0.0040893554687500, 0.0039062500000000, 0.0041503906250000,
+0.0036926269531250, 0.0041809082031250, 0.0033569335937500, 0.0042724609375000,
+0.0035705566406250, 0.0042419433593750, 0.0029907226562500, 0.0042724609375000,
+0.0029602050781250, 0.0043640136718750, 0.0028991699218750, 0.0043334960937500,
+0.0026245117187500, 0.0043334960937500, 0.0028686523437500, 0.0049438476562500,
+0.0028381347656250, 0.0048828125000000, 0.0027770996093750, 0.0049133300781250,
+0.0028991699218750, 0.0055847167968750, 0.0029602050781250, 0.0057678222656250,
+0.0031738281250000, 0.0053100585937500, 0.0036315917968750, 0.0056762695312500,
+0.0033874511718750, 0.0055236816406250, 0.0037231445312500, 0.0049438476562500,
+0.0043029785156250, 0.0052185058593750, 0.0040588378906250, 0.0046997070312500,
+0.0050964355468750, 0.0046691894531250, 0.0051879882812500, 0.0043334960937500,
+0.0051574707031250, 0.0039062500000000, 0.0056457519531250, 0.0040588378906250,
+0.0057678222656250, 0.0030517578125000, 0.0052185058593750, 0.0031738281250000,
+0.0056762695312500, 0.0028991699218750, 0.0053710937500000, 0.0020141601562500,
+0.0047912597656250, 0.0023803710937500, 0.0052795410156250, 0.0023498535156250,
+0.0042724609375000, 0.0021057128906250, 0.0045166015625000, 0.0023193359375000,
+0.0042419433593750, 0.0025634765625000, 0.0035095214843750, 0.0023803710937500,
+0.0040283203125000, 0.0028076171875000, 0.0027160644531250, 0.0028381347656250,
+0.0030822753906250, 0.0028686523437500, 0.0026550292968750, 0.0031433105468750,
+0.0015563964843750, 0.0029602050781250, 0.0022888183593750, 0.0032653808593750,
+0.0011596679687500, 0.0032348632812500, 0.0012207031250000, 0.0032043457031250,
+0.0012512207031250, 0.0033264160156250, 0.0007934570312500, 0.0032043457031250,
+0.0011291503906250, 0.0031127929687500, 0.0006713867187500, 0.0031433105468750,
+0.0009765625000000, 0.0029602050781250, 0.0008850097656250, 0.0028686523437500,
+0.0004882812500000, 0.0028991699218750, 0.0009155273437500, 0.0027160644531250,
+0.0006408691406250, 0.0027465820312500, 0.0007019042968750, 0.0026855468750000,
+0.0009155273437500, 0.0025939941406250, 0.0009460449218750, 0.0026550292968750,
+0.0009765625000000, 0.0023498535156250, 0.0012817382812500, 0.0025024414062500,
+0.0014038085937500, 0.0023193359375000, 0.0013427734375000, 0.0020141601562500,
+0.0014343261718750, 0.0021667480468750, 0.0016479492187500, 0.0015258789062500,
+0.0014343261718750, 0.0017700195312500, 0.0016174316406250, 0.0014343261718750,
+0.0017700195312500, 0.0008239746093750, 0.0016479492187500, 0.0012512207031250,
+0.0018310546875000, 0.0004272460937500, 0.0021362304687500, 0.0004272460937500,
+0.0021972656250000, 0.0003967285156250, 0.0024719238281250, -0.0001220703125000,
+0.0028076171875000, 0.0000610351562500, 0.0028076171875000, 0.0001831054687500,
+0.0027160644531250, -0.0001525878906250, 0.0029907226562500, 0.0000915527343750,
+0.0028686523437500, 0.0004882812500000, 0.0025939941406250, 0.0000305175781250,
+0.0028991699218750, 0.0005187988281250, 0.0024414062500000, 0.0004272460937500,
+0.0024108886718750, 0.0003051757812500, 0.0025634765625000, 0.0004577636718750,
+0.0025024414062500, 0.0002136230468750, 0.0024719238281250, 0.0001525878906250,
+0.0028076171875000, 0.0000915527343750, 0.0027160644531250, -0.0000915527343750,
+0.0027770996093750, -0.0002441406250000, 0.0030212402343750, -0.0002746582031250,
+0.0029907226562500, -0.0005187988281250, 0.0028381347656250, -0.0005798339843750,
+0.0030517578125000, -0.0005493164062500, 0.0028381347656250, -0.0005493164062500,
+0.0024719238281250, -0.0006408691406250, 0.0026550292968750, -0.0003356933593750,
+0.0021667480468750, -0.0003967285156250, 0.0022277832031250, -0.0002136230468750,
+0.0021362304687500, 0.0000305175781250, 0.0018005371093750, 0.0000000000000000,
+0.0019226074218750, 0.0002746582031250, 0.0016174316406250, 0.0003967285156250,
+0.0015258789062500, 0.0004882812500000, 0.0015563964843750, 0.0007019042968750,
+0.0013732910156250, 0.0008239746093750, 0.0013427734375000, 0.0007629394531250,
+0.0015258789062500, 0.0009460449218750, 0.0012817382812500, 0.0009460449218750,
+0.0014953613281250, 0.0007934570312500, 0.0017395019531250, 0.0010681152343750,
+0.0014648437500000, 0.0007324218750000, 0.0024414062500000, 0.0008850097656250,
+0.0022583007812500, 0.0009155273437500, 0.0026855468750000, 0.0007019042968750,
+0.0035400390625000, 0.0008544921875000, 0.0031127929687500, 0.0007934570312500,
+0.0044555664062500, 0.0008544921875000, 0.0043945312500000, 0.0008544921875000,
+0.0048217773437500, 0.0007629394531250, 0.0058898925781250, 0.0008850097656250,
+0.0054626464843750, 0.0006408691406250, 0.0063476562500000, 0.0007629394531250,
+0.0065612792968750, 0.0007324218750000, 0.0065307617187500, 0.0005798339843750,
+0.0069274902343750, 0.0005798339843750, 0.0069580078125000, 0.0006408691406250,
+0.0065307617187500, 0.0004577636718750, 0.0068664550781250, 0.0005493164062500,
+0.0065612792968750, 0.0006408691406250, 0.0060729980468750, 0.0004577636718750,
+0.0065002441406250, 0.0008544921875000, 0.0054931640625000, 0.0007019042968750,
+0.0055541992187500, 0.0006713867187500, 0.0053405761718750, 0.0009460449218750,
+0.0046386718750000, 0.0008239746093750, 0.0049743652343750, 0.0008239746093750,
+0.0043945312500000, 0.0008544921875000, 0.0042724609375000, 0.0007934570312500,
+0.0043334960937500, 0.0007324218750000, 0.0041198730468750, 0.0008544921875000,
+0.0041809082031250, 0.0007934570312500, 0.0045166015625000, 0.0008239746093750,
+0.0043029785156250, 0.0010681152343750, 0.0044860839843750, 0.0012207031250000,
+0.0049743652343750, 0.0011901855468750, 0.0046386718750000, 0.0016174316406250,
+0.0049133300781250, 0.0017395019531250, 0.0048828125000000, 0.0018310546875000,
+0.0046691894531250, 0.0021667480468750, 0.0046691894531250, 0.0022888183593750,
+0.0046386718750000, 0.0019226074218750, 0.0042114257812500, 0.0022888183593750,
+0.0040893554687500, 0.0019836425781250, 0.0040893554687500, 0.0013732910156250,
+0.0038146972656250},
+{0.0064086914062500, 0.0003967285156250, 0.0059509277343750, 0.0005493164062500,
+0.0063476562500000, 0.0000305175781250, 0.0057067871093750, 0.0000000000000000,
+0.0057983398437500, 0.0000305175781250, 0.0059814453125000, -0.0003356933593750,
+0.0057983398437500, -0.0003662109375000, 0.0059814453125000, -0.0002136230468750,
+0.0061340332031250, -0.0003051757812500, 0.0061950683593750, -0.0002746582031250,
+0.0063171386718750, -0.0000610351562500, 0.0065612792968750, -0.0000915527343750,
+0.0065917968750000, -0.0001525878906250, 0.0066528320312500, 0.0001220703125000,
+0.0067749023437500, 0.0000305175781250, 0.0067138671875000, -0.0001525878906250,
+0.0066223144531250, 0.0001525878906250, 0.0067749023437500, -0.0002746582031250,
+0.0065307617187500, -0.0001220703125000, 0.0065307617187500, -0.0000610351562500,
+0.0065612792968750, -0.0003356933593750, 0.0063781738281250, -0.0002136230468750,
+0.0064697265625000, -0.0002136230468750, 0.0060424804687500, -0.0003662109375000,
+0.0062255859375000, -0.0002746582031250, 0.0057373046875000, -0.0002441406250000,
+0.0051269531250000, -0.0003967285156250, 0.0054016113281250, -0.0001831054687500,
+0.0042114257812500, -0.0002441406250000, 0.0042114257812500, -0.0002136230468750,
+0.0039062500000000, -0.0000610351562500, 0.0030212402343750, -0.0001220703125000,
+0.0033264160156250, 0.0001220703125000, 0.0026550292968750, 0.0001220703125000,
+0.0025634765625000, 0.0002441406250000, 0.0025329589843750, 0.0004272460937500,
+0.0021972656250000, 0.0003356933593750, 0.0023193359375000, 0.0007324218750000,
+0.0023498535156250, 0.0007324218750000, 0.0021667480468750, 0.0007934570312500,
+0.0023498535156250, 0.0010375976562500, 0.0025329589843750, 0.0008544921875000,
+0.0022888183593750, 0.0011291503906250, 0.0029907226562500, 0.0010681152343750,
+0.0027465820312500, 0.0011596679687500, 0.0030212402343750, 0.0013427734375000,
+0.0035705566406250, 0.0010986328125000, 0.0032653808593750, 0.0014038085937500,
+0.0040588378906250, 0.0013122558593750, 0.0041198730468750, 0.0012817382812500,
+0.0041809082031250, 0.0014953613281250, 0.0046386718750000, 0.0013122558593750,
+0.0046081542968750, 0.0013122558593750, 0.0046691894531250, 0.0013732910156250,
+0.0049133300781250, 0.0011901855468750, 0.0047912597656250, 0.0010375976562500,
+0.0046691894531250, 0.0012207031250000, 0.0050354003906250, 0.0007019042968750,
+0.0042419433593750, 0.0008544921875000, 0.0046081542968750, 0.0007324218750000,
+0.0041809082031250, 0.0003356933593750, 0.0034179687500000, 0.0006103515625000,
+0.0039978027343750, 0.0004882812500000, 0.0026855468750000, 0.0004272460937500,
+0.0028381347656250, 0.0006103515625000, 0.0025634765625000, 0.0006713867187500,
+0.0016784667968750, 0.0006408691406250, 0.0020141601562500, 0.0009460449218750,
+0.0013732910156250, 0.0010070800781250, 0.0011291503906250, 0.0010986328125000,
+0.0013427734375000, 0.0013732910156250, 0.0010681152343750, 0.0014343261718750,
+0.0007934570312500, 0.0014648437500000, 0.0015869140625000, 0.0016784667968750,
+0.0011901855468750, 0.0015869140625000, 0.0014038085937500, 0.0014953613281250,
+0.0021362304687500, 0.0016784667968750, 0.0015258789062500, 0.0014953613281250,
+0.0021057128906250, 0.0014038085937500, 0.0021667480468750, 0.0016784667968750,
+0.0018310546875000, 0.0017089843750000, 0.0019226074218750, 0.0014953613281250,
+0.0018920898437500, 0.0021362304687500, 0.0014343261718750, 0.0021057128906250,
+0.0015869140625000, 0.0021057128906250, 0.0013427734375000, 0.0025634765625000,
+0.0009460449218750, 0.0025329589843750, 0.0012512207031250, 0.0023498535156250,
+0.0009460449218750, 0.0026855468750000, 0.0008850097656250, 0.0024719238281250,
+0.0010986328125000, 0.0021057128906250, 0.0011596679687500, 0.0025939941406250,
+0.0010986328125000, 0.0018920898437500, 0.0014953613281250, 0.0020141601562500,
+0.0013427734375000, 0.0020751953125000, 0.0015258789062500, 0.0016479492187500,
+0.0019226074218750, 0.0020446777343750, 0.0016784667968750, 0.0020141601562500,
+0.0025329589843750, 0.0020446777343750, 0.0024719238281250, 0.0021972656250000,
+0.0027465820312500, 0.0022277832031250, 0.0032958984375000, 0.0023803710937500,
+0.0030822753906250, 0.0024108886718750, 0.0037536621093750, 0.0025329589843750,
+0.0035705566406250, 0.0025024414062500, 0.0040588378906250, 0.0023498535156250,
+0.0046997070312500, 0.0025329589843750, 0.0040588378906250, 0.0027160644531250,
+0.0055236816406250, 0.0026855468750000, 0.0052185058593750, 0.0029907226562500,
+0.0054626464843750, 0.0033569335937500, 0.0066528320312500, 0.0033264160156250,
+0.0060729980468750, 0.0036010742187500, 0.0064086914062500, 0.0039062500000000,
+0.0067749023437500, 0.0033874511718750, 0.0061035156250000, 0.0032348632812500,
+0.0057067871093750, 0.0037841796875000, 0.0059509277343750, 0.0020141601562500,
+0.0044250488281250, 0.0025024414062500, 0.0046386718750000, 0.0021057128906250,
+0.0040588378906250, 0.0007019042968750, 0.0028686523437500, 0.0015869140625000,
+0.0034790039062500, 0.0007934570312500, 0.0018310546875000, 0.0003967285156250,
+0.0018615722656250, 0.0009460449218750, 0.0019836425781250, 0.0015563964843750,
+0.0006713867187500, 0.0034484863281250, 0.0064086914062500, 0.0048217773437500,
+0.0032348632812500, 0.0044860839843750, 0.0043029785156250, 0.0049133300781250,
+0.0035705566406250, 0.0057373046875000, 0.0017700195312500, 0.0054321289062500,
+0.0025939941406250, 0.0058593750000000, 0.0024414062500000, 0.0061950683593750,
+0.0020446777343750, 0.0055847167968750, 0.0025024414062500, 0.0053710937500000,
+0.0030822753906250, 0.0058593750000000, 0.0027770996093750, 0.0038452148437500,
+0.0031433105468750, 0.0044250488281250, 0.0032348632812500, 0.0037536621093750,
+0.0032653808593750, 0.0020446777343750, 0.0034179687500000, 0.0030517578125000,
+0.0034179687500000, 0.0014343261718750, 0.0039062500000000, 0.0013732910156250,
+0.0037536621093750, 0.0013427734375000, 0.0039978027343750, 0.0005798339843750,
+0.0045166015625000, 0.0008850097656250, 0.0043029785156250, 0.0002441406250000,
+0.0040893554687500, 0.0002441406250000, 0.0043334960937500, 0.0003356933593750,
+0.0038452148437500, -0.0000610351562500, 0.0031433105468750, -0.0000305175781250,
+0.0034484863281250, 0.0007019042968750, 0.0027770996093750, 0.0002441406250000,
+0.0026245117187500, 0.0008544921875000, 0.0028076171875000, 0.0018310546875000,
+0.0027465820312500, 0.0010375976562500, 0.0027770996093750, 0.0027770996093750,
+0.0029907226562500, 0.0025939941406250, 0.0031127929687500, 0.0026855468750000,
+0.0032348632812500, 0.0037841796875000, 0.0033569335937500, 0.0033264160156250,
+0.0033569335937500, 0.0034484863281250, 0.0033264160156250, 0.0036621093750000,
+0.0034484863281250, 0.0034179687500000, 0.0033569335937500, 0.0031738281250000,
+0.0032348632812500, 0.0034179687500000, 0.0034179687500000, 0.0031738281250000,
+0.0028381347656250, 0.0030822753906250, 0.0030517578125000, 0.0035705566406250,
+0.0028686523437500, 0.0037231445312500, 0.0023193359375000, 0.0033569335937500,
+0.0027160644531250, 0.0047912597656250, 0.0021057128906250, 0.0043945312500000,
+0.0022888183593750, 0.0048828125000000, 0.0022888183593750, 0.0060424804687500,
+0.0018615722656250, 0.0052490234375000, 0.0022583007812500, 0.0066223144531250,
+0.0015869140625000, 0.0065917968750000, 0.0017395019531250, 0.0065307617187500,
+0.0015563964843750, 0.0071411132812500, 0.0010375976562500, 0.0068664550781250,
+0.0012512207031250, 0.0071411132812500, 0.0007629394531250, 0.0072937011718750,
+0.0005493164062500, 0.0071716308593750, 0.0006408691406250, 0.0071716308593750,
+0.0004577636718750, 0.0072937011718750, 0.0002746582031250, 0.0070800781250000,
+0.0004272460937500, 0.0072021484375000, 0.0001525878906250, 0.0070800781250000,
+0.0001831054687500, 0.0068969726562500, 0.0002746582031250, 0.0070190429687500,
+-0.0001525878906250, 0.0065612792968750, 0.0002441406250000, 0.0066528320312500,
+0.0000000000000000, 0.0064392089843750, -0.0001220703125000, 0.0060119628906250,
+0.0001831054687500, 0.0062255859375000, -0.0001525878906250, 0.0059509277343750,
+0.0000915527343750, 0.0057678222656250, -0.0000915527343750, 0.0061035156250000,
+-0.0000305175781250, 0.0060424804687500, 0.0001525878906250, 0.0057983398437500,
+-0.0001525878906250, 0.0070190429687500, 0.0001831054687500, 0.0064392089843750,
+0.0001525878906250, 0.0071716308593750, 0.0000915527343750, 0.0083618164062500,
+0.0002746582031250, 0.0074157714843750, 0.0001220703125000, 0.0093383789062500,
+0.0002441406250000, 0.0092773437500000, 0.0002441406250000, 0.0092773437500000,
+0.0002746582031250, 0.0103759765625000, 0.0003967285156250, 0.0100097656250000,
+0.0003967285156250, 0.0097961425781250, 0.0004882812500000, 0.0102233886718750,
+0.0004882812500000, 0.0095214843750000, 0.0004882812500000, 0.0089416503906250,
+0.0005493164062500, 0.0094604492187500, 0.0004882812500000, 0.0076904296875000,
+0.0005187988281250, 0.0079650878906250, 0.0004577636718750, 0.0072326660156250,
+0.0003662109375000, 0.0057373046875000, 0.0003662109375000, 0.0064697265625000,
+0.0003356933593750, 0.0043945312500000, 0.0003051757812500, 0.0045166015625000,
+0.0003051757812500, 0.0039672851562500, 0.0003356933593750, 0.0025024414062500,
+0.0004272460937500, 0.0032043457031250, 0.0004882812500000, 0.0018005371093750,
+0.0006103515625000, 0.0017395019531250, 0.0005798339843750, 0.0016479492187500,
+0.0006713867187500, 0.0009155273437500, 0.0007934570312500, 0.0011291503906250,
+0.0006713867187500, 0.0008850097656250, 0.0007934570312500, 0.0007324218750000,
+0.0007324218750000, 0.0009460449218750, 0.0007019042968750, 0.0010070800781250,
+0.0007629394531250, 0.0008850097656250, 0.0006103515625000, 0.0014953613281250,
+0.0007629394531250, 0.0012512207031250, 0.0006408691406250, 0.0015869140625000,
+0.0006408691406250, 0.0021057128906250, 0.0007324218750000, 0.0017089843750000,
+0.0004882812500000, 0.0026245117187500, 0.0006408691406250, 0.0025024414062500,
+0.0005493164062500, 0.0027160644531250, 0.0004882812500000, 0.0033874511718750,
+0.0005187988281250, 0.0030212402343750, 0.0003662109375000, 0.0037841796875000,
+0.0003662109375000, 0.0037536621093750, 0.0003662109375000, 0.0037841796875000,
+0.0004882812500000, 0.0042724609375000, 0.0004272460937500, 0.0040283203125000,
+0.0002746582031250, 0.0040588378906250, 0.0008850097656250, 0.0042724609375000,
+0.0006103515625000, 0.0039672851562500, 0.0008239746093750, 0.0037536621093750,
+0.0014343261718750, 0.0039672851562500, 0.0010070800781250, 0.0032958984375000,
+0.0014648437500000, 0.0033874511718750, 0.0016174316406250, 0.0032348632812500,
+0.0013122558593750, 0.0027465820312500, 0.0013732910156250, 0.0029907226562500,
+0.0014648437500000, 0.0027770996093750, 0.0009460449218750, 0.0027160644531250,
+0.0010070800781250, 0.0028991699218750, 0.0009765625000000, 0.0029602050781250,
+0.0005493164062500, 0.0030212402343750, 0.0006408691406250, 0.0031738281250000,
+0.0007934570312500, 0.0032958984375000, 0.0005493164062500, 0.0032958984375000,
+0.0007629394531250, 0.0033874511718750, 0.0010375976562500, 0.0034790039062500,
+0.0007324218750000, 0.0033874511718750, 0.0014038085937500, 0.0034484863281250,
+0.0012207031250000, 0.0034790039062500, 0.0015258789062500, 0.0033264160156250,
+0.0021667480468750, 0.0033874511718750, 0.0017395019531250, 0.0034179687500000,
+0.0026550292968750},
+{0.0011596679687500, 0.0055236816406250, 0.0011596679687500, 0.0049743652343750,
+0.0014038085937500, 0.0057373046875000, 0.0012512207031250, 0.0074157714843750,
+0.0011596679687500, 0.0063476562500000, 0.0014648437500000, 0.0083618164062500,
+0.0009155273437500, 0.0085449218750000, 0.0010986328125000, 0.0083923339843750,
+0.0010375976562500, 0.0093994140625000, 0.0006713867187500, 0.0093383789062500,
+0.0009155273437500, 0.0087585449218750, 0.0006103515625000, 0.0094909667968750,
+0.0006103515625000, 0.0089721679687500, 0.0005493164062500, 0.0080566406250000,
+0.0003967285156250, 0.0089111328125000, 0.0005493164062500, 0.0075683593750000,
+0.0003967285156250, 0.0078430175781250, 0.0005187988281250, 0.0077209472656250,
+0.0004272460937500, 0.0069274902343750, 0.0003051757812500, 0.0075683593750000,
+0.0006103515625000, 0.0068359375000000, 0.0002136230468750, 0.0068359375000000,
+0.0004882812500000, 0.0068969726562500, 0.0004882812500000, 0.0065917968750000,
+0.0001831054687500, 0.0067443847656250, 0.0006408691406250, 0.0064392089843750,
+0.0003051757812500, 0.0064697265625000, 0.0004577636718750, 0.0063171386718750,
+0.0005187988281250, 0.0061340332031250, 0.0003356933593750, 0.0062866210937500,
+0.0005493164062500, 0.0055847167968750, 0.0003967285156250, 0.0057678222656250,
+0.0005493164062500, 0.0053405761718750, 0.0006103515625000, 0.0046691894531250,
+0.0005187988281250, 0.0050659179687500, 0.0006408691406250, 0.0039062500000000,
+0.0006103515625000, 0.0040588378906250, 0.0006713867187500, 0.0037231445312500,
+0.0007629394531250, 0.0028991699218750, 0.0008544921875000, 0.0032348632812500,
+0.0007934570312500, 0.0023498535156250, 0.0009155273437500, 0.0023498535156250,
+0.0009460449218750, 0.0022277832031250, 0.0009460449218750, 0.0016174316406250,
+0.0009460449218750, 0.0018005371093750, 0.0008850097656250, 0.0015258789062500,
+0.0008850097656250, 0.0013427734375000, 0.0007934570312500, 0.0015563964843750,
+0.0008239746093750, 0.0015563964843750, 0.0008239746093750, 0.0014038085937500,
+0.0006713867187500, 0.0020141601562500, 0.0010986328125000, 0.0018920898437500,
+0.0009155273437500, 0.0021667480468750, 0.0010681152343750, 0.0026245117187500,
+0.0015258789062500, 0.0022888183593750, 0.0011901855468750, 0.0031738281250000,
+0.0015563964843750, 0.0030822753906250, 0.0017395019531250, 0.0031127929687500,
+0.0014648437500000, 0.0036926269531250, 0.0014343261718750, 0.0035095214843750,
+0.0015869140625000, 0.0036315917968750, 0.0010986328125000, 0.0038757324218750,
+0.0011596679687500, 0.0036010742187500, 0.0011291503906250, 0.0034790039062500,
+0.0007934570312500, 0.0037536621093750, 0.0008850097656250, 0.0029602050781250,
+0.0010070800781250, 0.0032348632812500, 0.0007934570312500, 0.0031433105468750,
+0.0009765625000000, 0.0026245117187500, 0.0011901855468750, 0.0030822753906250,
+0.0009155273437500, 0.0029602050781250, 0.0015258789062500, 0.0028991699218750,
+0.0013427734375000, 0.0032653808593750, 0.0015563964843750, 0.0035095214843750,
+0.0021057128906250, 0.0033874511718750, 0.0017700195312500, 0.0041809082031250,
+0.0024719238281250, 0.0040893554687500, 0.0024719238281250, 0.0042419433593750,
+0.0025634765625000, 0.0047912597656250, 0.0030212402343750, 0.0045776367187500,
+0.0029602050781250, 0.0048522949218750, 0.0029907226562500, 0.0048828125000000,
+0.0030822753906250, 0.0048522949218750, 0.0030822753906250, 0.0050048828125000,
+0.0030517578125000, 0.0048828125000000, 0.0031127929687500, 0.0048217773437500,
+0.0030212402343750, 0.0049133300781250, 0.0031127929687500, 0.0045471191406250,
+0.0029602050781250, 0.0042419433593750, 0.0027465820312500, 0.0044555664062500,
+0.0029907226562500, 0.0035705566406250, 0.0025024414062500, 0.0036010742187500,
+0.0024719238281250, 0.0033874511718750, 0.0026245117187500, 0.0027160644531250,
+0.0024108886718750, 0.0029296875000000, 0.0023193359375000, 0.0022888183593750,
+0.0028686523437500, 0.0021362304687500, 0.0025939941406250, 0.0022583007812500,
+0.0029907226562500, 0.0020141601562500, 0.0037231445312500, 0.0018615722656250,
+0.0033569335937500, 0.0023193359375000, 0.0041198730468750, 0.0021667480468750,
+0.0042724609375000, 0.0022888183593750, 0.0040283203125000, 0.0025329589843750,
+0.0043029785156250, 0.0022888183593750, 0.0043640136718750, 0.0031127929687500,
+0.0037231445312500, 0.0027465820312500, 0.0038757324218750, 0.0031433105468750,
+0.0036315917968750, 0.0044250488281250, 0.0029602050781250, 0.0045776367187500,
+0.0031433105468750, 0.0056762695312500, 0.0028381347656250, 0.0054321289062500,
+0.0028076171875000, 0.0053710937500000, 0.0027770996093750, 0.0061035156250000,
+0.0026855468750000, 0.0058288574218750, 0.0028686523437500, 0.0054626464843750,
+0.0025939941406250, 0.0057373046875000, 0.0026245117187500, 0.0053405761718750,
+0.0026245117187500, 0.0047912597656250, 0.0023803710937500, 0.0052490234375000,
+0.0024108886718750, 0.0043029785156250, 0.0026550292968750, 0.0045776367187500,
+0.0023193359375000, 0.0043334960937500, 0.0026550292968750, 0.0036010742187500,
+0.0031127929687500, 0.0041503906250000, 0.0028076171875000, 0.0036315917968750,
+0.0039978027343750, 0.0035095214843750, 0.0037536621093750, 0.0037536621093750,
+0.0042114257812500, 0.0037231445312500, 0.0052185058593750, 0.0036926269531250,
+0.0045471191406250, 0.0039367675781250, 0.0059509277343750, 0.0040283203125000,
+0.0058898925781250, 0.0039062500000000, 0.0060424804687500, 0.0038757324218750,
+0.0068359375000000, 0.0039672851562500, 0.0064392089843750, 0.0034484863281250,
+0.0065917968750000, 0.0036315917968750, 0.0066833496093750, 0.0033569335937500,
+0.0064392089843750, 0.0028991699218750, 0.0062561035156250, 0.0032958984375000,
+0.0063781738281250, 0.0025024414062500, 0.0062255859375000, 0.0026855468750000,
+0.0059814453125000, 0.0025939941406250, 0.0062255859375000, 0.0021057128906250,
+0.0064086914062500, 0.0024108886718750, 0.0061950683593750, 0.0021667480468750,
+0.0066223144531250, 0.0020446777343750, 0.0067443847656250, 0.0022583007812500,
+0.0065917968750000, 0.0024108886718750, 0.0067443847656250, 0.0024108886718750,
+0.0068969726562500, 0.0026245117187500, 0.0060729980468750, 0.0028076171875000,
+0.0063476562500000, 0.0027465820312500, 0.0059204101562500, 0.0027465820312500,
+0.0050048828125000, 0.0030517578125000, 0.0054321289062500, 0.0024719238281250,
+0.0046386718750000, 0.0027160644531250, 0.0044860839843750, 0.0026550292968750,
+0.0045776367187500, 0.0021362304687500, 0.0043029785156250, 0.0025024414062500,
+0.0043029785156250, 0.0021057128906250, 0.0043029785156250, 0.0021362304687500,
+0.0043029785156250, 0.0021667480468750, 0.0041503906250000, 0.0019836425781250,
+0.0040893554687500, 0.0021057128906250, 0.0042114257812500, 0.0019836425781250,
+0.0039672851562500, 0.0019226074218750, 0.0038757324218750, 0.0019226074218750,
+0.0039672851562500, 0.0018920898437500, 0.0038757324218750, 0.0018310546875000,
+0.0038757324218750, 0.0015869140625000, 0.0039978027343750, 0.0017089843750000,
+0.0039672851562500, 0.0013732910156250, 0.0040283203125000, 0.0010070800781250,
+0.0041198730468750, 0.0010986328125000, 0.0041503906250000, 0.0005798339843750,
+0.0041809082031250, 0.0003967285156250, 0.0042724609375000, 0.0004272460937500,
+0.0043945312500000, 0.0001220703125000, 0.0044250488281250, 0.0000000000000000,
+0.0045166015625000, 0.0003051757812500, 0.0048828125000000, 0.0000915527343750,
+0.0046997070312500, 0.0001831054687500, 0.0050964355468750, 0.0004882812500000,
+0.0055541992187500, 0.0002746582031250, 0.0053100585937500, 0.0006103515625000,
+0.0063171386718750, 0.0006713867187500, 0.0062866210937500, 0.0006713867187500,
+0.0064392089843750, 0.0007629394531250, 0.0070800781250000, 0.0007934570312500,
+0.0068969726562500, 0.0007019042968750, 0.0068359375000000, 0.0007019042968750,
+0.0070190429687500, 0.0007629394531250, 0.0066528320312500, 0.0007324218750000,
+0.0063781738281250, 0.0006408691406250, 0.0065002441406250, 0.0007629394531250,
+0.0057983398437500, 0.0006713867187500, 0.0056152343750000, 0.0007019042968750,
+0.0054321289062500, 0.0007629394531250, 0.0050048828125000, 0.0006103515625000,
+0.0050964355468750, 0.0007934570312500, 0.0047607421875000, 0.0007019042968750,
+0.0047912597656250, 0.0007324218750000, 0.0047302246093750, 0.0008544921875000,
+0.0046081542968750, 0.0007019042968750, 0.0047912597656250, 0.0008544921875000,
+0.0045166015625000, 0.0007934570312500, 0.0047607421875000, 0.0007934570312500,
+0.0046691894531250, 0.0008239746093750, 0.0043945312500000, 0.0007629394531250,
+0.0046691894531250, 0.0007629394531250, 0.0040283203125000, 0.0007324218750000,
+0.0041503906250000, 0.0007324218750000, 0.0039367675781250, 0.0007019042968750,
+0.0033874511718750, 0.0007019042968750, 0.0037841796875000, 0.0007934570312500,
+0.0032043457031250, 0.0007019042968750, 0.0031127929687500, 0.0008239746093750,
+0.0033264160156250, 0.0010375976562500, 0.0031738281250000, 0.0009155273437500,
+0.0031738281250000, 0.0011596679687500, 0.0036621093750000, 0.0012207031250000,
+0.0034484863281250, 0.0010986328125000, 0.0038452148437500, 0.0012207031250000,
+0.0043945312500000, 0.0012817382812500, 0.0040283203125000, 0.0011291503906250,
+0.0050964355468750, 0.0012512207031250, 0.0050048828125000, 0.0012817382812500,
+0.0054016113281250, 0.0011291503906250, 0.0062255859375000, 0.0013427734375000,
+0.0059509277343750, 0.0014343261718750, 0.0068359375000000, 0.0013732910156250,
+0.0068969726562500, 0.0016174316406250, 0.0069580078125000, 0.0018310546875000,
+0.0075378417968750, 0.0017089843750000, 0.0073547363281250, 0.0022583007812500,
+0.0071411132812500, 0.0022277832031250, 0.0074768066406250, 0.0023498535156250,
+0.0070495605468750, 0.0026855468750000, 0.0065307617187500, 0.0025939941406250,
+0.0070190429687500, 0.0026245117187500, 0.0057678222656250, 0.0026550292968750,
+0.0058593750000000, 0.0026245117187500, 0.0057067871093750, 0.0025634765625000,
+0.0049438476562500, 0.0025634765625000, 0.0053100585937500, 0.0025939941406250,
+0.0046997070312500, 0.0025634765625000, 0.0046691894531250, 0.0025939941406250,
+0.0046386718750000, 0.0026245117187500, 0.0043640136718750, 0.0026550292968750,
+0.0044555664062500, 0.0026550292968750, 0.0041503906250000, 0.0027465820312500,
+0.0043029785156250, 0.0026245117187500, 0.0041198730468750, 0.0025329589843750,
+0.0037841796875000, 0.0026550292968750, 0.0040893554687500, 0.0023803710937500,
+0.0033264160156250, 0.0024414062500000, 0.0034484863281250, 0.0025024414062500,
+0.0033874511718750, 0.0023193359375000, 0.0029296875000000, 0.0023193359375000,
+0.0031738281250000, 0.0026550292968750, 0.0028686523437500, 0.0024719238281250,
+0.0027770996093750, 0.0025939941406250, 0.0027465820312500, 0.0029602050781250,
+0.0025329589843750, 0.0027465820312500, 0.0024719238281250, 0.0028076171875000,
+0.0026245117187500, 0.0030517578125000, 0.0024414062500000, 0.0026550292968750,
+0.0024108886718750, 0.0024414062500000, 0.0025939941406250, 0.0027770996093750,
+0.0025024414062500, 0.0015869140625000, 0.0023193359375000, 0.0017395019531250,
+0.0025024414062500, 0.0016479492187500, 0.0021362304687500, 0.0008239746093750,
+0.0017395019531250, 0.0010986328125000, 0.0020446777343750, 0.0013732910156250,
+0.0008239746093750, 0.0009460449218750, 0.0008850097656250, 0.0015563964843750,
+0.0008850097656250},
+{0.0032958984375000, 0.0011901855468750, 0.0038452148437500, 0.0011901855468750,
+0.0050354003906250, 0.0011291503906250, 0.0043640136718750, 0.0012207031250000,
+0.0057373046875000, 0.0012207031250000, 0.0057373046875000, 0.0011596679687500,
+0.0057373046875000, 0.0011901855468750, 0.0064086914062500, 0.0011291503906250,
+0.0059814453125000, 0.0010375976562500, 0.0062866210937500, 0.0010681152343750,
+0.0062561035156250, 0.0010070800781250, 0.0059509277343750, 0.0009155273437500,
+0.0059814453125000, 0.0010070800781250, 0.0059814453125000, 0.0010070800781250,
+0.0055236816406250, 0.0010070800781250, 0.0055847167968750, 0.0011291503906250,
+0.0053710937500000, 0.0012512207031250, 0.0049743652343750, 0.0012512207031250,
+0.0052185058593750, 0.0013427734375000, 0.0046386718750000, 0.0014953613281250,
+0.0046386718750000, 0.0013732910156250, 0.0046081542968750, 0.0013122558593750,
+0.0042419433593750, 0.0014648437500000, 0.0043334960937500, 0.0010986328125000,
+0.0044555664062500, 0.0012207031250000, 0.0043029785156250, 0.0012512207031250,
+0.0045776367187500, 0.0010070800781250, 0.0049438476562500, 0.0012817382812500,
+0.0046691894531250, 0.0014038085937500, 0.0054626464843750, 0.0013122558593750,
+0.0053710937500000, 0.0016479492187500, 0.0056152343750000, 0.0019226074218750,
+0.0061645507812500, 0.0017395019531250, 0.0059509277343750, 0.0023803710937500,
+0.0065307617187500, 0.0023803710937500, 0.0066223144531250, 0.0023803710937500,
+0.0068664550781250, 0.0026855468750000, 0.0072631835937500, 0.0024719238281250,
+0.0070495605468750, 0.0024414062500000, 0.0076599121093750, 0.0024108886718750,
+0.0076904296875000, 0.0024108886718750, 0.0077514648437500, 0.0022888183593750,
+0.0081481933593750, 0.0021972656250000, 0.0081176757812500, 0.0024414062500000,
+0.0079040527343750, 0.0022583007812500, 0.0081481933593750, 0.0024414062500000,
+0.0077819824218750, 0.0027465820312500, 0.0073242187500000, 0.0025634765625000,
+0.0077209472656250, 0.0029602050781250, 0.0067443847656250, 0.0030517578125000,
+0.0069274902343750, 0.0030212402343750, 0.0066528320312500, 0.0031433105468750,
+0.0060424804687500, 0.0031127929687500, 0.0064697265625000, 0.0030822753906250,
+0.0058288574218750, 0.0031127929687500, 0.0059814453125000, 0.0030822753906250,
+0.0059204101562500, 0.0029602050781250, 0.0055236816406250, 0.0028686523437500,
+0.0058288574218750, 0.0030822753906250, 0.0054016113281250, 0.0029602050781250,
+0.0054931640625000, 0.0030212402343750, 0.0053405761718750, 0.0032348632812500,
+0.0049133300781250, 0.0030517578125000, 0.0051574707031250, 0.0029907226562500,
+0.0047912597656250, 0.0031738281250000, 0.0047607421875000, 0.0027465820312500,
+0.0046997070312500, 0.0024108886718750, 0.0044555664062500, 0.0026855468750000,
+0.0044555664062500, 0.0015563964843750, 0.0042724609375000, 0.0017089843750000,
+0.0041809082031250, 0.0016784667968750, 0.0042114257812500, 0.0010070800781250,
+0.0041809082031250, 0.0013122558593750, 0.0040283203125000, 0.0015258789062500,
+0.0043640136718750, 0.0010681152343750, 0.0042419433593750, 0.0016174316406250,
+0.0043334960937500, 0.0024108886718750, 0.0046997070312500, 0.0017700195312500,
+0.0045471191406250, 0.0032043457031250, 0.0046386718750000, 0.0031127929687500,
+0.0050354003906250, 0.0033569335937500, 0.0042114257812500, 0.0043640136718750,
+0.0036926269531250, 0.0039672851562500, 0.0043945312500000, 0.0047302246093750,
+0.0018310546875000, 0.0049133300781250, 0.0024414062500000, 0.0048522949218750,
+0.0020446777343750, 0.0053100585937500, 0.0001831054687500, 0.0054931640625000,
+0.0012207031250000, 0.0046691894531250, 0.0006408691406250, 0.0052490234375000,
+0.0003051757812500, 0.0048522949218750, 0.0009155273437500, 0.0030517578125000,
+0.0015563964843750, 0.0018005371093750, 0.0030212402343750, 0.0028686523437500,
+0.0022583007812500, 0.0021667480468750, 0.0022583007812500, 0.0025939941406250,
+0.0024414062500000, 0.0035705566406250, 0.0024719238281250, 0.0031738281250000,
+0.0024719238281250, 0.0033569335937500, 0.0029296875000000, 0.0037536621093750,
+0.0026855468750000, 0.0035095214843750, 0.0030517578125000, 0.0031738281250000,
+0.0036315917968750, 0.0035705566406250, 0.0032653808593750, 0.0034790039062500,
+0.0042419433593750, 0.0033569335937500, 0.0043029785156250, 0.0038757324218750,
+0.0043029785156250, 0.0042114257812500, 0.0047912597656250, 0.0040283203125000,
+0.0046997070312500, 0.0046997070312500, 0.0048217773437500, 0.0047302246093750,
+0.0047302246093750, 0.0046081542968750, 0.0048828125000000, 0.0047912597656250,
+0.0050964355468750, 0.0046997070312500, 0.0048828125000000, 0.0049133300781250,
+0.0052185058593750, 0.0046691894531250, 0.0053710937500000, 0.0049133300781250,
+0.0050659179687500, 0.0052795410156250, 0.0051269531250000, 0.0049133300781250,
+0.0054321289062500, 0.0054626464843750, 0.0042724609375000, 0.0055847167968750,
+0.0045166015625000, 0.0052795410156250, 0.0042724609375000, 0.0054321289062500,
+0.0032348632812500, 0.0055236816406250, 0.0036315917968750, 0.0047607421875000,
+0.0033569335937500, 0.0050354003906250, 0.0029296875000000, 0.0045471191406250,
+0.0034179687500000, 0.0037231445312500, 0.0037231445312500, 0.0041809082031250,
+0.0032348632812500, 0.0031433105468750, 0.0040283203125000, 0.0031127929687500,
+0.0039367675781250, 0.0029602050781250, 0.0039062500000000, 0.0023803710937500,
+0.0042724609375000, 0.0027160644531250, 0.0039978027343750, 0.0020446777343750,
+0.0042114257812500, 0.0020751953125000, 0.0042724609375000, 0.0020141601562500,
+0.0041198730468750, 0.0016479492187500, 0.0041503906250000, 0.0018615722656250,
+0.0042114257812500, 0.0012817382812500, 0.0037231445312500, 0.0013732910156250,
+0.0038452148437500, 0.0012817382812500, 0.0036010742187500, 0.0007934570312500,
+0.0030822753906250, 0.0010375976562500, 0.0032958984375000, 0.0008850097656250,
+0.0028686523437500, 0.0006713867187500, 0.0028686523437500, 0.0011291503906250,
+0.0029296875000000, 0.0012512207031250, 0.0028381347656250, 0.0009460449218750,
+0.0029602050781250, 0.0022583007812500, 0.0028686523437500, 0.0018005371093750,
+0.0028991699218750, 0.0024108886718750, 0.0029296875000000, 0.0036315917968750,
+0.0028686523437500, 0.0027770996093750, 0.0028686523437500, 0.0043334960937500,
+0.0028381347656250, 0.0045166015625000, 0.0028686523437500, 0.0040893554687500,
+0.0027160644531250, 0.0046081542968750, 0.0026245117187500, 0.0047607421875000,
+0.0027465820312500, 0.0035400390625000, 0.0023193359375000, 0.0040283203125000,
+0.0023803710937500, 0.0034484863281250, 0.0022583007812500, 0.0022583007812500,
+0.0019226074218750, 0.0029602050781250, 0.0020141601562500, 0.0015563964843750,
+0.0017700195312500, 0.0015563964843750, 0.0016784667968750, 0.0014648437500000,
+0.0017700195312500, 0.0006408691406250, 0.0017395019531250, 0.0008850097656250,
+0.0017089843750000, 0.0007019042968750, 0.0018310546875000, 0.0003356933593750,
+0.0019226074218750, 0.0005187988281250, 0.0019836425781250, 0.0007324218750000,
+0.0021057128906250, 0.0005187988281250, 0.0021972656250000, 0.0007324218750000,
+0.0021362304687500, 0.0008239746093750, 0.0022583007812500, 0.0006103515625000,
+0.0022583007812500, 0.0005798339843750, 0.0021362304687500, 0.0007324218750000,
+0.0022277832031250, 0.0003356933593750, 0.0020141601562500, 0.0003967285156250,
+0.0020751953125000, 0.0004577636718750, 0.0019226074218750, 0.0002746582031250,
+0.0016784667968750, 0.0004272460937500, 0.0018005371093750, 0.0005798339843750,
+0.0013732910156250, 0.0004577636718750, 0.0014038085937500, 0.0008544921875000,
+0.0013122558593750, 0.0011291503906250, 0.0010681152343750, 0.0009765625000000,
+0.0011596679687500, 0.0020141601562500, 0.0010681152343750, 0.0018310546875000,
+0.0010070800781250, 0.0024108886718750, 0.0010681152343750, 0.0032958984375000,
+0.0010375976562500, 0.0027160644531250, 0.0008850097656250, 0.0043029785156250,
+0.0010375976562500, 0.0039672851562500, 0.0009155273437500, 0.0044250488281250,
+0.0007629394531250, 0.0056152343750000, 0.0008239746093750, 0.0049133300781250,
+0.0008239746093750, 0.0062561035156250, 0.0006103515625000, 0.0061950683593750,
+0.0006103515625000, 0.0062866210937500, 0.0006408691406250, 0.0071716308593750,
+0.0005187988281250, 0.0066833496093750, 0.0004577636718750, 0.0070190429687500,
+0.0008239746093750, 0.0072326660156250, 0.0006408691406250, 0.0068664550781250,
+0.0008850097656250, 0.0067443847656250, 0.0012512207031250, 0.0068664550781250,
+0.0010375976562500, 0.0059509277343750, 0.0015258789062500, 0.0061950683593750,
+0.0016174316406250, 0.0057373046875000, 0.0015563964843750, 0.0049133300781250,
+0.0017395019531250, 0.0054016113281250, 0.0018310546875000, 0.0043334960937500,
+0.0015563964843750, 0.0044860839843750, 0.0017395019531250, 0.0043945312500000,
+0.0016479492187500, 0.0037841796875000, 0.0013122558593750, 0.0041809082031250,
+0.0015563964843750, 0.0039672851562500, 0.0013732910156250, 0.0039367675781250,
+0.0012512207031250, 0.0041503906250000, 0.0013732910156250, 0.0042114257812500,
+0.0014038085937500, 0.0042724609375000, 0.0014648437500000, 0.0045471191406250,
+0.0016784667968750, 0.0046081542968750, 0.0016479492187500, 0.0048217773437500,
+0.0018310546875000, 0.0051269531250000, 0.0021057128906250, 0.0050659179687500,
+0.0019836425781250, 0.0056152343750000, 0.0024108886718750, 0.0054626464843750,
+0.0025024414062500, 0.0057983398437500, 0.0024719238281250, 0.0062561035156250,
+0.0027465820312500, 0.0059814453125000, 0.0028076171875000, 0.0066833496093750,
+0.0027465820312500, 0.0066833496093750, 0.0028381347656250, 0.0066833496093750,
+0.0028381347656250, 0.0070495605468750, 0.0028381347656250, 0.0069580078125000,
+0.0028991699218750, 0.0068359375000000, 0.0027770996093750, 0.0069580078125000,
+0.0028991699218750, 0.0067138671875000, 0.0027770996093750, 0.0063781738281250,
+0.0026245117187500, 0.0065307617187500, 0.0028686523437500, 0.0061645507812500,
+0.0023193359375000, 0.0059814453125000, 0.0025024414062500, 0.0061645507812500,
+0.0024414062500000, 0.0060424804687500, 0.0019531250000000, 0.0058288574218750,
+0.0022583007812500, 0.0063476562500000, 0.0022888183593750, 0.0061340332031250,
+0.0021972656250000, 0.0061035156250000, 0.0025939941406250, 0.0064697265625000,
+0.0029907226562500, 0.0061645507812500, 0.0027465820312500, 0.0061035156250000,
+0.0034790039062500, 0.0061645507812500, 0.0036010742187500, 0.0057983398437500,
+0.0035400390625000, 0.0054626464843750, 0.0038757324218750, 0.0055236816406250,
+0.0039367675781250, 0.0049438476562500, 0.0035400390625000, 0.0050048828125000,
+0.0038757324218750, 0.0049133300781250, 0.0037536621093750, 0.0046691894531250,
+0.0033874511718750, 0.0049743652343750, 0.0035705566406250, 0.0048217773437500,
+0.0032958984375000, 0.0048828125000000, 0.0033264160156250, 0.0050048828125000,
+0.0032958984375000, 0.0050354003906250, 0.0032348632812500, 0.0051879882812500,
+0.0032958984375000, 0.0050659179687500, 0.0029907226562500, 0.0053405761718750,
+0.0032653808593750, 0.0051269531250000, 0.0032653808593750, 0.0048217773437500,
+0.0029296875000000, 0.0048217773437500, 0.0032043457031250, 0.0035095214843750,
+0.0033569335937500, 0.0040588378906250, 0.0030822753906250, 0.0037536621093750,
+0.0035705566406250, 0.0026855468750000, 0.0040283203125000, 0.0033874511718750,
+0.0035400390625000},
+{0.0003356933593750, 0.0036621093750000, 0.0006713867187500, 0.0035095214843750,
+0.0005798339843750, 0.0035705566406250, 0.0005493164062500, 0.0037841796875000,
+0.0008239746093750, 0.0038452148437500, 0.0010375976562500, 0.0039367675781250,
+0.0009765625000000, 0.0044250488281250, 0.0012207031250000, 0.0044250488281250,
+0.0014038085937500, 0.0046691894531250, 0.0013427734375000, 0.0050964355468750,
+0.0013732910156250, 0.0049743652343750, 0.0014953613281250, 0.0053100585937500,
+0.0012512207031250, 0.0054016113281250, 0.0013732910156250, 0.0051574707031250,
+0.0013122558593750, 0.0050964355468750, 0.0010986328125000, 0.0050659179687500,
+0.0012817382812500, 0.0043945312500000, 0.0012207031250000, 0.0044860839843750,
+0.0012207031250000, 0.0040588378906250, 0.0014038085937500, 0.0032958984375000,
+0.0015258789062500, 0.0035400390625000, 0.0014038085937500, 0.0028686523437500,
+0.0019836425781250, 0.0026855468750000, 0.0018310546875000, 0.0027465820312500,
+0.0019531250000000, 0.0025634765625000, 0.0024108886718750, 0.0025634765625000,
+0.0020751953125000, 0.0030822753906250, 0.0027465820312500, 0.0029907226562500,
+0.0026550292968750, 0.0033264160156250, 0.0026855468750000, 0.0039062500000000,
+0.0031127929687500, 0.0036621093750000, 0.0029602050781250, 0.0044555664062500,
+0.0030822753906250, 0.0044860839843750, 0.0032348632812500, 0.0046386718750000,
+0.0031738281250000, 0.0050964355468750, 0.0032348632812500, 0.0048522949218750,
+0.0033264160156250, 0.0051574707031250, 0.0032653808593750, 0.0051269531250000,
+0.0033264160156250, 0.0051574707031250, 0.0032348632812500, 0.0052795410156250,
+0.0031738281250000, 0.0051269531250000, 0.0032958984375000, 0.0052795410156250,
+0.0029296875000000, 0.0052795410156250, 0.0030212402343750, 0.0051879882812500,
+0.0029602050781250, 0.0051879882812500, 0.0025939941406250, 0.0052185058593750,
+0.0027160644531250, 0.0050354003906250, 0.0027160644531250, 0.0051269531250000,
+0.0025024414062500, 0.0050659179687500, 0.0027160644531250, 0.0049133300781250,
+0.0029296875000000, 0.0049133300781250, 0.0027160644531250, 0.0049743652343750,
+0.0034179687500000, 0.0047912597656250, 0.0033264160156250, 0.0050964355468750,
+0.0033874511718750, 0.0053405761718750, 0.0038452148437500, 0.0050048828125000,
+0.0036010742187500, 0.0058593750000000, 0.0036010742187500, 0.0057373046875000,
+0.0037841796875000, 0.0057373046875000, 0.0035400390625000, 0.0062255859375000,
+0.0033874511718750, 0.0059814453125000, 0.0035705566406250, 0.0056762695312500,
+0.0032043457031250, 0.0058898925781250, 0.0031738281250000, 0.0053710937500000,
+0.0032043457031250, 0.0047912597656250, 0.0030212402343750, 0.0051879882812500,
+0.0030822753906250, 0.0042114257812500, 0.0029907226562500, 0.0041809082031250,
+0.0028991699218750, 0.0041198730468750, 0.0030212402343750, 0.0036315917968750,
+0.0030822753906250, 0.0039367675781250, 0.0030517578125000, 0.0037536621093750,
+0.0033874511718750, 0.0037231445312500, 0.0032348632812500, 0.0039367675781250,
+0.0035705566406250, 0.0039978027343750, 0.0040893554687500, 0.0041809082031250,
+0.0039672851562500, 0.0048828125000000, 0.0047302246093750, 0.0045776367187500,
+0.0044555664062500, 0.0046081542968750, 0.0044860839843750, 0.0050964355468750,
+0.0050659179687500, 0.0047912597656250, 0.0048217773437500, 0.0046997070312500,
+0.0049743652343750, 0.0049438476562500, 0.0048828125000000, 0.0045471191406250,
+0.0046386718750000, 0.0040588378906250, 0.0046081542968750, 0.0043029785156250,
+0.0045471191406250, 0.0037231445312500, 0.0039672851562500, 0.0034179687500000,
+0.0039672851562500, 0.0035095214843750, 0.0036315917968750, 0.0033264160156250,
+0.0030212402343750, 0.0031433105468750, 0.0031433105468750, 0.0036621093750000,
+0.0028076171875000, 0.0034179687500000, 0.0026245117187500, 0.0036315917968750,
+0.0026245117187500, 0.0040588378906250, 0.0025329589843750, 0.0036621093750000,
+0.0022888183593750, 0.0042724609375000, 0.0023193359375000, 0.0042114257812500,
+0.0023193359375000, 0.0040893554687500, 0.0020446777343750, 0.0044250488281250,
+0.0017089843750000, 0.0043334960937500, 0.0018005371093750, 0.0039367675781250,
+0.0013732910156250, 0.0041503906250000, 0.0012207031250000, 0.0038757324218750,
+0.0013427734375000, 0.0033874511718750, 0.0012207031250000, 0.0035705566406250,
+0.0012207031250000, 0.0034790039062500, 0.0018920898437500, 0.0032348632812500,
+0.0017395019531250, 0.0036315917968750, 0.0022888183593750, 0.0039062500000000,
+0.0031127929687500, 0.0036621093750000, 0.0027465820312500, 0.0043945312500000,
+0.0036926269531250, 0.0043334960937500, 0.0037841796875000, 0.0045471191406250,
+0.0038452148437500, 0.0051879882812500, 0.0043640136718750, 0.0048522949218750,
+0.0041809082031250, 0.0051574707031250, 0.0045166015625000, 0.0052795410156250,
+0.0046081542968750, 0.0050354003906250, 0.0047302246093750, 0.0050659179687500,
+0.0049743652343750, 0.0050964355468750, 0.0049743652343750, 0.0045471191406250,
+0.0051269531250000, 0.0046997070312500, 0.0053405761718750, 0.0045471191406250,
+0.0051879882812500, 0.0041198730468750, 0.0051269531250000, 0.0044555664062500,
+0.0052795410156250, 0.0039978027343750, 0.0045166015625000, 0.0040893554687500,
+0.0046386718750000, 0.0041503906250000, 0.0044555664062500, 0.0039978027343750,
+0.0037231445312500, 0.0041809082031250, 0.0040283203125000, 0.0041503906250000,
+0.0040893554687500, 0.0043640136718750, 0.0039062500000000, 0.0042114257812500,
+0.0040893554687500, 0.0042114257812500, 0.0043640136718750, 0.0045776367187500,
+0.0043334960937500, 0.0036010742187500, 0.0042114257812500, 0.0042114257812500,
+0.0044860839843750, 0.0037841796875000, 0.0041809082031250, 0.0028686523437500,
+0.0037841796875000, 0.0037536621093750, 0.0040588378906250, 0.0022583007812500,
+0.0032958984375000, 0.0025939941406250, 0.0032653808593750, 0.0025329589843750,
+0.0032958984375000, 0.0015869140625000, 0.0028991699218750, 0.0021972656250000,
+0.0029907226562500, 0.0016784667968750, 0.0030822753906250, 0.0017395019531250,
+0.0029602050781250, 0.0019226074218750, 0.0031127929687500, 0.0017700195312500,
+0.0033874511718750, 0.0018920898437500, 0.0032958984375000, 0.0020751953125000,
+0.0033569335937500, 0.0020751953125000, 0.0035705566406250, 0.0020751953125000,
+0.0033264160156250, 0.0021972656250000, 0.0031433105468750, 0.0021057128906250,
+0.0033264160156250, 0.0020141601562500, 0.0027465820312500, 0.0020141601562500,
+0.0028381347656250, 0.0019226074218750, 0.0028076171875000, 0.0016479492187500,
+0.0025024414062500, 0.0016479492187500, 0.0027160644531250, 0.0016479492187500,
+0.0028076171875000, 0.0014343261718750, 0.0027160644531250, 0.0015563964843750,
+0.0031738281250000, 0.0016479492187500, 0.0035705566406250, 0.0014038085937500,
+0.0035095214843750, 0.0019836425781250, 0.0044555664062500, 0.0017700195312500,
+0.0044555664062500, 0.0018920898437500, 0.0048217773437500, 0.0024719238281250,
+0.0055541992187500, 0.0020446777343750, 0.0052795410156250, 0.0025329589843750,
+0.0058898925781250, 0.0025329589843750, 0.0060424804687500, 0.0024108886718750,
+0.0058898925781250, 0.0025024414062500, 0.0060119628906250, 0.0024719238281250,
+0.0060729980468750, 0.0021667480468750, 0.0056457519531250, 0.0021667480468750,
+0.0057678222656250, 0.0021057128906250, 0.0055541992187500, 0.0019531250000000,
+0.0051269531250000, 0.0020446777343750, 0.0052795410156250, 0.0021667480468750,
+0.0047607421875000, 0.0020751953125000, 0.0047302246093750, 0.0021972656250000,
+0.0046081542968750, 0.0024719238281250, 0.0043334960937500, 0.0023803710937500,
+0.0044555664062500, 0.0026550292968750, 0.0040588378906250, 0.0026855468750000,
+0.0040588378906250, 0.0026855468750000, 0.0038146972656250, 0.0027770996093750,
+0.0034179687500000, 0.0027465820312500, 0.0036315917968750, 0.0026245117187500,
+0.0030822753906250, 0.0026245117187500, 0.0031433105468750, 0.0025634765625000,
+0.0029907226562500, 0.0023803710937500, 0.0025329589843750, 0.0024108886718750,
+0.0027465820312500, 0.0023498535156250, 0.0022888183593750, 0.0022583007812500,
+0.0022583007812500, 0.0023498535156250, 0.0022888183593750, 0.0024108886718750,
+0.0018920898437500, 0.0022888183593750, 0.0019836425781250, 0.0024414062500000,
+0.0021972656250000, 0.0023803710937500, 0.0019836425781250, 0.0024414062500000,
+0.0023498535156250, 0.0025634765625000, 0.0027465820312500, 0.0023498535156250,
+0.0024719238281250, 0.0026245117187500, 0.0035400390625000, 0.0025024414062500,
+0.0033874511718750, 0.0027160644531250, 0.0038146972656250, 0.0029907226562500,
+0.0046997070312500, 0.0027770996093750, 0.0043640136718750, 0.0033264160156250,
+0.0052490234375000, 0.0032653808593750, 0.0054931640625000, 0.0032958984375000,
+0.0054321289062500, 0.0037231445312500, 0.0057983398437500, 0.0036010742187500,
+0.0059814453125000, 0.0035705566406250, 0.0053405761718750, 0.0037231445312500,
+0.0057678222656250, 0.0035095214843750, 0.0054016113281250, 0.0032348632812500,
+0.0045471191406250, 0.0034484863281250, 0.0050964355468750, 0.0029296875000000,
+0.0039672851562500, 0.0028381347656250, 0.0038757324218750, 0.0028686523437500,
+0.0038146972656250, 0.0025329589843750, 0.0032653808593750, 0.0025024414062500,
+0.0035095214843750, 0.0031127929687500, 0.0033264160156250, 0.0026855468750000,
+0.0033264160156250, 0.0031738281250000, 0.0033264160156250, 0.0039672851562500,
+0.0032653808593750, 0.0032958984375000, 0.0032653808593750, 0.0046081542968750,
+0.0031127929687500, 0.0044860839843750, 0.0031127929687500, 0.0045166015625000,
+0.0030212402343750, 0.0051879882812500, 0.0028076171875000, 0.0048522949218750,
+0.0028381347656250, 0.0049438476562500, 0.0028076171875000, 0.0051879882812500,
+0.0026550292968750, 0.0048522949218750, 0.0028076171875000, 0.0046997070312500,
+0.0029907226562500, 0.0050354003906250, 0.0028381347656250, 0.0038452148437500,
+0.0029602050781250, 0.0041503906250000, 0.0030822753906250, 0.0036315917968750,
+0.0029907226562500, 0.0024108886718750, 0.0029602050781250, 0.0029602050781250,
+0.0030822753906250, 0.0020446777343750, 0.0031127929687500, 0.0016784667968750,
+0.0028686523437500, 0.0021362304687500, 0.0033569335937500, 0.0019836425781250,
+0.0037841796875000, 0.0016784667968750, 0.0034484863281250, 0.0030822753906250,
+0.0043334960937500, 0.0027160644531250, 0.0044860839843750, 0.0027465820312500,
+0.0042724609375000, 0.0042114257812500, 0.0045166015625000, 0.0042114257812500,
+0.0026855468750000, 0.0043640136718750, 0.0043945312500000, 0.0040893554687500,
+0.0042724609375000, 0.0044250488281250, 0.0040893554687500, 0.0048522949218750,
+0.0045166015625000, 0.0045776367187500, 0.0046081542968750, 0.0048828125000000,
+0.0039062500000000, 0.0049743652343750, 0.0040588378906250, 0.0046997070312500,
+0.0039978027343750, 0.0046081542968750, 0.0034790039062500, 0.0046691894531250,
+0.0036926269531250, 0.0041809082031250, 0.0035400390625000, 0.0042114257812500,
+0.0034179687500000, 0.0039978027343750, 0.0035095214843750, 0.0036010742187500,
+0.0034790039062500, 0.0039062500000000, 0.0033874511718750, 0.0031433105468750,
+0.0035400390625000, 0.0033264160156250, 0.0035095214843750, 0.0029907226562500,
+0.0036010742187500, 0.0022277832031250, 0.0038146972656250, 0.0026245117187500,
+0.0037231445312500, 0.0020141601562500, 0.0038452148437500, 0.0015258789062500,
+0.0039672851562500},
+{0.0048522949218750, 0.0023193359375000, 0.0049133300781250, 0.0024108886718750,
+0.0048522949218750, 0.0024108886718750, 0.0051269531250000, 0.0022277832031250,
+0.0051269531250000, 0.0026855468750000, 0.0047912597656250, 0.0024108886718750,
+0.0050354003906250, 0.0026550292968750, 0.0047607421875000, 0.0031738281250000,
+0.0043640136718750, 0.0027770996093750, 0.0046997070312500, 0.0033569335937500,
+0.0042114257812500, 0.0032958984375000, 0.0041809082031250, 0.0032043457031250,
+0.0042724609375000, 0.0035095214843750, 0.0040893554687500, 0.0033264160156250,
+0.0042114257812500, 0.0032043457031250, 0.0047607421875000, 0.0033569335937500,
+0.0044555664062500, 0.0030517578125000, 0.0050659179687500, 0.0028076171875000,
+0.0057983398437500, 0.0031127929687500, 0.0053405761718750, 0.0026245117187500,
+0.0068054199218750, 0.0026855468750000, 0.0067749023437500, 0.0029602050781250,
+0.0070190429687500, 0.0027160644531250, 0.0079956054687500, 0.0027160644531250,
+0.0075988769531250, 0.0034179687500000, 0.0078125000000000, 0.0030822753906250,
+0.0080261230468750, 0.0035095214843750, 0.0075683593750000, 0.0043029785156250,
+0.0073242187500000, 0.0037231445312500, 0.0074768066406250, 0.0047607421875000,
+0.0066528320312500, 0.0047912597656250, 0.0065917968750000, 0.0047302246093750,
+0.0063781738281250, 0.0051879882812500, 0.0058288574218750, 0.0049743652343750,
+0.0060119628906250, 0.0048522949218750, 0.0055236816406250, 0.0049743652343750,
+0.0055847167968750, 0.0045166015625000, 0.0053100585937500, 0.0041809082031250,
+0.0049743652343750, 0.0044860839843750, 0.0051879882812500, 0.0032958984375000,
+0.0041809082031250, 0.0036010742187500, 0.0044555664062500, 0.0032958984375000,
+0.0039672851562500, 0.0024108886718750, 0.0030212402343750, 0.0028991699218750,
+0.0036315917968750, 0.0023193359375000, 0.0026855468750000, 0.0019531250000000,
+0.0025939941406250, 0.0024108886718750, 0.0026245117187500, 0.0025024414062500,
+0.0022583007812500, 0.0021362304687500, 0.0023498535156250, 0.0034179687500000,
+0.0022888183593750, 0.0031433105468750, 0.0023498535156250, 0.0033569335937500,
+0.0022583007812500, 0.0044250488281250, 0.0019836425781250, 0.0039672851562500,
+0.0021362304687500, 0.0042419433593750, 0.0022277832031250, 0.0044250488281250,
+0.0019531250000000, 0.0042114257812500, 0.0024719238281250, 0.0041198730468750,
+0.0035705566406250, 0.0043945312500000, 0.0021057128906250, 0.0043640136718750,
+0.0023498535156250, 0.0046997070312500, 0.0017395019531250, 0.0042419433593750,
+0.0026245117187500, 0.0037841796875000, 0.0035095214843750, 0.0041503906250000,
+0.0028686523437500, 0.0032958984375000, 0.0041198730468750, 0.0031738281250000,
+0.0042114257812500, 0.0031738281250000, 0.0038757324218750, 0.0028076171875000,
+0.0042724609375000, 0.0027160644531250, 0.0043029785156250, 0.0028076171875000,
+0.0036315917968750, 0.0026855468750000, 0.0038452148437500, 0.0026550292968750,
+0.0035400390625000, 0.0026855468750000, 0.0028991699218750, 0.0025634765625000,
+0.0032348632812500, 0.0027160644531250, 0.0025939941406250, 0.0024108886718750,
+0.0024719238281250, 0.0027465820312500, 0.0025329589843750, 0.0029907226562500,
+0.0021362304687500, 0.0024108886718750, 0.0021362304687500, 0.0036010742187500,
+0.0024719238281250, 0.0034484863281250, 0.0022583007812500, 0.0032958984375000,
+0.0025634765625000, 0.0039062500000000, 0.0030822753906250, 0.0036315917968750,
+0.0028076171875000, 0.0035705566406250, 0.0034484863281250, 0.0034790039062500,
+0.0036010742187500, 0.0035400390625000, 0.0033874511718750, 0.0035095214843750,
+0.0035095214843750, 0.0031738281250000, 0.0036621093750000, 0.0038757324218750,
+0.0033264160156250, 0.0035400390625000, 0.0033264160156250, 0.0039062500000000,
+0.0036315917968750, 0.0045776367187500, 0.0036621093750000, 0.0039978027343750,
+0.0035705566406250, 0.0055236816406250, 0.0040588378906250, 0.0052795410156250,
+0.0041503906250000, 0.0057067871093750, 0.0040588378906250, 0.0068054199218750,
+0.0042419433593750, 0.0061340332031250, 0.0043029785156250, 0.0075378417968750,
+0.0039367675781250, 0.0073852539062500, 0.0040893554687500, 0.0073547363281250,
+0.0039062500000000, 0.0080871582031250, 0.0036621093750000, 0.0077209472656250,
+0.0038757324218750, 0.0076293945312500, 0.0033264160156250, 0.0078735351562500,
+0.0035095214843750, 0.0075073242187500, 0.0033874511718750, 0.0070190429687500,
+0.0029907226562500, 0.0073852539062500, 0.0032958984375000, 0.0065307617187500,
+0.0029602050781250, 0.0064697265625000, 0.0030822753906250, 0.0064697265625000,
+0.0032043457031250, 0.0060424804687500, 0.0031433105468750, 0.0061645507812500,
+0.0032958984375000, 0.0058288574218750, 0.0032958984375000, 0.0058593750000000,
+0.0034179687500000, 0.0058288574218750, 0.0033874511718750, 0.0057373046875000,
+0.0033874511718750, 0.0059204101562500, 0.0034484863281250, 0.0053710937500000,
+0.0034790039062500, 0.0056457519531250, 0.0034179687500000, 0.0054931640625000,
+0.0034484863281250, 0.0049133300781250, 0.0035705566406250, 0.0053405761718750,
+0.0034484863281250, 0.0052795410156250, 0.0034179687500000, 0.0050048828125000,
+0.0034790039062500, 0.0056457519531250, 0.0033569335937500, 0.0061950683593750,
+0.0032653808593750, 0.0057983398437500, 0.0033569335937500, 0.0071716308593750,
+0.0032653808593750, 0.0071105957031250, 0.0032348632812500, 0.0073852539062500,
+0.0032653808593750, 0.0083312988281250, 0.0032653808593750, 0.0080261230468750,
+0.0031738281250000, 0.0084533691406250, 0.0032958984375000, 0.0088806152343750,
+0.0032958984375000, 0.0084838867187500, 0.0032653808593750, 0.0084228515625000,
+0.0033569335937500, 0.0088806152343750, 0.0033264160156250, 0.0075683593750000,
+0.0031127929687500, 0.0080261230468750, 0.0032348632812500, 0.0077514648437500,
+0.0030212402343750, 0.0066528320312500, 0.0028076171875000, 0.0072631835937500,
+0.0030212402343750, 0.0066528320312500, 0.0025939941406250, 0.0065002441406250,
+0.0025634765625000, 0.0066528320312500, 0.0025329589843750, 0.0064086914062500,
+0.0021972656250000, 0.0064086914062500, 0.0022888183593750, 0.0065307617187500,
+0.0021057128906250, 0.0064697265625000, 0.0020446777343750, 0.0064086914062500,
+0.0019836425781250, 0.0064086914062500, 0.0018920898437500, 0.0062866210937500,
+0.0019531250000000, 0.0061645507812500, 0.0021362304687500, 0.0062866210937500,
+0.0020141601562500, 0.0060119628906250, 0.0021362304687500, 0.0057678222656250,
+0.0023193359375000, 0.0059814453125000, 0.0022583007812500, 0.0052795410156250,
+0.0023498535156250, 0.0054931640625000, 0.0024719238281250, 0.0052185058593750,
+0.0022888183593750, 0.0047607421875000, 0.0022277832031250, 0.0052490234375000,
+0.0023803710937500, 0.0042419433593750, 0.0018005371093750, 0.0045471191406250,
+0.0019226074218750, 0.0042724609375000, 0.0018615722656250, 0.0035400390625000,
+0.0014953613281250, 0.0041503906250000, 0.0016479492187500, 0.0028381347656250,
+0.0015563964843750, 0.0031127929687500, 0.0014648437500000, 0.0027160644531250,
+0.0016174316406250, 0.0016174316406250, 0.0015258789062500, 0.0022583007812500,
+0.0014038085937500, 0.0010681152343750, 0.0017700195312500, 0.0010070800781250,
+0.0015869140625000, 0.0010070800781250, 0.0017395019531250, 0.0003967285156250,
+0.0020751953125000, 0.0006713867187500, 0.0017700195312500, 0.0007934570312500,
+0.0024108886718750, 0.0007019042968750, 0.0022583007812500, 0.0010375976562500,
+0.0023193359375000, 0.0014953613281250, 0.0027465820312500, 0.0014343261718750,
+0.0025024414062500, 0.0019836425781250, 0.0028686523437500, 0.0021667480468750,
+0.0028381347656250, 0.0020751953125000, 0.0028076171875000, 0.0022888183593750,
+0.0029602050781250, 0.0024414062500000, 0.0028076171875000, 0.0018920898437500,
+0.0029602050781250, 0.0021362304687500, 0.0029296875000000, 0.0017395019531250,
+0.0028076171875000, 0.0010986328125000, 0.0029296875000000, 0.0015869140625000,
+0.0029602050781250, 0.0005187988281250, 0.0026855468750000, 0.0007019042968750,
+0.0028381347656250, 0.0005798339843750, 0.0026855468750000, -0.0000610351562500,
+0.0024108886718750, 0.0003356933593750, 0.0025939941406250, -0.0000305175781250,
+0.0025634765625000, -0.0000915527343750, 0.0023193359375000, 0.0001525878906250,
+0.0026550292968750, 0.0001220703125000, 0.0028991699218750, 0.0002136230468750,
+0.0026245117187500, 0.0009765625000000, 0.0034179687500000, 0.0008544921875000,
+0.0033569335937500, 0.0014343261718750, 0.0034179687500000, 0.0022583007812500,
+0.0039062500000000, 0.0018920898437500, 0.0037841796875000, 0.0031127929687500,
+0.0038452148437500, 0.0032043457031250, 0.0040283203125000, 0.0033264160156250,
+0.0037841796875000, 0.0039367675781250, 0.0036010742187500, 0.0038452148437500,
+0.0038146972656250, 0.0041503906250000, 0.0032653808593750, 0.0041198730468750,
+0.0033264160156250, 0.0041503906250000, 0.0032043457031250, 0.0042419433593750,
+0.0028076171875000, 0.0040893554687500, 0.0031738281250000, 0.0043029785156250,
+0.0028991699218750, 0.0043029785156250, 0.0028381347656250, 0.0042114257812500,
+0.0030212402343750, 0.0042419433593750, 0.0029907226562500, 0.0041198730468750,
+0.0029296875000000, 0.0039062500000000, 0.0033264160156250, 0.0039367675781250,
+0.0032653808593750, 0.0039367675781250, 0.0034484863281250, 0.0037536621093750,
+0.0038146972656250, 0.0036926269531250, 0.0036010742187500, 0.0039062500000000,
+0.0041198730468750, 0.0036621093750000, 0.0042114257812500, 0.0038452148437500,
+0.0041198730468750, 0.0041198730468750, 0.0042419433593750, 0.0038146972656250,
+0.0040893554687500, 0.0042419433593750, 0.0039367675781250, 0.0041809082031250,
+0.0039062500000000, 0.0041503906250000, 0.0037536621093750, 0.0046081542968750,
+0.0035400390625000, 0.0048217773437500, 0.0032958984375000, 0.0046081542968750,
+0.0031433105468750, 0.0047912597656250, 0.0031738281250000, 0.0045471191406250,
+0.0029602050781250, 0.0042724609375000, 0.0026245117187500, 0.0046691894531250,
+0.0025634765625000, 0.0036010742187500, 0.0022888183593750, 0.0036315917968750,
+0.0019531250000000, 0.0035400390625000, 0.0021362304687500, 0.0029296875000000,
+0.0021362304687500, 0.0031433105468750, 0.0018310546875000, 0.0031433105468750,
+0.0025329589843750, 0.0028991699218750, 0.0021667480468750, 0.0032348632812500,
+0.0024108886718750, 0.0036315917968750, 0.0030517578125000, 0.0032958984375000,
+0.0025329589843750, 0.0040893554687500, 0.0036926269531250, 0.0041198730468750,
+0.0034179687500000, 0.0040588378906250, 0.0038146972656250, 0.0045166015625000,
+0.0046997070312500, 0.0045471191406250, 0.0039978027343750, 0.0043640136718750,
+0.0055236816406250, 0.0048217773437500, 0.0053405761718750, 0.0045166015625000,
+0.0054626464843750, 0.0041198730468750, 0.0063476562500000, 0.0047912597656250,
+0.0059204101562500, 0.0036010742187500, 0.0062561035156250, 0.0039062500000000,
+0.0062255859375000, 0.0039367675781250, 0.0061950683593750, 0.0033264160156250,
+0.0063476562500000, 0.0038146972656250, 0.0061340332031250, 0.0033874511718750,
+0.0061340332031250, 0.0035400390625000, 0.0060424804687500, 0.0035095214843750,
+0.0059814453125000, 0.0032653808593750, 0.0060119628906250, 0.0035705566406250,
+0.0059509277343750, 0.0029602050781250, 0.0056457519531250, 0.0032348632812500,
+0.0057678222656250, 0.0029907226562500, 0.0055541992187500, 0.0024108886718750,
+0.0051879882812500},
+{0.0024719238281250, 0.0028076171875000, 0.0026855468750000, 0.0030517578125000,
+0.0024414062500000, 0.0023803710937500, 0.0032043457031250, 0.0024719238281250,
+0.0031127929687500, 0.0023498535156250, 0.0031738281250000, 0.0018310546875000,
+0.0036621093750000, 0.0019531250000000, 0.0035400390625000, 0.0014953613281250,
+0.0034484863281250, 0.0013732910156250, 0.0037231445312500, 0.0012817382812500,
+0.0036315917968750, 0.0009460449218750, 0.0034790039062500, 0.0009765625000000,
+0.0037231445312500, 0.0009765625000000, 0.0033264160156250, 0.0008544921875000,
+0.0035095214843750, 0.0010986328125000, 0.0033874511718750, 0.0014343261718750,
+0.0030212402343750, 0.0012512207031250, 0.0032958984375000, 0.0018310546875000,
+0.0029602050781250, 0.0019531250000000, 0.0030517578125000, 0.0018920898437500,
+0.0031738281250000, 0.0020751953125000, 0.0030822753906250, 0.0020751953125000,
+0.0032043457031250, 0.0021972656250000, 0.0032958984375000, 0.0020446777343750,
+0.0033264160156250, 0.0024719238281250, 0.0034790039062500, 0.0028381347656250,
+0.0036621093750000, 0.0026550292968750, 0.0035400390625000, 0.0034179687500000,
+0.0036926269531250, 0.0035400390625000, 0.0037841796875000, 0.0036926269531250,
+0.0038146972656250, 0.0041809082031250, 0.0039367675781250, 0.0042419433593750,
+0.0039062500000000, 0.0042419433593750, 0.0036315917968750, 0.0044555664062500,
+0.0036926269531250, 0.0042419433593750, 0.0034790039062500, 0.0039672851562500,
+0.0032043457031250, 0.0042114257812500, 0.0032958984375000, 0.0039367675781250,
+0.0028686523437500, 0.0039062500000000, 0.0027770996093750, 0.0038757324218750,
+0.0026245117187500, 0.0037231445312500, 0.0021362304687500, 0.0038452148437500,
+0.0019836425781250, 0.0040893554687500, 0.0019836425781250, 0.0039672851562500,
+0.0018005371093750, 0.0041198730468750, 0.0019226074218750, 0.0044250488281250,
+0.0019531250000000, 0.0042419433593750, 0.0016479492187500, 0.0046997070312500,
+0.0026550292968750, 0.0048217773437500, 0.0023193359375000, 0.0047302246093750,
+0.0026550292968750, 0.0047912597656250, 0.0034484863281250, 0.0048217773437500,
+0.0028991699218750, 0.0042419433593750, 0.0040893554687500, 0.0044555664062500,
+0.0038757324218750, 0.0041198730468750, 0.0042419433593750, 0.0035095214843750,
+0.0051574707031250, 0.0038757324218750, 0.0045471191406250, 0.0032653808593750,
+0.0056762695312500, 0.0032958984375000, 0.0055847167968750, 0.0033264160156250,
+0.0055236816406250, 0.0031127929687500, 0.0060424804687500, 0.0032043457031250,
+0.0057373046875000, 0.0034179687500000, 0.0057067871093750, 0.0033569335937500,
+0.0057067871093750, 0.0036315917968750, 0.0055541992187500, 0.0039978027343750,
+0.0053100585937500, 0.0038452148437500, 0.0053100585937500, 0.0042114257812500,
+0.0053710937500000, 0.0043640136718750, 0.0052490234375000, 0.0041503906250000,
+0.0051879882812500, 0.0041503906250000, 0.0051574707031250, 0.0043334960937500,
+0.0050048828125000, 0.0036621093750000, 0.0046691894531250, 0.0039062500000000,
+0.0046997070312500, 0.0038452148437500, 0.0043029785156250, 0.0033569335937500,
+0.0038452148437500, 0.0035705566406250, 0.0039062500000000, 0.0034484863281250,
+0.0032348632812500, 0.0033874511718750, 0.0031127929687500, 0.0034790039062500,
+0.0029907226562500, 0.0035095214843750, 0.0027160644531250, 0.0035705566406250,
+0.0026855468750000, 0.0032653808593750, 0.0025634765625000, 0.0035095214843750,
+0.0024719238281250, 0.0032958984375000, 0.0023193359375000, 0.0028991699218750,
+0.0022277832031250, 0.0032348632812500, 0.0021972656250000, 0.0027160644531250,
+0.0017089843750000, 0.0027465820312500, 0.0015869140625000, 0.0028686523437500,
+0.0013122558593750, 0.0026245117187500, 0.0007629394531250, 0.0026550292968750,
+0.0007934570312500, 0.0029907226562500, 0.0006408691406250, 0.0028686523437500,
+0.0002441406250000, 0.0030517578125000, 0.0006103515625000, 0.0033264160156250,
+0.0009765625000000, 0.0031127929687500, 0.0005798339843750, 0.0035095214843750,
+0.0016479492187500, 0.0036010742187500, 0.0015869140625000, 0.0035095214843750,
+0.0015563964843750, 0.0036926269531250, 0.0021362304687500, 0.0036926269531250,
+0.0018310546875000, 0.0035095214843750, 0.0016174316406250, 0.0036010742187500,
+0.0016784667968750, 0.0036315917968750, 0.0014343261718750, 0.0034179687500000,
+0.0011291503906250, 0.0034790039062500, 0.0011901855468750, 0.0036315917968750,
+0.0011901855468750, 0.0034484863281250, 0.0010070800781250, 0.0036315917968750,
+0.0013122558593750, 0.0037841796875000, 0.0015869140625000, 0.0035705566406250,
+0.0013732910156250, 0.0039367675781250, 0.0021362304687500, 0.0038146972656250,
+0.0021362304687500, 0.0037841796875000, 0.0020751953125000, 0.0038757324218750,
+0.0025329589843750, 0.0037231445312500, 0.0024414062500000, 0.0038452148437500,
+0.0021667480468750, 0.0038757324218750, 0.0022583007812500, 0.0036926269531250,
+0.0021667480468750, 0.0036621093750000, 0.0018005371093750, 0.0038146972656250,
+0.0019531250000000, 0.0032043457031250, 0.0020141601562500, 0.0035705566406250,
+0.0017700195312500, 0.0032348632812500, 0.0021667480468750, 0.0025634765625000,
+0.0026245117187500, 0.0031433105468750, 0.0023803710937500, 0.0023193359375000,
+0.0034484863281250, 0.0024108886718750, 0.0033569335937500, 0.0025024414062500,
+0.0037536621093750, 0.0020446777343750, 0.0046997070312500, 0.0023193359375000,
+0.0043334960937500, 0.0023803710937500, 0.0052185058593750, 0.0022888183593750,
+0.0053405761718750, 0.0024719238281250, 0.0052185058593750, 0.0027465820312500,
+0.0055236816406250, 0.0026550292968750, 0.0054931640625000, 0.0029296875000000,
+0.0047607421875000, 0.0030212402343750, 0.0051574707031250, 0.0029602050781250,
+0.0043945312500000, 0.0029907226562500, 0.0033874511718750, 0.0029907226562500,
+0.0041198730468750, 0.0028686523437500, 0.0019836425781250, 0.0028076171875000,
+0.0022583007812500, 0.0028076171875000, 0.0017700195312500, 0.0027465820312500,
+0.0001831054687500, 0.0026550292968750, 0.0009155273437500, 0.0027160644531250,
+-0.0003967285156250, 0.0025939941406250, -0.0006408691406250, 0.0025329589843750,
+-0.0004577636718750, 0.0025634765625000, -0.0010070800781250, 0.0024414062500000,
+-0.0009155273437500, 0.0025024414062500, -0.0002441406250000, 0.0024414062500000,
+-0.0006103515625000, 0.0025024414062500, 0.0001525878906250, 0.0025329589843750,
+0.0011291503906250, 0.0024108886718750, 0.0005187988281250, 0.0028686523437500,
+0.0023498535156250, 0.0026855468750000, 0.0023498535156250, 0.0028686523437500,
+0.0027465820312500, 0.0032958984375000, 0.0039978027343750, 0.0029296875000000,
+0.0035400390625000, 0.0035400390625000, 0.0043334960937500, 0.0035095214843750,
+0.0045471191406250, 0.0034179687500000, 0.0043640136718750, 0.0036621093750000,
+0.0044250488281250, 0.0036010742187500, 0.0045166015625000, 0.0032043457031250,
+0.0042114257812500, 0.0034179687500000, 0.0042419433593750, 0.0031433105468750,
+0.0041503906250000, 0.0027465820312500, 0.0039978027343750, 0.0029296875000000,
+0.0041198730468750, 0.0025634765625000, 0.0037536621093750, 0.0024719238281250,
+0.0037841796875000, 0.0026245117187500, 0.0037841796875000, 0.0024719238281250,
+0.0036621093750000, 0.0024108886718750, 0.0037231445312500, 0.0028686523437500,
+0.0036621093750000, 0.0026245117187500, 0.0037536621093750, 0.0029296875000000,
+0.0035705566406250, 0.0034790039062500, 0.0035400390625000, 0.0030822753906250,
+0.0036621093750000, 0.0038146972656250, 0.0031127929687500, 0.0038452148437500,
+0.0033569335937500, 0.0037536621093750, 0.0030212402343750, 0.0041198730468750,
+0.0023498535156250, 0.0040588378906250, 0.0028076171875000, 0.0037231445312500,
+0.0018920898437500, 0.0039367675781250, 0.0018005371093750, 0.0036621093750000,
+0.0018920898437500, 0.0033264160156250, 0.0015869140625000, 0.0037231445312500,
+0.0017089843750000, 0.0030517578125000, 0.0019531250000000, 0.0032043457031250,
+0.0019226074218750, 0.0031433105468750, 0.0020751953125000, 0.0026855468750000,
+0.0022583007812500, 0.0028991699218750, 0.0022277832031250, 0.0027770996093750,
+0.0023803710937500, 0.0027465820312500, 0.0025024414062500, 0.0030822753906250,
+0.0022888183593750, 0.0032653808593750, 0.0021362304687500, 0.0030517578125000,
+0.0023498535156250, 0.0038452148437500, 0.0020141601562500, 0.0038146972656250,
+0.0018920898437500, 0.0039062500000000, 0.0022888183593750, 0.0043640136718750,
+0.0022888183593750, 0.0041198730468750, 0.0021667480468750, 0.0043945312500000,
+0.0036621093750000, 0.0043945312500000, 0.0033874511718750, 0.0043334960937500,
+0.0040893554687500, 0.0044555664062500, 0.0056457519531250, 0.0043640136718750,
+0.0049438476562500, 0.0046691894531250, 0.0058593750000000, 0.0045166015625000,
+0.0063476562500000, 0.0044555664062500, 0.0058288574218750, 0.0045776367187500,
+0.0057373046875000, 0.0043640136718750, 0.0061340332031250, 0.0039672851562500,
+0.0050354003906250, 0.0041198730468750, 0.0053710937500000, 0.0038146972656250,
+0.0050354003906250, 0.0029602050781250, 0.0034484863281250, 0.0025329589843750,
+0.0050659179687500, 0.0032043457031250, 0.0052795410156250, 0.0027465820312500,
+0.0056152343750000, 0.0029296875000000, 0.0050048828125000, 0.0033874511718750,
+0.0046081542968750, 0.0030822753906250, 0.0050964355468750, 0.0034179687500000,
+0.0033874511718750, 0.0033874511718750, 0.0038757324218750, 0.0033569335937500,
+0.0032348632812500, 0.0036010742187500, 0.0016784667968750, 0.0035095214843750,
+0.0025634765625000, 0.0037231445312500, 0.0012207031250000, 0.0036315917968750,
+0.0008239746093750, 0.0036621093750000, 0.0014343261718750, 0.0036315917968750,
+0.0011901855468750, 0.0034179687500000, 0.0008850097656250, 0.0039367675781250,
+0.0025939941406250, 0.0034790039062500, 0.0020141601562500, 0.0037841796875000,
+0.0026550292968750, 0.0043334960937500, 0.0042114257812500, 0.0037841796875000,
+0.0032653808593750, 0.0051879882812500, 0.0046081542968750, 0.0049438476562500,
+0.0048522949218750, 0.0052795410156250, 0.0044860839843750, 0.0064392089843750,
+0.0048828125000000, 0.0059204101562500, 0.0049743652343750, 0.0065612792968750,
+0.0039062500000000, 0.0067749023437500, 0.0043334960937500, 0.0065612792968750,
+0.0039978027343750, 0.0066528320312500, 0.0029602050781250, 0.0067749023437500,
+0.0035095214843750, 0.0061035156250000, 0.0030212402343750, 0.0063476562500000,
+0.0026855468750000, 0.0058288574218750, 0.0030212402343750, 0.0050659179687500,
+0.0032348632812500, 0.0055541992187500, 0.0030212402343750, 0.0039062500000000,
+0.0033874511718750, 0.0042419433593750, 0.0035095214843750, 0.0035705566406250,
+0.0033569335937500, 0.0022277832031250, 0.0034484863281250, 0.0029907226562500,
+0.0036010742187500, 0.0013732910156250, 0.0032653808593750, 0.0013427734375000,
+0.0032348632812500, 0.0012512207031250, 0.0033874511718750, 0.0003967285156250,
+0.0031738281250000, 0.0007629394531250, 0.0031127929687500, 0.0003967285156250,
+0.0034790039062500, 0.0001220703125000, 0.0033264160156250, 0.0003662109375000,
+0.0034790039062500, 0.0003662109375000, 0.0038146972656250, 0.0000610351562500,
+0.0035095214843750, 0.0007019042968750, 0.0039978027343750, 0.0003967285156250,
+0.0039367675781250, 0.0007934570312500, 0.0039367675781250, 0.0014648437500000,
+0.0042419433593750, 0.0008850097656250, 0.0040893554687500, 0.0022277832031250,
+0.0041198730468750},
+{0.0042114257812500, 0.0039062500000000, 0.0041809082031250, 0.0039367675781250,
+0.0042114257812500, 0.0039672851562500, 0.0040893554687500, 0.0042114257812500,
+0.0040283203125000, 0.0040283203125000, 0.0041809082031250, 0.0042114257812500,
+0.0036010742187500, 0.0040588378906250, 0.0039367675781250, 0.0041198730468750,
+0.0033874511718750, 0.0042114257812500, 0.0025939941406250, 0.0039367675781250,
+0.0032043457031250, 0.0043334960937500, 0.0015869140625000, 0.0043334960937500,
+0.0016174316406250, 0.0042419433593750, 0.0015869140625000, 0.0045776367187500,
+0.0005493164062500, 0.0046081542968750, 0.0008544921875000, 0.0039062500000000,
+0.0016174316406250, 0.0042114257812500, 0.0010986328125000, 0.0037841796875000,
+0.0019531250000000, 0.0029296875000000, 0.0031738281250000, 0.0034484863281250,
+0.0024108886718750, 0.0026855468750000, 0.0042419433593750, 0.0025024414062500,
+0.0042114257812500, 0.0027160644531250, 0.0042724609375000, 0.0027160644531250,
+0.0058593750000000, 0.0035400390625000, 0.0014648437500000, 0.0037841796875000,
+0.0032958984375000, 0.0036315917968750, 0.0026855468750000, 0.0036315917968750,
+0.0032348632812500, 0.0036926269531250, 0.0040893554687500, 0.0036010742187500,
+0.0036926269531250, 0.0038757324218750, 0.0046081542968750, 0.0037231445312500,
+0.0045471191406250, 0.0037231445312500, 0.0045776367187500, 0.0038146972656250,
+0.0052795410156250, 0.0036315917968750, 0.0050048828125000, 0.0040283203125000,
+0.0044250488281250, 0.0036010742187500, 0.0050048828125000, 0.0039672851562500,
+0.0040588378906250, 0.0045471191406250, 0.0028381347656250, 0.0037841796875000,
+0.0037231445312500, 0.0052185058593750, 0.0017395019531250, 0.0050659179687500,
+0.0014953613281250, 0.0050964355468750, 0.0018920898437500, 0.0060729980468750,
+0.0011901855468750, 0.0057067871093750, 0.0010986328125000, 0.0057373046875000,
+0.0024414062500000, 0.0061035156250000, 0.0017700195312500, 0.0056762695312500,
+0.0025024414062500, 0.0053100585937500, 0.0039062500000000, 0.0057067871093750,
+0.0029602050781250, 0.0046386718750000, 0.0046386718750000, 0.0049133300781250,
+0.0046081542968750, 0.0044860839843750, 0.0043640136718750, 0.0036926269531250,
+0.0051574707031250, 0.0043029785156250, 0.0049133300781250, 0.0027770996093750,
+0.0043334960937500, 0.0030517578125000, 0.0046386718750000, 0.0027465820312500,
+0.0041198730468750, 0.0016174316406250, 0.0033569335937500, 0.0022888183593750,
+0.0037536621093750, 0.0013732910156250, 0.0030822753906250, 0.0011901855468750,
+0.0029296875000000, 0.0014343261718750, 0.0032043457031250, 0.0011901855468750,
+0.0031738281250000, 0.0010681152343750, 0.0030822753906250, 0.0016784667968750,
+0.0034790039062500, 0.0014953613281250, 0.0035705566406250, 0.0018005371093750,
+0.0035400390625000, 0.0022888183593750, 0.0037231445312500, 0.0019226074218750,
+0.0036926269531250, 0.0029907226562500, 0.0035400390625000, 0.0027770996093750,
+0.0036621093750000, 0.0031127929687500, 0.0036010742187500, 0.0039062500000000,
+0.0034790039062500, 0.0033569335937500, 0.0036621093750000, 0.0043640136718750,
+0.0035705566406250, 0.0043640136718750, 0.0036010742187500, 0.0041198730468750,
+0.0036315917968750, 0.0045166015625000, 0.0036621093750000, 0.0044555664062500,
+0.0037231445312500, 0.0035095214843750, 0.0038146972656250, 0.0039062500000000,
+0.0038757324218750, 0.0031738281250000, 0.0038757324218750, 0.0020141601562500,
+0.0039367675781250, 0.0026855468750000, 0.0039978027343750, 0.0013427734375000,
+0.0038757324218750, 0.0009460449218750, 0.0039062500000000, 0.0014343261718750,
+0.0038452148437500, 0.0010375976562500, 0.0037536621093750, 0.0007629394531250,
+0.0037841796875000, 0.0025024414062500, 0.0036010742187500, 0.0018310546875000,
+0.0036315917968750, 0.0026245117187500, 0.0035095214843750, 0.0043029785156250,
+0.0032958984375000, 0.0032958984375000, 0.0033264160156250, 0.0049438476562500,
+0.0029907226562500, 0.0052185058593750, 0.0030212402343750, 0.0048828125000000,
+0.0029296875000000, 0.0056152343750000, 0.0026855468750000, 0.0056457519531250,
+0.0027770996093750, 0.0045166015625000, 0.0026550292968750, 0.0052490234375000,
+0.0026245117187500, 0.0043640136718750, 0.0026855468750000, 0.0030212402343750,
+0.0026855468750000, 0.0041503906250000, 0.0026550292968750, 0.0015869140625000,
+0.0027465820312500, 0.0018615722656250, 0.0027160644531250, 0.0015563964843750,
+0.0027160644531250, -0.0000305175781250, 0.0028686523437500, 0.0007324218750000,
+0.0027465820312500, -0.0003662109375000, 0.0028686523437500, -0.0004882812500000,
+0.0028381347656250, -0.0004272460937500, 0.0028381347656250, -0.0010070800781250,
+0.0028381347656250, -0.0009765625000000, 0.0027770996093750, -0.0008544921875000,
+0.0027465820312500, -0.0012207031250000, 0.0027160644531250, -0.0009460449218750,
+0.0027770996093750, -0.0005798339843750, 0.0028991699218750, -0.0010070800781250,
+0.0028991699218750, -0.0003356933593750, 0.0029602050781250, -0.0004272460937500,
+0.0029602050781250, -0.0003356933593750, 0.0030212402343750, 0.0000610351562500,
+0.0030212402343750, -0.0001525878906250, 0.0029296875000000, 0.0003662109375000,
+0.0030822753906250, 0.0003356933593750, 0.0029602050781250, 0.0005798339843750,
+0.0029907226562500, 0.0010375976562500, 0.0030822753906250, 0.0009765625000000,
+0.0028991699218750, 0.0017089843750000, 0.0031433105468750, 0.0017700195312500,
+0.0029602050781250, 0.0019836425781250, 0.0030822753906250, 0.0025634765625000,
+0.0032653808593750, 0.0025329589843750, 0.0028991699218750, 0.0028686523437500,
+0.0032043457031250, 0.0032043457031250, 0.0031738281250000, 0.0030517578125000,
+0.0030822753906250, 0.0030822753906250, 0.0032348632812500, 0.0034484863281250,
+0.0030517578125000, 0.0026245117187500, 0.0031738281250000, 0.0029602050781250,
+0.0031738281250000, 0.0026245117187500, 0.0030822753906250, 0.0017395019531250,
+0.0030822753906250, 0.0022583007812500, 0.0030822753906250, 0.0012512207031250,
+0.0028991699218750, 0.0011291503906250, 0.0029296875000000, 0.0010986328125000,
+0.0027465820312500, 0.0006103515625000, 0.0025939941406250, 0.0006713867187500,
+0.0028076171875000, 0.0007019042968750, 0.0024719238281250, 0.0004577636718750,
+0.0024414062500000, 0.0005187988281250, 0.0026245117187500, 0.0006408691406250,
+0.0025939941406250, 0.0002746582031250, 0.0026245117187500, 0.0007934570312500,
+0.0031127929687500, 0.0005187988281250, 0.0030212402343750, 0.0006713867187500,
+0.0031738281250000, 0.0010070800781250, 0.0035705566406250, 0.0005493164062500,
+0.0035095214843750, 0.0016784667968750, 0.0036315917968750, 0.0011596679687500,
+0.0037536621093750, 0.0016479492187500, 0.0036621093750000, 0.0026245117187500,
+0.0036315917968750, 0.0017089843750000, 0.0036315917968750, 0.0032653808593750,
+0.0034179687500000, 0.0030822753906250, 0.0033569335937500, 0.0030212402343750,
+0.0033569335937500, 0.0039367675781250, 0.0032348632812500, 0.0033874511718750,
+0.0031738281250000, 0.0033874511718750, 0.0032043457031250, 0.0036315917968750,
+0.0031433105468750, 0.0031738281250000, 0.0031738281250000, 0.0028686523437500,
+0.0031738281250000, 0.0030822753906250, 0.0029907226562500, 0.0026550292968750,
+0.0033264160156250, 0.0025634765625000, 0.0032043457031250, 0.0026550292968750,
+0.0032958984375000, 0.0025634765625000, 0.0036926269531250, 0.0026550292968750,
+0.0035095214843750, 0.0028381347656250, 0.0037841796875000, 0.0028076171875000,
+0.0039062500000000, 0.0031127929687500, 0.0038452148437500, 0.0033569335937500,
+0.0039062500000000, 0.0033264160156250, 0.0039672851562500, 0.0039672851562500,
+0.0036926269531250, 0.0040283203125000, 0.0038757324218750, 0.0041198730468750,
+0.0036315917968750, 0.0044555664062500, 0.0032653808593750, 0.0043640136718750,
+0.0035705566406250, 0.0043334960937500, 0.0028686523437500, 0.0043640136718750,
+0.0029907226562500, 0.0041198730468750, 0.0027770996093750, 0.0039367675781250,
+0.0021972656250000, 0.0040283203125000, 0.0024719238281250, 0.0030822753906250,
+0.0020446777343750, 0.0031127929687500, 0.0018310546875000, 0.0026245117187500,
+0.0021057128906250, 0.0018005371093750, 0.0021057128906250, 0.0022277832031250,
+0.0018920898437500, 0.0012207031250000, 0.0028381347656250, 0.0011901855468750,
+0.0026245117187500, 0.0012817382812500, 0.0029296875000000, 0.0007019042968750,
+0.0039062500000000, 0.0009155273437500, 0.0038146972656250, 0.0016174316406250,
+0.0045166015625000, 0.0010986328125000, 0.0043029785156250, 0.0015869140625000,
+0.0044250488281250, 0.0023803710937500, 0.0048522949218750, 0.0016479492187500,
+0.0045166015625000, 0.0028381347656250, 0.0047912597656250, 0.0026855468750000,
+0.0048522949218750, 0.0025939941406250, 0.0045471191406250, 0.0033264160156250,
+0.0046386718750000, 0.0030212402343750, 0.0047607421875000, 0.0028686523437500,
+0.0040588378906250, 0.0029296875000000, 0.0042419433593750, 0.0029907226562500,
+0.0040588378906250, 0.0028381347656250, 0.0034790039062500, 0.0028381347656250,
+0.0038146972656250, 0.0033264160156250, 0.0033569335937500, 0.0031127929687500,
+0.0035705566406250, 0.0034790039062500, 0.0033264160156250, 0.0040283203125000,
+0.0029296875000000, 0.0037231445312500, 0.0033569335937500, 0.0044860839843750,
+0.0023803710937500, 0.0045166015625000, 0.0027465820312500, 0.0043945312500000,
+0.0023498535156250, 0.0046081542968750, 0.0014038085937500, 0.0046386718750000,
+0.0021362304687500, 0.0042724609375000, 0.0010986328125000, 0.0043945312500000,
+0.0010375976562500, 0.0043029785156250, 0.0013427734375000, 0.0039978027343750,
+0.0011291503906250, 0.0041809082031250, 0.0012207031250000, 0.0040893554687500,
+0.0016174316406250, 0.0039367675781250, 0.0016784667968750, 0.0040893554687500,
+0.0018005371093750, 0.0042724609375000, 0.0021057128906250, 0.0041809082031250,
+0.0021057128906250, 0.0044555664062500, 0.0021057128906250, 0.0045471191406250,
+0.0020751953125000, 0.0043945312500000, 0.0021972656250000, 0.0043945312500000,
+0.0022277832031250, 0.0045776367187500, 0.0021667480468750, 0.0040588378906250,
+0.0027465820312500, 0.0041503906250000, 0.0025634765625000, 0.0041809082031250,
+0.0028686523437500, 0.0038757324218750, 0.0035400390625000, 0.0040283203125000,
+0.0031738281250000, 0.0040283203125000, 0.0037231445312500, 0.0040283203125000,
+0.0038452148437500, 0.0040893554687500, 0.0036926269531250, 0.0041503906250000,
+0.0039062500000000, 0.0041503906250000, 0.0037841796875000, 0.0040283203125000,
+0.0034790039062500, 0.0041503906250000, 0.0033264160156250, 0.0039672851562500,
+0.0034484863281250, 0.0037231445312500, 0.0032958984375000, 0.0039062500000000,
+0.0031127929687500, 0.0034179687500000, 0.0039367675781250, 0.0035095214843750,
+0.0036926269531250, 0.0034179687500000, 0.0038757324218750, 0.0030517578125000,
+0.0045471191406250, 0.0032043457031250, 0.0042419433593750, 0.0031127929687500,
+0.0042724609375000, 0.0029296875000000, 0.0045471191406250, 0.0031127929687500,
+0.0040893554687500, 0.0032653808593750, 0.0036315917968750, 0.0030517578125000,
+0.0039062500000000, 0.0034790039062500, 0.0027770996093750, 0.0034484863281250,
+0.0028991699218750, 0.0035095214843750, 0.0025634765625000, 0.0037841796875000,
+0.0017089843750000, 0.0036621093750000, 0.0021057128906250, 0.0037536621093750,
+0.0014953613281250, 0.0037536621093750, 0.0014953613281250, 0.0036621093750000,
+0.0016174316406250},
+{0.0025024414062500, 0.0036926269531250, 0.0028686523437500, 0.0043029785156250,
+0.0036621093750000, 0.0039672851562500, 0.0031738281250000, 0.0037231445312500,
+0.0039062500000000, 0.0040283203125000, 0.0039978027343750, 0.0034484863281250,
+0.0038757324218750, 0.0025024414062500, 0.0040893554687500, 0.0021362304687500,
+0.0040893554687500, 0.0007324218750000, 0.0038757324218750, 0.0011596679687500,
+0.0039978027343750, 0.0010986328125000, 0.0039062500000000, 0.0000915527343750,
+0.0036926269531250, 0.0004272460937500, 0.0038452148437500, 0.0007019042968750,
+0.0035400390625000, 0.0000610351562500, 0.0035705566406250, 0.0007934570312500,
+0.0035400390625000, 0.0016479492187500, 0.0033264160156250, 0.0007324218750000,
+0.0033569335937500, 0.0024719238281250, 0.0031433105468750, 0.0025024414062500,
+0.0033264160156250, 0.0025024414062500, 0.0031433105468750, 0.0034484863281250,
+0.0030212402343750, 0.0032043457031250, 0.0032348632812500, 0.0032653808593750,
+0.0023498535156250, 0.0033874511718750, 0.0025634765625000, 0.0032653808593750,
+0.0024719238281250, 0.0030822753906250, 0.0017700195312500, 0.0031127929687500,
+0.0021667480468750, 0.0031433105468750, 0.0021057128906250, 0.0030212402343750,
+0.0018310546875000, 0.0032348632812500, 0.0024414062500000, 0.0035400390625000,
+0.0027770996093750, 0.0033874511718750, 0.0024108886718750, 0.0038452148437500,
+0.0036315917968750, 0.0038452148437500, 0.0036621093750000, 0.0039367675781250,
+0.0036621093750000, 0.0042114257812500, 0.0043640136718750, 0.0041198730468750,
+0.0041503906250000, 0.0041198730468750, 0.0041809082031250, 0.0042724609375000,
+0.0043334960937500, 0.0040283203125000, 0.0042419433593750, 0.0038146972656250,
+0.0040283203125000, 0.0039672851562500, 0.0040893554687500, 0.0036315917968750,
+0.0043029785156250, 0.0035705566406250, 0.0040893554687500, 0.0036010742187500,
+0.0042419433593750, 0.0034790039062500, 0.0044860839843750, 0.0034484863281250,
+0.0043029785156250, 0.0035095214843750, 0.0043640136718750, 0.0035095214843750,
+0.0045471191406250, 0.0034179687500000, 0.0041503906250000, 0.0033569335937500,
+0.0038757324218750, 0.0033874511718750, 0.0041503906250000, 0.0031433105468750,
+0.0034790039062500, 0.0032348632812500, 0.0034484863281250, 0.0031738281250000,
+0.0035705566406250, 0.0029907226562500, 0.0033264160156250, 0.0029907226562500,
+0.0032348632812500, 0.0030212402343750, 0.0038146972656250, 0.0028381347656250,
+0.0035705566406250, 0.0029602050781250, 0.0037536621093750, 0.0031127929687500,
+0.0043945312500000, 0.0028381347656250, 0.0039978027343750, 0.0031738281250000,
+0.0039978027343750, 0.0031433105468750, 0.0041503906250000, 0.0030212402343750,
+0.0036315917968750, 0.0031738281250000, 0.0031738281250000, 0.0030822753906250,
+0.0034484863281250, 0.0029602050781250, 0.0025634765625000, 0.0029907226562500,
+0.0025329589843750, 0.0028686523437500, 0.0025329589843750, 0.0027160644531250,
+0.0021667480468750, 0.0028686523437500, 0.0022888183593750, 0.0026550292968750,
+0.0023193359375000, 0.0025634765625000, 0.0022888183593750, 0.0026550292968750,
+0.0022888183593750, 0.0027160644531250, 0.0024414062500000, 0.0026245117187500,
+0.0024414062500000, 0.0028686523437500, 0.0023193359375000, 0.0028686523437500,
+0.0025024414062500, 0.0028076171875000, 0.0023803710937500, 0.0028991699218750,
+0.0021057128906250, 0.0028991699218750, 0.0023193359375000, 0.0026855468750000,
+0.0020751953125000, 0.0027465820312500, 0.0020141601562500, 0.0026245117187500,
+0.0021667480468750, 0.0024414062500000, 0.0021667480468750, 0.0025939941406250,
+0.0020751953125000, 0.0023803710937500, 0.0024719238281250, 0.0024719238281250,
+0.0022888183593750, 0.0024108886718750, 0.0023803710937500, 0.0023193359375000,
+0.0026855468750000, 0.0025329589843750, 0.0025024414062500, 0.0023193359375000,
+0.0027465820312500, 0.0024719238281250, 0.0027770996093750, 0.0024414062500000,
+0.0027160644531250, 0.0023498535156250, 0.0027465820312500, 0.0026245117187500,
+0.0027770996093750, 0.0025024414062500, 0.0025329589843750, 0.0026550292968750,
+0.0026245117187500, 0.0027160644531250, 0.0024719238281250, 0.0027160644531250,
+0.0021972656250000, 0.0028686523437500, 0.0023498535156250, 0.0027160644531250,
+0.0019836425781250, 0.0029296875000000, 0.0019836425781250, 0.0029296875000000,
+0.0020141601562500, 0.0027770996093750, 0.0018310546875000, 0.0030517578125000,
+0.0019531250000000, 0.0027770996093750, 0.0021057128906250, 0.0029602050781250,
+0.0020141601562500, 0.0029907226562500, 0.0021972656250000, 0.0028381347656250,
+0.0025024414062500, 0.0031433105468750, 0.0023193359375000, 0.0029602050781250,
+0.0027160644531250, 0.0031433105468750, 0.0027160644531250, 0.0032348632812500,
+0.0028076171875000, 0.0031433105468750, 0.0030822753906250, 0.0034179687500000,
+0.0030822753906250, 0.0030822753906250, 0.0032653808593750, 0.0032348632812500,
+0.0033569335937500, 0.0032043457031250, 0.0034790039062500, 0.0028686523437500,
+0.0036315917968750, 0.0031127929687500, 0.0036621093750000, 0.0028076171875000,
+0.0039672851562500, 0.0028076171875000, 0.0040588378906250, 0.0028686523437500,
+0.0041198730468750, 0.0027160644531250, 0.0043640136718750, 0.0027770996093750,
+0.0044555664062500, 0.0028381347656250, 0.0042724609375000, 0.0028686523437500,
+0.0045776367187500, 0.0030212402343750, 0.0043029785156250, 0.0031738281250000,
+0.0039062500000000, 0.0031738281250000, 0.0042419433593750, 0.0034179687500000,
+0.0034179687500000, 0.0035095214843750, 0.0035400390625000, 0.0034790039062500,
+0.0033264160156250, 0.0036315917968750, 0.0026855468750000, 0.0036621093750000,
+0.0029907226562500, 0.0033874511718750, 0.0026245117187500, 0.0034790039062500,
+0.0024108886718750, 0.0033569335937500, 0.0027770996093750, 0.0030822753906250,
+0.0028686523437500, 0.0032653808593750, 0.0026855468750000, 0.0031127929687500,
+0.0037841796875000, 0.0029907226562500, 0.0035705566406250, 0.0031433105468750,
+0.0039062500000000, 0.0032348632812500, 0.0048217773437500, 0.0030822753906250,
+0.0044250488281250, 0.0034179687500000, 0.0050964355468750, 0.0033569335937500,
+0.0052185058593750, 0.0033569335937500, 0.0051269531250000, 0.0035400390625000,
+0.0054321289062500, 0.0034484863281250, 0.0053710937500000, 0.0035095214843750,
+0.0053100585937500, 0.0035095214843750, 0.0052185058593750, 0.0035095214843750,
+0.0052185058593750, 0.0034484863281250, 0.0052490234375000, 0.0034484863281250,
+0.0051269531250000, 0.0035705566406250, 0.0050659179687500, 0.0034790039062500,
+0.0051879882812500, 0.0035705566406250, 0.0048828125000000, 0.0037841796875000,
+0.0046386718750000, 0.0036621093750000, 0.0048828125000000, 0.0039978027343750,
+0.0042724609375000, 0.0040588378906250, 0.0043334960937500, 0.0037536621093750,
+0.0042114257812500, 0.0037536621093750, 0.0038757324218750, 0.0037536621093750,
+0.0040893554687500, 0.0028991699218750, 0.0035705566406250, 0.0030517578125000,
+0.0035705566406250, 0.0027770996093750, 0.0035705566406250, 0.0020446777343750,
+0.0032653808593750, 0.0023803710937500, 0.0033569335937500, 0.0021362304687500,
+0.0036315917968750, 0.0018615722656250, 0.0033264160156250, 0.0022277832031250,
+0.0038452148437500, 0.0024108886718750, 0.0044250488281250, 0.0021057128906250,
+0.0038757324218750, 0.0030212402343750, 0.0050048828125000, 0.0029602050781250,
+0.0050964355468750, 0.0029602050781250, 0.0047302246093750, 0.0035095214843750,
+0.0050964355468750, 0.0033874511718750, 0.0050048828125000, 0.0033874511718750,
+0.0039062500000000, 0.0035095214843750, 0.0042724609375000, 0.0034179687500000,
+0.0037841796875000, 0.0032958984375000, 0.0019226074218750, 0.0032653808593750,
+0.0041809082031250, 0.0043029785156250, 0.0032653808593750, 0.0037536621093750,
+0.0036926269531250, 0.0042114257812500, 0.0033264160156250, 0.0048828125000000,
+0.0026550292968750, 0.0044555664062500, 0.0028686523437500, 0.0050659179687500,
+0.0023193359375000, 0.0050354003906250, 0.0025024414062500, 0.0049438476562500,
+0.0021972656250000, 0.0052185058593750, 0.0017089843750000, 0.0050354003906250,
+0.0021667480468750, 0.0049133300781250, 0.0012512207031250, 0.0049743652343750,
+0.0011596679687500, 0.0046997070312500, 0.0015258789062500, 0.0044250488281250,
+0.0010681152343750, 0.0046691894531250, 0.0009460449218750, 0.0042114257812500,
+0.0026245117187500, 0.0043029785156250, 0.0018920898437500, 0.0041809082031250,
+0.0026550292968750, 0.0039367675781250, 0.0043945312500000, 0.0040893554687500,
+0.0033264160156250, 0.0033874511718750, 0.0047607421875000, 0.0037536621093750,
+0.0049743652343750, 0.0034484863281250, 0.0043945312500000, 0.0027770996093750,
+0.0047912597656250, 0.0033264160156250, 0.0049133300781250, 0.0027160644531250,
+0.0034484863281250, 0.0027465820312500, 0.0039672851562500, 0.0030822753906250,
+0.0032348632812500, 0.0030212402343750, 0.0016784667968750, 0.0031127929687500,
+0.0025329589843750, 0.0036315917968750, 0.0010986328125000, 0.0036010742187500,
+0.0008544921875000, 0.0038452148437500, 0.0012817382812500, 0.0042724609375000,
+0.0009460449218750, 0.0041809082031250, 0.0009460449218750, 0.0045471191406250,
+0.0021667480468750, 0.0047302246093750, 0.0018920898437500, 0.0047302246093750,
+0.0023498535156250, 0.0048828125000000, 0.0034484863281250, 0.0049438476562500,
+0.0030517578125000, 0.0046997070312500, 0.0034179687500000, 0.0048828125000000,
+0.0037536621093750, 0.0045776367187500, 0.0034484863281250, 0.0042114257812500,
+0.0032043457031250, 0.0045166015625000, 0.0034179687500000, 0.0036010742187500,
+0.0030822753906250, 0.0037841796875000, 0.0029296875000000, 0.0036010742187500,
+0.0030212402343750, 0.0028991699218750, 0.0028991699218750, 0.0033264160156250,
+0.0026550292968750, 0.0032043457031250, 0.0029602050781250, 0.0028381347656250,
+0.0028381347656250, 0.0034179687500000, 0.0028076171875000, 0.0039672851562500,
+0.0029602050781250, 0.0036010742187500, 0.0027160644531250, 0.0048828125000000,
+0.0030212402343750, 0.0047912597656250, 0.0028686523437500, 0.0049133300781250,
+0.0027465820312500, 0.0059204101562500, 0.0029296875000000, 0.0057373046875000,
+0.0027770996093750, 0.0052185058593750, 0.0027465820312500, 0.0059204101562500,
+0.0026855468750000, 0.0051574707031250, 0.0026855468750000, 0.0040283203125000,
+0.0027160644531250, 0.0049743652343750, 0.0026245117187500, 0.0030212402343750,
+0.0026855468750000, 0.0030517578125000, 0.0026245117187500, 0.0030517578125000,
+0.0025634765625000, 0.0019226074218750, 0.0025939941406250, 0.0022888183593750,
+0.0025634765625000, 0.0022888183593750, 0.0026245117187500, 0.0019226074218750,
+0.0024414062500000, 0.0024414062500000, 0.0025634765625000, 0.0028381347656250,
+0.0026855468750000, 0.0024108886718750, 0.0024414062500000, 0.0034484863281250,
+0.0027160644531250, 0.0033874511718750, 0.0026855468750000, 0.0033874511718750,
+0.0026550292968750, 0.0039672851562500, 0.0028686523437500, 0.0038452148437500,
+0.0027770996093750, 0.0036315917968750, 0.0027160644531250, 0.0038452148437500,
+0.0028381347656250, 0.0037231445312500, 0.0028076171875000, 0.0033569335937500,
+0.0027465820312500, 0.0035095214843750, 0.0028991699218750, 0.0033874511718750,
+0.0027465820312500, 0.0032958984375000, 0.0028381347656250, 0.0032958984375000,
+0.0028991699218750, 0.0033569335937500, 0.0027465820312500, 0.0033874511718750,
+0.0028381347656250},
+{0.0028686523437500, 0.0040893554687500, 0.0029907226562500, 0.0039672851562500,
+0.0026550292968750, 0.0040283203125000, 0.0028381347656250, 0.0038757324218750,
+0.0023498535156250, 0.0037536621093750, 0.0019836425781250, 0.0039978027343750,
+0.0025024414062500, 0.0034484863281250, 0.0010986328125000, 0.0037231445312500,
+0.0012512207031250, 0.0035400390625000, 0.0013122558593750, 0.0029296875000000,
+0.0004272460937500, 0.0034179687500000, 0.0007324218750000, 0.0032348632812500,
+0.0013122558593750, 0.0029907226562500, 0.0007629394531250, 0.0035095214843750,
+0.0016479492187500, 0.0039062500000000, 0.0027160644531250, 0.0037231445312500,
+0.0019531250000000, 0.0046081542968750, 0.0038146972656250, 0.0046691894531250,
+0.0039978027343750, 0.0047912597656250, 0.0038146972656250, 0.0053100585937500,
+0.0047912597656250, 0.0052795410156250, 0.0047607421875000, 0.0053100585937500,
+0.0039062500000000, 0.0055541992187500, 0.0045471191406250, 0.0053405761718750,
+0.0037231445312500, 0.0052795410156250, 0.0024414062500000, 0.0055236816406250,
+0.0034179687500000, 0.0046691894531250, 0.0014953613281250, 0.0050354003906250,
+0.0013427734375000, 0.0045166015625000, 0.0017089843750000, 0.0036621093750000,
+0.0008544921875000, 0.0043945312500000, 0.0008850097656250, 0.0027770996093750,
+0.0018920898437500, 0.0028991699218750, 0.0013427734375000, 0.0028991699218750,
+0.0018615722656250, 0.0019836425781250, 0.0029907226562500, 0.0023803710937500,
+0.0022277832031250, 0.0020446777343750, 0.0032348632812500, 0.0019531250000000,
+0.0033264160156250, 0.0022277832031250, 0.0031738281250000, 0.0023498535156250,
+0.0033569335937500, 0.0023193359375000, 0.0032653808593750, 0.0028381347656250,
+0.0031127929687500, 0.0031127929687500, 0.0031127929687500, 0.0028991699218750,
+0.0030212402343750, 0.0030212402343750, 0.0028686523437500, 0.0033264160156250,
+0.0028991699218750, 0.0023498535156250, 0.0028686523437500, 0.0028991699218750,
+0.0026855468750000, 0.0022888183593750, 0.0028686523437500, 0.0011901855468750,
+0.0029602050781250, 0.0021362304687500, 0.0027160644531250, 0.0004882812500000,
+0.0031433105468750, 0.0006103515625000, 0.0030212402343750, 0.0007629394531250,
+0.0029907226562500, -0.0000305175781250, 0.0031738281250000, 0.0003967285156250,
+0.0030212402343750, 0.0005187988281250, 0.0031433105468750, 0.0004272460937500,
+0.0030517578125000, 0.0007934570312500, 0.0031433105468750, 0.0012512207031250,
+0.0032958984375000, 0.0011901855468750, 0.0030517578125000, 0.0015869140625000,
+0.0033874511718750, 0.0018005371093750, 0.0032653808593750, 0.0017395019531250,
+0.0032043457031250, 0.0018310546875000, 0.0033264160156250, 0.0020751953125000,
+0.0030822753906250, 0.0015869140625000, 0.0031433105468750, 0.0018005371093750,
+0.0029602050781250, 0.0017700195312500, 0.0028381347656250, 0.0013427734375000,
+0.0028991699218750, 0.0016784667968750, 0.0027160644531250, 0.0017089843750000,
+0.0025939941406250, 0.0014953613281250, 0.0026245117187500, 0.0018310546875000,
+0.0025329589843750, 0.0021667480468750, 0.0024108886718750, 0.0019226074218750,
+0.0025329589843750, 0.0025329589843750, 0.0025024414062500, 0.0025024414062500,
+0.0025024414062500, 0.0025634765625000, 0.0025329589843750, 0.0028381347656250,
+0.0025024414062500, 0.0026245117187500, 0.0025939941406250, 0.0030517578125000,
+0.0025634765625000, 0.0028991699218750, 0.0026245117187500, 0.0030517578125000,
+0.0026245117187500, 0.0034179687500000, 0.0026550292968750, 0.0031738281250000,
+0.0027160644531250, 0.0035095214843750, 0.0026855468750000, 0.0034790039062500,
+0.0027770996093750, 0.0033264160156250, 0.0028076171875000, 0.0034484863281250,
+0.0027160644531250, 0.0034179687500000, 0.0027465820312500, 0.0031433105468750,
+0.0028381347656250, 0.0032653808593750, 0.0027465820312500, 0.0030822753906250,
+0.0028381347656250, 0.0028381347656250, 0.0029907226562500, 0.0030822753906250,
+0.0028076171875000, 0.0027770996093750, 0.0030517578125000, 0.0028381347656250,
+0.0029602050781250, 0.0028991699218750, 0.0029296875000000, 0.0027465820312500,
+0.0030212402343750, 0.0028381347656250, 0.0028991699218750, 0.0027465820312500,
+0.0030517578125000, 0.0027465820312500, 0.0029907226562500, 0.0027160644531250,
+0.0029907226562500, 0.0025634765625000, 0.0031127929687500, 0.0024719238281250,
+0.0029907226562500, 0.0024108886718750, 0.0031433105468750, 0.0022583007812500,
+0.0031738281250000, 0.0022888183593750, 0.0031433105468750, 0.0021972656250000,
+0.0032653808593750, 0.0019836425781250, 0.0033569335937500, 0.0024108886718750,
+0.0032653808593750, 0.0021362304687500, 0.0033569335937500, 0.0024414062500000,
+0.0032348632812500, 0.0029907226562500, 0.0030822753906250, 0.0026550292968750,
+0.0033874511718750, 0.0036926269531250, 0.0028991699218750, 0.0035095214843750,
+0.0030517578125000, 0.0036621093750000, 0.0029602050781250, 0.0045166015625000,
+0.0026245117187500, 0.0041503906250000, 0.0028381347656250, 0.0043029785156250,
+0.0025329589843750, 0.0046081542968750, 0.0025329589843750, 0.0039672851562500,
+0.0025634765625000, 0.0036010742187500, 0.0023803710937500, 0.0039672851562500,
+0.0025329589843750, 0.0024719238281250, 0.0025939941406250, 0.0026855468750000,
+0.0025024414062500, 0.0024414062500000, 0.0027465820312500, 0.0014038085937500,
+0.0029907226562500, 0.0019226074218750, 0.0028076171875000, 0.0014343261718750,
+0.0033264160156250, 0.0013427734375000, 0.0032653808593750, 0.0014953613281250,
+0.0032958984375000, 0.0014343261718750, 0.0035400390625000, 0.0014648437500000,
+0.0033874511718750, 0.0012817382812500, 0.0032958984375000, 0.0013732910156250,
+0.0034484863281250, 0.0011291503906250, 0.0032653808593750, 0.0008544921875000,
+0.0030822753906250, 0.0010375976562500, 0.0031738281250000, 0.0006103515625000,
+0.0030517578125000, 0.0005187988281250, 0.0030822753906250, 0.0007324218750000,
+0.0030517578125000, 0.0006713867187500, 0.0029602050781250, 0.0006713867187500,
+0.0030822753906250, 0.0011901855468750, 0.0029296875000000, 0.0011596679687500,
+0.0028991699218750, 0.0014038085937500, 0.0030822753906250, 0.0018005371093750,
+0.0029907226562500, 0.0017089843750000, 0.0029602050781250, 0.0021667480468750,
+0.0035095214843750, 0.0021972656250000, 0.0032958984375000, 0.0024414062500000,
+0.0035705566406250, 0.0028076171875000, 0.0042114257812500, 0.0026855468750000,
+0.0038146972656250, 0.0034790039062500, 0.0043334960937500, 0.0033264160156250,
+0.0043945312500000, 0.0037231445312500, 0.0042114257812500, 0.0045776367187500,
+0.0043945312500000, 0.0044250488281250, 0.0043334960937500, 0.0049743652343750,
+0.0040588378906250, 0.0052490234375000, 0.0040893554687500, 0.0050659179687500,
+0.0039672851562500, 0.0050964355468750, 0.0037231445312500, 0.0052795410156250,
+0.0036621093750000, 0.0048828125000000, 0.0039367675781250, 0.0049438476562500,
+0.0036926269531250, 0.0049133300781250, 0.0038452148437500, 0.0045471191406250,
+0.0041503906250000, 0.0043640136718750, 0.0039062500000000, 0.0043640136718750,
+0.0041198730468750, 0.0042724609375000, 0.0041198730468750, 0.0043640136718750,
+0.0039978027343750, 0.0044860839843750, 0.0041198730468750, 0.0043945312500000,
+0.0041198730468750, 0.0043640136718750, 0.0038452148437500, 0.0046386718750000,
+0.0038757324218750, 0.0045776367187500, 0.0039367675781250, 0.0044250488281250,
+0.0037231445312500, 0.0046691894531250, 0.0037841796875000, 0.0040283203125000,
+0.0043029785156250, 0.0041503906250000, 0.0040588378906250, 0.0036926269531250,
+0.0044860839843750, 0.0028991699218750, 0.0051879882812500, 0.0032043457031250,
+0.0047912597656250, 0.0023498535156250, 0.0056152343750000, 0.0021362304687500,
+0.0059509277343750, 0.0022888183593750, 0.0055541992187500, 0.0020446777343750,
+0.0055847167968750, 0.0019836425781250, 0.0058898925781250, 0.0025939941406250,
+0.0049438476562500, 0.0023193359375000, 0.0051574707031250, 0.0025329589843750,
+0.0049133300781250, 0.0030212402343750, 0.0041503906250000, 0.0026855468750000,
+0.0045776367187500, 0.0032348632812500, 0.0041198730468750, 0.0032348632812500,
+0.0041198730468750, 0.0030212402343750, 0.0041503906250000, 0.0031738281250000,
+0.0040893554687500, 0.0030212402343750, 0.0041503906250000, 0.0026245117187500,
+0.0040283203125000, 0.0025939941406250, 0.0041503906250000, 0.0025329589843750,
+0.0040283203125000, 0.0021972656250000, 0.0037841796875000, 0.0021667480468750,
+0.0039978027343750, 0.0023803710937500, 0.0035705566406250, 0.0021667480468750,
+0.0035400390625000, 0.0023193359375000, 0.0036315917968750, 0.0026245117187500,
+0.0034484863281250, 0.0024414062500000, 0.0035400390625000, 0.0025939941406250,
+0.0036315917968750, 0.0027465820312500, 0.0036315917968750, 0.0024719238281250,
+0.0035400390625000, 0.0022888183593750, 0.0036010742187500, 0.0025939941406250,
+0.0036621093750000, 0.0019531250000000, 0.0031433105468750, 0.0020141601562500,
+0.0033874511718750, 0.0020446777343750, 0.0029602050781250, 0.0016479492187500,
+0.0023498535156250, 0.0017089843750000, 0.0027160644531250, 0.0018920898437500,
+0.0018615722656250, 0.0017395019531250, 0.0017089843750000, 0.0018005371093750,
+0.0017700195312500, 0.0018920898437500, 0.0014343261718750, 0.0017089843750000,
+0.0014343261718750, 0.0018920898437500, 0.0014648437500000, 0.0017700195312500,
+0.0014648437500000, 0.0018920898437500, 0.0013122558593750, 0.0019531250000000,
+0.0011596679687500, 0.0016174316406250, 0.0012512207031250, 0.0021972656250000,
+0.0009460449218750, 0.0017700195312500, 0.0009155273437500, 0.0019531250000000,
+0.0009460449218750, 0.0024719238281250, 0.0008239746093750, 0.0017700195312500,
+0.0009155273437500, 0.0026245117187500, 0.0012207031250000, 0.0024414062500000,
+0.0011596679687500, 0.0022888183593750, 0.0014648437500000, 0.0027160644531250,
+0.0018920898437500, 0.0023803710937500, 0.0018615722656250, 0.0024414062500000,
+0.0024414062500000, 0.0024719238281250, 0.0025329589843750, 0.0024108886718750,
+0.0026550292968750, 0.0023193359375000, 0.0029907226562500, 0.0024108886718750,
+0.0029602050781250, 0.0023803710937500, 0.0030212402343750, 0.0024414062500000,
+0.0030822753906250, 0.0024719238281250, 0.0030822753906250, 0.0025329589843750,
+0.0030212402343750, 0.0025634765625000, 0.0031127929687500, 0.0025329589843750,
+0.0032653808593750, 0.0026245117187500, 0.0031738281250000, 0.0026550292968750,
+0.0033569335937500, 0.0026245117187500, 0.0036315917968750, 0.0026855468750000,
+0.0035095214843750, 0.0025329589843750, 0.0037536621093750, 0.0026550292968750,
+0.0038452148437500, 0.0026245117187500, 0.0037841796875000, 0.0025024414062500,
+0.0038757324218750, 0.0026245117187500, 0.0039672851562500, 0.0025634765625000,
+0.0035400390625000, 0.0026550292968750, 0.0037536621093750, 0.0027160644531250,
+0.0035095214843750, 0.0027465820312500, 0.0030822753906250, 0.0029296875000000,
+0.0033264160156250, 0.0029296875000000, 0.0027160644531250, 0.0030212402343750,
+0.0027770996093750, 0.0030822753906250, 0.0026245117187500, 0.0030822753906250,
+0.0021972656250000, 0.0031433105468750, 0.0023803710937500, 0.0032958984375000,
+0.0019226074218750, 0.0032653808593750, 0.0018920898437500, 0.0033264160156250,
+0.0018005371093750, 0.0034179687500000, 0.0015258789062500, 0.0032958984375000,
+0.0016174316406250, 0.0035095214843750, 0.0015563964843750, 0.0034179687500000,
+0.0013732910156250},
+{0.0033874511718750, 0.0023498535156250, 0.0035400390625000, 0.0027160644531250,
+0.0034790039062500, 0.0030212402343750, 0.0029907226562500, 0.0025024414062500,
+0.0031433105468750, 0.0031738281250000, 0.0029296875000000, 0.0031738281250000,
+0.0027770996093750, 0.0028991699218750, 0.0028686523437500, 0.0031127929687500,
+0.0029296875000000, 0.0031127929687500, 0.0027770996093750, 0.0026855468750000,
+0.0029296875000000, 0.0027770996093750, 0.0030517578125000, 0.0028076171875000,
+0.0028686523437500, 0.0025024414062500, 0.0028686523437500, 0.0026245117187500,
+0.0030517578125000, 0.0028991699218750, 0.0024719238281250, 0.0026855468750000,
+0.0026855468750000, 0.0029296875000000, 0.0025024414062500, 0.0034179687500000,
+0.0019226074218750, 0.0031127929687500, 0.0022277832031250, 0.0034484863281250,
+0.0018310546875000, 0.0037231445312500, 0.0018615722656250, 0.0034790039062500,
+0.0018005371093750, 0.0034179687500000, 0.0016174316406250, 0.0036621093750000,
+0.0017700195312500, 0.0030517578125000, 0.0016174316406250, 0.0031433105468750,
+0.0018310546875000, 0.0030822753906250, 0.0016784667968750, 0.0026550292968750,
+0.0015258789062500, 0.0028381347656250, 0.0018615722656250, 0.0025634765625000,
+0.0014038085937500, 0.0025024414062500, 0.0015258789062500, 0.0024719238281250,
+0.0017700195312500, 0.0023803710937500, 0.0016174316406250, 0.0023498535156250,
+0.0018310546875000, 0.0022583007812500, 0.0020751953125000, 0.0021972656250000,
+0.0020446777343750, 0.0022277832031250, 0.0021362304687500, 0.0021972656250000,
+0.0024414062500000, 0.0021057128906250, 0.0024108886718750, 0.0022888183593750,
+0.0021972656250000, 0.0021362304687500, 0.0024108886718750, 0.0022583007812500,
+0.0021362304687500, 0.0023803710937500, 0.0017395019531250, 0.0021972656250000,
+0.0021057128906250, 0.0025024414062500, 0.0013122558593750, 0.0025024414062500,
+0.0014038085937500, 0.0024414062500000, 0.0013122558593750, 0.0025329589843750,
+0.0007324218750000, 0.0025634765625000, 0.0009765625000000, 0.0024108886718750,
+0.0009460449218750, 0.0025634765625000, 0.0008239746093750, 0.0025634765625000,
+0.0011596679687500, 0.0024719238281250, 0.0014343261718750, 0.0025939941406250,
+0.0012817382812500, 0.0027465820312500, 0.0019226074218750, 0.0027770996093750,
+0.0017700195312500, 0.0029602050781250, 0.0019836425781250, 0.0031433105468750,
+0.0024414062500000, 0.0031433105468750, 0.0021057128906250, 0.0033569335937500,
+0.0026550292968750, 0.0034179687500000, 0.0025024414062500, 0.0034179687500000,
+0.0025329589843750, 0.0034790039062500, 0.0028991699218750, 0.0034179687500000,
+0.0025939941406250, 0.0032958984375000, 0.0028686523437500, 0.0032348632812500,
+0.0029296875000000, 0.0032348632812500, 0.0027465820312500, 0.0031127929687500,
+0.0027160644531250, 0.0030517578125000, 0.0027770996093750, 0.0030517578125000,
+0.0025024414062500, 0.0029602050781250, 0.0026245117187500, 0.0029602050781250,
+0.0026245117187500, 0.0030212402343750, 0.0024719238281250, 0.0029907226562500,
+0.0026245117187500, 0.0031433105468750, 0.0026550292968750, 0.0030822753906250,
+0.0025939941406250, 0.0031738281250000, 0.0026855468750000, 0.0032653808593750,
+0.0028076171875000, 0.0030822753906250, 0.0027160644531250, 0.0033569335937500,
+0.0028076171875000, 0.0032043457031250, 0.0028076171875000, 0.0032958984375000,
+0.0027770996093750, 0.0034179687500000, 0.0028076171875000, 0.0032348632812500,
+0.0028686523437500, 0.0034484863281250, 0.0027465820312500, 0.0034179687500000,
+0.0027770996093750, 0.0033569335937500, 0.0027770996093750, 0.0034790039062500,
+0.0027465820312500, 0.0033874511718750, 0.0028076171875000, 0.0033569335937500,
+0.0028381347656250, 0.0033264160156250, 0.0028686523437500, 0.0032653808593750,
+0.0028991699218750, 0.0031738281250000, 0.0028991699218750, 0.0032043457031250,
+0.0030212402343750, 0.0031433105468750, 0.0026855468750000, 0.0031127929687500,
+0.0029296875000000, 0.0031127929687500, 0.0028076171875000, 0.0031738281250000,
+0.0024108886718750, 0.0031127929687500, 0.0027160644531250, 0.0029907226562500,
+0.0021057128906250, 0.0031127929687500, 0.0021667480468750, 0.0030212402343750,
+0.0021362304687500, 0.0028991699218750, 0.0017700195312500, 0.0029907226562500,
+0.0018005371093750, 0.0028686523437500, 0.0017700195312500, 0.0028686523437500,
+0.0016174316406250, 0.0029907226562500, 0.0016784667968750, 0.0029907226562500,
+0.0017395019531250, 0.0030212402343750, 0.0016479492187500, 0.0031738281250000,
+0.0020446777343750, 0.0033264160156250, 0.0019531250000000, 0.0032958984375000,
+0.0018920898437500, 0.0034484863281250, 0.0021362304687500, 0.0036010742187500,
+0.0021057128906250, 0.0033569335937500, 0.0016784667968750, 0.0036621093750000,
+0.0018920898437500, 0.0034484863281250, 0.0016174316406250, 0.0030822753906250,
+0.0011291503906250, 0.0035095214843750, 0.0013122558593750, 0.0027465820312500,
+0.0011291503906250, 0.0028991699218750, 0.0009460449218750, 0.0029907226562500,
+0.0013427734375000, 0.0025329589843750, 0.0016479492187500, 0.0027770996093750,
+0.0013732910156250, 0.0032653808593750, 0.0027160644531250, 0.0030822753906250,
+0.0024108886718750, 0.0033874511718750, 0.0029296875000000, 0.0039367675781250,
+0.0041198730468750, 0.0036315917968750, 0.0035705566406250, 0.0039062500000000,
+0.0043334960937500, 0.0041809082031250, 0.0046386718750000, 0.0039672851562500,
+0.0043640136718750, 0.0038757324218750, 0.0045166015625000, 0.0041809082031250,
+0.0046997070312500, 0.0037231445312500, 0.0041503906250000, 0.0036926269531250,
+0.0042724609375000, 0.0038452148437500, 0.0041809082031250, 0.0036315917968750,
+0.0038757324218750, 0.0035400390625000, 0.0040283203125000, 0.0040283203125000,
+0.0034484863281250, 0.0038452148437500, 0.0036315917968750, 0.0039672851562500,
+0.0031127929687500, 0.0043640136718750, 0.0023498535156250, 0.0041809082031250,
+0.0028686523437500, 0.0044555664062500, 0.0015563964843750, 0.0045166015625000,
+0.0014953613281250, 0.0044250488281250, 0.0018005371093750, 0.0044555664062500,
+0.0013122558593750, 0.0044250488281250, 0.0012207031250000, 0.0045471191406250,
+0.0021667480468750, 0.0045471191406250, 0.0018615722656250, 0.0045471191406250,
+0.0020751953125000, 0.0046081542968750, 0.0032958984375000, 0.0039978027343750,
+0.0014343261718750, 0.0038452148437500, 0.0011291503906250, 0.0038452148437500,
+0.0007934570312500, 0.0037231445312500, 0.0011901855468750, 0.0036315917968750,
+0.0014343261718750, 0.0036621093750000, 0.0010070800781250, 0.0034790039062500,
+0.0022277832031250, 0.0035400390625000, 0.0018310546875000, 0.0037536621093750,
+0.0024719238281250, 0.0037231445312500, 0.0037841796875000, 0.0037536621093750,
+0.0030212402343750, 0.0045776367187500, 0.0046081542968750, 0.0044555664062500,
+0.0047607421875000, 0.0049133300781250, 0.0046386718750000, 0.0057678222656250,
+0.0053710937500000, 0.0054321289062500, 0.0052795410156250, 0.0061035156250000,
+0.0050659179687500, 0.0064697265625000, 0.0050659179687500, 0.0060424804687500,
+0.0049743652343750, 0.0061340332031250, 0.0048217773437500, 0.0066833496093750,
+0.0047912597656250, 0.0050354003906250, 0.0047302246093750, 0.0057067871093750,
+0.0047607421875000, 0.0052185058593750, 0.0044250488281250, 0.0038146972656250,
+0.0042419433593750, 0.0047912597656250, 0.0044250488281250, 0.0029602050781250,
+0.0031433105468750, 0.0032043457031250, 0.0033569335937500, 0.0028686523437500,
+0.0028991699218750, 0.0015869140625000, 0.0017089843750000, 0.0023803710937500,
+0.0021057128906250, 0.0011901855468750, 0.0015563964843750, 0.0010375976562500,
+0.0010986328125000, 0.0013427734375000, 0.0015869140625000, 0.0008850097656250,
+0.0017089843750000, 0.0008850097656250, 0.0012817382812500, 0.0018615722656250,
+0.0028991699218750, 0.0014953613281250, 0.0025329589843750, 0.0019531250000000,
+0.0031433105468750, 0.0029907226562500, 0.0046081542968750, 0.0023193359375000,
+0.0039672851562500, 0.0031738281250000, 0.0049743652343750, 0.0034179687500000,
+0.0054016113281250, 0.0030212402343750, 0.0049438476562500, 0.0030517578125000,
+0.0050048828125000, 0.0031433105468750, 0.0051879882812500, 0.0024108886718750,
+0.0040893554687500, 0.0026245117187500, 0.0042114257812500, 0.0023803710937500,
+0.0039367675781250, 0.0017089843750000, 0.0031127929687500, 0.0019836425781250,
+0.0034790039062500, 0.0019836425781250, 0.0032043457031250, 0.0016479492187500,
+0.0029907226562500, 0.0020446777343750, 0.0032958984375000, 0.0024414062500000,
+0.0035095214843750, 0.0021972656250000, 0.0032043457031250, 0.0031127929687500,
+0.0039062500000000, 0.0029296875000000, 0.0038757324218750, 0.0031127929687500,
+0.0039062500000000, 0.0037231445312500, 0.0044250488281250, 0.0033569335937500,
+0.0043640136718750, 0.0040283203125000, 0.0041198730468750, 0.0039367675781250,
+0.0043945312500000, 0.0039062500000000, 0.0041809082031250, 0.0042114257812500,
+0.0038452148437500, 0.0039672851562500, 0.0041503906250000, 0.0039978027343750,
+0.0034484863281250, 0.0040283203125000, 0.0036315917968750, 0.0038146972656250,
+0.0034790039062500, 0.0036010742187500, 0.0029907226562500, 0.0035705566406250,
+0.0034484863281250, 0.0032043457031250, 0.0028381347656250, 0.0031433105468750,
+0.0030517578125000, 0.0028686523437500, 0.0030212402343750, 0.0025939941406250,
+0.0026550292968750, 0.0027160644531250, 0.0031127929687500, 0.0019226074218750,
+0.0027465820312500, 0.0020446777343750, 0.0028991699218750, 0.0017700195312500,
+0.0029296875000000, 0.0012512207031250, 0.0027160644531250, 0.0014953613281250,
+0.0029296875000000, 0.0008239746093750, 0.0026855468750000, 0.0007934570312500,
+0.0027770996093750, 0.0008850097656250, 0.0027770996093750, 0.0005493164062500,
+0.0026245117187500, 0.0005798339843750, 0.0027465820312500, 0.0008850097656250,
+0.0024719238281250, 0.0006408691406250, 0.0025329589843750, 0.0008850097656250,
+0.0025329589843750, 0.0012207031250000, 0.0023498535156250, 0.0008544921875000,
+0.0025024414062500, 0.0015258789062500, 0.0025329589843750, 0.0013732910156250,
+0.0024414062500000, 0.0015258789062500, 0.0025939941406250, 0.0020446777343750,
+0.0026855468750000, 0.0017395019531250, 0.0026855468750000, 0.0023498535156250,
+0.0028381347656250, 0.0023498535156250, 0.0028381347656250, 0.0024719238281250,
+0.0028076171875000, 0.0029296875000000, 0.0027770996093750, 0.0028076171875000,
+0.0028076171875000, 0.0030517578125000, 0.0027770996093750, 0.0031433105468750,
+0.0027465820312500, 0.0031127929687500, 0.0027770996093750, 0.0032348632812500,
+0.0027770996093750, 0.0032348632812500, 0.0026855468750000, 0.0030212402343750,
+0.0028991699218750, 0.0030822753906250, 0.0028686523437500, 0.0028686523437500,
+0.0028991699218750, 0.0026245117187500, 0.0031433105468750, 0.0027465820312500,
+0.0030822753906250, 0.0022583007812500, 0.0031127929687500, 0.0023498535156250,
+0.0031738281250000, 0.0021667480468750, 0.0030822753906250, 0.0018615722656250,
+0.0030212402343750, 0.0020751953125000, 0.0031433105468750, 0.0017395019531250,
+0.0028381347656250, 0.0018005371093750, 0.0028991699218750, 0.0018615722656250,
+0.0028076171875000, 0.0015869140625000, 0.0026245117187500, 0.0018005371093750,
+0.0027160644531250, 0.0015869140625000, 0.0024719238281250, 0.0015563964843750,
+0.0025329589843750, 0.0015869140625000, 0.0025329589843750, 0.0015869140625000,
+0.0022888183593750},
+{0.0033874511718750, 0.0026245117187500, 0.0044555664062500, 0.0029602050781250,
+0.0040588378906250, 0.0029907226562500, 0.0047912597656250, 0.0027465820312500,
+0.0051574707031250, 0.0032348632812500, 0.0047607421875000, 0.0036621093750000,
+0.0048217773437500, 0.0032958984375000, 0.0051269531250000, 0.0042724609375000,
+0.0038757324218750, 0.0042419433593750, 0.0042114257812500, 0.0042724609375000,
+0.0038146972656250, 0.0047302246093750, 0.0026550292968750, 0.0045166015625000,
+0.0032958984375000, 0.0045166015625000, 0.0023803710937500, 0.0046386718750000,
+0.0022277832031250, 0.0045166015625000, 0.0025329589843750, 0.0043334960937500,
+0.0023498535156250, 0.0043334960937500, 0.0023803710937500, 0.0043029785156250,
+0.0028991699218750, 0.0042114257812500, 0.0028686523437500, 0.0041809082031250,
+0.0029602050781250, 0.0041809082031250, 0.0033264160156250, 0.0040283203125000,
+0.0032653808593750, 0.0040588378906250, 0.0031433105468750, 0.0039672851562500,
+0.0033569335937500, 0.0040893554687500, 0.0032653808593750, 0.0040893554687500,
+0.0030212402343750, 0.0039062500000000, 0.0031738281250000, 0.0042419433593750,
+0.0028686523437500, 0.0041198730468750, 0.0028381347656250, 0.0041809082031250,
+0.0028686523437500, 0.0042724609375000, 0.0026550292968750, 0.0040588378906250,
+0.0026245117187500, 0.0045776367187500, 0.0027160644531250, 0.0042724609375000,
+0.0025634765625000, 0.0045471191406250, 0.0026550292968750, 0.0050659179687500,
+0.0028381347656250, 0.0045776367187500, 0.0026245117187500, 0.0055236816406250,
+0.0029907226562500, 0.0052185058593750, 0.0028991699218750, 0.0053710937500000,
+0.0028991699218750, 0.0060119628906250, 0.0031127929687500, 0.0055236816406250,
+0.0029602050781250, 0.0057678222656250, 0.0031127929687500, 0.0058593750000000,
+0.0030212402343750, 0.0054626464843750, 0.0029907226562500, 0.0052795410156250,
+0.0030212402343750, 0.0053405761718750, 0.0029296875000000, 0.0045776367187500,
+0.0027770996093750, 0.0046386718750000, 0.0028076171875000, 0.0042114257812500,
+0.0027465820312500, 0.0035705566406250, 0.0026550292968750, 0.0039367675781250,
+0.0027160644531250, 0.0028686523437500, 0.0026550292968750, 0.0030212402343750,
+0.0026855468750000, 0.0028076171875000, 0.0027465820312500, 0.0021667480468750,
+0.0028076171875000, 0.0026245117187500, 0.0027465820312500, 0.0018310546875000,
+0.0030212402343750, 0.0018920898437500, 0.0029602050781250, 0.0018615722656250,
+0.0030212402343750, 0.0013122558593750, 0.0032348632812500, 0.0015258789062500,
+0.0031433105468750, 0.0013732910156250, 0.0033569335937500, 0.0012512207031250,
+0.0033264160156250, 0.0014953613281250, 0.0033264160156250, 0.0016174316406250,
+0.0033874511718750, 0.0013732910156250, 0.0033264160156250, 0.0019226074218750,
+0.0032348632812500, 0.0018310546875000, 0.0033569335937500, 0.0019836425781250,
+0.0032958984375000, 0.0023803710937500, 0.0031127929687500, 0.0022277832031250,
+0.0033264160156250, 0.0024414062500000, 0.0029296875000000, 0.0025939941406250,
+0.0031433105468750, 0.0024414062500000, 0.0029602050781250, 0.0024414062500000,
+0.0026550292968750, 0.0025939941406250, 0.0030212402343750, 0.0021057128906250,
+0.0026550292968750, 0.0023498535156250, 0.0026550292968750, 0.0020751953125000,
+0.0027770996093750, 0.0016479492187500, 0.0026550292968750, 0.0021057128906250,
+0.0026855468750000, 0.0013122558593750, 0.0028381347656250, 0.0013732910156250,
+0.0028686523437500, 0.0014038085937500, 0.0028076171875000, 0.0010681152343750,
+0.0028991699218750, 0.0013122558593750, 0.0030212402343750, 0.0012817382812500,
+0.0028686523437500, 0.0013122558593750, 0.0029602050781250, 0.0014038085937500,
+0.0028991699218750, 0.0014648437500000, 0.0027465820312500, 0.0015258789062500,
+0.0029296875000000, 0.0015869140625000, 0.0026550292968750, 0.0015563964843750,
+0.0027160644531250, 0.0016784667968750, 0.0026550292968750, 0.0016784667968750,
+0.0024414062500000, 0.0015563964843750, 0.0026245117187500, 0.0021667480468750,
+0.0023498535156250, 0.0019226074218750, 0.0023803710937500, 0.0023193359375000,
+0.0025024414062500, 0.0030212402343750, 0.0023803710937500, 0.0025939941406250,
+0.0024414062500000, 0.0035705566406250, 0.0027770996093750, 0.0036315917968750,
+0.0026550292968750, 0.0035400390625000, 0.0029602050781250, 0.0039367675781250,
+0.0033874511718750, 0.0038146972656250, 0.0031433105468750, 0.0032958984375000,
+0.0036621093750000, 0.0034484863281250, 0.0036926269531250, 0.0030822753906250,
+0.0036926269531250, 0.0024719238281250, 0.0039672851562500, 0.0025634765625000,
+0.0038757324218750, 0.0021362304687500, 0.0038452148437500, 0.0018920898437500,
+0.0037841796875000, 0.0018920898437500, 0.0036926269531250, 0.0017700195312500,
+0.0036926269531250, 0.0017395019531250, 0.0036010742187500, 0.0016784667968750,
+0.0036315917968750, 0.0016784667968750, 0.0036315917968750, 0.0014343261718750,
+0.0036315917968750, 0.0012207031250000, 0.0035400390625000, 0.0014648437500000,
+0.0034790039062500, 0.0010986328125000, 0.0033874511718750, 0.0009460449218750,
+0.0033264160156250, 0.0014648437500000, 0.0033874511718750, 0.0016784667968750,
+0.0033569335937500, 0.0014648437500000, 0.0032043457031250, 0.0029602050781250,
+0.0037841796875000, 0.0029602050781250, 0.0036926269531250, 0.0032653808593750,
+0.0038146972656250, 0.0044555664062500, 0.0043334960937500, 0.0041809082031250,
+0.0041809082031250, 0.0042419433593750, 0.0045471191406250, 0.0047607421875000,
+0.0046997070312500, 0.0043945312500000, 0.0046081542968750, 0.0035705566406250,
+0.0047302246093750, 0.0033874511718750, 0.0048828125000000, 0.0023193359375000,
+0.0043945312500000, 0.0025634765625000, 0.0047302246093750, 0.0025329589843750,
+0.0045166015625000, 0.0018310546875000, 0.0039978027343750, 0.0022583007812500,
+0.0044555664062500, 0.0020751953125000, 0.0035400390625000, 0.0019531250000000,
+0.0036315917968750, 0.0022277832031250, 0.0036315917968750, 0.0023803710937500,
+0.0030517578125000, 0.0022583007812500, 0.0033874511718750, 0.0032043457031250,
+0.0031738281250000, 0.0030517578125000, 0.0032043457031250, 0.0033264160156250,
+0.0032348632812500, 0.0040588378906250, 0.0031738281250000, 0.0039062500000000,
+0.0032958984375000, 0.0042724609375000, 0.0029907226562500, 0.0044860839843750,
+0.0031127929687500, 0.0041809082031250, 0.0029907226562500, 0.0039367675781250,
+0.0026245117187500, 0.0040283203125000, 0.0028381347656250, 0.0038146972656250,
+0.0028076171875000, 0.0036315917968750, 0.0026550292968750, 0.0036010742187500,
+0.0030212402343750, 0.0035400390625000, 0.0033569335937500, 0.0035400390625000,
+0.0032043457031250, 0.0034179687500000, 0.0038452148437500, 0.0035400390625000,
+0.0039978027343750, 0.0034179687500000, 0.0040283203125000, 0.0033569335937500,
+0.0044250488281250, 0.0036315917968750, 0.0044860839843750, 0.0030212402343750,
+0.0042724609375000, 0.0032348632812500, 0.0045776367187500, 0.0032653808593750,
+0.0043945312500000, 0.0028381347656250, 0.0039978027343750, 0.0030822753906250,
+0.0042724609375000, 0.0030822753906250, 0.0038452148437500, 0.0029296875000000,
+0.0039062500000000, 0.0031738281250000, 0.0038757324218750, 0.0033874511718750,
+0.0036010742187500, 0.0032653808593750, 0.0037536621093750, 0.0037231445312500,
+0.0036621093750000, 0.0037536621093750, 0.0037231445312500, 0.0037841796875000,
+0.0037231445312500, 0.0041198730468750, 0.0037841796875000, 0.0041198730468750,
+0.0039062500000000, 0.0038146972656250, 0.0039672851562500, 0.0040893554687500,
+0.0039978027343750, 0.0038452148437500, 0.0041503906250000, 0.0033569335937500,
+0.0042724609375000, 0.0037841796875000, 0.0043029785156250, 0.0034179687500000,
+0.0044555664062500, 0.0034179687500000, 0.0046081542968750, 0.0036010742187500,
+0.0044860839843750, 0.0035095214843750, 0.0045166015625000, 0.0036315917968750,
+0.0046691894531250, 0.0038452148437500, 0.0040588378906250, 0.0039672851562500,
+0.0041809082031250, 0.0039367675781250, 0.0040283203125000, 0.0040893554687500,
+0.0035400390625000, 0.0043029785156250, 0.0037536621093750, 0.0038146972656250,
+0.0035705566406250, 0.0042114257812500, 0.0034179687500000, 0.0039672851562500,
+0.0037536621093750, 0.0034484863281250, 0.0039062500000000, 0.0040893554687500,
+0.0037536621093750, 0.0031127929687500, 0.0044250488281250, 0.0032958984375000,
+0.0044555664062500, 0.0032958984375000, 0.0045776367187500, 0.0026550292968750,
+0.0049438476562500, 0.0031127929687500, 0.0048828125000000, 0.0026550292968750,
+0.0050659179687500, 0.0026855468750000, 0.0051269531250000, 0.0027465820312500,
+0.0050048828125000, 0.0026550292968750, 0.0050354003906250, 0.0027465820312500,
+0.0050354003906250, 0.0027160644531250, 0.0046997070312500, 0.0027770996093750,
+0.0046691894531250, 0.0028381347656250, 0.0045166015625000, 0.0028381347656250,
+0.0041809082031250, 0.0029296875000000, 0.0041809082031250, 0.0029907226562500,
+0.0038757324218750, 0.0030212402343750, 0.0038757324218750, 0.0029907226562500,
+0.0037231445312500, 0.0030517578125000, 0.0034790039062500, 0.0030517578125000,
+0.0035095214843750, 0.0029907226562500, 0.0031127929687500, 0.0030212402343750,
+0.0032653808593750, 0.0029296875000000, 0.0028991699218750, 0.0028686523437500,
+0.0024414062500000, 0.0029907226562500, 0.0028076171875000, 0.0028381347656250,
+0.0018615722656250, 0.0028686523437500, 0.0020141601562500, 0.0029602050781250,
+0.0018615722656250, 0.0028686523437500, 0.0012817382812500, 0.0030212402343750,
+0.0016174316406250, 0.0030212402343750, 0.0013732910156250, 0.0030517578125000,
+0.0011901855468750, 0.0030822753906250, 0.0014038085937500, 0.0031433105468750,
+0.0014953613281250, 0.0032043457031250, 0.0013732910156250, 0.0030822753906250,
+0.0020141601562500, 0.0031738281250000, 0.0018615722656250, 0.0031127929687500,
+0.0021057128906250, 0.0029907226562500, 0.0025939941406250, 0.0032043457031250,
+0.0022583007812500, 0.0028381347656250, 0.0029296875000000, 0.0029602050781250,
+0.0028686523437500, 0.0029602050781250, 0.0028991699218750, 0.0028076171875000,
+0.0032958984375000, 0.0029907226562500, 0.0031433105468750, 0.0028076171875000,
+0.0031433105468750, 0.0028686523437500, 0.0032348632812500, 0.0029296875000000,
+0.0031127929687500, 0.0028381347656250, 0.0029907226562500, 0.0029907226562500,
+0.0030517578125000, 0.0028991699218750, 0.0027160644531250, 0.0030212402343750,
+0.0027770996093750, 0.0029907226562500, 0.0026550292968750, 0.0029602050781250,
+0.0024414062500000, 0.0031127929687500, 0.0025634765625000, 0.0029907226562500,
+0.0022583007812500, 0.0030822753906250, 0.0022583007812500, 0.0032043457031250,
+0.0021972656250000, 0.0031738281250000, 0.0020141601562500, 0.0032348632812500,
+0.0021362304687500, 0.0032958984375000, 0.0020446777343750, 0.0033569335937500,
+0.0020141601562500, 0.0033874511718750, 0.0021972656250000, 0.0034790039062500,
+0.0023193359375000, 0.0035705566406250, 0.0021362304687500, 0.0033569335937500,
+0.0025939941406250, 0.0035095214843750, 0.0024414062500000, 0.0034179687500000,
+0.0026245117187500, 0.0031738281250000, 0.0031127929687500, 0.0034179687500000,
+0.0028686523437500, 0.0032348632812500, 0.0031127929687500, 0.0031127929687500,
+0.0032348632812500, 0.0032348632812500, 0.0029907226562500, 0.0032348632812500,
+0.0028991699218750, 0.0031433105468750, 0.0030212402343750, 0.0036010742187500,
+0.0025024414062500},
+{0.0036010742187500, 0.0031127929687500, 0.0040588378906250, 0.0031433105468750,
+0.0041198730468750, 0.0032043457031250, 0.0041503906250000, 0.0034790039062500,
+0.0043945312500000, 0.0034179687500000, 0.0042724609375000, 0.0034484863281250,
+0.0042724609375000, 0.0035705566406250, 0.0043029785156250, 0.0034179687500000,
+0.0042419433593750, 0.0032348632812500, 0.0040893554687500, 0.0032958984375000,
+0.0039978027343750, 0.0030212402343750, 0.0041198730468750, 0.0029296875000000,
+0.0040283203125000, 0.0028991699218750, 0.0039978027343750, 0.0027160644531250,
+0.0040588378906250, 0.0026245117187500, 0.0040283203125000, 0.0025634765625000,
+0.0042114257812500, 0.0025024414062500, 0.0042114257812500, 0.0024108886718750,
+0.0043945312500000, 0.0022888183593750, 0.0046386718750000, 0.0022888183593750,
+0.0045776367187500, 0.0021667480468750, 0.0048828125000000, 0.0020751953125000,
+0.0049743652343750, 0.0021057128906250, 0.0050048828125000, 0.0020446777343750,
+0.0051269531250000, 0.0020141601562500, 0.0051574707031250, 0.0022277832031250,
+0.0050964355468750, 0.0021362304687500, 0.0051574707031250, 0.0022277832031250,
+0.0049438476562500, 0.0024414062500000, 0.0048522949218750, 0.0022888183593750,
+0.0049438476562500, 0.0025024414062500, 0.0044250488281250, 0.0025024414062500,
+0.0044860839843750, 0.0025329589843750, 0.0041809082031250, 0.0026550292968750,
+0.0036621093750000, 0.0025024414062500, 0.0039672851562500, 0.0026855468750000,
+0.0030822753906250, 0.0026855468750000, 0.0032348632812500, 0.0026550292968750,
+0.0030822753906250, 0.0028076171875000, 0.0025024414062500, 0.0027465820312500,
+0.0028076171875000, 0.0027465820312500, 0.0024414062500000, 0.0027770996093750,
+0.0023498535156250, 0.0027770996093750, 0.0024719238281250, 0.0028076171875000,
+0.0024108886718750, 0.0029296875000000, 0.0023498535156250, 0.0028991699218750,
+0.0025329589843750, 0.0030212402343750, 0.0025329589843750, 0.0031127929687500,
+0.0025634765625000, 0.0031127929687500, 0.0026855468750000, 0.0032043457031250,
+0.0025939941406250, 0.0032653808593750, 0.0028076171875000, 0.0032958984375000,
+0.0027465820312500, 0.0033874511718750, 0.0026245117187500, 0.0034484863281250,
+0.0025939941406250, 0.0035705566406250, 0.0025939941406250, 0.0036621093750000,
+0.0024108886718750, 0.0037231445312500, 0.0023193359375000, 0.0038146972656250,
+0.0022888183593750, 0.0038757324218750, 0.0021667480468750, 0.0038757324218750,
+0.0021362304687500, 0.0039367675781250, 0.0021057128906250, 0.0041198730468750,
+0.0021667480468750, 0.0039672851562500, 0.0020751953125000, 0.0038146972656250,
+0.0019531250000000, 0.0039978027343750, 0.0020446777343750, 0.0033874511718750,
+0.0018920898437500, 0.0035400390625000, 0.0019531250000000, 0.0034179687500000,
+0.0017700195312500, 0.0029907226562500, 0.0015563964843750, 0.0032348632812500,
+0.0017395019531250, 0.0029602050781250, 0.0014953613281250, 0.0028991699218750,
+0.0014343261718750, 0.0030212402343750, 0.0015563964843750, 0.0029602050781250,
+0.0014343261718750, 0.0029296875000000, 0.0014953613281250, 0.0031738281250000,
+0.0018005371093750, 0.0031127929687500, 0.0018310546875000, 0.0031738281250000,
+0.0021057128906250, 0.0034484863281250, 0.0025329589843750, 0.0032958984375000,
+0.0024414062500000, 0.0034790039062500, 0.0025939941406250, 0.0034790039062500,
+0.0029296875000000, 0.0034790039062500, 0.0025939941406250, 0.0035095214843750,
+0.0023498535156250, 0.0033874511718750, 0.0027465820312500, 0.0036315917968750,
+0.0017395019531250, 0.0034179687500000, 0.0018310546875000, 0.0036315917968750,
+0.0017700195312500, 0.0039672851562500, 0.0010986328125000, 0.0035400390625000,
+0.0013732910156250, 0.0043334960937500, 0.0014648437500000, 0.0041809082031250,
+0.0011901855468750, 0.0041198730468750, 0.0017395019531250, 0.0045471191406250,
+0.0021972656250000, 0.0043334960937500, 0.0018310546875000, 0.0045166015625000,
+0.0031738281250000, 0.0044555664062500, 0.0031127929687500, 0.0044250488281250,
+0.0030212402343750, 0.0045166015625000, 0.0037536621093750, 0.0043029785156250,
+0.0034484863281250, 0.0046386718750000, 0.0030212402343750, 0.0044860839843750,
+0.0032348632812500, 0.0045166015625000, 0.0029296875000000, 0.0047607421875000,
+0.0023193359375000, 0.0046386718750000, 0.0026855468750000, 0.0048522949218750,
+0.0022583007812500, 0.0048217773437500, 0.0022888183593750, 0.0048217773437500,
+0.0022277832031250, 0.0048217773437500, 0.0021972656250000, 0.0048217773437500,
+0.0023803710937500, 0.0047607421875000, 0.0014343261718750, 0.0047912597656250,
+0.0018615722656250, 0.0046691894531250, 0.0015258789062500, 0.0046386718750000,
+0.0004882812500000, 0.0046386718750000, 0.0010681152343750, 0.0045166015625000,
+0.0011901855468750, 0.0043945312500000, 0.0006103515625000, 0.0045471191406250,
+0.0015258789062500, 0.0047607421875000, 0.0033874511718750, 0.0054626464843750,
+0.0023193359375000, 0.0046997070312500, 0.0008544921875000, 0.0056152343750000,
+0.0008239746093750, 0.0045471191406250, 0.0013427734375000, 0.0034179687500000,
+0.0013122558593750, 0.0043640136718750, 0.0010375976562500, 0.0024719238281250,
+0.0023193359375000, 0.0024719238281250, 0.0019226074218750, 0.0026245117187500,
+0.0025634765625000, 0.0017395019531250, 0.0036926269531250, 0.0020446777343750,
+0.0028686523437500, 0.0020751953125000, 0.0050048828125000, 0.0021057128906250,
+0.0047912597656250, 0.0021972656250000, 0.0047912597656250, 0.0023498535156250,
+0.0062866210937500, 0.0024719238281250, 0.0057983398437500, 0.0026855468750000,
+0.0050964355468750, 0.0027465820312500, 0.0058288574218750, 0.0028686523437500,
+0.0047607421875000, 0.0030517578125000, 0.0033874511718750, 0.0031433105468750,
+0.0043945312500000, 0.0033264160156250, 0.0022277832031250, 0.0033569335937500,
+0.0021667480468750, 0.0031738281250000, 0.0022277832031250, 0.0031433105468750,
+0.0011291503906250, 0.0032348632812500, 0.0014038085937500, 0.0025024414062500,
+0.0016479492187500, 0.0026550292968750, 0.0014038085937500, 0.0024719238281250,
+0.0018005371093750, 0.0017700195312500, 0.0023498535156250, 0.0021057128906250,
+0.0021362304687500, 0.0019531250000000, 0.0026855468750000, 0.0017089843750000,
+0.0028076171875000, 0.0021667480468750, 0.0029296875000000, 0.0025024414062500,
+0.0031433105468750, 0.0022583007812500, 0.0030822753906250, 0.0032653808593750,
+0.0031738281250000, 0.0031738281250000, 0.0031738281250000, 0.0033874511718750,
+0.0032958984375000, 0.0041809082031250, 0.0034179687500000, 0.0038452148437500,
+0.0033264160156250, 0.0040588378906250, 0.0033874511718750, 0.0043640136718750,
+0.0033569335937500, 0.0041198730468750, 0.0031127929687500, 0.0039978027343750,
+0.0029296875000000, 0.0042114257812500, 0.0028991699218750, 0.0034484863281250,
+0.0022583007812500, 0.0035705566406250, 0.0023193359375000, 0.0035400390625000,
+0.0018920898437500, 0.0029907226562500, 0.0012512207031250, 0.0032348632812500,
+0.0015258789062500, 0.0029296875000000, 0.0011901855468750, 0.0029602050781250,
+0.0007934570312500, 0.0028381347656250, 0.0013427734375000, 0.0026855468750000,
+0.0016479492187500, 0.0028381347656250, 0.0012817382812500, 0.0024108886718750,
+0.0023803710937500, 0.0025634765625000, 0.0022888183593750, 0.0023803710937500,
+0.0024108886718750, 0.0020446777343750, 0.0031433105468750, 0.0023803710937500,
+0.0028381347656250, 0.0021057128906250, 0.0028991699218750, 0.0019836425781250,
+0.0031738281250000, 0.0022583007812500, 0.0028686523437500, 0.0022888183593750,
+0.0026245117187500, 0.0021667480468750, 0.0027770996093750, 0.0029602050781250,
+0.0021667480468750, 0.0026550292968750, 0.0021972656250000, 0.0030517578125000,
+0.0022277832031250, 0.0037841796875000, 0.0018920898437500, 0.0032958984375000,
+0.0020141601562500, 0.0042419433593750, 0.0022888183593750, 0.0041809082031250,
+0.0021057128906250, 0.0041198730468750, 0.0023193359375000, 0.0046691894531250,
+0.0027160644531250, 0.0045166015625000, 0.0025024414062500, 0.0044250488281250,
+0.0029602050781250, 0.0046081542968750, 0.0030212402343750, 0.0044250488281250,
+0.0029602050781250, 0.0042419433593750, 0.0032043457031250, 0.0044250488281250,
+0.0032043457031250, 0.0038757324218750, 0.0029907226562500, 0.0041503906250000,
+0.0032043457031250, 0.0039672851562500, 0.0030517578125000, 0.0034484863281250,
+0.0028381347656250, 0.0038452148437500, 0.0031127929687500, 0.0034484863281250,
+0.0027465820312500, 0.0034484863281250, 0.0027770996093750, 0.0036315917968750,
+0.0028686523437500, 0.0036315917968750, 0.0028076171875000, 0.0037841796875000,
+0.0028686523437500, 0.0040588378906250, 0.0030822753906250, 0.0041198730468750,
+0.0030822753906250, 0.0043029785156250, 0.0031433105468750, 0.0046691894531250,
+0.0033569335937500, 0.0046691894531250, 0.0033264160156250, 0.0047912597656250,
+0.0033874511718750, 0.0049438476562500, 0.0035400390625000, 0.0048217773437500,
+0.0034484863281250, 0.0046997070312500, 0.0033874511718750, 0.0048217773437500,
+0.0036926269531250, 0.0043334960937500, 0.0032958984375000, 0.0045166015625000,
+0.0035400390625000, 0.0041503906250000, 0.0034484863281250, 0.0036010742187500,
+0.0031433105468750, 0.0039367675781250, 0.0035705566406250, 0.0029907226562500,
+0.0030212402343750, 0.0030517578125000, 0.0031433105468750, 0.0028686523437500,
+0.0031738281250000, 0.0022583007812500, 0.0027465820312500, 0.0025024414062500,
+0.0031127929687500, 0.0021057128906250, 0.0028076171875000, 0.0019226074218750,
+0.0028076171875000, 0.0021972656250000, 0.0028381347656250, 0.0021667480468750,
+0.0027770996093750, 0.0019836425781250, 0.0027465820312500, 0.0028076171875000,
+0.0028076171875000, 0.0025024414062500, 0.0028076171875000, 0.0030822753906250,
+0.0027770996093750, 0.0039672851562500, 0.0028076171875000, 0.0032348632812500,
+0.0028076171875000, 0.0047607421875000, 0.0027770996093750, 0.0045166015625000,
+0.0028381347656250, 0.0046997070312500, 0.0028991699218750, 0.0056762695312500,
+0.0028381347656250, 0.0050354003906250, 0.0028686523437500, 0.0059204101562500,
+0.0030517578125000, 0.0058288574218750, 0.0029907226562500, 0.0057678222656250,
+0.0029907226562500, 0.0061340332031250, 0.0030517578125000, 0.0057983398437500,
+0.0029907226562500, 0.0058593750000000, 0.0028686523437500, 0.0059204101562500,
+0.0028991699218750, 0.0056762695312500, 0.0028381347656250, 0.0054931640625000,
+0.0027465820312500, 0.0055541992187500, 0.0028381347656250, 0.0050659179687500,
+0.0028686523437500, 0.0051574707031250, 0.0027770996093750, 0.0048828125000000,
+0.0030517578125000, 0.0045471191406250, 0.0032653808593750, 0.0047912597656250,
+0.0031433105468750, 0.0041503906250000, 0.0036926269531250, 0.0042724609375000,
+0.0037231445312500, 0.0041198730468750, 0.0037231445312500, 0.0037231445312500,
+0.0039672851562500, 0.0040283203125000, 0.0039672851562500, 0.0033264160156250,
+0.0038452148437500, 0.0035400390625000, 0.0038452148437500, 0.0032653808593750,
+0.0037841796875000, 0.0027465820312500, 0.0036315917968750, 0.0031738281250000,
+0.0036010742187500, 0.0024414062500000, 0.0036010742187500, 0.0026245117187500,
+0.0035095214843750, 0.0026550292968750, 0.0034484863281250, 0.0022888183593750,
+0.0034484863281250, 0.0025329589843750, 0.0034484863281250, 0.0021057128906250,
+0.0032958984375000, 0.0022888183593750, 0.0032653808593750, 0.0021057128906250,
+0.0032958984375000},
+{0.0035095214843750, 0.0017395019531250, 0.0035400390625000, 0.0009460449218750,
+0.0036010742187500, 0.0013427734375000, 0.0036010742187500, 0.0004882812500000,
+0.0036621093750000, 0.0003967285156250, 0.0036315917968750, 0.0004272460937500,
+0.0036621093750000, -0.0000305175781250, 0.0036926269531250, 0.0000305175781250,
+0.0036315917968750, 0.0000915527343750, 0.0036926269531250, -0.0000305175781250,
+0.0036010742187500, 0.0000610351562500, 0.0036621093750000, 0.0001525878906250,
+0.0037231445312500, 0.0000915527343750, 0.0035705566406250, 0.0003356933593750,
+0.0036315917968750, 0.0002136230468750, 0.0036010742187500, 0.0003967285156250,
+0.0035705566406250, 0.0006713867187500, 0.0036621093750000, 0.0005493164062500,
+0.0036926269531250, 0.0012817382812500, 0.0036621093750000, 0.0010681152343750,
+0.0038146972656250, 0.0015563964843750, 0.0037231445312500, 0.0023193359375000,
+0.0036926269531250, 0.0018310546875000, 0.0039062500000000, 0.0032043457031250,
+0.0036315917968750, 0.0030212402343750, 0.0038757324218750, 0.0034790039062500,
+0.0037841796875000, 0.0046691894531250, 0.0035095214843750, 0.0042114257812500,
+0.0038146972656250, 0.0057067871093750, 0.0036010742187500, 0.0056457519531250,
+0.0035705566406250, 0.0060119628906250, 0.0036926269531250, 0.0070190429687500,
+0.0036315917968750, 0.0065612792968750, 0.0036315917968750, 0.0072937011718750,
+0.0037231445312500, 0.0073547363281250, 0.0036926269531250, 0.0070800781250000,
+0.0036926269531250, 0.0071716308593750, 0.0037841796875000, 0.0071411132812500,
+0.0036926269531250, 0.0065612792968750, 0.0036315917968750, 0.0066223144531250,
+0.0036926269531250, 0.0063171386718750, 0.0035705566406250, 0.0057067871093750,
+0.0034484863281250, 0.0060424804687500, 0.0035705566406250, 0.0056152343750000,
+0.0033874511718750, 0.0055236816406250, 0.0033874511718750, 0.0056152343750000,
+0.0033874511718750, 0.0055541992187500, 0.0033569335937500, 0.0055847167968750,
+0.0033874511718750, 0.0056762695312500, 0.0032043457031250, 0.0057373046875000,
+0.0032043457031250, 0.0056762695312500, 0.0030517578125000, 0.0056457519531250,
+0.0028991699218750, 0.0057373046875000, 0.0028686523437500, 0.0053100585937500,
+0.0029602050781250, 0.0055541992187500, 0.0028076171875000, 0.0052185058593750,
+0.0029296875000000, 0.0047912597656250, 0.0031738281250000, 0.0051879882812500,
+0.0029296875000000, 0.0043945312500000, 0.0034790039062500, 0.0046081542968750,
+0.0034484863281250, 0.0043334960937500, 0.0035095214843750, 0.0037841796875000,
+0.0038757324218750, 0.0043334960937500, 0.0038757324218750, 0.0033874511718750,
+0.0037231445312500, 0.0036621093750000, 0.0039062500000000, 0.0034484863281250,
+0.0038452148437500, 0.0027770996093750, 0.0036315917968750, 0.0033264160156250,
+0.0037841796875000, 0.0027465820312500, 0.0036315917968750, 0.0028076171875000,
+0.0036315917968750, 0.0029296875000000, 0.0036621093750000, 0.0026245117187500,
+0.0035705566406250, 0.0028686523437500, 0.0036010742187500, 0.0028076171875000,
+0.0035400390625000, 0.0028686523437500, 0.0034790039062500, 0.0030212402343750,
+0.0034790039062500, 0.0031127929687500, 0.0033874511718750, 0.0031127929687500,
+0.0033569335937500, 0.0034179687500000, 0.0036315917968750, 0.0033569335937500,
+0.0035095214843750, 0.0033264160156250, 0.0036315917968750, 0.0034484863281250,
+0.0039672851562500, 0.0033874511718750, 0.0036926269531250, 0.0030822753906250,
+0.0040588378906250, 0.0031738281250000, 0.0040588378906250, 0.0028381347656250,
+0.0040283203125000, 0.0023803710937500, 0.0040893554687500, 0.0024108886718750,
+0.0039978027343750, 0.0014953613281250, 0.0040283203125000, 0.0015563964843750,
+0.0040588378906250, 0.0013122558593750, 0.0039367675781250, 0.0006103515625000,
+0.0038757324218750, 0.0007934570312500, 0.0039978027343750, 0.0005187988281250,
+0.0036315917968750, 0.0001220703125000, 0.0036621093750000, 0.0007324218750000,
+0.0036315917968750, 0.0011596679687500, 0.0034179687500000, 0.0006713867187500,
+0.0034790039062500, 0.0025024414062500, 0.0034484863281250, 0.0022277832031250,
+0.0034484863281250, 0.0023803710937500, 0.0034484863281250, 0.0039062500000000,
+0.0034179687500000, 0.0038146972656250, 0.0034179687500000, 0.0034179687500000,
+0.0031738281250000, 0.0037231445312500, 0.0033264160156250, 0.0035095214843750,
+0.0032653808593750, 0.0029296875000000, 0.0031433105468750, 0.0031433105468750,
+0.0033874511718750, 0.0025634765625000, 0.0034484863281250, 0.0024719238281250,
+0.0033569335937500, 0.0022888183593750, 0.0035705566406250, 0.0019531250000000,
+0.0038146972656250, 0.0021362304687500, 0.0036926269531250, 0.0019226074218750,
+0.0040283203125000, 0.0019226074218750, 0.0041503906250000, 0.0022277832031250,
+0.0039672851562500, 0.0024108886718750, 0.0039367675781250, 0.0025024414062500,
+0.0039978027343750, 0.0029907226562500, 0.0032653808593750, 0.0032348632812500,
+0.0035095214843750, 0.0030212402343750, 0.0031433105468750, 0.0031433105468750,
+0.0024108886718750, 0.0036315917968750, 0.0027160644531250, 0.0028686523437500,
+0.0021057128906250, 0.0033264160156250, 0.0018615722656250, 0.0032043457031250,
+0.0019531250000000, 0.0025329589843750, 0.0017700195312500, 0.0031127929687500,
+0.0017089843750000, 0.0026245117187500, 0.0019836425781250, 0.0025634765625000,
+0.0017700195312500, 0.0027465820312500, 0.0019531250000000, 0.0026245117187500,
+0.0023803710937500, 0.0026245117187500, 0.0021362304687500, 0.0028381347656250,
+0.0023803710937500, 0.0027770996093750, 0.0025024414062500, 0.0028076171875000,
+0.0022583007812500, 0.0029907226562500, 0.0021667480468750, 0.0029907226562500,
+0.0022583007812500, 0.0030517578125000, 0.0017089843750000, 0.0030822753906250,
+0.0018920898437500, 0.0031433105468750, 0.0017395019531250, 0.0031433105468750,
+0.0011901855468750, 0.0030517578125000, 0.0014953613281250, 0.0034179687500000,
+0.0010681152343750, 0.0031738281250000, 0.0010070800781250, 0.0033569335937500,
+0.0011291503906250, 0.0038452148437500, 0.0010375976562500, 0.0034179687500000,
+0.0010375976562500, 0.0036926269531250, 0.0012207031250000, 0.0037231445312500,
+0.0012512207031250, 0.0035095214843750, 0.0014038085937500, 0.0034484863281250,
+0.0016174316406250, 0.0034179687500000, 0.0015563964843750, 0.0032653808593750,
+0.0019531250000000, 0.0030822753906250, 0.0020446777343750, 0.0031433105468750,
+0.0021362304687500, 0.0030822753906250, 0.0022888183593750, 0.0028991699218750,
+0.0022888183593750, 0.0034179687500000, 0.0023498535156250, 0.0030517578125000,
+0.0023803710937500, 0.0032043457031250, 0.0023498535156250, 0.0037231445312500,
+0.0023803710937500, 0.0031738281250000, 0.0023193359375000, 0.0037536621093750,
+0.0023498535156250, 0.0036315917968750, 0.0021972656250000, 0.0035400390625000,
+0.0022583007812500, 0.0037536621093750, 0.0023193359375000, 0.0034790039062500,
+0.0021972656250000, 0.0035400390625000, 0.0022888183593750, 0.0033874511718750,
+0.0021972656250000, 0.0034484863281250, 0.0021362304687500, 0.0035705566406250,
+0.0021362304687500, 0.0032653808593750, 0.0020751953125000, 0.0038452148437500,
+0.0020446777343750, 0.0036621093750000, 0.0020446777343750, 0.0036926269531250,
+0.0019531250000000, 0.0040893554687500, 0.0018920898437500, 0.0038452148437500,
+0.0018615722656250, 0.0040893554687500, 0.0017700195312500, 0.0042419433593750,
+0.0018005371093750, 0.0041503906250000, 0.0017700195312500, 0.0041198730468750,
+0.0016479492187500, 0.0043029785156250, 0.0016479492187500, 0.0040588378906250,
+0.0018920898437500, 0.0041503906250000, 0.0017700195312500, 0.0040893554687500,
+0.0019836425781250, 0.0039367675781250, 0.0023498535156250, 0.0040588378906250,
+0.0021057128906250, 0.0039062500000000, 0.0026855468750000, 0.0038452148437500,
+0.0026855468750000, 0.0039367675781250, 0.0028381347656250, 0.0038757324218750,
+0.0031433105468750, 0.0038146972656250, 0.0030212402343750, 0.0039062500000000,
+0.0033569335937500, 0.0038452148437500, 0.0032958984375000, 0.0038757324218750,
+0.0035400390625000, 0.0039672851562500, 0.0037536621093750, 0.0039062500000000,
+0.0035095214843750, 0.0040588378906250, 0.0043029785156250, 0.0040588378906250,
+0.0041198730468750, 0.0040588378906250, 0.0043945312500000, 0.0041503906250000,
+0.0050659179687500, 0.0041503906250000, 0.0046997070312500, 0.0042419433593750,
+0.0056152343750000, 0.0041809082031250, 0.0054626464843750, 0.0042114257812500,
+0.0054931640625000, 0.0042114257812500, 0.0060424804687500, 0.0040283203125000,
+0.0057678222656250, 0.0042419433593750, 0.0059204101562500, 0.0040283203125000,
+0.0059814453125000, 0.0040588378906250, 0.0059204101562500, 0.0041809082031250,
+0.0060119628906250, 0.0038757324218750, 0.0060424804687500, 0.0041809082031250,
+0.0059814453125000, 0.0039978027343750, 0.0060119628906250, 0.0038452148437500,
+0.0058898925781250, 0.0040588378906250, 0.0058288574218750, 0.0037536621093750,
+0.0058898925781250, 0.0038757324218750, 0.0057678222656250, 0.0037231445312500,
+0.0058288574218750, 0.0036926269531250, 0.0057983398437500, 0.0037231445312500,
+0.0057373046875000, 0.0034790039062500, 0.0057373046875000, 0.0038452148437500,
+0.0054016113281250, 0.0036315917968750, 0.0054626464843750, 0.0036926269531250,
+0.0053100585937500, 0.0039978027343750, 0.0050659179687500, 0.0036621093750000,
+0.0051879882812500, 0.0039672851562500, 0.0047302246093750, 0.0038452148437500,
+0.0047912597656250, 0.0038146972656250, 0.0046691894531250, 0.0039672851562500,
+0.0044250488281250, 0.0037231445312500, 0.0046081542968750, 0.0038452148437500,
+0.0041198730468750, 0.0037231445312500, 0.0041809082031250, 0.0036621093750000,
+0.0040283203125000, 0.0037841796875000, 0.0037231445312500, 0.0036010742187500,
+0.0039367675781250, 0.0039672851562500, 0.0035095214843750, 0.0037841796875000,
+0.0035095214843750, 0.0038757324218750, 0.0034179687500000, 0.0043029785156250,
+0.0032043457031250, 0.0040588378906250, 0.0033874511718750, 0.0043640136718750,
+0.0030212402343750, 0.0044860839843750, 0.0031127929687500, 0.0043640136718750,
+0.0031127929687500, 0.0044250488281250, 0.0028686523437500, 0.0045776367187500,
+0.0029907226562500, 0.0042419433593750, 0.0032653808593750, 0.0044250488281250,
+0.0030212402343750, 0.0042724609375000, 0.0034484863281250, 0.0039062500000000,
+0.0039062500000000, 0.0041809082031250, 0.0035705566406250, 0.0039672851562500,
+0.0046691894531250, 0.0039367675781250, 0.0045471191406250, 0.0041503906250000,
+0.0044250488281250, 0.0041198730468750, 0.0052185058593750, 0.0039978027343750,
+0.0050354003906250, 0.0040893554687500, 0.0041503906250000, 0.0039672851562500,
+0.0047607421875000, 0.0039672851562500, 0.0039672851562500, 0.0039978027343750,
+0.0027160644531250, 0.0039062500000000, 0.0035400390625000, 0.0039978027343750,
+0.0019531250000000, 0.0038757324218750, 0.0019531250000000, 0.0039367675781250,
+0.0020446777343750, 0.0040283203125000, 0.0013732910156250, 0.0038757324218750,
+0.0016784667968750, 0.0041198730468750, 0.0014648437500000, 0.0039062500000000,
+0.0016174316406250, 0.0041503906250000, 0.0013732910156250, 0.0044250488281250,
+0.0009765625000000, 0.0039672851562500, 0.0013427734375000, 0.0047302246093750,
+0.0006408691406250, 0.0046997070312500, 0.0004882812500000, 0.0044860839843750,
+0.0010070800781250, 0.0048828125000000, 0.0009765625000000, 0.0047607421875000,
+0.0005798339843750},
+{0.0046081542968750, 0.0044860839843750, 0.0042114257812500, 0.0046997070312500,
+0.0049743652343750, 0.0045776367187500, 0.0049133300781250, 0.0045471191406250,
+0.0050048828125000, 0.0045776367187500, 0.0055236816406250, 0.0044250488281250,
+0.0053405761718750, 0.0044555664062500, 0.0056152343750000, 0.0044250488281250,
+0.0057678222656250, 0.0043334960937500, 0.0055236816406250, 0.0043334960937500,
+0.0054626464843750, 0.0042419433593750, 0.0056152343750000, 0.0041198730468750,
+0.0047302246093750, 0.0040893554687500, 0.0050659179687500, 0.0039367675781250,
+0.0044860839843750, 0.0038452148437500, 0.0035400390625000, 0.0038452148437500,
+0.0040893554687500, 0.0036621093750000, 0.0028381347656250, 0.0035400390625000,
+0.0028381347656250, 0.0035400390625000, 0.0025024414062500, 0.0034179687500000,
+0.0017700195312500, 0.0033264160156250, 0.0022277832031250, 0.0035095214843750,
+0.0014343261718750, 0.0033569335937500, 0.0012817382812500, 0.0034484863281250,
+0.0014343261718750, 0.0036926269531250, 0.0010986328125000, 0.0034484863281250,
+0.0010681152343750, 0.0038452148437500, 0.0012817382812500, 0.0036926269531250,
+0.0011901855468750, 0.0037536621093750, 0.0013732910156250, 0.0039978027343750,
+0.0016174316406250, 0.0037536621093750, 0.0013732910156250, 0.0040283203125000,
+0.0019226074218750, 0.0039062500000000, 0.0017700195312500, 0.0039367675781250,
+0.0019836425781250, 0.0040893554687500, 0.0024108886718750, 0.0038757324218750,
+0.0020751953125000, 0.0043029785156250, 0.0028991699218750, 0.0040588378906250,
+0.0027770996093750, 0.0042724609375000, 0.0028686523437500, 0.0046997070312500,
+0.0034179687500000, 0.0043334960937500, 0.0032348632812500, 0.0047607421875000,
+0.0032958984375000, 0.0048522949218750, 0.0035400390625000, 0.0046081542968750,
+0.0033569335937500, 0.0046997070312500, 0.0031433105468750, 0.0047607421875000,
+0.0033874511718750, 0.0043334960937500, 0.0029296875000000, 0.0045166015625000,
+0.0029602050781250, 0.0044555664062500, 0.0029296875000000, 0.0041809082031250,
+0.0027160644531250, 0.0043029785156250, 0.0028076171875000, 0.0040893554687500,
+0.0023498535156250, 0.0041198730468750, 0.0023803710937500, 0.0041503906250000,
+0.0022277832031250, 0.0040893554687500, 0.0018310546875000, 0.0042419433593750,
+0.0021972656250000, 0.0042114257812500, 0.0019226074218750, 0.0042724609375000,
+0.0017089843750000, 0.0042419433593750, 0.0021667480468750, 0.0042114257812500,
+0.0024108886718750, 0.0041809082031250, 0.0021362304687500, 0.0040588378906250,
+0.0032958984375000, 0.0039978027343750, 0.0031738281250000, 0.0040283203125000,
+0.0032958984375000, 0.0040283203125000, 0.0040283203125000, 0.0041198730468750,
+0.0038146972656250, 0.0042419433593750, 0.0037841796875000, 0.0041198730468750,
+0.0040283203125000, 0.0043029785156250, 0.0037841796875000, 0.0045166015625000,
+0.0034790039062500, 0.0042419433593750, 0.0037536621093750, 0.0047302246093750,
+0.0036315917968750, 0.0046386718750000, 0.0034790039062500, 0.0044555664062500,
+0.0036926269531250, 0.0046997070312500, 0.0038452148437500, 0.0045166015625000,
+0.0038146972656250, 0.0038757324218750, 0.0037231445312500, 0.0040588378906250,
+0.0040588378906250, 0.0035705566406250, 0.0034484863281250, 0.0028076171875000,
+0.0029602050781250, 0.0031127929687500, 0.0034484863281250, 0.0023193359375000,
+0.0018005371093750, 0.0021972656250000, 0.0018005371093750, 0.0024108886718750,
+0.0019836425781250, 0.0022277832031250, 0.0010070800781250, 0.0039367675781250,
+0.0050354003906250, 0.0025939941406250, 0.0033874511718750, 0.0033569335937500,
+0.0041809082031250, 0.0030822753906250, 0.0034790039062500, 0.0021972656250000,
+0.0023193359375000, 0.0026855468750000, 0.0029296875000000, 0.0026855468750000,
+0.0022888183593750, 0.0025024414062500, 0.0021667480468750, 0.0027160644531250,
+0.0024719238281250, 0.0028991699218750, 0.0022888183593750, 0.0028076171875000,
+0.0022277832031250, 0.0028686523437500, 0.0031433105468750, 0.0028991699218750,
+0.0028991699218750, 0.0029296875000000, 0.0032958984375000, 0.0028991699218750,
+0.0042114257812500, 0.0027770996093750, 0.0038146972656250, 0.0030822753906250,
+0.0044250488281250, 0.0029296875000000, 0.0046691894531250, 0.0029907226562500,
+0.0042114257812500, 0.0032958984375000, 0.0042419433593750, 0.0030822753906250,
+0.0046691894531250, 0.0032348632812500, 0.0034484863281250, 0.0032653808593750,
+0.0036010742187500, 0.0030517578125000, 0.0036315917968750, 0.0029296875000000,
+0.0028991699218750, 0.0029602050781250, 0.0032043457031250, 0.0028076171875000,
+0.0037536621093750, 0.0026550292968750, 0.0032958984375000, 0.0027770996093750,
+0.0039978027343750, 0.0027770996093750, 0.0050659179687500, 0.0025939941406250,
+0.0044860839843750, 0.0032653808593750, 0.0054931640625000, 0.0030212402343750,
+0.0057678222656250, 0.0031433105468750, 0.0056457519531250, 0.0037536621093750,
+0.0059509277343750, 0.0033569335937500, 0.0059204101562500, 0.0036010742187500,
+0.0054016113281250, 0.0036010742187500, 0.0054931640625000, 0.0035095214843750,
+0.0052185058593750, 0.0034484863281250, 0.0047302246093750, 0.0034790039062500,
+0.0049438476562500, 0.0034179687500000, 0.0043640136718750, 0.0032958984375000,
+0.0042419433593750, 0.0035400390625000, 0.0043640136718750, 0.0036010742187500,
+0.0042114257812500, 0.0033874511718750, 0.0042114257812500, 0.0042419433593750,
+0.0044860839843750, 0.0039672851562500, 0.0043945312500000, 0.0043640136718750,
+0.0042419433593750, 0.0052185058593750, 0.0043029785156250, 0.0047302246093750,
+0.0042724609375000, 0.0057983398437500, 0.0041198730468750, 0.0057983398437500,
+0.0039978027343750, 0.0057983398437500, 0.0039367675781250, 0.0064086914062500,
+0.0039062500000000, 0.0061645507812500, 0.0039062500000000, 0.0057983398437500,
+0.0038452148437500, 0.0061645507812500, 0.0039062500000000, 0.0055541992187500,
+0.0038757324218750, 0.0047912597656250, 0.0039672851562500, 0.0053100585937500,
+0.0039978027343750, 0.0037841796875000, 0.0037536621093750, 0.0038452148437500,
+0.0039672851562500, 0.0036010742187500, 0.0036621093750000, 0.0025939941406250,
+0.0032653808593750, 0.0029602050781250, 0.0037231445312500, 0.0025329589843750,
+0.0028686523437500, 0.0024414062500000, 0.0031433105468750, 0.0024719238281250,
+0.0031127929687500, 0.0023803710937500, 0.0025024414062500, 0.0024414062500000,
+0.0029602050781250, 0.0023193359375000, 0.0026855468750000, 0.0023193359375000,
+0.0026245117187500, 0.0024108886718750, 0.0028381347656250, 0.0022888183593750,
+0.0028686523437500, 0.0022888183593750, 0.0028991699218750, 0.0027770996093750,
+0.0032043457031250, 0.0025939941406250, 0.0032348632812500, 0.0029296875000000,
+0.0033874511718750, 0.0034484863281250, 0.0035705566406250, 0.0031127929687500,
+0.0035400390625000, 0.0038452148437500, 0.0036315917968750, 0.0037231445312500,
+0.0036315917968750, 0.0038146972656250, 0.0036926269531250, 0.0041809082031250,
+0.0037231445312500, 0.0038452148437500, 0.0037231445312500, 0.0043029785156250,
+0.0037231445312500, 0.0041198730468750, 0.0037231445312500, 0.0039978027343750,
+0.0037841796875000, 0.0041809082031250, 0.0038452148437500, 0.0039672851562500,
+0.0038452148437500, 0.0037231445312500, 0.0040588378906250, 0.0037841796875000,
+0.0039978027343750, 0.0034179687500000, 0.0040283203125000, 0.0030517578125000,
+0.0041809082031250, 0.0032043457031250, 0.0040588378906250, 0.0023498535156250,
+0.0040588378906250, 0.0025024414062500, 0.0041198730468750, 0.0021667480468750,
+0.0040283203125000, 0.0014648437500000, 0.0039978027343750, 0.0019531250000000,
+0.0040893554687500, 0.0010375976562500, 0.0038757324218750, 0.0011291503906250,
+0.0039367675781250, 0.0010681152343750, 0.0039062500000000, 0.0005493164062500,
+0.0037231445312500, 0.0008239746093750, 0.0039367675781250, 0.0005493164062500,
+0.0035400390625000, 0.0004882812500000, 0.0036926269531250, 0.0007324218750000,
+0.0036010742187500, 0.0007629394531250, 0.0033569335937500, 0.0007629394531250,
+0.0036621093750000, 0.0013122558593750, 0.0032043457031250, 0.0011901855468750,
+0.0033874511718750, 0.0014953613281250, 0.0033569335937500, 0.0020141601562500,
+0.0030517578125000, 0.0018005371093750, 0.0032958984375000, 0.0025024414062500,
+0.0029602050781250, 0.0026245117187500, 0.0030517578125000, 0.0027160644531250,
+0.0029602050781250, 0.0031738281250000, 0.0026855468750000, 0.0031433105468750,
+0.0029296875000000, 0.0032043457031250, 0.0027770996093750, 0.0034790039062500,
+0.0026245117187500, 0.0032348632812500, 0.0028076171875000, 0.0030212402343750,
+0.0029907226562500, 0.0032958984375000, 0.0028686523437500, 0.0025024414062500,
+0.0032958984375000, 0.0026550292968750, 0.0032653808593750, 0.0024108886718750,
+0.0033569335937500, 0.0018310546875000, 0.0036926269531250, 0.0021972656250000,
+0.0036315917968750, 0.0016479492187500, 0.0037841796875000, 0.0016479492187500,
+0.0038452148437500, 0.0018005371093750, 0.0038452148437500, 0.0016479492187500,
+0.0039062500000000, 0.0017700195312500, 0.0038452148437500, 0.0021667480468750,
+0.0039062500000000, 0.0021362304687500, 0.0038452148437500, 0.0022583007812500,
+0.0038452148437500, 0.0025634765625000, 0.0038452148437500, 0.0025634765625000,
+0.0037231445312500, 0.0025634765625000, 0.0036926269531250, 0.0027465820312500,
+0.0036010742187500, 0.0027160644531250, 0.0035095214843750, 0.0026550292968750,
+0.0034179687500000, 0.0028381347656250, 0.0033569335937500, 0.0025634765625000,
+0.0034790039062500, 0.0026550292968750, 0.0033264160156250, 0.0026550292968750,
+0.0033874511718750, 0.0025024414062500, 0.0035400390625000, 0.0025634765625000,
+0.0033569335937500, 0.0024108886718750, 0.0034790039062500, 0.0024719238281250,
+0.0035095214843750, 0.0022277832031250, 0.0032958984375000, 0.0019836425781250,
+0.0032348632812500, 0.0022277832031250, 0.0033569335937500, 0.0017395019531250,
+0.0030822753906250, 0.0018310546875000, 0.0030517578125000, 0.0018310546875000,
+0.0031433105468750, 0.0014953613281250, 0.0030212402343750, 0.0015869140625000,
+0.0030212402343750, 0.0016479492187500, 0.0028991699218750, 0.0015258789062500,
+0.0029907226562500, 0.0017395019531250, 0.0029296875000000, 0.0019226074218750,
+0.0028076171875000, 0.0019226074218750, 0.0029602050781250, 0.0024414062500000,
+0.0027160644531250, 0.0023803710937500, 0.0027465820312500, 0.0028381347656250,
+0.0027770996093750, 0.0034790039062500, 0.0026855468750000, 0.0032958984375000,
+0.0026855468750000, 0.0043029785156250, 0.0026855468750000, 0.0045166015625000,
+0.0027465820312500, 0.0044555664062500, 0.0028381347656250, 0.0049438476562500,
+0.0028991699218750, 0.0050964355468750, 0.0029602050781250, 0.0043640136718750,
+0.0032348632812500, 0.0047302246093750, 0.0032653808593750, 0.0044555664062500,
+0.0033264160156250, 0.0032958984375000, 0.0036010742187500, 0.0030212402343750,
+0.0036926269531250, 0.0025329589843750, 0.0033569335937500, 0.0025939941406250,
+0.0035705566406250, 0.0022888183593750, 0.0033569335937500, 0.0018615722656250,
+0.0029907226562500, 0.0020141601562500, 0.0031738281250000, 0.0015869140625000,
+0.0025329589843750, 0.0015258789062500, 0.0025024414062500, 0.0017089843750000,
+0.0024414062500000, 0.0015869140625000, 0.0020141601562500, 0.0015258789062500,
+0.0021362304687500, 0.0025329589843750, 0.0019226074218750, 0.0021057128906250,
+0.0018005371093750},
+{0.0036010742187500, 0.0019226074218750, 0.0036315917968750, 0.0019531250000000,
+0.0036010742187500, 0.0020446777343750, 0.0034179687500000, 0.0019226074218750,
+0.0035400390625000, 0.0021057128906250, 0.0033874511718750, 0.0021057128906250,
+0.0034484863281250, 0.0020141601562500, 0.0035400390625000, 0.0021057128906250,
+0.0034790039062500, 0.0020751953125000, 0.0034790039062500, 0.0020141601562500,
+0.0036621093750000, 0.0020141601562500, 0.0037841796875000, 0.0019836425781250,
+0.0038146972656250, 0.0019226074218750, 0.0039062500000000, 0.0019836425781250,
+0.0039672851562500, 0.0018615722656250, 0.0038452148437500, 0.0019226074218750,
+0.0039672851562500, 0.0018005371093750, 0.0038452148437500, 0.0015869140625000,
+0.0037231445312500, 0.0017700195312500, 0.0038452148437500, 0.0013427734375000,
+0.0036621093750000, 0.0014953613281250, 0.0037231445312500, 0.0012817382812500,
+0.0036621093750000, 0.0008544921875000, 0.0035705566406250, 0.0011291503906250,
+0.0036010742187500, 0.0008239746093750, 0.0034484863281250, 0.0007629394531250,
+0.0034790039062500, 0.0010070800781250, 0.0034484863281250, 0.0009765625000000,
+0.0032653808593750, 0.0009155273437500, 0.0032958984375000, 0.0016174316406250,
+0.0033264160156250, 0.0015258789062500, 0.0032348632812500, 0.0017700195312500,
+0.0033264160156250, 0.0023803710937500, 0.0034484863281250, 0.0022277832031250,
+0.0033264160156250, 0.0026855468750000, 0.0036010742187500, 0.0028686523437500,
+0.0036010742187500, 0.0028686523437500, 0.0035400390625000, 0.0030517578125000,
+0.0036010742187500, 0.0032348632812500, 0.0036315917968750, 0.0030822753906250,
+0.0033874511718750, 0.0032043457031250, 0.0034484863281250, 0.0031738281250000,
+0.0033874511718750, 0.0030517578125000, 0.0032653808593750, 0.0032043457031250,
+0.0032958984375000, 0.0030822753906250, 0.0031738281250000, 0.0031127929687500,
+0.0032653808593750, 0.0031738281250000, 0.0031127929687500, 0.0031738281250000,
+0.0030212402343750, 0.0032043457031250, 0.0032958984375000, 0.0031738281250000,
+0.0027770996093750, 0.0032348632812500, 0.0029602050781250, 0.0032348632812500,
+0.0030212402343750, 0.0031738281250000, 0.0026550292968750, 0.0032348632812500,
+0.0029296875000000, 0.0030517578125000, 0.0026855468750000, 0.0030212402343750,
+0.0026855468750000, 0.0030212402343750, 0.0027465820312500, 0.0028686523437500,
+0.0026855468750000, 0.0028076171875000, 0.0027770996093750, 0.0028381347656250,
+0.0029296875000000, 0.0027160644531250, 0.0028991699218750, 0.0026245117187500,
+0.0029296875000000, 0.0026550292968750, 0.0031127929687500, 0.0026855468750000,
+0.0030822753906250, 0.0026550292968750, 0.0028076171875000, 0.0026855468750000,
+0.0030517578125000, 0.0027770996093750, 0.0027160644531250, 0.0028381347656250,
+0.0020751953125000, 0.0030822753906250, 0.0023498535156250, 0.0036621093750000,
+0.0019226074218750, 0.0033264160156250, 0.0018920898437500, 0.0033569335937500,
+0.0019531250000000, 0.0038146972656250, 0.0016174316406250, 0.0035400390625000,
+0.0014953613281250, 0.0031127929687500, 0.0018310546875000, 0.0032653808593750,
+0.0015258789062500, 0.0028686523437500, 0.0015869140625000, 0.0023803710937500,
+0.0019531250000000, 0.0026550292968750, 0.0016479492187500, 0.0018310546875000,
+0.0018310546875000, 0.0016479492187500, 0.0018005371093750, 0.0017089843750000,
+0.0016784667968750, 0.0012207031250000, 0.0017395019531250, 0.0011291503906250,
+0.0016174316406250, 0.0017395019531250, 0.0016784667968750, 0.0013122558593750,
+0.0015563964843750, 0.0018005371093750, 0.0017089843750000, 0.0025939941406250,
+0.0017395019531250, 0.0018920898437500, 0.0014953613281250, 0.0030212402343750,
+0.0021667480468750, 0.0031433105468750, 0.0019226074218750, 0.0028686523437500,
+0.0020751953125000, 0.0033264160156250, 0.0026245117187500, 0.0032348632812500,
+0.0022888183593750, 0.0029296875000000, 0.0027465820312500, 0.0031127929687500,
+0.0026550292968750, 0.0029296875000000, 0.0025939941406250, 0.0025939941406250,
+0.0028381347656250, 0.0029907226562500, 0.0026855468750000, 0.0026550292968750,
+0.0025939941406250, 0.0027160644531250, 0.0025024414062500, 0.0027465820312500,
+0.0025634765625000, 0.0026245117187500, 0.0025634765625000, 0.0028991699218750,
+0.0024108886718750, 0.0028076171875000, 0.0026245117187500, 0.0028686523437500,
+0.0025634765625000, 0.0028381347656250, 0.0025024414062500, 0.0028686523437500,
+0.0027465820312500, 0.0029907226562500, 0.0026855468750000, 0.0027465820312500,
+0.0023498535156250, 0.0028381347656250, 0.0025939941406250, 0.0028991699218750,
+0.0023803710937500, 0.0028381347656250, 0.0019836425781250, 0.0029296875000000,
+0.0022888183593750, 0.0030517578125000, 0.0016784667968750, 0.0030212402343750,
+0.0016174316406250, 0.0030822753906250, 0.0016479492187500, 0.0031738281250000,
+0.0013122558593750, 0.0031127929687500, 0.0013427734375000, 0.0032043457031250,
+0.0013427734375000, 0.0032348632812500, 0.0011596679687500, 0.0030822753906250,
+0.0012207031250000, 0.0029602050781250, 0.0013122558593750, 0.0031433105468750,
+0.0011901855468750, 0.0028991699218750, 0.0011901855468750, 0.0027770996093750,
+0.0012512207031250, 0.0030212402343750, 0.0010986328125000, 0.0030822753906250,
+0.0009155273437500, 0.0028686523437500, 0.0010070800781250, 0.0034179687500000,
+0.0009765625000000, 0.0033569335937500, 0.0007934570312500, 0.0033569335937500,
+0.0009765625000000, 0.0036926269531250, 0.0011291503906250, 0.0035705566406250,
+0.0009460449218750, 0.0035095214843750, 0.0016174316406250, 0.0036010742187500,
+0.0014953613281250, 0.0036010742187500, 0.0017700195312500, 0.0034484863281250,
+0.0022888183593750, 0.0035095214843750, 0.0019531250000000, 0.0036315917968750,
+0.0027770996093750, 0.0035705566406250, 0.0026245117187500, 0.0036315917968750,
+0.0028686523437500, 0.0038146972656250, 0.0034790039062500, 0.0037231445312500,
+0.0031738281250000, 0.0037841796875000, 0.0039978027343750, 0.0039672851562500,
+0.0039062500000000, 0.0038452148437500, 0.0041503906250000, 0.0038146972656250,
+0.0048217773437500, 0.0039978027343750, 0.0044555664062500, 0.0036315917968750,
+0.0052185058593750, 0.0037536621093750, 0.0052795410156250, 0.0037536621093750,
+0.0052185058593750, 0.0035705566406250, 0.0056762695312500, 0.0036926269531250,
+0.0056457519531250, 0.0038146972656250, 0.0052490234375000, 0.0036621093750000,
+0.0054931640625000, 0.0039062500000000, 0.0050354003906250, 0.0040893554687500,
+0.0044250488281250, 0.0038146972656250, 0.0048522949218750, 0.0042114257812500,
+0.0036315917968750, 0.0041809082031250, 0.0036926269531250, 0.0040893554687500,
+0.0034790039062500, 0.0042724609375000, 0.0026855468750000, 0.0041198730468750,
+0.0029296875000000, 0.0040893554687500, 0.0023193359375000, 0.0040588378906250,
+0.0022277832031250, 0.0039367675781250, 0.0022277832031250, 0.0037536621093750,
+0.0019836425781250, 0.0037536621093750, 0.0019531250000000, 0.0036010742187500,
+0.0018920898437500, 0.0035095214843750, 0.0018005371093750, 0.0034179687500000,
+0.0016174316406250, 0.0033569335937500, 0.0015563964843750, 0.0032958984375000,
+0.0015869140625000, 0.0032958984375000, 0.0013732910156250, 0.0032348632812500,
+0.0013732910156250, 0.0031433105468750, 0.0014038085937500, 0.0031433105468750,
+0.0013427734375000, 0.0032043457031250, 0.0014343261718750, 0.0029602050781250,
+0.0015258789062500, 0.0030517578125000, 0.0015258789062500, 0.0029296875000000,
+0.0016479492187500, 0.0027160644531250, 0.0017089843750000, 0.0029296875000000,
+0.0017089843750000, 0.0027770996093750, 0.0020446777343750, 0.0028381347656250,
+0.0018920898437500, 0.0029602050781250, 0.0020751953125000, 0.0029602050781250,
+0.0024414062500000, 0.0030822753906250, 0.0021667480468750, 0.0032348632812500,
+0.0024719238281250, 0.0033569335937500, 0.0025634765625000, 0.0034790039062500,
+0.0024414062500000, 0.0036010742187500, 0.0024108886718750, 0.0036010742187500,
+0.0024108886718750, 0.0038452148437500, 0.0024719238281250, 0.0039062500000000,
+0.0024108886718750, 0.0039978027343750, 0.0025329589843750, 0.0041503906250000,
+0.0026550292968750, 0.0042724609375000, 0.0025634765625000, 0.0042114257812500,
+0.0028076171875000, 0.0043334960937500, 0.0028076171875000, 0.0043029785156250,
+0.0028076171875000, 0.0041809082031250, 0.0029602050781250, 0.0043029785156250,
+0.0028381347656250, 0.0042724609375000, 0.0029907226562500, 0.0042114257812500,
+0.0028686523437500, 0.0042419433593750, 0.0031127929687500, 0.0043029785156250,
+0.0033264160156250, 0.0041809082031250, 0.0030517578125000, 0.0043029785156250,
+0.0038757324218750, 0.0042114257812500, 0.0036621093750000, 0.0041809082031250,
+0.0038452148437500, 0.0042419433593750, 0.0045166015625000, 0.0041198730468750,
+0.0040283203125000, 0.0043334960937500, 0.0044555664062500, 0.0042114257812500,
+0.0044250488281250, 0.0043334960937500, 0.0042724609375000, 0.0044860839843750,
+0.0043334960937500, 0.0043334960937500, 0.0041809082031250, 0.0044250488281250,
+0.0042724609375000, 0.0043945312500000, 0.0041198730468750, 0.0042724609375000,
+0.0041503906250000, 0.0042419433593750, 0.0041809082031250, 0.0042419433593750,
+0.0040283203125000, 0.0040283203125000, 0.0041503906250000, 0.0040893554687500,
+0.0040893554687500, 0.0039062500000000, 0.0040893554687500, 0.0036315917968750,
+0.0041809082031250, 0.0037536621093750, 0.0041198730468750, 0.0029602050781250,
+0.0041809082031250, 0.0028686523437500, 0.0042114257812500, 0.0027770996093750,
+0.0042419433593750, 0.0023498535156250, 0.0043029785156250, 0.0024108886718750,
+0.0042419433593750, 0.0027465820312500, 0.0043640136718750, 0.0024719238281250,
+0.0044860839843750, 0.0028381347656250, 0.0043029785156250, 0.0035095214843750,
+0.0041503906250000, 0.0031127929687500, 0.0043334960937500, 0.0036621093750000,
+0.0036621093750000, 0.0040283203125000, 0.0039062500000000, 0.0037231445312500,
+0.0035400390625000, 0.0037536621093750, 0.0028381347656250, 0.0041809082031250,
+0.0033874511718750, 0.0032958984375000, 0.0028076171875000, 0.0035095214843750,
+0.0026245117187500, 0.0035705566406250, 0.0031738281250000, 0.0027770996093750,
+0.0036315917968750, 0.0034484863281250, 0.0041503906250000, 0.0032653808593750,
+0.0021057128906250, 0.0036010742187500, 0.0024108886718750, 0.0032958984375000,
+0.0024414062500000, 0.0029296875000000, 0.0019531250000000, 0.0031738281250000,
+0.0019836425781250, 0.0028076171875000, 0.0024414062500000, 0.0026855468750000,
+0.0022583007812500, 0.0028686523437500, 0.0025329589843750, 0.0027770996093750,
+0.0029907226562500, 0.0025634765625000, 0.0026550292968750, 0.0028381347656250,
+0.0032043457031250, 0.0027160644531250, 0.0031738281250000, 0.0027160644531250,
+0.0031127929687500, 0.0027770996093750, 0.0033264160156250, 0.0025024414062500,
+0.0032653808593750, 0.0027770996093750, 0.0028991699218750, 0.0026550292968750,
+0.0032043457031250, 0.0026245117187500, 0.0028381347656250, 0.0028686523437500,
+0.0023803710937500, 0.0027160644531250, 0.0028686523437500, 0.0024719238281250,
+0.0022583007812500, 0.0026550292968750, 0.0021362304687500, 0.0023193359375000,
+0.0023498535156250, 0.0018005371093750, 0.0021667480468750, 0.0021057128906250,
+0.0020446777343750, 0.0014038085937500, 0.0026855468750000, 0.0012512207031250,
+0.0026550292968750, 0.0013122558593750, 0.0026245117187500, 0.0009155273437500,
+0.0031127929687500},
+{0.0016479492187500, 0.0034790039062500, 0.0012207031250000, 0.0036315917968750,
+0.0015258789062500, 0.0036010742187500, 0.0012817382812500, 0.0035400390625000,
+0.0011596679687500, 0.0036010742187500, 0.0014343261718750, 0.0035400390625000,
+0.0016174316406250, 0.0036315917968750, 0.0014953613281250, 0.0035705566406250,
+0.0020751953125000, 0.0035705566406250, 0.0020751953125000, 0.0036621093750000,
+0.0021362304687500, 0.0037536621093750, 0.0024414062500000, 0.0035705566406250,
+0.0023193359375000, 0.0039062500000000, 0.0023803710937500, 0.0037841796875000,
+0.0022888183593750, 0.0039672851562500, 0.0024108886718750, 0.0043029785156250,
+0.0024414062500000, 0.0040283203125000, 0.0021362304687500, 0.0043945312500000,
+0.0027465820312500, 0.0044860839843750, 0.0025329589843750, 0.0043640136718750,
+0.0025329589843750, 0.0044250488281250, 0.0029296875000000, 0.0044250488281250,
+0.0026855468750000, 0.0041198730468750, 0.0028686523437500, 0.0042724609375000,
+0.0028076171875000, 0.0039062500000000, 0.0025939941406250, 0.0034790039062500,
+0.0025939941406250, 0.0037841796875000, 0.0025024414062500, 0.0030517578125000,
+0.0025329589843750, 0.0030212402343750, 0.0024414062500000, 0.0029907226562500,
+0.0023803710937500, 0.0025024414062500, 0.0023193359375000, 0.0025939941406250,
+0.0023193359375000, 0.0028076171875000, 0.0026245117187500, 0.0025024414062500,
+0.0024719238281250, 0.0027770996093750, 0.0025329589843750, 0.0032653808593750,
+0.0027160644531250, 0.0029907226562500, 0.0026550292968750, 0.0034179687500000,
+0.0031433105468750, 0.0036010742187500, 0.0029602050781250, 0.0032653808593750,
+0.0032958984375000, 0.0032043457031250, 0.0038146972656250, 0.0035705566406250,
+0.0036010742187500, 0.0027770996093750, 0.0045471191406250, 0.0028991699218750,
+0.0043334960937500, 0.0030212402343750, 0.0045776367187500, 0.0025634765625000,
+0.0054016113281250, 0.0028076171875000, 0.0050354003906250, 0.0030517578125000,
+0.0046997070312500, 0.0030212402343750, 0.0052490234375000, 0.0031433105468750,
+0.0045776367187500, 0.0035095214843750, 0.0028686523437500, 0.0029602050781250,
+0.0036926269531250, 0.0029907226562500, 0.0037841796875000, 0.0027465820312500,
+0.0044860839843750, 0.0029296875000000, 0.0037231445312500, 0.0031127929687500,
+0.0029296875000000, 0.0027160644531250, 0.0036010742187500, 0.0030822753906250,
+0.0020751953125000, 0.0030822753906250, 0.0023498535156250, 0.0027465820312500,
+0.0021667480468750, 0.0026855468750000, 0.0010986328125000, 0.0027465820312500,
+0.0016479492187500, 0.0021972656250000, 0.0010375976562500, 0.0023193359375000,
+0.0008850097656250, 0.0020751953125000, 0.0011596679687500, 0.0017089843750000,
+0.0009765625000000, 0.0020141601562500, 0.0009460449218750, 0.0012207031250000,
+0.0017700195312500, 0.0014038085937500, 0.0013732910156250, 0.0010681152343750,
+0.0018005371093750, 0.0002746582031250, 0.0027160644531250, 0.0007934570312500,
+0.0021972656250000, -0.0002746582031250, 0.0028381347656250, -0.0003356933593750,
+0.0030822753906250, -0.0001220703125000, 0.0026855468750000, -0.0005187988281250,
+0.0027770996093750, -0.0005187988281250, 0.0030212402343750, 0.0002136230468750,
+0.0021667480468750, -0.0000915527343750, 0.0022277832031250, 0.0003356933593750,
+0.0023498535156250, 0.0010986328125000, 0.0019836425781250, 0.0006713867187500,
+0.0021362304687500, 0.0015869140625000, 0.0021667480468750, 0.0016784667968750,
+0.0022583007812500, 0.0016174316406250, 0.0023193359375000, 0.0020141601562500,
+0.0024719238281250, 0.0018615722656250, 0.0027160644531250, 0.0018310546875000,
+0.0022888183593750, 0.0018920898437500, 0.0025634765625000, 0.0014953613281250,
+0.0023803710937500, 0.0011901855468750, 0.0019836425781250, 0.0015258789062500,
+0.0023803710937500, 0.0001525878906250, 0.0018920898437500, 0.0004577636718750,
+0.0018920898437500, 0.0000000000000000, 0.0021362304687500, -0.0011596679687500,
+0.0020446777343750, -0.0005187988281250, 0.0021057128906250, -0.0015563964843750,
+0.0025329589843750, -0.0016784667968750, 0.0024414062500000, -0.0014038085937500,
+0.0026855468750000, -0.0018005371093750, 0.0031127929687500, -0.0018920898437500,
+0.0028991699218750, -0.0012207031250000, 0.0034484863281250, -0.0015869140625000,
+0.0033264160156250, -0.0011596679687500, 0.0031738281250000, -0.0003967285156250,
+0.0035400390625000, -0.0009765625000000, 0.0033569335937500, 0.0004577636718750,
+0.0030517578125000, 0.0000305175781250, 0.0029907226562500, 0.0003662109375000,
+0.0031127929687500, 0.0015563964843750, 0.0030517578125000, 0.0006713867187500,
+0.0028991699218750, 0.0017700195312500, 0.0032043457031250, 0.0017395019531250,
+0.0031127929687500, 0.0016479492187500, 0.0030822753906250, 0.0021667480468750,
+0.0032043457031250, 0.0018615722656250, 0.0029907226562500, 0.0016174316406250,
+0.0031127929687500, 0.0018310546875000, 0.0030517578125000, 0.0012512207031250,
+0.0029296875000000, 0.0007019042968750, 0.0029296875000000, 0.0010986328125000,
+0.0028991699218750, -0.0002746582031250, 0.0028991699218750, -0.0001525878906250,
+0.0027465820312500, -0.0004272460937500, 0.0028076171875000, -0.0013427734375000,
+0.0029602050781250, -0.0009460449218750, 0.0028076171875000, -0.0012817382812500,
+0.0030212402343750, -0.0016174316406250, 0.0030212402343750, -0.0011901855468750,
+0.0029602050781250, -0.0010986328125000, 0.0029602050781250, -0.0014343261718750,
+0.0029602050781250, -0.0002441406250000, 0.0027160644531250, -0.0006408691406250,
+0.0028381347656250, -0.0001831054687500, 0.0027160644531250, 0.0008850097656250,
+0.0024719238281250, 0.0002136230468750, 0.0026245117187500, 0.0016479492187500,
+0.0025024414062500, 0.0015869140625000, 0.0024108886718750, 0.0016174316406250,
+0.0025939941406250, 0.0024108886718750, 0.0025634765625000, 0.0021972656250000,
+0.0024108886718750, 0.0023193359375000, 0.0030822753906250, 0.0025329589843750,
+0.0028381347656250, 0.0023498535156250, 0.0030212402343750, 0.0021972656250000,
+0.0035095214843750, 0.0023498535156250, 0.0031127929687500, 0.0017700195312500,
+0.0035400390625000, 0.0018310546875000, 0.0035705566406250, 0.0017089843750000,
+0.0034484863281250, 0.0013122558593750, 0.0036010742187500, 0.0014648437500000,
+0.0035705566406250, 0.0011901855468750, 0.0034484863281250, 0.0010986328125000,
+0.0035705566406250, 0.0011596679687500, 0.0035095214843750, 0.0010986328125000,
+0.0032958984375000, 0.0010986328125000, 0.0035095214843750, 0.0014648437500000,
+0.0033874511718750, 0.0012512207031250, 0.0032958984375000, 0.0014648437500000,
+0.0034484863281250, 0.0018310546875000, 0.0034790039062500, 0.0015258789062500,
+0.0032958984375000, 0.0019226074218750, 0.0036926269531250, 0.0018920898437500,
+0.0034790039062500, 0.0019226074218750, 0.0036010742187500, 0.0021057128906250,
+0.0039367675781250, 0.0019836425781250, 0.0036010742187500, 0.0022888183593750,
+0.0039672851562500, 0.0021972656250000, 0.0039978027343750, 0.0023498535156250,
+0.0039062500000000, 0.0026245117187500, 0.0039978027343750, 0.0023193359375000,
+0.0039978027343750, 0.0029296875000000, 0.0039978027343750, 0.0026855468750000,
+0.0038452148437500, 0.0028686523437500, 0.0038146972656250, 0.0033569335937500,
+0.0039367675781250, 0.0030212402343750, 0.0039672851562500, 0.0036621093750000,
+0.0039672851562500, 0.0035400390625000, 0.0039672851562500, 0.0036315917968750,
+0.0039672851562500, 0.0039978027343750, 0.0039062500000000, 0.0036621093750000,
+0.0038757324218750, 0.0039367675781250, 0.0038452148437500, 0.0038757324218750,
+0.0037841796875000, 0.0036621093750000, 0.0039062500000000, 0.0036315917968750,
+0.0039367675781250, 0.0034484863281250, 0.0038146972656250, 0.0033874511718750,
+0.0041503906250000, 0.0032958984375000, 0.0040893554687500, 0.0031127929687500,
+0.0040588378906250, 0.0030212402343750, 0.0042114257812500, 0.0029602050781250,
+0.0041809082031250, 0.0026550292968750, 0.0040283203125000, 0.0027465820312500,
+0.0040588378906250, 0.0025024414062500, 0.0039367675781250, 0.0020751953125000,
+0.0037536621093750, 0.0022888183593750, 0.0038146972656250, 0.0018005371093750,
+0.0035705566406250, 0.0017089843750000, 0.0035400390625000, 0.0017395019531250,
+0.0034484863281250, 0.0015258789062500, 0.0032348632812500, 0.0015563964843750,
+0.0032958984375000, 0.0015869140625000, 0.0030517578125000, 0.0015258789062500,
+0.0030822753906250, 0.0017089843750000, 0.0029602050781250, 0.0019226074218750,
+0.0027160644531250, 0.0017395019531250, 0.0028381347656250, 0.0020141601562500,
+0.0026855468750000, 0.0019836425781250, 0.0026855468750000, 0.0018005371093750,
+0.0028076171875000, 0.0019531250000000, 0.0027465820312500, 0.0018615722656250,
+0.0026855468750000, 0.0014038085937500, 0.0029602050781250, 0.0015563964843750,
+0.0028991699218750, 0.0011596679687500, 0.0029296875000000, 0.0007324218750000,
+0.0031738281250000, 0.0009460449218750, 0.0029907226562500, 0.0000000000000000,
+0.0030212402343750, 0.0002441406250000, 0.0031738281250000, 0.0000000000000000,
+0.0029296875000000, -0.0007019042968750, 0.0028076171875000, -0.0001831054687500,
+0.0029907226562500, -0.0008850097656250, 0.0024414062500000, -0.0007934570312500,
+0.0025329589843750, -0.0005187988281250, 0.0025024414062500, -0.0007019042968750,
+0.0021057128906250, -0.0005187988281250, 0.0022888183593750, 0.0001220703125000,
+0.0021667480468750, 0.0000305175781250, 0.0020751953125000, 0.0001525878906250,
+0.0020446777343750, 0.0009460449218750, 0.0019836425781250, 0.0014343261718750,
+0.0020751953125000, 0.0010375976562500, 0.0018920898437500, 0.0014038085937500,
+0.0019531250000000, 0.0012817382812500, 0.0019531250000000, 0.0007629394531250,
+0.0018920898437500, 0.0012207031250000, 0.0018920898437500, 0.0011291503906250,
+0.0019226074218750, 0.0008239746093750, 0.0018920898437500, 0.0010070800781250,
+0.0019836425781250, 0.0014038085937500, 0.0020751953125000, 0.0011901855468750,
+0.0019531250000000, 0.0011901855468750, 0.0021972656250000, 0.0014343261718750,
+0.0021362304687500, 0.0015563964843750, 0.0020141601562500, 0.0016174316406250,
+0.0020751953125000, 0.0018615722656250, 0.0019836425781250, 0.0021362304687500,
+0.0017089843750000, 0.0022277832031250, 0.0017700195312500, 0.0024108886718750,
+0.0014953613281250, 0.0025634765625000, 0.0010986328125000, 0.0026245117187500,
+0.0013122558593750, 0.0024719238281250, 0.0009460449218750, 0.0025939941406250,
+0.0007324218750000, 0.0024414062500000, 0.0009155273437500, 0.0020141601562500,
+0.0009155273437500, 0.0021362304687500, 0.0006713867187500, 0.0020751953125000,
+0.0011596679687500, 0.0018005371093750, 0.0010986328125000, 0.0021057128906250,
+0.0011291503906250, 0.0024108886718750, 0.0014953613281250, 0.0019226074218750,
+0.0013732910156250, 0.0026550292968750, 0.0012207031250000, 0.0025634765625000,
+0.0013732910156250, 0.0024719238281250, 0.0012817382812500, 0.0027770996093750,
+0.0010986328125000, 0.0025329589843750, 0.0011596679687500, 0.0025634765625000,
+0.0009765625000000, 0.0025939941406250, 0.0009155273437500, 0.0023803710937500,
+0.0010070800781250, 0.0021667480468750, 0.0009460449218750, 0.0021362304687500,
+0.0008239746093750, 0.0022888183593750, 0.0009460449218750, 0.0020751953125000,
+0.0009155273437500, 0.0022888183593750, 0.0008544921875000, 0.0026245117187500,
+0.0008239746093750, 0.0023498535156250, 0.0007934570312500, 0.0029602050781250,
+0.0007629394531250},
+{0.0036010742187500, 0.0016174316406250, 0.0034790039062500, 0.0016174316406250,
+0.0034179687500000, 0.0014953613281250, 0.0034179687500000, 0.0012207031250000,
+0.0033569335937500, 0.0012817382812500, 0.0032958984375000, 0.0011596679687500,
+0.0033569335937500, 0.0010986328125000, 0.0033569335937500, 0.0012817382812500,
+0.0032653808593750, 0.0014343261718750, 0.0033874511718750, 0.0012512207031250,
+0.0033264160156250, 0.0018005371093750, 0.0032958984375000, 0.0017089843750000,
+0.0033874511718750, 0.0019226074218750, 0.0033569335937500, 0.0024414062500000,
+0.0033264160156250, 0.0021667480468750, 0.0033569335937500, 0.0029296875000000,
+0.0031738281250000, 0.0028991699218750, 0.0032958984375000, 0.0029602050781250,
+0.0030822753906250, 0.0034790039062500, 0.0028381347656250, 0.0033569335937500,
+0.0031127929687500, 0.0031127929687500, 0.0026550292968750, 0.0036621093750000,
+0.0027160644531250, 0.0030822753906250, 0.0026550292968750, 0.0024108886718750,
+0.0024108886718750, 0.0031738281250000, 0.0026245117187500, 0.0016479492187500,
+0.0023803710937500, 0.0020751953125000, 0.0023498535156250, 0.0016784667968750,
+0.0023498535156250, 0.0005493164062500, 0.0021972656250000, 0.0014648437500000,
+0.0022888183593750, 0.0000610351562500, 0.0021057128906250, 0.0002136230468750,
+0.0020751953125000, 0.0002136230468750, 0.0021362304687500, -0.0008239746093750,
+0.0020751953125000, -0.0006713867187500, 0.0021362304687500, -0.0004577636718750,
+0.0022583007812500, -0.0006713867187500, 0.0022888183593750, -0.0005187988281250,
+0.0023803710937500, -0.0002136230468750, 0.0024719238281250, -0.0004272460937500,
+0.0025024414062500, 0.0000610351562500, 0.0024719238281250, 0.0000000000000000,
+0.0025634765625000, 0.0004577636718750, 0.0023803710937500, 0.0008544921875000,
+0.0021972656250000, 0.0003356933593750, 0.0023193359375000, 0.0012207031250000,
+0.0018310546875000, 0.0013427734375000, 0.0018005371093750, 0.0010070800781250,
+0.0016479492187500, 0.0011596679687500, 0.0012817382812500, 0.0012817382812500,
+0.0013732910156250, 0.0010375976562500, 0.0011596679687500, 0.0011596679687500,
+0.0010681152343750, 0.0014953613281250, 0.0010986328125000, 0.0014343261718750,
+0.0010070800781250, 0.0014038085937500, 0.0010070800781250, 0.0023193359375000,
+0.0011596679687500, 0.0022277832031250, 0.0012207031250000, 0.0024719238281250,
+0.0010681152343750, 0.0032348632812500, 0.0011291503906250, 0.0029907226562500,
+0.0012207031250000, 0.0031433105468750, 0.0010070800781250, 0.0033569335937500,
+0.0010375976562500, 0.0032043457031250, 0.0010375976562500, 0.0030517578125000,
+0.0008544921875000, 0.0031433105468750, 0.0008850097656250, 0.0032043457031250,
+0.0013122558593750, 0.0029907226562500, 0.0010375976562500, 0.0031127929687500,
+0.0013427734375000, 0.0033569335937500, 0.0018615722656250, 0.0031738281250000,
+0.0014953613281250, 0.0034790039062500, 0.0020141601562500, 0.0034484863281250,
+0.0020751953125000, 0.0033874511718750, 0.0019836425781250, 0.0036315917968750,
+0.0021057128906250, 0.0036926269531250, 0.0020751953125000, 0.0032958984375000,
+0.0020446777343750, 0.0035095214843750, 0.0019836425781250, 0.0034179687500000,
+0.0021667480468750, 0.0029907226562500, 0.0022888183593750, 0.0032043457031250,
+0.0021972656250000, 0.0031738281250000, 0.0026550292968750, 0.0030212402343750,
+0.0027465820312500, 0.0033264160156250, 0.0028686523437500, 0.0034484863281250,
+0.0032043457031250, 0.0032958984375000, 0.0032348632812500, 0.0038452148437500,
+0.0031738281250000, 0.0037841796875000, 0.0032958984375000, 0.0038757324218750,
+0.0032653808593750, 0.0042724609375000, 0.0031433105468750, 0.0040588378906250,
+0.0032043457031250, 0.0042114257812500, 0.0030822753906250, 0.0042724609375000,
+0.0030212402343750, 0.0042724609375000, 0.0031738281250000, 0.0043029785156250,
+0.0031433105468750, 0.0042419433593750, 0.0030212402343750, 0.0044250488281250,
+0.0034484863281250, 0.0042724609375000, 0.0033264160156250, 0.0043945312500000,
+0.0033569335937500, 0.0046691894531250, 0.0037231445312500, 0.0043945312500000,
+0.0036315917968750, 0.0046386718750000, 0.0035705566406250, 0.0046691894531250,
+0.0037841796875000, 0.0043945312500000, 0.0033874511718750, 0.0043640136718750,
+0.0030517578125000, 0.0043640136718750, 0.0033569335937500, 0.0039367675781250,
+0.0024719238281250, 0.0039672851562500, 0.0025634765625000, 0.0038757324218750,
+0.0023498535156250, 0.0035705566406250, 0.0016174316406250, 0.0037231445312500,
+0.0019531250000000, 0.0035400390625000, 0.0013427734375000, 0.0036010742187500,
+0.0013122558593750, 0.0036010742187500, 0.0012817382812500, 0.0035400390625000,
+0.0009460449218750, 0.0037231445312500, 0.0010375976562500, 0.0034484863281250,
+0.0008544921875000, 0.0036315917968750, 0.0009155273437500, 0.0036315917968750,
+0.0008239746093750, 0.0034484863281250, 0.0006408691406250, 0.0036621093750000,
+0.0007019042968750, 0.0034179687500000, 0.0000915527343750, 0.0035705566406250,
+0.0001525878906250, 0.0035705566406250, 0.0000305175781250, 0.0034484863281250,
+-0.0004577636718750, 0.0036010742187500, -0.0003356933593750, 0.0035400390625000,
+-0.0003356933593750, 0.0036315917968750, -0.0006408691406250, 0.0036621093750000,
+-0.0002136230468750, 0.0036010742187500, 0.0000915527343750, 0.0036926269531250,
+-0.0002746582031250, 0.0034790039062500, 0.0010375976562500, 0.0036010742187500,
+0.0007324218750000, 0.0036010742187500, 0.0012512207031250, 0.0034484863281250,
+0.0024719238281250, 0.0036315917968750, 0.0018310546875000, 0.0036621093750000,
+0.0032043457031250, 0.0036010742187500, 0.0031738281250000, 0.0038452148437500,
+0.0032348632812500, 0.0040588378906250, 0.0039978027343750, 0.0039062500000000,
+0.0036315917968750, 0.0044860839843750, 0.0040893554687500, 0.0044250488281250,
+0.0040588378906250, 0.0044250488281250, 0.0041198730468750, 0.0047607421875000,
+0.0043029785156250, 0.0047302246093750, 0.0041809082031250, 0.0047607421875000,
+0.0042724609375000, 0.0048217773437500, 0.0042419433593750, 0.0046997070312500,
+0.0041198730468750, 0.0046386718750000, 0.0041198730468750, 0.0046081542968750,
+0.0042419433593750, 0.0043945312500000, 0.0038757324218750, 0.0044555664062500,
+0.0040283203125000, 0.0044250488281250, 0.0038146972656250, 0.0042419433593750,
+0.0034790039062500, 0.0042114257812500, 0.0037536621093750, 0.0042114257812500,
+0.0029907226562500, 0.0040893554687500, 0.0031433105468750, 0.0041503906250000,
+0.0029602050781250, 0.0042724609375000, 0.0022888183593750, 0.0041503906250000,
+0.0025024414062500, 0.0043640136718750, 0.0022277832031250, 0.0043640136718750,
+0.0019531250000000, 0.0043945312500000, 0.0021972656250000, 0.0045166015625000,
+0.0023193359375000, 0.0045166015625000, 0.0021057128906250, 0.0045166015625000,
+0.0027770996093750, 0.0045471191406250, 0.0026855468750000, 0.0045166015625000,
+0.0027770996093750, 0.0045166015625000, 0.0032043457031250, 0.0045471191406250,
+0.0029907226562500, 0.0043029785156250, 0.0029296875000000, 0.0043945312500000,
+0.0029907226562500, 0.0040893554687500, 0.0026855468750000, 0.0037231445312500,
+0.0022888183593750, 0.0039367675781250, 0.0024414062500000, 0.0031738281250000,
+0.0019226074218750, 0.0032958984375000, 0.0017700195312500, 0.0030212402343750,
+0.0019226074218750, 0.0024108886718750, 0.0017700195312500, 0.0027465820312500,
+0.0016479492187500, 0.0020446777343750, 0.0019531250000000, 0.0019226074218750,
+0.0019531250000000, 0.0020141601562500, 0.0018615722656250, 0.0018005371093750,
+0.0019836425781250, 0.0018310546875000, 0.0020141601562500, 0.0022888183593750,
+0.0018920898437500, 0.0021667480468750, 0.0020141601562500, 0.0024108886718750,
+0.0018615722656250, 0.0029602050781250, 0.0016479492187500, 0.0026855468750000,
+0.0018615722656250, 0.0028381347656250, 0.0014648437500000, 0.0031127929687500,
+0.0014038085937500, 0.0027465820312500, 0.0017395019531250, 0.0025024414062500,
+0.0017089843750000, 0.0028686523437500, 0.0015869140625000, 0.0021667480468750,
+0.0023803710937500, 0.0022277832031250, 0.0021972656250000, 0.0023193359375000,
+0.0024719238281250, 0.0020751953125000, 0.0031738281250000, 0.0021667480468750,
+0.0027160644531250, 0.0022888183593750, 0.0034790039062500, 0.0022888183593750,
+0.0036010742187500, 0.0023193359375000, 0.0032958984375000, 0.0025634765625000,
+0.0033874511718750, 0.0025329589843750, 0.0034179687500000, 0.0024108886718750,
+0.0029602050781250, 0.0025634765625000, 0.0029907226562500, 0.0025329589843750,
+0.0026855468750000, 0.0023803710937500, 0.0023193359375000, 0.0025329589843750,
+0.0025329589843750, 0.0027160644531250, 0.0017700195312500, 0.0025329589843750,
+0.0018615722656250, 0.0027160644531250, 0.0019531250000000, 0.0033569335937500,
+0.0014343261718750, 0.0024108886718750, 0.0023498535156250, 0.0013427734375000,
+0.0016479492187500, 0.0017395019531250, 0.0018005371093750, 0.0014648437500000,
+0.0016784667968750, 0.0009155273437500, 0.0013427734375000, 0.0011901855468750,
+0.0014953613281250, 0.0008239746093750, 0.0017089843750000, 0.0008239746093750,
+0.0015258789062500, 0.0007934570312500, 0.0019226074218750, 0.0006103515625000,
+0.0023193359375000, 0.0006713867187500, 0.0021362304687500, 0.0006408691406250,
+0.0029907226562500, 0.0004882812500000, 0.0028381347656250, 0.0007324218750000,
+0.0032958984375000, 0.0009460449218750, 0.0040283203125000, 0.0007324218750000,
+0.0036010742187500, 0.0011596679687500, 0.0044250488281250, 0.0011596679687500,
+0.0047912597656250, 0.0012207031250000, 0.0041809082031250, 0.0014648437500000,
+0.0042724609375000, 0.0014038085937500, 0.0048522949218750, 0.0015563964843750,
+0.0030822753906250, 0.0016174316406250, 0.0035095214843750, 0.0015563964843750,
+0.0029907226562500, 0.0016479492187500, 0.0014648437500000, 0.0016784667968750,
+0.0022583007812500, 0.0014953613281250, 0.0013732910156250, 0.0017395019531250,
+0.0008239746093750, 0.0014343261718750, 0.0015258789062500, 0.0010986328125000,
+0.0017395019531250, 0.0014343261718750, 0.0012817382812500, 0.0005798339843750,
+0.0028076171875000, 0.0006713867187500, 0.0025634765625000, 0.0006713867187500,
+0.0027770996093750, 0.0000305175781250, 0.0037841796875000, 0.0003356933593750,
+0.0032043457031250, 0.0005187988281250, 0.0035705566406250, 0.0000915527343750,
+0.0036010742187500, 0.0008544921875000, 0.0035400390625000, 0.0015258789062500,
+0.0036621093750000, 0.0008850097656250, 0.0035095214843750, 0.0026550292968750,
+0.0035705566406250, 0.0023803710937500, 0.0035400390625000, 0.0027160644531250,
+0.0034484863281250, 0.0039978027343750, 0.0035095214843750, 0.0033569335937500,
+0.0036315917968750, 0.0043334960937500, 0.0036010742187500, 0.0044250488281250,
+0.0036010742187500, 0.0043945312500000, 0.0035705566406250, 0.0048217773437500,
+0.0036621093750000, 0.0046997070312500, 0.0037841796875000, 0.0048217773437500,
+0.0033264160156250, 0.0049133300781250, 0.0034790039062500, 0.0046691894531250,
+0.0035095214843750, 0.0046386718750000, 0.0031738281250000, 0.0047607421875000,
+0.0033874511718750, 0.0037231445312500, 0.0035400390625000, 0.0041198730468750,
+0.0034179687500000, 0.0037841796875000, 0.0036926269531250, 0.0028076171875000,
+0.0039978027343750, 0.0034179687500000, 0.0038757324218750, 0.0028381347656250,
+0.0042114257812500, 0.0026245117187500, 0.0043029785156250, 0.0031738281250000,
+0.0043640136718750},
+{0.0020141601562500, 0.0021972656250000, 0.0019836425781250, 0.0022888183593750,
+0.0018920898437500, 0.0024719238281250, 0.0018615722656250, 0.0020446777343750,
+0.0019836425781250, 0.0023193359375000, 0.0017395019531250, 0.0022583007812500,
+0.0019226074218750, 0.0018005371093750, 0.0023193359375000, 0.0021667480468750,
+0.0019226074218750, 0.0020141601562500, 0.0022888183593750, 0.0019531250000000,
+0.0022277832031250, 0.0022277832031250, 0.0022277832031250, 0.0024108886718750,
+0.0023803710937500, 0.0023498535156250, 0.0021057128906250, 0.0024719238281250,
+0.0024108886718750, 0.0026550292968750, 0.0023803710937500, 0.0024414062500000,
+0.0021667480468750, 0.0019531250000000, 0.0021972656250000, 0.0000915527343750,
+0.0017089843750000, 0.0003051757812500, 0.0018310546875000, -0.0000610351562500,
+0.0019531250000000, 0.0003051757812500, 0.0019836425781250, 0.0006408691406250,
+0.0021057128906250, 0.0002746582031250, 0.0021057128906250, 0.0010375976562500,
+0.0016479492187500, 0.0008239746093750, 0.0019836425781250, 0.0011596679687500,
+0.0018615722656250, 0.0017700195312500, 0.0014038085937500, 0.0013732910156250,
+0.0018310546875000, 0.0024108886718750, 0.0014343261718750, 0.0023193359375000,
+0.0016174316406250, 0.0023803710937500, 0.0017089843750000, 0.0028991699218750,
+0.0013732910156250, 0.0026855468750000, 0.0017089843750000, 0.0029296875000000,
+0.0021667480468750, 0.0028686523437500, 0.0018615722656250, 0.0028991699218750,
+0.0025329589843750, 0.0030517578125000, 0.0033264160156250, 0.0028686523437500,
+0.0029296875000000, 0.0030212402343750, 0.0039672851562500, 0.0031433105468750,
+0.0042419433593750, 0.0027160644531250, 0.0038452148437500, 0.0025329589843750,
+0.0042724609375000, 0.0027770996093750, 0.0044860839843750, 0.0017395019531250,
+0.0029296875000000, 0.0020141601562500, 0.0036010742187500, 0.0018005371093750,
+0.0029602050781250, 0.0010070800781250, 0.0013122558593750, 0.0014038085937500,
+0.0021057128906250, 0.0013427734375000, 0.0010681152343750, 0.0009765625000000,
+0.0005493164062500, 0.0014343261718750, 0.0012207031250000, 0.0018005371093750,
+0.0012512207031250, 0.0014648437500000, 0.0008544921875000, 0.0026550292968750,
+0.0024414062500000, 0.0025634765625000, 0.0023803710937500, 0.0027160644531250,
+0.0025329589843750, 0.0035095214843750, 0.0036621093750000, 0.0032348632812500,
+0.0035705566406250, 0.0037231445312500, 0.0037841796875000, 0.0037231445312500,
+0.0041198730468750, 0.0036926269531250, 0.0038757324218750, 0.0039062500000000,
+0.0036926269531250, 0.0038452148437500, 0.0039062500000000, 0.0036926269531250,
+0.0032348632812500, 0.0038146972656250, 0.0034484863281250, 0.0035705566406250,
+0.0034179687500000, 0.0032653808593750, 0.0030212402343750, 0.0035705566406250,
+0.0033569335937500, 0.0027770996093750, 0.0033264160156250, 0.0029602050781250,
+0.0032348632812500, 0.0026855468750000, 0.0033264160156250, 0.0019836425781250,
+0.0035095214843750, 0.0024108886718750, 0.0034484863281250, 0.0021972656250000,
+0.0034484863281250, 0.0019226074218750, 0.0034484863281250, 0.0024719238281250,
+0.0035400390625000, 0.0027770996093750, 0.0035095214843750, 0.0025329589843750,
+0.0033874511718750, 0.0038146972656250, 0.0037841796875000, 0.0037231445312500,
+0.0036315917968750, 0.0039367675781250, 0.0037841796875000, 0.0048522949218750,
+0.0041503906250000, 0.0045776367187500, 0.0038757324218750, 0.0045776367187500,
+0.0040893554687500, 0.0050048828125000, 0.0042114257812500, 0.0045471191406250,
+0.0039367675781250, 0.0041198730468750, 0.0038146972656250, 0.0046386718750000,
+0.0040283203125000, 0.0033264160156250, 0.0035705566406250, 0.0036010742187500,
+0.0036010742187500, 0.0034484863281250, 0.0036315917968750, 0.0025329589843750,
+0.0033264160156250, 0.0030822753906250, 0.0035400390625000, 0.0027160644531250,
+0.0034484863281250, 0.0024719238281250, 0.0034484863281250, 0.0030212402343750,
+0.0035400390625000, 0.0031433105468750, 0.0035705566406250, 0.0028381347656250,
+0.0036621093750000, 0.0042114257812500, 0.0036010742187500, 0.0038146972656250,
+0.0038146972656250, 0.0043640136718750, 0.0036621093750000, 0.0055847167968750,
+0.0035400390625000, 0.0047607421875000, 0.0038146972656250, 0.0060729980468750,
+0.0034179687500000, 0.0061035156250000, 0.0034179687500000, 0.0060119628906250,
+0.0034484863281250, 0.0066528320312500, 0.0032043457031250, 0.0065002441406250,
+0.0032958984375000, 0.0063476562500000, 0.0033874511718750, 0.0066223144531250,
+0.0031738281250000, 0.0062255859375000, 0.0033264160156250, 0.0058593750000000,
+0.0035095214843750, 0.0062561035156250, 0.0032653808593750, 0.0051269531250000,
+0.0036315917968750, 0.0054016113281250, 0.0035400390625000, 0.0049743652343750,
+0.0034790039062500, 0.0040893554687500, 0.0037536621093750, 0.0046691894531250,
+0.0036315917968750, 0.0033874511718750, 0.0036926269531250, 0.0035400390625000,
+0.0037841796875000, 0.0032043457031250, 0.0036621093750000, 0.0021972656250000,
+0.0035705566406250, 0.0025939941406250, 0.0037231445312500, 0.0017089843750000,
+0.0034179687500000, 0.0015563964843750, 0.0035400390625000, 0.0016784667968750,
+0.0033874511718750, 0.0013427734375000, 0.0031738281250000, 0.0013122558593750,
+0.0034179687500000, 0.0019531250000000, 0.0030212402343750, 0.0015258789062500,
+0.0031127929687500, 0.0022277832031250, 0.0029907226562500, 0.0030822753906250,
+0.0026855468750000, 0.0023803710937500, 0.0028991699218750, 0.0041198730468750,
+0.0025939941406250, 0.0039367675781250, 0.0025329589843750, 0.0041198730468750,
+0.0025939941406250, 0.0053100585937500, 0.0025634765625000, 0.0047607421875000,
+0.0025329589843750, 0.0052185058593750, 0.0026550292968750, 0.0053710937500000,
+0.0026550292968750, 0.0051574707031250, 0.0026550292968750, 0.0051574707031250,
+0.0027160644531250, 0.0051574707031250, 0.0027465820312500, 0.0048828125000000,
+0.0027465820312500, 0.0048522949218750, 0.0028381347656250, 0.0048522949218750,
+0.0028381347656250, 0.0047912597656250, 0.0028381347656250, 0.0047912597656250,
+0.0029907226562500, 0.0047912597656250, 0.0030212402343750, 0.0048828125000000,
+0.0030212402343750, 0.0046997070312500, 0.0031127929687500, 0.0046386718750000,
+0.0031738281250000, 0.0048217773437500, 0.0032348632812500, 0.0043029785156250,
+0.0030822753906250, 0.0043640136718750, 0.0032043457031250, 0.0043334960937500,
+0.0031433105468750, 0.0039672851562500, 0.0029907226562500, 0.0040893554687500,
+0.0031127929687500, 0.0040893554687500, 0.0029907226562500, 0.0039978027343750,
+0.0028991699218750, 0.0041198730468750, 0.0029907226562500, 0.0042419433593750,
+0.0030212402343750, 0.0041809082031250, 0.0029602050781250, 0.0044555664062500,
+0.0030212402343750, 0.0044555664062500, 0.0031738281250000, 0.0044860839843750,
+0.0030822753906250, 0.0046081542968750, 0.0030822753906250, 0.0045776367187500,
+0.0032958984375000, 0.0045166015625000, 0.0029907226562500, 0.0045776367187500,
+0.0032043457031250, 0.0043945312500000, 0.0029907226562500, 0.0042114257812500,
+0.0026245117187500, 0.0043029785156250, 0.0031127929687500, 0.0041198730468750,
+0.0024414062500000, 0.0039978027343750, 0.0026550292968750, 0.0040588378906250,
+0.0025939941406250, 0.0041503906250000, 0.0021667480468750, 0.0040588378906250,
+0.0025634765625000, 0.0042114257812500, 0.0021667480468750, 0.0042114257812500,
+0.0023193359375000, 0.0041809082031250, 0.0023803710937500, 0.0041198730468750,
+0.0022277832031250, 0.0040588378906250, 0.0024108886718750, 0.0042114257812500,
+0.0023193359375000, 0.0039978027343750, 0.0023803710937500, 0.0040588378906250,
+0.0024414062500000, 0.0042419433593750, 0.0022888183593750, 0.0039367675781250,
+0.0024719238281250, 0.0040283203125000, 0.0025939941406250, 0.0040283203125000,
+0.0025634765625000, 0.0036926269531250, 0.0026245117187500, 0.0034790039062500,
+0.0028381347656250, 0.0035705566406250, 0.0027770996093750, 0.0030212402343750,
+0.0025939941406250, 0.0030822753906250, 0.0027465820312500, 0.0031738281250000,
+0.0025024414062500, 0.0031127929687500, 0.0020751953125000, 0.0036010742187500,
+0.0021972656250000, 0.0039978027343750, 0.0018310546875000, 0.0040283203125000,
+0.0018005371093750, 0.0037841796875000, 0.0017089843750000, 0.0039672851562500,
+0.0014343261718750, 0.0041809082031250, 0.0013732910156250, 0.0029296875000000,
+0.0014648437500000, 0.0032348632812500, 0.0011901855468750, 0.0027770996093750,
+0.0013732910156250, 0.0016174316406250, 0.0016784667968750, 0.0021972656250000,
+0.0013427734375000, 0.0012817382812500, 0.0019531250000000, 0.0008239746093750,
+0.0018005371093750, 0.0012512207031250, 0.0018615722656250, 0.0011596679687500,
+0.0022583007812500, 0.0005493164062500, 0.0021057128906250, 0.0013732910156250,
+0.0023498535156250, 0.0011291503906250, 0.0023498535156250, 0.0011596679687500,
+0.0023498535156250, 0.0016479492187500, 0.0025329589843750, 0.0011901855468750,
+0.0025329589843750, 0.0015563964843750, 0.0027770996093750, 0.0016479492187500,
+0.0027465820312500, 0.0014343261718750, 0.0027770996093750, 0.0013732910156250,
+0.0030212402343750, 0.0013732910156250, 0.0030517578125000, 0.0014648437500000,
+0.0029907226562500, 0.0012512207031250, 0.0032043457031250, 0.0015869140625000,
+0.0030212402343750, 0.0019226074218750, 0.0028991699218750, 0.0016174316406250,
+0.0031738281250000, 0.0026550292968750, 0.0026245117187500, 0.0025939941406250,
+0.0026855468750000, 0.0027770996093750, 0.0027465820312500, 0.0035400390625000,
+0.0025024414062500, 0.0032043457031250, 0.0025939941406250, 0.0035705566406250,
+0.0026245117187500, 0.0038146972656250, 0.0026245117187500, 0.0035095214843750,
+0.0025939941406250, 0.0033569335937500, 0.0026855468750000, 0.0035705566406250,
+0.0027465820312500, 0.0032958984375000, 0.0025024414062500, 0.0031433105468750,
+0.0025329589843750, 0.0032348632812500, 0.0024719238281250, 0.0032348632812500,
+0.0021972656250000, 0.0030517578125000, 0.0022888183593750, 0.0034484863281250,
+0.0023498535156250, 0.0031127929687500, 0.0020446777343750, 0.0031738281250000,
+0.0022888183593750, 0.0035705566406250, 0.0025634765625000, 0.0032043457031250,
+0.0021362304687500, 0.0034484863281250, 0.0027160644531250, 0.0035095214843750,
+0.0026245117187500, 0.0032043457031250, 0.0025024414062500, 0.0031433105468750,
+0.0028076171875000, 0.0032348632812500, 0.0026550292968750, 0.0027465820312500,
+0.0026550292968750, 0.0028686523437500, 0.0026855468750000, 0.0028076171875000,
+0.0025024414062500, 0.0025024414062500, 0.0022888183593750, 0.0028076171875000,
+0.0023193359375000, 0.0025634765625000, 0.0022888183593750, 0.0028381347656250,
+0.0020446777343750, 0.0027465820312500, 0.0023193359375000, 0.0026245117187500,
+0.0025939941406250, 0.0031433105468750, 0.0021972656250000, 0.0025329589843750,
+0.0032958984375000, 0.0029296875000000, 0.0030517578125000, 0.0029907226562500,
+0.0032958984375000, 0.0025939941406250, 0.0040588378906250, 0.0032653808593750,
+0.0035705566406250, 0.0028991699218750, 0.0044250488281250, 0.0032348632812500,
+0.0043029785156250, 0.0034179687500000, 0.0043640136718750, 0.0033264160156250,
+0.0048828125000000, 0.0037841796875000, 0.0045776367187500, 0.0034484863281250,
+0.0050659179687500, 0.0037841796875000, 0.0050048828125000, 0.0038146972656250,
+0.0050048828125000, 0.0036010742187500, 0.0053405761718750, 0.0040283203125000,
+0.0052185058593750},
+{0.0012817382812500, 0.0044860839843750, 0.0012207031250000, 0.0036926269531250,
+0.0016174316406250, 0.0038757324218750, 0.0013122558593750, 0.0039062500000000,
+0.0014953613281250, 0.0035400390625000, 0.0018615722656250, 0.0037841796875000,
+0.0014648437500000, 0.0034790039062500, 0.0021362304687500, 0.0036621093750000,
+0.0020446777343750, 0.0033569335937500, 0.0020446777343750, 0.0028991699218750,
+0.0023803710937500, 0.0033264160156250, 0.0021972656250000, 0.0023803710937500,
+0.0022888183593750, 0.0024719238281250, 0.0022583007812500, 0.0024108886718750,
+0.0022888183593750, 0.0015869140625000, 0.0022888183593750, 0.0016174316406250,
+0.0021362304687500, 0.0018005371093750, 0.0024108886718750, 0.0014038085937500,
+0.0023193359375000, 0.0011901855468750, 0.0023193359375000, 0.0011596679687500,
+0.0025024414062500, 0.0008544921875000, 0.0023803710937500, 0.0010986328125000,
+0.0025634765625000, 0.0007019042968750, 0.0025634765625000, 0.0007629394531250,
+0.0024414062500000, 0.0010375976562500, 0.0024719238281250, 0.0006408691406250,
+0.0024719238281250, 0.0010375976562500, 0.0023193359375000, 0.0009460449218750,
+0.0023803710937500, 0.0007629394531250, 0.0023193359375000, 0.0008544921875000,
+0.0021362304687500, 0.0007324218750000, 0.0021972656250000, 0.0008239746093750,
+0.0019836425781250, 0.0005493164062500, 0.0019531250000000, 0.0008239746093750,
+0.0018920898437500, 0.0010375976562500, 0.0016784667968750, 0.0005187988281250,
+0.0017089843750000, 0.0017089843750000, 0.0014343261718750, 0.0013122558593750,
+0.0014038085937500, 0.0015869140625000, 0.0013427734375000, 0.0026855468750000,
+0.0010681152343750, 0.0019836425781250, 0.0010986328125000, 0.0026855468750000,
+0.0010070800781250, 0.0028686523437500, 0.0009155273437500, 0.0026855468750000,
+0.0009765625000000, 0.0028686523437500, 0.0010986328125000, 0.0027770996093750,
+0.0009765625000000, 0.0025634765625000, 0.0011291503906250, 0.0025634765625000,
+0.0010986328125000, 0.0024108886718750, 0.0010375976562500, 0.0021057128906250,
+0.0010681152343750, 0.0020751953125000, 0.0010375976562500, 0.0019836425781250,
+0.0010070800781250, 0.0018005371093750, 0.0009155273437500, 0.0018310546875000,
+0.0008239746093750, 0.0018005371093750, 0.0007324218750000, 0.0015563964843750,
+0.0007629394531250, 0.0017395019531250, 0.0007019042968750, 0.0016479492187500,
+0.0006103515625000, 0.0015869140625000, 0.0006408691406250, 0.0015869140625000,
+0.0006713867187500, 0.0014343261718750, 0.0005798339843750, 0.0014953613281250,
+0.0007629394531250, 0.0013427734375000, 0.0007324218750000, 0.0014038085937500,
+0.0007324218750000, 0.0015258789062500, 0.0008544921875000, 0.0012817382812500,
+0.0007629394531250, 0.0018310546875000, 0.0007934570312500, 0.0016479492187500,
+0.0007019042968750, 0.0017089843750000, 0.0007629394531250, 0.0020751953125000,
+0.0007324218750000, 0.0018005371093750, 0.0006103515625000, 0.0020751953125000,
+0.0011901855468750, 0.0020446777343750, 0.0009155273437500, 0.0020141601562500,
+0.0012817382812500, 0.0021972656250000, 0.0019226074218750, 0.0021362304687500,
+0.0014953613281250, 0.0022583007812500, 0.0027160644531250, 0.0023498535156250,
+0.0025329589843750, 0.0024108886718750, 0.0026855468750000, 0.0025939941406250,
+0.0035095214843750, 0.0027465820312500, 0.0030822753906250, 0.0027160644531250,
+0.0036926269531250, 0.0029296875000000, 0.0036926269531250, 0.0030212402343750,
+0.0035095214843750, 0.0029907226562500, 0.0036621093750000, 0.0033569335937500,
+0.0036315917968750, 0.0031433105468750, 0.0033874511718750, 0.0033874511718750,
+0.0034179687500000, 0.0035095214843750, 0.0032348632812500, 0.0033874511718750,
+0.0030212402343750, 0.0036926269531250, 0.0030822753906250, 0.0036621093750000,
+0.0026550292968750, 0.0037231445312500, 0.0027770996093750, 0.0039062500000000,
+0.0026550292968750, 0.0039978027343750, 0.0023803710937500, 0.0039672851562500,
+0.0025024414062500, 0.0042419433593750, 0.0022583007812500, 0.0042114257812500,
+0.0022583007812500, 0.0042419433593750, 0.0023193359375000, 0.0043945312500000,
+0.0021972656250000, 0.0043334960937500, 0.0022583007812500, 0.0044250488281250,
+0.0022277832031250, 0.0044250488281250, 0.0022277832031250, 0.0042419433593750,
+0.0022583007812500, 0.0041198730468750, 0.0022888183593750, 0.0041809082031250,
+0.0022277832031250, 0.0039367675781250, 0.0023803710937500, 0.0039062500000000,
+0.0023193359375000, 0.0037841796875000, 0.0024414062500000, 0.0035400390625000,
+0.0026245117187500, 0.0036010742187500, 0.0025634765625000, 0.0033569335937500,
+0.0027770996093750, 0.0033569335937500, 0.0027770996093750, 0.0032348632812500,
+0.0028686523437500, 0.0030517578125000, 0.0030517578125000, 0.0032043457031250,
+0.0030212402343750, 0.0027160644531250, 0.0033264160156250, 0.0028991699218750,
+0.0033264160156250, 0.0027770996093750, 0.0033874511718750, 0.0024108886718750,
+0.0035705566406250, 0.0027160644531250, 0.0035095214843750, 0.0022277832031250,
+0.0035400390625000, 0.0023498535156250, 0.0035400390625000, 0.0023193359375000,
+0.0035095214843750, 0.0019226074218750, 0.0034179687500000, 0.0022583007812500,
+0.0033874511718750, 0.0019531250000000, 0.0032043457031250, 0.0019531250000000,
+0.0031127929687500, 0.0021057128906250, 0.0030517578125000, 0.0021057128906250,
+0.0029296875000000, 0.0021667480468750, 0.0028076171875000, 0.0023498535156250,
+0.0028076171875000, 0.0023498535156250, 0.0026550292968750, 0.0024719238281250,
+0.0025634765625000, 0.0026245117187500, 0.0025939941406250, 0.0025939941406250,
+0.0024719238281250, 0.0026550292968750, 0.0024719238281250, 0.0027160644531250,
+0.0024414062500000, 0.0027465820312500, 0.0022583007812500, 0.0027465820312500,
+0.0021667480468750, 0.0028686523437500, 0.0022277832031250, 0.0027770996093750,
+0.0016174316406250, 0.0028381347656250, 0.0017700195312500, 0.0026855468750000,
+0.0016479492187500, 0.0025329589843750, 0.0010986328125000, 0.0026855468750000,
+0.0012817382812500, 0.0023498535156250, 0.0010375976562500, 0.0023803710937500,
+0.0010070800781250, 0.0024108886718750, 0.0009460449218750, 0.0022888183593750,
+0.0008239746093750, 0.0022888183593750, 0.0010375976562500, 0.0024719238281250,
+0.0008239746093750, 0.0024414062500000, 0.0009460449218750, 0.0024108886718750,
+0.0008239746093750, 0.0024414062500000, 0.0006103515625000, 0.0025939941406250,
+0.0008850097656250, 0.0022277832031250, 0.0006103515625000, 0.0025024414062500,
+0.0006103515625000, 0.0024108886718750, 0.0008544921875000, 0.0020446777343750,
+0.0008239746093750, 0.0024719238281250, 0.0007934570312500, 0.0021057128906250,
+0.0013122558593750, 0.0023193359375000, 0.0011901855468750, 0.0023498535156250,
+0.0015869140625000, 0.0022277832031250, 0.0019836425781250, 0.0026245117187500,
+0.0016479492187500, 0.0019531250000000, 0.0032043457031250, 0.0023803710937500,
+0.0029602050781250, 0.0019226074218750, 0.0033569335937500, 0.0011596679687500,
+0.0047302246093750, 0.0018920898437500, 0.0043029785156250, 0.0003662109375000,
+0.0048217773437500, 0.0005493164062500, 0.0051879882812500, 0.0004882812500000,
+0.0050354003906250, -0.0003967285156250, 0.0050964355468750, -0.0000610351562500,
+0.0052795410156250, -0.0001525878906250, 0.0046081542968750, -0.0003967285156250,
+0.0049133300781250, -0.0001525878906250, 0.0046691894531250, 0.0003662109375000,
+0.0034179687500000, 0.0014953613281250, 0.0031127929687500, 0.0024108886718750,
+0.0034484863281250, 0.0019531250000000, 0.0032653808593750, 0.0022888183593750,
+0.0033264160156250, 0.0029296875000000, 0.0036010742187500, 0.0025329589843750,
+0.0034790039062500, 0.0030822753906250, 0.0034179687500000, 0.0031127929687500,
+0.0033874511718750, 0.0030517578125000, 0.0032348632812500, 0.0032958984375000,
+0.0032043457031250, 0.0031738281250000, 0.0032958984375000, 0.0030822753906250,
+0.0023498535156250, 0.0032348632812500, 0.0023498535156250, 0.0028076171875000,
+0.0023193359375000, 0.0025024414062500, 0.0018310546875000, 0.0027465820312500,
+0.0018615722656250, 0.0018005371093750, 0.0018005371093750, 0.0017700195312500,
+0.0019531250000000, 0.0016784667968750, 0.0014953613281250, 0.0010986328125000,
+0.0011596679687500, 0.0012512207031250, 0.0016784667968750, 0.0013427734375000,
+0.0005798339843750, 0.0010681152343750, 0.0007629394531250, 0.0013427734375000,
+0.0006408691406250, 0.0017395019531250, -0.0001220703125000, 0.0014648437500000,
+0.0003662109375000, 0.0018310546875000, 0.0000305175781250, 0.0019836425781250,
+-0.0003967285156250, 0.0018005371093750, 0.0002746582031250, 0.0018310546875000,
+0.0006408691406250, 0.0018920898437500, 0.0002136230468750, 0.0013732910156250,
+0.0019226074218750, 0.0014648437500000, 0.0017395019531250, 0.0011901855468750,
+0.0021667480468750, 0.0007019042968750, 0.0036621093750000, 0.0010375976562500,
+0.0031127929687500, 0.0000610351562500, 0.0033874511718750, 0.0003356933593750,
+0.0038146972656250, -0.0000610351562500, 0.0031433105468750, -0.0008850097656250,
+0.0025634765625000, -0.0003662109375000, 0.0029296875000000, -0.0014038085937500,
+0.0015563964843750, -0.0014648437500000, 0.0014953613281250, -0.0013427734375000,
+0.0014038085937500, -0.0018920898437500, 0.0006408691406250, -0.0018310546875000,
+0.0009460449218750, -0.0011596679687500, 0.0011596679687500, -0.0016174316406250,
+0.0009155273437500, -0.0008544921875000, 0.0012817382812500, 0.0000915527343750,
+0.0018920898437500, -0.0007934570312500, 0.0016174316406250, 0.0010070800781250,
+0.0021362304687500, 0.0010375976562500, 0.0023498535156250, 0.0008850097656250,
+0.0021972656250000, 0.0017089843750000, 0.0022888183593750, 0.0015258789062500,
+0.0024108886718750, 0.0010070800781250, 0.0019836425781250, 0.0012512207031250,
+0.0021057128906250, 0.0007934570312500, 0.0020751953125000, 0.0000610351562500,
+0.0018005371093750, 0.0003967285156250, 0.0018615722656250, -0.0003967285156250,
+0.0018615722656250, -0.0006713867187500, 0.0017395019531250, -0.0004272460937500,
+0.0018920898437500, -0.0007629394531250, 0.0020446777343750, -0.0009765625000000,
+0.0018615722656250, -0.0003051757812500, 0.0022277832031250, -0.0006103515625000,
+0.0021972656250000, -0.0003051757812500, 0.0023498535156250, 0.0003051757812500,
+0.0025634765625000, -0.0001525878906250, 0.0024719238281250, 0.0005798339843750,
+0.0028381347656250, 0.0005493164062500, 0.0027770996093750, 0.0004577636718750,
+0.0028381347656250, 0.0008239746093750, 0.0031127929687500, 0.0006713867187500,
+0.0029602050781250, 0.0003356933593750, 0.0033264160156250, 0.0004882812500000,
+0.0032043457031250, 0.0001831054687500, 0.0032043457031250, -0.0002746582031250,
+0.0033874511718750, -0.0000305175781250, 0.0031127929687500, -0.0005187988281250,
+0.0032958984375000, -0.0005493164062500, 0.0031738281250000, -0.0004577636718750,
+0.0030517578125000, -0.0006103515625000, 0.0031433105468750, -0.0005187988281250,
+0.0029296875000000, -0.0000610351562500, 0.0029602050781250, -0.0002136230468750,
+0.0028381347656250, 0.0001220703125000, 0.0027465820312500, 0.0006103515625000,
+0.0027770996093750, 0.0004272460937500, 0.0026245117187500, 0.0013427734375000,
+0.0027160644531250, 0.0012817382812500, 0.0025939941406250, 0.0016174316406250,
+0.0025939941406250, 0.0023193359375000, 0.0026550292968750, 0.0020446777343750,
+0.0025329589843750, 0.0026855468750000, 0.0026550292968750, 0.0027160644531250,
+0.0025634765625000},
+{0.0025939941406250, 0.0003051757812500, 0.0025329589843750, 0.0005187988281250,
+0.0023803710937500, 0.0006713867187500, 0.0018920898437500, 0.0006713867187500,
+0.0020141601562500, 0.0005798339843750, 0.0015258789062500, 0.0007934570312500,
+0.0015563964843750, 0.0006103515625000, 0.0014038085937500, 0.0002136230468750,
+0.0010681152343750, 0.0005187988281250, 0.0012207031250000, 0.0001525878906250,
+0.0006103515625000, 0.0000915527343750, 0.0007019042968750, 0.0001831054687500,
+0.0004577636718750, 0.0001220703125000, -0.0001525878906250, 0.0000915527343750,
+0.0000915527343750, 0.0001525878906250, -0.0003356933593750, 0.0002136230468750,
+-0.0005187988281250, 0.0001220703125000, -0.0001220703125000, 0.0001525878906250,
+-0.0001525878906250, 0.0001831054687500, -0.0002746582031250, 0.0000915527343750,
+0.0012207031250000, -0.0001220703125000, 0.0007934570312500, 0.0003356933593750,
+0.0012512207031250, 0.0005493164062500, 0.0025634765625000, 0.0001525878906250,
+0.0019226074218750, 0.0018920898437500, 0.0025939941406250, 0.0013427734375000,
+0.0029907226562500, 0.0020751953125000, 0.0023193359375000, 0.0036926269531250,
+0.0019836425781250, 0.0026550292968750, 0.0022888183593750, 0.0044250488281250,
+0.0007934570312500, 0.0043640136718750, 0.0007934570312500, 0.0041198730468750,
+0.0007324218750000, 0.0049743652343750, -0.0001831054687500, 0.0046691894531250,
+0.0000000000000000, 0.0040893554687500, 0.0003051757812500, 0.0043334960937500,
+-0.0001220703125000, 0.0039672851562500, 0.0004577636718750, 0.0032653808593750,
+0.0011291503906250, 0.0035400390625000, 0.0005798339843750, 0.0030822753906250,
+0.0021057128906250, 0.0029296875000000, 0.0019836425781250, 0.0032348632812500,
+0.0022583007812500, 0.0031433105468750, 0.0033264160156250, 0.0029602050781250,
+0.0028076171875000, 0.0037536621093750, 0.0033874511718750, 0.0035705566406250,
+0.0034790039062500, 0.0037536621093750, 0.0032958984375000, 0.0043640136718750,
+0.0033569335937500, 0.0039062500000000, 0.0031738281250000, 0.0043334960937500,
+0.0031433105468750, 0.0043945312500000, 0.0030822753906250, 0.0040893554687500,
+0.0029296875000000, 0.0041503906250000, 0.0027770996093750, 0.0040893554687500,
+0.0028076171875000, 0.0035095214843750, 0.0027160644531250, 0.0035705566406250,
+0.0025329589843750, 0.0031433105468750, 0.0026550292968750, 0.0024719238281250,
+0.0026245117187500, 0.0027770996093750, 0.0025024414062500, 0.0020141601562500,
+0.0026855468750000, 0.0018615722656250, 0.0026550292968750, 0.0018310546875000,
+0.0026550292968750, 0.0014648437500000, 0.0027465820312500, 0.0014953613281250,
+0.0027465820312500, 0.0012512207031250, 0.0027465820312500, 0.0012512207031250,
+0.0028381347656250, 0.0010070800781250, 0.0027770996093750, 0.0008239746093750,
+0.0027465820312500, 0.0009765625000000, 0.0028991699218750, 0.0002441406250000,
+0.0027465820312500, 0.0004577636718750, 0.0028076171875000, 0.0000915527343750,
+0.0028381347656250, -0.0005493164062500, 0.0027465820312500, -0.0000915527343750,
+0.0027160644531250, -0.0009460449218750, 0.0029602050781250, -0.0009460449218750,
+0.0027160644531250, -0.0009155273437500, 0.0028686523437500, -0.0013122558593750,
+0.0031433105468750, -0.0011596679687500, 0.0028076171875000, -0.0009155273437500,
+0.0033874511718750, -0.0010986328125000, 0.0032348632812500, -0.0006408691406250,
+0.0032653808593750, -0.0001220703125000, 0.0036621093750000, -0.0003967285156250,
+0.0033874511718750, 0.0008239746093750, 0.0037231445312500, 0.0006713867187500,
+0.0037536621093750, 0.0009765625000000, 0.0036926269531250, 0.0019531250000000,
+0.0038452148437500, 0.0015869140625000, 0.0037841796875000, 0.0023193359375000,
+0.0037841796875000, 0.0023803710937500, 0.0038452148437500, 0.0022888183593750,
+0.0037536621093750, 0.0026245117187500, 0.0037536621093750, 0.0025024414062500,
+0.0038452148437500, 0.0023803710937500, 0.0036621093750000, 0.0024719238281250,
+0.0037231445312500, 0.0023193359375000, 0.0036926269531250, 0.0021667480468750,
+0.0036010742187500, 0.0023193359375000, 0.0035705566406250, 0.0018615722656250,
+0.0036010742187500, 0.0021057128906250, 0.0036010742187500, 0.0017395019531250,
+0.0035400390625000, 0.0012512207031250, 0.0035400390625000, 0.0017089843750000,
+0.0036010742187500, 0.0006713867187500, 0.0033569335937500, 0.0007629394531250,
+0.0034790039062500, 0.0004882812500000, 0.0034179687500000, -0.0003051757812500,
+0.0032043457031250, 0.0001525878906250, 0.0033264160156250, -0.0003967285156250,
+0.0031738281250000, -0.0006103515625000, 0.0032043457031250, -0.0001525878906250,
+0.0032043457031250, -0.0001220703125000, 0.0031127929687500, -0.0003662109375000,
+0.0031127929687500, 0.0011291503906250, 0.0032043457031250, 0.0007934570312500,
+0.0032043457031250, 0.0014343261718750, 0.0032653808593750, 0.0027770996093750,
+0.0034179687500000, 0.0020446777343750, 0.0033264160156250, 0.0033874511718750,
+0.0032958984375000, 0.0035400390625000, 0.0033874511718750, 0.0033874511718750,
+0.0032653808593750, 0.0038757324218750, 0.0032043457031250, 0.0038146972656250,
+0.0033264160156250, 0.0034790039062500, 0.0030517578125000, 0.0035705566406250,
+0.0030517578125000, 0.0034484863281250, 0.0029602050781250, 0.0031738281250000,
+0.0027770996093750, 0.0032348632812500, 0.0028076171875000, 0.0029602050781250,
+0.0026855468750000, 0.0029907226562500, 0.0026245117187500, 0.0028991699218750,
+0.0026855468750000, 0.0027465820312500, 0.0025939941406250, 0.0028076171875000,
+0.0025329589843750, 0.0025634765625000, 0.0027160644531250, 0.0026855468750000,
+0.0024719238281250, 0.0025329589843750, 0.0025024414062500, 0.0022583007812500,
+0.0026550292968750, 0.0025939941406250, 0.0024108886718750, 0.0021057128906250,
+0.0027160644531250, 0.0022277832031250, 0.0026245117187500, 0.0022277832031250,
+0.0026245117187500, 0.0019836425781250, 0.0027465820312500, 0.0022277832031250,
+0.0025939941406250, 0.0020141601562500, 0.0026245117187500, 0.0022277832031250,
+0.0026550292968750, 0.0021667480468750, 0.0025329589843750, 0.0019226074218750,
+0.0024414062500000, 0.0022583007812500, 0.0024719238281250, 0.0017395019531250,
+0.0023193359375000, 0.0018615722656250, 0.0023803710937500, 0.0016174316406250,
+0.0022888183593750, 0.0011596679687500, 0.0021362304687500, 0.0014648437500000,
+0.0022583007812500, 0.0006408691406250, 0.0021667480468750, 0.0006103515625000,
+0.0020751953125000, 0.0006103515625000, 0.0021362304687500, 0.0001220703125000,
+0.0021667480468750, 0.0001831054687500, 0.0021362304687500, 0.0006103515625000,
+0.0023803710937500, 0.0002136230468750, 0.0023193359375000, 0.0005798339843750,
+0.0023803710937500, 0.0012817382812500, 0.0025634765625000, 0.0010375976562500,
+0.0023193359375000, 0.0020141601562500, 0.0021057128906250, 0.0017089843750000,
+0.0021972656250000, 0.0020751953125000, 0.0021362304687500, 0.0029907226562500,
+0.0019836425781250, 0.0025939941406250, 0.0020751953125000, 0.0036621093750000,
+0.0017700195312500, 0.0039978027343750, 0.0018310546875000, 0.0038146972656250,
+0.0017700195312500, 0.0042419433593750, 0.0016174316406250, 0.0044555664062500,
+0.0017395019531250, 0.0035705566406250, 0.0016479492187500, 0.0042419433593750,
+0.0017089843750000, 0.0037536621093750, 0.0015563964843750, 0.0027160644531250,
+0.0013427734375000, 0.0035705566406250, 0.0015563964843750, 0.0022888183593750,
+0.0013122558593750, 0.0025329589843750, 0.0012512207031250, 0.0028381347656250,
+0.0013122558593750, 0.0021667480468750, 0.0011596679687500, 0.0024719238281250,
+0.0012512207031250, 0.0025634765625000, 0.0013427734375000, 0.0027160644531250,
+0.0013427734375000, 0.0025939941406250, 0.0013122558593750, 0.0025024414062500,
+0.0014648437500000, 0.0027160644531250, 0.0014953613281250, 0.0020446777343750,
+0.0015258789062500, 0.0023498535156250, 0.0015258789062500, 0.0020446777343750,
+0.0016784667968750, 0.0014038085937500, 0.0017700195312500, 0.0018310546875000,
+0.0017089843750000, 0.0011291503906250, 0.0023498535156250, 0.0011291503906250,
+0.0021972656250000, 0.0012512207031250, 0.0024719238281250, 0.0009765625000000,
+0.0029907226562500, 0.0011291503906250, 0.0026245117187500, 0.0011901855468750,
+0.0035095214843750, 0.0011901855468750, 0.0033569335937500, 0.0010986328125000,
+0.0035095214843750, 0.0009460449218750, 0.0040588378906250, 0.0010681152343750,
+0.0036315917968750, 0.0007019042968750, 0.0042419433593750, 0.0007019042968750,
+0.0041198730468750, 0.0006713867187500, 0.0041503906250000, 0.0004882812500000,
+0.0044555664062500, 0.0005187988281250, 0.0041198730468750, 0.0007324218750000,
+0.0044860839843750, 0.0004577636718750, 0.0044555664062500, 0.0008850097656250,
+0.0043029785156250, 0.0013732910156250, 0.0043945312500000, 0.0009765625000000,
+0.0042419433593750, 0.0018615722656250, 0.0043334960937500, 0.0018005371093750,
+0.0041503906250000, 0.0018920898437500, 0.0042419433593750, 0.0025024414062500,
+0.0043640136718750, 0.0021362304687500, 0.0040893554687500, 0.0023193359375000,
+0.0044860839843750, 0.0023803710937500, 0.0044250488281250, 0.0022583007812500,
+0.0044250488281250, 0.0021362304687500, 0.0046997070312500, 0.0021362304687500,
+0.0045166015625000, 0.0022583007812500, 0.0046997070312500, 0.0020141601562500,
+0.0047912597656250, 0.0021667480468750, 0.0046386718750000, 0.0023803710937500,
+0.0046691894531250, 0.0020751953125000, 0.0048522949218750, 0.0026550292968750,
+0.0042724609375000, 0.0024414062500000, 0.0045471191406250, 0.0025634765625000,
+0.0041503906250000, 0.0029602050781250, 0.0035400390625000, 0.0026855468750000,
+0.0039367675781250, 0.0031127929687500, 0.0027160644531250, 0.0030517578125000,
+0.0027465820312500, 0.0031433105468750, 0.0023803710937500, 0.0033874511718750,
+0.0013732910156250, 0.0031738281250000, 0.0017395019531250, 0.0036621093750000,
+0.0006713867187500, 0.0034179687500000, 0.0007019042968750, 0.0035095214843750,
+0.0005187988281250, 0.0038146972656250, -0.0001831054687500, 0.0033569335937500,
+0.0000915527343750, 0.0037841796875000, -0.0003051757812500, 0.0035095214843750,
+-0.0003662109375000, 0.0034484863281250, -0.0002441406250000, 0.0036315917968750,
+-0.0003967285156250, 0.0031738281250000, -0.0004272460937500, 0.0036010742187500,
+-0.0002441406250000, 0.0032348632812500, -0.0003051757812500, 0.0032348632812500,
+-0.0001831054687500, 0.0035095214843750, 0.0000000000000000, 0.0030212402343750,
+-0.0002136230468750, 0.0033569335937500, 0.0002746582031250, 0.0032348632812500,
+0.0001220703125000, 0.0030822753906250, 0.0003051757812500, 0.0032348632812500,
+0.0006103515625000, 0.0031433105468750, 0.0003356933593750, 0.0029296875000000,
+0.0008239746093750, 0.0029907226562500, 0.0006408691406250, 0.0028381347656250,
+0.0007629394531250, 0.0026550292968750, 0.0011291503906250, 0.0028076171875000,
+0.0010070800781250, 0.0025634765625000, 0.0014038085937500, 0.0025634765625000,
+0.0013122558593750, 0.0027465820312500, 0.0014038085937500, 0.0027770996093750,
+0.0016784667968750, 0.0027160644531250, 0.0014648437500000, 0.0029296875000000,
+0.0017089843750000, 0.0028991699218750, 0.0017089843750000, 0.0029602050781250,
+0.0015869140625000, 0.0030517578125000, 0.0015869140625000, 0.0030212402343750,
+0.0015869140625000, 0.0031127929687500, 0.0010375976562500, 0.0030822753906250,
+0.0010681152343750, 0.0030822753906250, 0.0008544921875000, 0.0031433105468750,
+0.0003967285156250},
+{0.0043029785156250, 0.0025024414062500, 0.0046386718750000, 0.0029296875000000,
+0.0043945312500000, 0.0024108886718750, 0.0044555664062500, 0.0024414062500000,
+0.0044555664062500, 0.0025634765625000, 0.0043029785156250, 0.0023193359375000,
+0.0041503906250000, 0.0024719238281250, 0.0040588378906250, 0.0023193359375000,
+0.0038452148437500, 0.0023803710937500, 0.0037841796875000, 0.0023193359375000,
+0.0037231445312500, 0.0022583007812500, 0.0035705566406250, 0.0024108886718750,
+0.0035095214843750, 0.0016174316406250, 0.0035095214843750, 0.0018310546875000,
+0.0034484863281250, 0.0016784667968750, 0.0034790039062500, 0.0010070800781250,
+0.0034484863281250, 0.0014038085937500, 0.0034484863281250, 0.0013427734375000,
+0.0036315917968750, 0.0011291503906250, 0.0035400390625000, 0.0015563964843750,
+0.0036621093750000, 0.0018005371093750, 0.0037841796875000, 0.0016174316406250,
+0.0036315917968750, 0.0026245117187500, 0.0039672851562500, 0.0025024414062500,
+0.0038146972656250, 0.0027160644531250, 0.0038146972656250, 0.0034179687500000,
+0.0039978027343750, 0.0031738281250000, 0.0036315917968750, 0.0035095214843750,
+0.0037536621093750, 0.0037231445312500, 0.0036926269531250, 0.0037231445312500,
+0.0035095214843750, 0.0038146972656250, 0.0034179687500000, 0.0038757324218750,
+0.0033569335937500, 0.0039062500000000, 0.0030517578125000, 0.0039367675781250,
+0.0030212402343750, 0.0039978027343750, 0.0029296875000000, 0.0040588378906250,
+0.0027160644531250, 0.0039672851562500, 0.0027160644531250, 0.0042114257812500,
+0.0028381347656250, 0.0042114257812500, 0.0027160644531250, 0.0042114257812500,
+0.0028991699218750, 0.0043640136718750, 0.0031127929687500, 0.0043640136718750,
+0.0029602050781250, 0.0042114257812500, 0.0035705566406250, 0.0043640136718750,
+0.0035095214843750, 0.0043640136718750, 0.0036621093750000, 0.0042419433593750,
+0.0040893554687500, 0.0043945312500000, 0.0037841796875000, 0.0043029785156250,
+0.0039978027343750, 0.0044250488281250, 0.0041503906250000, 0.0043945312500000,
+0.0038146972656250, 0.0044250488281250, 0.0036315917968750, 0.0046081542968750,
+0.0037536621093750, 0.0042724609375000, 0.0029296875000000, 0.0044555664062500,
+0.0030212402343750, 0.0043334960937500, 0.0025939941406250, 0.0039978027343750,
+0.0018615722656250, 0.0042724609375000, 0.0021057128906250, 0.0037231445312500,
+0.0010375976562500, 0.0037841796875000, 0.0010681152343750, 0.0036621093750000,
+0.0006408691406250, 0.0032043457031250, -0.0001220703125000, 0.0034484863281250,
+0.0002136230468750, 0.0031433105468750, -0.0005798339843750, 0.0030822753906250,
+-0.0007019042968750, 0.0031433105468750, -0.0007019042968750, 0.0030822753906250,
+-0.0010375976562500, 0.0030517578125000, -0.0010070800781250, 0.0031433105468750,
+-0.0007629394531250, 0.0032043457031250, -0.0010375976562500, 0.0032043457031250,
+-0.0007934570312500, 0.0032043457031250, -0.0004577636718750, 0.0032653808593750,
+-0.0007934570312500, 0.0032348632812500, -0.0000915527343750, 0.0032348632812500,
+-0.0002441406250000, 0.0032653808593750, 0.0000000000000000, 0.0032653808593750,
+0.0005798339843750, 0.0032653808593750, 0.0001831054687500, 0.0033874511718750,
+0.0009460449218750, 0.0033264160156250, 0.0008850097656250, 0.0034484863281250,
+0.0008544921875000, 0.0035095214843750, 0.0013122558593750, 0.0034179687500000,
+0.0011291503906250, 0.0038757324218750, 0.0011291503906250, 0.0036926269531250,
+0.0013122558593750, 0.0039062500000000, 0.0009765625000000, 0.0043029785156250,
+0.0007324218750000, 0.0039367675781250, 0.0010375976562500, 0.0045166015625000,
+0.0002136230468750, 0.0043945312500000, 0.0003356933593750, 0.0043029785156250,
+0.0003662109375000, 0.0045166015625000, -0.0002136230468750, 0.0043029785156250,
+-0.0000305175781250, 0.0043334960937500, 0.0000305175781250, 0.0042114257812500,
+-0.0001525878906250, 0.0040893554687500, 0.0001220703125000, 0.0039978027343750,
+0.0004272460937500, 0.0038757324218750, 0.0001831054687500, 0.0038757324218750,
+0.0006408691406250, 0.0037536621093750, 0.0006408691406250, 0.0038146972656250,
+0.0006713867187500, 0.0038757324218750, 0.0008850097656250, 0.0036621093750000,
+0.0007324218750000, 0.0038452148437500, 0.0010375976562500, 0.0037536621093750,
+0.0009155273437500, 0.0035705566406250, 0.0011901855468750, 0.0036010742187500,
+0.0015869140625000, 0.0036315917968750, 0.0013427734375000, 0.0031127929687500,
+0.0021362304687500, 0.0032653808593750, 0.0021057128906250, 0.0030517578125000,
+0.0022583007812500, 0.0025939941406250, 0.0028381347656250, 0.0028991699218750,
+0.0027160644531250, 0.0024108886718750, 0.0029296875000000, 0.0023803710937500,
+0.0031127929687500, 0.0023193359375000, 0.0030212402343750, 0.0020141601562500,
+0.0029907226562500, 0.0021667480468750, 0.0031127929687500, 0.0017395019531250,
+0.0028686523437500, 0.0018615722656250, 0.0029296875000000, 0.0016479492187500,
+0.0029602050781250, 0.0011901855468750, 0.0028381347656250, 0.0014953613281250,
+0.0028991699218750, 0.0010375976562500, 0.0029296875000000, 0.0009460449218750,
+0.0029602050781250, 0.0011901855468750, 0.0028991699218750, 0.0010681152343750,
+0.0028076171875000, 0.0008544921875000, 0.0028686523437500, 0.0015869140625000,
+0.0025329589843750, 0.0012817382812500, 0.0025329589843750, 0.0016174316406250,
+0.0025329589843750, 0.0022277832031250, 0.0024108886718750, 0.0017395019531250,
+0.0024108886718750, 0.0025939941406250, 0.0027465820312500, 0.0024719238281250,
+0.0025024414062500, 0.0025634765625000, 0.0027160644531250, 0.0030822753906250,
+0.0032348632812500, 0.0027160644531250, 0.0030212402343750, 0.0032958984375000,
+0.0029907226562500, 0.0031127929687500, 0.0031127929687500, 0.0032043457031250,
+0.0029602050781250, 0.0036315917968750, 0.0027770996093750, 0.0032348632812500,
+0.0028991699218750, 0.0036621093750000, 0.0024108886718750, 0.0036621093750000,
+0.0026550292968750, 0.0034179687500000, 0.0025024414062500, 0.0035095214843750,
+0.0017395019531250, 0.0023498535156250, 0.0026550292968750, 0.0009155273437500,
+0.0027160644531250, 0.0014038085937500, 0.0027770996093750, 0.0011596679687500,
+0.0027770996093750, 0.0003051757812500, 0.0027770996093750, 0.0005798339843750,
+0.0028381347656250, 0.0006103515625000, 0.0028991699218750, 0.0003662109375000,
+0.0027465820312500, 0.0006713867187500, 0.0030517578125000, 0.0009155273437500,
+0.0036010742187500, 0.0005493164062500, 0.0033874511718750, 0.0012512207031250,
+0.0031738281250000, 0.0011291503906250, 0.0033874511718750, 0.0012817382812500,
+0.0034179687500000, 0.0017700195312500, 0.0031127929687500, 0.0014343261718750,
+0.0031127929687500, 0.0021362304687500, 0.0035095214843750, 0.0020446777343750,
+0.0032348632812500, 0.0021667480468750, 0.0034179687500000, 0.0026550292968750,
+0.0038146972656250, 0.0024719238281250, 0.0034179687500000, 0.0027770996093750,
+0.0040283203125000, 0.0028991699218750, 0.0039672851562500, 0.0027770996093750,
+0.0037231445312500, 0.0028076171875000, 0.0039367675781250, 0.0029296875000000,
+0.0038757324218750, 0.0030822753906250, 0.0034179687500000, 0.0029296875000000,
+0.0035095214843750, 0.0033264160156250, 0.0033264160156250, 0.0036926269531250,
+0.0028076171875000, 0.0034179687500000, 0.0029296875000000, 0.0047912597656250,
+0.0027465820312500, 0.0045776367187500, 0.0023803710937500, 0.0048522949218750,
+0.0025329589843750, 0.0059814453125000, 0.0027160644531250, 0.0054931640625000,
+0.0023803710937500, 0.0056152343750000, 0.0026550292968750, 0.0061645507812500,
+0.0027465820312500, 0.0054626464843750, 0.0024719238281250, 0.0048217773437500,
+0.0024108886718750, 0.0054626464843750, 0.0025024414062500, 0.0037841796875000,
+0.0017700195312500, 0.0039062500000000, 0.0015869140625000, 0.0038146972656250,
+0.0015563964843750, 0.0028076171875000, 0.0010375976562500, 0.0032043457031250,
+0.0009155273437500, 0.0029602050781250, 0.0016784667968750, 0.0027160644531250,
+0.0011291503906250, 0.0030822753906250, 0.0016479492187500, 0.0033264160156250,
+0.0027465820312500, 0.0031127929687500, 0.0020141601562500, 0.0036926269531250,
+0.0033264160156250, 0.0038452148437500, 0.0033569335937500, 0.0036621093750000,
+0.0032653808593750, 0.0037231445312500, 0.0039062500000000, 0.0038757324218750,
+0.0036926269531250, 0.0032653808593750, 0.0033569335937500, 0.0034790039062500,
+0.0037536621093750, 0.0031127929687500, 0.0033264160156250, 0.0025634765625000,
+0.0026550292968750, 0.0031127929687500, 0.0032043457031250, 0.0021667480468750,
+0.0024719238281250, 0.0023498535156250, 0.0024108886718750, 0.0022888183593750,
+0.0027465820312500, 0.0016784667968750, 0.0026550292968750, 0.0021667480468750,
+0.0026550292968750, 0.0016479492187500, 0.0033264160156250, 0.0016784667968750,
+0.0031433105468750, 0.0018005371093750, 0.0033569335937500, 0.0015563964843750,
+0.0039978027343750, 0.0016174316406250, 0.0037536621093750, 0.0016479492187500,
+0.0040893554687500, 0.0017089843750000, 0.0042114257812500, 0.0015869140625000,
+0.0041809082031250, 0.0015869140625000, 0.0043640136718750, 0.0017395019531250,
+0.0043334960937500, 0.0014953613281250, 0.0043334960937500, 0.0014953613281250,
+0.0043640136718750, 0.0016784667968750, 0.0043029785156250, 0.0015869140625000,
+0.0042114257812500, 0.0015563964843750, 0.0040893554687500, 0.0022277832031250,
+0.0041503906250000, 0.0019226074218750, 0.0039978027343750, 0.0024719238281250,
+0.0039672851562500, 0.0031738281250000, 0.0039672851562500, 0.0026855468750000,
+0.0038452148437500, 0.0039062500000000, 0.0039367675781250, 0.0037841796875000,
+0.0038452148437500, 0.0039062500000000, 0.0038757324218750, 0.0047302246093750,
+0.0040283203125000, 0.0042724609375000, 0.0039062500000000, 0.0044555664062500,
+0.0041503906250000, 0.0046997070312500, 0.0042419433593750, 0.0042419433593750,
+0.0042724609375000, 0.0039367675781250, 0.0044250488281250, 0.0041503906250000,
+0.0044250488281250, 0.0029907226562500, 0.0044555664062500, 0.0031738281250000,
+0.0044555664062500, 0.0025939941406250, 0.0044860839843750, 0.0015563964843750,
+0.0044250488281250, 0.0020751953125000, 0.0044555664062500, 0.0007324218750000,
+0.0044555664062500, 0.0007629394531250, 0.0044555664062500, 0.0005798339843750,
+0.0044555664062500, -0.0003662109375000, 0.0045166015625000, -0.0000915527343750,
+0.0045166015625000, -0.0005187988281250, 0.0045166015625000, -0.0007324218750000,
+0.0045471191406250, -0.0004882812500000, 0.0046081542968750, -0.0004272460937500,
+0.0046691894531250, -0.0006408691406250, 0.0046386718750000, 0.0001831054687500,
+0.0046997070312500, 0.0000305175781250, 0.0047607421875000, 0.0003662109375000,
+0.0046691894531250, 0.0010681152343750, 0.0045776367187500, 0.0007324218750000,
+0.0047302246093750, 0.0014953613281250, 0.0043945312500000, 0.0014648437500000,
+0.0044555664062500, 0.0015869140625000, 0.0043640136718750, 0.0020751953125000,
+0.0041809082031250, 0.0017700195312500, 0.0042724609375000, 0.0018920898437500,
+0.0040893554687500, 0.0020141601562500, 0.0041198730468750, 0.0018615722656250,
+0.0040893554687500, 0.0018005371093750, 0.0040283203125000, 0.0018615722656250,
+0.0040283203125000, 0.0017089843750000, 0.0038757324218750, 0.0016784667968750,
+0.0038757324218750, 0.0016784667968750, 0.0038146972656250, 0.0016174316406250,
+0.0036315917968750, 0.0016784667968750, 0.0037231445312500, 0.0014648437500000,
+0.0034790039062500},
+{0.0014343261718750, 0.0001831054687500, 0.0012512207031250, 0.0003356933593750,
+0.0008544921875000, 0.0001831054687500, 0.0012207031250000, -0.0006713867187500,
+0.0015869140625000, -0.0002746582031250, 0.0011291503906250, -0.0009155273437500,
+0.0024108886718750, -0.0008544921875000, 0.0021362304687500, -0.0008850097656250,
+0.0025634765625000, -0.0011291503906250, 0.0036010742187500, -0.0009765625000000,
+0.0030212402343750, -0.0014038085937500, 0.0037231445312500, -0.0011596679687500,
+0.0039367675781250, -0.0013427734375000, 0.0036926269531250, -0.0017089843750000,
+0.0036926269531250, -0.0014038085937500, 0.0037841796875000, -0.0020141601562500,
+0.0036315917968750, -0.0019226074218750, 0.0035400390625000, -0.0019226074218750,
+0.0037536621093750, -0.0023193359375000, 0.0037841796875000, -0.0020446777343750,
+0.0035400390625000, -0.0021362304687500, 0.0041809082031250, -0.0021972656250000,
+0.0039978027343750, -0.0017700195312500, 0.0042419433593750, -0.0016784667968750,
+0.0047607421875000, -0.0018615722656250, 0.0043945312500000, -0.0006103515625000,
+0.0049743652343750, -0.0009155273437500, 0.0050354003906250, -0.0004272460937500,
+0.0049133300781250, 0.0007324218750000, 0.0051879882812500, 0.0001220703125000,
+0.0051574707031250, 0.0012817382812500, 0.0048828125000000, 0.0014343261718750,
+0.0050964355468750, 0.0013732910156250, 0.0048217773437500, 0.0018920898437500,
+0.0043945312500000, 0.0018310546875000, 0.0047912597656250, 0.0016174316406250,
+0.0039672851562500, 0.0018615722656250, 0.0040588378906250, 0.0016479492187500,
+0.0040588378906250, 0.0012207031250000, 0.0035400390625000, 0.0016174316406250,
+0.0037231445312500, 0.0011901855468750, 0.0035705566406250, 0.0011291503906250,
+0.0034484863281250, 0.0014343261718750, 0.0035400390625000, 0.0014038085937500,
+0.0036315917968750, 0.0013427734375000, 0.0035400390625000, 0.0021057128906250,
+0.0038757324218750, 0.0019531250000000, 0.0038146972656250, 0.0022583007812500,
+0.0038452148437500, 0.0029296875000000, 0.0040588378906250, 0.0025329589843750,
+0.0040283203125000, 0.0031738281250000, 0.0039062500000000, 0.0032348632812500,
+0.0039978027343750, 0.0030212402343750, 0.0039367675781250, 0.0032653808593750,
+0.0038146972656250, 0.0031433105468750, 0.0040283203125000, 0.0026855468750000,
+0.0036621093750000, 0.0029602050781250, 0.0038757324218750, 0.0025024414062500,
+0.0037536621093750, 0.0019226074218750, 0.0035400390625000, 0.0024108886718750,
+0.0038146972656250, 0.0015869140625000, 0.0032043457031250, 0.0015869140625000,
+0.0033569335937500, 0.0016174316406250, 0.0033264160156250, 0.0011291503906250,
+0.0029296875000000, 0.0012817382812500, 0.0031127929687500, 0.0014343261718750,
+0.0029296875000000, 0.0012207031250000, 0.0029296875000000, 0.0014343261718750,
+0.0029296875000000, 0.0017089843750000, 0.0028686523437500, 0.0014648437500000,
+0.0029907226562500, 0.0020751953125000, 0.0028991699218750, 0.0019531250000000,
+0.0028991699218750, 0.0020141601562500, 0.0029296875000000, 0.0024719238281250,
+0.0028686523437500, 0.0021667480468750, 0.0029296875000000, 0.0023803710937500,
+0.0029907226562500, 0.0023803710937500, 0.0029602050781250, 0.0021972656250000,
+0.0029602050781250, 0.0021667480468750, 0.0030212402343750, 0.0021972656250000,
+0.0030212402343750, 0.0018310546875000, 0.0030822753906250, 0.0018310546875000,
+0.0031127929687500, 0.0016174316406250, 0.0030517578125000, 0.0013732910156250,
+0.0030517578125000, 0.0014648437500000, 0.0029907226562500, 0.0009155273437500,
+0.0029602050781250, 0.0010070800781250, 0.0029907226562500, 0.0007934570312500,
+0.0028991699218750, 0.0003356933593750, 0.0028686523437500, 0.0005493164062500,
+0.0029296875000000, 0.0002136230468750, 0.0028381347656250, 0.0000000000000000,
+0.0028381347656250, 0.0003356933593750, 0.0027770996093750, 0.0003967285156250,
+0.0027465820312500, 0.0001831054687500, 0.0028686523437500, 0.0012817382812500,
+0.0026245117187500, 0.0010986328125000, 0.0027465820312500, 0.0014648437500000,
+0.0026245117187500, 0.0024414062500000, 0.0024108886718750, 0.0020141601562500,
+0.0025634765625000, 0.0027160644531250, 0.0021057128906250, 0.0028686523437500,
+0.0021667480468750, 0.0027770996093750, 0.0021057128906250, 0.0030212402343750,
+0.0017395019531250, 0.0030212402343750, 0.0018615722656250, 0.0031127929687500,
+0.0019226074218750, 0.0029907226562500, 0.0017395019531250, 0.0032043457031250,
+0.0018310546875000, 0.0033874511718750, 0.0019836425781250, 0.0031433105468750,
+0.0017395019531250, 0.0036621093750000, 0.0020446777343750, 0.0036010742187500,
+0.0020446777343750, 0.0036926269531250, 0.0020141601562500, 0.0039672851562500,
+0.0021972656250000, 0.0037536621093750, 0.0021057128906250, 0.0039062500000000,
+0.0021972656250000, 0.0040588378906250, 0.0021972656250000, 0.0040588378906250,
+0.0020751953125000, 0.0040588378906250, 0.0020141601562500, 0.0040588378906250,
+0.0020141601562500, 0.0041198730468750, 0.0019226074218750, 0.0041503906250000,
+0.0017700195312500, 0.0039978027343750, 0.0018310546875000, 0.0039062500000000,
+0.0018615722656250, 0.0039978027343750, 0.0017089843750000, 0.0034790039062500,
+0.0019531250000000, 0.0034790039062500, 0.0018920898437500, 0.0033264160156250,
+0.0019531250000000, 0.0028686523437500, 0.0020141601562500, 0.0029296875000000,
+0.0018920898437500, 0.0028381347656250, 0.0022888183593750, 0.0028686523437500,
+0.0020141601562500, 0.0028076171875000, 0.0022888183593750, 0.0028076171875000,
+0.0027465820312500, 0.0028381347656250, 0.0023498535156250, 0.0024108886718750,
+0.0032043457031250, 0.0025939941406250, 0.0030822753906250, 0.0024414062500000,
+0.0032043457031250, 0.0018615722656250, 0.0037536621093750, 0.0019836425781250,
+0.0034179687500000, 0.0022277832031250, 0.0037536621093750, 0.0016174316406250,
+0.0038146972656250, 0.0020141601562500, 0.0037231445312500, 0.0026245117187500,
+0.0038757324218750, 0.0017700195312500, 0.0037536621093750, 0.0030822753906250,
+0.0039062500000000, 0.0028991699218750, 0.0038452148437500, 0.0029602050781250,
+0.0038146972656250, 0.0036621093750000, 0.0038452148437500, 0.0032348632812500,
+0.0037536621093750, 0.0037841796875000, 0.0037231445312500, 0.0038452148437500,
+0.0036926269531250, 0.0038146972656250, 0.0035095214843750, 0.0041198730468750,
+0.0034484863281250, 0.0039978027343750, 0.0033874511718750, 0.0039367675781250,
+0.0027465820312500, 0.0042114257812500, 0.0028686523437500, 0.0038757324218750,
+0.0024719238281250, 0.0034484863281250, 0.0018615722656250, 0.0038757324218750,
+0.0021362304687500, 0.0028381347656250, 0.0011901855468750, 0.0029907226562500,
+0.0012512207031250, 0.0030212402343750, 0.0010986328125000, 0.0024108886718750,
+0.0004272460937500, 0.0026855468750000, 0.0007324218750000, 0.0026855468750000,
+0.0003051757812500, 0.0027465820312500, 0.0003662109375000, 0.0028991699218750,
+0.0003356933593750, 0.0031127929687500, 0.0001525878906250, 0.0032958984375000,
+0.0003967285156250, 0.0032653808593750, 0.0000610351562500, 0.0035400390625000,
+0.0002441406250000, 0.0034179687500000, 0.0002441406250000, 0.0032043457031250,
+-0.0000915527343750, 0.0035400390625000, 0.0001525878906250, 0.0030822753906250,
+0.0000000000000000, 0.0030822753906250, -0.0000610351562500, 0.0031738281250000,
+0.0000000000000000, 0.0030517578125000, -0.0000610351562500, 0.0031433105468750,
+-0.0001525878906250, 0.0030517578125000, -0.0000305175781250, 0.0030212402343750,
+-0.0000610351562500, 0.0031127929687500, -0.0001220703125000, 0.0031738281250000,
+-0.0000915527343750, 0.0031433105468750, -0.0002136230468750, 0.0030212402343750,
+-0.0002441406250000, 0.0031738281250000, -0.0003051757812500, 0.0030517578125000,
+-0.0003051757812500, 0.0027770996093750, -0.0003051757812500, 0.0028991699218750,
+-0.0003662109375000, 0.0026550292968750, -0.0001220703125000, 0.0025939941406250,
+-0.0002441406250000, 0.0026855468750000, 0.0000000000000000, 0.0025634765625000,
+0.0003051757812500, 0.0025939941406250, 0.0000915527343750, 0.0028381347656250,
+0.0008850097656250, 0.0028076171875000, 0.0007629394531250, 0.0028991699218750,
+0.0009765625000000, 0.0032348632812500, 0.0015563964843750, 0.0032043457031250,
+0.0012207031250000, 0.0033264160156250, 0.0018920898437500, 0.0034179687500000,
+0.0018310546875000, 0.0034484863281250, 0.0018920898437500, 0.0034179687500000,
+0.0023498535156250, 0.0036010742187500, 0.0020446777343750, 0.0032958984375000,
+0.0023498535156250, 0.0033264160156250, 0.0023193359375000, 0.0033874511718750,
+0.0023498535156250, 0.0032653808593750, 0.0025329589843750, 0.0033264160156250,
+0.0023498535156250, 0.0032653808593750, 0.0027770996093750, 0.0032958984375000,
+0.0026855468750000, 0.0033569335937500, 0.0028991699218750, 0.0033874511718750,
+0.0032653808593750, 0.0033569335937500, 0.0030822753906250, 0.0033874511718750,
+0.0035400390625000, 0.0033569335937500, 0.0036621093750000, 0.0034484863281250,
+0.0036010742187500, 0.0035095214843750, 0.0038452148437500, 0.0034179687500000,
+0.0039367675781250, 0.0036926269531250, 0.0036926269531250, 0.0035705566406250,
+0.0038757324218750, 0.0036621093750000, 0.0037536621093750, 0.0039062500000000,
+0.0034790039062500, 0.0037231445312500, 0.0036010742187500, 0.0040283203125000,
+0.0031127929687500, 0.0039367675781250, 0.0032043457031250, 0.0039672851562500,
+0.0030822753906250, 0.0041503906250000, 0.0025939941406250, 0.0039367675781250,
+0.0028686523437500, 0.0041198730468750, 0.0023498535156250, 0.0040283203125000,
+0.0023803710937500, 0.0040283203125000, 0.0022888183593750, 0.0041198730468750,
+0.0019836425781250, 0.0039978027343750, 0.0021972656250000, 0.0041198730468750,
+0.0016784667968750, 0.0040588378906250, 0.0018310546875000, 0.0040283203125000,
+0.0015869140625000, 0.0040893554687500, 0.0011901855468750, 0.0039978027343750,
+0.0015258789062500, 0.0039672851562500, 0.0008239746093750, 0.0040283203125000,
+0.0008850097656250, 0.0038146972656250, 0.0007629394531250, 0.0036621093750000,
+0.0002441406250000, 0.0037841796875000, 0.0005187988281250, 0.0033264160156250,
+0.0003662109375000, 0.0033874511718750, 0.0001220703125000, 0.0032653808593750,
+0.0004272460937500, 0.0029602050781250, 0.0005798339843750, 0.0031127929687500,
+0.0003356933593750, 0.0029602050781250, 0.0010986328125000, 0.0029602050781250,
+0.0009765625000000, 0.0029602050781250, 0.0011596679687500, 0.0028991699218750,
+0.0018615722656250, 0.0029296875000000, 0.0015258789062500, 0.0029602050781250,
+0.0016784667968750, 0.0029602050781250, 0.0019836425781250, 0.0029296875000000,
+0.0014038085937500, 0.0029602050781250, 0.0010681152343750, 0.0029296875000000,
+0.0014648437500000, 0.0028381347656250, 0.0000915527343750, 0.0028381347656250,
+0.0002746582031250, 0.0028076171875000, 0.0000000000000000, 0.0026550292968750,
+-0.0009765625000000, 0.0027160644531250, -0.0005493164062500, 0.0025634765625000,
+-0.0008239746093750, 0.0025634765625000, -0.0010681152343750, 0.0025329589843750,
+-0.0006408691406250, 0.0024719238281250, -0.0003662109375000, 0.0025329589843750,
+-0.0005493164062500, 0.0023803710937500, 0.0004272460937500, 0.0024414062500000,
+0.0002746582031250, 0.0023803710937500, 0.0005493164062500, 0.0021972656250000,
+0.0012207031250000, 0.0022888183593750, 0.0008850097656250, 0.0023803710937500,
+0.0015563964843750, 0.0022277832031250, 0.0014038085937500, 0.0024414062500000,
+0.0016479492187500},
+{0.0013122558593750, 0.0005493164062500, 0.0011901855468750, 0.0006408691406250,
+0.0008544921875000, 0.0003356933593750, 0.0009765625000000, 0.0010681152343750,
+0.0007019042968750, 0.0007934570312500, 0.0006103515625000, 0.0009765625000000,
+0.0005493164062500, 0.0015563964843750, 0.0003662109375000, 0.0011596679687500,
+0.0003662109375000, 0.0018310546875000, 0.0005187988281250, 0.0018005371093750,
+0.0003356933593750, 0.0018310546875000, 0.0005798339843750, 0.0021972656250000,
+0.0009155273437500, 0.0020446777343750, 0.0006713867187500, 0.0022583007812500,
+0.0010986328125000, 0.0023498535156250, 0.0011291503906250, 0.0024108886718750,
+0.0012512207031250, 0.0025329589843750, 0.0014648437500000, 0.0026550292968750,
+0.0014343261718750, 0.0027465820312500, 0.0017700195312500, 0.0028381347656250,
+0.0017089843750000, 0.0029296875000000, 0.0019836425781250, 0.0030822753906250,
+0.0024108886718750, 0.0030212402343750, 0.0021362304687500, 0.0030822753906250,
+0.0031433105468750, 0.0031127929687500, 0.0030517578125000, 0.0030822753906250,
+0.0033874511718750, 0.0030822753906250, 0.0041809082031250, 0.0030822753906250,
+0.0038146972656250, 0.0030212402343750, 0.0045166015625000, 0.0029602050781250,
+0.0046386718750000, 0.0029907226562500, 0.0044860839843750, 0.0030212402343750,
+0.0046997070312500, 0.0029296875000000, 0.0046997070312500, 0.0030517578125000,
+0.0041809082031250, 0.0029602050781250, 0.0043640136718750, 0.0028991699218750,
+0.0039367675781250, 0.0029602050781250, 0.0033264160156250, 0.0028991699218750,
+0.0036315917968750, 0.0027160644531250, 0.0025024414062500, 0.0027160644531250,
+0.0025634765625000, 0.0025024414062500, 0.0022888183593750, 0.0023193359375000,
+0.0015258789062500, 0.0024108886718750, 0.0018005371093750, 0.0019531250000000,
+0.0012512207031250, 0.0019531250000000, 0.0011901855468750, 0.0019226074218750,
+0.0011596679687500, 0.0016174316406250, 0.0009155273437500, 0.0017395019531250,
+0.0010070800781250, 0.0015563964843750, 0.0007934570312500, 0.0015563964843750,
+0.0007934570312500, 0.0015258789062500, 0.0007629394531250, 0.0014953613281250,
+0.0005798339843750, 0.0015869140625000, 0.0006713867187500, 0.0013732910156250,
+0.0007324218750000, 0.0014953613281250, 0.0006408691406250, 0.0015258789062500,
+0.0008544921875000, 0.0013122558593750, 0.0008850097656250, 0.0014343261718750,
+0.0007019042968750, 0.0014343261718750, 0.0014953613281250, 0.0013732910156250,
+0.0013427734375000, 0.0014953613281250, 0.0016174316406250, 0.0015869140625000,
+0.0022583007812500, 0.0015563964843750, 0.0018615722656250, 0.0017089843750000,
+0.0024414062500000, 0.0016479492187500, 0.0025634765625000, 0.0016174316406250,
+0.0022277832031250, 0.0016784667968750, 0.0023193359375000, 0.0015869140625000,
+0.0022888183593750, 0.0015258789062500, 0.0014343261718750, 0.0014648437500000,
+0.0016479492187500, 0.0015563964843750, 0.0012817382812500, 0.0015563964843750,
+0.0005187988281250, 0.0014953613281250, 0.0008850097656250, 0.0018920898437500,
+0.0004577636718750, 0.0017089843750000, 0.0002441406250000, 0.0020446777343750,
+0.0005493164062500, 0.0025634765625000, 0.0007629394531250, 0.0021057128906250,
+0.0006408691406250, 0.0030822753906250, 0.0012207031250000, 0.0031127929687500,
+0.0012817382812500, 0.0031433105468750, 0.0012817382812500, 0.0036621093750000,
+0.0015563964843750, 0.0035095214843750, 0.0015563964843750, 0.0036010742187500,
+0.0014343261718750, 0.0036621093750000, 0.0014953613281250, 0.0035095214843750,
+0.0014343261718750, 0.0034179687500000, 0.0012817382812500, 0.0034484863281250,
+0.0013732910156250, 0.0032958984375000, 0.0016479492187500, 0.0032043457031250,
+0.0014953613281250, 0.0032043457031250, 0.0018920898437500, 0.0031433105468750,
+0.0024108886718750, 0.0029907226562500, 0.0021362304687500, 0.0030212402343750,
+0.0029296875000000, 0.0029907226562500, 0.0028991699218750, 0.0028381347656250,
+0.0031433105468750, 0.0028076171875000, 0.0037536621093750, 0.0028381347656250,
+0.0035095214843750, 0.0021057128906250, 0.0037231445312500, 0.0023498535156250,
+0.0039062500000000, 0.0018005371093750, 0.0036926269531250, 0.0010070800781250,
+0.0034790039062500, 0.0014953613281250, 0.0035400390625000, 0.0003967285156250,
+0.0034179687500000, 0.0003967285156250, 0.0033264160156250, 0.0004577636718750,
+0.0032958984375000, -0.0001220703125000, 0.0034179687500000, 0.0000305175781250,
+0.0032348632812500, 0.0003356933593750, 0.0030517578125000, 0.0000610351562500,
+0.0032348632812500, 0.0003356933593750, 0.0028991699218750, 0.0012207031250000,
+0.0020751953125000, 0.0023803710937500, 0.0030517578125000, 0.0026550292968750,
+0.0029296875000000, 0.0024719238281250, 0.0032653808593750, 0.0026550292968750,
+0.0028076171875000, 0.0027770996093750, 0.0022888183593750, 0.0025329589843750,
+0.0025024414062500, 0.0031433105468750, 0.0018615722656250, 0.0030212402343750,
+0.0016784667968750, 0.0030822753906250, 0.0018005371093750, 0.0034484863281250,
+0.0017089843750000, 0.0032043457031250, 0.0015258789062500, 0.0037536621093750,
+0.0013732910156250, 0.0036315917968750, 0.0014953613281250, 0.0037841796875000,
+0.0013122558593750, 0.0042114257812500, 0.0010070800781250, 0.0039978027343750,
+0.0010986328125000, 0.0039978027343750, 0.0010375976562500, 0.0043029785156250,
+0.0006713867187500, 0.0037536621093750, 0.0012512207031250, 0.0033569335937500,
+0.0016784667968750, 0.0038452148437500, 0.0011291503906250, 0.0022583007812500,
+0.0030212402343750, 0.0025024414062500, 0.0028686523437500, 0.0022277832031250,
+0.0030212402343750, 0.0010681152343750, 0.0041503906250000, 0.0017089843750000,
+0.0036926269531250, 0.0010070800781250, 0.0043029785156250, 0.0007934570312500,
+0.0044250488281250, 0.0010070800781250, 0.0041198730468750, 0.0008850097656250,
+0.0042114257812500, 0.0007934570312500, 0.0043640136718750, 0.0008850097656250,
+0.0037536621093750, 0.0008850097656250, 0.0038452148437500, 0.0006713867187500,
+0.0037841796875000, 0.0006103515625000, 0.0033264160156250, 0.0006713867187500,
+0.0035095214843750, 0.0002441406250000, 0.0036621093750000, 0.0001525878906250,
+0.0032958984375000, 0.0001525878906250, 0.0036621093750000, -0.0000915527343750,
+0.0043334960937500, -0.0001220703125000, 0.0039672851562500, -0.0000305175781250,
+0.0041198730468750, -0.0002136230468750, 0.0044250488281250, 0.0000305175781250,
+0.0040588378906250, 0.0002136230468750, 0.0036621093750000, -0.0001525878906250,
+0.0038757324218750, 0.0004882812500000, 0.0032653808593750, 0.0003662109375000,
+0.0030212402343750, 0.0003051757812500, 0.0031433105468750, 0.0007019042968750,
+0.0028991699218750, 0.0003967285156250, 0.0026855468750000, 0.0003051757812500,
+0.0033569335937500, 0.0003356933593750, 0.0030822753906250, 0.0001525878906250,
+0.0031127929687500, -0.0000915527343750, 0.0036315917968750, -0.0000915527343750,
+0.0032043457031250, -0.0002136230468750, 0.0032348632812500, -0.0005187988281250,
+0.0033569335937500, -0.0000915527343750, 0.0028991699218750, 0.0001220703125000,
+0.0024719238281250, -0.0003356933593750, 0.0026550292968750, 0.0014343261718750,
+0.0018615722656250, 0.0008239746093750, 0.0018615722656250, 0.0015258789062500,
+0.0018005371093750, 0.0030822753906250, 0.0014038085937500, 0.0019531250000000,
+0.0014953613281250, 0.0037536621093750, 0.0016479492187500, 0.0038146972656250,
+0.0015258789062500, 0.0035095214843750, 0.0016784667968750, 0.0042724609375000,
+0.0019836425781250, 0.0039367675781250, 0.0020751953125000, 0.0033569335937500,
+0.0020446777343750, 0.0036315917968750, 0.0022277832031250, 0.0028076171875000,
+0.0022277832031250, 0.0018920898437500, 0.0021362304687500, 0.0025024414062500,
+0.0022277832031250, 0.0007934570312500, 0.0021667480468750, 0.0007629394531250,
+0.0021667480468750, 0.0006103515625000, 0.0022888183593750, -0.0003662109375000,
+0.0021667480468750, -0.0000915527343750, 0.0021057128906250, -0.0003356933593750,
+0.0023498535156250, -0.0006408691406250, 0.0021667480468750, -0.0003051757812500,
+0.0023498535156250, -0.0000915527343750, 0.0026245117187500, -0.0003967285156250,
+0.0023193359375000, 0.0003967285156250, 0.0028381347656250, 0.0003051757812500,
+0.0028381347656250, 0.0003662109375000, 0.0028381347656250, 0.0008544921875000,
+0.0031127929687500, 0.0006103515625000, 0.0028991699218750, 0.0005798339843750,
+0.0029602050781250, 0.0007934570312500, 0.0029602050781250, 0.0004882812500000,
+0.0028991699218750, 0.0001220703125000, 0.0028076171875000, 0.0003662109375000,
+0.0027770996093750, -0.0002136230468750, 0.0026550292968750, -0.0002441406250000,
+0.0025634765625000, -0.0000915527343750, 0.0025024414062500, -0.0003967285156250,
+0.0023498535156250, -0.0003662109375000, 0.0023193359375000, 0.0001525878906250,
+0.0022583007812500, -0.0001220703125000, 0.0021972656250000, 0.0004882812500000,
+0.0022583007812500, 0.0012512207031250, 0.0022888183593750, 0.0008239746093750,
+0.0022277832031250, 0.0025634765625000, 0.0022888183593750, 0.0022888183593750,
+0.0023193359375000, 0.0028991699218750, 0.0022277832031250, 0.0043029785156250,
+0.0022583007812500, 0.0036010742187500, 0.0022888183593750, 0.0051269531250000,
+0.0021972656250000, 0.0051574707031250, 0.0022583007812500, 0.0051574707031250,
+0.0022583007812500, 0.0058593750000000, 0.0021667480468750, 0.0055847167968750,
+0.0022277832031250, 0.0054626464843750, 0.0023193359375000, 0.0057678222656250,
+0.0022888183593750, 0.0053100585937500, 0.0023498535156250, 0.0049133300781250,
+0.0024108886718750, 0.0052185058593750, 0.0023498535156250, 0.0041809082031250,
+0.0025024414062500, 0.0043945312500000, 0.0025024414062500, 0.0040893554687500,
+0.0025939941406250, 0.0033264160156250, 0.0027465820312500, 0.0037536621093750,
+0.0026245117187500, 0.0031738281250000, 0.0028686523437500, 0.0031127929687500,
+0.0028381347656250, 0.0032958984375000, 0.0028076171875000, 0.0031127929687500,
+0.0028686523437500, 0.0031433105468750, 0.0027770996093750, 0.0033569335937500,
+0.0028076171875000, 0.0033264160156250, 0.0027160644531250, 0.0034179687500000,
+0.0026550292968750, 0.0036010742187500, 0.0027160644531250, 0.0036315917968750,
+0.0025939941406250, 0.0037536621093750, 0.0026855468750000, 0.0038757324218750,
+0.0025939941406250, 0.0038146972656250, 0.0025634765625000, 0.0038452148437500,
+0.0026245117187500, 0.0039672851562500, 0.0024719238281250, 0.0037841796875000,
+0.0024108886718750, 0.0038757324218750, 0.0023803710937500, 0.0038146972656250,
+0.0022583007812500, 0.0036315917968750, 0.0021667480468750, 0.0037231445312500,
+0.0021362304687500, 0.0031433105468750, 0.0020751953125000, 0.0032653808593750,
+0.0019836425781250, 0.0029602050781250, 0.0020141601562500, 0.0022888183593750,
+0.0020446777343750, 0.0025634765625000, 0.0020141601562500, 0.0019226074218750,
+0.0021667480468750, 0.0016479492187500, 0.0020751953125000, 0.0018005371093750,
+0.0020751953125000, 0.0015869140625000, 0.0021972656250000, 0.0014953613281250,
+0.0021057128906250, 0.0021057128906250, 0.0021362304687500, 0.0018920898437500,
+0.0021362304687500, 0.0021057128906250, 0.0020751953125000, 0.0025939941406250,
+0.0020446777343750, 0.0021972656250000, 0.0020751953125000, 0.0026550292968750,
+0.0019226074218750, 0.0027160644531250, 0.0019226074218750, 0.0025329589843750,
+0.0020141601562500, 0.0026245117187500, 0.0018920898437500, 0.0025939941406250,
+0.0018920898437500},
+{0.0029602050781250, 0.0053100585937500, 0.0032043457031250, 0.0043334960937500,
+0.0028686523437500, 0.0045776367187500, 0.0028686523437500, 0.0040893554687500,
+0.0029907226562500, 0.0031433105468750, 0.0028381347656250, 0.0036010742187500,
+0.0028991699218750, 0.0025024414062500, 0.0032043457031250, 0.0023803710937500,
+0.0030822753906250, 0.0023498535156250, 0.0033264160156250, 0.0017700195312500,
+0.0036315917968750, 0.0018615722656250, 0.0034790039062500, 0.0016479492187500,
+0.0039062500000000, 0.0016174316406250, 0.0038452148437500, 0.0016479492187500,
+0.0038452148437500, 0.0016174316406250, 0.0040893554687500, 0.0017395019531250,
+0.0039062500000000, 0.0016479492187500, 0.0040283203125000, 0.0018005371093750,
+0.0040283203125000, 0.0018615722656250, 0.0039367675781250, 0.0018005371093750,
+0.0039367675781250, 0.0019836425781250, 0.0039367675781250, 0.0020141601562500,
+0.0040588378906250, 0.0020751953125000, 0.0039672851562500, 0.0021667480468750,
+0.0040588378906250, 0.0022583007812500, 0.0042114257812500, 0.0022888183593750,
+0.0041198730468750, 0.0024719238281250, 0.0041503906250000, 0.0025329589843750,
+0.0041809082031250, 0.0025634765625000, 0.0041198730468750, 0.0027770996093750,
+0.0040283203125000, 0.0028381347656250, 0.0040893554687500, 0.0027770996093750,
+0.0039062500000000, 0.0029907226562500, 0.0038757324218750, 0.0027160644531250,
+0.0037841796875000, 0.0025329589843750, 0.0036926269531250, 0.0028686523437500,
+0.0036926269531250, 0.0020446777343750, 0.0035400390625000, 0.0022888183593750,
+0.0035400390625000, 0.0020446777343750, 0.0035095214843750, 0.0013122558593750,
+0.0032958984375000, 0.0017395019531250, 0.0033264160156250, 0.0010681152343750,
+0.0032348632812500, 0.0009765625000000, 0.0031127929687500, 0.0010986328125000,
+0.0030212402343750, 0.0008850097656250, 0.0028991699218750, 0.0009460449218750,
+0.0027465820312500, 0.0014038085937500, 0.0028381347656250, 0.0012207031250000,
+0.0026855468750000, 0.0016174316406250, 0.0026245117187500, 0.0022277832031250,
+0.0026550292968750, 0.0018920898437500, 0.0024719238281250, 0.0029296875000000,
+0.0025939941406250, 0.0029602050781250, 0.0024414062500000, 0.0030212402343750,
+0.0024108886718750, 0.0036621093750000, 0.0025024414062500, 0.0035095214843750,
+0.0023193359375000, 0.0037231445312500, 0.0025024414062500, 0.0038452148437500,
+0.0023193359375000, 0.0037841796875000, 0.0023803710937500, 0.0038146972656250,
+0.0025634765625000, 0.0037536621093750, 0.0022888183593750, 0.0038452148437500,
+0.0026550292968750, 0.0037841796875000, 0.0025024414062500, 0.0038452148437500,
+0.0025329589843750, 0.0040283203125000, 0.0027770996093750, 0.0038146972656250,
+0.0024414062500000, 0.0038757324218750, 0.0028381347656250, 0.0038757324218750,
+0.0026245117187500, 0.0034484863281250, 0.0026550292968750, 0.0032348632812500,
+0.0028991699218750, 0.0035400390625000, 0.0025634765625000, 0.0028686523437500,
+0.0028686523437500, 0.0028991699218750, 0.0028076171875000, 0.0029907226562500,
+0.0027160644531250, 0.0026550292968750, 0.0028686523437500, 0.0026550292968750,
+0.0026855468750000, 0.0029907226562500, 0.0026245117187500, 0.0027160644531250,
+0.0026245117187500, 0.0029296875000000, 0.0025329589843750, 0.0033874511718750,
+0.0024414062500000, 0.0030822753906250, 0.0024108886718750, 0.0033874511718750,
+0.0023803710937500, 0.0034484863281250, 0.0022888183593750, 0.0031738281250000,
+0.0022277832031250, 0.0031433105468750, 0.0021972656250000, 0.0032653808593750,
+0.0021667480468750, 0.0026245117187500, 0.0021057128906250, 0.0028076171875000,
+0.0021972656250000, 0.0025024414062500, 0.0019836425781250, 0.0018920898437500,
+0.0018310546875000, 0.0022888183593750, 0.0021057128906250, 0.0016784667968750,
+0.0013122558593750, 0.0016174316406250, 0.0015563964843750, 0.0017700195312500,
+0.0014038085937500, 0.0015869140625000, 0.0007019042968750, 0.0015563964843750,
+0.0010986328125000, 0.0020141601562500, 0.0007019042968750, 0.0017700195312500,
+0.0007629394531250, 0.0018005371093750, 0.0008239746093750, 0.0020751953125000,
+0.0006408691406250, 0.0017700195312500, 0.0007629394531250, 0.0022888183593750,
+0.0007324218750000, 0.0021057128906250, 0.0007934570312500, 0.0021667480468750,
+0.0008239746093750, 0.0026855468750000, 0.0007934570312500, 0.0024719238281250,
+0.0009460449218750, 0.0024414062500000, 0.0008239746093750, 0.0029907226562500,
+0.0008850097656250, 0.0024108886718750, 0.0010375976562500, 0.0019226074218750,
+0.0010070800781250, 0.0028686523437500, 0.0010375976562500, 0.0014648437500000,
+0.0014038085937500, 0.0015258789062500, 0.0013427734375000, 0.0018310546875000,
+0.0016174316406250, 0.0011596679687500, 0.0020141601562500, 0.0013732910156250,
+0.0018310546875000, 0.0018920898437500, 0.0026245117187500, 0.0014648437500000,
+0.0025939941406250, 0.0018005371093750, 0.0027465820312500, 0.0024719238281250,
+0.0033569335937500, 0.0019531250000000, 0.0032043457031250, 0.0025939941406250,
+0.0034179687500000, 0.0025939941406250, 0.0036010742187500, 0.0025329589843750,
+0.0035705566406250, 0.0028381347656250, 0.0036926269531250, 0.0026245117187500,
+0.0038146972656250, 0.0027770996093750, 0.0036315917968750, 0.0027770996093750,
+0.0037231445312500, 0.0027770996093750, 0.0038146972656250, 0.0028686523437500,
+0.0037841796875000, 0.0026855468750000, 0.0038146972656250, 0.0029296875000000,
+0.0039672851562500, 0.0027465820312500, 0.0040283203125000, 0.0028381347656250,
+0.0040588378906250, 0.0030822753906250, 0.0041809082031250, 0.0028686523437500,
+0.0041809082031250, 0.0030212402343750, 0.0041198730468750, 0.0033264160156250,
+0.0042419433593750, 0.0029602050781250, 0.0043334960937500, 0.0027770996093750,
+0.0043334960937500, 0.0033264160156250, 0.0044860839843750, 0.0023498535156250,
+0.0047302246093750, 0.0026245117187500, 0.0046691894531250, 0.0025939941406250,
+0.0050659179687500, 0.0019531250000000, 0.0054931640625000, 0.0025329589843750,
+0.0053100585937500, 0.0022277832031250, 0.0059509277343750, 0.0021972656250000,
+0.0060119628906250, 0.0025634765625000, 0.0060729980468750, 0.0028381347656250,
+0.0064086914062500, 0.0027465820312500, 0.0062866210937500, 0.0032653808593750,
+0.0060424804687500, 0.0033569335937500, 0.0061950683593750, 0.0034179687500000,
+0.0059204101562500, 0.0036621093750000, 0.0054931640625000, 0.0037841796875000,
+0.0057067871093750, 0.0035095214843750, 0.0051574707031250, 0.0037231445312500,
+0.0051269531250000, 0.0037536621093750, 0.0051879882812500, 0.0034484863281250,
+0.0049438476562500, 0.0036621093750000, 0.0050659179687500, 0.0035705566406250,
+0.0049743652343750, 0.0035400390625000, 0.0050964355468750, 0.0035705566406250,
+0.0049743652343750, 0.0036315917968750, 0.0048522949218750, 0.0035705566406250,
+0.0051269531250000, 0.0036010742187500, 0.0044555664062500, 0.0035400390625000,
+0.0048217773437500, 0.0034179687500000, 0.0044250488281250, 0.0033569335937500,
+0.0037231445312500, 0.0032653808593750, 0.0043029785156250, 0.0030822753906250,
+0.0029602050781250, 0.0029602050781250, 0.0032348632812500, 0.0028991699218750,
+0.0029602050781250, 0.0027160644531250, 0.0020141601562500, 0.0024719238281250,
+0.0025634765625000, 0.0027770996093750, 0.0016479492187500, 0.0024719238281250,
+0.0017395019531250, 0.0025329589843750, 0.0015869140625000, 0.0028076171875000,
+0.0010375976562500, 0.0023193359375000, 0.0013732910156250, 0.0028991699218750,
+0.0004272460937500, 0.0026550292968750, 0.0005187988281250, 0.0025939941406250,
+0.0003051757812500, 0.0029296875000000, -0.0004272460937500, 0.0025329589843750,
+-0.0002136230468750, 0.0027770996093750, -0.0004577636718750, 0.0027160644531250,
+-0.0007629394531250, 0.0026245117187500, -0.0004272460937500, 0.0027160644531250,
+-0.0003051757812500, 0.0025329589843750, -0.0006408691406250, 0.0025329589843750,
+0.0005798339843750, 0.0025634765625000, 0.0001220703125000, 0.0025024414062500,
+0.0007629394531250, 0.0025024414062500, 0.0019531250000000, 0.0025024414062500,
+0.0011291503906250, 0.0023803710937500, 0.0030822753906250, 0.0025024414062500,
+0.0028991699218750, 0.0024108886718750, 0.0032348632812500, 0.0022277832031250,
+0.0045776367187500, 0.0023803710937500, 0.0039672851562500, 0.0021972656250000,
+0.0046997070312500, 0.0022583007812500, 0.0048217773437500, 0.0022888183593750,
+0.0045776367187500, 0.0021362304687500, 0.0047302246093750, 0.0023498535156250,
+0.0048217773437500, 0.0022277832031250, 0.0041503906250000, 0.0022277832031250,
+0.0043334960937500, 0.0021972656250000, 0.0038757324218750, 0.0021057128906250,
+0.0031433105468750, 0.0021362304687500, 0.0035400390625000, 0.0022277832031250,
+0.0025024414062500, 0.0021057128906250, 0.0024108886718750, 0.0021667480468750,
+0.0022583007812500, 0.0022888183593750, 0.0017395019531250, 0.0020141601562500,
+0.0019531250000000, 0.0023193359375000, 0.0014648437500000, 0.0021057128906250,
+0.0014038085937500, 0.0020446777343750, 0.0014953613281250, 0.0021667480468750,
+0.0013427734375000, 0.0019531250000000, 0.0013122558593750, 0.0020141601562500,
+0.0014953613281250, 0.0019226074218750, 0.0014038085937500, 0.0018615722656250,
+0.0014343261718750, 0.0018615722656250, 0.0016174316406250, 0.0017700195312500,
+0.0014953613281250, 0.0018310546875000, 0.0013732910156250, 0.0017395019531250,
+0.0014953613281250, 0.0018615722656250, 0.0013732910156250, 0.0020141601562500,
+0.0011596679687500, 0.0019531250000000, 0.0013122558593750, 0.0022277832031250,
+0.0011596679687500, 0.0023193359375000, 0.0010070800781250, 0.0022277832031250,
+0.0012207031250000, 0.0022277832031250, 0.0013427734375000, 0.0023193359375000,
+0.0011291503906250, 0.0020141601562500, 0.0017395019531250, 0.0020751953125000,
+0.0016174316406250, 0.0020446777343750, 0.0016479492187500, 0.0017700195312500,
+0.0019836425781250, 0.0018920898437500, 0.0016784667968750, 0.0021057128906250,
+0.0018615722656250, 0.0018615722656250, 0.0017089843750000, 0.0019836425781250,
+0.0017089843750000, 0.0022583007812500, 0.0017395019531250, 0.0018920898437500,
+0.0015563964843750, 0.0024414062500000, 0.0019836425781250, 0.0023193359375000,
+0.0016784667968750, 0.0022277832031250, 0.0019531250000000, 0.0024414062500000,
+0.0025329589843750, 0.0023193359375000, 0.0020446777343750, 0.0020751953125000,
+0.0029602050781250, 0.0021972656250000, 0.0028686523437500, 0.0019226074218750,
+0.0027160644531250, 0.0015869140625000, 0.0031433105468750, 0.0016784667968750,
+0.0029907226562500, 0.0013732910156250, 0.0031738281250000, 0.0011291503906250,
+0.0031433105468750, 0.0013122558593750, 0.0030212402343750, 0.0012817382812500,
+0.0031738281250000, 0.0010070800781250, 0.0031433105468750, 0.0017395019531250,
+0.0026550292968750, 0.0014038085937500, 0.0028076171875000, 0.0016479492187500,
+0.0026245117187500, 0.0023193359375000, 0.0021667480468750, 0.0018310546875000,
+0.0023193359375000, 0.0025939941406250, 0.0020141601562500, 0.0025329589843750,
+0.0019226074218750, 0.0026245117187500, 0.0021362304687500, 0.0030517578125000,
+0.0021667480468750, 0.0028076171875000, 0.0021057128906250, 0.0032958984375000,
+0.0024108886718750, 0.0032653808593750, 0.0023803710937500, 0.0032653808593750,
+0.0023193359375000, 0.0037841796875000, 0.0024719238281250, 0.0038757324218750,
+0.0039978027343750, 0.0034790039062500, 0.0031738281250000, 0.0037231445312500,
+0.0036315917968750},
+{-0.0010681152343750, 0.0026855468750000, -0.0011901855468750, 0.0027770996093750,
+-0.0011596679687500, 0.0028381347656250, -0.0014343261718750, 0.0028381347656250,
+-0.0014343261718750, 0.0029296875000000, -0.0010681152343750, 0.0029907226562500,
+-0.0013122558593750, 0.0030517578125000, -0.0009460449218750, 0.0031127929687500,
+-0.0004272460937500, 0.0032043457031250, -0.0007629394531250, 0.0032958984375000,
+0.0004577636718750, 0.0032958984375000, 0.0002746582031250, 0.0033569335937500,
+0.0005187988281250, 0.0034790039062500, 0.0013427734375000, 0.0033569335937500,
+0.0009460449218750, 0.0035095214843750, 0.0014038085937500, 0.0034179687500000,
+0.0015869140625000, 0.0033874511718750, 0.0014038085937500, 0.0033874511718750,
+0.0014953613281250, 0.0032348632812500, 0.0015563964843750, 0.0032043457031250,
+0.0012817382812500, 0.0031738281250000, 0.0014648437500000, 0.0031127929687500,
+0.0012207031250000, 0.0030822753906250, 0.0009155273437500, 0.0030212402343750,
+0.0011901855468750, 0.0029602050781250, 0.0005493164062500, 0.0029602050781250,
+0.0006713867187500, 0.0029602050781250, 0.0006103515625000, 0.0029296875000000,
+0.0002136230468750, 0.0030517578125000, 0.0003967285156250, 0.0028381347656250,
+0.0002746582031250, 0.0030212402343750, 0.0001831054687500, 0.0028381347656250,
+0.0003051757812500, 0.0026245117187500, 0.0003051757812500, 0.0029602050781250,
+0.0002746582031250, 0.0022277832031250, 0.0003967285156250, 0.0024108886718750,
+0.0003051757812500, 0.0021362304687500, 0.0003356933593750, 0.0015258789062500,
+0.0003967285156250, 0.0018005371093750, 0.0002746582031250, 0.0013427734375000,
+0.0003967285156250, 0.0012512207031250, 0.0002441406250000, 0.0013427734375000,
+0.0003356933593750, 0.0011901855468750, 0.0004882812500000, 0.0011291503906250,
+0.0003051757812500, 0.0012512207031250, 0.0005493164062500, 0.0012207031250000,
+0.0005187988281250, 0.0011596679687500, 0.0004272460937500, 0.0011901855468750,
+0.0005187988281250, 0.0011901855468750, 0.0004882812500000, 0.0010070800781250,
+0.0003662109375000, 0.0010070800781250, 0.0003967285156250, 0.0009765625000000,
+0.0003662109375000, 0.0007934570312500, 0.0001220703125000, 0.0008544921875000,
+0.0000610351562500, 0.0010070800781250, 0.0002746582031250, 0.0007019042968750,
+0.0000915527343750, 0.0010681152343750, 0.0003356933593750, 0.0014343261718750,
+0.0006713867187500, 0.0008850097656250, 0.0004577636718750, 0.0019836425781250,
+0.0010070800781250, 0.0016479492187500, 0.0009155273437500, 0.0017700195312500,
+0.0013427734375000, 0.0025024414062500, 0.0018920898437500, 0.0019531250000000,
+0.0015258789062500, 0.0025939941406250, 0.0028686523437500, 0.0024719238281250,
+0.0028076171875000, 0.0025024414062500, 0.0029907226562500, 0.0028686523437500,
+0.0037536621093750, 0.0025634765625000, 0.0034179687500000, 0.0032043457031250,
+0.0038452148437500, 0.0028991699218750, 0.0038452148437500, 0.0031738281250000,
+0.0037231445312500, 0.0037536621093750, 0.0039062500000000, 0.0032653808593750,
+0.0037536621093750, 0.0040283203125000, 0.0038146972656250, 0.0039978027343750,
+0.0038452148437500, 0.0037231445312500, 0.0037841796875000, 0.0042114257812500,
+0.0039062500000000, 0.0021972656250000, 0.0030822753906250, 0.0015258789062500,
+0.0038146972656250, 0.0016174316406250, 0.0038146972656250, 0.0013122558593750,
+0.0035705566406250, 0.0009460449218750, 0.0037231445312500, 0.0010375976562500,
+0.0039062500000000, 0.0005493164062500, 0.0031127929687500, 0.0005187988281250,
+0.0033569335937500, 0.0005493164062500, 0.0032043457031250, 0.0003051757812500,
+0.0025329589843750, 0.0003051757812500, 0.0029296875000000, 0.0006713867187500,
+0.0031433105468750, 0.0004882812500000, 0.0028381347656250, 0.0006408691406250,
+0.0032043457031250, 0.0010375976562500, 0.0037536621093750, 0.0009155273437500,
+0.0034790039062500, 0.0010375976562500, 0.0039367675781250, 0.0010375976562500,
+0.0039062500000000, 0.0012512207031250, 0.0038757324218750, 0.0013122558593750,
+0.0040283203125000, 0.0011291503906250, 0.0037841796875000, 0.0021667480468750,
+0.0039367675781250, 0.0016784667968750, 0.0040283203125000, 0.0021667480468750,
+0.0037536621093750, 0.0031433105468750, 0.0036010742187500, 0.0024108886718750,
+0.0037841796875000, 0.0037231445312500, 0.0032958984375000, 0.0035095214843750,
+0.0033264160156250, 0.0035705566406250, 0.0028991699218750, 0.0043945312500000,
+0.0023803710937500, 0.0039978027343750, 0.0026550292968750, 0.0042419433593750,
+0.0018310546875000, 0.0044555664062500, 0.0015563964843750, 0.0041503906250000,
+0.0017089843750000, 0.0039672851562500, 0.0014953613281250, 0.0041503906250000,
+0.0014648437500000, 0.0036621093750000, 0.0020141601562500, 0.0037231445312500,
+0.0016784667968750, 0.0036621093750000, 0.0023498535156250, 0.0034179687500000,
+0.0032348632812500, 0.0035095214843750, 0.0027770996093750, 0.0036010742187500,
+0.0039978027343750, 0.0034179687500000, 0.0043945312500000, 0.0036315917968750,
+0.0044250488281250, 0.0039062500000000, 0.0049438476562500, 0.0037536621093750,
+0.0050048828125000, 0.0040283203125000, 0.0043029785156250, 0.0042114257812500,
+0.0048828125000000, 0.0039672851562500, 0.0045471191406250, 0.0040283203125000,
+0.0036621093750000, 0.0042724609375000, 0.0042419433593750, 0.0032653808593750,
+0.0041809082031250, 0.0037536621093750, 0.0038757324218750, 0.0031127929687500,
+0.0043945312500000, 0.0020141601562500, 0.0048828125000000, 0.0028381347656250,
+0.0046081542968750, 0.0010986328125000, 0.0052795410156250, 0.0012207031250000,
+0.0053710937500000, 0.0011596679687500, 0.0052490234375000, 0.0001525878906250,
+0.0054016113281250, 0.0006408691406250, 0.0054016113281250, 0.0005493164062500,
+0.0048217773437500, 0.0003356933593750, 0.0049743652343750, 0.0008239746093750,
+0.0046386718750000, 0.0012207031250000, 0.0040588378906250, 0.0009765625000000,
+0.0042724609375000, 0.0015258789062500, 0.0036926269531250, 0.0018310546875000,
+0.0036926269531250, 0.0016174316406250, 0.0036621093750000, 0.0017395019531250,
+0.0035095214843750, 0.0020446777343750, 0.0036926269531250, 0.0011901855468750,
+0.0035705566406250, 0.0015563964843750, 0.0037536621093750, 0.0012817382812500,
+0.0036621093750000, 0.0005798339843750, 0.0034179687500000, 0.0011291503906250,
+0.0036315917968750, 0.0007324218750000, 0.0031433105468750, 0.0007019042968750,
+0.0032958984375000, 0.0010070800781250, 0.0032958984375000, 0.0010070800781250,
+0.0029296875000000, 0.0010681152343750, 0.0031127929687500, 0.0016479492187500,
+0.0030822753906250, 0.0016479492187500, 0.0029907226562500, 0.0019531250000000,
+0.0030822753906250, 0.0025024414062500, 0.0033264160156250, 0.0024414062500000,
+0.0032958984375000, 0.0027770996093750, 0.0035095214843750, 0.0031738281250000,
+0.0036010742187500, 0.0029296875000000, 0.0035095214843750, 0.0028381347656250,
+0.0036010742187500, 0.0033264160156250, 0.0038146972656250, 0.0024108886718750,
+0.0032958984375000, 0.0027465820312500, 0.0035095214843750, 0.0025024414062500,
+0.0033874511718750, 0.0017395019531250, 0.0029296875000000, 0.0022277832031250,
+0.0033264160156250, 0.0012512207031250, 0.0029296875000000, 0.0013732910156250,
+0.0029602050781250, 0.0010681152343750, 0.0029602050781250, 0.0003356933593750,
+0.0028076171875000, 0.0007324218750000, 0.0029602050781250, -0.0003356933593750,
+0.0028686523437500, -0.0002746582031250, 0.0028686523437500, -0.0005798339843750,
+0.0029907226562500, -0.0014038085937500, 0.0030212402343750, -0.0011596679687500,
+0.0030822753906250, -0.0020751953125000, 0.0032043457031250, -0.0020446777343750,
+0.0031738281250000, -0.0022583007812500, 0.0032653808593750, -0.0028991699218750,
+0.0033569335937500, -0.0026550292968750, 0.0032958984375000, -0.0031127929687500,
+0.0033264160156250, -0.0032043457031250, 0.0032653808593750, -0.0030822753906250,
+0.0032348632812500, -0.0032043457031250, 0.0032348632812500, -0.0031433105468750,
+0.0032348632812500, -0.0028076171875000, 0.0031433105468750, -0.0030212402343750,
+0.0031433105468750, -0.0026855468750000, 0.0031127929687500, -0.0022277832031250,
+0.0030517578125000, -0.0026245117187500, 0.0030212402343750, -0.0015563964843750,
+0.0029907226562500, -0.0018615722656250, 0.0028991699218750, -0.0014038085937500,
+0.0028991699218750, -0.0004272460937500, 0.0028686523437500, -0.0010681152343750,
+0.0028076171875000, 0.0004577636718750, 0.0028381347656250, 0.0003356933593750,
+0.0027465820312500, 0.0005187988281250, 0.0027465820312500, 0.0014038085937500,
+0.0027770996093750, 0.0010375976562500, 0.0027160644531250, 0.0013732910156250,
+0.0027770996093750, 0.0016479492187500, 0.0026550292968750, 0.0012817382812500,
+0.0026855468750000, 0.0011291503906250, 0.0028076171875000, 0.0014038085937500,
+0.0027160644531250, 0.0005493164062500, 0.0028076171875000, 0.0006408691406250,
+0.0028381347656250, 0.0006408691406250, 0.0027770996093750, 0.0000610351562500,
+0.0027465820312500, 0.0002441406250000, 0.0027770996093750, 0.0004272460937500,
+0.0026550292968750, 0.0001525878906250, 0.0026855468750000, 0.0005187988281250,
+0.0026245117187500, 0.0009155273437500, 0.0025634765625000, 0.0005493164062500,
+0.0025939941406250, 0.0012817382812500, 0.0025329589843750, 0.0012817382812500,
+0.0026245117187500, 0.0012207031250000, 0.0025024414062500, 0.0016174316406250,
+0.0023803710937500, 0.0014648437500000, 0.0025939941406250, 0.0013732910156250,
+0.0021667480468750, 0.0014038085937500, 0.0023193359375000, 0.0013122558593750,
+0.0021972656250000, 0.0011291503906250, 0.0018920898437500, 0.0011291503906250,
+0.0021972656250000, 0.0007934570312500, 0.0018920898437500, 0.0007934570312500,
+0.0019226074218750, 0.0006408691406250, 0.0020141601562500, 0.0004577636718750,
+0.0018615722656250, 0.0006408691406250, 0.0019226074218750, -0.0001220703125000,
+0.0018005371093750, 0.0001220703125000, 0.0019226074218750, -0.0000915527343750,
+0.0018920898437500, -0.0008239746093750, 0.0017700195312500, -0.0005187988281250,
+0.0019531250000000, -0.0009765625000000, 0.0016174316406250, -0.0011291503906250,
+0.0017700195312500, -0.0007629394531250, 0.0016784667968750, -0.0007324218750000,
+0.0013427734375000, -0.0008239746093750, 0.0016479492187500, 0.0003356933593750,
+0.0014038085937500, 0.0001220703125000, 0.0013732910156250, 0.0006103515625000,
+0.0014648437500000, 0.0016174316406250, 0.0014343261718750, 0.0013122558593750,
+0.0014343261718750, 0.0022888183593750, 0.0014953613281250, 0.0024108886718750,
+0.0015869140625000, 0.0025329589843750, 0.0015869140625000, 0.0030822753906250,
+0.0014953613281250, 0.0030517578125000, 0.0016479492187500, 0.0036010742187500,
+0.0018005371093750, 0.0033874511718750, 0.0015869140625000, 0.0032958984375000,
+0.0018615722656250, 0.0036926269531250, 0.0021057128906250, 0.0035095214843750,
+0.0019226074218750, 0.0030212402343750, 0.0025024414062500, 0.0031433105468750,
+0.0023498535156250, 0.0027465820312500, 0.0025634765625000, 0.0020751953125000,
+0.0030822753906250, 0.0021972656250000, 0.0027770996093750, 0.0021057128906250,
+0.0032958984375000, 0.0016174316406250, 0.0033569335937500, 0.0018920898437500,
+0.0032348632812500, 0.0023193359375000, 0.0034790039062500, 0.0019226074218750,
+0.0034179687500000, 0.0028381347656250, 0.0032653808593750, 0.0028991699218750,
+0.0033874511718750, 0.0030212402343750, 0.0033264160156250, 0.0034790039062500,
+0.0031738281250000},
+{0.0023803710937500, -0.0023803710937500, 0.0024719238281250, -0.0025024414062500,
+0.0023498535156250, -0.0019531250000000, 0.0024414062500000, -0.0020446777343750,
+0.0024108886718750, -0.0018005371093750, 0.0023193359375000, -0.0012817382812500,
+0.0023803710937500, -0.0015258789062500, 0.0024108886718750, -0.0008544921875000,
+0.0023803710937500, -0.0008850097656250, 0.0023803710937500, -0.0007324218750000,
+0.0023498535156250, -0.0002441406250000, 0.0023498535156250, -0.0004882812500000,
+0.0022888183593750, 0.0001525878906250, 0.0024108886718750, 0.0001525878906250,
+0.0024414062500000, 0.0002441406250000, 0.0023498535156250, 0.0006408691406250,
+0.0024108886718750, 0.0004272460937500, 0.0024414062500000, 0.0006103515625000,
+0.0023803710937500, 0.0007019042968750, 0.0024414062500000, 0.0003967285156250,
+0.0025024414062500, 0.0002441406250000, 0.0024719238281250, 0.0003356933593750,
+0.0025634765625000, -0.0002136230468750, 0.0024719238281250, -0.0003051757812500,
+0.0025329589843750, -0.0003967285156250, 0.0024719238281250, -0.0007629394531250,
+0.0023803710937500, -0.0007629394531250, 0.0024719238281250, -0.0006408691406250,
+0.0023193359375000, -0.0008544921875000, 0.0024108886718750, -0.0006408691406250,
+0.0023498535156250, -0.0003662109375000, 0.0022277832031250, -0.0005493164062500,
+0.0023803710937500, -0.0000915527343750, 0.0022888183593750, -0.0000915527343750,
+0.0024108886718750, 0.0000000000000000, 0.0023193359375000, 0.0002746582031250,
+0.0022277832031250, 0.0000610351562500, 0.0023803710937500, 0.0000610351562500,
+0.0021362304687500, 0.0001220703125000, 0.0022888183593750, -0.0000305175781250,
+0.0021667480468750, -0.0001831054687500, 0.0019836425781250, 0.0000000000000000,
+0.0022888183593750, -0.0003662109375000, 0.0016174316406250, -0.0000915527343750,
+0.0018920898437500, -0.0002136230468750, 0.0017700195312500, -0.0005187988281250,
+0.0012817382812500, -0.0001831054687500, 0.0017089843750000, -0.0009460449218750,
+0.0012207031250000, -0.0007019042968750, 0.0011901855468750, -0.0008544921875000,
+0.0013122558593750, -0.0015563964843750, 0.0011291503906250, -0.0010375976562500,
+0.0012512207031250, -0.0012817382812500, 0.0012512207031250, -0.0016174316406250,
+0.0012512207031250, -0.0009155273437500, 0.0013427734375000, -0.0004882812500000,
+0.0014343261718750, -0.0009155273437500, 0.0014038085937500, 0.0006713867187500,
+0.0017395019531250, 0.0004577636718750, 0.0016784667968750, 0.0008239746093750,
+0.0019836425781250, 0.0023498535156250, 0.0024414062500000, 0.0026550292968750,
+0.0024719238281250, 0.0033569335937500, 0.0030212402343750, 0.0032043457031250,
+0.0028686523437500, 0.0035095214843750, 0.0029296875000000, 0.0041503906250000,
+0.0033264160156250, 0.0040588378906250, 0.0031738281250000, 0.0046386718750000,
+0.0032348632812500, 0.0048522949218750, 0.0032653808593750, 0.0046386718750000,
+0.0031433105468750, 0.0046386718750000, 0.0030212402343750, 0.0047607421875000,
+0.0030517578125000, 0.0039062500000000, 0.0027770996093750, 0.0037841796875000,
+0.0027160644531250, 0.0036621093750000, 0.0027160644531250, 0.0030517578125000,
+0.0026855468750000, 0.0028991699218750, 0.0025329589843750, 0.0028991699218750,
+0.0026550292968750, 0.0026855468750000, 0.0025634765625000, 0.0025329589843750,
+0.0025329589843750, 0.0025329589843750, 0.0025329589843750, 0.0025939941406250,
+0.0024719238281250, 0.0025329589843750, 0.0022888183593750, 0.0024414062500000,
+0.0022888183593750, 0.0028076171875000, 0.0018615722656250, 0.0030212402343750,
+0.0015869140625000, 0.0027465820312500, 0.0017700195312500, 0.0036621093750000,
+0.0008850097656250, 0.0034790039062500, 0.0009460449218750, 0.0035705566406250,
+0.0007629394531250, 0.0042114257812500, 0.0001525878906250, 0.0037536621093750,
+0.0004272460937500, 0.0038757324218750, -0.0000915527343750, 0.0040283203125000,
+-0.0002136230468750, 0.0036621093750000, -0.0001525878906250, 0.0033569335937500,
+-0.0002746582031250, 0.0035400390625000, -0.0003051757812500, 0.0030517578125000,
+-0.0003051757812500, 0.0029907226562500, -0.0003051757812500, 0.0030212402343750,
+-0.0003662109375000, 0.0028381347656250, -0.0003967285156250, 0.0027465820312500,
+-0.0003356933593750, 0.0032043457031250, -0.0006713867187500, 0.0029907226562500,
+-0.0006103515625000, 0.0031127929687500, -0.0007934570312500, 0.0035095214843750,
+-0.0011596679687500, 0.0032043457031250, -0.0009155273437500, 0.0037231445312500,
+-0.0014648437500000, 0.0036315917968750, -0.0014343261718750, 0.0035705566406250,
+-0.0014648437500000, 0.0037536621093750, -0.0017089843750000, 0.0036010742187500,
+-0.0015258789062500, 0.0038146972656250, -0.0018005371093750, 0.0036926269531250,
+-0.0017395019531250, 0.0037841796875000, -0.0017395019531250, 0.0040588378906250,
+-0.0018310546875000, 0.0037536621093750, -0.0017089843750000, 0.0041503906250000,
+-0.0021972656250000, 0.0039978027343750, -0.0019836425781250, 0.0038452148437500,
+-0.0021057128906250, 0.0041198730468750, -0.0024719238281250, 0.0039062500000000,
+-0.0021057128906250, 0.0035400390625000, -0.0024414062500000, 0.0037231445312500,
+-0.0025024414062500, 0.0033874511718750, -0.0023193359375000, 0.0029296875000000,
+-0.0023193359375000, 0.0032348632812500, -0.0023193359375000, 0.0025024414062500,
+-0.0023193359375000, 0.0025329589843750, -0.0023193359375000, 0.0025939941406250,
+-0.0024108886718750, 0.0022888183593750, -0.0025024414062500, 0.0025024414062500,
+-0.0025024414062500, 0.0024108886718750, -0.0025634765625000, 0.0025024414062500,
+-0.0027160644531250, 0.0025634765625000, -0.0025329589843750, 0.0025024414062500,
+-0.0024414062500000, 0.0026855468750000, -0.0026245117187500, 0.0026550292968750,
+-0.0017089843750000, 0.0027465820312500, -0.0020141601562500, 0.0027770996093750,
+-0.0014343261718750, 0.0026245117187500, -0.0005798339843750, 0.0027465820312500,
+-0.0011901855468750, 0.0025939941406250, 0.0003356933593750, 0.0025939941406250,
+0.0002136230468750, 0.0026550292968750, 0.0004272460937500, 0.0025939941406250,
+0.0014038085937500, 0.0026550292968750, 0.0009765625000000, 0.0026245117187500,
+0.0013732910156250, 0.0027465820312500, 0.0015563964843750, 0.0027770996093750,
+0.0012512207031250, 0.0027465820312500, 0.0011901855468750, 0.0028991699218750,
+0.0014343261718750, 0.0026855468750000, 0.0007629394531250, 0.0028686523437500,
+0.0009460449218750, 0.0028686523437500, 0.0007629394531250, 0.0027465820312500,
+0.0002441406250000, 0.0028991699218750, 0.0006103515625000, 0.0028991699218750,
+-0.0000610351562500, 0.0028991699218750, 0.0000305175781250, 0.0030517578125000,
+0.0000305175781250, 0.0031127929687500, -0.0003967285156250, 0.0030212402343750,
+-0.0001831054687500, 0.0032958984375000, -0.0003967285156250, 0.0031738281250000,
+-0.0004577636718750, 0.0031738281250000, -0.0004272460937500, 0.0033569335937500,
+-0.0004577636718750, 0.0031127929687500, -0.0005187988281250, 0.0032958984375000,
+-0.0005187988281250, 0.0032348632812500, -0.0005187988281250, 0.0030822753906250,
+-0.0005798339843750, 0.0031433105468750, -0.0006713867187500, 0.0030212402343750,
+-0.0006408691406250, 0.0028381347656250, -0.0009765625000000, 0.0027770996093750,
+-0.0009155273437500, 0.0026855468750000, -0.0010375976562500, 0.0025329589843750,
+-0.0013427734375000, 0.0024719238281250, -0.0013122558593750, 0.0025939941406250,
+-0.0011901855468750, 0.0024108886718750, -0.0014648437500000, 0.0024414062500000,
+-0.0010681152343750, 0.0025634765625000, -0.0006103515625000, 0.0024108886718750,
+-0.0010375976562500, 0.0024414062500000, -0.0000915527343750, 0.0025024414062500,
+-0.0002441406250000, 0.0023498535156250, 0.0000000000000000, 0.0021972656250000,
+0.0006408691406250, 0.0022888183593750, 0.0003051757812500, 0.0019226074218750,
+0.0008239746093750, 0.0019531250000000, 0.0008850097656250, 0.0018920898437500,
+0.0008544921875000, 0.0016479492187500, 0.0010681152343750, 0.0017700195312500,
+0.0010070800781250, 0.0018005371093750, 0.0009155273437500, 0.0017395019531250,
+0.0009460449218750, 0.0017395019531250, 0.0008544921875000, 0.0018310546875000,
+0.0007629394531250, 0.0018310546875000, 0.0007019042968750, 0.0016479492187500,
+0.0004272460937500, 0.0018920898437500, 0.0004577636718750, 0.0016174316406250,
+0.0003051757812500, 0.0013732910156250, 0.0000305175781250, 0.0017089843750000,
+0.0000610351562500, 0.0009460449218750, -0.0001220703125000, 0.0011291503906250,
+-0.0000915527343750, 0.0009765625000000, -0.0001220703125000, 0.0004272460937500,
+-0.0001831054687500, 0.0007019042968750, -0.0001525878906250, 0.0003662109375000,
+-0.0001525878906250, 0.0003051757812500, -0.0001525878906250, 0.0005493164062500,
+-0.0001220703125000, 0.0006103515625000, -0.0001220703125000, 0.0005493164062500,
+-0.0001525878906250, 0.0009765625000000, 0.0000305175781250, 0.0010986328125000,
+-0.0000305175781250, 0.0012512207031250, 0.0000915527343750, 0.0014648437500000,
+0.0003051757812500, 0.0014953613281250, 0.0002441406250000, 0.0016479492187500,
+0.0005493164062500, 0.0017700195312500, 0.0005798339843750, 0.0018920898437500,
+0.0006103515625000, 0.0018920898437500, 0.0007629394531250, 0.0020141601562500,
+0.0007629394531250, 0.0024108886718750, 0.0008239746093750, 0.0023498535156250,
+0.0007934570312500, 0.0025329589843750, 0.0008850097656250, 0.0029907226562500,
+0.0010070800781250, 0.0028991699218750, 0.0009155273437500, 0.0028686523437500,
+0.0010070800781250, 0.0031738281250000, 0.0010375976562500, 0.0027770996093750,
+0.0010986328125000, 0.0025329589843750, 0.0011901855468750, 0.0028686523437500,
+0.0010070800781250, 0.0017395019531250, 0.0011901855468750, 0.0019531250000000,
+0.0012512207031250, 0.0017395019531250, 0.0011901855468750, 0.0009765625000000,
+0.0011596679687500, 0.0014038085937500, 0.0011901855468750, 0.0006713867187500,
+0.0011596679687500, 0.0005798339843750, 0.0010681152343750, 0.0007324218750000,
+0.0010070800781250, 0.0003967285156250, 0.0010375976562500, -0.0000305175781250,
+0.0010070800781250, 0.0011596679687500, 0.0015258789062500, 0.0005798339843750,
+0.0013732910156250, 0.0012817382812500, 0.0014648437500000, 0.0021362304687500,
+0.0017089843750000, 0.0016174316406250, 0.0015563964843750, 0.0029907226562500,
+0.0015258789062500, 0.0029602050781250, 0.0015258789062500, 0.0031738281250000,
+0.0018310546875000, 0.0040588378906250, 0.0018005371093750, 0.0036315917968750,
+0.0015258789062500, 0.0044250488281250, 0.0024108886718750, 0.0044555664062500,
+0.0023803710937500, 0.0044250488281250, 0.0024108886718750, 0.0048828125000000,
+0.0028991699218750, 0.0047912597656250, 0.0027160644531250, 0.0042114257812500,
+0.0029907226562500, 0.0046386718750000, 0.0029907226562500, 0.0037841796875000,
+0.0029907226562500, 0.0028076171875000, 0.0031433105468750, 0.0035705566406250,
+0.0031738281250000, 0.0014648437500000, 0.0034179687500000, 0.0015869140625000,
+0.0033569335937500, 0.0012512207031250, 0.0034179687500000, -0.0000610351562500,
+0.0037536621093750, 0.0004882812500000, 0.0037231445312500, -0.0004272460937500,
+0.0034484863281250, -0.0005798339843750, 0.0036621093750000, -0.0005493164062500,
+0.0034179687500000, -0.0010375976562500, 0.0030212402343750, -0.0009765625000000,
+0.0033874511718750, -0.0006713867187500, 0.0031127929687500, -0.0010681152343750,
+0.0031738281250000, -0.0007019042968750, 0.0032348632812500, -0.0001831054687500,
+0.0031738281250000, -0.0006713867187500, 0.0033264160156250, 0.0001525878906250,
+0.0032653808593750},
+{-0.0005493164062500, 0.0018005371093750, -0.0002136230468750, 0.0018310546875000,
+-0.0001831054687500, 0.0017089843750000, -0.0001525878906250, 0.0014953613281250,
+-0.0000305175781250, 0.0016479492187500, -0.0000610351562500, 0.0014953613281250,
+0.0000000000000000, 0.0014953613281250, 0.0000305175781250, 0.0015869140625000,
+0.0000305175781250, 0.0015869140625000, 0.0000000000000000, 0.0015563964843750,
+0.0000915527343750, 0.0017395019531250, 0.0000305175781250, 0.0017700195312500,
+0.0000915527343750, 0.0017700195312500, -0.0000305175781250, 0.0018310546875000,
+-0.0000610351562500, 0.0019226074218750, 0.0000610351562500, 0.0017395019531250,
+-0.0001525878906250, 0.0017700195312500, -0.0000915527343750, 0.0018005371093750,
+-0.0001525878906250, 0.0017089843750000, -0.0003967285156250, 0.0017700195312500,
+-0.0003356933593750, 0.0019836425781250, -0.0003967285156250, 0.0018920898437500,
+-0.0004272460937500, 0.0020446777343750, -0.0004272460937500, 0.0023193359375000,
+-0.0004882812500000, 0.0022277832031250, -0.0005493164062500, 0.0023193359375000,
+-0.0004882812500000, 0.0025634765625000, -0.0004882812500000, 0.0022583007812500,
+-0.0005187988281250, 0.0020446777343750, -0.0004882812500000, 0.0024719238281250,
+-0.0004882812500000, 0.0013427734375000, -0.0003967285156250, 0.0015869140625000,
+-0.0004272460937500, 0.0013122558593750, -0.0003967285156250, 0.0003662109375000,
+-0.0003051757812500, 0.0007934570312500, -0.0003051757812500, 0.0000915527343750,
+0.0001525878906250, -0.0000610351562500, -0.0000305175781250, 0.0000000000000000,
+0.0001831054687500, -0.0003662109375000, 0.0006713867187500, -0.0002441406250000,
+0.0004577636718750, -0.0001525878906250, 0.0008544921875000, -0.0003662109375000,
+0.0007934570312500, -0.0001220703125000, 0.0008850097656250, 0.0003967285156250,
+0.0013732910156250, 0.0012512207031250, 0.0005187988281250, 0.0024719238281250,
+0.0005493164062500, 0.0021362304687500, 0.0002441406250000, 0.0024719238281250,
+0.0006103515625000, 0.0031433105468750, 0.0009460449218750, 0.0030212402343750,
+0.0006103515625000, 0.0032653808593750, 0.0014343261718750, 0.0036010742187500,
+0.0014343261718750, 0.0032958984375000, 0.0011901855468750, 0.0031433105468750,
+0.0014038085937500, 0.0035095214843750, 0.0014953613281250, 0.0023193359375000,
+0.0014343261718750, 0.0028076171875000, 0.0012512207031250, 0.0021057128906250,
+0.0013122558593750, 0.0007934570312500, 0.0014343261718750, 0.0017700195312500,
+0.0014953613281250, 0.0000305175781250, 0.0016784667968750, 0.0000000000000000,
+0.0016479492187500, 0.0000000000000000, 0.0018005371093750, -0.0007934570312500,
+0.0019531250000000, -0.0004882812500000, 0.0017395019531250, -0.0005798339843750,
+0.0019226074218750, -0.0007629394531250, 0.0018310546875000, -0.0006408691406250,
+0.0019836425781250, -0.0005187988281250, 0.0021362304687500, -0.0006408691406250,
+0.0019226074218750, -0.0001525878906250, 0.0028991699218750, -0.0003662109375000,
+0.0028381347656250, -0.0001220703125000, 0.0029296875000000, 0.0003051757812500,
+0.0036621093750000, -0.0000305175781250, 0.0035705566406250, 0.0005187988281250,
+0.0031738281250000, 0.0004882812500000, 0.0035095214843750, 0.0003662109375000,
+0.0031433105468750, 0.0005187988281250, 0.0026855468750000, 0.0004272460937500,
+0.0032653808593750, -0.0000305175781250, 0.0022888183593750, 0.0000305175781250,
+0.0023803710937500, -0.0000915527343750, 0.0025024414062500, -0.0005798339843750,
+0.0021057128906250, -0.0004577636718750, 0.0021362304687500, -0.0001525878906250,
+0.0019531250000000, -0.0005493164062500, 0.0020141601562500, -0.0000610351562500,
+0.0019836425781250, 0.0005798339843750, 0.0018920898437500, 0.0000610351562500,
+0.0019531250000000, 0.0013427734375000, 0.0014953613281250, 0.0012512207031250,
+0.0017700195312500, 0.0013427734375000, 0.0015258789062500, 0.0021362304687500,
+0.0010070800781250, 0.0018310546875000, 0.0013427734375000, 0.0019226074218750,
+0.0010681152343750, 0.0021362304687500, 0.0009765625000000, 0.0018005371093750,
+0.0013427734375000, 0.0015258789062500, 0.0014648437500000, 0.0016784667968750,
+0.0014038085937500, 0.0012817382812500, 0.0019531250000000, 0.0011596679687500,
+0.0021057128906250, 0.0011596679687500, 0.0021667480468750, 0.0010375976562500,
+0.0025329589843750, 0.0009765625000000, 0.0025939941406250, 0.0009765625000000,
+0.0022583007812500, 0.0009460449218750, 0.0024719238281250, 0.0008544921875000,
+0.0021667480468750, 0.0007324218750000, 0.0017089843750000, 0.0007934570312500,
+0.0019531250000000, 0.0003356933593750, 0.0013732910156250, 0.0004882812500000,
+0.0012817382812500, 0.0001220703125000, 0.0012512207031250, -0.0003356933593750,
+0.0010070800781250, 0.0000000000000000, 0.0010070800781250, -0.0007324218750000,
+0.0010070800781250, -0.0007324218750000, 0.0011291503906250, -0.0006103515625000,
+0.0011291503906250, -0.0009765625000000, 0.0011901855468750, -0.0009460449218750,
+0.0013427734375000, -0.0006103515625000, 0.0010681152343750, -0.0008544921875000,
+0.0012817382812500, -0.0006103515625000, 0.0011901855468750, -0.0002136230468750,
+0.0008850097656250, -0.0006408691406250, 0.0011291503906250, 0.0000000000000000,
+0.0009155273437500, -0.0001525878906250, 0.0008544921875000, -0.0003356933593750,
+0.0009460449218750, 0.0000305175781250, 0.0009460449218750, -0.0001525878906250,
+0.0008544921875000, -0.0007019042968750, 0.0010681152343750, -0.0005187988281250,
+0.0009155273437500, -0.0009765625000000, 0.0010070800781250, -0.0016784667968750,
+0.0012207031250000, -0.0013732910156250, 0.0010070800781250, -0.0022888183593750,
+0.0010681152343750, -0.0023803710937500, 0.0010681152343750, -0.0023193359375000,
+0.0009155273437500, -0.0027465820312500, 0.0008239746093750, -0.0026550292968750,
+0.0007324218750000, -0.0025024414062500, 0.0006408691406250, -0.0026245117187500,
+0.0005187988281250, -0.0023803710937500, 0.0005187988281250, -0.0021057128906250,
+0.0004882812500000, -0.0022583007812500, 0.0004272460937500, -0.0015258789062500,
+0.0004577636718750, -0.0015563964843750, 0.0004577636718750, -0.0012817382812500,
+0.0003967285156250, -0.0007629394531250, 0.0003967285156250, -0.0008850097656250,
+0.0004272460937500, -0.0001525878906250, 0.0003967285156250, -0.0002136230468750,
+0.0003662109375000, 0.0000610351562500, 0.0004882812500000, 0.0007324218750000,
+0.0005187988281250, 0.0003967285156250, 0.0005187988281250, 0.0012817382812500,
+0.0006408691406250, 0.0011291503906250, 0.0006103515625000, 0.0014038085937500,
+0.0006103515625000, 0.0021057128906250, 0.0007934570312500, 0.0017089843750000,
+0.0007019042968750, 0.0023803710937500, 0.0008850097656250, 0.0024414062500000,
+0.0008544921875000, 0.0022583007812500, 0.0010070800781250, 0.0025329589843750,
+0.0012512207031250, 0.0025024414062500, 0.0011901855468750, 0.0019226074218750,
+0.0014648437500000, 0.0021667480468750, 0.0015563964843750, 0.0017395019531250,
+0.0016479492187500, 0.0010375976562500, 0.0018615722656250, 0.0013732910156250,
+0.0018920898437500, 0.0005493164062500, 0.0018920898437500, 0.0004272460937500,
+0.0020751953125000, 0.0004577636718750, 0.0020446777343750, 0.0000915527343750,
+0.0020141601562500, 0.0001220703125000, 0.0022888183593750, 0.0003662109375000,
+0.0019226074218750, 0.0002136230468750, 0.0021057128906250, 0.0003967285156250,
+0.0020446777343750, 0.0007629394531250, 0.0018005371093750, 0.0006103515625000,
+0.0020751953125000, 0.0011596679687500, 0.0016784667968750, 0.0011596679687500,
+0.0017700195312500, 0.0013122558593750, 0.0017089843750000, 0.0017089843750000,
+0.0014343261718750, 0.0015869140625000, 0.0015869140625000, 0.0020751953125000,
+0.0014038085937500, 0.0021362304687500, 0.0013122558593750, 0.0021667480468750,
+0.0014038085937500, 0.0025024414062500, 0.0014343261718750, 0.0025329589843750,
+0.0013732910156250, 0.0024719238281250, 0.0015869140625000, 0.0026855468750000,
+0.0015869140625000, 0.0025634765625000, 0.0016174316406250, 0.0023803710937500,
+0.0017089843750000, 0.0026550292968750, 0.0017395019531250, 0.0022277832031250,
+0.0018005371093750, 0.0022888183593750, 0.0018310546875000, 0.0023498535156250,
+0.0017700195312500, 0.0020751953125000, 0.0017089843750000, 0.0021972656250000,
+0.0018005371093750, 0.0023803710937500, 0.0016784667968750, 0.0021667480468750,
+0.0017395019531250, 0.0024414062500000, 0.0017700195312500, 0.0028381347656250,
+0.0016784667968750, 0.0024414062500000, 0.0018005371093750, 0.0031433105468750,
+0.0017700195312500, 0.0030212402343750, 0.0016784667968750, 0.0030212402343750,
+0.0018615722656250, 0.0035095214843750, 0.0019531250000000, 0.0032348632812500,
+0.0018310546875000, 0.0033264160156250, 0.0024414062500000, 0.0033874511718750,
+0.0023193359375000, 0.0030517578125000, 0.0025329589843750, 0.0028076171875000,
+0.0030822753906250, 0.0028381347656250, 0.0027770996093750, 0.0022888183593750,
+0.0033569335937500, 0.0021057128906250, 0.0035095214843750, 0.0018005371093750,
+0.0033569335937500, 0.0013732910156250, 0.0035095214843750, 0.0013732910156250,
+0.0036010742187500, 0.0009155273437500, 0.0031433105468750, 0.0008239746093750,
+0.0032958984375000, 0.0007629394531250, 0.0031433105468750, 0.0003967285156250,
+0.0027160644531250, 0.0001525878906250, 0.0028076171875000, 0.0005187988281250,
+0.0022888183593750, 0.0003051757812500, 0.0025024414062500, 0.0003967285156250,
+0.0023498535156250, 0.0007324218750000, 0.0018920898437500, 0.0005187988281250,
+0.0021972656250000, 0.0007324218750000, 0.0014343261718750, 0.0008239746093750,
+0.0015563964843750, 0.0007019042968750, 0.0012817382812500, 0.0007324218750000,
+0.0007324218750000, 0.0009460449218750, 0.0010986328125000, 0.0007324218750000,
+0.0001831054687500, 0.0008850097656250, 0.0002441406250000, 0.0007934570312500,
+0.0000610351562500, 0.0006103515625000, -0.0006103515625000, 0.0007934570312500,
+-0.0003051757812500, 0.0009765625000000, -0.0006713867187500, 0.0005798339843750,
+-0.0008239746093750, 0.0009765625000000, -0.0005493164062500, 0.0014038085937500,
+-0.0004882812500000, 0.0009155273437500, -0.0006103515625000, 0.0021057128906250,
+0.0003051757812500, 0.0018920898437500, 0.0000305175781250, 0.0018310546875000,
+0.0004272460937500, 0.0026245117187500, 0.0012817382812500, 0.0024414062500000,
+0.0008544921875000, 0.0022888183593750, 0.0017700195312500, 0.0025634765625000,
+0.0017700195312500, 0.0023193359375000, 0.0018920898437500, 0.0019836425781250,
+0.0025024414062500, 0.0022583007812500, 0.0022583007812500, 0.0017089843750000,
+0.0024719238281250, 0.0016174316406250, 0.0025329589843750, 0.0017700195312500,
+0.0024719238281250, 0.0015563964843750, 0.0025024414062500, 0.0014343261718750,
+0.0025024414062500, 0.0017700195312500, 0.0024108886718750, 0.0015258789062500,
+0.0024414062500000, 0.0016784667968750, 0.0023803710937500, 0.0020751953125000,
+0.0022888183593750, 0.0017089843750000, 0.0023803710937500, 0.0019836425781250,
+0.0021667480468750, 0.0019531250000000, 0.0021362304687500, 0.0018310546875000,
+0.0020751953125000, 0.0018615722656250, 0.0019531250000000, 0.0016784667968750,
+0.0020141601562500, 0.0014343261718750, 0.0018310546875000, 0.0013427734375000,
+0.0018615722656250, 0.0012817382812500, 0.0018005371093750, 0.0010681152343750,
+0.0017700195312500, 0.0008544921875000, 0.0017700195312500, 0.0011901855468750,
+0.0016479492187500, 0.0009460449218750, 0.0016174316406250, 0.0009460449218750,
+0.0016174316406250},
+{0.0014648437500000, 0.0008544921875000, 0.0014648437500000, 0.0004577636718750,
+0.0014648437500000, 0.0008239746093750, 0.0014038085937500, 0.0002746582031250,
+0.0014648437500000, 0.0003662109375000, 0.0014038085937500, 0.0003967285156250,
+0.0015563964843750, 0.0000915527343750, 0.0017089843750000, 0.0002441406250000,
+0.0015258789062500, 0.0004577636718750, 0.0021057128906250, 0.0001831054687500,
+0.0018920898437500, 0.0006713867187500, 0.0022583007812500, 0.0011596679687500,
+0.0028991699218750, 0.0007019042968750, 0.0025329589843750, 0.0021972656250000,
+0.0032958984375000, 0.0018615722656250, 0.0033569335937500, 0.0022277832031250,
+0.0031738281250000, 0.0034179687500000, 0.0034790039062500, 0.0027770996093750,
+0.0035400390625000, 0.0036315917968750, 0.0032043457031250, 0.0037841796875000,
+0.0033264160156250, 0.0036010742187500, 0.0032348632812500, 0.0038146972656250,
+0.0028686523437500, 0.0038146972656250, 0.0030517578125000, 0.0035400390625000,
+0.0028076171875000, 0.0036010742187500, 0.0028991699218750, 0.0034179687500000,
+0.0027770996093750, 0.0029907226562500, 0.0025024414062500, 0.0027465820312500,
+0.0025634765625000, 0.0022583007812500, 0.0018615722656250, 0.0024108886718750,
+0.0021667480468750, 0.0022888183593750, 0.0019836425781250, 0.0018920898437500,
+0.0013427734375000, 0.0021057128906250, 0.0017700195312500, 0.0018615722656250,
+0.0009155273437500, 0.0018005371093750, 0.0009765625000000, 0.0018310546875000,
+0.0008544921875000, 0.0018310546875000, 0.0002746582031250, 0.0018615722656250,
+0.0005798339843750, 0.0018615722656250, 0.0003051757812500, 0.0021057128906250,
+0.0001831054687500, 0.0019531250000000, 0.0004882812500000, 0.0017395019531250,
+0.0005493164062500, 0.0021057128906250, 0.0004272460937500, 0.0012512207031250,
+0.0012207031250000, 0.0013732910156250, 0.0011291503906250, 0.0014343261718750,
+0.0014343261718750, 0.0009155273437500, 0.0020446777343750, 0.0009765625000000,
+0.0018005371093750, 0.0011291503906250, 0.0025634765625000, 0.0009155273437500,
+0.0026550292968750, 0.0011291503906250, 0.0026245117187500, 0.0014343261718750,
+0.0029602050781250, 0.0012207031250000, 0.0029296875000000, 0.0017089843750000,
+0.0028686523437500, 0.0017395019531250, 0.0029602050781250, 0.0018310546875000,
+0.0028991699218750, 0.0021972656250000, 0.0028076171875000, 0.0020446777343750,
+0.0028686523437500, 0.0019531250000000, 0.0027160644531250, 0.0021667480468750,
+0.0026550292968750, 0.0019531250000000, 0.0026245117187500, 0.0015869140625000,
+0.0026245117187500, 0.0018005371093750, 0.0025939941406250, 0.0013122558593750,
+0.0024108886718750, 0.0012512207031250, 0.0024108886718750, 0.0012817382812500,
+0.0023193359375000, 0.0010070800781250, 0.0021057128906250, 0.0010070800781250,
+0.0021362304687500, 0.0011291503906250, 0.0020141601562500, 0.0008850097656250,
+0.0018920898437500, 0.0010070800781250, 0.0018310546875000, 0.0011596679687500,
+0.0018310546875000, 0.0008850097656250, 0.0017395019531250, 0.0010681152343750,
+0.0017700195312500, 0.0011291503906250, 0.0017395019531250, 0.0007934570312500,
+0.0016479492187500, 0.0006713867187500, 0.0016784667968750, 0.0008239746093750,
+0.0016784667968750, 0.0001831054687500, 0.0016174316406250, 0.0002136230468750,
+0.0015563964843750, 0.0001220703125000, 0.0014953613281250, -0.0002746582031250,
+0.0015258789062500, -0.0001525878906250, 0.0014953613281250, -0.0003356933593750,
+0.0012817382812500, -0.0004272460937500, 0.0012817382812500, -0.0003051757812500,
+0.0013122558593750, -0.0002746582031250, 0.0011291503906250, -0.0003967285156250,
+0.0011291503906250, 0.0000000000000000, 0.0012512207031250, -0.0000915527343750,
+0.0011901855468750, 0.0000305175781250, 0.0013122558593750, 0.0003051757812500,
+0.0015258789062500, 0.0001831054687500, 0.0014038085937500, 0.0004882812500000,
+0.0013732910156250, 0.0004272460937500, 0.0014343261718750, 0.0007019042968750,
+0.0013122558593750, 0.0010681152343750, 0.0011596679687500, 0.0008239746093750,
+0.0011291503906250, 0.0014648437500000, 0.0009155273437500, 0.0014343261718750,
+0.0008239746093750, 0.0014953613281250, 0.0007629394531250, 0.0018005371093750,
+0.0005798339843750, 0.0017395019531250, 0.0007019042968750, 0.0016174316406250,
+0.0005493164062500, 0.0018005371093750, 0.0005798339843750, 0.0016479492187500,
+0.0005187988281250, 0.0014038085937500, 0.0004272460937500, 0.0016174316406250,
+0.0004577636718750, 0.0011901855468750, 0.0003967285156250, 0.0012207031250000,
+0.0003967285156250, 0.0011901855468750, 0.0003051757812500, 0.0008850097656250,
+0.0001525878906250, 0.0010070800781250, 0.0002136230468750, 0.0010681152343750,
+-0.0000305175781250, 0.0008850097656250, -0.0000305175781250, 0.0009765625000000,
+-0.0001525878906250, 0.0011291503906250, -0.0003051757812500, 0.0009460449218750,
+-0.0002746582031250, 0.0011596679687500, -0.0007324218750000, 0.0011596679687500,
+-0.0006408691406250, 0.0011291503906250, -0.0009460449218750, 0.0012207031250000,
+-0.0014038085937500, 0.0011291503906250, -0.0011291503906250, 0.0011596679687500,
+-0.0018005371093750, 0.0012207031250000, -0.0018005371093750, 0.0010986328125000,
+-0.0016784667968750, 0.0010375976562500, -0.0020141601562500, 0.0012207031250000,
+-0.0019531250000000, 0.0010375976562500, -0.0014343261718750, 0.0010681152343750,
+-0.0016784667968750, 0.0010681152343750, -0.0011901855468750, 0.0009765625000000,
+-0.0005493164062500, 0.0010986328125000, -0.0009765625000000, 0.0009460449218750,
+0.0003051757812500, 0.0010375976562500, 0.0003967285156250, 0.0009765625000000,
+0.0005493164062500, 0.0007934570312500, 0.0014038085937500, 0.0009765625000000,
+0.0013427734375000, 0.0007324218750000, 0.0011901855468750, 0.0007934570312500,
+0.0016174316406250, 0.0008544921875000, 0.0012207031250000, 0.0008239746093750,
+0.0006103515625000, 0.0008544921875000, 0.0010681152343750, 0.0011901855468750,
+0.0000610351562500, 0.0010986328125000, 0.0000000000000000, 0.0013122558593750,
+-0.0000915527343750, 0.0016784667968750, -0.0007019042968750, 0.0014953613281250,
+-0.0005798339843750, 0.0018615722656250, -0.0007324218750000, 0.0020141601562500,
+-0.0008239746093750, 0.0018920898437500, -0.0007324218750000, 0.0020141601562500,
+-0.0007019042968750, 0.0021362304687500, -0.0008239746093750, 0.0018615722656250,
+-0.0008239746093750, 0.0020446777343750, -0.0007324218750000, 0.0019836425781250,
+-0.0009460449218750, 0.0017089843750000, -0.0011596679687500, 0.0019836425781250,
+-0.0010070800781250, 0.0018005371093750, -0.0016174316406250, 0.0017089843750000,
+-0.0016174316406250, 0.0019531250000000, -0.0015869140625000, 0.0019226074218750,
+-0.0019226074218750, 0.0018920898437500, -0.0018920898437500, 0.0022888183593750,
+-0.0018005371093750, 0.0021667480468750, -0.0019226074218750, 0.0022277832031250,
+-0.0017089843750000, 0.0025634765625000, -0.0014648437500000, 0.0022888183593750,
+-0.0016479492187500, 0.0025634765625000, -0.0011291503906250, 0.0024719238281250,
+-0.0010681152343750, 0.0025329589843750, -0.0009765625000000, 0.0027160644531250,
+-0.0006408691406250, 0.0024719238281250, -0.0006713867187500, 0.0028381347656250,
+-0.0006408691406250, 0.0027465820312500, -0.0004272460937500, 0.0026855468750000,
+-0.0004272460937500, 0.0028991699218750, -0.0005187988281250, 0.0027160644531250,
+-0.0004272460937500, 0.0025024414062500, -0.0003967285156250, 0.0026245117187500,
+-0.0005187988281250, 0.0022888183593750, -0.0002441406250000, 0.0018920898437500,
+-0.0001525878906250, 0.0021362304687500, -0.0003051757812500, 0.0014953613281250,
+0.0003662109375000, 0.0014038085937500, 0.0002136230468750, 0.0014038085937500,
+0.0004577636718750, 0.0010375976562500, 0.0010375976562500, 0.0011291503906250,
+0.0006713867187500, 0.0008850097656250, 0.0009155273437500, 0.0008544921875000,
+0.0009765625000000, 0.0008544921875000, 0.0010070800781250, 0.0007629394531250,
+0.0010681152343750, 0.0007934570312500, 0.0009155273437500, 0.0008544921875000,
+0.0013732910156250, 0.0006713867187500, 0.0011901855468750, 0.0009460449218750,
+0.0014038085937500, 0.0012817382812500, 0.0019226074218750, 0.0010070800781250,
+0.0014953613281250, 0.0018920898437500, 0.0019836425781250, 0.0017700195312500,
+0.0021972656250000, 0.0020446777343750, 0.0018615722656250, 0.0026855468750000,
+0.0018005371093750, 0.0023803710937500, 0.0021057128906250, 0.0028686523437500,
+0.0012512207031250, 0.0030517578125000, 0.0014648437500000, 0.0029602050781250,
+0.0013427734375000, 0.0030517578125000, 0.0003662109375000, 0.0027770996093750,
+0.0011596679687500, 0.0011596679687500, 0.0015869140625000, 0.0015869140625000,
+0.0017089843750000, 0.0011291503906250, 0.0015563964843750, 0.0001525878906250,
+0.0014953613281250, 0.0004577636718750, 0.0016174316406250, 0.0000610351562500,
+0.0011291503906250, -0.0002441406250000, 0.0013732910156250, -0.0000305175781250,
+0.0010986328125000, 0.0000305175781250, 0.0004272460937500, -0.0002441406250000,
+0.0006103515625000, 0.0005187988281250, 0.0002136230468750, 0.0002746582031250,
+0.0000305175781250, 0.0004272460937500, 0.0003051757812500, 0.0010375976562500,
+0.0002136230468750, 0.0006103515625000, -0.0000915527343750, 0.0013122558593750,
+0.0011901855468750, 0.0011901855468750, 0.0007934570312500, 0.0014038085937500,
+0.0010681152343750, 0.0019531250000000, 0.0021057128906250, 0.0015563964843750,
+0.0015869140625000, 0.0022277832031250, 0.0025634765625000, 0.0021362304687500,
+0.0025329589843750, 0.0019836425781250, 0.0025329589843750, 0.0022277832031250,
+0.0031433105468750, 0.0020141601562500, 0.0030822753906250, 0.0019226074218750,
+0.0024414062500000, 0.0018920898437500, 0.0028381347656250, 0.0018920898437500,
+0.0022277832031250, 0.0017700195312500, 0.0013732910156250, 0.0016479492187500,
+0.0020446777343750, 0.0021667480468750, 0.0008544921875000, 0.0018920898437500,
+0.0007934570312500, 0.0021057128906250, 0.0009765625000000, 0.0026245117187500,
+0.0003967285156250, 0.0022277832031250, 0.0005187988281250, 0.0028076171875000,
+0.0012207031250000, 0.0028381347656250, 0.0007629394531250, 0.0026550292968750,
+0.0014038085937500, 0.0028381347656250, 0.0024414062500000, 0.0027465820312500,
+0.0016784667968750, 0.0022583007812500, 0.0029907226562500, 0.0023803710937500,
+0.0030517578125000, 0.0021362304687500, 0.0029296875000000, 0.0017089843750000,
+0.0036010742187500, 0.0019226074218750, 0.0032958984375000, 0.0014953613281250,
+0.0028991699218750, 0.0014953613281250, 0.0032043457031250, 0.0013732910156250,
+0.0028381347656250, 0.0011901855468750, 0.0022583007812500, 0.0013732910156250,
+0.0025939941406250, 0.0007629394531250, 0.0020141601562500, 0.0010681152343750,
+0.0020446777343750, 0.0005798339843750, 0.0022277832031250, -0.0001525878906250,
+0.0021057128906250, 0.0005187988281250, 0.0021362304687500, -0.0009460449218750,
+0.0020751953125000, -0.0009155273437500, 0.0023193359375000, -0.0009460449218750,
+0.0021057128906250, -0.0018310546875000, 0.0018310546875000, -0.0015563964843750,
+0.0021362304687500, -0.0014038085937500, 0.0015869140625000, -0.0019836425781250,
+0.0016784667968750, -0.0014038085937500, 0.0017700195312500, -0.0007324218750000,
+0.0014038085937500, -0.0014648437500000, 0.0015869140625000, 0.0001831054687500,
+0.0019531250000000, -0.0000915527343750, 0.0016784667968750, 0.0001220703125000,
+0.0020751953125000, 0.0012817382812500, 0.0025634765625000, 0.0007019042968750,
+0.0021362304687500},
+{0.0002441406250000, 0.0031738281250000, 0.0004272460937500, 0.0038757324218750,
+0.0003051757812500, 0.0040283203125000, 0.0000305175781250, 0.0037841796875000,
+0.0003356933593750, 0.0039367675781250, 0.0005798339843750, 0.0039978027343750,
+0.0003967285156250, 0.0033264160156250, 0.0008850097656250, 0.0035095214843750,
+0.0008544921875000, 0.0032043457031250, 0.0009155273437500, 0.0021057128906250,
+0.0014038085937500, 0.0014343261718750, 0.0011596679687500, 0.0010375976562500,
+0.0020446777343750, 0.0010070800781250, 0.0017395019531250, 0.0009765625000000,
+0.0018615722656250, 0.0008544921875000, 0.0023803710937500, 0.0007934570312500,
+0.0022888183593750, 0.0009765625000000, 0.0023498535156250, 0.0008239746093750,
+0.0024414062500000, 0.0009765625000000, 0.0023803710937500, 0.0010681152343750,
+0.0024108886718750, 0.0008850097656250, 0.0025939941406250, 0.0013427734375000,
+0.0025634765625000, 0.0011291503906250, 0.0027465820312500, 0.0013732910156250,
+0.0022583007812500, 0.0017395019531250, 0.0019531250000000, 0.0013122558593750,
+0.0024414062500000, 0.0021362304687500, 0.0010681152343750, 0.0019836425781250,
+0.0011291503906250, 0.0019531250000000, 0.0010375976562500, 0.0024108886718750,
+0.0003356933593750, 0.0020751953125000, 0.0006408691406250, 0.0022583007812500,
+0.0006103515625000, 0.0020446777343750, 0.0002746582031250, 0.0021057128906250,
+0.0009155273437500, 0.0023193359375000, 0.0014038085937500, 0.0019836425781250,
+0.0008239746093750, 0.0027160644531250, 0.0025329589843750, 0.0025634765625000,
+0.0025024414062500, 0.0025939941406250, 0.0027160644531250, 0.0030212402343750,
+0.0037841796875000, 0.0029296875000000, 0.0034484863281250, 0.0027770996093750,
+0.0035400390625000, 0.0029907226562500, 0.0038757324218750, 0.0025939941406250,
+0.0032348632812500, 0.0021972656250000, 0.0026855468750000, 0.0024719238281250,
+0.0030517578125000, 0.0014648437500000, 0.0016784667968750, 0.0015563964843750,
+0.0016479492187500, 0.0013427734375000, 0.0015258789062500, 0.0007019042968750,
+0.0006408691406250, 0.0010681152343750, 0.0009460449218750, 0.0003051757812500,
+0.0010375976562500, 0.0004577636718750, 0.0006103515625000, 0.0001220703125000,
+0.0011596679687500, -0.0004577636718750, 0.0017089843750000, 0.0000305175781250,
+0.0011901855468750, -0.0014038085937500, 0.0022583007812500, -0.0010375976562500,
+0.0021972656250000, -0.0014953613281250, 0.0020141601562500, -0.0026550292968750,
+0.0024414062500000, -0.0019226074218750, 0.0022583007812500, -0.0032348632812500,
+0.0018615722656250, -0.0033569335937500, 0.0019531250000000, -0.0031127929687500,
+0.0018005371093750, -0.0038452148437500, 0.0014038085937500, -0.0037841796875000,
+0.0015563964843750, -0.0029296875000000, 0.0017395019531250, -0.0034484863281250,
+0.0016479492187500, -0.0026855468750000, 0.0018920898437500, -0.0016174316406250,
+0.0022888183593750, -0.0025024414062500, 0.0021972656250000, -0.0004577636718750,
+0.0025329589843750, -0.0006408691406250, 0.0025634765625000, -0.0005493164062500,
+0.0026245117187500, 0.0007019042968750, 0.0028381347656250, 0.0000915527343750,
+0.0028381347656250, 0.0002746582031250, 0.0028381347656250, 0.0006103515625000,
+0.0030212402343750, -0.0000305175781250, 0.0030822753906250, -0.0005187988281250,
+0.0030517578125000, -0.0001525878906250, 0.0030822753906250, -0.0014343261718750,
+0.0031433105468750, -0.0013427734375000, 0.0031127929687500, -0.0016479492187500,
+0.0032043457031250, -0.0025024414062500, 0.0032653808593750, -0.0020446777343750,
+0.0030822753906250, -0.0025939941406250, 0.0030822753906250, -0.0028381347656250,
+0.0031127929687500, -0.0024719238281250, 0.0027770996093750, -0.0024108886718750,
+0.0025024414062500, -0.0026855468750000, 0.0027160644531250, -0.0014648437500000,
+0.0021057128906250, -0.0018615722656250, 0.0020446777343750, -0.0013122558593750,
+0.0021057128906250, -0.0001831054687500, 0.0018005371093750, -0.0008850097656250,
+0.0018615722656250, 0.0006713867187500, 0.0020141601562500, 0.0005187988281250,
+0.0018615722656250, 0.0007324218750000, 0.0021667480468750, 0.0017089843750000,
+0.0025329589843750, 0.0012207031250000, 0.0022888183593750, 0.0018005371093750,
+0.0027465820312500, 0.0018920898437500, 0.0028076171875000, 0.0017395019531250,
+0.0027160644531250, 0.0018615722656250, 0.0028076171875000, 0.0018615722656250,
+0.0026855468750000, 0.0013427734375000, 0.0025024414062500, 0.0014953613281250,
+0.0024719238281250, 0.0011901855468750, 0.0023498535156250, 0.0006713867187500,
+0.0021667480468750, 0.0010070800781250, 0.0021667480468750, 0.0001831054687500,
+0.0021972656250000, 0.0002441406250000, 0.0021057128906250, 0.0000305175781250,
+0.0021362304687500, -0.0006408691406250, 0.0022277832031250, -0.0003051757812500,
+0.0022277832031250, -0.0010681152343750, 0.0022277832031250, -0.0011901855468750,
+0.0023803710937500, -0.0012817382812500, 0.0023193359375000, -0.0017395019531250,
+0.0022888183593750, -0.0018310546875000, 0.0024414062500000, -0.0017395019531250,
+0.0022277832031250, -0.0020751953125000, 0.0023803710937500, -0.0017700195312500,
+0.0022277832031250, -0.0014953613281250, 0.0020141601562500, -0.0019226074218750,
+0.0022583007812500, -0.0005187988281250, 0.0018005371093750, -0.0008850097656250,
+0.0019226074218750, -0.0001831054687500, 0.0019226074218750, 0.0011596679687500,
+0.0016174316406250, 0.0003356933593750, 0.0017700195312500, 0.0021972656250000,
+0.0015258789062500, 0.0020751953125000, 0.0015563964843750, 0.0023193359375000,
+0.0015258789062500, 0.0035705566406250, 0.0013427734375000, 0.0031127929687500,
+0.0014343261718750, 0.0037841796875000, 0.0013122558593750, 0.0039367675781250,
+0.0013427734375000, 0.0038757324218750, 0.0013732910156250, 0.0041809082031250,
+0.0014038085937500, 0.0041198730468750, 0.0014343261718750, 0.0039062500000000,
+0.0014953613281250, 0.0041503906250000, 0.0015258789062500, 0.0039062500000000,
+0.0015563964843750, 0.0036010742187500, 0.0017089843750000, 0.0039672851562500,
+0.0017700195312500, 0.0030822753906250, 0.0016479492187500, 0.0033874511718750,
+0.0017700195312500, 0.0030212402343750, 0.0017395019531250, 0.0021972656250000,
+0.0015869140625000, 0.0026550292968750, 0.0017395019531250, 0.0015258789062500,
+0.0015563964843750, 0.0015563964843750, 0.0015563964843750, 0.0014038085937500,
+0.0016479492187500, 0.0006408691406250, 0.0015563964843750, 0.0009460449218750,
+0.0015869140625000, 0.0007934570312500, 0.0016174316406250, 0.0006103515625000,
+0.0016479492187500, 0.0008850097656250, 0.0017395019531250, 0.0010070800781250,
+0.0017700195312500, 0.0009460449218750, 0.0018005371093750, 0.0015563964843750,
+0.0021972656250000, 0.0014343261718750, 0.0020751953125000, 0.0017089843750000,
+0.0023803710937500, 0.0022277832031250, 0.0028076171875000, 0.0019836425781250,
+0.0025024414062500, 0.0025329589843750, 0.0032653808593750, 0.0026550292968750,
+0.0032348632812500, 0.0026245117187500, 0.0032043457031250, 0.0028686523437500,
+0.0036315917968750, 0.0029907226562500, 0.0034484863281250, 0.0027770996093750,
+0.0032653808593750, 0.0029907226562500, 0.0034790039062500, 0.0028381347656250,
+0.0032958984375000, 0.0024719238281250, 0.0030212402343750, 0.0027160644531250,
+0.0032348632812500, 0.0023193359375000, 0.0028076171875000, 0.0022888183593750,
+0.0028991699218750, 0.0023803710937500, 0.0027770996093750, 0.0023193359375000,
+0.0025024414062500, 0.0023803710937500, 0.0026855468750000, 0.0026550292968750,
+0.0023498535156250, 0.0027160644531250, 0.0023803710937500, 0.0027160644531250,
+0.0022583007812500, 0.0029907226562500, 0.0018920898437500, 0.0030822753906250,
+0.0020141601562500, 0.0027160644531250, 0.0020141601562500, 0.0029602050781250,
+0.0018310546875000, 0.0028686523437500, 0.0020141601562500, 0.0024108886718750,
+0.0022583007812500, 0.0024719238281250, 0.0021667480468750, 0.0018920898437500,
+0.0027160644531250, 0.0020446777343750, 0.0024414062500000, 0.0021667480468750,
+0.0027160644531250, 0.0019226074218750, 0.0033264160156250, 0.0020446777343750,
+0.0028991699218750, 0.0021667480468750, 0.0038146972656250, 0.0021667480468750,
+0.0037231445312500, 0.0022583007812500, 0.0036926269531250, 0.0024414062500000,
+0.0041503906250000, 0.0023498535156250, 0.0039672851562500, 0.0021972656250000,
+0.0038146972656250, 0.0022583007812500, 0.0039367675781250, 0.0020751953125000,
+0.0036010742187500, 0.0018920898437500, 0.0032958984375000, 0.0019836425781250,
+0.0034484863281250, 0.0016784667968750, 0.0027160644531250, 0.0018005371093750,
+0.0027770996093750, 0.0015869140625000, 0.0025024414062500, 0.0013427734375000,
+0.0019226074218750, 0.0015869140625000, 0.0021362304687500, 0.0010375976562500,
+0.0015869140625000, 0.0010070800781250, 0.0014953613281250, 0.0010375976562500,
+0.0014343261718750, 0.0008239746093750, 0.0011596679687500, 0.0008239746093750,
+0.0013122558593750, 0.0010681152343750, 0.0008239746093750, 0.0009460449218750,
+0.0009460449218750, 0.0009460449218750, 0.0007934570312500, 0.0011596679687500,
+0.0004577636718750, 0.0011596679687500, 0.0006408691406250, 0.0012817382812500,
+0.0000305175781250, 0.0012512207031250, 0.0001220703125000, 0.0014953613281250,
+0.0000610351562500, 0.0017395019531250, -0.0003356933593750, 0.0016784667968750,
+-0.0001525878906250, 0.0021667480468750, -0.0005187988281250, 0.0022277832031250,
+-0.0004882812500000, 0.0022277832031250, -0.0005798339843750, 0.0024414062500000,
+-0.0008544921875000, 0.0026245117187500, -0.0007019042968750, 0.0025024414062500,
+-0.0009155273437500, 0.0026855468750000, -0.0010070800781250, 0.0026855468750000,
+-0.0010375976562500, 0.0025939941406250, -0.0012207031250000, 0.0028686523437500,
+-0.0013122558593750, 0.0025024414062500, -0.0012512207031250, 0.0025939941406250,
+-0.0014038085937500, 0.0026245117187500, -0.0013122558593750, 0.0024108886718750,
+-0.0011901855468750, 0.0025634765625000, -0.0014343261718750, 0.0026245117187500,
+-0.0009765625000000, 0.0025634765625000, -0.0011901855468750, 0.0027770996093750,
+-0.0009765625000000, 0.0029296875000000, -0.0005493164062500, 0.0028076171875000,
+-0.0008239746093750, 0.0033874511718750, -0.0004577636718750, 0.0032043457031250,
+-0.0003967285156250, 0.0032653808593750, -0.0005798339843750, 0.0037536621093750,
+-0.0005187988281250, 0.0034484863281250, -0.0004272460937500, 0.0034790039062500,
+-0.0008239746093750, 0.0035705566406250, -0.0008850097656250, 0.0033264160156250,
+-0.0008850097656250, 0.0030517578125000, -0.0010375976562500, 0.0031433105468750,
+-0.0010986328125000, 0.0029296875000000, -0.0009460449218750, 0.0027770996093750,
+-0.0010681152343750, 0.0028381347656250, -0.0010070800781250, 0.0029296875000000,
+-0.0008239746093750, 0.0028381347656250, -0.0009460449218750, 0.0030517578125000,
+-0.0005493164062500, 0.0030212402343750, -0.0006713867187500, 0.0030212402343750,
+-0.0004577636718750, 0.0030517578125000, -0.0000915527343750, 0.0030517578125000,
+-0.0003051757812500, 0.0031433105468750, 0.0004882812500000, 0.0031127929687500,
+0.0003662109375000, 0.0031433105468750, 0.0005187988281250, 0.0032653808593750,
+0.0010070800781250, 0.0031738281250000, 0.0006713867187500, 0.0032958984375000,
+0.0011596679687500, 0.0033569335937500, 0.0011291503906250, 0.0033569335937500,
+0.0010681152343750, 0.0033874511718750, 0.0013122558593750, 0.0034790039062500,
+0.0010986328125000, 0.0031738281250000, 0.0010681152343750, 0.0034179687500000,
+0.0011596679687500},
+{0.0039062500000000, 0.0001220703125000, 0.0042114257812500, 0.0004272460937500,
+0.0039062500000000, 0.0004577636718750, 0.0032958984375000, 0.0002441406250000,
+0.0037536621093750, 0.0011901855468750, 0.0027465820312500, 0.0008850097656250,
+0.0029907226562500, 0.0010986328125000, 0.0026550292968750, 0.0018310546875000,
+0.0018920898437500, 0.0013122558593750, 0.0023803710937500, 0.0021057128906250,
+0.0013122558593750, 0.0021667480468750, 0.0014953613281250, 0.0020751953125000,
+0.0011901855468750, 0.0024719238281250, 0.0003356933593750, 0.0023498535156250,
+0.0008239746093750, 0.0023498535156250, -0.0001831054687500, 0.0025634765625000,
+-0.0001525878906250, 0.0024108886718750, -0.0003356933593750, 0.0020751953125000,
+-0.0010070800781250, 0.0022888183593750, -0.0007934570312500, 0.0021362304687500,
+-0.0013122558593750, 0.0020751953125000, -0.0014038085937500, 0.0021972656250000,
+-0.0013427734375000, 0.0021972656250000, -0.0015258789062500, 0.0021667480468750,
+-0.0014953613281250, 0.0022277832031250, -0.0014038085937500, 0.0023193359375000,
+-0.0014648437500000, 0.0021972656250000, -0.0013427734375000, 0.0020751953125000,
+-0.0011291503906250, 0.0021667480468750, -0.0012207031250000, 0.0019836425781250,
+-0.0009765625000000, 0.0019836425781250, -0.0009765625000000, 0.0020446777343750,
+-0.0009765625000000, 0.0020141601562500, -0.0008850097656250, 0.0020751953125000,
+-0.0008850097656250, 0.0022888183593750, -0.0007324218750000, 0.0023193359375000,
+-0.0007934570312500, 0.0024414062500000, -0.0006713867187500, 0.0025939941406250,
+-0.0004882812500000, 0.0025024414062500, -0.0006103515625000, 0.0029907226562500,
+-0.0002441406250000, 0.0028991699218750, -0.0001525878906250, 0.0029907226562500,
+-0.0000305175781250, 0.0033874511718750, 0.0002441406250000, 0.0032348632812500,
+0.0002136230468750, 0.0033874511718750, 0.0003662109375000, 0.0033569335937500,
+0.0005493164062500, 0.0033569335937500, 0.0003356933593750, 0.0033874511718750,
+0.0002441406250000, 0.0032958984375000, 0.0004272460937500, 0.0033264160156250,
+-0.0002746582031250, 0.0031127929687500, -0.0002136230468750, 0.0032043457031250,
+-0.0003662109375000, 0.0032043457031250, -0.0009460449218750, 0.0028991699218750,
+-0.0006408691406250, 0.0035095214843750, -0.0009765625000000, 0.0032958984375000,
+-0.0010681152343750, 0.0032348632812500, -0.0010070800781250, 0.0036621093750000,
+-0.0010986328125000, 0.0034484863281250, -0.0010681152343750, 0.0032653808593750,
+-0.0010986328125000, 0.0034484863281250, -0.0011596679687500, 0.0031433105468750,
+-0.0011901855468750, 0.0028686523437500, -0.0011596679687500, 0.0030822753906250,
+-0.0011901855468750, 0.0027160644531250, -0.0011596679687500, 0.0026550292968750,
+-0.0010986328125000, 0.0026550292968750, -0.0011291503906250, 0.0024414062500000,
+-0.0011901855468750, 0.0025024414062500, -0.0011596679687500, 0.0024414062500000,
+-0.0011291503906250, 0.0024719238281250, -0.0012207031250000, 0.0025024414062500,
+-0.0010375976562500, 0.0024719238281250, -0.0009765625000000, 0.0024414062500000,
+-0.0011901855468750, 0.0025329589843750, -0.0005187988281250, 0.0025024414062500,
+-0.0007019042968750, 0.0025634765625000, -0.0005187988281250, 0.0025634765625000,
+0.0000610351562500, 0.0025939941406250, -0.0002441406250000, 0.0026550292968750,
+0.0004272460937500, 0.0027160644531250, 0.0004882812500000, 0.0027160644531250,
+0.0005493164062500, 0.0028076171875000, 0.0009460449218750, 0.0028991699218750,
+0.0008239746093750, 0.0024414062500000, 0.0008544921875000, 0.0027770996093750,
+0.0009765625000000, 0.0024719238281250, 0.0009155273437500, 0.0020141601562500,
+0.0007629394531250, 0.0025024414062500, 0.0008239746093750, 0.0016784667968750,
+0.0005493164062500, 0.0018615722656250, 0.0005493164062500, 0.0018615722656250,
+0.0006408691406250, 0.0013427734375000, 0.0004272460937500, 0.0017395019531250,
+0.0004272460937500, 0.0014648437500000, 0.0006713867187500, 0.0014953613281250,
+0.0004577636718750, 0.0014953613281250, 0.0007019042968750, 0.0014343261718750,
+0.0011291503906250, 0.0016174316406250, 0.0007629394531250, 0.0014038085937500,
+0.0015869140625000, 0.0015258789062500, 0.0014953613281250, 0.0014038085937500,
+0.0015563964843750, 0.0012207031250000, 0.0021667480468750, 0.0015258789062500,
+0.0019531250000000, 0.0011291503906250, 0.0018920898437500, 0.0012207031250000,
+0.0021972656250000, 0.0012207031250000, 0.0017395019531250, 0.0010375976562500,
+0.0012512207031250, 0.0012817382812500, 0.0017089843750000, 0.0011596679687500,
+0.0004272460937500, 0.0012817382812500, 0.0005798339843750, 0.0013427734375000,
+0.0002746582031250, 0.0012817382812500, -0.0005798339843750, 0.0014953613281250,
+-0.0002441406250000, 0.0012817382812500, -0.0009765625000000, 0.0014343261718750,
+-0.0010070800781250, 0.0015258789062500, -0.0008239746093750, 0.0014038085937500,
+-0.0010070800781250, 0.0015869140625000, -0.0009765625000000, 0.0017089843750000,
+-0.0006408691406250, 0.0016784667968750, -0.0007324218750000, 0.0018920898437500,
+-0.0006103515625000, 0.0020446777343750, -0.0002746582031250, 0.0020446777343750,
+-0.0003356933593750, 0.0022888183593750, -0.0004882812500000, 0.0023193359375000,
+-0.0003051757812500, 0.0022888183593750, -0.0005798339843750, 0.0024414062500000,
+-0.0009155273437500, 0.0024108886718750, -0.0006713867187500, 0.0022583007812500,
+-0.0014648437500000, 0.0024108886718750, -0.0013732910156250, 0.0022888183593750,
+-0.0015258789062500, 0.0021362304687500, -0.0021362304687500, 0.0022888183593750,
+-0.0018920898437500, 0.0021057128906250, -0.0021667480468750, 0.0020751953125000,
+-0.0021972656250000, 0.0021667480468750, -0.0019836425781250, 0.0021057128906250,
+-0.0019531250000000, 0.0020751953125000, -0.0020141601562500, 0.0024719238281250,
+-0.0016784667968750, 0.0023498535156250, -0.0016174316406250, 0.0025024414062500,
+-0.0018005371093750, 0.0028686523437500, -0.0018005371093750, 0.0027465820312500,
+-0.0018005371093750, 0.0032348632812500, -0.0020446777343750, 0.0031433105468750,
+-0.0021362304687500, 0.0033264160156250, -0.0021667480468750, 0.0037841796875000,
+-0.0023498535156250, 0.0034790039062500, -0.0024414062500000, 0.0040283203125000,
+-0.0018310546875000, 0.0040283203125000, -0.0021667480468750, 0.0040283203125000,
+-0.0017395019531250, 0.0043945312500000, -0.0009460449218750, 0.0042114257812500,
+-0.0014038085937500, 0.0041198730468750, -0.0002746582031250, 0.0043029785156250,
+-0.0003356933593750, 0.0039978027343750, -0.0000915527343750, 0.0037536621093750,
+0.0006713867187500, 0.0040283203125000, 0.0003967285156250, 0.0032348632812500,
+0.0009155273437500, 0.0033569335937500, 0.0009765625000000, 0.0032043457031250,
+0.0010986328125000, 0.0026855468750000, 0.0012817382812500, 0.0028991699218750,
+0.0010986328125000, 0.0024414062500000, 0.0018615722656250, 0.0024414062500000,
+0.0016479492187500, 0.0023193359375000, 0.0018920898437500, 0.0020141601562500,
+0.0025329589843750, 0.0021057128906250, 0.0021667480468750, 0.0017700195312500,
+0.0026245117187500, 0.0017700195312500, 0.0027160644531250, 0.0017395019531250,
+0.0025024414062500, 0.0014648437500000, 0.0026245117187500, 0.0016174316406250,
+0.0000915527343750, 0.0018005371093750, 0.0015869140625000, 0.0017089843750000,
+0.0010681152343750, 0.0018920898437500, 0.0012207031250000, 0.0021667480468750,
+0.0018310546875000, 0.0020751953125000, 0.0015869140625000, 0.0022583007812500,
+0.0020141601562500, 0.0023498535156250, 0.0019836425781250, 0.0022277832031250,
+0.0018310546875000, 0.0022277832031250, 0.0019531250000000, 0.0022888183593750,
+0.0018920898437500, 0.0019836425781250, 0.0016784667968750, 0.0021057128906250,
+0.0016174316406250, 0.0020141601562500, 0.0018310546875000, 0.0017395019531250,
+0.0018005371093750, 0.0019531250000000, 0.0015869140625000, 0.0012817382812500,
+0.0025024414062500, 0.0016174316406250, 0.0023193359375000, 0.0012817382812500,
+0.0024719238281250, 0.0007019042968750, 0.0030822753906250, 0.0012817382812500,
+0.0027160644531250, -0.0001525878906250, 0.0029907226562500, 0.0003356933593750,
+0.0032348632812500, 0.0000000000000000, 0.0028991699218750, -0.0012207031250000,
+0.0028076171875000, -0.0004272460937500, 0.0032348632812500, -0.0013427734375000,
+0.0021667480468750, -0.0014038085937500, 0.0025024414062500, -0.0012207031250000,
+0.0022583007812500, -0.0014953613281250, 0.0014343261718750, -0.0013732910156250,
+0.0019836425781250, -0.0013427734375000, 0.0009765625000000, -0.0012512207031250,
+0.0009765625000000, -0.0014038085937500, 0.0010986328125000, -0.0014648437500000,
+0.0006103515625000, -0.0012817382812500, 0.0007019042968750, -0.0019226074218750,
+0.0014038085937500, -0.0018615722656250, 0.0009765625000000, -0.0018005371093750,
+0.0015563964843750, -0.0022583007812500, 0.0025329589843750, -0.0021057128906250,
+0.0018310546875000, -0.0015869140625000, 0.0026855468750000, -0.0020141601562500,
+0.0030212402343750, -0.0011596679687500, 0.0027465820312500, -0.0002746582031250,
+0.0028076171875000, -0.0010375976562500, 0.0028991699218750, 0.0011596679687500,
+0.0019226074218750, 0.0008544921875000, 0.0021667480468750, 0.0013122558593750,
+0.0018310546875000, 0.0029296875000000, 0.0009460449218750, 0.0021667480468750,
+0.0012512207031250, 0.0032348632812500, 0.0007324218750000, 0.0035705566406250,
+0.0006103515625000, 0.0031738281250000, 0.0007629394531250, 0.0034484863281250,
+0.0005798339843750, 0.0036621093750000, 0.0005798339843750, 0.0023193359375000,
+0.0007934570312500, 0.0028381347656250, 0.0008544921875000, 0.0021667480468750,
+0.0009155273437500, 0.0009155273437500, 0.0011291503906250, 0.0018310546875000,
+0.0012817382812500, 0.0002441406250000, 0.0012207031250000, 0.0001220703125000,
+0.0014038085937500, 0.0004577636718750, 0.0013732910156250, -0.0002136230468750,
+0.0010986328125000, -0.0002136230468750, 0.0013427734375000, 0.0007629394531250,
+0.0010986328125000, 0.0001525878906250, 0.0010070800781250, 0.0007934570312500,
+0.0010681152343750, 0.0018615722656250, 0.0010375976562500, 0.0010070800781250,
+0.0010986328125000, 0.0022888183593750, 0.0012207031250000, 0.0022583007812500,
+0.0012817382812500, 0.0018920898437500, 0.0013122558593750, 0.0023803710937500,
+0.0014343261718750, 0.0022583007812500, 0.0015258789062500, 0.0012512207031250,
+0.0015563964843750, 0.0017395019531250, 0.0016479492187500, 0.0010070800781250,
+0.0016784667968750, -0.0001220703125000, 0.0016784667968750, 0.0006408691406250,
+0.0016784667968750, -0.0012512207031250, 0.0018310546875000, -0.0011291503906250,
+0.0018310546875000, -0.0014648437500000, 0.0018920898437500, -0.0027465820312500,
+0.0020751953125000, -0.0021667480468750, 0.0019531250000000, -0.0029907226562500,
+0.0020751953125000, -0.0033264160156250, 0.0021362304687500, -0.0029296875000000,
+0.0020446777343750, -0.0031127929687500, 0.0020446777343750, -0.0033874511718750,
+0.0021362304687500, -0.0022583007812500, 0.0017395019531250, -0.0027160644531250,
+0.0018310546875000, -0.0020446777343750, 0.0017700195312500, -0.0008850097656250,
+0.0014953613281250, -0.0017700195312500, 0.0016174316406250, 0.0001831054687500,
+0.0016479492187500, -0.0000305175781250, 0.0015869140625000, 0.0003051757812500,
+0.0016784667968750, 0.0016174316406250, 0.0018615722656250, 0.0009155273437500,
+0.0018005371093750, 0.0020141601562500, 0.0019226074218750, 0.0020446777343750,
+0.0019836425781250, 0.0019531250000000, 0.0018615722656250, 0.0025634765625000,
+0.0018615722656250},
+{0.0012512207031250, -0.0006103515625000, 0.0010070800781250, -0.0005798339843750,
+0.0010375976562500, -0.0005187988281250, 0.0012817382812500, -0.0003051757812500,
+0.0011901855468750, -0.0004577636718750, 0.0013122558593750, -0.0006408691406250,
+0.0014343261718750, -0.0003356933593750, 0.0012207031250000, -0.0008850097656250,
+0.0016784667968750, -0.0007934570312500, 0.0018310546875000, -0.0006713867187500,
+0.0015258789062500, -0.0009765625000000, 0.0016174316406250, -0.0007324218750000,
+0.0018310546875000, -0.0003051757812500, 0.0010375976562500, -0.0005187988281250,
+0.0012512207031250, 0.0000610351562500, 0.0009765625000000, 0.0007324218750000,
+0.0002136230468750, 0.0004882812500000, 0.0005798339843750, 0.0018310546875000,
+0.0003967285156250, 0.0017395019531250, 0.0000915527343750, 0.0020446777343750,
+0.0004272460937500, 0.0030212402343750, 0.0007934570312500, 0.0026855468750000,
+0.0007629394531250, 0.0030212402343750, 0.0014038085937500, 0.0033264160156250,
+0.0015258789062500, 0.0028381347656250, 0.0014038085937500, 0.0026855468750000,
+0.0017700195312500, 0.0030822753906250, 0.0019531250000000, 0.0016784667968750,
+0.0011291503906250, 0.0019836425781250, 0.0014343261718750, 0.0015563964843750,
+0.0010070800781250, 0.0003662109375000, 0.0000610351562500, 0.0009460449218750,
+0.0005187988281250, -0.0000610351562500, -0.0002441406250000, -0.0000915527343750,
+-0.0004272460937500, 0.0001831054687500, -0.0002746582031250, -0.0001525878906250,
+-0.0004577636718750, -0.0000305175781250, -0.0004882812500000, 0.0007629394531250,
+0.0000915527343750, 0.0006103515625000, -0.0000305175781250, 0.0010681152343750,
+0.0002746582031250, 0.0019836425781250, 0.0008850097656250, 0.0016479492187500,
+0.0006408691406250, 0.0022583007812500, 0.0010681152343750, 0.0025024414062500,
+0.0013122558593750, 0.0021667480468750, 0.0011596679687500, 0.0021972656250000,
+0.0010986328125000, 0.0024414062500000, 0.0012207031250000, 0.0013732910156250,
+0.0008850097656250, 0.0016174316406250, 0.0010375976562500, 0.0012207031250000,
+0.0010070800781250, 0.0001831054687500, 0.0007629394531250, 0.0006713867187500,
+0.0009155273437500, -0.0001220703125000, 0.0008239746093750, -0.0003662109375000,
+0.0007629394531250, -0.0001525878906250, 0.0008544921875000, -0.0002746582031250,
+0.0007629394531250, -0.0004882812500000, 0.0007019042968750, 0.0001831054687500,
+0.0008544921875000, -0.0002136230468750, 0.0007324218750000, 0.0001831054687500,
+0.0007629394531250, 0.0008850097656250, 0.0008544921875000, 0.0003051757812500,
+0.0006408691406250, 0.0013122558593750, 0.0009765625000000, 0.0011901855468750,
+0.0007019042968750, 0.0012207031250000, 0.0009155273437500, 0.0018615722656250,
+0.0013122558593750, 0.0014953613281250, 0.0008850097656250, 0.0017089843750000,
+0.0016174316406250, 0.0018310546875000, 0.0014648437500000, 0.0016174316406250,
+0.0014343261718750, 0.0015563964843750, 0.0018310546875000, 0.0015869140625000,
+0.0014953613281250, 0.0011291503906250, 0.0016784667968750, 0.0012207031250000,
+0.0015869140625000, 0.0010375976562500, 0.0014343261718750, 0.0007324218750000,
+0.0014343261718750, 0.0008239746093750, 0.0012512207031250, 0.0004272460937500,
+0.0013427734375000, 0.0005187988281250, 0.0011596679687500, 0.0003051757812500,
+0.0011596679687500, -0.0000305175781250, 0.0011901855468750, 0.0001831054687500,
+0.0010375976562500, -0.0003051757812500, 0.0012207031250000, -0.0002441406250000,
+0.0011596679687500, -0.0002746582031250, 0.0011291503906250, -0.0005187988281250,
+0.0012512207031250, -0.0004577636718750, 0.0012207031250000, -0.0004272460937500,
+0.0013427734375000, -0.0005187988281250, 0.0013122558593750, -0.0003662109375000,
+0.0013427734375000, -0.0002441406250000, 0.0015258789062500, -0.0003356933593750,
+0.0014038085937500, -0.0001220703125000, 0.0015869140625000, -0.0000915527343750,
+0.0014953613281250, -0.0000915527343750, 0.0015258789062500, 0.0000610351562500,
+0.0016784667968750, 0.0000305175781250, 0.0015563964843750, -0.0000610351562500,
+0.0018005371093750, 0.0000610351562500, 0.0017089843750000, -0.0001220703125000,
+0.0017700195312500, -0.0003356933593750, 0.0019226074218750, -0.0000915527343750,
+0.0017395019531250, -0.0007019042968750, 0.0019531250000000, -0.0005798339843750,
+0.0018005371093750, -0.0007629394531250, 0.0018005371093750, -0.0012817382812500,
+0.0019226074218750, -0.0009155273437500, 0.0017395019531250, -0.0015258789062500,
+0.0020141601562500, -0.0014648437500000, 0.0018310546875000, -0.0014648437500000,
+0.0018615722656250, -0.0017700195312500, 0.0021362304687500, -0.0015869140625000,
+0.0018310546875000, -0.0013732910156250, 0.0022888183593750, -0.0014953613281250,
+0.0020446777343750, -0.0013122558593750, 0.0021972656250000, -0.0010375976562500,
+0.0025024414062500, -0.0011901855468750, 0.0020751953125000, -0.0009460449218750,
+0.0026855468750000, -0.0009155273437500, 0.0024108886718750, -0.0009155273437500,
+0.0023803710937500, -0.0007629394531250, 0.0027160644531250, -0.0008239746093750,
+0.0023193359375000, -0.0007629394531250, 0.0024108886718750, -0.0006408691406250,
+0.0023193359375000, -0.0007629394531250, 0.0021667480468750, -0.0008544921875000,
+0.0020751953125000, -0.0007324218750000, 0.0018920898437500, -0.0013427734375000,
+0.0020446777343750, -0.0011596679687500, 0.0018310546875000, -0.0014648437500000,
+0.0018920898437500, -0.0021057128906250, 0.0020751953125000, -0.0017395019531250,
+0.0017700195312500, -0.0022888183593750, 0.0020141601562500, -0.0024414062500000,
+0.0019226074218750, -0.0021667480468750, 0.0019226074218750, -0.0022583007812500,
+0.0020446777343750, -0.0023193359375000, 0.0017700195312500, -0.0015869140625000,
+0.0021972656250000, -0.0017700195312500, 0.0020751953125000, -0.0013122558593750,
+0.0022583007812500, -0.0006408691406250, 0.0026245117187500, -0.0009460449218750,
+0.0024719238281250, 0.0001831054687500, 0.0027770996093750, 0.0000305175781250,
+0.0028076171875000, 0.0003967285156250, 0.0028076171875000, 0.0012207031250000,
+0.0030212402343750, 0.0008239746093750, 0.0029907226562500, 0.0014343261718750,
+0.0029602050781250, 0.0015563964843750, 0.0030517578125000, 0.0013732910156250,
+0.0029602050781250, 0.0014343261718750, 0.0028381347656250, 0.0014343261718750,
+0.0028991699218750, 0.0013122558593750, 0.0025634765625000, 0.0011596679687500,
+0.0026245117187500, 0.0012512207031250, 0.0024719238281250, 0.0014343261718750,
+0.0021972656250000, 0.0015869140625000, 0.0022583007812500, 0.0018920898437500,
+0.0018920898437500, 0.0019836425781250, 0.0019836425781250, 0.0017700195312500,
+0.0019226074218750, 0.0018310546875000, 0.0016784667968750, 0.0020141601562500,
+0.0018920898437500, 0.0016479492187500, 0.0014648437500000, 0.0016174316406250,
+0.0015563964843750, 0.0016479492187500, 0.0014648437500000, 0.0015563964843750,
+0.0012207031250000, 0.0015563964843750, 0.0013732910156250, 0.0016174316406250,
+0.0010070800781250, 0.0016479492187500, 0.0010070800781250, 0.0016479492187500,
+0.0010986328125000, 0.0016174316406250, 0.0009460449218750, 0.0015258789062500,
+0.0009460449218750, 0.0016174316406250, 0.0011596679687500, 0.0014038085937500,
+0.0010375976562500, 0.0014038085937500, 0.0012817382812500, 0.0014343261718750,
+0.0015869140625000, 0.0010986328125000, 0.0014343261718750, 0.0014648437500000,
+0.0018920898437500, 0.0013732910156250, 0.0019836425781250, 0.0012207031250000,
+0.0019531250000000, 0.0013732910156250, 0.0022277832031250, 0.0012207031250000,
+0.0021972656250000, 0.0010986328125000, 0.0019531250000000, 0.0012817382812500,
+0.0021362304687500, 0.0011291503906250, 0.0019836425781250, 0.0008850097656250,
+0.0017089843750000, 0.0011901855468750, 0.0018920898437500, 0.0008239746093750,
+0.0014038085937500, 0.0008544921875000, 0.0015258789062500, 0.0009765625000000,
+0.0015258789062500, 0.0008239746093750, 0.0012207031250000, 0.0009765625000000,
+0.0013732910156250, 0.0011901855468750, 0.0012817382812500, 0.0010986328125000,
+0.0013427734375000, 0.0012207031250000, 0.0012512207031250, 0.0015563964843750,
+0.0010986328125000, 0.0012512207031250, 0.0012817382812500, 0.0017089843750000,
+0.0009155273437500, 0.0015563964843750, 0.0010681152343750, 0.0015563964843750,
+0.0008850097656250, 0.0019226074218750, 0.0005187988281250, 0.0017089843750000,
+0.0007934570312500, 0.0018005371093750, 0.0002746582031250, 0.0017700195312500,
+0.0002136230468750, 0.0017395019531250, 0.0001831054687500, 0.0017700195312500,
+-0.0001220703125000, 0.0017700195312500, -0.0001525878906250, 0.0019531250000000,
+-0.0000610351562500, 0.0018920898437500, -0.0002441406250000, 0.0020141601562500,
+-0.0000610351562500, 0.0021972656250000, 0.0001525878906250, 0.0020751953125000,
+-0.0000305175781250, 0.0023803710937500, 0.0003967285156250, 0.0024108886718750,
+0.0003662109375000, 0.0022888183593750, 0.0002441406250000, 0.0024414062500000,
+0.0003662109375000, 0.0024719238281250, 0.0002441406250000, 0.0019531250000000,
+0.0002746582031250, 0.0021057128906250, 0.0002136230468750, 0.0018615722656250,
+0.0001525878906250, 0.0014343261718750, 0.0001220703125000, 0.0016174316406250,
+0.0001220703125000, 0.0010986328125000, 0.0000915527343750, 0.0010070800781250,
+0.0000915527343750, 0.0010681152343750, 0.0001525878906250, 0.0008850097656250,
+0.0002136230468750, 0.0007629394531250, 0.0002136230468750, 0.0009460449218750,
+0.0003967285156250, 0.0008850097656250, 0.0002746582031250, 0.0008544921875000,
+0.0003051757812500, 0.0009765625000000, 0.0005187988281250, 0.0008544921875000,
+0.0004577636718750, 0.0007019042968750, 0.0003967285156250, 0.0007019042968750,
+0.0005493164062500, 0.0006713867187500, 0.0003356933593750, 0.0005493164062500,
+0.0001220703125000, 0.0004577636718750, 0.0003356933593750, 0.0004577636718750,
+-0.0000915527343750, 0.0003662109375000, -0.0000305175781250, 0.0003356933593750,
+-0.0000305175781250, 0.0003967285156250, -0.0003356933593750, 0.0001831054687500,
+-0.0002136230468750, 0.0003662109375000, -0.0001220703125000, 0.0002136230468750,
+-0.0002136230468750, 0.0002746582031250, 0.0000000000000000, 0.0003967285156250,
+0.0002136230468750, 0.0001525878906250, 0.0000305175781250, 0.0005187988281250,
+0.0004577636718750, 0.0003051757812500, 0.0003662109375000, 0.0005187988281250,
+0.0004272460937500, 0.0008850097656250, 0.0006713867187500, 0.0005187988281250,
+0.0004272460937500, 0.0012512207031250, 0.0008239746093750, 0.0010681152343750,
+0.0005798339843750, 0.0011596679687500, 0.0006103515625000, 0.0017089843750000,
+0.0009155273437500, 0.0014343261718750, 0.0006103515625000, 0.0016174316406250,
+0.0010070800781250, 0.0017089843750000, 0.0009460449218750, 0.0015869140625000,
+0.0009155273437500, 0.0015869140625000, 0.0010986328125000, 0.0016479492187500,
+0.0009765625000000, 0.0014343261718750, 0.0007934570312500, 0.0014038085937500,
+0.0010070800781250, 0.0013122558593750, 0.0007324218750000, 0.0011901855468750,
+0.0005187988281250, 0.0010986328125000, 0.0008239746093750, 0.0010375976562500,
+-0.0000305175781250, 0.0008239746093750, 0.0000915527343750, 0.0008544921875000,
+0.0000305175781250, 0.0008544921875000, -0.0005493164062500, 0.0006103515625000,
+-0.0003356933593750, 0.0009155273437500, -0.0005187988281250, 0.0007019042968750,
+-0.0006408691406250, 0.0008544921875000, -0.0005187988281250, 0.0011291503906250,
+-0.0005798339843750, 0.0007629394531250, -0.0006408691406250, 0.0012512207031250,
+-0.0003356933593750},
+{-0.0000915527343750, 0.0021057128906250, -0.0005493164062500, 0.0021362304687500,
+-0.0006103515625000, 0.0019836425781250, -0.0007019042968750, 0.0021362304687500,
+-0.0010070800781250, 0.0021057128906250, -0.0008544921875000, 0.0017700195312500,
+-0.0010375976562500, 0.0019531250000000, -0.0010681152343750, 0.0017089843750000,
+-0.0009460449218750, 0.0012207031250000, -0.0009765625000000, 0.0015563964843750,
+-0.0009765625000000, 0.0007019042968750, -0.0007934570312500, 0.0008239746093750,
+-0.0007934570312500, 0.0006713867187500, -0.0006713867187500, 0.0001525878906250,
+-0.0005798339843750, 0.0003356933593750, -0.0006408691406250, 0.0000915527343750,
+-0.0004272460937500, -0.0000305175781250, -0.0004882812500000, 0.0000610351562500,
+-0.0003662109375000, 0.0000000000000000, -0.0002441406250000, -0.0000610351562500,
+-0.0004272460937500, 0.0001525878906250, -0.0000610351562500, 0.0001525878906250,
+-0.0001831054687500, 0.0001525878906250, -0.0001525878906250, 0.0002746582031250,
+0.0000610351562500, 0.0001525878906250, -0.0000915527343750, 0.0002136230468750,
+0.0000915527343750, 0.0002746582031250, 0.0000610351562500, 0.0000610351562500,
+0.0000610351562500, -0.0001220703125000, 0.0002136230468750, -0.0000305175781250,
+0.0001831054687500, -0.0004577636718750, 0.0003662109375000, -0.0004577636718750,
+0.0003662109375000, -0.0003967285156250, 0.0003662109375000, -0.0004882812500000,
+0.0004882812500000, -0.0005493164062500, 0.0005187988281250, -0.0003967285156250,
+0.0008544921875000, -0.0003356933593750, 0.0007324218750000, -0.0004577636718750,
+0.0010375976562500, -0.0003967285156250, 0.0014343261718750, -0.0003051757812500,
+0.0011596679687500, -0.0006408691406250, 0.0017395019531250, -0.0004577636718750,
+0.0017700195312500, -0.0005187988281250, 0.0016174316406250, -0.0008239746093750,
+0.0017700195312500, -0.0004577636718750, 0.0018005371093750, -0.0007019042968750,
+0.0011901855468750, -0.0006713867187500, 0.0013122558593750, -0.0004882812500000,
+0.0008239746093750, -0.0005493164062500, 0.0000915527343750, -0.0003967285156250,
+0.0003662109375000, -0.0002136230468750, -0.0007934570312500, -0.0002136230468750,
+-0.0008544921875000, -0.0001525878906250, -0.0010986328125000, 0.0000000000000000,
+-0.0018310546875000, 0.0000000000000000, -0.0016174316406250, 0.0001831054687500,
+-0.0020141601562500, 0.0002136230468750, -0.0021972656250000, 0.0003356933593750,
+-0.0020141601562500, 0.0005493164062500, -0.0020751953125000, 0.0005493164062500,
+-0.0021362304687500, 0.0008239746093750, -0.0013122558593750, 0.0009155273437500,
+-0.0016479492187500, 0.0010070800781250, -0.0011291503906250, 0.0010986328125000,
+-0.0002441406250000, 0.0012512207031250, -0.0007934570312500, 0.0011596679687500,
+0.0005798339843750, 0.0013122558593750, 0.0004882812500000, 0.0012512207031250,
+0.0007019042968750, 0.0011291503906250, 0.0017089843750000, 0.0013427734375000,
+0.0012817382812500, 0.0010070800781250, 0.0019836425781250, 0.0010681152343750,
+0.0021057128906250, 0.0011596679687500, 0.0020446777343750, 0.0010070800781250,
+0.0022583007812500, 0.0010986328125000, 0.0021972656250000, 0.0010375976562500,
+0.0020751953125000, 0.0010070800781250, 0.0021362304687500, 0.0010375976562500,
+0.0020141601562500, 0.0010070800781250, 0.0019531250000000, 0.0010070800781250,
+0.0019836425781250, 0.0009155273437500, 0.0017700195312500, 0.0008850097656250,
+0.0018005371093750, 0.0009155273437500, 0.0016479492187500, 0.0008239746093750,
+0.0013122558593750, 0.0008239746093750, 0.0013427734375000, 0.0007934570312500,
+0.0010986328125000, 0.0007629394531250, 0.0010375976562500, 0.0008544921875000,
+0.0010681152343750, 0.0009765625000000, 0.0009765625000000, 0.0009155273437500,
+0.0009460449218750, 0.0012207031250000, 0.0011596679687500, 0.0012207031250000,
+0.0011596679687500, 0.0013427734375000, 0.0012207031250000, 0.0016479492187500,
+0.0012817382812500, 0.0015563964843750, 0.0011901855468750, 0.0018920898437500,
+0.0011291503906250, 0.0018920898437500, 0.0012207031250000, 0.0020446777343750,
+0.0011291503906250, 0.0022888183593750, 0.0009765625000000, 0.0021362304687500,
+0.0009765625000000, 0.0024414062500000, 0.0009765625000000, 0.0024414062500000,
+0.0007629394531250, 0.0023803710937500, 0.0009155273437500, 0.0024414062500000,
+0.0010986328125000, 0.0024108886718750, 0.0008239746093750, 0.0022277832031250,
+0.0013732910156250, 0.0021057128906250, 0.0012207031250000, 0.0019226074218750,
+0.0011901855468750, 0.0016479492187500, 0.0015563964843750, 0.0015869140625000,
+0.0013427734375000, 0.0015869140625000, 0.0011291503906250, 0.0012207031250000,
+0.0013122558593750, 0.0013427734375000, 0.0009155273437500, 0.0015258789062500,
+0.0004577636718750, 0.0010986328125000, 0.0007629394531250, 0.0015869140625000,
+0.0000915527343750, 0.0014343261718750, 0.0001220703125000, 0.0013427734375000,
+0.0000915527343750, 0.0016174316406250, -0.0002136230468750, 0.0014038085937500,
+-0.0000915527343750, 0.0013122558593750, -0.0004577636718750, 0.0013427734375000,
+-0.0005187988281250, 0.0012512207031250, -0.0004882812500000, 0.0010681152343750,
+-0.0006103515625000, 0.0011901855468750, -0.0007019042968750, 0.0009460449218750,
+-0.0004272460937500, 0.0009765625000000, -0.0005798339843750, 0.0010375976562500,
+-0.0004577636718750, 0.0010681152343750, -0.0000915527343750, 0.0010681152343750,
+-0.0002136230468750, 0.0013427734375000, 0.0001525878906250, 0.0013427734375000,
+0.0002136230468750, 0.0014953613281250, 0.0003662109375000, 0.0017700195312500,
+0.0007019042968750, 0.0017089843750000, 0.0007019042968750, 0.0019836425781250,
+0.0006408691406250, 0.0021362304687500, 0.0008239746093750, 0.0020141601562500,
+0.0007629394531250, 0.0018310546875000, 0.0004577636718750, 0.0009155273437500,
+-0.0003662109375000, 0.0000000000000000, -0.0001831054687500, 0.0004272460937500,
+-0.0001525878906250, 0.0000915527343750, -0.0002441406250000, -0.0004272460937500,
+-0.0001831054687500, -0.0001220703125000, -0.0000610351562500, -0.0008239746093750,
+-0.0002136230468750, -0.0007934570312500, -0.0002441406250000, -0.0007934570312500,
+-0.0002136230468750, -0.0011901855468750, -0.0002746582031250, -0.0010681152343750,
+-0.0002746582031250, -0.0011596679687500, -0.0001525878906250, -0.0012207031250000,
+-0.0002441406250000, -0.0011596679687500, -0.0003967285156250, -0.0010986328125000,
+-0.0003356933593750, -0.0011596679687500, -0.0003356933593750, -0.0009765625000000,
+-0.0008850097656250, -0.0011291503906250, -0.0007934570312500, -0.0009765625000000,
+-0.0006408691406250, -0.0007934570312500, -0.0009460449218750, -0.0010681152343750,
+-0.0009765625000000, -0.0003662109375000, -0.0002746582031250, -0.0007019042968750,
+-0.0005187988281250, -0.0003967285156250, -0.0002441406250000, 0.0003662109375000,
+0.0003967285156250, -0.0001831054687500, -0.0000610351562500, 0.0009460449218750,
+0.0006713867187500, 0.0009460449218750, 0.0007019042968750, 0.0010070800781250,
+0.0006713867187500, 0.0017395019531250, 0.0010681152343750, 0.0015869140625000,
+0.0008239746093750, 0.0019226074218750, 0.0008239746093750, 0.0021667480468750,
+0.0008544921875000, 0.0020141601562500, 0.0007629394531250, 0.0020446777343750,
+0.0006713867187500, 0.0021972656250000, 0.0006713867187500, 0.0014648437500000,
+0.0007019042968750, 0.0017395019531250, 0.0006713867187500, 0.0012817382812500,
+0.0008239746093750, 0.0004882812500000, 0.0010375976562500, 0.0009460449218750,
+0.0008544921875000, -0.0004577636718750, 0.0013427734375000, -0.0003356933593750,
+0.0014953613281250, -0.0004882812500000, 0.0014648437500000, -0.0013427734375000,
+0.0016479492187500, -0.0009460449218750, 0.0018005371093750, -0.0010375976562500,
+0.0014343261718750, -0.0013732910156250, 0.0017395019531250, -0.0007324218750000,
+0.0015563964843750, -0.0002746582031250, 0.0010681152343750, -0.0007629394531250,
+0.0014648437500000, 0.0006408691406250, 0.0013122558593750, 0.0006103515625000,
+0.0011291503906250, 0.0006103515625000, 0.0015563964843750, 0.0014648437500000,
+0.0016784667968750, 0.0011596679687500, 0.0013427734375000, 0.0010070800781250,
+0.0024108886718750, 0.0012512207031250, 0.0020141601562500, 0.0008239746093750,
+0.0021667480468750, 0.0003356933593750, 0.0028991699218750, 0.0005798339843750,
+0.0021972656250000, -0.0002136230468750, 0.0025634765625000, -0.0002746582031250,
+0.0026245117187500, -0.0002746582031250, 0.0019836425781250, -0.0006713867187500,
+0.0018310546875000, -0.0006713867187500, 0.0019836425781250, -0.0003662109375000,
+0.0008850097656250, -0.0006408691406250, 0.0011291503906250, -0.0003356933593750,
+0.0009155273437500, 0.0001525878906250, 0.0000915527343750, -0.0002136230468750,
+0.0006103515625000, 0.0006713867187500, 0.0001831054687500, 0.0006103515625000,
+0.0001831054687500, 0.0006103515625000, 0.0003967285156250, 0.0010986328125000,
+0.0003356933593750, 0.0009460449218750, 0.0005187988281250, 0.0010681152343750,
+0.0005187988281250, 0.0011291503906250, 0.0007629394531250, 0.0011291503906250,
+0.0007019042968750, 0.0011596679687500, 0.0006408691406250, 0.0011596679687500,
+0.0008544921875000, 0.0012817382812500, 0.0003662109375000, 0.0012207031250000,
+0.0005493164062500, 0.0013732910156250, 0.0005798339843750, 0.0015563964843750,
+0.0002746582031250, 0.0014038085937500, 0.0005187988281250, 0.0019226074218750,
+0.0003051757812500, 0.0018310546875000, 0.0004577636718750, 0.0018310546875000,
+0.0004272460937500, 0.0022583007812500, 0.0003051757812500, 0.0020751953125000,
+0.0005798339843750, 0.0019226074218750, 0.0002441406250000, 0.0021057128906250,
+0.0004882812500000, 0.0017089843750000, 0.0004272460937500, 0.0013427734375000,
+0.0001220703125000, 0.0015869140625000, 0.0005187988281250, 0.0006103515625000,
+0.0003051757812500, 0.0006713867187500, 0.0003967285156250, 0.0004577636718750,
+0.0005493164062500, -0.0002441406250000, 0.0005187988281250, 0.0000305175781250,
+0.0006713867187500, -0.0003662109375000, 0.0007629394531250, -0.0005493164062500,
+0.0008544921875000, -0.0004272460937500, 0.0009765625000000, -0.0004882812500000,
+0.0011291503906250, -0.0006713867187500, 0.0012207031250000, -0.0002441406250000,
+0.0012207031250000, -0.0004882812500000, 0.0012817382812500, -0.0002441406250000,
+0.0012512207031250, 0.0002441406250000, 0.0012207031250000, -0.0002136230468750,
+0.0013122558593750, 0.0006408691406250, 0.0011596679687500, 0.0004577636718750,
+0.0011901855468750, 0.0006713867187500, 0.0011901855468750, 0.0012512207031250,
+0.0010681152343750, 0.0008850097656250, 0.0011291503906250, 0.0015258789062500,
+0.0011596679687500, 0.0014648437500000, 0.0010681152343750, 0.0015869140625000,
+0.0011291503906250, 0.0020141601562500, 0.0011901855468750, 0.0017700195312500,
+0.0010070800781250, 0.0021667480468750, 0.0012817382812500, 0.0021667480468750,
+0.0011901855468750, 0.0022583007812500, 0.0012512207031250, 0.0025024414062500,
+0.0014343261718750, 0.0023803710937500, 0.0013122558593750, 0.0026550292968750,
+0.0016784667968750, 0.0026245117187500, 0.0014953613281250, 0.0025329589843750,
+0.0016479492187500, 0.0026550292968750, 0.0020141601562500, 0.0026855468750000,
+0.0018005371093750, 0.0022888183593750, 0.0022888183593750, 0.0024719238281250,
+0.0022583007812500, 0.0021057128906250, 0.0023193359375000, 0.0015869140625000,
+0.0026855468750000, 0.0019531250000000, 0.0025329589843750, 0.0009155273437500,
+0.0024414062500000, 0.0010681152343750, 0.0026245117187500, 0.0008544921875000,
+0.0024108886718750},
+{0.0011901855468750, 0.0009460449218750, 0.0013427734375000, 0.0009460449218750,
+0.0015258789062500, 0.0006408691406250, 0.0012207031250000, 0.0016174316406250,
+0.0018005371093750, 0.0013427734375000, 0.0016784667968750, 0.0016784667968750,
+0.0017700195312500, 0.0025024414062500, 0.0022888183593750, 0.0019226074218750,
+0.0020141601562500, 0.0028076171875000, 0.0021057128906250, 0.0028381347656250,
+0.0024414062500000, 0.0027465820312500, 0.0022583007812500, 0.0031433105468750,
+0.0019836425781250, 0.0029907226562500, 0.0022888183593750, 0.0031433105468750,
+0.0019226074218750, 0.0030822753906250, 0.0018615722656250, 0.0031127929687500,
+0.0020751953125000, 0.0031433105468750, 0.0019226074218750, 0.0030212402343750,
+0.0019226074218750, 0.0031738281250000, 0.0022888183593750, 0.0031127929687500,
+0.0020751953125000, 0.0029602050781250, 0.0023498535156250, 0.0029296875000000,
+0.0027770996093750, 0.0028991699218750, 0.0024108886718750, 0.0025024414062500,
+0.0030517578125000, 0.0025939941406250, 0.0030212402343750, 0.0023498535156250,
+0.0030212402343750, 0.0019836425781250, 0.0033569335937500, 0.0022277832031250,
+0.0032043457031250, 0.0013732910156250, 0.0033569335937500, 0.0016479492187500,
+0.0033874511718750, 0.0012512207031250, 0.0032958984375000, 0.0004577636718750,
+0.0033569335937500, 0.0009765625000000, 0.0033569335937500, 0.0001220703125000,
+0.0032958984375000, 0.0000305175781250, 0.0033569335937500, 0.0002746582031250,
+0.0033569335937500, 0.0000305175781250, 0.0032653808593750, 0.0000305175781250,
+0.0033264160156250, 0.0007324218750000, 0.0032043457031250, 0.0004882812500000,
+0.0031738281250000, 0.0009460449218750, 0.0032043457031250, 0.0017089843750000,
+0.0031433105468750, 0.0012817382812500, 0.0029907226562500, 0.0023193359375000,
+0.0033264160156250, 0.0022583007812500, 0.0030212402343750, 0.0023193359375000,
+0.0031738281250000, 0.0029296875000000, 0.0036010742187500, 0.0026245117187500,
+0.0031433105468750, 0.0025939941406250, 0.0037231445312500, 0.0027465820312500,
+0.0035400390625000, 0.0024414062500000, 0.0034484863281250, 0.0021667480468750,
+0.0037536621093750, 0.0023498535156250, 0.0034179687500000, 0.0014953613281250,
+0.0034790039062500, 0.0016479492187500, 0.0033569335937500, 0.0014648437500000,
+0.0031433105468750, 0.0008239746093750, 0.0030212402343750, 0.0011901855468750,
+0.0028381347656250, 0.0007934570312500, 0.0025329589843750, 0.0007629394531250,
+0.0023803710937500, 0.0007934570312500, 0.0022583007812500, 0.0006713867187500,
+0.0019531250000000, 0.0007324218750000, 0.0018005371093750, 0.0007019042968750,
+0.0017700195312500, 0.0006713867187500, 0.0013732910156250, 0.0007019042968750,
+0.0014648437500000, 0.0007019042968750, 0.0016174316406250, 0.0007324218750000,
+0.0012207031250000, 0.0007629394531250, 0.0019836425781250, 0.0007324218750000,
+0.0015258789062500, 0.0008850097656250, 0.0017089843750000, 0.0009460449218750,
+0.0023498535156250, 0.0009460449218750, 0.0016479492187500, 0.0013122558593750,
+0.0026245117187500, 0.0013427734375000, 0.0023498535156250, 0.0014343261718750,
+0.0024719238281250, 0.0018005371093750, 0.0031127929687500, 0.0017700195312500,
+0.0025939941406250, 0.0017089843750000, 0.0032958984375000, 0.0018615722656250,
+0.0032043457031250, 0.0016174316406250, 0.0031738281250000, 0.0012512207031250,
+0.0035400390625000, 0.0014953613281250, 0.0032958984375000, 0.0004882812500000,
+0.0033569335937500, 0.0005187988281250, 0.0033874511718750, 0.0002136230468750,
+0.0032348632812500, -0.0006103515625000, 0.0030822753906250, -0.0003356933593750,
+0.0031127929687500, -0.0009155273437500, 0.0027770996093750, -0.0010986328125000,
+0.0028076171875000, -0.0009155273437500, 0.0026245117187500, -0.0010681152343750,
+0.0024108886718750, -0.0011596679687500, 0.0025634765625000, -0.0005187988281250,
+0.0022888183593750, -0.0007324218750000, 0.0024108886718750, -0.0005493164062500,
+0.0024108886718750, 0.0000305175781250, 0.0022888183593750, -0.0002441406250000,
+0.0025024414062500, -0.0000610351562500, 0.0022583007812500, 0.0001220703125000,
+0.0024108886718750, 0.0000915527343750, 0.0023498535156250, 0.0000915527343750,
+0.0022583007812500, 0.0001831054687500, 0.0024414062500000, 0.0000610351562500,
+0.0019531250000000, 0.0000915527343750, 0.0022277832031250, 0.0000305175781250,
+0.0020751953125000, -0.0000915527343750, 0.0015869140625000, -0.0000610351562500,
+0.0020446777343750, -0.0002441406250000, 0.0013427734375000, -0.0003051757812500,
+0.0014038085937500, -0.0004272460937500, 0.0014343261718750, -0.0006408691406250,
+0.0010681152343750, -0.0005493164062500, 0.0012512207031250, -0.0006713867187500,
+0.0011291503906250, -0.0007934570312500, 0.0010986328125000, -0.0005187988281250,
+0.0011596679687500, -0.0002746582031250, 0.0011291503906250, -0.0003967285156250,
+0.0010986328125000, 0.0002136230468750, 0.0012512207031250, 0.0002746582031250,
+0.0011596679687500, 0.0002441406250000, 0.0012207031250000, 0.0006713867187500,
+0.0013427734375000, 0.0008850097656250, 0.0013732910156250, 0.0010070800781250,
+0.0016784667968750, 0.0009765625000000, 0.0015563964843750, 0.0011291503906250,
+0.0015869140625000, 0.0013122558593750, 0.0018920898437500, 0.0012207031250000,
+0.0018005371093750, 0.0015258789062500, 0.0016784667968750, 0.0016784667968750,
+0.0018310546875000, 0.0015563964843750, 0.0015869140625000, 0.0014953613281250,
+0.0013427734375000, 0.0016479492187500, 0.0015563964843750, 0.0012512207031250,
+0.0008850097656250, 0.0013122558593750, 0.0010375976562500, 0.0011291503906250,
+0.0009155273437500, 0.0007934570312500, 0.0004577636718750, 0.0010681152343750,
+0.0007934570312500, 0.0009155273437500, 0.0003662109375000, 0.0010375976562500,
+0.0003051757812500, 0.0012512207031250, 0.0003967285156250, 0.0012512207031250,
+0.0002746582031250, 0.0014648437500000, 0.0003967285156250, 0.0019531250000000,
+0.0003967285156250, 0.0020446777343750, 0.0004577636718750, 0.0022277832031250,
+0.0003967285156250, 0.0026245117187500, 0.0003662109375000, 0.0025024414062500,
+0.0004882812500000, 0.0025939941406250, 0.0000915527343750, 0.0027770996093750,
+0.0002136230468750, 0.0025634765625000, 0.0001831054687500, 0.0023803710937500,
+-0.0000915527343750, 0.0025024414062500, 0.0000000000000000, 0.0018310546875000,
+-0.0003356933593750, 0.0019226074218750, -0.0002746582031250, 0.0016174316406250,
+-0.0001831054687500, 0.0010070800781250, -0.0003356933593750, 0.0013427734375000,
+-0.0002136230468750, 0.0010070800781250, -0.0000610351562500, 0.0006713867187500,
+-0.0000305175781250, 0.0009460449218750, 0.0000915527343750, 0.0011596679687500,
+0.0003356933593750, 0.0007934570312500, 0.0003051757812500, 0.0015258789062500,
+0.0005798339843750, 0.0013427734375000, 0.0006103515625000, 0.0014343261718750,
+0.0007934570312500, 0.0019226074218750, 0.0009765625000000, 0.0016479492187500,
+0.0008850097656250, 0.0019226074218750, 0.0010681152343750, 0.0020141601562500,
+0.0011291503906250, 0.0018310546875000, 0.0010375976562500, 0.0018615722656250,
+0.0010681152343750, 0.0021362304687500, 0.0011291503906250, 0.0015563964843750,
+0.0009155273437500, 0.0018615722656250, 0.0009765625000000, 0.0019226074218750,
+0.0010375976562500, 0.0015563964843750, 0.0008850097656250, 0.0020446777343750,
+0.0009765625000000, 0.0018310546875000, 0.0010986328125000, 0.0020751953125000,
+0.0010375976562500, 0.0022888183593750, 0.0011901855468750, 0.0021972656250000,
+0.0012817382812500, 0.0024414062500000, 0.0011901855468750, 0.0024414062500000,
+0.0017089843750000, 0.0025634765625000, 0.0016479492187500, 0.0026245117187500,
+0.0018005371093750, 0.0026245117187500, 0.0022888183593750, 0.0026855468750000,
+0.0021362304687500, 0.0027160644531250, 0.0025634765625000, 0.0026855468750000,
+0.0026855468750000, 0.0028381347656250, 0.0026855468750000, 0.0028381347656250,
+0.0028381347656250, 0.0026855468750000, 0.0028991699218750, 0.0032043457031250,
+0.0026855468750000, 0.0029602050781250, 0.0028381347656250, 0.0030822753906250,
+0.0026245117187500, 0.0034790039062500, 0.0023498535156250, 0.0029907226562500,
+0.0025634765625000, 0.0034790039062500, 0.0017395019531250, 0.0033569335937500,
+0.0018310546875000, 0.0032043457031250, 0.0015563964843750, 0.0033569335937500,
+0.0008239746093750, 0.0030212402343750, 0.0010986328125000, 0.0031127929687500,
+0.0004272460937500, 0.0028991699218750, 0.0003356933593750, 0.0028076171875000,
+0.0003662109375000, 0.0028381347656250, 0.0000915527343750, 0.0024108886718750,
+0.0001831054687500, 0.0028076171875000, 0.0003051757812500, 0.0025329589843750,
+0.0002441406250000, 0.0025939941406250, 0.0004577636718750, 0.0028991699218750,
+0.0006713867187500, 0.0025329589843750, 0.0005798339843750, 0.0028076171875000,
+0.0012512207031250, 0.0027465820312500, 0.0011291503906250, 0.0025634765625000,
+0.0014648437500000, 0.0025634765625000, 0.0019836425781250, 0.0025024414062500,
+0.0016784667968750, 0.0023803710937500, 0.0025939941406250, 0.0022583007812500,
+0.0025329589843750, 0.0022583007812500, 0.0026550292968750, 0.0021972656250000,
+0.0032348632812500, 0.0020141601562500, 0.0028991699218750, 0.0020141601562500,
+0.0032958984375000, 0.0019226074218750, 0.0033874511718750, 0.0018920898437500,
+0.0032043457031250, 0.0019226074218750, 0.0032958984375000, 0.0017395019531250,
+0.0032958984375000, 0.0019836425781250, 0.0028076171875000, 0.0018615722656250,
+0.0028991699218750, 0.0018920898437500, 0.0025939941406250, 0.0019836425781250,
+0.0020446777343750, 0.0018005371093750, 0.0023193359375000, 0.0020446777343750,
+0.0015869140625000, 0.0018310546875000, 0.0014648437500000, 0.0018615722656250,
+0.0014343261718750, 0.0021057128906250, 0.0010681152343750, 0.0017395019531250,
+0.0010681152343750, 0.0020751953125000, 0.0012207031250000, 0.0019226074218750,
+0.0009155273437500, 0.0019226074218750, 0.0010681152343750, 0.0021057128906250,
+0.0013732910156250, 0.0018310546875000, 0.0010681152343750, 0.0020751953125000,
+0.0014648437500000, 0.0018920898437500, 0.0014648437500000, 0.0019531250000000,
+0.0014038085937500, 0.0021667480468750, 0.0016479492187500, 0.0018310546875000,
+0.0015563964843750, 0.0022583007812500, 0.0016174316406250, 0.0021057128906250,
+0.0016784667968750, 0.0021667480468750, 0.0015563964843750, 0.0025634765625000,
+0.0014953613281250, 0.0022277832031250, 0.0015563964843750, 0.0024719238281250,
+0.0012512207031250, 0.0025024414062500, 0.0013122558593750, 0.0023193359375000,
+0.0011596679687500, 0.0023193359375000, 0.0009155273437500, 0.0024108886718750,
+0.0009765625000000, 0.0018920898437500, 0.0006408691406250, 0.0020141601562500,
+0.0006408691406250, 0.0018310546875000, 0.0005798339843750, 0.0014343261718750,
+0.0003662109375000, 0.0016784667968750, 0.0003967285156250, 0.0012512207031250,
+0.0003356933593750, 0.0012817382812500, 0.0003051757812500, 0.0011901855468750,
+0.0003051757812500, 0.0009155273437500, 0.0002136230468750, 0.0011901855468750,
+0.0001831054687500, 0.0005798339843750, 0.0000915527343750, 0.0008544921875000,
+0.0001831054687500, 0.0007324218750000, 0.0000000000000000, 0.0002441406250000,
+-0.0003051757812500, 0.0007629394531250, -0.0001525878906250, 0.0000000000000000,
+-0.0002746582031250, 0.0002136230468750, -0.0003967285156250, 0.0003356933593750,
+-0.0002441406250000, -0.0000610351562500, -0.0001525878906250, 0.0003356933593750,
+-0.0003356933593750},
+{0.0002441406250000, 0.0028381347656250, 0.0003356933593750, 0.0029296875000000,
+0.0004577636718750, 0.0027770996093750, 0.0001831054687500, 0.0029602050781250,
+0.0006408691406250, 0.0032043457031250, 0.0010375976562500, 0.0030212402343750,
+0.0007019042968750, 0.0033569335937500, 0.0018920898437500, 0.0033569335937500,
+0.0017395019531250, 0.0033264160156250, 0.0021362304687500, 0.0035095214843750,
+0.0031127929687500, 0.0035705566406250, 0.0026245117187500, 0.0034484863281250,
+0.0035400390625000, 0.0036315917968750, 0.0036315917968750, 0.0036010742187500,
+0.0035095214843750, 0.0034484863281250, 0.0038146972656250, 0.0036926269531250,
+0.0037231445312500, 0.0033569335937500, 0.0036010742187500, 0.0035095214843750,
+0.0037536621093750, 0.0034790039062500, 0.0035705566406250, 0.0032348632812500,
+0.0033569335937500, 0.0034484863281250, 0.0034790039062500, 0.0031127929687500,
+0.0028991699218750, 0.0032653808593750, 0.0030212402343750, 0.0032043457031250,
+0.0027770996093750, 0.0029602050781250, 0.0021972656250000, 0.0031127929687500,
+0.0023803710937500, 0.0028991699218750, 0.0016784667968750, 0.0028991699218750,
+0.0016174316406250, 0.0028076171875000, 0.0014953613281250, 0.0025939941406250,
+0.0011291503906250, 0.0027160644531250, 0.0011901855468750, 0.0024719238281250,
+0.0009460449218750, 0.0024719238281250, 0.0008239746093750, 0.0024719238281250,
+0.0007934570312500, 0.0022888183593750, 0.0007019042968750, 0.0024108886718750,
+0.0006408691406250, 0.0023193359375000, 0.0006103515625000, 0.0023193359375000,
+0.0005798339843750, 0.0024108886718750, 0.0005798339843750, 0.0023803710937500,
+0.0005493164062500, 0.0023193359375000, 0.0004577636718750, 0.0025329589843750,
+0.0007324218750000, 0.0024414062500000, 0.0004882812500000, 0.0025329589843750,
+0.0008239746093750, 0.0028076171875000, 0.0012512207031250, 0.0025634765625000,
+0.0008850097656250, 0.0028686523437500, 0.0019836425781250, 0.0028381347656250,
+0.0018615722656250, 0.0027160644531250, 0.0022277832031250, 0.0027770996093750,
+0.0030212402343750, 0.0026855468750000, 0.0025634765625000, 0.0025939941406250,
+0.0033569335937500, 0.0025634765625000, 0.0035095214843750, 0.0024414062500000,
+0.0034179687500000, 0.0022888183593750, 0.0037231445312500, 0.0022277832031250,
+0.0037841796875000, 0.0020446777343750, 0.0035705566406250, 0.0020141601562500,
+0.0036926269531250, 0.0018615722656250, 0.0035095214843750, 0.0017395019531250,
+0.0031738281250000, 0.0016479492187500, 0.0033264160156250, 0.0016479492187500,
+0.0028991699218750, 0.0015869140625000, 0.0028686523437500, 0.0014953613281250,
+0.0026550292968750, 0.0014648437500000, 0.0023498535156250, 0.0014648437500000,
+0.0024108886718750, 0.0012512207031250, 0.0019531250000000, 0.0013732910156250,
+0.0019226074218750, 0.0011901855468750, 0.0017395019531250, 0.0008544921875000,
+0.0014343261718750, 0.0010986328125000, 0.0015258789062500, 0.0007629394531250,
+0.0011291503906250, 0.0007934570312500, 0.0011596679687500, 0.0008544921875000,
+0.0009155273437500, 0.0007019042968750, 0.0005798339843750, 0.0008850097656250,
+0.0007629394531250, 0.0008544921875000, -0.0000610351562500, 0.0009155273437500,
+0.0000000000000000, 0.0009155273437500, -0.0002746582031250, 0.0009460449218750,
+-0.0009460449218750, 0.0010375976562500, -0.0007324218750000, 0.0008544921875000,
+-0.0012817382812500, 0.0010070800781250, -0.0013427734375000, 0.0008850097656250,
+-0.0012817382812500, 0.0006713867187500, -0.0014953613281250, 0.0009460449218750,
+-0.0014648437500000, 0.0006713867187500, -0.0011901855468750, 0.0007324218750000,
+-0.0013122558593750, 0.0006713867187500, -0.0010070800781250, 0.0004882812500000,
+-0.0005798339843750, 0.0006713867187500, -0.0007324218750000, 0.0003967285156250,
+-0.0001220703125000, 0.0004577636718750, 0.0000000000000000, 0.0004577636718750,
+0.0000000000000000, 0.0003051757812500, 0.0003051757812500, 0.0003662109375000,
+0.0003662109375000, 0.0001220703125000, 0.0002441406250000, 0.0002136230468750,
+0.0003051757812500, 0.0000610351562500, 0.0003356933593750, -0.0001525878906250,
+0.0003051757812500, 0.0000000000000000, 0.0003356933593750, -0.0004882812500000,
+0.0001831054687500, -0.0005187988281250, 0.0001831054687500, -0.0004882812500000,
+-0.0000305175781250, -0.0008850097656250, -0.0004577636718750, -0.0006103515625000,
+0.0009765625000000, 0.0009765625000000, 0.0013122558593750, 0.0003662109375000,
+0.0014648437500000, 0.0008544921875000, 0.0010070800781250, 0.0017089843750000,
+0.0007629394531250, 0.0012817382812500, 0.0010070800781250, 0.0018920898437500,
+-0.0001831054687500, 0.0020141601562500, 0.0001525878906250, 0.0018310546875000,
+-0.0003356933593750, 0.0020751953125000, -0.0014648437500000, 0.0020446777343750,
+-0.0007934570312500, 0.0015563964843750, -0.0014953613281250, 0.0018310546875000,
+-0.0017089843750000, 0.0015563964843750, -0.0013732910156250, 0.0010986328125000,
+-0.0013427734375000, 0.0015258789062500, -0.0014953613281250, 0.0007934570312500,
+-0.0010681152343750, 0.0008850097656250, -0.0012207031250000, 0.0007934570312500,
+-0.0010070800781250, 0.0003356933593750, -0.0006408691406250, 0.0005187988281250,
+-0.0009765625000000, 0.0000610351562500, -0.0003051757812500, 0.0001220703125000,
+-0.0004577636718750, -0.0001220703125000, -0.0001525878906250, -0.0005493164062500,
+0.0003662109375000, -0.0001831054687500, -0.0000915527343750, -0.0006408691406250,
+0.0009765625000000, -0.0007324218750000, 0.0009460449218750, -0.0003662109375000,
+0.0008850097656250, -0.0003662109375000, 0.0014953613281250, -0.0005187988281250,
+0.0012817382812500, 0.0005798339843750, 0.0011596679687500, 0.0003662109375000,
+0.0012817382812500, 0.0007629394531250, 0.0010986328125000, 0.0017089843750000,
+0.0009460449218750, 0.0012207031250000, 0.0009765625000000, 0.0020751953125000,
+0.0008850097656250, 0.0021972656250000, 0.0006408691406250, 0.0018920898437500,
+0.0007324218750000, 0.0020751953125000, 0.0008239746093750, 0.0021667480468750,
+0.0007324218750000, 0.0015258789062500, 0.0013122558593750, 0.0016784667968750,
+0.0012207031250000, 0.0016174316406250, 0.0015563964843750, 0.0011291503906250,
+0.0021362304687500, 0.0012817382812500, 0.0020751953125000, 0.0015869140625000,
+0.0026550292968750, 0.0012817382812500, 0.0028686523437500, 0.0017089843750000,
+0.0027770996093750, 0.0023193359375000, 0.0028686523437500, 0.0018920898437500,
+0.0030517578125000, 0.0027465820312500, 0.0031433105468750, 0.0028686523437500,
+0.0031738281250000, 0.0027160644531250, 0.0032958984375000, 0.0030822753906250,
+0.0033264160156250, 0.0030822753906250, 0.0033569335937500, 0.0027770996093750,
+0.0036621093750000, 0.0028686523437500, 0.0036315917968750, 0.0028991699218750,
+0.0037841796875000, 0.0027160644531250, 0.0039978027343750, 0.0028076171875000,
+0.0037536621093750, 0.0029296875000000, 0.0039978027343750, 0.0028991699218750,
+0.0039062500000000, 0.0030517578125000, 0.0037841796875000, 0.0032348632812500,
+0.0037536621093750, 0.0032043457031250, 0.0035095214843750, 0.0032958984375000,
+0.0033264160156250, 0.0035095214843750, 0.0033264160156250, 0.0032653808593750,
+0.0031738281250000, 0.0031738281250000, 0.0030517578125000, 0.0034790039062500,
+0.0031127929687500, 0.0024108886718750, 0.0028076171875000, 0.0028076171875000,
+0.0030822753906250, 0.0023193359375000, 0.0030822753906250, 0.0013427734375000,
+0.0028991699218750, 0.0019531250000000, 0.0033264160156250, 0.0010375976562500,
+0.0028381347656250, 0.0008544921875000, 0.0032958984375000, 0.0010681152343750,
+0.0031738281250000, 0.0007019042968750, 0.0027160644531250, 0.0007019042968750,
+0.0032653808593750, 0.0010986328125000, 0.0025634765625000, 0.0008544921875000,
+0.0028686523437500, 0.0011901855468750, 0.0028686523437500, 0.0016174316406250,
+0.0023498535156250, 0.0011901855468750, 0.0028381347656250, 0.0022888183593750,
+0.0024414062500000, 0.0021057128906250, 0.0025329589843750, 0.0022583007812500,
+0.0026245117187500, 0.0030822753906250, 0.0024108886718750, 0.0026855468750000,
+0.0025329589843750, 0.0029907226562500, 0.0024719238281250, 0.0032653808593750,
+0.0025024414062500, 0.0029907226562500, 0.0024414062500000, 0.0028076171875000,
+0.0023498535156250, 0.0030212402343750, 0.0025024414062500, 0.0024108886718750,
+0.0022583007812500, 0.0025024414062500, 0.0024414062500000, 0.0023803710937500,
+0.0022888183593750, 0.0019836425781250, 0.0020446777343750, 0.0021362304687500,
+0.0023803710937500, 0.0016784667968750, 0.0018310546875000, 0.0016784667968750,
+0.0019226074218750, 0.0016174316406250, 0.0018615722656250, 0.0013122558593750,
+0.0014953613281250, 0.0013122558593750, 0.0016784667968750, 0.0013122558593750,
+0.0013427734375000, 0.0011291503906250, 0.0013427734375000, 0.0013427734375000,
+0.0013122558593750, 0.0014953613281250, 0.0011596679687500, 0.0012817382812500,
+0.0012512207031250, 0.0020446777343750, 0.0012207031250000, 0.0019226074218750,
+0.0011901855468750, 0.0020446777343750, 0.0013427734375000, 0.0026245117187500,
+0.0014038085937500, 0.0023498535156250, 0.0013122558593750, 0.0024414062500000,
+0.0017395019531250, 0.0026245117187500, 0.0016174316406250, 0.0022277832031250,
+0.0018005371093750, 0.0019836425781250, 0.0021057128906250, 0.0021362304687500,
+0.0018920898437500, 0.0012512207031250, 0.0023193359375000, 0.0013427734375000,
+0.0023193359375000, 0.0010986328125000, 0.0023498535156250, 0.0005187988281250,
+0.0026245117187500, 0.0008239746093750, 0.0025329589843750, 0.0004882812500000,
+0.0026550292968750, 0.0004272460937500, 0.0027160644531250, 0.0005798339843750,
+0.0025634765625000, 0.0005798339843750, 0.0024719238281250, 0.0006408691406250,
+0.0026550292968750, 0.0008850097656250, 0.0023498535156250, 0.0007629394531250,
+0.0024108886718750, 0.0009155273437500, 0.0023193359375000, 0.0010986328125000,
+0.0020446777343750, 0.0009155273437500, 0.0021972656250000, 0.0013732910156250,
+0.0018615722656250, 0.0011291503906250, 0.0018615722656250, 0.0013427734375000,
+0.0019226074218750, 0.0017700195312500, 0.0017089843750000, 0.0014038085937500,
+0.0017700195312500, 0.0022583007812500, 0.0017395019531250, 0.0020141601562500,
+0.0017089843750000, 0.0022583007812500, 0.0017395019531250, 0.0029602050781250,
+0.0018005371093750, 0.0024414062500000, 0.0019531250000000, 0.0031127929687500,
+0.0018005371093750, 0.0031433105468750, 0.0020446777343750, 0.0029296875000000,
+0.0020751953125000, 0.0031433105468750, 0.0019531250000000, 0.0029907226562500,
+0.0021667480468750, 0.0024719238281250, 0.0018920898437500, 0.0026245117187500,
+0.0021057128906250, 0.0022583007812500, 0.0020141601562500, 0.0016174316406250,
+0.0017700195312500, 0.0018615722656250, 0.0020141601562500, 0.0010375976562500,
+0.0017700195312500, 0.0009155273437500, 0.0017089843750000, 0.0007934570312500,
+0.0018310546875000, 0.0003662109375000, 0.0017395019531250, 0.0004272460937500,
+0.0017089843750000, 0.0003051757812500, 0.0018920898437500, 0.0001831054687500,
+0.0018005371093750, 0.0001525878906250, 0.0018310546875000, 0.0000305175781250,
+0.0019531250000000, 0.0000305175781250, 0.0017700195312500, -0.0002746582031250,
+0.0019531250000000, -0.0001831054687500, 0.0018615722656250, -0.0003356933593750,
+0.0018005371093750, -0.0006408691406250, 0.0018310546875000, -0.0004882812500000,
+0.0017700195312500, -0.0007324218750000, 0.0016479492187500, -0.0007324218750000,
+0.0017395019531250},
+{0.0028076171875000, 0.0014953613281250, 0.0027465820312500, 0.0018005371093750,
+0.0027160644531250, 0.0021972656250000, 0.0027465820312500, 0.0018615722656250,
+0.0026245117187500, 0.0028076171875000, 0.0028076171875000, 0.0027465820312500,
+0.0026855468750000, 0.0028686523437500, 0.0026855468750000, 0.0035705566406250,
+0.0028381347656250, 0.0033264160156250, 0.0027160644531250, 0.0035095214843750,
+0.0027770996093750, 0.0037841796875000, 0.0027160644531250, 0.0034179687500000,
+0.0026245117187500, 0.0031738281250000, 0.0025329589843750, 0.0035400390625000,
+0.0024719238281250, 0.0024108886718750, 0.0024719238281250, 0.0025939941406250,
+0.0023498535156250, 0.0022888183593750, 0.0023498535156250, 0.0013732910156250,
+0.0023193359375000, 0.0018005371093750, 0.0021972656250000, 0.0009460449218750,
+0.0023193359375000, 0.0008544921875000, 0.0022277832031250, 0.0008850097656250,
+0.0022583007812500, 0.0004272460937500, 0.0023803710937500, 0.0005493164062500,
+0.0021362304687500, 0.0007324218750000, 0.0025024414062500, 0.0005493164062500,
+0.0023803710937500, 0.0007629394531250, 0.0024108886718750, 0.0010375976562500,
+0.0025939941406250, 0.0008239746093750, 0.0024414062500000, 0.0010681152343750,
+0.0025024414062500, 0.0011291503906250, 0.0024414062500000, 0.0009460449218750,
+0.0024414062500000, 0.0008850097656250, 0.0025329589843750, 0.0008850097656250,
+0.0023498535156250, 0.0005493164062500, 0.0023803710937500, 0.0004882812500000,
+0.0022277832031250, 0.0004272460937500, 0.0021667480468750, 0.0002136230468750,
+0.0021057128906250, 0.0001220703125000, 0.0019531250000000, 0.0004272460937500,
+0.0021057128906250, 0.0001220703125000, 0.0018310546875000, 0.0004882812500000,
+0.0019836425781250, 0.0009765625000000, 0.0021972656250000, 0.0005187988281250,
+0.0018005371093750, 0.0016174316406250, 0.0025329589843750, 0.0014648437500000,
+0.0022583007812500, 0.0017700195312500, 0.0023803710937500, 0.0025939941406250,
+0.0028991699218750, 0.0021057128906250, 0.0023498535156250, 0.0030212402343750,
+0.0029602050781250, 0.0029907226562500, 0.0027465820312500, 0.0031127929687500,
+0.0026245117187500, 0.0036926269531250, 0.0029296875000000, 0.0034484863281250,
+0.0025329589843750, 0.0039978027343750, 0.0024719238281250, 0.0041198730468750,
+0.0024414062500000, 0.0041809082031250, 0.0021362304687500, 0.0044250488281250,
+0.0019531250000000, 0.0043945312500000, 0.0019836425781250, 0.0044555664062500,
+0.0014038085937500, 0.0045471191406250, 0.0014648437500000, 0.0043640136718750,
+0.0011596679687500, 0.0042419433593750, 0.0006408691406250, 0.0044250488281250,
+0.0010375976562500, 0.0039367675781250, 0.0004272460937500, 0.0040893554687500,
+0.0005798339843750, 0.0038757324218750, 0.0006713867187500, 0.0033874511718750,
+0.0003356933593750, 0.0036621093750000, 0.0006713867187500, 0.0030212402343750,
+0.0006103515625000, 0.0030212402343750, 0.0006103515625000, 0.0029602050781250,
+0.0008850097656250, 0.0025939941406250, 0.0010375976562500, 0.0026855468750000,
+0.0009460449218750, 0.0023498535156250, 0.0012817382812500, 0.0023193359375000,
+0.0013732910156250, 0.0021972656250000, 0.0013427734375000, 0.0020141601562500,
+0.0014648437500000, 0.0019836425781250, 0.0014648437500000, 0.0014038085937500,
+0.0012207031250000, 0.0013732910156250, 0.0012817382812500, 0.0009155273437500,
+0.0010986328125000, 0.0002746582031250, 0.0008239746093750, 0.0005493164062500,
+0.0008544921875000, -0.0000610351562500, 0.0006103515625000, -0.0002136230468750,
+0.0004882812500000, -0.0000915527343750, 0.0004882812500000, -0.0004272460937500,
+0.0002746582031250, -0.0004882812500000, 0.0000610351562500, 0.0006713867187500,
+0.0002136230468750, 0.0001220703125000, 0.0001220703125000, 0.0004577636718750,
+0.0000915527343750, 0.0015258789062500, 0.0001525878906250, 0.0008850097656250,
+0.0000610351562500, 0.0021362304687500, 0.0002746582031250, 0.0021057128906250,
+0.0002136230468750, 0.0022888183593750, 0.0003051757812500, 0.0031127929687500,
+0.0005187988281250, 0.0028381347656250, 0.0004272460937500, 0.0035400390625000,
+0.0008544921875000, 0.0036926269531250, 0.0007629394531250, 0.0035400390625000,
+0.0009155273437500, 0.0038146972656250, 0.0013122558593750, 0.0039062500000000,
+0.0010986328125000, 0.0035705566406250, 0.0014953613281250, 0.0037841796875000,
+0.0014648437500000, 0.0034179687500000, 0.0014648437500000, 0.0030517578125000,
+0.0016174316406250, 0.0034179687500000, 0.0014953613281250, 0.0024414062500000,
+0.0018920898437500, 0.0026245117187500, 0.0017089843750000, 0.0023803710937500,
+0.0017395019531250, 0.0017089843750000, 0.0020751953125000, 0.0023193359375000,
+0.0018310546875000, 0.0016479492187500, 0.0021667480468750, 0.0018005371093750,
+0.0020446777343750, 0.0018615722656250, 0.0020446777343750, 0.0015563964843750,
+0.0022583007812500, 0.0018615722656250, 0.0020141601562500, 0.0018310546875000,
+0.0020141601562500, 0.0017700195312500, 0.0019836425781250, 0.0020446777343750,
+0.0019531250000000, 0.0022583007812500, 0.0018920898437500, 0.0020446777343750,
+0.0018005371093750, 0.0026245117187500, 0.0018615722656250, 0.0024414062500000,
+0.0018310546875000, 0.0026245117187500, 0.0018005371093750, 0.0029907226562500,
+0.0017700195312500, 0.0026855468750000, 0.0016784667968750, 0.0030822753906250,
+0.0017089843750000, 0.0029602050781250, 0.0016784667968750, 0.0029602050781250,
+0.0016479492187500, 0.0032348632812500, 0.0017089843750000, 0.0029296875000000,
+0.0016479492187500, 0.0031127929687500, 0.0016784667968750, 0.0031433105468750,
+0.0016784667968750, 0.0029602050781250, 0.0016174316406250, 0.0028991699218750,
+0.0017089843750000, 0.0029907226562500, 0.0015869140625000, 0.0025939941406250,
+0.0017089843750000, 0.0026855468750000, 0.0016174316406250, 0.0026550292968750,
+0.0018005371093750, 0.0023803710937500, 0.0020446777343750, 0.0025024414062500,
+0.0018615722656250, 0.0023803710937500, 0.0022277832031250, 0.0024414062500000,
+0.0022583007812500, 0.0024719238281250, 0.0021362304687500, 0.0024108886718750,
+0.0021362304687500, 0.0025634765625000, 0.0021057128906250, 0.0023803710937500,
+0.0018310546875000, 0.0026245117187500, 0.0018005371093750, 0.0025024414062500,
+0.0017089843750000, 0.0023193359375000, 0.0014648437500000, 0.0025939941406250,
+0.0014648437500000, 0.0022583007812500, 0.0014648437500000, 0.0023498535156250,
+0.0012512207031250, 0.0024719238281250, 0.0014648437500000, 0.0022277832031250,
+0.0016479492187500, 0.0022888183593750, 0.0014648437500000, 0.0022583007812500,
+0.0020446777343750, 0.0022583007812500, 0.0019531250000000, 0.0023193359375000,
+0.0020751953125000, 0.0023498535156250, 0.0025024414062500, 0.0022277832031250,
+0.0023193359375000, 0.0025024414062500, 0.0026245117187500, 0.0024414062500000,
+0.0027770996093750, 0.0023193359375000, 0.0026245117187500, 0.0023193359375000,
+0.0026855468750000, 0.0022277832031250, 0.0027770996093750, 0.0021057128906250,
+0.0022277832031250, 0.0019836425781250, 0.0025329589843750, 0.0019531250000000,
+0.0021667480468750, 0.0018310546875000, 0.0016174316406250, 0.0017395019531250,
+0.0020141601562500, 0.0017089843750000, 0.0009765625000000, 0.0016479492187500,
+0.0010986328125000, 0.0016784667968750, 0.0008850097656250, 0.0016784667968750,
+0.0001525878906250, 0.0015563964843750, 0.0004882812500000, 0.0017089843750000,
+0.0000915527343750, 0.0016784667968750, -0.0000915527343750, 0.0017089843750000,
+0.0002441406250000, 0.0018005371093750, 0.0002746582031250, 0.0017395019531250,
+0.0000915527343750, 0.0018005371093750, 0.0011901855468750, 0.0017700195312500,
+0.0008544921875000, 0.0017700195312500, 0.0013732910156250, 0.0017700195312500,
+0.0023803710937500, 0.0017395019531250, 0.0018005371093750, 0.0018615722656250,
+0.0030212402343750, 0.0016479492187500, 0.0031433105468750, 0.0018920898437500,
+0.0031127929687500, 0.0020446777343750, 0.0036926269531250, 0.0017700195312500,
+0.0036010742187500, 0.0024414062500000, 0.0032348632812500, 0.0022277832031250,
+0.0036010742187500, 0.0024108886718750, 0.0030822753906250, 0.0028991699218750,
+0.0024108886718750, 0.0025329589843750, 0.0028381347656250, 0.0030822753906250,
+0.0018005371093750, 0.0029296875000000, 0.0018005371093750, 0.0029907226562500,
+0.0016479492187500, 0.0033874511718750, 0.0009460449218750, 0.0029907226562500,
+0.0011596679687500, 0.0032348632812500, 0.0009460449218750, 0.0032043457031250,
+0.0006408691406250, 0.0030212402343750, 0.0008544921875000, 0.0030212402343750,
+0.0009765625000000, 0.0028991699218750, 0.0008239746093750, 0.0027770996093750,
+0.0011291503906250, 0.0026245117187500, 0.0010986328125000, 0.0025329589843750,
+0.0010070800781250, 0.0024414062500000, 0.0012207031250000, 0.0022277832031250,
+0.0011901855468750, 0.0022583007812500, 0.0008850097656250, 0.0021057128906250,
+0.0008850097656250, 0.0019531250000000, 0.0007629394531250, 0.0019531250000000,
+0.0004272460937500, 0.0018310546875000, 0.0004882812500000, 0.0015869140625000,
+0.0004882812500000, 0.0016479492187500, 0.0003051757812500, 0.0014648437500000,
+0.0005798339843750, 0.0012207031250000, 0.0007934570312500, 0.0013427734375000,
+0.0005493164062500, 0.0011291503906250, 0.0013427734375000, 0.0010375976562500,
+0.0012512207031250, 0.0010986328125000, 0.0013732910156250, 0.0010986328125000,
+0.0018920898437500, 0.0010070800781250, 0.0017089843750000, 0.0011901855468750,
+0.0019226074218750, 0.0011596679687500, 0.0020751953125000, 0.0011291503906250,
+0.0019226074218750, 0.0011901855468750, 0.0018310546875000, 0.0011596679687500,
+0.0019226074218750, 0.0010070800781250, 0.0015563964843750, 0.0009460449218750,
+0.0014953613281250, 0.0008239746093750, 0.0016174316406250, 0.0006408691406250,
+0.0014953613281250, 0.0007019042968750, 0.0014953613281250, 0.0004272460937500,
+0.0018920898437500, 0.0003662109375000, 0.0018310546875000, 0.0003662109375000,
+0.0019836425781250, 0.0003051757812500, 0.0023803710937500, 0.0001831054687500,
+0.0023193359375000, 0.0003356933593750, 0.0030517578125000, 0.0002746582031250,
+0.0029296875000000, 0.0003967285156250, 0.0032958984375000, 0.0006408691406250,
+0.0039672851562500, 0.0004272460937500, 0.0036010742187500, 0.0006103515625000,
+0.0041809082031250, 0.0007019042968750, 0.0042114257812500, 0.0005187988281250,
+0.0040588378906250, 0.0004577636718750, 0.0041809082031250, 0.0005187988281250,
+0.0041198730468750, 0.0000610351562500, 0.0039062500000000, 0.0000610351562500,
+0.0039978027343750, -0.0000305175781250, 0.0036926269531250, -0.0003662109375000,
+0.0035095214843750, -0.0004272460937500, 0.0037536621093750, -0.0002136230468750,
+0.0025024414062500, -0.0005493164062500, 0.0030212402343750, -0.0001831054687500,
+0.0021972656250000, 0.0002441406250000, 0.0007629394531250, -0.0001525878906250,
+0.0017089843750000, 0.0007934570312500, -0.0003967285156250, 0.0007934570312500,
+-0.0004577636718750, 0.0007324218750000, -0.0003356933593750, 0.0014648437500000,
+-0.0018310546875000, 0.0010681152343750, 0.0038452148437500, 0.0006103515625000,
+0.0014648437500000, 0.0006713867187500, 0.0023498535156250, 0.0007629394531250,
+0.0014953613281250, 0.0006713867187500, 0.0001220703125000, 0.0005493164062500,
+0.0008850097656250, 0.0009155273437500, -0.0004272460937500, 0.0008239746093750,
+-0.0006103515625000, 0.0008544921875000, -0.0002746582031250, 0.0011596679687500,
+-0.0008239746093750},
+{0.0021667480468750, 0.0024414062500000, 0.0024414062500000, 0.0024108886718750,
+0.0024108886718750, 0.0023498535156250, 0.0020751953125000, 0.0023803710937500,
+0.0022888183593750, 0.0022583007812500, 0.0019226074218750, 0.0021057128906250,
+0.0015563964843750, 0.0021362304687500, 0.0018615722656250, 0.0020141601562500,
+0.0007629394531250, 0.0019531250000000, 0.0009155273437500, 0.0019836425781250,
+0.0006408691406250, 0.0020141601562500, -0.0001525878906250, 0.0019226074218750,
+0.0001220703125000, 0.0020446777343750, -0.0003967285156250, 0.0021057128906250,
+-0.0005187988281250, 0.0021362304687500, -0.0003356933593750, 0.0021972656250000,
+-0.0003967285156250, 0.0023193359375000, -0.0005187988281250, 0.0021667480468750,
+-0.0002136230468750, 0.0023498535156250, -0.0002441406250000, 0.0022583007812500,
+-0.0002746582031250, 0.0020446777343750, -0.0001525878906250, 0.0023193359375000,
+-0.0001525878906250, 0.0018920898437500, -0.0005493164062500, 0.0019226074218750,
+-0.0003967285156250, 0.0018920898437500, -0.0006408691406250, 0.0016479492187500,
+-0.0010986328125000, 0.0018005371093750, -0.0008544921875000, 0.0017089843750000,
+-0.0012512207031250, 0.0016479492187500, -0.0013427734375000, 0.0016479492187500,
+-0.0011291503906250, 0.0015869140625000, -0.0011901855468750, 0.0015869140625000,
+-0.0012207031250000, 0.0014038085937500, -0.0007019042968750, 0.0014038085937500,
+-0.0008239746093750, 0.0012817382812500, -0.0004577636718750, 0.0010986328125000,
+0.0001525878906250, 0.0011291503906250, -0.0000305175781250, 0.0008850097656250,
+0.0003662109375000, 0.0008544921875000, 0.0005493164062500, 0.0007019042968750,
+0.0004577636718750, 0.0005187988281250, 0.0005493164062500, 0.0004882812500000,
+0.0007324218750000, 0.0003662109375000, 0.0004882812500000, 0.0002746582031250,
+0.0005187988281250, 0.0002136230468750, 0.0005798339843750, 0.0001220703125000,
+0.0006103515625000, 0.0000305175781250, 0.0007019042968750, -0.0000915527343750,
+0.0008544921875000, -0.0000915527343750, 0.0008850097656250, -0.0003356933593750,
+0.0011596679687500, -0.0004882812500000, 0.0014038085937500, -0.0003967285156250,
+0.0012817382812500, -0.0007324218750000, 0.0018920898437500, -0.0008544921875000,
+0.0018920898437500, -0.0007324218750000, 0.0020751953125000, -0.0008850097656250,
+0.0024719238281250, -0.0009460449218750, 0.0023498535156250, -0.0004577636718750,
+0.0029907226562500, -0.0007629394531250, 0.0029296875000000, -0.0003051757812500,
+0.0031738281250000, 0.0003662109375000, 0.0037231445312500, -0.0000915527343750,
+0.0035400390625000, 0.0009765625000000, 0.0043640136718750, 0.0009765625000000,
+0.0043945312500000, 0.0009460449218750, 0.0044250488281250, 0.0014343261718750,
+0.0050964355468750, 0.0012512207031250, 0.0050659179687500, 0.0011901855468750,
+0.0042724609375000, 0.0013122558593750, 0.0047912597656250, 0.0011901855468750,
+0.0043945312500000, 0.0008544921875000, 0.0025634765625000, 0.0009460449218750,
+0.0033264160156250, 0.0012207031250000, 0.0049743652343750, 0.0011291503906250,
+0.0048522949218750, 0.0012512207031250, 0.0047302246093750, 0.0013732910156250,
+0.0050964355468750, 0.0013732910156250, 0.0051574707031250, 0.0013427734375000,
+0.0041809082031250, 0.0014648437500000, 0.0047607421875000, 0.0014038085937500,
+0.0037841796875000, 0.0012512207031250, 0.0023498535156250, 0.0013732910156250,
+0.0032043457031250, 0.0015258789062500, 0.0008239746093750, 0.0013427734375000,
+0.0008544921875000, 0.0018005371093750, 0.0006713867187500, 0.0021972656250000,
+-0.0008544921875000, 0.0018920898437500, -0.0003967285156250, 0.0028991699218750,
+-0.0004272460937500, 0.0028686523437500, -0.0011596679687500, 0.0028381347656250,
+-0.0002746582031250, 0.0033874511718750, 0.0005798339843750, 0.0032653808593750,
+-0.0003356933593750, 0.0027465820312500, 0.0014648437500000, 0.0030212402343750,
+0.0013732910156250, 0.0025329589843750, 0.0014038085937500, 0.0017700195312500,
+0.0024414062500000, 0.0021972656250000, 0.0019836425781250, 0.0013427734375000,
+0.0021057128906250, 0.0011596679687500, 0.0024108886718750, 0.0013122558593750,
+0.0019836425781250, 0.0009765625000000, 0.0015869140625000, 0.0008850097656250,
+0.0018005371093750, 0.0013122558593750, 0.0010986328125000, 0.0010070800781250,
+0.0010681152343750, 0.0010375976562500, 0.0009155273437500, 0.0013427734375000,
+0.0005798339843750, 0.0010070800781250, 0.0007629394531250, 0.0010070800781250,
+0.0004577636718750, 0.0008850097656250, 0.0001525878906250, 0.0007629394531250,
+0.0005187988281250, 0.0006103515625000, 0.0006713867187500, 0.0004882812500000,
+0.0003967285156250, 0.0008544921875000, 0.0013732910156250, 0.0005187988281250,
+0.0009460449218750, 0.0008544921875000, 0.0012817382812500, 0.0013732910156250,
+0.0022277832031250, 0.0008544921875000, 0.0016174316406250, 0.0019836425781250,
+0.0025024414062500, 0.0019836425781250, 0.0026245117187500, 0.0018615722656250,
+0.0023498535156250, 0.0024719238281250, 0.0025024414062500, 0.0023498535156250,
+0.0026245117187500, 0.0018920898437500, 0.0024414062500000, 0.0022277832031250,
+0.0024719238281250, 0.0015869140625000, 0.0024719238281250, 0.0008850097656250,
+0.0022583007812500, 0.0014038085937500, 0.0022277832031250, -0.0001831054687500,
+0.0028076171875000, -0.0000915527343750, 0.0025024414062500, -0.0003662109375000,
+0.0027770996093750, -0.0014038085937500, 0.0034179687500000, -0.0010070800781250,
+0.0029602050781250, -0.0013427734375000, 0.0035705566406250, -0.0017089843750000,
+0.0037536621093750, -0.0012207031250000, 0.0036621093750000, -0.0010070800781250,
+0.0037536621093750, -0.0014343261718750, 0.0037841796875000, -0.0002441406250000,
+0.0036621093750000, -0.0003662109375000, 0.0036926269531250, -0.0002136230468750,
+0.0037536621093750, 0.0006103515625000, 0.0036926269531250, 0.0003662109375000,
+0.0035400390625000, 0.0007324218750000, 0.0036010742187500, 0.0008544921875000,
+0.0035095214843750, 0.0007629394531250, 0.0035400390625000, 0.0008850097656250,
+0.0036010742187500, 0.0008850097656250, 0.0035400390625000, 0.0008239746093750,
+0.0033569335937500, 0.0008850097656250, 0.0035400390625000, 0.0009460449218750,
+0.0032653808593750, 0.0009460449218750, 0.0029296875000000, 0.0009765625000000,
+0.0031738281250000, 0.0009765625000000, 0.0025939941406250, 0.0010375976562500,
+0.0027770996093750, 0.0010681152343750, 0.0026550292968750, 0.0010070800781250,
+0.0022277832031250, 0.0011291503906250, 0.0026245117187500, 0.0011596679687500,
+0.0022277832031250, 0.0011901855468750, 0.0022888183593750, 0.0012817382812500,
+0.0023193359375000, 0.0013122558593750, 0.0021057128906250, 0.0013427734375000,
+0.0022888183593750, 0.0013122558593750, 0.0019836425781250, 0.0013427734375000,
+0.0020141601562500, 0.0013732910156250, 0.0019531250000000, 0.0013732910156250,
+0.0017395019531250, 0.0013427734375000, 0.0018005371093750, 0.0015869140625000,
+0.0016784667968750, 0.0014343261718750, 0.0016784667968750, 0.0015563964843750,
+0.0016479492187500, 0.0017700195312500, 0.0015563964843750, 0.0015563964843750,
+0.0016784667968750, 0.0019226074218750, 0.0016784667968750, 0.0018005371093750,
+0.0015869140625000, 0.0017395019531250, 0.0018310546875000, 0.0019531250000000,
+0.0019531250000000, 0.0018005371093750, 0.0018920898437500, 0.0018005371093750,
+0.0023193359375000, 0.0018005371093750, 0.0022277832031250, 0.0017395019531250,
+0.0023193359375000, 0.0016479492187500, 0.0025939941406250, 0.0017395019531250,
+0.0024719238281250, 0.0016174316406250, 0.0026245117187500, 0.0016479492187500,
+0.0026855468750000, 0.0017089843750000, 0.0026550292968750, 0.0016479492187500,
+0.0026550292968750, 0.0017089843750000, 0.0027160644531250, 0.0017089843750000,
+0.0025329589843750, 0.0017089843750000, 0.0026550292968750, 0.0016784667968750,
+0.0025634765625000, 0.0016479492187500, 0.0023193359375000, 0.0016479492187500,
+0.0024719238281250, 0.0014648437500000, 0.0020751953125000, 0.0013732910156250,
+0.0021362304687500, 0.0012817382812500, 0.0021057128906250, 0.0010681152343750,
+0.0018310546875000, 0.0010375976562500, 0.0019226074218750, 0.0010681152343750,
+0.0019531250000000, 0.0008544921875000, 0.0018920898437500, 0.0009765625000000,
+0.0019531250000000, 0.0011596679687500, 0.0020751953125000, 0.0009155273437500,
+0.0019531250000000, 0.0013732910156250, 0.0023193359375000, 0.0013427734375000,
+0.0022583007812500, 0.0011901855468750, 0.0022583007812500, 0.0012817382812500,
+0.0024719238281250, 0.0012817382812500, 0.0023498535156250, 0.0007629394531250,
+0.0023803710937500, 0.0009155273437500, 0.0025024414062500, 0.0007019042968750,
+0.0022888183593750, 0.0001831054687500, 0.0021057128906250, 0.0004272460937500,
+0.0022583007812500, 0.0000000000000000, 0.0017395019531250, -0.0000915527343750,
+0.0018005371093750, 0.0000000000000000, 0.0017395019531250, -0.0000915527343750,
+0.0013732910156250, -0.0000915527343750, 0.0015869140625000, 0.0000000000000000,
+0.0012512207031250, 0.0000000000000000, 0.0013427734375000, 0.0000000000000000,
+0.0012512207031250, -0.0000305175781250, 0.0010070800781250, 0.0000305175781250,
+0.0012207031250000, 0.0001220703125000, 0.0008850097656250, 0.0000305175781250,
+0.0009765625000000, 0.0000915527343750, 0.0010070800781250, 0.0002441406250000,
+0.0008239746093750, 0.0002441406250000, 0.0009765625000000, 0.0003356933593750,
+0.0011596679687500, 0.0003051757812500, 0.0010375976562500, 0.0003662109375000,
+0.0011291503906250, 0.0003967285156250, 0.0013427734375000, 0.0003356933593750,
+0.0012207031250000, 0.0003051757812500, 0.0011901855468750, 0.0003967285156250,
+0.0012817382812500, 0.0001831054687500, 0.0010986328125000, -0.0000305175781250,
+0.0008850097656250, 0.0001525878906250, 0.0008850097656250, -0.0004577636718750,
+0.0006103515625000, -0.0003967285156250, 0.0005493164062500, -0.0004882812500000,
+0.0004577636718750, -0.0009460449218750, 0.0002441406250000, -0.0007324218750000,
+0.0002136230468750, -0.0006713867187500, 0.0001525878906250, -0.0009460449218750,
+0.0000610351562500, -0.0004577636718750, 0.0002136230468750, -0.0000305175781250,
+0.0003051757812500, -0.0005187988281250, 0.0001831054687500, 0.0008850097656250,
+0.0006103515625000, 0.0007019042968750, 0.0005493164062500, 0.0007324218750000,
+0.0006103515625000, 0.0020446777343750, 0.0010070800781250, 0.0025634765625000,
+0.0009460449218750, 0.0028991699218750, 0.0011901855468750, 0.0029907226562500,
+0.0011901855468750, 0.0029296875000000, 0.0010986328125000, 0.0031433105468750,
+0.0012512207031250, 0.0032348632812500, 0.0012207031250000, 0.0027465820312500,
+0.0012512207031250, 0.0030212402343750, 0.0011901855468750, 0.0025329589843750,
+0.0012207031250000, 0.0018615722656250, 0.0011291503906250, 0.0023193359375000,
+0.0010375976562500, 0.0012207031250000, 0.0011901855468750, 0.0010375976562500,
+0.0009765625000000, 0.0011901855468750, 0.0010375976562500, 0.0007934570312500,
+0.0012207031250000, 0.0006408691406250, 0.0008850097656250, 0.0011901855468750,
+0.0011291503906250, 0.0010375976562500, 0.0010681152343750, 0.0012512207031250,
+0.0010070800781250, 0.0017089843750000, 0.0010681152343750, 0.0013732910156250,
+0.0008850097656250, 0.0021667480468750, 0.0009155273437500, 0.0020446777343750,
+0.0007934570312500, 0.0020446777343750, 0.0007324218750000, 0.0025024414062500,
+0.0007629394531250, 0.0022583007812500, 0.0006408691406250, 0.0024414062500000,
+0.0007019042968750},
+{0.0021667480468750, 0.0020446777343750, 0.0024414062500000, 0.0020446777343750,
+0.0024414062500000, 0.0018005371093750, 0.0023193359375000, 0.0013732910156250,
+0.0023803710937500, 0.0015258789062500, 0.0023498535156250, 0.0014343261718750,
+0.0021362304687500, 0.0012207031250000, 0.0022277832031250, 0.0014343261718750,
+0.0019836425781250, 0.0015563964843750, 0.0017395019531250, 0.0014038085937500,
+0.0019531250000000, 0.0019531250000000, 0.0013732910156250, 0.0019531250000000,
+0.0014648437500000, 0.0019531250000000, 0.0014038085937500, 0.0022277832031250,
+0.0009765625000000, 0.0021362304687500, 0.0011596679687500, 0.0019226074218750,
+0.0009765625000000, 0.0020141601562500, 0.0008850097656250, 0.0019226074218750,
+0.0010070800781250, 0.0016784667968750, 0.0009765625000000, 0.0017700195312500,
+0.0008544921875000, 0.0016174316406250, 0.0011596679687500, 0.0014648437500000,
+0.0010375976562500, 0.0015258789062500, 0.0011596679687500, 0.0015563964843750,
+0.0014038085937500, 0.0014343261718750, 0.0012512207031250, 0.0014648437500000,
+0.0014648437500000, 0.0014038085937500, 0.0014953613281250, 0.0013122558593750,
+0.0013427734375000, 0.0012817382812500, 0.0014038085937500, 0.0012207031250000,
+0.0014343261718750, 0.0010681152343750, 0.0011596679687500, 0.0010375976562500,
+0.0013122558593750, 0.0009155273437500, 0.0010986328125000, 0.0007629394531250,
+0.0007324218750000, 0.0007934570312500, 0.0010070800781250, 0.0005798339843750,
+0.0004882812500000, 0.0005187988281250, 0.0005798339843750, 0.0003967285156250,
+0.0005798339843750, 0.0002441406250000, 0.0002746582031250, 0.0001831054687500,
+0.0004577636718750, -0.0002136230468750, 0.0003662109375000, -0.0001525878906250,
+0.0002746582031250, -0.0004882812500000, 0.0006103515625000, -0.0008544921875000,
+0.0006713867187500, -0.0006103515625000, 0.0004882812500000, -0.0014648437500000,
+0.0012512207031250, -0.0014343261718750, 0.0010681152343750, -0.0016479492187500,
+0.0013122558593750, -0.0022277832031250, 0.0018920898437500, -0.0020446777343750,
+0.0015258789062500, -0.0021362304687500, 0.0019836425781250, -0.0023193359375000,
+0.0020141601562500, -0.0019226074218750, 0.0017700195312500, -0.0014648437500000,
+0.0018005371093750, -0.0011596679687500, 0.0017700195312500, -0.0000305175781250,
+0.0014038085937500, -0.0005187988281250, 0.0014953613281250, 0.0000000000000000,
+0.0014343261718750, 0.0011901855468750, 0.0011901855468750, 0.0005798339843750,
+0.0012817382812500, 0.0024719238281250, 0.0011596679687500, 0.0024719238281250,
+0.0011901855468750, 0.0027770996093750, 0.0010986328125000, 0.0040588378906250,
+0.0010375976562500, 0.0035400390625000, 0.0011596679687500, 0.0038452148437500,
+0.0007324218750000, 0.0041809082031250, 0.0009765625000000, 0.0036315917968750,
+0.0007019042968750, 0.0032043457031250, 0.0002136230468750, 0.0033569335937500,
+0.0005493164062500, 0.0023193359375000, -0.0000610351562500, 0.0022583007812500,
+-0.0000305175781250, 0.0020446777343750, 0.0000305175781250, 0.0013427734375000,
+-0.0002746582031250, 0.0015869140625000, -0.0002441406250000, 0.0013732910156250,
+-0.0000305175781250, 0.0010986328125000, -0.0001831054687500, 0.0013427734375000,
+0.0000305175781250, 0.0016479492187500, 0.0002746582031250, 0.0014648437500000,
+0.0000915527343750, 0.0021362304687500, 0.0007019042968750, 0.0021362304687500,
+0.0005798339843750, 0.0024414062500000, 0.0007019042968750, 0.0031127929687500,
+0.0011596679687500, 0.0029602050781250, 0.0009155273437500, 0.0036315917968750,
+0.0014038085937500, 0.0036315917968750, 0.0013122558593750, 0.0037536621093750,
+0.0014343261718750, 0.0043029785156250, 0.0018310546875000, 0.0041198730468750,
+0.0015563964843750, 0.0041198730468750, 0.0021362304687500, 0.0041809082031250,
+0.0020446777343750, 0.0039367675781250, 0.0020446777343750, 0.0037536621093750,
+0.0022888183593750, 0.0037536621093750, 0.0021972656250000, 0.0031433105468750,
+0.0021972656250000, 0.0032348632812500, 0.0022888183593750, 0.0029602050781250,
+0.0020446777343750, 0.0023803710937500, 0.0018920898437500, 0.0026855468750000,
+0.0020141601562500, 0.0020446777343750, 0.0013427734375000, 0.0020751953125000,
+0.0014343261718750, 0.0021057128906250, 0.0012512207031250, 0.0018005371093750,
+0.0007629394531250, 0.0019836425781250, 0.0008850097656250, 0.0019836425781250,
+0.0007019042968750, 0.0019531250000000, 0.0004882812500000, 0.0021972656250000,
+0.0008239746093750, 0.0023803710937500, 0.0010375976562500, 0.0024108886718750,
+0.0007934570312500, 0.0029296875000000, 0.0014038085937500, 0.0029296875000000,
+0.0014343261718750, 0.0031127929687500, 0.0014648437500000, 0.0035095214843750,
+0.0018005371093750, 0.0034179687500000, 0.0016784667968750, 0.0035095214843750,
+0.0019226074218750, 0.0036315917968750, 0.0018615722656250, 0.0033874511718750,
+0.0019226074218750, 0.0031738281250000, 0.0021667480468750, 0.0033569335937500,
+0.0019836425781250, 0.0027770996093750, 0.0021667480468750, 0.0028381347656250,
+0.0022888183593750, 0.0026855468750000, 0.0020751953125000, 0.0022583007812500,
+0.0020446777343750, 0.0024719238281250, 0.0021667480468750, 0.0022583007812500,
+0.0016174316406250, 0.0022583007812500, 0.0018005371093750, 0.0022583007812500,
+0.0016784667968750, 0.0021972656250000, 0.0012817382812500, 0.0022583007812500,
+0.0015258789062500, 0.0022888183593750, 0.0013122558593750, 0.0023498535156250,
+0.0012512207031250, 0.0023803710937500, 0.0014038085937500, 0.0024414062500000,
+0.0014648437500000, 0.0025024414062500, 0.0014343261718750, 0.0025634765625000,
+0.0018920898437500, 0.0025329589843750, 0.0017395019531250, 0.0026550292968750,
+0.0020141601562500, 0.0027770996093750, 0.0025329589843750, 0.0027770996093750,
+0.0022888183593750, 0.0028991699218750, 0.0028381347656250, 0.0029907226562500,
+0.0028381347656250, 0.0030212402343750, 0.0028686523437500, 0.0030517578125000,
+0.0032043457031250, 0.0031738281250000, 0.0031127929687500, 0.0030517578125000,
+0.0031433105468750, 0.0032958984375000, 0.0032653808593750, 0.0032653808593750,
+0.0030822753906250, 0.0031738281250000, 0.0028686523437500, 0.0034790039062500,
+0.0030822753906250, 0.0030517578125000, 0.0025024414062500, 0.0032958984375000,
+0.0026245117187500, 0.0031127929687500, 0.0024108886718750, 0.0027160644531250,
+0.0018920898437500, 0.0031433105468750, 0.0021057128906250, 0.0025024414062500,
+0.0013427734375000, 0.0025939941406250, 0.0013732910156250, 0.0026245117187500,
+0.0012512207031250, 0.0022277832031250, 0.0007324218750000, 0.0025024414062500,
+0.0009460449218750, 0.0021667480468750, 0.0006713867187500, 0.0022583007812500,
+0.0006408691406250, 0.0022888183593750, 0.0007019042968750, 0.0021362304687500,
+0.0007019042968750, 0.0024108886718750, 0.0007934570312500, 0.0021667480468750,
+0.0008544921875000, 0.0022888183593750, 0.0008850097656250, 0.0023193359375000,
+0.0009155273437500, 0.0021362304687500, 0.0009765625000000, 0.0023498535156250,
+0.0010681152343750, 0.0020446777343750, 0.0009460449218750, 0.0021972656250000,
+0.0010070800781250, 0.0021972656250000, 0.0011291503906250, 0.0019836425781250,
+0.0010986328125000, 0.0022277832031250, 0.0011596679687500, 0.0019531250000000,
+0.0014953613281250, 0.0019531250000000, 0.0014343261718750, 0.0019836425781250,
+0.0015563964843750, 0.0018005371093750, 0.0018920898437500, 0.0018920898437500,
+0.0018005371093750, 0.0018615722656250, 0.0017700195312500, 0.0017700195312500,
+0.0019226074218750, 0.0018310546875000, 0.0015869140625000, 0.0017700195312500,
+0.0012512207031250, 0.0016784667968750, 0.0014038085937500, 0.0017395019531250,
+0.0007019042968750, 0.0016784667968750, 0.0006408691406250, 0.0016174316406250,
+0.0005798339843750, 0.0016784667968750, 0.0001525878906250, 0.0014953613281250,
+0.0002441406250000, 0.0016479492187500, 0.0003356933593750, 0.0015258789062500,
+0.0001831054687500, 0.0015563964843750, 0.0003356933593750, 0.0017089843750000,
+0.0006408691406250, 0.0014648437500000, 0.0005187988281250, 0.0016784667968750,
+0.0008544921875000, 0.0015869140625000, 0.0009155273437500, 0.0015563964843750,
+0.0009155273437500, 0.0015563964843750, 0.0010070800781250, 0.0014953613281250,
+0.0010070800781250, 0.0012512207031250, 0.0009765625000000, 0.0014343261718750,
+0.0010681152343750, 0.0012207031250000, 0.0010986328125000, 0.0009765625000000,
+0.0010070800781250, 0.0012512207031250, 0.0010375976562500, 0.0008544921875000,
+0.0012512207031250, 0.0007934570312500, 0.0011901855468750, 0.0009155273437500,
+0.0013427734375000, 0.0008239746093750, 0.0016479492187500, 0.0009155273437500,
+0.0015258789062500, 0.0010681152343750, 0.0020141601562500, 0.0011291503906250,
+0.0019531250000000, 0.0013122558593750, 0.0021057128906250, 0.0014953613281250,
+0.0024719238281250, 0.0014343261718750, 0.0023193359375000, 0.0014343261718750,
+0.0025634765625000, 0.0016479492187500, 0.0025939941406250, 0.0014038085937500,
+0.0024719238281250, 0.0011901855468750, 0.0025634765625000, 0.0014648437500000,
+0.0025634765625000, 0.0008850097656250, 0.0022583007812500, 0.0009765625000000,
+0.0022888183593750, 0.0010070800781250, 0.0021667480468750, 0.0006408691406250,
+0.0019226074218750, 0.0007629394531250, 0.0019836425781250, 0.0009155273437500,
+0.0016784667968750, 0.0007629394531250, 0.0016784667968750, 0.0008850097656250,
+0.0015258789062500, 0.0010375976562500, 0.0013122558593750, 0.0008544921875000,
+0.0014038085937500, 0.0011291503906250, 0.0009765625000000, 0.0010986328125000,
+0.0009765625000000, 0.0010986328125000, 0.0008544921875000, 0.0012817382812500,
+0.0002746582031250, 0.0015869140625000, 0.0031738281250000, 0.0021667480468750,
+0.0022888183593750, 0.0016784667968750, 0.0027770996093750, 0.0019836425781250,
+0.0021972656250000, 0.0024719238281250, 0.0013427734375000, 0.0021057128906250,
+0.0016784667968750, 0.0024414062500000, 0.0012207031250000, 0.0025634765625000,
+0.0009765625000000, 0.0024108886718750, 0.0013122558593750, 0.0024719238281250,
+0.0014038085937500, 0.0024719238281250, 0.0012817382812500, 0.0022277832031250,
+0.0018920898437500, 0.0021972656250000, 0.0017089843750000, 0.0022277832031250,
+0.0020751953125000, 0.0020751953125000, 0.0025939941406250, 0.0020446777343750,
+0.0021667480468750, 0.0024414062500000, 0.0031433105468750, 0.0022888183593750,
+0.0030822753906250, 0.0025024414062500, 0.0028076171875000, 0.0028991699218750,
+0.0032348632812500, 0.0026245117187500, 0.0032348632812500, 0.0028381347656250,
+0.0027160644531250, 0.0029907226562500, 0.0028686523437500, 0.0027770996093750,
+0.0027160644531250, 0.0025939941406250, 0.0022277832031250, 0.0027160644531250,
+0.0025024414062500, 0.0020751953125000, 0.0019226074218750, 0.0021057128906250,
+0.0020751953125000, 0.0020141601562500, 0.0020751953125000, 0.0015563964843750,
+0.0017089843750000, 0.0016784667968750, 0.0019836425781250, 0.0016479492187500,
+0.0020141601562500, 0.0014953613281250, 0.0018920898437500, 0.0015563964843750,
+0.0022277832031250, 0.0016784667968750, 0.0024108886718750, 0.0016174316406250,
+0.0022583007812500, 0.0014648437500000, 0.0031433105468750, 0.0015563964843750,
+0.0030517578125000, 0.0013122558593750, 0.0029602050781250, 0.0010681152343750,
+0.0033874511718750, 0.0012207031250000, 0.0032653808593750, 0.0003662109375000,
+0.0031433105468750, 0.0004577636718750, 0.0031738281250000, 0.0000915527343750,
+0.0029907226562500},
+{0.0009155273437500, 0.0014038085937500, 0.0010986328125000, 0.0010681152343750,
+0.0011291503906250, 0.0012512207031250, 0.0010986328125000, 0.0010986328125000,
+0.0017089843750000, 0.0009765625000000, 0.0015869140625000, 0.0011901855468750,
+0.0017700195312500, 0.0013122558593750, 0.0021972656250000, 0.0011596679687500,
+0.0019531250000000, 0.0017395019531250, 0.0023498535156250, 0.0016784667968750,
+0.0022277832031250, 0.0017395019531250, 0.0023193359375000, 0.0021362304687500,
+0.0024719238281250, 0.0020141601562500, 0.0021972656250000, 0.0019836425781250,
+0.0027465820312500, 0.0021667480468750, 0.0025024414062500, 0.0019836425781250,
+0.0026550292968750, 0.0016784667968750, 0.0031433105468750, 0.0018615722656250,
+0.0028381347656250, 0.0015258789062500, 0.0032043457031250, 0.0014038085937500,
+0.0032653808593750, 0.0015258789062500, 0.0032653808593750, 0.0014038085937500,
+0.0035095214843750, 0.0012817382812500, 0.0034484863281250, 0.0017089843750000,
+0.0034484863281250, 0.0014648437500000, 0.0036010742187500, 0.0016479492187500,
+0.0033569335937500, 0.0021057128906250, 0.0031433105468750, 0.0017089843750000,
+0.0032958984375000, 0.0020751953125000, 0.0026855468750000, 0.0020751953125000,
+0.0026855468750000, 0.0019531250000000, 0.0028686523437500, 0.0020446777343750,
+0.0026855468750000, 0.0019226074218750, 0.0026245117187500, 0.0019226074218750,
+0.0030822753906250, 0.0018310546875000, 0.0029907226562500, 0.0018005371093750,
+0.0029602050781250, 0.0018920898437500, 0.0032958984375000, 0.0026245117187500,
+0.0021362304687500, 0.0029907226562500, 0.0028991699218750, 0.0028076171875000,
+0.0026550292968750, 0.0026855468750000, 0.0026855468750000, 0.0026245117187500,
+0.0029907226562500, 0.0025329589843750, 0.0028991699218750, 0.0021972656250000,
+0.0027160644531250, 0.0020141601562500, 0.0027770996093750, 0.0020141601562500,
+0.0025939941406250, 0.0018310546875000, 0.0022888183593750, 0.0014953613281250,
+0.0023803710937500, 0.0021057128906250, 0.0020446777343750, 0.0016479492187500,
+0.0021057128906250, 0.0018310546875000, 0.0019836425781250, 0.0024414062500000,
+0.0015869140625000, 0.0019226074218750, 0.0016174316406250, 0.0022277832031250,
+0.0015869140625000, 0.0022277832031250, 0.0012512207031250, 0.0019226074218750,
+0.0014648437500000, 0.0018005371093750, 0.0017395019531250, 0.0018615722656250,
+0.0013427734375000, 0.0015563964843750, 0.0018615722656250, 0.0014343261718750,
+0.0018310546875000, 0.0015258789062500, 0.0018005371093750, 0.0014648437500000,
+0.0021057128906250, 0.0013427734375000, 0.0021057128906250, 0.0017089843750000,
+0.0019836425781250, 0.0014953613281250, 0.0022583007812500, 0.0015869140625000,
+0.0020446777343750, 0.0019531250000000, 0.0017700195312500, 0.0016479492187500,
+0.0020751953125000, 0.0017089843750000, 0.0011901855468750, 0.0018005371093750,
+0.0012512207031250, 0.0014038085937500, 0.0010681152343750, 0.0010986328125000,
+0.0004272460937500, 0.0012817382812500, 0.0005798339843750, 0.0004272460937500,
+0.0004272460937500, 0.0004577636718750, 0.0000000000000000, 0.0002746582031250,
+0.0004272460937500, -0.0003356933593750, 0.0007934570312500, -0.0001220703125000,
+0.0003967285156250, -0.0003662109375000, 0.0010375976562500, -0.0005493164062500,
+0.0010681152343750, -0.0003967285156250, 0.0010070800781250, -0.0003967285156250,
+0.0014343261718750, -0.0005187988281250, 0.0014953613281250, -0.0000915527343750,
+0.0012512207031250, -0.0003051757812500, 0.0014648437500000, -0.0001220703125000,
+0.0011596679687500, 0.0003356933593750, 0.0007324218750000, 0.0000610351562500,
+0.0010986328125000, 0.0004882812500000, 0.0005493164062500, 0.0005187988281250,
+0.0005798339843750, 0.0004577636718750, 0.0006408691406250, 0.0006103515625000,
+0.0002746582031250, 0.0007019042968750, 0.0003967285156250, 0.0006408691406250,
+0.0008239746093750, 0.0006103515625000, 0.0005187988281250, 0.0007019042968750,
+0.0009460449218750, 0.0006713867187500, 0.0014648437500000, 0.0006103515625000,
+0.0010681152343750, 0.0007629394531250, 0.0020751953125000, 0.0006713867187500,
+0.0018920898437500, 0.0007019042968750, 0.0020141601562500, 0.0008544921875000,
+0.0025939941406250, 0.0006103515625000, 0.0020751953125000, 0.0008544921875000,
+0.0026550292968750, 0.0008239746093750, 0.0025634765625000, 0.0007629394531250,
+0.0025939941406250, 0.0008544921875000, 0.0028686523437500, 0.0007934570312500,
+0.0026550292968750, 0.0006408691406250, 0.0030517578125000, 0.0006713867187500,
+0.0029602050781250, 0.0005493164062500, 0.0030822753906250, 0.0004272460937500,
+0.0033569335937500, 0.0004882812500000, 0.0031127929687500, 0.0004577636718750,
+0.0035095214843750, 0.0003356933593750, 0.0034790039062500, 0.0005798339843750,
+0.0034790039062500, 0.0006713867187500, 0.0036315917968750, 0.0005187988281250,
+0.0036315917968750, 0.0011291503906250, 0.0036621093750000, 0.0009460449218750,
+0.0036926269531250, 0.0012207031250000, 0.0037536621093750, 0.0017395019531250,
+0.0037536621093750, 0.0014343261718750, 0.0038146972656250, 0.0023193359375000,
+0.0039978027343750, 0.0021972656250000, 0.0039367675781250, 0.0024108886718750,
+0.0039978027343750, 0.0031127929687500, 0.0042114257812500, 0.0027465820312500,
+0.0041198730468750, 0.0032653808593750, 0.0042724609375000, 0.0032348632812500,
+0.0042419433593750, 0.0030517578125000, 0.0042724609375000, 0.0031738281250000,
+0.0043334960937500, 0.0030822753906250, 0.0042419433593750, 0.0026550292968750,
+0.0043945312500000, 0.0028076171875000, 0.0043640136718750, 0.0024414062500000,
+0.0043334960937500, 0.0019531250000000, 0.0044250488281250, 0.0023193359375000,
+0.0043945312500000, 0.0013427734375000, 0.0043640136718750, 0.0015563964843750,
+0.0042724609375000, 0.0014038085937500, 0.0043029785156250, 0.0007019042968750,
+0.0043029785156250, 0.0011901855468750, 0.0040893554687500, 0.0006103515625000,
+0.0043029785156250, 0.0005798339843750, 0.0041809082031250, 0.0008239746093750,
+0.0041503906250000, 0.0006713867187500, 0.0042419433593750, 0.0007019042968750,
+0.0040893554687500, 0.0010986328125000, 0.0041198730468750, 0.0009460449218750,
+0.0041198730468750, 0.0013122558593750, 0.0040283203125000, 0.0017089843750000,
+0.0039672851562500, 0.0014038085937500, 0.0039978027343750, 0.0022888183593750,
+0.0039367675781250, 0.0021362304687500, 0.0039367675781250, 0.0023498535156250,
+0.0038757324218750, 0.0029602050781250, 0.0038452148437500, 0.0025634765625000,
+0.0038452148437500, 0.0032348632812500, 0.0036621093750000, 0.0031738281250000,
+0.0036926269531250, 0.0032958984375000, 0.0034179687500000, 0.0037231445312500,
+0.0031127929687500, 0.0035705566406250, 0.0032348632812500, 0.0039978027343750,
+0.0025939941406250, 0.0039978027343750, 0.0026245117187500, 0.0040893554687500,
+0.0024108886718750, 0.0043945312500000, 0.0019226074218750, 0.0042724609375000,
+0.0020751953125000, 0.0043334960937500, 0.0016479492187500, 0.0045471191406250,
+0.0015258789062500, 0.0042419433593750, 0.0014953613281250, 0.0040588378906250,
+0.0014038085937500, 0.0043029785156250, 0.0013122558593750, 0.0036315917968750,
+0.0016174316406250, 0.0037536621093750, 0.0014038085937500, 0.0037231445312500,
+0.0016174316406250, 0.0032348632812500, 0.0019226074218750, 0.0034484863281250,
+0.0016784667968750, 0.0032958984375000, 0.0022888183593750, 0.0031738281250000,
+0.0021362304687500, 0.0032653808593750, 0.0021972656250000, 0.0033569335937500,
+0.0026550292968750, 0.0033569335937500, 0.0023498535156250, 0.0034790039062500,
+0.0024414062500000, 0.0035705566406250, 0.0025329589843750, 0.0034484863281250,
+0.0023193359375000, 0.0034179687500000, 0.0022277832031250, 0.0035705566406250,
+0.0022277832031250, 0.0031738281250000, 0.0018310546875000, 0.0032653808593750,
+0.0018920898437500, 0.0032043457031250, 0.0017089843750000, 0.0028991699218750,
+0.0014038085937500, 0.0030517578125000, 0.0014953613281250, 0.0033264160156250,
+0.0011901855468750, 0.0031738281250000, 0.0011901855468750, 0.0035705566406250,
+0.0011901855468750, 0.0039978027343750, 0.0009765625000000, 0.0037231445312500,
+0.0010375976562500, 0.0046691894531250, 0.0011596679687500, 0.0045776367187500,
+0.0009765625000000, 0.0047912597656250, 0.0011291503906250, 0.0054321289062500,
+0.0014038085937500, 0.0051879882812500, 0.0010986328125000, 0.0057678222656250,
+0.0014953613281250, 0.0057373046875000, 0.0014038085937500, 0.0057678222656250,
+0.0013122558593750, 0.0061340332031250, 0.0014038085937500, 0.0058593750000000,
+0.0014343261718750, 0.0058593750000000, 0.0013122558593750, 0.0059509277343750,
+0.0012207031250000, 0.0055236816406250, 0.0011901855468750, 0.0051879882812500,
+0.0011596679687500, 0.0054321289062500, 0.0011291503906250, 0.0044555664062500,
+0.0012817382812500, 0.0045471191406250, 0.0012207031250000, 0.0043029785156250,
+0.0012207031250000, 0.0032348632812500, 0.0013427734375000, 0.0029296875000000,
+0.0013732910156250, 0.0024414062500000, 0.0011901855468750, 0.0025939941406250,
+0.0013427734375000, 0.0024108886718750, 0.0012207031250000, 0.0019531250000000,
+0.0009460449218750, 0.0022583007812500, 0.0011901855468750, 0.0012817382812500,
+0.0010375976562500, 0.0013122558593750, 0.0009460449218750, 0.0010986328125000,
+0.0010986328125000, 0.0004882812500000, 0.0010681152343750, 0.0006713867187500,
+0.0010375976562500, 0.0002441406250000, 0.0014648437500000, 0.0002441406250000,
+0.0013122558593750, 0.0000610351562500, 0.0015258789062500, -0.0002746582031250,
+0.0019836425781250, -0.0000305175781250, 0.0016479492187500, -0.0003051757812500,
+0.0020446777343750, -0.0004272460937500, 0.0020446777343750, -0.0000915527343750,
+0.0019531250000000, 0.0000915527343750, 0.0019836425781250, -0.0001831054687500,
+0.0018615722656250, 0.0007324218750000, 0.0018005371093750, 0.0006103515625000,
+0.0017089843750000, 0.0005798339843750, 0.0017700195312500, 0.0010681152343750,
+0.0018005371093750, 0.0008544921875000, 0.0016784667968750, 0.0007934570312500,
+0.0018310546875000, 0.0007629394531250, 0.0018005371093750, 0.0005798339843750,
+0.0017395019531250, 0.0003662109375000, 0.0018920898437500, 0.0003356933593750,
+0.0017395019531250, -0.0000305175781250, 0.0016784667968750, -0.0000915527343750,
+0.0018310546875000, -0.0000915527343750, 0.0016174316406250, -0.0003356933593750,
+0.0014038085937500, -0.0003356933593750, 0.0014953613281250, -0.0001220703125000,
+0.0011596679687500, -0.0003051757812500, 0.0010986328125000, -0.0000305175781250,
+0.0010375976562500, 0.0003356933593750, 0.0007324218750000, 0.0000305175781250,
+0.0007934570312500, 0.0008544921875000, 0.0008239746093750, 0.0006713867187500,
+0.0006103515625000, 0.0007934570312500, 0.0007934570312500, 0.0012817382812500,
+0.0008544921875000, 0.0009155273437500, 0.0006408691406250, 0.0014038085937500,
+0.0011291503906250, 0.0012817382812500, 0.0010681152343750, 0.0013732910156250,
+0.0012207031250000, 0.0016479492187500, 0.0016784667968750, 0.0013122558593750,
+0.0014038085937500, 0.0018310546875000, 0.0016479492187500, 0.0016174316406250,
+0.0017395019531250, 0.0018005371093750, 0.0017089843750000, 0.0021972656250000,
+0.0017395019531250, 0.0018920898437500, 0.0017395019531250, 0.0024414062500000,
+0.0018005371093750, 0.0022888183593750, 0.0018005371093750, 0.0024108886718750,
+0.0017089843750000, 0.0027160644531250, 0.0016784667968750, 0.0024108886718750,
+0.0017089843750000},
+{0.0025634765625000, 0.0032043457031250, 0.0024414062500000, 0.0031433105468750,
+0.0024719238281250, 0.0030517578125000, 0.0026245117187500, 0.0032653808593750,
+0.0024414062500000, 0.0034179687500000, 0.0021972656250000, 0.0032653808593750,
+0.0024414062500000, 0.0036315917968750, 0.0018310546875000, 0.0036926269531250,
+0.0018615722656250, 0.0038146972656250, 0.0017700195312500, 0.0040283203125000,
+0.0013122558593750, 0.0040283203125000, 0.0015258789062500, 0.0044860839843750,
+0.0011901855468750, 0.0044860839843750, 0.0010986328125000, 0.0046386718750000,
+0.0010681152343750, 0.0049133300781250, 0.0009155273437500, 0.0047912597656250,
+0.0010070800781250, 0.0047607421875000, 0.0008850097656250, 0.0050048828125000,
+0.0008544921875000, 0.0045776367187500, 0.0007934570312500, 0.0041198730468750,
+0.0006713867187500, 0.0041809082031250, 0.0007629394531250, 0.0030822753906250,
+0.0007324218750000, 0.0033569335937500, 0.0007019042968750, 0.0032043457031250,
+0.0008239746093750, 0.0023803710937500, 0.0009155273437500, 0.0026855468750000,
+0.0008544921875000, 0.0017089843750000, 0.0011596679687500, 0.0016784667968750,
+0.0011901855468750, 0.0014038085937500, 0.0013732910156250, 0.0005493164062500,
+0.0016784667968750, 0.0006713867187500, 0.0016784667968750, 0.0003356933593750,
+0.0018005371093750, 0.0000000000000000, 0.0019836425781250, 0.0000305175781250,
+0.0018920898437500, 0.0000000000000000, 0.0018310546875000, -0.0003051757812500,
+0.0020141601562500, -0.0003356933593750, 0.0018615722656250, -0.0004272460937500,
+0.0019531250000000, -0.0005187988281250, 0.0019836425781250, -0.0007324218750000,
+0.0018310546875000, -0.0008239746093750, 0.0018920898437500, -0.0003967285156250,
+0.0020141601562500, -0.0007629394531250, 0.0019836425781250, -0.0003967285156250,
+0.0020141601562500, 0.0001831054687500, 0.0021362304687500, -0.0003356933593750,
+0.0020751953125000, 0.0007324218750000, 0.0020446777343750, 0.0005798339843750,
+0.0020751953125000, 0.0008850097656250, 0.0019531250000000, 0.0017395019531250,
+0.0018615722656250, 0.0012207031250000, 0.0018615722656250, 0.0016174316406250,
+0.0015258789062500, 0.0018920898437500, 0.0015258789062500, 0.0016479492187500,
+0.0014953613281250, 0.0015258789062500, 0.0012817382812500, 0.0018005371093750,
+0.0012817382812500, 0.0013122558593750, 0.0013732910156250, 0.0013122558593750,
+0.0012817382812500, 0.0014038085937500, 0.0013732910156250, 0.0011596679687500,
+0.0015258789062500, 0.0013122558593750, 0.0014038085937500, 0.0014648437500000,
+0.0018005371093750, 0.0013732910156250, 0.0018310546875000, 0.0017089843750000,
+0.0019226074218750, 0.0020141601562500, 0.0022277832031250, 0.0018615722656250,
+0.0021972656250000, 0.0025939941406250, 0.0024108886718750, 0.0026245117187500,
+0.0024414062500000, 0.0028381347656250, 0.0024719238281250, 0.0033569335937500,
+0.0025939941406250, 0.0032348632812500, 0.0025024414062500, 0.0035705566406250,
+0.0025939941406250, 0.0036926269531250, 0.0025024414062500, 0.0035705566406250,
+0.0025329589843750, 0.0037231445312500, 0.0026855468750000, 0.0037536621093750,
+0.0025329589843750, 0.0032653808593750, 0.0026550292968750, 0.0034790039062500,
+0.0025939941406250, 0.0031127929687500, 0.0026245117187500, 0.0025634765625000,
+0.0026550292968750, 0.0029296875000000, 0.0025634765625000, 0.0020141601562500,
+0.0028381347656250, 0.0020141601562500, 0.0026855468750000, 0.0020446777343750,
+0.0028076171875000, 0.0015258789062500, 0.0031433105468750, 0.0016784667968750,
+0.0029907226562500, 0.0016784667968750, 0.0031738281250000, 0.0014038085937500,
+0.0032958984375000, 0.0016479492187500, 0.0030822753906250, 0.0018615722656250,
+0.0030517578125000, 0.0014953613281250, 0.0031738281250000, 0.0022583007812500,
+0.0024719238281250, 0.0020446777343750, 0.0026855468750000, 0.0021667480468750,
+0.0022583007812500, 0.0027160644531250, 0.0015563964843750, 0.0022583007812500,
+0.0020141601562500, 0.0027160644531250, 0.0008850097656250, 0.0027465820312500,
+0.0010070800781250, 0.0025329589843750, 0.0007934570312500, 0.0026245117187500,
+0.0000610351562500, 0.0025329589843750, 0.0003967285156250, 0.0023498535156250,
+-0.0003051757812500, 0.0023193359375000, -0.0003662109375000, 0.0022888183593750,
+-0.0002746582031250, 0.0021667480468750, -0.0006103515625000, 0.0021057128906250,
+-0.0006103515625000, 0.0021057128906250, -0.0002746582031250, 0.0020446777343750,
+-0.0004577636718750, 0.0019836425781250, -0.0001831054687500, 0.0020446777343750,
+0.0002136230468750, 0.0019836425781250, -0.0000305175781250, 0.0018005371093750,
+0.0007629394531250, 0.0018615722656250, 0.0007019042968750, 0.0017395019531250,
+0.0008544921875000, 0.0015869140625000, 0.0013732910156250, 0.0016479492187500,
+0.0011291503906250, 0.0015869140625000, 0.0014038085937500, 0.0014648437500000,
+0.0014038085937500, 0.0015563964843750, 0.0013427734375000, 0.0015563964843750,
+0.0014648437500000, 0.0014648437500000, 0.0014038085937500, 0.0018310546875000,
+0.0014953613281250, 0.0016479492187500, 0.0013732910156250, 0.0018005371093750,
+0.0014343261718750, 0.0020751953125000, 0.0014648437500000, 0.0018005371093750,
+0.0013122558593750, 0.0022583007812500, 0.0017089843750000, 0.0021667480468750,
+0.0015563964843750, 0.0021362304687500, 0.0017700195312500, 0.0023498535156250,
+0.0022277832031250, 0.0021667480468750, 0.0019531250000000, 0.0022277832031250,
+0.0025939941406250, 0.0022277832031250, 0.0025939941406250, 0.0021362304687500,
+0.0025634765625000, 0.0021057128906250, 0.0029296875000000, 0.0022277832031250,
+0.0028686523437500, 0.0021057128906250, 0.0027160644531250, 0.0021362304687500,
+0.0029602050781250, 0.0021362304687500, 0.0027160644531250, 0.0021362304687500,
+0.0024414062500000, 0.0021667480468750, 0.0027160644531250, 0.0022277832031250,
+0.0019836425781250, 0.0022583007812500, 0.0021362304687500, 0.0022277832031250,
+0.0019836425781250, 0.0022583007812500, 0.0014038085937500, 0.0023498535156250,
+0.0018005371093750, 0.0021362304687500, 0.0009765625000000, 0.0022277832031250,
+0.0010681152343750, 0.0020751953125000, 0.0010070800781250, 0.0018310546875000,
+0.0005187988281250, 0.0019836425781250, 0.0007019042968750, 0.0015869140625000,
+0.0003967285156250, 0.0015869140625000, 0.0003662109375000, 0.0016174316406250,
+0.0004882812500000, 0.0013427734375000, 0.0004577636718750, 0.0013427734375000,
+0.0004577636718750, 0.0015869140625000, 0.0008239746093750, 0.0013732910156250,
+0.0007629394531250, 0.0016479492187500, 0.0008850097656250, 0.0019836425781250,
+0.0012207031250000, 0.0017395019531250, 0.0011291503906250, 0.0022583007812500,
+0.0013427734375000, 0.0022888183593750, 0.0014343261718750, 0.0022888183593750,
+0.0014343261718750, 0.0026245117187500, 0.0015258789062500, 0.0025329589843750,
+0.0014648437500000, 0.0025329589843750, 0.0014343261718750, 0.0026550292968750,
+0.0013732910156250, 0.0025329589843750, 0.0013732910156250, 0.0024108886718750,
+0.0013427734375000, 0.0024414062500000, 0.0012512207031250, 0.0023803710937500,
+0.0017089843750000, 0.0022888183593750, 0.0014038085937500, 0.0023193359375000,
+0.0016174316406250, 0.0022888183593750, 0.0021667480468750, 0.0021972656250000,
+0.0019226074218750, 0.0024719238281250, 0.0023498535156250, 0.0023498535156250,
+0.0024414062500000, 0.0022583007812500, 0.0023803710937500, 0.0024108886718750,
+0.0025024414062500, 0.0022888183593750, 0.0024719238281250, 0.0021972656250000,
+0.0024108886718750, 0.0022277832031250, 0.0022888183593750, 0.0021362304687500,
+0.0022583007812500, 0.0020751953125000, 0.0021667480468750, 0.0021057128906250,
+0.0021362304687500, 0.0018920898437500, 0.0023193359375000, 0.0019531250000000,
+0.0020446777343750, 0.0020446777343750, 0.0023498535156250, 0.0019226074218750,
+0.0027465820312500, 0.0019226074218750, 0.0023803710937500, 0.0020751953125000,
+0.0032348632812500, 0.0020446777343750, 0.0031738281250000, 0.0021057128906250,
+0.0032043457031250, 0.0021972656250000, 0.0037536621093750, 0.0021057128906250,
+0.0035705566406250, 0.0022888183593750, 0.0034179687500000, 0.0022277832031250,
+0.0036926269531250, 0.0021972656250000, 0.0035095214843750, 0.0023498535156250,
+0.0028991699218750, 0.0015869140625000, 0.0030212402343750, 0.0014343261718750,
+0.0036010742187500, 0.0014953613281250, 0.0035095214843750, 0.0013427734375000,
+0.0033569335937500, 0.0011596679687500, 0.0034179687500000, 0.0012817382812500,
+0.0034179687500000, 0.0007324218750000, 0.0029907226562500, 0.0008239746093750,
+0.0031127929687500, 0.0006713867187500, 0.0029296875000000, 0.0002441406250000,
+0.0025939941406250, 0.0004577636718750, 0.0027770996093750, 0.0003051757812500,
+0.0022583007812500, 0.0001220703125000, 0.0023498535156250, 0.0003967285156250,
+0.0022888183593750, 0.0005798339843750, 0.0019226074218750, 0.0003967285156250,
+0.0019836425781250, 0.0011596679687500, 0.0018920898437500, 0.0010986328125000,
+0.0017395019531250, 0.0011596679687500, 0.0017395019531250, 0.0016174316406250,
+0.0016784667968750, 0.0014343261718750, 0.0015563964843750, 0.0015258789062500,
+0.0019836425781250, 0.0016479492187500, 0.0017700195312500, 0.0014953613281250,
+0.0018005371093750, 0.0013732910156250, 0.0021057128906250, 0.0014953613281250,
+0.0019226074218750, 0.0013122558593750, 0.0015563964843750, 0.0013427734375000,
+0.0018310546875000, 0.0012817382812500, 0.0014648437500000, 0.0011596679687500,
+0.0008544921875000, 0.0012207031250000, 0.0011291503906250, 0.0012207031250000,
+0.0006713867187500, 0.0011291503906250, 0.0006408691406250, 0.0012207031250000,
+0.0007019042968750, 0.0013427734375000, 0.0004882812500000, 0.0012512207031250,
+0.0005798339843750, 0.0015258789062500, 0.0007934570312500, 0.0015258789062500,
+0.0007629394531250, 0.0016784667968750, 0.0007629394531250, 0.0018005371093750,
+0.0009460449218750, 0.0018005371093750, 0.0009765625000000, 0.0017089843750000,
+0.0011291503906250, 0.0017700195312500, 0.0011291503906250, 0.0017089843750000,
+0.0010070800781250, 0.0015869140625000, 0.0010986328125000, 0.0016174316406250,
+0.0011901855468750, 0.0015869140625000, 0.0009765625000000, 0.0015258789062500,
+0.0009155273437500, 0.0014953613281250, 0.0010681152343750, 0.0014343261718750,
+0.0010986328125000, 0.0014038085937500, 0.0010070800781250, 0.0014343261718750,
+0.0011901855468750, 0.0013427734375000, 0.0012512207031250, 0.0013732910156250,
+0.0012207031250000, 0.0015258789062500, 0.0012512207031250, 0.0014343261718750,
+0.0012512207031250, 0.0013732910156250, 0.0011596679687500, 0.0014648437500000,
+0.0012512207031250, 0.0012207031250000, 0.0011291503906250, 0.0009765625000000,
+0.0009765625000000, 0.0013122558593750, 0.0009460449218750, 0.0007324218750000,
+0.0007324218750000, 0.0007629394531250, 0.0007934570312500, 0.0009460449218750,
+0.0007324218750000, 0.0006408691406250, 0.0004272460937500, 0.0007629394531250,
+0.0005187988281250, 0.0012512207031250, 0.0005187988281250, 0.0009155273437500,
+0.0004577636718750, 0.0014038085937500, 0.0006103515625000, 0.0020446777343750,
+0.0007019042968750, 0.0014953613281250, 0.0006408691406250, 0.0028076171875000,
+0.0008850097656250, 0.0025329589843750, 0.0009765625000000, 0.0026855468750000,
+0.0010375976562500, 0.0035705566406250, 0.0011596679687500, 0.0031127929687500,
+0.0011901855468750, 0.0033569335937500, 0.0011596679687500, 0.0036315917968750,
+0.0011596679687500},
+{0.0029602050781250, 0.0017089843750000, 0.0028686523437500, 0.0016479492187500,
+0.0028991699218750, 0.0015869140625000, 0.0035400390625000, 0.0012817382812500,
+0.0025024414062500, 0.0005493164062500, 0.0022888183593750, 0.0007019042968750,
+0.0021362304687500, 0.0006103515625000, 0.0024414062500000, 0.0002746582031250,
+0.0026245117187500, 0.0004272460937500, 0.0024414062500000, 0.0004272460937500,
+0.0028991699218750, 0.0003356933593750, 0.0029602050781250, 0.0005798339843750,
+0.0028381347656250, 0.0008239746093750, 0.0029296875000000, 0.0006713867187500,
+0.0027770996093750, 0.0010681152343750, 0.0027160644531250, 0.0011901855468750,
+0.0026550292968750, 0.0010681152343750, 0.0025634765625000, 0.0011901855468750,
+0.0024108886718750, 0.0013122558593750, 0.0024719238281250, 0.0010375976562500,
+0.0022277832031250, 0.0011291503906250, 0.0023803710937500, 0.0010681152343750,
+0.0020751953125000, 0.0009765625000000, 0.0017395019531250, 0.0010986328125000,
+0.0020751953125000, 0.0009155273437500, 0.0013732910156250, 0.0009765625000000,
+0.0013122558593750, 0.0009765625000000, 0.0014648437500000, 0.0008850097656250,
+0.0012512207031250, 0.0010986328125000, 0.0013122558593750, 0.0010070800781250,
+0.0013732910156250, 0.0010681152343750, 0.0013732910156250, 0.0011596679687500,
+0.0014038085937500, 0.0011901855468750, 0.0014343261718750, 0.0013427734375000,
+0.0014648437500000, 0.0013122558593750, 0.0012512207031250, 0.0014343261718750,
+0.0013732910156250, 0.0015258789062500, 0.0011596679687500, 0.0016479492187500,
+0.0008850097656250, 0.0016479492187500, 0.0010681152343750, 0.0015563964843750,
+0.0003967285156250, 0.0016784667968750, 0.0004577636718750, 0.0014648437500000,
+0.0003662109375000, 0.0011901855468750, -0.0001831054687500, 0.0014038085937500,
+0.0000610351562500, 0.0010070800781250, -0.0001831054687500, 0.0008850097656250,
+-0.0003967285156250, 0.0010070800781250, -0.0001831054687500, 0.0008544921875000,
+-0.0000915527343750, 0.0007934570312500, -0.0002136230468750, 0.0010986328125000,
+0.0001220703125000, 0.0009765625000000, 0.0001831054687500, 0.0010681152343750,
+0.0000915527343750, 0.0013427734375000, 0.0002136230468750, 0.0011596679687500,
+0.0002746582031250, 0.0013122558593750, 0.0001831054687500, 0.0014038085937500,
+0.0001831054687500, 0.0011596679687500, 0.0002136230468750, 0.0010986328125000,
+0.0001220703125000, 0.0010986328125000, 0.0001220703125000, 0.0007324218750000,
+0.0001831054687500, 0.0006408691406250, 0.0000305175781250, 0.0007934570312500,
+0.0000610351562500, 0.0006103515625000, 0.0001831054687500, 0.0004882812500000,
+0.0000915527343750, 0.0010986328125000, 0.0001831054687500, 0.0007934570312500,
+0.0002441406250000, 0.0011901855468750, 0.0001220703125000, 0.0019226074218750,
+0.0000610351562500, 0.0013732910156250, 0.0001525878906250, 0.0023498535156250,
+-0.0002441406250000, 0.0023498535156250, -0.0002746582031250, 0.0022583007812500,
+-0.0003967285156250, 0.0027465820312500, -0.0007324218750000, 0.0026245117187500,
+-0.0006408691406250, 0.0022277832031250, -0.0006408691406250, 0.0024719238281250,
+-0.0007019042968750, 0.0021057128906250, -0.0005493164062500, 0.0015563964843750,
+-0.0003967285156250, 0.0019836425781250, -0.0003356933593750, 0.0010681152343750,
+-0.0000915527343750, 0.0011901855468750, 0.0000610351562500, 0.0011901855468750,
+0.0000915527343750, 0.0007324218750000, 0.0002136230468750, 0.0009155273437500,
+0.0003051757812500, 0.0009765625000000, 0.0001831054687500, 0.0008239746093750,
+0.0003356933593750, 0.0011596679687500, 0.0002746582031250, 0.0015258789062500,
+0.0000915527343750, 0.0012817382812500, 0.0004272460937500, 0.0023193359375000,
+0.0000915527343750, 0.0021362304687500, 0.0001220703125000, 0.0024719238281250,
+0.0002746582031250, 0.0033264160156250, 0.0002136230468750, 0.0029296875000000,
+0.0002441406250000, 0.0035095214843750, 0.0005493164062500, 0.0037536621093750,
+0.0004577636718750, 0.0035095214843750, 0.0007019042968750, 0.0036010742187500,
+0.0010070800781250, 0.0037536621093750, 0.0008239746093750, 0.0028381347656250,
+0.0012512207031250, 0.0031433105468750, 0.0012207031250000, 0.0026550292968750,
+0.0013427734375000, 0.0016479492187500, 0.0015869140625000, 0.0022277832031250,
+0.0014953613281250, 0.0009765625000000, 0.0018005371093750, 0.0010681152343750,
+0.0016784667968750, 0.0009765625000000, 0.0018615722656250, 0.0003356933593750,
+0.0021972656250000, 0.0007324218750000, 0.0018615722656250, 0.0004577636718750,
+0.0024108886718750, 0.0003356933593750, 0.0022888183593750, 0.0005798339843750,
+0.0022583007812500, 0.0006103515625000, 0.0026245117187500, 0.0005493164062500,
+0.0024414062500000, 0.0009155273437500, 0.0026245117187500, 0.0007629394531250,
+0.0027465820312500, 0.0008850097656250, 0.0026245117187500, 0.0011291503906250,
+0.0025634765625000, 0.0008850097656250, 0.0027160644531250, 0.0013427734375000,
+0.0024108886718750, 0.0012207031250000, 0.0025024414062500, 0.0014648437500000,
+0.0024108886718750, 0.0018005371093750, 0.0021972656250000, 0.0014038085937500,
+0.0022888183593750, 0.0024108886718750, 0.0021057128906250, 0.0021362304687500,
+0.0021057128906250, 0.0025024414062500, 0.0020751953125000, 0.0033569335937500,
+0.0019836425781250, 0.0028381347656250, 0.0021057128906250, 0.0042114257812500,
+0.0020446777343750, 0.0040283203125000, 0.0020446777343750, 0.0043029785156250,
+0.0020141601562500, 0.0052795410156250, 0.0020141601562500, 0.0046691894531250,
+0.0019836425781250, 0.0057067871093750, 0.0018005371093750, 0.0057067871093750,
+0.0018920898437500, 0.0057067871093750, 0.0017395019531250, 0.0062866210937500,
+0.0014953613281250, 0.0060729980468750, 0.0017395019531250, 0.0062561035156250,
+0.0013732910156250, 0.0063476562500000, 0.0014648437500000, 0.0062561035156250,
+0.0014648437500000, 0.0062561035156250, 0.0013427734375000, 0.0063171386718750,
+0.0014953613281250, 0.0059814453125000, 0.0016174316406250, 0.0060729980468750,
+0.0015258789062500, 0.0058898925781250, 0.0018005371093750, 0.0056457519531250,
+0.0020141601562500, 0.0057678222656250, 0.0019531250000000, 0.0050659179687500,
+0.0022888183593750, 0.0051574707031250, 0.0023498535156250, 0.0048217773437500,
+0.0024108886718750, 0.0041503906250000, 0.0026245117187500, 0.0045166015625000,
+0.0025329589843750, 0.0034790039062500, 0.0025939941406250, 0.0034790039062500,
+0.0025939941406250, 0.0032958984375000, 0.0023803710937500, 0.0025939941406250,
+0.0022888183593750, 0.0027160644531250, 0.0023498535156250, 0.0022277832031250,
+0.0019226074218750, 0.0020446777343750, 0.0019531250000000, 0.0018920898437500,
+0.0018005371093750, 0.0016174316406250, 0.0014953613281250, 0.0016174316406250,
+0.0016174316406250, 0.0012207031250000, 0.0013427734375000, 0.0013122558593750,
+0.0013122558593750, 0.0009765625000000, 0.0012817382812500, 0.0005798339843750,
+0.0010986328125000, 0.0007629394531250, 0.0011901855468750, -0.0000305175781250,
+0.0010070800781250, 0.0001220703125000, 0.0010375976562500, -0.0000610351562500,
+0.0011901855468750, -0.0007019042968750, 0.0010681152343750, -0.0004272460937500,
+0.0011291503906250, -0.0008239746093750, 0.0014953613281250, -0.0009765625000000,
+0.0013427734375000, -0.0008850097656250, 0.0015869140625000, -0.0009765625000000,
+0.0020141601562500, -0.0010375976562500, 0.0018615722656250, -0.0008850097656250,
+0.0021972656250000, -0.0010375976562500, 0.0022888183593750, -0.0008544921875000,
+0.0022277832031250, -0.0006103515625000, 0.0022888183593750, -0.0006408691406250,
+0.0021972656250000, 0.0000000000000000, 0.0020141601562500, -0.0003662109375000,
+0.0021362304687500, -0.0001831054687500, 0.0019836425781250, 0.0003967285156250,
+0.0018005371093750, -0.0000305175781250, 0.0019836425781250, 0.0010681152343750,
+0.0018005371093750, 0.0010375976562500, 0.0018615722656250, 0.0011901855468750,
+0.0018920898437500, 0.0019836425781250, 0.0017700195312500, 0.0017089843750000,
+0.0017700195312500, 0.0020446777343750, 0.0018005371093750, 0.0021972656250000,
+0.0018615722656250, 0.0020446777343750, 0.0017395019531250, 0.0020446777343750,
+0.0016479492187500, 0.0021057128906250, 0.0017089843750000, 0.0017089843750000,
+0.0013122558593750, 0.0016784667968750, 0.0013122558593750, 0.0015869140625000,
+0.0011596679687500, 0.0013732910156250, 0.0008239746093750, 0.0014343261718750,
+0.0009765625000000, 0.0015258789062500, 0.0005798339843750, 0.0013427734375000,
+0.0006103515625000, 0.0015258789062500, 0.0005187988281250, 0.0017395019531250,
+0.0002746582031250, 0.0015869140625000, 0.0004577636718750, 0.0016479492187500,
+0.0003967285156250, 0.0019836425781250, 0.0003051757812500, 0.0015563964843750,
+0.0004882812500000, 0.0012512207031250, 0.0006103515625000, 0.0016174316406250,
+0.0005493164062500, 0.0007934570312500, 0.0008850097656250, 0.0007629394531250,
+0.0009460449218750, 0.0009155273437500, 0.0009155273437500, 0.0004882812500000,
+0.0011596679687500, 0.0006408691406250, 0.0011901855468750, 0.0007324218750000,
+0.0011596679687500, 0.0006713867187500, 0.0011291503906250, 0.0008544921875000,
+0.0011596679687500, 0.0010070800781250, 0.0012207031250000, 0.0009765625000000,
+0.0012817382812500, 0.0009765625000000, 0.0013427734375000, 0.0010375976562500,
+0.0013732910156250, 0.0009765625000000, 0.0014038085937500, 0.0009155273437500,
+0.0014343261718750, 0.0008544921875000, 0.0013732910156250, 0.0006408691406250,
+0.0015258789062500, 0.0006408691406250, 0.0015258789062500, 0.0005187988281250,
+0.0016479492187500, 0.0002746582031250, 0.0017395019531250, 0.0003051757812500,
+0.0017089843750000, 0.0000610351562500, 0.0019226074218750, -0.0000305175781250,
+0.0018310546875000, -0.0000305175781250, 0.0019531250000000, -0.0001525878906250,
+0.0021362304687500, -0.0001831054687500, 0.0021057128906250, -0.0000610351562500,
+0.0021667480468750, -0.0001220703125000, 0.0022583007812500, -0.0000610351562500,
+0.0021667480468750, 0.0000305175781250, 0.0021362304687500, 0.0000305175781250,
+0.0022888183593750, 0.0002136230468750, 0.0018310546875000, 0.0001525878906250,
+0.0019226074218750, 0.0000915527343750, 0.0019226074218750, 0.0002136230468750,
+0.0015869140625000, 0.0001831054687500, 0.0017089843750000, 0.0000000000000000,
+0.0014648437500000, 0.0000610351562500, 0.0014953613281250, -0.0000610351562500,
+0.0014953613281250, -0.0002136230468750, 0.0014038085937500, -0.0000305175781250,
+0.0014038085937500, -0.0001525878906250, 0.0014343261718750, -0.0000915527343750,
+0.0014343261718750, 0.0000305175781250, 0.0014038085937500, -0.0000610351562500,
+0.0014343261718750, 0.0000610351562500, 0.0013732910156250, 0.0001831054687500,
+0.0016479492187500, 0.0001831054687500, 0.0014953613281250, 0.0004272460937500,
+0.0016479492187500, 0.0006713867187500, 0.0020141601562500, 0.0006103515625000,
+0.0017395019531250, 0.0010681152343750, 0.0022277832031250, 0.0011596679687500,
+0.0021972656250000, 0.0012512207031250, 0.0022277832031250, 0.0015869140625000,
+0.0025329589843750, 0.0016479492187500, 0.0023498535156250, 0.0017395019531250,
+0.0025024414062500, 0.0019531250000000, 0.0025634765625000, 0.0018920898437500,
+0.0024108886718750, 0.0018005371093750, 0.0023498535156250, 0.0020751953125000,
+0.0023803710937500, 0.0016784667968750, 0.0020751953125000, 0.0018310546875000,
+0.0021972656250000, 0.0018005371093750, 0.0018920898437500, 0.0015258789062500,
+0.0015258789062500},
+{0.0017700195312500, 0.0006408691406250, 0.0012512207031250, 0.0004882812500000,
+0.0016174316406250, 0.0010070800781250, 0.0008850097656250, 0.0008850097656250,
+0.0009460449218750, 0.0009765625000000, 0.0008850097656250, 0.0013427734375000,
+0.0003967285156250, 0.0010986328125000, 0.0005798339843750, 0.0012207031250000,
+0.0004577636718750, 0.0013122558593750, 0.0003051757812500, 0.0011291503906250,
+0.0004577636718750, 0.0010070800781250, 0.0005187988281250, 0.0010986328125000,
+0.0003967285156250, 0.0010070800781250, 0.0007629394531250, 0.0009460449218750,
+0.0006103515625000, 0.0008850097656250, 0.0007629394531250, 0.0009155273437500,
+0.0011291503906250, 0.0009155273437500, 0.0009155273437500, 0.0006713867187500,
+0.0013122558593750, 0.0007629394531250, 0.0012817382812500, 0.0005187988281250,
+0.0013427734375000, 0.0001831054687500, 0.0016174316406250, 0.0003356933593750,
+0.0014953613281250, -0.0002136230468750, 0.0018615722656250, -0.0002746582031250,
+0.0019836425781250, -0.0001831054687500, 0.0019531250000000, -0.0003967285156250,
+0.0021362304687500, -0.0005187988281250, 0.0022277832031250, 0.0000915527343750,
+0.0021362304687500, -0.0002136230468750, 0.0023193359375000, 0.0000305175781250,
+0.0023498535156250, 0.0005493164062500, 0.0022583007812500, 0.0000305175781250,
+0.0024108886718750, 0.0007629394531250, 0.0025024414062500, 0.0007629394531250,
+0.0024719238281250, 0.0006103515625000, 0.0026245117187500, 0.0009460449218750,
+0.0027160644531250, 0.0008544921875000, 0.0027465820312500, 0.0005798339843750,
+0.0028686523437500, 0.0007629394531250, 0.0028686523437500, 0.0006408691406250,
+0.0028076171875000, 0.0003967285156250, 0.0028381347656250, 0.0006103515625000,
+0.0028381347656250, 0.0004577636718750, 0.0025634765625000, 0.0004272460937500,
+0.0026855468750000, 0.0006713867187500, 0.0024719238281250, 0.0007019042968750,
+0.0021667480468750, 0.0006103515625000, 0.0024108886718750, 0.0011901855468750,
+0.0018920898437500, 0.0010070800781250, 0.0019226074218750, 0.0013427734375000,
+0.0018310546875000, 0.0019226074218750, 0.0015869140625000, 0.0016174316406250,
+0.0017089843750000, 0.0021972656250000, 0.0014038085937500, 0.0023193359375000,
+0.0014648437500000, 0.0023193359375000, 0.0013732910156250, 0.0025634765625000,
+0.0011596679687500, 0.0024719238281250, 0.0013427734375000, 0.0025329589843750,
+0.0011596679687500, 0.0025329589843750, 0.0011291503906250, 0.0024108886718750,
+0.0012207031250000, 0.0022888183593750, 0.0011596679687500, 0.0022583007812500,
+0.0011291503906250, 0.0019226074218750, 0.0011291503906250, 0.0019226074218750,
+0.0011596679687500, 0.0017700195312500, 0.0011901855468750, 0.0014648437500000,
+0.0011291503906250, 0.0015563964843750, 0.0010681152343750, 0.0014648437500000,
+0.0013732910156250, 0.0012817382812500, 0.0011291503906250, 0.0014038085937500,
+0.0013427734375000, 0.0014648437500000, 0.0016479492187500, 0.0012512207031250,
+0.0014343261718750, 0.0018310546875000, 0.0020446777343750, 0.0015869140625000,
+0.0020141601562500, 0.0017700195312500, 0.0021362304687500, 0.0022277832031250,
+0.0025939941406250, 0.0018615722656250, 0.0024108886718750, 0.0024414062500000,
+0.0028076171875000, 0.0023498535156250, 0.0028686523437500, 0.0023498535156250,
+0.0027465820312500, 0.0026245117187500, 0.0028381347656250, 0.0024108886718750,
+0.0029602050781250, 0.0025634765625000, 0.0025939941406250, 0.0024414062500000,
+0.0027465820312500, 0.0024414062500000, 0.0025329589843750, 0.0024719238281250,
+0.0022583007812500, 0.0023498535156250, 0.0025329589843750, 0.0025024414062500,
+0.0019836425781250, 0.0023498535156250, 0.0019836425781250, 0.0024108886718750,
+0.0019531250000000, 0.0026245117187500, 0.0016479492187500, 0.0023803710937500,
+0.0018005371093750, 0.0026855468750000, 0.0018920898437500, 0.0026855468750000,
+0.0016784667968750, 0.0026550292968750, 0.0019226074218750, 0.0028076171875000,
+0.0022583007812500, 0.0027770996093750, 0.0020751953125000, 0.0026245117187500,
+0.0026245117187500, 0.0026855468750000, 0.0026550292968750, 0.0025634765625000,
+0.0027160644531250, 0.0024719238281250, 0.0029907226562500, 0.0025024414062500,
+0.0029296875000000, 0.0021057128906250, 0.0032348632812500, 0.0022277832031250,
+0.0032043457031250, 0.0020751953125000, 0.0033264160156250, 0.0017089843750000,
+0.0035705566406250, 0.0018615722656250, 0.0034790039062500, 0.0015563964843750,
+0.0035095214843750, 0.0016479492187500, 0.0036926269531250, 0.0016479492187500,
+0.0035400390625000, 0.0015563964843750, 0.0034179687500000, 0.0016784667968750,
+0.0036315917968750, 0.0014953613281250, 0.0029296875000000, 0.0015869140625000,
+0.0031433105468750, 0.0015869140625000, 0.0029296875000000, 0.0015563964843750,
+0.0022888183593750, 0.0017395019531250, 0.0026855468750000, 0.0014953613281250,
+0.0019836425781250, 0.0016784667968750, 0.0019531250000000, 0.0016479492187500,
+0.0019531250000000, 0.0014648437500000, 0.0016174316406250, 0.0017089843750000,
+0.0017700195312500, 0.0015563964843750, 0.0018310546875000, 0.0016174316406250,
+0.0016174316406250, 0.0017395019531250, 0.0016174316406250, 0.0018005371093750,
+0.0017395019531250, 0.0019226074218750, 0.0015563964843750, 0.0019226074218750,
+0.0015563964843750, 0.0020751953125000, 0.0014953613281250, 0.0020446777343750,
+0.0014648437500000, 0.0019531250000000, 0.0013427734375000, 0.0021667480468750,
+0.0011291503906250, 0.0019226074218750, 0.0014343261718750, 0.0020141601562500,
+0.0011596679687500, 0.0021667480468750, 0.0012817382812500, 0.0020446777343750,
+0.0015869140625000, 0.0022277832031250, 0.0012207031250000, 0.0023803710937500,
+0.0018310546875000, 0.0024108886718750, 0.0015869140625000, 0.0025329589843750,
+0.0017395019531250, 0.0025939941406250, 0.0022277832031250, 0.0025634765625000,
+0.0018005371093750, 0.0025939941406250, 0.0024108886718750, 0.0026855468750000,
+0.0023498535156250, 0.0025024414062500, 0.0021362304687500, 0.0023498535156250,
+0.0024108886718750, 0.0024414062500000, 0.0022583007812500, 0.0021972656250000,
+0.0020751953125000, 0.0021667480468750, 0.0020141601562500, 0.0021667480468750,
+0.0018615722656250, 0.0019836425781250, 0.0017395019531250, 0.0020141601562500,
+0.0016784667968750, 0.0019226074218750, 0.0014648437500000, 0.0018615722656250,
+0.0014953613281250, 0.0018920898437500, 0.0013732910156250, 0.0019226074218750,
+0.0011901855468750, 0.0018005371093750, 0.0011596679687500, 0.0017395019531250,
+0.0009765625000000, 0.0018310546875000, 0.0009765625000000, 0.0016479492187500,
+0.0010986328125000, 0.0014648437500000, 0.0010070800781250, 0.0015869140625000,
+0.0009765625000000, 0.0012512207031250, 0.0015258789062500, 0.0012817382812500,
+0.0013732910156250, 0.0013122558593750, 0.0015563964843750, 0.0009765625000000,
+0.0024108886718750, 0.0011901855468750, 0.0015869140625000, 0.0014648437500000,
+0.0019531250000000, 0.0014648437500000, 0.0018005371093750, 0.0014648437500000,
+0.0017089843750000, 0.0015258789062500, 0.0017700195312500, 0.0015258789062500,
+0.0017395019531250, 0.0014343261718750, 0.0015869140625000, 0.0014038085937500,
+0.0015869140625000, 0.0015258789062500, 0.0017700195312500, 0.0015258789062500,
+0.0017395019531250, 0.0014343261718750, 0.0016784667968750, 0.0018005371093750,
+0.0018615722656250, 0.0017089843750000, 0.0019531250000000, 0.0016784667968750,
+0.0019531250000000, 0.0019226074218750, 0.0020446777343750, 0.0018005371093750,
+0.0020751953125000, 0.0018005371093750, 0.0018005371093750, 0.0018615722656250,
+0.0019226074218750, 0.0018920898437500, 0.0018615722656250, 0.0018310546875000,
+0.0015869140625000, 0.0018615722656250, 0.0017395019531250, 0.0019836425781250,
+0.0016174316406250, 0.0020446777343750, 0.0016479492187500, 0.0020141601562500,
+0.0015563964843750, 0.0020141601562500, 0.0014953613281250, 0.0020751953125000,
+0.0016479492187500, 0.0019226074218750, 0.0011596679687500, 0.0019531250000000,
+0.0014343261718750, 0.0019226074218750, 0.0010681152343750, 0.0018005371093750,
+0.0006103515625000, 0.0018310546875000, 0.0010681152343750, 0.0019531250000000,
+0.0000610351562500, 0.0019226074218750, 0.0002441406250000, 0.0020751953125000,
+0.0003967285156250, 0.0022277832031250, -0.0002136230468750, 0.0022888183593750,
+0.0000610351562500, 0.0023193359375000, 0.0003356933593750, 0.0025024414062500,
+0.0000915527343750, 0.0023803710937500, 0.0003662109375000, 0.0022888183593750,
+0.0007629394531250, 0.0024414062500000, 0.0003967285156250, 0.0019531250000000,
+0.0010986328125000, 0.0021667480468750, 0.0010986328125000, 0.0019531250000000,
+0.0009460449218750, 0.0014953613281250, 0.0013427734375000, 0.0018310546875000,
+0.0013732910156250, 0.0013427734375000, 0.0009765625000000, 0.0012817382812500,
+0.0010986328125000, 0.0015563964843750, 0.0008544921875000, 0.0014038085937500,
+0.0004272460937500, 0.0013732910156250, 0.0007629394531250, 0.0021362304687500,
+0.0004272460937500, 0.0020446777343750, 0.0002441406250000, 0.0022888183593750,
+0.0006408691406250, 0.0029602050781250, 0.0007629394531250, 0.0026855468750000,
+0.0005798339843750, 0.0028991699218750, 0.0014648437500000, 0.0031738281250000,
+0.0012817382812500, 0.0027770996093750, 0.0014953613281250, 0.0025329589843750,
+0.0021057128906250, 0.0028381347656250, 0.0017089843750000, 0.0016174316406250,
+0.0021972656250000, 0.0018615722656250, 0.0021362304687500, 0.0014953613281250,
+0.0021667480468750, 0.0004882812500000, 0.0024414062500000, 0.0009460449218750,
+0.0021667480468750, 0.0003662109375000, 0.0026245117187500, 0.0000610351562500,
+0.0026245117187500, 0.0005187988281250, 0.0026550292968750, 0.0006103515625000,
+0.0028686523437500, 0.0003356933593750, 0.0028076171875000, 0.0015258789062500,
+0.0026245117187500, 0.0012207031250000, 0.0028381347656250, 0.0015869140625000,
+0.0026550292968750, 0.0025634765625000, 0.0023498535156250, 0.0020446777343750,
+0.0025939941406250, 0.0027160644531250, 0.0020141601562500, 0.0028686523437500,
+0.0022583007812500, 0.0025634765625000, 0.0021667480468750, 0.0027160644531250,
+0.0016784667968750, 0.0027160644531250, 0.0020751953125000, 0.0020141601562500,
+0.0016784667968750, 0.0023193359375000, 0.0016784667968750, 0.0018005371093750,
+0.0017395019531250, 0.0009765625000000, 0.0015869140625000, 0.0015258789062500,
+0.0017089843750000, 0.0003662109375000, 0.0016479492187500, 0.0003662109375000,
+0.0017089843750000, 0.0003356933593750, 0.0016784667968750, -0.0002746582031250,
+0.0016784667968750, -0.0000610351562500, 0.0017700195312500, 0.0000000000000000,
+0.0015563964843750, -0.0002746582031250, 0.0016784667968750, 0.0000915527343750,
+0.0015563964843750, 0.0004272460937500, 0.0014343261718750, 0.0001525878906250,
+0.0015258789062500, 0.0013427734375000, 0.0012207031250000, 0.0011291503906250,
+0.0013732910156250, 0.0016784667968750, 0.0012512207031250, 0.0026855468750000,
+0.0010070800781250, 0.0021972656250000, 0.0012207031250000, 0.0035705566406250,
+0.0008239746093750, 0.0035095214843750, 0.0009765625000000, 0.0037841796875000,
+0.0008850097656250, 0.0047912597656250, 0.0006713867187500, 0.0043334960937500,
+0.0009460449218750, 0.0050048828125000, 0.0005493164062500, 0.0051574707031250,
+0.0007629394531250, 0.0049743652343750, 0.0007019042968750, 0.0051574707031250,
+0.0003662109375000, 0.0051879882812500, 0.0007019042968750, 0.0046386718750000,
+0.0003051757812500},
+{0.0020446777343750, 0.0013427734375000, 0.0014648437500000, 0.0014343261718750,
+0.0017700195312500, 0.0012512207031250, 0.0014038085937500, 0.0010986328125000,
+0.0006408691406250, 0.0013122558593750, 0.0010986328125000, 0.0008544921875000,
+0.0003356933593750, 0.0007324218750000, 0.0001831054687500, 0.0008544921875000,
+0.0003662109375000, 0.0007019042968750, 0.0001831054687500, 0.0006408691406250,
+0.0001220703125000, 0.0010986328125000, 0.0004577636718750, 0.0008850097656250,
+0.0003356933593750, 0.0012207031250000, 0.0006103515625000, 0.0017395019531250,
+0.0010681152343750, 0.0014343261718750, 0.0008544921875000, 0.0021362304687500,
+0.0015563964843750, 0.0021667480468750, 0.0015869140625000, 0.0020446777343750,
+0.0017089843750000, 0.0024414062500000, 0.0021362304687500, 0.0024719238281250,
+0.0020446777343750, 0.0019226074218750, 0.0019836425781250, 0.0022888183593750,
+0.0021972656250000, 0.0017395019531250, 0.0019531250000000, 0.0009155273437500,
+0.0015869140625000, 0.0015258789062500, 0.0018310546875000, 0.0000915527343750,
+0.0013732910156250, 0.0001525878906250, 0.0013427734375000, 0.0001525878906250,
+0.0013122558593750, -0.0007019042968750, 0.0011291503906250, -0.0005493164062500,
+0.0012207031250000, -0.0001220703125000, 0.0009460449218750, -0.0006103515625000,
+0.0009460449218750, 0.0000305175781250, 0.0008544921875000, 0.0008544921875000,
+0.0007019042968750, 0.0001525878906250, 0.0007629394531250, 0.0019226074218750,
+0.0004882812500000, 0.0017089843750000, 0.0005187988281250, 0.0018920898437500,
+0.0004577636718750, 0.0031738281250000, 0.0002746582031250, 0.0027160644531250,
+0.0004577636718750, 0.0028686523437500, 0.0003356933593750, 0.0033874511718750,
+0.0003051757812500, 0.0028076171875000, 0.0001831054687500, 0.0023803710937500,
+0.0000915527343750, 0.0029296875000000, 0.0003051757812500, 0.0015869140625000,
+0.0000915527343750, 0.0017700195312500, 0.0001831054687500, 0.0015258789062500,
+0.0001525878906250, 0.0005187988281250, 0.0000610351562500, 0.0010070800781250,
+0.0002746582031250, 0.0003662109375000, 0.0000305175781250, 0.0001525878906250,
+0.0000915527343750, 0.0004577636718750, 0.0001831054687500, 0.0002441406250000,
+0.0000915527343750, 0.0001220703125000, 0.0001220703125000, 0.0010986328125000,
+0.0003051757812500, 0.0006408691406250, 0.0002441406250000, 0.0011901855468750,
+0.0003356933593750, 0.0022583007812500, 0.0004882812500000, 0.0014038085937500,
+0.0003967285156250, 0.0028991699218750, 0.0006713867187500, 0.0027465820312500,
+0.0006103515625000, 0.0028381347656250, 0.0007629394531250, 0.0037231445312500,
+0.0010375976562500, 0.0032958984375000, 0.0009460449218750, 0.0038146972656250,
+0.0011291503906250, 0.0038452148437500, 0.0012817382812500, 0.0037231445312500,
+0.0012817382812500, 0.0038757324218750, 0.0012817382812500, 0.0038452148437500,
+0.0014038085937500, 0.0035095214843750, 0.0012207031250000, 0.0036926269531250,
+0.0012512207031250, 0.0033569335937500, 0.0012817382812500, 0.0029907226562500,
+0.0011291503906250, 0.0032958984375000, 0.0011596679687500, 0.0025634765625000,
+0.0012512207031250, 0.0026855468750000, 0.0011901855468750, 0.0024719238281250,
+0.0011901855468750, 0.0019836425781250, 0.0013122558593750, 0.0023193359375000,
+0.0011901855468750, 0.0016174316406250, 0.0012817382812500, 0.0017089843750000,
+0.0013427734375000, 0.0015869140625000, 0.0013122558593750, 0.0010681152343750,
+0.0012512207031250, 0.0013427734375000, 0.0013427734375000, 0.0007934570312500,
+0.0011596679687500, 0.0008239746093750, 0.0012207031250000, 0.0007629394531250,
+0.0011596679687500, 0.0003967285156250, 0.0010070800781250, 0.0005493164062500,
+0.0010681152343750, 0.0003662109375000, 0.0009460449218750, 0.0003356933593750,
+0.0008850097656250, 0.0003356933593750, 0.0009460449218750, 0.0003051757812500,
+0.0008850097656250, 0.0003356933593750, 0.0008239746093750, 0.0003967285156250,
+0.0010375976562500, 0.0003967285156250, 0.0009460449218750, 0.0003356933593750,
+0.0010070800781250, 0.0003356933593750, 0.0011291503906250, 0.0003662109375000,
+0.0009765625000000, -0.0000610351562500, 0.0012817382812500, 0.0000915527343750,
+0.0011291503906250, -0.0002136230468750, 0.0012512207031250, -0.0007324218750000,
+0.0014953613281250, -0.0004577636718750, 0.0012207031250000, -0.0008544921875000,
+0.0015869140625000, -0.0010375976562500, 0.0016174316406250, -0.0008850097656250,
+0.0015258789062500, -0.0009460449218750, 0.0017395019531250, -0.0010681152343750,
+0.0017700195312500, -0.0005187988281250, 0.0015563964843750, -0.0007324218750000,
+0.0017395019531250, -0.0005187988281250, 0.0017089843750000, -0.0000610351562500,
+0.0015563964843750, -0.0003356933593750, 0.0018005371093750, 0.0001220703125000,
+0.0017395019531250, 0.0000610351562500, 0.0018615722656250, 0.0000610351562500,
+0.0020446777343750, 0.0003967285156250, 0.0020446777343750, 0.0002746582031250,
+0.0022277832031250, 0.0005187988281250, 0.0021972656250000, 0.0005187988281250,
+0.0022888183593750, 0.0003662109375000, 0.0021972656250000, 0.0003967285156250,
+0.0021667480468750, 0.0004272460937500, 0.0022583007812500, 0.0000915527343750,
+0.0021362304687500, 0.0001220703125000, 0.0021362304687500, -0.0000915527343750,
+0.0021667480468750, -0.0004577636718750, 0.0021362304687500, -0.0002746582031250,
+0.0022277832031250, -0.0008544921875000, 0.0021057128906250, -0.0008239746093750,
+0.0022277832031250, -0.0009155273437500, 0.0022277832031250, -0.0013122558593750,
+0.0020751953125000, -0.0012817382812500, 0.0021362304687500, -0.0011901855468750,
+0.0020751953125000, -0.0014343261718750, 0.0021057128906250, -0.0010375976562500,
+0.0020751953125000, -0.0007019042968750, 0.0020141601562500, -0.0010986328125000,
+0.0020751953125000, 0.0001525878906250, 0.0019531250000000, 0.0000000000000000,
+0.0019531250000000, 0.0004882812500000, 0.0019226074218750, 0.0015869140625000,
+0.0018005371093750, 0.0010681152343750, 0.0018920898437500, 0.0024414062500000,
+0.0018310546875000, 0.0024414062500000, 0.0018920898437500, 0.0026550292968750,
+0.0018310546875000, 0.0036621093750000, 0.0016784667968750, 0.0035400390625000,
+0.0016174316406250, 0.0035705566406250, 0.0014038085937500, 0.0036926269531250,
+0.0014648437500000, 0.0035095214843750, 0.0013427734375000, 0.0034179687500000,
+0.0010986328125000, 0.0035705566406250, 0.0010986328125000, 0.0028991699218750,
+0.0008850097656250, 0.0029602050781250, 0.0008544921875000, 0.0027465820312500,
+0.0007934570312500, 0.0021972656250000, 0.0006408691406250, 0.0023193359375000,
+0.0005493164062500, 0.0018920898437500, 0.0007629394531250, 0.0017700195312500,
+0.0005493164062500, 0.0018310546875000, 0.0007019042968750, 0.0017700195312500,
+0.0009765625000000, 0.0017395019531250, 0.0006408691406250, 0.0017089843750000,
+0.0013122558593750, 0.0018310546875000, 0.0011596679687500, 0.0017089843750000,
+0.0012207031250000, 0.0014953613281250, 0.0016479492187500, 0.0016479492187500,
+0.0013122558593750, 0.0015869140625000, 0.0017395019531250, 0.0014648437500000,
+0.0016174316406250, 0.0016174316406250, 0.0016784667968750, 0.0017700195312500,
+0.0018920898437500, 0.0015258789062500, 0.0017089843750000, 0.0019531250000000,
+0.0018005371093750, 0.0018005371093750, 0.0018005371093750, 0.0018310546875000,
+0.0017700195312500, 0.0021667480468750, 0.0018310546875000, 0.0019226074218750,
+0.0018005371093750, 0.0020446777343750, 0.0016479492187500, 0.0021972656250000,
+0.0017089843750000, 0.0020751953125000, 0.0016174316406250, 0.0020141601562500,
+0.0014038085937500, 0.0021057128906250, 0.0015258789062500, 0.0019226074218750,
+0.0014038085937500, 0.0019531250000000, 0.0013122558593750, 0.0018615722656250,
+0.0014343261718750, 0.0017700195312500, 0.0014953613281250, 0.0018310546875000,
+0.0014343261718750, 0.0013122558593750, 0.0016174316406250, 0.0014343261718750,
+0.0016174316406250, 0.0011901855468750, 0.0015258789062500, 0.0007934570312500,
+0.0014953613281250, 0.0010375976562500, 0.0015258789062500, 0.0003051757812500,
+0.0011291503906250, 0.0003967285156250, 0.0011596679687500, 0.0003662109375000,
+0.0010986328125000, -0.0001220703125000, 0.0007934570312500, 0.0000305175781250,
+0.0008544921875000, 0.0000305175781250, 0.0007019042968750, -0.0001525878906250,
+0.0006713867187500, -0.0000305175781250, 0.0006408691406250, 0.0000915527343750,
+0.0005798339843750, -0.0000610351562500, 0.0006103515625000, 0.0001220703125000,
+0.0006713867187500, 0.0000915527343750, 0.0006713867187500, 0.0000610351562500,
+0.0007934570312500, 0.0001831054687500, 0.0009765625000000, 0.0001220703125000,
+0.0009460449218750, -0.0000915527343750, 0.0011291503906250, 0.0000000000000000,
+0.0011901855468750, -0.0000915527343750, 0.0012512207031250, -0.0002136230468750,
+0.0014038085937500, 0.0000000000000000, 0.0013732910156250, -0.0001831054687500,
+0.0014648437500000, -0.0001220703125000, 0.0014648437500000, 0.0000610351562500,
+0.0015258789062500, 0.0001220703125000, 0.0015869140625000, 0.0001831054687500,
+0.0014648437500000, 0.0005493164062500, 0.0017395019531250, 0.0005798339843750,
+0.0015563964843750, 0.0007934570312500, 0.0016784667968750, 0.0011291503906250,
+0.0019226074218750, 0.0010986328125000, 0.0016174316406250, 0.0015258789062500,
+0.0021667480468750, 0.0016174316406250, 0.0020446777343750, 0.0016479492187500,
+0.0021362304687500, 0.0018920898437500, 0.0025024414062500, 0.0019531250000000,
+0.0022583007812500, 0.0018615722656250, 0.0025939941406250, 0.0020446777343750,
+0.0026245117187500, 0.0019531250000000, 0.0025634765625000, 0.0017700195312500,
+0.0027160644531250, 0.0020446777343750, 0.0026550292968750, 0.0016479492187500,
+0.0025939941406250, 0.0018310546875000, 0.0026245117187500, 0.0017395019531250,
+0.0025939941406250, 0.0014343261718750, 0.0025634765625000, 0.0017089843750000,
+0.0025634765625000, 0.0014038085937500, 0.0027160644531250, 0.0014648437500000,
+0.0026550292968750, 0.0014953613281250, 0.0027465820312500, 0.0014038085937500,
+0.0029907226562500, 0.0015258789062500, 0.0028076171875000, 0.0014953613281250,
+0.0030517578125000, 0.0015563964843750, 0.0030517578125000, 0.0015869140625000,
+0.0029602050781250, 0.0015563964843750, 0.0030822753906250, 0.0016174316406250,
+0.0030822753906250, 0.0015869140625000, 0.0027160644531250, 0.0016784667968750,
+0.0028686523437500, 0.0015869140625000, 0.0025329589843750, 0.0014648437500000,
+0.0021057128906250, 0.0016479492187500, 0.0023803710937500, 0.0012512207031250,
+0.0015258789062500, 0.0013122558593750, 0.0016784667968750, 0.0011901855468750,
+0.0013732910156250, 0.0008850097656250, 0.0006408691406250, 0.0010070800781250,
+0.0009765625000000, 0.0009155273437500, 0.0005187988281250, 0.0008239746093750,
+0.0003051757812500, 0.0009765625000000, 0.0004882812500000, 0.0010375976562500,
+0.0004577636718750, 0.0010375976562500, 0.0003051757812500, 0.0011901855468750,
+0.0006713867187500, 0.0013122558593750, 0.0007019042968750, 0.0013427734375000,
+0.0006103515625000, 0.0014953613281250, 0.0006713867187500, 0.0015563964843750,
+0.0007324218750000, 0.0014953613281250, 0.0004577636718750, 0.0016784667968750,
+0.0005493164062500, 0.0017089843750000, 0.0003967285156250, 0.0016174316406250,
+0.0001525878906250, 0.0018005371093750, 0.0002136230468750, 0.0017089843750000,
+0.0005187988281250, 0.0018310546875000, 0.0001831054687500, 0.0019226074218750,
+0.0006103515625000},
+{0.0022888183593750, -0.0003662109375000, 0.0023193359375000, -0.0005187988281250,
+0.0022583007812500, -0.0003662109375000, 0.0023193359375000, -0.0003967285156250,
+0.0022888183593750, -0.0003662109375000, 0.0022583007812500, -0.0001831054687500,
+0.0023803710937500, -0.0000915527343750, 0.0025024414062500, -0.0002136230468750,
+0.0024414062500000, 0.0003051757812500, 0.0026245117187500, 0.0001831054687500,
+0.0025939941406250, 0.0004882812500000, 0.0025939941406250, 0.0009765625000000,
+0.0027160644531250, 0.0006103515625000, 0.0027465820312500, 0.0014648437500000,
+0.0024719238281250, 0.0014648437500000, 0.0025939941406250, 0.0015869140625000,
+0.0024414062500000, 0.0021057128906250, 0.0022277832031250, 0.0019836425781250,
+0.0024719238281250, 0.0022583007812500, 0.0019836425781250, 0.0023193359375000,
+0.0021362304687500, 0.0024414062500000, 0.0020446777343750, 0.0025329589843750,
+0.0016784667968750, 0.0023193359375000, 0.0019836425781250, 0.0026550292968750,
+0.0016174316406250, 0.0025329589843750, 0.0016479492187500, 0.0025634765625000,
+0.0016174316406250, 0.0027160644531250, 0.0013427734375000, 0.0024414062500000,
+0.0015258789062500, 0.0028076171875000, 0.0011901855468750, 0.0026245117187500,
+0.0011901855468750, 0.0027465820312500, 0.0013122558593750, 0.0029907226562500,
+0.0011596679687500, 0.0025939941406250, 0.0011291503906250, 0.0030517578125000,
+0.0016174316406250, 0.0029296875000000, 0.0013427734375000, 0.0029296875000000,
+0.0017700195312500, 0.0032043457031250, 0.0024108886718750, 0.0028686523437500,
+0.0020446777343750, 0.0030517578125000, 0.0030822753906250, 0.0029907226562500,
+0.0030822753906250, 0.0028381347656250, 0.0032348632812500, 0.0028381347656250,
+0.0039367675781250, 0.0028381347656250, 0.0036010742187500, 0.0026855468750000,
+0.0035400390625000, 0.0026550292968750, 0.0038146972656250, 0.0025329589843750,
+0.0033264160156250, 0.0024414062500000, 0.0028381347656250, 0.0024719238281250,
+0.0031433105468750, 0.0022277832031250, 0.0020141601562500, 0.0022583007812500,
+0.0020751953125000, 0.0021972656250000, 0.0018005371093750, 0.0020446777343750,
+0.0009765625000000, 0.0021057128906250, 0.0012512207031250, 0.0021667480468750,
+0.0007934570312500, 0.0020751953125000, 0.0006103515625000, 0.0022277832031250,
+0.0008239746093750, 0.0023498535156250, 0.0007019042968750, 0.0022277832031250,
+0.0005798339843750, 0.0026855468750000, 0.0013122558593750, 0.0026550292968750,
+0.0010986328125000, 0.0026855468750000, 0.0014038085937500, 0.0030212402343750,
+0.0021057128906250, 0.0029296875000000, 0.0017395019531250, 0.0029296875000000,
+0.0021667480468750, 0.0030212402343750, 0.0022583007812500, 0.0028381347656250,
+0.0021667480468750, 0.0026855468750000, 0.0021972656250000, 0.0027770996093750,
+0.0021667480468750, 0.0022888183593750, 0.0022888183593750, 0.0022888183593750,
+0.0021362304687500, 0.0022277832031250, 0.0022888183593750, 0.0019226074218750,
+0.0025024414062500, 0.0018920898437500, 0.0023803710937500, 0.0019836425781250,
+0.0026855468750000, 0.0018005371093750, 0.0027160644531250, 0.0019226074218750,
+0.0025939941406250, 0.0021362304687500, 0.0026855468750000, 0.0018615722656250,
+0.0027770996093750, 0.0022583007812500, 0.0022888183593750, 0.0021667480468750,
+0.0024719238281250, 0.0021667480468750, 0.0023193359375000, 0.0024414062500000,
+0.0018920898437500, 0.0023193359375000, 0.0021667480468750, 0.0023803710937500,
+0.0019531250000000, 0.0024719238281250, 0.0018310546875000, 0.0024414062500000,
+0.0021362304687500, 0.0024719238281250, 0.0021972656250000, 0.0025329589843750,
+0.0021362304687500, 0.0025939941406250, 0.0027465820312500, 0.0027160644531250,
+0.0027160644531250, 0.0025939941406250, 0.0027160644531250, 0.0025634765625000,
+0.0030212402343750, 0.0027465820312500, 0.0030212402343750, 0.0022277832031250,
+0.0027465820312500, 0.0024108886718750, 0.0029602050781250, 0.0021972656250000,
+0.0026550292968750, 0.0017395019531250, 0.0022888183593750, 0.0019836425781250,
+0.0025939941406250, 0.0013122558593750, 0.0018920898437500, 0.0012817382812500,
+0.0019531250000000, 0.0010986328125000, 0.0019226074218750, 0.0006103515625000,
+0.0014953613281250, 0.0007629394531250, 0.0017700195312500, 0.0003356933593750,
+0.0015869140625000, 0.0001831054687500, 0.0015258789062500, 0.0003051757812500,
+0.0017089843750000, 0.0001220703125000, 0.0017089843750000, 0.0000915527343750,
+0.0018310546875000, 0.0005187988281250, 0.0019836425781250, 0.0003051757812500,
+0.0020751953125000, 0.0005493164062500, 0.0020446777343750, 0.0010070800781250,
+0.0021057128906250, 0.0006713867187500, 0.0021667480468750, 0.0012207031250000,
+0.0020141601562500, 0.0012207031250000, 0.0021057128906250, 0.0011596679687500,
+0.0020751953125000, 0.0014343261718750, 0.0018920898437500, 0.0012512207031250,
+0.0020141601562500, 0.0013122558593750, 0.0014953613281250, 0.0012512207031250,
+0.0015563964843750, 0.0011596679687500, 0.0014953613281250, 0.0011596679687500,
+0.0010070800781250, 0.0010986328125000, 0.0010986328125000, 0.0011291503906250,
+0.0008850097656250, 0.0010070800781250, 0.0008850097656250, 0.0010681152343750,
+0.0005798339843750, 0.0011596679687500, 0.0003051757812500, 0.0010986328125000,
+0.0005187988281250, 0.0014038085937500, -0.0004272460937500, 0.0014038085937500,
+-0.0003662109375000, 0.0014648437500000, -0.0003356933593750, 0.0017395019531250,
+-0.0011596679687500, 0.0023803710937500, 0.0016479492187500, 0.0016174316406250,
+0.0006103515625000, 0.0020141601562500, 0.0009765625000000, 0.0018005371093750,
+0.0005493164062500, 0.0014343261718750, -0.0002136230468750, 0.0017089843750000,
+0.0001220703125000, 0.0011596679687500, -0.0003051757812500, 0.0013732910156250,
+-0.0005187988281250, 0.0013122558593750, -0.0000915527343750, 0.0008850097656250,
+-0.0000610351562500, 0.0012512207031250, -0.0003662109375000, 0.0005798339843750,
+0.0008850097656250, 0.0007019042968750, 0.0004272460937500, 0.0006103515625000,
+0.0009765625000000, 0.0001525878906250, 0.0021667480468750, 0.0004882812500000,
+0.0013732910156250, 0.0003051757812500, 0.0027465820312500, 0.0002746582031250,
+0.0027465820312500, 0.0003662109375000, 0.0027160644531250, 0.0003967285156250,
+0.0033264160156250, 0.0004577636718750, 0.0030212402343750, 0.0003662109375000,
+0.0028381347656250, 0.0005798339843750, 0.0031433105468750, 0.0004577636718750,
+0.0025939941406250, 0.0002136230468750, 0.0020141601562500, 0.0005493164062500,
+0.0024108886718750, 0.0003967285156250, 0.0015258789062500, 0.0003356933593750,
+0.0014648437500000, 0.0006713867187500, 0.0014648437500000, 0.0007324218750000,
+0.0010375976562500, 0.0007629394531250, 0.0010681152343750, 0.0016174316406250,
+0.0010070800781250, 0.0014038085937500, 0.0008850097656250, 0.0018920898437500,
+0.0009765625000000, 0.0028686523437500, 0.0010375976562500, 0.0023803710937500,
+0.0008544921875000, 0.0032043457031250, 0.0010375976562500, 0.0033874511718750,
+0.0010681152343750, 0.0031738281250000, 0.0009765625000000, 0.0034790039062500,
+0.0011291503906250, 0.0034790039062500, 0.0012207031250000, 0.0029602050781250,
+0.0008239746093750, 0.0031127929687500, 0.0011291503906250, 0.0028991699218750,
+0.0007629394531250, 0.0023193359375000, 0.0002441406250000, 0.0025634765625000,
+0.0007019042968750, 0.0024719238281250, -0.0002746582031250, 0.0021667480468750,
+-0.0002746582031250, 0.0025939941406250, -0.0003051757812500, 0.0028381347656250,
+-0.0009460449218750, 0.0024719238281250, -0.0005798339843750, 0.0037536621093750,
+-0.0004882812500000, 0.0034179687500000, -0.0008239746093750, 0.0038146972656250,
+-0.0002746582031250, 0.0049743652343750, 0.0003356933593750, 0.0043029785156250,
+-0.0000305175781250, 0.0054626464843750, 0.0009155273437500, 0.0054931640625000,
+0.0008850097656250, 0.0052185058593750, 0.0010986328125000, 0.0056457519531250,
+0.0017700195312500, 0.0055541992187500, 0.0015869140625000, 0.0048828125000000,
+0.0019531250000000, 0.0051574707031250, 0.0020141601562500, 0.0046691894531250,
+0.0020141601562500, 0.0039062500000000, 0.0020446777343750, 0.0042419433593750,
+0.0020141601562500, 0.0034179687500000, 0.0018920898437500, 0.0031738281250000,
+0.0018005371093750, 0.0033264160156250, 0.0018005371093750, 0.0029602050781250,
+0.0017089843750000, 0.0028381347656250, 0.0015563964843750, 0.0035705566406250,
+0.0017700195312500, 0.0032043457031250, 0.0016174316406250, 0.0035095214843750,
+0.0017395019531250, 0.0042724609375000, 0.0020446777343750, 0.0036926269531250,
+0.0017089843750000, 0.0046691894531250, 0.0021362304687500, 0.0046691894531250,
+0.0021057128906250, 0.0045471191406250, 0.0019836425781250, 0.0049743652343750,
+0.0021667480468750, 0.0047912597656250, 0.0021057128906250, 0.0043640136718750,
+0.0020141601562500, 0.0045471191406250, 0.0021362304687500, 0.0040283203125000,
+0.0020141601562500, 0.0033874511718750, 0.0018615722656250, 0.0037841796875000,
+0.0020141601562500, 0.0024414062500000, 0.0016784667968750, 0.0026550292968750,
+0.0016174316406250, 0.0022888183593750, 0.0015869140625000, 0.0013122558593750,
+0.0013427734375000, 0.0018615722656250, 0.0012817382812500, 0.0011596679687500,
+0.0013122558593750, 0.0010681152343750, 0.0011596679687500, 0.0011291503906250,
+0.0010681152343750, 0.0008850097656250, 0.0010986328125000, 0.0010375976562500,
+0.0008850097656250, 0.0008239746093750, 0.0009765625000000, 0.0008850097656250,
+0.0009155273437500, 0.0008850097656250, 0.0008239746093750, 0.0007019042968750,
+0.0008850097656250, 0.0008239746093750, 0.0007629394531250, 0.0007629394531250,
+0.0008544921875000, 0.0007019042968750, 0.0008850097656250, 0.0007629394531250,
+0.0008850097656250, 0.0007934570312500, 0.0009460449218750, 0.0007324218750000,
+0.0010375976562500, 0.0007629394531250, 0.0010070800781250, 0.0007629394531250,
+0.0011291503906250, 0.0007324218750000, 0.0011291503906250, 0.0006408691406250,
+0.0010986328125000, 0.0006408691406250, 0.0012207031250000, 0.0005493164062500,
+0.0011901855468750, 0.0004882812500000, 0.0012207031250000, 0.0005493164062500,
+0.0013732910156250, 0.0006103515625000, 0.0013122558593750, 0.0005493164062500,
+0.0013427734375000, 0.0008239746093750, 0.0014953613281250, 0.0007629394531250,
+0.0015869140625000, 0.0008544921875000, 0.0015869140625000, 0.0010986328125000,
+0.0017395019531250, 0.0009765625000000, 0.0017700195312500, 0.0012817382812500,
+0.0017089843750000, 0.0012817382812500, 0.0018920898437500, 0.0013427734375000,
+0.0018615722656250, 0.0015869140625000, 0.0017700195312500, 0.0015258789062500,
+0.0019226074218750, 0.0016174316406250, 0.0016479492187500, 0.0016784667968750,
+0.0018005371093750, 0.0016784667968750, 0.0017089843750000, 0.0017089843750000,
+0.0014343261718750, 0.0017089843750000, 0.0017089843750000, 0.0019226074218750,
+0.0012512207031250, 0.0018310546875000, 0.0014038085937500, 0.0019836425781250,
+0.0012207031250000, 0.0022888183593750, 0.0008544921875000, 0.0021667480468750,
+0.0012207031250000, 0.0025024414062500, 0.0007019042968750, 0.0025939941406250,
+0.0008239746093750, 0.0025024414062500, 0.0009460449218750, 0.0025939941406250,
+0.0007629394531250, 0.0026550292968750, 0.0009765625000000, 0.0023498535156250,
+0.0011291503906250, 0.0023498535156250, 0.0010681152343750, 0.0021972656250000,
+0.0013427734375000, 0.0019836425781250, 0.0015563964843750, 0.0020141601562500,
+0.0014953613281250},
+{0.0015258789062500, 0.0014953613281250, 0.0016479492187500, 0.0024108886718750,
+0.0014038085937500, 0.0020751953125000, 0.0015869140625000, 0.0025634765625000,
+0.0014648437500000, 0.0035400390625000, 0.0012512207031250, 0.0030212402343750,
+0.0014648437500000, 0.0041198730468750, 0.0011901855468750, 0.0042419433593750,
+0.0010986328125000, 0.0042419433593750, 0.0011901855468750, 0.0048217773437500,
+0.0010070800781250, 0.0047302246093750, 0.0010986328125000, 0.0045166015625000,
+0.0013427734375000, 0.0049133300781250, 0.0011596679687500, 0.0046386718750000,
+0.0012512207031250, 0.0041809082031250, 0.0015563964843750, 0.0046081542968750,
+0.0013427734375000, 0.0036010742187500, 0.0014953613281250, 0.0037841796875000,
+0.0015258789062500, 0.0035705566406250, 0.0014343261718750, 0.0028381347656250,
+0.0014038085937500, 0.0032043457031250, 0.0014648437500000, 0.0025024414062500,
+0.0012817382812500, 0.0024719238281250, 0.0013427734375000, 0.0023498535156250,
+0.0012817382812500, 0.0019531250000000, 0.0010681152343750, 0.0020446777343750,
+0.0011291503906250, 0.0015869140625000, 0.0010070800781250, 0.0015869140625000,
+0.0009765625000000, 0.0013732910156250, 0.0009155273437500, 0.0010070800781250,
+0.0008850097656250, 0.0010681152343750, 0.0008850097656250, 0.0009155273437500,
+0.0008239746093750, 0.0007324218750000, 0.0008850097656250, 0.0008544921875000,
+0.0008239746093750, 0.0009155273437500, 0.0006408691406250, 0.0007324218750000,
+0.0007324218750000, 0.0011596679687500, 0.0005798339843750, 0.0010986328125000,
+0.0006103515625000, 0.0011901855468750, 0.0006103515625000, 0.0014953613281250,
+0.0004882812500000, 0.0013122558593750, 0.0006103515625000, 0.0015258789062500,
+0.0005187988281250, 0.0015869140625000, 0.0005187988281250, 0.0014953613281250,
+0.0005798339843750, 0.0014953613281250, 0.0006408691406250, 0.0014953613281250,
+0.0006103515625000, 0.0012512207031250, 0.0009155273437500, 0.0012817382812500,
+0.0009155273437500, 0.0012512207031250, 0.0010070800781250, 0.0010681152343750,
+0.0011596679687500, 0.0011291503906250, 0.0011291503906250, 0.0011596679687500,
+0.0012817382812500, 0.0010375976562500, 0.0013427734375000, 0.0012512207031250,
+0.0013122558593750, 0.0014343261718750, 0.0012817382812500, 0.0012817382812500,
+0.0013427734375000, 0.0017089843750000, 0.0013427734375000, 0.0017089843750000,
+0.0013427734375000, 0.0017700195312500, 0.0014343261718750, 0.0021972656250000,
+0.0014343261718750, 0.0021362304687500, 0.0014038085937500, 0.0023193359375000,
+0.0016479492187500, 0.0024414062500000, 0.0016174316406250, 0.0023498535156250,
+0.0017089843750000, 0.0023803710937500, 0.0018310546875000, 0.0024719238281250,
+0.0017700195312500, 0.0022583007812500, 0.0019226074218750, 0.0022277832031250,
+0.0019226074218750, 0.0022888183593750, 0.0018615722656250, 0.0020751953125000,
+0.0018615722656250, 0.0021057128906250, 0.0018920898437500, 0.0023803710937500,
+0.0014648437500000, 0.0021362304687500, 0.0016784667968750, 0.0023803710937500,
+0.0014343261718750, 0.0027160644531250, 0.0009460449218750, 0.0023193359375000,
+0.0013122558593750, 0.0030212402343750, 0.0006408691406250, 0.0028991699218750,
+0.0007629394531250, 0.0029602050781250, 0.0007324218750000, 0.0034179687500000,
+0.0003356933593750, 0.0031738281250000, 0.0006103515625000, 0.0033569335937500,
+0.0003051757812500, 0.0034484863281250, 0.0004272460937500, 0.0033264160156250,
+0.0006408691406250, 0.0032348632812500, 0.0004882812500000, 0.0032348632812500,
+0.0007019042968750, 0.0032043457031250, 0.0009765625000000, 0.0029907226562500,
+0.0008239746093750, 0.0030517578125000, 0.0012817382812500, 0.0032348632812500,
+0.0017395019531250, 0.0029907226562500, 0.0013122558593750, 0.0034179687500000,
+0.0023498535156250, 0.0034484863281250, 0.0021362304687500, 0.0032653808593750,
+0.0022277832031250, 0.0034484863281250, 0.0029602050781250, 0.0034790039062500,
+0.0024719238281250, 0.0029907226562500, 0.0026550292968750, 0.0032043457031250,
+0.0028076171875000, 0.0028686523437500, 0.0026245117187500, 0.0024108886718750,
+0.0025024414062500, 0.0027770996093750, 0.0025024414062500, 0.0019226074218750,
+0.0021362304687500, 0.0019531250000000, 0.0021972656250000, 0.0017700195312500,
+0.0019531250000000, 0.0012817382812500, 0.0016479492187500, 0.0015563964843750,
+0.0018310546875000, 0.0010375976562500, 0.0013122558593750, 0.0010986328125000,
+0.0013427734375000, 0.0010681152343750, 0.0012817382812500, 0.0007324218750000,
+0.0010375976562500, 0.0008544921875000, 0.0011291503906250, 0.0008850097656250,
+0.0010986328125000, 0.0007324218750000, 0.0010681152343750, 0.0007019042968750,
+0.0011596679687500, 0.0010070800781250, 0.0011901855468750, 0.0012817382812500,
+0.0011901855468750, 0.0009155273437500, 0.0015563964843750, 0.0012207031250000,
+0.0014038085937500, 0.0009765625000000, 0.0015563964843750, 0.0005493164062500,
+0.0018310546875000, 0.0010375976562500, 0.0015563964843750, -0.0001220703125000,
+0.0020751953125000, 0.0000915527343750, 0.0020141601562500, -0.0002746582031250,
+0.0021362304687500, -0.0011596679687500, 0.0024414062500000, -0.0005798339843750,
+0.0021667480468750, -0.0014343261718750, 0.0024108886718750, -0.0015258789062500,
+0.0023498535156250, -0.0011596679687500, 0.0022583007812500, -0.0014038085937500,
+0.0022888183593750, -0.0016174316406250, 0.0021362304687500, -0.0004577636718750,
+0.0021972656250000, -0.0010375976562500, 0.0020751953125000, -0.0003967285156250,
+0.0020446777343750, 0.0007934570312500, 0.0020446777343750, -0.0001831054687500,
+0.0018920898437500, 0.0017089843750000, 0.0020141601562500, 0.0016784667968750,
+0.0017700195312500, 0.0017700195312500, 0.0019226074218750, 0.0029602050781250,
+0.0021667480468750, 0.0025024414062500, 0.0019226074218750, 0.0027770996093750,
+0.0021972656250000, 0.0030517578125000, 0.0021667480468750, 0.0028076171875000,
+0.0021362304687500, 0.0027465820312500, 0.0022583007812500, 0.0029907226562500,
+0.0021362304687500, 0.0021057128906250, 0.0021057128906250, 0.0023803710937500,
+0.0021667480468750, 0.0020751953125000, 0.0020751953125000, 0.0013427734375000,
+0.0019531250000000, 0.0018920898437500, 0.0019531250000000, 0.0013427734375000,
+0.0019836425781250, 0.0012512207031250, 0.0018310546875000, 0.0015869140625000,
+0.0019226074218750, 0.0015258789062500, 0.0019836425781250, 0.0014343261718750,
+0.0018920898437500, 0.0020751953125000, 0.0019836425781250, 0.0019531250000000,
+0.0020446777343750, 0.0021362304687500, 0.0019836425781250, 0.0026855468750000,
+0.0019836425781250, 0.0023803710937500, 0.0019836425781250, 0.0026550292968750,
+0.0018615722656250, 0.0027770996093750, 0.0019531250000000, 0.0025329589843750,
+0.0018920898437500, 0.0024719238281250, 0.0018005371093750, 0.0025634765625000,
+0.0018005371093750, 0.0019836425781250, 0.0015258789062500, 0.0020446777343750,
+0.0015869140625000, 0.0019226074218750, 0.0014343261718750, 0.0016174316406250,
+0.0011596679687500, 0.0017089843750000, 0.0013427734375000, 0.0014038085937500,
+0.0010375976562500, 0.0014648437500000, 0.0010375976562500, 0.0014038085937500,
+0.0010986328125000, 0.0012207031250000, 0.0010375976562500, 0.0014038085937500,
+0.0010375976562500, 0.0009765625000000, 0.0012207031250000, 0.0010986328125000,
+0.0011291503906250, 0.0010681152343750, 0.0012817382812500, 0.0008544921875000,
+0.0014343261718750, 0.0010681152343750, 0.0012512207031250, 0.0007019042968750,
+0.0017700195312500, 0.0008239746093750, 0.0015563964843750, 0.0008239746093750,
+0.0018615722656250, 0.0007019042968750, 0.0023193359375000, 0.0008850097656250,
+0.0019531250000000, 0.0007019042968750, 0.0030212402343750, 0.0008239746093750,
+0.0027770996093750, 0.0008239746093750, 0.0031127929687500, 0.0006103515625000,
+0.0039978027343750, 0.0008544921875000, 0.0035095214843750, 0.0005798339843750,
+0.0045471191406250, 0.0006408691406250, 0.0046081542968750, 0.0006408691406250,
+0.0044555664062500, 0.0004882812500000, 0.0048828125000000, 0.0006713867187500,
+0.0048828125000000, 0.0004577636718750, 0.0043640136718750, 0.0005798339843750,
+0.0046386718750000, 0.0005187988281250, 0.0040588378906250, 0.0003051757812500,
+0.0034484863281250, 0.0004577636718750, 0.0038757324218750, 0.0003356933593750,
+0.0024414062500000, 0.0001831054687500, 0.0024719238281250, 0.0003967285156250,
+0.0021667480468750, 0.0004272460937500, 0.0012207031250000, 0.0002441406250000,
+0.0015563964843750, 0.0007324218750000, 0.0010070800781250, 0.0005493164062500,
+0.0007629394531250, 0.0006408691406250, 0.0009765625000000, 0.0010070800781250,
+0.0009460449218750, 0.0007019042968750, 0.0008239746093750, 0.0011291503906250,
+0.0016174316406250, 0.0010681152343750, 0.0014343261718750, 0.0009765625000000,
+0.0018310546875000, 0.0012207031250000, 0.0025634765625000, 0.0010986328125000,
+0.0021972656250000, 0.0011291503906250, 0.0031738281250000, 0.0011291503906250,
+0.0031738281250000, 0.0010375976562500, 0.0032653808593750, 0.0010375976562500,
+0.0038757324218750, 0.0010681152343750, 0.0036926269531250, 0.0010986328125000,
+0.0038146972656250, 0.0010375976562500, 0.0040588378906250, 0.0011596679687500,
+0.0036926269531250, 0.0011901855468750, 0.0034484863281250, 0.0011291503906250,
+0.0037231445312500, 0.0015258789062500, 0.0029296875000000, 0.0014343261718750,
+0.0029296875000000, 0.0014953613281250, 0.0028686523437500, 0.0018005371093750,
+0.0023803710937500, 0.0016174316406250, 0.0025329589843750, 0.0018920898437500,
+0.0022888183593750, 0.0018920898437500, 0.0022888183593750, 0.0018615722656250,
+0.0022583007812500, 0.0019836425781250, 0.0021667480468750, 0.0019226074218750,
+0.0022277832031250, 0.0019836425781250, 0.0020751953125000, 0.0019836425781250,
+0.0020751953125000, 0.0019836425781250, 0.0020446777343750, 0.0019836425781250,
+0.0019836425781250, 0.0019836425781250, 0.0019531250000000, 0.0020446777343750,
+0.0018920898437500, 0.0019836425781250, 0.0018615722656250, 0.0019836425781250,
+0.0018920898437500, 0.0020141601562500, 0.0018005371093750, 0.0018920898437500,
+0.0018005371093750, 0.0018005371093750, 0.0018005371093750, 0.0018310546875000,
+0.0016784667968750, 0.0016784667968750, 0.0017395019531250, 0.0014953613281250,
+0.0018005371093750, 0.0015563964843750, 0.0016784667968750, 0.0011291503906250,
+0.0016479492187500, 0.0010375976562500, 0.0016479492187500, 0.0009765625000000,
+0.0015258789062500, 0.0006713867187500, 0.0014343261718750, 0.0006408691406250,
+0.0014648437500000, 0.0005798339843750, 0.0014648437500000, 0.0003967285156250,
+0.0013732910156250, 0.0004882812500000, 0.0015563964843750, 0.0005493164062500,
+0.0017089843750000, 0.0003356933593750, 0.0016174316406250, 0.0008544921875000,
+0.0021362304687500, 0.0006713867187500, 0.0021972656250000, 0.0008239746093750,
+0.0022277832031250, 0.0013122558593750, 0.0025634765625000, 0.0010070800781250,
+0.0025939941406250, 0.0015869140625000, 0.0023803710937500, 0.0016479492187500,
+0.0026550292968750, 0.0016479492187500, 0.0023803710937500, 0.0018310546875000,
+0.0019531250000000, 0.0018005371093750, 0.0022583007812500, 0.0016784667968750,
+0.0015563964843750, 0.0017089843750000, 0.0016174316406250, 0.0016174316406250,
+0.0016479492187500, 0.0014343261718750, 0.0012817382812500, 0.0014648437500000,
+0.0014038085937500, 0.0014038085937500, 0.0011901855468750, 0.0012207031250000,
+0.0011596679687500},
+{0.0006713867187500, -0.0003967285156250, 0.0003051757812500, -0.0003051757812500,
+0.0009155273437500, -0.0003662109375000, 0.0018005371093750, -0.0002441406250000,
+0.0012817382812500, -0.0003051757812500, 0.0028381347656250, -0.0003051757812500,
+0.0028686523437500, -0.0001220703125000, 0.0031433105468750, -0.0001220703125000,
+0.0042114257812500, -0.0002441406250000, 0.0039672851562500, 0.0000915527343750,
+0.0043029785156250, 0.0000305175781250, 0.0046081542968750, 0.0001525878906250,
+0.0042724609375000, 0.0004577636718750, 0.0041503906250000, 0.0003356933593750,
+0.0043945312500000, 0.0007019042968750, 0.0034179687500000, 0.0006713867187500,
+0.0036926269531250, 0.0006713867187500, 0.0033264160156250, 0.0009460449218750,
+0.0024719238281250, 0.0009460449218750, 0.0028991699218750, 0.0008850097656250,
+0.0019226074218750, 0.0010070800781250, 0.0019531250000000, 0.0008850097656250,
+0.0018615722656250, 0.0007324218750000, 0.0013427734375000, 0.0008850097656250,
+0.0014953613281250, 0.0006713867187500, 0.0015869140625000, 0.0006713867187500,
+0.0014648437500000, 0.0007629394531250, 0.0016784667968750, 0.0006103515625000,
+0.0019531250000000, 0.0005798339843750, 0.0018310546875000, 0.0007629394531250,
+0.0021057128906250, 0.0006408691406250, 0.0021972656250000, 0.0007629394531250,
+0.0020141601562500, 0.0009155273437500, 0.0019836425781250, 0.0007019042968750,
+0.0020751953125000, 0.0010986328125000, 0.0018310546875000, 0.0009765625000000,
+0.0018005371093750, 0.0010681152343750, 0.0017700195312500, 0.0013122558593750,
+0.0016479492187500, 0.0010681152343750, 0.0016784667968750, 0.0014953613281250,
+0.0016174316406250, 0.0014038085937500, 0.0015869140625000, 0.0014343261718750,
+0.0016174316406250, 0.0017395019531250, 0.0016784667968750, 0.0015258789062500,
+0.0016174316406250, 0.0016174316406250, 0.0018310546875000, 0.0016174316406250,
+0.0017700195312500, 0.0014953613281250, 0.0018005371093750, 0.0014343261718750,
+0.0018920898437500, 0.0014038085937500, 0.0019226074218750, 0.0013427734375000,
+0.0019226074218750, 0.0011291503906250, 0.0018615722656250, 0.0012512207031250,
+0.0019226074218750, 0.0012817382812500, 0.0019531250000000, 0.0010375976562500,
+0.0018920898437500, 0.0014038085937500, 0.0019836425781250, 0.0012512207031250,
+0.0020141601562500, 0.0012207031250000, 0.0018920898437500, 0.0014038085937500,
+0.0018920898437500, 0.0010986328125000, 0.0019531250000000, 0.0012512207031250,
+0.0015563964843750, 0.0011291503906250, 0.0016784667968750, 0.0010681152343750,
+0.0015563964843750, 0.0011291503906250, 0.0011596679687500, 0.0008850097656250,
+0.0012817382812500, 0.0007934570312500, 0.0013122558593750, 0.0007324218750000,
+0.0010986328125000, 0.0006103515625000, 0.0013122558593750, 0.0004577636718750,
+0.0015869140625000, 0.0003356933593750, 0.0014038085937500, 0.0004577636718750,
+0.0018615722656250, 0.0001525878906250, 0.0018920898437500, 0.0003967285156250,
+0.0019531250000000, 0.0007324218750000, 0.0021972656250000, 0.0003662109375000,
+0.0021057128906250, 0.0014038085937500, 0.0021972656250000, 0.0012207031250000,
+0.0022277832031250, 0.0016479492187500, 0.0020751953125000, 0.0025024414062500,
+0.0020141601562500, 0.0019836425781250, 0.0021362304687500, 0.0027770996093750,
+0.0020751953125000, 0.0028991699218750, 0.0019226074218750, 0.0027465820312500,
+0.0021667480468750, 0.0030517578125000, 0.0023193359375000, 0.0030212402343750,
+0.0021362304687500, 0.0026855468750000, 0.0026550292968750, 0.0029296875000000,
+0.0026855468750000, 0.0026550292968750, 0.0025024414062500, 0.0022277832031250,
+0.0027770996093750, 0.0025634765625000, 0.0028991699218750, 0.0018005371093750,
+0.0020446777343750, 0.0019226074218750, 0.0023803710937500, 0.0017700195312500,
+0.0020141601562500, 0.0012817382812500, 0.0012207031250000, 0.0015563964843750,
+0.0017089843750000, 0.0012207031250000, 0.0006713867187500, 0.0012207031250000,
+0.0007934570312500, 0.0013122558593750, 0.0006103515625000, 0.0011901855468750,
+-0.0004577636718750, 0.0021972656250000, 0.0027770996093750, 0.0020141601562500,
+0.0031433105468750, 0.0021057128906250, 0.0036010742187500, 0.0020141601562500,
+0.0029296875000000, 0.0018615722656250, 0.0025634765625000, 0.0019226074218750,
+0.0031127929687500, 0.0015869140625000, 0.0014038085937500, 0.0015563964843750,
+0.0017395019531250, 0.0016174316406250, 0.0014343261718750, 0.0013732910156250,
+0.0001220703125000, 0.0013427734375000, 0.0007324218750000, 0.0018005371093750,
+0.0000000000000000, 0.0015258789062500, -0.0001831054687500, 0.0020141601562500,
+0.0000000000000000, 0.0026550292968750, -0.0002136230468750, 0.0021972656250000,
+-0.0002746582031250, 0.0034179687500000, 0.0001831054687500, 0.0032958984375000,
+0.0000000000000000, 0.0033874511718750, 0.0003662109375000, 0.0040893554687500,
+0.0008850097656250, 0.0037536621093750, 0.0003967285156250, 0.0040283203125000,
+0.0013122558593750, 0.0039978027343750, 0.0014343261718750, 0.0036926269531250,
+0.0012817382812500, 0.0036926269531250, 0.0016479492187500, 0.0035705566406250,
+0.0017089843750000, 0.0031738281250000, 0.0011901855468750, 0.0031127929687500,
+0.0014038085937500, 0.0028686523437500, 0.0012817382812500, 0.0025329589843750,
+0.0007629394531250, 0.0025329589843750, 0.0009765625000000, 0.0024414062500000,
+0.0013122558593750, 0.0022583007812500, 0.0010375976562500, 0.0023498535156250,
+0.0015258789062500, 0.0024414062500000, 0.0021667480468750, 0.0022583007812500,
+0.0017395019531250, 0.0025634765625000, 0.0025634765625000, 0.0024108886718750,
+0.0025024414062500, 0.0024108886718750, 0.0025329589843750, 0.0025939941406250,
+0.0028991699218750, 0.0022583007812500, 0.0025634765625000, 0.0021667480468750,
+0.0025939941406250, 0.0022277832031250, 0.0026855468750000, 0.0018615722656250,
+0.0022888183593750, 0.0014953613281250, 0.0019531250000000, 0.0016174316406250,
+0.0020751953125000, 0.0010070800781250, 0.0012512207031250, 0.0009765625000000,
+0.0012512207031250, 0.0010070800781250, 0.0009765625000000, 0.0006713867187500,
+0.0003662109375000, 0.0007324218750000, 0.0005187988281250, 0.0012817382812500,
+0.0002136230468750, 0.0009155273437500, -0.0000915527343750, 0.0014038085937500,
+0.0000915527343750, 0.0022583007812500, 0.0001525878906250, 0.0017395019531250,
+-0.0001220703125000, 0.0029296875000000, 0.0002746582031250, 0.0028991699218750,
+0.0000305175781250, 0.0029602050781250, 0.0000305175781250, 0.0036315917968750,
+0.0002441406250000, 0.0032653808593750, 0.0000305175781250, 0.0033874511718750,
+-0.0001831054687500, 0.0035705566406250, -0.0000610351562500, 0.0032348632812500,
+-0.0003662109375000, 0.0029602050781250, -0.0007324218750000, 0.0032043457031250,
+-0.0004882812500000, 0.0024108886718750, -0.0008850097656250, 0.0025024414062500,
+-0.0010375976562500, 0.0023498535156250, -0.0009460449218750, 0.0017395019531250,
+-0.0010681152343750, 0.0020751953125000, -0.0011596679687500, 0.0014953613281250,
+-0.0008544921875000, 0.0014648437500000, -0.0010070800781250, 0.0013427734375000,
+-0.0009460449218750, 0.0010375976562500, -0.0006713867187500, 0.0010681152343750,
+-0.0008850097656250, 0.0007324218750000, -0.0005798339843750, 0.0007324218750000,
+-0.0006408691406250, 0.0005493164062500, -0.0006408691406250, 0.0002441406250000,
+-0.0004882812500000, 0.0004882812500000, -0.0007019042968750, -0.0000610351562500,
+-0.0004882812500000, -0.0000305175781250, -0.0006408691406250, -0.0001525878906250,
+-0.0005798339843750, -0.0005493164062500, -0.0003967285156250, -0.0003051757812500,
+-0.0006103515625000, -0.0006103515625000, -0.0001831054687500, -0.0007629394531250,
+-0.0002746582031250, -0.0004577636718750, -0.0002136230468750, -0.0004882812500000,
+0.0000915527343750, -0.0006713867187500, -0.0000305175781250, 0.0003356933593750,
+0.0003967285156250, 0.0000610351562500, 0.0003051757812500, 0.0004882812500000,
+0.0004272460937500, 0.0013732910156250, 0.0007934570312500, 0.0008544921875000,
+0.0006713867187500, 0.0018615722656250, 0.0010375976562500, 0.0019226074218750,
+0.0010375976562500, 0.0019531250000000, 0.0011596679687500, 0.0025024414062500,
+0.0014038085937500, 0.0022277832031250, 0.0012512207031250, 0.0024719238281250,
+0.0016174316406250, 0.0025939941406250, 0.0015869140625000, 0.0023498535156250,
+0.0016174316406250, 0.0022888183593750, 0.0018615722656250, 0.0023498535156250,
+0.0017700195312500, 0.0018920898437500, 0.0018615722656250, 0.0019836425781250,
+0.0018615722656250, 0.0017700195312500, 0.0018005371093750, 0.0013732910156250,
+0.0018005371093750, 0.0016479492187500, 0.0017395019531250, 0.0012817382812500,
+0.0017089843750000, 0.0012207031250000, 0.0016479492187500, 0.0012512207031250,
+0.0016174316406250, 0.0010986328125000, 0.0016174316406250, 0.0011596679687500,
+0.0015258789062500, 0.0013122558593750, 0.0015869140625000, 0.0012817382812500,
+0.0015563964843750, 0.0013427734375000, 0.0014648437500000, 0.0014343261718750,
+0.0014648437500000, 0.0013427734375000, 0.0014648437500000, 0.0014038085937500,
+0.0014038085937500, 0.0013732910156250, 0.0013732910156250, 0.0014038085937500,
+0.0012817382812500, 0.0013122558593750, 0.0012207031250000, 0.0011901855468750,
+0.0012512207031250, 0.0013122558593750, 0.0010986328125000, 0.0011901855468750,
+0.0011291503906250, 0.0012207031250000, 0.0010070800781250, 0.0013122558593750,
+0.0007934570312500, 0.0011291503906250, 0.0009765625000000, 0.0013122558593750,
+0.0006408691406250, 0.0013122558593750, 0.0006103515625000, 0.0011901855468750,
+0.0005493164062500, 0.0013122558593750, 0.0003051757812500, 0.0012512207031250,
+0.0004577636718750, 0.0010070800781250, 0.0003967285156250, 0.0011596679687500,
+0.0004272460937500, 0.0008850097656250, 0.0004882812500000, 0.0006408691406250,
+0.0005187988281250, 0.0009155273437500, 0.0005798339843750, 0.0002441406250000,
+0.0007019042968750, 0.0003662109375000, 0.0007324218750000, 0.0002441406250000,
+0.0009155273437500, -0.0001525878906250, 0.0011291503906250, 0.0000915527343750,
+0.0011596679687500, -0.0002746582031250, 0.0016174316406250, -0.0002441406250000,
+0.0015869140625000, -0.0001525878906250, 0.0018310546875000, -0.0002746582031250,
+0.0021972656250000, -0.0001525878906250, 0.0020141601562500, -0.0000305175781250,
+0.0023803710937500, -0.0000305175781250, 0.0023803710937500, 0.0000000000000000,
+0.0021057128906250, 0.0000610351562500, 0.0021057128906250, 0.0001525878906250,
+0.0021667480468750, 0.0000915527343750, 0.0015563964843750, 0.0000610351562500,
+0.0017395019531250, 0.0000305175781250, 0.0014343261718750, -0.0000915527343750,
+0.0008239746093750, -0.0001525878906250, 0.0012512207031250, 0.0000000000000000,
+0.0004882812500000, -0.0002746582031250, 0.0004882812500000, -0.0000915527343750,
+0.0005187988281250, 0.0000915527343750, 0.0000610351562500, -0.0001525878906250,
+0.0001525878906250, 0.0005187988281250, 0.0003051757812500, 0.0003356933593750,
+0.0000000000000000, 0.0004882812500000, 0.0003662109375000, 0.0010375976562500,
+0.0006713867187500, 0.0007324218750000, 0.0003356933593750, 0.0012207031250000,
+0.0011901855468750, 0.0012207031250000, 0.0010070800781250, 0.0011291503906250,
+0.0012207031250000, 0.0013427734375000, 0.0019226074218750, 0.0013732910156250,
+0.0015563964843750, 0.0015869140625000, 0.0023803710937500, 0.0016479492187500,
+0.0024108886718750, 0.0018615722656250, 0.0024108886718750, 0.0023193359375000,
+0.0029296875000000},
+{0.0022888183593750, 0.0010681152343750, 0.0020751953125000, 0.0009155273437500,
+0.0021667480468750, 0.0014343261718750, 0.0020141601562500, 0.0013427734375000,
+0.0019531250000000, 0.0016174316406250, 0.0019531250000000, 0.0021972656250000,
+0.0018005371093750, 0.0018615722656250, 0.0018615722656250, 0.0025024414062500,
+0.0018310546875000, 0.0025634765625000, 0.0017700195312500, 0.0025024414062500,
+0.0018920898437500, 0.0027465820312500, 0.0018920898437500, 0.0026855468750000,
+0.0017700195312500, 0.0024414062500000, 0.0018310546875000, 0.0025024414062500,
+0.0018005371093750, 0.0023803710937500, 0.0018005371093750, 0.0020751953125000,
+0.0018005371093750, 0.0020751953125000, 0.0016174316406250, 0.0021057128906250,
+0.0018615722656250, 0.0018920898437500, 0.0017700195312500, 0.0019836425781250,
+0.0018310546875000, 0.0021362304687500, 0.0020141601562500, 0.0019531250000000,
+0.0018005371093750, 0.0023498535156250, 0.0020446777343750, 0.0023498535156250,
+0.0019226074218750, 0.0023498535156250, 0.0018920898437500, 0.0025939941406250,
+0.0019836425781250, 0.0025024414062500, 0.0018615722656250, 0.0025634765625000,
+0.0019531250000000, 0.0026550292968750, 0.0018615722656250, 0.0025329589843750,
+0.0018310546875000, 0.0024108886718750, 0.0019531250000000, 0.0025634765625000,
+0.0018615722656250, 0.0021972656250000, 0.0019531250000000, 0.0023193359375000,
+0.0019836425781250, 0.0022277832031250, 0.0018310546875000, 0.0019531250000000,
+0.0018920898437500, 0.0021972656250000, 0.0019226074218750, 0.0019226074218750,
+0.0015563964843750, 0.0020141601562500, 0.0016174316406250, 0.0020446777343750,
+0.0014648437500000, 0.0019226074218750, 0.0011901855468750, 0.0020751953125000,
+0.0012512207031250, 0.0018920898437500, 0.0008544921875000, 0.0019226074218750,
+0.0008544921875000, 0.0019531250000000, 0.0007629394531250, 0.0018310546875000,
+0.0005187988281250, 0.0018920898437500, 0.0006103515625000, 0.0018920898437500,
+0.0003051757812500, 0.0017700195312500, 0.0003967285156250, 0.0018920898437500,
+0.0003967285156250, 0.0020141601562500, 0.0001525878906250, 0.0018920898437500,
+0.0003662109375000, 0.0023498535156250, 0.0004272460937500, 0.0021667480468750,
+0.0003662109375000, 0.0022888183593750, 0.0006408691406250, 0.0027465820312500,
+0.0008544921875000, 0.0025024414062500, 0.0006713867187500, 0.0029602050781250,
+0.0012817382812500, 0.0029296875000000, 0.0011291503906250, 0.0028991699218750,
+0.0012207031250000, 0.0030822753906250, 0.0016174316406250, 0.0028991699218750,
+0.0013122558593750, 0.0030212402343750, 0.0016174316406250, 0.0029602050781250,
+0.0015258789062500, 0.0027160644531250, 0.0015563964843750, 0.0026855468750000,
+0.0017089843750000, 0.0026245117187500, 0.0014343261718750, 0.0020141601562500,
+0.0017089843750000, 0.0021362304687500, 0.0015563964843750, 0.0018615722656250,
+0.0014648437500000, 0.0012817382812500, 0.0016784667968750, 0.0014953613281250,
+0.0013732910156250, 0.0010070800781250, 0.0014648437500000, 0.0010986328125000,
+0.0013427734375000, 0.0008850097656250, 0.0012817382812500, 0.0005187988281250,
+0.0013427734375000, 0.0008850097656250, 0.0011596679687500, 0.0001525878906250,
+0.0012207031250000, 0.0003051757812500, 0.0011901855468750, 0.0003356933593750,
+0.0010986328125000, -0.0000305175781250, 0.0010681152343750, 0.0003662109375000,
+0.0010986328125000, 0.0007934570312500, 0.0011596679687500, 0.0005798339843750,
+0.0011291503906250, 0.0009155273437500, 0.0012207031250000, 0.0014953613281250,
+0.0012817382812500, 0.0014038085937500, 0.0011596679687500, 0.0018920898437500,
+0.0015869140625000, 0.0021667480468750, 0.0014343261718750, 0.0022583007812500,
+0.0017395019531250, 0.0024108886718750, 0.0022277832031250, 0.0024719238281250,
+0.0020446777343750, 0.0025024414062500, 0.0025024414062500, 0.0025024414062500,
+0.0026855468750000, 0.0023803710937500, 0.0025329589843750, 0.0023498535156250,
+0.0027160644531250, 0.0024414062500000, 0.0029602050781250, 0.0021667480468750,
+0.0024719238281250, 0.0022888183593750, 0.0027465820312500, 0.0023193359375000,
+0.0025634765625000, 0.0021667480468750, 0.0021667480468750, 0.0022888183593750,
+0.0025634765625000, 0.0021972656250000, 0.0017089843750000, 0.0022888183593750,
+0.0019226074218750, 0.0020446777343750, 0.0018005371093750, 0.0018005371093750,
+0.0013122558593750, 0.0020446777343750, 0.0017089843750000, 0.0010681152343750,
+0.0012207031250000, 0.0011901855468750, 0.0012512207031250, 0.0009460449218750,
+0.0013122558593750, 0.0001525878906250, 0.0010681152343750, 0.0004882812500000,
+0.0012512207031250, -0.0000610351562500, 0.0011596679687500, -0.0004272460937500,
+0.0011291503906250, 0.0000610351562500, 0.0011291503906250, 0.0001525878906250,
+0.0010681152343750, -0.0002136230468750, 0.0010375976562500, 0.0010986328125000,
+0.0009460449218750, 0.0006408691406250, 0.0009460449218750, 0.0011291503906250,
+0.0009460449218750, 0.0022277832031250, 0.0008544921875000, 0.0014648437500000,
+0.0007934570312500, 0.0026245117187500, 0.0009765625000000, 0.0026855468750000,
+0.0008850097656250, 0.0024719238281250, 0.0009765625000000, 0.0029602050781250,
+0.0011291503906250, 0.0028686523437500, 0.0009460449218750, 0.0023498535156250,
+0.0010986328125000, 0.0024719238281250, 0.0010986328125000, 0.0021667480468750,
+0.0010070800781250, 0.0016174316406250, 0.0010070800781250, 0.0018615722656250,
+0.0009155273437500, 0.0011596679687500, 0.0007324218750000, 0.0010986328125000,
+0.0006103515625000, 0.0011901855468750, 0.0005493164062500, 0.0008544921875000,
+0.0003662109375000, 0.0007934570312500, 0.0003356933593750, 0.0010681152343750,
+0.0005493164062500, 0.0008544921875000, 0.0003662109375000, 0.0010681152343750,
+0.0005493164062500, 0.0013732910156250, 0.0009460449218750, 0.0010986328125000,
+0.0006408691406250, 0.0016174316406250, 0.0010681152343750, 0.0016174316406250,
+0.0011291503906250, 0.0016479492187500, 0.0010375976562500, 0.0020141601562500,
+0.0011901855468750, 0.0018920898437500, 0.0011291503906250, 0.0019531250000000,
+0.0010681152343750, 0.0020446777343750, 0.0010986328125000, 0.0021667480468750,
+0.0010375976562500, 0.0021667480468750, 0.0009765625000000, 0.0020446777343750,
+0.0009155273437500, 0.0022888183593750, 0.0011291503906250, 0.0021667480468750,
+0.0010375976562500, 0.0022277832031250, 0.0011901855468750, 0.0023803710937500,
+0.0014038085937500, 0.0021667480468750, 0.0013122558593750, 0.0024414062500000,
+0.0018310546875000, 0.0021972656250000, 0.0017700195312500, 0.0022277832031250,
+0.0019226074218750, 0.0023498535156250, 0.0024108886718750, 0.0020446777343750,
+0.0022583007812500, 0.0023498535156250, 0.0025024414062500, 0.0021362304687500,
+0.0025634765625000, 0.0022277832031250, 0.0024719238281250, 0.0024719238281250,
+0.0025329589843750, 0.0021667480468750, 0.0025024414062500, 0.0024719238281250,
+0.0021667480468750, 0.0024108886718750, 0.0021972656250000, 0.0023193359375000,
+0.0020141601562500, 0.0025024414062500, 0.0016174316406250, 0.0024108886718750,
+0.0018005371093750, 0.0023803710937500, 0.0013122558593750, 0.0024414062500000,
+0.0012207031250000, 0.0023193359375000, 0.0012817382812500, 0.0022277832031250,
+0.0010070800781250, 0.0023193359375000, 0.0010681152343750, 0.0021667480468750,
+0.0012512207031250, 0.0021057128906250, 0.0010986328125000, 0.0021057128906250,
+0.0013732910156250, 0.0020446777343750, 0.0017089843750000, 0.0020446777343750,
+0.0015563964843750, 0.0020751953125000, 0.0022583007812500, 0.0019531250000000,
+0.0022583007812500, 0.0020446777343750, 0.0024414062500000, 0.0021362304687500,
+0.0029602050781250, 0.0019226074218750, 0.0027770996093750, 0.0022583007812500,
+0.0029602050781250, 0.0021667480468750, 0.0031127929687500, 0.0022888183593750,
+0.0028381347656250, 0.0025634765625000, 0.0026855468750000, 0.0024414062500000,
+0.0027770996093750, 0.0026245117187500, 0.0021362304687500, 0.0026855468750000,
+0.0020446777343750, 0.0025634765625000, 0.0020751953125000, 0.0026245117187500,
+0.0017395019531250, 0.0026245117187500, 0.0017700195312500, 0.0023498535156250,
+0.0019226074218750, 0.0024108886718750, 0.0017395019531250, 0.0022583007812500,
+0.0019226074218750, 0.0019836425781250, 0.0021972656250000, 0.0020141601562500,
+0.0020751953125000, 0.0016479492187500, 0.0024108886718750, 0.0016174316406250,
+0.0024414062500000, 0.0015563964843750, 0.0025024414062500, 0.0013427734375000,
+0.0026855468750000, 0.0013122558593750, 0.0025939941406250, 0.0013427734375000,
+0.0026550292968750, 0.0011596679687500, 0.0027160644531250, 0.0012207031250000,
+0.0027160644531250, 0.0013732910156250, 0.0027465820312500, 0.0010986328125000,
+0.0027465820312500, 0.0014038085937500, 0.0028686523437500, 0.0014038085937500,
+0.0028686523437500, 0.0013122558593750, 0.0028381347656250, 0.0014953613281250,
+0.0028991699218750, 0.0014648437500000, 0.0029602050781250, 0.0012817382812500,
+0.0028381347656250, 0.0014648437500000, 0.0028991699218750, 0.0014038085937500,
+0.0028076171875000, 0.0012512207031250, 0.0026550292968750, 0.0014648437500000,
+0.0027770996093750, 0.0011901855468750, 0.0026855468750000, 0.0013122558593750,
+0.0026550292968750, 0.0014648437500000, 0.0027160644531250, 0.0013732910156250,
+0.0028076171875000, 0.0014343261718750, 0.0028381347656250, 0.0016174316406250,
+0.0030212402343750, 0.0016479492187500, 0.0030822753906250, 0.0016784667968750,
+0.0031433105468750, 0.0018920898437500, 0.0032653808593750, 0.0019836425781250,
+0.0033264160156250, 0.0017700195312500, 0.0032958984375000, 0.0021057128906250,
+0.0034179687500000, 0.0018920898437500, 0.0033264160156250, 0.0015869140625000,
+0.0033264160156250, 0.0020446777343750, 0.0034790039062500, 0.0013427734375000,
+0.0030517578125000, 0.0014953613281250, 0.0031127929687500, 0.0014038085937500,
+0.0032653808593750, 0.0008239746093750, 0.0030822753906250, 0.0011596679687500,
+0.0030517578125000, 0.0007324218750000, 0.0033874511718750, 0.0006713867187500,
+0.0032958984375000, 0.0008239746093750, 0.0032653808593750, 0.0007629394531250,
+0.0035705566406250, 0.0007019042968750, 0.0036010742187500, 0.0010681152343750,
+0.0031433105468750, 0.0009460449218750, 0.0035095214843750, 0.0010681152343750,
+0.0030822753906250, 0.0013427734375000, 0.0024108886718750, 0.0011596679687500,
+0.0028076171875000, 0.0015869140625000, 0.0017395019531250, 0.0014648437500000,
+0.0018310546875000, 0.0016479492187500, 0.0014343261718750, 0.0019836425781250,
+0.0006408691406250, 0.0017700195312500, 0.0012207031250000, 0.0021667480468750,
+0.0001220703125000, 0.0021972656250000, 0.0001525878906250, 0.0021057128906250,
+0.0003662109375000, 0.0024108886718750, -0.0002746582031250, 0.0017700195312500,
+0.0024414062500000, 0.0025634765625000, 0.0003356933593750, 0.0021667480468750,
+0.0009460449218750, 0.0024719238281250, 0.0004577636718750, 0.0029907226562500,
+-0.0006408691406250, 0.0026245117187500, -0.0002136230468750, 0.0031433105468750,
+-0.0003356933593750, 0.0032348632812500, -0.0007934570312500, 0.0029907226562500,
+-0.0002136230468750, 0.0030212402343750, 0.0001831054687500, 0.0029602050781250,
+-0.0003356933593750, 0.0023498535156250, 0.0013122558593750, 0.0024108886718750,
+0.0010375976562500, 0.0020751953125000, 0.0015258789062500, 0.0015258789062500,
+0.0028076171875000, 0.0016784667968750, 0.0021057128906250, 0.0009765625000000,
+0.0034179687500000}};
diff -pruN 1:13.23.1~dfsg-2/bridges/bridge_softmix.c 1:16.1.1~dfsg-1/bridges/bridge_softmix.c
--- 1:13.23.1~dfsg-2/bridges/bridge_softmix.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridge_softmix.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,30 +33,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <sys/time.h>
-#include <signal.h>
-#include <errno.h>
-#include <unistd.h>
-
-#include "asterisk/module.h"
-#include "asterisk/channel.h"
-#include "asterisk/bridge.h"
-#include "asterisk/bridge_technology.h"
-#include "asterisk/frame.h"
-#include "asterisk/options.h"
-#include "asterisk/logger.h"
-#include "asterisk/slinfactory.h"
-#include "asterisk/astobj2.h"
-#include "asterisk/timing.h"
-#include "asterisk/translate.h"
+#include "asterisk/stream.h"
+#include "asterisk/test.h"
+#include "asterisk/vector.h"
 #include "asterisk/message.h"
-
-#define MAX_DATALEN 8096
+#include "bridge_softmix/include/bridge_softmix_internal.h"
 
 /*! The minimum sample rate of the bridge. */
 #define SOFTMIX_MIN_SAMPLE_RATE 8000	/* 8 kHz sample rate */
@@ -85,69 +66,17 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 /*! Default minimum average magnitude threshold to determine talking by the DSP. */
 #define DEFAULT_SOFTMIX_TALKING_THRESHOLD 160
 
-#define DEFAULT_ENERGY_HISTORY_LEN 150
-
-struct video_follow_talker_data {
-	/*! audio energy history */
-	int energy_history[DEFAULT_ENERGY_HISTORY_LEN];
-	/*! The current slot being used in the history buffer, this
-	 *  increments and wraps around */
-	int energy_history_cur_slot;
-	/*! The current energy sum used for averages. */
-	int energy_accum;
-	/*! The current energy average */
-	int energy_average;
-};
-
-/*! \brief Structure which contains per-channel mixing information */
-struct softmix_channel {
-	/*! Lock to protect this structure */
-	ast_mutex_t lock;
-	/*! Factory which contains audio read in from the channel */
-	struct ast_slinfactory factory;
-	/*! Frame that contains mixed audio to be written out to the channel */
-	struct ast_frame write_frame;
-	/*! Current expected read slinear format. */
-	struct ast_format *read_slin_format;
-	/*! DSP for detecting silence */
-	struct ast_dsp *dsp;
-	/*!
-	 * \brief TRUE if a channel is talking.
-	 *
-	 * \note This affects how the channel's audio is mixed back to
-	 * it.
-	 */
-	unsigned int talking:1;
-	/*! TRUE if the channel provided audio for this mixing interval */
-	unsigned int have_audio:1;
-	/*! Buffer containing final mixed audio from all sources */
-	short final_buf[MAX_DATALEN];
-	/*! Buffer containing only the audio from the channel */
-	short our_buf[MAX_DATALEN];
-	/*! Data pertaining to talker mode for video conferencing */
-	struct video_follow_talker_data video_talker;
-};
-
-struct softmix_bridge_data {
-	struct ast_timer *timer;
-	/*!
-	 * \brief Bridge pointer passed to the softmix mixing thread.
-	 *
-	 * \note Does not need a reference because the bridge will
-	 * always exist while the mixing thread exists even if the
-	 * bridge is no longer actively using the softmix technology.
-	 */
-	struct ast_bridge *bridge;
-	/*! Lock for signaling the mixing thread. */
-	ast_mutex_t lock;
-	/*! Condition, used if we need to wake up the mixing thread. */
-	ast_cond_t cond;
-	/*! Thread handling the mixing */
-	pthread_t thread;
-	unsigned int internal_rate;
-	unsigned int internal_mixing_interval;
-	/*! TRUE if the mixing thread should stop */
-	unsigned int stop:1;
+#define SOFTBRIDGE_VIDEO_DEST_PREFIX "softbridge_dest"
+#define SOFTBRIDGE_VIDEO_DEST_LEN strlen(SOFTBRIDGE_VIDEO_DEST_PREFIX)
+#define SOFTBRIDGE_VIDEO_DEST_SEPARATOR '_'
+
+struct softmix_remb_collector {
+	/*! The frame which will be given to each source stream */
+	struct ast_frame frame;
+	/*! The REMB to send to the source which is collecting REMB reports */
+	struct ast_rtp_rtcp_feedback feedback;
+	/*! The maximum bitrate */
+	unsigned int bitrate;
 };
 
 struct softmix_stats {
@@ -165,15 +94,9 @@ struct softmix_stats {
 	unsigned int locked_rate;
 };
 
-struct softmix_mixing_array {
-	unsigned int max_num_entries;
-	unsigned int used_entries;
-	int16_t **buffers;
-};
-
 struct softmix_translate_helper_entry {
 	int num_times_requested; /*!< Once this entry is no longer requested, free the trans_pvt
-	                              and re-init if it was usable. */
+								  and re-init if it was usable. */
 	struct ast_format *dst_format; /*!< The destination format for this helper */
 	struct ast_trans_pvt *trans_pvt; /*!< the translator for this slot. */
 	struct ast_frame *out_frame; /*!< The output frame from the last translation */
@@ -273,14 +196,14 @@ static int16_t *softmix_process_read_aud
  */
 static void softmix_process_write_audio(struct softmix_translate_helper *trans_helper,
 	struct ast_format *raw_write_fmt,
-	struct softmix_channel *sc)
+	struct softmix_channel *sc, unsigned int default_sample_size)
 {
 	struct softmix_translate_helper_entry *entry = NULL;
 	int i;
 
 	/* If we provided audio that was not determined to be silence,
 	 * then take it out while in slinear format. */
-	if (sc->have_audio && sc->talking) {
+	if (sc->have_audio && sc->talking && !sc->binaural) {
 		for (i = 0; i < sc->write_frame.samples; i++) {
 			ast_slinear_saturated_subtract(&sc->final_buf[i], &sc->our_buf[i]);
 		}
@@ -295,6 +218,13 @@ static void softmix_process_write_audio(
 		/* do not do any special write translate optimization if we had to make
 		 * a special mix for them to remove their own audio. */
 		return;
+	} else if (sc->have_audio && sc->talking && sc->binaural > 0) {
+		/*
+		 * Binaural audio requires special saturated substract since we have two
+		 * audio signals per channel now.
+		 */
+		softmix_process_write_binaural_audio(sc, default_sample_size);
+		return;
 	}
 
 	/* Attempt to optimize channels using the same translation path/codec. Build a list of entries
@@ -303,6 +233,9 @@ static void softmix_process_write_audio(
 	   multiple channels (>=2) using the same codec make sure resources are allocated only when
 	   needed and released when not (see also softmix_translate_helper_cleanup */
 	AST_LIST_TRAVERSE(&trans_helper->entries, entry, entry) {
+		if (sc->binaural != 0) {
+			continue;
+		}
 		if (ast_format_cmp(entry->dst_format, raw_write_fmt) == AST_FORMAT_CMP_EQUAL) {
 			entry->num_times_requested++;
 		} else {
@@ -362,12 +295,18 @@ static void softmix_translate_helper_cle
 	AST_LIST_TRAVERSE_SAFE_END;
 }
 
-static void set_softmix_bridge_data(int rate, int interval, struct ast_bridge_channel *bridge_channel, int reset)
+static void set_softmix_bridge_data(int rate, int interval, struct ast_bridge_channel *bridge_channel, int reset, int set_binaural, int binaural_pos_id, int is_announcement)
 {
 	struct softmix_channel *sc = bridge_channel->tech_pvt;
 	struct ast_format *slin_format;
 	int setup_fail;
 
+#ifdef BINAURAL_RENDERING
+	if (interval != BINAURAL_MIXING_INTERVAL) {
+		interval = BINAURAL_MIXING_INTERVAL;
+	}
+#endif
+
 	/* The callers have already ensured that sc is never NULL. */
 	ast_assert(sc != NULL);
 
@@ -392,6 +331,26 @@ static void set_softmix_bridge_data(int
 	sc->write_frame.datalen = SOFTMIX_DATALEN(rate, interval);
 	sc->write_frame.samples = SOFTMIX_SAMPLES(rate, interval);
 
+	/* We will store the rate here cause we need to set the data again when a channel is unsuspended */
+	sc->rate = rate;
+
+	/* If the channel will contain binaural data we will set a identifier in the channel
+	 * if set_binaural == -1 this is just a sample rate update, will ignore it. */
+	if (set_binaural == 1) {
+		sc->binaural = 1;
+	} else if (set_binaural == 0) {
+		sc->binaural = 0;
+	}
+
+	/* Setting the binaural position. This doesn't require a change of the overlaying channel infos
+	 * and doesn't have to be done if we just updating sample rates. */
+	if (binaural_pos_id != -1) {
+		sc->binaural_pos = binaural_pos_id;
+	}
+	if (is_announcement != -1) {
+		sc->is_announcement = is_announcement;
+	}
+
 	/*
 	 * NOTE: The read_slin_format does not hold a reference because it
 	 * will always be a signed linear format.
@@ -406,7 +365,13 @@ static void set_softmix_bridge_data(int
 	setup_fail |= ast_set_read_format_path(bridge_channel->chan,
 		ast_channel_rawreadformat(bridge_channel->chan), slin_format);
 	ast_channel_unlock(bridge_channel->chan);
-	setup_fail |= ast_set_write_format(bridge_channel->chan, slin_format);
+
+	/* If channel contains binaural data we will set it here for the trans_pvt. */
+	if (set_binaural == 1 || (set_binaural == -1 && sc->binaural == 1)) {
+		setup_fail |= ast_set_write_format_interleaved_stereo(bridge_channel->chan, slin_format);
+	} else if (set_binaural == 0) {
+		setup_fail |= ast_set_write_format(bridge_channel->chan, slin_format);
+	}
 
 	/* set up new DSP.  This is on the read side only right before the read frame enters the smoother.  */
 	sc->dsp = ast_dsp_new_with_rate(rate);
@@ -446,16 +411,259 @@ static void softmix_poke_thread(struct s
 /*! \brief Function called when a channel is unsuspended from the bridge */
 static void softmix_bridge_unsuspend(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel)
 {
+#ifdef BINAURAL_RENDERING
+	struct softmix_channel *sc = bridge_channel->tech_pvt;
+	if (sc->binaural) {
+		/* Restore some usefull data if it was a binaural channel */
+		struct ast_format *slin_format;
+
+		slin_format = ast_format_cache_get_slin_by_rate(sc->rate);
+		ast_set_write_format_interleaved_stereo(bridge_channel->chan, slin_format);
+	}
+#endif
 	if (bridge->tech_pvt) {
 		softmix_poke_thread(bridge->tech_pvt);
 	}
 }
 
+/*!
+ * \brief Determine if a stream is a video source stream.
+ *
+ * \param stream The stream to test
+ * \retval 1 The stream is a video source
+ * \retval 0 The stream is not a video source
+ */
+static int is_video_source(const struct ast_stream *stream)
+{
+	if (ast_stream_get_state(stream) != AST_STREAM_STATE_REMOVED
+		&& ast_stream_get_type(stream) == AST_MEDIA_TYPE_VIDEO
+		&& strncmp(ast_stream_get_name(stream), SOFTBRIDGE_VIDEO_DEST_PREFIX,
+			SOFTBRIDGE_VIDEO_DEST_LEN)) {
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Determine if a stream is a video destination stream.
+ *
+ * A source channel name can be provided to narrow this to a destination stream
+ * for a particular source channel. Further, a source stream name can be provided
+ * to narrow this to a particular source stream's destination. However, empty strings
+ * can be provided to match any destination video stream, regardless of source channel
+ * or source stream.
+ *
+ * \param stream The stream to test
+ * \param source_channel_name The name of a source video channel to match
+ * \param source_stream_name The name of the source video stream to match
+ * \retval 1 The stream is a video destination stream
+ * \retval 0 The stream is not a video destination stream
+ */
+static int is_video_dest(const struct ast_stream *stream, const char *source_channel_name,
+	const char *source_stream_name)
+{
+	char *dest_video_name;
+	size_t dest_video_name_len;
+
+	if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED
+		|| ast_stream_get_type(stream) != AST_MEDIA_TYPE_VIDEO) {
+		return 0;
+	}
+
+	dest_video_name_len = SOFTBRIDGE_VIDEO_DEST_LEN + 1;
+	if (!ast_strlen_zero(source_channel_name)) {
+		dest_video_name_len += strlen(source_channel_name) + 1;
+		if (!ast_strlen_zero(source_stream_name)) {
+			dest_video_name_len += strlen(source_stream_name) + 1;
+		}
+
+		dest_video_name = ast_alloca(dest_video_name_len);
+		if (!ast_strlen_zero(source_stream_name)) {
+			/* We are looking for an exact stream name */
+			snprintf(dest_video_name, dest_video_name_len, "%s%c%s%c%s",
+				SOFTBRIDGE_VIDEO_DEST_PREFIX, SOFTBRIDGE_VIDEO_DEST_SEPARATOR,
+				source_channel_name, SOFTBRIDGE_VIDEO_DEST_SEPARATOR,
+				source_stream_name);
+			return !strcmp(ast_stream_get_name(stream), dest_video_name);
+		}
+		snprintf(dest_video_name, dest_video_name_len, "%s%c%s",
+			SOFTBRIDGE_VIDEO_DEST_PREFIX, SOFTBRIDGE_VIDEO_DEST_SEPARATOR,
+			source_channel_name);
+	} else {
+		dest_video_name = SOFTBRIDGE_VIDEO_DEST_PREFIX;
+	}
+
+	return !strncmp(ast_stream_get_name(stream), dest_video_name, dest_video_name_len - 1);
+}
+
+static int append_source_streams(struct ast_stream_topology *dest,
+	const char *channel_name, const char *sdp_label,
+	const struct ast_stream_topology *source)
+{
+	int i;
+
+	for (i = 0; i < ast_stream_topology_get_count(source); ++i) {
+		struct ast_stream *stream;
+		struct ast_stream *stream_clone;
+		char *stream_clone_name = NULL;
+
+		stream = ast_stream_topology_get_stream(source, i);
+		if (!is_video_source(stream)) {
+			continue;
+		}
+
+		if (ast_asprintf(&stream_clone_name, "%s_%s_%s", SOFTBRIDGE_VIDEO_DEST_PREFIX,
+			channel_name, ast_stream_get_name(stream)) < 0) {
+			return -1;
+		}
+
+		stream_clone = ast_stream_clone(stream, stream_clone_name);
+		ast_free(stream_clone_name);
+		if (!stream_clone) {
+			return -1;
+		}
+
+		/* Sends an "a:label" attribute in the SDP for participant event correlation */
+		if (!ast_strlen_zero(sdp_label)) {
+			ast_stream_set_metadata(stream_clone, "SDP:LABEL", sdp_label);
+		}
+
+		if (ast_stream_topology_append_stream(dest, stream_clone) < 0) {
+			ast_stream_free(stream_clone);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+static int append_all_streams(struct ast_stream_topology *dest,
+	const struct ast_stream_topology *source)
+{
+	int i;
+	int dest_index = 0;
+
+	for (i = 0; i < ast_stream_topology_get_count(source); ++i) {
+		struct ast_stream *clone;
+		int added = 0;
+
+		clone = ast_stream_clone(ast_stream_topology_get_stream(source, i), NULL);
+		if (!clone) {
+			return -1;
+		}
+
+		/* If we can reuse an existing removed stream then do so */
+		while (dest_index < ast_stream_topology_get_count(dest)) {
+			struct ast_stream *stream = ast_stream_topology_get_stream(dest, dest_index);
+
+			dest_index++;
+
+			if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+				/* This cannot fail because dest_index - 1 is less than the
+				 * current count in dest. */
+				ast_stream_topology_set_stream(dest, dest_index - 1, clone);
+				added = 1;
+				break;
+			}
+		}
+
+		/* If no removed stream exists that we took the place of append the stream */
+		if (!added && ast_stream_topology_append_stream(dest, clone) < 0) {
+			ast_stream_free(clone);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Issue channel stream topology change requests.
+ *
+ * When in SFU mode, each participant needs to be able to
+ * send video directly to other participants in the bridge.
+ * This means that all participants need to have their topologies
+ * updated. The joiner needs to have destination streams for
+ * all current participants, and the current participants need
+ * to have destinations streams added for the joiner's sources.
+ *
+ * \param joiner The channel that is joining the softmix bridge
+ * \param participants The current participants in the softmix bridge
+ */
+static void sfu_topologies_on_join(struct ast_bridge *bridge,
+	struct ast_bridge_channel *joiner)
+{
+	struct ast_stream_topology *joiner_video = NULL;
+	struct ast_bridge_channels_list *participants = &bridge->channels;
+	struct ast_bridge_channel *participant;
+	int res;
+	struct softmix_channel *sc;
+
+	joiner_video = ast_stream_topology_alloc();
+	if (!joiner_video) {
+		return;
+	}
+
+	sc = joiner->tech_pvt;
+
+	ast_channel_lock(joiner->chan);
+	res = append_source_streams(joiner_video, ast_channel_name(joiner->chan),
+		bridge->softmix.send_sdp_label ? ast_channel_uniqueid(joiner->chan) : NULL,
+		ast_channel_get_stream_topology(joiner->chan));
+	sc->topology = ast_stream_topology_clone(ast_channel_get_stream_topology(joiner->chan));
+	ast_channel_unlock(joiner->chan);
+
+	if (res || !sc->topology) {
+		goto cleanup;
+	}
+
+	AST_LIST_TRAVERSE(participants, participant, entry) {
+		if (participant == joiner) {
+			continue;
+		}
+		ast_channel_lock(participant->chan);
+		res = append_source_streams(sc->topology, ast_channel_name(participant->chan),
+			bridge->softmix.send_sdp_label ? ast_channel_uniqueid(participant->chan) : NULL,
+			ast_channel_get_stream_topology(participant->chan));
+		ast_channel_unlock(participant->chan);
+		if (res) {
+			goto cleanup;
+		}
+	}
+
+	ast_channel_request_stream_topology_change(joiner->chan, sc->topology, NULL);
+
+	AST_LIST_TRAVERSE(participants, participant, entry) {
+		if (participant == joiner) {
+			continue;
+		}
+
+		sc = participant->tech_pvt;
+		if (append_all_streams(sc->topology, joiner_video)) {
+			goto cleanup;
+		}
+		ast_channel_request_stream_topology_change(participant->chan, sc->topology, NULL);
+	}
+
+cleanup:
+	ast_stream_topology_free(joiner_video);
+}
+
 /*! \brief Function called when a channel is joined into the bridge */
 static int softmix_bridge_join(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel)
 {
 	struct softmix_channel *sc;
 	struct softmix_bridge_data *softmix_data;
+	int set_binaural = 0;
+	/*
+	 * If false, the channel will be convolved, but since it is a non stereo channel, output
+	 * will be mono.
+	 */
+	int skip_binaural_output = 1;
+	int pos_id;
+	int is_announcement = 0;
+	int samplerate_change;
 
 	softmix_data = bridge->tech_pvt;
 	if (!softmix_data) {
@@ -467,6 +675,33 @@ static int softmix_bridge_join(struct as
 		return -1;
 	}
 
+	samplerate_change = softmix_data->internal_rate;
+	pos_id = -1;
+	if (bridge->softmix.binaural_active) {
+		if (strncmp(ast_channel_name(bridge_channel->chan), "CBAnn", 5) != 0) {
+			set_binaural = ast_format_get_channel_count(bridge_channel->write_format) > 1 ? 1 : 0;
+			if (set_binaural) {
+				softmix_data->internal_rate = samplerate_change;
+			}
+			skip_binaural_output = 0;
+		} else {
+			is_announcement = 1;
+		}
+		if (set_binaural) {
+			softmix_data->convolve.binaural_active = 1;
+		}
+		if (!skip_binaural_output)	{
+			pos_id = set_binaural_data_join(&softmix_data->convolve, softmix_data->default_sample_size);
+			if (pos_id == -1) {
+				ast_log(LOG_ERROR, "Bridge %s: Failed to join channel %s. "
+						"Could not allocate enough memory.\n", bridge->uniqueid,
+						ast_channel_name(bridge_channel->chan));
+				ast_free(sc);
+				return -1;
+			}
+		}
+	}
+
 	/* Can't forget the lock */
 	ast_mutex_init(&sc->lock);
 
@@ -477,22 +712,87 @@ static int softmix_bridge_join(struct as
 		softmix_data->internal_mixing_interval
 			? softmix_data->internal_mixing_interval
 			: DEFAULT_SOFTMIX_INTERVAL,
-		bridge_channel, 0);
+		bridge_channel, 0, set_binaural, pos_id, is_announcement);
+
+	if (bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU) {
+		sfu_topologies_on_join(bridge, bridge_channel);
+	}
 
 	softmix_poke_thread(softmix_data);
 	return 0;
 }
 
+static int remove_destination_streams(struct ast_stream_topology *topology,
+	const char *channel_name)
+{
+	int i;
+	int stream_removed = 0;
+
+	for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
+		struct ast_stream *stream;
+
+		stream = ast_stream_topology_get_stream(topology, i);
+
+		if (is_video_dest(stream, channel_name, NULL)) {
+			ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
+			stream_removed = 1;
+		}
+	}
+	return stream_removed;
+}
+
+static int sfu_topologies_on_leave(struct ast_bridge_channel *leaver, struct ast_bridge_channels_list *participants)
+{
+	struct ast_bridge_channel *participant;
+	struct softmix_channel *sc;
+
+	AST_LIST_TRAVERSE(participants, participant, entry) {
+		sc = participant->tech_pvt;
+		if (!remove_destination_streams(sc->topology, ast_channel_name(leaver->chan))) {
+			continue;
+		}
+		ast_channel_request_stream_topology_change(participant->chan, sc->topology, NULL);
+	}
+
+	sc = leaver->tech_pvt;
+	if (remove_destination_streams(sc->topology, "")) {
+		ast_channel_request_stream_topology_change(leaver->chan, sc->topology, NULL);
+	}
+
+	return 0;
+}
+
 /*! \brief Function called when a channel leaves the bridge */
 static void softmix_bridge_leave(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel)
 {
-	struct softmix_channel *sc = bridge_channel->tech_pvt;
+	struct softmix_channel *sc;
+	struct softmix_bridge_data *softmix_data;
 
+	softmix_data = bridge->tech_pvt;
+	sc = bridge_channel->tech_pvt;
 	if (!sc) {
 		return;
 	}
+
+	if (bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU) {
+		sfu_topologies_on_leave(bridge_channel, &bridge->channels);
+	}
+
+	if (bridge->softmix.binaural_active) {
+		if (sc->binaural) {
+			set_binaural_data_leave(&softmix_data->convolve, sc->binaural_pos,
+					softmix_data->default_sample_size);
+		}
+	}
+
 	bridge_channel->tech_pvt = NULL;
 
+	ast_stream_topology_free(sc->topology);
+
+	ao2_cleanup(sc->remb_collector);
+
+	AST_VECTOR_FREE(&sc->video_sources);
+
 	/* Drop mutex lock */
 	ast_mutex_destroy(&sc->lock);
 
@@ -568,6 +868,12 @@ static void softmix_bridge_write_video(s
 			softmix_pass_video_top_priority(bridge, frame);
 		}
 		break;
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		/* Nothing special to do here, the bridge channel stream map will ensure the
+		 * video goes everywhere it needs to
+		 */
+		ast_bridge_queue_everyone_else(bridge, bridge_channel, frame);
+		break;
 	}
 }
 
@@ -720,6 +1026,115 @@ static void softmix_bridge_check_voice(s
 	}
 }
 
+static int remove_all_original_streams(struct ast_stream_topology *dest,
+	const struct ast_stream_topology *source,
+	const struct ast_stream_topology *original)
+{
+	int i;
+
+	for (i = 0; i < ast_stream_topology_get_count(source); ++i) {
+		struct ast_stream *stream;
+		int original_index;
+
+		stream = ast_stream_topology_get_stream(source, i);
+
+		/* Mark the existing stream as removed so we get a new one, this will get
+		 * reused on a subsequent renegotiation.
+		 */
+		for (original_index = 0; original_index < ast_stream_topology_get_count(original); ++original_index) {
+			struct ast_stream *original_stream = ast_stream_topology_get_stream(original, original_index);
+
+			if (!strcmp(ast_stream_get_name(stream), ast_stream_get_name(original_stream))) {
+				struct ast_stream *removed;
+
+				/* Since the participant is still going to be in the bridge we
+				 * change the name so that routing does not attempt to route video
+				 * to this stream.
+				 */
+				removed = ast_stream_clone(stream, "removed");
+				if (!removed) {
+					return -1;
+				}
+
+				ast_stream_set_state(removed, AST_STREAM_STATE_REMOVED);
+
+				/* The destination topology can only ever contain the same, or more,
+				 * streams than the original so this is safe.
+				 */
+				if (ast_stream_topology_set_stream(dest, original_index, removed)) {
+					ast_stream_free(removed);
+					return -1;
+				}
+
+				break;
+			}
+		}
+	}
+
+	return 0;
+}
+
+static void sfu_topologies_on_source_change(struct ast_bridge *bridge,
+	struct ast_bridge_channel *source)
+{
+	struct ast_stream_topology *source_video = NULL;
+	struct ast_bridge_channels_list *participants = &bridge->channels;
+	struct ast_bridge_channel *participant;
+	int res;
+
+	source_video = ast_stream_topology_alloc();
+	if (!source_video) {
+		return;
+	}
+
+	ast_channel_lock(source->chan);
+	res = append_source_streams(source_video, ast_channel_name(source->chan),
+		bridge->softmix.send_sdp_label ? ast_channel_uniqueid(source->chan) : NULL,
+		ast_channel_get_stream_topology(source->chan));
+	ast_channel_unlock(source->chan);
+	if (res) {
+		goto cleanup;
+	}
+
+	AST_LIST_TRAVERSE(participants, participant, entry) {
+		struct ast_stream_topology *original_topology;
+		struct softmix_channel *sc;
+
+		if (participant == source) {
+			continue;
+		}
+
+		sc = participant->tech_pvt;
+
+		original_topology = ast_stream_topology_clone(sc->topology);
+		if (!original_topology) {
+			goto cleanup;
+		}
+
+		/* We add all the source streams back in, if any removed streams are already present they will
+		 * get used first followed by appending new ones.
+		 */
+		if (append_all_streams(sc->topology, source_video)) {
+			ast_stream_topology_free(original_topology);
+			goto cleanup;
+		}
+
+		/* And the original existing streams get marked as removed. This causes the remote side to see
+		 * a new stream for the source streams.
+		 */
+		if (remove_all_original_streams(sc->topology, source_video, original_topology)) {
+			ast_stream_topology_free(original_topology);
+			goto cleanup;
+		}
+
+		ast_channel_request_stream_topology_change(participant->chan, sc->topology, NULL);
+		ast_stream_topology_free(original_topology);
+	}
+
+cleanup:
+	ast_stream_topology_free(source_video);
+}
+
 /*!
  * \internal
  * \brief Determine what to do with a text frame.
@@ -770,6 +1185,8 @@ static void softmix_bridge_write_text(st
  */
 static int softmix_bridge_write_control(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel, struct ast_frame *frame)
 {
+	struct softmix_bridge_data *softmix_data = bridge->tech_pvt;
+
 	/*
 	 * XXX Softmix needs to use channel roles to determine what to
 	 * do with control frames.
@@ -788,7 +1205,16 @@ static int softmix_bridge_write_control(
 		clear_talking(bridge_channel);
 		break;
 	case AST_CONTROL_VIDUPDATE:
-		ast_bridge_queue_everyone_else(bridge, NULL, frame);
+		if (!bridge->softmix.video_mode.video_update_discard ||
+			ast_tvdiff_ms(ast_tvnow(), softmix_data->last_video_update) > bridge->softmix.video_mode.video_update_discard) {
+			ast_bridge_queue_everyone_else(bridge, NULL, frame);
+			softmix_data->last_video_update = ast_tvnow();
+		}
+		break;
+	case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
+		if (bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU) {
+			sfu_topologies_on_source_change(bridge, bridge_channel);
+		}
 		break;
 	default:
 		break;
@@ -799,6 +1225,39 @@ static int softmix_bridge_write_control(
 
 /*!
  * \internal
+ * \brief Determine what to do with an RTCP frame.
+ * \since 15.4.0
+ *
+ * \param bridge Which bridge is getting the frame
+ * \param bridge_channel Which channel is writing the frame.
+ * \param frame What is being written.
+ */
+static void softmix_bridge_write_rtcp(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel, struct ast_frame *frame)
+{
+	struct ast_rtp_rtcp_feedback *feedback = frame->data.ptr;
+	struct softmix_channel *sc = bridge_channel->tech_pvt;
+
+	/* We only care about REMB reports right now. In the future we may be able to use sender or
+	 * receiver reports to further tweak things, but not yet.
+	 */
+	if (frame->subclass.integer != AST_RTP_RTCP_PSFB || feedback->fmt != AST_RTP_RTCP_FMT_REMB ||
+		bridge->softmix.video_mode.mode != AST_BRIDGE_VIDEO_MODE_SFU ||
+		!bridge->softmix.video_mode.mode_data.sfu_data.remb_send_interval) {
+		return;
+	}
+
+	/* REMB is the total estimated maximum bitrate across all streams within the session, so we store
+	 * only the latest report and use it everywhere.
+	 */
+	ast_mutex_lock(&sc->lock);
+	sc->remb = feedback->remb;
+	ast_mutex_unlock(&sc->lock);
+
+	return;
+}
+
+/*!
+ * \internal
  * \brief Determine what to do with a frame written into the bridge.
  * \since 12.0.0
  *
@@ -847,6 +1306,9 @@ static int softmix_bridge_write(struct a
 	case AST_FRAME_CONTROL:
 		res = softmix_bridge_write_control(bridge, bridge_channel, frame);
 		break;
+	case AST_FRAME_RTCP:
+		softmix_bridge_write_rtcp(bridge, bridge_channel, frame);
+		break;
 	case AST_FRAME_BRIDGE_ACTION:
 		res = ast_bridge_queue_everyone_else(bridge, bridge_channel, frame);
 		break;
@@ -862,6 +1324,108 @@ static int softmix_bridge_write(struct a
 	return res;
 }
 
+static void remb_collect_report(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel,
+	struct softmix_bridge_data *softmix_data, struct softmix_channel *sc)
+{
+	int i;
+	unsigned int bitrate;
+
+	/* If there are no video sources that we are a receiver of then we have noone to
+	 * report REMB to.
+	 */
+	if (!AST_VECTOR_SIZE(&sc->video_sources)) {
+		return;
+	}
+
+	/* We evenly divide the available maximum bitrate across the video sources
+	 * to this receiver so each source gets an equal slice.
+	 */
+	bitrate = (sc->remb.br_mantissa << sc->remb.br_exp) / AST_VECTOR_SIZE(&sc->video_sources);
+
+	/* If this receiver has no bitrate yet ignore it */
+	if (!bitrate) {
+		return;
+	}
+
+	for (i = 0; i < AST_VECTOR_SIZE(&sc->video_sources); ++i) {
+		struct softmix_remb_collector *collector;
+
+		/* The collector will always exist if a video source is in our list */
+		collector = AST_VECTOR_GET(&softmix_data->remb_collectors, AST_VECTOR_GET(&sc->video_sources, i));
+
+		if (!collector->bitrate) {
+			collector->bitrate = bitrate;
+			continue;
+		}
+
+		switch (bridge->softmix.video_mode.mode_data.sfu_data.remb_behavior) {
+		case AST_BRIDGE_VIDEO_SFU_REMB_AVERAGE:
+			collector->bitrate = (collector->bitrate + bitrate) / 2;
+			break;
+		case AST_BRIDGE_VIDEO_SFU_REMB_LOWEST:
+			if (bitrate < collector->bitrate) {
+				collector->bitrate = bitrate;
+			}
+			break;
+		case AST_BRIDGE_VIDEO_SFU_REMB_HIGHEST:
+			if (bitrate > collector->bitrate) {
+				collector->bitrate = bitrate;
+			}
+			break;
+		}
+	}
+
+	/* After the report is integrated we reset this to 0 in case they stop producing
+	 * REMB reports.
+	 */
+	sc->remb.br_mantissa = 0;
+	sc->remb.br_exp = 0;
+}
+
+static void remb_send_report(struct ast_bridge_channel *bridge_channel, struct softmix_channel *sc)
+{
+	int i;
+
+	if (!sc->remb_collector) {
+		return;
+	}
+
+	/* We always do this calculation as even when the bitrate is zero the browser
+	 * still prefers it to be accurate instead of lying.
+	 */
+	sc->remb_collector->feedback.remb.br_mantissa = sc->remb_collector->bitrate;
+	sc->remb_collector->feedback.remb.br_exp = 0;
+
+	/* The mantissa only has 18 bits available, so while it exceeds them we bump
+	 * up the exp.
+	 */
+	while (sc->remb_collector->feedback.remb.br_mantissa > 0x3ffff) {
+		sc->remb_collector->feedback.remb.br_mantissa = sc->remb_collector->feedback.remb.br_mantissa >> 1;
+		sc->remb_collector->feedback.remb.br_exp++;
+	}
+
+	for (i = 0; i < AST_VECTOR_SIZE(&bridge_channel->stream_map.to_bridge); ++i) {
+		int bridge_num = AST_VECTOR_GET(&bridge_channel->stream_map.to_bridge, i);
+
+		/* If this stream is not being provided to the bridge there can be no receivers of it
+		 * so therefore no REMB reports.
+		 */
+		if (bridge_num == -1) {
+			continue;
+		}
+
+		/* We need to update the frame with this stream, or else it won't be
+		 * properly routed. We don't use the actual channel stream identifier as
+		 * the bridging core will do the translation from bridge stream identifier to
+		 * channel stream identifier.
+		 */
+		sc->remb_collector->frame.stream_num = bridge_num;
+		ast_bridge_channel_queue_frame(bridge_channel, &sc->remb_collector->frame);
+	}
+
+	sc->remb_collector->bitrate = 0;
+}
+
 static void gather_softmix_stats(struct softmix_stats *stats,
 	const struct softmix_bridge_data *softmix_data,
 	struct ast_bridge_channel *bridge_channel)
@@ -904,10 +1468,15 @@ static void gather_softmix_stats(struct
  * \retval 0, no changes to internal rate
  * \retval 1, internal rate was changed, update all the channels on the next mixing iteration.
  */
-static unsigned int analyse_softmix_stats(struct softmix_stats *stats, struct softmix_bridge_data *softmix_data)
+static unsigned int analyse_softmix_stats(struct softmix_stats *stats,
+		struct softmix_bridge_data *softmix_data, int binaural_active)
 {
 	int i;
 
+	if (binaural_active) {
+		stats->locked_rate = SOFTMIX_BINAURAL_SAMPLE_RATE;
+	}
+
 	/*
 	 * Re-adjust the internal bridge sample rate if
 	 * 1. The bridge's internal sample rate is locked in at a sample
@@ -979,7 +1548,8 @@ static unsigned int analyse_softmix_stat
 	return 0;
 }
 
-static int softmix_mixing_array_init(struct softmix_mixing_array *mixing_array, unsigned int starting_num_entries)
+static int softmix_mixing_array_init(struct softmix_mixing_array *mixing_array,
+		unsigned int starting_num_entries, unsigned int binaural_active)
 {
 	memset(mixing_array, 0, sizeof(*mixing_array));
 	mixing_array->max_num_entries = starting_num_entries;
@@ -987,17 +1557,30 @@ static int softmix_mixing_array_init(str
 		ast_log(LOG_NOTICE, "Failed to allocate softmix mixing structure.\n");
 		return -1;
 	}
+	if (binaural_active) {
+		if (!(mixing_array->chan_pairs = ast_calloc(mixing_array->max_num_entries,
+				sizeof(struct convolve_channel_pair *)))) {
+			ast_log(LOG_NOTICE, "Failed to allocate softmix mixing structure.\n");
+			return -1;
+		}
+	}
 	return 0;
 }
 
-static void softmix_mixing_array_destroy(struct softmix_mixing_array *mixing_array)
+static void softmix_mixing_array_destroy(struct softmix_mixing_array *mixing_array,
+		unsigned int binaural_active)
 {
 	ast_free(mixing_array->buffers);
+	if (binaural_active) {
+		ast_free(mixing_array->chan_pairs);
+	}
 }
 
-static int softmix_mixing_array_grow(struct softmix_mixing_array *mixing_array, unsigned int num_entries)
+static int softmix_mixing_array_grow(struct softmix_mixing_array *mixing_array,
+		unsigned int num_entries, unsigned int binaural_active)
 {
 	int16_t **tmp;
+
 	/* give it some room to grow since memory is cheap but allocations can be expensive */
 	mixing_array->max_num_entries = num_entries;
 	if (!(tmp = ast_realloc(mixing_array->buffers, (mixing_array->max_num_entries * sizeof(int16_t *))))) {
@@ -1005,6 +1588,16 @@ static int softmix_mixing_array_grow(str
 		return -1;
 	}
 	mixing_array->buffers = tmp;
+
+	if (binaural_active) {
+		struct convolve_channel_pair **tmp2;
+		if (!(tmp2 = ast_realloc(mixing_array->chan_pairs,
+				(mixing_array->max_num_entries * sizeof(struct convolve_channel_pair *))))) {
+			ast_log(LOG_NOTICE, "Failed to re-allocate softmix mixing structure.\n");
+			return -1;
+		}
+		mixing_array->chan_pairs = tmp2;
+	}
 	return 0;
 }
 
@@ -1022,6 +1615,10 @@ static int softmix_mixing_loop(struct as
 	struct ast_timer *timer;
 	struct softmix_translate_helper trans_helper;
 	int16_t buf[MAX_DATALEN];
+#ifdef BINAURAL_RENDERING
+	int16_t bin_buf[MAX_DATALEN];
+	int16_t ann_buf[MAX_DATALEN];
+#endif
 	unsigned int stat_iteration_counter = 0; /* counts down, gather stats at zero and reset. */
 	int timingfd;
 	int update_all_rates = 0; /* set this when the internal sample rate has changed */
@@ -1035,7 +1632,8 @@ static int softmix_mixing_loop(struct as
 	ast_timer_set_rate(timer, (1000 / softmix_data->internal_mixing_interval));
 
 	/* Give the mixing array room to grow, memory is cheap but allocations are expensive. */
-	if (softmix_mixing_array_init(&mixing_array, bridge->num_channels + 10)) {
+	if (softmix_mixing_array_init(&mixing_array, bridge->num_channels + 10,
+			bridge->softmix.binaural_active)) {
 		goto softmix_cleanup;
 	}
 
@@ -1049,6 +1647,7 @@ static int softmix_mixing_loop(struct as
 		struct ast_format *cur_slin = ast_format_cache_get_slin_by_rate(softmix_data->internal_rate);
 		unsigned int softmix_samples = SOFTMIX_SAMPLES(softmix_data->internal_rate, softmix_data->internal_mixing_interval);
 		unsigned int softmix_datalen = SOFTMIX_DATALEN(softmix_data->internal_rate, softmix_data->internal_mixing_interval);
+		int remb_update = 0;
 
 		if (softmix_datalen > MAX_DATALEN) {
 			/* This should NEVER happen, but if it does we need to know about it. Almost
@@ -1063,7 +1662,8 @@ static int softmix_mixing_loop(struct as
 
 		/* Grow the mixing array buffer as participants are added. */
 		if (mixing_array.max_num_entries < bridge->num_channels
-			&& softmix_mixing_array_grow(&mixing_array, bridge->num_channels + 5)) {
+			&& softmix_mixing_array_grow(&mixing_array, bridge->num_channels + 5,
+					bridge->softmix.binaural_active)) {
 			goto softmix_cleanup;
 		}
 
@@ -1082,6 +1682,18 @@ static int softmix_mixing_loop(struct as
 			softmix_translate_helper_change_rate(&trans_helper, softmix_data->internal_rate);
 		}
 
+#ifdef BINAURAL_RENDERING
+		check_binaural_position_change(bridge, softmix_data);
+#endif
+
+		/* If we need to do a REMB update to all video sources then do so */
+		if (bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU &&
+			bridge->softmix.video_mode.mode_data.sfu_data.remb_send_interval &&
+			ast_tvdiff_ms(ast_tvnow(), softmix_data->last_remb_update) > bridge->softmix.video_mode.mode_data.sfu_data.remb_send_interval) {
+			remb_update = 1;
+			softmix_data->last_remb_update = ast_tvnow();
+		}
+
 		/* Go through pulling audio from each factory that has it available */
 		AST_LIST_TRAVERSE(&bridge->channels, bridge_channel, entry) {
 			struct softmix_channel *sc = bridge_channel->tech_pvt;
@@ -1093,7 +1705,8 @@ static int softmix_mixing_loop(struct as
 
 			/* Update the sample rate to match the bridge's native sample rate if necessary. */
 			if (update_all_rates) {
-				set_softmix_bridge_data(softmix_data->internal_rate, softmix_data->internal_mixing_interval, bridge_channel, 1);
+				set_softmix_bridge_data(softmix_data->internal_rate,
+						softmix_data->internal_mixing_interval, bridge_channel, 1, -1, -1, -1);
 			}
 
 			/* If stat_iteration_counter is 0, then collect statistics during this mixing interation */
@@ -1109,12 +1722,19 @@ static int softmix_mixing_loop(struct as
 			/* Try to get audio from the factory if available */
 			ast_mutex_lock(&sc->lock);
 			if ((mixing_array.buffers[mixing_array.used_entries] = softmix_process_read_audio(sc, softmix_samples))) {
+#ifdef BINAURAL_RENDERING
+				add_binaural_mixing(bridge, softmix_data, softmix_samples, &mixing_array, sc,
+						ast_channel_name(bridge_channel->chan));
+#endif
 				mixing_array.used_entries++;
 			}
+			if (remb_update) {
+				remb_collect_report(bridge, bridge_channel, softmix_data, sc);
+			}
 			ast_mutex_unlock(&sc->lock);
 		}
 
-		/* mix it like crazy */
+		/* mix it like crazy (non binaural channels)*/
 		memset(buf, 0, softmix_datalen);
 		for (idx = 0; idx < mixing_array.used_entries; ++idx) {
 			for (x = 0; x < softmix_samples; ++x) {
@@ -1122,6 +1742,10 @@ static int softmix_mixing_loop(struct as
 			}
 		}
 
+#ifdef BINAURAL_RENDERING
+		binaural_mixing(bridge, softmix_data, &mixing_array, bin_buf, ann_buf);
+#endif
+
 		/* Next step go through removing the channel's own audio and creating a good frame... */
 		AST_LIST_TRAVERSE(&bridge->channels, bridge_channel, entry) {
 			struct softmix_channel *sc = bridge_channel->tech_pvt;
@@ -1136,22 +1760,37 @@ static int softmix_mixing_loop(struct as
 			/* Make SLINEAR write frame from local buffer */
 			ao2_t_replace(sc->write_frame.subclass.format, cur_slin,
 				"Replace softmix channel slin format");
-			sc->write_frame.datalen = softmix_datalen;
-			sc->write_frame.samples = softmix_samples;
-			memcpy(sc->final_buf, buf, softmix_datalen);
-
+#ifdef BINAURAL_RENDERING
+			if (bridge->softmix.binaural_active && softmix_data->convolve.binaural_active
+					&& sc->binaural) {
+				create_binaural_frame(bridge_channel, sc, bin_buf, ann_buf, softmix_datalen,
+						softmix_samples, buf);
+			} else
+#endif
+			{
+				sc->write_frame.datalen = softmix_datalen;
+				sc->write_frame.samples = softmix_samples;
+				memcpy(sc->final_buf, buf, softmix_datalen);
+			}
 			/* process the softmix channel's new write audio */
-			softmix_process_write_audio(&trans_helper, ast_channel_rawwriteformat(bridge_channel->chan), sc);
+			softmix_process_write_audio(&trans_helper,
+					ast_channel_rawwriteformat(bridge_channel->chan), sc,
+					softmix_data->default_sample_size);
 
 			ast_mutex_unlock(&sc->lock);
 
 			/* A frame is now ready for the channel. */
 			ast_bridge_channel_queue_frame(bridge_channel, &sc->write_frame);
+
+			if (remb_update) {
+				remb_send_report(bridge_channel, sc);
+			}
 		}
 
 		update_all_rates = 0;
 		if (!stat_iteration_counter) {
-			update_all_rates = analyse_softmix_stats(&stats, softmix_data);
+			update_all_rates = analyse_softmix_stats(&stats, softmix_data,
+					bridge->softmix.binaural_active);
 			stat_iteration_counter = SOFTMIX_STAT_INTERVAL;
 		}
 		stat_iteration_counter--;
@@ -1182,7 +1821,7 @@ static int softmix_mixing_loop(struct as
 
 softmix_cleanup:
 	softmix_translate_helper_destroy(&trans_helper);
-	softmix_mixing_array_destroy(&mixing_array);
+	softmix_mixing_array_destroy(&mixing_array, bridge->softmix.binaural_active);
 	return res;
 }
 
@@ -1220,6 +1859,32 @@ static void *softmix_mixing_thread(void
 			continue;
 		}
 
+		if (bridge->softmix.binaural_active && !softmix_data->binaural_init) {
+#ifndef BINAURAL_RENDERING
+			ast_bridge_lock(bridge);
+			bridge->softmix.binaural_active = 0;
+			ast_bridge_unlock(bridge);
+			ast_log(LOG_WARNING, "Bridge: %s: Binaural rendering active by config but not "
+					"compiled.\n", bridge->uniqueid);
+#else
+			/* Set and init binaural data if binaural is activated in the configuration. */
+			softmix_data->internal_rate = SOFTMIX_BINAURAL_SAMPLE_RATE;
+			softmix_data->default_sample_size = SOFTMIX_SAMPLES(softmix_data->internal_rate,
+					softmix_data->internal_mixing_interval);
+			/* If init for binaural processing fails we will fall back to mono audio processing. */
+			if (init_convolve_data(&softmix_data->convolve, softmix_data->default_sample_size)
+					== -1) {
+				ast_bridge_lock(bridge);
+				bridge->softmix.binaural_active = 0;
+				ast_bridge_unlock(bridge);
+				ast_log(LOG_ERROR, "Bridge: %s: Unable to allocate memory for "
+						"binaural processing,  Will only process mono audio.\n",
+						bridge->uniqueid);
+			}
+			softmix_data->binaural_init = 1;
+#endif
+		}
+
 		if (softmix_mixing_loop(bridge)) {
 			/*
 			 * A mixing error occurred.  Sleep and try again later so we
@@ -1246,6 +1911,8 @@ static void softmix_bridge_data_destroy(
 	}
 	ast_mutex_destroy(&softmix_data->lock);
 	ast_cond_destroy(&softmix_data->cond);
+	AST_VECTOR_RESET(&softmix_data->remb_collectors, ao2_cleanup);
+	AST_VECTOR_FREE(&softmix_data->remb_collectors);
 	ast_free(softmix_data);
 }
 
@@ -1271,6 +1938,13 @@ static int softmix_bridge_create(struct
 	softmix_data->internal_rate = SOFTMIX_MIN_SAMPLE_RATE;
 	softmix_data->internal_mixing_interval = DEFAULT_SOFTMIX_INTERVAL;
 
+#ifdef BINAURAL_RENDERING
+	softmix_data->default_sample_size = SOFTMIX_SAMPLES(softmix_data->internal_rate,
+			softmix_data->internal_mixing_interval);
+#endif
+
+	AST_VECTOR_INIT(&softmix_data->remb_collectors, 0);
+
 	bridge->tech_pvt = softmix_data;
 
 	/* Start the mixing thread. */
@@ -1330,11 +2004,241 @@ static void softmix_bridge_destroy(struc
 		ast_debug(1, "Bridge %s: Waiting for mixing thread to die.\n", bridge->uniqueid);
 		pthread_join(thread, NULL);
 	}
-
+#ifdef BINAURAL_RENDERING
+	free_convolve_data(&softmix_data->convolve);
+#endif
 	softmix_bridge_data_destroy(softmix_data);
 	bridge->tech_pvt = NULL;
 }
 
+/*!
+ * \brief Map a source stream to all of its destination streams.
+ *
+ * \param source_stream_name Name of the source stream
+ * \param source_channel_name Name of channel where the source stream originates
+ * \param bridge_stream_position The slot in the bridge where source video will come from
+ * \param participants The bridge_channels in the bridge
+ */
+static void map_source_to_destinations(const char *source_stream_name, const char *source_channel_name,
+	size_t bridge_stream_position, struct ast_bridge_channels_list *participants)
+{
+	struct ast_bridge_channel *participant;
+
+	AST_LIST_TRAVERSE(participants, participant, entry) {
+		int i;
+		struct ast_stream_topology *topology;
+
+		if (!strcmp(source_channel_name, ast_channel_name(participant->chan))) {
+			continue;
+		}
+
+		ast_bridge_channel_lock(participant);
+		ast_channel_lock(participant->chan);
+		topology = ast_channel_get_stream_topology(participant->chan);
+
+		for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
+			struct ast_stream *stream;
+
+			stream = ast_stream_topology_get_stream(topology, i);
+			if (is_video_dest(stream, source_channel_name, source_stream_name)) {
+				struct softmix_channel *sc = participant->tech_pvt;
+
+				AST_VECTOR_REPLACE(&participant->stream_map.to_channel, bridge_stream_position, i);
+				AST_VECTOR_APPEND(&sc->video_sources, bridge_stream_position);
+				break;
+			}
+		}
+		ast_channel_unlock(participant->chan);
+		ast_bridge_channel_unlock(participant);
+	}
+}
+
+/*!
+ * \brief Allocate a REMB collector
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+static struct softmix_remb_collector *remb_collector_alloc(void)
+{
+	struct softmix_remb_collector *collector;
+
+	collector = ao2_alloc_options(sizeof(*collector), NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!collector) {
+		return NULL;
+	}
+
+	collector->frame.frametype = AST_FRAME_RTCP;
+	collector->frame.subclass.integer = AST_RTP_RTCP_PSFB;
+	collector->feedback.fmt = AST_RTP_RTCP_FMT_REMB;
+	collector->frame.data.ptr = &collector->feedback;
+	collector->frame.datalen = sizeof(collector->feedback);
+
+	return collector;
+}
+
+/*!
+ * \brief Setup REMB collection for a particular bridge stream and channel.
+ *
+ * \param bridge The bridge
+ * \param bridge_channel Channel that is collecting REMB information
+ * \param bridge_stream_position The slot in the bridge where source video comes from
+ */
+static void remb_enable_collection(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel,
+	size_t bridge_stream_position)
+{
+	struct softmix_channel *sc = bridge_channel->tech_pvt;
+	struct softmix_bridge_data *softmix_data = bridge->tech_pvt;
+
+	if (!sc->remb_collector) {
+		sc->remb_collector = remb_collector_alloc();
+		if (!sc->remb_collector) {
+			/* This is not fatal. Things will still continue to work but we won't
+			 * produce a REMB report to the sender.
+			 */
+			return;
+		}
+	}
+
+	ao2_ref(sc->remb_collector, +1);
+	if (AST_VECTOR_REPLACE(&softmix_data->remb_collectors, bridge_stream_position,
+		sc->remb_collector)) {
+		ao2_ref(sc->remb_collector, -1);
+	}
+}
+
+/*!
+ * \brief stream_topology_changed callback
+ *
+ * For most video modes, nothing beyond the ordinary is required.
+ * For the SFU case, though, we need to completely remap the streams
+ * in order to ensure video gets directed where it is expected to go.
+ *
+ * \param bridge The bridge
+ * \param bridge_channel Channel whose topology has changed
+ */
+static void softmix_bridge_stream_topology_changed(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel)
+{
+	struct softmix_bridge_data *softmix_data = bridge->tech_pvt;
+	struct softmix_channel *sc;
+	struct ast_bridge_channel *participant;
+	struct ast_vector_int media_types;
+	int nths[AST_MEDIA_TYPE_END] = {0};
+	int idx;
+
+	switch (bridge->softmix.video_mode.mode) {
+	case AST_BRIDGE_VIDEO_MODE_NONE:
+	case AST_BRIDGE_VIDEO_MODE_SINGLE_SRC:
+	case AST_BRIDGE_VIDEO_MODE_TALKER_SRC:
+	default:
+		ast_bridge_channel_stream_map(bridge_channel);
+		return;
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		break;
+	}
+
+	AST_VECTOR_INIT(&media_types, AST_MEDIA_TYPE_END);
+
+	/* The bridge stream identifiers may change, so reset the mapping for them.
+	 * When channels end up getting added back in they'll reuse their existing
+	 * collector and won't need to allocate a new one (unless they were just added).
+	 */
+	for (idx = 0; idx < AST_VECTOR_SIZE(&softmix_data->remb_collectors); ++idx) {
+		ao2_cleanup(AST_VECTOR_GET(&softmix_data->remb_collectors, idx));
+		AST_VECTOR_REPLACE(&softmix_data->remb_collectors, idx, NULL);
+	}
+
+	/* First traversal: re-initialize all of the participants' stream maps */
+	AST_LIST_TRAVERSE(&bridge->channels, participant, entry) {
+		ast_bridge_channel_lock(participant);
+
+		AST_VECTOR_RESET(&participant->stream_map.to_channel, AST_VECTOR_ELEM_CLEANUP_NOOP);
+		AST_VECTOR_RESET(&participant->stream_map.to_bridge, AST_VECTOR_ELEM_CLEANUP_NOOP);
+
+		sc = participant->tech_pvt;
+		AST_VECTOR_RESET(&sc->video_sources, AST_VECTOR_ELEM_CLEANUP_NOOP);
+
+		ast_bridge_channel_unlock(participant);
+	}
+
+	/* Second traversal: Map specific video channels from their source to their destinations.
+	 *
+	 * This is similar to what is done in ast_stream_topology_map(),
+	 * except that video channels are handled differently.  Each video
+	 * source has it's own unique index on the bridge.  This way, a
+	 * particular channel's source video can be distributed to the
+	 * appropriate destination streams on the other channels.
+	 */
+	AST_LIST_TRAVERSE(&bridge->channels, participant, entry) {
+		int i;
+		struct ast_stream_topology *topology;
+
+		ast_bridge_channel_lock(participant);
+		ast_channel_lock(participant->chan);
+
+		topology = ao2_bump(ast_channel_get_stream_topology(participant->chan));
+		if (!topology) {
+			/* Oh, my, we are in trouble. */
+			ast_channel_unlock(participant->chan);
+			ast_bridge_channel_unlock(participant);
+			continue;
+		}
+
+		for (i = 0; i < ast_stream_topology_get_count(topology); ++i) {
+			struct ast_stream *stream = ast_stream_topology_get_stream(topology, i);
+
+			if (is_video_source(stream)) {
+				AST_VECTOR_APPEND(&media_types, AST_MEDIA_TYPE_VIDEO);
+				AST_VECTOR_REPLACE(&participant->stream_map.to_bridge, i, AST_VECTOR_SIZE(&media_types) - 1);
+				/*
+				 * There are cases where we need to bidirectionally send frames, such as for REMB reports
+				 * so we also map back to the channel.
+				 */
+				AST_VECTOR_REPLACE(&participant->stream_map.to_channel, AST_VECTOR_SIZE(&media_types) - 1, i);
+				remb_enable_collection(bridge, participant, AST_VECTOR_SIZE(&media_types) - 1);
+				/*
+				 * Unlock the channel and participant to prevent
+				 * potential deadlock in map_source_to_destinations().
+				 */
+				ast_channel_unlock(participant->chan);
+				ast_bridge_channel_unlock(participant);
+				map_source_to_destinations(ast_stream_get_name(stream), ast_channel_name(participant->chan),
+					AST_VECTOR_SIZE(&media_types) - 1, &bridge->channels);
+				ast_bridge_channel_lock(participant);
+				ast_channel_lock(participant->chan);
+			} else if (ast_stream_get_type(stream) == AST_MEDIA_TYPE_VIDEO) {
+				/* Video stream mapping occurs directly when a video source stream
+				 * is found on a channel. Video streams should otherwise remain
+				 * unmapped.
+				 */
+				AST_VECTOR_REPLACE(&participant->stream_map.to_bridge, i, -1);
+			} else if (ast_stream_get_state(stream) != AST_STREAM_STATE_REMOVED) {
+				/* XXX This is copied from ast_stream_topology_map(). This likely could
+				 * be factored out in some way
+				 */
+				enum ast_media_type type = ast_stream_get_type(stream);
+				int index = AST_VECTOR_GET_INDEX_NTH(&media_types, ++nths[type],
+					type, AST_VECTOR_ELEM_DEFAULT_CMP);
+
+				if (index == -1) {
+					AST_VECTOR_APPEND(&media_types, type);
+					index = AST_VECTOR_SIZE(&media_types) - 1;
+				}
+
+				AST_VECTOR_REPLACE(&participant->stream_map.to_bridge, i, index);
+				AST_VECTOR_REPLACE(&participant->stream_map.to_channel, index, i);
+			}
+		}
+
+		ast_stream_topology_free(topology);
+
+		ast_channel_unlock(participant->chan);
+		ast_bridge_channel_unlock(participant);
+	}
+
+	AST_VECTOR_FREE(&media_types);
+}
+
 static struct ast_bridge_technology softmix_bridge = {
 	.name = "softmix",
 	.capabilities = AST_BRIDGE_CAPABILITY_MULTIMIX,
@@ -1346,11 +2250,296 @@ static struct ast_bridge_technology soft
 	.leave = softmix_bridge_leave,
 	.unsuspend = softmix_bridge_unsuspend,
 	.write = softmix_bridge_write,
+	.stream_topology_changed = softmix_bridge_stream_topology_changed,
+};
+
+#ifdef TEST_FRAMEWORK
+struct stream_parameters {
+	const char *name;
+	const char *formats;
+	enum ast_media_type type;
 };
 
+static struct ast_stream_topology *build_topology(const struct stream_parameters *params, size_t num_streams)
+{
+	struct ast_stream_topology *topology;
+	size_t i;
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		return NULL;
+	}
+
+	for (i = 0; i < num_streams; ++i) {
+		RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+		struct ast_stream *stream;
+
+		caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+		if (!caps) {
+			goto fail;
+		}
+		if (ast_format_cap_update_by_allow_disallow(caps, params[i].formats, 1) < 0) {
+			goto fail;
+		}
+		stream = ast_stream_alloc(params[i].name, params[i].type);
+		if (!stream) {
+			goto fail;
+		}
+		ast_stream_set_formats(stream, caps);
+		if (ast_stream_topology_append_stream(topology, stream) < 0) {
+			ast_stream_free(stream);
+			goto fail;
+		}
+	}
+
+	return topology;
+
+fail:
+	ast_stream_topology_free(topology);
+	return NULL;
+}
+
+static int validate_stream(struct ast_test *test, struct ast_stream *stream,
+	const struct stream_parameters *params)
+{
+	struct ast_format_cap *stream_caps;
+	struct ast_format_cap *params_caps;
+
+	if (ast_stream_get_type(stream) != params->type) {
+		ast_test_status_update(test, "Expected stream type '%s' but got type '%s'\n",
+			ast_codec_media_type2str(params->type),
+			ast_codec_media_type2str(ast_stream_get_type(stream)));
+		return -1;
+	}
+	if (strcmp(ast_stream_get_name(stream), params->name)) {
+		ast_test_status_update(test, "Expected stream name '%s' but got type '%s'\n",
+			params->name, ast_stream_get_name(stream));
+		return -1;
+	}
+
+	stream_caps = ast_stream_get_formats(stream);
+	params_caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!params_caps) {
+		ast_test_status_update(test, "Allocation error on capabilities\n");
+		return -1;
+	}
+	ast_format_cap_update_by_allow_disallow(params_caps, params->formats, 1);
+
+	if (!ast_format_cap_identical(stream_caps, params_caps)) {
+		ast_test_status_update(test, "Formats are not as expected on stream '%s'\n",
+			ast_stream_get_name(stream));
+		ao2_cleanup(params_caps);
+		return -1;
+	}
+
+	ao2_cleanup(params_caps);
+	return 0;
+}
+
+static int validate_original_streams(struct ast_test *test, struct ast_stream_topology *topology,
+	const struct stream_parameters *params, size_t num_streams)
+{
+	int i;
+
+	if (ast_stream_topology_get_count(topology) < num_streams) {
+		ast_test_status_update(test, "Topology only has %d streams. Needs to have at least %zu\n",
+			ast_stream_topology_get_count(topology), num_streams);
+		return -1;
+	}
+
+	for (i = 0; i < num_streams; ++i) {
+		if (validate_stream(test, ast_stream_topology_get_stream(topology, i), &params[i])) {
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+AST_TEST_DEFINE(sfu_append_source_streams)
+{
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	static const struct stream_parameters bob_streams[] = {
+		{ "bob_audio", "ulaw,alaw,g722,opus", AST_MEDIA_TYPE_AUDIO, },
+		{ "bob_video", "h264,vp8", AST_MEDIA_TYPE_VIDEO, },
+	};
+	static const struct stream_parameters alice_streams[] = {
+		{ "alice_audio", "ulaw,opus", AST_MEDIA_TYPE_AUDIO, },
+		{ "alice_video", "vp8", AST_MEDIA_TYPE_VIDEO, },
+	};
+	static const struct stream_parameters alice_dest_stream = {
+		"softbridge_dest_PJSIP/Bob-00000001_bob_video", "h264,vp8", AST_MEDIA_TYPE_VIDEO,
+	};
+	static const struct stream_parameters bob_dest_stream = {
+		"softbridge_dest_PJSIP/Alice-00000000_alice_video", "vp8", AST_MEDIA_TYPE_VIDEO,
+	};
+	struct ast_stream_topology *topology_alice = NULL;
+	struct ast_stream_topology *topology_bob = NULL;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "sfu_append_source_streams";
+		info->category = "/bridges/bridge_softmix/";
+		info->summary = "Test appending of video streams";
+		info->description =
+			"This tests does stuff.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology_alice = build_topology(alice_streams, ARRAY_LEN(alice_streams));
+	if (!topology_alice) {
+		goto end;
+	}
+
+	topology_bob = build_topology(bob_streams, ARRAY_LEN(bob_streams));
+	if (!topology_bob) {
+		goto end;
+	}
+
+	if (append_source_streams(topology_alice, "PJSIP/Bob-00000001", NULL, topology_bob)) {
+		ast_test_status_update(test, "Failed to append Bob's streams to Alice\n");
+		goto end;
+	}
+
+	if (ast_stream_topology_get_count(topology_alice) != 3) {
+		ast_test_status_update(test, "Alice's topology isn't large enough! It's %d but needs to be %d\n",
+			ast_stream_topology_get_count(topology_alice), 3);
+		goto end;
+	}
+
+	if (validate_original_streams(test, topology_alice, alice_streams, ARRAY_LEN(alice_streams))) {
+		goto end;
+	}
+
+	if (validate_stream(test, ast_stream_topology_get_stream(topology_alice, 2), &alice_dest_stream)) {
+		goto end;
+	}
+
+	if (append_source_streams(topology_bob, "PJSIP/Alice-00000000", NULL, topology_alice)) {
+		ast_test_status_update(test, "Failed to append Alice's streams to Bob\n");
+		goto end;
+	}
+
+	if (ast_stream_topology_get_count(topology_bob) != 3) {
+		ast_test_status_update(test, "Bob's topology isn't large enough! It's %d but needs to be %d\n",
+			ast_stream_topology_get_count(topology_bob), 3);
+		goto end;
+	}
+
+	if (validate_original_streams(test, topology_bob, bob_streams, ARRAY_LEN(bob_streams))) {
+		goto end;
+	}
+
+	if (validate_stream(test, ast_stream_topology_get_stream(topology_bob, 2), &bob_dest_stream)) {
+		goto end;
+	}
+
+	res = AST_TEST_PASS;
+
+end:
+	ast_stream_topology_free(topology_alice);
+	ast_stream_topology_free(topology_bob);
+	return res;
+}
+
+AST_TEST_DEFINE(sfu_remove_destination_streams)
+{
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	static const struct stream_parameters params[] = {
+		{ "alice_audio", "ulaw,alaw,g722,opus", AST_MEDIA_TYPE_AUDIO, },
+		{ "alice_video", "h264,vp8", AST_MEDIA_TYPE_VIDEO, },
+		{ "softbridge_dest_PJSIP/Bob-00000001_video", "vp8", AST_MEDIA_TYPE_VIDEO, },
+		{ "softbridge_dest_PJSIP/Carol-00000002_video", "h264", AST_MEDIA_TYPE_VIDEO, },
+	};
+	static const struct {
+		const char *channel_name;
+		int num_streams;
+		int params_index[4];
+	} removal_results[] = {
+		{ "PJSIP/Bob-00000001", 4, { 0, 1, 2, 3 }, },
+		{ "PJSIP/Edward-00000004", 4, { 0, 1, 2, 3 }, },
+		{ "", 4, { 0, 1, 2, 3 }, },
+	};
+	struct ast_stream_topology *orig = NULL;
+	int i;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "sfu_remove_destination_streams";
+		info->category = "/bridges/bridge_softmix/";
+		info->summary = "Test removal of destination video streams";
+		info->description =
+			"This tests does stuff.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	orig = build_topology(params, ARRAY_LEN(params));
+	if (!orig) {
+		ast_test_status_update(test, "Unable to build initial stream topology\n");
+		goto end;
+	}
+
+	for (i = 0; i < ARRAY_LEN(removal_results); ++i) {
+		int j;
+
+		remove_destination_streams(orig, removal_results[i].channel_name);
+
+		if (ast_stream_topology_get_count(orig) != removal_results[i].num_streams) {
+			ast_test_status_update(test, "Resulting topology has %d streams, when %d are expected\n",
+				ast_stream_topology_get_count(orig), removal_results[i].num_streams);
+			goto end;
+		}
+
+		for (j = 0; j < removal_results[i].num_streams; ++j) {
+			struct ast_stream *actual;
+			struct ast_stream *expected;
+			int orig_index;
+
+			actual = ast_stream_topology_get_stream(orig, j);
+
+			orig_index = removal_results[i].params_index[j];
+			expected = ast_stream_topology_get_stream(orig, orig_index);
+
+			if (!ast_format_cap_identical(ast_stream_get_formats(actual),
+				ast_stream_get_formats(expected))) {
+				struct ast_str *expected_str;
+				struct ast_str *actual_str;
+
+				expected_str = ast_str_alloca(64);
+				actual_str = ast_str_alloca(64);
+
+				ast_test_status_update(test, "Mismatch between expected (%s) and actual (%s) stream formats\n",
+					ast_format_cap_get_names(ast_stream_get_formats(expected), &expected_str),
+					ast_format_cap_get_names(ast_stream_get_formats(actual), &actual_str));
+				goto end;
+			}
+
+			if (is_video_dest(actual, removal_results[i].channel_name, NULL) &&
+				ast_stream_get_state(actual) != AST_STREAM_STATE_REMOVED) {
+				ast_test_status_update(test, "Removed stream %s does not have a state of removed\n", ast_stream_get_name(actual));
+				goto end;
+			}
+		}
+	}
+
+	res = AST_TEST_PASS;
+
+end:
+	ast_stream_topology_free(orig);
+	return res;
+}
+
+#endif
+
 static int unload_module(void)
 {
 	ast_bridge_technology_unregister(&softmix_bridge);
+	AST_TEST_UNREGISTER(sfu_append_source_streams);
+	AST_TEST_UNREGISTER(sfu_remove_destination_streams);
 	return 0;
 }
 
@@ -1360,6 +2549,8 @@ static int load_module(void)
 		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
+	AST_TEST_REGISTER(sfu_append_source_streams);
+	AST_TEST_REGISTER(sfu_remove_destination_streams);
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
diff -pruN 1:13.23.1~dfsg-2/bridges/bridges.xml 1:16.1.1~dfsg-1/bridges/bridges.xml
--- 1:13.23.1~dfsg-2/bridges/bridges.xml	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/bridges.xml	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,8 @@
+	<member name="binaural_rendering_in_bridge_softmix"
+	displayname="Enable binaural rendering in bridge_softmix"
+	remove_on_change="bridges/bridge_softmix.o bridges/bridge_softmix.so bridges/bridge_softmix/bridge_softmix_binaural.o">
+		<support_level>option</support_level>
+		<depend>bridge_softmix</depend>
+		<depend>fftw3</depend>
+		<defaultenabled>no</defaultenabled>
+	</member>
diff -pruN 1:13.23.1~dfsg-2/bridges/Makefile 1:16.1.1~dfsg-1/bridges/Makefile
--- 1:13.23.1~dfsg-2/bridges/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/bridges/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -14,7 +14,15 @@
 MODULE_PREFIX=bridge
 MENUSELECT_CATEGORY=BRIDGES
 MENUSELECT_DESCRIPTION=Bridging Modules
+MODULE_EXCLUDE=binaural_rendering_in_bridge_softmix
 
 all: _all
 
+ifeq ($(findstring binaural_rendering_in_bridge_softmix,$(MENUSELECT_BRIDGES)),)
+bridge_softmix.o bridge_softmix/bridge_softmix_binaural.o: _ASTCFLAGS+=-DBINAURAL_RENDERING
+bridge_softmix.so: LIBS+=$(FFTW3_LIB)
+endif
+
 include $(ASTTOPDIR)/Makefile.moddir_rules
+
+$(call MOD_ADD_C,bridge_softmix,$(wildcard bridge_softmix/*.c))
diff -pruN 1:13.23.1~dfsg-2/build_tools/cflags.xml 1:16.1.1~dfsg-1/build_tools/cflags.xml
--- 1:13.23.1~dfsg-2/build_tools/cflags.xml	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/build_tools/cflags.xml	2018-12-26 18:45:13.000000000 +0000
@@ -10,15 +10,12 @@
 		<member name="DEBUG_THREADS" displayname="Enable Thread Debugging">
 			<support_level>core</support_level>
 		</member>
-		<member name="REF_DEBUG" displayname="Enable reference count debugging">
+		<member name="REF_DEBUG" displayname="Enable reference count debugging by default">
 			<support_level>extended</support_level>
 		</member>
 		<member name="AO2_DEBUG" displayname="Enable internal Astobj2 debugging">
 			<support_level>extended</support_level>
 		</member>
-		<member name="STATIC_BUILD" displayname="Build static binaries">
-			<support_level>extended</support_level>
-		</member>
 		<member name="DEBUG_FD_LEAKS" displayname="Enable File Descriptor Leak Detection">
 			<support_level>core</support_level>
 		</member>
diff -pruN 1:13.23.1~dfsg-2/build_tools/get_documentation.py 1:16.1.1~dfsg-1/build_tools/get_documentation.py
--- 1:13.23.1~dfsg-2/build_tools/get_documentation.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/build_tools/get_documentation.py	2018-12-26 18:45:13.000000000 +0000
@@ -9,11 +9,8 @@ the GNU General Public License Version 2
 '''
 
 import sys
-import os
 import xml.dom.minidom
 
-from xml.dom.minidom import Element
-
 
 def get_manager_event_method_type(candidate_string):
     if "ast_manager_event_multichan" in candidate_string:
diff -pruN 1:13.23.1~dfsg-2/build_tools/make_buildopts_h 1:16.1.1~dfsg-1/build_tools/make_buildopts_h
--- 1:13.23.1~dfsg-2/build_tools/make_buildopts_h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/build_tools/make_buildopts_h	2018-12-26 18:45:13.000000000 +0000
@@ -14,23 +14,36 @@ END
 if ${GREP} "AST_DEVMODE" makeopts | ${GREP} -q "yes"
 then
 	echo "#define AST_DEVMODE 1"
-	BUILDOPTS="AST_DEVMODE"
+	# AST_DEVMODE is no longer an API/ABI affecting option so it no longer
+	# gets added to BUILDOPTS.
 fi
 
 TMP=`${GREP} -e "^MENUSELECT_CFLAGS" menuselect.makeopts | sed 's/MENUSELECT_CFLAGS\=//g' | sed 's/-D//g'`
 for x in ${TMP}; do
-	echo "#define ${x} 1"
-	if test "${x}" = "DONT_OPTIMIZE" \
+	if test "${x}" = "AO2_DEBUG" \
 			-o "${x}" = "BETTER_BACKTRACES" \
-			-o "${x}" = "LOTS_OF_SPANS" \
 			-o "${x}" = "BUILD_NATIVE" \
-			-o "${x}" = "AO2_DEBUG" \
-			-o "${x}" = "REBUILD_PARSERS" \
-			-o "${x}" = "RADIO_RELAX" \
+			-o "${x}" = "COMPILE_DOUBLE" \
+			-o "${x}" = "DEBUG_CHAOS" \
 			-o "${x}" = "DEBUG_SCHEDULER" \
 			-o "${x}" = "DETECT_DEADLOCKS" \
-			-o "${x}" = "DUMP_SCHEDULER" ; then
-		# These aren't ABI affecting options, keep them out of AST_BUILDOPTS
+			-o "${x}" = "DONT_OPTIMIZE" \
+			-o "${x}" = "DUMP_SCHEDULER" \
+			-o "${x}" = "LOTS_OF_SPANS" \
+			-o "${x}" = "MALLOC_DEBUG" \
+			-o "${x}" = "RADIO_RELAX" \
+			-o "${x}" = "REBUILD_PARSERS" \
+			-o "${x}" = "REF_DEBUG" \
+			-o "${x}" = "USE_HOARD_ALLOCATOR" ; then
+		# These options are only for specific sources and have no effect on public ABI.
+		# Keep them out of buildopts.h so ccache does not invalidate all sources.
+		continue
+	fi
+
+	echo "#define ${x} 1"
+	if test "${x}" = "LOW_MEMORY" ; then
+		# LOW_MEMORY isn't an ABI affecting option but it is used in many sources
+		# so it gets defined globally but is not included in AST_BUILTOPTS.
 		continue
 	fi
 	if test "x${BUILDOPTS}" != "x" ; then
diff -pruN 1:13.23.1~dfsg-2/build_tools/menuselect-deps.in 1:16.1.1~dfsg-1/build_tools/menuselect-deps.in
--- 1:13.23.1~dfsg-2/build_tools/menuselect-deps.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/build_tools/menuselect-deps.in	2018-12-26 18:45:13.000000000 +0000
@@ -1,9 +1,11 @@
 ALSA=@PBX_ALSA@
 BLUETOOTH=@PBX_BLUETOOTH@
+BEANSTALK=@PBX_BEANSTALK@
 COROSYNC=@PBX_COROSYNC@
 CRYPTO=@PBX_CRYPTO@
 BFD=@PBX_BFD@
 BISON=@PBX_BISON@
+CODEC2=@PBX_CODEC2@
 CURL=@PBX_CURL@
 DAHDI=@PBX_DAHDI@
 DLADDR=@PBX_DLADDR@
@@ -52,8 +54,10 @@ PORTAUDIO=@PBX_PORTAUDIO@
 PRI=@PBX_PRI@
 OPENR2=@PBX_OPENR2@
 RESAMPLE=@PBX_RESAMPLE@
+FFTW3=@PBX_FFTW3@
 RADIUS=@PBX_RADIUS@
 LAUNCHD=@PBX_LAUNCHD@
+SNDFILE=@PBX_SNDFILE@
 SPANDSP=@PBX_SPANDSP@
 SPEEX=@PBX_SPEEX@
 SPEEXDSP=@PBX_SPEEXDSP@
@@ -66,6 +70,7 @@ OPENSSL=@PBX_OPENSSL@
 SUPPSERV=@PBX_SUPPSERV@
 SYSLOG=@PBX_SYSLOG@
 TONEZONE=@PBX_TONEZONE@
+UNBOUND=@PBX_UNBOUND@
 UNIXODBC=@PBX_UNIXODBC@
 VORBIS=@PBX_VORBIS@
 VPB=@PBX_VPB@
@@ -77,3 +82,4 @@ HAVE_ADDRESS_SANITIZER=@AST_ADDRESS_SANI
 HAVE_LEAK_SANITIZER=@AST_LEAK_SANITIZER@
 HAVE_THREAD_SANITIZER=@AST_THREAD_SANITIZER@
 HAVE_UNDEFINED_SANITIZER=@AST_UNDEFINED_SANITIZER@
+NO_BINARY_MODULES=@PBX_NO_BINARY_MODULES@
diff -pruN 1:13.23.1~dfsg-2/build_tools/post_process_documentation.py 1:16.1.1~dfsg-1/build_tools/post_process_documentation.py
--- 1:13.23.1~dfsg-2/build_tools/post_process_documentation.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/build_tools/post_process_documentation.py	2018-12-26 18:45:13.000000000 +0000
@@ -9,11 +9,9 @@ the GNU General Public License Version 2
 '''
 
 import sys
-import os
 import optparse
-import xml.dom.minidom
 
-from xml.dom.minidom import Element, parse
+from xml.dom.minidom import parse
 
 
 def merge_parameter_information(managerEvent):
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_adaptive_odbc.c 1:16.1.1~dfsg-1/cdr/cdr_adaptive_odbc.c
--- 1:13.23.1~dfsg-2/cdr/cdr_adaptive_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_adaptive_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <time.h>
 
@@ -83,6 +81,7 @@ struct tables {
 	char *connection;
 	char *table;
 	char *schema;
+	char quoted_identifiers;
 	unsigned int usegmtime:1;
 	AST_LIST_HEAD_NOLOCK(odbc_columns, columns) columns;
 	AST_RWLIST_ENTRY(tables) list;
@@ -102,6 +101,7 @@ static int load_config(void)
 	char connection[40];
 	char table[40];
 	char schema[40];
+	char quoted_identifiers;
 	int lenconnection, lentable, lenschema, usegmtime = 0;
 	SQLLEN sqlptr;
 	int res = 0;
@@ -150,6 +150,16 @@ static int load_config(void)
 		ast_copy_string(schema, tmp, sizeof(schema));
 		lenschema = strlen(schema);
 
+		if (ast_strlen_zero(tmp = ast_variable_retrieve(cfg, catg, "quoted_identifiers"))) {
+			tmp = "";
+		}
+		quoted_identifiers = tmp[0];
+		if (strlen(tmp) > 1) {
+			ast_log(LOG_ERROR, "The quoted_identifiers setting only accepts a single character,"
+				" while a value of '%s' was provided. This option has been disabled as a result.\n", tmp);
+			quoted_identifiers = '\0';
+		}
+
 		res = SQLAllocHandle(SQL_HANDLE_STMT, obj->con, &stmt);
 		if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
 			ast_log(LOG_WARNING, "SQL Alloc Handle failed on connection '%s'!\n", connection);
@@ -165,7 +175,7 @@ static int load_config(void)
 			continue;
 		}
 
-		tableptr = ast_calloc(sizeof(char), sizeof(*tableptr) + lenconnection + 1 + lentable + 1 + lenschema + 1);
+		tableptr = ast_calloc(sizeof(char), sizeof(*tableptr) + lenconnection + 1 + lentable + 1 + lenschema + 1 + 1);
 		if (!tableptr) {
 			ast_log(LOG_ERROR, "Out of memory creating entry for table '%s' on connection '%s'%s%s%s\n", table, connection,
 				lenschema ? " (schema '" : "", lenschema ? schema : "", lenschema ? "')" : "");
@@ -182,6 +192,7 @@ static int load_config(void)
 		ast_copy_string(tableptr->connection, connection, lenconnection + 1);
 		ast_copy_string(tableptr->table, table, lentable + 1);
 		ast_copy_string(tableptr->schema, schema, lenschema + 1);
+		tableptr->quoted_identifiers = quoted_identifiers;
 
 		ast_verb(3, "Found adaptive CDR table %s@%s.\n", tableptr->table, tableptr->connection);
 
@@ -346,11 +357,11 @@ static SQLHSTMT generic_prepare(struct o
 	return stmt;
 }
 
-#define LENGTHEN_BUF1(size)														\
+#define LENGTHEN_BUF(size, var_sql)														\
 			do {																\
 				/* Lengthen buffer, if necessary */								\
-				if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) {		\
-					if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 1) / 512 + 1) * 512) != 0) { \
+				if (ast_str_strlen(var_sql) + size + 1 > ast_str_size(var_sql)) {		\
+					if (ast_str_make_space(&var_sql, ((ast_str_size(var_sql) + size + 1) / 512 + 1) * 512) != 0) { \
 						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR '%s:%s' failed.\n", tableptr->connection, tableptr->table); \
 						ast_free(sql);											\
 						ast_free(sql2);											\
@@ -360,18 +371,10 @@ static SQLHSTMT generic_prepare(struct o
 				}																\
 			} while (0)
 
-#define LENGTHEN_BUF2(size)														\
-			do {																\
-				if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) {		\
-					if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) { \
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR '%s:%s' failed.\n", tableptr->connection, tableptr->table); \
-						ast_free(sql);											\
-						ast_free(sql2);											\
-						AST_RWLIST_UNLOCK(&odbc_tables);						\
-						return -1;												\
-					}															\
-				}																\
-			} while (0)
+#define LENGTHEN_BUF1(size)	\
+	LENGTHEN_BUF(size, sql);
+#define LENGTHEN_BUF2(size)	\
+	LENGTHEN_BUF(size, sql2);
 
 static int odbc_log(struct ast_cdr *cdr)
 {
@@ -383,6 +386,8 @@ static int odbc_log(struct ast_cdr *cdr)
 	char colbuf[1024], *colptr;
 	SQLHSTMT stmt = NULL;
 	SQLLEN rows = 0;
+	char *separator;
+	int quoted = 0;
 
 	if (!sql || !sql2) {
 		if (sql)
@@ -400,11 +405,28 @@ static int odbc_log(struct ast_cdr *cdr)
 	}
 
 	AST_LIST_TRAVERSE(&odbc_tables, tableptr, list) {
-		int first = 1;
+		separator = "";
+
+		quoted = 0;
+		if (tableptr->quoted_identifiers != '\0'){
+			quoted = 1;
+		}
+
 		if (ast_strlen_zero(tableptr->schema)) {
-			ast_str_set(&sql, 0, "INSERT INTO %s (", tableptr->table);
+			if (quoted) {
+				ast_str_set(&sql, 0, "INSERT INTO %c%s%c (",
+					tableptr->quoted_identifiers, tableptr->table, tableptr->quoted_identifiers );
+			}else{
+				ast_str_set(&sql, 0, "INSERT INTO %s (", tableptr->table);
+			}
 		} else {
-			ast_str_set(&sql, 0, "INSERT INTO %s.%s (", tableptr->schema, tableptr->table);
+			if (quoted) {
+				ast_str_set(&sql, 0, "INSERT INTO %c%s%c.%c%s%c (",
+						tableptr->quoted_identifiers, tableptr->schema, tableptr->quoted_identifiers,
+						tableptr->quoted_identifiers, tableptr->table,  tableptr->quoted_identifiers);
+			}else{
+				ast_str_set(&sql, 0, "INSERT INTO %s.%s (", tableptr->schema, tableptr->table);
+			}
 		}
 		ast_str_set(&sql2, 0, " VALUES (");
 
@@ -485,11 +507,10 @@ static int odbc_log(struct ast_cdr *cdr)
 						}
 					}
 
-					ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 					LENGTHEN_BUF2(strlen(colptr));
 
 					/* Encode value, with escaping */
-					ast_str_append(&sql2, 0, "%s'", first ? "" : ",");
+					ast_str_append(&sql2, 0, "%s'", separator);
 					for (tmp = colptr; *tmp; tmp++) {
 						if (*tmp == '\'') {
 							ast_str_append(&sql2, 0, "''");
@@ -521,9 +542,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							year += 2000;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(17);
-						ast_str_append(&sql2, 0, "%s{ d '%04d-%02d-%02d' }", first ? "" : ",", year, month, day);
+						ast_str_append(&sql2, 0, "%s{ d '%04d-%02d-%02d' }", separator, year, month, day);
 					}
 					break;
 				case SQL_TYPE_TIME:
@@ -538,9 +558,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(15);
-						ast_str_append(&sql2, 0, "%s{ t '%02d:%02d:%02d' }", first ? "" : ",", hour, minute, second);
+						ast_str_append(&sql2, 0, "%s{ t '%02d:%02d:%02d' }", separator, hour, minute, second);
 					}
 					break;
 				case SQL_TYPE_TIMESTAMP:
@@ -567,9 +586,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							year += 2000;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(26);
-						ast_str_append(&sql2, 0, "%s{ ts '%04d-%02d-%02d %02d:%02d:%02d' }", first ? "" : ",", year, month, day, hour, minute, second);
+						ast_str_append(&sql2, 0, "%s{ ts '%04d-%02d-%02d %02d:%02d:%02d' }", separator, year, month, day, hour, minute, second);
 					}
 					break;
 				case SQL_INTEGER:
@@ -582,9 +600,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(12);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_BIGINT:
@@ -597,9 +614,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(24);
-						ast_str_append(&sql2, 0, "%s%lld", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%lld", separator, integer);
 					}
 					break;
 				case SQL_SMALLINT:
@@ -612,9 +628,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(6);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_TINYINT:
@@ -627,9 +642,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(4);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_BIT:
@@ -644,9 +658,8 @@ static int odbc_log(struct ast_cdr *cdr)
 						if (integer != 0)
 							integer = 1;
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_NUMERIC:
@@ -677,9 +690,8 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(entry->decimals);
-						ast_str_append(&sql2, 0, "%s%*.*lf", first ? "" : ",", entry->decimals, entry->radix, number);
+						ast_str_append(&sql2, 0, "%s%*.*lf", separator, entry->decimals, entry->radix, number);
 					}
 					break;
 				case SQL_FLOAT:
@@ -711,16 +723,20 @@ static int odbc_log(struct ast_cdr *cdr)
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
 						LENGTHEN_BUF2(entry->decimals);
-						ast_str_append(&sql2, 0, "%s%lf", first ? "" : ",", number);
+						ast_str_append(&sql2, 0, "%s%lf", separator, number);
 					}
 					break;
 				default:
 					ast_log(LOG_WARNING, "Column type %d (field '%s:%s:%s') is unsupported at this time.\n", entry->type, tableptr->connection, tableptr->table, entry->name);
 					continue;
 				}
-				first = 0;
+				if (quoted) {
+					ast_str_append(&sql, 0, "%s%c%s%c", separator, tableptr->quoted_identifiers, entry->name, tableptr->quoted_identifiers);
+				} else {
+					ast_str_append(&sql, 0, "%s%s", separator, entry->name);
+				}
+				separator = ", ";
 			} else if (entry->filtervalue
 				&& ((!entry->negatefiltervalue && entry->filtervalue[0] != '\0')
 					|| (entry->negatefiltervalue && entry->filtervalue[0] == '\0'))) {
@@ -815,4 +831,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr,res_odbc",
 );
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_beanstalkd.c 1:16.1.1~dfsg-1/cdr/cdr_beanstalkd.c
--- 1:13.23.1~dfsg-2/cdr/cdr_beanstalkd.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_beanstalkd.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,270 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017
+ *
+ * Nir Simionovich <nirs@greenfieldtech.net>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Asterisk Beanstalkd CDR records.
+ *
+ * This module requires the beanstalk-client library, avaialble from
+ * https://github.com/deepfryed/beanstalk-client
+ *
+ * See also
+ * \arg \ref AstCDR
+ * \ingroup cdr_drivers
+ */
+
+/*! \li \ref cdr_beanstalkd.c uses the configuration file \ref cdr_beanstalkd.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*!
+ * \page cdr_beanstalkd.conf cdr_beanstalkd.conf
+ * \verbinclude cdr_beanstalkd.conf.sample
+ */
+
+/*** MODULEINFO
+	<depend>beanstalk</depend>
+	<support_level>extended</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <time.h>
+#include <stdio.h>
+
+#include "beanstalk.h"
+#include "asterisk/channel.h"
+#include "asterisk/cdr.h"
+#include "asterisk/module.h"
+#include "asterisk/utils.h"
+#include "asterisk/manager.h"
+#include "asterisk/config.h"
+#include "asterisk/pbx.h"
+#include "asterisk/json.h"
+
+#define DATE_FORMAT    "%Y-%m-%d %T"
+#define CONF_FILE    "cdr_beanstalkd.conf"
+#define BEANSTALK_JOB_SIZE 4096
+#define BEANSTALK_JOB_PRIORITY 99
+#define BEANSTALK_JOB_TTR 60
+#define BEANSTALK_JOB_DELAY 0
+#define DEFAULT_BEANSTALK_HOST "127.0.0.1"
+#define DEFAULT_BEANSTALK_PORT 11300
+#define DEFAULT_BEANSTALK_TUBE "asterisk-cdr"
+
+static const char name[] = "cdr_beanstalkd";
+
+static int enablecdr = 0;
+static char *bs_host;
+static int bs_port;
+static char *bs_tube;
+static int priority;
+
+AST_RWLOCK_DEFINE_STATIC(config_lock);
+
+static int beanstalk_put(struct ast_cdr *cdr);
+
+static int load_config(int reload) {
+	char *cat = NULL;
+	struct ast_config *cfg;
+	struct ast_variable *v;
+	struct ast_flags config_flags = {reload ? CONFIG_FLAG_FILEUNCHANGED : 0};
+	int newenablecdr = 0;
+
+	cfg = ast_config_load(CONF_FILE, config_flags);
+	if (cfg == CONFIG_STATUS_FILEUNCHANGED) {
+		return 0;
+	}
+
+	if (cfg == CONFIG_STATUS_FILEINVALID) {
+		ast_log(LOG_ERROR, "Config file '%s' could not be parsed\n", CONF_FILE);
+		return -1;
+	}
+
+	if (!cfg) {
+		/* Standard configuration */
+		ast_log(LOG_WARNING, "Failed to load configuration file. Module not activated.\n");
+		if (enablecdr) {
+			ast_cdr_backend_suspend(name);
+		}
+		enablecdr = 0;
+		return -1;
+	}
+
+	if (reload) {
+		ast_rwlock_wrlock(&config_lock);
+		ast_free(bs_host);
+		ast_free(bs_tube);
+	}
+
+	/* Bootstrap the default configuration */
+	bs_host = ast_strdup(DEFAULT_BEANSTALK_HOST);
+	bs_port = DEFAULT_BEANSTALK_PORT;
+	bs_tube = ast_strdup(DEFAULT_BEANSTALK_TUBE);
+	priority = BEANSTALK_JOB_PRIORITY;
+
+	while ((cat = ast_category_browse(cfg, cat))) {
+		if (!strcasecmp(cat, "general")) {
+			v = ast_variable_browse(cfg, cat);
+			while (v) {
+
+				if (!strcasecmp(v->name, "enabled")) {
+					newenablecdr = ast_true(v->value);
+				} else if (!strcasecmp(v->name, "host")) {
+					ast_free(bs_host);
+					bs_host = ast_strdup(v->value);
+				} else if (!strcasecmp(v->name, "port")) {
+					bs_port = atoi(v->value);
+				} else if (!strcasecmp(v->name, "tube")) {
+					ast_free(bs_tube);
+					bs_tube = ast_strdup(v->value);
+				} else if (!strcasecmp(v->name, "priority")) {
+					priority = atoi(v->value);
+				}
+				v = v->next;
+
+			}
+		}
+	}
+
+	if (reload) {
+		ast_rwlock_unlock(&config_lock);
+	}
+
+	ast_config_destroy(cfg);
+
+	if (!newenablecdr) {
+		ast_cdr_backend_suspend(name);
+	} else if (newenablecdr) {
+		ast_cdr_backend_unsuspend(name);
+		ast_log(LOG_NOTICE, "Added beanstalkd server %s at port %d with tube %s", bs_host, bs_port, bs_tube);
+	}
+	enablecdr = newenablecdr;
+
+	return 0;
+}
+
+static int beanstalk_put(struct ast_cdr *cdr) {
+	struct ast_tm timeresult;
+	char strAnswerTime[80] = "";
+	char strStartTime[80];
+	char strEndTime[80];
+	char *cdr_buffer;
+	int bs_id;
+	int bs_socket;
+	struct ast_json *t_cdr_json;
+
+	if (!enablecdr) {
+		return 0;
+	}
+
+	ast_rwlock_rdlock(&config_lock);
+	bs_socket = bs_connect(bs_host, bs_port);
+
+	if (bs_use(bs_socket, bs_tube) != BS_STATUS_OK) {
+		ast_log(LOG_ERROR, "Connection to Beanstalk tube %s @ %s:%d had failed", bs_tube, bs_host, bs_port);
+		ast_rwlock_unlock(&config_lock);
+		return 0;
+	}
+
+	ast_localtime(&cdr->start, &timeresult, NULL);
+	ast_strftime(strStartTime, sizeof(strStartTime), DATE_FORMAT, &timeresult);
+
+	if (cdr->answer.tv_sec) {
+		ast_localtime(&cdr->answer, &timeresult, NULL);
+		ast_strftime(strAnswerTime, sizeof(strAnswerTime), DATE_FORMAT, &timeresult);
+	}
+
+	ast_localtime(&cdr->end, &timeresult, NULL);
+	ast_strftime(strEndTime, sizeof(strEndTime), DATE_FORMAT, &timeresult);
+
+	ast_rwlock_unlock(&config_lock);
+
+	t_cdr_json = ast_json_pack("{s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:i, s:i, s:s, s:s, s:s, s:s}",
+							   "AccountCode", S_OR(cdr->accountcode, ""),
+							   "Source", S_OR(cdr->src, ""),
+							   "Destination", S_OR(cdr->dst, ""),
+							   "DestinationContext", S_OR(cdr->dcontext, ""),
+							   "CallerID", S_OR(cdr->clid, ""),
+							   "Channel", S_OR(cdr->channel, ""),
+							   "DestinationChannel", S_OR(cdr->dstchannel, ""),
+							   "LastApplication", S_OR(cdr->lastapp, ""),
+							   "LastData", S_OR(cdr->lastdata, ""),
+							   "StartTime", S_OR(strStartTime, ""),
+							   "AnswerTime", S_OR(strAnswerTime, ""),
+							   "EndTime", S_OR(strEndTime, ""),
+							   "Duration", cdr->duration,
+							   "Billsec", cdr->billsec,
+							   "Disposition", S_OR(ast_cdr_disp2str(cdr->disposition), ""),
+							   "AMAFlags", S_OR(ast_channel_amaflags2string(cdr->amaflags), ""),
+							   "UniqueID", S_OR(cdr->uniqueid, ""),
+							   "UserField", S_OR(cdr->userfield, ""));
+
+	cdr_buffer = ast_json_dump_string(t_cdr_json);
+
+	ast_json_unref(t_cdr_json);
+
+	bs_id = bs_put(bs_socket, priority, BEANSTALK_JOB_DELAY, BEANSTALK_JOB_TTR, cdr_buffer, strlen(cdr_buffer));
+
+	if (bs_id > 0) {
+		ast_log(LOG_DEBUG, "Successfully created job %d with %s\n", bs_id, cdr_buffer);
+	} else {
+		ast_log(LOG_ERROR, "CDR job creation failed for %s\n", cdr_buffer);
+	}
+
+	bs_disconnect(bs_socket);
+	ast_json_free(cdr_buffer);
+	return 0;
+}
+
+static int unload_module(void) {
+	if (ast_cdr_unregister(name)) {
+		return -1;
+	}
+
+	ast_free(bs_host);
+	ast_free(bs_tube);
+
+	return 0;
+}
+
+static int load_module(void) {
+	if (ast_cdr_register(name, "Asterisk CDR Beanstalkd Backend", beanstalk_put)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (load_config(0)) {
+		ast_cdr_unregister(name);
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int reload(void) {
+	return load_config(1);
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk Beanstalkd CDR Backend",
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_csv.c 1:16.1.1~dfsg-1/cdr/cdr_csv.c
--- 1:13.23.1~dfsg-2/cdr/cdr_csv.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_csv.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
 #include "asterisk/config.h"
 #include "asterisk/channel.h"
@@ -60,6 +58,7 @@ static int loguserfield = 0;
 static int loaded = 0;
 static int newcdrcolumns = 0;
 static const char config[] = "cdr.conf";
+static char file_csv_master[PATH_MAX];
 
 /* #define CSV_LOGUNIQUEID 1 */
 /* #define CSV_LOGUSERFIELD 1 */
@@ -94,8 +93,7 @@ static const char config[] = "cdr.conf";
 
 static char *name = "csv";
 
-AST_MUTEX_DEFINE_STATIC(mf_lock);
-AST_MUTEX_DEFINE_STATIC(acf_lock);
+AST_MUTEX_DEFINE_STATIC(f_lock);
 
 static int load_config(int reload)
 {
@@ -121,6 +119,12 @@ static int load_config(int reload)
 		return 0;
 	}
 
+	/* compute the location of the csv master file */
+	ast_mutex_lock(&f_lock);
+	snprintf(file_csv_master, sizeof(file_csv_master),
+		"%s/%s/%s", ast_config_AST_LOG_DIR, CSV_LOG_DIR, CSV_MASTER);
+	ast_mutex_unlock(&f_lock);
+
 	for (; v; v = v->next) {
 		if (!strcasecmp(v->name, "usegmtime")) {
 			usegmtime = ast_true(v->value);
@@ -245,7 +249,7 @@ static int build_csv_record(char *buf, s
 		append_string(buf, cdr->uniqueid, bufsize);
 	/* append the user field */
 	if(loguserfield)
-		append_string(buf, cdr->userfield,bufsize);
+		append_string(buf, cdr->userfield, bufsize);
 	if (newcdrcolumns) {
 		append_string(buf, cdr->peeraccount, bufsize);
 		append_string(buf, cdr->linkedid, bufsize);
@@ -261,65 +265,53 @@ static int build_csv_record(char *buf, s
 	return -1;
 }
 
-static int writefile(char *s, char *acc)
+static int writefile(char *s, char *file_path)
 {
-	char tmp[PATH_MAX];
 	FILE *f;
-
-	if (strchr(acc, '/') || (acc[0] == '.')) {
-		ast_log(LOG_WARNING, "Account code '%s' insecure for writing file\n", acc);
-		return -1;
-	}
-
-	snprintf(tmp, sizeof(tmp), "%s/%s/%s.csv", ast_config_AST_LOG_DIR,CSV_LOG_DIR, acc);
-
-	ast_mutex_lock(&acf_lock);
-	if (!(f = fopen(tmp, "a"))) {
-		ast_mutex_unlock(&acf_lock);
-		ast_log(LOG_ERROR, "Unable to open file %s : %s\n", tmp, strerror(errno));
+	/* because of the absolutely unconditional need for the
+	   highest reliability possible in writing billing records,
+	   we open write and close the log file each time */
+	if (!(f = fopen(file_path, "a"))) {
+		ast_log(LOG_ERROR, "Unable to open file %s : %s\n", file_path, strerror(errno));
 		return -1;
 	}
 	fputs(s, f);
-	fflush(f);
+	fflush(f); /* be particularly anal here */
 	fclose(f);
-	ast_mutex_unlock(&acf_lock);
 
 	return 0;
 }
 
 
+static int writefile_account(char *s, char *acc)
+{
+	char file_account[PATH_MAX];
+	if (strchr(acc, '/') || (acc[0] == '.')) {
+		ast_log(LOG_WARNING, "Account code '%s' insecure for writing file\n", acc);
+		return -1;
+	}
+	snprintf(file_account, sizeof(file_account), "%s/%s/%s.csv", ast_config_AST_LOG_DIR,CSV_LOG_DIR, acc);
+	return writefile(s, file_account);
+}
+
 static int csv_log(struct ast_cdr *cdr)
 {
-	FILE *mf = NULL;
 	/* Make sure we have a big enough buf */
 	char buf[1024];
-	char csvmaster[PATH_MAX];
-	snprintf(csvmaster, sizeof(csvmaster),"%s/%s/%s", ast_config_AST_LOG_DIR, CSV_LOG_DIR, CSV_MASTER);
 	if (build_csv_record(buf, sizeof(buf), cdr)) {
 		ast_log(LOG_WARNING, "Unable to create CSV record in %d bytes.  CDR not recorded!\n", (int)sizeof(buf));
 		return 0;
 	}
 
-	/* because of the absolutely unconditional need for the
-	   highest reliability possible in writing billing records,
-	   we open write and close the log file each time */
-	ast_mutex_lock(&mf_lock);
-	if ((mf = fopen(csvmaster, "a"))) {
-		fputs(buf, mf);
-		fflush(mf); /* be particularly anal here */
-		fclose(mf);
-		mf = NULL;
-		ast_mutex_unlock(&mf_lock);
-	} else {
-		ast_mutex_unlock(&mf_lock);
-		ast_log(LOG_ERROR, "Unable to re-open master file %s : %s\n", csvmaster, strerror(errno));
-	}
+	ast_mutex_lock(&f_lock);
+	if (writefile(buf, file_csv_master))
+		ast_log(LOG_WARNING, "Unable to write CSV record to master '%s' : %s\n", file_csv_master, strerror(errno));
 
 	if (accountlogs && !ast_strlen_zero(cdr->accountcode)) {
-		if (writefile(buf, cdr->accountcode))
+		if (writefile_account(buf, cdr->accountcode))
 			ast_log(LOG_WARNING, "Unable to write CSV record to account file '%s' : %s\n", cdr->accountcode, strerror(errno));
 	}
-
+	ast_mutex_unlock(&f_lock);
 	return 0;
 }
 
@@ -363,9 +355,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Comma Separated Values CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_custom.c 1:16.1.1~dfsg-1/cdr/cdr_custom.c
--- 1:13.23.1~dfsg-2/cdr/cdr_custom.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_custom.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
@@ -228,9 +226,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Customizable Comma Separated Values CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_manager.c 1:16.1.1~dfsg-1/cdr/cdr_manager.c
--- 1:13.23.1~dfsg-2/cdr/cdr_manager.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_manager.c	2018-12-26 18:45:13.000000000 +0000
@@ -166,8 +166,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 
 #include "asterisk/channel.h"
@@ -368,9 +366,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk Manager Interface CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_odbc.c 1:16.1.1~dfsg-1/cdr/cdr_odbc.c
--- 1:13.23.1~dfsg-2/cdr/cdr_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/config.h"
 #include "asterisk/channel.h"
 #include "asterisk/cdr.h"
@@ -91,7 +89,7 @@ static SQLHSTMT execute_cb(struct odbc_o
 		snprintf(sqlcmd,sizeof(sqlcmd),"INSERT INTO %s "
 		"(calldate,clid,src,dst,dcontext,channel,dstchannel,lastapp,"
 		"lastdata,duration,billsec,disposition,amaflags,accountcode,uniqueid,userfield%s) "
-		"VALUES ({ts '%s'},?,?,?,?,?,?,?,?,?,?,?,?,?,?,? %s)", table, new_columns, timestr, new_values);
+		"VALUES ({ts '%s'},?,?,?,?,?,?,?,?,?,?,?,?,?,?,?%s)", table, new_columns, timestr, new_values);
 	} else {
 		snprintf(sqlcmd,sizeof(sqlcmd),"INSERT INTO %s "
 		"(calldate,clid,src,dst,dcontext,channel,dstchannel,lastapp,lastdata,"
@@ -278,10 +276,10 @@ static int odbc_load_module(int reload)
 		}
 		if (((tmp = ast_variable_retrieve(cfg, "global", "newcdrcolumns"))) && ast_true(tmp)) {
 			ast_set_flag(&config, CONFIG_NEWCDRCOLUMNS);
-			ast_debug(1, "cdr_odbc: Add new cdr fields\n");
+			ast_debug(1, "cdr_odbc: Add new cdr columns\n");
 		} else {
 			ast_clear_flag(&config, CONFIG_NEWCDRCOLUMNS);
-			ast_debug(1, "cdr_odbc: Not add new cdr fields\n");
+			ast_debug(1, "cdr_odbc: Not add new cdr columns\n");
 		}
 	} while (0);
 
@@ -325,9 +323,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "ODBC CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_pgsql.c 1:16.1.1~dfsg-1/cdr/cdr_pgsql.c
--- 1:13.23.1~dfsg-2/cdr/cdr_pgsql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_pgsql.c	2018-12-26 18:45:13.000000000 +0000
@@ -49,8 +49,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libpq-fe.h>
 
 #include "asterisk/config.h"
@@ -61,6 +59,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #define DATE_FORMAT "'%Y-%m-%d %T'"
 
+#define PGSQL_MIN_VERSION_SCHEMA 70300
+
 static const char name[] = "pgsql";
 static const char config[] = "cdr_pgsql.conf";
 
@@ -75,6 +75,7 @@ static char *encoding;
 static char *tz;
 
 static int connected = 0;
+/* Optimization to reduce number of memory allocations */
 static int maxsize = 512, maxsize2 = 512;
 static time_t connect_time = 0;
 static int totalrecords = 0;
@@ -100,35 +101,26 @@ struct columns {
 
 static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
 
-#define LENGTHEN_BUF1(size)                                               \
-			do {                                                          \
-				/* Lengthen buffer, if necessary */                       \
-				if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) { \
-					if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 3) / 512 + 1) * 512) != 0) {	\
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n"); \
-						ast_free(sql);                                    \
-						ast_free(sql2);                                   \
-						AST_RWLIST_UNLOCK(&psql_columns);                 \
-						ast_mutex_unlock(&pgsql_lock);                    \
-						return -1;                                        \
-					}                                                     \
-				}                                                         \
-			} while (0)
-
-#define LENGTHEN_BUF2(size)                               \
-			do {                                          \
-				if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) {  \
-					if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) {	\
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CDR failed.\n");	\
-						ast_free(sql);                    \
-						ast_free(sql2);                   \
+#define LENGTHEN_BUF(size, var_sql) \
+			do { \
+				/* Lengthen buffer, if necessary */ \
+				if (ast_str_strlen(var_sql) + size + 1 > ast_str_size(var_sql)) { \
+					if (ast_str_make_space(&var_sql, ((ast_str_size(var_sql) + size + 3) / 512 + 1) * 512) != 0) { \
+						ast_log(LOG_ERROR, "Unable to allocate sufficient memory. Insert CDR '%s:%s' failed.\n", pghostname, table); \
+						ast_free(sql); \
+						ast_free(sql2); \
 						AST_RWLIST_UNLOCK(&psql_columns); \
-						ast_mutex_unlock(&pgsql_lock);    \
-						return -1;                        \
-					}                                     \
-				}                                         \
+						ast_mutex_unlock(&pgsql_lock); \
+						return -1; \
+					} \
+				} \
 			} while (0)
 
+#define LENGTHEN_BUF1(size) \
+	LENGTHEN_BUF(size, sql);
+#define LENGTHEN_BUF2(size) \
+	LENGTHEN_BUF(size, sql2);
+
 /*! \brief Handle the CLI command cdr show pgsql status */
 static char *handle_cdr_pgsql_status(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
@@ -147,7 +139,9 @@ static char *handle_cdr_pgsql_status(str
 		return CLI_SHOWUSAGE;
 
 	if (connected) {
-		char status[256], status2[100] = "";
+		char status[256];
+		char status2[100] = "";
+		char buf[362]; /* 256+100+" for "+NULL */
 		int ctime = time(NULL) - connect_time;
 
 		if (pgdbport) {
@@ -162,17 +156,10 @@ static char *handle_cdr_pgsql_status(str
 		if (table && *table) {
 			snprintf(status2, 99, " using table %s", table);
 		}
-		if (ctime > 31536000) {
-			ast_cli(a->fd, "%s%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 31536000, (ctime % 31536000) / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 86400) {
-			ast_cli(a->fd, "%s%s for %d days, %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 86400, (ctime % 86400) / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 3600) {
-			ast_cli(a->fd, "%s%s for %d hours, %d minutes, %d seconds.\n", status, status2, ctime / 3600, (ctime % 3600) / 60, ctime % 60);
-		} else if (ctime > 60) {
-			ast_cli(a->fd, "%s%s for %d minutes, %d seconds.\n", status, status2, ctime / 60, ctime % 60);
-		} else {
-			ast_cli(a->fd, "%s%s for %d seconds.\n", status, status2, ctime);
-		}
+
+		snprintf(buf, sizeof(buf), "%s%s for ", status, status2);
+		ast_cli_print_timestr_fromseconds(a->fd, ctime, buf);
+
 		if (records == totalrecords) {
 			ast_cli(a->fd, "  Wrote %d records since last restart.\n", totalrecords);
 		} else {
@@ -249,7 +236,7 @@ static int pgsql_log(struct ast_cdr *cdr
 		struct ast_str *sql = ast_str_create(maxsize), *sql2 = ast_str_create(maxsize2);
 		char buf[257];
 		char *escapebuf = NULL, *value;
-		int first = 1;
+		char *separator = "";
 		size_t bufsize = 513;
 
 		escapebuf = ast_malloc(bufsize);
@@ -271,86 +258,86 @@ static int pgsql_log(struct ast_cdr *cdr
 				if (cur->notnull && !cur->hasdefault) {
 					/* Field is NOT NULL (but no default), must include it anyway */
 					LENGTHEN_BUF1(strlen(cur->name) + 2);
-					ast_str_append(&sql, 0, "%s\"%s\"", first ? "" : ",", cur->name);
+					ast_str_append(&sql, 0, "%s\"%s\"", separator, cur->name);
 					LENGTHEN_BUF2(3);
-					ast_str_append(&sql2, 0, "%s''", first ? "" : ",");
-					first = 0;
+					ast_str_append(&sql2, 0, "%s''", separator);
+					separator = ", ";
 				}
 				continue;
 			}
 
 			LENGTHEN_BUF1(strlen(cur->name) + 2);
-			ast_str_append(&sql, 0, "%s\"%s\"", first ? "" : ",", cur->name);
+			ast_str_append(&sql, 0, "%s\"%s\"", separator, cur->name);
 
 			if (strcmp(cur->name, "start") == 0 || strcmp(cur->name, "calldate") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->start.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->start.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->start.tv_sec + (double)cdr->start.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator, (double)cdr->start.tv_sec + (double)cdr->start.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->start, &tm, tz);
 					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_str_append(&sql2, 0, "%s%s", separator, buf);
 				}
 			} else if (strcmp(cur->name, "answer") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->answer.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->answer.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->answer.tv_sec + (double)cdr->answer.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator, (double)cdr->answer.tv_sec + (double)cdr->answer.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->answer, &tm, tz);
 					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_str_append(&sql2, 0, "%s%s", separator, buf);
 				}
 			} else if (strcmp(cur->name, "end") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%ld", first ? "" : ",", (long) cdr->end.tv_sec);
+					ast_str_append(&sql2, 0, "%s%ld", separator, (long) cdr->end.tv_sec);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double)cdr->end.tv_sec + (double)cdr->end.tv_usec / 1000000.0);
+					ast_str_append(&sql2, 0, "%s%f", separator, (double)cdr->end.tv_sec + (double)cdr->end.tv_usec / 1000000.0);
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
 					ast_localtime(&cdr->end, &tm, tz);
 					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", buf);
+					ast_str_append(&sql2, 0, "%s%s", separator, buf);
 				}
 			} else if (strcmp(cur->name, "duration") == 0 || strcmp(cur->name, "billsec") == 0) {
 				if (cur->type[0] == 'i') {
 					/* Get integer, no need to escape anything */
 					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
+					ast_str_append(&sql2, 0, "%s%s", separator, value);
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					struct timeval *when = cur->name[0] == 'd' ? &cdr->start : ast_tvzero(cdr->answer) ? &cdr->end : &cdr->answer;
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s%f", first ? "" : ",", (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					ast_str_append(&sql2, 0, "%s%f", separator, (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
 				} else {
 					/* Char field, probably */
 					struct timeval *when = cur->name[0] == 'd' ? &cdr->start : ast_tvzero(cdr->answer) ? &cdr->end : &cdr->answer;
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s'%f'", first ? "" : ",", (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
+					ast_str_append(&sql2, 0, "%s'%f'", separator, (double) (ast_tvdiff_us(cdr->end, *when) / 1000000.0));
 				}
 			} else if (strcmp(cur->name, "disposition") == 0 || strcmp(cur->name, "amaflags") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
 					/* Integer, no need to escape anything */
 					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 1);
 					LENGTHEN_BUF2(13);
-					ast_str_append(&sql2, 0, "%s%s", first ? "" : ",", value);
+					ast_str_append(&sql2, 0, "%s%s", separator, value);
 				} else {
 					/* Although this is a char field, there are no special characters in the values for these fields */
 					ast_cdr_format_var(cdr, cur->name, &value, buf, sizeof(buf), 0);
 					LENGTHEN_BUF2(31);
-					ast_str_append(&sql2, 0, "%s'%s'", first ? "" : ",", value);
+					ast_str_append(&sql2, 0, "%s'%s'", separator, value);
 				}
 			} else {
 				/* Arbitrary field, could be anything */
@@ -359,19 +346,19 @@ static int pgsql_log(struct ast_cdr *cdr
 					long long whatever;
 					if (value && sscanf(value, "%30lld", &whatever) == 1) {
 						LENGTHEN_BUF2(26);
-						ast_str_append(&sql2, 0, "%s%lld", first ? "" : ",", whatever);
+						ast_str_append(&sql2, 0, "%s%lld", separator, whatever);
 					} else {
 						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s0", first ? "" : ",");
+						ast_str_append(&sql2, 0, "%s0", separator);
 					}
 				} else if (strncmp(cur->type, "float", 5) == 0) {
 					long double whatever;
 					if (value && sscanf(value, "%30Lf", &whatever) == 1) {
 						LENGTHEN_BUF2(51);
-						ast_str_append(&sql2, 0, "%s%30Lf", first ? "" : ",", whatever);
+						ast_str_append(&sql2, 0, "%s%30Lf", separator, whatever);
 					} else {
 						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s0", first ? "" : ",");
+						ast_str_append(&sql2, 0, "%s0", separator);
 					}
 				/* XXX Might want to handle dates, times, and other misc fields here XXX */
 				} else {
@@ -398,10 +385,10 @@ static int pgsql_log(struct ast_cdr *cdr
 						escapebuf[0] = '\0';
 					}
 					LENGTHEN_BUF2(strlen(escapebuf) + 3);
-					ast_str_append(&sql2, 0, "%s'%s'", first ? "" : ",", escapebuf);
+					ast_str_append(&sql2, 0, "%s'%s'", separator, escapebuf);
 				}
 			}
-			first = 0;
+			separator = ", ";
 		}
 
 		LENGTHEN_BUF1(ast_str_strlen(sql2) + 2);
@@ -465,6 +452,14 @@ static int pgsql_log(struct ast_cdr *cdr
 		}
 		PQclear(result);
 
+		/* Next time, just allocate buffers that are that big to start with. */
+		if (ast_str_strlen(sql) > maxsize) {
+			maxsize = ast_str_strlen(sql);
+		}
+		if (ast_str_strlen(sql2) > maxsize2) {
+			maxsize2 = ast_str_strlen(sql2);
+		}
+
 ast_log_cleanup:
 		ast_free(escapebuf);
 		ast_free(sql);
@@ -684,7 +679,7 @@ static int config_module(int reload)
 		}
 		version = PQserverVersion(conn);
 
-		if (version >= 70300) {
+		if (version >= PGSQL_MIN_VERSION_SCHEMA) {
 			char *schemaname, *tablename, *tmp_schemaname, *tmp_tablename;
 			if (strchr(table, '.')) {
 				tmp_schemaname = ast_strdupa(table);
@@ -794,9 +789,10 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PostgreSQL CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_radius.c 1:16.1.1~dfsg-1/cdr/cdr_radius.c
--- 1:13.23.1~dfsg-2/cdr/cdr_radius.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_radius.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include RADIUS_HEADER_STR
 
 #include "asterisk/channel.h"
@@ -261,18 +259,6 @@ static int load_module(void)
 	} else
 		return AST_MODULE_LOAD_DECLINE;
 
-	/*
-	 * start logging
-	 *
-	 * NOTE: Yes this causes a slight memory leak if the module is
-	 * unloaded.  However, it is better than a crash if cdr_radius
-	 * and cel_radius are both loaded.
-	 */
-	tmp = ast_strdup("asterisk");
-	if (tmp) {
-		rc_openlog((char *) tmp);
-	}
-
 	/* read radiusclient-ng config file */
 	if (!(rh = rc_read_config(radiuscfg))) {
 		ast_log(LOG_NOTICE, "Cannot load radiusclient-ng configuration file %s.\n", radiuscfg);
@@ -297,8 +283,9 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "RADIUS CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_sqlite3_custom.c 1:16.1.1~dfsg-1/cdr/cdr_sqlite3_custom.c
--- 1:13.23.1~dfsg-2/cdr/cdr_sqlite3_custom.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_sqlite3_custom.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sqlite3.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
@@ -63,6 +61,7 @@ static sqlite3 *db = NULL;
 
 static char table[80];
 static char *columns;
+static int busy_timeout;
 
 struct values {
 	AST_LIST_ENTRY(values) list;
@@ -183,6 +182,15 @@ static int load_config(int reload)
 		strcpy(table, "cdr");
 	}
 
+	/* sqlite3_busy_timeout in miliseconds */
+	if ((tmp = ast_variable_retrieve(cfg, "master", "busy_timeout")) != NULL) {
+		if (ast_parse_arg(tmp, PARSE_INT32|PARSE_DEFAULT, &busy_timeout, 1000) != 0) {
+			ast_log(LOG_WARNING, "Invalid busy_timeout value '%s' specified. Using 1000 instead.\n", tmp);
+		}
+	} else {
+		busy_timeout = 1000;
+	}
+
 	/* Columns */
 	if (load_column_config(ast_variable_retrieve(cfg, "master", "columns"))) {
 		ast_config_destroy(cfg);
@@ -307,7 +315,7 @@ static int load_module(void)
 		free_config(0);
 		return AST_MODULE_LOAD_DECLINE;
 	}
-	sqlite3_busy_timeout(db, 1000);
+	sqlite3_busy_timeout(db, busy_timeout);
 	/* is the table there? */
 	sql = sqlite3_mprintf("SELECT COUNT(AcctId) FROM %q;", table);
 	res = sqlite3_exec(db, sql, NULL, NULL, NULL);
@@ -352,4 +360,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
 );
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_sqlite.c 1:16.1.1~dfsg-1/cdr/cdr_sqlite.c
--- 1:13.23.1~dfsg-2/cdr/cdr_sqlite.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_sqlite.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,253 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2004 - 2005, Holger Schurig
- *
- *
- * Ideas taken from other cdr_*.c files
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*!
- * \file
- * \brief Store CDR records in a SQLite database.
- *
- * \author Holger Schurig <hs4233@mail.mn-solutions.de>
- * SQLite http://www.sqlite.org/
- *
- * See also
- * \arg \ref Config_cdr
- * \arg http://www.sqlite.org/
- *
- * Creates the database and table on-the-fly
- * \ingroup cdr_drivers
- *
- * \note This module has been marked deprecated in favor for cdr_sqlite3_custom
- */
-
-/*** MODULEINFO
-	<depend>sqlite</depend>
-	<defaultenabled>no</defaultenabled>
-	<support_level>deprecated</support_level>
-	<replacement>sqlite3_custom</replacement>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include <sqlite.h>
-
-#include "asterisk/channel.h"
-#include "asterisk/module.h"
-#include "asterisk/utils.h"
-#include "asterisk/paths.h"
-
-#define LOG_UNIQUEID    0
-#define LOG_USERFIELD   0
-#define LOG_HRTIME      0
-
-/* When you change the DATE_FORMAT, be sure to change the CHAR(19) below to something else */
-#define DATE_FORMAT "%Y-%m-%d %T"
-
-static const char name[] = "sqlite";
-static sqlite* db = NULL;
-
-AST_MUTEX_DEFINE_STATIC(sqlite_lock);
-
-/*! \brief SQL table format */
-static const char sql_create_table[] = "CREATE TABLE cdr ("
-"	AcctId		INTEGER PRIMARY KEY,"
-"	clid		VARCHAR(80),"
-"	src		VARCHAR(80),"
-"	dst		VARCHAR(80),"
-"	dcontext	VARCHAR(80),"
-"	channel		VARCHAR(80),"
-"	dstchannel	VARCHAR(80),"
-"	lastapp		VARCHAR(80),"
-"	lastdata	VARCHAR(80),"
-"	start		CHAR(19),"
-"	answer		CHAR(19),"
-"	end		CHAR(19),"
-#if LOG_HRTIME
-"	duration	FLOAT,"
-"	billsec		FLOAT,"
-#else
-"	duration	INTEGER,"
-"	billsec		INTEGER,"
-#endif
-"	disposition	INTEGER,"
-"	amaflags	INTEGER,"
-"	accountcode	VARCHAR(20)"
-#if LOG_UNIQUEID
-"	,uniqueid	VARCHAR(32)"
-#endif
-#if LOG_USERFIELD
-"	,userfield	VARCHAR(255)"
-#endif
-");";
-
-static void format_date(char *buffer, size_t length, struct timeval *when)
-{
-	struct ast_tm tm;
-
-	ast_localtime(when, &tm, NULL);
-	ast_strftime(buffer, length, DATE_FORMAT, &tm);
-}
-
-static int sqlite_log(struct ast_cdr *cdr)
-{
-	int res = 0;
-	char *zErr = 0;
-	char startstr[80], answerstr[80], endstr[80];
-	int count;
-#if LOG_HRTIME
-	double hrbillsec = 0.0;
-	double hrduration;
-#endif
-
-	ast_mutex_lock(&sqlite_lock);
-
-	format_date(startstr, sizeof(startstr), &cdr->start);
-	format_date(answerstr, sizeof(answerstr), &cdr->answer);
-	format_date(endstr, sizeof(endstr), &cdr->end);
-
-#if LOG_HRTIME
-	if (!ast_tvzero(cdr->answer)) {
-		hrbillsec = (double) ast_tvdiff_us(cdr->end, cdr->answer) / 1000000.0;
-	}
-	hrduration = (double) ast_tvdiff_us(cdr->end, cdr->start) / 1000000.0;
-#endif
-
-	for(count=0; count<5; count++) {
-		res = sqlite_exec_printf(db,
-			"INSERT INTO cdr ("
-				"clid,src,dst,dcontext,"
-				"channel,dstchannel,lastapp,lastdata, "
-				"start,answer,end,"
-				"duration,billsec,disposition,amaflags, "
-				"accountcode"
-#				if LOG_UNIQUEID
-				",uniqueid"
-#				endif
-#				if LOG_USERFIELD
-				",userfield"
-#				endif
-			") VALUES ("
-				"'%q', '%q', '%q', '%q', "
-				"'%q', '%q', '%q', '%q', "
-				"'%q', '%q', '%q', "
-#if LOG_HRTIME
-				"%f, %f, %d, %d, "
-#else
-				"%d, %d, %d, %d, "
-#endif
-				"'%q'"
-#				if LOG_UNIQUEID
-				",'%q'"
-#				endif
-#				if LOG_USERFIELD
-				",'%q'"
-#				endif
-			")", NULL, NULL, &zErr,
-				cdr->clid, cdr->src, cdr->dst, cdr->dcontext,
-				cdr->channel, cdr->dstchannel, cdr->lastapp, cdr->lastdata,
-				startstr, answerstr, endstr,
-#if LOG_HRTIME
-				hrduration, hrbillsec, cdr->disposition, cdr->amaflags,
-#else
-				cdr->duration, cdr->billsec, cdr->disposition, cdr->amaflags,
-#endif
-				cdr->accountcode
-#				if LOG_UNIQUEID
-				,cdr->uniqueid
-#				endif
-#				if LOG_USERFIELD
-				,cdr->userfield
-#				endif
-			);
-		if (res != SQLITE_BUSY && res != SQLITE_LOCKED)
-			break;
-		usleep(200);
-	}
-
-	if (zErr) {
-		ast_log(LOG_ERROR, "cdr_sqlite: %s\n", zErr);
-		ast_free(zErr);
-	}
-
-	ast_mutex_unlock(&sqlite_lock);
-	return res;
-}
-
-static int unload_module(void)
-{
-	if (ast_cdr_unregister(name)) {
-		return -1;
-	}
-
-	if (db) {
-		sqlite_close(db);
-	}
-	return 0;
-}
-
-static int load_module(void)
-{
-	char *zErr;
-	char fn[PATH_MAX];
-	int res;
-
-	ast_log(LOG_NOTICE, "This module has been marked deprecated in favor of "
-		"using cdr_sqlite3_custom.\n");
-
-	/* is the database there? */
-	snprintf(fn, sizeof(fn), "%s/cdr.db", ast_config_AST_LOG_DIR);
-	db = sqlite_open(fn, AST_FILE_MODE, &zErr);
-	if (!db) {
-		ast_log(LOG_ERROR, "cdr_sqlite: %s\n", zErr);
-		ast_free(zErr);
-		return AST_MODULE_LOAD_DECLINE;
-	}
-
-	/* is the table there? */
-	res = sqlite_exec(db, "SELECT COUNT(AcctId) FROM cdr;", NULL, NULL, NULL);
-	if (res) {
-		res = sqlite_exec(db, sql_create_table, NULL, NULL, &zErr);
-		if (res) {
-			ast_log(LOG_ERROR, "cdr_sqlite: Unable to create table 'cdr': %s\n", zErr);
-			ast_free(zErr);
-			goto err;
-		}
-
-		/* TODO: here we should probably create an index */
-	}
-
-	res = ast_cdr_register(name, ast_module_info->description, sqlite_log);
-	if (res) {
-		ast_log(LOG_ERROR, "Unable to register SQLite CDR handling\n");
-		return AST_MODULE_LOAD_DECLINE;
-	}
-	return AST_MODULE_LOAD_SUCCESS;
-
-err:
-	if (db)
-		sqlite_close(db);
-	return AST_MODULE_LOAD_DECLINE;
-}
-
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "SQLite CDR Backend",
-	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
-	.load = load_module,
-	.unload = unload_module,
-	.load_pri = AST_MODPRI_CDR_DRIVER,
-);
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_syslog.c 1:16.1.1~dfsg-1/cdr/cdr_syslog.c
--- 1:13.23.1~dfsg-2/cdr/cdr_syslog.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_syslog.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,14 +37,13 @@
  */
 
 /*** MODULEINFO
+	<defaultenabled>no</defaultenabled>
 	<depend>syslog</depend>
-	<support_level>core</support_level>
+	<support_level>deprecated</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/lock.h"
 #include "asterisk/cdr.h"
@@ -293,4 +292,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
 );
diff -pruN 1:13.23.1~dfsg-2/cdr/cdr_tds.c 1:16.1.1~dfsg-1/cdr/cdr_tds.c
--- 1:13.23.1~dfsg-2/cdr/cdr_tds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cdr/cdr_tds.c	2018-12-26 18:45:13.000000000 +0000
@@ -64,8 +64,6 @@ CREATE TABLE [dbo].[cdr] (
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/config.h"
 #include "asterisk/channel.h"
 #include "asterisk/cdr.h"
@@ -632,9 +630,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "FreeTDS CDR Backend",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CDR_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/cel/cel_beanstalkd.c 1:16.1.1~dfsg-1/cel/cel_beanstalkd.c
--- 1:13.23.1~dfsg-2/cel/cel_beanstalkd.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_beanstalkd.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,276 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Greenfield Technologies Ltd.
+ *
+ * Nir Simionovich <nirs@greenfieldtech.net>
+ * who freely borrowed code from the cel manager equivalents
+ *     (see cel/cel_manager.c)
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Asterisk Channel Event Beanstalkd backend
+ *
+ * This module requires the beanstalk-client library, avaialble from
+ * https://github.com/deepfryed/beanstalk-client
+ *
+ * See also
+ * \arg \ref AstCEL
+ * \ingroup cel_drivers
+ */
+
+/*! \li \ref cek_beanstalkd.c uses the configuration file \ref cel.conf
+ * \addtogroup configuration_file Configuration Files
+ */
+
+/*!
+ * \page cel.conf cel.conf
+ * \verbinclude cel.conf.sample
+ */
+
+/*** MODULEINFO
+	<depend>beanstalk</depend>
+	<support_level>extended</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/channel.h"
+#include "asterisk/cel.h"
+#include "asterisk/module.h"
+#include "asterisk/logger.h"
+#include "asterisk/utils.h"
+#include "asterisk/manager.h"
+#include "asterisk/config.h"
+#include "asterisk/json.h"
+
+#include "beanstalk.h"
+
+static const char DATE_FORMAT[] = "%Y-%m-%d %T";
+
+static const char CONF_FILE[] = "cel_beanstalkd.conf";
+
+/*! \brief Beanstalk CEL is off by default */
+#define CEL_BEANSTALK_ENABLED_DEFAULT		0
+
+static int enablecel;
+
+/*! \brief show_user_def is off by default */
+#define CEL_SHOW_USERDEF_DEFAULT	0
+
+#define CEL_BACKEND_NAME "Beanstalk Event Logging"
+
+#define BEANSTALK_JOB_SIZE 4096
+#define BEANSTALK_JOB_PRIORITY 99
+#define BEANSTALK_JOB_TTR 60
+#define BEANSTALK_JOB_DELAY 0
+#define DEFAULT_BEANSTALK_HOST "127.0.0.1"
+#define DEFAULT_BEANSTALK_PORT 11300
+#define DEFAULT_BEANSTALK_TUBE "asterisk-cel"
+
+static char *bs_host;
+static int bs_port;
+static char *bs_tube;
+static int priority;
+
+AST_RWLOCK_DEFINE_STATIC(config_lock);
+
+static void cel_bs_put(struct ast_event *event)
+{
+	struct ast_tm timeresult;
+	char start_time[80];
+	char *cel_buffer;
+	int bs_id;
+	int bs_socket;
+	struct ast_json *t_cel_json;
+
+	struct ast_cel_event_record record = {
+		.version = AST_CEL_EVENT_RECORD_VERSION,
+	};
+
+	if (!enablecel) {
+		return;
+	}
+
+	if (ast_cel_fill_record(event, &record)) {
+		return;
+	}
+
+	ast_rwlock_rdlock(&config_lock);
+	bs_socket = bs_connect(bs_host, bs_port);
+
+	if (bs_use(bs_socket, bs_tube) != BS_STATUS_OK) {
+		ast_log(LOG_ERROR, "Connection to Beanstalk tube %s @ %s:%d had failed", bs_tube, bs_host, bs_port);
+		ast_rwlock_unlock(&config_lock);
+		return;
+	}
+
+	ast_localtime(&record.event_time, &timeresult, NULL);
+	ast_strftime(start_time, sizeof(start_time), DATE_FORMAT, &timeresult);
+
+	ast_rwlock_unlock(&config_lock);
+
+	t_cel_json = ast_json_pack("{s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s, s:s}",
+							   "EventName", S_OR(record.event_name, ""),
+							   "AccountCode", S_OR(record.account_code, ""),
+							   "CallerIDnum", S_OR(record.caller_id_num, ""),
+							   "CallerIDname", S_OR(record.caller_id_name, ""),
+							   "CallerIDani", S_OR(record.caller_id_ani, ""),
+							   "CallerIDrdnis", S_OR(record.caller_id_rdnis, ""),
+							   "CallerIDdnid", S_OR(record.caller_id_dnid, ""),
+							   "Exten", S_OR(record.extension, ""),
+							   "Context", S_OR(record.context, ""),
+							   "Channel", S_OR(record.channel_name, ""),
+							   "Application", S_OR(record.application_name, ""),
+							   "AppData", S_OR(record.application_data, ""),
+							   "EventTime", S_OR(start_time, ""),
+							   "AMAFlags", S_OR(ast_channel_amaflags2string(record.amaflag), ""),
+							   "UniqueID", S_OR(record.unique_id, ""),
+							   "LinkedID", S_OR(record.linked_id, ""),
+							   "Userfield", S_OR(record.user_field, ""),
+							   "Peer", S_OR(record.peer_account, ""),
+							   "PeerAccount", S_OR(record.peer_account, ""),
+							   "Extra", S_OR(record.extra, "")
+
+	);
+
+	cel_buffer = ast_json_dump_string(t_cel_json);
+
+	ast_json_unref(t_cel_json);
+
+	bs_id = bs_put(bs_socket, priority, BEANSTALK_JOB_DELAY, BEANSTALK_JOB_TTR, cel_buffer, strlen(cel_buffer));
+
+	if (bs_id > 0) {
+		ast_log(LOG_DEBUG, "Successfully created job %d with %s\n", bs_id, cel_buffer);
+	} else {
+		ast_log(LOG_ERROR, "CDR job creation failed for %s\n", cel_buffer);
+	}
+
+	bs_disconnect(bs_socket);
+	ast_json_free(cel_buffer);
+}
+
+static int load_config(int reload)
+{
+	const char *cat = NULL;
+	struct ast_config *cfg;
+	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
+	struct ast_variable *v;
+	int newenablecel = CEL_BEANSTALK_ENABLED_DEFAULT;
+
+	cfg = ast_config_load(CONF_FILE, config_flags);
+	if (cfg == CONFIG_STATUS_FILEUNCHANGED) {
+		return 0;
+	}
+
+	if (cfg == CONFIG_STATUS_FILEINVALID) {
+		ast_log(LOG_WARNING, "Configuration file '%s' is invalid. CEL Beanstalkd Module not activated.\n",
+			CONF_FILE);
+		return -1;
+	} else if (!cfg) {
+		ast_log(LOG_WARNING, "Failed to load configuration file. CEL Beanstalkd Module not activated.\n");
+		if (enablecel) {
+			ast_cel_backend_unregister(CEL_BACKEND_NAME);
+		}
+		enablecel = 0;
+		return -1;
+	}
+
+	if (reload) {
+		ast_rwlock_wrlock(&config_lock);
+		ast_free(bs_host);
+		ast_free(bs_tube);
+	}
+
+	/* Bootstrap the default configuration */
+	bs_host = ast_strdup(DEFAULT_BEANSTALK_HOST);
+	bs_port = DEFAULT_BEANSTALK_PORT;
+	bs_tube = ast_strdup(DEFAULT_BEANSTALK_TUBE);
+	priority = BEANSTALK_JOB_PRIORITY;
+
+	while ((cat = ast_category_browse(cfg, cat))) {
+
+		if (strcasecmp(cat, "general")) {
+			continue;
+		}
+
+		for (v = ast_variable_browse(cfg, cat); v; v = v->next) {
+			if (!strcasecmp(v->name, "enabled")) {
+				newenablecel = ast_true(v->value) ? 1 : 0;
+			} else if (!strcasecmp(v->name, "host")) {
+				ast_free(bs_host);
+				bs_host = ast_strdup(v->value);
+			} else if (!strcasecmp(v->name, "port")) {
+				bs_port = atoi(v->value);
+			} else if (!strcasecmp(v->name, "tube")) {
+				ast_free(bs_tube);
+				bs_tube = ast_strdup(v->value);
+			} else if (!strcasecmp(v->name, "priority")) {
+				priority = atoi(v->value);
+			} else {
+				ast_log(LOG_NOTICE, "Unknown option '%s' specified "
+						"for CEL beanstalk backend.\n", v->name);
+			}
+		}
+	}
+
+	if (reload) {
+		ast_rwlock_unlock(&config_lock);
+	}
+
+	ast_config_destroy(cfg);
+
+	if (enablecel && !newenablecel) {
+		ast_cel_backend_unregister(CEL_BACKEND_NAME);
+	} else if (!enablecel && newenablecel) {
+		if (ast_cel_backend_register(CEL_BACKEND_NAME, cel_bs_put)) {
+			ast_log(LOG_ERROR, "Unable to register Beanstalkd CEL handling\n");
+		}
+	}
+
+	enablecel = newenablecel;
+
+	return 0;
+}
+
+static int unload_module(void)
+{
+	ast_cel_backend_unregister(CEL_BACKEND_NAME);
+	ast_free(bs_host);
+	ast_free(bs_tube);
+	return 0;
+}
+
+static int load_module(void)
+{
+	if (load_config(0)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int reload(void)
+{
+	return load_config(1);
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Beanstalkd CEL Backend",
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
+);
diff -pruN 1:13.23.1~dfsg-2/cel/cel_custom.c 1:16.1.1~dfsg-1/cel/cel_custom.c
--- 1:13.23.1~dfsg-2/cel/cel_custom.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_custom.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/paths.h"
 #include "asterisk/channel.h"
 #include "asterisk/cel.h"
@@ -225,4 +223,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
-	);
+	.requires = "cel",
+);
diff -pruN 1:13.23.1~dfsg-2/cel/cel_manager.c 1:16.1.1~dfsg-1/cel/cel_manager.c
--- 1:13.23.1~dfsg-2/cel/cel_manager.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_manager.c	2018-12-26 18:45:13.000000000 +0000
@@ -195,8 +195,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/cel.h"
 #include "asterisk/module.h"
@@ -382,4 +380,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
 );
diff -pruN 1:13.23.1~dfsg-2/cel/cel_odbc.c 1:16.1.1~dfsg-1/cel/cel_odbc.c
--- 1:13.23.1~dfsg-2/cel/cel_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <time.h>
 #include <math.h>
@@ -344,11 +342,11 @@ static SQLHSTMT generic_prepare(struct o
 	return stmt;
 }
 
-#define LENGTHEN_BUF1(size)														\
+#define LENGTHEN_BUF(size, var_sql)														\
 			do {																\
 				/* Lengthen buffer, if necessary */								\
-				if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) {		\
-					if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 1) / 512 + 1) * 512) != 0) { \
+				if (ast_str_strlen(var_sql) + size + 1 > ast_str_size(var_sql)) {		\
+					if (ast_str_make_space(&var_sql, ((ast_str_size(var_sql) + size + 1) / 512 + 1) * 512) != 0) { \
 						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", tableptr->connection, tableptr->table); \
 						ast_free(sql);											\
 						ast_free(sql2);											\
@@ -358,18 +356,11 @@ static SQLHSTMT generic_prepare(struct o
 				}																\
 			} while (0)
 
-#define LENGTHEN_BUF2(size)														\
-			do {																\
-				if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) {		\
-					if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) { \
-						ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", tableptr->connection, tableptr->table); \
-						ast_free(sql);											\
-						ast_free(sql2);											\
-						AST_RWLIST_UNLOCK(&odbc_tables);						\
-						return;													\
-					}															\
-				}																\
-			} while (0)
+#define LENGTHEN_BUF1(size) \
+	LENGTHEN_BUF(size, sql);
+
+#define LENGTHEN_BUF2(size) \
+	LENGTHEN_BUF(size, sql2);
 
 static void odbc_log(struct ast_event *event)
 {
@@ -405,7 +396,7 @@ static void odbc_log(struct ast_event *e
 	}
 
 	AST_LIST_TRAVERSE(&odbc_tables, tableptr, list) {
-		int first = 1;
+		char *separator = "";
 		ast_str_set(&sql, 0, "INSERT INTO %s (", tableptr->table);
 		ast_str_set(&sql2, 0, " VALUES (");
 
@@ -539,11 +530,11 @@ static void odbc_log(struct ast_event *e
 						}
 					}
 
-					ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+					ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 					LENGTHEN_BUF2(strlen(colptr));
 
 					/* Encode value, with escaping */
-					ast_str_append(&sql2, 0, "%s'", first ? "" : ",");
+					ast_str_append(&sql2, 0, "%s'", separator);
 					for (tmp = colptr; *tmp; tmp++) {
 						if (*tmp == '\'') {
 							ast_str_append(&sql2, 0, "''");
@@ -583,9 +574,9 @@ static void odbc_log(struct ast_event *e
 							}
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(17);
-						ast_str_append(&sql2, 0, "%s{d '%04d-%02d-%02d'}", first ? "" : ",", year, month, day);
+						ast_str_append(&sql2, 0, "%s{d '%04d-%02d-%02d'}", separator, year, month, day);
 					}
 					break;
 				case SQL_TYPE_TIME:
@@ -608,9 +599,9 @@ static void odbc_log(struct ast_event *e
 							}
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(15);
-						ast_str_append(&sql2, 0, "%s{t '%02d:%02d:%02d'}", first ? "" : ",", hour, minute, second);
+						ast_str_append(&sql2, 0, "%s{t '%02d:%02d:%02d'}", separator, hour, minute, second);
 					}
 					break;
 				case SQL_TYPE_TIMESTAMP:
@@ -623,9 +614,9 @@ static void odbc_log(struct ast_event *e
 							 * We've already properly formatted the timestamp so there's no need
 							 * to parse it and re-format it.
 							 */
-							ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+							ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 							LENGTHEN_BUF2(27);
-							ast_str_append(&sql2, 0, "%s{ts '%s'}", first ? "" : ",", colptr);
+							ast_str_append(&sql2, 0, "%s{ts '%s'}", separator, colptr);
 						} else {
 							int year = 0, month = 0, day = 0, hour = 0, minute = 0;
 							/* MUST use double for microsecond precision */
@@ -670,9 +661,9 @@ static void odbc_log(struct ast_event *e
 								}
 							}
 
-							ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+							ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 							LENGTHEN_BUF2(27);
-							ast_str_append(&sql2, 0, "%s{ts '%04d-%02d-%02d %02d:%02d:%09.6lf'}", first ? "" : ",", year, month, day, hour, minute, second);
+							ast_str_append(&sql2, 0, "%s{ts '%04d-%02d-%02d %02d:%02d:%09.6lf'}", separator, year, month, day, hour, minute, second);
 						}
 					}
 					break;
@@ -684,9 +675,9 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(12);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_BIGINT:
@@ -698,9 +689,9 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(24);
-						ast_str_append(&sql2, 0, "%s%lld", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%lld", separator, integer);
 					}
 					break;
 				case SQL_SMALLINT:
@@ -711,9 +702,9 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(7);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_TINYINT:
@@ -724,9 +715,9 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(4);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_BIT:
@@ -739,9 +730,9 @@ static void odbc_log(struct ast_event *e
 						if (integer != 0)
 							integer = 1;
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(2);
-						ast_str_append(&sql2, 0, "%s%d", first ? "" : ",", integer);
+						ast_str_append(&sql2, 0, "%s%d", separator, integer);
 					}
 					break;
 				case SQL_NUMERIC:
@@ -753,9 +744,9 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(entry->decimals + 2);
-						ast_str_append(&sql2, 0, "%s%*.*lf", first ? "" : ",", entry->decimals, entry->radix, number);
+						ast_str_append(&sql2, 0, "%s%*.*lf", separator, entry->decimals, entry->radix, number);
 					}
 					break;
 				case SQL_FLOAT:
@@ -768,16 +759,16 @@ static void odbc_log(struct ast_event *e
 							continue;
 						}
 
-						ast_str_append(&sql, 0, "%s%s", first ? "" : ",", entry->name);
+						ast_str_append(&sql, 0, "%s%s", separator, entry->name);
 						LENGTHEN_BUF2(entry->decimals);
-						ast_str_append(&sql2, 0, "%s%lf", first ? "" : ",", number);
+						ast_str_append(&sql2, 0, "%s%lf", separator, number);
 					}
 					break;
 				default:
 					ast_log(LOG_WARNING, "Column type %d (field '%s:%s:%s') is unsupported at this time.\n", entry->type, tableptr->connection, tableptr->table, entry->name);
 					continue;
 				}
-				first = 0;
+				separator = ", ";
 			}
 		}
 
@@ -865,4 +856,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel,res_odbc",
 );
diff -pruN 1:13.23.1~dfsg-2/cel/cel_pgsql.c 1:16.1.1~dfsg-1/cel/cel_pgsql.c
--- 1:13.23.1~dfsg-2/cel/cel_pgsql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_pgsql.c	2018-12-26 18:45:13.000000000 +0000
@@ -44,8 +44,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libpq-fe.h>
 
 #include "asterisk/config.h"
@@ -60,6 +58,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #define PGSQL_BACKEND_NAME "CEL PGSQL backend"
 
+#define PGSQL_MIN_VERSION_SCHEMA 70300
+
 static char *config = "cel_pgsql.conf";
 
 static char *pghostname;
@@ -69,9 +69,12 @@ static char *pgpassword;
 static char *pgappname;
 static char *pgdbport;
 static char *table;
+static char *schema;
 
 static int connected = 0;
+/* Optimization to reduce number of memory allocations */
 static int maxsize = 512, maxsize2 = 512;
+static int usegmtime = 0;
 
 /*! \brief show_user_def is off by default */
 #define CEL_SHOW_USERDEF_DEFAULT	0
@@ -95,11 +98,11 @@ struct columns {
 
 static AST_RWLIST_HEAD_STATIC(psql_columns, columns);
 
-#define LENGTHEN_BUF1(size) \
+#define LENGTHEN_BUF(size, var_sql) \
 	do { \
 		/* Lengthen buffer, if necessary */ \
-		if (ast_str_strlen(sql) + size + 1 > ast_str_size(sql)) { \
-			if (ast_str_make_space(&sql, ((ast_str_size(sql) + size + 3) / 512 + 1) * 512) != 0) { \
+		if (ast_str_strlen(var_sql) + size + 1 > ast_str_size(var_sql)) { \
+			if (ast_str_make_space(&var_sql, ((ast_str_size(var_sql) + size + 3) / 512 + 1) * 512) != 0) { \
 				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", pghostname, table); \
 				ast_free(sql); \
 				ast_free(sql2); \
@@ -109,18 +112,10 @@ static AST_RWLIST_HEAD_STATIC(psql_colum
 		} \
 	} while (0)
 
+#define LENGTHEN_BUF1(size) \
+	LENGTHEN_BUF(size, sql);
 #define LENGTHEN_BUF2(size) \
-	do { \
-		if (ast_str_strlen(sql2) + size + 1 > ast_str_size(sql2)) { \
-			if (ast_str_make_space(&sql2, ((ast_str_size(sql2) + size + 3) / 512 + 1) * 512) != 0) { \
-				ast_log(LOG_ERROR, "Unable to allocate sufficient memory.  Insert CEL '%s:%s' failed.\n", pghostname, table); \
-				ast_free(sql); \
-				ast_free(sql2); \
-				AST_RWLIST_UNLOCK(&psql_columns); \
-				return; \
-			} \
-		} \
-	} while (0)
+	LENGTHEN_BUF(size, sql2);
 
 static void pgsql_reconnect(void)
 {
@@ -166,7 +161,7 @@ static void pgsql_log(struct ast_event *
 
 	ast_mutex_lock(&pgsql_lock);
 
-	ast_localtime(&record.event_time, &tm, NULL);
+	ast_localtime(&record.event_time, &tm, usegmtime ? "GMT" : NULL);
 	ast_strftime(timestr, sizeof(timestr), DATE_FORMAT, &tm);
 
 	if ((!connected) && pghostname && pgdbuser && pgpassword && pgdbname) {
@@ -203,7 +198,7 @@ static void pgsql_log(struct ast_event *
 		AST_RWLIST_RDLOCK(&psql_columns);
 		AST_RWLIST_TRAVERSE(&psql_columns, cur, list) {
 			LENGTHEN_BUF1(strlen(cur->name) + 2);
-			ast_str_append(&sql, 0, "%s\"%s\"", first ? "" : ",", cur->name);
+			ast_str_append(&sql, 0, "%s\"%s\"", SEP, cur->name);
 
 			if (strcmp(cur->name, "eventtime") == 0) {
 				if (strncmp(cur->type, "int", 3) == 0) {
@@ -218,7 +213,7 @@ static void pgsql_log(struct ast_event *
 				} else {
 					/* char, hopefully */
 					LENGTHEN_BUF2(31);
-					ast_localtime(&record.event_time, &tm, NULL);
+					ast_localtime(&record.event_time, &tm, usegmtime ? "GMT" : NULL);
 					ast_strftime(buf, sizeof(buf), DATE_FORMAT, &tm);
 					ast_str_append(&sql2, 0, "%s'%s'", SEP, buf);
 				}
@@ -389,6 +384,14 @@ static void pgsql_log(struct ast_event *
 		}
 		PQclear(result);
 
+		/* Next time, just allocate buffers that are that big to start with. */
+		if (ast_str_strlen(sql) > maxsize) {
+			maxsize = ast_str_strlen(sql);
+		}
+		if (ast_str_strlen(sql2) > maxsize2) {
+			maxsize2 = ast_str_strlen(sql2);
+		}
+
 ast_log_cleanup:
 		ast_free(sql);
 		ast_free(sql2);
@@ -436,6 +439,10 @@ static int my_unload_module(void)
 		ast_free(table);
 		table = NULL;
 	}
+	if (schema) {
+		ast_free(schema);
+		schema = NULL;
+	}
 	while ((current = AST_RWLIST_REMOVE_HEAD(&psql_columns, list))) {
 		ast_free(current);
 	}
@@ -534,6 +541,21 @@ static int process_my_load_module(struct
 	if ((tmp = ast_variable_retrieve(cfg, "global", "show_user_defined"))) {
 		cel_show_user_def = ast_true(tmp) ? 1 : 0;
 	}
+	if ((tmp = ast_variable_retrieve(cfg, "global", "usegmtime"))) {
+		usegmtime = ast_true(tmp);
+	} else {
+		usegmtime = 0;
+	}
+	if (!(tmp = ast_variable_retrieve(cfg, "global", "schema"))) {
+		tmp = "";
+	}
+	if (schema) {
+		ast_free(schema);
+	}
+	if (!(schema = ast_strdup(tmp))) {
+		ast_log(LOG_WARNING,"PostgreSQL Ran out of memory copying schema info\n");
+		return AST_MODULE_LOAD_DECLINE;
+	}
 	if (DEBUG_ATLEAST(3)) {
 		if (ast_strlen_zero(pghostname)) {
 			ast_log(LOG_DEBUG, "cel_pgsql: using default unix socket\n");
@@ -551,23 +573,50 @@ static int process_my_load_module(struct
 
 	pgsql_reconnect();
 	if (PQstatus(conn) != CONNECTION_BAD) {
-		char sqlcmd[512];
-		char *fname, *ftype, *flen, *fnotnull, *fdef;
-		char *tableptr;
-		int i, rows;
+		char sqlcmd[768];
+		char *fname, *ftype, *flen, *fnotnull, *fdef, *tablename, *tmp_tablename;
+		int i, rows, version;
 
 		ast_debug(1, "Successfully connected to PostgreSQL database.\n");
 		connected = 1;
 
+		version = PQserverVersion(conn);
 		/* Remove any schema name from the table */
-		if ((tableptr = strrchr(table, '.'))) {
-			tableptr++;
+		if ((tmp_tablename = strrchr(table, '.'))) {
+			tmp_tablename++;
 		} else {
-			tableptr = table;
+			tmp_tablename = table;
+		}
+		tablename = ast_alloca(strlen(tmp_tablename) * 2 + 1);
+		PQescapeStringConn(conn, tablename, tmp_tablename, strlen(tmp_tablename), NULL);
+		if (version >= PGSQL_MIN_VERSION_SCHEMA) {
+			char *schemaname;
+			int lenschema;
+			lenschema = strlen(schema);
+			schemaname = ast_alloca(lenschema * 2 + 1);
+			PQescapeStringConn(conn, schemaname, schema, lenschema, NULL);
+
+			snprintf(sqlcmd, sizeof(sqlcmd),
+			"SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod "
+			"FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace "
+			         "AND c.relname = '%s' AND n.nspname = %s%s%s) "
+			       "INNER JOIN pg_catalog.pg_attribute a ON ("
+			           "NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) "
+			    "INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) "
+			"LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid "
+			  "AND d.adnum = a.attnum "
+			"ORDER BY n.nspname, c.relname, attnum",
+				tablename,
+				lenschema == 0 ? "" : "'", lenschema == 0 ? "current_schema()" : schemaname, lenschema == 0 ? "" : "'");
+		} else {
+			snprintf(sqlcmd, sizeof(sqlcmd),
+			"SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod "
+			"FROM pg_class c, pg_type t, pg_attribute a "
+			"LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid "
+			"AND d.adnum = a.attnum WHERE c.oid = a.attrelid AND a.atttypid = t.oid "
+			"AND (a.attnum > 0) AND c.relname = '%s' ORDER BY c.relname, attnum", tablename);
 		}
-
 		/* Query the columns */
-		snprintf(sqlcmd, sizeof(sqlcmd), "select a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc from pg_class c, pg_type t, pg_attribute a left outer join pg_attrdef d on a.atthasdef and d.adrelid = a.attrelid and d.adnum = a.attnum where c.oid = a.attrelid and a.atttypid = t.oid and (a.attnum > 0) and c.relname = '%s' order by c.relname, attnum", tableptr);
 		result = PQexec(conn, sqlcmd);
 		if (PQresultStatus(result) != PGRES_TUPLES_OK) {
 			pgerror = PQresultErrorMessage(result);
@@ -660,4 +709,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
 );
diff -pruN 1:13.23.1~dfsg-2/cel/cel_radius.c 1:16.1.1~dfsg-1/cel/cel_radius.c
--- 1:13.23.1~dfsg-2/cel/cel_radius.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_radius.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Rev$")
-
 #include RADIUS_HEADER_STR
 
 #include "asterisk/channel.h"
@@ -233,18 +231,6 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	/*
-	 * start logging
-	 *
-	 * NOTE: Yes this causes a slight memory leak if the module is
-	 * unloaded.  However, it is better than a crash if cdr_radius
-	 * and cel_radius are both loaded.
-	 */
-	tmp = ast_strdup("asterisk");
-	if (tmp) {
-		rc_openlog((char *) tmp);
-	}
-
 	/* read radiusclient-ng config file */
 	if (!(rh = rc_read_config(radiuscfg))) {
 		ast_log(LOG_NOTICE, "Cannot load radiusclient-ng configuration file %s.\n", radiuscfg);
@@ -273,4 +259,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
 );
diff -pruN 1:13.23.1~dfsg-2/cel/cel_sqlite3_custom.c 1:16.1.1~dfsg-1/cel/cel_sqlite3_custom.c
--- 1:13.23.1~dfsg-2/cel/cel_sqlite3_custom.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_sqlite3_custom.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sqlite3.h>
 
 #include "asterisk/paths.h"
@@ -70,6 +68,7 @@ static char table[80];
  * \bug Handling of this var is crash prone on reloads
  */
 static char *columns;
+static int busy_timeout;
 
 struct values {
 	char *expression;
@@ -189,6 +188,15 @@ static int load_config(int reload)
 		strcpy(table, "cel");
 	}
 
+	/* sqlite3_busy_timeout in miliseconds */
+	if ((tmp = ast_variable_retrieve(cfg, "master", "busy_timeout")) != NULL) {
+		if (ast_parse_arg(tmp, PARSE_INT32|PARSE_DEFAULT, &busy_timeout, 1000) != 0) {
+			ast_log(LOG_WARNING, "Invalid busy_timeout value '%s' specified. Using 1000 instead.\n", tmp);
+		}
+	} else {
+		busy_timeout = 1000;
+	}
+
 	/* Columns */
 	if (load_column_config(ast_variable_retrieve(cfg, "master", "columns"))) {
 		ast_config_destroy(cfg);
@@ -308,7 +316,7 @@ static int load_module(void)
 		free_config();
 		return AST_MODULE_LOAD_DECLINE;
 	}
-	sqlite3_busy_timeout(db, 1000);
+	sqlite3_busy_timeout(db, busy_timeout);
 	/* is the table there? */
 	sql = sqlite3_mprintf("SELECT COUNT(*) FROM %q;", table);
 	res = sqlite3_exec(db, sql, NULL, NULL, NULL);
@@ -352,4 +360,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
 );
diff -pruN 1:13.23.1~dfsg-2/cel/cel_tds.c 1:16.1.1~dfsg-1/cel/cel_tds.c
--- 1:13.23.1~dfsg-2/cel/cel_tds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/cel/cel_tds.c	2018-12-26 18:45:13.000000000 +0000
@@ -61,8 +61,6 @@ CREATE TABLE [dbo].[cel] (
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 #include <math.h>
 
@@ -240,21 +238,21 @@ retry:
 done:
 	ast_mutex_unlock(&tds_lock);
 
-	free(accountcode_ai);
-	free(clidnum_ai);
-	free(clid_ai);
-	free(cidani_ai);
-	free(cidrdnis_ai);
-	free(ciddnid_ai);
-	free(exten_ai);
-	free(context_ai);
-	free(channel_ai);
-	free(app_ai);
-	free(appdata_ai);
-	free(uniqueid_ai);
-	free(linkedid_ai);
-	free(userfield_ai);
-	free(peer_ai);
+	ast_free(accountcode_ai);
+	ast_free(clidnum_ai);
+	ast_free(clid_ai);
+	ast_free(cidani_ai);
+	ast_free(cidrdnis_ai);
+	ast_free(ciddnid_ai);
+	ast_free(exten_ai);
+	ast_free(context_ai);
+	ast_free(channel_ai);
+	ast_free(app_ai);
+	ast_free(appdata_ai);
+	ast_free(uniqueid_ai);
+	ast_free(linkedid_ai);
+	ast_free(userfield_ai);
+	ast_free(peer_ai);
 
 	return;
 }
@@ -317,11 +315,11 @@ static int execute_and_consume(DBPROCESS
 	va_end(ap);
 
 	if (dbfcmd(dbproc, buffer) == FAIL) {
-		free(buffer);
+		ast_free(buffer);
 		return 1;
 	}
 
-	free(buffer);
+	ast_free(buffer);
 
 	if (dbsqlexec(dbproc) == FAIL) {
 		return 1;
@@ -582,4 +580,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CDR_DRIVER,
+	.requires = "cel",
 );
diff -pruN 1:13.23.1~dfsg-2/ChangeLog 1:16.1.1~dfsg-1/ChangeLog
--- 1:13.23.1~dfsg-2/ChangeLog	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/ChangeLog	2018-12-26 18:45:13.000000000 +0000
@@ -1,43044 +1,62668 @@
-2018-09-20 18:38 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2018-12-26 18:45 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-	* asterisk 13.23.1 Released.
+	* asterisk 16.1.1 Released.
 
-2018-08-16 10:45 +0000 [e6b0c4d27e]  Sean Bright <sean.bright@gmail.com>
+2018-12-26 10:24 +0000 [227c8bb802]  George Joseph <gjoseph@digium.com>
 
-	* AST-2018-009: Fix crash processing websocket HTTP Upgrade requests
+	* Revert "stasis_cache:  Stop caching stasis subscription change messages"
 
-	  The HTTP request processing in res_http_websocket allocates additional
-	  space on the stack for various headers received during an Upgrade request.
-	  An attacker could send a specially crafted request that causes this code
-	  to overflow the stack, resulting in a crash.
+	  This commit caused issues with polling when combined with
+	  the revert commit "Revert "app_voicemail: Remove need to subscribe to stasis"
 
-	  * No longer allocate memory from the stack in a loop to parse the header
-	  values.  NOTE: There is a slight API change when using the passed in
-	  strings as is.  We now require the passed in strings to no longer have
-	  leading or trailing whitespace.  This isn't a problem as the only callers
-	  have already done this before passing the strings to the affected
-	  function.
+	  This reverts commit 17d6d9e1e7d0db04ebd8d2e0cd9e087ec5462e2f.
 
-	  ASTERISK-28013 #close
+	  ASTERISK-28222
+	  Reported by: abelbeck
 
-	  Change-Id: Ia564825a8a95e085fd17e658cb777fe1afa8091a
+	  Change-Id: Ib6a16cbe2c0b74c76c64264f525ab647959c1483
 
-2018-09-17 07:10 +0000 [43e816ead9]  George Joseph <gjoseph@digium.com>
+2018-12-11 20:39 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-	* CI: Fix typo in testsuite git checkout
+	* asterisk 16.1.0 Released.
 
-	  Change-Id: I30024515e5b00a5044fd39fbff27d818f016b719
+2018-12-04 00:03 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-2018-09-14 12:31 +0000 [5f777e483a]  George Joseph <gjoseph@digium.com>
+	* asterisk 16.1.0-rc1 Released.
 
-	* CI: Use proper credentials for Security testsuite checkout
+2018-12-03 17:45 +0000 [12a30c71d8]  Kevin Harwell <kharwell@digium.com>
 
-	  Can't do anonymous http checkout from Security-testsuite.
-	  Need to use same credentials as the gerrit review checkout.
+	* pjsip_add_use_callerid_contact: fixed alembic script
 
-	  Change-Id: I87af68c995cb8926f5e87f9af245600d76984f05
+	  Change-Id: I413f1583c797fb79651786cd8d0b003599f8ed10
 
-2018-09-05 17:12 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2018-12-03 16:41 +0000 [27806fd04d]  Sean Bright <sean.bright@gmail.com>
 
-	* asterisk 13.23.0 Released.
+	* core: Add some documentation to the malloc_trim code
 
-2018-08-28 19:07 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  This adds documentation to handle_cli_malloc_trim() indicating how it
+	  can be useful when debugging OOM conditions.
 
-	* asterisk 13.23.0-rc1 Released.
+	  Change-Id: I1936185e78035bf123cd5e097b793a55eeebdc78
 
-2018-08-13 08:12 +0000 [2a5d408733]  Jaco Kroon <jaco@uls.co.za>
+2018-12-03 14:01 +0000 [1f09c61804]  Chris-Savinovich <csavinovich@digium.com>
 
-	* chan_sip: improved ip:port finding of peers for non-UDP transports.
+	* core: Merge malloc_trim patch
 
-	  Also remove function peer_ipcmp_cb since it's not used (according to
-	  rmudgett).
+	  We've had multiple opportunities where Richard Mudgett's
+	  malloc_trim patch has been useful. Let's get it
+	  pushed up to gerrit and merged.
 
-	  Prior to b2c4e8660a9c89d07041271371151779b7ec75f6 (ASTERISK_27457)
-	  insecure=port was the defacto standard.  That commit also prevented
-	  insecure=port from being applied for sip/tcp or sip/tls.
+	  Since malloc_trim is only available in libc, an entry is
+	  added to configure.ac to create a definition for
+	  HAVE_MALLOC_TRIM.
 
-	  Into consideration there are three sets of behaviour:
+	  Change-Id: Ia38308c550149d9d6eae4ca414a649957de9700c
 
-	  1.  "previous" - before the above commit.
-	  2.  "current" - post above commit, pre this one.
-	  3.  "new" - post this commit.
+2018-11-29 13:26 +0000 [a2a8a41db3]  George Joseph <gjoseph@digium.com>
 
-	  The problem that the above commit tried to address was guests over TCP.
-	  It succeeded in doing that but broke transport!=udp with host!=dynamic.
+	* Revert "app_voicemail: Remove need to subscribe to stasis"
 
-	  This commit attempts to restore sane behaviour with respect to
-	  transport!=udp for host!=dynamic whilst still retaining the guest users
-	  over tcp.
+	  This reverts commit 29115e23848cceee0e2763bc70e87cb311919cdd.
 
-	  It should be noted that when looking for a peer, two passes are made, the
-	  first pass doesn't have SIP_INSECURE_PORT set for the searched-for peer,
-	  thus looking for full matches (IP + Port), the second pass sets
-	  SIP_INSECURE_PORT, thus expecting matches on IP only where the matched
-	  peer allows for that (in the author's opinion:  UDP with insecure=port,
-	  or any TCP based, non-dynamic host).
+	  That commit closed a long standing hole which allowed subscriptions
+	  to mailboxes that weren't configured in voicemail.conf.  This
+	  caused an issue with FreePBX which depdended on that behavior.
+	  The commit is being reverted until FreePBX can handle the new
+	  behavior.
 
-	  In previous behaviour there was special handling for transport=tcp|tls
-	  whereby a peer would match during the first pass if the utilized
-	  transport was TCP|TLS (and the peer allowed that specific transport).
+	  ASTERISK-28151
+	  Reported by: Ronald Raikes
 
-	  This behaviour was wrong, or dubious at best.  Consider two dynamic tcp
-	  peers, both registering from the same IP (NAT), in this case either peer
-	  could match for connections from an IP.  It's also this behaviour that
-	  prevented SIP guests over tcp.
+	  Change-Id: I57b7b85e75d7dd97c742b5c69d718a0f61260c15
 
-	  The above referenced commit removed this behaviour, but kept applying
-	  the SIP_INSECURE_PORT only to WS|WSS|UDP.  Since WS and WSS is also TCP
-	  based, the logic here should fall into the TCP category.
+2018-10-22 07:47 +0000 [aa004295f9]  lvl <digium@lvlconsultancy.nl>
 
-	  This patch updates things such that the previously non-explicit (TCP
-	  behaviour) transport test gets performed explicitly (ie, matched peer
-	  must allow for the used transport), as well as the indeterministic
-	  source-port nature of the TCP protocol is taken into account.  The new
-	  match algorithm now looks like:
+	* app_queue: Revert broken queue channel reference patch
 
-	  1.  As per previous behaviour, IP address is matched first.
+	  Revert commit 6409e7b11a2310196a9978b30a6b79e2760be592, and add
+	  NULL checks for all app_queue event handling code.
 
-	  2.  Explicit filter with respect to transport protocol, previous
-	      behaviour was semi-implied in the test for TCP pure IP match - this now
-	      made explicit.
+	  Related issues: ASTERISK~25185, ASTERISK~27006, ASTERISK~25844
 
-	  3.  During first pass (without SIP_INSECURE_PORT), always match on port.
+	  ASTERISK-28125
 
-	  4.  If doing UDP, match if matched against peer also has
-	      SIP_INSECURE_PORT, else don't match.
+	  Change-Id: I37334ea184ebb56e54471496b82937d4927815a0
 
-	  5.  Match if not a dynamic host (for non-UDP protocols)
+2018-11-26 16:18 +0000 [a5e77e9c37]  George Joseph <gjoseph@digium.com>
 
-	  6.  Don't match if this is WS|WSS, or we can't trust the Contact address
-	      (presumably due to NAT)
+	* test_cel:  Plug a few ref leaks
 
-	  7.  Match (we have a valid Contact thus if the IP matches we have no
-	      choice, this will likely only apply to non-NAT).
+	  These are only a few of the leaks.  The large number of macros
+	  and return paths in this file would make a weeks worth of work
+	  to plug them all.
 
-	  To logic-test this we need a few different scenarios.  Towards this end,
-	  I work with a set number of peers defined in sip.conf:
+	  Change-Id: Ie2369fa944023d44767871c5c30974cb077ffb56
 
-	  [peer1]
-	  host=1.1.1.1
-	  transport=tcp
+2018-11-26 06:09 +0000 [606304eb54]  Corey Farrell <git@cfware.com>
 
-	  [peer2]
-	  host=1.1.1.1
-	  transport=udp
+	* jansson: Upgrade to 2.12.
 
-	  [peer3]
-	  host=1.1.1.1
-	  port=5061
-	  insecure=port
-	  transport=udp
+	  This brings in jansson-2.12, removes all patches that were merged
+	  upstream.  README is created in third-party/jansson/patches to explain
+	  how to add patches but also because the patches folder must exist for
+	  the build process to succeed.
 
-	  [peer4]
-	  host=1.1.1.2
-	  transport=udp,tcp
+	  Change-Id: If0f2d541c50997690660c21fb7b03d625a5cdadd
 
-	  [peer5]
-	  host=dynamic
-	  transport=udp,tcp
+2018-11-23 09:40 +0000 [331c906c48]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Test cases for UDP:
+	* RTP: need to reset DTMF last seqno/timestamp on voice packet with marker bit
 
-	  1 - incoming UDP request from 1.1.1.1:
-	    - previous:
-	      - pass 1:
-	        * peer1 or peer2 if from port 5060 (indeterminate, depends on peer
-	          ordering)
-	        * peer3 if from port 5061
-	        * peer5 if registered from 1.1.1.1 and source port matches
-	      - pass 2:
-	        * peer3
-	    - current: as per previous.
-	    - new:
-	      - pass 1:
-	        * peer2 if from port 5060
-	        * peer3 if from port 5061
-	        * peer5 if registered from 1.1.1.1 and source port matches
-	      - pass 2:
-	        * peer3
+	  The marker bit set on the voice packet indicates the start
+	  of a new stream and a new time stamp.
+	  Need to reset the DTMF last sequence number and the timestamp
+	  of the last END packet.
 
-	  2 - incoming UDP request from 1.1.1.2:
-	    - previous:
-	      - pass 1:
-	        * peer5 if registered from 1.1.1.2 and port matches
-	        * peer4 if source port is 5060
-	      - pass 2:
-	        * no match (guest)
-	    - current: as previous.
-	    - new as previous (with the variation that if peer5 didn't have udp as
-	            allowed transport it would not match peer5 whereas previous
-	            and current code could).
+	  If the new time stamp is lower then the timestamp of the last DTMF END packet
+	  the asterisk drops all DTMF frames as out of order.
 
-	  3 - incoming UDP request from anywhere else:
-	    - previous:
-	      - pass 1:
-	        * peer5 if registered from that address and source port matches.
-	      - pass 2:
-	        * peer5 if insecure=port is additionally set.
-	        * no match (guest)
-	    - current - as per previous
-	    - new - as per previous
+	  This bug was caught using Cisco ip-phone SPA50X and codec g722.
+	  On SIP session update the SPA50X resets stream indicating it with market bit
+	  and a new timestamp is twice smaller then the previous.
 
-	  Test cases for TCP based transports:
+	  ASTERISK-28162 #close
 
-	  4 - incoming TCP request from 1.1.1.1
-	    - previous:
-	      - pass 1 (indeterministic, depends on ordering of peers in memory):
-	        * peer1; or
-	        * peer5 if peer5 registered from 1.1.1.1 (irrespective of source port); or
-	        * peer2 if the source port happens to be 5060; or
-	        * peer3 if the source port happens to be 5061.
-	      - pass 2: cannot happen since pass 1 will always find a peer.
-	    - current:
-	      - pass 1:
-	        * peer1 or peer2 if from source port 5060
-	        * peer3 if from source port 5060
-	        * peer5 if registered as 1.1.1.1 and source port matches
-	      - pass 2:
-	        * no match (guest)
-	    - new:
-	      - pass 1:
-	        * peer 1 if from port 5060
-	        * peer 5 if registered and source port matches
-	      - pass 2:
-	        * peer 1
+	  Change-Id: If9c5742158fa836ad549713a9814d46a5d2b1620
 
-	  5 - incoming TCP request from 1.1.1.2
-	    - previous (indeterminate, depends on ordering):
-	      - pass 1:
-	        * peer4; or
-	        * peer5 if peer5 registered from 1.1.1.2
-	      - pass 2: cannot happen since pass 1 will always find a peer.
-	    - current:
-	      - pass 1:
-	        * peer4 if source port is 5060
-	        * peer5 if peer5 registered as 1.1.1.2 and source port matches
-	      - pass 2:
-	        * no match (guest).
-	    - new:
-	      - pass 1:
-	        * peer4 if source port is 5060
-	        * peer5 if peer5 registered as 1.1.1.2 and source port matches
-	      - pass 2:
-	        * peer4
+2018-11-19 14:10 +0000 [ed7a5664b6]  Corey Farrell <git@cfware.com>
 
-	  6 - incoming TCP request from anywhere else:
-	    - previous:
-	      - pass 1:
-	        * peer5 if registered from that address
-	      - pass 2: cannot happen since pass 1 will always find a peer.
-	    - current:
-	      - pass 1:
-	        * peer5 if registered from that address and port matches.
-	      - pass 2:
-	        * no match (guest)
-	    - new: as per current.
+	* astobj2: Eliminate usage of legacy ao2_container_alloc routine.
 
-	  It should be noted the test cases don't make explicit mention of TLS, WS
-	  or WSS.  WS and WSS previously followed UDP semantics, they will now
-	  enforce source port matching.  TLS follow TCP semantics.
+	  Replace usage of ao2_container_alloc with ao2_container_alloc_hash or
+	  ao2_container_alloc_list.
 
-	  The previous commit specifically tried to address test-case 6, but broke
-	  test-cases 4 and 5 in the process.
+	  ao2_container_alloc is now restricted to modules only and is being
+	  removed from Asterisk 17.
 
-	  ASTERISK-27881 #close
+	  Change-Id: I0907d78bc66efc775672df37c8faad00f2f6c088
 
-	  Change-Id: I61a9804e4feba9c7224c481f7a10bf7eb7c7f2a2
+2018-11-14 05:02 +0000 [0d2b39065f]  Corey Farrell <git@cfware.com>
 
-2018-08-20 07:23 +0000 [b528715fa0]  Jaco Kroon <jaco@uls.co.za>
+	* astobj2: Create function to copy weak proxied objects from container.
 
-	* AMI: be less verbose when adding HTTP headers to AMI/HTTP messages.
+	  Create ao2_container_dup_weakproxy_objs to perform a similar function to
+	  ao2_container_dup.  This function expects the source container to have
+	  weakproxy objects, inserts the associated non-weak objects into the
+	  destination container.  Orphaned weakproxy objects are ignored.
 
-	  All HTTP/AMI message headers are being sent to the verbose channel.
-	  There are multiple places this is happening.  Consolidate the loop into
-	  a function.  Drop the debug/verbose message.
+	  Create test for this new function and for ao2_weakproxy_find.
 
-	  Convert to using ast_asprintf to perform the length calculation, memory
-	  allocation and snprintf all in one step.
+	  Change-Id: I898387f058057e08696fe9070f8cd94ef3a27482
 
-	  Change-Id: Ic45e673fde05bd544be95ad5cdbc69518207c1a1
+2018-11-16 14:45 +0000 [72e532136c]  Michael Walton (license 6502)
 
-2018-08-22 10:50 +0000 [adb3195697]  Matthew Fredrickson <creslin@digium.com>
+	* func_strings: HASHKEY - negative array index can cause corruption
 
-	* sample_configs: noload res_hep.so by default
+	  This patch makes it so only matching non-empty key names, and keys created by
+	  the HASH function are eligible for inclusion in the comma separated string. It
+	  also fixes a bug where it was possible to write to a negative index if the
+	  result buffer was empty.
 
-	  Change disables loading of res_hep.so in default installation.  Loading
-	  res_hep has a performance impact whether it's used or not.  This disables
-	  loading of it in sample config files.
+	  ASTERISK-28159
+	  patches:
+	    ASTERISK-28159.diff submitted by Michael Walton (license 6502)
 
-	  Change-Id: I5ec150cf941634fabc72973e5bf1a965cb0ef9d0
-	  (cherry picked from commit c8bacd45f17d2ae6f6d05c8cff538dd651f683a6)
+	  Change-Id: I6e57fe7307dfd856271753aed5ba64c59b511487
 
-2018-08-23 06:57 +0000 [1928e00d90]  Florian Floimair <f.floimair@commend.com>
+2018-11-19 11:59 +0000 [729967e99c]  George Joseph <gjoseph@digium.com>
 
-	* alembic: increase uri column size
+	* CI: Get job timeouts from environment
 
-	  When mobile SIP clients register with Asterisk that use some sort of
-	  push notifications, the URI can get quite lengthy due to the
-	  additional push-service annotations (things like tokens, pn-type, etc.)
-	  contained in it.
+	  The job timeouts were hard coded in the jenkinsfiles which
+	  means changes had to go through gerrit.  Now they are taken
+	  from the following environment variables (and their defaults) that
+	  can be set in Jenkins configuration...
 
-	  ASTERISK-28022 #close
+	  TIMEOUT_GATES =      "60 MINUTES"
+	  TIMEOUT_DAILIES =    "3 HOURS"
+	  TIMEOUT_REF_DEBUG =  "24 HOURS"
+	  TIMEOUT_UNITTESTS =  "30 MINUTES"
 
-	  Change-Id: I08e17b37af5a446c36cc7d1f903861f81fd41cca
+	  Change-Id: I673a551c1780bf665a3bc160b245da574aa4bbab
 
-2018-08-21 13:50 +0000 [b9d9c0a8b9]  Sean Bright <sean.bright@gmail.com>
+2018-11-18 17:53 +0000 [d05c219aad]  Joshua C. Colp <jcolp@digium.com>
 
-	* app_queue: Silence GCC 8 compiler warning
+	* stasis: Remove stringfields and lock from change message.
 
-	  I'm only seeing an error in 14+, so I assume it is due to different
-	  compiler options:
+	  When a subscribe or unsubscribe occurs a message is published
+	  containing this information. This change makes it so that the
+	  message no longer uses stringfields or a lock, as both are not
+	  really needed for the message.
 
-	  app_queue.c: In function ‘handle_queue_add_member’:
-	  app_queue.c:10234:19: error: ‘%d’ directive writing between 1 and 11
-	      bytes into a region of size 3 [-Werror=format-overflow=]
-	       sprintf(num, "%d", state);
-	                     ^~
-	  app_queue.c:10234:18: note: directive argument in the range
-	      [-2147483648, 99]
-	       sprintf(num, "%d", state);
-	                    ^~~~
+	  Change-Id: I3f4831931d79f94fd979baf48048738df5dc1632
 
-	  Compiler: gcc version 8.0.1 20180414 (experimental)
-	      [trunk revision 259383] (Ubuntu 8-20180414-1ubuntu2) 
+2018-11-19 07:00 +0000 [f9b9bdd0b0]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I18577590da46829c1ea7d8b82e41d69f105baa10
+	* app_queue: Cleanup queue_ref / queue_unref routines.
 
-2018-08-21 18:45 +0000 [6964bc37e0]  Alec Davis <sivad.a@paradise.net.nz>
+	  This replaces the inline functions with macros.  This removes the need
+	  to directly use __ao2_ref, opts instead for standard ao2_bump and
+	  ao2_cleanup macros.
 
-	* chan_sip:  remove unnecessary ast_channel_unlock(peer) as RAII looks after it
+	  Change-Id: If4e04e9bab2e3c883188437cb9f487b3e498a21b
 
-	  Otherwise console output
+2018-11-08 09:53 +0000 [41eab5b3b8]  George Joseph <gjoseph@digium.com>
 
-	          (get_refer_info): mutex 'peer' freed more times than we've locked!
-	          (get_refer_info): Error releasing mutex: Operation not permitted
+	* backtrace:  Refactor ast_bt_get_symbols so it doesn't crash
 
-	      or
-	          (get_refer_info): attempted unlock mutex 'peer' without owning it!
-	          (__ast_read): 'peer' was locked here.
-	          ...dump_backtrace
+	  We've been seeing crashes in libbfd when we attempt to generate
+	  a stack trace from multiple threads.  It turns out that libbfd
+	  is NOT thread-safe.  It can cache the bfd structure and give it to
+	  multiple threads without protecting itself.  To get around this,
+	  we've added a global mutex around the bfd functions and also have
+	  refactored the use of those functions to be more efficient and
+	  to provide more information about inlined functions.
 
-	          (get_refer_info): Error releasing mutex: Operation not permitted
-	          (__ast_read): mutex 'chan' freed more times than we've locked!
+	  Also added a few more tests to test_pbx.c.  One just calls
+	  ast_assert() and the other calls ast_log_backtrace().  Neither are
+	  run by default.
 
-	  ASTERISK-28011 #close
+	  WARNING:  This change necessitated changing the return value of
+	  ast_bt_get_symbols() from an array of strings to a VECTOR of
+	  strings.  However, the use of this function outside Asterisk is not
+	  likely.
 
-	  Change-Id: I6e45f2764ba4f3273a943300f91ac9b461ac2893
+	  ASTERISK-28140
 
-2018-08-21 07:59 +0000 [49d388bfa8]  Emmanuel BUU <emmanuel.buu@ives.fr>
+	  Change-Id: I79d02862ddaa2423a0809caa4b3b85c128131621
 
-	* res/res_rtp_asterisk: remove debug traces generated by an empty frame
+2018-11-16 20:33 +0000 [cb83350230]  Sungtae Kim <pchero21@gmail.com>
 
-	  The realtime text timer pops regularly and sends text frames even if
-	  the buffer is empty. This causes a lot of unecessary debug logging.
+	* res/res_ari: Fix null endpoint handle
 
-	  * Made red_write() test if we need to send a frame before calling
-	  ast_rtp_write()
+	  The res_ari(POST /channels/create handler) deos not check the endpoint
+	  parameter length. And it causes core
+	  dump.
+	  Fixed it to check the parameter length. Also fixed memory leak.
 
-	  ASTERISK-28002
-	  Reported by: Emmanuel BUU
-	  Tested by: Emmanuel BUU
+	  ASTERISK-28169
 
-	  Change-Id: Icf81310c3b8080b615a42060afc02ab41f9523dd
+	  Change-Id: Ibf10a9eb8a2e3a9ee1e13fbe748b2ecf955c3993
 
-2018-08-20 11:23 +0000 [9b9dee2d2c]  Richard Mudgett <rmudgett@digium.com>
+2018-11-13 09:28 +0000 [e6005f1227]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* AMI: Remove docs for nonexistent AMI ContactStatus event headers
+	* pjsip: New function PJSIP_PARSE_URI to parse URI and return part of URI
 
-	  Change-Id: I5736965c64c44338f7330e85a24bb46818607f19
+	  New dialplan function PJSIP_PARSE_URI added to parse an URI and return
+	  a specified part of the URI.
 
-2018-08-09 18:46 +0000 [a66fa4db24]  Richard Mudgett <rmudgett@digium.com>
+	  This is useful when need to get part of the URI instead of cutting it
+	  using a CUT function.
 
-	* res_sorcery_realtime.c: Fix unqualified fetch warning.
+	  For example to get 'user' part of Remote URI
+	  ${PJSIP_PARSE_URI(${CHANNEL(pjsip,remote_uri)},user)}
 
-	  The allow_unqualified_fetch option for the sorcery realtime backend
-	  blocked actually fetching all rows when the option is set to warn.
+	  ASTERISK-28144 #close
 
-	  * Made issue a warning and actually do the request when
-	  allow_unqualified_fetch=warn is set.
+	  Change-Id: I5d828fb87f6803b6c1152bb7b44835f027bb9d5a
 
-	  Change-Id: I74456c80a03a62dce66fc3dc3cb0cf2351ac4312
+2018-09-23 15:50 +0000 [8d436a95e7]  Joshua Colp <jcolp@digium.com>
 
-2018-08-15 21:31 +0000 [f3924b576a]  Richard Mudgett <rmudgett@digium.com>
+	* stasis: Add internal filtering of messages.
 
-	* pbx_dundi.c: Misc memory management fixes when destroying peers
+	  This change adds the ability for subscriptions to indicate
+	  which message types they are interested in accepting. By
+	  doing so the filtering is done before being dispatched
+	  to the subscriber, reducing the amount of work that has
+	  to be done.
 
-	  * In destroy_peer(), fixed memory leaks of lookup history strings and
-	  qualify transactions when destroying peers.
+	  This is optional and if a subscriber does not add
+	  message types they wish to accept and set the subscription
+	  to selective filtering the previous behavior is preserved
+	  and they receive all messages.
 
-	  * In destroy_peer(), fixed leaving the registerexpire scheduled callback
-	  active when a peer is destroyed on a reload.  The reload marks and sweeps
-	  peers so any peers not explicitly configured get destroyed.  Peers created
-	  dynamically from the '*' peer will not exist until they re-register after
-	  the reload.  These destroyed peers caused memory corruption when the
-	  registerexpire timer expired.
+	  There is also the ability to explicitly force the reception
+	  of all messages for cases such as AMI or ARI where a large
+	  number of messages are expected that are then generically
+	  converted into a different format.
 
-	  * Made build_peer() not schedule any callbacks on the '*' peer
-	  (empty_eid).  It is a special peer that is cloned to dynamically created
-	  peers so it doesn't actually get involved in any message transactions.
+	  ASTERISK-28103
 
-	  * Made do_register_expire() remove the dundi/dpeers AstDB entry when a
-	  peer registration expires.
+	  Change-Id: I99bee23895baa0a117985d51683f7963b77aa190
 
-	  * Fix deep_copy_peer() to not copy some things that cannot be copied to
-	  the cloned peer structure.  Timers, message transactions, and lookup
-	  history are specific to a peer instance.
+2018-11-18 10:38 +0000 [dd0a3c0bba]  George Joseph <gjoseph@digium.com>
 
-	  * Made set_config() lock around processing the mappings configuration.
+	* CI:  Add tmpfs to all jenkinsfiles
 
-	  * Reordered unload_module() to handle load_module() declining the load due
-	  to error.
+	  Change-Id: Ida29d70d48d5f39aabf0b25c66b51f79324a8cba
 
-	  Change-Id: Ib846b2b60d027f3a2c2b3b563d9a83a357dce1d6
+2018-11-17 15:40 +0000 [77e9c877b5]  George Joseph <gjoseph@digium.com>
 
-2018-08-15 23:49 +0000 [fbc53412f6]  Richard Mudgett <rmudgett@digium.com>
+	* CI:  Mount a tmpfs on /tmp for testsuite docker containers
 
-	* pbx_dundi.c: Handle thread shutdown better.
+	  Change-Id: I0566d81b0852f22066cd76d58eae5f1fda5602aa
+	  (cherry picked from commit 73efe86436427e5f43c532e5d42505ab4ec104d9)
 
-	  Change-Id: Id52f99bd6a948fe6dd82acc0a28b2447a224fe87
+2018-11-17 13:07 +0000 [0552bc2a94]  George Joseph <gjoseph@digium.com>
 
-2018-08-15 18:14 +0000 [ad2dfb07b0]  Richard Mudgett <rmudgett@digium.com>
+	* CI:  Pass work directory to runTestsuite
 
-	* pbx_dundi: Fix debug frame decode string.
+	  The testsuite can now use a user-specified work directory for
+	  all it's temp files.  This allows the docker containers to use
+	  a tmpfs backed directory for the temp files instead of it's
+	  own write-layer image.
 
-	  * Fixed a typo in the name of the REGREQ frame decode string array.
-	  * Fixed off by one range check indexing into the frame decode string
-	  array.
-	  * Removed some unneeded casts associated with the decode string array.
+	  * runTestsuite.sh now accepts a --work-dir command line argument
+	    that gets exported as AST_WORK_DIR before running the testsuite.
 
-	  Change-Id: I77435e81cd284bab6209d545919bf236ad7933c2
+	  * gates.jenkinsfile now specifies --work-dir to be
+	    <testsuite_dir>/astroot.
 
-2018-08-16 16:21 +0000 [0a7dab8904]  Richard Mudgett <rmudgett@digium.com>
+	  Since the Asterisk CI docker hosts now mount /srv/jenkins/workspace
+	  on a tmpfs, asterisk should be compiled and the testsuite run all in
+	  memory.
 
-	* pbx_dundi: Update sample config documentation.
+	  Change-Id: If5ee905a15821296c355bb84cda38950ad8edc45
+	  (cherry picked from commit a335f4c9adb0a00211345634f61917bdf5b412c2)
 
-	  Change-Id: I33d0ad0611c2124ca3440f0f811fa0f45e4e2849
+2018-11-15 11:41 +0000 [7ba8f32632]  George Joseph <gjoseph@digium.com>
 
-2018-08-16 13:28 +0000 [33a1593fba]  George Joseph <gjoseph@digium.com>
+	* CI: Allow runUnittests to use 'expect' to run the tests
 
-	* CI:  Final version of setting correct gerrit creds
+	  There seems to be a race condition between starting the asterisk
+	  daemon and attempting to use 'asterisk -r' that can cause the
+	  control socket file to not be created.  Since all of the Jenkins
+	  slaves have 'expect' installed, the runUnittests script can use
+	  it to start asterisk in the forground and issue the commands
+	  interactively.  This is much more reliable and it can also make
+	  startup errors more visible since they'll be in the Jenkins console
+	  output.
 
-	  Change-Id: I7729ecceedceb12f52bf18dae259846aa1d993b3
+	  If 'expect' isn't installed, the original daemon/asterisk -r
+	  process is used.
 
-2018-08-16 13:23 +0000 [ab095ca4c6]  George Joseph <gjoseph@digium.com>
+	  Also added a "core show settings" before running the tests
+	  and added "notice,warning,error" to the console log.
 
-	* CI: Yet Another stab at creds
+	  Change-Id: Idd656085f854afede813ac241b9e312b31358160
 
-	  Change-Id: I768850780d39151c5dd8e2bb1a4b24776727958e
+2018-11-12 12:23 +0000 [699dfa9401]  Corey Farrell <git@cfware.com>
 
-2018-08-16 13:11 +0000 [b1e294c670]  George Joseph <gjoseph@digium.com>
+	* taskprocessor: Prevent race creating new taskprocessor.
 
-	* CI: Another stab at creds
+	  Task processors are retrieved using a 'get or create' pattern.  The
+	  singleton container was unlocked between the get and create steps so
+	  it's possible that two threads could create task processors with the
+	  same name at the same time.
 
-	  Change-Id: I5b1dc3b354789e676c27a4e1fc42f5c3343cc994
+	  Change-Id: Id64fae94a6a1e940ddf38fde622dcd4391635382
 
-2018-08-16 12:08 +0000 [45d096fe8d]  George Joseph <gjoseph@digium.com>
+2018-11-16 06:20 +0000 [3bdf6ac918]  Corey Farrell <git@cfware.com>
 
-	* CI:  Add https credentials to gerrit checkouts
+	* pjproject-bundled: Use AST_DEVMODE for conditional compilation.
 
-	  If the review to be tested is in a project with restricted access,
-	  we need to use the jenkins user's gerrit https credentials when we
-	  do the checkout or the checkout will fail.
+	  We previously allowed resample and g711 codecs to be built when
+	  TEST_FRAMEWORK was enabled.  This could cause errors if the testsuite
+	  was run without this option enabled.  Switch the build system to allow
+	  those codecs to be built when --enable-dev-mode is used.  This removes a
+	  chance for strange testsuite errors from use of an inadequate pjsua
+	  binary.
 
-	  Change-Id: I9dc9994763c5ebfeb9f1cff60fb53f6902b7fd5f
+	  Change-Id: Iee8a3613cdb711fa7e7d217c5a775a575907ae22
 
-2018-08-15 13:51 +0000 [db367ddbbf]  Corey Farrell <git@cfware.com>
+2018-11-15 14:47 +0000 [2a76489eba]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip: Resolve transport management leak at shutdown.
+	* res_pjsip_caller_id: Use static pj_str_t for fromto header names.
 
-	  Cleanup idle check scheduled events at shutdown.
+	  PJSIP assumes that these header names are not allocated, does not clone
+	  the name strings when reusing headers.
 
-	  Change-Id: I61bfbb56bac69fe840c3242927d31ff3593be461
+	  Block unload of res_pjsip_caller_id until shutdown to ensure static
+	  memory stays valid.  It was previously unsafe to unload while any
+	  sessions are active.
 
-2018-08-15 11:44 +0000 [7f4420a222]  Corey Farrell <git@cfware.com>
+	  Change-Id: I190854dea943d6e441cf03733f8a0da661aea27f
 
-	* CI: Disable res_odbc if REF_DEBUG is requested.
+2018-10-24 07:38 +0000 [7b2282c890]  Torrey Searle <torrey@voxbone.com>
 
-	  This is for 13/15 only, res_odbc never unloads so it is impossible to do
-	  REF_DEBUG testing with that module.
+	* res/res_pjsip_nat: Fix logic for REINVITES
 
-	  Change-Id: I2c1e32b80564e8fb08b6b5900ede6b5d304ebd10
+	  The presence of Record-Route in re-invites is optional, thus it is
+	  important to make sure the dialog doesn't have a routset before
+	  rewriting the contact header.
 
-2018-08-15 11:12 +0000 [7df97d0a00]  Corey Farrell <git@cfware.com>
+	  ASTERISK-28129 #close
 
-	* res_pjsip: Fix leak in pjsip_options.
+	  Change-Id: Ic8ceb54ccfc93f7e315e476c514a2c777f2da7dc
 
-	  sip_options_get_endpoint_state_compositor_state leaked a reference to
-	  the first available endpoint state compositor that was found.
+2018-11-15 05:33 +0000 [a36cd6960c]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Idb6be19f7219b6eed1dfb19c1e740dd40cb3fdc7
+	* core: Fix handling of restart from remote console.
 
-2018-08-14 11:55 +0000 [e8ac75eed0]  Richard Mudgett <rmudgett@digium.com>
+	  We cannot use need_el_end and SIGURG when restarting.  Instead we need
+	  to run el_end within the SIGHUP restartnow handler.
 
-	* contrib/scripts: Make astgenkey executable
+	  ASTERISK-28158
 
-	  Change-Id: I11641d65592536dea9cbca5aa94a24c25d24dd5f
+	  Change-Id: Ia852276363c81bdcf1aa29eb4558c5c2fa1218a0
 
-2018-08-14 07:29 +0000 [555516233d]  Joshua Colp <jcolp@digium.com>
+2018-10-25 10:25 +0000 [009d67cd28]  Jan Hoffmann <jan@3e8.eu> (license 6986)
 
-	* res_pjsip_caller_id: Add "party" parameter to RPID header.
+	* AST-2018-010: Fix length of buffer needed for SRV and NAPTR results
 
-	  This change adds the "party" parameter to the Remote-Party-ID header
-	  which indicates which party the header information is applicable
-	  to. In Asterisk this is determined on whether we are the calling
-	  or called party. This is added to improve interoperability with some
-	  implementations.
+	  When dn_expand was being called on SRV and NAPTR results, the
+	  return value was being used to calculate the size of the buffer
+	  needed to store the host names.  Since dn_expand returns the
+	  length of the COMPRESSED name the buffer could be too short
+	  to hold the EXPANDED name.  The expanded name is NULL terminated
+	  so using strlen() is the correct way to determine the length
+	  actually needed for the buffer.
 
-	  ASTERISK-28006
+	  ASTERISK-28127
+	  Reported by: Jan Hoffmann
 
-	  Change-Id: I1eec3e377ffff8633b5c1dd59a05e9533122cfca
+	  patches:
+	    patch.diff submitted by janhoffmann (license 6986)
 
-2018-08-12 11:04 +0000 [f48761907a]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Change-Id: I4d35d6c431c6c6836cb61d37b1378cc47f0b414d
 
-	* app_queue: set QUEUESTATUS to LEAVEEMPTY instead of CONTINUE
+2018-11-13 10:51 +0000 [89d97da8a4]  Corey Farrell <git@cfware.com>
 
-	  When a call leaves a queue on leaveempty condition, QUEUESTATUS
-	  must be set to LEAVEEMPTY, no matter whether Queue was executed with or
-	  without the "c" (continue) option.
+	* test_res_pjsip_scheduler: Fix possible write after free in scheduler_policy.
 
-	  The regression was introduced in the fix for ASTERISK_25665.
-	  The following fix (ASTERISK_27065) was incomplete, as QUEUESTATUS was
-	  overwritten in case when "c" is set, regardless of what was the cause
-	  for leaving the queue.
+	  It's possible for a 4th task to be spawned before we cancel.  This
+	  results in a write to the already freed test_data1.  Wait long enough to
+	  verify success of the cancelation before freeing test_data1.
 
-	  ASTERISK-27973 #close
-	  Reported-by: Valentin Safonov
+	  Change-Id: I057e2fcbe97f8a175e50890be89c28c20490a20f
 
-	  Change-Id: Iec013fe6a26a4e825ca572a1dda4f3cee5f6f80c
+2018-10-17 08:48 +0000 [df703c1853]  Robert Cripps <rcripps@voxbone.com>
 
-2018-08-09 15:25 +0000 [dc786aa576]  Corey Farrell <git@cfware.com>
+	* bridge_native_rtp.c: Fail native bridge if no framing match.
 
-	* Sample configs: Fix pjsip.conf syntax error.
+	  ASTERISK-28110 #close
 
-	  It is valid for a config file to be empty or contain only comments, but
-	  not valid for a config value to be set when no uncommented context
-	  exists.  This caused an error to be loged numerous times during start
-	  when loading the default pjsip.conf.
+	  Change-Id: Ic64b8fc6a140a93fbdb2f97550a40d0ff334e607
 
-	  Change-Id: Icf3b0d69b4ecb6e935eecd43c99ed8b32a5a1cf6
+2018-11-11 18:32 +0000 [05c79fdf06]  Corey Farrell <git@cfware.com>
 
-2018-08-09 02:34 +0000 [a1b0db826a]  Torrey Searle <torrey@voxbone.com>
+	* taskprocessor: Do not use separate allocation for stats or name.
 
-	* res/res_pjsip_sdp_rtp:  put rtcp-mux in answer only if offered
+	  Merge storage for the stats object and name string into the main
+	  allocation for struct ast_taskprocessor.
 
-	  If in the initial sdp the caller doesn't include the line
-	  a=rtcp-mux
+	  Change-Id: I74fe9a7f357f0e6d63152f163cf5eef6428218e1
 
-	  Then asterisk shoud not include rtcp-mux in the response regardless
-	  of rtcp-mux being enabled on the endpoint
+2018-11-11 07:34 +0000 [3cde7e30ba]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-28007 #close
+	* core: Ensure that el_end is always run when needed.
 
-	  Change-Id: I58e9b9f40a139afc0da5de41906cc608fb62adc7
+	  * Ignore console=yes configuration option in remote console processes.
+	  * Use new flag to tell consolethread to run el_end and exit when needed.
 
-2018-07-19 22:28 +0000 [f3a8bfff36]  Corey Farrell <git@cfware.com>
+	  ASTERISK-28158
 
-	* CI: Add support for coverage processing.
+	  Change-Id: I9e23b31d4211417ddc88c6bbfd83ea4c9f3e5438
 
-	  Enable coverage with `./tests/CI/buildAsterisk.sh --coverage`.  This
-	  will cause Asterisk to be compiled with coverage support.  It also
-	  initializes 'before' coverage data for all sources.  Accept
-	  --tested-only to disable modules which are not run by any test.
-	  Enabling coverage also sets tested-only true by default.  To build
-	  everything with coverage enabled use `--coverage --tested-only=0`.
+2018-11-08 15:37 +0000 [1c008c6de5]  Corey Farrell <git@cfware.com>
 
-	  ./tests/CI/processCoverage.sh is used to process the coverage and
-	  generate HTML reports.
+	* jansson-bundled: Patch for off-nominal crash.
 
-	  Fix utils/check_expr2 which failed to compiled with coverage enabled.
+	  pack_string crashed on non-NULL strings returned when s->has_error was
+	  true if the string was the result of 's' format without '#', '%' or '+'.
 
-	  Add status output 5 times per stage of astobj2_test_perf to ensure
-	  remote CLI does not timeout when compiled with coverage.  Remote CLI
-	  disconnects if no output is received for 60 seconds.  When coverage is
-	  enabled it takes about 70 seconds for my laptop to run the stages of
-	  this test, so with the change a message is printed every 14 seconds.
+	  Change-Id: Ic125df691d81ba2cbc413e37bdae657b304d20d0
 
-	  Change-Id: I890f7d5665087426ad7d3e363187691b9afc2222
+2018-11-02 06:38 +0000 [07a59b783e]  Corey Farrell <git@cfware.com>
 
-2018-08-06 12:19 +0000 [b7f195b6fd]  Richard Mudgett <rmudgett@digium.com>
+	* pbx_config: Only the first [globals] section is seen.
 
-	* res_pjsip.h: Fix doxygen comments.
+	  If multiple [globals] sections are used (for example via separate
+	  included files), only the first one is processed.  This can result in
+	  lost global variables when using a modular extensions.conf.
 
-	  Change-Id: I9cf97bdc756012d1f552ab007f4aa85e0ddb4e62
+	  ASTERISK-28146 #close
 
-2018-08-06 06:36 +0000 [856b6d1954]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Iaac810c0a7c4d9b1bf8989fcc041cdb910ef08a0
 
-	* stasis: Reduce calculation of stasis message type hash.
+2018-11-06 16:44 +0000 [9c9bc5acca]  Chris-Savinovich <csavinovich@digium.com>
 
-	  When the stasis cache is used a hash is calculated for
-	  retrieving or inserting messages. This change calculates
-	  a hash when the message type is initialized that is then
-	  used each time needed. This ensures that the hash is
-	  calculated only once for the message type.
+	* res_pjsip: Send a 503 response when overload state if reliable transport.
 
-	  Change-Id: I4fe6bfdafb55bf5c322dd313fbd8c32cce73ef37
+	  When Asterisk's taskprocessors get overloaded we need to reduce the work
+	  load. res_pjsip currently ignores new SIP requests and relies on SIP
+	  retransmissions in the hope that the overload condition will clear soon
+	  enough to handle the retransmitted SIP request.
+	  This change adds the following code after ast_taskprocessor_alert_get()
+	  has returned TRUE:
+	  1- identifies transport type. If non-udp then send a 503 response
+	  2- if transport type is udp/udp6 then ignore, as before.
 
-2018-07-30 07:49 +0000 [819842f7fb]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I1c230b40d43a254ea0f226b7acf9ee480a5d3836
 
-	* pjproject_bundled: Fix for Solaris builds. Do not undef s_addr.
+2018-11-06 16:35 +0000 [03efafbd4d]  Kevin Harwell <kharwell@digium.com>
 
-	  The authors of PJProject undef s_addr because of some issue in Microsoft
-	  Windows. However in Oracle Solaris, s_addr is not a structure member, but
-	  defined to map to the real structure member.
+	* res_pjsip: formatting error in documentation
 
-	  Updates the patch from ASTERISK_20366
+	  The use of a '|' in the "global/debug" synopsis documentation caused the
+	  generated html table on the wiki to add an extra column that included the
+	  text after the pipe.
 
-	  ASTERISK-27997
+	  This patch replaces the pipe with a comma.
 
-	  Change-Id: I8223026d4d54e2a46521085fcc94bfa6ebe35b11
+	  ASTERISK-28150
 
-2018-08-03 15:59 +0000 [1f97ea7e2c]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I3d79a6ca6d733d9cb290e779438114884b98a719
 
-	* res_pjsip: Make pjlib.h consistently included.
+2018-11-05 12:44 +0000 [3e3f3bfb07]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  * Don't include pjlib.h twice in res_pjsip.h
-	  * Consistently use #include <> form for pjproject includes.
-	  (pjsip.h and pjlib.h)
+	* res_pjsip.c: Make taskprocessor scheduling algorithm pick the shortest queue
 
-	  Change-Id: I3f7b42044840de64edf7e9d7695cb60c45990dc7
+	  The current round-robin method does not take the current taskprocessor
+	  load into consideration when distributing requests.  Using the least-size
+	  method the request goes to the taskprocessor that is servicing the least
+	  number of active tasks at the current time.
 
-2018-07-30 06:05 +0000 [0d880fac93]  Alexander Traud <pabstraud@compuserve.com>
+	  Longer running tasks with the round-robin method can delay processing
+	  tasks.
 
-	* pjproject_bundled: Find shared libraries in root --with-ssl=PATH.
+	  * Change the algorithm from round-robin to least-size for picking the
+	  PJSIP taskprocessor from the default serializer pool.
 
-	  The script configure from Teluu expects shared libraries (.so) in a subfolder
-	  called 'lib', when --with-xyz=PATH is specified. However for OpenSSL, the
-	  default location is the root of the source folder = PATH. Furthermore, Asterisk
-	  supports both, 'lib' and root. For consistency and because Asterisk is using
-	  (only) OpenSSL in PJProject, it is enhanced to support both locations, just
-	  like Asterisk.
+	  Change-Id: I7b8d8cc2c2490494f579374b6af0a4868e3a37cd
 
-	  ASTERISK-27995
+2018-11-05 08:30 +0000 [27897a850d]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I8eb916a88b6b8c22e29bb40bee8faaca6c73406f
+	* stasis: Clarify lifetime of topics.
 
-2018-08-01 09:45 +0000 [a5ce9b6c4b]  Joshua Colp <jcolp@digium.com>
+	  As mentioned in the comment I've added in the code there is no
+	  ability to unsubscribe all subscribers from a topic and explicitly
+	  destroy it. This is not currently a problem as we have two types of
+	  topics:
 
-	* res_pjsip_registrar: Improve performance on inbound handling.
+	  Long lived topics which exist for the lifetime of the system.
+	  Ephemeral topics which feed a long lived topic.
 
-	  This change removes a sorcery lookup for retrieving all
-	  contacts at the end of the registration process by keeping
-	  track of the contacts that are added/updated/deleted.
+	  In the case of the ephemeral topics there is no subscriber which does
+	  not have its lifetime managed by the same entity that has created
+	  the topic. This ensures that when the topic is being unreferenced the
+	  subscribers are also unsubscribed and destroyed, allowing the topic
+	  to ultimately be destroyed as well.
 
-	  This ensures at the end of the process the container of
-	  contacts we have is the current state.
+	  Change-Id: Ic5e244da7b16b1895ba1fc5ece481ebba5809c9a
 
-	  Pool usage has also been reduced by allocating one for
-	  usage throughout the handling of a REGISTER and resetting
-	  it to a clean state. This ensures that in most cases
-	  we allocate once and just reuse it.
+2018-10-09 07:44 +0000 [7f6ac5dc2f]  Jasper Hafkenscheid <jasper.hafkenscheid@wearespindle.com>
 
-	  ASTERISK-28001
+	* chan_sip:  Attempt ast_do_pickup in handle_invite_replaces
 
-	  Change-Id: I1a78b2d46f9a2045dbbff1a3fd6dba84b612b3cb
+	  When a call pickup is performed using and invite with replaces header
+	  the ast_do_pickup method is attempted and a PICKUP stasis message is sent.
 
-2018-08-02 14:37 +0000 [4aa91c6f11]  Salah Ahmed <txrubel@gmail.com>
+	  ASTERISK-28081 #close
+	  Reported-by: Luit van Drongelen
 
-	* dialplan_functions: wrong srtp use status report of a dialplan function
+	  Change-Id: Ieb1442027a3ce6ae55faca47bc095e53972f947a
 
-	  If asterisk offer an endpoint with SRTP and that endpoint respond
-	  with non srtp, in that case channel(rtp,secure,audio) reply wrong
-	  status.
+2018-10-26 10:53 +0000 [a6d1723727]  Pascal Cadotte Michaud <pcm@wazo.io>
 
-	  Why delete flag AST_SRTP_CRYPTO_OFFER_OK while check identical remote_key:
-	  Currently this flag has being set redundantly. In either case identical
-	  or different remote_key this flag has being set. So if we
-	  don't set it while we receive identical remote_key or non SRTP SDP
-	  response then we can take decision of srtp use by using that flag.
+	* contrib/sip_to_pjsip: add a --quiet option to avoid prints
 
-	  ASTERISK-27999
+	  Using the --quiet or -q option in conjonction with /dev/stdout as the output
+	  file allow the output to be used as a valid configuration.
 
-	  Change-Id: I29dc2843cf4e5ae2604301cb4ff258f1822dc2d7
+	  Given a script that generates a valid sip.conf I can pipe the output of that
+	  script into `sip_to_pjsip.py -q /dev/stdin /dev/stdout`. This allow me to use
+	  that piped command in my pjsip.conf using the `exec` command.
 
-2018-07-17 07:13 +0000 [ff94fcb6f1]  Torrey Searle <torrey@voxbone.com>
+	  ASTERISK-28136
 
-	* thirdparty/pjproject: fix deadlock in response retransmissions
+	  Change-Id: I7b0e2e90e2549f3f8e01dc96701f111b5874c88d
 
-	  The tdata containing the response can be shared by both the dialog
-	  object and the tsx object.  In order to prevent the race condition
-	  between the tsx retransmission and the dialog sending a response,
-	  clone the tdata before modifying it for the dialog send response.
+2018-10-31 07:53 +0000 [9c5e75acb0]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-27966 #close
+	* res_pjsip: Add XML documentation for "use_callerid_contact"
 
-	  Change-Id: Ic381004a3a212fe1d8eca0e707fe09dba4a6ab4e
+	  ASTERISK-28087
 
-2018-07-31 11:24 +0000 [75131c9e1c]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I69d48813ec514f5ef06c6de994cba52630e0a3b4
 
-	* pjsip_wizard.conf.sample: Update remote_hosts description.
+2018-10-30 10:52 +0000 [719daf3299]  Richard Mudgett <rmudgett@digium.com>
 
-	  Remove the note that SRV records are not supported as that is no longer
-	  true.
+	* alembic: Fix use_callerid_contact option add script.
 
-	  ASTERISK-27993
+	  ASTERISK-28087
 
-	  Change-Id: Id0dd6ef40e52702be9727a2b6122216cb00bb4ca
+	  Change-Id: I046d018015427d0916fab571b5a4f5367476f729
 
-2018-07-27 13:23 +0000 [7f203d2bd8]  George Joseph <gjoseph@digium.com>
+2018-10-12 16:46 +0000 [5cbe77cc46]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* CI: Add optional uninstall step before installing asterisk
+	* pjsip: new endpoint's options to control Connected Line updates
 
-	  Change-Id: I7dedf1e925eafc3a0adf01dd9dfbe44eb642aab7
+	  This patch adds new options 'trust_connected_line' and 'send_connected_line'
+	  to the endpoint.
 
-2018-07-28 11:49 +0000 [4aa1a42159]  Alexander Traud <pabstraud@compuserve.com>
+	  The option 'trust_connected_line' is to control if connected line updates
+	  are accepted from this endpoint.
 
-	* BuildSystem: Enable ncurses for menuselect in Solaris 11.
+	  The option 'send_connected_line' is to control if connected line updates
+	  can be sent to this endpoint.
 
-	  The check for the library ncurses should use not use the header <curses.h> but
-	  <ncurses.h>, because on some platforms <curses.h> is not a drop-in replacement
-	  for <ncurses.h>: For example in Solaris, the symbol initscr is a typedef in
-	  <curses.h> to a symbol which does not exist in the library ncurses (initscr32).
-	  Simply use <ncurses.h> when you link to ncurses.
+	  The default value is 'yes' for both options.
 
-	  Furthermore in Solaris, the header <ncurses.h> is in a subdirectory
-	  /usr/include/ncurses and not available via pkg-config.
+	  Change-Id: I16af967815efd904597ec2f033337e4333d097cd
 
-	  ASTERISK-15331
-	  ASTERISK-14935
-	  ASTERISK-12382
-	  ASTERISK-9107
+2018-10-26 16:18 +0000 [fbee505611]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: Ife367776b0ccf17d3fefed868245376bfb93745d
+	* res_pjsip_notify: improve realtime performance on CLI completion on the endpoint
 
-2018-07-28 08:08 +0000 [d9da161b5c]  Alexander Traud <pabstraud@compuserve.com>
+	  The module 'res_pjsip_notify' inefficiently makes a lot of DB requests
+	  on CLI completion on the endpoint.
 
-	* BuildSystem: Enable Jansson in Solaris 11.
+	  For example if there are 10k endpoints the module makes 10k requests
+	  of these 10k records.
 
-	  In Solaris, the header <jansson.h> is in /usr/include/jansson. To find
-	  Jansson even in such a subdirectory, the tool pkg-config is queried via
-	  AST_PKG_CONFIG_CHECK. For those platforms, which do not list Jansson via
-	  pkg-config, the previous check remains and is executed thereafter.
+	  Even if a part of the endpoint entered
+	  the module makes the same 10k requests and then filtered them by itself.
 
-	  Because the check for the NetBSD Editline library uses the tool pkg-config
-	  conditionally PKG_PROG_PKG_CONFIG must be used. Because that check happens
-	  earlier than Jansson, it must be placed in front of that.
+	  This patch gathers endpoints container by prefix
+	  and adds all gathered endpoints to completion at once.
 
-	  The script configure does some pre-checks for the script configure of the
-	  Asterisk internal NetBSD Editline library. The check for the library ncurses
-	  should use not use the header <curses.h> but <ncurses.h>, because on some
-	  platforms <curses.h> is not a drop-in replacement for <ncurses.h>: For example
-	  in Solaris, the symbol initscr is a typedef in <curses.h> to a symbol which
-	  does not exist in the library ncurses (initscr32). Simply use <ncurses.h> when
-	  you link to ncurses.
+	  ASTERISK-28137 #close
 
-	  ASTERISK-27991
+	  Change-Id: Ic20024912cc77bf4d3e476c4cd853293c52b254b
 
-	  Change-Id: I69ea0f379f87a50049654b2487c76ee1c04fa53a
+2018-10-27 09:59 +0000 [6c2f9b730e]  Pascal Cadotte Michaud <pcm@wazo.io>
 
-2018-07-28 08:00 +0000 [edddcb3dc6]  Joshua Colp <jcolp@digium.com>
+	* contrib/sip_to_pjsip: handle setvar in conversion
 
-	* res_pjsip_pubsub: Use ast_true for "prune_on_boot".
+	  Given a sip.conf with the following content:
 
-	  Change-Id: Iedec4e7390b3e821987681da24d0298632b9873d
+	  setvar FOO=1
+	  setvar BAR=42
 
-2018-07-24 13:44 +0000 [8baedc2204]  Richard Mudgett <rmudgett@digium.com>
+	  I want my generated pjsip.conf to containt the following set_vars
 
-	* res_pjsip_endpoint_identifier_ip.c: Added regex support to match_header
+	  set_var FOO=1
+	  set_var BAR=42
 
-	  This patch adds regular expression support to make the identify section's
-	  match_header option more useful when attempting to match complex headers
-	  like the 'To' or 'From' headers.  The 'From' header has variable
-	  components such as the tag parameter that you cannot predict.  To specify
-	  a regular expression put slashes around the regular expression in place of
-	  the header value.
+	  in the matching endpoint section.
 
-	  [identify-alice]
-	  type=identify
-	  endpoint=alice
-	  match_header=From: /<sip:alice@127\\.0\\.0\\.1>/
+	  Change-Id: I6c822401fda4133c3b44bf31e655b4eb939d4d26
 
-	  * Added regex support to match_header so you could match a 'To' header
-	  among other complex headers.
+2018-10-02 07:31 +0000 [3ba66b8a9d]  Torrey Searle <torrey@voxbone.com>
 
-	  Fixed reported crashes when trying to match special headers like 'Contact'.
-	  The identify section's match_header method used code that assumed you were
-	  matching a generic header.  Any other type of header could cause a crash
-	  if the header structure variant did not match the generic header enough.
+	* res_pjsip_session: add new flag use_callerid_contact
 
-	  * Made use code that will work for any header type instead of code
-	  specific to generic headers.
+	  Add a new global flag to res_pjsip to allow the callerid to be used
+	  as the username in the contact header.  This allows chan_pjsip to have
+	  the same behavour as chan_sip
 
-	  Other fixes while in the area:
+	  ASTERISK-28087 #close
 
-	  * Made check all headers of the requested name.
-	  * Added some more sanity checks to the configured identify matching
-	  options when applying the configuration.
+	  Change-Id: I9a720e058323f6862a91c62f8a8c1a4b5c087b95
 
-	  ASTERISK-27548
+2018-10-17 19:34 +0000 [fbb577577c]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I27dfd4ff5e2259b906640e3c330681b76b4ed1f1
+	* logger.c: Fix default console logging when no logger.conf available.
 
-2018-07-27 10:46 +0000 [d33ed08afc]  Joshua Colp <jcolp@digium.com>
+	  Default logging was not setup correctly when there was no logger.conf.
+	  This resulted in many expected log messages not actually getting out to
+	  the console.
 
-	* res_pjsip_pubsub: Treat "prune_on_boot" as a yes / no.
+	  Change-Id: I542e61c03b2f630ff5327f9de5641d776c6fa70c
 
-	  The alembic for the PJSIP subscription persistence table has the
-	  "prune_on_boot" field as a boolean. While in Asterisk we are
-	  tolerant of many different definitions of true and false in the
-	  database we only accept "yes" and "no". This change makes the
-	  field treated as a yes/no instead of an integer, thus storing
-	  "yes" and "no" instead of "1" and "0".
+2018-09-26 15:05 +0000 [190b3d7412]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: Ic8b9211b36babefe78f70def6828a135a6ae7ab6
+	* app_dial/queue/followme: 'I' options to block initial updates in both directions
 
-2018-07-27 08:54 +0000 [a4b105c99b]  Alexander Traud <pabstraud@compuserve.com>
+	  The 'I' option currently blocks initial CONNECTEDLINE or REDIRECTING updates
+	  from the called parties to the caller.
 
-	* res_rtp_asterisk: In Developer Mode, do not require OpenSSL.
+	  This patch also blocks updates in the other direction before call is
+	  answered.
 
-	  OpenSSL is an optional external library and should stay optional even when
-	  Developer Mode is configured.
+	  ASTERISK-27980
 
-	  ASTERISK-27990
+	  Change-Id: I6ce9e151a2220ce9e95aa66666933cfb9e2a4a01
 
-	  Change-Id: Ia68a4cd5474b26d45e0f43b04032ad598022853b
+2018-10-22 14:31 +0000 [11427a9665]  Richard Mudgett <rmudgett@digium.com>
 
-2018-07-26 18:54 +0000 [2b0d003a35]  George Joseph <gjoseph@digium.com>
+	* modules.conf.sample: Update preload usage documentation.
 
-	* CI: Fix placement of job summary statments
+	  Change-Id: Id449d4435c38148b56ac4cfd61ae4d90ac66bb90
 
-	  Change-Id: Iace19e718f4e8fb48eb7dc9f98af53b115cc45f3
+2018-10-16 07:02 +0000 [fec66b8f01]  George Joseph <gjoseph@digium.com>
 
-2018-07-26 11:34 +0000 [5fbf2ce248]  George Joseph <gjoseph@digium.com>
+	* bridge_softmix:  Add SDP "label" attribute to streams
 
-	* CI:  Add docker info to job summary
+	  Adding the "label" attribute used for participant info correlation
+	  was previously done in app_confbridge but it wasn't working
+	  correctly because it didn't have knowledge about which video
+	  streams belonged to which channel.  Only bridge_softmix has that
+	  data so now it's set when the bridge topology is changed.
 
-	  Change-Id: I45d52005a9b692ad303c11792f226ace1e449901
+	  ASTERISK-28107
 
-2018-07-23 13:49 +0000 [eacf8fe96c]  Corey Farrell <git@cfware.com>
+	  Change-Id: Ieddeca5799d710cad083af3fcc3e677fa2a2a499
 
-	* Build System: Create 'make install-configs' target.
+2018-10-16 14:06 +0000 [dee1165d31]  Corey Farrell <git@cfware.com>
 
-	  This target requires specifying CONFIG_SRC=path_to_configs.  This can be
-	  used to install custom configs for the Asterisk build while still
-	  performing directory replacements on asterisk.conf.
+	* astobj2: Eliminate usage of legacy container allocation macros.
 
-	  Modify internal INSTALL_CONFIGS so first argument requires full path to
-	  the config sources relative to Asterisk source root.
+	  These macros have been documented as legacy for a long time but are
+	  still used in new code because they exist.  Remove all references to:
+	  * ao2_container_alloc_options
+	  * ao2_t_container_alloc_options
+	  * ao2_t_container_alloc
 
-	  Change-Id: Idcd841df3c8d5bfe23d566bb9e2e448e9df4f8ab
+	  These macro's are still available for use but only in modules.  Only
+	  ao2_container_alloc remains due to it's use in over 100 places.
 
-2018-07-25 15:33 +0000 [059e1e3bc8]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: I1a26258b5bf3deb081aaeed11a0baa175c933c7a
 
-	* json.c: improve ast_json_to_ast_variables performance
+2018-09-28 13:31 +0000 [90887af808]  Corey Farrell <git@cfware.com>
 
-	  When converting from a json object to an ast variables list the conversion
-	  algorithm was doing a complete traversal of the entire variables list for
-	  every item appended from the json structure.
+	* lock: Replace __ast_mutex_logger with private log_mutex_error.
 
-	  This patch makes it so the list is no longer traversed for each new ast
-	  variable being appended.
+	  __ast_mutex_logger used the variable `canlog` without accepting it as a
+	  argument.  Replace with internal macro `log_mutex_error` which takes
+	  canlog as the first arguement.  This will prevent confusion when working
+	  with lock.c code, many of the function declare the canlog variable and
+	  in some cases it previously appeared to be unused.
 
-	  Change-Id: I8bf496a1fc449485150d6db36bfc0354934a3977
+	  Change-Id: I83b372cb0654c5c18eadc512f65a57fa6c2e9853
 
-2018-07-25 05:32 +0000 [dd1b4b38cf]  Joshua Colp <jcolp@digium.com>
+2018-10-18 14:36 +0000 [00284352b3]  Richard Mudgett <rmudgett@digium.com>
 
-	* devicestate: Don't create topic when change isn't cached.
+	* app_dial/app_queue: Update application option documentation
 
-	  When publishing a device state the change can be marked as being
-	  cachable or not. If it is not cached the change is just published
-	  to all interested and not stored away for later query. This was not
-	  fully taken into account when publishing in stasis. The act of
-	  publishing would create a topic for the device even if it may be
-	  ephemeral.
+	  * Update the post-answer documentation and example.  The Dial example was
+	  incorrect and misleading for the post-answer subroutine useage.
 
-	  This change makes it so messages which are not cached won't create
-	  a topic for the device. If a topic does already exist it will be
-	  published to but otherwise the change will only be published to
-	  the device state all topic.
+	  * Fix note and warning paragraphs in option descriptions.  They don't show
+	  up in the wiki.
 
-	  ASTERISK-27591
+	  Change-Id: I81019a1fd75d5b9151f76b52c38e2a90da682d14
 
-	  Change-Id: I18da0e8cbb18e79602e731020c46ba4101e59f0a
+2018-10-18 14:56 +0000 [495a1e7e5c]  Sean Bright <sean.bright@gmail.com>
 
-2018-07-25 10:20 +0000 [21168ba931]  George Joseph <gjoseph@digium.com>
+	* samples: PARKINGSLOT -> PARKING_SPACE in parking sample config
 
-	* CI: Explicitly pass BRANCH_NAME to buildAsterisk and installAsterisk
+	  PARKINGSLOT was deprecated in Asterisk 12 but the sample config was not
+	  updated to reflect that.
 
-	  Change-Id: I652f4a0ea5107c778e27a78bccb67b18b0c4e087
+	  Change-Id: I3e087c19d9ee587094fa5304102d8084a79c2b3c
 
-2018-07-24 13:29 +0000 [9b4fb9d1aa]  George Joseph <gjoseph@digium.com>
+2018-10-18 12:32 +0000 [2384d6eb87]  Richard Mudgett <rmudgett@digium.com>
 
-	* CI: Add options to initialize and cleanup database to runTestsuite.sh
+	* Fix 'statement' typo throughout code.
 
-	  Change-Id: I352333233bab5377723bf37d490ba84fc55bc853
+	  Most were in comments.  A couple were in warning messages.
 
-2018-07-25 09:07 +0000 [90325f0569]  Corey Farrell <git@cfware.com>
+	  Pointed out by Jonathan H on the Asterisk users mailing list.
 
-	* CI: Do not `mkdir 2`.
+	  Change-Id: I6286939dff5d0a27a2758140570106f1cb351855
 
-	  Change-Id: Ib7377d26a6c98b38bad463f47c84f1875ac84eb7
+2018-10-17 16:08 +0000 [718ccd51a3]  Richard Mudgett <rmudgett@digium.com>
 
-2018-07-25 07:34 +0000 [3ed6abcc1f]  Corey Farrell <git@cfware.com>
+	* res_rtp_asterisk.c: Add conditional module dependency to res_pjproject
 
-	* Build System: Silence build of bundled jansson.
+	  * The dependency ensures that res_pjproject cannot be manually unloaded
+	  before res_rtp_asterisk.
+	  * The dependency allows startup loading errors to report that
+	  res_rtp_asterisk depends upon res_pjproject.
 
-	  Change-Id: I7392c79c0173057f5378010bf1fe65e300e8fc56
+	  Change-Id: Icf5e7581f4ddd6189929f6174c74dd951f887377
 
-2018-07-25 07:13 +0000 [b971faf740]  George Joseph <gjoseph@digium.com>
+2018-10-17 14:34 +0000 [7687630074]  Richard Mudgett <rmudgett@digium.com>
 
-	* CI: RefDebug: Fix reference to testsuite URL
+	* modules: Add missing run time module support levels.
 
-	  Change-Id: I0ee41d95a87f0d97b01f2757012b846bcfe6443d
+	  Change-Id: I29b9dbfa4bbfc49f21eba356858e38b1d3041824
 
-2018-07-24 14:28 +0000 [ab103d66db]  Corey Farrell <git@cfware.com>
+2018-10-14 07:58 +0000 [abee4fd072]  Corey Farrell <git@cfware.com>
 
-	* Build System: Fix bundled jansson install.
+	* taskprocessor: Warn on unused result from pushing task.
 
-	  Update the bundled jansson Makefile to do nothing during Asterisk
-	  install, use a target that is not phony to initiate the jansson make and
-	  install.
+	  Add attribute_warn_unused_result to ast_taskprocessor_push,
+	  ast_taskprocessor_push_local and ast_threadpool_push.  This will help
+	  ensure we perform the necessary cleanup upon failure.
 
-	  Change-Id: I7643cc3d39af9feba8fc0da676b646efc5f8b3bb
+	  Change-Id: I7e4079bd7b21cfe52fb431ea79e41314520c3f6d
 
-2018-07-24 04:57 +0000 [fd1b8c57e9]  Florian Floimair <f.floimair@commend.com>
+2018-10-16 12:28 +0000 [3fe9c41342]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip: Change log message from error to warning for valid use cases
+	* bundled pjproject: Remove timer cleanup usage patch.
 
-	  If a SIP MESSAGE is triggered for an endpoint that is currently not registered
-	  - and therefore has no valid contact associated - an error message was logged.
-	  Since this is a valid request in a valid use cases this is now changed to a
-	  warning, as discussed with Matt Fredrickson on the asterisk-dev mailing list.
+	  This patch is not in the upstream pjproject and does unsafe things with
+	  the timer->_timer_id and timer->_grp_lock values in pj_timer_entry_reset()
+	  outside of the timer heap lock.  pj_timer_entry_reset() is also called for
+	  timers that are not about to be rescheduled in a few places.
 
-	  Change-Id: I55eb62d2712818a58c7532119dec288bd98cf0c0
+	  Change-Id: I4fe0b4bc648f7be5903cf4531b94fc87275713c1
 
-2018-07-24 05:39 +0000 [eccd5886c3]  George Joseph <gjoseph@digium.com>
+2018-10-10 04:37 +0000 [35f44be460]  Corey Farrell <git@cfware.com>
 
-	* CI:  Add --privileged flag to docker options
+	* refdebug: Create refstats.py script.
 
-	  Change-Id: If92d55f15306e55dd7091ac3c47b13ebbbb03488
+	  This allows us to process AO2 statistics for total objects, memory
+	  usage, memory overhead and lock usage.
 
-2018-07-24 05:22 +0000 [32ae565da2]  George Joseph <gjoseph@digium.com>
+	  * Install refstats.py and reflocks.py into the Asterisk scripts folder.
+	  * Enable support for reflocks.py without DEBUG_THREADS.
 
-	* CI: Set correct user:group when publishing docs
+	  Steal a bit from the ao2 magic to flag when an object lock is used.
+	  Remove 'lockobj' from reflocks.py since we can now record 'used' or
+	  'unused' for those objects.
 
-	  Change-Id: Ibabeb9ac730d9755cf54318d0da74771c939b86b
+	  Add comments to explain thread safety of the 'struct __priv_data'
+	  bitfields.
 
-2018-07-23 10:23 +0000 [46ea59a0ca]  Corey Farrell <git@cfware.com>
+	  Change-Id: I84e9d679cc86d772cc97c888d9d856a17e0d3a4a
 
-	* CI: Split --test-command argument.
+2018-10-12 12:14 +0000 [0f53930c05]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  The --test-command argument has now been split, unit tests now use
-	  `--unittest-command` and the testsuite uses --testsuite-command.
+	* res_pjsip: set callerid_tag to empty string
 
-	  This will make it easier to create a script which run everything by
-	  forwarding the same arguments to all CI scripts.
+	  This patch sets the callerid_tag to empty string by default.
 
-	  Change-Id: Ia54aa4848eaffbdf13175fcda40fc0b23080ad71
+	  If the callerid_tag is set to NULL then the tag does not
+	  become part of a connected line update.
+	  For example:
+	  Alice's tag is "Alice".
+	  Bob's tag is empty.
+	  Charlie's tag is "Charlie".
+	  Alice calls Bob and then does attended transfer to Charlie.
+	  When Alice hangs up the CONNECTEDLINE(tag) is "Alice"
+	  on the interception routine on the Charlie's channel, but should be empty.
 
-2018-07-21 11:58 +0000 [f34c59e745]  Corey Farrell <git@cfware.com>
+	  Ths patch also fix memory leaks if there are more then one options
+	  "callerid", "callerid_tag", "voicemail_extension" and "contact_user"
+	  in the pjsip.conf endpoint definition.
 
-	* CI: Fix mkdir CACHE_DIR.
+	  Change-Id: I86ba455c4677ca8d516d9a04ce7fb4d24dd576e4
 
-	  Change-Id: Ic9f9a61e230047836c836206731f8ff7eb3538c9
+2018-10-11 06:24 +0000 [08e5bbfec0]  Corey Farrell <git@cfware.com>
 
-2018-07-21 11:22 +0000 [82d351ca0e]  Corey Farrell <git@cfware.com>
+	* threadpool: Eliminate pointless AO2 usage.
 
-	* build_tools/make_version: Get MAINLINE_BRANCH from .gitreview.
+	  thread_worker_pair, set_size_data and task_pushed_data structures are
+	  allocated with AO2 objects, passed to a taskprocessor, then released.
+	  They never have multiple owners or use locking so AO2 only adds
+	  overhead.
 
-	  Use .gitreview defaultbranch setting to determine the mainline branch.
-	  This allows the script to be used against other directories which might
-	  not be on the same defaultbranch.  This can be used by CI scripts to
-	  report the testsuite version being used:
-	  ./build_tools/make_version ${TESTSUITE_DIR}
+	  Change-Id: I2204d2615d9d952670fcb48e0a9c0dd1a6ba5036
 
-	  Change-Id: Ifdad4a9d8a26138c41bc6b630ecc3e34ea1c2758
+2018-10-12 12:21 +0000 [a5023d4f75]  Corey Farrell <git@cfware.com>
 
-2018-07-22 10:41 +0000 [1fbe1b1eff]  Joshua Colp <jcolp@digium.com>
+	* main/astfd: Fix GCC8 format-truncation warning.
 
-	* sched: Make ABI compatible between dev mode and non-dev mode.
+	  The field used to store call arguments was not large enough to hold the
+	  arguments string that can be constructed for 'open'.  Expand it to
+	  prevent this warning/error.
 
-	  In the past there was an assertion in the ast_sched_del function
-	  and in order to ensure it was useful the calling function name,
-	  line number, and filename had to be passed in. This cause the ABI
-	  to be different between dev mode and non-dev mode.
+	  Change-Id: I514927f256481bc84df10a51b19d5b5fb1bc387e
 
-	  This assertion is no longer present so the special logic can be
-	  removed to make it the same between them both.
+2018-10-10 06:38 +0000 [5ea517f80e]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Icbc69c801e357d7004efc5cf2ab936d9b83b6ab8
+	* chan_sip: Tell module loader that chan_sip is extended support.
 
-2018-07-16 15:08 +0000 [f36b4091f5]  Corey Farrell <git@cfware.com>
+	  Change-Id: I33508c134b1be888b8884f5dcfee19087634e415
 
-	* Enable bundling of jansson.
+2018-10-09 16:18 +0000 [c001974f4f]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ib3111b151d37cbda40768cf2a8a9c6cf6c5c7cbd
+	* res_statsd.c: Fix returned reload status.
 
-2018-07-18 09:32 +0000 [e5721b01f6]  Emmanuel BUU <emmanuel.buu@ives.fr>
+	  The return status when there was no change in statsd.conf was incorrect.
+	  This resulted in the wrong status message on the CLI when reloading the
+	  module.
 
-	* res_rtp_asterisk: Avoid merging command and regular T.140 text packets
+	  * Fixed cleanup on initial load if initializing statsd failed.
 
-	  When realtime text packets are to be sent, the text is accumulated in a
-	  buffer and sent regularly by a timer.  It can happen that commands such as
-	  a backspace, CR, or LF get merged with regular text.  This breaks some
-	  UAs.
+	  Change-Id: Id24fae75f1a7ff584a444a5680e867d989792481
 
-	  The proposed change:
-	  * We test if the current packet contains a command.  If so we send the
-	  buffer immediately.
-	  * We test if the buffer contained a command.  If so we send the buffer
-	  immediately.
-	  * We accumulate the text (or the command) in the buffer.
+2018-10-03 16:51 +0000 [e0496fe062]  Emmanuel BUU <emmanuel.buu@ives.fr>
 
-	  ASTERISK-27970
+	* core/frame: generate correct T.140 payload in ast_sendtext_data()
 
-	  Change-Id: Ifbe993311410fa855cb8aa4a12084db75f413462
+	  ast_sendtext_data() would create an incorrect T.140 text frame which
+	  length include the null terminator byte. It causes ultimately RTP
+	  packets to be send with this trailing 0. The proposed fix just set the
+	  correct length to the text frame
 
-2018-07-20 09:25 +0000 [fd84eb65b3]  Corey Farrell <git@cfware.com>
+	  ASTERISK-28089
+	  Reported by: Emmanuel BUU
+	  Tested by: Emmanuel BUU
 
-	* CI: Fix logger.conf for unit tests.
+	  Change-Id: I7ab1b9ed1e21683b2b667ea0a59d9aba3c77dd96
 
-	  Change-Id: Idea59d60eab20105de50b34f0f0d506e6ef55d5c
+2018-10-04 18:33 +0000 [c6ee3cf639]  Corey Farrell <git@cfware.com>
 
-2018-07-19 10:34 +0000 [2acba0f418]  George Joseph <gjoseph@digium.com>
+	* loader: Flag module as declined in all cases where it fails to load.
 
-	* CI:  Add wiki doc publish to periodics
+	  This has no effect on startup since AST_MODULE_LOAD_FAILURE aborts
+	  startup, but it's possible for this code to be returned on manual load
+	  of a module after startup.
 
-	  Change-Id: I29ba26134e5083bc6788ede235f1a5d4383c148a
+	  It is an error for a module to not have a load callback but this is not
+	  a fatal system error.  In this case flag the module as declined, return
+	  AST_MODULE_LOAD_FAILURE only if a required module is broken.
 
-2018-07-20 06:54 +0000 [18e9acc3c1]  Joshua Colp <jcolp@digium.com>
+	  Expand doxygen documentation for AST_MODULE_LOAD_*.
 
-	* res_pjsip: Update default keepalive interval to 90 seconds.
+	  Change-Id: I3c030bb917f6e5a0dfd9d91491a4661b348cabf8
 
-	  A change recently went in which disabled the built-in PJSIP
-	  keepalive. This defaulted to 90 seconds and kept TCP/TLS
-	  connections alive. Disabling this functionality has resulted
-	  in a behavior change of not doing keepalives by default resulting
-	  in TCP/TLS connections dropping for some people.
+2018-10-04 13:13 +0000 [77f151d927]  Richard Mudgett <rmudgett@digium.com>
 
-	  This change makes our default keepalive interval 90 seconds
-	  to match the previous behavior and preserve it.
+	* func_periodic_hook.c: Cleanup module resources on failure.
 
-	  ASTERISK-27978
+	  * Make load_module() cleanup if it failed to setup the module.
 
-	  Change-Id: Ibd9a45f3cbe5d9bb6d2161268696645ff781b1d6
+	  * Make unload_module() always return 0.  It is silly to fail unloading if
+	  the hook function we try to unregister was not even registered.
 
-2018-07-20 06:20 +0000 [58c7b64ed6]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I280fc6e8ba2a7ee2588ca01d870eebaf74b4ffe6
 
-	* xmldoc.c:  Fix dump of xml document
+2018-10-04 11:49 +0000 [bc335ceda8]  Richard Mudgett <rmudgett@digium.com>
 
-	  The "xmldoc dump" cli command was simply concatenating xml documents
-	  into the output file.  The resulting file had multiple "xml"
-	  processing instructions and multiple root elements which is illegal.
-	  Normally this isn't an issue because Asterisk has only 1 main xml
-	  documentation file but codec_opus has its own file so if it's
-	  downloaded and you do "xmldoc dump", the result is invalid.
+	* codec_speex.c: Cleanup module loading to DECLINE and not FAILURE.
 
-	  * Added 2 new functions to xml.c:
-	      ast_xml_copy_node_list creates a copy of a list of children.
-	      ast_xml_add_child_list adds a list to an existing list.
+	  If codec_speex fails to register a translator it would cause Asterisk to
+	  exit instead of continue as a DECLINED module.
 
-	  * Modified handle_dump_docs to create a new output document and
-	    add to it the children from each input file.  It then dumps the
-	    new document to the output file.
+	  * Make unload_module() always return 0.  It is silly to fail unloading if
+	  any translators we try to unregister were not even registered.
 
-	  Change-Id: I3f182d38c75776aee76413dadd2d489d54a85c07
+	  Change-Id: Ia262591f68333dad17673ba7104d11c88096f51a
 
-2018-07-19 16:17 +0000 [0ade9df3b6]  Richard Mudgett <rmudgett@digium.com>
+2018-10-04 13:03 +0000 [6fd21a6af6]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip: Update endpoint transport option documentation.
+	* CI: Fix missing () in gates.jenkinsfile
 
-	  Change-Id: I5394fdff6a296efc8e1695a156e616acd932ae52
+	  Change-Id: I2f252e0f8c7f1a6328438fbd2be5d6574b7dfa5b
 
-2018-07-09 04:42 +0000 [bd36ec69e2]  Torrey Searle <torrey@voxbone.com>
+2018-10-04 10:13 +0000 [1598c7a306]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_sdp_rtp: include ice in ANSWER only if offered
+	* CI: Add timestamps and timeouts to jenkinsfiles
 
-	  Keep track if ICE candidates were in the SDP offer & only put them
-	  in the corresponding SDP answer if the offer condaind ICE candidates
+	  Change-Id: Ide83574dc957bc1df28e30a69079140050dfc35f
 
-	  ASTERISK-27957 #close
+2018-10-03 17:02 +0000 [b2f96fdcfe]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Idf2597ee48e9a287e07aa4030bfa705430a13a92
+	* ast_coredumper: Remove .gdbinit file on exit
 
-2018-05-29 09:31 +0000 [6cf1aefff8]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I1297de78628773ca368e687c6f148bf74857cae9
 
-	* pjproject_bundled: Repair ./configure --with-ssl=PATH.
+2018-10-03 09:33 +0000 [d0a1237c19]  Sean Bright <sean.bright@gmail.com>
 
-	  Previously, Asterisk did not tell its bundled PJProject about this configure
-	  parameter. Therefore, PJProject used the platform provided OpenSSL always.
+	* CI: Look up configured kernel.core_pattern sysctl
 
-	  ASTERISK-27880
+	  Change-Id: I8246a0147df8d821fbbcabc1db1887104b8bedc4
 
-	  Change-Id: Iea545aec854dd0e2c061c69bb118a76ce56c5dc6
+2018-10-03 15:51 +0000 [48bb4543d9]  Corey Farrell <git@cfware.com>
 
-2018-07-18 11:34 +0000 [ae5ba776ae]  Chris-Savinovich <csavinovich@digium.com>
+	* jenkins: Fix cleanup command redirection.
 
-	* stasis: Improve message type "Use of before/init after destruction"
+	  Fix redirection to /dev/null of cleanup commands.  The '2' was being
+	  interpreted as part of the command instead of part of the redirect.
 
-	  Fixes issue where error msg
-	  "Use of before/init after destruction"
-	  was being printed on disabled messages
-	  in dev mode.  With this
-	  fix if message is disabled
-	  a warning will print.
+	  Change-Id: I2e3a591b165e0288c4b82b9ef475fdfd5392a90a
 
-	  ASTERISK-25548
-	  Change-Id: Ie0d866d1cbc60c16dbef08bc65e99505c3c1adfa
+2018-10-03 15:29 +0000 [d027d193b8]  George Joseph <gjoseph@digium.com>
 
-2018-07-18 11:12 +0000 [03c1bbffd2]  Joshua Colp <jcolp@digium.com>
+	* ast_coredumper: Don't use "declare -n"
 
-	* res_sorcery_config: Allow configuration section to be used based on name.
+	  Change-Id: I7ddfed4cd6549a0cd458e4d5cf9ac95d784de6cb
 
-	  A problem I've seen countless times is a global or system section
-	  for PJSIP not getting applied. This is inevitably the result of
-	  the "type=" line missing. This change alleviates that problem.
+2018-10-02 16:15 +0000 [7eda6263c2]  Richard Mudgett <rmudgett@digium.com>
 
-	  The ability to specify an explicit section name has been
-	  added to res_sorcery_config. If the configured section
-	  name matches this and there are no unknown things configured
-	  the section is taken as being for the given type.
+	* res_smdi.c: Fix module ref counting and inverted test.
 
-	  Both the PJSIP "global" and "system" types now support this
-	  so you can just name your section "global" or "system" and it
-	  will be matched and used, even without a "type=" line.
+	  I think this module is so screwed up that it doesn't work anymore.  Even
+	  with these attempts to fix things it still won't gracefully shut down.
+	  The module refs will not go to zero to allow unloading the module.
 
-	  ASTERISK-27972
+	  * Fix module ref counting dealing with the SMDI interface object.  There
+	  were several off-nominal paths that unbalanced the module ref count.  Also
+	  the destructor freed the ao2 object itself which is bad.  Made the
+	  smdi_read thread not hold its own ref to the SMDI interface object so when
+	  all refs go away the destructor will stop the listener thread.
 
-	  Change-Id: Ie22723663c1ddd24f869af8c9b4c1b59e2476893
+	  * Fixed the smdi_load() return code of 1 concerning the number of
+	  listeners.  The test was inverted.
 
-2018-07-17 13:56 +0000 [6a0847cfc6]  Nick French <naf@ou.edu>
+	  Change-Id: Ic288db51b58e395d6a2fc3847f77176c16988784
 
-	* SRTP: Lower SDES key lifetime minimum to 2^20
+2018-10-02 16:23 +0000 [5b72bb0278]  Richard Mudgett <rmudgett@digium.com>
 
-	  SRTP SDES key lifetime support was added in ASTERISK_17899.
+	* res_smdi.c: Made use defaults if the smdi.conf file does not exist.
 
-	  In that addition, the minimum key lifetime to be accepted was
-	  set at the 10 hours @ 20ms/packet = 1800000 packets.
+	  This module is an optional dependency of a couple of other modules.  If it
+	  declines to load, it then forces other modules that can optionally use
+	  this module to also decline.
 
-	  The firmware in the obi1xx ATA uses a hardcoded lifetime of
-	  2^20 packets.
+	  * Made use the default configuration if the config file does not exist and
+	  simplified some of the logic.
 
-	  Lower the limit to 2^20 to support a wider field of clients.
+	  Change-Id: Ib93191f1fe28c0dd9ebe3d84c7762b32f83c4eb9
 
-	  ASTERISK-27967 #close
+2018-10-02 17:15 +0000 [fa9cd68589]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I81a0703c595a0c9101dfdf02300149a3cc39bf94
+	* astobj2: Comment on OBJ_NOLOCK in ao2_container_clone.
 
-2018-07-17 11:09 +0000 [f8fa7182ac]  George Joseph <gjoseph@digium.com>
+	  The test for OBJ_NOLOCK looks wrong but it isn't.  Add comments to
+	  prevent confusion.
 
-	* CI: Fix merge strategy
+	  Change-Id: I9662b82eb39e7627a1f1944fd18f967a2b987344
 
-	  Change-Id: I5e3fb6adfa6cbf694c0deecf02e3879297b0c12e
+2018-10-03 09:05 +0000 [8b3b504b79]  Sean Bright <sean.bright@gmail.com>
 
-2018-07-17 10:41 +0000 [ca651c8973]  George Joseph <gjoseph@digium.com>
+	* CI: Use brace expansion instead of calling out to seq
 
-	* CI: Fix regex in daily and ref_debug jobs
+	  Also make the shebang in publishAsteriskDocs.sh the first line.
 
-	  Change-Id: Icf2e67818b2155a158d2390b138613e1f653ea92
+	  Change-Id: I3fdd6f22e652e4fb5b5fe85df46fa34eb6d0cf08
 
-2018-07-17 09:09 +0000 [47572a5831]  Nick French <naf@ou.edu>
+2018-10-03 08:59 +0000 [5f7350d68b]  Sean Bright <sean.bright@gmail.com>
 
-	* res_pjsip:  Remove spurious error logging when printing silent headers
+	* CI: Use bindport instead of port in test http.conf
 
-	  Asterisk patched the pjproject source to avoid crashing when pjproject
-	  sip_msg headers are encountered with NULL vptr's, but the patch also
-	  output error messages for some valid headers which simply did not need
-	  to be added to the message itself, such as hidden route headers.
+	  Change-Id: Ife9a6879da63a56e5b8348a2024eeed4e7b1615b
 
-	  pjproject has since applied a similar patch to their baseline to avoid
-	  crashes, but their version also avoids the spurious error logging.
+2018-10-03 07:56 +0000 [2b4752911f]  Sean Bright <sean.bright@gmail.com>
 
-	  Lets use their patch instead.
+	* http.c: Reload TLS even if http.conf hasn't changed
 
-	  ASTERISK-27961 #close
+	  There is currently no way to indicate to Asterisk that TLS certificates
+	  and/or keys have been updated other than by modifying http.conf or
+	  restarting Asterisk.
 
-	  Change-Id: I2ddbd82c8da10e0dcc9807a48089d1f3c2d6e389
+	  There is already code in main/tcptls.c that determines if a reload is
+	  actually necessary based on the hashes of the certicate and dependent
+	  files, so this change merely gives us a way to request a reload without
+	  explicitly modifying http.conf.
 
-2018-07-17 10:18 +0000 [33668ab46a]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie795420dcc7eb3d91336820688a29adbcc321276
 
-	* res_config_ldap.c: Fix mem leak in CLI "realtime show ldap status"
+2018-10-01 22:12 +0000 [3b2310332d]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ib2a3622b297b0363c62d23958f7a20039bcb9b4c
+	* core: Disable astobj2 locking for some common objects.
 
-2018-07-17 10:15 +0000 [081f689bc5]  George Joseph <gjoseph@digium.com>
+	  * ACO options
+	  * Indications
+	  * Module loader ref_debug object
+	  * Media index info and variants
+	  * xmldoc items
 
-	* CI: Add pre-build merge back in as RECURSIVE
+	  These allocation locations were identified using reflocks.py on the
+	  master branch.
 
-	  Change-Id: I0ff1730ef4a4f0ac9f18ccc9bc0dfe7a782f57a8
+	  Change-Id: Ie999b9941760be3d1946cdb6e30cb85fd97504d8
 
-2018-07-17 09:01 +0000 [694503d378]  George Joseph <gjoseph@digium.com>
+2018-10-02 13:29 +0000 [c2e6efa63d]  Richard Mudgett <rmudgett@digium.com>
 
-	* CI: Remove pre-build merge from gates and checks
+	* res_statsd.c: Made use defaults if the statsd.conf file does not exist.
 
-	  Change-Id: Ibc151f63dcec4db847915c2f3cbe5b467dd59574
+	  This module is an optional dependency of many modules.  If it declines to
+	  load it then forces other modules that can optionally use this module to
+	  also decline.
 
-2018-07-17 07:13 +0000 [65cf0001fe]  George Joseph <gjoseph@digium.com>
+	  * Made use default configuration if there is a config error or the config
+	  file does not exist.
 
-	* CI: Fix logic inversion in runTestsuite
+	  Change-Id: If1068a582ec54ab7fb437265cb5370a97a825737
 
-	  Change-Id: I56399aa384468f45494c2c3650420563a0b6efe1
+2018-09-13 13:03 +0000 [326574d4fc]  Corey Farrell <git@cfware.com>
 
-2018-07-17 04:03 +0000 [c025f09452]  George Joseph <gjoseph@digium.com>
+	* Resolve warning about duplicate 'dialplan' CLI.
 
-	* CI: Add teardownRealtime
+	  Change-Id: I029db1b4a32ccfb38374d6fe944dc430866f4b30
 
-	  Change-Id: I2fe55c38607eaec2fbf69ef23a5019e0c443a64b
+2018-10-02 01:33 +0000 [65f06a4505]  Corey Farrell <git@cfware.com>
 
-2018-07-16 13:30 +0000 [0da16dd5d0]  George Joseph <gjoseph@digium.com>
+	* loader: Fix result of module reload error.
 
-	* CI:  Prevent Jenkins from triggering jobs back to itself
+	  When a module reload fails we never set AST_MODULE_RELOAD_ERROR.  This
+	  caused reload failures to incorrectly report 'No module found'.
 
-	  Change-Id: I9cae8bb3d1a2cea335d3ccd88d471832549666fd
+	  Change-Id: I5f3953e0f7d135e53ec797f24c97ee3f73f232e7
 
-2018-07-13 18:26 +0000 [c8ea924b50]  Richard Mudgett <rmudgett@digium.com>
+2018-09-28 10:13 +0000 [f066dbc353]  Corey Farrell <git@cfware.com>
 
-	* Build: Fix modules getting their optimization setting overridden.
+	* loader: Improve error handling.
 
-	  Asterisk modules that use PJPROJECT services have their compiler
-	  optimization and possibly their symbolic debug options overridden by the
-	  PJPROJECT configure script selected settings.
+	  * Display list of unavailable dependencies when they cause another
+	    module to fail loading.
+	  * When a module declines to load find all modules which depend on it so
+	    they can be declined and listed together.
+	  * Prevent retry of declined modules during startup.
+	  * When a module fails to dlopen try loading it with RTLD_LAZY so we can
+	    attempt to display the list of missing dependencies.
 
-	  * We need to filter-out any -O and -g options in PJ_CFLAGS before echoing
-	  out the result so the PJPROJECT_INCLUDE variable does not override the
-	  Asterisk module settings when using bundled PJPROJECT.
+	  These changes are meant to reduce logger spam that is caused when a
+	  module has many dependencies and declines to load.  This also fixes some
+	  error paths which failed to recognize required modules.
 
-	  NOTE: This patch only has an effect when using bundled PJPROJECT.
+	  Module load/start errors are delayed until the end of loader startup.
 
-	  ASTERISK-27563
+	  Change-Id: I046052c71331c556c09d39f47a3b92975f3e1758
 
-	  Change-Id: If124169735ecf572ad1535cd43bff94cb44d5b30
+2018-09-25 16:19 +0000 [406c5bef0a]  Emmanuel BUU <emmanuel.buu@ives.fr>
 
-2018-07-16 11:08 +0000 [bb52ce0b64]  George Joseph <gjoseph@digium.com>
+	* core/frame: Fix ast_frdup() and ast_frisolate() for empty text frames
 
-	* CI: runUnittests: loop a few times on waitfullybooted
+	  If a channel creates an AST_TEXT_FRAME with datalen == 0, the ast_frdup()
+	  and ast_frisolate() functions could create a clone frame with an invalid
+	  data.ptr which would cause a crash.  The proposed fix is to make sure that
+	  for such empty text frames, ast_frdup() and ast_frisolate() return cloned
+	  text frames with a valid data.ptr.
 
-	  Change-Id: Icebc0d013896f3b2a7214945cac60647435c1651
+	  ASTERISK-28076
+	  Reported by: Emmanuel BUU
+	  Tested by: Emmanuel BUU
 
-2018-07-16 10:49 +0000 [a5e6ffa1f2]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ib882dd028598f13c4c233edbfdd7e54ad44a68e9
 
-	* CI:  Add realtime checks to dailies
+2018-09-30 23:11 +0000 [693e00eee6]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I6dc8ab1679b3505c6dde1d47e1b9276df47814f8
+	* astobj2: Record lock usage to refs log when DEBUG_THREADS is enabled.
 
-2018-07-16 09:13 +0000 [1817d5a3e7]  George Joseph <gjoseph@digium.com>
+	  When DEBUG_THREADS is enabled we can know if the astobj2 mutex / rwlock
+	  was ever used, so it can be recorded in the REF_DEBUG destructor entry.
 
-	* CI:  Add weekly REF_DEBUG testsuite run
+	  Create contrib/scripts/reflocks.py to process locking used by
+	  allocator.  This can be used to identify places where
+	  AO2_ALLOC_OPT_LOCK_NOLOCK should be used to reduce memory usage.
 
-	  Change-Id: I5b581d0a0d1d1bb9b38961d40b112fb448355037
+	  Change-Id: I2e3cd23336a97df2692b545f548fd79b14b53bf4
 
-2018-07-16 08:44 +0000 [0023e40308]  George Joseph <gjoseph@digium.com>
+2018-10-01 12:11 +0000 [5a9230eacb]  Corey Farrell <git@cfware.com>
 
-	* CI: Fix bad reporting of status by the verification pub
+	* app_page: Add dependency against app_confbridge.
 
-	  Change-Id: I6f31a130b3ba0187149aaaa2ce94195a79e0f6a6
+	  Change-Id: I1946509f518961d23fb21229d91676ee3e441921
 
-2018-07-16 07:16 +0000 [014f226bf1]  George Joseph <gjoseph@digium.com>
+2018-09-28 13:55 +0000 [a2554d5e5a]  Richard Mudgett <rmudgett@digium.com>
 
-	* CI: Make build tag an acceptable docker name
+	* app_queue.c: Fix json ref leak
 
-	  Change-Id: I3a4b8a4a9c488ddabf9daf651dc1334222056f38
+	  Declining the queue_member_status_type stasis message in stasis.conf
+	  causes these messages to leak json objects.
 
-2018-07-13 22:44 +0000 [871715f77c]  Corey Farrell <git@cfware.com>
+	  * Add missing ast_json_unref() if the type is NULL in
+	  queue_publish_member_blob().
 
-	* Fix declaration of PBX_CURL for ./configure --without-libcurl
+	  ASTERISK-28084
 
-	  When `--without-libcurl` is used PBX_CURL is never set.  Set default
-	  value 0 so the proper value is passed to menuselect.
+	  Change-Id: I691ecf49bd1f7d9c29182e1eee8c4bb7103be9fc
 
-	  Change-Id: I03e2842a00899cbca2dbde52bb1f6636d54bae1e
+2018-10-01 03:07 +0000 [266ed3d68b]  Corey Farrell <git@cfware.com>
 
-2018-07-13 06:56 +0000 [b97b39fe1b]  George Joseph <gjoseph@digium.com>
+	* Append CHANGES/UPGRADE.txt for module loader changes.
 
-	* CI: Add daily periodics to CI
+	  Change-Id: Ib8db4e14187f5c11ecbff532df17d30c5d36fa3e
 
-	  Change-Id: I26933e73928e091ae72e838c02f4f2ec7c3983d6
+2018-09-25 17:33 +0000 [ab90479642]  Alexei Gradinari <alex2grad@gmail.com>
 
-2018-07-12 16:34 +0000 [1b1271c3ea]  George Joseph <gjoseph@digium.com>
+	* res_pjsip: improve realtime performance on CLI 'pjsip show contacts'
 
-	* CI: Add Asterisk Gates
+	  CLI command 'pjsip show contacts' inefficiently make a lot of DB requests.
 
-	  Change-Id: I7e2467f9120812551238d8005deb97f965279205
+	  For example if there are 10k aors then asterisk requests these 10k records
+	  of aor and then does 10k requests of contact - one request per aor.
 
-2018-07-11 15:55 +0000 [df67a98938]  George Joseph <gjoseph@digium.com>
+	  Even if use 'like <pattern>' the asterisk requests all aor's and contact's
+	  records and then filters them by itself.
 
-	* CI: Remove duplicate checkout
+	  This patch gathers contact's container by
+	  - retrieving all dynamic contacts by regex (filtered by reg_server)
+	  - retrieving all aors with permanent contacts
+	  - finally filters container by regex
 
-	  Change-Id: If5f925b4c4ed7000b153f3ed8386ce2140c886f8
+	  ASTERISK-28077 #close
 
-2018-07-11 15:09 +0000 [947d024539]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Id0ad65d14952a02fb213273a90f3f680a8149618
 
-	* CI: Update cleanup steps and permissions
+2018-09-28 14:45 +0000 [3337fe85c5]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I7ca92935979d94845af8e1caf4468cbd6209b7de
+	* jansson-bundled: Add patches to improve json_pack error reporting.
 
-2018-07-11 14:54 +0000 [578d381164]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I045e420d5e73e60639079246e810da6ae21ae22b
 
-	* CI: Fix log artifact paths
+2018-09-27 19:32 +0000 [b3b95c8d36]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I55136de8f4d9c3b56bd4d054306a187bb04a4b7d
+	* lock: Improve performance of DEBUG_THREADS.
 
-2018-07-11 14:45 +0000 [96464a523b]  George Joseph <gjoseph@digium.com>
+	  Add a volatile flag to lock tracking structures so we only need to use
+	  the global lock when first initializing tracking.
 
-	* CI: Remove CleanBeforeCheckout option for testsuite
+	  Additionally add support for DEBUG_THREADS_LOOSE_ABI.  This is used by
+	  astobj2.c to eliminate storage for tracking fields when DEBUG_THREADS is
+	  not defined.
 
-	  Change-Id: I510231c9087f7be5272b8ef3f3223eadaaffb754
+	  Change-Id: Iabd650908901843e9fff47ef1c539f0e1b8cb13b
 
-2018-07-11 14:00 +0000 [2d16c85574]  George Joseph <gjoseph@digium.com>
+2018-09-27 13:19 +0000 [b4ab26e72f]  George Joseph <gjoseph@digium.com>
 
-	* CI: Move gates into source repo
+	* app_confbridge:  Use bridge join hook to send join and leave events
 
-	  Change-Id: If028ede5f3b127fa274c63ce166bc04ad7c1e5db
+	  The first attempt at publishing confbridge events to participants
+	  involved publishing them at the same time stasis events were
+	  created.  This caused issues with bridge and channel locks.  The
+	  second attempt involved publishing them when the stasis events
+	  were received by the code that published the confbridge AMI events.
+	  This caused timing issues because, depending on resources available,
+	  the event could be received before channels actually joined the
+	  bridge and would therefore fail to send messages to the participant.
 
-2018-07-11 11:57 +0000 [d739aa71f8]  Alexander Traud (License 6520)
+	  This attempt reverts to the original mechanism with one exception.
+	  The join and leave events are published via bridge join and leave
+	  hooks.  This guarantees the states of the channels and bridge and
+	  provides deterministic timing for event publishing.
 
-	* Bundled PJPROJECT: Disable internal connection oriented keep-alive.
+	  Change-Id: I2660074f8a30a5224cb953d5e047ee84484a9036
 
-	  Turn off the periodic sending of CRLNCRLN.  Default is on (90 seconds),
-	  which conflicts with the global section's keep_alive_interval option in
-	  pjsip.conf.
+2018-09-27 04:51 +0000 [33455f8d6f]  Corey Farrell <git@cfware.com>
 
-	  patches:
-	    pjsip_keep_not_alive.patch submitted by Alexander Traud (License 6520)
+	* astobj2: Reduce memory overhead.
 
-	  ASTERISK-27347
+	  Reduce options to 2-bit field, magic to 30 bit field.  Move ref_counter
+	  next to options and explicitly use int32_t so the fields will pack.
 
-	  Change-Id: I6a197f56e1830d3b7e5ec70f17025840a290b057
+	  This reduces memory overhead for every ao2 object by 8 bytes on x86_64.
 
-2018-07-11 06:14 +0000 [8167048804]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Idc1baabb35ec3b3d8de463c4fa3011eaf7fcafb5
 
-	* CI:  Initial commit for moving CI into source repo
+2018-09-27 15:01 +0000 [d1698b4e0b]  Sean Bright <sean.bright@gmail.com>
 
-	  Create tests/CI directory and add files used by Jenkins to
-	  build and test Asterisk.
+	* config.c: Cleanup AST_INCLUDE_GLOB
 
-	  With this commit, Jenkins will run the Asterisk Unit Tests using
-	  the Jenkinsfile at tests/CI/unittests.jenkinsfile.  Bash scripts
-	  to do the actual building and testing are also in the same directory.
-	  Output is placed in tests/CI/output so that directory has been
-	  added to .gitignore.
+	  * In main/config.c, AST_INCLUDE_GLOB is fixed to '1' making the #ifdefs
+	    pointless.
 
-	  Change-Id: I9448065465e6de2b878634510ace8fd1ef378608
+	  * In utils/extconf.c, AST_INCLUDE_GLOB is never defined so there is a
+	    lot of dead code.
 
-2018-06-23 01:40 +0000 [167baff944]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I1bad1a46d7466ddf90d52cc724e997195495226c
 
-	* chan_ooh323: IPTOS_MINCOST is not defined on Solaris.
+2018-09-27 05:33 +0000 [0fe08c5e79]  Corey Farrell <git@cfware.com>
 
-	  Furthermore, <sys/sockio.h> is required for SIOCGIF*.
+	* astobj2: Fix shutdown order.
 
-	  ASTERISK-27938
+	  When REF_DEBUG and AO2_DEBUG are both enabled we closed the refs log
+	  before we shutdown astobj2_container.  This caused the AO2_DEBUG
+	  container registration container to be reported as a leak.
 
-	  Change-Id: Idc9153ece769944765b66122efb11728d8d8ebde
+	  Change-Id: If9111c4c21c68064b22c546d5d7a41fac430430e
 
-2018-07-05 16:02 +0000 [9e615dac63]  Alexei Gradinari <alex2grad@gmail.com>
+2018-09-05 21:14 +0000 [958727cece]  Cao Minh Hiep <chiep@infinitalk.co.jp>
 
-	* res_pjsip_pubsub: segfault in function publish_expire
+	* app_queue: Fix Attended transfer hangup with removing pending member.
 
-	  The function pubsub_on_rx_publish_request incorrectly uses
-	  of AST_SCHED_REPLACE_UNREF.
+	  This issue related to setting of holdtime, announcements, member delays.
+	  It works well if we set the member delays to "0" and no announcements
+	  and no holdtime.This issue will happen if we set member delays to "1",
+	  "2"... or announcements or holdtime and hangs up the call during
+	  processing it.
 
-	  The AST_SCHED_REPLACE_UNREF should unref old '_data'.
+	  And here is the reason:
+	  (At the step of answering a phone.)
+	  It takes care any holdtime, announcements, member delays,
+	  or other options after a call has been answered if it exists.
 
-	  Because of this, there may be a double unref
-	  of variable 'publication' when ast_sched_del is unsuccessful
-	  that leads to use after free of the 'publication' in publish_expire.
+	  Normally, After the call has been aswered,
+	  and we wait for the processing one of the cases of the member delays
+	  or hold time or announcements finished, "if (ast_check_hangup(peer))"
+	  will be not executed, then queue will be updated at update_queue().
+	  Here, pending member will be removed.
 
-	  ASTERISK-27956 #close
+	  However, after the call has been aswered,
+	  if we hangs up the call during one of the cases of the member delays
+	  or hold time or announcements, "if (ast_check_hangup(peer))"
+	  will be executed.
+	  outgoing = NULL and at hangupcalls, pending members will not be removed.
 
-	  Change-Id: Ie0f0cfc7e036953d890b188656010b325a5cdc82
+	  * This fixed patch will remove the pending member from container
+	  before hanging up the call with outgoing is NULL.
 
-2018-07-06 09:04 +0000 [7666b80378]  George Joseph <gjoseph@digium.com>
+	  ASTERISK-27920
 
-	* test.c:  Make output jUnit compatible
+	  Reported by: Cao Minh Hiep
+	  Tested by: Cao Minh Hiep
 
-	  Separate "name" into "classname" and "name".
-	  Use '.' for classname separator instead of '/'.
-	  Prefix reserved words with '_'.
-	  Wrap output with a top-level "testsuites" element.
+	  Change-Id: Ib780fbf48ace9d2d8eaa1270b9d530a4fc14c855
 
-	  Change-Id: Iec1a985eba1c478e5c1d65d5dfd95cb708442099
+2018-06-26 09:17 +0000 [1f68614f3c]  Moritz Fain <moritz@fain.io>
 
-2018-07-06 07:57 +0000 [3470409dd6]  George Joseph <gjoseph@digium.com>
+	* res_stasis: Fix stale data in ARI bridges
 
-	* res_pjsip:  Add 'suppress_q850_reason_headers' option to endpoint
+	  Fixed an issue that resulted in "Allocation failed" each time an ARI
+	  request was made to start playing MOH on a bridge.
 
-	  A new option 'suppress_q850_reason_headers' has been added to the
-	  endpoint object. Some devices can't accept multiple Reason headers and
-	  get confused when both 'SIP' and 'Q.850' Reason headers are received.
-	  This option allows the 'Q.850' Reason header to be suppressed.
-	  The default value is 'no'.
+	  In bridge_moh_create() we were attaching the after bridge callbacks to
+	  chan which is the ;1 channel of the unreal channel pair.  We should have
+	  attached them to the ;2 channel which is pushed into the bridge by
+	  ast_unreal_channel_push_to_bridge().  The callbacks are called when the
+	  specific channel leaves the bridging system.  Since the ;1 channel is
+	  never put into a bridge the callbacks never get called.  The callbacks
+	  then never remove the moh_wrapper from the app_bridges_moh container.  As
+	  a result we cannot find the channel associated with the wrapper to start
+	  MOH because it has hungup.  This is the reason causing the reported issue.
 
-	  ASTERISK-27949
-	  Reported-by: Ross Beer
+	  * Rather than using after bridge callbacks to cleanup, we now have
+	  moh_channel_thread() doing the cleanup when the channel hangs up.
 
-	  Change-Id: I54cf37a827d77de2079256bb3de7e90fa5e1deb1
+	  * Fixed moh_channel_thread() accumulating control frames on the stasis
+	  bridge MOH channel until MOH is stopped.  Control frames are no longer
+	  accumulated while MOH is playing.
 
-2018-07-05 15:43 +0000 [76f2699d2d]  Joshua Colp <jcolp@digium.com>
+	  * Fixed channel ref counting issue.  stasis_app_bridge_moh_channel() may
+	  or may not return a channel ref.  As a result ast_ari_bridges_start_moh()
+	  wouldn't know it may have a channel ref to release.
+	  stasis_app_bridge_moh_channel() will now return a ref with the channel it
+	  returns.
 
-	* res_pjsip_t38: Decline T.38 stream on failure case.
+	  * Eliminated RAII_VAR in bridge_moh_create().
 
-	  When negotiating an incoming T.38 stream the code incorrectly
-	  returned failure instead of a decline for the stream when a
-	  problem occurred or the configuration didn't allow it. This
-	  resulted in SDP offers being rejected with a 488 response
-	  in all cases, even when another valid stream was present.
+	  ASTERISK-26094 #close
 
-	  This change makes it so the stream is now declined. If no
-	  streams are accepted a 488 response is sent while if at least
-	  one stream is accepted all the declined streams are, well,
-	  declined.
+	  Change-Id: Ibff479e167b3320c68aaabfada7e1d0ef7bd548c
 
-	  ASTERISK-27763
+2018-09-10 11:28 +0000 [67e1e49e08]  Ben Ford <bford@digium.com>
 
-	  Change-Id: I88bcf793788c412a9839d111a5c736bf6867807c
+	* res_rtp_asterisk.c: Add "seqno" strictrtp option
 
-2018-07-02 18:43 +0000 [a82f9762fd]  Richard Mudgett <rmudgett@digium.com>
+	  When networks experience disruptions, there can be large gaps of time
+	  between receiving packets. When strictrtp is enabled, this created
+	  issues where a flood of packets could come in and be seen as an attack.
+	  Another option - seqno - has been added to the strictrtp option that
+	  ignores the time interval and goes strictly by sequence number for
+	  validity.
 
-	* res_pjsip_t38.c: Be smarter about how we respond when T.38 is disabled.
+	  Change-Id: I8a42b8d193673899c8fc22fe7f98ea87df89be71
 
-	  We were blindly responding with AST_T38_REFUSED when ANY T.38 control
-	  frame came accross the bridge.  This causes T.38 Gateway to get confused
-	  and the T.38 session to get in a strange state.
+2018-09-20 13:59 +0000 [e04c49b517]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  * Made the T.38 framehook only respond to request frames and ignore
-	  response frames.
+	* res_odbc: fix missing SQL error diagnostic
 
-	  ASTERISK-27657
-	  ASTERISK-27080
+	  On SQL error there is not diagnostic information about this error.
+	  There is only
+	  WARNING res_odbc.c: SQL Execute error -1!
 
-	  Change-Id: I5fb5967c7d1efb30a7ff375f82887ca82a55b05b
+	  The function ast_odbc_print_errors calls a SQLGetDiagField to get the number
+	  of available diagnostic records, but the SQLGetDiagField returns 0.
+	  However SQLGetDiagRec could return one diagnostic records in this case.
 
-2018-07-03 12:10 +0000 [b6b49071af]  Richard Mudgett <rmudgett@digium.com>
+	  Looking at many example of getting diagnostics error information
+	  I found out that the best way it's to use only SQLGetDiagRec
+	  while it returns SQL_SUCCESS.
 
-	* res_pjsip/pjsip_transport_management.c: Fix deadlock with transport keep alive.
+	  Also this patch adds calls of ast_odbc_print_errors on SQL_ERROR
+	  to res_config_odbc.
 
-	  Using the keep_alive_interval option can result in a deadlock between the
-	  pjproject transport manager group lock and the monitored transports ao2
-	  container lock.  The pjproject transport manager group lock has to be
-	  superior in the locking order to the monitored transports ao2 container
-	  lock because of pjproject callbacks called when already holding the group
-	  lock.  The lock inversion happens when Asterisk attempts to send a keep
-	  alive packet over the reliable transports.
+	  ASTERISK-28065 #close
 
-	  * Made keepalive_transport_thread() iterate over the monitored transports
-	  container rather than use the ao2_callback() method.  This avoids holding
-	  the container lock when sending the keep alive packet.
+	  Change-Id: Iba5ae5470ac49ecd911dd084effbe9efac68ccc1
 
-	  ASTERISK-26686
+2018-09-26 08:12 +0000 [227d9719da]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I5d5392a52e698bbe41a93f7d8e92bf0e61fe3951
+	* CI:  Add --test-timeout option to runTestsuite.sh
 
-2018-06-29 18:28 +0000 [140ce56f50]  Richard Mudgett <rmudgett@digium.com>
+	  The default is 600 seconds.
+	  Also added timeouts to the *TestGroups.json files.
 
-	* res_pjsip_t38.c: Fix crash by ignoring 1xx messages.
+	  Change-Id: I8ab6a69e704b6a10f06a0e52ede02312a2b72fe0
 
-	  If we initiated a T.38 reINVITE, we would crash if we received any other
-	  1xx response message except 100 if it were followed by a 200 response.
+2018-09-18 08:01 +0000 [681e115ff1]  Peter Katzmann <peter.katzmann@edag.de>
 
-	  * Made ignore any 1xx response so we do not close out the T.38 negotiation
-	  too early.  For good measure we'll now accept any 2xx response as
-	  acceptance of the reINVITE T.38 offer.
+	* chan_sip: SipNotify on Chan_Sip vi AMI behave different to CLI
 
-	  NOTE: In v13 we don't actually crash but we should deal with the non-final
-	  response messages and not switch to rejected and then to enabled when the
-	  200 comes in.
+	  With tls and udp enabled asterisk generates a warning about sending
+	  message via udp instead of tls.
+	  sip notify command via cli works as expected and without warning.
 
-	  ASTERISK-27944
+	  asterisk has to set the connection information accordingly to connection
+	  and not on presumption
 
-	  Change-Id: I0ca88aae708d091db7335af73f41035a212adff4
+	  ASTERISK-28057 #close
 
-2018-07-03 18:07 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  Change-Id: Ib43315aba1f2c14ba077b52d8c5b00be0006656e
 
-	* asterisk 13.22.0-rc1 Released.
+2018-09-24 17:56 +0000 [cc8e0dfabd]  George Joseph <gjoseph@digium.com>
 
-2018-07-02 18:44 +0000 [62859ad526]  Joshua Colp <jcolp@digium.com>
+	* configure.ac:  Check for unbound version >= 1.5
 
-	* pjsip: Clarify certificate configuration for Websocket.
+	  In order to do this and provide good feedback, a new macro was
+	  created (AST_EXT_LIB_EXTRA_CHECK) which does the normal check and
+	  path setups for the library then compiles, links and runs a supplied
+	  code fragment to do the final determination.  In this case, the
+	  final code fragment compares UNBOUND_VERSION_MAJOR
+	  and UNBOUND_VERSION_MINOR to determine if they're greater than or
+	  equal to 1.5.
 
-	  The Websocket transport uses the built-in HTTP server. As a result
-	  the TLS configuration is done in http.conf and not in pjsip.conf.
+	  Since we require version 1.5, some code in res_resolver_unbound
+	  was also simplified.
 
-	  This change adds a warning if this is configured in pjsip.conf and
-	  also clarifies in the sample configuration file.
+	  ASTERISK-28045
+	  Reported by: Samuel Galarneau
 
-	  Change-Id: I187d994d328c3ed274b6754fd4c2a4955bdc6dd9
+	  Change-Id: Iee94ad543cd6f8b118df8c4c7afd9c4e2ca1fa72
 
-2018-07-01 13:54 +0000 [5f3869608a]  Joshua Colp <jcolp@digium.com>
+2018-09-24 12:43 +0000 [7c49ea7cff]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_pubsub: Hold module reference for publications.
+	* res_rtp_asterisk: Raise event when RTP port is allocated
 
-	  Incoming publications need to ensure that the module remains
-	  loaded for the lifetime of them. This is now done by holding
-	  a reference to the module while the publication exists. This
-	  mirrors that of inbound subscriptions.
+	  This change raises a testsuite event to provide what port
+	  Asterisk has actually allocated for RTP. This ensures that
+	  testsuite tests can remove any assumption of ports and instead
+	  use the actual port in use.
 
-	  ASTERISK-27783
+	  ASTERISK-28070
 
-	  Change-Id: Ia98c95a15e11af25728d5fb3e56e12cda0cfc7c0
+	  Change-Id: I91bd45782e84284e01c89acf4b2da352e14ae044
 
-2018-05-21 07:24 +0000 [447ec4e472]  Robert Mordec <r.mordec@slican.pl>
+2018-07-16 22:55 +0000 [ede59966d9]  Corey Farrell <git@cfware.com>
 
-	* app_confbridge: Bridge and announcers not removed if conference ends quickly
+	* jansson: Backport fixes to bundled, use json_vsprintf if available.
 
-	  If a conference is ended very quickly after it was created (i.e., the
-	  first user immediately hangs up) then the conference bridge and announcer
-	  channels are not removed.
+	  Use json_vsprintf from versions which contain fix for va_copy leak.
 
-	  When a conference is created, the push_announcer() function is added to
-	  the playback queue task processor and the conference object reference is
-	  bumped.  If a conference is ended while the push_announcer() function is
-	  still going then the ao2_cleanup(conference) at the end of
-	  push_announcer() will call the destructor function -
-	  destroy_conference_bridge().
+	  Apply fixes from jansson master:
+	  * va_copy leak fix.
+	  * Avoid potential invalid memory read in json_pack.
+	  * Rename variable that shadowed another.
 
-	  The destroy_conference_bridge() function will then add the
-	  hangup_playback() task to the playback queue and will wait for it to end.
-	  Since it is already a current task of the playback queue it will wait
-	  forever.
+	  Change-Id: I7522e462d2a52f53010ffa1e7d705c666ec35539
 
-	  This patch makes the conference thread call push_announcer() directly.
-	  This way the conference object reference bump is not needed.  Since the
-	  playback queue task processor is only used by the conference thread
-	  itself, there is no danger of trying to play announcements before the
-	  announcer is pushed to the bridge.
+2018-07-16 22:55 +0000 [7b0dbda936]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27870 #close
+	* json: Take advantage of new API's.
 
-	  Change-Id: I947a50fb121422d90fd1816d643a54d75185a477
+	  * Use "o*" format specifier for optional fields in ast_json_party_id.
+	  * Stop using ast_json_deep_copy on immutable objects, it is now thread
+	    safe to just use ast_json_ref.
 
-2018-06-21 00:28 +0000 [33f4716d13]  Matthew Fredrickson <creslin@digium.com>
+	  Additional changes to ast_json_pack calls in the vicinity:
+	  * Use "O" when an object needs to be bumped.  This was previously
+	    avoided as it was not thread safe.
+	  * Use "o?" and "O?" to replace NULL with ast_json_null().  The
+	    "?" is a new feature of ast_json_pack starting with Asterisk 16.
 
-	* main/cdr.c: Alleviate CDR deadlock
+	  Change-Id: I8382d28d7d83ee0ce13334e51ae45dbc0bdaef48
 
-	  There is a rare case (do to the infrequent timing involved) where
-	  CDR submission threads in batch mode can deadlock with a currently
-	  running CDR batch process.  This patch should remove the need for
-	  holding the lock in the scheduler and should clean a few code
-	  paths up that inconsistently submitted new work to the CDR batch
-	  processor.
+2018-09-17 15:35 +0000 [755febe497]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-27909
+	* rtp_engine: rtcp_report_to_json can overflow the ssrc integer value
 
-	  Change-Id: I6333e865db7c593c102c2fd948cecdb96481974d
-	  Reported-by: Denis Lebedev
+	  When writing an RTCP report to json the code attempts to pack the "ssrc" and
+	  "source_ssrc" unsigned integer values as a signed int value type. This of course
+	  means if the ssrc's unsigned value is greater than that which can fit into a
+	  signed integer value it gets converted to a negative number. Subsequently, the
+	  negative value goes out in the json report.
 
-2018-06-25 22:08 +0000 [3203c3491a]  Kirsty Tyerman <kirsty.tyerman@boeing.com>
+	  This patch now packs the value as a json_int_t, which is the widest integer type
+	  available on a given system. This should make it so the value no longer
+	  overflows.
 
-	* pbx_dundi: reordered unloading of module pbx_dundi
+	  Note, this was caught by two failing tests hep/rtcp-receiver/ and
+	  hep/rtcp-sender.
 
-	  Destroy scheduler after peers are pruned to stop dundi crashing when
-	  unloading module.
+	  Change-Id: I2af275286ee5e795b79f0c3d450d9e4b28e958b0
 
-	  ASTERISK-26987
-	  Reported-by: Kirsty Tyerman
+2018-09-21 14:32 +0000 [ba91c73f8e]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ic12e562cd90d8d813a9e97f302045091f59e3c05
+	* app_voicemail:  Fix stack overrun in append_mailbox
 
-2018-06-28 12:07 +0000 [d0e4cbfc51]  Richard Mudgett <rmudgett@digium.com>
+	  The append_mailbox function wasn't calculating the correct length
+	  to pass to ast_alloca and it wasn't handling the case where context
+	  might be empty.
 
-	* AMI SendText action: Fix to use correct thread to send the text.
+	  Found by the Address Sanitizer.
 
-	  The AMI action was directly sending the text to the channel driver.
-	  However, this makes two threads attempt to handle media and runs afowl of
-	  CHECK_BLOCKING.
+	  Change-Id: I7eb51c7bd18a7a8dbdba261462a95cc69e84f161
 
-	  * Queue a read action to make the channel's media handling thread actually
-	  send the text message.  This changes the AMI actions success/fail response
-	  to just mean the text was queued to be sent not that the text actually got
-	  sent.  The channel driver may not even support sending text messages.
+2018-09-21 15:23 +0000 [0a1a96d331]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27943
+	* channel.c:  Address stack overflow in does_id_conflict()
 
-	  Change-Id: I9dce343d8fa634ba5a416a1326d8a6340f98c379
+	  does_id_conflict() was passing a pointer to an int to a callback
+	  that expected a pointer to a size_t.
 
-2018-06-25 07:37 +0000 [6087c1e575]  George Joseph <gjoseph@digium.com>
+	  Found by the Address Sanitizer.
 
-	* res_pjsip_messaging:  Allow application/* for in-dialog MESSAGEs
+	  Change-Id: I0ff542067eef63a14a60301654d65d34fe2ad503
 
-	  In addition to text/* content types, incoming_in_dialog_request now
-	  accepts application/* content types.
+2018-09-21 10:19 +0000 [7a7b21f3a0]  Corey Farrell <git@cfware.com>
 
-	  Also fixed a length issue when copying the body text.  It was one
-	  character short.
+	* res_rtp_asterisk: Fix crash on ast_rtp_new failure.
 
-	  ASTERISK-27942
+	  ast_rtp_new free'd rtp upon failure, but rtp_engine.c would also call
+	  the destroy callback.  Remove call to ast_free from ast_rtp_new, leave
+	  it to rtp_engine.c to initiate the full cleanup.  Add error detection
+	  for the ssrc_mapping vector initialization.  In rtp_allocate_transport
+	  set rtp->s = -1 in the failure path where we close that FD to ensure we
+	  don't try closing it twice.
 
-	  Change-Id: I4e54d8cc6158dc47eb8fdd6ba0108c6fd53f2818
+	  ASTERISK-27854 #close
 
-2018-06-18 21:22 +0000 [06966e91fe]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ie02aecbb46228ca804e24b19cec2bb6f7b94e451
 
-	* res_pjsip_session:  Add ability to accept multiple sdp answers
+2018-09-20 15:26 +0000 [880905e7eb]  Sean Bright <sean.bright@gmail.com>
 
-	  pjproject by default currently will follow media forked during an INVITE
-	  on outbound calls if the To tag is different on a subsequent response as
-	  that on an earlier response.  We handle this correctly.  There have
-	  been reported cases where the To tag is the same but we still need to
-	  follow the media.  The pjproject patch in this commit adds the
-	  capability to sip_inv and also adds the capability to control it at
-	  runtime.  The original "different tag" behavior was always controllable
-	  at runtime but we never did anything with it and left it to default to
-	  TRUE.
+	* res_rtp_asterisk: Reset all settings on module reload
 
-	  So, along with the pjproject patch, this commit adds options to both the
-	  system and endpoint objects to control the two behaviors, and a small
-	  logic change to session_inv_on_media_update in res_pjsip_session to
-	  control the behavior at the endpoint level.
+	  'rtpchecksums' and 'rtcpinterval' are not being reset to their defaults
+	  if they are not present in the updated configuration file.
 
-	  The default behavior for "different tags" remains the same at TRUE and
-	  the default for "same tag" is FALSE.
+	  Change-Id: I1162e40199314d46cf3225d5e1271c4c81176670
 
-	  Change-Id: I64d071942b79adb2f0a4e13137389b19404fe3d6
-	  ASTERISK-27936
-	  Reported-by: Ross Beer
+2018-09-20 10:15 +0000 [fa1b836374]  George Joseph <gjoseph@digium.com>
 
-2018-06-21 11:45 +0000 [5e9230b528]  Alexander Traud <pabstraud@compuserve.com>
+	* app_voicemail:  Cleanup mailbox topic and cache
 
-	* uuid: Enable UUID in Solaris 11.
+	  app_voicemail wasn't properly cleaning up the stasis cache or the
+	  mwi topic pool when the module was unloaded or when a user was
+	  deleted as a result of a reload.  This resulted in leaks in both
+	  areas.
 
-	  ASTERISK-27933
-	  Reported by: bautsche
+	  * app_voicemail now calls ast_delete_mwi_state_full when it frees
+	    a user structure and ast_delete_mwi_state_full in turn now calls
+	    the new stasis_topic_pool_delete_topic function to clear the topic
+	    from the pool.
 
-	  Change-Id: I9b8362824efbfb2a16981e46e85f7c8322908c49
+	  Change-Id: Ide23144a4a810e7e0faad5a8e988d15947965df8
 
-2018-06-21 16:39 +0000 [cb9475eb3d]  Richard Mudgett <rmudgett@digium.com>
+2018-08-16 10:45 +0000 [339bf0cf7b]  Sean Bright <sean.bright@gmail.com>
 
-	* VECTOR: Passing parameters with side effects to macros is dangerous.
+	* AST-2018-009: Fix crash processing websocket HTTP Upgrade requests
 
-	  * Fix several instances where we were bumping a ref in the parameter and
-	  then unrefing the object if it failed.  The way the AST_VECTOR_APPEND()
-	  and AST_VECTOR_REPLACE() macros are implemented means if it fails the new
-	  value was never evaluated.
+	  The HTTP request processing in res_http_websocket allocates additional
+	  space on the stack for various headers received during an Upgrade request.
+	  An attacker could send a specially crafted request that causes this code
+	  to overflow the stack, resulting in a crash.
 
-	  Change-Id: I2847872a455b11ea7e5b7ce697c0a455a1d0ac9a
+	  * No longer allocate memory from the stack in a loop to parse the header
+	  values.  NOTE: There is a slight API change when using the passed in
+	  strings as is.  We now require the passed in strings to no longer have
+	  leading or trailing whitespace.  This isn't a problem as the only callers
+	  have already done this before passing the strings to the affected
+	  function.
 
-2018-06-21 11:22 +0000 [6d201a5df4]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-28013 #close
 
-	* smsq: Remove an left-over special case for Solaris.
+	  Change-Id: Ia564825a8a95e085fd17e658cb777fe1afa8091a
 
-	  Actually, this case was never needed because the check below does the same.
+2018-09-20 09:41 +0000 [1a9c69d729]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ia2fca4ba6c58c644a8b7cb2d9db8539728c14ffb
+	* stasis:  Add function to delete topic from pool
 
-2018-06-21 11:17 +0000 [c8966b8c29]  Alexander Traud <pabstraud@compuserve.com>
+	  There's been a long standing leak when using topic pools.  The
+	  topics in the pool get cleaned up when the last pool reference is
+	  released but you can't remove a topic specifically.  If you reloaded
+	  app_voicemail for instance, and mailboxes went away, their topics
+	  were left in the pool.
 
-	* res_http_post: Enable GMime in Solaris 11.
+	  * Added stasis_topic_pool_delete_topic() so modules can clean up
+	    topics from pools.
+	  * Registered the topic pool containers so it can be examined from
+	    the CLI when AO2_DEBUG is enabled.  They'll be named
+	    "<topic_pool_name>-pool".
 
-	  Change-Id: Ie434541f18f894c751d2e44bcb3efb3cac626019
+	  Change-Id: Ib7957951ee5c9b9b4482af7b9b4349112d62bc25
 
-2018-06-21 05:08 +0000 [81f39649ea]  Alexander Traud <pabstraud@compuserve.com>
+2018-09-03 09:55 +0000 [8811ab1803]  David Hajek <david.hajek@daktela.com>
 
-	* codecs/ilbc: Compile in Solaris 11.
+	* chan_sip.c: chan_sip unstable with TLS after asterisk start or reloads
 
-	  The symbol FS is the sampling frequency. That symbol is not used in Asterisk at
-	  all and was a copy-and-paste of the iLBC reference code from the IETF RFC.
-	  However, in Solaris, that symbol is defined by another header already. To
-	  compile in Solaris, that symbol has to go.
+	  Fixes random asterisk crash on start or reload with TLS phones.
 
-	  Change-Id: I91ddbe5be7c00069c3a25abd5f58d7b2f04c51b1
+	  ASTERISK-28034 #close
+	  Reported-by: David Hajek
 
-2018-06-21 05:07 +0000 [78855e13e8]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I2a859f97dc80c348e2fa56e918214ee29521c4ac
 
-	* chan_oss: Compile in Solaris 11.
+2018-09-20 04:48 +0000 [2f38bcdfc0]  Joshua Colp <jcolp@digium.com>
 
-	  M_READ existed already and was conflicting in name.
+	* res_remb_modifier: Add module for controlling REMB from CLI.
 
-	  Change-Id: I02108e07ae7d2dc314fe1e6c706c17731095a3e4
+	  This adds a module which registers a CLI command that can set the
+	  REMB bitrate value for REMB as it enters or exits Asterisk. This
+	  allows you to ignore what Asterisk or a client produces and is
+	  useful for demonstrations.
 
-2018-06-21 05:04 +0000 [0b3c090ada]  Alexander Traud <pabstraud@compuserve.com>
+	  This does not generate REMB frames, however, but just modifies
+	  them as they flow to or from a channel.
 
-	* func_env: Compile in Solaris 11.
+	  Change-Id: Ib089427c46a4a36d645cecfe02406adb38c17bec
 
-	  Change-Id: Idc9b36720f3d29c90a35a6a1ae79a7f9e1aaf50e
+2018-09-14 15:51 +0000 [f6695249a5]  Richard Mudgett <rmudgett@digium.com>
 
-2018-06-21 05:01 +0000 [a0445c7a3e]  Alexander Traud <pabstraud@compuserve.com>
+	* stasis: No need to keep a stasis type ref in a stasis msg or cache object.
 
-	* utils: Avoid an unused variable in Solaris 11.
+	  Stasis message types are global ao2 objects and we make stasis messages
+	  and cache entries hold references to them.  Since there are currently
+	  situations where cache objects are never deleted, the reference count on
+	  the types can exceed 100000 and generate a FRACK assertion message.  The
+	  stasis message cache could conceivably also have that many messages
+	  legitimately on large systems.
 
-	  With ./configure --enable-dev-mode[=noisy], the build fails because every
-	  warning gets an error. Therefore, Asterisk has to be free of warnings and this
-	  variable must go.
+	  The only down side to not holding the message type ref in the stasis
+	  message is it only makes a crash either at shutdown or when manually
+	  unloading a busy module slightly more likely.  However, this is more
+	  exposing a pre-existing stasis shutdown ordering issue than a problem with
+	  not holding a message type ref in stasis messages.
 
-	  Change-Id: I63dd2bc4833b9bdb04602f83422d16caf289d46a
+	  * Made stasis messages and cache entries no longer hold a ref to the
+	  message type.
 
-2018-06-21 04:59 +0000 [d3fe2aceab]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ibaa28efa8d8ad3836f0c65957192424c7f561707
 
-	* BuildSystem: Enable ./configure in Solaris 11.
+2018-09-18 13:59 +0000 [c008c27c85]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27931
+	* pjproject: Update initial 2.8 patches to apply cleanly.
 
-	  Change-Id: If298ce7f03be227a3687b9c20d382c9c55a72404
+	  ASTERISK-28059
 
-2018-06-20 13:24 +0000 [7686e674bf]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I027472f2753391646dde594a709a75f14422db93
 
-	* BuildSystem: Enable autotools in Solaris 11.
+2018-09-14 15:48 +0000 [192f71b7de]  Richard Mudgett <rmudgett@digium.com>
 
-	  Because this was the last operating system which required a special case, a
-	  version appended to the autotools, the whole version stuff is removed by this
-	  change. This simplifies the script ./bootstrap.sh. Hopefully, this gives even
-	  broader platform compatibility.
+	* stasis_message.c: Don't create immutable stasis objects with locks.
 
-	  ASTERISK-27929
-	  ASTERISK-27926
+	  * Create the stasis message object without a lock as it is immutable.
+	  * Create the stasis message type object without a lock as it is immutable.
+	  * Creating the stasis message type could crash if the passed in type name
+	  is NULL and REF_DEBUG is enabled.  Added missing NULL check when passing
+	  the ao2 object tag string.
 
-	  Change-Id: Id4cf433a1a7fa861d0210e1a2e16ca592b49fd5a
+	  Change-Id: I28763c58bb9f0b427c11971d0103bf94055e7b32
 
-2018-06-13 11:33 +0000 [f878de44af]  Richard Mudgett <rmudgett@digium.com>
+2018-09-17 11:38 +0000 [60258b4ec1]  Joshua Colp <jcolp@digium.com>
 
-	* channel.c: Make CHECK_BLOCKING() save thread LWP id for messages.
+	* pjproject: Upgrade to 2.8.
 
-	  * Removed an unnecessary call to ast_channel_blocker_set() in
-	  __ast_read().
+	  This change brings in PJSIP 2.8, removes all the patches
+	  that were merged upstream, and makes a minor change to
+	  support a breaking change that was done.
 
-	  ASTERISK-27625
+	  ASTERISK-28059
 
-	  Change-Id: I342168b999984666fb869cd519fe779583a73834
+	  Change-Id: I5097772b11b0f95c3c1f52df6400158666f0a189
 
-2018-06-13 16:41 +0000 [99f439dc01]  Richard Mudgett <rmudgett@digium.com>
+2018-09-18 09:39 +0000 [3e48c34f14]  Florian Floimair <f.floimair@commend.com>
 
-	* ARI POST DTMF: Make not compete with channel's media thread.
+	* alembic: fix suppress_q850_reason_headers column name
 
-	  There can be one and only one thread handling a channel's media at a time.
-	  Otherwise, we don't know which thread is going to handle the media frames.
+	  In the original commit introducing the feature the column in the alembic
+	  script was called 'suppress_q850_reason_header'.
+	  In the code however the option is called 'suppress_q850_reason_headers'
+	  (trailing 's'). This leads to errors when ARI push configuration is used.
 
-	  ASTERISK-27625
+	  Change-Id: Ie84808adbca6fcc9136556e4f5d741adbef5d14f
 
-	  Change-Id: I4d6a2fe7386ea447ee199003bf8ad681cb30454e
+2018-09-13 07:55 +0000 [29115e2384]  George Joseph <gjoseph@digium.com>
 
-2018-06-13 13:05 +0000 [6a1626c265]  Richard Mudgett <rmudgett@digium.com>
+	* app_voicemail: Remove need to subscribe to stasis
 
-	* AMI PlayDTMF Action: Make not compete with channel's media thread.
+	  app_voicemail was using the stasis cache to build and maintain a
+	  list of mailboxes that had subscribers.  It then used this list
+	  to determine if a mailbox should be polled for new messages if
+	  polling was enabled.  For this to work, stasis had to cache every
+	  subscription and unsubscription to the mailbox which caused a lot of
+	  overhead, both cpu and memory related.
 
-	  There can be one and only one thread handling a channel's media at a time.
-	  Otherwise, we don't know which thread is going to handle the media frames.
+	  Since polling is only required when changes are being made to
+	  mailboxes outside of app_voicemail and since the number of mailboxes
+	  that don't have any subscribers is likely to be very low, all
+	  mailboxes are now polled instead of just the ones with subscribers.
 
-	  ASTERISK-27625
+	  This paves the way for disabling the caching of stasis subscription
+	  change messages.
 
-	  Change-Id: Ia341f1a6f4d54f2022261abec9021fe5b2eb4905
+	  Also fixed cleanup in some of the unit tests that not only left
+	  test users in the users list but also caused segfaults if the tests
+	  were run more than once.
 
-2018-06-12 14:09 +0000 [1abcc41fff]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27121
 
-	* channel.c: Fix usage of CHECK_BLOCKING()
+	  Change-Id: I5cceb737246949f9782955c64425b8bd25a9e9ee
 
-	  The CHECK_BLOCKING() macro is used to indicate if a channel's handling
-	  thread is about to do a blocking operation (poll, read, or write) of
-	  media.  A few operations such as ast_queue_frame(), soft hangup, and
-	  masquerades use the indication to wake up the blocked thread to reevaluate
-	  what is going on.
+2018-09-18 06:08 +0000 [6e79e6b097]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-27625
+	* res_pjsip_session: Don't add declined stream if one does not exist.
 
-	  Change-Id: I4dfc33e01e60627d962efa29d0a4244cf151a84d
+	  Given a scenario where a session refresh was done with a removed
+	  stream we would always add a removed stream to the outgoing SDP
+	  even if one did not already exist.
 
-2018-06-18 18:04 +0000 [5d34ca5b33]  Richard Mudgett <rmudgett@digium.com>
+	  This change makes it so that a removed stream is only placed into
+	  the SDP if one already exists.
 
-	* autoservice: Don't start channel autoservice if the thread is a user interface.
+	  ASTERISK-28047
 
-	  Executing dialplan functions from either AMI or ARI by getting a variable
-	  could place the channel into autoservice.  However, these user interface
-	  threads do not handle the channel's media so we wind up with two threads
-	  attempting to handle the media.
+	  Change-Id: Ibb97d21cdeb87a8acae0c720861b0ff255708442
 
-	  There can be one and only one thread handling a channel's media at a time.
-	  Otherwise, we don't know which thread is going to handle the media frames.
+2018-09-17 10:38 +0000 [b0a0b975c5]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27625
+	* autoconf: Check for srtp_get_version_string() before using it
 
-	  Change-Id: If2dc94ce15ddabf923ed1e2a65ea0ef56e013e49
+	  Change-Id: Id2a916ff9448706090e72ff2c7fb3f5ba24a05df
 
-2018-06-18 16:07 +0000 [a6edbc9be1]  Richard Mudgett <rmudgett@digium.com>
+2018-09-17 07:10 +0000 [4a309839eb]  George Joseph <gjoseph@digium.com>
 
-	* Dialplan functions: Fix some channel autoservice misuse.
+	* CI: Fix typo in testsuite git checkout
 
-	  * Fix off nominal paths leaving the channel in autoservice.
-	  * Remove unnecessary start/stop channel autoservice.
-	  * Fix channel locking around a channel datastore search.
+	  Change-Id: I30024515e5b00a5044fd39fbff27d818f016b719
 
-	  Change-Id: I7ff2e42388064fe3149034ecae57604040b8b540
+2018-09-16 06:08 +0000 [55ca51af21]  Sean Bright <sean.bright@gmail.com>
 
-2018-06-19 10:43 +0000 [7b30158147]  Richard Mudgett <rmudgett@digium.com>
+	* res_srtp.c: Show linked version of libsrtp on module init
 
-	* Fix some doxygen and curly placement.
+	  Change-Id: Ib0a645d6985de5757cc4399ed2524b2d02c4f342
 
-	  Change-Id: I9a784a7c804120a8fa826c2a4cb9957e4b0b2fc8
+2018-09-07 09:40 +0000 [887a315e17]  Sean Bright <sean.bright@gmail.com>
 
-2018-06-15 15:21 +0000 [3570a62b32]  Matthew Fredrickson <creslin@digium.com>
+	* res_pjsip: Log IPv6 addresses correctly
 
-	* menuselect/menuselect_curses: Resolves sprintf usage error
+	  Both pjsip_tx_data.tp_info.dst_name and pjsip_rx_data.pkt_info.src_name
+	  store IPv6 addresses without enclosing brackets. This causes some log
+	  output to be confusing because it is difficult to separate the IPv6
+	  address from a port specification.
 
-	  Acccording to the man page for sprintf, using the same buffer for
-	  output as one used as an input yields undefined behavior.
-	  This patch should work around this problem.
+	  * Use pj_sockaddr_print() along with pjsip_tx_data.tp_info.dst_addr and
+	    pjsip_rx_data.pkt_info.src_addr where possible for consistent IPv6
+	    output.
 
-	  ASTERISK-27903
-	  Reported-by: Alexander Traud
+	  * When a pj_sockaddr is not available, explicitly wrap IPv6 addresses
+	    in brackets.
 
-	  Change-Id: I2213dcb454aff26457e2e4cc9c6821276463ae3a
+	  * When assigning pjsip_rx_data.pkt_info.src_name ourselves, make sure
+	    to also set pjsip_rx_data.pkt_info.src_addr.
 
-2018-06-12 15:13 +0000 [f94a310ca0]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I5cfe997ced7883862a12b9c7d8551d76ae02fcf8
 
-	* channel: Fix some more unprotected channel flag setting.
+2018-09-14 12:31 +0000 [3f9544c1f5]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I34c3b1201b1de539945bcfdcb264fff30332d48c
+	* CI: Use proper credentials for Security testsuite checkout
 
-2018-06-12 09:30 +0000 [bb0ce22b2b]  Sam Wierema <sam@messagebird.com>
+	  Can't do anonymous http checkout from Security-testsuite.
+	  Need to use same credentials as the gerrit review checkout.
 
-	* app_mp3: remove 10 seconds of silence after mp3 playback
+	  Change-Id: I87af68c995cb8926f5e87f9af245600d76984f05
 
-	  This patch changes the way asterisk polls output from mpg123, instead
-	  of waiting for 10 seconds(when playing an http url) it now uses a
-	  timeout of one second and iterates 10 times using this same timeout.
+2018-09-13 11:06 +0000 [17d6d9e1e7]  George Joseph <gjoseph@digium.com>
 
-	  The main difference is that for every timeout asterisk receives it now
-	  checks if mpg123 is still running before poll again.
+	* stasis_cache:  Stop caching stasis subscription change messages
 
-	  ASTERISK-27752
+	  Since app_voicemail no longer uses the cache to maintain its state
+	  there is no longer a need to cache these messages.
 
-	  Change-Id: Ib7df8462e3e380cb328011890ad9270d9e9b4620
+	  ASTERISK-27121
 
-2018-06-13 04:40 +0000 [b004b44a46]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I321c708505f5ad8d00e1b0afc4c27dc2ac12ecb4
 
-	* tests/test_utils: Repair ./configure --with-ssl=PATH.
+2018-09-12 12:39 +0000 [5842741689]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27914
+	* CI: Use .gitreview to default BRANCH_NAME.
 
-	  Change-Id: Ibcab8f556ee77776f203cff8b06d776a673b7bc4
+	  This ensures that binary modules are avoided in the master branch even
+	  if BRANCH_NAME is not set.
 
-2018-06-04 20:31 +0000 [c6116a3234]  Kirsty Tyerman <kirsty.tyerman@boeing.com>
+	  Change-Id: I79162d2063f22fa9d6b31fde4827ace2dd5bf0da
 
-	* chan_iax2: better handling for timeout and EINTR
+2018-09-11 07:22 +0000 [78453e65fd]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  The iax2 module is not handling timeout and EINTR case properly. Mainly when
-	  there is an interupt to the kernel thread. In case of ast_io_wait recieves a
-	  signal, or timeout it can be an error or return 0 which eventually escapes the
-	  thread loop, so that it cant recieve any data. This then causes the modules
-	  receive queue to build up on the kernel and stop any communications via iax in
-	  asterisk.
+	* optional_api: Remove unused nonoptreq fields
 
-	  The proposed patch is for the iax module, so that timeout and EINTR does not
-	  exit the thread.
+	  As they're not actively used, they only grow stale. The moduleinfo field itself
+	  is kept in Asterisk 13/15 for ABI compatibility.
 
-	  ASTERISK-27705
-	  Reported-by: Kirsty Tyerman
+	  ASTERISK-28046 #close
 
-	  Change-Id: Ib4c32562f69335869adc1783608e940c3535fbfb
+	  Change-Id: I8df66a7007f807840414bb348511a8c14c05a9fc
 
-2018-06-13 05:14 +0000 [e3de4bc46e]  Alexander Traud <pabstraud@compuserve.com>
+2018-09-03 06:50 +0000 [f4bffe2326]  lvl <digium@lvlconsultancy.nl>
 
-	* res_rtp_asterisk: Instead of ./configure use OPENSSL_NO_SRTP.
+	* manager: Set AMI event "Newexten" to the EVENT_FLAG_DIALPLAN class
 
-	  Previously, Asterisk used its script ./configure, to test whether OpenSSL was
-	  built with no-srtp (or was simply too old). However, the header file
-	  <openssl/opensslconf.h> is the preferred way to detect the local configuration
-	  of OpenSSL.
+	  The documentation already specified EVENT_FLAG_DIALPLAN for this
+	  event, but the implementation was using EVENT_FLAG_CALL.
 
-	  As a positive side-effect the script ./configure does not interleave the
-	  detection of the Open Settlement Protocol Toolkit (OSPTK) with the detection of
-	  individual features of OpenSSL anymore.
+	  Using EVENT_FLAG_DIALPLAN allows AMI clients to opt out of receiving
+	  this highly verbose event.
 
-	  Change-Id: I3c77c7b00b2ffa2e935632097fa057b9fdf480c0
+	  ASTERISK-28033
 
-2018-04-30 17:38 +0000 [034a04af2a]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I45b3119f30e4dbc17b49831f2b1a4f2c1beadafe
 
-	* AST-2018-008: Fix enumeration of endpoints from ACL rejected addresses.
+2018-09-12 07:18 +0000 [e5739c494c]  Sean Bright <sean.bright@gmail.com>
 
-	  When endpoint specific ACL rules block a SIP request they respond with a
-	  403 forbidden.  However, if an endpoint is not identified then a 401
-	  unauthorized response is sent.  This vulnerability just discloses which
-	  requests hit a defined endpoint.  The ACL rules cannot be bypassed to gain
-	  access to the disclosed endpoints.
+	* res_pjproject: Fix sockaddr conversion routines for non-bundled PJSIP
 
-	  * Made endpoint specific ACL rules now respond with a 401 unauthorized
-	  which is the same as if an endpoint were not identified.  The fix is
-	  accomplished by replacing the found endpoint with the artificial endpoint
-	  which always fails authentication.
+	  The bundled version of pjproject has a patch for Solaris compatability
+	  that changes the definition of various socket structures which we need
+	  to account for when compiling against a non-bundled version.
 
-	  ASTERISK-27818
+	  ASTERISK-28049 #close
 
-	  Change-Id: Icb275a54ff8e2df6c671a6d9bda37b5d732b3b32
+	  Change-Id: Ia1ea47c433fc2d915115193ee889a752373925f0
 
-2018-06-08 15:09 +0000 [ef2386fcd7]  Alexander Traud <pabstraud@compuserve.com>
+2018-09-10 22:28 +0000 [ecb3b23b07]  Corey Farrell <git@cfware.com>
 
-	* res_rtp_asterisk: Allow OpenSSL configured with no-deprecated.
+	* Build System: Resolve conflict between DESTDIR and bundled jansson.
 
-	  Furthermore, allow OpenSSL configured with no-dh. Additionally, this change
-	  allows auto-negotiation of the elliptic curve/group for servers, not only with
-	  OpenSSL 1.0.2 but also with OpenSSL 1.1.0 and newer. This enables X25519
-	  (since OpenSSL 1.1.0) and X448 (since OpenSSL 1.1.1) as a side-effect.
+	  If Asterisk is built using a DESTDIR this will cause the bundled jansson
+	  to be installed to an unexpected location and we will fail to find it.
 
-	  ASTERISK-27910
+	  Change-Id: Id033e2813261e0d45232383d44c6391122169548
 
-	  Change-Id: I5b0dd47c5194ee17f830f869d629d7ef212cf537
+2018-08-30 03:42 +0000 [ccfd2e0f5d]  Frederic LE FOLL <frederic.lefoll@c-s.fr>
 
-2018-06-08 06:01 +0000 [2c3ad1e40d]  Alexander Traud <pabstraud@compuserve.com>
+	* res_musiconhold.c: Restart MOH if previous hold just reached end-of-file
 
-	* crypto.h: Repair ./configure --with-ssl=PATH.
+	  On MOH activation, moh_files_readframe() is called while the current
+	  stream attached to the channel is NULL and it calls ast_moh_files_next()
+	  immediately.  However, it won't call ast_moh_files_next() again if sample
+	  reading fails.  The failure may occur because res_musiconhold retains the
+	  last sample reading position in the channel data and MOH during the
+	  previous hold/retrieve just reached EOF.  Obviously, a bit of bad luck is
+	  required here.
 
-	  ASTERISK-27908
+	  * Restructured moh_files_readframe() to try a second time to start MOH if
+	  there was no stream setup and the saved position was at EOF.  Also added
+	  comments describing what is going on for each step.
 
-	  Change-Id: Iac49d9f82faeb8a4611c6805906bd6d650b1b1d8
+	  ASTERISK-28029
 
-2018-06-08 04:06 +0000 [7763aa824e]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I1508cf2c094f8feca22d6f76deaa9fdfa9944860
 
-	* res_crypto: Allow OpenSSL configured with no-deprecated.
+2018-09-05 06:39 +0000 [af6a3d02e1]  Joshua Colp <jcolp@digium.com>
 
-	  The header <openssl/rsa.h> had to be included explicitly.
+	* core: Don't stop generators when writing RTCP frames.
 
-	  ASTERISK-27906
+	  Generators provide such functionality as tone generation or
+	  silence generation. RTCP frames provide RTCP information and
+	  should not stop generators from operating.
 
-	  Change-Id: I41743801eed998c039d73db7a0762d104a4f75b2
+	  ASTERISK-28005
 
-2018-06-08 02:41 +0000 [e62e069705]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ieadada07b068a7aa426e8763f1b73a18e1ac34a9
 
-	* res_srtp: Repair ./configure --with-ssl=PATH.
+2018-09-03 06:28 +0000 [034a3d8b86]  lvl <digium@lvlconsultancy.nl>
 
-	  ASTERISK-27905
+	* app_queue: Update realtime queuemembers after wait_a_bit(), not before
 
-	  Change-Id: Ibb7dc148a0048f4f9c3b12937ba4240dff0d15e2
+	  This ensures the most up-to-date information is used for the next
+	  call attempt.
 
-2018-05-31 10:25 +0000 [d9d50396cf]  Alexei Gradinari <alex2grad@gmail.com>
+	  ASTERISK-28032
 
-	* func_odbc: NODATA if SQLNumResultCols returned 0 columns on readsql
+	  Change-Id: I02fc17c6ffb50bb60ea97c2d2e6023e8061815ce
 
-	  The functions acf_odbc_read/cli_odbc_read ignore a number of columns
-	  returned by the SQLNumResultCols.
-	  If the number of columns is zero it means no data.
-	  In this case, a SQLFetch function has to be not called,
-	  because it will cause an error.
+2018-08-28 08:42 +0000 [3134fd95a9]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27888 #close
+	* res_pjproject: Add utility functions to convert between socket structures
 
-	  Change-Id: Ie0f7bdac6c405aa5bbd38932c7b831f90729ee19
+	  Currently, to convert from a pj_sockaddr to an ast_sockaddr, the address
+	  needs to be rendered to a string and then parsed into the correct
+	  structure. This also involves a call to getaddrinfo(3). The same is true
+	  for the inverse operation.
 
-2018-06-07 08:46 +0000 [98da1971e3]  George Joseph <gjoseph@digium.com>
+	  Instead, because we know the internal structure of both ast_sockaddr and
+	  pj_sockaddr, we can translate directly between the two without the
+	  need for an intermediate string.
 
-	* chan_pjsip:  Register for "BEFORE_MEDIA" responses
+	  Change-Id: If0fc4bba9643f755604c6ffbb0d7cc46020bc761
 
-	  chan_pjsip wasn't registering for "BEFORE_MEDIA" responses which meant
-	  it was not updating HANGUPCAUSE for 4XX responses.  If the remote end
-	  sent a "180 Ringing", then a "486 Busy", the hangup cause was left at
-	  "180 Normal Clearing".
+2018-08-30 13:08 +0000 [9fb166cf3b]  George Joseph <gjoseph@digium.com>
 
-	  * Removed chan_pjsip_incoming_response from the original session
-	    supplement (which was handling only "AFTER MEDIA") and added it to a
-	    new session supplement which accepts both "BEFORE_MEDIA" and
-	    "AFTER_MEDIA".
+	* stasis_cache: Prune stasis_subscription_change messages
 
-	  * Also cleaned up some cleanup code in load module.
+	  The stasis cache provides a way to reconstruct the current state
+	  of topic subscribers.  Unfortunately, since every subscribe and
+	  unsubscribe is cached, the cache continues to grow unabated while
+	  asterisk is running.  This patch removes subscribe messages from
+	  the cache when the corresponding unsubscribe is received.
 
-	  ASTERISK-27902
+	  This patch also registers the cache containers with ao2 so that if
+	  AO2_DEBUG is turned on, you can list the container and get its
+	  stats from the CLI.
 
-	  Change-Id: If9b860541887aca8ac2c9f2ed51ceb0550fb007a
+	  ASTERISK-27121
 
-2018-06-07 07:19 +0000 [1205bcb69f]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I3d18905e477f3721815da91f30da8d3fbb2d4f56
 
-	* ooh323c: GCC 8.1 warned about output truncated before terminating nul.
+2018-09-03 09:27 +0000 [8879a62c1c]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  ASTERISK-27901
+	* app_dial: set the comment for OPT_ARG_ANNOUNCE to really what is done
 
-	  Change-Id: I5a8e894f4924ef52e3094f6870656a559d67f3d7
+	  Change-Id: I08f88adb09f7e5813f37e70fecd787468cdb32c8
 
-2018-06-04 09:50 +0000 [db2413b446]  George Joseph <gjoseph@digium.com>
+2018-08-15 14:27 +0000 [cfb854e241]  Chris-Savinovich <csavinovich@digium.com>
 
-	* app_sendtext:  Allow content types other than text/plain
+	* pbx_config.c: Fix reloading module if initially declined to load
 
-	  There was no real reason to limit the conteny type to text/plain other
-	  than that's what it was limited to before.  Now any text/* content
-	  type will be allowed for channel drivers that don't support enhanced
-	  messaging and any type will be allowed for channel drivers that do
-	  support enhanced messaging.
+	  Added decline if extensions.conf file not available
+	  when loading pbx_config, and also made sure everything
+	  gets properly unregistered and/or destroyed on unload.
 
-	  Change-Id: I94a90cfee98b4bc8e22aa5c0b6afb7b862f979d9
+	  Change-Id: Ib00665106043b1be5148ffa7a477396038915854
 
-2018-05-30 01:12 +0000 [8b67e2bd14]  Pirmin Walthert <infos@nappsoft.ch>
+2018-08-30 14:42 +0000 [4fcdcfaa37]  Richard Mudgett <rmudgett@digium.com>
 
-	* bridge_channel.c: Fix Deadlock when using Local channels and fax gateway
+	* http.c: Give HTTP error response when received lines are too long.
 
-	  ast_indicate is invoked with the bridge locked. As ast_indicate locks the
-	  other end of the bridge as well this can lead to a deadlock in some situations.
-	  (Especially when a different thread does the same in the reverse order).
-	  This patch calls ast_indicate after unlocking the bridge which fixes the
-	  deadlock. Calling ast_indicate with these parameters without locking the
-	  bridge should be safe as this is done at different places without a
-	  bridge lock.
+	  Added a check when we receive a HTTP request line or header line that is
+	  too long.  We now return an error response to the sender because we are
+	  not able to process the request.
 
-	  ASTERISK-27094 #close
-	  Reported-by: David Brillert
+	  Change-Id: I6df2705435fd7dde4d5d3bdf7acec859cfb7c12d
 
-	  Change-Id: I5f86c1e2ce75b9929a36ab589b18c450e62ea35f
+2018-08-29 16:14 +0000 [f6a165208b]  Richard Mudgett <rmudgett@digium.com>
 
-2018-05-28 19:17 +0000 [9ff4779f03]  William McCall <william.mccall@gmail.com>
+	* iostream.c: Fix ast_iostream_gets() needlessly returning failure.
 
-	* app_confbridge: Add talking indicator for ConfBridgeList AMI response
+	  Providing a buffer larger than the internal buffer of ast_iostream_gets()
+	  fails to get lines longer than the internal buffer.
 
-	  When an AMI client connects, it cannot determine if a user was talking
-	  prior to a transition in the user speaking state (which would generate
-	  a ConfbridgeTalking event). This patch causes app_confbridge to track the
-	  talking state and make this state available via ConfBridgeList.
+	  * Made ast_iostream_gets() fill the supplied buffer with read data until
+	  either a '\n' is found or the supplied buffer is filled just like fgets().
 
-	  ASTERISK-27877 #close
+	  Change-Id: If18b3f6ee500e22f0633a68779ed09f7e0f305ed
 
-	  Change-Id: I19b5284f34966c3fda94f5b99a7e40e6b89767c6
+2018-08-06 15:37 +0000 [4dd8b5bbb4]  Richard Mudgett <rmudgett@digium.com>
 
-2018-05-29 12:28 +0000 [071232244a]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip: Fix mwi_subscribe_replaces_unsolicited type mismatch
 
-	* app_meetme: Fix manager event documentation for several events.
+	  ASTERISK-27988
 
-	  The MeetmeJoin, MeetmeLeave, MeetmeEnd, MeetmeMute, MeetmeTalking, and
-	  MeetmeTalkRequest AMI events were documented with sending out a Usernum
-	  header when the User header was actually output.
+	  Change-Id: Iccafdd0552ea8aaed647620fb14499f1bf341843
 
-	  * Change the online documentation to match reality.
+2018-08-16 09:04 +0000 [1edd9eb309]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  ASTERISK-27873
-	  ASTERISK-25261
+	* make config: os-release output error.
 
-	  Change-Id: I437bc70618d07c183c9624b7069c2fcae7f17a39
+	  Fix not show the error
+	  "/bin/sh: /etc/os-release: No such file or directory" when the command
+	  'make config' is run in a System without systemv.
 
-2018-05-28 10:32 +0000 [8c78337479]  Alexander Traud <pabstraud@compuserve.com>
+	  The instruction 'make config' pre execute the syntax
+	  "$(shell . /etc/os-release && echo $$ID)" to identified if system is a
+	  Slackware and Opensuse.
 
-	* tcptls.h: Repair ./configure --with-ssl=PATH.
+	  This change prevent show the message and is send to the /dev/null
 
-	  asterisk/tcptls.h was included (explicitly, implicitly, or transitively). Those
-	  inclusions got replaced by forward declarations. As side effect, the inclusions
-	  got completed.
+	  Change-Id: I7f43e281a8d9405b2519fc653de82d9b8b645fdf
 
-	  ASTERISK-27878
+2018-08-29 05:18 +0000 [390d0b42ca]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I9d102728e30336d6522e5e4ae9e964013a0835f7
+	* res_fax: Handle fax gateway being started more than once.
 
-2018-05-22 16:21 +0000 [9ad3918acd]  Alexei Gradinari <alex2grad@gmail.com>
+	  The T.38 fax gateway state machine can cause the fax gateway
+	  to be started more than once on a channel depending on the
+	  responses of the remote endpoint. This would previously leak
+	  the channel name, channel unique id, and underlying fax engine
+	  state. This change instead makes it so that if the fax gateway
+	  session is already present and not reserved the fax gateway
+	  is not started again.
 
-	* pjsip_options: handle modification of qualify options in realtime
+	  ASTERISK-27981
 
-	  Currentrly pjsip_options code does not handle the situation when the
-	  qualify options were changed in realtime database.
-	  Only 'module reload res_pjsip' helps.
+	  Change-Id: I552d95086860cb18f2522ee40ef47b13b6da2e0e
 
-	  This patch add a check on contact add/update observers if the contact
-	  qualify options are different than local aor qualify options.
-	  If the qualify options were modified then synchronize
-	  the pjsip_options AOR local state.
+2018-08-28 08:01 +0000 [245fb462d6]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27872
+	* res_pjsip_transport_websocket: Properly set src_name for IPv6
 
-	  Change-Id: Id55210a18e62ed5d35a88e408d5fe84a3c513c62
+	  SIP responses over WebSockets when the client is using IPv6 have invalid
+	  Via headers according to RFC 3261. The 'received' header parameter
+	  should not be wrapped in brackets if it is an IPv6 address.
 
-2018-05-25 09:55 +0000 [6833c763c7]  Alexander Traud <pabstraud@compuserve.com>
+	  When src_name is populated by the built-in PJSIP transports, the code
+	  uses pj_sockaddr_print() with 'flags' set to 0, meaning that the
+	  brackets are not rendered around IPv6 addresses.
 
-	* tcptls: Allow OpenSSL configured with no-dh.
+	  This may be related to ASTERISK~27101.
 
-	  Additionally, this change allows auto-negotiation of the elliptic curve/group
-	  for servers, not only with OpenSSL 1.0.2 but also with OpenSSL 1.1.0 and newer.
-	  This enables X25519 (since OpenSSL 1.1.0) and X448 (since OpenSSL 1.1.1) as a
-	  side-effect.
+	  See also: https://github.com/onsip/SIP.js/pull/594
 
-	  ASTERISK-27876
+	  ASTERISK-28020 #close
 
-	  Change-Id: I62c2aba4a630aefc231b71f646207e8c027d9497
+	  Change-Id: I8ea9d289901b837512bee2ca2535e3dc14f04d77
 
-2018-05-25 07:24 +0000 [204cc25a27]  Alexander Traud <pabstraud@compuserve.com>
+2018-08-26 13:18 +0000 [1b1f47bef6]  Corey Farrell <git@cfware.com>
 
-	* tcptls: Allow OpenSSL 1.1.x configured with enable-ssl3-method no-deprecated.
+	* Create --disable-binary-modules option.
 
-	  ASTERISK-27874
+	  This new option can be passed for ./configure or
+	  ./tests/CI/buildAsterisk.sh to prevent download/install of binary
+	  modules.
 
-	  Change-Id: Ica65113511c7a1c13f7988e7d9e7d9e7f3f620dd
+	  Normally enabling the categories MENUSELECT_CODECS or MENUSELECT_RES
+	  will result in binary modules being enabled even if the build target is
+	  incompatible with those modules.  This includes CI scripts which enable
+	  categories before disabling specific modules.
+
+	  If more binary modules are offered in the future this will help avoid
+	  accidentally downloading them if unwanted or incompatible.  Adding a
+	  binary module will only require creating a new menuselect entry similar
+	  to the existing ones, it will not be necessary to modify the CI scripts.
 
-2018-05-15 08:45 +0000 [acfdfcd19e]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I6b1bd1c75a2e48f05b8b8a45b7a7a2d00a079166
 
-	* ast_coredumper:  Fix output directory and variable precedence
+2018-08-21 07:59 +0000 [aa2755cbb3]  Emmanuel BUU <emmanuel.buu@ives.fr>
 
-	  The OUTPUTDIR variable in ast_debug_tools.conf.sample is now set
-	  to "/tmp" instead of "/some/directory".
+	* res/res_rtp_asterisk: remove debug traces generated by an empty frame
 
-	  Variables set on the command line or that are already in the
-	  environment now take predecence over variables set in the config files.
+	  The realtime text timer pops regularly and sends text frames even if
+	  the buffer is empty. This causes a lot of unecessary debug logging.
 
-	  ASTERISK-27846
-	  Reported by: Ted G
+	  * Made red_write() test if we need to send a frame before calling
+	  ast_rtp_write()
 
-	  Change-Id: Ie8baec52d531886bf5849ec1d59bb59dc87ad387
+	  ASTERISK-28002
+	  Reported by: Emmanuel BUU
+	  Tested by: Emmanuel BUU
 
-2018-05-09 08:31 +0000 [4b7fd478d5]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: Icf81310c3b8080b615a42060afc02ab41f9523dd
 
-	* res/res_rtp_asterisk: ensure marker bit is correctly set on ssrc change
+2018-08-13 08:12 +0000 [46442aa9e5]  Jaco Kroon <jaco@uls.co.za>
 
-	  Certain race conditions between changing bridge types and DTMF can
-	  cause the current FLAG_NEED_MARKER_BIT to send the marker bit before
-	  the actual first packet of native bridging.
+	* chan_sip: improved ip:port finding of peers for non-UDP transports.
 
-	  This logic keeps track of the ssrc the bridge is currently sending
-	  and will correctly ensure the marker bit is set if SSRC as changed
-	  from the previous sent packet.
+	  Also remove function peer_ipcmp_cb since it's not used (according to
+	  rmudgett).
 
-	  ASTERISK-27845
+	  Prior to b2c4e8660a9c89d07041271371151779b7ec75f6 (ASTERISK_27457)
+	  insecure=port was the defacto standard.  That commit also prevented
+	  insecure=port from being applied for sip/tcp or sip/tls.
 
-	  Change-Id: I01858bd0235f1e5e629e20de71b422b16f55759b
+	  Into consideration there are three sets of behaviour:
 
-2018-05-23 16:20 +0000 [a576f50362]  Alexei Gradinari <alex2grad@gmail.com>
+	  1.  "previous" - before the above commit.
+	  2.  "current" - post above commit, pre this one.
+	  3.  "new" - post this commit.
 
-	* pjsip_options: show/reload AOR qualify options using CLI
+	  The problem that the above commit tried to address was guests over TCP.
+	  It succeeded in doing that but broke transport!=udp with host!=dynamic.
 
-	  Currentrly pjsip_options code does not handle the situation when the
-	  AOR qualify options were changed.
+	  This commit attempts to restore sane behaviour with respect to
+	  transport!=udp for host!=dynamic whilst still retaining the guest users
+	  over tcp.
 
-	  Also there is no way to find out what qualify options are using.
+	  It should be noted that when looking for a peer, two passes are made, the
+	  first pass doesn't have SIP_INSECURE_PORT set for the searched-for peer,
+	  thus looking for full matches (IP + Port), the second pass sets
+	  SIP_INSECURE_PORT, thus expecting matches on IP only where the matched
+	  peer allows for that (in the author's opinion:  UDP with insecure=port,
+	  or any TCP based, non-dynamic host).
 
-	  This patch add CLI commands to show and synchronize Aor qualify options:
-	  pjsip show qualify endpoint <id>
-	      Show the current qualify options for all Aors on the PJSIP endpoint.
-	  pjsip show qualify aor <id>
-	      Show the PJSIP Aor current qualify options.
-	  pjsip reload qualify endpoint <id>
-	      Synchronize the qualify options for all Aors on the PJSIP endpoint.
-	  pjsip reload qualify aor <id>
-	      Synchronize the PJSIP Aor qualify options.
+	  In previous behaviour there was special handling for transport=tcp|tls
+	  whereby a peer would match during the first pass if the utilized
+	  transport was TCP|TLS (and the peer allowed that specific transport).
 
-	  ASTERISK-27872
+	  This behaviour was wrong, or dubious at best.  Consider two dynamic tcp
+	  peers, both registering from the same IP (NAT), in this case either peer
+	  could match for connections from an IP.  It's also this behaviour that
+	  prevented SIP guests over tcp.
 
-	  Change-Id: I1746d10ef2b7954f2293f2e606cdd7428068c38c
+	  The above referenced commit removed this behaviour, but kept applying
+	  the SIP_INSECURE_PORT only to WS|WSS|UDP.  Since WS and WSS is also TCP
+	  based, the logic here should fall into the TCP category.
 
-2018-05-22 17:17 +0000 [90a075221b]  Richard Mudgett <rmudgett@digium.com>
+	  This patch updates things such that the previously non-explicit (TCP
+	  behaviour) transport test gets performed explicitly (ie, matched peer
+	  must allow for the used transport), as well as the indeterministic
+	  source-port nature of the TCP protocol is taken into account.  The new
+	  match algorithm now looks like:
 
-	* channel.c: Fix off nominal channel allocation failure path.
+	  1.  As per previous behaviour, IP address is matched first.
 
-	  __ast_channel_alloc_ap() had a failure exit path that hadn't setup the fd
-	  descriptors to -1 yet.  The destructor would then attempt to close these
-	  fd's that had never been opened.
+	  2.  Explicit filter with respect to transport protocol, previous
+	      behaviour was semi-implied in the test for TCP pure IP match - this now
+	      made explicit.
 
-	  Change-Id: Icf21093f36c60781e8cf6ee9d586536302af33e3
+	  3.  During first pass (without SIP_INSECURE_PORT), always match on port.
 
-2018-05-18 16:45 +0000 [d6145087cf]  Alexei Gradinari <alex2grad@gmail.com>
+	  4.  If doing UDP, match if matched against peer also has
+	      SIP_INSECURE_PORT, else don't match.
 
-	* config.c: Fix successful DELETE treated as failure
+	  5.  Match if not a dynamic host (for non-UDP protocols)
 
-	  The config engine destroy_func callback function returns the number of
-	  rows deleted or -1 on error.  But the function
-	  ast_destroy_realtime_fields treated non-zero return values as error.
+	  6.  Don't match if this is WS|WSS, or we can't trust the Contact address
+	      (presumably due to NAT)
 
-	  ASTERISK-27863
+	  7.  Match (we have a valid Contact thus if the IP matches we have no
+	      choice, this will likely only apply to non-NAT).
 
-	  Change-Id: Ied02b38e8196cb03043e609a0679feebd288d17b
+	  To logic-test this we need a few different scenarios.  Towards this end,
+	  I work with a set number of peers defined in sip.conf:
 
-2018-05-20 06:41 +0000 [ec40bd945c]  Alexander Traud <pabstraud@compuserve.com>
+	  [peer1]
+	  host=1.1.1.1
+	  transport=tcp
 
-	* libasteriskssl: Allow OpenSSL 1.0.2 configured with no-deprecated.
+	  [peer2]
+	  host=1.1.1.1
+	  transport=udp
 
-	  Use CRYPTO_set_id_callback(.) only with OpenSSL 0.9.8 and older.
+	  [peer3]
+	  host=1.1.1.1
+	  port=5061
+	  insecure=port
+	  transport=udp
 
-	  ASTERISK-27867
+	  [peer4]
+	  host=1.1.1.2
+	  transport=udp,tcp
 
-	  Change-Id: Iadd58d5bf6f538eb224203970a4e88e26f259655
+	  [peer5]
+	  host=dynamic
+	  transport=udp,tcp
 
-2018-05-19 08:23 +0000 [b6234f9577]  Alexander Traud <pabstraud@compuserve.com>
+	  Test cases for UDP:
 
-	* tcptls: Repair ./configure --with-ssl=PATH.
+	  1 - incoming UDP request from 1.1.1.1:
+	    - previous:
+	      - pass 1:
+	        * peer1 or peer2 if from port 5060 (indeterminate, depends on peer
+	          ordering)
+	        * peer3 if from port 5061
+	        * peer5 if registered from 1.1.1.1 and source port matches
+	      - pass 2:
+	        * peer3
+	    - current: as per previous.
+	    - new:
+	      - pass 1:
+	        * peer2 if from port 5060
+	        * peer3 if from port 5061
+	        * peer5 if registered from 1.1.1.1 and source port matches
+	      - pass 2:
+	        * peer3
 
-	  SSL_OP_NO_TLSv1_1 and SSL_OP_NO_TLSv1_2 got discovered without honoring a PATH.
+	  2 - incoming UDP request from 1.1.1.2:
+	    - previous:
+	      - pass 1:
+	        * peer5 if registered from 1.1.1.2 and port matches
+	        * peer4 if source port is 5060
+	      - pass 2:
+	        * no match (guest)
+	    - current: as previous.
+	    - new as previous (with the variation that if peer5 didn't have udp as
+	            allowed transport it would not match peer5 whereas previous
+	            and current code could).
 
-	  ASTERISK-27865
+	  3 - incoming UDP request from anywhere else:
+	    - previous:
+	      - pass 1:
+	        * peer5 if registered from that address and source port matches.
+	      - pass 2:
+	        * peer5 if insecure=port is additionally set.
+	        * no match (guest)
+	    - current - as per previous
+	    - new - as per previous
 
-	  Change-Id: I8cd358eed7411726d08fa7b01691bef122fbeb71
+	  Test cases for TCP based transports:
 
-2018-05-12 06:53 +0000 [436d17fa50]  Nic Colledge <nic@njcolledge.net>
+	  4 - incoming TCP request from 1.1.1.1
+	    - previous:
+	      - pass 1 (indeterministic, depends on ordering of peers in memory):
+	        * peer1; or
+	        * peer5 if peer5 registered from 1.1.1.1 (irrespective of source port); or
+	        * peer2 if the source port happens to be 5060; or
+	        * peer3 if the source port happens to be 5061.
+	      - pass 2: cannot happen since pass 1 will always find a peer.
+	    - current:
+	      - pass 1:
+	        * peer1 or peer2 if from source port 5060
+	        * peer3 if from source port 5060
+	        * peer5 if registered as 1.1.1.1 and source port matches
+	      - pass 2:
+	        * no match (guest)
+	    - new:
+	      - pass 1:
+	        * peer 1 if from port 5060
+	        * peer 5 if registered and source port matches
+	      - pass 2:
+	        * peer 1
 
-	* app_voicemail: Fix incorrect msg leaving/retrieving an ODBC voicemail
+	  5 - incoming TCP request from 1.1.1.2
+	    - previous (indeterminate, depends on ordering):
+	      - pass 1:
+	        * peer4; or
+	        * peer5 if peer5 registered from 1.1.1.2
+	      - pass 2: cannot happen since pass 1 will always find a peer.
+	    - current:
+	      - pass 1:
+	        * peer4 if source port is 5060
+	        * peer5 if peer5 registered as 1.1.1.2 and source port matches
+	      - pass 2:
+	        * no match (guest).
+	    - new:
+	      - pass 1:
+	        * peer4 if source port is 5060
+	        * peer5 if peer5 registered as 1.1.1.2 and source port matches
+	      - pass 2:
+	        * peer4
 
-	  Correct the log warning message shown when ODBC voicemail
-	  retrieve_file is called and there is a null value in the category
-	  column.
-	  A more meaningfull message is now written at debug level.
+	  6 - incoming TCP request from anywhere else:
+	    - previous:
+	      - pass 1:
+	        * peer5 if registered from that address
+	      - pass 2: cannot happen since pass 1 will always find a peer.
+	    - current:
+	      - pass 1:
+	        * peer5 if registered from that address and port matches.
+	      - pass 2:
+	        * no match (guest)
+	    - new: as per current.
 
-	  ASTERISK-27853
+	  It should be noted the test cases don't make explicit mention of TLS, WS
+	  or WSS.  WS and WSS previously followed UDP semantics, they will now
+	  enforce source port matching.  TLS follow TCP semantics.
 
-	  Change-Id: Ic36e97d5eb070a23a12ba45972f6b53e2182a3f4
+	  The previous commit specifically tried to address test-case 6, but broke
+	  test-cases 4 and 5 in the process.
 
-2018-04-17 21:15 +0000 [8c1202beb9]  Brian P. Martin <asterisk-forum@silverflash.net>
+	  ASTERISK-27881 #close
 
-	* chan_mobile: support handling of caller-id names ("cnam").
+	  Change-Id: I61a9804e4feba9c7224c481f7a10bf7eb7c7f2a2
 
-	  Add support to handle caller-ID names ("cnam") in addition to caller-ID
-	  numbers.  The prior code ignored the caller-ID name altogether, and
-	  used the local name for the cell phone (e.g. "my-iphone") in its place.
+2018-08-20 07:23 +0000 [d84de695ed]  Jaco Kroon <jaco@uls.co.za>
 
-	  Note: as of this writing, at least some Android phones don't pass cnam to
-	  us. This can be seen by issuing "core set debug 2" in the CLI and watching
-	  the "CLIP" record when a call comes in.  If cnam isn't in the CLIP record,
-	  there's nothing we can do to provide one.  We'll provide a null cnam field,
-	  so later Asterisk processes know to try other sources (e.g. cidname database,
-	  OpenCNAM, etc.).
+	* AMI: be less verbose when adding HTTP headers to AMI/HTTP messages.
 
-	  Reported by: Brian Martin
-	  Tested by: Brian Martin
-	  ASTERISK-27726
+	  All HTTP/AMI message headers are being sent to the verbose channel.
+	  There are multiple places this is happening.  Consolidate the loop into
+	  a function.  Drop the debug/verbose message.
 
-	  Change-Id: I89490d85fa406c36261879c50ae5e65595538ba5
+	  Convert to using ast_asprintf to perform the length calculation, memory
+	  allocation and snprintf all in one step.
 
-2018-05-17 01:58 +0000 [b615df06d3]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ic45e673fde05bd544be95ad5cdbc69518207c1a1
 
-	* res_pjsip_endpoint_identifier_ip: Unregister the module for headers.
+2018-08-22 10:50 +0000 [4188e7d6dd]  Matthew Fredrickson <creslin@digium.com>
 
-	  Asterisk uses Reference Counting to track whether a module can be unloaded.
-	  Every consumer who requires a module, increases the reference count. When the
-	  consumer goes, is unloaded itself, it has to decrease the reference count on
-	  all its used/required modules. That way
-	   core stop gracefully
-	  works on the command-line interface (CLI): One module after the other is
-	  unloaded. A recent change broke this for the module res_pjsip.
+	* sample_configs: noload res_hep.so by default
 
-	  ASTERISK-27861
+	  Change disables loading of res_hep.so in default installation.  Loading
+	  res_hep has a performance impact whether it's used or not.  This disables
+	  loading of it in sample config files.
 
-	  Change-Id: I261abcb411d026bbb0691cc78f28300bfd3103a3
+	  Change-Id: I5ec150cf941634fabc72973e5bf1a965cb0ef9d0
+	  (cherry picked from commit c8bacd45f17d2ae6f6d05c8cff538dd651f683a6)
 
-2018-05-17 00:34 +0000 [99b24dc63b]  Alexander Traud <pabstraud@compuserve.com>
+2018-08-23 06:57 +0000 [595e358761]  Florian Floimair <f.floimair@commend.com>
 
-	* res_pjsip: Register pjsip_transport_management not externally but internally.
+	* alembic: increase uri column size
 
-	  The module (res_)pjsip_transport_management got moved into res_pjsip. It is no
-	  longer an independent/external module with (un)load_module and therefore has to
-	  register just internally with res_pjsip.
+	  When mobile SIP clients register with Asterisk that use some sort of
+	  push notifications, the URI can get quite lengthy due to the
+	  additional push-service annotations (things like tokens, pn-type, etc.)
+	  contained in it.
 
-	  ASTERISK-27860
+	  ASTERISK-28022 #close
 
-	  Change-Id: Icd0413be7d2e98b92f51e6d6c353f2570bb4be95
+	  Change-Id: I6c55013bafe79f7e7a1fb6722d2558f553709f2e
 
-2018-03-27 18:53 +0000 [36f08075da]  Nic Colledge <nic@njcolledge.net>
+2018-08-21 13:50 +0000 [4b88cb383d]  Sean Bright <sean.bright@gmail.com>
 
-	* app_voicemail: Fix data-type mismatch between app_voicemail and database
+	* app_queue: Silence GCC 8 compiler warning
 
-	  Fix data-type mismatch between app_voicemail and database columns
-	  exposed by new version of MariaDB
+	  I'm only seeing an error in 14+, so I assume it is due to different
+	  compiler options:
 
-	  ASTERISK-27760
+	  app_queue.c: In function ‘handle_queue_add_member’:
+	  app_queue.c:10234:19: error: ‘%d’ directive writing between 1 and 11
+	      bytes into a region of size 3 [-Werror=format-overflow=]
+	       sprintf(num, "%d", state);
+	                     ^~
+	  app_queue.c:10234:18: note: directive argument in the range
+	      [-2147483648, 99]
+	       sprintf(num, "%d", state);
+	                    ^~~~
 
-	  Change-Id: I8543ad480a08c98be78bde1ee870e6e6c84b2c5b
+	  Compiler: gcc version 8.0.1 20180414 (experimental)
+	      [trunk revision 259383] (Ubuntu 8-20180414-1ubuntu2) 
 
-2018-05-11 12:49 +0000 [308a967470]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I18577590da46829c1ea7d8b82e41d69f105baa10
 
-	* rtp_engine: Remove the double assigned RTP payload ID of H.263+.
+2018-08-20 11:23 +0000 [fb2f242c49]  Richard Mudgett <rmudgett@digium.com>
 
-	  Mantis-3709 (Commit 68ff3c3, Asterisk 1.2) added support for the video format
-	  H.263+. For this, the RTP payload ID 103 got assigned statically. Commit f1aadc8
-	  assigned another payload ID 98 for this format in Asterisk 1.6.
+	* AMI: Remove docs for nonexistent AMI ContactStatus event headers
 
-	  Change-Id: I90e35b158487f8f1f8187da6241b54cd3b74e667
+	  Change-Id: I5736965c64c44338f7330e85a24bb46818607f19
 
-2018-05-11 12:26 +0000 [67212eae87]  Corey Farrell <git@cfware.com>
+2018-08-06 06:22 +0000 [b9cd4c6d92]  Joshua Colp <jcolp@digium.com>
 
-	* cli: Display correct unit for HTTP timeout in "manager show settings".
+	* res_pjsip: Reduce processing when a Contact is updated.
 
-	  HTTP timeout is in seconds, not minutes.
+	  When a Contact is updated the only material change that qualify
+	  support cares about is the underlying configuration for the AOR.
+	  In this case we will update things with the new AOR information but
+	  otherwise the callback to indicate the Contact has changed can be
+	  ignored.
 
-	  ASTERISK-27852 #close
+	  This is because it is only when a Contact is added or deleted that
+	  material changes occur within the qualify support. An update can't
+	  change the URI since it would result in a new Contact so it can be
+	  ignored.
 
-	  Change-Id: Ie6640835cb07307555741f9b559c2eb876d9343e
+	  Change-Id: I2f97ebfa79969a36a97bb7b9afd5b6268cf1a07d
 
-2018-05-11 10:37 +0000 [9fe4f99cba]  Alexander Traud <pabstraud@compuserve.com>
+2018-08-10 19:28 +0000 [236826a111]  Richard Mudgett <rmudgett@digium.com>
 
-	* rtp_engine: Avoid a typo error in Doxygen for ast_rtp_codecs_find_payload_code.
+	* res_pjsip_t38.c: Fix crash if already saw a final T.38 reINVITE response.
 
-	  Change-Id: Ica089d4507a27ddfc4ce3a88d697ffbef378de48
+	  We were still getting crashes after the first fix.  Somehow we receive a
+	  non-2xx final response before we get a 200 final response.  With the
+	  failure response we had already cleaned up and destroyed some data
+	  structures.  When the unexpected 200 response comes in we crash.
 
-2018-05-07 10:49 +0000 [d893e57c90]  Corey Farrell <git@cfware.com>
+	  * Add protection code to prevent processing another final T.38 reINVITE
+	  response.
 
-	* Fix GCC 8 build issues.
+	  ASTERISK-27944
 
-	  This fixes build warnings found by GCC 8.  In some cases format
-	  truncation is intentional so the warning is just suppressed.
+	  Change-Id: I8b5baba8d07fe4d63f0d7d05d3eb9a3d27d40a74
 
-	  ASTERISK-27824 #close
+2018-08-09 18:46 +0000 [19298141cf]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I724f146cbddba8b86619d4c4a9931ee877995c84
+	* res_sorcery_realtime.c: Fix unqualified fetch warning.
 
-2018-05-04 16:07 +0000 [316efcddb9]  Matthew Fredrickson <creslin@digium.com>
+	  The allow_unqualified_fetch option for the sorcery realtime backend
+	  blocked actually fetching all rows when the option is set to warn.
 
-	* res_hep: Adds hostname resolution support for capture_address
+	  * Made issue a warning and actually do the request when
+	  allow_unqualified_fetch=warn is set.
 
-	  Previously, only an IP address would be accepted for the capture_address config
-	  setting in hep.conf.  This change allows capture_address to be a resolvable
-	  hostname or an IP address.
+	  Change-Id: I74456c80a03a62dce66fc3dc3cb0cf2351ac4312
 
-	  ASTERISK-27796 #close
-	  Reported-By: Sebastian Gutierrez
+2018-06-11 00:07 +0000 [f6c1d94d91]  Kirsty Tyerman <ktyerman@barrukka.local>
 
-	  Change-Id: I33e1a37a8b86e20505dadeda760b861a9ef51f6f
+	* pbx_dundi: Added IPv6 support for dundi
 
-2018-05-09 09:30 +0000 [19ebad0d30]  Corey Farrell <git@cfware.com>
+	  Change includes move to netsock2 library.
 
-	* git: Ignore *.orig.
+	  ASTERISK-27164
+	  Reported-by: Adam Secombe
 
-	  This prevents accidental commit of files created by patch.
+	  Change-Id: Ia9e8dc3d153de7a291dbda4bd87fc827dd2bb846
 
-	  Change-Id: I68380db61f0f9d620046f719ccd978811d0e9964
+2018-08-15 21:31 +0000 [4e43823d19]  Richard Mudgett <rmudgett@digium.com>
 
-2018-04-18 02:27 +0000 [c6a4d34b7f]  Alexander Traud <pabstraud@compuserve.com>
+	* pbx_dundi.c: Misc memory management fixes when destroying peers
 
-	* sip_to_pjsip: Enable python3 compatibility.
+	  * In destroy_peer(), fixed memory leaks of lookup history strings and
+	  qualify transactions when destroying peers.
 
-	  The script remains compatible with Python 2.7 but now also works with
-	  Python 3.3 and newer; to ease the migration from chan_sip to chan_pjsip.
+	  * In destroy_peer(), fixed leaving the registerexpire scheduled callback
+	  active when a peer is destroyed on a reload.  The reload marks and sweeps
+	  peers so any peers not explicitly configured get destroyed.  Peers created
+	  dynamically from the '*' peer will not exist until they re-register after
+	  the reload.  These destroyed peers caused memory corruption when the
+	  registerexpire timer expired.
 
-	  ASTERISK-27811
+	  * Made build_peer() not schedule any callbacks on the '*' peer
+	  (empty_eid).  It is a special peer that is cloned to dynamically created
+	  peers so it doesn't actually get involved in any message transactions.
 
-	  Change-Id: I59cc6b52a1a89777eebcf25b3023bdf93babf835
+	  * Made do_register_expire() remove the dundi/dpeers AstDB entry when a
+	  peer registration expires.
 
-2018-05-08 14:28 +0000 [ba6034db48]  Corey Farrell <git@cfware.com>
+	  * Fix deep_copy_peer() to not copy some things that cannot be copied to
+	  the cloned peer structure.  Timers, message transactions, and lookup
+	  history are specific to a peer instance.
 
-	* makeopts.in: Remove unused/undefined AST_MARCH_NATIVE.
+	  * Made set_config() lock around processing the mappings configuration.
 
-	  Change-Id: I617a96ebb83ec99f5d3176bbbee2d2a272ccb203
+	  * Reordered unload_module() to handle load_module() declining the load due
+	  to error.
 
-2018-05-08 04:59 +0000 [47548ff485]  Jaco Kroon <jaco@uls.co.za>
+	  Change-Id: Ib846b2b60d027f3a2c2b3b563d9a83a357dce1d6
 
-	* manager: fix digest auth for ami/http mechanism.
+2018-08-15 23:49 +0000 [857926d197]  Richard Mudgett <rmudgett@digium.com>
 
-	  Due to a fixed size buffer the digest authentication could be
-	  incorrectly calculated if a large URI was provided, causing
-	  authentication failure. The buffer is now dynamically allocated to allow
-	  any size URI within the normal limits of the HTTP request size.
+	* pbx_dundi.c: Handle thread shutdown better.
 
-	  ASTERISK-27841
+	  Change-Id: Id52f99bd6a948fe6dd82acc0a28b2447a224fe87
 
-	  Change-Id: I660609db13b8f9e5f9567f339dd804f4985d41b3
+2018-08-15 18:14 +0000 [111e73678e]  Richard Mudgett <rmudgett@digium.com>
 
-2018-05-04 13:47 +0000 [5dffdf79d1]  Corey Farrell <git@cfware.com>
+	* pbx_dundi: Fix debug frame decode string.
 
-	* app_macro: Prevent infinite loop in find_matching_priority.
+	  * Fixed a typo in the name of the REGREQ frame decode string array.
+	  * Fixed off by one range check indexing into the frame decode string
+	  array.
+	  * Removed some unneeded casts associated with the decode string array.
 
-	  Use AST_PBX_MAX_STACK to escape if we recurse 128 times.  This will
-	  prevent crash if dialplan contains an include loop.  Log an error when
-	  this occurs, at most one message per call to Macro() so we avoid logger
-	  spam.
+	  Change-Id: I77435e81cd284bab6209d545919bf236ad7933c2
 
-	  ASTERISK-26570 #close
+2018-08-16 16:21 +0000 [a7ace944ae]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I6c71b76998c31434391b150de055ae9a531e31da
+	* pbx_dundi: Update sample config documentation.
 
-2018-05-03 06:34 +0000 [a36d6e3674]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I33d0ad0611c2124ca3440f0f811fa0f45e4e2849
 
-	* res_ari: Remove requirement that body exists when debug is on.
+2018-08-15 14:44 +0000 [0874d5b316]  Richard Mudgett <rmudgett@digium.com>
 
-	  The "ari set debug" code for incoming requests incorrectly assumed
-	  that all requests would contain a body. If one did not exist the
-	  request would be incorrectly rejected. The response that was sent
-	  was also incomplete as an incorrect function was used to construct
-	  the response.
+	* res_rtp_asterisk.c: Fix unused variable warnings
 
-	  The code has now been changed to no longer require a request to have
-	  a body and the response updated to use the correct function.
+	  Compiling without SRTP support installed resulted in some unused variable
+	  warnings.  These warnings also showed that the srtp variable was obtained
+	  and passed around some functions but not really used even when a system
+	  has SRTP installed.
 
-	  ASTERISK-27801
+	  Change-Id: I6daad34be3e89b19adef6e2fbe738018975155fc
 
-	  Change-Id: I4eef036ad54550a4368118cc348765ecac25e0f8
+2018-08-16 13:51 +0000 [df78227e4a]  George Joseph <gjoseph@digium.com>
 
-2018-05-02 07:43 +0000 [850e554793]  Sean Bright <sean.bright@gmail.com>
+	* CI: Fixup for non-13 branches
 
-	* pjsip: Increase maximum number of usable ciphers & other cleanups
+	  Change-Id: I5e1d4a09e58b92b541bc8ed6f9e10e54c4e5101f
 
-	  * Increase maximum number of ciphers from 100 to 256 (or whatever
-	    PJ_SSL_SOCK_MAX_CIPHERS is #define'd to)
+2018-08-16 13:28 +0000 [4ca959c72c]  George Joseph <gjoseph@digium.com>
 
-	  * Simplify logic in cipher_name_to_id()
+	* CI:  Final version of setting correct gerrit creds
 
-	  * Make signed/unsigned comparison consistent
+	  Change-Id: I7729ecceedceb12f52bf18dae259846aa1d993b3
 
-	  Re: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=897412
+2018-08-16 12:08 +0000 [33a5b8ccaf]  George Joseph <gjoseph@digium.com>
 
-	  Reported by: Ondřej Holas
+	* CI:  Add https credentials to gerrit checkouts
 
-	  Change-Id: Iea620f03915a1b873e79743154255c3148a514e7
+	  If the review to be tested is in a project with restricted access,
+	  we need to use the jenkins user's gerrit https credentials when we
+	  do the checkout or the checkout will fail.
 
-2018-04-30 17:24 +0000 [51a6a986f1]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I9dc9994763c5ebfeb9f1cff60fb53f6902b7fd5f
 
-	* res_pjsip/pjsip_distributor.c: Add missing off-nominal request response.
+2018-08-09 02:34 +0000 [0d4bde84d1]  Torrey Searle <torrey@voxbone.com>
 
-	  Change-Id: I389579b39c523d1d1e8ce020ef549a8bb5781c9b
+	* res/res_pjsip_sdp_rtp:  put rtcp-mux in answer only if offered
 
-2018-04-30 17:20 +0000 [3663968705]  Richard Mudgett <rmudgett@digium.com>
+	  If in the initial sdp the caller doesn't include the line
+	  a=rtcp-mux
 
-	* res_pjsip/pjsip_distributor.c: Pull some assignments out of if tests.
+	  Then asterisk shoud not include rtcp-mux in the response regardless
+	  of rtcp-mux being enabled on the endpoint
 
-	  Change-Id: I3d30d638b53a4bbe9bf9aad853c649d583894112
+	  ASTERISK-28007 #close
 
-2017-12-11 12:34 +0000 [bea52b3706]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I58e9b9f40a139afc0da5de41906cc608fb62adc7
 
-	* pjsip: Rewrite OPTIONS support with new eyes.
+2018-08-15 14:49 +0000 [167efe3a47]  Corey Farrell <git@cfware.com>
 
-	  The OPTIONS support in PJSIP has organically grown, like many things in
-	  Asterisk.  It has been tweaked, changed, and adapted based on situations
-	  run into.  Unfortunately this has taken its toll.  Configuration file
-	  based objects have poor performance and even dynamic ones aren't that
-	  great.
+	* res_resolver_unbound: Fix leak of config nameserver strings.
 
-	  This change scraps the existing code and starts fresh with new eyes.  It
-	  leverages all of the APIs made available such as sorcery observers and
-	  serializers to provide a better implementation.
+	  Change-Id: I3f396316bb40d1ae6e91f5f688042420f1a540ed
 
-	  1.  The state of contacts, AORs, and endpoints relevant to the qualify
-	  process is maintained.  This state can be updated by external forces (such
-	  as a device registering/unregistering) and also the reload process.  This
-	  state also includes the association between endpoints and AORs.
+2018-08-15 13:51 +0000 [72dbc9fb70]  Corey Farrell <git@cfware.com>
 
-	  2.  AORs are scheduled and not contacts.  This reduces the amount of work
-	  spent juggling scheduled items.
+	* res_pjsip: Resolve transport management leak at shutdown.
 
-	  3.  Manipulation of which AORs are being qualified and the endpoint states
-	  all occur within a serializer to reduce the conflict that can occur with
-	  multiple threads attempting to modify things.
+	  Cleanup idle check scheduled events at shutdown.
 
-	  4.  Operations regarding an AOR use a serializer specific to that AOR.
+	  Change-Id: I61bfbb56bac69fe840c3242927d31ff3593be461
 
-	  5.  AORs and endpoint state act as state compositors.  They take input
-	  from lower level objects (contacts feed AORs, AORs feed endpoint state)
-	  and determine if a sufficient enough change has occurred to be fed further
-	  up the chain.
+2018-08-15 11:12 +0000 [6e0f4a2127]  Corey Farrell <git@cfware.com>
 
-	  6.  Realtime is supported by using observers to know when a contact has
-	  been registered.  If state does not exist for the associated AOR then it
-	  is retrieved and becomes active as appropriate.
+	* res_pjsip: Fix leak in pjsip_options.
 
-	  The end result of all of this is best shown with a configuration file of
-	  3000 endpoints each with an AOR that has a static contact.  In the old
-	  code it would take over a minute to load and use all 8 of my cores.  This
-	  new code takes 2-3 seconds and barely touches the CPU even while dealing
-	  with all of the OPTIONS requests.
+	  sip_options_get_endpoint_state_compositor_state leaked a reference to
+	  the first available endpoint state compositor that was found.
 
-	  ASTERISK-26806
+	  Change-Id: Idb6be19f7219b6eed1dfb19c1e740dd40cb3fdc7
 
-	  Change-Id: I6a5ebbfca9001dfe933eaeac4d3babd8d2e6f082
+2018-08-15 11:31 +0000 [b370482786]  Corey Farrell <git@cfware.com>
 
-2018-04-18 15:59 +0000 [63c68d023c]  Kevin Harwell <kharwell@digium.com>
+	* res_odbc: Allow unload at shutdown.
 
-	* translate: generic plc not filled in after translation
+	  This makes it possible for REF_DEBUG to report no leaks when loading
+	  res_odbc.
 
-	  If during translation a codec could not handle a given frame the translation
-	  core would return NULL, thus not passing along the "missing" frame. Due to this
-	  there was no frame to apply generic plc to, thus rendering it useless.
+	  Change-Id: I1a3dea786bd6e7f4820a6dd5cbaa197fa783ce93
 
-	  This patch makes it so the translation core produces an interpolated slin frame
-	  in the cases where an attempt was made to translate to slin, but failed. This
-	  interpolated frame is then passed along and can be used by the generic plc
-	  algorithms to fill in the frame.
+2018-07-31 23:54 +0000 [d61a8791b5]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27814 #close
+	* Build System: Improve ccache matching for different menuselect options.
 
-	  Change-Id: I133d084da87adef913bf2ecc9c9240e3eaf4f40a
+	  Changing any Menuselect option in the `Compiler Flags` section causes a
+	  full rebuild of the Asterisk source tree.  Every enabled option causes
+	  a #define to be added to buildopts.h, thus breaking ccache caching for
+	  every source file that includes "asterisk.h".  In most cases each option
+	  only applies to one or two files.  Now we only define those options for
+	  the specific sources which use them, this causes much better cache
+	  matching when working with multiple builds.  For example testing code
+	  with an without MALLOC_DEBUG will now use just over half the ccache
+	  size, only main/astmm.o will have two builds cached instead of every
+	  file.
 
-2018-04-20 14:07 +0000 [1593ff91fe]  Alexander Anikin <may213@yandex.ru>
+	  Reorder main/Makefile so _ASTCFLAGS set on specific object files are all
+	  together, sorted by filename.  Stop adding -DMALLOC_DEBUG to CFLAGS of
+	  bundled pjproject, this define is no longer used by any header so only
+	  serves to break cache.
+
+	  The only code change is a slight adjustment to how main/astmm.c is
+	  initialized.  Initialization functions always exist so main/asterisk.c
+	  can call them unconditionally.  Additionally rename the astmm
+	  initialization functions so they are not exported.
 
-	* chan_ooh323: fix ooManualProgress/ooManualRingback on ooh323 debuggin on
+	  Change-Id: Ie2085237a964f6e1e6fff55ed046e2afff83c027
 
-	  Call ooManualProgress/Ringback outside of ast_debug function
-	  when ooh323 debugging is on
+2018-08-14 11:55 +0000 [0e4f5f5189]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27812 #close
-	  ASTERISK-26893 #close
-	  Reported by: Dimos, Marco Giordani
+	* contrib/scripts: Make astgenkey executable
 
-	  Change-Id: I5873762e4f05824e7b6e94a19dd4eb56adbbbb79
+	  Change-Id: I11641d65592536dea9cbca5aa94a24c25d24dd5f
 
-2018-04-20 07:13 +0000 [94c8b20f5f]  Alexander Traud <pabstraud@compuserve.com>
+2018-08-14 07:29 +0000 [56c1285b8a]  Joshua Colp <jcolp@digium.com>
 
-	* BuildSystem: Enable IMAP storage on FreeBSD and DragonFly BSD.
+	* res_pjsip_caller_id: Add "party" parameter to RPID header.
 
-	  ASTERISK-27639
+	  This change adds the "party" parameter to the Remote-Party-ID header
+	  which indicates which party the header information is applicable
+	  to. In Asterisk this is determined on whether we are the calling
+	  or called party. This is added to improve interoperability with some
+	  implementations.
 
-	  Change-Id: I1347f3f2f3737010d0a80a5c30b5aaf71cf3ccb0
+	  ASTERISK-28006
 
-2018-04-20 05:54 +0000 [7b219311eb]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I1eec3e377ffff8633b5c1dd59a05e9533122cfca
 
-	* BuildSystem: Add DragonFly BSD.
+2018-07-19 22:28 +0000 [3bf968470b]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27820
+	* CI: Add support for coverage processing.
 
-	  Change-Id: I310896143e94d65da1c2be3bb448204a8b86d557
+	  Enable coverage with `./tests/CI/buildAsterisk.sh --coverage`.  This
+	  will cause Asterisk to be compiled with coverage support.  It also
+	  initializes 'before' coverage data for all sources.  Accept
+	  --tested-only to disable modules which are not run by any test.
+	  Enabling coverage also sets tested-only true by default.  To build
+	  everything with coverage enabled use `--coverage --tested-only=0`.
 
-2018-04-20 05:40 +0000 [00711bce99]  Alexander Traud <pabstraud@compuserve.com>
+	  ./tests/CI/processCoverage.sh is used to process the coverage and
+	  generate HTML reports.
 
-	* menuselect: Add DragonFly BSD.
+	  Fix utils/check_expr2 which failed to compiled with coverage enabled.
 
-	  In DragonFly BSD, added libraries from ports are placed into /usr/local.
-	  Therefore, this directory must be added for the preprocessor, compiler, and
-	  linker.
+	  Add status output 5 times per stage of astobj2_test_perf to ensure
+	  remote CLI does not timeout when compiled with coverage.  Remote CLI
+	  disconnects if no output is received for 60 seconds.  When coverage is
+	  enabled it takes about 70 seconds for my laptop to run the stages of
+	  this test, so with the change a message is printed every 14 seconds.
 
-	  Beside that, the script ./configure was updated:
-	  * OSARCH list was outdated and not used, removed.
-	  * AC_CANONICAL_BUILD was not used.
-	  * _REENTRANT, this feature test macro is obsolete.
+	  Change-Id: I890f7d5665087426ad7d3e363187691b9afc2222
 
-	  ASTERISK-27820
+2018-08-07 10:57 +0000 [a46fcaca7b]  Ben Ford <bford@digium.com>
 
-	  Change-Id: I186d88d99cfa4de6569888e12ac97bd2f441c422
+	* res_pjsip/rtp: No joint capabilities between streams.
 
-2018-04-20 05:23 +0000 [866b884eab]  Alexander Traud <pabstraud@compuserve.com>
+	  When a conference contained a mixture of audio/video and audio-only
+	  users, a NOTICE message would pop up stating there are no joint
+	  capabilities between streams. This happens because streams can never be
+	  removed, but they can be in a REMOVED state. If we have the scenario
+	  where user A joins with audio/video, user B joins with audio-only, and
+	  user C joins with audio/video, then user A leaves, the message would
+	  be triggered. That removed stream is still in the SDP, but Asterisk
+	  would pass it through, causing it to be seen as a ulaw stream. A check
+	  has been added for removed streams, setting their status to REMOVED when
+	  handling negotiated SDPs.
+
+	  Also addressed an issue where user A joins, then user B joins but does
+	  not receive video until much later. Full frames were not being sent,
+	  causing some PLI from the browser. Because the video was flowing in one
+	  direction, the browser sets the SSRC to 1, but Asterisk was dropping the
+	  PLI because of that. Added a check to see if the SSRC is 1 or not, which
+	  sends full frames and allows video to flow between user A and user B.
+	  This should only happen when dealing with PSFB or FUR, and in the case
+	  of PSFB, only for PLI.
 
-	* install_prereq: Add DragonFly BSD.
+	  ASTERISK-27398
 
-	  ASTERISK-27820
+	  Change-Id: I26e7c6f101bc119549eeca406b5bcd25ad8ebc5e
 
-	  Change-Id: I718ddb000fe5184b1bdc7759da67a370a7520144
+2018-08-12 11:04 +0000 [fce68efbbe]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-2018-04-18 16:24 +0000 [ea14243971]  Corey Farrell <git@cfware.com>
+	* app_queue: set QUEUESTATUS to LEAVEEMPTY instead of CONTINUE
 
-	* res_pjsip: Fix initialization of extended stringfields.
+	  When a call leaves a queue on leaveempty condition, QUEUESTATUS
+	  must be set to LEAVEEMPTY, no matter whether Queue was executed with or
+	  without the "c" (continue) option.
 
-	  It is possible for initialization of extended stringfields to fail.  Add
-	  checks for this failure.
+	  The regression was introduced in the fix for ASTERISK_25665.
+	  The following fix (ASTERISK_27065) was incomplete, as QUEUESTATUS was
+	  overwritten in case when "c" is set, regardless of what was the cause
+	  for leaving the queue.
 
-	  Change-Id: I062e09852db3d37ceefaf6c2048958fa0118304f
+	  ASTERISK-27973 #close
+	  Reported-by: Valentin Safonov
 
-2018-04-18 13:40 +0000 [7b05184bca]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: Iec013fe6a26a4e825ca572a1dda4f3cee5f6f80c
 
-	* format_pcm: Correct behavior of fseek and ftell for G.722
+2018-08-09 15:25 +0000 [661273a078]  Corey Farrell <git@cfware.com>
 
-	  There are twice as many samples in the same number of bytes, so redefine
-	  some of the G.722 format functions in terms of their PCM counterparts.
+	* Sample configs: Fix pjsip.conf syntax error.
 
-	  Change-Id: I6a8c7352624b930a5f2d9e4857f75283fa5dd9f9
+	  It is valid for a config file to be empty or contain only comments, but
+	  not valid for a config value to be set when no uncommented context
+	  exists.  This caused an error to be loged numerous times during start
+	  when loading the default pjsip.conf.
 
-2018-04-18 05:32 +0000 [8611135e96]  Alexander Anikin <may213@yandex.ru>
+	  Change-Id: Icf3b0d69b4ecb6e935eecd43c99ed8b32a5a1cf6
 
-	* chan_ooh323: Fix cppcheck warnings
+2018-08-06 12:19 +0000 [eb87a98a1a]  Richard Mudgett <rmudgett@digium.com>
 
-	  Fix cppcheck warnings about redundant conditions and possible
-	  null pointer usage
+	* res_pjsip.h: Fix doxygen comments.
 
-	  ASTERISK-27793 #close
-	  Reported by: Ilya Shipitsin
-	  Tested by: Ilya Shipitsin
+	  Change-Id: I9cf97bdc756012d1f552ab007f4aa85e0ddb4e62
 
-	  Change-Id: I0b31933b062a23331dbac9a82b8bcfe345f406f6
+2018-08-06 06:36 +0000 [5030756bfa]  Joshua Colp <jcolp@digium.com>
 
-2018-04-10 16:09 +0000 [3c2249fd37]  George Joseph <gjoseph@digium.com>
+	* stasis: Reduce calculation of stasis message type hash.
 
-	* app_sendtext:  Enhance SendText to support Enhanced Messaging
+	  When the stasis cache is used a hash is calculated for
+	  retrieving or inserting messages. This change calculates
+	  a hash when the message type is initialized that is then
+	  used each time needed. This ensures that the hash is
+	  calculated only once for the message type.
 
-	  SendText now accepts new channel variables that can be used
-	  to override the To and From display names and set the Content-Type
-	  of a message.  Since you can now set Content-Type, other text/*
-	  content types are now valid.
+	  Change-Id: I4fe6bfdafb55bf5c322dd313fbd8c32cce73ef37
 
-	  Change-Id: I648b4574478119f95de09d9f08e9595831b02830
+2018-07-30 07:49 +0000 [04974a0ca2]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-09-27 11:44 +0000 [be7d4faed5]  George Joseph <gjoseph@digium.com>
+	* pjproject_bundled: Fix for Solaris builds. Do not undef s_addr.
 
-	* bridge_softmix:  Forward TEXT frames
+	  The authors of PJProject undef s_addr because of some issue in Microsoft
+	  Windows. However in Oracle Solaris, s_addr is not a structure member, but
+	  defined to map to the real structure member.
 
-	  Core bridging and, more specifically, bridge_softmix have been
-	  enhanced to relay received frames of type TEXT or TEXT_DATA to all
-	  participants in a softmix bridge.  res_pjsip_messaging and
-	  chan_pjsip have been enhanced to take advantage of this so when
-	  res_pjsip_messaging receives an in-dialog MESSAGE message from a
-	  user in a conference call, it's relayed to all other participants
-	  in the call.
+	  Updates the patch from ASTERISK_20366
 
-	  res_pjsip_messaging already queues TEXT frames to the channel when
-	  it receives an in-dialog MESSAGE from an endpoint and chan_pjsip
-	  will send an MESSAGE when it gets a TEXT frame.  On a normal
-	  point-to-point call, the frames are forwarded between the two
-	  correctly.  bridge_softmix was not though so messages weren't
-	  getting forwarded to conference bridge participants.  Even if they
-	  were, the bridging code had no way to tell the participants who
-	  sent the message so it would look like it came from the bridge
-	  itself.
+	  ASTERISK-27997
 
-	  * The TEXT frame type doesn't allow storage of any meta data, such
-	  as sender, on the frame so a new TEXT_DATA frame type was added that
-	  uses the new ast_msg_data structure as its payload.  A channel
-	  driver can queue a frame of that type when it receives a message
-	  from outside.  A channel driver can use it for sending messages
-	  by implementing the new send_text_data channel tech callback and
-	  setting the new AST_CHAN_TP_SEND_TEXT_DATA flag in its tech
-	  properties.  If set, the bridging/channel core will use it instead
-	  of the original send_text callback and it will get the ast_msg_data
-	  structure. Channel drivers aren't required to implement this.  Even
-	  if a TEXT_DATA enabled driver uses it for incoming messages, an
-	  outgoing channel driver that doesn't will still have it's send_text
-	  callback called with only the message text just as before.
+	  Change-Id: I8223026d4d54e2a46521085fcc94bfa6ebe35b11
 
-	  * res_pjsip_messaging now creates a TEXT_DATA frame for incoming
-	  in-dialog messages and sets the "from" to the display name in the
-	  "From" header, or if that's empty, the caller id name from the
-	  channel.  This allows the chat client user to set a friendly name
-	  for the chat.
+2018-08-03 15:59 +0000 [99a0586ec1]  Richard Mudgett <rmudgett@digium.com>
 
-	  * bridge_softmix now forwards TEXT and TEXT_DATA frames to all
-	  participants (except the sender).
+	* res_pjsip: Make pjlib.h consistently included.
 
-	  * A new function "ast_sendtext_data" was added to channel which
-	  takes an ast_msg_data structure and calls a channel's
-	  send_text_data callback, or if that's not defined, the original
-	  send_text callback.
+	  * Don't include pjlib.h twice in res_pjsip.h
+	  * Consistently use #include <> form for pjproject includes.
+	  (pjsip.h and pjlib.h)
 
-	  * bridge_channel now calls ast_sendtext_data for TEXT_DATA frame
-	  types and ast_sendtext for TEXT frame types.
+	  Change-Id: I3f7b42044840de64edf7e9d7695cb60c45990dc7
 
-	  * chan_pjsip now uses the "from" name in the ast_msg_data structure
-	  (if it exists) to set the "From" header display name on outgoing text
-	  messages.
+2018-08-02 14:37 +0000 [523b7b2ffc]  Salah Ahmed <txrubel@gmail.com>
 
-	  Change-Id: Idacf5900bfd5f22ab8cd235aa56dfad090d18489
+	* dialplan_functions: wrong srtp use status report of a dialplan function
 
-2018-04-17 05:31 +0000 [fb9634bcb3]  Alexander Traud <pabstraud@compuserve.com>
+	  If asterisk offer an endpoint with SRTP and that endpoint respond
+	  with non srtp, in that case channel(rtp,secure,audio) reply wrong
+	  status.
 
-	* chan_vpb: Avoid GNU old-style field designator extension.
+	  Why delete flag AST_SRTP_CRYPTO_OFFER_OK while check identical remote_key:
+	  Currently this flag has being set redundantly. In either case identical
+	  or different remote_key this flag has being set. So if we
+	  don't set it while we receive identical remote_key or non SRTP SDP
+	  response then we can take decision of srtp use by using that flag.
 
-	  clang 6.0 warned about this. Beside that, this change removes the used variable
-	  'desc'.
+	  ASTERISK-27999
 
-	  ASTERISK-27808
+	  Change-Id: I29dc2843cf4e5ae2604301cb4ff258f1822dc2d7
 
-	  Change-Id: Ia26bdcc0a562c058151814511cfcf70ecafa595b
+2018-07-30 06:05 +0000 [e9d41572b3]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-04-19 18:28 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* pjproject_bundled: Find shared libraries in root --with-ssl=PATH.
 
-	* asterisk 13.21.0-rc1 Released.
+	  The script configure from Teluu expects shared libraries (.so) in a subfolder
+	  called 'lib', when --with-xyz=PATH is specified. However for OpenSSL, the
+	  default location is the root of the source folder = PATH. Furthermore, Asterisk
+	  supports both, 'lib' and root. For consistency and because Asterisk is using
+	  (only) OpenSSL in PJProject, it is enhanced to support both locations, just
+	  like Asterisk.
 
-2018-04-17 07:09 +0000 [92f88f164a]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27995
 
-	* utils/pval: Add -lBlocksRuntime for compiler clang conditionally.
+	  Change-Id: I8eb916a88b6b8c22e29bb40bee8faaca6c73406f
 
-	  ASTERISK-27809
+2018-08-01 09:45 +0000 [1e837e13f5]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I930b364a33d54cc08dedfcd5bb45f7e83242f134
+	* res_pjsip_registrar: Improve performance on inbound handling.
 
-2018-04-13 15:17 +0000 [39c51394c8]  George Joseph <gjoseph@digium.com>
+	  This change removes a sorcery lookup for retrieving all
+	  contacts at the end of the registration process by keeping
+	  track of the contacts that are added/updated/deleted.
 
-	* utils:  Add ast_assert_return
+	  This ensures at the end of the process the container of
+	  contacts we have is the current state.
 
-	  Similar to pjproject's PJ_ASSERT_RETURN macro, this one will do the
-	  following...
+	  Pool usage has also been reduced by allocating one for
+	  usage throughout the handling of a REGISTER and resetting
+	  it to a clean state. This ensures that in most cases
+	  we allocate once and just reuse it.
 
-	  If the assert passes... NoOp
+	  ASTERISK-28001
 
-	  If the assert fails and AST_DEVMODE is defined, execute ast_assert()
-	  then, if DO_CRASH isn't set, return from the calling function with
-	  the supplied value.
+	  Change-Id: I1a78b2d46f9a2045dbbff1a3fd6dba84b612b3cb
 
-	  If the assert fails and AST_DEVMODE is not defined, return from the
-	  calling function with the supplied value.
+2018-07-17 07:13 +0000 [e818925a58]  Torrey Searle <torrey@voxbone.com>
 
-	  The macro will execute a return without a value if one isn't suppled.
+	* thirdparty/pjproject: fix deadlock in response retransmissions
 
-	  Change-Id: I0003844affeab550d5ff5bca7aa7cf8a559b873e
+	  The tdata containing the response can be shared by both the dialog
+	  object and the tsx object.  In order to prevent the race condition
+	  between the tsx retransmission and the dialog sending a response,
+	  clone the tdata before modifying it for the dialog send response.
 
-2018-04-13 14:32 +0000 [f2e140e8a7]  Ben Ford <bford@digium.com>
+	  ASTERISK-27966 #close
 
-	* res_musiconhold: Don't restart MOH from beginning after announcement.
+	  Change-Id: Ic381004a3a212fe1d8eca0e707fe09dba4a6ab4e
 
-	  This reverts a problem introduced by the fix for ASTERISK_24329.
-	  Now, when an announcement is played while waiting in a queue, music on
-	  hold will not restart from the beginning of the sound file and will
-	  instead pick up where it left off. However, the incorrect behavior in
-	  ASTERISK_24329 is now present again; if an announcement X seconds
-	  long is played when music on hold starts, music on hold will start X
-	  seconds into the file.
+2018-07-31 11:24 +0000 [20b53b5cee]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27774 #close
-	  Reported by: lvl
+	* pjsip_wizard.conf.sample: Update remote_hosts description.
 
-	  Change-Id: I86b2885ee7063268f9b9747eddb788336ade989b
+	  Remove the note that SRV records are not supported as that is no longer
+	  true.
 
-2018-03-28 15:13 +0000 [b92ebdba5f]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27993
 
-	* pjsip_scheduler.c: Add ability to trace scheduled tasks.
+	  Change-Id: Id0dd6ef40e52702be9727a2b6122216cb00bb4ca
 
-	  When a scheduled task is created you can pass in the
-	  AST_SIP_SCHED_TASK_TRACK flag.  This new flag causes scheduling events to
-	  be logged.
+2018-07-27 13:23 +0000 [914b9e1934]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I91967eb3d5a220915ce86881a28af772f9a7f56b
+	* CI: Add optional uninstall step before installing asterisk
 
-2018-03-27 11:04 +0000 [12aa25b2e1]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I7dedf1e925eafc3a0adf01dd9dfbe44eb642aab7
 
-	* res_pjsip.c: Split ast_sip_push_task_synchronous() to fit expectations.
+2018-07-28 11:49 +0000 [93d1c34efe]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ast_sip_push_task_synchronous() did not necessarily execute the passed in
-	  task under the specified serializer.  If the current thread is any
-	  registered pjsip thread then it would execute the task immediately instead
-	  of under the specified serializer.  Reentrancy issues could result if the
-	  task does not execute with the right serializer.
+	* BuildSystem: Enable ncurses for menuselect in Solaris 11.
 
-	  The original reason ast_sip_push_task_synchronous() checked to see if the
-	  current thread was a registered pjsip thread was because of a deadlock
-	  with masquerades and the channel technology's fixup callback
-	  (ASTERISK_22936).  A subsequent masquerade deadlock fix (ASTERISK_24356)
-	  involving call pickups avoided the original deadlock situation entirely.
-	  The PJSIP channel technology's fixup callback no longer needed to call
-	  ast_sip_push_task_synchronous().
+	  The check for the library ncurses should use not use the header <curses.h> but
+	  <ncurses.h>, because on some platforms <curses.h> is not a drop-in replacement
+	  for <ncurses.h>: For example in Solaris, the symbol initscr is a typedef in
+	  <curses.h> to a symbol which does not exist in the library ncurses (initscr32).
+	  Simply use <ncurses.h> when you link to ncurses.
 
-	  However, there are a few places where this unexpected behavior is still
-	  required to avoid deadlocks.  The pjsip monitor thread executes callbacks
-	  that do calls to ast_sip_push_task_synchronous() that would deadlock if
-	  the task were actually pushed to the specified serializer.  I ran into one
-	  dealing with the pubsub subscriptions where an ao2 destructor called
-	  ast_sip_push_task_synchronous().
+	  Furthermore in Solaris, the header <ncurses.h> is in a subdirectory
+	  /usr/include/ncurses and not available via pkg-config.
 
-	  * Split ast_sip_push_task_synchronous() into
-	  ast_sip_push_task_wait_servant() and ast_sip_push_task_wait_serializer().
-	  ast_sip_push_task_wait_servant() has the old behavior of
-	  ast_sip_push_task_synchronous().  ast_sip_push_task_wait_serializer() has
-	  the new behavior where the task is always executed by the specified
-	  serializer or a picked serializer if one is not passed in.  Both functions
-	  behave the same if the current thread is not a SIP servant.
+	  ASTERISK-15331
+	  ASTERISK-14935
+	  ASTERISK-12382
+	  ASTERISK-9107
 
-	  * Redirected ast_sip_push_task_synchronous() to
-	  ast_sip_push_task_wait_servant() to preserve API for released branches.
+	  Change-Id: Ife367776b0ccf17d3fefed868245376bfb93745d
 
-	  ASTERISK_26806
+2018-07-28 08:00 +0000 [ee9794d741]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Id040fa42c0e5972f4c8deef380921461d213b9f3
+	* res_pjsip_pubsub: Use ast_true for "prune_on_boot".
 
-2018-03-21 19:43 +0000 [dfdc9a2575]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Iedec4e7390b3e821987681da24d0298632b9873d
 
-	* pjsip_scheduler.c: Fix some corner cases.
+2018-07-28 07:39 +0000 [c10b3ccd5f]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Fix the periodic interval wander because it may take significant time
-	  between the sched thread queueing the task in the serializer and the
-	  serializer actually executing the task.  The time it takes to actually
-	  execute the task was already taken into account.
+	* BuildSystem: Enable Jansson in Solaris 11.
 
-	  * Pass a schtd ref to the serializer when we queue a scheduled task on
-	  the serializer.  We don't want it going away on us while it is in the
-	  serializer queue.
+	  In Solaris, the header <jansson.h> is in /usr/include/jansson. To find
+	  Jansson even in such a subdirectory, the tool pkg-config is queried via
+	  AST_PKG_CONFIG_CHECK. For those platforms, which do not list Jansson via
+	  pkg-config, the previous check remains and is executed thereafter.
 
-	  * Skip the scheduled task if the task was canceled between queueing the
-	  task to the serializer and the serializer actually executing the task.
+	  Because the check for the NetBSD Editline library uses the tool pkg-config
+	  the code of PKG_PROG_PKG_CONFIG must be used. Because that check happens
+	  earlier than Jansson, it must be placed in front of that.
 
-	  * Reorder struct ast_sip_sched_task to avoid unnecessary padding.  Removed
-	  task_id and added next_periodic.
+	  ASTERISK-27991
 
-	  * Hold a ref to the passed in serializer so the serializer cannot go away
-	  on the scheduled task.
+	  Change-Id: I69ea0f379f87a50049654b2487c76ee1c04fa53a
 
-	  ASTERISK_26806
+2018-07-24 13:44 +0000 [32ce8e5cf3]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I6c8046b75f6953792c8c30e55b836a4291143f24
+	* res_pjsip_endpoint_identifier_ip.c: Added regex support to match_header
 
-2018-03-22 19:09 +0000 [af36823907]  Richard Mudgett <rmudgett@digium.com>
+	  This patch adds regular expression support to make the identify section's
+	  match_header option more useful when attempting to match complex headers
+	  like the 'To' or 'From' headers.  The 'From' header has variable
+	  components such as the tag parameter that you cannot predict.  To specify
+	  a regular expression put slashes around the regular expression in place of
+	  the header value.
 
-	* pjsip_scheduler.c: Sort "pjsip show scheduled_tasks" output.
+	  [identify-alice]
+	  type=identify
+	  endpoint=alice
+	  match_header=From: /<sip:alice@127\\.0\\.0\\.1>/
 
-	  * A side benefit is that the scheduled tasks are not completely blocked
-	  while the CLI command executes.
+	  * Added regex support to match_header so you could match a 'To' header
+	  among other complex headers.
 
-	  * Adjusted the "Task Name" column width to have more room for longer
-	  names.
+	  Fixed reported crashes when trying to match special headers like 'Contact'.
+	  The identify section's match_header method used code that assumed you were
+	  matching a generic header.  Any other type of header could cause a crash
+	  if the header structure variant did not match the generic header enough.
 
-	  Change-Id: Iec64aa463ee8b10eef90120e00c38b1fb444087e
+	  * Made use code that will work for any header type instead of code
+	  specific to generic headers.
 
-2018-04-02 15:59 +0000 [7886354343]  Evandro Cesar Arruda <ecarruda@gmail.com>
+	  Other fixes while in the area:
 
-	* cdr_mysql: Compile error because MYSQL_PORT definition is missing
+	  * Made check all headers of the requested name.
+	  * Added some more sanity checks to the configured identify matching
+	  options when applying the configuration.
 
-	  If it is not defined, it will add MYSQL_PORT definition. After some
-	  research on MySQL/MariaDB development tree, I couldn't find any reference
-	  to MYSQL_PORT definition in include files.
+	  ASTERISK-27548
 
-	  ASTERISK-27782 #close
+	  Change-Id: I27dfd4ff5e2259b906640e3c330681b76b4ed1f1
 
-	  Change-Id: Ieee56c836fc2e8bd021c456145bba04c6068bb77
+2018-07-27 10:46 +0000 [59f53514ce]  Joshua Colp <jcolp@digium.com>
 
-2018-04-09 20:00 +0000 [724d926d46]  Chris-Savinovich <csavinovich@digium.com>
+	* res_pjsip_pubsub: Treat "prune_on_boot" as a yes / no.
 
-	* res_pjsip_session: Rewrite o= with external_media_address.
+	  The alembic for the PJSIP subscription persistence table has the
+	  "prune_on_boot" field as a boolean. While in Asterisk we are
+	  tolerant of many different definitions of true and false in the
+	  database we only accept "yes" and "no". This change makes the
+	  field treated as a yes/no instead of an integer, thus storing
+	  "yes" and "no" instead of "1" and "0".
 
-	  It now appends the external IP address on the
-	  o= line of the SDP packet.  The decision was made to write
-	  the numeric IP address as opposed to the RFC that states
-	  the FQDN should be used if and when available.  We believe
-	  the usage of literal IP address will help avoid
-	  potential problems.
+	  Change-Id: Ic8b9211b36babefe78f70def6828a135a6ae7ab6
 
-	  ASTERISK-27614 #close
+2018-07-27 08:26 +0000 [8da81a208f]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I84f3360f3606b8c4e8d161edb228799ec0b8a302
+	* res_rtp_asterisk: In Developer Mode, do not require OpenSSL.
 
-2018-02-22 12:18 +0000 [6a4afe09ce]  Nathan Bruning <nathan@iperity.com>
+	  OpenSSL is an optional external library and should stay optional even when
+	  Developer Mode is configured.
 
-	* res_pjsip_notify.c: enable in-dialog NOTIFY
+	  ASTERISK-27990
 
-	  This patch adds support to send in-dialog SIP NOTIFY commands on
-	  chan_pjsip channels, similar to the functionality recently added
-	  for chan_sip (ASTERISK_27461).
+	  Change-Id: Ia68a4cd5474b26d45e0f43b04032ad598022853b
 
-	  This extends res_pjsip_notify to allow for in-dialog messages.
+2018-07-26 18:54 +0000 [e097dca4d7]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27697
+	* CI: Fix placement of job summary statments
 
-	  Change-Id: If7f3151a6d633e414d5dc319d5efc1443c43dd29
+	  Change-Id: Iace19e718f4e8fb48eb7dc9f98af53b115cc45f3
 
-2018-03-22 13:35 +0000 [c4f02c975b]  Richard Mudgett <rmudgett@digium.com>
+2018-07-26 12:52 +0000 [e5de8a70f8]  Corey Farrell <git@cfware.com>
 
-	* pjsip_scheduler.c: Fix ao2 usage errors.
+	* loader: Process dependencies for built-in modules.
 
-	  * Removed several invalid uses of OBJ_NOLOCK.  These uses resulted in the
-	  'tasks' container being accessed without a lock in a multi-threaded
-	  environment.  A recipe for crashes.
+	  With the new module loader it was missed that built-in modules never
+	  parsed dependencies from mod->info into vectors of mod.  This caused
+	  manager to be initialized before acl (named_acl).  If manager.conf
+	  used any named ACL's they would not be found and result in no ACL being
+	  applied to the AMI user.
 
-	  * Removed needlessly obtaining schtd object references.  If the caller
-	  providing you a pointer to an object doesn't have a valid reference then
-	  you cannot safely get one from it.
+	  In addition to the manager ACL fix this adds "extconfig" to all builtin
+	  modules which support realtime configuration.  This only matters if one
+	  of the builtin modules is configured with 'preload', depending on
+	  "extconfig" will cause config.c to automatically be initialize during
+	  the preload stage.
 
-	  * Getting a ref to 'tasks' when you aren't copying the pointer into
-	  another location is useless.  The 'tasks' container pointer is global.
+	  Change-Id: I482ed6bca6c1064b05bb538d7861cd7a4f02d9fc
 
-	  * Removed many unnecessary uses of RAII_VAR.
+2018-07-18 09:32 +0000 [d3809d61cb]  Emmanuel BUU <emmanuel.buu@ives.fr>
 
-	  * Make ast_sip_schedule_task() name parameter const.
+	* res_rtp_asterisk: Avoid merging command and regular T.140 text packets
 
-	  ASTERISK_26806
+	  When realtime text packets are to be sent, the text is accumulated in a
+	  buffer and sent regularly by a timer.  It can happen that commands such as
+	  a backspace, CR, or LF get merged with regular text.  This breaks some
+	  UAs.
 
-	  Change-Id: I5c62488e651314e2a1dbc01f5b078a15512d73db
+	  The proposed change:
+	  * We test if the current packet contains a command.  If so we send the
+	  buffer immediately.
+	  * We test if the buffer contained a command.  If so we send the buffer
+	  immediately.
+	  * We accumulate the text (or the command) in the buffer.
 
-2018-03-23 06:49 +0000 [7593359787]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27970
 
-	* Build System: Enable python3 compatibility.
+	  Change-Id: Ifbe993311410fa855cb8aa4a12084db75f413462
 
-	  * Consistently use spaces in rest-api-templates/asterisk_processor.py.
-	  * Exclude third-party from docs/full-en_US.xml.
-	  * Add docs/full-en_US.xml to .gitignore.
-	  * Use list() to convert python3 view.
-	  * Use python3 print function.
-	  * Replace cmp() with equivalent equation.
-	  * Replace reference to out of scope subtype variable with name
-	    parameter.
-	  * Use unescaping triple bracket notation in mustache templates where
-	    needed.  This causes behavior of Python2 to be maintained when using
-	    Python3.
-	  * Fix references to has_websocket / is_websocket in
-	    res_ari_resource.c.mustache.
-	  * Update calculation of has_websocket to use any().
-	  * Use unicode mode for writing output file in transform.py.
-	  * Replace 'from swagger_model import *' with explicit import of required
-	    symbols.
-	  * Add missing 'import os'
-	  * Fix invalid reference to swagger_version from exception handler.
+2018-07-26 11:34 +0000 [798f049e82]  George Joseph <gjoseph@digium.com>
 
-	  I have not tested voicemailpwcheck.py, only the print syntax has
-	  been fixed.
+	* CI:  Add docker info to job summary
 
-	  Change-Id: If5c5b556a2800d41a3e2cfef080ac2e151178c33
+	  Change-Id: I45d52005a9b692ad303c11792f226ace1e449901
 
-2018-04-05 18:33 +0000 [72b16ee400]  Richard Mudgett <rmudgett@digium.com>
+2018-07-23 13:49 +0000 [7578881b3d]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_refer/chan_sip: Fix INVITE with replaces transfer to ConfBridge
+	* Build System: Create 'make install-configs' target.
 
-	  There is a problem when an INVITE-with-Replaces transfer targets a channel
-	  in a ConfBridge.  The transfer will unconditionally swap out the
-	  ConfBridge channel.  Unfortunately, the ConfBridge state will not be aware
-	  of this change.  Unexpected behavior will happen as a result since
-	  ConfBridge channels currently can only be replaced by a masquerade and not
-	  normal bridge channel moves.
+	  This target requires specifying CONFIG_SRC=path_to_configs.  This can be
+	  used to install custom configs for the Asterisk build while still
+	  performing directory replacements on asterisk.conf.
 
-	  * We just need to pretend that the channel isn't in a bridge (like other
-	  transfer methods already do) so the transfer channel will masquerade into
-	  the ConfBridge channel.
+	  Modify internal INSTALL_CONFIGS so first argument requires full path to
+	  the config sources relative to Asterisk source root.
 
-	  Change-Id: I209beb0e748fa4f4b92a576f36afa8f495ba4c82
+	  Change-Id: Idcd841df3c8d5bfe23d566bb9e2e448e9df4f8ab
 
-2018-04-05 17:40 +0000 [ea055386e0]  Richard Mudgett <rmudgett@digium.com>
+2018-07-25 15:33 +0000 [8ea13d62cd]  Kevin Harwell <kharwell@digium.com>
 
-	* chan_sip.c: Fix INVITE with replaces channel ref leak.
+	* json.c: improve ast_json_to_ast_variables performance
 
-	  Given the below call scenario:
-	  A -> Ast1 -> B
-	  C <- Ast2 <- B
+	  When converting from a json object to an ast variables list the conversion
+	  algorithm was doing a complete traversal of the entire variables list for
+	  every item appended from the json structure.
 
-	  1) A calls B through Ast1
-	  2) B calls C through Ast2
-	  3) B transfers A to C
+	  This patch makes it so the list is no longer traversed for each new ast
+	  variable being appended.
 
-	  When party B transfers A to C, B sends a REFER to Ast1 causing Ast1 to
-	  send an INVITE with replaces to Ast2.  Ast2 then leaks a channel ref of
-	  the channel between Ast1 and Ast2.
+	  Change-Id: I8bf496a1fc449485150d6db36bfc0354934a3977
 
-	  Channel ref leaks are easily seen in the CLI "core show channels" output.
-	  The leaked channels appear in the output but you can do nothing with them
-	  and they never go away unless you restart Asterisk.
+2018-07-25 05:32 +0000 [e5c0ac6a64]  Joshua Colp <jcolp@digium.com>
 
-	  * Properly account for the channel refs when imparting a channel into a
-	  bridge when handling an INVITE with replaces in handle_invite_replaces().
-	  The ast_bridge_impart() function steals a channel ref but the code didn't
-	  account for how many refs were held by the code at the time and which ref
-	  was stolen.
+	* devicestate: Don't create topic when change isn't cached.
 
-	  * Eliminated RAII_VAR in handle_invite_replaces().
+	  When publishing a device state the change can be marked as being
+	  cachable or not. If it is not cached the change is just published
+	  to all interested and not stored away for later query. This was not
+	  fully taken into account when publishing in stasis. The act of
+	  publishing would create a topic for the device even if it may be
+	  ephemeral.
 
-	  ASTERISK-27740
+	  This change makes it so messages which are not cached won't create
+	  a topic for the device. If a topic does already exist it will be
+	  published to but otherwise the change will only be published to
+	  the device state all topic.
 
-	  Change-Id: I7edbed774314b55acf0067b2762bfe984ecaa9a4
+	  ASTERISK-27591
 
-2018-03-21 19:40 +0000 [e94f8e4a24]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I18da0e8cbb18e79602e731020c46ba4101e59f0a
 
-	* res_pjsip: Update authenticate_qualify documentation.
+2018-07-25 10:20 +0000 [93997f9698]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I3811de0014b1ffe96d4a3b49cddd5d4ca02ee5d4
+	* CI: Explicitly pass BRANCH_NAME to buildAsterisk and installAsterisk
 
-2018-04-02 16:49 +0000 [906db6a3ff]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I652f4a0ea5107c778e27a78bccb67b18b0c4e087
 
-	* app_agent_pool.c: Fix off nominal ref leak.
+2018-07-24 13:29 +0000 [0baf660bde]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ib427ffc2c802620eaafb08b1c2a17dddd8fb8eb6
+	* CI: Add options to initialize and cleanup database to runTestsuite.sh
 
-2018-04-04 10:02 +0000 [07e408e588]  Corey Farrell <git@cfware.com>
+	  Change-Id: I352333233bab5377723bf37d490ba84fc55bc853
 
-	* Build System: Strip '-std=c99' from CFLAGS provided by libraries.
+2018-07-25 09:07 +0000 [74cbeabb97]  Corey Farrell <git@cfware.com>
 
-	  Asterisk requires GNU C extensions.  On some systems certain libraries
-	  may incorrectly push -std=c99 into CFLAGS, thus breaking the build.
-	  This change causes that flag to be stripped so the Asterisk build is not
-	  broken by those libraries.  This change is made for both pkgconfig and
-	  tool based libraries.
+	* CI: Do not `mkdir 2`.
 
-	  ASTERISK-27629 #close
+	  Change-Id: Ib7377d26a6c98b38bad463f47c84f1875ac84eb7
 
-	  Change-Id: I13389613b194abbac77becf90cd950dc168704db
+2018-07-25 07:34 +0000 [adf3c569a2]  Corey Farrell <git@cfware.com>
 
-2018-04-03 14:39 +0000 [01b3c435d8]  Corey Farrell <git@cfware.com>
+	* Build System: Silence build of bundled jansson.
 
-	* Build System: Fixes for configure script.
+	  Change-Id: I7392c79c0173057f5378010bf1fe65e300e8fc56
 
-	  * Replace all 'else if' statements with 'elif'.
-	  * Use loop to detect versioned lua headers and libraries.
+2018-07-25 07:13 +0000 [e4b0f09bcc]  George Joseph <gjoseph@digium.com>
 
-	  The loop for detecting lua fixes a bug where LUA_INCLUDE would be
-	  appended with the directory of every lua version after the first one is
-	  found.
+	* CI: RefDebug: Fix reference to testsuite URL
 
-	  Change-Id: I3276f9aee955014108345be6092f51c932b43a0f
+	  Change-Id: I0ee41d95a87f0d97b01f2757012b846bcfe6443d
 
-2018-01-02 07:54 +0000 [cea1a22ef3]  George Joseph <gjoseph@digium.com>
+2018-07-24 14:28 +0000 [5537048d60]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip:  Correct usages of pjproject's timer heap
+	* Build System: Fix bundled jansson install.
 
-	  Fix some timer heap initializations and cancels to try and prevent
-	  crashes and timer heap issues.
+	  Update the bundled jansson Makefile to do nothing during Asterisk
+	  install, use a target that is not phony to initiate the jansson make and
+	  install.
 
-	  Change-Id: I64885d190fa22097d1b55987091375541e57a7ee
+	  Change-Id: I7643cc3d39af9feba8fc0da676b646efc5f8b3bb
 
-2018-03-25 13:35 +0000 [158c0efd4f]  George Joseph <gjoseph@digium.com>
+2018-07-24 10:43 +0000 [9f3620a578]  Corey Farrell <git@cfware.com>
 
-	* pjroject_bundled:  Add already-destroyed check to tsx_timer_callback
+	* CI: Use bundled jansson if needed.
 
-	  There have been cases that when the transaction timer callback is called
-	  the tsx is already destroyed.  This causes a crash.  We now check the
-	  tsx state and return if the tsx is already destroyed.
+	  Use pkg-config to determine if jansson is at least 2.11, enabled bundled
+	  version otherwise.
 
-	  Change-Id: If93acd5e48d9ca5bb553f2405d5afc836842fe1c
+	  Change-Id: Ib555a8b72ff6f6925f9280ef035caa0b91ca4bd2
 
-2018-03-25 13:25 +0000 [69c61237cd]  George Joseph <gjoseph@digium.com>
+2018-07-24 04:57 +0000 [1f46e2e91c]  Florian Floimair <f.floimair@commend.com>
 
-	* pjproject_bundled: timer: Clean up usage of timer heap
+	* res_pjsip: Change log message from error to warning for valid use cases
 
-	  Added a new pj_timer_entry_reset function that resets a timer_entry
-	  for re-use.
+	  If a SIP MESSAGE is triggered for an endpoint that is currently not registered
+	  - and therefore has no valid contact associated - an error message was logged.
+	  Since this is a valid request in a valid use cases this is now changed to a
+	  warning, as discussed with Matt Fredrickson on the asterisk-dev mailing list.
 
-	  Changed direct settings of timer_entry fields to use
-	  pj_timer_entry_init and pj_timer_entry_reset.
+	  Change-Id: I55eb62d2712818a58c7532119dec288bd98cf0c0
 
-	  Fixed issues where timers were being rescheduled incorrectly.
+2018-07-24 05:39 +0000 [8550fad21a]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I5b624bfbc5c1429117484b9b24567293002148e6
+	* CI:  Add --privileged flag to docker options
 
-2018-03-22 08:54 +0000 [83353997f4]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: If92d55f15306e55dd7091ac3c47b13ebbbb03488
 
-	* BuildSystem: Add support for building RADIUS with radcli.
+2018-07-24 05:22 +0000 [f2580d037f]  George Joseph <gjoseph@digium.com>
 
-	  Radcli is yet another RADIUS client library, generally compatible with
-	  freeradius and radiusclient-ng.
+	* CI: Set correct user:group when publishing docs
 
-	  This commit adds autoconf option for detecting it as well and changes
-	  cdr_radius and cel_radius to use its header file in that case.
+	  Change-Id: Ibabeb9ac730d9755cf54318d0da74771c939b86b
 
-	  ASTERISK-26540
-	  Reported by: Tzafrir Cohen
+2018-07-23 12:21 +0000 [35d06e46ae]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Icc056d476b7acf481309219e9abdca416866c6ec
+	* core: AST_DEVMODE no longer affects ABI.
 
-2018-03-29 17:07 +0000 [e1363283d7]  Richard Mudgett <rmudgett@digium.com>
+	  Remove AST_DEVMODE from the AST_BUILDOPTS list and the AST_BUILDOPTS_SUM
+	  calculation as it no longer affects API/ABI compatibility.
 
-	* res_pjsip: Fix deadlock on reliable transport shutdown.
+	  Change-Id: Id5bd6dfade173a53b3a49f715586b86e3fb24acb
 
-	  A deadlock can happen when the PJSIP monitor thread is shutting down a
-	  connection oriented transport (TCP/TLS) used by a subscription at the same
-	  time as another thread tries to send something for that subscription.  The
-	  deadlock is between the pjsip monitor thread attempting to get the dialog
-	  lock and another thread sending something for that dialog when it tries to
-	  get the transport manager lock.
+2018-07-23 10:23 +0000 [09b258fd69]  Corey Farrell <git@cfware.com>
 
-	  * res_pjsip_pubsub.c: Avoid the deadlock by pushing the subscription
-	  removal to the subscription serializer.
+	* CI: Split --test-command argument.
 
-	  * res_pjsip_registrar.c: Pushed off incoming registration contact removals
-	  to a default serializer as a precaution.  Removing the contacts involves
-	  sorcery access which in this case will involve database access.  Depending
-	  upon the setup, the database may not be on the same machine and could take
-	  awhile.  We don't want to hold up the pjsip monitor thread with
-	  potentially long access times.
+	  The --test-command argument has now been split, unit tests now use
+	  `--unittest-command` and the testsuite uses --testsuite-command.
 
-	  ASTERISK-27706
+	  This will make it easier to create a script which run everything by
+	  forwarding the same arguments to all CI scripts.
 
-	  Change-Id: I56b647aea565f24dba33e9e5ebeed4cd3f31f8c4
+	  Change-Id: Ia54aa4848eaffbdf13175fcda40fc0b23080ad71
 
-2018-03-25 13:12 +0000 [c63a2e28f1]  George Joseph <gjoseph@digium.com>
+2018-07-20 06:20 +0000 [460ef41f7a]  George Joseph <gjoseph@digium.com>
 
-	* pjproject_bundled:  Add patch for pj_atomic crashes
+	* xmldoc.c:  Fix dump of xml document
 
-	  There have been some crashes in the past where something attempts
-	  to use a pj_atomic after it's already been destroyed.  This patch
-	  tries to prevent it by making sure that pj_atomic_destroy sets
-	  its mutex to NULL when it's done.  The pj_mutex functions already check
-	  for a NULL mutex and just return PJ_EINVAL.
+	  The "xmldoc dump" cli command was simply concatenating xml documents
+	  into the output file.  The resulting file had multiple "xml"
+	  processing instructions and multiple root elements which is illegal.
+	  Normally this isn't an issue because Asterisk has only 1 main xml
+	  documentation file but codec_opus has its own file so if it's
+	  downloaded and you do "xmldoc dump", the result is invalid.
 
-	  Teluu also added some checks to the win32 implementation as well.
+	  * Added 2 new functions to xml.c:
+	      ast_xml_copy_node_list creates a copy of a list of children.
+	      ast_xml_add_child_list adds a list to an existing list.
 
-	  Change-Id: Id25f70b79fdedf44ead6e6e1763a4417d3b3f825
+	  * Modified handle_dump_docs to create a new output document and
+	    add to it the children from each input file.  It then dumps the
+	    new document to the output file.
 
-2018-03-28 08:18 +0000 [5908c6753b]  Corey Farrell <git@cfware.com>
+	  Change-Id: I3f182d38c75776aee76413dadd2d489d54a85c07
 
-	* core: Create main/options.c.
+2018-07-21 11:58 +0000 [fd1f74af40]  Corey Farrell <git@cfware.com>
 
-	  This creates a separate source to 'own' symbols related to options.h and
-	  paths.h.  This significantly reduces the number of exports created by
-	  main/asterisk.o.  This change is required to eventually be able to
-	  link unmodified Asterisk sources to utilities and/or stand-alone tests.
+	* CI: Fix mkdir CACHE_DIR.
 
-	  ASTERISK~26245
+	  Change-Id: Ic9f9a61e230047836c836206731f8ff7eb3538c9
 
-	  Change-Id: I5cf184f4757f9363b80c9e678bdc35c477122380
+2018-07-21 11:22 +0000 [9b3267641f]  Corey Farrell <git@cfware.com>
 
-2018-03-23 13:15 +0000 [e8e4fc6202]  Alexander Traud <pabstraud@compuserve.com>
+	* build_tools/make_version: Get MAINLINE_BRANCH from .gitreview.
 
-	* install_prereq: Add Slackware (somehow).
+	  Use .gitreview defaultbranch setting to determine the mainline branch.
+	  This allows the script to be used against other directories which might
+	  not be on the same defaultbranch.  This can be used by CI scripts to
+	  report the testsuite version being used:
+	  ./build_tools/make_version ${TESTSUITE_DIR}
 
-	  ASTERISK-27770
+	  Change-Id: Ifdad4a9d8a26138c41bc6b630ecc3e34ea1c2758
 
-	  Change-Id: Ib87e0483c785542238cfe34c1e884d5a31edfaab
+2018-07-22 10:41 +0000 [0f999a834f]  Joshua Colp <jcolp@digium.com>
 
-2018-03-23 09:18 +0000 [96a668e60b]  Alexander Traud <pabstraud@compuserve.com>
+	* sched: Make ABI compatible between dev mode and non-dev mode.
 
-	* install_prereq: Add Gentoo Linux.
+	  In the past there was an assertion in the ast_sched_del function
+	  and in order to ensure it was useful the calling function name,
+	  line number, and filename had to be passed in. This cause the ABI
+	  to be different between dev mode and non-dev mode.
 
-	  ASTERISK-27769
+	  This assertion is no longer present so the special logic can be
+	  removed to make it the same between them both.
 
-	  Change-Id: Ieb13293cd67481f3a33f58f6f7c8c3ee1e338e7a
+	  Change-Id: Icbc69c801e357d7004efc5cf2ab936d9b83b6ab8
 
-2018-03-17 01:02 +0000 [6615469a05]  Corey Farrell <git@cfware.com>
+2018-07-20 15:52 +0000 [aea0a0294d]  Richard Mudgett <rmudgett@digium.com>
 
-	* main/indications: Use ast_cli_completion_add for all completions.
+	* asterisk.c: Update displayed copyright year for v16 release.
 
-	  Change-Id: I371be01f178fb542a9fbe8d97e7ae21aa4d82c36
+	  Change-Id: I60622731d928ee9506b1d28934095f0dc3e5306e
 
-2018-03-22 07:27 +0000 [de4e2c1de8]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-16 15:08 +0000 [d2dace81d4]  Corey Farrell <git@cfware.com>
 
-	* BuildSystem: pjsip_evsub_set_uas_timeout was not used (part 2).
+	* Enable bundling of jansson, require 2.11.
 
-	  The previous change was not complete.
+	  Change-Id: Ib3111b151d37cbda40768cf2a8a9c6cf6c5c7cbd
 
-	  ASTERISK-27435
+2018-07-20 09:25 +0000 [c8bd8f6ce9]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I11082c14c0ef9c6af8c995084a6851337ea2a90f
+	* CI: Fix logger.conf for unit tests.
 
-2018-03-22 05:52 +0000 [79bb956b69]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Idea59d60eab20105de50b34f0f0d506e6ef55d5c
 
-	* BuildSystem: With external editline, do not require libs for internal editline.
+2018-07-19 10:34 +0000 [0f82db82cb]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27761
+	* CI:  Add wiki doc publish to periodics
 
-	  Change-Id: Ib17a7415297a210cfcdbf149e4df9b6edadbfab6
+	  Change-Id: I29ba26134e5083bc6788ede235f1a5d4383c148a
 
-2018-03-21 19:25 +0000 [3a4a441817]  George Joseph <gjoseph@digium.com>
+2018-07-20 06:54 +0000 [a2a3ad2438]  Joshua Colp <jcolp@digium.com>
 
-	* Revert "BuildSystem: In NetBSD, the Python Programming Language is python-X.Y."
+	* res_pjsip: Update default keepalive interval to 90 seconds.
 
-	  Something is causing a python2/python3 mismatch on Fedora27.
+	  A change recently went in which disabled the built-in PJSIP
+	  keepalive. This defaulted to 90 seconds and kept TCP/TLS
+	  connections alive. Disabling this functionality has resulted
+	  in a behavior change of not doing keepalives by default resulting
+	  in TCP/TLS connections dropping for some people.
 
-	  PYTHON='/usr/bin/python2'
-	  PYTHONDEV_CFLAGS='-I/usr/include/python3.6m '
-	  PYTHONDEV_INCLUDE='-I/usr/include/python3.6m '
-	  PYTHONDEV_LIB='-lpython3.6m '
-	  PYTHONDEV_LIBS='-lpython3.6m '
+	  This change makes our default keepalive interval 90 seconds
+	  to match the previous behavior and preserve it.
 
+	  ASTERISK-27978
 
-	  This reverts commit c0c537c1d3aab05358df697d8d259aa73f55a43c.
+	  Change-Id: Ibd9a45f3cbe5d9bb6d2161268696645ff781b1d6
 
-	  Change-Id: I670a82aa7c872fa5facea1e3bc718e2d3124cc28
+2018-07-19 16:17 +0000 [dbffcdc561]  Richard Mudgett <rmudgett@digium.com>
 
-2018-03-20 15:28 +0000 [17cd687679]  Kevin Harwell <kharwell@digium.com>
+	* res_pjsip: Update endpoint transport option documentation.
 
-	* bridge_softmix: Clear "talking" when a channel is put on hold
+	  Change-Id: I5394fdff6a296efc8e1695a156e616acd932ae52
 
-	  This patch clears the talking flag from the channel (if already set), and
-	  notifies listeners when that channel is put on hold. Note however, if the
-	  endpoint continues to send audio frames and these are received by the bridge
-	  then that channel will be put back into a "talking" state even though they
-	  are on hold.
+2018-07-19 13:27 +0000 [709b795cb0]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27755 #close
+	* pjsip_resolver.c: Use replacement function
 
-	  Change-Id: I930e16c4662810f9f02043d69062f88173c5e2ef
+	  * Use the replacement function ast_sip_push_task_wait_servant() instead of
+	  the deprecated ast_sip_push_task_synchronous().
 
-2018-03-07 06:15 +0000 [5955cdf5ae]  Ross Beer <ross.beer@voicehost.co.uk>
+	  Change-Id: I145b550ba7054640c7faa3b644e63137f505c612
 
-	* pjsip_transport_events.c: Fix crash using stale transport pointer.
+2018-07-18 17:13 +0000 [c40412bd13]  Corey Farrell <git@cfware.com>
 
-	  Apparently it is possible for the transport to be destroyed without
-	  triggering the transport callback logic.  As a result the transport gets
-	  destroyed and we have a stale pointer in the active_transports container.
+	* contrib: Update systemd README.txt.
 
-	  * Invoke the transport monitor callback checks when the transport is
-	  destroyed in addition to when it is disconnected and shutdown.
+	  Mention need to compile Asterisk with systemd development package
+	  installed.
 
-	  ASTERISK-27688
+	  ASTERISK-27968
 
-	  Change-Id: Ia9b5469fea8f2b3f2d8476fae6b748a4d23e7261
+	  Change-Id: Ib3a973be403c61cbe09572b0f912fb1aa1bff026
 
-2018-03-20 09:58 +0000 [69de659224]  Ivan Poddubny <ivan.poddubny@gmail.com>
+2018-07-18 14:19 +0000 [0b3bc4852f]  Joshua Colp <jcolp@digium.com>
 
-	* func_channel: Delete dead CHANNEL_TRACE code
+	* Update mainline version for the 16 branch.
 
-	  The functions behind the flag and the flag itself were removed
-	  from Asterisk 12 as incompatible with the new architecture.
+	  Change-Id: I4d36277d10335349d83ae218fa10fee99c3e4c14
 
-	  Change-Id: I058493ef7a53ee290fd225bbcbb07bf46b623ccf
+2018-08-08 16:02 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-2018-03-20 11:55 +0000 [7580d1e0bb]  Alexander Traud <pabstraud@compuserve.com>
+	* asterisk 16.0.0-rc1 Released.
 
-	* BuildSystem: For consistency, avoid extra libs to be empty.
+2018-07-27 13:23 +0000 [d3789cc420]  George Joseph <gjoseph@digium.com>
 
-	  AST_EXT_LIB_CHECK has several optional parameters. When an optional parameter
-	  is left empty, [] is used to indicate this. However, this is done in the script
-	  ./configure only then, when a further parameter is not empty. For example, when
-	  no extra libraries are needed to test the checked library, parameter 5 is not
-	  mentioned. Except parameter 6 and higher are used, then parameter 5 must be
-	  empty.
+	* CI: Add optional uninstall step before installing asterisk
 
-	  However, this general rule was broken
-	  * three times for parameter 5 (extra libs) and
-	  * three times for parameter 4 (header)
-	  as found via the Regular Expression \[\]\). In case of parameter 5, all cases
-	  were changed, because that happened for no reason. In case of parameter 4, an
-	  [] improves readability actually. Therefore for parameter 4, the only case which
-	  did not do it was changed. All this aims to create more consistency: Only do
-	  something different if there is a reason to do so.
+	  Change-Id: I7dedf1e925eafc3a0adf01dd9dfbe44eb642aab7
 
-	  Change-Id: I037ef170cf1ad94497151a9ea5071a31c656cafe
+2018-07-28 08:00 +0000 [89b669a227]  Joshua Colp <jcolp@digium.com>
 
-2018-03-17 20:03 +0000 [6f304697b0]  Corey Farrell <git@cfware.com>
+	* res_pjsip_pubsub: Use ast_true for "prune_on_boot".
 
-	* core: Remove dead symbols from asterisk.exports.in.
+	  Change-Id: Iedec4e7390b3e821987681da24d0298632b9873d
 
-	  * dahdi_chan_name
-	  * dahdi_chan_name_len
-	  * dahdi_chan_mode
-	  * __manager_event
-	  * dialed_interface_info
+2018-07-27 10:46 +0000 [0028db48cc]  Joshua Colp <jcolp@digium.com>
 
-	  Added comment about __progname and environ being needed for FreeBSD to
-	  prevent accidental removal in the future.
+	* res_pjsip_pubsub: Treat "prune_on_boot" as a yes / no.
 
-	  Change-Id: I3ae026bc541cd9cb572be2ffa95fc359547642b5
+	  The alembic for the PJSIP subscription persistence table has the
+	  "prune_on_boot" field as a boolean. While in Asterisk we are
+	  tolerant of many different definitions of true and false in the
+	  database we only accept "yes" and "no". This change makes the
+	  field treated as a yes/no instead of an integer, thus storing
+	  "yes" and "no" instead of "1" and "0".
 
-2018-03-17 01:39 +0000 [8df4811cfa]  Corey Farrell <git@cfware.com>
+	  Change-Id: Ic8b9211b36babefe78f70def6828a135a6ae7ab6
 
-	* named_acl: Use ast_cli_completion_add.
+2018-07-26 18:54 +0000 [24e4e45177]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I317a82de976bbdbfe4352c243e32a7bb8f66c377
+	* CI: Fix placement of job summary statments
 
-2018-03-17 01:58 +0000 [a89323236f]  Corey Farrell <git@cfware.com>
+	  Change-Id: Iace19e718f4e8fb48eb7dc9f98af53b115cc45f3
 
-	* main/sounds: Use ast_cli_completion_add.
+2018-07-26 12:52 +0000 [c384a4cdcd]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I140e1137906bbfcdb61c0c6304159be459ad873e
+	* loader: Process dependencies for built-in modules.
 
-2018-03-17 01:09 +0000 [dc520b6d8f]  Corey Farrell <git@cfware.com>
+	  With the new module loader it was missed that built-in modules never
+	  parsed dependencies from mod->info into vectors of mod.  This caused
+	  manager to be initialized before acl (named_acl).  If manager.conf
+	  used any named ACL's they would not be found and result in no ACL being
+	  applied to the AMI user.
 
-	* manager: Use ast_cli_completion_add for completion generators.
+	  In addition to the manager ACL fix this adds "extconfig" to all builtin
+	  modules which support realtime configuration.  This only matters if one
+	  of the builtin modules is configured with 'preload', depending on
+	  "extconfig" will cause config.c to automatically be initialize during
+	  the preload stage.
 
-	  Change-Id: I658141c6ec490a3e866b02d2afea757928ceaabf
+	  Change-Id: I482ed6bca6c1064b05bb538d7861cd7a4f02d9fc
 
-2018-03-17 02:16 +0000 [534fda0a59]  Corey Farrell <git@cfware.com>
+2018-07-26 11:34 +0000 [9f1041c4d0]  George Joseph <gjoseph@digium.com>
 
-	* main/test: Use ast_cli_completion_add.
+	* CI:  Add docker info to job summary
 
-	  Change-Id: I5133ff2ba4e030f9733fb3d050c863d72a22ae6b
+	  Change-Id: I45d52005a9b692ad303c11792f226ace1e449901
 
-2018-03-16 10:19 +0000 [373e7e3fb0]  George Joseph <gjoseph@digium.com>
+2018-07-25 15:33 +0000 [c5761ee58e]  Kevin Harwell <kharwell@digium.com>
 
-	* channel.c:  Allow generic plc then channel formats are equal
+	* json.c: improve ast_json_to_ast_variables performance
 
-	  If the two formats on a channel are equal, we don't transcode and since
-	  the generic plc needs slin to work, it doesn't get invoked.
+	  When converting from a json object to an ast variables list the conversion
+	  algorithm was doing a complete traversal of the entire variables list for
+	  every item appended from the json structure.
 
-	  * A new configuration option "genericplc_on_equal_codecs" was added
-	    to the "plc" section of codecs.conf to allow generic packet loss
-	    concealment even if no transcoding was originally needed.
-	    Transcoding via SLIN is forced in this case.
+	  This patch makes it so the list is no longer traversed for each new ast
+	  variable being appended.
 
-	  ASTERISK-27743
+	  Change-Id: I8bf496a1fc449485150d6db36bfc0354934a3977
 
-	  Change-Id: I0577026a179dea34232e63123254b4e0508378f4
+2018-07-25 10:20 +0000 [cfd61ba237]  George Joseph <gjoseph@digium.com>
 
-2018-03-17 02:25 +0000 [bd926539b0]  Corey Farrell <git@cfware.com>
+	* CI: Explicitly pass BRANCH_NAME to buildAsterisk and installAsterisk
 
-	* main/translate: Use ast_cli_completion_add.
+	  Change-Id: I652f4a0ea5107c778e27a78bccb67b18b0c4e087
 
-	  Change-Id: I0e2402660e54d91f74ab0804c62a5b1925577413
+2018-07-24 13:29 +0000 [a81870110a]  George Joseph <gjoseph@digium.com>
 
-2018-03-17 02:00 +0000 [d202b56a74]  Corey Farrell <git@cfware.com>
+	* CI: Add options to initialize and cleanup database to runTestsuite.sh
 
-	* main/taskprocessor: Use ast_cli_completion_add.
+	  Change-Id: I352333233bab5377723bf37d490ba84fc55bc853
 
-	  Change-Id: Ie5f812a988ed811fd11967151932de62bc131b48
+2018-07-25 09:07 +0000 [4a01be5c80]  Corey Farrell <git@cfware.com>
 
-2018-03-17 00:51 +0000 [18cbfcf4f0]  Corey Farrell <git@cfware.com>
+	* CI: Do not `mkdir 2`.
 
-	* aco: Use ast_cli_completion_add for 'config show help'.
+	  Change-Id: Ib7377d26a6c98b38bad463f47c84f1875ac84eb7
 
-	  In addition this removes:
-	  * RAII_VAR usage
-	  * Duplicate check of pos
-	  * Unneeded arguments.
+2018-07-25 07:34 +0000 [e6f2bae0cc]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I2da8eac2670d1d8d6474c04037129804f55ebf39
+	* Build System: Silence build of bundled jansson.
 
-2018-03-15 15:06 +0000 [d57b58f5e9]  Corey Farrell <git@cfware.com>
+	  Change-Id: I7392c79c0173057f5378010bf1fe65e300e8fc56
 
-	* main/config: Use ast_cli_completion_add for reload completion.
+2018-07-25 07:13 +0000 [f1156f0cfd]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ia3fa4c03f2285a1ec8814bbe7f4624ead9111ad1
+	* CI: RefDebug: Fix reference to testsuite URL
 
-2018-03-17 04:31 +0000 [34e16cbf3a]  Corey Farrell <git@cfware.com>
+	  Change-Id: I0ee41d95a87f0d97b01f2757012b846bcfe6443d
 
-	* main/bridge: Use ast_cli_completion_add.
+2018-07-24 14:28 +0000 [7e99090c9d]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I3775a696d6a57139fdf09651ecb786bcf1774509
+	* Build System: Fix bundled jansson install.
 
-2018-03-17 16:41 +0000 [98a3baca70]  Corey Farrell <git@cfware.com>
+	  Update the bundled jansson Makefile to do nothing during Asterisk
+	  install, use a target that is not phony to initiate the jansson make and
+	  install.
 
-	* core: Minor cleanup of ast_el_read_char.
+	  Change-Id: I7643cc3d39af9feba8fc0da676b646efc5f8b3bb
 
-	  * Define CHAR_T_LIBEDIT and CHAR_TO_LIBEDIT based on
-	    HAVE_LIBEDIT_IS_UNICODE.  This avoids needing to repeatedly use
-	    conditional blocks, eliminates having multiple function prototypes.
-	  * Remove parenthesis from return values.
-	  * Add missing code block brackets {}.
-	  * Reduce use of 'else' conditional statements where possible.
+2018-07-24 10:43 +0000 [b32adca9b4]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I4315328ebea2f62641faf6881de2ac20a9f9d08e
+	* CI: Use bundled jansson if needed.
 
-2018-03-17 10:54 +0000 [7b40605d8e]  Alexander Traud <pabstraud@compuserve.com>
+	  Use pkg-config to determine if jansson is at least 2.11, enabled bundled
+	  version otherwise.
 
-	* BuildSystem: Check for header file of OGG.
+	  Change-Id: Ib555a8b72ff6f6925f9280ef035caa0b91ca4bd2
 
-	  Asterisk uses various symbols of the shared library libogg within the module
-	  format_ogg_vorbis. However, the source code of that module did not include the
-	  header file of libogg explicitly but implicitly. Because that header was not
-	  included before Asterisk 14, the script ./configure was told not to check for
-	  it.
+2018-07-24 05:39 +0000 [e22cbe7c17]  George Joseph <gjoseph@digium.com>
 
-	  Anyway, even Asterisk 13 LTS uses symbols of libogg. Therefore, that header
-	  should be included explicitly. Therefore, ./configure should check for that
-	  header.
+	* CI:  Add --privileged flag to docker options
 
-	  Change-Id: I98c50d56311b68880d1084fcc62c35ab2f8692db
+	  Change-Id: If92d55f15306e55dd7091ac3c47b13ebbbb03488
 
-2018-03-09 06:26 +0000 [06ba605988]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-24 05:22 +0000 [3509ada06f]  George Joseph <gjoseph@digium.com>
 
-	* BuildSystem: When no download utility is available, display the explanation.
+	* CI: Set correct user:group when publishing docs
 
-	  ./configure --with-pjproject-bundled
-	  did not display an explanation, when no download utility like wget, curl, or
-	  fetch was installed beforehand, although an explanation existed in code. This
-	  happened because the code expected the variable DOWNLOAD_TO_STDOUT to be empty.
-	  However, the script ./configure set that variable always.
+	  Change-Id: Ibabeb9ac730d9755cf54318d0da74771c939b86b
 
-	  Change-Id: I64c99b76a03525c69471e5055bf124b36a51bbd4
+2018-07-23 12:21 +0000 [008d304be2]  Richard Mudgett <rmudgett@digium.com>
 
-2018-03-17 05:00 +0000 [0f634c1446]  Alexander Traud <pabstraud@compuserve.com>
+	* core: AST_DEVMODE no longer affects ABI.
 
-	* BuildSystem: Remove unused dependency on libltdl.
+	  Remove AST_DEVMODE from the AST_BUILDOPTS list and the AST_BUILDOPTS_SUM
+	  calculation as it no longer affects API/ABI compatibility.
 
-	  Asterisk does not need the development package of libltdl, because it does not
-	  use any symbol of -lltdl directly. Instead, it uses the runtime package via the
-	  shared library -lodbc. On the supported platforms, that shared library declares
-	  its dependency on -lltdl correctly, otherwise AST_EXT_LIB_CHECK would have
-	  failed.
+	  Change-Id: Id5bd6dfade173a53b3a49f715586b86e3fb24acb
 
-	  ASTERISK-27745
+2018-07-23 10:23 +0000 [5dbbc68311]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Icd315809b8e7978203431f3afb66240dd3a040ba
+	* CI: Split --test-command argument.
 
-2018-03-05 06:50 +0000 [c0c537c1d3]  Alexander Traud <pabstraud@compuserve.com>
+	  The --test-command argument has now been split, unit tests now use
+	  `--unittest-command` and the testsuite uses --testsuite-command.
 
-	* BuildSystem: In NetBSD, the Python Programming Language is python-X.Y.
+	  This will make it easier to create a script which run everything by
+	  forwarding the same arguments to all CI scripts.
 
-	  ASTERISK-27717
+	  Change-Id: Ia54aa4848eaffbdf13175fcda40fc0b23080ad71
 
-	  Change-Id: If90ddf9c396c32e7402a894f42dce215c30049d1
+2018-07-21 11:58 +0000 [2a13a4344e]  Corey Farrell <git@cfware.com>
 
-2018-03-16 09:53 +0000 [afcd1dd8e5]  Alexander Traud <pabstraud@compuserve.com>
+	* CI: Fix mkdir CACHE_DIR.
 
-	* BuildSystem: Avoid an extra case for OpenBSD.
+	  Change-Id: Ic9f9a61e230047836c836206731f8ff7eb3538c9
 
-	  Nine years ago with Mantis 13639 (now ASTERISK-12841) an extra case for OpenBSD
-	  was introduced: Vorbis required Ogg to be specified manually, because the shared
-	  library libvorbis.so did not specify its required dependency on -logg itself.
+2018-07-22 10:41 +0000 [9742fb07c9]  Joshua Colp <jcolp@digium.com>
 
-	  Today with OpenBSD 6.2, all libvorbis*.so declare their dependencies correctly.
-	  Therefore, an extra case is not required anymore.
+	* sched: Make ABI compatible between dev mode and non-dev mode.
 
-	  Change-Id: Ifd04e0994ce9f1e4ad29c3948a0398b91d1e97bc
+	  In the past there was an assertion in the ast_sched_del function
+	  and in order to ensure it was useful the calling function name,
+	  line number, and filename had to be passed in. This cause the ABI
+	  to be different between dev mode and non-dev mode.
 
-2018-03-05 10:17 +0000 [5b80e97fff]  Alexander Traud <pabstraud@compuserve.com>
+	  This assertion is no longer present so the special logic can be
+	  removed to make it the same between them both.
 
-	* BuildSystem: Enable Advanced Linux Sound Architecture (ALSA) in NetBSD.
+	  Change-Id: Icbc69c801e357d7004efc5cf2ab936d9b83b6ab8
 
-	  In the script ./configure, AST_EXT_LIB_CHECK checks for external libraries. Some
-	  libraries do not specify all their dependencies and require additional shared
-	  libraries. In AST_EXT_LIB_CHECK, this is the fifth parameter. However, if a
-	  library is specified there, it must exist on the platform, because ./configure
-	  tries to compile/link/execute a small app using those statements. For example,
-	  the library libdl.so is Linux specific and does not exist on BSD-like platforms.
+2018-07-20 15:52 +0000 [2c51079d05]  Richard Mudgett <rmudgett@digium.com>
 
-	  Furthermore, no supported platform/version was found, which still (ever?)
-	  requires those additional libraries. Therefore, they were simply removed.
+	* asterisk.c: Update displayed copyright year for v16 release.
 
-	  Finally, this change adds the error code ESTRPIPE to the channel driver
-	  chan_alsa for those platforms which lack it, again for example NetBSD.
+	  Change-Id: I60622731d928ee9506b1d28934095f0dc3e5306e
 
-	  ASTERISK-27720
+2018-07-16 15:08 +0000 [3cdffa1342]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I3b21f2135f6cbfac7590ccdc2df753257f426e0b
+	* Enable bundling of jansson, require 2.11.
 
-2018-03-16 09:02 +0000 [d5af24bb14]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ib3111b151d37cbda40768cf2a8a9c6cf6c5c7cbd
 
-	* app_voicemail:  Fix json blob errors
+2018-07-20 09:25 +0000 [136d855f69]  Corey Farrell <git@cfware.com>
 
-	  When app_voicemail calls ast_test_suite_notify with the results of
-	  a user keypress, it formats the keypress as '%c'.  If the user hung up
-	  or some other error occurrs, the result of the keypress is a non
-	  printable character.  This ultimately causes json_vpack_ex to think
-	  it's being passed a non utf-8 string and return an error.
+	* CI: Fix logger.conf for unit tests.
 
-	  * Keypress results passed to ast_test_suite_notify are now checked with
-	    isprint() and a '?' is substituted if the check fails.
+	  Change-Id: Idea59d60eab20105de50b34f0f0d506e6ef55d5c
 
-	  Change-Id: I78ee188916bbac840f3d03f40201b692347ea865
+2018-07-19 10:34 +0000 [0c1513d8a0]  George Joseph <gjoseph@digium.com>
 
-2018-02-27 03:01 +0000 [69463c612d]  Florian Floimair <f.floimair@commend.com>
+	* CI:  Add wiki doc publish to periodics
 
-	* app_dial: Enable early-media video
+	  Change-Id: I29ba26134e5083bc6788ede235f1a5d4383c148a
 
-	  Certain applications (e.g. door-phone) require that also video is transmitted
-	  before a call is accepted.
+2018-07-20 06:20 +0000 [61a974ed4e]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I9842e1dc2f6e1c2c49dc33fe615255007d2f821e
+	* xmldoc.c:  Fix dump of xml document
 
-2018-03-15 09:32 +0000 [a7ebb9409d]  Corey Farrell <git@cfware.com>
+	  The "xmldoc dump" cli command was simply concatenating xml documents
+	  into the output file.  The resulting file had multiple "xml"
+	  processing instructions and multiple root elements which is illegal.
+	  Normally this isn't an issue because Asterisk has only 1 main xml
+	  documentation file but codec_opus has its own file so if it's
+	  downloaded and you do "xmldoc dump", the result is invalid.
 
-	* main/cdr: Use ast_cli_completion_add for CDR channel completion.
+	  * Added 2 new functions to xml.c:
+	      ast_xml_copy_node_list creates a copy of a list of children.
+	      ast_xml_add_child_list adds a list to an existing list.
 
-	  Change-Id: Ie81830647a23aad61c1162583b6d50adbe6e7822
+	  * Modified handle_dump_docs to create a new output document and
+	    add to it the children from each input file.  It then dumps the
+	    new document to the output file.
 
-2018-03-15 08:19 +0000 [5ac64a4464]  Corey Farrell <git@cfware.com>
+	  Change-Id: I3f182d38c75776aee76413dadd2d489d54a85c07
 
-	* main/ccss: Use ast_cli_completion_add for core id.
+2018-07-20 06:54 +0000 [50a26b15a3]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I44b25d6d24c7d9bc1bb38a50774b38883162f98f
+	* res_pjsip: Update default keepalive interval to 90 seconds.
 
-2018-03-12 10:24 +0000 [e04be32fc5]  Alexander Traud <pabstraud@compuserve.com>
+	  A change recently went in which disabled the built-in PJSIP
+	  keepalive. This defaulted to 90 seconds and kept TCP/TLS
+	  connections alive. Disabling this functionality has resulted
+	  in a behavior change of not doing keepalives by default resulting
+	  in TCP/TLS connections dropping for some people.
 
-	* install_prereq: Add Arch Linux.
+	  This change makes our default keepalive interval 90 seconds
+	  to match the previous behavior and preserve it.
 
-	  ASTERISK-27738
+	  ASTERISK-27978
 
-	  Change-Id: I7ca620e3c4dfb4b064a19382c4915aeb42a2a09f
+	  Change-Id: Ibd9a45f3cbe5d9bb6d2161268696645ff781b1d6
 
-2018-03-15 04:49 +0000 [eb699f3631]  Corey Farrell <git@cfware.com>
+2018-07-18 14:19 +0000 [958f76205b]  Joshua Colp <jcolp@digium.com>
 
-	* core: Backport compatible MALLOC_DEBUG changes.
+	* Update mainline version for the 16 branch.
 
-	  * Add support for MALLOC_DEBUG and DEBUG_CHAOS to be used together.
-	  * Add utils/astmm.c to .gitignore.
-	  * Fix MALLOC_DEBUG variant of __ast_vasprintf.  This function called
-	    va_end(ap) upon allocation failure.  This is incorrect since ap is
-	    passed as an argument.
+	  Change-Id: I4d36277d10335349d83ae218fa10fee99c3e4c14
 
-	  Change-Id: I9f27ced4ce3cbe4b39547a67f994fdff491978c0
+2018-07-18 14:18 +0000 [e7a76ffee1]  Joshua Colp <jcolp@digium.com>
 
-2018-03-15 07:29 +0000 [7e0c56f800]  Corey Farrell <git@cfware.com>
+	* Update ARI version for master/16.
 
-	* astobj2_container: Use ast_cli_completion_add for container names.
+	  ARI goes from 3.0.0 to 4.0.0
 
-	  Change-Id: I4f0fc09e820eb8d8da2354a177dbcf503c56ddd1
+	  Change-Id: I0649fa34926dc4fc89a166f1d2e3bbd965ef9ebe
 
-2017-12-09 04:52 +0000 [7e041d6233]  Corey Farrell <git@cfware.com>
+2018-05-29 09:31 +0000 [fe78d374b0]  Alexander Traud <pabstraud@compuserve.com>
 
-	* main/channel: Use ast_cli_completion_add for channeltypes.
+	* pjproject_bundled: Repair ./configure --with-ssl=PATH.
 
-	  Change-Id: Ia845fae6a84801cc7d9996767b99efb2753cbb48
+	  Previously, Asterisk did not tell its bundled PJProject about this configure
+	  parameter. Therefore, PJProject used the platform provided OpenSSL always.
 
-2018-03-14 12:38 +0000 [6539b89254]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27880
 
-	* cli: Enable ast_cli_completion_add on public completion generators.
+	  Change-Id: Iea545aec854dd0e2c061c69bb118a76ce56c5dc6
 
-	  * ast_cli_complete
-	  * ast_complete_channels
-	  * ast_complete_applications
+2018-05-10 13:11 +0000 [5bacde37a2]  Ben Ford <bford@digium.com>
 
-	  These generators will now use ast_cli_completion_add if state == -1.
+	* res_rtp_asterisk: Add support for sending NACK requests.
 
-	  Change-Id: I7ff311f0873099be0e43a3dc5415c0cd06d15756
+	  Support has been added for receiving a NACK request and handling it.
+	  Now, Asterisk can detect when a NACK request should be sent and knows
+	  how to construct one based on the packets we've received from the remote
+	  end. A buffer has been added that will store out of order packets until
+	  we receive the packet we are expecting. Then, these packets are handled
+	  like normal and frames are queued to the core like normal. Asterisk
+	  knows which packets to request in the NACK request using a vector
+	  which stores the sequence numbers of the packets we are currently missing.
 
-2018-03-14 11:17 +0000 [96eaabd920]  Ross Beer <ross.beer@voicehost.co.uk>
+	  If a missing packet is received, cycle through the buffer until we reach
+	  another packet we have not received yet. If the buffer reaches a certain
+	  size, send a NACK request. If the buffer reaches its max size, queue all
+	  frames to the core and wipe the buffer and vector.
 
-	* res_pjsip_rfc3326.c: Account for more than one 'Reason' header
+	  According to RFC3711, the NACK request must be sent out in a compound
+	  packet. All compound packets must start with a sender or receiver
+	  report, so some work was done to refactor the current sender / receiver
+	  code to allow it to be used without having to also include sdes
+	  information and automatically send the report.
 
-	  ASTERISK-27741
+	  Also added additional functionality to ast_data_buffer, along with some
+	  testing.
 
-	  Change-Id: I0aa59a54735c6d20b95c54db1bd095dbf93e7adf
+	  For more information, refer to the wiki page:
+	  https://wiki.asterisk.org/wiki/display/AST/WebRTC+User+Experience+Improvements
 
-2018-03-12 08:10 +0000 [b237ef7f27]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27810 #close
 
-	* install_prereq: Add SUSE.
+	  Change-Id: Idab644b08a1593659c92cda64132ccc203fe991d
 
-	  ASTERISK-27736
+2018-07-18 11:12 +0000 [59323121f3]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I4cafc8973349d50a7cb7919ddf0bb1aaef4bfc3e
+	* res_sorcery_config: Allow configuration section to be used based on name.
 
-2018-03-13 16:37 +0000 [dc738b145f]  Corey Farrell <git@cfware.com>
+	  A problem I've seen countless times is a global or system section
+	  for PJSIP not getting applied. This is inevitably the result of
+	  the "type=" line missing. This change alleviates that problem.
 
-	* core: Remove incorrect usage of attribute_malloc.
+	  The ability to specify an explicit section name has been
+	  added to res_sorcery_config. If the configured section
+	  name matches this and there are no unknown things configured
+	  the section is taken as being for the given type.
 
-	  GCC documentation states that when __attribute__((malloc)) is used it
-	  should not return storage which contains any valid pointers.  It
-	  specifically mentions that realloc functions should not have the malloc
-	  attribute, but this also means that complex initializers which could
-	  contain initialized pointers should not use this attribute.
+	  Both the PJSIP "global" and "system" types now support this
+	  so you can just name your section "global" or "system" and it
+	  will be matched and used, even without a "type=" line.
 
-	  Change-Id: If507f33ffb3ca3b83b702196eb0e8215d27fc7d2
+	  ASTERISK-27972
 
-2018-03-12 05:19 +0000 [7533d25e8d]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ie22723663c1ddd24f869af8c9b4c1b59e2476893
 
-	* BuildSystem: Enable IMAP storage on openSUSE and Arch Linux.
+2018-07-17 05:24 +0000 [134e2f0ddc]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-27734
+	* module: Remove deprecated modules and update support levels.
 
-	  Change-Id: I8d6e6a1c08c031649764f5277fbbb85e57c3a9d4
+	  I have removed the STATIC_BUILD option immediately as it has not
+	  been maintained in many years and is non-functional.
 
-2018-03-12 04:22 +0000 [09c43fdc1a]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27965
 
-	* res_srtp: Add support for libsrtp2.x on openSUSE.
+	  Change-Id: I64783d017b86dba9ee3c7bcfb97e59889a3f76d7
 
-	  Since ASTERISK-26976, libSRTP 2.x can be used for sRTP. However, that change
-	  added a private header which is not available on openSUSE for example. To
-	  remain compatibility with very old libSRTP versions, the affected/missing
-	  symbols AES_128_ICM and HMAC_SHA1 are defined manually.
+2018-07-18 11:34 +0000 [94dd0544e5]  Chris-Savinovich <csavinovich@digium.com>
 
-	  ASTERISK-27733
+	* stasis: Improve message type "Use of before/init after destruction"
 
-	  Change-Id: I25c5cb8fa966043d1506ebef449e5a724412b4b6
+	  Fixes issue where error msg
+	  "Use of before/init after destruction"
+	  was being printed on disabled messages
+	  in dev mode.  With this
+	  fix if message is disabled
+	  a warning will print.
 
-2018-03-10 03:33 +0000 [c09a10bb1b]  Corey Farrell <git@cfware.com>
+	  ASTERISK-25548
+	  Change-Id: Ie0d866d1cbc60c16dbef08bc65e99505c3c1adfa
 
-	* core: Remove non-critical cleanup from startup aborts.
+2018-07-17 14:12 +0000 [993ba84cd3]  Nick French <naf@ou.edu>
 
-	  When built-in components of Asterisk fail to start they cause the
-	  Asterisk startup to abort.  In these cases only the most critical
-	  cleanup should be performed - closing databases and terminating
-	  proceses.  These cleanups are registered using ast_register_atexit, all
-	  other cleanups should not be run during startup abort.
+	* SRTP: Lower SDES key lifetime minimum to 2^20
 
-	  The main reason for this change is that these cleanup procedures are
-	  untestable from the partially initialized states, if they fail it could
-	  prevent us from ever running the critical cleanup with ast_run_atexits.
+	  SRTP SDES key lifetime support was added in ASTERISK_17899.
 
-	  Change-Id: Iecc2df98008b21509925ff16740bd5fa29527db3
+	  In that addition, the minimum key lifetime to be accepted was
+	  set at the 10 hours @ 20ms/packet = 1800000 packets.
 
-2018-03-08 09:14 +0000 [bd6e0b1a72]  Alexander Traud <pabstraud@compuserve.com>
+	  The firmware in the obi1xx ATA uses a hardcoded lifetime of
+	  2^20 packets.
 
-	* BuildSystem: Add NetBSD.
+	  Lower the limit to 2^20 to support a wider field of clients.
 
-	  Headers, libraries, and rpath.
+	  ASTERISK-27967 #close
 
-	  ASTERISK-27728
-	  ASTERISK-11015
-	  Reported by: Curt Sampson
+	  Change-Id: I81a0703c595a0c9101dfdf02300149a3cc39bf94
 
-	  Change-Id: I50aa5fcd095937df32a2e33307caac7e79a8b5b7
+2018-07-17 11:09 +0000 [fcc0a6fe8a]  George Joseph <gjoseph@digium.com>
 
-2018-03-09 03:23 +0000 [ba88af8a25]  Alexander Traud <pabstraud@compuserve.com>
+	* CI: Fix merge strategy
 
-	* BuildSystem: For consistency, avoid double-checking via if clauses.
+	  Change-Id: I5e3fb6adfa6cbf694c0deecf02e3879297b0c12e
 
-	  In the script ./configure, AST_EXT_LIB_CHECK and AST_PKG_CONFIG_CHECK first test
-	  whether parameter 1 was already found. Consequently, an if-test on PBX_ just a
-	  line below is redundant, if exactly the same parameter 1 is used again.
+2018-07-17 10:41 +0000 [3e5a6a6cfc]  George Joseph <gjoseph@digium.com>
 
-	  No performance gain is expected by this change. However, because this strategy
-	  is used all over in ./configure except for two places, this change aims to
-	  create more consistency: Only do something different if there is a reason to do
-	  so.
+	* CI: Fix regex in daily and ref_debug jobs
 
-	  Change-Id: I4a6f48127b7af3a48168c917e888be1f70625027
+	  Change-Id: Icf2e67818b2155a158d2390b138613e1f653ea92
 
-2018-03-09 02:44 +0000 [c227462370]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-17 09:09 +0000 [0e8976116f]  Nick French <naf@ou.edu>
 
-	* BuildSystem: Enable dladdr on non-Linux platforms like FreeBSD.
+	* res_pjsip:  Remove spurious error logging when printing silent headers
 
-	  ASTERISK-27641
+	  Asterisk patched the pjproject source to avoid crashing when pjproject
+	  sip_msg headers are encountered with NULL vptr's, but the patch also
+	  output error messages for some valid headers which simply did not need
+	  to be added to the message itself, such as hidden route headers.
 
-	  Change-Id: I587e8ba0123c70fc10cfd8b0ac3299551f61d84b
+	  pjproject has since applied a similar patch to their baseline to avoid
+	  crashes, but their version also avoids the spurious error logging.
 
-2018-03-07 13:50 +0000 [9ae427c7fd]  Richard Mudgett <rmudgett@digium.com>
+	  Lets use their patch instead.
 
-	* res_pjproject.c: Upgrade bundled PJPROJECT to 2.7.2
+	  ASTERISK-27961 #close
 
-	  Update patches included in bundled PJPROJECT for the new version.
+	  Change-Id: I2ddbd82c8da10e0dcc9807a48089d1f3c2d6e389
 
-	  ASTERISK-27730
+2018-07-17 10:15 +0000 [fa333dedd0]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Id3c8c8ad82126846bcd9768bc3d0a18d89be8944
+	* CI: Add pre-build merge back in as RECURSIVE
 
-2018-03-08 12:05 +0000 [57e8754a3a]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I0ff1730ef4a4f0ac9f18ccc9bc0dfe7a782f57a8
 
-	* install_prereq: Add NetBSD.
+2018-07-17 09:01 +0000 [2553255ace]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27729
+	* CI: Remove pre-build merge from gates and checks
 
-	  Change-Id: I7a706d51375d54cf5e36d32397bfe09a48670804
+	  Change-Id: Ibc151f63dcec4db847915c2f3cbe5b467dd59574
 
-2018-03-08 09:04 +0000 [0d70a9a006]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-17 07:13 +0000 [524f900382]  George Joseph <gjoseph@digium.com>
 
-	* BuildSystem: Re-check for another UUID library only when previous check failed.
+	* CI: Fix logic inversion in runTestsuite
 
-	  As a side-effect, this avoids the ambiguous output:
-	   checking for uuid_generate_random... no
-	  which was printed always previously.
+	  Change-Id: I56399aa384468f45494c2c3650420563a0b6efe1
 
-	  ASTERISK-25586
-	  Reported by: John Nemeth
+2018-07-17 04:03 +0000 [0af4a558da]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I6d541dfcf453932a9856c5e251aa22e0e6c233c9
+	* CI: Add teardownRealtime
 
-2018-03-08 05:32 +0000 [7b6b6b0a7e]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I2fe55c38607eaec2fbf69ef23a5019e0c443a64b
 
-	* BuildSystem: Instead of $PJPROJECT_LIBS with s, use $PJPROJECT_LIB everywhere.
+2018-07-15 13:58 +0000 [49f83a7490]  Corey Farrell <git@cfware.com>
 
-	  In the script ./configure,
-	  xyz_LIB  is set by AST_PKG_CONFIG_CHECK and
-	  xyz_LIBS is set by PKG_CHECK_MODULES within
-	  AST_PKG_CONFIG_CHECK. Both are the same. In Asterisk normally the former and
-	  only three times the latter was used. Let us use xyz_LIB without s, for
-	  consistency with AST_EXT_LIB_CHECK. That eases understanding because now readers
-	  do not have to know that xyz_LIB equals xyz_LIBS.
+	* loader: Fix startup issues.
 
-	  Change-Id: I7359860a5d730cdc784c2c48e501a082196434d3
+	  * Merge the preload and load stages, use load ordering to try preload's
+	    first.  This fixes an issue where `preload=res_config_curl` would fail
+	    unless res_curl and func_curl were also preloaded.  Now it is only
+	    required that those modules be loaded during startup: autoload or
+	    regular load is good enough.
+	  * The configuration option `require` and `preload-require` were only
+	    effective if the modules failed to load.  These options will now abort
+	    Asterisk startup if required modules fail to reach the 'Running'
+	    state.
+	  * Missing or invalid 'module.conf' did not prevent startup.  Asterisk
+	    doesn't do anything without modules so this a fatal error.
 
-2018-03-06 06:28 +0000 [54874eb44d]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ie4176699133f0e3a823b43f90c3348677e43a5f3
 
-	* BuildSystem: Enable PortAudio in NetBSD.
+2018-07-16 13:30 +0000 [a9cef123d9]  George Joseph <gjoseph@digium.com>
 
-	  In NetBSD, PortAudio 1 is still the default version. PortAudio 2 can be
-	  installed side by side but gets placed in a 'portaudio2' subdirectory. To
-	  find PortAudio 2 even in a subdirectory, the tool pkg-config is queried via
-	  AST_PKG_CONFIG_CHECK. For those platforms, which do not list PowerAudio 2
-	  via pkg-config, the previous check remains and is executed thereafter.
+	* CI:  Prevent Jenkins from triggering jobs back to itself
 
-	  ASTERISK-27721
+	  Change-Id: I9cae8bb3d1a2cea335d3ccd88d471832549666fd
 
-	  Change-Id: I4175500126909ad1b181fff8e11bb4a3a6ae4fa9
+2018-07-13 18:26 +0000 [5febc995df]  Richard Mudgett <rmudgett@digium.com>
 
-2018-03-07 00:29 +0000 [b81eadcefc]  Corey Farrell <git@cfware.com>
+	* Build: Fix modules getting their optimization setting overridden.
 
-	* Replace direct checks of option_debug with DEBUG_ATLEAST macro.
+	  Asterisk modules that use PJPROJECT services have their compiler
+	  optimization and possibly their symbolic debug options overridden by the
+	  PJPROJECT configure script selected settings.
 
-	  Checking option_debug directly is incorrect as it ignores file/module
-	  specific debug settings.  This system-wide change replaces nearly all
-	  direct checks for option_debug with the DEBUG_ATLEAST macro.
+	  * We need to filter-out any -O and -g options in PJ_CFLAGS before echoing
+	  out the result so the PJPROJECT_INCLUDE variable does not override the
+	  Asterisk module settings when using bundled PJPROJECT.
 
-	  Change-Id: Ic342d4799a945dbc40ac085ac142681094a4ebf0
+	  NOTE: This patch only has an effect when using bundled PJPROJECT.
 
-2018-03-07 13:13 +0000 [342aec5980]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27563
 
-	* BuildSystem regression: Fix errors reported by clean targets.
+	  Change-Id: If124169735ecf572ad1535cd43bff94cb44d5b30
 
-	  Doing a 'make clean', 'make distclean', or 'make dist-clean' gets errors
-	  about an invalid shell option: "/bin/sh: 0: Illegal option -".
+2018-07-16 11:08 +0000 [d15ef68892]  George Joseph <gjoseph@digium.com>
 
-	  The clean targets do not include the makeopts file which defines GREP and
-	  LDCONFIG because the file may not exist and the distclean/dist-clean
-	  targets will delete it anyway.
+	* CI: runUnittests: loop a few times on waitfullybooted
 
-	  ASTERISK-27715
+	  Change-Id: Icebc0d013896f3b2a7214945cac60647435c1651
 
-	  Change-Id: I33d40acdb03862bc89aeb6fb1ff497894a8ea7f5
+2018-07-16 10:49 +0000 [252c4284df]  George Joseph <gjoseph@digium.com>
 
-2018-03-06 19:08 +0000 [0597e72e1d]  Sungtae Kim <pchero21@gmail.com>
+	* CI:  Add realtime checks to dailies
 
-	* voicemail: Fixed wrong voicemail message count
+	  Change-Id: I6dc8ab1679b3505c6dde1d47e1b9276df47814f8
 
-	  Fixed wrong voicemail mailbox reference for Action: VoicemailUsersList.
+2018-07-16 09:13 +0000 [1a52ab70c7]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27703
+	* CI:  Add weekly REF_DEBUG testsuite run
 
-	  Change-Id: I99bfec14bd4ae475b0fa1fac5a7992f3e2e8d64a
+	  Change-Id: I5b581d0a0d1d1bb9b38961d40b112fb448355037
 
-2018-02-14 07:33 +0000 [a35a654a52]  Jean Aunis <jean.aunis@prescom.fr>
+2018-07-16 08:44 +0000 [9633e9dfd7]  George Joseph <gjoseph@digium.com>
 
-	* chan_sip: Fix improper RTP framing on outgoing calls
+	* CI: Fix bad reporting of status by the verification pub
 
-	  The "ptime" SDP parameter received in a SIP response was not honoured.
-	  Moreover, in the abscence of this "ptime" parameter, locally configured
-	  framing was lost during response processing.
+	  Change-Id: I6f31a130b3ba0187149aaaa2ce94195a79e0f6a6
 
-	  This patch systematically stores the framing information in the
-	  ast_rtp_codecs structure, taking it from the response or from the
-	  configuration as appropriate.
+2018-07-16 07:16 +0000 [b8d75bbb37]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27674
+	* CI: Make build tag an acceptable docker name
 
-	  Change-Id: I828a6a98d27a45a8afd07236a2bd0aa3cbd3fb2c
+	  Change-Id: I3a4b8a4a9c488ddabf9daf651dc1334222056f38
 
-2018-03-06 13:31 +0000 [ecff05d51c]  Ross Beer <ross.beer@voicehost.co.uk>
+2018-07-13 22:44 +0000 [0885ab8afc]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_rfc3326: Order of 'Reason' headers break many endpoints
+	* Fix declaration of PBX_CURL for ./configure --without-libcurl
 
-	  ASTERISK-27554
+	  When `--without-libcurl` is used PBX_CURL is never set.  Set default
+	  value 0 so the proper value is passed to menuselect.
 
-	  Change-Id: If61c7faab7d2fa1031c056ed6268fe928e2391cf
+	  Change-Id: I03e2842a00899cbca2dbde52bb1f6636d54bae1e
 
-2018-03-07 09:32 +0000 [725fd32428]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-10 13:28 +0000 [34f3fe9552]  George Joseph <gjoseph@digium.com>
 
-	* utils: In Solaris, avoid a warning about an unused variable.
+	* app_confbridge:  Use the SDP 'label' attribute to correlate users
 
-	  When HAVE_GETHOSTBYNAME_R_5 was set by the script ./configure, GCC 7.3.0 found
-	  an unused variable. Actually, the variable was used (set to a dummy value) but
-	  the compiler optimization might have removed that. Instead, this change ensures
-	  that the variable 'res' is only used when it is really required.
+	  Previously, the msid "label" attribute was used to correlate
+	  participant info but because streams could be reused, the msid
+	  wasn't being updated correctly when someone left the bridge and
+	  another joined.
 
-	  Change-Id: Ic3ea23ccf84ac4bc2d501b514985b989030abab5
+	  Now, instead of looking for the msid attribute on a channel's streams,
+	  app_confbridge sets an "SDP:LABEL" attribute on the stream which
+	  res_pjsip_sdp_rtp looks for.  If it finds it, it adds a "label"
+	  attribute to the current sdp.
 
-2018-03-05 08:01 +0000 [93df498f4e]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I6cbaa87fb59a2e0688d956e72d2d09e4ac20d5a5
 
-	* BuildSystem: Enable Lua in NetBSD.
+2018-07-13 06:56 +0000 [e8727fcfa8]  George Joseph <gjoseph@digium.com>
 
-	  luaL_openlib got removed with Lua 5.2.
-	  luaL_newstate is available in all versions.
+	* CI: Add daily periodics to CI
 
-	  ASTERISK-27718
+	  Change-Id: I26933e73928e091ae72e838c02f4f2ec7c3983d6
 
-	  Change-Id: I9c8c8880315ee36ab740d7c40153306c0bfd6f71
+2018-07-11 11:57 +0000 [e19080a184]  Alexander Traud (License 6520)
 
-2018-03-06 07:33 +0000 [44dbb8251d]  Alexander Traud <pabstraud@compuserve.com>
+	* Bundled PJPROJECT: Disable internal connection oriented keep-alive.
 
-	* BuildSystem: Depend not implicitly but explicitly on external libraries.
+	  Turn off the periodic sending of CRLNCRLN.  Default is on (90 seconds),
+	  which conflicts with the global section's keep_alive_interval option in
+	  pjsip.conf.
 
-	  ASTERISK-27722
+	  patches:
+	    pjsip_keep_not_alive.patch submitted by Alexander Traud (License 6520)
 
-	  Change-Id: Ie7b8c30d86cb00a54d6ac4e09e6f28f42d2bd52c
+	  ASTERISK-27347
 
-2018-03-05 08:15 +0000 [2d63c185f0]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I6a197f56e1830d3b7e5ec70f17025840a290b057
 
-	* res_http_post: Enable GMime in NetBSD.
+2018-07-09 04:42 +0000 [1445384699]  Torrey Searle <torrey@voxbone.com>
 
-	  ASTERISK-27719
+	* res_pjsip_sdp_rtp: include ice in ANSWER only if offered
 
-	  Change-Id: I230c5f9f316b2e9465c093c13580f72ebbaf67a7
+	  Keep track if ICE candidates were in the SDP offer & only put them
+	  in the corresponding SDP answer if the offer condaind ICE candidates
 
-2018-03-05 04:16 +0000 [0af1446d6e]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27957 #close
 
-	* BuildSystem: Enable autotools in NetBSD.
+	  Change-Id: Idf2597ee48e9a287e07aa4030bfa705430a13a92
 
-	  ASTERISK-27716
+2018-07-12 16:34 +0000 [33a84745d0]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I52525e35e1620341272219911d054a1e3d3ec01e
+	* CI: Add Asterisk Gates
 
-2018-03-05 03:42 +0000 [95ecbaa6b8]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I7e2467f9120812551238d8005deb97f965279205
 
-	* BuildSystem: AC_PATH_PROG sets to colon character when not found.
+2018-07-11 15:55 +0000 [65b002ab8f]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27715
-	  Reported by: Corey Farrell
+	* CI: Remove duplicate checkout
 
-	  Change-Id: I0d6d9572d1352dc7ad30c9917173f1e980d8c938
+	  Change-Id: If5f925b4c4ed7000b153f3ed8386ce2140c886f8
 
-2018-03-03 09:06 +0000 [7b5e0960ef]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-11 15:09 +0000 [ba8f8a2813]  George Joseph <gjoseph@digium.com>
 
-	* chan_unistim: NetBSD has an incompatible struct in_pktinfo.
+	* CI: Update cleanup steps and permissions
 
-	  ASTERISK-27714
-	  Reported by: John Nemeth
+	  Change-Id: I7ca92935979d94845af8e1caf4468cbd6209b7de
 
-	  Change-Id: I1b84a89315a5f61222123d21bf35c59224da8990
+2018-07-11 14:54 +0000 [ad36c4ba9b]  George Joseph <gjoseph@digium.com>
 
-2018-03-03 08:30 +0000 [6b899b2849]  Alexander Traud <pabstraud@compuserve.com>
+	* CI: Fix log artifact paths
 
-	* BuildSystem: Cast any intptr_t explicitly to its proposed type.
+	  Change-Id: I55136de8f4d9c3b56bd4d054306a187bb04a4b7d
 
-	  ASTERISK-27713
+2018-07-11 14:45 +0000 [4842af6364]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I90c769e3c7f8c26de8a3af11335862cec15a1b22
+	* CI: Remove CleanBeforeCheckout option for testsuite
 
-2018-03-03 06:56 +0000 [98e8e849da]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I510231c9087f7be5272b8ef3f3223eadaaffb754
 
-	* BuildSystem: Detect whether uselocale(.) is available.
+2018-07-11 14:00 +0000 [3dfc37c60a]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27712
-	  Reported by: Joerg Sonnenberger, D'Arcy Cain
+	* CI: Move gates into source repo
 
-	  Change-Id: Idf1c9d43617a3e13028b95b313415903d80ef807
+	  Change-Id: If028ede5f3b127fa274c63ce166bc04ad7c1e5db
 
-2018-03-03 03:53 +0000 [8348862103]  Alexander Traud <pabstraud@compuserve.com>
+2018-07-11 06:14 +0000 [b302ee6bd5]  George Joseph <gjoseph@digium.com>
 
-	* BuildSystem: Avoid re-defining of pthread_* on NetBSD.
+	* CI:  Initial commit for moving CI into source repo
 
-	  ASTERISK-27711
+	  Create tests/CI directory and add files used by Jenkins to
+	  build and test Asterisk.
 
-	  Change-Id: Idc9194035b2958b99f6b01eb5b438d45a074565b
+	  With this commit, Jenkins will run the Asterisk Unit Tests using
+	  the Jenkinsfile at tests/CI/unittests.jenkinsfile.  Bash scripts
+	  to do the actual building and testing are also in the same directory.
+	  Output is placed in tests/CI/output so that directory has been
+	  added to .gitignore.
 
-2018-03-02 07:05 +0000 [47d45ba025]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I9448065465e6de2b878634510ace8fd1ef378608
 
-	* BuildSystem: Install init scripts on openSUSE Tumbleweed.
+2018-07-06 17:00 +0000 [f7137e1230]  Joshua Elson <joshelson@gmail.com>
 
-	  ASTERISK-27710
+	* res_parking: Add dialplan function for lot channel
 
-	  Change-Id: I4c777e41b31d4415bbe21cb435ad47b43ebb5467
+	  This commit adds a new function to res_parking.
 
-2018-03-02 05:12 +0000 [4035582b64]  Alexander Traud <pabstraud@compuserve.com>
+	  This function, PARK_GET_CHANNEL allows the retrieval
+	  of the channel name of the channel occupying the parking slot.
 
-	* BuildSystem: Avoid == for comparison in ./configure.
+	  ASTERISK-22825 #close
 
-	  ASTERISK-27709
-	  Reported by: John Nemeth
+	  Change-Id: Idba6ae55b8a53f734238cb3d995cedb95c0e7b74
 
-	  Change-Id: I11b1ae8fd404c04066f1458f5d71f9536359d58d
+2018-06-23 01:33 +0000 [10de9fcbf1]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-02-27 15:40 +0000 [104468ad3a]  Richard Mudgett <rmudgett@digium.com>
+	* chan_ooh323: IPTOS_MINCOST is not defined on Solaris.
 
-	* pjproject: Add cache_pools debugging option.
+	  Furthermore, <sys/sockio.h> is required for SIOCGIF*.
 
-	  The pool cache gets in the way of finding use after free errors of memory
-	  pool contents.  Tools like valgrind and MALLOC_DEBUG don't know when a
-	  pool is released because it gets put into the cache instead of being
-	  freed.
+	  ASTERISK-27938
 
-	  * Added the "cache_pools" option to pjproject.conf.  Disabling the option
-	  helps track down pool content mismanagement when using valgrind or
-	  MALLOC_DEBUG.  The cache gets in the way of determining if the pool
-	  contents are used after free and who freed it.
+	  Change-Id: Idc9153ece769944765b66122efb11728d8d8ebde
 
-	  To disable the pool caching simply disable the cache_pools option in
-	  pjproject.conf and restart Asterisk.
+2018-07-06 15:05 +0000 [5bb874ee09]  Kevin Harwell <kharwell@digium.com>
 
-	  Sample pjproject.conf setting:
-	  [startup]
-	  cache_pools=no
+	* res_pjsip_session: sdp group:BUNDLE attribute being truncated
 
-	  * Made current users of the caching pool factory initialization and
-	  destruction calls call common routines to create and destroy cached pools.
+	  When setting/appending the media id's to the bundle group attribute a '-1' was
+	  being passed to the 'ast_str_set/append' function for the 'max_len' parameter.
+	  This essentially capped the length of the string to what it was originally
+	  allocated with. In this case 64 bytes.
 
-	  ASTERISK-27704
+	  This patch makes it so a '0' is passed as in for the 'max_len', which means
+	  "no maximum length".
 
-	  Change-Id: I64d5befbaeed2532f93aa027a51eb52347d2b828
+	  ASTERISK-27955 #close
 
-2018-01-31 11:49 +0000 [41894965f2]  Corey Farrell <git@cfware.com>
+	  Change-Id: Iec565df6600401d54a502854a53d19bb4cc34876
 
-	* gitreview: Reorder and add padding.
+2018-07-05 16:02 +0000 [96abe79ddf]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: I459dc320a8c9452a01eed6f403d786741587c890
+	* res_pjsip_pubsub: segfault in function publish_expire
 
-2018-02-23 21:24 +0000 [bb973aeceb]  Michael Cargile <mikec@vicidial.com>
+	  The function pubsub_on_rx_publish_request incorrectly uses
+	  of AST_SCHED_REPLACE_UNREF.
 
-	* apps/app_amd.c: Fixed total time and silence calculations
+	  The AST_SCHED_REPLACE_UNREF should unref old '_data'.
 
-	  Between Asterisk 11 and Asterisk 13 there was a significant increase
-	  in the number of AST_FRAME_NULL frames being processed by app_amd.c's
-	  main loop. Each AST_FRAME_NULL frame was being counted as 100ms
-	  towards the total time and silence. This may have been accurate
-	  when app_amd.c was orginally added, but it is not in Asterisk 13.
-	  As such the total analysis time and silence calculations were way
-	  off effectively breaking app_amd.c
+	  Because of this, there may be a double unref
+	  of variable 'publication' when ast_sched_del is unsuccessful
+	  that leads to use after free of the 'publication' in publish_expire.
 
-	  * Additional debug messages were added
-	  * AST_FRAME_NULL are now ignored
+	  ASTERISK-27956 #close
 
-	  ASTERISK-27610
+	  Change-Id: Ie0f0cfc7e036953d890b188656010b325a5cdc82
 
-	  Change-Id: I18aca01af98f87c1e168e6ae0d85c136d1df5ea9
+2018-07-06 09:04 +0000 [c1e49720fa]  George Joseph <gjoseph@digium.com>
 
-2018-02-23 14:58 +0000 [68df9e609e]  George Joseph <gjoseph@digium.com>
+	* test.c:  Make output jUnit compatible
 
-	* ast_coredumper:  Minor fixes
+	  Separate "name" into "classname" and "name".
+	  Use '.' for classname separator instead of '/'.
+	  Prefix reserved words with '_'.
+	  Wrap output with a top-level "testsuites" element.
 
-	  * Fix --tarball-config so the option doesn't cause an error.
+	  Change-Id: Iec1a985eba1c478e5c1d65d5dfd95cb708442099
 
-	  * Allow for missing /etc/os-release.
+2018-07-06 07:57 +0000 [8f42447c68]  George Joseph <gjoseph@digium.com>
 
-	  * Add a sleep between tarballing the coredump and removing the
-	    output directory to allow the filesystem to settle.
+	* res_pjsip:  Add 'suppress_q850_reason_headers' option to endpoint
 
-	  Change-Id: I73e03b13087978bcc7f6bc9f45753990f82d9d77
+	  A new option 'suppress_q850_reason_headers' has been added to the
+	  endpoint object. Some devices can't accept multiple Reason headers and
+	  get confused when both 'SIP' and 'Q.850' Reason headers are received.
+	  This option allows the 'Q.850' Reason header to be suppressed.
+	  The default value is 'no'.
 
-2018-02-23 10:09 +0000 [caad0c09cd]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27949
+	  Reported-by: Ross Beer
 
-	* core: Fix handling of maximum length lines in config files.
+	  Change-Id: I54cf37a827d77de2079256bb3de7e90fa5e1deb1
 
-	  When a line is the maximum length "\n" is found at sizeof(buf) - 2 since
-	  the last character is actually the null terminator.  In addition if a
-	  line was exactly 8190 plus a multiple of 8192 characters long the config
-	  parser would skip the following line.
+2018-07-05 15:43 +0000 [c9f8e068ed]  Joshua Colp <jcolp@digium.com>
 
-	  Additionally fix comment in voicemail.conf sample config.  It previously
-	  stated that emailbody can only contain up to 512 characters which is
-	  always wrong.  The buffer is normally 8192 characters unless LOW_MEMORY
-	  is enabled then it is 512 characters.  The updated comment states that
-	  the line can be up to 8190 or 510 characters since the line feed and
-	  NULL terminator each use a character.
+	* res_pjsip_t38: Decline T.38 stream on failure case.
 
-	  ASTERISK-26688 #close
+	  When negotiating an incoming T.38 stream the code incorrectly
+	  returned failure instead of a decline for the stream when a
+	  problem occurred or the configuration didn't allow it. This
+	  resulted in SDP offers being rejected with a 488 response
+	  in all cases, even when another valid stream was present.
 
-	  Change-Id: I80864a0d40d2e2d8cd79d72af52a8f0a3a99c015
+	  This change makes it so the stream is now declined. If no
+	  streams are accepted a 488 response is sent while if at least
+	  one stream is accepted all the declined streams are, well,
+	  declined.
 
-2018-02-22 13:53 +0000 [3e5864bcc8]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27763
 
-	* res_pjsip_refer.c: Fix attended transfer race condition crash.
+	  Change-Id: I88bcf793788c412a9839d111a5c736bf6867807c
 
-	  The transferrer's session channel was destroyed by the transferrer's
-	  serializer thread in a race condition with the transfer target's
-	  serializer thread during an attended transfer.  The transfer target's
-	  serializer was attempting to clean up a deferred end status on behalf of
-	  the transferrer's channel when it should have passed the action to the
-	  transferrer's serializer.  When the transfer target's serializer lost the
-	  race then both threads wind up trying to end the transferrer's session.
+2018-07-02 18:43 +0000 [d5db664d70]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Push the ast_sip_session_end_if_deferred() call onto the transferrer's
-	  serializer to avoid a race condition that results in a crash.  The
-	  session_end() function that could be called by
-	  ast_sip_session_end_if_deferred() really must be executed by the
-	  transferrer's serializer to avoid this kind of crash.
+	* res_pjsip_t38.c: Be smarter about how we respond when T.38 is disabled.
 
-	  ASTERISK-27568
+	  We were blindly responding with AST_T38_REFUSED when ANY T.38 control
+	  frame came accross the bridge.  This causes T.38 Gateway to get confused
+	  and the T.38 session to get in a strange state.
 
-	  Change-Id: Iacda724e7cb24d7520e49b2fd7e504aa398d7238
+	  * Made the T.38 framehook only respond to request frames and ignore
+	  response frames.
 
-2018-02-17 03:28 +0000 [05fc37bbc2]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27657
+	  ASTERISK-27080
 
-	* install_prereq: Update FreeBSD libraries.
+	  Change-Id: I5fb5967c7d1efb30a7ff375f82887ca82a55b05b
 
-	  deleted
-	   autoconf gcc libsamplerate sqlite
+2018-07-03 12:10 +0000 [0aff1a278e]  Richard Mudgett <rmudgett@digium.com>
 
-	  changed
-	   binutils to libbfd
-	   freetds-devel to freetds
-	   gmime2 to gmime26
-	   mysql55-client to mysql57-client
+	* res_pjsip/pjsip_transport_management.c: Fix deadlock with transport keep alive.
 
-	  added
-	   alsa-lib bison bzip2 cclient corosync doxygen libedit flex graphviz
-	   libhoard libical libilbc libltdl lua neon newt net-snmp
-	   openldap-client openssl patch pkgconf portaudio postgresql10-client
-	   python radcli speexdsp subversion uriparser xmlstarlet libzip
+	  Using the keep_alive_interval option can result in a deadlock between the
+	  pjproject transport manager group lock and the monitored transports ao2
+	  container lock.  The pjproject transport manager group lock has to be
+	  superior in the locking order to the monitored transports ao2 container
+	  lock because of pjproject callbacks called when already holding the group
+	  lock.  The lock inversion happens when Asterisk attempts to send a keep
+	  alive packet over the reliable transports.
 
-	  ASTERISK-27686
+	  * Made keepalive_transport_thread() iterate over the monitored transports
+	  container rather than use the ao2_callback() method.  This avoids holding
+	  the container lock when sending the keep alive packet.
 
-	  Change-Id: Ibe88c9b26e59c30d26cdb313a3ef01c9f37ac80d
+	  ASTERISK-26686
 
-2018-02-22 10:46 +0000 [220accb461]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I5d5392a52e698bbe41a93f7d8e92bf0e61fe3951
 
-	* modules: Set deprecated modules to not build by default
+2018-07-02 18:44 +0000 [de5144e751]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I09090f70224866aead6b3207fa784cfefea7d539
+	* pjsip: Clarify certificate configuration for Websocket.
 
-2018-02-20 10:33 +0000 [4da0e19b89]  Joshua Colp <jcolp@digium.com>
+	  The Websocket transport uses the built-in HTTP server. As a result
+	  the TLS configuration is done in http.conf and not in pjsip.conf.
 
-	* chan_sip: Emit a second ringing event to ensure channel is found.
+	  This change adds a warning if this is configured in pjsip.conf and
+	  also clarifies in the sample configuration file.
 
-	  When constructing a dialog-info+xml NOTIFY message a ringing channel
-	  is found if the state is ringing and further information is placed into
-	  the message. Due to the migration to the Stasis message bus this did
-	  not always work as expected.
+	  Change-Id: I187d994d328c3ed274b6754fd4c2a4955bdc6dd9
 
-	  This change raises a second ringing event in such a way to guarantee
-	  that the event is received by chan_sip and another lookup is done to
-	  find the ringing channel.
+2018-06-23 04:50 +0000 [804d931f27]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-24488
+	* bridge_softmix_binaural: Enable FFTW3 in Solaris 11.
 
-	  Change-Id: I547a458fc59721c918cb48be060cbfc3c88bcf9c
+	  ASTERISK-27939
 
-2018-02-19 04:21 +0000 [9325de11f8]  Thomas Guebels <tgu@escaux.com>
+	  Change-Id: Ice5640e08385a64a0a6555deaccd91e86bca154f
 
-	* res_rtp_asterisk: Fix ICE candidate nomination
+2018-06-29 18:28 +0000 [1aa45ffdfa]  Richard Mudgett <rmudgett@digium.com>
 
-	  If the ICE role is not set right away, we might have a role conflict
-	  that stays undetected and ICE finishing with successful tests and no
-	  candidate nominated. This was introduced by ASTERISK-27088.
+	* res_pjsip_t38.c: Fix crash by ignoring 1xx messages.
 
-	  To avoid this, we set the role as soon as before but only if the ICE
-	  state permits it: still checking and not yet nominating candidates or
-	  completed.
+	  If we initiated a T.38 reINVITE, we would crash if we received any other
+	  1xx response message except 100 if it were followed by a 200 response.
 
-	  ASTERISK-27646
+	  * Made ignore any 1xx response so we do not close out the T.38 negotiation
+	  too early.  For good measure we'll now accept any 2xx response as
+	  acceptance of the reINVITE T.38 offer.
 
-	  Change-Id: I5dbc69ad63cacbb067922850fbb113d479bd729c
+	  ASTERISK-27944
 
-2017-12-04 05:27 +0000 [64942276d1]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I0ca88aae708d091db7335af73f41035a212adff4
 
-	* chan_sip: Peers with distinct source ports don't match, regardless of transport.
+2018-07-01 13:54 +0000 [f30ebd3823]  Joshua Colp <jcolp@digium.com>
 
-	  Previously, peers connected via TCP (or TLS) were matched by ignoring their
-	  source port. One cannot say anything when protocol:IP:port match, yes (see
-	  <http://stackoverflow.com/q/3329641>). However, when the ports do not match, the
-	  peers do not match as well.
+	* res_pjsip_pubsub: Hold module reference for publications.
 
-	  This change allows two peers connected to an Asterisk server via TCP (or TLS)
-	  behind a NAT (= same source IP address) to be differentiated via their port as
-	  well.
+	  Incoming publications need to ensure that the module remains
+	  loaded for the lifetime of them. This is now done by holding
+	  a reference to the module while the publication exists. This
+	  mirrors that of inbound subscriptions.
 
-	  ASTERISK-27457
-	  Reported by: Stephane Chazelas
+	  ASTERISK-27783
 
-	  Change-Id: Id190428bf1d931f2dbfd4b293f53ff8f20d98efa
+	  Change-Id: Ia98c95a15e11af25728d5fb3e56e12cda0cfc7c0
 
-2018-02-22 23:15 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2018-05-21 07:24 +0000 [9d3f3a4b0a]  Robert Mordec <r.mordec@slican.pl>
 
-	* asterisk 13.20.0-rc1 Released.
+	* app_confbridge: Bridge and announcers not removed if conference ends quickly
 
-2018-02-22 11:11 +0000 [ffb15b2bc7]  Kevin Harwell <kharwell@digium.com>
+	  If a conference is ended very quickly after it was created (i.e., the
+	  first user immediately hangs up) then the conference bridge and announcer
+	  channels are not removed.
 
-	* AMI: Bumping AMI non-breaking number for Asterisk 13.20.0 release
+	  When a conference is created, the push_announcer() function is added to
+	  the playback queue task processor and the conference object reference is
+	  bumped.  If a conference is ended while the push_announcer() function is
+	  still going then the ao2_cleanup(conference) at the end of
+	  push_announcer() will call the destructor function -
+	  destroy_conference_bridge().
 
-	  A few changes were made to AMI:
+	  The destroy_conference_bridge() function will then add the
+	  hangup_playback() task to the playback queue and will wait for it to end.
+	  Since it is already a current task of the playback queue it will wait
+	  forever.
 
-	   * Fixed "(null):" header in AMI AsyncAGIEnd event
-	   * A mute header was added to the ConfbridgeJoin AMI event
-	   * ConfbridgeList action's ConfbridgeList events now output all
-	     the standard channel snapshot headers
+	  This patch makes the conference thread call push_announcer() directly.
+	  This way the conference object reference bump is not needed.  Since the
+	  playback queue task processor is only used by the conference thread
+	  itself, there is no danger of trying to play announcements before the
+	  announcer is pushed to the bridge.
 
-	  Change-Id: I94a82a44b02c91becae08d254e9a56abba5697cf
+	  ASTERISK-27870 #close
 
-2018-01-31 13:37 +0000 [05af43b240]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: I947a50fb121422d90fd1816d643a54d75185a477
 
-	* AST-2018-003: Crash with an invalid SDP fmtp attribute
+2018-06-21 00:28 +0000 [db02218db2]  Matthew Fredrickson <creslin@digium.com>
 
-	  pjproject's fmtp retrieval function failed to catch invalid fmtp attributes.
-	  Because of this Asterisk would crash if given an SDP with an invalid fmtp
-	  attribute.
+	* main/cdr.c: Alleviate CDR deadlock
 
-	  When retrieving the format this patch now makes sure the fmtp attribute is
-	  available. If not available it now returns an error status.
+	  There is a rare case (do to the infrequent timing involved) where
+	  CDR submission threads in batch mode can deadlock with a currently
+	  running CDR batch process.  This patch should remove the need for
+	  holding the lock in the scheduler and should clean a few code
+	  paths up that inconsistently submitted new work to the CDR batch
+	  processor.
 
-	  ASTERISK-27583 #close
+	  ASTERISK-27909
 
-	  Change-Id: I5cebe000ce2d846cae3af33b6d72c416e51caf2f
+	  Change-Id: I6333e865db7c593c102c2fd948cecdb96481974d
+	  Reported-by: Denis Lebedev
 
-2018-01-31 13:33 +0000 [c5dc2fb4ab]  Kevin Harwell <kharwell@digium.com>
+2018-06-25 22:08 +0000 [4b9bf4f5e0]  Kirsty Tyerman <kirsty.tyerman@boeing.com>
 
-	* AST-2018-002: Crash with an invalid SDP media format description
+	* pbx_dundi: reordered unloading of module pbx_dundi
 
-	  pjproject's media format parsing algorithm failed to catch invalid values.
-	  Because of this Asterisk would crash if given an SDP with a invalid media
-	  format description.
+	  Destroy scheduler after peers are pruned to stop dundi crashing when
+	  unloading module.
 
-	  When parsing the media format description this patch now properly parses the
-	  value and returns an error status if it can't successfully parse/convert the
-	  value.
+	  ASTERISK-26987
+	  Reported-by: Kirsty Tyerman
 
-	  ASTERISK-27582 #close
+	  Change-Id: Ic12e562cd90d8d813a9e97f302045091f59e3c05
 
-	  Change-Id: I883b3a4ef85b6972397f7b56bf46c5779c55fdd6
+2018-06-28 12:07 +0000 [7a238fe74d]  Richard Mudgett <rmudgett@digium.com>
 
-2018-02-06 12:07 +0000 [5947fd148b]  George Joseph <gjoseph@digium.com>
+	* AMI SendText action: Fix to use correct thread to send the text.
 
-	* AST-2018-005: res_pjsip_transport_management:  Move to core
+	  The AMI action was directly sending the text to the channel driver.
+	  However, this makes two threads attempt to handle media and runs afowl of
+	  CHECK_BLOCKING.
 
-	  Since res_pjsip_transport_management provides several attack
-	  mitigation features, its functionality moved to res_pjsip and
-	  this module has been removed.  This way the features will always
-	  be available if res_pjsip is loaded.
+	  * Queue a read action to make the channel's media handling thread actually
+	  send the text message.  This changes the AMI actions success/fail response
+	  to just mean the text was queued to be sent not that the text actually got
+	  sent.  The channel driver may not even support sending text messages.
 
-	  ASTERISK-27618
-	  Reported By: Sandro Gauci
+	  ASTERISK-27943
 
-	  Change-Id: I21a2d33d9dda001452ea040d350d7a075f9acf0d
+	  Change-Id: I9dce343d8fa634ba5a416a1326d8a6340f98c379
 
-2018-02-06 11:28 +0000 [a780386dbb]  George Joseph <gjoseph@digium.com>
+2018-06-25 07:37 +0000 [e3585353f6]  George Joseph <gjoseph@digium.com>
 
-	* AST-2018-005: Fix tdata leaks when calling pjsip_endpt_send_response(2)
+	* res_pjsip_messaging:  Allow application/* for in-dialog MESSAGEs
 
-	  pjsip_distributor:
-	     authenticate() creates a tdata and uses it to send a challenge or
-	     failure response.  When pjsip_endpt_send_response2() succeeds, it
-	     automatically decrements the tdata ref count but when it fails, it
-	     doesn't.  Since we weren't checking for a return status, we weren't
-	     decrementing the count ourselves on error and were therefore leaking
-	     tdatas.
+	  In addition to text/* content types, incoming_in_dialog_request now
+	  accepts application/* content types.
 
-	  res_pjsip_session:
-	     session_reinvite_on_rx_request wasn't decrementing the ref count
-	     if an error happened while sending a 491 response.
-	     pre_session_setup wasn't decrementing the ref count if
-	     while sending an error after a pjsip_inv_verify_request failure.
+	  Also fixed a length issue when copying the body text.  It was one
+	  character short.
 
-	  res_pjsip:
-	     ast_sip_send_response wasn't decrementing the ref count on error.
+	  ASTERISK-27942
 
-	  ASTERISK-27618
-	  Reported By: Sandro Gauci
+	  Change-Id: I4e54d8cc6158dc47eb8fdd6ba0108c6fd53f2818
 
-	  Change-Id: Iab33a6c7b6fba96148ed465b690ba8534ac961bf
+2018-06-25 15:42 +0000 [5f12e2bd07]  George Joseph <gjoseph@digium.com>
 
-2018-02-07 08:09 +0000 [de5df002a0]  Joshua Colp <jcolp@digium.com>
+	* app_confbridge:  Move participant info code to confbridge_manager.
 
-	* AST-2018-004: Restrict the number of Accept headers in a SUBSCRIBE.
+	  With the participant info code in app_confbridge, we were still
+	  in the process of adding the channel to the bridge when trying to send
+	  an in-dialog MESSAGE.  This caused 2 threads to grab the channel
+	  blocking flag at the same time.  To mitigate this, the participant
+	  info code was moved to confbridge_manager so it runs after all
+	  channel/bridge actions have finished.
 
-	  When receiving a SUBSCRIBE request the Accept headers from it are
-	  stored locally. This operation has a fixed limit of 32 Accept headers
-	  but this limit was not enforced. As a result it was possible for
-	  memory outside of the allocated space to get written to resulting
-	  in a crash.
+	  Change-Id: I228806ac153074f45e0b35d5236166e92e132abd
 
-	  This change enforces the limit so only 32 Accept headers are
-	  processed.
+2018-06-18 21:22 +0000 [880fbff6b7]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27640
-	  Reported By: Sandro Gauci
+	* res_pjsip_session:  Add ability to accept multiple sdp answers
 
-	  Change-Id: I99a814b10b554b13a6021ccf41111e5bc95e7301
+	  pjproject by default currently will follow media forked during an INVITE
+	  on outbound calls if the To tag is different on a subsequent response as
+	  that on an earlier response.  We handle this correctly.  There have
+	  been reported cases where the To tag is the same but we still need to
+	  follow the media.  The pjproject patch in this commit adds the
+	  capability to sip_inv and also adds the capability to control it at
+	  runtime.  The original "different tag" behavior was always controllable
+	  at runtime but we never did anything with it and left it to default to
+	  TRUE.
 
-2018-02-20 04:31 +0000 [cacb72f5ae]  Corey Farrell <git@cfware.com>
+	  So, along with the pjproject patch, this commit adds options to both the
+	  system and endpoint objects to control the two behaviors, and a small
+	  logic change to session_inv_on_media_update in res_pjsip_session to
+	  control the behavior at the endpoint level.
 
-	* doc/lang/language-criteria.txt: Link to wiki.
+	  The default behavior for "different tags" remains the same at TRUE and
+	  the default for "same tag" is FALSE.
 
-	  This document is out of date and is superseded by content on the
-	  Asterisk wiki.
+	  Change-Id: I64d071942b79adb2f0a4e13137389b19404fe3d6
+	  ASTERISK-27936
+	  Reported-by: Ross Beer
 
-	  ASTERISK-24386 #close
+2018-06-21 11:45 +0000 [675e2ddb49]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Idbf95b27b096c205251e1bbb560c79224ba81822
+	* uuid: Enable UUID in Solaris 11.
 
-2018-02-18 10:27 +0000 [5ebc94da02]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27933
+	  Reported by: bautsche
 
-	* res_http_websocket: Don't leak memory on read failure
+	  Change-Id: I9b8362824efbfb2a16981e46e85f7c8322908c49
 
-	  Change-Id: Ic449ea832bc81a1671c0e910c5fbe8c683e3da89
+2018-06-13 02:25 +0000 [184b375b41]  Kristian F. Høgh <kfh@uni-tel.dk>
 
-2018-02-19 03:57 +0000 [e532b982b3]  Corey Farrell <git@cfware.com>
+	* app_queue: Add option for predial handlers on caller and callee channels
 
-	* core: Rename sounds_index.c to sounds.c.
+	  Add predial handler support to app_queue.  app_dial (ASTERISK_19548) and
+	  app_originate (ASTERISK_26587) have the ability to execute predial
+	  handlers on caller and callee channels.  This patch adds predial handlers
+	  to app_queue and uses the same options as Dial and Originate (b and B).
+	  The caller routine gets executed when the caller first enters the queue.
+	  The callee routine gets executed for each queue member when they are about
+	  to be called.
 
-	  This will make the source filename match the 'module reload sounds'
-	  command.  This will allow conversion to a built-in module in Asterisk 16
-	  without needing to redefine AST_MODULE.
+	  ASTERISK-27912
 
-	  Change-Id: Ifb8e489575b27eb33d8c0b6a531f266670557f6e
+	  Change-Id: I5acf5c32587ee008658d12e8a8049eb8fa4d0f24
 
-2018-02-19 02:49 +0000 [9c8763e4d2]  Corey Farrell <git@cfware.com>
+2018-06-21 16:39 +0000 [cad50d6dbf]  Richard Mudgett <rmudgett@digium.com>
 
-	* config: Fix locking for extconfig reload.
+	* VECTOR: Passing parameters with side effects to macros is dangerous.
 
-	  Expand locking to include full reload process for extconfig to ensure
-	  nothing can read the config mappings between clearing and reloading.
+	  * Fix several instances where we were bumping a ref in the parameter and
+	  then unrefing the object if it failed.  The way the AST_VECTOR_APPEND()
+	  and AST_VECTOR_REPLACE() macros are implemented means if it fails the new
+	  value was never evaluated.
 
-	  Change-Id: I378316bad04f1b599ea82d0fef62b8978a644b92
+	  Change-Id: I2847872a455b11ea7e5b7ce697c0a455a1d0ac9a
 
-2018-02-15 14:09 +0000 [1a7ad4c9cf]  Sean Bright <sean.bright@gmail.com>
+2018-06-20 16:57 +0000 [aaaa6f4a4b]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_header_funcs: Various cleanups
+	* bridge_softmix.c: Fix memory leak.
 
-	   * Prefer strcasecmp() over stricmp()
-	   * Use a list with no lock since we never actually lock
-	   * Minor cleanups to error messages
+	  Made release the media_types vector in
+	  softmix_bridge_stream_topology_changed().
 
-	  Change-Id: I8446f44795ee8f3072e1c1f9193c6912dfc0c42b
+	  Change-Id: Ide3f47e379b614879220dfeeb13843f9f2b177b5
 
-2018-02-17 08:49 +0000 [6eb22964ba]  Alexander Traud <pabstraud@compuserve.com>
+2018-06-21 11:22 +0000 [bfeded7e62]  Alexander Traud <pabstraud@compuserve.com>
 
-	* rtp_engine: Load format name / mime type in uppercase again.
+	* smsq: Remove an left-over special case for Solaris.
 
-	  This reverts a previous change partly.
+	  Actually, this case was never needed because the check below does the same.
 
-	  ASTERISK-27689
+	  Change-Id: Ia2fca4ba6c58c644a8b7cb2d9db8539728c14ffb
 
-	  Change-Id: Ia3d2f282db6995be8c1c253b5d52f6038761e8af
+2018-06-21 11:17 +0000 [bbea9cfc3b]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-02-16 17:58 +0000 [27d28116d9]  Corey Farrell <git@cfware.com>
+	* res_http_post: Enable GMime in Solaris 11.
 
-	* BuildSystem: Use single bootstrap.sh for Asterisk and menuselect.
+	  Change-Id: Ie434541f18f894c751d2e44bcb3efb3cac626019
 
-	  This causes the root bootstrap.sh script to generate configure scripts
-	  for both Asterisk and menuselect.  This ensures that both configure
-	  scripts are generated with the same version of autotools and avoids
-	  situations where shared autoconf macros get modified without
-	  regenerating the menuselect script.
+2018-06-21 05:08 +0000 [7f3882c8e9]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I2bfd8537bbb63b3d46b11efabbb15eaaf9ef731a
+	* codecs/ilbc: Compile in Solaris 11.
 
-2018-02-16 13:33 +0000 [9ad9433257]  Sean Bright <sean.bright@gmail.com>
+	  The symbol FS is the sampling frequency. That symbol is not used in Asterisk at
+	  all and was a copy-and-paste of the iLBC reference code from the IETF RFC.
+	  However, in Solaris, that symbol is defined by another header already. To
+	  compile in Solaris, that symbol has to go.
 
-	* res_pjsip: Endpoint destruction does not free DTLS configuration
+	  Change-Id: I91ddbe5be7c00069c3a25abd5f58d7b2f04c51b1
 
-	  ASTERISK-27679 #close
-	  Reported by: Mak Dee
+2018-06-21 05:07 +0000 [9704c424f5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I89a2783a11be0763bf123d1619ed176b6225cf42
+	* chan_oss: Compile in Solaris 11.
 
-2018-02-16 12:44 +0000 [c5e34f0794]  Alexander Traud <pabstraud@compuserve.com>
+	  M_READ existed already and was conflicting in name.
 
-	* install_prereq: Update OpenBSD libraries.
+	  Change-Id: I02108e07ae7d2dc314fe1e6c706c17731095a3e4
 
-	  deleted
-	   jack sqlite
+2018-06-21 05:04 +0000 [6f47b84fbd]  Alexander Traud <pabstraud@compuserve.com>
 
-	  renamed
-	   freetds-0.63p1-msdblib to freetds
-	   mysql-client to mariadb-client
+	* func_env: Compile in Solaris 11.
 
-	  added
-	   bison bzip2 c-client doxygen e2fsprogs graphviz gsm libical jansson libltdl
-	   lua neon net-snmp libsrtp portaudio-svn postgresql-client python speexdsp
-	   subversion uriparser xmlstarlet
+	  Change-Id: Idc9b36720f3d29c90a35a6a1ae79a7f9e1aaf50e
 
-	  ASTERISK-27684
+2018-06-21 05:01 +0000 [a5c53bd323]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I26bdcb0a1d0e484a8dad1052da97f194aefd3370
+	* utils: Avoid an unused variable in Solaris 11.
 
-2018-02-16 12:30 +0000 [88ec9be78a]  Alexander Traud <pabstraud@compuserve.com>
+	  With ./configure --enable-dev-mode[=noisy], the build fails because every
+	  warning gets an error. Therefore, Asterisk has to be free of warnings and this
+	  variable must go.
 
-	* BuildSystem: Allow newer autotools on OpenBSD.
+	  Change-Id: I63dd2bc4833b9bdb04602f83422d16caf289d46a
 
-	  ASTERISK-27683
+2018-06-21 04:59 +0000 [92109cf496]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I5ec9dafbb0c16b6f2740c641980bc2eaaf995624
+	* BuildSystem: Enable ./configure in Solaris 11.
 
-2018-02-16 07:52 +0000 [0ba6337d32]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27931
 
-	* BuildSystem: Fix a typo related to ./configure --prefix=<path> on OpenBSD.
+	  Change-Id: If298ce7f03be227a3687b9c20d382c9c55a72404
 
-	  Reported by: Stuart Henderson
+2018-06-20 13:24 +0000 [d6721e1e4c]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ieae8624f48b6ae78cf29930b9a45a3c842c7a764
+	* BuildSystem: Enable autotools in Solaris 11.
 
-2018-02-16 06:41 +0000 [58aa0d1014]  Alexander Traud <pabstraud@compuserve.com>
+	  Because this was the last operating system which required a special case, a
+	  version appended to the autotools, the whole version stuff is removed by this
+	  change. This simplifies the script ./bootstrap.sh. Hopefully, this gives even
+	  broader platform compatibility.
 
-	* BuildSystem: Enable IMAP storage on OpenBSD.
+	  ASTERISK-27929
+	  ASTERISK-27926
 
-	  ASTERISK-27681
-	  Reported by: Stuart Henderson
+	  Change-Id: Id4cf433a1a7fa861d0210e1a2e16ca592b49fd5a
 
-	  Change-Id: Ifb6b614acb251b695b9417d76510e73eb335b679
+2018-06-13 11:33 +0000 [eb8bbe660e]  Richard Mudgett <rmudgett@digium.com>
 
-2018-02-16 05:58 +0000 [139dd38193]  Alexander Traud <pabstraud@compuserve.com>
+	* channel.c: Make CHECK_BLOCKING() save thread LWP id for messages.
 
-	* res_calendar: Specialized calendars depend on symbols of general calendar.
+	  * Removed an unnecessary call to ast_channel_blocker_set() in
+	  __ast_read().
 
-	  ASTERISK-27680
+	  ASTERISK-27625
 
-	  Change-Id: Ifb77912e424fe3710a025c18526fada673ec0b79
+	  Change-Id: I342168b999984666fb869cd519fe779583a73834
 
-2018-02-16 05:02 +0000 [94ff290361]  Alexander Traud <pabstraud@compuserve.com>
+2018-06-13 16:41 +0000 [da54605b8a]  Richard Mudgett <rmudgett@digium.com>
 
-	* BuildSystem: Enable system provided libedit on OpenBSD.
+	* ARI POST DTMF: Make not compete with channel's media thread.
 
-	  ASTERISK-27677
+	  There can be one and only one thread handling a channel's media at a time.
+	  Otherwise, we don't know which thread is going to handle the media frames.
 
-	  Change-Id: I0854e3616d1361ae9b6907d3d3444a02784ac62b
+	  ASTERISK-27625
 
-2018-02-15 14:30 +0000 [48acd5cd26]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I4d6a2fe7386ea447ee199003bf8ad681cb30454e
 
-	* bridge_roles: Use a non-locking linked list where appropriate
+2018-06-13 13:05 +0000 [7d874c1af7]  Richard Mudgett <rmudgett@digium.com>
 
-	  Also explicitly initialize with the AST_LIST_HEAD_NOLOCK_INIT macro for
-	  clarity.
+	* AMI PlayDTMF Action: Make not compete with channel's media thread.
 
-	  Change-Id: I4bc39ec33bc3ff77e1a971a01ace87deb965be3f
+	  There can be one and only one thread handling a channel's media at a time.
+	  Otherwise, we don't know which thread is going to handle the media frames.
 
-2018-02-15 13:29 +0000 [54efc0c637]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27625
 
-	* res_pjsip: Use pjsip_sip_uri.user_param instead of other_param
+	  Change-Id: Ia341f1a6f4d54f2022261abec9021fe5b2eb4905
 
-	  There is a dedicated slot in the pjsip_sip_uri for the 'user'
-	  parameter, so use that instead of adding to the list of generic URI
-	  parameters.
+2018-06-12 14:09 +0000 [080508d2eb]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I0a0ce8a60ecee27489735bf56fd707719d8c2ed6
+	* channel.c: Fix usage of CHECK_BLOCKING()
 
-2018-02-12 07:48 +0000 [4a5221cd43]  Alexander Traud <pabstraud@compuserve.com>
+	  The CHECK_BLOCKING() macro is used to indicate if a channel's handling
+	  thread is about to do a blocking operation (poll, read, or write) of
+	  media.  A few operations such as ast_queue_frame(), soft hangup, and
+	  masquerades use the indication to wake up the blocked thread to reevaluate
+	  what is going on.
 
-	* BuildSystem: Remove chan_h323 leftovers.
+	  ASTERISK-27625
 
-	  ASTERISK-27670
+	  Change-Id: I4dfc33e01e60627d962efa29d0a4244cf151a84d
 
-	  Change-Id: I07a8ef8bbd6001e25711fa1bff152eb6c9efa729
+2018-06-18 18:04 +0000 [0989b63047]  Richard Mudgett <rmudgett@digium.com>
 
-2018-01-17 08:17 +0000 [eabe72a679]  Alexander Traud <pabstraud@compuserve.com>
+	* autoservice: Don't start channel autoservice if the thread is a user interface.
 
-	* BuildSystem: Invoke ldconfig with previous path.
+	  Executing dialplan functions from either AMI or ARI by getting a variable
+	  could place the channel into autoservice.  However, these user interface
+	  threads do not handle the channel's media so we wind up with two threads
+	  attempting to handle the media.
 
-	  On OpenBSD, gmake uninstall{-all} registered only libraries from /usr/lib and
-	  lost those from /usr/local/lib. Instead, invoke ldconfig on a path.
+	  There can be one and only one thread handling a channel's media at a time.
+	  Otherwise, we don't know which thread is going to handle the media frames.
 
-	  ASTERISK-27595
+	  ASTERISK-27625
 
-	  Change-Id: I4aa2c0b5e07119d1a556f8ff6349eaf09e986888
+	  Change-Id: If2dc94ce15ddabf923ed1e2a65ea0ef56e013e49
 
-2018-01-28 03:02 +0000 [c2283fca21]  Alexander Traud <pabstraud@compuserve.com>
+2018-06-18 16:07 +0000 [91c3ac19cb]  Richard Mudgett <rmudgett@digium.com>
 
-	* BuildSystem: Do not warn when bash is not installed.
+	* Dialplan functions: Fix some channel autoservice misuse.
 
-	  ASTERISK-27631
+	  * Fix off nominal paths leaving the channel in autoservice.
+	  * Remove unnecessary start/stop channel autoservice.
+	  * Fix channel locking around a channel datastore search.
 
-	  Change-Id: Iefdf268b0b98c3e7d8089ba87cf78136ac1d785b
+	  Change-Id: I7ff2e42388064fe3149034ecae57604040b8b540
 
-2017-11-16 12:57 +0000 [4e5d5b2ce2]  Corey Farrell <git@cfware.com>
+2018-06-19 10:43 +0000 [720c2d1da2]  Richard Mudgett <rmudgett@digium.com>
 
-	* main/asterisk.c: Remove silly usage of RAII_VAR.
+	* Fix some doxygen and curly placement.
 
-	  Change-Id: I7e2996397fbd3c3a6a69dd805c38448ddfc34ae9
+	  Change-Id: I9a784a7c804120a8fa826c2a4cb9957e4b0b2fc8
 
-2018-02-11 15:27 +0000 [ba63dad12d]  Richard Mudgett <rmudgett@digium.com>
+2018-06-18 13:17 +0000 [c1686b8b3e]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_sip.c: Fix crash processing CANCEL.
+	* tcptls.h: Remove redundant SSL_CTX typedef.
 
-	  Check if initreq data string exists before using it when processing a
-	  CANCEL request.
+	  It is invalid to typedef something more than once.  Though not all gcc
+	  compilers on different OS's complain about it.
 
-	  ASTERISK-27666
+	  Change-Id: I5a7d4565990c985822d61ce75bde0b45f9870540
 
-	  Change-Id: Id1d0f0fa4ec94e81b332b2973d93e5a14bb4cc97
+2018-06-12 15:13 +0000 [a470bb9e27]  Richard Mudgett <rmudgett@digium.com>
 
-2018-01-30 20:31 +0000 [6c72c22d51]  Corey Farrell <git@cfware.com>
+	* channel: Fix some more unprotected channel flag setting.
 
-	* json: Add conditionals to avoid locking if Jansson is thread safe.
+	  Change-Id: I34c3b1201b1de539945bcfdcb264fff30332d48c
 
-	  Jansson is thread safe for all read-only functions and reference
-	  counting starting v2.11.  This allows simplification of our code and
-	  removal of locking around reference counting and dumping.
+2018-06-15 15:21 +0000 [8732d62334]  Matthew Fredrickson <creslin@digium.com>
 
-	  Change-Id: Id985cb3ffa6681f9ac765642e20fcd187bd4aeee
+	* menuselect/menuselect_curses: Resolves sprintf usage error
 
-2018-02-12 06:16 +0000 [0ab97e9774]  Alexander Traud <pabstraud@compuserve.com>
+	  Acccording to the man page for sprintf, using the same buffer for
+	  output as one used as an input yields undefined behavior.
+	  This patch should work around this problem.
 
-	* pjproject_bundled: Disable G.729 from Belledonne Communications.
+	  ASTERISK-27903
+	  Reported-by: Alexander Traud
 
-	  When <http://github.com/BelledonneCommunications/bcg729> is installed, PJProject
-	  tries to link that. Support for this bcg729 was added with PJProject 2.7. The
-	  issue happens, because Teluu enabled that new feature on default.
+	  Change-Id: I2213dcb454aff26457e2e4cc9c6821276463ae3a
 
-	  ASTERISK-27584
-	  Reported by: Stuart Henderson
+2018-06-12 09:30 +0000 [4c7ab73468]  Sam Wierema <sam@messagebird.com>
 
-	  Change-Id: I88b6b18ad777bcfe2d8201187b4b90eec0a172a6
+	* app_mp3: remove 10 seconds of silence after mp3 playback
 
-2018-02-12 05:38 +0000 [81bec7c359]  Alexander Traud <pabstraud@compuserve.com>
+	  This patch changes the way asterisk polls output from mpg123, instead
+	  of waiting for 10 seconds(when playing an http url) it now uses a
+	  timeout of one second and iterates 10 times using this same timeout.
 
-	* codecs: Add support for WebRTC iLBC 2.0.
+	  The main difference is that for every timeout asterisk receives it now
+	  checks if mpg123 is still running before poll again.
 
-	  When the latest version of that library was installed, Asterisk did not build.
+	  ASTERISK-27752
 
-	  ASTERISK-27669
-	  Reported by: Николай Михо
+	  Change-Id: Ib7df8462e3e380cb328011890ad9270d9e9b4620
 
-	  Change-Id: I27e09bb875fdd56423bd9fae1be85fddb428eb96
+2018-06-13 04:40 +0000 [9d7958672b]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-30 09:58 +0000 [b1c6a644d3]  Alexander Traud <pabstraud@compuserve.com>
+	* tests/test_utils: Repair ./configure --with-ssl=PATH.
 
-	* backtrace: Avoid potential spurious output.
+	  ASTERISK-27914
 
-	  clang 4.0 found this via -Wlogical-not-parentheses.
+	  Change-Id: Ibcab8f556ee77776f203cff8b06d776a673b7bc4
 
-	  ASTERISK-27642
+2018-06-04 20:31 +0000 [e1908ea484]  Kirsty Tyerman <kirsty.tyerman@boeing.com>
 
-	  Change-Id: I9ec3e144d425a976c02811bd23cd0c533d2eca4e
+	* chan_iax2: better handling for timeout and EINTR
 
-2018-02-10 05:39 +0000 [21bd7421ef]  Alexander Traud <pabstraud@compuserve.com>
+	  The iax2 module is not handling timeout and EINTR case properly. Mainly when
+	  there is an interupt to the kernel thread. In case of ast_io_wait recieves a
+	  signal, or timeout it can be an error or return 0 which eventually escapes the
+	  thread loop, so that it cant recieve any data. This then causes the modules
+	  receive queue to build up on the kernel and stop any communications via iax in
+	  asterisk.
 
-	* install_prereq: Update Debian/Ubuntu libraries.
+	  The proposed patch is for the iax module, so that timeout and EINTR does not
+	  exit the thread.
 
-	  ASTERISK-27555
+	  ASTERISK-27705
+	  Reported-by: Kirsty Tyerman
 
-	  Change-Id: Idc36e91db30c0163c560d04c5a82bca5d6ce92a8
+	  Change-Id: Ib4c32562f69335869adc1783608e940c3535fbfb
 
-2018-02-09 12:06 +0000 [dcfcf310a1]  Richard Mudgett <rmudgett@digium.com>
+2018-05-31 16:22 +0000 [e7a7506f9c]  George Joseph <gjoseph@digium.com>
 
-	* cdr.c: Fix runtime leak of CDR records.
+	* app_confbridge:  Enable sending events to participants
 
-	  Need to remove all CDR's listed by a CDR object from the active_cdrs_all
-	  container including the root/master record.
+	  ConfBridge can now send events to participants via in-dialog MESSAGEs.
+	  All current Confbridge events are supported, such as ConfbridgeJoin,
+	  ConfbridgeLeave, etc.  In addition to those events, a new event
+	  ConfbridgeWelcome has been added that will send a list of all
+	  current participants to a new participant.
 
-	  ASTERISK-27656
+	  For all but the ConfbridgeWelcome event, the JSON message contains
+	  information about the bridge, such as its id and name, and information
+	  about the channel that triggered the event such as channel name,
+	  callerid info, mute status, and the MSID labels for their audio and
+	  video tracks. You can use the labels to correlate callerid and mute
+	  status to specific video elements in a webrtc client.
 
-	  Change-Id: I48b4970663fea98baa262593d2204ef304aaf80e
+	  To control this behavior, the following options have been added to
+	  confbridge.conf:
 
-2017-12-19 02:52 +0000 [0fc3e831a7]  Oron Peled <oron.peled@xorcom.com>
+	  bridge_profile/enable_events:  This must be enabled on any bridge where
+	  events are desired.
 
-	* chan_console: don't read and write at the same time
+	  user_profile/send_events:  This must be set for a user profile to send
+	  events.  Different user profiles connected to the same bridge can have
+	  different settings.  This allows admins to get events but not normal
+	  users for instance.
 
-	  It seems that the ALSA backend of PortAudio doesn't know how to both
-	  read and write at the same time by adding a per-device mutex.
+	  user_profile/echo_events:  In some cases, you might not want the user
+	  triggering the event to get the event sent back to them.  To prevent it,
+	  set this to false.
 
-	  FIXME: currently only a draft version. Need to either auto-detect
-	  we work with the ALSA backend or add an extra configuration option
-	  to use this mutex.
+	  A change was also made to res_pjsip_sdp_rtp to save the generated msid
+	  to the stream so it can be re-used.  This allows participant A's video
+	  stream to appear as the same label to all other participants.
 
-	  ASTERISK-27426 #close
+	  Change-Id: I26420aa9f101f0b2387dc9e2fd10733197f1318e
 
-	  Change-Id: I635eacee45f5413faa18f5a3b606af03b926dacb
+2018-06-13 05:06 +0000 [b01fc2ef3d]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-02-02 17:20 +0000 [da987df122]  Richard Mudgett <rmudgett@digium.com>
+	* res_rtp_asterisk: Instead of ./configure use OPENSSL_NO_SRTP.
 
-	* res_pjsip/config_domain_aliases.c: Add check for missing domain.
+	  Previously, Asterisk used its script ./configure, to test whether OpenSSL was
+	  built with no-srtp (or was simply too old). However, the header file
+	  <openssl/opensslconf.h> is the preferred way to detect the local configuration
+	  of OpenSSL.
 
-	  What is the point of defining an alias and not saying what is being
-	  aliased?
+	  As a positive side-effect the script ./configure does not interleave the
+	  detection of the Open Settlement Protocol Toolkit (OSPTK) with the detection of
+	  individual features of OpenSSL anymore.
 
-	  Change-Id: I98a892016ed61dcf5efeb6619fd748925103f0be
+	  Change-Id: I3c77c7b00b2ffa2e935632097fa057b9fdf480c0
 
-2018-01-31 17:48 +0000 [1ff580bb25]  Richard Mudgett <rmudgett@digium.com>
+2018-06-05 04:36 +0000 [41175caee0]  Joshua Colp <jcolp@digium.com>
 
-	* app_confbridge: ConfbridgeList event has standard channel shapshot headers.
+	* rtp: Don't negotiate dynamic codecs using payload.
 
-	  * Made the AMI ConfbridgeList action's ConfbridgeList events output all
-	  the standard channel snapshot headers instead of a few hand-coded channel
-	  snapshot headers.  The benefit is that the CallerIDName gets disruptive
-	  characters like CR, LF, Tab, and a few others escaped.  However, an empty
-	  CallerIDName is now output as "<unknown>" instead of "<no name>".
+	  In Asterisk there are some dynamic codecs that have
+	  a fixed payload number. This number was being improperly
+	  used to negotiate the codec, instead of using the name
+	  and sample rate. This could result in the wrong payload
+	  number being negotiated for a codec.
 
-	  ASTERISK-27651
+	  This change makes it so that only static payloads
+	  will be negotiated using their payload number.
 
-	  Change-Id: Iaf7d54a9d40194c2db060bc9b4979fab6720d977
+	  ASTERISK-27848
 
-2018-01-31 15:45 +0000 [0cf7a9e0ca]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ia865830170fd3f808cdb33104f3d4c4ffdc77570
 
-	* app_confbridge: Add the Muted header to ConfbridgeJoin AMI event.
+2018-04-16 14:13 +0000 [b649682caa]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27651
+	* AST-2018-007: iostreams potential DoS when client connection closed prematurely
 
-	  Change-Id: Idef2ca54d242d1b894efd3fc7b360bc6fd5bdc34
+	  Before Asterisk sends an HTTP response (at least in the case of errors),
+	  it attempts to read & discard the content of the request. If the client
+	  lies about the Content-Length, or the connection is closed from the
+	  client side before "Content-Length" bytes are sent, the request handling
+	  thread will busy loop.
 
-2018-02-02 17:35 +0000 [1adf90320d]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27807
 
-	* endpoint identifiers: Some code cleanup.
+	  Change-Id: I945c5fc888ed92be625b8c35039fc6d2aa89c762
 
-	  res_pjsip_endpoint_identifier_user.c:
-	  * Fix copy/paste error in find_endpoint().  We were using a constant
-	  "anonymous" string instead of the passed in endpoint_name when checking
-	  the transport domain for an endpoint match.
-	  * Eliminate RAII_VAR in find_endpoint().
-	  * Remove always true check in find_transport_state_in_use().
-	  * Remove useless CMD_STOP in find_transport_state_in_use().
+2018-04-30 17:38 +0000 [81ac32a85f]  Richard Mudgett <rmudgett@digium.com>
 
-	  res_pjsip_endpoint_identifier_anonymous.c:
-	  * Eliminate RAII_VAR in anonymous_identify().
-	  * Remove always true check in find_transport_state_in_use().
-	  * Remove useless CMD_STOP in find_transport_state_in_use().
+	* AST-2018-008: Fix enumeration of endpoints from ACL rejected addresses.
 
-	  Change-Id: I86924c31db5bd225ca0c1219c761b668c6f91189
+	  When endpoint specific ACL rules block a SIP request they respond with a
+	  403 forbidden.  However, if an endpoint is not identified then a 401
+	  unauthorized response is sent.  This vulnerability just discloses which
+	  requests hit a defined endpoint.  The ACL rules cannot be bypassed to gain
+	  access to the disclosed endpoints.
 
-2018-02-02 15:11 +0000 [93a1ffc834]  Richard Mudgett <rmudgett@digium.com>
+	  * Made endpoint specific ACL rules now respond with a 401 unauthorized
+	  which is the same as if an endpoint were not identified.  The fix is
+	  accomplished by replacing the found endpoint with the artificial endpoint
+	  which always fails authentication.
 
-	* res_pjsip.c: Fix documentation typos.
+	  ASTERISK-27818
 
-	  Change-Id: I82ae0b92bfa2ece84a5c684efd9eefdc83ebd068
+	  Change-Id: Icb275a54ff8e2df6c671a6d9bda37b5d732b3b32
 
-2018-02-02 15:43 +0000 [5d16cefd63]  Richard Mudgett <rmudgett@digium.com>
+2018-06-08 15:02 +0000 [0743ad6422]  Alexander Traud <pabstraud@compuserve.com>
 
-	* res_sorcery_realtime.c: Fix ref leak if object failed to apply.
+	* res_rtp_asterisk: Allow OpenSSL configured with no-deprecated.
 
-	  Change-Id: I3c7106ff77009754725cee790eadf5da44154ab6
+	  Furthermore, allow OpenSSL configured with no-dh. Additionally, this change
+	  allows auto-negotiation of the elliptic curve/group for servers, not only with
+	  OpenSSL 1.0.2 but also with OpenSSL 1.1.0 and newer. This enables X25519
+	  (since OpenSSL 1.1.0) and X448 (since OpenSSL 1.1.1) as a side-effect.
 
-2018-01-24 19:58 +0000 [dd9690f68c]  Sungtae Kim <pchero21@gmail.com>
+	  ASTERISK-27910
 
-	* manager.c: Fixed "(null):" header in AMI AsyncAGIEnd event
+	  Change-Id: I5b0dd47c5194ee17f830f869d629d7ef212cf537
 
-	  * Changed to create ami_event string only when the given blob is not
-	  json_null().
-	  * Fixed bad expression.
+2018-06-08 06:01 +0000 [99aed78078]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27621
+	* crypto.h: Repair ./configure --with-ssl=PATH.
 
-	  Change-Id: Ice58c16361f9d9e8648261c9ed5d6c8245fb0d8f
+	  ASTERISK-27908
 
-2018-02-01 13:01 +0000 [8125b825d3]  Joshua Elson <joshelson@gmail.com>
+	  Change-Id: Iac49d9f82faeb8a4611c6805906bd6d650b1b1d8
 
-	* res_pjsip_mwi.c: Fix null pointer crash
+2018-06-08 04:03 +0000 [ca682f0030]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27652 #close
+	* res_crypto: Allow OpenSSL configured with no-deprecated.
 
-	  Change-Id: I78a0d38bfd8d0d82830f3d53da04872d6b67284d
+	  The header <openssl/rsa.h> had to be included explicitly.
 
-2018-02-01 15:03 +0000 [09edcd93aa]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27906
 
-	* appdocsxml.xslt: Add Language to channel snapshot transformation
+	  Change-Id: I41743801eed998c039d73db7a0762d104a4f75b2
 
-	  Change-Id: I8f494b0c895a69b8bc94656d0c6ceebecb0394d8
+2018-06-08 02:41 +0000 [234bf4b7ff]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-31 15:40 +0000 [665444b772]  Richard Mudgett <rmudgett@digium.com>
+	* res_srtp: Repair ./configure --with-ssl=PATH.
 
-	* manager.c: Fix potential memory leak and corruption.
+	  ASTERISK-27905
 
-	  ast_str_append_event_header() could potentially leak and corrupt memory if
-	  the ast_str needed to expand to add the AMI event header.
+	  Change-Id: Ibb7dc148a0048f4f9c3b12937ba4240dff0d15e2
 
-	  * Fixed to return error if the ast_str_append() failed.
+2018-05-31 10:25 +0000 [65ff2f057a]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: I92f36b855540743b208d76e274152ee2d758176d
+	* func_odbc: NODATA if SQLNumResultCols returned 0 columns on readsql
 
-2018-01-31 17:27 +0000 [ef1aeb9437]  Richard Mudgett <rmudgett@digium.com>
+	  The functions acf_odbc_read/cli_odbc_read ignore a number of columns
+	  returned by the SQLNumResultCols.
+	  If the number of columns is zero it means no data.
+	  In this case, a SQLFetch function has to be not called,
+	  because it will cause an error.
 
-	* manager_channels.c: Reordered ast_manager_build_channel_state_string_prefix()
+	  ASTERISK-27888 #close
 
-	  * Made not allocate memory if the channel snapshot is an internal channel.
+	  Change-Id: Ie0f7bdac6c405aa5bbd38932c7b831f90729ee19
 
-	  * Free memory earlier when no longer needed.
+2018-06-07 08:46 +0000 [1725eaf8fb]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ia06e0c065f1bd095781aa3f4a626d58fa4d28b38
+	* chan_pjsip:  Register for "BEFORE_MEDIA" responses
 
-2018-01-28 10:10 +0000 [7debdd285c]  George Joseph <gjoseph@digium.com>
+	  chan_pjsip wasn't registering for "BEFORE_MEDIA" responses which meant
+	  it was not updating HANGUPCAUSE for 4XX responses.  If the remote end
+	  sent a "180 Ringing", then a "486 Busy", the hangup cause was left at
+	  "180 Normal Clearing".
 
-	* res_pjsip_pubsub: Prune subs with reliable transports at startup
+	  * Removed chan_pjsip_incoming_response from the original session
+	    supplement (which was handling only "AFTER MEDIA") and added it to a
+	    new session supplement which accepts both "BEFORE_MEDIA" and
+	    "AFTER_MEDIA".
 
-	  In an earlier release, inbound registrations on a reliable transport
-	  were pruned on Asterisk restart since the TCP connection would have
-	  been torn down and become unusable when Asterisk stopped.  This same
-	  process is now also applied to inbound subscriptions.
+	  * Also cleaned up some cleanup code in load module.
 
-	  Also fixed issues in res_pjsip_registrar where it wasn't handling the
-	  monitoring correctly when multiple registrations came in over the same
-	  transport.
+	  ASTERISK-27902
 
-	  To accomplish this, the pjsip_transport_event feature needed to
-	  be refactored to allow multiple monitors (multiple subcriptions or
-	  registrations from the same endpoint) to exist on the same transport.
-	  Since this changed the API, any external modules that may have used the
-	  transport monitor feature (highly unlikey) will need to be changed.
+	  Change-Id: If9b860541887aca8ac2c9f2ed51ceb0550fb007a
 
-	  ASTERISK-27612
-	  Reported by: Ross Beer
+2018-06-07 07:19 +0000 [9f2eb17005]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Iee87cf4eb9b7b2b93d5739a72af52d6ca8fbbe36
+	* ooh323c: GCC 8.1 warned about output truncated before terminating nul.
 
-2018-01-31 15:15 +0000 [698b28e9c7]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27901
 
-	* res_pjsip_registrar_expire: Mark module deprecated.
+	  Change-Id: I5a8e894f4924ef52e3094f6870656a559d67f3d7
 
-	  The functionality of this module was already moved to
-	  res_pjsip_registrar, mark it deprecated and add message to CHANGES.
+2018-06-05 13:43 +0000 [7af5e86821]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: I90c7d52c7e15e85fde3389d5eaccb05b97848813
+	* pjsip_options: show/reload AOR qualify options using CLI
 
-2018-01-30 19:22 +0000 [6520489952]  Richard Mudgett <rmudgett@digium.com>
+	  Currentrly pjsip_options code does not handle the situation when the
+	  AOR qualify options were changed.
 
-	* bridge_softmix.c: Report not talking immediately when muted.
+	  Also there is no way to find out what qualify options are using.
 
-	  Currently in app_confbridge if someone mutes a channel while that channel
-	  is talking, the talk detection code is suspended while the channel is
-	  muted.  As far an an external observer is concerned, the muted channel's
-	  talk status is still "talking" even though the channel is not contributing
-	  audio to the conference bridge.  When the channel is later unmuted, it
-	  takes the usual 'dsp_silence_threshold' option time to clear the talking
-	  status even though the channel may have stopped talking while the channel
-	  was muted.
+	  This patch add CLI commands to show and synchronize Aor qualify options:
+	  pjsip show qualify endpoint <id>
+	      Show the current qualify options for all Aors on the PJSIP endpoint.
+	  pjsip show qualify aor <id>
+	      Show the PJSIP Aor current qualify options.
+	  pjsip reload qualify endpoint <id>
+	      Synchronize the qualify options for all Aors on the PJSIP endpoint.
+	  pjsip reload qualify aor <id>
+	      Synchronize the PJSIP Aor qualify options.
 
-	  * In bridge_softmix.c, clear the talking status and report talking stopped
-	  if the channel was talking when the channel is muted.  When the channel is
-	  unmuted and the channel is still talking then report the channel as
-	  talking since it is contributing audio to the bridge again.
+	  ASTERISK-27872
 
-	  ASTERISK-27647
+	  Change-Id: I1746d10ef2b7954f2293f2e606cdd7428068c38c
 
-	  Change-Id: Ie4fdbc05a0bc7343c2972bab012e2567917b3d4e
+2018-05-22 16:21 +0000 [e46b442e38]  Alexei Gradinari <alex2grad@gmail.com>
 
-2018-01-30 15:00 +0000 [4a337b1a76]  Richard Mudgett <rmudgett@digium.com>
+	* pjsip_options: handle modification of qualify options in realtime
 
-	* app_confbridge: Update dsp_silence_threshold and dsp_talking_threshold docs.
+	  Currentrly pjsip_options code does not handle the situation when the
+	  qualify options were changed in realtime database.
+	  Only 'module reload res_pjsip' helps.
 
-	  The dsp_talking_threshold does not represent time in milliseconds.  It
-	  represents the average magnitude per sample in the audio packets.  This is
-	  what the DSP uses to determine if a packet is silence or talking/noise.
+	  This patch add a check on contact add/update observers if the contact
+	  qualify options are different than local aor qualify options.
+	  If the qualify options were modified then synchronize
+	  the pjsip_options AOR local state.
 
-	  Change-Id: If6f939c100eb92a5ac6c21236559018eeaf58443
+	  ASTERISK-27872
 
-2018-01-31 11:00 +0000 [2c4dde0ed0]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Id55210a18e62ed5d35a88e408d5fe84a3c513c62
 
-	* res_pjsip_registrar.c: Fix compiler error.
+2018-05-30 01:12 +0000 [e078558038]  Pirmin Walthert <infos@nappsoft.ch>
 
-	  Need to include signal.h to define pthread_kill() and SIGURG.
+	* bridge_channel.c: Fix Deadlock when using Local channels and fax gateway
 
-	  Change-Id: I10ae3aa4bf8e7386ac29ade78c0f2caed8e674fa
+	  ast_indicate is invoked with the bridge locked. As ast_indicate locks the
+	  other end of the bridge as well this can lead to a deadlock in some situations.
+	  (Especially when a different thread does the same in the reverse order).
+	  This patch calls ast_indicate after unlocking the bridge which fixes the
+	  deadlock. Calling ast_indicate with these parameters without locking the
+	  bridge should be safe as this is done at different places without a
+	  bridge lock.
 
-2018-01-30 23:05 +0000 [4c8c0e4c22]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27094 #close
+	  Reported-by: David Brillert
 
-	* res_pjsip_session: Prevent crash during shutdown.
+	  Change-Id: I5f86c1e2ce75b9929a36ab589b18c450e62ea35f
 
-	  pjproject does not have a function to reverse pjsip_inv_usage_init.
-	  This means we need to ignore any calls to the functions once shutdown is
-	  final.
+2018-06-04 09:50 +0000 [437ab41881]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27571 #close
+	* app_sendtext:  Allow content types other than text/plain
 
-	  Change-Id: Ia550fcba563e2328f03162d79fb185f16b7c9b9d
+	  There was no real reason to limit the conteny type to text/plain other
+	  than that's what it was limited to before.  Now any text/* content
+	  type will be allowed for channel drivers that don't support enhanced
+	  messaging and any type will be allowed for channel drivers that do
+	  support enhanced messaging.
 
-2018-01-29 13:46 +0000 [5b699c812e]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I94a90cfee98b4bc8e22aa5c0b6afb7b862f979d9
 
-	* res_pjsip_registrar_expire:  Refactor into res_pjsip_register
+2018-05-28 19:17 +0000 [a7f4121238]  William McCall <william.mccall@gmail.com>
 
-	  res_pjsip_registrar_expire remains as an empty module for now.
+	* app_confbridge: Add talking indicator for ConfBridgeList AMI response
 
-	  Change-Id: Ib93698938bae548d2199cb542f3692d1a171239f
+	  When an AMI client connects, it cannot determine if a user was talking
+	  prior to a transition in the user speaking state (which would generate
+	  a ConfbridgeTalking event). This patch causes app_confbridge to track the
+	  talking state and make this state available via ConfBridgeList.
 
-2018-01-29 10:20 +0000 [967c70acb5]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27877 #close
 
-	* BuildSystem: Enable autotools in FreeBSD.
+	  Change-Id: I19b5284f34966c3fda94f5b99a7e40e6b89767c6
 
-	  In the current versions of FreeBSD, the apps of GNU autotools do not need to
-	  be called with a version anymore. The latest version can be invoked directly.
-	  Additionally, the script ./bootstrap.sh asked for autoconf 2.62 and
-	  automake 1.9, versions which are not available as port anymore.
+2018-05-29 12:28 +0000 [6bbede84fb]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27637
+	* app_meetme: Fix manager event documentation for several events.
 
-	  Change-Id: Id7b94b80e78cc943a40ba79b697e3f70019820a7
+	  The MeetmeJoin, MeetmeLeave, MeetmeEnd, MeetmeMute, MeetmeTalking, and
+	  MeetmeTalkRequest AMI events were documented with sending out a Usernum
+	  header when the User header was actually output.
 
-2018-01-29 10:00 +0000 [dd6b8cd0b2]  Alexander Traud <pabstraud@compuserve.com>
+	  * Change the online documentation to match reality.
 
-	* app_voicemail: Avoid always true when using pointer address.
+	  ASTERISK-27873
+	  ASTERISK-25261
 
-	  clang 4.0 warned about this.
+	  Change-Id: I437bc70618d07c183c9624b7069c2fcae7f17a39
 
-	  ASTERISK-27635
+2018-05-28 10:29 +0000 [24503fb600]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I213f230607d7fbe97c0f5f2d60da9cbf5a2d8231
+	* tcptls.h: Repair ./configure --with-ssl=PATH.
 
-2018-01-19 05:16 +0000 [8daed642c7]  Alexander Traud <pabstraud@compuserve.com>
+	  asterisk/tcptls.h was included (explicitly, implicitly, or transitively). Those
+	  inclusions got replaced by forward declarations. As side effect, the inclusions
+	  got completed.
 
-	* install_prereq: Update RHEL/CentOS/Fedora libraries.
+	  ASTERISK-27878
 
-	  deleted
-	   automake git ncurses-devel pjproject-devel sqlite2-devel libsqlite3x-devel
+	  Change-Id: I9d102728e30336d6522e5e4ae9e964013a0835f7
 
-	  renamed
-	   radiusclient-ng-devel to radcli-devel
-	   gmime22-devel to gmime-dev
+2018-05-25 09:55 +0000 [d36338ce2b]  Alexander Traud <pabstraud@compuserve.com>
 
-	  added
-	   alsa-lib-devel bash binutils-devel bison doxygen flex hoard make pkgconfig
-	   speexdsp-devel uriparser-devel uw-imap-devel wget xmlstarlet zlib-devel
+	* tcptls: Allow OpenSSL configured with no-dh.
 
-	  ASTERISK-27599
-	  Reported by: Said Masoud
+	  Additionally, this change allows auto-negotiation of the elliptic curve/group
+	  for servers, not only with OpenSSL 1.0.2 but also with OpenSSL 1.1.0 and newer.
+	  This enables X25519 (since OpenSSL 1.1.0) and X448 (since OpenSSL 1.1.1) as a
+	  side-effect.
 
-	  Change-Id: I05bb0af98ae532b2d5f37478e38b8f0762b1c035
+	  ASTERISK-27876
 
-2018-01-28 05:20 +0000 [f59fb7fec8]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I62c2aba4a630aefc231b71f646207e8c027d9497
 
-	* BuildSystem: Remove unused variables.
+2018-05-25 07:22 +0000 [91616f4524]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Because of a copy-and-paste from the script build_tools/download_externals,
-	  the script build_tools/list_valid_installed_externals got its local variables.
-	  However in the latter, three variables were not used actually.
+	* tcptls: Allow OpenSSL 1.1.x configured with enable-ssl3-method no-deprecated.
 
-	  Change-Id: I252de5a98c17ea54459174875357c22c2eebe8d5
+	  ASTERISK-27874
 
-2018-01-25 12:06 +0000 [154bccf147]  Corey Farrell <git@cfware.com>
+	  Change-Id: Ica65113511c7a1c13f7988e7d9e7d9e7f3f620dd
 
-	* loader: Use ast_cli_completion_add for 'module load' completion.
+2018-05-15 08:45 +0000 [2bf26ce5ac]  George Joseph <gjoseph@digium.com>
 
-	  This addresses all performance issues with 'module load' completion.  In
-	  addition to using ast_cli_completion_add we stop using libedit's
-	  filename_completion_function, instead using ast_file_read_dir.  This
-	  ensures all results are produced from a single call to opendir.
+	* ast_coredumper:  Fix output directory and variable precedence
 
-	  Change-Id: I8bf51ffaa7ef1606f3bd1b5bb13f1905d72c6134
+	  The OUTPUTDIR variable in ast_debug_tools.conf.sample is now set
+	  to "/tmp" instead of "/some/directory".
 
-2018-01-27 09:44 +0000 [4bb38022ea]  Alexander Traud <pabstraud@compuserve.com>
+	  Variables set on the command line or that are already in the
+	  environment now take predecence over variables set in the config files.
 
-	* core: Fix unused variable error in handle_show_sysinfo.
+	  ASTERISK-27846
+	  Reported by: Ted G
 
-	  The previous fix broke the case
-	  HAVE_SYSINFO = no
-	  HAVE_SYSCTL = yes
-	  HAVE_SWAPCTL = no
-	  which occurs on FreeBSD 11.1 for example.
+	  Change-Id: Ie8baec52d531886bf5849ec1d59bb59dc87ad387
 
-	  ASTERISK-26563
+2018-05-09 08:31 +0000 [c5d2bf05f4]  Torrey Searle <torrey@voxbone.com>
 
-	  Change-Id: If77c39bc75f0b83a6c8a24ecb2fa69be8846160a
+	* res/res_rtp_asterisk: ensure marker bit is correctly set on ssrc change
 
-2018-01-27 08:54 +0000 [021168f06f]  Alexander Traud <pabstraud@compuserve.com>
+	  Certain race conditions between changing bridge types and DTMF can
+	  cause the current FLAG_NEED_MARKER_BIT to send the marker bit before
+	  the actual first packet of native bridging.
 
-	* editline: Avoid shifting a negative signed value.
+	  This logic keeps track of the ssrc the bridge is currently sending
+	  and will correctly ensure the marker bit is set if SSRC as changed
+	  from the previous sent packet.
 
-	  clang 4.0 warned about this.
+	  ASTERISK-27845
 
-	  ASTERISK-27630
+	  Change-Id: I01858bd0235f1e5e629e20de71b422b16f55759b
 
-	  Change-Id: Ie2725048c661c1792d8b1d498575144350b6e9ba
+2018-04-23 09:04 +0000 [a507c73a78]  Joshua Colp <jcolp@digium.com>
 
-2018-01-27 03:25 +0000 [e998c906a7]  Alexander Traud <pabstraud@compuserve.com>
+	* rtp: Add support for RTP extension negotiation and abs-send-time.
 
-	* headers: Consistent use of typeof and/or __typeof__.
+	  When RTP was originally created it had the ability to place a single
+	  extension in an RTP packet. In practice people wanted to potentially
+	  put multiple extensions in one and so RFC 5285 (obsoleted by RFC
+	  8285) came into existence. This allows RTP extensions to be negotiated
+	  with a unique identifier to be used in the RTP packet, allowing
+	  multiple extensions to be present in the packet.
 
-	  Because of a copy-and-paste error, the Asterisk project was using __typeof
-	  instead of typeof. It works because typeof, __typeof, and __typeof__ are
-	  supported by GCC, but here the escaped variant was not intended. Therefore,
-	  for consistence, we change this to typeof.
+	  This change extends the RTP engine API to add support for this. A
+	  user of it can enable extensions and the API provides the ability to
+	  retrieve the information (to construct SDP for example) and to provide
+	  negotiated information (from SDP). The end result is that the RTP
+	  engine can then query to see if the extension has been negotiated and
+	  what unique identifier is to be used. It is then up to the RTP engine
+	  implementation to construct the packet appropriately.
 
-	  Change-Id: I2a962c3e596e882f691a19345445b14571a5f07c
+	  The first extension to use this support is abs-send-time which is
+	  defined in the REMB draft[1] and is a second timestamp placed in an
+	  RTP packet which is for when the packet has left the sending system.
+	  It is used to more accurately determine the available bandwidth.
 
-2018-01-24 18:25 +0000 [87f8459410]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27831
 
-	* Update sounds release to fix siren7 and siren14 files.
+	  [1] https://tools.ietf.org/html/draft-alvestrand-rmcat-remb-03
 
-	  ASTERISK-16172
+	  Change-Id: I508deac557867b1e27fc7339be890c8018171588
 
-	  Change-Id: I2fb564258cd4db0f35952ad48b8687355c2dcad3
+2018-05-22 17:17 +0000 [1bec0c73b3]  Richard Mudgett <rmudgett@digium.com>
 
-2018-01-15 11:08 +0000 [6b771e346b]  Alexander Traud <pabstraud@compuserve.com>
+	* channel.c: Fix off nominal channel allocation failure path.
 
-	* BuildSystem: Raise autoconf version requirement to 2.60a.
+	  __ast_channel_alloc_ap() had a failure exit path that hadn't setup the fd
+	  descriptors to -1 yet.  The destructor would then attempt to close these
+	  fd's that had never been opened.
 
-	  AC_COMPUTE_INT requires at least autoconf 2.60a.
+	  Change-Id: Icf21093f36c60781e8cf6ee9d586536302af33e3
 
-	  This affects only those who contribute to Asterisk, only those who had to use
-	  the script ./bootstrap.sh. Furthermore, this change just makes sure nobody is
-	  using a too old autoconf.
+2018-05-18 12:46 +0000 [d402594f74]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  ASTERISK-16951
+	* app_queue: Update year Copyright and fix missing tabs in documentation
 
-	  Change-Id: Ibca850e2fe0e77d935207bd959bacf7197d7f637
+	  Change-Id: Ieb8faf37dc765463ee5dbca1d1343242c756b1c7
 
-2018-01-26 06:48 +0000 [7ad6792011]  Alexander Traud <pabstraud@compuserve.com>
+2018-05-18 16:45 +0000 [39632c7e00]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* install_prereq: Download latest Jansson.
+	* config.c: Fix successful DELETE treated as failure
 
-	  ASTERISK-27603
+	  The config engine destroy_func callback function returns the number of
+	  rows deleted or -1 on error.  But the function
+	  ast_destroy_realtime_fields treated non-zero return values as error.
 
-	  Change-Id: I65c587534c0ae364f063d68da1bed40bb3d5e8aa
+	  ASTERISK-27863
 
-2018-01-18 20:19 +0000 [85b384728c]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ied02b38e8196cb03043e609a0679feebd288d17b
 
-	* pbx_variables.c: Misc fixes in variable substitution.
+2018-05-14 06:07 +0000 [9f9dce05b2]  Matthew Fredrickson <creslin@digium.com>
 
-	  * Copy more than one character at a time when there is nothing to
-	  substitute.
+	* netsock2: Add ast_sockaddr_resolve_first_af to netsock2 public API
 
-	  * Fix off by one error if a '}' or ']' is missing.
+	  This function originally was used in chan_sip to enable some simplifying
+	  assumptions and eventually was copy and pasted into res_pjsip_logger and
+	  res_hep.  Since it's replicated in three places, it's probably best to
+	  move it into the public netsock2 API for these modules to use.
 
-	  * Eliminated the requirement that the "used" parameter had to point to a
-	  variable.  The current callers were always declaring a variable to meet
-	  the requirement and discarding the value put into that variable.  Now it
-	  can be NULL.
+	  Change-Id: Id52e23be885601c51d70259f62de1a5e59d38d04
 
-	  * In ast_str_substitute_variables_full() fixed using the bogus channel to
-	  evaluate a function.  We were not using the bogus channel we just created
-	  to help evaluate a subexpression.
+2018-05-20 06:36 +0000 [1424f42d25]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ia83d99f4f16abe47f329eb39b6ff2013ae7c9854
+	* libasteriskssl: Allow OpenSSL 1.0.2 configured with no-deprecated.
 
-2018-01-22 09:18 +0000 [420586beab]  Alexander Traud <pabstraud@compuserve.com>
+	  Use CRYPTO_set_id_callback(.) only with OpenSSL 0.9.8 and older.
 
-	* res_config_mysql: Avoid the header mysql_version.h.
+	  ASTERISK-27867
 
-	  ASTERISK-27607
+	  Change-Id: Iadd58d5bf6f538eb224203970a4e88e26f259655
 
-	  Change-Id: I23d00ded955c4afd5f2c3c9dc96dcb48b3f74eec
+2018-05-19 08:23 +0000 [2228ae3f27]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-05 14:46 +0000 [d5a24cfdf7]  Alexander Traud <pabstraud@compuserve.com>
+	* tcptls: Repair ./configure --with-ssl=PATH.
 
-	* install_prereq: For PJProject, point users to configure script.
+	  SSL_OP_NO_TLSv1_1 and SSL_OP_NO_TLSv1_2 got discovered without honoring a PATH.
 
-	  The installation script and the new configure option --with-pjproject-bundled
-	  aimed to accomplish the same. However, the installation script was out of
-	  date. Users should go for the maintained configure option, or the Wiki.
+	  ASTERISK-27865
 
-	  ASTERISK-24598
+	  Change-Id: I8cd358eed7411726d08fa7b01691bef122fbeb71
 
-	  Change-Id: Icbf4b562f81f7c05bd24a3805bd46c0beb4ebd44
+2018-03-27 18:53 +0000 [2ca3b6d9cc]  Nic Colledge <nic@njcolledge.net>
 
-2018-01-20 12:58 +0000 [31e5008b47]  Alexander Traud <pabstraud@compuserve.com>
+	* app_voicemail: Fix data-type mismatch between app_voicemail and database
 
-	* BuildSystem: Remove AC_CONFIG_AUX_DIR.
+	  Fix data-type mismatch between app_voicemail and database columns
+	  exposed by new version of MariaDB
 
-	  ASTERISK-27602
+	  ASTERISK-27760
 
-	  Change-Id: I9f4d3d2bc1481748e39ad1e2b0a364d38e38978b
+	  Change-Id: I8543ad480a08c98be78bde1ee870e6e6c84b2c5b
 
-2018-01-19 12:21 +0000 [ffb7b62019]  Alexander Traud <pabstraud@compuserve.com>
+2018-05-12 06:53 +0000 [97f20fe5ed]  Nic Colledge <nic@njcolledge.net>
 
-	* BuildSystem: Remove orphaned .PHONY targets.
+	* app_voicemail: Fix incorrect msg leaving/retrieving an ODBC voicemail
 
-	  Change-Id: Ic44d75141b9bf99e7d72fcc82ee111b5cf6989d2
+	  Correct the log warning message shown when ODBC voicemail
+	  retrieve_file is called and there is a null value in the category
+	  column.
+	  A more meaningfull message is now written at debug level.
 
-2018-01-19 12:14 +0000 [3b566fc8ec]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27853
 
-	* BuildSystem: Allow make clean all again.
+	  Change-Id: Ic36e97d5eb070a23a12ba45972f6b53e2182a3f4
 
-	  ASTERISK-27600
-	  Reported by: Hamid R. Hashmi
+2018-04-17 21:15 +0000 [52ed6bcc8f]  Brian P. Martin <asterisk-forum@silverflash.net>
 
-	  Change-Id: I683d14d024650be04074b037b6300464519409f4
+	* chan_mobile: support handling of caller-id names ("cnam").
 
-2018-01-19 06:19 +0000 [f20a522d43]  Alexander Traud <pabstraud@compuserve.com>
+	  Add support to handle caller-ID names ("cnam") in addition to caller-ID
+	  numbers.  The prior code ignored the caller-ID name altogether, and
+	  used the local name for the cell phone (e.g. "my-iphone") in its place.
 
-	* install_prereq: Update Debian/Ubuntu libraries.
+	  Note: as of this writing, at least some Android phones don't pass cnam to
+	  us. This can be seen by issuing "core set debug 2" in the CLI and watching
+	  the "CLIP" record when a call comes in.  If cnam isn't in the CLIP record,
+	  there's nothing we can do to provide one.  We'll provide a null cnam field,
+	  so later Asterisk processes know to try other sources (e.g. cidname database,
+	  OpenCNAM, etc.).
 
-	  ASTERISK-27555
+	  Reported by: Brian Martin
+	  Tested by: Brian Martin
+	  ASTERISK-27726
 
-	  Change-Id: Ieb41b0cbf968af12882b39454b819ebb48b9ea46
+	  Change-Id: I89490d85fa406c36261879c50ae5e65595538ba5
 
-2018-01-19 04:46 +0000 [1fbca9a40d]  Alexander Traud <pabstraud@compuserve.com>
+2018-05-17 01:58 +0000 [f10fc135d4]  Alexander Traud <pabstraud@compuserve.com>
 
-	* install_prereq: Support package manager DNF and yum option strict=1.
+	* res_pjsip_endpoint_identifier_ip: Unregister the module for headers.
 
-	  This re-enables the script ./contrib/scripts/install_prereq on Fedora 22 and
-	  newer, and on RHEL/CentOS when the option strict=1 was set for yum install.
+	  Asterisk uses Reference Counting to track whether a module can be unloaded.
+	  Every consumer who requires a module, increases the reference count. When the
+	  consumer goes, is unloaded itself, it has to decrease the reference count on
+	  all its used/required modules. That way
+	   core stop gracefully
+	  works on the command-line interface (CLI): One module after the other is
+	  unloaded. A recent change broke this for the module res_pjsip.
 
-	  ASTERISK-27598
-	  Reported by: Hunter Stevens, Said Masoud
+	  ASTERISK-27861
 
-	  Change-Id: I40f9517122aaa6906e8fc0962b4b8008dfddb368
+	  Change-Id: I261abcb411d026bbb0691cc78f28300bfd3103a3
 
-2018-01-09 11:29 +0000 [d5bcbd460e]  Benoît Dereck-Tricot <benoit.dereck-tricot@eyepea.eu>
+2018-05-11 12:49 +0000 [71d1e8d8c8]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pbx: Reduce verbosity while loading extensions
+	* rtp_engine: Remove the double assigned RTP payload ID of H.263+.
 
-	  Each time the dial plan is reloaded, a lot of logs like these are generated:
-	  "Added extension 'XXXXX' priority 1 to YYYYYYYYYYY"
-	  This patch changes the log level for those logs.
+	  Mantis-3709 (Commit 68ff3c3, Asterisk 1.2) added support for the video format
+	  H.263+. For this, the RTP payload ID 103 got assigned statically. Commit f1aadc8
+	  assigned another payload ID 98 for this format in Asterisk 1.6.
 
-	  ASTERISK-27084
+	  Change-Id: I90e35b158487f8f1f8187da6241b54cd3b74e667
 
-	  Change-Id: I5662902161c50890997ddc56835d4cafb456c529
+2018-05-11 12:26 +0000 [4722a653f4]  Corey Farrell <git@cfware.com>
 
-2018-01-18 14:55 +0000 [b353c90627]  Sean Bright <sean.bright@gmail.com>
+	* cli: Display correct unit for HTTP timeout in "manager show settings".
 
-	* res_pjsip: Document tlsv1_1 and tlsv1_2 methods
+	  HTTP timeout is in seconds, not minutes.
 
-	  Change-Id: I67ed9039bf3f132fb20ee7a750e0aef0f704d7d3
+	  ASTERISK-27852 #close
 
-2018-01-08 23:50 +0000 [1488efb3a8]  Igor Goncharovsky <igor.goncharovsky@gmail.com>
+	  Change-Id: Ie6640835cb07307555741f9b559c2eb876d9343e
 
-	* chan_unistim: Fix hold function ability to lock/crash asterisk
+2018-05-11 10:37 +0000 [263637a38d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  This patch fix chan_unistim hold functions to correctly support
-	  hold function in different states possible in case of multiple lines
-	  established on the phone
+	* rtp_engine: Avoid a typo error in Doxygen for ast_rtp_codecs_find_payload_code.
 
-	  ASTERISK-26596 #close
+	  Change-Id: Ica089d4507a27ddfc4ce3a88d697ffbef378de48
 
-	  Change-Id: Ib1e04e482e7c8939607a42d7fddacc07e26e14d4
+2018-05-06 21:17 +0000 [b5914d90ac]  Corey Farrell <git@cfware.com>
 
-2017-12-27 17:44 +0000 [0b399013c6]  Graham Mainwaring <graham@mhn.org>
+	* Fix GCC 8 build issues.
 
-	* app_followme:  Add a prompt to be read when a call is connected
+	  This fixes build warnings found by GCC 8.  In some cases format
+	  truncation is intentional so the warning is just suppressed.
 
-	  This patch adds the ability to configure a prompt which will be read
-	  to the "winner" who pressed 1 (or the configured value) and received
-	  the call.
+	  ASTERISK-27824 #close
 
-	  ASTERISK-24372 #close
+	  Change-Id: I724f146cbddba8b86619d4c4a9931ee877995c84
 
-	  Change-Id: I6ec1c6c883347f7d1e1f597189544993c8d65272
+2018-05-11 07:10 +0000 [919b0eb3f2]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-17 00:28 +0000 [22edb10c44]  Corey Farrell <git@cfware.com>
+	* rtp_engine: Allow Media Formats with add_static_payload(-1) on egress again.
 
-	* loader: Miscellaneous fixes.
+	  This issue affected only installations with rtp_use_dynamic=yes in asterisk.conf
+	  which is the default since Asterisk 15. Codec 2 and SiLK were built-in examples
+	  of media formats which were affected.
 
-	  * Remove comment about lazy load.
-	  * Improve message about module already being loaded and running.
-	  * Handle allocation error in add_to_load_order.
-	  * Dead code elimination from modules_shutdown.
+	  ASTERISK-27850
+	  Reported by: Dinis Brazão, Selene Feigl
 
-	  Change-Id: I22261599c46d0f416e568910ec9502f45143197f
+	  Change-Id: I08c1e76433a67e4350141d38cacf3a1cb5086496
 
-2018-01-17 08:36 +0000 [9ffa1f4301]  Alexander Traud <pabstraud@compuserve.com>
+2018-05-09 09:30 +0000 [2e37684913]  Corey Farrell <git@cfware.com>
 
-	* BuildSystem: Use the detected name for MD5 everywhere.
+	* git: Ignore *.orig.
 
-	  Affacted the (automatic) download script for external modules:
-	  ./build_tools/download_externals
+	  This prevents accidental commit of files created by patch.
 
-	  ASTERISK-27596
+	  Change-Id: I68380db61f0f9d620046f719ccd978811d0e9964
 
-	  Change-Id: If4c3176f7bf58df32fec6e02a659f1a78d57cf4b
+2018-04-18 02:27 +0000 [2d81709ab1]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-17 07:11 +0000 [de65fd11fe]  Alexander Traud <pabstraud@compuserve.com>
+	* sip_to_pjsip: Enable python3 compatibility.
 
-	* BuildSystem: Invoke install not in GNU but POSIX style.
+	  The script remains compatible with Python 2.7 but now also works with
+	  Python 3.3 and newer; to ease the migration from chan_sip to chan_pjsip.
 
-	  ASTERISK-27594
+	  ASTERISK-27811
 
-	  Change-Id: Iaaa6a19d2fe031dffcba441d0502a7ea65c93cb3
+	  Change-Id: I59cc6b52a1a89777eebcf25b3023bdf93babf835
 
-2018-01-17 06:47 +0000 [72845d54f0]  Alexander Traud <pabstraud@compuserve.com>
+2018-05-08 14:28 +0000 [cea87fe7b8]  Corey Farrell <git@cfware.com>
 
-	* BuildSystem: In OpenBSD, xmlstarlet is xml.
+	* makeopts.in: Remove unused/undefined AST_MARCH_NATIVE.
 
-	  ASTERISK-27593
+	  Change-Id: I617a96ebb83ec99f5d3176bbbee2d2a272ccb203
 
-	  Change-Id: I1c7087f7f7582e40b3312c690d912c9a86466805
+2018-05-08 04:59 +0000 [9f1e1d153a]  Jaco Kroon <jaco@uls.co.za>
 
-2018-01-17 02:51 +0000 [985834607b]  Alexander Traud <pabstraud@compuserve.com>
+	* manager: fix digest auth for ami/http mechanism.
 
-	* BuildSystem: Detect external library Lua in version 5.3.
+	  Due to a fixed size buffer the digest authentication could be
+	  incorrectly calculated if a large URI was provided, causing
+	  authentication failure. The buffer is now dynamically allocated to allow
+	  any size URI within the normal limits of the HTTP request size.
 
-	  On some platforms, you decide to go for one specific version of Lua, for
-	  example in OpenBSD. On other platforms, you are able to install several versions
-	  side-by-side, for example in Ubuntu and Fedora. Asterisk already works with
-	  Lua 5.3. Asterisk failed to detect Lua 5.3 on those platforms which allow
-	  several versions.
+	  ASTERISK-27841
 
-	  ASTERISK-27592
+	  Change-Id: I660609db13b8f9e5f9567f339dd804f4985d41b3
 
-	  Change-Id: If7a4b395d844a464e9a1f4f626c5bff4ee67eed8
+2018-05-04 13:47 +0000 [d855658f23]  Corey Farrell <git@cfware.com>
 
-2018-01-16 08:32 +0000 [1bbd9f4a47]  Richard Mudgett <rmudgett@digium.com>
+	* app_macro: Prevent infinite loop in find_matching_priority.
 
-	* taskprocessor.c: Increase the number of tps_singletons container buckets.
+	  Use AST_PBX_MAX_STACK to escape if we recurse 128 times.  This will
+	  prevent crash if dialplan contains an include loop.  Log an error when
+	  this occurs, at most one message per call to Macro() so we avoid logger
+	  spam.
 
-	  Since v12 the number of taskprocessors in the system has increased a lot.
-	  Small systems can easily have over a hundred and larger systems can have
-	  thousands.
+	  ASTERISK-26570 #close
 
-	  Most uses of the tps_singletons container deal with creating and
-	  destroying the taskprocessors.  However, the pjsip distributor looks up
-	  taskprocessors/serializers by name frequently.  It needs to find the
-	  serializer for incoming SIP responses to distribute them to the
-	  appropriate serializer.
+	  Change-Id: I6c71b76998c31434391b150de055ae9a531e31da
 
-	  Change-Id: Ice0603606614ba49f7c0c316c524735c064e7e43
+2018-01-11 06:37 +0000 [f4c360143b]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-2018-01-16 08:20 +0000 [fbd0667c84]  George Joseph <gjoseph@digium.com>
+	* cdr_mysql: my_connect_db(): reduce indentation
 
-	* pjproject_bundled:  Prevent crash on bad outgoing header
+	  ASTERISK-27572
 
-	  We still need to figure out how a bad header is getting into the
-	  outgoing message but this patch to pjproject prevents attempting
-	  to print that header and causing a crash.
+	  Change-Id: I00bd5363ac94c764c56d8626a5945ed7f3934fcb
 
-	  For several users, this crash happens when sending 183 progress
-	  messages.
+2018-01-11 06:33 +0000 [2e44adf1c3]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  ASTERISK-26832
-	  Reported by: Ross Beer, Jan Rozhon
+	* cdr_mysql: split mysql init out of my_load_module
 
-	  Change-Id: Ie5c5a921c890c843587763e7f33f987dfe66bd16
+	  Split out mysql connection parts to a separate my_connect_db().
 
-2018-01-16 06:34 +0000 [637bd26575]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27572
 
-	* BuildSystem: Avoid $EUID and use id -u instead.
+	  Change-Id: If2ee676056067cc693ff08be68ee4944bf35b49f
 
-	  Makefile included a call to ${EUID} which requires the shell bash. To keep
-	  compatibility with other shells like dash or ksh, use id -u instead.
+2018-05-04 16:07 +0000 [8f55f7c333]  Matthew Fredrickson <creslin@digium.com>
 
-	  ASTERISK-27589
+	* res_hep: Adds hostname resolution support for capture_address
 
-	  Change-Id: Ia6e74f5bc9aab4e6dc62b7439f647b7964e6f657
+	  Previously, only an IP address would be accepted for the capture_address config
+	  setting in hep.conf.  This change allows capture_address to be a resolvable
+	  hostname or an IP address.
 
-2018-01-15 18:03 +0000 [b0a78bdc3e]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27796 #close
+	  Reported-By: Sebastian Gutierrez
 
-	* cel_odbc.c: Fix menuslect module description display.
+	  Change-Id: I33e1a37a8b86e20505dadeda760b861a9ef51f6f
 
-	  Asterisk's makefile for menuselect has a very simple source file parsing
-	  script that looks for AST_MODULE_INFO lines to extract the quoted string
-	  as a module description.  If it does not find a quoted string it uses the
-	  whole line as the description.
+2018-04-20 18:12 +0000 [7528b86cad]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I80f13a63818e4e28d683639a94a4dfaea405c1d5
+	* stream: Make the topology a reference counted object.
 
-2018-01-15 01:41 +0000 [c0a4a939cc]  Yasuhiko Kamata <yasuhiko.kamata@nxtg.co.jp>
+	  The stream topology has no lock of its own resulting in
+	  another lock protecting it in some way (for example the
+	  channel lock). If multiple channels are being juggled at
+	  the same time this can be problematic. This change makes
+	  the topology a reference counted object instead which
+	  guarantees it will remain valid even without the channel
+	  lock being held.
 
-	* chan_sip: 3PCC patch for AMI "SIPnotify"
+	  Change-Id: I4f4d3dd856a033ed55fe218c3a4fab364afedb03
 
-	  A patch for sending in-dialog SIP NOTIFY message
-	  with "SIPnotify" AMI action.
+2018-03-21 07:30 +0000 [6301531416]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  ASTERISK-27461
+	* chan_dahdi: Configurable dialed digit timeouts
 
-	  (created patch for 13 branch manually due to merge conflict)
+	  Analog phones dial overlap dialing and it is chan_dahdi's job to read the
+	  numbers.  It has three timeout constants that this commit converts to
+	  channel-level configuration options:
 
-	  Change-Id: I255067f02e2ce22c4b244f12134b9a48d210c22a
+	  * firstdigit_timeout: Default time (ms) to detect first digit
 
-2018-01-13 13:49 +0000 [948b852f48]  George Joseph <gjoseph@digium.com>
+	  * interdigit_timeout: Default time (ms) to detect following digits
 
-	* config_transport:  Enable TCP_NODELAY on TLS transports
+	  * matchdigit_timeout: Default time (ms) to wait in case of ambiguous
+	  match.  This happens when the dialed digits match a number in the current
+	  context but are also the prefix of another number.
 
-	  We did this for TCP transports already but I'm not sure why we
-	  didn't do it for TLS transports.
+	  Change-Id: Ib728fa900a4f6ae56d1ed810aba61b6593fb7213
 
-	  ASTERISK_27474 #not_final_fix
+2018-05-03 06:34 +0000 [de3ca9bada]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I5b1ef4b882f7b859e718236686b7898751dbb262
+	* res_ari: Remove requirement that body exists when debug is on.
 
-2018-01-12 18:37 +0000 [f01bac230d]  Corey Farrell <git@cfware.com>
+	  The "ari set debug" code for incoming requests incorrectly assumed
+	  that all requests would contain a body. If one did not exist the
+	  request would be incorrectly rejected. The response that was sent
+	  was also incomplete as an incorrect function was used to construct
+	  the response.
 
-	* res_stasis_recording: Allow symbolic links in configured recordings dir.
+	  The code has now been changed to no longer require a request to have
+	  a body and the response updated to use the correct function.
 
-	  If any component of ast_config_AST_RECORDING_DIR is a symbolic link we
-	  would incorrectly assume the ARI user was trying to escape the recording
-	  path.  Create additional check to check the recording directory's
-	  realpath, only deny access if both do not match.
+	  ASTERISK-27801
 
-	  This is needed by the testsuite when run by 'run-local'.
+	  Change-Id: I4eef036ad54550a4368118cc348765ecac25e0f8
 
-	  Change-Id: I9145e841865edadcb5f75cead3471ad06bbb56c0
+2018-04-30 15:15 +0000 [069a0b7593]  Sean Bright <sean.bright@gmail.com>
 
-2018-01-12 12:00 +0000 [97b574c927]  Corey Farrell <git@cfware.com>
+	* iostreams: Add some documentation for the ast_iostream_* functions
 
-	* menuselect: Remove unused dev-mode option TRACE_FRAMES.
+	  Change-Id: Id71b87637f0a484eb5a1cd26c3d1c7c15c7dcf26
 
-	  ASTERISK-27575 #close
+2018-05-02 07:43 +0000 [239074c759]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ica3a522892afed7a96816a5ecf140e1671f46ad4
+	* pjsip: Increase maximum number of usable ciphers & other cleanups
 
-2018-01-12 03:50 +0000 [f07b0df627]  Alexander Traud <pabstraud@compuserve.com>
+	  * Increase maximum number of ciphers from 100 to 256 (or whatever
+	    PJ_SSL_SOCK_MAX_CIPHERS is #define'd to)
 
-	* res_config_pgsql: Avoid typecasting an int to unsigned char.
+	  * Simplify logic in cipher_name_to_id()
 
-	  clang 5.0 warned about this.
+	  * Make signed/unsigned comparison consistent
 
-	  ASTERISK-27576
+	  Re: https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=897412
 
-	  Change-Id: If41f400a51973c06cdb9b75462e535b616bfe385
+	  Reported by: Ondřej Holas
 
-2018-01-12 03:17 +0000 [659c44f5fb]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Iea620f03915a1b873e79743154255c3148a514e7
 
-	* BuildSystem: Really do not pass unknown-warning options to the compiler.
+2018-04-30 17:24 +0000 [11b7de82c5]  Richard Mudgett <rmudgett@digium.com>
 
-	  When an older GCC version is called with a too new warning option, GCC exited
-	  with an error and Asterisk was not built. Therefore, the configure script tests
-	  the installed compiler whether it supports that warning option. If not, Asterisk
-	  does not pass it to the installed compiler. However, some compilers (like clang)
-	  do not exit (error) but give just a warning in such a case. Because the compiler
-	  did not exit, Asterisk passed the unknown-warning option.
+	* res_pjsip/pjsip_distributor.c: Add missing off-nominal request response.
 
-	  ASTERISK-27560
+	  Change-Id: I389579b39c523d1d1e8ce020ef549a8bb5781c9b
 
-	  Change-Id: Ia9d148e689c173df4e91699113605dab2de36038
+2018-04-30 17:20 +0000 [6cab3c836a]  Richard Mudgett <rmudgett@digium.com>
 
-2018-01-12 04:27 +0000 [45008c604d]  Alexander Traud <pabstraud@compuserve.com>
+	* res_pjsip/pjsip_distributor.c: Pull some assignments out of if tests.
 
-	* app_osplookup.c: Avoid two format truncations.
+	  Change-Id: I3d30d638b53a4bbe9bf9aad853c649d583894112
 
-	  GCC 7 warned about this.
+2018-04-30 09:38 +0000 [afdca5c68c]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-27578
+	* res_rtp_asterisk: Always update SRTP on local SSRC change.
 
-	  Change-Id: I4a00458dbe9b575ef04338b6a7852272745e1552
+	  When the local SSRC changes we need to update the SRTP information
+	  so that the proper key is used. This is commonly done as a result
+	  of bridging two channels together. Previously we only updated
+	  the SRTP information if media had already flowed, but in practice
+	  the channel driver may have already performed SRTP negotiation and
+	  set up the previous SSRC. We now always do it on a local SSRC
+	  change.
 
-2018-01-12 04:03 +0000 [d53a312a03]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27795
+	  ASTERISK-27800
 
-	* chan_ooh323: Avoid typecasting an int to unsigned short.
+	  Change-Id: Ia7c8e74c28841388b5244ac0b8fd6c1dc6ee4c10
 
-	  clang 5.0 warned about this.
+2018-02-13 12:55 +0000 [0827d5cc53]  Gaurav Khurana <gkhurana@godaddy.com>
 
-	  ASTERISK-27577
+	* Add the ability to read the media file type from HTTP header for playback
 
-	  Change-Id: I898fe4255023138a9e8b579fe4482fcf582f2b78
+	  How it works today:
+	  media_cache tries to parse out the extension of the media file to be played
+	  from the URI provided to Asterisk while caching the file.
 
-2018-01-05 15:17 +0000 [1d625a7428]  Alexander Traud <pabstraud@compuserve.com>
+	  What's expected:
+	  Better will be to have Asterisk get extension from other ways too. One of the
+	  common ways is to get the type of content from the CONTENT-TYPE header in the
+	  HTTP response for fetching the media file using the URI provided.
 
-	* install_prereq: Update Debian/Ubuntu libraries.
+	  Steps to Reproduce:
+	  Provide a URL of the form: http://host/media/1234 to Asterisk for media
+	  playback. It fails to play and logs show the following error line:
 
-	  ASTERISK-27555
+	  [Sep 15 15:48:05] WARNING [29148] [C-00000092] file.c:
+	  File http://host/media/1234 does not exist in any format
 
-	  Change-Id: I0818b6e42631be1b69237e2b41d3415275693e53
+	  Scenario this issue is blocking:
+	  In the case where the media files are stored in some cloud object store,
+	  following can block the media being played via Asterisk:
 
-2017-12-22 19:50 +0000 [f35960d55b]  Richard Mudgett <rmudgett@digium.com>
+	  Cloud storage generally needs authenticated access to the storage. The way
+	  to do that is by using signed URIs. With the signed URIs there's no way to
+	  preserve the name of the file.
+	  In most cases Cloud storage returns a key to access the object and preserving
+	  file name is also not a thing there
 
-	* res_pjsip: Split type=identify to IP address and SIP header matching priorities
+	  ASTERISK-27286
 
-	  The type=identify endpoint identification method can match by IP address
-	  and by SIP header.  However, the SIP header matching has limited
-	  usefulness because you cannot specify the SIP header matching priority
-	  relative to the IP address matching.  All the matching happens at the same
-	  priority and the order of evaluating the identify sections is
-	  indeterminate.  e.g., If you had two type=identify sections where one
-	  matches by IP address for endpoint alice and the other matches by SIP
-	  header for endpoint bob then you couldn't predict which endpoint is
-	  matched when a request comes in that matches both.
+	   Reporter: Gaurav Khurana
 
-	  * Extract the SIP header matching criteria into its own "header" endpoint
-	  identification method so the user can specify the relative priority of the
-	  SIP header and the IP address matching criteria in the global
-	  endpoint_identifier_order option.  The "ip" endpoint identification method
-	  now only matches by IP address.
+	  Change-Id: I1b14692a49b2c1ac67688f58757184122e92ba89
 
-	  ASTERISK-27491
+2018-04-25 01:57 +0000 [9c9f314f64]  Christof Lauber <christof.lauber@annax.ch>
 
-	  Change-Id: I9df142a575b7e1e3471b7cda5d3ea156cef08095
+	* pbx_lua:  Support displaying lua error message if no debug table exists
 
-2018-01-11 08:09 +0000 [90b2dc1450]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  The lua_error_function assumed that lua's debug table and traceback function
+	  are always accessible, which is not the case. This fixes the error message
+	  'Error in the lua error handler' triggred by switch exec() function.
+	  If this happens lua's error message is shown without traceback.
 
-	* Ignore quilt .pc directory, used in deb packaging
+	  Change-Id: I34ba0a098f1ae06a3af7b4d1b098bd43f42f96c8
 
-	  Debian packaging uses quilt to manage patches. Book-keeping for them is
-	  done using quilt (either directly, or in a compatible format), and
-	  tracked in the directory .pc .
+2017-12-11 12:34 +0000 [882e79b77e]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I22c90f3d7ab8918e6216e7b686de6fa0e1fdaa7b
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* pjsip: Rewrite OPTIONS support with new eyes.
 
-2018-01-09 11:23 +0000 [608112ced3]  Corey Farrell <git@cfware.com>
+	  The OPTIONS support in PJSIP has organically grown, like many things in
+	  Asterisk.  It has been tweaked, changed, and adapted based on situations
+	  run into.  Unfortunately this has taken its toll.  Configuration file
+	  based objects have poor performance and even dynamic ones aren't that
+	  great.
 
-	* stasis: Remove silly usage of RAII_VAR.
+	  This change scraps the existing code and starts fresh with new eyes.  It
+	  leverages all of the APIs made available such as sorcery observers and
+	  serializers to provide a better implementation.
 
-	  Change-Id: Ib11193531e797bcb16bba560a408eab155f706d1
+	  1.  The state of contacts, AORs, and endpoints relevant to the qualify
+	  process is maintained.  This state can be updated by external forces (such
+	  as a device registering/unregistering) and also the reload process.  This
+	  state also includes the association between endpoints and AORs.
 
-2018-01-09 11:16 +0000 [b7c25740fa]  Corey Farrell <git@cfware.com>
+	  2.  AORs are scheduled and not contacts.  This reduces the amount of work
+	  spent juggling scheduled items.
 
-	* stasis_bridges: Remove silly usage of RAII_VAR.
+	  3.  Manipulation of which AORs are being qualified and the endpoint states
+	  all occur within a serializer to reduce the conflict that can occur with
+	  multiple threads attempting to modify things.
 
-	  Change-Id: I0fa7ab05454f183dc4ff10e26d18776d2b0fcf1f
+	  4.  Operations regarding an AOR use a serializer specific to that AOR.
 
-2018-01-03 17:26 +0000 [2e09ed3b18]  Richard Mudgett <rmudgett@digium.com>
+	  5.  AORs and endpoint state act as state compositors.  They take input
+	  from lower level objects (contacts feed AORs, AORs feed endpoint state)
+	  and determine if a sufficient enough change has occurred to be fed further
+	  up the chain.
 
-	* res_pjsip.c: Update the endpoint identification documentation.
+	  6.  Realtime is supported by using observers to know when a contact has
+	  been registered.  If state does not exist for the associated AOR then it
+	  is retrieved and becomes active as appropriate.
 
-	  * Endpoint identify_by documentation.
-	  * IP/Header endpoint identifier documentation.
+	  The end result of all of this is best shown with a configuration file of
+	  3000 endpoints each with an AOR that has a static contact.  In the old
+	  code it would take over a minute to load and use all 8 of my cores.  This
+	  new code takes 2-3 seconds and barely touches the CPU even while dealing
+	  with all of the OPTIONS requests.
 
-	  Change-Id: Id92f00b495acca7be945daf749d2abd7f76a0b5a
+	  ASTERISK-26806
 
-2018-01-03 15:20 +0000 [dd6dee7e50]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I6a5ebbfca9001dfe933eaeac4d3babd8d2e6f082
 
-	* res_pjsip_endpoint_identifier_ip.c: Remove unnecessary requirement.
+2017-12-22 13:11 +0000 [661fec4b59]  Richard Mudgett <rmudgett@digium.com>
 
-	  The requirement that "ip" must be in the endpoint identify_by list to
-	  allow the type=identify method to identify the endpoint is not necessary.
-	  The "ip" identifier method can match one and only one endpoint.  To even
-	  work, the "ip" identifier method configuration must explicitly specify the
-	  identified endpoint.  Therefore, why bother configuring the type=identify
-	  identifier in the first place?  The requirement only adds the potential
-	  for configuration errors for no benefit.  Even worse, those configuration
-	  errors cannot be detected when the configuration loads.  The requirement
-	  was introduced with the ASTERISK_27206 patch.
+	* core: Remove unused/incomplete SDP modules.
 
-	  * Remove the code change that enforces the requiremnt.  Listing the "ip"
-	  method in the identify_by value is simply documentation.
+	  Change-Id: Icc28fbdc46f58e54a21554e6fe8b078f841b1f86
 
-	  Change-Id: Ia057f92a33fb5d9f51dc5d5692e3d5ee1a6f2c11
+2018-04-18 15:59 +0000 [ff652711c7]  Kevin Harwell <kharwell@digium.com>
 
-2018-01-05 19:03 +0000 [96040fa6bb]  Richard Mudgett <rmudgett@digium.com>
+	* translate: generic plc not filled in after translation
 
-	* res_pjsip.c: Fix ident_to_str() and refactor ident_handler().
+	  If during translation a codec could not handle a given frame the translation
+	  core would return NULL, thus not passing along the "missing" frame. Due to this
+	  there was no frame to apply generic plc to, thus rendering it useless.
 
-	  * Extracted sip_endpoint_identifier_type2str() and
-	  sip_endpoint_identifier_str2type() to simplify the calling functions.
+	  This patch makes it so the translation core produces an interpolated slin frame
+	  in the cases where an attempt was made to translate to slin, but failed. This
+	  interpolated frame is then passed along and can be used by the generic plc
+	  algorithms to fill in the frame.
 
-	  * Fixed pjsip_configuration.c:ident_to_str() building the endpoint's
-	  identify_by value string.
+	  ASTERISK-27814 #close
 
-	  Change-Id: Ide876768a8d5d828b12052e2a75008b0563fc509
+	  Change-Id: I133d084da87adef913bf2ecc9c9240e3eaf4f40a
 
-2018-01-04 17:04 +0000 [a8a2f39f04]  Richard Mudgett <rmudgett@digium.com>
+2018-04-20 07:40 +0000 [de9c0ede4a]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_endpoint_identifier_ip.c: Allow multiple IdentifyDetail AMI events.
+	* bridge_softmix: Fix sporadic incorrect video stream mapping.
 
-	  The AMI PJSIPShowEndpoint action could only list one IdentifyDetail AMI
-	  event per endpoint.  However, there is no reason that multiple
-	  type=identify sections cannot identify the same endpoint.
+	  When an externally initiated renegotiation occurred it was
+	  possible for video streams to be incorrectly remapped,
+	  resulting in no video flowing to some receivers.
 
-	  * Reworked format_ami_endpoint_identify() to generate as many
-	  IdentifyDetail AMI events as there are matching identifiers.
+	  This change ensures that only the video source sets up
+	  mappings and also that removed streams do not have mappings
+	  set up.
 
-	  Change-Id: Ie146792aef72d78e05416ab5b27bc552a30399db
+	  Change-Id: Iab05f2254df3606670774844bb0935f833d3a9b0
 
-2018-01-09 11:10 +0000 [dedf0ff2bf]  Corey Farrell <git@cfware.com>
+2018-04-20 14:07 +0000 [c481afe873]  Alexander Anikin <may213@yandex.ru>
 
-	* stasis_cache: Remove silly usage of RAII_VAR.
+	* chan_ooh323: fix ooManualProgress/ooManualRingback on ooh323 debuggin on
 
-	  Change-Id: Ifa95e5801c949df296c7e4376347730fb0ed52ef
+	  Call ooManualProgress/Ringback outside of ast_debug function
+	  when ooh323 debugging is on
 
-2018-01-09 11:09 +0000 [8a6b3537b1]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27812 #close
+	  ASTERISK-26893 #close
+	  Reported by: Dimos, Marco Giordani
 
-	* stasis_cache_pattern: Remove silly usage of RAII_VAR.
+	  Change-Id: I5873762e4f05824e7b6e94a19dd4eb56adbbbb79
 
-	  Change-Id: Ic98a51f555062cd863b6db3f8d76065943a9dea3
+2018-04-19 13:44 +0000 [5712a0ae52]  Joshua Colp <jcolp@digium.com>
 
-2018-01-09 10:57 +0000 [909e642918]  Corey Farrell <git@cfware.com>
+	* bridge_softmix: Fix some REMB bugs.
 
-	* stasis_endpoints: Remove silly usage of RAII_VAR.
+	  This change fixes a bug where a REMB collector may be
+	  freed twice, and also tweaks REMB combining such that if
+	  there is no bitrate from anyone (or there are no sources)
+	  we report 0 instead of using an old bitrate.
 
-	  Change-Id: Ic099dc552f36c353c89783a4bcfd09f010432733
+	  ASTERISK-27804
 
-2018-01-09 10:55 +0000 [bd11159dbc]  Corey Farrell <git@cfware.com>
+	  Change-Id: Ia9dc9c150043890ee7ff85e9cdec007f1a77fcfd
 
-	* stasis_message_router: Remove silly usage of RAII_VAR.
+2018-04-20 07:13 +0000 [fe072f4405]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I50d6ae230920e0b878ed9cc8f79eef746e06701d
+	* BuildSystem: Enable IMAP storage on FreeBSD and DragonFly BSD.
 
-2018-01-09 10:53 +0000 [975a38e524]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27639
 
-	* stasis_system: Remove silly usage of RAII_VAR.
+	  Change-Id: I1347f3f2f3737010d0a80a5c30b5aaf71cf3ccb0
 
-	  Change-Id: Iedbe5656cee68cd3a96a953558764aa02d4a0c3b
+2018-04-20 05:50 +0000 [efe40ff671]  Alexander Traud <pabstraud@compuserve.com>
 
-2018-01-05 05:51 +0000 [b45fb3a6ec]  Alexander Traud <pabstraud@compuserve.com>
+	* BuildSystem: Add DragonFly BSD.
 
-	* translate: Avoid absolute value on unsigned substraction.
+	  ASTERISK-27820
 
-	  ast_format_get_sample_rate(.) returns an unsigned type. The difference of a
-	  substraction between two unsigned types does not get implicitly converted to a
-	  signed type. Therefore, using abs(.) did not make sense.
+	  Change-Id: I310896143e94d65da1c2be3bb448204a8b86d557
 
-	  ASTERISK-27549
+2018-04-20 05:40 +0000 [d54637373a]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ib904d9ee0d46b6fdd1476fbc464fbbf813304017
+	* menuselect: Add DragonFly BSD.
 
-2018-01-09 08:22 +0000 [1da8846ca7]  Sean Bright <sean.bright@gmail.com>
+	  In DragonFly BSD, added libraries from ports are placed into /usr/local.
+	  Therefore, this directory must be added for the preprocessor, compiler, and
+	  linker.
 
-	* Revert "codec_opus: Make libcurl a dependency in menuselect"
+	  Beside that, the script ./configure was updated:
+	  * OSARCH list was outdated and not used, removed.
+	  * AC_CANONICAL_BUILD was not used.
+	  * _REENTRANT, this feature test macro is obsolete.
 
-	  This reverts commit 028f4320de60a204e457ad606ab0a3318493b431.
+	  ASTERISK-27820
 
-	  Change-Id: Ieb91f825cb55202a937f5361c01d356e7662b70c
+	  Change-Id: I186d88d99cfa4de6569888e12ac97bd2f441c422
 
-2018-01-08 18:47 +0000 [c67eb7031b]  Corey Farrell <git@cfware.com>
+2018-04-20 05:18 +0000 [6e9a612293]  Alexander Traud <pabstraud@compuserve.com>
 
-	* app_confbridge: Fix NULL check in action_kick_last.
+	* install_prereq: Add DragonFly BSD.
 
-	  The check for last_user == NULL needs to happen before we dereference
-	  the variable, previously it was possible for us to check flags of a NULL
-	  last_user.
+	  ASTERISK-27820
 
-	  Change-Id: I274f737aa8af9d2d53e4a78cdd7ad57561003945
+	  Change-Id: I718ddb000fe5184b1bdc7759da67a370a7520144
 
-2018-01-06 02:17 +0000 [17480f6ea4]  Corey Farrell <git@cfware.com>
+2018-04-18 11:41 +0000 [b437656c2e]  Chris-Savinovich <csavinovich@digium.com>
 
-	* res_stasis: Reduce RAII_VAR usage.
+	* "confbridge show profile bridge" does not output "sfu" when video_mode is sfu
 
-	  In addition to being a micro-optimization (RAII_VAR has overhead), this
-	  change improves output of REF_DEBUG.  Unfortunately when RAII_VAR calls
-	  ao2_cleanup it does so from a generated _dtor_varname function.  For
-	  example this caused _dtor_app to release a reference instead of
-	  __stasis_app_unregister.
+	  Fixes a bug on the "confbridge show profile bridge" cli command
+	  that showed "video_mode=no video" when video_mode was set
+	  to "sfu"
 
-	  Change-Id: I4ce67120583a446babf9adeec678b71d37fcd9e5
+	  ASTERISK-27418  #close
 
-2018-01-07 20:21 +0000 [c1acc4f364]  Corey Farrell <git@cfware.com>
+	  Change-Id: I481e3172c7f872664c7ac7809879d541c9f031e9
 
-	* res_stasis: Fix app_is_subscribed_bridge_id.
+2018-04-18 15:40 +0000 [179ae87cf4]  Corey Farrell <git@cfware.com>
 
-	  Instead of searching for bridge_id provided in an argument this function
-	  always searched for BRIDGE_ALL first.  Rewrite this function to work
-	  like the similar functions for channel and endpoint functions.
+	* Build System: Add missing ASTMM_LIBC to flex output.
 
-	  Change-Id: Ib5caca69e11727c5c8a7284a1d00621f40f1e60a
+	  Redirect libc allocation functions to use Asterisk functions for
+	  main/ast_expr2f.c and res/ael/ael_lex.c.  This will resolve errors
+	  produced by astmm.h when these files are regenerated, though other
+	  issues still remain.
 
-2018-01-05 07:58 +0000 [da1a9f392a]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK~27813
 
-	* General: Silence modules on (un)load.
+	  Change-Id: I7263e9e4217a17bde4ffaa2087a8f8aeb2a8588c
 
-	  Some (normally optional) modules created notices, warnings, and even errors
-	  in normal situations like (un)load. This cluttered the command-line interface
-	  (CLI) on start and while stopping gracefully. However, when an user went for
-	  the script './contrib/scripts/install_prereq', those modules get compiled-in
-	  because their prerequisites were met at compile time. Furthermore, because of
-	  ASTERISK_27475, the former talkative module 'res_curl' is built as side-effect.
+2018-04-18 13:40 +0000 [80e6952013]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27553
+	* format_pcm: Correct behavior of fseek and ftell for G.722
 
-	  Change-Id: I9f105f46d72553994e820679bfde3478a551b281
+	  There are twice as many samples in the same number of bytes, so redefine
+	  some of the G.722 format functions in terms of their PCM counterparts.
 
-2018-01-06 15:40 +0000 [aa52c52b21]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I6a8c7352624b930a5f2d9e4857f75283fa5dd9f9
 
-	* BuildSystem: Really do not pass unknown-warning options to the compiler.
+2018-04-17 05:33 +0000 [95e8450194]  Alexander Anikin <may213@yandex.ru>
 
-	  When an older GCC version is called with a too new warning option, GCC exited
-	  with an error and Asterisk was not built. Therefore, the configure script tests
-	  the installed compiler whether it supports that warning option. If not, Asterisk
-	  does not pass it to the installed compiler. However, some compilers (like clang)
-	  do not exit (error) but give just a warning in such a case. Because the compiler
-	  did not exit, Asterisk passed the unknown-warning option.
-
-	  ASTERISK-27560
-
-	  Change-Id: Ia9b7747f649b27ff5e9f75c3db3fee4fe7a29621
+	* chan_ooh323: introduce localras config parameter
 
-2018-01-06 01:42 +0000 [9865e689d2]  Alexander Traud <pabstraud@compuserve.com>
+	  Introduce localras parameter that specify source IP
+	  for connecting to Gatekeeper. Useful for multihome configurations.
 
-	* General: Avoid implicit conversion to char when changes value to negative.
+	  ASTERISK-25129 #close
+	  Reported by: Dmitry Melekhov
+	  Tested by: Dmitry Melekhov
 
-	  clang 5.0 warned about this.
+	  Change-Id: I0b604b01793f3e02a776502659e07cd3fc7e3097
 
-	  ASTERISK-27557
+2018-04-18 05:32 +0000 [446320f1d4]  Alexander Anikin <may213@yandex.ru>
 
-	  Change-Id: I7cceaa88e147cbdf81a3a7beec5c1c20210fa41e
+	* chan_ooh323: Fix cppcheck warnings
 
-2018-01-06 06:45 +0000 [a68da30069]  Alexander Traud <pabstraud@compuserve.com>
+	  Fix cppcheck warnings about redundant conditions and possible
+	  null pointer usage
 
-	* editline: Avoid comparison between pointer and zero character constant.
+	  ASTERISK-27793 #close
+	  Reported by: Ilya Shipitsin
+	  Tested by: Ilya Shipitsin
 
-	  gcc 7.2 warned about this.
+	  Change-Id: I0b31933b062a23331dbac9a82b8bcfe345f406f6
 
-	  ASTERISK-27559
+2018-04-04 13:12 +0000 [8de3fa2b56]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I48960dda9cf0a11b6a9426f775e632363f8caa74
+	* bridge_softmix / app_confbridge: Add support for REMB combining.
 
-2018-01-06 05:01 +0000 [e60135efd4]  Alexander Traud <pabstraud@compuserve.com>
+	  This change adds the ability for multiple REMB reports in
+	  bridge_softmix to be combined according to a configured
+	  behavior into a single report. This single report is sent
+	  back to the sender of video, which adjusts the encoding bitrate
+	  to be at or below the bitrate of the report. The available
+	  behaviors are: lowest, highest, and average. Lowest uses the
+	  lowest received bitrate. Highest uses the highest received
+	  bitrate. Average goes through the received bitrates adding
+	  them to the previous average and creates a new average.
 
-	* codec_gsm: Avoid shifting a negative signed value.
+	  Other behaviors can be added in the future and the existing
+	  average one may be adjusted, but this provides the foundation
+	  to do so.
 
-	  clang 5.0 warned about this.
+	  Support for configuring which behavior to use has been
+	  added to app_confbridge.
 
-	  ASTERISK-27558
+	  ASTERISK-27804
 
-	  Change-Id: Icc452ecb0d86bbeba78dae768cc472ec540699df
+	  Change-Id: I9eafe4e7c1f72d67074a8d6acb26bfcf19322b66
 
-2018-01-04 12:23 +0000 [ba57c03740]  Richard Mudgett <rmudgett@digium.com>
+2018-04-13 15:14 +0000 [f79a372941]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_endpoint_identifier_ip.c: Fix apply identify validation.
+	* streams: Add string metadata capability
 
-	  The ip_identify_apply() did not validate the configuration for simple
-	  static configuration errors or deal well with address resolution errors.
+	  Replaces the never used opaque data array.
 
-	  * Added missing configuration validation checks.
-	  * Fixed address resolution error handling.
-	  * Demoted an error message to a warning since it does not fail applying
-	  the identify object configuration.
+	  Updated stream tests to include get/set metadata and
+	  stream clone with metadata.
 
-	  Change-Id: I8b519607263fe88e8ce964f526a45359fd362b6e
+	  Added stream metadata dump to "core show channel"
 
-2018-01-04 17:42 +0000 [0feca9bc18]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Id7473aa4b374d7ab53046c20e321037ba9a56863
 
-	* res_pjsip.c: Fix endpoint identifier registration name search.
+2018-04-13 15:17 +0000 [f7e7ce6ba2]  George Joseph <gjoseph@digium.com>
 
-	  If an endpoint identifier name in the endpoint_identifier_order list is a
-	  prefix to the identifier we are registering, we could install it in the
-	  wrong position of the list.
+	* utils: Add ast_assert_return
 
-	  Assuming
-	  endpoint_identifier_order=username,ip,anonymous
+	  Similar to pjproject's PJ_ASSERT_RETURN macro, this one will do the
+	  following...
 
-	  then registering the "ip_only" identifier would put the identifier in the
-	  wrong position of the priority list.
+	  If the assert passes... NoOp
 
-	  * Fix incorrect strncmp() string prefix matching.
+	  If the assert fails and AST_DEVMODE is defined, execute ast_assert()
+	  then, if DO_CRASH isn't set, return from the calling function with
+	  the supplied value.
 
-	  Change-Id: Ib8819ec4b811da8a27419fd93528c54d34f01484
+	  If the assert fails and AST_DEVMODE is not defined, return from the
+	  calling function with the supplied value.
 
-2018-01-05 03:33 +0000 [440fb4d02b]  Alexander Traud <pabstraud@compuserve.com>
+	  The macro will execute a return without a value if one isn't suppled.
 
-	* BuildSystem: Find ptlib-config on Debian/Ubuntu.
+	  Change-Id: I0003844affeab550d5ff5bca7aa7cf8a559b873e
 
-	  The current configure script requires that tool when libpt-dev is installed.
-	  libpt-dev was installed by libopenh323-dev, bacause you wanted to go for H.323
-	  based channel drivers.
+2018-04-10 16:09 +0000 [8135558bab]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-25329
+	* app_sendtext:  Enhance SendText to support Enhanced Messaging
 
-	  Change-Id: I9c6ab78b7246c21536e1d252dcbffe682f63f83d
+	  SendText now accepts new channel variables that can be used
+	  to override the To and From display names and set the Content-Type
+	  of a message.  Since you can now set Content-Type, other text/*
+	  content types are now valid.
 
-2018-01-05 06:42 +0000 [dda0ab236d]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I648b4574478119f95de09d9f08e9595831b02830
 
-	* chan_ooh323: Limit outgoinglimit to positive values as intended.
+2017-09-27 11:44 +0000 [4fb7967c73]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27552
+	* bridge_softmix:  Forward TEXT frames
 
-	  Change-Id: Ifbf9d51e7374ca2e8b27ec568f6770050fc1a854
+	  Core bridging and, more specifically, bridge_softmix have been
+	  enhanced to relay received frames of type TEXT or TEXT_DATA to all
+	  participants in a softmix bridge.  res_pjsip_messaging and
+	  chan_pjsip have been enhanced to take advantage of this so when
+	  res_pjsip_messaging receives an in-dialog MESSAGE message from a
+	  user in a conference call, it's relayed to all other participants
+	  in the call.
 
-2018-01-05 06:19 +0000 [c11999f140]  Alexander Traud <pabstraud@compuserve.com>
+	  res_pjsip_messaging already queues TEXT frames to the channel when
+	  it receives an in-dialog MESSAGE from an endpoint and chan_pjsip
+	  will send an MESSAGE when it gets a TEXT frame.  On a normal
+	  point-to-point call, the frames are forwarded between the two
+	  correctly.  bridge_softmix was not though so messages weren't
+	  getting forwarded to conference bridge participants.  Even if they
+	  were, the bridging code had no way to tell the participants who
+	  sent the message so it would look like it came from the bridge
+	  itself.
 
-	* ooh323cDriver: Fix typo in header guard.
+	  * The TEXT frame type doesn't allow storage of any meta data, such
+	  as sender, on the frame so a new TEXT_DATA frame type was added that
+	  uses the new ast_msg_data structure as its payload.  A channel
+	  driver can queue a frame of that type when it receives a message
+	  from outside.  A channel driver can use it for sending messages
+	  by implementing the new send_text_data channel tech callback and
+	  setting the new AST_CHAN_TP_SEND_TEXT_DATA flag in its tech
+	  properties.  If set, the bridging/channel core will use it instead
+	  of the original send_text callback and it will get the ast_msg_data
+	  structure. Channel drivers aren't required to implement this.  Even
+	  if a TEXT_DATA enabled driver uses it for incoming messages, an
+	  outgoing channel driver that doesn't will still have it's send_text
+	  callback called with only the message text just as before.
 
-	  ASTERISK-27551
+	  * res_pjsip_messaging now creates a TEXT_DATA frame for incoming
+	  in-dialog messages and sets the "from" to the display name in the
+	  "From" header, or if that's empty, the caller id name from the
+	  channel.  This allows the chat client user to set a friendly name
+	  for the chat.
 
-	  Change-Id: I39ff66031e3373e895e2bc47b23a5e860ea4e012
+	  * bridge_softmix now forwards TEXT and TEXT_DATA frames to all
+	  participants (except the sender).
 
-2018-01-05 03:36 +0000 [cee6de4e63]  Alexander Traud <pabstraud@compuserve.com>
+	  * A new function "ast_sendtext_data" was added to channel which
+	  takes an ast_msg_data structure and calls a channel's
+	  send_text_data callback, or if that's not defined, the original
+	  send_text callback.
 
-	* BuildSystem: Avoid obsolete warning with HELP_STRING on autoconf.
+	  * bridge_channel now calls ast_sendtext_data for TEXT_DATA frame
+	  types and ast_sendtext for TEXT frame types.
 
-	  ASTERISK-26046
+	  * chan_pjsip now uses the "from" name in the ast_msg_data structure
+	  (if it exists) to set the "From" header display name on outgoing text
+	  messages.
 
-	  Change-Id: I48f05698c235f709225b92bec5aa260fb57d69d1
+	  Change-Id: Idacf5900bfd5f22ab8cd235aa56dfad090d18489
 
-2018-01-04 16:07 +0000 [0f141351f9]  Corey Farrell <git@cfware.com>
+2018-04-17 07:06 +0000 [8a1ffb050b]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pbx: Prevent execution of NULL pointer.
+	* utils/pval: Add -lBlocksRuntime for compiler clang conditionally.
 
-	  pbx_extension_helper has a check for q->swo.exec == NULL but it doesn't
-	  actually return so we would still run the function.  Fix the return.
-	  Move the 'int res' variable into the only scope which uses it.
+	  ASTERISK-27809
 
-	  Change-Id: I0693af921fdc7f56b6a72a21fb816ed08b960a69
+	  Change-Id: I930b364a33d54cc08dedfcd5bb45f7e83242f134
 
-2018-01-04 09:30 +0000 [d73a3a1764]  Corey Farrell <git@cfware.com>
+2018-04-17 05:27 +0000 [3d9345e3ae]  Alexander Traud <pabstraud@compuserve.com>
 
-	* res_pjsip_history: Add missing unlock to CLI command.
+	* chan_vpb: Avoid GNU old-style field designator extension.
 
-	  Change-Id: I872060a30543776a176a316309602d924a23eb29
+	  clang 6.0 warned about this. Beside that, this change removes the used variable
+	  'desc'.
 
-2018-01-04 09:27 +0000 [ec74570598]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27808
 
-	* aco: Fix NULL dereference in error path.
+	  Change-Id: Ia26bdcc0a562c058151814511cfcf70ecafa595b
 
-	  Change-Id: Id505167cf0f9414a3c144fa2c1e181a2cf288694
+2018-04-09 17:09 +0000 [f5d5083ea7]  Ben Ford <bford@digium.com>
 
-2018-01-03 19:07 +0000 [74b6fafbe6]  Corey Farrell <git@cfware.com>
+	* res_rtp_asterisk: Add support for receiving and handling NACK requests.
 
-	* func_odbc: Add missing unlock's to acf_odbc_read.
+	  Adds the ability to receive and handle incoming NACK requests if
+	  retransmissions are enabled. If retransmissions are enabled, a data
+	  buffer is allocated that stores packets being sent. If a NACK request
+	  is received, the packet requested for retransmission is sent if it is
+	  still in the buffer. In the same request, if any of the following 16
+	  packets are marked as not received, those will be sent as well if
+	  available, as outlined in RFC4585.
 
-	  Change-Id: I828329ecbd252ae8f27a369a046d2b03102b07c6
+	  Also changes RTCP RR and SR to use media source SSRC instead of packet
+	  source SSRC when determining which instance to use for RTCP reports.
 
-2018-01-03 10:41 +0000 [d25a9bc7d3]  Kevin Harwell <kharwell@digium.com>
+	  For more information, refer to the wiki page:
+	  https://wiki.asterisk.org/wiki/display/AST/WebRTC+User+Experience+Improvements
 
-	* res_pjsip_session: Check if sequence header is missing
+	  ASTERISK-27806 #close
 
-	  The pjsip_msg_find_hdr function can return NULL. This patch adds a check
-	  when searching for the sequence header to make sure a NULL pointer is never
-	  de-referenced.
+	  Change-Id: I7f7f124af3b9d5d2fd9cffc6ba8cb48a6fff06ec
 
-	  Change-Id: I19af23aeeded65be016be92360e8cb7ffe51fad2
+2018-04-16 16:38 +0000 [d50d637764]  Richard Mudgett <rmudgett@digium.com>
 
-2018-01-02 07:36 +0000 [00b0c67144]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* stringfields: Collect extended stringfields into the stringfield section.
 
-	* cdr: submit: fix logic of test for batch mode
+	  Use of extended stringfields is a temporary mechanism to avoid ABI
+	  breakage in released branches without resorting to more inconvienient
+	  methods.
 
-	  ASTERISK-27539 #close
+	  * Collect existing extended stringfields into the parent stringfield
+	  section of the struct.
 
-	  Change-Id: I33cdf329d2bb4486dcae975c450f6aae94c515f7
+	  Change-Id: I8d46d037801b4518837c3ea4b6df95ceadc9436b
 
-2018-01-02 00:26 +0000 [dd528c53c0]  Corey Farrell <git@cfware.com>
+2018-04-13 14:32 +0000 [4aeec6100f]  Ben Ford <bford@digium.com>
 
-	* aco: Add missing aco_option_type_string for OPT_TIMELEN_T.
+	* res_musiconhold: Don't restart MOH from beginning after announcement.
 
-	  ASTERISK-27117
+	  This reverts a problem introduced by the fix for ASTERISK_24329.
+	  Now, when an announcement is played while waiting in a queue, music on
+	  hold will not restart from the beginning of the sound file and will
+	  instead pick up where it left off. However, the incorrect behavior in
+	  ASTERISK_24329 is now present again; if an announcement X seconds
+	  long is played when music on hold starts, music on hold will start X
+	  seconds into the file.
 
-	  Change-Id: I8f6c34bb30830be9f7a40823723eb4dcaaa91c61
+	  ASTERISK-27774 #close
+	  Reported by: lvl
 
-2017-12-29 22:59 +0000 [5b395a7b97]  Corey Farrell <git@cfware.com>
+	  Change-Id: I86b2885ee7063268f9b9747eddb788336ade989b
 
-	* core: Use macros to generate ao2_container callbacks where possible.
+2018-03-28 15:13 +0000 [3bb6cf43b5]  Richard Mudgett <rmudgett@digium.com>
 
-	  This uses AO2_STRING_FIELD_HASH_FN and AO2_STRING_FIELD_CMP_FN where
-	  possible in the Asterisk core.
+	* pjsip_scheduler.c: Add ability to trace scheduled tasks.
 
-	  This removes CMP_STOP from the result of CMP_FN callbacks for the
-	  following structure types:
-	  * ast_bucket_metadata
-	  * ast_bucket_scheme
-	  * generic_monitor_instance_list (ccss.c)
-	  * named_acl
+	  When a scheduled task is created you can pass in the
+	  AST_SIP_SCHED_TASK_TRACK flag.  This new flag causes scheduling events to
+	  be logged.
 
-	  Change-Id: Ide4c1449a894bce70dea1fef664dade9b57578f1
+	  Change-Id: I91967eb3d5a220915ce86881a28af772f9a7f56b
 
-2017-12-31 10:26 +0000 [d2c836d24a]  Sean Bright <sean.bright@gmail.com>
+2018-03-27 11:04 +0000 [237d341bbd]  Richard Mudgett <rmudgett@digium.com>
 
-	* ice: Increase foundation buffer size
+	* res_pjsip.c: Split ast_sip_push_task_synchronous() to fit expectations.
 
-	  Per RFC 5245, the foundation specified with an ICE candidate can be up
-	  to 32 characters but we are only allowing for 31.
+	  ast_sip_push_task_synchronous() did not necessarily execute the passed in
+	  task under the specified serializer.  If the current thread is any
+	  registered pjsip thread then it would execute the task immediately instead
+	  of under the specified serializer.  Reentrancy issues could result if the
+	  task does not execute with the right serializer.
 
-	  ASTERISK-27498 #close
-	  Reported by: Michele Prà
+	  The original reason ast_sip_push_task_synchronous() checked to see if the
+	  current thread was a registered pjsip thread was because of a deadlock
+	  with masquerades and the channel technology's fixup callback
+	  (ASTERISK_22936).  A subsequent masquerade deadlock fix (ASTERISK_24356)
+	  involving call pickups avoided the original deadlock situation entirely.
+	  The PJSIP channel technology's fixup callback no longer needed to call
+	  ast_sip_push_task_synchronous().
 
-	  Change-Id: I05ce7a5952721a76a2b4c90366168022558dc7cf
+	  However, there are a few places where this unexpected behavior is still
+	  required to avoid deadlocks.  The pjsip monitor thread executes callbacks
+	  that do calls to ast_sip_push_task_synchronous() that would deadlock if
+	  the task were actually pushed to the specified serializer.  I ran into one
+	  dealing with the pubsub subscriptions where an ao2 destructor called
+	  ast_sip_push_task_synchronous().
 
-2017-12-29 22:03 +0000 [b275b0a84f]  Corey Farrell <git@cfware.com>
+	  * Split ast_sip_push_task_synchronous() into
+	  ast_sip_push_task_wait_servant() and ast_sip_push_task_wait_serializer().
+	  ast_sip_push_task_wait_servant() has the old behavior of
+	  ast_sip_push_task_synchronous().  ast_sip_push_task_wait_serializer() has
+	  the new behavior where the task is always executed by the specified
+	  serializer or a picked serializer if one is not passed in.  Both functions
+	  behave the same if the current thread is not a SIP servant.
 
-	* astobj2: Create case-insensitive variants of container function macros.
+	  * Redirected ast_sip_push_task_synchronous() to
+	  ast_sip_push_task_wait_servant() to preserve API for released branches.
 
-	  * AO2_STRING_FIELD_CASE_HASH_FN
-	  * AO2_STRING_FIELD_CASE_CMP_FN
-	  * AO2_STRING_FIELD_CASE_SORT_FN
+	  ASTERISK_26806
 
-	  Change-Id: I11af8c6a0c43380a42732553f519c667abb842cf
+	  Change-Id: Id040fa42c0e5972f4c8deef380921461d213b9f3
 
-2017-12-28 13:27 +0000 [751fd9b628]  Richard Mudgett <rmudgett@digium.com>
+2018-03-21 19:43 +0000 [c2f85e881d]  Richard Mudgett <rmudgett@digium.com>
 
-	* stasis_channels.c: Misc cleanup.
+	* pjsip_scheduler.c: Fix some corner cases.
 
-	  * Use current OBJ_SEARCH_xxx defines instead of the deprecated versions.
+	  * Fix the periodic interval wander because it may take significant time
+	  between the sched thread queueing the task in the serializer and the
+	  serializer actually executing the task.  The time it takes to actually
+	  execute the task was already taken into account.
 
-	  * Fix hash_cb and cmp_cb container functions to correctly use the
-	  OBJ_SEARCH_xxx values.
+	  * Pass a schtd ref to the serializer when we queue a scheduled task on
+	  the serializer.  We don't want it going away on us while it is in the
+	  serializer queue.
 
-	  * Remove incorrect usage of CMP_STOP.  Most uses in the system have no
-	  effect.  This allows the collapse of channel_role_single_cmp_cb() and
-	  channel_role_multi_cmp_cb() into channel_role_cmp_cb().
+	  * Skip the scheduled task if the task was canceled between queueing the
+	  task to the serializer and the serializer actually executing the task.
 
-	  * Remove unnecessary usage of RAII_VAR().
+	  * Reorder struct ast_sip_sched_task to avoid unnecessary padding.  Removed
+	  task_id and added next_periodic.
 
-	  Change-Id: I02c405518cab22aa2a082b61e2353bf7cd629a70
+	  * Hold a ref to the passed in serializer so the serializer cannot go away
+	  on the scheduled task.
 
-2017-12-13 15:43 +0000 [bae301c18c]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK_26806
 
-	* cdr_mysql: Make sure connection charset is always set
+	  Change-Id: I6c8046b75f6953792c8c30e55b836a4291143f24
 
-	  When the MYSQL_OPT_RECONNECT option is enabled, the MySQL client API
-	  will transparently reconnect when it needs to. Ideally this simplifies
-	  our code, but when this reconnection occurs all connection state is
-	  lost. Because we are not notified that this has happened, we don't know
-	  to set our character set again (with "SET NAMES 'xyz'").
+2018-03-22 19:09 +0000 [96c4a57edf]  Richard Mudgett <rmudgett@digium.com>
 
-	  Rather than calling SET NAMES, we instead set the MYSQL_SET_CHARSET_NAME
-	  option which will do it for us under the hood on each connect. This
-	  option has been present in the MySQL C API for at least 15 years, so it
-	  should be safe for most installations.
+	* pjsip_scheduler.c: Sort "pjsip show scheduled_tasks" output.
 
-	  I also snuck a few other changes into this patch:
+	  * A side benefit is that the scheduled tasks are not completely blocked
+	  while the CLI command executes.
 
-	  * Default the MySQL port to MYSQL_PORT (3306) instead of 0 if it's not
-	    defined.
+	  * Adjusted the "Task Name" column width to have more room for longer
+	  names.
 
-	  * Fix some erroneous and/or silly checks on the contents of the
-	    configuration ast_str values.
+	  Change-Id: Iec64aa463ee8b10eef90120e00c38b1fb444087e
 
-	  ASTERISK-27366 #close
-	  Reported by: Halil İbrahim YILDIZ
+2018-04-02 15:59 +0000 [429c758e48]  Evandro Cesar Arruda <ecarruda@gmail.com>
 
-	  Change-Id: I36bf8dc5d5f83584e803b3b1a151dea9396ab8f5
+	* cdr_mysql: Compile error because MYSQL_PORT definition is missing
 
-2017-12-27 20:48 +0000 [5de8f49294]  Richard Mudgett <rmudgett@digium.com>
+	  If it is not defined, it will add MYSQL_PORT definition. After some
+	  research on MySQL/MariaDB development tree, I couldn't find any reference
+	  to MYSQL_PORT definition in include files.
 
-	* manager.c: Update AMI Status event documentation
+	  ASTERISK-27782 #close
 
-	  The AMI Status event had linkedid listed twice and was missing the
-	  effective connected line name and number headers.
+	  Change-Id: Ieee56c836fc2e8bd021c456145bba04c6068bb77
 
-	  NOTE: The linkedid and other standard channel snapshot fields in the XML
-	  documentation are part of the <channel_snapshot/> XML template defined in
-	  doc/appdocsxml.xslt.
+2018-04-09 20:00 +0000 [0747ac893b]  Chris-Savinovich <csavinovich@digium.com>
 
-	  * Cached the effective connected line party id so it doesn't get
-	  calculated four times.
+	* res_pjsip_session: Rewrite o= with external_media_address.
 
-	  Change-Id: I004c4c4f9e7b40ef55035c831702721bec82496c
+	  It now appends the external IP address on the
+	  o= line of the SDP packet.  The decision was made to write
+	  the numeric IP address as opposed to the RFC that states
+	  the FQDN should be used if and when available.  We believe
+	  the usage of literal IP address will help avoid
+	  potential problems.
 
-2017-12-27 22:36 +0000 [9f1cfbafca]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27614 #close
 
-	* bridge_native_rtp.c: Fix reentrancy framehook crash.
+	  Change-Id: I84f3360f3606b8c4e8d161edb228799ec0b8a302
 
-	  If two channels enter different native rtp bridges at the same time it is
-	  possible that the framehook interface data pointer can be corrupted
-	  because the struct variable was declared static.
+2018-02-22 12:18 +0000 [1cd704de36]  Nathan Bruning <nathan@iperity.com>
 
-	  * Fixed the reentrancy corruption by changing the framehook interface
-	  struct static variable to a stack local variable.
+	* res_pjsip_notify.c: enable in-dialog NOTIFY
 
-	  * Moved the hook.data assignment outside of the channel lock.  It did not
-	  need the lock's protection.  It probably was giving a false sense of
-	  security.
+	  This patch adds support to send in-dialog SIP NOTIFY commands on
+	  chan_pjsip channels, similar to the functionality recently added
+	  for chan_sip (ASTERISK_27461).
 
-	  The testsuite
-	  channels/pjsip/basic_calls/two_parties/nominal/alice_initiated/bob_hangs_up
-	  test caught this with MALLOC_DEBUG and DO_CRASH enabled.
+	  This extends res_pjsip_notify to allow for in-dialog messages.
 
-	  Change-Id: If9e35b97d19209b0f984941c1d8eb5f7c55eea91
+	  ASTERISK-27697
 
-2017-12-27 20:22 +0000 [eead5d0c30]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: If7f3151a6d633e414d5dc319d5efc1443c43dd29
 
-	* func_channel.c: Update MASTER_CHANNEL documentation
+2018-03-22 13:35 +0000 [7157dcf83b]  Richard Mudgett <rmudgett@digium.com>
 
-	  Be more explicit in what is meant by the master channel to eliminate
-	  misunderstanding.
+	* pjsip_scheduler.c: Fix ao2 usage errors.
 
-	  ASTERISK-23133
+	  * Removed several invalid uses of OBJ_NOLOCK.  These uses resulted in the
+	  'tasks' container being accessed without a lock in a multi-threaded
+	  environment.  A recipe for crashes.
 
-	  Change-Id: I453bcaf4b99404a5a3e345dbf093ac6c1afcfc72
+	  * Removed needlessly obtaining schtd object references.  If the caller
+	  providing you a pointer to an object doesn't have a valid reference then
+	  you cannot safely get one from it.
 
-2017-12-27 19:27 +0000 [0bdddbe526]  Corey Farrell <git@cfware.com>
+	  * Getting a ref to 'tasks' when you aren't copying the pointer into
+	  another location is useless.  The 'tasks' container pointer is global.
 
-	* menuselect: Fix check for running configure.
+	  * Removed many unnecessary uses of RAII_VAR.
 
-	  menuselect/Makefile checks that autoconfig.h and makeopts were newer
-	  than the '.in' files.  Unfortunately running ./configure does not touch
-	  autoconfig.h unless the contents will change.
+	  * Make ast_sip_schedule_task() name parameter const.
 
-	  Instead of looking at autoconfig.h we just need to ensure that makeopts
-	  is newer than configure.
+	  ASTERISK_26806
 
-	  Also make change to configure.ac so bootstrap.sh doesn't re-add the
-	  extra trailing line-feed.
+	  Change-Id: I5c62488e651314e2a1dbc01f5b078a15512d73db
 
-	  Change-Id: Ief1f831d6717007f9cebb668c14e92782cd2b794
+2018-03-23 06:49 +0000 [879e592baf]  Corey Farrell <git@cfware.com>
 
-2017-12-25 20:39 +0000 [d62c87bb8d]  Corey Farrell <git@cfware.com>
+	* Build System: Enable python3 compatibility.
 
-	* cdr: Missing NULL check and unlock.
+	  * Consistently use spaces in rest-api-templates/asterisk_processor.py.
+	  * Exclude third-party from docs/full-en_US.xml.
+	  * Add docs/full-en_US.xml to .gitignore.
+	  * Use list() to convert python3 view.
+	  * Use python3 print function.
+	  * Replace cmp() with equivalent equation.
+	  * Replace reference to out of scope subtype variable with name
+	    parameter.
+	  * Use unescaping triple bracket notation in mustache templates where
+	    needed.  This causes behavior of Python2 to be maintained when using
+	    Python3.
+	  * Fix references to has_websocket / is_websocket in
+	    res_ari_resource.c.mustache.
+	  * Update calculation of has_websocket to use any().
+	  * Use unicode mode for writing output file in transform.py.
+	  * Replace 'from swagger_model import *' with explicit import of required
+	    symbols.
 
-	  * handle_dial_message: Missing a check for NULL peer.
-	  * ast_cdr_register: Missing unlock on allocation failure.
+	  I have not tested spandspflow2pcap.py or voicemailpwcheck.py, only the
+	  print syntax has been fixed.
 
-	  ast_cdr_register is fixed by reordering so the new structure is
-	  allocated and initialized before locking the list.
+	  Change-Id: If5c5b556a2800d41a3e2cfef080ac2e151178c33
 
-	  Change-Id: I5799b99270d1a7a716a555c31ac85f4b00ce8686
+2018-04-05 18:33 +0000 [0c03eab962]  Richard Mudgett <rmudgett@digium.com>
 
-2017-12-23 22:51 +0000 [92fb393cab]  Corey Farrell <git@cfware.com>
+	* res_pjsip_refer/chan_sip: Fix INVITE with replaces transfer to ConfBridge
 
-	* loader: Add volatile to resource_being_loaded.
+	  There is a problem when an INVITE-with-Replaces transfer targets a channel
+	  in a ConfBridge.  The transfer will unconditionally swap out the
+	  ConfBridge channel.  Unfortunately, the ConfBridge state will not be aware
+	  of this change.  Unexpected behavior will happen as a result since
+	  ConfBridge channels currently can only be replaced by a masquerade and not
+	  normal bridge channel moves.
 
-	  Some compiler optimizers seem to assume that dlopen will not use
-	  __attribute__((constructor)) functions to call back to the program.
-	  This was causing resource_being_loaded to be optimized away completely.
+	  * We just need to pretend that the channel isn't in a bridge (like other
+	  transfer methods already do) so the transfer channel will masquerade into
+	  the ConfBridge channel.
 
-	  ASTERISK-27531 #close
-	  Tested By: abelbeck
+	  Change-Id: I209beb0e748fa4f4b92a576f36afa8f495ba4c82
 
-	  Change-Id: If17a3b889e06811a0e7119f0539d052494d6ece9
+2018-03-28 07:27 +0000 [c7bd554094]  Joshua Colp <jcolp@digium.com>
 
-2017-12-20 16:17 +0000 [53799318bc]  Kevin Harwell <kharwell@digium.com>
+	* pjsip / res_rtp_asterisk: Add support for sending REMB
 
-	* AST-2017-014: res_pjsip - Missing contact header can cause crash
+	  This change allows chan_pjsip to be given an AST_FRAME_RTCP
+	  containing REMB feedback and pass it to res_rtp_asterisk.
+	  Once res_rtp_asterisk receives the frame a REMB RTCP feedback
+	  packet is constructed with the appropriate contents and sent
+	  to the remote endpoint.
 
-	  Those SIP messages that create dialogs require a contact header to be present.
-	  If the contact header was missing from the message it could cause Asterisk to
-	  crash.
+	  ASTERISK-27776
 
-	  This patch checks to make sure SIP messages that create a dialog contain the
-	  contact header. If the message does not and it is required Asterisk now returns
-	  a "400 Missing Contact header" response. Also added NULL checks when retrieving
-	  the contact header that were missing as a "just in case".
+	  Change-Id: Ic53f821c1560d8924907ad82c4d9c0bc322b38cd
 
-	  ASTERISK-27480 #close
+2018-04-05 20:02 +0000 [39016e3582]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I1810db87683fc637a9e3e1384a746037fec20afe
+	* res_rtp_asterisk: Fix minimum block word length for REMB.
 
-2017-12-22 08:14 +0000 [ce3d56920b]  Sean Bright <sean.bright@gmail.com>
+	  The minimum block word length is actually 4, not 5.
 
-	* Remove as much trailing whitespace as possible.
+	  Change-Id: I878542218225aed72c72bdf1b856fc822cd2d649
 
-	  Change-Id: I873c1c6d00f447269bd841494459efccdd2c19c0
+2018-04-05 18:48 +0000 [8a602f18db]  Joshua Colp <jcolp@digium.com>
 
-2017-12-21 09:51 +0000 [b4ae112e3a]  Sean Bright <sean.bright@gmail.com>
+	* res_rtp_asterisk: Queue video update on picture loss indication.
 
-	* Fix some invalid Unicode characters
+	  The previous payload specific feedback handling was very single
+	  minded in that it just assumed everything should trigger a video
+	  update. This was changed but the handling of picture loss indication
+	  was not added. The result was that video may not flow. This change
+	  adds it explicitly in.
 
-	  configs/samples/minivm.conf.sample contains invalid UTF-8, but that
-	  appears to be intentional.
+	  Change-Id: I1894be02e39ee10a0af841b5a1dca5f0ec7d60b6
 
-	  Change-Id: I7b1e0d332f3380fd0425962a3c9c55f9b200c8cc
+2018-04-05 17:40 +0000 [d72a2966da]  Richard Mudgett <rmudgett@digium.com>
 
-2017-12-20 21:11 +0000 [719e8eee03]  Corey Farrell <git@cfware.com>
+	* chan_sip.c: Fix INVITE with replaces channel ref leak.
 
-	* app_voicemail: Fix file copy error handling.
+	  Given the below call scenario:
+	  A -> Ast1 -> B
+	  C <- Ast2 <- B
 
-	  Fix error where input/output file descriptors would be closed multiple
-	  times.
+	  1) A calls B through Ast1
+	  2) B calls C through Ast2
+	  3) B transfers A to C
 
-	  Change-Id: Iba5140b60cb7de79e3d5d92be3c256947aa99da9
+	  When party B transfers A to C, B sends a REFER to Ast1 causing Ast1 to
+	  send an INVITE with replaces to Ast2.  Ast2 then leaks a channel ref of
+	  the channel between Ast1 and Ast2.
 
-2017-12-20 14:54 +0000 [6892c13a2c]  Sean Bright <sean.bright@gmail.com>
+	  Channel ref leaks are easily seen in the CLI "core show channels" output.
+	  The leaked channels appear in the output but you can do nothing with them
+	  and they never go away unless you restart Asterisk.
 
-	* docs: Remove old API changes documentation
+	  * Properly account for the channel refs when imparting a channel into a
+	  bridge when handling an INVITE with replaces in handle_invite_replaces().
+	  The ast_bridge_impart() function steals a channel ref but the code didn't
+	  account for how many refs were held by the code at the time and which ref
+	  was stolen.
 
-	  Change-Id: I1bc7957121cc7ae27dca04acc3613f4e1858476a
+	  * Eliminated RAII_VAR in handle_invite_replaces().
 
-2017-12-20 11:14 +0000 [82b6ba976f]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27740
 
-	* Fix Common Typo's.
+	  Change-Id: I7edbed774314b55acf0067b2762bfe984ecaa9a4
 
-	  Fix instances of:
-	  * Retreive
-	  * Recieve
-	  * other then
-	  * different then
-	  * Repeated words ("the the", "an an", "and and", etc).
-	  * othterwise, teh
+2018-03-21 19:40 +0000 [71a67a98c4]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-24198 #close
+	* res_pjsip: Update authenticate_qualify documentation.
 
-	  Change-Id: I3809a9c113b92fd9d0d9f9bac98e9c66dc8b2d31
+	  Change-Id: I3811de0014b1ffe96d4a3b49cddd5d4ca02ee5d4
 
-2017-12-20 00:53 +0000 [6b3188fb8c]  Corey Farrell <git@cfware.com>
+2018-04-02 16:49 +0000 [6774913e82]  Richard Mudgett <rmudgett@digium.com>
 
-	* app_festival: Fix fd leak on connection failure.
+	* app_agent_pool.c: Fix off nominal ref leak.
 
-	  Change-Id: If5efaddcf735ff7d17e55c36cc1388946cee9e0f
+	  Change-Id: Ib427ffc2c802620eaafb08b1c2a17dddd8fb8eb6
 
-2017-12-20 10:23 +0000 [be3800c49d]  Corey Farrell <git@cfware.com>
+2018-04-04 10:02 +0000 [e40fd7a232]  Corey Farrell <git@cfware.com>
 
-	* bridge: Old channel video source not set to NULL after unref.
+	* Build System: Strip '-std=c99' from CFLAGS provided by libraries.
 
-	  The bridge holds onto the old channel video source after it's been
-	  released.  This can lead to use after free errors.
+	  Asterisk requires GNU C extensions.  On some systems certain libraries
+	  may incorrectly push -std=c99 into CFLAGS, thus breaking the build.
+	  This change causes that flag to be stripped so the Asterisk build is not
+	  broken by those libraries.  This change is made for both pkgconfig and
+	  tool based libraries.
 
-	  ASTERISK-27229 #close
+	  ASTERISK-27629 #close
 
-	  Change-Id: Ib2dab61677dd8a21f7ad53cdc9b8ca93297838b3
+	  Change-Id: I13389613b194abbac77becf90cd950dc168704db
 
-2017-12-20 10:13 +0000 [d05ec48145]  Corey Farrell <git@cfware.com>
+2018-04-03 14:39 +0000 [66f13ed694]  Corey Farrell <git@cfware.com>
 
-	* core: Fix unused variable error in handle_show_sysinfo.
+	* Build System: Fixes for configure script.
 
-	  Apparently in OSX it's possible for OSX to HAVE_SYSCTL but not
-	  HAVE_SYSINFO or HAVE_SWAPCTL.  In this case freeswap caused an unused
-	  variable error.
+	  * Replace all 'else if' statements with 'elif'.
+	  * Use loop to detect versioned lua headers and libraries.
 
-	  ASTERISK-26563 #close
+	  The loop for detecting lua fixes a bug where LUA_INCLUDE would be
+	  appended with the directory of every lua version after the first one is
+	  found.
 
-	  Change-Id: I8ec5b1897b786cc1abaf62264aa75039eea05510
+	  Change-Id: I3276f9aee955014108345be6092f51c932b43a0f
 
-2017-10-16 07:36 +0000 [e7f8302d30]  Torrey Searle <torrey@voxbone.com>
+2018-04-02 08:53 +0000 [0f6431e8e4]  Joshua Colp <jcolp@digium.com>
 
-	* contrib/script/sip_to_pjsip: add support for realtime
+	* app_confbridge / bridge_softmix: Add ability to configure REMB interval.
 
-	  Add a new script that can read from legacy realtime peers & generate
-	  an sql file for populating pjsip endpoints, identify, and aor records.
+	  This change adds a configuration option to app_confbridge which can be
+	  used to set the interval at which we will send a combined REMB (remote
+	  estimated maximum bitrate) frame to sources of video. The bridging API
+	  has also been extended slightly to allow setting this so bridge_softmix
+	  can use it.
 
-	  ASTERISK-27348 #close
+	  ASTERISK-27786
 
-	  Change-Id: Idd3d7968a3c9c3ee7936d21acbdaf001b429bf65
+	  Change-Id: I0e49eae60f369c86434414f3cb8278709c793c82
 
-2017-12-20 19:05 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2018-01-02 07:54 +0000 [f91263cf46]  George Joseph <gjoseph@digium.com>
 
-	* asterisk 13.19.0-rc1 Released.
+	* res_pjsip:  Correct usages of pjproject's timer heap
 
-2017-12-20 11:30 +0000 [4f45748f52]  Richard Mudgett <rmudgett@digium.com>
+	  Fix some timer heap initializations and cancels to try and prevent
+	  crashes and timer heap issues.
 
-	* manager.h: Bump AMI version
+	  Change-Id: I64885d190fa22097d1b55987091375541e57a7ee
 
-	  Change-Id: I62e6ddeb261ef012687e1fb6734c554e2499b6bf
+2018-03-25 13:35 +0000 [48720e7def]  George Joseph <gjoseph@digium.com>
 
-2017-12-18 20:12 +0000 [9adffca9c7]  Corey Farrell <git@cfware.com>
+	* pjroject_bundled:  Add already-destroyed check to tsx_timer_callback
 
-	* CLI: Address multiple issues.
+	  There have been cases that when the transaction timer callback is called
+	  the tsx is already destroyed.  This causes a crash.  We now check the
+	  tsx state and return if the tsx is already destroyed.
 
-	  * listen uses the variable `s` for the result from ast_poll() then
-	    overwrites it with the result of accept().  Create a separate variable
-	    poll_result to avoid confusion since ast_poll does not return a file
-	    descriptor.
-	  * Resolve fd leak that would occur if setsockopt failed in listen.
-	  * Reserve an extra byte while processing completion results from remote
-	    daemon.  This fixes a bug where completion processing used strstr() on
-	    a string that was not '\0' terminated.  This was no risk to the Asterisk
-	    daemon, the bug was only reachable the remote console process.
-	  * Resolve leak in handle_showchan when the channel is not found.
-	  * Multiple leaks and a deadlock in pbx_config CLI completion.
-	  * Fix leaks in "manager show command".
+	  Change-Id: If93acd5e48d9ca5bb553f2405d5afc836842fe1c
 
-	  Change-Id: I8f633ceb1714867ae30ef4e421858f77c14485a9
+2018-03-25 13:25 +0000 [7c03b2713e]  George Joseph <gjoseph@digium.com>
 
-2017-12-19 14:15 +0000 [c38b750810]  Aaron An <anjb@ti-net.com.cn>
+	* pjproject_bundled: timer: Clean up usage of timer heap
 
-	* res_rtp_asterisk:  Avoid close the rtp/rtcp fd twice.
+	  Added a new pj_timer_entry_reset function that resets a timer_entry
+	  for re-use.
 
-	  When RTCP-MUX enabled. rtp->s is the same as rtcp->s, check this before
-	  close the file descriptor. Close the FD twice will hangs the asterisk
-	  under heavy load.
+	  Changed direct settings of timer_entry fields to use
+	  pj_timer_entry_init and pj_timer_entry_reset.
 
-	  ASTERISK-27299 #close
-	  Reported-by: Aaron An
-	  Tested-by: AaronAn
+	  Fixed issues where timers were being rescheduled incorrectly.
 
-	  Change-Id: I870a072d73fd207463ac116ef97100addbc0820a
+	  Change-Id: I5b624bfbc5c1429117484b9b24567293002148e6
 
-2017-12-16 07:51 +0000 [fef23297b7]  Ivan Poddubny <ivan.poddubny@gmail.com>
+2018-03-29 17:07 +0000 [97cc67b12f]  Richard Mudgett <rmudgett@digium.com>
 
-	* bridge: Stop music on hold on adding an arbitrary channel to a bridge
+	* res_pjsip: Fix deadlock on reliable transport shutdown.
 
-	  When a channel that is on hold gets added to a bridge by
-	  the Bridge AMI action or the dialplan application of the same name,
-	  music continues to play, causing "robotic sound".
+	  A deadlock can happen when the PJSIP monitor thread is shutting down a
+	  connection oriented transport (TCP/TLS) used by a subscription at the same
+	  time as another thread tries to send something for that subscription.  The
+	  deadlock is between the pjsip monitor thread attempting to get the dialog
+	  lock and another thread sending something for that dialog when it tries to
+	  get the transport manager lock.
 
-	  This commit adds a call to ast_moh_stop to stop the music.
-	  Also, it makes the AMI Park action use the right MOH class when the
-	  channel gets parked.
+	  * res_pjsip_pubsub.c: Avoid the deadlock by pushing the subscription
+	  removal to the subscription serializer.
 
-	  Reported by: Zane Conkle
+	  * res_pjsip_registrar.c: Pushed off incoming registration contact removals
+	  to a default serializer as a precaution.  Removing the contacts involves
+	  sorcery access which in this case will involve database access.  Depending
+	  upon the setup, the database may not be on the same machine and could take
+	  awhile.  We don't want to hold up the pjsip monitor thread with
+	  potentially long access times.
 
-	  ASTERISK-25079 #close
+	  ASTERISK-27706
 
-	  Change-Id: I4b129c5a20c15e63968842460ac5a1a85903cf9f
+	  Change-Id: I56b647aea565f24dba33e9e5ebeed4cd3f31f8c4
 
-2017-12-18 18:59 +0000 [0e5d8ad09b]  Corey Farrell <git@cfware.com>
+2018-03-07 06:15 +0000 [f65488f546]  Ross Beer <ross.beer@voicehost.co.uk>
 
-	* chan_sip: Fix memory leaks.
+	* pjsip_transport_events.c: Fix crash using stale transport pointer.
 
-	  In change_redirecting_information variables we use ast_strlen_zero to
-	  see if a value should be saved.  In the case where the value is not NULL
-	  but is a zero length string we leaked.
+	  Apparently it is possible for the transport to be destroyed without
+	  triggering the transport callback logic.  As a result the transport gets
+	  destroyed and we have a stale pointer in the active_transports container.
 
-	  handle_response_subscribe leaked a reference to the ccss monitor
-	  instance.
+	  * Invoke the transport monitor callback checks when the transport is
+	  destroyed in addition to when it is disconnected and shutdown.
 
-	  Change-Id: Ib11444de69c3d5b2360a88ba2feb54d2c2e9f05f
+	  ASTERISK-27688
 
-2017-12-19 02:50 +0000 [fc86e58a5a]  Oron Peled <oron.peled@xorcom.com>
+	  Change-Id: Ia9b5469fea8f2b3f2d8476fae6b748a4d23e7261
 
-	* chan_console: Use correct parameter for 'set active'
+2018-03-19 09:36 +0000 [879743ab8f]  Ben Ford <bford@digium.com>
 
-	  chan_console supports multiple devices but the CLI only works on a
-	  single device. 'console set active' selects this device.
+	* test_data_buffer.c: Add unit tests for data buffer API.
 
-	  Sadly that CLI picks the wrong command-line parameter and will only
-	  work for a device called 'active'.
+	  Added unit tests for the data buffer API. These tests include creating a
+	  data buffer, putting payloads into the buffer, resizing the buffer, and
+	  the nominal case for data buffer usage, which consists of adding
+	  the max number of payloads to the buffer, checking to see if the correct
+	  payloads are present, then adding more payloads and checking again to
+	  see if the previous payloads were replaced or not.
 
-	  ASTERISK-27490 #close
+	  For more information, refer to the wiki page:
+	  https://wiki.asterisk.org/wiki/display/AST/WebRTC+User+Experience+Improvements
 
-	  Change-Id: I2f0e5fe63db19845bee862575b739360797dc73d
+	  Change-Id: Id5b599aa15a5e61d0ec080f97cd0c57bd07e6f8f
 
-2017-12-18 15:36 +0000 [d6b2f457d9]  Corey Farrell <git@cfware.com>
+2018-02-23 13:49 +0000 [138e0eff4e]  Ben Ford <bford@digium.com>
 
-	* Remove constant conditionals (dead-code).
+	* Add data buffer API to store packets.
 
-	  Some variables are set and never changed, making them constant.  This
-	  means that code in the 'false' block of the conditional is unreachable.
+	  Adds a data buffer with a configurable size that can store different
+	  kinds of packets (like RTP packets for retransmission). Given a number
+	  it will store a data packet at that position relative to the others.
+	  Given a number it will retrieve the given data packet if it is present.
+	  This is purposely a storage of arbitrary things so it can be used not
+	  just for RTP packets but also Asterisk frames in the future if needed.
+	  The API does not internally use a lock, so it will be up to the user of
+	  the API to properly protect the data buffer.
 
-	  In chan_skinny and res_config_ldap I used preprocessor directive `#if 0`
-	  as I'm unsure if the unreachable code could be enabled in the future.
+	  For more information, refer to the wiki page:
+	  https://wiki.asterisk.org/wiki/display/AST/WebRTC+User+Experience+Improvements
 
-	  Change-Id: I62e2aac353d739fb3c983cf768933120f5fba059
+	  Change-Id: Iff13c5d4795d52356959fe2a57360cd57dfade07
 
-2017-12-18 23:17 +0000 [381ed4f1cc]  Corey Farrell <git@cfware.com>
+2018-03-25 13:12 +0000 [a87141ddfd]  George Joseph <gjoseph@digium.com>
 
-	* core: Fix multiple trivial issues in the core.
+	* pjproject_bundled:  Add patch for pj_atomic crashes
 
-	  * Fix small leaks in from error condition in translate.c.
-	  * Check new file descriptor is less than 0, not less than or equal.
+	  There have been some crashes in the past where something attempts
+	  to use a pj_atomic after it's already been destroyed.  This patch
+	  tries to prevent it by making sure that pj_atomic_destroy sets
+	  its mutex to NULL when it's done.  The pj_mutex functions already check
+	  for a NULL mutex and just return PJ_EINVAL.
 
-	  Change-Id: Id7782775486175c739e0c4bf3ea5e17e3f452a99
+	  Teluu also added some checks to the win32 implementation as well.
 
-2017-12-18 19:47 +0000 [67b5a4e616]  Corey Farrell <git@cfware.com>
+	  Change-Id: Id25f70b79fdedf44ead6e6e1763a4417d3b3f825
 
-	* main/app: Fix leaks.
+2018-03-21 08:52 +0000 [e14b0e960d]  Joshua Colp <jcolp@digium.com>
 
-	  * ast_linear_stream would leak a file descriptor if it failed to allocate
-	    lin.
-	  * ast_control_tone leaked zone and ts if ast_playtones_start failed.
+	* res_rtp_asterisk: Add support for raising additional RTCP messages.
 
-	  Additionally added whitespace to ast_linear_stream, pulled assignments
-	  out of conditionals for improved readability.
+	  This change extends the existing AST_FRAME_RTCP frame type to be
+	  able to contain additional RTCP message types, such as feedback
+	  messages. The payload type is contained in the subclass which allows
+	  knowing what is in the frame itself.
 
-	  Change-Id: I6d1a10cf9161b1529d939b9b2d63ea36d395b657
+	  The RTCP feedback message type is now handled and REMB[1] messages
+	  are raised with their containing information.
 
-2017-12-18 19:19 +0000 [3782230e00]  Corey Farrell <git@cfware.com>
+	  This also fixes a bug where all feedback messages were triggering
+	  video updates instead of just FIR and FUR.
 
-	* func_callerid: Initialize app argument structures.
+	  Finally RTCP frames are now passed up through the Asterisk core to
+	  what is handling the channel, mapped appropriately in the case of
+	  bridging, and written to an outgoing stream. Since RTCP frames are
+	  on a per-stream basis this is only done on multistream capable
+	  channels.
 
-	  This module uses AST_DEFINE_APP_ARGS_TYPE to define struct's instead of
-	  directly using AST_DECLARE_APP_ARGS.  Initialize the variables declared
-	  in this way.
+	  [1] https://tools.ietf.org/html/draft-alvestrand-rmcat-remb-03
 
-	  Change-Id: If97fbdd8d63a204e2efd498a192effc14e90fb31
+	  ASTERISK-27758
+	  ASTERISK-26366
 
-2017-12-18 18:04 +0000 [1d636f4afa]  Corey Farrell <git@cfware.com>
+	  Change-Id: I680da0ad8d5059d5e9655d896fb9d92e9da8491e
 
-	* app_voicemail: Fix memory management issues.
+2018-03-27 08:27 +0000 [455cee99ae]  Florian Floimair <f.floimair@commend.com>
 
-	  * mwi_sub_event_cb: mwist leaked on separate_mailbox failure.
-	  * add_email_attachment: A reference to sox_gain_tmpdir was used
-	    after the storage was out of scope.
+	* main: Update copyright notice with year 2018
 
-	  Change-Id: I6282c542ff7b82fa091177a912d11234a8b00a30
+	  Change-Id: I2d80bc5edf940fab914cba3d8a0fa0b5eb2a3148
 
-2015-11-11 17:20 +0000 [5801917a75]  Richard Mudgett <rmudgett@digium.com>
+2018-03-26 07:42 +0000 [48190c7f93]  Guido Falsi <madpilot@freebsd.org>
 
-	* ast_json_pack(): Use safer json ref mechanism.
+	* core: fix getopt(3) usage
 
-	  Change-Id: I49204db2e57ae96eee43909c18ed007c09ac817e
+	  Setting optind = 0 is forced to 1 in glibc implementation, but
+	  causes option parsing to be flawed in other implementations, for
+	  example on FreeBSD.
 
-2015-11-11 16:52 +0000 [46ed7afdd9]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27773 #close
 
-	* rtp_engine.c: Eliminate rtcp_report_to_json() RAII_VAR usage.
+	  Change-Id: Ia548e69f8302e9754dbbedb6bc451c0700c66f61
 
-	  Change-Id: I58a22c2ca82e91d7537409b7b3af2d735827a54d
+2018-03-23 13:15 +0000 [07cf6b1437]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-12-18 12:23 +0000 [5cdc65cf1e]  Corey Farrell <git@cfware.com>
+	* install_prereq: Add Slackware (somehow).
 
-	* CLI: Fix 'core set debug channel' completion bug.
+	  ASTERISK-27770
 
-	  The completion generator is missing a return so typing "core set debug
-	  all off <tab>" causes the command to actually execute.
+	  Change-Id: Ib87e0483c785542238cfe34c1e884d5a31edfaab
 
-	  Change-Id: Ibf6462088a74eee66967732b50445783ebefc20b
+2018-03-23 09:13 +0000 [307a295d00]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-12-18 08:25 +0000 [7f2df9e277]  Joshua Colp <jcolp@digium.com>
+	* install_prereq: Add Gentoo Linux.
 
-	* confbridge: Clarify mute sound documentation.
+	  ASTERISK-27769
 
-	  The mute/unmute sounds are only played when the
-	  action is initiated using the DTMF menu.
+	  Change-Id: Ieb13293cd67481f3a33f58f6f7c8c3ee1e338e7a
 
-	  ASTERISK-24756
+2018-03-17 01:02 +0000 [318bf45928]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I55b3dd5bc166096bf5e2f547ddd0ce355f36e3dc
+	* main/indications: Use ast_cli_completion_add for all completions.
 
-2017-12-18 06:36 +0000 [4cda942ebd]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I371be01f178fb542a9fbe8d97e7ae21aa4d82c36
 
-	* app_transfer: Remove LOCAL from documentation.
+2018-03-21 14:54 +0000 [75715b95b4]  Russell Bryant <russell@russellbryant.net>
 
-	  The Local channel has never supported app_transfer
-	  from what I can see so remove it from the documentation.
+	* app_originate: Add async option.
 
-	  ASTERISK-25649
+	  Add an option to make app_originate not wait for the created channel
+	  to answer.
 
-	  Change-Id: Icbcfe297f6f866285a26b3e9fd5c6d00fa22e0e9
+	  Change-Id: I7fc2facd77079abc6321f44e8bcd4e39298de2ae
+	  Requested-by: Frederic Steinfels <fst@highdefinition.ch>
+	  Signed-off-by: Russell Bryant <russell@russellbryant.net>
 
-2017-12-15 19:01 +0000 [a368ad9229]  Richard Mudgett <rmudgett@digium.com>
+2018-03-22 07:27 +0000 [4f33f56a72]  Alexander Traud <pabstraud@compuserve.com>
 
-	* chan_pjsip.c: Improve ast_request() diagnostic msgs.
+	* BuildSystem: pjsip_evsub_set_uas_timeout was not used (part 2).
 
-	  Attempting to dial PJSIP/endpoint when the endpoint doesn't exist and
-	  disable_multi_domain=no results in a misleading empty endpoint name
-	  message.  The message should say the endpoint was not found.
+	  The previous change was not complete.
 
-	  * Added missing endpoint not found message.
+	  ASTERISK-27435
 
-	  * Added more information to the empty endpoint name msgs if available.
+	  Change-Id: I11082c14c0ef9c6af8c995084a6851337ea2a90f
 
-	  * Eliminated RAII_VAR in request().
+2018-03-22 05:43 +0000 [d6fda173a4]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I21da85ebd62dcc32115b2ffcb5157416ebae51e4
+	* BuildSystem: With external editline, do not require libs for internal editline.
 
-2017-12-15 10:26 +0000 [b3b7367e6b]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27761
 
-	* cdr: Minor optimizations.
+	  Change-Id: Ib17a7415297a210cfcdbf149e4df9b6edadbfab6
 
-	  * bridge_candidate_process: remove SCOPED_AO2LOCK and return value.
-	  * handle_standard_bridge_enter_message: replace recursive call with goto
-	    statement.
+2018-03-21 22:00 +0000 [a6d58c518a]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-24297
+	* core: Create main/options.c.
 
-	  Change-Id: Id2eaa0822fb8dc799f63422bb3aa89de9d4ee2a2
+	  This creates a separate source to 'own' symbols related to options.h and
+	  paths.h.  This significantly reduces the number of exports created by
+	  main/asterisk.o.  This change is required to eventually be able to
+	  link unmodified Asterisk sources to utilities and/or stand-alone tests.
 
-2017-12-11 17:07 +0000 [8921b2581d]  Corey Farrell <git@cfware.com>
+	  ASTERISK~26245
 
-	* loader: Use vector to build apha sorted module lists.
+	  Change-Id: I5cf184f4757f9363b80c9e678bdc35c477122380
 
-	  Change-Id: I9c519f4dec3cda98b2f34d314255a31d49a6a467
+2018-03-21 19:25 +0000 [745b5134cd]  George Joseph <gjoseph@digium.com>
 
-2017-11-21 00:28 +0000 [d5a7a98528]  Corey Farrell <git@cfware.com>
+	* Revert "BuildSystem: In NetBSD, the Python Programming Language is python-X.Y."
 
-	* loader: Replace priority heap with vector.
+	  Something is causing a python2/python3 mismatch on Fedora27.
 
-	  This is needed for future changes which will require being able to
-	  process the load priority out of order.
+	  PYTHON='/usr/bin/python2'
+	  PYTHONDEV_CFLAGS='-I/usr/include/python3.6m '
+	  PYTHONDEV_INCLUDE='-I/usr/include/python3.6m '
+	  PYTHONDEV_LIB='-lpython3.6m '
+	  PYTHONDEV_LIBS='-lpython3.6m '
 
-	  Change-Id: Ia23421197f09789940510b03ebbbf3bf24d51bea
+	  This reverts commit be0e9920b64e3b07501b299d131309b58f9b0ddf.
 
-2017-11-20 23:10 +0000 [5e9d70ae5d]  Corey Farrell <git@cfware.com>
+	  Change-Id: I86dd102eb3ead199fe89178cdbadb36b4e2cfd1b
 
-	* loader: Rework of load_dynamic_module.
+2018-02-08 13:23 +0000 [411915af28]  Corey Farrell <git@cfware.com>
 
-	  * Split off load_dlopen to perform actual dlopen, check results and log
-	    warnings when needed.
-	  * Use flags which minimize number of calls to dlopen required.  First
-	    attempt always uses RTLD_GLOBAL when global_symbols_only is enabled,
-	    RTLD_LOCAL when it is not.
+	* loader: Reserve space for additional pointers in ast_module_info.
 
-	  This patch significantly reduces the number of dlopen's performed.  With
-	  299 modules my system ran dlopen 857 times before this patch, 655 times
-	  after this patch.
+	  This creates 4 reserved pointers in case we need additional dependency
+	  management fields.
 
-	  Change-Id: Ib2c9903cfddcc01aed3e01c1e7fe4a3fb9af0f8b
+	  Change-Id: If991ec99b779df1b2dfbd38ce1a0cd79f9e01821
 
-2017-11-21 20:34 +0000 [4de95d4bf0]  Corey Farrell <git@cfware.com>
+2018-03-20 15:28 +0000 [cf73a4203f]  Kevin Harwell <kharwell@digium.com>
 
-	* loader: Minor fix to module registration.
+	* bridge_softmix: Clear "talking" when a channel is put on hold
 
-	  This protects the module loader itself against crashing if dlopen is
-	  called on a module from outside loader.c.
+	  This patch clears the talking flag from the channel (if already set), and
+	  notifies listeners when that channel is put on hold. Note however, if the
+	  endpoint continues to send audio frames and these are received by the bridge
+	  then that channel will be put back into a "talking" state even though they
+	  are on hold.
 
-	  * Expand scope of lock inside ast_module_register to include reading of
-	    resource_being_loaded.
-	  * NULL check resource_being_loaded.
-	  * Set resource_being_loaded NULL as soon as dlopen returns.  This fixes
-	    some error paths where it was not NULL'ed.
-	  * Create module_destroy function to deduplicate code from
-	    ast_module_unregister and modules_shutdown.
-	  * Resolve leak that occured if a module did not successfully register.
-	  * Simplify checking for successful registration.
+	  ASTERISK-27755 #close
 
-	  Change-Id: I40f07a315e55b92df4fc7faf525ed6d4f396e7d2
+	  Change-Id: I930e16c4662810f9f02043d69062f88173c5e2ef
 
-2016-10-06 01:29 +0000 [e3bd95f55c]  Corey Farrell <git@cfware.com>
+2018-03-20 11:53 +0000 [bfefde5b07]  Alexander Traud <pabstraud@compuserve.com>
 
-	* chan_sip: Add security event for calls to invalid extension.
+	* BuildSystem: For consistency, avoid extra libs to be empty.
 
-	  Log a message to security events when an INVITE is received to an
-	  invalid extension.
+	  AST_EXT_LIB_CHECK has several optional parameters. When an optional parameter
+	  is left empty, [] is used to indicate this. However, this is done in the script
+	  ./configure only then, when a further parameter is not empty. For example, when
+	  no extra libraries are needed to test the checked library, parameter 5 is not
+	  mentioned. Except parameter 6 and higher are used, then parameter 5 must be
+	  empty.
 
-	  ASTERISK-25869 #close
+	  However, this general rule was broken
+	  * four times for parameter 5 (extra libs) and
+	  * three times for parameter 4 (header)
+	  as found via the Regular Expression \[\]\). In case of parameter 5, all cases
+	  were changed, because that happened for no reason. In case of parameter 4, an
+	  [] improves readability actually. Therefore for parameter 4, the only case which
+	  did not do it was changed. All this aims to create more consistency: Only do
+	  something different if there is a reason to do so.
 
-	  Change-Id: I0da40cd7c2206c825c2f0d4e172275df331fcc8f
+	  Change-Id: I037ef170cf1ad94497151a9ea5071a31c656cafe
 
-2017-12-12 12:55 +0000 [501f4dcdd8]  Corey Farrell <git@cfware.com>
+2018-03-20 09:58 +0000 [8bd5980e14]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-	* aco: Minimize use of regex.
+	* func_channel: Delete dead CHANNEL_TRACE code
 
-	  Remove nearly all use of regex from ACO users.  Still remaining:
-	  * app_confbridge has a legitamate use of option name regex.
-	  * ast_sorcery_object_fields_register is implemented with regex, all
-	    callers use simple prefix based regex.  I haven't decided the best
-	    way to fix this in both 13/15 and master.
+	  The functions behind the flag and the flag itself were removed
+	  from Asterisk 12 as incompatible with the new architecture.
 
-	  Change-Id: Ib5ed478218d8a661ace4d2eaaea98b59a897974b
+	  Change-Id: I058493ef7a53ee290fd225bbcbb07bf46b623ccf
 
-2017-12-12 12:36 +0000 [7413bcbeb5]  Corey Farrell <git@cfware.com>
+2018-03-17 21:26 +0000 [040bb21771]  Corey Farrell <git@cfware.com>
 
-	* aco: Create ways to minimize use of regex.
+	* core: Remove additional symbols.
 
-	  ACO uses regex in many situations where it is completely unneeded.  In
-	  some cases this doubles the total processing performed by
-	  aco_process_config.
+	  Remove symbols that are depreacated and replaced:
+	  * ast_channel_datastore_alloc
+	  * ast_channel_datastore_free
+	  * ast_channel_cmpwhentohangup
+	  * ast_channel_setwhentohangup
+	  * config_text_file_save
+	  * devstate2str
+	  * ast_device_state_changed
+	  * ast_device_state_changed_literal
+	  * ast_verbose_get_by_module
+
+	  Remove unused symbols:
+	  * channelreloadreason2txt (last used in Asterisk 12).
+
+	  Remove unused ast_options flags:
+	  * AST_OPT_FLAG_END_CDR_BEFORE_H_EXTEN / ast_opt_end_cdr_before_h_exten
+	  * AST_OPT_FLAG_VERBOSE_MODULE / ast_opt_verb_module
+	  * AST_OPT_FLAG_INITIATED_SECONDS
 
-	  * Create ACO_IGNORE category type for use in place of skip_category
-	    regex source string.
-	  * Create additional aco_category_op values to allow specifying category
-	    filter using either a single plain string or a NULL terminated array
-	    of plain strings.
-	  * Create ACO_PREFIX to allow matching option names to case insensitive
-	    prefixes.
+	  Change-Id: I841255995d195f8efc1ed47af9c7a2f131c08645
 
-	  Change-Id: I66a920dcd8e2b0301f73f968016440a985e72821
+2018-03-17 20:03 +0000 [de77cf8698]  Corey Farrell <git@cfware.com>
 
-2017-12-15 07:56 +0000 [8165115faa]  Corey Farrell <git@cfware.com>
+	* core: Remove dead symbols from asterisk.exports.in.
 
-	* res_smdi: Fix shutdown ref.
+	  * dahdi_chan_name
+	  * dahdi_chan_name_len
+	  * dahdi_chan_mode
+	  * __manager_event
+	  * dialed_interface_info
 
-	  When adding shutdown refs for OPTIONAL_API components I accidentally
-	  added it to the unload_module function in res_smdi.  Move it to
-	  load_module.
+	  Added comment about __progname and environ being needed for FreeBSD to
+	  prevent accidental removal in the future.
 
-	  Change-Id: I2b9da38fbc11ef78ea23dbb2df92b684be7f647c
+	  Change-Id: I3ae026bc541cd9cb572be2ffa95fc359547642b5
 
-2017-12-14 18:55 +0000 [8338f3ec14]  Sean Bright <sean.bright@gmail.com>
+2018-03-17 01:39 +0000 [201762f161]  Corey Farrell <git@cfware.com>
 
-	* res_hep: hepv3_is_loaded() should check if we are enabled
+	* named_acl: Use ast_cli_completion_add.
 
-	  res_hep_pjsip.so and res_hep_rtcp.so will still load and do a lot of
-	  unnecessary work even if 'enabled' is set to 'no' in hep.conf.
+	  Change-Id: I317a82de976bbdbfe4352c243e32a7bb8f66c377
 
-	  Change-Id: I3eddfeea09c6b5bc7c641952ee0ae487fd09b64b
+2018-03-17 01:58 +0000 [645203a422]  Corey Farrell <git@cfware.com>
 
-2017-12-14 15:27 +0000 [85dec2ae4f]  Corey Farrell <git@cfware.com>
+	* main/sounds: Use ast_cli_completion_add.
 
-	* res_clialiases: Fix completion pass-through.
+	  Change-Id: I140e1137906bbfcdb61c0c6304159be459ad873e
 
-	  Never ignore contents of line when generating completion options.
+2018-03-16 10:19 +0000 [5d097f8236]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I74389efdfea154019d3b56a9f381610614c044c8
+	* channel.c:  Allow generic plc then channel formats are equal
 
-2017-12-11 18:20 +0000 [61e81338d9]  Richard Mudgett <rmudgett@digium.com>
+	  If the two formats on a channel are equal, we don't transcode and since
+	  the generic plc needs slin to work, it doesn't get invoked.
 
-	* res_rtp_asterisk.c: Disable packet flood detection for video streams.
+	  * A new configuration option "genericplc_on_equal_codecs" was added
+	    to the "plc" section of codecs.conf to allow generic packet loss
+	    concealment even if no transcoding was originally needed.
+	    Transcoding via SLIN is forced in this case.
 
-	  We should not do flood detection on video RTP streams.  Video RTP streams
-	  are very bursty by nature.  They send out a burst of packets to update the
-	  video frame then wait for the next video frame update.  Really only audio
-	  streams can be checked for flooding.  The others are either bursty or
-	  don't have a set rate.
+	  ASTERISK-27743
 
-	  * Added code to selectively disable packet flood detection for video RTP
-	  streams.
+	  Change-Id: I0577026a179dea34232e63123254b4e0508378f4
 
-	  ASTERISK-27440
+2018-03-17 01:09 +0000 [8d01ec572d]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I78031491a6e75c2d4b1e9c2462dc498fe9880a70
+	* manager: Use ast_cli_completion_add for completion generators.
 
-2017-12-14 12:14 +0000 [62e743e6a7]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I658141c6ec490a3e866b02d2afea757928ceaabf
 
-	* res_musiconhold: Start playlist after initial announcement
+2018-03-17 02:16 +0000 [2c1ad2f510]  Corey Farrell <git@cfware.com>
 
-	  Reset the samples counter to zero when we are done playing an
-	  announcement so that we don't skip into the middle of the first file in
-	  the playlist.
+	* main/test: Use ast_cli_completion_add.
 
-	  Also add the selected annoucement to the output of 'moh show classes.'
+	  Change-Id: I5133ff2ba4e030f9733fb3d050c863d72a22ae6b
 
-	  ASTERISK-24329 #close
-	  Reported by: Thomas Frederiksen
+2018-03-18 10:16 +0000 [115939caeb]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I2a5f986a31279c981592f49391409ebf38d6f6d0
+	* rtp: Add REMB RTP property and set it on PJSIP video RTP.
 
-2017-12-14 10:51 +0000 [74073c395b]  Sean Bright <sean.bright@gmail.com>
+	  This change adds a property to RTP instances to indicate that
+	  REMB support is enabled and that sending/receiving should be
+	  passed through.
 
-	* coverity: Fix warnings in res_smdi
+	  This also enables it on video RTP instances in PJSIP if
+	  WebRTC support is enabled.
 
-	  ASTERISK-19657 #close
-	  Reported by: Matt Jordan III, Esq.
+	  Finally the goog-remb extension is added to the SDP using
+	  the rtcp-fb attribute to indicate our support for it.
 
-	  Change-Id: I59a5e6ef3e7d9e848bec1f4b40cb73321bc7956a
+	  Details about REMB can be found on the draft document for it:
+	  https://tools.ietf.org/html/draft-alvestrand-rmcat-remb-03
 
-2017-12-14 10:22 +0000 [a1fcb7b5a6]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I1902dda1c0882bd1a0d71b2f120684b44b97e789
 
-	* configs: Comment out and change IP of iax.conf [demo]
+2018-03-17 04:31 +0000 [8c25a72d57]  Corey Farrell <git@cfware.com>
 
-	  This no longer appears to exist, so no sense in causing confusion.
+	* main/bridge: Use ast_cli_completion_add.
 
-	  ASTERISK-27175 #close
-	  Reported by: Tzafrir Cohen
+	  Change-Id: I3775a696d6a57139fdf09651ecb786bcf1774509
 
-	  Change-Id: Idde967924c69f6a741dc9a5ab7dacb44d22cf100
+2018-03-17 16:41 +0000 [5b40441197]  Corey Farrell <git@cfware.com>
 
-2017-12-13 14:26 +0000 [6d290f1880]  George Joseph <gjoseph@digium.com>
+	* core: Minor cleanup of ast_el_read_char.
 
-	* README: Remove outdated references to tex docs
+	  * Define CHAR_T_LIBEDIT and CHAR_TO_LIBEDIT based on
+	    HAVE_LIBEDIT_IS_UNICODE.  This avoids needing to repeatedly use
+	    conditional blocks, eliminates having multiple function prototypes.
+	  * Remove parenthesis from return values.
+	  * Add missing code block brackets {}.
+	  * Reduce use of 'else' conditional statements where possible.
 
-	  Added links to the wiki to replace references to outdated
-	  tex docs.
+	  Change-Id: I4315328ebea2f62641faf6881de2ac20a9f9d08e
 
-	  ASTERISK-27430
-	  Reported by: Corey Farrell
+2018-03-17 10:49 +0000 [e61b50b67a]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I5007e732b30bc7b63d124c530ae8857c89991209
+	* BuildSystem: Check for header file of OGG.
 
-2017-12-13 09:50 +0000 [dd72844e4d]  Corey Farrell <git@cfware.com>
+	  Asterisk uses various symbols of the shared library libogg within the module
+	  format_ogg_vorbis. However, the source code of that module did not include the
+	  header file of libogg explicitly but implicitly. Because that header was not
+	  included before Asterisk 14, the script ./configure was told not to check for
+	  it.
 
-	* CLI: Remove special handling of 'core set verbose' from rasterisk.
+	  Anyway, even Asterisk 13 LTS uses symbols of libogg. Therefore, that header
+	  should be included explicitly. Therefore, ./configure should check for that
+	  header.
 
-	  rasterisk does not need to handle setting verbose levels locally, it
-	  should just tell the daemon what it wants and print what it is given.
-	  Just max out the verbose level on the local client so all filtering
-	  happens on the daemon.
+	  Change-Id: I98c50d56311b68880d1084fcc62c35ab2f8692db
 
-	  ASTERISK-20281 #close
+2018-03-09 06:26 +0000 [f697025ae5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ia305f75f1fc424a9169bfa30ef70d626ace2c8a8
+	* BuildSystem: When no download utility is available, display the explanation.
 
-2017-11-30 10:12 +0000 [5705e8ae0e]  Joshua Colp <jcolp@digium.com>
+	  ./configure --with-pjproject-bundled
+	  did not display an explanation, when no download utility like wget, curl, or
+	  fetch was installed beforehand, although an explanation existed in code. This
+	  happened because the code expected the variable DOWNLOAD_TO_STDOUT to be empty.
+	  However, the script ./configure set that variable always.
 
-	* AST-2017-012: Place single RTCP report block at beginning of report.
+	  Change-Id: I64c99b76a03525c69471e5055bf124b36a51bbd4
 
-	  When the RTCP code was transitioned over to Stasis a code change
-	  was made to keep track of how many reports are present. This count
-	  controlled where report blocks were placed in the RTCP report.
+2018-03-17 05:00 +0000 [10a978829e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  If a compound RTCP packet was received this logic would incorrectly
-	  place a report block in the wrong location resulting in a write
-	  to an invalid location.
+	* BuildSystem: Remove unused dependency on libltdl.
 
-	  This change removes this counting logic and always places the report
-	  block at the first position. If in the future multiple reports are
-	  supported the logic can be extended but for now keeping a count
-	  serves no purpose.
+	  Asterisk does not need the development package of libltdl, because it does not
+	  use any symbol of -lltdl directly. Instead, it uses the runtime package via the
+	  shared library -lodbc. On the supported platforms, that shared library declares
+	  its dependency on -lltdl correctly, otherwise AST_EXT_LIB_CHECK would have
+	  failed.
 
-	  ASTERISK-27382
-	  ASTERISK-27429
+	  ASTERISK-27745
 
-	  Change-Id: Iad6c8a9985c4b608ef493e19c421211615485116
+	  Change-Id: Icd315809b8e7978203431f3afb66240dd3a040ba
 
-2017-12-06 08:24 +0000 [7a68faeb1d]  Joshua Colp <jcolp@digium.com>
+2018-03-17 02:25 +0000 [1136a22a1e]  Corey Farrell <git@cfware.com>
 
-	* pjsip: Ignore state changes from old transactions.
+	* main/translate: Use ast_cli_completion_add.
 
-	  When we fail over to a new target we create a new transaction
-	  and it becomes the current INVITE transaction. This does not
-	  prevent the previous transaction from raising state changes
-	  and causing the session to be prematurely disconnected if a
-	  transport error occurs immediately.
+	  Change-Id: I0e2402660e54d91f74ab0804c62a5b1925577413
 
-	  This change backports a fix from PJSIP that eliminates the
-	  incorrect state change and reduces when they would be raised
-	  in the first place.
+2018-03-17 02:00 +0000 [91ac95993e]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27408
+	* main/taskprocessor: Use ast_cli_completion_add.
 
-	  Change-Id: Id22d087591782eee31311753d11e7eca4b95ef34
+	  Change-Id: Ie5f812a988ed811fd11967151932de62bc131b48
 
-2017-12-12 15:38 +0000 [7ffc41d19f]  Ivan Poddubny <ivan.poddubny@gmail.com>
+2018-03-15 15:06 +0000 [3ad56aa929]  Corey Farrell <git@cfware.com>
 
-	* app_queue: Fix extension state subscriptions removed on dialplan reload
+	* main/config: Use ast_cli_completion_add for reload completion.
 
-	  The approach with having a single global subscription to all extension
-	  state changes has one issue: dynamically created hints don't have any
-	  watchers and are therefore garbage collected on the first dialplan
-	  reload.
+	  Change-Id: Ia3fa4c03f2285a1ec8814bbe7f4624ead9111ad1
 
-	  This change creates a state subscription for every queue member with a
-	  hint as state_interface, thus increasing the count of watches for
-	  hints, so they are not destroyed prematurely anymore.
+2018-03-17 00:51 +0000 [9e335f22e7]  Corey Farrell <git@cfware.com>
 
-	  There are 2 side effects:
-	  1. The state change callback in app_queue is not executed when
-	     there are no members referring to the extension.
-	  2. The callback is called multiple times for the same hint if it's
-	     associated with more than one queue member.
+	* aco: Use ast_cli_completion_add for 'config show help'.
 
-	  Reported by: Steven T. Wheeler
+	  In addition this removes:
+	  * RAII_VAR usage
+	  * Duplicate check of pos
+	  * Unneeded arguments.
 
-	  ASTERISK-18411 #close
+	  Change-Id: I2da8eac2670d1d8d6474c04037129804f55ebf39
 
-	  Change-Id: I4956af2136ea2a7f110ac9272eae5f6e676d8f89
+2018-03-14 04:27 +0000 [4d1c9d8711]  Corey Farrell <git@cfware.com>
 
-2017-12-12 15:28 +0000 [e1a358a6e4]  Sean Bright <sean.bright@gmail.com>
+	* core: Stop using AST_INLINE_API for allocator functions.
 
-	* chan_sip: Don't send trailing \0 on keep alive packets
+	  This replaces AST_INLINE_API allocators in utils.h with real functions
+	  implemented in astmm.c.  Associated macro's are also moved from utils.h
+	  to astmm.h.
 
-	  This is a partial fix for ASTERISK~25817 but does not address the
-	  comments regarding RFC 5626.
+	  Remove menuselect conflicts between MALLOC_DEBUG and DEBUG_CHAOS as they
+	  can now be combined.
 
-	  Change-Id: I227e2d10c0035bbfa1c6e46ae2318fd1122d8420
+	  This has multiple benefits:
+	  * Simplifies asterisk/utils.h by removing inline functions and use of
+	    the logger.
+	  * Removal of these inline functions decreases size of Asterisk and
+	    module binaries by 1% or more.
+	  * Puts memory management functions together with and without
+	    MALLOC_DEBUG enabled, simplifying management of the code.
+	  * Enables DEBUG_CHAOS for ASTMM_REDIRECT and bundled pjproject.
 
-2017-12-12 15:19 +0000 [ce2c89ce68]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: If9df4377f74bdbb627461b27a473123e05525887
 
-	* chan_sip: Don't crash in Dial on invalid destination
+2018-02-27 03:01 +0000 [ecc846b26b]  Florian Floimair <f.floimair@commend.com>
 
-	  Stripping the DNID in a SIP dial string can result in attempting to call
-	  the argument parsing macros on an empty string, causing a crash.
+	* app_dial: Enable early-media video
 
-	  ASTERISK-26131 #close
-	  Reported by: Dwayne Hubbard
-	  Patches:
-	  	dw-asterisk-master-dnid-crash.patch (license #6257) patch
-	  	uploaded by Dwayne Hubbard
+	  Certain applications (e.g. door-phone) require that also video is transmitted
+	  before a call is accepted.
 
-	  Change-Id: Ib84c1f740a9ec0539d582b09d847fc85ddca1c5e
+	  Change-Id: I9842e1dc2f6e1c2c49dc33fe615255007d2f821e
 
-2017-12-12 15:16 +0000 [6632f61153]  Corey Farrell <git@cfware.com>
+2018-03-05 06:50 +0000 [be0e9920b6]  Alexander Traud <pabstraud@compuserve.com>
 
-	* menuselect: Tweak check for recently run configure.
+	* BuildSystem: In NetBSD, the Python Programming Language is python-X.Y.
 
-	  Recently menuselect has randomly produced an error stating that
-	  configure was just run and make had to be restarted.  I believe this is
-	  due to an incorrect menuselect/Makefile rule.  The original rule
-	  produced an error if makeopts or autoconfig.h were older than
-	  makeopts.in or autoconfig.h.in.  I believe this can create an issue if
-	  makeopts is older than autoconfig.h.in or if autoconfig.h is older than
-	  makeopts.in.  The new rules compare files independently.
+	  ASTERISK-27717
 
-	  Change-Id: Ibca155035fa1392c95e33cbf25f257902abba17b
+	  Change-Id: If90ddf9c396c32e7402a894f42dce215c30049d1
 
-2017-12-07 17:51 +0000 [73b3390dbe]  Richard Mudgett <rmudgett@digium.com>
+2018-03-16 09:53 +0000 [02fa145a1b]  Alexander Traud <pabstraud@compuserve.com>
 
-	* chan_pjsip/res_pjsip: Add CHANNEL(pjsip,request_uri)
+	* BuildSystem: Avoid an extra case for OpenBSD.
 
-	  This patch does three things associated with the initial incoming INVITE
-	  request URI.
+	  Nine years ago with Mantis 13639 (now ASTERISK-12841) an extra case for OpenBSD
+	  was introduced: Vorbis required Ogg to be specified manually, because the shared
+	  library libvorbis.so did not specify its required dependency on -logg itself.
 
-	  1) Add access to the full initial incoming INVITE request URI.
+	  Today with OpenBSD 6.2, all libvorbis*.so declare their dependencies correctly.
+	  Therefore, an extra case is not required anymore.
 
-	  2) We were not setting DNID on incoming PJSIP channels.  The DNID is the
-	  user portion of the initial incoming INVITE Request-URI.  The value is
-	  accessed by reading CALLERID(dnid).
+	  Change-Id: Ifd04e0994ce9f1e4ad29c3948a0398b91d1e97bc
 
-	  3) Fix CHANNEL(pjsip,target_uri) documentation.
+2018-03-05 10:10 +0000 [00789174f6]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * The initial incoming INVITE request URI is now available using
-	  CHANNEL(pjsip,request_uri).
+	* BuildSystem: Enable Advanced Linux Sound Architecture (ALSA) in NetBSD.
 
-	  * Set the DNID on PJSIP channel creation so CALLERID(dnid) can return the
-	  initial incoming INVITE request URI user portion.
+	  In the script ./configure, AST_EXT_LIB_CHECK checks for external libraries. Some
+	  libraries do not specify all their dependencies and require additional shared
+	  libraries. In AST_EXT_LIB_CHECK, this is the fifth parameter. However, if a
+	  library is specified there, it must exist on the platform, because ./configure
+	  tries to compile/link/execute a small app using those statements. For example,
+	  the library libdl.so is Linux specific and does not exist on BSD-like platforms.
 
-	  * CHANNEL(pjsip,target_uri) now correctly documents that the target URI is
-	  the contact URI.
+	  Furthermore, no supported platform/version was found, which still (ever?)
+	  requires those additional libraries. Therefore, they were simply removed.
 
-	  * Refactored print_escaped_uri() out of channel_read_pjsip() to handle
-	  pjsip_uri_print() error condition when the buffer is too small.
+	  Finally, this change adds the error code ESTRPIPE to the channel driver
+	  chan_alsa for those platforms which lack it, again for example NetBSD.
 
-	  ASTERISK-27478
+	  ASTERISK-27720
 
-	  Change-Id: I512e60d1f162395c946451becb37af3333337b33
+	  Change-Id: I3b21f2135f6cbfac7590ccdc2df753257f426e0b
 
-2017-12-12 09:28 +0000 [ca448bf150]  Sean Bright <sean.bright@gmail.com>
+2018-03-16 09:02 +0000 [4d1e3fef6b]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip: Add TLSv1.1 and TLSv1.2 support
+	* app_voicemail:  Fix json blob errors
 
-	  Support for these protocols was added in the same commit as the 'proto'
-	  field, so we can safely use the same ./configure check.
+	  When app_voicemail calls ast_test_suite_notify with the results of
+	  a user keypress, it formats the keypress as '%c'.  If the user hung up
+	  or some other error occurrs, the result of the keypress is a non
+	  printable character.  This ultimately causes json_vpack_ex to think
+	  it's being passed a non utf-8 string and return an error.
 
-	  For reference: https://trac.pjsip.org/repos/changeset/4968
+	  * Keypress results passed to ast_test_suite_notify are now checked with
+	    isprint() and a '?' is substituted if the check fails.
 
-	  Change-Id: Icf4975d785d6bfb8f30ac7ffa695a0adf9382dac
+	  Change-Id: I78ee188916bbac840f3d03f40201b692347ea865
 
-2017-12-12 08:06 +0000 [d9b932a455]  Sean Bright <sean.bright@gmail.com>
+2018-03-15 09:32 +0000 [ebe957c5e9]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip: Assign support levels to a few modules
+	* main/cdr: Use ast_cli_completion_add for CDR channel completion.
 
-	  Change-Id: I51f6945c4023cb93fc7b87be5ab4c50e9e6ee27d
+	  Change-Id: Ie81830647a23aad61c1162583b6d50adbe6e7822
 
-2017-12-09 00:35 +0000 [4decf4e492]  Corey Farrell <git@cfware.com>
+2018-03-12 10:20 +0000 [dbf5ff6ed0]  Alexander Traud <pabstraud@compuserve.com>
 
-	* CLI: Fix 'core show sysinfo' function ordering.
+	* install_prereq: Add Arch Linux.
 
-	  Handle CLI initialization before any processing occurs.
+	  ASTERISK-27738
 
-	  Change-Id: I598b911d2e409214bbdfd0ba0882be1d602d221c
+	  Change-Id: I7ca620e3c4dfb4b064a19382c4915aeb42a2a09f
 
-2017-12-08 12:04 +0000 [28eddc7ea6]  Richard Mudgett <rmudgett@digium.com>
+2018-03-15 08:19 +0000 [89ba4d4e3d]  Corey Farrell <git@cfware.com>
 
-	* stasis_channels.c: Don't set channel snapshot caller_dnid twice.
+	* main/ccss: Use ast_cli_completion_add for core id.
 
-	  Change-Id: Ib8d45bbdfbda81e65045f6dff874d189b74e5471
+	  Change-Id: I44b25d6d24c7d9bc1bb38a50774b38883162f98f
 
-2017-12-11 09:45 +0000 [028f4320de]  Sean Bright <sean.bright@gmail.com>
+2018-03-15 07:29 +0000 [aa0d95c730]  Corey Farrell <git@cfware.com>
 
-	* codec_opus: Make libcurl a dependency in menuselect
+	* astobj2_container: Use ast_cli_completion_add for container names.
 
-	  ASTERISK-27475 #close
+	  Change-Id: I4f0fc09e820eb8d8da2354a177dbcf503c56ddd1
 
-	  Change-Id: If7384bc6ed002ef140dec69798d14c52b7cfd800
+2017-12-09 04:52 +0000 [b929a7fb8d]  Corey Farrell <git@cfware.com>
 
-2017-12-08 12:48 +0000 [4838557132]  Sean Bright <sean.bright@gmail.com>
+	* main/channel: Use ast_cli_completion_add for channeltypes.
 
-	* pjsip: Improve CLI completion performance
+	  Change-Id: Ia845fae6a84801cc7d9996767b99efb2753cbb48
 
-	  Use the new ast_cli_completion_add() function to improve completion
-	  performance for commands like 'pjsip show endpoint.'
+2018-03-14 12:38 +0000 [b45bb476bb]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I76d802294d2ac1766110dc75f7d117c8541ce348
+	* cli: Enable ast_cli_completion_add on public completion generators.
 
-2017-12-07 14:19 +0000 [a4b291029f]  Sean Bright <sean.bright@gmail.com>
+	  * ast_cli_complete
+	  * ast_complete_channels
+	  * ast_complete_applications
 
-	* astdb: Improve prefix searches in astdb
+	  These generators will now use ast_cli_completion_add if state == -1.
 
-	  Using the LIKE operator requires a full table scan of 'astdb', whereas a
-	  comparison operation is able to use the primary key index.
+	  Change-Id: I7ff311f0873099be0e43a3dc5415c0cd06d15756
 
-	  This patch adds a new function to the AstDB API for quick prefix matches
-	  and updates res_sorcery_astdb to utilize it. This showed substantial
-	  performance improvement in my test environment.
+2018-03-14 11:17 +0000 [92158b7f37]  Ross Beer <ross.beer@voicehost.co.uk>
 
-	  Related to ASTERISK~26806, but does not completely resolve it.
+	* res_pjsip_rfc3326.c: Account for more than one 'Reason' header
 
-	  Change-Id: I7d37f9ba2aea139dabf2ca72d31fbe34bd9b2fa1
+	  ASTERISK-27741
 
-2017-12-08 18:19 +0000 [9fc4f10e0e]  Corey Farrell <git@cfware.com>
+	  Change-Id: I0aa59a54735c6d20b95c54db1bd095dbf93e7adf
 
-	* loader: Refactor resource_name_match.
+2018-03-12 08:05 +0000 [b0fff03bb5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Optimize resource_name_match.  This change eliminates use of
-	  ast_strdupa, instead verifying that both basename's are the same length,
-	  then using strncasecmp.
+	* install_prereq: Add SUSE.
 
-	  Change-Id: I477275c0e954c99d74be5abfc8bb6545b04e5a3d
+	  ASTERISK-27736
 
-2017-12-08 14:58 +0000 [c3bc44fa1b]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I4cafc8973349d50a7cb7919ddf0bb1aaef4bfc3e
 
-	* pjsip_configuration: Add correct file header
+2018-02-16 21:11 +0000 [572a508ef2]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I25348c386a222bb704aff07f54375108a6402906
+	* loader: Convert reload_classes to built-in modules.
 
-2017-12-07 09:52 +0000 [f726f11974]  Sean Bright <sean.bright@gmail.com>
+	  * acl (named_acl.c)
+	  * cdr
+	  * cel
+	  * ccss
+	  * dnsmgr
+	  * dsp
+	  * enum
+	  * extconfig (config.c)
+	  * features
+	  * http
+	  * indications
+	  * logger
+	  * manager
+	  * plc
+	  * sounds
+	  * udptl
+
+	  These modules are now loaded at appropriate time by the module loader.
+	  Unlike loadable modules these use AST_MODULE_LOAD_FAILURE on error so
+	  the module loader will abort startup on failure of these modules.
+
+	  Some of these modules are still initialized or shutdown from outside the
+	  module loader.  logger.c is initialized very early and shutdown very
+	  late, manager.c is initialized by the module loader but is shutdown by
+	  the Asterisk core (too much uses it without holding references).
 
-	* utils: Add convenience function for setting fd flags
+	  Change-Id: I371a9a45064f20026c492623ea8062d02a1ab97f
 
-	  There are many places in the code base where we ignore the return value
-	  of fcntl() when getting/setting file descriptior flags. This patch
-	  introduces a convenience function that allows setting or clearing file
-	  descriptor flags and will also log an error on failure for later
-	  analysis.
+2018-03-13 16:39 +0000 [9e488dd482]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I8b81901e1b1bd537ca632567cdb408931c6eded7
+	* core: Remove incorrect usage of attribute_malloc.
 
-2017-12-07 19:41 +0000 [3d79c34350]  Corey Farrell <git@cfware.com>
+	  GCC documentation states that when __attribute__((malloc)) is used it
+	  should not return storage which contains any valid pointers.  It
+	  specifically mentions that realloc functions should not have the malloc
+	  attribute, but this also means that complex initializers which could
+	  contain initialized pointers should not use this attribute.
 
-	* res_stasis and res_speech: Fix load order.
+	  Change-Id: If507f33ffb3ca3b83b702196eb0e8215d27fc7d2
 
-	  res_stasis was missing AST_MODFLAG_LOAD_ORDER.  Set res_stasis and
-	  res_speech to start at (AST_MODPRI_APP_DEPEND - 1) so they are ready for
-	  dependent modules.
+2018-03-12 05:19 +0000 [d9776870e8]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I27f4f3810a95b6be8a5bfbf62be2ace6bfab6ff3
+	* BuildSystem: Enable IMAP storage on openSUSE and Arch Linux.
 
-2017-12-07 18:22 +0000 [ecdccb8071]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-27734
 
-	* pjsip_options: contacts sometimes not being updated on reload
+	  Change-Id: I8d6e6a1c08c031649764f5277fbbb85e57c3a9d4
 
-	  For both dynamic and static contacts it was possible that potential AOR
-	  changes were not being applied to all contacts. This was because the qualify
-	  and schedule code was only retrieving AOR's, and contacts with frequencies
-	  greater than zero.
+2018-02-23 07:41 +0000 [ea9768ff07]  Corey Farrell <git@cfware.com>
 
-	  For instance the following could happen: and AOR/contact has a frequency of 5,
-	  it then gets set to 0, and then a reload occurs. All scheduled OPTIONS are
-	  stopped, a list of AOR's is retrieved with frequency > 0, but none are
-	  selected since in this scenario all are 0. The contact for the one previously
-	  set to 5 though does not get updated, so it's status remains "AVAILABLE".
+	* stringfields: Remove MALLOC_DEBUG fields from struct ast_string_field_mgr.
 
-	  This patch makes it so all contacts (static and dynamic) are selected, and
-	  appropriately updated if need be.
+	  This causes MALLOC_DEBUG reporting to be slightly different, calls which
+	  cause additional memory pools to be allocated now report the callers
+	  location rather than the location which originally allocated the
+	  string field structure.  This reduces storage needed by string fields
+	  and allows MALLOC_DEBUG to identify the source of additional allocations
+	  rather than obscuring it by reporting the original allocation caller.
 
-	  ASTERISK-27467 #close
+	  Change-Id: Idd18e6639a87ab862079b580c114d90361412289
 
-	  Change-Id: I7a920170f89c683af9505d4723a44fc6841decdb
+2018-03-10 03:33 +0000 [fee929c8ac]  Corey Farrell <git@cfware.com>
 
-2017-12-07 18:18 +0000 [f20ab2b65f]  Kevin Harwell <kharwell@digium.com>
+	* core: Remove non-critical cleanup from startup aborts.
 
-	* pjsip_options: dynamic contact's fields not updated on reload
+	  When built-in components of Asterisk fail to start they cause the
+	  Asterisk startup to abort.  In these cases only the most critical
+	  cleanup should be performed - closing databases and terminating
+	  proceses.  These cleanups are registered using ast_register_atexit, all
+	  other cleanups should not be run during startup abort.
 
-	  Dynamic contacts were not being properly updated on reload. As a matter of
-	  fact any changes to the AOR that a dynamic contact was associated with were
-	  not being applied.
+	  The main reason for this change is that these cleanup procedures are
+	  untestable from the partially initialized states, if they fail it could
+	  prevent us from ever running the critical cleanup with ast_run_atexits.
 
-	  On reload, this patch makes it so for each dynamic contact, the associated
-	  AOR is now retrieved and the AOR's fields are applied to the contact.
+	  Create separate initialization for dns_core.c to be run unconditionally
+	  during startup instead of being initialized by the first dns resolver to
+	  be registered. This ensures that 'sched' is initialized before it can be
+	  potentially used.
 
-	  ASTERISK-27467
+	  Replace ast_register_atexit with ast_register_cleanup in media_cache.c.
+	  There is no reason for this cleanup to happen unconditionally.
 
-	  Change-Id: I8e3165dc6a745218c1c9db837f77fafa0516985d
+	  Change-Id: Iecc2df98008b21509925ff16740bd5fa29527db3
 
-2017-12-07 10:35 +0000 [5a0dcd9275]  Corey Farrell <git@cfware.com>
+2018-03-12 06:40 +0000 [ea3b8bb080]  Alexander Traud <pabstraud@compuserve.com>
 
-	* CLI: Fix remote console completion.
+	* install_prereq: Update FreeBSD libraries.
 
-	  Duplicate checking was done incorrectly when parsing completion options
-	  from a remote console causing all options to be ignored as duplicates.
-	  Once fixed I had to separate processing of the best match to ensure it
-	  was not identified as a duplicate when it is the only match.
+	  Because the code review system Gerrit creates merge conflicts even when one line
+	  apart another change happened, the previous update to the FreeBSD libraries had
+	  to be rebased via Git. Because of a break for training of the original
+	  contributor, this rebase was done by another contributor and the variant for
+	  Asterisk 13 was cherry-picked to all branches. By this, dependencies for new
+	  features added in newer Asterisk version got lost. This can be seen, when not
+	  the original path set but a previous patch set is compared.
 
-	  ASTERISK-27465
+	  This change here fixes this by adding those (optional) dependencies for
+	  Asterisk 15 and newer (again).
 
-	  Change-Id: Ibbdb29f88211742071836c9b3f4d2aa1221cd0f9
+	  ASTERISK-27686
 
-2017-12-06 23:35 +0000 [2691ee68ce]  Corey Farrell <git@cfware.com>
+	  Change-Id: I6638a3d0dc37ad4ff5f94be15463e3dd8a2bfe74
 
-	* translate: Skip matrix_rebuild during shutdown.
+2018-03-12 04:11 +0000 [9164be19d2]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I1e5eef4029cba56e33d786c5a5ade8091e531a1e
+	* res_srtp: Add support for libsrtp2.x on openSUSE.
 
-2017-12-06 14:49 +0000 [db6602f6f7]  Corey Farrell <git@cfware.com>
+	  Since ASTERISK-27253, no symbols from the header srtp2/crypto_types.h are used
+	  anymore. Therefore, its include statement can be removed. This allows to compile
+	  Asterisk on platforms which do not offer this private header, like openSUSE.
 
-	* sounds_index: Avoid repeatedly reindexing.
+	  ASTERISK-27733
 
-	  The sounds index is rebuilt each time a format is registered or
-	  unregistered.  This causes the index to be repeatedly rebuilt during
-	  startup and shutdown.
+	  Change-Id: I25c5cb8fa966043d1506ebef449e5a724412b4b6
 
-	  This patch significantly reduces the work done by delaying sound index
-	  initialization until after modules are loaded.  This way a reindex only
-	  occurs if a format module is loaded after startup.  We also skip
-	  reindexing when format modules are unloaded during shutdown.
+2018-03-08 09:14 +0000 [5b525c9781]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I585fd6ee04200612ab1490dc804f76805f89cf0a
+	* BuildSystem: Add NetBSD.
 
-2017-12-06 12:42 +0000 [289549d659]  Corey Farrell <git@cfware.com>
+	  Headers, libraries, and rpath.
 
-	* media_index: Improve startup.
+	  ASTERISK-27728
+	  ASTERISK-11015
+	  Reported by: Curt Sampson
 
-	  This eliminates some wasteful operations in media_index startup.
+	  Change-Id: I50aa5fcd095937df32a2e33307caac7e79a8b5b7
 
-	  * Replace statically set string-fields with char[0].
-	  * Eliminate pointless RAII_VAR's.
-	  * alloc_variant: Avoid pointless ao2_find on new info->variant.
-	  * Stop trying find_variant before alloc_variant.
-	  * process_media_file: replace ast_str with ast_asprintf.  This avoids
-	    reallocation of file_id_str.
+2018-03-09 03:13 +0000 [c5f2332953]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Overall sounds_index.c is about 27% of Asterisk startup time when using
-	  sample configs.  This patch reduces it to 20%.  This is a half-fix.  The
-	  real problem is that the media_index is regenerated repeatedly - 68
-	  times in my test.
+	* BuildSystem: For consistency, avoid double-checking via if clauses.
 
-	  Change-Id: Ia50b752f8efb356f852b05c4be495a6631af8652
+	  In the script ./configure, AST_EXT_LIB_CHECK and AST_PKG_CONFIG_CHECK first test
+	  whether parameter 1 was already found. Consequently, an if-test on PBX_ just a
+	  line below is redundant, if exactly the same parameter 1 is used again.
 
-2017-12-05 18:04 +0000 [f59a75c6fa]  Richard Mudgett <rmudgett@digium.com>
+	  No performance gain is expected by this change. However, because this strategy
+	  is used all over in ./configure except for two places, this change aims to
+	  create more consistency: Only do something different if there is a reason to do
+	  so.
 
-	* CDR: Fix deadlock setting some CDR values.
+	  Change-Id: I4a6f48127b7af3a48168c917e888be1f70625027
 
-	  Setting channel variables with the AMI Originate action caused a deadlock
-	  when you set CDR(amaflags) or CDR(accountcode).  This path has the channel
-	  locked when the CDR function is called.  The CDR function then
-	  synchronously passes the job to a stasis thread.  The stasis handling
-	  function then attempts to lock the channel.  Deadlock results.
+2018-03-09 02:44 +0000 [36c8885c66]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Avoid deadlock by making the CDR function handle setting amaflags and
-	  accountcode directly on the channel rather than passing it off to the CDR
-	  processing code under a stasis thread to do it.
+	* BuildSystem: Enable dladdr on non-Linux platforms like FreeBSD.
 
-	  * Made the CHANNEL function and the CDR function process amaflags the same
-	  way.
+	  ASTERISK-27641
 
-	  * Fixed referencing the wrong message type in cdr_prop_write().
+	  Change-Id: I587e8ba0123c70fc10cfd8b0ac3299551f61d84b
 
-	  ASTERISK-27460
+2018-03-08 13:53 +0000 [e6738b79b3]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I5eacb47586bc0b8f8ff76a19bd92d1dc38b75e8f
+	* Complete deprecating legacy modules.
 
-2017-12-06 07:36 +0000 [93859f9aca]  Richard Mudgett <rmudgett@digium.com>
+	  The menuselect comment was updated to deprecate these modules but the
+	  AST_MODULE_INFO block at the end of file was missed.
 
-	* bridge_basic.c: Update transfer diagnostic messages addendum.
+	  ASTERISK-27671
 
-	  * Added start DTMF transfer verbose messages.
-	  * Made associated transfer messages use a similar message format.
-	  * Adjusted message verbose level as requested by initial reporter.
+	  Change-Id: I63070b5c4d4f08af010c6034acd4793c1bcef839
 
-	  ASTERISK-27449
+2018-03-07 13:50 +0000 [7f4354c10f]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I2045714586414b3c5ef1f3cc56c1c4af4b31f551
+	* res_pjproject.c: Upgrade bundled PJPROJECT to 2.7.2
 
-2017-11-29 06:21 +0000 [aacdbdcfd2]  Niklas Larsson <niklas@tese.se>
+	  Update patches included in bundled PJPROJECT for the new version.
 
-	* bridge_basic.c: Update transfer diagnostic messages.
+	  ASTERISK-27730
 
-	  * Add the channel name to diagnostic messages so you will know which
-	  channel failed to transfer.
+	  Change-Id: Id3c8c8ad82126846bcd9768bc3d0a18d89be8944
 
-	  * Promoted some debug messages to verbose 4 messages.
+2018-03-08 12:02 +0000 [9ff95e46e3]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27449 #close
+	* install_prereq: Add NetBSD.
 
-	  Change-Id: Idac66b7628c99379cc9269158377fd87dc97a880
+	  ASTERISK-27729
 
-2017-12-01 13:54 +0000 [594faa192d]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I7a706d51375d54cf5e36d32397bfe09a48670804
 
-	* security-events: Fix SuccessfulAuth using_password declaration.
+2018-03-08 09:04 +0000 [75cebc3e71]  Alexander Traud <pabstraud@compuserve.com>
 
-	  The SuccessfulAuth using_password field was declared as a pointer to a
-	  uint32_t when the field was later read as a uint32_t value.  This resulted
-	  in unnecessary casts and a non-portable field value reinterpret in
-	  main/security_events.c:add_json_object().  i.e., It would work on a 32 bit
-	  architecture but not on a 64 bit big endian architecture.
+	* BuildSystem: Re-check for another UUID library only when previous check failed.
 
-	  Change-Id: Ia08bc797613a62f07e5473425f9ccd8d77c80935
+	  As a side-effect, this avoids the ambiguous output:
+	   checking for uuid_generate_random... no
+	  which was printed always previously.
 
-2017-12-04 03:40 +0000 [e7201c93cc]  Sungtae Kim <pchero21@gmail.com>
+	  ASTERISK-25586
+	  Reported by: John Nemeth
 
-	* Add new object for VoicemailUserEntry
+	  Change-Id: I6d541dfcf453932a9856c5e251aa22e0e6c233c9
 
-	  Currently, when the app_voicemail sending VoicemailUserEntry AMI event, there's
-	  no OldMessageCount info for default.
-	  To check the OldMessageCount info, it required IMAP_STORAGE define, but this is
-	  not correct.
-	  Added OldMessageCount item as a default.
+2018-03-08 05:28 +0000 [fc64a0e2b3]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27456
+	* BuildSystem: Instead of $PJPROJECT_LIBS with s, use $PJPROJECT_LIB everywhere.
 
-	  Change-Id: I5c71521c2d1daf8b7b161e31c34d28cca6aea4c7
+	  In the script ./configure,
+	  xyz_LIB  is set by AST_PKG_CONFIG_CHECK and
+	  xyz_LIBS is set by PKG_CHECK_MODULES within
+	  AST_PKG_CONFIG_CHECK. Both are the same. In Asterisk normally the former and
+	  only three times the latter was used. Let us use xyz_LIB without s, for
+	  consistency with AST_EXT_LIB_CHECK. That eases understanding because now readers
+	  do not have to know that xyz_LIB equals xyz_LIBS.
 
-2017-11-30 12:50 +0000 [9330eacc50]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I7359860a5d730cdc784c2c48e501a082196434d3
 
-	* res_rtp_asterisk.c: Increase strictrtp learning timeout time.
+2018-03-06 06:28 +0000 [16f6e94033]  Alexander Traud <pabstraud@compuserve.com>
 
-	  More complicated direct media reinvite negotiations can result in longer
-	  delays before direct media flows.  The strictrtp learning timeout time
-	  was too short.  One log showed that the first RTP packet came in just
-	  after three seconds.
+	* BuildSystem: Enable PortAudio in NetBSD.
 
-	  * Increase the strictrtp learning timeout time from 1.5 to 5 seconds.
+	  In NetBSD, PortAudio 1 is still the default version. PortAudio 2 can be
+	  installed side by side but gets placed in a 'portaudio2' subdirectory. To
+	  find PortAudio 2 even in a subdirectory, the tool pkg-config is queried via
+	  AST_PKG_CONFIG_CHECK. For those platforms, which do not list PowerAudio 2
+	  via pkg-config, the previous check remains and is executed thereafter.
 
-	  ASTERISK-27453
+	  ASTERISK-27721
 
-	  Change-Id: Ic5e711164cbb91b4d1c1e40c83697755640f138c
+	  Change-Id: I4175500126909ad1b181fff8e11bb4a3a6ae4fa9
 
-2017-12-04 08:33 +0000 [e819cf7826]  Alexander Traud <pabstraud@compuserve.com>
+2018-03-07 14:36 +0000 [c8a521b6c8]  Corey Farrell <git@cfware.com>
 
-	* res_rtp_asterisk: Correct default in sample configuration file.
+	* Replace direct checks of option_debug with DEBUG_ATLEAST macro.
 
-	  With Asterisk 12 (commit 866d968), the default of "icesupport" changed to
-	  - "yes" in the module "res_rtp_asterisk" and
-	  - "no" in the module "chan_sip".
-	  The latter was reflected in the sample configuration file for "sip.conf". The
-	  former did not make it into "rtp.conf.sample".
+	  Checking option_debug directly is incorrect as it ignores file/module
+	  specific debug settings.  This system-wide change replaces nearly all
+	  direct checks for option_debug with the DEBUG_ATLEAST macro.
 
-	  ASTERISK-20643
+	  Change-Id: Ic342d4799a945dbc40ac085ac142681094a4ebf0
 
-	  Change-Id: I2a2e0a900455d0767a99ea576e30adc6d7608a36
+2018-03-07 13:13 +0000 [1fe913f7bd]  Richard Mudgett <rmudgett@digium.com>
 
-2017-12-03 18:49 +0000 [41c14fd807]  Joshua Colp <jcolp@digium.com>
+	* BuildSystem regression: Fix errors reported by clean targets.
 
-	* pjproject: Clean up disabling of WebRTC support.
+	  Doing a 'make clean', 'make distclean', or 'make dist-clean' gets errors
+	  about an invalid shell option: "/bin/sh: 0: Illegal option -".
 
-	  The definition in config_site.h and the argument to the
-	  configure script are not necessary to disable WebRTC
-	  support. The correct argument, --disable-libwebrtc, is
-	  already passed.
+	  The clean targets do not include the makeopts file which defines GREP and
+	  LDCONFIG because the file may not exist and the distclean/dist-clean
+	  targets will delete it anyway.
 
-	  ASTERISK-26980
+	  ASTERISK-27715
 
-	  Change-Id: I27da2c894f87914956a72710222e17462d8a44bc
+	  Change-Id: I33d40acdb03862bc89aeb6fb1ff497894a8ea7f5
 
-2017-12-02 15:55 +0000 [f5cfd87c4a]  Corey Farrell <git@cfware.com>
+2018-03-06 13:31 +0000 [88cef40f6e]  Ross Beer <ross.beer@voicehost.co.uk>
 
-	* autoconf: Remove use of m4_ifblank.
+	* res_pjsip_rfc3326: Order of 'Reason' headers break many endpoints
 
-	  The m4_ifblank macro is not available on CentOS 6, reverse conditionals
-	  to allow use of m4_ifval instead.  ./bootstrap.sh was run but this patch
-	  does not result in any difference to the generated configure script.
+	  ASTERISK-27554
 
-	  Change-Id: I280785deb872ed8d3339d99cce63a2b54d5f1438
+	  Change-Id: If61c7faab7d2fa1031c056ed6268fe928e2391cf
 
-2017-12-01 16:18 +0000 [ef25628b10]  Corey Farrell <git@cfware.com>
+2018-03-06 08:14 +0000 [961dd9fe52]  Sungtae Kim <pchero21@gmail.com>
 
-	* README-SERIOUSLY.bestpractices.txt: Convert to markdown
+	* voicemail: Fixed wrong voicemail message count
 
-	  Follow-up to conversion of README.md.
+	  Fixed wrong voicemail mailbox reference for Action: VoicemailUsersList.
 
-	  Change-Id: I17ee7cf25bc027ece844efa2c1dfe613aff1e35b
+	  ASTERISK-27703
 
-2017-11-30 14:38 +0000 [0cdd31ee10]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ie6578ad80bba2bfaf34b84f0be978f59045ce6cd
 
-	* AST-2017-013: chan_skinny: Call pthread_detach when sess threads end
+2018-03-07 09:32 +0000 [58f44f225a]  Alexander Traud <pabstraud@compuserve.com>
 
-	  chan_skinny creates a new thread for each new session.  In trying
-	  to be a good cleanup citizen, the threads are joinable and the
-	  unload_module function does a pthread_cancel() and a pthread_join()
-	  on any sessions that are active at that time.  This has an
-	  unintended side effect though. Since you can call pthread_join on a
-	  thread that's already terminated, pthreads keeps the thread's
-	  storage around until you explicitly call pthread_join (or
-	  pthread_detach()).   Since only the module_unload function was
-	  calling pthread_join, and even then only on the ones active at the
-	  tme, the storage for every thread/session ever created sticks
-	  around until asterisk exits.
+	* utils: In Solaris, avoid a warning about an unused variable.
 
-	  * A thread can detach itself so the session_destroy() function
-	    now calls pthread_detach() just before it frees the session
-	    memory allocation.  The module_unload function still takes care
-	    of the ones that are still active should the module be unloaded.
+	  When HAVE_GETHOSTBYNAME_R_5 was set by the script ./configure, GCC 7.3.0 found
+	  an unused variable. Actually, the variable was used (set to a dummy value) but
+	  the compiler optimization might have removed that. Instead, this change ensures
+	  that the variable 'res' is only used when it is really required.
 
-	  ASTERISK-27452
-	  Reported by: Juan Sacco
+	  Change-Id: Ic3ea23ccf84ac4bc2d501b514985b989030abab5
 
-	  Change-Id: I9af7268eba14bf76960566f891320f97b974e6dd
+2018-03-07 01:45 +0000 [add03e207c]  Corey Farrell <git@cfware.com>
 
-2017-12-01 10:01 +0000 [6635ddc819]  Sean Bright <sean.bright@gmail.com>
+	* app_osplookup: Move header defines into the app.
 
-	* config: Speed up config template lookup
+	  astosp.h is leftover from when logic was split between app_osplookup and
+	  res_osp.  All logic was moved into app_osplookup by 109737eb1c in 2006,
+	  but astosp.h remained.  This moves the remaining defines into
+	  app_osplookup and deletes astosp.h.
 
-	  ast_category_get() has an (undocumented) implementation detail where it
-	  tries to match the category name first by an explicit pointer comparison
-	  and if that fails falls back to a normal match.
+	  Change-Id: I0a6c4debd7c9543b608520b1765abfa4fab7b2fd
 
-	  When initially building an ast_config during ast_config_load, this
-	  pointer comparison can never succeed, but we will end up iterating all
-	  categories twice. As the number of categories using a template
-	  increases, this dual looping becomes quite expensive. So we pass a flag
-	  to category_get_sep() indicating if a pointer match is even possible
-	  before trying to do so, saving us a full pass over the list of current
-	  categories.
+2018-02-14 07:33 +0000 [75a35ee5e8]  Jean Aunis <jean.aunis@prescom.fr>
 
-	  In my tests, loading a file with 3 template categories and 12000
-	  additional categories that use those 3 templates (this file configures
-	  4000 PJSIP endpoints with AOR & Auth) takes 1.2 seconds. After this
-	  change, that drops to 22ms.
+	* chan_sip: Fix improper RTP framing on outgoing calls
 
-	  Change-Id: I59b95f288e11eb6bb34f31ce4cc772136b275e4a
+	  The "ptime" SDP parameter received in a SIP response was not honoured.
+	  Moreover, in the abscence of this "ptime" parameter, locally configured
+	  framing was lost during response processing.
 
-2017-12-01 08:29 +0000 [077ceacd48]  Sean Bright <sean.bright@gmail.com>
+	  This patch systematically stores the framing information in the
+	  ast_rtp_codecs structure, taking it from the response or from the
+	  configuration as appropriate.
 
-	* config: Speed up ACO & sorcery initialization
+	  ASTERISK-27674
 
-	  When starting Asterisk in the foreground, there is a perceptible delay
-	  when loading modules that use the ACO and sorcery config frameworks.
-	  For example, a lightly configured res_pjsip took 853ms to load on my
-	  VM.
+	  Change-Id: I828a6a98d27a45a8afd07236a2bd0aa3cbd3fb2c
 
-	  I tracked down the slowness to the XPath queries used to associate the
-	  relevant documentation with the config options. One improvement was
-	  adding a call to xmlXPathOrderDocElems after loading an XML document.
-	  From the libxml2 docs:
+2018-02-20 11:48 +0000 [3fb26df4ac]  lvl <digium@lvlconsultancy.nl>
 
-	    Call this routine to speed up XPath computation on static documents.
+	* res_pjsip_session: properly handle SDP from a forked call with early media
 
-	  The second change was to remove recursive descent and wildcard
-	  operators from the XPath queries. After these changes, res_pjsip takes
-	  85ms to load on my VM and there is no longer a perceptible delay when
-	  starting Asterisk in the foreground.
+	  In handle_negotiated_sdp(), use session->active_media_state when
+	  session->pending_media_state is empty.  The 200's SDP should be fed into
+	  handle_negotiated_sdp_session_media() together with the already negotiated
+	  state, which is now in session->active_media_state instead.  Only if both
+	  the session's pending and active media are empty should
+	  handle_negotiated_sdp() abort.
 
-	  Change-Id: I45d457f1580e26bf5a2b0dab16e8e9ae46dcbd82
+	  ASTERISK-27441
 
-2017-12-01 06:07 +0000 [dac70daf1b]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: If0d5150ffe6f38d8a854831fef37942258d4629c
 
-	* res_http_post: Not all versions of gmime have GMIME_MAJOR_VERSION.
+2018-03-05 08:01 +0000 [ef79e583ec]  Alexander Traud <pabstraud@compuserve.com>
 
-	  This change makes the presence of the GMIME_MAJOR_VERSION
-	  definition optional, as not all versions of gmime actually
-	  define it.
+	* BuildSystem: Enable Lua in NetBSD.
 
-	  ASTERISK-27454
+	  luaL_openlib got removed with Lua 5.2.
+	  luaL_newstate is available in all versions.
 
-	  Change-Id: I01d99590045971ed6787899147170a5954077238
+	  ASTERISK-27718
 
-2017-11-17 10:38 +0000 [6274e58a25]  Corey Farrell <git@cfware.com>
+	  Change-Id: I9c8c8880315ee36ab740d7c40153306c0bfd6f71
 
-	* autoconf: Use m4 conditionals where possible.
+2018-03-06 07:33 +0000 [162fc4fba6]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I530c0a72f965437acef6a9a4fbfe5c487f078b65
+	* BuildSystem: Depend not implicitly but explicitly on external libraries.
 
-2017-11-17 09:15 +0000 [635fe8ec87]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27722
 
-	* autoconf: Fix call to AC_CONFIG_AUX_DIR.
+	  Change-Id: Ie7b8c30d86cb00a54d6ac4e09e6f28f42d2bd52c
 
-	  The `pwd` parameter to AC_CONFIG_AUX_DIR is unnecessary, the default
-	  value is $srcdir.
+2018-03-05 08:15 +0000 [99b6a14737]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Additionally remove the AC_REVISION call.  It only added a comment and
-	  is pointless without SVN tag replacements.
+	* res_http_post: Enable GMime in NetBSD.
 
-	  Change-Id: I99299a3217f095bddcb2edefb3b9af0ab147bc29
+	  ASTERISK-27719
 
-2017-11-26 11:47 +0000 [c46cab49a3]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I230c5f9f316b2e9465c093c13580f72ebbaf67a7
 
-	* translate: Transcode siren14, speex32, silk24, and silk12 via slin16.
+2018-03-05 04:16 +0000 [7e9734a858]  Alexander Traud <pabstraud@compuserve.com>
 
-	  When a format has no pre-recorded sound files, Asterisk has to transcode between
-	  formats. For this, Asterisk has a fixed translation table. If the pre-recorded
-	  sound files are not available in the same sample rate, Asterisk has not only to
-	  transcode but also to resample.
+	* BuildSystem: Enable autotools in NetBSD.
 
-	  Asterisk has pre-recorded files for SLN (8000 kHz) and SLN16 (16000 kHz).
-	  However before this change, Asterisk did not take the sample rate into account,
-	  because the translation paths to SLN and SLN16 got the same score/weight in the
-	  table. Consequently, you might have got narrow-band audio with siren14, speex32,
-	  silk24, and silk12 although those are (ultra) wide-band audio codecs.
+	  ASTERISK-27716
 
-	  With this change, the distance in sample-rates is taken into account. Now on the
-	  Command-Line interface (CLI) 'core show channels', you should see:
-	  (slin@16000)->(slin@32000)->(speex@32000).
+	  Change-Id: I52525e35e1620341272219911d054a1e3d3ec01e
 
-	  ASTERISK-23735
-	  Reported by: Richard Kenner
+2018-03-05 03:42 +0000 [b97905aaf2]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I9448295c1978be26f8633b6066395e7bbbe2e213
+	* BuildSystem: AC_PATH_PROG sets to colon character when not found.
 
-2017-11-26 09:44 +0000 [81e2d8aa9a]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27715
+	  Reported by: Corey Farrell
 
-	* res_ari: Fix inverted test giving wrong error message.
+	  Change-Id: I0d6d9572d1352dc7ad30c9917173f1e980d8c938
 
-	  The patch for ASTERISK_24560 inverted a test checking if the bridge name
-	  is being updated to a different name.
+2018-03-03 09:06 +0000 [aabbb49e33]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Fix the test to return "Changing bridge name is not implemented" when
-	  someone attempts to change the bridge name.
+	* chan_unistim: NetBSD has an incompatible struct in_pktinfo.
 
-	  ASTERISK-27445
+	  ASTERISK-27714
+	  Reported by: John Nemeth
 
-	  Change-Id: I4b70bf08b0e02e016108b077ff75b345dec12fc9
+	  Change-Id: I1b84a89315a5f61222123d21bf35c59224da8990
 
-2017-11-25 04:09 +0000 [0f719aa051]  Alexander Traud <pabstraud@compuserve.com>
+2018-03-03 08:30 +0000 [5d19762b5f]  Alexander Traud <pabstraud@compuserve.com>
 
-	* translate: Show sample rate for silk, speex, and slin in translation table.
+	* BuildSystem: Cast any intptr_t explicitly to its proposed type.
 
-	  ASTERISK-24662
+	  ASTERISK-27713
 
-	  Change-Id: I3822956984292c99c48bca8e97807e498ccc0e88
+	  Change-Id: I90c769e3c7f8c26de8a3af11335862cec15a1b22
 
-2017-11-23 13:27 +0000 [8dd9a79e6e]  Richard Mudgett <rmudgett@digium.com>
+2018-03-03 06:56 +0000 [9749524520]  Alexander Traud <pabstraud@compuserve.com>
 
-	* features.conf.sample: Clarify ActivatedBy documentation wording.
+	* BuildSystem: Detect whether uselocale(.) is available.
 
-	  Change-Id: Id2899331fe05d1909a862ea879742879d086bc64
+	  ASTERISK-27712
+	  Reported by: Joerg Sonnenberger, D'Arcy Cain
 
-2017-11-22 18:37 +0000 [a78d747ee8]  Corey Farrell <git@cfware.com>
+	  Change-Id: Idf1c9d43617a3e13028b95b313415903d80ef807
 
-	* Add defaultbranch to .gitreview.
+2018-03-03 03:53 +0000 [f7b845ff41]  Alexander Traud <pabstraud@compuserve.com>
 
-	  This will cause `git review` to post changes to the branch it is based
-	  on instead of always using master.  The defaultbranch setting should be
-	  updated when new major branches are created.
+	* BuildSystem: Avoid re-defining of pthread_* on NetBSD.
 
-	  Change-Id: I3db009217c5ae399fb84bee95076f4dbb7fa52d2
+	  ASTERISK-27711
 
-2017-11-22 18:43 +0000 [d6568aa72a]  Alexander Anikin <may213@yandex.ru>
+	  Change-Id: Idc9194035b2958b99f6b01eb5b438d45a074565b
 
-	* add cmd connection creation on creation ooh323 call data structure
+2018-03-02 07:05 +0000 [313a9fe255]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27353 #close
+	* BuildSystem: Install init scripts on openSUSE Tumbleweed.
 
-	  Reported by: Marco Giordani
+	  ASTERISK-27710
 
-	  Change-Id: I455096bd7da016b871afe09af86067c2c7c9f33f
+	  Change-Id: I4c777e41b31d4415bbe21cb435ad47b43ebb5467
 
-2017-11-22 10:42 +0000 [5abab0a34c]  Kevin Harwell <kharwell@digium.com>
+2018-03-02 05:12 +0000 [a9c02e484a]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pjsip: 183 without To tag does not negotiate media
+	* BuildSystem: Avoid == for comparison in ./configure.
 
-	  If a 183 with sdp response is receive without a To tag the sdp is not
-	  negotiated. According to RFC 3261 section 12.1.2 while a To tag is required,
-	  the client needs to still be able to handle the missing tag case for
-	  backwards compatibility.
+	  ASTERISK-27709
+	  Reported by: John Nemeth
 
-	  This patch, accepted by and applied to pjproject, makes it so if an incoming
-	  180/183 with SDP comes in without a To tag it gets appropriately handled.
+	  Change-Id: I11b1ae8fd404c04066f1458f5d71f9536359d58d
 
-	  ASTERISK-27442 #close
+2018-02-19 19:55 +0000 [c711e4076a]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ic9d6b01e05e8f4874eebbd7adfe05d932025d203
+	* core: Remove ABI effects of MALLOC_DEBUG.
 
-2017-11-21 06:39 +0000 [6ebe03d4b3]  Alexander Traud <pabstraud@compuserve.com>
+	  This allows asterisk to be compiled with MALLOC_DEBUG to load modules
+	  built without MALLOC_DEBUG.  Now pre-compiled third-party modules will
+	  still work regardless of MALLOC_DEBUG being enabled or not.
 
-	* res_rtp_asterisk: ICE server-reflexive candidates (srflx) with Dual-Stack.
+	  Change-Id: Ic07ad80b2c2df894db984cf27b16a69383ce0e10
 
-	  Previously, Asterisk sent srflx only when configured exclusively for IPv4. Now,
-	  srflx is gathered and sent via SDP, even when Asterisk is enabled for
-	  Dual Stack (IPv4+IPv6) and an IPv4 interface is available/used.
+2018-02-27 15:40 +0000 [1a36a452bd]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27437
+	* pjproject: Add cache_pools debugging option.
 
-	  Change-Id: Ie07d8e2bfa7b6fe06fcdc73d390a7a9a4d8c0bc1
+	  The pool cache gets in the way of finding use after free errors of memory
+	  pool contents.  Tools like valgrind and MALLOC_DEBUG don't know when a
+	  pool is released because it gets put into the cache instead of being
+	  freed.
 
-2017-11-20 13:05 +0000 [2bff38b8a8]  Corey Farrell <git@cfware.com>
+	  * Added the "cache_pools" option to pjproject.conf.  Disabling the option
+	  helps track down pool content mismanagement when using valgrind or
+	  MALLOC_DEBUG.  The cache gets in the way of determining if the pool
+	  contents are used after free and who freed it.
 
-	* res_parking: Make load_pri explicit.
+	  To disable the pool caching simply disable the cache_pools option in
+	  pjproject.conf and restart Asterisk.
 
-	  res_parking has an implicit load_pri of 0 meaining it's one of the very
-	  first modules loaded after modules with global symbols.  Set it
-	  explicitly in the AST_MODULE_INFO block.
+	  Sample pjproject.conf setting:
+	  [startup]
+	  cache_pools=no
 
-	  Change-Id: I297b6fb3ff6993ec004e667b22a74f5925906259
+	  * Made current users of the caching pool factory initialization and
+	  destruction calls call common routines to create and destroy cached pools.
 
-2017-11-21 09:16 +0000 [c6e1e6e968]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27704
 
-	* README: Convert to README.md.
+	  Change-Id: I64d5befbaeed2532f93aa027a51eb52347d2b828
 
-	  Convert the README file to markdown format, remove the old README.  This
-	  causes websites like github to display the README in a much nicer
-	  format with live links.  The raw file is still very readable from
-	  plain text editors and terminals.
+2018-01-31 11:48 +0000 [eacee03f0e]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I7d13131764a9a9026e5f8a6ddb245a01bbd788e7
+	* gitreview: Reorder and add padding.
 
-2017-11-20 16:48 +0000 [e9ba6a6b88]  Corey Farrell <git@cfware.com>
+	  Change-Id: I459dc320a8c9452a01eed6f403d786741587c890
 
-	* CLI: Finish conversion of completion handling to vectors.
+2018-02-23 21:24 +0000 [7b01236028]  Michael Cargile <mikec@vicidial.com>
 
-	  Change-Id: Ib81318f4ee52a5e73b003316e13fe9be1dd897a1
+	* apps/app_amd.c: Fixed total time and silence calculations
 
-2017-11-07 15:34 +0000 [26a400c67a]  Corey Farrell <git@cfware.com>
+	  Between Asterisk 11 and Asterisk 13 there was a significant increase
+	  in the number of AST_FRAME_NULL frames being processed by app_amd.c's
+	  main loop. Each AST_FRAME_NULL frame was being counted as 100ms
+	  towards the total time and silence. This may have been accurate
+	  when app_amd.c was orginally added, but it is not in Asterisk 13.
+	  As such the total analysis time and silence calculations were way
+	  off effectively breaking app_amd.c
 
-	* CLI: Refactor cli_complete.
+	  * Additional debug messages were added
+	  * AST_FRAME_NULL are now ignored
 
-	  * Stop using "_COMMAND NUMMATCHES" on remote consoles.  Using this
-	    command had doubled the amount of work needed from the Asterisk
-	    daemon for each completion request.
-	  * Fix code formatting.
-	  * Remove static buffer used to send the command, use the same buffer
-	    that will receive the results.
-	  * Move sort from ast_cli_display_match_list.
+	  ASTERISK-27610
 
-	  Change-Id: Ie2211b519a3d4bec45bf46e0095bdd01d384cb69
+	  Change-Id: I18aca01af98f87c1e168e6ae0d85c136d1df5ea9
 
-2017-11-07 14:13 +0000 [e723331f4f]  Corey Farrell <git@cfware.com>
+2018-02-23 14:58 +0000 [7e2128c8e6]  George Joseph <gjoseph@digium.com>
 
-	* CLI: Rewrite ast_el_strtoarr to use vector's internally.
+	* ast_coredumper:  Minor fixes
 
-	  This rewrites ast_el_strtoarr to use vector's internally, but still
-	  return the original NULL terminated array of strings.
+	  * Fix --tarball-config so the option doesn't cause an error.
 
-	  Change-Id: Ibfe776cbe14f750effa9ca360930acaccc02e957
+	  * Allow for missing /etc/os-release.
 
-2017-11-07 14:47 +0000 [e07d94111d]  Corey Farrell <git@cfware.com>
+	  * Add a sleep between tarballing the coredump and removing the
+	    output directory to allow the filesystem to settle.
 
-	* CLI: Refactor ast_cli_display_match_list.
+	  Change-Id: I73e03b13087978bcc7f6bc9f45753990f82d9d77
 
-	  * Stop estimating line count, just print until we run out of matches.
-	  * Stop freeing entries, the caller does that anyways.
-	  * Stop calculating / returning numoutput, it was ignored.
+2018-02-22 14:27 +0000 [0be1c388e4]  Ben Ford <bford@digium.com>
 
-	  Change-Id: I7f92afa8bea92241a95227587367424c8c32a5cb
+	* Add extended properties to rtp_engine for RTP retransmission support.
 
-2017-11-08 23:42 +0000 [8a63f4e709]  Corey Farrell <git@cfware.com>
+	  A couple of additional properties are needed in rtp_engine to enable
+	  support for packet retransmission: AST_RTP_PROPERTY_RETRANS_RECV and
+	  AST_RTP_PROPERTY_RETRANS_SEND. These will both be enabled automatically
+	  if an endpoint has the webrtc option enabled. While this adds no
+	  functionality currently, it will serve as a building block for future
+	  changes for RTP retransmission support.
 
-	* CLI: Create ast_cli_completion_add function.
+	  For more information, refer to the wiki page:
+	  https://wiki.asterisk.org/wiki/display/AST/WebRTC+User+Experience+Improvements
 
-	  Some completion generators are very inefficent due to the way CLI
-	  requests matches one at a time.  ast_cli_completion_add can be called
-	  multiple times during one invokation of a CLI generator to add all
-	  results without having to reinitialize the search state for each match.
+	  Change-Id: Ic598acd042a045f9d10e5bdccb66f4efc9e587cc
 
-	  Change-Id: I73d26d270bbbe1e3e6390799cfc1b639e39cceec
+2018-02-23 10:09 +0000 [a7927471ad]  Corey Farrell <git@cfware.com>
 
-2017-11-09 00:39 +0000 [115f53a0fb]  Corey Farrell <git@cfware.com>
+	* core: Fix handling of maximum length lines in config files.
 
-	* CLI: Remove calls to ast_cli_generator.
+	  When a line is the maximum length "\n" is found at sizeof(buf) - 2 since
+	  the last character is actually the null terminator.  In addition if a
+	  line was exactly 8190 plus a multiple of 8192 characters long the config
+	  parser would skip the following line.
 
-	  The ability to add to localized storage cannot be supported by
-	  ast_cli_generator.  The only calls to ast_cli_generator should be by
-	  functions that need to proxy the CLI generator, for example 'cli check
-	  permissions' or 'core show help'.
+	  Additionally fix comment in voicemail.conf sample config.  It previously
+	  stated that emailbody can only contain up to 512 characters which is
+	  always wrong.  The buffer is normally 8192 characters unless LOW_MEMORY
+	  is enabled then it is 512 characters.  The updated comment states that
+	  the line can be up to 8190 or 510 characters since the line feed and
+	  NULL terminator each use a character.
 
-	  * ast_cli_generatornummatches now retrieves the vector of matches and
-	    reports the number of elements (not including 'best' match).
-	  * test_substitution retrieves and iterates the vector.
+	  ASTERISK-26688 #close
 
-	  Change-Id: I8cd6b93905363cf7a33a2d2b0e2a8f8446d9f248
+	  Change-Id: I80864a0d40d2e2d8cd79d72af52a8f0a3a99c015
 
-2017-11-20 09:13 +0000 [41498dcb5d]  Alexander Traud <pabstraud@compuserve.com>
+2018-02-22 13:53 +0000 [bb9c1938a0]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_sip: ICE contained square brackets around IPv6 addresses.
+	* res_pjsip_refer.c: Fix attended transfer race condition crash.
 
-	  ASTERISK-27434
+	  The transferrer's session channel was destroyed by the transferrer's
+	  serializer thread in a race condition with the transfer target's
+	  serializer thread during an attended transfer.  The transfer target's
+	  serializer was attempting to clean up a deferred end status on behalf of
+	  the transferrer's channel when it should have passed the action to the
+	  transferrer's serializer.  When the transfer target's serializer lost the
+	  race then both threads wind up trying to end the transferrer's session.
 
-	  Change-Id: Iaeed89b4fa05d94c5f0ec2d3b7cd6e93d2d5a8f7
+	  * Push the ast_sip_session_end_if_deferred() call onto the transferrer's
+	  serializer to avoid a race condition that results in a crash.  The
+	  session_end() function that could be called by
+	  ast_sip_session_end_if_deferred() really must be executed by the
+	  transferrer's serializer to avoid this kind of crash.
 
-2017-11-19 21:23 +0000 [84fd41729e]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27568
 
-	* loader: Fix comments in struct ast_module.
+	  Change-Id: Iacda724e7cb24d7520e49b2fd7e504aa398d7238
 
-	  Make the comments follow doxygen format, move comments to the line
-	  before each field they describe.
+2018-02-17 03:28 +0000 [c4c5d00528]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ic445468398b5e88f13910f7c2f70bd15aad33a27
+	* install_prereq: Update FreeBSD libraries.
 
-2017-11-16 17:25 +0000 [df6161b70e]  Corey Farrell <git@cfware.com>
+	  deleted
+	   autoconf gcc libsamplerate sqlite
 
-	* cli: Remove silly usage of RAII_VAR.
+	  changed
+	   binutils to libbfd
+	   freetds-devel to freetds
+	   gmime2 to gmime26
+	   mysql55-client to mysql57-client
 
-	  Change-Id: I81aacfee7cd26e4fc5eef07bca582700c2975bd7
+	  added
+	   alsa-lib bison bzip2 cclient corosync doxygen libedit flex graphviz
+	   libhoard libical libilbc libltdl lua neon newt net-snmp
+	   openldap-client openssl patch pkgconf portaudio postgresql10-client
+	   python radcli speexdsp subversion uriparser xmlstarlet libzip
 
-2017-11-16 13:19 +0000 [1e2b38adf5]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27686
 
-	* ccss: Remove silly usage of RAII_VAR.
+	  Change-Id: Ibe88c9b26e59c30d26cdb313a3ef01c9f37ac80d
 
-	  Change-Id: I5ce40035e0a940e4e56f6322c1dcd47fbd509b98
+2018-02-22 10:54 +0000 [50d9af101e]  Sean Bright <sean.bright@gmail.com>
 
-2017-11-16 12:51 +0000 [ddc86ec664]  Corey Farrell <git@cfware.com>
+	* func_audiohookinherit: Remove deprecated module.
 
-	* app: Remove silly usage of RAII_VAR.
+	  Change-Id: Id52f719078a65c4b2eee7ab99d761eba6b6aed94
 
-	  Change-Id: Ideb594f7aae134974fb78d5477ba0853b97b8625
+2018-02-21 12:52 +0000 [f083edc43c]  Richard Mudgett <rmudgett@digium.com>
 
-2017-11-16 12:19 +0000 [0e90d74604]  Corey Farrell <git@cfware.com>
+	* manager.c: Fix lseek() parameter order.
 
-	* aoc: Remove silly usage of RAII_VAR.
+	  ASTERISK-27659
 
-	  Change-Id: I07907f833b81aeb0128bc9442a2abb52679c7511
+	  Change-Id: I04a2705d2cb7df250769967bc59e2b397a49b797
 
-2017-11-16 12:55 +0000 [746dd80eab]  Corey Farrell <git@cfware.com>
+2018-02-20 13:11 +0000 [39f733406d]  Richard Mudgett <rmudgett@digium.com>
 
-	* abstract_jb: Remove silly usage of RAII_VAR.
+	* bridge_simple.c: Fix stream topology handling.
 
-	  Change-Id: I9d56175369363d1dc735504cf78a3a5577069f49
+	  The handling of stream topologies was not protected by channel locks in
+	  simple_bridge_request_stream_topology_change().
 
-2017-11-20 12:54 +0000 [8971a7ff3c]  Corey Farrell <git@cfware.com>
+	  * Fixed topology handling to be protected by channel locks where needed in
+	  simple_bridge_request_stream_topology_change().
 
-	* Loader: Remove unneeded load_pri declarations.
+	  ASTERISK-27692
 
-	  Instead of specifying AST_MODFLAG_LOAD_ORDER with load_pri
-	  AST_MODPRI_DEFAULT just use AST_MODFLAG_DEFAULT.
+	  Change-Id: Ica5d78a6c7ecf4f0b95fb16de28d3889b32c4776
 
-	  Change-Id: I0123258eafce324249433a69df15a85cc16e509f
+2018-02-05 16:46 +0000 [6436137959]  Sean Bright <sean.bright@gmail.com>
 
-2017-11-20 13:08 +0000 [9d688cf12c]  Corey Farrell <git@cfware.com>
+	* AST-2018-006: Properly handle WebSocket frames with 0 length payload.
 
-	* res_mwi_external_ami: Remove incorrect load priority.
+	  In ast_websocket_read() we were not adequately checking that the
+	  payload_len was non-zero before passing it to ws_safe_read(). Calling
+	  ws_safe_read with a len argument of 0 will result in a busy loop until
+	  the underlying socket is closed.
 
-	  res_mwi_external_ami specified AST_MODFLAG_LOAD_ORDER but didn't set
-	  load_pri, resulting in an actual load priority of 0.  This module only
-	  provides AMI actions so it has no reason to load early.
+	  ASTERISK-27658 #close
 
-	  Change-Id: I82987fcf10d3ea42716b2f9df915b16687fd5839
+	  Change-Id: I9d59f83bc563f711df1a6197c57de473f6b0663a
 
-2017-11-20 09:49 +0000 [50cb5199c1]  Alexander Traud <pabstraud@compuserve.com>
+2018-01-31 13:37 +0000 [880c69f00f]  Kevin Harwell <kharwell@digium.com>
 
-	* BuildSystem: pjsip_evsub_set_uas_timeout was not used.
+	* AST-2018-003: Crash with an invalid SDP fmtp attribute
 
-	  ASTERISK-27435
+	  pjproject's fmtp retrieval function failed to catch invalid fmtp attributes.
+	  Because of this Asterisk would crash if given an SDP with an invalid fmtp
+	  attribute.
 
-	  Change-Id: Id318a7ae6d7d69b53f911d30bf3eece64852f15c
+	  When retrieving the format this patch now makes sure the fmtp attribute is
+	  available. If not available it now returns an error status.
 
-2017-11-19 13:52 +0000 [6a5ab65c88]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27583 #close
 
-	* Build: Fix issues building without SSL.
+	  Change-Id: I5cebe000ce2d846cae3af33b6d72c416e51caf2f
 
-	  * Fix conditional in libasteriskssl.
-	  * Use variables produced by configure to link the SSL and uuid libraries
-	    into libasteriskpj.so instead of hard-coding them.
+2018-01-31 13:33 +0000 [d3a398cf90]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-27431
+	* AST-2018-002: Crash with an invalid SDP media format description
 
-	  Change-Id: I3977931fd3ef8c4e4376349ccddb354eb839b58d
+	  pjproject's media format parsing algorithm failed to catch invalid values.
+	  Because of this Asterisk would crash if given an SDP with a invalid media
+	  format description.
 
-2017-11-19 13:28 +0000 [366cc259bc]  Corey Farrell <git@cfware.com>
+	  When parsing the media format description this patch now properly parses the
+	  value and returns an error status if it can't successfully parse/convert the
+	  value.
 
-	* res_pjsip: Fix warning by deferring implicit type cast.
+	  ASTERISK-27582 #close
 
-	  Mac doesn't like the comparison of -1 to an enum, so store the result of
-	  ast_sip_str_to_dtmf to an int so we can check for the negative return
-	  value.  ast_sip_str_to_dtmf returns an int so this is only delaying the
-	  implicit type cast.
+	  Change-Id: I883b3a4ef85b6972397f7b56bf46c5779c55fdd6
 
-	  Change-Id: I0c262c1719ee951aae1f437d733a301cf5f8ad29
+2018-02-06 12:07 +0000 [758409de56]  George Joseph <gjoseph@digium.com>
 
-2017-11-19 09:57 +0000 [69113388e9]  Corey Farrell <git@cfware.com>
+	* AST-2018-005: res_pjsip_transport_management:  Move to core
 
-	* Build: Fix OSX build issues.
+	  Since res_pjsip_transport_management provides several attack
+	  mitigation features, its functionality moved to res_pjsip and
+	  this module has been removed.  This way the features will always
+	  be available if res_pjsip is loaded.
 
-	  OSX does not support 'readlink -f' or 'sed -r'.  Replace readlink with
-	  the GNU make macro 'realpath'.  Replace sed with grep in one place, cut
-	  in the other.
+	  ASTERISK-27618
+	  Reported By: Sandro Gauci
 
-	  ASTERISK-27332
+	  Change-Id: I21a2d33d9dda001452ea040d350d7a075f9acf0d
 
-	  Change-Id: I5d34ecca905384decb22ead45c913ae5e8aff748
+2018-02-06 11:28 +0000 [de871515ba]  George Joseph <gjoseph@digium.com>
 
-2017-11-18 21:13 +0000 [5fe2e7bfdc]  Corey Farrell <git@cfware.com>
+	* AST-2018-005: Fix tdata leaks when calling pjsip_endpt_send_response(2)
 
-	* tests: Fix warnings found on Mac.
+	  pjsip_distributor:
+	     authenticate() creates a tdata and uses it to send a challenge or
+	     failure response.  When pjsip_endpt_send_response2() succeeds, it
+	     automatically decrements the tdata ref count but when it fails, it
+	     doesn't.  Since we weren't checking for a return status, we weren't
+	     decrementing the count ourselves on error and were therefore leaking
+	     tdatas.
 
-	  test_pbx used raise without explicitly including signal.h.  On Mac for
-	  some reason nothing else includes it.
+	  res_pjsip_session:
+	     session_reinvite_on_rx_request wasn't decrementing the ref count
+	     if an error happened while sending a 491 response.
+	     pre_session_setup wasn't decrementing the ref count if
+	     while sending an error after a pjsip_inv_verify_request failure.
 
-	  test_logger checked if an unsigned int was negative.  Switch the
-	  variable to 'int' so that error check can be effective.
+	  res_pjsip:
+	     ast_sip_send_response wasn't decrementing the ref count on error.
 
-	  Change-Id: Ie1db5dd1818ac25cc2ae41b644f848b5865b1362
+	  ASTERISK-27618
+	  Reported By: Sandro Gauci
 
-2017-11-18 20:25 +0000 [d68e54b7ab]  Corey Farrell <git@cfware.com>
+	  Change-Id: Iab33a6c7b6fba96148ed465b690ba8534ac961bf
 
-	* res_snmp: Declare RONLY if net-snmp headers do not.
+2018-02-06 11:21 +0000 [c53d8dcb68]  George Joseph <gjoseph@digium.com>
 
-	  Some net-snmp builds do not provide the RONLY declare, only
-	  NETSNMP_OLDAPI_RONLY.  Map RONLY to NETSNMP_OLDAPI_RONLY to get around
-	  this error.
+	* AST-2018-005: Add a check for NULL tdata in ast_sip_failover_request
 
-	  Change-Id: Ida5c7ad9406515825485c4d3b4a34fd6ad0da577
+	  It was discovered that there are some corner cases where a pjsip tsx
+	  might have no last_tx so calling ast_sip_failover_request with
+	  a NULL last_tx as its tdata would cause a crash.
 
-2017-11-18 20:02 +0000 [0f270cbe9e]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27618
+	  Reported By:  Sandro Gauci
 
-	* res_fax: Remove checks for unsigned values being >= 0.
+	  Change-Id: Ic2b63f6d4ae617c4c19dcdec2a7a6156b54fd15b
 
-	  It's impossible for gwtimeout or fdtimeout to be less than 0 because
-	  they are unsigned int's.  Remove checks and unreachable branches.
+2018-02-07 08:09 +0000 [d424850d58]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ib2286960621e6ee245e40013c84986143302bc78
+	* AST-2018-004: Restrict the number of Accept headers in a SUBSCRIBE.
 
-2017-11-18 19:29 +0000 [3dca4c7742]  Corey Farrell <git@cfware.com>
+	  When receiving a SUBSCRIBE request the Accept headers from it are
+	  stored locally. This operation has a fixed limit of 32 Accept headers
+	  but this limit was not enforced. As a result it was possible for
+	  memory outside of the allocated space to get written to resulting
+	  in a crash.
 
-	* app_minivm: Fix possible uninitialized return value.
+	  This change enforces the limit so only 32 Accept headers are
+	  processed.
 
-	  Declare 'res' initialized to -1 to deal with earlier error paths that
-	  could cause 'res' to be returned uninitialized.
+	  ASTERISK-27640
+	  Reported By: Sandro Gauci
 
-	  Change-Id: I8ac2a5755bf4174d89ef893e924c940f702b104e
+	  Change-Id: I99a814b10b554b13a6021ccf41111e5bc95e7301
 
-2017-11-17 19:36 +0000 [5f0b4a1645]  Corey Farrell <git@cfware.com>
+2018-01-13 08:04 +0000 [e70c4ec84d]  Joshua Colp <jcolp@digium.com>
 
-	* README: Send people to secure websites where available.
+	* AST-2018-001: rtp / channel: Don't allow an unnegotiated format to be passed up.
 
-	  We should be sending people to secure web URL's where available.
-	  Update README's and docs.
+	  When an RTP packet is received by an RTP engine it has to map the
+	  payload into the Asterisk format. The code was incorrectly checking
+	  our own static list for ALL payloads if it couldn't find a negotiated one.
+	  This included dynamic payloads. If the payload mapped to a format
+	  of a different type (for example receiving a video packet on an audio
+	  RTP instance) then the core stream code could cause a crash if a legacy
+	  channel driver was in use as no stream would be present.
 
-	  Change-Id: Id5b1e049b0b18b49a784f1254605aefa244ce19a
+	  To provide further protection the core stream code will no longer assume
+	  that a video or audio frame will always have a stream for legacy channel
+	  drivers. If no stream is present the frame is dropped.
 
-2017-11-17 19:54 +0000 [a369bfc4b2]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27488
 
-	* doxygen: Remove obsolete contents.
+	  Change-Id: I022556f524ad8379ee73f14037040af17ea3316a
 
-	  Remove doxygen contents that have nothing to do with the current state
-	  of Asterisk.
+2018-02-20 13:11 +0000 [e2f98fbd63]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ic072cc8641f9533a202990ccf275ce87e3efd95c
+	* channel.c: Fix typo.
 
-2017-11-16 02:47 +0000 [6b0963c42a]  Pirmin Walthert <infos@nappsoft.ch>
+	  Change-Id: I4eeedf89085697e81c354eb92d546686c67b0b5b
 
-	* res_rtp_asterisk.c: Fix rtp source address learning for broken clients
+2018-02-20 10:33 +0000 [259c80675e]  Joshua Colp <jcolp@digium.com>
 
-	  Some clients do not send rtp packets every ptime ms. This can lead to
-	  situations in which the rtp source learning algorithm will never learn
-	  the address of the client. This has been discovered on a Mac mini with
-	  a pjsip based softphone after updating to Sierra: as soon as USB
-	  headsets are involved, the softphone will send the second packet 30ms
-	  after the first, the third 30ms after the second and the fourth 1ms
-	  after the third. So in the old implmentation the rtp source learning
-	  algorithm was repeatedly reset on the fourth packet.
+	* chan_sip: Emit a second ringing event to ensure channel is found.
 
-	  The patch changes the algorithm in a way that doesn't take the arrival
-	  time between two consecutive packets into account but the time between
-	  the first and the last packet of a learning sequence.
+	  When constructing a dialog-info+xml NOTIFY message a ringing channel
+	  is found if the state is ringing and further information is placed into
+	  the message. Due to the migration to the Stasis message bus this did
+	  not always work as expected.
 
-	  The patch also fixes a second problem: when a user was using a wrong
-	  value for the probation setting there was a LOG_WARNING output stating
-	  that the value had been set to the default value instead. However
-	  the code for setting the value back to defaults was missing.
+	  This change raises a second ringing event in such a way to guarantee
+	  that the event is received by chan_sip and another lookup is done to
+	  find the ringing channel.
 
-	  ASTERISK-27421 #close
+	  ASTERISK-24488
 
-	  Change-Id: If778fe07678a6fd2041eaca7cd78267d0ef4fc6c
+	  Change-Id: I547a458fc59721c918cb48be060cbfc3c88bcf9c
 
-2017-11-17 09:57 +0000 [3c72064d3f]  Sean Bright <sean.bright@gmail.com>
+2018-02-20 04:31 +0000 [0ad13949c1]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip: Use reasonable buffer lengths for endpoint identification
+	* doc/lang/language-criteria.txt: Link to wiki.
 
-	  Domains themselves can be up to 255 characters long (per RFC 1035), so
-	  our current buffer sizes are wholly inadequate for many use cases.
+	  This document is out of date and is superseded by content on the
+	  Asterisk wiki.
 
-	  Change-Id: If3f30a68307f1365a1fe06bc4b854c62842c9292
+	  ASTERISK-24386 #close
 
-2017-11-11 10:09 +0000 [455b1bdeb0]  Corey Farrell <git@cfware.com>
+	  Change-Id: Idbf95b27b096c205251e1bbb560c79224ba81822
 
-	* menuselect: Remove ineffective weak attribute detection.
+2018-02-19 04:21 +0000 [4b555d7147]  Thomas Guebels <tgu@escaux.com>
 
-	  menuselect detects compiler support for multiple styles of weak
-	  functions.  This is a remnant from 2013 when OPTIONAL_API required weak
-	  functions.  It is no longer correct for menuselect to switch
-	  dependencies from optional to required based on lack of weak function
-	  support.
+	* res_rtp_asterisk: Fix ICE candidate nomination
 
-	  Note an issue remains - dependencies should switch from optional to
-	  required based on OPTIONAL_API being enabled or disabled.  I don't think
-	  this is possible.  menuselect needs to know at startup if OPTIONAL_API
-	  is enabled or disabled, so the only way to fix this is to remove
-	  OPTIONAL_API from menuselect and create a configure option.  I've left
-	  the code that switches in place but it's preprocessed out.
+	  If the ICE role is not set right away, we might have a role conflict
+	  that stays undetected and ICE finishing with successful tests and no
+	  candidate nominated. This was introduced by ASTERISK-27088.
 
-	  Additionally removed:
-	  - WEAKREF variable from Asterisk makeopts.in.
-	  - Related disabled code from test_utils.
-	  - Pointless AC_REVISION call from menuselect/configure.ac.
+	  To avoid this, we set the role as soon as before but only if the ICE
+	  state permits it: still checking and not yet nominating candidates or
+	  completed.
 
-	  Change-Id: Ifa702e5f98eb45f338b2f131a93354632a8fb389
+	  ASTERISK-27646
 
-2017-11-16 09:48 +0000 [549542e50e]  Corey Farrell <git@cfware.com>
+	  Change-Id: I5dbc69ad63cacbb067922850fbb113d479bd729c
 
-	* acl: Fix allocation related issues.
+2018-02-18 10:27 +0000 [8b18247af6]  Sean Bright <sean.bright@gmail.com>
 
-	  Add checks for allocation errors, cleanup and report failure when they
-	  occur.
+	* res_http_websocket: Don't leak memory on read failure
 
-	  * ast_duplicate_acl_list: Replace log warnings with errors, add missing
-	    line-feed.
-	  * ast_append_acl: Add missing line-feed to logger message.
-	  * ast_append_ha: Avoid ast_strdupa in loop by moving debug message to
-	    separate function.
-	  * ast_ha_join: Use two separate calls to ast_str_append to avoid using
-	    ast_strdupa in a loop.
+	  Change-Id: Ic449ea832bc81a1671c0e910c5fbe8c683e3da89
 
-	  Change-Id: Ia19eaaeb0b139ff7ce7b971c7550e85c8b78ab76
+2018-02-19 03:57 +0000 [97c21e9cb3]  Corey Farrell <git@cfware.com>
 
-2017-11-16 11:02 +0000 [2852dac480]  Corey Farrell <git@cfware.com>
+	* core: Rename sounds_index.c to sounds.c.
 
-	* acl: Update logger message to match 15+.
+	  This will make the source filename match the 'module reload sounds'
+	  command.  This will allow conversion to a built-in module in Asterisk 16
+	  without needing to redefine AST_MODULE.
 
-	  This patch causes a logger message to be the same as it is in 15+.  This
-	  will allow a follow-up patch to be cherry-picked to all 3 branches.
+	  Change-Id: Ifb8e489575b27eb33d8c0b6a531f266670557f6e
 
-	  Change-Id: Ic0665a3d49987e4eb6df28dcd9e90b1c3ca191e0
+2018-02-19 02:49 +0000 [e03f0f9572]  Corey Farrell <git@cfware.com>
 
-2017-11-07 17:07 +0000 [0b8b153d3c]  Corey Farrell <git@cfware.com>
+	* config: Fix locking for extconfig reload.
 
-	* CLI: Create ast_cli_completion_vector.
+	  Expand locking to include full reload process for extconfig to ensure
+	  nothing can read the config mappings between clearing and reloading.
 
-	  This is a rewrite of ast_cli_completion_matches using a vector to build
-	  the list.  The original function calls the vector version, NULL
-	  terminates the vector and extracts the elements array.
+	  Change-Id: I378316bad04f1b599ea82d0fef62b8978a644b92
 
-	  One change in behavior the results are now sorted and deduplicated. This
-	  will solve bugs where some duplicate checking was done before the list
-	  was sorted.
+2018-02-15 14:09 +0000 [e4a5c9ccf4]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Iede20c5b4d965fa5ec71fda136ce9425eeb69519
+	* res_pjsip_header_funcs: Various cleanups
 
-2017-11-16 09:04 +0000 [0a7bbb068b]  Joshua Colp <jcolp@digium.com>
+	   * Prefer strcasecmp() over stricmp()
+	   * Use a list with no lock since we never actually lock
+	   * Minor cleanups to error messages
 
-	* bridge_basic: Ignore answer from transfer target when they've timed out.
+	  Change-Id: I8446f44795ee8f3072e1c1f9193c6912dfc0c42b
 
-	  This is a fun one.
+2018-02-17 08:49 +0000 [a70c92121d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Given the following attended transfer scenario:
+	* rtp_engine: Load format name / mime type in uppercase again.
 
-	  1. Transfer target is called
-	  2. Transferer hangs up
-	  3. Transfer target call attempt reaches timeout
-	  4. Transfer target is told to hang up
-	  5. Transfer target answers before channel is hung up
-	  6. Transferer recall target is called
+	  This reverts a previous change partly.
 
-	  A crash would occur. This is because the transfer target call
-	  attempt, despite being told to hang up, would raise a recall
-	  target answer before the recall target had been answered. As it
-	  had not answered there would be no recall target channel and it
-	  would implode.
+	  ASTERISK-27689
 
-	  This change makes it so that if the transfer target has been
-	  hung up we don't tell the attended transfer code that it has
-	  answered. We also clear out the stimulus that the recall target
-	  has been answered after telling the transfer target to hang up,
-	  in case it was able to raise the information before we told it
-	  to hangup.
+	  Change-Id: Ia3d2f282db6995be8c1c253b5d52f6038761e8af
 
-	  ASTERISK-27361
+2018-02-16 17:58 +0000 [525c0251c0]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ifb8b255a9c4d2c5c1b8ad77bf54f659ed286df99
+	* BuildSystem: Use single bootstrap.sh for Asterisk and menuselect.
 
-2017-11-16 19:39 +0000 [69055724ce]  Corey Farrell <git@cfware.com>
+	  This causes the root bootstrap.sh script to generate configure scripts
+	  for both Asterisk and menuselect.  This ensures that both configure
+	  scripts are generated with the same version of autotools and avoids
+	  situations where shared autoconf macros get modified without
+	  regenerating the menuselect script.
 
-	* aoc: Fix memory management issues.
+	  Change-Id: I2bfd8537bbb63b3d46b11efabbb15eaaf9ef731a
 
-	  aoc_publish_blob failed to check for msg allocation error and never
-	  released msg.
+2018-02-16 13:33 +0000 [65a4084060]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ib31a9ffb81056a0d496a49d7eec795005a44bcd5
+	* res_pjsip: Endpoint destruction does not free DTLS configuration
 
-2017-11-16 16:18 +0000 [60cfe00c4e]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27679 #close
+	  Reported by: Mak Dee
 
-	* res_pjsip_transport_websocket: Give transport a meaningful description
+	  Change-Id: I89a2783a11be0763bf123d1619ed176b6225cf42
 
-	  We were not \0 terminating this string, so any attempt to print it would
-	  in the best case show an empty string and in the worst case potentially
-	  crash.
+2018-02-16 12:42 +0000 [a7e7302ab6]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I63d96ef8f7516ac02a0f91e22dfa8acdc615042c
+	* install_prereq: Update OpenBSD libraries.
 
-2017-11-16 15:00 +0000 [db2677133c]  Sean Bright <sean.bright@gmail.com>
+	  deleted
+	   jack sqlite
 
-	* res_pjsip: Use sorcery prefix operation for contact lookup
+	  renamed
+	   freetds-0.63p1-msdblib to freetds
+	   mysql-client to mariadb-client
 
-	  This improves performance for registrations assuming that
-	  res_config_astdb is not in use.
+	  added
+	   bison bzip2 c-client doxygen e2fsprogs graphviz gsm libical jansson libltdl
+	   lua neon net-snmp libsrtp portaudio-svn postgresql-client python speexdsp
+	   subversion uriparser xmlstarlet
+	   fftw3 libsndfile
 
-	  Change-Id: I86f37aa9ef07a4fe63448cb881bbadd996834bb1
+	  ASTERISK-27684
 
-2017-11-09 19:58 +0000 [8a7dd5cc44]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I26bdcb0a1d0e484a8dad1052da97f194aefd3370
 
-	* chan_pjsip.c: Improve answer failure log messages.
+2018-02-16 12:30 +0000 [14796f529e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Balanced the session->inv_session refs on answer failure.
+	* BuildSystem: Allow newer autotools on OpenBSD.
 
-	  Change-Id: I33542d639d37e692cb46550b972a5fcfc3b804b8
+	  ASTERISK-27683
 
-2017-11-14 18:00 +0000 [7f916d621a]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I5ec9dafbb0c16b6f2740c641980bc2eaaf995624
 
-	* audiohook.c: Fix freeing a frame and still using it.
+2017-10-16 07:36 +0000 [976afd26ab]  Torrey Searle <torrey@voxbone.com>
 
-	  Memory corruption happened to the media frame caches when an audio hook
-	  freed a frame when it shouldn't.  I think the freed frame was because a
-	  jitter buffer interpolated a missing frame and the audio hook
-	  unconditionally freed it.
+	* contrib/script/sip_to_pjsip: add support for realtime
 
-	  * Made audiohook.c:audio_audiohook_write_list() not free an interpolated
-	  frame if it is the same frame as what was passed into the routine.
+	  Add a new script that can read from legacy realtime peers & generate
+	  an sql file for populating pjsip endpoints, identify, and aor records.
 
-	  * Made plc.c:normalise_history() use memmove() instead of memcpy() on a
-	  memory block that could overlap.  Found by valgrind investigating this
-	  issue.
+	  ASTERISK-27348 #close
 
-	  ASTERISK-27238
-	  ASTERISK-27412
+	  Change-Id: Idd3d7968a3c9c3ee7936d21acbdaf001b429bf65
 
-	  Change-Id: I548d86894281fc4529aefeb9f161f2131ecc6fde
+2018-02-16 07:52 +0000 [dda73c5018]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-11-15 12:10 +0000 [7e874eae7a]  George Joseph <gjoseph@digium.com>
+	* BuildSystem: Fix a typo related to ./configure --prefix=<path> on OpenBSD.
 
-	* app_record:  Don't set RECORD_STATUS chan var until file is closed
+	  Reported by: Stuart Henderson
 
-	  We've been calling pbx_builtin_setvar_helper to set the
-	  RECORD_STATUS variable before actually closing the recorded file.
-	  If a client is watching VarSet events and tries to do something with
-	  the file when a RECORD_STATUS event is seen, they might attempt to
-	  do so while the file it's still open.
+	  Change-Id: Ieae8624f48b6ae78cf29930b9a45a3c842c7a764
 
-	  We now delay calling pbx_builtin_setvar_helper until after we close
-	  the file.
+2018-02-16 05:58 +0000 [5fd59014a5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27423
+	* res_calendar: Specialized calendars depend on symbols of general calendar.
 
-	  Change-Id: I7fe9de99953e46b4bafa2b38cf151fe8f6488254
+	  ASTERISK-27680
 
-2017-11-07 08:25 +0000 [062a4390ac]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ifb77912e424fe3710a025c18526fada673ec0b79
 
-	* ast_coredumper:  Add ability to use directory other than /tmp
+2018-02-16 06:41 +0000 [c674efa996]  Alexander Traud <pabstraud@compuserve.com>
 
-	  The OUTPUTDIR environment variable can now be set either in the
-	  environment itself or in ast_debug_tools.conf.  If set, it's used
-	  for all work products instead of /tmp.
+	* BuildSystem: Enable IMAP storage on OpenBSD.
 
-	  Also added the --tarball-config option that includes the contents
-	  of /etc/asterisk when either --tarball-coredumps or --tarball-results
-	  are used.
+	  ASTERISK-27681
+	  Reported by: Stuart Henderson
 
-	  Change-Id: I66b2553319df61caea5b313d084f51978f730b4c
+	  Change-Id: Ifb6b614acb251b695b9417d76510e73eb335b679
 
-2017-11-13 07:14 +0000 [c2dddb001a]  Joshua Colp <jcolp@digium.com>
+2018-02-16 04:50 +0000 [2c814afb86]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pjsip / hep: Provide correct local address for Websockets.
+	* BuildSystem: Enable system provided libedit on OpenBSD.
 
-	  Previously for PJSIP the local address of WebSocket connections
-	  was set to the remote address. For logging purposes this is
-	  not particularly useful.
+	  ASTERISK-27677
 
-	  The WebSocket API has been extended to allow the local
-	  address to be queried and this is used in PJSIP to set the
-	  local address to the correct value.
+	  Change-Id: I0854e3616d1361ae9b6907d3d3444a02784ac62b
 
-	  The PJSIP HEP support has also been tweaked so that reliable
-	  transports always use the local address on the transport
-	  and do not try to (wrongly) guess. As they are connection
-	  based it is impossible for the source to be anything else.
+2018-02-15 14:30 +0000 [af2dd3a678]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26758
-	  ASTERISK-27363
+	* bridge_roles: Use a non-locking linked list where appropriate
 
-	  Change-Id: Icd305fd038ad755e2682ab2786e381f6bf29e8ca
+	  Also explicitly initialize with the AST_LIST_HEAD_NOLOCK_INIT macro for
+	  clarity.
 
-2017-11-13 17:47 +0000 [b8209a1273]  Corey Farrell <git@cfware.com>
+	  Change-Id: I4bc39ec33bc3ff77e1a971a01ace87deb965be3f
 
-	* alertpipe: Correct documented return of ast_alertpipe_write.
+2018-02-15 13:29 +0000 [303e43f8a6]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I4ea49c441890a81384144479dc93ab5a3989486d
+	* res_pjsip: Use pjsip_sip_uri.user_param instead of other_param
 
-2017-11-13 16:20 +0000 [9c6d4ec022]  Corey Farrell <git@cfware.com>
+	  There is a dedicated slot in the pjsip_sip_uri for the 'user'
+	  parameter, so use that instead of adding to the list of generic URI
+	  parameters.
 
-	* core: Fix configuration of remote console socket path.
+	  Change-Id: I0a0ce8a60ecee27489735bf56fd707719d8c2ed6
 
-	  The remote console socket path is the combination of asterisk.conf
-	  settings astrundir from [directories] and astctl from [files].
-	  Unconditionally combine the two strings after processing all values
-	  to ensure we end up with the correct socket path.
+2018-02-12 07:37 +0000 [8ac198aff3]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27415
+	* BuildSystem: Remove chan_h323 leftovers.
 
-	  Change-Id: Ib1e2805d55d6b0955c6430a1a2a93acbf9b091e8
+	  ASTERISK-27670
 
-2017-11-10 10:37 +0000 [b5f2779a23]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I07a8ef8bbd6001e25711fa1bff152eb6c9efa729
 
-	* bundled_pjproject: sip_parser:  Fix return code in pjsip_find_msg
+2018-01-17 08:17 +0000 [6b6b3ffa5b]  Alexander Traud <pabstraud@compuserve.com>
 
-	  The default return code for pjsip_find_msg was PJ_SUCCESS so if
-	  a Content-Length header wasn't found at all, pjsip_find_msg was
-	  returning PJ_SUCCESS instead of PJSIP_EMISSINGHDR.
+	* BuildSystem: Invoke ldconfig with previous path.
 
-	  Also added the volatile keyword to a few variables that are used
-	  both inside and outside the PJ_TRY/PJ_CATCH block.
+	  On OpenBSD, gmake uninstall{-all} registered only libraries from /usr/lib and
+	  lost those from /usr/local/lib. Instead, invoke ldconfig on a path.
 
-	  Partial fix for ASTERISK_27408
+	  ASTERISK-27595
 
-	  Change-Id: If82ba9de921e3d57df9c68cf96ee45ccc1491f7a
+	  Change-Id: I4aa2c0b5e07119d1a556f8ff6349eaf09e986888
 
-2017-11-13 14:35 +0000 [e6ada55430]  Ben Ford <bford@digium.com>
+2017-12-22 15:27 +0000 [9f74afbdcf]  Corey Farrell <git@cfware.com>
 
-	* bundled_pjproject: Update to 2.7.1
+	* Deprecate legacy modules.
 
-	  Update from 2.7 to 2.7.1 for bundled pjproject. Changed version
-	  and removed patch files included in the update.
+	  * app_fax (replaced by res_fax).
+	  * res_config_sqlite (replaced by res_config_sqlite3).
+	  * res_monitor (replaced by app_mixmonitor).
 
-	  Change-Id: I55cea8e734b318c2df9daf86aa0802c559ec8357
+	  This is related to ASTERISK~23657 but does not resolve that ticket.
+	  Resolving that ticket would require complete removal of res_monitor.
 
-2017-11-09 08:21 +0000 [cf062303e3]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27671 #close
 
-	* sorcery: Add ast_sorcery_retrieve_by_prefix()
+	  Change-Id: I16a3edd61fc1abd4a7b2e9357693ed663f62dd49
 
-	  Some consumers of the sorcery API use ast_sorcery_retrieve_by_regex
-	  only so that they can anchor the potential match as a prefix and not
-	  because they truly need regular expressions.
+2018-01-28 03:02 +0000 [f9ba31bb21]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Rather than using regular expressions for simple prefix lookups, add
-	  a new operation - ast_sorcery_retrieve_by_prefix - that does them.
+	* BuildSystem: Do not warn when bash is not installed.
 
-	  Patches against 13 and 15 have a compatibility layer needed to
-	  maintain ABI that is not needed in master.
+	  ASTERISK-27631
 
-	  Change-Id: I56f4e20ba1154bd52281f995c27a429a854f6a79
+	  Change-Id: Iefdf268b0b98c3e7d8089ba87cf78136ac1d785b
 
-2017-11-07 14:00 +0000 [2ad6210dd7]  Corey Farrell <git@cfware.com>
+2018-02-12 22:15 +0000 [9e45d3f893]  Corey Farrell <git@cfware.com>
 
-	* vectors: Add new macro and a string vector definition.
+	* main/asterisk.c: Remove silly usage of RAII_VAR.
 
-	  * AST_VECTOR_STEAL_ELEMENTS - steal the array of elements for use
-	    with non-vector code.
-	  * struct ast_vector_string - a vector of 'char *'.
+	  Change-Id: I7e2996397fbd3c3a6a69dd805c38448ddfc34ae9
 
-	  Change-Id: I104d1b204be03fccf67e02a195596adcb5ab1e42
+2017-12-25 19:32 +0000 [02ee296f81]  Corey Farrell <git@cfware.com>
 
-2017-10-30 22:09 +0000 [76e640bd53]  Corey Farrell <git@cfware.com>
+	* optional_api: Refactor to use vector's and standard allocators.
 
-	* Build: Make function constructor/destructor attributes mandatory.
+	  * Replace ad-hoc array management with macro's from vector.h.
+	  * Remove redundent logger messages.
+	  * Use normal Asterisk allocators instead of directly using libc
+	    allocators.
+	  * Free memory when an API has no implementation or users.
 
-	  This change causes the configure script to fail if the C compiler does
-	  not support both function attributes constructor and destructor.  These
-	  were already required as modules cannot function without these attributes
-	  and Asterisk requires modules.
+	  Change-Id: Ic6ecb31798d4a78e7df39ece86a68b60eac05bf5
 
-	  This also has AST_GCC_ATTRIBUTE set a variable
-	  ax_cv_have_func_attribute_$1.  This is the same variable name used by
-	  autoconf-archive's AX_GCC_FUNC_ATTRIBUTE, used for the same purpose.
+2018-02-11 15:27 +0000 [8372138cce]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Id68e8a1447f2a6d707c54b56350e7bfdb33fb663
+	* chan_sip.c: Fix crash processing CANCEL.
 
-2017-11-10 22:04 +0000 [b03d389bec]  Corey Farrell <git@cfware.com>
+	  Check if initreq data string exists before using it when processing a
+	  CANCEL request.
 
-	* menuselect: Delete and ignore aclocal.m4.
+	  ASTERISK-27666
 
-	  This file is temporary output from the bootstrap.sh command, it does not
-	  need to be committed.
+	  Change-Id: Id1d0f0fa4ec94e81b332b2973d93e5a14bb4cc97
 
-	  Change-Id: Ie0fd113aff6eac44924c0bd0c900833c6c86a6d9
+2018-02-06 14:27 +0000 [cb4cfb8c43]  Sungtae Kim <pchero21@gmail.com>
 
-2017-11-11 13:01 +0000 [507d9b5f9e]  Richard Mudgett <rmudgett@digium.com>
+	* manager: Add AMI event Load/Unload
 
-	* core: Add cache_media_frames debugging option.
+	  Add an AMI events Load and Unload for notify when the
+	  module has been loaded and unloaded.
 
-	  The media frame cache gets in the way of finding use after free errors of
-	  media frames.  Tools like valgrind and MALLOC_DEBUG don't know when a
-	  frame is released because it gets put into the cache instead of being
-	  freed.
+	  ASTERISK-27661
 
-	  * Added the "cache_media_frames" option to asterisk.conf.  Disabling the
-	  option helps track down media frame mismanagement when using valgrind or
-	  MALLOC_DEBUG.  The cache gets in the way of determining if the frame is
-	  used after free and who freed it.  NOTE: This option has no effect when
-	  Asterisk is compiled with the LOW_MEMORY compile time option enabled
-	  because the cache code does not exist.
+	  Change-Id: Ib916c41eddd63651952998f2f49c57c42ef87a64
 
-	  To disable the media frame cache simply disable the cache_media_frames
-	  option in asterisk.conf and restart Asterisk.
+2018-01-30 20:31 +0000 [04490fb1d8]  Corey Farrell <git@cfware.com>
 
-	  Sample asterisk.conf setting:
-	  [options]
-	  cache_media_frames=no
+	* json: Add conditionals to avoid locking if Jansson is thread safe.
 
-	  ASTERISK-27413
+	  Jansson is thread safe for all read-only functions and reference
+	  counting starting v2.11.  This allows simplification of our code and
+	  removal of locking around reference counting and dumping.
 
-	  Change-Id: I0ab2ce0f4547cccf2eb214901835c2d951b78c00
+	  Change-Id: Id985cb3ffa6681f9ac765642e20fcd187bd4aeee
 
-2017-11-11 09:42 +0000 [bb2173275a]  Richard Mudgett <rmudgett@digium.com>
+2018-02-12 06:16 +0000 [b21915bd1c]  Alexander Traud <pabstraud@compuserve.com>
 
-	* frame.c: Make ast_frame_free()/ast_frfree() NULL tolerant
+	* pjproject_bundled: Disable G.729 from Belledonne Communications.
 
-	  Change-Id: Ic49d821ef88ada38a31bdd835b9531443c55d793
+	  When <http://github.com/BelledonneCommunications/bcg729> is installed, PJProject
+	  tries to link that. Support for this bcg729 was added with PJProject 2.7. The
+	  issue happens, because Teluu enabled that new feature on default.
 
-2017-11-10 07:06 +0000 [360d50d74c]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27584
+	  Reported by: Stuart Henderson
 
-	* pjsip: Add patch to allow all transports to be destroyed.
+	  Change-Id: I88b6b18ad777bcfe2d8201187b4b90eec0a172a6
 
-	  If a transport is created with the same transport type, source
-	  IP address, and source port as one that already exists the old
-	  transport is moved into a linked list called "tp_list".
+2018-02-12 05:38 +0000 [97f45d5816]  Alexander Traud <pabstraud@compuserve.com>
 
-	  If this old transport is later shutdown it will not be destroyed
-	  as the process checks whether the transport is valid or not. This
-	  check does not look at the "tp_list" when making the determination
-	  causing the transport to not be destroyed.
+	* codecs: Add support for WebRTC iLBC 2.0.
 
-	  This change updates the logic to query not just the main storage
-	  method for transports but also the "tp_list".
+	  When the latest version of that library was installed, Asterisk did not build.
 
-	  Upstream issue https://trac.pjsip.org/repos/ticket/2061
+	  ASTERISK-27669
+	  Reported by: Николай Михо
 
-	  ASTERISK-27411
+	  Change-Id: I27e09bb875fdd56423bd9fae1be85fddb428eb96
 
-	  Change-Id: Ic5c2bb60226df0ef1c8851359ed8d4cd64469429
+2018-02-12 01:26 +0000 [9fddc8b4dc]  Corey Farrell <git@cfware.com>
 
-2017-11-09 20:34 +0000 [211f5d8dd4]  Corey Farrell <git@cfware.com>
+	* core: Remove embedded editline.
 
-	* core: Remove disabled code.
+	  This removes the embedded copy of editline from the Asterisk source
+	  tree, making a system copy of libedit mandatory in Asterisk 16+.
 
-	  handle_quit has been disabled since 2003, remove it.
+	  ASTERISK-27634 #close
 
-	  Change-Id: Idc3aaa6c81676160547078f9b71e8aa43de2db18
+	  Change-Id: Iedb64ad92acb78419f3caefedaa2bb7cd2a1a33f
 
-2017-11-09 13:24 +0000 [d12be40750]  Corey Farrell <git@cfware.com>
+2018-01-30 09:58 +0000 [32e610d9e6]  Alexander Traud <pabstraud@compuserve.com>
 
-	* Build System: Disable parallel make in the root Makefile.
+	* backtrace: Avoid potential spurious output.
 
-	  This ensures that the root Makefile runs only a single target at a time.
-	  SUBMAKE will still honor requested parallelism, so 'make -j8' will build
-	  one directory at a time but allow 8 jobs at once when building a sub
-	  directory.
+	  clang 4.0 found this via -Wlogical-not-parentheses.
 
-	  This will fix some display glitches related to rebuild of XML
-	  documentation.  It will also prevent some edge case errors where
-	  bundled pjproject needs to be rebuild before other parts of Asterisk.
+	  ASTERISK-27642
 
-	  Change-Id: I4f2ec6fbbec1ada0ccb1109a28ea303524239b1e
+	  Change-Id: I9ec3e144d425a976c02811bd23cd0c533d2eca4e
 
-2017-03-29 20:46 +0000 [32042c6c3c]  Richard Mudgett <rmudgett@digium.com>
+2018-02-10 05:39 +0000 [971378bbdb]  Alexander Traud <pabstraud@compuserve.com>
 
-	* chan_pjsip.c: Fix uninitialized cause value on failure.
+	* install_prereq: Update Debian/Ubuntu libraries.
 
-	  Change-Id: I3f9dd3c31bd582e54a30381500077de2319d8cc3
+	  ASTERISK-27555
 
-2017-10-19 13:35 +0000 [4b3e03ae87]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: Idc36e91db30c0163c560d04c5a82bca5d6ce92a8
 
-	* AST-2017-011 - res_pjsip_session: session leak when a call is rejected
-
-	  A previous commit made it so when an invite session transitioned into a
-	  disconnected state destruction of the Asterisk pjsip session object was
-	  postponed until either a transport error occurred or the event timer
-	  expired. However, if a call was rejected (for instance a 488) before the
-	  session was fully established the event timer may not have been initiated,
-	  or it was canceled without triggering either of the session finalizing states
-	  mentioned above.
+2018-02-09 12:06 +0000 [b2fcb30d38]  Richard Mudgett <rmudgett@digium.com>
 
-	  Really the only time destruction of the session should be delayed is when a
-	  BYE is being transacted. This is because it's possible in some cases for the
-	  session to be disconnected, but the BYE is still transacting.
+	* cdr.c: Fix runtime leak of CDR records.
 
-	  This patch makes it so the session object always gets released (no more
-	  memory leak) when the pjsip session is in a disconnected state. Except when
-	  the method is a BYE. Then it waits until a transport error occurs or an event
-	  timeout.
+	  Need to remove all CDR's listed by a CDR object from the active_cdrs_all
+	  container including the root/master record.
 
-	  ASTERISK-27345 #close
+	  ASTERISK-27656
 
-	  Reported by: Corey Farrell
+	  Change-Id: I48b4970663fea98baa262593d2204ef304aaf80e
 
-	  Change-Id: I1e724737b758c20ac76d19d3611e3d2876ae10ed
+2018-01-31 17:48 +0000 [67cd90f10d]  Richard Mudgett <rmudgett@digium.com>
 
-2017-10-03 16:19 +0000 [2b85799512]  Richard Mudgett <rmudgett@digium.com>
+	* app_confbridge: ConfbridgeList event has standard channel shapshot headers.
 
-	* AST-2017-010: Fix cdr_object_update_party_b_userfield_cb() buf overrun
+	  * Made the AMI ConfbridgeList action's ConfbridgeList events output all
+	  the standard channel snapshot headers instead of a few hand-coded channel
+	  snapshot headers.  The benefit is that the CallerIDName gets disruptive
+	  characters like CR, LF, Tab, and a few others escaped.  However, an empty
+	  CallerIDName is now output as "<unknown>" instead of "<no name>".
 
-	  cdr_object_update_party_b_userfield_cb() could overrun the fixed buffer if
-	  the supplied string is too long.  The long string could be supplied by
-	  external means using the CDR(userfield) function.
+	  ASTERISK-27651
 
-	  This may seem reminiscent to AST-2017-001 (ASTERISK_26897) and it is.  The
-	  earlier patch fixed the buffer overrun for Party A's userfield while this
-	  patch fixes the same thing for Party B's userfield.
+	  Change-Id: Iaf7d54a9d40194c2db060bc9b4979fab6720d977
 
-	  ASTERISK-27337
+2018-01-31 15:45 +0000 [f4b161440b]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I0fa767f65ecec7e676ca465306ff9e0edbf3b652
+	* app_confbridge: Add the Muted header to ConfbridgeJoin AMI event.
 
-2017-10-19 13:53 +0000 [2faa3e3bab]  George Joseph <gjoseph@digium.com>
+	  ASTERISK-27651
 
-	* AST-2017-009: pjproject: Add validation of numeric header values
+	  Change-Id: Idef2ca54d242d1b894efd3fc7b360bc6fd5bdc34
 
-	  Parsing the numeric header fields like cseq, ttl, port, etc. all
-	  had the potential to overflow, either causing unintended values to
-	  be captured or, if the values were subsequently converted back to
-	  strings, a buffer overrun.  To address this, new "strto" functions
-	  have been created that do range checking and those functions are
-	  used wherever possible in the parser.
+2017-12-19 02:52 +0000 [5b8fea93d1]  Oron Peled <oron.peled@xorcom.com>
 
-	   * Created pjlib/include/limits.h and pjlib/include/compat/limits.h
-	     to either include the system limits.h or define common numeric
-	     limits if there is no system limits.h.
+	* chan_console: don't read and write at the same time
 
-	   * Created strto*_validate functions in sip_parser that take bounds
-	     and on failure call the on_str_parse_error function which prints
-	     an error message and calls PJ_THROW.
+	  It seems that the ALSA backend of PortAudio doesn't know how to both
+	  read and write at the same time by adding a per-device mutex.
 
-	   * Updated sip_parser to validate the numeric fields.
+	  FIXME: currently only a draft version. Need to either auto-detect
+	  we work with the ALSA backend or add an extra configuration option
+	  to use this mutex.
 
-	   * Fixed an issue in sip_transport that prevented error messages
-	     from being properly displayed.
+	  ASTERISK-27426 #close
 
-	   * Added "volatile" to some variables referenced in PJ_CATCH blocks
-	     as the optimizer was sometimes optimizing them away.
+	  Change-Id: I635eacee45f5413faa18f5a3b606af03b926dacb
 
-	   * Fixed length calculation in sip_transaction/create_tsx_key_2543
-	     to account for signed ints being 11 characters, not 9.
+2018-02-02 17:35 +0000 [1017db107c]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27319
-	  Reported by: Youngsung Kim at LINE Corporation
+	* endpoint identifiers: Some code cleanup.
 
-	  Change-Id: I48de2e4ccf196990906304e8d7061f4ffdd772ff
+	  res_pjsip_endpoint_identifier_user.c:
+	  * Fix copy/paste error in find_endpoint().  We were using a constant
+	  "anonymous" string instead of the passed in endpoint_name when checking
+	  the transport domain for an endpoint match.
+	  * Eliminate RAII_VAR in find_endpoint().
+	  * Remove always true check in find_transport_state_in_use().
+	  * Remove useless CMD_STOP in find_transport_state_in_use().
 
-2017-11-06 17:58 +0000 [16df0e9786]  Corey Farrell <git@cfware.com>
+	  res_pjsip_endpoint_identifier_anonymous.c:
+	  * Eliminate RAII_VAR in anonymous_identify().
+	  * Remove always true check in find_transport_state_in_use().
+	  * Remove useless CMD_STOP in find_transport_state_in_use().
 
-	* res_pjsip_pubsub: Fix multiple leaks on failure to append vectors.
+	  Change-Id: I86924c31db5bd225ca0c1219c761b668c6f91189
 
-	  Change-Id: I68ece0073ea79667ca41eb10405f516f1d30d482
+2018-02-02 17:20 +0000 [b71e469d68]  Richard Mudgett <rmudgett@digium.com>
 
-2017-11-06 18:12 +0000 [cf91dde509]  Corey Farrell <git@cfware.com>
+	* res_pjsip/config_domain_aliases.c: Add check for missing domain.
 
-	* res_pjsip_history: Fix multiple leaks on vector append failure.
+	  What is the point of defining an alias and not saying what is being
+	  aliased?
 
-	  Change-Id: I41e8d5183ace284095cc721f3b1fb32ade3f940f
+	  Change-Id: I98a892016ed61dcf5efeb6619fd748925103f0be
 
-2017-11-06 16:37 +0000 [7ad4c19b20]  Richard Mudgett <rmudgett@digium.com>
+2018-02-02 15:11 +0000 [0960de71ae]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_registrar.c: Fix named AOR and pjproject group deadlock.
+	* res_pjsip.c: Fix documentation typos.
 
-	  One of the patches for ASTERISK_27147 introduced a deadlock regression.
-	  When the connection oriented transport shut down, the code attempted to
-	  remove the associated contact.  However, that same transport had just
-	  requested a registration that we hadn't responded to yet.  Depending
-	  upon timing we could deadlock.
+	  Change-Id: I82ae0b92bfa2ece84a5c684efd9eefdc83ebd068
 
-	  * Made send the REGISTER response after we completed processing the
-	  request contacts and released the named AOR lock to avoid the deadlock.
+2018-02-02 15:43 +0000 [bef49d90c1]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27391
+	* res_sorcery_realtime.c: Fix ref leak if object failed to apply.
 
-	  Change-Id: I89a90f87cb7a02facbafb44c75d8845f93417364
+	  Change-Id: I3c7106ff77009754725cee790eadf5da44154ab6
 
-2017-11-06 17:21 +0000 [c1a608f862]  Corey Farrell <git@cfware.com>
+2018-01-24 19:58 +0000 [7e32adf044]  Sungtae Kim <pchero21@gmail.com>
 
-	* res_stasis: Fix multiple leaks.
+	* manager.c: Fixed "(null):" header in AMI AsyncAGIEnd event
 
-	  * res/stasis/app.c JSON passed to app_send needs to be released.
-	  * res/stasis_message.c: objects leak if vector append fails.
+	  * Changed to create ami_event string only when the given blob is not
+	  json_null().
+	  * Fixed bad expression.
 
-	  Change-Id: I8dd5385b9f50a5cadf2b1d16efecffd6ddb4db4a
+	  ASTERISK-27621
 
-2017-11-05 22:06 +0000 [d95bfcd013]  Aaron An <anjb@ti-net.com.cn>
+	  Change-Id: Ice58c16361f9d9e8648261c9ed5d6c8245fb0d8f
 
-	* res_pjsip:  Avoid crash when contact uri is empty string
+2018-02-01 13:01 +0000 [73f92c2c52]  Joshua Elson <joshelson@gmail.com>
 
-	  Asterisk will crash if contact uri is invalid, so contact_apply_handler
-	  should check if the uri is NULL or empty.
+	* res_pjsip_mwi.c: Fix null pointer crash
 
-	  ASTERISK-27393 #close
-	  Reported-by: Aaron An
-	  Tested-by: AaronAn
+	  ASTERISK-27652 #close
 
-	  Change-Id: Ia0309bdc6b697c73c9c736e1caec910b77ca69f5
+	  Change-Id: I78a0d38bfd8d0d82830f3d53da04872d6b67284d
 
-2017-11-07 06:56 +0000 [ec58521a48]  Richard Mudgett <rmudgett@digium.com>
+2018-02-01 15:03 +0000 [fc98843d4b]  Sean Bright <sean.bright@gmail.com>
 
-	* res_pjproject.c: Fix ast_strdup() alloc failure.
+	* appdocsxml.xslt: Add Language to channel snapshot transformation
 
-	  Change-Id: I74688038e7afe3a279359cce53aadb28ade51ead
+	  Change-Id: I8f494b0c895a69b8bc94656d0c6ceebecb0394d8
 
-2017-11-06 17:55 +0000 [69af7eb663]  Corey Farrell <git@cfware.com>
+2018-01-31 15:40 +0000 [3419a048b9]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_outbound_registration: Fix leak on vector add failure.
+	* manager.c: Fix potential memory leak and corruption.
 
-	  Change-Id: I774b88b3c9da41edd4dc8d78f095481f52f2bd46
+	  ast_str_append_event_header() could potentially leak and corrupt memory if
+	  the ast_str needed to expand to add the AMI event header.
 
-2017-11-06 17:44 +0000 [4f75655cb6]  Corey Farrell <git@cfware.com>
+	  * Fixed to return error if the ast_str_append() failed.
 
-	* res_pjsip_config_wizard: Fix leaks and add check for malloc failure.
+	  Change-Id: I92f36b855540743b208d76e274152ee2d758176d
 
-	  wizard_apply_handler():
-	  - Free host if we fail to add it to the vector.
+2018-01-31 17:27 +0000 [bcfe172f8d]  Richard Mudgett <rmudgett@digium.com>
 
-	  wizard_mapped_observer():
-	  - Check for otw allocation failure.
-	  - Free otw if we fail to add it to the vector.
+	* manager_channels.c: Reordered ast_manager_build_channel_state_string_prefix()
 
-	  Change-Id: Ib5d3bcabbd9c24dd8a3c9cc692a794a5f60243ad
+	  * Made not allocate memory if the channel snapshot is an internal channel.
 
-2017-11-06 17:33 +0000 [8cdc0ef385]  Corey Farrell <git@cfware.com>
+	  * Free memory earlier when no longer needed.
 
-	* test_sorcery_memory_cache_thrash: Handle error from vector append.
+	  Change-Id: Ia06e0c065f1bd095781aa3f4a626d58fa4d28b38
 
-	  Cleanup resources when we fail to append the vector and report test
-	  failure.
+2018-01-31 12:44 +0000 [4e4428ef3c]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I6eb41586fd11dee8c0dfe35e91cb465a4cab7298
+	* res_pjsip_registrar_expire: Delete empty module.
 
-2017-11-06 17:28 +0000 [e4fba95022]  Corey Farrell <git@cfware.com>
+	  Verified nothing in the testsuite lists this module as a dependency.
 
-	* res_pjsip: Fix leak on error in ast_sip_auth_vector_init.
+	  Change-Id: I90c7d52c7e15e85fde3389d5eaccb05b97848813
 
-	  Change-Id: Ib0fc7a18f3135ca8990c3984c9e15f6d26e556e8
+2018-01-30 19:22 +0000 [1ccac0be0e]  Richard Mudgett <rmudgett@digium.com>
 
-2017-11-06 17:17 +0000 [b96dd55add]  Corey Farrell <git@cfware.com>
+	* bridge_softmix.c: Report not talking immediately when muted.
 
-	* res_pjproject: Handle error from adding to the buildopts vector.
+	  Currently in app_confbridge if someone mutes a channel while that channel
+	  is talking, the talk detection code is suspended while the channel is
+	  muted.  As far an an external observer is concerned, the muted channel's
+	  talk status is still "talking" even though the channel is not contributing
+	  audio to the conference bridge.  When the channel is later unmuted, it
+	  takes the usual 'dsp_silence_threshold' option time to clear the talking
+	  status even though the channel may have stopped talking while the channel
+	  was muted.
 
-	  Change-Id: I076c7bd207c7989a23005395ce1735392657be65
+	  * In bridge_softmix.c, clear the talking status and report talking stopped
+	  if the channel was talking when the channel is muted.  When the channel is
+	  unmuted and the channel is still talking then report the channel as
+	  talking since it is contributing audio to the bridge again.
 
-2017-11-06 15:33 +0000 [3dc600ef84]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27647
 
-	* stasis: Release object if vector append fails.
+	  Change-Id: Ie4fdbc05a0bc7343c2972bab012e2567917b3d4e
 
-	  Change-Id: I3e5cc669169aab6175ddfaf7486edeaeb4fdcfb1
+2018-01-30 15:00 +0000 [b9024197ab]  Richard Mudgett <rmudgett@digium.com>
 
-2017-11-06 15:30 +0000 [fc041554c3]  Corey Farrell <git@cfware.com>
+	* app_confbridge: Update dsp_silence_threshold and dsp_talking_threshold docs.
 
-	* RTP Engine: Deal with errors returned from AST_VECTOR_REPLACE.
+	  The dsp_talking_threshold does not represent time in milliseconds.  It
+	  represents the average magnitude per sample in the audio packets.  This is
+	  what the DSP uses to determine if a packet is silence or talking/noise.
 
-	  Check for errors from AST_VECTOR_REPLACE and clean memory if needed.
+	  Change-Id: If6f939c100eb92a5ac6c21236559018eeaf58443
 
-	  Change-Id: I124d15cc1d645f85a72a1279f623c1993b304b0b
+2018-01-31 11:00 +0000 [6c5e3226ec]  Richard Mudgett <rmudgett@digium.com>
 
-2017-11-06 15:10 +0000 [75539dbf2c]  Corey Farrell <git@cfware.com>
+	* res_pjsip_registrar.c: Fix compiler error.
 
-	* Messaging: Report error on failure to register tech or handler.
+	  Need to include signal.h to define pthread_kill() and SIGURG.
 
-	  Message tech and handler registrations use a vector which could fail to
-	  expand.  If it does log and error and return error.
+	  Change-Id: I10ae3aa4bf8e7386ac29ade78c0f2caed8e674fa
 
-	  Change-Id: I593a8de81a07fb0452e9b0efd5d4018b77bca6f4
+2018-01-30 23:05 +0000 [60701b3252]  Corey Farrell <git@cfware.com>
 
-2017-11-06 15:07 +0000 [b4240bea6c]  Corey Farrell <git@cfware.com>
+	* res_pjsip_session: Prevent crash during shutdown.
 
-	* format_cap: Fix leak on AST_VECTOR_APPEND error.
+	  pjproject does not have a function to reverse pjsip_inv_usage_init.
+	  This means we need to ignore any calls to the functions once shutdown is
+	  final.
 
-	  format_cap_framed_init can fail on AST_VECTOR_APPEND.  This should
-	  report failure to the caller and clean the newly allocated frame.
+	  ASTERISK-27571 #close
 
-	  Change-Id: Ica0661235bf09497bf23d844ceb01f21b41a55b0
+	  Change-Id: Ia550fcba563e2328f03162d79fb185f16b7c9b9d
 
-2017-11-06 14:23 +0000 [803dc3dee5]  Corey Farrell <git@cfware.com>
+2018-01-27 13:03 +0000 [720dbb5745]  Corey Farrell <git@cfware.com>
 
-	* stasis: Remove silly use of RAII_VAR in stasis_forward_all.
+	* core: Create ast_atomic macro's.
 
-	  Change-Id: I46de4c968d40144d5b049966304ff66c1469fb65
+	  Create ast_atomic macro's to provide a consistent interface to the
+	  common functionality of __atomic and __sync built-in functions.
 
-2017-11-06 12:51 +0000 [0a0e2c4253]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27619
 
-	* CLI: Remove unused internal command.
+	  Change-Id: Ieba3f81832a0e25c5725ea067e5d6f742d33eb5b
 
-	  The internal CLI command "_command complete" was last used by Asterisk
-	  0.2.0.  Since then we've been using "_command nummatches" and "_command
-	  matchesarray".
+2018-01-28 10:10 +0000 [2b9aa6b5bb]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I682fe1e21a24a3bb5bd04146e639f1c5866bcfce
+	* res_pjsip_pubsub: Prune subs with reliable transports at startup
 
-2017-11-03 18:08 +0000 [593f789563]  Richard Mudgett <rmudgett@digium.com>
+	  In an earlier release, inbound registrations on a reliable transport
+	  were pruned on Asterisk restart since the TCP connection would have
+	  been torn down and become unusable when Asterisk stopped.  This same
+	  process is now also applied to inbound subscriptions.
 
-	* stasis_bridges.c: Fix off-nominal json memory leaks.
+	  Also fixed issues in res_pjsip_registrar where it wasn't handling the
+	  monitoring correctly when multiple registrations came in over the same
+	  transport.
 
-	  Change-Id: Ib1181a36b317c86bff1ef2e44a17a0b1c73cfdc8
+	  To accomplish this, the pjsip_transport_event feature needed to
+	  be refactored to allow multiple monitors (multiple subcriptions or
+	  registrations from the same endpoint) to exist on the same transport.
+	  Since this changed the API, any external modules that may have used the
+	  transport monitor feature (highly unlikey) will need to be changed.
 
-2017-11-06 10:29 +0000 [56e32ca355]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27612
+	  Reported by: Ross Beer
 
-	* res_pjsip_pubsub: Ensure remote URI contains URI only.
+	  Change-Id: Iee87cf4eb9b7b2b93d5739a72af52d6ca8fbbe36
 
-	  This change makes it so that any user of the pubsub
-	  API that requests the remote URI receives only the URI.
-	  Previously the entire string was returned, which could
-	  contain a display name.
+2018-01-29 13:46 +0000 [81db0aca0f]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-27290
+	* res_pjsip_registrar_expire:  Refactor into res_pjsip_register
 
-	  Change-Id: If1d0cd6630f0a264856d31d2a67933109187a017
+	  res_pjsip_registrar_expire remains as an empty module for now.
 
-2017-11-03 17:43 +0000 [6bc8af2433]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ib93698938bae548d2199cb542f3692d1a171239f
 
-	* stasis_channels.c: Remove a very silly RAII_VAR().
+2018-01-29 07:51 +0000 [cf21e9fc97]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I28b458b3c1a442c4ef0be7b4986a95ea4149e14f
+	* Sample modules.conf: comment out example load statement.
 
-2017-11-03 16:14 +0000 [33d02f5090]  Richard Mudgett <rmudgett@digium.com>
+	  The sample modules.conf explicitly loaded res_musiconhold.so.  This is
+	  redundent as autoload=yes is already set.  It causes warnings if
+	  res_musiconhold.so was not installed and results in an unexpected load
+	  if the admin disables autoload without remembering to remove the
+	  res_musiconhold load statement.
 
-	* stasis/app.c: Optimize stasis_app_get_debug_by_name()
+	  Also remove reference to unknown module pbx_gtkconsole.
 
-	  * Eliminate RAII_VAR()
-	  * Short circuit application name lookup if global debug enabled.
+	  Change-Id: Ib01888994d9f1364b14d3c9fb6ff96774a6e580a
 
-	  Change-Id: I5f78b7bd6ca7fd2c3b07cbbe036c6a93b4681123
+2018-01-29 10:20 +0000 [913773cd75]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-11-02 18:40 +0000 [9013415593]  Richard Mudgett <rmudgett@digium.com>
+	* BuildSystem: Enable autotools in FreeBSD.
 
-	* Fix ast_(v)asprintf() malloc failure usage conditions.
+	  In the current versions of FreeBSD, the apps of GNU autotools do not need to
+	  be called with a version anymore. The latest version can be invoked directly.
+	  Additionally, the script ./bootstrap.sh asked for autoconf 2.62 and
+	  automake 1.9, versions which are not available as port anymore.
 
-	  When (v)asprintf() fails, the state of the allocated buffer is undefined.
-	  The library had better not leave an allocated buffer as a result or no one
-	  will know to free it.  The most likely way it can return failure is for an
-	  allocation failure.  If the printf conversion fails then you actually have
-	  a threading problem which is much worse because another thread modified
-	  the parameter values.
+	  ASTERISK-27637
 
-	  * Made __ast_asprintf()/__ast_vasprintf() set the returned buffer to NULL
-	  on failure.  That is much more useful than either an uninitialized pointer
-	  or a pointer that has already been freed.  Many uses won't have to check
-	  for failure to ensure that the buffer won't be double freed or prevent an
-	  attempt to free an uninitialized pointer.
+	  Change-Id: Id7b94b80e78cc943a40ba79b697e3f70019820a7
 
-	  * stasis.c: Fixed memory leak in multi_object_blob_to_ami() allocated by
-	  ast_asprintf().
+2018-01-29 10:00 +0000 [156b12340e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * ari/resource_bridges.c:ari_bridges_play_helper(): Remove assignment to
-	  the wrong thing which is now not needed even if assigning to the right
-	  thing.
+	* app_voicemail: Avoid always true when using pointer address.
 
-	  Change-Id: Ib5252fb8850ecf0f78ed0ee2ca0796bda7e91c23
+	  clang 4.0 warned about this.
 
-2017-11-06 08:05 +0000 [250c173cfb]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27635
 
-	* res_pjsip: Ignore empty TLS configuration
+	  Change-Id: I213f230607d7fbe97c0f5f2d60da9cbf5a2d8231
 
-	  When using realtime, fields that are not explicitly set by an
-	  administrator are still presented to sorcery as empty strings. Handle
-	  this case explicitly.
+2018-01-19 05:13 +0000 [e7f8ef1935]  Alexander Traud <pabstraud@compuserve.com>
 
-	  In this particular case, if any of these fields are required for TLS
-	  support, their existence should be validated in the 'apply' handler once
-	  we have a complete transport definition.
+	* install_prereq: Update RHEL/CentOS/Fedora libraries.
 
-	  ASTERISK-27032 #close
-	  Reported by: seanchann.zhou
+	  deleted
+	   automake git ncurses-devel pjproject-devel sqlite2-devel libsqlite3x-devel
 
-	  Change-Id: Ie3b5fb421977ccdb33e415d4ec52c3fd192601b7
+	  renamed
+	   radiusclient-ng-devel to radcli-devel
+	   gmime22-devel to gmime-dev
 
-2017-11-06 03:18 +0000 [7dd2b18e32]  Alexander Traud <pabstraud@compuserve.com>
+	  added
+	   alsa-lib-devel bash binutils-devel bison doxygen flex hoard make pkgconfig
+	   speexdsp-devel uriparser-devel uw-imap-devel wget xmlstarlet zlib-devel
+	   codec2-devel fftw-devel libsndfile-devel unbound-devel
 
-	* tcptls: Print notice when TLS is enabled but not configured.
+	  ASTERISK-27599
+	  Reported by: Said Masoud
 
-	  Asterisk can be compiled without a SSL/TLS library, without the Development
-	  Headers of OpenSSL. However, if TLS (SIP) or Secure-WebSockets (WebRTC) was
-	  enabled in a configuration file, Asterisk did not notice the user. Asterisk
-	  failed silently, only the corresponding TCP ports were not open.
+	  Change-Id: I05bb0af98ae532b2d5f37478e38b8f0762b1c035
 
-	  ASTERISK-27394
-	  Reported-by: mossley74
+2018-01-28 05:20 +0000 [aaf14670b5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ib8b7539a5b2af8154c22e5f7a40fc68f95d95b93
+	* BuildSystem: Remove unused variables.
 
-2017-11-06 03:21 +0000 [b5331af53b]  Corey Farrell <git@cfware.com>
+	  Because of a copy-and-paste from the script build_tools/download_externals,
+	  the script build_tools/list_valid_installed_externals got its local variables.
+	  However in the latter, three variables were not used actually.
 
-	* configure: Add autoconf check for libopusfile.
+	  Change-Id: I252de5a98c17ea54459174875357c22c2eebe8d5
 
-	  This check is being added to make it easier for end-users of third party
-	  open source Opus modules.  This was removed by ASTERISK-26426 but only
-	  the module needed to be removed.
+2018-01-25 12:06 +0000 [84a6365164]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I62b9cd0c4fa8a77596ab0e042948a643a1152677
+	* loader: Use ast_cli_completion_add for 'module load' completion.
 
-2017-11-04 06:05 +0000 [5f4197735d]  Alexander Traud <pabstraud@compuserve.com>
+	  This addresses all performance issues with 'module load' completion.  In
+	  addition to using ast_cli_completion_add we stop using libedit's
+	  filename_completion_function, instead using ast_file_read_dir.  This
+	  ensures all results are produced from a single call to opendir.
 
-	* install_prereq: Checkout of libSRTP 2.x.
+	  Change-Id: I8bf51ffaa7ef1606f3bd1b5bb13f1905d72c6134
 
-	  Since Asterisk 13.17, libSRTP 2.x is supported. Therefore, its latest version
-	  is installed again via the script install_prereq.
+2018-01-27 09:44 +0000 [d9e42f27b9]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27356
+	* core: Fix unused variable error in handle_show_sysinfo.
 
-	  Change-Id: I13125839a79052356469e41edacbebff0a937d39
+	  The previous fix broke the case
+	  HAVE_SYSINFO = no
+	  HAVE_SYSCTL = yes
+	  HAVE_SWAPCTL = no
+	  which occurs on FreeBSD 11.1 for example.
 
-2017-11-01 17:47 +0000 [3bce5a9dfa]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26563
 
-	* Stasis/ARI: Fix off-nominal path json memory leaks.
+	  Change-Id: If77c39bc75f0b83a6c8a24ecb2fa69be8846160a
 
-	  Change-Id: Id569c624c426e3b22a99936473c730592d8b83fb
+2018-01-27 08:54 +0000 [3c26eec043]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-11-02 11:38 +0000 [675e50b903]  Richard Mudgett <rmudgett@digium.com>
+	* editline: Avoid shifting a negative signed value.
 
-	* AOC: Fix AOC-S json memory leak.
+	  clang 4.0 warned about this.
 
-	  Change-Id: I3a1d40a41a8a7d00fa4a187de6a343a79155d3ef
+	  ASTERISK-27630
 
-2017-11-01 18:04 +0000 [4ea61eef94]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie2725048c661c1792d8b1d498575144350b6e9ba
 
-	* res_stasis_device_state.c: Optimize stasis_app_device_states_to_json()
+2018-01-27 03:25 +0000 [c38da18ec6]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Eliminate RAII_VAR()
-	  * Replace looped alloca with a char[] since that is how it is used anyway.
+	* headers: Consistent use of typeof and/or __typeof__.
 
-	  Change-Id: Ia27e64a884afa0f50b9ffdb1cf23da6bfa51ffdf
+	  Because of a copy-and-paste error, the Asterisk project was using __typeof
+	  instead of typeof. It works because typeof, __typeof, and __typeof__ are
+	  supported by GCC, but here the escaped variant was not intended. Therefore,
+	  for consistence, we change this to typeof.
 
-2017-11-01 18:58 +0000 [8ec2db7084]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I2a962c3e596e882f691a19345445b14571a5f07c
 
-	* res_stasis_mailbox.c: Fix leak of mailbox container.
+2018-01-24 18:25 +0000 [5d320d2d4b]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I7d33c1635713047e7d1597c9d882f7dc006d94b4
+	* Update sounds release to fix siren7 and siren14 files.
 
-2017-11-03 10:35 +0000 [d3cfcc0da6]  Corey Farrell <git@cfware.com>
+	  ASTERISK-16172
 
-	* Build System: Fix build failure caused by recent CLI improvements.
+	  Change-Id: I2fb564258cd4db0f35952ad48b8687355c2dcad3
 
-	  We use the editline library to help with filename completion in our CLI
-	  interface.  Some systems failed to find the header when included from
-	  loader.c.  This is fixed by setting the proper CFLAGS for the build of
-	  loader.o.
+2018-01-15 11:08 +0000 [6da970bfb9]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27378
+	* BuildSystem: Raise autoconf version requirement to 2.60a.
 
-	  Change-Id: Ib7fd496f1d7ed48141a2eadd5dd61cab2f2308be
+	  AC_COMPUTE_INT requires at least autoconf 2.60a.
 
-2017-11-01 11:12 +0000 [ffcb7e2a25]  Ben Ford <bford@digium.com>
+	  This affects only those who contribute to Asterisk, only those who had to use
+	  the script ./bootstrap.sh. Furthermore, this change just makes sure nobody is
+	  using a too old autoconf.
 
-	* res_pjsip: Add to list of valid characters for from_user.
+	  ASTERISK-16951
 
-	  Fixes a regression where some characters were unable to be used in
-	  the from_user field of an endpoint. Additionally, the backtick was
-	  removed from the list of valid characters, since it is not valid,
-	  and it was replaced with a single quote, which is a valid character.
+	  Change-Id: Ibca850e2fe0e77d935207bd959bacf7197d7f637
 
-	  ASTERISK-27387
+2018-01-26 06:48 +0000 [0afff31ed0]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Id80c10a644508365c87b3182e99ea49da11b0281
+	* install_prereq: Download latest Jansson.
 
-2017-10-30 17:30 +0000 [bfb0f27c87]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27603
 
-	* Modules: Additional improvements to CLI completion.
+	  Change-Id: I65c587534c0ae364f063d68da1bed40bb3d5e8aa
 
-	  Replace 'needsreload' argument with a 'type' argument to specify which
-	  type of modules you want completion.  This provides more accurate CLI
-	  completion for load and unload commands.
+2018-01-01 15:59 +0000 [39fcecad59]  Corey Farrell <git@cfware.com>
 
-	  * 'module unload' now excludes modules that have active references or are
-	    not running.
-	  * 'module load' now excludes modules that are already running.
-	  * 'core set debug [atleast] <level> [module]' shows running modules only.
+	* core: Tweak startup order.
 
-	  ASTERISK-27378
+	  Move initialization of units which do not require configuration to occur
+	  before preload modules.  This leaves only units which load config between
+	  module preload and regular load stages.
 
-	  Change-Id: Iea3e00054461484196c46f688f02635cc886bad1
+	  Change-Id: I1d15384acad16a22c3498124421af474fa517478
 
-2017-11-02 02:11 +0000 [7c35740ba1]  Corey Farrell <git@cfware.com>
+2018-01-25 01:37 +0000 [23381d2c5e]  Corey Farrell <git@cfware.com>
 
-	* Add missing menuselect dependencies.
+	* Build System: Require __sync or __atomic functions.
 
-	  This adds menuselect dependencies for modules that use symbols of other
-	  modules.
+	  This change causes the configure script to throw an error if neither
+	  __sync nor __atomic builtin functions are available.
 
-	  ASTERISK-27390
+	  ASTERISK-27619
 
-	  Change-Id: Ia2d2849f5b87a72af7324a82edc3f283eafb5385
+	  Change-Id: Ie01a281e0f5c41dfeeb5f250c1ccea8752f56ef9
 
-2017-11-01 19:46 +0000 [73a5e9f0e9]  Corey Farrell <git@cfware.com>
+2018-01-24 22:44 +0000 [a164b7ccfb]  Corey Farrell <git@cfware.com>
 
-	* Prevent unload of modules which implement an Optional API.
+	* loader: Correct overly strict startup checks.
 
-	  Once an Optional API module is loaded it should stay loaded.  Unloading
-	  an optional API module runs the risk of a crash if something else is
-	  using it.  This patch causes all optional API providers to tell the
-	  module loader not to unload except at shutdown.
+	  The code which handled loading modules had too many situations which
+	  would result in halting Asterisk startup.  Treat most errors as declines
+	  instead of failures.  The exception is when the module load function
+	  returns AST_MODULE_LOAD_FAILURE or an invalid code.
 
-	  ASTERISK-27389
+	  Clear the missingdeps vector when appropriate to ensure the next loop
+	  starts clean.
 
-	  Change-Id: Ia07786fe655681aec49cc8d3d96e06483b11f5e6
+	  ASTERISK-27620
 
-2017-11-01 13:58 +0000 [d524ad523d]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I45547d9641fd45bd86d80250224417625631ad84
 
-	* pjsip_message_filter: Only do interface lookup for wildcard addresses.
+2018-01-24 18:49 +0000 [6fbd855228]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ie083987e69dc43b6861671c218cacacc11b2072f
+	* Build System: Add support for __atomic built-in operators.
 
-2017-10-31 15:08 +0000 [a7c00707a5]  Kevin Harwell <kharwell@digium.com>
+	  Add a check to configure.ac for __atomic_fetch_add support.  If found
+	  use the __atomic built-in operators for ast_atomic_dec_and_test and
+	  ast_atomic_fetchadd_int.
 
-	* features: Bridge application's BRIDGERESULT not appropriately set
+	  ASTERISK~27619
 
-	  The dialplan application "Bridge" was not setting the BRIDGERESULT to failure
-	  when a failure did occur. Even worse if it did fail to join the bridge it would
-	  still report success.
+	  Change-Id: I65b4feb02bae368904ed0fb03f585c05f50a690e
 
-	  This patch now sets the BRIDGERESULT variable to an appropriate value for a
-	  given condition state. Also, removed the value INCOMPATIBLE as a valid result
-	  type since it is no longer used.
+2017-12-29 02:57 +0000 [527cf5a570]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27369 #close
+	* Remove redundant module checks and references.
 
-	  Change-Id: I22588e7125a765edf35cff28c98ca143e9927554
+	  This removes references that are no longer needed due to automatic
+	  references created by module dependencies.
 
-2017-10-30 09:20 +0000 [1b535d8457]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  In addition this removes most calls to ast_module_check as they were
+	  checking modules which are listed as dependencies.
 
-	* ast_coredumper: allow setting asterisk binary explicitly
+	  Change-Id: I332a6e8383d4c72c8e89d988a184ab8320c4872e
 
-	  Adds an extra option, --asterisk-bin=<path> to ast_coredumper. If
-	  provided, the binary given to gdb will be the parameter, rather than
-	  asterisk from the PATH.
+2018-01-24 10:30 +0000 [b9e35bf6d3]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27380 #close
+	* CHANGES: Add AMI action 'PJSIPShowContacts' note.
 
-	  Change-Id: I25f5b91eb75059b0fb2f142e468c26b283b0a9f3
+	  ASTERISK-27581
 
-2017-10-30 00:32 +0000 [cee81b3c8b]  Corey Farrell <git@cfware.com>
+	  Change-Id: If6af275764741a11030f0a4fd324fa29b376d74e
 
-	* Modules: Fix issues with CLI completion.
+2018-01-14 12:33 +0000 [5b8e71ab9f]  Sungtae Kim <pchero21@gmail.com>
 
-	  * Stop using ast_module_helper to check if a module is loaded, use
-	    ast_module_check instead (app_confbridge and app_meetme).
-	  * Stop ast_module_helper from listing reload classes when needsreload
-	    was not requested.
+	* res_pjsip: Add AMI action 'PJSIPShowContacts'
 
-	  ASTERISK-27378
+	  Add an AMI action which provides information on all
+	  configured Contacts.
 
-	  Change-Id: Iaed8c1e4fcbeb242921dbac7929a0fe75ff4b239
+	  ASTERISK-27581
 
-2017-10-28 19:18 +0000 [8436f1d35a]  Igor Goncharovskiy <igorg@iqtek.ru>
+	  Change-Id: I2eed42c74bbc725fad26b8b33b1a5b3161950c73
 
-	* app_agent_spool: Fix typo in dtmf features usage desctiption
+2018-01-18 20:19 +0000 [2f78dc2bfa]  Richard Mudgett <rmudgett@digium.com>
 
-	  Fix typo, that specify usage wrong option 'dtmf-features' for CHANNEL() function
-	  instead of correct 'dtmf_features'
+	* pbx_variables.c: Misc fixes in variable substitution.
 
-	  ASTERISK-27377 #close
+	  * Copy more than one character at a time when there is nothing to
+	  substitute.
 
-	  Change-Id: I15ecc829c1035b359584673e12cdb5c9291ac930
+	  * Fix off by one error if a '}' or ']' is missing.
 
-2017-10-27 13:41 +0000 [987b16e8f9]  Corey Farrell <git@cfware.com>
+	  * Eliminated the requirement that the "used" parameter had to point to a
+	  variable.  The current callers were always declaring a variable to meet
+	  the requirement and discarding the value put into that variable.  Now it
+	  can be NULL.
 
-	* res_pjsip_pubsub: Resolve potential crash in allocate_subscription.
+	  * In ast_str_substitute_variables_full() fixed using the bogus channel to
+	  evaluate a function.  We were not using the bogus channel we just created
+	  to help evaluate a subexpression.
 
-	  When allocate_subscription fails to initialize fields of the new sub it
-	  calls destroy_subscription.
+	  Change-Id: Ia83d99f4f16abe47f329eb39b6ff2013ae7c9854
 
-	  Change-Id: I5b79c915ec216dc00c13c1e4172137864a4bec85
+2018-01-18 09:01 +0000 [679fa5fb34]  Corey Farrell <git@cfware.com>
 
-2017-10-26 12:18 +0000 [2db97ee219]  Richard Mudgett <rmudgett@digium.com>
+	* Add missing OPTIONAL_API and ARI dependences.
 
-	* app_voicemail.c: Fix compiler warning with IMAP build.
+	  I've audited all modules that include any header which includes
+	  asterisk/optional_api.h.  All modules which use OPTIONAL_API now declare
+	  those dependencies in AST_MODULE_INFO using requires or optional_modules
+	  as appropriate.
+
+	  In addition ARI dependency declarations have been reworked.  Instead of
+	  declaring additional required modules in res/ari/resource_*.c we now add
+	  them to an optional array "requiresModules" in api-docs for each module.
+	  This allows the AST_MODULE_INFO dependencies to include those missing
+	  modules.
 
-	  ASTERISK-27181
+	  Change-Id: Ia0c70571f5566784f63605e78e1ceccb4f79c606
 
-	  Change-Id: Ic4468b49860bd7f67e922baf4c9e96828c184d17
+2018-01-22 09:18 +0000 [140f937c7e]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-10-25 14:38 +0000 [ca7f3d297b]  Richard Mudgett <rmudgett@digium.com>
+	* res_config_mysql: Avoid the header mysql_version.h.
 
-	* codec.c: Defensively check the returned samples.
+	  ASTERISK-27607
 
-	  Earlier versions of the codec_opus samples_count callback can return
-	  negative error values on undecodable frames.  This resulted in a divide by
-	  zero exception.
+	  Change-Id: I23d00ded955c4afd5f2c3c9dc96dcb48b3f74eec
 
-	  * Added a defensive check in ast_codec_samples_count() for a "negative"
-	  samples count return value.  Log the event and set the count to zero.
+2018-01-05 14:46 +0000 [fd557ad041]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27194
+	* install_prereq: For PJProject, point users to configure script.
 
-	  Change-Id: Icf69350307ecbbc80a3d74de46af9bd80ea17819
+	  The installation script and the new configure option --with-pjproject-bundled
+	  aimed to accomplish the same. However, the installation script was out of
+	  date. Users should go for the maintained configure option, or the Wiki.
 
-2017-10-24 10:33 +0000 [7385d1e017]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24598
 
-	* res_pjsip: Add 'ip' as a valid option to 'identify_by' on endpoint.
+	  Change-Id: Icbf4b562f81f7c05bd24a3805bd46c0beb4ebd44
 
-	  When the identify_by option on an endpoint is set to ip it will
-	  only be identified using the res_pjsip_endpoint_identifier_ip module.
-	  This ensures that it is not mistakenly matched using the username of
-	  the From header. To ensure behavior has not changed the default has
-	  been changed to "username,ip" for the identify_by option.
+2018-01-20 12:58 +0000 [d427bb84a2]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27206
+	* BuildSystem: Remove AC_CONFIG_AUX_DIR.
 
-	  Change-Id: I2170b86a7f7e221b4f00bf14aa1ef1ac5b050bbd
+	  ASTERISK-27602
 
-2017-10-25 12:26 +0000 [3f7ad66245]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I9f4d3d2bc1481748e39ad1e2b0a364d38e38978b
 
-	* ast_coredumper:  Add gzipping of binaries and display of signal info
+2018-01-19 12:21 +0000 [693e509566]  Alexander Traud <pabstraud@compuserve.com>
 
-	  The --tarball-coredump option now creates a gzipped tarball of
-	  coredumps processed, their results txt files and copies of
-	  /etc/os-release, /usr/sbin/asterisk, /usr/lib(64)/libasterisk* and
-	  /usr/lib(64)/asterisk as those files are needed to properly examine
-	  the coredump.  The file will be named
-	  /tmp/asterisk.<timestamp>.coredumps.tar.gz or
-	  /tmp/asterisk-<uniqueid>.coredumps.tar.gz if --tarball-uniqueid was
-	  specified.
+	* BuildSystem: Remove orphaned .PHONY targets.
 
-	  Added dumps of *_siginfo to the top of the txt files so you can
-	  tell what signal was invoked.
+	  Change-Id: Ic44d75141b9bf99e7d72fcc82ee111b5cf6989d2
 
-	  Change-Id: Ib9ee6d83592d4b1bc90cb3419a05376a88d1ded9
+2018-01-19 12:14 +0000 [70137794e9]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-03-30 09:51 +0000 [4772849016]  Corey Farrell <git@cfware.com>
+	* BuildSystem: Allow make clean all again.
 
-	* Build System: Fix --disable-xmldoc option.
+	  ASTERISK-27600
+	  Reported by: Hamid R. Hashmi
 
-	  The configure option to disable XML documentation does not currently
-	  work.  This patch makes it effective, but also causes an ABI change by
-	  removing the ast_xmldoc_* symbols.  Disabling xmldoc also prevents docs
-	  from being automatically generated, but they can still be manually
-	  generated with 'make doc/core-en_US.xml'.
+	  Change-Id: I683d14d024650be04074b037b6300464519409f4
 
-	  ASTERISK-26639
+2018-01-19 06:16 +0000 [93471373f6]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ifac562340c09f80c83e0203de098fcac93bf8c44
+	* install_prereq: Update Debian/Ubuntu libraries.
 
-2017-10-23 13:42 +0000 [dfe00b80e7]  Ben Ford <bford@digium.com>
+	  ASTERISK-27555
 
-	* http.c: Fix http header send content.
+	  Change-Id: Ieb41b0cbf968af12882b39454b819ebb48b9ea46
 
-	  Currently ast_http_send barricades a portion of the content that
-	  needs to be sent in order to establish a connection for things
-	  like the ARI client. The conditional and contents have been changed
-	  to ensure that everything that needs to be sent, will be sent.
+2018-01-19 04:46 +0000 [4c511c1a4d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27372
+	* install_prereq: Support package manager DNF and yum option strict=1.
 
-	  Change-Id: I8816d2d8f80f4fefc6dcae4b5fdfc97f1e46496d
-
-2017-10-07 12:14 +0000 [23d05ebc49]  Corey Farrell <git@cfware.com>
+	  This re-enables the script ./contrib/scripts/install_prereq on Fedora 22 and
+	  newer, and on RHEL/CentOS when the option strict=1 was set for yum install.
 
-	* hashtab: Use ast_free.
+	  ASTERISK-27598
+	  Reported by: Hunter Stevens, Said Masoud
 
-	  A few places in hashtab use free instead of ast_free.
+	  Change-Id: I40f9517122aaa6906e8fc0962b4b8008dfddb368
 
-	  Change-Id: I2ff089bad71640c03c3ce97f1b00fc962ef79427
+2018-01-09 11:29 +0000 [77f2814d01]  Benoît Dereck-Tricot <benoit.dereck-tricot@eyepea.eu>
 
-2017-10-24 09:43 +0000 [c95ab4c1ce]  Corey Farrell <git@cfware.com>
+	* pbx: Reduce verbosity while loading extensions
 
-	* chan_sip: Fix SUBSCRIBE with missing "Expires" header.
+	  Each time the dial plan is reloaded, a lot of logs like these are generated:
+	  "Added extension 'XXXXX' priority 1 to YYYYYYYYYYY"
+	  This patch changes the log level for those logs.
 
-	  When chan_sip receives a SUBSCRIBE request with no "Expires" header it
-	  processes the request as an unsubscribe.  This is incorrect, per RFC3264
-	  when the "Expires" header is missing a default expiry should be used.
+	  ASTERISK-27084
 
-	  ASTERISK-18140
+	  Change-Id: I5662902161c50890997ddc56835d4cafb456c529
 
-	  Change-Id: Ibf6dcd4fdd07a32c2bc38be1dd557981f08188b5
+2018-01-18 14:55 +0000 [5964061a21]  Sean Bright <sean.bright@gmail.com>
 
-2017-10-24 07:24 +0000 [5351e862ee]  Alexander Traud <pabstraud@compuserve.com>
+	* res_pjsip: Document tlsv1_1 and tlsv1_2 methods
 
-	* lpc10: Avoid compiler warning when DONT_OPTIMIZE/COMPILE_DOUBLE.
+	  Change-Id: I67ed9039bf3f132fb20ee7a750e0aef0f704d7d3
 
-	  ASTERISK-23556
-	  Reported by: Marcello Ceschia
+2018-01-08 23:50 +0000 [33d5ab3e69]  Igor Goncharovsky <igor.goncharovsky@gmail.com>
 
-	  Change-Id: Ic27e88e0336a0d83877dc857938659dc5560b93c
+	* chan_unistim: Fix hold function ability to lock/crash asterisk
 
-2017-10-23 13:44 +0000 [ec278955a3]  Corey Farrell <git@cfware.com>
+	  This patch fix chan_unistim hold functions to correctly support
+	  hold function in different states possible in case of multiple lines
+	  established on the phone
 
-	* main/Makefile: Remove rule for non-existant testexpr2.
+	  ASTERISK-26596 #close
 
-	  Change-Id: Ibb3e47f27a395d74d8c5263db015b05434f5969b
+	  Change-Id: Ib1e04e482e7c8939607a42d7fddacc07e26e14d4
 
-2017-10-23 12:42 +0000 [6cec3149be]  Corey Farrell <git@cfware.com>
+2017-10-29 22:00 +0000 [25cb1ab05b]  Corey Farrell <git@cfware.com>
 
-	* test_config: Fix failure and segfault when config_hook is run twice.
+	* loader: Add support for built-in modules.
 
-	  On second run the config_hook test was unexpectedly failing to load
-	  test_config.conf because it was still unmodified since the last load.
-	  This is fixed by not passing CONFIG_FLAG_FILEUNCHANGED for the initial
-	  loads, only using it when we are tested that a reload of unmodified
-	  files do not initiate the hook.
+	  * Add SRC_EMBEDDED variable to main/Makefile.  Built-in module sources
+	    must be listed in this variable to ensure they get the correct CFLAGS.
 
-	  ASTERISK-25960
+	  Change-Id: I920852bc17513a9c2627061a4ad40511e3a20499
 
-	  Change-Id: Ifd679509a23ed163e5cc647490bf7df4ae3cd856
+2017-12-09 00:03 +0000 [e6142a1282]  Corey Farrell <git@cfware.com>
 
-2017-10-23 12:23 +0000 [4a2575a107]  George Joseph <gjoseph@digium.com>
+	* loader: Rework load_resource_list.
 
-	* res_pjsip_sdp_rtp:  Fix setting of address type for rtp_ipv6
+	  Use a single loop in a loop to scan the resource list attempting to
+	  dlopen each module.  The inner loop is repeated until it doesn't do any
+	  work, then it is run one more time to allow printing of error messages.
 
-	  create_outgoing_sdp_stream was setting "addr_type = STR_IP6" only
-	  when an ipv6 media_address was specified on the endpoint.  If
-	  rtp_ipv6 was set and ast_sip_get_host_ip_string returned an ipv6
-	  address, we were leaving the addr_type set at the default of
-	  STR_IP4.  This caused the address type to be set incorrectly on the
-	  "o" and "c" SDP attributes even though the address was set
-	  correctly.  Some clients don't like the mismatch.
+	  Change-Id: I60c15cd57ff9680b62e2a94c7519401fa4a38e45
 
-	   * Removed the test for endpoint/media_address and now check all
-	     addresses for ipv6.
+2017-12-08 23:30 +0000 [a80cbb046e]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27198
-	  Reported by: Martin Cisárik
+	* loader: Remove global symbol only startup phase.
 
-	  Change-Id: I5214fc31b728117842243807e7927a319cf77592
+	  Dependency loader is now in place so we no longer need a separate loader
+	  phase for global symbols only.  This simplifies the loader and allows us
+	  to minimize calls to dlopen.
 
-2017-10-23 07:53 +0000 [b4cb0050bd]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I33e3174d67f3b4552d3d536326dcaf0ebabb097d
 
-	* app_agent_pool.c: Fix online documentation typo.
+2017-11-21 23:39 +0000 [3b73ed28c5]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ib0bc95fd0ec288c78c313823254d7a84ebfc4429
+	* loader: Process module dependencies.
 
-2017-10-22 17:32 +0000 [07e17fd04f]  Joshua Colp <jcolp@digium.com>
+	  * Add string vectors for requires, optional_apis and enhances.
+	  * Add reffed_deps module vector for holding references to dependencies.
+	  * Initialize string vectors after final dlopen of each module.
+	  * Free string vectors and clear references from reffed_deps in
+	    module_destroy.
+	  * Create functions necessary to process module dependencies and enforce
+	    load order.
 
-	* res_xmpp: Ensure the connection filter is available.
+	  Module dependencies result in automatic references being managed by the
+	  module loader.  This enforces unload order.
 
-	  Users of the API that res_xmpp provides expect that a
-	  filter be available on the client at all times. When
-	  OAuth authentication support was added this requirement
-	  was not maintained.
+	  Change-Id: I9be08d1dd331aceadc1dcba00b804d71360b2fbb
 
-	  This change merely moves the OAuth authentication to
-	  after the filter is created, ensuring users of res_xmpp
-	  can add things to the filter as needed.
+2017-12-27 17:44 +0000 [86b484dec7]  Graham Mainwaring <graham@mhn.org>
 
-	  ASTERISK-27346
+	* app_followme:  Add a prompt to be read when a call is connected
 
-	  Change-Id: I4ac474afe220e833288ff574e32e2b9a23394886
+	  This patch adds the ability to configure a prompt which will be read
+	  to the "winner" who pressed 1 (or the configured value) and received
+	  the call.
 
-2017-10-21 03:44 +0000 [c7a9a6ef0c]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-24372 #close
 
-	* chan_sip: Crypto attribute not last but first on SDP media level.
+	  Change-Id: I6ec1c6c883347f7d1e1f597189544993c8d65272
 
-	  This matches the behavior of the other SIP channel driver, chan_pjsip.
+2018-01-17 00:28 +0000 [4fd303b630]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27365
+	* loader: Miscellaneous fixes.
 
-	  Change-Id: I8f23a51290a58b75816da2999ed1965441dfc5d6
+	  * Remove comment about lazy load.
+	  * Improve message about module already being loaded and running.
+	  * Handle allocation error in add_to_load_order.
+	  * Dead code elimination from modules_shutdown.
 
-2017-10-17 10:53 +0000 [b57dcc8d9c]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I22261599c46d0f416e568910ec9502f45143197f
 
-	* res_pjproject.c: Upgrade bundled PJPROJECT to 2.7
+2018-01-17 08:36 +0000 [2a1b52cc67]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Update patches included in bundled PJPROJECT for the new version.
+	* BuildSystem: Use the detected name for MD5 everywhere.
 
-	  ASTERISK-27355
+	  Affacted the (automatic) download script for external modules:
+	  ./build_tools/download_externals
 
-	  Change-Id: I9ac5dbbffaadca25ad24fac8b9ab615e5ace6083
+	  ASTERISK-27596
 
-2017-10-18 13:41 +0000 [fb4ed70e62]  Corey Farrell <git@cfware.com>
+	  Change-Id: If4c3176f7bf58df32fec6e02a659f1a78d57cf4b
 
-	* res_pjsip_pubsub: Prevent unload except during shutdown.
+2018-01-17 07:11 +0000 [4cd3f5c162]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Prevent unload of the module as certain pjsip initialization functions
-	  cannot be reversed.  This required a reorder of the module_load so that
-	  the non-reversable pjsip functions are not called until all potential
-	  errors have been ruled out.
+	* BuildSystem: Invoke install not in GNU but POSIX style.
 
-	  ASTERISK-24483
+	  ASTERISK-27594
 
-	  Change-Id: Iee900f20bdd6ee1bfe23efdec0d87765eadce8a7
+	  Change-Id: Iaaa6a19d2fe031dffcba441d0502a7ea65c93cb3
 
-2017-10-18 13:37 +0000 [216251abcb]  Corey Farrell <git@cfware.com>
+2018-01-17 06:47 +0000 [7e7a20642c]  Alexander Traud <pabstraud@compuserve.com>
 
-	* res_pjsip_refer: Prevent unload except during shutdown.
+	* BuildSystem: In OpenBSD, xmlstarlet is xml.
 
-	  Prevent unload of the module as certain pjsip initialization functions
-	  cannot be reversed.
+	  ASTERISK-27593
 
-	  ASTERISK-24483
+	  Change-Id: I1c7087f7f7582e40b3312c690d912c9a86466805
 
-	  Change-Id: I94597ec8b8491f5af9c57bf66dbc3b078fe2d49d
+2018-01-17 02:51 +0000 [8f31b70246]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-10-18 12:04 +0000 [7dd7ca2858]  Corey Farrell <git@cfware.com>
+	* BuildSystem: Detect external library Lua in version 5.3.
 
-	* chan_sip: Fix output of 'sip set debug off'.
+	  On some platforms, you decide to go for one specific version of Lua, for
+	  example in OpenBSD. On other platforms, you are able to install several versions
+	  side-by-side, for example in Ubuntu and Fedora. Asterisk already works with
+	  Lua 5.3. Asterisk failed to detect Lua 5.3 on those platforms which allow
+	  several versions.
 
-	  When sip.conf contains 'sipdebug=yes' it is impossible to disable it
-	  using CLI 'sip set debug off'.  This corrects the output of that CLI
-	  command to instruct the user to turn sipdebug off in the configuration
-	  file.
+	  ASTERISK-27592
 
-	  ASTERISK-23462 #close
+	  Change-Id: If7a4b395d844a464e9a1f4f626c5bff4ee67eed8
 
-	  Change-Id: I1cceade9caa9578e1b060feb832e3495ef5ad318
+2017-12-22 19:50 +0000 [8494e78010]  Richard Mudgett <rmudgett@digium.com>
 
-2017-10-06 10:51 +0000 [0a11097a22]  Matt Jordan <mjordan@digium.com>
+	* res_pjsip: Split type=identify to IP address and SIP header matching priorities
 
-	* res_corosync: Fix linking issue with Corosync 2.x
+	  The type=identify endpoint identification method can match by IP address
+	  and by SIP header.  However, the SIP header matching has limited
+	  usefulness because you cannot specify the SIP header matching priority
+	  relative to the IP address matching.  All the matching happens at the same
+	  priority and the order of evaluating the identify sections is
+	  indeterminate.  e.g., If you had two type=identify sections where one
+	  matches by IP address for endpoint alice and the other matches by SIP
+	  header for endpoint bob then you couldn't predict which endpoint is
+	  matched when a request comes in that matches both.
 
-	  At some point in time in the history of Corosync (certainly within the
-	  2.x branch), the corosync_cfg_state_track function was removed.
-	  Unfortunately, the cfg library is only linked if this function is
-	  present. Without the cfg library being linked to res_corosync, loading
-	  of res_corosync will fail.
+	  * Extract the SIP header matching criteria into its own "header" endpoint
+	  identification method so the user can specify the relative priority of the
+	  SIP header and the IP address matching criteria in the global
+	  endpoint_identifier_order option.  The "ip" endpoint identification method
+	  now only matches by IP address.
 
-	  This patch makes it so that detecting corosync's core libraries,
-	  determined by the COROSYNC external library checks, links both the cpg
-	  and cfg libraries with res_corosync.
+	  ASTERISK-27491
 
-	  Change-Id: I674e9e1c8fea11c3bf81154aaa7c1fd43f945465
+	  Change-Id: I9df142a575b7e1e3471b7cda5d3ea156cef08095
 
-2017-10-06 15:55 +0000 [7f8c212e75]  Richard Mudgett <rmudgett@digium.com>
+2018-01-16 08:32 +0000 [7ed7d525fb]  Richard Mudgett <rmudgett@digium.com>
 
-	* cdr.c: Rename the Party A CDR container.
+	* taskprocessor.c: Increase the number of tps_singletons container buckets.
 
-	  * Rename the Party A CDR container from active_cdrs_by_channel to
-	  active_cdrs_master.
+	  Since v12 the number of taskprocessors in the system has increased a lot.
+	  Small systems can easily have over a hundred and larger systems can have
+	  thousands.
 
-	  * Renamed the support functions associated with active_cdrs_master
-	  appropriately.
+	  Most uses of the tps_singletons container deal with creating and
+	  destroying the taskprocessors.  However, the pjsip distributor looks up
+	  taskprocessors/serializers by name frequently.  It needs to find the
+	  serializer for incoming SIP responses to distribute them to the
+	  appropriate serializer.
 
-	  ASTERISK-27335
+	  Change-Id: Ice0603606614ba49f7c0c316c524735c064e7e43
 
-	  Change-Id: I6104bb3edc3a0b7243ce502e45e8832b0cff14f7
+2018-01-16 08:20 +0000 [f0a3c977d6]  George Joseph <gjoseph@digium.com>
 
-2017-10-02 17:42 +0000 [85723a9e50]  Richard Mudgett <rmudgett@digium.com>
+	* pjproject_bundled:  Prevent crash on bad outgoing header
 
-	* cdr.c: Add container to key off of Party B channel names.
+	  We still need to figure out how a bad header is getting into the
+	  outgoing message but this patch to pjproject prevents attempting
+	  to print that header and causing a crash.
 
-	  The CDR performance gets worse the further it gets behind in processing
-	  stasis messages.  One of the reasons is because of a n*m loop used when
-	  processing Party B information.
+	  For several users, this crash happens when sending 183 progress
+	  messages.
 
-	  * Added a new CDR container that is keyed to Party B so we don't need such
-	  a large loop when processing Party B information.
+	  ASTERISK-26832
+	  Reported by: Ross Beer, Jan Rozhon
 
-	  NOTE: To reduce the size of the patch I deferred to another patch the
-	  renaming of the Party A active_cdrs_by_channel container to
-	  active_cdrs_master and renaming the container's hash and cmp functions
-	  appropriately.
+	  Change-Id: Ie5c5a921c890c843587763e7f33f987dfe66bd16
 
-	  ASTERISK-27335
+2018-01-16 06:34 +0000 [a046305fae]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I0bf66e8868f8adaa4b5dcf9e682e34951c350249
+	* BuildSystem: Avoid $EUID and use id -u instead.
 
-2017-10-10 07:42 +0000 [944e500e95]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Makefile included a call to ${EUID} which requires the shell bash. To keep
+	  compatibility with other shells like dash or ksh, use id -u instead.
 
-	* declare optional openssl dependencies in moduleinfo
+	  ASTERISK-27589
 
-	  Declare optional openssl dependencies in:
-	  * res_rtp_asterisk.c
-	  * tcptls.c
+	  Change-Id: Ia6e74f5bc9aab4e6dc62b7439f647b7964e6f657
 
-	  ASTERISK-27328 #close
+2018-01-15 18:03 +0000 [6fbe315f77]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I2636f1c05b8104b4fe6f36cce0ebd9a98b9c78ab
+	* cel_odbc.c: Fix menuslect module description display.
 
-2017-10-13 09:43 +0000 [dcbf61a31e]  Alexander Traud <pabstraud@compuserve.com>
+	  Asterisk's makefile for menuselect has a very simple source file parsing
+	  script that looks for AST_MODULE_INFO lines to extract the quoted string
+	  as a module description.  If it does not find a quoted string it uses the
+	  whole line as the description.
 
-	* res_pjsip_session: Rewrite o= with external_media_address.
+	  Change-Id: I80f13a63818e4e28d683639a94a4dfaea405c1d5
 
-	  PJSIP allows a domain name as external_media_address. This allows chan_pjsip to
-	  be used behind a NAT with changing IP addresses. The IP address of that domain
-	  is resolved to the c= line already. This change sets also the o= line to that
-	  domain.
+2017-11-19 16:30 +0000 [9cfdb81e91]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27341 #close
+	* loader: Add dependency fields to module structures.
 
-	  Change-Id: I690163b6e762042ec38b3995aa5c9bea909d8ec4
+	  * Declare 'requires' and 'enhances' text fields on module info structure.
+	  * Rename 'nonoptreq' to 'optional_modules'.
+	  * Update doxygen comments.
 
-2017-10-14 04:11 +0000 [85cada85d6]  Guido Falsi <madpilot@freebsd.org>
+	  Still need to investigate dependencies among modules I cannot compile.
 
-	* chan_dahdi: wrap include file which is not present on BSD systems in #ifdef
+	  Change-Id: I3ad9547a0a6442409ff4e352a6d897bef2cc04bf
 
-	  The sys/sysmacros.h include file does not exist in BSD systems and
-	  is not required to build this module there.
-	  Since an "#if defined(__NetBSD__) || defined(__FreeBSD__)" section
-	  already exist I moved that include line inside it's #else branch.
+2017-11-19 20:10 +0000 [35ae99c712]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27343 #close
+	* vector: Additional string vector definitions.
 
-	  Change-Id: Ibfb64f4e9a0ce8b6eda7a7695cfe57916f175dc1
+	  ast_vector_string_split:
+	  This function will add items to an ast_vector_string by splitting values
+	  of a string buffer.  Items are appended to the vector in the order they
+	  are found.
 
-2017-10-13 08:51 +0000 [89f11364f6]  Corey Farrell <git@cfware.com>
+	  ast_vector_const_string:
+	  A vector of 'const char *'.
 
-	* ast_bt_get_symbols: Prevent double-free.
+	  Change-Id: I1bf02a1efeb2baeea11c59c557d39dd1197494d7
 
-	  It's possible for bfdobj to be created but syms not created.  If syms
-	  was not allocated in the current loop iteration but was allocated in the
-	  previous iteration it would crash.
+2018-01-15 10:57 +0000 [645297614e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27340
+	* BuildSystem: Resolve resolv.h not via Generic but Particular Header-Check.
 
-	  Change-Id: I5b110c609f6dfe91339f782a99a431bca5837363
+	  ASTERISK-27585
 
-2017-10-13 08:12 +0000 [114ed320ea]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I27c67563788e6f67eeda5fb51a741823a50a95e2
 
-	* tcptls: NULL-check the parameter of ast_ssl_teardown before accessing it.
+2018-01-13 13:49 +0000 [cabe80631b]  George Joseph <gjoseph@digium.com>
 
-	  This avoids a crash on stopping a chan_sip which failed to start its TLS server.
+	* config_transport:  Enable TCP_NODELAY on TLS transports
 
-	  ASTERISK-27339 #close
+	  We did this for TCP transports already but I'm not sure why we
+	  didn't do it for TLS transports.
 
-	  Change-Id: I327fc70db68eaaca5b50a15c7fd687fde79263d5
+	  ASTERISK_27474 #not_final_fix
 
-2017-09-29 14:26 +0000 [a8c6c3bd7d]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I5b1ef4b882f7b859e718236686b7898751dbb262
 
-	* cdr.c: Eliminated many calls to ao2_global_obj_ref().
+2018-01-12 18:37 +0000 [de7f2a6cb4]  Corey Farrell <git@cfware.com>
 
-	  The CDR performance gets worse the further it gets behind in processing
-	  stasis messages.  One of the reasons is we were getting the global config
-	  to determine if we needed to log a debugging message.
+	* res_stasis_recording: Allow symbolic links in configured recordings dir.
 
-	  * Many calls to ao2_global_obj_ref() were just so we could determine if
-	  debug mode is enabled.  Made a global flag to check instead.
+	  If any component of ast_config_AST_RECORDING_DIR is a symbolic link we
+	  would incorrectly assume the ARI user was trying to escape the recording
+	  path.  Create additional check to check the recording directory's
+	  realpath, only deny access if both do not match.
 
-	  * Eliminated many RAII_VAR() usages associated with the remaining
-	  ao2_global_obj_ref() calls.
+	  This is needed by the testsuite when run by 'run-local'.
 
-	  * Added missing NULL checks for the returned ao2_global_obj_ref() value.
+	  Change-Id: I9145e841865edadcb5f75cead3471ad06bbb56c0
 
-	  ASTERISK-27335
+2018-01-12 12:00 +0000 [99535b0497]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Iceaad93172862f610cad0188956634187bfcc7cd
+	* menuselect: Remove unused dev-mode option TRACE_FRAMES.
 
-2017-10-06 13:45 +0000 [b56fa63db8]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27575 #close
 
-	* cdr.c: Defer getting ao2_global_obj_ref() until needed.
+	  Change-Id: Ica3a522892afed7a96816a5ecf140e1671f46ad4
 
-	  The CDR performance gets worse the further it gets behind in processing
-	  stasis messages.  One of the reasons is we were getting the global config
-	  even if we didn't need it.
+2018-01-12 03:50 +0000 [eb9b85baec]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Most uses of the global config were only needed on off nominal code
-	  paths so it makes sense to not get it until absolutely needed.
+	* res_config_pgsql: Avoid typecasting an int to unsigned char.
 
-	  ASTERISK-27335
+	  clang 5.0 warned about this.
 
-	  Change-Id: I00c63b7ec233e5bfffd5d976f05568613d3c2365
+	  ASTERISK-27576
 
-2017-10-05 18:08 +0000 [92c5cf3920]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: If41f400a51973c06cdb9b75462e535b616bfe385
 
-	* cdr.c: Set stringfields only if they are different.
+2018-01-12 03:17 +0000 [cff3add680]  Alexander Traud <pabstraud@compuserve.com>
 
-	  The CDR performance gets worse the further it gets behind in processing
-	  stasis messages.  One of the reasons is we were repeatedly setting string
-	  fields to potentially the same string in base_process_party_a().  Setting
-	  a string field involves allocating room for the new string out of a memory
-	  pool which may have to allocate even more memory.
+	* BuildSystem: Really do not pass unknown-warning options to the compiler.
 
-	  * Check to see if the string field is already set to the desired string.
+	  When an older GCC version is called with a too new warning option, GCC exited
+	  with an error and Asterisk was not built. Therefore, the configure script tests
+	  the installed compiler whether it supports that warning option. If not, Asterisk
+	  does not pass it to the installed compiler. However, some compilers (like clang)
+	  do not exit (error) but give just a warning in such a case. Because the compiler
+	  did not exit, Asterisk passed the unknown-warning option.
 
-	  ASTERISK-27335
+	  ASTERISK-27560
 
-	  Change-Id: I3ccb7e23f1488417e08cafe477755033eed65a7c
+	  Change-Id: Ia9d148e689c173df4e91699113605dab2de36038
 
-2017-10-05 18:03 +0000 [6393c106ca]  Richard Mudgett <rmudgett@digium.com>
+2018-01-12 04:27 +0000 [685bab254c]  Alexander Traud <pabstraud@compuserve.com>
 
-	* cdr.c: Fix setting dnid, callingsubaddr, and calledsubaddr
+	* app_osplookup.c: Avoid two format truncations.
 
-	  The string comparisons for setting these CDR variables was inverted.  We
-	  were repeatedly setting these CDR variables only if the channel snapshots
-	  had the same value.
+	  GCC 7 warned about this.
 
-	  ASTERISK-27335
+	  ASTERISK-27578
 
-	  Change-Id: I9482073524411e7ea6c03805b16de200cb1669ea
+	  Change-Id: I4a00458dbe9b575ef04338b6a7852272745e1552
 
-2017-10-11 06:04 +0000 [407b78504d]  Torrey Searle <torrey@voxbone.com>
+2018-01-12 04:03 +0000 [797747afa7]  Alexander Traud <pabstraud@compuserve.com>
 
-	* contrib/script/sip_to_pjsip: implement 'all' for allow/disallow
+	* chan_ooh323: Avoid typecasting an int to unsigned short.
 
-	  when 'all' is specified in an allow or disallow section, it should erase
-	  all values from the inverse section in the default config. E.G.
-	  allow=all should erase any deny values from default config &
-	  vice-versa
+	  clang 5.0 warned about this.
 
-	  ASTERISK-27333 #close
+	  ASTERISK-27577
 
-	  Change-Id: I99219478fb98f08751d769daaee0b7795118a5a6
+	  Change-Id: I898fe4255023138a9e8b579fe4482fcf582f2b78
 
-2017-10-13 17:46 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2018-01-05 15:13 +0000 [b9e2b72de6]  Alexander Traud <pabstraud@compuserve.com>
 
-	* asterisk 13.18.0-rc1 Released.
+	* install_prereq: Update Debian/Ubuntu libraries.
 
-2017-10-13 12:09 +0000 [4bc2aca9b7]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-27555
 
-	* AMI: Increase version number
+	  Change-Id: I0818b6e42631be1b69237e2b41d3415275693e53
 
-	  Bump the AMI patch number since the following new addition was made:
+2018-01-11 12:05 +0000 [6d5f4768a4]  Joshua Colp <jcolp@digium.com>
 
-	  * Added a new CancelAtxfer action that cancels an attended transfer.
+	* chan_sip: Check that an iostream exists before accessing.
 
-	  Change-Id: I9bac528791bd62ef0e99243903b6bc7a6c7ab182
+	  Before getting the file descriptor for an iostream check
+	  that it is present.
 
-2017-08-25 08:19 +0000 [6d3ee9fb93]  Thomas Sevestre <thomassevestre@free.fr>
+	  ASTERISK-27534
 
-	* features, manager : Add CancelAtxfer AMI action
+	  Change-Id: Ie0aa1394007a37c30e337ea1176a6fb3a63bc99c
 
-	  Add action to cancel feature attended transfer with AMI interface
+2018-01-11 08:09 +0000 [30b5ec023f]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  ASTERISK-27215 #close
+	* Ignore quilt .pc directory, used in deb packaging
 
-	  Change-Id: Iab8a81362b5a1757e2608f70b014ef863200cb42
+	  Debian packaging uses quilt to manage patches. Book-keeping for them is
+	  done using quilt (either directly, or in a compatible format), and
+	  tracked in the directory .pc .
 
-2017-10-06 04:55 +0000 [21d502818f]  Daniel Tryba <daniel@tryba.nl>
+	  Change-Id: I22c90f3d7ab8918e6216e7b686de6fa0e1fdaa7b
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* res_pjsip_session: Prevent user=phone being added to anonimized URIs.
+2018-01-09 11:23 +0000 [f0eb00d1e7]  Corey Farrell <git@cfware.com>
 
-	  Move ast_sip_add_usereqphone to be called after anonymization of URIs,
-	  to prevent the user_eq_phone adding "user=phone" to URIs containing a
-	  username that is not a phonenumber (RFC3261 19.1.1). An extra call to
-	  ast_sip_add_usereqphone on the saved version before anonymization is
-	  added to add user=phone" to the PAI.
+	* stasis: Remove silly usage of RAII_VAR.
 
-	  ASTERISK-27047 #close
+	  Change-Id: Ib11193531e797bcb16bba560a408eab155f706d1
 
-	  Change-Id: Ie5644bc66341b86dc08b1f7442210de2e6acdec6
+2018-01-09 11:09 +0000 [a383e1ddb1]  Corey Farrell <git@cfware.com>
 
-2017-10-06 05:14 +0000 [af09996178]  Daniel Tryba <daniel@tryba.nl>
+	* stasis_cache_pattern: Remove silly usage of RAII_VAR.
 
-	* res_pjsip: Prevent "user=phone" being added multiple times to header
+	  Change-Id: Ic98a51f555062cd863b6db3f8d76065943a9dea3
 
-	  ast_sip_add_usereqphone adds "user=phone" to the header every time is is
-	  called without checking whether the param already exists. Preventing
-	  this by searching to string representation of header for "user=phone".
+2018-01-09 16:23 +0000 [9e2fcb82ed]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26988 #close
+	* cdr_syslog: Deprecate unmaintained module
 
-	  Change-Id: Ib84383b07254de357dc6a98d91fc1d2c2c3719e6
+	  There has been an open issue against cdr_syslog (ASTERISK~14441) about
+	  a race condition for 7.5 years that has never been addressed. Because
+	  this module is effectively unmaintained and currently broken, there is
+	  no sense in keeping it around.
+
+	  If logging CDRs to syslog is a desirable feature, it would probably be
+	  better to write the logs directly to the syslog server via socket
+	  instead of using the facilities provided by openlog/syslog/closelog.
+	  Doing so would address the race condition referenced in the associated
+	  issue.
 
-2017-10-10 09:49 +0000 [8e05796e81]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Change-Id: Ic77b94cd97f355a9cf5b1d3f3444964a6e0ba5dc
 
-	* cdr_mysql: avoid releasing a config string
+2018-01-09 11:16 +0000 [0de004dd85]  Corey Farrell <git@cfware.com>
 
-	  Fixes a memory corruption issue after a reload of cdr_mysql.
+	* stasis_bridges: Remove silly usage of RAII_VAR.
 
-	  Issue was accidentally included in 747beb1ed159f89a3b58742e4257740b3d6d6bba .
+	  Change-Id: I0fa7ab05454f183dc4ff10e26d18776d2b0fcf1f
 
-	  ASTERISK-27270 #close
+2018-01-09 11:10 +0000 [01127e1664]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I90b6a9d18710c0f9009466370bd5f4bac5d5d12e
+	* stasis_cache: Remove silly usage of RAII_VAR.
 
-2017-10-05 18:12 +0000 [5f6bad6733]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ifa95e5801c949df296c7e4376347730fb0ed52ef
 
-	* cdr.c: Defer misc checks.
+2018-01-09 10:57 +0000 [175a9ef873]  Corey Farrell <git@cfware.com>
 
-	  Try to defer some checks until needed in case there is an early exit.
+	* stasis_endpoints: Remove silly usage of RAII_VAR.
 
-	  Change-Id: Ibc6b34c38a4f60ad4f9b67984b7d070a07257064
+	  Change-Id: Ic099dc552f36c353c89783a4bcfd09f010432733
 
-2017-10-11 07:03 +0000 [f3f141781c]  George Joseph <gjoseph@digium.com>
+2018-01-09 10:55 +0000 [4b655184b0]  Corey Farrell <git@cfware.com>
 
-	* chan_vpb:  Fix a gcc 7 out-of-bounds complaint
+	* stasis_message_router: Remove silly usage of RAII_VAR.
 
-	  chan_vpb was trying to use sizeof(*p->play_dtmf), where
-	  p->play_dtmf is defined as char[16], to get the length of the array
-	  but since p->play_dtmf is an actual array, sizeof(*p->play_dtmf)
-	  returns the size of the first array element, which is 1.  gcc7
-	  validly complains because the context in which it's used could
-	  cause an out-of-bounds condition.
+	  Change-Id: I50d6ae230920e0b878ed9cc8f79eef746e06701d
 
-	  Change-Id: If9c4bfdb6b02fa72d39e0c09bf88900663c000ba
+2018-01-09 10:53 +0000 [3074c4165c]  Corey Farrell <git@cfware.com>
 
-2017-10-06 02:39 +0000 [416e35589e]  Nathan Bruning <nathan@iperity.com>
+	* stasis_system: Remove silly usage of RAII_VAR.
 
-	* app_queue.c: clear moh field in init_queue
+	  Change-Id: Iedbe5656cee68cd3a96a953558764aa02d4a0c3b
 
-	  ASTERISK-27301 #close
+2018-01-03 17:26 +0000 [8f3167c5f1]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ic31361f34e2de3b6470e68fc37205a7711082eba
+	* res_pjsip.c: Update the endpoint identification documentation.
 
-2017-10-10 12:01 +0000 [e71a65a358]  Sean Bright <sean.bright@gmail.com>
+	  * Endpoint identify_by documentation.
+	  * IP/Header endpoint identifier documentation.
 
-	* app_originate: Set ORIGINATE_STATUS correctly on failure
+	  Change-Id: Id92f00b495acca7be945daf749d2abd7f76a0b5a
 
-	  We were ignoring the return value from ast_pbx_outgoing_exten() and
-	  ast_pbx_outgoing_app() which could fail before setting the reason code.
-	  This resulted in failures being reported as success.
+2018-01-03 15:20 +0000 [42a61d9db6]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25266 #close
-	  Reported by: Allen Ford
+	* res_pjsip_endpoint_identifier_ip.c: Remove unnecessary requirement.
 
-	  Change-Id: Idf16237b7e41b527d2c69c865829128686beeb3b
+	  The requirement that "ip" must be in the endpoint identify_by list to
+	  allow the type=identify method to identify the endpoint is not necessary.
+	  The "ip" identifier method can match one and only one endpoint.  To even
+	  work, the "ip" identifier method configuration must explicitly specify the
+	  identified endpoint.  Therefore, why bother configuring the type=identify
+	  identifier in the first place?  The requirement only adds the potential
+	  for configuration errors for no benefit.  Even worse, those configuration
+	  errors cannot be detected when the configuration loads.  The requirement
+	  was introduced with the ASTERISK_27206 patch.
 
-2017-10-02 16:46 +0000 [42fdfffefc]  Richard Mudgett <rmudgett@digium.com>
+	  * Remove the code change that enforces the requiremnt.  Listing the "ip"
+	  method in the identify_by value is simply documentation.
 
-	* cdr.c: Eliminated simple RAII_VAR usages.
+	  Change-Id: Ia057f92a33fb5d9f51dc5d5692e3d5ee1a6f2c11
 
-	  Change-Id: I150505db307249a962987e7b941bdd369bb91f35
+2018-01-05 19:03 +0000 [a7bbb18e5c]  Richard Mudgett <rmudgett@digium.com>
 
-2017-10-09 22:51 +0000 [48971e4d43]  Corey Farrell <git@cfware.com>
+	* res_pjsip.c: Fix ident_to_str() and refactor ident_handler().
 
-	* res_pjproject: Fix cleanup of buildopts vector.
+	  * Extracted sip_endpoint_identifier_type2str() and
+	  sip_endpoint_identifier_str2type() to simplify the calling functions.
 
-	  ASTERISK-27306
+	  * Fixed pjsip_configuration.c:ident_to_str() building the endpoint's
+	  identify_by value string.
 
-	  Change-Id: I3bed0edf3f55b1d4adcbabb25ec14f11dc766c72
+	  Change-Id: Ide876768a8d5d828b12052e2a75008b0563fc509
 
-2017-10-03 16:09 +0000 [128f7ffaa2]  Richard Mudgett <rmudgett@digium.com>
+2018-01-04 17:04 +0000 [be488eb14a]  Richard Mudgett <rmudgett@digium.com>
 
-	* cdr.c: Replace redundant check with an ast_assert()
+	* res_pjsip_endpoint_identifier_ip.c: Allow multiple IdentifyDetail AMI events.
 
-	  The only caller of cdr_object_fn_table.process_party_b() explicitly does
-	  the check before calling.
+	  The AMI PJSIPShowEndpoint action could only list one IdentifyDetail AMI
+	  event per endpoint.  However, there is no reason that multiple
+	  type=identify sections cannot identify the same endpoint.
 
-	  Change-Id: Ib0c53cdf5048227842846e0df9d2c19117c45618
+	  * Reworked format_ami_endpoint_identify() to generate as many
+	  IdentifyDetail AMI events as there are matching identifiers.
 
-2017-10-02 17:41 +0000 [3525081a7c]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie146792aef72d78e05416ab5b27bc552a30399db
 
-	* cdr.c: Replace inlined code with ao2_t_replace()
+2018-01-05 05:51 +0000 [3a7d917256]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I9f424f5282ca7d833592f958d95f1b2bafb549b0
+	* translate: Avoid absolute value on unsigned substraction.
 
-2017-09-29 12:07 +0000 [7366657a9a]  Richard Mudgett <rmudgett@digium.com>
+	  ast_format_get_sample_rate(.) returns an unsigned type. The difference of a
+	  substraction between two unsigned types does not get implicitly converted to a
+	  signed type. Therefore, using abs(.) did not make sense.
 
-	* cdr.c: Use current ao2 flag names
+	  ASTERISK-27549
 
-	  Change-Id: Ib59d7d2f2a4a822754628f2c48a308d6791a6e6e
+	  Change-Id: Ib904d9ee0d46b6fdd1476fbc464fbbf813304017
 
-2017-09-29 12:31 +0000 [34d55352a5]  Richard Mudgett <rmudgett@digium.com>
+2018-01-09 08:22 +0000 [25022de875]  Sean Bright <sean.bright@gmail.com>
 
-	* cdr.h: Fix doxygen comments.
+	* Revert "codec_opus: Make libcurl a dependency in menuselect"
 
-	  * Also some misc formatting in cdr.c.
+	  This reverts commit 028f4320de60a204e457ad606ab0a3318493b431.
 
-	  Change-Id: Ied89a28802a662c37c43326a1aafdce596e0df4a
+	  Change-Id: Ieb91f825cb55202a937f5361c01d356e7662b70c
 
-2017-09-20 18:36 +0000 [d388c18abf]  Richard Mudgett <rmudgett@digium.com>
+2018-01-08 10:54 +0000 [a21841bf40]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_registrar.c: Update remove_existing AOR contact handling.
+	* res_pjsip_session: Always bundle streams if WebRTC is enabled.
 
-	  When "rewrite_contact" is enabled, the "max_contacts" count option can
-	  block re-registrations because the source port from the endpoint can be
-	  random.  When the re-registration is blocked, the endpoint may give up
-	  re-registering and require manual intervention.
+	  Some WebRTC clients can't handle renegotiation with the addition of
+	  streams that include an offer to bundle. They instead expect the
+	  newly added streams to already be bundled. This change does such a thing
+	  if WebRTC support is enabled on an endpoint.
 
-	  * The "remove_existing" option now allows a registration to succeed by
-	  displacing any existing contacts that now exceed the "max_contacts" count.
-	  Any removed contacts are the next to expire.  The behaviour change is
-	  beneficial when "rewrite_contact" is enabled and "max_contacts" is greater
-	  than one.  The removed contact is likely the old contact created by
-	  "rewrite_contact" that the device is refreshing.
+	  ASTERISK-27566
 
-	  ASTERISK-27192
+	  Change-Id: I7fe9b7ac35a2798627d9c2c8369129f407af6461
 
-	  Change-Id: I64c107a10b70db1697d17136051ae6bf22b5314b
+2018-01-08 20:25 +0000 [d46cbe788a]  Corey Farrell <git@cfware.com>
 
-2017-10-04 10:46 +0000 [82592c3673]  Corey Farrell <git@cfware.com>
+	* bridge_softmix: Fix sfu_append_source_streams test.
 
-	* res_pjsip: Fix issues that prevented shutdown of modules.
+	  * validate_stream: zero result from ast_format_cap_identical indicates
+	    they are not identical, rather than non-zero indicating an error.
+	  * validate_original_streams: use num_streams instead of
+	    ARRAY_LEN(params).
+	  * Fix declaration of alice_dest_stream and bob_dest_stream.
 
-	  res_pjsip and res_pjsip_session had circular references, preventing both
-	  modules from shutting down.
-	  * Move session supplement registration to res_pjsip.
-	  * Use create internal functions for use by pjsip_message_filter.c.
+	  Change-Id: I6b1dd8bed10439d3c7406f033eb1896b6c419147
 
-	  ASTERISK-27306
+2018-01-08 18:47 +0000 [5380fb9978]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ifbd5c19ec848010111afeab2436f9699da06ba6b
+	* app_confbridge: Fix NULL check in action_kick_last.
 
-2017-10-09 08:15 +0000 [6b16fa12c8]  Sean Bright <sean.bright@gmail.com>
+	  The check for last_user == NULL needs to happen before we dereference
+	  the variable, previously it was possible for us to check flags of a NULL
+	  last_user.
 
-	* res_config_sqlite: Don't enable SQLite CDRs when running 'make samples'
+	  Change-Id: I274f737aa8af9d2d53e4a78cdd7ad57561003945
 
-	  Change-Id: I65a5190b2732b2246d67472db70dd37db64ddad4
+2018-01-06 02:17 +0000 [55a540272f]  Corey Farrell <git@cfware.com>
 
-2017-10-08 14:05 +0000 [39b68a41f7]  David Hajek <david.hajek@daktela.com>
+	* res_stasis: Reduce RAII_VAR usage.
 
-	* res/res_ari.c Fix: Memory leaks in ARI when using Content-Type: application/json
+	  In addition to being a micro-optimization (RAII_VAR has overhead), this
+	  change improves output of REF_DEBUG.  Unfortunately when RAII_VAR calls
+	  ao2_cleanup it does so from a generated _dtor_varname function.  For
+	  example this caused _dtor_app to release a reference instead of
+	  __stasis_app_unregister.
 
-	  ASTERISK-27305
-	  Reported by: David Hajek
-	  Tested by: David Hajek
+	  Change-Id: I4ce67120583a446babf9adeec678b71d37fcd9e5
 
-	  Change-Id: Ife3e289062e6cf7d0e7d342dbf79ed96feff441e
+2018-01-04 18:47 +0000 [faeb9e1b26]  Sungtae Kim <pchero21@gmail.com>
 
-2017-10-08 09:11 +0000 [209916981a]  Alexander Traud <pabstraud@compuserve.com>
+	* res_pjsip: Add AMI action 'PJSIPShowAuths'
 
-	* tcptls: Do not re-bind to wildcard on client creation.
+	  Add an AMI action which provides information on all
+	  configured Auths.
 
-	  Since ASTERISK-26922, this issue affected only those chan_sip which were
-	  * enabled for dual-stack (bindaddr=::), and
-	  * enabled for TCP (tcpenable=yes) and/or TLS (tlsenable=yes), and
-	  * tried to register and/or invite a IPv4-only service,
-	  * via TCP and/or TLS.
-	  Now, ast_tcptls_client_create does not re-bind to [::] anymore.
+	  ASTERISK-27547
 
-	  ASTERISK-27324 #close
+	  Change-Id: I1a88a75b38a2b1dd9d1de6c0307b20a3f584c817
 
-	  Change-Id: I4b242837bdeb1ec7130dc82505c6180a946fd9b5
+2018-01-07 21:38 +0000 [8b3083cac5]  Corey Farrell <git@cfware.com>
 
-2017-10-05 16:26 +0000 [f1163c0f6f]  Corey Farrell <git@cfware.com>
+	* res_stasis: Fix dial bridge unload.
 
-	* res_pjsip: Fix leak of persistent endpoint references.
+	  If the dial bridge has been created it must be released by calling
+	  ast_bridge_destroy, simply releasing the ao2 reference is not enough.
 
-	  Do not manually call sip_endpoint_apply_handler from load_all_endpoints.
-	  This is not necessary and causes memory leaks.
+	  Also move stasis_app_control_shutdown earlier in unload to ensure the
+	  bridge cannot be created or grabbed after the app_bridges container is
+	  released.
 
-	  Additionally reinitialize persistent->aors when we reuse a persistent
-	  object with a new endpoint.
+	  Change-Id: I372302de94ca63876069e2585a049c5060e5e767
 
-	  ASTERISK-27306
+2018-01-07 20:21 +0000 [6870ba5f26]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I59bbfc8da8a14d5f4af8c5bb1e71f8592ae823eb
+	* res_stasis: Fix app_is_subscribed_bridge_id.
 
-2017-10-05 17:59 +0000 [8bf4be1048]  Corey Farrell <git@cfware.com>
+	  Instead of searching for bridge_id provided in an argument this function
+	  always searched for BRIDGE_ALL first.  Rewrite this function to work
+	  like the similar functions for channel and endpoint functions.
 
-	* vector: multiple evaluation of elem in AST_VECTOR_ADD_SORTED.
+	  Change-Id: Ib5caca69e11727c5c8a7284a1d00621f40f1e60a
 
-	  Use temporary variable to prevent multiple evaluations of elem argument.
-	  This resolves a memory leak in res_pjproject startup.
+2018-01-05 07:44 +0000 [7e9781c25e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27317 #close
+	* General: Silence modules on (un)load.
 
-	  Change-Id: Ib960d7f5576f9e1a3c478ecb48995582a574e06d
+	  Some (normally optional) modules created notices, warnings, and even errors
+	  in normal situations like (un)load. This cluttered the command-line interface
+	  (CLI) on start and while stopping gracefully. However, when an user went for
+	  the script './contrib/scripts/install_prereq', those modules get compiled-in
+	  because their prerequisites were met at compile time. Furthermore, because of
+	  ASTERISK_27475, the former talkative module 'res_curl' is built as side-effect.
 
-2017-10-05 15:54 +0000 [5110600f1e]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27553
 
-	* res_pjsip: Fix leak of fake_auth references.
+	  Change-Id: I9f105f46d72553994e820679bfde3478a551b281
 
-	  pjsip_distributor leaks references to fake_auth when the default realm
-	  has not changed.
+2018-01-06 15:40 +0000 [512286e3c8]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27306
+	* BuildSystem: Really do not pass unknown-warning options to the compiler.
 
-	  Change-Id: I3fcf103b3680ad2d1d4610dcd6738eeaebf4d202
+	  When an older GCC version is called with a too new warning option, GCC exited
+	  with an error and Asterisk was not built. Therefore, the configure script tests
+	  the installed compiler whether it supports that warning option. If not, Asterisk
+	  does not pass it to the installed compiler. However, some compilers (like clang)
+	  do not exit (error) but give just a warning in such a case. Because the compiler
+	  did not exit, Asterisk passed the unknown-warning option.
 
-2017-10-05 20:23 +0000 [462dd7c2de]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27560
 
-	* main/strings: Fix uninitialized value.
+	  Change-Id: Ia9b7747f649b27ff5e9f75c3db3fee4fe7a29621
 
-	  ast_strings_match uses sscanf and checks for non-zero return to verify a
-	  token was parsed. This is incorrect as sscanf returns EOF (-1) for errors.
+2018-01-06 01:25 +0000 [f84fcc1fc1]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-27318 #close
+	* General: Avoid implicit conversion to char when changes value to negative.
 
-	  Change-Id: Ifcece92605f58116eff24c5a0a3b0ee08b3c87b1
+	  clang 5.0 warned about this.
 
-2017-09-28 02:56 +0000 [29c442b587]  Benoît Dereck-Tricot <benoit.dereck-tricot@eyepea.eu>
+	  ASTERISK-27557
 
-	* res_calendar_icalendar: Filter out occurrences superceded by another VEVENT
+	  Change-Id: I7cceaa88e147cbdf81a3a7beec5c1c20210fa41e
 
-	  When we are loading the calendars, we call libical's
-	  icalcomponent_foreach_recurrence method for each VEVENT component that
-	  we have in our calendar.
+2018-01-05 06:06 +0000 [b12c8cffad]  Alexander Traud <pabstraud@compuserve.com>
 
-	  That method has no knowledge concerning the existence of the other
-	  VEVENT components and will feed our callback with all ocurrences
-	  matching the requested time span.
+	* bridge_softmix: Removed unused parameter from check_binaural_position_change(.).
 
-	  The occurrences generated by icalcomponent_foreach_recurrence while
-	  expanding a recurring VEVENT's RRULE and RDATE properties can be
-	  superceded by an other VEVENT sharing the same UID.
+	  Found as a result of the function being passed an uninitalized variable by
+	  clang.
 
-	  I use an external iterator (in libical terminology) to avoid messing
-	  with the internal ones from the calling function, and search for
-	  VEVENTS which could supersede the current occurrence.
+	  ASTERISK-27550
 
-	  The event which can invalidate this occurence needs to have:
+	  Change-Id: I8af3bd84656b685a956d498459f8db3613f68954
 
-	  - the same UID as our recurrent component (comp)
-	  - a RECURRENCE-ID property, which represents the start time of this
-	    occurrence
+2018-01-06 06:45 +0000 [ad3252ccef]  Alexander Traud <pabstraud@compuserve.com>
 
-	  If one component is found, just clean and return.
+	* editline: Avoid comparison between pointer and zero character constant.
 
-	  ASTERISK-27296 #close
-	  Reported by: Benoît Dereck-Tricot
+	  gcc 7.2 warned about this.
 
-	  Change-Id: I8587ae3eaa765af7cb21eda3b6bf84e8a1c87af8
+	  ASTERISK-27559
 
-2017-10-03 15:16 +0000 [6c30f4a2d1]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: I48960dda9cf0a11b6a9426f775e632363f8caa74
 
-	* contrib/thirdparty/sip_to_pjsip: add additional flag mappings
+2018-01-06 05:01 +0000 [ef68df9111]  Alexander Traud <pabstraud@compuserve.com>
 
-	  add mappings for udptl redundancy, rtptimeout, and debug flags
+	* codec_gsm: Avoid shifting a negative signed value.
 
-	  Change-Id: Ie73cf5c83c05dee01eb9624ede76c1a30225d73a
+	  clang 5.0 warned about this.
 
-2017-10-02 07:48 +0000 [6dfe5b29b6]  Daniel Tryba <daniel@pocos.nl>
+	  ASTERISK-27558
 
-	* res_pjsip_caller_id chan_sip: Comply to RFC 3323 values for privacy
+	  Change-Id: Icc452ecb0d86bbeba78dae768cc472ec540699df
 
-	  Currently privacy requests are only granted if the Privacy header
-	  value is exactly "id" (defined in RFC 3325). It ignores any other
-	  possible value (or a combination there of). This patch reverses the
-	  logic from testing for "id" to grant privacy, to testing for "none" and
-	  granting privacy for any other value. "none" must not be used in
-	  combination with any other value (RFC 3323 section 4.2).
+2018-01-04 12:23 +0000 [b20b5758d9]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27284 #close
+	* res_pjsip_endpoint_identifier_ip.c: Fix apply identify validation.
 
-	  Change-Id: If438a21f31a962da32d7a33ff33bdeb1e776fe56
+	  The ip_identify_apply() did not validate the configuration for simple
+	  static configuration errors or deal well with address resolution errors.
 
-2017-09-28 17:37 +0000 [0945f10d3b]  Richard Mudgett <rmudgett@digium.com>
+	  * Added missing configuration validation checks.
+	  * Fixed address resolution error handling.
+	  * Demoted an error message to a warning since it does not fail applying
+	  the identify object configuration.
 
-	* app_queue.c: Fix announcements when announce-to-first-user not enabled.
+	  Change-Id: I8b519607263fe88e8ce964f526a45359fd362b6e
 
-	  The previous patch for ASTERISK-27216 made it so you wouldn't get any
-	  position or periodic announcements unless you had announce-to-first-user
-	  enabled.  The announce-to-first-user feature was added by ASTERISK_21782
-	  as a result of the patch which introduced the redundant announcements that
-	  ASTERISK-27216 removes.
+2018-01-04 17:42 +0000 [705e6c04b3]  Richard Mudgett <rmudgett@digium.com>
 
-	  * By noting that the makeannouncement variable is used to suppresses the
-	  first user announcement, we set its initial value to the
-	  announce-to-first-user enable setting.
+	* res_pjsip.c: Fix endpoint identifier registration name search.
 
-	  ASTERISK-27216
+	  If an endpoint identifier name in the endpoint_identifier_order list is a
+	  prefix to the identifier we are registering, we could install it in the
+	  wrong position of the list.
 
-	  Change-Id: Ieaeb7dbea8ae7073086b775fbafe0625b000b10a
+	  Assuming
+	  endpoint_identifier_order=username,ip,anonymous
 
-2017-09-21 14:43 +0000 [a433bb38b5]  Richard Mudgett <rmudgett@digium.com>
+	  then registering the "ip_only" identifier would put the identifier in the
+	  wrong position of the priority list.
 
-	* heap.c: No need to calloc heap pointer array.
+	  * Fix incorrect strncmp() string prefix matching.
 
-	  Change-Id: I5ae2f316229f336eb90d99c7af7ed07a33097e68
+	  Change-Id: Ib8819ec4b811da8a27419fd93528c54d34f01484
 
-2017-09-27 13:45 +0000 [47620ea862]  George Joseph <gjoseph@digium.com>
+2018-01-05 03:33 +0000 [af064eaf13]  Alexander Traud <pabstraud@compuserve.com>
 
-	* logger:  Bring back ability to  turn debug on by source file
+	* BuildSystem: Find ptlib-config on Debian/Ubuntu.
 
-	  Somewhere along the way we lost the ability to debug individual
-	  source files.  For modules, this wasn't a big deal but all the
-	  source files in ./main are in the one "core" module so debugging
-	  individual core capabilities was almost impossible.
+	  The current configure script requires that tool when libpt-dev is installed.
+	  libpt-dev was installed by libopenh323-dev, bacause you wanted to go for H.323
+	  based channel drivers.
 
-	  * Added a test to DEBUG_ATLEAST that also checks __FILE__ instead
-	  of just module name.  Any source file will work even if it's in
-	  a module subdirectory.
+	  ASTERISK-25329
 
-	  Change-Id: Icc0af41837f3b1679dec7af21fa32cd1f7469f6e
+	  Change-Id: I9c6ab78b7246c21536e1d252dcbffe682f63f83d
 
-2017-09-26 11:01 +0000 [d70d7b2f5d]  George Joseph <gjoseph@digium.com>
+2018-01-05 06:42 +0000 [f0c8f04c73]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pjsip_message_filter: Fix regression causing bad contact address
+	* chan_ooh323: Limit outgoinglimit to positive values as intended.
 
-	  The "res_pjsip:  Filter out non SIP(S) requests" commit moved the
-	  filtering of messages to pjproject's PJSIP_MOD_PRIORITY_TRANSPORT_LAYER
-	  in order to filter out incoming bad uri schemes as early as possible.
-	  Since the change affected outgoing messages as well and the TRANSPORT
-	  layer is the last to be run on outgoing messages, we were overwriting
-	  the setting of external_signaling_address (which is set earlier by
-	  res_pjsip_nat) with an internal address.
+	  ASTERISK-27552
 
-	  * pjsip_message_filter now registers itself as a pjproject module
-	  twice.  Once in the TSX layer for the outgoing messages (as it was
-	  originally), then a second time in the TRANSPORT layer for the
-	  incoming messages to catch the invalid uri schemes.
+	  Change-Id: Ifbf9d51e7374ca2e8b27ec568f6770050fc1a854
 
-	  ASTERISK-27295
-	  Reported by: Sean Bright
+2018-01-05 06:19 +0000 [09f339bda5]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I2c90190c43370f8a9d1c4693a19fd65840689c8c
+	* ooh323cDriver: Fix typo in header guard.
 
-2017-09-13 21:31 +0000 [221d8a5c24]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27551
 
-	* res_rtp_asterisk.c: Fix bridge_p2p_rtp_write() reentrancy potential.
+	  Change-Id: I39ff66031e3373e895e2bc47b23a5e860ea4e012
 
-	  The bridge_p2p_rtp_write() has potential reentrancy problems.
+2018-01-05 03:36 +0000 [bc1b4f4d43]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Accessing the bridged RTP members must be done with the instance1 lock
-	  held.  The DTMF and asymmetric codec checks must be split to be done with
-	  the correct RTP instance struct locked.  i.e., They must be done when
-	  working on the appropriate side of the point to point bridge.
+	* BuildSystem: Avoid obsolete warning with HELP_STRING on autoconf.
 
-	  * Forcing the RTP mark bit was referencing the wrong side of the point to
-	  point bridge.  The set mark bit is used everywhere else to set the mark
-	  bit when sending not receiving.
+	  ASTERISK-26046
 
-	  The patches for ASTERISK_26745 and ASTERISK_27158 did not take into
-	  account that not everything carried by RTP uses a codec.  The telephony
-	  DTMF events are not exchanged with a codec.  As a result when
-	  RFC2833/RFC4733 sent digits you would crash if "core set debug 1" is
-	  enabled, the DTMF digits would always get passed to the core even though
-	  the local native RTP bridge is active, and the DTMF digits would go out
-	  using the wrong SSRC id.
+	  Change-Id: I48f05698c235f709225b92bec5aa260fb57d69d1
 
-	  * Add protection for non-format payload types like DTMF when updating the
-	  lastrxformat and lasttxformat.  Also protect against non-format payload
-	  types when checking for asymmetric codecs.
+2018-01-04 15:37 +0000 [cfb88f3ac1]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27292
+	* pbx: Prevent execution of NULL pointer.
 
-	  Change-Id: I6344ab7de21e26f84503c4d1fca1a41579364186
+	  pbx_extension_helper has a check for q->swo.exec == NULL but it doesn't
+	  actually return so we would still run the function.  Fix the return.
+	  Move the 'int res' variable into the only scope which uses it.
 
-2017-09-25 13:09 +0000 [f3b1b64d21]  Sean Bright <sean.bright@gmail.com>
+	  Also fix a copy-paste error in ast_pbx_init which could result in a
+	  crash on allocation failure (we exit with a normal error instead).
 
-	* pjproject: Patch to correct STUN FINGERPRINT usage
+	  Change-Id: I0693af921fdc7f56b6a72a21fb816ed08b960a69
 
-	  Change-Id: I0e453253dff1388b0186b36c754457c1d0d12db6
+2018-01-04 10:50 +0000 [82cf585fb5]  Corey Farrell <git@cfware.com>
 
-2017-09-25 10:59 +0000 [8d2c3effc2]  Richard Mudgett <rmudgett@digium.com>
+	* translators: Don't use ast_module_running_ref.
 
-	* channel.c: Fix invalid reference in conditionaled out code.
+	  Translators are run during module load before the module is actually
+	  running, so it cannot use ast_module_running_ref.
 
-	  ASTERISK-27289
+	  ASTERISK-20346
 
-	  Change-Id: I7a415948116493050614d9f4fa91ffbe0c21ec4c
+	  Change-Id: Iaa0e75da99c696e38000f1a41e340abbd7a88f56
 
-2017-09-25 07:25 +0000 [690f7f7c76]  George Joseph <gjoseph@digium.com>
+2018-01-04 09:39 +0000 [da365affbd]  Corey Farrell <git@cfware.com>
 
-	* build:  A few gcc 7 error fixes
+	* rtp_engine: Add missing unlock.
 
-	  Change-Id: I7b5300fbf1af7d88d47129db13ad6dbdc9b553ec
+	  Change-Id: I380c31a255e060309f4916da11176e0d00813215
 
-2017-09-22 10:02 +0000 [f39af4d36d]  Sean Bright <sean.bright@gmail.com>
+2018-01-04 09:30 +0000 [73bf5035b8]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip: Use ast_sip_is_content_type() where appropriate
+	* res_pjsip_history: Add missing unlock to CLI command.
 
-	  Change-Id: If3ab0d73d79ac4623308bd48508af2bfd554937d
+	  Change-Id: I872060a30543776a176a316309602d924a23eb29
 
-2017-09-19 05:22 +0000 [c98e980fff]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+2018-01-04 09:27 +0000 [aaed0b8b3a]  Corey Farrell <git@cfware.com>
 
-	* res_config_pgsql: Fix removed support to previous for versions PostgreSQL 9.1
+	* aco: Fix NULL dereference in error path.
 
-	  In PostgreSQL 9.1 the backslash are string literals and not the escape
-	  of characters.
+	  Change-Id: Id505167cf0f9414a3c144fa2c1e181a2cf288694
 
-	  In previous issue ASTERISK_26057 was fixed the use of escape LIKE but the
-	  support for old version of Postgresql than 9.1 was dropped. The sentence
-	  before make was "ESCAPE '\'" but in version before than 9.1  need it to be
-	  as follow "ESCAPE '\\'".
+2018-01-03 19:07 +0000 [e3c9314a2e]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27283
+	* func_odbc: Add missing unlock's to acf_odbc_read.
 
-	  Change-Id: I96d9ee1ed7693ab17503cb36a9cd72847165f949
+	  Change-Id: I828329ecbd252ae8f27a369a046d2b03102b07c6
 
-2017-09-15 02:59 +0000 [0adf6f3bd9]  Stefan Engström <stefanen@kth.se>
+2017-12-29 18:24 +0000 [55f1d69c43]  Corey Farrell <git@cfware.com>
 
-	* app_queue: Only do announcement logic between ringing cycles
+	* loader: Create ast_module_running_ref.
 
-	  This patch reverts the change by patch 2263 from old reviewboard.
-	  Note that reverting that 2263-patch still preserves the behaviour that
-	  the commit log of the 2263-patch claimed to add. The reason for this is:
+	  This function returns NULL if the module in question is not running.  I
+	  did not change ast_module_ref as most callers do not check the result
+	  and they always call ast_module_unref.
+
+	  Make use of this function when running registered items from:
+	  * app_stack API's
+	  * bridge technologies
+	  * CLI commands
+	  * File formats
+	  * Manager Actions
+	  * RTP engines
+	  * Sorcery Wizards
+	  * Timing Interfaces
+	  * Translators
+	  * AGI Commands
+	  * Fax Technologies
 
-	  The function wait_for_answer is only called from try_calling which
-	  in turn is only called from the main for loop in queue_exec, and
-	  earlier in that loop we already check the things that's removed by
-	  this patch. There's no need to check those things twice each loop
-	  iteration, and I think the proper place to check it is before each
-	  ringing cycle. By checking it in wait_for_answer, you allow the issue
-	  explained in the jira - that the head caller hears announcements while
-	  the agents' sip phones are actively ringing.
+	  ASTERISK-20346 #close
 
-	  Reported-by: Stefan Engström
-	  Tested-by: Stefan Engström
-	  ASTERISK-27216 #close
+	  Change-Id: Ia16fd28e188b2fc0b9d18b8a5d9cacc31df73fcc
 
-	  Change-Id: Ic4290dc75256f9743900c6762ee1bb915f672db0
+2018-01-03 10:41 +0000 [62f862e2cd]  Kevin Harwell <kharwell@digium.com>
 
-2017-09-07 04:41 +0000 [da40976987]  Jean Aunis <jean.aunis@prescom.fr>
+	* res_pjsip_session: Check if sequence header is missing
 
-	* bridge : Fix one-way direct-media when early bridging with native_rtp
+	  The pjsip_msg_find_hdr function can return NULL. This patch adds a check
+	  when searching for the sequence header to make sure a NULL pointer is never
+	  de-referenced.
 
-	  When two channels were early bridged in a native_rtp bridge, the RTP description
-	  on one side was not updated when the other side answered.
-	  This patch forbids non-answered channels to enter a native_rtp bridge, and
-	  triggers a bridge reconfiguration when an ANSWER frame is received.
+	  Change-Id: I19af23aeeded65be016be92360e8cb7ffe51fad2
 
-	  ASTERISK-27257
+2018-01-02 07:36 +0000 [9b5d1454b4]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  Change-Id: If1aaee1b4ed9658a1aa91ab715ee0a6413b878df
+	* cdr: submit: fix logic of test for batch mode
 
-2017-09-19 10:38 +0000 [828a0611bc]  George Joseph <gjoseph@digium.com>
+	  ASTERISK-27539 #close
 
-	* res_pjsip_pubsub:  Check for Content-Type header in rx_notify_request
+	  Change-Id: I33cdf329d2bb4486dcae975c450f6aae94c515f7
 
-	  pubsub_on_rx_notify_request wasn't checking for a null
-	  Content-Type header before checking that it was
-	  application/simple-message-summary.
+2017-12-29 23:14 +0000 [ffbf5be116]  Sungtae Kim <pchero21@gmail.com>
 
-	  ASTERISK-27279
-	  Reported by: Ross Beer
+	* res_pjsip: Add AMI action 'PJSIPShowAors'
 
-	  Change-Id: Iec2a6c4d2e74af37ff779ecc9fd35644c5c4ea52
+	  Add an AMI action which provides information on all
+	  configured AORs.
 
-2017-09-19 09:34 +0000 [94f616e5e2]  David J. Pryke <david+extra.asterisk@pryke.us>
+	  ASTERISK-27537
 
-	* chan_sip: Expose read-only access to the full SIP INVITE Request-URI
+	  Change-Id: If8b990a00909e5b6c0f04a3b8dccd9903dc445eb
 
-	  Provide a way to get the contents of the the Request URI from the initial SIP
-	  INVITE in dial plan function call. (In this case "${CHANNEL(ruri)}")
+2018-01-02 00:26 +0000 [f298178583]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27278
-	  Reported by: David J. Pryke
-	  Tested by: David J. Pryke
+	* aco: Add missing aco_option_type_string for OPT_TIMELEN_T.
 
-	  Change-Id: I1dd4d6988eed1b6c98a9701e0e833a15ef0dac3e
+	  ASTERISK-27117
 
-2017-09-18 10:27 +0000 [cfc0ca1fb5]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I8f6c34bb30830be9f7a40823723eb4dcaaa91c61
 
-	* tcptls: Fixed a white space error.
+2017-12-31 10:26 +0000 [15f8b9b8bf]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26606
+	* ice: Increase foundation buffer size
 
-	  Change-Id: I81a7268ef7ba012d4d80d44c70b6276d48e397fa
+	  Per RFC 5245, the foundation specified with an ICE candidate can be up
+	  to 32 characters but we are only allowing for 31.
 
-2017-09-18 10:00 +0000 [99a08eb7ab]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-27498 #close
+	  Reported by: Michele Prà
 
-	* res_srtp: lower log level of auth failures
+	  Change-Id: I05ce7a5952721a76a2b4c90366168022558dc7cf
 
-	  Previously, sRTP authentication failures were reported on log level WARNING.
-	  When such failures happen, each RT(C)P packet is affected, spamming the log.
-	  Now, those failures are reported at log level VERBOSE 2. Furthermore, the
-	  amount is further reduced (previously all two seconds, now all three seconds).
-	  Additionally, the new log entry informs whether media (RTP) or statistics (RTCP)
-	  are affected.
+2017-12-29 22:03 +0000 [b32d6d5e2d]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-16898 #close
+	* astobj2: Create case-insensitive variants of container function macros.
 
-	  Change-Id: I6c98d46b711f56e08655abeb01c951ab8e8d7fa0
+	  * AO2_STRING_FIELD_CASE_HASH_FN
+	  * AO2_STRING_FIELD_CASE_CMP_FN
+	  * AO2_STRING_FIELD_CASE_SORT_FN
 
-2017-09-13 03:46 +0000 [f1eb36ea51]  alex <alexandr.revin@gmail.com>
+	  Change-Id: I11af8c6a0c43380a42732553f519c667abb842cf
 
-	* cdr_mysql.c: Apply cdrzone to start and answer
+2017-12-29 22:59 +0000 [bc73337e07]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I7de0a5adc89824a5f2b696fc22c80fc22dff36b0
+	* core: Use macros to generate ao2_container callbacks where possible.
 
-2017-08-25 17:01 +0000 [6d4b801c83]  Richard Mudgett <rmudgett@digium.com>
+	  This uses AO2_STRING_FIELD_HASH_FN and AO2_STRING_FIELD_CMP_FN where
+	  possible in the Asterisk core.
 
-	* AST-2017-008: Improve RTP and RTCP packet processing.
+	  This removes CMP_STOP from the result of CMP_FN callbacks for the
+	  following structure types:
+	  * ast_bucket_metadata
+	  * ast_bucket_scheme
+	  * generic_monitor_instance_list (ccss.c)
+	  * ast_bucket_file (media_cache.c)
+	  * named_acl
 
-	  Validate RTCP packets before processing them.
+	  Change-Id: Ide4c1449a894bce70dea1fef664dade9b57578f1
 
-	  * Validate that the received packet is of a minimum length and apply the
-	  RFC3550 RTCP packet validation checks.
+2017-12-29 14:50 +0000 [0fe7df641a]  Corey Farrell <git@cfware.com>
 
-	  * Fixed potentially reading garbage beyond the received RTCP record data.
+	* datastore: Add automatic module references.
 
-	  * Fixed rtp->themssrc only being set once when the remote could change
-	  the SSRC.  We would effectively stop handling the RTCP statistic records.
+	  Add a reference to the calling module when it is active to protect
+	  access to datastore->info.  Remove module references done by
+	  func_periodic_hook as the datastore now handles it.
 
-	  * Fixed rtp->themssrc to not treat a zero value as special by adding
-	  rtp->themssrc_valid to indicate if rtp->themssrc is available.
+	  ASTERISK-25128 #close
 
-	  ASTERISK-27274
+	  Change-Id: I8357a3711e77591d0d1dd8ab4211a7eedd782c89
 
-	  Make strict RTP learning more flexible.
+2017-12-28 13:27 +0000 [2dde5bef47]  Richard Mudgett <rmudgett@digium.com>
 
-	  Direct media can cause strict RTP to attempt to learn a remote address
-	  again before it has had a chance to learn the remote address the first
-	  time.  Because of the rapid relearn requests, strict RTP could latch onto
-	  the first remote address and fail to latch onto the direct media remote
-	  address.  As a result, you have one way audio until the call is placed on
-	  and off hold.
+	* stasis_channels.c: Misc cleanup.
 
-	  The new algorithm learns remote addresses for a set time (1.5 seconds)
-	  before locking the remote address.  In addition, we must see a configured
-	  number of remote packets from the same address in a row before switching.
+	  * Use current OBJ_SEARCH_xxx defines instead of the deprecated versions.
 
-	  * Fixed strict RTP learning from always accepting the first new address
-	  packet as the new stream.
+	  * Fix hash_cb and cmp_cb container functions to correctly use the
+	  OBJ_SEARCH_xxx values.
 
-	  * Fixed strict RTP to initialize the expected sequence number with the
-	  last received sequence number instead of the last transmitted sequence
-	  number.
+	  * Remove incorrect usage of CMP_STOP.  Most uses in the system have no
+	  effect.  This allows the collapse of channel_role_single_cmp_cb() and
+	  channel_role_multi_cmp_cb() into channel_role_cmp_cb().
 
-	  * Fixed the predicted next sequence number calculation in
-	  rtp_learning_rtp_seq_update() to handle overflow.
+	  * Remove unnecessary usage of RAII_VAR().
 
-	  ASTERISK-27252
+	  Change-Id: I02c405518cab22aa2a082b61e2353bf7cd629a70
 
-	  Change-Id: Ia2d3aa6e0f22906c25971e74f10027d96525f31c
+2017-12-13 15:43 +0000 [898b3b080a]  Sean Bright <sean.bright@gmail.com>
 
-2017-09-13 14:14 +0000 [5075cc8eed]  Sean Bright <sean.bright@gmail.com>
+	* cdr_mysql: Make sure connection charset is always set
 
-	* res_calendar: On reload, update all configuration
+	  When the MYSQL_OPT_RECONNECT option is enabled, the MySQL client API
+	  will transparently reconnect when it needs to. Ideally this simplifies
+	  our code, but when this reconnection occurs all connection state is
+	  lost. Because we are not notified that this has happened, we don't know
+	  to set our character set again (with "SET NAMES 'xyz'").
 
-	  This changes the behavior of res_calendar to drop all existing calendars
-	  and re-create them whenever a reload is done. The Calendar API provides
-	  no way for configuration information to be pushed down to calendar
-	  'techs' so updated settings would not take affect until a module
-	  unload/load was done or Asterisk was restarted.
+	  Rather than calling SET NAMES, we instead set the MYSQL_SET_CHARSET_NAME
+	  option which will do it for us under the hood on each connect. This
+	  option has been present in the MySQL C API for at least 15 years, so it
+	  should be safe for most installations.
 
-	  Asterisk 15+ already has a configuration option 'fetch_again_at_reload'
-	  that performs a similar function.
+	  I also snuck a few other changes into this patch:
 
-	  Also fix a tiny memory leak in res_calendar_caldav while we're at it.
+	  * Default the MySQL port to MYSQL_PORT (3306) instead of 0 if it's not
+	    defined.
 
-	  ASTERISK-25524 #close
-	  Reported by: Jesper
+	  * Fix some erroneous and/or silly checks on the contents of the
+	    configuration ast_str values.
 
-	  Change-Id: Ib0f8057642e9d471960f1a79fd42e5a3ce587d3b
+	  ASTERISK-27366 #close
+	  Reported by: Halil İbrahim YILDIZ
 
-2017-09-13 16:23 +0000 [63900374fa]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I36bf8dc5d5f83584e803b3b1a151dea9396ab8f5
 
-	* res_pjsip:  Filter out non SIP(S) requests
+2017-12-27 20:48 +0000 [d69b7c6c6d]  Richard Mudgett <rmudgett@digium.com>
 
-	  Incoming requests with non sip(s) URIs in the Request, To, From
-	  or Contact URIs are now rejected with
-	  PJSIP_SC_UNSUPPORTED_URI_SCHEME (416).  This is performed in
-	  pjsip_message_filter (formerly pjsip_message_ip_updater) and is
-	  done at pjproject's "TRANSPORT" layer before a request can even
-	  reach the distributor.
+	* manager.c: Update AMI Status event documentation
 
-	  URIs read by res_pjsip_outbound_publish from pjsip.conf are now
-	  also checked for both length and sip(s) scheme.  Those URIs read
-	  by outbound registration and aor were already being checked for
-	  scheme but their error messages needed to be updated to include
-	  scheme failure as well as length failure.
+	  The AMI Status event had linkedid listed twice and was missing the
+	  effective connected line name and number headers.
 
-	  Change-Id: Ibb2f9f1d2dc7549da562af4cbd9156c44ffdd460
+	  NOTE: The linkedid and other standard channel snapshot fields in the XML
+	  documentation are part of the <channel_snapshot/> XML template defined in
+	  doc/appdocsxml.xslt.
 
-2017-09-13 14:08 +0000 [db785ddb92]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I004c4c4f9e7b40ef55035c831702721bec82496c
 
-	* res_calendar: Various fixes
+2017-12-27 22:36 +0000 [fa36f9c01b]  Richard Mudgett <rmudgett@digium.com>
 
-	  * The way that we were looking at XML elements for CalDAV was extremely
-	    fragile, so use SAX2 for increased robustness.
+	* bridge_native_rtp.c: Fix reentrancy framehook crash.
 
-	  * Don't complain about a 'channel' not be specified if autoreminder is
-	    not set. Assume that if 'channel' is not set, we don't want to be
-	    notified.
+	  If two channels enter different native rtp bridges at the same time it is
+	  possible that the framehook interface data pointer can be corrupted
+	  because the struct variable was declared static.
 
-	  * Fix some truncated CLI output in 'calendar show calendar' and make the
-	    'Autoreminder' description a bit more clear
+	  * Fixed the reentrancy corruption by changing the framehook interface
+	  struct static variable to a stack local variable.
 
-	  ASTERISK-24588 #close
-	  Reported by: Stefan Gofferje
+	  * Moved the hook.data assignment outside of the channel lock.  It did not
+	  need the lock's protection.  It probably was giving a false sense of
+	  security.
 
-	  ASTERISK-25523 #close
-	  Reported by: Jesper
+	  The testsuite
+	  channels/pjsip/basic_calls/two_parties/nominal/alice_initiated/bob_hangs_up
+	  test caught this with MALLOC_DEBUG and DO_CRASH enabled.
 
-	  Change-Id: I200d11afca6a47e7d97888f286977e2e69874b2c
+	  Change-Id: If9e35b97d19209b0f984941c1d8eb5f7c55eea91
 
-2017-09-13 09:38 +0000 [0688f61a01]  Sean Bright <sean.bright@gmail.com>
+2017-12-27 20:22 +0000 [1d3dc9aea2]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_rtp: Use μ-law by default instead of signed linear
+	* func_channel.c: Update MASTER_CHANNEL documentation
 
-	  Multicast/Unicast RTP do not use SDP so we need to use a format that
-	  cleanly maps to one of the static RTP payload types. Without this
-	  change, an Originate to a Multicast or Unicast channel without a format
-	  specified would produce no audio on the receiving device.
+	  Be more explicit in what is meant by the master channel to eliminate
+	  misunderstanding.
 
-	  ASTERISK-21399 #close
-	  Reported by: Tzafrir Cohen
+	  ASTERISK-23133
 
-	  Change-Id: I97e332b566e85da04b0004b9b0daae746cfca0e3
+	  Change-Id: I453bcaf4b99404a5a3e345dbf093ac6c1afcfc72
 
-2017-09-11 05:46 +0000 [ed2a4ee81e]  George Joseph <gjoseph@digium.com>
+2017-12-27 19:27 +0000 [6338a03ce9]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip:  Add handling for incoming unsolicited MWI NOTIFY
+	* menuselect: Fix check for running configure.
 
-	  A new endpoint parameter "incoming_mwi_mailbox" allows Asterisk to
-	  receive unsolicited MWI NOTIFY requests and make them available to
-	  other modules via the stasis message bus.
+	  menuselect/Makefile checks that autoconfig.h and makeopts were newer
+	  than the '.in' files.  Unfortunately running ./configure does not touch
+	  autoconfig.h unless the contents will change.
 
-	  res_pjsip_pubsub has a new handler "pubsub_on_rx_mwi_notify_request"
-	  that parses a simple-message-summary body and, if
-	  endpoint->incoming_mwi_account is set, calls ast_publish_mwi_state
-	  with the voice-message counts from the message.
+	  Instead of looking at autoconfig.h we just need to ensure that makeopts
+	  is newer than configure.
 
-	  Change-Id: I08bae3d16e77af48fcccc2c936acce8fc0ef0f3c
+	  Also make change to configure.ac so bootstrap.sh doesn't re-add the
+	  extra trailing line-feed.
 
-2017-09-08 21:41 +0000 [044674c0cd]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ief1f831d6717007f9cebb668c14e92782cd2b794
 
-	* res_rtp_asterisk.c: Add doxygen to RTCP payload types.
+2017-12-21 23:56 +0000 [94eb12ca56]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I3f20ce428777cc4ce9c13b2f808d29ff8c873998
+	* cdr: Missing NULL check and unlock.
 
-2017-09-11 05:52 +0000 [5ff2d06aa6]  George Joseph <gjoseph@digium.com>
+	  * handle_dial_message: Missing a check for NULL peer.
+	  * cdr_generic_register: Missing unlock on allocation failure.
 
-	* alembic:  Fix typo in add_auto_info_to_endpoint_dtmf_mode
+	  cdr_generic_register is fixed by reordering so the new structure is
+	  allocated and initialized before locking the list.
 
-	  The downgrade function was missing "_v2" at the end of the
-	  alter column type.
+	  Change-Id: I5799b99270d1a7a716a555c31ac85f4b00ce8686
 
-	  Change-Id: Iaa9bcef48d6f3590ce07a61342d8e66f00263d8e
+2017-12-23 22:51 +0000 [23aa20bf20]  Corey Farrell <git@cfware.com>
 
-2017-09-10 06:17 +0000 [babb617f20]  Walter Doekes <walter+asterisk@wjd.nu>
+	* loader: Add volatile to resource_being_loaded.
 
-	* res/res_pjsip: Fix localnet checks in pjsip, part 2.
+	  Some compiler optimizers seem to assume that dlopen will not use
+	  __attribute__((constructor)) functions to call back to the program.
+	  This was causing resource_being_loaded to be optimized away completely.
 
-	  In 45744fc53, I mistakenly broke SDP media address rewriting by
-	  misinterpreting which address was checked in the localnet comparison.
+	  ASTERISK-27531 #close
+	  Tested By: abelbeck
 
-	  Instead of checking the remote peer address to decide whether we need
-	  media address rewriting, we check our local media address: if it's
-	  local, then we rewrite. This feels awkward, but works and even made
-	  directmedia work properly if you set local_net. (For the record: for
-	  local peers, the SDP media rewrite code is not called, so the
-	  comparison does no harm there.)
+	  Change-Id: If17a3b889e06811a0e7119f0539d052494d6ece9
 
-	  ASTERISK-27248 #close
+2017-12-20 16:17 +0000 [553306548c]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I566be1c33f4d0a689567d451ed46bab9c3861d4f
+	* AST-2017-014: res_pjsip - Missing contact header can cause crash
 
-2017-09-05 11:13 +0000 [ad606844be]  Florian Floimair <f.floimair@commend.com>
+	  Those SIP messages that create dialogs require a contact header to be present.
+	  If the contact header was missing from the message it could cause Asterisk to
+	  crash.
 
-	* alembic: Add support for MS-SQL
+	  This patch checks to make sure SIP messages that create a dialog contain the
+	  contact header. If the message does not and it is required Asterisk now returns
+	  a "400 Missing Contact header" response. Also added NULL checks when retrieving
+	  the contact header that were missing as a "just in case".
 
-	  MS-SQL has no native Enum-type support and therefore
-	  needs to work with constraints.
-	  Since these constraints need unique names the suggested approach
-	  referenced in the following alembic documentation has been applied:
-	  http://bit.ly/2x9r8pb
+	  ASTERISK-27480 #close
 
-	  ASTERISK-27255 #close
+	  Change-Id: I1810db87683fc637a9e3e1384a746037fec20afe
 
-	  Change-Id: I4a399ba3eed41a33ce8cb294968ad340221580ee
+2017-12-22 14:00 +0000 [c2529a352c]  Corey Farrell <git@cfware.com>
 
-2017-09-05 07:31 +0000 [2aefc6e5fe]  Jacek Konieczny <j.konieczny@eggsoft.pl>
+	* astobj.h: Remove from Asterisk core.
 
-	* func_cdr: honour 'u' flag on dummy channel
+	  This is the old ASTOBJ macro's which are no longer used except by the
+	  deprecated netsock.c.  Move it to the chan_iax2 include folder so it
+	  does not get used elsewhere.
 
-	  Fixes ${CDR(...,u)} when used in cdr_custom.conf
+	  Change-Id: I7e4ae96678b36b9f41d3cae14b167f110eb5d349
 
-	  ASTERISK-27165 #close
+2017-12-22 08:23 +0000 [fd0ca1c3f9]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ia4e0b6ba93e03d27886354c279737790e2cd6a83
+	* Remove as much trailing whitespace as possible.
 
-2017-09-06 16:05 +0000 [c0d4f1880e]  Scott Griepentrog <scott@griepentrog.com>
+	  Change-Id: I873c1c6d00f447269bd841494459efccdd2c19c0
 
-	* chan_sip: when getting sip pvt return failure if not found
+2017-12-21 09:51 +0000 [a1a179c09d]  Sean Bright <sean.bright@gmail.com>
 
-	  In handle_request_invite, when processing a pickup, a call
-	  is made to get_sip_pvt_from_replaces to locate the pvt for
-	  the subscription. The pvt is assumed to be valid when zero
-	  is returned indicating no error, and is dereferenced which
-	  can cause a crash if it was not found.
+	* Fix some invalid Unicode characters
 
-	  This change checks the not found case and returns -1 which
-	  allows the calling code to fail appropriately.
+	  configs/samples/minivm.conf.sample contains invalid UTF-8, but that
+	  appears to be intentional.
 
-	  ASTERISK-27217 #close
-	  Reported-by: Bryan Walters
+	  Change-Id: I7b1e0d332f3380fd0425962a3c9c55f9b200c8cc
 
-	  Change-Id: I6bee92b8b8b85fcac3fd66f8c00ab18bc1765612
+2017-12-20 21:11 +0000 [f2f51ff4ea]  Corey Farrell <git@cfware.com>
 
-2017-09-06 10:50 +0000 [e4797b2cbd]  Sean Bright <sean.bright@gmail.com>
+	* app_voicemail: Fix file copy error handling.
 
-	* app_waitforsilence: Cleanup & don't treat missing frames as 'noise'
+	  Fix error where input/output file descriptors would be closed multiple
+	  times.
 
-	  * WaitForSilence completes successfully if it receives no media in the
-	    specified timeout, but when acting as WaitForNoise that logic needs
-	    to be reversed.
+	  Change-Id: Iba5140b60cb7de79e3d5d92be3c256947aa99da9
 
-	  * Use standard argument parsing macros and add some error checking for
-	    invalid values.
+2017-12-20 14:54 +0000 [9415ec2877]  Sean Bright <sean.bright@gmail.com>
 
-	  * The documentation indicated that the first argument to both
-	    WaitForSilence and WaitForNoise was required when it was not. Update
-	    the documentation to reflect that.
+	* docs: Remove old API changes documentation
 
-	  * Wrap up some behavior in structs to avoid boolean checks all over the
-	    place.
+	  Change-Id: I1bc7957121cc7ae27dca04acc3613f4e1858476a
 
-	  ASTERISK-24066 #close
-	  Reported by: M vd S
+2017-12-20 11:14 +0000 [1b80ffa495]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I01d40adc5b63342bb5018a1bea2081a0aa191ef9
+	* Fix Common Typo's.
 
-2017-09-01 05:17 +0000 [186ef1a657]  George Joseph <gjoseph@digium.com>
+	  Fix instances of:
+	  * Retreive
+	  * Recieve
+	  * other then
+	  * different then
+	  * Repeated words ("the the", "an an", "and and", etc).
+	  * othterwise, teh
 
-	* stasis/control:  Fix possible deadlock with swap channel
+	  ASTERISK-24198 #close
 
-	  If an error occurs during a bridge impart it's possible that
-	  the "bridge_after" callback might try to run before
-	  control_swap_channel_in_bridge has been signalled to continue.
-	  Since control_swap_channel_in_bridge is holding the control lock
-	  and the callback needs it, a deadlock will occur.
+	  Change-Id: I3809a9c113b92fd9d0d9f9bac98e9c66dc8b2d31
 
-	  * control_swap_channel_in_bridge now only holds the control
-	    lock while it's actually modifying the control structure and
-	    releases it while the bridge impart is running.
-	  * bridge_after_cb is now tolerant of impart failures.
+2017-12-20 11:30 +0000 [3625e91586]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ifd239aa93955b3eb475521f61e284fcb0da2c3b3
+	* manager.h: Bump AMI version
 
-2017-09-06 05:23 +0000 [597d1f8951]  Vitezslav Novy <a1@vnovy.net>
+	  Change-Id: I62e6ddeb261ef012687e1fb6734c554e2499b6bf
 
-	* chan_sip: Do not change IP address in SDP origin line (o=) in SIP reINVITE
+2017-12-20 10:23 +0000 [aaa3884d4a]  Corey Farrell <git@cfware.com>
 
-	  If directmedia=yes is configured, when call is answered, Asterisk sends reINVITE
-	  to both parties to set up media path directly between the endpoints.
-	  In this reINVITE msg SDP origin line (o=) contains IP address of endpoint
-	  instead of IP of asterisk. This behavior violates RFC3264, sec 8:
-	  "When issuing an offer that modifies the session,
-	  the "o=" line of the new SDP MUST be identical to that in the
-	  previous SDP, except that the version in the origin field MUST
-	  increment by one from the previous SDP."
-	  This patch assures IP address of Asterisk is always sent in
-	  SDP origin line.
+	* bridge: Old channel video source not set to NULL after unref.
 
-	  ASTERISK-17540
-	  Reported by:  saghul
+	  The bridge holds onto the old channel video source after it's been
+	  released.  This can lead to use after free errors.
 
-	  Change-Id: I533a047490c43dcff32eeca8378b2ba02345b64e
+	  ASTERISK-27229 #close
 
-2017-09-06 07:54 +0000 [15ddc9acb3]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ib2dab61677dd8a21f7ad53cdc9b8ca93297838b3
 
-	* alembic: Fix enum creation for dtls_fingerprint
+2017-12-20 10:13 +0000 [c2850bfebc]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ic061c5066a146616a68376881c7e4cf6d6e7e7db
+	* core: Fix unused variable error in handle_show_sysinfo.
 
-2017-09-05 11:08 +0000 [2370469645]  Florian Floimair <f.floimair@commend.com>
+	  Apparently in OSX it's possible for OSX to HAVE_SYSCTL but not
+	  HAVE_SYSINFO or HAVE_SWAPCTL.  In this case freeswap caused an unused
+	  variable error.
 
-	* alembic: fix erroneous commit for add_prune_on_boot
+	  ASTERISK-26563 #close
 
-	  Added include for postgresql ENUM type and
-	  redefined values in the same way as in the
-	  other migration scripts.
+	  Change-Id: I8ec5b1897b786cc1abaf62264aa75039eea05510
 
-	  ASTERISK-27254 #close
+2017-12-20 00:53 +0000 [fff7782cf5]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Id667304cdf3891b1c2f7d35fab3e2a84026159fa
+	* app_festival: Fix fd leak on connection failure.
 
-2017-09-06 03:15 +0000 [13aa1241c3]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: If5efaddcf735ff7d17e55c36cc1388946cee9e0f
 
-	* res_srtp: Add support for libsrtp2.1.
+2017-12-18 20:12 +0000 [d51837a1b9]  Corey Farrell <git@cfware.com>
 
-	  Asterisk is able to use libSRTP 2.0.x. However since libSRTP 2.1.x, the macro
-	  SRTP_AES_ICM got renamed to SRTP_AES_ICM_128. Beside to still compile with
-	  previous versions of libSRTP, this change allows libSRTP 2.1.x as well.
+	* CLI: Address multiple issues.
 
-	  ASTERISK-27253 #close
+	  * listen uses the variable `s` for the result from ast_poll() then
+	    overwrites it with the result of accept().  Create a separate variable
+	    poll_result to avoid confusion since ast_poll does not return a file
+	    descriptor.
+	  * Resolve fd leak that would occur if setsockopt failed in listen.
+	  * Reserve an extra byte while processing completion results from remote
+	    daemon.  This fixes a bug where completion processing used strstr() on
+	    a string that was not '\0' terminated.  This was no risk to the Asterisk
+	    daemon, the bug was only reachable the remote console process.
+	  * Resolve leak in handle_showchan when the channel is not found.
+	  * Multiple leaks and a deadlock in pbx_config CLI completion.
+	  * Fix leaks in "manager show command".
 
-	  Change-Id: I2e6eb3c3bc844fee8a624060a2eb6f182dc70315
+	  Change-Id: I8f633ceb1714867ae30ef4e421858f77c14485a9
 
-2017-09-05 09:35 +0000 [598a18ffee]  Ben Ford <bford@digium.com>
+2017-12-18 22:48 +0000 [b8f54f742f]  Corey Farrell <git@cfware.com>
 
-	* chan_pjsip: Suppress frame warnings.
+	* dns_core: Protect against array index violation.
 
-	  When rtp_keepalive is on for a PJSIP endpoint dialing to another
-	  Asterisk instance also using PJSIP, Asterisk will continue to print
-	  warning messages about not being able to send frames of a certain
-	  type. This suppresses that warning message.
+	  Add a check to allocate_dns_record to prevent calling a pointer
+	  retrieved from beyond dns_alloc_table.
 
-	  Change-Id: I0332a05519d7bda9cacfa26d433909ff1909be67
+	  ASTERISK-27495 #close
 
-2017-08-25 17:05 +0000 [6c922b3157]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie2f6e4991cea46baa12e837bd64cc22b44d322bb
 
-	* res_rtp_asterisk.c: Check RTP packet version earlier.
+2017-12-18 18:59 +0000 [3c037ef972]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ic6493a7d79683f3e5845dff1cee49445fd5a0adf
+	* chan_sip: Fix memory leaks.
 
-2017-09-05 10:05 +0000 [3f7d0b63fc]  Sean Bright <sean.bright@gmail.com>
+	  In change_redirecting_information variables we use ast_strlen_zero to
+	  see if a value should be saved.  In the case where the value is not NULL
+	  but is a zero length string we leaked.
 
-	* formats: Restore previous fread() behavior
+	  handle_response_subscribe leaked a reference to the ccss monitor
+	  instance.
 
-	  Some formats are able to handle short reads while others are not, so
-	  restore the previous behavior for the format modules so that we don't
-	  have spurious errors when playing back files.
+	  Change-Id: Ib11444de69c3d5b2360a88ba2feb54d2c2e9f05f
 
-	  ASTERISK-27232 #close
-	  Reported by: Jens T.
+2017-12-16 07:51 +0000 [3b99a0332c]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-	  Change-Id: Iab7f52b25a394f277566c8a2a4b15a692280a300
+	* bridge: Stop music on hold on adding an arbitrary channel to a bridge
 
-2017-09-05 09:16 +0000 [45744fc53d]  Walter Doekes <walter+asterisk@wjd.nu>
+	  When a channel that is on hold gets added to a bridge by
+	  the Bridge AMI action or the dialplan application of the same name,
+	  music continues to play, causing "robotic sound".
 
-	* res/res_pjsip: Standardize/fix localnet checks across pjsip.
+	  This commit adds a call to ast_moh_stop to stop the music.
+	  Also, it makes the AMI Park action use the right MOH class when the
+	  channel gets parked.
 
-	  In 2dee95cc (ASTERISK-27024) and 776ffd77 (ASTERISK-26879) there was
-	  confusion about whether the transport_state->localnet ACL has ALLOW or
-	  DENY semantics.
+	  Reported by: Zane Conkle
 
-	  For the record: the localnet has DENY semantics, meaning that "not in
-	  the list" means ALLOW, and the local nets are in the list.
+	  ASTERISK-25079 #close
 
-	  Therefore, checks like this look wrong, but are right:
+	  Change-Id: I4b129c5a20c15e63968842460ac5a1a85903cf9f
 
-	      /* See if where we are sending this request is local or not, and if
-	         not that we can get a Contact URI to modify */
-	      if (ast_apply_ha(transport_state->localnet, &addr) != AST_SENSE_ALLOW) {
-	          ast_debug(5, "Request is being sent to local address, "
-	                       "skipping NAT manipulation\n");
+2017-12-18 15:36 +0000 [b3e839debd]  Corey Farrell <git@cfware.com>
 
-	  (In the list == localnet == DENY == skip NAT manipulation.)
+	* Remove constant conditionals (dead-code).
 
-	  And conversely, other checks that looked right, were wrong.
+	  Some variables are set and never changed, making them constant.  This
+	  means that code in the 'false' block of the conditional is unreachable.
 
-	  This change adds two macro's to reduce the confusion and uses those
-	  instead:
+	  In chan_skinny and res_config_ldap I used preprocessor directive `#if 0`
+	  as I'm unsure if the unreachable code could be enabled in the future.
 
-	      ast_sip_transport_is_nonlocal(transport_state, addr)
-	      ast_sip_transport_is_local(transport_state, addr)
+	  Change-Id: I62e2aac353d739fb3c983cf768933120f5fba059
 
-	  ASTERISK-27248 #close
+2017-12-19 02:50 +0000 [c02e256407]  Oron Peled <oron.peled@xorcom.com>
 
-	  Change-Id: Ie7767519eb5a822c4848e531a53c0fd054fae934
+	* chan_console: Use correct parameter for 'set active'
 
-2017-09-05 05:23 +0000 [786c4791f9]  George Joseph <gjoseph@digium.com>
+	  chan_console supports multiple devices but the CLI only works on a
+	  single device. 'console set active' selects this device.
 
-	* res_pjsip_t38:  Make t38_reinvite_response_cb tolerant of NULL channel
+	  Sadly that CLI picks the wrong command-line parameter and will only
+	  work for a device called 'active'.
 
-	  t38_reinvite_response_cb can get called by res_pjsip_session's
-	  session_inv_on_tsx_state_changed in situations where session->channel
-	  is NULL.  If it is, the ast_log warning segfaults because it tries
-	  to get the channel name from a NULL channel.
+	  ASTERISK-27490 #close
 
-	  * Check session->channel and print "unknown channel" when it's NULL.
+	  Change-Id: I2f0e5fe63db19845bee862575b739360797dc73d
 
-	  ASTERISK-27236
-	  Reported by: Ross Beer
+2017-12-18 23:17 +0000 [bf33a09c37]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I4326e288d36327f6c79ab52226d54905cdc87dc7
+	* core: Fix multiple trivial issues in the core.
 
-2017-09-01 16:17 +0000 [55f30c29fd]  Sean Bright <sean.bright@gmail.com>
+	  * Fix small leaks in from error conditions in sdp.c and translate.c.
+	  * Check new file descriptor is less than 0, not less than or equal.
 
-	* rtp_engine: Prevent possible double free with DTLS config
+	  Change-Id: Id7782775486175c739e0c4bf3ea5e17e3f452a99
 
-	  ASTERISK-27225 #close
-	  Reported by: Richard Kenner
+2017-12-18 06:14 +0000 [81474dfb23]  Aaron An <anjb@ti-net.com.cn>
 
-	  Change-Id: I097b81734ef730f8603c0b972909d212a3a5cf89
+	* res_rtp_asterisk:  Avoid close the rtp/rtcp fd twice.
 
-2017-09-01 13:15 +0000 [f36db2dbdc]  Sean Bright <sean.bright@gmail.com>
+	  When RTCP-MUX enabled. rtp->s is the same as rtcp->s, check this before
+	  close the file descriptor. Close the FD twice will hangs the asterisk
+	  under heavy load.
 
-	* chan_ooh323: Fix confusing indentation warning
+	  ASTERISK-27299 #close
+	  Reported-by: Aaron An
+	  Tested-by: AaronAn
 
-	  ASTERISK-27177 #close
-	  Reported by: Tzafrir Cohen
+	  Change-Id: I870a072d73fd207463ac116ef97100addbc0820a
 
-	  Change-Id: I40311c404edb2302a7543ad5ca7a06b2a38f2d97
+2017-12-18 19:47 +0000 [8dfc973d64]  Corey Farrell <git@cfware.com>
 
-2017-09-01 09:51 +0000 [5f4863d4f9]  Sean Bright <sean.bright@gmail.com>
+	* main/app: Fix leaks.
 
-	* app_directory: Handle a NULL mailbox without crashing
+	  * ast_linear_stream would leak a file descriptor if it failed to allocate
+	    lin.
+	  * ast_control_tone leaked zone and ts if ast_playtones_start failed.
 
-	  ASTERISK-27241 #close
-	  Reported by: David Moore
+	  Additionally added whitespace to ast_linear_stream, pulled assignments
+	  out of conditionals for improved readability.
 
-	  Change-Id: Ibbbca85517b04c315406ebfe3b6f7e0763daedc6
+	  Change-Id: I6d1a10cf9161b1529d939b9b2d63ea36d395b657
 
-2017-07-24 10:48 +0000 [990b017668]  George Joseph <gjoseph@digium.com>
+2017-12-18 19:19 +0000 [a790ced2e8]  Corey Farrell <git@cfware.com>
 
-	* pjsip_message_ip_updater:  Fix issue handling "tel" URIs
+	* func_callerid: Initialize app argument structures.
 
-	  sanitize_tdata was assuming all URIs were SIP URIs so when a non
-	  SIP uri was in the From, To or Contact headers, the unconditional
-	  cast of a non-pjsip_sip_uri structure to pjsip_sip_uri caused
-	  a segfault when trying to access uri->other_param.
+	  This module uses AST_DEFINE_APP_ARGS_TYPE to define struct's instead of
+	  directly using AST_DECLARE_APP_ARGS.  Initialize the variables declared
+	  in this way.
 
-	  * Added PJSIP_URI_SCHEME_IS_SIP(uri) || PJSIP_URI_SCHEME_IS_SIPS(uri)
-	    checks before attempting to cast or use the returned uri.
+	  Change-Id: If97fbdd8d63a204e2efd498a192effc14e90fb31
 
-	  ASTERISK-27152
-	  Reported-by: Ross Beer
+2017-08-11 17:02 +0000 [4c04e13783]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Id380df790e6622c8058a96035f8b8f4aa0b8551f
+	* bridge_softmix.c: Change remove_destination_streams() return meaning.
 
-2017-07-01 19:24 +0000 [04ee3eb774]  Corey Farrell <git@cfware.com>
+	  The return value of remove_destination_streams() now means we removed a
+	  stream from the topology by making it a dead stream.  Now we won't try to
+	  request a topology change if we didn't remove any streams.
 
-	* AST-2017-006: Fix app_minivm application MinivmNotify command injection
+	  Change-Id: Icd91571d856a1d04299a24c411e325c1d9d5c61d
 
-	  An admin can configure app_minivm with an externnotify program to be run
-	  when a voicemail is received.  The app_minivm application MinivmNotify
-	  uses ast_safe_system() for this purpose which is vulnerable to command
-	  injection since the Caller-ID name and number values given to externnotify
-	  can come from an external untrusted source.
+2017-08-11 16:57 +0000 [ea4179599f]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Add ast_safe_execvp() function.  This gives modules the ability to run
-	  external commands with greater safety compared to ast_safe_system().
-	  Specifically when some parameters are filled by untrusted sources the new
-	  function does not allow malicious input to break argument encoding.  This
-	  may be of particular concern where CALLERID(name) or CALLERID(num) may be
-	  used as a parameter to a script run by ast_safe_system() which could
-	  potentially allow arbitrary command execution.
+	* bridge_softmix.c: Don't match dead streams.
 
-	  * Changed app_minivm.c:run_externnotify() to use the new ast_safe_execvp()
-	  instead of ast_safe_system() to avoid command injection.
+	  * Made is_video_source() and is_video_dest() not match dead streams.
 
-	  * Document code injection potential from untrusted data sources for other
-	  shell commands that are under user control.
+	  * Optimized is_video_dest() to reduce duplicated code.
 
-	  ASTERISK-27103
+	  Change-Id: I4e7ab762c7ee98395e78e6516399f57a2609b9a1
 
-	  Change-Id: I7552472247a84cde24e1358aaf64af160107aef1
+2017-12-18 18:40 +0000 [91d9eae79b]  Corey Farrell <git@cfware.com>
 
-2017-05-22 10:36 +0000 [1a022285dd]  Joshua Colp <jcolp@digium.com>
+	* bridge_softmix: Fix memory leaks.
 
-	* res_rtp_asterisk: Only learn a new source in learn state.
+	  Change-Id: Ifaf3e93b398595d21d07f535330fef77ff15a80c
 
-	  This change moves the logic which learns a new source address
-	  for RTP so it only occurs in the learning state. The learning
-	  state is entered on initial allocation of RTP or if we are
-	  told that the remote address for the media has changed. While
-	  in the learning state if we continue to receive media from
-	  the original source we restart the learning process. It is
-	  only once we receive a sufficient number of RTP packets from
-	  the new source that we will switch to it. Once this is done
-	  the closed state is entered where all packets that do not
-	  originate from the expected source are dropped.
+2015-11-11 17:20 +0000 [f6393b59af]  Richard Mudgett <rmudgett@digium.com>
 
-	  The learning process has also been improved to take into
-	  account the time between received packets so a flood of them
-	  while in the learning state does not cause media to be switched.
+	* ast_json_pack(): Use safer json ref mechanism.
 
-	  Finally RTCP now drops packets which are not for the learned
-	  SSRC if strict RTP is enabled.
+	  Change-Id: I49204db2e57ae96eee43909c18ed007c09ac817e
 
-	  ASTERISK-27013
+2017-12-18 18:04 +0000 [dc04d1ec93]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I56a96e993700906355e79bc880ad9d4ad3ab129c
+	* app_voicemail: Fix memory management issues.
 
-2017-08-29 14:22 +0000 [4aaccb7795]  Richard Mudgett <rmudgett@digium.com>
+	  * mwi_sub_event_cb: mwist leaked on separate_mailbox failure.
+	  * add_email_attachment: A reference to sox_gain_tmpdir was used
+	    after the storage was out of scope.
 
-	* bridge_native_rtp.c: Fixup native_rtp_framehook()
+	  Change-Id: I6282c542ff7b82fa091177a912d11234a8b00a30
 
-	  * Fix framehook to test frame type for control frame.
-	  * Made framehook exit early if frame type is not a control frame.
-	  * Eliminated RAII_VAR in framehook.
-	  * Use switch instead of else-if ladder for control frame handling.
+2015-11-11 16:52 +0000 [7054fb8756]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ia555fc3600bd85470e3c0141147dbe3ad07c1d18
+	* rtp_engine.c: Eliminate rtcp_report_to_json() RAII_VAR usage.
 
-2017-08-29 09:26 +0000 [d2ace23248]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I58a22c2ca82e91d7537409b7b3af2d735827a54d
 
-	* confbridge: Handle user hangup during name recording
+2017-12-06 20:35 +0000 [5335ad117d]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  This prevents orphaned CBAnn channels from getting stuck in the bridge.
+	* app_queue: Add feature to set wrapuptime on the queue member
 
-	  ASTERISK-26994 #close
-	  Reported by: James Terhune
+	  This patch adds the ability to set the wrapuptime on the queue member
+	  config.
 
-	  Change-Id: I5e43e832a9507ec3f2c59752cd900b41dab80457
+	  When the option is set the wrapuptime on the queue member is used instead
+	  of the queue's wrapuptime.
 
-2017-08-25 21:06 +0000 [a45af32983]  Andre Nazario <samoied@users.sourceforge.net>
+	  ASTERISK-27483 #close
 
-	* chan_pjsip: Add tag info in CHANNEL function
+	  Change-Id: I11c85809537f974eb44dc5bbf82bcedd8a458902
 
-	  Create local_tag and remote_tag in CHANNEL info to get tag from From and
-	  To headers of a SIP dialog.
+2017-12-18 14:00 +0000 [064c74e4af]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27220
+	* netsock: Remove from Asterisk core.
 
-	  Change-Id: I59b16c4b928896fcbde02ad88f0e98922b15d524
+	  This moves netsock.c / netsock.h to the chan_iax2 module.  netsock.h has
+	  been marked deprecated since 13.0.0, chan_iax2 is the only remaining
+	  user.
 
-2017-08-25 13:44 +0000 [9e6efcace5]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I28c6578043bac18de5ea608e136acec4f83d5dd3
 
-	* voicemail: Fix various abuses of mkstemp
+2017-12-18 12:23 +0000 [731a23fba7]  Corey Farrell <git@cfware.com>
 
-	  mkstemp() returns a unique filename, but appending an extension to that
-	  filename does not guarantee uniqueness. Instead, use mkdtemp() and we
-	  can put whatever extension we want on the files that we create inside
-	  the directory.
+	* CLI: Fix 'core set debug channel' completion bug.
 
-	  In the case of app_minivm, we also now properly clean up any temporary
-	  files that we create.
+	  The completion generator is missing a return so typing "core set debug
+	  all off <tab>" causes the command to actually execute.
 
-	  ASTERISK-20858 #close
-	  Reported by: Walter Doekes
+	  Change-Id: Ibf6462088a74eee66967732b50445783ebefc20b
 
-	  Change-Id: I30ad04f0e115f0b11693ff678ba5184d8b938e43
+2017-12-18 08:25 +0000 [1769d4a5c6]  Joshua Colp <jcolp@digium.com>
 
-2017-08-25 12:20 +0000 [01b5913ce0]  Sean Bright <sean.bright@gmail.com>
+	* confbridge: Clarify mute sound documentation.
 
-	* app_record: Resolve some absolute vs. relative filename bugs
+	  The mute/unmute sounds are only played when the
+	  action is initiated using the DTMF menu.
 
-	  If the Record() application is called with a relative filename that
-	  includes directories, we were not properly creating the intermediate
-	  directories and Record() would fail.
+	  ASTERISK-24756
 
-	  Secondarily, updated the documentation for RECORDED_FILE to mention
-	  that it does not include a filename extension.
+	  Change-Id: I55b3dd5bc166096bf5e2f547ddd0ce355f36e3dc
 
-	  Finally, rewrote the '%d' functionality to be a bit more straight
-	  forward and less noisy.
+2017-12-18 06:36 +0000 [b40c00c97b]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-16777 #close
-	  Reported by: klaus3000
+	* app_transfer: Remove LOCAL from documentation.
 
-	  Change-Id: Ibc2640cba3a8c7f17d97b02f76b7608b1e7ffde2
+	  The Local channel has never supported app_transfer
+	  from what I can see so remove it from the documentation.
 
-2017-08-23 10:01 +0000 [bf178a0f4f]  Florian Floimair <f.floimair@commend.com>
+	  ASTERISK-25649
 
-	* alembic: Add dtls_fingerprint column in ps_endpoints table
+	  Change-Id: Icbcfe297f6f866285a26b3e9fd5c6d00fa22e0e9
 
-	  The ps_endpoints table was missing the dtls_fingerprint column
-	  introduced with commit adba2a8d7fd.
+2017-12-15 19:01 +0000 [4a461bcde4]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27168 #close
+	* chan_pjsip.c: Improve ast_request() diagnostic msgs.
 
-	  Change-Id: I9cb5006f7f50718b5239919562773adabb334cfd
+	  Attempting to dial PJSIP/endpoint when the endpoint doesn't exist and
+	  disable_multi_domain=no results in a misleading empty endpoint name
+	  message.  The message should say the endpoint was not found.
 
-2016-02-28 19:05 +0000 [fff2f68616]  Matt Jordan <mjordan@digium.com>
+	  * Added missing endpoint not found message.
 
-	* main/app: Only look to end of file if ':end' is specified, and not just ':'
+	  * Added more information to the empty endpoint name msgs if available.
 
-	  There is a little known feature in app_controlplayback that will cause the
-	  specified offset to be used relative to the end of a file if a ':end' is
-	  detected within the filename.
+	  * Eliminated RAII_VAR in request().
 
-	  This feature is pretty bad, but okay.
+	  Change-Id: I21da85ebd62dcc32115b2ffcb5157416ebae51e4
 
-	  However, a bug exists in this code where a ':' detected in the filename
-	  will cause the end pointer to be non-NULL, even if the full ':end' isn't
-	  specified. This causes us to treat an unspecified offset (0) as being
-	  "start playing from the end of the file", resulting in no file playback
-	  occurring.
+2016-10-06 01:29 +0000 [6f8b34f9c1]  Corey Farrell <git@cfware.com>
 
-	  This patch fixes this bug by resetting the end pointer if ':end' is not
-	  found in the filename.
+	* chan_sip: Add security event for calls to invalid extension.
 
-	  ASTERISK-23608 #close
-	  Reported by: Jonathan White
+	  Log a message to security events when an INVITE is received to an
+	  invalid extension.
 
-	  Change-Id: Ib4c7b1b45283e4effd622a970055c51146892f35
-	  (cherry picked from commit 13efea24f7ce6ccc01d1a5a0603be2636d83a408)
+	  ASTERISK-25869 #close
 
-2017-08-24 09:42 +0000 [579d4593ac]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I0da40cd7c2206c825c2f0d4e172275df331fcc8f
 
-	* app_queue: Evaluate realtime queues when running dialplan functions
+2017-12-15 10:26 +0000 [e6768c0f81]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-19103 #close
-	  Reported by: Jim Van Meggelen
+	* cdr: Minor optimizations.
 
-	  Change-Id: I4bd32a9d1fcebb8ac56bff0e084d4f53e31b692b
+	  * bridge_candidate_process: remove SCOPED_AO2LOCK and return value.
+	  * handle_standard_bridge_enter_message: replace recursive call with goto
+	    statement.
 
-2017-08-23 09:19 +0000 [0af145de2d]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-24297
 
-	* app_voicemail: Honor escape digits in "greeting only" mode
+	  Change-Id: Id2eaa0822fb8dc799f63422bb3aa89de9d4ee2a2
 
-	  ASTERISK-21241 #close
-	  Reported by: Eelco Brolman
-	  Patches:
-	  	Patch uploaded by Eelco Brolman (License 6442)
+2017-12-12 12:55 +0000 [bf2d35931d]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Icbe39b5c82a49b46cf1d168dc17766f3d84f54fe
+	* aco: Minimize use of regex.
 
-2017-08-24 08:35 +0000 [d251a961ac]  Sean Bright <sean.bright@gmail.com>
+	  Remove nearly all use of regex from ACO users.  Still remaining:
+	  * app_confbridge has a legitamate use of option name regex.
+	  * ast_sorcery_object_fields_register is implemented with regex, all
+	    callers use simple prefix based regex.  I haven't decided the best
+	    way to fix this in both 13/15 and master.
 
-	* res_smdi: Clean up memory leak
+	  Change-Id: Ib5ed478218d8a661ace4d2eaaea98b59a897974b
 
-	  Change-Id: I1e33290929e1aa7c5b9cb513f8254f2884974de8
+2017-12-12 12:36 +0000 [a455e18320]  Corey Farrell <git@cfware.com>
 
-2017-08-11 11:40 +0000 [3f22b53349]  Richard Mudgett <rmudgett@digium.com>
+	* aco: Create ways to minimize use of regex.
 
-	* bridge_softmix.c: Remove always true test.
+	  ACO uses regex in many situations where it is completely unneeded.  In
+	  some cases this doubles the total processing performed by
+	  aco_process_config.
 
-	  Change-Id: I26238df2ff0d0f6dfe95c3aa35da588f1ee71727
+	  * Create ACO_IGNORE category type for use in place of skip_category
+	    regex source string.
+	  * Create additional aco_category_op values to allow specifying category
+	    filter using either a single plain string or a NULL terminated array
+	    of plain strings.
+	  * Create ACO_PREFIX to allow matching option names to case insensitive
+	    prefixes.
 
-2017-08-17 16:46 +0000 [b88c3a4209]  Sungtae Kim <pchero21@gmail.com>
+	  Change-Id: I66a920dcd8e2b0301f73f968016440a985e72821
 
-	* app_queue: Fix initial hold time queue statistic
+2017-12-15 07:56 +0000 [03c25a869f]  Corey Farrell <git@cfware.com>
 
-	  Fixed to use correct initial value and fixed to use the
-	  correct queue info to check the first value.
+	* res_smdi: Fix shutdown ref.
 
-	  ASTERISK-27204
+	  When adding shutdown refs for OPTIONAL_API components I accidentally
+	  added it to the unload_module function in res_smdi.  Move it to
+	  load_module.
 
-	  Change-Id: Ia9e36c828e566e1cc25c66f73307566e4acb8e73
+	  Change-Id: I2b9da38fbc11ef78ea23dbb2df92b684be7f647c
 
-2017-08-21 04:28 +0000 [8e99969000]  Torrey Searle <torrey@voxbone.com>
+2017-12-11 17:07 +0000 [9d5797616c]  Corey Farrell <git@cfware.com>
 
-	* res/res_pjsip_session: allow SDP answer to be regenerated
+	* loader: Use vector to build apha sorted module lists.
 
-	  If an SDP answer hasn't been sent yet, it's legal to change it.
-	  This is required for PJSIP_DTMF_MODE to work correctly, and can
-	  also have use in the future for updating codecs too.
+	  Change-Id: I9c519f4dec3cda98b2f34d314255a31d49a6a467
 
-	  ASTERISK-27209 #close
+2017-11-21 00:28 +0000 [7b54903313]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Idbbfb7cb3f72fbd96c94d10d93540f69bd51e7a1
+	* loader: Replace priority heap with vector.
 
-2017-08-20 08:15 +0000 [4faf77feec]  Michael Kuron <m.kuron@gmx.de>
+	  This is needed for future changes which will require being able to
+	  process the load priority out of order.
 
-	* res_xmpp: fix inverted return code check in OAuth
+	  Change-Id: Ia23421197f09789940510b03ebbbf3bf24d51bea
 
-	  fetch_access_token calls func_curl via ast_func_read. The latter returns 0 upon
-	  success and -1 if the function is not available.
-	  This commit inverts the return code check so that an error is printed if the
-	  module is not loaded and not if it is loaded.
+2017-12-14 18:55 +0000 [9755eff46f]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27207 #close
+	* res_hep: hepv3_is_loaded() should check if we are enabled
 
-	  Change-Id: I9ef903f80702d1218e8701f65a4e5e918e6548fb
+	  res_hep_pjsip.so and res_hep_rtcp.so will still load and do a lot of
+	  unnecessary work even if 'enabled' is set to 'no' in hep.conf.
 
-2017-08-17 12:00 +0000 [a6251ec373]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I3eddfeea09c6b5bc7c641952ee0ae487fd09b64b
 
-	* res_calendar_icalendar: Properly handle recurring events
+2017-11-20 23:10 +0000 [3505cc88e8]  Corey Farrell <git@cfware.com>
 
-	  When looking for recurring events, use the correct end time based on the
-	  configured 'timeframe.'
+	* loader: Rework of load_dynamic_module.
 
-	  ASTERISK-27174 #close
-	  Reported by: Mark Thompson
+	  * Split off load_dlopen to perform actual dlopen, check results and log
+	    warnings when needed.
+	  * Always use RTLD_NOW.
+	  * Use flags which minimize number of calls to dlopen required.  First
+	    attempt always uses RTLD_GLOBAL when global_symbols_only is enabled,
+	    RTLD_LOCAL when it is not.
 
-	  Change-Id: Id90c3cfc79d561a5521d79be176683e225f2edef
+	  This patch significantly reduces the number of dlopen's performed.  With
+	  299 modules my system ran dlopen 857 times before this patch, 655 times
+	  after this patch.
 
-2017-08-16 15:43 +0000 [572b5307e0]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ib2c9903cfddcc01aed3e01c1e7fe4a3fb9af0f8b
 
-	* Fix downloader not working with curl
+2017-11-21 20:34 +0000 [80bf0ee99a]  Corey Farrell <git@cfware.com>
 
-	  The codec/dpma downloader wasn't handling curl correctly.  The logic
-	  that transforms makeopts into a bash-sourceable file wasn't
-	  handling the make 'or' command in DOWNLOAD_TIMEOUT so bash was
-	  looking for an 'or' command.
+	* loader: Minor fix to module registration.
 
-	  That logic has been eliminated.  Instead of trying to transform
-	  and source makeopts, the downloader now calls a make scriptlet
-	  to print the value of a specific variable.  This way, make handles
-	  the ors (or any other make construct that happens to creep into
-	  that file).
+	  This protects the module loader itself against crashing if dlopen is
+	  called on a module from outside loader.c.
 
-	  ASTERISK-27202
-	  Reported by: Sean McCord
+	  * Expand scope of lock inside ast_module_register to include reading of
+	    resource_being_loaded.
+	  * NULL check resource_being_loaded.
+	  * Set resource_being_loaded NULL as soon as dlopen returns.  This fixes
+	    some error paths where it was not NULL'ed.
+	  * Create module_destroy function to deduplicate code from
+	    ast_module_unregister and modules_shutdown.
+	  * Resolve leak that occured if a module did not successfully register.
+	  * Simplify checking for successful registration.
 
-	  Change-Id: Iadfb6693528e4d4da7b8bb201fa66da2c71c7f99
+	  Change-Id: I40f07a315e55b92df4fc7faf525ed6d4f396e7d2
 
-2017-08-15 15:15 +0000 [8594f73a81]  Richard Mudgett <rmudgett@digium.com>
+2017-12-14 15:27 +0000 [a8aa209901]  Corey Farrell <git@cfware.com>
 
-	* configure: Check cache for valid pjproject tarball before downloading.
+	* res_clialiases: Fix completion pass-through.
 
-	  On a fresh Asterisk source directory, the bundled pjproject tarball is
-	  unconditionally downloaded even if the tarball is already in a specified
-	  cache directory.
+	  Never ignore contents of line when generating completion options.
 
-	  * Made check if the pjproject tarball is valid in the cache directory
-	  before downloading the tarball on a fresh source directory.
+	  Change-Id: I74389efdfea154019d3b56a9f381610614c044c8
 
-	  Change-Id: Ic7ec842d3c97ecd8dafbad6f056b7fdbce41cae5
+2017-12-11 18:20 +0000 [98f7e9251f]  Richard Mudgett <rmudgett@digium.com>
 
-2017-08-15 11:14 +0000 [d08342b0cb]  Richard Mudgett <rmudgett@digium.com>
+	* res_rtp_asterisk.c: Disable packet flood detection for video streams.
 
-	* res_pjsip: Fix prune_on_boot to remove only contacts for the host.
+	  We should not do flood detection on video RTP streams.  Video RTP streams
+	  are very bursty by nature.  They send out a burst of packets to update the
+	  video frame then wait for the next video frame update.  Really only audio
+	  streams can be checked for flooding.  The others are either bursty or
+	  don't have a set rate.
 
-	  * Check that the contact's reg_server matches the host's name before
-	  deleting any prune_on_boot contacts.  We don't want to delete reliable
-	  transport contacts made with other servers if the ps_contacts database
-	  table is shared with other servers.
+	  * Added code to selectively disable packet flood detection for video RTP
+	  streams.
 
-	  Thanks to Ross Beer for pointing out that the original prune logic would
-	  delete reliable transport contacts from other servers.
+	  ASTERISK-27440
 
-	  ASTERISK-27147
+	  Change-Id: I78031491a6e75c2d4b1e9c2462dc498fe9880a70
 
-	  Change-Id: I8e439d0d1c266ffdfd7b73d1e5e466180a689bd0
+2017-12-14 14:05 +0000 [283d2df680]  George Joseph <gjoseph@digium.com>
 
-2017-08-04 09:25 +0000 [54e3ac402f]  Andrey Egorov <andr06@gmail.com>
+	* res_pjsip_sdp_rtp: Add NULL check in add_crypto_to_stream
 
-	* res_xmpp: Google OAuth 2.0 protocol support for XMPP / Motif
+	  add_crypto_to_stream wasn't checking for a NULL
+	  session->inv_session->neg before calling pjmedia_sdp_neg_get_state.
+	  This was causing a crash if the negotiation hadn't already been
+	  completed and asterisk was compiled with --enable-dev-mode.
 
-	  Add ability to use tokens instead of passwords according to Google OAuth 2.0
-	  protocol.
+	  Change-Id: I57c6229954a38145da9810fc18657bfcc4d9d0c9
 
-	  ASTERISK-27169
-	  Reported by: Andrey Egorov
-	  Tested by: Andrey Egorov
+2017-12-14 12:14 +0000 [c387beb456]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I07f7052a502457ab55010a4d3686653b60f4c8db
+	* res_musiconhold: Start playlist after initial announcement
 
-2017-08-10 14:18 +0000 [bac3e8c08b]  Richard Mudgett <rmudgett@digium.com>
+	  Reset the samples counter to zero when we are done playing an
+	  announcement so that we don't skip into the middle of the first file in
+	  the playlist.
 
-	* STUN/netsock2: Fix some valgrind uninitialized memory findings.
+	  Also add the selected annoucement to the output of 'moh show classes.'
 
-	  * netsock2.c: Test the addr->len member first as it may be the only member
-	  initialized in the struct.
+	  ASTERISK-24329 #close
+	  Reported by: Thomas Frederiksen
 
-	  * stun.c:ast_stun_handle_packet(): The combinded[] local array could get
-	  used uninitialized by ast_stun_request().  The uninitialized string gets
-	  copied to another location and could overflow the destination memory
-	  buffer.
+	  Change-Id: I2a5f986a31279c981592f49391409ebf38d6f6d0
 
-	  These valgrind findings were found for ASTERISK_27150 but are not
-	  necessarily a fix for the issue.
+2017-12-14 10:51 +0000 [7a8a187a56]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I55f8687ba4ffc0f69578fd850af006a56cbc9a57
+	* coverity: Fix warnings in res_smdi
 
-2017-08-02 18:44 +0000 [1cf2c79f37]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-19657 #close
+	  Reported by: Matt Jordan III, Esq.
 
-	* res_pjsip_outbound_registration.c: Re-REGISTER on transport shutdown.
+	  Change-Id: I59a5e6ef3e7d9e848bec1f4b40cb73321bc7956a
 
-	  The fix for the issue is broken up into three parts.
+2017-12-14 10:22 +0000 [dac5e3a0df]  Sean Bright <sean.bright@gmail.com>
 
-	  This is part three which handles the client side of REGISTER requests.
-	  The registered contact may no longer be valid on the server when the
-	  transport used is reliable and the connection is broken.
+	* configs: Comment out and change IP of iax.conf [demo]
 
-	  * Re-REGISTER our contact if the reliable transport is broken after
-	  registration completes.  We attempt to re-REGISTER immediately to minimize
-	  the time we are unreachable.  Time may have already passed between the
-	  connection being broken and the loss being detected.
+	  This no longer appears to exist, so no sense in causing confusion.
 
-	  * Reorder sip_outbound_registration_state_alloc() so the STATSD_GUAGE's
-	  are still correct if an allocation failure happens.
+	  ASTERISK-27175 #close
+	  Reported by: Tzafrir Cohen
 
-	  ASTERISK-27147
+	  Change-Id: Idde967924c69f6a741dc9a5ab7dacb44d22cf100
 
-	  Change-Id: I3668405b1ee75dfefb07c0d637826176f741ce83
+2017-12-13 14:26 +0000 [a51bfe5a79]  George Joseph <gjoseph@digium.com>
 
-2017-07-31 14:21 +0000 [07d026b4cd]  Richard Mudgett <rmudgett@digium.com>
+	* README: Remove outdated references to tex docs
 
-	* res_pjsip: Remove ephemeral registered contacts on transport shutdown.
+	  Added links to the wiki to replace references to outdated
+	  tex docs.
 
-	  The fix for the issue is broken up into three parts.
+	  ASTERISK-27430
+	  Reported by: Corey Farrell
 
-	  This is part two which handles the server side of REGISTER requests when
-	  rewrite_contact is enabled.  Any registered reliable transport contact
-	  becomes invalid when the transport connection becomes disconnected.
+	  Change-Id: I5007e732b30bc7b63d124c530ae8857c89991209
 
-	  * Monitor the rewrite_contact's reliable transport REGISTER contact for
-	  shutdown.  If it is shutdown then the contact must be removed because it
-	  is no longer valid.  Otherwise, when the client attempts to re-REGISTER it
-	  may be blocked because the invalid contact is there.  Also if we try to
-	  send a call to the endpoint using the invalid contact then the endpoint is
-	  not likely to see the request.  The endpoint either won't be listening on
-	  that port for new connections or a NAT/firewall will block it.
+2017-12-13 09:50 +0000 [5f6a3c4399]  Corey Farrell <git@cfware.com>
 
-	  * Prune any rewrite_contact's registered reliable transport contacts on
-	  boot.  The reliable transport no longer exists so the contact is invalid.
+	* CLI: Remove special handling of 'core set verbose' from rasterisk.
 
-	  * Websockets always rewrite the REGISTER contact address and the transport
-	  needs to be monitored for shutdown.
+	  rasterisk does not need to handle setting verbose levels locally, it
+	  should just tell the daemon what it wants and print what it is given.
+	  Just max out the verbose level on the local client so all filtering
+	  happens on the daemon.
 
-	  * Made the websocket transport set a unique name since that is what we use
-	  as the ao2 container key.  Otherwise, we would not know which transport we
-	  find when one of them shuts down.  The names are also used for PJPROJECT
-	  debug logging.
+	  ASTERISK-20281 #close
 
-	  * Made the websocket transport post the PJSIP_TP_STATE_CONNECTED state
-	  event.  Now the global keep_alive_interval option, initially idle shutdown
-	  timer, and the server REGISTER contact monitor can work on wetsocket
-	  transports.
+	  Change-Id: Ia305f75f1fc424a9169bfa30ef70d626ace2c8a8
 
-	  * Made the websocket transport set the PJSIP_TP_DIR_INCOMING direction.
-	  Now initially idle websockets will automatically shutdown.
+2017-12-08 06:48 +0000 [daa3a3009a]  sungtae kim <pchero21@gmail.com>
 
-	  ASTERISK-27147
+	* Add new AMI action for app_voicemail
 
-	  Change-Id: I397a5e7d18476830f7ffe1726adf9ee6c15964f4
+	  Currently, to figure out specified voicemail's status, there's only one
+	  way to do it, which is use a VoicemailUserEntry AMI message.
+	  But it consumed it too much resource(it check everything).
+	  So, added new AMI action.
 
-2017-07-28 18:26 +0000 [ca261d4b70]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27470
 
-	* res_pjsip: PJSIP Transport state monitor refactor.
+	  Change-Id: Ie4eba1424a142e5fbd1d9fb1821a3fc1a1e238b7
 
-	  The fix for the issue is broken up into three parts.
+2017-11-30 10:12 +0000 [62f2860c39]  Joshua Colp <jcolp@digium.com>
 
-	  This is part one which refactors the transport state monitor code to allow
-	  more modules to be able to monitor transports.
+	* AST-2017-012: Place single RTCP report block at beginning of report.
 
-	  * Pull the management of PJPROJECT's transport state callback code from
-	  res_pjsip_transport_management.c into res_pjsip.  Now other modules can
-	  dynamically add and remove themselves from transport monitoring without
-	  worrying about breaking PJPROJECT's callback chain.
+	  When the RTCP code was transitioned over to Stasis a code change
+	  was made to keep track of how many reports are present. This count
+	  controlled where report blocks were placed in the RTCP report.
 
-	  * Add the ability for other modules to get a callback whenever a specific
-	  transport is shutdown.
+	  If a compound RTCP packet was received this logic would incorrectly
+	  place a report block in the wrong location resulting in a write
+	  to an invalid location.
 
-	  ASTERISK-27147
+	  This change removes this counting logic and always places the report
+	  block at the first position. If in the future multiple reports are
+	  supported the logic can be extended but for now keeping a count
+	  serves no purpose.
 
-	  Change-Id: I7d9a31371eb1487c9b7050cf82a9af5180a57912
+	  ASTERISK-27382
+	  ASTERISK-27429
 
-2017-07-27 15:36 +0000 [162f6ab845]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Iad6c8a9985c4b608ef493e19c421211615485116
 
-	* res_pjsip_transport_management.c: Rename some variables.
+2017-12-13 06:54 +0000 [3370cd21df]  Joshua Colp <jcolp@digium.com>
 
-	  * Use monitored instead of the misleading keepalive name.
+	* res_pjsip_session: Reinvite using active stream topology if none requested.
 
-	  Change-Id: I9e5bcbb4ab2b82d49bcd0f06dfe85d15e0b552b6
+	  When a connected line update is sent to an endpoint we do not request
+	  a specific stream topology to be used. Previously this resulted in the
+	  configured stream topology being used which may actually differ from the
+	  currently negotiated topology. PJSIP is helpful in this regard in that
+	  it will fill in any missing streams with removed ones. This results in
+	  our own state not matching the SDP, though, and we do not apply the
+	  negotiated SDP.
 
-2017-08-10 09:09 +0000 [22575b6342]  Scott Griepentrog <scott@griepentrog.com>
+	  This change tweaks the code to use the actively negotiated stream
+	  topology if it is present with a fallback to the configured one. This
+	  results in the SDP and the state having matching information and the
+	  world is happy.
 
-	* res_pjsip_messaging: IPv6 receive address needs brackets
+	  ASTERISK*27397
 
-	  When handling an incoming SIP MESSAGE, PJSIP
-	  attaches the IP address that the message was
-	  received from to the message in the variable
-	  PJSIP_RECVADDR.  When the IP address is IPv6
-	  the :PORT appended results in an unparseable
-	  mess. By using an additional bit flag on the
-	  pj_sockaddr_print call, the conventional use
-	  of brackets around the address is achieved.
+	  Change-Id: I7a57117f0183479e6884b7bf3a53bb8c7464f604
 
-	  ASTERISK-27193 #close
+2017-12-06 08:24 +0000 [0b532367bd]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I12342521f2ce87a5b6e4883d480a3fd957aa9fd9
+	* pjsip: Ignore state changes from old transactions.
 
-2017-08-09 08:01 +0000 [363d61ef58]  George Joseph <gjoseph@digium.com>
+	  When we fail over to a new target we create a new transaction
+	  and it becomes the current INVITE transaction. This does not
+	  prevent the previous transaction from raising state changes
+	  and causing the session to be prematurely disconnected if a
+	  transport error occurs immediately.
 
-	* configure:  Add --with-download-cache option
+	  This change backports a fix from PJSIP that eliminates the
+	  incorrect state change and reduces when they would be raised
+	  in the first place.
 
-	  To make building without an internet connection easier, a new
-	  ./configure option '--with-download-cache' was added that sets
-	  the cache for externals (like pjproject, the codecs and the DPMA),
-	  AND the sounds files.  It can also be specified as an environment
-	  variable named "AST_DOWNLOAD_CACHE".  The existing
-	  '--with-sounds-cache' option / SOUNDS_CACHE_DIR env variable and
-	  '--with-externals-cache' option / EXTERNALS_CACHE_DIR env variable
-	  remain and if specified, will override '--with-downloads-cache'.
+	  ASTERISK-27408
 
-	  Change-Id: I5c3cf15ee61e8fe191b52732303e969854f8d861
+	  Change-Id: Id22d087591782eee31311753d11e7eca4b95ef34
 
-2017-07-26 09:17 +0000 [3608f96ea3]  Torrey Searle <torrey@voxbone.com>
+2017-12-12 22:42 +0000 [cb249b2419]  Yasuhiko Kamata <yasuhiko.kamata@nxtg.co.jp>
 
-	* res_rtp_asterisk: enable rtcp & QOS stats on native bridge
+	* chan_sip: 3PCC patch for AMI "SIPnotify"
 
-	  Asterisk wasn't generating or forwarding RTCP packets when native
-	  bridge was activated.  Also the stats weren't available via
-	  CHANNEL(qos). Now the RTCP stats are always calculated.
+	  A patch for sending in-dialog SIP NOTIFY message
+	  with "SIPnotify" AMI action.
 
-	  ASTERISK-27158 #close
+	  ASTERISK-27461
 
-	  Change-Id: I46fb8f61c95e836b9d2dda6054b0cf205c16037b
+	  Change-Id: I5797ded4752acd966db6b13971284db684cc5ab4
 
-2017-07-26 11:39 +0000 [0de7312fac]  Joshua Colp <jcolp@digium.com>
+2017-12-12 15:38 +0000 [c7f94e570e]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-	* res_pjsip_session: Release media resources on session end quicker.
+	* app_queue: Fix extension state subscriptions removed on dialplan reload
 
-	  A change was made long ago where the session was kept around
-	  until the underlying INVITE session had been destroyed. This
-	  had the side effect of also keeping the underlying media resources
-	  around for this time as well.
+	  The approach with having a single global subscription to all extension
+	  state changes has one issue: dynamically created hints don't have any
+	  watchers and are therefore garbage collected on the first dialplan
+	  reload.
 
-	  This change ensures that when we know the session is ending we
-	  release the media resources immediately.
+	  This change creates a state subscription for every queue member with a
+	  hint as state_interface, thus increasing the count of watches for
+	  hints, so they are not destroyed prematurely anymore.
 
-	  ASTERISK-27110
+	  There are 2 side effects:
+	  1. The state change callback in app_queue is not executed when
+	     there are no members referring to the extension.
+	  2. The callback is called multiple times for the same hint if it's
+	     associated with more than one queue member.
 
-	  Change-Id: I643e431d5c3bf05cda220c1d39e824a505a29b82
+	  Reported by: Steven T. Wheeler
 
-2017-08-02 16:08 +0000 [905c4ca3dc]  Corey Farrell <git@cfware.com>
+	  ASTERISK-18411 #close
 
-	* app_privacy: remove unused header asterisk/image.h
+	  Change-Id: I4956af2136ea2a7f110ac9272eae5f6e676d8f89
 
-	  Change-Id: I56ed530633a642633b18383821069e806c92ae82
+2017-12-12 15:28 +0000 [0c9cc7e975]  Sean Bright <sean.bright@gmail.com>
 
-2017-08-03 13:13 +0000 [38dbc708e7]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* chan_sip: Don't send trailing \0 on keep alive packets
 
-	* Support GMIME 3.0
+	  This is a partial fix for ASTERISK~25817 but does not address the
+	  comments regarding RFC 5626.
 
-	  Support building the Asterisk httpd with version 3.0 of gmime as
-	  well as earlier versions of that library.
+	  Change-Id: I227e2d10c0035bbfa1c6e46ae2318fd1122d8420
 
-	  ASTERISK-27173
+2017-12-12 15:19 +0000 [5039b5741c]  Dwayne Hubbard
 
-	  Change-Id: I7e13dd05a3083ccb0df2dabf83110223f6a9fa8f
+	* chan_sip: Don't crash in Dial on invalid destination
 
-2017-07-28 07:53 +0000 [c4f201cd73]  Torrey Searle <torrey@voxbone.com>
+	  Stripping the DNID in a SIP dial string can result in attempting to call
+	  the argument parsing macros on an empty string, causing a crash.
 
-	* res_rtp_asterisk:  Make P2P bridge Asymmetric codec aware
+	  ASTERISK-26131 #close
+	  Reported by: Dwayne Hubbard
+	  Patches:
+	  	dw-asterisk-master-dnid-crash.patch (license #6257) patch
+	  	uploaded by Dwayne Hubbard
 
-	  Introduce a new property to rtp-engine to make it aware of
-	  the desire for assymetric codecs or not.  If asymmetric codecs
-	  is not allowed, the bridge will compare read/write formats
-	  and shut down the p2p bridge if needed
+	  Change-Id: Ib84c1f740a9ec0539d582b09d847fc85ddca1c5e
 
-	  ASTERISK-26745 #close
+2017-12-12 15:16 +0000 [6a67828b46]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I0d9c83e5356df81661e58d40a8db565833501a6f
+	* menuselect: Tweak check for recently run configure.
 
-2017-08-03 21:30 +0000 [84b6a5efd7]  Corey Farrell <git@cfware.com>
+	  Recently menuselect has randomly produced an error stating that
+	  configure was just run and make had to be restarted.  I believe this is
+	  due to an incorrect menuselect/Makefile rule.  The original rule
+	  produced an error if makeopts or autoconfig.h were older than
+	  makeopts.in or autoconfig.h.in.  I believe this can create an issue if
+	  makeopts is older than autoconfig.h.in or if autoconfig.h is older than
+	  makeopts.in.  The new rules compare files independently.
 
-	* Correct some leaks in unit tests.
+	  Change-Id: Ibca155035fa1392c95e33cbf25f257902abba17b
 
-	  * chan_sip: channel in test_sip_rtpqos_1.
-	  * test_config: config hook, config info and global config holder.
-	  * test_core_format: format in format_attribute_set_without_interface.
-	  * test_stream: unneeded frame duplication.
-	  * test_taskprocessor: task_data.
+2017-12-07 17:51 +0000 [22810fc635]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I94d364d195cf3b3b5de2bf3ad565343275c7ad31
+	* chan_pjsip/res_pjsip: Add CHANNEL(pjsip,request_uri)
 
-2017-07-26 17:49 +0000 [f9a823e9dc]  Richard Mudgett <rmudgett@digium.com>
+	  This patch does three things associated with the initial incoming INVITE
+	  request URI.
 
-	* res_pjsip_transport_websocket.c: Fix serializer ref leak.
+	  1) Add access to the full initial incoming INVITE request URI.
 
-	  Change-Id: Ib5a19bfd597f63d9021baeb645fc11153b3afa57
+	  2) We were not setting DNID on incoming PJSIP channels.  The DNID is the
+	  user portion of the initial incoming INVITE Request-URI.  The value is
+	  accessed by reading CALLERID(dnid).
 
-2017-08-02 18:41 +0000 [631180a0c3]  Richard Mudgett <rmudgett@digium.com>
+	  3) Fix CHANNEL(pjsip,target_uri) documentation.
 
-	* res_pjsip_outbound_registration.c: Misc fixes.
+	  * The initial incoming INVITE request URI is now available using
+	  CHANNEL(pjsip,request_uri).
 
-	  * Remove unnecessary CMP_STOP.
+	  * Set the DNID on PJSIP channel creation so CALLERID(dnid) can return the
+	  initial incoming INVITE request URI user portion.
 
-	  * In handle_client_registration() use DEBUG_ATLEAST() to only do work
-	  needed for the debug log message when the debug log message is needed.
+	  * CHANNEL(pjsip,target_uri) now correctly documents that the target URI is
+	  the contact URI.
 
-	  * In sip_outbound_registration_state_destroy() check state->registration
-	  for NULL.
+	  * Refactored print_escaped_uri() out of channel_read_pjsip() to handle
+	  pjsip_uri_print() error condition when the buffer is too small.
 
-	  Change-Id: I656d0fa11dda0b00048103efb1558e67a426fd80
+	  ASTERISK-27478
 
-2017-07-31 20:20 +0000 [7b84c6693e]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I512e60d1f162395c946451becb37af3333337b33
 
-	* res_pjsip_nat.c: Remove unnecessary CMP_STOP.
+2017-12-12 09:28 +0000 [ec1f4bf48d]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I6279b0d723bc3b75b8d65e81e02da9ea9bc0c3da
+	* res_pjsip: Add TLSv1.1 and TLSv1.2 support
 
-2017-07-31 14:20 +0000 [a32614a2a8]  Richard Mudgett <rmudgett@digium.com>
+	  Support for these protocols was added in the same commit as the 'proto'
+	  field, so we can safely use the same ./configure check.
 
-	* res_pjsip_registrar.c: Remove unnecessary CMP_STOP.
+	  For reference: https://trac.pjsip.org/repos/changeset/4968
 
-	  Most uses of CMP_STOP are superfluous and are only respected when
-	  OBJ_MULTIPLE is used to search the container.
+	  Change-Id: Icf4975d785d6bfb8f30ac7ffa695a0adf9382dac
 
-	  Change-Id: I20571a202ec0aa1098bb2749eeba18de7ca110b8
+2017-12-12 08:06 +0000 [0b9d2135a9]  Sean Bright <sean.bright@gmail.com>
 
-2017-08-03 11:30 +0000 [d066758a4c]  Corey Farrell <git@cfware.com>
+	* res_pjsip: Assign support levels to a few modules
 
-	* Fix compile error for old versions of GCC.
+	  Change-Id: I51f6945c4023cb93fc7b87be5ab4c50e9e6ee27d
 
-	  Use -Wno-format-truncation only if supported by compiler.
+2017-12-09 00:35 +0000 [c01ba7437e]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27171 #close
+	* CLI: Fix 'core show sysinfo' function ordering.
 
-	  Change-Id: Iac0aed7a5bcaa16c21b7d62c4e4678d244c4ccb6
+	  Handle CLI initialization before any processing occurs.
 
-2017-08-01 15:57 +0000 [ed1bce956e]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I598b911d2e409214bbdfd0ba0882be1d602d221c
 
-	* Revert "res_pjsip_session: Release media resources on session end quicker."
+2017-12-11 15:27 +0000 [b088cddc03]  Kevin Harwell <kharwell@digium.com>
 
-	  This reverts commit 98709642d640b490f327d220fdcdea6d45fd65d7.
+	* pjsip_options: wrongly applied "UNKNOWN" status
 
-	  See the 15 branch review.
+	  A couple of places were setting the status to "UNKNOWN" when qualifies were
+	  being disabled. Instead this should be set to the "CREATED" status that
+	  represents when a contact is given (uri available), but the qualify frequency
+	  is set to zero so we don't know the status.
+
+	  This patch updates the relevant places with "CREATED". It also updates the
+	  "CREATED" status description (value shown in CLI/AMI/ARI output) to a value
+	  of "NonQualified"/"NonQual" as this description is hopefully less confusing.
 
-	  Change-Id: I8476b3cdacaad5157fa36b6247d0e4cdf1e8d5c6
+	  ASTERISK-27467
 
-2017-06-29 03:47 +0000 [9a09f7dd5d]  Niklas Larsson <niklas@tese.se>
+	  Change-Id: Id67509d25df92a72eb3683720ad2a95a27b50c89
 
-	* app_queue: Add priority to AMI QueueStatus
+2017-12-08 12:04 +0000 [c2ec82bf36]  Richard Mudgett <rmudgett@digium.com>
 
-	  Add priority to callers in AMI QueueStatus response
+	* stasis_channels.c: Don't set channel snapshot caller_dnid twice.
 
-	  ASTERISK-27092 #close
+	  Change-Id: Ib8d45bbdfbda81e65045f6dff874d189b74e5471
 
-	  Change-Id: I8d1f737a72c7c38f4cfe1a4ee3ecc0a4f85bd199
+2017-12-11 09:45 +0000 [00578fae0a]  Sean Bright <sean.bright@gmail.com>
 
-2017-07-26 11:39 +0000 [3418d8d145]  Joshua Colp <jcolp@digium.com>
+	* codec_opus: Make libcurl a dependency in menuselect
 
-	* res_pjsip_session: Release media resources on session end quicker.
+	  ASTERISK-27475 #close
 
-	  A change was made long ago where the session was kept around
-	  until the underlying INVITE session had been destroyed. This
-	  had the side effect of also keeping the underlying media resources
-	  around for this time as well.
+	  Change-Id: If7384bc6ed002ef140dec69798d14c52b7cfd800
 
-	  This change ensures that when we know the session is ending we
-	  release the media resources immediately.
+2017-12-08 12:48 +0000 [521f741b04]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-27110
+	* pjsip: Improve CLI completion performance
 
-	  Change-Id: I3c6a82fe7d2c50b9dc9197cb12ef22f20d337501
+	  Use the new ast_cli_completion_add() function to improve completion
+	  performance for commands like 'pjsip show endpoint.'
 
-2017-07-26 08:48 +0000 [4d318cac68]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I76d802294d2ac1766110dc75f7d117c8541ce348
 
-	* res_pjsip_pidf_eyebeam_body_supplement: Correct status presentation
+2017-12-07 14:19 +0000 [9a9edc6c9e]  Sean Bright <sean.bright@gmail.com>
 
-	  This change fixes PIDF content generation when the underlying device
-	  state is considered in use. Previously it was incorrectly marked
-	  as closed meaning they were offline/unavailable. The code now
-	  correctly marks them as open.
+	* astdb: Improve prefix searches in astdb
 
-	  Additionally:
+	  Using the LIKE operator requires a full table scan of 'astdb', whereas a
+	  comparison operation is able to use the primary key index.
 
-	    * Generate an XML element for our activity instead of a using a text
-	      node.
+	  This patch adds a new function to the AstDB API for quick prefix matches
+	  and updates res_sorcery_astdb to utilize it. This showed substantial
+	  performance improvement in my test environment.
 
-	    * Consider every extension state other than "unavailable" to be 'open'
-	      status.
+	  Related to ASTERISK~26806, but does not completely resolve it.
 
-	    * Update the XML namespaces and structure to reflect those
-	      documented in RFC 4480
+	  Change-Id: I7d37f9ba2aea139dabf2ca72d31fbe34bd9b2fa1
 
-	    * Use 'on-the-phone' (defined in RFC 4880) instead of 'busy' as the
-	      "in use" activity. This change results in eyeBeam using the
-	      appropriate icon for the watched user.
+2017-12-08 18:19 +0000 [d2e87b8e14]  Corey Farrell <git@cfware.com>
 
-	  This was tested on eyeBeam 1.5.20.2 build 59030 on Windows.
+	* loader: Refactor resource_name_match.
 
-	  ASTERISK-26659 #close
-	  Reported by: Abraham Liebsch
-	  patches:
-	    ASTERISK-26659.diff submitted by snuffy (license 5024)
+	  Optimize resource_name_match.  This change eliminates use of
+	  ast_strdupa, instead verifying that both basename's are the same length,
+	  then using strncasecmp.
 
-	  Change-Id: I6e5ad450f91106029fb30517b8c0ea0c2058c810
+	  Change-Id: I477275c0e954c99d74be5abfc8bb6545b04e5a3d
 
-2017-07-23 18:34 +0000 [114602f434]  Joshua Colp <jcolp@digium.com>
+2017-12-08 14:58 +0000 [dbb376f166]  Sean Bright <sean.bright@gmail.com>
 
-	* res_pjsip: Add support for dnsmgr to external_media_address.
+	* pjsip_configuration: Add correct file header
 
-	  The "external_media_address" option on transports is now
-	  resolved using dnsmgr. This allows it to be automatically
-	  refreshed regularly if refreshes are enabled in dnsmgr.
-	  If the system is using a dynamic IP address a dynamic DNS
-	  hostname can be provided to keep the IP address up to
-	  date.
+	  Change-Id: I25348c386a222bb704aff07f54375108a6402906
 
-	  Change-Id: Ia54771720dff0105bde55d5bbb81a3ba437e05b2
+2017-12-07 09:52 +0000 [2ffe52a116]  Sean Bright <sean.bright@gmail.com>
 
-2017-07-27 20:58 +0000 [0f49e6ee2e]  Corey Farrell <git@cfware.com>
+	* utils: Add convenience function for setting fd flags
 
-	* Fix compiler warnings on Fedora 26 / GCC 7.
+	  There are many places in the code base where we ignore the return value
+	  of fcntl() when getting/setting file descriptior flags. This patch
+	  introduces a convenience function that allows setting or clearing file
+	  descriptor flags and will also log an error on failure for later
+	  analysis.
 
-	  GCC 7 has added capability to produce warnings, this fixes most of those
-	  warnings.  The specific warnings are disabled in a few places:
+	  Change-Id: I8b81901e1b1bd537ca632567cdb408931c6eded7
 
-	  * app_voicemail.c: truncation of paths more than 4096 chars in many places.
-	  * chan_mgcp.c: callid truncated to 80 chars.
-	  * cdr.c: two userfields are combined to cdr copy, fix would break ABI.
-	  * tcptls.c: ignore use of deprecated method SSLv3_client_method().
+2017-12-07 19:33 +0000 [e2dbc26376]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27156 #close
+	* res_stasis and res_speech: Fix load order.
 
-	  Change-Id: I65f280e7d3cfad279d16f41823a4d6fddcbc4c88
+	  res_stasis was missing AST_MODFLAG_LOAD_ORDER.  Set res_stasis and
+	  res_speech to start at (AST_MODPRI_APP_DEPEND - 1) so they are ready for
+	  dependent modules.
 
-2017-07-27 06:35 +0000 [0d58fefa30]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I27f4f3810a95b6be8a5bfbf62be2ace6bfab6ff3
 
-	* bundled_pjproject:  Improve SSL/TLS error handling
+2017-12-07 18:22 +0000 [0e4d31eb9c]  Kevin Harwell <kharwell@digium.com>
 
-	  OpenSSL has 2 levels or error processing.  It's possible for the
-	  top layer to return SSL_ERROR_SYSCALL but the lower layer return
-	  no error, in which case processing should continue.  Only the top
-	  layer was being examined though so connections were being torn
-	  down when they didn't need to be.  This patch adds the examination
-	  of the lower level codes, and if they return no errors, allows
-	  processing to continue.
+	* pjsip_options: contacts sometimes not being updated on reload
 
-	  ASTERISK-27001
-	  Reported-by: Ian Gilmour
-	  patches:
-	  	pjproject-2.6.patch submitted by Ian Gilmour (license 6889)
+	  For both dynamic and static contacts it was possible that potential AOR
+	  changes were not being applied to all contacts. This was because the qualify
+	  and schedule code was only retrieving AOR's, and contacts with frequencies
+	  greater than zero.
 
-	  Updated-by: George Joseph and Sauw Ming (Teluu)
+	  For instance the following could happen: and AOR/contact has a frequency of 5,
+	  it then gets set to 0, and then a reload occurs. All scheduled OPTIONS are
+	  stopped, a list of AOR's is retrieved with frequency > 0, but none are
+	  selected since in this scenario all are 0. The contact for the one previously
+	  set to 5 though does not get updated, so it's status remains "AVAILABLE".
 
-	  Merged to upstream pjproject on 7/27/2017 (commit 5631)
+	  This patch makes it so all contacts (static and dynamic) are selected, and
+	  appropriately updated if need be.
 
-	  Change-Id: I23844ca0c68ef1ee550f14d46f6dae57d33b7bd2
+	  ASTERISK-27467 #close
 
-2017-06-26 07:52 +0000 [423d01cf16]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: I7a920170f89c683af9505d4723a44fc6841decdb
 
-	* chan_pjsip: add a new function PJSIP_DTMF_MODE
+2017-12-07 18:18 +0000 [bd2218ce63]  Kevin Harwell <kharwell@digium.com>
 
-	  This function is a replica of SIPDtmfMode, allowing the DTMF mode of a
-	  PJSIP call to be modified on a per-call basis
+	* pjsip_options: dynamic contact's fields not updated on reload
 
-	  ASTERISK-27085 #close
+	  Dynamic contacts were not being properly updated on reload. As a matter of
+	  fact any changes to the AOR that a dynamic contact was associated with were
+	  not being applied.
 
-	  Change-Id: I20eef5da3e5d1d3e58b304416bc79683f87e7612
+	  On reload, this patch makes it so for each dynamic contact, the associated
+	  AOR is now retrieved and the AOR's fields are applied to the contact.
 
-2017-07-25 15:17 +0000 [c16000f201]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27467
 
-	* res_rtp_asterisk: Fix mapping of pjsip's ICE roles to ours
+	  Change-Id: I8e3165dc6a745218c1c9db837f77fafa0516985d
 
-	  Change-Id: Ia578ede1a55b21014581793992a429441903278b
+2017-12-06 23:35 +0000 [c2c9995830]  Corey Farrell <git@cfware.com>
 
-2017-07-20 08:08 +0000 [708cdc0b8e]  Sergej Kasumovic <sergej@bicomsystems.com>
+	* translate: Skip matrix_rebuild during shutdown.
 
-	* res_stasis_device_state: Unsubscribe should remove old subscriptions
+	  Change-Id: I1e5eef4029cba56e33d786c5a5ade8091e531a1e
 
-	  Case scenario with Applications ARI:
+2017-12-06 14:49 +0000 [ab191e9782]  Corey Farrell <git@cfware.com>
 
-	  * Once you subscribe to deviceState with Applications REST API, it will be
-	  added into subscription pool.
+	* sounds_index: Avoid repeatedly reindexing.
 
-	  * When you unsubscribe it will remove from the device_state_subscription
-	  hash table but not from the subscription pool.
+	  The sounds index is rebuilt each time a format is registered or
+	  unregistered.  This causes the index to be repeatedly rebuilt during
+	  startup and shutdown.
 
-	  * When you subscribe again, it will add it to pool again.
+	  This patch significantly reduces the work done by delaying sound index
+	  initialization until after modules are loaded.  This way a reindex only
+	  occurs if a format module is loaded after startup.  We also skip
+	  reindexing when format modules are unloaded during shutdown.
 
-	  * Now you will have two subscriptions and you will receive same event
-	  twice.
+	  Change-Id: I585fd6ee04200612ab1490dc804f76805f89cf0a
 
-	  This fix should now remove deviceState subscription from pool and it
-	  should fix unsubscribe on deviceState.
+2017-12-05 18:04 +0000 [3078b7adc2]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27130 #close
+	* CDR: Fix deadlock setting some CDR values.
 
-	  Change-Id: I718b70d770a086e39b4ddba4f69a3c616d4476c4
+	  Setting channel variables with the AMI Originate action caused a deadlock
+	  when you set CDR(amaflags) or CDR(accountcode).  This path has the channel
+	  locked when the CDR function is called.  The CDR function then
+	  synchronously passes the job to a stasis thread.  The stasis handling
+	  function then attempts to lock the channel.  Deadlock results.
 
-2017-07-24 13:30 +0000 [24bb5a8908]  Joshua Colp <jcolp@digium.com>
+	  * Avoid deadlock by making the CDR function handle setting amaflags and
+	  accountcode directly on the channel rather than passing it off to the CDR
+	  processing code under a stasis thread to do it.
 
-	* core: Add VP9 passthrough support.
+	  * Made the CHANNEL function and the CDR function process amaflags the same
+	  way.
 
-	  This change adds VP9 as a known codec and creates a cached
-	  "vp9" media format for use.
+	  * Fixed referencing the wrong message type in cdr_prop_write().
 
-	  Change-Id: I025a93ed05cf96153d66f36db1839109cc24c5cc
+	  ASTERISK-27460
 
-2017-07-21 15:57 +0000 [07f8e45a90]  Matthew Fredrickson <creslin@digium.com>
+	  Change-Id: I5eacb47586bc0b8f8ff76a19bd92d1dc38b75e8f
 
-	* format.h: Fix a few minor errors in comments.
+2017-12-06 12:42 +0000 [2af59ebb3a]  Corey Farrell <git@cfware.com>
 
-	  A few minor problems were found in comments in format.h.  This patch fixes them.
+	* media_index: Improve startup.
 
-	  Change-Id: I07f0bdb47b93359b361c4c3d8ecc87cd3199dd94
+	  This eliminates some wasteful operations in media_index startup.
 
-2017-07-21 17:04 +0000 [7e9aa74daa]  Rusty Newton <rnewton@digium.com>
+	  * Replace statically set string-fields with char[0].
+	  * Eliminate pointless RAII_VAR's.
+	  * alloc_variant: Avoid pointless ao2_find on new info->variant.
+	  * Stop trying find_variant before alloc_variant.
+	  * process_media_file: replace ast_str with ast_asprintf.  This avoids
+	    reallocation of file_id_str.
 
-	* say.c: Fix file locations for second, seconds, minute, minutes files
+	  Overall sounds_index.c is about 27% of Asterisk startup time when using
+	  sample configs.  This patch reduces it to 20%.  This is a half-fix.  The
+	  real problem is that the media_index is regenerated repeatedly - 68
+	  times in my test.
 
-	  The seconds and minutes files have always existed in the base language
-	  directory of the Core package. So say.c has always been calling the wrong
-	  location (under digits/) for those two files and in the case of second and
-	  minute they didn't exist in the Core packages at all.
+	  Change-Id: Ia50b752f8efb356f852b05c4be495a6631af8652
 
-	  The 1.6 sounds release moves the second and minute files into Core from
-	  Extra for the languages that already had them. A future release will include
-	  the second and minute files for languages that didn't already have them.
+2017-12-06 07:36 +0000 [e97e41552e]  Richard Mudgett <rmudgett@digium.com>
 
-	  This patch just changes all the target locations for second, seconds,
-	  minute, and minutes that were under the digits subdir to be under the root of
-	  sounds instead. Which is where the sounds will be for some languages after 1.6
-	  sounds and for all languages after a future release.
+	* bridge_basic.c: Update transfer diagnostic messages addendum.
 
-	  ASTERISK-25810 #close
+	  * Added start DTMF transfer verbose messages.
+	  * Made associated transfer messages use a similar message format.
+	  * Adjusted message verbose level as requested by initial reporter.
 
-	  Change-Id: I05d9d4bee6a7237030530a46e7eb3df15f13f702
-	  Reported-by: Nicolas Riendeau
+	  ASTERISK-27449
 
-2017-07-19 18:11 +0000 [7ff9d8785d]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I2045714586414b3c5ef1f3cc56c1c4af4b31f551
 
-	* app_voicemail.c: Allow mailbox entry on authentication retry prompt.
+2017-11-29 06:21 +0000 [9d00583164]  Niklas Larsson <niklas@tese.se>
 
-	  The following testsuite voicemail tests were failing to re-enter the
-	  mailbox after the first login attempt.
+	* bridge_basic.c: Update transfer diagnostic messages.
 
-	  tests/apps/voicemail/authenticate_invalid_mailbox
-	  tests/apps/voicemail/authenticate_invalid_password
+	  * Add the channel name to diagnostic messages so you will know which
+	  channel failed to transfer.
 
-	  The tests were noting the start of the vm-incorrect-mailbox prompt and
-	  immediately sending the mailbox for the next login attempt.  Since the
-	  invalid message playback had to complete before the digits were
-	  recognized, the test passed for the wrong reason and added approximately
-	  20 seconds to the test times.
+	  * Promoted some debug messages to verbose 4 messages.
 
-	  * Allow the vm-incorrect-mailbox prompt to get interrupted by the mailbox
-	  digits like the initial vm-login prompt so the tests are able to enter the
-	  intended mailbox.
+	  ASTERISK-27449 #close
 
-	  Change-Id: I1dc53fe917bfe03a4587b2c4cd24c94696a69df8
+	  Change-Id: Idac66b7628c99379cc9269158377fd87dc97a880
 
-2017-07-21 14:20 +0000 [4f93f75e7e]  Rusty Newton <rnewton@digium.com>
+2017-12-01 13:54 +0000 [8536a09b86]  Richard Mudgett <rmudgett@digium.com>
 
-	* Sounds: Update Makefile for Extra sounds 1.5.1 release
+	* security-events: Fix SuccessfulAuth using_password declaration.
 
-	  Incrementing version for the Extra sounds release. 1.5.1 Extra sounds
-	  removes two prompts that were moved into the Core packages in the 1.6 Core
-	  sounds release.
+	  The SuccessfulAuth using_password field was declared as a pointer to a
+	  uint32_t when the field was later read as a uint32_t value.  This resulted
+	  in unnecessary casts and a non-portable field value reinterpret in
+	  main/security_events.c:add_json_object().  i.e., It would work on a 32 bit
+	  architecture but not on a 64 bit big endian architecture.
 
-	  ASTERISK-27142 #close
+	  Change-Id: Ia08bc797613a62f07e5473425f9ccd8d77c80935
 
-	  Change-Id: I82f017812b0ea9599e19dd4635afd55611f13ee7
+2017-11-30 12:50 +0000 [ab63448fa6]  Richard Mudgett <rmudgett@digium.com>
 
-2017-07-20 09:57 +0000 [cea4ce246d]  Sean Bright <sean.bright@gmail.com>
+	* res_rtp_asterisk.c: Increase strictrtp learning timeout time.
 
-	* corosync: Fix corosync library name in configure.ac
+	  More complicated direct media reinvite negotiations can result in longer
+	  delays before direct media flows.  The strictrtp learning timeout time
+	  was too short.  One log showed that the first RTP packet came in just
+	  after three seconds.
 
-	  Also add new corosync packages to install_prereq.
+	  * Increase the strictrtp learning timeout time from 1.5 to 5 seconds.
 
-	  Reported by Travis Ryan in #asterisk-dev
+	  ASTERISK-27453
 
-	  Change-Id: Ib861c95ba630fed62dc54e56784ad8446ed9d2db
+	  Change-Id: Ic5e711164cbb91b4d1c1e40c83697755640f138c
 
-2017-07-18 15:04 +0000 [9a47dd7113]  Benjamin Keith Ford <bford@digium.com>
+2017-12-04 08:33 +0000 [e0354bbe82]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pjsip: Increase maximum packet size.
+	* res_rtp_asterisk: Correct default in sample configuration file.
 
-	  The maximum packet size for PJSIP has been increased to handle the
-	  multiple streams being added for WebRTC.
+	  With Asterisk 12 (commit 866d968), the default of "icesupport" changed to
+	  - "yes" in the module "res_rtp_asterisk" and
+	  - "no" in the module "chan_sip".
+	  The latter was reflected in the sample configuration file for "sip.conf". The
+	  former did not make it into "rtp.conf.sample".
 
-	  Change-Id: I9ea1e8d02668c544acadcb1c6200e1cc1bd588b3
+	  ASTERISK-20643
 
-2017-07-11 04:48 +0000 [1c3e7df26e]  Holger Hans Peter Freyther <holger@moiji-mobile.com>
+	  Change-Id: I2a2e0a900455d0767a99ea576e30adc6d7608a36
 
-	* app_playback.c: Use the timezonename parameter
+2017-12-04 05:27 +0000 [b2c4e8660a]  Alexander Traud <pabstraud@compuserve.com>
 
-	  In say_date_generic the timezonename parameter is passed but never
-	  used. Fix it by passing it to the ast_localtime function.
+	* chan_sip: Peers with distinct source ports don't match, regardless of transport.
 
-	  ASTERISK-27124
+	  Previously, peers connected via TCP (or TLS) were matched by ignoring their
+	  source port. One cannot say anything when protocol:IP:port match, yes (see
+	  <http://stackoverflow.com/q/3329641>). However, when the ports do not match, the
+	  peers do not match as well.
 
-	  Change-Id: I6afa98f9163190043244b9f3ba91eb1874d1b586
+	  This change allows two peers connected to an Asterisk server via TCP (or TLS)
+	  behind a NAT (= same source IP address) to be differentiated via their port as
+	  well.
 
-2017-07-16 12:18 +0000 [51761b759d]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27457
+	  Reported by: Stephane Chazelas
 
-	* res_rtp_asterisk: Use RTP component for ICE if RTCP-MUX is in use.
+	  Change-Id: Id190428bf1d931f2dbfd4b293f53ff8f20d98efa
 
-	  This change makes it so that if an RTCP packet is being sent
-	  the RTP ICE component is used for sending if RTCP-MUX is in use.
+2017-12-04 03:40 +0000 [0611fe581c]  Sungtae Kim <pchero21@gmail.com>
 
-	  ASTERISK-27133
+	* Add new object for VoicemailUserEntry
 
-	  Change-Id: I6200f611ede709602ee9b89501720c29545ed68b
+	  Currently, when the app_voicemail sending VoicemailUserEntry AMI event, there's
+	  no OldMessageCount info for default.
+	  To check the OldMessageCount info, it required IMAP_STORAGE define, but this is
+	  not correct.
+	  Added OldMessageCount item as a default.
 
-2017-07-11 09:55 +0000 [a4c85309f0]  Torrey Searle <torrey@voxbone.com>
+	  ASTERISK-27456
 
-	* res/res_stasis_snoop: generate silence when audiohook returns null
+	  Change-Id: I5c71521c2d1daf8b7b161e31c34d28cca6aea4c7
 
-	  Currently when rtp is paused, no packets are written to the
-	  recorded audio file, causing the silence to be skipped and recording
-	  not properly time aligned.  The read handler as been adapted to
-	  return a silence frame of the correct size.
+2017-12-03 18:49 +0000 [e2715d2cd4]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-27128 #close
+	* pjproject: Clean up disabling of WebRTC support.
 
-	  Change-Id: I2d7f60650457860b9c70907b14426756b058a844
+	  The definition in config_site.h and the argument to the
+	  configure script are not necessary to disable WebRTC
+	  support. The correct argument, --disable-libwebrtc, is
+	  already passed.
 
-2017-07-14 01:25 +0000 [3858d99b73]  Sergej Kasumovic <sergej@bicomsystems.com>
+	  ASTERISK-26980
 
-	* app_confbridge: Make sure name recordings are always removed from the filesystem
+	  Change-Id: I27da2c894f87914956a72710222e17462d8a44bc
 
-	  This commit fixes two possible scenarios:
+2017-12-02 15:55 +0000 [39939cecfa]  Corey Farrell <git@cfware.com>
 
-	  * When recording name and if during recording you hangup, file is never
-	  removed. This is due to the fact file location is nulled.
-	  * When recording name and if you hangup during thank-you prompt, file
-	  is never removed.
+	* autoconf: Remove use of m4_ifblank.
 
-	  ASTERISK-27123 #close
+	  The m4_ifblank macro is not available on CentOS 6, reverse conditionals
+	  to allow use of m4_ifval instead.  ./bootstrap.sh was run but this patch
+	  does not result in any difference to the generated configure script.
 
-	  Change-Id: I39b7271408b4b54ce880c5111a886aa8f28c2625
+	  Change-Id: I280785deb872ed8d3339d99cce63a2b54d5f1438
 
-2017-07-14 01:11 +0000 [cdd6ca488a]  Sergej Kasumovic <sergej@bicomsystems.com>
+2017-11-30 14:38 +0000 [075faac2fd]  George Joseph <gjoseph@digium.com>
 
-	* chan_iax2: On reload make sure to check for existing MWI subscription
+	* AST-2017-013: chan_skinny: Call pthread_detach when sess threads end
 
-	  On every reload of chan_iax2 module, MWI subscription was added, which
-	  results in additional taskprocessors being accumulated over time.
+	  chan_skinny creates a new thread for each new session.  In trying
+	  to be a good cleanup citizen, the threads are joinable and the
+	  unload_module function does a pthread_cancel() and a pthread_join()
+	  on any sessions that are active at that time.  This has an
+	  unintended side effect though. Since you can call pthread_join on a
+	  thread that's already terminated, pthreads keeps the thread's
+	  storage around until you explicitly call pthread_join (or
+	  pthread_detach()).   Since only the module_unload function was
+	  calling pthread_join, and even then only on the ones active at the
+	  tme, the storage for every thread/session ever created sticks
+	  around until asterisk exits.
 
-	  This commit fixes it by making sure we check for existing subscription
-	  first.
+	  * A thread can detach itself so the session_destroy() function
+	    now calls pthread_detach() just before it frees the session
+	    memory allocation.  The module_unload function still takes care
+	    of the ones that are still active should the module be unloaded.
 
-	  This was verified with 'core show taskprocessors' CLI command.
+	  ASTERISK-27452
+	  Reported by: Juan Sacco
 
-	  ASTERISK-27122 #close
+	  Change-Id: I9af7268eba14bf76960566f891320f97b974e6dd
+	  (cherry picked from commit 8f5dff543e457ee3450d21e741901609af0cd779)
 
-	  Change-Id: Ie2ef528fd5ca01b933eeb88188cc10967899cfb9
+2017-12-01 10:01 +0000 [d9fdeae6a4]  Sean Bright <sean.bright@gmail.com>
 
-2017-07-13 15:43 +0000 [9f66fb7901]  Rusty Newton <rnewton@digium.com>
+	* config: Speed up config template lookup
 
-	* Sounds: Update for core sounds 1.6 release
+	  ast_category_get() has an (undocumented) implementation detail where it
+	  tries to match the category name first by an explicit pointer comparison
+	  and if that fails falls back to a normal match.
 
-	  Added necessary lines to make the en_NZ language set selectable and to get
-	  core sounds 1.6 pulled down.
+	  When initially building an ast_config during ast_config_load, this
+	  pointer comparison can never succeed, but we will end up iterating all
+	  categories twice. As the number of categories using a template
+	  increases, this dual looping becomes quite expensive. So we pass a flag
+	  to category_get_sep() indicating if a pointer match is even possible
+	  before trying to do so, saving us a full pass over the list of current
+	  categories.
 
-	  ASTERISK-26807 #close
-	  ASTERISK-25816 #close
-	  ASTERISK-26274 #close
+	  In my tests, loading a file with 3 template categories and 12000
+	  additional categories that use those 3 templates (this file configures
+	  4000 PJSIP endpoints with AOR & Auth) takes 1.2 seconds. After this
+	  change, that drops to 22ms.
 
-	  Change-Id: I84e4dd4696568cc1ba318d12ac4b075461d6eed4
+	  Change-Id: I59b95f288e11eb6bb34f31ce4cc772136b275e4a
 
-2017-07-10 14:04 +0000 [df49ad2528]  Corey Farrell <git@cfware.com>
+2017-12-01 08:29 +0000 [1ad0fbc80e]  Sean Bright <sean.bright@gmail.com>
 
-	* core: Add PARSE_TIMELEN support to ast_parse_arg and ACO.
+	* config: Speed up ACO & sorcery initialization
 
-	  This adds support for parsing timelen values from config files.  This
-	  includes support for all flags which apply to PARSE_INT32.  Support for
-	  this parser is added to ACO via the OPT_TIMELEN_T option type.
+	  When starting Asterisk in the foreground, there is a perceptible delay
+	  when loading modules that use the ACO and sorcery config frameworks.
+	  For example, a lightly configured res_pjsip took 853ms to load on my
+	  VM.
 
-	  Fixes an issue where extra characters provided to ast_app_parse_timelen
-	  were ignored, they now cause an error.
+	  I tracked down the slowness to the XPath queries used to associate the
+	  relevant documentation with the config options. One improvement was
+	  adding a call to xmlXPathOrderDocElems after loading an XML document.
+	  From the libxml2 docs:
 
-	  Testing is included.
+	    Call this routine to speed up XPath computation on static documents.
 
-	  ASTERISK-27117 #close
+	  The second change was to remove recursive descent and wildcard
+	  operators from the XPath queries. After these changes, res_pjsip takes
+	  85ms to load on my VM and there is no longer a perceptible delay when
+	  starting Asterisk in the foreground.
 
-	  Change-Id: I6b333feca7e3f83b4ef5bf2636fc0fd613742554
+	  Change-Id: I45d457f1580e26bf5a2b0dab16e8e9ae46dcbd82
 
-2017-07-12 15:07 +0000 [6d0ff310c6]  Sean Bright <sean.bright@gmail.com>
+2017-12-01 06:07 +0000 [892df22ccd]  Joshua Colp <jcolp@digium.com>
 
-	* basic-pbx: Remove res_pjsip_multihomed from sample config
+	* res_http_post: Not all versions of gmime have GMIME_MAJOR_VERSION.
 
-	  ASTERISK-27127 #close
-	  Reported by: HZMI8gkCvPpom0tM
+	  This change makes the presence of the GMIME_MAJOR_VERSION
+	  definition optional, as not all versions of gmime actually
+	  define it.
 
-	  Change-Id: I2b0c54570d58156e37166ac536728af3b6c01789
+	  ASTERISK-27454
 
-2017-07-11 07:26 +0000 [4e555437dc]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I01d99590045971ed6787899147170a5954077238
 
-	* res_musiconhold:  Add kill_escalation_delay, kill_method to class
+2017-11-30 21:24 +0000 [35a7036a0d]  Corey Farrell <git@cfware.com>
 
-	  By default, when res_musiconhold reloads or unloads, it sends a HUP
-	  signal to custom applications (and all descendants), waits 100ms,
-	  then sends a TERM signal, waits 100ms, then finally sends a KILL
-	  signal.  An application which is interacting with an external
-	  device and/or spawns children of its own may not be able to exit
-	  cleanly in the default times, expecially if sent a KILL signal, or
-	  if it's children are getting signals directly from
-	  res_musiconhoild.
+	* README-SERIOUSLY.bestpractices.txt: Convert to markdown
 
-	  * To allow extra time, the 'kill_escalation_delay'
-	    class option can be used to set the number of milliseconds
-	    res_musiconhold waits before escalating kill signals, with the
-	    default being the current 100ms.
+	  Follow-up to conversion of README.md.
 
-	  * To control to whom the signals are sent, the "kill_method" class
-	    option can be set to "process_group" (the default, existing
-	    behavior), which sends signals to the application and its
-	    descendants directly, or "process" which sends signals only to the
-	    application itself.
+	  Change-Id: I17ee7cf25bc027ece844efa2c1dfe613aff1e35b
 
-	  Change-Id: Iff70a1a9405685a9021a68416830c0db5158603b
+2017-11-17 10:38 +0000 [ce5cfc8ffb]  Corey Farrell <git@cfware.com>
 
-2017-07-03 07:30 +0000 [4f2f3bfebf]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* autoconf: Use m4 conditionals where possible.
 
-	* Avoid setting maxfiles for a remote asterisk
+	  Change-Id: I530c0a72f965437acef6a9a4fbfe5c487f078b65
 
-	  Setting maxfiles (maximum number of open files) has no practical
-	  effect on a remote asterisk (rasterisk, rasterisk -x).
+2017-11-17 09:15 +0000 [87a57e8d46]  Corey Farrell <git@cfware.com>
 
-	  It has an ill effect of printing an extra message, which
-	  may be annoying in case of -x.
+	* autoconf: Fix call to AC_CONFIG_AUX_DIR.
 
-	  ASTERISK-27105 #close
+	  The `pwd` parameter to AC_CONFIG_AUX_DIR is unnecessary, the default
+	  value is $srcdir.
 
-	  Change-Id: Iaf9eb344e4b4b517df91b736b27ec55f6a6921a2
+	  Additionally remove the AC_REVISION call.  It only added a comment and
+	  is pointless without SVN tag replacements.
 
-2017-07-05 15:31 +0000 [32b98ad956]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I99299a3217f095bddcb2edefb3b9af0ab147bc29
 
-	* http.c:  Reduce log spam
+2017-11-20 16:58 +0000 [d12a2ab400]  Corey Farrell <git@cfware.com>
 
-	  Messages like "fwrite() failed: Connection reset by peer" are no
-	  help whatsoever, especially since they can be caused simply by a
-	  client disconnecting.
+	* CLI: Remove compatibility code.
 
-	  * Make those WARNINGs DEBUGs.
-	  * Check the return of the headers fprintf.
+	  Previous commits maintained compatibility with older remote console
+	  clients as well as maintaining all API's.
 
-	  Change-Id: I17bd5f3621514152a7b2b263c801324c5e96568b
+	  Remove the following compatibility code:
+	  * ast_cli_generatornummatches.
+	  * Remote command "_command nummatches".
+	  * Sorting / duplicate removal by remote console.
 
-2017-07-07 11:19 +0000 [25e18bf514]  Benjamin Keith Ford <bford@digium.com>
+	  Change-Id: I59e6ce94fa57ae564888442049695f7e46746437
 
-	* res_pjsip: Fix crash with from_user containing invalid characters.
+2017-11-26 11:47 +0000 [58115e9c21]  Alexander Traud <pabstraud@compuserve.com>
 
-	  If the from_user field contains certain characters (like @, {, ^, etc.),
-	  PJSIP will return a null value for the URI when attempting to parse it.
-	  This causes a crash when trying to dial out through a trunk that contains
-	  these invalid characters in its from_user field.
+	* translate: Transcode siren14, speex32, silk24, and silk12 via slin16.
 
-	  This change checks the configuration and ensures that an endpoint will
-	  not be created if the from_user contains an invalid character. It also
-	  adds a null check to the PJSIP URI parsing as a backup.
+	  When a format has no pre-recorded sound files, Asterisk has to transcode between
+	  formats. For this, Asterisk has a fixed translation table. If the pre-recorded
+	  sound files are not available in the same sample rate, Asterisk has not only to
+	  transcode but also to resample.
 
-	  ASTERISK-27036 #close
-	  Reported by: Maxim Vasilev
+	  Asterisk has pre-recorded files for SLN (8000 kHz) and SLN16 (16000 kHz).
+	  However before this change, Asterisk did not take the sample rate into account,
+	  because the translation paths to SLN and SLN16 got the same score/weight in the
+	  table. Consequently, you might have got narrow-band audio with siren14, speex32,
+	  silk24, and silk12 although those are (ultra) wide-band audio codecs.
 
-	  Change-Id: I0396fdb5080604e0bdf1277464d5c8a85db913d0
+	  With this change, the distance in sample-rates is taken into account. Now on the
+	  Command-Line interface (CLI) 'core show channels', you should see:
+	  (slin@16000)->(slin@32000)->(speex@32000).
 
-2017-06-27 19:27 +0000 [8a803f75a0]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-23735
+	  Reported by: Richard Kenner
 
-	* json.c: Add backtrace log to find 'Invalid UTF-8 string' errors
+	  Change-Id: I9448295c1978be26f8633b6066395e7bbbe2e213
 
-	  Change-Id: I9020ff9f2b3749904317c0c173f47a1bbed6f929
+2017-11-26 09:44 +0000 [55c4d8e008]  Richard Mudgett <rmudgett@digium.com>
 
-2017-07-05 13:39 +0000 [aa514f420b]  Richard Mudgett <rmudgett@digium.com>
+	* res_ari: Fix inverted test giving wrong error message.
 
-	* res_rtp_asterisk.c: Fix TURN deadlock by using ICE session group lock.
+	  The patch for ASTERISK_24560 inverted a test checking if the bridge name
+	  is being updated to a different name.
 
-	  When a message is received on the TURN socket, the code processing the
-	  message needs to call into the ICE/STUN session for further processing.
-	  This code path locks the TURN group lock then the ICE/STUN group lock.  In
-	  another thread an ICE/STUN timer can fire off to send a keep alive message
-	  over the TURN socket.  In this code path, the ICE/STUN group lock is
-	  obtained then the TURN group lock is obtained to send the packet.  A
-	  classic deadlock case if the group locks are not the same.
+	  * Fix the test to return "Changing bridge name is not implemented" when
+	  someone attempts to change the bridge name.
 
-	  * Made TURN get created using the ICE/STUN session's group lock.
+	  ASTERISK-27445
 
-	  NOTE: I was originally concerned that the ICE/STUN session can get
-	  recreated by ice_reset_session() for an event like RTCP multiplexing
-	  causing a change during SDP negotiation.  In this case the TURN group lock
-	  would become different.  However, TURN is also recreated as part of the
-	  ICE/STUN recreation in ice_create() when all known ICE candidates are
-	  added to the new ICE session.  While the ICE/STUN and TURN sessions are
-	  being recreated there is a period where the group locks could be
-	  different.
+	  Change-Id: I4b70bf08b0e02e016108b077ff75b345dec12fc9
 
-	  ASTERISK-27023 #close
-	  Patches:
-	      res_rtp_asterisk-turn-deadlock-fix.patch (license #6502)
-	          patch uploaded by Michael Walton (modified)
+2017-11-25 04:09 +0000 [74e7005a74]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: Ic870edb99ce4988a8c8eb6e678ca7f19da1432b9
+	* translate: Show sample rate for silk, speex, and slin in translation table.
 
-2017-06-23 11:17 +0000 [22c4c1a0ba]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24662
 
-	* bridge_native_rtp.c: Fix direct media video RTP instance ACL check.
+	  Change-Id: I3822956984292c99c48bca8e97807e498ccc0e88
 
-	  The video stream was using the audio stream RTP instance addresses to
-	  check if the video RTP gets directed to an allowed direct media Access
-	  Control List (ACL) address.  There is no guarantee that the video RTP
-	  instance uses the same addresses as the audio RTP instance.
+2017-11-23 13:27 +0000 [02a9952709]  Richard Mudgett <rmudgett@digium.com>
 
-	  This looks like it has been a bug since v11 when direct media ACL was
-	  first added to chan_sip and then faithfully reproduced through a couple
-	  code refactorings into the new bridging architecture.
+	* features.conf.sample: Clarify ActivatedBy documentation wording.
 
-	  Change-Id: I8ddd56320e0eea769f3ceed3fa5b6bdfb51d681a
+	  Change-Id: Id2899331fe05d1909a862ea879742879d086bc64
 
-2017-06-29 13:58 +0000 [194625c1de]  Sean Bright <sean.bright@gmail.com>
+2017-11-22 18:37 +0000 [4b1262c94b]  Corey Farrell <git@cfware.com>
 
-	* app_voicemail: Cleanup ODBC connection handling
+	* Add defaultbranch to .gitreview.
 
-	  The primary focus of this patch is adding a missing call to
-	  ast_odbc_release_obj(), but is also a general cleanup of the ODBC
-	  related code in app_voicemail.
+	  Although the default value of defaultbranch is master I'm adding it
+	  anyways.  This way when new major branches are being created the value
+	  can be updated instead of having to remember the name of the key.
 
-	  ASTERISK-27093 #close
+	  Change-Id: I3db009217c5ae399fb84bee95076f4dbb7fa52d2
 
-	  Change-Id: I8e285142eaeb3146b4287a928276b70db76c902b
+2017-11-22 18:43 +0000 [fcd9ba2b87]  Alexander Anikin <may213@yandex.ru>
 
-2017-06-22 07:47 +0000 [154d2914fa]  Torrey Searle <torrey@voxbone.com>
+	* add cmd connection creation on creation ooh323 call data structure
 
-	* res/res_pjsip_t38  ensure t38 requests get rejected quickly
+	  ASTERISK-27353 #close
 
-	  arm the t38 webhook always, so we can correctly reject a
-	  T38 negotiation request when t38 is disabled on a channel
+	  Reported by: Marco Giordani
 
-	  Change-Id: Ib1ffe35aee145d4e0fe61dd012580be11aae079d
+	  Change-Id: I455096bd7da016b871afe09af86067c2c7c9f33f
 
-2017-07-06 11:52 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2017-11-22 10:42 +0000 [db21f7f2e1]  Kevin Harwell <kharwell@digium.com>
 
-	* asterisk 13.17.0-rc1 Released.
+	* pjsip: 183 without To tag does not negotiate media
 
-2017-07-06 05:55 +0000 [379fe65831]  George Joseph <gjoseph@digium.com>
+	  If a 183 with sdp response is receive without a To tag the sdp is not
+	  negotiated. According to RFC 3261 section 12.1.2 while a To tag is required,
+	  the client needs to still be able to handle the missing tag case for
+	  backwards compatibility.
 
-	* Fix alembic branches
+	  This patch, accepted by and applied to pjproject, makes it so if an incoming
+	  180/183 with SDP comes in without a To tag it gets appropriately handled.
 
-	  Change-Id: I04f607f084bda9b1b7f626e8e9735c37dc751187
+	  ASTERISK-27442 #close
 
-2017-07-05 07:42 +0000 [6258de458b]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: Ic9d6b01e05e8f4874eebbd7adfe05d932025d203
 
-	* core: Fix segfault when invoking 'data get' CLI command
+2017-11-21 06:39 +0000 [1a349d832d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Invoking 'data get /asterisk/core/channeltypes' caused a crash because
-	  of an assumption of a tech's capabilities to be non-NULL. The
-	  'Surrogate' tech, however, does have a NULL capabilities member,
-	  resulting in a crash.
+	* res_rtp_asterisk: ICE server-reflexive candidates (srflx) with Dual-Stack.
 
-	  ASTERISK-27108 #close
+	  Previously, Asterisk sent srflx only when configured exclusively for IPv4. Now,
+	  srflx is gathered and sent via SDP, even when Asterisk is enabled for
+	  Dual Stack (IPv4+IPv6) and an IPv4 interface is available/used.
 
-	  Change-Id: I2fbe7715681f43d5565d1e1599269468c26b0e0a
+	  ASTERISK-27437
 
-2017-07-03 10:59 +0000 [39d2ebbf56]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ie07d8e2bfa7b6fe06fcdc73d390a7a9a4d8c0bc1
 
-	* chan_sip: Only when different, add TCP|TLS in autodomain (SIP Domain Support).
+2017-11-20 13:05 +0000 [8e1506154f]  Corey Farrell <git@cfware.com>
 
-	  When sip.conf contained tcpenable=yes and autodomain=yes, the TCP domain was
-	  added in any case, because of a local Boolean-negation error of the return value
-	  of ast_sockaddr_cmp. After fixing this error for TCP and TLS, the TLS domain was
-	  still always added with tlsenable=yes, because the domains were not compared
-	  just on the address but also on the port – and TLS is always on a different port
-	  than UDP/TCP.
+	* res_parking: Set load_pri more appropriately.
 
-	  ASTERISK-27106
+	  res_parking had an inplicit load_pri of 0 meaning it was one of the very
+	  first modules loaded after modules with global symbols.  Set it to
+	  AST_MODPRI_DEVSTATE_PROVIDER as it provides device state for parking
+	  lots.
 
-	  Change-Id: I14fe9e319e238320b094016980445ef3a5b3337c
+	  Change-Id: I297b6fb3ff6993ec004e667b22a74f5925906259
 
-2017-07-03 10:38 +0000 [9f4b3b966e]  Alexander Traud <pabstraud@compuserve.com>
+2017-11-17 21:33 +0000 [90f9885f73]  Corey Farrell <git@cfware.com>
 
-	* chan_sip: Fix a typo for tlsbindaddr in autodomain (SIP Domain Support).
+	* README: Convert to README.md.
 
-	  Because of a copy-and-paste error when the struct ast_sockaddr changed,
-	  tlsbindaddr was not added, when sip.conf contained autodomain=yes; see
-	  "show sip domains" on the command-line interface (CLI) of Asterisk.
+	  Convert the README file to markdown format, remove the old README.  This
+	  causes websites like github to display the README in a much nicer
+	  format with live links.  The raw file is still very readable from
+	  plain text editors and terminals.
 
-	  ASTERISK-27106
+	  Change-Id: I7d13131764a9a9026e5f8a6ddb245a01bbd788e7
 
-	  Change-Id: I3d0957150017c223136968ef1266f275d0d6695e
+2017-11-20 16:48 +0000 [b79d04f8f8]  Corey Farrell <git@cfware.com>
 
-2017-06-30 23:57 +0000 [73520e9f58]  Corey Farrell <git@cfware.com>
+	* CLI: Finish conversion of completion handling to vectors.
 
-	* channel: Clear channel flag in error branch.
+	  Change-Id: Ib81318f4ee52a5e73b003316e13fe9be1dd897a1
 
-	  Clear channel flag AST_FLAG_END_DTMF_ONLY in ast_waitfordigit_full when
-	  ast_read returns NULL.
+2017-11-07 15:34 +0000 [fbb8c0d3e4]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27100 #close
+	* CLI: Refactor cli_complete.
 
-	  Change-Id: Id3039e9a4e74e0cb359f636c9fd0c9740ebf7d9d
+	  * Stop using "_COMMAND NUMMATCHES" on remote consoles.  Using this
+	    command had doubled the amount of work needed from the Asterisk
+	    daemon for each completion request.
+	  * Fix code formatting.
+	  * Remove static buffer used to send the command, use the same buffer
+	    that will receive the results.
+	  * Move sort from ast_cli_display_match_list.
 
-2017-06-29 18:27 +0000 [0d64cbde57]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie2211b519a3d4bec45bf46e0095bdd01d384cb69
 
-	* pjsip_distributor.c: Fix deadlock with TCP type transports.
+2017-11-07 14:13 +0000 [1cd24cd726]  Corey Farrell <git@cfware.com>
 
-	  When a SIP message comes in on a transport, pjproject obtains the lock on
-	  the transport and pulls the data out of the socket.  Unlike UDP, the TCP
-	  transport does not allow concurrent access.  Without concurrency the
-	  transport lock is not released when the transport's message complete
-	  callback is called.  The processing continues and eventually Asterisk
-	  starts processing the SIP message.  The first thing Asterisk tries to do
-	  is determine the associated dialog of the message to determine the
-	  associated serializer.  To get the associated serializer safely requires
-	  us to get the dialog lock.
+	* CLI: Rewrite ast_el_strtoarr to use vector's internally.
 
-	  To send a request or response message for a dialog, pjproject obtains the
-	  dialog lock and then obtains the transport lock.  Deadlock can result
-	  because of the opposite order the locks are obtained.
+	  This rewrites ast_el_strtoarr to use vector's internally, but still
+	  return the original NULL terminated array of strings.
 
-	  * Fix the deadlock by obtaining the serializer associated with the dialog
-	  another way that doesn't involve obtaining the dialog lock.  In this case,
-	  we use an ao2 container to hold the associated endpoint and serializer.
-	  The new locks are held a brief time and won't overlap other existing lock
-	  times.
+	  Change-Id: Ibfe776cbe14f750effa9ca360930acaccc02e957
 
-	  ASTERISK-27090 #close
+2017-11-07 14:47 +0000 [9c0a2110f0]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I9ed63f4da9649e9db6ed4be29c360968917a89bd
+	* CLI: Refactor ast_cli_display_match_list.
 
-2017-06-29 18:22 +0000 [905d18e8bf]  Richard Mudgett <rmudgett@digium.com>
+	  * Stop estimating line count, just print until we run out of matches.
+	  * Stop freeing entries, the caller does that anyways.
+	  * Stop calculating / returning numoutput, it was ignored.
 
-	* pjsip_distributor.c: Fix unidentified_requests hash functions.
+	  Change-Id: I7f92afa8bea92241a95227587367424c8c32a5cb
 
-	  The OBJ_SEARCH_xxx defines should not be used as if they were individual
-	  bits.  They represent a multi-bit enumeration value field.
+2017-11-08 23:42 +0000 [9587a61f4c]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I32abc9a475396dab02402a7014357dd94284e17b
+	* CLI: Create ast_cli_completion_add function.
 
-2017-06-30 08:31 +0000 [bbe68f139d]  George Joseph <gjoseph@digium.com>
+	  Some completion generators are very inefficent due to the way CLI
+	  requests matches one at a time.  ast_cli_completion_add can be called
+	  multiple times during one invokation of a CLI generator to add all
+	  results without having to reinitialize the search state for each match.
 
-	* pjproject_bundled:  Allow passing configure options to bundled
+	  Change-Id: I73d26d270bbbe1e3e6390799cfc1b639e39cceec
 
-	  There wasn't any good way to pass options like --host or --build
-	  down to the pjproject configure which makes cross-compiling difficult.
+2017-11-09 00:39 +0000 [a02cbc2ef3]  Corey Farrell <git@cfware.com>
 
-	  * Added a new PJPROJECT_CONFIGURE_OPTS environment variable which
-	    can be used to pass arbitrary options to pjproject configure.
-	  * Automatically set the pjproject configure --host and --build
-	    options to match those supplied for the asterisk configure.
+	* CLI: Remove calls to ast_cli_generator.
 
-	  ASTERISK-27097 #close
-	  Reported-by: Kinsey Moore
+	  The ability to add to localized storage cannot be supported by
+	  ast_cli_generator.  The only calls to ast_cli_generator should be by
+	  functions that need to proxy the CLI generator, for example 'cli check
+	  permissions' or 'core show help'.
 
-	  Change-Id: I5fa776e110262851173002a26ffe1172e4c35b2e
+	  * ast_cli_generatornummatches now retrieves the vector of matches and
+	    reports the number of elements (not including 'best' match).
+	  * test_substitution retrieves and iterates the vector.
 
-2017-06-29 14:50 +0000 [6bd7c0f37c]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I8cd6b93905363cf7a33a2d2b0e2a8f8446d9f248
 
-	* chan_pjsip:  Fix ability to send UPDATE on COLP
+2017-11-20 09:13 +0000 [491e2eba0d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  When connected_line_method is "invite", we're supposed to determine
-	  if the client can support UPDATE and if it can, send UPDATE instead
-	  of INVITE to avoid the SDP renegotiation.  Not only was pjproject
-	  not setting the PJSIP_INV_SUPPORT_UPDATE flag, we were testing
-	  that invite_tsx wasn't NULL which isn't always the case.
+	* chan_sip: ICE contained square brackets around IPv6 addresses.
 
-	  * Updated chan_pjsip/update_connected_line_information to drop the
-	    requirement that invite_tsx isn't NULL.
-	  * Submitted patch to pjproject sip_inv.c that sets the
-	    PJSIP_INV_SUPPORT_UPDATE flag correctly.
-	  * Updated pjsip.conf.sample to clarify what happens when "invite"
-	    is specified.
+	  ASTERISK-27434
 
-	  ASTERISK-27095
+	  Change-Id: Iaeed89b4fa05d94c5f0ec2d3b7cd6e93d2d5a8f7
 
-	  Change-Id: Ic2381b3567b8052c616d96fbe79564c530e81560
+2017-11-19 21:23 +0000 [10b4b5d200]  Corey Farrell <git@cfware.com>
 
-2017-06-27 04:37 +0000 [2c43ca0ac5]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	* loader: Fix comments in struct ast_module.
 
-	* app_queue: Fix returning to dialplan when a queue is empty
+	  Make the comments follow doxygen format, move comments to the line
+	  before each field they describe.
 
-	  The fix for ASTERISK-25665 introduced a regression.
-	  The return value of queue_exec used to be 0 in case of leavewhenempty
-	  but it was changed to -1 (returned from wait_our_turn and passed
-	  transparently by queue_exec), thus leading to hangup instead of returning
-	  back to dialplan.
+	  Change-Id: Ic445468398b5e88f13910f7c2f70bd15aad33a27
 
-	  This commit resets the value back to 0 in this case, restoring
-	  original behavior.
+2017-11-16 17:25 +0000 [9ae805c900]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27065 #close
-	  Reported by: Marek Cervenka
+	* cli: Remove silly usage of RAII_VAR.
 
-	  Change-Id: Id9c83b75aeda463250155e88c5004be52bbca5ac
+	  Change-Id: I81aacfee7cd26e4fc5eef07bca582700c2975bd7
 
-2017-06-28 09:03 +0000 [0426b1d88a]  Joshua Colp <jcolp@digium.com>
+2017-11-16 13:19 +0000 [89ccab95c2]  Corey Farrell <git@cfware.com>
 
-	* res_rtp_asterisk: Fix issues with ICE renegotiation.
+	* ccss: Remove silly usage of RAII_VAR.
 
-	  When re-inviting to add more streams it is possible for
-	  the role of existing ICE sessions to be changed to the
-	  incorrect value. This results in subsequent refreshes
-	  within the sessions getting a role conflict and the ICE
-	  session breaking down. This change only sets the role to
-	  be the new value if an ICE renegotiation is actually
-	  going to happen, otherwise the existing role is preserved.
+	  Change-Id: I5ce40035e0a940e4e56f6322c1dcd47fbd509b98
 
-	  As well if we encounter a situation where a unidirectional
-	  ICE negotiation happens and the other side does not send us
-	  candidates we will not store any information for sending
-	  traffic, even though we know where they are reachable. This
-	  change fixes this by using the source of the ICE traffic
-	  itself as the target if no candidates are known and we
-	  receive some ICE traffic.
+2017-11-16 12:51 +0000 [5e99c334d1]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27088
+	* app: Remove silly usage of RAII_VAR.
 
-	  Change-Id: I71228181e358917fcefc3100fad21b2fc02a59a9
+	  Change-Id: Ideb594f7aae134974fb78d5477ba0853b97b8625
 
-2017-06-08 22:50 +0000 [eb48e99bd4]  George Joseph <gjoseph@digium.com>
+2017-11-16 12:19 +0000 [abdd9fa1a8]  Corey Farrell <git@cfware.com>
 
-	* bridge_native_rtp: Keep rtp instance refs on bridge_channel
+	* aoc: Remove silly usage of RAII_VAR.
 
-	  There have been reports of deadlocks caused by an attempt to send a frame
-	  to a channel's rtp instance after the channel has left the native bridge
-	  and been destroyed.  This patch effectively causes the bridge channel to
-	  keep a reference to the glue and both the audio and video rtp instances
-	  so what gets started will get stopped.
+	  Change-Id: I07907f833b81aeb0128bc9442a2abb52679c7511
 
-	  ASTERISK-26978 #close
-	  Reported-by: Ross Beer
+2017-11-16 12:55 +0000 [48e1b39b28]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I9e1ac49fa4af68d64826ccccd152593cf8cdb21a
+	* abstract_jb: Remove silly usage of RAII_VAR.
 
-2017-06-27 10:46 +0000 [1f59d08924]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: I9d56175369363d1dc735504cf78a3a5577069f49
 
-	* res/res_pjsip_t38: fix incorrect increment of media_count
+2017-11-20 13:08 +0000 [d6bbcec571]  Corey Farrell <git@cfware.com>
 
-	  The T38 sdp callback incorrectly has a side effect of incrementing
-	  the media_count.  This can lead to core dumps.
+	* res_mwi_external_ami: Remove incorrect load priority.
 
-	  Change-Id: I7bb2f4987de4046ec52cfc34e5ea0662dae32af8
+	  res_mwi_external_ami specified AST_MODFLAG_LOAD_ORDER but didn't set
+	  load_pri, resulting in an actual load priority of 0.  This module only
+	  provides AMI actions so it has no reason to load early.
 
-2017-06-15 03:12 +0000 [9fbc34d2bd]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: I82987fcf10d3ea42716b2f9df915b16687fd5839
 
-	* res_pjsip:  Add DTMF INFO Failback mode
+2017-11-20 12:54 +0000 [58fa3885cc]  Corey Farrell <git@cfware.com>
 
-	  The existing auto dtmf mode reverts to inband if 4733 fails to be
-	  negotiated.  This patch adds a new mode auto_info which will
-	  switch to INFO instead of inband if 4733 is not available.
+	* Loader: Remove unneeded load_pri declarations.
 
-	  ASTERISK-27066 #close
+	  Instead of specifying AST_MODFLAG_LOAD_ORDER with load_pri
+	  AST_MODPRI_DEFAULT just use AST_MODFLAG_DEFAULT.
 
-	  Change-Id: Id185b11e84afd9191a2f269e8443019047765e91
+	  Change-Id: I0123258eafce324249433a69df15a85cc16e509f
 
-2017-06-21 17:57 +0000 [764d04fa87]  Richard Mudgett <rmudgett@digium.com>
+2017-11-20 09:49 +0000 [7397961b02]  Alexander Traud <pabstraud@compuserve.com>
 
-	* res_pjsip_mwi.c: Eliminate RAII_VAR in contact delete observer
+	* BuildSystem: pjsip_evsub_set_uas_timeout was not used.
 
-	  Change-Id: I0bc97c6608de1d1a4228826b3b3be43f162f05f3
+	  ASTERISK-27435
 
-2017-06-16 18:08 +0000 [0f6a9617eb]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: Id318a7ae6d7d69b53f911d30bf3eece64852f15c
 
-	* res_pjsip_mwi: update unsolicited MWI subscriptions on updating contact
+2017-11-19 09:57 +0000 [b4f7f8250f]  Corey Farrell <git@cfware.com>
 
-	  Do not need to unsubscribe/subscribe on creating the ednpoint's contact.
-	  The modified function create_mwi_subscriptions_for_endpoint adds
-	  the subscription only if it does not exist.
+	* Build: Fix OSX build issues.
 
-	  The subscriptions aren't added for active contacts
-	  which are retrieved on startup from realtime
-	  if mwi_disable_initial_unsolicited=yes.
-	  Because the mwi_contact_added is not called.
-	  So the subscriptions also should be created on updating contact.
+	  OSX does not support 'readlink -f' or 'sed -r'.  Replace readlink with
+	  the GNU make macro 'realpath'.  Replace sed with grep in one place, cut
+	  in the other.
 
-	  ASTERISK-26230 #close
+	  ASTERISK-27332
 
-	  Change-Id: I47e265af9296ca09aa42a316fdacac104148cee4
+	  Change-Id: I5d34ecca905384decb22ead45c913ae5e8aff748
 
-2017-06-20 15:41 +0000 [1f9913f272]  Kevin Harwell <kharwell@digium.com>
+2017-11-19 13:52 +0000 [999e0c17d7]  Corey Farrell <git@cfware.com>
 
-	* core_local: local channel data not being properly unref'ed and unlocked
+	* Build: Fix issues building without SSL.
 
-	  In an earlier version of Asterisk a local channel [un]lock all functions were
-	  added in order to keep a crash from occurring when a channel hung up too early
-	  during an attended transfer. Unfortunately, when a transfer failure occurs and
-	  depending on the timing, the local channels sometime do not get properly
-	  unlocked and deref'ed after being locked and ref'ed. This happens because the
-	  underlying local channel structure gets NULLed out before unlocking.
+	  * Fix conditional in libasteriskssl.
+	  * Use variables produced by configure to link the SSL and uuid libraries
+	    into libasteriskpj.so instead of hard-coding them.
 
-	  This patch reworks those [un]lock functions and makes sure the values that get
-	  locked and ref'ed later get unlocked and deref'ed.
+	  ASTERISK-27431
 
-	  ASTERISK-27074 #close
+	  Change-Id: I3977931fd3ef8c4e4376349ccddb354eb839b58d
 
-	  Change-Id: Ice96653e29bd9d6674ed5f95feb6b448ab148b09
+2017-11-19 13:28 +0000 [53f42cc052]  Corey Farrell <git@cfware.com>
 
-2017-06-20 16:01 +0000 [67664fbf95]  Kevin Harwell <kharwell@digium.com>
+	* res_pjsip: Fix warning by deferring implicit type cast.
 
-	* bridge: stuck channel(s) after failed attended transfer
+	  Mac doesn't like the comparison of -1 to an enum, so store the result of
+	  ast_sip_str_to_dtmf to an int so we can check for the negative return
+	  value.  ast_sip_str_to_dtmf returns an int so this is only delaying the
+	  implicit type cast.
 
-	  If an attended transfer failed it was possible for some of the channels
-	  involved to get "stuck" because Asterisk was not hanging up the transfer target.
+	  Change-Id: I0c262c1719ee951aae1f437d733a301cf5f8ad29
 
-	  This patch ensures Asterisk hangs up the transfer target when an attended
-	  transfer failure occurs.
+2017-11-18 21:13 +0000 [75cb403775]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-27075 #close
+	* tests: Fix warnings found on Mac.
 
-	  Change-Id: I98a6ecd92d3461ab98c36f0d9451d23adaf3e5f9
+	  test_pbx used raise without explicitly including signal.h.  On Mac for
+	  some reason nothing else includes it.
 
-2017-06-19 11:28 +0000 [cecf6540dc]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  test_logger checked if an unsigned int was negative.  Switch the
+	  variable to 'int' so that error check can be effective.
 
-	* cdr: fix mistake spelling of a word for Unanswered.
+	  Change-Id: Ie1db5dd1818ac25cc2ae41b644f848b5865b1362
 
-	  Change-Id: I7a610bef369924523a445c7e849ee88cc45dc5df
+2017-11-18 20:25 +0000 [83a2c4d2ae]  Corey Farrell <git@cfware.com>
 
-2017-06-19 17:21 +0000 [8f356192d1]  Alexei Gradinari <alex2grad@gmail.com>
+	* res_snmp: Declare RONLY if net-snmp headers do not.
 
-	* app_voicemail: IMAP connection control
+	  Some net-snmp builds do not provide the RONLY declare, only
+	  NETSNMP_OLDAPI_RONLY.  Map RONLY to NETSNMP_OLDAPI_RONLY to get around
+	  this error.
 
-	  A new global option "imap_poll_logout" was added to specify whether need to
-	  disconnect from the IMAP server after polling of mailboxes.
+	  Change-Id: Ida5c7ad9406515825485c4d3b4a34fd6ad0da577
 
-	  ASTERISK-27068 #close
+2017-11-18 20:02 +0000 [5a899fc503]  Corey Farrell <git@cfware.com>
 
-	  Closing IMAP connection after loading mailbox from voicemail.conf
+	* res_fax: Remove checks for unsigned values being >= 0.
 
-	  ASTERISK-24052 #close
+	  It's impossible for gwtimeout or fdtimeout to be less than 0 because
+	  they are unsigned int's.  Remove checks and unreachable branches.
 
-	  Change-Id: Ib7558ba04516240a32b65f42e9be64372a0ae12a
+	  Change-Id: Ib2286960621e6ee245e40013c84986143302bc78
 
-2017-06-12 16:17 +0000 [8e749c8f51]  Alexei Gradinari <alex2grad@gmail.com>
+2017-11-18 19:50 +0000 [b4862e463c]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_mwi: unsubscribe unsolicited MWI on deleting endpoint last contact
+	* iostream: Fix ast_iostream_printf declaration.
 
-	  If the endpoint's last contact is deleted unsolicited MWI has to be
-	  unsubscribed.
+	  This adds the printf attribute and changes 'fmt' from 'const void *' to
+	  'const char *'.  This resolves a warning from some compiler for
+	  vsnprintf needing a literal string for format.
 
-	  ASTERISK-27051 #close
+	  Change-Id: I71c33a8262590042ee451e1146760c10bb22fb78
 
-	  Change-Id: I33e174e0b9dba0998927d16d6d100fda5c7254e0
+2017-11-18 19:29 +0000 [2fab3aacd6]  Corey Farrell <git@cfware.com>
 
-2017-06-16 09:31 +0000 [edfdb4dff5]  George Joseph <gjoseph@digium.com>
+	* app_minivm: Fix possible uninitialized return value.
 
-	* res_stasis:  Plug reference leak on stolen channels
+	  Declare 'res' initialized to -1 to deal with earlier error paths that
+	  could cause 'res' to be returned uninitialized.
 
-	  When a stasis channel is stolen by another app, the control
-	  structure is unreffed but never unlinked from the app_controls
-	  container.  This causes the channel reference to leak.
+	  Change-Id: I8ac2a5755bf4174d89ef893e924c940f702b104e
 
-	  Added OBJ_UNLINK to the callback in channel_stolen_cb.
+2017-11-16 02:47 +0000 [0ca406c202]  Pirmin Walthert <infos@nappsoft.ch>
 
-	  Also added some additional channel lifecycle debug messages to
-	  channel.c.
+	* res_rtp_asterisk.c: Fix rtp source address learning for broken clients
 
-	  ASTERISK-27059 #close
-	  Repoorted-by: George Joseph
+	  Some clients do not send rtp packets every ptime ms. This can lead to
+	  situations in which the rtp source learning algorithm will never learn
+	  the address of the client. This has been discovered on a Mac mini with
+	  a pjsip based softphone after updating to Sierra: as soon as USB
+	  headsets are involved, the softphone will send the second packet 30ms
+	  after the first, the third 30ms after the second and the fourth 1ms
+	  after the third. So in the old implmentation the rtp source learning
+	  algorithm was repeatedly reset on the fourth packet.
 
-	  Change-Id: Ib820936cd49453f20156971785e7f4f182c56e14
+	  The patch changes the algorithm in a way that doesn't take the arrival
+	  time between two consecutive packets into account but the time between
+	  the first and the last packet of a learning sequence.
 
-2017-06-16 14:56 +0000 [0a40073750]  Matthew Fredrickson <creslin@digium.com>
+	  The patch also fixes a second problem: when a user was using a wrong
+	  value for the probation setting there was a LOG_WARNING output stating
+	  that the value had been set to the default value instead. However
+	  the code for setting the value back to defaults was missing.
 
-	* formats/format_g729: Fix typo in comment
+	  ASTERISK-27421 #close
 
-	  There was a typo in a comment.  This commit is to fix the typo.
+	  Change-Id: If778fe07678a6fd2041eaca7cd78267d0ef4fc6c
 
-	  ASTERISK-27060 #close
+2017-11-17 19:36 +0000 [9316a064fd]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ic2699f8dbeaacd58ccb6ec3203e853e1babe3235
+	* README: Send people to secure websites where available.
 
-2017-06-12 09:23 +0000 [a6e4899612]  Alexei Gradinari <alex2grad@gmail.com>
+	  We should be sending people to secure web URL's where available.
+	  Update README's and docs.
 
-	* res_pjsip: New endpoint option "notify_early_inuse_ringing"
+	  Change-Id: Id5b1e049b0b18b49a784f1254605aefa244ce19a
 
-	  This option was added to control whether to notify dialog-info state
-	  'early' or 'confirmed' on Ringing when already INUSE.
-	  The value "yes" is useful for some SIP phones (Cisco SPA)
-	  to be able to indicate and pick up ringing devices.
+2017-11-17 19:54 +0000 [5d0529c4d9]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26919 #close
+	* doxygen: Remove obsolete contents.
 
-	  Change-Id: Ie050bc30023543c7dfb4365c5be3ce58c738c711
+	  Remove doxygen contents that have nothing to do with the current state
+	  of Asterisk.
 
-2017-03-30 09:33 +0000 [005a4afa6b]  Jan Friesse <jfriesse@redhat.com>
+	  Change-Id: Ic072cc8641f9533a202990ccf275ce87e3efd95c
 
-	* res_corosync: Change thread stack size
+2017-11-17 09:57 +0000 [1b6e4c1175]  Sean Bright <sean.bright@gmail.com>
 
-	  In Corosync 2.x libraries were changed to use LibQB IPC.
-	  Sadly LibQB IPC doesn't support copy-free access to received buffer, so
-	  Corosync libraries were rewritten to use stack as buffer. Mostly the
-	  needed stack size is quite small, but for all *_dispatch functions, 1MiB
-	  is needed.
+	* res_pjsip: Use reasonable buffer lengths for endpoint identification
 
-	  Asterisk function ast_pthread_create_background set stack size for new
-	  thread to much smaller AST_BACKGROUND_STACKSIZE (~500KiB).
+	  Domains themselves can be up to 255 characters long (per RFC 1035), so
+	  our current buffer sizes are wholly inadequate for many use cases.
 
-	  This results in Asterisk crash when running with Corosync 2.x.
+	  Change-Id: If3f30a68307f1365a1fe06bc4b854c62842c9292
 
-	  Patch solves this issue by creating it's own version of
-	  ast_pthread_create_background which sets stack size to much higher value
-	  (actually it's AST_BACKGROUND_STACKSIZE + 3MiB).
+2017-11-11 10:09 +0000 [b9f4bb5988]  Corey Farrell <git@cfware.com>
 
-	  Another problem may appear when "corosync show members" netconsole
-	  command is executed. It is also executed in thread and also has only
-	  500KiB stack size. Sadly it calls corosync_cfg_get_node_addrs which
-	  again needs at least 1MiB stack.
+	* menuselect: Remove ineffective weak attribute detection.
 
-	  Solution is to use HAVE_COROSYNC_CFG_STATE_TRACK as a discriminator
-	  between Corosync 1.x and 2.x. If 1.x is found, nothing changes. If 2.x
-	  is found, NodeID is displayed instead of IP address.
+	  menuselect detects compiler support for multiple styles of weak
+	  functions.  This is a remnant from 2013 when OPTIONAL_API required weak
+	  functions.  It is no longer correct for menuselect to switch
+	  dependencies from optional to required based on lack of weak function
+	  support.
 
-	  ASTERISK-25370 #close
-	  Reported by: mdu113
+	  Note an issue remains - dependencies should switch from optional to
+	  required based on OPTIONAL_API being enabled or disabled.  I don't think
+	  this is possible.  menuselect needs to know at startup if OPTIONAL_API
+	  is enabled or disabled, so the only way to fix this is to remove
+	  OPTIONAL_API from menuselect and create a configure option.  I've left
+	  the code that switches in place but it's preprocessed out.
 
-	  Change-Id: Id95b0d21ab6e708e7d74ad8786c587211676fa08
+	  Additionally removed:
+	  - WEAKREF variable from Asterisk makeopts.in.
+	  - Related disabled code from test_utils.
+	  - Pointless AC_REVISION call from menuselect/configure.ac.
 
-2017-06-13 11:33 +0000 [7901b9853e]  George Joseph <gjoseph@digium.com>
+	  Change-Id: Ifa702e5f98eb45f338b2f131a93354632a8fb389
 
-	* res_ari:  Add "module loaded" check to ari stubs
+2017-11-16 09:48 +0000 [c4f11911ea]  Corey Farrell <git@cfware.com>
 
-	  The recent change to make the use of LOAD_DECLINE more consistent
-	  caused res_ari to unload itself before declining if the ari.conf
-	  file wasn't found.  The ari stubs though still tried to use the
-	  configuration resulting in segfaults.
+	* acl: Fix allocation related issues.
 
-	  This patch creates a new CHECK_ARI_MODULE_LOADED macro which tests
-	  to see if res_ari is actually loaded and causes the stubs to also
-	  decline if it isn't.  The macro was then added to the mustache
-	  template's "load_module" function.
+	  Add checks for allocation errors, cleanup and report failure when they
+	  occur.
 
-	  ASTERISK-27026 #close
-	  Reported-by: Ronald Raikes
+	  * ast_duplicate_acl_list: Replace log warnings with errors, add missing
+	    line-feed.
+	  * ast_append_acl: Add missing line-feed to logger message.
+	  * ast_append_ha: Avoid ast_strdupa in loop by moving debug message to
+	    separate function.
+	  * ast_ha_join: Use two separate calls to ast_str_append to avoid using
+	    ast_strdupa in a loop.
 
-	  Change-Id: I263d56efa628ee3c411bdcd16d49af6260c6c91d
+	  Change-Id: Ia19eaaeb0b139ff7ce7b971c7550e85c8b78ab76
 
-2017-06-15 13:48 +0000 [3b6c327c51]  Alexei Gradinari <alex2grad@gmail.com>
+2017-11-16 09:04 +0000 [781a520b73]  Joshua Colp <jcolp@digium.com>
 
-	* app_voicemail: IMAP logout on reload/unload
+	* bridge_basic: Ignore answer from transfer target when they've timed out.
 
-	  Closing IMAP connection on module reload or unload.
+	  This is a fun one.
 
-	  ASTERISK-24052 #close
+	  Given the following attended transfer scenario:
 
-	  Change-Id: I2a40182aa9ef249fa6865d33570430e9ada68525
+	  1. Transfer target is called
+	  2. Transferer hangs up
+	  3. Transfer target call attempt reaches timeout
+	  4. Transfer target is told to hang up
+	  5. Transfer target answers before channel is hung up
+	  6. Transferer recall target is called
 
-2017-06-15 12:33 +0000 [b9a4ab8c8c]  Richard Mudgett <rmudgett@digium.com>
+	  A crash would occur. This is because the transfer target call
+	  attempt, despite being told to hang up, would raise a recall
+	  target answer before the recall target had been answered. As it
+	  had not answered there would be no recall target channel and it
+	  would implode.
 
-	* chan_pjsip: Fix PJSIP_MEDIA_OFFER dialplan function read.
+	  This change makes it so that if the transfer target has been
+	  hung up we don't tell the attended transfer code that it has
+	  answered. We also clear out the stimulus that the recall target
+	  has been answered after telling the transfer target to hang up,
+	  in case it was able to raise the information before we told it
+	  to hangup.
 
-	  The construction of the returned string assumed incorrectly that the
-	  supplied buffer would always be initialized as an empty string.  If it is
-	  not an empty string we could overrun the supplied buffer by the length of
-	  the non-empty buffer string plus one.  It is also theoreticaly possible
-	  for the supplied buffer to be overrun by a string terminator during a read
-	  operation even if the supplied buffer is an empty string.
+	  ASTERISK-27361
 
-	  * Fix the assumption that the supplied buffer would already be an empty
-	  string.  The buffer is not guaranteed to contain an empty string by all
-	  possible callers.
+	  Change-Id: Ifb8b255a9c4d2c5c1b8ad77bf54f659ed286df99
 
-	  * Fix string terminator buffer overrun potential.
+2017-11-16 19:39 +0000 [a95f2994c6]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: If6a0806806527678c8554b1dcb34fd7808aa95c9
+	* aoc: Fix memory management issues.
 
-2017-06-15 07:32 +0000 [4910a3bf40]  Joshua Colp <jcolp@digium.com>
+	  aoc_publish_blob failed to check for msg allocation error and never
+	  released msg.
 
-	* channel: Fix reference counting in ast_channel_suppress.
+	  Change-Id: Ib31a9ffb81056a0d496a49d7eec795005a44bcd5
 
-	  The ast_channel_suppress function wrongly decremented the
-	  reference count of the underlying structure used to keep
-	  track of what should be suppressed on a channel if the
-	  function was called multiple times on the same channel.
+2017-11-16 16:18 +0000 [7a735d45e2]  Sean Bright <sean.bright@gmail.com>
 
-	  This change cleans up the reference counting a bit so
-	  this no longer occurs.
+	* res_pjsip_transport_websocket: Give transport a meaningful description
 
-	  ASTERISK-27016
+	  We were not \0 terminating this string, so any attempt to print it would
+	  in the best case show an empty string and in the worst case potentially
+	  crash.
 
-	  Change-Id: I2eed4077cb4916e6626f9f120b63b963acc5c136
+	  Change-Id: I63d96ef8f7516ac02a0f91e22dfa8acdc615042c
 
-2017-06-14 12:34 +0000 [f1a209d5ac]  Richard Mudgett <rmudgett@digium.com>
+2017-11-16 15:00 +0000 [6c53fb5d21]  Sean Bright <sean.bright@gmail.com>
 
-	* app_voicemail.c: Fix compile error when IMAP enabled.
+	* res_pjsip: Use sorcery prefix operation for contact lookup
 
-	  Change-Id: I2703f15b4099b4210c68eccf293105d1975c1fc1
+	  This improves performance for registrations assuming that
+	  res_config_astdb is not in use.
 
-2017-06-08 12:28 +0000 [dc307af7f2]  Frederic LE FOLL <frederic.lefoll@c-s.fr>
+	  Change-Id: I86f37aa9ef07a4fe63448cb881bbadd996834bb1
 
-	* Core/PBX: Deadlock between dialplan execution and application unregistration.
+2017-10-19 14:44 +0000 [d995064fb7]  Nir Simionovich <nirs@greenfieldtech.net>
 
-	  Not easy to reproduce, but we have noticed deadlocks when unloading a module
-	  while dialplan is handling a request.
+	* This patch adds a beanstalk CEL backend.
 
-	  The deadlock is between :
-	  1) Dialplan execution: pbx_extension_helper() first taking conlock,
-	  then pbx_findapp() [when called] asking for lock on apps list.
-	  2) Application unregistration: ast_unregister_application() first taking lock
-	  on apps list, then unreference_cached_app() [when called] asking for conlock.
+	  Beanstalkd is a simple to use job queue. It provides a means to
+	  create multiple job queues called "tubes". Each tube can store
+	  multiple jobs, with varying priorities with the queue. Queue
+	  processing is available via a simple TCP socket or via well defined
+	  libraries, avaialble at
+	  https://github.com/kr/beanstalkd/wiki/client-libraries
 
-	  As a protection, I suggest to modify ast_unregister_application(), so that it
-	  anticipates the need of conlock, before taking the lock on apps list.
-	  The side effect is a longer unavailability of conlock when unregistering an
-	  application.
+	  This module is based upon the beanstalk-client library, available
+	  for download at: https://github.com/deepfryed/beanstalk-client
 
-	  ASTERISK-27041
+	  This module currently doesn't support user defined events.
 
-	  Change-Id: I0db0f1eb320da6a5758cce3a47d765be1face8e2
+	  Change-Id: Ic3a087faeeac045d69a2a018e60e29831ddb95ab
 
-2017-06-14 11:12 +0000 [c2eea791e4]  George Joseph <gjoseph@digium.com>
+2017-11-09 19:58 +0000 [e793501084]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_pubsub:  Fix reference to released endpoint
+	* chan_pjsip.c: Improve answer failure log messages.
 
-	  destroy_subscription was attempting to get the id of the
-	  subscription tree's endpoint after we'd already called ao2_cleanup
-	  on it causing a segfault.
+	  * Balanced the session->inv_session refs on answer failure.
 
-	  Moved the cleanup until after the debug statement and since
-	  endpoint could also be NULL at this point, check for that as well.
+	  Change-Id: I33542d639d37e692cb46550b972a5fcfc3b804b8
 
-	  ASTERISK-27057 #close
-	  Reported-by: Ryan Smith
+2017-11-14 18:00 +0000 [b7b800b689]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ice0a7727f560cf204d870a774c6df71e159b1678
+	* audiohook.c: Fix freeing a frame and still using it.
 
-2017-06-14 08:29 +0000 [2dee95cc7a]  Florian Floimair (license 6892)
+	  Memory corruption happened to the media frame caches when an audio hook
+	  freed a frame when it shouldn't.  I think the freed frame was because a
+	  jitter buffer interpolated a missing frame and the audio hook
+	  unconditionally freed it.
 
-	* res_pjsip_session:  Correct inverted test in session_outgoing_nat_hook
+	  * Made audiohook.c:audio_audiohook_write_list() not free an interpolated
+	  frame if it is the same frame as what was passed into the routine.
 
-	  There was a typo introduced in commit 776ffd77 which was preventing
-	  the transport's external media address from being used.
+	  * Made plc.c:normalise_history() use memmove() instead of memcpy() on a
+	  memory block that could overlap.  Found by valgrind investigating this
+	  issue.
 
-	  ASTERISK-27024 #close
-	  Reported-by: Christopher van de Sande
-	  patches:
-	  	patch.diff submitted by Florian Floimair (license 6892)
+	  ASTERISK-27238
+	  ASTERISK-27412
 
-	  Change-Id: I7ec617171eaa2d86d2680b00cf37d5088adafc27
+	  Change-Id: I548d86894281fc4529aefeb9f161f2131ecc6fde
 
-2017-06-08 17:31 +0000 [e16a669c70]  Jørgen H <asterisk.org@hovland.cx>
+2017-11-15 12:10 +0000 [f512707362]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_transport_websocket: Add NULL check in get_write_timeout
+	* app_record:  Don't set RECORD_STATUS chan var until file is closed
 
-	  Added check for NULL return value when calling
-	  ast_sorcery_retrieve_by_id in function get_write_timeout
+	  We've been calling pbx_builtin_setvar_helper to set the
+	  RECORD_STATUS variable before actually closing the recorded file.
+	  If a client is watching VarSet events and tries to do something with
+	  the file when a RECORD_STATUS event is seen, they might attempt to
+	  do so while the file it's still open.
 
-	  ASTERISK-27046
+	  We now delay calling pbx_builtin_setvar_helper until after we close
+	  the file.
 
-	  Change-Id: I9357717278da631c3a1cb502c412693929b0cb41
+	  ASTERISK-27423
 
-2017-06-14 08:54 +0000 [7dafe82751]  George Joseph <gjoseph@digium.com>
+	  Change-Id: I7fe9de99953e46b4bafa2b38cf151fe8f6488254
 
-	* res_rtp_asterisk:  Fix ssrc change for rtcp srtp
+2017-11-07 08:25 +0000 [cf1cb3345e]  George Joseph <gjoseph@digium.com>
 
-	  It looks like there was a copy/paste error in ast_rtp_change_source
-	  where if there was a rtcp srtp instance, instead of updating its
-	  ssrc we were updating the srtp instance ssrc twice.
+	* ast_coredumper:  Add ability to use directory other than /tmp
 
-	  ASTERISK-27022 #close
-	  Reported-by: Michael Walton
+	  The OUTPUTDIR environment variable can now be set either in the
+	  environment itself or in ast_debug_tools.conf.  If set, it's used
+	  for all work products instead of /tmp.
 
-	  Change-Id: Ic88f3aee7227b401c58745ac265ff92c19620095
+	  Also added the --tarball-config option that includes the contents
+	  of /etc/asterisk when either --tarball-coredumps or --tarball-results
+	  are used.
 
-2017-06-08 14:38 +0000 [e414833f6e]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I66b2553319df61caea5b313d084f51978f730b4c
 
-	* bridge: Add a deferred queue.
+2017-11-13 07:14 +0000 [29e0add14f]  Joshua Colp <jcolp@digium.com>
 
-	  This change adds a deferred queue to bridging. If a bridge
-	  technology determines that a frame can not be written and
-	  should be deferred it can indicate back to bridging to do so.
-	  Bridging will then requeue any deferred frames upon a new
-	  channel joining the bridge.
+	* pjsip / hep: Provide correct local address for Websockets.
 
-	  This change has been leveraged for T.38 request negotiate
-	  control frames. Without the deferred queue there is a race
-	  condition between the bridge receiving the T.38 request
-	  negotiate and the second channel joining and being in the
-	  bridge. If the channel is not yet in the bridge then the T.38
-	  negotiation fails.
+	  Previously for PJSIP the local address of WebSocket connections
+	  was set to the remote address. For logging purposes this is
+	  not particularly useful.
 
-	  A unit test has also been added that confirms that a T.38
-	  request negotiate control frame is deferred when no other
-	  channel is in the bridge and that it is requeued when a new
-	  channel joins the bridge.
+	  The WebSocket API has been extended to allow the local
+	  address to be queried and this is used in PJSIP to set the
+	  local address to the correct value.
 
-	  ASTERISK-26923
+	  The PJSIP HEP support has also been tweaked so that reliable
+	  transports always use the local address on the transport
+	  and do not try to (wrongly) guess. As they are connection
+	  based it is impossible for the source to be anything else.
 
-	  Change-Id: Ie05b08523f399eae579130f4a5f562a344d2e415
+	  ASTERISK-26758
+	  ASTERISK-27363
 
-2017-06-13 14:17 +0000 [6cdf3191d3]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: Icd305fd038ad755e2682ab2786e381f6bf29e8ca
 
-	* res_pjsip_refer/session: Calls dropped during transfer
+2017-11-13 17:47 +0000 [14253f9535]  Corey Farrell <git@cfware.com>
 
-	  When doing an attended transfer it's possible for the transferer, after
-	  receiving an accepted response from Asterisk, to send a BYE to Asterisk,
-	  which can then be processed before Asterisk has time to start and/or
-	  complete the transfer process. This of course causes the transfer to not
-	  complete successfully, thus dropping the call.
+	* alertpipe: Correct documented return of ast_alertpipe_write.
 
-	  This patch makes it so any BYEs received from the transferer, after the REFER,
-	  that initiate a session end are deferred until the transfer is complete. This
-	  allows the channel that would have otherwise been hung up by Asterisk to
-	  remain available throughout the transfer process.
+	  Change-Id: I4ea49c441890a81384144479dc93ab5a3989486d
 
-	  ASTERISK-27053 #close
+2017-11-09 19:47 +0000 [edd1016dd8]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I43586db79079457d92d71f1fd993be9a3b409d5a
+	* core: Use ast_alertpipe for Asterisk signal monitoring thread.
 
-2017-06-13 10:47 +0000 [0bde568669]  George Joseph <gjoseph@digium.com>
+	  Reduce the signal monitoring thread file descriptor use from two to one
+	  on systems that support eventfd.
 
-	* pjproject_bundled:  Use the asterisk github mirror for download
+	  Change-Id: Id4041a237d481ff699639e153ea6982fee14a462
 
-	  We now mirror the pjproject tarball and md5 at
-	  https://github.com/asterisk/third-party/tree/master/pjproject
+2017-11-13 16:20 +0000 [cdaaa14a5f]  Corey Farrell <git@cfware.com>
 
-	  To improve download reliability, we now get the tarball from
-	  our mirror instead of from pjsip.org.
+	* core: Fix configuration of remote console socket path.
 
-	  ASTERISK-27052 #close
-	  Reported-by: 'alex'
+	  The remote console socket path is the combination of asterisk.conf
+	  settings astrundir from [directories] and astctl from [files].
+	  Unconditionally combine the two strings after processing all values
+	  to ensure we end up with the correct socket path.
 
-	  Change-Id: I60236587a8935bfa71fcc391f4e2ecb31918c08a
+	  ASTERISK-27415
 
-2017-06-12 17:55 +0000 [08be5e01e8]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: Ib1e2805d55d6b0955c6430a1a2a93acbf9b091e8
 
-	* app_voicemail: IMAP logout on MWI unsubscribe
+2017-11-10 10:37 +0000 [f6ebd16bb8]  George Joseph <gjoseph@digium.com>
 
-	  Closing IMAP connection on MWI unsubscribe.
+	* bundled_pjproject: sip_parser:  Fix return code in pjsip_find_msg
 
-	  ASTERISK-24052 #close
+	  The default return code for pjsip_find_msg was PJ_SUCCESS so if
+	  a Content-Length header wasn't found at all, pjsip_find_msg was
+	  returning PJ_SUCCESS instead of PJSIP_EMISSINGHDR.
 
-	  Change-Id: I4ff964026002b2817b48c20fb4239f0a880228fd
+	  Also added the volatile keyword to a few variables that are used
+	  both inside and outside the PJ_TRY/PJ_CATCH block.
 
-2017-06-12 09:57 +0000 [59c9bbe696]  Alexei Gradinari <alex2grad@gmail.com>
+	  Partial fix for ASTERISK_27408
 
-	* res_pjsip_mwi: don't create mwi subscriptions if initial unsolicited disabled
+	  Change-Id: If82ba9de921e3d57df9c68cf96ee45ccc1491f7a
 
-	  If sending unsolicited mwi to all endpoints on startup is disabled
-	  (mwi_disable_initial_unsolicited=yes) do not need to create subscriptions.
-	  If there are many (thousands) realtime endpoints configured with unsolicited mwi
-	  and Vociemail Storage configured as ODBC or IMAP there will be huge number of
-	  DB/IMAP requests on startup.
+2017-11-13 14:35 +0000 [2e7f6cd31b]  Ben Ford <bford@digium.com>
 
-	  ASTERISK-26230 #close
+	* bundled_pjproject: Update to 2.7.1
 
-	  Change-Id: I50ae909639e3ee298b931a54def4b2b9e0fb86c5
+	  Update from 2.7 to 2.7.1 for bundled pjproject. Changed version
+	  and removed patch files included in the update.
 
-2017-06-06 14:54 +0000 [68de35a6a0]  David M. Lee <dlee@respoke.io>
+	  Change-Id: I55cea8e734b318c2df9daf86aa0802c559ec8357
 
-	* CFLAGS for BIND8 support
+2017-11-09 08:21 +0000 [ffccce76d9]  Sean Bright <sean.bright@gmail.com>
 
-	  Some systems (like macOS) require BIND_8_COMPAT to be defined so that
-	  the nameser libraries are, well, BIND8 compatible.
+	* sorcery: Add ast_sorcery_retrieve_by_prefix()
 
-	  Change-Id: If79fc27a64f90de1835b5aa3aadfa9be22bd16b0
+	  Some consumers of the sorcery API use ast_sorcery_retrieve_by_regex
+	  only so that they can anchor the potential match as a prefix and not
+	  because they truly need regular expressions.
 
-2017-06-11 12:06 +0000 [da3312457e]  Sean Bright <sean.bright@gmail.com>
+	  Rather than using regular expressions for simple prefix lookups, add
+	  a new operation - ast_sorcery_retrieve_by_prefix - that does them.
 
-	* codecs.conf.sample: Fix max_bandwidth speling error
+	  Change-Id: I56f4e20ba1154bd52281f995c27a429a854f6a79
 
-	  Reported by Sylvain Boily via asterisk-dev mailing list.
+2017-11-07 17:07 +0000 [14d60cee0c]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Idc7623f335aea3e144dd369ba383b9a757480a9d
+	* CLI: Create ast_cli_completion_vector.
 
-2017-06-08 10:54 +0000 [6a64f65fe6]  Guido Falsi <madpilot@freebsd.org>
+	  This is a rewrite of ast_cli_completion_matches using a vector to build
+	  the list.  The original function calls the vector version, NULL
+	  terminates the vector and extracts the elements array.
 
-	* BuildSystem: Add patches to allow building with recent LibreSSL
+	  One change in behavior the results are now sorted and deduplicated. This
+	  will solve bugs where some duplicate checking was done before the list
+	  was sorted.
 
-	  Add some #if defined checks which allow building against LibreSSL.
-	  These patchess come from OpenBSD ports:
-	  https://cvsweb.openbsd.org/cgi-bin/cvsweb/ports/telephony/asterisk/patches/
+	  Change-Id: Iede20c5b4d965fa5ec71fda136ce9425eeb69519
 
-	  ASTERISK-27043 #close
-	  Reported by: OpenBSD ports
+2017-11-07 14:00 +0000 [4930404715]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I2f6c08a5840b85ad4d2b75370b947ddde7a9a572
+	* vectors: Add new macro and a string vector definition.
 
-2017-06-08 10:36 +0000 [44cee2f4a1]  Guido Falsi <madpilot@freebsd.org>
+	  * AST_VECTOR_STEAL_ELEMENTS - steal the array of elements for use
+	    with non-vector code.
+	  * struct ast_vector_string - a vector of 'char *'.
 
-	* BuildSystem: Fix build on FreeBSD due to missing crypt.h
+	  Change-Id: I104d1b204be03fccf67e02a195596adcb5ab1e42
 
-	  FreeBSD does not include a crypt.h include file. Definitions for
-	  crypt() and crypt_r() are in unistd.h
+2017-11-11 13:01 +0000 [90bb0a3e10]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-27042 #close
+	* core: Add cache_media_frames debugging option.
 
-	  Change-Id: Ib307ee5e384870c6af50efa89fb73722dd0c3a7e
+	  The media frame cache gets in the way of finding use after free errors of
+	  media frames.  Tools like valgrind and MALLOC_DEBUG don't know when a
+	  frame is released because it gets put into the cache instead of being
+	  freed.
 
-2017-06-07 15:19 +0000 [1f10c6b3b0]  Joshua Colp <jcolp@digium.com>
+	  * Added the "cache_media_frames" option to asterisk.conf.  Disabling the
+	  option helps track down media frame mismanagement when using valgrind or
+	  MALLOC_DEBUG.  The cache gets in the way of determining if the frame is
+	  used after free and who freed it.  NOTE: This option has no effect when
+	  Asterisk is compiled with the LOW_MEMORY compile time option enabled
+	  because the cache code does not exist.
 
-	* chan_pjsip: Update device state when in early media.
+	  To disable the media frame cache simply disable the cache_media_frames
+	  option in asterisk.conf and restart Asterisk.
 
-	  The chan_pjsip module uses a calculation approach for
-	  determining device state. This means that in situations
-	  where we would expect device state to change we need to
-	  tell the core to query. A scenario that was missed is
-	  when early media was signaled.
+	  Sample asterisk.conf setting:
+	  [options]
+	  cache_media_frames=no
 
-	  This change adds the notification for the core to
-	  query device state when we are told that early media
-	  is being provided.
+	  ASTERISK-27413
 
-	  ASTERISK-27039
+	  Change-Id: I0ab2ce0f4547cccf2eb214901835c2d951b78c00
 
-	  Change-Id: Iafebfd152894966344ff2e950a3cee9f59a3eb6f
+2017-11-11 09:42 +0000 [b865d29f1c]  Richard Mudgett <rmudgett@digium.com>
 
-2017-06-07 14:32 +0000 [590ffcaf0b]  Sean Bright <sean.bright@gmail.com>
+	* frame.c: Make ast_frame_free()/ast_frfree() NULL tolerant
 
-	* eventfd: Disable during cross compilation
+	  Change-Id: Ic49d821ef88ada38a31bdd835b9531443c55d793
 
-	  Reported by Lonnie Abelbeck <lonnie@abelbeck.com> via private e-mail.
+2017-11-10 22:04 +0000 [96987737b9]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Icc80f12b8d8d591e14a8e0ed9f1c02cbd193a89b
+	* menuselect: Delete and ignore aclocal.m4.
 
-2017-06-07 11:21 +0000 [5520b6c201]  Alexei Gradinari <alex2grad@gmail.com>
+	  This file is temporary output from the bootstrap.sh command, it does not
+	  need to be committed.
 
-	* CHANGES: correct version for a new option 'refer_blind_progress'
+	  Change-Id: Ie0fd113aff6eac44924c0bd0c900833c6c86a6d9
 
-	  Change-Id: If4817d26a8974610827624fb8a4e56d681d6bf97
+2017-10-30 22:09 +0000 [e9f8b317c3]  Corey Farrell <git@cfware.com>
 
-2017-06-06 07:04 +0000 [996a4791ff]  Joshua Colp <jcolp@digium.com>
+	* Build: Make function constructor/destructor attributes mandatory.
 
-	* pjsip: Extend 'asymmetric_rtp_codec' option to include us changing.
+	  This change causes the configure script to fail if the C compiler does
+	  not support both function attributes constructor and destructor.  These
+	  were already required as modules cannot function without these attributes
+	  and Asterisk requires modules.
 
-	  PJSIP support in Asterisk differs from chan_sip in that it
-	  allows media to be sent as-is without transcoding provided
-	  the codecs were negotiated in the SDP. This is allowed
-	  according to the RFC. Support for this differs quite a lot
-	  though and some endpoints do not handle it well.
+	  This also has AST_GCC_ATTRIBUTE set a variable
+	  ax_cv_have_func_attribute_$1.  This is the same variable name used by
+	  autoconf-archive's AX_GCC_FUNC_ATTRIBUTE, used for the same purpose.
 
-	  This change extends the 'asymmetric_rtp_codec' option to
-	  also cover this case. When set to no (the default) the code
-	  behaves as chan_sip does - the best codec is selected and
-	  we will only ever send that, unless we change what we are
-	  sending if the remote side changes. When set to yes we
-	  will send media as-is without transcoding if the codec
-	  has been negotiated in the SDP.
+	  Change-Id: Id68e8a1447f2a6d707c54b56350e7bfdb33fb663
 
-	  ASTERISK-26996
+2017-11-10 07:06 +0000 [96f2ee865e]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ib1647f6902a0843e8c435946f831c2159e8d1d51
+	* pjsip: Add patch to allow all transports to be destroyed.
 
-2017-06-06 10:04 +0000 [c093bf8072]  Sean Bright <sean.bright@gmail.com>
+	  If a transport is created with the same transport type, source
+	  IP address, and source port as one that already exists the old
+	  transport is moved into a linked list called "tp_list".
 
-	* res_rtp_multicast: Use consistent timestamps when possible
+	  If this old transport is later shutdown it will not be destroyed
+	  as the process checks whether the transport is valid or not. This
+	  check does not look at the "tp_list" when making the determination
+	  causing the transport to not be destroyed.
 
-	  When a frame destined for a MulticastRTP channel does not have timing
-	  information (such as when an 'originate' is done), we generate the RTP
-	  timestamps ourselves without regard to the number of samples we are
-	  about to send.
+	  This change updates the logic to query not just the main storage
+	  method for transports but also the "tp_list".
 
-	  Instead, use the same method as res_rtp_asterisk and 'predict' a
-	  timestamp given the number of samples. If the difference between the
-	  timestamp that we generate and the one we predict is within a specific
-	  threshold, use the predicted timestamp so that we end up with timestamps
-	  that are consistent with the number of samples we are actually sending.
+	  Upstream issue https://trac.pjsip.org/repos/ticket/2061
 
-	  Change-Id: I2bf0db3541b1573043330421cbb114ff0f22ec1f
+	  ASTERISK-27411
 
-2017-05-31 10:41 +0000 [746c2c5745]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Ic5c2bb60226df0ef1c8851359ed8d4cd64469429
 
-	* res_pjsip: Add support for returning only reachable contacts and use it.
+2017-11-09 20:34 +0000 [bb77666620]  Corey Farrell <git@cfware.com>
 
-	  This introduces the ability for PJSIP code to specify filtering flags
-	  when retrieving PJSIP contacts. The first flag for use causes the
-	  query code to only retrieve contacts that are not unreachable. This
-	  change has been leveraged by both the Dial() process and the
-	  PJSIP_DIAL_CONTACTS dialplan function so they will now only attempt
-	  calls to contacts which are not unreachable.
+	* core: Remove disabled code.
 
-	  ASTERISK-26281
+	  handle_quit has been disabled since 2003, remove it.
 
-	  Change-Id: I8233b4faa21ba3db114f5a42e946e4b191446f6c
+	  Change-Id: Idc3aaa6c81676160547078f9b71e8aa43de2db18
 
-2017-06-05 10:45 +0000 [adfb28882b]  Kevin Harwell <kharwell@digium.com>
+2017-11-09 13:24 +0000 [23b0ef3e9b]  Corey Farrell <git@cfware.com>
 
-	* channel: ast_write frame wrongly freed after call to audiohooks
+	* Build System: Disable parallel make in the root Makefile.
 
-	  ASTERISK-26419 introduced a bug when calling ast_audiohook_write_list in
-	  ast_write. It would free the frame given to ast_write if the frame returned
-	  by ast_audiohook_write_list was different than the given one. The frame
-	  give to ast_write should never be freed within that function. It is the
-	  caller's resposibility to free the frame after writing (or when it its done
-	  with it). By freeing it within ast_write this of course led to some memory
-	  corruption problems.
+	  This ensures that the root Makefile runs only a single target at a time.
+	  SUBMAKE will still honor requested parallelism, so 'make -j8' will build
+	  one directory at a time but allow 8 jobs at once when building a sub
+	  directory.
 
-	  This patch makes it so the frame given to ast_write is no longer freed within
-	  the function. The frame returned by ast_audiohook_write_list is now subsequently
-	  used in ast_write and is freed later. It is freed either after translate if the
-	  frame returned by translate is different, or near the end of ast_write prior
-	  to function exit.
+	  This will fix some display glitches related to rebuild of XML
+	  documentation.  It will also prevent some edge case errors where
+	  bundled pjproject needs to be rebuild before other parts of Asterisk.
 
-	  ASTERISK-26973 #close
+	  Change-Id: I4f2ec6fbbec1ada0ccb1109a28ea303524239b1e
 
-	  Change-Id: I463d4ac3b736ced95de986ee74a489c7c7ab103b
+2017-03-29 20:46 +0000 [12010fc5c0]  Richard Mudgett <rmudgett@digium.com>
 
-2017-05-31 11:45 +0000 [283cc59af7]  Sean Bright <sean.bright@gmail.com>
+	* chan_pjsip.c: Fix uninitialized cause value on failure.
 
-	* pbx_builtin: Properly handle hangup during Background
+	  Change-Id: I3f9dd3c31bd582e54a30381500077de2319d8cc3
 
-	  Before this patch, when a user hung up during a Background, we would
-	  stuff 0xff into a char and attempt a dialplan lookup of it. This caused
-	  problems for some realtime engines which interpreted the value as the
-	  beginning of an invalid UTF-8 sequence.
+2017-11-08 01:40 +0000 [0bda39c668]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-19291 #close
-	  Reported by: Andrew Nowrot
+	* DEBUG_FD_LEAKS: Add missing FD creators.
 
-	  Change-Id: I8ca6da93252d61c76ebdb46a4aa65e73ca985358
+	  This adds FD tracking for the following functions:
+	  * eventfd
+	  * timerfd_create
+	  * socketpair
+	  * accept
 
-2017-05-31 04:25 +0000 [dc05183f4b]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27404
 
-	* channel / app_meetme: Fix parentheses.
+	  Change-Id: Id6848fe904ade2d34eb39d2a20bd6b223e1111fc
 
-	  ASTERISK-27025
+2017-11-07 11:49 +0000 [05f557820b]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Id736b0aa4ec6b6b0f04663d64fa8d151f81fdbed
+	* bridge_softmix: Note why ast_stream_topology_set_stream cannot fail.
 
-2017-05-30 16:07 +0000 [cf6cf59646]  Sean Bright <sean.bright@gmail.com>
+	  This appeared in my audit of ast_stream_topology_set_stream callers
+	  not checking for errors but in this situation the call cannot fail.
+	  Add comment so this can be ignored in the future.
 
-	* stasis_recording: Correct ast_asprintf error checking
+	  Change-Id: I91d25704859efbe50b8b82cfe1cd3c40ba177c9f
 
-	  ASTERISK-27021 #close
-	  Reported by: Tim Morgan
+2017-10-19 13:35 +0000 [dd1a914495]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I0ac061f040093e806c3b1f4e2340864f3ce4dd75
+	* AST-2017-011 - res_pjsip_session: session leak when a call is rejected
 
-2017-05-28 15:43 +0000 [70e5887906]  Sean Bright <sean.bright@gmail.com>
+	  A previous commit made it so when an invite session transitioned into a
+	  disconnected state destruction of the Asterisk pjsip session object was
+	  postponed until either a transport error occurred or the event timer
+	  expired. However, if a call was rejected (for instance a 488) before the
+	  session was fully established the event timer may not have been initiated,
+	  or it was canceled without triggering either of the session finalizing states
+	  mentioned above.
 
-	* format: Reintroduce smoother flags
+	  Really the only time destruction of the session should be delayed is when a
+	  BYE is being transacted. This is because it's possible in some cases for the
+	  session to be disconnected, but the BYE is still transacting.
 
-	  In review 4843 (ASTERISK-24858), we added a hack that forced a smoother
-	  creation when sending signed linear so that the byte order was adjusted
-	  during transmission. This was needed because smoother flags were lost
-	  during the new format work that was done in Asterisk 13.
+	  This patch makes it so the session object always gets released (no more
+	  memory leak) when the pjsip session is in a disconnected state. Except when
+	  the method is a BYE. Then it waits until a transport error occurs or an event
+	  timeout.
 
-	  Rather than rolling that same hack into res_rtp_multicast, re-introduce
-	  smoother flags so that formats can dictate their own options.
+	  ASTERISK-27345 #close
 
-	  Change-Id: I77b835fba0e539c6ce50014a984766f63cab2c16
+	  Reported by: Corey Farrell
 
-2017-05-30 09:34 +0000 [97b003f5e2]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I1e724737b758c20ac76d19d3611e3d2876ae10ed
 
-	* format_mp3: Re-work menuselect/build issues
+2017-10-03 16:19 +0000 [b358e441cd]  Richard Mudgett <rmudgett@digium.com>
 
-	  Rather than removing format_mp3 from ALL_C_MODS (which caused format_mp3
-	  to not show up in menuselect), use .PHONY targets when the necessary
-	  source files are not present.
+	* AST-2017-010: Fix cdr_object_update_party_b_userfield_cb() buf overrun
 
-	  ASTERISK-23951
-	  Reported by: Tzafrir Cohen
+	  cdr_object_update_party_b_userfield_cb() could overrun the fixed buffer if
+	  the supplied string is too long.  The long string could be supplied by
+	  external means using the CDR(userfield) function.
 
-	  Change-Id: I0a7512c51acc9e86043671795020b0de725bd9e8
+	  This may seem reminiscent to AST-2017-001 (ASTERISK_26897) and it is.  The
+	  earlier patch fixed the buffer overrun for Party A's userfield while this
+	  patch fixes the same thing for Party B's userfield.
 
-2017-05-30 09:43 +0000 [c10341646d]  George Joseph <gjoseph@digium.com>
+	  ASTERISK-27337
 
-	* test_json:  Fix test names with reserved words
+	  Change-Id: I0fa767f65ecec7e676ca465306ff9e0edbf3b652
 
-	  Some of the test names were actually reserved words (true, false,
-	  int, null, string, bool).  When the jenkins test results analyzer
-	  does its thing it tries to create a map using the test names as
-	  keys and fails because they're reserved words.
+2017-10-19 13:53 +0000 [74432f51f9]  George Joseph <gjoseph@digium.com>
 
-	  Added "type_" to those test names.
+	* AST-2017-009: pjproject: Add validation of numeric header values
 
-	  Change-Id: I90d809f46969c78a1c605b736ff0635196a2cf1b
+	  Parsing the numeric header fields like cseq, ttl, port, etc. all
+	  had the potential to overflow, either causing unintended values to
+	  be captured or, if the values were subsequently converted back to
+	  strings, a buffer overrun.  To address this, new "strto" functions
+	  have been created that do range checking and those functions are
+	  used wherever possible in the parser.
 
-2017-05-26 11:41 +0000 [b07b216235]  Joshua Colp <jcolp@digium.com>
+	   * Created pjlib/include/limits.h and pjlib/include/compat/limits.h
+	     to either include the system limits.h or define common numeric
+	     limits if there is no system limits.h.
 
-	* manager: Clear the flag on the other channel.
+	   * Created strto*_validate functions in sip_parser that take bounds
+	     and on failure call the on_str_parse_error function which prints
+	     an error message and calls PJ_THROW.
 
-	  During the channel flag audit an incorrect change was
-	  done. The flag should be cleared on the second channel.
+	   * Updated sip_parser to validate the numeric fields.
 
-	  ASTERISK-26469
+	   * Fixed an issue in sip_transport that prevented error messages
+	     from being properly displayed.
 
-	  Change-Id: I770c5a389550a2fb5a6ade942fccbb2e1d9199c8
+	   * Added "volatile" to some variables referenced in PJ_CATCH blocks
+	     as the optimizer was sometimes optimizing them away.
 
-2017-05-26 11:06 +0000 [5e9cd1f20d]  Sean Bright <sean.bright@gmail.com>
+	   * Fixed length calculation in sip_transaction/create_tsx_key_2543
+	     to account for signed ints being 11 characters, not 9.
 
-	* res_srtp: Add support for libsrtp2
+	  ASTERISK-27319
+	  Reported by: Youngsung Kim at LINE Corporation
 
-	  ASTERISK-25294 #close
-	  Reported by: Tzafrir Cohen
+	  Change-Id: I48de2e4ccf196990906304e8d7061f4ffdd772ff
 
-	  ASTERISK-26976 #close
-	  Reported by: Alex
+2017-11-06 17:58 +0000 [2c4db2a3d5]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I789b1c3d1ed31365bbd9339fa58ef36f48833c40
+	* res_pjsip_pubsub: Fix multiple leaks on failure to append vectors.
 
-2017-05-25 11:10 +0000 [72213c98e3]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I68ece0073ea79667ca41eb10405f516f1d30d482
 
-	* format_mp3: Don't try to build format_mp3 if we don't have sources
+2017-11-06 18:12 +0000 [48e96aba6a]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-23951 #close
-	  Reported by: Tzafrir Cohen
+	* res_pjsip_history: Fix multiple leaks on vector append failure.
 
-	  Change-Id: Iebf181d44bb735787fde4b5be863c4d7e2478a30
+	  Change-Id: I41e8d5183ace284095cc721f3b1fb32ade3f940f
 
-2017-05-24 15:50 +0000 [65898c3af8]  George Joseph <gjoseph@digium.com>
+2017-11-06 18:01 +0000 [ecb81ae4de]  Corey Farrell <git@cfware.com>
 
-	* unittests:  Add a unit test that causes a SEGV and...
+	* res_pjsip_session: Fix multiple leaks.
 
-	  ...that can only be run by explicitly calling it with
-	  'test execute category /DO_NOT_RUN/ name RAISE_SEGV'
+	  * Pre-initialize cloned media state vectors to final size to ensure
+	    vector errors cannot happen later in the clone initialization.
+	  * Release session_media on vector replace failure in
+	    ast_sip_session_media_state_add.
+	  * Release clone and media_state in ast_sip_session_refresh if we fail to
+	    append to the stream topology, return an error.
 
-	  This allows us to more easily test CI and debugging tools that
-	  should do certain things when asterisk coredumps.
+	  Change-Id: Ib5ffc9b198683fa7e9bf166d74d30c1334c23acb
 
-	  To allow this a new member was added to the ast_test_info
-	  structure named 'explicit_only'.  If set by a test, the test
-	  will be skipped during a 'test execute all' or
-	  'test execute category ...'.
+2017-11-07 12:03 +0000 [9b3db9a7fd]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ia3a11856aae4887df9a02b6b081cc777b36eb6ed
+	* main/sdp_state: Check for errors from ast_stream_topology_set_stream.
 
-2017-05-23 15:42 +0000 [90237dca11]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I84a83ae69daba5d185cc1d939b133a4c23565497
 
-	* res_agi: Allow configuration of audio format of EAGI pipe
+2017-11-06 16:37 +0000 [0cfc3cbf02]  Richard Mudgett <rmudgett@digium.com>
 
-	  This change allows the format of the EAGI audio pipe to be changed by
-	  setting the dialplan variable 'EAGI_AUDIO_FORMAT' to the name of one of
-	  the loaded formats.
+	* res_pjsip_registrar.c: Fix AOR and pjproject group deadlock.
 
-	  ASTERISK-26124 #close
+	  One of the patches for ASTERISK_27147 introduced a deadlock regression.
+	  When the connection oriented transport shut down, the code attempted to
+	  remove the associated contact.  However, that same transport had just
+	  requested a registration that we hadn't responded to yet.  Depending
+	  upon timing we could deadlock.
 
-	  Change-Id: I7a10fad401ad2a21c68c2e7246fa357d5cee5bbd
+	  * Made send the REGISTER response after we completed processing the
+	  request contacts and released the AOR lock to avoid the deadlock.
 
-2017-05-23 13:33 +0000 [3eb7fbba72]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27391
 
-	* res_agi: Clarify 'RECORD FILE' documentation
+	  Change-Id: I89a90f87cb7a02facbafb44c75d8845f93417364
 
-	  Documented the 'beep' option in both the parameters list and the command
-	  description.
+2017-11-07 11:40 +0000 [eba1179795]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-23839 #close
+	* res_pjsip_session: Check for errors from ast_stream_topology_set_stream.
 
-	  Change-Id: I4970395c922dbdce3f7cf0f56d5b065ec9aa53ea
+	  Free memory and return error if ast_stream_topology_set_stream fails.
 
-2017-05-23 13:06 +0000 [f306e451f6]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I9f4dbf44bed627243d2f1dd8aea2eab6c38a028d
 
-	* res_agi: Prevent crash when SET VARIABLE called without arguments
+2017-11-07 11:34 +0000 [4ac6dd4e95]  Corey Farrell <git@cfware.com>
 
-	  Explicitly check that the appropriate number of arguments were passed to
-	  SET VARIABLE before attempting to reference them. Also initialize the
-	  arguments array to zeroes before populating it.
+	* res_pjsip_t38: Better error checking for t38_create_media_state.
 
-	  ASTERISK-22432 #close
+	  Change-Id: I81b2587427c6982aa3e2a3f9ad69cce8d316eb10
 
-	  Change-Id: I5143607d80a2724f749c1674f3126b04ed32ea97
+2017-11-06 15:38 +0000 [fb18895108]  Corey Farrell <git@cfware.com>
 
-2017-05-23 12:35 +0000 [a007e438c3]  Sean Bright <sean.bright@gmail.com>
+	* stream: Return error from ast_stream_topology_set_stream.
 
-	* res_agi: Fix malformed AGI usage response
+	  ast_stream_topology_set_stream had suppressed error codes from
+	  AST_VECTOR_APPEND.  The result of AST_VECTOR_APPEND needs to be returned
+	  to the caller so they can take appropriate action on the stream.
 
-	  If the generated XML documentation for a command does not end with a \n,
-	  the postamble of the usage message does not appear on its own line.
+	  Change-Id: I6c0d12755743eadba1357f6153526cc055592856
 
-	  ASTERISK-25662 #close
+2017-11-06 17:21 +0000 [801094da7b]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: If190f1e9e37fe215fed95897d78d4a6e142b0020
+	* res_stasis: Fix multiple leaks.
 
-2017-05-23 12:00 +0000 [971a401ce9]  Sean Bright <sean.bright@gmail.com>
+	  * res/stasis/app.c JSON passed to app_send needs to be released.
+	  * res/stasis_message.c: objects leak if vector append fails.
 
-	* sip.conf.sample: Clarify where DTLS settings are permitted
+	  Change-Id: I8dd5385b9f50a5cadf2b1d16efecffd6ddb4db4a
 
-	  ASTERISK-25101 #close
+2017-11-07 06:56 +0000 [02329b9a34]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I09a97793e5577b4422d0ae883fadb3f0d86725cc
+	* res_pjproject.c: Fix ast_strdup() alloc failure.
 
-2017-05-23 10:06 +0000 [700ef6861a]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I74688038e7afe3a279359cce53aadb28ade51ead
 
-	* res_format_attr_h26x: Trim blanks in fmtp attributes
+2017-11-05 22:06 +0000 [a36d8cc533]  Aaron An <anjb@ti-net.com.cn>
 
-	  Some devices separate format attributes with a semicolon followed by a
-	  space, so trim blanks before trying to match them.
+	* res_pjsip:  Avoid crash when contact uri is empty string
 
-	  ASTERISK-27008 #close
+	  Asterisk will crash if contact uri is invalid, so contact_apply_handler
+	  should check if the uri is NULL or empty.
 
-	  Change-Id: Ia44cb2e4fef5c73dc541a29da79cb0e19c22d9cc
+	  ASTERISK-27393 #close
+	  Reported-by: Aaron An
+	  Tested-by: AaronAn
 
-2017-05-15 15:03 +0000 [6bfcb1acc7]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Ia0309bdc6b697c73c9c736e1caec910b77ca69f5
 
-	* app_queue: Fix members showing as being in call when not.
+2017-11-06 17:55 +0000 [7ef38d399a]  Corey Farrell <git@cfware.com>
 
-	  A change was done which added an 'in_call' flag to queue
-	  members that was set to true while talking to an agent.
-	  Unfortunately in practice this does not accurately reflect
-	  whether they are talking to an agent or not. If a Local
-	  channel is involved and a transfer is performed then the
-	  app_queue application would incorrectly think the agent
-	  was still in a call with the caller. This was done to
-	  fix a race condition between an agent becoming available
-	  by device state and the checking of the last call information
-	  for the wrapup time. There was a small window where the
-	  last call information would be the previous value instead
-	  of the new one.
+	* res_pjsip_outbound_registration: Fix leak on vector add failure.
 
-	  This change goes about fixing the original issue in a
-	  different way by considering the call completed if device
-	  state is received which would make the agent available
-	  and if they are currently in a call. If this occurs the
-	  last call information is updated before the agent becomes
-	  available ensuring that old information is not present
-	  when checking if the member should be called. This also
-	  improves the transfer situation by actually updating
-	  and enforcing the wrapup time.
+	  Change-Id: I774b88b3c9da41edd4dc8d78f095481f52f2bd46
 
-	  ASTERISK-26399
-	  ASTERISK-26400
-	  ASTERISK-26715
-	  ASTERISK-26975
+2017-11-06 17:48 +0000 [8684219f79]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ife1cb686e3173b3a6d368601adef9aff69d4beea
+	* res_pjsip_exten_state: Check for vector append failure.
 
-2017-05-23 05:45 +0000 [f1b32de2c5]  Robert Mordec <r.mordec@slican.pl>
+	  Release reference to publisher if we fail to add it to the vector.
 
-	* app_confbridge: Race between removing and playing name recording while leaving
+	  Change-Id: I64dff3f481b67b9884f37cadba7a5ccf23d084f3
 
-	  When user leaves a conference, its channel calls async_play_sound_file()
-	  in order to play the name announcement and then unlinks the sound file.
-	  The async_play_sound_file() function adds a task to conference playback queue,
-	  which then runs playback_common() function in a different thread.
+2017-11-06 17:44 +0000 [f899368cd6]  Corey Farrell <git@cfware.com>
 
-	  It leads to a race condition when, in some cases, channel thread may unlink
-	  the sound file before playback_common() had a chance to open it.
+	* res_pjsip_config_wizard: Fix leaks and add check for malloc failure.
 
-	  This patch creates a file deletion task, that is queued after playback.
+	  wizard_apply_handler():
+	  - Free host if we fail to add it to the vector.
 
-	  ASTERISK-27012 #close
+	  wizard_mapped_observer():
+	  - Check for otw allocation failure.
+	  - Free otw if we fail to add it to the vector.
 
-	  Change-Id: I412f7922d412004b80917d4e892546c15bd70dd3
+	  Change-Id: Ib5d3bcabbd9c24dd8a3c9cc692a794a5f60243ad
 
-2017-05-22 13:51 +0000 [e91efef2bb]  Kevin Harwell <kharwell@digium.com>
+2017-11-06 17:38 +0000 [4016884ef3]  Corey Farrell <git@cfware.com>
 
-	* res_rtp_asterisk: rtcp mux using the wrong srtp unprotecting algorithm
+	* res_stasis_playback: Check for failure to append vector.
 
-	  When using rtcp mux if an rtcp payload came in it would still use the srtp
-	  unprotect algorithm instead of the srtp unprotect rtcp method. Since rtcp
-	  data was being passed to the rtp unprotect method this would result in an
-	  error.
+	  Free resources and return error if we fail to append the vector in
+	  stasis_app_control_play_uri.
 
-	  This patch ensures that the correct unprotect method is chosen by making
-	  sure the passed in rtcp flag is appropriately set when rtcp mux is enabled
-	  and an rtcp payload is received.
+	  Change-Id: I22c4a90dd859b253f2850c6511de48b25609422b
 
-	  ASTERISK-26979 #close
+2017-11-06 17:33 +0000 [24b9751aaa]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ic5409f9d1a267f1d4785fc5aed867daaecca6241
+	* test_sorcery_memory_cache_thrash: Handle error from vector append.
 
-2017-05-19 10:05 +0000 [4479038073]  Sean Bright <sean.bright@gmail.com>
+	  Cleanup resources when we fail to append the vector and report test
+	  failure.
 
-	* chan_sip: Better ICE handling for RTCP-MUX
+	  Change-Id: I6eb41586fd11dee8c0dfe35e91cb465a4cab7298
 
-	  If we are offered or are offering RTCP-MUX, don't consider RTCP ICE
-	  candidates. This confuses certain browsers (current Firefox for
-	  example) and causes intial audio setup delays.
+2017-11-06 17:28 +0000 [29205e7adc]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26982 #close
+	* res_pjsip: Fix leak on error in ast_sip_auth_vector_init.
 
-	  Change-Id: Ifeaf47e83972fe8dbe58b7fb3d6d1823400cfb91
+	  Change-Id: Ib0fc7a18f3135ca8990c3984c9e15f6d26e556e8
 
-2017-04-26 09:22 +0000 [36628cc9c4]  Yasin CANER <yasin.caner@netgsm.com.tr>
+2017-11-06 17:17 +0000 [70fcc043bb]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_session : fixed wrong From Header number On Re-invite
+	* res_pjproject: Handle error from adding to the buildopts vector.
 
-	  ASTERISK-26964 #close
+	  Change-Id: I076c7bd207c7989a23005395ce1735392657be65
 
-	  Change-Id: I55a9caa7dc90e6c4c219cb09b5c2ec08af84a302
+2017-11-06 17:11 +0000 [5247ba4b88]  Corey Farrell <git@cfware.com>
 
-2017-05-13 11:40 +0000 [1618203964]  Joshua Colp <jcolp@digium.com>
+	* res_ari_events: Fix use after free / double-free of JSON message.
 
-	* asterisk: Audit locking of channel when manipulating flags.
+	  When stasis_app_message_handler needs to queue a message for a later
+	  connection it needs to bump the message reference so it doesn't get
+	  freed when the caller releases it's reference.
 
-	  When manipulating flags on a channel the channel has to be
-	  locked to guarantee that nothing else is also manipulating
-	  the flags. This change introduces locking where necessary to
-	  guarantee this. It also adds helper functions that manipulate
-	  channel flags and lock to reduce repeated code.
+	  Change-Id: I82696df8fe723b3365c15c3f7089501da8daa892
 
-	  ASTERISK-26789
+2017-11-06 15:33 +0000 [adb4fdcb7b]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I489280662dba0f4c50981bfc5b5a7073fef2db10
+	* stasis: Release object if vector append fails.
 
-2017-05-08 15:56 +0000 [6af2dd34af]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: I3e5cc669169aab6175ddfaf7486edeaeb4fdcfb1
 
-	* res_pjsip: New endpoint option "refer_blind_progress"
+2017-11-06 15:20 +0000 [2f4f216026]  Corey Farrell <git@cfware.com>
 
-	  This option was added to turn off notifying the progress details
-	  on Blind Transfer. If this option is not set then the chan_pjsip
-	  will send NOTIFY "200 OK" immediately after "202 Accepted".
+	* RTP Engine: Deal with errors returned from AST_VECTOR_REPLACE.
 
-	  Some SIP phones like Mitel/Aastra or Snom keep the line busy until
-	  receive "200 OK".
+	  Check for errors from AST_VECTOR_REPLACE and clean memory if needed.
 
-	  ASTERISK-26333 #close
+	  Change-Id: I124d15cc1d645f85a72a1279f623c1993b304b0b
 
-	  Change-Id: Id606fbff2e02e967c02138457badc399144720f2
+2017-11-06 15:16 +0000 [5762f72425]  Corey Farrell <git@cfware.com>
 
-2017-05-22 20:20 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* PBX: Handle errors from AST_VECTOR_APPEND.
 
-	* asterisk 13.16.0-rc1 Released.
+	  This resolves potentials leaks on AST_VECTOR_APPEND error in:
+	  * ast_context_add_include2
+	  * ast_context_add_switch2
+	  * ast_context_add_ignorepat2
 
-2017-04-13 17:16 +0000 [919ccdb9ac]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: Ib60e95c4f622fa3b832d87227c0523a695d736b6
 
-	* AST-2017-002: Ensure transaction key buffer is large enough.
+2017-11-06 15:10 +0000 [714026b32e]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26938 #close
+	* Messaging: Report error on failure to register tech or handler.
 
-	  Change-Id: I266490792fd8896a23be7cb92f316b7e69356413
+	  Message tech and handler registrations use a vector which could fail to
+	  expand.  If it does log and error and return error.
 
-2017-04-13 17:17 +0000 [49c032abef]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I593a8de81a07fb0452e9b0efd5d4018b77bca6f4
 
-	* AST-2017-003: Handle zero-length body parts correctly.
+2017-11-06 15:07 +0000 [e43c8af77c]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26939 #close
+	* format_cap: Fix leak on AST_VECTOR_APPEND error.
 
-	  Change-Id: I7ea235ab39833a187db4e078f0788bd0af0a24fd
+	  format_cap_framed_init can fail on AST_VECTOR_APPEND.  This should
+	  report failure to the caller and clean the newly allocated frame.
 
-2017-04-13 11:14 +0000 [1cc18d4025]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: Ica0661235bf09497bf23d844ceb01f21b41a55b0
 
-	* AST-2017-004: chan_skinny:  Add EOF check in skinny_session
+2017-11-06 14:23 +0000 [64bcb65a78]  Corey Farrell <git@cfware.com>
 
-	  The while(1) loop in skinny_session wasn't checking for EOF so
-	  a packet that was longer than a header but still truncated
-	  would spin the while loop infinitely.  Not only does this
-	  permanently tie up a thread and drive a core to 100% utilization,
-	  the call of ast_log() in such a tight loop eats all available
-	  process memory.
+	* stasis: Remove silly use of RAII_VAR in stasis_forward_all.
 
-	  Added poll with timeout to top of read loop
+	  Change-Id: I46de4c968d40144d5b049966304ff66c1469fb65
 
-	  ASTERISK-26940 #close
-	  Reported-by: Sandro Gauci
+2017-11-06 12:51 +0000 [b7e1034009]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I2ce65f3c5cb24b4943a9f75b64d545a1e2cd2898
+	* CLI: Remove unused internal command.
 
-2017-05-18 16:35 +0000 [c107ab4c04]  Sean Bright <sean.bright@gmail.com>
+	  The internal CLI command "_command complete" was last used by Asterisk
+	  0.2.0.  Since then we've been using "_command nummatches" and "_command
+	  matchesarray".
 
-	* res_hep_rtcp: Add support level to module info
+	  Change-Id: I682fe1e21a24a3bb5bd04146e639f1c5866bcfce
 
-	  Change-Id: I5661478f9cf12d431f730e42be79323b62831e92
+2017-11-03 18:08 +0000 [923424019b]  Richard Mudgett <rmudgett@digium.com>
 
-2017-05-11 00:25 +0000 [cfeae52c0f]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	* stasis_bridges.c: Fix off-nominal json memory leaks.
 
-	* app_queue: Fix duplicate queue_log entries for EXITEMPTY and ABANDON
+	  Change-Id: Ib1181a36b317c86bff1ef2e44a17a0b1c73cfdc8
 
-	  There are 2 places in app_queue.c that log EXITEMPTY event: one in
-	  wait_our_turn, and another one in queue_exec in the loop trying to
-	  call an agent after wait_our_turn.
+2017-11-03 17:43 +0000 [f81970d3fc]  Richard Mudgett <rmudgett@digium.com>
 
-	  In most cases it leads to logging EXITEMPTY twice.
+	* stasis_channels.c: Remove a very silly RAII_VAR().
 
-	  ABANDON is also logged on two places, and in the rare case when an agent
-	  and caller hang up simultaneously it's also possible to get duplicates
-	  in queue_log.
+	  Change-Id: I28b458b3c1a442c4ef0be7b4986a95ea4149e14f
 
-	  This commit changes wait_our_turn to return -1 ("the caller should exit
-	  the queue") instead of 0 ("the caller's turn has arrived") in case of
-	  leaving when empty, so queue_exec skips the agent calling loop.
+2017-11-06 10:29 +0000 [36fedea8c1]  Joshua Colp <jcolp@digium.com>
 
-	  Also, leave_queue is now executed only once in this case, because 2nd
-	  time is just a noop when the queue entry has already been removed.
+	* res_pjsip_pubsub: Ensure remote URI contains URI only.
 
-	  Also, it sets qe->handled to -1 to indicate that the call was not
-	  answered by an agent, but the necessary handling has already been done
-	  in order to avoid logging an extra ABANDON entry.
+	  This change makes it so that any user of the pubsub
+	  API that requests the remote URI receives only the URI.
+	  Previously the entire string was returned, which could
+	  contain a display name.
 
-	  ASTERISK-25665 #close
-	  Reported by: Ove Aursand
+	  ASTERISK-27290
 
-	  Change-Id: I4578dd383bf2ac41589cf167865e8aaebcd4c11e
+	  Change-Id: If1d0cd6630f0a264856d31d2a67933109187a017
 
-2017-05-14 00:37 +0000 [5da91c65be]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+2017-11-03 16:14 +0000 [9771f089f5]  Richard Mudgett <rmudgett@digium.com>
 
-	* Fix spelling queues.conf.sample file
+	* stasis/app.c: Optimize stasis_app_get_debug_by_name()
 
-	  Change-Id: Ie1c2d83af66f27a449da09a68d987e0992627fee
+	  * Eliminate RAII_VAR()
+	  * Short circuit application name lookup if global debug enabled.
 
-2017-05-12 21:04 +0000 [b67363006f]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I5f78b7bd6ca7fd2c3b07cbbe036c6a93b4681123
 
-	* res_pjsip_session.c: Process initial INVITE sooner. (key exists)
+2017-11-02 18:40 +0000 [ee08f10d06]  Richard Mudgett <rmudgett@digium.com>
 
-	  Retransmissions of an initial INVITE could be queued in the serializer
-	  before we have processed the first INVITE message.  If the first INVITE
-	  message doesn't get completely processed before the retransmissions are
-	  seen then we could try to setup the same call from the retransmissions.  A
-	  symptom of this is seeing a (key exists) message associated with an
-	  INVITE.  An earlier change attempted to address this kind of problem by
-	  calculating a distributor serializer to use for unassociated messages.
-	  Part of that change also made incoming calls keep using that distributor
-	  serializer.  (ASTERISK-26088) However, some leftover code was still
-	  deferring the INVITE processing to the session's serializer even though we
-	  were already in that serializer.  This not only is unnecessary but would
-	  cause the same call resetup problem.
+	* Fix ast_(v)asprintf() malloc failure usage conditions.
 
-	  * Removed the code to defer processing the initial INVITE to the session's
-	  serializer because we are already running in that serializer.
+	  When (v)asprintf() fails, the state of the allocated buffer is undefined.
+	  The library had better not leave an allocated buffer as a result or no one
+	  will know to free it.  The most likely way it can return failure is for an
+	  allocation failure.  If the printf conversion fails then you actually have
+	  a threading problem which is much worse because another thread modified
+	  the parameter values.
 
-	  ASTERISK-26998 #close
+	  * Made __ast_asprintf()/__ast_vasprintf() set the returned buffer to NULL
+	  on failure.  That is much more useful than either an uninitialized pointer
+	  or a pointer that has already been freed.  Many uses won't have to check
+	  for failure to ensure that the buffer won't be double freed or prevent an
+	  attempt to free an uninitialized pointer.
 
-	  Change-Id: I1e822d82dcc650e508bc2d40d545d5de4f3421f6
+	  * stasis.c: Fixed memory leak in multi_object_blob_to_ami() allocated by
+	  ast_asprintf().
 
-2017-05-08 13:40 +0000 [1bcce442d0]  Vitezslav Novy <a1@vnovy.net>
+	  * ari/resource_bridges.c:ari_bridges_play_helper(): Remove assignment to
+	  the wrong thing which is now not needed even if assigning to the right
+	  thing.
 
-	* chan_sip: Change sip_get_codec() to return correct codec list
+	  Change-Id: Ib5252fb8850ecf0f78ed0ee2ca0796bda7e91c23
 
-	  Return cahnnel nativeformats to fix bridge technology selection process.
-	  Same approach as in pjsip module.
+2017-11-06 08:05 +0000 [ca4e6b568f]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26143
-	  Reported-by: Henning Holtschneider
+	* res_pjsip: Ignore empty TLS configuration
 
-	  Change-Id: I64e863753954d6ad67a9e722df2ebc328705ad48
+	  When using realtime, fields that are not explicitly set by an
+	  administrator are still presented to sorcery as empty strings. Handle
+	  this case explicitly.
 
-2017-05-09 10:34 +0000 [6fba0a41f0]  Joshua Colp <jcolp@digium.com>
+	  In this particular case, if any of these fields are required for TLS
+	  support, their existence should be validated in the 'apply' handler once
+	  we have a complete transport definition.
 
-	* tcptls: Improve error messages for TLS connections.
+	  ASTERISK-27032 #close
+	  Reported by: seanchann.zhou
 
-	  This change uses the functions provided by OpenSSL to query
-	  and better construct error messages for situations where
-	  the connection encounters a problem.
+	  Change-Id: Ie3b5fb421977ccdb33e415d4ec52c3fd192601b7
 
-	  ASTERISK-26606
+2017-09-29 09:50 +0000 [04d3785a79]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I7ae40ce88c0dc4e185c4df1ceb3a6ccc198f075b
+	* dtls: Add support for ephemeral DTLS certificates.
 
-2017-05-04 17:28 +0000 [8ec6e19c86]  Joshua Elson <joshelson@gmail.com>
+	  This mimics the behavior of Chrome and Firefox and creates an ephemeral
+	  X.509 certificate for each DTLS session.
 
-	* Prevent Undefined Capath Crash
+	  Currently, the only supported key type is ECDSA because of its faster
+	  generation time, but other key types can be added in the future as
+	  necessary.
 
-	  It is possible to initialize a valid config without a capath
-	  or cafile definition. This will cause a crash on a reload.
+	  ASTERISK-27395
 
-	  This fix ensures capath is always allocated.
+	  Change-Id: I5122e5f4b83c6320cc17407a187fcf491daf30b4
 
-	  ASTERISK-26983 #close
+2017-11-06 03:21 +0000 [4013bfa52b]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I63ff715d9d9023427543a5b8a4ba7b0d82533c12
+	* configure: Add autoconf check for libopusfile.
 
-2017-05-05 11:33 +0000 [d6325373ac]  gtjoseph <gjoseph@digium.com>
+	  This check is being added to make it easier for end-users of third party
+	  open source Opus modules.  This was removed by ASTERISK-26426 but only
+	  the module needed to be removed.
 
-	* cel_odbc:  Fix timestamp processing for microseconds
+	  Change-Id: I62b9cd0c4fa8a77596ab0e042948a643a1152677
 
-	  When a column is of type timestamp, the fraction part of the event
-	  field's seconds was frequently parsed incorrectly especially if
-	  there were leading zeros.  For instance "2017-05-23 23:55:03.023"
-	  would be parsed into an int as "23" then when the timestamp was
-	  formatted again to be inserted into the database column it'd be
-	  "2017-05-23 23:55:03.23" which is now 230 milliseconds instead of
-	  23 milliseconds.  "03.000001" would be transformed to "03.1", etc.
+2017-11-06 03:18 +0000 [19332e6968]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * If the event field is 'eventtime' and the db column is timestamp,
-	    then existing processing has already correctly formatted the
-	    timestamp so now we simply use it rather than parsing it and
-	    re-printing it. This is the most common use case anyway.
+	* tcptls: Print notice when TLS is enabled but not configured.
 
-	  * If the event field is other than 'eventtime' and the db column
-	    is timestamp, we now parse the seconds, including the fractional
-	    part into a double rather than 2 ints.  This preserves the
-	    magnitude and precision of the fractional part.  When we print
-	    it, we now print it as a "%09.6lf" which correctly represents the
-	    input.
+	  Asterisk can be compiled without a SSL/TLS library, without the Development
+	  Headers of OpenSSL. However, if TLS (SIP) or Secure-WebSockets (WebRTC) was
+	  enabled in a configuration file, Asterisk did not notice the user. Asterisk
+	  failed silently, only the corresponding TCP ports were not open.
 
-	  To be honest, why we parse the string timestamp into components,
-	  test the components, then print the components back into a string
-	  timestamp is beyond me.  We should use parse it, test it, then if
-	  it passes, use the original string representation in the database
-	  call.  Maybe someone thought that some implementations wouldn't
-	  take a partial timestamp string like "2017-05-06" and decided to
-	  always produce a full timestamp string even if an abbreviated one
-	  was supplied.  Anyway, I'm leaving it as it is.
+	  ASTERISK-27394
+	  Reported-by: mossley74
 
-	  ASTERISK-25032 #close
-	  Reported-by: Etienne Lessard
+	  Change-Id: Ib8b7539a5b2af8154c22e5f7a40fc68f95d95b93
 
-	  Change-Id: Id407e6221f79a5c1120e1a70bc7e893bbcaf1938
+2017-11-04 06:05 +0000 [2ebea5aa03]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-05-09 05:25 +0000 [10a49ab362]  Joshua Colp <jcolp@digium.com>
+	* install_prereq: Checkout of libSRTP 2.x.
 
-	* res_hep_rtcp: Provide chan_sip Call-ID for RTCP messages.
+	  Since Asterisk 13.17, libSRTP 2.x is supported. Therefore, its latest version
+	  is installed again via the script install_prereq.
 
-	  This change adds the required logic to allow the SIP
-	  Call-ID to be placed into the HEP RTCP traffic if the
-	  chan_sip module is used. In cases where the option is
-	  enabled but the channel is not either SIP or PJSIP then
-	  the code will fallback to the channel name as done
-	  previously.
+	  ASTERISK-27356
 
-	  Based on the change on Nir's branch at:
-	  team/nirs/hep-chan-sip-support
+	  Change-Id: I13125839a79052356469e41edacbebff0a937d39
 
-	  ASTERISK-26427
+2017-11-01 17:47 +0000 [79ddcdbc70]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I09ffa5f6e2fdfd99ee999650ba4e0a7aad6dc40d
+	* Stasis/ARI: Fix off-nominal path json memory leaks.
 
-2017-05-08 16:11 +0000 [7d4a22bf2e]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: Id569c624c426e3b22a99936473c730592d8b83fb
 
-	* logger:  Added logger_queue_limit to the configuration options.
+2017-11-02 11:38 +0000 [229790ea3d]  Richard Mudgett <rmudgett@digium.com>
 
-	  All log messages go to a queue serviced by a single thread
-	  which does all the IO.  This setting controls how big that
-	  queue can get (and therefore how much memory is allocated)
-	  before new messages are discarded. The default is 1000.
-	  Should something go bezerk and log tons of messages in a tight
-	  loop, this will prevent memory escalation.
+	* AOC: Fix AOC-S json memory leak.
 
-	  When the limit is reached, a WARNING is logged to that effect
-	  and messages are discarded until the queue is empty again.  At
-	  that time another WARNING will be logged with the count of
-	  discarded messages.  There's no "low water mark" for this queue
-	  because the logger thread empties the entire queue and processes it
-	  in 1 batch before going back and waiting on the queue again.
-	  Implementing a low water mark would mean additional locking as
-	  the thread processes each message and it's not worth it.
+	  Change-Id: I3a1d40a41a8a7d00fa4a187de6a343a79155d3ef
 
-	  A "test" was added to test_logger.c but since the outcome is
-	  non-deterministic, it's really just a cli command, not a unit
-	  test.
+2017-11-01 18:04 +0000 [de4a4796d0]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ib4520c95e1ca5325dbf584c7989ce391649836d1
+	* res_stasis_device_state.c: Optimize stasis_app_device_states_to_json()
 
-2017-05-04 17:32 +0000 [614eda785d]  Richard Mudgett <rmudgett@digium.com>
+	  * Eliminate RAII_VAR()
+	  * Replace looped alloca with a char[] since that is how it is used anyway.
 
-	* netsock2.c: Made get/set addr port avoid potential uninitialized memory.
+	  Change-Id: Ia27e64a884afa0f50b9ffdb1cf23da6bfa51ffdf
 
-	  Change-Id: I532052bd7cd95a4b3565485fc01e2a1ea07ee647
+2017-11-01 18:58 +0000 [103b05bb4b]  Richard Mudgett <rmudgett@digium.com>
 
-2017-05-05 08:48 +0000 [c3ed63cb2c]  Joshua Colp <jcolp@digium.com>
+	* res_stasis_mailbox.c: Fix leak of mailbox container.
 
-	* func_cdr: Allow empty value for CDR dialplan function.
+	  Change-Id: I7d33c1635713047e7d1597c9d882f7dc006d94b4
 
-	  A regression was introduced in 12 where passing an empty value
-	  to the CDR dialplan function was not longer allowed. This
-	  change returns to the behavior of 11 where it is permitted.
+2017-11-03 10:35 +0000 [290bad22c9]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26173
+	* Build System: Fix build failure caused by recent CLI improvements.
 
-	  Change-Id: I3f148203b54ec088007e29e30005a5de122e51c5
+	  We use the editline library to help with filename completion in our CLI
+	  interface.  Some systems failed to find the header when included from
+	  loader.c.  This is fixed by setting the proper CFLAGS for the build of
+	  loader.o.
 
-2017-05-04 16:04 +0000 [bed6c0d04b]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-27378
 
-	* app_confbridge:  Fix reference to cfg in menu_template_handler
+	  Change-Id: Ib7fd496f1d7ed48141a2eadd5dd61cab2f2308be
 
-	  menu_template_handler wasn't properly accounting for the fact that
-	  it might be called both during a load/reload (which isn't really
-	  valid but not prevented) and by a dialplan function.  In both cases
-	  it was attempting to use the "pending" config which wasn't valid in
-	  the latter case.  aco_process_config is also partly to blame because
-	  it wasn't properly cleaning "pending" up when a reload was done and
-	  no changes were made.  Both of these contributed to a crash if
-	  CONFBRIDGE(menu,template) was called in a dialplan after a reload.
+2017-11-01 11:12 +0000 [f8e0f9be22]  Ben Ford <bford@digium.com>
 
-	  * aco_process_config now sets info->internal->pending to NULL
-	    after it unrefs it although this isn't strictly necessary in the
-	    context of this fix.
-	  * menu_template_handler now uses the "current" config and silently
-	    ignores any attempt to be called as a result of someone uses the
-	    "template" parameter in the conf file.
+	* res_pjsip: Add to list of valid characters for from_user.
 
-	  Luckily there's no other place in the codebase where
-	  aco_pending_config is used outside of aco_process_config.
+	  Fixes a regression where some characters were unable to be used in
+	  the from_user field of an endpoint. Additionally, the backtick was
+	  removed from the list of valid characters, since it is not valid,
+	  and it was replaced with a single quote, which is a valid character.
 
-	  ASTERISK-25506 #close
-	  Reported-by: Frederic LE FOLL
+	  ASTERISK-27387
 
-	  Change-Id: Ib349a17d3d088f092480b19addd7122fcaac21a7
+	  Change-Id: Id80c10a644508365c87b3182e99ea49da11b0281
 
-2017-04-30 16:40 +0000 [7ffd80cc04]  Joshua Colp <jcolp@digium.com>
+2017-11-02 05:34 +0000 [8701479386]  Joshua Colp <jcolp@digium.com>
 
-	* bridge: Fix returning to dialplan when executing Bridge() from AMI.
+	* core: Don't attempt to write to a stream that does not exist.
 
-	  When using the Bridge AMI action on the same channel multiple times
-	  it was possible for the channel to return to the wrong location in
-	  the dialplan if the other party hung up. This happened because the
-	  priority of the channel was not preserved across each action
-	  invocation and it would fail to move on to the next priority in
-	  other cases.
+	  When a frame is provided to ast_write ensure that a multistream
+	  capable channel has a stream for it before attempting to give it
+	  to the channel driver. In some cases (such as a deferred SDP
+	  negotiation) the stream may not yet exist.
 
-	  This change makes it so that the priority of a channel is preserved
-	  when taking control of it from another thread and it is incremented
-	  as appropriate such that the priority reflects where the channel
-	  should next be executed in the dialplan, not where it may or may not
-	  currently be.
+	  ASTERISK-27364
 
-	  The Bridge AMI action was also changed to ensure that it too
-	  starts the channels at the next location in the dialplan.
+	  Change-Id: Icf84ca982a67cdd6e9a71851eb7eb1bd0e865276
 
-	  ASTERISK-24529
+2017-11-02 01:57 +0000 [606ae3484a]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I52406669cf64208aef7252a65b63ade31fbf7a5a
+	* Add missing menuselect dependencies.
 
-2017-05-01 13:04 +0000 [bbe90d6aed]  Kevin Harwell <kharwell@digium.com>
+	  This adds menuselect dependencies for modules that use symbols of other
+	  modules.
 
-	* res_rtp_asterisk: Clearing the remote RTCP address causes RTCP failures
+	  ASTERISK-27390
 
-	  When a call gets put on hold RTP is temporarily stopped and Asterisk was
-	  setting the remote RTCP address to NULL. Then when RTCP data was received
-	  from the remote endpoint, Asterisk would be missing this information when
-	  publishing the rtcp_message stasis event. Consequently, message subscribers
-	  (in this case res_hep_rtcp) trying to parse the "from" field output the
-	  following error:
+	  Change-Id: Ia2d2849f5b87a72af7324a82edc3f283eafb5385
 
-	  "ast_sockaddr_split_hostport: Port missing in (null)"
+2017-11-01 22:57 +0000 [b616b7e4a9]  Corey Farrell <git@cfware.com>
 
-	  This patch makes it so the remote RTCP address is no longer set to NULL when
-	  stopping RTP. There was only one place that appeared to check if the remote
-	  RTCP address was NULL as a way to tell if RTCP was running. This patch added
-	  an additional check on the RTCP schedid for that case to make sure RTCP was
-	  truly not running.
+	* res/ari/resource_bridges.h: Update from 'make ari-stubs'.
 
-	  ASTERISK-26860 #close
+	  A comment was updated when I ran 'make ari-stubs'.
 
-	  Change-Id: I6be200fb20db647e48b5138ea4b81dfa7962974b
+	  Change-Id: Ib5154ae3ad72aff53374c28ead540fe349c42175
 
-2017-05-02 11:34 +0000 [526a0081a0]  Sean Bright <sean.bright@gmail.com>
+2017-11-01 19:46 +0000 [79f111e1f3]  Corey Farrell <git@cfware.com>
 
-	* cleanup: Change severity of fread short-read warning
+	* Prevent unload of modules which implement an Optional API.
 
-	  Many sound files don't have a full frame's worth of data at EOF, so the
-	  warning messages were a bit too noisy. So we demote them to debug
-	  messages.
+	  Once an Optional API module is loaded it should stay loaded.  Unloading
+	  an optional API module runs the risk of a crash if something else is
+	  using it.  This patch causes all optional API providers to tell the
+	  module loader not to unload except at shutdown.
 
-	  Change-Id: I6b617467d687658adca39170a81797a11cc766f6
+	  ASTERISK-27389
 
-2017-04-26 07:58 +0000 [23db04ed93]  Thierry Magnien <thierry.magnien@gmail.com>
+	  Change-Id: Ia07786fe655681aec49cc8d3d96e06483b11f5e6
 
-	* channels/chan_sip.c: use binding IP address for outgoing TCP SIP connections
+2017-10-30 17:30 +0000 [b9f457eac0]  Corey Farrell <git@cfware.com>
 
-	  For outgoing TCP connections, Asterisk uses the first IP address of the
-	  interface instead of the IP address we asked him to bind to.
+	* Modules: Additional improvements to CLI completion.
 
-	  ASTERISK-26922 #close
-	  Reported-by: Ksenia
+	  Replace 'needsreload' argument with a 'type' argument to specify which
+	  type of modules you want completion.  This provides more accurate CLI
+	  completion for load and unload commands.
 
-	  Change-Id: I43c71ca89211dbf1838e5bcdb9be8d06d98e54eb
+	  * 'module unload' now excludes modules that have active references or are
+	    not running.
+	  * 'module load' now excludes modules that are already running.
+	  * 'core set debug [atleast] <level> [module]' shows running modules only.
 
-2017-04-29 16:18 +0000 [02234e920c]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27378
 
-	* rtp_engine.c: Fix deadlock potential copying RTP payload maps.
+	  Change-Id: Iea3e00054461484196c46f688f02635cc886bad1
 
-	  There is a theoretical potential to deadlock in
-	  ast_rtp_codecs_payloads_copy() because it locks two different
-	  ast_rtp_codecs locks.  It is theoretical because the callers of the
-	  function are either copying between a local ast_rtp_codecs struct and a
-	  RTP instance of the ast_rtp_codecs struct.  Or they are copying between
-	  the caller and callee channel RTP instances before initiating the call to
-	  the callee.  Neither of these situations could actually result in a
-	  deadlock because there cannot be another thread involved at the time.
+2017-11-01 13:58 +0000 [1bfd1cf640]  Sean Bright <sean.bright@gmail.com>
 
-	  * Add deadlock avoidance code to ast_rtp_codecs_payloads_copy() since it
-	  locks two ast_rtp_codecs locks to perform a copy.
+	* pjsip_message_filter: Only do interface lookup for wildcard addresses.
 
-	  This only affects v13 since this deadlock avoidance code is already in
-	  newer branches.
+	  Change-Id: Ie083987e69dc43b6861671c218cacacc11b2072f
 
-	  Change-Id: I1aa0b168f94049bd59bbd74a85bd1e78718f09e5
+2017-10-31 15:08 +0000 [1e70011710]  Kevin Harwell <kharwell@digium.com>
 
-2017-04-29 16:11 +0000 [9d5df48968]  Richard Mudgett <rmudgett@digium.com>
+	* features: Bridge application's BRIDGERESULT not appropriately set
 
-	* res_pjsip_t38.c: Fix deadlock in T.38 framehook.
+	  The dialplan application "Bridge" was not setting the BRIDGERESULT to failure
+	  when a failure did occur. Even worse if it did fail to join the bridge it would
+	  still report success.
 
-	  A deadlock can happen between a channel lock and a pjsip session media
-	  container lock.  One thread is processing a reINVITE's SDP and walking
-	  through the session's media container when it waits for the channel lock
-	  to put the determined format capabilities onto the channel.  The other
-	  thread is writing a frame to the channel and processing the T.38 frame
-	  hook.  The T.38 frame hook then waits for the pjsip session's media
-	  container lock.  The two threads are now deadlocked.
+	  This patch now sets the BRIDGERESULT variable to an appropriate value for a
+	  given condition state. Also, removed the value INCOMPATIBLE as a valid result
+	  type since it is no longer used.
 
-	  * Made the T.38 frame hook release the channel lock before searching the
-	  session's media container.  This fix has been done to several other
-	  frame hooks to fix deadlocks.
+	  ASTERISK-27369 #close
 
-	  ASTERISK-26974 #close
+	  Change-Id: I22588e7125a765edf35cff28c98ca143e9927554
 
-	  Change-Id: Ie984a76ce00bef6ec9aa239010e51e8dd74c8186
+2017-10-31 13:18 +0000 [f2175c5a39]  Corey Farrell <git@cfware.com>
 
-2017-04-28 10:56 +0000 [623832b94e]  gtjoseph <gjoseph@digium.com>
+	* res_ari_channels: Fix reference leak in channel_state_invalid.
 
-	* res_pjsip_outbound_authenticator_digest: Add context to log messages
+	  channel_state_invalid leaked a reference to the channel snapshot any
+	  time it was aquired.
 
-	  There was no context info in this module's log messages so it was
-	  impossible to toubleshoot.
+	  ASTERISK-27067 #close
 
-	  Added endpoint or host to all messages and added the realms in the
-	  challenge for the "No auth credentials for any realm" message.
+	  Change-Id: I8c653f00416b39978513c5605c4be0f03b1df29a
 
-	  Change-Id: Ifeed2786f35fbea7d141237ae15625e472acff9b
+2017-10-25 17:31 +0000 [4c535f5c30]  Joshua Colp <jcolp@digium.com>
 
-2017-04-27 08:02 +0000 [c5b9ed20fd]  gtjoseph <gjoseph@digium.com>
+	* core / pjsip: Add support for grouping streams together.
 
-	* res_pjsip_session:  Add cleanup to ast_sip_session_terminate
+	  In WebRTC streams (or media tracks in their world) can be grouped
+	  together using the mslabel. This informs the browser that each
+	  should be synchronized with each other.
 
-	  If you use ast_request to create a PJSIP channel but then hang it
-	  up without causing a transaction to be sent, the session will
-	  never be destroyed.  This is due ot the fact that it's pjproject
-	  that triggers the session cleanup when the transaction ends.
-	  app_chanisavail was doing this to get more granular channel state
-	  and it's also possible for this to happen via ARI.
+	  This change extends the stream API so this information can
+	  be stored with streams. The PJSIP support has been extended
+	  to use the mslabel to determine grouped streams and store
+	  this association on the streams. Finally when creating the
+	  SDP the group information is used to cause each media stream
+	  to use the same mslabel.
 
-	  * ast_sip_session_terminate was modified to explicitly call the
-	    cleanup tasks and unreference session if the invite state is NULL
-	    AND invite_tsx is NULL (meaning we never sent a transaction).
+	  ASTERISK-27379
 
-	  * chan_pjsip/hangup was modified to bump session before it calls
-	    ast_sip_session_terminate to insure that session stays valid
-	    while it does its own cleanup.
+	  Change-Id: Id6299aa031efe46254edbdc7973c534d54d641ad
 
-	  * Added test events to session_destructor for a future testsuite
-	    test.
+2017-10-30 09:20 +0000 [022de525be]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  ASTERISK-26908 #close
-	  Reported-by: Richard Mudgett
+	* ast_coredumper: allow setting asterisk binary explicitly
 
-	  Change-Id: I52daf6f757184e5544c261f64f6fe9602c4680a9
+	  Adds an extra option, --asterisk-bin=<path> to ast_coredumper. If
+	  provided, the binary given to gdb will be the parameter, rather than
+	  asterisk from the PATH.
 
-2017-04-26 14:20 +0000 [c853cfdc7c]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-27380 #close
 
-	* res_pjsip/res_pjsip_callerid: NULL check on caller id name string
+	  Change-Id: I25f5b91eb75059b0fb2f142e468c26b283b0a9f3
 
-	  It's possible for a name in a party id structure to be marked as valid, but the
-	  name string itself be NULL (for instance this is possible to do by using the
-	  dialplan CALLERID function). There were a couple of places where the name was
-	  validated, but the string itself was not checked before passing it to functions
-	  like 'strlen'. This of course caused a crashed.
+2017-10-25 01:10 +0000 [3052b56423]  Florian Floimair <f.floimair@commend.com>
 
-	  This patch adds in a NULL check before attempting to pass it into a function
-	  that is not NULL tolerant.
+	* alembic: Add bundle column in ps_endpoints table
 
-	  ASTERISK-25823 #close
+	  The ps_endpoints table was missing the bundle column
+	  introduced with the bundle feature in
+	  commit 065c3005ad92.
 
-	  Change-Id: Iaa6ffe9d92f598fe9e3c8ae373fadbe3dfbf1d4a
+	  ASTERISK-27374 #close
 
-2017-04-25 11:43 +0000 [4d3b4fbf22]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: Ic900f4f2c20f64b99ea898d50f5c0a7117472d46
 
-	* vector: defaults and indexes
+2017-10-30 00:32 +0000 [e82b921c35]  Corey Farrell <git@cfware.com>
 
-	  Added an pre-defined integer vector declaration. This makes integer vectors
-	  easier to declare and pass around. Also, added the ability to default a vector
-	  up to a given size with a default value. Lastly, added functionality that
-	  returns the "nth" index of a matching value.
+	* Modules: Fix issues with CLI completion.
 
-	  Also, updated a unit test to test these changes.
+	  * Stop using ast_module_helper to check if a module is loaded, use
+	    ast_module_check instead (app_confbridge and app_meetme).
+	  * Stop ast_module_helper from listing reload classes when needsreload
+	    was not requested.
 
-	  Change-Id: Iaf4b51b2540eda57cb43f67aa59cf1d96cdbcaa5
+	  ASTERISK-27378
 
-2017-04-20 02:13 +0000 [566ad7c35d]  Jean Aunis <jean.aunis@prescom.fr>
+	  Change-Id: Iaed8c1e4fcbeb242921dbac7929a0fe75ff4b239
 
-	* chan_sip: Trigger reinvite if the SDP answer is included in the SIP ACK
+2017-10-28 19:18 +0000 [9bad4c74cc]  Igor Goncharovskiy <igorg@iqtek.ru>
 
-	  Some equipments may send a re-INVITE containing an SDP in the final ACK
-	  request. If this happens in the context of direct media, the remote end
-	  should be updated with a re-INVITE.
-	  This patch queues an "update RTP peer" frame to trigger the re-INVITE,
-	  instead of the "source change" frame wich was used previously.
+	* app_agent_spool: Fix typo in dtmf features usage desctiption
 
-	  ASTERISK-26951
+	  Fix typo, that specify usage wrong option 'dtmf-features' for CHANNEL() function
+	  instead of correct 'dtmf_features'
 
-	  Change-Id: I3644d2025f20e086ea9f8f62b486172c52b5b2e6
+	  ASTERISK-27377 #close
 
-2017-04-26 08:45 +0000 [001dc2ade6]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I15ecc829c1035b359584673e12cdb5c9291ac930
 
-	* pjproject_bundled:  Add --disable-libwebrtc to configure
+2017-10-27 13:41 +0000 [0991874430]  Corey Farrell <git@cfware.com>
 
-	  Without the disable, pjproject tries to build it's internal
-	  webrtc implementation which requires sse2.  This fails on
-	  platforms without sse2.
+	* res_pjsip_pubsub: Resolve potential crash in allocate_subscription.
 
-	  ASTERISK-26930 #close
-	  Reported-by: abelbeck
+	  When allocate_subscription fails to initialize fields of the new sub it
+	  calls destroy_subscription.
 
-	  Change-Id: I07231f9160c35cfa42b194d3aad4e7d51fd9a410
+	  Change-Id: I5b79c915ec216dc00c13c1e4172137864a4bec85
 
-2017-04-26 05:38 +0000 [ae696132a2]  Joshua Colp <jcolp@digium.com>
+2017-10-26 12:18 +0000 [26607e4e3b]  Richard Mudgett <rmudgett@digium.com>
 
-	* frame: Better handle interpolated frames.
+	* app_voicemail.c: Fix compiler warning with IMAP build.
 
-	  Interpolated frames are frames which contain a number of
-	  samples but have no actual data. Audiohooks did not
-	  handle this case when translating an incoming frame into
-	  signed linear. It assumed that a frame would always contain
-	  media when it may not. If this occurs audiohooks will now
-	  immediately return and not act on the frame.
+	  ASTERISK-27181
 
-	  As well for users of ast_trans_frameout the function has
-	  been changed to be a bit more sane and ensure that the data
-	  pointer on a frame is set to NULL if no data is actually
-	  on the frame. This allows the various spots in Asterisk that
-	  check for an interpolated frame based on the presence of a
-	  data pointer to work as expected.
+	  Change-Id: Ic4468b49860bd7f67e922baf4c9e96828c184d17
 
-	  ASTERISK-26926
+2017-10-25 14:38 +0000 [2ca3dbb197]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I7fa22f631fa28d540722ed789ce28e84c7f8662b
+	* codec.c: Defensively check the returned samples.
 
-2017-04-21 12:04 +0000 [1b50df78d0]  Sean Bright <sean.bright@gmail.com>
+	  Earlier versions of the codec_opus samples_count callback can return
+	  negative error values on undecodable frames.  This resulted in a divide by
+	  zero exception.
 
-	* cleanup: Fix fread() and fwrite() error handling
+	  * Added a defensive check in ast_codec_samples_count() for a "negative"
+	  samples count return value.  Log the event and set the count to zero.
 
-	  Cleaned up some of the incorrect uses of fread() and fwrite(), mostly in
-	  the format modules. Neither of these functions will ever return a value
-	  less than 0, which we were checking for in some cases.
+	  ASTERISK-27194
 
-	  I've introduced a fair amount of duplication in the format modules, but
-	  I plan to change how format modules work internally in a subsequent
-	  patch set, so this is simply a stop-gap.
+	  Change-Id: Icf69350307ecbbc80a3d74de46af9bd80ea17819
 
-	  Change-Id: I8ca1cd47c20b2c0b72088bd13b9046f6977aa872
+2017-10-24 10:33 +0000 [9e1fbab382]  Joshua Colp <jcolp@digium.com>
 
-2017-04-25 07:52 +0000 [c09b9dba90]  Joshua Colp <jcolp@digium.com>
+	* res_pjsip: Add 'ip' as a valid option to 'identify_by' on endpoint.
 
-	* alembic: Add table for 'resource_list' PJSIP RLS type.
+	  When the identify_by option on an endpoint is set to ip it will
+	  only be identified using the res_pjsip_endpoint_identifier_ip module.
+	  This ensures that it is not mistakenly matched using the username of
+	  the From header. To ensure behavior has not changed the default has
+	  been changed to "username,ip" for the identify_by option.
 
-	  This change adds an Alembic migration which adds a
-	  ps_resource_list table that can contain resource_list
-	  RLS configuration objects.
+	  ASTERISK-27206
 
-	  ASTERISK-26929
+	  Change-Id: I2170b86a7f7e221b4f00bf14aa1ef1ac5b050bbd
 
-	  Change-Id: I7c888fafc67b3e87012de974f71ca7a5b8b1ec05
+2017-10-25 12:26 +0000 [4aec70690d]  George Joseph <gjoseph@digium.com>
 
-2017-04-24 13:16 +0000 [1b88a3a4cf]  Sean Bright <sean.bright@gmail.com>
+	* ast_coredumper:  Add gzipping of binaries and display of signal info
 
-	* res_hep: Add additional config initialization and validation
+	  The --tarball-coredump option now creates a gzipped tarball of
+	  coredumps processed, their results txt files and copies of
+	  /etc/os-release, /usr/sbin/asterisk, /usr/lib(64)/libasterisk* and
+	  /usr/lib(64)/asterisk as those files are needed to properly examine
+	  the coredump.  The file will be named
+	  /tmp/asterisk.<timestamp>.coredumps.tar.gz or
+	  /tmp/asterisk-<uniqueid>.coredumps.tar.gz if --tarball-uniqueid was
+	  specified.
 
-	  * Initialize hepv3_runtime_data.sockfd to -1 so that our ao2 destructor
-	    does not close fd 0
+	  Added dumps of *_siginfo to the top of the txt files so you can
+	  tell what signal was invoked.
 
-	  * Add logging output when the required option - capture_address - is not
-	    specified.
+	  Change-Id: Ib9ee6d83592d4b1bc90cb3419a05376a88d1ded9
 
-	  * Remove a no longer relevant #define and correct related documentation
+2017-10-25 09:23 +0000 [3821be1c68]  Ben Ford <bford@digium.com>
 
-	  * Pass appropriate flags to aco_option_register so that capture_address
-	    cannot be the empty string.
+	* http.c: Fix http header send content.
 
-	  ASTERISK-26953 #close
+	  Currently ast_http_send barricades a portion of the content that
+	  needs to be sent in order to establish a connection for things
+	  like the ARI client. The conditional and contents have been changed
+	  to ensure that everything that needs to be sent, will be sent.
 
-	  Change-Id: Ief08441bc6596d6f1718fa810e54a5048124f076
+	  ASTERISK-27372
 
-2017-04-17 19:06 +0000 [cea3742c54]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I8816d2d8f80f4fefc6dcae4b5fdfc97f1e46496d
 
-	* core: Use eventfd for alert pipes on Linux when possible
+2017-03-30 09:51 +0000 [5553adb8ba]  Corey Farrell <git@cfware.com>
 
-	  The primary win of switching to eventfd when possible is that it only
-	  uses a single file descriptor while pipe() will use two. This means for
-	  each bridge channel we're reducing the number of required file
-	  descriptors by 1, and - if you're using timerfd - we also now have 1
-	  less file descriptor per Asterisk channel.
+	* Build System: Fix --disable-xmldoc option.
 
-	  The API is not ideal (passing int arrays), but this is the cleanest
-	  approach I could come up with to maintain API/ABI.
+	  The configure option to disable XML documentation does not currently
+	  work.  This patch makes it effective, but also causes an ABI change by
+	  removing the ast_xmldoc_* symbols.  Disabling xmldoc also prevents docs
+	  from being automatically generated, but they can still be manually
+	  generated with 'make doc/core-en_US.xml'.
 
-	  I've also removed what I believe to be an erroneous code block that
-	  checked the non-blocking flag on the pipe ends for each read. If the
-	  file descriptor is 'losing' its non-blocking mode, it is because of a
-	  bug somewhere else in our code.
+	  ASTERISK-26639
 
-	  In my testing I haven't seen any measurable difference in performance.
+	  Change-Id: Ifac562340c09f80c83e0203de098fcac93bf8c44
 
-	  Change-Id: Iff0fb1573e7f7a187d5211ddc60aa8f3da3edb1d
+2017-10-23 00:55 +0000 [569e9a8391]  Corey Farrell <git@cfware.com>
 
-2017-04-21 12:33 +0000 [1213ac1ac5]  Richard Mudgett <rmudgett@digium.com>
+	* Single API for ast_store_lock_info and ast_remove_lock_info.
 
-	* res_pjsip_session.c: Send 100 Trying out earlier to prevent retransmissions.
+	  This makes the 'bt' parameter unconditional for ast_store_lock_info and
+	  ast_remove_lock_info.  The 'bt' parameter is unused when HAVE_BKTR is
+	  undefined.
 
-	  If ICE is enabled and a STUN server does not respond then we will block
-	  until we give up on the STUN response.  This will take nine seconds.  In
-	  the mean time the peer that sent the INVITE will send retransmissions.
+	  Change-Id: Ieced0e920928b735a39c3b5952b806c473d67453
 
-	  * Restructure res_pjsip_session.c:new_invite() to send a 100 Trying out
-	  earlier to prevent these retransmissions.
+2017-10-24 09:43 +0000 [6474de5f72]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26890
+	* chan_sip: Fix SUBSCRIBE with missing "Expires" header.
 
-	  Change-Id: Ie3fc611e53a0eff6586ad55e4aacad81cf6319a8
+	  When chan_sip receives a SUBSCRIBE request with no "Expires" header it
+	  processes the request as an unsubscribe.  This is incorrect, per RFC3264
+	  when the "Expires" header is missing a default expiry should be used.
 
-2017-04-21 12:07 +0000 [80fd7fd908]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-18140
 
-	* res_pjsip_session.c: Restructure ast_sip_session_alloc()
+	  Change-Id: Ibf6dcd4fdd07a32c2bc38be1dd557981f08188b5
 
-	  * Restructure ast_sip_session_alloc() to need less cleanup on off nominal
-	  error paths.
+2017-10-24 07:24 +0000 [7126520b3e]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Made ast_sip_session_alloc() and ast_sip_session_create_outgoing() avoid
-	  unnecessary ref manipulation to return a session.  This is faster than
-	  calling a function.  That function may do logging of the ref changes with
-	  REF_DEBUG enabled.
+	* lpc10: Avoid compiler warning when DONT_OPTIMIZE/COMPILE_DOUBLE.
 
-	  Change-Id: I2a0affc4be51013d3f0485782c96b8fee3ddb00a
+	  ASTERISK-23556
+	  Reported by: Marcello Ceschia
 
-2017-04-19 15:08 +0000 [98e38daf82]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: Ic27e88e0336a0d83877dc857938659dc5560b93c
 
-	* pbx: Use same thread if AST_OUTGOING_WAIT_COMPLETE specified
+2017-10-07 12:14 +0000 [841ac3ded6]  Corey Farrell <git@cfware.com>
 
-	  Both ast_pbx_outgoing_app() and ast_pbx_outgoing_exten() cause the core
-	  to spawn a new thread to perform the dial. When AST_OUTGOING_WAIT_COMPLETE
-	  is passed to these functions, the calling thread will be blocked until
-	  the newly created channel has been hung up.
+	* hashtab: Use ast_free.
 
-	  After this patch, we run the dial on the current thread rather than
-	  spawning a new one. The only in-tree code that passes
-	  AST_OUTGOING_WAIT_COMPLETE is pbx_spool, so you should see reduced
-	  thread usage if you are using .call files.
+	  A few places in hashtab use free instead of ast_free, remove declaration
+	  of ASTMM_LIBC from hashtab.c as it's no longer needed.
 
-	  Change-Id: I512735d243f0a9da2bcc128f7a96dece71f2d913
+	  Change-Id: I2ff089bad71640c03c3ce97f1b00fc962ef79427
 
-2017-04-19 13:23 +0000 [55f452884f]  Richard Mudgett <rmudgett@digium.com>
+2017-10-23 01:02 +0000 [fb585cf185]  Corey Farrell <git@cfware.com>
 
-	* res_rtp_asterisk.c: Fix crash in RTCP DTLS operation.
+	* Bundled pjproject: Enable pj_assert when dev-mode is enabled.
 
-	  Occasionally a crash happens when processing the RTCP DTLS timeout
-	  handler.  The RTCP DTLS timeout timer could be left running if we have not
-	  completed the DTLS handshake before we place the call on hold or we
-	  attempt direct media.
+	  ASTERISK-27359
 
-	  * Made ast_rtp_prop_set() stop the RTCP DTLS timer when disabling RTCP.
+	  Change-Id: Ib01fb6c01f9bb87129374a51cb9318c474147517
 
-	  * Made some sanity tweaks to ast_rtp_prop_set() when switching from
-	  standard RTCP mode to RTCP multiplexed mode.
+2017-10-23 13:44 +0000 [ee21076151]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26692 #close
+	* main/Makefile: Remove rule for non-existant testexpr2.
 
-	  Change-Id: If6c64c79129961acfa4b3d63a864e8f6b664acc0
+	  Change-Id: Ibb3e47f27a395d74d8c5263db015b05434f5969b
 
-2017-03-22 16:05 +0000 [f856cfbb51]  Richard Mudgett <rmudgett@digium.com>
+2017-10-23 12:42 +0000 [a9e9608982]  Corey Farrell <git@cfware.com>
 
-	* rtp_engine/res_rtp_asterisk: Fix RTP struct reentrancy crashes.
+	* test_config: Fix failure and segfault when config_hook is run twice.
 
-	  The struct ast_rtp_instance has historically been indirectly protected
-	  from reentrancy issues by the channel lock because early channel drivers
-	  held the lock for really long times.  Holding the channel lock for such a
-	  long time has caused many deadlock problems in the past.  Along comes
-	  chan_pjsip/res_pjsip which doesn't necessarily hold the channel lock
-	  because sometimes there may not be an associated channel created yet or
-	  the channel pointer isn't available.
+	  On second run the config_hook test was unexpectedly failing to load
+	  test_config.conf because it was still unmodified since the last load.
+	  This is fixed by not passing CONFIG_FLAG_FILEUNCHANGED for the initial
+	  loads, only using it when we are tested that a reload of unmodified
+	  files do not initiate the hook.
 
-	  In the case of ASTERISK-26835 a pjsip serializer thread was processing a
-	  message's SDP body while another thread was reading a RTP packet from the
-	  socket.  Both threads wound up changing the rtp->rtcp->local_addr_str
-	  string and interfering with each other.  The classic reentrancy problem
-	  resulted in a crash.
+	  ASTERISK-25960
 
-	  In the case of ASTERISK-26853 a pjsip serializer thread was processing a
-	  message's SDP body while another thread was reading a RTP packet from the
-	  socket.  Both threads wound up processing ICE candidates in PJPROJECT and
-	  interfering with each other.  The classic reentrancy problem resulted in a
-	  crash.
+	  Change-Id: Ifd679509a23ed163e5cc647490bf7df4ae3cd856
 
-	  * rtp_engine.c: Make the ast_rtp_instance_xxx() calls lock the RTP
-	  instance struct.
+2017-10-23 12:23 +0000 [6f0431798e]  George Joseph <gjoseph@digium.com>
 
-	  * rtp_engine.c: Make ICE and DTLS wrapper functions to lock the RTP
-	  instance struct for the API call.
+	* res_pjsip_sdp_rtp:  Fix setting of address type for rtp_ipv6
 
-	  * res_rtp_asterisk.c: Lock the RTP instance to prevent a reentrancy
-	  problem with rtp->rtcp->local_addr_str in the scheduler thread running
-	  ast_rtcp_write().
+	  create_outgoing_sdp_stream was setting "addr_type = STR_IP6" only
+	  when an ipv6 media_address was specified on the endpoint.  If
+	  rtp_ipv6 was set and ast_sip_get_host_ip_string returned an ipv6
+	  address, we were leaving the addr_type set at the default of
+	  STR_IP4.  This caused the address type to be set incorrectly on the
+	  "o" and "c" SDP attributes even though the address was set
+	  correctly.  Some clients don't like the mismatch.
 
-	  * res_rtp_asterisk.c: Avoid deadlock when local RTP bridging in
-	  bridge_p2p_rtp_write() because there are two RTP instance structs
-	  involved.
+	   * Removed the test for endpoint/media_address and now check all
+	     addresses for ipv6.
 
-	  * res_rtp_asterisk.c: Avoid deadlock when trying to stop scheduler
-	  callbacks.  We cannot hold the instance lock when trying to stop a
-	  scheduler callback.
+	  ASTERISK-27198
+	  Reported by: Martin Cisárik
 
-	  * res_rtp_asterisk.c: Remove the lock in struct dtls_details and use the
-	  struct ast_rtp_instance ao2 object lock instead.  The lock was used to
-	  synchronize two threads to prevent a race condition between starting and
-	  stopping a timeout timer.  The race condition is no longer present between
-	  dtls_perform_handshake() and __rtp_recvfrom() because the instance lock
-	  prevents these functions from overlapping each other with regards to the
-	  timeout timer.
+	  Change-Id: I5214fc31b728117842243807e7927a319cf77592
 
-	  * res_rtp_asterisk.c: Remove the lock in struct ast_rtp and use the struct
-	  ast_rtp_instance ao2 object lock instead.  The lock was used to
-	  synchronize two threads using a condition signal to know when TURN
-	  negotiations complete.
+2017-10-23 07:53 +0000 [488f98310f]  Richard Mudgett <rmudgett@digium.com>
 
-	  * res_rtp_asterisk.c: Avoid deadlock when trying to stop the TURN
-	  ioqueue_worker_thread().  We cannot hold the instance lock when trying to
-	  create or shut down the worker thread without a risk of deadlock.
+	* app_agent_pool.c: Fix online documentation typo.
 
-	  This patch exposed a race condition between a PJSIP serializer thread
-	  setting up an ICE session in ice_create() and another thread reading RTP
-	  packets.
+	  Change-Id: Ib0bc95fd0ec288c78c313823254d7a84ebfc4429
 
-	  * res_rtp_asterisk.c:ice_create(): Set the new rtp->ice pointer after we
-	  have re-locked the RTP instance to prevent the other thread from trying to
-	  process ICE packets on an incomplete ICE session setup.
+2017-10-22 17:32 +0000 [252353e0a9]  Joshua Colp <jcolp@digium.com>
 
-	  A similar race condition is between a PJSIP serializer thread resetting up
-	  an ICE session in ice_create() and the timer_worker_thread() processing
-	  the completion of the previous ICE session.
+	* res_xmpp: Ensure the connection filter is available.
 
-	  * res_rtp_asterisk.c:ast_rtp_on_ice_complete(): Protect against an
-	  uninitialized/null remote_address after calling
-	  update_address_with_ice_candidate().
+	  Users of the API that res_xmpp provides expect that a
+	  filter be available on the client at all times. When
+	  OAuth authentication support was added this requirement
+	  was not maintained.
 
-	  * res_rtp_asterisk.c: Eliminate the chance of ice_reset_session()
-	  destroying and setting the rtp->ice pointer to NULL while other threads
-	  are using it by adding an ao2 wrapper around the PJPROJECT ice pointer.
-	  Now when we have to unlock the RTP instance object to call a PJPROJECT ICE
-	  function we will hold a ref to the wrapper.  Also added some rtp->ice NULL
-	  checks after we relock the RTP instance and have to do something with the
-	  ICE structure.
+	  This change merely moves the OAuth authentication to
+	  after the filter is created, ensuring users of res_xmpp
+	  can add things to the filter as needed.
 
-	  ASTERISK-26835 #close
-	  ASTERISK-26853 #close
+	  ASTERISK-27346
 
-	  Change-Id: I780b39ec935dcefcce880d50c1a7261744f1d1b4
+	  Change-Id: I4ac474afe220e833288ff574e32e2b9a23394886
 
-2017-04-19 08:39 +0000 [dafcd97a77]  Sean Bright <sean.bright@gmail.com>
+2017-10-21 03:44 +0000 [840e08716b]  Alexander Traud <pabstraud@compuserve.com>
 
-	* build: Update config.guess and config.sub
+	* chan_sip: Crypto attribute not last but first on SDP media level.
 
-	  Change-Id: Id078a1df07a771808775e1053cdfe1d99c8fb172
+	  This matches the behavior of the other SIP channel driver, chan_pjsip.
 
-2017-04-14 13:52 +0000 [9bbfa6fda1]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27365
 
-	* format_wav: Read 16khz wav samples properly
+	  Change-Id: I8f23a51290a58b75816da2999ed1965441dfc5d6
 
-	  When opening a PCM wave file for reading, we aren't tracking the
-	  frequency of the opened file, so we treat 16khz files as 8khz and do
-	  half reads.
+2017-10-17 10:53 +0000 [e41561fc2a]  Richard Mudgett <rmudgett@digium.com>
 
-	  This patch also cleans up some of the data types and an unnecessarily
-	  complex `if` expression.
+	* res_pjproject.c: Upgrade bundled PJPROJECT to 2.7
 
-	  ASTERISK-26613 #close
-	  Reported by: Vitaly K
+	  Update patches included in bundled PJPROJECT for the new version.
 
-	  Change-Id: I05f8b263058dc573ea8ffe0c62e7964506e11815
+	  ASTERISK-27355
 
-2017-04-16 19:54 +0000 [4ccaffe644]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I9ac5dbbffaadca25ad24fac8b9ab615e5ace6083
 
-	* make ari-stubs so doc periodic jobs can run
+2017-10-16 16:46 +0000 [4559cd0e28]  Nir Simionovich <nirs@greenfieldtech.net>
 
-	  The periodic doc job does a make ari-stubs and checks that
-	  there are no changes before generating the docs.  Since I changed
-	  the mustache template (and the generated code directly) recently
-	  and forgot to regenerate the stubs, the doc job thinks they're out
-	  of date.
+	* This patch adds a beanstalk CDR backend.
 
-	  Change-Id: Ibd4bc649556615ff714d44534c45b6c2f6aa449d
+	  Beanstalkd is a simple to use job queue. It provides a means to
+	  create multiple job queues called "tubes". Each tube can store
+	  multiple jobs, with varying priorities with the queue. Queue
+	  processing is available via a simple TCP socket or via well defined
+	  libraries, avaialble at
+	  https://github.com/kr/beanstalkd/wiki/client-libraries
 
-2017-04-14 12:51 +0000 [90c630aaa1]  Sean Bright <sean.bright@gmail.com>
+	  This module is based upon the beanstalk-client library, available
+	  for download at: https://github.com/deepfryed/beanstalk-client
 
-	* format_ogg_vorbis: Clear ogg/vorbis data structures on close
+	  Change-Id: I5fe4089a34ab3b39230786d9bbfddafa56715f48
 
-	  On filestream close, we need to clear out the ogg & vorbis data
-	  structures to prevent a memory leak.
+2017-10-18 13:41 +0000 [4760b2445c]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26169 #close
-	  Reported by: Ivan Myalkin
+	* res_pjsip_pubsub: Prevent unload except during shutdown.
 
-	  Change-Id: Iee94c5a5d5bdafbf8b181c5c064d15d90ace8274
+	  Prevent unload of the module as certain pjsip initialization functions
+	  cannot be reversed.  This required a reorder of the module_load so that
+	  the non-reversable pjsip functions are not called until all potential
+	  errors have been ruled out.
 
-2017-04-14 17:31 +0000 [9084c85cb1]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24483
 
-	* Revert "bridging:  Ensure successful T.38 negotation"
+	  Change-Id: Iee900f20bdd6ee1bfe23efdec0d87765eadce8a7
 
-	  This reverts commit 3e7c396a51b240088c475dd53e7bac9869376129.
+2017-10-18 13:37 +0000 [449ee66a11]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I61d49d563babff788bb557345729b200d116bd88
+	* res_pjsip_refer: Prevent unload except during shutdown.
 
-2017-04-14 16:50 +0000 [357d1fbdcc]  Sean Bright <sean.bright@gmail.com>
+	  Prevent unload of the module as certain pjsip initialization functions
+	  cannot be reversed.
 
-	* res_stun_monitor: Don't fail to load if DNS resolution fails
+	  ASTERISK-24483
 
-	  res_stun_monitor will fail to load if DNS resolution of the STUN server
-	  fails. Instead, we continue without the STUN server being resolved and
-	  we will re-attempt the resolution on the STUN refresh interval.
+	  Change-Id: I94597ec8b8491f5af9c57bf66dbc3b078fe2d49d
 
-	  ASTERISK-21856 #close
-	  Reported by: Jeremy Kister
+2017-10-18 12:04 +0000 [c9e19b31f5]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I6334c54a1cc798f8a836b4b47948e0bb4ef59254
+	* chan_sip: Fix output of 'sip set debug off'.
 
-2017-04-14 14:36 +0000 [ac15ebc379]  Roman S.
+	  When sip.conf contains 'sipdebug=yes' it is impossible to disable it
+	  using CLI 'sip set debug off'.  This corrects the output of that CLI
+	  command to instruct the user to turn sipdebug off in the configuration
+	  file.
 
-	* format_pcm: Track actual header size of .au files
+	  ASTERISK-23462 #close
 
-	  Sun's Au file format has a minimum data offset 24 bytes, but this
-	  offset is encoded in each .au file. Instead of assuming the minimum,
-	  read the actual value and store it for later use.
+	  Change-Id: I1cceade9caa9578e1b060feb832e3495ef5ad318
 
-	  ASTERISK-20984 #close
-	  Reported by: Roman S.
-	  Patches:
-	  	asterisk-1.8.20.0-au-clicks-2.diff (license #6474) patch
-	  	uploaded by Roman S.
+2017-10-16 10:53 +0000 [955a891a84]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I524022fb19ff2fd5af2cc2d669d27a780ab2057c
+	* app_macro deprecation.
 
-2017-04-11 11:07 +0000 [f882ca2572]  gtjoseph <gjoseph@digium.com>
+	  * Mark the module deprecated.
+	  * Disable the module by default.
+	  * Produce a warning the first time a macro is used.
+	  * Note deprecation related options in app_dial and app_queue.
 
-	* modules:  change module LOAD_FAILUREs to LOAD_DECLINES
+	  ASTERISK-27350
 
-	  In all non-pbx modules, AST_MODULE_LOAD_FAILURE has been changed
-	  to AST_MODULE_LOAD_DECLINE.  This prevents asterisk from exiting
-	  if a module can't be loaded.  If the user wishes to retain the
-	  FAILURE behavior for a specific module, they can use the "require"
-	  or "preload-require" keyword in modules.conf.
+	  Change-Id: I560ea043bacdbc5534a17d97854273d52c2f1bdc
 
-	  A new API was added to logger: ast_is_logger_initialized().  This
-	  allows asterisk.c/check_init() to print to the error log once the
-	  logger subsystem is ready instead of just to stdout.  If something
-	  does fail before the logger is initialized, we now print to stderr
-	  instead of stdout.
+2017-10-18 03:30 +0000 [95b45d1c46]  Alexander Traud <pabstraud@compuserve.com>
 
-	  Change-Id: I5f4b50623d9b5a6cb7c5624a8c5c1274c13b2b25
+	* res_srtp: Add support for libsrtp2 with AES-GCM.
 
-2017-04-07 16:14 +0000 [cd80af508e]  Richard Mudgett <rmudgett@digium.com>
+	  Beside allowing AES-GCM again, this adds AES-192 again.
 
-	* res_rtp_asterisk.c: Add stun_blacklist option
+	  ASTERISK-27356
 
-	  Added the stun_blacklist option to rtp.conf.  Some multihomed servers have
-	  IP interfaces that cannot reach the STUN server specified by stunaddr.
-	  Blacklist those interface subnets from trying to send a STUN packet to
-	  find the external IP address.  Attempting to send the STUN packet
-	  needlessly delays processing incoming and outgoing SIP INVITEs because we
-	  will wait for a response that can never come until we give up on the
-	  response.  Multiple subnets may be listed.
+	  Change-Id: Ia97a435faf26300335d9552fa676b5d17e5f7233
 
-	  ASTERISK-26890 #close
+2017-10-14 14:41 +0000 [5d8c517960]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I3ff4f729e787f00c3e6e670fe6435acce38be342
+	* bridge_softmix: Reduce topology cloning and improve renegotiation.
 
-2017-04-06 17:31 +0000 [f8219a2e12]  Richard Mudgett <rmudgett@digium.com>
+	  As channels join and leave an SFU the bridge_softmix module
+	  needs to renegotiate to add and remove their streams from
+	  the other participants. Previously this was done by constructing
+	  the ideal stream topology every time but in the case of leave
+	  this was incomplete.
 
-	* stun.c: Fix ast_stun_request() erratic timeout.
+	  This change makes it so bridge_softmix keeps an ideal stream
+	  topology for each channel and uses it when making changes. This
+	  ensures that when we request a renegotiation we are always
+	  certain that we are aiming for the best stream topology
+	  possible. In the case of a channel leaving this ensures that
+	  we try to have an existing participant fill their place if
+	  a participant has a fixed limit on the maximum number of video
+	  streams they allow.
 
-	  If ast_stun_request() receives packets other than a STUN response then we
-	  could conceivably never exit if we continue to receive packets with less
-	  than three seconds between them.
+	  ASTERISK-27354
 
-	  * Fix poll timeout to keep track of the time when we sent the STUN
-	  request.  We will now send a STUN request every three seconds regardless
-	  of how many other packets we receive while waiting for a response until we
-	  have completed three STUN request transmission cycles.
+	  Change-Id: I58070f421ddeadd2844a33b869b052630cf2e514
 
-	  Change-Id: Ib606cb08585e06eb50877f67b8d3bd385a85c266
+2017-10-06 15:55 +0000 [73164d0d7f]  Richard Mudgett <rmudgett@digium.com>
 
-2017-04-06 18:30 +0000 [19b82a8644]  Richard Mudgett <rmudgett@digium.com>
+	* cdr.c: Rename the Party A CDR container.
 
-	* sorcery.c: Speed up ast_sorcery_retrieve_by_id()
+	  * Rename the Party A CDR container from active_cdrs_by_channel to
+	  active_cdrs_master.
 
-	  Return early if ast_sorcery_retrieve_by_id() is not passed an id to find.
-	  Also eliminated the RAII_VAR() usage in the function.
+	  * Renamed the support functions associated with active_cdrs_master
+	  appropriately.
 
-	  Change-Id: I871dbe162a301b5ced8b4393cec27180c7c6b218
+	  ASTERISK-27335
 
-2017-04-10 11:30 +0000 [aecf19e7d2]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I6104bb3edc3a0b7243ce502e45e8832b0cff14f7
 
-	* res_pjsip: Fix pointer use after unref.
+2017-10-02 17:42 +0000 [fe1120cf88]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I4b6e1b0070563eeaee223cb58326f1b962ed5bc1
+	* cdr.c: Add container to key off of Party B channel names.
 
-2017-04-06 18:18 +0000 [304f652cda]  Richard Mudgett <rmudgett@digium.com>
+	  The CDR performance gets worse the further it gets behind in processing
+	  stasis messages.  One of the reasons is because of a n*m loop used when
+	  processing Party B information.
 
-	* res_pjsip_sdp_rtp.c: Don't use deprecated transport struct member.
+	  * Added a new CDR container that is keyed to Party B so we don't need such
+	  a large loop when processing Party B information.
 
-	  * create_rtp(): Eliminate use of deprecated transport struct member.  That
-	  member and several others in the transport structure were deprecated
-	  because of an infinite loop created when using realtime configuration.
-	  See 2451d4e4550336197ee2e482750cc53f30afa352
+	  NOTE: To reduce the size of the patch I deferred to another patch the
+	  renaming of the Party A active_cdrs_by_channel container to
+	  active_cdrs_master and renaming the container's hash and cmp functions
+	  appropriately.
 
-	  ASTERISK-26851
+	  ASTERISK-27335
 
-	  Change-Id: I0533aa13c9ce3c6cc394e0fd2b5bf1cd1b2ef3bc
+	  Change-Id: I0bf66e8868f8adaa4b5dcf9e682e34951c350249
 
-2017-04-10 17:45 +0000 [bb8cd2add7]  Richard Mudgett <rmudgett@digium.com>
+2017-10-11 06:04 +0000 [da24d425eb]  Torrey Searle <torrey@voxbone.com>
 
-	* tcptls.c: Cleanup TCP/TLS listener thread on abnormal exit.
+	* contrib/script/sip_to_pjsip: implement 'all' for allow/disallow
 
-	  Temporarily running out of file descriptors should not terminate the
-	  listener thread.  Otherwise, when there becomes more file descriptors
-	  available, nothing is listening.
+	  when 'all' is specified in an allow or disallow section, it should erase
+	  all values from the inverse section in the default config. E.G.
+	  allow=all should erase any deny values from default config &
+	  vice-versa
 
-	  * Added EMFILE exception to abnormal thread exit.
+	  ASTERISK-27333 #close
 
-	  * Added an abnormal TCP/TLS listener exit error message.
+	  Change-Id: I99219478fb98f08751d769daaee0b7795118a5a6
 
-	  * Closed the TCP/TLS listener socket on abnormal exit so Asterisk does not
-	  appear dead if something tries to connect to the socket.
+2017-10-14 04:11 +0000 [c4f40b778a]  Guido Falsi <madpilot@freebsd.org>
 
-	  ASTERISK-26903 #close
+	* chan_dahdi: wrap include file which is not present on BSD systems in #ifdef
 
-	  Change-Id: I10f2f784065136277f271159f0925927194581b5
+	  The sys/sysmacros.h include file does not exist in BSD systems and
+	  is not required to build this module there.
+	  Since an "#if defined(__NetBSD__) || defined(__FreeBSD__)" section
+	  already exist I moved that include line inside it's #else branch.
 
-2017-04-07 08:58 +0000 [d8967ff2c0]  Torrey Searle <torrey@voxbone.com>
+	  ASTERISK-27343 #close
 
-	* strings.h:  Avoid overflows in the string hash functions
+	  Change-Id: Ibfb64f4e9a0ce8b6eda7a7695cfe57916f175dc1
 
-	  On 2's compliment machines abs(INT_MIN) behavior is undefined and
-	  results in a negative value still being returnd.  This results in
-	  negative hash codes that can result in crashes.
+2017-10-13 09:43 +0000 [8f65d91dfd]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-26528 #close
+	* res_pjsip_session: Rewrite o= with external_media_address.
 
-	  Change-Id: Idff550145ca2133792a61a2e212b4a3e82c6517b
+	  PJSIP allows a domain name as external_media_address. This allows chan_pjsip to
+	  be used behind a NAT with changing IP addresses. The IP address of that domain
+	  is resolved to the c= line already. This change sets also the o= line to that
+	  domain.
 
-2017-04-08 03:05 +0000 [bbbd262ec0]  Walter Doekes <walter+github@wjd.nu>
+	  ASTERISK-27341 #close
 
-	* samples: Undo removal of include from canonicalize-app-names commit.
+	  Change-Id: I690163b6e762042ec38b3995aa5c9bea909d8ec4
 
-	  This include was accidentally removed in changeset
-	  Ia79aea64de89531362e993e34230c2044a70aa93. My bad.
+2017-10-12 12:03 +0000 [7d51a79beb]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I1d716c7f9590b4e97909fb8bca1f2ed9bd0e4082
+	* bridge_simple: Improve renegotiation success rate.
 
-2017-04-07 08:35 +0000 [b3f4a6365e]  Joshua Colp <jcolp@digium.com>
+	  When making channels compatible the bridge_simple module
+	  will renegotiate one to better match the other. Some
+	  endpoints incorrectly terminate the call if this process
+	  fails.
 
-	* pjsip: Add Alembic for PUBLISH support.
+	  To better handle this scenario the audio streams present
+	  on the new requested topology will include any existing
+	  negotiated formats that happen to exist on the first
+	  valid audio stream. This ensures formats are persent that
+	  are known to be acceptable to the remote endpoint.
 
-	  This change adds database tables for the PUBLISH support so it
-	  can be configured using realtime. A minor fix to the
-	  res_pjsip_publish_asterisk module was done so that it read the
-	  sorcery configuration from the correct section. Finally the
-	  sample configuration files have been updated.
+	  ASTERISK-27259
 
-	  ASTERISK-26928
+	  Change-Id: I8fc0cc03e8bcfd0be8302f13b9f32d8268977f43
 
-	  Change-Id: I81991ae5c75af98d247f7eacd1c0b0a763675952
+2017-10-13 08:51 +0000 [ee65d5ac7c]  Corey Farrell <git@cfware.com>
 
-2017-04-07 08:06 +0000 [e0e5a337fd]  Alexander Traud <pabstraud@compuserve.com>
+	* ast_bt_get_symbols: Prevent double-free.
 
-	* pjproject_bundled: Crash on pj_ssl_get_info() while ioqueue_on_read_complete().
+	  It's possible for bfdobj to be created but syms not created.  If syms
+	  was not allocated in the current loop iteration but was allocated in the
+	  previous iteration it would crash.
 
-	  When the Asterisk channel driver res_pjsip offers SIP-over-TLS, sometimes, not
-	  reproducible, Asterisk crashed in pj_ssl_sock_get_info() because a NULL pointer
-	  was read. This change avoids this crash.
+	  ASTERISK-27340
 
-	  ASTERISK-26927 #close
+	  Change-Id: I5b110c609f6dfe91339f782a99a431bca5837363
 
-	  Change-Id: I24a6011b44d1426d159742ff4421cf806a52938b
+2017-10-13 08:12 +0000 [44d9446eb5]  Alexander Traud <pabstraud@compuserve.com>
 
-2017-04-05 06:41 +0000 [3e7c396a51]  Torrey Searle <torrey@voxbone.com>
+	* tcptls: NULL-check the parameter of ast_ssl_teardown before accessing it.
 
-	* bridging:  Ensure successful T.38 negotation
+	  This avoids a crash on stopping a chan_sip which failed to start its TLS server.
 
-	  When a T.38 happens immediatly after call establishment, the control
-	  frame can be lost because the other leg is not yet in the bridge.
+	  ASTERISK-27339 #close
 
-	  This patch detects this case an makes sure T.38 negotation happens
-	  when the 2nd leg is being made compatible with the negotating
-	  first leg
+	  Change-Id: I327fc70db68eaaca5b50a15c7fd687fde79263d5
 
-	  ASTERISK-26923 #close
+2017-09-29 14:26 +0000 [f369be21a8]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: If334125ee61ed63550d242fc9efe7987e37e1d94
+	* cdr.c: Eliminated many calls to ao2_global_obj_ref().
 
-2017-04-04 16:20 +0000 [4e6e069491]  gtjoseph <gjoseph@digium.com>
+	  The CDR performance gets worse the further it gets behind in processing
+	  stasis messages.  One of the reasons is we were getting the global config
+	  to determine if we needed to log a debugging message.
 
-	* pjproject_bundled: Add 3 upstream patches
+	  * Many calls to ao2_global_obj_ref() were just so we could determine if
+	  debug mode is enabled.  Made a global flag to check instead.
 
-	  0035-r5572-svn-backport-dialog-transaction-deadlock.patch
-	  0036-r5573-svn-backport-ua-pjsua-transaction-deadlock.patch
-	  0037-r5576-svn-backport-session-timer-crash.patch
+	  * Eliminated many RAII_VAR() usages associated with the remaining
+	  ao2_global_obj_ref() calls.
 
-	  Also removed the progress bar from wget download to stdout.
+	  * Added missing NULL checks for the returned ao2_global_obj_ref() value.
 
-	  ASTERISK-26905 #close
-	  Reported-by: Ross Beer
+	  ASTERISK-27335
 
-	  Change-Id: I268fb3cf71a3bb24283ff0d24bd8b03239d81256
+	  Change-Id: Iceaad93172862f610cad0188956634187bfcc7cd
 
-2017-04-05 14:50 +0000 [d2a33cdedc]  gtjoseph <gjoseph@digium.com>
+2017-10-06 13:45 +0000 [2eea087401]  Richard Mudgett <rmudgett@digium.com>
 
-	* sample_config:  Add samples for pubsub to pjsip.conf.sample
+	* cdr.c: Defer getting ao2_global_obj_ref() until needed.
 
-	  Added:
-	   * outbound-publish
-	   * resource_list
-	   * inbound-publication
-	   * asterisk-publication
+	  The CDR performance gets worse the further it gets behind in processing
+	  stasis messages.  One of the reasons is we were getting the global config
+	  even if we didn't need it.
 
-	  Change-Id: I65043a896c35483f30a92d30b5b118359af7ba5a
+	  * Most uses of the global config were only needed on off nominal code
+	  paths so it makes sense to not get it until absolutely needed.
 
-2017-04-05 09:10 +0000 [ab9d2fc86d]  Walter Doekes <walter+github@wjd.nu>
+	  ASTERISK-27335
 
-	* samples: Canonicalize app names in extensions.conf.sample.
+	  Change-Id: I00c63b7ec233e5bfffd5d976f05568613d3c2365
 
-	  This takes care of warnings by ossobv/asterisklint.
+2017-10-05 18:08 +0000 [7c7a917874]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ia79aea64de89531362e993e34230c2044a70aa93
+	* cdr.c: Set stringfields only if they are different.
 
-2017-04-03 15:38 +0000 [6906765381]  Richard Mudgett <rmudgett@digium.com>
+	  The CDR performance gets worse the further it gets behind in processing
+	  stasis messages.  One of the reasons is we were repeatedly setting string
+	  fields to potentially the same string in base_process_party_a().  Setting
+	  a string field involves allocating room for the new string out of a memory
+	  pool which may have to allocate even more memory.
 
-	* res_pjsip_sdp_rtp.c: Don't alter global addr variable.
+	  * Check to see if the string field is already set to the desired string.
 
-	  * create_rtp(): Fix unexpected alteration of global address_rtp if a
-	  transport is bound to an address.
+	  ASTERISK-27335
 
-	  * create_rtp(): Fix use of uninitialized memory if the endpoint RTP media
-	  address is invalid or the transport has an invalid address.
+	  Change-Id: I3ccb7e23f1488417e08cafe477755033eed65a7c
 
-	  ASTERISK-26851
+2017-10-05 18:03 +0000 [c80c8f2ab9]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Icde42e65164a88913cb5c2601b285eebcff397b7
+	* cdr.c: Fix setting dnid, callingsubaddr, and calledsubaddr
 
-2017-03-27 09:03 +0000 [68bde0f07d]  Corey Farrell <git@cfware.com>
+	  The string comparisons for setting these CDR variables was inverted.  We
+	  were repeatedly setting these CDR variables only if the channel snapshots
+	  had the same value.
 
-	* CDR: Protect from data overflow in ast_cdr_setuserfield.
+	  ASTERISK-27335
 
-	  ast_cdr_setuserfield wrote to a fixed length field using strcpy. This could
-	  result in a buffer overrun when called from chan_sip or func_cdr. This patch
-	  adds a maximum bytes written to the field by using ast_copy_string instead.
+	  Change-Id: I9482073524411e7ea6c03805b16de200cb1669ea
 
-	  ASTERISK-26897 #close
-	  patches:
-	    0001-CDR-Protect-from-data-overflow-in-ast_cdr_setuserfie.patch submitted
-	      by Corey Farrell (license #5909)
+2017-08-25 08:19 +0000 [21c0283b78]  Thomas Sevestre <thomassevestre@free.fr>
 
-	  Change-Id: Ib23ca77e9b9e2803a450e1206af45df2d2fdf65c
+	* features, manager : Add CancelAtxfer AMI action
 
-2017-03-25 19:01 +0000 [70e5a2655d]  Daniel Journo <dan@keshercommunications.com>
+	  Add action to cancel feature attended transfer with AMI interface
 
-	* Unused realtime MOH classes not purged on 'moh reload'
+	  ASTERISK-27215 #close
 
-	  Purge Realtime MOH classes on 'moh reload' even when musiconhold.conf
-	  hasn't changed.
+	  Change-Id: Iab8a81362b5a1757e2608f70b014ef863200cb42
 
-	  ASTERISK-25974 #close
+2017-10-06 04:55 +0000 [6576e4320a]  Daniel Tryba <daniel@tryba.nl>
 
-	  Change-Id: I42c78ea76528473a656f204595956c9eedcf3246
+	* res_pjsip_session: Prevent user=phone being added to anonimized URIs.
 
-2017-04-03 13:56 +0000 [27b556778d]  Richard Mudgett <rmudgett@digium.com>
+	  Move ast_sip_add_usereqphone to be called after anonymization of URIs,
+	  to prevent the user_eq_phone adding "user=phone" to URIs containing a
+	  username that is not a phonenumber (RFC3261 19.1.1). An extra call to
+	  ast_sip_add_usereqphone on the saved version before anonymization is
+	  added to add user=phone" to the PAI.
 
-	* res_pjsip: Fix transport ref leak.
+	  ASTERISK-27047 #close
 
-	  We were leaking a transport ref in multihomed_on_rx_message() which
-	  resulted in the FRACK about excessive ref counts.
+	  Change-Id: Ie5644bc66341b86dc08b1f7442210de2e6acdec6
 
-	  ASTERISK-26916 #close
+2017-10-06 05:14 +0000 [a56316423f]  Daniel Tryba <daniel@tryba.nl>
 
-	  Change-Id: I7a96658a9614a060565bb9ad51cb1c9c11ee145f
+	* res_pjsip: Prevent "user=phone" being added multiple times to header
 
-2017-04-03 02:30 +0000 [94bd529f9e]  Alexander Traud <pabstraud@compuserve.com>
+	  ast_sip_add_usereqphone adds "user=phone" to the header every time is is
+	  called without checking whether the param already exists. Preventing
+	  this by searching to string representation of header for "user=phone".
 
-	* chan_sip: Session Timers required but refused wrongly.
+	  ASTERISK-26988 #close
 
-	  SIP user-agents indicate which protocol extensions are allowed in headers
-	  like Supported and Required. Such protocol extensions are Session Timers
-	  (RFC 4028) for example. Session Timers are supported since Mantis-10665.
-	  Since ASTERISK-21721, not only the first but multiple Supported/Required
-	  headers in a message are parsed. In that change, an existing variable was
-	  re-used within a newly added do-loop. Currently, at the end of that loop,
-	  that variable is an empty string always. Previously, that variable was used
-	  within log output. However, the log output was not changed.
+	  Change-Id: Ib84383b07254de357dc6a98d91fc1d2c2c3719e6
 
-	  ASTERISK-26915 #close
+2017-10-05 18:12 +0000 [e5b9eb0460]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I09315f31b4d78fb214bb2a9fb6c0f5e143eae990
+	* cdr.c: Defer misc checks.
 
-2017-03-31 16:31 +0000 [bca9685d39]  Joshua Colp <jcolp@digium.com>
+	  Try to defer some checks until needed in case there is an early exit.
 
-	* res_pjsip_session: Allow BYE to be sent on disconnected session.
+	  Change-Id: Ibc6b34c38a4f60ad4f9b67984b7d070a07257064
 
-	  It is perfectly acceptable for a BYE to be sent on a disconnected
-	  session. This occurs when we respond to a challenge to the BYE
-	  for authentication credentials.
+2017-10-06 20:48 +0000 [e8bde6916a]  Seán C McCord <ulexus@gmail.com>
 
-	  ASTERISK-26363
+	* ari/bridge: Add mute, dtmf suppression controls
 
-	  Change-Id: I6ef0ddece812fea6665a1dd2549ef44fb9d90045
+	  Add bridge_features structure to bridge creation.  Specifically, this
+	  implements mute and DTMF suppression, but others should be able to be
+	  easily added to the same structure.
 
-2017-03-30 18:28 +0000 [c701550803]  Corey Farrell <git@cfware.com>
+	  ASTERISK-27322 #close
+	  Reported by: Darren Sessions
+	  Sponsored by: AVOXI
 
-	* Forward declare 'struct ast_json' in asterisk.h
+	  Change-Id: Id4002adfb65c9a8027ee9e1a5f477e0f01cf9d61
 
-	  The ast_json structure is used in many Asterisk headers and is often the
-	  only part of json.h used.  This adds a forward declaration to asterisk.h
-	  and removes the include of json.h from many headers.  The declaration
-	  has been left in endpoints.h and stasis.h to avoid problems with source
-	  files that use ast_json functions without directly including json.h.
+2017-10-11 07:03 +0000 [ab4d36533c]  George Joseph <gjoseph@digium.com>
 
-	  ari.h continues to include json.h as it uses enum
-	  ast_json_encoding_format.
+	* chan_vpb:  Fix a gcc 7 out-of-bounds complaint
 
-	  Change-Id: Id766aabce6bed56626d27e8d29f559b5e687b769
+	  chan_vpb was trying to use sizeof(*p->play_dtmf), where
+	  p->play_dtmf is defined as char[16], to get the length of the array
+	  but since p->play_dtmf is an actual array, sizeof(*p->play_dtmf)
+	  returns the size of the first array element, which is 1.  gcc7
+	  validly complains because the context in which it's used could
+	  cause an out-of-bounds condition.
 
-2017-03-30 08:11 +0000 [754e99d517]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: If9c4bfdb6b02fa72d39e0c09bf88900663c000ba
 
-	* cdr_pgsql: Fix buffer overflow calling libpq
+2017-10-06 02:39 +0000 [be7da57546]  Nathan Bruning <nathan@iperity.com>
 
-	  Implement the same buffer size checking done in cel_pgsql.
+	* app_queue.c: clear moh field in init_queue
 
-	  ASTERISK-26896 #close
-	  Reported by: twisted
+	  ASTERISK-27301 #close
 
-	  Change-Id: Iaacfa1f1de7cb1e9414d121850d2d8c2888f3f48
+	  Change-Id: Ic31361f34e2de3b6470e68fc37205a7711082eba
 
-2017-03-28 13:01 +0000 [7954b39a50]  Walter Doekes <walter+github@wjd.nu>
+2017-10-09 21:00 +0000 [b8dadccbe1]  Corey Farrell <git@cfware.com>
 
-	* build: Fix deb build issues with fakeroot
+	* sorcery: Use ao2_weakproxy to hold list of instances.
 
-	  If DESTDIR is set, don't call ldconfig. Assume that DESTDIR is used to
-	  create a binary archive. The ldconfig call should be delegated to the
-	  archive postinst script. This fixes the case where fakeroot wraps 'make
-	  install' causing $EUID to be 0 even though it doesn't have permission to
-	  call ldconfig.
+	  * Store weak proxy objects in instances container.
+	  * Remove special unreference function and replace with macro that calls
+	  ao2_cleanup.
+	  * Add REF_DEBUG information to ast_sorcery_open.
 
-	  The previous logic in configure.ac to detect and correct libdir
-	  has been removed as it was not completely accurate.  CentOS 64-bit
-	  users should again specifiy --libdir=/usr/lib64 when configuring
-	  to prevent install to /usr/lib.
+	  Change-Id: I5a150a4e13cee319d46b5a4654f95a4623a978f8
 
-	  Updated Makefile:check-old-libdir to check for orphans in
-	  lib64 when installing to lib as well as orphans in lib when installing
-	  to lib64.
+2017-10-09 21:55 +0000 [7774623804]  Corey Farrell <git@cfware.com>
 
-	  Updated Makefile and main/Makefile uninstall targets to remove the
-	  orphans using the new logic.
+	* named_locks: Use ao2_weakproxy_find.
 
-	  ASTERISK-26705
+	  Change-Id: I0ce8a1b7101b6caac6a19f83a89f00eaba1e9d9c
 
-	  Change-Id: I51739d4a03e60bff38be719b8d2ead0007afdd51
+2017-10-09 17:51 +0000 [b058f8673a]  Corey Farrell <git@cfware.com>
 
-2017-03-29 10:11 +0000 [c9648f4690]  Sean Bright <sean.bright@gmail.com>
+	* astobj2: Add ao2_weakproxy_find function.
 
-	* astobj2: Prevent potential deadlocks with ao2_global_obj_release
+	  This function finds a weak proxy in an ao2_container and returns the
+	  real object associated with it.
 
-	  The ao2_global_obj_release() function holds an exclusive lock on the
-	  global object while it is being dereferenced. Any destructors that
-	  run during this time that call ao2_global_obj_ref() will deadlock
-	  because a read lock is required.
+	  Change-Id: I9da822049747275f5961b5c0a7f14e87157d65d8
 
-	  Instead, we make the global object inaccessible inside of the write
-	  lock and only dereference it once we have released the lock. This
-	  allows the affected destructors to fail gracefully.
+2017-10-10 15:09 +0000 [fd3101e8ad]  Corey Farrell <git@cfware.com>
 
-	  While this doesn't completely solve the referenced issue (the error
-	  message about not being able to create an IQ continues to be shown)
-	  it does solve the backtrace spew that accompanied it.
+	* astobj2: Run weakproxy callbacks outside of lock.
 
-	  ASTERISK-21009 #close
-	  Reported by: Marcello Ceschia
+	  Copy the list of weakproxy callbacks to temporary memory so they can be
+	  run without holding the weakproxy lock.
 
-	  Change-Id: Idf40ae136b5070dba22cb576ea8414fbc9939385
+	  Change-Id: Ib167622a8a0f873fd73938f7611b2a5914308047
 
-2017-03-30 11:57 +0000 [1d1309b1ed]  Joshua Colp <jcolp@digium.com>
+2017-10-10 12:01 +0000 [3ad7d2f36c]  Sean Bright <sean.bright@gmail.com>
 
-	* Revert "Update for 13.15.0-rc1"
+	* app_originate: Set ORIGINATE_STATUS correctly on failure
 
-	  This reverts commit 552cf009c0939c8b6597708135412bdc596df4bb.
+	  We were ignoring the return value from ast_pbx_outgoing_exten() and
+	  ast_pbx_outgoing_app() which could fail before setting the reason code.
+	  This resulted in failures being reported as success.
 
-	  Change-Id: Ie345bea481261b761c44079e9472622040fda302
+	  ASTERISK-25266 #close
+	  Reported by: Allen Ford
 
-2017-03-30 10:18 +0000 [3c23ebdef4]  Corey Farrell <git@cfware.com>
+	  Change-Id: Idf16237b7e41b527d2c69c865829128686beeb3b
 
-	* CEL: Remove header declarations of non-existant functions.
+2017-10-03 15:16 +0000 [b1d9fc87bc]  Torrey Searle <torrey@voxbone.com>
 
-	  ast_cel_alloc and ast_cel_destroy do not exist in code, remove them from
-	  the headers.
+	* contrib/thirdparty/sip_to_pjsip: add additional flag mappings
 
-	  Change-Id: I99ce848e2e109e7d61771559f559b9e57973e45c
+	  add mappings for udptl redundancy, rtptimeout, and debug flags
 
-2017-03-29 08:27 +0000 [ef19db9261]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ie73cf5c83c05dee01eb9624ede76c1a30225d73a
 
-	* srtp: Allow zero as tag value for a sRTP Crypto Suite.
+2017-10-02 16:46 +0000 [b0408d05c0]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25490 #close
+	* cdr.c: Eliminated simple RAII_VAR usages.
 
-	  Change-Id: I1c5fc0942c33c96d62b24203aad0f1e1a1a0131f
+	  Change-Id: I150505db307249a962987e7b941bdd369bb91f35
 
-2017-03-28 13:10 +0000 [a827892ff7]  gtjoseph <gjoseph@digium.com>
+2017-10-10 09:49 +0000 [11cefdf621]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* res_pjsip_config_wizard: Add 2 new parameters to help with proxy config
+	* cdr_mysql: avoid releasing a config string
 
-	  Two new parameters have been added to the pjsip config wizard.
+	  Fixes a memory corruption issue after a reload of cdr_mysql.
 
-	   * Setting 'sends_line_with_registrations' to true will cause the wizard
-	     to skip the creation of an identify object to match incoming request
-	     to the endpoint and instead add the line and endpoint parameters to
-	     the outbound registration object.
+	  Issue was accidentally included in 747beb1ed159f89a3b58742e4257740b3d6d6bba .
 
-	   * Setting 'outbound_proxy' is a shortcut for adding individual
-	     endpoint/outbound_proxy, aor/outbound_proxy and
-	     registration/outbound_proxy parameters.
+	  ASTERISK-27270 #close
 
-	  Change-Id: I678e5f80765734c056620528a6d40d82736ceeb0
+	  Change-Id: I90b6a9d18710c0f9009466370bd5f4bac5d5d12e
 
-2017-03-28 09:29 +0000 [864dda07f3]  Sean Bright <sean.bright@gmail.com>
+2017-10-10 07:42 +0000 [b228f5c5e6]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* alembic: Turn off execute bit on non-executable python scripts
+	* declare optional openssl dependencies in moduleinfo
 
-	  Change-Id: I744c986da4a38aeff8c00837eb89de7841fbc86c
+	  Declare optional openssl dependencies in:
+	  * res_rtp_asterisk.c
+	  * tcptls.c
 
-2017-03-27 12:37 +0000 [a9529cbb21]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27328 #close
 
-	* Add DTLS sanity check.
+	  Change-Id: I2636f1c05b8104b4fe6f36cce0ebd9a98b9c78ab
 
-	  Change-Id: Ib32612cf6c7ce9213a11b9cba82f630f8cd3564b
+2017-10-09 22:51 +0000 [fae09c6676]  Corey Farrell <git@cfware.com>
 
-2017-03-27 11:49 +0000 [bb68f57a03]  Josh Roberson <josh@asteriasgi.com>
+	* res_pjproject: Fix cleanup of buildopts vector.
 
-	* cel_pgsql.c: Fix buffer overflow calling libpq
+	  ASTERISK-27306
 
-	  PQEscapeStringConn() expects the buffer passed in to be an
-	  adequitely sized buffer to write out the escaped SQL value string
-	  into.  It is possible, for large values (such as large values to
-	  Dial with a lot of devices) to have more than our 512+1 byte
-	  allocation and thus cause libpq to create a buffer overrun.
+	  Change-Id: I3bed0edf3f55b1d4adcbabb25ec14f11dc766c72
 
-	  glibc will nicely ABRT asterisk for you, citing a stack smash.
+2017-10-03 16:09 +0000 [fdf9aacca3]  Richard Mudgett <rmudgett@digium.com>
 
-	  Let's only allocate it to be as large as needed:
-	  If we have a value, then (strlen(value) * 2) + 1 (as recommended
-	  by libpq), and if we have none, just one byte to hold our null
-	  will do.
+	* cdr.c: Replace redundant check with an ast_assert()
 
-	  ASTERISK-26896 #close
+	  The only caller of cdr_object_fn_table.process_party_b() explicitly does
+	  the check before calling.
 
-	  Change-Id: If611c734292618ed68dde17816d09dd16667dea2
+	  Change-Id: Ib0c53cdf5048227842846e0df9d2c19117c45618
 
-2017-03-24 07:43 +0000 [79a2c26c03]  Sean Bright <sean.bright@gmail.com>
+2017-10-02 17:41 +0000 [2e4b5fadbd]  Richard Mudgett <rmudgett@digium.com>
 
-	* core: Remove embedded module support
+	* cdr.c: Replace inlined code with ao2_t_replace()
 
-	  This has not worked for some time and is no longer actively maintained.
+	  Change-Id: I9f424f5282ca7d833592f958d95f1b2bafb549b0
 
-	  Change-Id: I5110b0db69c152761b58fa025cb0a53b0e544d99
+2017-09-29 12:07 +0000 [62980eedc3]  Richard Mudgett <rmudgett@digium.com>
 
-2017-03-27 09:35 +0000 [2c28f7a922]  Sean Bright <sean.bright@gmail.com>
+	* cdr.c: Use current ao2 flag names
 
-	* res_musiconhold: Document the 'format' option
+	  Change-Id: Ib59d7d2f2a4a822754628f2c48a308d6791a6e6e
 
-	  ASTERISK-26086 #close
-	  Reported by: Jens Bürger
+2017-09-29 12:31 +0000 [e769846f11]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I6aab666c0bf01fd0c64d7a5bcb22fa7f5d41335e
+	* cdr.h: Fix doxygen comments.
 
-2017-03-27 08:58 +0000 [61fd70c250]  Sean Bright <sean.bright@gmail.com>
+	  * Also some misc formatting in cdr.c.
 
-	* res_musiconhold: Don't chdir() when scanning MoH files
+	  Change-Id: Ied89a28802a662c37c43326a1aafdce596e0df4a
 
-	  There doesn't appear to be any reason that we are chdir'ing in
-	  moh_scan_files, and in the event of an Asterisk crash, the core files
-	  may not get written because we have changed into a read-only directory.
+2017-09-20 18:36 +0000 [fb19799b62]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-23996 #close
-	  Reported by: Walter Doekes
+	* res_pjsip_registrar.c: Update remove_existing AOR contact handling.
 
-	  Change-Id: Iac806dce01b3335963fbd62d4b4da9a65c614354
+	  When "rewrite_contact" is enabled, the "max_contacts" count option can
+	  block re-registrations because the source port from the endpoint can be
+	  random.  When the re-registration is blocked, the endpoint may give up
+	  re-registering and require manual intervention.
 
-2017-03-23 09:48 +0000 [73bb08fd6a]  Sean Bright <sean.bright@gmail.com>
+	  * The "remove_existing" option now allows a registration to succeed by
+	  displacing any existing contacts that now exceed the "max_contacts" count.
+	  Any removed contacts are the next to expire.  The behaviour change is
+	  beneficial when "rewrite_contact" is enabled and "max_contacts" is greater
+	  than one.  The removed contact is likely the old contact created by
+	  "rewrite_contact" that the device is refreshing.
 
-	* res_xmpp: Use incremental backoff when a read error occurs
+	  ASTERISK-27192
 
-	  If a read error occurs, we immediately attempt a reconnect without any
-	  delay. Instead, let's sleep and backoff up to 60 seconds before we try
-	  again.
+	  Change-Id: I64c107a10b70db1697d17136051ae6bf22b5314b
 
-	  ASTERISK-24712 #close
-	  Reported by: Matthias Urlichs
+2017-10-09 08:15 +0000 [ad38a55a2d]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I6fe10ef4734837727437beab715e336777f13f48
+	* res_config_sqlite: Don't enable SQLite CDRs when running 'make samples'
 
-2017-03-23 05:19 +0000 [55693383e2]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I65a5190b2732b2246d67472db70dd37db64ddad4
 
-	* res_xmpp: Fix ref counting issue
+2017-10-08 14:05 +0000 [a0a1f95abf]  David Hajek <david.hajek@daktela.com>
 
-	  The only remaining reference to the endpoint is in the endpoints
-	  container, and because it is unlinked in ast_endpoint_shutdown, we don't
-	  have to explicitly cleanup the endpoint ourselves.
+	* res/res_ari.c Fix: Memory leaks in ARI when using Content-Type: application/json
 
-	  Change-Id: I912a2692e52d3e2ed445b32d8ae3f9004bc2f2e8
+	  ASTERISK-27305
+	  Reported by: David Hajek
+	  Tested by: David Hajek
 
-2017-03-23 09:45 +0000 [1966265562]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: Ife3e289062e6cf7d0e7d342dbf79ed96feff441e
 
-	* res_xmpp: Try to provide useful errors messages from OpenSSL
+2017-10-08 09:11 +0000 [feeb0974eb]  Alexander Traud <pabstraud@compuserve.com>
 
-	  If any errors occur during the TLS connection setup, we currently dump a
-	  fairly generic error message. So instead we try to pull in something
-	  useful from OpenSSL to report instead.
+	* tcptls: Do not re-bind to wildcard on client creation.
 
-	  ASTERISK-24712
-	  Reported by: Matthias Urlichs
+	  Since ASTERISK-26922, this issue affected only those chan_sip which were
+	  * enabled for dual-stack (bindaddr=::), and
+	  * enabled for TCP (tcpenable=yes) and/or TLS (tlsenable=yes), and
+	  * tried to register and/or invite a IPv4-only service,
+	  * via TCP and/or TLS.
+	  Now, ast_tcptls_client_create does not re-bind to [::] anymore.
 
-	  Change-Id: I288500991a9681f447d92913b11fedaf426087f4
+	  ASTERISK-27324 #close
 
-2017-03-23 09:30 +0000 [03b99ae3d2]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I4b242837bdeb1ec7130dc82505c6180a946fd9b5
 
-	* res_xmpp: Correctly check return value of SSL_connect
+2017-10-07 15:47 +0000 [eb224fea5e]  Corey Farrell <git@cfware.com>
 
-	  SSL_connect returns non-zero for both success and some error conditions
-	  so simply negating is inadequate.
+	* res_pjsip_session: Fix format_cap leak.
 
-	  Change-Id: Ifbf882896e598703b6c615407fa456d3199f95b1
+	  ASTERISK-27306
 
-2017-03-24 11:29 +0000 [d9d2beba1c]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I2c8d3fc148f9f53715c958314e1146f9611741f3
 
-	* res_pjsip_sdp_rtp: Set hangup cause for RTP timeouts
+2017-10-06 10:51 +0000 [f4798faacc]  Matt Jordan <mjordan@digium.com>
 
-	  chan_sip sets the hangup cause code to AST_CAUSE_REQUESTED_CHAN_UNAVAIL
-	  (44) when a channel is hung up due to an RTP timeout. So do the same
-	  when it happens with PJSIP for parity.
+	* res_corosync: Fix linking issue with Corosync 2.x
 
-	  Change-Id: I3546ebbde6460c22a27c9da1bf321711b5961ab8
+	  At some point in time in the history of Corosync (certainly within the
+	  2.x branch), the corosync_cfg_state_track function was removed.
+	  Unfortunately, the cfg library is only linked if this function is
+	  present. Without the cfg library being linked to res_corosync, loading
+	  of res_corosync will fail.
 
-2017-03-23 12:07 +0000 [e6aeeabddf]  Kevin Harwell <kharwell@digium.com>
+	  This patch makes it so that detecting corosync's core libraries,
+	  determined by the COROSYNC external library checks, links both the cpg
+	  and cfg libraries with res_corosync.
 
-	* pjproject_bundled: raise timeout value used when downloading
+	  Change-Id: I674e9e1c8fea11c3bf81154aaa7c1fd43f945465
 
-	  After configuring Asterisk with '--with-pjproject-bundled' the configure/build
-	  process attempts to download pjproject from its download site. Currently, a
-	  timeout of 10 seconds is used that will stop the download process if pjproject
-	  has not been fully downloaded in that time. For some systems this was not enough
-	  time and the process was timing out too early.
+2017-10-05 16:26 +0000 [a68a91f722]  Corey Farrell <git@cfware.com>
 
-	  This patch raises the download timeout value to '60'. Also, this patch fixes
-	  another bug where the DOWNLOAD_TIMEOUT variable was not being properly exported
-	  due to a naming error. DOWNLOAD_MAX_TIMEOUT is now properly renamed to
-	  DOWNLOAD_TIMEOUT.
+	* res_pjsip: Fix leak of persistent endpoint references.
 
-	  ASTERISK-26814 #close
+	  Do not manually call sip_endpoint_apply_handler from load_all_endpoints.
+	  This is not necessary and causes memory leaks.
 
-	  Change-Id: Ia56e4e8a3d39db76bc8a1852b2cf07ec10b39842
+	  Additionally reinitialize persistent->aors when we reuse a persistent
+	  object with a new endpoint.
 
-2017-03-22 20:33 +0000 [0939a19cff]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27306
 
-	* res_xmpp: Correct implementation of JABBER_STATUS & JabberStatus
+	  Change-Id: I59bbfc8da8a14d5f4af8c5bb1e71f8592ae823eb
 
-	  The documentation for JABBER_STATUS (and the deprecated JabberStatus
-	  app) indicate that a return value of 7 indicates that the specified
-	  buddy was not in the roster. It also indicates that you can specify a
-	  "bare" JID (one without a resource). Unfortunately the actual behavior
-	  does not match the documented behavior.
+2017-10-05 17:59 +0000 [3bd00c4a7e]  Corey Farrell <git@cfware.com>
 
-	  Assuming that our roster includes the buddy online and available
-	  "valid@example.org/Valid" and does *not* include the buddy
-	  "invalid@example.org", the JABBER_STATUS() function returns the
-	  following before this patch:
+	* vector: multiple evaluation of elem in AST_VECTOR_ADD_SORTED.
 
-	  +------------------------------+------------+--------------------------+
-	  | Buddy                        | Status     | Result                   |
-	  +------------------------------+------------+--------------------------+
-	  | valid@example.org            |  Online    |  7 (Not in roster)       |
-	  | valid@example.org/Valid      |  Online    |  1 (Online)              |
-	  | valid@example.org/Invalid    |  N/A       |  7 (Not in roster)       |
-	  | invalid@example.org          |  N/A       |  Error logged, no return |
-	  | invalid@example.org/Valid    |  N/A       |  Error logged, no return |
-	  +------------------------------+------------+--------------------------+
+	  Use temporary variable to prevent multiple evaluations of elem argument.
+	  This resolves a memory leak in res_pjproject startup.
 
-	  And after this patch:
+	  ASTERISK-27317 #close
 
-	  +------------------------------+------------+--------------------------+
-	  | Buddy                        | Status     | Result                   |
-	  +------------------------------+------------+--------------------------+
-	  | valid@example.org            |  Online    |  1 (Online)              |
-	  | valid@example.org/Valid      |  Online    |  1 (Online)              |
-	  | valid@example.org/Invalid    |  N/A       |  6 (Offline)             |
-	  | invalid@example.org          |  N/A       |  7 (Not in roster)       |
-	  | invalid@example.org/Valid    |  N/A       |  7 (Not in roster)       |
-	  +------------------------------+------------+--------------------------+
+	  Change-Id: Ib960d7f5576f9e1a3c478ecb48995582a574e06d
 
-	  This brings the behavior in line with the documentation.
+2017-10-05 15:54 +0000 [b35ac9e566]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-23510 #close
-	  Reported by: Anthony Critelli
+	* res_pjsip: Fix leak of fake_auth references.
 
-	  Change-Id: I9c3241035363ef4a6bdc21fabfd8ffcd9ec657bf
+	  pjsip_distributor leaks references to fake_auth when the default realm
+	  has not changed.
 
-2017-03-22 17:32 +0000 [a487f6fb97]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27306
 
-	* res_xmpp: Don't crash when trying to send a message without a connection
+	  Change-Id: I3fcf103b3680ad2d1d4610dcd6738eeaebf4d202
 
-	  If we never establish a connection to our Jabber server, iksemel never sets up
-	  its internal transport pointer, so attempting to send a message dereferences a
-	  NULL pointer and causes a crash.
+2017-10-05 20:23 +0000 [0f3e725503]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-21855 #close
-	  Reported by: Jeremy Kister
+	* main/strings: Fix uninitialized value.
 
-	  Change-Id: I204a568894e4a53ab929783ecc594a000f04d79c
+	  ast_strings_match uses sscanf and checks for non-zero return to verify a
+	  token was parsed. This is incorrect as sscanf returns EOF (-1) for errors.
 
-2017-03-22 15:40 +0000 [90fb1fca41]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27318 #close
 
-	* res_xmpp: Include client name in connection related error messages
+	  Change-Id: Ifcece92605f58116eff24c5a0a3b0ee08b3c87b1
 
-	  ASTERISK-25622 #close
-	  Reported by: Sean Darcy
+2017-10-05 19:55 +0000 [0b6be1b2d4]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I8472cb7bfb58d411a3cfbd482da98cae2d94d1e9
+	* res_sdp_translator_pjmedia: Fix test unregistration.
 
-2017-03-21 12:32 +0000 [e196190f11]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+	  ASTERISK-27306
 
-	* cdr: Allow setting of user field from 'h' extension
+	  Change-Id: Ib3ed47167cb697ab7bd0a56cab589893f491651b
 
-	  The CDR code previously did not allow the user field to be set
-	  from the 'h' extension in the dialplan. This change removes that
-	  limitation and allows it to be set.
+2017-10-02 07:48 +0000 [59b6e8467a]  Daniel Tryba <daniel@pocos.nl>
 
-	  ASTERISK-26818
+	* res_pjsip_caller_id chan_sip: Comply to RFC 3323 values for privacy
 
-	  Change-Id: I0fed8a79b5e408bac4e30542b8f33a61c5ed9aa6
+	  Currently privacy requests are only granted if the Privacy header
+	  value is exactly "id" (defined in RFC 3325). It ignores any other
+	  possible value (or a combination there of). This patch reverses the
+	  logic from testing for "id" to grant privacy, to testing for "none" and
+	  granting privacy for any other value. "none" must not be used in
+	  combination with any other value (RFC 3323 section 4.2).
 
-2017-03-14 23:49 +0000 [d5b480afca]  Aaron An <anjb@ti-net.com.cn>
+	  ASTERISK-27284 #close
 
-	* audiohook.c: Lost RTP packets lead to out-of-sync MixMonitor.
+	  Change-Id: If438a21f31a962da32d7a33ff33bdeb1e776fe56
 
-	  Fixed a bug in function "ast_audiohook_write_frame" that checked the
-	  variable other_factory_samples and only flushed the factories, so they
-	  would be in sync, when other_factory_samples > 0. When there is not any
-	  rtp incoming the variable other_factory_samples will be 0, and although
-	  the result of "our_factory_ms - other_factory_ms" may be very large,
-	  this led to the record file not syncing.
-
-	  ASTERISK-26875 #close
-	  Reported-by: Aaron An
-	  Tested-by: Aaron An
-
-	  Change-Id: Ia4d890fb8fc1636a7188502bab35f555685aea22
+2017-10-04 10:59 +0000 [65399a5eda]  Corey Farrell <git@cfware.com>
 
-2017-03-23 20:33 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* res_pjsip: Add REF_DEBUG info to module references.
 
-	* asterisk 13.15.0-rc1 Released.
+	  This provides better information to REF_DEBUG log for troubleshooting
+	  when the system is unable to unload res_pjsip.so during shutdown due to
+	  module references.
 
-2017-03-23 14:03 +0000 [f1b34e6eb4]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-27306
 
-	* AMI: Updated version
+	  Change-Id: I63197ad33d1aebe60d12e0a6561718bdc54e4612
 
-	  Updated the AMI version for the following reason (see CHANGES for more details):
+2017-10-04 10:46 +0000 [7d04544986]  Corey Farrell <git@cfware.com>
 
-	  The 'PJSIPShowEndpoint' command's response event of 'IdentifyDetail' now
-	  contains a new optional parameter, 'MatchHeader'.
+	* res_pjsip: Fix issues that prevented shutdown of modules.
 
-	  Change-Id: I9aeac4decc89f9b464b3f026e97c7ef1acc79242
+	  res_pjsip and res_pjsip_session had circular references, preventing both
+	  modules from shutting down.
+	  * Move session supplement registration to res_pjsip.
+	  * Use create internal functions for use by pjsip_message_filter.c.
 
-2017-03-14 16:45 +0000 [398e5ec16c]  Richard Begg <asterisk@meric.id.au>
+	  ASTERISK-27306
 
-	* res_pjsip_session: Enable RFC3578 overlap dialing support.
+	  Change-Id: Ifbd5c19ec848010111afeab2436f9699da06ba6b
 
-	  Support for RFC3578 overlap dialling (i.e. 484 Response to partially matched
-	  destinations) as currently provided by chan_sip is missing from res_pjsip.
-	  This patch adds a new endpoint attribute (allow_overlap) [defaults to yes]
-	  which when set to yes enables 484 responses to partial destination
-	  matches rather than the current 404.
+2017-09-28 02:56 +0000 [2301447a20]  Benoît Dereck-Tricot <benoit.dereck-tricot@eyepea.eu>
 
-	  ASTERISK-26864
+	* res_calendar_icalendar: Filter out occurrences superceded by another VEVENT
 
-	  Change-Id: Iea444da3ee7c7d4f1fde1d01d138a3d7b0fe40f6
+	  When we are loading the calendars, we call libical's
+	  icalcomponent_foreach_recurrence method for each VEVENT component that
+	  we have in our calendar.
 
-2017-03-21 06:59 +0000 [218f618095]  Sean Bright <sean.bright@gmail.com>
+	  That method has no knowledge concerning the existence of the other
+	  VEVENT components and will feed our callback with all ocurrences
+	  matching the requested time span.
 
-	* res_hep: Capture actual transport type in use
+	  The occurrences generated by icalcomponent_foreach_recurrence while
+	  expanding a recurring VEVENT's RRULE and RDATE properties can be
+	  superceded by an other VEVENT sharing the same UID.
 
-	  Rather than hard-coding UDP, allow consumers of the HEP API to specify
-	  which protocol is in use. Update the PJSIP provider to pass in the
-	  current protocol type.
+	  I use an external iterator (in libical terminology) to avoid messing
+	  with the internal ones from the calling function, and search for
+	  VEVENTS which could supersede the current occurrence.
 
-	  ASTERISK-26850 #close
+	  The event which can invalidate this occurence needs to have:
 
-	  Change-Id: I54bbb0a001cfe4c6a87ad4b6f2014af233349978
+	  - the same UID as our recurrent component (comp)
+	  - a RECURRENCE-ID property, which represents the start time of this
+	    occurrence
 
-2017-03-21 09:57 +0000 [1c8b81a2a4]  Sean Bright <sean.bright@gmail.com>
+	  If one component is found, just clean and return.
 
-	* Revert "app_queue: Handle the caller being redirected out of a queue bridge"
+	  ASTERISK-27296 #close
+	  Reported by: Benoît Dereck-Tricot
 
-	  This reverts commit 163e9e53dc7d84dd42721e733b7706c8147bdd27.
+	  Change-Id: I8587ae3eaa765af7cb21eda3b6bf84e8a1c87af8
 
-	  Change-Id: Ief28479c77a298879dfe2c56be7ee92dc465da4b
+2017-09-28 17:37 +0000 [b2dbfe23ef]  Richard Mudgett <rmudgett@digium.com>
 
-2017-03-21 08:26 +0000 [b3cc20799b]  Sean Bright <sean.bright@gmail.com>
+	* app_queue.c: Fix announcements when announce-to-first-user not enabled.
 
-	* res_pjsip_messaging: Check URI type before dereferencing
+	  The previous patch for ASTERISK-27216 made it so you wouldn't get any
+	  position or periodic announcements unless you had announce-to-first-user
+	  enabled.  The announce-to-first-user feature was added by ASTERISK_21782
+	  as a result of the patch which introduced the redundant announcements that
+	  ASTERISK-27216 removes.
 
-	  We aren't validating that the URI we just parsed is a SIP/SIPS one before
-	  trying to access the user, host, and port members of a possibly uninitialized
-	  structure.
+	  * By noting that the makeannouncement variable is used to suppresses the
+	  first user announcement, we set its initial value to the
+	  announce-to-first-user enable setting.
 
-	  Also update the MessageSend documentation to indicate what 'from' formats are
-	  accepted.
+	  ASTERISK-27216
 
-	  ASTERISK-26484 #close
-	  Reported by: Vinod Dharashive
+	  Change-Id: Ieaeb7dbea8ae7073086b775fbafe0625b000b10a
 
-	  Change-Id: I476b5cc5f18a7713d0ee945374f2a1c164857d30
+2017-09-21 14:43 +0000 [80097676e7]  Richard Mudgett <rmudgett@digium.com>
 
-2017-03-13 15:21 +0000 [91c97b5da5]  Joshua Elson <joshelson@gmail.com>
+	* heap.c: No need to calloc heap pointer array.
 
-	* pjsip: prevent memory corruption on creation of xml bodies
+	  Change-Id: I5ae2f316229f336eb90d99c7af7ed07a33097e68
 
-	  ASTERISK-26776 #close
+2017-09-27 13:45 +0000 [d1de7948fe]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I884b6f4e8233a355d0be687ec78d41bc0e4d3fd2
+	* logger:  Bring back ability to  turn debug on by source file
 
-2017-03-20 16:27 +0000 [7f34c11b6a]  Sean Bright <sean.bright@gmail.com>
+	  Somewhere along the way we lost the ability to debug individual
+	  source files.  For modules, this wasn't a big deal but all the
+	  source files in ./main are in the one "core" module so debugging
+	  individual core capabilities was almost impossible.
 
-	* bridge_softmix: Ignore non-voice frames from translator
+	  * Added a test to DEBUG_ATLEAST that also checks __FILE__ instead
+	  of just module name.  Any source file will work even if it's in
+	  a module subdirectory.
 
-	  Some codecs - codec_speex specifically - take voice frames and return
-	  other types of frames, like CNG. If we subsequently treat those as
-	  voice frames, we'll run into trouble when destroying the frame because
-	  of the requirement that each voice frame have an associated format.
+	  Change-Id: Icc0af41837f3b1679dec7af21fa32cd1f7469f6e
 
-	  ASTERISK-26880 #close
-	  Reported by: Kirsty Tyerman
+2017-09-28 05:33 +0000 [f21408c866]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I43f8450c48fb276ad8b99db8512be82949c1ca7c
+	* res_stasis: Add 'video_sfu' as a requested bridge type.
 
-2017-03-18 12:30 +0000 [38cebc73a3]  Sean Bright <sean.bright@gmail.com>
+	  This change adds 'video_sfu' as a requested bridge type when
+	  creating a bridge. By specifying this a mixing type bridge is
+	  created that exchanges video in an SFU fashion.
 
-	* thread safety: Don't use getprotobyname()
+	  Change-Id: I2ada47cf5f3fc176518b647c0b4aa39d55339606
 
-	  POSIX does not require getprotobyname() to be thread safe and some
-	  implementations use static memory which causes issues when multiple
-	  threads are used.
+2017-09-27 11:16 +0000 [a6dc0527a2]  Richard Mudgett <rmudgett@digium.com>
 
-	  Further, our usage of it today is just to ultimately get IPPROTO_TCP
-	  for calls to setsockopt(). So instead we just use IPPROTO_TCP directly.
+	* res_pjsip_outbound_publish.c: Fix misplaced parenthesis.
 
-	  Change-Id: I2e14e58674808f7ce99b2f5e900d0f90d0d8da48
+	  The pjsip_publishc_init() call was referenced with a misplaced
+	  parentheses.  As a result, outbound publication messages went out with an
+	  expiration of 1 second.
 
-2017-03-19 13:26 +0000 [265455bc2d]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27298
 
-	* res_rtp_asterisk: Pass correct data length to ast_rtcp_interpret
+	  Change-Id: I93622eabc8ee83e7a22e98c107f921284c605a08
 
-	  We are currently passing in the capacity of the read buffer instead of the
-	  number of bytes that we actually read off the wire.
+2017-09-26 11:01 +0000 [61ea872233]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I60465049727d955c7f9a5e529e6f2aaff04cda36
+	* pjsip_message_filter: Fix regression causing bad contact address
 
-2017-03-14 09:27 +0000 [76afb9e18a]  Robert Mordec <r.mordec@slican.pl>
+	  The "res_pjsip:  Filter out non SIP(S) requests" commit moved the
+	  filtering of messages to pjproject's PJSIP_MOD_PRIORITY_TRANSPORT_LAYER
+	  in order to filter out incoming bad uri schemes as early as possible.
+	  Since the change affected outgoing messages as well and the TRANSPORT
+	  layer is the last to be run on outgoing messages, we were overwriting
+	  the setting of external_signaling_address (which is set earlier by
+	  res_pjsip_nat) with an internal address.
 
-	* app_queue: Member stuck as pending after forwarding previous call from queue
+	  * pjsip_message_filter now registers itself as a pjproject module
+	  twice.  Once in the TSX layer for the outgoing messages (as it was
+	  originally), then a second time in the TRANSPORT layer for the
+	  incoming messages to catch the invalid uri schemes.
 
-	  Queue member will get stuck in pending_members if queue calls a device
-	  that is different from the one observed for state changes.
+	  ASTERISK-27295
+	  Reported by: Sean Bright
 
-	  This patch removes members from pending_members as a result of channel stasis
-	  events such as blind or attended transfers and hangup.
+	  Change-Id: I2c90190c43370f8a9d1c4693a19fd65840689c8c
 
-	  ASTERISK-26862 #close
+2017-09-13 21:31 +0000 [9d65057cdf]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I8bf6df487b9bb35726c08049ff25cdad5e357727
+	* res_rtp_asterisk.c: Fix bridge_p2p_rtp_write() reentrancy potential.
 
-2017-02-22 23:26 +0000 [60b372a883]  Richard Mudgett <rmudgett@digium.com>
+	  The bridge_p2p_rtp_write() has potential reentrancy problems.
 
-	* CHANNEL(callid): Give dialplan access to the callid.
+	  * Accessing the bridged RTP members must be done with the instance1 lock
+	  held.  The DTMF and asymmetric codec checks must be split to be done with
+	  the correct RTP instance struct locked.  i.e., They must be done when
+	  working on the appropriate side of the point to point bridge.
 
-	  * Added CHANNEL(callid) to retrieve the call identifier log tag associated
-	  with the channel.  Dialplan now has access to the call log search key
-	  associated with the channel so it can be saved in case there is a problem
-	  with the call.
+	  * Forcing the RTP mark bit was referencing the wrong side of the point to
+	  point bridge.  The set mark bit is used everywhere else to set the mark
+	  bit when sending not receiving.
 
-	  ASTERISK-26878
+	  The patches for ASTERISK_26745 and ASTERISK_27158 did not take into
+	  account that not everything carried by RTP uses a codec.  The telephony
+	  DTMF events are not exchanged with a codec.  As a result when
+	  RFC2833/RFC4733 sent digits you would crash if "core set debug 1" is
+	  enabled, the DTMF digits would always get passed to the core even though
+	  the local native RTP bridge is active, and the DTMF digits would go out
+	  using the wrong SSRC id.
 
-	  Change-Id: I2c97ebd928b6f3c5bc80c5729e4d3c07f453049f
+	  * Add protection for non-format payload types like DTMF when updating the
+	  lastrxformat and lasttxformat.  Also protect against non-format payload
+	  types when checking for asymmetric codecs.
 
-2017-03-16 08:42 +0000 [9a57b24e17]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27292
 
-	* app_queue: Fix locking behavior in stasis message handlers
+	  Change-Id: I6344ab7de21e26f84503c4d1fca1a41579364186
 
-	  The queue_stasis_data structure contains various mutable fields that require
-	  appropriate locking. Specifically, the 'dying,' 'member_uniqueid,' and
-	  'caller_uniqueid' fields need to be locked when read from or written to.
+2017-09-26 10:55 +0000 [c9e972a26a]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I246b7dbff8447acc957a1299f6ad0ebd0fd39088
+	* res_rtp_asterisk: Trim trailing byte off of SDES packet
 
-2017-03-07 19:28 +0000 [8721d0bf1b]  Sean Bright <sean.bright@gmail.com>
+	  This could have been fixed by subtracting 1 from the final value of
+	  'len' but the way the packet was being constructed was confusing so I
+	  took the opportunity to (I think) make it more clear.
 
-	* chan_sip: Add rtcp-mux support
+	  We were sending 1 extra byte at the end of the SDES RTCP packet which
+	  caused Chrome to complain (in its debug log):
 
-	  ASTERISK-26846 #close
+	      Too little data (1 byte) remaining in buffer to parse
+	      RTCP header (4 bytes).
 
-	  Change-Id: I541a1602ff55ab73684e9f8002edb9e0e745d639
+	  We now send the correct number of bytes.
 
-2017-03-16 16:50 +0000 [792171ea9e]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I9dcf087cdaf97da0374ae0acb7d379746a71e81b
 
-	* app_confbridge: Fix ConfbridgeTalking AMI event description.
+2017-09-25 13:00 +0000 [721947ebae]  Sean Bright <sean.bright@gmail.com>
 
-	  Thanks to Chris Howard for pointing this out on the wiki.
+	* webrtc: Allow 'webrtc' to be set on endpoints without dtls_ca_file
 
-	  Change-Id: I18e56de09a70e736b5d04719d45ef29cf0636705
+	  If using a legitimate certificate from a trusted certificate authority,
+	  you don't need to provide CA file.
 
-2017-03-16 16:37 +0000 [047fb7f11e]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I8623973b4209b44889243716d7880274caed8a6d
 
-	* res_pjsip_asterisk.c: Fix compile error if libsrtp is not installed.
+2017-09-25 13:09 +0000 [0cbeaa5589]  Sean Bright <sean.bright@gmail.com>
 
-	  struct ast_rtcp does not define the dtls member if SRTP is not enabled.
+	* pjproject: Patch to correct STUN FINGERPRINT usage
 
-	  ASTERISK-26732
+	  Change-Id: I0e453253dff1388b0186b36c754457c1d0d12db6
 
-	  Change-Id: Id15ea212e04490e012f2cf4a56818b4dd948875e
+2017-09-25 12:30 +0000 [b74cbadd05]  Kevin Harwell <kharwell@digium.com>
 
-2017-03-16 15:45 +0000 [a75f02c089]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_session: outgoing call did not offer all configured codecs
 
-	* res_pjsip_sdp_rtp.c: Fix cut-n-paste error
+	  For some scenarios when an outgoing call was made only a subset of the
+	  configured codecs were offered. If the codecs being offered happened to
+	  not have a codec supported by the phone then the call would fail.
 
-	  We were inadvertenly referencing the cos_video option to determine if we
-	  should set the tos_audio and cos_audio value on the RTP instance.
+	  For instance Alice and Bob both are configured in Asterisk for g722 and ulaw(
+	  allow=!all,g722,ulaw). Alice's endpoint however only supports g722 while Bob's
+	  only supports ulaw. When Alice calls Bob, Alice negotiates g722 fine with
+	  Asterisk. But when Asterisk sends the outgoing offer to Bob it only contains
+	  g722 and not both g722 and ulaw, so the call ends.
 
-	  Change-Id: Ia7964f486801d39dc6f5dae570baff079e1595b0
+	  This patch makes it so all the audio codecs configured on the endpoint always
+	  get sent, and not just a subset. However priority is given to those codecs that
+	  are compatible with the "other side".
 
-2017-03-16 10:39 +0000 [776ffd7724]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-27259 #close
 
-	* res/res_pjsip_session: Only check localnet if it is defined
+	  Change-Id: Iffabc373bd94cd1dc700925dcfe406e12918c696
 
-	  If local_net is not defined on a transport, transport_state->localnet
-	  will be NULL. ast_apply_ha will, be default, return AST_SENSE_ALLOW in
-	  this case, causing the external_media_address, if set, to be skipped.
+2017-09-25 10:59 +0000 [08e67f814b]  Richard Mudgett <rmudgett@digium.com>
 
-	  This patch causes us to only check if we are sending within a network if
-	  local_net is defined.
+	* channel.c: Fix invalid reference in conditionaled out code.
 
-	  ASTERISK-26879 #close
+	  ASTERISK-27289
 
-	  Change-Id: Ib661c31a954cabc9c99f1f25c9c9a5c5b82cbbfb
+	  Change-Id: I7a415948116493050614d9f4fa91ffbe0c21ec4c
 
-2017-03-14 16:22 +0000 [139bc3495f]  Richard Begg <asterisk@meric.id.au>
+2017-09-25 07:25 +0000 [4275ca16a1]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_sdp_rtp: RTP instance does not use same IP as explicit transport
+	* build:  A few gcc 7 error fixes
 
-	  Currently a wildcard address is used for the local RTP socket, which
-	  will not always result in the same address as used by the SIP socket
-	  (e.g. if explicit transport addresses are configured).
-	  Use the transport's host address when binding new local RTP sockets if
-	  available.
+	  Change-Id: I7b5300fbf1af7d88d47129db13ad6dbdc9b553ec
 
-	  ASTERISK-26851
+2017-09-15 02:59 +0000 [c3c73b3511]  Stefan Engström <stefanen@kth.se>
 
-	  Change-Id: I098c29c9d1f79a4f970d72ba894874ac75954f1a
+	* app_queue: Only do announcement logic between ringing cycles
 
-2017-03-16 09:07 +0000 [7ea7797e12]  Joshua Colp <jcolp@digium.com>
+	  This patch reverts the change by patch 2263 from old reviewboard.
+	  Note that reverting that 2263-patch still preserves the behaviour that
+	  the commit log of the 2263-patch claimed to add. The reason for this is:
 
-	* res_rtp_asterisk: Fix crash when RTCP is not present when DTLS is stopped.
+	  The function wait_for_answer is only called from try_calling which
+	  in turn is only called from the main for loop in queue_exec, and
+	  earlier in that loop we already check the things that's removed by
+	  this patch. There's no need to check those things twice each loop
+	  iteration, and I think the proper place to check it is before each
+	  ringing cycle. By checking it in wait_for_answer, you allow the issue
+	  explained in the jira - that the head caller hears announcements while
+	  the agents' sip phones are actively ringing.
 
-	  This change removes an assumption that when DTLS is stopped
-	  an RTCP session will be present on the RTP session. This is not
-	  always the case.
+	  Reported-by: Stefan Engström
+	  Tested-by: Stefan Engström
+	  ASTERISK-27216 #close
 
-	  ASTERISK-26732
+	  Change-Id: Ic4290dc75256f9743900c6762ee1bb915f672db0
 
-	  Change-Id: Ib9f7c09ce0b005efe362dbcc8795202b18f94611
+2017-09-23 12:32 +0000 [0fad11f21c]  Sean Bright <sean.bright@gmail.com>
 
-2017-03-07 08:33 +0000 [9b756662a8]  gtjoseph <gjoseph@digium.com>
+	* app_stream_echo: Don't echo declined streams
 
-	* res_pjsip:  Symmetric transports
+	  Discovered while experimenting with Cyber Mega Phone 2K Ultimate Dynamic
+	  Edition after accepting the audio request but declining the video one.
 
-	  A new transport parameter 'symmetric_transport' has been added.
+	  Change-Id: Iaa86d41fccfbc1b559a30ccf740d78a3b5f8a98c
 
-	  When a request from a dynamic contact comes in on a transport with
-	  this option set to 'yes', the transport name will be saved and used
-	  for subsequent outgoing requests like OPTIONS, NOTIFY and INVITE.
-	  It's saved as a contact uri parameter named 'x-ast-txp' and will
-	  display with the contact uri in CLI, AMI, and ARI output.  On the
-	  outgoing request, if a transport wasn't explicitly set on the
-	  endpoint AND the request URI is not a hostname, the saved transport
-	  will be used and the 'x-ast-txp' parameter stripped from the
-	  outgoing packet.
+2017-09-22 17:49 +0000 [601e0c563f]  Joshua Colp <jcolp@digium.com>
 
-	  * config_transport was modified to accept and store the new parameter.
+	* res_pjsip_session: Reduce (and improve) SDP renegotiation.
 
-	  * config_transport/transport_apply was updated to store the transport
-	    name in the pjsip_transport->info field using the pjsip_transport->pool
-	    on UDP transports.
+	  When pruning a request to change the topology of a channel be
+	  more intelligent about the resulting topology that is actually
+	  used for SDP renegotiation.
 
-	  * A 'multihomed_on_rx_message' function was added to
-	    pjsip_message_ip_updater that, for incoming requests, retrieves the
-	    transport name from pjsip_transport->info and retrieves the transport.
-	    If transport->symmetric_transport is set, an 'x-ast-txp' uri parameter
-	    containing the transport name is added to the incoming Contact header.
+	  In a case where a stream has not already been negotiated we
+	  don't need to renegotiate and offer a declined stream. This can
+	  occur if something in Asterisk (such as ConfBridge) requests
+	  to add video to a PJSIP channel that has no video codecs configured.
+	  In this case since the stream did not already exist we can safely
+	  remove the stream from the requested topology, resulting in no
+	  renegotiation occurring.
 
-	  * An 'ast_sip_get_transport_name' function was added to res_pjsip.
-	    It takes an ast_sip_endpoint and a pjsip_sip_uri and returns a
-	    transport name if endpoint->transport is set or if there's an
-	    'x-ast-txp' parameter on the uri and the uri host is an ipv4 or
-	    ipv6 address.  Otherwise it returns NULL.
+	  In a case where a renegotiation is requested with a codec that is
+	  not supported we can reuse the formats of the existing stream if
+	  it exists to ensure that the stream continues to flow, instead of
+	  removing it.
 
-	  * An 'ast_sip_dlg_set_transport' function was added to res_pjsip
-	    which takes an ast_sip_endpoint, a pjsip_dialog, and an optional
-	    pjsip_tpselector.  It calls ast_sip_get_transport_name() and if
-	    a non-NULL is returned, sets the selector and sets the transport
-	    on the dialog.  If a selector was passed in, it's updated.
+	  Change-Id: I636540798d55922377318fe619c510fb6ed125fb
 
-	  * res_pjsip/ast_sip_create_dialog_uac and ast_sip_create_dialog_uas
-	    were modified to call ast_sip_dlg_set_transport() instead of their
-	    original logic.
+2017-09-22 15:29 +0000 [36690c26f8]  Kevin Harwell <kharwell@digium.com>
 
-	  * res_pjsip/create_out_of_dialog_request was modified to call
-	    ast_sip_get_transport_name() and pjsip_tx_data_set_transport()
-	    instead of its original logic.
+	* res_pjsip_session: Don't end session when receiving a 500 on a reinvite
 
-	  * Existing transport logic was removed from endpt_send_request
-	    since that can only be called after a create_out_of_dialog_request.
+	  During a reinvite, if a remote endpoint error occurs and it returns a 500 the
+	  session would end. This patch makes it so the session is not terminated, but
+	  continues as it was.
 
-	  * res_pjsip/ast_sip_create_rdata was converted to a wrapper around
-	    a new 'ast_sip_create_rdata_with_contact' function which allows
-	    a contact_uri to be specified in addition to the existing
-	    parameters.  (See below)
+	  The reason for this is because some endpoints may send non session terminating
+	  "server errors" like a failed codec negotiation. So in this case instead of
+	  ending the call it can hopefully continue. In the case of a real server error
+	  the session is already "doomed", will be known soon enough and appropriately
+	  ended by Asterisk later.
 
-	  * res_pjsip_pubsub/internal_pjsip_evsub_send_request was eliminated
-	    since all it did was transport selection and that is now done in
-	    ast_sip_create_dialog_uac and ast_sip_create_dialog_uas.
+	  Change-Id: Ifeedae86b8cb44b92d52c79046522ec5f0aff1d5
 
-	  * 'contact_uri' was added to subscription_persistence.  This was
-	    necessary because although the parsed rdata contact header has the
-	    x-ast-txp parameter added (if appropriate),
-	    subscription_persistence_update stores the raw packet which
-	    doesn't have it.  subscription_persistence_recreate was then
-	    updated to call ast_sip_create_rdata_with_contact with the
-	    persisted contact_uri so the recreated subscription has the
-	    correct transport info to send the NOTIFYs.
+2017-09-22 10:02 +0000 [ebd0a4bebf]  Sean Bright <sean.bright@gmail.com>
 
-	  * res_pjsip_session/internal_pjsip_inv_send_msg was eliminated since
-	    all it did was transport selection and that is now done in
-	    ast_sip_create_dialog_uac.
+	* res_pjsip: Use ast_sip_is_content_type() where appropriate
 
-	  * pjsip_message_ip_updater/multihomed_on_tx_message was updated
-	    to remove all traces of the x-ast-txp parameter from the
-	    outgoing headers.
+	  Change-Id: If3ab0d73d79ac4623308bd48508af2bfd554937d
 
-	  NOTE:  This change does NOT modify the behavior of permanent
-	  contacts specified on an aor.  To do so would require that the
-	  permanent contact's contact uri be updated with the x-ast-txp
-	  parameter and the aor sorcery object updated.  If we need to
-	  persue this, we need to think about cloning permanent contacts into
-	  the same store as the dynamic ones on an aor load so they can be
-	  updated without disturbing the originally configured value.
+2017-09-21 09:47 +0000 [6c0e13da22]  George Joseph <gjoseph@digium.com>
 
-	  You CAN add the x-ast-txp parameter to a permanent contact's uri
-	  but it would be much simpler to just set endpoint->transport.
+	* res_pjsip_session/BUNDLE:  Handle no audio codecs on endpoint
 
-	  Change-Id: I4ee1f51473da32ca54b877cd158523efcef9655f
+	  When an INVITE came in with both audio and video streams but there
+	  were no audio codecs defined for the endpoint, we weren't declining
+	  the audio stream.  Since it's usually the first/transport stream,
+	  when the video stream was processed and tried to use the transport,
+	  it was empty and caused a crash.  We now decline the the stream if
+	  there are no matching codecs so when the video stream is processed,
+	  it's now the first/transport stream and processes normally.
 
-2017-03-15 13:24 +0000 [adad6020be]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ic854eda54c95031e66b076ecfae3041d34daa692
 
-	* autochan/mixmonitor/chanspy: Fix unsafe channel locking and references.
+2017-09-19 14:28 +0000 [7c93982e9d]  Richard Mudgett <rmudgett@digium.com>
 
-	  Dereferencing struct ast_autochan.chan without first calling
-	  ast_autochan_channel_lock() is unsafe because the pointer could change at
-	  any time due to a masquerade.  Unfortunately, ast_autochan_channel_lock()
-	  itself uses struct ast_autochan.chan unsafely and can result in a deadlock
-	  if the original channel happens to get destroyed after a masquerade in
-	  addition to the pointer getting changed.
+	* res_rtp_asterisk.c: Fix bundled SSRC handling.
 
-	  The problem is more likely to happen with v11 and earlier because
-	  masquerades are used to optimize out local channels on those versions.
-	  However, it could still happen on newer versions if the channel is
-	  executing a dialplan application when the channel is transferred or
-	  redirected.  In this situation a masquerade still must be used.
+	  Assertions in the v15+ AST-2017-008 patches found that we were not
+	  handling the case if the incoming SDP did not specify the required SSRC
+	  attributes for bundled to work.
 
-	  * Added a lock to struct ast_autochan to safely be able to use
-	  ast_autochan.chan while trying to get the channel lock in
-	  ast_autochan_channel_lock().  The locking order is the channel lock then
-	  the autochan lock.  Locking in the other direction requires deadlock
-	  avoidance.
+	  * Be strict on matching SSRC for bundled instances including the parent
+	  instance.  If the SSRC doesn't match then discard the packet.  Bundled has
+	  to tell us in the SDP signaling what SSRC to expect.  Otherwise, we will
+	  not know how to find the bundled instance structure.
 
-	  * Fix unsafe ast_autochan.chan usages in app_mixmonitor.c.
+	  Change-Id: I152830bbff71c662408909042068fada39e617f9
 
-	  * Fix unsafe ast_autochan.chan usages in app_chanspy.c.
+2017-09-16 09:19 +0000 [f2985e3106]  Joshua Colp <jcolp@digium.com>
 
-	  * app_chanspy.c: Removed unused autochan parameter from next_channel().
+	* bridge: Change participant SFU streams when source streams change.
 
-	  ASTERISK-26867
+	  Some endpoints do not like a stream being reused for a new
+	  media stream. The frame/jitterbuffer can rely on underlying
+	  attributes of the media stream in order to order the packets.
+	  When a new stream takes its place without any notice the
+	  buffer can get confused and the media ends up getting dropped.
 
-	  Change-Id: Id29dd22bc0f369b44e23ca423d2f3657187cc592
+	  This change uses the SSRC change to determine that a new source
+	  is reusing an existing stream and then bridge_softmix renegotiates
+	  each participant such that they see a new media stream. This
+	  causes the frame/jitterbuffer to start fresh and work as expected.
 
-2017-03-07 14:13 +0000 [7bc69753bc]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-27277
 
-	* Add rtcp-mux support
+	  Change-Id: I30ccbdba16ca073d7f31e0e59ab778c153afae07
 
-	  This commit adds support for RFC 5761: Multiplexing RTP Data and Control
-	  Packets on a Single Port. Specifically, it enables the feature when
-	  using chan_pjsip.
+2017-09-20 10:45 +0000 [971548405b]  George Joseph <gjoseph@digium.com>
 
-	  A new option, "rtcp_mux" has been added to endpoint configuration in
-	  pjsip.conf. If set, then Asterisk will attempt to use rtcp-mux with
-	  whatever it communicates with. Asterisk follows the rules set forth in
-	  RFC 5761 with regards to falling back to standard RTCP behavior if the
-	  far end does not indicate support for rtcp-mux.
+	* res_pjsip_session:  Change some asserts to warning/debug messages
 
-	  The lion's share of the changes in this commit are in
-	  res_rtp_asterisk.c. This is because it was pretty much hard wired to
-	  have an RTP and an RTCP transport. The strategy used here is that when
-	  rtcp-mux is enabled, the current RTCP transport and its trappings (such
-	  as DTLS SSL session) are freed, and the RTCP session instead just
-	  mooches off the RTP session. This leads to a lot of specialized if
-	  statements throughout.
+	  There was an issue reported where an SDP received on a 183 Session
+	  Progress message caused a crash because the pending streams had
+	  already been processed when the OK was received.  In that case the
+	  pending topology was legitimately NULL.  There was an assert for an
+	  incorrect number of streams in the topology but not one for
+	  topology being NULL.  In any case, if you're not in dev-mode the
+	  asserts don't do anything and since the scenario is legit, the
+	  asserts weren't appropriate anyway.
 
-	  ASTERISK-26732 #close
-	  Reported by Dan Jenkins
+	  * Changed several asserts to warning or debug messages and return
+	  codes as appropriate.
 
-	  Change-Id: If46a93ba1282418d2803e3fd7869374da8b77ab5
+	  ASTERISK-27264
+	  Reported by: Daniel Heckl
 
-2017-03-09 11:05 +0000 [163e9e53dc]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I58daaa9d2938fa980857ab3ec41925ab5ff9c848
 
-	* app_queue: Handle the caller being redirected out of a queue bridge
+2017-09-19 05:22 +0000 [cad68137a7]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  A caller can leave the Queue() application after being bridged with a
-	  member in a few ways:
+	* res_config_pgsql: Fix removed support to previous for versions PostgreSQL 9.1
 
-	    * Caller or member hangup
-	    * Caller is transferred somewhere else (blind or atx)
-	    * Caller is externally redirected elsewhere
+	  In PostgreSQL 9.1 the backslash are string literals and not the escape
+	  of characters.
 
-	  The first 2 scenarios are currently handled by subscribing to stasis
-	  messages, but the 3rd is not explicitly covered. If a caller is
-	  redirected away from the Queue() application, the member who was last
-	  bridged with that caller will remain in an "In use" state until the
-	  caller hangs up.
+	  In previous issue ASTERISK_26057 was fixed the use of escape LIKE but the
+	  support for old version of Postgresql than 9.1 was dropped. The sentence
+	  before make was "ESCAPE '\'" but in version before than 9.1  need it to be
+	  as follow "ESCAPE '\\'".
 
-	  This patch adds handling of the caller leaving the queue via
-	  redirection. We monitor the caller-member bridge, and if the caller is
-	  the one that leaves, we treat it the same as we would a caller hangup.
+	  ASTERISK-27283
 
-	  ASTERISK-26400 #close
-	  Reported by: Etienne Lessard
+	  Change-Id: I96d9ee1ed7693ab17503cb36a9cd72847165f949
 
-	  Change-Id: Iba160907770de5a6c9efeffc9df5a13e9ea75334
+2017-09-15 09:43 +0000 [e666051d79]  Ben Ford <bford@digium.com>
 
-2017-03-15 08:44 +0000 [7612601964]  Joshua Colp <jcolp@digium.com>
+	* res_pjsip_session: Check for removed stream state.
 
-	* res_pjsip_endpoint_identifier_ip: Don't output error if no header_match.
+	  When a sip session is refreshed, the stream topology is looped
+	  through, checking each stream for compatible formats. This would
+	  cause a crash if the stream state was AST_STREAM_STATE_REMOVED,
+	  since the formats would never be set for this stream, causing
+	  a NULL value to be returned from ast_stream_get_formats. This
+	  commit adds a check for streams with removed states.
 
-	  This change ensures that if no header_match option is set on an
-	  identify an error message is not output stating the option is set
-	  to an invalid value.
+	  Also removed a stray semicolon.
 
-	  ASTERISK-26863
+	  Change-Id: Ic86f8b65a4a26a60885b28b8b1a0b22e1b471d42
 
-	  Change-Id: I239bc6d2319dd3da24ba96a38d4d6e9b5526d62a
+2017-09-19 05:44 +0000 [b6aa728a58]  George Joseph <gjoseph@digium.com>
 
-2017-03-14 08:49 +0000 [48447313b6]  Torrey Searle <torrey@voxbone.com>
+	* chan_pjsip: Ignore AST_CONTROL_STREAM_TOPOLOGY_CHANGED for now
 
-	* res/res_pjsip_refer: call xfer w/o extension
+	  chan_pjsip_indicate was missing a case for the recently added
+	  AST_CONTROL_STREAM_TOPOLOGY_CHANGED condition and was returning an
+	  error and causing the call to be hung up instead of just ignoring
+	  it.
 
-	  When transfering to a URI without an extension, ensure that the
-	  s extension of the dialplan is entered
+	  ASTERISK-27260
+	  Reported by: Daniel Heckl
 
-	  ASTERISK-26869 #close
+	  Change-Id: I4fecbb00a0b8a853da85155065c1a6bddf235e80
 
-	  Change-Id: I07403df66cf93f09e00a40ab5b41bfc6f72b1525
+2017-09-07 04:41 +0000 [6b7d5671d1]  Jean Aunis <jean.aunis@prescom.fr>
 
-2017-03-14 16:16 +0000 [9fd9b39e8b]  Richard Mudgett <rmudgett@digium.com>
+	* bridge : Fix one-way direct-media when early bridging with native_rtp
 
-	* pbx.c: Fix crash from malformed exten pattern.
+	  When two channels were early bridged in a native_rtp bridge, the RTP description
+	  on one side was not updated when the other side answered.
+	  This patch forbids non-answered channels to enter a native_rtp bridge, and
+	  triggers a bridge reconfiguration when an ANSWER frame is received.
 
-	  Forgetting to indicate an exten is a pattern can cause a crash if the
-	  "pattern" has a character set range.  e.g., "9999[3-5]" The crash is due
-	  to a buffer overwrite because the '-' exten eye-candy wasn't removed as
-	  expected and overran the allocated space.
+	  ASTERISK-27257
 
-	  The buffer overwrite is fixed two ways in this patch.
+	  Change-Id: If1aaee1b4ed9658a1aa91ab715ee0a6413b878df
 
-	  1) Fix ext_strncpy() to distinguish between pattern and non-pattern
-	  extens.  Now '-' characters are removed when they are eye-candy and not
-	  when they are part of a pattern character set.  Since the function is
-	  private to pbx.c, the return value now returns the number of bytes written
-	  to the destination buffer instead of the strlen() of the final buffer so
-	  the callers that care don't need to add one.
+2017-09-18 09:51 +0000 [1e4c1cec7f]  Alexander Traud <pabstraud@compuserve.com>
 
-	  2) Fix callers to ext_strncpy() to supply the correct available buffer
-	  size of the destination buffer.
+	* res_srtp: lower log level of auth failures
 
-	  ASTERISK-26668
+	  Previously, sRTP authentication failures were reported on log level WARNING.
+	  When such failures happen, each RT(C)P packet is affected, spamming the log.
+	  Now, those failures are reported at log level VERBOSE 2. Furthermore, the
+	  amount is further reduced (previously all two seconds, now all three seconds).
+	  Additionally, the new log entry informs whether media (RTP) or statistics (RTCP)
+	  are affected.
 
-	  Change-Id: I555d97411140e47e0522684062d174fbe32aa84a
+	  ASTERISK-16898 #close
 
-2017-03-14 16:51 +0000 [5389666d6f]  Richard Begg <asterisk@meric.id.au>
+	  Change-Id: I6c98d46b711f56e08655abeb01c951ab8e8d7fa0
 
-	* chan_iax2: Reload of iax peer results in loss of host address/port
+2017-09-19 10:38 +0000 [b748038230]  George Joseph <gjoseph@digium.com>
 
-	  When using a non-dynamic peer address, build_peer() invalidates the
-	  peer address structure by setting the address family to unspecified.
-	  However, if dnsmgr is enabled, the subsequent call to ast_dnsmgr_lookup()
-	  will not amend the peer address if the cache is still valid, resulting
-	  in peer connectivity failures.
-	  To fix this, we call ast_dnsmgr_refresh() instead.
+	* res_pjsip_pubsub:  Check for Content-Type header in rx_notify_request
 
-	  ASTERISK-26865
+	  pubsub_on_rx_notify_request wasn't checking for a null
+	  Content-Type header before checking that it was
+	  application/simple-message-summary.
 
-	  Change-Id: Id8a89a2f771ebbaf32255a35fe596a6dcb97a082
+	  ASTERISK-27279
+	  Reported by: Ross Beer
 
-2017-03-14 15:12 +0000 [658d59c683]  Matt Jordan <mjordan@digium.com>
+	  Change-Id: Iec2a6c4d2e74af37ff779ecc9fd35644c5c4ea52
 
-	* configure: Don't use the progress bar with curl when downloading to stdout
+2017-09-19 09:34 +0000 [a5f1d58fe1]  David J. Pryke <david+extra.asterisk@pryke.us>
 
-	  In some scenarios, such as when there may not be a terminal (such as
-	  inside a Docker container), curl will apparently direct the progress bar
-	  to stdout. This can cause extra data to be appended to a file curl'd
-	  down to stdout, resulting in md5 verification failures.
+	* chan_sip: Expose read-only access to the full SIP INVITE Request-URI
 
-	  This patch removes the progress bar, and tells curl to download the file
-	  silently.
+	  Provide a way to get the contents of the the Request URI from the initial SIP
+	  INVITE in dial plan function call. (In this case "${CHANNEL(ruri)}")
 
-	  ASTERISK-26872 #close
+	  ASTERISK-27278
+	  Reported by: David J. Pryke
+	  Tested by: David J. Pryke
 
-	  Change-Id: Ie860b020f627d4372b3e7ce9453de5faafeebe6c
+	  Change-Id: I1dd4d6988eed1b6c98a9701e0e833a15ef0dac3e
 
-2017-03-14 07:50 +0000 [b3c2c996f1]  Matt Jordan <mjordan@digium.com>
+2017-09-19 07:53 +0000 [6fd3db51e8]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_endpoint_identifier_ip: Add an option to match requests by header
+	* app_confbridge: Only create a channel that records audio.
 
-	  This patch adds a new features to the endpoint identifier module,
-	  'match_header'. When set, inbound requests are matched by a provided SIP
-	  header: value pair. This option works in conjunction with the existing
-	  'match' configuration option, such that if any 'match*' attribute
-	  matches an inbound request, the request is associated with the specified
-	  endpoint.
+	  This change makes it so that the conference recorder channel
+	  that is created only contains audio formats and an audio stream.
+	  This is because the underlying application used by ConfBridge to
+	  record, MixMonitor, only allows recording audio.
 
-	  Since this module now identifies by more than just IP address,
-	  appropriate renaming of the module and/or variables can be done in a
-	  non-release branch.
+	  Having additional streams (and in particular a video stream) can
+	  result in clients needlessly renegotiating to add a video stream
+	  that will never receive video.
 
-	  ASTERISK-26863 #close
+	  Change-Id: I89d38aedc9205eca7741d5435e73e73bb9de97a0
 
-	  Change-Id: Icfc14835c962f92e35e67bbdb235cf0589de5453
-	  (cherry picked from commit 30f52d79d7fc9ab0b628bef2b61ea515413795a2)
+2017-09-19 06:34 +0000 [56f0d5fc0f]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-2017-03-14 09:55 +0000 [51985565ef]  Matt Jordan <mjordan@digium.com>
+	* res_config_pgsql: Add missing \n in debug log and update copyright year
 
-	* configs/samples/hep.conf.sample: Clarify how the HEP stack works
+	  Change-Id: I4ba338ecbdecc6a814a902eddc4121c8ef3cda58
 
-	  This patch updates the documenation in hep.conf.sample to better specify
-	  how the various HEP modules interact.
+2017-09-13 14:14 +0000 [55567ee1d8]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26717 #close
+	* res_calendar: Plug memory leak and micro-optimization
 
-	  Change-Id: I337fb742a89e3ec5edc7fc7a7a0295218d841124
+	  ast_variables_destroy is NULL safe, so there is no need to check its
+	  argument before passing it.
 
-2017-03-14 09:59 +0000 [f9b791debe]  Roman Bedros (License 6842)
+	  ASTERISK-25524 #close
+	  Reported by: Jesper
 
-	* funcs/func_devstate: Remove new line in Device field of during module load
+	  Change-Id: Ib0f8057642e9d471960f1a79fd42e5a3ce587d3b
 
-	  During module loading of func_devstate, Asterisk emits the current
-	  device state of all Custom device states currently stored in the AstDB.
-	  This was erroneously including a new line character ('\n') to the end of
-	  the device state, causing two new lines to be emitted in
-	  DeviceStateChange AMI events.
+2017-09-13 03:46 +0000 [1199927fc0]  alex <alexandr.revin@gmail.com>
 
-	  Note that this only happened for those device state changes that
-	  occurred during startup. Regular device state changes for Custom device
-	  states are handled elsewhere, and did not have the newline.
+	* cdr_mysql.c: Apply cdrzone to start and answer
 
-	  ASTERISK-26643 #close
-	  Reported by: Roman Bedros
-	  Tested by: Matt Jordan
-	  patches:
-	    ami_devstate.diff uploaded by Roman Bedros (License 6842)
+	  Change-Id: I7de0a5adc89824a5f2b696fc22c80fc22dff36b0
 
-	  Change-Id: I1f4c02fc79c448d43bf725f5039c83d9611d7d93
+2017-08-25 17:01 +0000 [087f667ab1]  Richard Mudgett <rmudgett@digium.com>
 
-2017-03-14 09:37 +0000 [216e28aa95]  Matt Jordan <mjordan@digium.com>
+	* AST-2017-008: Improve RTP and RTCP packet processing.
 
-	* main/stasis_cache: Demote the ERROR message when removing a nonexistent item
+	  Validate RTCP packets before processing them.
 
-	  This patch demotes the ERROR message that is displayed when a
-	  nonexistent item is removed from the Stasis cache. The genesis of this
-	  demotion is due to chan_sip's realtime peers and their interaction with
-	  Asterisk's core ast_endpoint code, but ostensibly it could happen from
-	  other channel drivers as well.
+	  * Validate that the received packet is of a minimum length and apply the
+	  RFC3550 RTCP packet validation checks.
 
-	  Since Mark Michelson already did an excellent job of explaining on this
-	  issue, it is quoted here for posterity:
+	  * Fixed potentially reading garbage beyond the received RTCP record data.
 
-	  "Internally, when a realtime peer is retrieved, Asterisk creates an
-	  ast_endpoint structure. When that peer is destroyed, the ast_endpoint is
-	  destroyed as well. Part of the destruction of the ast_endpoint involves
-	  clearing the Stasis cache of all information about that endpoint. The
-	  problem here is that the act of creating the ast_endpoint is not enough
-	  to actually put any information in the Stasis cache. Instead, something
-	  has to happen, such as a state change, in order for the Stasis cache to
-	  have any information about that endpoint. When a device registers,
-	  chan_sip creates an ast_endpoint structure, processes the REGISTER, and
-	  then destroys the ast_endpoint. When the ast_endpoint is destroyed,
-	  there is nothing to destroy in the Stasis cache, so an error message is
-	  emitted. When you use rtcachefriends, ast_endpoint structures persist
-	  for the lifetime of the module and so you do not see this error
-	  message."
+	  * Fixed rtp->themssrc only being set once when the remote could change
+	  the SSRC.  We would effectively stop handling the RTCP statistic records.
 
-	  ASTERISK-25237 #close
+	  * Fixed rtp->themssrc to not treat a zero value as special by adding
+	  rtp->themssrc_valid to indicate if rtp->themssrc is available.
 
-	  Change-Id: I53cebc6b4a897a1ab9564182b75c177780feff70
+	  ASTERISK-27274
 
-2017-03-12 09:21 +0000 [c8d1b915d7]  Joshua Colp <jcolp@digium.com>
+	  Make strict RTP learning more flexible.
 
-	* chan_pjsip: Don't assume a session will have a channel.
+	  Direct media can cause strict RTP to attempt to learn a remote address
+	  again before it has had a chance to learn the remote address the first
+	  time.  Because of the rapid relearn requests, strict RTP could latch onto
+	  the first remote address and fail to latch onto the direct media remote
+	  address.  As a result, you have one way audio until the call is placed on
+	  and off hold.
 
-	  When querying for PJSIP specific information using the dialplan
-	  function CHANNEL() it is possible that the underlying session
-	  will no longer have a channel associated with it. This is
-	  most likely to occur when the RTCP HEP module attempts to get
-	  the channel name. If this happens then a crash will occur.
+	  The new algorithm learns remote addresses for a set time (1.5 seconds)
+	  before locking the remote address.  In addition, we must see a configured
+	  number of remote packets from the same address in a row before switching.
 
-	  This change just adds a check that the channel exists on the
-	  session before querying it.
+	  * Fixed strict RTP learning from always accepting the first new address
+	  packet as the new stream.
 
-	  ASTERISK-26857
+	  * Fixed strict RTP to initialize the expected sequence number with the
+	  last received sequence number instead of the last transmitted sequence
+	  number.
 
-	  Change-Id: I113479cffff6ae64cf8ed089e9e1565223426f01
+	  * Fixed the predicted next sequence number calculation in
+	  rtp_learning_rtp_seq_update() to handle overflow.
 
-2017-03-13 10:45 +0000 [6d1eb880c2]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-27252
 
-	* menuselect: Add a new 'options' support type
+	  Change-Id: Ia2d3aa6e0f22906c25971e74f10027d96525f31c
 
-	  The Binaural Rendering patches in the master branch required
-	  menuselect to be updated with a new support type called 'option'.
-	  This allows binaural rendering to be turned on or off when
-	  bridge_softmix is built.  This patch backports the 'option'
-	  functionality to the 13 and 14 branches.
-
-	  Here's what it looks like in menuselect:
-
-	    [*] bridge_simple
-	    [*] bridge_softmix
-	        --- Module Options ---
-	    [ ] binaural_rendering_in_bridge_softmix
-
-	  To create an option for a module, you can create (or update) the
-	  menuselect-tree xml snippet in the directory where the module
-	  resides and add a member element with an 'option' support_level.
-
-	  Example (abbreviated) from bridges/bridges.xml:
-
-	  <member name="binaural_rendering_in_bridge_softmix"
-	  	displayname="Enable binaural rendering in bridge_softmix"
-	  	remove_on_change="bridges/bridge_softmix.o bridges/bridge_softmix.so">
-	  	<support_level>option</support_level>
-	  	<depend>bridge_softmix</depend>
-	  	<depend>fftw3</depend>
-	  	<defaultenabled>no</defaultenabled>
-	  </member>
+2017-09-13 16:23 +0000 [d178f497d2]  George Joseph <gjoseph@digium.com>
 
-	  The 'name' will be added or removed from the MENUSELECT_<dir>
-	  make variable following the standard module "missing means yes"
-	  rules.
-
-	  Example (abbreviated) from bridges/Makefile:
-
-	  ifeq ($(findstring binaural_rendering,$(MENUSELECT_BRIDGES)),)
-	  bridge_softmix.o: _ASTCFLAGS+=-DBINAURAL_RENDERING
-	  bridge_softmix.so: LIBS+=$(FFTW3_LIB)
-	  endif
+	* res_pjsip:  Filter out non SIP(S) requests
 
-	  Change-Id: I66d23755ed6e81f8d439cad410f2ffa7c30f25ad
+	  Incoming requests with non sip(s) URIs in the Request, To, From
+	  or Contact URIs are now rejected with
+	  PJSIP_SC_UNSUPPORTED_URI_SCHEME (416).  This is performed in
+	  pjsip_message_filter (formerly pjsip_message_ip_updater) and is
+	  done at pjproject's "TRANSPORT" layer before a request can even
+	  reach the distributor.
 
-2017-03-10 20:29 +0000 [523de8eb8e]  gtjoseph <gjoseph@digium.com>
+	  URIs read by res_pjsip_outbound_publish from pjsip.conf are now
+	  also checked for both length and sip(s) scheme.  Those URIs read
+	  by outbound registration and aor were already being checked for
+	  scheme but their error messages needed to be updated to include
+	  scheme failure as well as length failure.
 
-	* pjproject_bundled:  Reduce the need for rebuilds
+	  Change-Id: Ibb2f9f1d2dc7549da562af4cbd9156c44ffdd460
 
-	  Bundled pjproject should now only rebuild if one of the menuselect
-	  "Compiler Flags" options changes.
+2017-09-14 07:54 +0000 [01f2220bec]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: If114a2e16b9e77af371a600d6a5e197bbf28fe43
+	* tcptls: Change error message to debug.
 
-2017-03-07 08:12 +0000 [d3ef833b3b]  Jean Aunis <jean.aunis@prescom.fr>
+	  The Websocket implementation will steal the underlying stream of
+	  TCP/TLS sessions. This results in an error message being output
+	  about a stream not being present when in reality this is actually
+	  fine.
 
-	* chan_sip: Call not cancelled after receiving a 422 response
+	  This change moves it to a debug message instead.
 
-	  When receiving a 422 response, the invitestate variable must be reset to
-	  INV_CALLING.
+	  Change-Id: I66cc639080b4b4599beadb4faa7d313f2721d094
 
-	  ASTERISK-26841
+2017-09-13 14:08 +0000 [d8112cd98b]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ia0502d6b02192664cefa4e75bafdd2645ce56099
+	* res_calendar: Various fixes
 
-2017-03-05 15:26 +0000 [67c989ce78]  Daniel Journo <dan@keshercommunications.com>
+	  * The way that we were looking at XML elements for CalDAV was extremely
+	    fragile, so use SAX2 for increased robustness.
 
-	* pjsip/cli_commands: pjsip show channelstats shows wrong codec
+	  * Don't complain about a 'channel' not be specified if autoreminder is
+	    not set. Assume that if 'channel' is not set, we don't want to be
+	    notified.
 
-	  * cli_commands.c Fixed CLI output
+	  * Fix some truncated CLI output in 'calendar show calendar' and make the
+	    'Autoreminder' description a bit more clear
 
-	  ASTERISK-26822 #close
+	  ASTERISK-24588 #close
+	  Reported by: Stefan Gofferje
 
-	  Change-Id: I3889ef6a8f6738fc312fab42db5efacd6e452b01
+	  ASTERISK-25523 #close
+	  Reported by: Jesper
 
-2017-03-07 07:37 +0000 [2a85888262]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I200d11afca6a47e7d97888f286977e2e69874b2c
 
-	* res_pjsip_transport_websocket: Add support for IPv6.
+2017-09-13 09:38 +0000 [eec0396395]  Sean Bright <sean.bright@gmail.com>
 
-	  This change adds a PJSIP patch (which has been contributed upstream)
-	  to allow the registration of IPv6 transport types.
+	* chan_rtp: Use μ-law by default instead of signed linear
 
-	  Using this the res_pjsip_transport_websocket module now registers
-	  an IPv6 Websocket transport and uses it for the corresponding
-	  traffic.
+	  Multicast/Unicast RTP do not use SDP so we need to use a format that
+	  cleanly maps to one of the static RTP payload types. Without this
+	  change, an Originate to a Multicast or Unicast channel without a format
+	  specified would produce no audio on the receiving device.
 
-	  ASTERISK-26685
+	  ASTERISK-21399 #close
+	  Reported by: Tzafrir Cohen
 
-	  Change-Id: Id1f9126f995b31dc38db8fdb58afd289b4ad1647
+	  Change-Id: I97e332b566e85da04b0004b9b0daae746cfca0e3
 
-2017-03-08 08:16 +0000 [bc6eeab822]  Daniel Journo <dan@keshercommunications.com>
+2017-09-11 05:46 +0000 [446d48fd49]  George Joseph <gjoseph@digium.com>
 
-	* app_voicemail: Cannot set fromstring on a per-mailbox basis
+	* res_pjsip:  Add handling for incoming unsolicited MWI NOTIFY
 
-	  * apps/app_voicemail.c fromstring field added to mailbox which will
-	  override the global fromstring if set.
+	  A new endpoint parameter "incoming_mwi_mailbox" allows Asterisk to
+	  receive unsolicited MWI NOTIFY requests and make them available to
+	  other modules via the stasis message bus.
 
-	  ASTERISK-24562 #close
+	  res_pjsip_pubsub has a new handler "pubsub_on_rx_mwi_notify_request"
+	  that parses a simple-message-summary body and, if
+	  endpoint->incoming_mwi_account is set, calls ast_publish_mwi_state
+	  with the voice-message counts from the message.
 
-	  Change-Id: I5e90e3a1ec2b2d5340b49a0db825e4bbb158b2fe
+	  Change-Id: I08bae3d16e77af48fcccc2c936acce8fc0ef0f3c
 
-2017-03-06 15:54 +0000 [d9972423d1]  Daniel Journo <dan@keshercommunications.com>
+2017-09-08 21:41 +0000 [4889574ff5]  Richard Mudgett <rmudgett@digium.com>
 
-	* Saynumber is trying to get "and" from "digits/" subfolder
+	* res_rtp_asterisk.c: Add doxygen to RTCP payload types.
 
-	  * say.c Changed 'digits/and' to 'vm-and' for en_GB
+	  Change-Id: I3f20ce428777cc4ce9c13b2f808d29ff8c873998
 
-	  ASTERISK-26598 #close
+2017-09-11 05:52 +0000 [f9bad3bd61]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: If1b713e5daea6f952b339f139178d292a6c4fcfe
+	* alembic:  Fix typo in add_auto_info_to_endpoint_dtmf_mode
 
-2017-03-06 13:15 +0000 [77901a58ca]  Sean Bright <sean.bright@gmail.com>
+	  The downgrade function was missing "_v2" at the end of the
+	  alter column type.
 
-	* pbx_spool: Gracefully handle long lines in call files
+	  Change-Id: Iaa9bcef48d6f3590ce07a61342d8e66f00263d8e
 
-	  Per the linked issue, we aren't checking the buffer filled by fgets()
-	  to determine if it contains a newline, so we will fail to correctly
-	  parse the trailing portion of a long line.
+2017-09-10 06:17 +0000 [680aba21ec]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  This patch increases the buffer size from 256 to 1024, and skips any
-	  line that exceeds that length, logging a warning in the process.
+	* res/res_pjsip: Fix localnet checks in pjsip, part 2.
 
-	  ASTERISK-17067 #close
-	  Reported by: Dave Olszewski
+	  In 45744fc53, I mistakenly broke SDP media address rewriting by
+	  misinterpreting which address was checked in the localnet comparison.
 
-	  Change-Id: I51bcf270c1b4347ba05b43f18dc2094c76f5d7b0
+	  Instead of checking the remote peer address to decide whether we need
+	  media address rewriting, we check our local media address: if it's
+	  local, then we rewrite. This feels awkward, but works and even made
+	  directmedia work properly if you set local_net. (For the record: for
+	  local peers, the SDP media rewrite code is not called, so the
+	  comparison does no harm there.)
 
-2017-03-02 21:27 +0000 [4271c700f7]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27248 #close
 
-	* core: Cleanup ast_get_hint() usage.
+	  Change-Id: I566be1c33f4d0a689567d451ed46bab9c3861d4f
 
-	  * manager.c:manager_state_cb() Fix potential use of uninitialized hint[]
-	  if a hint does not exist for the requested extension.  Ran into this when
-	  developing a testsuite test.  The AMI event ExtensionStatus came out with
-	  the hint header value containing garbage.  The AMI event PresenceStatus
-	  also had the same issue.
+2017-09-08 21:19 +0000 [c8d53a1638]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  * manager.c:action_extensionstate() no need to completely initialize the
-	  hint[].  Only initialize the first element.
+	* cdr_pgsql: Refactor magic number by definition for version
 
-	  * pbx.c:ast_add_hint() Remove unnecessary assignment.
+	  Change-Id: I43f25976aa3069793ddbe0086833965a6fb0a518
 
-	  * chan_sip.c: Eliminate an unneeded hint[] local variable.  We only care
-	  about the return value of ast_get_hint() there.
+2017-09-05 11:13 +0000 [e9a81157ac]  Florian Floimair <f.floimair@commend.com>
 
-	  Change-Id: Ia9a8786f01f93f1f917200f0a50bead0319af97b
+	* alembic: Add support for MS-SQL
 
-2017-02-16 04:22 +0000 [e510595c86]  Jørgen H <asterisk.org@hovland.cx>
+	  MS-SQL has no native Enum-type support and therefore
+	  needs to work with constraints.
+	  Since these constraints need unique names the suggested approach
+	  referenced in the following alembic documentation has been applied:
+	  http://bit.ly/2x9r8pb
 
-	* res_pjsip WebRTC/websockets: Fix usage of WS vs WSS.
+	  ASTERISK-27255 #close
 
-	  According to the RFC[1] WSS should only be used in the Via header
-	  for secure Websockets.
+	  Change-Id: I8b579750dae0c549f1103ee50172644afb9b2f95
 
-	  * Use WSS in Via for secure transport.
+2017-09-05 07:31 +0000 [525f84bb35]  Jacek Konieczny <j.konieczny@eggsoft.pl>
 
-	  * Only register one transport with the WS name because it would be
-	  ambiguous.  Outgoing requests may try to find the transport by name and
-	  pjproject only finds the first one registered.  This may mess up unsecure
-	  websockets but the impact should be minimal.  Firefox and Chrome do not
-	  support anything other than secure websockets anymore.
+	* func_cdr: honour 'u' flag on dummy channel
 
-	  * Added and updated some debug messages concerning websockets.
+	  Fixes ${CDR(...,u)} when used in cdr_custom.conf
 
-	  * security_events.c: Relax case restriction when determining security
-	  transport type.
+	  ASTERISK-27165 #close
 
-	  * The res_pjsip_nat module has been updated to not touch the transport
-	  on Websocket originating messages.
+	  Change-Id: Ia4e0b6ba93e03d27886354c279737790e2cd6a83
 
-	  [1] https://tools.ietf.org/html/rfc7118
+2017-09-06 10:50 +0000 [2b3f903e6f]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26796 #close
+	* app_waitforsilence: Cleanup & don't treat missing frames as 'noise'
 
-	  Change-Id: Ie3a0fb1a41101a4c1e49d875a8aa87b189e7ab12
+	  * WaitForSilence completes successfully if it receives no media in the
+	    specified timeout, but when acting as WaitForNoise that logic needs
+	    to be reversed.
 
-2017-03-01 07:23 +0000 [76971d4c4a]  Sean Bright <sean.bright@gmail.com>
+	  * Use standard argument parsing macros and add some error checking for
+	    invalid values.
 
-	* res_config_pgsql: Make 'require' return consistent with other backends
+	  * The documentation indicated that the first argument to both
+	    WaitForSilence and WaitForNoise was required when it was not. Update
+	    the documentation to reflect that.
 
-	  res_config_pgsql should match the behavior of other realtime backend
-	  drivers so that queue_log can disable adaptive logging.
+	  * Wrap up some behavior in structs to avoid boolean checks all over the
+	    place.
 
-	  ASTERISK-25628 #close
-	  Reported by: Dmitry Wagin
+	  ASTERISK-24066 #close
+	  Reported by: M vd S
 
-	  Change-Id: Ic1fb1600c7ce10fdfb1bcdc43c5576b7e0014372
+	  Change-Id: I01d40adc5b63342bb5018a1bea2081a0aa191ef9
 
-2017-02-28 09:41 +0000 [fa8f6c2fc4]  Sean Bright <sean.bright@gmail.com>
+2017-09-06 16:05 +0000 [5553644284]  Scott Griepentrog <scott@griepentrog.com>
 
-	* res_config_pgsql: Release table locks where appropriate
+	* chan_sip: when getting sip pvt return failure if not found
 
-	  The find_table() functions NULL or a locked table pointer. We are
-	  not consistently calling release_table() in failure paths.
+	  In handle_request_invite, when processing a pickup, a call
+	  is made to get_sip_pvt_from_replaces to locate the pvt for
+	  the subscription. The pvt is assumed to be valid when zero
+	  is returned indicating no error, and is dereferenced which
+	  can cause a crash if it was not found.
 
-	  Change-Id: I6f665b455799c84b036e5b34904b82b05eab9544
+	  This change checks the not found case and returns -1 which
+	  allows the calling code to fail appropriately.
 
-2017-02-28 05:41 +0000 [5b34b751a0]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  ASTERISK-27217 #close
+	  Reported-by: Bryan Walters
 
-	* pjsip.conf.sample: user_agent: not a specific version
+	  Change-Id: I6bee92b8b8b85fcac3fd66f8c00ab18bc1765612
 
-	  Use the description of useragent from sip.conf here.
+2017-09-06 13:38 +0000 [23571f31ac]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26825 #close
+	* stasis/control.c: Fix set_interval_hook() ref leak.
 
-	  Change-Id: I5b33a4aaa0ae1d793289d05e3bc09521affbf755
+	  Change-Id: Ia0edb7dc0dbbb879c079ff7000f1b722d86ce7dc
 
-2017-02-27 20:07 +0000 [8e6ecdade2]  gtjoseph <gjoseph@digium.com>
+2017-09-01 05:17 +0000 [94091c7b96]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_pubsub:  Remove unneeded endpoint unref
+	* stasis/control:  Fix possible deadlock with swap channel
 
-	  When a subscription was being recreated and the endpoint wasn't
-	  found, we were trying to unref the endpoint.  This was causing
-	  FRACKs.  Removed the unref.
+	  If an error occurs during a bridge impart it's possible that
+	  the "bridge_after" callback might try to run before
+	  control_swap_channel_in_bridge has been signalled to continue.
+	  Since control_swap_channel_in_bridge is holding the control lock
+	  and the callback needs it, a deadlock will occur.
 
-	  ASTERISK-26823 #close
+	  * control_swap_channel_in_bridge now only holds the control
+	    lock while it's actually modifying the control structure and
+	    releases it while the bridge impart is running.
+	  * bridge_after_cb is now tolerant of impart failures.
 
-	  Change-Id: If86d2aecff8fe853c7f38a1bfde721fcef3cd164
+	  Change-Id: Ifd239aa93955b3eb475521f61e284fcb0da2c3b3
 
-2017-02-16 04:16 +0000 [0595c31da7]  Jørgen H <asterisk.org@hovland.cx>
+2017-09-06 05:23 +0000 [67a2ca31f5]  Vitezslav Novy <a1@vnovy.net>
 
-	* res_pjsip: Fix crash when contact has no status
+	* chan_sip: Do not change IP address in SDP origin line (o=) in SIP reINVITE
 
-	  This change fixes an assumption in res_pjsip that a contact will
-	  always have a status. There is a race condition where this is
-	  not true and would crash. The status will now be unknown when
-	  this situation occurs.
+	  If directmedia=yes is configured, when call is answered, Asterisk sends reINVITE
+	  to both parties to set up media path directly between the endpoints.
+	  In this reINVITE msg SDP origin line (o=) contains IP address of endpoint
+	  instead of IP of asterisk. This behavior violates RFC3264, sec 8:
+	  "When issuing an offer that modifies the session,
+	  the "o=" line of the new SDP MUST be identical to that in the
+	  previous SDP, except that the version in the origin field MUST
+	  increment by one from the previous SDP."
+	  This patch assures IP address of Asterisk is always sent in
+	  SDP origin line.
 
-	  ASTERISK-26623 #close
+	  ASTERISK-17540
+	  Reported by:  saghul
 
-	  Change-Id: Id52d3ca4d788562d236da49990a319118f8d22b5
+	  Change-Id: I533a047490c43dcff32eeca8378b2ba02345b64e
 
-2017-02-21 18:06 +0000 [c07bcca87e]  gtjoseph <gjoseph@digium.com>
+2017-09-06 07:54 +0000 [0cbb17ce8f]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_outbound_registration:  Subscribe to network change events
+	* alembic: Fix enum creation for dtls_fingerprint
 
-	  Outbound registration now subscribes to network change events
-	  published by res_stun_monitor and refreshes all registrations
-	  when an event happens.
+	  Change-Id: Ic061c5066a146616a68376881c7e4cf6d6e7e7db
 
-	  The 'pjsip send (un)register' CLI commands were updated to accept
-	  '*all' as an argument to operate on all registrations.
+2017-09-05 11:08 +0000 [a133c5cc53]  Florian Floimair <f.floimair@commend.com>
 
-	  The 'PJSIP(Un)Register' AMI commands were also updated to
-	  accept '*all'.
+	* alembic: fix erroneous commit for add_prune_on_boot
 
-	  ASTERISK-26808 #close
+	  Added include for postgresql ENUM type and
+	  redefined values in the same way as in the
+	  other migration scripts.
 
-	  Change-Id: Iad58a9e0aa5d340477fca200bf293187a6ca5a25
+	  ASTERISK-27254 #close
 
-2017-02-26 10:09 +0000 [d91f61f0b5]  Vitezslav Novy <a1@vnovy.net>
+	  Change-Id: Id667304cdf3891b1c2f7d35fab3e2a84026159fa
 
-	* chan_sip: Allow DTLS to be disabled when reloading.
+2017-09-06 03:02 +0000 [2d395793b7]  Alexander Traud <pabstraud@compuserve.com>
 
-	  This change fixes a problem where removing the DTLS configuration
-	  options and reloading would not disable DTLS. This occurred
-	  because the DTLS configuration was not reset to an unconfigured
-	  state on reload.
+	* res_srtp: Add support for libsrtp2.1.
 
-	  ASTERISK-26313
+	  Asterisk is able to use libSRTP 2.0.x. However since libSRTP 2.1.x, the macro
+	  SRTP_AES_ICM got renamed to SRTP_AES_ICM_128. Beside to still compile with
+	  previous versions of libSRTP, this change allows libSRTP 2.1.x as well.
 
-	  Change-Id: I10952709cc4a7727fb50534b042bce9d64894b39
+	  ASTERISK-27253 #close
 
-2017-02-27 12:25 +0000 [3d2c119778]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I2e6eb3c3bc844fee8a624060a2eb6f182dc70315
 
-	* build:  Warn if asterisk is installed in both 32 and 64 bit sys dirs
+2017-09-05 09:35 +0000 [bfc29de3ea]  Ben Ford <bford@digium.com>
 
-	  ... and clean them both up on uninstall.
+	* chan_pjsip: Suppress frame warnings.
 
-	  We've fixed the issue where 'make install' was installing to
-	  /usr/lib on 64-bit systems that use /usr/lib64.  Now we need
-	  to clean up the remnants in /usr/lib.
+	  When rtp_keepalive is on for a PJSIP endpoint dialing to another
+	  Asterisk instance also using PJSIP, Asterisk will continue to print
+	  warning messages about not being able to send frames of a certain
+	  type. This suppresses that warning message.
 
-	  * 'make install' now prints a warning if DESTDIR/ASTLIBDIR
-	    contains 'lib64' and libasterisk* shared libraries or modules
-	    are also found in DESTDIR/ASTLIBDIR with 'lib64' transformed
-	    to 'lib'.
+	  Change-Id: I0332a05519d7bda9cacfa26d433909ff1909be67
 
-	  * 'make uninstall' ALWAYS cleans up both DESTDIR/ASTLIBDIR and
-	    DESTDIR/ASTLIBDIR with 'lib64' transformed to 'lib'.
+2017-09-05 10:05 +0000 [c3a6c8fd2d]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26705
+	* formats: Restore previous fread() behavior
 
-	  Change-Id: I6edddeb3c07a51e7c7ba7cac3c05e4bf3ec3f01f
+	  Some formats are able to handle short reads while others are not, so
+	  restore the previous behavior for the format modules so that we don't
+	  have spurious errors when playing back files.
 
-2017-02-27 07:02 +0000 [eac818801b]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27232 #close
+	  Reported by: Jens T.
 
-	* bridge_native_rtp: Handle case where channel joins already suspended.
+	  Change-Id: Iab7f52b25a394f277566c8a2a4b15a692280a300
 
-	  The bridge_native_rtp module did not properly handle the case where
-	  a smart bridge operation occurs while a channel is suspended. In this
-	  scenario the module would incorrectly set up local or remote RTP
-	  bridging despite the media having to flow through Asterisk. The remote
-	  endpoint would see two media streams and experience wonky audio.
+2017-09-05 09:16 +0000 [f856d9b42b]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  The module has been changed so that it ensures both channels are
-	  not suspended when performing the native RTP bridging and this
-	  requirement has been documented in the bridge technology.
+	* res/res_pjsip: Standardize/fix localnet checks across pjsip.
 
-	  ASTERISK-26781
+	  In 2dee95cc (ASTERISK-27024) and 776ffd77 (ASTERISK-26879) there was
+	  confusion about whether the transport_state->localnet ACL has ALLOW or
+	  DENY semantics.
 
-	  Change-Id: Id4022d73ace837d4a293106445e3ade10dbc7c7c
+	  For the record: the localnet has DENY semantics, meaning that "not in
+	  the list" means ALLOW, and the local nets are in the list.
 
-2017-02-24 11:49 +0000 [d49af061bc]  Joshua Colp <jcolp@digium.com>
+	  Therefore, checks like this look wrong, but are right:
 
-	* config: Improve documentation and behavior of outbound_proxy option.
+	      /* See if where we are sending this request is local or not, and if
+	         not that we can get a Contact URI to modify */
+	      if (ast_apply_ha(transport_state->localnet, &addr) != AST_SENSE_ALLOW) {
+	          ast_debug(5, "Request is being sent to local address, "
+	                       "skipping NAT manipulation\n");
 
-	  This change updates the documentation for the outbound_proxy option
-	  to ensure it is consistently stated that a full SIP URI must be
-	  provided for the option.
+	  (In the list == localnet == DENY == skip NAT manipulation.)
 
-	  The res_pjsip_outbound_registration module has also been changed so
-	  that the provided outbound_proxy value is checked to ensure it is a
-	  URI and if not an error is output stating so.
+	  And conversely, other checks that looked right, were wrong.
 
-	  ASTERISK-26782
+	  This change adds two macro's to reduce the confusion and uses those
+	  instead:
 
-	  Change-Id: I6c239a32274846fd44e65b44ad9bf6373479b593
+	      ast_sip_transport_is_nonlocal(transport_state, addr)
+	      ast_sip_transport_is_local(transport_state, addr)
 
-2017-02-09 18:05 +0000 [9c05ddbddd]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-27248 #close
 
-	* pjproject_bundled: Update for pjproject 2.6
+	  Change-Id: Ie7767519eb5a822c4848e531a53c0fd054fae934
 
-	   * Removed all 2.5.5 functional patches.
-	   * Updated usages of pj_release_pool to be "safe".
-	   * Updated configure options to disable webrtc.
-	   * Updated config_site.h to disable webrtc in pjmedia.
-	   * Added Richard Mudgett's recent resolver patches.
+2017-09-05 08:39 +0000 [68bcfccd52]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ib400cc4dfca68b3d07ce14d314e829bfddc252c7
+	* res_pjsip_session: Preserve stream name during renegotiation.
 
-2017-02-23 15:49 +0000 [bee55aaf2c]  gtjoseph <gjoseph@digium.com>
+	  Stream names within Asterisk can have meaning so when an externally
+	  initiated renegotiation occurs we need to preserve the name of
+	  the stream if it already exists.
 
-	* build: Execute ldconfig to build cache. (take two)
+	  Change-Id: I29f50d0cc7f3238287d6d647777e76e1bdf8c596
 
-	  On some platforms a multiarch approach is used for libraries.
-	  The build system does not take this into account and still
-	  places libraries into the lib directory if no --libdir is
-	  specified to configure. On initial startup this results in
-	  libasteriskssl.so not being found, as it is not in the multiarch
-	  lib directory.  To make matters worse, options were being passed
-	  to ldconfig on both Linux and FreeBSD that actually prevented
-	  the rebuild of the cache.
+2017-09-05 07:50 +0000 [0ec95515f3]  George Joseph <gjoseph@digium.com>
 
-	   * Fedora has a /usr/share/config.site that automatically tells
-	     autoconf to use /usr/lib64 but CentOS does not. This logic was
-	     copied to configure.ac and modified so systems like Ubuntu,
-	     which still use /usr/lib for 64-bit systems, aren't affected.
+	* res_calendar*, res_smdi: Move to "extended" support
 
-	  Now that we have them in the correct directory...
+	  Change-Id: I31eee8be30c6b0fc3dadb31111dd47742da8892d
 
-	  In order for the system loader to find libasteriskssl and
-	  libasteriskpj, one of 3 things has to happen...
+2017-09-05 05:23 +0000 [9b3f6d26bd]  George Joseph <gjoseph@digium.com>
 
-	    - The linker cache must be rebuilt including the directory
-	      where the libasterisk* libraries were installed.  Only root
-	      can rebuild the cache.  This was busted.
-	    - We have to link the asterisk binary with an rpath pointing
-	      to the directrory where the libasterisk* libraries were
-	      installed.  This makes things very complicated and will happen
-	      over the collective dead bodies of everyone who's had to
-	      package a distribution with an rpath.
-	    - Finally, you can start asterisk with LD_LIBRARY_PATH set to the
-	      directrory where the libasterisk* libraries were installed.
+	* res_pjsip_t38:  Make t38_reinvite_response_cb tolerant of NULL channel
 
-	  There are no other options. So...
+	  t38_reinvite_response_cb can get called by res_pjsip_session's
+	  session_inv_on_tsx_state_changed in situations where session->channel
+	  is NULL.  If it is, the ast_log warning segfaults because it tries
+	  to get the channel name from a NULL channel.
 
-	   * The invokation of ldconfig has been moved from main/Makefile
-	     to ASTTOPDIR/Makefile, the options have been removed, and
-	     DESTDIR/ASTLIBDIR appended.  If you aren't root, you will be
-	     warned after the "Asterisk Installation Compete" banner that
-	     you must re-run 'make install' as root, manually run
-	     'ldconfig DESTDIR/ASTLIBDIR' as root, or run asterisk with
-	     LD_LIBRARY_PATH.
+	  * Check session->channel and print "unknown channel" when it's NULL.
 
-	  ASTERISK-26705
+	  ASTERISK-27236
+	  Reported by: Ross Beer
 
-	  Change-Id: I2a64b7c33a7d3e9bde20f47e3d3ab771977af982
+	  Change-Id: I4326e288d36327f6c79ab52226d54905cdc87dc7
 
-2017-02-23 14:48 +0000 [da0cadd100]  Sean Bright <sean.bright@gmail.com>
+2017-09-01 16:17 +0000 [60b44d1e38]  Sean Bright <sean.bright@gmail.com>
 
-	* res_config_pgsql: Fix thread safety problems
+	* rtp_engine: Prevent possible double free with DTLS config
 
-	  * A missing AST_LIST_UNLOCK() in find_table()
+	  ASTERISK-27225 #close
+	  Reported by: Richard Kenner
 
-	  * The ESCAPE_STRING() macro uses pgsqlConn under the hood and we were
-	    not consistently locking before calling it.
+	  Change-Id: I097b81734ef730f8603c0b972909d212a3a5cf89
 
-	  * There were a handful of other places where pgsqlConn was accessed
-	    directly without appropriate locking.
+2017-09-01 13:15 +0000 [ef8eb9d11b]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Iea63f0728f76985a01e95b9912c3c5c6065836ed
+	* chan_ooh323: Fix confusing indentation warning
 
-2017-02-22 08:53 +0000 [f1963c5b8d]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27177 #close
+	  Reported by: Tzafrir Cohen
 
-	* res_config_ldap: Various code improvements
+	  Change-Id: I40311c404edb2302a7543ad5ca7a06b2a38f2d97
 
-	  The initial motivation for this patch was to properly handle memory
-	  allocation failures - we weren't checking the return values from the
-	  various LDAP library allocation functions.
+2017-09-01 09:51 +0000 [1bdbefbe76]  Sean Bright <sean.bright@gmail.com>
 
-	  In the process, because update_ldap() and update2_ldap() were
-	  substantially the same code, they've been consolidated.
+	* app_directory: Handle a NULL mailbox without crashing
 
-	  Change-Id: Iebcfe404177cc6860ee5087976fe97812221b822
+	  ASTERISK-27241 #close
+	  Reported by: David Moore
 
-2017-02-22 13:08 +0000 [1ec796ce18]  Michael L. Young <elgueromexicano@gmail.com>
+	  Change-Id: Ibbbca85517b04c315406ebfe3b6f7e0763daedc6
 
-	* build_tools:  Fix download_externals to allow the use of curl or wget
+2017-07-24 10:48 +0000 [f78f5278ff]  George Joseph <gjoseph@digium.com>
 
-	  Not sure if this is really a bug versus an improvement. I can see it being
-	  viewed as a bug though by some.
+	* pjsip_message_ip_updater:  Fix issue handling "tel" URIs
 
-	  The current build_tools/download_externals file depends on wget in order to
-	  download external modules.  The current build system is able to discover
-	  which tool to use for fetching remote files - either wget or curl.
+	  sanitize_tdata was assuming all URIs were SIP URIs so when a non
+	  SIP uri was in the From, To or Contact headers, the unconditional
+	  cast of a non-pjsip_sip_uri structure to pjsip_sip_uri caused
+	  a segfault when trying to access uri->other_param.
 
-	  This patch takes advantage of this capability by modifying the two calls to
-	  the wget binary to instead use what was discovered by the build system.
+	  * Added PJSIP_URI_SCHEME_IS_SIP(uri) || PJSIP_URI_SCHEME_IS_SIPS(uri)
+	    checks before attempting to cast or use the returned uri.
 
-	  ASTERISK-26812 #close
+	  ASTERISK-27152
+	  Reported-by: Ross Beer
 
-	  Change-Id: If9411a2554f009274d377445613ae91192d948a1
+	  Change-Id: Id380df790e6622c8058a96035f8b8f4aa0b8551f
 
-2017-02-22 11:13 +0000 [5c9c097d17]  Joshua Colp <jcolp@digium.com>
+2017-07-01 19:24 +0000 [1bf3dfffd7]  Corey Farrell <git@cfware.com>
 
-	* Revert "build: Execute ldconfig to build cache."
+	* AST-2017-006: Fix app_minivm application MinivmNotify command injection
 
-	  This reverts commit d90430953c508670a67de68de400fef44f5e9fba.
+	  An admin can configure app_minivm with an externnotify program to be run
+	  when a voicemail is received.  The app_minivm application MinivmNotify
+	  uses ast_safe_system() for this purpose which is vulnerable to command
+	  injection since the Caller-ID name and number values given to externnotify
+	  can come from an external untrusted source.
 
-	  Change-Id: I758fe7ea0408f83a6df8e1774310d69f482700f6
+	  * Add ast_safe_execvp() function.  This gives modules the ability to run
+	  external commands with greater safety compared to ast_safe_system().
+	  Specifically when some parameters are filled by untrusted sources the new
+	  function does not allow malicious input to break argument encoding.  This
+	  may be of particular concern where CALLERID(name) or CALLERID(num) may be
+	  used as a parameter to a script run by ast_safe_system() which could
+	  potentially allow arbitrary command execution.
 
-2017-02-21 10:47 +0000 [ca6d001144]  Sean Bright <sean.bright@gmail.com>
+	  * Changed app_minivm.c:run_externnotify() to use the new ast_safe_execvp()
+	  instead of ast_safe_system() to avoid command injection.
 
-	* pbx_realtime: Prevent premature extension matching
+	  * Document code injection potential from untrusted data sources for other
+	  shell commands that are under user control.
 
-	  The patterns provided by pbx_realtime were checked in the order in
-	  which they were returned from the realtime backend. If there was
-	  overlap between multiple patterns, the first one to correctly match was
-	  chosen even though it may not have been the best match.
+	  ASTERISK-27103
 
-	  We now sort the patterns descending by their length and compare in that
-	  order. There may be cases where this still results in a sub-optimal
-	  match, but this patch should improve the overall behavior.
+	  Change-Id: I7552472247a84cde24e1358aaf64af160107aef1
 
-	  ASTERISK-18271 #close
-	  Reported by: Charlie Smurthwaite
+2017-05-22 10:36 +0000 [7f2a60fb38]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I56d9ac15810eb1775966b669c3028e32cc7bd809
+	* res_rtp_asterisk: Only learn a new source in learn state.
 
-2017-02-21 15:09 +0000 [0654bf637c]  Peter Racz
+	  This change moves the logic which learns a new source address
+	  for RTP so it only occurs in the learning state. The learning
+	  state is entered on initial allocation of RTP or if we are
+	  told that the remote address for the media has changed. While
+	  in the learning state if we continue to receive media from
+	  the original source we restart the learning process. It is
+	  only once we receive a sufficient number of RTP packets from
+	  the new source that we will switch to it. Once this is done
+	  the closed state is entered where all packets that do not
+	  originate from the expected source are dropped.
 
-	* pbx_dundi: DUNDi weight parameter not processed correctly
+	  The learning process has also been improved to take into
+	  account the time between received packets so a flood of them
+	  while in the learning state does not cause media to be switched.
 
-	  The DUNDi weight field is not always converted from network byte order
-	  to host byte order. This can result in incorrect weight values and
-	  incorrect selection of DUNDi destinations.
+	  Finally RTCP now drops packets which are not for the learned
+	  SSRC if strict RTP is enabled.
 
-	  ASTERISK-18731 #close
-	  Reported by: Peter Racz
-	  Patches:
-	  	dundi_weight.patch (license #6290) patch uploaded by Peter Racz
+	  ASTERISK-27013
 
-	  Change-Id: Iba3e1a700ff539db57211a7bbc26f7b22ea9a1be
+	  Change-Id: I56a96e993700906355e79bc880ad9d4ad3ab129c
 
-2017-02-21 10:47 +0000 [d5522de597]  Sean Bright <sean.bright@gmail.com>
+2017-08-30 07:28 +0000 [5ba82cedc6]  Joshua Colp <jcolp@digium.com>
 
-	* realtime: Fix ast_load_realtime_multientry handling
+	* res_rtp_asterisk: Allow remote SSRC to change on an RTP instance.
 
-	  ast_load_realtime_multientry() returns an ast_config structure whose
-	  ast_categorys are keyed with the empty strings. Several modules were
-	  giving semantic meaning to the category names causing problems at
-	  runtime.
+	  When SDP renegotiation occurs it is possible for an RTP
+	  instance to be reused for a new stream, resulting in the remote
+	  SSRC changing if it is part of a bundle group. This change
+	  allows this and updates its mapping in the current bundle
+	  group.
 
-	  * app_directory: Treated the category name as the mailbox name, and
-	    would fail to direct calls to the appropriate extension after an
-	    entry was chosen.
+	  ASTERISK-27231
 
-	  * app_queue: Queues, queue members, and queue rules were all affected
-	    and needed to be updated.
+	  Change-Id: I6e3703974f236bc024c5dbe9bd43adae0c6fb490
 
-	  * pbx_realtime: Pattern matching would never succeed because the
-	    extension entered by the user was always compared to the empty
-	    string.
+2017-08-25 21:06 +0000 [71be8d5bbe]  Andre Nazario <samoied@users.sourceforge.net>
 
-	  Change-Id: Ie7e44986344b0b76ea8f6ddb5879f5040c6ca8a7
+	* chan_pjsip: Add tag info in CHANNEL function
 
-2017-02-21 08:56 +0000 [5eb7875243]  Sean Bright <sean.bright@gmail.com>
+	  Create local_tag and remote_tag in CHANNEL info to get tag from From and
+	  To headers of a SIP dialog.
 
-	* realtime: Centralize some common realtime backend code
+	  ASTERISK-27220
 
-	  All of the realtime backends create artificial ast_categorys to pass
-	  back into the core as query results. These categories have no filename
-	  or line number information associated with them and the backends differ
-	  slightly on how they create them. So create a couple helper macros to
-	  help make things more consistent.
+	  Change-Id: I59b16c4b928896fcbde02ad88f0e98922b15d524
 
-	  Also updated the call sites to remove redundant error messages about
-	  memory allocation failure.
+2017-08-29 14:22 +0000 [4650fc477a]  Richard Mudgett <rmudgett@digium.com>
 
-	  Note that res_config_ldap sets the category filename to the 'table name'
-	  but that is not read by anything in the core, so I've dropped it.
+	* bridge_native_rtp.c: Fixup native_rtp_framehook()
 
-	  Change-Id: I3a1fd91e0c807dea1ce3b643b0a6fe5be9002897
+	  * Fix framehook to test frame type for control frame.
+	  * Made framehook exit early if frame type is not a control frame.
+	  * Eliminated RAII_VAR in framehook.
+	  * Use switch instead of else-if ladder for control frame handling.
 
-2017-02-16 10:30 +0000 [d90430953c]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Ia555fc3600bd85470e3c0141147dbe3ad07c1d18
 
-	* build: Execute ldconfig to build cache.
+2017-08-29 09:26 +0000 [06cc5ae9ff]  Sean Bright <sean.bright@gmail.com>
 
-	  On some platforms a multiarch approach is used for libraries.
-	  The build system does not take this into account and still
-	  places libraries into the lib directory if no --libdir is
-	  specified to configure. On initial startup this results in
-	  libasteriskssl.so not being found, as it is not in the multiarch
-	  lib directory.
+	* confbridge: Handle user hangup during name recording
 
-	  This change does the minimally invasive thing and executes
-	  ldconfig so that the libraries in the lib directory are found
-	  and their location cached. By doing so Asterisk starts up fine.
+	  This prevents orphaned CBAnn channels from getting stuck in the bridge.
 
-	  If DESTDIR is specified, however, the old logic is executed as
-	  the install process may not have permission to alter the ldconfig
-	  cache.
+	  ASTERISK-26994 #close
+	  Reported by: James Terhune
 
-	  ASTERISK-26705
+	  Change-Id: I5e43e832a9507ec3f2c59752cd900b41dab80457
 
-	  Change-Id: If4eca46ac510c6fea5568256280ffdb3888d7bb4
+2017-08-24 11:45 +0000 [9a9589e8e1]  Joshua Colp <jcolp@digium.com>
 
-2017-01-08 20:32 +0000 [3b606093d3]  Richard Mudgett <rmudgett@digium.com>
+	* core: Reduce video update queueing.
 
-	* res_pjsip_authenticator_digest.c: Fix sorcery's immutable contract violation.
+	  A video update frame is used to indicate that a channel
+	  with video negotiated should provide a full frame so the
+	  decoder decoding the stream is able to do so. In situations
+	  where a queue is used to store frames it makes no sense
+	  for the queue to contain multiple video update frames. One
+	  is sufficient to have a full frame be sent.
 
-	  The inbound authentication object is supposed to be immutable when it is
-	  stored in sorcery.  However, the immutable property is violated if the
-	  authentication object does not have a realm set.
+	  ASTERISK-27222
 
-	  The immutable contract violation has a different effect depending upon
-	  what sorcery back end is used.  If it is the config file back end you
-	  would get the same object back until res_pjsip is reloaded.  If it is the
-	  real-time or AstDB back end you would get a new object on each query.  If
-	  it is cached you would get the same object back until it is refreshed from
-	  the database.
+	  Change-Id: Id3f40a6f51b740ae4704003a1800185c0c658ee7
 
-	  Once an inbound authentication object has its realm set it may or may not
-	  get updated again if the default_realm changes.
+2017-08-25 13:44 +0000 [da13cdb9e7]  Sean Bright <sean.bright@gmail.com>
 
-	  If the same authentication object is used for inbound and outbound
-	  authentication then the immutable violation can make it very hard to
-	  determine why the outbound authentication now fails.  The only diagnostic
-	  message is a complaint about no realms matching when it had worked
-	  earlier.  It fails because of the difference in behaviour for an empty
-	  realm setting between inbound and outbound authentication objects.
+	* voicemail: Fix various abuses of mkstemp
 
-	  * Fixed the sorcery object immutable violation by creating a new object
-	  and setting the default_realm on it instead.  The new object is a shallow
-	  copy for speed.
+	  mkstemp() returns a unique filename, but appending an extension to that
+	  filename does not guarantee uniqueness. Instead, use mkdtemp() and we
+	  can put whatever extension we want on the files that we create inside
+	  the directory.
 
-	  * The auth_store thread storage no longer holds an auth ref.  It
-	  interferes with the shallow copy and never needed a ref anyway.
+	  In the case of app_minivm, we also now properly clean up any temporary
+	  files that we create.
 
-	  ASTERISK-26799 #close
+	  ASTERISK-20858 #close
+	  Reported by: Walter Doekes
 
-	  Change-Id: I2328a52f61b78ed5fbba38180b7f183ee7e08956
+	  Change-Id: I30ad04f0e115f0b11693ff678ba5184d8b938e43
 
-2017-02-04 20:17 +0000 [6208962b00]  Richard Mudgett <rmudgett@digium.com>
+2017-08-25 12:20 +0000 [43670e471f]  Sean Bright <sean.bright@gmail.com>
 
-	* res_pjsip: Update artificial auth whenever default_realm changes.
+	* app_record: Resolve some absolute vs. relative filename bugs
 
-	  There was code attempting to update the artificial authentication object
-	  whenever the default_realm changed.  However, once the artificial
-	  authentication object was created it would never get updated.  The
-	  artificial authentication object would require a system restart for a
-	  change to the default_realm to take effect.
+	  If the Record() application is called with a relative filename that
+	  includes directories, we were not properly creating the intermediate
+	  directories and Record() would fail.
 
-	  ASTERISK-26799
+	  Secondarily, updated the documentation for RECORDED_FILE to mention
+	  that it does not include a filename extension.
 
-	  Change-Id: Id59036e9529c2d3ed728af2ed904dc36e7094802
+	  Finally, rewrote the '%d' functionality to be a bit more straight
+	  forward and less noisy.
 
-2017-01-01 08:02 +0000 [9f11da85a2]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-16777 #close
+	  Reported by: klaus3000
 
-	* res_pjsip: Update authentication realm documentation.
+	  Change-Id: Ibc2640cba3a8c7f17d97b02f76b7608b1e7ffde2
 
-	  Using the same auth section for inbound and outbound authentication is not
-	  recommended.  There is a difference in meaning for an empty realm setting
-	  between inbound and outbound authentication uses.
+2017-08-23 10:01 +0000 [2ee644aacf]  Florian Floimair <f.floimair@commend.com>
 
-	  An empty inbound auth realm represents the global section's default_realm
-	  value when the authentication object is used to challenge an incoming
-	  request.  An empty outgoing auth realm is treated as a don't care wildcard
-	  when the authentication object is used to respond to an incoming
-	  authentication challenge.
+	* alembic: Add dtls_fingerprint column in ps_endpoints table
 
-	  ASTERISK-26799
+	  The ps_endpoints table was missing the dtls_fingerprint column
+	  introduced with commit adba2a8d7fd.
 
-	  Change-Id: Id3952f7cfa1b6683b9954f2c5d2352d2f11059ce
+	  ASTERISK-27168 #close
 
-2017-02-13 17:11 +0000 [473813311b]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I9cb5006f7f50718b5239919562773adabb334cfd
 
-	* pjproject: Fixes to resolve DNS SRV crashes.
+2017-08-21 04:28 +0000 [33a648d4c6]  Torrey Searle <torrey@voxbone.com>
 
-	  * Re #1945 (misc): Don't trigger SRV complete callback when there is a
-	  parse error.
+	* res/res_pjsip_session: allow SDP answer to be regenerated
 
-	  * srv_resolver.c: Don't try to send query if already considered resolved.
+	  If an SDP answer hasn't been sent yet, it's legal to change it.
+	  This is required for PJSIP_DTMF_MODE to work correctly, and can
+	  also have use in the future for updating codecs too.
 
-	  ** In resolve_hostnames() don't try to resolve a query that is already
-	  considered resolved.
+	  ASTERISK-27209 #close
 
-	  ** In resolve_hostnames() fix DNS typo in comments.
+	  Change-Id: Idbbfb7cb3f72fbd96c94d10d93540f69bd51e7a1
 
-	  ** In build_server_entries() move a common expression assigning to cnt
-	  earlier.
+2017-08-24 09:42 +0000 [02f95d290f]  Sean Bright <sean.bright@gmail.com>
 
-	  * sip_transport.c: Fix tdata object name to actually contain the pointer.
+	* app_queue: Evaluate realtime queues when running dialplan functions
 
-	  It helps if the logs referencing a tdata object buffer actually have a
-	  name that includes the correct pointer as part of the name.  Also since
-	  the tdata has its own pool it helps if any logs referencing the pool have
-	  the same name as the tdata object.  This change brings tdata logging in
-	  line with how tsx objects are named.
+	  ASTERISK-19103 #close
+	  Reported by: Jim Van Meggelen
 
-	  ASTERISK-26669 #close
-	  ASTERISK-26738 #close
+	  Change-Id: I4bd32a9d1fcebb8ac56bff0e084d4f53e31b692b
 
-	  Change-Id: I56af2ded25476b3e870ca586ee69ed6954ef75af
+2017-08-23 09:19 +0000 [b1097be134]  Eelco Brolman (License 6442)
 
-2017-02-06 14:26 +0000 [d58fdae811]  Richard Mudgett <rmudgett@digium.com>
+	* app_voicemail: Honor escape digits in "greeting only" mode
 
-	* pjsip_distributor.c: Update some debug messages to get transaction name.
+	  ASTERISK-21241 #close
+	  Reported by: Eelco Brolman
+	  Patches:
+	  	Patch uploaded by Eelco Brolman (License 6442)
 
-	  * Removed overloaded unmatched response ignore.  We obviously sent the
-	  request so we shouldn't ignore it because it isn't new work.
+	  Change-Id: Icbe39b5c82a49b46cf1d168dc17766f3d84f54fe
 
-	  ASTERISK-26669
-	  ASTERISK-26738
+2017-08-24 08:35 +0000 [7937d5b8b3]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I55fb5cadc83a8e6699b347c6dc7fa32c5a617d37
+	* res_smdi: Clean up memory leak
 
-2017-02-04 16:00 +0000 [eb9ae4f7cb]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I1e33290929e1aa7c5b9cb513f8254f2884974de8
 
-	* res_pjsip: Record the serializer earlier on the tdata.
+2017-08-18 17:37 +0000 [f2c14f00b8]  Richard Mudgett <rmudgett@digium.com>
 
-	  When PJPROJECT needs to do a DNS resolution and there is not a cached
-	  entry available, the SIP request message goes out on the PJSIP monitor
-	  thread instead of the original serializer thread.  Thus when the response
-	  comes back it does not get processed by the original sending serializer.
-
-	  This patch records the serializer on tdata before passing a request
-	  message to PJPROJECT where it can in Asterisk code.  There are several
-	  places in PJPROJECT for outbound registration and publishing support that
-	  would need to record the serializer.  Unfortunately, without replacing the
-	  PJPROJECT DNS resolver as was done in v14 we cannot fix those without
-	  modifying PJPROJECT.
-
-	  Even if we backported the DNS resolver from v14, the outbound registration
-	  refresh timer does not go out on a serializer thread but the PJSIP monitor
-	  thread.  Fortunately, Asterisk's outbound publish support doesn't use the
-	  auto refresh timer that would also not go out under the serializer thread.
+	* res_pjsip_session.c: Fix crash when declining an active stream.
 
-	  This patch is v13 only.
+	  If a previously active stream is declined we could crash because the
+	  channel's thread is still using the stream while we are updating the
+	  topology in the serializer thread.
 
-	  ASTERISK-26669
-	  ASTERISK-26738
+	  * Defer removing any declined stream's handler until we have blocked the
+	  channel's thread with the channel lock.
 
-	  Change-Id: I9997b9ed6dbcebd2c37d6a67dc6dcee9c78914a4
+	  ASTERISK-27212
 
-2017-02-20 13:38 +0000 [57f19d6efb]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I50e1d3ef26f8e41948f4c411ee329aa3b960a420
 
-	* pjproject: Increase SENDER_WIDTH column size for 64-bit system logs.
+2017-08-16 17:50 +0000 [17976d1b4e]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26669
-	  ASTERISK-26738
+	* bridge_channel.c: Fix FRACK when mapping frames to the bridge.
 
-	  Change-Id: Ibae6fc8cae69a1f04df0c577c4c11200499d6fe0
+	  * Add protection checks when mapping streams to the bridge.  The channel
+	  and bridge may be in the process of updating the stream mapping when a
+	  media frame comes in so we may not be able to map the frame at the time.
 
-2017-02-20 06:28 +0000 [47daca8a2b]  Sean Bright <sean.bright@gmail.com>
+	  * We need to map the streams to the bridge's stream numbers right before
+	  they are written into the bridge.  That way we don't have to keep
+	  locking/unlocking the bridge and we won't have any synchronization
+	  problems before the frames actually go into the bridge.
 
-	* app_voicemail: vm_authenticate accesses uninitialized memory
+	  * Protect the deferred queue with the bridge_channel lock.
 
-	  vm_authenticate doesn't always set the passed ast_vm_user argument, so
-	  we initialize to 0 before passing it in.
+	  ASTERISK-27212
 
-	  ASTERISK-25893 #close
-	  Reported by: Filip Jenicek
+	  Change-Id: Id6860dd61b594b90c8395f6e2c0150219094c21a
 
-	  Change-Id: Ia3cc0128f93d352ed9add8d5c2f0f7232c2cbe4a
+2017-08-11 16:31 +0000 [9c70c88369]  Richard Mudgett <rmudgett@digium.com>
 
-2017-02-20 11:19 +0000 [06214173a9]  Joshua Colp <jcolp@digium.com>
+	* channel: Fix topology API locking.
 
-	* Revert "build: Execute ldconfig to build cache."
+	  * ast_channel_request_stream_topology_change() must not be called with any
+	  channel locks held.
 
-	  This reverts commit e910dbab90ef3d628955c49f441b2c9dda1f222c.
+	  * ast_channel_stream_topology_changed() must be called with only the
+	  passed channel lock held.
 
-	  Change-Id: I242aa0a965a79738dc898299959c6d2e020c86bd
+	  ASTERISK-27212
 
-2017-02-20 08:04 +0000 [c9ea98f9bf]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I843de7956d9f1cc7cc02025aea3463d8fe19c691
 
-	* pjproject cli:  Add object count after object lists
+2017-08-16 15:22 +0000 [6ad8249233]  Richard Mudgett <rmudgett@digium.com>
 
-	  When listing a container, we now print the number of objects
-	  in the container at the end of the list.
+	* bridge: Fix softmix bridge deadlock.
 
-	  Change-Id: I791cbc3ee9da9a2af9adc655164b5d32953df812
+	  * Fix deadlock in
+	  bridge_softmix.c:softmix_bridge_stream_topology_changed() between
+	  bridge_channel and channel locks.
 
-2017-02-20 05:53 +0000 [d8972f50f4]  Sean Bright <sean.bright@gmail.com>
+	  * The new bridge technology topology change callbacks must be called with
+	  the bridge locked.  The callback references the bridge channel list, the
+	  bridge technology could change, and the bridge stream mapping is updated.
 
-	* res_config_ldap: Don't try to delete non-existent attributes
+	  ASTERISK-27212
 
-	  OpenLDAP will raise an error when we try to delete an LDAP attribute
-	  that doesn't exist. We need to filter out LDAP_MOD_DELETE requests
-	  based on which attributes the current LDAP entry actually has. There
-	  is of course a small window of opportunity for this to still fail,
-	  but it is much less likely now.
+	  Change-Id: Ide4360ab853607e738ad471721af3f561ddd83be
 
-	  Change-Id: I3fe1b04472733e43151563aaf9f8b49980273e6b
+2017-08-14 12:20 +0000 [850a3fd017]  Richard Mudgett <rmudgett@digium.com>
 
-2017-02-20 05:49 +0000 [b980cae1f7]  Sean Bright <sean.bright@gmail.com>
+	* chan_pjsip.c: Fix topology refresh response code accuracy.
 
-	* res_config_ldap: Remove extraneous line numbers from log messages
+	  There are other 1xx and 2xx codes than 100 and 200 respectively.
 
-	  Extraneous line numbers were being output in many log messages. These
-	  have been removed.
+	  Change-Id: I680db0997343256add1478714f5bf5b5569aee17
 
-	  Change-Id: Ice9efa3d252ee87f37fa8f5ea852fda482675431
+2017-08-11 17:06 +0000 [87c7a1c79c]  Richard Mudgett <rmudgett@digium.com>
 
-2017-02-20 05:45 +0000 [011b7be62a]  Sean Bright <sean.bright@gmail.com>
+	* bridge_softmix.c: Restored softmix_bridge_leave() shortcut exit.
 
-	* res_config_ldap: Make memory allocation more consistent
+	  Change-Id: I13026cd90954e0265eab94a0faf635a3e11f0e35
 
-	  The code in update_ldap() and update2_ldap() was using both Asterisk's
-	  memory allocation routines as well as OpenLDAP's. I've changed it so
-	  that everything that is passed to OpenLDAP's functions are allocated
-	  with their routines.
+2017-08-17 17:07 +0000 [5bbf7b2aad]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Iafec9c1fd8ea49ccc496d6316769a6a426daa804
+	* app_confbridge: Document sfu video_mode value.
 
-2017-02-20 05:30 +0000 [b2836dde7e]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I26e17df2c93f3933b23f78070603adbcc84ba204
 
-	* res_config_ldap: Fix configuration inheritance from _general
+2017-08-17 17:06 +0000 [f96536b1ea]  Richard Mudgett <rmudgett@digium.com>
 
-	  The "_general" configuration section allows administrators to provide
-	  both general configuration options (host, port, url, etc.) as well as a
-	  global realtime-to-LDAP-attribute mapping that is a fallback if one of
-	  the later sections do not override it. This neglected to exclude the
-	  general configuration options from the mapping. As an example, during
-	  my testing, chan_sip requested 'port' from realtime, and because I did
-	  not have it defined, it pulled in the 'port' configuration option from
-	  "_general." We now filter those out explicitly.
+	* confbridge.h: Fix doxygen comments.
 
-	  Change-Id: I1fc61560bf96b8ba623063cfb7e0a49c4690d778
+	  Change-Id: I16133166a85fdb557c66ffcbfe8128d0b4725b0e
 
-2017-02-20 05:27 +0000 [6d5e9993b2]  Sean Bright <sean.bright@gmail.com>
+2017-08-11 11:40 +0000 [946ef2d711]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_config_ldap: Fix erroneous LDAP_MOD_REPLACE in LDAP modify
+	* bridge_softmix.c: Remove always true test.
 
-	  We always treat the first change of our modification batch as a
-	  replacement when it sometimes is actually a delete. So we have to pass
-	  the correct arguments to the OpenLDAP library.
+	  Change-Id: I26238df2ff0d0f6dfe95c3aa35da588f1ee71727
 
-	  ASTERISK-26580 #close
-	  Reported by: Nicholas John Koch
-	  Patches:
-	  	res_config_ldap.c-11.24.1.patch (license #6833) patch uploaded
-	  	by Nicholas John Koch
+2017-08-17 16:46 +0000 [22af5e3784]  Sungtae Kim <pchero21@gmail.com>
 
-	  Change-Id: I0741d25de07c9539f1edc6eff3696165dfb64fbe
+	* app_queue: Fix initial hold time queue statistic
 
-2017-02-15 11:55 +0000 [5b7c6678ae]  Sean Bright <sean.bright@gmail.com>
+	  Fixed to use correct initial value and fixed to use the
+	  correct queue info to check the first value.
 
-	* res_config_sqlite3: Fix crash when loading with invalid config
+	  ASTERISK-27204
 
-	  When ast_config_load() fails with CONFIG_STATUS_FILEINVALID, it has
-	  already destroyed the ast_config struct for us. Trying to do it again
-	  results in a crash.
+	  Change-Id: Ia9e36c828e566e1cc25c66f73307566e4acb8e73
 
-	  Change-Id: If6a5c0ca718ad428e01a1fb25beb209a9ac18bc6
+2017-08-20 08:15 +0000 [83b81d1f8d]  Michael Kuron <m.kuron@gmx.de>
 
-2017-02-17 16:57 +0000 [096496e13e]  Richard Mudgett <rmudgett@digium.com>
+	* res_xmpp: fix inverted return code check in OAuth
 
-	* tcptls.c: Add some missing allocation failure checks.
+	  fetch_access_token calls func_curl via ast_func_read. The latter returns 0 upon
+	  success and -1 if the function is not available.
+	  This commit inverts the return code check so that an error is printed if the
+	  module is not loaded and not if it is loaded.
 
-	  Change-Id: I0ddf01cd3c10d3b6666d7bf68d4e206a37f4fbdb
+	  ASTERISK-27207 #close
 
-2017-02-17 17:06 +0000 [047a1e7dcc]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I9ef903f80702d1218e8701f65a4e5e918e6548fb
 
-	* pjproject-bundled: Fix checksum verification when using cURL
+2017-08-17 12:00 +0000 [667986d875]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26802 #close
-	  Reported by: Michael L. Young
+	* res_calendar_icalendar: Properly handle recurring events
 
-	  Change-Id: Iad293080f55d4d69ab615717a15211d916eed613
+	  When looking for recurring events, use the correct end time based on the
+	  configured 'timeframe.'
 
-2017-02-16 08:38 +0000 [2cd75fe311]  Sean Bright <sean.bright@gmail.com>
+	  ASTERISK-27174 #close
+	  Reported by: Mark Thompson
 
-	* realtime: Fix LIKE escaping in SQL backends
+	  Change-Id: Id90c3cfc79d561a5521d79be176683e225f2edef
 
-	  The realtime framework allows for components to look up values using a
-	  LIKE clause with similar syntax to SQL's. pbx_realtime uses this
-	  functionality to search for pattern matching extensions that start with
-	  an underscore (_).
+2017-08-16 15:43 +0000 [0e777258be]  George Joseph <gjoseph@digium.com>
 
-	  When passing an underscore to SQL's LIKE clause, it will be interpreted
-	  as a wildcard matching a single character and therefore needs to be
-	  escaped. It is (for better or for worse) the responsibility of the
-	  component that is querying realtime to escape it with a backslash before
-	  passing it in. Some RDBMs support escape characters by default, but the
-	  SQL92 standard explicitly says that there are no escape characters
-	  unless they are specified with an ESCAPE clause, e.g.
+	* Fix downloader not working with curl
 
-	  	SELECT * FROM table WHERE column LIKE '\_%' ESCAPE '\'
+	  The codec/dpma downloader wasn't handling curl correctly.  The logic
+	  that transforms makeopts into a bash-sourceable file wasn't
+	  handling the make 'or' command in DOWNLOAD_TIMEOUT so bash was
+	  looking for an 'or' command.
 
-	  This patch instructs 3 backends - res_config_mysql, res_config_pgsql,
-	  and res_config_sqlite3 - to use the ESCAPE clause where appropriate.
+	  That logic has been eliminated.  Instead of trying to transform
+	  and source makeopts, the downloader now calls a make scriptlet
+	  to print the value of a specific variable.  This way, make handles
+	  the ors (or any other make construct that happens to creep into
+	  that file).
 
-	  Looking through documentation and source tarballs, I was able to
-	  determine that the ESCAPE clause is supported in:
+	  ASTERISK-27202
+	  Reported by: Sean McCord
 
-	  MySQL 5.0.15   (released 2005-10-22 - earliest version available from
-	                  archives)
-	  PostgreSQL 7.1 (released 2001-04-13)
-	  SQLite 3.1.0   (released 2005-01-21)
+	  Change-Id: Iadfb6693528e4d4da7b8bb201fa66da2c71c7f99
 
-	  The versions of the relevant libraries that we depend on to access MySQL
-	  and PostgreSQL will not work on versions that old, and I've added an
-	  explicit check in res_config_sqlite3 to only use the ESCAPE clause when
-	  we have a sufficiently new version of SQLite3.
+2017-08-15 13:12 +0000 [e4e2e53c8a]  Kevin Harwell <kharwell@digium.com>
 
-	  res_config_odbc already handles the escape characters appropriately, so
-	  no changes were required there.
+	* manager: hook event is not being raised
 
-	  ASTERISK-15858 #close
-	  Reported by: Humberto Figuera
+	  When the iostream code went in it introduced a conditional that made it so the
+	  hook event was not being raised even if a hook is present. This patch adds a
+	  check to see if a hook is present in astman_append. If so then call into the
+	  send_string function, which in turn raises the even for specified hook.
 
-	  ASTERISK-26057 #close
-	  Reported by: Stepan
+	  Also updated the ami hooks unit test, so the test could be automated.
 
-	  Change-Id: I93117fbb874189ae819f4a31222df7c82cd20efa
+	  ASTERISK-27200 #close
 
-2017-02-16 10:30 +0000 [e910dbab90]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Iff37f02f9708195d8f23e68f959d6eab720e1e36
 
-	* build: Execute ldconfig to build cache.
+2017-08-15 15:15 +0000 [c049d1c3b2]  Richard Mudgett <rmudgett@digium.com>
 
-	  On some platforms a multiarch approach is used for libraries.
-	  The build system does not take this into account and still
-	  places libraries into the lib directory if no --libdir is
-	  specified to configure. On initial startup this results in
-	  libasteriskssl.so not being found, as it is not in the multiarch
-	  lib directory.
+	* configure: Check cache for valid pjproject tarball before downloading.
 
-	  This change does the minimally invasive thing and executes
-	  ldconfig so that the libraries in the lib directory are found
-	  and their location cached. By doing so Asterisk starts up fine.
+	  On a fresh Asterisk source directory, the bundled pjproject tarball is
+	  unconditionally downloaded even if the tarball is already in a specified
+	  cache directory.
 
-	  ASTERISK-26705
+	  * Made check if the pjproject tarball is valid in the cache directory
+	  before downloading the tarball on a fresh source directory.
 
-	  Change-Id: I6d30b6427e9d5e69470e11327c7ff203fa7da519
+	  Change-Id: Ic7ec842d3c97ecd8dafbad6f056b7fdbce41cae5
 
-2017-02-16 05:46 +0000 [9b02bbfa88]  Sean Bright <sean.bright@gmail.com>
+2017-08-15 11:14 +0000 [9e2b2a9837]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_config_sqlite3: Properly create missing columns when necessary
+	* res_pjsip: Fix prune_on_boot to remove only contacts for the host.
 
-	  There were two specific issues resolved here:
+	  * Check that the contact's reg_server matches the host's name before
+	  deleting any prune_on_boot contacts.  We don't want to delete reliable
+	  transport contacts made with other servers if the ps_contacts database
+	  table is shared with other servers.
 
-	  1) The code that iterated over the required fields
-	     (via ast_realtime_require) was broken for the RQ_INTEGER1 field
-	     type. Iteration would stop when the first RQ_INTEGER1 (0) field
-	     was encountered.
+	  Thanks to Ross Beer for pointing out that the original prune logic would
+	  delete reliable transport contacts from other servers.
 
-	  2) sqlite3_changes() was used to try and count the number of rows
-	     returned by a SELECT statement. sqlite3_changes() only counts
-	     affected rows, so this was always returning the value from the
-	     most recent data modification statement. We now separate read-only
-	     queries from data modification queries and count rows appropriately
-	     in both cases.
+	  ASTERISK-27147
 
-	  ASTERISK-23457 #close
-	  Reported by: Scott Griepentrog
+	  Change-Id: I8e439d0d1c266ffdfd7b73d1e5e466180a689bd0
 
-	  Change-Id: I91ed20494efc3fcfbc2a96ac7646999a49814884
+2017-08-04 09:25 +0000 [15fbcc74d8]  Andrey Egorov <andr06@gmail.com>
 
-2017-02-15 14:44 +0000 [0fc27fa364]  Joshua Elson <joshelson@gmail.com>
+	* res_xmpp: Google OAuth 2.0 protocol support for XMPP / Motif
 
-	* http: Ensure capath is defined on all http creations
+	  Add ability to use tokens instead of passwords according to Google OAuth 2.0
+	  protocol.
 
-	  ASTERISK-26794 #close
+	  ASTERISK-27169
+	  Reported by: Andrey Egorov
+	  Tested by: Andrey Egorov
 
-	  Change-Id: I9cbc3b6b6a8aab590f5ccde9c262a98e4d5253a1
+	  Change-Id: I07f7052a502457ab55010a4d3686653b60f4c8db
 
-2017-02-15 23:09 +0000 [7aa731c1c7]  Igor Goncharovsky <igor.goncharovsky@gmail.com>
+2017-08-10 14:18 +0000 [bd28a9bbd8]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_unistim: fix char type to have consistent behavior on ARM
+	* STUN/netsock2: Fix some valgrind uninitialized memory findings.
 
-	  There is difference exists in behaviour of char type on x86 and ARM.
-	  On x86 by default char variable type means signed char, but in ARM
-	  unsigned char used. This make binary calculations and negative values
-	  works wrong on ARM.
+	  * netsock2.c: Test the addr->len member first as it may be the only member
+	  initialized in the struct.
 
-	  This patch change type of char variables used for store negative
-	  values and binary calculations to signed char.
+	  * stun.c:ast_stun_handle_packet(): The combinded[] local array could get
+	  used uninitialized by ast_stun_request().  The uninitialized string gets
+	  copied to another location and could overflow the destination memory
+	  buffer.
 
-	  ASTERISK-26714
+	  These valgrind findings were found for ASTERISK_27150 but are not
+	  necessarily a fix for the issue.
 
-	  Change-Id: Id78716dee9568a58419d4ef63c038affc3dfc7ab
+	  Change-Id: I55f8687ba4ffc0f69578fd850af006a56cbc9a57
 
-2017-02-07 13:17 +0000 [be77b845d9]  gtjoseph <gjoseph@digium.com>
+2017-08-02 18:44 +0000 [1bec781cce]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_pubsub:  Correctly implement persisted subscriptions
+	* res_pjsip_outbound_registration.c: Re-REGISTER on transport shutdown.
 
-	  This patch fixes 2 original issues and more that those 2 exposed.
+	  The fix for the issue is broken up into three parts.
 
-	  * When we send a NOTIFY, and the client either doesn't respond or
-	    responds with a non OK, pjproject only calls our
-	    pubsub_on_evsub_state callback, no others.  Since
-	    pubsub_on_evsub_state (which does the sub_tree cleanup) does not
-	    expect to be called back without the other callbacks being called
-	    first, it just returns leaving the sub_tree orphaned.  Now
-	    pubsub_on_evsub_state checks the event for PJSIP_EVENT_TSX_STATE
-	    which is what pjproject will set to tell us that it was the
-	    transaction that timed out or failed and not the subscription
-	    itself timing our or being terminated by the client. If is
-	    TSX_STATE, pubsub_on_evsub_state now does the proper cleanup
-	    regardless of the state of the subscription.
+	  This is part three which handles the client side of REGISTER requests.
+	  The registered contact may no longer be valid on the server when the
+	  transport used is reliable and the connection is broken.
 
-	  * When a client renews a subscription, we don't update the
-	    persisted subscription with the new expires timestamp.  This causes
-	    subscription_persistence_recreate to prune the subscription if/when
-	    asterisk restarts.  Now, pubsub_on_rx_refresh calls
-	    subscription_persistence_update to apply the new expires timestamp.
-	    This exposed other issues however...
+	  * Re-REGISTER our contact if the reliable transport is broken after
+	  registration completes.  We attempt to re-REGISTER immediately to minimize
+	  the time we are unreachable.  Time may have already passed between the
+	  connection being broken and the loss being detected.
 
-	  * When creating a dialog from rdata (which sub_persistence_recreate
-	    does from the packet buffer) there must NOT be a tag on the To
-	    header (which there will be when a client refreshes a
-	    subscription).  If there is one, pjsip_dlg_create_uas will fail.
-	    To address this, subscription_persistence_update now accepts a flag
-	    that indicates that the original packet buffer must not be updated.
-	    New subscribes don't set the flag and renews do.  This makes sure
-	    that when the rdata is recreated on asterisk startup, it's done
-	    from the original subscribe packet which won't have the tag on To.
+	  * Reorder sip_outbound_registration_state_alloc() so the STATSD_GUAGE's
+	  are still correct if an allocation failure happens.
 
-	  * When creating a dialog from rdata, we were setting the dialog's
-	    remote (SUBSCRIBE) cseq to be the same as the local (NOTIFY) cseq.
-	    When the client tried to resubscribe after a restart with the
-	    correct cseq, we'd reject the request with an Invalid CSeq error.
+	  ASTERISK-27147
 
-	  * The acts of creating a dialog and evsub by themselves when
-	    recreating a subscription does NOT restart pjproject's subscription
-	    timer.  The result was that even if we did correctly recreate the
-	    subscription, we never removed it if the client happened to go away
-	    or send a non-OK response to a NOTIFY.  However, there is no
-	    pjproject function exposed to just set the timer on an evsub that
-	    wasn't created by an incoming subscribe request.  To address this,
-	    we create our own timer using ast_sip_schedule_task.  This timer is
-	    used only for re-establishing subscriptions after a restart.
+	  Change-Id: I3668405b1ee75dfefb07c0d637826176f741ce83
 
-	    An earlier approach was to add support for setting pjproject's
-	    timer (via a pjproject patch) and while that patch is still included
-	    here, we don't use that call at the moment.
+2017-07-31 14:21 +0000 [82f4ade959]  Richard Mudgett <rmudgett@digium.com>
 
-	  While addressing these issues, additional debugging was added and
-	  some existing messages made more useful.  A few formatting changes
-	  were also made to 'pjsip show scheduled tasks' to make displaying
-	  the subscription timers a little more friendly.
+	* res_pjsip: Remove ephemeral registered contacts on transport shutdown.
 
-	  ASTERISK-26696
-	  ASTERISK-26756
+	  The fix for the issue is broken up into three parts.
 
-	  Change-Id: I8c605fc1e3923f466a74db087d5ab6f90abce68e
+	  This is part two which handles the server side of REGISTER requests when
+	  rewrite_contact is enabled.  Any registered reliable transport contact
+	  becomes invalid when the transport connection becomes disconnected.
 
-2017-02-15 11:03 +0000 [73133d5354]  Sean Bright <sean.bright@gmail.com>
+	  * Monitor the rewrite_contact's reliable transport REGISTER contact for
+	  shutdown.  If it is shutdown then the contact must be removed because it
+	  is no longer valid.  Otherwise, when the client attempts to re-REGISTER it
+	  may be blocked because the invalid contact is there.  Also if we try to
+	  send a call to the endpoint using the invalid contact then the endpoint is
+	  not likely to see the request.  The endpoint either won't be listening on
+	  that port for new connections or a NAT/firewall will block it.
 
-	* res_rtp_asterisk: Use PJ_ICE_MAX_CAND instead of hard-coding 16
+	  * Prune any rewrite_contact's registered reliable transport contacts on
+	  boot.  The reliable transport no longer exists so the contact is invalid.
 
-	  pjsip limits the total number of ICE candidates to PJ_ICE_MAX_CAND,
-	  which is a compile-time constant. Instead of hard-coding 16 when we
-	  enumerate local interfaces, use PJ_ICE_MAX_CAND so that we can
-	  potentially collect more interfaces if the compile time options are
-	  changed.
+	  * Websockets always rewrite the REGISTER contact address and the transport
+	  needs to be monitored for shutdown.
 
-	  Tangentially related to ASTERISK~24464
+	  * Made the websocket transport set a unique name since that is what we use
+	  as the ao2 container key.  Otherwise, we would not know which transport we
+	  find when one of them shuts down.  The names are also used for PJPROJECT
+	  debug logging.
 
-	  Change-Id: I1b85509e39e33b1fed63c86261fc229ba14bbabd
+	  * Made the websocket transport post the PJSIP_TP_STATE_CONNECTED state
+	  event.  Now the global keep_alive_interval option, initially idle shutdown
+	  timer, and the server REGISTER contact monitor can work on wetsocket
+	  transports.
 
-2017-02-03 02:25 +0000 [99b40e72ae]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  * Made the websocket transport set the PJSIP_TP_DIR_INCOMING direction.
+	  Now initially idle websockets will automatically shutdown.
 
-	* libasteriskssl: do nothing with OpenSSL >= 1.1
+	  ASTERISK-27147
 
-	  OpenSSL 1.1 requires no explicit initialization. The hacks in the
-	  library are not needed. They also happen to fail running Asterisk.
+	  Change-Id: I397a5e7d18476830f7ffe1726adf9ee6c15964f4
 
-	  ASTERISK-26109 #close
+2017-07-28 18:26 +0000 [1dcb92bba8]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I3b3efd5d80234a4c45a8ee58dcfe25b15d9ad100
+	* res_pjsip: PJSIP Transport state monitor refactor.
 
-2017-02-13 16:50 +0000 [4c31e03e80]  Sean Bright <sean.bright@gmail.com>
+	  The fix for the issue is broken up into three parts.
 
-	* app_voicemail: Allow 'Comedian Mail' branding to be overriden
+	  This is part one which refactors the transport state monitor code to allow
+	  more modules to be able to monitor transports.
 
-	  Original patch by John Covert, slight modifications by me.
+	  * Pull the management of PJPROJECT's transport state callback code from
+	  res_pjsip_transport_management.c into res_pjsip.  Now other modules can
+	  dynamically add and remove themselves from transport monitoring without
+	  worrying about breaking PJPROJECT's callback chain.
 
-	  ASTERISK-17428 #close
-	  Reported by: John Covert
-	  Patches:
-	  	app_voicemail.c.patch (license #5512) patch uploaded by
-	          John Covert
+	  * Add the ability for other modules to get a callback whenever a specific
+	  transport is shutdown.
 
-	  Change-Id: Ic3361b0782e5a5397a19ab18eb8550923a9bd6a6
+	  ASTERISK-27147
 
-2017-01-20 23:59 +0000 [e97e50b68b]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Change-Id: I7d9a31371eb1487c9b7050cf82a9af5180a57912
 
-	* tcptls: use TLS_client_method with OpenSSL 1.1
+2017-07-27 15:36 +0000 [ee5edfb050]  Richard Mudgett <rmudgett@digium.com>
 
-	  OpenSSL 1.1 introduced TLS_client_method() and deprecated the previous
-	  version-specific methods (such as TLSv1_client_method(). Other than
-	  being simpler to use and more correct (gain support for TLS newer that
-	  TLS1, in our case), the older ones produce a deprecation warning that
-	  fails the build in dev-mode.
+	* res_pjsip_transport_management.c: Rename some variables.
 
-	  ASTERISK-26109 #close
+	  * Use monitored instead of the misleading keepalive name.
 
-	  Change-Id: I257b1c8afd09dcb0d96cda3a41cb9f7a15d0ba07
+	  Change-Id: I9e5bcbb4ab2b82d49bcd0f06dfe85d15e0b552b6
 
-2017-01-20 23:57 +0000 [0d555f0d81]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+2017-08-09 15:24 +0000 [ecd1f87edf]  Richard Mudgett <rmudgett@digium.com>
 
-	* openssl 1.1 support: use OPENSSL_VERSION_NUMBER
+	* UPGRADE notes: Prepare for the eventual 16 branch.
 
-	  Use OPENSSL_VERSION_NUMBER instead of OPENSSL_API_COMPAT to detect
-	  the openssl 1.1 API.
+	  Change-Id: I4ca2f07ed62d77f1fdd10c3b216f6a28dd75720c
 
-	  ASTERISK-26109 #close
+2017-08-10 09:09 +0000 [4ed2733dde]  Scott Griepentrog <scott@griepentrog.com>
 
-	  Change-Id: I4e448f55ef516aedf6ad154037c35577a421a458
+	* res_pjsip_messaging: IPv6 receive address needs brackets
 
-2017-01-25 16:25 +0000 [9d34df9a5e]  Ryan Rittgarn <rrittgarn@techpro.com>
+	  When handling an incoming SIP MESSAGE, PJSIP
+	  attaches the IP address that the message was
+	  received from to the message in the variable
+	  PJSIP_RECVADDR.  When the IP address is IPv6
+	  the :PORT appended results in an unparseable
+	  mess. By using an additional bit flag on the
+	  pj_sockaddr_print call, the conventional use
+	  of brackets around the address is achieved.
 
-	* app_voicemail: VoiceMailPlayMsg did not play database stored messages
+	  ASTERISK-27193 #close
 
-	  When attempting to use VoiceMailPlayMsg with a realtime data backend
-	  the message is located, but never retrieved. This patch adds the
-	  required RETRIEVE and DISPOSE calls that will fetch the message from
-	  the database (and IMAP storage as well for that matter).
+	  Change-Id: I12342521f2ce87a5b6e4883d480a3fd957aa9fd9
 
-	  Also, removed extraneous make_file call.
+2017-07-26 09:17 +0000 [d430f718f5]  Torrey Searle <torrey@voxbone.com>
 
-	  ASTERISK-26723 #close
+	* res_rtp_asterisk: enable rtcp & QOS stats on native bridge
 
-	  Change-Id: I1e122dd53c0f3d7faa10f3c2b7e7e76a47d51b8c
+	  Asterisk wasn't generating or forwarding RTCP packets when native
+	  bridge was activated.  Also the stats weren't available via
+	  CHANNEL(qos). Now the RTCP stats are always calculated.
 
-2017-02-14 08:12 +0000 [f99e5f4de4]  var <cbrunker@appenbutlerhill.com>
+	  ASTERISK-27158 #close
 
-	* app_record: Add option to prevent silence from being truncated
+	  Change-Id: I46fb8f61c95e836b9d2dda6054b0cf205c16037b
 
-	  When using Record() with the silence detection feature, the stream is
-	  written out to the given file. However, if only 'silence' is detected,
-	  this file is then truncated to the first second of the recording.
+2017-07-28 07:53 +0000 [a2dde59154]  Torrey Searle <torrey@voxbone.com>
 
-	  This patch adds the 'u' option to Record() to override that behavior.
+	* res_rtp_asterisk:  Make P2P bridge Asymmetric codec aware
 
-	  ASTERISK-18286 #close
-	  Reported by: var
-	  Patches:
-	  	app_record-1.8.7.1.diff (license #6184) patch uploaded by var
+	  Introduce a new property to rtp-engine to make it aware of
+	  the desire for assymetric codecs or not.  If asymmetric codecs
+	  is not allowed, the bridge will compare read/write formats
+	  and shut down the p2p bridge if needed
 
-	  Change-Id: Ia1cd163483235efe2db05e52f39054288553b957
+	  ASTERISK-26745 #close
 
-2017-02-11 09:57 +0000 [ea8a610776]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: I0d9c83e5356df81661e58d40a8db565833501a6f
 
-	* cli: Fix various CLI documentation and completion issues
+2017-08-08 13:33 +0000 [305bd0d99f]  George Joseph <gjoseph@digium.com>
 
-	  * app_minivm: Use built-in completion facilities to complete optional
-	  arguments.
+	* Make --with-pjproject-bundled the default for Asterisk 15
 
-	  * app_voicemail: Use built-in completion facilities to complete
-	  optional arguments.
+	  '--with-pjproject-bundled' is now the default when running
+	  ./configure. It can be disabled with '--without-pjproject-bundled'.
 
-	  * app_confbridge: Add missing colons after 'Usage' text.
+	  To make building without an internet connection easier, a new
+	  ./configure option '--with-download-cache' was added that sets
+	  the cache for externals (like pjproject, the codecs and the DPMA),
+	  AND the sounds files.  It can also be specified as an environment
+	  variable named "AST_DOWNLOAD_CACHE".  The existing
+	  '--with-sounds-cache' option / SOUNDS_CACHE_DIR env variable and
+	  '--with-externals-cache' option / EXTERNALS_CACHE_DIR env variable
+	  remain and if specified, will override '--with-downloads-cache'.
 
-	  * chan_alsa: Use built-in completion facilities to complete optional
-	  arguments.
+	  ASTERISK-27189
 
-	  * chan_sip: Use built-in completion facilities to complete optional
-	  arguments. Add completions for 'load' for 'sip show user', 'sip show
-	  peer', and 'sip qualify peer.'
+	  Change-Id: Ifa9783fddf44aafadb060c9feba713dfa81d38ce
 
-	  * chan_skinny: Correct and extend completions for 'skinny reset' and
-	  'skinny show line.'
+2017-08-05 06:36 +0000 [62092bc114]  Joshua Colp <jcolp@digium.com>
 
-	  * func_odbc: Correct completions for 'odbc read' and 'odbc write'
+	* res_pjsip_session: Release media resources on session end quicker.
 
-	  * main/asterisk: Correct and extend completions for 'core show file
-	  version.'
+	  A change was made long ago where the session was kept around
+	  until the underlying INVITE session had been destroyed. This
+	  had the side effect of also keeping the underlying media resources
+	  around for this time as well.
 
-	  * main/astmm: Use built-in completion facilities to complete arguments
-	  for 'memory' commands.
+	  This change ensures that when we are told to terminate the
+	  session we immediately release any media sessions associated
+	  with it.
 
-	  * main/bridge: Correct completions for 'bridge kick.'
+	  ASTERISK-27110
 
-	  * main/ccss: Use built-in completion facilities to complete arguments
-	  for 'cc cancel' command.
+	  Change-Id: I643e431d5c3bf05cda220c1d39e824a505a29b82
 
-	  * main/cli: Add 'all' completion for 'channel request hangup.' Correct
-	  completions for 'core set debug channel.' Correct completions for 'core
-	  show calls.'
+2017-07-29 20:03 +0000 [4b58609c33]  Kirill Katsnelson <kkm@smartaction.com>
 
-	  * main/pbx_app: Remove redundant completions for 'core show
-	  applications.'
+	* chan_sip: Access incoming REFER headers in dialplan
 
-	  * main/pbx_hangup_handler: Remove unused completions for 'core show
-	  hanguphandlers all.'
+	  This adds a way to access information passed along with SIP headers in
+	  a REFER message that initiates a transfer. Headers matching a dialplan
+	  variable GET_TRANSFERRER_DATA in the transferrer channel are added to
+	  a HASH object TRANSFER_DATA to be accessed with functions HASHKEY and HASH.
 
-	  * res_sorcery_memory_cache: Add completion for 'reload' argument of
-	  'sorcery memory cache stale' and properly implement.
+	  The variable GET_TRANSFERRER_DATA is interpreted to be a prefix for
+	  headers that should be put into the hash. If not set, no headers are
+	  included. If set to a string (perhaps 'X-' in a typical case), all headers
+	  starting this string are added. Empty string matches all headers.
 
-	  Change-Id: Iee58c7392f6fec34ad9d596109117af87697bbca
+	  If there are multiple of the same header, only the latest occurrence in
+	  the REFER message is available in the hash.
 
-2017-01-13 11:21 +0000 [17030100ca]  Norbert Varga <vnorbix@gmail.com>
+	  Obviously, the variable GET_TRANSFERRER_DATA must be inherited by the
+	  referrer channel, and should be set with the '_' or '__' prefix.
 
-	* chan_pjsip: Multidomain endpoint finding on call
+	  I avoided a specific reference to SIP or REFER, as in my mind the mechanism
+	  can be generalized to other channel techs.
 
-	  When PJSIP tries to call an endpoint with a domain (e.g. 1000@test.com),
-	  the user part is stripped down as it would be a trunk with a specified user,
-	  and only the host part is called as a PJSIP endpoint and can't be found.
-	  This is not correct in the case of a multidomain SIP account, so the stripping
-	  after the @ sign is done only if the whole endpoint (in multidomain case
-	  1000@test.com) can't be found.
+	  ASTERISK-27162
 
-	  ASTERISK-26248
+	  Change-Id: I73d7a1e95981693bc59aa0d5093c074b555f708e
 
-	  Change-Id: I3a2dd6f57f3bd042df46b961eccd81d31ab202e6
+2017-08-06 11:15 +0000 [88c65f7cb6]  Joshua Colp <jcolp@digium.com>
 
-2017-02-13 05:05 +0000 [18f1b52601]  Joshua Colp <jcolp@digium.com>
+	* bridge: Fix stream topology/participant locking and video misrouting.
 
-	* channel: Protect flags in ast_waitfor_nandfds operation.
+	  This change fixes a few locking issues and some video misrouting.
 
-	  The ast_waitfor_nandfds operation will manipulate the flags
-	  of channels passed in. This was previously done without
-	  the channel lock being held. This could result in incorrect
-	  values existing for the flags if another thread manipulated
-	  the flags at the same time.
+	  1. When accessing the stream topology of a channel the channel lock
+	  must be held to guarantee the topology remains valid.
 
-	  This change locks the channel during flag manipulation.
+	  2. When a channel was joined to a bridge the bridge specific
+	  implementation for stream mapping was not invoked, causing video
+	  to be misrouted for a brief period of time.
 
-	  ASTERISK-26788
+	  ASTERISK-27182
 
-	  Change-Id: I2c5c8edec17c9bdad4a93291576838cb552ca5ed
+	  Change-Id: I5d2f779248b84d41c5bb3896bf22ba324b336b03
 
-2017-02-11 11:25 +0000 [a46a21642e]  Richard Mudgett <rmudgett@digium.com>
+2017-08-05 14:43 +0000 [16cfc3a954]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip.c: Fix inconsistency between warning and action.
+	* channel: Fix leak on successful call to chan->tech->requester.
 
-	  The original return value corresponded to AST_SIP_AUTHENTICATION_CHALLENGE
-	  but we have no authenticator registered to create the challenge.
+	  joint_cap needs to be released unconditionally as chan->tech->requester
+	  does not steal the reference even on success.
 
-	  Change-Id: I62368180d774b497411b80fbaabd0c80841f8512
+	  ASTERISK-27180 #close
 
-2017-02-11 11:26 +0000 [67b21dc63a]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I647728992559bdb0a9c7357c20be1b36400d68b6
 
-	* pjsip_distributor.c: Fix off-nominal tdata ref leak.
+2017-08-04 16:47 +0000 [104a8047a5]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I571f371d0956a8039b197b4dbd8af6b18843598d
+	* res_pjsip_session/_sdp_rtp: Handling of 'msid' is incorrect
 
-2017-02-09 10:01 +0000 [8936568515]  Sean Bright <sean.bright@gmail.com>
+	  Currently, the handling of the msid attribute is not quite right. According to
+	  the spec the msid's between the offer/answer are not dependent upon one another.
+	  Meaning the same msid's given in an offer do not have to be returned in the
+	  answer for a given stream. And they probably shouldn't be (copied/reused) since
+	  this can potentially cause some browser side confusion.
 
-	* manager: Restore Originate failure behavior from Asterisk 11
+	  This patch generates new msids when both an offer and answer are sent from
+	  Asterisk. However, Asterisk does reuse the original msid it sent out for a
+	  reinvite. Also audio+video streams are paired together by sharing the same
+	  stream id, but a different track id.
 
-	  In Asterisk 11, if the 'Originate' AMI command failed to connect the provided
-	  Channel while in extension mode, a 'failed' extension would be looked up and
-	  run. This was, I believe, unintentionally removed in 51b6c49. This patch
-	  restores that behavior.
+	  ASTERISK-27179 #close
 
-	  This also adds an enum for the various 'synchronous' modes in an attempt to
-	  make them meaningful.
+	  Change-Id: Ifaec06dc7e65ad841633a24ebec8c8a9302d6643
 
-	  ASTERISK-26115 #close
-	  Reported by: Nasir Iqbal
+2017-08-03 20:58 +0000 [7f8f3ca4dd]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I8afbd06725e99610e02adb529137d4800c05345d
+	* Correct some leaks in unit tests.
 
-2017-02-08 14:27 +0000 [2817f87d27]  Richard Mudgett <rmudgett@digium.com>
+	  * chan_sip: channel in test_sip_rtpqos_1.
+	  * test_config: config hook, config info and global config holder.
+	  * test_core_format: format in format_attribute_set_without_interface.
+	  * test_stream: unneeded frame duplication.
+	  * test_taskprocessor: task_data.
 
-	* core: Cleanup some channel snapshot staging anomalies.
+	  Change-Id: I94d364d195cf3b3b5de2bf3ad565343275c7ad31
 
-	  We shouldn't unlock the channel after starting a snapshot staging because
-	  another thread may interfere and do its own snapshot staging.
+2017-07-26 17:49 +0000 [842e1414d0]  Richard Mudgett <rmudgett@digium.com>
 
-	  * app_dial.c:dial_exec_full() made hold the channel lock while setting up
-	  the outgoing channel staging.  Made hold the channel lock after the called
-	  party answers while updating the caller channel staging.
+	* res_pjsip_transport_websocket.c: Fix serializer ref leak.
 
-	  * chan_sip.c:sip_new() completed the channel staging on off-nominal exit.
-	  Also we need to use ast_hangup() instead of ast_channel_unref() at that
-	  location.
+	  Change-Id: Ib5a19bfd597f63d9021baeb645fc11153b3afa57
 
-	  * channel.c:__ast_channel_alloc_ap() added a comment about not needing to
-	  complete the channel snapshot staging on off-nominal exit paths.
+2017-08-02 18:41 +0000 [615b6a200a]  Richard Mudgett <rmudgett@digium.com>
 
-	  * rtp_engine.c:ast_rtp_instance_set_stats_vars() made hold the channel
-	  locks while staging the channels for the stats channel variables.
+	* res_pjsip_outbound_registration.c: Misc fixes.
 
-	  Change-Id: Iefb6336893163f6447bad65568722ad5d5d8212a
+	  * Remove unnecessary CMP_STOP.
 
-2017-02-10 09:35 +0000 [c7fcc4468f]  gtjoseph <gjoseph@digium.com>
+	  * In handle_client_registration() use DEBUG_ATLEAST() to only do work
+	  needed for the debug log message when the debug log message is needed.
 
-	* configs/samples: Fix placement of 'identify' entry in sorcery.conf
+	  * In sip_outbound_registration_state_destroy() check state->registration
+	  for NULL.
 
-	  The entry for 'identify' was incorrectly placed in the
-	  res_pjsip section when it should be in
-	  res_pjsip_endpoint_identifier_ip.
+	  Change-Id: I656d0fa11dda0b00048103efb1558e67a426fd80
 
-	  ASTERISK-26785 #close
+2017-07-31 20:20 +0000 [564927c5ed]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ia1372b12a952bfe2df6b1b1e0e725ca306a5d41a
+	* res_pjsip_nat.c: Remove unnecessary CMP_STOP.
 
-2017-02-08 11:50 +0000 [cbc23c31cf]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I6279b0d723bc3b75b8d65e81e02da9ea9bc0c3da
 
-	* Revert "Update qualifies when AOR configuration changes."
+2017-07-31 14:20 +0000 [5655cded78]  Richard Mudgett <rmudgett@digium.com>
 
-	  This reverts commit 6492e91392b8fd394193e411c6eb64b45486093f.
+	* res_pjsip_registrar.c: Remove unnecessary CMP_STOP.
 
-	  The change in question was intended to prevent the need to reload in
-	  order to update qualifies on contacts when an AOR changes. However, this
-	  ended up causing a deadlock instead.
+	  Most uses of CMP_STOP are superfluous and are only respected when
+	  OBJ_MULTIPLE is used to search the container.
 
-	  Change-Id: I1a835c90a5bb65b6dc3a1e94cddc12a4afc3d71e
+	  Change-Id: I20571a202ec0aa1098bb2749eeba18de7ca110b8
 
-2017-02-07 12:01 +0000 [7e14e086cf]  nappsoft <infos@nappsoft.com> (license 6822)
+2017-08-03 13:13 +0000 [123c93a77c]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* srv: Fix crash when ast_srv_lookup is used and 0 records are returned.
+	* Support GMIME 3.0
 
-	  When performing an SRV lookup using the ast_srv_lookup function it
-	  did not properly handle the situation where 0 records are returned.
-	  If this happened it would wrongly assume that at least one record
-	  was present.
+	  Support building the Asterisk httpd with version 3.0 of gmime as
+	  well as earlier versions of that library.
 
-	  This change fixes the code so it will exit early if an error occurs
-	  or if 0 records are returned.
+	  ASTERISK-27173
 
-	  ASTERISK-26772
-	  patches:
-	    srv_lookup.patch submitted by nappsoft (license 6822)
+	  Change-Id: I7e13dd05a3083ccb0df2dabf83110223f6a9fa8f
 
-	  Change-Id: I09b19081c74e0ad11c12bf54a257243b1bcb2351
+2017-08-02 09:43 +0000 [521b6fed12]  Kevin Harwell <kharwell@digium.com>
 
-2017-02-06 11:40 +0000 [7b39d6901a]  Joshua Colp <jcolp@digium.com>
+	* alembic/res_pjsip: Add "webrtc" configuration option
 
-	* res_stasis_device_state: Protect the adding/removing of subscriptions.
+	  When the "webrtc" option was added in res_pjsip it was not added to the alembic
+	  scripts. This patch adds the option for alembic.
 
-	  The adding and removing of device state subscriptions did not protect
-	  fully against simultaneous manipulation. In particular the subscribe
-	  case allowed a small window where two subscriptions could be added for
-	  the same device state instead of just one.
+	  Also, changed the sorcery configuration type to an OPT_YESNO_T value instead of
+	  an OPT_BOOL_T so if this field is ever written to a database it will write out
+	  the correct value.
 
-	  This change makes the code hold the subscriptions lock for the entirety
-	  of each operation to ensure that two are not occurring at the same time.
+	  ASTERISK-27119 #close
 
-	  ASTERISK-26770
+	  Change-Id: I3e199f060aea25e193c439fc5cf96be4d3ed1c7b
 
-	  Change-Id: I3e7f8eb9d09de440c9024d2dd52029f6f20e725b
+2017-07-30 01:17 +0000 [4c0798e91d]  Kirill Katsnelson <kkm@smartaction.com>
 
-2017-02-01 17:56 +0000 [c384dfd6b0]  Richard Mudgett <rmudgett@digium.com>
+	* chan_sip: Add dialplan function SIP_HEADERS
 
-	* res_pjsip: Fix some off nominal tdata leaks.
+	  Syntax: SIP_HEADERS([prefix])
 
-	  Change-Id: I243a4be5e7fbfe604923764969c4ee04eee89b9d
+	  If the argument is specified, only the headers matching the given prefix
+	  are returned.
 
-2017-02-06 16:27 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  The function returns a comma-separated list of SIP header names from an
+	  incoming INVITE message. Multiple headers with the same name are included
+	  in the list only once. The returned list can be iterated over using the
+	  functions POP() and SIP_HEADER().
 
-	* asterisk 13.14.0-rc1 Released.
+	  For example, '${SIP_HEADERS(Co)}' might return the string
+	  'Contact,Content-Length,Content-Type'.
 
-2017-02-02 11:26 +0000 [70aff89e5d]  Sean Bright <sean.bright@gmail.com>
+	  Practical use is rather '${SIP_HEADERS(X-)}' to enumerate optional
+	  extended headers sent by a peer.
 
-	* res_odbc: Remove deprecated settings from sample configuration file
+	  ASTERISK-27163
 
-	  ASTERISK-26704 #close
-	  Reported by: Anthony Messina
+	  Change-Id: I2076d3893d03a2f82429f393b5b46db6cf68a267
 
-	  Change-Id: I976a1f94cf79c5f31e76174c61f5c6a65fd6354f
+2017-08-02 14:16 +0000 [4b03eb5c38]  Corey Farrell <git@cfware.com>
 
-2017-02-01 15:56 +0000 [3aee199913]  Sean Bright <sean.bright@gmail.com>
+	* Fix compile error for old versions of GCC.
 
-	* audiohooks:  Muting a hook can mute underlying frames
+	  Use -Wno-format-truncation only if supported by compiler.
 
-	  If an audiohook is placed on a channel that does not require transcoding,
-	  muting that hook will cause the underlying frames to be muted as well.
+	  ASTERISK-27171 #close
 
-	  The original patch is from David Woolley but I have modified slightly.
+	  Change-Id: Iac0aed7a5bcaa16c21b7d62c4e4678d244c4ccb6
 
-	  ASTERISK-21094 #close
-	  Reported by: David Woolley
-	  Patches:
-	        ASTERISK-21094-Patch-1.8-1.txt (license #5737) patch uploaded
-	        by David Woolley
+2017-08-02 16:08 +0000 [148cf2e0f7]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ib2b68c6283e227cbeb5fa478b2d0f625dae338ed
+	* app_privacy: remove unused header asterisk/image.h
 
-2017-02-01 13:54 +0000 [6492e91392]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I56ed530633a642633b18383821069e806c92ae82
 
-	* Update qualifies when AOR configuration changes.
+2017-07-26 08:48 +0000 [2be8d91c0f]  snuffy <snuffy22@gmail.com> (license 5024)
 
-	  Prior to this change, qualifies would only update in the following
-	  cases:
-	  * A reload of res_pjsip.so was issued.
-	  * A dynamic contact was re-registered after its AOR's qualify_frequency
-	    had been changed
-	  This does not work well if you are using realtime for your AORs. You can
-	  update your database to have a new qualify_frequency, but the permanent
-	  contacts on that AOR will not have their qualifies updated. And the
-	  dynamic contacts on that AOR will not have their qualifies updated until
-	  the next registration, which could be a long time.
+	* res_pjsip_pidf_eyebeam_body_supplement: Correct status presentation
 
-	  This change seeks to fix this problem by making it so that whenever AOR
-	  configuration is applied, the contacts pertaining to that AOR have their
-	  qualifies updated.
+	  This change fixes PIDF content generation when the underlying device
+	  state is considered in use. Previously it was incorrectly marked
+	  as closed meaning they were offline/unavailable. The code now
+	  correctly marks them as open.
 
-	  Additions from this patch:
-	  * AOR sorcery objects now have an apply handler that calls into a newly
-	    added function in the OPTIONS code. This causes all contacts
-	    associated with that AOR to re-schedule qualifies.
-	  * When it is time to qualify a contact, the OPTIONS code checks to see
-	    if the AOR can still be retrieved. If not, then qualification is
-	    canceled on the contact.
+	  Additionally:
 
-	  Alterations from this patch:
-	  * The registrar code no longer updates contact's qualify_frequence and
-	    qualify_timeout. There is no point to this since those values already
-	    get updated when the AOR changes.
-	  * Reloading res_pjsip.so no longer calls the OPTIONS initialization
-	    function. Reloading res_pjsip.so results in re-loading AORs, which
-	    results in re-scheduling qualifies.
+	    * Generate an XML element for our activity instead of a using a text
+	      node.
 
-	  Change-Id: I2e7c3316da28f389c45954f24c4e9389abac1121
+	    * Consider every extension state other than "unavailable" to be 'open'
+	      status.
 
-2017-01-31 18:28 +0000 [43f0ff4b69]  Richard Mudgett <rmudgett@digium.com>
+	    * Update the XML namespaces and structure to reflect those
+	      documented in RFC 4480
 
-	* channel.c: Fix unbalanced read queue deadlocking local channels.
+	    * Use 'on-the-phone' (defined in RFC 4880) instead of 'busy' as the
+	      "in use" activity. This change results in eyeBeam using the
+	      appropriate icon for the watched user.
 
-	  Using the timerfd timing module can cause channel freezing, lingering, or
-	  deadlock issues.  The problem is because this is the only timing module
-	  that uses an associated alert-pipe.  When the alert-pipe becomes
-	  unbalanced with respect to the number of frames in the read queue bad
-	  things can happen.  If the alert-pipe has fewer alerts queued than the
-	  read queue then nothing might wake up the thread to handle received frames
-	  from the channel driver.  For local channels this is the only way to wake
-	  up the thread to handle received frames.  Being unbalanced in the other
-	  direction is less of an issue as it will cause unnecessary reads into the
-	  channel driver.
+	  This was tested on eyeBeam 1.5.20.2 build 59030 on Windows.
 
-	  ASTERISK-26716 is an example of this deadlock which was indirectly fixed
-	  by the change that found the need for this patch.
+	  ASTERISK-26659 #close
+	  Reported by: Abraham Liebsch
+	  patches:
+	    ASTERISK-26659.diff submitted by snuffy (license 5024)
 
-	  * In channel.c:__ast_queue_frame(): Adding frame lists to the read queue
-	  did not add the same number of alerts to the alert-pipe.  Correspondingly,
-	  when there is an exceptionally long queue event, any removed frames did
-	  not also remove the corresponding number of alerts from the alert-pipe.
+	  Change-Id: I6e5ad450f91106029fb30517b8c0ea0c2058c810
 
-	  ASTERISK-26632 #close
+2017-07-23 18:34 +0000 [2a4283f3e7]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ia98137c5bf6e9d6d202ce0eb36441851875863f6
+	* res_pjsip: Add support for dnsmgr to external_media_address.
 
-2017-01-31 16:38 +0000 [a199f94908]  Richard Mudgett <rmudgett@digium.com>
+	  The "external_media_address" option on transports is now
+	  resolved using dnsmgr. This allows it to be automatically
+	  refreshed regularly if refreshes are enabled in dnsmgr.
+	  If the system is using a dynamic IP address a dynamic DNS
+	  hostname can be provided to keep the IP address up to
+	  date.
 
-	* res_agi: Prevent an AGI from eating frames it should not. (Re-do)
+	  Change-Id: Ia54771720dff0105bde55d5bbb81a3ba437e05b2
 
-	  A dialplan intercept routine is equivalent to an interrupt routine.  As
-	  such, the routine must be done quickly and you do not have access to the
-	  media stream.  These restrictions are necessary because the media stream
-	  is the responsibility of some other code and interfering with or delaying
-	  that processing is bad.  A possible future dialplan processing
-	  architecture change may allow the interception routine to run in a
-	  different thread from the main thread handling the media and remove the
-	  execution time restriction.
+2017-07-27 20:58 +0000 [58d032112b]  Corey Farrell <git@cfware.com>
 
-	  * Made res_agi.c:run_agi() running an AGI in an interception routine run
-	  in DeadAGI mode.  No touchy channel frames.
+	* Fix compiler warnings on Fedora 26 / GCC 7.
 
-	  ASTERISK-25951
+	  GCC 7 has added capability to produce warnings, this fixes most of those
+	  warnings.  The specific warnings are disabled in a few places:
 
-	  ASTERISK-26343
+	  * app_voicemail.c: truncation of paths more than 4096 chars in many places.
+	  * chan_mgcp.c: callid truncated to 80 chars.
+	  * cdr.c: two userfields are combined to cdr copy, fix would break ABI.
+	  * tcptls.c: ignore use of deprecated method SSLv3_client_method().
 
-	  ASTERISK-26716
+	  ASTERISK-27156 #close
 
-	  Change-Id: I638f147ca7a7f2590d7194a8ef4090eb191e4e43
+	  Change-Id: I65f280e7d3cfad279d16f41823a4d6fddcbc4c88
 
-2017-01-31 16:32 +0000 [6bed318a66]  Richard Mudgett <rmudgett@digium.com>
+2017-07-26 09:27 +0000 [3f98488279]  Sean Bright <sean.bright@gmail.com>
 
-	* Frame deferral: Revert API refactoring.
+	* app_queue: Add announce-position-only-up option
 
-	  There are several issues with deferring frames that are caused by the
-	  refactoring.
+	  Setting this option will cause the Queue application to only announce
+	  the caller's position if it has improved since the last time that we
+	  announced it.
 
-	  1) The code deferring frames mishandles adding a deferred frame to the
-	  deferred queue.  As a result the deferred queue can only be one frame
-	  long.
+	  Change-Id: I173a124121422209485b043e2bf784f54242fce6
 
-	  2) Deferrable frames can come directly from the channel driver as well as
-	  the read queue.  These frames need to be added to the deferred queue.
+2017-07-27 06:35 +0000 [ac6d98b28d]  Ian Gilmour (license 6889)
 
-	  3) Whoever is deferring frames is really only doing the __ast_read() to
-	  collect deferred frames and doesn't care about the returned frames except
-	  to detect a hangup event.  When frame deferral is completed we must make
-	  the normal frame processing see the hangup as a frame anyway.  As such,
-	  there is no need to have varying hangup frame deferral methods.  We also
-	  need to be aware of the AST_SOFTHANGUP_ASYNCGOTO hangup that isn't real.
-	  That fake hangup is to cause the PBX thread to break out of loops to go
-	  execute a new dialplan location.
+	* bundled_pjproject:  Improve SSL/TLS error handling
 
-	  4) To properly deal with deferrable frames from the channel driver as
-	  pointed out by (2) above, means that it is possible to process a dialplan
-	  interception routine while frames are deferred because of the
-	  AST_CONTROL_READ_ACTION control frame.  Deferring frames is not
-	  implemented as a re-entrant operation so you could have the unsupported
-	  case of two sections of code thinking they have control of the media
-	  stream.
+	  OpenSSL has 2 levels or error processing.  It's possible for the
+	  top layer to return SSL_ERROR_SYSCALL but the lower layer return
+	  no error, in which case processing should continue.  Only the top
+	  layer was being examined though so connections were being torn
+	  down when they didn't need to be.  This patch adds the examination
+	  of the lower level codes, and if they return no errors, allows
+	  processing to continue.
 
-	  A worse problem is because of the bad implementation of the AMI PlayDTMF
-	  action.  It can cause two threads to be deferring frames on the same
-	  channel at the same time.  (ASTERISK_25940)
+	  ASTERISK-27001
+	  Reported-by: Ian Gilmour
+	  patches:
+	  	pjproject-2.6.patch submitted by Ian Gilmour (license 6889)
 
-	  * Rather than fix all these problems simply revert the API refactoring as
-	  there is going to be only autoservice and safe_sleep deferring frames
-	  anyway.
+	  Updated-by: George Joseph and Sauw Ming (Teluu)
 
-	  ASTERISK-26343
+	  Merged to upstream pjproject on 7/27/2017 (commit 5631)
 
-	  ASTERISK-26716 #close
+	  Change-Id: I23844ca0c68ef1ee550f14d46f6dae57d33b7bd2
 
-	  Change-Id: I45069c779aa3a35b6c863f65245a6df2c7865496
+2017-06-26 07:52 +0000 [65c560894d]  Torrey Searle <torrey@voxbone.com>
 
-2017-01-31 11:17 +0000 [e371e13b9e]  Joshua Colp <jcolp@digium.com>
+	* chan_pjsip: add a new function PJSIP_DTMF_MODE
 
-	* res_pjsip: Handle invocation of callback on outgoing request when error occurs.
+	  This function is a replica of SIPDtmfMode, allowing the DTMF mode of a
+	  PJSIP call to be modified on a per-call basis
 
-	  There are some error cases in PJSIP when sending a request that will
-	  result in the callback for the request being invoked.  The code did not
-	  handle this case and assumed on every error case that the callback was not
-	  invoked.
+	  ASTERISK-27085 #close
 
-	  The code has been changed to check whether the callback has been invoked
-	  and if so to absorb the error and treat it as a success.
+	  Change-Id: I20eef5da3e5d1d3e58b304416bc79683f87e7612
 
-	  ASTERISK-26679
-	  ASTERISK-26699
+2017-07-25 15:17 +0000 [b3914df10b]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I563982ba204da5aa1428989a11c06dd9087fea91
+	* res_rtp_asterisk: Fix mapping of pjsip's ICE roles to ours
 
-2017-01-30 09:02 +0000 [339c30f2b6]  Sean Bright <sean.bright@gmail.com>
+	  Change-Id: Ia578ede1a55b21014581793992a429441903278b
 
-	* res_rtp_asterisk:  Swap byte-order when sending signed linear
+2017-07-20 08:08 +0000 [4f4936fd72]  Sergej Kasumovic <sergej@bicomsystems.com>
 
-	  Before Asterisk 13, signed linear was converted into network byte order by a
-	  smoother before being sent over the network. We restore this behavior by
-	  forcing the creation of a smoother when slinear is in use and setting the
-	  appropriate flags so that the byte order conversion is always done.
+	* res_stasis_device_state: Unsubscribe should remove old subscriptions
 
-	  ASTERISK-24858 #close
-	  Reported-by: Frankie Chin
+	  Case scenario with Applications ARI:
 
-	  Change-Id: I868449617d1a7819578f218c8c6b2111ad84f5a9
+	  * Once you subscribe to deviceState with Applications REST API, it will be
+	  added into subscription pool.
 
-2017-01-31 12:46 +0000 [7fd28cefdb]  gtjoseph <gjoseph@digium.com>
+	  * When you unsubscribe it will remove from the device_state_subscription
+	  hash table but not from the subscription pool.
 
-	* debug_utilities: Install ast_logescalator to /var/lib/asterisk/scripts
+	  * When you subscribe again, it will add it to pool again.
 
-	  Forgot to install it with the original patch
+	  * Now you will have two subscriptions and you will receive same event
+	  twice.
 
-	  Change-Id: I8bdb540a6694971ae5fe21f48d532332c6482e4c
+	  This fix should now remove deviceState subscription from pool and it
+	  should fix unsubscribe on deviceState.
 
-2017-01-25 06:50 +0000 [456bc3c704]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-27130 #close
 
-	* debug_utilities:  Add ast_logescalator
+	  Change-Id: I718b70d770a086e39b4ddba4f69a3c616d4476c4
 
-	  The escalator works by creating a set of startup commands in cli.conf
-	  that set up logger channels and issue the debug commands for the
-	  subsystems specified.  If asterisk is running when it is executed,
-	  the same commands will be issued to the running instance.  The original
-	  cli.conf is saved before any changes are made and can be restored by
-	  executing '$prog --reset'.
+2017-07-24 13:30 +0000 [a6eb9ee7d2]  Joshua Colp <jcolp@digium.com>
 
-	  The log output will be stored in...
-	  $astlogdir/message.$uniqueid
-	  $astlogdir/debug.$uniqueid
-	  $astlogdir/dtmf.$uniqueid
-	  $astlogdir/fax.$uniqueid
-	  $astlogdir/security.$uniqueid
-	  $astlogdir/pjsip_history.$uniqueid
-	  $astlogdir/sip_history.$uniqueid
+	* core: Add VP9 passthrough support.
 
-	  Some minor tweaks were made to chan_sip, and res_pjsip_history
-	  so their history output could be send to a log channel as packets
-	  are captured.
+	  This change adds VP9 as a known codec and creates a cached
+	  "vp9" media format for use.
 
-	  A minor tweak was also made to manager so events are output to verbose
-	  when "manager set debug on" is issued.
+	  Change-Id: I025a93ed05cf96153d66f36db1839109cc24c5cc
 
-	  Change-Id: I799f8e5013b86dc5282961b27383d134bf09e543
+2017-07-19 18:11 +0000 [922930753c]  Richard Mudgett <rmudgett@digium.com>
 
-2017-01-23 09:35 +0000 [54b027916a]  Torrey Searle <torrey@voxbone.com>
+	* app_voicemail.c: Allow mailbox entry on authentication retry prompt.
 
-	* libastssl/pj: libastssl/pj should have an so_version
+	  The following testsuite voicemail tests were failing to re-enter the
+	  mailbox after the first login attempt.
 
-	  Issue introduced in b59956a87.  In the non-darwin case libastssl/pj
-	  should be versioned.  This causes the symbol file for this lib
-	  to not be generated.
+	  tests/apps/voicemail/authenticate_invalid_mailbox
+	  tests/apps/voicemail/authenticate_invalid_password
 
-	  Change-Id: Ib07ae8c40252813c488e2c1ac6204fd42816dd4c
+	  The tests were noting the start of the vm-incorrect-mailbox prompt and
+	  immediately sending the mailbox for the next login attempt.  Since the
+	  invalid message playback had to complete before the digits were
+	  recognized, the test passed for the wrong reason and added approximately
+	  20 seconds to the test times.
 
-2017-01-24 19:51 +0000 [3c8f84786e]  Kirill Katsnelson <kkm@smartaction.com>
+	  * Allow the vm-incorrect-mailbox prompt to get interrupted by the mailbox
+	  digits like the initial vm-login prompt so the tests are able to enter the
+	  intended mailbox.
 
-	* make_build_h: handle backslashes in external strings
+	  Change-Id: I1dc53fe917bfe03a4587b2c4cd24c94696a69df8
 
-	  LikewiseOpen creates user names with a backslash in them. A gentle
-	  massage with sed(1) allows such strings to be inserted into build.h
-	  properly quoted. I am also adding the same for host name and other
-	  strings used in the script that are more or less user-controlled.
+2017-07-21 15:57 +0000 [2697e45157]  Matthew Fredrickson <creslin@digium.com>
 
-	  ASTERISK-26754
+	* format.h: Fix a few minor errors in comments.
 
-	  Change-Id: Iac5ef2b67a68ee58f35ddbf86bb818ba6eabecae
+	  A few minor problems were found in comments in format.h.  This patch fixes them.
 
-2017-01-17 20:46 +0000 [555e8cd2ba]  Kirill Katsnelson <kkm@smartaction.com>
+	  Change-Id: I07f0bdb47b93359b361c4c3d8ecc87cd3199dd94
 
-	* ast_careful_fwrite to support EPIPE gracefully
+2017-07-14 13:47 +0000 [19b080b547]  Rusty Newton <rnewton@digium.com>
 
-	  When a reading end of the network socket is closed by an AMI manager,
-	  the EPIPE is signaled when writing to our end, resulting in the
-	  spurious log error message
+	* say.c: Fix file locations for second, seconds, minute, minutes files
 
-	    ast_careful_fwrite: fwrite() returned error: Broken pipe
+	  The seconds and minutes files have always existed in the base language
+	  directory of the Core package. So say.c has always been calling the wrong
+	  location (under digits/) for those two files and in the case of second and
+	  minute they didn't exist in the Core packages at all.
 
-	  Previously EPIPE was handled in ast_carefulwrite() a few lines above,
-	  but not in this function.
+	  The 1.6 sounds release moves the second and minute files into Core from
+	  Extra for the languages that already had them. A future release will include
+	  the second and minute files for languages that didn't already have them.
 
-	  ASTERISK-26753
+	  This patch just changes all the target locations for second, seconds,
+	  minute, and minutes that were under the digits subdir to be under the root of
+	  sounds instead. Which is where the sounds will be for some languages after 1.6
+	  sounds and for all languages after a future release.
 
-	  Change-Id: I6a67335cd6526608bb9b78f796c626b1677664b8
+	  ASTERISK-25810 #close
 
-2017-01-24 22:31 +0000 [be92f10a16]  Kirill Katsnelson <kkm@smartaction.com>
+	  Change-Id: I05d9d4bee6a7237030530a46e7eb3df15f13f702
+	  Reported-by: Nicolas Riendeau
 
-	* app_queue: Fix queues randomly disappearing on reload
+2017-07-21 14:20 +0000 [a2f6028a51]  Rusty Newton <rnewton@digium.com>
 
-	  With 500+ queues and a reload every minute, a random queue disappears
-	  upon reload. The cause is mususe of the 'dead' flag. Namely, all queues
-	  were marked dead up front, and then "resurrected" by dropping this flag
-	  for those found in the configuration. But a queue marked dead can be
-	  removed also when control leaves the app entry point on a PBX thread.
+	* Sounds: Update Makefile for Extra sounds 1.5.1 release
 
-	  With this change, the queue is marked only not found, and at the end of
-	  reload only the queues that are still not found are actually marked as
-	  dead, so the dead flag is never reset, and set only on positively dead
-	  queues.
+	  Incrementing version for the Extra sounds release. 1.5.1 Extra sounds
+	  removes two prompts that were moved into the Core packages in the 1.6 Core
+	  sounds release.
 
-	  ASTERISK-26755
+	  ASTERISK-27142 #close
 
-	  Change-Id: I3a4537aec9eb8d8aeeaa0193407e3523feb004bf
+	  Change-Id: I82f017812b0ea9599e19dd4635afd55611f13ee7
 
-2017-01-26 07:57 +0000 [aae9df0643]  Joshua Colp <jcolp@digium.com>
+2017-07-21 11:17 +0000 [063c9a935f]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip_endpoint_identifier_ip: Fix memory leak of hosts when resolving.
+	* Update make_ari_stubs in master to make the version 16
 
-	  This change adds a missing unreference of the hostname when resolving and
-	  also cleans up the iterator.
+	  Ready for next major version
 
-	  ASTERISK-26735
+	  Change-Id: If9dc99b3b78768529e69a297d8f87e23582ca6d0
 
-	  Change-Id: Ic012ebaf3d89e714eec340b7b0c5e63c66af857a
+2017-07-21 11:24 +0000 [ba52a36ff2]  George Joseph <gjoseph@digium.com>
 
-2017-01-25 15:26 +0000 [9e3150b98d]  Mark Michelson <mmichelson@digium.com>
+	* Restore the incorrectly deleted spandspflow2pcap.log
 
-	* Add reload options to CLI/AMI stale object commands.
+	  Change-Id: Iafe78cf0fb1e7064223d4dea279eeb776c8fa8e5
 
-	  Marking an object as stale in a memory cache is supposed to prime the
-	  cache so that the next time the item is retrieved, the stale item is
-	  deleted from the cache and a background task is run to re-populate the
-	  cache with a fresh version of the object.
+2017-07-20 09:57 +0000 [25c9464325]  Sean Bright <sean.bright@gmail.com>
 
-	  The problem is, there are some object types out there for which there is
-	  no natural reason that they would be retrieved from the backend with any
-	  regularity. Outbound PJSIP registrations are a good example of this. At
-	  startup, they are read, and an object-specific state is created that
-	  refers to the initially-retrieved object for all time.
+	* corosync: Fix corosync library name in configure.ac
 
-	  Adding the "reload" option to the CLI/AMI commands gives the cache the
-	  opportunity to manually re-retrieve the object from the backend, both
-	  storing the new object in the cache and applying the new object's
-	  configuration to the module that uses that object.
+	  Also add new corosync packages to install_prereq.
 
-	  Change-Id: Ieb1fe7270ceed491f057ec5cbf0e097bde96c5c8
+	  Reported by Travis Ryan in #asterisk-dev
 
-2017-01-10 17:39 +0000 [c54f9d2bf0]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ib861c95ba630fed62dc54e56784ad8446ed9d2db
 
-	* T.140: Fix format ref and memory leaks.
+2017-07-17 11:01 +0000 [680c491a62]  Joshua Colp <jcolp@digium.com>
 
-	  * channel.c:ast_sendtext(): Fix T.140 SendText memory leak.
+	* bridge_softmix / res_rtp_asterisk: Fix packet loss and renegotiation issues.
 
-	  * format_compatibility.c: T.140 RED and T.140 were swapped.
+	  This change does a few things to improve packet loss and renegotiation:
 
-	  * res_rtp_asterisk.c:rtp_red_init(): Fix ast_format_t140_red ref leak.
+	  1. On outgoing RTP streams we will now properly reflect out of order
+	  packets and packet loss in the sequence number. This allows the
+	  remote jitterbuffer to better reorder things.
 
-	  * res_rtp_asterisk.c:rtp_red_init(): Fix data race after starting periodic
-	  scheduled red_write().
+	  2. Video updates can now be discarded for a period of time
+	  after one has been sent to prevent flooding of clients.
 
-	  * res_rtp_asterisk.c: Some other minor misc tweaks.
+	  3. For declined and removed streams we will now release any
+	  media session resources associated with them. This was not
+	  previously done and caused an issue where old state was being
+	  used for a new stream.
 
-	  Change-Id: Ifa27a2e0f8a966b1cf628607c86fc4374b0b88cb
+	  4. RTP bundling was not actually removing bundled RTP instances
+	  from the parent. This has been resolved by removing based on
+	  the RTP instance itself and not the SSRC.
 
-2017-01-24 15:39 +0000 [a2f0adccbd]  Joshua Colp <jcolp@digium.com>
+	  5. The code did not properly handle explicitly unbundling an
+	  RTP instance from its parent. This now works as expected.
 
-	* res_pjsip_endpoint_identifier_ip: Ensure error defaults to 0.
+	  ASTERISK-27143
 
-	  When configuring a match using a netmask the error variable was
-	  not defaulting to 0. For some people this would cause the code
-	  to think an error occurred when adding the match when in reality
-	  it added perfectly fine.
+	  Change-Id: Ibd91362f0e4990b6129638e712bc8adf0899fd45
 
-	  ASTERISK-26693
+2017-05-19 23:28 +0000 [d2fbbdd692]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I850c250813742bddde65c84e739093c9e01dfe56
+	* SDP: Create declined m= SDP lines using remote SDP if applicable.
 
-2017-01-10 17:37 +0000 [607b3ac736]  Richard Mudgett <rmudgett@digium.com>
+	  * Update SDP unit tests to test negotiating with declined streams.
+	  Generation of declined m= lines created and responded tested.
 
-	* astobj2.c: Add excessive ref count trap.
+	  Change-Id: I5cb99f5010994ab0c7d9cf2d395eca23fab37b98
 
-	  Change-Id: I32e6a589cf9009450e4ff7cb85c07c9d9ef7fe4a
+2017-05-02 18:51 +0000 [3a18a09030]  Richard Mudgett <rmudgett@digium.com>
 
-2017-01-10 13:11 +0000 [ab8cb5a7ce]  Richard Mudgett <rmudgett@digium.com>
+	* SDP: Rework SDP offer/answer model and update capabilities merges.
 
-	* main/app.c: Memory corruption from early format destruction.
-
-	  * make_silence() created a malloced silence slin frame without adding a
-	  slin format ref.  When the frame is destroyed it will unref the slin
-	  format that never had a ref added.  Memory corruption is expected to
-	  follow.
+	  The SDP offer/answer model requires an answer to an offer before a new SDP
+	  can be processed.  This allows our local SDP creation to be deferred until
+	  we know that we need to create an offer or an answer SDP.  Once the local
+	  SDP is created it won't change until the SDP negotiation is restarted.
 
-	  * Simplified and fixed counting the number of samples in a frame list for
-	  make_silence().
+	  An offer SDP in an initial SIP INVITE can receive more than one answer
+	  SDP.  In this case, we need to merge each answer SDP with our original
+	  offer capabilities to get the currently negotiated capabilities.  To
+	  satisfy this requirement means that we cannot update our proposed
+	  capabilities until the negotiations are restarted.
 
-	  * Eliminated an unnecessary RAII_VAR associated with the make_silence()
-	  frame.
+	  Local topology updates from ast_sdp_state_update_local_topology() are
+	  merged together until the next offer SDP is created.  These accumulated
+	  updates are then merged with the current negotiated capabilities to create
+	  the new proposed capabilities that the offer SDP is built.
 
-	  Change-Id: I47de3f9b92635b7f8b4d72309444d6c0aee6f747
+	  Local topology updates are merged in several passes to attempt to be smart
+	  about how streams from the system are matched with the previously
+	  negotiated stream slots.  To allow for T.38 support when merging, type
+	  matching considers audio and image types to be equivalent.  First streams
+	  are matched by stream name and type.  Then streams are matched by stream
+	  type only.  Any remaining unmatched existing streams are declined.  Any
+	  new active streams are either backfilled into pre-merge declined slots or
+	  appended onto the end of the merged topology.  Any excess new streams
+	  above the maximum supported number of streams are simply discarded.
 
-2017-01-11 14:59 +0000 [dcd8e4b1a0]  Richard Mudgett <rmudgett@digium.com>
+	  Remote topology negotiation merges depend if the topology is an offer or
+	  answer.  An offer remote topology negotiation dictates the stream slot
+	  ordering and new streams can be added.  A remote offer can do anything to
+	  the previously negotiated streams except reduce the number of stream
+	  slots.  An answer remote topology negotiation is limited to what our offer
+	  requested.  The answer can only decline streams, pick codecs from the
+	  offered list, or indicate the remote's stream hold state.
 
-	* frame.c: Fix off-nominal format ref leaks.
+	  I had originally kept the RTP instance if the remote offer SDP changed a
+	  stream type between audio and video since they both use RTP.  However, I
+	  later removed this support in favor of simply creating a new RTP instance
+	  since the stream's purpose has to be changing anyway.  Any RTP packets
+	  from the old stream type might cause mischief for the bridged peer.
 
-	  * ast_frisolate() could leak frame format refs on allocation
-	  failures.
+	  * Added ast_sdp_state_restart_negotiations() to restart the SDP
+	  offer/answer negotiations.  We will thus know to create a new local SDP
+	  when it is time to create an offer or answer.
 
-	  * Similified code in ast_frisolate() and code used by
-	  ast_frisolate().
+	  * Removed ast_sdp_state_reset().  Save the current topology before
+	  starting T.38.  To recover from T.38 simply update the local topology to
+	  the saved topology and restart the SDP negotiations to get the offer SDP
+	  renegotiating the previous configuration.
 
-	  Change-Id: I79566d4d36b3d7801bf0c8294fcd3e9a86a2ed6d
+	  * Allow initial topology for ast_sdp_state_alloc() to be NULL so an
+	  initial remote offer SDP can dictate the streams we start with.  We can
+	  always update the local topology later if it turns out we need to offer
+	  SDP first because the remote chose to defer sending us a SDP.
 
-2017-01-13 19:08 +0000 [00a227e93d]  Richard Mudgett <rmudgett@digium.com>
+	  * Made the ast_sdp_state_alloc() initial topology limit to max_streams,
+	  limit to configured codecs, handle declined streams, and discard
+	  unsupported types.
 
-	* stasis_bridge.c: Fix off-nominal stasis control ref leak.
+	  * Convert struct ast_sdp to ao2 object.  Needed to easily save off a
+	  remote SDP to refer to later for various reasons such as generating
+	  declined m= lines in the local SDP.
 
-	  Change-Id: Ib17218343a6596832060180e19386da9df150ac8
+	  * Improve converting remote SDP streams to a topology including stream
+	  state.  A stream state of AST_STREAM_STATE_REMOVED indicates the stream is
+	  declined/dead.
 
-2017-01-10 12:30 +0000 [38a2021c68]  Richard Mudgett <rmudgett@digium.com>
+	  * Improve merging streams to take into account the stream state.
 
-	* res_musiconhold.c: Fix format ref leak when parsing MOH config class.
+	  * Added query for remote hold state.
 
-	  Change-Id: Ica8e8e2ce7604c2c61ec55bef07dc675361d2ea5
+	  * Added maximum streams allowed SDP config option.
 
-2017-01-10 14:03 +0000 [ab7a9fc5b2]  Richard Mudgett <rmudgett@digium.com>
+	  * Added ability to create new streams as needed.  New streams are created
+	  with configured default audio, video, or image codecs depending on stream
+	  type.
 
-	* chan_oss.c: Fix format ref leak in oss_read().
+	  * Added global locally_held state along with a per stream local hold
+	  state.  Historically, Asterisk only has a global locally held state
+	  because when the we put the remote on hold we do it for all active
+	  streams.
 
-	  Change-Id: I0a5d56c7dcf327d60f86a4c25a23571733709fd0
+	  * Added queries for a rejected offer and current SDP negotiation role.
+	  The rejected query allows the using module to know how to respond to a
+	  failed remote SDP set.  Should the using module respond with a 488 Not
+	  Acceptable Here or 500 Internal Error to the offer SDP?
 
-2017-01-10 17:48 +0000 [1484a991e1]  Richard Mudgett <rmudgett@digium.com>
+	  * Moved sdp_state_capabilities.connection_address to ast_sdp_state.  There
+	  seems no reason to keep it in the sdp_state_capabilities struct since it
+	  was only used by the ast_sdp_state.proposed_capabilities instance.
 
-	* Add notes about embedded ast_frame structs holding a format ref.
+	  * Callbacks are now available to allow the using module some customization
+	  of negotiated streams and to complete setting up streams for use.  See the
+	  typedef doxygen for each callback for what is allowable and when they are
+	  called.
+	      * Added topology answerer modify callback.
+	      * Added topology pre and post apply callbacks.
+	      * Added topology offerer modify callback.
+	      * Added topology offerer configure callback.
 
-	  mod_format.h: Note ast_filestream.fr holds a format ref.
+	  * Had to rework the unit tests because I changed how SDP topologies are
+	  merged.  Replaced several unit tests with new negotiation tests.
 
-	  translate.h: Note ast_trans_pvt.f holds a format ref.
+	  Change-Id: If07fe6d79fbdce33968a9401d41d908385043a06
 
-	  Change-Id: I86bda354d725207b41e08920355d7c31b2d7f749
+2017-06-18 19:24 +0000 [70d2ccb9da]  Corey Farrell <git@cfware.com>
 
-2017-01-19 09:05 +0000 [17f4989d49]  gtjoseph <gjoseph@digium.com>
+	* Core: Add support for systemd socket activation.
 
-	* ari: Implement 'debug all' and request/response logging
+	  This change adds support for socket activation of certain SOCK_STREAM
+	  listeners in Asterisk:
+	  * AMI / AMI over TLS
+	  * CLI
+	  * HTTP / HTTPS
 
-	  The 'ari set debug' command has been enhanced to accept 'all' as an
-	  application name.  This allows dumping of all apps even if an app
-	  hasn't registered yet.  To accomplish this, a new global_debug global
-	  variable was added to res/stasis/app.c and new APIs were added to
-	  set and query the value.
+	  Example systemd units are provided.  This support extends to any socket
+	  which is initialized using ast_tcptls_server_start, so any unknown
+	  modules using this function will support socket activation.
 
-	  'ari set debug' now displays requests and responses as well as events.
-	  This required refactoring the existing debug code.
+	  Asterisk continues to function as normal if socket activation is not
+	  enabled or if systemd development headers are not available during
+	  build.
 
-	  * The implementation for 'ari set debug' was moved from stasis/cli.{c,h}
-	    to ari/cli.{c,h}, and stasis/cli.{c,h} were deleted.
-	  * In order to print the body of incoming requests even if a request
-	    failed, the consumption of the body was moved from the ari stubs
-	    to ast_ari_callback in res_ari.c and the moustache templates were
-	    then regenerated.  The body is now passed to ast_ari_invoke and then
-	    on to the handlers.  This results in code savings since that template
-	    was inserted multiple times into all the stubs.
+	  ASTERISK-27063 #close
 
-	  An additional change was made to the ao2_str_container implementation
-	  to add partial key searching and a sort function.  The existing cli
-	  code assumed it was already there when it wasn't so the tab completion
-	  was never working.
+	  Change-Id: Id814ee6a892f4b80d018365c8ad8d89063474f4d
 
-	  Change-Id: Ief936f747ce47f1fb14035fbe61152cf766406bf
+2017-09-01 19:29 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-2017-01-20 21:13 +0000 [30cb4eb57f]  Richard Mudgett <rmudgett@digium.com>
+	* asterisk 15.0.0-rc1 Released.
 
-	* PJPROJECT logging: Fix detection of max supported log level.
+2017-07-24 10:48 +0000 [35c8fb1590]  George Joseph <gjoseph@digium.com>
 
-	  The mechanism used for detecting the maximum log level compiled into the
-	  linked pjproject did not work.  The API call simply stores the requested
-	  level into an integer and does no range checking.  Asterisk was assuming
-	  that there was range checking and limited the new value to the allowable
-	  range.  To get the actual maximum log level compiled into the linked
-	  pjproject we need to get and save off the initial set log level from
-	  pjproject.  This is the maximum log level supported.
+	* pjsip_message_ip_updater:  Fix issue handling "tel" URIs
 
-	  * Get and save off the initial log level setting before altering it to the
-	  desired level on startup.  This has to be done by a macro rather than
-	  calling a core function to avoid incorrectly linking pjproject.
+	  sanitize_tdata was assuming all URIs were SIP URIs so when a non
+	  SIP uri was in the From, To or Contact headers, the unconditional
+	  cast of a non-pjsip_sip_uri structure to pjsip_sip_uri caused
+	  a segfault when trying to access uri->other_param.
 
-	  * Split the initial log level warning messages to warn if the linked
-	  pjproject cannot support the requested startup level and if it is too low
-	  to get the pjproject buildopts for "pjproject show buildopts".
+	  * Added PJSIP_URI_SCHEME_IS_SIP(uri) || PJSIP_URI_SCHEME_IS_SIPS(uri)
+	    checks before attempting to cast or use the returned uri.
 
-	  * Adjust the CLI "pjproject set log level" to check the saved max log
-	  level and to generate normal output messages instead of a warning message.
+	  ASTERISK-27152
+	  Reported-by: Ross Beer
 
-	  ASTERISK-26743 #close
+	  Change-Id: Id380df790e6622c8058a96035f8b8f4aa0b8551f
 
-	  Change-Id: I40aa76653e2a1dece66c3f8734594b4f0471cfb4
+2017-07-01 19:24 +0000 [231ee5e6c6]  Corey Farrell <git@cfware.com>
 
-2017-01-21 14:43 +0000 [cd2677f966]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* AST-2017-006: Fix app_minivm application MinivmNotify command injection
 
-	* tests: use datadir for sound files
+	  An admin can configure app_minivm with an externnotify program to be run
+	  when a voicemail is received.  The app_minivm application MinivmNotify
+	  uses ast_safe_system() for this purpose which is vulnerable to command
+	  injection since the Caller-ID name and number values given to externnotify
+	  can come from an external untrusted source.
 
-	  Some (voicemail-related) tests API symlinks beep.gsm and other files
-	  from ast_config_AST_VAR_DIR. It should use ast_config_AST_DATA_DIR.
+	  * Add ast_safe_execvp() function.  This gives modules the ability to run
+	  external commands with greater safety compared to ast_safe_system().
+	  Specifically when some parameters are filled by untrusted sources the new
+	  function does not allow malicious input to break argument encoding.  This
+	  may be of particular concern where CALLERID(name) or CALLERID(num) may be
+	  used as a parameter to a script run by ast_safe_system() which could
+	  potentially allow arbitrary command execution.
 
-	  ASTERISK-26740 #close
+	  * Changed app_minivm.c:run_externnotify() to use the new ast_safe_execvp()
+	  instead of ast_safe_system() to avoid command injection.
 
-	  Change-Id: Id49c56fb9e16df64b1a2b829693ca7601252df89
+	  * Document code injection potential from untrusted data sources for other
+	  shell commands that are under user control.
 
-2017-01-20 23:41 +0000 [b62f84bfb1]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  ASTERISK-27103
 
-	* test_voicemail_api: order of params to VERIFY macros
+	  Change-Id: I7552472247a84cde24e1358aaf64af160107aef1
 
-	  Fix order of parameters in calls to VM_API_INT_VERIFY and
-	  VM_API_STRING_VERIFY
+2017-05-22 10:36 +0000 [ba2c8f1458]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-26739 #close
+	* res_rtp_asterisk: Only learn a new source in learn state.
 
-	  Change-Id: I30dc6b36893aadad6012be3f16f93aa5720870d6
-	  Note: status: builds. Not tested any further.
+	  This change moves the logic which learns a new source address
+	  for RTP so it only occurs in the learning state. The learning
+	  state is entered on initial allocation of RTP or if we are
+	  told that the remote address for the media has changed. While
+	  in the learning state if we continue to receive media from
+	  the original source we restart the learning process. It is
+	  only once we receive a sufficient number of RTP packets from
+	  the new source that we will switch to it. Once this is done
+	  the closed state is entered where all packets that do not
+	  originate from the expected source are dropped.
 
-2017-01-05 13:21 +0000 [e3dcb9ddd9]  Richard Mudgett <rmudgett@digium.com>
+	  The learning process has also been improved to take into
+	  account the time between received packets so a flood of them
+	  while in the learning state does not cause media to be switched.
 
-	* res_pjsip_pubsub.c: Implement "pjsip show subscriptions" commands.
+	  Finally RTCP now drops packets which are not for the learned
+	  SSRC if strict RTP is enabled.
 
-	  ASTERISK-23828 #close
+	  ASTERISK-27013
 
-	  Change-Id: Ifb8a3b61f447aedc58a8e6b36a810f7566018567
+	  Change-Id: I56a96e993700906355e79bc880ad9d4ad3ab129c
 
-2017-01-23 16:18 +0000 [75497c33ea]  Mark Michelson <mmichelson@digium.com>
+2017-08-30 07:28 +0000 [663fe3e31f]  Joshua Colp <jcolp@digium.com>
 
-	* Free endpoint ACLs when destroying PJSIP endpoints.
+	* res_rtp_asterisk: Allow remote SSRC to change on an RTP instance.
 
-	  If endpoint ACLs were specified, they were not being freed
-	  when endpoints were destroyed. On systems with realtime endpoints, this
-	  could add up quickly since each DB lookup would allocate the ACL without
-	  freeing it.
+	  When SDP renegotiation occurs it is possible for an RTP
+	  instance to be reused for a new stream, resulting in the remote
+	  SSRC changing if it is part of a bundle group. This change
+	  allows this and updates its mapping in the current bundle
+	  group.
 
-	  ASTERISK-26731 #close
-	  Reported by Ustinov Artem
+	  ASTERISK-27231
 
-	  Change-Id: Ie1f8bf5b7a0de628c975beba01e69c56893331ad
+	  Change-Id: I6e3703974f236bc024c5dbe9bd43adae0c6fb490
 
-2017-01-23 09:10 +0000 [177e81ee47]  gtjoseph <gjoseph@digium.com>
+2017-08-24 11:45 +0000 [dab0389e24]  Joshua Colp <jcolp@digium.com>
 
-	* pjproject_bundled: Fix setting max log level
+	* core: Reduce video update queueing.
 
-	  An earlier attempt to prevent pjsua from spitting out an extra 6795
-	  lines of debug output every time the testsuite called it was also
-	  turning off the ability for asterisk to output debug info when it
-	  needed to.  This patch reverts the earlier fix and instead adds
-	  a pjproject patch that sets the startup log level to 1 for pjsua
-	  pjsystest and the pjsua python binding.  This is an asterisk-only
-	  patch that does not affect pjproject functionality and will not be
-	  submitted upstream.
+	  A video update frame is used to indicate that a channel
+	  with video negotiated should provide a full frame so the
+	  decoder decoding the stream is able to do so. In situations
+	  where a queue is used to store frames it makes no sense
+	  for the queue to contain multiple video update frames. One
+	  is sufficient to have a full frame be sent.
 
-	  Change-Id: I347a8b58b2626f2906ccfc1d339e907627a0c9e8
+	  ASTERISK-27222
 
-2017-01-23 10:08 +0000 [6d23b2e360]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Id3f40a6f51b740ae4704003a1800185c0c658ee7
 
-	* res_pjsip_endpoint_identifier_ip: Read settings before resolving.
+2017-08-14 12:20 +0000 [0a0ef8a1b1]  Richard Mudgett <rmudgett@digium.com>
 
-	  An option has been added, srv_lookups, which controls whether
-	  SRV lookups are performed on the provided match hosts or not.
-	  It was possible for this option to be applied after resolution
-	  had already happened.
+	* chan_pjsip.c: Fix topology refresh response code accuracy.
 
-	  This change makes it so hosts are stored away, settings are read
-	  and applied, and then resolution is done. This ensures that no
-	  matter the ordering the srv_lookups option is in effect.
+	  There are other 1xx and 2xx codes than 100 and 200 respectively.
 
-	  ASTERISK-26735
+	  Change-Id: I680db0997343256add1478714f5bf5b5569aee17
 
-	  Change-Id: I750378cb277be0140f8c5539450270afbfc43388
+2017-08-18 17:37 +0000 [00b10fa1e1]  Richard Mudgett <rmudgett@digium.com>
 
-2017-01-22 17:25 +0000 [a969bf3577]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_session.c: Fix crash when declining an active stream.
 
-	* LISTFILTER: Remove outdated ERROR message.
+	  If a previously active stream is declined we could crash because the
+	  channel's thread is still using the stream while we are updating the
+	  topology in the serializer thread.
 
-	  Feeding LISTFILTER an empty variable results in an invalid ERROR message.
-	  Earlier changes made the message useless because we can no longer tell if
-	  the variable is empty or does not exist.  It is valid to try to remove a
-	  value from an empty list just as it is valid to try to remove a value that
-	  is not in a non-empty list.
+	  * Defer removing any declined stream's handler until we have blocked the
+	  channel's thread with the channel lock.
 
-	  * Removed the outdated ERROR message.
+	  ASTERISK-27212
 
-	  * Added more test cases to the LISTFILTER unit test.
+	  Change-Id: I50e1d3ef26f8e41948f4c411ee329aa3b960a420
 
-	  Change-Id: Ided9040e6359c44a335ef54e02ef5950a1863134
+2017-08-16 17:50 +0000 [6acc945533]  Richard Mudgett <rmudgett@digium.com>
 
-2017-01-05 15:11 +0000 [3890337e7a]  Richard Mudgett <rmudgett@digium.com>
+	* bridge_channel.c: Fix FRACK when mapping frames to the bridge.
 
-	* res_pjsip_pubsub.c: Fix AMI event list counts.
+	  * Add protection checks when mapping streams to the bridge.  The channel
+	  and bridge may be in the process of updating the stream mapping when a
+	  media frame comes in so we may not be able to map the frame at the time.
 
-	  Fix the AMI PJSIPShowSubscriptionsInbound, PJSIPShowSubscriptionsOutbound,
-	  and PJSIPShowResourceLists actions event counts.  The reported counts may
-	  not necessarily be accurate depending on what happens.
+	  * We need to map the streams to the bridge's stream numbers right before
+	  they are written into the bridge.  That way we don't have to keep
+	  locking/unlocking the bridge and we won't have any synchronization
+	  problems before the frames actually go into the bridge.
 
-	  The subscriptions count would be wrong if Asterisk ever has outbound
-	  subscriptions.
+	  * Protect the deferred queue with the bridge_channel lock.
 
-	  The resource list count could be wrong if a list were added or removed
-	  during the AMI action being processed.
+	  ASTERISK-27212
 
-	  Change-Id: I4344301827523fa174960a42c413fd19abe4aed5
+	  Change-Id: Id6860dd61b594b90c8395f6e2c0150219094c21a
 
-2017-01-05 13:02 +0000 [fe4801c4f9]  Richard Mudgett <rmudgett@digium.com>
+2017-08-11 16:31 +0000 [efbf0aa8df]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_pubsub.c: Fix incorrect message string wrapping.
+	* channel: Fix topology API locking.
 
-	  Change-Id: Id771e6fe56d89ce365ddcbb423f820af97211120
+	  * ast_channel_request_stream_topology_change() must not be called with any
+	  channel locks held.
 
-2017-01-05 13:01 +0000 [46484b8730]  Richard Mudgett <rmudgett@digium.com>
+	  * ast_channel_stream_topology_changed() must be called with only the
+	  passed channel lock held.
 
-	* res_pjsip_pubsub.c: Eliminate trivial SCOPED_LOCK usage.
+	  ASTERISK-27212
 
-	  Change-Id: Ie0b69a830385452042fa19e7d267c6790ec6b6be
+	  Change-Id: I843de7956d9f1cc7cc02025aea3463d8fe19c691
 
-2017-01-05 12:58 +0000 [8160474d7d]  Richard Mudgett <rmudgett@digium.com>
+2017-08-16 15:22 +0000 [6bad253669]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip: alloca can never fail.
+	* bridge: Fix softmix bridge deadlock.
 
-	  Change-Id: Ia2a6158e5fdf311bc2a1c0c43417978de504b1f1
+	  * Fix deadlock in
+	  bridge_softmix.c:softmix_bridge_stream_topology_changed() between
+	  bridge_channel and channel locks.
 
-2017-01-13 11:03 +0000 [c628a7acac]  gtjoseph <gjoseph@digium.com>
+	  * The new bridge technology topology change callbacks must be called with
+	  the bridge locked.  The callback references the bridge channel list, the
+	  bridge technology could change, and the bridge stream mapping is updated.
 
-	* debug_utilities:  Create ast_loggrabber
+	  ASTERISK-27212
 
-	  ast_loggrabber gathers log files from customizable search patterns,
-	  optionally converts POSIX timestamps to a readable format and
-	  tarballs the results.
+	  Change-Id: Ide4360ab853607e738ad471721af3f561ddd83be
 
-	  Also a few tweaks were made to ast_coredumper.
+2017-08-17 17:07 +0000 [40faa22ce8]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I8bfe1468ada24c1344ce4abab7b002a59a659495
-	  (cherry picked from commit 5fa1c56d7e76999aa14f133a33f6b168e7c3b99c)
+	* app_confbridge: Document sfu video_mode value.
 
-2017-01-01 03:47 +0000 [e335b706ee]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I26e17df2c93f3933b23f78070603adbcc84ba204
 
-	* res_pjsip_outbound_authenticator_digest.c: Fix spacing in warning messages.
+2017-08-16 15:43 +0000 [e52f9b041a]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I573f0343c0c63a785cd4da60d57cc9f8b9ce7f49
+	* Fix downloader not working with curl
 
-2017-01-12 15:58 +0000 [883e7fde31]  Kevin Harwell <kharwell@digium.com>
+	  The codec/dpma downloader wasn't handling curl correctly.  The logic
+	  that transforms makeopts into a bash-sourceable file wasn't
+	  handling the make 'or' command in DOWNLOAD_TIMEOUT so bash was
+	  looking for an 'or' command.
 
-	* abstract/fixed/adpative jitter buffer: disallow frame re-inserts
+	  That logic has been eliminated.  Instead of trying to transform
+	  and source makeopts, the downloader now calls a make scriptlet
+	  to print the value of a specific variable.  This way, make handles
+	  the ors (or any other make construct that happens to creep into
+	  that file).
 
-	  It was possible for a frame to be re-inserted into a jitter buffer after it
-	  had been removed from it. A case when this happened was if a frame was read
-	  out of the jitterbuffer, passed to the translation core, and then multiple
-	  frames were returned from said translation core. Upon multiple frames being
-	  returned the first is passed on, but sebsequently "chained" frames are put
-	  back into the read queue. Thus it was possible for a frame to go back into
-	  the jitter buffer where this would cause problems.
+	  ASTERISK-27202
+	  Reported by: Sean McCord
 
-	  This patch adds a flag to frames that are inserted into the channel's read
-	  queue after translation. The abstract jitter buffer code then checks for this
-	  flag and ignores any frames marked as such.
+	  Change-Id: Iadfb6693528e4d4da7b8bb201fa66da2c71c7f99
 
-	  Change-Id: I276c44edc9dcff61e606242f71274265c7779587
+2017-08-15 13:12 +0000 [d7b04f22de]  Kevin Harwell <kharwell@digium.com>
 
-2017-01-13 21:23 +0000 [473330983b]  Richard Mudgett <rmudgett@digium.com>
+	* manager: hook event is not being raised
 
-	* taskprocessor.c: Change when high water warning logged.
+	  When the iostream code went in it introduced a conditional that made it so the
+	  hook event was not being raised even if a hook is present. This patch adds a
+	  check to see if a hook is present in astman_append. If so then call into the
+	  send_string function, which in turn raises the even for specified hook.
 
-	  The task processor queue reached X scheduled tasks message was originally
-	  intended to get logged only once per task processor to prevent spamming
-	  the log.  This is no longer necessary since high and low water thresholds
-	  can better control when the message is logged.
+	  Also updated the ami hooks unit test, so the test could be automated.
 
-	  It is beneficial to generate the warning each time a task processor
-	  reaches the high water level because PJSIP stops processing new requests
-	  while any high water alert is active.  Without this change you would have
-	  to enable at least debug level 3 logging to know about a repeated alert
-	  trigger.
+	  ASTERISK-27200 #close
 
-	  * Made generate the warning message whenever a task is pushed into the
-	  task processor that triggers the high water alert.
+	  Change-Id: Iff37f02f9708195d8f23e68f959d6eab720e1e36
 
-	  * Appended 'again' to the warning for a repeated high water alert trigger.
+2017-08-15 15:15 +0000 [44d316ef4a]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Iabf75a004f7edaf1e5e8c323099418e667cac999
+	* configure: Check cache for valid pjproject tarball before downloading.
 
-2017-01-10 05:54 +0000 [0047b1bc49]  Aaron An <anjb@ti-net.com.cn>
+	  On a fresh Asterisk source directory, the bundled pjproject tarball is
+	  unconditionally downloaded even if the tarball is already in a specified
+	  cache directory.
 
-	* res_rtp_asterisk:  Fix bug in function CHANNEL(rtcp, all_rtt)
+	  * Made check if the pjproject tarball is valid in the cache directory
+	  before downloading the tarball on a fresh source directory.
 
-	  Function CHANNEL(rtcp,all_rtt) CHANNEL(rtcp,all_loss) CHANNEL(rtcp,all_jitter)
-	  always return 0.0 due to wrong define of macro "AST_RTP_SATA_SET" and
-	  "AST_RTP_STAT_STRCPY".
-	  It should compare "combined" with "stat" not "current_stat".
+	  Change-Id: Ic7ec842d3c97ecd8dafbad6f056b7fdbce41cae5
 
-	  ASTERISK-26710 #close
-	  Reported-by: Aaron An
-	  Tested-by: AaronAn
+2017-08-09 15:24 +0000 [012391920c]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Id4140fafbf92e2db689dac5b17d9caa009028a15
+	* UPGRADE notes: Fixup for the 15 branch
 
-2017-01-10 18:10 +0000 [47474cfd54]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I4ca2f07ed62d77f1fdd10c3b216f6a28dd75720c
 
-	* debug_utilities:  Create the ast_coredumper utility
+2017-08-04 16:47 +0000 [4d3e66eabc]  Kevin Harwell <kharwell@digium.com>
 
-	  This utility allows easy manipulation of asterisk coredumps.
+	* res_pjsip_session/_sdp_rtp: Handling of 'msid' is incorrect
 
-	  * Configurable search paths and patterns for existing coredumps
-	  * Can generate a consistent coredump from the running instance
-	  * Can dump the lock_infos table from a coredump
-	  * Dumps backtraces to separate files...
-	    - thread apply 1 bt full -> <coredump>.thread1.txt
-	    - thread apply all bt -> <coredump>.brief.txt
-	    - thread apply all bt full -> <coredump>.full.txt
-	    - lock_infos table -> <coredump>.locks.txt
-	  * Can tarball corefiles and optionally delete them after processing
-	  * Can tarball results files and optionally delete them after processing
-	  * Converts ':' in coredump and results file names '-' to facilitate
-	    uploading.  Jira for instance, won't accept file names with colons
-	    in them.
+	  Currently, the handling of the msid attribute is not quite right. According to
+	  the spec the msid's between the offer/answer are not dependent upon one another.
+	  Meaning the same msid's given in an offer do not have to be returned in the
+	  answer for a given stream. And they probably shouldn't be (copied/reused) since
+	  this can potentially cause some browser side confusion.
 
-	  Tested on Fedora24+, Ubuntu14+, Debian6+, CentOS6+ and FreeBSD9+[1].
+	  This patch generates new msids when both an offer and answer are sent from
+	  Asterisk. However, Asterisk does reuse the original msid it sent out for a
+	  reinvite. Also audio+video streams are paired together by sharing the same
+	  stream id, but a different track id.
 
-	  [1] For *BSDs, the "devel/gdb" package might have to be installed to
-	  get a recent gdb.  The utility will check all instances of gdb
-	  it finds in $PATH and if one isn't found that can run python, it
-	  prints a friendly error.
+	  ASTERISK-27179 #close
 
-	  Change-Id: I935d37ab9db85ef923f32b05579897f0893d33cd
+	  Change-Id: Ifaec06dc7e65ad841633a24ebec8c8a9302d6643
 
-2017-01-08 10:29 +0000 [f8cd73ec3c]  gtjoseph <gjoseph@digium.com>
+2017-08-06 11:15 +0000 [71d0424ed5]  Joshua Colp <jcolp@digium.com>
 
-	* pjproject_bundled:  Fix compilation with MALLOC_DEBUG
+	* bridge: Fix stream topology/participant locking and video misrouting.
 
-	  When MALLOC_DEBUG was specified, make was failing.  Immediately
-	  remaking would work.  The issues was in the ordering of the make
-	  dependencies.
+	  This change fixes a few locking issues and some video misrouting.
 
-	  Change-Id: If6030b54fc693f3179f32bfd20c6b5d5f1b3f7cd
+	  1. When accessing the stream topology of a channel the channel lock
+	  must be held to guarantee the topology remains valid.
 
-2017-01-05 06:11 +0000 [37aaaa2da2]  Joshua Colp <jcolp@digium.com>
+	  2. When a channel was joined to a bridge the bridge specific
+	  implementation for stream mapping was not invoked, causing video
+	  to be misrouted for a brief period of time.
 
-	* res_pjsip_endpoint_identifier_ip: Add support for SRV lookups.
+	  ASTERISK-27182
 
-	  This change implements SRV support for the IP based endpoint
-	  identifier module. All possible addresses through SRV are looked
-	  up and added as matches. If no SRV records are available a
-	  fallback to normal host resolution is done. If an IP address
-	  is provided then no SRV lookup occurs.
+	  Change-Id: I5d2f779248b84d41c5bb3896bf22ba324b336b03
+	  (cherry picked from commit 0e352ec5100331c6a32008acc88d69d0fc58ccdd)
 
-	  This is configured using the "srv_lookups" option on the
-	  identify section and defaults to "yes".
+2017-08-08 13:33 +0000 [84600e2682]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-26693
+	* Make --with-pjproject-bundled the default for Asterisk 15
 
-	  Change-Id: I6b641e275bf96629320efa8b479737062aed82ac
+	  '--with-pjproject-bundled' is now the default when running
+	  ./configure. It can be disabled with '--without-pjproject-bundled'.
 
-2016-12-22 09:13 +0000 [569dac8e50]  Alexander Traud <pabstraud@compuserve.com>
+	  To make building without an internet connection easier, a new
+	  ./configure option '--with-download-cache' was added that sets
+	  the cache for externals (like pjproject, the codecs and the DPMA),
+	  AND the sounds files.  It can also be specified as an environment
+	  variable named "AST_DOWNLOAD_CACHE".  The existing
+	  '--with-sounds-cache' option / SOUNDS_CACHE_DIR env variable and
+	  '--with-externals-cache' option / EXTERNALS_CACHE_DIR env variable
+	  remain and if specified, will override '--with-downloads-cache'.
 
-	* res_pjsip_session: Access SIPDOMAIN via Dialplan.
+	  ASTERISK-27189
 
-	  This feature was available in the SIP channel driver chan_sip. For example,
-	  Asterisk is the outbound proxy and has to handle all SIP-URIs, even domains not
-	  local to Asterisk. In that case, SIPDOMAIN is used in the Dialplan, to detect
-	  and dial remote SIP-URIs. This change here sets the SIP destination domain of
-	  an inbound call (SIPDOMAIN) in the SIP channel driver res_pjsip as well.
+	  Change-Id: Ifa9783fddf44aafadb060c9feba713dfa81d38ce
 
-	  ASTERISK-26670 #close
+2017-08-05 14:43 +0000 [afd7875e82]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I27c880dc404a3c1c6792e1ba3545475339577243
+	* channel: Fix leak on successful call to chan->tech->requester.
 
-2017-01-04 05:50 +0000 [367128e70b]  Alexander Traud <pabstraud@compuserve.com>
+	  joint_cap needs to be released unconditionally as chan->tech->requester
+	  does not steal the reference even on success.
 
-	* chan_sip: Remember SDP negotiation on SIP_CODEC_INBOUND.
+	  ASTERISK-27180 #close
 
-	  After a SIP_CODEC_INBOUND in the dialplan, do not continue with cached formats
-	  but remember the joint format. Cached formats contain default parameters,
-	  often create an empty fmtp line. However, a joint format might have passed
-	  format_get_joint(.) in a res_format_attr_* module (like Opus Codec) and
-	  contain the resulting format parameters from a SDP negotiation.
+	  Change-Id: I647728992559bdb0a9c7357c20be1b36400d68b6
+	  (cherry picked from commit 3dbb1b9f48b0fa23cec2d8e3f94173004da320a4)
 
-	  ASTERISK-26691 #close
+2017-08-02 14:16 +0000 [53bba12340]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I35712d98a793d4c3efdd156cec57deab9014b1dc
+	* Fix compile error for old versions of GCC.
 
-2017-01-03 15:14 +0000 [d7e5a747c3]  gtjoseph <gjoseph@digium.com>
+	  Use -Wno-format-truncation only if supported by compiler.
 
-	* pjproject_bundled:  Compile pjsua with max log level = 2
+	  ASTERISK-27171 #close
 
-	  A while back, we changed config_site.h to set PJ_LOG_MAX_LEVEL = 6.
-	  This allowed us to control the log level better from inside Asterisk.
-	  An unfortunate side effect of this was that the pjsua binary and
-	  python bindings were also compiled with log level set to 6 so whenever
-	  a testsuite test that uses pjsua runs, it spits out 6795 lines of
-	  debug in an instant even before the test starts.  I believe this
-	  overruns the Jenkins capture buffer and prevents the test from
-	  properly terminating.  In turn, this results in the testsuite just
-	  hanging until the job is killed.  It's more frequent on the higher
-	  end agents because they can spit out the messages faster.
+	  Change-Id: Iac0aed7a5bcaa16c21b7d62c4e4678d244c4ccb6
+	  (cherry picked from commit cd79a15b2f9411c6e77f0f6594ff0c46f0ece080)
 
-	  Unfortunately, the messages are all spit out before we have control
-	  of the python pj.Lib instance where we can set logging levels so the
-	  only alternative was to actually compile pjsua and _pjsua.so with an
-	  overridden PJ_LOG_MAX_LEVEL.  Although defining a lower max level was
-	  done in the Makefile, the define in config_site.h had to be wrapped
-	  with "#ifndef" so the change would take effect.
+2017-08-02 09:43 +0000 [c042ad8343]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I2af9e7d48dde1927279c586c9c725d868fe6f3ff
+	* alembic/res_pjsip: Add "webrtc" configuration option
 
-2016-12-22 16:00 +0000 [34e728cfb9]  Joshua Colp <jcolp@digium.com>
+	  When the "webrtc" option was added in res_pjsip it was not added to the alembic
+	  scripts. This patch adds the option for alembic.
 
-	* chan_pjsip: Use session for retrieving CHANNEL() information.
+	  Also, changed the sorcery configuration type to an OPT_YESNO_T value instead of
+	  an OPT_BOOL_T so if this field is ever written to a database it will write out
+	  the correct value.
 
-	  The CHANNEL() dialplan function implementation for PJSIP allows
-	  querying of PJSIP specific information. This used the channel
-	  passed in to get the PJSIP session and associated information.
-	  It is possible for this channel to be masqueraded and end
-	  up as a different channel type by the time the information
-	  request is actually acted upon.
+	  ASTERISK-27119 #close
 
-	  This change retrieves the PJSIP session safely and accesses
-	  data from it (including channel). This provides a guarantee
-	  that the session and channel will not be altered when the
-	  request is being acted upon.
+	  Change-Id: I3e199f060aea25e193c439fc5cf96be4d3ed1c7b
+	  (cherry picked from commit b0c016cf6e0bcbe743f4f8286fb9b5ded830ccf7)
 
-	  ASTERISK-26673
+2017-08-02 11:44 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-	  Change-Id: I335e12b89e1820cafdd92b3e7526b8ba649eb7e6
+	* asterisk 15.0.0-beta1 Released.
 
-2016-12-31 19:56 +0000 [a398f98b08]  Joshua Elson <joshelson@gmail.com>
+2017-07-27 20:58 +0000 [aba08692df]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip: Fix known compact header issues
+	* Fix compiler warnings on Fedora 26 / GCC 7.
 
-	  ASTERISK-26684 #close
+	  GCC 7 has added capability to produce warnings, this fixes most of those
+	  warnings.  The specific warnings are disabled in a few places:
 
-	  Change-Id: Ifd7e401c45015119dd5e8421dbfe3afa6381744a
+	  * app_voicemail.c: truncation of paths more than 4096 chars in many places.
+	  * chan_mgcp.c: callid truncated to 80 chars.
+	  * cdr.c: two userfields are combined to cdr copy, fix would break ABI.
+	  * tcptls.c: ignore use of deprecated method SSLv3_client_method().
 
-2016-12-30 09:10 +0000 [0ab9d103f6]  JoshE (license 6075)
+	  ASTERISK-27156 #close
 
-	* res_pjsip_refer:  Handle compact Refer-To header.
+	  Change-Id: I65f280e7d3cfad279d16f41823a4d6fddcbc4c88
 
-	  refer_incoming_refer_request needed to look for the "r" header as well
-	  as the "Refer-To" header.
+2017-07-27 06:35 +0000 [64edb4ed21]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-26655 #close
-	  patches:
-	  	refer_compact_fix.diff	submitted by JoshE (license 6075)
+	* bundled_pjproject:  Improve SSL/TLS error handling
 
-	  Change-Id: I610410a99b02427ea5db887aeb454d5f12c2259f
+	  OpenSSL has 2 levels or error processing.  It's possible for the
+	  top layer to return SSL_ERROR_SYSCALL but the lower layer return
+	  no error, in which case processing should continue.  Only the top
+	  layer was being examined though so connections were being torn
+	  down when they didn't need to be.  This patch adds the examination
+	  of the lower level codes, and if they return no errors, allows
+	  processing to continue.
 
-2016-12-23 12:11 +0000 [21151408f7]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27001
+	  Reported-by: Ian Gilmore
+	  Patch-by: Ian Gilmore (pjproject-2.6.patch License 6889)
+	      Updated-by: George Joseph and Sauw Ming (Teluu)
 
-	* bridge_native_rtp.c: Minor code cleanups.
+	  Merged to upstream pjproject on 7/27/2017 (commit 5631)
 
-	  In native_rtp_bridge_compatible_check()
+	  Change-Id: I23844ca0c68ef1ee550f14d46f6dae57d33b7bd2
 
-	  * Made one variable declaration per line.
+2017-07-25 15:17 +0000 [d056f6b2fe]  Sean Bright <sean.bright@gmail.com>
 
-	  * Extracted if test assignment to make the test easier to see.
+	* res_rtp_asterisk: Fix mapping of pjsip's ICE roles to ours
 
-	  * Made long if tests easier to see the combinatorial logic.
+	  Change-Id: Ia578ede1a55b21014581793992a429441903278b
 
-	  * Added bridge id to a couple debug messages.
+2017-07-26 08:48 +0000 [11cd3be506]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I65bc5732aa7c9a2537f062f106fbea711cf2daad
+	* res_pjsip_pidf_eyebeam_body_supplement: Correct status presentation
 
-2016-12-23 12:10 +0000 [9dcf9e9cea]  Richard Mudgett <rmudgett@digium.com>
+	  This change fixes PIDF content generation when the underlying device
+	  state is considered in use. Previously it was incorrectly marked
+	  as closed meaning they were offline/unavailable. The code now
+	  correctly marks them as open.
 
-	* bridge_native_rtp.c: Fix native rtp bridge data race.
+	  Additionally:
 
-	  native_rtp_bridge_compatible() didn't lock the bridge channels before
-	  checking the channels for native bridging ability.  As a result, one of
-	  the channel's native format capabilities structure got replaced out from
-	  under the native bridge check.  Use of a stale pointer to freed memory
-	  causes bad things to happen.
+	    * Generate an XML element for our activity instead of a using a text
+	      node.
 
-	  MALLOC_DEBUG, DO_CRASH, and the
-	  tests/channels/pjsip/transfers/blind_transfer/caller_direct_media
-	  testsuite test caught this.
+	    * Consider every extension state other than "unavailable" to be 'open'
+	      status.
 
-	  * Add missing channel locking in native_rtp_bridge_compatible().
+	    * Update the XML namespaces and structure to reflect those
+	      documented in RFC 4480
 
-	  Change-Id: If25fdb3ac8e85563c4857fb8216b3d9dc3d0fa53
+	    * Use 'on-the-phone' (defined in RFC 4880) instead of 'busy' as the
+	      "in use" activity. This change results in eyeBeam using the
+	      appropriate icon for the watched user.
 
-2016-12-21 16:28 +0000 [a9e459f8ac]  Richard Mudgett <rmudgett@digium.com>
+	  This was tested on eyeBeam 1.5.20.2 build 59030 on Windows.
 
-	* res_rtp_asterisk.c: Fix uninitialized memory crash.
+	  ASTERISK-26659 #close
+	  Reported by: Abraham Liebsch
+	  patches:
+	    ASTERISK-26659.diff submitted by snuffy (license 5024)
 
-	  ast_rtp_remote_address_set() could pass an uninitialized 'us' parameter to
-	  ast_ouraddrfor().  If ast_ouraddrfor() returns an error then the 'us'
-	  parameter may not get initialized.  Thus when the code tries to save the
-	  'us' parameter to the local address we could try to copy a ridiculous
-	  sized memory buffer and segfault.
+	  Change-Id: I6e5ad450f91106029fb30517b8c0ea0c2058c810
 
-	  * Made pass an initialized 'us' parameter to ast_ouraddrfor().
+2017-07-26 09:27 +0000 [76270c0f78]  Sean Bright <sean.bright@gmail.com>
 
-	  * Optimized out the 'us' struct variable.
+	* app_queue: Add announce-position-only-up option
 
-	  ASTERISK-26672 #close
+	  Setting this option will cause the Queue application to only announce
+	  the caller's position if it has improved since the last time that we
+	  announced it.
 
-	  Change-Id: I4acea5dcdf0813da2c7d3e11c2d6067d160d17dc
+	  Change-Id: I173a124121422209485b043e2bf784f54242fce6
 
-2016-12-21 17:55 +0000 [bcdd282ada]  Richard Mudgett <rmudgett@digium.com>
+2017-06-26 07:52 +0000 [154e74eced]  Torrey Searle <torrey@voxbone.com>
 
-	* res_rtp_asterisk.c: Initialize ourip passed to ast_find_ourip().
+	* chan_pjsip: add a new function PJSIP_DTMF_MODE
 
-	  We access uninitialized memory when the 'ourip' parameter does not
-	  have an initial guess to our IP address.
+	  This function is a replica of SIPDtmfMode, allowing the DTMF mode of a
+	  PJSIP call to be modified on a per-call basis
 
-	  ASTERISK-26672
+	  ASTERISK-27085 #close
 
-	  Change-Id: I35507ea1ad7455d2be188f6ccdd4add7bd150e15
+	  Change-Id: I20eef5da3e5d1d3e58b304416bc79683f87e7612
 
-2016-12-21 16:25 +0000 [ac31233dbe]  Richard Mudgett <rmudgett@digium.com>
+2017-07-17 11:01 +0000 [451d86d62e]  Joshua Colp <jcolp@digium.com>
 
-	* acl.c: Improve ast_ouraddrfor() diagnostic messages.
+	* bridge_softmix / res_rtp_asterisk: Fix packet loss and renegotiation issues.
 
-	  * Made not generate strings unless they will actually be used.
+	  This change does a few things to improve packet loss and renegotiation:
 
-	  ASTERISK-26672
+	  1. On outgoing RTP streams we will now properly reflect out of order
+	  packets and packet loss in the sequence number. This allows the
+	  remote jitterbuffer to better reorder things.
 
-	  Change-Id: I155fbe7fdff5ce47dfe5326f3baf5446849702c3
+	  2. Video updates can now be discarded for a period of time
+	  after one has been sent to prevent flooding of clients.
 
-2016-12-21 17:54 +0000 [0aa5db4b38]  Richard Mudgett <rmudgett@digium.com>
+	  3. For declined and removed streams we will now release any
+	  media session resources associated with them. This was not
+	  previously done and caused an issue where old state was being
+	  used for a new stream.
 
-	* chan_rtp.c: Fix uninitialized memory crash.
+	  4. RTP bundling was not actually removing bundled RTP instances
+	  from the parent. This has been resolved by removing based on
+	  the RTP instance itself and not the SSRC.
 
-	  unicast_rtp_request() could pass an uninitialized 'us' parameter to
-	  ast_ouraddrfor().  If ast_ouraddrfor() returns an error then the 'us'
-	  parameter may not get initialized.  Thus when the code tries to save the
-	  'us' parameter to the local address we could try to copy a ridiculous
-	  sized memory buffer and segfault.
+	  5. The code did not properly handle explicitly unbundling an
+	  RTP instance from its parent. This now works as expected.
 
-	  * Made pass an initialized 'us' parameter to ast_ouraddrfor() and abort
-	  the UnicastRTP channel request if it fails.
+	  ASTERISK-27143
 
-	  ASTERISK-26672
+	  Change-Id: Ibd91362f0e4990b6129638e712bc8adf0899fd45
 
-	  Change-Id: I1ef7a7c09f4da4f15dcb6de660d2bcac5f2a95c0
+2017-07-20 08:08 +0000 [2128dc7c87]  Sergej Kasumovic <sergej@bicomsystems.com>
 
-2016-12-07 15:23 +0000 [e2fa3c7eda]  Richard Mudgett <rmudgett@digium.com>
+	* res_stasis_device_state: Unsubscribe should remove old subscriptions
 
-	* res_rtp_asterisk.c: Fix off nominal memory leak.
+	  Case scenario with Applications ARI:
 
-	  Change-Id: I95b1088d11244a2edae6607c12fbf33b38658a75
+	  * Once you subscribe to deviceState with Applications REST API, it will be
+	  added into subscription pool.
 
-2016-12-09 12:23 +0000 [d13be4eff6]  Martin Tomec <tomec@ipex.cz>
+	  * When you unsubscribe it will remove from the device_state_subscription
+	  hash table but not from the subscription pool.
 
-	* app_queue: Ensure member is removed from pending when hanging up.
+	  * When you subscribe again, it will add it to pool again.
 
-	  In some cases member is added to pending_members, and the channel
-	  is hung up before any extension state change. So the member would
-	  stay in pending_members forever. So when we call do_hang, we
-	  should also remove member from pending.
+	  * Now you will have two subscriptions and you will receive same event
+	  twice.
 
-	  ASTERISK-26621 #close
+	  This fix should now remove deviceState subscription from pool and it
+	  should fix unsubscribe on deviceState.
 
-	  Change-Id: Iae476b5c06481db18ebe0fa594b3e80fdc9a7d54
+	  ASTERISK-27130 #close
 
-2016-12-18 15:23 +0000 [815f755155]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I718b70d770a086e39b4ddba4f69a3c616d4476c4
 
-	* pjproject_bundled:  Make build single threaded
+2017-07-24 13:30 +0000 [927fc6bbd9]  Joshua Colp <jcolp@digium.com>
 
-	  There were just too many issues in various environments with
-	  multi threaded building of pjproject.  It doesn't really speed
-	  things up anyway since asterisk is already being compiled in
-	  parallel.
+	* core: Add VP9 passthrough support.
 
-	  Change-Id: Ie5648fb91bb89b4224b6bf43a0daa1af793c4ce1
+	  This change adds VP9 as a known codec and creates a cached
+	  "vp9" media format for use.
 
-2016-12-08 20:00 +0000 [493849dcd7]  Corey Farrell <git@cfware.com>
+	  Change-Id: I025a93ed05cf96153d66f36db1839109cc24c5cc
 
-	* chan_sip: Reorder unload_module to deal with stuck TCP threads.
+2017-07-21 15:57 +0000 [9aa4942a49]  Matthew Fredrickson <creslin@digium.com>
 
-	  In some situations TCP threads may become frozen.  This creates the
-	  possibility that Asterisk could segfault if they become unfrozen after
-	  chan_sip has been dlclose'd.  This reorders the unload_module process to
-	  allow abort if threads do not exit within 5 seconds.
+	* format.h: Fix a few minor errors in comments.
 
-	  High level order as follows:
-	  1) Unregister from the core to stop new requests.
-	  2) Signal threads to stop
-	  3) Clear config based tables (but do not free the table itself).
-	  4) Verify that threads have shutdown, cancel unload if not.
-	  5) Clean all remaining resources.
+	  A few minor problems were found in comments in format.h.  This patch fixes them.
 
-	  ASTERISK-26586
+	  Change-Id: I07f0bdb47b93359b361c4c3d8ecc87cd3199dd94
 
-	  Change-Id: Ie23692041d838fbd35ece61868f4c640960ff882
+2017-07-23 18:34 +0000 [0219d25e4e]  Joshua Colp <jcolp@digium.com>
 
-2016-12-16 01:32 +0000 [ab447f8a6a]  David M. Lee <dlee@respoke.io>
+	* res_pjsip: Add support for dnsmgr to external_media_address.
 
-	* configure: fix with-pjproject-bundled
+	  The "external_media_address" option on transports is now
+	  resolved using dnsmgr. This allows it to be automatically
+	  refreshed regularly if refreshes are enabled in dnsmgr.
+	  If the system is using a dynamic IP address a dynamic DNS
+	  hostname can be provided to keep the IP address up to
+	  date.
 
-	  The AC_ARG_WITH macro's shell variable is withval; not enableval. Purely
-	  coincidentally, the option would work when --enable-dev-mode is given.
+	  Change-Id: Ia54771720dff0105bde55d5bbb81a3ba437e05b2
 
-	  Also fixed a portability problem with bootstrap.sh, since -printf is not
-	  a portable option for find.
+2017-07-19 18:11 +0000 [85c631294a]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I0f0e5b1a934b5af5737713834361e9c95b96b376
+	* app_voicemail.c: Allow mailbox entry on authentication retry prompt.
 
-2016-12-15 13:25 +0000 [35736d419a]  Richard Mudgett <rmudgett@digium.com>
+	  The following testsuite voicemail tests were failing to re-enter the
+	  mailbox after the first login attempt.
 
-	* autosupport: Add 'pjproject show buildopts'
+	  tests/apps/voicemail/authenticate_invalid_mailbox
+	  tests/apps/voicemail/authenticate_invalid_password
 
-	  Change-Id: I8aa55a7c3fb175235ddc7f85e9457d5102d06fa7
+	  The tests were noting the start of the vm-incorrect-mailbox prompt and
+	  immediately sending the mailbox for the next login attempt.  Since the
+	  invalid message playback had to complete before the digits were
+	  recognized, the test passed for the wrong reason and added approximately
+	  20 seconds to the test times.
 
-2016-12-14 14:21 +0000 [4b285d226d]  Richard Mudgett <rmudgett@digium.com>
+	  * Allow the vm-incorrect-mailbox prompt to get interrupted by the mailbox
+	  digits like the initial vm-login prompt so the tests are able to enter the
+	  intended mailbox.
 
-	* chan_dahdi.c: Fix bounds check regression.
+	  Change-Id: I1dc53fe917bfe03a4587b2c4cd24c94696a69df8
 
-	  Caused by ASTERISK-25494
+2017-07-21 14:20 +0000 [e0ad75ec2a]  Rusty Newton <rnewton@digium.com>
 
-	  Change-Id: I1fc408c1a083745ff59da5c4113041bbfce54bcb
+	* Sounds: Update Makefile for Extra sounds 1.5.1 release
 
-2016-12-13 14:34 +0000 [9114574188]  Richard Mudgett <rmudgett@digium.com>
+	  Incrementing version for the Extra sounds release. 1.5.1 Extra sounds
+	  removes two prompts that were moved into the Core packages in the 1.6 Core
+	  sounds release.
 
-	* res_pjsip: Add/update ERROR msg if invalid URI.
+	  ASTERISK-27142 #close
 
-	  ASTERISK-24499
+	  Change-Id: I82f017812b0ea9599e19dd4635afd55611f13ee7
 
-	  Change-Id: Ie305153e47e922233b2ff24715e0e326e5fa3a6c
+2017-07-14 13:47 +0000 [715d79b60d]  Rusty Newton <rnewton@digium.com>
 
-2016-12-12 18:38 +0000 [75a6afbec5]  Richard Mudgett <rmudgett@digium.com>
+	* say.c: Fix file locations for second, seconds, minute, minutes files
 
-	* MESSAGE: Flush Message/ast_msg_queue channel alert pipe.
+	  The seconds and minutes files have always existed in the base language
+	  directory of the Core package. So say.c has always been calling the wrong
+	  location (under digits/) for those two files and in the case of second and
+	  minute they didn't exist in the Core packages at all.
 
-	  ASTERISK-25083
+	  The 1.6 sounds release moves the second and minute files into Core from
+	  Extra for the languages that already had them. A future release will include
+	  the second and minute files for languages that didn't already have them.
 
-	  Change-Id: Id54baa57a8dbca84e29f28bcd2ffc0a5ac12d8b2
+	  This patch just changes all the target locations for second, seconds,
+	  minute, and minutes that were under the digits subdir to be under the root of
+	  sounds instead. Which is where the sounds will be for some languages after 1.6
+	  sounds and for all languages after a future release.
 
-2016-12-13 14:06 +0000 [91485734a4]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-25810 #close
 
-	* res_sorcery_memory_cache:  Change an error to a debug message
+	  Change-Id: I05d9d4bee6a7237030530a46e7eb3df15f13f702
+	  Reported-by: Nicolas Riendeau
 
-	  When a sorcery user calls ast_sorcery_delete on an object that
-	  may have already expired from the cache, res_sorcery_memory_cache
-	  spits out an ERROR.  Since this can happen frequently and validly when
-	  an inbound registration expires after the cache entry expired, the
-	  errors are unnecessary and misleading.  Changed to a debug/1.
+2017-06-18 19:24 +0000 [eea9da2f42]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Idf3a67038c16e3da814cf612ff4d6d18ad29ecd7
+	* Core: Add support for systemd socket activation.
 
-2016-12-09 08:14 +0000 [cd46e86491]  gtjoseph <gjoseph@digium.com>
+	  This change adds support for socket activation of certain SOCK_STREAM
+	  listeners in Asterisk:
+	  * AMI / AMI over TLS
+	  * CLI
+	  * HTTP / HTTPS
 
-	* pjproject_bundled:  Retry download if previously saved tarball is bad
+	  Example systemd units are provided.  This support extends to any socket
+	  which is initialized using ast_tcptls_server_start, so any unknown
+	  modules using this function will support socket activation.
 
-	  If a tarball is corrupted during download, the makefile will attempt to
-	  download it again. If the tarball somehow gets corrupted after it's
-	  downloaded however, the makefile was just failing.  We now
-	  retry the download.
+	  Asterisk continues to function as normal if socket activation is not
+	  enabled or if systemd development headers are not available during
+	  build.
 
-	  ASTERISK-26653 #close
+	  ASTERISK-27063 #close
 
-	  Change-Id: I1b24d454852d80186f60c5a65dc4624ea8a1c359
+	  Change-Id: Id814ee6a892f4b80d018365c8ad8d89063474f4d
 
-2016-12-08 12:54 +0000 [22820e10fe]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+2017-07-21 11:24 +0000 [94de9d3eea]  George Joseph <gjoseph@digium.com>
 
-	* chan_sip: Delete unneeded check
+	* Restore the incorrectly deleted spandspflow2pcap.log
 
-	  P is always true. We check it before
+	  Change-Id: Iafe78cf0fb1e7064223d4dea279eeb776c8fa8e5
 
-	  Change-Id: Iee61cda002a9f61aee26b9f66c5f9b59e3389efb
+2017-07-21 07:56 +0000 [6239203628]  George Joseph <gjoseph@digium.com>
 
-2016-12-08 12:58 +0000 [6aa2c5e5f9]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	* Update make_ari_stubs to correct version
 
-	* Small code cleanup in chan_sip
+	  Change-Id: I18575b46db48d62edc72f37dc23b4ab22b43a8b1
 
-	  The conditional expressions of the 'if' operators situated
-	  alongside each other are identical.
+2017-07-20 09:57 +0000 [6650ae43e1]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I2cf7c317b106ec14440c7f1b5dcfbf03639f748a
+	* corosync: Fix corosync library name in configure.ac
 
-2016-12-08 12:43 +0000 [b596fac838]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	  Also add new corosync packages to install_prereq.
 
-	* Fix typo in chan_sip
+	  Reported by Travis Ryan in #asterisk-dev
 
-	  The conditional expressions of the 'if' operators
-	  situated alongside each other are identical.
+	  Change-Id: Ib861c95ba630fed62dc54e56784ad8446ed9d2db
 
-	  Change-Id: I652b6dcddb3be007e669a6aa8107edb31a1ddafb
+2017-07-20 13:06 +0000 [b172474728]  George Joseph <gjoseph@digium.com>
 
-2016-12-08 12:30 +0000 [483ed9f1aa]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	* Update MAINLINE_BRANCH to 15
 
-	* res_pjsip: Fix 'A = B != C' kind.
+	  Change-Id: I425d542b600ceabeef2342e9adfeb68c484a043d
 
-	  Consider reviewing the expression of the 'A = B != C' kind.
-	  The expression is calculated as following: 'A = (B != C)'
+2017-07-20 10:52 +0000 [3e8d628c0e]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ibaa637dfda47d51a20e26069d3103e05ce80003d
+	* Update AMI and ARI versions for master/15 and update UPDATE.txt
 
-2016-11-30 09:31 +0000 [41c6319c4e]  Walter Doekes <walter+asterisk@wjd.nu>
+	  AMI goes from 3.2.0 to 4.0.0
+	  ARI goes from 2.0.0 to 3.0.0
 
-	* chan_sip: Do not allow non-SP/HTAB between header key and colon.
+	  Copied UPGRADE.txt -> UPGRADE-15.txt
+	  Created new UPGRADE.txt
+	  Removed a log file that was accidentally checked in a while ago
 
-	  RFC says SIP headers look like:
+	  Change-Id: I1c794f910038459b13e16f9c3a12c44e56f142f7
 
-	      HCOLON  =  *( SP / HTAB ) ":" SWS
-	      SWS     =  [LWS]                    ; sep whitespace
-	      LWS     =  [*WSP CRLF] 1*WSP        ; linear whitespace
-	      WSP     =  SP / HTAB                ; from rfc2234
+2017-07-18 15:04 +0000 [e7d9e42616]  Benjamin Keith Ford <bford@digium.com>
 
-	  chan_sip implemented this:
+	* pjsip: Increase maximum packet size.
 
-	      HCOLON  =  *( LOWCTL / SP ) ":" SWS
-	      LOWCTL  = %x00-1F                   ; CTL without DEL
+	  The maximum packet size for PJSIP has been increased to handle the
+	  multiple streams being added for WebRTC.
 
-	  This discrepancy meant that SIP proxies in front of Asterisk with
-	  chan_sip could pass on unknown headers with \x00-\x1F in them, which
-	  would be treated by Asterisk as a different (known) header.  For
-	  example, the "To\x01:" header would gladly be forwarded by some proxies
-	  as irrelevant, but chan_sip would treat it as the relevant "To:" header.
+	  Change-Id: I9ea1e8d02668c544acadcb1c6200e1cc1bd588b3
 
-	  Those relying on a SIP proxy to scrub certain headers could mistakenly
-	  get unexpected and unvalidated data fed to Asterisk.
+2017-07-17 07:19 +0000 [bcd3f65174]  Joshua Colp <jcolp@digium.com>
 
-	  This change fixes so chan_sip only considers SP/HTAB as valid tokens
-	  before the colon, making it agree on the headers with other speakers of
-	  SIP.
+	* bridge_softmix: Don't reorder streams on participant leaving.
 
-	  ASTERISK-26433 #close
-	  AST-2016-009
+	  When a participant leaves a bridge while operating in SFU mode
+	  their respective stream on every other participant needs to be
+	  removed. Leaving the stream out of the new topology results in
+	  every stream after it being moved and reordered. This causes
+	  problems with clients. Instead simply mark the stream as removed
+	  which leaves it in place in the SDP and doesn't reorder or touch
+	  any other streams.
 
-	  Change-Id: I78086fbc524ac733b8f7f78cb423c91075fd489b
+	  ASTERISK-27136
 
-2016-11-14 18:18 +0000 [888142e891]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I4b3f840adcdf69b83842b0d8a737665ba0ef9cb1
 
-	* res_format_attr_opus: Fix crash when fmtp contains spaces.
+2017-07-16 12:31 +0000 [f48695ce5b]  Joshua Colp <jcolp@digium.com>
 
-	  When an opus offer or answer was received that contained an
-	  fmtp line with spaces between the attributes the module would
-	  fail to properly parse it and crash due to recursion.
+	* bridge_softmix: Use removed stream spots when renegotiating.
 
-	  This change makes the module handle the space properly and
-	  also removes the recursion requirement.
+	  Streams are never truly removed in SDP, they still occupy
+	  a location within the SDP. This location can be reused by
+	  another stream if it so chooses.
 
-	  ASTERISK-26579
+	  This change takes advantage of this such that if a new stream
+	  is needing to be added for a new participant any removed streams
+	  are instead replaced first. This reduces the size of the SDP
+	  and the number of streams.
 
-	  Change-Id: I01f53e5d9fa9f1925a7365f8d25071b5b3ac2dc3
+	  ASTERISK-27134
 
-2016-12-06 14:54 +0000 [ebc67d3053]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I95cdcfd55cf47e02ea52abb5d94008db3fb68b1d
 
-	* res_pjsip_registrar: AMI Add RegistrationInboundContactStatuses command
+2017-07-16 12:18 +0000 [942ee54b53]  Joshua Colp <jcolp@digium.com>
 
-	  The PJSIPShowRegistrationsInbound AMI command was just dumping out
-	  all AORs which was pretty useless and resource heavy since it had
-	  to get all endpoints, then all aors for each endpoint, then all
-	  contacts for each aor.
+	* res_rtp_asterisk: Use RTP component for ICE if RTCP-MUX is in use.
 
-	  PJSIPShowRegistrationInboundContactStatuses sends ContactStatusDetail
-	  events which meets the intended purpose of the other command and has
-	  significantly less overhead.  Also, some additional fields that were
-	  added to Contact since the original creation of the ContactStatusDetail
-	  event have been added to the end of the event.
+	  This change makes it so that if an RTCP packet is being sent
+	  the RTP ICE component is used for sending if RTCP-MUX is in use.
 
-	  For compatibility purposes, PJSIPShowRegistrationsInbound is left
-	  intact.
+	  ASTERISK-27133
 
-	  ASTERISK-26644 #close
+	  Change-Id: I6200f611ede709602ee9b89501720c29545ed68b
 
-	  Change-Id: I326f12c9ecb52bf37ba03f0748749de4da01490a
+2017-07-14 01:25 +0000 [26f149ab0a]  Sergej Kasumovic <sergej@bicomsystems.com>
 
-2016-12-06 16:45 +0000 [d506874477]  Richard Mudgett <rmudgett@digium.com>
+	* app_confbridge: Make sure name recordings are always removed from the filesystem
 
-	* Bundled pjproject:  Fix finding SIP transactions.
+	  This commit fixes two possible scenarios:
 
-	  Occasionally SIP message transactions are not found when they should be.
-	  In the particular case an incoming INVITE transaction is CANCELed but the
-	  INVITE transaction cannot be found so a 481 response is returned for the
-	  CANCEL.  The problematic calls have a '_' character in the Via branch
-	  parameter.
+	  * When recording name and if during recording you hangup, file is never
+	  removed. This is due to the fact file location is nulled.
+	  * When recording name and if you hangup during thank-you prompt, file
+	  is never removed.
 
-	  The problem is in the pjproject PJ_HASH_USE_OWN_TOLOWER feature's code.
-	  The problem with the "own tolower" code is that it does not calculate the
-	  same hash value as when the pj_tolower() function is used.  The "own
-	  tolower" code will erroneously modify the ASCII characters '@', '[', '\\',
-	  ']', '^', and '_'.  Calls to pj_hash_calc_tolower() can use the
-	  PJ_HASH_USE_OWN_TOLOWER substitute algorithm when enabled.  Calls to
-	  pj_hash_get_lower(), pj_hash_set_lower(), and pj_hash_set_np_lower() call
-	  find_entry() which never uses the PJ_HASH_USE_OWN_TOLOWER algorithm.  As a
-	  result you may not be able to find a hash tabled entry because the
-	  calculated hash values would differ.
+	  ASTERISK-27123 #close
 
-	  * Simply disable PJ_HASH_USE_OWN_TOLOWER.
+	  Change-Id: I39b7271408b4b54ce880c5111a886aa8f28c2625
 
-	  ASTERISK-26490 #close
+2017-07-14 01:11 +0000 [d3f5b265c7]  Sergej Kasumovic <sergej@bicomsystems.com>
 
-	  Change-Id: If89bfdb5f301b8b685881a9a2a6e0c3c5af32253
+	* chan_iax2: On reload make sure to check for existing MWI subscription
 
-2016-12-06 12:06 +0000 [4b233675d8]  gtjoseph <gjoseph@digium.com>
+	  On every reload of chan_iax2 module, MWI subscription was added, which
+	  results in additional taskprocessors being accumulated over time.
 
-	* pjproject_bundled:  Fix missing inclusion of symbols
+	  This commit fixes it by making sure we check for existing subscription
+	  first.
 
-	  Added back in a -g3, and an -O3 when DONT_OPTIMIZE is not set, to
-	  the CFLAGS.  Not sure how they went missing.
+	  This was verified with 'core show taskprocessors' CLI command.
 
-	  Also fixed an uninstall problem where we weren't removing the
-	  symlink from libasteriskpj.so.2 to libasteriskpj.so.  While I was
-	  there, I fixed it for libasteriskssl as well.
+	  ASTERISK-27122 #close
 
-	  Change-Id: I9e00873b1e9082d05b5549d974534b48a2142556
+	  Change-Id: Ie2ef528fd5ca01b933eeb88188cc10967899cfb9
 
-2016-12-02 12:04 +0000 [580f83dac7]  Richard Mudgett <rmudgett@digium.com>
+2017-07-10 18:17 +0000 [7da6ddda30]  Kevin Harwell <kharwell@digium.com>
 
-	* Remove files that got merged in error somehow to the 13 branch.
+	* res_pjsip: Add "webrtc" configuration option
 
-	  Change-Id: Id79e2226c31084f9252d5aede9050d3cf13322c8
+	  This patch creates a new configuration option called "webrtc". When enabled it
+	  defaults and enables the following options that are needed in order for webrtc
+	  to work in Asterisk:
 
-2016-11-30 18:25 +0000 [61ba2a014a]  Richard Mudgett <rmudgett@digium.com>
+	    rtcp-mux, use_avpf, ice_support, and use_received_transport=enabled
+	    media_encryption=dtls
+	    dtls_verify=fingerprint
+	    dtls_setup=actpass
 
-	* res_pjsip_outbound_registration.c: Filter redundant statsd reporting.
+	  When "webrtc" is enabled, this patch also parses the "msid" media level
+	  attribute from an SDP. It will also appropriately add it onto the outgoing
+	  session when applicable.
 
-	  Increasing the testsuite shutdown timeout before forcibly killing
-	  Asterisk allowed more events to be sent out.  Some tests failed as
-	  a result.  The tests/channels/pjsip/statsd/registrations failed
-	  because we now get the statsd events that a comment in the test
-	  configuration stated couldn't be intercepted.  Unfortunately, we
-	  get a variable number of events because of internal status state
-	  transition races generating redundant statsd events.
+	  Lastly, when "webrtc" is enabled h264 RTCP FIR feedback frames are now sent.
 
-	  We were reporting redundant statsd PJSIP.registrations.state changes
-	  for internal state changes that equated to the same thing publicly.
+	  ASTERISK-27119 #close
 
-	  * Made update_client_state_status() filter out redundant statsd
-	  updates.
+	  Change-Id: I5ec02e07c5d5b9ad86a34fdf31bf2f9da9aac6fd
 
-	  ASTERISK-26527
+2017-07-13 15:43 +0000 [3fbb4a0a08]  Rusty Newton <rnewton@digium.com>
 
-	  Change-Id: If851c7d514bb530d9226e4941ba97dcf52000646
+	* Sounds: Update for core sounds 1.6 release
 
-2016-11-22 11:20 +0000 [2ceb609edb]  Guido Falsi <mad@madpilot.net>
+	  Added necessary lines to make the en_NZ language set selectable and to get
+	  core sounds 1.6 pulled down.
 
-	* res_rtp: Fix regression when IPv6 is not available.
+	  ASTERISK-26807 #close
+	  ASTERISK-25816 #close
+	  ASTERISK-26274 #close
 
-	  The latest Release candidate fails to create RTP streams when IPv6
-	  is not available. Due to the changes made in September the ast_sockaddr
-	  structure passed around to create these streams is always of AF_INET6
-	  type, causing failure when used for IPv4. This patch adds a utility
-	  function to check for availability of IPv6 and applies such check
-	  at startup to determine how to create the ast_sockaddr structures.
+	  Change-Id: I84e4dd4696568cc1ba318d12ac4b075461d6eed4
 
-	  ASTERISK-26617 #close
+2017-07-10 14:04 +0000 [78a50b0343]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I627a4e91795e821111e1cda523f083a40d0e0c3e
+	* core: Add PARSE_TIMELEN support to ast_parse_arg and ACO.
 
-2016-11-28 19:43 +0000 [53459cdaa9]  Eduardo S. Libardi <eslibardi@gmail.com>
+	  This adds support for parsing timelen values from config files.  This
+	  includes support for all flags which apply to PARSE_INT32.  Support for
+	  this parser is added to ACO via the OPT_TIMELEN_T option type.
 
-	* res_calendar_caldav: Add support reading gmail calendar
+	  Fixes an issue where extra characters provided to ast_app_parse_timelen
+	  were ignored, they now cause an error.
 
-	  The response from gmail calendar includes the string name
-	  "caldav:calendar-data". res_calendar_caldav implements
-	  the example included in RFC 4791: string "C:calendar-data".
-	  When reading the calendar, res_calendar_caldav compare the
-	  string and if does not match just discards the event.
-	  This commit compares the response to both strings,
-	  successfully loading gmail calendar events.
-	  Writing to gmail calendar is working prior to this fix.
+	  Testing is included.
 
-	  ASTERISK-26624
-	  Reported by: Eduardo S. Libardi
+	  ASTERISK-27117 #close
 
-	  Change-Id: Ia1eef10552ae616efb645d390f5ffe81260d7d4a
+	  Change-Id: I6b333feca7e3f83b4ef5bf2636fc0fd613742554
 
-2016-11-23 18:27 +0000 [44fe4a5769]  Richard Mudgett <rmudgett@digium.com>
+2017-06-30 13:55 +0000 [065c3005ad]  Joshua Colp <jcolp@digium.com>
 
-	* PJPROJECT logging: Made easier to get available logging levels.
+	* res_rtp_asterisk / res_pjsip: Add support for BUNDLE.
 
-	  Use of the new logging is as simple as issuing the new CLI command or
-	  setting the new pjproject.conf option.
+	  BUNDLE is a specification used in WebRTC to allow multiple
+	  streams to use the same underlying transport. This reduces
+	  the number of ICE and DTLS negotiations that has to occur
+	  to 1 normally.
+
+	  This change implements this by adding support for it to
+	  the RTP SDP module in PJSIP. BUNDLE can be turned on using
+	  the "bundle" option and on an offer we will offer to
+	  bundle streams together. On an answer we will accept any
+	  bundle groups provided. Once accepted each stream is bundled
+	  to another RTP instance for transport.
+
+	  For the res_rtp_asterisk changes the ability to bundle
+	  an RTP instance to another based on the SSRC received
+	  from the remote side has been added. For outgoing traffic
+	  if an RTP instance is bundled to another we will use the
+	  other RTP instance for any transport related things. For
+	  incoming traffic received from the transport instance we
+	  look up the correct instance based on the SSRC and use it
+	  for any non-transport related data.
 
-	  Other options that can affect the logging are how you have the pjproject
-	  log levels mapped to Asterisk log types in pjproject.conf and if you have
-	  configured Asterisk to log the DEBUG type messages.  Altering the
-	  pjproject.conf level mapping shouldn't be necessary for most installations
-	  as the default mapping is sensible.  Configuring Asterisk to log the DEBUG
-	  message type is standard practice for collecting debug information.
+	  ASTERISK-27118
 
-	  * Added CLI "pjproject set log level" command to dynamically adjust the
-	  maximum pjproject log message level.
+	  Change-Id: I96c0920b9f9aca7382256484765a239017973c11
 
-	  * Added CLI "pjproject show log level" command to see the currently set
-	  maximum pjproject log message level.
+2017-07-11 09:55 +0000 [8b535a406b]  Torrey Searle <torrey@voxbone.com>
 
-	  * Added pjproject.conf startup section "log_level" option to set the
-	  initial maximum pjproject log message level so all messages could be
-	  captured from initialization.
+	* res/res_stasis_snoop: generate silence when audiohook returns null
 
-	  * Set PJ_LOG_MAX_LEVEL to 6 to compile in all defined logging levels into
-	  bundled pjproject.  Pjproject will use the currently set run time log
-	  level to determine if a log message is generated just like Asterisk
-	  verbose and debug logging levels.
+	  Currently when rtp is paused, no packets are written to the
+	  recorded audio file, causing the silence to be skipped and recording
+	  not properly time aligned.  The read handler as been adapted to
+	  return a silence frame of the correct size.
 
-	  * In log_forwarder(), made always log enabled and mapped pjproject log
-	  messages.  DEBUG mapped log messages are no longer gated by the current
-	  Asterisk debug logging level.
+	  ASTERISK-27128 #close
 
-	  * Removed RAII_VAR() from res_pjproject.c:get_log_level().
+	  Change-Id: I2d7f60650457860b9c70907b14426756b058a844
 
-	  ASTERISK-26630 #close
+2017-06-22 07:47 +0000 [d42a9cc9dc]  Torrey Searle <torrey@voxbone.com>
 
-	  Change-Id: I6dca12979f482ffb0450aaf58db0fe0f6d2e5389
+	* res/res_pjsip_t38  ensure t38 requests get rejected quickly
 
-2016-11-30 10:48 +0000 [17b0b91afa]  Mark Michelson <mmichelson@digium.com>
+	  arm the t38 webhook always, so we can correctly reject a
+	  T38 negotiation request when t38 is disabled on a channel
 
-	* Frame deferral: Re-queue deferred frames one-at-a-time.
+	  Change-Id: Ib1ffe35aee145d4e0fe61dd012580be11aae079d
 
-	  The recent change that made frame deferral into an API had a behavior
-	  change to it. When frame deferral was completed, we would take all of
-	  the deferred frames and queue them all onto the channel in one call to
-	  ast_queue_frame_head(). Before frame deferral was API-ized, places that
-	  performed manual frame deferral would actually take each deferred frame
-	  and queue them onto the channel.
+2017-07-12 13:24 +0000 [6b138046e7]  Corey Farrell <git@cfware.com>
 
-	  This change in behavior caused the confbridge_recording test to start
-	  failing consistently. Without going too crazily deep into the details,
-	  a channel was getting "stuck" in an ast_safe_sleep(). An AMI redirect
-	  was attempting to break it out of the sleep, but because there were more
-	  frames in the channel read queue than expected, the channel ended up
-	  being unable to break from its sleep loop.
+	* core: Add digit filtering to ast_waitfordigit_full
 
-	  By restoring the behavior of individual frame queuing after deferral,
-	  the test starts passing again.
+	  This adds a parameter to ast_waitfordigit_full which can be used to only
+	  stop waiting when certain expected digits are received.  Any unexpected
+	  DTMF digits are simply ignored.
 
-	  Note, this points to a potential underlying issue pointing to an
-	  "unbalance" that can occur when queuing multiple frames at once,
-	  and so a follow-up issue is being created to investigate that
-	  possibility.
+	  This also creates a new dialplan application WaitDigit.
 
-	  Change-Id: Ied5dacacda06d343dea751ed5814a03364fe5a7d
+	  ASTERISK-27129 #close
 
-2016-06-28 16:26 +0000 [b0c9f07f04]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Change-Id: Id233935ea3d13e71c75a0861834c5936c3700ef9
 
-	* OpenSSL 1.1.0 support
+2017-07-11 04:48 +0000 [b54eb167b4]  Holger Hans Peter Freyther <holger@moiji-mobile.com>
 
-	  OpenSSL 1.1.0 includes some major changes in the interface. See
-	  https://wiki.openssl.org/index.php/1.1_API_Changes .
+	* app_playback.c: Use the timezonename parameter
 
-	  Status: Right now there are still a few deprecation notes with OpenSSL
-	  1.1.0. But it's a start.
+	  In say_date_generic the timezonename parameter is passed but never
+	  used. Fix it by passing it to the ast_localtime function.
 
-	  Changes:
-	  * CRYPTO_LOCK is no longer available. Replace it with its value for now.
-	    I don't completely understand what it is used for there.
-	  * Remove several functions from libasteriskssl that seem to no longer be
-	    needed.
-	  * Structures have become opaque and are accesses with accessors.
-	  * ERR_remove_thread_state() no longer needed.
-	  * SSLv2 code now could no longer be used in 1.1.
+	  ASTERISK-27124
 
-	  ASTERISK-26109 #close
+	  Change-Id: I63106b8db10426d417d7275f22554a616e92fae4
 
-	  Change-Id: I5e29d477d486ca29b6aae0dc2f5dff960c1cb82b
+2017-07-12 15:07 +0000 [e83b9d141a]  Sean Bright <sean.bright@gmail.com>
 
-2016-11-28 15:12 +0000 [a33ed3327a]  Matt Jordan <mjordan@digium.com>
+	* basic-pbx: Remove res_pjsip_multihomed from sample config
 
-	* res/res_pjsip: Fix documentation whitespace issues
+	  ASTERISK-27127 #close
+	  Reported by: HZMI8gkCvPpom0tM
 
-	  Tabs > Spaces.
+	  Change-Id: I2b0c54570d58156e37166ac536728af3b6c01789
 
-	  Change-Id: If1e43a71822615a898e958e0f8b2e882606f0bd0
+2017-07-11 14:33 +0000 [7f09fd2c2f]  Joshua Colp <jcolp@digium.com>
 
-2016-11-22 10:27 +0000 [09c36a6535]  Matt Jordan <mjordan@digium.com>
+	* bridge/core_unreal: Fix SFU bugs with forwarding frames.
 
-	* res_pjsip/chan_sip: Advertise 'ws' in the SIP URI transport parameter
+	  This change fixes a few things uncovered during SFU testing.
 
-	  Per RFC 7118 5.2, the SIP URI 'transport' parameter should advertise
-	  'ws' when WebSockets are to be used as the transport. This applies to
-	  both secure and insecure WebSockets.
+	  1. Unreal channels incorrectly forwarded video frames when
+	  no video stream was present on them. This caused a crash when
+	  they were read as the core requires a stream to exist for the
+	  underlying media type. The Unreal channel will now ensure a
+	  stream exists for the media type before forwarding the frame
+	  and if no stream exists then the frame is dropped.
 
-	  There were two bugs in Asterisk with respect to this:
+	  2. Mapping of frames during bridging from the stream number of
+	  the underlying channel to the stream number of the bridge was
+	  done in the wrong location. This resulted in the frame getting
+	  dropped. This mapping now occurs on reading of the frame from
+	  the channel.
 
-	  (1) The most egregious occurs in res_pjsip. There, we advertise 'ws' for
-	      insecure websockets and 'wss' for secure websockets. While this
-	      would seem to make sense - since 'WS' and 'WSS' are used for the Via
-	      Transport parameter - this is not the case for the SIP URI. This
-	      patch corrects that by registering the secure websockets with
-	      pjproject using the shorthand 'WS', and by returning 'ws' when asked
-	      for the transport parameter. Note that in pjproject, it is perfectly
-	      valid to have multiple transports use the same shorthand.
+	  3. Bridging was using the wrong ast_read function resulting in
+	  it living in a non-multistream world.
 
-	  (2) In chan_sip, we return an upper-case version of the transport 'WS'
-	      instead of 'ws'. Since we should be strict in what we send and
-	      liberal in what we accept (within reason), this patch lower-cases
-	      the transport before appending it to the parameter.
+	  4. In bridge_softmix when adding new streams to existing channels
+	  the wrong stream topology was copied resulting in no streams
+	  being added.
 
-	  ASTERISK-24330 #close
-	  Reported by: cervajs, Inaki Baz Castillo
+	  Change-Id: Ib7445722c3219951d6740802a0feddf2908c18c8
 
-	  Change-Id: Iff77b645f8cc3b7cd35168a6676c26b147f22f42
+2017-07-11 07:26 +0000 [b7a875778a]  George Joseph <gjoseph@digium.com>
 
-2016-11-28 11:03 +0000 [29e887e9e1]  gtjoseph <gjoseph@digium.com>
+	* res_musiconhold:  Add kill_escalation_delay, kill_method to class
 
-	* build_tools:  Fix download_externals to handle certified branches
+	  By default, when res_musiconhold reloads or unloads, it sends a HUP
+	  signal to custom applications (and all descendants), waits 100ms,
+	  then sends a TERM signal, waits 100ms, then finally sends a KILL
+	  signal.  An application which is interacting with an external
+	  device and/or spawns children of its own may not be able to exit
+	  cleanly in the default times, expecially if sent a KILL signal, or
+	  if it's children are getting signals directly from
+	  res_musiconhoild.
 
-	  download_externals wasn't handling the "certified/13.x" version
-	  correctly.
+	  * To allow extra time, the 'kill_escalation_delay'
+	    class option can be used to set the number of milliseconds
+	    res_musiconhold waits before escalating kill signals, with the
+	    default being the current 100ms.
 
-	  Change-Id: I124d195bb117ca36fd7bf1150c630f3b474a9d9a
+	  * To control to whom the signals are sent, the "kill_method" class
+	    option can be set to "process_group" (the default, existing
+	    behavior), which sends signals to the application and its
+	    descendants directly, or "process" which sends signals only to the
+	    application itself.
 
-2016-11-02 05:05 +0000 [bfb8c962c4]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Change-Id: Iff70a1a9405685a9021a68416830c0db5158603b
 
-	* autoconf: more variants for OSARCH linux-gnu
+2017-07-05 12:44 +0000 [5d86da61a6]  Benjamin Keith Ford <bford@digium.com>
 
-	  There are quite a few odd GNU/Linux platforms. Just call all of them
-	  linux-gnu.
+	* manager: Remove AMI "Queues" action.
 
-	  Specifically this fixes building the Debian platforms mips64el and x32.
-	  And maybe also others.
+	  When performing the "Queues" action via AMI, it outputs the same
+	  text that the Asterisk CLI outputs when running a "queue show"
+	  command, which does not conform with the AMI spec. "QueueStatus"
+	  already does what the "Queues" action should do, so instead of
+	  correcting the output, the "Queues" action will be removed and
+	  "QueueStatus" should be used instead.
 
-	  ASTERISK-26546 #close
+	  ASTERISK-27073 #close
+	  Reported by: Brian
 
-	  Change-Id: I06ec4bd7f0ee1c84b6b24d81538223b07c4174b1
+	  Change-Id: Id11743859758255b69cc3a557750d7a56c6d16f8
 
-2016-11-17 08:25 +0000 [a1fa909033]  Timo Teräs <timo.teras@iki.fi>
+2017-07-03 07:30 +0000 [d58ef31acd]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* codec_dahdi: Fix poll.h include.
+	* Avoid setting maxfiles for a remote asterisk
 
-	  POSIX defines poll.h. sys/poll.h should not be used as it is c-library
-	  internal header which may or may not exist. Notably in musl including
-	  sys/poll.h generates warning of being incorrect.
+	  Setting maxfiles (maximum number of open files) has no practical
+	  effect on a remote asterisk (rasterisk, rasterisk -x).
 
-	  Change-Id: Ib318c1c7142a737bcf3caa4d8d72560bebe39252
+	  It has an ill effect of printing an extra message, which
+	  may be annoying in case of -x.
 
-2016-11-26 10:57 +0000 [0cc8351484]  Michael Kuron <m.kuron@gmx.de>
+	  ASTERISK-27105 #close
 
-	* chan_sip: Fix segfault during module unload
+	  Change-Id: Iaf9eb344e4b4b517df91b736b27ec55f6a6921a2
 
-	  If a TCP/TLS connection was pending (not accepted and not timed out) during
-	  unload of chan_sip, Asterisk would segfault when trying to send a signal to
-	  a thread whose thread ID hadn't been recorded yet. This commit fixes that by
-	  recording the thread ID before calling the blocking connect() syscall.
-	  This was a regression introduced by 776a14386a55b5425c7e9617eff8af8b45427144.
+2017-07-05 15:31 +0000 [303f935a50]  George Joseph <gjoseph@digium.com>
 
-	  The above wasn't enough to fix the segfault, which was now delayed to the
-	  point where connect() timed out. Therefore, it was necessary to also remove
-	  the SA_RESTART flag from the SIGURG sigaction so that pthread_kill() could be
-	  used to interruput the connect() syscall.
-	  This was a regression introduced by 5d313f51b982a18f7321adcf7c7a4e822d8b2714.
+	* http.c:  Reduce log spam
 
-	  ASTERISK-26586 #close
+	  Messages like "fwrite() failed: Connection reset by peer" are no
+	  help whatsoever, especially since they can be caused simply by a
+	  client disconnecting.
 
-	  Change-Id: I76fd9d47d56e4264e2629bce8ec15fecba673e7b
+	  * Make those WARNINGs DEBUGs.
+	  * Check the return from ast_iostream_printf of headers.
 
-2016-11-11 08:16 +0000 [8756ce64b7]  gestoip2 <gestoip2@ull.edu.es>
+	  Change-Id: I17bd5f3621514152a7b2b263c801324c5e96568b
 
-	* res_rtp_asterisk: RTT miscalculation in RTCP
+2017-07-07 11:19 +0000 [8f72128e66]  Benjamin Keith Ford <bford@digium.com>
 
-	  When retrieving RTCP stats for PJSIP channels, RTT values are unreliable.
-	  RTT calculation is correct, but the data representation isn't.  RTT is
-	  represented by a 32-bit fixed-point number with the integer part in the
-	  first 16 bits and the fractional part in the last 16 bits.  In order to
-	  get the RTT value, the fractional part is miscalculated, there is an
-	  unnecessary 16 bit shift that causes overflow.  Besides this there is
-	  another mistake, when transforming the integer value to the fixed point
-	  fractional part via bitwise operation, that loses precision.
+	* res_pjsip: Fix crash with from_user containing invalid characters.
 
-	  * RTT fractional part is no longer shifted, avoiding overflow.
+	  If the from_user field contains certain characters (like @, {, ^, etc.),
+	  PJSIP will return a null value for the URI when attempting to parse it.
+	  This causes a crash when trying to dial out through a trunk that contains
+	  these invalid characters in its from_user field.
 
-	  * RTT fractional part is transformed to its fixed-point value more
-	  precisely.
+	  This change checks the configuration and ensures that an endpoint will
+	  not be created if the from_user contains an invalid character. It also
+	  adds a null check to the PJSIP URI parsing as a backup.
 
-	  * Fixed timeval2ntp() and ntp2timeval() second fraction conversions.
+	  ASTERISK-27036 #close
+	  Reported by: Maxim Vasilev
 
-	  * Fixed NTP timestamp report logging.  The usec was inexplicably
-	  multiplied by 4096.
+	  Change-Id: I0396fdb5080604e0bdf1277464d5c8a85db913d0
 
-	  ASTERISK-26566 #close
-	  Reported by Hector Royo Concepcion
+2017-06-27 19:27 +0000 [03ae8b0105]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ie09bdabfee75afb3f1b8ddfd963e5219ada3b96f
+	* json.c: Add backtrace log to find 'Invalid UTF-8 string' errors
 
-2016-11-15 13:44 +0000 [8e77d6f520]  Michael Kuron <m.kuron@gmx.de>
+	  Change-Id: I9020ff9f2b3749904317c0c173f47a1bbed6f929
 
-	* tcptls: Use new certificate upon sip reload
+2017-07-05 13:39 +0000 [9cd8a1df79]  Richard Mudgett <rmudgett@digium.com>
 
-	  Previously, a TLS server socket would only be restarted upon sip reload if the
-	  bind address had changed. This commit adds checking for changes to TLS
-	  parameters like certificate, ciphers, etc. so they get picked up without
-	  requiring a reload of the entire chan_sip module. This does not affect open
-	  connections in any way, but new connections will use the new TLS parameters.
-	  The changes also apply to HTTP and Manager.
+	* res_rtp_asterisk.c: Fix TURN deadlock by using ICE session group lock.
 
-	  ASTERISK-26604 #close
+	  When a message is received on the TURN socket, the code processing the
+	  message needs to call into the ICE/STUN session for further processing.
+	  This code path locks the TURN group lock then the ICE/STUN group lock.  In
+	  another thread an ICE/STUN timer can fire off to send a keep alive message
+	  over the TURN socket.  In this code path, the ICE/STUN group lock is
+	  obtained then the TURN group lock is obtained to send the packet.  A
+	  classic deadlock case if the group locks are not the same.
 
-	  Change-Id: I169e86cefc6dcd627c915134015a6a1ab1aadbe6
-2016-11-11 00:29 +0000 [86d824b7ff]  Timo Teräs <timo.teras@iki.fi>
+	  * Made TURN get created using the ICE/STUN session's group lock.
 
-	* addons/chan_mobile: do not use strerror_r
+	  NOTE: I was originally concerned that the ICE/STUN session can get
+	  recreated by ice_reset_session() for an event like RTCP multiplexing
+	  causing a change during SDP negotiation.  In this case the TURN group lock
+	  would become different.  However, TURN is also recreated as part of the
+	  ICE/STUN recreation in ice_create() when all known ICE candidates are
+	  added to the new ICE session.  While the ICE/STUN and TURN sessions are
+	  being recreated there is a period where the group locks could be
+	  different.
 
-	  The two reasons why it might be used are that some systems do not
-	  implement strerror in thread safe manner, and that strerror_r returns
-	  the error code in the string in case there's no error message.
+	  ASTERISK-27023 #close
+	  Patches:
+	      res_rtp_asterisk-turn-deadlock-fix.patch (license #6502)
+	          patch uploaded by Michael Walton (modified)
 
-	  However, all of asterisk elsewhere uses strerror() and assumes it
-	  to be thread safe. And in chan_mobile the errno is also explicitly
-	  printed so neither of the above reasons are valid.
+	  Change-Id: Ic870edb99ce4988a8c8eb6e678ca7f19da1432b9
 
-	  The reasoning to remove usage is that there are actually two versions
-	  of strerror_r: XSI and GNU. They are incompatible in their return
-	  value, and there's no easy way to figure out which one is being
-	  used. glibc gives you the GNU version if _GNU_SOURCE is defined,
-	  but the same feature test macro is needed for other symbols. On
-	  all other systems you assumedly get XSI symbol, and compilation warnings
-	  as well as non-working error printing.
+2017-07-06 05:55 +0000 [7a4f577eb7]  George Joseph <gjoseph@digium.com>
 
-	  Thus the easiest solution is to just remove strerror_r and use
-	  strerror as rest of the code. Alternative is to introduce ast_strerror
-	  in separate translation unit so it can request the XSI symbol in
-	  glibc case, and replace all usage of strerror.
+	* Fix alembic branches
 
-	  Change-Id: I84d35225b5642d85d48bc35fdf399afbae28a91d
+	  Change-Id: I04f607f084bda9b1b7f626e8e9735c37dc751187
 
-2016-11-21 09:40 +0000 [425da14927]  gtjoseph <gjoseph@digium.com>
+2017-06-23 11:17 +0000 [1028f64be4]  Richard Mudgett <rmudgett@digium.com>
 
-	* build:  Backport addition of librt check to configure.ac
+	* bridge_native_rtp.c: Fix direct media video RTP instance ACL check.
 
-	  A while back, a master-only change was made to check for librt which
-	  should probably have been cherry-picked to 13 at that time.  Sometime
-	  between then and now, part of that change did make it into 13 but it
-	  was incomplete and non-functional.  This patch backports the rest
-	  of the librt check and allows the link of libasteriskpj to use the
-	  results.
+	  The video stream was using the audio stream RTP instance addresses to
+	  check if the video RTP gets directed to an allowed direct media Access
+	  Control List (ACL) address.  There is no guarantee that the video RTP
+	  instance uses the same addresses as the audio RTP instance.
 
-	  Change-Id: I1424008fd8c90f389dda53162ec4a340b253a3c1
+	  This looks like it has been a bug since v11 when direct media ACL was
+	  first added to chan_sip and then faithfully reproduced through a couple
+	  code refactorings into the new bridging architecture.
 
-2016-11-16 12:05 +0000 [2a40c3a867]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I8ddd56320e0eea769f3ceed3fa5b6bdfb51d681a
 
-	* pjproject_bundled:  Improve reliability of pjproject download
+2017-07-05 10:29 +0000 [325eeced6a]  Sean Bright <sean.bright@gmail.com>
 
-	  The download process now has a timeout which will cause wget to retry
-	  if it stops retrieving data for 5 seconds and fetch and curl to timeout
-	  if the whole retrieval take smore than 30 seconds.
+	* core: Remove 'Data Retrieval API'
 
-	  If the tarball retrieval works, the MD5SUM file is retrieved from
-	  the downloads site and the md5 checksum is verified.
+	  This API was not actively maintained, was not added to new modules
+	  (such as res_pjsip), and there exist better alternatives to acquire the
+	  same information, such as the ARI.
 
-	  If either the tarball retrieval or MD5SUM retrieval fails, or the
-	  checksums don't match, the entire process is retried once.  If it
-	  fails again, any incomplete tarball is deleted.
+	  Change-Id: I4b2185a83aeb74798b4ad43ff8f89f971096aa83
 
-	  .DELETE_ON_ERROR: was also added to the Makefile.  Not only does
-	  this delete the tarball on failure, it till also delete corrupted
-	  library files from the pjproject source directory should they
-	  fail to build correctly.
+2017-06-19 11:22 +0000 [d556c67f9f]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  Tested all the way back to FreeBSD 9, CentOS 6, Debian 6 and
-	  Ubuntu 14.
+	* app_queue: Add change priority of call
 
-	  Change-Id: Iea7d33b96a31622ab1b6e54baebaf271959514e1
+	  This patch include a feature to change the priority a caller in a
+	  queue by CLI and AMI.
 
-2016-11-11 07:13 +0000 [12c4e664bc]  Mikheili Dautashvili <mishadaut@gmail.com>
+	  Change-Id: I55d520d71cc1cefe9a9b81fefaefc14679e96133
 
-	* main/app.c: Transmit Silence on ControlPlayback pause
+2017-07-03 10:59 +0000 [910c05455d]  Alexander Traud <pabstraud@compuserve.com>
 
-	  ASTERISK-26562 #close
+	* chan_sip: Only when different, add TCP|TLS in autodomain (SIP Domain Support).
 
-	  Change-Id: Ie6cb0ffc2b8c775639ce7784fe96f4ea00cfa2f8
+	  When sip.conf contained tcpenable=yes and autodomain=yes, the TCP domain was
+	  added in any case, because of a local Boolean-negation error of the return value
+	  of ast_sockaddr_cmp. After fixing this error for TCP and TLS, the TLS domain was
+	  still always added with tlsenable=yes, because the domains were not compared
+	  just on the address but also on the port – and TLS is always on a different port
+	  than UDP/TCP.
 
-2016-11-15 15:01 +0000 [cf6d13180e]  Alexei Gradinari <alex2grad@gmail.com>
+	  ASTERISK-27106
 
-	* chan_pjsip: fix switching sending codec when asymmetric_rtp_codec=no
+	  Change-Id: I14fe9e319e238320b094016980445ef3a5b3337c
 
-	  The sending codec is switched to the receiving codec and then
-	  is switched back to the best native codec on EVERY receiving RTP packets.
-	  This is because after call of ast_channel_set_rawwriteformat there is call
-	  of ast_set_write_format which calls set_format which sets rawwriteformat
-	  to the best native format.
+2017-07-03 10:38 +0000 [4398aa8fa4]  Alexander Traud <pabstraud@compuserve.com>
 
-	  This patch adds a new function ast_set_write_format_path which set
-	  specific write path on channel and uses this function to switch
-	  the sending codec.
+	* chan_sip: Fix a typo for tlsbindaddr in autodomain (SIP Domain Support).
 
-	  ASTERISK-26603 #close
-
-	  Change-Id: I5b7d098f8b254ce8f45546e6c36e5d324737f71d
-
-2016-11-10 13:34 +0000 [ee73af1d88]  gtjoseph <gjoseph@digium.com>
+	  Because of a copy-and-paste error when the struct ast_sockaddr changed,
+	  tlsbindaddr was not added, when sip.conf contained autodomain=yes; see
+	  "show sip domains" on the command-line interface (CLI) of Asterisk.
 
-	* Update for 13.12.2
+	  ASTERISK-27106
 
-2016-11-04 10:57 +0000 [a3614d75f6]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: I3d0957150017c223136968ef1266f275d0d6695e
 
-	* Revert "chan_sip: Fix lastrtprx always updated"
+2017-06-29 13:58 +0000 [950b39a4f5]  Sean Bright <sean.bright@gmail.com>
 
-	  This reverts commit 93332cb1d0eea18021ea6538237297e627d6e2fc.
+	* app_voicemail: Cleanup ODBC connection handling
 
-	  Unfortunately, the aforementioned commit caused a regression (incoming calls
-	  would eventually disconnect). Thus it is being removed.
+	  The primary focus of this patch is adding a missing call to
+	  ast_odbc_release_obj(), but is also a general cleanup of the ODBC
+	  related code in app_voicemail.
 
-	  ASTERISK-26523 #close
-	  ASTERISK-25270
+	  ASTERISK-27093 #close
 
-	  Change-Id: Ibf5586adc303073a8eac667a4cbfdb6be184a64d
+	  Change-Id: I8e285142eaeb3146b4287a928276b70db76c902b
 
-2016-10-27 13:48 +0000 [7d7b52c434]  Mark Michelson <mmichelson@digium.com>
+2017-06-30 23:57 +0000 [50ddb56dad]  Corey Farrell <git@cfware.com>
 
-	* Update for 13.12.1
+	* channel: Clear channel flag in error branch.
 
-2016-10-26 07:51 +0000 [9c761b8f45]  Joshua Colp <jcolp@digium.com>
+	  Clear channel flag AST_FLAG_END_DTMF_ONLY in ast_waitfordigit_full when
+	  ast_read returns NULL.
 
-	* app_voicemail: Clear voice mailbox in MailboxExists and MAILBOX_EXISTS.
+	  ASTERISK-27100 #close
 
-	  When executing the MailboxExists dialplan application and
-	  MAILBOX_EXISTS dialplan function the passed in temporary voice
-	  mailbox was not cleared, causing it to try to free garbage.
+	  Change-Id: Id3039e9a4e74e0cb359f636c9fd0c9740ebf7d9d
 
-	  ASTERISK-26503 #close
+2017-06-29 18:27 +0000 [b485f6c59c]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ie21ccfa1b80b9c59318e596f6b8e17da2b5a7cb3
+	* pjsip_distributor.c: Fix deadlock with TCP type transports.
 
-2016-10-25 14:13 +0000 [226a7e36c5]  Mark Michelson <mmichelson@digium.com>
+	  When a SIP message comes in on a transport, pjproject obtains the lock on
+	  the transport and pulls the data out of the socket.  Unlike UDP, the TCP
+	  transport does not allow concurrent access.  Without concurrency the
+	  transport lock is not released when the transport's message complete
+	  callback is called.  The processing continues and eventually Asterisk
+	  starts processing the SIP message.  The first thing Asterisk tries to do
+	  is determine the associated dialog of the message to determine the
+	  associated serializer.  To get the associated serializer safely requires
+	  us to get the dialog lock.
 
-	* Update for 13.12.0
+	  To send a request or response message for a dialog, pjproject obtains the
+	  dialog lock and then obtains the transport lock.  Deadlock can result
+	  because of the opposite order the locks are obtained.
 
-2016-10-17 14:08 +0000 [df75b647da]  Mark Michelson <mmichelson@digium.com>
+	  * Fix the deadlock by obtaining the serializer associated with the dialog
+	  another way that doesn't involve obtaining the dialog lock.  In this case,
+	  we use an ao2 container to hold the associated endpoint and serializer.
+	  The new locks are held a brief time and won't overlap other existing lock
+	  times.
 
-	* Update for 13.12.0-rc1
+	  ASTERISK-27090 #close
 
-2016-11-18 18:59 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  Change-Id: I9ed63f4da9649e9db6ed4be29c360968917a89bd
 
-	* asterisk 13.13.0-rc1 Released.
+2017-06-29 18:22 +0000 [65a5ac0168]  Richard Mudgett <rmudgett@digium.com>
 
-2016-11-18 09:45 +0000 [cb624b10ae]  Mark Michelson <mmichelson@digium.com>
+	* pjsip_distributor.c: Fix unidentified_requests hash functions.
 
-	* Bump ARI version to 1.10.0
+	  The OBJ_SEARCH_xxx defines should not be used as if they were individual
+	  bits.  They represent a multi-bit enumeration value field.
 
-	  The video-related bridge changes mean that the version needs to be
-	  bumped.
+	  Change-Id: I32abc9a475396dab02402a7014357dd94284e17b
 
-	  Change-Id: I41c4495068562bef03aa76728f188b8ac4bd393d
+2017-06-29 15:06 +0000 [e7d41050e0]  Kevin Harwell <kharwell@digium.com>
 
-2016-11-17 10:50 +0000 [bde3d022a3]  Mark Michelson <mmichelson@digium.com>
+	* app_stream_echo: misc bug fixes
 
-	* manager: update minor version
+	  Fixed the following bugs:
 
-	  Based on bridge video AMI event changes, bump the minor version of AMI.
+	  * calls to stream_echo_write had the last two parameters swapped
+	  * ast_read should have been ast_read_stream
+	  * added a null check on the frame's subclass format
 
-	  Change-Id: I02586bd6cafc0baa33ea98c2f75356c0f5e03435
+	  This also resets the update_sent flag upon receiving SRRCHANGE control frame.
+	  This will then force a video update.
 
-2016-11-16 20:24 +0000 [b213045fe4]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-26997
 
-	* build:  Various OpenBSD issues
+	  Change-Id: I6ad7c8253559b800800433c52339e7f5aa583566
 
-	  OpenBSD's 'find' doesn't take the -delete argument so you have to pipe
-	  through 'xargs rm -rf'.
+2017-06-29 14:56 +0000 [7df7b8a90c]  Kevin Harwell <kharwell@digium.com>
 
-	  'echo -e' doesn't like \t starting a line. It just prints 't' which
-	  causes the libasteriskpj.exports file to be garbage.  They were just
-	  cosmetic so they were removed.
+	* res_rtp_asterisk: trigger source change control frame when dtls is established
 
-	  librt doesn't exist so the link of libasteriskpj.so fails. It's not
-	  actually needed for linux anyway so -lrt was removed from the link.
+	  There needed to be a way to notify handlers upstream that DTLS had been
+	  established. This patch makes it so once DTLS has been estalished a source
+	  change control frame is put into the read queue. Any handlers can then watch
+	  for that frame and trigger off of it.
 
-	  res_rtp_asterisk was failing to load because of an undefined
-	  DTLS_method. '|| defined(LIBRESSL_VERSION_NUMBER)' was added to the #if
-	  so DTLSv1_method is used instead.
+	  ASTERISK-27096 #close
 
-	  ASTERISK-26608
+	  Change-Id: I27ff344f5a8c691a1890dfe3254a4b1a49e7f4a0
 
-	  Change-Id: I926ec95b0b69633231e3ad1d6e803b977272c49c
+2017-06-30 08:31 +0000 [f573e599c0]  George Joseph <gjoseph@digium.com>
 
-2016-11-14 18:45 +0000 [404596b790]  gtjoseph <gjoseph@digium.com>
+	* pjproject_bundled:  Allow passing configure options to bundled
 
-	* channel:  Fix issues in hangup scenarios caused by frame deferral
+	  There wasn't any good way to pass options like --host or --build
+	  down to the pjproject configure which makes cross-compiling difficult.
 
-	  ASTERISK-26343
+	  * Added a new PJPROJECT_CONFIGURE_OPTS environment variable which
+	    can be used to pass arbitrary options to pjproject configure.
+	  * Automatically set the pjproject configure --host and --build
+	    options to match those supplied for the asterisk configure.
 
-	  Change-Id: I06dbf7366e26028251964143454a77d017bb61c8
+	  ASTERISK-27097 #close
+	  Reported-by: Kinsey Moore
 
-2016-11-16 15:42 +0000 [2c031b67d3]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I5fa776e110262851173002a26ffe1172e4c35b2e
 
-	* res_format_attr_opus: Fix fmtp generation.
+2017-06-29 14:50 +0000 [c0c99c7618]  George Joseph <gjoseph@digium.com>
 
-	  res_format_attr_opus assumed that the string being passed into it was
-	  empty. It tried to determine if the only thing it had written was
+	* chan_pjsip:  Fix ability to send UPDATE on COLP
 
-	  a=fmtp:<num>
+	  When connected_line_method is "invite", we're supposed to determine
+	  if the client can support UPDATE and if it can, send UPDATE instead
+	  of INVITE to avoid the SDP renegotiation.  Not only was pjproject
+	  not setting the PJSIP_INV_SUPPORT_UPDATE flag, we were testing
+	  that invite_tsx wasn't NULL which isn't always the case.
 
-	  And if it had, it would reset the string. Its calculation was off when
-	  working with chan_sip, though. chan_sip passes the entire built SDP
-	  rather than an empty string. This resulted in always putting an empty
-	  fmtp line in the SDP.
+	  * Updated chan_pjsip/update_connected_line_information to drop the
+	    requirement that invite_tsx isn't NULL.
+	  * Submitted patch to pjproject sip_inv.c that sets the
+	    PJSIP_INV_SUPPORT_UPDATE flag correctly.
+	  * Updated pjsip.conf.sample to clarify what happens when "invite"
+	    is specified.
 
-	  ASTERISK-26520 #close
-	  Reported by scgm11
+	  ASTERISK-27095
 
-	  Change-Id: Ib2e8712d26a47067e5f36d5973577added01dbb5
+	  Change-Id: Ic2381b3567b8052c616d96fbe79564c530e81560
 
-2016-11-15 16:23 +0000 [ed0f1afc8c]  Richard Mudgett <rmudgett@digium.com>
+2017-06-15 03:12 +0000 [fb7247c57c]  Torrey Searle <torrey@voxbone.com>
 
-	* codec_opus: Fix warning when Opus negotiated but codec_opus not loaded.
+	* res_pjsip:  Add DTMF INFO Failback mode
 
-	  When Opus is negotiated but not loaded, the log is spammed with messages
-	  because the system does not know how to calculate the number of samples in
-	  a frame.
+	  The existing auto dtmf mode reverts to inband if 4733 fails to be
+	  negotiated.  This patch adds a new mode auto_info which will
+	  switch to INFO instead of inband if 4733 is not available.
 
-	  * Suppress the warning by supplying a function that assumes 20ms of
-	  samples in the frame.  For pass through support it doesn't really seem to
-	  matter what number of samples is returned anyway.
+	  ASTERISK-27066 #close
 
-	  ASTERISK-26605 #close
+	  Change-Id: Id185b11e84afd9191a2f269e8443019047765e91
 
-	  Change-Id: Icf2273692f040dc2c45b01e72a790d11092f9e0f
+2017-06-29 03:47 +0000 [ab7d99e62d]  Niklas Larsson <niklas@tese.se>
 
-2016-11-14 14:36 +0000 [e632222bc4]  Richard Mudgett <rmudgett@digium.com>
+	* app_queue: Add priority to AMI QueueStatus
 
-	* res_pjsip_outbound_authenticator_digest.c: Fix memory pool leak.
+	  Add priority to callers in AMI QueueStatus response
 
-	  Responding to authentication challenges leaks PJSIP memory pools.
+	  ASTERISK-27092 #close
 
-	  The leak was introduced with a pjproject 2.5.5 API change.
-	  https://trac.pjsip.org/repos/ticket/1929 changed the API usage of
-	  pjsip_auth_clt_init() to require the new API pjsip_auth_clt_deinit() to
-	  clean up cached authentication allocations that get allocated with
-	  pjsip_auth_clt_reinit_req().
+	  Change-Id: I8d1f737a72c7c38f4cfe1a4ee3ecc0a4f85bd199
 
-	  ASTERISK-26516 #close
+2017-05-30 09:12 +0000 [45df25a579]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I4473141b8c3961d0dc91c382beb3876b3efb45c8
+	* chan_pjsip: Add support for multiple streams of the same type.
 
-2016-11-15 12:01 +0000 [c92dcc76da]  gtjoseph <gjoseph@digium.com>
+	  The stream topology (list of streams and order) is now stored with the
+	  configured PJSIP endpoints and used during the negotiation process.
 
-	* file.c/__ast_file_read_dirs:  Fix issues on filesystems without d_type
+	  Media negotiation state information has been changed to be stored
+	  in a separate object. Two of these objects exist at any one time
+	  on a session. The active media state information is what was previously
+	  negotiated and the pending media state information is what the
+	  media state will become if negotiation succeeds. Streams and other
+	  state information is stored in this object using the index (or
+	  position) of each individual stream for easy lookup.
 
-	  One of the code paths in __ast_file_read_dirs will only get executed if
-	  the OS doesn't support dirent->d_type OR if the filesystem the
-	  particular file is on doesn't support it.  So, while standard Linux
-	  systems support the field, some filesystems like XFS do not.  In this
-	  case, we need to call stat() to determine whether the directory entry
-	  is a file or directory so we append the filename to the supplied
-	  directory path and call stat.  We forgot to truncate path back to just
-	  the directory afterwards though so we were passing a complete file name
-	  to the callback in the dir_name parameter instead of just the directory
-	  name.
+	  The ability for a media type handler to specify a callback for
+	  writing has been added as well as the ability to add file
+	  descriptors with a callback which is invoked when data is available
+	  to be read on them. This allows media logic to live outside of
+	  the chan_pjsip module.
 
-	  The logic has been re-written to only create a full_path if we need to
-	  call stat() or if we need to descend into another directory.
+	  Direct media has been changed so that only the first audio and
+	  video stream are directly connected. In the future once the RTP
+	  engine glue API has been updated to know about streams each individual
+	  stream can be directly connected as appropriate.
 
-	  Change-Id: I54e4228bd8355fad65200c6df3ec4c9c8a98dfba
+	  Media negotiation itself will currently answer all the provided streams
+	  on an offer within configured limits and on an offer will use the
+	  topology created as a result of the disallow/allow codec lines.
 
-2015-05-14 17:12 +0000 [7b96e8cc3d]  Maciej Szmigiero <mail@maciej.szmigiero.name>
+	  If a stream has been removed or declined we will now mark it as such
+	  within the resulting SDP.
 
-	* Add X.509 subject alternative name support to TLS certificate
-	  verification.
+	  Applications can now also request that the stream topology change.
+	  If we are told to do so we will limit any provided formats to the ones
+	  configured on the endpoint and send a re-invite with the new topology.
 
-	  This way one X.509 certificate can be used for hosts that
-	  can be reached under multiple DNS names or for multiple hosts.
+	  Two new configuration options have also been added to PJSIP endpoints:
 
-	  Signed-off-by: Maciej Szmigiero <mail@maciej.szmigiero.name>
+	  max_audio_streams: determines the maximum number of audio streams to
+	  offer/accept from an endpoint. Defaults to 1.
 
-	  ASTERISK-25063 #close
+	  max_video_streams: determines the maximum number of video streams to
+	  offer/accept from an endpoint. Defaults to 1.
 
-	  Change-Id: I13302c80490a0b44c43f1b45376c9bd7b15a538f
+	  ASTERISK-27076
 
-2016-11-14 15:57 +0000 [0790aa528a]  Matt Jordan <mjordan@digium.com>
+	  Change-Id: I8afd8dd2eb538806a39b887af0abd046266e14c7
 
-	* pjproject: Use a much higher limit for PJ_ICE_MAX_CHECKS
+2017-06-28 09:03 +0000 [642f8356ab]  Joshua Colp <jcolp@digium.com>
 
-	  The PJ_ICE_MAX_CHECKS constant is used by pjproject to determine how
-	  many pairs of local/remote candidates will be made. If for some reason
-	  we reach this upper bound, ICE will generally fail and no media will
-	  flow between the browser and Asterisk.
+	* res_rtp_asterisk: Fix issues with ICE renegotiation.
 
-	  This patch makes PJ_ICE_MAX_CHECKS set to the total possible number of
-	  pairs of candidates we'd theoretically allow, which is
-	  PJ_ICE_MAX_CAND^2. Prior to this patch, we simply multiplied
-	  PJ_ICE_MAX_CAND by two; on systems with multiple interfaces (I blame
-	  Docker), this is far too low to allow WebRTC calls to succeed.
+	  When re-inviting to add more streams it is possible for
+	  the role of existing ICE sessions to be changed to the
+	  incorrect value. This results in subsequent refreshes
+	  within the sessions getting a role conflict and the ICE
+	  session breaking down. This change only sets the role to
+	  be the new value if an ICE renegotiation is actually
+	  going to happen, otherwise the existing role is preserved.
 
-	  Setting this to be PJ_ICE_MAX_CAND^2 allowed WebRTC calls to succeed
-	  even when the system Asterisk was running on had quite a few virtual
-	  interfaces.
+	  As well if we encounter a situation where a unidirectional
+	  ICE negotiation happens and the other side does not send us
+	  candidates we will not store any information for sending
+	  traffic, even though we know where they are reachable. This
+	  change fixes this by using the source of the ICE traffic
+	  itself as the target if no candidates are known and we
+	  receive some ICE traffic.
 
-	  Change-Id: Icd4f17de0ac9d3a83dddfc8bf1cb7616bc107d55
+	  ASTERISK-27088
 
-2016-11-14 15:32 +0000 [993a6f96c7]  Matt Jordan <mjordan@digium.com>
+	  Change-Id: I71228181e358917fcefc3100fad21b2fc02a59a9
 
-	* apps/app_echo: Only relay a single video source change frame
+2017-06-27 10:46 +0000 [a48d3e4d31]  Torrey Searle <torrey@voxbone.com>
 
-	  In 9785e8d0, app_echo was updated to relay video source updates to the
-	  channel for the purposes of displaying video in WebRTC tests.
-	  Unfortunately, this can cause a Kafkaesque nightmare if two or more
-	  Local channels are in a bridge together where their ends are in
-	  app_echo. When this situation occurs, a video update sent into app_echo
-	  will cause the video update to be relayed to the other Local channels,
-	  causing another round of video updates, etc. In not much time at all,
-	  the channel length queues will be overwhelmed, channel alert pipes will
-	  fail, and all hell will break loose as Asterisk merrily continues to
-	  throw more video update requests onto the channels.
+	* res/res_pjsip_t38: fix incorrect increment of media_count
 
-	  This patch updates app_echo to *only* relay a single video update. Once
-	  a video update has been made, all further video updates are dropped.
-	  This meets the intended purpose of the original patch: if we get a video
-	  update and we're in app_echo, go ahead and ask the sender to update
-	  themselves. However, once we've got that video stream sync'd up, don't
-	  keep spamming the world.
+	  The T38 sdp callback incorrectly has a side effect of incrementing
+	  the media_count.  This can lead to core dumps.
 
-	  Change-Id: I9210780b08d4c17ddb38599d1c64453adfc34f74
+	  Change-Id: I7bb2f4987de4046ec52cfc34e5ea0662dae32af8
 
-2016-11-08 10:11 +0000 [d23b4af477]  Matt Jordan <mjordan@digium.com>
+2017-06-08 22:50 +0000 [80e11bd79b]  George Joseph <gjoseph@digium.com>
 
-	* res/ari/resource_bridges: Add the ability to manipulate the video source
+	* bridge_native_rtp: Keep rtp instance refs on bridge_channel
 
-	  In multi-party bridges, Asterisk currently supports two video modes:
-	   * Follow the talker, in which the speaker with the most energy is shown
-	     to all participants but the speaker, and the speaker sees the
-	     previous video source
-	   * Explicitly set video sources, in which all participants see a locked
-	     video source
+	  There have been reports of deadlocks caused by an attempt to send a frame
+	  to a channel's rtp instance after the channel has left the native bridge
+	  and been destroyed.  This patch effectively causes the bridge channel to
+	  keep a reference to the glue and both the audio and video rtp instances
+	  so what gets started will get stopped.
 
-	  Prior to this patch, ARI had no ability to manipulate the video source.
-	  This isn't important for two-party bridges, in which Asterisk merely
-	  relays the video between the participants. However, in a multi-party
-	  bridge, it can be advantageous to allow an external application to
-	  manipulate the video source.
+	  ASTERISK-26978 #close
+	  Reported-by: Ross Beer
 
-	  This patch provides two new routes to accomplish this:
-	  (1) setVideoSource: POST /bridges/{bridgeId}/videoSource/{channelId}
-	      Sets a video source to an explicit channel
-	  (2) clearVideoSource: DELETE /bridges/{bridgeId}/videoSource
-	      Removes any explicit video source, and sets the video mode to talk
-	      detection
+	  Change-Id: I9e1ac49fa4af68d64826ccccd152593cf8cdb21a
 
-	  ASTERISK-26595 #close
+2017-06-27 04:37 +0000 [7827755570]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-	  Change-Id: I98e455d5bffc08ea5e8d6b84ccaf063c714e6621
+	* app_queue: Fix returning to dialplan when a queue is empty
 
-2016-11-14 14:22 +0000 [404a62eeee]  gtjoseph <gjoseph@digium.com>
+	  The fix for ASTERISK-25665 introduced a regression.
+	  The return value of queue_exec used to be 0 in case of leavewhenempty
+	  but it was changed to -1 (returned from wait_our_turn and passed
+	  transparently by queue_exec), thus leading to hangup instead of returning
+	  back to dialplan.
 
-	* Revert "Revert "channel: Use frame deferral API for safe sleep.""
+	  This commit resets the value back to 0 in this case, restoring
+	  original behavior.
 
-	  This reverts commit 58c88cfbaa80cb43419cde9186d643d1c5d24baf.
+	  ASTERISK-27065 #close
+	  Reported by: Marek Cervenka
 
-	  Change-Id: I72692e2b2e83ef6da9390075ff20b138b2c374b6
+	  Change-Id: Id9c83b75aeda463250155e88c5004be52bbca5ac
 
-2016-11-14 14:22 +0000 [09d8febc91]  gtjoseph <gjoseph@digium.com>
+2017-06-19 17:21 +0000 [0cef7b9d4e]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* Revert "Revert "autoservice: Use frame deferral API""
+	* app_voicemail: IMAP connection control
 
-	  This reverts commit 1df434e2b4bd7cc34b9b4addf405a3caa7ac16b8.
+	  A new global option "imap_poll_logout" was added to specify whether need to
+	  disconnect from the IMAP server after polling of mailboxes.
 
-	  Change-Id: Id2b8a8bccbb4bbdd82b792275d4cd6f32563e401
+	  ASTERISK-27068 #close
 
-2016-11-14 14:21 +0000 [ffad2b44df]  gtjoseph <gjoseph@digium.com>
+	  Closing IMAP connection after loading mailbox from voicemail.conf
 
-	* Revert "Revert "AGI: Only defer frames when in an interception routine.""
+	  ASTERISK-24052 #close
 
-	  This reverts commit 6be5d8de0da7e804544507f70382425af9a07b3f.
+	  Change-Id: Ib7558ba04516240a32b65f42e9be64372a0ae12a
 
-	  Change-Id: I4b548137f52ae0686d8f09e21496b778d1c6a797
+2017-06-21 17:57 +0000 [975e271b01]  Richard Mudgett <rmudgett@digium.com>
 
-2016-11-14 14:21 +0000 [2fefb6187f]  gtjoseph <gjoseph@digium.com>
+	* res_pjsip_mwi.c: Eliminate RAII_VAR in contact delete observer
 
-	* Revert "Revert "Add API for channel frame deferral.""
+	  Change-Id: I0bc97c6608de1d1a4228826b3b3be43f162f05f3
 
-	  This reverts commit 6b5a7ced136b7178ae0b2ba39221eba1cd2e37c9.
+2017-06-16 18:08 +0000 [34db4c3993]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: I61d1dbb2e69e1977f684b7dfc8e98211024e1cd1
+	* res_pjsip_mwi: update unsolicited MWI subscriptions on updating contact
 
-2016-11-14 12:16 +0000 [5e0c224043]  gtjoseph <gjoseph@digium.com>
+	  Do not need to unsubscribe/subscribe on creating the ednpoint's contact.
+	  The modified function create_mwi_subscriptions_for_endpoint adds
+	  the subscription only if it does not exist.
 
-	* cli:  Fix ast_el_read_char to work with libedit >= 3.1
+	  The subscriptions aren't added for active contacts
+	  which are retrieved on startup from realtime
+	  if mwi_disable_initial_unsolicited=yes.
+	  Because the mwi_contact_added is not called.
+	  So the subscriptions also should be created on updating contact.
 
-	  Libedit 3.1 is not build with unicode on as a default and so the
-	  prototype for the el_gets callback changed from expecting a char buffer
-	  to accepting a wchar buffer.  If ast_el_read_char isn't changed,
-	  the cli reads garbage from teh terminal.
+	  ASTERISK-26230 #close
 
-	  Added a configure test for (*el_rfunc_t)(EditLine *, wchar_t *) and
-	  updated ast_el_read_char to use the HAVE_ define to detemrine whether
-	  to use char or wchar.
+	  Change-Id: I47e265af9296ca09aa42a316fdacac104148cee4
 
-	  ASTERISK-26592 #close
+2017-06-20 16:05 +0000 [27dae55fb6]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I9099b46f68e06d0202ff80e53022a2b68b08871a
+	* core_local: local channel data not being properly unref'ed and unlocked
 
-2016-11-11 02:41 +0000 [3faca1d4ff]  Igor Goncharovskiy <igor.goncharovsky@gmail.com>
+	  In an earlier version of Asterisk a local channel [un]lock all functions were
+	  added in order to keep a crash from occurring when a channel hung up too early
+	  during an attended transfer. Unfortunately, when a transfer failure occurs and
+	  depending on the timing, the local channels sometime do not get properly
+	  unlocked and deref'ed after being locked and ref'ed. This happens because the
+	  underlying local channel structure gets NULLed out before unlocking.
 
-	* Fix closing rtp ports after call finished in chan_unistim.
+	  This patch reworks those [un]lock functions and makes sure the values that get
+	  locked and ref'ed later get unlocked and deref'ed.
 
-	  Fix ASTERISK-26565 by adding ast_rtp_instance_stop before
-	  rtp instance destroy for chan_unistim. Also several fixes
-	  for displayed text translation.
+	  ASTERISK-27074 #close
 
-	  Change-Id: If42a03eea09bd1633471406bdc829cf98bf6affc
+	  Change-Id: Ice96653e29bd9d6674ed5f95feb6b448ab148b09
 
-2016-09-23 17:54 +0000 [412d43fa21]  Richard Mudgett <rmudgett@digium.com>
+2017-06-20 16:01 +0000 [45a1f4e2ae]  Kevin Harwell <kharwell@digium.com>
 
-	* res_pjsip.c: Rework endpt_send_request() req_wrapper code.
+	* bridge: stuck channel(s) after failed attended transfer
 
-	  * Don't hold the req_wrapper lock too long in endpt_send_request().  We
-	  could block the PJSIP monitor thread if the timeout timer expires.
-	  sip_get_tpselector_from_endpoint() does a sorcery access that could take
-	  awhile accessing a database.  pjsip_endpt_send_request() might take awhile
-	  if selecting a transport.
+	  If an attended transfer failed it was possible for some of the channels
+	  involved to get "stuck" because Asterisk was not hanging up the transfer target.
 
-	  * Shorten the time that the req_wrapper lock is held in the callback
-	  functions.
+	  This patch ensures Asterisk hangs up the transfer target when an attended
+	  transfer failure occurs.
 
-	  * Simplify endpt_send_request() req_wrapper->timeout code.
+	  ASTERISK-27075 #close
 
-	  * Removed some redundant req_wrapper->timeout_timer->id assignments.
+	  Change-Id: I98a6ecd92d3461ab98c36f0d9451d23adaf3e5f9
 
-	  Change-Id: I3195e3a8e0207bb8e7f49060ad2742cf21a6e4c9
+2017-06-19 11:28 +0000 [a7488f8a70]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-2016-09-21 15:10 +0000 [2e7fc56d3c]  Richard Mudgett <rmudgett@digium.com>
+	* cdr: fix mistake spelling of a word for Unanswered.
 
-	* res_pjsip: Fix tdata leaks in off nominal paths.
+	  Change-Id: I7a610bef369924523a445c7e849ee88cc45dc5df
 
-	  Change-Id: Ie83e06e88c2d60157775263b07e40b61718ac97b
+2017-06-12 16:17 +0000 [d7b6e06abb]  Alexei Gradinari <alex2grad@gmail.com>
 
-2016-10-24 12:41 +0000 [da68b185b3]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_mwi: unsubscribe unsolicited MWI on deleting endpoint last contact
 
-	* res_pjsip_registrar_expire.c: Remove extra linefeed in debug message.
+	  If the endpoint's last contact is deleted unsolicited MWI has to be
+	  unsubscribed.
 
-	  Change-Id: I1f9adb911f23376503396ec8867e8005b755eb94
+	  ASTERISK-27051 #close
 
-2016-11-10 10:57 +0000 [b70eb07c53]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I33e174e0b9dba0998927d16d6d100fda5c7254e0
 
-	* res_pjsip_sdp_rtp: Reject offer of required SRTP without res_srtp.
+2017-06-16 09:31 +0000 [854a6de819]  George Joseph <gjoseph@digium.com>
 
-	  When optimistic SRTP was on it was possible for us to still
-	  set up a call without an audio stream if an offer was received
-	  with required SRTP.
+	* res_stasis:  Plug reference leak on stolen channels
 
-	  This change makes it so this scenario will now fail with a 488
-	  response.
+	  When a stasis channel is stolen by another app, the control
+	  structure is unreffed but never unlinked from the app_controls
+	  container.  This causes the channel reference to leak.
 
-	  ASTERISK-26575
+	  Added OBJ_UNLINK to the callback in channel_stolen_cb.
 
-	  Change-Id: I7d14187037681f48879bd20319ac79d0877318f3
+	  Also added some additional channel lifecycle debug messages to
+	  channel.c.
 
-2016-11-10 08:33 +0000 [71dc333565]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-27059 #close
+	  Repoorted-by: George Joseph
 
-	* app_queue: Add mention of 'ABANDON' variable to CHANGES.
+	  Change-Id: Ib820936cd49453f20156971785e7f4f182c56e14
 
-	  ASTERISK-26558
+2017-06-16 14:56 +0000 [e33bd96638]  Matthew Fredrickson <creslin@digium.com>
 
-	  Change-Id: I1127010181e79c8ac291f72f036cb8e430dc7f7e
+	* formats/format_g729: Fix typo in comment
 
-2016-11-10 07:41 +0000 [6b5a7ced13]  gtjoseph <gjoseph@digium.com>
+	  There was a typo in a comment.  This commit is to fix the typo.
 
-	* Revert "Add API for channel frame deferral."
+	  ASTERISK-27060 #close
 
-	  This reverts commit 9231a56cf3d6f5eca1bf2d37d827453400690773.
-	  Multiple testsuite failures were detected after the fact.
+	  Change-Id: Ic2699f8dbeaacd58ccb6ec3203e853e1babe3235
 
-	  Change-Id: I3bac8d7c3ddb69a4ddf6c5d6de0ffa5ff7ff3af7
+2017-06-08 12:28 +0000 [0ad95bc8a0]  Frederic LE FOLL <frederic.lefoll@c-s.fr>
 
-2016-11-10 07:41 +0000 [6be5d8de0d]  gtjoseph <gjoseph@digium.com>
+	* Core/PBX: Deadlock between dialplan execution and application unregistration.
 
-	* Revert "AGI: Only defer frames when in an interception routine."
+	  Not easy to reproduce, but we have noticed deadlocks when unloading a module
+	  while dialplan is handling a request.
 
-	  This reverts commit 5c10091f3d1430c6fc04015226f8c3e3aa9d8282.
-	  Multiple testsuite failures were detected after the fact.
+	  The deadlock is between :
+	  1) Dialplan execution: pbx_extension_helper() first taking conlock,
+	  then pbx_findapp() [when called] asking for lock on apps list.
+	  2) Application unregistration: ast_unregister_application() first taking lock
+	  on apps list, then unreference_cached_app() [when called] asking for conlock.
 
-	  Change-Id: I397a841acc17ae230c512449cd6bed89d2ef3b73
+	  As a protection, I suggest to modify ast_unregister_application(), so that it
+	  anticipates the need of conlock, before taking the lock on apps list.
+	  The side effect is a longer unavailability of conlock when unregistering an
+	  application.
 
-2016-11-10 07:41 +0000 [1df434e2b4]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-27041
 
-	* Revert "autoservice: Use frame deferral API"
+	  Change-Id: I0db0f1eb320da6a5758cce3a47d765be1face8e2
 
-	  This reverts commit 2e3a3545754749de21873bfdc6d1a40ec7d8893f.
-	  Multiple testsuite failures were detected after the fact.
+2017-06-12 09:23 +0000 [7a46309d3d]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: Ia45fa4633fae74dca345b24bb6722737c63035de
+	* res_pjsip: New endpoint option "notify_early_inuse_ringing"
 
-2016-11-10 07:40 +0000 [58c88cfbaa]  gtjoseph <gjoseph@digium.com>
+	  This option was added to control whether to notify dialog-info state
+	  'early' or 'confirmed' on Ringing when already INUSE.
+	  The value "yes" is useful for some SIP phones (Cisco SPA)
+	  to be able to indicate and pick up ringing devices.
 
-	* Revert "channel: Use frame deferral API for safe sleep."
+	  ASTERISK-26919 #close
 
-	  This reverts commit 44f7e252397fd87420b3374df26941d7436401b3.
-	  Multiple testsuite failures were detected after the fact.
+	  Change-Id: Ie050bc30023543c7dfb4365c5be3ce58c738c711
 
-	  Change-Id: I56299087da22128a95f0c8f3955f740890d7ca65
+2017-06-15 13:48 +0000 [53b7df82f4]  Alexei Gradinari <alex2grad@gmail.com>
 
-2016-11-09 18:18 +0000 [a562fbe618]  gtjoseph <gjoseph@digium.com>
+	* app_voicemail: IMAP logout on reload/unload
 
-	* build:  Fix default values for some SANITIZER options
+	  Closing IMAP connection on module reload or unload.
 
-	  2 of the sanitizers didn't have default values so in systems that
-	  don't support sanitizers menuselect would spit out warnings.  They
-	  were harmless but confusing.  They've now been set to "0".
+	  ASTERISK-24052 #close
 
-	  Change-Id: I08dc495e3b83f1feac3160b421f538c375fc5d58
+	  Change-Id: I2a40182aa9ef249fa6865d33570430e9ada68525
 
-2016-11-06 06:04 +0000 [7fd5031c1c]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+2017-03-30 09:33 +0000 [9aeab4aced]  Jan Friesse <jfriesse@redhat.com>
 
-	* app_queue: new variable set when abandoned
+	* res_corosync: Change thread stack size
 
-	  sets the variable ABANDONED to TRUE if the call was not answered.
+	  In Corosync 2.x libraries were changed to use LibQB IPC.
+	  Sadly LibQB IPC doesn't support copy-free access to received buffer, so
+	  Corosync libraries were rewritten to use stack as buffer. Mostly the
+	  needed stack size is quite small, but for all *_dispatch functions, 1MiB
+	  is needed.
 
-	  ASTERISK-26558
+	  Asterisk function ast_pthread_create_background set stack size for new
+	  thread to much smaller AST_BACKGROUND_STACKSIZE (~500KiB).
 
-	  Change-Id: I4729af9bff4eba436d8a776afd3374065d0036d3
+	  This results in Asterisk crash when running with Corosync 2.x.
 
-2016-11-08 10:48 +0000 [e043d1a55c]  Mark Michelson <mmichelson@digium.com>
+	  Patch solves this issue by creating it's own version of
+	  ast_pthread_create_background which sets stack size to much higher value
+	  (actually it's AST_BACKGROUND_STACKSIZE + 3MiB).
 
-	* res_pjsip_session: Do not call session supplements when it's too late.
+	  Another problem may appear when "corosync show members" netconsole
+	  command is executed. It is also executed in thread and also has only
+	  500KiB stack size. Sadly it calls corosync_cfg_get_node_addrs which
+	  again needs at least 1MiB stack.
 
-	  res_pjsip_sesssion was hooking into transaction and invite state
-	  changes. One of the reasons for doing so was due to the
-	  PJSIP_EVENT_TX_MSG event. The idea was that we were hooking into the
-	  message sending process, and so we should call session supplements to
-	  alter the outgoing message.
+	  Solution is to use HAVE_COROSYNC_CFG_STATE_TRACK as a discriminator
+	  between Corosync 1.x and 2.x. If 1.x is found, nothing changes. If 2.x
+	  is found, NodeID is displayed instead of IP address.
 
-	  In reality, this event was meant to indicate that the message either
-	  a) had already been sent, or
-	  b) required a DNS lookup and would be sent when the DNS query
-	  completed.
+	  ASTERISK-25370 #close
+	  Reported by: mdu113
 
-	  In case (a), this meant we were altering an already-sent
-	  request/response for no reason. In case (b), this potentially meant we
-	  could be trying to alter a request/response at the same time that the
-	  DNS resolution completed. In this case, it meant we might be stomping on
-	  memory being used by the thread actually sending the message. This
-	  caused potential crashes and memory corruption.
+	  Change-Id: Id95b0d21ab6e708e7d74ad8786c587211676fa08
 
-	  This patch removes the calls to session supplements from the case where
-	  the PJSIP_EVENT_TX_MSG event occurs. In all of these cases, trying to
-	  alter the message at this point is too late, and it can cause nothing
-	  but harm to try to do it. Because there were no longer any calls to the
-	  handle_outgoing() function, it has been removed.
+2017-06-13 11:33 +0000 [1ac0096512]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ibcc223fb1c3a237927f38754e0429e80ee301e92
+	* res_ari:  Add "module loaded" check to ari stubs
 
-2016-11-03 16:46 +0000 [44f7e25239]  Mark Michelson <mmichelson@digium.com>
+	  The recent change to make the use of LOAD_DECLINE more consistent
+	  caused res_ari to unload itself before declining if the ari.conf
+	  file wasn't found.  The ari stubs though still tried to use the
+	  configuration resulting in segfaults.
 
-	* channel: Use frame deferral API for safe sleep.
+	  This patch creates a new CHECK_ARI_MODULE_LOADED macro which tests
+	  to see if res_ari is actually loaded and causes the stubs to also
+	  decline if it isn't.  The macro was then added to the mustache
+	  template's "load_module" function.
 
-	  This is another case where manual frame deferral can be replaced with
-	  centralized routines instead.
+	  ASTERISK-27026 #close
+	  Reported-by: Ronald Raikes
 
-	  Change-Id: I42cdf205f8f29a7977e599751a57efbaac07c30e
+	  Change-Id: I263d56efa628ee3c411bdcd16d49af6260c6c91d
 
-2016-11-03 16:46 +0000 [2e3a354575]  Mark Michelson <mmichelson@digium.com>
+2017-06-15 12:33 +0000 [11ec2945c7]  Richard Mudgett <rmudgett@digium.com>
 
-	* autoservice: Use frame deferral API
+	* chan_pjsip: Fix PJSIP_MEDIA_OFFER dialplan function read.
 
-	  Rather than use manual frame deferral, just let the channel API do it
-	  for us.
+	  The construction of the returned string assumed incorrectly that the
+	  supplied buffer would always be initialized as an empty string.  If it is
+	  not an empty string we could overrun the supplied buffer by the length of
+	  the non-empty buffer string plus one.  It is also theoreticaly possible
+	  for the supplied buffer to be overrun by a string terminator during a read
+	  operation even if the supplied buffer is an empty string.
 
-	  ASTERISK-26343
+	  * Fix the assumption that the supplied buffer would already be an empty
+	  string.  The buffer is not guaranteed to contain an empty string by all
+	  possible callers.
 
-	  Change-Id: I688386f36e765dbc07be863943a43f26bd5eac49
+	  * Fix string terminator buffer overrun potential.
 
-2016-11-03 16:42 +0000 [5c10091f3d]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: If6a0806806527678c8554b1dcb34fd7808aa95c9
 
-	* AGI: Only defer frames when in an interception routine.
+2017-06-08 11:38 +0000 [e563a1920e]  Richard Mudgett <rmudgett@digium.com>
 
-	  AGI recently was modified to defer important frames. This was because
-	  when AGI was used in a connected line interception routine, the
-	  resulting connected line frame would end up getting discarded by the
-	  AGI.
+	* SDP: Add get/set option calls for RTP sched context per type.
 
-	  However, this caused bad behavior in other cases. Specifically, during a
-	  transfer, if someone attempted to manually set the Caller ID on a
-	  channel in an AGI, the deferred connected line frame would end up
-	  overwriting what had been manually set in the AGI.
+	  Change-Id: I82dc75c63c48904e9e5a49e2205dcc06e88487e4
 
-	  Since the initial issue was specific to interception routines, this
-	  change removes the manual frame deferral from AGI and instead uses the
-	  new frame deferral API in interception routines.
+2017-05-11 18:49 +0000 [716abaf33d]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26343 #close
-	  Reported by Morton Tryfoss
+	* SDP: Search for the ice-lite attribute in the right place.
 
-	  Change-Id: Iab7d39436d0ee99bfe32ad55ef91e9bd88db4208
+	  * Pulled finding the rtcp-mux attribute flag out of the ICE candidate for
+	  loop.  Also ordered the RTCP ICE candidate skip test to fail earlier.
 
-2016-11-03 16:36 +0000 [9231a56cf3]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I8905d9c68563027a46cd3ae14dbcc27e9c814809
 
-	* Add API for channel frame deferral.
+2017-05-11 18:46 +0000 [a95584d079]  Richard Mudgett <rmudgett@digium.com>
 
-	  There are several places in Asterisk that have duplicated logic
-	  for deferring important frames until later.
+	* SDP: Set the remote c= line in RTP instance.
 
-	  This commit adds a couple of API calls to facilitate this automatically.
+	  Change-Id: I23b646392082deab65bedeb19b12dcbcb9216d0c
 
-	  ast_channel_start_defer_frames(): Future reads of deferrable frames on
-	  this channel will be deferred until later.
+2017-06-09 19:03 +0000 [06265b8c8a]  Richard Mudgett <rmudgett@digium.com>
 
-	  ast_channel_stop_defer_frames(): Any frames that have been deferred get
-	  requeued onto the channel.
+	* stream: Add ast_stream_topology_del_stream() and unit test.
 
-	  ASTERISK-26343
+	  Change-Id: If07e3c716a2e3ff85ae905c17572ea6ec3cdc1f9
 
-	  Change-Id: I3e1b87bc6796f222442fa6f7d1b6a4706fb33641
+2017-05-11 14:09 +0000 [0fdb99c268]  Richard Mudgett <rmudgett@digium.com>
 
-2016-11-03 07:42 +0000 [a9ac1f5de4]  Alexander Anikin <may213@yandex.ru>
+	* SDP: Add t= line in sdp_create_from_state()
 
-	* chan_ooh323: Fixes to work right with Cisco devices
+	  Change-Id: I4060391328a893101ed87d0d9bacbbab4fd8b141
 
-	  Changed output packets queue processing algo to one read-one write
-	  instead of all read-all send
+2017-06-14 13:07 +0000 [4797a8bb81]  Richard Mudgett <rmudgett@digium.com>
 
-	  Remove h.245 tunneling parameter from ReleaseComplete packet
+	* stream: Ignore declined streams for some topology calls.
 
-	  ASTERISK-24400 #close
-	  Reported by: Dmitry Melekhov
-	  Tested by: Dmitry Melekhov
+	  * Made ast_format_cap_from_stream_topology() not include any formats from
+	  declined streams.
 
-	  Change-Id: I0b31933b062a21011dbac9a82b8bcfe345f406f6
+	  * Made ast_stream_topology_get_first_stream_by_type() ignore declined
+	  streams to return the first active stream of the type.
 
-2016-11-03 13:10 +0000 [0ee249075a]  Alexander Anikin <may213@yandex.ru>
+	  * Updated unit tests to check these changes have the expected effect.
 
-	* chan_ooh323: reset rrq count on gk registration
+	  Change-Id: Iabbc6a3e8edf263a25fd3056c3c614407c7897df
 
-	  reset registration attempts count on success registration on gatekeeper
+2017-06-15 07:32 +0000 [bd16c3c524]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I5f47351852e0ca76c9ac78421659600e0f106336
+	* channel: Fix reference counting in ast_channel_suppress.
 
-2016-11-06 03:46 +0000 [59c23e1768]  Michael Kuron <m.kuron@gmx.de>
+	  The ast_channel_suppress function wrongly decremented the
+	  reference count of the underlying structure used to keep
+	  track of what should be suppressed on a channel if the
+	  function was called multiple times on the same channel.
 
-	* automon: restore mixing of the both channels after recording stops
+	  This change cleans up the reference counting a bit so
+	  this no longer occurs.
 
-	  This is a regression over Asterisk 11, introduced by
-	  2dc8a060064f359a17f5ebcd515d85fe5203c019. Previously, recordings started via
-	  the automon DTMF code would automatically be mixed together using sox because
-	  app_monitor would be called with the m option. This commit restores this
-	  behavior.
+	  ASTERISK-27016
 
-	  Change-Id: Ibaf58684285c3f1b6ca3714524e6d638ae3b3759
+	  Change-Id: I2eed4077cb4916e6626f9f120b63b963acc5c136
 
-2016-11-04 15:42 +0000 [e79acaeb75]  Matt Jordan <mjordan@digium.com>
+2017-06-14 12:34 +0000 [b8b0b61a24]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_http_websocket: Increase the buffer size for non-LOW_MEMORY systems
+	* app_voicemail.c: Fix compile error when IMAP enabled.
 
-	  Not surprisingly, using Respoke (and possibly other systems) it is
-	  possible to blow past the 16k limit for a WebSocket packet size. This
-	  patch bumps it up to 32k, which, at least for Respoke, is sufficient.
-	  For now.
+	  Change-Id: I2703f15b4099b4210c68eccf293105d1975c1fc1
 
-	  Because 32k is laughable on a LOW_MEMORY system (as is 16k, for that
-	  matter), this patch adds a LOW_MEMORY directive that sets the buffer to
-	  8k for systems who have asked for their reduced memory availability to
-	  be considered.
+2017-06-12 17:55 +0000 [023eede265]  Alexei Gradinari <alex2grad@gmail.com>
 
-	  Change-Id: Id235902537091b58608196844dc4b045e383cd2e
+	* app_voicemail: IMAP logout on MWI unsubscribe
 
-2016-11-04 15:40 +0000 [7a83196985]  Matt Jordan <mjordan@digium.com>
+	  Closing IMAP connection on MWI unsubscribe.
 
-	* res_stasis: Set a video source mode on Stasis created bridges
+	  ASTERISK-24052 #close
 
-	  When a bridge is created via ARI (through res_stasis), no video source
-	  mode is set by default. As a result, any endpoint sending video media
-	  won't ever see any video reflected back to it.
+	  Change-Id: I4ff964026002b2817b48c20fb4239f0a880228fd
 
-	  This patch defaults a bridge to a 'follow the talker' video mode.
-	  Further work can be done to add routes that allow for the video mode to
-	  be controlled through the /bridges resource.
+2017-06-14 11:12 +0000 [65ed2ea311]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I7e9d530a5d7a97a4524a9ee4e468e1a6b3443866
+	* res_pjsip_pubsub:  Fix reference to released endpoint
 
-2016-11-04 15:37 +0000 [e7dc536b7a]  Matt Jordan <mjordan@digium.com>
+	  destroy_subscription was attempting to get the id of the
+	  subscription tree's endpoint after we'd already called ao2_cleanup
+	  on it causing a segfault.
 
-	* main/bridge_channel: Fix channel reference leak on video source
+	  Moved the cleanup until after the debug statement and since
+	  endpoint could also be NULL at this point, check for that as well.
 
-	  When a channel is made the video source, the bridge holds a reference to
-	  it. Whenever the video source changes, that reference is released.
-	  However, a ref leak does occur if the channel leaves the bridge (such as
-	  being hung up) while it is the video source, as the bridge never
-	  releases the ref in such a case.
+	  ASTERISK-27057 #close
+	  Reported-by: Ryan Smith
 
-	  This patch adds a line to the bridge_channel_internal_join routine such
-	  that, when a channel finishes its time in the bridge, it notifies the
-	  bridge via ast_bridge_remove_video_src that if it is a video source its
-	  reference should be released.
+	  Change-Id: Ice0a7727f560cf204d870a774c6df71e159b1678
 
-	  ASTERISK-26555 #close
+2017-06-14 08:29 +0000 [ea3f8c6889]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I3a2f5238a9d2fc49c591f0e65199d782ab0be76a
+	* res_pjsip_session:  Correct inverted test in session_outgoing_nat_hook
 
-2016-11-04 15:36 +0000 [7c824b955d]  Matt Jordan <mjordan@digium.com>
+	  There was a typo introduced in commit 776ffd77 which was preventing
+	  the transport's external media address from being used.
 
-	* main/bridge: Add some verbose logging for video source changes
+	  ASTERISK-27024 #close
+	  Reported-by: Christopher van de Sande
+	  patches:
+	  	patch.diff submitted by Florian Floimair (license 6892)
 
-	  It's actually quite useful to see the source of a video stream change.
-	  This doesn't happen terribly often, even with talk detection - but when
-	  it does, it's nice to know which channel is now providing your video
-	  stream.
+	  Change-Id: I7ec617171eaa2d86d2680b00cf37d5088adafc27
 
-	  As a verbose 5 level message, it shouldn't be terribly spammy or costly
-	  to have, and is 'lower level' then most other verbose messages that the
-	  bridge system emits.
+2017-06-14 08:54 +0000 [88f18faf2a]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-26555
+	* res_rtp_asterisk:  Fix ssrc change for rtcp srtp
 
-	  Change-Id: Ia1c20ecafa9670171fd38bddcf3beccae47fb15c
+	  It looks like there was a copy/paste error in ast_rtp_change_source
+	  where if there was a rtcp srtp instance, instead of updating its
+	  ssrc we were updating the srtp instance ssrc twice.
 
-2016-11-04 15:33 +0000 [fd6af2dee8]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-27022 #close
+	  Reported-by: Michael Walton
 
-	* bridges/bridge_softmix: Remove SSRC changes on join/leave; update video source
+	  Change-Id: Ic88f3aee7227b401c58745ac265ff92c19620095
 
-	  WebRTC clients really, really want to know the SSRC of the media they're
-	  getting. Changing the SSRC is generally not a good thing.
+2017-06-08 14:38 +0000 [d6386a8f0c]  Joshua Colp <jcolp@digium.com>
 
-	  bridge_softmix, starting in Asterisk 12, started changing the SSRC of
-	  parties as they joined or left the bridge. With most phones, this isn't
-	  a problem: phones just play back the stream they're getting. With WebRTC
-	  clients, however, the SSRC is tied to a media stream that may be
-	  negotiated. When a new SSRC just shows up, the media can be dropped.
+	* bridge: Add a deferred queue.
 
-	  As it turns out, the SSRC change shouldn't even be necessary. From the
-	  perspective of the client, it's still talking to Asterisk with the same
-	  media stream: why indicate that the far party has suddenly changed to a
-	  different source of media?
+	  This change adds a deferred queue to bridging. If a bridge
+	  technology determines that a frame can not be written and
+	  should be deferred it can indicate back to bridging to do so.
+	  Bridging will then requeue any deferred frames upon a new
+	  channel joining the bridge.
 
-	  This patch opts to just remove the SSRC changes. With this patch, video
-	  clients that join/leave a softmix bridge actually get the video stream
-	  instead of freaking out.
+	  This change has been leveraged for T.38 request negotiate
+	  control frames. Without the deferred queue there is a race
+	  condition between the bridge receiving the T.38 request
+	  negotiate and the second channel joining and being in the
+	  bridge. If the channel is not yet in the bridge then the T.38
+	  negotiation fails.
 
-	  ASTERISK-26555
+	  A unit test has also been added that confirms that a T.38
+	  request negotiate control frame is deferred when no other
+	  channel is in the bridge and that it is requeued when a new
+	  channel joins the bridge.
 
-	  Change-Id: I27fec098b32e7c8718b4b65f3fd5fa73527968bf
+	  ASTERISK-26923
 
-2016-10-28 15:11 +0000 [bd4d7d8ad0]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: Ie05b08523f399eae579130f4a5f562a344d2e415
 
-	* stasis_recording/stored: remove calls to deprecated readdir_r function.
+2017-06-13 14:17 +0000 [9e53c30610]  Kevin Harwell <kharwell@digium.com>
 
-	  The readdir_r function has been deprecated and should no longer be used. This
-	  patch removes the readdir_r dependency (replaced it with readdir) and also moves
-	  the directory search code to a more centralized spot (file.c)
+	* res_pjsip_refer/session: Calls dropped during transfer
 
-	  Also removed a strict dependency on the dirent structure's d_type field as it
-	  is not portable. The code now checks to see if the value is available. If so,
-	  it tries to use it, but defaults back to using the stats function if necessary.
+	  When doing an attended transfer it's possible for the transferer, after
+	  receiving an accepted response from Asterisk, to send a BYE to Asterisk,
+	  which can then be processed before Asterisk has time to start and/or
+	  complete the transfer process. This of course causes the transfer to not
+	  complete successfully, thus dropping the call.
 
-	  Lastly, for most implementations of readdir it *should* be thread-safe to make
-	  concurrent calls to it as long as different directory streams are specified.
-	  glibc falls into this category. However, since it is possible that there exist
-	  some implementations that are not safe, locking has been added for those other
-	  than glibc.
+	  This patch makes it so any BYEs received from the transferer, after the REFER,
+	  that initiate a session end are deferred until the transfer is complete. This
+	  allows the channel that would have otherwise been hung up by Asterisk to
+	  remain available throughout the transfer process.
 
-	  ASTERISK-26412
-	  ASTERISK-26509 #close
+	  ASTERISK-27053 #close
 
-	  Change-Id: Id8f54689b1e2873e82a09d0d0d2faf41964e80ba
+	  Change-Id: I43586db79079457d92d71f1fd993be9a3b409d5a
 
-2016-11-04 10:57 +0000 [cb30963d22]  Kevin Harwell <kharwell@digium.com>
+2017-06-13 10:47 +0000 [b2fd7e5069]  George Joseph <gjoseph@digium.com>
 
-	* Revert "chan_sip: Fix lastrtprx always updated"
+	* pjproject_bundled:  Use the asterisk github mirror for download
 
-	  This reverts commit 93332cb1d0eea18021ea6538237297e627d6e2fc.
+	  We now mirror the pjproject tarball and md5 at
+	  https://github.com/asterisk/third-party/tree/master/pjproject
 
-	  Unfortunately, the aforementioned commit caused a regression (incoming calls
-	  would eventually disconnect). Thus it is being removed.
+	  To improve download reliability, we now get the tarball from
+	  our mirror instead of from pjsip.org.
 
-	  ASTERISK-26523 #close
-	  ASTERISK-25270
+	  ASTERISK-27052 #close
+	  Reported-by: 'alex'
 
-	  Change-Id: Ibf5586adc303073a8eac667a4cbfdb6be184a64d
+	  Change-Id: I60236587a8935bfa71fcc391f4e2ecb31918c08a
 
-2016-11-02 10:52 +0000 [3a1f9c5dab]  Joshua Colp <jcolp@digium.com>
+2017-06-12 09:57 +0000 [42f738e052]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* res_stasis: Don't unsubscribe from a NULL bridge.
+	* res_pjsip_mwi: don't create mwi subscriptions if initial unsolicited disabled
 
-	  A NULL bridge has special meaning in res_stasis for
-	  unsubscribing. It means that a subscription to ALL
-	  bridges should be removed. This should not be done
-	  as part of the normal subscription management in
-	  the res_stasis channel loop.
+	  If sending unsolicited mwi to all endpoints on startup is disabled
+	  (mwi_disable_initial_unsolicited=yes) do not need to create subscriptions.
+	  If there are many (thousands) realtime endpoints configured with unsolicited mwi
+	  and Vociemail Storage configured as ODBC or IMAP there will be huge number of
+	  DB/IMAP requests on startup.
 
-	  ASTERISK-26468
+	  ASTERISK-26230 #close
 
-	  Change-Id: I6d5bea8246dd13a22ef86b736aefbf2a39c15af0
+	  Change-Id: I50ae909639e3ee298b931a54def4b2b9e0fb86c5
 
-2016-11-03 13:45 +0000 [eceab15f33]  Alexander Anikin <may213@yandex.ru>
+2017-06-11 12:06 +0000 [847087a4ff]  Sean Bright <sean.bright@gmail.com>
 
-	* chan_ooh323: Fix infinite loop on read second part of H.225 packet
+	* codecs.conf.sample: Fix max_bandwidth speling error
 
-	  Fix logic on read second part of H.225 packet. There was infinite loop on
-	  wrong connections due to read before poll.
+	  Reported by Sylvain Boily via asterisk-dev mailing list.
 
-	  Change-Id: I42b4bf75c46e4a5c5df5c5ca1f0bd74b8944e7ff
+	  Change-Id: Idc7623f335aea3e144dd369ba383b9a757480a9d
 
-2016-11-03 11:55 +0000 [a9992da4aa]  gtjoseph <gjoseph@digium.com>
+2017-06-08 17:31 +0000 [8d1f54b92e]  Jørgen H <asterisk.org@hovland.cx>
 
-	* pjproject_bundled:  Fix issue with libasteriskpj needing libresample
+	* res_pjsip_transport_websocket: Add NULL check in get_write_timeout
 
-	  libresample is only needed by pjproject if we're building pjsua, which
-	  we only do if TEST_FRAMEWORK is selected.  It's required by pjsua to
-	  process audio which is needed by some testsuite tests.  Unfortunately,
-	  pjproject relies on a newer version of libresample than the version
-	  that ships by most distros so we need to compile the version that's
-	  bundled with pjproject.  Since we only need it for pjsua, we DON'T want
-	  it's symbols exposed when we actually build asterisk.
+	  Added check for NULL return value when calling
+	  ast_sorcery_retrieve_by_id in function get_write_timeout
 
-	  There was a problem however... TEST_FRAMEWORK is only known AFTER we've
-	  already run ./configure on both asterisk and pjproject but pjproject's
-	  ./configure needs to test it to know whether to set up to build
-	  libresample or not.  The previous way of figuring this out was to
-	  always tell ./configure "yes" but not actually build the library.  This
-	  caused an issue where building libasteriskpj was being told to include
-	  libresample but it wasn't actually there.
+	  ASTERISK-27046
 
-	  The solution is to still do a default pjproject configure during an
-	  asterisk ./configure but if makeopts or menuselect.makeopts changes
-	  subsequently, we now reconfigure pjproject, taking into account the
-	  current state of TEST_FRAMEWORK.  Previously, if makeopts or
-	  menuselect.makeopts changed, only a recompile of pjproject was done.
+	  Change-Id: I9357717278da631c3a1cb502c412693929b0cb41
 
-	  Change-Id: I9b5d84c61384a3ae07fe30e85c49698378cc4685
+2017-06-08 10:54 +0000 [d27168d36f]  Guido Falsi <madpilot@freebsd.org>
 
-2016-11-01 19:48 +0000 [714412f6c4]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+	* BuildSystem: Add patches to allow building with recent LibreSSL
 
-	* chan_sip: add missing account code
+	  Add some #if defined checks which allow building against LibreSSL.
+	  These patchess come from OpenBSD ports:
+	  https://cvsweb.openbsd.org/cgi-bin/cvsweb/ports/telephony/asterisk/patches/
 
-	  Added missing account to AMI event of sip show peers
+	  ASTERISK-27043 #close
+	  Reported by: OpenBSD ports
 
-	  ASTERISK-26176 #close
+	  Change-Id: I2f6c08a5840b85ad4d2b75370b947ddde7a9a572
 
-	  Change-Id: Ieb6c2c80a838a1b59c82103eba4c63ba238dc482
+2017-06-06 14:54 +0000 [fcb1a0d7e8]  David M. Lee <dlee@respoke.io>
 
-2016-09-13 04:08 +0000 [0cf1778eed]  Alexander Traud <pabstraud@compuserve.com>
+	* CFLAGS for BIND8 support
 
-	* rtp_engine: Allow more than 32 dynamic payload types.
+	  Some systems (like macOS) require BIND_8_COMPAT to be defined so that
+	  the nameser libraries are, well, BIND8 compatible.
 
-	  The dynamic range (96-127) allows 32 RTP Payload Types. RFC 3551 section 3
-	  allows to reassign other ranges. Consequently, when the dynamic range is
-	  exhausted, you can go for "rtp_pt_dynamic = 35" (or 0) in asterisk.conf. This
-	  enables the range 35-63 (or 0-63) giving room for another 29 (or 64) payload
-	  types.
+	  Change-Id: If79fc27a64f90de1835b5aa3aadfa9be22bd16b0
 
-	  ASTERISK-26311 #close
+2017-06-08 10:36 +0000 [7b668297f3]  Guido Falsi <madpilot@freebsd.org>
 
-	  Change-Id: I7bc96ab764bc30098a178b841cbf7146f9d64964
-	  (cherry picked from commit 9ac53877f688c06acaa7c377f15da8770e4ee88b)
+	* BuildSystem: Fix build on FreeBSD due to missing crypt.h
 
-2016-11-02 09:15 +0000 [d971647949]  Joshua Colp <jcolp@digium.com>
+	  FreeBSD does not include a crypt.h include file. Definitions for
+	  crypt() and crypt_r() are in unistd.h
 
-	* app_dial: Fix incorrect device state when channel is picked up.
+	  ASTERISK-27042 #close
 
-	  Given the scenario where multiple channels are dialed using Dial()
-	  but the caller is picked up using PickupChan() all outgoing channels
-	  except the channel specified to PickupChan() would be marked
-	  as ringing until the call had been hung up.
+	  Change-Id: Ib307ee5e384870c6af50efa89fb73722dd0c3a7e
 
-	  When using the PickupChan application the channel executing the
-	  application is swapped into place of another channel. As part
-	  of this process the channel is answered. The Dial application
-	  has explicit logic which checks if the channel is answered,
-	  cancels all other outgoing channels, and bridges. This logic is
-	  different than the normal logic that is executed when an outgoing
-	  channel is answered. This different logic failed to publish dial
-	  events stating that the other outgoing channels had been canceled.
-	  As a result references to the outgoing channels were held onto by
-	  the dial masquerade process until the call had been ended and
-	  the channels had gone away. This would result in the channels
-	  appearing in the "core show channels" list despite not being present
-	  anymore and would also result in incorrect device state.
+2017-06-07 15:19 +0000 [5b80496b42]  Joshua Colp <jcolp@digium.com>
 
-	  This change makes it so that this logic also publishes
-	  dial events stating that the other outgoing channels have been
-	  canceled.
+	* chan_pjsip: Update device state when in early media.
 
-	  ASTERISK-26549
+	  The chan_pjsip module uses a calculation approach for
+	  determining device state. This means that in situations
+	  where we would expect device state to change we need to
+	  tell the core to query. A scenario that was missed is
+	  when early media was signaled.
 
-	  Change-Id: Iea7168e6e82f7d4609ec0366153804e4f55ea64f
+	  This change adds the notification for the core to
+	  query device state when we are told that early media
+	  is being provided.
 
-2016-11-01 13:13 +0000 [afecb2cfc0]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-27039
 
-	* bundled pjproject: Fix DNS write to freed memory.
+	  Change-Id: Iafebfd152894966344ff2e950a3cee9f59a3eb6f
 
-	  PJPROJECT 2.5.5 introduced a race condition with the -r5349 IPv6 DNS
-	  patch.
+2017-06-07 14:32 +0000 [e497a76d24]  Sean Bright <sean.bright@gmail.com>
 
-	  The patch below fixes a write to freed memory under cartain DNS lookup
-	  conditions.
+	* eventfd: Disable during cross compilation
 
-	  0006-r5477-svn-backport-Fix-DNS-write-on-freed-memory.patch
+	  Reported by Lonnie Abelbeck <lonnie@abelbeck.com> via private e-mail.
 
-	  ASTERISK-26516
-	  Reported by:  Richard Mudgett
+	  Change-Id: Icc80f12b8d8d591e14a8e0ed9f1c02cbd193a89b
 
-	  Change-Id: Ifdfae9ecf1e41b53080f33aab44ce1a220f349c5
+2017-06-07 11:21 +0000 [19da99df2f]  Alexei Gradinari <alex2grad@gmail.com>
 
-2016-11-01 06:56 +0000 [5f188bb7a8]  Joshua Colp <jcolp@digium.com>
+	* CHANGES: correct version for a new option 'refer_blind_progress'
 
-	* res_pjsip_sdp_rtp: Limit number of formats to defined maximum.
+	  Change-Id: If4817d26a8974610827624fb8a4e56d681d6bf97
 
-	  The res_pjsip_sdp_rtp module did not restrict the number of
-	  formats added to a media stream in the SDP to the defined
-	  limit. If allow=all was used with additional loaded codecs this
-	  could result in the next media stream being overwritten some.
+2017-06-06 07:04 +0000 [d3e951edf5]  Joshua Colp <jcolp@digium.com>
 
-	  This change restricts the module to limit it to the defined
-	  maximum and also increases the maximum in our bundled pjproject.
+	* pjsip: Extend 'asymmetric_rtp_codec' option to include us changing.
 
-	  ASTERISK-26541 #close
+	  PJSIP support in Asterisk differs from chan_sip in that it
+	  allows media to be sent as-is without transcoding provided
+	  the codecs were negotiated in the SDP. This is allowed
+	  according to the RFC. Support for this differs quite a lot
+	  though and some endpoints do not handle it well.
 
-	  Change-Id: I0dc5f59d3891246cafa2f3df5ec406f088559ee8
+	  This change extends the 'asymmetric_rtp_codec' option to
+	  also cover this case. When set to no (the default) the code
+	  behaves as chan_sip does - the best codec is selected and
+	  we will only ever send that, unless we change what we are
+	  sending if the remote side changes. When set to yes we
+	  will send media as-is without transcoding if the codec
+	  has been negotiated in the SDP.
 
-2016-11-01 04:18 +0000 [94c9496ed5]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  ASTERISK-26996
 
-	* netsock.c: fix includes for HURD
+	  Change-Id: Ib1647f6902a0843e8c435946f831c2159e8d1d51
 
-	  ASTERISK-25070
+2017-06-06 10:04 +0000 [b3ca24d216]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I43bf94d2d36d3d8a8d0df40cd6c027d65a462814
+	* res_rtp_multicast: Use consistent timestamps when possible
 
-2016-11-01 04:00 +0000 [c1c9487375]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  When a frame destined for a MulticastRTP channel does not have timing
+	  information (such as when an 'originate' is done), we generate the RTP
+	  timestamps ourselves without regard to the number of samples we are
+	  about to send.
 
-	* define PATH_MAX for HURD
+	  Instead, use the same method as res_rtp_asterisk and 'predict' a
+	  timestamp given the number of samples. If the difference between the
+	  timestamp that we generate and the one we predict is within a specific
+	  threshold, use the predicted timestamp so that we end up with timestamps
+	  that are consistent with the number of samples we are actually sending.
 
-	  PATH_MAX is not guaranteed to be defined. In parctice, all but the HURD
-	  define it to a constant. It is indeed not safe to assume there won't be
-	  longer paths and Asterisk generally does err safely on such cases.
+	  Change-Id: I2bf0db3541b1573043330421cbb114ff0f22ec1f
 
-	  So even for HURD we'll just pretend PATH_MAX is 4096.
+2017-05-31 10:41 +0000 [861984eac0]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25070 #close
+	* res_pjsip: Add support for returning only reachable contacts and use it.
 
-	  Change-Id: I53d10ba18c34c132bcb640a5fd8e0da1d9b22db3
+	  This introduces the ability for PJSIP code to specify filtering flags
+	  when retrieving PJSIP contacts. The first flag for use causes the
+	  query code to only retrieve contacts that are not unreachable. This
+	  change has been leveraged by both the Dial() process and the
+	  PJSIP_DIAL_CONTACTS dialplan function so they will now only attempt
+	  calls to contacts which are not unreachable.
 
-2016-10-31 17:35 +0000 [50fa868ab8]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-26281
 
-	* codecs.conf.sample: Add sample and option descriptions for codec_opus
+	  Change-Id: I8233b4faa21ba3db114f5a42e946e4b191446f6c
 
-	  codecs.conf.sample was missing codec opus's configuration options, descriptions,
-	  and examples. This patch adds the configuration options and examples to
-	  codecs.conf.sample that can be used with codec_opus.
+2017-06-05 11:27 +0000 [d8802a6a0f]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-26538 #close
+	* channel: ast_write frame wrongly freed after call to audiohooks
 
-	  Change-Id: I1d89bb5e01d3e3b5bd78951b8dd0ff077a83dc8b
+	  ASTERISK-26419 introduced a bug when calling ast_audiohook_write_list in
+	  ast_write. It would free the frame given to ast_write if the frame returned
+	  by ast_audiohook_write_list was different than the given one. The frame give
+	  to ast_write should never be freed within that function. It is the caller's
+	  resposibility to free the frame after writing (or when it its done with it).
+	  By freeing it within ast_write this of course led to some memory corruption
+	  problems.
 
-2016-11-01 08:32 +0000 [b3f10b7b94]  Grachev Sergey <grachev@mcn.ru>
+	  This patch makes it so the frame given to ast_write is no longer freed within
+	  the function. The frame returned by ast_audiohook_write_list is now subsequently
+	  used in ast_write and is freed later. It is freed either after translate if the
+	  frame returned by translate is different, or near the end of ast_write prior to
+	  function exit.
 
-	* chan_sip: Incorrect display option Outbound reg. retry 403
+	  ASTERISK-26973 #close
 
-	  If in sip.conf (general section) set option register_retry_403=no,
-	  the command "sip show settings" return value:
-	  Outbound reg. retry 403:0
-	  If in sip.conf (general section) set option register_retry_403=yes,
-	  the command "sip show settings" return value:
-	  Outbound reg. retry 403:-1
+	  Change-Id: Ic9085ba5f555eeed12f6e565a638c3649695988b
 
-	  * In static char "sip show settings" for "Outbound.reg. retry 403"
-	  option use AST_CLI_YESNO
+2017-05-31 11:45 +0000 [001f4ddda4]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26476 #close
+	* pbx_builtin: Properly handle hangup during Background
 
-	  Change-Id: I3c14272f05f1067bd2aeaa8b3ef9cf8fcb12dcf9
+	  Before this patch, when a user hung up during a Background, we would
+	  stuff 0xff into a char and attempt a dialplan lookup of it. This caused
+	  problems for some realtime engines which interpreted the value as the
+	  beginning of an invalid UTF-8 sequence.
 
-2016-10-20 07:27 +0000 [29692d4aa4]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-19291 #close
+	  Reported by: Andrew Nowrot
 
-	* res/stasis: Add CLI commands for displaying/debugging ARI apps
+	  Change-Id: I8ca6da93252d61c76ebdb46a4aa65e73ca985358
 
-	  This patch adds three new CLI commands:
-	   - ari show apps: list the registered ARI applications
-	   - ari show app: show detailed information about an ARI application
-	   - ari set debug: dump events being sent to an ARI application
+2017-05-31 04:25 +0000 [f6eeaaafd5]  Joshua Colp <jcolp@digium.com>
 
-	  Note that while these CLI commands live in the res_stasis module, we use
-	  the 'ari' family for these commands. This was done as most users of
-	  Asterisk aren't aware of the semantic differences between ARI and
-	  res_stasis, and some 'ari' CLI commands already exist.
+	* channel / app_meetme: Fix parentheses.
 
-	  ASTERISK-26488 #close
+	  ASTERISK-27025
 
-	  Change-Id: I51ad6ff0cabee0d69db06858c13f18b1c513c9f5
+	  Change-Id: Id736b0aa4ec6b6b0f04663d64fa8d151f81fdbed
 
-2016-10-31 16:12 +0000 [a36a7d0cf4]  gtjoseph <gjoseph@digium.com>
+2017-05-30 16:07 +0000 [9dce4a947b]  Sean Bright <sean.bright@gmail.com>
 
-	* pjproject_bundled:  Fix compile of pjsua so it handles audio
+	* stasis_recording: Correct ast_asprintf error checking
 
-	  In order for pjsua and its python binding to actually negotiate
-	  audio for the testsuite tests, it needs g711 and resample.  The
-	  pj* libraries themselves do not.  Unfortunately, pjproject relies
-	  on a brand new libresample that most distros don't ship so we need
-	  to use the libresample already bundled with pjproject.  Only the pjsua
-	  executable and the _pjsua.so python library are linked with it so it
-	  shouldn't interfere with asterisk itself.
+	  ASTERISK-27021 #close
+	  Reported by: Tim Morgan
 
-	  Also it was pointed out that apply_patches couldn't handle multiple
-	  patches that depended on each other during the dry-run, so the
-	  dry-run was removed.
+	  Change-Id: I0ac061f040093e806c3b1f4e2340864f3ce4dd75
 
-	  Change-Id: I24f397462b486dcdde0dcafe40e6c55a6593f098
+2017-05-28 15:43 +0000 [5c27fe2187]  Sean Bright <sean.bright@gmail.com>
 
-2016-10-31 13:46 +0000 [42bd70b29f]  Etienne Lessard <elessard@proformatique.com>
+	* format: Reintroduce smoother flags
 
-	* manager: Add documentation for NewConnectedLine event.
+	  In review 4843 (ASTERISK-24858), we added a hack that forced a smoother
+	  creation when sending signed linear so that the byte order was adjusted
+	  during transmission. This was needed because smoother flags were lost
+	  during the new format work that was done in Asterisk 13.
 
-	  The NewConnectedLine event has been added by commit fe7671f, but the
-	  documentation was missing.
+	  Rather than rolling that same hack into res_rtp_multicast, re-introduce
+	  smoother flags so that formats can dictate their own options.
 
-	  ASTERISK-26537 #close
+	  Change-Id: I77b835fba0e539c6ce50014a984766f63cab2c16
 
-	  Change-Id: I7fc331f18caa28492da9303e576f70884ca8c9e6
+2017-05-24 10:09 +0000 [39d14834f8]  Mark Michelson <mmichelson@digium.com>
 
-2016-10-30 13:33 +0000 [30b1bc77d2]  Corey Farrell <git@cfware.com>
+	* Confbridge: Add "sfu" video mode to bridge profile options.
 
-	* vector: Prevent NULL argument to memcpy.
+	  A previous commit added plumbing to bridge_softmix to allow for an SFU
+	  experience with Asterisk. This commit adds an option to app_confbridge
+	  that allows for a confbridge to actually make use of the SFU video mode.
+
+	  SFU mode is implemented in a "set it and forget it" kind of way. That
+	  is, when the bridge is created, if SFU mode is enabled, then the video
+	  mode gets set to SFU and cannot be changed. Future improvements may
+	  allow for a hybrid experience (e.g. forward multiple video streams,
+	  specifically those of the most recent talkers), but for this addition,
+	  no such capability is present.
+
+	  Change-Id: I87bbcb63dec6dbbb42488f894871b86f112b2020
+
+2017-05-05 11:56 +0000 [2da869408a]  Mark Michelson <mmichelson@digium.com>
+
+	* Add primitive SFU support to bridge_softmix.
+
+	  This sets up the "plumbing" in bridge_softmix to
+	  be able to accommodate Asterisk asking as an SFU
+	  (selective forwarding unit) for conferences.
+
+	  The way this works is that whenever a channel enters or leaves a
+	  conference, all participants in the bridge get sent a stream topology
+	  change request. The topologies consist of the channels' original
+	  topology, along with video destination streams corresponding to each
+	  participants' source video streams. So for instance, if Alice, Bob, and
+	  Carol are in the conference, and each supplies one video stream, then
+	  the topologies for each would look like so:
+
+	  Alice:
+	  Audio,
+	  Source video(Alice),
+	  Destination Video(Bob),
+	  Destination video (Carol)
+
+	  Bob:
+	  Audio,
+	  Source video(Bob)
+	  Destination Video(Alice),
+	  Destination video (Carol)
+
+	  Carol:
+	  Audio,
+	  Source video(Carol)
+	  Destination Video(Alice),
+	  Destination video (Bob)
+
+	  This way, video that arrives from a source video stream can then be
+	  copied out to the destination video streams on the other participants'
+	  channels.
+
+	  Once the bridge gets told that a topology on a channel has changed, the
+	  bridge constructs a map in order to get the video frames routed to the
+	  proper destination streams. This is done using the bridge channel's
+	  stream_map.
+
+	  This change is bare-bones with regards to SFU support. Some key features
+	  are missing at this point:
+
+	  * Stream limits. This commit makes no effort to limit the number of
+	    streams on a specific channel. This means that if there were 50 video
+	    callers in a conference, bridge_softmix will happily send out topology
+	    change requests to every channel in the bridge, requesting 50+
+	    streams.
+
+	  * Configuration. The plumbing has been added to bridge_softmix, but
+	    there has been nothing added as of yet to app_confbridge to enable SFU
+	    video mode.
+
+	  * Testing. Some functions included here have unit tests.
+	    However, the functionality as a whole has only been verified by
+	    hand-tracing the code.
+
+	  * Selectivenss. For a "selective" forwarding unit, this does not
+	    currently have any means of being selective.
+
+	  * Features. Presumably, someone might wish to only receive video from
+	    specific sources. There are no external-facing functions at the moment
+	    that allow for users to select who they receive video from.
+
+	  * Efficiency. The current scheme treats all video streams as being
+	    unidirectional. We could be re-using a source video stream as a
+	    desetnation, too. But to simplify things on this first round, I did it
+	    this way.
 
-	  Headers declare that memcpy does not accept NULL argument for the first
-	  two parameters.  Add a conditional block to prevent memcpy and ast_free
-	  from running on vectors with NULL element array.
+	  Change-Id: I7c44a829cc63acf8b596a337b2dc3c13898a6c4d
 
-	  ASTERISK-26526 #close
+2017-05-30 09:34 +0000 [045d7b8cb7]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I988a476bb5fcfcbd3f6d6c6b3e7769e4f9629b71
+	* format_mp3: Re-work menuselect/build issues
 
-2016-10-29 10:31 +0000 [b96f18560b]  Corey Farrell <git@cfware.com>
+	  Rather than removing format_mp3 from ALL_C_MODS (which caused format_mp3
+	  to not show up in menuselect), use .PHONY targets when the necessary
+	  source files are not present.
 
-	* astobj2: Declare private variable data_size for AO2_DEBUG only.
+	  ASTERISK-23951
+	  Reported by: Tzafrir Cohen
 
-	  Every ao2 object contains storage for a private variable data_size,
-	  though the value is never read if AO2_DEBUG is disabled.  This change
-	  makes the variable conditional, reducing memory usage.
+	  Change-Id: I0a7512c51acc9e86043671795020b0de725bd9e8
 
-	  ASTERISK-26524 #close
+2017-05-30 09:43 +0000 [80206cdc65]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: If859929e507676ebc58b0f84247a4231e11da07f
+	* test_json:  Fix test names with reserved words
 
-2016-10-28 16:59 +0000 [6b1c55dc9b]  gtjoseph <gjoseph@digium.com>
+	  Some of the test names were actually reserved words (true, false,
+	  int, null, string, bool).  When the jenkins test results analyzer
+	  does its thing it tries to create a map using the test names as
+	  keys and fails because they're reserved words.
 
-	* pjproject_bundled:  Fix issue where "/version.mak" wasn't found
+	  Added "type_" to those test names.
 
-	  main/Makefile includes third-party/pjproject/build.mak but
-	  doesn't set PJDIR beforehand so "include $(PJDIR)/version.mak"
-	  evaluates to "/version.mak".  Fix is to set PJDIR in main/Makefile
-	  before the include.
+	  Change-Id: I90d809f46969c78a1c605b736ff0635196a2cf1b
 
-	  Change-Id: I0f7c67d60209049056fe9c4b041bf0463aa95604
+2017-05-26 11:41 +0000 [9c4f63263c]  Joshua Colp <jcolp@digium.com>
 
-2016-10-28 14:55 +0000 [d7f457e4c1]  Richard Mudgett <rmudgett@digium.com>
+	* manager: Clear the flag on the other channel.
 
-	* bundled pjproject: Crashes while resolving DNS names.
+	  During the channel flag audit an incorrect change was
+	  done. The flag should be cleared on the second channel.
 
-	  PJPROJECT 2.5.5 introduced a race condition with the -r5349 IPv6 DNS
-	  patch.
+	  ASTERISK-26469
 
-	  The patches below fix the DNS lookup race condition crash caused by
-	  attempting to send the same message twice for the single DNS lookup.
+	  Change-Id: I770c5a389550a2fb5a6ade942fccbb2e1d9199c8
 
-	  0006-r5471-svn-backport-Various-fixes-for-DNS-IPv6.patch
-	  0006-r5473-svn-backport-Fix-pending-query.patch
+2017-05-26 11:15 +0000 [1f136fe885]  Sean Bright <sean.bright@gmail.com>
 
-	  The patch below removes a cached DNS response from the hash table when
-	  another thread is referencing the old entry.  The table still contained
-	  the entry when it was destroyed which can result in inexplicable crashes.
+	* res_srtp: Add support for libsrtp2
 
-	  0006-r5475-svn-backport-Remove-DNS-cache-entry.patch
+	  ASTERISK-25294 #close
+	  Reported by: Tzafrir Cohen
 
-	  ASTERISK-26344 #close
-	  Reported by: Ian Gilmour
+	  ASTERISK-26976 #close
+	  Reported by: Alex
 
-	  ASTERISK-26387 #close
-	  Reported by: Harley Peters
+	  Change-Id: I789b1c3d1ed31365bbd9339fa58ef36f48833c40
 
-	  Change-Id: I17fde80359e66f65a91341ceca58d914d0f61cc4
+2017-05-25 11:10 +0000 [59348aa182]  Sean Bright <sean.bright@gmail.com>
 
-2016-10-28 09:50 +0000 [87903a6848]  Rusty Newton <rnewton@digium.com>
+	* format_mp3: Don't try to build format_mp3 if we don't have sources
 
-	* SAC documentation: don't specify transports for endpoints and registrations
+	  ASTERISK-23951 #close
+	  Reported by: Tzafrir Cohen
 
-	  Removing explicit transport definition for endpoints and registrations. It
-	  isn't necessary and isn't generally advised.
+	  Change-Id: Iebf181d44bb735787fde4b5be863c4d7e2478a30
 
-	  ASTERISK-26514 #close
+2017-05-23 11:07 +0000 [44c5a144ce]  Martin Tomec <tomec.martin@gmail.com>
 
-	  Change-Id: Ifdec5e631962438a4683600968dfa4bfd15909fb
+	* Sqlite3: make busy_timeout configurable.
 
-2016-10-27 21:49 +0000 [f373de3020]  Corey Farrell <git@cfware.com>
+	  Enables runtime configuration of busy_timeout for sqlite databases.
+	  Default timeout remains 1000ms.
 
-	* Fix shutdown crash caused by modules being left open.
+	  ASTERISK-27014 #close
 
-	  It is only safe to run ast_register_cleanup callbacks when all modules
-	  have been unloaded.  Previously these callbacks were run during graceful
-	  shutdown, making it possible to crash during shutdown.
+	  Change-Id: I8921a3aac3c335843be4cb17d2dd0a5c157a36da
 
-	  ASTERISK-26513 #close
+2017-05-24 15:50 +0000 [08edd54c1b]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Ibfa635bb688d1227ec54aa211d90d6bd45052e21
+	* unittests:  Add a unit test that causes a SEGV and...
 
-2016-10-26 18:48 +0000 [61a5c3460e]  gtjoseph <gjoseph@digium.com>
+	  ...that can only be run by explicitly calling it with
+	  'test execute category /DO_NOT_RUN/ name RAISE_SEGV'
 
-	* pjproject_bundled:  Remove usage of tar's --strip-components option
+	  This allows us to more easily test CI and debugging tools that
+	  should do certain things when asterisk coredumps.
 
-	  Older versions of tar don't support the --strip-components option so
-	  instead of doing 'tar --strip-components=1 -C source', we now just
-	  untar to the tarball's root directory (pjproject-<version>) and
-	  rename that directory to 'source'.
+	  To allow this a new member was added to the ast_test_info
+	  structure named 'explicit_only'.  If set by a test, the test
+	  will be skipped during a 'test execute all' or
+	  'test execute category ...'.
 
-	  Also fixed an issue where the pjproject source directory is a hard
-	  coded absolute pathname.
+	  Change-Id: Ia3a11856aae4887df9a02b6b081cc777b36eb6ed
 
-	  ASTERISK-26510 #close
-	  ASTERISK-22480 #close
+2017-05-23 15:42 +0000 [d847fe6585]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I9ec92952507a91ff4e4d01e0149e09fd8e8f32b0
+	* res_agi: Allow configuration of audio format of EAGI pipe
 
-2016-10-27 08:07 +0000 [675c71ae8c]  Joshua Colp <jcolp@digium.com>
+	  This change allows the format of the EAGI audio pipe to be changed by
+	  setting the dialplan variable 'EAGI_AUDIO_FORMAT' to the name of one of
+	  the loaded formats.
 
-	* res_pjsip_caller_id: Fix crash on session timers UPDATE on inbound calls.
-
-	  The res_pjsip_caller_id module wrongly assumed that a
-	  saved From header would always exist on sessions. This
-	  is true until an inbound call is received and a session
-	  timer causes an UPDATE to be sent. In this case there will
-	  be no saved From header and a crash will occur. This change
-	  makes it fall back to the From header of the outgoing request
-	  if no saved From header is present.
-
-	  ASTERISK-26307 #close
+	  ASTERISK-26124 #close
 
-	  Change-Id: Iccc3bc8d243b5ede9b81abf960292930c908d4fa
+	  Change-Id: I7a10fad401ad2a21c68c2e7246fa357d5cee5bbd
 
-2016-10-26 07:51 +0000 [14496ce1e5]  Joshua Colp <jcolp@digium.com>
+2017-05-23 13:33 +0000 [e2e6baa8d8]  Sean Bright <sean.bright@gmail.com>
 
-	* app_voicemail: Clear voice mailbox in MailboxExists and MAILBOX_EXISTS.
+	* res_agi: Clarify 'RECORD FILE' documentation
 
-	  When executing the MailboxExists dialplan application and
-	  MAILBOX_EXISTS dialplan function the passed in temporary voice
-	  mailbox was not cleared, causing it to try to free garbage.
+	  Documented the 'beep' option in both the parameters list and the command
+	  description.
 
-	  ASTERISK-26503 #close
+	  ASTERISK-23839 #close
 
-	  Change-Id: Ie21ccfa1b80b9c59318e596f6b8e17da2b5a7cb3
+	  Change-Id: I4970395c922dbdce3f7cf0f56d5b065ec9aa53ea
 
-2016-10-23 07:38 +0000 [e0bc17edff]  Joshua Colp <jcolp@digium.com>
+2017-05-23 13:06 +0000 [3dcb3c88aa]  Sean Bright <sean.bright@gmail.com>
 
-	* pjsip: Fix a few media bugs with reinvites and asymmetric payloads.
+	* res_agi: Prevent crash when SET VARIABLE called without arguments
 
-	  When channel format changes occurred as a result of an RTP
-	  re-negotiation the bridge was not informed this had happened.
-	  As a result the bridge technology was not re-evaluated and the
-	  channel may have been in a bridge technology that was incompatible
-	  with its formats. The bridge is now unbridged and the technology
-	  re-evaluated when this occurs.
+	  Explicitly check that the appropriate number of arguments were passed to
+	  SET VARIABLE before attempting to reference them. Also initialize the
+	  arguments array to zeroes before populating it.
 
-	  The chan_pjsip module also allowed asymmetric codecs for sending
-	  and receiving. This did not work with all devices and caused one
-	  way audio problems. The default has been changed to NOT do this
-	  but to match the sending codec to the receiving codec. For users
-	  who want asymmetric codecs an option has been added, asymmetric_rtp_codec,
-	  which will return chan_pjsip to the previous behavior.
+	  ASTERISK-22432 #close
 
-	  The codecs returned by the chan_pjsip module when queried by
-	  the bridge_native_rtp module were also not reflective of the
-	  actual negotiated codecs. The nativeformats are now returned as
-	  they reflect the actual negotiated codecs.
+	  Change-Id: I5143607d80a2724f749c1674f3126b04ed32ea97
 
-	  ASTERISK-26423 #close
+2017-05-23 12:35 +0000 [e490aa3176]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I6ec88c6e3912f52c334f1a26983ccb8f267020dc
+	* res_agi: Fix malformed AGI usage response
 
-2016-10-26 06:32 +0000 [f534f67f52]  Joshua Colp <jcolp@digium.com>
+	  If the generated XML documentation for a command does not end with a \n,
+	  the postamble of the usage message does not appear on its own line.
 
-	* res_pjsip_sdp_rtp: Fix address family of explicit media_address.
+	  ASTERISK-25662 #close
 
-	  When an explicit media_address is provided the address family
-	  in the SDP needs to be set to reflect it.
+	  Change-Id: If190f1e9e37fe215fed95897d78d4a6e142b0020
 
-	  ASTERISK-26309
+2017-05-23 10:06 +0000 [8ae0227cf3]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ib9350cc91c120eb2f96f0623d3907d12af67eb79
+	* res_format_attr_h26x: Trim blanks in fmtp attributes
 
-2016-10-25 11:20 +0000 [3a2092b722]  gtjoseph <gjoseph@digium.com>
+	  Some devices separate format attributes with a semicolon followed by a
+	  space, so trim blanks before trying to match them.
 
-	* test_astobj2_thrash:  Fix multithreaded issues
+	  ASTERISK-27008 #close
 
-	  The test uses 4 threads to grow, count, lookup and shrink 15K objects
-	  in a container.  If there's only 1 execution engine available, the test
-	  will complete in <50ms.  If each threads gets its own execution engine,
-	  the test may timeout after 60 seconds because the count thread does a
-	  locked ao2_callback on the whole container in a tight loop with only
-	  a sched_yield to give up time.  The lock contention makes the test
-	  execution times wildly variable and mostly timeout.  2 execution
-	  engines are OK, 3 results in about 33% failure rate and >=4 causes
-	  a 80% failure rate.
+	  Change-Id: Ia44cb2e4fef5c73dc541a29da79cb0e19c22d9cc
 
-	  To fix, the sched_yield was changed to a usleep(500).
+2017-05-15 15:03 +0000 [faab058014]  Joshua Colp <jcolp@digium.com>
 
-	  Also, the number of buckets specified for the container was an even
-	  number so that was changed to the next prime number greater than
-	  (MAX_HASH_ENTRIES / 100).  That's 151 currently.
+	* app_queue: Fix members showing as being in call when not.
 
-	  Change-Id: I50cd2344161ea61bfe4b96d2a29a6ccf88385c77
+	  A change was done which added an 'in_call' flag to queue
+	  members that was set to true while talking to an agent.
+	  Unfortunately in practice this does not accurately reflect
+	  whether they are talking to an agent or not. If a Local
+	  channel is involved and a transfer is performed then the
+	  app_queue application would incorrectly think the agent
+	  was still in a call with the caller. This was done to
+	  fix a race condition between an agent becoming available
+	  by device state and the checking of the last call information
+	  for the wrapup time. There was a small window where the
+	  last call information would be the previous value instead
+	  of the new one.
 
-2016-10-24 14:13 +0000 [640203802e]  Pascal Cadotte Michaud <pcadotte@proformatique.com>
+	  This change goes about fixing the original issue in a
+	  different way by considering the call completed if device
+	  state is received which would make the agent available
+	  and if they are currently in a call. If this occurs the
+	  last call information is updated before the agent becomes
+	  available ensuring that old information is not present
+	  when checking if the member should be called. This also
+	  improves the transfer situation by actually updating
+	  and enforcing the wrapup time.
 
-	* typo: s/paranthesis/parenthesis/ in a comment
+	  ASTERISK-26399
+	  ASTERISK-26400
+	  ASTERISK-26715
+	  ASTERISK-26975
 
-	  Change-Id: I7c1f4eb051177ee22cbe97e063d4a3effe29be30
+	  Change-Id: Ife1cb686e3173b3a6d368601adef9aff69d4beea
 
-2016-10-24 10:55 +0000 [9b3557e054]  gtjoseph <gjoseph@digium.com>
+2017-05-23 05:45 +0000 [36e90952ec]  Robert Mordec <r.mordec@slican.pl>
 
-	* pjproject_bundled:  Fixed various build issues
+	* app_confbridge: Race between removing and playing name recording while leaving
 
-	  * CFLAGS is now properly set when using older gcc.
-	  * All third-party pjproject targets have been removed.  This fixes
-	    an issue with older libsrtp in some distros.
-	  * Manually removing the source directory now causes a rebuild.
-	  * EXTERNALS_CACHE_DIR is now properly checked.
-	  * Whitespace fixes.
+	  When user leaves a conference, its channel calls async_play_sound_file()
+	  in order to play the name announcement and then unlinks the sound file.
+	  The async_play_sound_file() function adds a task to conference playback queue,
+	  which then runs playback_common() function in a different thread.
 
-	  Change-Id: I98fec6847efc5602a9f41cb95096fd660a49fa60
+	  It leads to a race condition when, in some cases, channel thread may unlink
+	  the sound file before playback_common() had a chance to open it.
 
-2016-09-19 06:13 +0000 [bb982480d8]  Joshua Colp <jcolp@digium.com>
+	  This patch creates a file deletion task, that is queued after playback.
 
-	* pjsip: Support dual stack automatically.
+	  ASTERISK-27012 #close
 
-	  This change adds support for dual stack automatically. No
-	  configuration is required and the IP address and version
-	  in the SIP messages and SDP will be automatically changed
-	  based on the transport over which the message is being
-	  sent. RTP usage has also been changed to listen on both
-	  IPv4 and IPv6 simultaneously to allow media to flow, and
-	  to allow ICE support on both simultaneously. This also
-	  allows failover between IPv6 and IPv4 to work as expected.
+	  Change-Id: I412f7922d412004b80917d4e892546c15bd70dd3
 
-	  ASTERISK-26309 #close
+2017-05-22 13:51 +0000 [440ff38c08]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I235a421d8f9a326606d861b449fa6fe3a030572d
+	* res_rtp_asterisk: rtcp mux using the wrong srtp unprotecting algorithm
 
-2016-10-17 14:18 +0000 [eff97808fb]  Mark Michelson <mmichelson@digium.com>
+	  When using rtcp mux if an rtcp payload came in it would still use the srtp
+	  unprotect algorithm instead of the srtp unprotect rtcp method. Since rtcp
+	  data was being passed to the rtp unprotect method this would result in an
+	  error.
 
-	* ARI: Detect duplicate channel IDs
+	  This patch ensures that the correct unprotect method is chosen by making
+	  sure the passed in rtcp flag is appropriately set when rtcp mux is enabled
+	  and an rtcp payload is received.
 
-	  ARI and AMI allow for an explicit channel ID to be specified
-	  when originating channels. Unfortunately, there is nothing in
-	  place to prevent someone from using the same ID for multiple
-	  channels. Further complicating things, adding ID validation to channel
-	  allocation makes it impossible for ARI to discern why channel allocation
-	  failed, resulting in a vague error code being returned.
+	  ASTERISK-26979 #close
 
-	  The fix for this is to institute a new method for channel errors to be
-	  discerned. The method mirrors errno, in that when an error occurs, the
-	  caller can consult the channel errno value to determine what the error
-	  was. This initial iteration of the feature only introduces "unknown" and
-	  "channel ID exists" errors. However, it's possible to add more errors as
-	  needed.
+	  Change-Id: Ic5409f9d1a267f1d4785fc5aed867daaecca6241
 
-	  ARI uses this feature to determine why channel allocation failed and can
-	  return a 409 error during origination to show that a channel with the
-	  given ID already exists.
+2017-05-19 10:05 +0000 [0f487978a9]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26421
+	* chan_sip: Better ICE handling for RTCP-MUX
 
-	  Change-Id: Ibba7ae68842dab6df0c2e9c45559208bc89d3d06
+	  If we are offered or are offering RTCP-MUX, don't consider RTCP ICE
+	  candidates. This confuses certain browsers (current Firefox for
+	  example) and causes intial audio setup delays.
 
-2016-10-19 17:53 +0000 [c2036c827c]  snuffy <snuffy22@gmail.com>
+	  ASTERISK-26982 #close
 
-	* Fix issue with CLI not returning to prompt after running "features show"
+	  Change-Id: Ifeaf47e83972fe8dbe58b7fb3d6d1823400cfb91
 
-	  ASTERISK-26444 #close
+2017-05-12 10:38 +0000 [be4beff3e4]  Steve Davies <steve@one47.co.uk>
 
-	  Change-Id: I91d645b7e6e5dba35f8c410df2be77a8c0e3acb8
+	* app_queue: Add QUEUE_RAISE_PENALTY feature
 
-2016-10-04 18:24 +0000 [3c62b60e56]  Michael Walton <mike@farsouthnet.com>
+	  Additional variable to work alongside QUEUE_MAX_PENALTY and QUEUE_MIN_PENALTY,
+	  including an extra parameter in queuerules.conf. This value causes lower
+	  Agent penalty values to "raise up" so that they can join higher penalty agents
+	  and be treated equally after a period of time.
 
-	* res_rtp_asterisk: Add ice_blacklist option
+	  ASTERISK-26995 #close
 
-	  Introduces ice_blacklist configuration in rtp.conf. Subnets listed in the
-	  form ice_blacklist = <subnet spec>, e.g. ice_blacklist =
-	  192.168.1.0/255.255.255.0, are excluded from ICE host, srflx and relay
-	  discovery. This is useful for optimizing the ICE process where a system
-	  has multiple host address ranges and/or physical interfaces and certain
-	  of them are not expected to be used for RTP. Multiple ice_blacklist
-	  configuration lines may be used. If left unconfigured, all discovered
-	  host addresses are used, as per previous behavior.
+	  Change-Id: If1c6421a983667a5ac4c359f6dac25b212b4c459
 
-	  Documention in rtp.conf.sample.
+2017-04-13 17:17 +0000 [7c0466092c]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-26418 #close
+	* AST-2017-003: Handle zero-length body parts correctly.
 
-	  Change-Id: Ibee88f80d7693874fda1cceaef94a03bd86012c9
+	  ASTERISK-26939 #close
 
-2016-10-18 16:30 +0000 [012fda29d2]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I7ea235ab39833a187db4e078f0788bd0af0a24fd
 
-	* CDR: Alter destruction pattern for CDR chains.
+2017-04-13 11:14 +0000 [949e9147bf]  George Joseph <gjoseph@digium.com>
 
-	  CDRs form chains. When the root of the chain is destroyed, it then
-	  unreferences the next CDR in the chain. That CDR is destroyed, and it
-	  then unreferences the next CDR in the chain. This repeats until the end
-	  of the chain is reached. While this typically does not cause any sort of
-	  problems, it is possible in strange scenarios for the CDR chain to grow
-	  way longer than expected. In such a scenario, the destruction pattern
-	  can result in a stack overflow.
+	* AST-2017-004: chan_skinny:  Add EOF check in skinny_session
 
-	  This patch fixes the problem by switching from a recursive pattern to an
-	  iterative pattern for destruction. When the root CDR is destroyed, it is
-	  responsible for iterating over the rest of the CDRs and unreferencing
-	  each one. Other CDRs in the chain, since they are not the root, will
-	  simply destroy themselves and be done. This causes the stack depth not
-	  to increase.
+	  The while(1) loop in skinny_session wasn't checking for EOF so
+	  a packet that was longer than a header but still truncated
+	  would spin the while loop infinitely.  Not only does this
+	  permanently tie up a thread and drive a core to 100% utilization,
+	  the call of ast_log() in such a tight loop eats all available
+	  process memory.
 
-	  ASTERISK-26421 #close
-	  Reported by Andrew Nagy
+	  Added poll with timeout to top of read loop
 
-	  Change-Id: I3ca90c2b8051f3b7ead2e0e43f60d2c18fb204b8
+	  ASTERISK-26940 #close
+	  Reported-by: Sandro Gauci
 
-2016-10-18 09:04 +0000 [6d462b9eaf]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: I2ce65f3c5cb24b4943a9f75b64d545a1e2cd2898
 
-	* chan_pjsip: segfault on already disconnected session
+2017-04-13 17:16 +0000 [2bb98d8fac]  Mark Michelson <mmichelson@digium.com>
 
-	  On heavy loaded system the TCP/TLS incoming calls could be
-	  disconnected by pjproject while these calls are being
-	  processed by asterisk.
+	* AST-2017-002: Ensure transaction key buffer is large enough.
 
-	  This patch uses functions pjsip_inv_add_ref/pjsip_inv_dec_ref
-	  to inform pjproject that an INVITE session is in use.
+	  ASTERISK-26938 #close
 
-	  ASTERISK-26482 #close
+	  Change-Id: I266490792fd8896a23be7cb92f316b7e69356413
 
-	  Change-Id: Ia2e3e2f75358cdb530252a9ce158af3d5d9fdf33
+2017-05-18 16:35 +0000 [4141748e85]  Sean Bright <sean.bright@gmail.com>
 
-2016-10-18 03:01 +0000 [662b560c35]  Alexander Traud <pabstraud@compuserve.com>
+	* res_hep_rtcp: Add support level to module info
 
-	* cli: Auto-complete File not Module for core set debug.
+	  Change-Id: I5661478f9cf12d431f730e42be79323b62831e92
 
-	  Since Asterisk 1.8, the command "core set debug" on the command-line interface
-	  asks not for a file (.c) but a module name. This change shows modules (.so) on
-	  the auto-completion via a tabulator or the question mark. Now, when you
-	  partially type a module name, TAB or ?, you get the correct candidiates.
+2017-05-15 13:26 +0000 [a60d1f3974]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-26480
+	* app_stream_echo: Added a multi-stream echo application
 
-	  Change-Id: I1213f1dd409bd4ff8de08ad80cb0c73cafb1bae0
+	  If the channel does not have multi-stream support then this application acts
+	  just like app_echo. If it does have multi-stream support then each stream is
+	  echoed back to itself (one-to-one).
 
-2016-09-11 10:13 +0000 [6f5880913f]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  If a "num" is specified, then a new topology is made that contains clones (from
+	  the channel's topology) of all media types that are not equal to the given
+	  "type". If the media type differs then the first stream matching the "type" is
+	  cloned into the new topology and then up to "num" - 1 of the same stream are
+	  also cloned into it. Any additional streams from the original topology matching
+	  the "type" are subsequently ignored (i.e. not added to the new topology).
 
-	* menuselect: invalid test for GTK2
+	  For this same case when a frame is read from a stream that frame is still
+	  echoed back like before, but now that frame is also echoed out to the
+	  additional streams that matched on the specified "type".
 
-	  configuire.ac was only checking for the existence of pkg-config
-	  and not the gtk2 package itself.  Now it calls AST_PKG_CONFIG_CHECK
-	  for gtk+-2.0.
+	  ASTERISK-26997 #close
 
-	  ASTERISK-26356 #close
+	  Change-Id: I254144486734178e196c7f590a26ffc13543ff2c
 
-	  Change-Id: I8079d515d6ea99f9ab320a7eaa71c2aaa101ccd5
+2017-05-15 13:25 +0000 [51375686f7]  Kevin Harwell <kharwell@digium.com>
 
-2016-10-13 02:06 +0000 [644fad7477]  Moises Silva <moises.silva@gmail.com>
+	* core/conversions: Added string to unsigned integer and long conversions
 
-	* chan_rtp: Set a sane default rtp engine for unicast.
+	  Added functions that convert a string to an unsigned integer or unsigned long.
+	  A couple of unit test were also created to test the routines. The reasons for
+	  adding these conversion utilities (and hopefully eventually more) are as
+	  follows:
 
-	  ASTERISK-26439
+	    * Conversion routines are functionally contained with consistent and
+	      better error checking
+	    * The function names offer a better description of what is happening
+	    * It encourages code reuse for easier bug fixing at a single source
+	    * It's simpler to use
+	    * It's unit testable
 
-	  Change-Id: I7f5ee2eeba8906e9ecb3293dbe3a747770bb5011
+	  For instance, currently in a lot of places when converting to an integer or
+	  similar the "sscanf" function is used. When using "sscanf" it may not be
+	  immediately clear what's happening as it lacks semantic naming. Limited error
+	  checking is usually done as well. For example, most of the time a check is done
+	  to make sure the value converted, but does not check for overflows or negative
+	  valued conversions when converting unsigned numbers.
 
-2016-10-15 20:05 +0000 [42cfdcd1b7]  Matt Jordan <mjordan@digium.com>
+	  Why use/wrap "strtoul" and not "sscanf" then? Primarily, it lacks some of the
+	  built in error handling that "strtoul" has. For instance "strtoul" contains
+	  overflow checks. Less so, but can still factor as reasons, "sscanf" is slightly
+	  more complex in its use. And maybe a bit controversial, but it may be ("big if")
+	  potentially slower than "strtoul" in some cases.
 
-	* res/ari: Add the Asterisk EID field to outgoing events
+	  Change-Id: If7eaca4a48f8c7b89cc8b5a1f4bed2852fca82bb
 
-	  This patch adds the Asterisk EID field to all outgoing ARI events.
-	  Because this field should be added to all events as they are
-	  transmitted, it is appended to the JSON message just prior to it being
-	  handed off to the application message handler. This makes it somewhat
-	  resilient to both new events being added to ARI, as well as other
-	  potential event transport mechanisms.
+2017-05-13 11:40 +0000 [5a7af00e80]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-26470 #close
+	* asterisk: Audit locking of channel when manipulating flags.
 
-	  Change-Id: Ieff0ecc24464e83f3f44e9c3e7bd9a5d70b87a1d
+	  When manipulating flags on a channel the channel has to be
+	  locked to guarantee that nothing else is also manipulating
+	  the flags. This change introduces locking where necessary to
+	  guarantee this. It also adds helper functions that manipulate
+	  channel flags and lock to reduce repeated code.
 
-2016-10-16 17:25 +0000 [74d9385273]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-26789
 
-	* utils.c:  Fix ast_set_default_eid for multiple platforms
+	  Change-Id: I489280662dba0f4c50981bfc5b5a7073fef2db10
 
-	  ast_set_default_eid was searching for ethX, emX, enoX, ensX and even
-	  pciD#U interface names.  While this was a good attempt, it wasn't
-	  inclusive enough to capture interfaces like enp6s0 or ens6d1, etc.
+2017-05-12 21:04 +0000 [30fbed65f1]  Richard Mudgett <rmudgett@digium.com>
 
-	  Rather than relying on interface names, we now simply find the first
-	  interface returned by the OS that has a hardware address and that
-	  address isn't all 0x00 or all 0xff.  The code IS different for BSD,
-	  Solaris and Linux based on what method is available for enumerating
-	  interfaces.
+	* res_pjsip_session.c: Process initial INVITE sooner. (key exists)
 
-	  Tested on:
-	  FreeBSD9
-	  CentOS6
-	  Ubuntu14
-	  Fedora24
+	  Retransmissions of an initial INVITE could be queued in the serializer
+	  before we have processed the first INVITE message.  If the first INVITE
+	  message doesn't get completely processed before the retransmissions are
+	  seen then we could try to setup the same call from the retransmissions.  A
+	  symptom of this is seeing a (key exists) message associated with an
+	  INVITE.  An earlier change attempted to address this kind of problem by
+	  calculating a distributor serializer to use for unassociated messages.
+	  Part of that change also made incoming calls keep using that distributor
+	  serializer.  (ASTERISK-26088) However, some leftover code was still
+	  deferring the INVITE processing to the session's serializer even though we
+	  were already in that serializer.  This not only is unnecessary but would
+	  cause the same call resetup problem.
 
-	  I was unable to test on Solaris at this time but the code for Solaris
-	  is used elsewhere at Digium.
+	  * Removed the code to defer processing the initial INVITE to the session's
+	  serializer because we are already running in that serializer.
 
-	  Change-Id: Iaa6db87ca78a9a375e47d70e043ae08c1448cb72
+	  ASTERISK-26998 #close
 
-2016-10-13 14:09 +0000 [0306869399]  Leandro Dardini <ldardini@gmail.com>
+	  Change-Id: I1e822d82dcc650e508bc2d40d545d5de4f3421f6
 
-	* app_queue: Added initialization for "context" parameter
+2017-05-14 00:37 +0000 [6e7b78414f]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
 
-	  When using Asterisk Realtime Architecture, empty fields are skipped and the
-	  default values are used. If the "context" parameter in queue was set and then
-	  cleared from the database, the old value remains in memory and it continues
-	  to be used. This change initialize the "context" parameter with an empty value,
-	  allowing clearing the parameter.
+	* Fix spelling queues.conf.sample file
 
-	  ASTERISK-26462 #close
+	  Change-Id: Ie1c2d83af66f27a449da09a68d987e0992627fee
 
-	  Change-Id: I64be73d5044ce38dd02408bd0e53de965ef65905
+2017-05-08 13:40 +0000 [93b7f84c1a]  Vitezslav Novy <a1@vnovy.net>
 
-2016-10-11 06:55 +0000 [a859bcb49c]  Alexander Traud <pabstraud@compuserve.com>
+	* chan_sip: Change sip_get_codec() to return correct codec list
 
-	* chan_sip: Support nat=auto_comedia or nat=force_rport,auto_comedia.
+	  Return cahnnel nativeformats to fix bridge technology selection process.
+	  Same approach as in pjsip module.
 
-	  In the SIP channel driver chan_sip, auto_comedia was expected to be used in
-	  tandem with auto_force_rport. Or stated differently: Only when auto_force_rport
-	  was chosen (the default), auto_comedia worked. This change allows auto_comedia
-	  to be set independently of the state of (auto_)force_rport. For example,
-	  nat=force_rport,auto_comedia is useful for IPv4/IPv6 Dual Stack deployments
-	  when IPv6 clients are behind a Firewall.
+	  ASTERISK-26143
+	  Reported-by: Henning Holtschneider
 
-	  ASTERISK-26457 #close
+	  Change-Id: I64e863753954d6ad67a9e722df2ebc328705ad48
 
-	  Change-Id: Ib29d66c6dbb61648e371e01fc36c6978ddae5bc2
+2017-05-08 15:56 +0000 [808f299808]  Alexei Gradinari <alex2grad@gmail.com>
 
-2016-10-17 19:08 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* res_pjsip: New endpoint option "refer_blind_progress"
 
-	* asterisk 13.12.0-rc1 Released.
+	  This option was added to turn off notifying the progress details
+	  on Blind Transfer. If this option is not set then the chan_pjsip
+	  will send NOTIFY "200 OK" immediately after "202 Accepted".
 
-2016-10-17 11:39 +0000 [546ec4b038]  gtjoseph <gjoseph@digium.com>
+	  Some SIP phones like Mitel/Aastra or Snom keep the line busy until
+	  receive "200 OK".
 
-	* pjproject_bundled:  Add patch to address SSL crash
+	  ASTERISK-26333 #close
 
-	  Addresses crashes when an attempt is made to operate on an SSL socket
-	  after the socket has been closed.
+	  Change-Id: Id606fbff2e02e967c02138457badc399144720f2
 
-	  ASTERISK-26477 #close
+2017-05-11 00:25 +0000 [045dbcc2d6]  Ivan Poddubny <ivan.poddubny@gmail.com>
 
-	  Change-Id: I421305b357558b4f9e690210dc0f4831ef4b3002
+	* app_queue: Fix duplicate queue_log entries for EXITEMPTY and ABANDON
 
-2016-10-15 04:58 +0000 [f1fd873df0]  Michael Kuron <m.kuron@gmx.de>
+	  There are 2 places in app_queue.c that log EXITEMPTY event: one in
+	  wait_our_turn, and another one in queue_exec in the loop trying to
+	  call an agent after wait_our_turn.
 
-	* chan_sip: Only send video on outgoing channel if incoming channel supports it
+	  In most cases it leads to logging EXITEMPTY twice.
 
-	  Previously, the settings videosupport=always and videosupport=yes behaved
-	  identically and unconditionally caused a video offer to be sent in the SDP on
-	  an outgoing call. This was a regression introduced with commit
-	  5a1d90e1fbfc4b48927aad55311f3b38efbf1f54 in Asterisk 1.6.1.
+	  ABANDON is also logged on two places, and in the rare case when an agent
+	  and caller hang up simultaneously it's also possible to get duplicates
+	  in queue_log.
 
-	  This commit restores correct behavior: videosupport=always causes a video offer
-	  to be sent unconditionally, while videosupport=yes will only offer video on an
-	  outbound channel if the incoming channel it is bridged to also supports video.
-	  That way, the device receiving the outgoing call can display the correct user
-	  interface elements for audio or video and will not unnecessarily show a blank
-	  video window on an audio-only call.
+	  This commit changes wait_our_turn to return -1 ("the caller should exit
+	  the queue") instead of 0 ("the caller's turn has arrived") in case of
+	  leaving when empty, so queue_exec skips the agent calling loop.
 
-	  ASTERISK-17470 #close
+	  Also, leave_queue is now executed only once in this case, because 2nd
+	  time is just a noop when the queue entry has already been removed.
 
-	  Change-Id: I782f4409d436114dbc97061c3570c0cd24f7c3ae
+	  Also, it sets qe->handled to -1 to indicate that the call was not
+	  answered by an agent, but the necessary handling has already been done
+	  in order to avoid logging an extra ABANDON entry.
 
-2016-10-14 00:18 +0000 [ce4cfd2eca]  Corey Farrell <git@cfware.com>
+	  ASTERISK-25665 #close
+	  Reported by: Ove Aursand
 
-	* Fix issues with bundled pjproject cached download.
+	  Change-Id: I4578dd383bf2ac41589cf167865e8aaebcd4c11e
 
-	  Previously when testing I had a preexisting makeopts in ASTTOPDIR.  The
-	  ordering of configure.ac causes --with-externals-cache to be processed
-	  after third-party configure.  In cases where the Asterisk clone is
-	  cleaned it would cause pjproject to be downloaded to /tmp.  This
-	  moves processing of the externals cache and sounds cache to happen
-	  before third-party configure.
+2017-04-27 19:37 +0000 [b8659be9b0]  Richard Mudgett <rmudgett@digium.com>
 
-	  This also addresses a possible issue with the third-party Makefile.  If
-	  TMPDIR is set by the environment it would override the path given to
-	  --with-externals-cache.
+	* SDP: Make process possible multiple fmtp attributes per rtpmap.
 
-	  ASTERISK-26416
+	  Change-Id: Ie7511008d82b59590e0eb520a21b5e1da4bd7349
 
-	  Change-Id: Ifab7f35bfcd5a31a31a3a4353cc26a68c8c6592d
+2017-04-28 11:53 +0000 [c2906dfa05]  Richard Mudgett <rmudgett@digium.com>
 
-2016-10-12 16:24 +0000 [3c54328c57]  Richard Mudgett <rmudgett@digium.com>
+	* SDP: Remove sdp_state.remote_capabilities
 
-	* Audit ast_json_pack() calls for needed UTF-8 checks.
+	  The sdp_state.remote_capabilities was only used inside merge_sdps() and
+	  subsequent calls to merge_sdps() by re-INVITE's would leak them.
 
-	  Added needed UTF-8 checks before constructing json objects in various
-	  files for strings obtained outside the system.  In this case string values
-	  from a channel driver's peer and not from the user setting channel
-	  variables.
+	  Change-Id: I0ceb7838ea044cc913e8ad4a255c39c9740ae0ce
 
-	  * aoc.c: Fixed type mismatch in s_to_json() for time and granularity json
-	  object construction.
+2017-05-05 14:30 +0000 [16785c0908]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26466
-	  Reported by: Richard Mudgett
+	* SDP: Add interface_address to specify our address to use.
 
-	  Change-Id: Iac2d867fa598daba5c5dbc619b5464625a7f2096
+	  When we optionally set the interface_address we are forcing the media to
+	  go out a specific interface address.  This allows us to optionally have
+	  the media go out the interface that SIP signalling came in on or if we are
+	  configured to have the media always go out a specific address.
 
-2016-10-12 16:20 +0000 [7f8f125738]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I160d9fac322a075bd2557b430632544178196189
 
-	* json: Check party id name, number, subaddresses for UTF-8.
+2017-05-05 14:49 +0000 [367042bd3e]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Updated unit test as ast_json_name_number() is now NULL tolerant.
+	* SDP: Explicitly stop a RTP instance before destoying it.
 
-	  ASTERISK-26466 #close
-	  Reported by: Richard Mudgett
+	  * Made sdp_add_m_from_rtp_stream() and sdp_add_m_from_udptl_stream()
+	  handle generating disabled/declined streams.
 
-	  Change-Id: I7d4e14194f8f81f24a1dc34d1b8602c0950265a6
+	  * Added /main/sdp/sdp_merge_asymmetric unit test.  It currently does not
+	  check the offerer side negotiated SDP because that isn't the purpose of
+	  this patch and there is much to be done to handle declined/dummy streams.
 
-2016-10-11 18:14 +0000 [9621c9bcbc]  Richard Mudgett <rmudgett@digium.com>
+	  * Added T.38 image streams to the /main/sdp/sdp_merge_symmetric and
+	  /main/sdp/sdp_merge_crisscross unit tests.
 
-	* json: Add UTF-8 check call.
+	  Change-Id: Ib4dcb3ca4f9a9133b376f4e3302f9a1f963f2b31
 
-	  Since the json library does not make the check function public we
-	  recreate/copy the function in our interface module.
+2017-04-28 19:48 +0000 [be5809fac8]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26466
-	  Reported by: Richard Mudgett
+	* SDP: Rework merge_capabilities().
 
-	  Change-Id: I36d3d750b6f5f1a110bc69ea92b435ecdeeb2a99
+	  * Tried to give better variable names.
+	  * Made our SDP answer use the offer's RTP payload types as the SDP RFC
+	  says we SHOULD.
+	  * Updating the local topology now takes the stream format caps.  We are
+	  likely preparing to send an offer.
 
-2016-10-12 17:42 +0000 [e4bb9f9a37]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I34d3be8e3036402a8575ffcae3eebc5ce348d7c0
 
-	* aoc.c: Whitespace cleanup
+2017-04-28 12:30 +0000 [ae7689f093]  Richard Mudgett <rmudgett@digium.com>
 
-	  * In s_to_json() removed unnecessary ast_json_ref() to ast_json_null()
-	  when creating the type json object.  The ref is a noop.
+	* SDP: Update ast_get_topology_from_sdp() to keep RTP map.
 
-	  Change-Id: I2be8b836876fc2e34a27c161f8b1c53b58a3889a
+	  * Add failure exits to ast_get_topology_from_sdp().
 
-2016-10-12 17:27 +0000 [bcac905bd3]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I4cc85c1ede8d712766ed20f544dbcef04c8c1049
 
-	* app_queue.c: Fix clearing of pause reason string.
+2017-05-09 10:34 +0000 [cbbd119c21]  Joshua Colp <jcolp@digium.com>
 
-	  The pause reason is not always cleared when it should be cleared.
+	* tcptls: Improve error messages for TLS connections.
 
-	  * Made set_queue_member_pause() always clear pause reason if not pausing
-	  with a reason string.
+	  This change uses the functions provided by OpenSSL to query
+	  and better construct error messages for situations where
+	  the connection encounters a problem.
 
-	  Change-Id: I993dad19626ec017478a230e980989438b778c53
+	  ASTERISK-26606
 
-2016-10-12 16:22 +0000 [ee4ae2b648]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I7ae40ce88c0dc4e185c4df1ceb3a6ccc198f075b
 
-	* app_minivm.c: Fix malformed ast_json_pack() call.
+2017-05-04 17:28 +0000 [10a4439ac9]  Joshua Elson <joshelson@gmail.com>
 
-	  Change-Id: I082b239022fac462666e52a14a44304748908dc0
+	* Prevent Undefined Capath Crash
 
-2016-10-12 16:30 +0000 [90ae4e4337]  gtjoseph <gjoseph@digium.com>
+	  It is possible to initialize a valid config without a capath
+	  or cafile definition. This will cause a crash on a reload.
 
-	* res_config_mysql:  Fix several issues related to recent table changes
+	  This fix ensures capath is always allocated.
 
-	  Unlike any of the other database drivers, res_config_mysql checks that
-	  the table definition matches the requirements for every insert and
-	  update statement.  Since all requirements are forced to 'char', any
-	  column that isn't a char, like ps_contacts' expiration_time,
-	  qualify_timeout, etc., will throw a warning.  It's kinda harmless but
-	  very misleading.  Since no other driver does those checks on insert
-	  or update, they've been removed from res_config_mysql.  Also, all
-	  the logic that actually attempted to ALTER the table to fix the issue
-	  has been removed.  With the move to alembic, the auto-alter
-	  functionality is not only unnecessary, it's also dangerous.
+	  ASTERISK-26983 #close
 
-	  The other issue is that res_config_mysql calls the mysql_insert_id
-	  function inside store_mysql.  Presumably the intention was to return
-	  the number of rows inserted DESPITE A NOTE IN THE CODE THAT THE VALUE
-	  IS NON_PORTABLE AND MAY CHANGE.  That value is then returned to
-	  config realtime as the number of rows inserted.  Guess what?  The value
-	  changed.  It now only returns the number of rows inserted if there's an
-	  auto increment column on the table, which ps_contacts doesn't have.
-	  Otherwise it returns 0.  So now, the insert worked but we tell config
-	  realtime and sorcery that no rows were inserted.  That call to
-	  mysql_insert_id was removed and we now always return 1 if the insert
-	  succeeded.  We're only inserting 1 row at a time anyway.  If the insert
-	  fails, we still return -1.
+	  Change-Id: I63ff715d9d9023427543a5b8a4ba7b0d82533c12
 
-	  ASTERISK-26362 #close
-	  Reported-by: Carlos Chavez
+2017-05-05 11:33 +0000 [1a1c86239d]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I83ce633efdb477b03c8399946994ee16fefceaf4
+	* cel_odbc:  Fix timestamp processing for microseconds
 
-2016-09-29 13:08 +0000 [86c15db6a1]  Torrey Searle <torrey@voxbone.com>
+	  When a column is of type timestamp, the fraction part of the event
+	  field's seconds was frequently parsed incorrectly especially if
+	  there were leading zeros.  For instance "2017-05-23 23:55:03.023"
+	  would be parsed into an int as "23" then when the timestamp was
+	  formatted again to be inserted into the database column it'd be
+	  "2017-05-23 23:55:03.23" which is now 230 milliseconds instead of
+	  23 milliseconds.  "03.000001" would be transformed to "03.1", etc.
 
-	* res_fax: Fix a tight race condition causing fax to crash in audio fallback
+	  * If the event field is 'eventtime' and the db column is timestamp,
+	    then existing processing has already correctly formatted the
+	    timestamp so now we simply use it rather than parsing it and
+	    re-printing it. This is the most common use case anyway.
 
-	  When T.38 gets rejected and G711 failback occurs there is a period of
-	  time where neither AST_FAX_TECH_T38 nor AST_FAX_TECH_AUDIO is set,
-	  leading to a crash.
+	  * If the event field is other than 'eventtime' and the db column
+	    is timestamp, we now parse the seconds, including the fractional
+	    part into a double rather than 2 ints.  This preserves the
+	    magnitude and precision of the fractional part.  When we print
+	    it, we now print it as a "%09.6lf" which correctly represents the
+	    input.
 
-	  Change-Id: Icc3f457b2292d48a9d7843dac0028347420cc982
+	  To be honest, why we parse the string timestamp into components,
+	  test the components, then print the components back into a string
+	  timestamp is beyond me.  We should use parse it, test it, then if
+	  it passes, use the original string representation in the database
+	  call.  Maybe someone thought that some implementations wouldn't
+	  take a partial timestamp string like "2017-05-06" and decided to
+	  always produce a full timestamp string even if an abbreviated one
+	  was supplied.  Anyway, I'm leaving it as it is.
 
-2016-09-30 16:29 +0000 [29b7a5b00f]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  ASTERISK-25032 #close
+	  Reported-by: Etienne Lessard
 
-	* Add text of cdr directory into README.md for ast-db-manage
+	  Change-Id: Id407e6221f79a5c1120e1a70bc7e893bbcaf1938
 
-	  Change-Id: I68321c4bea50730c39fdb486e5f23aeadd1ad636
+2017-05-09 05:25 +0000 [3c36c29c81]  Joshua Colp <jcolp@digium.com>
 
-2016-10-06 09:58 +0000 [f919edc4e2]  gtjoseph <gjoseph@digium.com>
+	* res_hep_rtcp: Provide chan_sip Call-ID for RTCP messages.
 
-	* app_dial:  Add the "Q" option to set the cause on unanswered channels
+	  This change adds the required logic to allow the SIP
+	  Call-ID to be placed into the HEP RTCP traffic if the
+	  chan_sip module is used. In cases where the option is
+	  enabled but the channel is not either SIP or PJSIP then
+	  the code will fallback to the channel name as done
+	  previously.
 
-	  The "Q" option will set the cause on the unanswered channels when
-	  another channel answers.  It overrides the default of
-	  ANSWERED_ELSEWHERE.
+	  Based on the change on Nir's branch at:
+	  team/nirs/hep-chan-sip-support
 
-	  NOTE:  chan_sip does not support setting the cause on a CANCEL to
-	  anything other than ANSWERED_ELSEWHERE.
+	  ASTERISK-26427
 
-	  ASTERISK-26446 #close
+	  Change-Id: I09ffa5f6e2fdfd99ee999650ba4e0a7aad6dc40d
 
-	  Change-Id: I71742e0919aaa16784c30a2b2e73fbeed7672e47
+2017-05-08 16:11 +0000 [201346fb7d]  George Joseph <gjoseph@digium.com>
 
-2016-10-10 16:59 +0000 [a884b26392]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	* logger:  Added logger_queue_limit to the configuration options.
 
-	* vector: After remove element recheck index
+	  All log messages go to a queue serviced by a single thread
+	  which does all the IO.  This setting controls how big that
+	  queue can get (and therefore how much memory is allocated)
+	  before new messages are discarded. The default is 1000.
+	  Should something go bezerk and log tons of messages in a tight
+	  loop, this will prevent memory escalation.
 
-	  Small fix. It is necessary to double-check
-	  the index that we just removed because there
-	  is a new element.
-
-	  ASTERISK-26453 #close
+	  When the limit is reached, a WARNING is logged to that effect
+	  and messages are discarded until the queue is empty again.  At
+	  that time another WARNING will be logged with the count of
+	  discarded messages.  There's no "low water mark" for this queue
+	  because the logger thread empties the entire queue and processes it
+	  in 1 batch before going back and waiting on the queue again.
+	  Implementing a low water mark would mean additional locking as
+	  the thread processes each message and it's not worth it.
 
-	  Change-Id: Ib947fa94dc91dcd9341f357f1084782c64434eb7
+	  A "test" was added to test_logger.c but since the outcome is
+	  non-deterministic, it's really just a cli command, not a unit
+	  test.
 
-2016-09-29 12:52 +0000 [349c34f72a]  Torrey Searle <torrey@voxbone.com>
+	  Change-Id: Ib4520c95e1ca5325dbf584c7989ce391649836d1
 
-	* res_rtp_asterisk: Fix infinite DTMF issue when switching to P2P bridge
+2017-05-02 18:05 +0000 [56c5c51076]  Richard Mudgett <rmudgett@digium.com>
 
-	  If a bridge switched to P2P when a DTMF was in progress it
-	  was possible for the DTMF to continue being sent indefinitely.
+	* stream: ast_stream_clone() cannot copy the opaque user data.
 
-	  Change-Id: I7e2a3efe0d59d4b214ed50cd0b5d0317e2d92e29
+	  ast_stream_clone() cannot copy the opaque user data stored on a stream.
+	  We don't know how to clone the data so it isn't copied into the clone.
 
-2016-10-10 10:59 +0000 [9da3489d24]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	  Change-Id: Ia51321bf38ecbfdcc53787ca77ea5fd2cabdf367
 
-	* res_pjsip_config_wizard: Memory leak in module_unload
+2017-05-04 17:32 +0000 [924628812b]  Richard Mudgett <rmudgett@digium.com>
 
-	  Fixed a memory leak. It removes only the first element.
-	  Added a useful feature in vector.h to remove all items
-	  under the CMP through a callback function / macro.
+	* netsock2.c: Made get/set addr port avoid potential uninitialized memory.
 
-	  ASTERISK-26453 #close
+	  Change-Id: I532052bd7cd95a4b3565485fc01e2a1ea07ee647
 
-	  Change-Id: I84508353463456d2495678f125738e20052da950
+2017-05-05 08:48 +0000 [4146facfec]  Joshua Colp <jcolp@digium.com>
 
-2016-10-09 21:53 +0000 [fa2885b3ff]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+	* func_cdr: Allow empty value for CDR dialplan function.
 
-	* cel_odbc: Fix memory leak on module unload
+	  A regression was introduced in 12 where passing an empty value
+	  to the CDR dialplan function was not longer allowed. This
+	  change returns to the behavior of 11 where it is permitted.
 
-	  Change-Id: Ic7a1236eba2408090fdabb5f717b5fa455ead715
+	  ASTERISK-26173
 
-2016-10-03 11:30 +0000 [e6b0053d75]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I3f148203b54ec088007e29e30005a5de122e51c5
 
-	* bundled_pjproject:  Add tests for programs used by the Makefile, et al.
+2017-05-04 16:04 +0000 [0001834157]  George Joseph <gjoseph@digium.com>
 
-	  Added tests for bzip2, tar, patch, sed and nm to configure.ac.
+	* app_confbridge:  Fix reference to cfg in menu_template_handler
 
-	  Set DOWNLOAD_TO_STDOUT to a working command line regardless of
-	  whether the download program is wget, curl or fetch.
+	  menu_template_handler wasn't properly accounting for the fact that
+	  it might be called both during a load/reload (which isn't really
+	  valid but not prevented) and by a dialplan function.  In both cases
+	  it was attempting to use the "pending" config which wasn't valid in
+	  the latter case.  aco_process_config is also partly to blame because
+	  it wasn't properly cleaning "pending" up when a reload was done and
+	  no changes were made.  Both of these contributed to a crash if
+	  CONFBRIDGE(menu,template) was called in a dialplan after a reload.
 
-	  Added a 'configure.m4' file to the third-party directory which takes
-	  care of calling any third-party project setup.  Had to move some
-	  pjproject_bundled stuff up in configure.ac so it was called before
-	  the third-party configure macro.
+	  * aco_process_config now sets info->internal->pending to NULL
+	    after it unrefs it although this isn't strictly necessary in the
+	    context of this fix.
+	  * menu_template_handler now uses the "current" config and silently
+	    ignores any attempt to be called as a result of someone uses the
+	    "template" parameter in the conf file.
 
-	  The pjproject tarball is now downloaded to the externals_cache_dir if
-	  it was specified on the ./configure command line
+	  Luckily there's no other place in the codebase where
+	  aco_pending_config is used outside of aco_process_config.
 
-	  Removed regeneration of the pjproject aconfigure file.  It was only
-	  needed for an old patch that no longer applies.
+	  ASTERISK-25506 #close
+	  Reported-by: Frederic LE FOLL
 
-	  Converted the tests for symbols to explicit tests since we know that
-	  they're now available in the bundled version.  Saves a little time
-	  during configure.
+	  Change-Id: Ib349a17d3d088f092480b19addd7122fcaac21a7
 
-	  ASTERISK-26416 #close
-	  Reported-by: Corey Farrell
+2017-04-30 16:40 +0000 [c90d81ef51]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Id1d94251c0155f8dd41b7de7067f35cfbaafbb9b
+	* bridge: Fix returning to dialplan when executing Bridge() from AMI.
 
-2016-10-05 14:53 +0000 [0dc0356e39]  gtjoseph <gjoseph@digium.com>
+	  When using the Bridge AMI action on the same channel multiple times
+	  it was possible for the channel to return to the wrong location in
+	  the dialplan if the other party hung up. This happened because the
+	  priority of the channel was not preserved across each action
+	  invocation and it would fail to move on to the next priority in
+	  other cases.
 
-	* pjproject_bundled:  Add MALLOC_DEBUG capability
+	  This change makes it so that the priority of a channel is preserved
+	  when taking control of it from another thread and it is incremented
+	  as appropriate such that the priority reflects where the channel
+	  should next be executed in the dialplan, not where it may or may not
+	  currently be.
 
-	  pjproject_bundled will now use the asterisk memory debugging APIs
-	  if MALLOC_DEBUG is turned on in menuselect.
+	  The Bridge AMI action was also changed to ensure that it too
+	  starts the channels at the next location in the dialplan.
 
-	  Because this required stubs for the executable programs and the python
-	  bindings, some Makefile reorganization was needed to properly handle
-	  the dependencies.  As a result, the makefile now individually makes
-	  each of the pjproject libraries separately instead of making them all
-	  in 1 shot.  The only visible change is that there are separate status
-	  lines printed for each library instead oif 1 for all libs.  Also, the
-	  making of the pjproject dependency files was eliminated.  They're not
-	  needed for building unless you're actively modifying pjproject source
-	  files and it makes the build process faster.  Finally, any issues with
-	  parallel builds should be resolved again making the build faster.
+	  ASTERISK-24529
 
-	  Change-Id: Icc5e3d658fbfb00e0a46b44c66dcc2522d5171b0
+	  Change-Id: I52406669cf64208aef7252a65b63ade31fbf7a5a
 
-2016-10-07 17:32 +0000 [dd873bcada]  Corey Farrell <git@cfware.com>
+2017-04-25 11:49 +0000 [7b0e3b92fd]  Kevin Harwell <kharwell@digium.com>
 
-	* astobj2: Add backtrace to log_bad_ao2.
+	* bridge_simple: Added support for streams
 
-	  * Compile __ast_assert_failed unconditionally.
-	  * Use __ast_assert_failed to log messages from log_bad_ao2
-	  * Remove calls to ast_assert(0) that happen after log_bad_ao2 was run.
+	  This patch is the first cut at adding stream support to the bridging framework.
+	  Changes were made to the framework that allows mapping of stream topologies to
+	  a bridge's supported media types.
 
-	  Change-Id: I48f1af44b2718ad74a421ff75cb6397b924a9751
+	  The first channel to enter a bridge initially defines the media types for a
+	  bridge (i.e. a one to one mapping is created between the bridge and the first
+	  channel). Subsequently added channels merge their media types into the bridge's
+	  adding to it when necessary. This allows channels with different sized
+	  topologies to map correctly to each other according to media type. The bridge
+	  drops any frame that does not have a matching index into a given write stream.
 
-2016-10-04 16:59 +0000 [86550f9c17]  gtjoseph <gjoseph@digium.com>
+	  For now though, bridge_simple will align its two channels according to size or
+	  first to join. Once both channels join the bridge the one with the most streams
+	  will indicate to the other channel to update its streams to be the same as that
+	  of the other. If both channels have the same number of streams then the first
+	  channel to join is chosen as the stream base.
 
-	* alembic:  Allow cdr, config and voicemail to exist in the same schema
+	  A topology change source was also added to a channel when a stream toplogy
+	  change request is made. This allows subsystems to know whether or not they
+	  initiated a change request. Thus avoiding potential recursive situations.
 
-	  cdr, config and voicemail are all separate alembic trees.  Because
-	  alembic's default is to use a table named 'alembic_version' to store
-	  the current tree revision, the 3 trees can't exist in the same schema
-	  without stepping on each other.
+	  ASTERISK-26966 #close
 
-	  Now each tree uses 'alembic_version_<tree_name>' as the version table.
-	  Each tree's env.py script now first checks for 'alembic_version'.  If
-	  it finds it AND its revision is in the tree's history, the script
-	  renames it to 'alembic_version_<tree_name>'.  Regardless, the script
-	  then continues with the migration using 'alembic_version_<tree_name>'
-	  and creates that table if it's not found.  The result is that if an
-	  existing 'alembic_version' table was found but it didn't belong to this
-	  tree, it's left alone and 'alembic_version_<tree_name>' is used or
-	  created.
+	  Change-Id: I1eb5987921dd80c3cdcf52accc136393ca2d4163
 
-	  WARNING:  If multiple trees are using the same schema, they MUST NOT
-	  CRU or D any objects with names that might exist in the other trees.
-	  An example would be 'yesno_values' type.  If two trees perform
-	  operations on it, one tree could pull it out from under the other.
-	  Thankfully we currently don't share any names among cdr, config and
-	  voicemail.
+2017-05-01 13:04 +0000 [008e25def9]  Kevin Harwell <kharwell@digium.com>
 
-	  NOTE:  Since the env.py scripts in each tree were identical, a common
-	  env.py has been placed in the ast-db-manage directory and a symlink
-	  to it has been placed in each tree directory.
+	* res_rtp_asterisk: Clearing the remote RTCP address causes RTCP failures
 
-	  ASTERISK-24311 #close
-	  Reported-by: Dafi Ni
+	  When a call gets put on hold RTP is temporarily stopped and Asterisk was
+	  setting the remote RTCP address to NULL. Then when RTCP data was received
+	  from the remote endpoint, Asterisk would be missing this information when
+	  publishing the rtcp_message stasis event. Consequently, message subscribers
+	  (in this case res_hep_rtcp) trying to parse the "from" field output the
+	  following error:
 
-	  Change-Id: I4d593f000350deb5d21a14fa1e9bc3896844d898
+	  "ast_sockaddr_split_hostport: Port missing in (null)"
 
-2016-10-05 04:25 +0000 [f166681c12]  Alexander Traud <pabstraud@compuserve.com>
+	  This patch makes it so the remote RTCP address is no longer set to NULL when
+	  stopping RTP. There was only one place that appeared to check if the remote
+	  RTCP address was NULL as a way to tell if RTCP was running. This patch added
+	  an additional check on the RTCP schedid for that case to make sure RTCP was
+	  truly not running.
 
-	* chan_sip: Honor support of Symmetric Response (rport) for SIP requests.
+	  ASTERISK-26860 #close
 
-	  In the SIP channel driver chan_sip, the default is "auto_force_rport". When no
-	  NAT was detected, for example in case of IPv6, Asterisk uses the IP address
-	  from the headers within the SIP-REGISTER for subsequent SIP signaling. When
-	  the remote party specifies support for Symmetric Response (RFC 3581) via the
-	  parameter "rport", Asterisk should not extract the port from the SIP headers
-	  but reuse the port of the transport. This did not happen because of a typo.
+	  Change-Id: I6be200fb20db647e48b5138ea4b81dfa7962974b
 
-	  ASTERISK-26438 #close
+2017-05-02 11:34 +0000 [675e058e77]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: If6e7891848aaf96666dee5305695f7c6667cd5a6
+	* cleanup: Change severity of fread short-read warning
 
-2016-10-04 20:46 +0000 [430f6e5388]  Michael Walton <mike@farsouthnet.com>
+	  Many sound files don't have a full frame's worth of data at EOF, so the
+	  warning messages were a bit too noisy. So we demote them to debug
+	  messages.
 
-	* audiohooks: Remove redundant codec translations when using audiohooks
+	  Change-Id: I6b617467d687658adca39170a81797a11cc766f6
 
-	  The main frame read and write handlers in main/channel.c don't use the
-	  optimum placement in the processing flow for calling audiohooks
-	  callbacks, as far as codec translation is concerned. This change places
-	  the audiohooks callback code:
-	   * After the channel read translation if the frame is not linear before
-	  the translation, thereby increasing the chance that the frame is linear
-	  as required by audiohooks
-	   * Before the channel write translation if the frame is linear at this
-	  point
-	  This prevents the audiohooks code from instantiating additional
-	  translation paths to/from linear where a linear frame format is already
-	  available, saving valuable CPU cycles
+2017-04-26 16:22 +0000 [cd272da7a8]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26419
+	* SDP: Replace SDP telephone_event option with dtmf option
 
-	  Change-Id: I6edd5771f0740e758e7eb42558b953f046c01f8f
+	  The telephone_event option was used as a flag and a bit mapped value in
+	  different places when it is a boolean.  It is also inadequate to configure
+	  the DTMF operation of the RTP instance created for the stream.
 
-2016-09-29 14:02 +0000 [2449d2877c]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: Ib1addeaf0ce86f07039f2f979cab29405dc5239b
 
-	* Remove "format_ogg_opus: New format"
+2017-04-29 16:11 +0000 [52e4f02b1a]  Richard Mudgett <rmudgett@digium.com>
 
-	  This reverts commit 40aa28131bc30b4516da2b20eb1a1e043920169c.
+	* res_pjsip_t38.c: Fix deadlock in T.38 framehook.
 
-	  ASTERISK-26426 #close
+	  A deadlock can happen between a channel lock and a pjsip session media
+	  container lock.  One thread is processing a reINVITE's SDP and walking
+	  through the session's media container when it waits for the channel lock
+	  to put the determined format capabilities onto the channel.  The other
+	  thread is writing a frame to the channel and processing the T.38 frame
+	  hook.  The T.38 frame hook then waits for the pjsip session's media
+	  container lock.  The two threads are now deadlocked.
 
-	  Change-Id: I81e55c3c512f1dd6f49896f0c6b97a07d74fd8f5
+	  * Made the T.38 frame hook release the channel lock before searching the
+	  session's media container.  This fix has been done to several other
+	  frame hooks to fix deadlocks.
 
-2016-09-27 16:10 +0000 [f0a2e628d6]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-26974 #close
 
-	* download_externals: Fix issue with re-install
+	  Change-Id: Ie984a76ce00bef6ec9aa239010e51e8dd74c8186
 
-	  Needed to ignore an xmlstarlet return code for optional element.
+2017-04-28 10:56 +0000 [8170793be6]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I6a96f709b4b38c9a3f3dda4e8b07903787e16873
-	  Reported-by: Dan Jenkins
+	* res_pjsip_outbound_authenticator_digest: Add context to log messages
 
-2016-09-22 09:49 +0000 [5258c067ae]  gtjoseph <gjoseph@digium.com>
+	  There was no context info in this module's log messages so it was
+	  impossible to toubleshoot.
 
-	* codec_opus: Add download ability to menuselect
+	  Added endpoint or host to all messages and added the realms in the
+	  challenge for the "No auth credentials for any realm" message.
 
-	  Updated codecs/codecs.xml to add codec_opus to the external
-	  download list.
+	  Change-Id: Ifeed2786f35fbea7d141237ae15625e472acff9b
 
-	  ASTERISK-26409
+2017-04-27 16:46 +0000 [48566b8c66]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ia07b36539f30e852125fb2b94147dc9774df31a4
+	* res_sdp_translator_pjmedia.c: Add TODO notes.
 
-2016-07-23 14:50 +0000 [a5af8709c8]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: If27ca61f79accc882c3376d2e876d2b44aa1347b
 
-	* codec_opus: Replace res_format_attr_opus with the one from codec_opus
+2017-04-24 18:13 +0000 [ede90e4aa5]  Richard Mudgett <rmudgett@digium.com>
 
-	  Preparation
+	* SDP: Make SDP translation to/from internal representation more const.
 
-	  ASTERISK-26409
+	  Change-Id: I473a174b869728604b37c60853896b0c458bc504
 
-	  Change-Id: I9f20e7cce00c32464d9a180e81283d49d199d0a3
-	  (cherry picked from commit 59f7662a93bf9c07204fb50e1020a0f5bfbbd5c9)
+2017-04-20 19:25 +0000 [5c1851cbc0]  Richard Mudgett <rmudgett@digium.com>
 
-2016-07-23 15:56 +0000 [44c0c51cf1]  gtjoseph <gjoseph@digium.com>
+	* stream: Make ast_stream_topology_create_from_format_cap() allow NULL cap.
 
-	* format_ogg_opus: New format
+	  Change-Id: Ie29760c49c25d7022ba2124698283181a0dd5d08
 
-	  Add Ogg/Opus playback support.
+2017-04-24 16:55 +0000 [d71c6e3bfd]  Richard Mudgett <rmudgett@digium.com>
 
-	  This uses libopusfile in order to be able to read .opus files and play
-	  them back.
+	* SDP: Make ast_sdp_state_set_remote_sdp() return error.
 
-	  Writing/recording support is not present at this time.
+	  Change-Id: I7707c9d872c476d897ff459008652b35142a35e1
 
-	  ASTERISK-26409
+2017-04-14 11:52 +0000 [176123e76c]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I8815d23345108d8ca7c0bd640f6a1ce6b4f56955
+	* SDP: Misc cleanups (Mostly memory leaks)
 
-2016-09-24 19:05 +0000 [0ab443007b]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I74431b385da333f2c5f5a6d7c55e70b69a4f05d2
 
-	* build_tools:  Add ability to download variants to download_externals
+2017-04-27 18:15 +0000 [bad091b317]  Richard Mudgett <rmudgett@digium.com>
 
-	  Some external packages have multiple variants that apply to different
-	  builds of asterisk.  The DPMA for instance has a "bundled" variant that
-	  needs to be downloaded if asterisk was configured with
-	  --with-pjproject-bundled.
+	* chan_vpb.cc: Fix compile error.
 
-	  There are 2 ways to specify variants:
+	  Change-Id: I6d9edd34d8b2474222c86f44e379ead61e57a54f
 
-	  If you need the user to make the decision about which variant to
-	  download, simply create multiple menuselect "member" entries like so...
+2017-04-26 16:14 +0000 [d6535c0080]  Mark Michelson <mmichelson@digium.com>
 
-	  <member name="res_digium_phone" displayname="..snipped..">
-	    <support_level>external</support_level>
-	    <depend>xmlstarlet</depend>
-	    <depend>bash</depend>
-	    <defaultenabled>no</defaultenabled>
-	  </member>
+	* SDP API: Add SSRC-level attributes
 
-	  <member name="res_digium_phone-bundled" displayname="..snipped..">
-	    <support_level>external</support_level>
-	    <depend>xmlstarlet</depend>
-	    <depend>bash</depend>
-	    <defaultenabled>no</defaultenabled>
-	  </member>
+	  RFC 5576 defines how SSRC-level attributes may be added to SDP media
+	  descriptions. In general, this is useful for grouping related SSRCes,
+	  indicating SSRC-level format attributes, and resolving collisions in RTP
+	  SSRC values. These attributes are used widely by browsers during WebRTC
+	  communications, including attributes defined by documents outside of RFC
+	  5576.
 
-	  Note that the second entry has "-<variant>" appended to the name.
-	  You can then use the existing menuselect facilities to restrict which
-	  members to enable or disable.  Youy probably don't want the user to
-	  enable multiple at the same time.
+	  This commit introduces the addition of SSRC-level attributes into SDPs
+	  generated by Asterisk. Since Asterisk does not tend to use multiple
+	  SSRCs on a media stream, the initial support is minimal. Asterisk
+	  includes an SSRC-level CNAME attribute if configured to do so. This at
+	  least gives browsers (and possibly others) the ability to resolve SSRC
+	  collisions at offer-answer time.
 
-	  If you want to hide the details of the variants, the better way to
-	  do it is to create 1 member with "variant" elements.
+	  In order to facilitate this, the RTP engine API has been enhanced to be
+	  able to retrieve the SSRC and CNAME on a given RTP instance.
 
-	  <member name="res_digium_phone" displayname="..snipped..">
-	    <support_level>external</support_level>
-	    <depend>xmlstarlet</depend>
-	    <depend>bash</depend>
-	    <defaultenabled>no</defaultenabled>
-	    <member_data>
-	      <downloader>
-	        <variants>
-	          <variant tag="bundled"
-	            condition='[[ "$PJPROJECT_BUNDLED" = "yes" ]]'/>
-	        </variants>
-	      </downloader>
-	    </member_data>
-	  </member>
+	  res_rtp_asterisk currently does not provide meaningful CNAME values in
+	  its RTCP SDES items, and therefore it currently will always return an
+	  empty string as the CNAME value. A task in the near future will result
+	  in res_rtp_asterisk generating more meaningful CNAMEs.
 
-	  The condition must be a bash expression suitable for use with an "if"
-	  statement.  Any environment variable can be used plus those available
-	  in makeopts.
+	  Change-Id: I29e7f23e7db77524f82a3b6e8531b1195ff57789
 
-	  In this case, if asterisk was configured with --with-pjproject-bundled
-	  the bundled variant will be automatically downloaded.  Otherwise the
-	  normal version will be downloaded.
+2017-04-27 08:02 +0000 [d6b2a58736]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: I4de23e06d4492b0a65e105c8369966547d0faa3e
+	* res_pjsip_session:  Add cleanup to ast_sip_session_terminate
 
-2016-09-22 01:40 +0000 [a0a17a8c6f]  Aaron An <anjb@ti-net.com.cn>
+	  If you use ast_request to create a PJSIP channel but then hang it
+	  up without causing a transaction to be sent, the session will
+	  never be destroyed.  This is due ot the fact that it's pjproject
+	  that triggers the session cleanup when the transaction ends.
+	  app_chanisavail was doing this to get more granular channel state
+	  and it's also possible for this to happen via ARI.
 
-	* channels/chan_pjsip: fix HANGUPCAUSE function bug.
+	  * ast_sip_session_terminate was modified to explicitly call the
+	    cleanup tasks and unreference session if the invite state is NULL
+	    AND invite_tsx is NULL (meaning we never sent a transaction).
 
-	  HANGUPCAUSE not return 'SIP 200 Ok' when dialed channel answered.
-	  This patch change the call order of ast_queue_control_data
-	  and ast_queue_control in chan_pjsip_incoming_response.
+	  * chan_pjsip/hangup was modified to bump session before it calls
+	    ast_sip_session_terminate to insure that session stays valid
+	    while it does its own cleanup.
 
-	  ASTERISK-26396 #close
-	  Reported by: AaronAn
-	  Tested by: AaronAn
+	  * Added test events to session_destructor for a future testsuite
+	    test.
 
-	  Change-Id: Ide2d31723d8d425961e985de7de625694580be61
+	  ASTERISK-26908 #close
+	  Reported-by: Richard Mudgett
 
-2016-09-23 09:54 +0000 [0502675e5c]  Alessandro Crespi
+	  Change-Id: I52daf6f757184e5544c261f64f6fe9602c4680a9
 
-	* chan_sip: Resolve externhost not to IPv6; instead go for IPv4.
+2017-04-24 10:59 +0000 [2b22c3c84b]  Joshua Colp <jcolp@digium.com>
 
-	  For the channel driver chan_sip, you specify externhost=example.com in sip.conf
-	  when your Asterisk is behind a NAT and your IP address is assigned dynamically.
-	  Or stated differently: You do not have a static IP address to use "externaddr"
-	  directly. This NAT support is quite handy but just about IPv4. Previously,
-	  Asterisk resolved "externhost" to any IP version. When the first DNS answer
-	  resolved to an IPv6, Asterisk sent an IPv6 in SIP/SDP for origin (o=) and
-	  connection (c=). This happened in outgoing SIP-REGISTER and while answering
-	  SIP-INVITE. If the remote peer is IPv4-only, it might not handle o=/c= with an
-	  IPv6. This change makes sure, no IPv6 is resolved anymore for "externhost".
+	* channel: Add ability to request an outgoing channel with stream topology.
 
-	  ASTERISK-18232 #close
-	  Reported by: Jacek Kowalski
-	  Tested by: Alexander Traud
-	  patches:
-	   changes.patch submitted by Alessandro Crespi
+	  This change extends the ast_request functionality by adding another
+	  function and callback to create an outgoing channel with a requested
+	  stream topology. Fallback is provided by either converting the
+	  requested stream topology into a format capabilities structure if
+	  the channel driver does not support streams or by converting the
+	  requested format capabilities into a stream topology if the channel
+	  driver does support streams.
 
-	  Change-Id: If68eedbeff65bd1c1d8a9ed921c02ba464b32dac
+	  The Dial application has also been updated to request an outgoing
+	  channel with the stream topology of the calling channel.
 
-2016-09-20 09:42 +0000 [0056bcaebd]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-26959
 
-	* chan_sip:  Address runaway when realtime peers subscribe to mailboxes
+	  Change-Id: Ifa9037a672ac21d42dd7125aa09816dc879a70e6
 
-	  Users upgrading from asterisk 13.5 to a later version and who use
-	  realtime with peers that have mailboxes were experiencing runaway
-	  situations that manifested as a continuous stream of taskprocessor
-	  congestion errors, memory leaks and an unresponsive chan_sip.
+2017-04-26 14:20 +0000 [c6b757fa05]  Kevin Harwell <kharwell@digium.com>
 
-	  A related issue was that setting rtcachefriends=no NEVER worked in
-	  asterisk 13 (since the move to stasis).  In 13.5 and earlier, when a
-	  peer tried to register, all of the stasis threads would block and
-	  chan_sip would again become unresponsive.  After 13.5, the runaway
-	  would happen.
+	* res_pjsip/res_pjsip_callerid: NULL check on caller id name string
 
-	  There were a number of causes...
-	  * mwi_event_cb was (indirectly) calling build_peer even though calls to
-	    mwi_event_cb are often caused by build_peer.
-	  * In an effort to prevent chan_sip from being unloaded while messages
-	    were still in flight, destroy_mailboxes was calling
-	    stasis_unsubscribe_and_join but in some cases waited forever for the
-	    final message.
-	  * add_peer_mailboxes wasn't properly marking the existing mailboxes
-	    on a peer as "keep" so build_peer would always delete them all.
-	  * add_peer_mwi_subs was unsubscribing existing mailbox subscriptions
-	    then just creating them again.
+	  It's possible for a name in a party id structure to be marked as valid, but the
+	  name string itself be NULL (for instance this is possible to do by using the
+	  dialplan CALLERID function). There were a couple of places where the name was
+	  validated, but the string itself was not checked before passing it to functions
+	  like 'strlen'. This of course caused a crashed.
 
-	  All of this was causing a flood of subscribes and unsubscribes on
-	  multiple threads all for the same peer and mailbox.
+	  This patch adds in a NULL check before attempting to pass it into a function
+	  that is not NULL tolerant.
 
-	  Fixes...
-	  * add_peer_mailboxes now marks mailboxes correctly and build_peer only
-	    deletes the ones that really are no longer needed by the peer.
-	  * add_peer_mwi_subs now only adds subscriptions marked as "new" instead
-	    of unsubscribing and resubscribing everything.  It also adds the peer
-	    object's address to the mailbox instead of its name to the subscription
-	    userdata so mwi_event_cb doesn't have to call build_peer.
+	  ASTERISK-25823 #close
 
-	  With these changes, with rtcachefriends=yes (the most common setting),
-	  there are no leaks, locks, loops or crashes at shutdown.
+	  Change-Id: Iaa6ffe9d92f598fe9e3c8ae373fadbe3dfbf1d4a
 
-	  rtcachefriends=no still causes leaks but at least it doesn't lock, loop
-	  or crash.  Since making rtcachefriends=no work wasnt in scope for this
-	  issue, further work will have to be deferred to a separate patch.
+2017-04-25 11:43 +0000 [cf3429b934]  Kevin Harwell <kharwell@digium.com>
 
-	  Side fixes...
-	   * The ast_lock_track structure had a member named "thread" which gdb
-	     doesn't like since it conflicts with it's "thread" command.  That
-	     member was renamed to "thread_id".
+	* vector: defaults and indexes
 
-	  ASTERISK-25468 #close
+	  Added an pre-defined integer vector declaration. This makes integer vectors
+	  easier to declare and pass around. Also, added the ability to default a vector
+	  up to a given size with a default value. Lastly, added functionality that
+	  returns the "nth" index of a matching value.
 
-	  Change-Id: I07519ef7f092629e1e844f855abd279d6475cdd0
+	  Also, updated a unit test to test these changes.
 
-2016-09-21 15:03 +0000 [323aff3a09]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Iaf4b51b2540eda57cb43f67aa59cf1d96cdbcaa5
 
-	* core: Ensure presencestate subtype and message are NULL.
+2017-04-26 05:38 +0000 [985a5fd7aa]  Joshua Colp <jcolp@digium.com>
 
-	  When retrieving presence state information there is no
-	  guarantee that the subtype and message passed in are
-	  set to NULL. This change ensures they are.
+	* frame: Better handle interpolated frames.
 
-	  ASTERISK-26397 #close
+	  Interpolated frames are frames which contain a number of
+	  samples but have no actual data. Audiohooks did not
+	  handle this case when translating an incoming frame into
+	  signed linear. It assumed that a frame would always contain
+	  media when it may not. If this occurs audiohooks will now
+	  immediately return and not act on the frame.
 
-	  Change-Id: I61f8187972d5d8bbd7d6b7f4daa4f4f7e8237b23
+	  As well for users of ast_trans_frameout the function has
+	  been changed to be a bit more sane and ensure that the data
+	  pointer on a frame is set to NULL if no data is actually
+	  on the frame. This allows the various spots in Asterisk that
+	  check for an interpolated frame based on the presence of a
+	  data pointer to work as expected.
 
-2016-09-21 10:48 +0000 [10c180760c]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-26926
 
-	* res_odbc: Make pooling option deprecation notice more useful.
+	  Change-Id: I7fa22f631fa28d540722ed789ce28e84c7f8662b
 
-	  This changes the notice for the deprecation of the old
-	  pooling options to point to the new option for doing
-	  pooling. This gives a clearer direction as to what to
-	  look into.
+2017-04-26 09:22 +0000 [99dea9ba84]  Yasin CANER <yasin.caner@netgsm.com.tr>
 
-	  ASTERISK-26389 #close
+	* res_pjsip_session : fixed wrong From Header number On Re-invite
 
-	  Change-Id: I2ca9cdfdcd75aec170a7db9d5ff69a4cd25b7c10
+	  ASTERISK-26964 #close
 
-2016-09-12 07:37 +0000 [42cc267016]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Change-Id: I55a9caa7dc90e6c4c219cb09b5c2ec08af84a302
 
-	* cdr_mysql: fix UTC support
+2017-04-26 08:45 +0000 [858ed60446]  George Joseph <gjoseph@digium.com>
 
-	  * Make 'cdrzone=UTC' work properly.
-	  * Fix the documentation of cdr_mysql.conf: it's cdrzone and not timezone
+	* pjproject_bundled:  Add --disable-libwebrtc to configure
 
-	  ASTERISK-26359 #close
+	  Without the disable, pjproject tries to build it's internal
+	  webrtc implementation which requires sse2.  This fails on
+	  platforms without sse2.
 
-	  Change-Id: I2a6f67b71bbbe77cac31a34d0bbfb1d67c933778
+	  ASTERISK-26930 #close
+	  Reported-by: abelbeck
 
-2016-09-21 08:46 +0000 [f16ab19292]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I07231f9160c35cfa42b194d3aad4e7d51fd9a410
 
-	* odbc: Remove options that are no longer applicable.
+2017-04-26 07:58 +0000 [585f9405b1]  Thierry Magnien <thierry.magnien@gmail.com>
 
-	  The pooling, shared_connection, limit, and idlecheck options
-	  are no longer used in res_odbc.
+	* channels/chan_sip.c: use binding IP address for outgoing TCP SIP connections
 
-	  ASTERISK-26389
+	  For outgoing TCP connections, Asterisk uses the first IP address of the
+	  interface instead of the IP address we asked him to bind to.
 
-	  Change-Id: I2fde7b467d01f9d1c82cc0a339bb4f7e1dd6bbe6
+	  ASTERISK-26922 #close
+	  Reported-by: Ksenia
 
-2016-09-20 15:17 +0000 [c9ce299b64]  Corey Farrell <git@cfware.com>
+	  Change-Id: I43c71ca89211dbf1838e5bcdb9be8d06d98e54eb
 
-	* core: Fix LOW_MEMORY missing symbol ast_pbx_uuid_get.
+2017-04-21 12:04 +0000 [f5b67871df]  Sean Bright <sean.bright@gmail.com>
 
-	  Move the function outside the conditional block that excludes
-	  LOW_MEMORY.
+	* cleanup: Fix fread() and fwrite() error handling
 
-	  ASTERISK-26273 #close
+	  Cleaned up some of the incorrect uses of fread() and fwrite(), mostly in
+	  the format modules. Neither of these functions will ever return a value
+	  less than 0, which we were checking for in some cases.
 
-	  Change-Id: Ic290fa128222c410c3531107e30efacabc8493b4
+	  I've introduced a fair amount of duplication in the format modules, but
+	  I plan to change how format modules work internally in a subsequent
+	  patch set, so this is simply a stop-gap.
 
-2016-09-20 10:05 +0000 [610eb4c189]  Corey Farrell <git@cfware.com>
+	  Change-Id: I8ca1cd47c20b2c0b72088bd13b9046f6977aa872
 
-	* logger: Fix default console settings.
+2017-04-25 07:52 +0000 [199d4776c0]  Joshua Colp <jcolp@digium.com>
 
-	  When logger.conf is missing or invalid we should be printing notices,
-	  warnings and errors to the console.  The logmask was incorrectly
-	  calculated.
+	* alembic: Add table for 'resource_list' PJSIP RLS type.
 
-	  Change-Id: Ibaa9465a8682854bc1a5e9ba07079bea1bfb6bb3
+	  This change adds an Alembic migration which adds a
+	  ps_resource_list table that can contain resource_list
+	  RLS configuration objects.
 
-2016-06-27 14:26 +0000 [36092ee3a0]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  ASTERISK-26929
 
-	* sd_notify (systemd status notifications) support
+	  Change-Id: I7c888fafc67b3e87012de974f71ca7a5b8b1ec05
 
-	  sd_notify() is used to notify systemd of changes to the status of the
-	  process. This allows the systemd daemon to know when the process
-	  finished loading (and thus only start another program after Asterisk has
-	  finished loading).
+2017-04-14 05:21 +0000 [19a79ae12c]  Joshua Colp <jcolp@digium.com>
 
-	  To use this, use a systemd unit with 'Type=notify' for Asterisk.
+	* sdp: Add support for T.38
 
-	  This commit also adds the function ast_sd_notify(), a wrapper around
-	  sd_notify that does nothing if not built with systemd support.
+	  This change adds a T.38 format which can be used in a stream
+	  topology to specify that a UDPTL stream needs to be created.
+	  The SDP API has been changed to understand T.38 and create
+	  the UDPTL session, add the attributes, and parse the attributes.
 
-	  Also adds support for libsystemd detection in the configure script.
+	  This change does not change the boundary of the T.38 state
+	  machine. It is still up to the channel driver to implement and
+	  act on it (such as queueing control frames or reacting to them).
 
-	  Change-Id: Ied6a59dafd5ef331c5c7ae8f3ccd2dfc94be7811
-	  (cherry picked from commit 07b95f7c65b7c083724f1af2b26f93cc22cad58c)
+	  ASTERISK-26949
 
-2016-09-19 14:21 +0000 [9372d32100]  Walter Doekes <walter+github@wjd.nu>
+	  Change-Id: If28956762ccb8ead562ac6c03d162d3d6014f2c7
 
-	* asterisk.c: Non-root users also get the astcanary after core restart.
+2017-03-21 15:44 +0000 [32b3e36c68]  Mark Michelson <mmichelson@digium.com>
 
-	  Without this change, a 'core restart' would kill the astcanary forever
-	  if you're not running as root. Both with and without this patch, the
-	  scheduling priority was still SCHED_RR after restart.
+	* SDP: Ensure SDPs "merge" properly.
 
-	  Additionally, the astcanary is now spawned if you start with high
-	  priority and Asterisk doesn't get a chance to lower it. For example
-	  through: `chrt -r 10 sudo -u asterisk asterisk -c`
+	  The gist of this work ensures that when a remote SDP is received, it is
+	  merged properly with the local capabilities. The remote SDP is converted
+	  into a stream topology. That topology is then merged with the current
+	  local topology on the SDP state. That new merged topology is then used
+	  to create an SDP. Finally, adjustments are made to RTP instances based
+	  on knowledge gained from the remote SDP.
 
-	  Also reap killed astcanary processes on core restart.
+	  There are also a battery of tests in this commit that ensure that some
+	  basic SDP merges work as expected.
 
-	  ASTERISK-26352 #close
+	  While this may not sound like a big change, it has the property that it
+	  caused lots of ancillary changes.
 
-	  Change-Id: Iacb49f26491a0717084ad46ed96b0bea5f627a55
+	  * The remote SDP is no longer stored on the SDP state. Biggest reason:
+	    there's no need for it. The remote SDP is used at the time it is being
+	    set and nowhere else.
 
-2016-09-19 09:40 +0000 [e96448e991]  Walter Doekes <walter+github@wjd.nu>
+	  * Some new SDP APIs were added in order to find attributes and convert
+	    generic SDP attributes into rtpmap structures.
 
-	* asterisk.c: When astcanary dies on linux, reset priority on all threads.
+	  * Writing tests made me realize that retrieving a value from an SDP
+	    options structure, the SDP options needs to be made const.
 
-	  Previously only the canary checking thread itself had its priority set
-	  to SCHED_OTHER. Now all threads are traversed and adjusted.
+	  * The SDP state machine was essentially gutted by a previous commit.
+	    Initially, I attempted to reinstate it, but I found that as it had
+	    been defined, it was not all that useful. What was more useful was
+	    knowing the role we play in SDP negotiation, so the SDP state machine
+	    has been transformed into an indicator of role.
 
-	  ASTERISK-19867 #close
-	  Reported by: Xavier Hienne
+	  * Rather than storing separate local and joint stream state
+	    capabilities, it makes more sense to keep track of current stream
+	    state and update it as things change.
 
-	  Change-Id: Ie0dd02a3ec42f66a78303e9c1aac28f7ed9aae39
+	  Change-Id: I5938c2be3c6f0a003aa88a39a59e0880f8b2df3d
 
-2016-09-09 06:35 +0000 [01884a7af6]  Timo Teräs <timo.teras@iki.fi>
+2017-04-24 13:16 +0000 [0611f2ca17]  Sean Bright <sean.bright@gmail.com>
 
-	* Fix showing of swap details when sysinfo() is available
+	* res_hep: Add additional config initialization and validation
 
-	  If sysinfo() is available, but not sysctl() or swapctl() the
-	  printing code for swap buffer sizes is incorrectly omitted.
-	  The above condition happens with musl c-library.
+	  * Initialize hepv3_runtime_data.sockfd to -1 so that our ao2 destructor
+	    does not close fd 0
 
-	  Fix #if rule to consider defined(HAVE_SYSINFO). And also
-	  remove the redundant || defined(HAVE_SYSCTL) which was
-	  incorrectly there to start with. Now swap information is
-	  displayed only if an actual libc function to get it is
-	  available.
+	  * Add logging output when the required option - capture_address - is not
+	    specified.
 
-	  This also fixes warnings previously seen with musl libc:
+	  * Remove a no longer relevant #define and correct related documentation
 
-	     [CC] asterisk.c -> asterisk.o
-	  asterisk.c: In function 'handle_show_sysinfo':
-	  asterisk.c:773:6: warning: variable 'totalswap' set but not used
-	   [-Wunused-but-set-variable]
-	    int totalswap = 0;
-	        ^~~~~~~~~
-	  asterisk.c:770:11: warning: variable 'freeswap' set but not used
-	   [-Wunused-but-set-variable]
-	    uint64_t freeswap = 0;
-	             ^~~~~~~~
+	  * Pass appropriate flags to aco_option_register so that capture_address
+	    cannot be the empty string.
 
-	  Change-Id: I1fb21dad8f27e416c60f138c6f2bff03fb626eca
+	  ASTERISK-26953 #close
 
-2016-09-12 18:00 +0000 [cdbad152c7]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ief08441bc6596d6f1718fa810e54a5048124f076
 
-	* res_config_odbc.c: Fix buffer size limitation creating invalid SQL.
+2017-04-17 19:06 +0000 [59203c51cc]  Sean Bright <sean.bright@gmail.com>
 
-	  Creating ODBC SQL queries resulted in queries too large to fit into the
-	  supplied buffer.  The resulting truncated buffer contained an invalid SQL
-	  query.
+	* core: Use eventfd for alert pipes on Linux when possible
 
-	  * Made SQL query generation code use a thread storage buffer that can
-	  increase in size as needed.
+	  The primary win of switching to eventfd when possible is that it only
+	  uses a single file descriptor while pipe() will use two. This means for
+	  each bridge channel we're reducing the number of required file
+	  descriptors by 1, and - if you're using timerfd - we also now have 1
+	  less file descriptor per Asterisk channel.
 
-	  * Fixed bad multi-line warning messages.
+	  The API is not ideal (passing int arrays), but this is the cleanest
+	  approach I could come up with to maintain API/ABI.
 
-	  ASTERISK-26263 #close
-	  Reported by: Jeppe Ryskov Larsen
+	  I've also removed what I believe to be an erroneous code block that
+	  checked the non-blocking flag on the pipe ends for each read. If the
+	  file descriptor is 'losing' its non-blocking mode, it is because of a
+	  bug somewhere else in our code.
 
-	  Change-Id: I23f3cdd43c2dac80bed3ded4dd77d18cb17f21ae
+	  In my testing I haven't seen any measurable difference in performance.
 
-2016-09-14 08:42 +0000 [449719be00]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: Iff0fb1573e7f7a187d5211ddc60aa8f3da3edb1d
 
-	* res_pjsip_multihomed: Change Contact port to listening port.
+2017-04-21 12:33 +0000 [f1d20c84a1]  Richard Mudgett <rmudgett@digium.com>
 
-	  The res_pjsip_multihomed module determines what interface and transport
-	  a request is going out on and updates the SIP message accordingly with
-	  the address information. This currently incorrectly updates the Contact
-	  header for connectionful protocols to the ephemeral connection port,
-	  instead of the bound address for the listening socket which can actually
-	  accept the connection back. If the remote side attempts to connect back on
-	  the epehemeral port it will fail.
+	* res_pjsip_session.c: Send 100 Trying out earlier to prevent retransmissions.
 
-	  This change makes it so the port is updated to the bound port on
-	  connectionful protocols and is maintained on UDP (as there can be
-	  multiple of those).
+	  If ICE is enabled and a STUN server does not respond then we will block
+	  until we give up on the STUN response.  This will take nine seconds.  In
+	  the mean time the peer that sent the INVITE will send retransmissions.
 
-	  ASTERISK-26374 #close
+	  * Restructure res_pjsip_session.c:new_invite() to send a 100 Trying out
+	  earlier to prevent these retransmissions.
 
-	  Change-Id: I50f8dab65b9f75117d73ba5f6bbcf6c9871854ab
+	  ASTERISK-26890
 
-2016-09-07 14:48 +0000 [4d64b176eb]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: Ie3fc611e53a0eff6586ad55e4aacad81cf6319a8
 
-	* pjproject_bundled:  Prevent SERVFAIL from marking name server bad
+2017-04-21 12:07 +0000 [835c209445]  Richard Mudgett <rmudgett@digium.com>
 
-	  A name server that returns "Server Failure" is indicating only that
-	  the server couldn't process that particular request.  We should NOT
-	  assume that the name server is incapable of serving other requests.
+	* res_pjsip_session.c: Restructure ast_sip_session_alloc()
 
-	  Here's the scenario we've been encountering...
+	  * Restructure ast_sip_session_alloc() to need less cleanup on off nominal
+	  error paths.
 
-	  * 2 local name servers configured in resolv.conf.
-	  * An OPTIONS request causes a request for A and AAAA records to go out
-	    to both nameservers.
-	  * The A responses both come back successfully resolved.
-	  * Because of an issue at some upstream nameserver, the AAAA responses
-	    for that particular query come back as "SERVFAIL" from both local
-	    name servers.
-	  * Both local servers are marked as bad and no further queries can be
-	    sent until the 60 second ttl expires.  Only previously cached results
-	    can be used.
-	  * In this case, 60 seconds is just enough time for another OPTIONS
-	    request to go out to the same host so the cycle repeats.
+	  * Made ast_sip_session_alloc() and ast_sip_session_create_outgoing() avoid
+	  unnecessary ref manipulation to return a session.  This is faster than
+	  calling a function.  That function may do logging of the ref changes with
+	  REF_DEBUG enabled.
 
-	  We could set the bad ttl really low but that also affects REFUSED and
-	  NOTAUTH which probably DO signal a real server issue.  Besides, even
-	  a really low bad ttl would be an issue on a pbx.
+	  Change-Id: I2a0affc4be51013d3f0485782c96b8fee3ddb00a
 
-	  Although we use our own resolver in 14 and master and don't have this
-	  issue there, Teluu has merged this patch upstream so it's appropriate
-	  to cherry-pick to 14 and master to keep pjproject consistent.
+2017-04-20 02:13 +0000 [b4b1943c5d]  Jean Aunis <jean.aunis@prescom.fr>
 
+	* chan_sip: Trigger reinvite if the SDP answer is included in the SIP ACK
 
-	  Change-Id: Ie03ba902288e274aff23f9b9bb2786e1e8be09e0
+	  Some equipments may send a re-INVITE containing an SDP in the final ACK
+	  request. If this happens in the context of direct media, the remote end
+	  should be updated with a re-INVITE.
+	  This patch queues an "update RTP peer" frame to trigger the re-INVITE,
+	  instead of the "source change" frame wich was used previously.
 
-2016-09-14 07:59 +0000 [1cac856e17]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-26951
 
-	* rtp: Preserve timestamps on video frames.
+	  Change-Id: I3644d2025f20e086ea9f8f62b486172c52b5b2e6
 
-	  Currently when receiving video over RTP we store only
-	  a calculated samples on the frame. When starting the video
-	  it can take some time for this calculation to actually yield
-	  a value as it requires constant changing timestamps. As well
-	  if a video frame passes over multiple RTP packets this calculation
-	  will fail as the timestamp is the same as the previous RTP
-	  packet and the number of samples calculated will be 0.
+2017-04-19 15:08 +0000 [c47b3e74d2]  Sean Bright <sean.bright@gmail.com>
 
-	  This change preserves the timestamp on the frame and allows
-	  it to pass through the core. When sending the video this timestamp
-	  is used instead of a new one being calculated.
+	* pbx: Use same thread if AST_OUTGOING_WAIT_COMPLETE specified
 
-	  ASTERISK-26367 #close
+	  Both ast_pbx_outgoing_app() and ast_pbx_outgoing_exten() cause the core
+	  to spawn a new thread to perform the dial. When AST_OUTGOING_WAIT_COMPLETE
+	  is passed to these functions, the calling thread will be blocked until
+	  the newly created channel has been hung up.
 
-	  Change-Id: Iba8179fb5c14c9443aee4baf670d2185da3ecfbd
+	  After this patch, we run the dial on the current thread rather than
+	  spawning a new one. The only in-tree code that passes
+	  AST_OUTGOING_WAIT_COMPLETE is pbx_spool, so you should see reduced
+	  thread usage if you are using .call files.
 
-2016-09-14 09:51 +0000 [9df4056d70]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I512735d243f0a9da2bcc128f7a96dece71f2d913
 
-	* res_pjsip_transport_management: Convert time in log message to seconds.
+2017-04-19 13:23 +0000 [afad2ffd9f]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26375 #close
+	* res_rtp_asterisk.c: Fix crash in RTCP DTLS operation.
 
-	  Change-Id: I46496af5cae41413e76d44d2068a7431279f09dc
+	  Occasionally a crash happens when processing the RTCP DTLS timeout
+	  handler.  The RTCP DTLS timeout timer could be left running if we have not
+	  completed the DTLS handshake before we place the call on hold or we
+	  attempt direct media.
 
-2016-09-13 05:34 +0000 [98e42cc662]  Steve Davies <steve@one47.co.uk>
+	  * Made ast_rtp_prop_set() stop the RTCP DTLS timer when disabling RTCP.
 
-	* chan_sip: Fix session timeout on retransmit of non-UDP packets
+	  * Made some sanity tweaks to ast_rtp_prop_set() when switching from
+	  standard RTCP mode to RTCP multiplexed mode.
 
-	  Change-Id I1cd33453c77c56c8e1394cd60a6f17bb61c1d957 Enable Session-Timers for
-	  SIP over TCP (and TLS) also disables SIP retransmits in chan_sip for non-UDP
-	  connections, allowing the TCP layer to handle the retransmits. Unfortunately,
-	  this caused sessions to be terminated with a retransmit timeout becasue it
-	  stopped at the point of the first retrans call.
+	  ASTERISK-26692 #close
 
-	  This patch waits for the 64*T1 timer to expire instead.
+	  Change-Id: If6c64c79129961acfa4b3d63a864e8f6b664acc0
 
-	  ASTERISK-19968
+2017-03-22 16:05 +0000 [d165079cbc]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I844f26801aada10bc94e9bebe6e151f0a8443204
+	* rtp_engine/res_rtp_asterisk: Fix RTP struct reentrancy crashes.
 
-2016-09-12 12:25 +0000 [0388882cdb]  Richard Mudgett <rmudgett@digium.com>
+	  The struct ast_rtp_instance has historically been indirectly protected
+	  from reentrancy issues by the channel lock because early channel drivers
+	  held the lock for really long times.  Holding the channel lock for such a
+	  long time has caused many deadlock problems in the past.  Along comes
+	  chan_pjsip/res_pjsip which doesn't necessarily hold the channel lock
+	  because sometimes there may not be an associated channel created yet or
+	  the channel pointer isn't available.
 
-	* app_queue: Fix CLI "queue show" and AMI Queues action output truncation.
+	  In the case of ASTERISK-26835 a pjsip serializer thread was processing a
+	  message's SDP body while another thread was reading a RTP packet from the
+	  socket.  Both threads wound up changing the rtp->rtcp->local_addr_str
+	  string and interfering with each other.  The classic reentrancy problem
+	  resulted in a crash.
 
-	  The output of CLI "queue show" and AMI Queues action is truncated and
-	  "failed to extend from 240 to 327" messages are generated if the queue
-	  member and interface names are lengthy.
+	  In the case of ASTERISK-26853 a pjsip serializer thread was processing a
+	  message's SDP body while another thread was reading a RTP packet from the
+	  socket.  Both threads wound up processing ICE candidates in PJPROJECT and
+	  interfering with each other.  The classic reentrancy problem resulted in a
+	  crash.
 
-	  * Increase the string buffer size from 240 to 512 in order to accommodate
-	  for more information fields added to the output since v1.8.
+	  * rtp_engine.c: Make the ast_rtp_instance_xxx() calls lock the RTP
+	  instance struct.
 
-	  ASTERISK-26360 #close
-	  Reported by: Richard Mudgett
+	  * rtp_engine.c: Make ICE and DTLS wrapper functions to lock the RTP
+	  instance struct for the API call.
 
-	  Change-Id: Id99c03cf5362453b80491a4b3b0434cb67aa966d
+	  * res_rtp_asterisk.c: Lock the RTP instance to prevent a reentrancy
+	  problem with rtp->rtcp->local_addr_str in the scheduler thread running
+	  ast_rtcp_write().
 
-2016-09-12 03:28 +0000 [da8ba990d1]  Walter Doekes <walter+github@wjd.nu>
+	  * res_rtp_asterisk.c: Avoid deadlock when local RTP bridging in
+	  bridge_p2p_rtp_write() because there are two RTP instance structs
+	  involved.
 
-	* chan_sip: Allow target refresh (Contact update) on re-INVITE.
+	  * res_rtp_asterisk.c: Avoid deadlock when trying to stop scheduler
+	  callbacks.  We cannot hold the instance lock when trying to stop a
+	  scheduler callback.
 
-	  Previously, the Contact was stored only on initial INVITE and on any
-	  18X and 200. That meant that after re-INVITEs from *us* the Contact
-	  could get updated, but after re-INVITEs from the *peer*, it did not.
+	  * res_rtp_asterisk.c: Remove the lock in struct dtls_details and use the
+	  struct ast_rtp_instance ao2 object lock instead.  The lock was used to
+	  synchronize two threads to prevent a race condition between starting and
+	  stopping a timeout timer.  The race condition is no longer present between
+	  dtls_perform_handshake() and __rtp_recvfrom() because the instance lock
+	  prevents these functions from overlapping each other with regards to the
+	  timeout timer.
 
-	  This changeset fixes this inconsistency, properly allowing target
-	  refreshes through re-INVITES (RFC3261, 12.2).
+	  * res_rtp_asterisk.c: Remove the lock in struct ast_rtp and use the struct
+	  ast_rtp_instance ao2 object lock instead.  The lock was used to
+	  synchronize two threads using a condition signal to know when TURN
+	  negotiations complete.
 
-	  If your strictrtp setting allows it, this change allows you to switch
-	  the source IP of a connected/calling device mid-call with a simple
-	  re-INVITE from the new IP.
+	  * res_rtp_asterisk.c: Avoid deadlock when trying to stop the TURN
+	  ioqueue_worker_thread().  We cannot hold the instance lock when trying to
+	  create or shut down the worker thread without a risk of deadlock.
 
-	  ASTERISK-26358 #close
+	  This patch exposed a race condition between a PJSIP serializer thread
+	  setting up an ICE session in ice_create() and another thread reading RTP
+	  packets.
 
-	  Change-Id: Ibb8512054ab27c8c3d2514022568fde943bf2435
+	  * res_rtp_asterisk.c:ice_create(): Set the new rtp->ice pointer after we
+	  have re-locked the RTP instance to prevent the other thread from trying to
+	  process ICE packets on an incomplete ICE session setup.
 
-2016-08-31 15:22 +0000 [e9ddab4685]  Richard Mudgett <rmudgett@digium.com>
+	  A similar race condition is between a PJSIP serializer thread resetting up
+	  an ICE session in ice_create() and the timer_worker_thread() processing
+	  the completion of the previous ICE session.
 
-	* sip_to_pjsip.py: Map legacy_useroption_parsing.
+	  * res_rtp_asterisk.c:ast_rtp_on_ice_complete(): Protect against an
+	  uninitialized/null remote_address after calling
+	  update_address_with_ice_candidate().
 
-	  Map the sip.conf general section legacy_useroption_parsing to the
-	  new pjsip.conf global ignore_uri_user_options.
+	  * res_rtp_asterisk.c: Eliminate the chance of ice_reset_session()
+	  destroying and setting the rtp->ice pointer to NULL while other threads
+	  are using it by adding an ao2 wrapper around the PJPROJECT ice pointer.
+	  Now when we have to unlock the RTP instance object to call a PJPROJECT ICE
+	  function we will hold a ref to the wrapper.  Also added some rtp->ice NULL
+	  checks after we relock the RTP instance and have to do something with the
+	  ICE structure.
 
-	  ASTERISK-26316
-	  Reported by: Kevin Harwell
+	  ASTERISK-26835 #close
+	  ASTERISK-26853 #close
 
-	  Change-Id: I78108a31995db19d41f4e1a07b3324692c5363fc
+	  Change-Id: I780b39ec935dcefcce880d50c1a7261744f1d1b4
 
-2016-08-29 18:08 +0000 [30af92e78d]  Richard Mudgett <rmudgett@digium.com>
+2017-04-19 08:39 +0000 [b8b3380944]  Sean Bright <sean.bright@gmail.com>
 
-	* res_pjsip: Add ignore_uri_user_options option.
+	* build: Update config.guess and config.sub
 
-	  This implements the chan_sip legacy_useroption_parsing option but with a
-	  better name.
+	  Change-Id: Id078a1df07a771808775e1053cdfe1d99c8fb172
 
-	  * Made the caller-id number and redirecting number strings obtained from
-	  incoming SIP URI user fields always truncated at the first semicolon.
-	  People don't care about anything after the semicolon showing up on their
-	  displays even though the RFC allows the semicolon.
+2017-04-14 13:52 +0000 [6c0ab9afa7]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26316 #close
-	  Reported by: Kevin Harwell
+	* format_wav: Read 16khz wav samples properly
 
-	  Change-Id: Ib42b0e940dd34d84c7b14bc2e90d1ba392624f62
+	  When opening a PCM wave file for reading, we aren't tracking the
+	  frequency of the opened file, so we treat 16khz files as 8khz and do
+	  half reads.
 
-2016-09-09 06:26 +0000 [7ed5dc2c58]  Walter Doekes <walter+github@wjd.nu>
+	  This patch also cleans up some of the data types and an unnecessarily
+	  complex `if` expression.
 
-	* contrib: Let safe_asterisk script continue without /dev/tty9.
+	  ASTERISK-26613 #close
+	  Reported by: Vitaly K
 
-	  If you use the safe_asterisk script, it uses hardcoded defaults before
-	  running configurable values from /etc/asterisk/startup.d. The hardcoded
-	  default has TTY=9. Some containerized environments don't have such a
-	  TTY, and safe_asterisk would stop.
+	  Change-Id: I05f8b263058dc573ea8ffe0c62e7964506e11815
 
-	  The custom configuration from /etc/asterisk/startup.d/* isn't read until
-	  after it stopped, so changing TTY in a custom config did not help.
+2017-04-16 19:59 +0000 [b55d21ad91]  George Joseph <gjoseph@digium.com>
 
-	  This changeset changes safe_asterisk to continue if the TTY setting was
-	  untouched and /dev/tty9 and /dev/vc/9 aren't found.
+	* make ari-stubs so doc periodic jobs can run
 
-	  Change-Id: I2c7cdba549b77f418a0af4cb1227e8e6fe4148fc
+	  The periodic doc job does a make ari-stubs and checks that
+	  there are no changes before generating the docs.  Since I changed
+	  the mustache template (and the generated code directly) recently
+	  and forgot to regenerate the stubs, the doc job thinks they're out
+	  of date.
 
-2016-09-09 05:39 +0000 [7580a736bb]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I94b97035311eccf52b0101b8590223265a7881d4
 
-	* res_pjsip: Only invoke unidentified endpoint logic when unidentified.
+2017-04-14 12:51 +0000 [4fb9f5d60e]  Sean Bright <sean.bright@gmail.com>
 
-	  The code was incorrectly invoking the unidentified logic when
-	  an endpoint had actually been identified, causing log messages
-	  to be output.
+	* format_ogg_vorbis: Clear ogg/vorbis data structures on close
 
-	  ASTERISK-26349 #close
+	  On filestream close, we need to clear out the ogg & vorbis data
+	  structures to prevent a memory leak.
 
-	  Change-Id: Id8104fc9e3d138d5e8b6f6977ecc08765fd17d4f
+	  ASTERISK-26169 #close
+	  Reported by: Ivan Myalkin
 
-2016-08-23 06:35 +0000 [efcfc4c1ee]  Corey Farrell <git@cfware.com> (license 5909)
+	  Change-Id: Iee94c5a5d5bdafbf8b181c5c064d15d90ace8274
 
-	* chan_sip: Don't allocate new RTP instances on top of old ones.
+2017-04-14 17:32 +0000 [a3e623dd70]  Richard Mudgett <rmudgett@digium.com>
 
-	  In some scenarios dialog_initialize_rtp can be called multiple times on
-	  the same dialog.  This can cause RTP instances to be leaked along with
-	  multiple file descriptors for each instance.
+	* Revert "bridging:  Ensure successful T.38 negotation"
 
-	  This change makes it so the existing RTP instances are destroyed and
-	  not overwritten, stopping the memory leak.
+	  This reverts commit 7819f95791fe0ca0e0cdc417e2687a5900444053.
 
-	  ASTERISK-26272 #close
-	  patches:
-	    ASTERISK-26272-13.patch submitted by Corey Farrell (license 5909)
+	  Change-Id: Ib91a7e6c9856f5f41329e42f40ba2394fee861a4
 
-	  Change-Id: Id529de1184c68f2f4d254ab41a1f458dafdb5f73
+2017-04-14 16:50 +0000 [f6600f2c2e]  Sean Bright <sean.bright@gmail.com>
 
-2016-08-16 15:34 +0000 [f1ffc22933]  Mark Michelson <mmichelson@digium.com>
+	* res_stun_monitor: Don't fail to load if DNS resolution fails
 
-	* res_pjsip: Do not crash on ACKs from unknown endpoints.
+	  res_stun_monitor will fail to load if DNS resolution of the STUN server
+	  fails. Instead, we continue without the STUN server being resolved and
+	  we will re-attempt the resolution on the STUN refresh interval.
 
-	  The endpoint identification PJSIP module is intended to identify which
-	  endpoint an incoming request is from. If an endpoint is not identified,
-	  then an artificial endpoint is used in its place when proceeding.
+	  ASTERISK-21856 #close
+	  Reported by: Jeremy Kister
 
-	  The problem is that the ACK request type is an exception to the rule.
-	  The artificial endpoint is not used when processing an ACK. This results
-	  in the possibility of having a NULL endpoint being used further on.
+	  Change-Id: I6334c54a1cc798f8a836b4b47948e0bb4ef59254
 
-	  The reason ACK is an exception is an attempt not to spam security logs
-	  with unidentified requests. Presumably, you've already logged the
-	  unidentified request on the preceeding INVITE.
+2017-04-14 14:36 +0000 [be71be7ed2]  Sean Bright <sean.bright@gmail.com>
 
-	  Up until Asterisk 13.10, retrieving a NULL endpoint in this fashion
-	  didn't cause an issue. A new change in 13.10 added endpoint ACL checking
-	  shortly after endpoint identification. Because we are accessing a NULL
-	  endpoint, this ACL check resulted in a crash.
+	* format_pcm: Track actual header size of .au files
 
-	  The fix here is to be sure to retrieve the artificial endpoint for all
-	  request types. ACKs still do not generate unidentified request security
-	  events.
+	  Sun's Au file format has a minimum data offset 24 bytes, but this
+	  offset is encoded in each .au file. Instead of assuming the minimum,
+	  read the actual value and store it for later use.
 
-	  ASTERISK-26264 #close
-	  Reported by nappsoft
+	  ASTERISK-20984 #close
+	  Reported by: Roman S.
+	  Patches:
+	  	asterisk-1.8.20.0-au-clicks-2.diff (license #6474) patch
+	  	uploaded by Roman S.
 
-	  AST-2016-006
+	  Change-Id: I524022fb19ff2fd5af2cc2d669d27a780ab2057c
 
-	  Change-Id: Ie0c795ae2d72273decb972dd74b6a1489fb6b703
+2017-04-12 07:50 +0000 [2e6075c51f]  George Joseph <gjoseph@digium.com>
 
-2016-09-06 11:46 +0000 [23d6ec7417]  Richard Mudgett <rmudgett@digium.com>
+	* modules:  change module LOAD_FAILUREs to LOAD_DECLINES (master)
 
-	* res_pjsip_messaging.c: Misc cleanups and fixes.
+	  Change-Id: Iac40ecb20e10513d67bf0eaf61807f306067b258
 
-	  * Eliminated RAII_VAR in get_outbound_endpoint().
+2017-04-10 05:13 +0000 [72c5f3b0ba]  Alexander Traud <pabstraud@compuserve.com>
 
-	  * Simplify update_to() coding.  However, this function can only be a NoOp
-	  because the To string can only be a URI and not a name-address formatted
-	  string.
+	* res_pjsip_sdp_rtp: No rtpmap for static RTP payload IDs in SDP.
 
-	  * Simplify update_from() coding.  Also fixed a code path modifying the
-	  from string when the caller could still want to use the original string.
+	  This saves around 100 bytes when G.711, G.722, G.729, and GSM are advertised in
+	  SDP. This reduces the chance to hit the MTU bearer of 1300 bytes for SIP over
+	  UDP, if many codecs are allowed in Asterisk. This new feature is enabled
+	  together with the optional feature compact_headers=yes via the file pjsip.conf.
 
-	  * Fixed msg_data_create() incompletely removing the "pjsip:" to then add
-	  back the "sip:" string if needed.  The code didn't handle the "pjsip:sip:"
-	  case because it left the colon after pjsip in the string.
+	  ASTERISK-26932 #close
 
-	  Change-Id: I68a09a665f6d4daa9eaa59069045ab69122e28db
+	  Change-Id: Iaa556ab4c8325cd34c334387ab2847fab07b1689
 
-2016-09-07 16:00 +0000 [5f19657710]  Joshua Colp <jcolp@digium.com>
+2017-04-12 07:47 +0000 [6db0939b96]  George Joseph <gjoseph@digium.com>
 
-	* res_pjsip: Allow global headers to be overridden.
+	* modules:  change module LOAD_FAILUREs to LOAD_DECLINES (14)
 
-	  Currently when you add global headers from the dialplan both
-	  the header in the dialplan and the globally configured header
-	  are added to the resulting SIP INVITE. This change makes it
-	  so the headers in the dialplan take precedence and are the
-	  only ones added.
+	  Change-Id: If99e3b4fc2d7e86fc3e61182aa6c835b407ed49e
 
-	  Change-Id: I36f864298f38db3632ad503edc11267cb8ffb3ad
+2017-04-11 11:07 +0000 [747beb1ed1]  George Joseph <gjoseph@digium.com>
 
-2016-08-11 12:10 +0000 [206d4f57dc]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* modules:  change module LOAD_FAILUREs to LOAD_DECLINES
 
-	* followme: initialize all config items on reload
+	  In all non-pbx modules, AST_MODULE_LOAD_FAILURE has been changed
+	  to AST_MODULE_LOAD_DECLINE.  This prevents asterisk from exiting
+	  if a module can't be loaded.  If the user wishes to retain the
+	  FAILURE behavior for a specific module, they can use the "require"
+	  or "preload-require" keyword in modules.conf.
 
-	  Some configuration directives were not initialized on reload, and hence
-	  were not reset to default if they were removed from followme.conf.
+	  A new API was added to logger: ast_is_logger_initialized().  This
+	  allows asterisk.c/check_init() to print to the error log once the
+	  logger subsystem is ready instead of just to stdout.  If something
+	  does fail before the logger is initialized, we now print to stderr
+	  instead of stdout.
 
-	  ASTERISK-26288 #close
+	  Change-Id: I5f4b50623d9b5a6cb7c5624a8c5c1274c13b2b25
 
-	  Change-Id: Ief829e16374ad1e0ecfd63e6ee4923b5a1d1c150
+2017-04-05 06:41 +0000 [7819f95791]  Torrey Searle <torrey@voxbone.com>
 
-2016-08-01 20:55 +0000 [117a7741c8]  gtjoseph <gjoseph@digium.com>
+	* bridging:  Ensure successful T.38 negotation
 
-	* build: Add download capability for external packages
+	  When a T.38 happens immediatly after call establishment, the control
+	  frame can be lost because the other leg is not yet in the bridge.
 
-	  The DPMA and g729a, silk, siren7 and siren14 codecs hosted at
-	  http://downloads.digium.com/pub/telephony/ are now listed in the
-	  "External" sections of the "Resource Modules" and "Codec Translators"
-	  pages in menuselect.  Any that are selected will automatically be
-	  downloaded and installed when "make install" is run.  Their LICENSE and
-	  README (if avaialble) files will be installed to
-	  ASTVARLIBDIR/documentation/thirdparty/<product_name>.
+	  This patch detects this case an makes sure T.38 negotation happens
+	  when the 2nd leg is being made compatible with the negotating
+	  first leg
 
-	  Example use with codecs:
+	  ASTERISK-26923 #close
 
-	  The codecs/codecs.xml file is a menuselect style xml file that lists
-	  the codecs to be included.  Their support levels are 'external', which
-	  triggers the download and install, and defaultenabled is no.  Also
-	  because codec_g729a is actually in a directory named codec_g729 on the
-	  download server, the newly added 'member_data' element is used to
-	  override the default of the directory name being the package name.  You
-	  can use the 'directory_name' attribute to keep default base URL
-	  (http://downloads.digium.com/pub/telephony/) but use the new directory,
-	  or you use the 'remote_url' attribute to specify a full URL to the
-	  download directory.  In this case, you must still follow the same
-	  subdirectory naming conventions as that used for the packages located
-	  at 'http://downloads.digium.com/pub/telephony'.
+	  Change-Id: If334125ee61ed63550d242fc9efe7987e37e1d94
 
-	  A new configure option '--with-externals-cache' was added and like
-	  '--with-sounds-cache' it allows the installer to cache tarballs so
-	  they're not downloaded every time.
+2017-04-07 08:58 +0000 [7901225261]  Torrey Searle <torrey@voxbone.com>
 
-	  To assist with the download and install process, each external package
-	  now has a manifest.xml file that, among other things, contains a package
-	  version and checksums for each file in the tarball.  The manifest is
-	  saved to both the cache directory and ASTMODDIR and together with the
-	  manifest.xml on the downloads site, tells the install scripts whether
-	  a download and/or update is needed.
+	* strings.h:  Avoid overflows in the string hash functions
 
-	  bash and xmlstarlet are required for downloader operation.  If they're
-	  not installed, the external items in menuselect will be unavailable.
+	  On 2's compliment machines abs(INT_MIN) behavior is undefined and
+	  results in a negative value still being returnd.  This results in
+	  negative hash codes that can result in crashes.
 
-	  Change-Id: Id3dcf1289ffd3cb0bbd7dfab3cafbb87be60323a
+	  ASTERISK-26528 #close
 
-2016-09-06 02:41 +0000 [d04ae7d1d8]  Walter Doekes <walter+github@wjd.nu>
+	  Change-Id: Idff550145ca2133792a61a2e212b4a3e82c6517b
 
-	* chan_sip: Don't refuse calls with "optional crypto"; fall back to RTP.
+2017-04-07 16:14 +0000 [7312cbe803]  Richard Mudgett <rmudgett@digium.com>
 
-	  Certain SNOM phones send so-called "optional crypto" in their SDP body.
-	  Regular SRTP setup looks like this:
+	* res_rtp_asterisk.c: Add stun_blacklist option
 
-	      m=audio 64620 RTP/SAVP 8 0 9 99 3 18 4 101
-	      a=crypto:1 AES_CM_128_HMAC_SHA1_32 inline:...
+	  Added the stun_blacklist option to rtp.conf.  Some multihomed servers have
+	  IP interfaces that cannot reach the STUN server specified by stunaddr.
+	  Blacklist those interface subnets from trying to send a STUN packet to
+	  find the external IP address.  Attempting to send the STUN packet
+	  needlessly delays processing incoming and outgoing SIP INVITEs because we
+	  will wait for a response that can never come until we give up on the
+	  response.  Multiple subnets may be listed.
 
-	  SNOM-style "optional crypto" looks like this:
+	  ASTERISK-26890 #close
 
-	      m=audio 61438 RTP/AVP 8 0 9 99 3 18 4 101
-	      a=crypto:1 AES_CM_128_HMAC_SHA1_32 inline:...
+	  Change-Id: I3ff4f729e787f00c3e6e670fe6435acce38be342
 
-	  A crypto line is supplied, but the m-line does not have SAVP.
+2017-04-06 17:31 +0000 [7c37365f03]  Richard Mudgett <rmudgett@digium.com>
 
-	  When res_srtp.so is *not* loaded, then chan_sip.so treats the optional
-	  crypto as regular RTP, but when res_srtp.so *is* loaded, it refuses the
-	  incoming call with the following message:
+	* stun.c: Fix ast_stun_request() erratic timeout.
 
-	      WARNING: process_sdp: Failed to receive SDP offer/answer with
-	      required SRTP crypto attributes for audio
+	  If ast_stun_request() receives packets other than a STUN response then we
+	  could conceivably never exit if we continue to receive packets with less
+	  than three seconds between them.
 
-	  For platforms that want to start providing SRTP this presents a
-	  compatibility problem.
+	  * Fix poll timeout to keep track of the time when we sent the STUN
+	  request.  We will now send a STUN request every three seconds regardless
+	  of how many other packets we receive while waiting for a response until we
+	  have completed three STUN request transmission cycles.
 
-	  This changeset lets chan_sip handle the SDP as if no crypto-line was
-	  supplied: i.e. accept the call as regular RTP, just like it did before
-	  res_srtp was loaded.
+	  Change-Id: Ib606cb08585e06eb50877f67b8d3bd385a85c266
 
-	  Now you'll get this informative warning instead:
+2017-04-06 18:30 +0000 [8d323c74fa]  Richard Mudgett <rmudgett@digium.com>
 
-	      WARNING: Ignoring crypto attribute in SDP because RTP transport is
-	      insecure
+	* sorcery.c: Speed up ast_sorcery_retrieve_by_id()
 
-	  ASTERISK-23989 #close
-	  Reported by: Olle Johansson
+	  Return early if ast_sorcery_retrieve_by_id() is not passed an id to find.
+	  Also eliminated the RAII_VAR() usage in the function.
 
-	  Change-Id: I91a15ae05a0296e398d6b65f53bb11afde1d80e2
+	  Change-Id: I871dbe162a301b5ced8b4393cec27180c7c6b218
 
-2016-09-03 16:04 +0000 [df3d0188e4]  Matt Jordan <mjordan@digium.com>
+2017-04-10 11:30 +0000 [5b4e2ec267]  Richard Mudgett <rmudgett@digium.com>
 
-	* apps/app_dial: Fix crash on non-connect call paths for Privacy/Screening option
+	* res_pjsip: Fix pointer use after unref.
 
-	  In any scenario in which the callee is not connected to the caller, the
-	  current code in app_dial will crash due to raising a Dial End Stasis
-	  Message after the callee channel has been hung up. This patch corrects
-	  the error by simply moving the explicit hangup of the callee (peer)
-	  channel until after the dial end message.
+	  Change-Id: I4b6e1b0070563eeaee223cb58326f1b962ed5bc1
 
-	  ASTERISK-25691 #close
+2017-04-06 18:18 +0000 [6f793ac149]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I816a414014424d0d8c80e2a3cbef13ef8c63798d
+	* res_pjsip_sdp_rtp.c: Don't use deprecated transport struct member.
 
-2016-09-03 16:02 +0000 [a64063cc97]  Matt Jordan <mjordan@digium.com>
+	  * create_rtp(): Eliminate use of deprecated transport struct member.  That
+	  member and several others in the transport structure were deprecated
+	  because of an infinite loop created when using realtime configuration.
+	  See 2451d4e4550336197ee2e482750cc53f30afa352
 
-	* apps/app_dial: Set the DIALSTATUS to NOANSWER on privacy option 5
+	  ASTERISK-26851
 
-	  If the callee selects option '5' using the Dial application's privacy
-	  (P) option, the DIALSTATUS is erroneously set to ANSWER. This option
-	  reflects the callee sending the caller to VoiceMail one time; the call
-	  is definitely *not* ANSWERed in such a scenario. With this patch, the
-	  DIALSTATUS is instead set to NOANSWER, which is the same DIALSTATUS that
-	  is set when the 'send to VoiceMail every time' option is set.
+	  Change-Id: I0533aa13c9ce3c6cc394e0fd2b5bf1cd1b2ef3bc
 
-	  ASTERISK-25691
+2017-04-10 17:45 +0000 [d76bc0565c]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Iaf0c9f0fa00545e7366443875e2bb7d9a89a1358
+	* tcptls.c: Cleanup TCP/TLS listener thread on abnormal exit.
 
-2016-08-30 16:40 +0000 [03fc438f6e]  Richard Mudgett <rmudgett@digium.com>
+	  Temporarily running out of file descriptors should not terminate the
+	  listener thread.  Otherwise, when there becomes more file descriptors
+	  available, nothing is listening.
 
-	* res_pjsip_registrar.c: Reduce stack usage in find_aor_name().
+	  * Added EMFILE exception to abnormal thread exit.
 
-	  Change-Id: I8aebad1fdcf303bd115b59a4b57fbbd5b2267f09
+	  * Added an abnormal TCP/TLS listener exit error message.
 
-2016-08-29 18:06 +0000 [b5e753227d]  Richard Mudgett <rmudgett@digium.com>
+	  * Closed the TCP/TLS listener socket on abnormal exit so Asterisk does not
+	  appear dead if something tries to connect to the socket.
 
-	* pjsip_configuration.c: Ignore repeated identify by methods.
+	  ASTERISK-26903 #close
 
-	  Change-Id: Ied0c06043d1dfef8fdc9c9a808cf89b118119838
+	  Change-Id: I10f2f784065136277f271159f0925927194581b5
 
-2016-08-30 17:26 +0000 [9b7501b6ad]  Richard Mudgett <rmudgett@digium.com>
+2017-04-08 03:05 +0000 [2b8dbc9e00]  Walter Doekes <walter+github@wjd.nu>
 
-	* config_global.c: Comments and a default expression adjustment.
+	* samples: Undo removal of include from canonicalize-app-names commit.
 
-	  Change-Id: Ia6a58f8c73a30da6874b3f94364dce162d6f1ad3
+	  This include was accidentally removed in changeset
+	  Ia79aea64de89531362e993e34230c2044a70aa93. My bad.
 
-2016-08-31 15:14 +0000 [3314e1cec2]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I1d716c7f9590b4e97909fb8bca1f2ed9bd0e4082
 
-	* sip_to_pjsip.py: Map canreinvite as directmedia alias.
+2017-04-07 08:35 +0000 [270b485f04]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I48b8e150f96a3d2a24d8fc25fbe4f5aff9f4a6b2
+	* pjsip: Add Alembic for PUBLISH support.
 
-2016-08-31 15:37 +0000 [6372f40ba0]  Richard Mudgett <rmudgett@digium.com>
+	  This change adds database tables for the PUBLISH support so it
+	  can be configured using realtime. A minor fix to the
+	  res_pjsip_publish_asterisk module was done so that it read the
+	  sorcery configuration from the correct section. Finally the
+	  sample configuration files have been updated.
 
-	* sip_to_pjsip.py: Fix typo converting outboundproxy registration.
+	  ASTERISK-26928
 
-	  Change-Id: I6f30e5f9fcf8469ba0079fbf884047d54c2c0b15
+	  Change-Id: I81991ae5c75af98d247f7eacd1c0b0a763675952
 
-2016-08-31 15:13 +0000 [11eb1afd2d]  Richard Mudgett <rmudgett@digium.com>
+2017-04-07 08:06 +0000 [7a46cd7433]  Alexander Traud <pabstraud@compuserve.com>
 
-	* sip_to_pjsip.py: Fix comment typo and tabs.
+	* pjproject_bundled: Crash on pj_ssl_get_info() while ioqueue_on_read_complete().
 
-	  Change-Id: If35174614545727817d329c60ba4456c028941b5
+	  When the Asterisk channel driver res_pjsip offers SIP-over-TLS, sometimes, not
+	  reproducible, Asterisk crashed in pj_ssl_sock_get_info() because a NULL pointer
+	  was read. This change avoids this crash.
 
-2016-08-31 15:56 +0000 [0f9b144c1a]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26927 #close
 
-	* Sample configs: Eliminate false multiline comment block starts.
+	  Change-Id: I24a6011b44d1426d159742ff4421cf806a52938b
 
-	  Change-Id: Ie627def9604ae30abd80754f9e6f09874825aec6
+2017-04-05 09:10 +0000 [e6ae3651b8]  Walter Doekes <walter+github@wjd.nu>
 
-2016-09-02 11:36 +0000 [8d1c535bd6]  Richard Mudgett <rmudgett@digium.com>
+	* samples: Canonicalize app names in extensions.conf.sample.
 
-	* format_cap.c: Fix CLI "core show channeltype Surrogate" crash.
+	  This takes care of warnings by ossobv/asterisklint.
 
-	  * Make ast_format_cap_get_names() NULL tolerant.
+	  Change-Id: Ia79aea64de89531362e993e34230c2044a70aa93
 
-	  ASTERISK-26331 #close
-	  Reported by: CGI.NET
+2017-04-04 16:20 +0000 [01e9eaf3a6]  George Joseph <gjoseph@digium.com>
 
-	  Change-Id: Id67e93936dc8ec2a33a9d33655843d43b59285a3
+	* pjproject_bundled: Add 3 upstream patches
 
-2016-08-18 14:45 +0000 [9bca895469]  Alexei Gradinari <alex2grad@gmail.com>
+	  0035-r5572-svn-backport-dialog-transaction-deadlock.patch
+	  0036-r5573-svn-backport-ua-pjsua-transaction-deadlock.patch
+	  0037-r5576-svn-backport-session-timer-crash.patch
 
-	* res_pjsip_session: segfault on already disconnected session
+	  Also removed the progress bar from wget download to stdout.
 
-	  On heavy loaded system the TCP/TLS incoming calls could be
-	  disconnected by pjproject while these calls are being
-	  processed by asterisk which could use the session's memory pools.
-	  If the session in the disconnected state then the session memory
-	  pools were already freed, so we get segfault.
+	  ASTERISK-26905 #close
+	  Reported-by: Ross Beer
 
-	  This patch adds a lifetime control on an INVITE session to pjproject.
-	  The lifetime of the session is manipulated by calling
-	  pjsip_inv_add_ref/pjsip_inv_dec_ref.
-	  This patch uses these functions to inform pjproject that the
-	  session is in use.
+	  Change-Id: I268fb3cf71a3bb24283ff0d24bd8b03239d81256
 
-	  This patch adds check if the session state is not disconnected
-	  and also checks if the memory pool is not NULL.
+2017-04-04 11:44 +0000 [fac5115c43]  Troy Bowman <troy@lump.net>
 
-	  This patch also places tasks 'session_end' and 'session_end_completion'
-	  into session's serializer to avoid race condition.
+	* app_queue: Log reason for PAUSEALL/UNPAUSEALL
 
-	  ASTERISK-26291 #close
+	  We needed the reason for our reporting when agents pause/unpause all of
+	  their queues at once.  This is a small, simple patch that adds a reason
+	  for PAUSEALL and UNPAUSEALL.  I have been using it in production for years.
 
-	  Change-Id: I4d28b1fb3b91f0492a911d110049d670fdc3c8d7
+	  ASTERISK-26920 #close
 
-2016-08-10 15:14 +0000 [63feffa126]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: Ifb3f0d1a0abd5194253d9794023546e1395baf3d
 
-	* ConfBridge: Make some announcements asynchronous.
+2017-04-05 14:50 +0000 [40e9d5e8b7]  George Joseph <gjoseph@digium.com>
 
-	  Confbridge announcements tend to block a channel while they are being
-	  played. In some circumstances, this is warranted since you want that
-	  particular channel not to hear the announcement (Example: "John Doe has
-	  entered the conference"). For others it makes less sense.
+	* sample_config:  Add samples for pubsub to pjsip.conf.sample
 
-	  This change first introduces methods for playing sounds asynchronously
-	  into the conference. This is very similar to how synchronous sounds are
-	  played, except the channel initiating the playback does not wait for the
-	  sound to complete before moving on.
+	  Added:
+	   * outbound-publish
+	   * resource_list
+	   * inbound-publication
+	   * asterisk-publication
 
-	  Asynchronous announcements are used for two circumstances:
-	  * Sounds played for a user after they have left the bridge
-	  * Sounds that play first to a single user and then the rest of the
-	    conference (if the channel and conference use the same language)
+	  Change-Id: I65043a896c35483f30a92d30b5b118359af7ba5a
 
-	  ASTERISK-26289 #close
-	  Reported by Mark Michelson
+2017-04-03 15:38 +0000 [f2ee8ac21e]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ie486bb3de1646d50894489030326a423e594ab0a
+	* res_pjsip_sdp_rtp.c: Don't alter global addr variable.
 
-2016-08-31 12:23 +0000 [a002a4d2db]  Michael Kuron <m.kuron@gmx.de>
+	  * create_rtp(): Fix unexpected alteration of global address_rtp if a
+	  transport is bound to an address.
 
-	* app_mp3: Use correct buffer size and the same sample rate as the channel
+	  * create_rtp(): Fix use of uninitialized memory if the endpoint RTP media
+	  address is invalid or the transport has an invalid address.
 
-	  Previously, the buffer used for MP3 streamed from HTTP servers had a size of
-	  1 MB. For 8 kHz mono audio at 16 bit resolution, such a buffer covers about 1
-	  minute. Only when the buffer is full does audio start to play.
-	  For MP3 files streamed from a server, that is usually not a big deal as long as
-	  the connection to the server is fast enough to supply that much data within a
-	  second or two. For MP3 live streams however, it takes 1 minute to download 1
-	  minute of audio, so without this change, app_mp3 wasn't really usable for MP3
-	  live streams.
-	  This commit changes the buffer size so that it covers 6 seconds of an MP3 file
-	  streamed from a server and 0.5 seconds of an MP3 live stream. The latter is
-	  identified by the use of a .m3u file extension.
+	  ASTERISK-26851
 
-	  app_mp3 so far only supported 8 kHz audio.
-	  Now it always runs at the sample rate of the channel.
+	  Change-Id: Icde42e65164a88913cb5c2601b285eebcff397b7
 
-	  ASTERISK-26085 #close
+2017-03-27 09:03 +0000 [380973cc47]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Id1ee274733cd804a0edecf7450329b72f1235af0
+	* CDR: Protect from data overflow in ast_cdr_setuserfield.
 
-2016-08-26 10:39 +0000 [308a65fe6c]  Alexei Gradinari <alex2grad@gmail.com>
+	  ast_cdr_setuserfield wrote to a fixed length field using strcpy. This could
+	  result in a buffer overrun when called from chan_sip or func_cdr. This patch
+	  adds a maximum bytes written to the field by using ast_copy_string instead.
 
-	* res_pjsip: qualify/unqualify added/deleted realtime endpoints
+	  ASTERISK-26897 #close
+	  patches:
+	    0001-CDR-Protect-from-data-overflow-in-ast_cdr_setuserfie.patch submitted
+	      by Corey Farrell (license #5909)
 
-	  If the PJSIP endpoint's AOR with the permanent contact
-	  was deleted from the realtime storage the res_pjsip module
-	  continues trying to qualify this contact.
-	  The error 'Unable to find an endpoint to qualify contact'
-	  appeares every 'qualify_frequency' seconds.
-	  This patch deletes this contact in this case.
+	  Change-Id: Ib23ca77e9b9e2803a450e1206af45df2d2fdf65c
 
-	  The PJSIP endpoint's AOR with the permanent contact
-	  is never qualified if it is added to realtime storage
-	  after asterisk started.
-	  This patch adds qualifying for the AOR's permanent contacts
-	  on the first handling of this AOR.
+2017-03-25 19:01 +0000 [6c3ae397cb]  Daniel Journo <dan@keshercommunications.com>
 
-	  ASTERISK-26319 #close
+	* Unused realtime MOH classes not purged on 'moh reload'
 
-	  Change-Id: Ib93dded9121edb113076903d1aa95402f799f8fe
+	  Purge Realtime MOH classes on 'moh reload' even when musiconhold.conf
+	  hasn't changed.
 
-2016-08-17 02:51 +0000 [2fa168348e]  chris de rock <chris@derock.de>
+	  ASTERISK-25974 #close
 
-	* app_macro: Consider '~~s~~' as a macro start extension.
+	  Change-Id: I42c78ea76528473a656f204595956c9eedcf3246
 
-	  As described in issue ASTERISK-26282 the AEL parser creates macros with
-	  extension '~~s~~'.  app_macro searches only for extension 's' so the
-	  created extension cannot be found.  with this patch app_macro searches for
-	  both extensions and performs the right extension.
+2017-03-31 12:09 +0000 [8e36064109]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26282 #close
+	* core: Improve/simplify handling of required headers.
 
-	  Change-Id: I939aa2a694148cc1054dd75ec0c47c47f47c90fb
+	  * Report failures if configure finds a required header is missing.
+	  * Deduplicate includes between asterisk.h, astmm.h and compat.h.
+	  * Unconditionally include headers in compat.h if required elsewhere.
 
-2016-08-29 07:10 +0000 [27951792c4]  Etienne Lessard <elessard@proformatique.com>
+	  Change-Id: Ie67d0185ca71fbfb81c9bdfaebe46a49e3c56dc5
 
-	* pbx.c: Prevent infinite recursion in manager_show_dialplan_helper.
+2017-04-03 13:56 +0000 [a889621b14]  Richard Mudgett <rmudgett@digium.com>
 
-	  Previously, if context A was including context B and context B was including
-	  context A, i.e. if there was a circular dependency between contexts, then
-	  calling manager_show_dialplan_helper could lead to an infinite recursion,
-	  resulting in a crash.
+	* res_pjsip: Fix transport ref leak.
 
-	  This commit applies the same solution as the one implemented in the
-	  show_dialplan_helper function. The manager_show_dialplan_helper and
-	  show_dialplan_helper functions contain lots of code in common, but the former
-	  was missing the "infinite recursion avoidance" code.
+	  We were leaking a transport ref in multihomed_on_rx_message() which
+	  resulted in the FRACK about excessive ref counts.
 
-	  ASTERISK-26226 #close
+	  ASTERISK-26916 #close
 
-	  Change-Id: I1aea85133c21787226f4f8442253a93000aa0897
+	  Change-Id: I7a96658a9614a060565bb9ad51cb1c9c11ee145f
 
-2016-08-26 14:34 +0000 [fb82fdb013]  gtjoseph <gjoseph@digium.com>
+2017-04-03 02:30 +0000 [4fc22c7673]  Alexander Traud <pabstraud@compuserve.com>
 
-	* pjproject_bundled:  Disable srtp use by pjmedia
+	* chan_sip: Session Timers required but refused wrongly.
 
-	  The reason for the disable is that while Asterisk works fine with older
-	  libsrtp versions, newer versions of pjproject won't compile with them.
-	  Debian 6 for instance, has libsrtp 1.4.4 which is older than what
-	  pjproject is expecting.
+	  SIP user-agents indicate which protocol extensions are allowed in headers
+	  like Supported and Required. Such protocol extensions are Session Timers
+	  (RFC 4028) for example. Session Timers are supported since Mantis-10665.
+	  Since ASTERISK-21721, not only the first but multiple Supported/Required
+	  headers in a message are parsed. In that change, an existing variable was
+	  re-used within a newly added do-loop. Currently, at the end of that loop,
+	  that variable is an empty string always. Previously, that variable was used
+	  within log output. However, the log output was not changed.
 
-	  We don't use most of pjmedia but we DO use it for SDP negotiation.
-	  Luckily disabling srtp in pjmedia doesn't interfere with it's ability
-	  to negitiate a secure channel.  The proper crypto attributes are
-	  negotiated in both directions.
+	  ASTERISK-26915 #close
 
-	  ASTERISK-26279 #close
+	  Change-Id: I09315f31b4d78fb214bb2a9fb6c0f5e143eae990
 
-	  Change-Id: Id25a92cdf3df97a26c53cffae65b6b82de33c8e2
+2017-03-31 16:31 +0000 [48be02c5d8]  Joshua Colp <jcolp@digium.com>
 
-2016-08-26 08:41 +0000 [847bd47ff0]  Alexander Traud <pabstraud@compuserve.com>
+	* res_pjsip_session: Allow BYE to be sent on disconnected session.
 
-	* channel: No hung-up on failing security requirements.
+	  It is perfectly acceptable for a BYE to be sent on a disconnected
+	  session. This occurs when we respond to a challenge to the BYE
+	  for authentication credentials.
 
-	  In your Diaplan, if you specify
-	   same => n,Set(CHANNEL(secure_bridge_media)=1)
-	   same => n,Set(CHANNEL(secure_bridge_signaling)=1)
-	  only the SIP channel driver chan_sip supports this. All other channels drivers
-	  like res_pjsip fail. In case of failure, the original sRTP source code released
-	  the whole channel, even if not hung-up, yet. This change does not release the
-	  channel but instead hangs-up the channel.
+	  ASTERISK-26363
 
-	  ASTERISK-26306
+	  Change-Id: I6ef0ddece812fea6665a1dd2549ef44fb9d90045
 
-	  Change-Id: I0489f0cb660fab6673b0db8af027d116e70a66db
+2017-03-31 13:14 +0000 [e8b1bb3041]  Richard Mudgett <rmudgett@digium.com>
 
-2016-08-20 09:04 +0000 [b59d3b48d0]  Alexander Traud <pabstraud@compuserve.com>
+	* chan_vpb.cc: Fix compiler error.
 
-	* sip_to_pjsip: Migrate IPv4/IPv6 (Dual Stack) configurations.
+	  Added missing channel technology read/write stream callback
+	  initialization.
 
-	  When using the migration script sip_to_pjsip.py, and your sip.conf is
-	  configured with bindaddr=::, two transports are written to pjsip.conf, one for
-	  0.0.0.0 (IPv4) and one for [::] (IPv6). That way, PJProject listens on the IPv4
-	  and IPv6 wildcards; a IPv4/IPv6 Dual Stack configuration on a single interface
-	  like in chan_sip.
+	  Change-Id: I829043a327d987e0d964485dd3d27964bebbd623
 
-	  Furthermore, the script internal functions "build_host" and "split_hostport"
-	  did not parse Literal IPv6 addresses as expected (like [::1]:5060). This change
-	  makes sure, even such addresses are parsed correctly.
+2017-03-30 18:28 +0000 [f9695dc057]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-26309
+	* Forward declare 'struct ast_json' in asterisk.h
 
-	  Change-Id: Ia4799a0f80fc30c0550fc373efc207c3330aeb48
+	  The ast_json structure is used in many Asterisk headers and is often the
+	  only part of json.h used.  This adds a forward declaration to asterisk.h
+	  and removes the include of json.h from many headers.  The declaration
+	  has been left in endpoints.h and stasis.h to avoid problems with source
+	  files that use ast_json functions without directly including json.h.
 
-2016-08-25 07:06 +0000 [f69f5cd3c4]  Joshua Colp <jcolp@digium.com>
+	  ari.h continues to include json.h as it uses enum
+	  ast_json_encoding_format.
 
-	* app_queue: Ensure member is removed from pending when hanging up.
+	  Change-Id: Id766aabce6bed56626d27e8d29f559b5e687b769
 
-	  When dialing channels it is possible that they may not ever
-	  leave the not in use state (Local channels in particular) by
-	  the time we cancel them. If this occurs but we know they were
-	  dialed we explicitly remove them from the pending members
-	  container so that subsequent call attempts occur.
+2017-03-30 08:11 +0000 [c537f99488]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-26299 #close
+	* cdr_pgsql: Fix buffer overflow calling libpq
 
-	  Change-Id: I6ad0d17c36480c92cebf840626228ce3f7e4bd65
+	  Implement the same buffer size checking done in cel_pgsql.
 
-2016-08-04 20:11 +0000 [5cd583d7a2]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26896 #close
+	  Reported by: twisted
 
-	* res_pjsip: Cache global config options.
+	  Change-Id: Iaacfa1f1de7cb1e9414d121850d2d8c2888f3f48
 
-	  We may check a global config option hundreds of times a second or more.
-	  Asking sorcery for the global configuration from the config files backend
-	  involves several allocations and container traversals.  Using realtime
-	  without a memory cache is a lot worse because you have to lookup in the
-	  realtime database each time to reconstitute the sorcery object.  With a
-	  memory cache for realtime, there is about the same amount of overhead as
-	  for config files.  Either way, it is still fairly expensive to access the
-	  sorcery object that much.
+2017-03-28 13:01 +0000 [a7d94f504f]  Walter Doekes <walter+github@wjd.nu>
 
-	  * Cache the global config options so we can access them faster.  You must
-	  now always perform a res_pjsip reload to change the global options.
+	* build: Fix deb build issues with fakeroot
 
-	  Change-Id: Ice16c7a4cbca4614da344aaea21a072b86263ef7
+	  If DESTDIR is set, don't call ldconfig. Assume that DESTDIR is used to
+	  create a binary archive. The ldconfig call should be delegated to the
+	  archive postinst script. This fixes the case where fakeroot wraps 'make
+	  install' causing $EUID to be 0 even though it doesn't have permission to
+	  call ldconfig.
 
-2016-08-23 11:02 +0000 [8b4b2500ee]  Richard Mudgett <rmudgett@digium.com>
+	  The previous logic in configure.ac to detect and correct libdir
+	  has been removed as it was not completely accurate.  CentOS 64-bit
+	  users should again specifiy --libdir=/usr/lib64 when configuring
+	  to prevent install to /usr/lib.
 
-	* res_fax: Fix deadlock in ast_channel_get_t38_state().
+	  Updated Makefile:check-old-libdir to check for orphans in
+	  lib64 when installing to lib as well as orphans in lib when installing
+	  to lib64.
 
-	  ast_channel_get_t38_state() calls ast_channel_queryoption() with
-	  AST_OPTION_T38_STATE.  If the passed in channel is a local channel then a
-	  deadlock can happen if a channel lock is held when called.
+	  Updated Makefile and main/Makefile uninstall targets to remove the
+	  orphans using the new logic.
 
-	  * Made ast_channel_get_t38_state() callers not hold a channel lock before
-	  calling.
+	  ASTERISK-26705
 
-	  * Update ast_channel_get_t38_state() doxygen to note that no channel locks
-	  can be held when calling the function.
+	  Change-Id: I51739d4a03e60bff38be719b8d2ead0007afdd51
 
-	  ASTERISK-26203 #close
-	  Reported by: Etienne Lessard
+2017-03-27 15:32 +0000 [f3290d6b66]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-24822 #close
-	  Reported by: David Brillert
+	* sdp: Add support for setting connection address and clean up state.
 
-	  ASTERISK-22732 #close
-	  Reported by: Richard Mudgett
+	  This change cleans up state management for media streams by moving
+	  RTP instances into their own session structure and adding additional
+	  details that are not relevant to the core (such as connection address).
+	  These can live either in the local capabilities or joint capabilities.
 
-	  Change-Id: I49fd76fa9af628b4198009b5c0b82c8b03681214
+	  The ability to set explicit connection address information for
+	  the purposes of direct media and NAT has also been added at the
+	  global and stream specific level.
 
-2016-08-23 10:39 +0000 [e8d4f40022]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26900
 
-	* res_fax: Fix deadlock setting FAXMODE channel variable.
+	  Change-Id: If7e5307239a9534420732de11c451a2705b6b681
 
-	  ASTERISK-25980 added the FAXMODE channel variable to res_fax.c.
-	  Unfortunately, it also introduced a deadlock potential because
-	  set_channel_variables() which sets FAXMODE can be called during a
-	  masquerade.  The ast_channel_get_t38_state() which gets the value used to
-	  set FAXMODE cannot be called with the channel locked.  As a result, local
-	  channels can deadlock because of how they must acquire the locks necessary
-	  to operate.
+2017-03-29 10:11 +0000 [5c1ea3ebbd]  Sean Bright <sean.bright@gmail.com>
 
-	  The intent of FAXMODE is for dialplan to know how a fax was transferred
-	  after the fax completes.  However, the previous patch sets FAXMODE to the
-	  channel's current T.38 state AFTER the fax has completed and where T.38
-	  may have already disconnected.
+	* astobj2: Prevent potential deadlocks with ao2_global_obj_release
 
-	  * Set FAXMODE based upon T.38 negotiations exchanged either with the fax
-	  applications or the fax framehooks.
+	  The ao2_global_obj_release() function holds an exclusive lock on the
+	  global object while it is being dereferenced. Any destructors that
+	  run during this time that call ao2_global_obj_ref() will deadlock
+	  because a read lock is required.
 
-	  ASTERISK-26203
-	  Reported by: Etienne Lessard
+	  Instead, we make the global object inaccessible inside of the write
+	  lock and only dereference it once we have released the lock. This
+	  allows the affected destructors to fail gracefully.
 
-	  ASTERISK-24822
-	  Reported by: David Brillert
+	  While this doesn't completely solve the referenced issue (the error
+	  message about not being able to create an IQ continues to be shown)
+	  it does solve the backtrace spew that accompanied it.
 
-	  ASTERISK-22732
-	  Reported by: Richard Mudgett
+	  ASTERISK-21009 #close
+	  Reported by: Marcello Ceschia
 
-	  Change-Id: Id525747254b64c1efe8b1b5973d52ff9719c2ae1
+	  Change-Id: Idf40ae136b5070dba22cb576ea8414fbc9939385
 
-2016-08-22 12:31 +0000 [35cf6c7702]  Richard Mudgett <rmudgett@digium.com>
+2017-03-30 10:18 +0000 [4e5cc70fb4]  Corey Farrell <git@cfware.com>
 
-	* res_fax.c: Fix deadlock in fax_gateway_indicate_t38().
+	* CEL: Remove header declarations of non-existant functions.
 
-	  fax_gateway_indicate_t38() calls ast_indicate_data() which cannot be
-	  called with any channel locks already held.  A deadlock can happen if the
-	  function is operating on a local channel.
+	  ast_cel_alloc and ast_cel_destroy do not exist in code, remove them from
+	  the headers.
 
-	  * Made fax_gateway_indicate_t38() unlock the channel before calling
-	  ast_indicate_data() since fax_gateway_indicate_t38() is always called with
-	  the channel locked.
+	  Change-Id: I99ce848e2e109e7d61771559f559b9e57973e45c
 
-	  * Made fax_gateway_indicate_t38() return void since nothing cared about
-	  its return value.
+2017-03-27 11:49 +0000 [f66edcb8b0]  Josh Roberson <josh@asteriasgi.com>
 
-	  ASTERISK-26203
-	  Reported by: Etienne Lessard
+	* cel_pgsql.c: Fix buffer overflow calling libpq
 
-	  ASTERISK-24822
-	  Reported by: David Brillert
+	  PQEscapeStringConn() expects the buffer passed in to be an
+	  adequitely sized buffer to write out the escaped SQL value string
+	  into.  It is possible, for large values (such as large values to
+	  Dial with a lot of devices) to have more than our 512+1 byte
+	  allocation and thus cause libpq to create a buffer overrun.
 
-	  ASTERISK-22732
-	  Reported by: Richard Mudgett
+	  glibc will nicely ABRT asterisk for you, citing a stack smash.
 
-	  Change-Id: I701ff2d26c5fc23e0d5a48a3fd98759a9fd09407
+	  Let's only allocate it to be as large as needed:
+	  If we have a value, then (strlen(value) * 2) + 1 (as recommended
+	  by libpq), and if we have none, just one byte to hold our null
+	  will do.
 
-2016-08-23 11:16 +0000 [50b2aa506f]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26896 #close
 
-	* res_fax.c: Add chan locked precondition comments.
+	  Change-Id: If611c734292618ed68dde17816d09dd16667dea2
 
-	  Change-Id: Ic10ae434536bbf7fb7055d6ab36cc50b8748a4e7
+2017-03-29 08:04 +0000 [e76cc51d5e]  Alexander Traud <pabstraud@compuserve.com>
 
-2016-08-23 10:42 +0000 [038cbc0215]  Richard Mudgett <rmudgett@digium.com>
+	* srtp: Allow zero as tag value for a sRTP Crypto Suite.
 
-	* ast_framehook_detach() must be called with the channel locked.
+	  ASTERISK-25490 #close
 
-	  The framehook container could become corrupted if the channel lock is not
-	  held before calling.
+	  Change-Id: I1c5fc0942c33c96d62b24203aad0f1e1a1a0131f
 
-	  Change-Id: If0a1c7ba0484ed3a191106a7516526b905952584
+2017-03-28 13:10 +0000 [2fe52174de]  George Joseph <gjoseph@digium.com>
 
-2016-08-22 15:01 +0000 [88e9d05ef7]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_config_wizard: Add 2 new parameters to help with proxy config
 
-	* ast_framehook_attach() must be called with the channel locked.
+	  Two new parameters have been added to the pjsip config wizard.
 
-	  The framehook container could become corrupted if the channel lock is not
-	  held before calling.
+	   * Setting 'sends_line_with_registrations' to true will cause the wizard
+	     to skip the creation of an identify object to match incoming request
+	     to the endpoint and instead add the line and endpoint parameters to
+	     the outbound registration object.
 
-	  Change-Id: I1a6b957a1f7b899eb29a186915f8cccab886a438
+	   * Setting 'outbound_proxy' is a shortcut for adding individual
+	     endpoint/outbound_proxy, aor/outbound_proxy and
+	     registration/outbound_proxy parameters.
 
-2016-08-24 14:42 +0000 [c9e83f6d0b]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I678e5f80765734c056620528a6d40d82736ceeb0
+	  (cherry picked from commit a827892ff77cd37912b528d9c45b446be091bbc0)
+	  (cherry picked from commit 27344675be1941d30508c6e6bd684acdd0791e1a)
 
-	* res_rtp_multicast:  Fix SEGV in ast_multicast_rtp_create_options
+2017-03-28 09:29 +0000 [7c0b12dc41]  Sean Bright <sean.bright@gmail.com>
 
-	  ast_multicast_rtp_create_options now checks for NULL or empty options
+	* alembic: Turn off execute bit on non-executable python scripts
 
-	  Change-Id: Ib845eae46a67a9787e89a87ebd1027344e5e0362
+	  Change-Id: I744c986da4a38aeff8c00837eb89de7841fbc86c
 
-2016-08-19 18:19 +0000 [cb8fd610e2]  Corey Farrell <git@cfware.com>
+2017-03-27 12:37 +0000 [3d8899bacf]  Richard Mudgett <rmudgett@digium.com>
 
-	* Fix checks for allocation debugging.
+	* Add DTLS sanity check.
 
-	  MALLOC_DEBUG should not be used to check if debugging is actually
-	  enabled, __AST_DEBUG_MALLOC should be used instead.  MALLOC_DEBUG only
-	  indicates that debugging is requested, __AST_DEBUG_MALLOC indicates it
-	  is active.
+	  Change-Id: Ib32612cf6c7ce9213a11b9cba82f630f8cd3564b
 
-	  Change-Id: I3ce9cdb6ec91b74ee1302941328462231be1ea53
+2017-03-08 07:24 +0000 [5d938045d4]  Joshua Colp <jcolp@digium.com>
 
-2016-08-10 15:14 +0000 [b8b5d52b5e]  Mark Michelson <mmichelson@digium.com>
+	* channel: Remove old epoll support and fixed max number of file descriptors.
 
-	* ConfBridge: Rework announcer channel methodology
+	  This change removes the old epoll support which has not been used or
+	  maintained in quite some time.
 
-	  NOTE: This patch was submitted earlier and reverted because of a failing
-	  test. The test has been patched so that it adjusts for the changes here,
-	  so this is being resubmitted for review.
+	  The fixed number of file descriptors on a channel has also been removed.
+	  File descriptors are now contained in a growable vector. This can be
+	  used like before by specifying a specific position to store a file
+	  descriptor at or using a new API call, ast_channel_fd_add, which adds
+	  a file descriptor to the channel and returns its position.
 
-	  One feature that confbridge has is the ability to play sounds to all
-	  participants in the conference. Prior to this commit, the algorithm for
-	  this was as follows:
+	  Tests have been added which cover the growing behavior of the vector
+	  and the new API call.
 
-	  * Grab the playback lock
-	  * Push the conference announcer channel into the bridge
-	  * Play back the sound
-	  * Pull the conference announcer channel from the bridge
-	  * Release the playback lock
+	  ASTERISK-26885
 
-	  The issue here is that the act of adding the playback channel to the
-	  bridge and removing it for each announcement is expensive. Amongst the
-	  expenses:
+	  Change-Id: I1a754b506c009b83dfdeeb08c2d2815db30ef928
 
-	  * The announcer channel is imparted into the bridge, meaning a new
-	    thread is spun up for each playback.
-	  * When the announcer is added or removed from the bridge, it results
-	    in the BRIDGEPEER channel variable being set on all channels in the
-	    bridge. This requires keeping the bridge locked and locking each
-	    individual channel in order to set it.
-	  * There's also just the general overhead of adding the channel and
-	    removing it from the bridge. The bridge potentially has to reconfigure
-	    every single time
+2017-03-27 09:35 +0000 [fd204d5c65]  Sean Bright <sean.bright@gmail.com>
 
-	  With this commit, the paradigm for playing back announcements has
-	  shifted.
+	* res_musiconhold: Document the 'format' option
 
-	  * The announcer channel is now added to the bridge when the conference
-	    is allocated, and it is hung up when the conference is destroyed.
-	  * A taskprocessor is used to queue playbacks onto the announcer channel.
-	    This keeps the behavior from before where playbacks do not overlap.
-	  * The announcer channel is no longer placed into the bridge as
-	    departable. Since we are not constantly removing the channel from
-	    the bridge, it is safe to add the channel using an independent thread
-	    and simply hang the channel up when it is time for the conference to
-	    be destroyed.
+	  ASTERISK-26086 #close
+	  Reported by: Jens Bürger
 
-	  The use of the taskprocessor for playbacks opens up the interesting
-	  possibility of having asynchronous announcements played. In this commit,
-	  however, the behavior is still exactly the same as it previously was.
+	  Change-Id: I6aab666c0bf01fd0c64d7a5bcb22fa7f5d41335e
 
-	  ASTERISK-26289
-	  Reported by Mark Michelson
+2017-03-24 07:43 +0000 [cf6a6226ab]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ica9fa4907c2f3728cdd1cf0bc564ef4eb40754a0
+	* core: Remove embedded module support
 
-2016-08-23 05:54 +0000 [d5d7cbfcfb]  Joshua Colp <jcolp@digium.com>
+	  This has not worked for some time and is no longer actively maintained.
 
-	* Revert "ConfBridge: Rework announcer channel methodology"
+	  Change-Id: I5110b0db69c152761b58fa025cb0a53b0e544d99
 
-	  This reverts commit 0cdeb2bfb0f4203384c08858951af3c77be8b9b3.
+2017-03-27 08:58 +0000 [d22c678999]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I18ba73b6d4dc0b994f4ffb01ae0b6cfad36ac636
+	* res_musiconhold: Don't chdir() when scanning MoH files
 
-2016-08-22 17:08 +0000 [c16ef02318]  Mark Michelson <mmichelson@digium.com>
+	  There doesn't appear to be any reason that we are chdir'ing in
+	  moh_scan_files, and in the event of an Asterisk crash, the core files
+	  may not get written because we have changed into a read-only directory.
 
-	* res_pjsip: Default endpoints to the "offline" status.
+	  ASTERISK-23996 #close
+	  Reported by: Walter Doekes
 
-	  A recent change attempted to optimize startup by not updating contact
-	  status. Instead, code responsible for qualifying contacts updates the
-	  status as it becomes known. The code even accounts for contacts/AORs
-	  that are not set to be qualified.
+	  Change-Id: Iac806dce01b3335963fbd62d4b4da9a65c614354
 
-	  The problem, though, is when there are no contacts associated with an
-	  endpoint. A common case is when an endpoint is set to register its
-	  contacts but has not done so yet. In this case, prior to registration,
-	  the endpoint's device state will appear to be "not in use" and hints
-	  associated with that device will appear to be "idle". In actuality, the
-	  device state and hint should both appear as "unavailable". The reason
-	  for the failure is that the optimization change made all persistent
-	  endpoint states set to "unknown".
+2017-03-23 09:48 +0000 [d5a8799c4b]  Sean Bright <sean.bright@gmail.com>
 
-	  The fix here is to change the hard-coded "unknown" to be "offline"
-	  instead. The default state will be offline until the qualifying code
-	  determines that the contact is actually online. This way, if there are
-	  no contacts at all, then the state stays as offline, and device state
-	  and hints appear correctly.
+	* res_xmpp: Use incremental backoff when a read error occurs
 
-	  ASTERISK-26269 #close
-	  Reported by nappsoft
+	  If a read error occurs, we immediately attempt a reconnect without any
+	  delay. Instead, let's sleep and backoff up to 60 seconds before we try
+	  again.
 
-	  Change-Id: Ie99b84169393983453076f5e9c0d35ff313a456a
+	  ASTERISK-24712 #close
+	  Reported by: Matthias Urlichs
 
-2016-08-20 14:51 +0000 [e54dcf4fd5]  David M. Lee <dlee@respoke.io>
+	  Change-Id: I6fe10ef4734837727437beab715e336777f13f48
 
-	* res_odbc_transaction: add dep on generic_odbc
+2017-03-24 11:29 +0000 [d08c69a9e2]  Sean Bright <sean.bright@gmail.com>
 
-	  When res_odbc_transaction depended on res_odbc, it got the generic_odbc
-	  headers and libs implicitly. Now that it no longer depends on res_odbc,
-	  its dependency on generic_odbc must be explicit.
+	* res_pjsip_sdp_rtp: Set hangup cause for RTP timeouts
 
-	  Change-Id: I9db88f7af7388437f49903d3008ba8d4890d5911
+	  chan_sip sets the hangup cause code to AST_CAUSE_REQUESTED_CHAN_UNAVAIL
+	  (44) when a channel is hung up due to an RTP timeout. So do the same
+	  when it happens with PJSIP for parity.
 
-2016-08-20 11:18 +0000 [be38c95def]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I3546ebbde6460c22a27c9da1bf321711b5961ab8
 
-	* pjproject_bundled: Allow IPv4/IPv6 (Dual Stack) configurations.
+2017-03-23 14:01 +0000 [d2f2cdf476]  Kevin Harwell <kharwell@digium.com>
 
-	  PJProject supports a lot of platforms even Windows, some with different defaults
-	  when it comes to IPv6. In many Linux platforms like Ubuntu 16.04 LTS,
-	  "/proc/sys/net/ipv6/bindv6only" is set to 0 (false). Different than in Windows.
+	* AMI: Updated version
 
-	  Because of this, if configured with just an IPv6 address/transport, PJProject
-	  listens to both IPv4 and IPv6. However, this is not supported by the PJProject
-	  team. As consequence, you end-up with IPv4-mapped IPv6 addresses in SDP,
-	  incompatible with IPv4-only clients. Technically, you end-up with an IPv6-only
-	  server which accepts incoming connections on IPv4.
+	  Updated the AMI version for the following reason (see CHANGES for more details):
 
-	  If you try to configure two transports, one with IPv4 and one with IPv6 on the
-	  same interface, as expected by the PJProject team, the IPv4 transport is not
-	  able to bind because the IPv6 transport listens to both already.
+	  The 'PJSIPShowEndpoint' command's response event of 'IdentifyDetail' now
+	  contains a new optional parameter, 'MatchHeader'.
 
-	  One solution would be to change "/proc/sys/net/ipv6/bindv6only" system-wide.
-	  Then, you are able to configure two transports, one for each IP version on the
-	  same interface. That way, you get a server which works with IPv4 clients and
-	  IPv6 clients at the same time over the same interface.
+	  Change-Id: Ie206913ef1dcfa6a2ebe3282da2387e52d6f05b9
 
-	  Here, this change sets this parameter directly within PJProject to match the
-	  expectations of the PJProject team in any case. This allows IPv4/IPv6 Dual Stack
-	  servers out of the box like in chan_sip. This change was accepted by the
-	  PJProject team as <http://trac.pjsip.org/repos/changeset/5403> and is expected
-	  to arrive in the next version, PJProject 2.6.0. Until then, this change is
-	  incorporated in the bundled PJProject of Asterisk.
+2017-03-23 12:07 +0000 [12dde3b568]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-26309
+	* pjproject_bundled: raise timeout value used when downloading
 
-	  Change-Id: I3335d8718f79f4b2feae91b5b005a3ce684a63ae
+	  After configuring Asterisk with '--with-pjproject-bundled' the configure/build
+	  process attempts to download pjproject from its download site. Currently, a
+	  timeout of 10 seconds is used that will stop the download process if pjproject
+	  has not been fully downloaded in that time. For some systems this was not enough
+	  time and the process was timing out too early.
 
-2016-08-10 15:14 +0000 [0cdeb2bfb0]  Mark Michelson <mmichelson@digium.com>
+	  This patch raises the download timeout value to '60'. Also, this patch fixes
+	  another bug where the DOWNLOAD_TIMEOUT variable was not being properly exported
+	  due to a naming error. DOWNLOAD_MAX_TIMEOUT is now properly renamed to
+	  DOWNLOAD_TIMEOUT.
 
-	* ConfBridge: Rework announcer channel methodology
+	  ASTERISK-26814 #close
 
-	  One feature that confbridge has is the ability to play sounds to all
-	  participants in the conference. Prior to this commit, the algorithm for
-	  this was as follows:
+	  Change-Id: Ia56e4e8a3d39db76bc8a1852b2cf07ec10b39842
 
-	  * Grab the playback lock
-	  * Push the conference announcer channel into the bridge
-	  * Play back the sound
-	  * Pull the conference announcer channel from the bridge
-	  * Release the playback lock
+2017-03-22 20:33 +0000 [98a88e9ffa]  Sean Bright <sean.bright@gmail.com>
 
-	  The issue here is that the act of adding the playback channel to the
-	  bridge and removing it for each announcement is expensive. Amongst the
-	  expenses:
+	* res_xmpp: Correct implementation of JABBER_STATUS & JabberStatus
 
-	  * The announcer channel is imparted into the bridge, meaning a new
-	    thread is spun up for each playback.
-	  * When the announcer is added or removed from the bridge, it results
-	    in the BRIDGEPEER channel variable being set on all channels in the
-	    bridge. This requires keeping the bridge locked and locking each
-	    individual channel in order to set it.
-	  * There's also just the general overhead of adding the channel and
-	    removing it from the bridge. The bridge potentially has to reconfigure
-	    every single time
+	  The documentation for JABBER_STATUS (and the deprecated JabberStatus
+	  app) indicate that a return value of 7 indicates that the specified
+	  buddy was not in the roster. It also indicates that you can specify a
+	  "bare" JID (one without a resource). Unfortunately the actual behavior
+	  does not match the documented behavior.
 
-	  With this commit, the paradigm for playing back announcements has
-	  shifted.
-
-	  * The announcer channel is now added to the bridge when the conference
-	    is allocated, and it is hung up when the conference is destroyed.
-	  * A taskprocessor is used to queue playbacks onto the announcer channel.
-	    This keeps the behavior from before where playbacks do not overlap.
-	  * The announcer channel is no longer placed into the bridge as
-	    departable. Since we are not constantly removing the channel from
-	    the bridge, it is safe to add the channel using an independent thread
-	    and simply hang the channel up when it is time for the conference to
-	    be destroyed.
+	  Assuming that our roster includes the buddy online and available
+	  "valid@example.org/Valid" and does *not* include the buddy
+	  "invalid@example.org", the JABBER_STATUS() function returns the
+	  following before this patch:
 
-	  The use of the taskprocessor for playbacks opens up the interesting
-	  possibility of having asynchronous announcements played. In this commit,
-	  however, the behavior is still exactly the same as it previously was.
+	  +------------------------------+------------+--------------------------+
+	  | Buddy                        | Status     | Result                   |
+	  +------------------------------+------------+--------------------------+
+	  | valid@example.org            |  Online    |  7 (Not in roster)       |
+	  | valid@example.org/Valid      |  Online    |  1 (Online)              |
+	  | valid@example.org/Invalid    |  N/A       |  7 (Not in roster)       |
+	  | invalid@example.org          |  N/A       |  Error logged, no return |
+	  | invalid@example.org/Valid    |  N/A       |  Error logged, no return |
+	  +------------------------------+------------+--------------------------+
 
-	  ASTERISK-26289
-	  Reported by Mark Michelson
+	  And after this patch:
 
-	  Change-Id: Ic5cd2c4b98a1eaa1715eb7a5b35d62f1a76d78a5
+	  +------------------------------+------------+--------------------------+
+	  | Buddy                        | Status     | Result                   |
+	  +------------------------------+------------+--------------------------+
+	  | valid@example.org            |  Online    |  1 (Online)              |
+	  | valid@example.org/Valid      |  Online    |  1 (Online)              |
+	  | valid@example.org/Invalid    |  N/A       |  6 (Offline)             |
+	  | invalid@example.org          |  N/A       |  7 (Not in roster)       |
+	  | invalid@example.org/Valid    |  N/A       |  7 (Not in roster)       |
+	  +------------------------------+------------+--------------------------+
 
-2016-08-19 10:21 +0000 [b494b9f88c]  Alexei Gradinari <alex2grad@gmail.com>
+	  This brings the behavior in line with the documentation.
 
-	* compilation failed with -Werror=maybe-uninitialized
+	  ASTERISK-23510 #close
+	  Reported by: Anthony Critelli
 
-	  The compilation failed for devmode
-	  --enable DONT_OPTIMIZE
-	  --enable BETTER_BACKTRACES
-	  --enable DO_CRASH
-	  --enable TEST_FRAMEWORK
+	  Change-Id: I9c3241035363ef4a6bdc21fabfd8ffcd9ec657bf
 
-	  res_pjsip/pjsip_configuration.c: In function dtls_handler:
-	  res_pjsip/pjsip_configuration.c:974:20: error:
-	  back may be used uninitialized in this function [-Werror=maybe-uninitialized]
-	  int size = strlen(front);
-	             ^
-	  cc1: all warnings being treated as errors
+2017-03-23 09:45 +0000 [be94105d6d]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I7f082ead0312792a577ec7c73015ba64dabca580
+	* res_xmpp: Try to provide useful errors messages from OpenSSL
 
-2016-08-19 03:59 +0000 [a628009eb9]  Alexander Traud <pabstraud@compuserve.com>
+	  If any errors occur during the TLS connection setup, we currently dump a
+	  fairly generic error message. So instead we try to pull in something
+	  useful from OpenSSL to report instead.
 
-	* sip_to_pjsip: Add cert_file.
+	  ASTERISK-24712
+	  Reported by: Matthias Urlichs
 
-	  When using the migration script sip_to_pjsip.py, cert_file was not migrated to
-	  pjsip.conf. A previous change regarding this contained a copy/paste error.
+	  Change-Id: I288500991a9681f447d92913b11fedaf426087f4
 
-	  ASTERISK-22374
+2017-03-23 05:19 +0000 [ee81ee1f14]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I0fa72e9412117d53b4284fc6b83fa5b2b95ba03b
+	* res_xmpp: Fix ref counting issue
 
-2016-08-18 09:21 +0000 [b1fe070d0b]  Alexander Traud <pabstraud@compuserve.com>
+	  The only remaining reference to the endpoint is in the endpoints
+	  container, and because it is unlinked in ast_endpoint_shutdown, we don't
+	  have to explicitly cleanup the endpoint ourselves.
 
-	* sip.conf: tlsclientmethod is using sslv23 as default.
+	  Change-Id: I912a2692e52d3e2ed445b32d8ae3f9004bc2f2e8
 
-	  When 'tlsclientmethod' is not specified in sip.conf, chan_sip uses the OpenSSL
-	  SSLv23_method. This was documented incorrectly in the file sip.conf.sample.
+2017-03-23 09:30 +0000 [9493981419]  Sean Bright <sean.bright@gmail.com>
 
-	  SSLv23_method got its name in the 90s. Today, with OpenSSL 1.0.2, this method
-	  enables (just) the secure TLSv1.0 and TLSv1.2. Or stated differently, that
-	  function should have been called 'secure_method' or 'automatic_method' back in
-	  the 90s.
+	* res_xmpp: Correctly check return value of SSL_connect
 
-	  Consequently please, specify 'tlsclientmethod=tlsv1' in your sip.conf only if
-	  you face a server which has problems like not falling back to TLSv1.0
-	  automatically.
+	  SSL_connect returns non-zero for both success and some error conditions
+	  so simply negating is inadequate.
 
-	  ASTERISK-24425
+	  Change-Id: Ifbf882896e598703b6c615407fa456d3199f95b1
 
-	  Change-Id: I502ce6146b4504cadfd3973af8d6ec3994f54fa3
+2017-03-22 17:32 +0000 [7657c279b5]  Sean Bright <sean.bright@gmail.com>
 
-2016-08-18 17:16 +0000 [ff2378c735]  Kevin Harwell <kharwell@digium.com>
+	* res_xmpp: Don't crash when trying to send a message without a connection
 
-	* rest-api: Swagger scripts were not replacing format variable in file brief
+	  If we never establish a connection to our Jabber server, iksemel never sets up
+	  its internal transport pointer, so attempting to send a message dereferences a
+	  NULL pointer and causes a crash.
 
-	  Given resource paths did not have 'json' substituted in for the '{format}'. For
-	  some auto generated documentation/comment strings it resulted in something like
-	  the following:
+	  ASTERISK-21855 #close
+	  Reported by: Jeremy Kister
 
-	  "... REST handler for /api-docs/sounds.{format}"
+	  Change-Id: I204a568894e4a53ab929783ecc594a000f04d79c
 
-	  This patch makes sure the resource api's path is properly substituted.
+2017-03-22 15:40 +0000 [0136ec12a3]  Sean Bright <sean.bright@gmail.com>
 
-	  ASTERISK-25472 #close
+	* res_xmpp: Include client name in connection related error messages
 
-	  Change-Id: Ie3e950a35db4043e284019d6c9061f3b03922e23
+	  ASTERISK-25622 #close
+	  Reported by: Sean Darcy
 
-2016-08-16 15:57 +0000 [43f400ef95]  Jason Parker (license 4993)
+	  Change-Id: I8472cb7bfb58d411a3cfbd482da98cae2d94d1e9
 
-	* res_format_attr_g729: Add annexb=no format parameter to SDPs
+2017-03-20 13:27 +0000 [9b103e7bea]  Kevin Harwell <kharwell@digium.com>
 
-	  Historically, Asterisk has always specified annexb=no for the g729 format.
-	  However, when using res_pjsip no format attribute was specified. This patch
-	  makes it so the SDP now contains a format attribute line with annexb=no.
+	* rtp_engine: allocate RTP dynamic payloads per session
 
-	  Note, that this means only g729a is negotiated. Even for pass through support.
-	  According to rfc7261 the type of annex used (a or b) is dependent upon the
-	  answerer. However, Asterisk being a back to back user agent makes this tricky
-	  to support at this time, thus we only allow annex 'a' for now.
+	  Dynamic payload types were statically defined in Asterisk. This unfortunately
+	  limited the number of dynamic payloads that could be registered. With this patch
+	  dynamic payload type numbers are now assigned dynamically and per RTP instance.
+	  However, in order to limit any issues where some clients expect the old
+	  statically defined value this patch makes it so the value Asterisk used to pre-
+	  designate is used for the dynamic assignment if available.
+
+	  An option, "rtp_use_dynamic", has also been added (can be set in asterisk.conf)
+	  that turns the new dynamic behavior on or off. When off it reverts back to using
+	  statically defined payload values. This option defaults to "yes" in Asterisk 15.
 
-	  ASTERISK-26228 #close
+	  ASTERISK-26515 #close
 	  patches:
-	    res_format_attr_g729.c submitted by Jason Parker (license 4993)
-
-	  Change-Id: I76bc20cc0a01af01536e9915afef319c269c22d0
+	    ASTERISK-26515.diff submitted by jcolp (license 5000
 
-2016-08-18 15:15 +0000 [4c1ae07d51]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I7653465c5ebeaf968f1a1cc8f3f4f5c4321da7fc
 
-	* res_odbc:  Correct the dependency relationship with res_odbc_transaction
-
-	  The MODULEINFO dependencies between these 2 modules was reversed.
-	  res_odbc should depend on res_odbc_transaction, not the other way
-	  around.
+2017-03-21 12:32 +0000 [bb2936f3e4]  Sebastian Gutierrez <sgutierrez@integraccs.com>
 
-	  ASTERISK-25984 #close
+	* cdr: Allow setting of user field from 'h' extension
 
-	  Change-Id: Ifcfbb49c0b51cf6640a5446d47cd6c48caf1331f
+	  The CDR code previously did not allow the user field to be set
+	  from the 'h' extension in the dialplan. This change removes that
+	  limitation and allows it to be set.
 
-2016-08-18 12:04 +0000 [cab6975b02]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-26818
 
-	* sip_to_pjsip: Set correct tls transport method
+	  Change-Id: I0fed8a79b5e408bac4e30542b8f33a61c5ed9aa6
 
-	  A recent update had a copy/paste error where the unused variable 'val' was
-	  being passed to the set_value function instead of the 'method' value itself.
+2017-03-14 16:45 +0000 [6b7697ed48]  Richard Begg <asterisk@meric.id.au>
 
-	  This patch passes in the right variable.
+	* res_pjsip_session: Enable RFC3578 overlap dialing support.
 
-	  ASTERISK-22374
+	  Support for RFC3578 overlap dialling (i.e. 484 Response to partially matched
+	  destinations) as currently provided by chan_sip is missing from res_pjsip.
+	  This patch adds a new endpoint attribute (allow_overlap) [defaults to yes]
+	  which when set to yes enables 484 responses to partial destination
+	  matches rather than the current 404.
 
-	  Change-Id: I895b7b3779ce4442bc58b8ec40d59dd29bb43f06
+	  ASTERISK-26864
 
-2016-08-18 08:19 +0000 [2381ddde63]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Iea444da3ee7c7d4f1fde1d01d138a3d7b0fe40f6
 
-	* sip_to_pjsip: Map the TLS method correctly.
+2017-03-21 06:59 +0000 [d4fcf196a2]  Sean Bright <sean.bright@gmail.com>
 
-	  When using the migration script sip_to_pjsip.py and tlsclientmethod is not set
-	  in sip.conf, the default value of chan_sip (sslv23) is copied to pjsip.conf, to
-	  overwrite the default of the PJProject (tlsv1). This makes sure, res_pjsip is
-	  offering/using not just TLSv1.0 but TLSv1.2 as well.
+	* res_hep: Capture actual transport type in use
 
-	  ASTERISK-22374
+	  Rather than hard-coding UDP, allow consumers of the HEP API to specify
+	  which protocol is in use. Update the PJSIP provider to pass in the
+	  current protocol type.
 
-	  Change-Id: Ie530a3dae9926ae14f3920a21be1e2edb15bda4f
+	  ASTERISK-26850 #close
 
-2016-08-18 08:17 +0000 [6500f5e138]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I54bbb0a001cfe4c6a87ad4b6f2014af233349978
 
-	* sip_to_pjsip: Add compactheaders, timerb, timert1, and useragent.
+2017-03-21 09:57 +0000 [1bf839d44b]  Sean Bright <sean.bright@gmail.com>
 
-	  When using the migration script sip_to_pjsip.py, no section of type=system or
-	  type=general were created. Therefore the keys compactheaders, timerb, timert1,
-	  and useragent were not migrated to pjsip.conf.
+	* Revert "app_queue: Handle the caller being redirected out of a queue bridge"
 
-	  ASTERISK-22374
+	  This reverts commit 163e9e53dc7d84dd42721e733b7706c8147bdd27.
 
-	  Change-Id: I318a453843227ea36bf130d392d4abd7bd26b5a1
+	  Change-Id: Ief28479c77a298879dfe2c56be7ee92dc465da4b
 
-2016-08-18 08:16 +0000 [21e9c69e56]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-21 08:26 +0000 [6b4b87787c]  Sean Bright <sean.bright@gmail.com>
 
-	* sip_to_pjsip: Map (session-)timers correctly.
+	* res_pjsip_messaging: Check URI type before dereferencing
 
-	  When using the migration script sip_to_pjsip.py, session-timers=accept and
-	  session-timers=refuse were mapped to wrong values.
+	  We aren't validating that the URI we just parsed is a SIP/SIPS one before
+	  trying to access the user, host, and port members of a possibly uninitialized
+	  structure.
 
-	  ASTERISK-22374
+	  Also update the MessageSend documentation to indicate what 'from' formats are
+	  accepted.
 
-	  Change-Id: Ie4e90d5f6a29aff07837b7fe5bc8aea5fb6fc092
+	  ASTERISK-26484 #close
+	  Reported by: Vinod Dharashive
 
-2016-08-18 08:15 +0000 [c9a97398f7]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: I476b5cc5f18a7713d0ee945374f2a1c164857d30
 
-	* sip_to_pjsip: Write username even without authname.
+2017-03-13 15:21 +0000 [65ad554c98]  Joshua Elson <joshelson@gmail.com>
 
-	  When using the migration script sip_to_pjsip.py, now the (mandatory) username is
-	  written to pjsip.conf, even if there was no (optional) authname in the register
-	  string in sip.conf.
+	* pjsip: prevent memory corruption on creation of xml bodies
 
-	  ASTERISK-22374
+	  ASTERISK-26776 #close
 
-	  Change-Id: Ie53e1997104cd2674821688b8a8247249f5e156f
+	  Change-Id: I884b6f4e8233a355d0be687ec78d41bc0e4d3fd2
 
-2016-08-18 08:14 +0000 [60275359bc]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-20 16:27 +0000 [fc794de756]  Sean Bright <sean.bright@gmail.com>
 
-	* sip_to_pjsip: Parse register even with transport.
+	* bridge_softmix: Ignore non-voice frames from translator
 
-	  When using the migration script sip_to_pjsip.py and the register string
-	  started with a transport in sip.conf - like tls://... - register was not parsed
-	  correctly and therefore not migrated correctly to pjsip.conf.
+	  Some codecs - codec_speex specifically - take voice frames and return
+	  other types of frames, like CNG. If we subsequently treat those as
+	  voice frames, we'll run into trouble when destroying the frame because
+	  of the requirement that each voice frame have an associated format.
 
-	  ASTERISK-22374
+	  ASTERISK-26880 #close
+	  Reported by: Kirsty Tyerman
 
-	  Change-Id: I44c12104eea2bd8558ada6d25d77edfecd92edd2
+	  Change-Id: I43f8450c48fb276ad8b99db8512be82949c1ca7c
 
-2016-08-18 08:13 +0000 [0d479232eb]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-14 23:49 +0000 [25016a74f8]  Aaron An <anjb@ti-net.com.cn>
 
-	* sip_to_pjsip: Write local_net, contact_acl, contact_deny, and contact_permit.
+	* audiohook.c: Lost RTP packets lead to out-of-sync MixMonitor.
 
-	  When using the migration script sip_to_pjsip.py, those keys got missing. These
-	  keys might appear several times and the function "merge_value" tried to collect
-	  those. However, because these keys have different names in sip.conf and
-	  pjsip.conf, "merge_value" was not able to find the new key name in sip.conf.
-	  This change lets "merge_value" search with the old key name in sip.conf and
-	  write with the new key name in pjsip.conf.
+	  Fixed a bug in function "ast_audiohook_write_frame" that checked the
+	  variable other_factory_samples and only flushed the factories, so they
+	  would be in sync, when other_factory_samples > 0. When there is not any
+	  rtp incoming the variable other_factory_samples will be 0, and although
+	  the result of "our_factory_ms - other_factory_ms" may be very large,
+	  this led to the record file not syncing.
 
-	  ASTERISK-22374
+	  ASTERISK-26875 #close
+	  Reported-by: Aaron An
+	  Tested-by: Aaron An
 
-	  Change-Id: Ie53c5278ae6f1cb8fa7e96c5289877d46981d9d2
+	  Change-Id: Ia4d890fb8fc1636a7188502bab35f555685aea22
 
-2016-08-18 08:11 +0000 [cbc1b2d020]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-18 12:30 +0000 [fc71c18a9b]  Sean Bright <sean.bright@gmail.com>
 
-	* sip_to_pjsip: Map externhost/ip to Transports.
+	* thread safety: Don't use getprotobyname()
 
-	  When using the migration script sip_to_pjsip.py, the externhost or externip of
-	  sip.conf were erroneously written to Endpoints instead to Transports.
+	  POSIX does not require getprotobyname() to be thread safe and some
+	  implementations use static memory which causes issues when multiple
+	  threads are used.
 
-	  ASTERISK-22374
+	  Further, our usage of it today is just to ultimately get IPPROTO_TCP
+	  for calls to setsockopt(). So instead we just use IPPROTO_TCP directly.
 
-	  Change-Id: I2c5873386cfc388899fa9cf2368639dd12f1b8e4
+	  Change-Id: I2e14e58674808f7ce99b2f5e900d0f90d0d8da48
 
-2016-08-18 08:04 +0000 [5f33e99534]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-19 13:26 +0000 [516e028b44]  Sean Bright <sean.bright@gmail.com>
 
-	* sip_to_pjsip: Add defaultexpiry, maxexpiry, and minexpiry.
+	* res_rtp_asterisk: Pass correct data length to ast_rtcp_interpret
 
-	  When using the migration script sip_to_pjsip.py, defaultexpiry, maxexpiry, and
-	  minexpiry were not migrated to pjsip.conf.
+	  We are currently passing in the capacity of the read buffer instead of the
+	  number of bytes that we actually read off the wire.
 
-	  ASTERISK-22374
+	  Change-Id: I60465049727d955c7f9a5e529e6f2aaff04cda36
 
-	  Change-Id: I007fbf543dcadc96fc3ed71c54da502bcb209b7b
+2017-03-14 09:27 +0000 [79069f8ccb]  Robert Mordec <r.mordec@slican.pl>
 
-2016-08-18 08:03 +0000 [231ea0350d]  Alexander Traud <pabstraud@compuserve.com>
+	* app_queue: Member stuck as pending after forwarding previous call from queue
 
-	* sip_to_pjsip: Write media_encryption.
+	  Queue member will get stuck in pending_members if queue calls a device
+	  that is different from the one observed for state changes.
 
-	  When using the migration script sip_to_pjsip.py, encryption=yes got missing and
-	  media_encryption=sdes was not written to pjsip.conf, because of a typo.
+	  This patch removes members from pending_members as a result of channel stasis
+	  events such as blind or attended transfers and hangup.
 
-	  ASTERISK-22374
+	  ASTERISK-26862 #close
 
-	  Change-Id: I0fc3e55dc512a57603ae0fef41baacccf2a35c05
+	  Change-Id: I8bf6df487b9bb35726c08049ff25cdad5e357727
 
-2016-08-18 08:02 +0000 [23eb065121]  Alexander Traud <pabstraud@compuserve.com>
+2017-02-22 23:26 +0000 [8cb4f9cea1]  Richard Mudgett <rmudgett@digium.com>
 
-	* sip_to_pjsip: Write cos and tos.
+	* CHANNEL(callid): Give dialplan access to the callid.
 
-	  When using the migration script sip_to_pjsip.py, both tos_sip and cos_sip got
-	  missed, because of a typo. Therefore, cos and tos were not written to
-	  pjsip.conf. Furthermore, that revealed a misuse of an internal function, caused
-	  by a copy-and-paste error.
+	  * Added CHANNEL(callid) to retrieve the call identifier log tag associated
+	  with the channel.  Dialplan now has access to the call log search key
+	  associated with the channel so it can be saved in case there is a problem
+	  with the call.
 
-	  ASTERISK-22374
+	  ASTERISK-26878
 
-	  Change-Id: Id245ebadf70ab9776eb280c026288540af3af5c2
+	  Change-Id: I2c97ebd928b6f3c5bc80c5729e4d3c07f453049f
 
-2016-08-18 07:55 +0000 [0b675a208b]  Alexander Traud <pabstraud@compuserve.com>
+2017-03-16 08:42 +0000 [c13ea6080e]  Sean Bright <sean.bright@gmail.com>
 
-	* sip_to_pjsip: Add cert_file and ca_list_path.
+	* app_queue: Fix locking behavior in stasis message handlers
 
-	  When using the migration script sip_to_pjsip.py, cert_file and ca_list_path were
-	  not migrated to pjsip.conf.
+	  The queue_stasis_data structure contains various mutable fields that require
+	  appropriate locking. Specifically, the 'dying,' 'member_uniqueid,' and
+	  'caller_uniqueid' fields need to be locked when read from or written to.
 
-	  ASTERISK-22374
+	  Change-Id: I246b7dbff8447acc957a1299f6ad0ebd0fd39088
 
-	  Change-Id: I4612877d190b7f86a48698cefbf5c4db6c265825
+2017-03-07 19:28 +0000 [15aa3c0a23]  Sean Bright <sean.bright@gmail.com>
 
-2016-08-17 14:13 +0000 [1cd12d73a6]  Richard Mudgett <rmudgett@digium.com>
+	* chan_sip: Add rtcp-mux support
 
-	* res_pjsip_session.c: Fix unbound srv failover tests.
+	  ASTERISK-26846 #close
 
-	  Commit 1b666549f33d69dc080b212bf92126f3bc3a18b2 broke the srv failover
-	  functionality if a TCP connection gets disconnected.  Under these
-	  conditions, session_inv_on_state_changed() gets a
-	  PJSIP_EVENT_TRANSPORT_ERROR and restarts the INVITE transaction on a new
-	  transport.  Unfortunately, session_inv_on_tsx_state_changed() also gets
-	  the same PJSIP_EVENT_TRANSPORT_ERROR event and unconditionally terminates
-	  the session.
+	  Change-Id: I541a1602ff55ab73684e9f8002edb9e0e745d639
 
-	  * Made session_inv_on_tsx_state_changed() complete terminating the session
-	  on PJSIP_EVENT_TRANSPORT_ERROR only if the session state is still
-	  PJSIP_INV_STATE_DISCONNECTED.
+2017-03-16 16:50 +0000 [57656e2b5b]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26305 #close
-	  Reported by: Richard Mudgett
+	* app_confbridge: Fix ConfbridgeTalking AMI event description.
 
-	  Change-Id: If736e766b5c55b970fa38ca6c8a885caf27b897d
+	  Thanks to Chris Howard for pointing this out on the wiki.
 
-2016-08-16 15:36 +0000 [329507fe20]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: I18e56de09a70e736b5d04719d45ef29cf0636705
 
-	* res_pjsip:  Add contact_user to endpoint
+2017-03-16 16:37 +0000 [82982a191c]  Richard Mudgett <rmudgett@digium.com>
 
-	  contact_user, when specified on an endpoint, will override the user
-	  portion of the Contact header on outgoing requests.
+	* res_pjsip_asterisk.c: Fix compile error if libsrtp is not installed.
 
-	  Change-Id: Icd4ebfda2f2e44d3ac749d0b4066630e988407d4
+	  struct ast_rtcp does not define the dtls member if SRTP is not enabled.
 
-2016-08-17 08:10 +0000 [6f448f32fe]  Torrey Searle <torrey@voxbone.com>
+	  ASTERISK-26732
 
-	* res_ari: Add http prefix to generated docs
+	  Change-Id: Id15ea212e04490e012f2cf4a56818b4dd948875e
 
-	  updated the uri handler to include the url prefix of the http server
-	  this enables res_ari to add it to the uris when generating docs
+2017-03-16 15:45 +0000 [49b1f1ca16]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I279335a2625261a8492206c37219698f42591c2e
+	* res_pjsip_sdp_rtp.c: Fix cut-n-paste error
 
-2016-08-17 06:12 +0000 [56e0aed177]  Alexander Traud <pabstraud@compuserve.com>
+	  We were inadvertenly referencing the cos_video option to determine if we
+	  should set the tos_audio and cos_audio value on the RTP instance.
 
-	* BuildSystem: Detect ca_list_path capabilities in external PJProject.
+	  Change-Id: Ia7964f486801d39dc6f5dae570baff079e1595b0
 
-	  Since Asterisk 13.8, pj_ssl_cert_load_from_files2 got detected only in the
-	  bundled PJProject but not in an external PJProject. Therefore, ca_list_path
-	  could not be used in pjsip.conf. With this change, pj_ssl_cert_load_from_files2
-	  is detected again to enable ca_list_path again.
+2017-03-16 10:39 +0000 [e6dc28b78f]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-26303 #close
+	* res/res_pjsip_session: Only check localnet if it is defined
 
-	  Change-Id: I4a4a0cdc5cdff33730911fb4cfc0498c069043d0
+	  If local_net is not defined on a transport, transport_state->localnet
+	  will be NULL. ast_apply_ha will, be default, return AST_SENSE_ALLOW in
+	  this case, causing the external_media_address, if set, to be skipped.
 
-2016-08-16 12:24 +0000 [2edcfcf1eb]  gtjoseph <gjoseph@digium.com>
+	  This patch causes us to only check if we are sending within a network if
+	  local_net is defined.
 
-	* ari:  Add documentation that path parameters are case-sensitive
+	  ASTERISK-26879 #close
 
-	  Added to api.wiki.mustache so that the generated object pages
-	  have the notation in the table header as well as under each method
-	  that has path parameters.
+	  Change-Id: Ib661c31a954cabc9c99f1f25c9c9a5c5b82cbbfb
 
-	  ASTERISK-25492 #close
+2017-03-14 16:22 +0000 [44568fc712]  Richard Begg <asterisk@meric.id.au>
 
-	  Change-Id: I36c46c6dc0c9ac350470394a999a1b19ef3fcdaf
+	* res_pjsip_sdp_rtp: RTP instance does not use same IP as explicit transport
 
-2016-08-15 15:29 +0000 [f4e28b3a09]  Corey Farrell <git@cfware.com>
+	  Currently a wildcard address is used for the local RTP socket, which
+	  will not always result in the same address as used by the SIP socket
+	  (e.g. if explicit transport addresses are configured).
+	  Use the transport's host address when binding new local RTP sockets if
+	  available.
 
-	* Refactor usage pattern of xmldoc info tag.
+	  ASTERISK-26851
 
-	  This updates func_channel.c and main/message.c to use a generic xpointer
-	  include instead of including info from each channel driver.  Now the
-	  name attribute of info is CHANNEL or CHANNEL_EXAMPLES to be included in
-	  documentation for func_channel.  Setting the name attribute of info to
-	  MessageToInfo or MessageFromInfo causes it to be included in the
-	  MessageSend application and AMI action.
+	  Change-Id: I098c29c9d1f79a4f970d72ba894874ac75954f1a
 
-	  Change-Id: I89fd8276a3250824241a618009714267d3a8d1ea
+2017-03-07 08:33 +0000 [5013d8f5d3]  George Joseph <gjoseph@digium.com>
 
-2016-08-04 20:00 +0000 [a8d9a53bae]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip:  Symmetric transports
 
-	* res_sorcery_config.c: Cleanup ao2 container usage idioms.
+	  A new transport parameter 'symmetric_transport' has been added.
 
-	  Change-Id: Iad24b335fb121a2bc7f1d048ab7420569edcba5a
+	  When a request from a dynamic contact comes in on a transport with
+	  this option set to 'yes', the transport name will be saved and used
+	  for subsequent outgoing requests like OPTIONS, NOTIFY and INVITE.
+	  It's saved as a contact uri parameter named 'x-ast-txp' and will
+	  display with the contact uri in CLI, AMI, and ARI output.  On the
+	  outgoing request, if a transport wasn't explicitly set on the
+	  endpoint AND the request URI is not a hostname, the saved transport
+	  will be used and the 'x-ast-txp' parameter stripped from the
+	  outgoing packet.
 
-2016-08-04 15:57 +0000 [74a91b9ee5]  Richard Mudgett <rmudgett@digium.com>
+	  * config_transport was modified to accept and store the new parameter.
 
-	* sorcery.c: Minor optimizations.
+	  * config_transport/transport_apply was updated to store the transport
+	    name in the pjsip_transport->info field using the pjsip_transport->pool
+	    on UDP transports.
 
-	  * Remove some unused parameters from internal functions:
-	  sorcery_wizard_create()
-	  sorcery_wizard_update()
-	  sorcery_wizard_delete()
+	  * A 'multihomed_on_rx_message' function was added to
+	    pjsip_message_ip_updater that, for incoming requests, retrieves the
+	    transport name from pjsip_transport->info and retrieves the transport.
+	    If transport->symmetric_transport is set, an 'x-ast-txp' uri parameter
+	    containing the transport name is added to the incoming Contact header.
 
-	  * Created the struct sorcery_observer_invocation ao2 object without a lock
-	  since it is not needed in sorcery_observer_invocation_alloc().
+	  * An 'ast_sip_get_transport_name' function was added to res_pjsip.
+	    It takes an ast_sip_endpoint and a pjsip_sip_uri and returns a
+	    transport name if endpoint->transport is set or if there's an
+	    'x-ast-txp' parameter on the uri and the uri host is an ipv4 or
+	    ipv6 address.  Otherwise it returns NULL.
 
-	  * Cleanup generic ao2 container sorcery object id hash, sort, and cmp
-	  functions.
+	  * An 'ast_sip_dlg_set_transport' function was added to res_pjsip
+	    which takes an ast_sip_endpoint, a pjsip_dialog, and an optional
+	    pjsip_tpselector.  It calls ast_sip_get_transport_name() and if
+	    a non-NULL is returned, sets the selector and sets the transport
+	    on the dialog.  If a selector was passed in, it's updated.
 
-	  Change-Id: Iff71d75f52bc1b8cee955456838c149faaa4f92e
+	  * res_pjsip/ast_sip_create_dialog_uac and ast_sip_create_dialog_uas
+	    were modified to call ast_sip_dlg_set_transport() instead of their
+	    original logic.
 
-2016-08-01 11:04 +0000 [29beb2890c]  Richard Mudgett <rmudgett@digium.com>
+	  * res_pjsip/create_out_of_dialog_request was modified to call
+	    ast_sip_get_transport_name() and pjsip_tx_data_set_transport()
+	    instead of its original logic.
 
-	* sorcery.c: Tweak some container declaration formatting.
+	  * Existing transport logic was removed from endpt_send_request
+	    since that can only be called after a create_out_of_dialog_request.
 
-	  * Tweak sorcery_object_type_alloc() formatting.
-	  * Tweak ast_sorcery_init() formatting.
+	  * res_pjsip/ast_sip_create_rdata was converted to a wrapper around
+	    a new 'ast_sip_create_rdata_with_contact' function which allows
+	    a contact_uri to be specified in addition to the existing
+	    parameters.  (See below)
 
-	  Change-Id: Ib02430023f15268cd7a2ea53f2c331213e4d3944
+	  * res_pjsip_pubsub/internal_pjsip_evsub_send_request was eliminated
+	    since all it did was transport selection and that is now done in
+	    ast_sip_create_dialog_uac and ast_sip_create_dialog_uas.
 
-2016-08-11 23:30 +0000 [9b822293bd]  Corey Farrell <git@cfware.com>
+	  * 'contact_uri' was added to subscription_persistence.  This was
+	    necessary because although the parsed rdata contact header has the
+	    x-ast-txp parameter added (if appropriate),
+	    subscription_persistence_update stores the raw packet which
+	    doesn't have it.  subscription_persistence_recreate was then
+	    updated to call ast_sip_create_rdata_with_contact with the
+	    persisted contact_uri so the recreated subscription has the
+	    correct transport info to send the NOTIFYs.
 
-	* pbx.c: Additional fixes to ast_context_remove_extension_callerid2.
+	  * res_pjsip_session/internal_pjsip_inv_send_msg was eliminated since
+	    all it did was transport selection and that is now done in
+	    ast_sip_create_dialog_uac.
 
-	  Do not check registrar of the first extension head.  We should only check
-	  the registrar when we match the priority.
+	  * pjsip_message_ip_updater/multihomed_on_tx_message was updated
+	    to remove all traces of the x-ast-txp parameter from the
+	    outgoing headers.
 
-	  Additionally fix a couple calls to strcmp which used the input callerid
-	  instead of the clean version ex.cidmatch.
+	  NOTE:  This change does NOT modify the behavior of permanent
+	  contacts specified on an aor.  To do so would require that the
+	  permanent contact's contact uri be updated with the x-ast-txp
+	  parameter and the aor sorcery object updated.  If we need to
+	  persue this, we need to think about cloning permanent contacts into
+	  the same store as the dynamic ones on an aor load so they can be
+	  updated without disturbing the originally configured value.
 
-	  ASTERISK-26233
+	  You CAN add the x-ast-txp parameter to a permanent contact's uri
+	  but it would be much simpler to just set endpoint->transport.
 
-	  Change-Id: I17ea6881a18f40840ae9c1f5394aab1fbb3769f1
+	  Change-Id: I4ee1f51473da32ca54b877cd158523efcef9655f
 
-2016-08-10 14:41 +0000 [403c794684]  Alexei Gradinari <alex2grad@gmail.com>
+2017-03-16 09:07 +0000 [68749a9fa7]  Joshua Colp <jcolp@digium.com>
 
-	* core: Entity ID is not set or invalid
+	* res_rtp_asterisk: Fix crash when RTCP is not present when DTLS is stopped.
 
-	  The Exchanging Device and Mailbox States could not working
-	  if the Entity ID (EID) is not set manually and can't be obtained
-	  from ethernet interface.
+	  This change removes an assumption that when DTLS is stopped
+	  an RTCP session will be present on the RTP session. This is not
+	  always the case.
 
-	  This patch replaces debug message to warning
-	  and addes missing description about option 'entityid' to
-	  asterisk.conf.sample.
+	  ASTERISK-26732
 
-	  With this patch the asterisk also:
-	  (1) decline loading the modules which won't work without EID:
-	      res_corosync and res_pjsip_publish_asterisk.
-	  (2) warn if EID is empty on loading next modules:
-	      pbx_dundi, res_xmpp
+	  Change-Id: Ib9f7c09ce0b005efe362dbcc8795202b18f94611
 
-	  Starting with v197 systemd/udev will automatically assign "predictable"
-	  names for all local Ethernet interfaces.
-	  This patch also addes some new ethernet prefixes "eno" and "ens".
+2017-03-15 13:24 +0000 [c87e7dd9ec]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26164 #close
+	* autochan/mixmonitor/chanspy: Fix unsafe channel locking and references.
 
-	  Change-Id: I72d712f1ad5b6f64571bb179c5cb12461e7c58c6
+	  Dereferencing struct ast_autochan.chan without first calling
+	  ast_autochan_channel_lock() is unsafe because the pointer could change at
+	  any time due to a masquerade.  Unfortunately, ast_autochan_channel_lock()
+	  itself uses struct ast_autochan.chan unsafely and can result in a deadlock
+	  if the original channel happens to get destroyed after a masquerade in
+	  addition to the pointer getting changed.
 
-2016-06-15 17:10 +0000 [93332cb1d0]  Evgeniy Tsybra <cjack@yandex.ru>
+	  The problem is more likely to happen with v11 and earlier because
+	  masquerades are used to optimize out local channels on those versions.
+	  However, it could still happen on newer versions if the channel is
+	  executing a dialplan application when the channel is transferred or
+	  redirected.  In this situation a masquerade still must be used.
 
-	* chan_sip: Fix lastrtprx always updated
+	  * Added a lock to struct ast_autochan to safely be able to use
+	  ast_autochan.chan while trying to get the channel lock in
+	  ast_autochan_channel_lock().  The locking order is the channel lock then
+	  the autochan lock.  Locking in the other direction requires deadlock
+	  avoidance.
 
-	  Packets are read regulary, when there is no data in buffer fr->frametype
-	  is AST_FRAME_NULL. There was no check of frametype and lastrtprx always 
-	  updated and, therefore, rtptimeout did not work at all.
+	  * Fix unsafe ast_autochan.chan usages in app_mixmonitor.c.
 
-	  ASTERISK-25270 #close
+	  * Fix unsafe ast_autochan.chan usages in app_chanspy.c.
 
-	  Change-Id: If3b5ca0dbb822582a86eb7d01dcae4e83448c41d
+	  * app_chanspy.c: Removed unused autochan parameter from next_channel().
 
-2016-08-15 07:17 +0000 [2735ec899a]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-26867
 
-	* manager: Clarify that dialplan manipulation actions are under system class.
+	  Change-Id: Id29dd22bc0f369b44e23ca423d2f3657187cc592
 
-	  ASTERISK-26246 #close
+2017-03-07 14:13 +0000 [10fa49e327]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: Id673b9786389f9d2a87f638ce1a25161f5f31657
+	* Add rtcp-mux support
 
-2016-08-13 22:02 +0000 [f59bd47ed3]  Matt Jordan <mjordan@digium.com>
+	  This commit adds support for RFC 5761: Multiplexing RTP Data and Control
+	  Packets on a Single Port. Specifically, it enables the feature when
+	  using chan_pjsip.
 
-	* app_dial: Improve documentation
+	  A new option, "rtcp_mux" has been added to endpoint configuration in
+	  pjsip.conf. If set, then Asterisk will attempt to use rtcp-mux with
+	  whatever it communicates with. Asterisk follows the rules set forth in
+	  RFC 5761 with regards to falling back to standard RTCP behavior if the
+	  far end does not indicate support for rtcp-mux.
 
-	  * Add some helpful <literal> and other embedded paragraph tags
+	  The lion's share of the changes in this commit are in
+	  res_rtp_asterisk.c. This is because it was pretty much hard wired to
+	  have an RTP and an RTCP transport. The strategy used here is that when
+	  rtcp-mux is enabled, the current RTCP transport and its trappings (such
+	  as DTLS SSL session) are freed, and the RTCP session instead just
+	  mooches off the RTP session. This leads to a lot of specialized if
+	  statements throughout.
 
-	  * Document some of the lesser known channel variables set by Dial
+	  ASTERISK-26732 #close
+	  Reported by Dan Jenkins
 
-	  * Add examples for some common Dial uses, along with some more
-	    challenging but useful options
+	  Change-Id: If46a93ba1282418d2803e3fd7869374da8b77ab5
 
-	  Change-Id: Ib2fb9301e8e044d14fbb2815ec64161f19bbfbc1
+2017-03-14 08:49 +0000 [dc4cdafd42]  Torrey Searle <torrey@voxbone.com>
 
-2016-08-13 20:16 +0000 [4facaac408]  Matt Jordan <mjordan@digium.com>
+	* res/res_pjsip_refer: call xfer w/o extension
 
-	* manager: Add <see-also> tags to relate interrelated events/actions together
+	  When transfering to a URI without an extension, ensure that the
+	  s extension of the dialplan is entered
 
-	  Change-Id: Idbac539205aa732bf786c4f765577d8e9ff28ba4
+	  ASTERISK-26869 #close
 
-2016-08-13 20:15 +0000 [232d4fe24f]  Matt Jordan <mjordan@digium.com>
+	  Change-Id: I07403df66cf93f09e00a40ab5b41bfc6f72b1525
 
-	* manager: Add <see-also> tags to relate Bridge related events,actions, and apps
+2017-03-09 11:05 +0000 [982d6173c5]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I67e6b79fa3102e494b5fe6cc7510472249080e85
+	* app_queue: Handle the caller being redirected out of a queue bridge
 
-2016-08-13 20:14 +0000 [63c0b2f7c9]  Matt Jordan <mjordan@digium.com>
+	  A caller can leave the Queue() application after being bridged with a
+	  member in a few ways:
 
-	* manager: Add <see-also> tags to relate AoC events and actions
+	    * Caller or member hangup
+	    * Caller is transferred somewhere else (blind or atx)
+	    * Caller is externally redirected elsewhere
 
-	  Change-Id: Iea89a36222712148c1775c05ed0ad1049d67a70e
+	  The first 2 scenarios are currently handled by subscribing to stasis
+	  messages, but the 3rd is not explicitly covered. If a caller is
+	  redirected away from the Queue() application, the member who was last
+	  bridged with that caller will remain in an "In use" state until the
+	  caller hangs up.
 
-2016-08-13 20:13 +0000 [0422667d6c]  Matt Jordan <mjordan@digium.com>
+	  This patch adds handling of the caller leaving the queue via
+	  redirection. We monitor the caller-member bridge, and if the caller is
+	  the one that leaves, we treat it the same as we would a caller hangup.
 
-	* manager: Add <see-also> tags to relate UserEvent actions/apps/events
+	  ASTERISK-26400 #close
+	  Reported by: Etienne Lessard
 
-	  Change-Id: I80f8a981f62f50e74609c69c49edcaca6c95efa4
+	  Change-Id: Iba160907770de5a6c9efeffc9df5a13e9ea75334
 
-2016-08-12 15:53 +0000 [f9e734974b]  Matt Jordan <mjordan@digium.com>
+2017-03-15 08:44 +0000 [0b8a57af6d]  Joshua Colp <jcolp@digium.com>
 
-	* res_agi: Improve documentation
+	* res_pjsip_endpoint_identifier_ip: Don't output error if no header_match.
 
-	  * Groups of AGI commands that have similar functionality now reference
-	    each other, and all reference the AGI application for ease of wiki
-	    reference.
+	  This change ensures that if no header_match option is set on an
+	  identify an error message is not output stating the option is set
+	  to an invalid value.
 
-	  * The documentation for the AGI application has been improved, in
-	    particular noting the various AGI types and how they are invoked.
+	  ASTERISK-26863
 
-	  * A warning message has been added to DeadAGI, noting that it is
-	    deprecated.
+	  Change-Id: I239bc6d2319dd3da24ba96a38d4d6e9b5526d62a
 
-	  Change-Id: I479ccdee8a7393f01b18692c3d4ab7e6bdd1875d
+2017-03-14 07:50 +0000 [1475604eff]  Matt Jordan <mjordan@digium.com>
 
-2016-08-12 13:53 +0000 [781bb410d0]  Matt Jordan <mjordan@digium.com>
+	* res_pjsip_endpoint_identifier_ip: Add an option to match requests by header
 
-	* manager: Add <see-also> links between related events
+	  This patch adds a new features to the endpoint identifier module,
+	  'match_header'. When set, inbound requests are matched by a provided SIP
+	  header: value pair. This option works in conjunction with the existing
+	  'match' configuration option, such that if any 'match*' attribute
+	  matches an inbound request, the request is associated with the specified
+	  endpoint.
 
-	  This patch adds some see-also references between related AMI events. It
-	  focuses primarily on those events that are guaranteed to come in pairs,
-	  such as DTMFBegin/DTMFEnd, as well as those that occur during the life
-	  cycle of an Asterisk channel, such as Newchannel/Hangup.
+	  Since this module now identifies by more than just IP address,
+	  appropriate renaming of the module and/or variables can be done in a
+	  non-release branch.
 
-	  Change-Id: Iaab600477052018d0f8c03d0c624c0856e9ff1f3
+	  ASTERISK-26863 #close
 
-2016-08-12 11:15 +0000 [cfd6852d39]  Matt Jordan <mjordan@digium.com>
+	  Change-Id: Icfc14835c962f92e35e67bbdb235cf0589de5453
+	  (cherry picked from commit 30f52d79d7fc9ab0b628bef2b61ea515413795a2)
 
-	* func_channel: Reorganize documentation
+2017-03-14 16:16 +0000 [f997090877]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Following the example of the PJSIP channel driver, the channel
-	    technology specific documentation has been moved to the respective
-	    channel drivers that provide that functionality. This has the benefit
-	    of locating the documentation of items with those modules that provide
-	    it.
+	* pbx.c: Fix crash from malformed exten pattern.
 
-	  * Examples of using the CHANNEL function for both standard items as well
-	    as for PJSIP have been added.
+	  Forgetting to indicate an exten is a pattern can cause a crash if the
+	  "pattern" has a character set range.  e.g., "9999[3-5]" The crash is due
+	  to a buffer overwrite because the '-' exten eye-candy wasn't removed as
+	  expected and overran the allocated space.
 
-	  * The 'max_forwards' standard item has been documented.
+	  The buffer overwrite is fixed two ways in this patch.
 
-	  Change-Id: Ifaa79a232c8ac99cf8da6ef6cc7815d398b1b79b
+	  1) Fix ext_strncpy() to distinguish between pattern and non-pattern
+	  extens.  Now '-' characters are removed when they are eye-candy and not
+	  when they are part of a pattern character set.  Since the function is
+	  private to pbx.c, the return value now returns the number of bytes written
+	  to the destination buffer instead of the strlen() of the final buffer so
+	  the callers that care don't need to add one.
 
-2016-08-11 22:11 +0000 [cb043249b6]  Corey Farrell <git@cfware.com>
+	  2) Fix callers to ext_strncpy() to supply the correct available buffer
+	  size of the destination buffer.
 
-	* Run mandatory cleanup when startup fails.
+	  ASTERISK-26668
 
-	  Errors during startup result in an exit.  These error branches should be
-	  calling ast_run_atexit(0) to ensure mandatory cleanup is run.
+	  Change-Id: I555d97411140e47e0522684062d174fbe32aa84a
 
-	  ASTERISK-26267 #close
+2017-03-14 16:51 +0000 [0dc007e94d]  Richard Begg <asterisk@meric.id.au>
 
-	  Change-Id: If226f2326ae2df7add20040696132214cf2bb680
+	* chan_iax2: Reload of iax peer results in loss of host address/port
 
-2016-08-11 11:24 +0000 [4d5e96ab53]  gtjoseph <gjoseph@digium.com>
+	  When using a non-dynamic peer address, build_peer() invalidates the
+	  peer address structure by setting the address family to unspecified.
+	  However, if dnsmgr is enabled, the subsequent call to ast_dnsmgr_lookup()
+	  will not amend the peer address if the cache is still valid, resulting
+	  in peer connectivity failures.
+	  To fix this, we call ast_dnsmgr_refresh() instead.
 
-	* res_pjsip_caller_id:  Copy header name to short header name
+	  ASTERISK-26865
 
-	  When compact_headers was set, we were sending a zero-length header name
-	  for PAI and RPID because we always forced the short header name length
-	  to 0.  We did this because we cloned the header from "From" and wanted
-	  to clear "f" from the sname.  By cloning however, we bypass pjproject's
-	  automatic logic that sets sname to name if there's no compact form of
-	  the header, which there isn't for PAI and RPID.  So now we force sname
-	  to be the same as name right after we set name.
+	  Change-Id: Id8a89a2f771ebbaf32255a35fe596a6dcb97a082
 
-	  res_pjsip_diversion needed the same treatment for the Diversion header.
+2017-03-14 15:12 +0000 [59130260e7]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-26241 #close
+	* configure: Don't use the progress bar with curl when downloading to stdout
 
-	  Change-Id: I633ec139630cd83809aae00336cee4a10077e467
+	  In some scenarios, such as when there may not be a terminal (such as
+	  inside a Docker container), curl will apparently direct the progress bar
+	  to stdout. This can cause extra data to be appended to a file curl'd
+	  down to stdout, resulting in md5 verification failures.
 
-2016-08-11 12:18 +0000 [143df33110]  gtjoseph <gjoseph@digium.com>
+	  This patch removes the progress bar, and tells curl to download the file
+	  silently.
 
-	* res_pjsip:  Fail global load if debug or default_from_user are empty
+	  ASTERISK-26872 #close
 
-	  If debug was specified in the global configuration but left blank,
-	  the logger would treat it as a wildcard and log all hosts.  If
-	  default_from_user was empty, a crash would result.
+	  Change-Id: Ie860b020f627d4372b3e7ce9453de5faafeebe6c
 
-	  The global apply handler now checks for empty strings.
+2017-03-02 17:11 +0000 [8470c2bdea]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-26239 #close
-	  ASTERISK-26238 #close
+	* RFC sdp: Initial SDP creation
 
-	  Change-Id: Ie75727f5cd5808845d92cc81f5713842fb203336
+	  * Added additional fields to ast_sdp_options.
+	  * Re-organized ast_sdp.
+	  * Updated field names to correspond to RFC4566 terminology.
+	  * Created allocs/frees for SDP children.
+	  * Created getters/setters for SDP children where appropriate.
+	  * Added ast_sdp_create_from_state.
+	  * Refactored res_sdp_translator_pjmedia for changes.
 
-2016-08-01 15:07 +0000 [1fc5c90014]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Iefbd877af7f5a4d3c74deead1bff8802661b0d48
 
-	* res_pjsip res_pjsip_mwi: Misc fixes and cleanups.
+2017-03-14 09:55 +0000 [05713c36ea]  Matt Jordan <mjordan@digium.com>
 
-	  * Eliminated RAII_VAR() usage in
-	  ast_sip_persistent_endpoint_update_state().
+	* configs/samples/hep.conf.sample: Clarify how the HEP stack works
 
-	  * Added a missing allocation failure check to
-	  persistent_endpoint_find_or_create().
+	  This patch updates the documenation in hep.conf.sample to better specify
+	  how the various HEP modules interact.
 
-	  * Made persistent_endpoint_find_or_create() create the new object without
-	  a lock as it isn't needed.
+	  ASTERISK-26717 #close
 
-	  * Cleaned up some ao2 container allocation idioms.
+	  Change-Id: I337fb742a89e3ec5edc7fc7a7a0295218d841124
 
-	  * Reordered res_pjsip_mwi.c load_module() and unload_module()
+2017-03-14 09:59 +0000 [0ded269bfa]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: If8ce88fbd82a0c72a37a2388f74f77237a6a36a8
+	* funcs/func_devstate: Remove new line in Device field of during module load
 
-2016-08-04 18:03 +0000 [73052e5732]  Richard Mudgett <rmudgett@digium.com>
+	  During module loading of func_devstate, Asterisk emits the current
+	  device state of all Custom device states currently stored in the AstDB.
+	  This was erroneously including a new line character ('\n') to the end of
+	  the device state, causing two new lines to be emitted in
+	  DeviceStateChange AMI events.
 
-	* location.c: Misc fixes and cleanups.
+	  Note that this only happened for those device state changes that
+	  occurred during startup. Regular device state changes for Custom device
+	  states are handled elsewhere, and did not have the newline.
 
-	  * Eliminated most RAII_VAR() usage.
+	  ASTERISK-26643 #close
+	  Reported by: Roman Bedros
+	  Tested by: Matt Jordan
+	  patches:
+	    ami_devstate.diff uploaded by Roman Bedros (License 6842)
 
-	  * Added several missing allocation failure checks.
+	  Change-Id: I1f4c02fc79c448d43bf725f5039c83d9611d7d93
 
-	  * Made ast_sip_for_each_contact() allocate the wrapper ao2 object without
-	  a lock as it is not needed.
+2017-03-14 09:37 +0000 [b03b72717f]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ie20913365156c95dd79e5d471cfd25e99ae880bc
+	* main/stasis_cache: Demote the ERROR message when removing a nonexistent item
 
-2016-08-02 13:53 +0000 [9d4bd3d763]  Richard Mudgett <rmudgett@digium.com>
+	  This patch demotes the ERROR message that is displayed when a
+	  nonexistent item is removed from the Stasis cache. The genesis of this
+	  demotion is due to chan_sip's realtime peers and their interaction with
+	  Asterisk's core ast_endpoint code, but ostensibly it could happen from
+	  other channel drivers as well.
 
-	* taskprocessor.c: Tweak high water checks.
+	  Since Mark Michelson already did an excellent job of explaining on this
+	  issue, it is quoted here for posterity:
 
-	  * The high water check in ast_taskprocessor_alert_set_levels() would
-	  trigger immediately if the new high water level is zero and the queue was
-	  empty.
+	  "Internally, when a realtime peer is retrieved, Asterisk creates an
+	  ast_endpoint structure. When that peer is destroyed, the ast_endpoint is
+	  destroyed as well. Part of the destruction of the ast_endpoint involves
+	  clearing the Stasis cache of all information about that endpoint. The
+	  problem here is that the act of creating the ast_endpoint is not enough
+	  to actually put any information in the Stasis cache. Instead, something
+	  has to happen, such as a state change, in order for the Stasis cache to
+	  have any information about that endpoint. When a device registers,
+	  chan_sip creates an ast_endpoint structure, processes the REGISTER, and
+	  then destroys the ast_endpoint. When the ast_endpoint is destroyed,
+	  there is nothing to destroy in the Stasis cache, so an error message is
+	  emitted. When you use rtcachefriends, ast_endpoint structures persist
+	  for the lifetime of the module and so you do not see this error
+	  message."
 
-	  * The high water check in taskprocessor_push() was off by one.
+	  ASTERISK-25237 #close
 
-	  Change-Id: I687729fb4efa6a0ba38ec9c1c133c4d407bc3d5d
+	  Change-Id: I53cebc6b4a897a1ab9564182b75c177780feff70
 
-2016-08-03 16:24 +0000 [e1248c3075]  Richard Mudgett <rmudgett@digium.com>
+2017-03-08 12:39 +0000 [2d7e68c075]  Matt Jordan <mjordan@digium.com>
 
-	* res_pjsip: Make aor named lock a mutex.
+	* res_pjsip_endpoint_identifier_ip: Clean up a spaces/tabs issue
 
-	  The named aor lock was always being locked for writes so a rwlock adds no
-	  benefit and may be slower because rwlocks are biased toward read locking.
+	  Tabs > spaces. Always.
 
-	  Change-Id: I8c5c2c780eb30ce5441832257beeb3506fd12b28
+	  Change-Id: I899ff662361c7ab0327173bd7851a67b53dd65f1
 
-2016-07-29 17:41 +0000 [6e40334d89]  Richard Mudgett <rmudgett@digium.com>
+2017-03-12 09:21 +0000 [12460b05c1]  Joshua Colp <jcolp@digium.com>
 
-	* pjsip_distributor.c: Add missing allocation failure check.
+	* chan_pjsip: Don't assume a session will have a channel.
 
-	  Change-Id: I932ab2cea845e534d9ff318035b6de39972d3b28
+	  When querying for PJSIP specific information using the dialplan
+	  function CHANNEL() it is possible that the underlying session
+	  will no longer have a channel associated with it. This is
+	  most likely to occur when the RTCP HEP module attempts to get
+	  the channel name. If this happens then a crash will occur.
 
-2016-08-11 11:13 +0000 [a3c5488ff4]  Matt Jordan <mjordan@digium.com>
+	  This change just adds a check that the channel exists on the
+	  session before querying it.
 
-	* app_queue: Prevent crash when a call is forwarded to an invalid location
+	  ASTERISK-26857
 
-	  When a call forward attempt is made from a Queue member, the current
-	  code will hang up the forwarding channel in an off-nominal condition
-	  prior to raising the Stasis events informing the rest of Asterisk that
-	  the call was forwarded. This will result in a slew of dreaded FRACKs,
-	  most likely leading to a crash.
+	  Change-Id: I113479cffff6ae64cf8ed089e9e1565223426f01
 
-	  This patch modifies the code such that we don't hang up the forwarding
-	  channel even in an off-nominal condition until we've safely raised the
-	  Stasis messages.
+2017-03-10 20:29 +0000 [d1ef127084]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-25797 #close
+	* pjproject_bundled:  Reduce the need for rebuilds
 
-	  Change-Id: Ife5abed351691fd79105321636eaa8ea8dcdba38
+	  Bundled pjproject should now only rebuild if one of the menuselect
+	  "Compiler Flags" options changes.
 
-2016-08-11 10:50 +0000 [5913929d31]  Kevin Harwell <kharwell@digium.com>
+	  Change-Id: If114a2e16b9e77af371a600d6a5e197bbf28fe43
 
-	* alembic: add auth_username to endpoint's identify_by enum
+2017-03-05 15:26 +0000 [36fed72614]  Daniel Journo <dan@keshercommunications.com>
 
-	  A new identify_by option was added recently, auth_username. However, this
-	  setting was not added as an allowable choice in the database enumeration
-	  value.
+	* pjsip/cli_commands: pjsip show channelstats shows wrong codec
 
-	  This patch updates the current enumeration, adding in the new setting.
+	  * cli_commands.c Fixed CLI output
 
-	  ASTERISK-26268 #close
+	  ASTERISK-26822 #close
 
-	  Change-Id: Ib4788e8485e4cd40172ec0abbf5810a147ab8bf8
+	  Change-Id: I3889ef6a8f6738fc312fab42db5efacd6e452b01
 
-2016-08-06 10:57 +0000 [1589452fdc]  Alexei Gradinari <alex2grad@gmail.com>
+2017-03-08 14:29 +0000 [b14724adb3]  Daniel Journo <dan@keshercommunications.com>
 
-	* pjsip: Fix deadlock with suspend taskprocessor on masquerade
+	* res_musiconhold: moh general section is a class and issues warning
 
-	  If both channels which should be masqueraded
-	  are in the same serializer:
-	  1st channel will be locked waiting condition 'complete'
-	  2nd channel will be locked waiting condition 'suspended'
+	  * res_musiconhold.c: Ensure the general section is not treated as
+	  a moh class.
 
-	  On heavy load system a chance that both channels will be in
-	  the same serializer 'pjsip/distibutor' is very high.
+	  ASTERISK-26353 #close
 
-	  To reproduce compile res_pjsip/pjsip_distributor.c with
-	  DISTRIBUTOR_POOL_SIZE=1
+	  Change-Id: Ia3dbd11ea2b43ab3e6c820a9827811dd24bea82d
 
-	  Steps to reproduce:
-	  1. Party A calls Party B (bridged call 'AB')
-	  2. Party B places Party A on hold
-	  3. Party B calls Voicemail app (non-bridged call 'BV')
-	  4. Party B attended transfers Party A to voicemail using REFER.
-	  5. When asterisk masquerades calls 'AB' and 'BV',
-	     a deadlock is happened.
+2017-03-08 17:08 +0000 [35cfd2c0cc]  Sean Bright <sean.bright@gmail.com>
 
-	  This patch adds a suspension indicator to the taskprocessor.
-	  When a session suspends/unsuspends the serializer
-	  it sets the indicator to the appropriate state.
-	  The session checks the suspension indicator before
-	  suspend the serializer.
+	* media_cache: Prefer ast_file_is_readable() over access()
 
-	  ASTERISK-26145 #close
+	  Change-Id: Icc0dc6e61b2e68d5cdcb74b016b2726a388c7def
 
-	  Change-Id: Iaaebee60013a58c942ba47b1b4930a63e686663b
+2017-03-07 06:25 +0000 [bc2c66b594]  Sean Bright <sean.bright@gmail.com>
 
-2016-08-09 12:07 +0000 [f6ec94cca6]  Kevin Harwell <kharwell@digium.com>
+	* pbx_spool: Set AST_OUTGOING_ATTEMPT variable on channel
 
-	* alembic/sqlalchemy: auto increment only allowed on a single column
+	  Set a variable on the channel that indicates which attempt number we
+	  are currently performing to allow for attempt-specific behavior.
 
-	  The extensions table defined two columns (id and priority) as primary key
-	  autoincrement columns. However only one is allowed when defining the primary
-	  key.
+	  ASTERISK-26568 #close
+	  Reported by: Roman Shubovich
 
-	  This patch removes the autoincrement attribute from the priority column since
-	  it does not need to be as such and really should not have been on there in the
-	  first place.
+	  Change-Id: Iacd7e8d43b0ed5b6cb021c62f41f1a1f5733dd89
 
-	  This patch also removes 'context', 'exten', and 'priority' from the primary key
-	  index and creates a new combined unique contraint index on them.
+2017-03-07 07:37 +0000 [4e3b0cedba]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-26183 #close
+	* res_pjsip_transport_websocket: Add support for IPv6.
 
-	  Change-Id: Ib9c712c612a4d7ec1edb0dcb77f1bae0905a470b
+	  This change adds a PJSIP patch (which has been contributed upstream)
+	  to allow the registration of IPv6 transport types.
 
-2016-08-07 09:58 +0000 [5f815f9dba]  Matt Jordan <mjordan@digium.com>
+	  Using this the res_pjsip_transport_websocket module now registers
+	  an IPv6 Websocket transport and uses it for the corresponding
+	  traffic.
 
-	* channels/chan_pjsip: Add PJSIP_SEND_SESSION_REFRESH
+	  ASTERISK-26685
 
-	  This patch adds a new PJSIP specific dialplan function,
-	  PJSIP_SEND_SESSION_REFRESH. When invoked on a PJSIP channel, the media
-	  session will be refreshed via either an UPDATE or re-INVITE request.
-	  When used in conjunction with the PJSIP_MEDIA_OFFER dialplan function,
-	  the formats in use on a PJSIP channel can be re-negotiated and changed
-	  dynamically after call setup.
+	  Change-Id: Id1f9126f995b31dc38db8fdb58afd289b4ad1647
 
-	  ASTERISK-26277 #close
+2017-03-08 08:16 +0000 [60998371e3]  Daniel Journo <dan@keshercommunications.com>
 
-	  Change-Id: Ib98fe09ba889aafe26d58d32f0fd1323f8fd9b1b
+	* app_voicemail: Cannot set fromstring on a per-mailbox basis
 
-2016-08-09 16:19 +0000 [a119bab6a6]  Mark Michelson <mmichelson@digium.com>
+	  * apps/app_voicemail.c fromstring field added to mailbox which will
+	  override the global fromstring if set.
 
-	* res_rtp_asterisk: Cache local RTCP address.
+	  ASTERISK-24562 #close
 
-	  When an RTCP packet is sent or received, res_rtp_asterisk generates a
-	  Stasis event that contains the RTCP report as well as the local and
-	  remote addresses that the report pertains to.
+	  Change-Id: I5e90e3a1ec2b2d5340b49a0db825e4bbb158b2fe
 
-	  The addresses are determined using ast_find_ourip(). For the local
-	  address, this will typically result in a lookup of the hostname of the
-	  server, and then a DNS lookup of that hostname. If you do not have the
-	  host in /etc/hosts, then this results in a full DNS lookup, which can
-	  potentially block for some time.
+2017-03-07 13:38 +0000 [5d0371d743]  Mark Michelson <mmichelson@digium.com>
 
-	  This is especially problematic when performing RTCP reads, since those
-	  are done on the same thread responsible for reading and writing media.
+	* res_http_websocket: Fix faulty read logic.
 
-	  This patch addresses the issue by performing a lookup of the local
-	  address when RTCP is allocated. We then use this cached local address
-	  for the Stasis events when necessary.
+	  When doing some WebRTC testing, I found that the websocket would
+	  disconnect whenever I attempted to place a call into Asterisk. After
+	  looking into it, I pinpointed the problem to be due to the iostreams
+	  change being merged in.
 
-	  ASTERISK-26280 #close
+	  Under certain circumstances, a call to ast_iostream_read() can return a
+	  negative value. However, in this circumstance, the websocket code was
+	  treating this negative return as if it were a partial read from the
+	  websocket. The expected length would get adjusted by this negative
+	  value, resulting in the expected length being too large.
+
+	  This patch simply adds an if check to be sure that we are only updating
+	  the expected length of a read when the return from a read is positive.
+
+	  ASTERISK-26842 #close
 	  Reported by Mark Michelson
 
-	  Change-Id: I3dd61882c2e57036f09f0c390cf38f7c87e9b556
+	  Change-Id: Ib4423239828a013d27d7bc477d317d2f02db61ab
 
-2016-08-08 12:53 +0000 [a06a1af0eb]  Alexei Gradinari <alex2grad@gmail.com>
+2017-03-07 08:12 +0000 [d51ca4b406]  Jean Aunis <jean.aunis@prescom.fr>
 
-	* res_pjsip_mwi: fix unsolicited mwi blocks PJSIP stack
+	* chan_sip: Call not cancelled after receiving a 422 response
 
-	  The PJSIP taskprocessors could be overflowed on startup
-	  if there are many (thousands) realtime endpoints
-	  configured with unsolicited mwi.
-	  The PJSIP stack could be totally unresponsive for a few minutes
-	  after boot completed.
+	  When receiving a 422 response, the invitestate variable must be reset to
+	  INV_CALLING.
 
-	  This patch creates a separate PJSIP serializers pool for mwi
-	  and makes unsolicited mwi use serializers from this pool.
-	  This patch also adds 2 new global options to tune taskprocessor
-	  alert levels: 'mwi_tps_queue_high' and 'mwi_tps_queue_low'.
+	  ASTERISK-26841
 
-	  This patch also adds new global option 'mwi_disable_initial_unsolicited'
-	  to disable sending unsolicited mwi to all endpoints on startup.
-	  If disabled then unsolicited mwi will start processing
-	  on next endpoint's contact update.
+	  Change-Id: Ia0502d6b02192664cefa4e75bafdd2645ce56099
 
-	  ASTERISK-26230 #close
+2017-03-07 05:22 +0000 [3ed05badb9]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I4c8ecb82c249eb887930980a800c9f87f28f861a
+	* core: Add stream topology changing primitives with tests.
 
-2016-08-04 10:16 +0000 [485fd27f7c]  Joshua Colp <jcolp@digium.com>
+	  This change adds a few things to facilitate stream topology changing:
 
-	* res_pjsip_outbound_publish: Use a serializer shutdown group for unload.
+	  1. Control frame types have been added for use by the channel driver
+	  to notify the application that the channel wants to change the stream
+	  topology or that a stream topology change has been accepted. They are
+	  also used by the indicate interface to the channel that the application
+	  uses to indicate it wants to do the same.
 
-	  This change replaces the custom unload process for the outbound
-	  publish module with the common serializer shutdown group.
+	  2. Legacy behavior has been adopted in ast_read() such that if a
+	  channel requests a stream topology change it is denied automatically
+	  and the current stream topology is preserved if the application is
+	  not capable of handling streams.
 
-	  ASTERISK-25217 #close
+	  Tests have also been written which confirm the multistream and
+	  non-multistream behavior.
 
-	  Change-Id: I280a0384d860c486202d87d2d674394cca77ffb6
+	  ASTERISK-26839
 
-2016-08-03 15:39 +0000 [805f105f88]  Corey Farrell <git@cfware.com>
+	  Change-Id: Ia68ef22bca8e8457265ca4f0f9de600cbcc10bc9
 
-	* Add missing checks during startup.
+2017-03-06 15:54 +0000 [272259a2c6]  Daniel Journo <dan@keshercommunications.com>
 
-	  This ensures startup is canceled due to allocation failures from the
-	  following initializations.
-	  * channel.c: ast_channels_init
-	  * config_options.c: aco_init
+	* Saynumber is trying to get "and" from "digits/" subfolder
 
-	  ASTERISK-26265 #close
+	  * say.c Changed 'digits/and' to 'vm-and' for en_GB
 
-	  Change-Id: I911ed08fa2a3be35de55903e0225957bcdbe9611
+	  ASTERISK-26598 #close
 
-2016-07-22 16:37 +0000 [ea71bd6e3e]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: If1b713e5daea6f952b339f139178d292a6c4fcfe
 
-	* app_voicemail: Add taskprocessor alert level options.
+2017-03-06 13:15 +0000 [5a74abc53b]  Sean Bright <sean.bright@gmail.com>
 
-	  On heavy loaded system with IMAP or DB storage,
-	  'app_voicemail' taskprocessor queue could reach 500 scheduled tasks.
-	  It could happen when the IMAP or DB server dies or is unreachable.
-	  It could happen on startup when there are many (thousands)
-	  realtime endpoints configured with unsolicited mwi.
-	  If the taskprocessor queue reaches the high water level
-	  then the alert is triggered and pjsip stops processing new requests
-	  until the queue reaches the low water level to clear the alert.
+	* pbx_spool: Gracefully handle long lines in call files
 
-	  This patch adds 2 new 'general' configuration options
-	  to tune taskprocessor alert levels:
-	  'tps_queue_high' - Taskprocessor high water alert trigger level.
-	  'tps_queue_low' - Taskprocessor low water clear alert level
+	  Per the linked issue, we aren't checking the buffer filled by fgets()
+	  to determine if it contains a newline, so we will fail to correctly
+	  parse the trailing portion of a long line.
 
-	  ASTERISK-26229 #close
+	  This patch increases the buffer size from 256 to 1024, and skips any
+	  line that exceeds that length, logging a warning in the process.
 
-	  Change-Id: I766294fbffedf64053c0d9ac0bedd3109f043ee8
+	  ASTERISK-17067 #close
+	  Reported by: Dave Olszewski
 
-2016-08-03 09:47 +0000 [9dc8cfabd5]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I51bcf270c1b4347ba05b43f18dc2094c76f5d7b0
 
-	* astconfigparser: Really handle case where line is simply a comment.
+2017-03-02 21:27 +0000 [c9296b23d1]  Richard Mudgett <rmudgett@digium.com>
 
-	  The regular expression would match causing the code that handled
-	  the line if it was merely a comment to never get executed.
+	* core: Cleanup ast_get_hint() usage.
 
-	  Change-Id: I3e4022481037ebcba9905587fe8c764b4ce21819
+	  * manager.c:manager_state_cb() Fix potential use of uninitialized hint[]
+	  if a hint does not exist for the requested extension.  Ran into this when
+	  developing a testsuite test.  The AMI event ExtensionStatus came out with
+	  the hint header value containing garbage.  The AMI event PresenceStatus
+	  also had the same issue.
 
-2016-07-23 08:51 +0000 [ad3e65433c]  gtjoseph <gjoseph@digium.com>
+	  * manager.c:action_extensionstate() no need to completely initialize the
+	  hint[].  Only initialize the first element.
 
-	* asterisk.c:  Add auto generation and persistence of UUID
+	  * pbx.c:ast_add_hint() Remove unnecessary assignment.
 
-	  Upcoming features will require the generation and persistence
-	  of a UUID.
+	  * chan_sip.c: Eliminate an unneeded hint[] local variable.  We only care
+	  about the return value of ast_get_hint() there.
 
-	  Change-Id: I3ec0062427e133217db6ef496a4216f427c3b92d
+	  Change-Id: Ia9a8786f01f93f1f917200f0a50bead0319af97b
 
-2016-08-02 12:55 +0000 [efc4034d72]  Kevin Harwell <kharwell@digium.com>
+2017-02-16 04:22 +0000 [7922f26cb0]  Jørgen H <asterisk.org@hovland.cx>
 
-	* rest-api: Code out of sync with the model
+	* res_pjsip WebRTC/websockets: Fix usage of WS vs WSS.
 
-	  Change-Id: Idccaa26fd4a423d47d013ee592b8fa6a0349c006
+	  According to the RFC[1] WSS should only be used in the Via header
+	  for secure Websockets.
 
-2016-07-29 13:13 +0000 [f6821fbaec]  Mark Michelson <mmichelson@digium.com>
+	  * Use WSS in Via for secure transport.
 
-	* Remove SILK payload mappings from Asterisk core.
+	  * Only register one transport with the WS name because it would be
+	  ambiguous.  Outgoing requests may try to find the transport by name and
+	  pjproject only finds the first one registered.  This may mess up unsecure
+	  websockets but the impact should be minimal.  Firefox and Chrome do not
+	  support anything other than secure websockets anymore.
 
-	  SILK is a bit of a hog when it comes to using up our limited number of
-	  dynamic payload types in the RTP engine. By freeing up four slots, it
-	  allows for other codecs to potentially take the place.
+	  * Added and updated some debug messages concerning websockets.
 
-	  Now, codec_silk.so will dynamically use the payload slots in the RTP
-	  engine when it loads.
+	  * security_events.c: Relax case restriction when determining security
+	  transport type.
 
-	  A better fix would be make RTP dynamic payload types actually
-	  dynamic. However, at this stage of Asterisk 14 development, this is a
-	  risky move that would be imprudent.
+	  * The res_pjsip_nat module has been updated to not touch the transport
+	  on Websocket originating messages.
 
-	  Change-Id: I5774e09408f9a203db189529eabdc0d3f4c1e612
-	  (cherry picked from commit d50895c7b04036aeaad58990089399e46db4c817)
+	  [1] https://tools.ietf.org/html/rfc7118
 
-2016-08-01 11:08 +0000 [102d28c11a]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-26796 #close
 
-	* sorcery: Use more compatible regex for local expressions.
+	  Change-Id: Ie3a0fb1a41101a4c1e49d875a8aa87b189e7ab12
 
-	  This changes the use of an empty regex for both res_sorcery_config
-	  and res_sorcery_memory to "." instead. This is a more compatible
-	  regular expression which also works on FreeBSD.
+2017-02-24 15:30 +0000 [0560c32375]  George Joseph <gjoseph@digium.com>
 
-	  ASTERISK-26206 #close
+	* stream: Unit tests for stream read and tweaks framework
 
-	  Change-Id: Ia9166dd176f1597555ba22b6931180d0626c1388
+	  * Removed the AST_CHAN_TP_MULTISTREAM tech property.  We now rely
+	    on read_stream being set to indicate a multi stream channel.
+	  * Added ast_channel_is_multistream convenience function.
+	  * Fixed issue where stream and default_stream weren't being set on
+	    a frame retrieved from the queue.
+	  * Now testing for NULL being returned from the driver's read or
+	    read_stream callback.
+	  * Fixed issue where the dropnondefault code was crashing on a
+	    NULL f.
+	  * Now enforcing that if either read_stream or write_stream are
+	    set when ast_channel_tech_set is called that BOTH are set.
+	  * Added the unit tests.
 
-2016-08-02 03:08 +0000 [b78d10a2df]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-26816
 
-	* res_pjsip: SIP/SDP origin (o=) contained square brackets on IP6 transports.
+	  Change-Id: If7792b20d782e71e823dabd3124572cf0a4caab2
 
-	  ASTERISK-26256 #close
+2017-03-01 07:23 +0000 [1dacf317f3]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I3fd68df561f81fdb8c6c497d465b50c12422f058
+	* res_config_pgsql: Make 'require' return consistent with other backends
 
-2016-08-01 16:13 +0000 [1f95c011c7]  gtjoseph <gjoseph@digium.com>
+	  res_config_pgsql should match the behavior of other realtime backend
+	  drivers so that queue_log can disable adaptive logging.
 
-	* menuselect:  Add an opaque "member_data" string to the acceptable xml
+	  ASTERISK-25628 #close
+	  Reported by: Dmitry Wagin
 
-	  Change-Id: Id5ac43b95c8d7395f3be37f983632169db3d1afe
+	  Change-Id: Ic1fb1600c7ce10fdfb1bcdc43c5576b7e0014372
 
-2016-07-27 09:56 +0000 [df42f64d62]  David M. Lee <dlee@respoke.io>
+2017-02-22 15:11 +0000 [9c55a71798]  Mark Michelson <mmichelson@digium.com>
 
-	* Replace strdupa with more portable ast_strdupa
+	* SDP: Add initial SDP state machine.
 
-	  The strdupa function is a GNU extension, and not widely portable. We
-	  have an ast_strdupa function used within Asterisk which is preferred.
-	  I pulled the definition up from menuselect.c into the menuselect.h
-	  header file so it can be shared across menuselect.
+	  This introduces and documents the various states in the state machine.
+	  This also introduces API functions that induce state changes, and places
+	  TODO comments telling what needs to be done in addition to what is
+	  already there. Those TODOs will be replaced with real code in upcoming
+	  changes.
 
-	  Change-Id: I9593c97f78386b47dc1e83201e80cb2f62b36c2e
+	  Change-Id: I871c0eb480b4c84d83e91ac5628e7a673e8b89ed
 
-2016-07-24 18:27 +0000 [56a07fbab9]  gtjoseph <gjoseph@digium.com>
+2017-02-28 13:48 +0000 [60e9e4fcc0]  Sean Bright <sean.bright@gmail.com>
 
-	* menuselect:  Various menuselect enhancements
+	* media_cache: Mark cache entry stale if cache file is removed
 
-	  * Add 'external' as a support level.
-	  * Add ability for module directories to add entries to the menu
-	    by adding members to the <module_prefix>/<module_prefix>.xml file.
-	  * Expand the description field to 3 lines in the ncurses implementation.
-	  * Allow the description field to wrap in the newt implementation.
-	  * Add description field to the gtk implementation.
+	  In the event that a cache file is removed out from under us, we should
+	  treat the cache entry as stale and force a refresh.
 
-	  Change-Id: I7f9600a1984a42ce0696db574c1051bc9ad7c808
-	  (cherry picked from commit 90f445729d5d86050d9d379485ff0a99f4a006c1)
+	  ASTERISK-26774 #close
+	  Reported by: Igor Gamayunov
 
-2016-07-29 04:48 +0000 [7f9369c1b6]  Joshua Colp <jcolp@digium.com>
+	  Change-Id: I3b1bd0c999d59d18664ef73a29823bc5b431dc52
 
-	* astconfigparser: Handle case where line is simply a comment.
+2017-02-28 09:41 +0000 [e5b44c26b4]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: I2dea5815363f4d787d709228a04f33baee383ef5
+	* res_config_pgsql: Release table locks where appropriate
 
-2016-07-28 14:10 +0000 [57e9c66819]  Corey Farrell <git@cfware.com>
+	  The find_table() functions NULL or a locked table pointer. We are
+	  not consistently calling release_table() in failure paths.
 
-	* pbx.c: Fix handling of '-' in extension name and callerid
+	  Change-Id: I6f665b455799c84b036e5b34904b82b05eab9544
 
-	  This adds a two strings to ast_exten.  name to go with exten and
-	  cidmatch_display to go with cidmatch.  The new fields contain input used
-	  to add the extension in the first place.  The existing fields now
-	  contain stripped input that excludes insignificant spaces and dashes.
-	  These stripped fields should always be used for comparisons.  The
-	  unstripped fields should normally be used for display, but displaying
-	  stripped values will not cause runtime errors.
+2017-02-28 05:41 +0000 [6ebdcfe27d]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  Note the actual string is only stored twice if it contains dashes.  If
-	  no dashes are found then both 'char *' fields point to the same memory.
-	  So this change has a minimum effect on memory usage.
+	* pjsip.conf.sample: user_agent: not a specific version
 
-	  The existing functions ast_get_extension_name and
-	  ast_get_extension_cidmatch return unstripped values as they did before
-	  this change.  Other similar bugs likely still exist where unstripped
-	  extensions are saved outside pbx.c then passed back in.
+	  Use the description of useragent from sip.conf here.
 
-	  ASTERISK-26233 #close
+	  ASTERISK-26825 #close
 
-	  Change-Id: I6cd61ce57acc1570ca6cc14960c4c3b0a9eb837f
+	  Change-Id: I5b33a4aaa0ae1d793289d05e3bc09521affbf755
 
-2016-07-27 17:17 +0000 [873fc0fda5]  Richard Mudgett <rmudgett@digium.com>
+2017-02-27 20:07 +0000 [fb68db87b1]  George Joseph <gjoseph@digium.com>
 
-	* pbx.c: Allow dangerous functions when adding a hint to dialplan.
+	* res_pjsip_pubsub:  Remove unneeded endpoint unref
 
-	  We can allow dangerous functions when adding a hint since altering
-	  dialplan is itself a privileged activity.  Otherwise, we could never
-	  execute dangerous functions.
+	  When a subscription was being recreated and the endpoint wasn't
+	  found, we were trying to unref the endpoint.  This was causing
+	  FRACKs.  Removed the unref.
 
-	  ASTERISK-25996 #close
-	  Reported by: Andrew Nagy
+	  ASTERISK-26823 #close
 
-	  Change-Id: I4929ff100ad1200a0198262d069a34f2296e77ba
+	  Change-Id: If86d2aecff8fe853c7f38a1bfde721fcef3cd164
 
-2016-07-21 10:36 +0000 [f00525a6f6]  Alexei Gradinari <alex2grad@gmail.com>
+2017-02-16 04:16 +0000 [ee0a123f43]  Jørgen H <asterisk.org@hovland.cx>
 
-	* pjproject: fixed a few bugs
+	* res_pjsip: Fix crash when contact has no status
 
-	  This patch fixes the issue in pjsip_tx_data_dec_ref()
-	  when tx_data_destroy can be called more than once,
-	  and checks if invalid value (e.g. NULL) is passed to.
+	  This change fixes an assumption in res_pjsip that a contact will
+	  always have a status. There is a race condition where this is
+	  not true and would crash. The status will now be unknown when
+	  this situation occurs.
 
-	  This patch updates array limit checks and docs
-	  in pjsip_evsub_register_pkg() and pjsip_endpt_add_capability().
+	  ASTERISK-26623 #close
 
-	  Change-Id: I4c7a132b9664afaecbd6bf5ea4c951e43e273e40
+	  Change-Id: Id52d3ca4d788562d236da49990a319118f8d22b5
 
-2016-07-17 18:28 +0000 [972cee2e4c]  gtjoseph <gjoseph@digium.com>
+2017-02-21 18:06 +0000 [22242fef5d]  George Joseph <gjoseph@digium.com>
 
-	* pjproject_bundled:  Update for pjproject 2.5.5
+	* res_pjsip_outbound_registration:  Subscribe to network change events
 
-	  Add more --disable-* switches to Makefile.rules including
-	  --disable-opus which was causing bundled pjproject to fail with
-	  "undefined reference" errors in libasteriskpj.
+	  Outbound registration now subscribes to network change events
+	  published by res_stun_monitor and refreshes all registrations
+	  when an event happens.
 
-	  Changed PJ_ENABLE_EXTRA_CHECK to 1.
+	  The 'pjsip send (un)register' CLI commands were updated to accept
+	  '*all' as an argument to operate on all registrations.
 
-	  Removed 2 obsolete patches and added a new one.
-	  The new one was merged by Teluu on 6/27/2016.
+	  The 'PJSIP(Un)Register' AMI commands were also updated to
+	  accept '*all'.
 
-	  ASTERISK-26148 #close
+	  ASTERISK-26808 #close
 
-	  Change-Id: Ib8af6c6a9d31f7238ce65b336134c2efdc855063
+	  Change-Id: Iad58a9e0aa5d340477fca200bf293187a6ca5a25
 
-2016-07-27 10:33 +0000 [8902a51d59]  David M. Lee <dlee@respoke.io>
+2017-02-27 12:25 +0000 [4692a32ed7]  George Joseph <gjoseph@digium.com>
 
-	* Portably sscanf tv_usec
+	* build:  Warn if asterisk is installed in both 32 and 64 bit sys dirs
 
-	  In a timeval, tv_usec is defined as a suseconds_t, which could be
-	  different underlying types on different platforms. Instead of trying to
-	  scanf directly into the timeval, scanf into a long int, then copy that
-	  into the timeval.
+	  ... and clean them both up on uninstall.
 
-	  Change-Id: I29f22d049d3f7746b6c0cc23fbf4293bdaa5eb95
+	  We've fixed the issue where 'make install' was installing to
+	  /usr/lib on 64-bit systems that use /usr/lib64.  Now we need
+	  to clean up the remnants in /usr/lib.
 
-2016-07-27 12:36 +0000 [852e763571]  Kevin Harwell <kharwell@digium.com>
+	  * 'make install' now prints a warning if DESTDIR/ASTLIBDIR
+	    contains 'lib64' and libasterisk* shared libraries or modules
+	    are also found in DESTDIR/ASTLIBDIR with 'lib64' transformed
+	    to 'lib'.
 
-	* rtp_engine: Failed assertion and wrong name given for codec
+	  * 'make uninstall' ALWAYS cleans up both DESTDIR/ASTLIBDIR and
+	    DESTDIR/ASTLIBDIR with 'lib64' transformed to 'lib'.
 
-	  Fixed an assert check that would trigger when the passed in value was negative.
-	  The negative value was being cast to an unsigned value. This resulted in the
-	  check failing.
+	  ASTERISK-26705
 
-	  Also fixed another problem when loading formats in the engine. When setting the
-	  mime type the format's name was being passed in instead of the codec's name.
+	  Change-Id: I6edddeb3c07a51e7c7ba7cac3c05e4bf3ec3f01f
 
-	  Change-Id: I1a201cd419ba4d8e9a40d337e36b6fbe1737192c
+2017-02-27 07:02 +0000 [ff2b4308d1]  Joshua Colp <jcolp@digium.com>
 
-2016-07-21 22:44 +0000 [e8c34680ca]  Richard Mudgett <rmudgett@digium.com>
+	* bridge_native_rtp: Handle case where channel joins already suspended.
 
-	* dsp.c: Add fax and DTMF detection unit tests.
+	  The bridge_native_rtp module did not properly handle the case where
+	  a smart bridge operation occurs while a channel is suspended. In this
+	  scenario the module would incorrectly set up local or remote RTP
+	  bridging despite the media having to flow through Asterisk. The remote
+	  endpoint would see two media streams and experience wonky audio.
 
-	  * Add fax amplitude and frequency sweep tests.
-	  * Add DTMF amplitude and twist unit tests.
+	  The module has been changed so that it ensures both channels are
+	  not suspended when performing the native RTP bridging and this
+	  requirement has been documented in the bridge technology.
 
-	  Change-Id: I8d77c9a1eec89e440d715f998c928687e870c3f7
+	  ASTERISK-26781
 
-2016-07-21 11:56 +0000 [c1f240b818]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Id4022d73ace837d4a293106445e3ade10dbc7c7c
 
-	* dsp.c: Added descriptive comments to Goertzel calculations.
+2016-08-12 11:23 +0000 [5b1796f59d]  frahaase <fra.haase@googlemail.com>
 
-	  * Added doxygen to describe some struct members and what is going on in
-	  the code.
+	* Binaural synthesis (confbridge): DTMF conference management.
 
-	  Change-Id: I2ec706a33b52aee42b16dcc356c2bd916a45190d
+	  DTMF configuration options for the binaural softmix bridge:
+	  toggle binaural rendering (per channel).
 
-2016-07-13 13:48 +0000 [003a52fd62]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-26292
 
-	* dsp.c: Fix incorrect format reference typo.
+	  Change-Id: Ibfe708b9fe26097c1798fcbfcc4dc461267d8af8
 
-	  Change-Id: Ia131da3ec29acf385cb43a586a29ecc975eb3896
+2017-02-24 11:49 +0000 [2046743938]  Joshua Colp <jcolp@digium.com>
 
-2016-07-25 21:18 +0000 [4c0a0cbe02]  Richard Mudgett <rmudgett@digium.com>
+	* config: Improve documentation and behavior of outbound_proxy option.
 
-	* dsp.c: Correct DTMF twist dsp.conf documentation.
+	  This change updates the documentation for the outbound_proxy option
+	  to ensure it is consistently stated that a full SIP URI must be
+	  provided for the option.
 
-	  Change-Id: Idf97e3a72f1edc5fca58f2fa7b20785922be0cae
+	  The res_pjsip_outbound_registration module has also been changed so
+	  that the provided outbound_proxy value is checked to ensure it is a
+	  URI and if not an error is output stating so.
 
-2016-07-22 04:43 +0000 [87433c2566]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-26782
 
-	* astconfigparser.py: Update with realtime fixes.
+	  Change-Id: I6c239a32274846fd44e65b44ad9bf6373479b593
 
-	  When configuring SIP URIs in the pjsip.conf file it is
-	  necessary to escape the semicolon so the parser does not
-	  treat it as a comment. This change allows this to work in
-	  the astconfigparser implementation.
+2017-02-23 13:03 +0000 [c07c6714f2]  Joshua Colp <jcolp@digium.com>
 
-	  A secondary bug where some data was lost if a configuration
-	  option included a "=" in its value was also fixed.
+	* channel: Add ast_read_stream function for reading frames from all streams.
 
-	  A bug where sections would be considered equal despite
-	  being different has also been fixed.
+	  This change introduces an ast_read_stream function and callback in
+	  the channel technology which allows reading frames from all streams
+	  and not just the default streams.
 
-	  Change-Id: If229f656ef22050b50e7b34e90c4bffe796431f8
+	  The stream number has also been added to frames. This is to allow the
+	  case where frames are queued onto the channel instead of being read
+	  directly from the driver.
 
-2016-07-28 14:32 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  This change does impose a restriction on reading though: a chain of
+	  frames can only contain frames from the same stream.
 
-	* asterisk 13.11.0-rc1 Released.
+	  ASTERISK-26816
 
-2016-07-28 09:29 +0000 [3bfaf6b172]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I5d7dc35e86694df91fd025126f6cfe0453aa38ce
 
-	* Release summaries: Add summaries for 13.11.0-rc1
+2017-02-09 18:05 +0000 [a537dae6d0]  George Joseph <gjoseph@digium.com>
 
-2016-07-28 09:27 +0000 [ca145e1807]  Mark Michelson <mmichelson@digium.com>
+	* pjproject_bundled: Update for pjproject 2.6
 
-	* .version: Update for 13.11.0-rc1
+	   * Removed all 2.5.5 functional patches.
+	   * Updated usages of pj_release_pool to be "safe".
+	   * Updated configure options to disable webrtc.
+	   * Updated config_site.h to disable webrtc in pjmedia.
+	   * Added Richard Mudgett's recent resolver patches.
 
-2016-07-28 09:27 +0000 [918ebf79ff]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: Ib400cc4dfca68b3d07ce14d314e829bfddc252c7
 
-	* .lastclean: Update for 13.11.0-rc1
+2017-02-23 15:49 +0000 [b0067bcf2c]  George Joseph <gjoseph@digium.com>
 
-2016-07-28 09:27 +0000 [d7afc1cf9d]  Mark Michelson <mmichelson@digium.com>
+	* build: Execute ldconfig to build cache. (take two)
 
-	* realtime: Add database scripts for 13.11.0-rc1
+	  On some platforms a multiarch approach is used for libraries.
+	  The build system does not take this into account and still
+	  places libraries into the lib directory if no --libdir is
+	  specified to configure. On initial startup this results in
+	  libasteriskssl.so not being found, as it is not in the multiarch
+	  lib directory.  To make matters worse, options were being passed
+	  to ldconfig on both Linux and FreeBSD that actually prevented
+	  the rebuild of the cache.
 
-2016-07-21 22:28 +0000 [159e437e5a]  Richard Mudgett <rmudgett@digium.com>
+	   * Fedora has a /usr/share/config.site that automatically tells
+	     autoconf to use /usr/lib64 but CentOS does not. This logic was
+	     copied to configure.ac and modified so systems like Ubuntu,
+	     which still use /usr/lib for 64-bit systems, aren't affected.
 
-	* dsp.c: Fix erroneous fax tone detection.
+	  Now that we have them in the correct directory...
 
-	  The Goertzel calculations get less accurate the lower the signal level
-	  being worked with becomes because there is less resolution remaining.
-	  If it is too low we can erroneously detect a tone where none really
-	  exists.  The searched for fax frequencies not only need to be so much
-	  stronger than the background noise they must also be a minimum strength.
+	  In order for the system loader to find libasteriskssl and
+	  libasteriskpj, one of 3 things has to happen...
 
-	  * Add needed minimum threshold test to tone_detect().
+	    - The linker cache must be rebuilt including the directory
+	      where the libasterisk* libraries were installed.  Only root
+	      can rebuild the cache.  This was busted.
+	    - We have to link the asterisk binary with an rpath pointing
+	      to the directrory where the libasterisk* libraries were
+	      installed.  This makes things very complicated and will happen
+	      over the collective dead bodies of everyone who's had to
+	      package a distribution with an rpath.
+	    - Finally, you can start asterisk with LD_LIBRARY_PATH set to the
+	      directrory where the libasterisk* libraries were installed.
 
-	  * Set TONE_THRESHOLD to allow low volume frequency spread detection.
+	  There are no other options. So...
 
-	  ASTERISK-26237 #close
-	  Reported by: Richard Mudgett
+	   * The invokation of ldconfig has been moved from main/Makefile
+	     to ASTTOPDIR/Makefile, the options have been removed, and
+	     DESTDIR/ASTLIBDIR appended.  If you aren't root, you will be
+	     warned after the "Asterisk Installation Compete" banner that
+	     you must re-run 'make install' as root, manually run
+	     'ldconfig DESTDIR/ASTLIBDIR' as root, or run asterisk with
+	     LD_LIBRARY_PATH.
 
-	  Change-Id: I84dbba7f7628fa13720add6a88eae3b129e066fc
+	  ASTERISK-26705
 
-2016-07-22 14:44 +0000 [eda95236d1]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: I2a64b7c33a7d3e9bde20f47e3d3ab771977af982
 
-	* Fix sqlalchemy error regarding identifier length.
+2017-02-23 14:48 +0000 [0f4b349d37]  Sean Bright <sean.bright@gmail.com>
 
-	  sqlalchemy was complaining:
+	* res_config_pgsql: Fix thread safety problems
 
-	  sqlalchemy.exc.IdentifierError: Identifier
-	  'ps_contacts_qualifyfreq_exptime' exceeds maximum length of 30
-	  characters
+	  * A missing AST_LIST_UNLOCK() in find_table()
 
-	  This fixes the problem by changing the index name to be
-	  "ps_contacts_qualifyfreq_exp" instead.
+	  * The ESCAPE_STRING() macro uses pgsqlConn under the hood and we were
+	    not consistently locking before calling it.
 
-	  ASTERISK-26227 #close
-	  Reported by Mark Michelson
+	  * There were a handful of other places where pgsqlConn was accessed
+	    directly without appropriate locking.
 
-	  Change-Id: I0ed784f87504be2a59ee8d3242ef6f625d5ed1a9
+	  Change-Id: Iea63f0728f76985a01e95b9912c3c5c6065836ed
 
-2016-07-22 05:46 +0000 [66c9dfb272]  Alexander Traud <pabstraud@compuserve.com>
+2017-02-22 05:00 +0000 [6cc890b880]  Joshua Colp <jcolp@digium.com>
 
-	* chan_sip: Enable Session-Timers for SIP over TCP (and TLS).
+	* channel: Add support for writing to a specific stream.
 
-	  Asterisk defaults to timers=accept/refresher=uas. In that scenario, only in that
-	  scenario, Sessions-Timers (RFC 4028) had no effect via TCP. This change enables
-	  Session-Timers for SIP over TCP (and for SIP over TLS).
+	  This change adds an ast_write_stream function which allows
+	  writing a frame to a specific media stream. It also moves
+	  ast_write() to using this underneath by writing media
+	  frames provided to it to the default streams of the channel.
+	  Existing functionality (such as audiohooks, framehooks, etc)
+	  are limited to being applied to the default stream only.
+
+	  Unit tests have also been added which test the behavior of
+	  both non-multistream and multistream channels to confirm that
+	  the write() and write_stream() callbacks are invoked
+	  appropriately.
 
-	  However with longer international calls via TCP, the SIP channel might break,
-	  because all hops on the Internet route must stay online (have not a single power
-	  outage, for example). Therefore with Session-Timers enabled (which are enabled
-	  at default), you might see dropped calls. Consequently even with this change,
-	  you might be better-off going for session-timers=refuse in your sip.conf.
+	  ASTERISK-26793
 
-	  ASTERISK-19968 #close
+	  Change-Id: I4df20d1b65bd4d787fce0b4b478e19d2dfea245c
 
-	  Change-Id: I1cd33453c77c56c8e1394cd60a6f17bb61c1d957
+2016-08-12 11:23 +0000 [094c26aa68]  frahaase <fra.haase@googlemail.com>
 
-2016-07-15 16:16 +0000 [33716106e0]  Richard Mudgett <rmudgett@digium.com>
+	* Binaural synthesis (confbridge): Adds binaural synthesis to bridge_softmix.
 
-	* res_pjsip: Whitespace and comment cleanup.
+	  Adds binaural synthesis to bridge_softmix (via convolution using libfftw3).
+	  Binaural synthesis is conducted at 48kHz.
+	  For a conference, only one spatial representation is rendered.
+	  The default rendering is applied for mono-capable channels.
 
-	  Change-Id: I11139a4a95df34e223ba622aa6227e33ab8f6c38
+	  ASTERISK-26292
 
-2016-07-21 09:05 +0000 [52ab0bf258]  gtjoseph <gjoseph@digium.com>
+	  Change-Id: Iecdb381b6adc17c961049658678f6219adae1ddf
 
-	* chan_sip: Prevent deadlock when issuing "sip show channels"
+2017-02-22 08:53 +0000 [e57961db84]  Sean Bright <sean.bright@gmail.com>
 
-	  sip_show_channels locks the dialogs container first then locks each
-	  sip_pvt so it can spit out the details.  The rest of sip dialog
-	  processing locks the sip_pvt first then locks the dialogs container
-	  if it needs to.  Both lock in the order they need but deadlocks can
-	  result.  To fix, sip_show_channels and sip_show_channelstats have
-	  been converted to use an iterator rather than ao2_callback.  This way
-	  the container is locked only while getting the next entry and is
-	  unlocked when the callback is called.
+	* res_config_ldap: Various code improvements
 
-	  ASTERISK-23013 #close
+	  The initial motivation for this patch was to properly handle memory
+	  allocation failures - we weren't checking the return values from the
+	  various LDAP library allocation functions.
 
-	  Change-Id: Id9980419909e811f89484950ed46ef117b9eb990
+	  In the process, because update_ldap() and update2_ldap() were
+	  substantially the same code, they've been consolidated.
 
-2016-07-19 15:22 +0000 [5997ec7c9e]  Alexei Gradinari <alex2grad@gmail.com>
+	  Change-Id: Iebcfe404177cc6860ee5087976fe97812221b822
 
-	* res_pjsip_pubsub: fixed a bug when pjsip_tx_data_dec_ref is called twice.
+2017-02-22 13:08 +0000 [66a35e2451]  Michael L. Young <elgueromexicano@gmail.com>
 
-	  This patch removed call of pjsip_tx_data_dec_ref in send_notify
-	  if send_request failed.
-	  The pjsip_dlg_send_request deletes the message on error by itself.
+	* build_tools:  Fix download_externals to allow the use of curl or wget
 
-	  It seems this patch fixes next issues:
-	  ASTERISK-26199
-	  ASTERISK-26166
-	  ASTERISK-26174
+	  Not sure if this is really a bug versus an improvement. I can see it being
+	  viewed as a bug though by some.
 
-	  Change-Id: I8b05917c93d993f95d604c042ace5f1a5500f59a
+	  The current build_tools/download_externals file depends on wget in order to
+	  download external modules.  The current build system is able to discover
+	  which tool to use for fetching remote files - either wget or curl.
 
-2016-07-18 22:46 +0000 [7fdf7c3d4c]  Corey Farrell <git@cfware.com>
+	  This patch takes advantage of this capability by modifying the two calls to
+	  the wget binary to instead use what was discovered by the build system.
 
-	* Add conditional support for noreturn functions.
+	  ASTERISK-26812 #close
 
-	  This adds support for tagging functions with the noreturn attribute.
-	  If DO_CRASH is enabled then ast_do_crash never returns.  If AST_DEVMODE
-	  and DO_CRASH are enabled then failed assertions never return.  This can
-	  resolve a large number of false positives with static analyzers.
+	  Change-Id: If9411a2554f009274d377445613ae91192d948a1
 
-	  ASTERISK-26220 #close
+2017-02-22 11:12 +0000 [ced73d5b79]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Icfb61e5fe54574eced4c3e88b317244f467ec753
+	* Revert "build: Execute ldconfig to build cache."
 
-2016-07-19 13:18 +0000 [dcb8aa8c1c]  Richard Mudgett <rmudgett@digium.com>
+	  This reverts commit 28c8e4f58f0f38792c7c79a05bd07788ebf15332.
 
-	* chan_dahdi.c: Fix deadlock potential in fax redirection.
+	  Change-Id: Ie2e1aaf61fd49045994974a4581545ac8348fe4c
 
-	  The dahdi_handle_dtmf() and my_handle_dtmf() have the potential to
-	  deadlock if an incoming fax happens during the Playback or similar
-	  application.
+2017-02-21 10:47 +0000 [15ed7af027]  Sean Bright <sean.bright@gmail.com>
 
-	  * Fixed the potential deadlock by not calling ast_async_goto() with the
-	  channel lock held.
+	* pbx_realtime: Prevent premature extension matching
 
-	  ASTERISK-26216 #close
-	  Reported by: Richard Mudgett
+	  The patterns provided by pbx_realtime were checked in the order in
+	  which they were returned from the realtime backend. If there was
+	  overlap between multiple patterns, the first one to correctly match was
+	  chosen even though it may not have been the best match.
 
-	  Change-Id: I9144b84ade5f96690996624ec8a2d40c56af40aa
+	  We now sort the patterns descending by their length and compare in that
+	  order. There may be cases where this still results in a sub-optimal
+	  match, but this patch should improve the overall behavior.
 
-2016-07-13 18:49 +0000 [fa91cf3eec]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-18271 #close
+	  Reported by: Charlie Smurthwaite
 
-	* chan_sip.c: Fix deadlock potential in fax redirection.
+	  Change-Id: I56d9ac15810eb1775966b669c3028e32cc7bd809
 
-	  The sip_read() has the potential to deadlock if an incoming fax happens
-	  during the Playback or similar application.
+2017-02-22 08:32 +0000 [f58aefba5b]  Joshua Colp <jcolp@digium.com>
 
-	  * Fixed the potential deadlock by not calling ast_async_goto() with the
-	  channel lock held.
+	* core: Show streams in "core show channel".
 
-	  * Made always eat the fax detection frame whether there is a fax extension
-	  or not.
+	  The "core show channel" CLI command will now output the streams
+	  present on the channel with their details.
 
-	  ASTERISK-26216
-	  Reported by: Richard Mudgett
+	  ASTERISK-26811
 
-	  Change-Id: I6d3f5cccd4b77c3aa6ffc1a54c0f6bde61c9278e
+	  Change-Id: I9c95b57aa09415005f0677a1949a0feb07e4987a
 
-2016-07-13 18:48 +0000 [2e1bdc3775]  Richard Mudgett <rmudgett@digium.com>
+2017-02-21 15:09 +0000 [fc70ca9499]  Sean Bright <sean.bright@gmail.com>
 
-	* chan_pjsip.c: Fix deadlock potential in fax redirection.
+	* pbx_dundi: DUNDi weight parameter not processed correctly
 
-	  The chan_pjsip_cng_tone_detected() has the potential to deadlock if an
-	  incoming fax happens during the Playback or similar application.
+	  The DUNDi weight field is not always converted from network byte order
+	  to host byte order. This can result in incorrect weight values and
+	  incorrect selection of DUNDi destinations.
 
-	  * Fixed the potential deadlock by not calling ast_async_goto() with the
-	  channel lock held.
+	  ASTERISK-18731 #close
+	  Reported by: Peter Racz
+	  Patches:
+	  	dundi_weight.patch (license #6290) patch uploaded by Peter Racz
 
-	  * Made always eat the fax detection frame whether there is a fax extension
-	  or not.
+	  Change-Id: Iba3e1a700ff539db57211a7bbc26f7b22ea9a1be
 
-	  ASTERISK-26216
-	  Reported by: Richard Mudgett
+2017-02-15 14:43 +0000 [a738772edd]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I32aecbb4818af646dc5a619f0dc040e9b1f222e5
+	* Add initial SDP state code.
 
-2016-07-12 17:33 +0000 [628e8c91d5]  Richard Mudgett <rmudgett@digium.com>
+	  This establishes the basic allocation/destruction of an SDP state
+	  object, plus some of the simpler getter methods involved. Subsequent
+	  tasks will deal with adding a state machine, creating SDPs from
+	  capabilities and options, and merging SDPs into a joint SDP.
 
-	* res_fax.c: Fix deadlock potential in FAXOPT(faxdetect) framehook.
+	  Change-Id: Ie3757ce186f04b65e9d1883f5aace53f24e53709
 
-	  The fax_detect_framehook() has the potential to deadlock if an incoming
-	  fax happens during the Playback or similar application.
+2017-02-21 10:47 +0000 [ab04a018e4]  Sean Bright <sean.bright@gmail.com>
 
-	  * Fixed the potential deadlock by not calling ast_async_goto() with the
-	  channel lock held.
+	* realtime: Fix ast_load_realtime_multientry handling
 
-	  * Made always eat the fax detection frame whether there is a fax extension
-	  or not.
+	  ast_load_realtime_multientry() returns an ast_config structure whose
+	  ast_categorys are keyed with the empty strings. Several modules were
+	  giving semantic meaning to the category names causing problems at
+	  runtime.
 
-	  * Made only detach the framehook if we detected a fax and not on other
-	  possible frames.
+	  * app_directory: Treated the category name as the mailbox name, and
+	    would fail to direct calls to the appropriate extension after an
+	    entry was chosen.
 
-	  ASTERISK-26216
-	  Reported by: Richard Mudgett
+	  * app_queue: Queues, queue members, and queue rules were all affected
+	    and needed to be updated.
 
-	  Change-Id: I99da35c26d1cd802626ffb4c1b4eb5b015581b6d
+	  * pbx_realtime: Pattern matching would never succeed because the
+	    extension entered by the user was always compared to the empty
+	    string.
 
-2016-07-12 17:24 +0000 [676aeede36]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: Ie7e44986344b0b76ea8f6ddb5879f5040c6ca8a7
 
-	* res_fax: Fix FAXOPT(faxdetect) timeout option.
+2017-02-21 08:56 +0000 [6e6c96d713]  Sean Bright <sean.bright@gmail.com>
 
-	  The fax detection timeout option did not work because basically the wrong
-	  variable was checked in fax_detect_framehook().  As a result, the timer
-	  would timeout immediately and disable fax detection.
+	* realtime: Centralize some common realtime backend code
 
-	  * Fixed ignoring negative timeout values.  We'd complain and then go right
-	  on using the negative value.
+	  All of the realtime backends create artificial ast_categorys to pass
+	  back into the core as query results. These categories have no filename
+	  or line number information associated with them and the backends differ
+	  slightly on how they create them. So create a couple helper macros to
+	  help make things more consistent.
 
-	  * Fixed destroy_faxdetect() in the off-nominal case of an incomplete
-	  object creation.
+	  Also updated the call sites to remove redundant error messages about
+	  memory allocation failure.
 
-	  * Added more range checking to FAXOPT(gateway) timeout parameter.
+	  Note that res_config_ldap sets the category filename to the 'table name'
+	  but that is not read by anything in the core, so I've dropped it.
 
-	  ASTERISK-26214 #close
-	  Reported by: Richard Mudgett
+	  Change-Id: I3a1fd91e0c807dea1ce3b643b0a6fe5be9002897
 
-	  Change-Id: Idc5e698dfe33572de9840bc68cd9fc043cbad976
+2017-02-16 10:30 +0000 [28c8e4f58f]  Joshua Colp <jcolp@digium.com>
 
-2016-07-18 16:16 +0000 [652130feb2]  Richard Mudgett <rmudgett@digium.com>
+	* build: Execute ldconfig to build cache.
 
-	* chan_dahdi: Add faxdetect_timeout option.
+	  On some platforms a multiarch approach is used for libraries.
+	  The build system does not take this into account and still
+	  places libraries into the lib directory if no --libdir is
+	  specified to configure. On initial startup this results in
+	  libasteriskssl.so not being found, as it is not in the multiarch
+	  lib directory.
 
-	  The new option allows the channel driver's faxdetect option to timeout on
-	  a call after the specified number of seconds into a call.  The new feature
-	  is disabled if the timeout is set to zero.  The option is disabled by
-	  default.
+	  This change does the minimally invasive thing and executes
+	  ldconfig so that the libraries in the lib directory are found
+	  and their location cached. By doing so Asterisk starts up fine.
 
-	  * Don't clear dsp_features after passing them to the dsp code in
-	  my_pri_ss7_open_media().  We should still remember them especially for the
-	  new faxdetect_timeout option.
+	  If DESTDIR is specified, however, the old logic is executed as
+	  the install process may not have permission to alter the ldconfig
+	  cache.
 
-	  ASTERISK-26214
-	  Reported by: Richard Mudgett
+	  ASTERISK-26705
 
-	  Change-Id: Ieffd3fe788788d56282844774365546dce8ac810
+	  Change-Id: If4eca46ac510c6fea5568256280ffdb3888d7bb4
 
-2016-07-15 20:44 +0000 [851b1c3a17]  Richard Mudgett <rmudgett@digium.com>
+2017-01-08 20:32 +0000 [6f15500ced]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip: Add fax_detect_timeout endpoint option.
+	* res_pjsip_authenticator_digest.c: Fix sorcery's immutable contract violation.
 
-	  The new endpoint option allows the PJSIP channel driver's fax_detect
-	  endpoint option to timeout on a call after the specified number of
-	  seconds into a call.  The new feature is disabled if the timeout is set
-	  to zero.  The option is disabled by default.
+	  The inbound authentication object is supposed to be immutable when it is
+	  stored in sorcery.  However, the immutable property is violated if the
+	  authentication object does not have a realm set.
 
-	  ASTERISK-26214
-	  Reported by: Richard Mudgett
+	  The immutable contract violation has a different effect depending upon
+	  what sorcery back end is used.  If it is the config file back end you
+	  would get the same object back until res_pjsip is reloaded.  If it is the
+	  real-time or AstDB back end you would get a new object on each query.  If
+	  it is cached you would get the same object back until it is refreshed from
+	  the database.
 
-	  Change-Id: Id5a87375fb2c4f9dc1d4b44c78ec8735ba65453d
+	  Once an inbound authentication object has its realm set it may or may not
+	  get updated again if the default_realm changes.
 
-2016-07-19 04:48 +0000 [021d4892cd]  Alexander Traud <pabstraud@compuserve.com>
+	  If the same authentication object is used for inbound and outbound
+	  authentication then the immutable violation can make it very hard to
+	  determine why the outbound authentication now fails.  The only diagnostic
+	  message is a complaint about no realms matching when it had worked
+	  earlier.  It fails because of the difference in behaviour for an empty
+	  realm setting between inbound and outbound authentication objects.
 
-	* Makefile: Retain XML Declaration and DTD in docs.
+	  * Fixed the sorcery object immutable violation by creating a new object
+	  and setting the default_realm on it instead.  The new object is a shallow
+	  copy for speed.
 
-	  Since Asterisk 12, the documentation got an XML Stylesheet. Because of a typo,
-	  the XML Declaration and DTD were overwritten by this.
+	  * The auth_store thread storage no longer holds an auth ref.  It
+	  interferes with the shallow copy and never needed a ref anyway.
 
-	  ASTERISK-26212 #close
+	  ASTERISK-26799 #close
 
-	  Change-Id: If5ee4625068042e98ab3fcb22a25e2f15d0c68bd
+	  Change-Id: I2328a52f61b78ed5fbba38180b7f183ee7e08956
 
-2016-07-18 18:39 +0000 [c8e41d14a1]  Corey Farrell <git@cfware.com>
+2017-02-04 20:17 +0000 [6400f5f309]  Richard Mudgett <rmudgett@digium.com>
 
-	* Unit tests: Use AST_TEST_DEFINE in conditional code only.
+	* res_pjsip: Update artificial auth whenever default_realm changes.
 
-	  If AST_TEST_DEFINE is not conditional to TEST_FRAMEWORK it produces dead
-	  code.  This places all existing unit tests into a conditional block if
-	  they weren't already.
+	  There was code attempting to update the artificial authentication object
+	  whenever the default_realm changed.  However, once the artificial
+	  authentication object was created it would never get updated.  The
+	  artificial authentication object would require a system restart for a
+	  change to the default_realm to take effect.
 
-	  ASTERISK-26211 #close
+	  ASTERISK-26799
 
-	  Change-Id: I8ef83ee11cbc991b07b7a37ecb41433e8c734686
+	  Change-Id: Id59036e9529c2d3ed728af2ed904dc36e7094802
 
-2016-07-18 05:13 +0000 [e404f51b42]  Alexander Traud <pabstraud@compuserve.com>
+2017-01-01 08:02 +0000 [0b660c9989]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_rtp_asterisk: Count a roll-over of the sequence number even on lost packets.
+	* res_pjsip: Update authentication realm documentation.
 
-	  With this change, the initial RTP sequence number is randomly chosen not between
-	  0 and 65535 (0xffff) but 0 and 32767 (0x7fff). This assures, the roll-over
-	  counter (ROC) synchronization is not lost for sRTP, when the very first RTP
-	  packets get lost; see http://srtp.sourceforge.net/faq.html#Q6
+	  Using the same auth section for inbound and outbound authentication is not
+	  recommended.  There is a difference in meaning for an empty realm setting
+	  between inbound and outbound authentication uses.
 
-	  ASTERISK-26207 #close
+	  An empty inbound auth realm represents the global section's default_realm
+	  value when the authentication object is used to challenge an incoming
+	  request.  An empty outgoing auth realm is treated as a don't care wildcard
+	  when the authentication object is used to respond to an incoming
+	  authentication challenge.
 
-	  Change-Id: I9a527e3aa3ce8f3becc5131d7ba32b57b5845464
+	  ASTERISK-26799
 
-2016-07-18 04:14 +0000 [5f24874ebb]  Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Id3952f7cfa1b6683b9954f2c5d2352d2f11059ce
 
-	* Makefile: Suppress echoing of target 'config' again.
+2017-02-13 17:11 +0000 [7f83bcd63d]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26038 #close
+	* pjproject: Fixes to resolve DNS SRV crashes.
 
-	  Change-Id: I5746cf639f3fdc6332e8a97cf01f979e30bf403f
+	  * Re #1945 (misc): Don't trigger SRV complete callback when there is a
+	  parse error.
 
-2016-07-14 03:25 +0000 [76d4983c15]  Corey Farrell <git@cfware.com>
+	  * srv_resolver.c: Don't try to send query if already considered resolved.
 
-	* features.c: Remove unneeded adsi.h include.
+	  ** In resolve_hostnames() don't try to resolve a query that is already
+	  considered resolved.
 
-	  adsi.h is no longer used by features.c since parking was moved to a
-	  module.
+	  ** In resolve_hostnames() fix DNS typo in comments.
 
-	  Change-Id: I2248b8a455225a17cb6ddaafd6c20c511a1eaf59
+	  ** In build_server_entries() move a common expression assigning to cnt
+	  earlier.
 
-2016-07-14 18:06 +0000 [cb58f853e1]  Alexei Gradinari <alex2grad@gmail.com>
+	  * sip_transport.c: Fix tdata object name to actually contain the pointer.
 
-	* res_pjsip_mwi: remove unneeded check on endpoint's contacts.
+	  It helps if the logs referencing a tdata object buffer actually have a
+	  name that includes the correct pointer as part of the name.  Also since
+	  the tdata has its own pool it helps if any logs referencing the pool have
+	  the same name as the tdata object.  This change brings tdata logging in
+	  line with how tsx objects are named.
 
-	  The function create_mwi_subscriptions_for_endpoint checks
-	  if there is active contacts by retrieving aors and contacts.
+	  ASTERISK-26669 #close
+	  ASTERISK-26738 #close
 
-	  This function is used to create all unsolicited mwi subscriptions
-	  on startup and is used when contact added.
+	  Change-Id: I56af2ded25476b3e870ca586ee69ed6954ef75af
 
-	  In both cases it's not necessary to check if there are contacts.
-	  The contacts are needed when asterisk sends mwi.
+2017-02-20 13:38 +0000 [bf78c3c9c3]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26200 #close
+	* pjproject: Increase SENDER_WIDTH column size for 64-bit system logs.
 
-	  Change-Id: I98e43bdc97f3c0829951cd9bf5f3c6348c6ac1fa
+	  ASTERISK-26669
+	  ASTERISK-26738
 
-2016-06-30 15:58 +0000 [28501051b4]  Mark Michelson <mmichelson@digium.com>
+	  Change-Id: Ibae6fc8cae69a1f04df0c577c4c11200499d6fe0
 
-	* Update support for SILK format.
+2017-02-06 14:26 +0000 [54812f18b5]  Richard Mudgett <rmudgett@digium.com>
 
-	  This commit adds scaffolding in order to support the SILK audio format
-	  on calls. Roughly, this is what is added:
+	* pjsip_distributor.c: Update some debug messages to get transaction name.
 
-	  * Cached silk formats. One for each possible sample rate.
-	  * ast_codec structures for each possible sample rate.
-	  * RTP payload mappings for "SILK".
+	  * Removed overloaded unmatched response ignore.  We obviously sent the
+	  request so we shouldn't ignore it because it isn't new work.
 
-	  In addition, this change overhauls the res_format_attr_silk file in the
-	  following ways:
+	  ASTERISK-26669
+	  ASTERISK-26738
 
-	  * The "samplerate" attribute is scrapped. That's native to the format.
-	  * There are far more checks to ensure that attributes have been
-	    allocated before attempting to reference them.
-	  * We do not SDP fmtp lines for attributes set to 0.
+	  Change-Id: I55fb5cadc83a8e6699b347c6dc7fa32c5a617d37
 
-	  These changes make way to be able to install a codec_silk module and
-	  have it actually work. It also should allow for passthrough silk calls
-	  in Asterisk.
+2017-02-20 06:28 +0000 [b18f1bfb13]  Sean Bright <sean.bright@gmail.com>
 
-	  Change-Id: Ieeb39c95a9fecc9246bcfd3c45a6c9b51c59380e
+	* app_voicemail: vm_authenticate accesses uninitialized memory
 
-2016-07-14 07:45 +0000 [43b5f8d57b]  Richard Miller (license 5685)
+	  vm_authenticate doesn't always set the passed ast_vm_user argument, so
+	  we initialize to 0 before passing it in.
 
-	* app_queue: Only remove queue member from pending when state changes.
+	  ASTERISK-25893 #close
+	  Reported by: Filip Jenicek
 
-	  It is possible for a not in use state change to occur multiple
-	  times causing a queue member to be removed from the pending call
-	  container prematurely.
+	  Change-Id: Ia3cc0128f93d352ed9add8d5c2f0f7232c2cbe4a
 
-	  The first not in use state change will remove the queue member
-	  from the container. At this moment the member may be called and
-	  placed in the pending container. After this another not in use
-	  state change can be received which will remove it from the
-	  container. Despite being called at this point the code will
-	  incorrectly see that there are no pending calls to it.
+2017-02-20 11:19 +0000 [7739b0b3ae]  Joshua Colp <jcolp@digium.com>
 
-	  This change only removes it from the pending container if the
-	  state has actually changed.
+	* Revert "build: Execute ldconfig to build cache."
 
-	  ASTERISK-26133 #close
-	  patches:
-	    app_queue.diff submitted by Richard Miller (license 5685)
+	  This reverts commit 8851c3e0885cb704a5a6159a51768ea5297e9b10.
 
-	  Change-Id: Ie5a7f17a44f98e9159e9b85009ce3f8393aa78c0
+	  Change-Id: I124380be5e3bd57da978428a2a93604336ccd0db
 
-2016-07-14 02:40 +0000 [a17b071e36]  Corey Farrell <git@cfware.com>
+2017-02-20 08:04 +0000 [ffa7d69766]  George Joseph <gjoseph@digium.com>
 
-	* pbx: Fix leak of timezone for time based includes.
+	* pjproject cli:  Add object count after object lists
 
-	  Create include_free to run ast_destroy_timing and ast_free, use that in
-	  all places that freed an ast_include structure.  This fixes a couple of
-	  paths that previously did not run ast_destroy_timing.
+	  When listing a container, we now print the number of objects
+	  in the container at the end of the list.
+
+	  Change-Id: I791cbc3ee9da9a2af9adc655164b5d32953df812
+
+2017-02-20 05:53 +0000 [e84353b8a8]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_ldap: Don't try to delete non-existent attributes
+
+	  OpenLDAP will raise an error when we try to delete an LDAP attribute
+	  that doesn't exist. We need to filter out LDAP_MOD_DELETE requests
+	  based on which attributes the current LDAP entry actually has. There
+	  is of course a small window of opportunity for this to still fail,
+	  but it is much less likely now.
+
+	  Change-Id: I3fe1b04472733e43151563aaf9f8b49980273e6b
+
+2017-02-20 05:49 +0000 [9f392574f9]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_ldap: Remove extraneous line numbers from log messages
+
+	  Extraneous line numbers were being output in many log messages. These
+	  have been removed.
+
+	  Change-Id: Ice9efa3d252ee87f37fa8f5ea852fda482675431
+
+2017-02-20 05:45 +0000 [ef0944395e]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_ldap: Make memory allocation more consistent
+
+	  The code in update_ldap() and update2_ldap() was using both Asterisk's
+	  memory allocation routines as well as OpenLDAP's. I've changed it so
+	  that everything that is passed to OpenLDAP's functions are allocated
+	  with their routines.
+
+	  Change-Id: Iafec9c1fd8ea49ccc496d6316769a6a426daa804
+
+2017-02-20 05:30 +0000 [dd3efdf525]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_ldap: Fix configuration inheritance from _general
+
+	  The "_general" configuration section allows administrators to provide
+	  both general configuration options (host, port, url, etc.) as well as a
+	  global realtime-to-LDAP-attribute mapping that is a fallback if one of
+	  the later sections do not override it. This neglected to exclude the
+	  general configuration options from the mapping. As an example, during
+	  my testing, chan_sip requested 'port' from realtime, and because I did
+	  not have it defined, it pulled in the 'port' configuration option from
+	  "_general." We now filter those out explicitly.
+
+	  Change-Id: I1fc61560bf96b8ba623063cfb7e0a49c4690d778
+
+2017-02-20 05:27 +0000 [d6d86f1c09]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_ldap: Fix erroneous LDAP_MOD_REPLACE in LDAP modify
+
+	  We always treat the first change of our modification batch as a
+	  replacement when it sometimes is actually a delete. So we have to pass
+	  the correct arguments to the OpenLDAP library.
+
+	  ASTERISK-26580 #close
+	  Reported by: Nicholas John Koch
+	  Patches:
+	  	res_config_ldap.c-11.24.1.patch (license #6833) patch uploaded
+	  	by Nicholas John Koch
+
+	  Change-Id: I0741d25de07c9539f1edc6eff3696165dfb64fbe
+
+2017-02-15 11:55 +0000 [44abe214d2]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_sqlite3: Fix crash when loading with invalid config
+
+	  When ast_config_load() fails with CONFIG_STATUS_FILEINVALID, it has
+	  already destroyed the ast_config struct for us. Trying to do it again
+	  results in a crash.
+
+	  Change-Id: If6a5c0ca718ad428e01a1fb25beb209a9ac18bc6
+
+2017-02-17 17:06 +0000 [51e3b11989]  Sean Bright <sean.bright@gmail.com>
+
+	* pjproject-bundled: Fix checksum verification when using cURL
+
+	  ASTERISK-26802 #close
+	  Reported by: Michael L. Young
+
+	  Change-Id: Iad293080f55d4d69ab615717a15211d916eed613
+
+2017-02-17 16:57 +0000 [0b427f9b59]  Richard Mudgett <rmudgett@digium.com>
+
+	* tcptls.c: Add some missing allocation failure checks.
+
+	  * Fix tcptls_session ref and fd leak in ast_tcptls_server_root().
+
+	  Change-Id: I0ddf01cd3c10d3b6666d7bf68d4e206a37f4fbdb
+
+2017-02-17 14:58 +0000 [dbc3598014]  Mark Michelson <mmichelson@digium.com>
+
+	* Remove extra ast_iostream_close() calls.
+
+	  When AMI encounters an error at the beginning of a session, it would
+	  explicitly call ast_iostream_close() on its tcptls session's iostream.
+	  It then would jump to a label where it would shut down the tcptls
+	  session instance. The tcptls session instance would again attempt to
+	  close the iostream.
+
+	  Under normal circumstances, this might go by unnoticed. However, when
+	  MALLOC_DEBUG is enabled, all fields on the iostream get set to
+	  0xdeaddead when the iostream is freed. Thus a second call to
+	  ast_iostream_close() after the iostream has been freed would reslt in an
+	  attempt to call SSL_shutdown on 0xdeaddead, which would crash and burn
+	  horribly.
+
+	  The fix here is to not directly close the iostream from the dangerous
+	  scenarios. The specific scenarios are:
+	  * Exceeding the configured authlimit
+	  * Failing to build a mansession on a new connection
+
+	  Change-Id: I908f98d516afd5a263bd36b072221008a4731acd
+
+2017-02-14 09:54 +0000 [5a130b2e17]  Mark Michelson <mmichelson@digium.com>
+
+	* Add SDP translator and PJMEDIA implementation.
+
+	  This creates the following:
+	  * Asterisk's internal representation of an SDP
+	  * An API for translating SDPs from one format to another
+	  * An implementation of a translator for PJMEDIA
+
+	  Change-Id: Ie2ecd3cbebe76756577be9b133e84d2ee356d46b
+
+2017-02-07 09:50 +0000 [8af6342555]  Mark Michelson <mmichelson@digium.com>
+
+	* Add initial SDP options.
+
+	  This is step one of adding an SDP API: defining some
+	  configurable settings for SDPs. This is based on options
+	  that are currently supported in Asterisk.
+
+	  Change-Id: I1ede91aafed403b12a9ccdfb91a88389baa7e5d7
+
+2017-02-16 10:30 +0000 [8851c3e088]  Joshua Colp <jcolp@digium.com>
+
+	* build: Execute ldconfig to build cache.
+
+	  On some platforms a multiarch approach is used for libraries.
+	  The build system does not take this into account and still
+	  places libraries into the lib directory if no --libdir is
+	  specified to configure. On initial startup this results in
+	  libasteriskssl.so not being found, as it is not in the multiarch
+	  lib directory.
+
+	  This change does the minimally invasive thing and executes
+	  ldconfig so that the libraries in the lib directory are found
+	  and their location cached. By doing so Asterisk starts up fine.
+
+	  ASTERISK-26705
+
+	  Change-Id: I6d30b6427e9d5e69470e11327c7ff203fa7da519
+
+2017-02-16 08:38 +0000 [e93f2a5142]  Sean Bright <sean.bright@gmail.com>
+
+	* realtime: Fix LIKE escaping in SQL backends
+
+	  The realtime framework allows for components to look up values using a
+	  LIKE clause with similar syntax to SQL's. pbx_realtime uses this
+	  functionality to search for pattern matching extensions that start with
+	  an underscore (_).
+
+	  When passing an underscore to SQL's LIKE clause, it will be interpreted
+	  as a wildcard matching a single character and therefore needs to be
+	  escaped. It is (for better or for worse) the responsibility of the
+	  component that is querying realtime to escape it with a backslash before
+	  passing it in. Some RDBMs support escape characters by default, but the
+	  SQL92 standard explicitly says that there are no escape characters
+	  unless they are specified with an ESCAPE clause, e.g.
+
+	  	SELECT * FROM table WHERE column LIKE '\_%' ESCAPE '\'
+
+	  This patch instructs 3 backends - res_config_mysql, res_config_pgsql,
+	  and res_config_sqlite3 - to use the ESCAPE clause where appropriate.
+
+	  Looking through documentation and source tarballs, I was able to
+	  determine that the ESCAPE clause is supported in:
+
+	  MySQL 5.0.15   (released 2005-10-22 - earliest version available from
+	                  archives)
+	  PostgreSQL 7.1 (released 2001-04-13)
+	  SQLite 3.1.0   (released 2005-01-21)
+
+	  The versions of the relevant libraries that we depend on to access MySQL
+	  and PostgreSQL will not work on versions that old, and I've added an
+	  explicit check in res_config_sqlite3 to only use the ESCAPE clause when
+	  we have a sufficiently new version of SQLite3.
+
+	  res_config_odbc already handles the escape characters appropriately, so
+	  no changes were required there.
+
+	  ASTERISK-15858 #close
+	  Reported by: Humberto Figuera
+
+	  ASTERISK-26057 #close
+	  Reported by: Stepan
+
+	  Change-Id: I93117fbb874189ae819f4a31222df7c82cd20efa
+
+2017-02-16 08:28 +0000 [f8f513d363]  George Joseph <gjoseph@digium.com>
+
+	* stream:  Rename creates/destroys to allocs/frees
+
+	  To be consistent with sdp implementation.
+
+	  Change-Id: I714e300939b4188f58ca66ce9d1e84b287009500
+
+2017-02-16 05:46 +0000 [30aaeec5a1]  Sean Bright <sean.bright@gmail.com>
+
+	* res_config_sqlite3: Properly create missing columns when necessary
+
+	  There were two specific issues resolved here:
+
+	  1) The code that iterated over the required fields
+	     (via ast_realtime_require) was broken for the RQ_INTEGER1 field
+	     type. Iteration would stop when the first RQ_INTEGER1 (0) field
+	     was encountered.
+
+	  2) sqlite3_changes() was used to try and count the number of rows
+	     returned by a SELECT statement. sqlite3_changes() only counts
+	     affected rows, so this was always returning the value from the
+	     most recent data modification statement. We now separate read-only
+	     queries from data modification queries and count rows appropriately
+	     in both cases.
+
+	  ASTERISK-23457 #close
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: I91ed20494efc3fcfbc2a96ac7646999a49814884
+
+2017-02-15 14:44 +0000 [ac7a34c531]  Joshua Elson <joshelson@gmail.com>
+
+	* http: Ensure capath is defined on all http creations
+
+	  ASTERISK-26794 #close
+
+	  Change-Id: I9cbc3b6b6a8aab590f5ccde9c262a98e4d5253a1
+
+2017-02-15 23:09 +0000 [135bea931c]  Igor Goncharovsky <igor.goncharovsky@gmail.com>
+
+	* chan_unistim: fix char type to have consistent behavior on ARM
+
+	  There is difference exists in behaviour of char type on x86 and ARM.
+	  On x86 by default char variable type means signed char, but in ARM
+	  unsigned char used. This make binary calculations and negative values
+	  works wrong on ARM.
+
+	  This patch change type of char variables used for store negative
+	  values and binary calculations to signed char.
+
+	  ASTERISK-26714
+
+	  Change-Id: Id78716dee9568a58419d4ef63c038affc3dfc7ab
+
+2017-02-07 13:17 +0000 [4bdf5d329f]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip_pubsub:  Correctly implement persisted subscriptions
+
+	  This patch fixes 2 original issues and more that those 2 exposed.
+
+	  * When we send a NOTIFY, and the client either doesn't respond or
+	    responds with a non OK, pjproject only calls our
+	    pubsub_on_evsub_state callback, no others.  Since
+	    pubsub_on_evsub_state (which does the sub_tree cleanup) does not
+	    expect to be called back without the other callbacks being called
+	    first, it just returns leaving the sub_tree orphaned.  Now
+	    pubsub_on_evsub_state checks the event for PJSIP_EVENT_TSX_STATE
+	    which is what pjproject will set to tell us that it was the
+	    transaction that timed out or failed and not the subscription
+	    itself timing our or being terminated by the client. If is
+	    TSX_STATE, pubsub_on_evsub_state now does the proper cleanup
+	    regardless of the state of the subscription.
+
+	  * When a client renews a subscription, we don't update the
+	    persisted subscription with the new expires timestamp.  This causes
+	    subscription_persistence_recreate to prune the subscription if/when
+	    asterisk restarts.  Now, pubsub_on_rx_refresh calls
+	    subscription_persistence_update to apply the new expires timestamp.
+	    This exposed other issues however...
+
+	  * When creating a dialog from rdata (which sub_persistence_recreate
+	    does from the packet buffer) there must NOT be a tag on the To
+	    header (which there will be when a client refreshes a
+	    subscription).  If there is one, pjsip_dlg_create_uas will fail.
+	    To address this, subscription_persistence_update now accepts a flag
+	    that indicates that the original packet buffer must not be updated.
+	    New subscribes don't set the flag and renews do.  This makes sure
+	    that when the rdata is recreated on asterisk startup, it's done
+	    from the original subscribe packet which won't have the tag on To.
+
+	  * When creating a dialog from rdata, we were setting the dialog's
+	    remote (SUBSCRIBE) cseq to be the same as the local (NOTIFY) cseq.
+	    When the client tried to resubscribe after a restart with the
+	    correct cseq, we'd reject the request with an Invalid CSeq error.
+
+	  * The acts of creating a dialog and evsub by themselves when
+	    recreating a subscription does NOT restart pjproject's subscription
+	    timer.  The result was that even if we did correctly recreate the
+	    subscription, we never removed it if the client happened to go away
+	    or send a non-OK response to a NOTIFY.  However, there is no
+	    pjproject function exposed to just set the timer on an evsub that
+	    wasn't created by an incoming subscribe request.  To address this,
+	    we create our own timer using ast_sip_schedule_task.  This timer is
+	    used only for re-establishing subscriptions after a restart.
+
+	    An earlier approach was to add support for setting pjproject's
+	    timer (via a pjproject patch) and while that patch is still included
+	    here, we don't use that call at the moment.
+
+	  While addressing these issues, additional debugging was added and
+	  some existing messages made more useful.  A few formatting changes
+	  were also made to 'pjsip show scheduled tasks' to make displaying
+	  the subscription timers a little more friendly.
+
+	  ASTERISK-26696
+	  ASTERISK-26756
+
+	  Change-Id: I8c605fc1e3923f466a74db087d5ab6f90abce68e
+
+2017-02-15 11:03 +0000 [11886dea82]  Sean Bright <sean.bright@gmail.com>
+
+	* res_rtp_asterisk: Use PJ_ICE_MAX_CAND instead of hard-coding 16
+
+	  pjsip limits the total number of ICE candidates to PJ_ICE_MAX_CAND,
+	  which is a compile-time constant. Instead of hard-coding 16 when we
+	  enumerate local interfaces, use PJ_ICE_MAX_CAND so that we can
+	  potentially collect more interfaces if the compile time options are
+	  changed.
+
+	  Tangentially related to ASTERISK~24464
+
+	  Change-Id: I1b85509e39e33b1fed63c86261fc229ba14bbabd
+
+2016-12-22 09:42 +0000 [b58de2fab7]  Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+
+	* Binaural synthesis (confbridge): Adds utils/conf_bridge_binaural_hrir_importer
+
+	  Adds the import tool for converting a HRIR database to hrirs.h
+
+	  ASTERISK-26292
+
+	  Change-Id: I51eb31b54c23ffd9b544bdc6a09d20c112c8a547
+
+2017-02-14 12:33 +0000 [a9c15a0e4c]  Joshua Colp <jcolp@digium.com>
+
+	* stream: Add unit tests for channel stream usage.
+
+	  This change adds unit tests cover the following:
+
+	  1. That retrieving the first media stream of a specific media
+	  type from a stream topology retrieves the expected media
+	  stream.
+
+	  2. That setting the native formats of a channel which does
+	  not support streams results in the creation of streams on
+	  its behalf according to the formats of the channel.
+
+	  3. That setting a stream topology on a channel which supports
+	  streams sets the topology to the provided one.
+
+	  ASTERISK-26790
+
+	  Change-Id: Ic53176dd3e4532e8c3e97d9e22f8a4b66a2bb755
+
+2017-02-13 16:50 +0000 [275f469a4d]  Sean Bright <sean.bright@gmail.com>
+
+	* app_voicemail: Allow 'Comedian Mail' branding to be overriden
+
+	  Original patch by John Covert, slight modifications by me.
+
+	  ASTERISK-17428 #close
+	  Reported by: John Covert
+	  Patches:
+	  	app_voicemail.c.patch (license #5512) patch uploaded by
+	          John Covert
+
+	  Change-Id: Ic3361b0782e5a5397a19ab18eb8550923a9bd6a6
+
+2017-02-13 11:50 +0000 [bf2f091bbb]  George Joseph <gjoseph@digium.com>
+
+	* stream:  Add stream topology to channel
+
+	  Adds topology set and get to channel.
+
+	  ASTERISK-26790
+
+	  Change-Id: Ic379ea82a9486fc79dbd8c4d95c29fa3b46424f4
+
+2017-01-25 16:25 +0000 [2b245b12d9]  Ryan Rittgarn <rrittgarn@techpro.com>
+
+	* app_voicemail: VoiceMailPlayMsg did not play database stored messages
+
+	  When attempting to use VoiceMailPlayMsg with a realtime data backend
+	  the message is located, but never retrieved. This patch adds the
+	  required RETRIEVE and DISPOSE calls that will fetch the message from
+	  the database (and IMAP storage as well for that matter).
+
+	  Also, removed extraneous make_file call.
+
+	  ASTERISK-26723 #close
+
+	  Change-Id: I1e122dd53c0f3d7faa10f3c2b7e7e76a47d51b8c
+
+2017-02-14 08:12 +0000 [662c9e69fa]  Sean Bright <sean.bright@gmail.com>
+
+	* app_record: Add option to prevent silence from being truncated
+
+	  When using Record() with the silence detection feature, the stream is
+	  written out to the given file. However, if only 'silence' is detected,
+	  this file is then truncated to the first second of the recording.
+
+	  This patch adds the 'u' option to Record() to override that behavior.
+
+	  ASTERISK-18286 #close
+	  Reported by: var
+	  Patches:
+	  	app_record-1.8.7.1.diff (license #6184) patch uploaded by var
+
+	  Change-Id: Ia1cd163483235efe2db05e52f39054288553b957
+
+2017-02-07 11:13 +0000 [9f394d074a]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+
+	* app_queue:  reset abandoned in sl for sl2 calculations
+
+	  ASTERISK-26775 #close
+
+	  Change-Id: I86de4b1a699d6edc77fea9b70d839440e4088284
+
+2017-02-13 11:00 +0000 [6c4657e28e]  Joshua Colp <jcolp@digium.com>
+
+	* stream: Add stream topology unit tests and fix uncovered bugs.
+
+	  This change adds unit tests for the various API calls relating
+	  to stream topologies. This includes creation, destruction,
+	  inspection, and manipulation.
+
+	  Through this a few bugs were uncovered in the implementation:
+
+	  1. Creating a topology using a format capabilities would fail as
+	  the code considered a return value of 0 from the append stream
+	  function to indicate an error which is incorrect.
+
+	  2. Not all functions which placed a stream into a topology
+	  set the position on the stream itself.
+
+	  3. Appending a stream would cause a frack if the position
+	  provided was the last one. This occurred because the existing
+	  stream was queried but the index was outside of what the
+	  vector was currently at for size.
+
+	  ASTERISK-26786
+
+	  Change-Id: Id5590e87c8a605deea1a89e53169a9c011d66fa0
+
+2017-02-11 09:57 +0000 [3f94373778]  Sean Bright <sean.bright@gmail.com>
+
+	* cli: Fix various CLI documentation and completion issues
+
+	  * app_minivm: Use built-in completion facilities to complete optional
+	  arguments.
+
+	  * app_voicemail: Use built-in completion facilities to complete
+	  optional arguments.
+
+	  * app_confbridge: Add missing colons after 'Usage' text.
+
+	  * chan_alsa: Use built-in completion facilities to complete optional
+	  arguments.
+
+	  * chan_sip: Use built-in completion facilities to complete optional
+	  arguments. Add completions for 'load' for 'sip show user', 'sip show
+	  peer', and 'sip qualify peer.'
+
+	  * chan_skinny: Correct and extend completions for 'skinny reset' and
+	  'skinny show line.'
+
+	  * func_odbc: Correct completions for 'odbc read' and 'odbc write'
+
+	  * main/astmm: Use built-in completion facilities to complete arguments
+	  for 'memory' commands.
+
+	  * main/bridge: Correct completions for 'bridge kick.'
+
+	  * main/ccss: Use built-in completion facilities to complete arguments
+	  for 'cc cancel' command.
+
+	  * main/cli: Add 'all' completion for 'channel request hangup.' Correct
+	  completions for 'core set debug channel.' Correct completions for 'core
+	  show calls.'
+
+	  * main/pbx_app: Remove redundant completions for 'core show
+	  applications.'
+
+	  * main/pbx_hangup_handler: Remove unused completions for 'core show
+	  hanguphandlers all.'
+
+	  * res_sorcery_memory_cache: Add completion for 'reload' argument of
+	  'sorcery memory cache stale' and properly implement.
+
+	  Change-Id: Iee58c7392f6fec34ad9d596109117af87697bbca
+
+2017-02-10 15:45 +0000 [8b72ec312b]  George Joseph <gjoseph@digium.com>
+
+	* stream:  Add media stream topology definition and API
+
+	  This change adds the media stream topology definition and API for
+	  accessing and using it.
+
+	  Some refactoring of the stream was also done.
+
+	  ASTERISK-26786
+
+	  Change-Id: Ic930232d24d5ad66dcabc14e9b359e0ff8e7f568
+
+2017-01-13 11:21 +0000 [75f8167e66]  Norbert Varga <vnorbix@gmail.com>
+
+	* chan_pjsip: Multidomain endpoint finding on call
+
+	  When PJSIP tries to call an endpoint with a domain (e.g. 1000@test.com),
+	  the user part is stripped down as it would be a trunk with a specified user,
+	  and only the host part is called as a PJSIP endpoint and can't be found.
+	  This is not correct in the case of a multidomain SIP account, so the stripping
+	  after the @ sign is done only if the whole endpoint (in multidomain case
+	  1000@test.com) can't be found.
+
+	  ASTERISK-26248
+
+	  Change-Id: I3a2dd6f57f3bd042df46b961eccd81d31ab202e6
+
+2017-02-13 05:05 +0000 [89871576b9]  Joshua Colp <jcolp@digium.com>
+
+	* channel: Protect flags in ast_waitfor_nandfds operation.
+
+	  The ast_waitfor_nandfds operation will manipulate the flags
+	  of channels passed in. This was previously done without
+	  the channel lock being held. This could result in incorrect
+	  values existing for the flags if another thread manipulated
+	  the flags at the same time.
+
+	  This change locks the channel during flag manipulation.
+
+	  ASTERISK-26788
+
+	  Change-Id: I2c5c8edec17c9bdad4a93291576838cb552ca5ed
+
+2017-02-11 11:25 +0000 [07abb39d6a]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.c: Fix inconsistency between warning and action.
+
+	  The original return value corresponded to AST_SIP_AUTHENTICATION_CHALLENGE
+	  but we have no authenticator registered to create the challenge.
+
+	  Change-Id: I62368180d774b497411b80fbaabd0c80841f8512
+
+2017-02-11 11:26 +0000 [ce810a892b]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Fix off-nominal tdata ref leak.
+
+	  Change-Id: I571f371d0956a8039b197b4dbd8af6b18843598d
+
+2017-02-09 10:01 +0000 [0910773077]  Sean Bright <sean.bright@gmail.com>
+
+	* manager: Restore Originate failure behavior from Asterisk 11
+
+	  In Asterisk 11, if the 'Originate' AMI command failed to connect the provided
+	  Channel while in extension mode, a 'failed' extension would be looked up and
+	  run. This was, I believe, unintentionally removed in 51b6c49. This patch
+	  restores that behavior.
+
+	  This also adds an enum for the various 'synchronous' modes in an attempt to
+	  make them meaningful.
+
+	  ASTERISK-26115 #close
+	  Reported by: Nasir Iqbal
+
+	  Change-Id: I8afbd06725e99610e02adb529137d4800c05345d
+
+2017-02-08 14:27 +0000 [16fdb11bc3]  Richard Mudgett <rmudgett@digium.com>
+
+	* core: Cleanup some channel snapshot staging anomalies.
+
+	  We shouldn't unlock the channel after starting a snapshot staging because
+	  another thread may interfere and do its own snapshot staging.
+
+	  * app_dial.c:dial_exec_full() made hold the channel lock while setting up
+	  the outgoing channel staging.  Made hold the channel lock after the called
+	  party answers while updating the caller channel staging.
+
+	  * chan_sip.c:sip_new() completed the channel staging on off-nominal exit.
+	  Also we need to use ast_hangup() instead of ast_channel_unref() at that
+	  location.
+
+	  * channel.c:__ast_channel_alloc_ap() added a comment about not needing to
+	  complete the channel snapshot staging on off-nominal exit paths.
+
+	  * rtp_engine.c:ast_rtp_instance_set_stats_vars() made hold the channel
+	  locks while staging the channels for the stats channel variables.
+
+	  Change-Id: Iefb6336893163f6447bad65568722ad5d5d8212a
+
+2017-02-07 06:56 +0000 [bab4885f1e]  Joshua Colp <jcolp@digium.com>
+
+	* stream: Add media stream definition and API with unit tests.
+
+	  This change adds the media stream definition and API for
+	  accessing and using it. Unit tests have also been written
+	  which exercise aspects of the API.
+
+	  ASTERISK-26773
+
+	  Change-Id: I3dbe54065b55aaa51f467e1a3bafd67fb48cac87
+
+2017-02-10 09:35 +0000 [648d181d2f]  George Joseph <gjoseph@digium.com>
+
+	* configs/samples: Fix placement of 'identify' entry in sorcery.conf
+
+	  The entry for 'identify' was incorrectly placed in the
+	  res_pjsip section when it should be in
+	  res_pjsip_endpoint_identifier_ip.
+
+	  ASTERISK-26785 #close
+
+	  Change-Id: Ia1372b12a952bfe2df6b1b1e0e725ca306a5d41a
+
+2017-02-08 11:50 +0000 [46147a8f30]  Mark Michelson <mmichelson@digium.com>
+
+	* Revert "Update qualifies when AOR configuration changes."
+
+	  This reverts commit 6492e91392b8fd394193e411c6eb64b45486093f.
+
+	  The change in question was intended to prevent the need to reload in
+	  order to update qualifies on contacts when an AOR changes. However, this
+	  ended up causing a deadlock instead.
+
+	  Change-Id: I1a835c90a5bb65b6dc3a1e94cddc12a4afc3d71e
+
+2017-02-07 12:01 +0000 [5422ec140c]  Joshua Colp <jcolp@digium.com>
+
+	* srv: Fix crash when ast_srv_lookup is used and 0 records are returned.
+
+	  When performing an SRV lookup using the ast_srv_lookup function it
+	  did not properly handle the situation where 0 records are returned.
+	  If this happened it would wrongly assume that at least one record
+	  was present.
+
+	  This change fixes the code so it will exit early if an error occurs
+	  or if 0 records are returned.
+
+	  ASTERISK-26772
+	  patches:
+	    srv_lookup.patch submitted by nappsoft (license 6822)
+
+	  Change-Id: I09b19081c74e0ad11c12bf54a257243b1bcb2351
+
+2017-02-06 11:40 +0000 [b79cc62057]  Joshua Colp <jcolp@digium.com>
+
+	* res_stasis_device_state: Protect the adding/removing of subscriptions.
+
+	  The adding and removing of device state subscriptions did not protect
+	  fully against simultaneous manipulation. In particular the subscribe
+	  case allowed a small window where two subscriptions could be added for
+	  the same device state instead of just one.
+
+	  This change makes the code hold the subscriptions lock for the entirety
+	  of each operation to ensure that two are not occurring at the same time.
+
+	  ASTERISK-26770
+
+	  Change-Id: I3e7f8eb9d09de440c9024d2dd52029f6f20e725b
+
+2017-02-01 17:56 +0000 [b47cf1a7d6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Fix some off nominal tdata leaks.
+
+	  Change-Id: I243a4be5e7fbfe604923764969c4ee04eee89b9d
+
+2017-02-03 15:26 +0000 [7b280e7ccf]  Sebastien Duthil <sduthil@wazo.community>
+
+	* res_ari: fix memory leak for channelvars
+
+	  In ari.conf, when setting the option channelvars, every Stasis channel
+	  snapshot would create a list of variable/value that would not be freed
+	  when the snapshot is freed, resulting in a often-recurring memory
+	  leak.
+
+	  ASTERISK-26767 #close
+
+	  Change-Id: Ia37dd9d68063d7f879193df02ede293e5ded716d
+
+2017-02-03 02:25 +0000 [c6c7f17206]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* libasteriskssl: do nothing with OpenSSL >= 1.1
+
+	  OpenSSL 1.1 requires no explicit initialization. The hacks in the
+	  library are not needed. They also happen to fail running Asterisk.
+
+	  Change-Id: I3b3efd5d80234a4c45a8ee58dcfe25b15d9ad100
+
+2017-01-20 23:59 +0000 [bc041ca14a]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* tcptls: use TLS_client_method with OpenSSL 1.1
+
+	  OpenSSL 1.1 introduced TLS_client_method() and deprecated the previous
+	  version-specific methods (such as TLSv1_client_method(). Other than
+	  being simpler to use and more correct (gain support for TLS newer that
+	  TLS1, in our case), the older ones produce a deprecation warning that
+	  fails the build in dev-mode.
+
+	  Change-Id: I257b1c8afd09dcb0d96cda3a41cb9f7a15d0ba07
+
+2017-01-20 23:57 +0000 [2c8d0764de]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* openssl 1.1 support: use OPENSSL_VERSION_NUMBER
+
+	  Use OPENSSL_VERSION_NUMBER instead of OPENSSL_API_COMPAT to detect
+	  the openssl 1.1 API.
+
+	  Change-Id: I4e448f55ef516aedf6ad154037c35577a421a458
+
+2017-01-31 18:28 +0000 [50029f585e]  Richard Mudgett <rmudgett@digium.com>
+
+	* channel.c: Fix unbalanced read queue deadlocking local channels.
+
+	  Using the timerfd timing module can cause channel freezing, lingering, or
+	  deadlock issues.  The problem is because this is the only timing module
+	  that uses an associated alert-pipe.  When the alert-pipe becomes
+	  unbalanced with respect to the number of frames in the read queue bad
+	  things can happen.  If the alert-pipe has fewer alerts queued than the
+	  read queue then nothing might wake up the thread to handle received frames
+	  from the channel driver.  For local channels this is the only way to wake
+	  up the thread to handle received frames.  Being unbalanced in the other
+	  direction is less of an issue as it will cause unnecessary reads into the
+	  channel driver.
+
+	  ASTERISK-26716 is an example of this deadlock which was indirectly fixed
+	  by the change that found the need for this patch.
+
+	  * In channel.c:__ast_queue_frame(): Adding frame lists to the read queue
+	  did not add the same number of alerts to the alert-pipe.  Correspondingly,
+	  when there is an exceptionally long queue event, any removed frames did
+	  not also remove the corresponding number of alerts from the alert-pipe.
+
+	  ASTERISK-26632 #close
+
+	  Change-Id: Ia98137c5bf6e9d6d202ce0eb36441851875863f6
+
+2017-01-31 16:38 +0000 [97c308471d]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_agi: Prevent an AGI from eating frames it should not. (Re-do)
+
+	  A dialplan intercept routine is equivalent to an interrupt routine.  As
+	  such, the routine must be done quickly and you do not have access to the
+	  media stream.  These restrictions are necessary because the media stream
+	  is the responsibility of some other code and interfering with or delaying
+	  that processing is bad.  A possible future dialplan processing
+	  architecture change may allow the interception routine to run in a
+	  different thread from the main thread handling the media and remove the
+	  execution time restriction.
+
+	  * Made res_agi.c:run_agi() running an AGI in an interception routine run
+	  in DeadAGI mode.  No touchy channel frames.
+
+	  ASTERISK-25951
+
+	  ASTERISK-26343
+
+	  ASTERISK-26716
+
+	  Change-Id: I638f147ca7a7f2590d7194a8ef4090eb191e4e43
+
+2017-01-31 16:32 +0000 [72e3fc5845]  Richard Mudgett <rmudgett@digium.com>
+
+	* Frame deferral: Revert API refactoring.
+
+	  There are several issues with deferring frames that are caused by the
+	  refactoring.
+
+	  1) The code deferring frames mishandles adding a deferred frame to the
+	  deferred queue.  As a result the deferred queue can only be one frame
+	  long.
+
+	  2) Deferrable frames can come directly from the channel driver as well as
+	  the read queue.  These frames need to be added to the deferred queue.
+
+	  3) Whoever is deferring frames is really only doing the __ast_read() to
+	  collect deferred frames and doesn't care about the returned frames except
+	  to detect a hangup event.  When frame deferral is completed we must make
+	  the normal frame processing see the hangup as a frame anyway.  As such,
+	  there is no need to have varying hangup frame deferral methods.  We also
+	  need to be aware of the AST_SOFTHANGUP_ASYNCGOTO hangup that isn't real.
+	  That fake hangup is to cause the PBX thread to break out of loops to go
+	  execute a new dialplan location.
+
+	  4) To properly deal with deferrable frames from the channel driver as
+	  pointed out by (2) above, means that it is possible to process a dialplan
+	  interception routine while frames are deferred because of the
+	  AST_CONTROL_READ_ACTION control frame.  Deferring frames is not
+	  implemented as a re-entrant operation so you could have the unsupported
+	  case of two sections of code thinking they have control of the media
+	  stream.
+
+	  A worse problem is because of the bad implementation of the AMI PlayDTMF
+	  action.  It can cause two threads to be deferring frames on the same
+	  channel at the same time.  (ASTERISK_25940)
+
+	  * Rather than fix all these problems simply revert the API refactoring as
+	  there is going to be only autoservice and safe_sleep deferring frames
+	  anyway.
+
+	  ASTERISK-26343
+
+	  ASTERISK-26716 #close
+
+	  Change-Id: I45069c779aa3a35b6c863f65245a6df2c7865496
+
+2017-02-02 11:26 +0000 [4c51ad158d]  Sean Bright <sean.bright@gmail.com>
+
+	* res_odbc: Remove deprecated settings from sample configuration file
+
+	  ASTERISK-26704 #close
+	  Reported by: Anthony Messina
+
+	  Change-Id: I976a1f94cf79c5f31e76174c61f5c6a65fd6354f
+
+2017-02-01 17:14 +0000 [7d9b50a7b2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_resolver_unbound.c: Fix frequent ref leak caught by excessive ref trap.
+
+	  ASTERISK-26765
+
+	  Change-Id: I27eb97df7f8d7e624b0b9a61c0fcee4718c86d8d
+
+2017-02-01 15:56 +0000 [2849b726b6]  Sean Bright <sean.bright@gmail.com>
+
+	* audiohooks:  Muting a hook can mute underlying frames
+
+	  If an audiohook is placed on a channel that does not require transcoding,
+	  muting that hook will cause the underlying frames to be muted as well.
+
+	  The original patch is from David Woolley but I have modified slightly.
+
+	  ASTERISK-21094 #close
+	  Reported by: David Woolley
+	  Patches:
+	        ASTERISK-21094-Patch-1.8-1.txt (license #5737) patch uploaded
+	        by David Woolley
+
+	  Change-Id: Ib2b68c6283e227cbeb5fa478b2d0f625dae338ed
+
+2017-02-01 13:54 +0000 [bbed75c3ba]  Mark Michelson <mmichelson@digium.com>
+
+	* Update qualifies when AOR configuration changes.
+
+	  Prior to this change, qualifies would only update in the following
+	  cases:
+	  * A reload of res_pjsip.so was issued.
+	  * A dynamic contact was re-registered after its AOR's qualify_frequency
+	    had been changed
+	  This does not work well if you are using realtime for your AORs. You can
+	  update your database to have a new qualify_frequency, but the permanent
+	  contacts on that AOR will not have their qualifies updated. And the
+	  dynamic contacts on that AOR will not have their qualifies updated until
+	  the next registration, which could be a long time.
+
+	  This change seeks to fix this problem by making it so that whenever AOR
+	  configuration is applied, the contacts pertaining to that AOR have their
+	  qualifies updated.
+
+	  Additions from this patch:
+	  * AOR sorcery objects now have an apply handler that calls into a newly
+	    added function in the OPTIONS code. This causes all contacts
+	    associated with that AOR to re-schedule qualifies.
+	  * When it is time to qualify a contact, the OPTIONS code checks to see
+	    if the AOR can still be retrieved. If not, then qualification is
+	    canceled on the contact.
+
+	  Alterations from this patch:
+	  * The registrar code no longer updates contact's qualify_frequence and
+	    qualify_timeout. There is no point to this since those values already
+	    get updated when the AOR changes.
+	  * Reloading res_pjsip.so no longer calls the OPTIONS initialization
+	    function. Reloading res_pjsip.so results in re-loading AORs, which
+	    results in re-scheduling qualifies.
+
+	  Change-Id: I2e7c3316da28f389c45954f24c4e9389abac1121
+
+2017-01-31 11:17 +0000 [aeea634bc0]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Handle invocation of callback on outgoing request when error occurs.
+
+	  There are some error cases in PJSIP when sending a request that will
+	  result in the callback for the request being invoked.  The code did not
+	  handle this case and assumed on every error case that the callback was not
+	  invoked.
+
+	  The code has been changed to check whether the callback has been invoked
+	  and if so to absorb the error and treat it as a success.
+
+	  ASTERISK-26679
+	  ASTERISK-26699
+
+	  Change-Id: I563982ba204da5aa1428989a11c06dd9087fea91
+
+2017-01-30 09:02 +0000 [7a16524a83]  Sean Bright <sean.bright@gmail.com>
+
+	* res_rtp_asterisk:  Swap byte-order when sending signed linear
+
+	  Before Asterisk 13, signed linear was converted into network byte order by a
+	  smoother before being sent over the network. We restore this behavior by
+	  forcing the creation of a smoother when slinear is in use and setting the
+	  appropriate flags so that the byte order conversion is always done.
+
+	  ASTERISK-24858 #close
+	  Reported-by: Frankie Chin
+
+	  Change-Id: I868449617d1a7819578f218c8c6b2111ad84f5a9
+
+2017-01-31 12:46 +0000 [e252aff9ad]  George Joseph <gjoseph@digium.com>
+
+	* debug_utilities: Install ast_logescalator to /var/lib/asterisk/scripts
+
+	  Forgot to install it with the original patch
+
+	  Change-Id: I8bdb540a6694971ae5fe21f48d532332c6482e4c
+
+2017-01-25 06:50 +0000 [ef4deb8ecd]  George Joseph <gjoseph@digium.com>
+
+	* debug_utilities:  Add ast_logescalator
+
+	  The escalator works by creating a set of startup commands in cli.conf
+	  that set up logger channels and issue the debug commands for the
+	  subsystems specified.  If asterisk is running when it is executed,
+	  the same commands will be issued to the running instance.  The original
+	  cli.conf is saved before any changes are made and can be restored by
+	  executing '$prog --reset'.
+
+	  The log output will be stored in...
+	  $astlogdir/message.$uniqueid
+	  $astlogdir/debug.$uniqueid
+	  $astlogdir/dtmf.$uniqueid
+	  $astlogdir/fax.$uniqueid
+	  $astlogdir/security.$uniqueid
+	  $astlogdir/pjsip_history.$uniqueid
+	  $astlogdir/sip_history.$uniqueid
+
+	  Some minor tweaks were made to chan_sip, and res_pjsip_history
+	  so their history output could be send to a log channel as packets
+	  are captured.
+
+	  A minor tweak was also made to manager so events are output to verbose
+	  when "manager set debug on" is issued.
+
+	  Change-Id: I799f8e5013b86dc5282961b27383d134bf09e543
+
+2017-01-23 09:35 +0000 [178b90af02]  Torrey Searle <torrey@voxbone.com>
+
+	* libastssl/pj: libastssl/pj should have an so_version
+
+	  Issue introduced in b59956a87.  In the non-darwin case libastssl/pj
+	  should be versioned.  This causes the symbol file for this lib
+	  to not be generated.
+
+	  Change-Id: Ib07ae8c40252813c488e2c1ac6204fd42816dd4c
+	  (cherry picked from commit 54b027916a71f2b83b2050cef5ef704ea5de39b2)
+
+2017-01-24 19:51 +0000 [138cd8d019]  Kirill Katsnelson <kkm@smartaction.com>
+
+	* make_build_h: handle backslashes in external strings
+
+	  LikewiseOpen creates user names with a backslash in them. A gentle
+	  massage with sed(1) allows such strings to be inserted into build.h
+	  properly quoted. I am also adding the same for host name and other
+	  strings used in the script that are more or less user-controlled.
+
+	  ASTERISK-26754
+
+	  Change-Id: Iac5ef2b67a68ee58f35ddbf86bb818ba6eabecae
+
+2017-01-24 22:31 +0000 [8270d2436d]  Kirill Katsnelson <kkm@smartaction.com>
+
+	* app_queue: Fix queues randomly disappearing on reload
+
+	  With 500+ queues and a reload every minute, a random queue disappears
+	  upon reload. The cause is mususe of the 'dead' flag. Namely, all queues
+	  were marked dead up front, and then "resurrected" by dropping this flag
+	  for those found in the configuration. But a queue marked dead can be
+	  removed also when control leaves the app entry point on a PBX thread.
+
+	  With this change, the queue is marked only not found, and at the end of
+	  reload only the queues that are still not found are actually marked as
+	  dead, so the dead flag is never reset, and set only on positively dead
+	  queues.
+
+	  ASTERISK-26755
+
+	  Change-Id: I3a4537aec9eb8d8aeeaa0193407e3523feb004bf
+
+2017-01-26 07:57 +0000 [7fa3de7ae9]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_endpoint_identifier_ip: Fix memory leak of hosts when resolving.
+
+	  This change adds a missing unreference of the hostname when resolving and
+	  also cleans up the iterator.
+
+	  ASTERISK-26735
+
+	  Change-Id: Ic012ebaf3d89e714eec340b7b0c5e63c66af857a
+
+2017-01-25 15:26 +0000 [d32bd63860]  Mark Michelson <mmichelson@digium.com>
+
+	* Add reload options to CLI/AMI stale object commands.
+
+	  Marking an object as stale in a memory cache is supposed to prime the
+	  cache so that the next time the item is retrieved, the stale item is
+	  deleted from the cache and a background task is run to re-populate the
+	  cache with a fresh version of the object.
+
+	  The problem is, there are some object types out there for which there is
+	  no natural reason that they would be retrieved from the backend with any
+	  regularity. Outbound PJSIP registrations are a good example of this. At
+	  startup, they are read, and an object-specific state is created that
+	  refers to the initially-retrieved object for all time.
+
+	  Adding the "reload" option to the CLI/AMI commands gives the cache the
+	  opportunity to manually re-retrieve the object from the backend, both
+	  storing the new object in the cache and applying the new object's
+	  configuration to the module that uses that object.
+
+	  Change-Id: Ieb1fe7270ceed491f057ec5cbf0e097bde96c5c8
+
+2017-01-10 17:39 +0000 [20aed30d9a]  Richard Mudgett <rmudgett@digium.com>
+
+	* T.140: Fix format ref and memory leaks.
+
+	  * channel.c:ast_sendtext(): Fix T.140 SendText memory leak.
+
+	  * format_compatibility.c: T.140 RED and T.140 were swapped.
+
+	  * res_rtp_asterisk.c:rtp_red_init(): Fix ast_format_t140_red ref leak.
+
+	  * res_rtp_asterisk.c:rtp_red_init(): Fix data race after starting periodic
+	  scheduled red_write().
+
+	  * res_rtp_asterisk.c: Some other minor misc tweaks.
+
+	  Change-Id: Ifa27a2e0f8a966b1cf628607c86fc4374b0b88cb
+
+2017-01-24 15:39 +0000 [ee2b0f2eef]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_endpoint_identifier_ip: Ensure error defaults to 0.
+
+	  When configuring a match using a netmask the error variable was
+	  not defaulting to 0. For some people this would cause the code
+	  to think an error occurred when adding the match when in reality
+	  it added perfectly fine.
+
+	  ASTERISK-26693
+
+	  Change-Id: I850c250813742bddde65c84e739093c9e01dfe56
+
+2017-01-10 17:37 +0000 [930a24a730]  Richard Mudgett <rmudgett@digium.com>
+
+	* astobj2.c: Add excessive ref count trap.
+
+	  Change-Id: I32e6a589cf9009450e4ff7cb85c07c9d9ef7fe4a
+
+2017-01-10 13:11 +0000 [de28c1b9f1]  Richard Mudgett <rmudgett@digium.com>
+
+	* main/app.c: Memory corruption from early format destruction.
+
+	  * make_silence() created a malloced silence slin frame without adding a
+	  slin format ref.  When the frame is destroyed it will unref the slin
+	  format that never had a ref added.  Memory corruption is expected to
+	  follow.
+
+	  * Simplified and fixed counting the number of samples in a frame list for
+	  make_silence().
+
+	  * Eliminated an unnecessary RAII_VAR associated with the make_silence()
+	  frame.
+
+	  Change-Id: I47de3f9b92635b7f8b4d72309444d6c0aee6f747
+
+2017-01-11 14:59 +0000 [2039eb8edf]  Richard Mudgett <rmudgett@digium.com>
+
+	* frame.c: Fix off-nominal format ref leaks.
+
+	  * ast_frisolate() could leak frame format refs on allocation
+	  failures.
+
+	  * Similified code in ast_frisolate() and code used by
+	  ast_frisolate().
+
+	  Change-Id: I79566d4d36b3d7801bf0c8294fcd3e9a86a2ed6d
+
+2017-01-13 19:08 +0000 [e922979d49]  Richard Mudgett <rmudgett@digium.com>
+
+	* stasis_bridge.c: Fix off-nominal stasis control ref leak.
+
+	  Change-Id: Ib17218343a6596832060180e19386da9df150ac8
+
+2017-01-10 12:30 +0000 [56854f22d2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_musiconhold.c: Fix format ref leak when parsing MOH config class.
+
+	  Change-Id: Ica8e8e2ce7604c2c61ec55bef07dc675361d2ea5
+
+2017-01-10 14:03 +0000 [d87f81ddb1]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_oss.c: Fix format ref leak in oss_read().
+
+	  Change-Id: I0a5d56c7dcf327d60f86a4c25a23571733709fd0
+
+2017-01-10 17:48 +0000 [36bdd7c1a0]  Richard Mudgett <rmudgett@digium.com>
+
+	* Add notes about embedded ast_frame structs holding a format ref.
+
+	  mod_format.h: Note ast_filestream.fr holds a format ref.
+
+	  translate.h: Note ast_trans_pvt.f holds a format ref.
+
+	  Change-Id: I86bda354d725207b41e08920355d7c31b2d7f749
+
+2017-01-20 21:13 +0000 [6f3e8c8e01]  Richard Mudgett <rmudgett@digium.com>
+
+	* PJPROJECT logging: Fix detection of max supported log level.
+
+	  The mechanism used for detecting the maximum log level compiled into the
+	  linked pjproject did not work.  The API call simply stores the requested
+	  level into an integer and does no range checking.  Asterisk was assuming
+	  that there was range checking and limited the new value to the allowable
+	  range.  To get the actual maximum log level compiled into the linked
+	  pjproject we need to get and save off the initial set log level from
+	  pjproject.  This is the maximum log level supported.
+
+	  * Get and save off the initial log level setting before altering it to the
+	  desired level on startup.  This has to be done by a macro rather than
+	  calling a core function to avoid incorrectly linking pjproject.
+
+	  * Split the initial log level warning messages to warn if the linked
+	  pjproject cannot support the requested startup level and if it is too low
+	  to get the pjproject buildopts for "pjproject show buildopts".
+
+	  * Adjust the CLI "pjproject set log level" to check the saved max log
+	  level and to generate normal output messages instead of a warning message.
+
+	  ASTERISK-26743 #close
+
+	  Change-Id: I40aa76653e2a1dece66c3f8734594b4f0471cfb4
+
+2017-01-05 13:21 +0000 [0ea3c371c5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Implement "pjsip show subscriptions" commands.
+
+	  ASTERISK-23828 #close
+
+	  Change-Id: Ifb8a3b61f447aedc58a8e6b36a810f7566018567
+
+2017-01-23 16:18 +0000 [4bfeda6ee4]  Mark Michelson <mmichelson@digium.com>
+
+	* Free endpoint ACLs when destroying PJSIP endpoints.
+
+	  If endpoint ACLs were specified, they were not being freed
+	  when endpoints were destroyed. On systems with realtime endpoints, this
+	  could add up quickly since each DB lookup would allocate the ACL without
+	  freeing it.
+
+	  ASTERISK-26731 #close
+	  Reported by Ustinov Artem
+
+	  Change-Id: Ie1f8bf5b7a0de628c975beba01e69c56893331ad
+
+2017-01-19 09:05 +0000 [6691606723]  George Joseph <gjoseph@digium.com>
+
+	* ari: Implement 'debug all' and request/response logging
+
+	  The 'ari set debug' command has been enhanced to accept 'all' as an
+	  application name.  This allows dumping of all apps even if an app
+	  hasn't registered yet.  To accomplish this, a new global_debug global
+	  variable was added to res/stasis/app.c and new APIs were added to
+	  set and query the value.
+
+	  'ari set debug' now displays requests and responses as well as events.
+	  This required refactoring the existing debug code.
+
+	  * The implementation for 'ari set debug' was moved from stasis/cli.{c,h}
+	    to ari/cli.{c,h}, and stasis/cli.{c,h} were deleted.
+	  * In order to print the body of incoming requests even if a request
+	    failed, the consumption of the body was moved from the ari stubs
+	    to ast_ari_callback in res_ari.c and the moustache templates were
+	    then regenerated.  The body is now passed to ast_ari_invoke and then
+	    on to the handlers.  This results in code savings since that template
+	    was inserted multiple times into all the stubs.
+
+	  An additional change was made to the ao2_str_container implementation
+	  to add partial key searching and a sort function.  The existing cli
+	  code assumed it was already there when it wasn't so the tab completion
+	  was never working.
+
+	  Change-Id: Ief936f747ce47f1fb14035fbe61152cf766406bf
+	  (cherry picked from commit 1d890874f39a5a81b20da44358143ed9b54ab0fe)
+
+2017-01-20 23:41 +0000 [f3f9175df0]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* test_voicemail_api: order of params to VERIFY macros
+
+	  Fix order of parameters in calls to VM_API_INT_VERIFY and
+	  VM_API_STRING_VERIFY
+
+	  ASTERISK-26739 #close
+
+	  Change-Id: I30dc6b36893aadad6012be3f16f93aa5720870d6
+	  Note: status: builds. Not tested any further.
+
+2017-01-23 09:10 +0000 [96e7291cbd]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled: Fix setting max log level
+
+	  An earlier attempt to prevent pjsua from spitting out an extra 6795
+	  lines of debug output every time the testsuite called it was also
+	  turning off the ability for asterisk to output debug info when it
+	  needed to.  This patch reverts the earlier fix and instead adds
+	  a pjproject patch that sets the startup log level to 1 for pjsua
+	  pjsystest and the pjsua python binding.  This is an asterisk-only
+	  patch that does not affect pjproject functionality and will not be
+	  submitted upstream.
+
+	  Change-Id: I347a8b58b2626f2906ccfc1d339e907627a0c9e8
+
+2017-01-23 10:08 +0000 [23690c1b35]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_endpoint_identifier_ip: Read settings before resolving.
+
+	  An option has been added, srv_lookups, which controls whether
+	  SRV lookups are performed on the provided match hosts or not.
+	  It was possible for this option to be applied after resolution
+	  had already happened.
+
+	  This change makes it so hosts are stored away, settings are read
+	  and applied, and then resolution is done. This ensures that no
+	  matter the ordering the srv_lookups option is in effect.
+
+	  ASTERISK-26735
+
+	  Change-Id: I750378cb277be0140f8c5539450270afbfc43388
+
+2016-11-29 09:31 +0000 [1061539b75]  Lorenzo Miniero <lminiero@gmail.com>
+
+	* media: Add experimental support for RTCP feedback.
+
+	  This change adds experimental support for providing RTCP
+	  feedback information to codec modules so they can dynamically
+	  change themselves based on conditions.
+
+	  ASTERISK-26584
+
+	  Change-Id: Ifd6aa77fb4a7ff546c6025900fc2baf332c31857
+
+2017-01-22 17:25 +0000 [cfe72c39cf]  Richard Mudgett <rmudgett@digium.com>
+
+	* LISTFILTER: Remove outdated ERROR message.
+
+	  Feeding LISTFILTER an empty variable results in an invalid ERROR message.
+	  Earlier changes made the message useless because we can no longer tell if
+	  the variable is empty or does not exist.  It is valid to try to remove a
+	  value from an empty list just as it is valid to try to remove a value that
+	  is not in a non-empty list.
+
+	  * Removed the outdated ERROR message.
+
+	  * Added more test cases to the LISTFILTER unit test.
+
+	  Change-Id: Ided9040e6359c44a335ef54e02ef5950a1863134
+
+2017-01-21 14:43 +0000 [dbb9c8141d]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* tests: use datadir for sound files
+
+	  Some (voicemail-related) tests API symlinks beep.gsm and other files
+	  from ast_config_AST_VAR_DIR. It should use ast_config_AST_DATA_DIR.
+
+	  ASTERISK-26740 #close
+
+	  Change-Id: Id49c56fb9e16df64b1a2b829693ca7601252df89
+
+2017-01-05 15:11 +0000 [ef9164b9ca]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Fix AMI event list counts.
+
+	  Fix the AMI PJSIPShowSubscriptionsInbound, PJSIPShowSubscriptionsOutbound,
+	  and PJSIPShowResourceLists actions event counts.  The reported counts may
+	  not necessarily be accurate depending on what happens.
+
+	  The subscriptions count would be wrong if Asterisk ever has outbound
+	  subscriptions.
+
+	  The resource list count could be wrong if a list were added or removed
+	  during the AMI action being processed.
+
+	  Change-Id: I4344301827523fa174960a42c413fd19abe4aed5
+
+2017-01-05 13:02 +0000 [ab858295a2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Fix incorrect message string wrapping.
+
+	  Change-Id: Id771e6fe56d89ce365ddcbb423f820af97211120
+
+2017-01-05 13:01 +0000 [6d648185bc]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Eliminate trivial SCOPED_LOCK usage.
+
+	  Change-Id: Ie0b69a830385452042fa19e7d267c6790ec6b6be
+
+2017-01-05 12:58 +0000 [90f3b1270c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: alloca can never fail.
+
+	  Change-Id: Ia2a6158e5fdf311bc2a1c0c43417978de504b1f1
+
+2017-01-13 11:03 +0000 [d16b3a9917]  George Joseph <gjoseph@digium.com>
+
+	* debug_utilities:  Create ast_loggrabber
+
+	  ast_loggrabber gathers log files from customizable search patterns,
+	  optionally converts POSIX timestamps to a readable format and
+	  tarballs the results.
+
+	  Also a few tweaks were made to ast_coredumper.
+
+	  Change-Id: I8bfe1468ada24c1344ce4abab7b002a59a659495
+	  (cherry picked from commit c70915287837704090d75f181525765de7a17221)
+
+2017-01-01 03:47 +0000 [48730ae65e]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_authenticator_digest.c: Fix spacing in warning messages.
+
+	  Change-Id: I573f0343c0c63a785cd4da60d57cc9f8b9ce7f49
+
+2016-12-22 04:07 +0000 [40b9766a31]  Martin Tomec <tomec.martin@gmail.com>
+
+	* app_queue: add RINGCANCELED log event on caller hang up
+
+	  QueueLog did not log ringnoanswer when the caller abandoned call
+	  before first timeout. It was impossible to get agent membername
+	  and ringing duration for this short calls. After some discusions
+	  it seems that the best way is to add new event RINGCANCELED,
+	  which is generated after caller hangup during ringing.
+
+	  ASTERISK-26665
+
+	  Change-Id: Ic70f7b0f32fc95c9378e5bcf63865519014805d3
+
+2017-01-12 15:58 +0000 [283c16c6b6]  Kevin Harwell <kharwell@digium.com>
+
+	* abstract/fixed/adpative jitter buffer: disallow frame re-inserts
+
+	  It was possible for a frame to be re-inserted into a jitter buffer after it
+	  had been removed from it. A case when this happened was if a frame was read
+	  out of the jitterbuffer, passed to the translation core, and then multiple
+	  frames were returned from said translation core. Upon multiple frames being
+	  returned the first is passed on, but sebsequently "chained" frames are put
+	  back into the read queue. Thus it was possible for a frame to go back into
+	  the jitter buffer where this would cause problems.
+
+	  This patch adds a flag to frames that are inserted into the channel's read
+	  queue after translation. The abstract jitter buffer code then checks for this
+	  flag and ignores any frames marked as such.
+
+	  Change-Id: I276c44edc9dcff61e606242f71274265c7779587
+
+2016-11-06 06:30 +0000 [8cc1cd5df7]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+
+	* app_queue: Add QueueUpdate application.
+
+	  Add an application that allows tracking outbound calls
+	  using app_queue.
+
+	  ASTERISK-19862
+
+	  Change-Id: Ia0ab64aed934c25b2a25022adcc7c0624224346e
+
+2017-01-13 21:23 +0000 [f4e77a5678]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Change when high water warning logged.
+
+	  The task processor queue reached X scheduled tasks message was originally
+	  intended to get logged only once per task processor to prevent spamming
+	  the log.  This is no longer necessary since high and low water thresholds
+	  can better control when the message is logged.
+
+	  It is beneficial to generate the warning each time a task processor
+	  reaches the high water level because PJSIP stops processing new requests
+	  while any high water alert is active.  Without this change you would have
+	  to enable at least debug level 3 logging to know about a repeated alert
+	  trigger.
+
+	  * Made generate the warning message whenever a task is pushed into the
+	  task processor that triggers the high water alert.
+
+	  * Appended 'again' to the warning for a repeated high water alert trigger.
+
+	  Change-Id: Iabf75a004f7edaf1e5e8c323099418e667cac999
+
+2017-01-10 05:54 +0000 [e0e502d9d2]  Aaron An <anjb@ti-net.com.cn>
+
+	* res_rtp_asterisk:  Fix bug in function CHANNEL(rtcp, all_rtt)
+
+	  Function CHANNEL(rtcp,all_rtt) CHANNEL(rtcp,all_loss) CHANNEL(rtcp,all_jitter)
+	  always return 0.0 due to wrong define of macro "AST_RTP_SATA_SET" and
+	  "AST_RTP_STAT_STRCPY".
+	  It should compare "combined" with "stat" not "current_stat".
+
+	  ASTERISK-26710 #close
+	  Reported-by: Aaron An
+	  Tested-by: AaronAn
+
+	  Change-Id: Id4140fafbf92e2db689dac5b17d9caa009028a15
+
+2017-01-10 18:10 +0000 [0d53c91fba]  George Joseph <gjoseph@digium.com>
+
+	* debug_utilities:  Create the ast_coredumper utility
+
+	  This utility allows easy manipulation of asterisk coredumps.
+
+	  * Configurable search paths and patterns for existing coredumps
+	  * Can generate a consistent coredump from the running instance
+	  * Can dump the lock_infos table from a coredump
+	  * Dumps backtraces to separate files...
+	    - thread apply 1 bt full -> <coredump>.thread1.txt
+	    - thread apply all bt -> <coredump>.brief.txt
+	    - thread apply all bt full -> <coredump>.full.txt
+	    - lock_infos table -> <coredump>.locks.txt
+	  * Can tarball corefiles and optionally delete them after processing
+	  * Can tarball results files and optionally delete them after processing
+	  * Converts ':' in coredump and results file names '-' to facilitate
+	    uploading.  Jira for instance, won't accept file names with colons
+	    in them.
+
+	  Tested on Fedora24+, Ubuntu14+, Debian6+, CentOS6+ and FreeBSD9+[1].
+
+	  [1] For *BSDs, the "devel/gdb" package might have to be installed to
+	  get a recent gdb.  The utility will check all instances of gdb
+	  it finds in $PATH and if one isn't found that can run python, it
+	  prints a friendly error.
+
+	  Change-Id: I935d37ab9db85ef923f32b05579897f0893d33cd
+	  (cherry picked from commit cb47b4556053cd50d9102eef913671ad0306062d)
+
+2017-01-08 10:29 +0000 [e54c8aec34]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fix compilation with MALLOC_DEBUG
+
+	  When MALLOC_DEBUG was specified, make was failing.  Immediately
+	  remaking would work.  The issues was in the ordering of the make
+	  dependencies.
+
+	  Change-Id: If6030b54fc693f3179f32bfd20c6b5d5f1b3f7cd
+
+2017-01-05 06:11 +0000 [a7d856cd96]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_endpoint_identifier_ip: Add support for SRV lookups.
+
+	  This change implements SRV support for the IP based endpoint
+	  identifier module. All possible addresses through SRV are looked
+	  up and added as matches. If no SRV records are available a
+	  fallback to normal host resolution is done. If an IP address
+	  is provided then no SRV lookup occurs.
+
+	  This is configured using the "srv_lookups" option on the
+	  identify section and defaults to "yes".
+
+	  ASTERISK-26693
+
+	  Change-Id: I6b641e275bf96629320efa8b479737062aed82ac
+
+2016-11-06 06:37 +0000 [740ca862e4]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+
+	* app_queue: add new Service Level calculation
+
+	  Adds a new formula for SL2 and documentation
+
+	  ASTERISK-26559
+
+	  Change-Id: I0970c620460507cd9d45b0d43600779c8915e770
+
+2016-12-19 15:03 +0000 [d96e350256]  Jonathan R. Rose <jonathan.rose@motorolasolutions.com>
+
+	* core/pbx: dialplan show - display filename/line#
+
+	  Adds the ability for extensions to be registered to include filename and
+	  line number so that dialplan show output can show the filename and line
+	  number of a config file responsible for generating a given extension.
+
+	  This only affects config modules that are written to use the new extension
+	  registering functions. In this patch, that only includes pbx_config, so
+	  extensions registered in extensions.conf and any included extension will
+	  be shown in this manner. Extensions registered in this manner will show
+	  the filename and line number *instead* of the registrar.
+
+	  ASTERISK-26658 #close
+	  Reported by: Jonathan R. Rose
+
+	  Change-Id: Ieccc6abccdff34ed5c7da3511fd24972b8f2dd30
+
+2016-12-22 09:13 +0000 [aea2285865]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_pjsip_session: Access SIPDOMAIN via Dialplan.
+
+	  This feature was available in the SIP channel driver chan_sip. For example,
+	  Asterisk is the outbound proxy and has to handle all SIP-URIs, even domains not
+	  local to Asterisk. In that case, SIPDOMAIN is used in the Dialplan, to detect
+	  and dial remote SIP-URIs. This change here sets the SIP destination domain of
+	  an inbound call (SIPDOMAIN) in the SIP channel driver res_pjsip as well.
+
+	  ASTERISK-26670 #close
+
+	  Change-Id: I27c880dc404a3c1c6792e1ba3545475339577243
+
+2017-01-04 05:50 +0000 [e220c11bec]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Remember SDP negotiation on SIP_CODEC_INBOUND.
+
+	  After a SIP_CODEC_INBOUND in the dialplan, do not continue with cached formats
+	  but remember the joint format. Cached formats contain default parameters,
+	  often create an empty fmtp line. However, a joint format might have passed
+	  format_get_joint(.) in a res_format_attr_* module (like Opus Codec) and
+	  contain the resulting format parameters from a SDP negotiation.
+
+	  ASTERISK-26691 #close
+
+	  Change-Id: I35712d98a793d4c3efdd156cec57deab9014b1dc
+
+2017-01-03 15:14 +0000 [ceb9dae566]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Compile pjsua with max log level = 2
+
+	  A while back, we changed config_site.h to set PJ_LOG_MAX_LEVEL = 6.
+	  This allowed us to control the log level better from inside Asterisk.
+	  An unfortunate side effect of this was that the pjsua binary and
+	  python bindings were also compiled with log level set to 6 so whenever
+	  a testsuite test that uses pjsua runs, it spits out 6795 lines of
+	  debug in an instant even before the test starts.  I believe this
+	  overruns the Jenkins capture buffer and prevents the test from
+	  properly terminating.  In turn, this results in the testsuite just
+	  hanging until the job is killed.  It's more frequent on the higher
+	  end agents because they can spit out the messages faster.
+
+	  Unfortunately, the messages are all spit out before we have control
+	  of the python pj.Lib instance where we can set logging levels so the
+	  only alternative was to actually compile pjsua and _pjsua.so with an
+	  overridden PJ_LOG_MAX_LEVEL.  Although defining a lower max level was
+	  done in the Makefile, the define in config_site.h had to be wrapped
+	  with "#ifndef" so the change would take effect.
+
+	  Change-Id: I2af9e7d48dde1927279c586c9c725d868fe6f3ff
+
+2016-12-22 16:00 +0000 [ae57652983]  Joshua Colp <jcolp@digium.com>
+
+	* chan_pjsip: Use session for retrieving CHANNEL() information.
+
+	  The CHANNEL() dialplan function implementation for PJSIP allows
+	  querying of PJSIP specific information. This used the channel
+	  passed in to get the PJSIP session and associated information.
+	  It is possible for this channel to be masqueraded and end
+	  up as a different channel type by the time the information
+	  request is actually acted upon.
+
+	  This change retrieves the PJSIP session safely and accesses
+	  data from it (including channel). This provides a guarantee
+	  that the session and channel will not be altered when the
+	  request is being acted upon.
+
+	  ASTERISK-26673
+
+	  Change-Id: I335e12b89e1820cafdd92b3e7526b8ba649eb7e6
+
+2016-12-31 19:56 +0000 [386e3a01b3]  Joshua Elson <joshelson@gmail.com>
+
+	* res_pjsip: Fix known compact header issues
+
+	  ASTERISK-26684 #close
+
+	  Change-Id: Ifd7e401c45015119dd5e8421dbfe3afa6381744a
+
+2016-12-30 06:59 +0000 [aad29b9bca]  Martin Tomec <tomec.martin@gmail.com>
+
+	* res_calendar: delete old calendars after reload
+
+	  When "fetch_again_at_reload" is set in config, we create now
+	  new object and thread for each reloaded calendar (with new
+	  configuration). Old calendar should be then unlinked, so the
+	  old thread can exit and free memory.
+
+	  ASTERISK-26683
+
+	  Change-Id: Ic17fba9371c5a8b26a6bc54ea4957c13a32a343e
+
+2016-12-30 09:10 +0000 [5a5953f98c]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip_refer:  Handle compact Refer-To header.
+
+	  refer_incoming_refer_request needed to look for the "r" header as well
+	  as the "Refer-To" header.
+
+	  ASTERISK-26655 #close
+	  patches:
+	  	refer_compact_fix.diff	submitted by JoshE (license 6075)
+
+	  Change-Id: I610410a99b02427ea5db887aeb454d5f12c2259f
+
+2016-12-23 12:11 +0000 [ac04e63ac2]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_native_rtp.c: Minor code cleanups.
+
+	  In native_rtp_bridge_compatible_check()
+
+	  * Made one variable declaration per line.
+
+	  * Extracted if test assignment to make the test easier to see.
+
+	  * Made long if tests easier to see the combinatorial logic.
+
+	  * Added bridge id to a couple debug messages.
+
+	  Change-Id: I65bc5732aa7c9a2537f062f106fbea711cf2daad
+
+2016-12-23 12:10 +0000 [da6f40c9ff]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_native_rtp.c: Fix native rtp bridge data race.
+
+	  native_rtp_bridge_compatible() didn't lock the bridge channels before
+	  checking the channels for native bridging ability.  As a result, one of
+	  the channel's native format capabilities structure got replaced out from
+	  under the native bridge check.  Use of a stale pointer to freed memory
+	  causes bad things to happen.
+
+	  MALLOC_DEBUG, DO_CRASH, and the
+	  tests/channels/pjsip/transfers/blind_transfer/caller_direct_media
+	  testsuite test caught this.
+
+	  * Add missing channel locking in native_rtp_bridge_compatible().
+
+	  Change-Id: If25fdb3ac8e85563c4857fb8216b3d9dc3d0fa53
+
+2016-12-21 16:28 +0000 [b576b58d74]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_rtp_asterisk.c: Fix uninitialized memory crash.
+
+	  ast_rtp_remote_address_set() could pass an uninitialized 'us' parameter to
+	  ast_ouraddrfor().  If ast_ouraddrfor() returns an error then the 'us'
+	  parameter may not get initialized.  Thus when the code tries to save the
+	  'us' parameter to the local address we could try to copy a ridiculous
+	  sized memory buffer and segfault.
+
+	  * Made pass an initialized 'us' parameter to ast_ouraddrfor().
+
+	  * Optimized out the 'us' struct variable.
+
+	  ASTERISK-26672 #close
+
+	  Change-Id: I4acea5dcdf0813da2c7d3e11c2d6067d160d17dc
+
+2016-12-21 16:25 +0000 [67cc8499a2]  Richard Mudgett <rmudgett@digium.com>
+
+	* acl.c: Improve ast_ouraddrfor() diagnostic messages.
+
+	  * Made not generate strings unless they will actually be used.
+
+	  ASTERISK-26672
+
+	  Change-Id: I155fbe7fdff5ce47dfe5326f3baf5446849702c3
+
+2016-12-21 17:54 +0000 [67b47191e9]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_rtp.c: Fix uninitialized memory crash.
+
+	  unicast_rtp_request() could pass an uninitialized 'us' parameter to
+	  ast_ouraddrfor().  If ast_ouraddrfor() returns an error then the 'us'
+	  parameter may not get initialized.  Thus when the code tries to save the
+	  'us' parameter to the local address we could try to copy a ridiculous
+	  sized memory buffer and segfault.
+
+	  * Made pass an initialized 'us' parameter to ast_ouraddrfor() and abort
+	  the UnicastRTP channel request if it fails.
+
+	  ASTERISK-26672
+
+	  Change-Id: I1ef7a7c09f4da4f15dcb6de660d2bcac5f2a95c0
+
+2016-12-21 17:55 +0000 [2fc65173e5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_rtp_asterisk.c: Initialize ourip passed to ast_find_ourip().
+
+	  We access uninitialized memory when the 'ourip' parameter does not
+	  have an initial guess to our IP address.
+
+	  ASTERISK-26672
+
+	  Change-Id: I35507ea1ad7455d2be188f6ccdd4add7bd150e15
+
+2016-12-07 15:23 +0000 [8b7d252987]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_rtp_asterisk.c: Fix off nominal memory leak.
+
+	  Change-Id: I95b1088d11244a2edae6607c12fbf33b38658a75
+
+2016-12-14 02:21 +0000 [bab253ac9f]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* Fixes to various issues reported by pyflakes
+
+	  Pyflake is a python (2) source checker. This patch fixes various
+	  (mostly trivial) errors and warnings it reports.
+
+	  Change-Id: Ia35c5ac61751b927814cf693994c632c412386ea
+
+2016-12-09 12:23 +0000 [f461f65dea]  Martin Tomec <tomec@ipex.cz>
+
+	* app_queue: Ensure member is removed from pending when hanging up.
+
+	  In some cases member is added to pending_members, and the channel
+	  is hung up before any extension state change. So the member would
+	  stay in pending_members forever. So when we call do_hang, we
+	  should also remove member from pending.
+
+	  ASTERISK-26621 #close
+
+	  Change-Id: Iae476b5c06481db18ebe0fa594b3e80fdc9a7d54
+
+2016-12-18 15:23 +0000 [d29eb3b99d]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Make build single threaded
+
+	  There were just too many issues in various environments with
+	  multi threaded building of pjproject.  It doesn't really speed
+	  things up anyway since asterisk is already being compiled in
+	  parallel.
+
+	  Change-Id: Ie5648fb91bb89b4224b6bf43a0daa1af793c4ce1
+
+2016-12-08 20:00 +0000 [8fbb384ea2]  Corey Farrell <git@cfware.com>
+
+	* chan_sip: Reorder unload_module to deal with stuck TCP threads.
+
+	  In some situations TCP threads may become frozen.  This creates the
+	  possibility that Asterisk could segfault if they become unfrozen after
+	  chan_sip has been dlclose'd.  This reorders the unload_module process to
+	  allow abort if threads do not exit within 5 seconds.
+
+	  High level order as follows:
+	  1) Unregister from the core to stop new requests.
+	  2) Signal threads to stop
+	  3) Clear config based tables (but do not free the table itself).
+	  4) Verify that threads have shutdown, cancel unload if not.
+	  5) Clean all remaining resources.
+
+	  ASTERISK-26586
+
+	  Change-Id: Ie23692041d838fbd35ece61868f4c640960ff882
+
+2016-12-16 01:32 +0000 [147b8e636e]  David M. Lee <dlee@respoke.io>
+
+	* configure: fix with-pjproject-bundled
+
+	  The AC_ARG_WITH macro's shell variable is withval; not enableval. Purely
+	  coincidentally, the option would work when --enable-dev-mode is given.
+
+	  Also fixed a portability problem with bootstrap.sh, since -printf is not
+	  a portable option for find.
+
+	  Change-Id: I0f0e5b1a934b5af5737713834361e9c95b96b376
+
+2016-12-15 13:25 +0000 [d27dee3cca]  Richard Mudgett <rmudgett@digium.com>
+
+	* autosupport: Add 'pjproject show buildopts'
+
+	  Change-Id: I8aa55a7c3fb175235ddc7f85e9457d5102d06fa7
+
+2016-12-14 14:21 +0000 [9404efa6f4]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi.c: Fix bounds check regression.
+
+	  Caused by ASTERISK-25494
+
+	  Change-Id: I1fc408c1a083745ff59da5c4113041bbfce54bcb
+
+2016-12-13 14:34 +0000 [45a5e2abc6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add/update ERROR msg if invalid URI.
+
+	  ASTERISK-24499
+
+	  Change-Id: Ie305153e47e922233b2ff24715e0e326e5fa3a6c
+
+2016-12-12 18:38 +0000 [44e72c9d44]  Richard Mudgett <rmudgett@digium.com>
+
+	* MESSAGE: Flush Message/ast_msg_queue channel alert pipe.
+
+	  ASTERISK-25083
+
+	  Change-Id: Id54baa57a8dbca84e29f28bcd2ffc0a5ac12d8b2
+
+2016-12-13 14:06 +0000 [19328de2ab]  George Joseph <gjoseph@digium.com>
+
+	* res_sorcery_memory_cache:  Change an error to a debug message
+
+	  When a sorcery user calls ast_sorcery_delete on an object that
+	  may have already expired from the cache, res_sorcery_memory_cache
+	  spits out an ERROR.  Since this can happen frequently and validly when
+	  an inbound registration expires after the cache entry expired, the
+	  errors are unnecessary and misleading.  Changed to a debug/1.
+
+	  Change-Id: Idf3a67038c16e3da814cf612ff4d6d18ad29ecd7
+
+2016-12-09 08:14 +0000 [31268e0a28]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Retry download if previously saved tarball is bad
+
+	  If a tarball is corrupted during download, the makefile will attempt to
+	  download it again. If the tarball somehow gets corrupted after it's
+	  downloaded however, the makefile was just failing.  We now
+	  retry the download.
+
+	  ASTERISK-26653 #close
+
+	  Change-Id: I1b24d454852d80186f60c5a65dc4624ea8a1c359
+
+2016-12-08 12:43 +0000 [4c6ba1dbba]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* Fix typo in chan_sip
+
+	  The conditional expressions of the 'if' operators
+	  situated alongside each other are identical.
+
+	  Change-Id: I652b6dcddb3be007e669a6aa8107edb31a1ddafb
+
+2016-12-08 12:30 +0000 [934aa2c768]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* res_pjsip: Fix 'A = B != C' kind.
+
+	  Consider reviewing the expression of the 'A = B != C' kind.
+	  The expression is calculated as following: 'A = (B != C)'
+
+	  Change-Id: Ibaa637dfda47d51a20e26069d3103e05ce80003d
+
+2016-12-08 12:54 +0000 [51118e7d70]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* chan_sip: Delete unneeded check
+
+	  P is always true. We check it before
+
+	  Change-Id: Iee61cda002a9f61aee26b9f66c5f9b59e3389efb
+
+2016-12-08 12:58 +0000 [fe5be81821]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* Small code cleanup in chan_sip
+
+	  The conditional expressions of the 'if' operators situated
+	  alongside each other are identical.
+
+	  Change-Id: I2cf7c317b106ec14440c7f1b5dcfbf03639f748a
+
+2016-12-08 12:34 +0000 [149d8db96c]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* Fix IO conversion bug
+
+	  Expression 'rlen < 0' is always false.
+	  Unsigned type value is never < 0.
+
+	  Change-Id: Id9f393ff25b009a6c4a6e40b95f561a9369e4585
+
+2016-11-30 09:31 +0000 [c796f00c35]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* chan_sip: Do not allow non-SP/HTAB between header key and colon.
+
+	  RFC says SIP headers look like:
+
+	      HCOLON  =  *( SP / HTAB ) ":" SWS
+	      SWS     =  [LWS]                    ; sep whitespace
+	      LWS     =  [*WSP CRLF] 1*WSP        ; linear whitespace
+	      WSP     =  SP / HTAB                ; from rfc2234
+
+	  chan_sip implemented this:
+
+	      HCOLON  =  *( LOWCTL / SP ) ":" SWS
+	      LOWCTL  = %x00-1F                   ; CTL without DEL
+
+	  This discrepancy meant that SIP proxies in front of Asterisk with
+	  chan_sip could pass on unknown headers with \x00-\x1F in them, which
+	  would be treated by Asterisk as a different (known) header.  For
+	  example, the "To\x01:" header would gladly be forwarded by some proxies
+	  as irrelevant, but chan_sip would treat it as the relevant "To:" header.
+
+	  Those relying on a SIP proxy to scrub certain headers could mistakenly
+	  get unexpected and unvalidated data fed to Asterisk.
+
+	  This change fixes so chan_sip only considers SP/HTAB as valid tokens
+	  before the colon, making it agree on the headers with other speakers of
+	  SIP.
+
+	  ASTERISK-26433 #close
+	  AST-2016-009
+
+	  Change-Id: I78086fbc524ac733b8f7f78cb423c91075fd489b
+
+2016-11-14 18:18 +0000 [5c89604a32]  Joshua Colp <jcolp@digium.com>
+
+	* res_format_attr_opus: Fix crash when fmtp contains spaces.
+
+	  When an opus offer or answer was received that contained an
+	  fmtp line with spaces between the attributes the module would
+	  fail to properly parse it and crash due to recursion.
+
+	  This change makes the module handle the space properly and
+	  also removes the recursion requirement.
+
+	  ASTERISK-26579
+
+	  Change-Id: I01f53e5d9fa9f1925a7365f8d25071b5b3ac2dc3
+
+2016-12-06 14:54 +0000 [79b09b5f18]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip_registrar: AMI Add RegistrationInboundContactStatuses command
+
+	  The PJSIPShowRegistrationsInbound AMI command was just dumping out
+	  all AORs which was pretty useless and resource heavy since it had
+	  to get all endpoints, then all aors for each endpoint, then all
+	  contacts for each aor.
+
+	  PJSIPShowRegistrationInboundContactStatuses sends ContactStatusDetail
+	  events which meets the intended purpose of the other command and has
+	  significantly less overhead.  Also, some additional fields that were
+	  added to Contact since the original creation of the ContactStatusDetail
+	  event have been added to the end of the event.
+
+	  For compatibility purposes, PJSIPShowRegistrationsInbound is left
+	  intact.
+
+	  ASTERISK-26644 #close
+
+	  Change-Id: I326f12c9ecb52bf37ba03f0748749de4da01490a
+
+2016-12-07 14:22 +0000 [3b6e6cd01c]  snuffy <snuffy22@gmail.com>
+
+	* tests_dns: Make DNS tests older nameser.h compatible
+
+	  Fix the tests for DNS to use older style nameser.h as
+	  in ASTERISK-26608.
+
+	  Tested on: OpenBSD 6.0, Debian 8
+
+	  ASTERISK-26647 #close
+
+	  Change-Id: I285913c44202537c04b3ed09c015efa6e5f9052d
+
+2016-12-06 16:45 +0000 [76d52dc228]  Richard Mudgett <rmudgett@digium.com>
+
+	* Bundled pjproject:  Fix finding SIP transactions.
+
+	  Occasionally SIP message transactions are not found when they should be.
+	  In the particular case an incoming INVITE transaction is CANCELed but the
+	  INVITE transaction cannot be found so a 481 response is returned for the
+	  CANCEL.  The problematic calls have a '_' character in the Via branch
+	  parameter.
+
+	  The problem is in the pjproject PJ_HASH_USE_OWN_TOLOWER feature's code.
+	  The problem with the "own tolower" code is that it does not calculate the
+	  same hash value as when the pj_tolower() function is used.  The "own
+	  tolower" code will erroneously modify the ASCII characters '@', '[', '\\',
+	  ']', '^', and '_'.  Calls to pj_hash_calc_tolower() can use the
+	  PJ_HASH_USE_OWN_TOLOWER substitute algorithm when enabled.  Calls to
+	  pj_hash_get_lower(), pj_hash_set_lower(), and pj_hash_set_np_lower() call
+	  find_entry() which never uses the PJ_HASH_USE_OWN_TOLOWER algorithm.  As a
+	  result you may not be able to find a hash tabled entry because the
+	  calculated hash values would differ.
+
+	  * Simply disable PJ_HASH_USE_OWN_TOLOWER.
+
+	  ASTERISK-26490 #close
+
+	  Change-Id: If89bfdb5f301b8b685881a9a2a6e0c3c5af32253
+
+2016-12-01 16:49 +0000 [503006123a]  Mark Michelson <mmichelson@digium.com>
+
+	* http: Send headers and body in one write.
+
+	  This is a semi-regression caused by the iostreams change. Prior to
+	  iostreams, HTTP headers were written to a FILE handle using fprintf.
+	  Then the body was written using a call to fwrite(). Because of internal
+	  buffering, the result was that the HTTP headers and body would be sent
+	  out in a single write to the socket.
+
+	  With the change to iostreams, the HTTP headers are written using
+	  ast_iostream_printf(), which under the hood calls write(). The HTTP body
+	  calls ast_iostream_write(), which also calls write() under the hood.
+	  This results in two separate writes to the socket.
+
+	  Most HTTP client libraries out there will handle this change just fine.
+	  However, a few of our testsuite tests started failing because of the
+	  change. As a result, in order to reduce frustration for users, this
+	  change alters the HTTP code to write the headers and body in a single
+	  write operation.
+
+	  ASTERISK-26629 #close
+	  Reported by Joshua Colp
+
+	  Change-Id: Idc2d2fb3d9b3db14b8631a1e302244fa18b0e518
+
+2016-12-06 10:56 +0000 [bf6423a336]  Mark Michelson <mmichelson@digium.com>
+
+	* Iostreams: Correct off-by-one error.
+
+	  ast_iostream_printf() attempts first to use a fixed-size buffer to
+	  perform its printf-like operation. If the fixed-size buffer is too
+	  small, then a heap allocation is used instead. The heap allocation in
+	  this case was exactly the length of the string to print. The issue here
+	  is that the ensuing call to vsnprintf() will print a NULL byte in the
+	  final space of the string. This meant that the final character was being
+	  chopped off the string and replaced with a NULL byte. For HTTP in
+	  particular, this caused problems because HTTP publishes the expected
+	  Contact-Length. This meant HTTP was publishing a length one character
+	  larger than what was actually present in the message.
+
+	  This patch corrects the issue by adding one to the allocation length.
+
+	  ASTERISK-26629
+	  Reported by Joshua Colp
+
+	  Change-Id: Ib3c5f41e96833d0415cf000656ac368168add639
+
+2016-12-06 12:06 +0000 [fe9f070885]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fix missing inclusion of symbols
+
+	  Added back in a -g3, and an -O3 when DONT_OPTIMIZE is not set, to
+	  the CFLAGS.  Not sure how they went missing.
+
+	  Also fixed an uninstall problem where we weren't removing the
+	  symlink from libasteriskpj.so.2 to libasteriskpj.so.  While I was
+	  there, I fixed it for libasteriskssl as well.
+
+	  Change-Id: I9e00873b1e9082d05b5549d974534b48a2142556
+
+2016-11-30 18:25 +0000 [4b3d3fc741]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Filter redundant statsd reporting.
+
+	  Increasing the testsuite shutdown timeout before forcibly killing
+	  Asterisk allowed more events to be sent out.  Some tests failed as
+	  a result.  The tests/channels/pjsip/statsd/registrations failed
+	  because we now get the statsd events that a comment in the test
+	  configuration stated couldn't be intercepted.  Unfortunately, we
+	  get a variable number of events because of internal status state
+	  transition races generating redundant statsd events.
+
+	  We were reporting redundant statsd PJSIP.registrations.state changes
+	  for internal state changes that equated to the same thing publicly.
+
+	  * Made update_client_state_status() filter out redundant statsd
+	  updates.
+
+	  ASTERISK-26527
+
+	  Change-Id: If851c7d514bb530d9226e4941ba97dcf52000646
+
+2016-06-28 16:26 +0000 [26c8552fff]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* OpenSSL 1.1.0 support
+
+	  OpenSSL 1.1.0 includes some major changes in the interface. See
+	  https://wiki.openssl.org/index.php/1.1_API_Changes .
+
+	  Status: Right now there are still a few deprecation notes with OpenSSL
+	  1.1.0. But it's a start.
+
+	  Changes:
+	  * CRYPTO_LOCK is no longer available. Replace it with its value for now.
+	    I don't completely understand what it is used for there.
+	  * Remove several functions from libasteriskssl that seem to no longer be
+	    needed.
+	  * Structures have become opaque and are accesses with accessors.
+	  * ERR_remove_thread_state() no longer needed.
+	  * SSLv2 code now could no longer be used in 1.1.
+
+	  ASTERISK-26109 #close
+
+	  Change-Id: I5e29d477d486ca29b6aae0dc2f5dff960c1cb82b
+
+2016-11-22 11:20 +0000 [75230f4c01]  Guido Falsi <mad@madpilot.net>
+
+	* res_rtp: Fix regression when IPv6 is not available.
+
+	  The latest Release candidate fails to create RTP streams when IPv6
+	  is not available. Due to the changes made in September the ast_sockaddr
+	  structure passed around to create these streams is always of AF_INET6
+	  type, causing failure when used for IPv4. This patch adds a utility
+	  function to check for availability of IPv6 and applies such check
+	  at startup to determine how to create the ast_sockaddr structures.
+
+	  ASTERISK-26617 #close
+
+	  Change-Id: I627a4e91795e821111e1cda523f083a40d0e0c3e
+
+2016-11-23 18:27 +0000 [1dfa11b65c]  Richard Mudgett <rmudgett@digium.com>
+
+	* PJPROJECT logging: Made easier to get available logging levels.
+
+	  Use of the new logging is as simple as issuing the new CLI command or
+	  setting the new pjproject.conf option.
+
+	  Other options that can affect the logging are how you have the pjproject
+	  log levels mapped to Asterisk log types in pjproject.conf and if you have
+	  configured Asterisk to log the DEBUG type messages.  Altering the
+	  pjproject.conf level mapping shouldn't be necessary for most installations
+	  as the default mapping is sensible.  Configuring Asterisk to log the DEBUG
+	  message type is standard practice for collecting debug information.
+
+	  * Added CLI "pjproject set log level" command to dynamically adjust the
+	  maximum pjproject log message level.
+
+	  * Added CLI "pjproject show log level" command to see the currently set
+	  maximum pjproject log message level.
+
+	  * Added pjproject.conf startup section "log_level" option to set the
+	  initial maximum pjproject log message level so all messages could be
+	  captured from initialization.
+
+	  * Set PJ_LOG_MAX_LEVEL to 6 to compile in all defined logging levels into
+	  bundled pjproject.  Pjproject will use the currently set run time log
+	  level to determine if a log message is generated just like Asterisk
+	  verbose and debug logging levels.
+
+	  * In log_forwarder(), made always log enabled and mapped pjproject log
+	  messages.  DEBUG mapped log messages are no longer gated by the current
+	  Asterisk debug logging level.
+
+	  * Removed RAII_VAR() from res_pjproject.c:get_log_level().
+
+	  ASTERISK-26630 #close
+
+	  Change-Id: I6dca12979f482ffb0450aaf58db0fe0f6d2e5389
+
+2016-11-30 10:48 +0000 [621d886ca7]  Mark Michelson <mmichelson@digium.com>
+
+	* Frame deferral: Re-queue deferred frames one-at-a-time.
+
+	  The recent change that made frame deferral into an API had a behavior
+	  change to it. When frame deferral was completed, we would take all of
+	  the deferred frames and queue them all onto the channel in one call to
+	  ast_queue_frame_head(). Before frame deferral was API-ized, places that
+	  performed manual frame deferral would actually take each deferred frame
+	  and queue them onto the channel.
+
+	  This change in behavior caused the confbridge_recording test to start
+	  failing consistently. Without going too crazily deep into the details,
+	  a channel was getting "stuck" in an ast_safe_sleep(). An AMI redirect
+	  was attempting to break it out of the sleep, but because there were more
+	  frames in the channel read queue than expected, the channel ended up
+	  being unable to break from its sleep loop.
+
+	  By restoring the behavior of individual frame queuing after deferral,
+	  the test starts passing again.
+
+	  Note, this points to a potential underlying issue pointing to an
+	  "unbalance" that can occur when queuing multiple frames at once,
+	  and so a follow-up issue is being created to investigate that
+	  possibility.
+
+	  Change-Id: Ied5dacacda06d343dea751ed5814a03364fe5a7d
+
+2016-11-15 15:01 +0000 [e5e887be53]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* chan_pjsip: fix switching sending codec when asymmetric_rtp_codec=no
+
+	  The sending codec is switched to the receiving codec and then
+	  is switched back to the best native codec on EVERY receiving RTP packets.
+	  This is because after call of ast_channel_set_rawwriteformat there is call
+	  of ast_set_write_format which calls set_format which sets rawwriteformat
+	  to the best native format.
+
+	  This patch adds a new function ast_set_write_format_path which set
+	  specific write path on channel and uses this function to switch
+	  the sending codec.
+
+	  ASTERISK-26603 #close
+
+	  Change-Id: I5b7d098f8b254ce8f45546e6c36e5d324737f71d
+
+2016-11-21 15:43 +0000 [ddc951060a]  David Kerr <david@kerr.net>
+
+	* app_originate: Add option to execute gosub prior to dial
+
+	  Issue/patch ASTERISK-26587 was inspired by issue ASTERISK-22992
+	  that requested ability to add callerid into app_originate.
+	  Comments in that issue suggested that it was better solved by
+	  adding an option to gosub prior to originating the call.  The
+	  attached patch implements this much like app_dial with two
+	  options one to gosub on the originating channel and one to gosub
+	  on the newly created channel and behaves just like app_dial.
+	  I have tested this patch by adding callerid info to the new
+	  channel and also SIPAddHeader (to e.g. add header to force auto
+	  answer) and confirmed it works.  Have also tested both 'exten'
+	  and 'app' versions of app_originate.
+
+	  Opened by: dkerr
+	  Patch by: dkerr
+
+	  Change-Id: I36abc39b58567ffcab4a636ea196ef48be234c57
+
+2016-11-28 19:43 +0000 [0e214c4932]  Eduardo S. Libardi <eslibardi@gmail.com>
+
+	* res_calendar_caldav: Add support reading gmail calendar
+
+	  The response from gmail calendar includes the string name
+	  "caldav:calendar-data". res_calendar_caldav implements
+	  the example included in RFC 4791: string "C:calendar-data".
+	  When reading the calendar, res_calendar_caldav compare the
+	  string and if does not match just discards the event.
+	  This commit compares the response to both strings,
+	  successfully loading gmail calendar events.
+	  Writing to gmail calendar is working prior to this fix.
+
+	  ASTERISK-26624
+	  Reported by: Eduardo S. Libardi
+
+	  Change-Id: Ia1eef10552ae616efb645d390f5ffe81260d7d4a
+
+2016-11-28 15:12 +0000 [a3f48be0da]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip: Fix documentation whitespace issues
+
+	  Tabs > Spaces.
+
+	  Change-Id: If1e43a71822615a898e958e0f8b2e882606f0bd0
+
+2016-11-22 10:27 +0000 [0e15760795]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip/chan_sip: Advertise 'ws' in the SIP URI transport parameter
+
+	  Per RFC 7118 5.2, the SIP URI 'transport' parameter should advertise
+	  'ws' when WebSockets are to be used as the transport. This applies to
+	  both secure and insecure WebSockets.
+
+	  There were two bugs in Asterisk with respect to this:
+
+	  (1) The most egregious occurs in res_pjsip. There, we advertise 'ws' for
+	      insecure websockets and 'wss' for secure websockets. While this
+	      would seem to make sense - since 'WS' and 'WSS' are used for the Via
+	      Transport parameter - this is not the case for the SIP URI. This
+	      patch corrects that by registering the secure websockets with
+	      pjproject using the shorthand 'WS', and by returning 'ws' when asked
+	      for the transport parameter. Note that in pjproject, it is perfectly
+	      valid to have multiple transports use the same shorthand.
+
+	  (2) In chan_sip, we return an upper-case version of the transport 'WS'
+	      instead of 'ws'. Since we should be strict in what we send and
+	      liberal in what we accept (within reason), this patch lower-cases
+	      the transport before appending it to the parameter.
+
+	  ASTERISK-24330 #close
+	  Reported by: cervajs, Inaki Baz Castillo
+
+	  Change-Id: Iff77b645f8cc3b7cd35168a6676c26b147f22f42
+
+2016-11-28 11:03 +0000 [8a68289766]  George Joseph <gjoseph@digium.com>
+
+	* build_tools:  Fix download_externals to handle certified branches
+
+	  download_externals wasn't handling the "certified/13.x" version
+	  correctly.
+
+	  Change-Id: I124d195bb117ca36fd7bf1150c630f3b474a9d9a
+
+2016-11-28 07:36 +0000 [e3dae763ee]  Joshua Colp <jcolp@digium.com>
+
+	* iostream: Move include of asterisk.h
+
+	  The asterisk.h header file needs to be included first or else
+	  some things go awry, such as:
+
+	  implicit declaration of function 'vasprintf'
+
+	  Change-Id: I981dc2a77a1ba791888e4f1726644d4656c0407c
+
+2016-11-26 10:57 +0000 [0b588778c0]  Michael Kuron <m.kuron@gmx.de>
+
+	* chan_sip: Fix segfault during module unload
+
+	  If a TCP/TLS connection was pending (not accepted and not timed out) during
+	  unload of chan_sip, Asterisk would segfault when trying to send a signal to
+	  a thread whose thread ID hadn't been recorded yet. This commit fixes that by
+	  recording the thread ID before calling the blocking connect() syscall.
+	  This was a regression introduced by 776a14386a55b5425c7e9617eff8af8b45427144.
+
+	  The above wasn't enough to fix the segfault, which was now delayed to the
+	  point where connect() timed out. Therefore, it was necessary to also remove
+	  the SA_RESTART flag from the SIGURG sigaction so that pthread_kill() could be
+	  used to interruput the connect() syscall.
+	  This was a regression introduced by 5d313f51b982a18f7321adcf7c7a4e822d8b2714.
+
+	  ASTERISK-26586 #close
+
+	  Change-Id: I76fd9d47d56e4264e2629bce8ec15fecba673e7b
+
+2016-11-23 14:52 +0000 [ead773f801]  Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+
+	* pbx_lua: On configuration errors report module load failure instead of decline.
+
+	  Switched from AST_MODULE_LOAD_DECLINE to AST_MODULE_LOAD_FAILURE.
+	  Therefore, if pbx_lua fails to load and pbx_lua is marked as required,
+	  Asterisk exits as expected.
+	  If extensions.lua cannot be opened, AST_MODULE_LOAD_DECLINE is reported.
+
+	  Change-Id: I8e5a0037e69b41743db60c568541ebb2f52a7a8f
+
+2016-11-11 08:16 +0000 [d9b24cce0a]  gestoip2 <gestoip2@ull.edu.es>
+
+	* res_rtp_asterisk: RTT miscalculation in RTCP
+
+	  When retrieving RTCP stats for PJSIP channels, RTT values are unreliable.
+	  RTT calculation is correct, but the data representation isn't.  RTT is
+	  represented by a 32-bit fixed-point number with the integer part in the
+	  first 16 bits and the fractional part in the last 16 bits.  In order to
+	  get the RTT value, the fractional part is miscalculated, there is an
+	  unnecessary 16 bit shift that causes overflow.  Besides this there is
+	  another mistake, when transforming the integer value to the fixed point
+	  fractional part via bitwise operation, that loses precision.
+
+	  * RTT fractional part is no longer shifted, avoiding overflow.
+
+	  * RTT fractional part is transformed to its fixed-point value more
+	  precisely.
+
+	  * Fixed timeval2ntp() and ntp2timeval() second fraction conversions.
+
+	  * Fixed NTP timestamp report logging.  The usec was inexplicably
+	  multiplied by 4096.
+
+	  ASTERISK-26566 #close
+	  Reported by Hector Royo Concepcion
+
+	  Change-Id: Ie09bdabfee75afb3f1b8ddfd963e5219ada3b96f
+
+2016-11-15 13:44 +0000 [635b0a0a55]  Michael Kuron <m.kuron@gmx.de>
+
+	* tcptls: Use new certificate upon sip reload
+
+	  Previously, a TLS server socket would only be restarted upon sip reload if the
+	  bind address had changed. This commit adds checking for changes to TLS
+	  parameters like certificate, ciphers, etc. so they get picked up without
+	  requiring a reload of the entire chan_sip module. This does not affect open
+	  connections in any way, but new connections will use the new TLS parameters.
+	  The changes also apply to HTTP and Manager.
+
+	  ASTERISK-26604 #close
+
+	  Change-Id: I169e86cefc6dcd627c915134015a6a1ab1aadbe6
+
+2016-11-21 09:49 +0000 [abae3dc36e]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Use $(LIB_RT) for link of libasteriskpj
+
+	  libasteriskpj was hard coded to use -lrt but librt is linux specific
+	  so we now use the LIB_RT variable which gets set by configure.
+
+	  Change-Id: I41148884517e3031f7675a413d524c86e8614694
+
+2016-11-19 16:19 +0000 [b546497fe0]  snuffy <snuffy22@gmail.com>
+
+	* Add support for older name resolving version libraries like openBSD
+
+	  Fix support of OS's like openBSD that use an older nameser.h,
+	  this change reverts the defines to the older style which on other
+	  systems is found in nameser_compat.h
+
+	  Tested on openBSD 6.0, Debian 8
+
+	  ASTERISK-26608 #close
+
+	  Change-Id: Iffb36caab8c5aa9dece0ce2d009041f7b56cc86a
+
+2016-11-18 09:46 +0000 [7a8d6bc81b]  Mark Michelson <mmichelson@digium.com>
+
+	* Bump ARI version to 2.0.0
+
+	  In order to not have version number overlap between different versions
+	  of Asterisk, each new major version of Asterisk will mean we also bump
+	  the ARI major version number.
+
+	  This particular change does NOT introduce any known breaking changes to
+	  ARI.
+
+	  For discussion relating to this topice, see:
+	  http://lists.digium.com/pipermail/asterisk-dev/2016-November/075964.html
+
+	  Change-Id: I712ee0df177a8fe1252da2bc029705268b97b665
+
+2016-11-16 12:05 +0000 [d3f070c7a2]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Improve reliability of pjproject download
+
+	  The download process now has a timeout which will cause wget to retry
+	  if it stops retrieving data for 5 seconds and fetch and curl to timeout
+	  if the whole retrieval take smore than 30 seconds.
+
+	  If the tarball retrieval works, the MD5SUM file is retrieved from
+	  the downloads site and the md5 checksum is verified.
+
+	  If either the tarball retrieval or MD5SUM retrieval fails, or the
+	  checksums don't match, the entire process is retried once.  If it
+	  fails again, any incomplete tarball is deleted.
+
+	  .DELETE_ON_ERROR: was also added to the Makefile.  Not only does
+	  this delete the tarball on failure, it till also delete corrupted
+	  library files from the pjproject source directory should they
+	  fail to build correctly.
+
+	  Tested all the way back to FreeBSD 9, CentOS 6, Debian 6 and
+	  Ubuntu 14.
+
+	  Change-Id: Iea7d33b96a31622ab1b6e54baebaf271959514e1
+
+2016-11-11 07:13 +0000 [e822a50f86]  Mikheili Dautashvili <mishadaut@gmail.com>
+
+	* main/app.c: Transmit Silence on ControlPlayback pause
+
+	  ASTERISK-26562 #close
+
+	  Change-Id: Ie6cb0ffc2b8c775639ce7784fe96f4ea00cfa2f8
+
+2016-11-17 10:52 +0000 [d670ea6297]  Mark Michelson <mmichelson@digium.com>
+
+	* manager: update minor version
+
+	  Based on bridge video AMI event changes, bump the minor version of AMI.
+
+	  Change-Id: Idf84507354170400813cda780906c94c9f1b60b4
+
+2016-11-17 08:25 +0000 [349e08cb48]  Timo Teräs <timo.teras@iki.fi>
+
+	* codec_dahdi: Fix poll.h include.
+
+	  POSIX defines poll.h. sys/poll.h should not be used as it is c-library
+	  internal header which may or may not exist. Notably in musl including
+	  sys/poll.h generates warning of being incorrect.
+
+	  Change-Id: Ib318c1c7142a737bcf3caa4d8d72560bebe39252
+
+2016-11-16 20:24 +0000 [935f5d003b]  George Joseph <gjoseph@digium.com>
+
+	* build:  Various OpenBSD issues
+
+	  OpenBSD's 'find' doesn't take the -delete argument so you have to pipe
+	  through 'xargs rm -rf'.
+
+	  'echo -e' doesn't like \t starting a line. It just prints 't' which
+	  causes the libasteriskpj.exports file to be garbage.  They were just
+	  cosmetic so they were removed.
+
+	  librt doesn't exist so the link of libasteriskpj.so fails. It's not
+	  actually needed for linux anyway so -lrt was removed from the link.
+
+	  res_rtp_asterisk was failing to load because of an undefined
+	  DTLS_method. '|| defined(LIBRESSL_VERSION_NUMBER)' was added to the #if
+	  so DTLSv1_method is used instead.
+
+	  ASTERISK-26608
+
+	  Change-Id: I926ec95b0b69633231e3ad1d6e803b977272c49c
+
+2016-11-16 15:42 +0000 [dc8f99ee27]  Mark Michelson <mmichelson@digium.com>
+
+	* res_format_attr_opus: Fix fmtp generation.
+
+	  res_format_attr_opus assumed that the string being passed into it was
+	  empty. It tried to determine if the only thing it had written was
+
+	  a=fmtp:<num>
+
+	  And if it had, it would reset the string. Its calculation was off when
+	  working with chan_sip, though. chan_sip passes the entire built SDP
+	  rather than an empty string. This resulted in always putting an empty
+	  fmtp line in the SDP.
+
+	  ASTERISK-26520 #close
+	  Reported by scgm11
+
+	  Change-Id: Ib2e8712d26a47067e5f36d5973577added01dbb5
+
+2016-11-15 16:23 +0000 [ed9ced0531]  Richard Mudgett <rmudgett@digium.com>
+
+	* codec_opus: Fix warning when Opus negotiated but codec_opus not loaded.
+
+	  When Opus is negotiated but not loaded, the log is spammed with messages
+	  because the system does not know how to calculate the number of samples in
+	  a frame.
+
+	  * Suppress the warning by supplying a function that assumes 20ms of
+	  samples in the frame.  For pass through support it doesn't really seem to
+	  matter what number of samples is returned anyway.
+
+	  ASTERISK-26605 #close
+
+	  Change-Id: Icf2273692f040dc2c45b01e72a790d11092f9e0f
+
+2016-11-14 14:36 +0000 [0cd0e70c16]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_authenticator_digest.c: Fix memory pool leak.
+
+	  Responding to authentication challenges leaks PJSIP memory pools.
+
+	  The leak was introduced with a pjproject 2.5.5 API change.
+	  https://trac.pjsip.org/repos/ticket/1929 changed the API usage of
+	  pjsip_auth_clt_init() to require the new API pjsip_auth_clt_deinit() to
+	  clean up cached authentication allocations that get allocated with
+	  pjsip_auth_clt_reinit_req().
+
+	  ASTERISK-26516 #close
+
+	  Change-Id: I4473141b8c3961d0dc91c382beb3876b3efb45c8
+
+2016-11-15 12:01 +0000 [3017f09f22]  George Joseph <gjoseph@digium.com>
+
+	* file.c/__ast_file_read_dirs:  Fix issues on filesystems without d_type
+
+	  One of the code paths in __ast_file_read_dirs will only get executed if
+	  the OS doesn't support dirent->d_type OR if the filesystem the
+	  particular file is on doesn't support it.  So, while standard Linux
+	  systems support the field, some filesystems like XFS do not.  In this
+	  case, we need to call stat() to determine whether the directory entry
+	  is a file or directory so we append the filename to the supplied
+	  directory path and call stat.  We forgot to truncate path back to just
+	  the directory afterwards though so we were passing a complete file name
+	  to the callback in the dir_name parameter instead of just the directory
+	  name.
+
+	  The logic has been re-written to only create a full_path if we need to
+	  call stat() or if we need to descend into another directory.
+
+	  Change-Id: I54e4228bd8355fad65200c6df3ec4c9c8a98dfba
+
+2016-06-02 14:10 +0000 [070a51bf7c]  Timo Teräs <timo.teras@iki.fi>
+
+	* Implement internal abstraction for iostreams
+
+	  fopencookie/funclose is a non-standard API and should not be used
+	  in portable software. Additionally, the way FILE's fd is used in
+	  non-blocking mode is undefined behaviour and cannot be relied on.
+
+	  This introduces internal abstraction for io streams, that allows
+	  implementing the desired virtualization of read/write operations
+	  with necessary timeout handling.
+
+	  ASTERISK-24515 #close
+	  ASTERISK-24517 #close
+
+	  Change-Id: Id916aef418b665ced6a7489aef74908b6e376e85
+
+2016-11-15 08:07 +0000 [d3b61a98f4]  Joshua Colp <jcolp@digium.com>
+
+	* manager: Bump AMI version number.
+
+	  During the development of Asterisk 14 the behavior of
+	  the Command AMI action was altered such that the result
+	  was returned on lines with a prefix of "Output: ". While
+	  this was documented in the UPGRADE.txt file it is also
+	  reasonable that this should bump the AMI version number.
+
+	  ASTERISK-26556
+
+	  Change-Id: Idf1bf01608e53f7bfdf43ddb4d0683e53f74ee42
+
+2016-11-14 15:57 +0000 [edd7ae85e8]  Matt Jordan <mjordan@digium.com>
+
+	* pjproject: Use a much higher limit for PJ_ICE_MAX_CHECKS
+
+	  The PJ_ICE_MAX_CHECKS constant is used by pjproject to determine how
+	  many pairs of local/remote candidates will be made. If for some reason
+	  we reach this upper bound, ICE will generally fail and no media will
+	  flow between the browser and Asterisk.
+
+	  This patch makes PJ_ICE_MAX_CHECKS set to the total possible number of
+	  pairs of candidates we'd theoretically allow, which is
+	  PJ_ICE_MAX_CAND^2. Prior to this patch, we simply multiplied
+	  PJ_ICE_MAX_CAND by two; on systems with multiple interfaces (I blame
+	  Docker), this is far too low to allow WebRTC calls to succeed.
+
+	  Setting this to be PJ_ICE_MAX_CAND^2 allowed WebRTC calls to succeed
+	  even when the system Asterisk was running on had quite a few virtual
+	  interfaces.
+
+	  Change-Id: Icd4f17de0ac9d3a83dddfc8bf1cb7616bc107d55
+
+2016-11-14 15:32 +0000 [cc86329228]  Matt Jordan <mjordan@digium.com>
+
+	* apps/app_echo: Only relay a single video source change frame
+
+	  In 9785e8d0, app_echo was updated to relay video source updates to the
+	  channel for the purposes of displaying video in WebRTC tests.
+	  Unfortunately, this can cause a Kafkaesque nightmare if two or more
+	  Local channels are in a bridge together where their ends are in
+	  app_echo. When this situation occurs, a video update sent into app_echo
+	  will cause the video update to be relayed to the other Local channels,
+	  causing another round of video updates, etc. In not much time at all,
+	  the channel length queues will be overwhelmed, channel alert pipes will
+	  fail, and all hell will break loose as Asterisk merrily continues to
+	  throw more video update requests onto the channels.
+
+	  This patch updates app_echo to *only* relay a single video update. Once
+	  a video update has been made, all further video updates are dropped.
+	  This meets the intended purpose of the original patch: if we get a video
+	  update and we're in app_echo, go ahead and ask the sender to update
+	  themselves. However, once we've got that video stream sync'd up, don't
+	  keep spamming the world.
+
+	  Change-Id: I9210780b08d4c17ddb38599d1c64453adfc34f74
+
+2016-11-08 10:11 +0000 [a72ef38113]  Matt Jordan <mjordan@digium.com>
+
+	* res/ari/resource_bridges: Add the ability to manipulate the video source
+
+	  In multi-party bridges, Asterisk currently supports two video modes:
+	   * Follow the talker, in which the speaker with the most energy is shown
+	     to all participants but the speaker, and the speaker sees the
+	     previous video source
+	   * Explicitly set video sources, in which all participants see a locked
+	     video source
+
+	  Prior to this patch, ARI had no ability to manipulate the video source.
+	  This isn't important for two-party bridges, in which Asterisk merely
+	  relays the video between the participants. However, in a multi-party
+	  bridge, it can be advantageous to allow an external application to
+	  manipulate the video source.
+
+	  This patch provides two new routes to accomplish this:
+	  (1) setVideoSource: POST /bridges/{bridgeId}/videoSource/{channelId}
+	      Sets a video source to an explicit channel
+	  (2) clearVideoSource: DELETE /bridges/{bridgeId}/videoSource
+	      Removes any explicit video source, and sets the video mode to talk
+	      detection
+
+	  ASTERISK-26595 #close
+
+	  Change-Id: I98e455d5bffc08ea5e8d6b84ccaf063c714e6621
+
+2016-11-14 14:03 +0000 [7263a17ca0]  George Joseph <gjoseph@digium.com>
+
+	* channel:  Fix issues in hangup scenarios caused by frame deferral
+
+	  ASTERISK-26343
+
+	  Change-Id: I06dbf7366e26028251964143454a77d017bb61c8
+	  (cherry picked from commit 0be46aaf6b8b9eb5b0160ec591cdc2c6e1802a6d)
+
+2016-11-14 13:55 +0000 [0dc4567133]  George Joseph <gjoseph@digium.com>
+
+	* Revert "Revert "channel: Use frame deferral API for safe sleep.""
+
+	  This reverts commit e5365dada5052b87275c048f6e29ac7d5e2b2415.
+
+	  Change-Id: Icc40cf0c7687454760762912dd29e4ae79e8e9ee
+
+2016-11-14 13:55 +0000 [6d61f7bfd1]  George Joseph <gjoseph@digium.com>
+
+	* Revert "Revert "autoservice: Use frame deferral API""
+
+	  This reverts commit edca6911f392f47c1a5a25d1d3a357c72b04a78a.
+
+	  Change-Id: I76030b87333a2c390cd05392b74b75678d78ddfa
+
+2016-11-14 13:55 +0000 [f62c9c42fa]  George Joseph <gjoseph@digium.com>
+
+	* Revert "Revert "AGI: Only defer frames when in an interception routine.""
+
+	  This reverts commit 6bce938c2fcb60b7a77a0e997a6518860c0bfa39.
+
+	  Change-Id: Iadbf462bf2a52e8b2fa9ebc75b37b1f688ba51d9
+
+2016-11-14 13:54 +0000 [2966fa5ad7]  George Joseph <gjoseph@digium.com>
+
+	* Revert "Revert "Add API for channel frame deferral.""
+
+	  This reverts commit fa749866c17f91860d3e9f89742eab3e6f03ecbc.
+
+	  Change-Id: Idcd1b88fa0766b1326dcc87d8905dbc314c71bd7
+
+2016-11-11 10:45 +0000 [c6d755de11]  Sebastien Duthil <sduthil@proformatique.com>
+
+	* res_ari: Add support for channel variables in ARI events.
+
+	  This works the same as for AMI manager variables. Set
+	  "channelvars=foo,bar" in your ari.conf general section, and then the
+	  channel variables "foo" and "bar" (along with their values), will
+	  appear in every Stasis websocket channel event.
+
+	  ASTERISK-26492 #close
+	  patches:
+	    ari_vars.diff submitted by Mark Michelson
+
+	  Change-Id: I5609ba239259577c0948645df776d7f3bc864229
+
+2016-11-14 12:16 +0000 [72da2ef9ff]  George Joseph <gjoseph@digium.com>
+
+	* cli:  Fix ast_el_read_char to work with libedit >= 3.1
+
+	  Libedit 3.1 is not build with unicode on as a default and so the
+	  prototype for the el_gets callback changed from expecting a char buffer
+	  to accepting a wchar buffer.  If ast_el_read_char isn't changed,
+	  the cli reads garbage from teh terminal.
+
+	  Added a configure test for (*el_rfunc_t)(EditLine *, wchar_t *) and
+	  updated ast_el_read_char to use the HAVE_ define to detemrine whether
+	  to use char or wchar.
+
+	  ASTERISK-26592 #close
+
+	  Change-Id: I9099b46f68e06d0202ff80e53022a2b68b08871a
+
+2016-11-12 12:15 +0000 [97a75e3829]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* Add support for building RADIUS with radcli
+
+	  Radcli is yet another RADIUS client library, generally compatible with
+	  freeradius and radiusclient-ng.
+
+	  This commit adds autoconf option for detecting it as well and changes
+	  cdr_radius and cel_radius to use its header file in that case.
+
+	  ASTERISK-26540 #close
+
+	  Change-Id: I271f0715406334874865ffbce0b354b3a2ca148f
+
+2016-11-10 10:57 +0000 [1bd49040c4]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_sdp_rtp: Reject offer of required SRTP without res_srtp.
+
+	  When optimistic SRTP was on it was possible for us to still
+	  set up a call without an audio stream if an offer was received
+	  with required SRTP.
+
+	  This change makes it so this scenario will now fail with a 488
+	  response.
+
+	  ASTERISK-26575
+
+	  Change-Id: I7d14187037681f48879bd20319ac79d0877318f3
+
+2016-11-11 02:41 +0000 [dfb951817f]  Igor Goncharovskiy <igor.goncharovsky@gmail.com>
+
+	* Fix closing rtp ports after call finished in chan_unistim.
+
+	  Fix ASTERISK-26565 by adding ast_rtp_instance_stop before
+	  rtp instance destroy for chan_unistim. Also several fixes
+	  for displayed text translation.
+
+	  Change-Id: If42a03eea09bd1633471406bdc829cf98bf6affc
+
+2016-11-11 00:29 +0000 [939dcf66b0]  Timo Teräs <timo.teras@iki.fi>
+
+	* addons/chan_mobile: do not use strerror_r
+
+	  The two reasons why it might be used are that some systems do not
+	  implement strerror in thread safe manner, and that strerror_r returns
+	  the error code in the string in case there's no error message.
+
+	  However, all of asterisk elsewhere uses strerror() and assumes it
+	  to be thread safe. And in chan_mobile the errno is also explicitly
+	  printed so neither of the above reasons are valid.
+
+	  The reasoning to remove usage is that there are actually two versions
+	  of strerror_r: XSI and GNU. They are incompatible in their return
+	  value, and there's no easy way to figure out which one is being
+	  used. glibc gives you the GNU version if _GNU_SOURCE is defined,
+	  but the same feature test macro is needed for other symbols. On
+	  all other systems you assumedly get XSI symbol, and compilation warnings
+	  as well as non-working error printing.
+
+	  Thus the easiest solution is to just remove strerror_r and use
+	  strerror as rest of the code. Alternative is to introduce ast_strerror
+	  in separate translation unit so it can request the XSI symbol in
+	  glibc case, and replace all usage of strerror.
+
+	  Change-Id: I84d35225b5642d85d48bc35fdf399afbae28a91d
+
+2016-09-23 17:54 +0000 [338f35edcc]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.c: Rework endpt_send_request() req_wrapper code.
+
+	  * Don't hold the req_wrapper lock too long in endpt_send_request().  We
+	  could block the PJSIP monitor thread if the timeout timer expires.
+	  sip_get_tpselector_from_endpoint() does a sorcery access that could take
+	  awhile accessing a database.  pjsip_endpt_send_request() might take awhile
+	  if selecting a transport.
+
+	  * Shorten the time that the req_wrapper lock is held in the callback
+	  functions.
+
+	  * Simplify endpt_send_request() req_wrapper->timeout code.
+
+	  * Removed some redundant req_wrapper->timeout_timer->id assignments.
+
+	  Change-Id: I3195e3a8e0207bb8e7f49060ad2742cf21a6e4c9
+
+2016-09-21 15:10 +0000 [bb196323f9]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Fix tdata leaks in off nominal paths.
+
+	  Change-Id: Ie83e06e88c2d60157775263b07e40b61718ac97b
+
+2016-10-24 12:41 +0000 [9df59d9ff4]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_registrar_expire.c: Remove extra linefeed in debug message.
+
+	  Change-Id: I1f9adb911f23376503396ec8867e8005b755eb94
+
+2016-11-10 13:38 +0000 [73524bde9c]  C.J. Collier <cjcollier@linuxfoundation.org>
+
+	* chan_sip: Fix typo and re-wrap surrounding docs
+
+	  Correct typo of end-pints to end-points
+	  Re-wrap session timer parameter docs to max 80 chars wide; this
+	  eases reading on terminals with lower resolution, commonly the case
+	  for those with visual impairments.
+
+	  ASTERISK-26573
+
+	  Change-Id: I22c94459f4bb6b8a2f6713cfd22e87c32f204e6b
+	  Signed-off-by: C.J. Collier <cjcollier@linuxfoundation.org>
+
+2016-11-09 15:14 +0000 [bdb6d928c5]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Perform resolution when explicit IPv6 transport is used.
+
+	  This change fixes the SIP resolver such that if an IPv6 transport
+	  is explicitly used it will resolve NAPTR, SRV, and AAAA records.
+
+	  You can explicitly use one by specifying it on an endpoint.
+
+	  ASTERISK-26571
+
+	  Change-Id: I2ed3ce81b43a6a8a937c0ebc1b8ed2da5ac2ef36
+
+2016-11-10 08:33 +0000 [93a0de1f0e]  Joshua Colp <jcolp@digium.com>
+
+	* app_queue: Add mention of 'ABANDON' variable to CHANGES.
+
+	  ASTERISK-26558
+
+	  Change-Id: I1127010181e79c8ac291f72f036cb8e430dc7f7e
+
+2016-11-10 07:34 +0000 [fa749866c1]  George Joseph <gjoseph@digium.com>
+
+	* Revert "Add API for channel frame deferral."
+
+	  This reverts commit f073f648b87d45e4729969fd2d83695c300757d1.
+	  Multiple testsuite failures were detected after the fact.
+
+	  Change-Id: I968c380418bf65c7166f6ecff30fe8e247ea6682
+
+2016-11-10 07:33 +0000 [6bce938c2f]  George Joseph <gjoseph@digium.com>
+
+	* Revert "AGI: Only defer frames when in an interception routine."
+
+	  This reverts commit 28926d1c81540bbeb16802814d3f2e63c2347bd2.
+	  Multiple testsuite failures were detected after the fact.
+
+	  Change-Id: I8d4f5ccbb421a351d616254844ae7e5a31053edb
+
+2016-11-10 07:32 +0000 [edca6911f3]  George Joseph <gjoseph@digium.com>
+
+	* Revert "autoservice: Use frame deferral API"
+
+	  This reverts commit afef1b8e4a311d33b3e485b9bab3c6e7fd13fbc9.
+	  Multiple testsuite failures were detected after the fact.
+
+	  Change-Id: Ib4cb0c0a6475681ce817f71b4050be25640ab67f
+
+2016-11-10 07:31 +0000 [e5365dada5]  George Joseph <gjoseph@digium.com>
+
+	* Revert "channel: Use frame deferral API for safe sleep."
+
+	  This reverts commit 392202304d248147378f1e16f1f012285dc1221f.
+
+	  Multiple testsuite issues were discovered after the fact.
+
+	  Change-Id: I848c4196dca2994b1a368087004326ea354cff95
+
+2016-11-09 18:18 +0000 [edea41126b]  George Joseph <gjoseph@digium.com>
+
+	* build:  Fix default values for some SANITIZER options
+
+	  2 of the sanitizers didn't have default values so in systems that
+	  don't support sanitizers menuselect would spit out warnings.  They
+	  were harmless but confusing.  They've now been set to "0".
+
+	  Change-Id: I08dc495e3b83f1feac3160b421f538c375fc5d58
+
+2016-11-06 06:04 +0000 [4e8ab6cda9]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+
+	* app_queue: new variable set when abandoned
+
+	  sets the variable ABANDONED to TRUE if the call was not answered.
+
+	  ASTERISK-26558
+
+	  Change-Id: I4729af9bff4eba436d8a776afd3374065d0036d3
+
+2016-11-08 10:48 +0000 [e5860ce07d]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_session: Do not call session supplements when it's too late.
+
+	  res_pjsip_sesssion was hooking into transaction and invite state
+	  changes. One of the reasons for doing so was due to the
+	  PJSIP_EVENT_TX_MSG event. The idea was that we were hooking into the
+	  message sending process, and so we should call session supplements to
+	  alter the outgoing message.
+
+	  In reality, this event was meant to indicate that the message either
+	  a) had already been sent, or
+	  b) required a DNS lookup and would be sent when the DNS query
+	  completed.
+
+	  In case (a), this meant we were altering an already-sent
+	  request/response for no reason. In case (b), this potentially meant we
+	  could be trying to alter a request/response at the same time that the
+	  DNS resolution completed. In this case, it meant we might be stomping on
+	  memory being used by the thread actually sending the message. This
+	  caused potential crashes and memory corruption.
+
+	  This patch removes the calls to session supplements from the case where
+	  the PJSIP_EVENT_TX_MSG event occurs. In all of these cases, trying to
+	  alter the message at this point is too late, and it can cause nothing
+	  but harm to try to do it. Because there were no longer any calls to the
+	  handle_outgoing() function, it has been removed.
+
+	  Change-Id: Ibcc223fb1c3a237927f38754e0429e80ee301e92
+
+2016-11-03 16:46 +0000 [392202304d]  Mark Michelson <mmichelson@digium.com>
+
+	* channel: Use frame deferral API for safe sleep.
+
+	  This is another case where manual frame deferral can be replaced with
+	  centralized routines instead.
+
+	  Change-Id: I42cdf205f8f29a7977e599751a57efbaac07c30e
+	  (cherry picked from commit d149c4b9e07eeb880d8428ad52c6fdb315cc15f5)
+
+2016-11-03 16:46 +0000 [afef1b8e4a]  Mark Michelson <mmichelson@digium.com>
+
+	* autoservice: Use frame deferral API
+
+	  Rather than use manual frame deferral, just let the channel API do it
+	  for us.
+
+	  ASTERISK-26343
+
+	  Change-Id: I688386f36e765dbc07be863943a43f26bd5eac49
+	  (cherry picked from commit 8ba3e2fc27f9966b8c7ce75c1eca6208613a9315)
+
+2016-11-03 16:42 +0000 [28926d1c81]  Mark Michelson <mmichelson@digium.com>
+
+	* AGI: Only defer frames when in an interception routine.
+
+	  AGI recently was modified to defer important frames. This was because
+	  when AGI was used in a connected line interception routine, the
+	  resulting connected line frame would end up getting discarded by the
+	  AGI.
+
+	  However, this caused bad behavior in other cases. Specifically, during a
+	  transfer, if someone attempted to manually set the Caller ID on a
+	  channel in an AGI, the deferred connected line frame would end up
+	  overwriting what had been manually set in the AGI.
+
+	  Since the initial issue was specific to interception routines, this
+	  change removes the manual frame deferral from AGI and instead uses the
+	  new frame deferral API in interception routines.
+
+	  ASTERISK-26343 #close
+	  Reported by Morton Tryfoss
+
+	  Change-Id: Iab7d39436d0ee99bfe32ad55ef91e9bd88db4208
+
+2016-11-03 16:36 +0000 [f073f648b8]  Mark Michelson <mmichelson@digium.com>
+
+	* Add API for channel frame deferral.
+
+	  There are several places in Asterisk that have duplicated logic
+	  for deferring important frames until later.
+
+	  This commit adds a couple of API calls to facilitate this automatically.
+
+	  ast_channel_start_defer_frames(): Future reads of deferrable frames on
+	  this channel will be deferred until later.
+
+	  ast_channel_stop_defer_frames(): Any frames that have been deferred get
+	  requeued onto the channel.
+
+	  ASTERISK-26343
+
+	  Change-Id: I3e1b87bc6796f222442fa6f7d1b6a4706fb33641
+
+2016-11-02 10:52 +0000 [d30415bfa1]  Joshua Colp <jcolp@digium.com>
+
+	* res_stasis: Don't unsubscribe from a NULL bridge.
+
+	  A NULL bridge has special meaning in res_stasis for
+	  unsubscribing. It means that a subscription to ALL
+	  bridges should be removed. This should not be done
+	  as part of the normal subscription management in
+	  the res_stasis channel loop.
+
+	  ASTERISK-26468
+
+	  Change-Id: I6d5bea8246dd13a22ef86b736aefbf2a39c15af0
+
+2016-11-03 07:42 +0000 [0a698cd932]  Alexander Anikin <may213@yandex.ru>
+
+	* chan_ooh323: Fixes to work right with Cisco devices
+
+	  Changed output packets queue processing algo to one read-one write
+	  instead of all read-all send
+
+	  Remove h.245 tunneling parameter from ReleaseComplete packet
+
+	  ASTERISK-24400 #close
+	  Reported by: Dmitry Melekhov
+	  Tested by: Dmitry Melekhov
+
+	  Change-Id: I0b31933b062a21011dbac9a82b8bcfe345f406f6
+
+2016-11-03 13:10 +0000 [a1cdc3891a]  Alexander Anikin <may213@yandex.ru>
+
+	* chan_ooh323: reset rrq count on gk registration
+
+	  reset registration attempts count on success registration on gatekeeper
+
+	  Change-Id: I5f47351852e0ca76c9ac78421659600e0f106336
+
+2016-11-06 05:40 +0000 [b2b5f9d897]  frahaase <fra.haase@googlemail.com>
+
+	* ast_format: Adds an identifier for interleaved audio formats to the ast_format
+
+	  Adds an identifier (with a getter and setter) to detect channels with
+	  interleaved audio.
+	  This is needed by the binaural bridge_softmix patch (ASTERISK-26292) and
+	  was already discussed here:
+	  http://lists.digium.com/pipermail/asterisk-dev/2016-October/075900.html
+	  The identifier can be set during fmtp parsing (to be seen in the
+	  res_format_attr_opus.c change).
+
+	  ASTERISK-26292
+
+	  Change-Id: I359801cc5f98c35671c48dabc81a7f4ee1183d63
+
+2016-11-06 03:46 +0000 [fbbbd0add9]  Michael Kuron <m.kuron@gmx.de>
+
+	* automon: restore mixing of the both channels after recording stops
+
+	  This is a regression over Asterisk 11, introduced by
+	  2dc8a060064f359a17f5ebcd515d85fe5203c019. Previously, recordings started via
+	  the automon DTMF code would automatically be mixed together using sox because
+	  app_monitor would be called with the m option. This commit restores this
+	  behavior.
+
+	  Change-Id: Ibaf58684285c3f1b6ca3714524e6d638ae3b3759
+
+2016-11-04 15:42 +0000 [367d4903cc]  Matt Jordan <mjordan@digium.com>
+
+	* res_http_websocket: Increase the buffer size for non-LOW_MEMORY systems
+
+	  Not surprisingly, using Respoke (and possibly other systems) it is
+	  possible to blow past the 16k limit for a WebSocket packet size. This
+	  patch bumps it up to 32k, which, at least for Respoke, is sufficient.
+	  For now.
+
+	  Because 32k is laughable on a LOW_MEMORY system (as is 16k, for that
+	  matter), this patch adds a LOW_MEMORY directive that sets the buffer to
+	  8k for systems who have asked for their reduced memory availability to
+	  be considered.
+
+	  Change-Id: Id235902537091b58608196844dc4b045e383cd2e
+
+2016-11-04 15:40 +0000 [7a449b6819]  Matt Jordan <mjordan@digium.com>
+
+	* res_stasis: Set a video source mode on Stasis created bridges
+
+	  When a bridge is created via ARI (through res_stasis), no video source
+	  mode is set by default. As a result, any endpoint sending video media
+	  won't ever see any video reflected back to it.
+
+	  This patch defaults a bridge to a 'follow the talker' video mode.
+	  Further work can be done to add routes that allow for the video mode to
+	  be controlled through the /bridges resource.
+
+	  Change-Id: I7e9d530a5d7a97a4524a9ee4e468e1a6b3443866
+
+2016-11-04 15:37 +0000 [bbe943729a]  Matt Jordan <mjordan@digium.com>
+
+	* main/bridge_channel: Fix channel reference leak on video source
+
+	  When a channel is made the video source, the bridge holds a reference to
+	  it. Whenever the video source changes, that reference is released.
+	  However, a ref leak does occur if the channel leaves the bridge (such as
+	  being hung up) while it is the video source, as the bridge never
+	  releases the ref in such a case.
+
+	  This patch adds a line to the bridge_channel_internal_join routine such
+	  that, when a channel finishes its time in the bridge, it notifies the
+	  bridge via ast_bridge_remove_video_src that if it is a video source its
+	  reference should be released.
+
+	  ASTERISK-26555 #close
+
+	  Change-Id: I3a2f5238a9d2fc49c591f0e65199d782ab0be76a
+
+2016-11-04 15:36 +0000 [a70d6dba8c]  Matt Jordan <mjordan@digium.com>
+
+	* main/bridge: Add some verbose logging for video source changes
+
+	  It's actually quite useful to see the source of a video stream change.
+	  This doesn't happen terribly often, even with talk detection - but when
+	  it does, it's nice to know which channel is now providing your video
+	  stream.
+
+	  As a verbose 5 level message, it shouldn't be terribly spammy or costly
+	  to have, and is 'lower level' then most other verbose messages that the
+	  bridge system emits.
+
+	  ASTERISK-26555
+
+	  Change-Id: Ia1c20ecafa9670171fd38bddcf3beccae47fb15c
+
+2016-11-04 15:33 +0000 [fb17b630a5]  Matt Jordan <mjordan@digium.com>
+
+	* bridges/bridge_softmix: Remove SSRC changes on join/leave; update video source
+
+	  WebRTC clients really, really want to know the SSRC of the media they're
+	  getting. Changing the SSRC is generally not a good thing.
+
+	  bridge_softmix, starting in Asterisk 12, started changing the SSRC of
+	  parties as they joined or left the bridge. With most phones, this isn't
+	  a problem: phones just play back the stream they're getting. With WebRTC
+	  clients, however, the SSRC is tied to a media stream that may be
+	  negotiated. When a new SSRC just shows up, the media can be dropped.
+
+	  As it turns out, the SSRC change shouldn't even be necessary. From the
+	  perspective of the client, it's still talking to Asterisk with the same
+	  media stream: why indicate that the far party has suddenly changed to a
+	  different source of media?
+
+	  This patch opts to just remove the SSRC changes. With this patch, video
+	  clients that join/leave a softmix bridge actually get the video stream
+	  instead of freaking out.
+
+	  ASTERISK-26555
+
+	  Change-Id: I27fec098b32e7c8718b4b65f3fd5fa73527968bf
+
+2016-10-28 15:11 +0000 [70d5f90e3d]  Kevin Harwell <kharwell@digium.com>
+
+	* stasis_recording/stored: remove calls to deprecated readdir_r function.
+
+	  The readdir_r function has been deprecated and should no longer be used. This
+	  patch removes the readdir_r dependency (replaced it with readdir) and also moves
+	  the directory search code to a more centralized spot (file.c)
+
+	  Also removed a strict dependency on the dirent structure's d_type field as it
+	  is not portable. The code now checks to see if the value is available. If so,
+	  it tries to use it, but defaults back to using the stats function if necessary.
+
+	  Lastly, for most implementations of readdir it *should* be thread-safe to make
+	  concurrent calls to it as long as different directory streams are specified.
+	  glibc falls into this category. However, since it is possible that there exist
+	  some implementations that are not safe, locking has been added for those other
+	  than glibc.
+
+	  ASTERISK-26412
+	  ASTERISK-26509 #close
+
+	  Change-Id: Id8f54689b1e2873e82a09d0d0d2faf41964e80ba
+
+2016-11-04 10:57 +0000 [bf01ff53f8]  Kevin Harwell <kharwell@digium.com>
+
+	* Revert "chan_sip: Fix lastrtprx always updated"
+
+	  This reverts commit 93332cb1d0eea18021ea6538237297e627d6e2fc.
+
+	  Unfortunately, the aforementioned commit caused a regression (incoming calls
+	  would eventually disconnect). Thus it is being removed.
+
+	  ASTERISK-26523 #close
+	  ASTERISK-25270
+
+	  Change-Id: Ibf5586adc303073a8eac667a4cbfdb6be184a64d
+
+2016-11-03 13:45 +0000 [1504194215]  Alexander Anikin <may213@yandex.ru>
+
+	* chan_ooh323: Fix infinite loop on read second part of H.225 packet
+
+	  Fix logic on read second part of H.225 packet. There was infinite loop on
+	  wrong connections due to read before poll.
+
+	  Change-Id: I42b4bf75c46e4a5c5df5c5ca1f0bd74b8944e7ff
+
+2016-11-03 11:55 +0000 [78dc6ceaf6]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fix issue with libasteriskpj needing libresample
+
+	  libresample is only needed by pjproject if we're building pjsua, which
+	  we only do if TEST_FRAMEWORK is selected.  It's required by pjsua to
+	  process audio which is needed by some testsuite tests.  Unfortunately,
+	  pjproject relies on a newer version of libresample than the version
+	  that ships by most distros so we need to compile the version that's
+	  bundled with pjproject.  Since we only need it for pjsua, we DON'T want
+	  it's symbols exposed when we actually build asterisk.
+
+	  There was a problem however... TEST_FRAMEWORK is only known AFTER we've
+	  already run ./configure on both asterisk and pjproject but pjproject's
+	  ./configure needs to test it to know whether to set up to build
+	  libresample or not.  The previous way of figuring this out was to
+	  always tell ./configure "yes" but not actually build the library.  This
+	  caused an issue where building libasteriskpj was being told to include
+	  libresample but it wasn't actually there.
+
+	  The solution is to still do a default pjproject configure during an
+	  asterisk ./configure but if makeopts or menuselect.makeopts changes
+	  subsequently, we now reconfigure pjproject, taking into account the
+	  current state of TEST_FRAMEWORK.  Previously, if makeopts or
+	  menuselect.makeopts changed, only a recompile of pjproject was done.
+
+	  Change-Id: I9b5d84c61384a3ae07fe30e85c49698378cc4685
+
+2016-11-01 19:48 +0000 [0904c1f4cc]  Sebastian Gutierrez <sgutierrez@integraccs.com>
+
+	* chan_sip: add missing account code
+
+	  Added missing account to AMI event of sip show peers
+
+	  ASTERISK-26176 #close
+
+	  Change-Id: Ieb6c2c80a838a1b59c82103eba4c63ba238dc482
+
+2016-11-02 09:15 +0000 [4de5454ef1]  Joshua Colp <jcolp@digium.com>
+
+	* app_dial: Fix incorrect device state when channel is picked up.
+
+	  Given the scenario where multiple channels are dialed using Dial()
+	  but the caller is picked up using PickupChan() all outgoing channels
+	  except the channel specified to PickupChan() would be marked
+	  as ringing until the call had been hung up.
+
+	  When using the PickupChan application the channel executing the
+	  application is swapped into place of another channel. As part
+	  of this process the channel is answered. The Dial application
+	  has explicit logic which checks if the channel is answered,
+	  cancels all other outgoing channels, and bridges. This logic is
+	  different than the normal logic that is executed when an outgoing
+	  channel is answered. This different logic failed to publish dial
+	  events stating that the other outgoing channels had been canceled.
+	  As a result references to the outgoing channels were held onto by
+	  the dial masquerade process until the call had been ended and
+	  the channels had gone away. This would result in the channels
+	  appearing in the "core show channels" list despite not being present
+	  anymore and would also result in incorrect device state.
+
+	  This change makes it so that this logic also publishes
+	  dial events stating that the other outgoing channels have been
+	  canceled.
+
+	  ASTERISK-26549
+
+	  Change-Id: Iea7168e6e82f7d4609ec0366153804e4f55ea64f
+
+2016-09-13 04:08 +0000 [9ac53877f6]  Alexander Traud <pabstraud@compuserve.com>
+
+	* rtp_engine: Allow more than 32 dynamic payload types.
+
+	  Since adding all remaining rates of Signed Linear (ASTERISK-24274), SILK
+	  (Gerrit 3136) and Codec 2 (ASTERISK-26217), no RTP Payload Type is left in the
+	  dynamic range (96-127). RFC 3551 section 3 allows to reassign other ranges.
+	  Consequently, when the dynamic range is exhausted, this change utilizes payload
+	  types in the range between 35 and 63 giving room for another 29 payload types.
+
+	  ASTERISK-26311 #close
+
+	  Change-Id: I7bc96ab764bc30098a178b841cbf7146f9d64964
+
+2016-11-02 05:05 +0000 [6a99f007d6]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* autoconf: more variants for OSARCH linux-gnu
+
+	  There are quite a few odd GNU/Linux platforms. Just call all of them
+	  linux-gnu.
+
+	  Specifically this fixes building the Debian platforms mips64el and x32.
+	  And maybe also others.
+
+	  ASTERISK-26546 #close
+
+	  Change-Id: I06ec4bd7f0ee1c84b6b24d81538223b07c4174b1
+
+2016-11-01 13:13 +0000 [f29b8d62bb]  Richard Mudgett <rmudgett@digium.com>
+
+	* bundled pjproject: Fix DNS write to freed memory.
+
+	  PJPROJECT 2.5.5 introduced a race condition with the -r5349 IPv6 DNS
+	  patch.
+
+	  The patch below fixes a write to freed memory under cartain DNS lookup
+	  conditions.
+
+	  0006-r5477-svn-backport-Fix-DNS-write-on-freed-memory.patch
+
+	  ASTERISK-26516
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: Ifdfae9ecf1e41b53080f33aab44ce1a220f349c5
+
+2016-11-01 06:56 +0000 [6233e146c6]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_sdp_rtp: Limit number of formats to defined maximum.
+
+	  The res_pjsip_sdp_rtp module did not restrict the number of
+	  formats added to a media stream in the SDP to the defined
+	  limit. If allow=all was used with additional loaded codecs this
+	  could result in the next media stream being overwritten some.
+
+	  This change restricts the module to limit it to the defined
+	  maximum and also increases the maximum in our bundled pjproject.
+
+	  ASTERISK-26541 #close
+
+	  Change-Id: I0dc5f59d3891246cafa2f3df5ec406f088559ee8
+
+2016-10-31 17:35 +0000 [8060cd1ec1]  Kevin Harwell <kharwell@digium.com>
+
+	* codecs.conf.sample: Add sample and option descriptions for codec_opus
+
+	  codecs.conf.sample was missing codec opus's configuration options, descriptions,
+	  and examples. This patch adds the configuration options and examples to
+	  codecs.conf.sample that can be used with codec_opus.
+
+	  ASTERISK-26538 #close
+
+	  Change-Id: I1d89bb5e01d3e3b5bd78951b8dd0ff077a83dc8b
+
+2016-10-20 07:27 +0000 [c30d677333]  Matt Jordan <mjordan@digium.com>
+
+	* res/stasis: Add CLI commands for displaying/debugging ARI apps
+
+	  This patch adds three new CLI commands:
+	   - ari show apps: list the registered ARI applications
+	   - ari show app: show detailed information about an ARI application
+	   - ari set debug: dump events being sent to an ARI application
+
+	  Note that while these CLI commands live in the res_stasis module, we use
+	  the 'ari' family for these commands. This was done as most users of
+	  Asterisk aren't aware of the semantic differences between ARI and
+	  res_stasis, and some 'ari' CLI commands already exist.
+
+	  ASTERISK-26488 #close
+
+	  Change-Id: I51ad6ff0cabee0d69db06858c13f18b1c513c9f5
+
+2016-11-01 08:32 +0000 [2526dff94d]  Grachev Sergey <grachev@mcn.ru>
+
+	* chan_sip: Incorrect display option Outbound reg. retry 403
+
+	  If in sip.conf (general section) set option register_retry_403=no,
+	  the command "sip show settings" return value:
+	  Outbound reg. retry 403:0
+	  If in sip.conf (general section) set option register_retry_403=yes,
+	  the command "sip show settings" return value:
+	  Outbound reg. retry 403:-1
+
+	  * In static char "sip show settings" for "Outbound.reg. retry 403"
+	  option use AST_CLI_YESNO
+
+	  ASTERISK-26476 #close
+
+	  Change-Id: I3c14272f05f1067bd2aeaa8b3ef9cf8fcb12dcf9
+
+2016-11-01 04:18 +0000 [ed08811e64]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* netsock.c: fix includes for HURD
+
+	  ASTERISK-25070
+
+	  Change-Id: I43bf94d2d36d3d8a8d0df40cd6c027d65a462814
+
+2016-11-01 04:00 +0000 [69fed26deb]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* define PATH_MAX for HURD
+
+	  PATH_MAX is not guaranteed to be defined. In parctice, all but the HURD
+	  define it to a constant. It is indeed not safe to assume there won't be
+	  longer paths and Asterisk generally does err safely on such cases.
+
+	  So even for HURD we'll just pretend PATH_MAX is 4096.
+
+	  ASTERISK-25070 #close
+
+	  Change-Id: I53d10ba18c34c132bcb640a5fd8e0da1d9b22db3
+
+2016-10-31 16:12 +0000 [f27f837a9f]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fix compile of pjsua so it handles audio
+
+	  In order for pjsua and its python binding to actually negotiate
+	  audio for the testsuite tests, it needs g711 and resample.  The
+	  pj* libraries themselves do not.  Unfortunately, pjproject relies
+	  on a brand new libresample that most distros don't ship so we need
+	  to use the libresample already bundled with pjproject.  Only the pjsua
+	  executable and the _pjsua.so python library are linked with it so it
+	  shouldn't interfere with asterisk itself.
+
+	  Also it was pointed out that apply_patches couldn't handle multiple
+	  patches that depended on each other during the dry-run, so the
+	  dry-run was removed.
+
+	  Change-Id: I24f397462b486dcdde0dcafe40e6c55a6593f098
+
+2016-10-31 13:46 +0000 [1648ca06c3]  Etienne Lessard <elessard@proformatique.com>
+
+	* manager: Add documentation for NewConnectedLine event.
+
+	  The NewConnectedLine event has been added by commit fe7671f, but the
+	  documentation was missing.
+
+	  ASTERISK-26537 #close
+
+	  Change-Id: I7fc331f18caa28492da9303e576f70884ca8c9e6
+
+2016-10-30 13:33 +0000 [273debd261]  Corey Farrell <git@cfware.com>
+
+	* vector: Prevent NULL argument to memcpy.
+
+	  Headers declare that memcpy does not accept NULL argument for the first
+	  two parameters.  Add a conditional block to prevent memcpy and ast_free
+	  from running on vectors with NULL element array.
+
+	  ASTERISK-26526 #close
+
+	  Change-Id: I988a476bb5fcfcbd3f6d6c6b3e7769e4f9629b71
+
+2016-10-29 10:19 +0000 [ad60927a40]  Corey Farrell <git@cfware.com>
+
+	* astobj2: Declare private variable data_size for AO2_DEBUG only.
+
+	  Every ao2 object contains storage for a private variable data_size,
+	  though the value is never read if AO2_DEBUG is disabled.  This change
+	  makes the variable conditional, reducing memory usage.
+
+	  ASTERISK-26524 #close
+
+	  Change-Id: If859929e507676ebc58b0f84247a4231e11da07f
+
+2016-10-28 14:55 +0000 [6feee22e09]  Richard Mudgett <rmudgett@digium.com>
+
+	* bundled pjproject: Crashes while resolving DNS names.
+
+	  PJPROJECT 2.5.5 introduced a race condition with the -r5349 IPv6 DNS
+	  patch.
+
+	  The patches below fix the DNS lookup race condition crash caused by
+	  attempting to send the same message twice for the single DNS lookup.
+
+	  0006-r5471-svn-backport-Various-fixes-for-DNS-IPv6.patch
+	  0006-r5473-svn-backport-Fix-pending-query.patch
+
+	  The patch below removes a cached DNS response from the hash table when
+	  another thread is referencing the old entry.  The table still contained
+	  the entry when it was destroyed which can result in inexplicable crashes.
+
+	  0006-r5475-svn-backport-Remove-DNS-cache-entry.patch
+
+	  ASTERISK-26344 #close
+	  Reported by: Ian Gilmour
+
+	  ASTERISK-26387 #close
+	  Reported by: Harley Peters
+
+	  Change-Id: I17fde80359e66f65a91341ceca58d914d0f61cc4
+
+2016-10-28 16:59 +0000 [12bdde6a6c]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fix issue where "/version.mak" wasn't found
+
+	  main/Makefile includes third-party/pjproject/build.mak but
+	  doesn't set PJDIR beforehand so "include $(PJDIR)/version.mak"
+	  evaluates to "/version.mak".  Fix is to set PJDIR in main/Makefile
+	  before the include.
+
+	  Change-Id: I0f7c67d60209049056fe9c4b041bf0463aa95604
+
+2016-10-28 13:30 +0000 [9d8b9b6ca5]  Matt Krokosz <mkrokosz@vonage.com>
+
+	* res_pjsip_outbound_publish: Fix crash when publishing device state.
+
+	  While publishing device state between multiple instances of Asterisk,
+	  a crash will sporadically occur under high CPS which looks to be a
+	  race condition operating on the publisher queue.
+
+	  ASTERISK-26506
+
+	  Change-Id: I28da25d346deb358eff1d563485cabc433ce1ed6
+
+2016-10-27 21:49 +0000 [d6ad867897]  Corey Farrell <git@cfware.com>
+
+	* Fix shutdown crash caused by modules being left open.
+
+	  It is only safe to run ast_register_cleanup callbacks when all modules
+	  have been unloaded.  Previously these callbacks were run during graceful
+	  shutdown, making it possible to crash during shutdown.
+
+	  ASTERISK-26513 #close
+
+	  Change-Id: Ibfa635bb688d1227ec54aa211d90d6bd45052e21
+
+2016-10-28 09:50 +0000 [badd38f031]  Rusty Newton <rnewton@digium.com>
+
+	* SAC documentation: don't specify transports for endpoints and registrations
+
+	  Removing explicit transport definition for endpoints and registrations. It
+	  isn't necessary and isn't generally advised.
+
+	  ASTERISK-26514 #close
+
+	  Change-Id: Ifdec5e631962438a4683600968dfa4bfd15909fb
+
+2016-10-18 09:06 +0000 [0646b48ece]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* chan_dahdi: remove by_name support
+
+	  Support for referring to DAHDI channels by logical names was added in
+	  (FIXME: when? Asterisk 11? 1.8?) and was intended to be part of support
+	  of refering to channels by name.
+
+	  While technically usable, it has never been properly supported in
+	  dahdi-tools, as using it would require many changes at the Asterisk
+	  level. Instead logical mapping was added at the kernel level.
+
+	  Thus it seems that refering to DAHDI channels by name is not really used
+	  by anyone, and therefore should probably be removed.
+
+	  Change-Id: I7d50bbfd9d957586f5cd06570244ef87bd54b485
+
+2016-10-26 18:48 +0000 [4f45d62653]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Remove usage of tar's --strip-components option
+
+	  Older versions of tar don't support the --strip-components option so
+	  instead of doing 'tar --strip-components=1 -C source', we now just
+	  untar to the tarball's root directory (pjproject-<version>) and
+	  rename that directory to 'source'.
+
+	  Also fixed an issue where the pjproject source directory is a hard
+	  coded absolute pathname.
+
+	  ASTERISK-26510 #close
+	  ASTERISK-22480 #close
+
+	  Change-Id: I9ec92952507a91ff4e4d01e0149e09fd8e8f32b0
+
+2016-10-26 21:40 +0000 [a6e5bae3ef]  Corey Farrell <git@cfware.com>
+
+	* Remove ASTERISK_REGISTER_FILE.
+
+	  ASTERISK_REGISTER_FILE no longer has any purpose so this commit removes
+	  all traces of it.
+
+	  Previously exported symbols removed:
+	  * __ast_register_file
+	  * __ast_unregister_file
+	  * ast_complete_source_filename
+
+	  This also removes the mtx_prof static variable that was declared when
+	  MTX_PROFILE was enabled.  This variable was only used in lock.c so it
+	  is now initialized in that file only.
+
+	  ASTERISK-26480 #close
+
+	  Change-Id: I1074af07d71f9e159c48ef36631aa432c86f9966
+
+2016-10-27 08:07 +0000 [6993f3c9c3]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_caller_id: Fix crash on session timers UPDATE on inbound calls.
+
+	  The res_pjsip_caller_id module wrongly assumed that a
+	  saved From header would always exist on sessions. This
+	  is true until an inbound call is received and a session
+	  timer causes an UPDATE to be sent. In this case there will
+	  be no saved From header and a crash will occur. This change
+	  makes it fall back to the From header of the outgoing request
+	  if no saved From header is present.
+
+	  ASTERISK-26307 #close
+
+	  Change-Id: Iccc3bc8d243b5ede9b81abf960292930c908d4fa
+
+2016-10-26 07:51 +0000 [95062fe220]  Joshua Colp <jcolp@digium.com>
+
+	* app_voicemail: Clear voice mailbox in MailboxExists and MAILBOX_EXISTS.
+
+	  When executing the MailboxExists dialplan application and
+	  MAILBOX_EXISTS dialplan function the passed in temporary voice
+	  mailbox was not cleared, causing it to try to free garbage.
+
+	  ASTERISK-26503 #close
+
+	  Change-Id: Ie21ccfa1b80b9c59318e596f6b8e17da2b5a7cb3
+
+2016-10-23 07:38 +0000 [aed6c219a3]  Joshua Colp <jcolp@digium.com>
+
+	* pjsip: Fix a few media bugs with reinvites and asymmetric payloads.
+
+	  When channel format changes occurred as a result of an RTP
+	  re-negotiation the bridge was not informed this had happened.
+	  As a result the bridge technology was not re-evaluated and the
+	  channel may have been in a bridge technology that was incompatible
+	  with its formats. The bridge is now unbridged and the technology
+	  re-evaluated when this occurs.
+
+	  The chan_pjsip module also allowed asymmetric codecs for sending
+	  and receiving. This did not work with all devices and caused one
+	  way audio problems. The default has been changed to NOT do this
+	  but to match the sending codec to the receiving codec. For users
+	  who want asymmetric codecs an option has been added, asymmetric_rtp_codec,
+	  which will return chan_pjsip to the previous behavior.
+
+	  The codecs returned by the chan_pjsip module when queried by
+	  the bridge_native_rtp module were also not reflective of the
+	  actual negotiated codecs. The nativeformats are now returned as
+	  they reflect the actual negotiated codecs.
+
+	  ASTERISK-26423 #close
+
+	  Change-Id: I6ec88c6e3912f52c334f1a26983ccb8f267020dc
+
+2016-10-26 06:32 +0000 [7925f60cd9]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_sdp_rtp: Fix address family of explicit media_address.
+
+	  When an explicit media_address is provided the address family
+	  in the SDP needs to be set to reflect it.
+
+	  ASTERISK-26309
+
+	  Change-Id: Ib9350cc91c120eb2f96f0623d3907d12af67eb79
+
+2016-10-25 11:20 +0000 [802bbf8752]  George Joseph <gjoseph@digium.com>
+
+	* test_astobj2_thrash:  Fix multithreaded issues
+
+	  The test uses 4 threads to grow, count, lookup and shrink 15K objects
+	  in a container.  If there's only 1 execution engine available, the test
+	  will complete in <50ms.  If each threads gets its own execution engine,
+	  the test may timeout after 60 seconds because the count thread does a
+	  locked ao2_callback on the whole container in a tight loop with only
+	  a sched_yield to give up time.  The lock contention makes the test
+	  execution times wildly variable and mostly timeout.  2 execution
+	  engines are OK, 3 results in about 33% failure rate and >=4 causes
+	  a 80% failure rate.
+
+	  To fix, the sched_yield was changed to a usleep(500).
+
+	  Also, the number of buckets specified for the container was an even
+	  number so that was changed to the next prime number greater than
+	  (MAX_HASH_ENTRIES / 100).  That's 151 currently.
+
+	  Change-Id: I50cd2344161ea61bfe4b96d2a29a6ccf88385c77
+
+2016-10-18 09:04 +0000 [2b9ad3a5f7]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* chan_pjsip: segfault on already disconnected session
+
+	  On heavy loaded system the TCP/TLS incoming calls could be
+	  disconnected by pjproject while these calls are being
+	  processed by asterisk.
+
+	  This patch uses functions pjsip_inv_add_ref/pjsip_inv_dec_ref
+	  to inform pjproject that an INVITE session is in use.
+
+	  ASTERISK-26482 #close
+
+	  Change-Id: Ia2e3e2f75358cdb530252a9ce158af3d5d9fdf33
+
+2016-10-10 11:49 +0000 [01d1d3763f]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* cdr_radius,cel_radius: Fix old memleak in unload
+
+	  - Call "rc_openlog" optional. If you do not call,
+	  you will simply NULL instead of a name.
+
+	  - On the one PID can be only one syslog channel.
+	  And it can already be run in logger.c
+
+	  - Calling rc_openlog we assigns a new name for
+	  the channel syslog. This unexpected behavior for logger.c.
+
+	  Most lesser evil, is to agree on a NULL name syslog
+	  if the channel was not launched in logger.c.
+
+	  It also solves the problem of memory leaks.
+
+	  ASTERISK-26455 #close
+
+	  Change-Id: Ic17c38de67583e971d78fe18807d1a9faf8f0afd
+
+2016-10-24 10:55 +0000 [16c23b57c7]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Fixed various build issues
+
+	  * CFLAGS is now properly set when using older gcc.
+	  * All third-party pjproject targets have been removed.  This fixes
+	    an issue with older libsrtp in some distros.
+	  * Manually removing the source directory now causes a rebuild.
+	  * EXTERNALS_CACHE_DIR is now properly checked.
+	  * Whitespace fixes.
+
+	  Change-Id: I98fec6847efc5602a9f41cb95096fd660a49fa60
+
+2016-10-24 14:13 +0000 [1d277e7cb6]  Pascal Cadotte Michaud <pcadotte@proformatique.com>
+
+	* typo: s/paranthesis/parenthesis/ in a comment
+
+	  Change-Id: I7c1f4eb051177ee22cbe97e063d4a3effe29be30
+
+2016-09-19 06:13 +0000 [403c4f5833]  Joshua Colp <jcolp@digium.com>
+
+	* pjsip: Support dual stack automatically.
+
+	  This change adds support for dual stack automatically. No
+	  configuration is required and the IP address and version
+	  in the SIP messages and SDP will be automatically changed
+	  based on the transport over which the message is being
+	  sent. RTP usage has also been changed to listen on both
+	  IPv4 and IPv6 simultaneously to allow media to flow, and
+	  to allow ICE support on both simultaneously. This also
+	  allows failover between IPv6 and IPv4 to work as expected.
+
+	  ASTERISK-26309 #close
+
+	  Change-Id: I235a421d8f9a326606d861b449fa6fe3a030572d
+
+2016-10-19 12:05 +0000 [3bd76dd679]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Add duplicate channel ID checking for channel creation.
+
+	  This is similar to what is done for origination, but for the 14 and up
+	  channel creation method. When attempting to create a channel, if a
+	  channel ID is specified and a channel already exists with that ID, then
+	  a 409 is returned.
+
+	  Change-Id: I77f9253278c6947939c418073b6b31065489187c
+
+2016-10-17 14:18 +0000 [e459b8dadf]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Detect duplicate channel IDs
+
+	  ARI and AMI allow for an explicit channel ID to be specified
+	  when originating channels. Unfortunately, there is nothing in
+	  place to prevent someone from using the same ID for multiple
+	  channels. Further complicating things, adding ID validation to channel
+	  allocation makes it impossible for ARI to discern why channel allocation
+	  failed, resulting in a vague error code being returned.
+
+	  The fix for this is to institute a new method for channel errors to be
+	  discerned. The method mirrors errno, in that when an error occurs, the
+	  caller can consult the channel errno value to determine what the error
+	  was. This initial iteration of the feature only introduces "unknown" and
+	  "channel ID exists" errors. However, it's possible to add more errors as
+	  needed.
+
+	  ARI uses this feature to determine why channel allocation failed and can
+	  return a 409 error during origination to show that a channel with the
+	  given ID already exists.
+
+	  ASTERISK-26421
+
+	  Change-Id: Ibba7ae68842dab6df0c2e9c45559208bc89d3d06
+
+2016-10-19 17:53 +0000 [e03364c40a]  snuffy <snuffy22@gmail.com>
+
+	* Fix issue with CLI not returning to prompt after running "features show"
+
+	  ASTERISK-26444 #close
+
+	  Change-Id: I91d645b7e6e5dba35f8c410df2be77a8c0e3acb8
+
+2016-10-04 18:24 +0000 [3e96d491d0]  Michael Walton <mike@farsouthnet.com>
+
+	* res_rtp_asterisk: Add ice_blacklist option
+
+	  Introduces ice_blacklist configuration in rtp.conf. Subnets listed in the
+	  form ice_blacklist = <subnet spec>, e.g. ice_blacklist =
+	  192.168.1.0/255.255.255.0, are excluded from ICE host, srflx and relay
+	  discovery. This is useful for optimizing the ICE process where a system
+	  has multiple host address ranges and/or physical interfaces and certain
+	  of them are not expected to be used for RTP. Multiple ice_blacklist
+	  configuration lines may be used. If left unconfigured, all discovered
+	  host addresses are used, as per previous behavior.
+
+	  Documention in rtp.conf.sample.
+
+	  ASTERISK-26418 #close
+
+	  Change-Id: Ibee88f80d7693874fda1cceaef94a03bd86012c9
+
+2016-10-18 16:30 +0000 [f14ef51ead]  Mark Michelson <mmichelson@digium.com>
+
+	* CDR: Alter destruction pattern for CDR chains.
+
+	  CDRs form chains. When the root of the chain is destroyed, it then
+	  unreferences the next CDR in the chain. That CDR is destroyed, and it
+	  then unreferences the next CDR in the chain. This repeats until the end
+	  of the chain is reached. While this typically does not cause any sort of
+	  problems, it is possible in strange scenarios for the CDR chain to grow
+	  way longer than expected. In such a scenario, the destruction pattern
+	  can result in a stack overflow.
+
+	  This patch fixes the problem by switching from a recursive pattern to an
+	  iterative pattern for destruction. When the root CDR is destroyed, it is
+	  responsible for iterating over the rest of the CDRs and unreferencing
+	  each one. Other CDRs in the chain, since they are not the root, will
+	  simply destroy themselves and be done. This causes the stack depth not
+	  to increase.
+
+	  ASTERISK-26421 #close
+	  Reported by Andrew Nagy
+
+	  Change-Id: I3ca90c2b8051f3b7ead2e0e43f60d2c18fb204b8
+
+2016-10-18 11:51 +0000 [f31772ec20]  Joshua Colp <jcolp@digium.com>
+
+	* ari: Update model validator based on addition of asterisk_id.
+
+	  ASTERISK-26470
+
+	  Change-Id: I9c386f7a1c7d969161b28f189eb6298bbc5b7541
+
+2016-09-11 10:13 +0000 [18a6f250e2]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* menuselect: invalid test for GTK2
+
+	  configuire.ac was only checking for the existence of pkg-config
+	  and not the gtk2 package itself.  Now it calls AST_PKG_CONFIG_CHECK
+	  for gtk+-2.0.
+
+	  ASTERISK-26356 #close
+
+	  Change-Id: I93e9d0166341f0e7f84b52955bb6f81da42f2ef6
+
+2016-10-18 03:01 +0000 [a43ee21211]  Alexander Traud <pabstraud@compuserve.com>
+
+	* cli: Auto-complete File not Module for core set debug.
+
+	  Since Asterisk 1.8, the command "core set debug" on the command-line interface
+	  asks not for a file (.c) but a module name. This change shows modules (.so) on
+	  the auto-completion via a tabulator or the question mark. Now, when you
+	  partially type a module name, TAB or ?, you get the correct candidiates.
+
+	  ASTERISK-26480
+
+	  Change-Id: I1213f1dd409bd4ff8de08ad80cb0c73cafb1bae0
+
+2016-08-12 11:22 +0000 [dce31f90ba]  frahaase <fra.haase@googlemail.com>
+
+	* Binaural synthesis (confbridge): On/off setting for binaural synthesis.
+
+	  Adds setting to confbridge.conf (binaural_active) that determines if binaural
+	  synthesis can be available in bridge_softmix.
+
+	  ASTERISK-26292
+
+	  Change-Id: I59dfcb8e55fe1df4ef32045882fea5bb58fc71db
+
+2016-10-17 11:39 +0000 [2a808b2fa6]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Add patch to address SSL crash
+
+	  Addresses crashes when an attempt is made to operate on an SSL socket
+	  after the socket has been closed.
+
+	  ASTERISK-26477 #close
+
+	  Change-Id: I421305b357558b4f9e690210dc0f4831ef4b3002
+
+2016-10-13 14:09 +0000 [973e57d5ce]  Leandro Dardini <ldardini@gmail.com>
+
+	* app_queue: Added initialization for "context" parameter
+
+	  When using Asterisk Realtime Architecture, empty fields are skipped and the
+	  default values are used. If the "context" parameter in queue was set and then
+	  cleared from the database, the old value remains in memory and it continues
+	  to be used. This change initialize the "context" parameter with an empty value,
+	  allowing clearing the parameter.
+
+	  ASTERISK-26462 #close
+
+	  Change-Id: I64be73d5044ce38dd02408bd0e53de965ef65905
+
+2016-10-15 20:05 +0000 [dd5129d84a]  Matt Jordan <mjordan@digium.com>
+
+	* res/ari: Add the Asterisk EID field to outgoing events
+
+	  This patch adds the Asterisk EID field to all outgoing ARI events.
+	  Because this field should be added to all events as they are
+	  transmitted, it is appended to the JSON message just prior to it being
+	  handed off to the application message handler. This makes it somewhat
+	  resilient to both new events being added to ARI, as well as other
+	  potential event transport mechanisms.
+
+	  ASTERISK-26470 #close
+
+	  Change-Id: Ieff0ecc24464e83f3f44e9c3e7bd9a5d70b87a1d
+
+2016-10-13 02:06 +0000 [2b03017022]  Moises Silva <moises.silva@gmail.com>
+
+	* chan_rtp: Set a sane default rtp engine for unicast.
+
+	  ASTERISK-26439
+
+	  Change-Id: I7f5ee2eeba8906e9ecb3293dbe3a747770bb5011
+
+2016-10-16 17:25 +0000 [6651c66e68]  George Joseph <gjoseph@digium.com>
+
+	* utils.c:  Fix ast_set_default_eid for multiple platforms
+
+	  ast_set_default_eid was searching for ethX, emX, enoX, ensX and even
+	  pciD#U interface names.  While this was a good attempt, it wasn't
+	  inclusive enough to capture interfaces like enp6s0 or ens6d1, etc.
+
+	  Rather than relying on interface names, we now simply find the first
+	  interface returned by the OS that has a hardware address and that
+	  address isn't all 0x00 or all 0xff.  The code IS different for BSD,
+	  Solaris and Linux based on what method is available for enumerating
+	  interfaces.
+
+	  Tested on:
+	  FreeBSD9
+	  CentOS6
+	  Ubuntu14
+	  Fedora24
+
+	  I was unable to test on Solaris at this time but the code for Solaris
+	  is used elsewhere at Digium.
+
+	  Change-Id: Iaa6db87ca78a9a375e47d70e043ae08c1448cb72
+
+2016-10-15 04:58 +0000 [e9315791b3]  Michael Kuron <m.kuron@gmx.de>
+
+	* chan_sip: Only send video on outgoing channel if incoming channel supports it
+
+	  Previously, the settings videosupport=always and videosupport=yes behaved
+	  identically and unconditionally caused a video offer to be sent in the SDP on
+	  an outgoing call. This was a regression introduced with commit
+	  5a1d90e1fbfc4b48927aad55311f3b38efbf1f54 in Asterisk 1.6.1.
+
+	  This commit restores correct behavior: videosupport=always causes a video offer
+	  to be sent unconditionally, while videosupport=yes will only offer video on an
+	  outbound channel if the incoming channel it is bridged to also supports video.
+	  That way, the device receiving the outgoing call can display the correct user
+	  interface elements for audio or video and will not unnecessarily show a blank
+	  video window on an audio-only call.
+
+	  ASTERISK-17470 #close
+
+	  Change-Id: I782f4409d436114dbc97061c3570c0cd24f7c3ae
+
+2016-10-14 00:18 +0000 [aa39a87697]  Corey Farrell <git@cfware.com>
+
+	* Fix issues with bundled pjproject cached download.
+
+	  Previously when testing I had a preexisting makeopts in ASTTOPDIR.  The
+	  ordering of configure.ac causes --with-externals-cache to be processed
+	  after third-party configure.  In cases where the Asterisk clone is
+	  cleaned it would cause pjproject to be downloaded to /tmp.  This
+	  moves processing of the externals cache and sounds cache to happen
+	  before third-party configure.
+
+	  This also addresses a possible issue with the third-party Makefile.  If
+	  TMPDIR is set by the environment it would override the path given to
+	  --with-externals-cache.
+
+	  ASTERISK-26416
+
+	  Change-Id: Ifab7f35bfcd5a31a31a3a4353cc26a68c8c6592d
+
+2016-10-12 16:24 +0000 [9c49b96374]  Richard Mudgett <rmudgett@digium.com>
+
+	* Audit ast_json_pack() calls for needed UTF-8 checks.
+
+	  Added needed UTF-8 checks before constructing json objects in various
+	  files for strings obtained outside the system.  In this case string values
+	  from a channel driver's peer and not from the user setting channel
+	  variables.
+
+	  * aoc.c: Fixed type mismatch in s_to_json() for time and granularity json
+	  object construction.
+
+	  ASTERISK-26466
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Iac2d867fa598daba5c5dbc619b5464625a7f2096
+
+2016-10-12 16:20 +0000 [774d5f7ef7]  Richard Mudgett <rmudgett@digium.com>
+
+	* json: Check party id name, number, subaddresses for UTF-8.
+
+	  * Updated unit test as ast_json_name_number() is now NULL tolerant.
+
+	  ASTERISK-26466 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I7d4e14194f8f81f24a1dc34d1b8602c0950265a6
+
+2016-10-11 18:14 +0000 [1c4c6c082d]  Richard Mudgett <rmudgett@digium.com>
+
+	* json: Add UTF-8 check call.
+
+	  Since the json library does not make the check function public we
+	  recreate/copy the function in our interface module.
+
+	  ASTERISK-26466
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I36d3d750b6f5f1a110bc69ea92b435ecdeeb2a99
+
+2016-10-12 17:42 +0000 [6fe5202c2c]  Richard Mudgett <rmudgett@digium.com>
+
+	* aoc.c: Whitespace cleanup
+
+	  * In s_to_json() removed unnecessary ast_json_ref() to ast_json_null()
+	  when creating the type json object.  The ref is a noop.
+
+	  Change-Id: I2be8b836876fc2e34a27c161f8b1c53b58a3889a
+
+2016-10-12 16:22 +0000 [c3bf1632cd]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_minivm.c: Fix malformed ast_json_pack() call.
+
+	  Change-Id: I082b239022fac462666e52a14a44304748908dc0
+
+2016-10-12 17:27 +0000 [9c54964dc5]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Fix clearing of pause reason string.
+
+	  The pause reason is not always cleared when it should be cleared.
+
+	  * Made set_queue_member_pause() always clear pause reason if not pausing
+	  with a reason string.
+
+	  Change-Id: I993dad19626ec017478a230e980989438b778c53
+
+2016-10-12 16:30 +0000 [3b3d06884c]  George Joseph <gjoseph@digium.com>
+
+	* res_config_mysql:  Fix several issues related to recent table changes
+
+	  Unlike any of the other database drivers, res_config_mysql checks that
+	  the table definition matches the requirements for every insert and
+	  update statement.  Since all requirements are forced to 'char', any
+	  column that isn't a char, like ps_contacts' expiration_time,
+	  qualify_timeout, etc., will throw a warning.  It's kinda harmless but
+	  very misleading.  Since no other driver does those checks on insert
+	  or update, they've been removed from res_config_mysql.  Also, all
+	  the logic that actually attempted to ALTER the table to fix the issue
+	  has been removed.  With the move to alembic, the auto-alter
+	  functionality is not only unnecessary, it's also dangerous.
+
+	  The other issue is that res_config_mysql calls the mysql_insert_id
+	  function inside store_mysql.  Presumably the intention was to return
+	  the number of rows inserted DESPITE A NOTE IN THE CODE THAT THE VALUE
+	  IS NON_PORTABLE AND MAY CHANGE.  That value is then returned to
+	  config realtime as the number of rows inserted.  Guess what?  The value
+	  changed.  It now only returns the number of rows inserted if there's an
+	  auto increment column on the table, which ps_contacts doesn't have.
+	  Otherwise it returns 0.  So now, the insert worked but we tell config
+	  realtime and sorcery that no rows were inserted.  That call to
+	  mysql_insert_id was removed and we now always return 1 if the insert
+	  succeeded.  We're only inserting 1 row at a time anyway.  If the insert
+	  fails, we still return -1.
+
+	  ASTERISK-26362 #close
+	  Reported-by: Carlos Chavez
+
+	  Change-Id: I83ce633efdb477b03c8399946994ee16fefceaf4
+
+2016-08-12 11:22 +0000 [dd6fc1bb7d]  frahaase <fra.haase@googlemail.com>
+
+	* Binaural synthesis (confbridge): Adds libfftw3 as dependency.
+
+	  Adds libfftw3 to the build chain that is is going to be used for binaural
+	  synthesis by bridge_softmix.
+
+	  ASTERISK-26292
+
+	  Change-Id: Iedc2f174e4ccb39ae5d9e698e339c6a17155867b
+
+2016-09-29 13:08 +0000 [20c3dba39e]  Torrey Searle <torrey@voxbone.com>
+
+	* res_fax: Fix a tight race condition causing fax to crash in audio fallback
+
+	  When T.38 gets rejected and G711 failback occurs there is a period of
+	  time where neither AST_FAX_TECH_T38 nor AST_FAX_TECH_AUDIO is set,
+	  leading to a crash.
+
+	  Change-Id: Icc3f457b2292d48a9d7843dac0028347420cc982
+
+2016-10-06 09:58 +0000 [86e8716952]  George Joseph <gjoseph@digium.com>
+
+	* app_dial:  Add the "Q" option to set the cause on unanswered channels
+
+	  The "Q" option will set the cause on the unanswered channels when
+	  another channel answers.  It overrides the default of
+	  ANSWERED_ELSEWHERE.
+
+	  NOTE:  chan_sip does not support setting the cause on a CANCEL to
+	  anything other than ANSWERED_ELSEWHERE.
+
+	  ASTERISK-26446 #close
+
+	  Change-Id: I71742e0919aaa16784c30a2b2e73fbeed7672e47
+
+2016-10-11 06:55 +0000 [4f7f8a7e95]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Support nat=auto_comedia or nat=force_rport,auto_comedia.
+
+	  In the SIP channel driver chan_sip, auto_comedia was expected to be used in
+	  tandem with auto_force_rport. Or stated differently: Only when auto_force_rport
+	  was chosen (the default), auto_comedia worked. This change allows auto_comedia
+	  to be set independently of the state of (auto_)force_rport. For example,
+	  nat=force_rport,auto_comedia is useful for IPv4/IPv6 Dual Stack deployments
+	  when IPv6 clients are behind a Firewall.
+
+	  ASTERISK-26457 #close
+
+	  Change-Id: Ib29d66c6dbb61648e371e01fc36c6978ddae5bc2
+
+2016-10-10 16:59 +0000 [17031f12fe]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* vector: After remove element recheck index
+
+	  Small fix. It is necessary to double-check
+	  the index that we just removed because there
+	  is a new element.
+
+	  ASTERISK-26453 #close
+
+	  Change-Id: Ib947fa94dc91dcd9341f357f1084782c64434eb7
+
+2016-09-29 12:52 +0000 [cc269766b8]  Torrey Searle <torrey@voxbone.com>
+
+	* res_rtp_asterisk: Fix infinite DTMF issue when switching to P2P bridge
+
+	  If a bridge switched to P2P when a DTMF was in progress it
+	  was possible for the DTMF to continue being sent indefinitely.
+
+	  Change-Id: I7e2a3efe0d59d4b214ed50cd0b5d0317e2d92e29
+
+2016-10-09 21:28 +0000 [fafdde322c]  Corey Farrell <git@cfware.com>
+
+	* logger: Prevent output of verbose messages initiated from rasterisk.
+
+	  Remote asterisk consoles should only display verbose log messages
+	  created by the daemon.  The first patch for ASTERISK-26410 caused
+	  a couple verbose messages to be printed when the rasterisk process
+	  ended.
+
+	  ASTERISK-26410
+
+	  Change-Id: Ie2a1bb3753ad2724c0349ec1a336f52f7117b52a
+
+2016-10-04 20:46 +0000 [7af7490e42]  Michael Walton <mike@farsouthnet.com>
+
+	* audiohooks: Remove redundant codec translations when using audiohooks
+
+	  The main frame read and write handlers in main/channel.c don't use the
+	  optimum placement in the processing flow for calling audiohooks
+	  callbacks, as far as codec translation is concerned. This change places
+	  the audiohooks callback code:
+	   * After the channel read translation if the frame is not linear before
+	  the translation, thereby increasing the chance that the frame is linear
+	  as required by audiohooks
+	   * Before the channel write translation if the frame is linear at this
+	  point
+	  This prevents the audiohooks code from instantiating additional
+	  translation paths to/from linear where a linear frame format is already
+	  available, saving valuable CPU cycles
+
+	  ASTERISK-26419
+
+	  Change-Id: I6edd5771f0740e758e7eb42558b953f046c01f8f
+
+2016-10-10 10:59 +0000 [3ab7fae96b]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* res_pjsip_config_wizard: Memory leak in module_unload
+
+	  Fixed a memory leak. It removes only the first element.
+	  Added a useful feature in vector.h to remove all items
+	  under the CMP through a callback function / macro.
+
+	  ASTERISK-26453 #close
+
+	  Change-Id: I84508353463456d2495678f125738e20052da950
+
+2016-09-29 12:45 +0000 [9f62feca60]  Ludovic Gasc (GMLudo) <gmludo@gmail.com>
+
+	* res_calendar: Add support for fetching calendars when reloading
+
+	  We use a lot res_calendar, we are very happy with that, especially
+	  because you use libical, the almost alone opensource library that
+	  supports really ical format with all types of recurrency.
+
+	  Nevertheless, some features are missed for our business use cases.
+
+	  This first patch adds a new option in calendar.conf:
+	  fetch_again_at_reload. Be my guest for a better name.
+
+	  If it's true, when you'll launch "module reload res_calendar.so",
+	  Asterisk will download again the calendar.
+
+	  The business use case is that we have a WebUI with a scheduler planner,
+	  we know when the calendars are modified.
+
+	  For now, we need to define 1 minute of timeout to have a chance that
+	  our user doesn't wait too long between the modification and the real
+	  test.  But it generates a lot of useless HTTP traffic.
+
+
+	  ASTERISK-26422 #close
+
+	  Change-Id: I384b02ebfa42b142bbbd5b7221458c7f4dee7077
+
+2016-10-09 21:53 +0000 [ca2f3e5b99]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* cel_odbc: Fix memory leak on module unload
+
+	  Change-Id: Ic7a1236eba2408090fdabb5f717b5fa455ead715
+
+2016-10-03 11:30 +0000 [5fb848eebd]  George Joseph <gjoseph@digium.com>
+
+	* bundled_pjproject:  Add tests for programs used by the Makefile, et al.
+
+	  Added tests for bzip2, tar, patch, sed and nm to configure.ac.
+
+	  Set DOWNLOAD_TO_STDOUT to a working command line regardless of
+	  whether the download program is wget, curl or fetch.
+
+	  Added a 'configure.m4' file to the third-party directory which takes
+	  care of calling any third-party project setup.  Had to move some
+	  pjproject_bundled stuff up in configure.ac so it was called before
+	  the third-party configure macro.
+
+	  The pjproject tarball is now downloaded to the externals_cache_dir if
+	  it was specified on the ./configure command line
+
+	  Removed regeneration of the pjproject aconfigure file.  It was only
+	  needed for an old patch that no longer applies.
+
+	  Converted the tests for symbols to explicit tests since we know that
+	  they're now available in the bundled version.  Saves a little time
+	  during configure.
+
+	  ASTERISK-26416 #close
+	  Reported-by: Corey Farrell
+
+	  Change-Id: Id1d94251c0155f8dd41b7de7067f35cfbaafbb9b
+	  (cherry picked from commit e6b0053d7561032b7adbf6f3afaecf30f5046605)
+	  (cherry picked from commit a0d02f38322c2c4d7743504003fd376d32a133db)
+
+2016-10-09 18:54 +0000 [73f75c246b]  Joshua Colp <jcolp@digium.com>
+
+	* Revert "Packet-Loss Concealment (PLC) for supporting codecs."
+
+	  This change introduced some fax test failures
+	  that have not yet been addressed. So this is
+	  not forgotten I'm submitting a change which
+	  reverts it.
+
+	  This reverts:
+	  d56fc3b36b7bb59b5506129b9895b6c3341350c9.
+
+	  ASTERISK-25629
+
+	  Change-Id: Ibc2f23c38643f5a2c89cf8915ae2d805b81bc3d5
+
+2016-10-05 14:53 +0000 [c5e8f50169]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Add MALLOC_DEBUG capability
+
+	  pjproject_bundled will now use the asterisk memory debugging APIs
+	  if MALLOC_DEBUG is turned on in menuselect.
+
+	  Because this required stubs for the executable programs and the python
+	  bindings, some Makefile reorganization was needed to properly handle
+	  the dependencies.  As a result, the makefile now individually makes
+	  each of the pjproject libraries separately instead of making them all
+	  in 1 shot.  The only visible change is that there are separate status
+	  lines printed for each library instead oif 1 for all libs.  Also, the
+	  making of the pjproject dependency files was eliminated.  They're not
+	  needed for building unless you're actively modifying pjproject source
+	  files and it makes the build process faster.  Finally, any issues with
+	  parallel builds should be resolved again making the build faster.
+
+	  Change-Id: Icc5e3d658fbfb00e0a46b44c66dcc2522d5171b0
+
+2016-10-04 16:59 +0000 [442b597929]  George Joseph <gjoseph@digium.com>
+
+	* alembic:  Allow cdr, config and voicemail to exist in the same schema
+
+	  cdr, config and voicemail are all separate alembic trees.  Because
+	  alembic's default is to use a table named 'alembic_version' to store
+	  the current tree revision, the 3 trees can't exist in the same schema
+	  without stepping on each other.
+
+	  Now each tree uses 'alembic_version_<tree_name>' as the version table.
+	  Each tree's env.py script now first checks for 'alembic_version'.  If
+	  it finds it AND its revision is in the tree's history, the script
+	  renames it to 'alembic_version_<tree_name>'.  Regardless, the script
+	  then continues with the migration using 'alembic_version_<tree_name>'
+	  and creates that table if it's not found.  The result is that if an
+	  existing 'alembic_version' table was found but it didn't belong to this
+	  tree, it's left alone and 'alembic_version_<tree_name>' is used or
+	  created.
+
+	  WARNING:  If multiple trees are using the same schema, they MUST NOT
+	  CRU or D any objects with names that might exist in the other trees.
+	  An example would be 'yesno_values' type.  If two trees perform
+	  operations on it, one tree could pull it out from under the other.
+	  Thankfully we currently don't share any names among cdr, config and
+	  voicemail.
+
+	  NOTE:  Since the env.py scripts in each tree were identical, a common
+	  env.py has been placed in the ast-db-manage directory and a symlink
+	  to it has been placed in each tree directory.
+
+	  ASTERISK-24311 #close
+	  Reported-by: Dafi Ni
+
+	  Change-Id: I4d593f000350deb5d21a14fa1e9bc3896844d898
+
+2016-10-05 04:25 +0000 [c4268ec734]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Honor support of Symmetric Response (rport) for SIP requests.
+
+	  In the SIP channel driver chan_sip, the default is "auto_force_rport". When no
+	  NAT was detected, for example in case of IPv6, Asterisk uses the IP address
+	  from the headers within the SIP-REGISTER for subsequent SIP signaling. When
+	  the remote party specifies support for Symmetric Response (RFC 3581) via the
+	  parameter "rport", Asterisk should not extract the port from the SIP headers
+	  but reuse the port of the transport. This did not happen because of a typo.
+
+	  ASTERISK-26438 #close
+
+	  Change-Id: If6e7891848aaf96666dee5305695f7c6667cd5a6
+
+2016-08-12 11:22 +0000 [c455823657]  frahaase <fra.haase@googlemail.com>
+
+	* Binaural synthesis (confbridge): interleaved two-channel audio.
+
+	  Asterisk only supports mono audio at the moment.
+	  This patch adds interleaved two-channel audio to Asterisk's channels.
+
+	  ASTERISK-26292
+
+	  Change-Id: I7a547cea0fd3c6d1e502709d9e7e39605035757a
+
+2016-09-16 18:54 +0000 [2a03575c30]  Corey Farrell <git@cfware.com>
+
+	* astobj2: Add backtrace to log_bad_ao2.
+
+	  * Compile __ast_assert_failed unconditionally.
+	  * Use __ast_assert_failed to log messages from log_bad_ao2
+	  * Remove calls to ast_assert(0) that happen after log_bad_ao2 was run.
+
+	  Change-Id: I48f1af44b2718ad74a421ff75cb6397b924a9751
+
+2016-09-30 16:29 +0000 [79532bca75]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* Add text of cdr directory into README.md for ast-db-manage
+
+	  Change-Id: I68321c4bea50730c39fdb486e5f23aeadd1ad636
+
+2016-09-09 12:38 +0000 [806d08b675]  Etienne Lessard <elessard@proformatique.com>
+
+	* app_queue: Update dynamic members ringinuse on reload.
+
+	  Previously, when reloading the members of a queue, the members added statically
+	  (i.e. defined in queues.conf) would see their "ringinuse" value updated but not
+	  the members added dynamically.
+
+	  This change makes dynamic members ringuse value to be updated on reload.
+
+	  Note that it's impossible to add a dynamic member with a specific ringinuse
+	  value. For both static and dynamic members, the ringinuse value can always be
+	  changed later on with command like "queue set ringinuse" or with the AMI action
+	  "QueueMemberRingInUse". So it's possible this commit could break a user workflow
+	  if he was changing the ringinuse value of dynamic members via such commands and
+	  was also relying on the fact that a queue reload would not update the dynamic
+	  members ringinuse value.
+
+	  ASTERISK-26330
+
+	  Change-Id: I3745cc9a06ba7e02c399636f1ee9e58c04081f3f
+
+2016-09-29 14:02 +0000 [d31ffb421c]  Kevin Harwell <kharwell@digium.com>
+
+	* Remove "format_ogg_opus: New format"
+
+	  This reverts commit 40aa28131bc30b4516da2b20eb1a1e043920169c.
+
+	  ASTERISK-26426 #close
+
+	  Change-Id: I81e55c3c512f1dd6f49896f0c6b97a07d74fd8f5
+
+2016-09-19 04:46 +0000 [8c5c95ad89]  Corey Farrell <git@cfware.com>
+
+	* core: Remove ABI effects of LOW_MEMORY.
+
+	  This allows asterisk to compiled with LOW_MEMORY to load modules built
+	  without LOW_MEMORY.
+
+	  ASTERISK-26398 #close
+
+	  Change-Id: I24b78ac9493ab933b11087a8b6794f3c96d4872d
+
+2016-09-27 16:10 +0000 [a77ebb2017]  George Joseph <gjoseph@digium.com>
+
+	* download_externals: Fix issue with re-install
+
+	  Needed to ignore an xmlstarlet return code for optional element.
+
+	  Change-Id: I6a96f709b4b38c9a3f3dda4e8b07903787e16873
+	  Reported-by: Dan Jenkins
+
+2016-09-27 15:35 +0000 [2d2a8944be]  Corey Farrell <git@cfware.com>
+
+	* logger: Output early verbose messages to console.
+
+	  Verbose messages should be printed to the console if the sublevel is
+	  less than option_verbose.  This fix ensures the welcome message with
+	  copyright and license are printed at daemon and interactive rasterisk
+	  startup.
+
+	  ASTERISK-26410 #close
+
+	  Change-Id: Ia44235e30ec328aba92ea2c8a837b094e65c9a03
+
+2016-09-22 09:49 +0000 [c7ef1e0af3]  George Joseph <gjoseph@digium.com>
+
+	* codec_opus: Add download ability to menuselect
+
+	  Updated codecs/codecs.xml to add codec_opus to the external
+	  download list.
+
+	  ASTERISK-26409
+
+	  Change-Id: Ia07b36539f30e852125fb2b94147dc9774df31a4
+	  (cherry picked from commit 2cdab0e36eec4997ca3bd85aa09efc477038e31c)
+	  (cherry picked from commit e9684f3acd0e8def0df582c1505dd39dd3fd1610)
+
+2016-07-23 14:50 +0000 [5cc3c6679f]  George Joseph <gjoseph@digium.com>
+
+	* codec_opus: Replace res_format_attr_opus with the one from codec_opus
+
+	  Preparation
+
+	  ASTERISK-26409
+
+	  Change-Id: I9f20e7cce00c32464d9a180e81283d49d199d0a3
+	  (cherry picked from commit 59f7662a93bf9c07204fb50e1020a0f5bfbbd5c9)
+
+2016-07-23 15:56 +0000 [40aa28131b]  George Joseph <gjoseph@digium.com>
+
+	* format_ogg_opus: New format
+
+	  Add Ogg/Opus playback support.
+
+	  This uses libopusfile in order to be able to read .opus files and play
+	  them back.
+
+	  Writing/recording support is not present at this time.
+
+	  ASTERISK-26409
+
+	  Change-Id: I8815d23345108d8ca7c0bd640f6a1ce6b4f56955
+	  (cherry picked from commit daee8bbd5209b4158bc1785eede845a26e6cbeaa)
+
+2016-09-24 19:05 +0000 [43901e9418]  George Joseph <gjoseph@digium.com>
+
+	* build_tools:  Add ability to download variants to download_externals
+
+	  Some external packages have multiple variants that apply to different
+	  builds of asterisk.  The DPMA for instance has a "bundled" variant that
+	  needs to be downloaded if asterisk was configured with
+	  --with-pjproject-bundled.
+
+	  There are 2 ways to specify variants:
+
+	  If you need the user to make the decision about which variant to
+	  download, simply create multiple menuselect "member" entries like so...
+
+	  <member name="res_digium_phone" displayname="..snipped..">
+	    <support_level>external</support_level>
+	    <depend>xmlstarlet</depend>
+	    <depend>bash</depend>
+	    <defaultenabled>no</defaultenabled>
+	  </member>
+
+	  <member name="res_digium_phone-bundled" displayname="..snipped..">
+	    <support_level>external</support_level>
+	    <depend>xmlstarlet</depend>
+	    <depend>bash</depend>
+	    <defaultenabled>no</defaultenabled>
+	  </member>
+
+	  Note that the second entry has "-<variant>" appended to the name.
+	  You can then use the existing menuselect facilities to restrict which
+	  members to enable or disable.  Youy probably don't want the user to
+	  enable multiple at the same time.
+
+	  If you want to hide the details of the variants, the better way to
+	  do it is to create 1 member with "variant" elements.
+
+	  <member name="res_digium_phone" displayname="..snipped..">
+	    <support_level>external</support_level>
+	    <depend>xmlstarlet</depend>
+	    <depend>bash</depend>
+	    <defaultenabled>no</defaultenabled>
+	    <member_data>
+	      <downloader>
+	        <variants>
+	          <variant tag="bundled"
+	            condition='[[ "$PJPROJECT_BUNDLED" = "yes" ]]'/>
+	        </variants>
+	      </downloader>
+	    </member_data>
+	  </member>
+
+	  The condition must be a bash expression suitable for use with an "if"
+	  statement.  Any environment variable can be used plus those available
+	  in makeopts.
+
+	  In this case, if asterisk was configured with --with-pjproject-bundled
+	  the bundled variant will be automatically downloaded.  Otherwise the
+	  normal version will be downloaded.
+
+	  Change-Id: I4de23e06d4492b0a65e105c8369966547d0faa3e
+
+2016-09-23 09:54 +0000 [5dd99465d3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Resolve externhost not to IPv6; instead go for IPv4.
+
+	  For the channel driver chan_sip, you specify externhost=example.com in sip.conf
+	  when your Asterisk is behind a NAT and your IP address is assigned dynamically.
+	  Or stated differently: You do not have a static IP address to use "externaddr"
+	  directly. This NAT support is quite handy but just about IPv4. Previously,
+	  Asterisk resolved "externhost" to any IP version. When the first DNS answer
+	  resolved to an IPv6, Asterisk sent an IPv6 in SIP/SDP for origin (o=) and
+	  connection (c=). This happened in outgoing SIP-REGISTER and while answering
+	  SIP-INVITE. If the remote peer is IPv4-only, it might not handle o=/c= with an
+	  IPv6. This change makes sure, no IPv6 is resolved anymore for "externhost".
+
+	  ASTERISK-18232 #close
+	  Reported by: Jacek Kowalski
+	  Tested by: Alexander Traud
+	  patches:
+	   changes.patch submitted by Alessandro Crespi
+
+	  Change-Id: If68eedbeff65bd1c1d8a9ed921c02ba464b32dac
+
+2016-09-20 09:42 +0000 [d425971009]  George Joseph <gjoseph@digium.com>
+
+	* chan_sip:  Address runaway when realtime peers subscribe to mailboxes
+
+	  Users upgrading from asterisk 13.5 to a later version and who use
+	  realtime with peers that have mailboxes were experiencing runaway
+	  situations that manifested as a continuous stream of taskprocessor
+	  congestion errors, memory leaks and an unresponsive chan_sip.
+
+	  A related issue was that setting rtcachefriends=no NEVER worked in
+	  asterisk 13 (since the move to stasis).  In 13.5 and earlier, when a
+	  peer tried to register, all of the stasis threads would block and
+	  chan_sip would again become unresponsive.  After 13.5, the runaway
+	  would happen.
+
+	  There were a number of causes...
+	  * mwi_event_cb was (indirectly) calling build_peer even though calls to
+	    mwi_event_cb are often caused by build_peer.
+	  * In an effort to prevent chan_sip from being unloaded while messages
+	    were still in flight, destroy_mailboxes was calling
+	    stasis_unsubscribe_and_join but in some cases waited forever for the
+	    final message.
+	  * add_peer_mailboxes wasn't properly marking the existing mailboxes
+	    on a peer as "keep" so build_peer would always delete them all.
+	  * add_peer_mwi_subs was unsubscribing existing mailbox subscriptions
+	    then just creating them again.
+
+	  All of this was causing a flood of subscribes and unsubscribes on
+	  multiple threads all for the same peer and mailbox.
+
+	  Fixes...
+	  * add_peer_mailboxes now marks mailboxes correctly and build_peer only
+	    deletes the ones that really are no longer needed by the peer.
+	  * add_peer_mwi_subs now only adds subscriptions marked as "new" instead
+	    of unsubscribing and resubscribing everything.  It also adds the peer
+	    object's address to the mailbox instead of its name to the subscription
+	    userdata so mwi_event_cb doesn't have to call build_peer.
+
+	  With these changes, with rtcachefriends=yes (the most common setting),
+	  there are no leaks, locks, loops or crashes at shutdown.
+
+	  rtcachefriends=no still causes leaks but at least it doesn't lock, loop
+	  or crash.  Since making rtcachefriends=no work wasnt in scope for this
+	  issue, further work will have to be deferred to a separate patch.
+
+	  Side fixes...
+	   * The ast_lock_track structure had a member named "thread" which gdb
+	     doesn't like since it conflicts with it's "thread" command.  That
+	     member was renamed to "thread_id".
+
+	  ASTERISK-25468 #close
+
+	  Change-Id: I07519ef7f092629e1e844f855abd279d6475cdd0
+
+2016-09-22 01:40 +0000 [18a8ca06eb]  Aaron An <anjb@ti-net.com.cn>
+
+	* channels/chan_pjsip: fix HANGUPCAUSE function bug.
+
+	  HANGUPCAUSE not return 'SIP 200 Ok' when dialed channel answered.
+	  This patch change the call order of ast_queue_control_data
+	  and ast_queue_control in chan_pjsip_incoming_response.
+
+	  ASTERISK-26396 #close
+	  Reported by: AaronAn
+	  Tested by: AaronAn
+
+	  Change-Id: Ide2d31723d8d425961e985de7de625694580be61
+
+2016-09-21 14:24 +0000 [a805d779e8]  Joshua Colp <jcolp@digium.com>
+
+	* core: Ensure presencestate subtype and message are NULL.
+
+	  When retrieving presence state information there is no
+	  guarantee that the subtype and message passed in are
+	  set to NULL. This change ensures they are.
+
+	  ASTERISK-26397 #close
+
+	  Change-Id: If38cd730e409e9a9b6eb9adef6591d15a9e61f86
+
+2016-09-21 10:48 +0000 [077caf566e]  Joshua Colp <jcolp@digium.com>
+
+	* res_odbc: Make pooling option deprecation notice more useful.
+
+	  This changes the notice for the deprecation of the old
+	  pooling options to point to the new option for doing
+	  pooling. This gives a clearer direction as to what to
+	  look into.
+
+	  ASTERISK-26389 #close
+
+	  Change-Id: I2ca9cdfdcd75aec170a7db9d5ff69a4cd25b7c10
+
+2016-09-21 08:46 +0000 [78b6190a11]  Joshua Colp <jcolp@digium.com>
+
+	* odbc: Remove options that are no longer applicable.
+
+	  The pooling, shared_connection, limit, and idlecheck options
+	  are no longer used in res_odbc.
+
+	  ASTERISK-26389
+
+	  Change-Id: I2fde7b467d01f9d1c82cc0a339bb4f7e1dd6bbe6
+
+2016-08-16 15:21 +0000 [923edf2596]  Corey Farrell <git@cfware.com>
+
+	* logger: Simplify ast_callid handling code.
+
+	  Routines responsible for managing ast_callid's are overly complicated.
+	  This is left-over code from when ast_callid was an AO2 object.  Now that
+	  it is an integer the code can be reduced.
+
+	  ast_callid handler code no longer prints it's own error message upon failure
+	  to allocate threadstorage as ast_calloc would have already printed a
+	  message.  Debug messages that were printed when TEST_FRAMEWORK was
+	  enabled have been also been removed.
+
+	  Change-Id: I65a768a78dc6cf3cfa071e97f33ce3dce280258e
+
+2016-09-20 15:17 +0000 [5cb905a227]  Corey Farrell <git@cfware.com>
+
+	* core: Fix LOW_MEMORY missing symbol ast_pbx_uuid_get.
+
+	  Move the function outside the conditional block that excludes
+	  LOW_MEMORY.
+
+	  ASTERISK-26273 #close
+
+	  Change-Id: Ic290fa128222c410c3531107e30efacabc8493b4
+
+2016-09-20 09:22 +0000 [00f1d05d34]  Corey Farrell <git@cfware.com>
+
+	* logger: Always enable verbose for console channel.
+
+	  Previous versions of Asterisk did not require verbose to be specified in
+	  logger.conf for the console channel, if it was requested by command line
+	  or asterisk.conf it just worked.  This change causes Asterisk to always
+	  enable verbose in the console channel level mask.  Verbose is displayed
+	  on consoles if requested by command line, option_verbose or 'core set
+	  verbose'.
+
+	  This also delays initialization of the logger until after threadstorage
+	  is initialized.  Initializing too early can cause messages to be printed
+	  multiple times to the console (stdout).
+
+	  ASTERISK-26391 #close
+
+	  Change-Id: I52187d67c2fcb3efd5561bf04b3e5e23e5ee8a04
+
+2016-09-20 10:16 +0000 [74f562a8e2]  Corey Farrell <git@cfware.com>
+
+	* logger: Fix default console settings.
+
+	  When logger.conf is missing or invalid we should be printing notices,
+	  warnings and errors to the console.  The logmask was incorrectly
+	  calculated.
+
+	  Change-Id: Ibaa9465a8682854bc1a5e9ba07079bea1bfb6bb3
+
+2016-09-19 14:21 +0000 [0bc9912739]  Walter Doekes <walter+github@wjd.nu>
+
+	* asterisk.c: Non-root users also get the astcanary after core restart.
+
+	  Without this change, a 'core restart' would kill the astcanary forever
+	  if you're not running as root. Both with and without this patch, the
+	  scheduling priority was still SCHED_RR after restart.
+
+	  Additionally, the astcanary is now spawned if you start with high
+	  priority and Asterisk doesn't get a chance to lower it. For example
+	  through: `chrt -r 10 sudo -u asterisk asterisk -c`
+
+	  Also reap killed astcanary processes on core restart.
+
+	  ASTERISK-26352 #close
+
+	  Change-Id: Iacb49f26491a0717084ad46ed96b0bea5f627a55
+
+2016-09-19 09:40 +0000 [bffaf46690]  Walter Doekes <walter+github@wjd.nu>
+
+	* asterisk.c: When astcanary dies on linux, reset priority on all threads.
+
+	  Previously only the canary checking thread itself had its priority set
+	  to SCHED_OTHER. Now all threads are traversed and adjusted.
+
+	  ASTERISK-19867 #close
+	  Reported by: Xavier Hienne
+
+	  Change-Id: Ie0dd02a3ec42f66a78303e9c1aac28f7ed9aae39
+
+2016-09-12 18:00 +0000 [2820b13393]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_config_odbc.c: Fix buffer size limitation creating invalid SQL.
+
+	  Creating ODBC SQL queries resulted in queries too large to fit into the
+	  supplied buffer.  The resulting truncated buffer contained an invalid SQL
+	  query.
+
+	  * Made SQL query generation code use a thread storage buffer that can
+	  increase in size as needed.
+
+	  * Fixed bad multi-line warning messages.
+
+	  ASTERISK-26263 #close
+	  Reported by: Jeppe Ryskov Larsen
+
+	  Change-Id: I23f3cdd43c2dac80bed3ded4dd77d18cb17f21ae
+
+2016-09-14 06:53 +0000 [0376af9519]  Joshua Colp <jcolp@digium.com>
+
+	* rtp: Only accept the first payload for a format in SDP.
+
+	  When receiving an SDP offer with multiple payloads for
+	  the same format we would generate an answer with the first
+	  payload, but during the payload crossover operation
+	  (to set the payloads for receiving) we would remove all
+	  payloads but the last. This would result in incoming
+	  traffic being matched against the wrong format and outgoing
+	  traffic being sent using the wrong payload.
+
+	  This change makes it so that once a format has a payload
+	  number put into the mapping all subsequent ones are ignored.
+	  This ensures there is only ever one payload in the mapping
+	  and that it is the payload placed into the answer SDP.
+
+	  ASTERISK-26365 #close
+
+	  Change-Id: I1e8150860a3518cab36d00b1fab50f9352b64e60
+
+2016-09-14 08:42 +0000 [9d894ee0a1]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_multihomed: Change Contact port to listening port.
+
+	  The res_pjsip_multihomed module determines what interface and transport
+	  a request is going out on and updates the SIP message accordingly with
+	  the address information. This currently incorrectly updates the Contact
+	  header for connectionful protocols to the ephemeral connection port,
+	  instead of the bound address for the listening socket which can actually
+	  accept the connection back. If the remote side attempts to connect back on
+	  the epehemeral port it will fail.
+
+	  This change makes it so the port is updated to the bound port on
+	  connectionful protocols and is maintained on UDP (as there can be
+	  multiple of those).
+
+	  ASTERISK-26374 #close
+
+	  Change-Id: I50f8dab65b9f75117d73ba5f6bbcf6c9871854ab
+
+2016-09-07 14:48 +0000 [47c527df0a]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Prevent SERVFAIL from marking name server bad
+
+	  A name server that returns "Server Failure" is indicating only that
+	  the server couldn't process that particular request.  We should NOT
+	  assume that the name server is incapable of serving other requests.
+
+	  Here's the scenario we've been encountering...
+
+	  * 2 local name servers configured in resolv.conf.
+	  * An OPTIONS request causes a request for A and AAAA records to go out
+	    to both nameservers.
+	  * The A responses both come back successfully resolved.
+	  * Because of an issue at some upstream nameserver, the AAAA responses
+	    for that particular query come back as "SERVFAIL" from both local
+	    name servers.
+	  * Both local servers are marked as bad and no further queries can be
+	    sent until the 60 second ttl expires.  Only previously cached results
+	    can be used.
+	  * In this case, 60 seconds is just enough time for another OPTIONS
+	    request to go out to the same host so the cycle repeats.
+
+	  We could set the bad ttl really low but that also affects REFUSED and
+	  NOTAUTH which probably DO signal a real server issue.  Besides, even
+	  a really low bad ttl would be an issue on a pbx.
+
+	  Although we use our own resolver in 14 and master and don't have this
+	  issue there, Teluu has merged this patch upstream so it's appropriate
+	  to cherry-pick to 14 and master to keep pjproject consistent.
+
+
+	  Change-Id: Ie03ba902288e274aff23f9b9bb2786e1e8be09e0
+
+2016-09-12 07:37 +0000 [d3ddf4b0fd]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* cdr_mysql: fix UTC support
+
+	  * Make 'cdrzone=UTC' work properly.
+	  * Fix the documentation of cdr_mysql.conf: it's cdrzone and not timezone
+
+	  ASTERISK-26359 #close
+
+	  Change-Id: I2a6f67b71bbbe77cac31a34d0bbfb1d67c933778
+
+2016-06-27 14:26 +0000 [07b95f7c65]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* sd_notify (systemd status notifications) support
+
+	  sd_notify() is used to notify systemd of changes to the status of the
+	  process. This allows the systemd daemon to know when the process
+	  finished loading (and thus only start another program after Asterisk has
+	  finished loading).
+
+	  To use this, use a systemd unit with 'Type=notify' for Asterisk.
+
+	  This commit also adds the function ast_sd_notify(), a wrapper around
+	  sd_notify that does nothing if not built with systemd support.
+
+	  Also adds support for libsystemd detection in the configure script.
+
+	  Change-Id: Ied6a59dafd5ef331c5c7ae8f3ccd2dfc94be7811
+
+2016-09-09 06:35 +0000 [bc81765bb4]  Timo Teräs <timo.teras@iki.fi>
+
+	* Fix showing of swap details when sysinfo() is available
+
+	  If sysinfo() is available, but not sysctl() or swapctl() the
+	  printing code for swap buffer sizes is incorrectly omitted.
+	  The above condition happens with musl c-library.
+
+	  Fix #if rule to consider defined(HAVE_SYSINFO). And also
+	  remove the redundant || defined(HAVE_SYSCTL) which was
+	  incorrectly there to start with. Now swap information is
+	  displayed only if an actual libc function to get it is
+	  available.
+
+	  This also fixes warnings previously seen with musl libc:
+
+	     [CC] asterisk.c -> asterisk.o
+	  asterisk.c: In function 'handle_show_sysinfo':
+	  asterisk.c:773:6: warning: variable 'totalswap' set but not used
+	   [-Wunused-but-set-variable]
+	    int totalswap = 0;
+	        ^~~~~~~~~
+	  asterisk.c:770:11: warning: variable 'freeswap' set but not used
+	   [-Wunused-but-set-variable]
+	    uint64_t freeswap = 0;
+	             ^~~~~~~~
+
+	  Change-Id: I1fb21dad8f27e416c60f138c6f2bff03fb626eca
+
+2016-09-14 07:59 +0000 [89764f7ae9]  Joshua Colp <jcolp@digium.com>
+
+	* rtp: Preserve timestamps on video frames.
+
+	  Currently when receiving video over RTP we store only
+	  a calculated samples on the frame. When starting the video
+	  it can take some time for this calculation to actually yield
+	  a value as it requires constant changing timestamps. As well
+	  if a video frame passes over multiple RTP packets this calculation
+	  will fail as the timestamp is the same as the previous RTP
+	  packet and the number of samples calculated will be 0.
+
+	  This change preserves the timestamp on the frame and allows
+	  it to pass through the core. When sending the video this timestamp
+	  is used instead of a new one being calculated.
+
+	  ASTERISK-26367 #close
+
+	  Change-Id: Iba8179fb5c14c9443aee4baf670d2185da3ecfbd
+
+2016-09-14 09:51 +0000 [5f54ac3a80]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_transport_management: Convert time in log message to seconds.
+
+	  ASTERISK-26375 #close
+
+	  Change-Id: I46496af5cae41413e76d44d2068a7431279f09dc
+
+2016-09-13 05:34 +0000 [6ba68b486e]  Steve Davies <steve@one47.co.uk>
+
+	* chan_sip: Fix session timeout on retransmit of non-UDP packets
+
+	  Change-Id I1cd33453c77c56c8e1394cd60a6f17bb61c1d957 Enable Session-Timers for
+	  SIP over TCP (and TLS) also disables SIP retransmits in chan_sip for non-UDP
+	  connections, allowing the TCP layer to handle the retransmits. Unfortunately,
+	  this caused sessions to be terminated with a retransmit timeout becasue it
+	  stopped at the point of the first retrans call.
+
+	  This patch waits for the 64*T1 timer to expire instead.
+
+	  ASTERISK-19968
+
+	  Change-Id: I844f26801aada10bc94e9bebe6e151f0a8443204
+
+2016-09-13 06:08 +0000 [e3487b9360]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Don't assume a request will have any addresses.
+
+	  When performing DNS resolution the failover code present in
+	  res_pjsip currently assumes that a request will always have
+	  at least one viable address. In practice this is not true.
+	  A domain may be used that has no records.
+
+	  The code now checks that at least one address exists on the
+	  request which prevents looping.
+
+	  ASTERISK-26364 #close
+
+	  Change-Id: Ic0761b0264864acd85915c94d878a81624940f4c
+
+2016-09-12 12:25 +0000 [7d7b23f04f]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue: Fix CLI "queue show" and AMI Queues action output truncation.
+
+	  The output of CLI "queue show" and AMI Queues action is truncated and
+	  "failed to extend from 240 to 327" messages are generated if the queue
+	  member and interface names are lengthy.
+
+	  * Increase the string buffer size from 240 to 512 in order to accommodate
+	  for more information fields added to the output since v1.8.
+
+	  ASTERISK-26360 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Id99c03cf5362453b80491a4b3b0434cb67aa966d
+
+2016-09-12 03:28 +0000 [740292e6ae]  Walter Doekes <walter+github@wjd.nu>
+
+	* chan_sip: Allow target refresh (Contact update) on re-INVITE.
+
+	  Previously, the Contact was stored only on initial INVITE and on any
+	  18X and 200. That meant that after re-INVITEs from *us* the Contact
+	  could get updated, but after re-INVITEs from the *peer*, it did not.
+
+	  This changeset fixes this inconsistency, properly allowing target
+	  refreshes through re-INVITES (RFC3261, 12.2).
+
+	  If your strictrtp setting allows it, this change allows you to switch
+	  the source IP of a connected/calling device mid-call with a simple
+	  re-INVITE from the new IP.
+
+	  ASTERISK-26358 #close
+
+	  Change-Id: Ibb8512054ab27c8c3d2514022568fde943bf2435
+
+2016-08-31 15:22 +0000 [82ec58aa91]  Richard Mudgett <rmudgett@digium.com>
+
+	* sip_to_pjsip.py: Map legacy_useroption_parsing.
+
+	  Map the sip.conf general section legacy_useroption_parsing to the
+	  new pjsip.conf global ignore_uri_user_options.
+
+	  ASTERISK-26316
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I78108a31995db19d41f4e1a07b3324692c5363fc
+
+2016-08-29 18:08 +0000 [ba362822f3]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add ignore_uri_user_options option.
+
+	  This implements the chan_sip legacy_useroption_parsing option but with a
+	  better name.
+
+	  * Made the caller-id number and redirecting number strings obtained from
+	  incoming SIP URI user fields always truncated at the first semicolon.
+	  People don't care about anything after the semicolon showing up on their
+	  displays even though the RFC allows the semicolon.
+
+	  ASTERISK-26316 #close
+	  Reported by: Kevin Harwell
+
+	  Change-Id: Ib42b0e940dd34d84c7b14bc2e90d1ba392624f62
+
+2016-09-09 06:26 +0000 [56caf5402c]  Walter Doekes <walter+github@wjd.nu>
+
+	* contrib: Let safe_asterisk script continue without /dev/tty9.
+
+	  If you use the safe_asterisk script, it uses hardcoded defaults before
+	  running configurable values from /etc/asterisk/startup.d. The hardcoded
+	  default has TTY=9. Some containerized environments don't have such a
+	  TTY, and safe_asterisk would stop.
+
+	  The custom configuration from /etc/asterisk/startup.d/* isn't read until
+	  after it stopped, so changing TTY in a custom config did not help.
+
+	  This changeset changes safe_asterisk to continue if the TTY setting was
+	  untouched and /dev/tty9 and /dev/vc/9 aren't found.
+
+	  Change-Id: I2c7cdba549b77f418a0af4cb1227e8e6fe4148fc
+
+2016-09-09 05:39 +0000 [901e612739]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Only invoke unidentified endpoint logic when unidentified.
+
+	  The code was incorrectly invoking the unidentified logic when
+	  an endpoint had actually been identified, causing log messages
+	  to be output.
+
+	  ASTERISK-26349 #close
+
+	  Change-Id: Id8104fc9e3d138d5e8b6f6977ecc08765fd17d4f
+
+2016-08-29 22:26 +0000 [2a50c29101]  Aaron An <anjb@ti-net.com.cn>
+
+	* res/res_pjsip: Add preferred_codec_only config to pjsip endpoint.
+
+	  This patch add config to pjsip by endpoint.
+	  ;preferred_codec_only=yes
+	  ; Respond to a SIP invite with the single most preferred codec
+	  ; rather than advertising all joint codec capabilities. This
+	  ; limits the other side's codec choice to exactly what we prefer.
+
+	  ASTERISK-26317 #close
+	  Reported by: AaronAn
+	  Tested by: AaronAn
+
+	  Change-Id: Iad04dc55055403bbf5ec050997aee2dadc4f0762
+
+2016-08-16 15:34 +0000 [28b2aeba0b]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Do not crash on ACKs from unknown endpoints.
+
+	  The endpoint identification PJSIP module is intended to identify which
+	  endpoint an incoming request is from. If an endpoint is not identified,
+	  then an artificial endpoint is used in its place when proceeding.
+
+	  The problem is that the ACK request type is an exception to the rule.
+	  The artificial endpoint is not used when processing an ACK. This results
+	  in the possibility of having a NULL endpoint being used further on.
+
+	  The reason ACK is an exception is an attempt not to spam security logs
+	  with unidentified requests. Presumably, you've already logged the
+	  unidentified request on the preceeding INVITE.
+
+	  Up until Asterisk 13.10, retrieving a NULL endpoint in this fashion
+	  didn't cause an issue. A new change in 13.10 added endpoint ACL checking
+	  shortly after endpoint identification. Because we are accessing a NULL
+	  endpoint, this ACL check resulted in a crash.
+
+	  The fix here is to be sure to retrieve the artificial endpoint for all
+	  request types. ACKs still do not generate unidentified request security
+	  events.
+
+	  ASTERISK-26264 #close
+	  Reported by nappsoft
+
+	  AST-2016-006
+
+	  Change-Id: Ie0c795ae2d72273decb972dd74b6a1489fb6b703
+
+2016-08-23 06:35 +0000 [82a3d659dc]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip: Don't allocate new RTP instances on top of old ones.
+
+	  In some scenarios dialog_initialize_rtp can be called multiple times on
+	  the same dialog.  This can cause RTP instances to be leaked along with
+	  multiple file descriptors for each instance.
+
+	  This change makes it so the existing RTP instances are destroyed and
+	  not overwritten, stopping the memory leak.
+
+	  ASTERISK-26272 #close
+	  patches:
+	    ASTERISK-26272-13.patch submitted by Corey Farrell (license 5909)
+
+	  Change-Id: Id529de1184c68f2f4d254ab41a1f458dafdb5f73
+
+2016-09-06 11:46 +0000 [f369dbb705]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_messaging.c: Misc cleanups and fixes.
+
+	  * Eliminated RAII_VAR in get_outbound_endpoint().
+
+	  * Simplify update_to() coding.  However, this function can only be a NoOp
+	  because the To string can only be a URI and not a name-address formatted
+	  string.
+
+	  * Simplify update_from() coding.  Also fixed a code path modifying the
+	  from string when the caller could still want to use the original string.
+
+	  * Fixed msg_data_create() incompletely removing the "pjsip:" to then add
+	  back the "sip:" string if needed.  The code didn't handle the "pjsip:sip:"
+	  case because it left the colon after pjsip in the string.
+
+	  Change-Id: I68a09a665f6d4daa9eaa59069045ab69122e28db
+
+2016-09-07 16:00 +0000 [2e5da0c715]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Allow global headers to be overridden.
+
+	  Currently when you add global headers from the dialplan both
+	  the header in the dialplan and the globally configured header
+	  are added to the resulting SIP INVITE. This change makes it
+	  so the headers in the dialplan take precedence and are the
+	  only ones added.
+
+	  Change-Id: I36f864298f38db3632ad503edc11267cb8ffb3ad
+
+2016-08-10 15:14 +0000 [ac02bbd9a0]  Mark Michelson <mmichelson@digium.com>
+
+	* ConfBridge: Make some announcements asynchronous.
+
+	  Confbridge announcements tend to block a channel while they are being
+	  played. In some circumstances, this is warranted since you want that
+	  particular channel not to hear the announcement (Example: "John Doe has
+	  entered the conference"). For others it makes less sense.
+
+	  This change first introduces methods for playing sounds asynchronously
+	  into the conference. This is very similar to how synchronous sounds are
+	  played, except the channel initiating the playback does not wait for the
+	  sound to complete before moving on.
+
+	  Asynchronous announcements are used for two circumstances:
+	  * Sounds played for a user after they have left the bridge
+	  * Sounds that play first to a single user and then the rest of the
+	    conference (if the channel and conference use the same language)
+
+	  ASTERISK-26289 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: Ie486bb3de1646d50894489030326a423e594ab0a
+
+2016-07-19 09:41 +0000 [7a12355dbd]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Allow Preferred sRTP.
+
+	  Following the Encrypt-all-the-things paradigm:
+
+	  The user enters his SIP-URI and password. Thanks to DNS-NAPTR, the phone
+	  determines SIP-over-TLS as preferred transport. In SIP/SDP, the phone starts
+	  the call with a crypto attribute, but not as RTP/sAVP but the RTP/AVP profile
+	  (sRTP is preferred aka optional; not mandatory). If the VoIP server does not
+	  support sRTP and TLS, the phone shows an open padlock icon.
+
+	  This paradigm is supported by several VoIP/SIP clients on default. Some
+	  implementations even cannot be changed to RTP/sAVP. Therefore here, this
+	  change allows Preferred sRTP for ingress. For egress, please, create a dial
+	  plan which starts with RTP/SAVP, and when rejected tries again with RTP/AVP.
+
+	  ASTERISK-20234 #close
+	  Reported by: tootai
+	  Tested by: tootai, Alexander Traud
+	  patches:
+	   srtp_patches.diff submitted by Matt Jordan
+
+	  Change-Id: I42cb779df3a9c7b3dd03a629fb3a296aa4ceb0fd
+
+2016-09-07 05:59 +0000 [baa7dba180]  Joshua Colp <jcolp@digium.com>
+
+	* res_resolver_unbound: Fix config documentation.
+
+	  The code was referencing the config section as 'globals'
+	  instead of 'general'. This change swaps it over to 'general'.
+
+	  Change-Id: I9dfe7788f41c4a6754c77e103880dc1a747de7fe
+
+2016-09-06 15:25 +0000 [e769c19a31]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_stasis_playback: Cancel the entire playlist when a stop occurs
+
+	  Prior to this patch, a stop issued by a delete of a Playback resource
+	  (indicated by the control frame AST_CONTROL_STREAM_STOP) would only stop
+	  the current media URI playing. Subsequent URIs specified by a playback
+	  operation would then proceed on, even though we had just indicated to
+	  the User that the Playback was finished *and* after they had just
+	  'deleted' the resource. Whoops.
+
+	  This patch corrects it by bailing out of the sequence of URIs to play if
+	  one of them is terminated with an AST_CONTROL_STREAM_STOP indication.
+
+	  ASTERISK-26341 #close
+
+	  Change-Id: I2da9ec43545ba46cdfffe287c7e4907eae7fca42
+
+2016-08-01 20:55 +0000 [6caf6bcdad]  George Joseph <gjoseph@digium.com>
+
+	* build: Add download capability for external packages
+
+	  The DPMA and g729a, silk, siren7 and siren14 codecs hosted at
+	  http://downloads.digium.com/pub/telephony/ are now listed in the
+	  "External" sections of the "Resource Modules" and "Codec Translators"
+	  pages in menuselect.  Any that are selected will automatically be
+	  downloaded and installed when "make install" is run.  Their LICENSE and
+	  README (if avaialble) files will be installed to
+	  ASTVARLIBDIR/documentation/thirdparty/<product_name>.
+
+	  Example use with codecs:
+
+	  The codecs/codecs.xml file is a menuselect style xml file that lists
+	  the codecs to be included.  Their support levels are 'external', which
+	  triggers the download and install, and defaultenabled is no.  Also
+	  because codec_g729a is actually in a directory named codec_g729 on the
+	  download server, the newly added 'member_data' element is used to
+	  override the default of the directory name being the package name.  You
+	  can use the 'directory_name' attribute to keep default base URL
+	  (http://downloads.digium.com/pub/telephony/) but use the new directory,
+	  or you use the 'remote_url' attribute to specify a full URL to the
+	  download directory.  In this case, you must still follow the same
+	  subdirectory naming conventions as that used for the packages located
+	  at 'http://downloads.digium.com/pub/telephony'.
+
+	  A new configure option '--with-externals-cache' was added and like
+	  '--with-sounds-cache' it allows the installer to cache tarballs so
+	  they're not downloaded every time.
+
+	  To assist with the download and install process, each external package
+	  now has a manifest.xml file that, among other things, contains a package
+	  version and checksums for each file in the tarball.  The manifest is
+	  saved to both the cache directory and ASTMODDIR and together with the
+	  manifest.xml on the downloads site, tells the install scripts whether
+	  a download and/or update is needed.
+
+	  bash and xmlstarlet are required for downloader operation.  If they're
+	  not installed, the external items in menuselect will be unavailable.
+
+	  Change-Id: Id3dcf1289ffd3cb0bbd7dfab3cafbb87be60323a
+
+2016-08-18 14:45 +0000 [7bb7f7b9d5]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_session: segfault on already disconnected session
+
+	  On heavy loaded system the TCP/TLS incoming calls could be
+	  disconnected by pjproject while these calls are being
+	  processed by asterisk which could use the session's memory pools.
+	  If the session in the disconnected state then the session memory
+	  pools were already freed, so we get segfault.
+
+	  This patch adds a lifetime control on an INVITE session to pjproject.
+	  The lifetime of the session is manipulated by calling
+	  pjsip_inv_add_ref/pjsip_inv_dec_ref.
+	  This patch uses these functions to inform pjproject that the
+	  session is in use.
+
+	  This patch adds check if the session state is not disconnected
+	  and also checks if the memory pool is not NULL.
+
+	  This patch also places tasks 'session_end' and 'session_end_completion'
+	  into session's serializer to avoid race condition.
+
+	  ASTERISK-26291 #close
+
+	  Change-Id: I4d28b1fb3b91f0492a911d110049d670fdc3c8d7
+
+2016-09-06 02:41 +0000 [d80b28560c]  Walter Doekes <walter+github@wjd.nu>
+
+	* chan_sip: Don't refuse calls with "optional crypto"; fall back to RTP.
+
+	  Certain SNOM phones send so-called "optional crypto" in their SDP body.
+	  Regular SRTP setup looks like this:
+
+	      m=audio 64620 RTP/SAVP 8 0 9 99 3 18 4 101
+	      a=crypto:1 AES_CM_128_HMAC_SHA1_32 inline:...
+
+	  SNOM-style "optional crypto" looks like this:
+
+	      m=audio 61438 RTP/AVP 8 0 9 99 3 18 4 101
+	      a=crypto:1 AES_CM_128_HMAC_SHA1_32 inline:...
+
+	  A crypto line is supplied, but the m-line does not have SAVP.
+
+	  When res_srtp.so is *not* loaded, then chan_sip.so treats the optional
+	  crypto as regular RTP, but when res_srtp.so *is* loaded, it refuses the
+	  incoming call with the following message:
+
+	      WARNING: process_sdp: Failed to receive SDP offer/answer with
+	      required SRTP crypto attributes for audio
+
+	  For platforms that want to start providing SRTP this presents a
+	  compatibility problem.
+
+	  This changeset lets chan_sip handle the SDP as if no crypto-line was
+	  supplied: i.e. accept the call as regular RTP, just like it did before
+	  res_srtp was loaded.
+
+	  Now you'll get this informative warning instead:
+
+	      WARNING: Ignoring crypto attribute in SDP because RTP transport is
+	      insecure
+
+	  ASTERISK-23989 #close
+	  Reported by: Olle Johansson
+
+	  Change-Id: I91a15ae05a0296e398d6b65f53bb11afde1d80e2
+
+2016-09-03 16:04 +0000 [730cb3b0b7]  Matt Jordan <mjordan@digium.com>
+
+	* apps/app_dial: Fix crash on non-connect call paths for Privacy/Screening option
+
+	  In any scenario in which the callee is not connected to the caller, the
+	  current code in app_dial will crash due to raising a Dial End Stasis
+	  Message after the callee channel has been hung up. This patch corrects
+	  the error by simply moving the explicit hangup of the callee (peer)
+	  channel until after the dial end message.
+
+	  ASTERISK-25691 #close
+
+	  Change-Id: I816a414014424d0d8c80e2a3cbef13ef8c63798d
+
+2016-09-03 16:02 +0000 [6e1a3b924e]  Matt Jordan <mjordan@digium.com>
+
+	* apps/app_dial: Set the DIALSTATUS to NOANSWER on privacy option 5
+
+	  If the callee selects option '5' using the Dial application's privacy
+	  (P) option, the DIALSTATUS is erroneously set to ANSWER. This option
+	  reflects the callee sending the caller to VoiceMail one time; the call
+	  is definitely *not* ANSWERed in such a scenario. With this patch, the
+	  DIALSTATUS is instead set to NOANSWER, which is the same DIALSTATUS that
+	  is set when the 'send to VoiceMail every time' option is set.
+
+	  ASTERISK-25691
+
+	  Change-Id: Iaf0c9f0fa00545e7366443875e2bb7d9a89a1358
+
+2016-08-30 16:40 +0000 [68c7694abb]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_registrar.c: Reduce stack usage in find_aor_name().
+
+	  Change-Id: I8aebad1fdcf303bd115b59a4b57fbbd5b2267f09
+
+2016-08-29 18:06 +0000 [35ce4d25c7]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_configuration.c: Ignore repeated identify by methods.
+
+	  Change-Id: Ied0c06043d1dfef8fdc9c9a808cf89b118119838
+
+2016-08-30 17:26 +0000 [c1e438fdf7]  Richard Mudgett <rmudgett@digium.com>
+
+	* config_global.c: Comments and a default expression adjustment.
+
+	  Change-Id: Ia6a58f8c73a30da6874b3f94364dce162d6f1ad3
+
+2016-08-31 15:14 +0000 [edcf09e47c]  Richard Mudgett <rmudgett@digium.com>
+
+	* sip_to_pjsip.py: Map canreinvite as directmedia alias.
+
+	  Change-Id: I48b8e150f96a3d2a24d8fc25fbe4f5aff9f4a6b2
+
+2016-08-31 15:37 +0000 [47336a0bdd]  Richard Mudgett <rmudgett@digium.com>
+
+	* sip_to_pjsip.py: Fix typo converting outboundproxy registration.
+
+	  Change-Id: I6f30e5f9fcf8469ba0079fbf884047d54c2c0b15
+
+2016-08-31 15:13 +0000 [dba02575fc]  Richard Mudgett <rmudgett@digium.com>
+
+	* sip_to_pjsip.py: Fix comment typo and tabs.
+
+	  Change-Id: If35174614545727817d329c60ba4456c028941b5
+
+2016-08-31 15:56 +0000 [4aaa27e532]  Richard Mudgett <rmudgett@digium.com>
+
+	* Sample configs: Eliminate false multiline comment block starts.
+
+	  Change-Id: Ie627def9604ae30abd80754f9e6f09874825aec6
+
+2016-09-02 11:36 +0000 [c3b965a2c0]  Richard Mudgett <rmudgett@digium.com>
+
+	* format_cap.c: Fix CLI "core show channeltype Surrogate" crash.
+
+	  * Make ast_format_cap_get_names() NULL tolerant.
+
+	  ASTERISK-26331 #close
+	  Reported by: CGI.NET
+
+	  Change-Id: Id67e93936dc8ec2a33a9d33655843d43b59285a3
+
+2016-08-26 17:22 +0000 [e875e1c12a]  Corey Farrell <git@cfware.com>
+
+	* sorcery: Create function ast_sorcery_lockable_alloc.
+
+	  Create an alternative to ast_sorcery_generic_alloc which uses astobj2
+	  shared locking. Use this new method for the 'struct ast_sip_aor' allocator.
+
+	  Change-Id: I3f62f2ada64b622571950278fbb6ad57395b5d6f
+
+2016-08-18 13:28 +0000 [131baf70d6]  Corey Farrell <git@cfware.com>
+
+	* named_locks: Use ao2_weakproxy to deal with cleanup from container.
+
+	  This allows standard ao2 functions to be used to release references to
+	  an ast_named_lock.  This change can cause less frequent locking of the
+	  global named_locks container.  The container is no longer locked when a
+	  named_lock reference is being release except when this causes the
+	  named_lock to be destroyed.
+
+	  Change-Id: I644e39c6d83a153d71b3fae77ec05599d725e7e6
+
+2016-08-26 13:18 +0000 [0c5b6e9ff5]  Corey Farrell <git@cfware.com>
+
+	* astobj2: Support using a separate object for locking.
+
+	  Create ao2_alloc_with_lockobj function to support shared locking.
+
+	  Change-Id: Iba687eb9843922be7e481e23a32c0700ecf88a80
+
+2016-08-31 12:23 +0000 [48fd4c815c]  Michael Kuron <m.kuron@gmx.de>
+
+	* app_mp3: Use correct buffer size and the same sample rate as the channel
+
+	  Previously, the buffer used for MP3 streamed from HTTP servers had a size of
+	  1 MB. For 8 kHz mono audio at 16 bit resolution, such a buffer covers about 1
+	  minute. Only when the buffer is full does audio start to play.
+	  For MP3 files streamed from a server, that is usually not a big deal as long as
+	  the connection to the server is fast enough to supply that much data within a
+	  second or two. For MP3 live streams however, it takes 1 minute to download 1
+	  minute of audio, so without this change, app_mp3 wasn't really usable for MP3
+	  live streams.
+	  This commit changes the buffer size so that it covers 6 seconds of an MP3 file
+	  streamed from a server and 0.5 seconds of an MP3 live stream. The latter is
+	  identified by the use of a .m3u file extension.
+
+	  app_mp3 so far only supported 8 kHz audio.
+	  Now it always runs at the sample rate of the channel.
+
+	  ASTERISK-26085 #close
+
+	  Change-Id: Id1ee274733cd804a0edecf7450329b72f1235af0
+
+2016-08-31 05:33 +0000 [91993ebaa5]  Jean Aunis <jean.aunis@prescom.fr>
+
+	* resource_channels.c: add hangup reason "answered_elsewhere".
+
+	  In ARI, the channels API allows to hangup a channel with a hangup reason.
+	  This commit adds a new reason "answered_elsewhere".
+	  When using a SIP channel, this will eventually allow Asterisk to add a proper
+	  "Reason" header to a CANCEL message.
+
+	  ASTERISK-26321
+
+	  Change-Id: Ia97675bd4acd6a7f58eb467953dfb94559f6583d
+
+2016-08-26 10:39 +0000 [faf9bdebb7]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: qualify/unqualify added/deleted realtime endpoints
+
+	  If the PJSIP endpoint's AOR with the permanent contact
+	  was deleted from the realtime storage the res_pjsip module
+	  continues trying to qualify this contact.
+	  The error 'Unable to find an endpoint to qualify contact'
+	  appeares every 'qualify_frequency' seconds.
+	  This patch deletes this contact in this case.
+
+	  The PJSIP endpoint's AOR with the permanent contact
+	  is never qualified if it is added to realtime storage
+	  after asterisk started.
+	  This patch adds qualifying for the AOR's permanent contacts
+	  on the first handling of this AOR.
+
+	  ASTERISK-26319 #close
+
+	  Change-Id: Ib93dded9121edb113076903d1aa95402f799f8fe
+
+2016-08-22 17:08 +0000 [c98a047ee6]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Default endpoints to the "offline" status.
+
+	  A recent change attempted to optimize startup by not updating contact
+	  status. Instead, code responsible for qualifying contacts updates the
+	  status as it becomes known. The code even accounts for contacts/AORs
+	  that are not set to be qualified.
+
+	  The problem, though, is when there are no contacts associated with an
+	  endpoint. A common case is when an endpoint is set to register its
+	  contacts but has not done so yet. In this case, prior to registration,
+	  the endpoint's device state will appear to be "not in use" and hints
+	  associated with that device will appear to be "idle". In actuality, the
+	  device state and hint should both appear as "unavailable". The reason
+	  for the failure is that the optimization change made all persistent
+	  endpoint states set to "unknown".
+
+	  The fix here is to change the hard-coded "unknown" to be "offline"
+	  instead. The default state will be offline until the qualifying code
+	  determines that the contact is actually online. This way, if there are
+	  no contacts at all, then the state stays as offline, and device state
+	  and hints appear correctly.
+
+	  ASTERISK-26269 #close
+	  Reported by nappsoft
+
+	  Change-Id: Ie99b84169393983453076f5e9c0d35ff313a456a
+
+2016-08-29 07:07 +0000 [5e0758575c]  Etienne Lessard <elessard@proformatique.com>
+
+	* pbx.c: Prevent infinite recursion in manager_show_dialplan_helper.
+
+	  Previously, if context A was including context B and context B was including
+	  context A, i.e. if there was a circular dependency between contexts, then
+	  calling manager_show_dialplan_helper could lead to an infinite recursion,
+	  resulting in a crash.
+
+	  This commit applies the same solution as the one implemented in the
+	  show_dialplan_helper function. The manager_show_dialplan_helper and
+	  show_dialplan_helper functions contain lots of code in common, but the former
+	  was missing the "infinite recursion avoidance" code.
+
+	  ASTERISK-26226 #close
+
+	  Change-Id: I1aea85133c21787226f4f8442253a93000aa0897
+
+2016-08-25 07:06 +0000 [c21e6764f1]  Joshua Colp <jcolp@digium.com>
+
+	* app_queue: Ensure member is removed from pending when hanging up.
+
+	  When dialing channels it is possible that they may not ever
+	  leave the not in use state (Local channels in particular) by
+	  the time we cancel them. If this occurs but we know they were
+	  dialed we explicitly remove them from the pending members
+	  container so that subsequent call attempts occur.
+
+	  ASTERISK-26299 #close
+
+	  Change-Id: I6ad0d17c36480c92cebf840626228ce3f7e4bd65
+
+2016-08-26 14:34 +0000 [a7487e9261]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Disable srtp use by pjmedia
+
+	  The reason for the disable is that while Asterisk works fine with older
+	  libsrtp versions, newer versions of pjproject won't compile with them.
+	  Debian 6 for instance, has libsrtp 1.4.4 which is older than what
+	  pjproject is expecting.
+
+	  We don't use most of pjmedia but we DO use it for SDP negotiation.
+	  Luckily disabling srtp in pjmedia doesn't interfere with it's ability
+	  to negitiate a secure channel.  The proper crypto attributes are
+	  negotiated in both directions.
+
+	  ASTERISK-26279 #close
+
+	  Change-Id: Id25a92cdf3df97a26c53cffae65b6b82de33c8e2
+
+2016-08-26 08:41 +0000 [858fa5eb2c]  Alexander Traud <pabstraud@compuserve.com>
+
+	* channel: No hung-up on failing security requirements.
+
+	  In your Diaplan, if you specify
+	   same => n,Set(CHANNEL(secure_bridge_media)=1)
+	   same => n,Set(CHANNEL(secure_bridge_signaling)=1)
+	  only the SIP channel driver chan_sip supports this. All other channels drivers
+	  like res_pjsip fail. In case of failure, the original sRTP source code released
+	  the whole channel, even if not hung-up, yet. This change does not release the
+	  channel but instead hangs-up the channel.
+
+	  ASTERISK-26306
+
+	  Change-Id: I0489f0cb660fab6673b0db8af027d116e70a66db
+
+2016-08-20 09:04 +0000 [f35501b8c9]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Migrate IPv4/IPv6 (Dual Stack) configurations.
+
+	  When using the migration script sip_to_pjsip.py, and your sip.conf is
+	  configured with bindaddr=::, two transports are written to pjsip.conf, one for
+	  0.0.0.0 (IPv4) and one for [::] (IPv6). That way, PJProject listens on the IPv4
+	  and IPv6 wildcards; a IPv4/IPv6 Dual Stack configuration on a single interface
+	  like in chan_sip.
+
+	  Furthermore, the script internal functions "build_host" and "split_hostport"
+	  did not parse Literal IPv6 addresses as expected (like [::1]:5060). This change
+	  makes sure, even such addresses are parsed correctly.
+
+	  ASTERISK-26309
+
+	  Change-Id: Ia4799a0f80fc30c0550fc373efc207c3330aeb48
+
+2016-08-04 20:11 +0000 [ea929d766d]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Cache global config options.
+
+	  We may check a global config option hundreds of times a second or more.
+	  Asking sorcery for the global configuration from the config files backend
+	  involves several allocations and container traversals.  Using realtime
+	  without a memory cache is a lot worse because you have to lookup in the
+	  realtime database each time to reconstitute the sorcery object.  With a
+	  memory cache for realtime, there is about the same amount of overhead as
+	  for config files.  Either way, it is still fairly expensive to access the
+	  sorcery object that much.
+
+	  * Cache the global config options so we can access them faster.  You must
+	  now always perform a res_pjsip reload to change the global options.
+
+	  Change-Id: Ice16c7a4cbca4614da344aaea21a072b86263ef7
+
+2016-08-23 11:02 +0000 [5eb6cb969f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax: Fix deadlock in ast_channel_get_t38_state().
+
+	  ast_channel_get_t38_state() calls ast_channel_queryoption() with
+	  AST_OPTION_T38_STATE.  If the passed in channel is a local channel then a
+	  deadlock can happen if a channel lock is held when called.
+
+	  * Made ast_channel_get_t38_state() callers not hold a channel lock before
+	  calling.
+
+	  * Update ast_channel_get_t38_state() doxygen to note that no channel locks
+	  can be held when calling the function.
+
+	  ASTERISK-26203 #close
+	  Reported by: Etienne Lessard
+
+	  ASTERISK-24822 #close
+	  Reported by: David Brillert
+
+	  ASTERISK-22732 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I49fd76fa9af628b4198009b5c0b82c8b03681214
+
+2016-08-23 10:39 +0000 [277a2d667a]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax: Fix deadlock setting FAXMODE channel variable.
+
+	  ASTERISK-25980 added the FAXMODE channel variable to res_fax.c.
+	  Unfortunately, it also introduced a deadlock potential because
+	  set_channel_variables() which sets FAXMODE can be called during a
+	  masquerade.  The ast_channel_get_t38_state() which gets the value used to
+	  set FAXMODE cannot be called with the channel locked.  As a result, local
+	  channels can deadlock because of how they must acquire the locks necessary
+	  to operate.
+
+	  The intent of FAXMODE is for dialplan to know how a fax was transferred
+	  after the fax completes.  However, the previous patch sets FAXMODE to the
+	  channel's current T.38 state AFTER the fax has completed and where T.38
+	  may have already disconnected.
+
+	  * Set FAXMODE based upon T.38 negotiations exchanged either with the fax
+	  applications or the fax framehooks.
+
+	  ASTERISK-26203
+	  Reported by: Etienne Lessard
+
+	  ASTERISK-24822
+	  Reported by: David Brillert
+
+	  ASTERISK-22732
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Id525747254b64c1efe8b1b5973d52ff9719c2ae1
+
+2016-08-22 12:31 +0000 [edca14c8a5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax.c: Fix deadlock in fax_gateway_indicate_t38().
+
+	  fax_gateway_indicate_t38() calls ast_indicate_data() which cannot be
+	  called with any channel locks already held.  A deadlock can happen if the
+	  function is operating on a local channel.
+
+	  * Made fax_gateway_indicate_t38() unlock the channel before calling
+	  ast_indicate_data() since fax_gateway_indicate_t38() is always called with
+	  the channel locked.
+
+	  * Made fax_gateway_indicate_t38() return void since nothing cared about
+	  its return value.
+
+	  ASTERISK-26203
+	  Reported by: Etienne Lessard
+
+	  ASTERISK-24822
+	  Reported by: David Brillert
+
+	  ASTERISK-22732
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I701ff2d26c5fc23e0d5a48a3fd98759a9fd09407
+
+2016-08-23 11:16 +0000 [141cd42880]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax.c: Add chan locked precondition comments.
+
+	  Change-Id: Ic10ae434536bbf7fb7055d6ab36cc50b8748a4e7
+
+2016-08-23 10:42 +0000 [b86771d1bf]  Richard Mudgett <rmudgett@digium.com>
+
+	* ast_framehook_detach() must be called with the channel locked.
+
+	  The framehook container could become corrupted if the channel lock is not
+	  held before calling.
+
+	  Change-Id: If0a1c7ba0484ed3a191106a7516526b905952584
+
+2016-08-22 15:01 +0000 [5744f434f0]  Richard Mudgett <rmudgett@digium.com>
+
+	* ast_framehook_attach() must be called with the channel locked.
+
+	  The framehook container could become corrupted if the channel lock is not
+	  held before calling.
+
+	  Change-Id: I1a6b957a1f7b899eb29a186915f8cccab886a438
+
+2016-08-17 02:51 +0000 [93b7533d74]  chris de rock <chris@derock.de>
+
+	* app_macro: Consider '~~s~~' as a macro start extension.
+
+	  As described in issue ASTERISK-26282 the AEL parser creates macros with
+	  extension '~~s~~'.  app_macro searches only for extension 's' so the
+	  created extension cannot be found.  with this patch app_macro searches for
+	  both extensions and performs the right extension.
+
+	  ASTERISK-26282 #close
+
+	  Change-Id: I939aa2a694148cc1054dd75ec0c47c47f47c90fb
+
+2016-08-24 04:44 +0000 [d2e03c252d]  Eugene <varnavruz@gmail.com>
+
+	* chan_iax2: Set plaintext auth to deprecated as per ASTERISK-22820
+
+	  Starting from draft 2 of RFC 5456 (October 23, 2006) plaintext auth
+	  is not supported in IAX2 protocol. Please refer to section 8.6.13 of
+	  RFC 5456.
+
+	  But plaintext auth is still supported by Asterisk implementation of IAX2.
+	  This support should be dropped.
+
+	  Patch, based on asterisk-dev discussion, adds deprecation warning on
+	  startup if 'auth' is set to 'plaintext', changes default values of
+	  'auth' from 'md5, plaintext' to 'md5'.
+
+	  Patch is safe in terms of backwards compatibility, will work even if
+	  remote peers have auth=plaintext and we have defaults.
+
+	  auth=plaintext setting will remain deprecated in Asterisk 14 and 15,
+	  and IAX2 plaintext support will be removed in Asterisk 16.
+
+	  ASTERISK-22820 #close
+
+	  Change-Id: I5d2f3830cb57645604818f87518916e8a5c317bf
+
+2016-08-24 14:42 +0000 [e40aa40aca]  George Joseph <gjoseph@digium.com>
+
+	* res_rtp_multicast:  Fix SEGV in ast_multicast_rtp_create_options
+
+	  ast_multicast_rtp_create_options now checks for NULL or empty options
+
+	  Change-Id: Ib845eae46a67a9787e89a87ebd1027344e5e0362
+
+2016-07-19 13:14 +0000 [2e79f52d71]  Alexander Traud <pabstraud@compuserve.com>
+
+	* codecs: Add Codec 2 mode 2400.
+
+	  ASTERISK-26217 #close
+
+	  Change-Id: I1e45d8084683fab5f2b272bf35f4a149cea8b8d6
+
+2016-08-10 15:14 +0000 [ded22c712a]  Mark Michelson <mmichelson@digium.com>
+
+	* ConfBridge: Rework announcer channel methodology
+
+	  NOTE: This patch was submitted earlier and reverted because of a failing
+	  test. The test has been patched so that it adjusts for the changes here,
+	  so this is being resubmitted for review.
+
+	  One feature that confbridge has is the ability to play sounds to all
+	  participants in the conference. Prior to this commit, the algorithm for
+	  this was as follows:
+
+	  * Grab the playback lock
+	  * Push the conference announcer channel into the bridge
+	  * Play back the sound
+	  * Pull the conference announcer channel from the bridge
+	  * Release the playback lock
+
+	  The issue here is that the act of adding the playback channel to the
+	  bridge and removing it for each announcement is expensive. Amongst the
+	  expenses:
+
+	  * The announcer channel is imparted into the bridge, meaning a new
+	    thread is spun up for each playback.
+	  * When the announcer is added or removed from the bridge, it results
+	    in the BRIDGEPEER channel variable being set on all channels in the
+	    bridge. This requires keeping the bridge locked and locking each
+	    individual channel in order to set it.
+	  * There's also just the general overhead of adding the channel and
+	    removing it from the bridge. The bridge potentially has to reconfigure
+	    every single time
+
+	  With this commit, the paradigm for playing back announcements has
+	  shifted.
+
+	  * The announcer channel is now added to the bridge when the conference
+	    is allocated, and it is hung up when the conference is destroyed.
+	  * A taskprocessor is used to queue playbacks onto the announcer channel.
+	    This keeps the behavior from before where playbacks do not overlap.
+	  * The announcer channel is no longer placed into the bridge as
+	    departable. Since we are not constantly removing the channel from
+	    the bridge, it is safe to add the channel using an independent thread
+	    and simply hang the channel up when it is time for the conference to
+	    be destroyed.
+
+	  The use of the taskprocessor for playbacks opens up the interesting
+	  possibility of having asynchronous announcements played. In this commit,
+	  however, the behavior is still exactly the same as it previously was.
+
+	  ASTERISK-26289
+	  Reported by Mark Michelson
+
+	  Change-Id: Ica9fa4907c2f3728cdd1cf0bc564ef4eb40754a0
+
+2016-08-23 05:54 +0000 [065d810d3f]  Joshua Colp <jcolp@digium.com>
+
+	* Revert "ConfBridge: Rework announcer channel methodology"
+
+	  This reverts commit 5aa877305223faab5a1119276a934893ab9dc138.
+
+	  Change-Id: I9ab45776e54a54ecf1bac9ae62d976dec30ef491
+
+2016-08-19 10:21 +0000 [41ee14bfae]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* compilation failed with -Werror=maybe-uninitialized
+
+	  The compilation failed for devmode
+	  --enable DONT_OPTIMIZE
+	  --enable BETTER_BACKTRACES
+	  --enable DO_CRASH
+	  --enable TEST_FRAMEWORK
+
+	  res_pjsip/pjsip_configuration.c: In function dtls_handler:
+	  res_pjsip/pjsip_configuration.c:974:20: error:
+	  back may be used uninitialized in this function [-Werror=maybe-uninitialized]
+	  int size = strlen(front);
+	             ^
+	  cc1: all warnings being treated as errors
+
+	  Change-Id: I7f082ead0312792a577ec7c73015ba64dabca580
+
+2016-08-20 14:51 +0000 [eb0c9c476f]  David M. Lee <dlee@respoke.io>
+
+	* res_odbc_transaction: add dep on generic_odbc
+
+	  When res_odbc_transaction depended on res_odbc, it got the generic_odbc
+	  headers and libs implicitly. Now that it no longer depends on res_odbc,
+	  its dependency on generic_odbc must be explicit.
+
+	  Change-Id: I9db88f7af7388437f49903d3008ba8d4890d5911
+
+2016-08-20 11:18 +0000 [12752c64cc]  Alexander Traud <pabstraud@compuserve.com>
+
+	* pjproject_bundled: Allow IPv4/IPv6 (Dual Stack) configurations.
+
+	  PJProject supports a lot of platforms even Windows, some with different defaults
+	  when it comes to IPv6. In many Linux platforms like Ubuntu 16.04 LTS,
+	  "/proc/sys/net/ipv6/bindv6only" is set to 0 (false). Different than in Windows.
+
+	  Because of this, if configured with just an IPv6 address/transport, PJProject
+	  listens to both IPv4 and IPv6. However, this is not supported by the PJProject
+	  team. As consequence, you end-up with IPv4-mapped IPv6 addresses in SDP,
+	  incompatible with IPv4-only clients. Technically, you end-up with an IPv6-only
+	  server which accepts incoming connections on IPv4.
+
+	  If you try to configure two transports, one with IPv4 and one with IPv6 on the
+	  same interface, as expected by the PJProject team, the IPv4 transport is not
+	  able to bind because the IPv6 transport listens to both already.
+
+	  One solution would be to change "/proc/sys/net/ipv6/bindv6only" system-wide.
+	  Then, you are able to configure two transports, one for each IP version on the
+	  same interface. That way, you get a server which works with IPv4 clients and
+	  IPv6 clients at the same time over the same interface.
+
+	  Here, this change sets this parameter directly within PJProject to match the
+	  expectations of the PJProject team in any case. This allows IPv4/IPv6 Dual Stack
+	  servers out of the box like in chan_sip. This change was accepted by the
+	  PJProject team as <http://trac.pjsip.org/repos/changeset/5403> and is expected
+	  to arrive in the next version, PJProject 2.6.0. Until then, this change is
+	  incorporated in the bundled PJProject of Asterisk.
+
+	  ASTERISK-26309
+
+	  Change-Id: I3335d8718f79f4b2feae91b5b005a3ce684a63ae
+
+2016-08-19 18:19 +0000 [55ccdf93c3]  Corey Farrell <git@cfware.com>
+
+	* Fix checks for allocation debugging.
+
+	  MALLOC_DEBUG should not be used to check if debugging is actually
+	  enabled, __AST_DEBUG_MALLOC should be used instead.  MALLOC_DEBUG only
+	  indicates that debugging is requested, __AST_DEBUG_MALLOC indicates it
+	  is active.
+
+	  Change-Id: I3ce9cdb6ec91b74ee1302941328462231be1ea53
+
+2016-08-19 14:09 +0000 [8061d9f66f]  Corey Farrell <git@cfware.com>
+
+	* Fix naming mismatch of allocator functions.
+
+	  Allocator functions that take file/line/func parameters are prefixed
+	  with single-underscore when MALLOC_DEBUG is not defined,
+	  double-underscore when it is defined.  This change updates all
+	  allocators that accept file/line/func to have the same prototype in
+	  either ABI mode.  The parameter order of __ast_vasprintf and
+	  __ast_asprintf in utils.h have been changed to match that of astmm.h.
+
+	  End-use allocator macro's have been removed from astmm.h and moved to an
+	  unconditional part of utils.h.
+
+	  Change-Id: I823bb6ce2b5675b3a4735948f10a3b420e9a023a
+
+2016-08-17 08:10 +0000 [c1b6a79686]  Torrey Searle <torrey@voxbone.com>
+
+	* res_ari: Add http prefix to generated docs
+
+	  updated the uri handler to include the url prefix of the http server
+	  this enables res_ari to add it to the uris when generating docs
+
+	  Change-Id: I279335a2625261a8492206c37219698f42591c2e
+	  (cherry picked from commit 6f448f32fe9b7379e2630fab7b06205f901f2ded)
+
+2016-08-19 03:59 +0000 [02a82f758e]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Add cert_file.
+
+	  When using the migration script sip_to_pjsip.py, cert_file was not migrated to
+	  pjsip.conf. A previous change regarding this contained a copy/paste error.
+
+	  ASTERISK-22374
+
+	  Change-Id: I0fa72e9412117d53b4284fc6b83fa5b2b95ba03b
+
+2016-08-18 09:21 +0000 [1a9555f036]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip.conf: tlsclientmethod is using sslv23 as default.
+
+	  When 'tlsclientmethod' is not specified in sip.conf, chan_sip uses the OpenSSL
+	  SSLv23_method. This was documented incorrectly in the file sip.conf.sample.
+
+	  SSLv23_method got its name in the 90s. Today, with OpenSSL 1.0.2, this method
+	  enables (just) the secure TLSv1.0 and TLSv1.2. Or stated differently, that
+	  function should have been called 'secure_method' or 'automatic_method' back in
+	  the 90s.
+
+	  Consequently please, specify 'tlsclientmethod=tlsv1' in your sip.conf only if
+	  you face a server which has problems like not falling back to TLSv1.0
+	  automatically.
+
+	  ASTERISK-24425
+
+	  Change-Id: I502ce6146b4504cadfd3973af8d6ec3994f54fa3
+
+2016-08-16 15:57 +0000 [53a2f7dc88]  Kevin Harwell <kharwell@digium.com>
+
+	* res_format_attr_g729: Add annexb=no format parameter to SDPs
+
+	  Historically, Asterisk has always specified annexb=no for the g729 format.
+	  However, when using res_pjsip no format attribute was specified. This patch
+	  makes it so the SDP now contains a format attribute line with annexb=no.
+
+	  Note, that this means only g729a is negotiated. Even for pass through support.
+	  According to rfc7261 the type of annex used (a or b) is dependent upon the
+	  answerer. However, Asterisk being a back to back user agent makes this tricky
+	  to support at this time, thus we only allow annex 'a' for now.
+
+	  ASTERISK-26228 #close
+	  patches:
+	    res_format_attr_g729.c submitted by Jason Parker (license 4993)
+
+	  Change-Id: I76bc20cc0a01af01536e9915afef319c269c22d0
+
+2016-08-18 17:02 +0000 [7ea133f2ab]  Kevin Harwell <kharwell@digium.com>
+
+	* rest-api: Swagger scripts were not replacing format variable in file brief
+
+	  Given resource paths did not have 'json' substituted in for the '{format}'. For
+	  some auto generated documentation/comment strings it resulted in something like
+	  the following:
+
+	  "... REST handler for /api-docs/sounds.{format}"
+
+	  This patch makes sure the resource api's path is properly substituted.
+
+	  ASTERISK-25472 #close
+
+	  Change-Id: Ie3e950a35db4043e284019d6c9061f3b03922e23
+
+2016-08-18 15:15 +0000 [c7ffd6111d]  George Joseph <gjoseph@digium.com>
+
+	* res_odbc:  Correct the dependency relationship with res_odbc_transaction
+
+	  The MODULEINFO dependencies between these 2 modules was reversed.
+	  res_odbc should depend on res_odbc_transaction, not the other way
+	  around.
+
+	  ASTERISK-25984 #close
+
+	  Change-Id: Ifcfbb49c0b51cf6640a5446d47cd6c48caf1331f
+
+2016-08-18 12:04 +0000 [966527249e]  Kevin Harwell <kharwell@digium.com>
+
+	* sip_to_pjsip: Set correct tls transport method
+
+	  A recent update had a copy/paste error where the unused variable 'val' was
+	  being passed to the set_value function instead of the 'method' value itself.
+
+	  This patch passes in the right variable.
+
+	  ASTERISK-22374
+
+	  Change-Id: I895b7b3779ce4442bc58b8ec40d59dd29bb43f06
+
+2016-08-10 15:14 +0000 [5aa8773052]  Mark Michelson <mmichelson@digium.com>
+
+	* ConfBridge: Rework announcer channel methodology
+
+	  One feature that confbridge has is the ability to play sounds to all
+	  participants in the conference. Prior to this commit, the algorithm for
+	  this was as follows:
+
+	  * Grab the playback lock
+	  * Push the conference announcer channel into the bridge
+	  * Play back the sound
+	  * Pull the conference announcer channel from the bridge
+	  * Release the playback lock
+
+	  The issue here is that the act of adding the playback channel to the
+	  bridge and removing it for each announcement is expensive. Amongst the
+	  expenses:
+
+	  * The announcer channel is imparted into the bridge, meaning a new
+	    thread is spun up for each playback.
+	  * When the announcer is added or removed from the bridge, it results
+	    in the BRIDGEPEER channel variable being set on all channels in the
+	    bridge. This requires keeping the bridge locked and locking each
+	    individual channel in order to set it.
+	  * There's also just the general overhead of adding the channel and
+	    removing it from the bridge. The bridge potentially has to reconfigure
+	    every single time
+
+	  With this commit, the paradigm for playing back announcements has
+	  shifted.
+
+	  * The announcer channel is now added to the bridge when the conference
+	    is allocated, and it is hung up when the conference is destroyed.
+	  * A taskprocessor is used to queue playbacks onto the announcer channel.
+	    This keeps the behavior from before where playbacks do not overlap.
+	  * The announcer channel is no longer placed into the bridge as
+	    departable. Since we are not constantly removing the channel from
+	    the bridge, it is safe to add the channel using an independent thread
+	    and simply hang the channel up when it is time for the conference to
+	    be destroyed.
+
+	  The use of the taskprocessor for playbacks opens up the interesting
+	  possibility of having asynchronous announcements played. In this commit,
+	  however, the behavior is still exactly the same as it previously was.
+
+	  ASTERISK-26289
+	  Reported by Mark Michelson
+
+	  Change-Id: Ic5cd2c4b98a1eaa1715eb7a5b35d62f1a76d78a5
+
+2016-08-18 08:19 +0000 [e55d1e47aa]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Map the TLS method correctly.
+
+	  When using the migration script sip_to_pjsip.py and tlsclientmethod is not set
+	  in sip.conf, the default value of chan_sip (sslv23) is copied to pjsip.conf, to
+	  overwrite the default of the PJProject (tlsv1). This makes sure, res_pjsip is
+	  offering/using not just TLSv1.0 but TLSv1.2 as well.
+
+	  ASTERISK-22374
+
+	  Change-Id: Ie530a3dae9926ae14f3920a21be1e2edb15bda4f
+
+2016-08-18 08:17 +0000 [da14c439a3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Add compactheaders, timerb, timert1, and useragent.
+
+	  When using the migration script sip_to_pjsip.py, no section of type=system or
+	  type=general were created. Therefore the keys compactheaders, timerb, timert1,
+	  and useragent were not migrated to pjsip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: I318a453843227ea36bf130d392d4abd7bd26b5a1
+
+2016-08-18 08:16 +0000 [675721a7ab]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Map (session-)timers correctly.
+
+	  When using the migration script sip_to_pjsip.py, session-timers=accept and
+	  session-timers=refuse were mapped to wrong values.
+
+	  ASTERISK-22374
+
+	  Change-Id: Ie4e90d5f6a29aff07837b7fe5bc8aea5fb6fc092
+
+2016-08-18 08:15 +0000 [acc5237e91]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Write username even without authname.
+
+	  When using the migration script sip_to_pjsip.py, now the (mandatory) username is
+	  written to pjsip.conf, even if there was no (optional) authname in the register
+	  string in sip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: Ie53e1997104cd2674821688b8a8247249f5e156f
+
+2016-08-18 08:14 +0000 [3eb02235f5]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Parse register even with transport.
+
+	  When using the migration script sip_to_pjsip.py and the register string
+	  started with a transport in sip.conf - like tls://... - register was not parsed
+	  correctly and therefore not migrated correctly to pjsip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: I44c12104eea2bd8558ada6d25d77edfecd92edd2
+
+2016-08-18 08:13 +0000 [9907e2b1c1]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Write local_net, contact_acl, contact_deny, and contact_permit.
+
+	  When using the migration script sip_to_pjsip.py, those keys got missing. These
+	  keys might appear several times and the function "merge_value" tried to collect
+	  those. However, because these keys have different names in sip.conf and
+	  pjsip.conf, "merge_value" was not able to find the new key name in sip.conf.
+	  This change lets "merge_value" search with the old key name in sip.conf and
+	  write with the new key name in pjsip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: Ie53c5278ae6f1cb8fa7e96c5289877d46981d9d2
+
+2016-08-18 08:11 +0000 [c0e0075718]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Map externhost/ip to Transports.
+
+	  When using the migration script sip_to_pjsip.py, the externhost or externip of
+	  sip.conf were erroneously written to Endpoints instead to Transports.
+
+	  ASTERISK-22374
+
+	  Change-Id: I2c5873386cfc388899fa9cf2368639dd12f1b8e4
+
+2016-08-18 08:04 +0000 [a937c2ccb1]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Add defaultexpiry, maxexpiry, and minexpiry.
+
+	  When using the migration script sip_to_pjsip.py, defaultexpiry, maxexpiry, and
+	  minexpiry were not migrated to pjsip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: I007fbf543dcadc96fc3ed71c54da502bcb209b7b
+
+2016-08-18 08:03 +0000 [163cc2d68f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Write media_encryption.
+
+	  When using the migration script sip_to_pjsip.py, encryption=yes got missing and
+	  media_encryption=sdes was not written to pjsip.conf, because of a typo.
+
+	  ASTERISK-22374
+
+	  Change-Id: I0fc3e55dc512a57603ae0fef41baacccf2a35c05
+
+2016-08-18 08:02 +0000 [d8b5970749]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Write cos and tos.
+
+	  When using the migration script sip_to_pjsip.py, both tos_sip and cos_sip got
+	  missed, because of a typo. Therefore, cos and tos were not written to
+	  pjsip.conf. Furthermore, that revealed a misuse of an internal function, caused
+	  by a copy-and-paste error.
+
+	  ASTERISK-22374
+
+	  Change-Id: Id245ebadf70ab9776eb280c026288540af3af5c2
+
+2016-08-18 07:55 +0000 [38491401b5]  Alexander Traud <pabstraud@compuserve.com>
+
+	* sip_to_pjsip: Add cert_file and ca_list_path.
+
+	  When using the migration script sip_to_pjsip.py, cert_file and ca_list_path were
+	  not migrated to pjsip.conf.
+
+	  ASTERISK-22374
+
+	  Change-Id: I4612877d190b7f86a48698cefbf5c4db6c265825
+
+2016-08-16 15:36 +0000 [534063fd67]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip:  Add contact_user to endpoint
+
+	  contact_user, when specified on an endpoint, will override the user
+	  portion of the Contact header on outgoing requests.
+
+	  Change-Id: Icd4ebfda2f2e44d3ac749d0b4066630e988407d4
+
+2016-08-17 14:13 +0000 [0b4fa65532]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Fix unbound srv failover tests.
+
+	  Commit 1b666549f33d69dc080b212bf92126f3bc3a18b2 broke the srv failover
+	  functionality if a TCP connection gets disconnected.  Under these
+	  conditions, session_inv_on_state_changed() gets a
+	  PJSIP_EVENT_TRANSPORT_ERROR and restarts the INVITE transaction on a new
+	  transport.  Unfortunately, session_inv_on_tsx_state_changed() also gets
+	  the same PJSIP_EVENT_TRANSPORT_ERROR event and unconditionally terminates
+	  the session.
+
+	  * Made session_inv_on_tsx_state_changed() complete terminating the session
+	  on PJSIP_EVENT_TRANSPORT_ERROR only if the session state is still
+	  PJSIP_INV_STATE_DISCONNECTED.
+
+	  ASTERISK-26305 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: If736e766b5c55b970fa38ca6c8a885caf27b897d
+
+2016-08-11 12:10 +0000 [046069011b]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* followme: initialize all config items on reload
+
+	  Some configuration directives were not initialized on reload, and hence
+	  were not reset to default if they were removed from followme.conf.
+
+	  ASTERISK-26288 #close
+
+	  Change-Id: Ief829e16374ad1e0ecfd63e6ee4923b5a1d1c150
+
+2016-08-17 06:12 +0000 [57f4e4428a]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Detect ca_list_path capabilities in external PJProject.
+
+	  Since Asterisk 13.8, pj_ssl_cert_load_from_files2 got detected only in the
+	  bundled PJProject but not in an external PJProject. Therefore, ca_list_path
+	  could not be used in pjsip.conf. With this change, pj_ssl_cert_load_from_files2
+	  is detected again to enable ca_list_path again.
+
+	  ASTERISK-26303 #close
+
+	  Change-Id: I4a4a0cdc5cdff33730911fb4cfc0498c069043d0
+
+2016-08-16 12:24 +0000 [a5c0cf4922]  George Joseph <gjoseph@digium.com>
+
+	* ari:  Add documentation that path parameters are case-sensitive
+
+	  Added to api.wiki.mustache so that the generated object pages
+	  have the notation in the table header as well as under each method
+	  that has path parameters.
+
+	  ASTERISK-25492 #close
+
+	  Change-Id: I36c46c6dc0c9ac350470394a999a1b19ef3fcdaf
+
+2016-08-15 15:29 +0000 [824a4e84d1]  Corey Farrell <git@cfware.com>
+
+	* Refactor usage pattern of xmldoc info tag.
+
+	  This updates func_channel.c and main/message.c to use a generic xpointer
+	  include instead of including info from each channel driver.  Now the
+	  name attribute of info is CHANNEL or CHANNEL_EXAMPLES to be included in
+	  documentation for func_channel.  Setting the name attribute of info to
+	  MessageToInfo or MessageFromInfo causes it to be included in the
+	  MessageSend application and AMI action.
+
+	  Change-Id: I89fd8276a3250824241a618009714267d3a8d1ea
+
+2016-06-15 17:10 +0000 [957df73301]  Evgeniy Tsybra <cjack@yandex.ru>
+
+	* chan_sip: Fix lastrtprx always updated
+
+	  Packets are read regulary, when there is no data in buffer fr->frametype
+	  is AST_FRAME_NULL. There was no check of frametype and lastrtprx always 
+	  updated and, therefore, rtptimeout did not work at all.
+
+	  ASTERISK-25270 #close
+
+	  Change-Id: If3b5ca0dbb822582a86eb7d01dcae4e83448c41d
+
+2016-08-10 14:41 +0000 [e85adbd947]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* core: Entity ID is not set or invalid
+
+	  The Exchanging Device and Mailbox States could not working
+	  if the Entity ID (EID) is not set manually and can't be obtained
+	  from ethernet interface.
+
+	  This patch replaces debug message to warning
+	  and addes missing description about option 'entityid' to
+	  asterisk.conf.sample.
+
+	  With this patch the asterisk also:
+	  (1) decline loading the modules which won't work without EID:
+	      res_corosync and res_pjsip_publish_asterisk.
+	  (2) warn if EID is empty on loading next modules:
+	      pbx_dundi, res_xmpp
+
+	  Starting with v197 systemd/udev will automatically assign "predictable"
+	  names for all local Ethernet interfaces.
+	  This patch also addes some new ethernet prefixes "eno" and "ens".
+
+	  ASTERISK-26164 #close
+
+	  Change-Id: I72d712f1ad5b6f64571bb179c5cb12461e7c58c6
+
+2016-08-04 20:00 +0000 [13450c80ce]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_config.c: Cleanup ao2 container usage idioms.
+
+	  Change-Id: Iad24b335fb121a2bc7f1d048ab7420569edcba5a
+
+2016-08-04 15:57 +0000 [d526aa5cbe]  Richard Mudgett <rmudgett@digium.com>
+
+	* sorcery.c: Minor optimizations.
+
+	  * Remove some unused parameters from internal functions:
+	  sorcery_wizard_create()
+	  sorcery_wizard_update()
+	  sorcery_wizard_delete()
+
+	  * Created the struct sorcery_observer_invocation ao2 object without a lock
+	  since it is not needed in sorcery_observer_invocation_alloc().
+
+	  * Cleanup generic ao2 container sorcery object id hash, sort, and cmp
+	  functions.
+
+	  Change-Id: Iff71d75f52bc1b8cee955456838c149faaa4f92e
+
+2016-08-01 11:04 +0000 [45e143576f]  Richard Mudgett <rmudgett@digium.com>
+
+	* sorcery.c: Tweak some container declaration formatting.
+
+	  * Tweak sorcery_object_type_alloc() formatting.
+	  * Tweak ast_sorcery_init() formatting.
+
+	  Change-Id: Ib02430023f15268cd7a2ea53f2c331213e4d3944
+
+2016-08-11 23:30 +0000 [eca3d2698a]  Corey Farrell <git@cfware.com>
+
+	* pbx.c: Additional fixes to ast_context_remove_extension_callerid2.
+
+	  Do not check registrar of the first extension head.  We should only check
+	  the registrar when we match the priority.
+
+	  Additionally fix a couple calls to strcmp which used the input callerid
+	  instead of the clean version ex.cidmatch.
+
+	  ASTERISK-26233
+
+	  Change-Id: I17ea6881a18f40840ae9c1f5394aab1fbb3769f1
+
+2016-08-13 22:02 +0000 [9202ca34a8]  Matt Jordan <mjordan@digium.com>
+
+	* app_dial: Improve documentation
+
+	  * Add some helpful <literal> and other embedded paragraph tags
+
+	  * Document some of the lesser known channel variables set by Dial
+
+	  * Add examples for some common Dial uses, along with some more
+	    challenging but useful options
+
+	  Change-Id: Ib2fb9301e8e044d14fbb2815ec64161f19bbfbc1
+
+2016-08-13 20:16 +0000 [e9fe08ea37]  Matt Jordan <mjordan@digium.com>
+
+	* manager: Add <see-also> tags to relate interrelated events/actions together
+
+	  Change-Id: Idbac539205aa732bf786c4f765577d8e9ff28ba4
+
+2016-08-13 20:15 +0000 [a93cd39ac1]  Matt Jordan <mjordan@digium.com>
+
+	* manager: Add <see-also> tags to relate Bridge related events,actions, and apps
+
+	  Change-Id: I67e6b79fa3102e494b5fe6cc7510472249080e85
+
+2016-08-13 20:14 +0000 [d8a7594ffd]  Matt Jordan <mjordan@digium.com>
+
+	* manager: Add <see-also> tags to relate AoC events and actions
+
+	  Change-Id: Iea89a36222712148c1775c05ed0ad1049d67a70e
+
+2016-08-13 20:13 +0000 [243f0cf99a]  Matt Jordan <mjordan@digium.com>
+
+	* manager: Add <see-also> tags to relate UserEvent actions/apps/events
+
+	  Change-Id: I80f8a981f62f50e74609c69c49edcaca6c95efa4
+
+2016-08-12 15:53 +0000 [3269cf4c17]  Matt Jordan <mjordan@digium.com>
+
+	* res_agi: Improve documentation
+
+	  * Groups of AGI commands that have similar functionality now reference
+	    each other, and all reference the AGI application for ease of wiki
+	    reference.
+
+	  * The documentation for the AGI application has been improved, in
+	    particular noting the various AGI types and how they are invoked.
+
+	  * A warning message has been added to DeadAGI, noting that it is
+	    deprecated.
+
+	  Change-Id: I479ccdee8a7393f01b18692c3d4ab7e6bdd1875d
+
+2016-08-12 13:53 +0000 [a19f4affe8]  Matt Jordan <mjordan@digium.com>
+
+	* manager: Add <see-also> links between related events
+
+	  This patch adds some see-also references between related AMI events. It
+	  focuses primarily on those events that are guaranteed to come in pairs,
+	  such as DTMFBegin/DTMFEnd, as well as those that occur during the life
+	  cycle of an Asterisk channel, such as Newchannel/Hangup.
+
+	  Change-Id: Iaab600477052018d0f8c03d0c624c0856e9ff1f3
+
+2016-08-12 11:15 +0000 [ddab42e296]  Matt Jordan <mjordan@digium.com>
+
+	* func_channel: Reorganize documentation
+
+	  * Following the example of the PJSIP channel driver, the channel
+	    technology specific documentation has been moved to the respective
+	    channel drivers that provide that functionality. This has the benefit
+	    of locating the documentation of items with those modules that provide
+	    it.
+
+	  * Examples of using the CHANNEL function for both standard items as well
+	    as for PJSIP have been added.
+
+	  * The 'max_forwards' standard item has been documented.
+
+	  Change-Id: Ifaa79a232c8ac99cf8da6ef6cc7815d398b1b79b
+
+2016-08-15 07:17 +0000 [922b74169f]  Joshua Colp <jcolp@digium.com>
+
+	* manager: Clarify that dialplan manipulation actions are under system class.
+
+	  ASTERISK-26246 #close
+
+	  Change-Id: Id673b9786389f9d2a87f638ce1a25161f5f31657
+
+2016-08-11 22:12 +0000 [9debe1ca26]  Corey Farrell <git@cfware.com>
+
+	* Run mandatory cleanup when startup fails.
+
+	  Errors during startup result in an exit.  These error branches should be
+	  calling ast_run_atexit(0) to ensure mandatory cleanup is run.
+
+	  ASTERISK-26267 #close
+
+	  Change-Id: If226f2326ae2df7add20040696132214cf2bb680
+
+2016-08-11 11:24 +0000 [d7534e016b]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip_caller_id:  Copy header name to short header name
+
+	  When compact_headers was set, we were sending a zero-length header name
+	  for PAI and RPID because we always forced the short header name length
+	  to 0.  We did this because we cloned the header from "From" and wanted
+	  to clear "f" from the sname.  By cloning however, we bypass pjproject's
+	  automatic logic that sets sname to name if there's no compact form of
+	  the header, which there isn't for PAI and RPID.  So now we force sname
+	  to be the same as name right after we set name.
+
+	  res_pjsip_diversion needed the same treatment for the Diversion header.
+
+	  ASTERISK-26241 #close
+
+	  Change-Id: I633ec139630cd83809aae00336cee4a10077e467
+
+2016-08-11 11:13 +0000 [225fd1003f]  Matt Jordan <mjordan@digium.com>
+
+	* app_queue: Prevent crash when a call is forwarded to an invalid location
+
+	  When a call forward attempt is made from a Queue member, the current
+	  code will hang up the forwarding channel in an off-nominal condition
+	  prior to raising the Stasis events informing the rest of Asterisk that
+	  the call was forwarded. This will result in a slew of dreaded FRACKs,
+	  most likely leading to a crash.
+
+	  This patch modifies the code such that we don't hang up the forwarding
+	  channel even in an off-nominal condition until we've safely raised the
+	  Stasis messages.
+
+	  ASTERISK-25797 #close
+
+	  Change-Id: Ife5abed351691fd79105321636eaa8ea8dcdba38
+
+2016-08-11 12:18 +0000 [aeb859dba9]  George Joseph <gjoseph@digium.com>
+
+	* res_pjsip:  Fail global load if debug or default_from_user are empty
+
+	  If debug was specified in the global configuration but left blank,
+	  the logger would treat it as a wildcard and log all hosts.  If
+	  default_from_user was empty, a crash would result.
+
+	  The global apply handler now checks for empty strings.
+
+	  ASTERISK-26239 #close
+	  ASTERISK-26238 #close
+
+	  Change-Id: Ie75727f5cd5808845d92cc81f5713842fb203336
+
+2016-08-01 15:07 +0000 [2275494e80]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip res_pjsip_mwi: Misc fixes and cleanups.
+
+	  * Eliminated RAII_VAR() usage in
+	  ast_sip_persistent_endpoint_update_state().
+
+	  * Added a missing allocation failure check to
+	  persistent_endpoint_find_or_create().
+
+	  * Made persistent_endpoint_find_or_create() create the new object without
+	  a lock as it isn't needed.
+
+	  * Cleaned up some ao2 container allocation idioms.
+
+	  * Reordered res_pjsip_mwi.c load_module() and unload_module()
+
+	  Change-Id: If8ce88fbd82a0c72a37a2388f74f77237a6a36a8
+
+2016-08-04 18:03 +0000 [d4ffbccef6]  Richard Mudgett <rmudgett@digium.com>
+
+	* location.c: Misc fixes and cleanups.
+
+	  * Eliminated most RAII_VAR() usage.
+
+	  * Added several missing allocation failure checks.
+
+	  * Made ast_sip_for_each_contact() allocate the wrapper ao2 object without
+	  a lock as it is not needed.
+
+	  Change-Id: Ie20913365156c95dd79e5d471cfd25e99ae880bc
+
+2016-08-11 12:01 +0000 [36b2a40533]  George Joseph <gjoseph@digium.com>
+
+	* autohints:  Update CHANGES and extensions.conf.sample
+
+	  Make it clear that we're talking about device state hints and add
+	  an entry to the sample config.
+
+	  Change-Id: Iaef58ffb960191a21b713e8e0b51ce1fcd47e433
+
+2016-08-02 13:53 +0000 [4a5da6c9b4]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Tweak high water checks.
+
+	  * The high water check in ast_taskprocessor_alert_set_levels() would
+	  trigger immediately if the new high water level is zero and the queue was
+	  empty.
+
+	  * The high water check in taskprocessor_push() was off by one.
+
+	  Change-Id: I687729fb4efa6a0ba38ec9c1c133c4d407bc3d5d
+
+2016-08-03 16:24 +0000 [5ba6357be2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Make aor named lock a mutex.
+
+	  The named aor lock was always being locked for writes so a rwlock adds no
+	  benefit and may be slower because rwlocks are biased toward read locking.
+
+	  Change-Id: I8c5c2c780eb30ce5441832257beeb3506fd12b28
+
+2016-07-29 17:41 +0000 [b6e03a5ff3]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Add missing allocation failure check.
+
+	  Change-Id: I932ab2cea845e534d9ff318035b6de39972d3b28
+
+2016-08-11 10:50 +0000 [ac0454f9fa]  David M. Lee <dlee@respoke.io>
+
+	* Fixed compile flags for non-module libs
+
+	  The non-module libs libasteriskssl.dylib and libasteriskpj.dylib have
+	  long been missing the AST_NOT_MODULE compile flag. This was mostly
+	  okay, until a recent fix to improve compiler warnings when the
+	  AST_MODULE_SELF_SYM is missing broke the build on OS X/macOS/whatever
+	  they are calling it these days.
+
+	  Change-Id: I2cb51c890824f001280a5114f2e775f97c163516
+
+2016-08-11 10:50 +0000 [b3c2f1164b]  Kevin Harwell <kharwell@digium.com>
+
+	* alembic: add auth_username to endpoint's identify_by enum
+
+	  A new identify_by option was added recently, auth_username. However, this
+	  setting was not added as an allowable choice in the database enumeration
+	  value.
+
+	  This patch updates the current enumeration, adding in the new setting.
+
+	  ASTERISK-26268 #close
+
+	  Change-Id: Ib4788e8485e4cd40172ec0abbf5810a147ab8bf8
+
+2016-08-08 14:50 +0000 [41aba83ff6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_srtp: Move SDP SRTP code from the core to res_srtp.
+
+	  A patch made to the master branch (Now the 14 branch) inadvertently made
+	  libsrtp a required dependency in order to compile Asterisk.  Rather than
+	  create dummy defines to substitute for the defines supplied by libsrtp
+	  when libsrtp is not available, most of the code in sdp_srtp.c is moved
+	  into res_srtp.c.  This gets more code out of Asterisk's core that isn't
+	  used when SRTP is not available.  This also makes another inadvertent
+	  required dependency on libsrtp by Asterisk's core unlikely.
+
+	  ASTERISK-26253 #close
+	  Reported by: Ben Merrills
+
+	  Change-Id: I0a46cde81501c0405399c2588633ae32706d1ee7
+
+2016-08-06 10:57 +0000 [820879415f]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* pjsip: Fix deadlock with suspend taskprocessor on masquerade
+
+	  If both channels which should be masqueraded
+	  are in the same serializer:
+	  1st channel will be locked waiting condition 'complete'
+	  2nd channel will be locked waiting condition 'suspended'
+
+	  On heavy load system a chance that both channels will be in
+	  the same serializer 'pjsip/distibutor' is very high.
+
+	  To reproduce compile res_pjsip/pjsip_distributor.c with
+	  DISTRIBUTOR_POOL_SIZE=1
+
+	  Steps to reproduce:
+	  1. Party A calls Party B (bridged call 'AB')
+	  2. Party B places Party A on hold
+	  3. Party B calls Voicemail app (non-bridged call 'BV')
+	  4. Party B attended transfers Party A to voicemail using REFER.
+	  5. When asterisk masquerades calls 'AB' and 'BV',
+	     a deadlock is happened.
+
+	  This patch adds a suspension indicator to the taskprocessor.
+	  When a session suspends/unsuspends the serializer
+	  it sets the indicator to the appropriate state.
+	  The session checks the suspension indicator before
+	  suspend the serializer.
+
+	  ASTERISK-26145 #close
+
+	  Change-Id: Iaaebee60013a58c942ba47b1b4930a63e686663b
+
+2016-08-09 12:07 +0000 [d4170df40a]  Kevin Harwell <kharwell@digium.com>
+
+	* alembic/sqlalchemy: auto increment only allowed on a single column
+
+	  The extensions table defined two columns (id and priority) as primary key
+	  autoincrement columns. However only one is allowed when defining the primary
+	  key.
+
+	  This patch removes the autoincrement attribute from the priority column since
+	  it does not need to be as such and really should not have been on there in the
+	  first place.
+
+	  This patch also removes 'context', 'exten', and 'priority' from the primary key
+	  index and creates a new combined unique contraint index on them.
+
+	  ASTERISK-26183 #close
+
+	  Change-Id: Ib9c712c612a4d7ec1edb0dcb77f1bae0905a470b
+
+2016-08-10 11:47 +0000 [8d42ff784d]  George Joseph <gjoseph@digium.com>
+
+	* res_resolver_unbound:  Allow compilation with libunbound version < 1.5
+
+	  libunbound at version 1.4.20 (which CentOS still uses) declared all
+	  of their string function parameters as as 'char *'.  1.4.21 changed
+	  them all to 'const char *'.  Thankfully 1.4.21 also introduced the
+	  UNBOUND_VERSION_MAJOR define so configure now checks for that and
+	  sets HAVE_UNBOUND_CONST_PARAMS.  res_resolver_unbound then checks
+	  that and casts away the 'const' if it's not set.
+
+	  Tested compile and testsuite on CentOS6 (1.4.20), Ubuntu14 (1.4.22) and
+	  Fedora24 (1.5.4).  There are a few failing tests to be addressed though.
+
+	  ASTERISK-26283 #close
+
+	  Change-Id: Ib708b19b706c5d0ba7b7d5473e6df339d9ae4148
+
+2016-08-07 09:58 +0000 [c315460abb]  Matt Jordan <mjordan@digium.com>
+
+	* channels/chan_pjsip: Add PJSIP_SEND_SESSION_REFRESH
+
+	  This patch adds a new PJSIP specific dialplan function,
+	  PJSIP_SEND_SESSION_REFRESH. When invoked on a PJSIP channel, the media
+	  session will be refreshed via either an UPDATE or re-INVITE request.
+	  When used in conjunction with the PJSIP_MEDIA_OFFER dialplan function,
+	  the formats in use on a PJSIP channel can be re-negotiated and changed
+	  dynamically after call setup.
+
+	  ASTERISK-26277 #close
+
+	  Change-Id: Ib98fe09ba889aafe26d58d32f0fd1323f8fd9b1b
+	  (cherry picked from commit eec60dd77394f0519895fc6abce3a6f90f6470f1)
+
+2016-08-09 16:19 +0000 [8fe9f1f7f1]  Mark Michelson <mmichelson@digium.com>
+
+	* res_rtp_asterisk: Cache local RTCP address.
+
+	  When an RTCP packet is sent or received, res_rtp_asterisk generates a
+	  Stasis event that contains the RTCP report as well as the local and
+	  remote addresses that the report pertains to.
+
+	  The addresses are determined using ast_find_ourip(). For the local
+	  address, this will typically result in a lookup of the hostname of the
+	  server, and then a DNS lookup of that hostname. If you do not have the
+	  host in /etc/hosts, then this results in a full DNS lookup, which can
+	  potentially block for some time.
+
+	  This is especially problematic when performing RTCP reads, since those
+	  are done on the same thread responsible for reading and writing media.
+
+	  This patch addresses the issue by performing a lookup of the local
+	  address when RTCP is allocated. We then use this cached local address
+	  for the Stasis events when necessary.
+
+	  ASTERISK-26280 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I3dd61882c2e57036f09f0c390cf38f7c87e9b556
+
+2016-08-08 19:14 +0000 [827457dca0]  Corey Farrell <git@cfware.com>
+
+	* Produce friendly error when AST_MODULE_SELF_SYM is not defined.
+
+	  Modules must define AST_MODULE_SELF_SYM to be used as the name of a
+	  generated function.  This produces a friendly error when it's not
+	  defined.
+
+	  ASTERISK-26278 #close
+
+	  Change-Id: Ib9d35a08104529c516d636771365e02c6e77a45b
+
+2016-08-08 12:53 +0000 [403b63571c]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_mwi: fix unsolicited mwi blocks PJSIP stack
+
+	  The PJSIP taskprocessors could be overflowed on startup
+	  if there are many (thousands) realtime endpoints
+	  configured with unsolicited mwi.
+	  The PJSIP stack could be totally unresponsive for a few minutes
+	  after boot completed.
+
+	  This patch creates a separate PJSIP serializers pool for mwi
+	  and makes unsolicited mwi use serializers from this pool.
+	  This patch also adds 2 new global options to tune taskprocessor
+	  alert levels: 'mwi_tps_queue_high' and 'mwi_tps_queue_low'.
+
+	  This patch also adds new global option 'mwi_disable_initial_unsolicited'
+	  to disable sending unsolicited mwi to all endpoints on startup.
+	  If disabled then unsolicited mwi will start processing
+	  on next endpoint's contact update.
+
+	  ASTERISK-26230 #close
+
+	  Change-Id: I4c8ecb82c249eb887930980a800c9f87f28f861a
+
+2016-08-06 01:37 +0000 [0749f6e6f3]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* res_odbc: Show only when there a fail attempt of connection in CLI
+
+	  When is executed CLI command "odbc show all" every time is show
+	  information about variable last_negative_connect. If not there  a fail
+	  attempt of connection will show date like "1969-12-31 21:00:00".
+
+	  This patch fix there situation for to show only this information when
+	  exists a fail attempt before.
+
+	  Change-Id: I7c058b0be6f7642e922de75ee6b82c7276c9f113
+
+2016-08-05 22:06 +0000 [b156a291af]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* cdr_adaptive_odbc: Fix DNSs mixed config quote quoted_identifiers
+
+	  When haved more than once DNSs config and one of their dont set
+	  quoted_identifiers and before this is with configurated with
+	  quoted_identifiers resulting a truncate statement for a reference null
+	  for quote character identifier.
+
+	  This patch initializes quoted flag before build SQL Query
+
+	  Example config for this bugfix case in cdr_adaptive_odbc.conf file
+
+	  	[first]
+	  	connection=asterisk-server1
+	  	table=cdr
+	  	quoted_identifiers="
+
+	  	[second]
+	  	connection=asterisk-server2
+	  	table=cdr
+
+	  	[third]
+	  	connection=asterisk-server3
+	  	table=cdr
+	  	quoted_identifiers=`
+
+	  Change-Id: Ibd95667b468e10d4a19a2b9d88b9934ec7207e1d
+
+2016-08-05 15:34 +0000 [9042ad40f2]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* app_voicemail: Add taskprocessor alert level options.
+
+	  On heavy loaded system with IMAP or DB storage,
+	  'app_voicemail' taskprocessor queue could reach 500 scheduled tasks.
+	  It could happen when the IMAP or DB server dies or is unreachable.
+	  It could happen on startup when there are many (thousands)
+	  realtime endpoints configured with unsolicited mwi.
+	  If the taskprocessor queue reaches the high water level
+	  then the alert is triggered and pjsip stops processing new requests
+	  until the queue reaches the low water level to clear the alert.
+
+	  This patch adds 2 new 'general' configuration options
+	  to tune taskprocessor alert levels:
+	  'tps_queue_high' - Taskprocessor high water alert trigger level.
+	  'tps_queue_low' - Taskprocessor low water clear alert level
+
+	  ASTERISK-26229 #close
+
+	  Change-Id: I766294fbffedf64053c0d9ac0bedd3109f043ee8
+
+2016-08-04 10:16 +0000 [54869e4823]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_outbound_publish: Use a serializer shutdown group for unload.
+
+	  This change replaces the custom unload process for the outbound
+	  publish module with the common serializer shutdown group.
+
+	  ASTERISK-25217 #close
+
+	  Change-Id: I280a0384d860c486202d87d2d674394cca77ffb6
+
+2016-08-04 10:27 +0000 [e711e57106]  Kevin Harwell <kharwell@digium.com>
+
+	* resource_channels: Sync with ARI stubs
+
+	  This file was out of sync with the current ARI definitions.
+
+	  Change-Id: Ie7cb7d6d3c2eeb9cc9d683ca87b43b117e713d0a
+
+2016-08-03 15:41 +0000 [29b0f733a0]  Corey Farrell <git@cfware.com>
+
+	* Add missing checks during startup.
+
+	  This ensures startup is canceled due to allocation failures from the
+	  following initializations.
+	  * channel.c: ast_channels_init
+	  * config_options.c: aco_init
+
+	  ASTERISK-26265 #close
+
+	  Change-Id: I911ed08fa2a3be35de55903e0225957bcdbe9611
+
+2016-08-03 09:47 +0000 [90b30b21ac]  Joshua Colp <jcolp@digium.com>
+
+	* astconfigparser: Really handle case where line is simply a comment.
+
+	  The regular expression would match causing the code that handled
+	  the line if it was merely a comment to never get executed.
+
+	  Change-Id: I3e4022481037ebcba9905587fe8c764b4ce21819
+
+2016-08-01 11:08 +0000 [73bce50ef8]  Joshua Colp <jcolp@digium.com>
+
+	* sorcery: Use more compatible regex for local expressions.
+
+	  This changes the use of an empty regex for both res_sorcery_config
+	  and res_sorcery_memory to "." instead. This is a more compatible
+	  regular expression which also works on FreeBSD.
+
+	  ASTERISK-26206 #close
+
+	  Change-Id: Ia9166dd176f1597555ba22b6931180d0626c1388
+
+2016-08-02 03:08 +0000 [3ff964c6b6]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_pjsip: SIP/SDP origin (o=) contained square brackets on IP6 transports.
+
+	  ASTERISK-26256 #close
+
+	  Change-Id: I3fd68df561f81fdb8c6c497d465b50c12422f058
+
+2016-08-01 16:13 +0000 [f6276441b1]  George Joseph <gjoseph@digium.com>
+
+	* menuselect:  Add an opaque "member_data" string to the acceptable xml
+
+	  Change-Id: Id5ac43b95c8d7395f3be37f983632169db3d1afe
+
+2016-07-29 13:13 +0000 [1cd79d6ee5]  Mark Michelson <mmichelson@digium.com>
+
+	* Remove SILK payload mappings from Asterisk core.
+
+	  SILK is a bit of a hog when it comes to using up our limited number of
+	  dynamic payload types in the RTP engine. By freeing up four slots, it
+	  allows for other codecs to potentially take the place.
+
+	  Now, codec_silk.so will dynamically use the payload slots in the RTP
+	  engine when it loads.
+
+	  A better fix would be make RTP dynamic payload types actually
+	  dynamic. However, at this stage of Asterisk 14 development, this is a
+	  risky move that would be imprudent.
+
+	  Change-Id: I5774e09408f9a203db189529eabdc0d3f4c1e612
+
+2016-07-29 04:48 +0000 [a7ae48441f]  Joshua Colp <jcolp@digium.com>
+
+	* astconfigparser: Handle case where line is simply a comment.
+
+	  Change-Id: I2dea5815363f4d787d709228a04f33baee383ef5
+
+2016-07-28 14:10 +0000 [89a0a1eb45]  Corey Farrell <git@cfware.com>
+
+	* pbx.c: Fix handling of '-' in extension name and callerid
+
+	  This adds a two strings to ast_exten.  name to go with exten and
+	  cidmatch_display to go with cidmatch.  The new fields contain input used
+	  to add the extension in the first place.  The existing fields now
+	  contain stripped input that excludes insignificant spaces and dashes.
+	  These stripped fields should always be used for comparisons.  The
+	  unstripped fields should normally be used for display, but displaying
+	  stripped values will not cause runtime errors.
+
+	  Note the actual string is only stored twice if it contains dashes.  If
+	  no dashes are found then both 'char *' fields point to the same memory.
+	  So this change has a minimum effect on memory usage.
+
+	  The existing functions ast_get_extension_name and
+	  ast_get_extension_cidmatch return unstripped values as they did before
+	  this change.  Other similar bugs likely still exist where unstripped
+	  extensions are saved outside pbx.c then passed back in.
+
+	  ASTERISK-26233 #close
+
+	  Change-Id: I6cd61ce57acc1570ca6cc14960c4c3b0a9eb837f
+
+2016-07-27 17:17 +0000 [68ebf86e2f]  Richard Mudgett <rmudgett@digium.com>
+
+	* pbx.c: Allow dangerous functions when adding a hint to dialplan.
+
+	  We can allow dangerous functions when adding a hint since altering
+	  dialplan is itself a privileged activity.  Otherwise, we could never
+	  execute dangerous functions.
+
+	  ASTERISK-25996 #close
+	  Reported by: Andrew Nagy
+
+	  Change-Id: I4929ff100ad1200a0198262d069a34f2296e77ba
+
+2016-07-21 10:36 +0000 [b5bc2fdda8]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* pjproject: fixed a few bugs
+
+	  This patch fixes the issue in pjsip_tx_data_dec_ref()
+	  when tx_data_destroy can be called more than once,
+	  and checks if invalid value (e.g. NULL) is passed to.
+
+	  This patch updates array limit checks and docs
+	  in pjsip_evsub_register_pkg() and pjsip_endpt_add_capability().
+
+	  Change-Id: I4c7a132b9664afaecbd6bf5ea4c951e43e273e40
+
+2016-07-17 18:28 +0000 [b4f1c6380e]  George Joseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Update for pjproject 2.5.5
+
+	  Add more --disable-* switches to Makefile.rules including
+	  --disable-opus which was causing bundled pjproject to fail with
+	  "undefined reference" errors in libasteriskpj.
+
+	  Changed PJ_ENABLE_EXTRA_CHECK to 1.
+
+	  Removed 2 obsolete patches and added a new one.
+	  The new one was merged by Teluu on 6/27/2016.
+
+	  ASTERISK-26148 #close
+
+	  Change-Id: Ib8af6c6a9d31f7238ce65b336134c2efdc855063
+
+2016-07-27 10:33 +0000 [feb1a43412]  David M. Lee <dlee@respoke.io>
+
+	* Portably sscanf tv_usec
+
+	  In a timeval, tv_usec is defined as a suseconds_t, which could be
+	  different underlying types on different platforms. Instead of trying to
+	  scanf directly into the timeval, scanf into a long int, then copy that
+	  into the timeval.
+
+	  Change-Id: I29f22d049d3f7746b6c0cc23fbf4293bdaa5eb95
+
+2016-07-27 12:36 +0000 [1d364ac54f]  Kevin Harwell <kharwell@digium.com>
+
+	* rtp_engine: Failed assertion and wrong name given for codec
+
+	  Fixed an assert check that would trigger when the passed in value was negative.
+	  The negative value was being cast to an unsigned value. This resulted in the
+	  check failing.
+
+	  Also fixed another problem when loading formats in the engine. When setting the
+	  mime type the format's name was being passed in instead of the codec's name.
+
+	  Change-Id: I1a201cd419ba4d8e9a40d337e36b6fbe1737192c
+
+2016-07-27 09:56 +0000 [8802e55c26]  David M. Lee <dlee@respoke.io>
+
+	* Replace strdupa with more portable ast_strdupa
+
+	  The strdupa function is a GNU extension, and not widely portable. We
+	  have an ast_strdupa function used within Asterisk which is preferred.
+	  I pulled the definition up from menuselect.c into the menuselect.h
+	  header file so it can be shared across menuselect.
+
+	  Change-Id: I9593c97f78386b47dc1e83201e80cb2f62b36c2e
+
+2016-07-21 22:44 +0000 [737471f131]  Richard Mudgett <rmudgett@digium.com>
+
+	* dsp.c: Add fax and DTMF detection unit tests.
+
+	  * Add fax amplitude and frequency sweep tests.
+	  * Add DTMF amplitude and twist unit tests.
+
+	  Change-Id: I8d77c9a1eec89e440d715f998c928687e870c3f7
+
+2016-07-21 11:56 +0000 [a8cd5d255a]  Richard Mudgett <rmudgett@digium.com>
+
+	* dsp.c: Added descriptive comments to Goertzel calculations.
+
+	  * Added doxygen to describe some struct members and what is going on in
+	  the code.
+
+	  Change-Id: I2ec706a33b52aee42b16dcc356c2bd916a45190d
+
+2016-07-13 13:48 +0000 [6dfb34cf13]  Richard Mudgett <rmudgett@digium.com>
+
+	* dsp.c: Fix incorrect format reference typo.
+
+	  Change-Id: Ia131da3ec29acf385cb43a586a29ecc975eb3896
+
+2016-07-25 21:18 +0000 [327136088e]  Richard Mudgett <rmudgett@digium.com>
+
+	* dsp.c: Correct DTMF twist dsp.conf documentation.
+
+	  Change-Id: Idf97e3a72f1edc5fca58f2fa7b20785922be0cae
+
+2016-07-22 04:43 +0000 [1e7168aee0]  Joshua Colp <jcolp@digium.com>
+
+	* astconfigparser.py: Update with realtime fixes.
+
+	  When configuring SIP URIs in the pjsip.conf file it is
+	  necessary to escape the semicolon so the parser does not
+	  treat it as a comment. This change allows this to work in
+	  the astconfigparser implementation.
+
+	  A secondary bug where some data was lost if a configuration
+	  option included a "=" in its value was also fixed.
+
+	  A bug where sections would be considered equal despite
+	  being different has also been fixed.
+
+	  Change-Id: If229f656ef22050b50e7b34e90c4bffe796431f8
+
+2016-07-21 22:28 +0000 [49461f37b7]  Richard Mudgett <rmudgett@digium.com>
+
+	* dsp.c: Fix erroneous fax tone detection.
+
+	  The Goertzel calculations get less accurate the lower the signal level
+	  being worked with becomes because there is less resolution remaining.
+	  If it is too low we can erroneously detect a tone where none really
+	  exists.  The searched for fax frequencies not only need to be so much
+	  stronger than the background noise they must also be a minimum strength.
+
+	  * Add needed minimum threshold test to tone_detect().
+
+	  * Set TONE_THRESHOLD to allow low volume frequency spread detection.
+
+	  ASTERISK-26237 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I84dbba7f7628fa13720add6a88eae3b129e066fc
+
+2016-07-24 18:27 +0000 [b4c5dcad01]  George Joseph <gjoseph@digium.com>
+
+	* menuselect:  Various menuselect enhancements
+
+	  * Add 'external' as a support level.
+	  * Add ability for module directories to add entries to the menu
+	    by adding members to the <module_prefix>/<module_prefix>.xml file.
+	  * Expand the description field to 3 lines in the ncurses implementation.
+	  * Allow the description field to wrap in the newt implementation.
+	  * Add description field to the gtk implementation.
+
+	  Change-Id: I7f9600a1984a42ce0696db574c1051bc9ad7c808
+
+2016-07-24 16:51 +0000 [9db420c69d]  Joshua Colp <jcolp@digium.com>
+
+	* ari: Update version.
+
+	  New functionality has been added so the version has been
+	  bumped to one over the 13 version.
+
+	  Change-Id: I5d30077f62640c0ac83599b4e9a9b657bf184f69
+
+2016-07-23 08:51 +0000 [8852a4c3db]  George Joseph <gjoseph@digium.com>
+
+	* asterisk.c:  Add auto generation and persistence of UUID
+
+	  Upcoming features will require the generation and persistence
+	  of a UUID.
+
+	  Change-Id: I3ec0062427e133217db6ef496a4216f427c3b92d
+
+2016-07-22 14:44 +0000 [76781a0964]  Mark Michelson <mmichelson@digium.com>
+
+	* Fix sqlalchemy error regarding identifier length.
+
+	  sqlalchemy was complaining:
+
+	  sqlalchemy.exc.IdentifierError: Identifier
+	  'ps_contacts_qualifyfreq_exptime' exceeds maximum length of 30
+	  characters
+
+	  This fixes the problem by changing the index name to be
+	  "ps_contacts_qualifyfreq_exp" instead.
+
+	  ASTERISK-26227 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I0ed784f87504be2a59ee8d3242ef6f625d5ed1a9
+
+2016-07-19 06:16 +0000 [9be69c1636]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Enable Session-Timers for SIP over TCP (and TLS).
+
+	  Asterisk defaults to timers=accept/refresher=uas. In that scenario, only in that
+	  scenario, Sessions-Timers (RFC 4028) had no effect via TCP. This change enables
+	  Session-Timers for SIP over TCP (and for SIP over TLS).
+
+	  However with longer international calls via TCP, the SIP channel might break,
+	  because all hops on the Internet route must stay online (have not a single power
+	  outage, for example). Therefore with Session-Timers enabled (which are enabled
+	  at default), you might see dropped calls. Consequently even with this change,
+	  you might be better-off going for session-timers=refuse in your sip.conf.
+
+	  ASTERISK-19968 #close
+
+	  Change-Id: I1cd33453c77c56c8e1394cd60a6f17bb61c1d957
+
+2016-07-19 13:39 +0000 [8fb807009f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* codecs: Add iLBC 20.
+
+	  Asterisk already supported iLBC 30. This change adds iLBC 20. Now, Asterisk
+	  defaults to iLBC 20 but falls back to iLBC 30, when the remote party requests
+	  this.
+
+	  ASTERISK-26218 #close
+	  ASTERISK-26221 #close
+	  Reported by: Aaron Meriwether
+
+	  Change-Id: I07f523a3aa1338bb5217a1bf69c1eeb92adedffa
+
+2016-07-15 16:16 +0000 [4286a369a1]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Whitespace and comment cleanup.
+
+	  Change-Id: I11139a4a95df34e223ba622aa6227e33ab8f6c38
+
+2016-07-21 22:34 +0000 [68de3a9e51]  Corey Farrell <git@cfware.com>
+
+	* pbx.c: Remove duplicate code.
+
+	  Merge code found in both branches of a conditional in
+	  ast_add_extension2_lockopt.
+
+	  The updated code initializes peer_table and peer_label_table of the
+	  extension before linking it to the context.
+
+	  Change-Id: Ic759e27cdc9906c6877df41d28ee9c5be8f41c20
+
+2016-07-21 16:35 +0000 [15bf6a87dc]  George Joseph <gjoseph@digium.com>
+
+	* Create Asterisk-14:  Update CHANGES and UPGRADE files
+
+	  Change-Id: I35b5f6657670cfa8985796fa1e1fe86ad299efdc
+
+2016-07-21 09:05 +0000 [1b4922466b]  George Joseph <gjoseph@digium.com>
+
+	* chan_sip: Prevent deadlock when issuing "sip show channels"
+
+	  sip_show_channels locks the dialogs container first then locks each
+	  sip_pvt so it can spit out the details.  The rest of sip dialog
+	  processing locks the sip_pvt first then locks the dialogs container
+	  if it needs to.  Both lock in the order they need but deadlocks can
+	  result.  To fix, sip_show_channels and sip_show_channelstats have
+	  been converted to use an iterator rather than ao2_callback.  This way
+	  the container is locked only while getting the next entry and is
+	  unlocked when the callback is called.
+
+	  ASTERISK-23013 #close
+
+	  Change-Id: Id9980419909e811f89484950ed46ef117b9eb990
+
+2016-07-15 19:28 +0000 [a36a174c4b]  Corey Farrell <git@cfware.com>
+
+	* pbx: Create pbx_sw.c for management of 'struct ast_sw'.
+
+	  This changes context switches from a linked list to a vector, makes
+	  'struct ast_sw' opaque to pbx.c.
+
+	  Although ast_walk_context_switches is maintained the procedure is no
+	  longer efficient except for the first call (inc==NULL).  This
+	  functionality is replaced by two new functions implemented by vector
+	  macros.
+	  * ast_context_switches_count (AST_VECTOR_SIZE)
+	  * ast_context_switches_get (AST_VECTOR_GET)
+
+	  As with ast_walk_context_switches callers of these functions are
+	  expected to have locked contexts.  Only a few places in Asterisk walked
+	  the switches, they have been converted to use the new functions.
+
+	  Change-Id: I08deb016df22eee8288eb03de62593e45a1f0998
+
+2016-07-21 10:28 +0000 [81ea024d93]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_pubsub: fixed a bug when pjsip_tx_data_dec_ref is called twice.
+
+	  This patch removed call of pjsip_tx_data_dec_ref in send_notify
+	  if send_request failed.
+	  The pjsip_dlg_send_request deletes the message on error by itself.
+
+	  It seems this patch fixes next issues:
+	  ASTERISK-26199
+	  ASTERISK-26166
+	  ASTERISK-26174
+
+	  Change-Id: I8b05917c93d993f95d604c042ace5f1a5500f59a
+
+2016-07-13 05:24 +0000 [1d2173c7ae]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_srtp: Enable AES-256 and AES-GCM.
+
+	  ASTERISK-26190 #close
+
+	  Change-Id: I11326d80edd656524a51a19450e586c583aa0a0b
+
+2016-07-18 22:46 +0000 [8f6e9ffcc6]  Corey Farrell <git@cfware.com>
+
+	* Add conditional support for noreturn functions.
+
+	  This adds support for tagging functions with the noreturn attribute.
+	  If DO_CRASH is enabled then ast_do_crash never returns.  If AST_DEVMODE
+	  and DO_CRASH are enabled then failed assertions never return.  This can
+	  resolve a large number of false positives with static analyzers.
+
+	  ASTERISK-26220 #close
+
+	  Change-Id: Icfb61e5fe54574eced4c3e88b317244f467ec753
+
+2016-07-19 13:18 +0000 [3d62f317dd]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi.c: Fix deadlock potential in fax redirection.
+
+	  The dahdi_handle_dtmf() and my_handle_dtmf() have the potential to
+	  deadlock if an incoming fax happens during the Playback or similar
+	  application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  ASTERISK-26216 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I9144b84ade5f96690996624ec8a2d40c56af40aa
+
+2016-07-13 18:49 +0000 [db4979fa79]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix deadlock potential in fax redirection.
+
+	  The sip_read() has the potential to deadlock if an incoming fax happens
+	  during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I6d3f5cccd4b77c3aa6ffc1a54c0f6bde61c9278e
+
+2016-07-13 18:48 +0000 [3db468ea9e]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_pjsip.c: Fix deadlock potential in fax redirection.
+
+	  The chan_pjsip_cng_tone_detected() has the potential to deadlock if an
+	  incoming fax happens during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I32aecbb4818af646dc5a619f0dc040e9b1f222e5
+
+2016-07-12 17:33 +0000 [9abbea162c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax.c: Fix deadlock potential in FAXOPT(faxdetect) framehook.
+
+	  The fax_detect_framehook() has the potential to deadlock if an incoming
+	  fax happens during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  * Made only detach the framehook if we detected a fax and not on other
+	  possible frames.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I99da35c26d1cd802626ffb4c1b4eb5b015581b6d
+
+2016-07-12 17:24 +0000 [804fbd9c2b]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax: Fix FAXOPT(faxdetect) timeout option.
+
+	  The fax detection timeout option did not work because basically the wrong
+	  variable was checked in fax_detect_framehook().  As a result, the timer
+	  would timeout immediately and disable fax detection.
+
+	  * Fixed ignoring negative timeout values.  We'd complain and then go right
+	  on using the negative value.
+
+	  * Fixed destroy_faxdetect() in the off-nominal case of an incomplete
+	  object creation.
+
+	  * Added more range checking to FAXOPT(gateway) timeout parameter.
+
+	  ASTERISK-26214 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Idc5e698dfe33572de9840bc68cd9fc043cbad976
+
+2016-07-18 16:16 +0000 [0d1744e132]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi: Add faxdetect_timeout option.
+
+	  The new option allows the channel driver's faxdetect option to timeout on
+	  a call after the specified number of seconds into a call.  The new feature
+	  is disabled if the timeout is set to zero.  The option is disabled by
+	  default.
+
+	  * Don't clear dsp_features after passing them to the dsp code in
+	  my_pri_ss7_open_media().  We should still remember them especially for the
+	  new faxdetect_timeout option.
+
+	  ASTERISK-26214
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Ieffd3fe788788d56282844774365546dce8ac810
+
+2016-07-15 20:44 +0000 [e739888d99]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add fax_detect_timeout endpoint option.
+
+	  The new endpoint option allows the PJSIP channel driver's fax_detect
+	  endpoint option to timeout on a call after the specified number of
+	  seconds into a call.  The new feature is disabled if the timeout is set
+	  to zero.  The option is disabled by default.
+
+	  ASTERISK-26214
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Id5a87375fb2c4f9dc1d4b44c78ec8735ba65453d
+
+2016-07-17 07:43 +0000 [d56fc3b36b]  Alexander Traud <pabstraud@compuserve.com>
+
+	* translate: Enables native Packet-Loss Concealment (PLC) for supporting codecs.
+
+	  ASTERISK-25629 #close
+
+	  Change-Id: I66c0086e6c17764b8141ec60a3e2aaefe088eb78
+
+2016-09-19 14:18 +0000  Asterisk Development Team <asteriskteam@digium.com>
+
+	* asterisk 14.0.0-rc1 Released.
+
+2016-09-19 09:17 +0000 [a23b33576f]  Joshua Colp <jcolp@digium.com>
+
+	* Release summaries: Add summaries for 14.0.0-rc1
+
+2016-09-19 09:17 +0000 [e11354b864]  Joshua Colp <jcolp@digium.com>
+
+	* Release summaries: Remove previous versions
+
+2016-09-19 09:17 +0000 [24fac2271a]  Joshua Colp <jcolp@digium.com>
+
+	* .version: Update for 14.0.0-rc1
+
+2016-09-19 09:17 +0000 [52c101d441]  Joshua Colp <jcolp@digium.com>
+
+	* .lastclean: Update for 14.0.0-rc1
+
+2016-09-19 09:17 +0000 [edae56dc65]  Joshua Colp <jcolp@digium.com>
+
+	* realtime: Add database scripts for 14.0.0-rc1
+
+2016-09-14 09:51 +0000 [205e2ea351]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_transport_management: Convert time in log message to seconds.
+
+	  ASTERISK-26375 #close
+
+	  Change-Id: I46496af5cae41413e76d44d2068a7431279f09dc
+
+2016-09-13 06:08 +0000 [bc085bba24]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Don't assume a request will have any addresses.
+
+	  When performing DNS resolution the failover code present in
+	  res_pjsip currently assumes that a request will always have
+	  at least one viable address. In practice this is not true.
+	  A domain may be used that has no records.
+
+	  The code now checks that at least one address exists on the
+	  request which prevents looping.
+
+	  ASTERISK-26364 #close
+
+	  Change-Id: Ic0761b0264864acd85915c94d878a81624940f4c
+
+2016-09-09 05:39 +0000 [9a800b24ac]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Only invoke unidentified endpoint logic when unidentified.
+
+	  The code was incorrectly invoking the unidentified logic when
+	  an endpoint had actually been identified, causing log messages
+	  to be output.
+
+	  ASTERISK-26349 #close
+
+	  Change-Id: Id8104fc9e3d138d5e8b6f6977ecc08765fd17d4f
+
+2016-08-16 15:34 +0000 [137aa2f13c]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Do not crash on ACKs from unknown endpoints.
+
+	  The endpoint identification PJSIP module is intended to identify which
+	  endpoint an incoming request is from. If an endpoint is not identified,
+	  then an artificial endpoint is used in its place when proceeding.
+
+	  The problem is that the ACK request type is an exception to the rule.
+	  The artificial endpoint is not used when processing an ACK. This results
+	  in the possibility of having a NULL endpoint being used further on.
+
+	  The reason ACK is an exception is an attempt not to spam security logs
+	  with unidentified requests. Presumably, you've already logged the
+	  unidentified request on the preceeding INVITE.
+
+	  Up until Asterisk 13.10, retrieving a NULL endpoint in this fashion
+	  didn't cause an issue. A new change in 13.10 added endpoint ACL checking
+	  shortly after endpoint identification. Because we are accessing a NULL
+	  endpoint, this ACL check resulted in a crash.
+
+	  The fix here is to be sure to retrieve the artificial endpoint for all
+	  request types. ACKs still do not generate unidentified request security
+	  events.
+
+	  ASTERISK-26264 #close
+	  Reported by nappsoft
+
+	  AST-2016-006
+
+	  Change-Id: Ie0c795ae2d72273decb972dd74b6a1489fb6b703
+
+2016-08-23 06:35 +0000 [f877e62cc9]  Corey Farrell <git@cfware.com> (license 5909)
+
+	* chan_sip: Don't allocate new RTP instances on top of old ones.
+
+	  In some scenarios dialog_initialize_rtp can be called multiple times on
+	  the same dialog.  This can cause RTP instances to be leaked along with
+	  multiple file descriptors for each instance.
+
+	  This change makes it so the existing RTP instances are destroyed and
+	  not overwritten, stopping the memory leak.
+
+	  ASTERISK-26272 #close
+	  patches:
+	    ASTERISK-26272-13.patch submitted by Corey Farrell (license 5909)
+
+	  Change-Id: Id529de1184c68f2f4d254ab41a1f458dafdb5f73
+
+2016-09-06 15:25 +0000 [b17ee86148]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_stasis_playback: Cancel the entire playlist when a stop occurs
+
+	  Prior to this patch, a stop issued by a delete of a Playback resource
+	  (indicated by the control frame AST_CONTROL_STREAM_STOP) would only stop
+	  the current media URI playing. Subsequent URIs specified by a playback
+	  operation would then proceed on, even though we had just indicated to
+	  the User that the Playback was finished *and* after they had just
+	  'deleted' the resource. Whoops.
+
+	  This patch corrects it by bailing out of the sequence of URIs to play if
+	  one of them is terminated with an AST_CONTROL_STREAM_STOP indication.
+
+	  ASTERISK-26341 #close
+
+	  Change-Id: I2da9ec43545ba46cdfffe287c7e4907eae7fca42
+
+2016-08-29 12:30 +0000  Asterisk Development Team <asteriskteam@digium.com>
+
+	* asterisk 14.0.0-beta2 Released.
+
+2016-08-29 07:29 +0000 [9cdf44668d]  Joshua Colp <jcolp@digium.com>
+
+	* Release summaries: Add summaries for 14.0.0-beta2
+
+2016-08-29 07:29 +0000 [73d39f2029]  Joshua Colp <jcolp@digium.com>
+
+	* Release summaries: Remove previous versions
+
+2016-08-29 07:29 +0000 [e8a97775ee]  Joshua Colp <jcolp@digium.com>
+
+	* .version: Update for 14.0.0-beta2
+
+2016-08-29 07:29 +0000 [345409825a]  Joshua Colp <jcolp@digium.com>
+
+	* .lastclean: Update for 14.0.0-beta2
+
+2016-08-29 07:29 +0000 [105c1168f7]  Joshua Colp <jcolp@digium.com>
+
+	* realtime: Add database scripts for 14.0.0-beta2
+
+2016-08-29 06:31 +0000 [8927b52634]  Joshua Colp <jcolp@digium.com>
+
+	* alembic: Fix downgrade path.
+
+	  The 3772f8f828da version was referencing a previous version
+	  that did not exist in the 14.0 branch. It has been fixed to
+	  reference the correct previous version.
+
+	  Change-Id: I004d0fcfdfe1d1bb6f01c6dac2b69f6b1f40ae51
+
+2016-08-11 12:18 +0000 [9a95c6dea3]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip:  Fail global load if debug or default_from_user are empty
+
+	  If debug was specified in the global configuration but left blank,
+	  the logger would treat it as a wildcard and log all hosts.  If
+	  default_from_user was empty, a crash would result.
+
+	  The global apply handler now checks for empty strings.
+
+	  ASTERISK-26239 #close
+	  ASTERISK-26238 #close
+
+	  Change-Id: Ie75727f5cd5808845d92cc81f5713842fb203336
+
+2016-08-11 11:24 +0000 [aaee8160bc]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip_caller_id:  Copy header name to short header name
+
+	  When compact_headers was set, we were sending a zero-length header name
+	  for PAI and RPID because we always forced the short header name length
+	  to 0.  We did this because we cloned the header from "From" and wanted
+	  to clear "f" from the sname.  By cloning however, we bypass pjproject's
+	  automatic logic that sets sname to name if there's no compact form of
+	  the header, which there isn't for PAI and RPID.  So now we force sname
+	  to be the same as name right after we set name.
+
+	  res_pjsip_diversion needed the same treatment for the Diversion header.
+
+	  ASTERISK-26241 #close
+
+	  Change-Id: I633ec139630cd83809aae00336cee4a10077e467
+
+2016-08-11 12:01 +0000 [7af0eac02a]  gtjoseph <gjoseph@digium.com>
+
+	* autohints:  Update CHANGES and extensions.conf.sample
+
+	  Make it clear that we're talking about device state hints and add
+	  an entry to the sample config.
+
+	  Change-Id: Iaef58ffb960191a21b713e8e0b51ce1fcd47e433
+
+2016-08-11 10:50 +0000 [ef0bf47bb3]  Kevin Harwell <kharwell@digium.com>
+
+	* alembic: add auth_username to endpoint's identify_by enum
+
+	  A new identify_by option was added recently, auth_username. However, this
+	  setting was not added as an allowable choice in the database enumeration
+	  value.
+
+	  This patch updates the current enumeration, adding in the new setting.
+
+	  ASTERISK-26268 #close
+
+	  Change-Id: Ib4788e8485e4cd40172ec0abbf5810a147ab8bf8
+
+2016-08-08 14:50 +0000 [a1d6b14c40]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_srtp: Move SDP SRTP code from the core to res_srtp.
+
+	  A patch made to the master branch (Now the 14 branch) inadvertently made
+	  libsrtp a required dependency in order to compile Asterisk.  Rather than
+	  create dummy defines to substitute for the defines supplied by libsrtp
+	  when libsrtp is not available, most of the code in sdp_srtp.c is moved
+	  into res_srtp.c.  This gets more code out of Asterisk's core that isn't
+	  used when SRTP is not available.  This also makes another inadvertent
+	  required dependency on libsrtp by Asterisk's core unlikely.
+
+	  ASTERISK-26253 #close
+	  Reported by: Ben Merrills
+
+	  Change-Id: I0a46cde81501c0405399c2588633ae32706d1ee7
+
+2016-08-09 12:07 +0000 [a783e1e60d]  Kevin Harwell <kharwell@digium.com>
+
+	* alembic/sqlalchemy: auto increment only allowed on a single column
+
+	  The extensions table defined two columns (id and priority) as primary key
+	  autoincrement columns. However only one is allowed when defining the primary
+	  key.
+
+	  This patch removes the autoincrement attribute from the priority column since
+	  it does not need to be as such and really should not have been on there in the
+	  first place.
+
+	  This patch also removes 'context', 'exten', and 'priority' from the primary key
+	  index and creates a new combined unique contraint index on them.
+
+	  ASTERISK-26183 #close
+
+	  Change-Id: Ib9c712c612a4d7ec1edb0dcb77f1bae0905a470b
+
+2016-08-10 11:47 +0000 [9c56f798f6]  gtjoseph <gjoseph@digium.com>
+
+	* res_resolver_unbound:  Allow compilation with libunbound version < 1.5
+
+	  libunbound at version 1.4.20 (which CentOS still uses) declared all
+	  of their string function parameters as as 'char *'.  1.4.21 changed
+	  them all to 'const char *'.  Thankfully 1.4.21 also introduced the
+	  UNBOUND_VERSION_MAJOR define so configure now checks for that and
+	  sets HAVE_UNBOUND_CONST_PARAMS.  res_resolver_unbound then checks
+	  that and casts away the 'const' if it's not set.
+
+	  Tested compile and testsuite on CentOS6 (1.4.20), Ubuntu14 (1.4.22) and
+	  Fedora24 (1.5.4).  There are a few failing tests to be addressed though.
+
+	  ASTERISK-26283 #close
+
+	  Change-Id: Ib708b19b706c5d0ba7b7d5473e6df339d9ae4148
+
+2016-08-01 16:13 +0000 [1ad00c1c30]  gtjoseph <gjoseph@digium.com>
+
+	* menuselect:  Add an opaque "member_data" string to the acceptable xml
+
+	  Change-Id: Id5ac43b95c8d7395f3be37f983632169db3d1afe
+
+2016-07-17 18:28 +0000 [815b6f72f8]  gtjoseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Update for pjproject 2.5.5
+
+	  Add more --disable-* switches to Makefile.rules including
+	  --disable-opus which was causing bundled pjproject to fail with
+	  "undefined reference" errors in libasteriskpj.
+
+	  Changed PJ_ENABLE_EXTRA_CHECK to 1.
+
+	  Removed 2 obsolete patches and added a new one.
+	  The new one was merged by Teluu on 6/27/2016.
+
+	  ASTERISK-26148 #close
+
+	  Change-Id: Ib8af6c6a9d31f7238ce65b336134c2efdc855063
+	  (cherry picked from commit 4cf02b5584ce33bb0a64408c27bf20c19bc4ce13)
+
+2016-07-29 13:13 +0000 [c95b611a73]  Mark Michelson <mmichelson@digium.com>
+
+	* Remove SILK payload mappings from Asterisk core.
+
+	  SILK is a bit of a hog when it comes to using up our limited number of
+	  dynamic payload types in the RTP engine. By freeing up four slots, it
+	  allows for other codecs to potentially take the place.
+
+	  Now, codec_silk.so will dynamically use the payload slots in the RTP
+	  engine when it loads.
+
+	  A better fix would be make RTP dynamic payload types actually
+	  dynamic. However, at this stage of Asterisk 14 development, this is a
+	  risky move that would be imprudent.
+
+	  Change-Id: I5774e09408f9a203db189529eabdc0d3f4c1e612
+
+2016-07-27 12:36 +0000 [bc94ccbcdd]  Kevin Harwell <kharwell@digium.com>
+
+	* rtp_engine: Failed assertion and wrong name given for codec
+
+	  Fixed an assert check that would trigger when the passed in value was negative.
+	  The negative value was being cast to an unsigned value. This resulted in the
+	  check failing.
+
+	  Also fixed another problem when loading formats in the engine. When setting the
+	  mime type the format's name was being passed in instead of the codec's name.
+
+	  Change-Id: I1a201cd419ba4d8e9a40d337e36b6fbe1737192c
+
+2016-07-26 23:19 +0000  Asterisk Development Team <asteriskteam@digium.com>
+
+	* asterisk 14.0.0-beta1 Released.
+
+2016-07-26 17:22 +0000 [a7233fbf3e]  Mark Michelson <mmichelson@digium.com>
+
+	* Release summaries: Add summaries for 14.0.0-beta1
+
+2016-07-26 16:24 +0000 [c327430ea0]  Mark Michelson <mmichelson@digium.com>
+
+	* Release summaries: Remove previous versions
+
+2016-07-26 16:24 +0000 [763a18bc9d]  Mark Michelson <mmichelson@digium.com>
+
+	* .version: Update for 14.0.0-beta1
+
+2016-07-26 16:24 +0000 [ce6898bd3c]  Mark Michelson <mmichelson@digium.com>
+
+	* .lastclean: Update for 14.0.0-beta1
+
+2016-07-26 16:24 +0000 [ebc477aa5d]  Mark Michelson <mmichelson@digium.com>
+
+	* realtime: Add database scripts for 14.0.0-beta1
+
+2016-07-26 16:00 +0000 [1838b283aa]  Mark Michelson <mmichelson@digium.com>
+
+	* ChangeLog: Updated for 14.0.0
+
+2016-07-26 15:02 +0000 [f196cf975d]  Mark Michelson <mmichelson@digium.com>
+
+	* Release summaries: Add summaries for 14.0.0
+
+2016-07-26 14:01 +0000 [699a7390eb]  Mark Michelson <mmichelson@digium.com>
+
+	* .version: Update for 14.0.0
+
+2016-07-26 14:01 +0000 [4b17a11d7d]  Mark Michelson <mmichelson@digium.com>
+
+	* .lastclean: Update for 14.0.0
+
+2016-07-26 14:01 +0000 [bb9dcae98c]  Mark Michelson <mmichelson@digium.com>
+
+	* realtime: Add database scripts for 14.0.0
+
+2016-07-24 18:27 +0000 [90f445729d]  gtjoseph <gjoseph@digium.com>
+
+	* menuselect:  Various menuselect enhancements
+
+	  * Add 'external' as a support level.
+	  * Add ability for module directories to add entries to the menu
+	    by adding members to the <module_prefix>/<module_prefix>.xml file.
+	  * Expand the description field to 3 lines in the ncurses implementation.
+	  * Allow the description field to wrap in the newt implementation.
+	  * Add description field to the gtk implementation.
+
+	  Change-Id: I7f9600a1984a42ce0696db574c1051bc9ad7c808
+
+2016-07-24 16:51 +0000 [f75401b1e3]  Joshua Colp <jcolp@digium.com>
+
+	* ari: Update version.
+
+	  New functionality has been added so the version has been
+	  bumped to one over the 13 version.
+
+	  Change-Id: I5d30077f62640c0ac83599b4e9a9b657bf184f69
+
+2016-07-23 08:51 +0000 [58759bd77c]  gtjoseph <gjoseph@digium.com>
+
+	* asterisk.c:  Add auto generation and persistence of UUID
+
+	  Upcoming features will require the generation and persistence
+	  of a UUID.
+
+	  Change-Id: I3ec0062427e133217db6ef496a4216f427c3b92d
+
+2016-07-22 14:44 +0000 [46b4e673ae]  Mark Michelson <mmichelson@digium.com>
+
+	* Fix sqlalchemy error regarding identifier length.
+
+	  sqlalchemy was complaining:
+
+	  sqlalchemy.exc.IdentifierError: Identifier
+	  'ps_contacts_qualifyfreq_exptime' exceeds maximum length of 30
+	  characters
+
+	  This fixes the problem by changing the index name to be
+	  "ps_contacts_qualifyfreq_exp" instead.
+
+	  ASTERISK-26227 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I0ed784f87504be2a59ee8d3242ef6f625d5ed1a9
+
+2016-07-22 07:01 +0000 [633c34c411]  gtjoseph <gjoseph@digium.com>
+
+	* build_tools: Update make_version for 14
+
+	  Also remove svn stuff
+
+	  Change-Id: I95d762f7cbbe5eb01117bde8779515d51a0bb06a
+
+2016-07-19 13:39 +0000 [c82f24f36a]  Alexander Traud <pabstraud@compuserve.com>
+
+	* codecs: Add iLBC 20.
+
+	  Asterisk already supported iLBC 30. This change adds iLBC 20. Now, Asterisk
+	  defaults to iLBC 20 but falls back to iLBC 30, when the remote party requests
+	  this.
+
+	  ASTERISK-26218 #close
+	  ASTERISK-26221 #close
+	  Reported by: Aaron Meriwether
+
+	  Change-Id: I07f523a3aa1338bb5217a1bf69c1eeb92adedffa
+
+2016-07-15 16:16 +0000 [6e2e3915c8]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Whitespace and comment cleanup.
+
+	  Change-Id: I11139a4a95df34e223ba622aa6227e33ab8f6c38
+
+2016-07-19 13:18 +0000 [5efb5b38e8]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi.c: Fix deadlock potential in fax redirection.
+
+	  The dahdi_handle_dtmf() and my_handle_dtmf() have the potential to
+	  deadlock if an incoming fax happens during the Playback or similar
+	  application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  ASTERISK-26216 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I9144b84ade5f96690996624ec8a2d40c56af40aa
+
+2016-07-13 18:49 +0000 [a1d36c89e0]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix deadlock potential in fax redirection.
+
+	  The sip_read() has the potential to deadlock if an incoming fax happens
+	  during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I6d3f5cccd4b77c3aa6ffc1a54c0f6bde61c9278e
+
+2016-07-13 18:48 +0000 [4dfadcb025]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_pjsip.c: Fix deadlock potential in fax redirection.
+
+	  The chan_pjsip_cng_tone_detected() has the potential to deadlock if an
+	  incoming fax happens during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I32aecbb4818af646dc5a619f0dc040e9b1f222e5
+
+2016-07-12 17:33 +0000 [964ae54ecf]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax.c: Fix deadlock potential in FAXOPT(faxdetect) framehook.
+
+	  The fax_detect_framehook() has the potential to deadlock if an incoming
+	  fax happens during the Playback or similar application.
+
+	  * Fixed the potential deadlock by not calling ast_async_goto() with the
+	  channel lock held.
+
+	  * Made always eat the fax detection frame whether there is a fax extension
+	  or not.
+
+	  * Made only detach the framehook if we detected a fax and not on other
+	  possible frames.
+
+	  ASTERISK-26216
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I99da35c26d1cd802626ffb4c1b4eb5b015581b6d
+
+2016-07-12 17:24 +0000 [c3462adeb8]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax: Fix FAXOPT(faxdetect) timeout option.
+
+	  The fax detection timeout option did not work because basically the wrong
+	  variable was checked in fax_detect_framehook().  As a result, the timer
+	  would timeout immediately and disable fax detection.
+
+	  * Fixed ignoring negative timeout values.  We'd complain and then go right
+	  on using the negative value.
+
+	  * Fixed destroy_faxdetect() in the off-nominal case of an incomplete
+	  object creation.
+
+	  * Added more range checking to FAXOPT(gateway) timeout parameter.
+
+	  ASTERISK-26214 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Idc5e698dfe33572de9840bc68cd9fc043cbad976
+
+2016-07-18 16:16 +0000 [c03e27c1c8]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi: Add faxdetect_timeout option.
+
+	  The new option allows the channel driver's faxdetect option to timeout on
+	  a call after the specified number of seconds into a call.  The new feature
+	  is disabled if the timeout is set to zero.  The option is disabled by
+	  default.
+
+	  * Don't clear dsp_features after passing them to the dsp code in
+	  my_pri_ss7_open_media().  We should still remember them especially for the
+	  new faxdetect_timeout option.
+
+	  ASTERISK-26214
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Ieffd3fe788788d56282844774365546dce8ac810
+
+2016-07-15 20:44 +0000 [d11731ac2f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add fax_detect_timeout endpoint option.
+
+	  The new endpoint option allows the PJSIP channel driver's fax_detect
+	  endpoint option to timeout on a call after the specified number of
+	  seconds into a call.  The new feature is disabled if the timeout is set
+	  to zero.  The option is disabled by default.
+
+	  ASTERISK-26214
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Id5a87375fb2c4f9dc1d4b44c78ec8735ba65453d
+
+2016-07-21 10:28 +0000 [56b4112659]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_pubsub: fixed a bug when pjsip_tx_data_dec_ref is called twice.
+
+	  This patch removed call of pjsip_tx_data_dec_ref in send_notify
+	  if send_request failed.
+	  The pjsip_dlg_send_request deletes the message on error by itself.
+
+	  It seems this patch fixes next issues:
+	  ASTERISK-26199
+	  ASTERISK-26166
+	  ASTERISK-26174
+
+	  Change-Id: I8b05917c93d993f95d604c042ace5f1a5500f59a
+
+2016-07-21 09:05 +0000 [52cbdf2393]  gtjoseph <gjoseph@digium.com>
+
+	* chan_sip: Prevent deadlock when issuing "sip show channels"
+
+	  sip_show_channels locks the dialogs container first then locks each
+	  sip_pvt so it can spit out the details.  The rest of sip dialog
+	  processing locks the sip_pvt first then locks the dialogs container
+	  if it needs to.  Both lock in the order they need but deadlocks can
+	  result.  To fix, sip_show_channels and sip_show_channelstats have
+	  been converted to use an iterator rather than ao2_callback.  This way
+	  the container is locked only while getting the next entry and is
+	  unlocked when the callback is called.
+
+	  ASTERISK-23013 #close
+
+	  Change-Id: Id9980419909e811f89484950ed46ef117b9eb990
+
+2016-07-13 05:24 +0000 [2103ad1fec]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_srtp: Enable AES-256 and AES-GCM.
+
+	  ASTERISK-26190 #close
+
+	  Change-Id: I11326d80edd656524a51a19450e586c583aa0a0b
+
+2016-07-18 22:46 +0000 [05cfe1a76e]  Corey Farrell <git@cfware.com>
+
+	* Add conditional support for noreturn functions.
+
+	  This adds support for tagging functions with the noreturn attribute.
+	  If DO_CRASH is enabled then ast_do_crash never returns.  If AST_DEVMODE
+	  and DO_CRASH are enabled then failed assertions never return.  This can
+	  resolve a large number of false positives with static analyzers.
+
+	  ASTERISK-26220 #close
+
+	  Change-Id: Icfb61e5fe54574eced4c3e88b317244f467ec753
+
+2016-07-15 19:28 +0000 [0c88fb460f]  Corey Farrell <git@cfware.com>
+
+	* pbx: Create pbx_sw.c for management of 'struct ast_sw'.
+
+	  This changes context switches from a linked list to a vector, makes
+	  'struct ast_sw' opaque to pbx.c.
+
+	  Although ast_walk_context_switches is maintained the procedure is no
+	  longer efficient except for the first call (inc==NULL).  This
+	  functionality is replaced by two new functions implemented by vector
+	  macros.
+	  * ast_context_switches_count (AST_VECTOR_SIZE)
+	  * ast_context_switches_get (AST_VECTOR_GET)
+
+	  As with ast_walk_context_switches callers of these functions are
+	  expected to have locked contexts.  Only a few places in Asterisk walked
+	  the switches, they have been converted to use the new functions.
+
+	  Change-Id: I08deb016df22eee8288eb03de62593e45a1f0998
+
+2016-07-19 04:48 +0000 [6fca2b3bf0]  Alexander Traud <pabstraud@compuserve.com>
+
+	* Makefile: Retain XML Declaration and DTD in docs.
+
+	  Since Asterisk 12, the documentation got an XML Stylesheet. Because of a typo,
+	  the XML Declaration and DTD were overwritten by this.
+
+	  ASTERISK-26212 #close
+
+	  Change-Id: If5ee4625068042e98ab3fcb22a25e2f15d0c68bd
+
+2016-07-18 18:40 +0000 [cf1188a1be]  Corey Farrell <git@cfware.com>
+
+	* Unit tests: Use AST_TEST_DEFINE in conditional code only.
+
+	  If AST_TEST_DEFINE is not conditional to TEST_FRAMEWORK it produces dead
+	  code.  This places all existing unit tests into a conditional block if
+	  they weren't already.
+
+	  ASTERISK-26211 #close
+
+	  Change-Id: I8ef83ee11cbc991b07b7a37ecb41433e8c734686
+
+2016-07-18 09:22 +0000 [e9daa34261]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_mwi: remove unneeded check on endpoint's contacts.
+
+	  The function create_mwi_subscriptions_for_endpoint checks
+	  if there is active contacts by retrieving aors and contacts.
+
+	  This function is used to create all unsolicited mwi subscriptions
+	  on startup and is used when contact added.
+
+	  In both cases it's not necessary to check if there are contacts.
+	  The contacts are needed when asterisk sends mwi.
+
+	  ASTERISK-26200 #close
+
+	  Change-Id: I98e43bdc97f3c0829951cd9bf5f3c6348c6ac1fa
+
+2016-07-18 05:13 +0000 [cb5e3445be]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_rtp_asterisk: Count a roll-over of the sequence number even on lost packets.
+
+	  With this change, the initial RTP sequence number is randomly chosen not between
+	  0 and 65535 (0xffff) but 0 and 32767 (0x7fff). This assures, the roll-over
+	  counter (ROC) synchronization is not lost for sRTP, when the very first RTP
+	  packets get lost; see http://srtp.sourceforge.net/faq.html#Q6
+
+	  ASTERISK-26207 #close
+
+	  Change-Id: I9a527e3aa3ce8f3becc5131d7ba32b57b5845464
+
+2016-07-18 04:14 +0000 [6428580e7f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* Makefile: Suppress echoing of target 'config' again.
+
+	  ASTERISK-26038 #close
+
+	  Change-Id: I5746cf639f3fdc6332e8a97cf01f979e30bf403f
+
+2016-07-15 02:59 +0000 [e2e8713b84]  Corey Farrell <git@cfware.com>
+
+	* pbx: Create pbx_ignorepat.c for management of 'struct ast_ignorepat'.
+
+	  This changes context ignore patterns from a linked list to a vector,
+	  makes 'struct ast_ignorepat' opaque to pbx.c.
+
+	  Although ast_walk_context_ignorepats is maintained the procedure is no
+	  longer efficient except for the first call (inc==NULL).  This
+	  functionality is replaced by two new functions implemented by vector
+	  macros.
+	  * ast_context_ignorepats_count (AST_VECTOR_SIZE)
+	  * ast_context_ignorepats_get (AST_VECTOR_GET)
+
+	  As with ast_walk_context_ignorepats callers of these functions are
+	  expected to have locked contexts.  Only a few places in Asterisk walked
+	  the ignorepats, they have been converted to use the new functions.
+
+	  Change-Id: I78f2157d275ef1b7d624b4ff7d770d38e5d7f20a
+
+2016-07-14 13:51 +0000 [be36bd7ca5]  Corey Farrell <git@cfware.com>
+
+	* pbx: Create pbx_include.c for management of 'struct ast_include'.
+
+	  This changes context includes from a linked list to a vector, makes
+	  'struct ast_include' opaque to pbx.c.
+
+	  Although ast_walk_context_includes is maintained the procedure is no
+	  longer efficient except for the first call (inc==NULL).  This
+	  functionality is replaced by two new functions implemented by vector
+	  macros.
+	  * ast_context_includes_count (AST_VECTOR_SIZE)
+	  * ast_context_includes_get (AST_VECTOR_GET)
+
+	  As with ast_walk_context_includes callers of these functions are
+	  expected to have locked contexts.  Only a few places in Asterisk walked
+	  the includes, they have been converted to use the new functions.
+
+	  const have been applied where possible to parameters for ast_include
+	  functions.
+
+	  Change-Id: Ib5c882e27cf96fb2aec67a39c18b4c71c9c83b60
+
+2016-07-14 03:25 +0000 [d3348c51b5]  Corey Farrell <git@cfware.com>
+
+	* features.c: Remove unneeded adsi.h include.
+
+	  adsi.h is no longer used by features.c since parking was moved to a
+	  module.
+
+	  Change-Id: I2248b8a455225a17cb6ddaafd6c20c511a1eaf59
+
+2016-06-30 15:58 +0000 [273052f404]  Mark Michelson <mmichelson@digium.com>
+
+	* Update support for SILK format.
+
+	  This commit adds scaffolding in order to support the SILK audio format
+	  on calls. Roughly, this is what is added:
+
+	  * Cached silk formats. One for each possible sample rate.
+	  * ast_codec structures for each possible sample rate.
+	  * RTP payload mappings for "SILK".
+
+	  In addition, this change overhauls the res_format_attr_silk file in the
+	  following ways:
+
+	  * The "samplerate" attribute is scrapped. That's native to the format.
+	  * There are far more checks to ensure that attributes have been
+	    allocated before attempting to reference them.
+	  * We do not SDP fmtp lines for attributes set to 0.
+
+	  These changes make way to be able to install a codec_silk module and
+	  have it actually work. It also should allow for passthrough silk calls
+	  in Asterisk.
+
+	  Change-Id: Ieeb39c95a9fecc9246bcfd3c45a6c9b51c59380e
+
+2016-07-14 07:45 +0000 [31967dacdf]  Richard Miller (license 5685)
+
+	* app_queue: Only remove queue member from pending when state changes.
+
+	  It is possible for a not in use state change to occur multiple
+	  times causing a queue member to be removed from the pending call
+	  container prematurely.
+
+	  The first not in use state change will remove the queue member
+	  from the container. At this moment the member may be called and
+	  placed in the pending container. After this another not in use
+	  state change can be received which will remove it from the
+	  container. Despite being called at this point the code will
+	  incorrectly see that there are no pending calls to it.
+
+	  This change only removes it from the pending container if the
+	  state has actually changed.
+
+	  ASTERISK-26133 #close
+	  patches:
+	    app_queue.diff submitted by Richard Miller (license 5685)
+
+	  Change-Id: Ie5a7f17a44f98e9159e9b85009ce3f8393aa78c0
+
+2016-07-14 02:40 +0000 [f3608b50d7]  Corey Farrell <git@cfware.com>
+
+	* pbx: Fix leak of timezone for time based includes.
+
+	  Create include_free to run ast_destroy_timing and ast_free, use that in
+	  all places that freed an ast_include structure.  This fixes a couple of
+	  paths that previously did not run ast_destroy_timing.
 
 	  ASTERISK-26196 #close
 
-	  Change-Id: I1671bd111bef0dc113e8bf8f77f89fcfc395d838
+	  Change-Id: I1671bd111bef0dc113e8bf8f77f89fcfc395d838
+
+2016-07-13 17:45 +0000 [63ac4c9487]  Kevin Harwell <kharwell@digium.com>
+
+	* translate: explicit format destination not properly set
+
+	  If the destination format's name differed from the codec name then the
+	  translator's explict_dst field would be improperly set. In some circumstances
+	  it would end up setting it to a newly created format that has the same name
+	  as the codec when it actually needed to be the given destination codec.
+
+	  This could cause the translation path to use the wrong format. For instance,
+	  if an endpoint had specified 'myulaw' as a format the translator could end up
+	  using a 'ulaw' format (with whatever/default settings) instead. If the format
+	  attribute settings differed between the two then there may unexpected results
+	  during processing.
+
+	  This patch removes the name check when building the translation path. This
+	  should make it always set the translator's explicit_dst to the given destination
+	  format as long as the sample rate and types match.
+
+	  Change-Id: Iaf8a03831d68e657d89569d54b505074efbefab5
+
+2016-07-08 11:46 +0000 [2f26512fd8]  Richard Mudgett <rmudgett@digium.com>
+
+	* stasis_endpoint.c: Fix contactstatus_to_json().
+
+	  The roundtrip_usec json member is optional.  If it isn't present then
+	  don't put it into the converted json structure where ast_json_pack()
+	  will choke on it.
+
+	  Change-Id: I39bb2f86154ef54591270c58bfda8635070f9ea0
+
+2016-07-11 10:22 +0000 [bc1ff41be7]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_options.c: Fix container operation.
+
+	  aor_observer_deleted() needs to operate on all contacts found for the
+	  deleted AOR instead of only the first one found.  This is really only a
+	  problem if there is more than one contact for the AOR.
+
+	  Change-Id: Id24ac0d5e8c931330231fb45dd2a331a84339dc1
+
+2016-07-11 10:21 +0000 [eabcfeeaa3]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_configuration.c: Misc cleanups.
+
+	  * Fix some whitespace in various routines.
+
+	  * Rename i to iter in persistent_endpoint_update_state().
+
+	  * Fix off-nominal copy/paste message wording in
+	  persistent_endpoint_contact_deleted_observer()
+
+	  Change-Id: Id8e34f5d09e7eebac3af22501c44c1110a3e29d8
+
+2016-07-13 13:45 +0000 [f73ddde7d4]  Corey Farrell <git@cfware.com>
+
+	* chan_sip: Fix reference leak in mwi_event_cb
+
+	  Cleanup the peer reference when stasis_subscription_final_message is
+	  true.  Also free peer_name even if peer exists, after reload a new
+	  peer_name will be allocated.
+
+	  ASTERISK-26193 #close
+
+	  Change-Id: If7ecd52facdc5c227f701c760841e3f6ca53cc69
+
+2016-07-13 11:30 +0000 [fd54d69feb]  Corey Farrell <git@cfware.com>
+
+	* threadpool: Fix leak in ast_threadpool_serializer_group error path.
+
+	  ast_threadpool_serializer_group leaks a reference to ser when listener
+	  is allocated but tps is not.  Although listener takes the reference to
+	  ser cleanup functions are not run without tps.
+
+	  ASTERISK-26191 #close
+
+	  Change-Id: Ie3ccf69a3f1e676c2ef62a77067c0cb57dc9a585
+
+2016-06-22 07:13 +0000 [85212f2799]  Eugene Voityuk <eugene@thirdlane.com>,Alexander Traud <pabstraud@compuserve.com>
+
+	* res_rtp_asterisk: Enable Forward Secrecy (PFS) for DTLS.
+
+	  Since July 2014, TLS based protocols (SIP over TLS, Secure WebSockets, HTTPS)
+	  support PFS thanks to ASTERISK-23905. In July 2015, the same feature was added
+	  for DTLS. The source code from main/tcptls.c should have been re-used to ease
+	  security audits. Therefore, this change rolls back the change from July 2015 and
+	  re-uses the code from July 2014. This has the additional benefits to work under
+	  CentOS 7 and enabling not just ECDHE but DHE based cipher suites as well.
+
+	  ASTERISK-25659 #close
+	  Reported by: StefanEng86, urbaniak, pay123
+	  Tested by: sarumjanuch, traud
+	  patches:
+	  res_rtp_asterisk.patch submitted by sarumjanuch
+	  dtls_centos_step_1.patch submitted by traud
+	  dtls_centos_step_2.patch submitted by traud
+
+	  Change-Id: I537cadf4421f092a613146b230f2c0ee1be28d5c
+
+2016-06-24 19:55 +0000 [0d487b53b1]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_session: Check for presence of an active negotiator
+
+	  It is possible in a hypothetical situation for a session refresh to be
+	  invoked on a PJSIP when the negotiatior on the INVITE session has not
+	  yet been established. While this shouldn't occur with existing uses of
+	  ast_sip_session_refresh, the crashes that occur due to improperly
+	  calling PJSIP functions that expect a non-NULL negotiatior are
+	  avoidable. PJSIP will create the negotiator in pjsip_inv_reinvite; this
+	  means that simply checking for the presence of the negotiator before
+	  passing it to other PJSIP functions that use it is allowable. As such,
+	  this patch adds checks for the presence of the negotiator before calling
+	  PJSIP functions that assume it is non-NULL.
+
+	  Change-Id: I1028323e7e01b0a531865e5412a71b6f6ec4276d
+
+2015-10-19 18:55 +0000 [c49833653b]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_pubsub: Add additional debug statements
+
+	  When something very sad and wrong occurs, it's challenging sometimes to
+	  figure out why. This patch adds some additional debug statements on
+	  off-nominal paths to try and make debugging easier.
+
+	  Change-Id: I7bffb73cc733b6f80193a23340881db4a102b640
+
+2015-10-19 18:55 +0000 [f12311ee69]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_corosync: Raise a Stasis message on node join/leave events
+
+	  When res_corosync detects that a node leaves or joins, it currently is
+	  informed of this via Corosync callbacks. However, there are a few
+	  limitations with the information presented:
+	  (1) While we have information that Corosync is aware of - such as the
+	      Corosync nodeid - that information is really only useful inside of
+	      Corosync or res_corosync. There's no way to translate a Corosync
+	      nodeid to some other internally useful unique identifier for the
+	      Asterisk instance that just joined or left the cluster.
+	  (2) While res_corosync is notified of the instance joining or leaving
+	      the cluster, it has no mechanism to inform the Asterisk core or
+	      other modules of this event. This limits the usefulness of res_corosync
+	      as a heartbeat mechanism for other modules.
+
+	  This patch addresses both issues.
+
+	  First, it adds the notion of a cluster discovery message both within the
+	  Stasis message bus, as well as the binary event messages that
+	  res_corosync uses to transmit data back and forth within the cluster.
+	  When Asterisk joins the cluster, it sends a discovery message to the other
+	  nodes in the cluster, which correlates the Corosync nodeid along with
+	  the Asterisk EID. res_corosync now maintains a hash of Corosync nodeids
+	  to Asterisk EIDs, such that it can map changes in cluster state with the
+	  Asterisk instance that has that nodeid. Likewise, when an Asterisk
+	  instance receives a discovery message from a node in the cluster, it now
+	  sends its own discovery message back to the originating node with the
+	  local Asterisk EID. This lets Asterisk instances within the cluster
+	  build a complete picture of the other Asterisk instances within the
+	  cluster.
+
+	  Second, it publishes the discovery messages onto the Stasis message bus.
+	  Said messages are published whenever a node joins or leaves the cluster.
+	  Interested modules can subscribe for the ast_cluster_discovery_type()
+	  message under the ast_system_topic() and be notified when changes in
+	  cluster state occur.
+
+	  Change-Id: I9015f418d6ae7f47e4994e04e18948df4d49b465
+
+2016-07-13 08:57 +0000 [a3f4141f6f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid obsolete warning with pthread.m4 on autoconf.
+
+	  Updated the macro-set autoconf/ax_pthread.m4 to its latest upstream version.
+
+	  ASTERISK-26046 #close
+
+	  Change-Id: I11abc11d17acd2b6a8a5a5be8ae8e0949dab9cc7
+
+2016-07-11 20:07 +0000 [886f2cab23]  gtjoseph <gjoseph@digium.com>
+
+	* rest_api/channels:  Fix multiple issues with create and dial
+
+	  * We weren't properly subscribing to the channel and it's originator
+	    on create.
+	  * We weren't doing a publish_dial after calling ast_call on dial.
+	  * We weren't calling depart_bridge when a channel left the dial bridge.
+
+	  The first 2 issues were causing events to not be generated and the third
+	  was actually causing channels to not get properly destroyed when hung up.
+
+	  Together these 3 issues were causing the new
+	  rest_apichannels/create_dial_bridge tests to fail.
+
+	  As a result of the fixes, the cdr state machine had to be slightly
+	  tweaked to allow bridge leave events without asserting and the tests
+	  themselves had to be updated to account for the channels now cleaning
+	  themselves up.
+
+	  Change-Id: Ibf23abf5a62de76e82afb4461af5099c961b97d8
+
+2016-07-11 10:25 +0000 [b85446d039]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Fix statsd regression.
+
+	  The ASTERISK-25904 change-id I8fad8aae9305481469c38d2146e1ba3a56d3108f
+	  patch introduced several regressions when the newly created "Updated"
+	  state goes out for each endpoint registration refresh.
+
+	  1) It restarted any OPTIONS RTT ping cycle.
+
+	  2) It would interfere with a currently active ping and throw off that
+	  ping's resulting RTT calculation.
+
+	  3) It cleared the RTT time each time the endpoint was refreshed.
+
+	  4) The cleared RTT time was sent out as a statsd update each time.
+
+	  5) It created two AMI events for each update.
+
+	  * Revert the original patch and reimplement it.  Now the current contact
+	  status state is re-sent instead of the state being momentarily toggled
+	  every time the endpoint refreshes its registration.  The statsd events are
+	  not created for the re-sent refresh because they are sent after every
+	  OPTIONS ping.
+
+	  ASTERISK-26160 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: Ie072be790fbb2a8f5c1c874266e4143fa31f66d1
+
+2016-07-10 19:08 +0000 [4ad333bb0e]  Joshua Colp <jcolp@digium.com>
+
+	* func_odbc: Fix connection deadlock.
+
+	  The func_odbc module was modified to ensure that the
+	  previous behavior of using a single database connection
+	  was maintained. This was done by getting a single database
+	  connection and holding on to it. With the new multiple
+	  connection support in res_odbc this will actually starve
+	  every other thread from getting access to the database as
+	  it also maintains the previous behavior of having only
+	  a single database connection.
+
+	  This change disables the func_odbc specific behavior if
+	  the res_odbc module is running with only a single database
+	  connection active. The connection is only kept for the
+	  duration of the request.
+
+	  ASTERISK-26177 #close
+
+	  Change-Id: I9bdbd8a300fb3233877735ad3fd07bce38115b7f
+
+2016-07-12 03:50 +0000 [110b01a0bc]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Allow own CFLAGS on ./configure.
+
+	  Before this change, make failed with the error
+	  Unknown value '' found in build_tools/menuselect-deps for NATIVE_ARCH
+	  when CFLAGS were supplied to the configure script. This was introduced with
+	  <https://reviewboard.asterisk.org/r/1852/> which disabled BUILD_NATIVE when
+	  CFLAGS were supplied. Those who need different -march= values, please, go for
+	  ./configure
+	  make menuselect.makeopts or make menuselect
+	  ./menuselect/menuselect --disable BUILD_NATIVE
+
+	  ASTERISK-25289 #close
+
+	  Change-Id: Ic6365d5a97bb9b3556858f06432a8d1cfa83eebc
+
+2016-07-11 13:42 +0000 [44f16af7cc]  Richard Mudgett <rmudgett@digium.com>
+
+	* ast_expr2: Fix off-nominal memory leak.
+
+	  Thanks to ibercom for pointing out a memory leak that was missed
+	  in the earlier patch for the issue.
+
+	  ASTERISK-26119
+	  Reported by: Alexei Gradinari
+
+	  Change-Id: I9a151f5c4725d97fb82a9e938bc73dc659532b71
+
+2016-07-11 10:17 +0000 [8476a9332f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* install_prereq: Checkout of libSRTP 1.5.x.
+
+	  Since 5th November 2014, the master branch of libSRTP changed the prefix of
+	  several member names and is not compatible with the source code in Asterisk
+	  anymore. Therefore instead, this change checks out the latest version of the
+	  libSRTP 1.5.x branch. Furthermore now, libSRTP is compiled with OpenSSL as
+	  backend. This makes AES-GCM and AES-IN possible.
+
+	  ASTERISK-22131 #close
+
+	  Change-Id: I2e396cdc01da0ff610686e398ed210ca7408f7d6
+
+2016-07-09 13:32 +0000 [ad30d60c69]  Corey Farrell <git@cfware.com>
+
+	* chan_sip: Fix reference leaks in error paths.
+
+	  * get_sip_pvt_from_replaces leaks sip_pvt_ptr on any error.
+	  * build_peer leaks peer on failure to allocate the endpoint.
+
+	  This patch fixes get_sip_pvt by using an RAII_VAR, build_peer is fixed
+	  with an unref in the appropriate place.
+
+	  ASTERISK-26184 #close
+
+	  Change-Id: I728b424648ad041409f7d90880f4c28b3ce2ca12
+
+2016-07-07 12:44 +0000 [7408c51a48]  Corey Farrell <git@cfware.com>
+
+	* REF_DEBUG: Prevent logging of container node objects.
+
+	  Using AO2_CONTAINER_ALLOC_OPT_DUPS_REPLACE can result in an unref being
+	  recorded to the refs log for the node being replaced.  This prevents
+	  logging of those unrefs since they would produce errors in
+	  refcounter.py.
+
+	  ASTERISK-26181 #close
+
+	  Change-Id: Ie4fded84e8a1a58b3a59ce59dfd7eb0da3ddc5d4
+
+2016-07-04 16:38 +0000 [c832f100d9]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_sorcery_realtime: fix bug when successful UPDATE is treated as failed
+
+	  If the SQL UPDATE statement changes nothing then SQLRowCount returns 0.
+	  This value should be treated as success.
+	  But the function sorcery_realtime_update treats it as failed.
+
+	  This bug was found using stress tests on PJSIP.
+	  If there are 2 consecutive SIP REGISTER requests with the same contact data
+	  during 1 second then res_pjsip_registrar adds contact location on 1st request
+	  and tries to update contact location on 2nd.
+	  The update fails and res_pjsip_registrar even removes correct contact location.
+
+	  The test "object_update_uncreated" was removed from test_sorcery_realtime.c
+	  because it's now a valid situation.
+
+	  This patch also adds missing debug of extra SQL parameter.
+
+	  ASTERISK-26172 #close
+
+	  Change-Id: I05a7f3051455336c9dda29efc229decf86071303
+
+2016-07-07 10:38 +0000 [302be4809a]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip/res_pjsip_t38: Handle a request to negotiate T.38 after it is enabled.
+
+	  Some T.38 implementations may send another re-invite after the initial
+	  one which adds additional negotiation details (such as the max bitrate).
+	  Currently this will fail when passthrough is being done in chan_sip as we
+	  do nothing if T.38 is already active.
+
+	  Other handlers of T.38 inside of Asterisk (such as res_fax) handle this
+	  scenario so this change adds support for it to chan_sip and res_pjsip_t38.
+	  If a request to negotiate is received while T.38 is already enabled a
+	  new re-INVITE is sent and negotiation is done again.
+
+	  ASTERISK-26179 #close
+
+	  Change-Id: I0298494d3da6df3219bbfa4be9aa04015043145c
+
+2016-07-07 10:55 +0000 [fb96492ec4]  Scott Griepentrog <scott@griepentrog.com>
+
+	* PJSIP: provide valid tcp nodelay option for reuse
+
+	  When using TCP transport with chan_pjsip, the TCP_NODELAY
+	  option value was allocated on the stack, then passed as a
+	  pointer to the tcp transport configuration structure, and
+	  later re-used on subsequently created sockets when it was
+	  no longer valid.  This patch changes the allocation to be
+	  a static.
+
+	  ASTERISK-26180 #close
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: I3251164c7f710dbdab031282f00e30a9770626a0
+
+2016-07-06 09:29 +0000 [1c949eea6c]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: Added "subscribe_context" to endpoint
+
+	  If specified, incoming SUBSCRIBE requests will be searched for the matching
+	  extension in the indicated context. If no "subscribe_context" is specified,
+	  then the "context" setting is used.
+
+	  ASTERISK-25471 #close
+
+	  Change-Id: I3fb7a15f5bc154079bd348c08b7ad1cdd2d5e514
+
+2016-07-04 05:58 +0000 [32cb981d04]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid obsolete warning with libcurl.m4 on autoconf.
+
+	  Updated the macro-set autoconf/libcurl.m4 to its latest upstream version. This
+	  avoids a warning about an obsolete macro on AC_HELP_STRING, because Asterisk is
+	  using AS_HELP_STRING everywhere else already.
+
+	  ASTERISK-26046
+
+	  Change-Id: I8299faf504ceaeee3e39930c59293809e116c631
+
+2016-06-22 17:26 +0000 [9f2c007254]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Don't send extra BYE if SDP invalid.
+
+	  When an answer SDP is invalid we were disconnecting the outgoing call and
+	  sending two BYE requests.  The first BYE was sent by PJPROJECT because of
+	  the invalid SDP answer.  The second BYE was sent by Asterisk because it
+	  thought the canceled call was the result of the RFC5407 section 3.1.2 race
+	  condition.
+
+	  * Made not send the BYE on a canceled session if the SDP negotiation is
+	  incomplete because PJPROJECT has already sent a BYE for the failed
+	  negotiation.
+
+	  ASTERISK-25772 #close
+	  Reported by:  Dmitriy Serov
+
+	  Change-Id: I44ad0bd0605e8eeb7035c890d6f97a1331f1a836
+
+2016-06-27 17:19 +0000 [08d3b9a89e]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: End call on initial invalid SDP negotiation.
+
+	  When an incoming call defers SDP negotiation and then sends us an invalid
+	  SDP in the ACK, we need to send a BYE to disconnect the call.  In this
+	  case SDP negotiation has failed and we don't have valid media streams
+	  negotiated.
+
+	  ASTERISK-25772
+
+	  Change-Id: Ia358516b0fc1e6c4c139b78246f10b9da7a2dfb8
+
+2016-06-23 15:13 +0000 [e6e12c752c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.c: Register PJMEDIA error code decoder.
+
+	  Registering the PJMEDIA error codes allows errors found when parsing an
+	  incoming SDP to be easier to figure out.
+
+	  "Missing SDP rtpmap for dynamic payload type (PJMEDIA_SDP_EMISSINGRTPMAP)"
+	  is much easier to understand than "Unknown error 220030".
+
+	  ASTERISK-25772
+
+	  Change-Id: I44b2dcea656fedd7593171be9e845880a2c70ca0
+
+2016-06-27 16:56 +0000 [5d2fc6bab7]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Remove unused parameter from handle_incoming().
+
+	  Change-Id: Iedd182d189ec947c42edc2c66c4bda3c22060daa
+
+2016-06-22 18:02 +0000 [656ed73ac6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add missing NULL checks when using pjsip_inv_end_session().
+
+	  pjsip_inv_end_session() is documented as being able to return the
+	  passed in tdata parameter set to NULL on success.
+
+	  Change-Id: I09d53725c49b7183c41bfa1be3ff225f3a8d3047
+
+2016-06-30 15:17 +0000 [4f7b859726]  Richard Mudgett <rmudgett@digium.com>
+
+	* features: Fix channel datastore access.
+
+	  Found as a result of the testsuite tests/callparking test crashing.
+
+	  Several calls to ast_get_chan_featuremap_config() and
+	  ast_get_chan_features_xfer_config() did not lock the channel before
+	  calling so the channel's datastore list was accessed without the lock's
+	  protection.  Apparently another thread deleted a datastore on the
+	  channel's list while the crashing thread was walking the list.  Crash at
+	  0xdeaddead due to MALLOC_DEBUG's memory filler value as a result.
+
+	  * Add missing channel locks to calls that were not already protected
+	  as the doxygen for those calls indicates.
+
+	  Change-Id: Id273b3d305cc616406c353cbc841b2b7655efaa1
+
+2016-06-30 08:25 +0000 [5ad7e1c09a]  gtjoseph <gjoseph@digium.com>
+
+	* configure:  Fix HAVE_PJSIP_EVSUB_GRP_LOCK not set with external pjproject
+
+	  There was a typo in configure.ac preventing HAVE_PJSIP_EVSUB_GRP_LOCK
+	  from getting set when using an external pjproject.
+
+	  ASTERISK-26099 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: I709af70428e125fb5ccd44b171d25dd29141f0ae
+
+2016-06-29 15:31 +0000 [dab2a6b689]  Matt Jordan <mjordan@digium.com>
+
+	* hep.conf.sample: Default 'enabled' to 'no'
+
+	  Following the principle of least surprise, we should not be sending
+	  massive numbers of PJSIP and RTCP HEP packets out into the ether to some
+	  only-slightly-random IP address. Having 'enabled' set to 'no' in the
+	  sample configuration file should prevent this from happening for those
+	  who run 'make samples'.
+
+	  ASTERISK-26159 #close
+
+	  Change-Id: I1753a64ca83a3442a6ebdc31061f8185c062d9b1
+
+2016-06-29 15:09 +0000 [9129ac8e73]  Matt Jordan <mjordan@digium.com>
+
+	* pjproject/patches/config_site: Increase the max number of ICE candidates
+
+	  When negotiating ICE candidates with WebRTC capable endpoints, many
+	  networks will result in a browser offering ICE candidates that exceeds
+	  the default number of max candidates, 16. This patch bumps the max
+	  candidates to 32, with the max checks at twice the number of candidates.
+	  In practice, this has shown to be sufficient for browser/WebRTC
+	  negotiation.
+
+	  Change-Id: Ifd8da8b315f5ae14814d4ce20e10d2e6355020e5
+
+2016-06-28 09:00 +0000 [4045e6d8ba]  gtjoseph <gjoseph@digium.com>
+
+	* codecs:  Fix ABI incompatibility created by adding format_name to ast_codec
+
+	  Adding format_name even to the end of ast_codec caused issued with
+	  binary codec modules because the pointer would be garbage in asterisk
+	  when they registered.  So, the ast_codec structure was reverted and an
+	  internal_ast_codec structure was created just for use in codec.c.  A new
+	  internal-only API was also added (__ast_codec_register_with_format) so
+	  that codec_builtin could register codecs with the format_name in a
+	  separate parameter rather than in the ast_codec structure.
+
+	  ASTERISK-26144 #close
+	  Reported-by: Alexei Gradinari
+
+	  Change-Id: I6df1b08f6a6ae089db23adfe1ebc8636330265ba
+
+2016-06-28 08:22 +0000 [651290a809]  gtjoseph <gjoseph@digium.com>
+
+	* BuildSystem:  Fix a few issues hightlighted by gcc 6.x
+
+	  gcc 6.1.1 caught a few more issues.
+	  Made sure the unit tests still pass for the func_env and stdtime
+	  issues.
+
+	  ASTERISK-26157 #close
+
+	  Change-Id: I6664d8f34a45bc1481d2a854481c7878b0c1cf8e
+
+2016-06-28 10:33 +0000 [83f2c2573b]  Matt Jordan <mjordan@digium.com>
+
+	* configs/basic-pbx/modules.conf: Remove 'bad' modules
+
+	  This patch removes the following modules:
+	   - pbx_functions: It never existed.
+	   - res_pjsip_log_forwarder: It no longer exists.
+	   - res_hep_pjsip: The base HEP module wasn't loaded, and most basic PBXs
+	                    aren't going to be installing HOMER
+	   - res_pjsip_phoneprov_provider: The basic res_phoneprov module isn't
+	                    loaded, and we aren't configured to make use of the
+	                    module
+
+	  Change-Id: Id91f68cae7c9c8c3d370029fe1268cb51e4ff5a5
+
+2016-06-22 11:19 +0000 [75818b4084]  Joshua Colp <jcolp@digium.com>
+
+	* siren: Add format attribute modules for Siren7 and Siren14.
+
+	  This change removes hardcoded SDP parsing and generation for
+	  Siren7 and Siren14 from chan_sip and moves it to format attribute
+	  modules so it can also be used by chan_pjsip.
+
+	  With this the fmtp lines for both are added with the bitrate
+	  information.
+
+	  ASTERISK-26021
+
+	  Change-Id: Ibb004eda37a14c0a35ef0613f6237977fc800037
+
+2016-06-23 04:33 +0000 [6e87bf746a]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid obsolete warning with AC_TYPE_SIGNAL on autoconf.
+
+	  Removed the obsolete macro AC_TYPE_SIGNAL because Asterisk does not use K&R C
+	  but requires ANSI C anyway.
+
+	  ASTERISK-26046
+
+	  Change-Id: I914c014385e1862102d90fe7650621def78db02e
+
+2016-06-22 15:04 +0000 [8c7017f76e]  Corey Farrell <git@cfware.com>
+
+	* res_fax: Fix reference leak in fax_v21_session_new.
+
+	  fax_v21_session_new created a session details object but only released
+	  the allocation reference during error conditions.  fax_session_new adds
+	  it's own reference to details if needed so the caller is always
+	  responsible for cleaning it's own reference.
+
+	  ASTERISK-26141 #close
+
+	  Change-Id: Ie7fc52a83b6596ce9ce2d5a2bd9f3e204f48fc88
+
+2016-06-22 14:25 +0000 [6fa3ed0679]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: improve realtime performance #2
+
+	  The patch removes updating all Endpoints' status on startup.
+	  Instead, only non-qualified aors with static contact
+	  and non-qualified non-expired contacts are retrieved from the realtime to
+	  update the endpoint status to ONLINE.
+	  The endpoint name was added to the contact object to simply find the endpoint
+	  that created this contact.
+
+	  The status of endpoints with qualified aors will be updated by 'qualify'
+	  functions.
+
+	  ASTERISK-26061 #close
+
+	  Change-Id: Id324c1776fa55d3741e0c5457ecac0304cb1a0df
+
+2016-06-22 13:41 +0000 [d293ead077]  gtjoseph <gjoseph@digium.com>
+
+	* res_rtp_asterisk:  Fix a self-comparison identified by gcc 6
+
+	  gcc 6 caught a previously unidentified self-comparison in
+	  ice_candidate_cmp.  Fixed it and re-ordered the predicates for better
+	  short-circuiting.
+
+	  ASTERISK-26140 #close
+
+	  Change-Id: I3da713c568e24064430257b3502fbdafd35af7a7
+
+2016-06-22 10:37 +0000 [c7309a5254]  gtjoseph <gjoseph@digium.com>
+
+	* chan_unistim:  Fix memcpy in get_to_address
+
+	  A code block only enabled when HAVE_PKTINFO is not defined (FreeBSD)
+	  was using a pointer to a pointer as the destination of a memcpy and a
+	  '&' instead of '*' in the sizeof.
+
+	  ASTERISK-26138 #close
+
+	  Change-Id: Id4927ff256c0e470bdf7bcfc025146a2f656e708
+
+2016-06-20 13:21 +0000 [b6bd97eea2]  Mark Michelson <mmichelson@digium.com>
+
+	* Fix Alembic upgrades.
+
+	  A non-existent constraint was being referenced in the upgrade script.
+	  This patch corrects the problem by removing the reference.
+
+	  In addition, the head of the alembic branch referred to a non-existent
+	  revision. This has been fixed by referring to the proper revision.
+
+	  This patch fixes another realtime problem as well. Our Alembic scripts
+	  store booleans as yes or no values. However, Sorcery tries to insert
+	  "true" or "false" instead. This patch introduces a new boolean type that
+	  translates to "yes" or "no" instead.
+
+	  ASTERISK-26128 #close
+
+	  Change-Id: I51574736a881189de695a824883a18d66a52dcef
+
+2016-06-22 10:51 +0000 [3b4f5d1345]  gtjoseph <gjoseph@digium.com>
+
+	* test_res_pjsip_scheduler: Add 'depends' on pjproject in MODULEINFO
+
+	  Since the file was missing the depends on pjproject, it wasn't
+	  picking up the pjproject related include path.  If there was no
+	  system installed pjproject and pjproject-bundled was used, a compile
+	  would fail because pjsip.h wasn't found.
+
+	  ASTERISK-26139 #close
+
+	  Change-Id: I2ee64a999051452bc198c4e2c168c70769cd3757
+
+2016-06-22 10:55 +0000 [5f23aacda4]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid obsolete warning with AC_FUNC_SETVBUF_REVERSED on autoconf.
+
+	  Removed the obsolete macro AC_FUNC_SETVBUF_REVERSED because Asterisk does not
+	  support the platform SVR2 from the year 1987 anymore.
+
+	  ASTERISK-26046
+
+	  Change-Id: I28161b037feb2d29ab46ed20e785928460226c22
+
+2016-06-21 06:52 +0000 [804005d251]  Torrey Searle <torrey@voxbone.com>
+
+	* res_rtp_asterisk: fix memory leak in dtls
+
+	  ensure that cert bios get freed after creating the fingerprint
+
+	  ASTERISK-26129 #close
+
+	  Change-Id: I44d23aea07dce80176ca1ff877c5ace9452ef451
+
+2016-06-21 17:42 +0000 [f572b26495]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjproject.c: Replace inlined DEBUG_ATLEAST() with macro.
+
+	  Change-Id: I8799fb0a347ad76e747dafd0eacf1ea1086b9a8c
+
+2016-06-12 11:19 +0000 [b57cd01404]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip_pubsub: Address SEGV when attempting to terminate a subscription
+
+	  Occasionally under load we'll attempt to send a final NOTIFY on a
+	  subscription that's already been terminated and a SEGV will occur
+	  down in pjproject's evsub_destroy function.  This is a result of a
+	  race condition between all the paths that can generate a notify
+	  and/or destroy the underlying pjproject evsub object:
+
+	   * The client can send a SUBSCRIBE with Expires: 0.
+	   * The client can send a SUBSCRIBE/refresh.
+	   * The subscription timer can expire.
+	   * An extension state can change.
+	   * An MWI event can be generated.
+	   * The pjproject transaction timer (timer_b) can expire.
+
+	  Normally when our pubsub_on_evsub_state is called with a terminate,
+	  we push a task to the serializer and return at which point the dialog
+	  is unlocked.  This is usually not a problem because the task runs
+	  immediately and locks the dialog again.  When the system is heavily
+	  loaded though, there may be a delay between the unlock and relock
+	  during which another event may occur such as the subscription timer
+	  or timer_b expiring, an extension state change, etc.  These may also
+	  cause a terminate to be processed and if so, we could cause pjproject
+	  to try to destroy the evsub structure twice.  There's no way for us to
+	  tell that the evsub was already destroyed and the evsub's group lock
+	  can't tolerate this and SEGVs.
+
+	  The remedy is twofold.
+
+	   * A patch has been submitted to Teluu and added to the bundled
+	     pjproject which adds add/decrement operations on evsub's group lock.
+
+	   * In res_pjsip_pubsub:
+	     * configure.ac and pjproject-bundled's configure.m4 were updated
+	       to check for the new evsub group lock APIs.
+	     * We now add a reference to the evsub group lock when we create
+	       the subscription and remove the reference when we clean up the
+	       subscription.  This prevents evsub from being destroyed before
+	       we're done with it.
+	     * A state has been added to the subscription tree structure so
+	       termination progress can be tracked through the asyncronous tasks.
+	     * The pubsub_on_evsub_state callback has been split so it's not doing
+	       double duty.  It now only handles the final cleanup of the
+	       subscription tree.  pubsub_on_rx_refresh now handles both client
+	       refreshes and client terminates.  It was always being called for
+	       both anyway.
+	     * The serialized_on_server_timeout task was removed since
+	       serialized_pubsub_on_rx_refresh was almost identical.
+	     * Missing state checks and ao2_cleanups were added.
+	     * Some debug levels were adjusted to make seeing only off-nominal
+	       things at level 1 and nominal or progress things at level 2+.
+
+	  ASTERISK-26099 #close
+	  Reported-by: Ross Beer.
+
+	  Change-Id: I779d11802cf672a51392e62a74a1216596075ba1
+
+2016-06-21 07:05 +0000 [6eb0354f2d]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_rtp_asterisk: Use latest DTLS version available by underlying platform.
+
+	  Do not use DTLSv1_method() but DTLS_method() when available in OpenSSL of the
+	  underlying platform. This change enables DTLS 1.2 since OpenSSL 1.0.2, for
+	  WebRTC (DTLS-SRTP via SIP-over-WebSockets). This change enables AEAD-based
+	  cipher-suites.
+
+	  ASTERISK-26130 #close
+
+	  Change-Id: I41f24448d6d2953e8bdb97c9f4a6bc8a8f055fd0
+
+2016-06-21 10:53 +0000 [596d0b0bc3]  Scott Griepentrog <scott@griepentrog.com>
+
+	* PJSIP: provide transport type with received messages
+
+	  The receipt of a SIP MESSAGE may occur over any transport including TCP
+	  and TLS. When the message is received, the original URI is added to the
+	  message in the field PJSIP_RECVADDR, but this is insufficient to ensure
+	  a reply message can reach the originating endpoint. This patch adds the
+	  PJSIP_TRANSPORT field populated with the transport type.
+
+	  ASTERISK-26132 #close
+
+	  Change-Id: I28c4b1e40d573a056c81deb213ecf53e968f725e
+
+2016-06-21 08:01 +0000 [9e222efbf2]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid obsolete warning with HELP_STRING on autoconf.
+
+	  Some configure scripts used both AC_HELP_STRING and its replacement
+	  AS_HELP_STRING. For consistency and to avoid obsolete warnings, those were
+	  changed to AS_HELP_STRING.
+
+	  ASTERISK-26046
+
+	  Change-Id: I8aad4fd2bdee40aa2a31ce3339a1eb33ff4f5b0f
+
+2016-06-20 10:29 +0000 [e94aae00a7]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_session: Handle race condition at shutdown with timer.
+
+	  When shutting down res_pjsip_session will get unloaded before res_pjsip.
+	  The act of unloading unregisters all the PJSIP services and sets
+	  their module IDs to -1. In some cases it is possible for a timer to
+	  occur after this happens which calls into res_pjsip_session. The
+	  res_pjsip_session module can then try to get the session from the
+	  INVITE session using the module ID. Since the module ID is now -1
+	  this fails.
+
+	  This change stores a copy of the module ID and uses it for the timer
+	  callback scenario. If the module ID is -1 the callback immediately
+	  returns but if the module ID is valid then it continues as normal.
+
+	  This works as the original ID of the module is guaranteed to still
+	  be valid when used with the INVITE session.
+
+	  ASTERISK-26127 #close
+
+	  Change-Id: I88df72525c4e9ef9f19c13aedddd3ac4a335c573
+
+2016-06-20 12:13 +0000 [0a30008224]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_voicemail.c: Fix IMAP compile error.
+
+	  Fix compile error introduced by the patch for
+	  ASTERISK-26045
+
+	  Change-Id: I5b02876266f2824f4cec2b54d6ff4db5de5778d3
+
+2016-06-17 13:51 +0000 [820ed3d4b3]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* fix: memory leaks, resource leaks, out of bounds and bugs
+
+	  ASTERISK-26119 #close
+
+	  Change-Id: Iecbf7d0f360a021147344c4e83ab242fd1e7512c
+
+2016-06-13 17:40 +0000 [11caa10cf5]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Ensure announcer channels are destroyed.
+
+	  Announcer channels were not being destroyed because the
+	  stasis_app_control structure that referenced them was not being
+	  destroyed. The control structure was not being destroyed because it was
+	  not being unlinked from its container. It was not being unlinked from
+	  its container because the after bridge callback for the announcer
+	  channel was not being run. The after bridge callback was not being run
+	  because the after bridge datastore was not being removed from the
+	  channel on destruction. The channel was not being destroyed because the
+	  hangup that used to destroy the channel was now only reducing the
+	  reference count to one. The reference count of the channel was only
+	  being reduced to one because the stasis_app_control structure was
+	  holding the final reference...
+
+	  The control structure used to not keep a reference to the channel, so
+	  that loop described above did not happen.
+
+	  The solution is to manually remove the control structure from its
+	  container when the playback on a bridge is complete.
+
+	  ASTERISK-26083 #close
+	  Reported by Joshua Colp
+
+	  Change-Id: I0ddc0f64484ea0016245800b409b567dfe85cfb4
+
+2016-06-20 08:05 +0000 [f72ffc1ff9]  Alexander Traud <pabstraud@compuserve.com>
+
+	* http: leverage 'bindaddr' for TLS in http.conf
+
+	  The internal HTTP/WebSocket server supports both TCP and TLS, which can be
+	  activated separately via the file http.conf. The source code intends to re-use
+	  the TCP parameter 'bindaddr' for TLS, even if 'tlsbindaddr' is not specified
+	  explicitly. This did not work because of a typo. This change resolves this typo.
+
+	  ASTERISK-26126 #close
+
+	  Change-Id: I5efb0409ae12044dfb3495b6b97b6d40a8c9c51f
+
+2016-05-18 17:37 +0000 [3c80f84cd0]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_transport_management.c: Misc cleanups to survive shutdown.
+
+	  * In unload_module(), reordered destroying things to minimize the window
+	  that the global transports container could be used by other threads on
+	  shutdown.  When shutting down you need to stop things in the opposite
+	  order of creation.
+
+	  * Put the global transports container into an AO2_GLOBAL_OBJ_STATIC to
+	  eliminate the crash potential by other threads using the container on
+	  shutdown.
+
+	  * Made struct monitored_transport.sip_received not use
+	  ast_atomic_fetchadd_int() since it is used as a boolean value that is only
+	  set TRUE.  It was previously incremented for every received SIP message
+	  and could theoretically overflow.
+
+	  * In monitored_transport_state_callback(), allocated the monitored
+	  transport object without a lock since the lock was unused.
+
+	  * In keepalive_global_loaded(), removed releasing the transports container
+	  if the keepalive_thread could not be started.  I set it up to be tried
+	  again if the user reloads the configuration.
+
+	  Change-Id: I8d12d16ef564290fa6d25a32334bb5ce8fdf87ff
+
+2016-01-05 19:08 +0000 [7c59f2126f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.c: Add check that timer actually got scheduled.
+
+	  Change-Id: Iabaa2e5dccf0762c258101ea0eb1487cf6959ad1
+
+2016-06-13 13:33 +0000 [51cc5c31c4]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_rtp_multicast.c: Fix warning message typo.
+
+	  Change-Id: Ic9928208b9957e09866abe3d9649030942ec52b3
+
+2016-02-11 18:15 +0000 [3d0632a9c2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Reorganize ast_sip_session_terminate().
+
+	  Change-Id: I68a2128bcba4830985d2d441e70dfd1ac5bd712b
+
+2016-06-08 06:15 +0000 [ac683f13c9]  Alexander Traud <pabstraud@compuserve.com>
+
+	* core: Not the configured but granted number of possible file descriptors.
+
+	  With CLI "core show settings", simply the parameter maxfiles of the file
+	  asterisk.conf was shown. If that parameter was not set, nothing was displayed
+	  although the environment might have set a default number itself. Or if maxfiles
+	  were not granted (completely), still maxfiles was shown. Now, the maximum number
+	  of possible file descriptors in the environment is shown.
+
+	  ASTERISK-26097
+
+	  Change-Id: I2df5c58863b5007b34b77adbe28b885dfcdf7e0b
+
+2016-06-10 10:39 +0000 [4eb8cf2684]  Joshua Colp <jcolp@digium.com>
+
+	* translate: Enables native Packet-Loss Concealment (PLC) for supporting codecs.
+
+	  This reverts commit 5bfef2a8b4674382f959b21a3b8e14cf1d942bab as it
+	  caused fax test failures.
+
+	  ASTERISK-25629
+
+	  Change-Id: I79de974dc4f63a1cafe0d2509169fd9a6b3cbaf4
+
+2016-06-08 06:05 +0000 [0bf1a53db3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* astfd: With RLIMIT_NOFILE only the current value is sensible.
+
+	  With menuselect "DEBUG_FD_LEAKS" and CLI "core show fd", both the maximum max
+	  and current max of possible file descriptors were shown. Both show the same
+	  value always. Not to confuse users, just the current maximum is shown now.
+
+	  ASTERISK-26097
+
+	  Change-Id: I49cf7952d73aec9e3f6a88942842c39be18380fa
+
+2016-06-07 18:45 +0000 [d338343dac]  Joshua Colp <jcolp@digium.com>
+
+	* cel: Ensure only one dial status per channel exists.
+
+	  CEL wrongly assumed that a channel would only have a single dial
+	  event on it. This is incorrect. Particularly in a queue each
+	  call attempt to a member will result in a dial event, adding
+	  a new dial status in CEL without removing the old one. This
+	  would cause the container to grow with only one dial status
+	  being removed when the channel went away. The other dial status
+	  entries would remain leaking memory.
+
+	  This change fixes the memory leak by ensuring that only one dial
+	  status will only ever exist for each channel.
+
+	  The behavior during the scenario where multiple events are received
+	  has also been improved. For failure cases the first failure will
+	  be the dial status. If an answer dial status is received, though,
+	  it will take priority and the dial status for the channel will be
+	  answer.
+
+	  Memory usage has also been decreased by storing the minimal
+	  amount of information and the code has been cleaned up slightly.
+
+	  ASTERISK-25262 #close
+
+	  Change-Id: I5944eb923db17b6a0faa7317ff6abc9307c009fe
+
+2016-06-01 13:48 +0000 [1fd3a7849e]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Ensure proper channel state on operations.
+
+	  ARI was recently outfitted with operations to create and dial channels.
+	  This leads to the ability to try funny stuff. You could create a channel
+	  and then immediately try to play back media on it. You could create a
+	  channel, dial it, and while it is ringing attempt to make it continue in
+	  the dialplan.
+
+	  This commit attempts to fix this by adding a channel state check to
+	  operations that should not be able to operate on outbound channels that
+	  have not yet answered. If a channel is in an invalid state, we will send
+	  a 412 response.
+
+	  ASTERISK-26047 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I2ca51bf9ef2b44a1dc5a73f2d2de35c62c37dfd8
+
+2016-06-08 11:27 +0000 [10019dc70c]  Mark Michelson <mmichelson@digium.com>
+
+	* test_http_media_cache: Fix failing test.
+
+	  The retrieve_cache_control_directives test has been failing occasionally
+	  in Jenkins. The apparent failure occurs when attempting to validate the
+	  expiration of the retrieved file.
+
+	  After reproducing, the problem was pretty clear. At the beginning of the
+	  test, the current time is retrieved. The seconds value of this timestamp
+	  is X. When the file is retrieved, res_http_media_cache calculates the
+	  expiration and in doing so retrieves the current time. In most cases,
+	  since the test executes quickly, it will also retrieve a timestamp with
+	  X seconds. However, if the test starts very near to when the timestamp
+	  seconds are set to increment, res_http_media_cache may retrieve a
+	  timestamp with X+1 seconds instead.
+
+	  The test attempted to account for this by allowing a tolerance of 1
+	  second when validating the expiration. However, the problem was that the
+	  comparisons being used in the validation used > and < operations. This
+	  meant that values that fell within the tolerance (because they equaled
+	  the upper bound of the tolerance) would fail.
+
+	  The solution is to use >= and <= operators in the expiration validation.
+
+	  However, I estimated that while the one second tolerance should be
+	  fine on most machines, it would still be possible on a very slow machine
+	  to end up falling outside the one second tolerance. So I have also
+	  relaxed the tolerance of expiration validation to be three seconds
+	  instead.
+
+	  The final change here is to add a debug message when validating
+	  expiration so that we can see what values are being compared.
+
+	  ASTERISK-25959 #close
+	  Reported by Joshua Colp
+
+	  Change-Id: Ic1a0e10722c1c5d276d5a4d6a67136d6ec26c247
+
+2016-06-03 01:20 +0000 [56bdf048d2]  Timo Teräs <timo.teras@iki.fi>
+
+	* Add support for OGG/Speex file format
+
+	  ASTERISK-18995 #close
+
+	  Change-Id: I98518bd28fc8f95668b3fe27d2cab45045ff3f7a
+
+2016-06-09 10:33 +0000 [f0855358a6]  gtjoseph <gjoseph@digium.com>
+
+	* cdr.c: Remove assert in base_process_dial_end
+
+	  Scenario: Caller blonde transfer
+	  Bob calls Charlie who answers.
+	  Bob puts Charlie on hold and calls Alice.
+	  Before Alice answers, Bob transfers Charlie to Alice.
+
+	  Charlie's channel triggers an assert because he gets an "ANSWERED"
+	  event even though he never dialed anything. With recent changes to dial
+	  events, this is now a valid scenario so the assert needed to be removed.
+
+	  ASTERISK-26103 #close
+
+	  Change-Id: I2679b517b696e7952ab7fb29403df9140e7d1de2
+
+2016-06-09 10:37 +0000 [cdb7edbe7b]  Mark Michelson <mmichelson@digium.com>
+
+	* chan_pjsip: Lock channel when checking for RTP changes.
+
+	  bridge_native_rtp can call into an RTP-capable channel driver in order
+	  for the driver to update information about who the channel is
+	  communicating with. For SIP channel drivers, this means deactivating
+	  RTCP and sending a reinvite so that the endpoints can communicate
+	  directly.
+
+	  bridge_native_rtp does the right thing and has the channel locked when
+	  calling into the channel driver. chan_pjsip can't alter session
+	  properties in this thread, though. chan_pjsip queues a task on the
+	  session serializer in order to update properties there.
+
+	  The problem is that this queued task was not locking the channel. This
+	  meant that the queued task could attempt to deactivate RTCP at the same
+	  time that the channel thread was attempting to process an incoming RTCP
+	  packet. This could lead to a crash.
+
+	  This patch fixes the issue by locking the channel in the queued task
+	  when altering RTP properties.
+
+	  ASTERISK-26092 #close
+	  Reported by Niklas Larsson
+
+	  Change-Id: I3464e226a3c41f6b915f97891e07fa1599e2a159
+
+2016-06-03 22:44 +0000 [04ec9c745e]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_registrar.c: Eliminate rx REGISTER request race condition.
+
+	  This patch fixes a race condition processing received REGISTER requests
+	  and their retransmissions caused by REGISTER requests being processed by
+	  two threads.  The "sip_transaction Unable to register REGISTER transaction
+	  (key exists)" message is a notable symptom of this issue.
+
+	  This issue was more likely to happen before the pjsip/distributor
+	  serializers were created.  Instead of steps one and two below placing the
+	  REGISTER messages into the same pjsip/distributor they were placed in
+	  random pjsip/default serializers.
+
+	  1) REGISTER requests come in and get placed on the pjsip/distributor
+	  serializer.
+
+	  2) Before the first request is processed a retransmission comes in and is
+	  placed on the same pjsip/distributor serializer.
+
+	  3) The first request goes up the pjsip stack and is then shunted off to
+	  the pjsip/aor/<aor> serializer.
+
+	  4) Before the first request is completed processing in the pjsip/aor/<aor>
+	  serializer, the second request goes up the pjsip stack and is also shunted
+	  off to the pjsip/aor/<aor> serializer.
+
+	  5) The first request completes processing and sends out its response.
+
+	  6) The second request completes processing and tries to send out its
+	  response but pjlib complains that the REGISTER transaction key already
+	  exists.
+
+	  7) Sadness ensues.
+
+	  * The race is eliminated by removing the pjsip/aor/<aor> serializer and
+	  continuing the processing in the pjsip/distributor serializer.  Now any
+	  retransmissions queued in the pjsip/distributor serializer will be
+	  processed after the first message is completely processed.
+
+	  ASTERISK-26088 #close
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I842d714346088bf717ea27437f1dd85bff0bab5a
+
+2016-06-03 11:35 +0000 [dcfef53ee2]  Richard Mudgett <rmudgett@digium.com>
+
+	* stasis: Add setting subscription congestion levels.
+
+	  Stasis subscriptions and message routers create taskprocessors to process
+	  the event messages.  API calls are needed to be able to set the congestion
+	  levels of these taskprocessors for selected subscriptions and message
+	  routers.
+
+	  * Updated CDR, CEL, and manager's stasis subscription congestion levels
+	  based upon stress testing.  Increased the congestion levels to reduce the
+	  potential for bursty call setup/teardown activity from triggering the
+	  taskprocessor overload alert.  CDRs in particular need an extra high
+	  congestion level because they can take awhile to process the stasis
+	  messages.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: Id0a716394b4eee746dd158acc63d703902450244
+
+2016-06-02 18:19 +0000 [4879cd875c]  Richard Mudgett <rmudgett@digium.com>
+
+	* sorcery: Add setting object type congestion levels.
+
+	  Sorcery creates taskprocessors for object types to process object observer
+	  callbacks.  An API call is needed to be able to set the congestion levels
+	  of these taskprocessors for selected object types.
+
+	  * Updated PJSIP's contact and contact_status sorcery object type observer
+	  default congestion levels based upon stress testing.  Increased the
+	  congestion levels to reduce the potential for bursty register/unregister
+	  and subscribe/unsubscribe activity from triggering the taskprocessor
+	  overload alert.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I4542e83b556f0714009bfeff89505c801f1218c6
+
+2016-06-02 16:08 +0000 [2cd67d5b07]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessors: Implement high/low water mark alerts.
+
+	  When taskprocessors get backed up, there is a good chance that we are
+	  being overloaded and need to defer adding new work to the system.
+
+	  * Implemented a high/low water alert mechanism for modules to check if the
+	  system is being overloaded and take appropriate action.  When a
+	  taskprocessor is created it has default congestion levels set.  A
+	  taskprocessor can later have those congestion levels altered for specific
+	  needs if stress testing shows that the taskprocessor is a symptom of
+	  overloading or needs to handle bursty activity without triggering an
+	  overload alert.
+
+	  * Add CLI "core show taskprocessor" low/high water columns.
+
+	  * Fixed __allocate_taskprocessor() to not use RAII_VAR().  RAII_VAR() was
+	  never a good thing to use when creating a taskprocessor because of the
+	  nature of how its references needed to be cleaned up on a partial
+	  creation.
+
+	  * Made res_pjsip's distributor check if the taskprocessor overload alert
+	  is active before placing a message representing brand new work onto a
+	  distributor serializer.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I182f1be603529cd665958661c4c05ff9901825fa
+
+2016-05-27 17:31 +0000 [c966a035e0]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session: Use distributor serializer for incoming calls.
+
+	  We must continue using the serializer that the original INVITE came in on
+	  for the dialog.  There may be retransmissions already enqueued in the
+	  original serializer that can result in reentrancy and message sequencing
+	  problems.
+
+	  Outgoing call legs create the pjsip/outsess/<endpoint> serializers for
+	  their dialogs.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I24d7948749c582b8045d5389ba3f6588508adbbc
+
+2016-05-27 16:28 +0000 [5b7b16a87f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Recreate subscriptions using distributor serializer.
+
+	  * Resolves potential reentrancy problems if system restarted in the middle
+	  of subscription message transactions.
+
+	  * Fixes memory leak recreating persistent subscriptions when the
+	  subscription resource tree could not be created.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I71e34d7ae8ed35a694f1030e820e2548c48697be
+
+2016-05-27 12:50 +0000 [c2ae49249c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Use distributor serializer for incoming subscriptions.
+
+	  We must continue using the serializer that the original SUBSCRIBE came in
+	  on for the dialog.  There may be retransmissions already enqueued in the
+	  original serializer that can result in reentrancy and message sequencing
+	  problems.  The "sip_transaction Unable to register SUBSCRIBE transaction
+	  (key exists)" message is a notable symptom of this issue.
+
+	  Outgoing subscriptions still create the pjsip/pubsub/<endpoint>
+	  serializers for their dialogs.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I18b00bb74a56747b2c8c29543a82440b110bf0b0
+
+2016-05-26 17:35 +0000 [2ff26e9746]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Consistently pick a serializer for messages.
+
+	  Incoming messages that are not part of a dialog or a recognized response
+	  to one of our requests need to be sent to a consistent serializer.  Under
+	  load we may be queueing retransmissions before we can process the original
+	  message.  We don't need to throw these messages onto random serializers
+	  and cause reentrancy and message sequencing problems.
+
+	  * Created a pool of pjsip/distributor serializers that get picked by
+	  hashing the call-id and remote tag strings of the received messages.
+
+	  * Made ast_sip_destroy_distributor() destroy items in the reverse order of
+	  creation.
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I2ce769389fc060d9f379977f559026fbcb632407
+
+2016-06-02 12:51 +0000 [df2791da8f]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Ignore messages until fully booted.
+
+	  We should not be processing any incoming messages until we are fully
+	  booted.  We may not have dialplan or other needed configuration loaded
+	  yet.
+
+	  ASTERISK-26089 #close
+	  Reported by: Scott Griepentrog
+
+	  ASTERISK-26088
+	  Reported by:  Richard Mudgett
+
+	  Change-Id: I584aefb4f34b885a8927e1f13a2c64babd606264
+
+2016-06-09 09:20 +0000 [d21a77b325]  gtjoseph <gjoseph@digium.com>
+
+	* build:  Fix ast_sockaddr initialization to be more portable
+
+	  A change to glibc 2.22 changed the order of the sockadddr_storage
+	  members which caused the places where we do an initialization of
+	  ast_sockaddr with '{ { 0, 0, } }' to fail compilation.  Those
+	  initializers (which we shouldn't have been using anyway) have been
+	  replaced with memsets.
+
+	  Change-Id: Idd1b3b320903d8771bfe221f0b015685de628fa4
+
+2016-06-03 00:59 +0000 [72d190eb69]  Timo Teräs <timo.teras@iki.fi>
+
+	* Detect and use proper libraries for musl toolchains
+
+	  Change-Id: I8d9b212f70813404b82918a3f99439e500d4bfcb
+
+2016-06-03 00:57 +0000 [39b69ab537]  Timo Teräs <timo.teras@iki.fi>
+
+	* Fixes to include signal.h
+
+	  POSIX defines signal.h. sys/signal.h should not be used as it is
+	  c-library internal header which may or may not exist. Notably with
+	  musl it generates warning of being incorrect.
+
+	  Change-Id: Ia56b0aa1d84b5c590114867b1b384a624f39a6fc
+
+2016-06-08 12:26 +0000 [7f5ca67e5f]  Matt Jordan <mjordan@digium.com>
+
+	* res_hep_{pjsip|rtcp}: Decline module loads if res_hep had not loaded
+
+	  A crash can occur in res_hep_pjsip or res_hep_rtcp if res_hep has not
+	  loaded and does not have a configuration file. Previously when this
+	  occurred, checks were put in to see if the configuration was loaded
+	  successfully. While this is a good idea - and has been added to the
+	  offending function in res_hep - the reality is res_hep_pjsip and
+	  res_hep_rtcp have no business running if res_hep isn't also running.
+
+	  As such, this patch also adds a function to res_hep that returns whether
+	  or not it successfully loaded. Oddly enough, ast_module_check returns
+	  "everything is peachy" even if a module declined its load - so it cannot
+	  be solely relied on. res_hep_pjsip and res_hep_rtcp now also check this
+	  function to see if they should continue to load; if it fails, they
+	  decline their load as well.
+
+	  ASTERISK-26096 #close
+
+	  Change-Id: I007e535fcc2e51c2ca48534f48c5fc2ac38935ea
+
+2016-06-08 02:11 +0000 [784c18128b]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: No rtpmap for static RTP payload IDs in SDP.
+
+	  This saves around 100 bytes when G.711, G.722, G.729, and GSM are advertised in
+	  SDP. This reduces the chance to hit the MTU bearer of 1300 bytes for SIP over
+	  UDP, if many codecs are allowed in Asterisk. This new feature is enabled
+	  together with the optional feature compactheaders=yes via the file sip.conf.
+
+	  ASTERISK-25578 #close
+
+	  Change-Id: I16491b1937862de26f84fa0ffe679a6bab925044
+
+2016-06-02 12:04 +0000 [31a5c28339]  Joshua Colp <jcolp@digium.com>
+
+	* res_odbc: Implement a connection pool.
+
+	  Testing has shown that our usage of UnixODBC is problematic
+	  due to bugs within UnixODBC itself as well as the heavy weight
+	  cost of connecting and disconnecting database connections, even
+	  when pooling is enabled.
+
+	  For users of UnixODBC 2.3.1 and earlier crashes would occur due
+	  to insufficient protection of the disconnect operation. This was
+	  fixed in UnixODBC 2.3.2 and above.
+
+	  For users of UnixODBC 2.3.3 and higher a slow-down would occur
+	  under heavy database use due to repeated connection establishment.
+	  A regression is present where on each connection the database
+	  configuration is cached again, with the cache growing out of
+	  control.
+
+	  The connection pool implementation present in this change helps
+	  to mitigate these issues by reducing how much we connect and
+	  disconnect database connections. We also solve the issue of
+	  crashes under UnixODBC 2.3.1 by defaulting the maximum number of
+	  connections to 1, returning us to the previous working behavior.
+	  For users who may have a fixed version the maximum concurrent
+	  connection limit can be increased helping with performance.
+
+	  The connection pool works by keeping a list of active connections.
+	  If the connection limit has not been reached a new connection is
+	  established. If the connection limit has been reached then the
+	  request waits until a connection becomes available before
+	  continuing.
+
+	  ASTERISK-26074 #close
+	  ASTERISK-26054 #close
+
+	  Change-Id: I6774bf4bac49a0b30242c76a09c403d2e856ecff
+
+2016-05-31 09:10 +0000 [80ff7912a1]  Vasil Kolev <vasil.kolev@securax.org>
+
+	* chan_sip: bigger buffers for headers, better failure mode
+
+	  Currently chan_sip can give weird messages if the contacts don't
+	  fit in the From: or To: headers. This fix changes the from,to and
+	  invite variables to use ast_str, allocates and deallocates them and
+	  resizes them if needed.
+
+	  ASTERISK-26069 #close
+
+	  Change-Id: I1b68fcbddca6f6cc7d7a92fe1cb0d5430282b2b3
+
+2016-06-06 11:13 +0000 [60caebc738]  Örn Arnarson <orn@arnarson.net>
+
+	* apps/app_voicemail.c and main/say.c: Add support for Icelandic language
+
+	  Icelandic has some weird grammar rules when dealing with dates and
+	  numbers. There are different genders used depending on which number
+	  you're dealing with, and only a handful of numbers do change depending
+	  on the gender. There is also an implied gender in several cases.
+
+	  This patch was originally written for asterisk 1.6, and has been in use
+	  for several years without crashes. I cleaned it up a bit and rewrote
+	  what was necessary for Asterisk 13.
+
+	  The functions were copied from other similar languages and modified
+	  where appropriate. If i recall correctly, the German and Danish
+	  functions were used as a base.
+
+	  ASTERISK-26087
+	  Reported by: Örn Arnarson
+	  Tested by: Örn Arnarson
+
+	  Change-Id: Ib7d8bd7b0fede5767921ed821315b5b508c0e665
+
+2016-06-07 05:45 +0000 [52120204c9]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_srtp: Instead of libSRTP use OpenSSL as random source.
+
+	  Since libSRTP 1.5, its Random Number Generator (RNG) is not maintained anymore.
+	  Therefore, the symbol RAND_bytes is used instead of crypto_get_random.
+
+	  ASTERISK-24436 #close
+
+	  Change-Id: Iea0bae4d4e3c9aa0926ea442b6484b5159789d96
+
+2016-06-07 02:16 +0000 [da943ec5c0]  Alexander Traud <pabstraud@compuserve.com>
+
+	* BuildSystem: Avoid 'ar cru' and use 'ar cr' instead.
+
+	  In several internal library projects, the files are archived with the help of
+	  'ar cr'. Only the projects editline and the Objective Open H.323 stack
+	  implementation in C (ooh323c) use 'ar cru' instead. Recently, some platforms
+	  changed the default parameters of AR which creates "/usr/bin/ar: `u' modifier
+	  ignored since `D' is the default (see `U')". For consistency and to avoid this
+	  message all projects use 'ar cr' now.
+
+	  ASTERISK-26091 #close
+
+	  Change-Id: I710a9b1c01c1b5a1931a646098c044c8161ead40
+
+2016-06-01 16:57 +0000 [dca052e531]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_rtp.c: Simplify options to UnicastRTP channel creation.
+
+	  Change the awkward and not as flexible UnicastRTP options format
+	  From:
+	  Dial(UnicastRTP/127.0.0.1[/[<engine>][/[<codec>]]])
+	  To:
+	  Dial(UnicastRTP/127.0.0.1[/[<options>]])
+
+	  Where <options> can be standard Asterisk flag options:
+	  c(<codec>) - Specify which codec/format to use such as 'ulaw'.
+	  e(<engine>) - Specify which RTP engine to use such as 'asterisk'.
+
+	  More option flags can be easily added later such as the codec's RTP
+	  payload type to use when the codec does not have a static payload type
+	  defined.
+
+	  Change-Id: I0c297aaf09e2ee515536cb7437bb8042ff8ff3c9
+
+2016-05-02 05:57 +0000 [5bfef2a8b4]  Jaco Kroon <jaco@uls.co.za>
+
+	* translate: Enables native Packet-Loss Concealment (PLC) for supporting codecs.
+
+	  ASTERISK-25629 #close
+
+	  Change-Id: Ibfcf0670e094e9718d82fd9920f1fb2dae122006
+
+2016-05-25 10:34 +0000 [3e8d523d88]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* core/dial: New channel variable FORWARDERNAME
+
+	  Added a new channel variable FORWARDERNAME which indicates which
+	  channel was responsible for a forwarding requests received on dial attempt.
+
+	  Fixed a bug in the app_queue: FORWARD_CONTEXT is not used.
+
+	  ASTERISK-26059 #close
+
+	  Change-Id: I34e93e8c1b5e17776a77b319703c48c8ca48e7b2
+
+2016-05-27 14:49 +0000 [a2f820e8dc]  gtjoseph <gjoseph@digium.com>
+
+	* ari/resource_channels:  Add 'formats' to channel create/originate
+
+	  If you create a local channel and don't specify an originator channel
+	  to take capabilities from, we automatically add all audio formats to
+	  the new channel's capabilities. When we try to make the channel
+	  compatible with another, the "best format" functions pick the best
+	  format available, which in this case will be slin192.  While this is
+	  great for preserving quality, it's the worst for performance and
+	  overkill for the vast majority of applications.
+
+	  In the absense of any other information, adding all formats is the
+	  correct thing to do and it's not always possible to supply an
+	  originator so a new parameter 'formats' has been added to the channel
+	  create/originate functions. It's just a comma separated list of formats
+	  to make availalble for the channel. Example: "ulaw,slin,slin16".
+	  'formats' and 'originator' are mutually exclusive.
+
+	  To facilitate determination of format names, the format name has been
+	  added to "core show codecs".
+
+	  ASTERISK-26070 #close
+
+	  Change-Id: I091b23ecd41c1b4128d85028209772ee139f604b
+
+2016-06-03 01:33 +0000 [538c6415c6]  Timo Teräs <timo.teras@iki.fi>
+
+	* chan_sip: Support auth username for callbackextension feature
+
+	  ASTERISK-20527 #close
+
+	  Change-Id: I659cf7f00836a09d09d146ad226a40477d731239
+
+2016-06-03 00:39 +0000 [797695c5cc]  Timo Teräs <timo.teras@iki.fi>
+
+	* Make use of GLOB_BRACE and GLOB_NOMAGIC optional
+
+	  These flags are non-portable GNU extensions. Make their use
+	  optional. This fixes complication error on e.g. musl c-library
+	  based systems.
+
+	  Change-Id: I0aa06efc62aa8995f091445c8b762a75a91042f3
+
+2016-06-02 14:57 +0000 [3c1fec8099]  Timo Teräs <timo.teras@iki.fi>
+
+	* Fix res_search usage
+
+	  Resolver state is not part of res_search API. This fixes
+	  compilation error:
+
+	  dns.c:261:8: error: too many arguments to function 'res_search'
+	    ret = res_search(&dns_state,
+
+	  Change-Id: Ia600a58557040df83f744da3dde23225293845a5
+
+2016-06-02 14:53 +0000 [9c1d95e873]  Timo Teräs <timo.teras@iki.fi>
+
+	* Fix #include poll.h and sys/cdefs.h
+
+	  POSIX defines poll.h, sys/poll.h should not be used at is c-library
+	  internal header which may or may not exist. Notable in musl it
+	  generates warning of being incorrect. And add explict include of
+	  sys/cdefs.h where needed.
+
+	  Change-Id: I142930df53fe7585a06b854b6faddc5301e024be
+
+2016-05-25 08:45 +0000 [8a5c2e736c]  Niklas Larsson <niklas@tese.se>
+
+	* core/manager: Add uptime field to FullyBooted
+
+	  Add Uptime and LastReload to event FullyBooted.
+
+	  ASTERISK-26058 #close
+	  Reported by: Niklas Larsson
+
+	  Change-Id: I909b330801c0990d78df9b272ab0adc95aecb15e
+
+2016-06-02 04:59 +0000 [4505a59dc9]  Joshua Colp <jcolp@digium.com>
+
+	* alembic: Fix migration.
+
+	  The 81b01a191a46_pjsip_add_contact_reg_server.py script was attempting
+	  to use UniqueConstraint and failing. It was not imported and after
+	  importing it also continued to fail.
+
+	  I've changed the script to use the explicit name of the constraint
+	  instead.
+
+	  Change-Id: I2438b0be90b7ce583b47dd27983c0c1a02cea5b9
+
+2016-06-01 13:57 +0000 [40d19f2e55]  Richard Mudgett <rmudgett@digium.com>
+
+	* logging,cdr,cel: Fix stringfield memory leak.
+
+	  The stringfields refactor to allow adding stringfields to the end of a
+	  structure (f6f4cf459f43f072604927209b39646f84aaa2e2) exposed some
+	  incomplete cleanup code by some stringfield users.
+
+	  The most noticeable leaker is the logging system where there is a leak for
+	  every log message generated.
+
+	  ASTERISK-26078 #close
+	  Reported by:  Etienne Lessard
+	  Patches:
+	        jira_asterisk_26078_v13.patch (license #5621) patch uploaded
+	        by Richard Mudgett
+
+	  Change-Id: If6a08b31336b492c3de6f9dfd07c447f8d5a8782
+
+2016-05-31 13:02 +0000 [aec7916595]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Use correct rdata info access method (Part 2).
+
+	  The pjproject doxygen for rdata->msg_info.info says to call
+	  pjsip_rx_data_get_info() instead of accessing the struct member directly.
+	  You need to call the function mostly because the function will generate
+	  the struct member value if it is not already setup.
+
+	  Change-Id: I4d519385a577f3e9d9193a88125e493cf17fa799
+
+2016-05-09 15:00 +0000 [205a31f86c]  Mark Michelson <mmichelson@digium.com>
+
+	* Expand the scope of Dial Events
+
+	  Dial events up to this point have come in two flavors
+	  * A Dial event with no status to indicate that dialing has begun
+	  * A Dial event with a status to indicate that dialing has ended
+
+	  With this change, Dial events have been expanded to also give
+	  intermediate events, such as "RINGING", "PROCEEDING", and "PROGRESS".
+	  This is especially useful for ARI dialing, as it gives the application
+	  writer the opportunity to place a channel into an early bridge when
+	  early media is detected.
+
+	  AMI handles these in-progress dial events by sending a new event called
+	  "DialState" that simply indicates that dial state has changed but has
+	  not ended. ARI never distinguished between DialBegin and DialEnd, so no
+	  change was made to the event itself.
+
+	  Another change here relates to dial forwards. A forward-related event
+	  was previously only sent when a channel was successfully able to forward
+	  a call to a new channel. With this set of changes, if forwarding is
+	  blocked, we send a Dial event with a forwarding destination but no
+	  forwarding channel, since we were prevented from creating one. This is
+	  again useful for ARI since application writers can now handle call
+	  forward attempts from within their own application.
+
+	  ASTERISK-25925 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I42cbec7730d84640a434d143a0d172a740995543
+
+2016-05-30 19:27 +0000 [8a6a14590d]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip_mwi_body_generator:  Re-order the body items
+
+	  Re-ordered the body items so Message-Account is second.
+
+	  Messages-Waiting: no
+	  Message-Account: sip:1571@<IP Removed>:5060
+	  Voice-Message: 0/0 (0/0)
+
+	  ASTERISK-26065 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: If5d35a64656eac98c2dd5e490cc0b2807bed80c3
+
+2016-05-30 10:58 +0000 [7fa5766752]  gtjoseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Move to pjproject 2.5
+
+	  Although all the patches we had against 2.4.5 were applied by Teluu,
+	  a new bug was introduced preventing re-use of tcp and tls transports
+	  This patch removes all the previous patches against 2.4.5, updates
+	  the version to 2.5, and adds a new patch to correct the transport
+	  re-use problem.
+
+	  Change-Id: I0dc6c438c3910f7887418a5832ca186aea23d068
+
+2016-05-27 12:25 +0000 [b56f611856]  Rusty Newton <rnewton@digium.com>
+
+	* res_pjsip: Add clarifying documentation to PJSIP_HEADER help text
+
+	  Added notes about when you can read or write headers. Specifically
+	  about being able to read on the inbound channel and write on an
+	  outbound channel.
+
+	  ASTERISK-26063 #close
+	  Reported by: Private Name
+	  Tested by: Rusty Newton
+
+	  Change-Id: Ibeb64af17d1f6451028b3c29855a3f151a01d8c5
+
+2016-05-26 15:14 +0000 [bb0f4a6310]  Mark Michelson <mmichelson@digium.com>
+
+	* multicast RTP: Add dialing options
+
+	  This adds a new parameter to the end of a multicast RTP dialing string.
+	  This parameter defines the following options:
+
+	  * i: Set the interface from which multicast RTP is sent
+	  * l: Set whether multicast packets are looped back to the sender
+	  * t: Set the TTL for multicast packets
+	  * c: Set the codec to use for RTP
+
+	  ASTERISK-26068 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I033b706b533f0aa635c342eb738e0bcefa07e219
+
+2016-05-09 14:48 +0000 [88d997913f]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Re-implement the ARI dial command, allowing for early bridging.
+
+	  ARI dial had been implemented using the Dial API. This made great sense
+	  when dialing was 100% separate from bridging. However, if a channel were
+	  to be added to a bridge during the dial attempt, there would be a
+	  conflict between the dialing thread and the bridging thread. Each would
+	  be attempting to read frames from the dialed channel and act on them.
+
+	  The initial attempt to make the two play nice was to have the Dial API
+	  suspend the channel in the bridge and stay in charge of the channel
+	  until the dial was complete. The problem with this was that it was
+	  riddled with potential race conditions. It also was not well-suited for
+	  the case where the channel changed which bridge it was in during the
+	  dial.
+
+	  This new approach removes the use of the Dial API altogether. Instead,
+	  the channel we are dialing is placed into an invisible ARI dialing
+	  bridge. The bridge channel thread handles incoming frames from the
+	  channel. If the channel is added to a real bridge, it is departed from
+	  the invisible bridge and then added to the real bridge. Similarly, if
+	  the channel is removed from the real bridge, it is automatically added
+	  back to the invisible bridge if the dial attempt is still active.
+
+	  This approach keeps the threading simple by always having the channel
+	  being handled by bridge channel threads.
+
+	  ASTERISK-25925
+
+	  Change-Id: I7750359ddf45fcd45eaec749c5b3822de4a8ddbb
+
+2016-05-19 14:56 +0000 [31f17abe44]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: add "via_addr", "via_port", "call_id" to contact
+
+	  As res_pjsip_nat rewrites contact's address, only the last Via header
+	  can contain the source address of registered endpoint.
+	  Also Call-Id header may contain the source address of registered
+	  endpoint.
+
+	  Added "via_addr", "via_port", "call_id" to contact.
+	  Added new fields ViaAddress, CallID to AMI event ContactStatus.
+
+	  ASTERISK-26011
+
+	  Change-Id: I36bcc0bf422b3e0623680152d80486aeafe4c576
+
+2016-05-24 16:56 +0000 [574c9e77eb]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: chatty verbose messages
+
+	  There are a lot of verbose messages about Endpoint and Contact status
+	  changes if there are many dynamic endpoints.
+	  The patch sets verbose level 2 for Endpoint status changes
+	  and verbose level 3 for Contact status changes.
+
+	  ASTERISK-26055 #close
+
+	  Change-Id: Ie64e261ddbbc41bfff0f0190241152cc123fe6d7
+
+2016-05-20 13:56 +0000 [b3142e99e4]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* app_voicemail: fix bugs, imap mm_status log change to debug
+
+	  Fixed some bugs:
+	  - create dirpath when save downloading message from IMAP storage.
+	  - create IMAP folder if not exists when saving to IMAP storage
+	  - check if file successfully opened before write to it
+	  - some IMAP checks
+	  - remove non-standard flag 'Unseen'
+	  etc
+
+	  Change to debug IMAP mm_status log instead of verbose.
+
+	  Remove unused X-Asterisk-VM-Caller-channel message header
+	  for security reason. The clients should not know name of peer/endpoint.
+
+	  ASTERISK-26045 #close
+
+	  Change-Id: I7f83d88b69b36934e2539c114b9fb612deed971b
+
+2016-05-25 18:30 +0000 [7d44d12816]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_distributor.c: Use correct rdata info access method.
+
+	  The pjproject doxygen for rdata->msg_info.info says to call
+	  pjsip_rx_data_get_info() instead of accessing the struct member directly.
+	  You need to call the function mostly because the function will generate
+	  the struct member value if it is not already setup.
+
+	  Change-Id: Iafe8b01242b7deb0ebfdc36685e21374a43936d2
+
+2016-05-03 11:11 +0000 [1d60bfcdf1]  Tzafrir Cohen <tzafrir@debian.org>
+
+	* followme: allow disabling callee prompt
+
+	  Add the option 'enable_callee_prompt' to followme.conf. Enabled by
+	  default. If disabled, a callee is not prompted to accept or reject
+	  the forwarded call.
+
+	  ASTERISK-26064 #close
+
+	  Change-Id: I0a8b19d4cf95c86a07c992813babb9e4a4acfff5
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-02-12 09:59 +0000 [80ff2c2540]  Corey Farrell <git@cfware.com>
+
+	* threadpool: Fix potential data race.
+
+	  worker_start checked for ZOMBIE status without holding a lock.  All
+	  other read/write of worker status are performed with a lock, so this
+	  check should do the same.
+
+	  ASTERISK-25777 #close
+
+	  Change-Id: I5e33685a5c26fdb300851989a3b82be8c4e03781
+
+2016-05-24 05:28 +0000 [070eab6ed2]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_outbound_publish: Ensure publish is valid when explicitly destroying.
+
+	  Recent changes to res_pjsip_outbound_publish have introduced a
+	  race condition at shutdown where an outbound publish may be shutdown
+	  twice. In this case the first succeeds as a result of the unpublish.
+	  In the second invocation since it's been unpublished a task is
+	  queued to just destroy the client. This task holds no ref to the
+	  publish and as a result the publish may be destroyed before the
+	  task is run, causing a crash.
+
+	  This explicit destruction task now holds a reference to the publish
+	  to ensure it remains valid.
+
+	  ASTERISK-26053 #close
+
+	  Change-Id: I10789b98add3e50292ee3b33a55a1d9061cec94b
+
+2016-05-09 14:27 +0000 [f6c33771f6]  Mark Michelson <mmichelson@digium.com>
+
+	* Bridging: introduce "invisible" bridges.
+
+	  Invisible bridges function the same as normal bridges, but they have the
+	  following restrictions:
+
+	  * They never show up in CLI, AMI, or ARI queries.
+	  * They do not have Stasis messages published about them.
+
+	  Invisible bridges' main use is for when use of the bridging system is
+	  desired, but the bridge should not be known to users of the Asterisk
+	  system.
+
+	  ASTERISK-25925
+
+	  Change-Id: I804a209d3181d7c54e3d61a60eb462e7ce0e3670
+
+2016-05-22 11:03 +0000 [85d0272e76]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Only check transaction on transaction state events.
+
+	  The send request callback function currently assumes that it
+	  will only ever be called on transaction state changes. This is
+	  not always true. If our own timer callback occurs we will call
+	  the callback with a timer event instead of a transaction state
+	  change event. In this case the transaction on the event is
+	  invalid and accessing it will result in a crash.
+
+	  ASTERISK-26049 #close
+
+	  Change-Id: I623211c8533eb73056b0250b4580b49ad4174dfc
+
+2016-05-21 05:42 +0000 [31897d2d99]  Jesper (License 5518)
+
+	* func_curl: Don't trim response text on non-ASCII characters
+
+	  The characters 0x80-0xFF were trimmed as well as 0x00-0x20 because of
+	  a signed comparison.
+
+	  ASTERISK-25669 #close
+	  Reported by: Jesper
+	  patches:
+	    strings.curl.trim.patch submitted by Jesper (License 5518)
+
+	  Change-Id: Ia51e169f24e3252a7ebbaab3728630138ec6f60a
+
+2016-05-20 19:03 +0000 [2a77af9ed0]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_rtp.c: Cleanup ast_request() parameter parsing.
+
+	  * Fixed NULL crash potential if parameters are missing.
+
+	  * Reordered some operations so further diagnostic messages can be
+	  more helpful.
+
+	  Change-Id: Ibbdc67a2496508cbfbfef0cf19c35177ae2fbd70
+
+2016-05-20 16:59 +0000 [ade5275a3e]  Richard Mudgett <rmudgett@digium.com>
+
+	* parking.h: Update ast_parking_park_call() doxygen to reality.
+
+	  ASTERISK-26029
+
+	  Change-Id: I2db14d102a48d3224010e6d1c69e856373cc1260
+
+2016-05-12 15:18 +0000 [c378b00a83]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* func_odbc: single database connection should be optional
+
+	  func_odbc was changed in Asterisk 13.9.0
+	  to make func_odbc use a single database connection per DSN
+	  because of reported bug ASTERISK-25938
+	  with MySQL/MariaDB LAST_INSERT_ID().
+
+	  This is drawback in performance when func_odbc is used
+	  very often in dialplan.
+
+	  Single database connection should be optional.
+
+	  ASTERISK-26010
+
+	  Change-Id: I7091783a7150252de8eeb455115bd00514dfe843
+
+2016-05-20 09:39 +0000 [1c02b19b79]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Match dialogs on responses better.
+
+	  When receiving an incoming response to a dialog-starting INVITE, we were
+	  not matching the response to the INVITE dialog. Since we had not
+	  recorded the to-tag to the dialog structure, the PJSIP-provided method
+	  to find the dialog did not match.
+
+	  Most of the time, this was not a problem, because there is a fall-back
+	  that makes the response get routed to the same serializer that the
+	  request was sent on. However, in cases where an asynchronous DNS lookup
+	  occurs in the PJSIP core, the thread that sends the INVITE is not
+	  actually a threadpool serializer thread. This means we are unable to
+	  record a serializer to handle the incoming response.
+
+	  Now, imagine what happens when an INVITE is sent on a non-serialized
+	  thread, and an error response (such as a 486) arrives. The 486 ends up
+	  getting put on some random threadpool thread. Eventually, a hangup task
+	  gets queued on the INVITE dialog serializer. Since the 486 is being
+	  handled on a different thread, the hangup task can execute at the same
+	  time that the 486 is being handled. The hangup task assumes that it is
+	  the sole owner of the INVITE session and channel, so it ends up
+	  potentially freeing the channel and NULLing the session's channel
+	  pointer. The thread handling the 486 can crash as a result.
+
+	  This change has the incoming response match the INVITE transaction, and
+	  then get the dialog from that transaction. It's the same method we had
+	  been using for matching incoming CANCEL requests. By doing this, we get
+	  the INVITE dialog and can ensure that the 486 response ends up being
+	  handled by the same thread as the hangup, ensuring that the hangup runs
+	  after the 486 has been completely handled.
+
+	  ASTERISK-25941 #close
+	  Reported by Javier Riveros
+
+	  Change-Id: I0d4cc5d07e2a8d03e9db704d34bdef2ba60794a0
+
+2016-05-18 06:19 +0000 [e773e3a9bb]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Add the ability to download the media associated with a stored recording
+
+	  This patch adds a new feature to ARI that allows a client to download
+	  the media associated with a stored recording. The new route is
+	  /recordings/stored/{name}/file, and transmits the underlying binary file
+	  using Asterisk's HTTP server's underlying file transfer facilities.
+
+	  Because this REST route returns non-JSON, a few small enhancements had
+	  to be made to the Python Swagger generation code, as well as the
+	  mustache templates that generate the ARI bindings.
+
+	  ASTERISK-26042 #close
+
+	  Change-Id: I49ec5c4afdec30bb665d9c977ab423b5387e0181
+
+2016-05-19 11:41 +0000 [40cb032009]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_astdb: Filter fields to only the registered ones.
+
+	  This change introduces the same filtering that is done in res_sorcery_realtime
+	  to the res_sorcery_astdb module. This allows persisted sorcery objects
+	  that may contain unknown fields to still be read in from the AstDB
+	  and used. This is particularly useful when switching between different
+	  versions of Asterisk that may have introduced additional fields.
+
+	  ASTERISK-26014 #close
+
+	  Change-Id: Ib655130485a3ccfd635b7ed5546010ca14690fb2
+
+2016-05-09 21:40 +0000 [9766a12b4c]  snuffy <snuffy22@gmail.com>
+
+	* res_pjsip_empty_info: Respond to empty SIP INFO packets
+
+	  Some SBCs require responses to empty SIP INFO packets
+	  after establishing call via INVITE, if not responded to
+	  they may drop your call after unspecified timeout of X minutes.
+
+	  They are identified by having no Content-Type, check for this
+	  and respond with 200 - OK message.
+
+	  ASTERISK-24986 #close
+	  Reported-by: Ilya Trikoz, Federico Santulli
+
+	  Change-Id: Ib27e4f07151e5aef28fa587e4ead36c5b87c43e0
+
+2016-05-18 10:58 +0000 [111c4b0324]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* Makefile: remove OSARCH check for init install
+
+	  There are more specific checks for the platform.
+
+	  Specifically this allows installing OS/X init scripts.
+
+	  ASTERISK-26038 #close
+
+	  Change-Id: If08933621145b10362a0cfe73c079301d9c13f50
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-10 11:28 +0000 [d4b77dad1b]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_exten_state: Use the extension for publishing to.
+
+	  This change uses the newly added multi-user support for
+	  outbound publish to publish to the specific user that an
+	  extension state change is for.
+
+	  This also extends the res_pjsip_outbound_publish support
+	  to include the user specific From and To URI information in
+	  the outbound publishing of extension state. Since the URI
+	  is used when constructing the body it is important to ensure
+	  that the correct local and remote URIs are used.
+
+	  Finally the max string growths for the dialog-info+xml
+	  body generator has been increased as through testing it has
+	  proven to be too conservative.
+
+	  ASTERISK-25965
+
+	  Change-Id: I668fdf697b1e171d4c7e6f282b2e1590f8356ca1
+
+2016-05-03 16:07 +0000 [3905997bae]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publish: Add multi-user support per configuration
+
+	  Added a new multi_user option that when specified allows a particular
+	  configuration to be used for multiple users. It does this by replacing
+	  the user portion of the server uri with a dynamically created one.
+
+	  Two new API calls have been added in order to make use of the new
+	  functionality:
+
+	  ast_sip_publish_user_send - Sends an outgoing publish message based on the
+	  given user. If state for the user already exists it uses that, otherwise
+	  it dynamically creates new outbound publishing state for the user at that
+	  time.
+
+	  ast_sip_publish_user_remove - Removes all outbound publish state objects
+	  associated with the user. This essentially stops outbound publishing for
+	  the user.
+
+	  ASTERISK-25965 #close
+
+	  Change-Id: Ib88dde024cc83c916424645d4f5bb84a0fa936cc
+
+2016-05-18 07:54 +0000 [6e5e84458f]  gtjoseph <gjoseph@digium.com>
+
+	* udptl:  Don't eat sequence numbers until OK is received
+
+	  Scenario:
+	  Local fax -> Asterisk w/ firewall -> Provider -> Remote fax
+
+	  * Local fax starts rtp call to remote fax
+	  * Remote fax starts t38 call back to local fax.
+	  * Local fax sends t38 no-signal to Asterisk before sending an OK.
+	  * udptl processes the frame and increments the expected sequence number.
+	  * chan_sip drops the frame because the call isn't up so nothing goes out
+	    the external interface to open the port for incoming packets.
+	  * Local fax sends OK and Asterisk sends OK to the remote fax.
+	  * Remote fax sends t38 packets which are dropped by the firewall.
+	  * Local fax re-sends t38 no-signal with the same sequence number.
+	  * udptl drops the frame because it thinks it's a dup.
+	  * Still no outgoing packets to open the firewall.
+	  * t38 negotiation fails.
+
+	  The patch drops frames t38 received before udptl sequence processing
+	  when the call hasn't been answered yet.  The second no-signal frame
+	  is then seen as new and is relayed out the external interface which
+	  opens the port and allows negotiation to continue.
+
+	  ASTERISK-26034 #close
+
+	  Change-Id: I11744b39748bd2ecbbe8ea84cdb4f3c5943c5af9
+
+2016-05-15 12:22 +0000 [52148d93f4]  Matt Jordan <mjordan@digium.com>
+
+	* CHANGES: Update formatting of items
+
+	  * Provide consistent indenting of lines in bulleted paragraphs
+	  * Respect the 80 character column width
+	  * Group all like items together, e.g., all dialplan applications under
+	    "Applications", etc.
+	  * Use a single blank line to break up functionality changes within a
+	    larger section
+	  * Use two blanks lines to delineate larger sections
+
+	  Change-Id: I0488554f5cb7c51da70003d69288a21c9aab9647
+
+2016-04-18 18:17 +0000 [03d88b5656]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Add the ability to play multiple media URIs in a single operation
+
+	  Many ARI applications will want to play multiple media files in a row to
+	  a resource. The most common use case is when building long-ish IVR prompts
+	  made up of multiple, smaller sound files. Today, that requires building a
+	  small state machine, listening for each PlaybackFinished event, and triggering
+	  the next sound file to play. While not especially challenging, it is tedious
+	  work. Since requiring developers to write tedious code to do normal activities
+	  stinks, this patch adds the ability to play back a list of media files to a
+	  resource.
+
+	  Each of the 'play' operations on supported resources (channels and bridges)
+	  now accepts a comma delineated list of media URIs to play. A single Playback
+	  resource is created as a handle to the entire list. The operation of playing
+	  a list is identical to playing a single media URI, save that a new event,
+	  PlaybackContinuing, is raised instead of a PlaybackFinished for each non-final
+	  media URI. When the entire list is finished being played, a PlaybackFinished
+	  event is raised.
+
+	  In order to help inform applications where they are in the list playback, the
+	  Playback resource now includes a new, optional attribute, 'next_media_uri',
+	  that contains the next URI in the list to be played.
+
+	  It's important to note the following:
+	   - If an offset is provided to the 'play' operations, it only applies to the
+	     first media URI, as it would be weird to skip n seconds forward in every
+	     media resource.
+	   - Operations that control the position of the media only affect the current
+	     media being played. For example, once a media resource in the list
+	     completes, a 'reverse' operation on a subsequent media resource will not
+	     start a previously completed media resource at the appropiate offset.
+	   - This patch does not add any new operations to control the list. Hopefully,
+	     user feedback and/or future patches would add that if people want it.
+
+	  ASTERISK-26022 #close
+
+	  Change-Id: Ie1ea5356573447b8f51f2e7964915ea01792f16f
+
+2016-05-17 11:14 +0000 [5bd1bf2816]  gtjoseph <gjoseph@digium.com>
+
+	* chan_sip:  Prevent extra Session-Expires headers from being added
+
+	  When chan_sip does a re-INVITE to refresh a session and authentication
+	  is required, the INVITE with the Authorization header containes a
+	  second Session-Expires header without the ";refersher=" parameter.
+	  This is causing some proxies to return a 400.  Also, when Asterisk is
+	  the uas and the refresher, it is including the Session-Expires and
+	  Min-SE headers in OPTIONS messages which is not allowed per RFC4028.
+
+	  This patch (based on the reporter's) Checks to see if a Session-Expires
+	  header is already in the message before adding another one.  It also
+	  checks that the method is INVITE or UPDATE.
+
+	  ASTERISK-26030 #close
+
+	  Change-Id: I58a7b07bab5a3177748d8a7034fb8ad8e11ce1d9
+
+2016-05-16 15:29 +0000 [ae81b55361]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip_outbound_registration:  Clean up state when registration is deleted
+
+	  Nothing was cleaning up the registration state object when ast_sorcery_delete
+	  was called on a registration.  So, the registration was deleted from sorcery
+	  but the state object went right on refreshing the registration (or failing
+	  to refresh the registration) with the peer.
+
+	  * Added a 'deleted' observer on registration that removes the state object.
+
+	  ASTERISK-25964 #close
+	  Reported-by Matt Jordan
+
+	  Change-Id: I2db792145cdb1f72ebbf57dd9099596dbbf12c23
+
+2016-05-15 19:05 +0000 [8b5cee4a4f]  gtjoseph <gjoseph@digium.com>
+
+	* res_pjsip:  Set TCP_NODELAY on TCP transports
+
+	  Although it's perfectly legal to place multiple SIP messages in the same packet,
+	  it can cause problems because the Linux default is to enable Path MTU Discovery
+	  which sets the Don't Fragment bit on the packets. If adding a second message to
+	  the packet causes the MTU to be exceeded, and the destination isn't equipped to
+	  send a FRAGMENTATION NEEDED response to a large packet, the packet will just be
+	  dropped.
+
+	  We can't specifically tell the stack to send only 1 message per packet, but we
+	  can turn on TCP_NODELAY when we create the transport. This will at least tell
+	  the stack to send packets as soon as possible.
+
+	  ASTERISK-26005 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: I820f23227183f2416ca5e393bec510e8fe1c8fbd
+
+2016-05-14 07:24 +0000 [3522376512]  Matt Jordan <mjordan@digium.com>
+
+	* logger: Support JSON logging with Verbose messages
+
+	  When 2d7a4a3357 was merged, it missed the fact that Verbose log messages
+	  are formatted and handled by 'verbosers'. Verbosers are registered
+	  functions that handle verbose messages only; they exist as a separate
+	  class of callbacks. This was done to handle the 'magic' that must be
+	  inserted into Verbose messages sent to remote consoles, so that the
+	  consoles can format the messages correctly, i.e., the leading
+	  tabs/characters.
+
+	  In reality, verbosers are a weird appendage: they're a separate class of
+	  formatters/message handlers outside of what handles all other log
+	  messages in Asterisk. After some code inspection, it became clear that
+	  simply passing a Verbose message along with its 'sublevel' importance
+	  through the normal logging mechanisms removes the need for verbosers
+	  altogether.
+
+	  This patch removes the verbosers, and makes the default log formatter
+	  aware that, if the log channel is a console log, it should simply insert
+	  the 'verbose magic' into the log messages itself. This allows the
+	  console handlers to interpret and format the verbose message
+	  themselves.
+
+	  This simplifies the code quite a lot, and should improve the performance
+	  of printing verbose messages by a reasonable factor:
+	  (1) It removes a number of memory allocations that were done on each
+	      verobse message
+	  (2) It removes the need to strip the verbose magic out of the verbose
+	      log messages before passing them to non-console log channels
+	  (3) It now performs fewer iterations over lists when handling verbose
+	      messages
+
+	  Since verbose messages are now handled like other log messages (for the
+	  most part), the JSON formatting of the messages works as well.
+
+	  ASTERISK-25425
+
+	  Change-Id: I21bf23f0a1e489b5102f8a035fe8871552ce4f96
+
+2016-05-14 21:48 +0000 [a1803cb5f4]  Matt Jordan <mjordan@digium.com>
+
+	* configs/samples/pjsip.conf.sample: Fix typo
+
+	  A ':' is not a valid token for starting a comment.
+
+	  Change-Id: I123592d93a83d1bdde3e352822881eb9da85e5ad
+
+2016-05-12 07:08 +0000 [d29c17834c]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_hep_pjsip: Fix reported local IP address when bound to 'any'
+
+	  When bound to an 'any' address, e.g., 0.0.0.0, PJSIP reports as its
+	  local address the 'any' address, as opposed to the IP address we
+	  actually received the packet on. This can cause some confusion in Homer,
+	  as it will dutifully report what we send it.
+
+	  This patch uses the PJSIP inspection routines to determine which IP
+	  address we probably received the packet on based on the remote party's
+	  IP address. In the event that this fails, it falls back to the IP
+	  address natively reported by the transport.
+
+	  Change-Id: I076f835d2aef489e1ee1d01595b211eb2ce62da3
+
+2016-05-14 12:29 +0000 [14938184a3]  Sean Bright <sean.bright@gmail.com>
+
+	* res_ari: Correct Location headers returned by some ARI resources
+
+	  The Location headers returned by:
+
+	   * /bridges/{bridgeId}/play
+	   * /bridges/{bridgeId}/record
+	   * /channels/{channelId}/play
+	   * /channels/{channelId}/record
+
+	  Did not have the '/ari' prefix, and in the case of the 'play' resources, were
+	  using 'playback' instead of 'playbacks.'
+
+	  Change-Id: I957c58a3a1471bf477dae7c67faa1b74fcd9241c
+
+2016-05-11 20:17 +0000 [e06a23681c]  Matt Jordan <mjordan@digium.com>
+
+	* res_hep: Provide an option to pick the UUID type
+
+	  At one point in time, it seemed like a good idea to use the Asterisk
+	  channel name as the HEP correlation UUID. In particular, it felt like
+	  this would be a useful identifier to tie PJSIP messages and RTCP
+	  messages together, along with whatever other data we may eventually send
+	  to Homer. This also had the benefit of keeping the correlation UUID
+	  channel technology agnostic.
+
+	  In practice, it isn't as useful as hoped, for two reasons:
+	  1) The first INVITE request received doesn't have a channel. As a
+	     result, there is always an 'odd message out', leading it to be
+	     potentially uncorrelated in Homer.
+	  2) Other systems sending capture packets (Kamailio) use the SIP Call-ID.
+	     This causes RTCP information to be uncorrelated to the SIP message
+	     traffic seen by those capture nodes.
+
+	  In order to support both (in case someone is trying to use res_hep_rtcp
+	  with a non-PJSIP channel), this patch adds a new option, uuid_type, with
+	  two valid values - 'call-id' and 'channel'. The uuid_type option is used
+	  by a module to determine the preferred UUID type. When available, that
+	  source of a correlation UUID is used; when not, the more readily available
+	  source is used.
+
+	  For res_hep_pjsip:
+	   - uuid_type = call-id: the module uses the SIP Call-ID header value
+	   - uuid_type = channel: the module uses the channel name if available,
+	                          falling back to SIP Call-ID if not
+	  For res_hep_rtcp:
+	   - uuid_type = call-id: the module uses the SIP Call-ID header if the
+	                          channel type is PJSIP and we have a channel,
+	                          falling back to the Stasis event provided
+	                          channel name if not
+	   - uuid_type = channel: the module uses the channel name
+
+	  ASTERISK-25352 #close
+
+	  Change-Id: Ide67e59a52d9c806e3cc0a797ea1a4b88a00122c
+
+2016-05-13 11:46 +0000 [69a85a519f]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: Endpoint IP Access Controls
+
+	  With the old SIP module we can use IP access controls per peer.
+	  PJSIP module missing this feature.
+
+	  This patch added next configuration Endpoint options:
+	      "acl" - list of IP ACL section names in acl.conf
+	      "deny" - List of IP addresses to deny access from
+	      "permit" - List of IP addresses to permit access from
+	      "contact_acl" - List of Contact ACL section names in acl.conf
+	      "contact_deny" - List of Contact header addresses to deny
+	      "contact_permit" - List of Contact header addresses to permit
+
+	  This patch also better logging failed request:
+	      add custom message instead of "No matching endpoint found"
+	      add SIP method to logging
+
+	  ASTERISK-25900
+
+	  Change-Id: I456dea3909d929d413864fb347d28578415ebf02
+
+2016-05-12 14:36 +0000 [fd3f70598d]  Mark Michelson <mmichelson@digium.com>
+
+	* Use doubles instead of floats for conversions when comparing strings.
+
+	  In 13.9.0, there was an issue where PJSIP contacts added to an AOR would
+	  be deleted at seemingly random times.
+
+	  One reason this was happening was because of an operation to retrieve
+	  the contacts whose expiration time was less than or equal to the current
+	  time. When retrieving existing contacts, the contact's expiration time
+	  and the current time were converted from a string to a float, and those
+	  two floats were compared.
+
+	  On some systems, including mine, this conversion was horribly off. For
+	  instance, I could regularly see the string "1463079214" get converted
+	  into 1463079168.000000. When switching from using a float to using a
+	  double, the conversion was as expected.
+
+	  Why was the conversion to float off? My best guess is that the
+	  conversion to float was attempting to store the entire value in the 23
+	  bit significand of the IEEE-754 floating point number. In particular, if
+	  you take only the 23 most significant bits of 1463079214, you get the
+	  messed up 1463079168 that we were seeing in the conversion. It likely
+	  was possible to get a more precise value by composing the number using
+	  an exponent, but the conversion did not work that way. With a double,
+	  you have a 52 bit significand, allowing the entire value to fit there,
+	  and thereby allowing an accurate conversion.
+
+	  ASTERISK-26007 #close
+	  Reported by Greg Siemon
+
+	  Change-Id: I83ca7944aae8b7cd994b254c78ec02411d321070
+
+2016-05-12 09:13 +0000 [4f8cfa0220]  gtjoseph <gjoseph@digium.com>
+
+	* pjsip_distributor:  Add missing newline to NOTICE
+
+	  There was a newline missing from the end of the "no matching endpoint" notice.
+
+	  Change-Id: Idc11fe5bc0354072291663dbffe648c471e39181
+
+2016-05-10 10:19 +0000 [d14d1ba826]  Sebastian Damm <damm@sipgate.de>
+
+	* res_pjsip_outbound_registration: generate correct Contact URI for TLS
+
+	  There are two types of SIP URIs indicating a secure transport:
+	  * sips:user@example.org
+	  * sip:user@example.org;transport=tls
+
+	  When using a sips URI, Asterisk checks incoming INVITEs and answers from
+	  the other side for sips URIs, and rejects the packet if there are only
+	  sip URIs. So Asterisk should only generate a sips Contact URI if the
+	  other side supports it.
+
+	  This patch makes Asterisk generate either a sip or sips Contact URI
+	  depending on the format of the server URI.
+
+	  If you want a sip URI, use:
+	  server_uri=sip:example.org\;transport=tls
+
+	  If you want a sips URI, use:
+	  server_uri=sips:example.org
+
+	  ASTERISK-25990 #close
+	  Reported-by: Sebastian Damm
+
+	  Change-Id: I5ae57d6531ce940b5fc64d5cd2673e60db0f9ba2
+
+2016-05-05 16:41 +0000 [9f996624b0]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* logger: Add PID to syslog messages.
+
+	  During refactoring of this support the addition of
+	  the PID to messages was removed. This change adds it
+	  back in.
+
+	  ASTERISK-25538 #close
+
+	  Change-Id: Ie2d43b0652e59b7ac319a7dba94501540d70ba36
+
+2016-05-11 14:07 +0000 [5236ffed97]  Matt Jordan <mjordan@digium.com>
+
+	* configure: Fix errors with AST_UNDEFINED_SANITIZER/AST_LEAK_SANITIZER
+
+	  When running on a system that does not support or use AST_UNDEFINED_SANITIZER
+	  or AST_LEAK_SANITIZER, the configure script would incorrectly set those
+	  constants to a blank value, e.g., 'AST_UNDEFINED_SANITIZER='. This would
+	  cause menuselect to error out, complaining that a blank value is not a
+	  valid option. This patch corrects the issue by setting the value to 0 if
+	  the options that those constants enable/disable is not found.
+
+	  Change-Id: Ib39814aaf940f308d500c1e026edb3d70de47fba
+
+2016-05-10 08:17 +0000 [b5c471b339]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* followme: delete the right recorded name file
+
+	  FollowMe with the option a records the name of the caller and plays it
+	  to the callee. However it has failed to clean up that recorded file
+	  as it tried to delete the file name without the '.sln' extension.
+
+	  ASTERISK-26008 #close
+
+	  Change-Id: I79d7b1be7d5cde57bf076d9389e2a8a4422776ec
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-10 03:10 +0000 [ec85ea3c21]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* basic-cfg: asterisk.conf: don't set languages
+
+	  * No need to set language in a miniml configuration. 'en' will do just
+	    fine.
+	  * It would be useful to have an example of setting it to a different
+	    language.
+	  * Setting the documentation language explicitly is likewise not
+	    required. Setting it to a different value is not common. At least
+	    until there is a set of translated documentation.
+
+	  Change-Id: I94d91ea34e129925f25af81ef8dc0906fb568cb7
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-10 03:08 +0000 [1b0a9bb2c4]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* basic-cfg: asterisk.conf: debug level 5 spams
+
+	  Don't suggest users to use debug level 5, which spews (usually
+	  non-useful) debug information. Reduce the suggestion to (an
+	  arbitrarily-selected) level 2.
+
+	  Change-Id: Ib53195f78945970956ff59ef13fa89b90e0fcd60
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-10 03:06 +0000 [d0ba3e8196]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* basic-cfg: asterisk.conf: defaults of options
+
+	  Note the default of remmed-out options. To clarify that those values are
+	  not the defaults.
+
+	  Change-Id: I849c29b7a710f0abc37355fcb5bfee335ae30738
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-10 02:56 +0000 [f943a1fd84]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+	* basic-cfg: asterisk.conf: remove [directories]
+
+	  A minimal configuration does not need to explicitly spell out the
+	  directories. The built-in defaults will do just fine. In many cases
+	  they are wrong.
+
+	  Change-Id: Id1a671e5c5e9923765a4156b57f9f7e263fdd26c
+	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+
+2016-05-05 11:37 +0000 [1e876d6915]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_authenticator_digest: Don't use source port in nonce verification
+
+	  From the issue reporter:
+	  "res_pjsip_outbound_authenticator_digest builds a nonce that is a hash of
+	  the timestamp, the source address, the source port, a server UUID that is
+	  calculated at startup, and the authentication realm.
+
+	  Rather than caching nonces that we create, we instead attempt to re-calculate
+	  the nonce when receiving an incoming request with authentication. We then
+	  compare the re-calculated nonce to the incoming nonce, and if they don't match,
+	  then authentication has failed early.
+
+	  The problem is that it is possible, especially when using TCP, to receive two
+	  requests from the same endpoint but have differing source ports for those
+	  requests. Asterisk itself commonly will use different source ports for
+	  outbound TCP requests."
+
+	  This patch removes the source port dependency when building the nonce.
+
+	  ASTERISK-25978 #close
+
+	  Change-Id: I871b5f4adce102df1c4988066283095ec509dffe
+
+2016-05-07 14:39 +0000 [dfefbf8731]  gtjoseph <gjoseph@digium.com>
+
+	* config_transport:  Tell pjproject to allow all SSL/TLS protocols
+
+	  The default tls settings for pjproject only allow TLS 1, TLS 1.1 and TLS 1.2.
+	  SSL is not allowed.   So, even if you specify "sslv3" for a transport method,
+	  it's silently ignored and one of the TLS protocols is used.  This was a new
+	  behavior of pjsip_tls_setting_default() in 2.4 (when tls.proto was added) that
+	  we never caught.
+
+	  Now we need to set tls.proto = 0 after we call pjsip_tls_setting_default().
+	  This tells pjproject to set the socket protocol to match the method.
+
+	  ASTERISK-26004 #close
+
+	  Change-Id: Icfb55c1ebe921298dedb4b1a1d3bdc3ca41dd078
+
+2016-05-05 09:14 +0000 [d03e170ae7]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_pubsub: Use common datastores container API.
+
+	  This migrates res_pjsip_pubsub over to using the newly
+	  introduce common datastores management API instead of using
+	  its own implementations for both subscriptions and
+	  publications.
+
+	  As well the extension state data now provides a generic
+	  datastores container instead of a subscription. This allows
+	  the dialog-info+xml body generator to work for both
+	  subscriptions and publications.
+
+	  ASTERISK-25999 #close
+
+	  Change-Id: I773f9e4f35092da0f653566736a8647e8cfebef1
+
+2016-05-05 09:12 +0000 [94cd351ec4]  Joshua Colp <jcolp@digium.com>
+
+	* datastore: Add common container based datastores API.
+
+	  This change introduces a common container based datastores
+	  management API. This has been done in a few places across
+	  the tree but this consolidates all of the logic into one
+	  place in a generic fashion.
+
+	  ASTERISK-25999
+
+	  Change-Id: I72eb15941dcdbc2a37bb00a33ce00f8755bd336a
+
+2016-05-04 02:40 +0000 [8923c9ac96]  Jaco Kroon <jaco@uls.co.za>
+
+	* app_confbridge: Add a regcontext option for confbridge bridge profiles.
+
+	  This patch allows for having app_confbridge register the name of the
+	  conference as an extension into a specific context, similar to
+	  regcontext for chan_sip.  This variant is not quite as involved as the
+	  one in chan_sip and doesn't allow for multiple contexts or custom
+	  extensions, you can only specify the context and the conference name
+	  will always be used as the extension to register.
+
+	  ASTERISK-25989 #close
+
+	  Change-Id: Icacf94d9f2b5dfd31ef36f6cb702392619a7902f
+
+2016-05-08 20:19 +0000 [facce6f632]  gtjoseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Check for python-dev and TEST_FRAMEWORK
+
+	  The pjsua and pjsystest apps are now built only if TEST_FRAMEWORK is set.
+	  The python bindings are now built only if TEST_FRAMEWORK is set and a
+	  python development package is installed.
+
+	  libresample was also disabled.
+
+	  ASTERISK-25993 #close
+	  Reported-by: Joshua Colp
+
+	  Change-Id: If4e91c503a02f113d5b71bc8b972081fa3ff6f03
+
+2016-05-06 11:54 +0000 [322c3b4262]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: module load priority
+
+	  The res_pjsip_authenticator_digest, res_pjsip_endpoint_identifier_*
+	  and res_pjsip_registrar modules should load ASAP
+	  to avoid "No matching endpoint found" for legitimate endpoint.
+
+	  ASTERISK-25994
+
+	  Change-Id: Iac95d95ad031e0be104189d29e923a2ad7c24a1b
+
+2016-05-05 15:16 +0000 [516f49f316]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* stasis_endpoints: Add new Status and Headers to ContactStatus
+
+	  ASTERISK-25903 added a new headers to AMI Event ContactStatusDetail.
+	  ASTERISK-25904 added a new Status to AMI Event ContactStatusDetail.
+	  These additions should be also in stasis_endpoints
+	  to include in command "manager show event ContactStatus"
+
+	  Change-Id: I7610ad02a998e1f26c20caa27aa50279d0164f6a
+
+2016-05-03 15:43 +0000 [64e058f75a]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publish: state potential dropped on reloads/realtime fetches
+
+	  When reloading, or fetching realtime data, if the "apply" failed for any
+	  numerous reasons the current state object would not be maintained. This
+	  potentially resulted in publishes being stopped for some states/clients when
+	  they should not have been.
+
+	  This patch makes it so the current state object is kept upon any type of reload/
+	  fetch failures.
+
+	  Change-Id: Iab6020c116d628ed2ae81183e987e2eaa3c90b30
+
+2016-05-03 15:35 +0000 [adc82a2260]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publishing: After unloading the library won't load again
+
+	  The same thing was happening in res_pjsip_publish_asterisk. When the library
+	  was unloaded it did not unregister the object type from sorcery. Subsequent
+	  loads resulted in a failed load due to the sorcery type already existing.
+
+	  Change-Id: Ifdc25e94e4cd40bc5a19eb4d0a00b86c2e9fedc9
+
+2016-05-03 15:39 +0000 [3b0ce5169d]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publish: Won't unload if condition wait times out
+
+	  When res_pjsip_outbound_publish unloads it has to wait for all current
+	  publishing objects to get done. However if the wait condition times out
+	  then it does not fail the unload. This sometimes results in an infinite
+	  loop check while unloading. This patch now fails the unload operation if
+	  the condition times out.
+
+	  Change-Id: Id57b8cbed9d61222690fcba1e4f18e259df4c7ec
+
+2016-05-03 14:59 +0000 [41fccbfeb1]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publish: Ref leak in off nominal callback paths
+
+	  There were a few spots where the client object's reference was being leaked in
+	  sip_outbound_publish_callback. This patch cleans up those leaks.
+
+	  Change-Id: I485d0bc9335090f373026f77c548042e258461df
+
+2016-05-03 15:31 +0000 [dfbb03cc8e]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_publish: Potential crash due to off nominal path
+
+	  It was possible for the explicit publish destroy function to be called without
+	  the pjsip client ever being initialized. This fix checks to make sure there is
+	  a client to destroy before attempting.
+
+	  Change-Id: I8eea1bfa3bd472149bfc255310be2a6248688f5c
+
+2016-05-05 05:07 +0000 [17b6ba49ef]  Joshua Colp <jcolp@digium.com>
+
+	* file: Ensure nativeformats remains valid for lifetime of use.
+
+	  It is possible for the nativeformats of a channel to change
+	  throughout its lifetime. As a result a user of it needs to either
+	  ensure the channel is locked when accessing the formats or keep
+	  a reference to the nativeformats themselves.
+
+	  This change fixes the file playback support so it keeps a
+	  reference to the nativeformats when accessing things.
+
+	  ASTERISK-25998 #close
+
+	  Change-Id: Ie45b65475e1481ddf05b874ee48f63e39fff8915
+
+2016-04-15 09:32 +0000 [cc4c5f5693]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: improve realtime performance
+
+	  This patch modified pjsip_options to retrieve only
+	  permament contacts for aor if the qualify_frequency is > 0
+	  and persisted contacts if the qualify_frequency is > 0.
+
+	  This patch also fixed a bug in res_sorcery_astdb.
+	  res_sorcery_astdb doesn't save object data retrived from astdb.
+
+	  ASTERISK-25826
+
+	  Change-Id: I1831fa46c4578eae5a3e574ee3362fddf08a1f05
+
+2016-05-02 16:52 +0000 [92f85fe766]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_fax/t38_gateway: Peer V.21 session is created on wrong channel
+
+	  The channel and peer V.21 sessions are created on the same channel now.
+	  The peer V.21 session should be created only on peer channel
+	  when one of channel can handle T.38.
+
+	  Also this patch enable debug for T.38 gateway session
+	  if global fax debug enabled.
+
+	  ASTERISK-25982
+
+	  Change-Id: I78387156ea521a77eb0faf170179ddd37a50430e
+
+2016-05-04 16:11 +0000 [4df48581f1]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* pjsip: Added "reg_server" to contacts (fixed alembic)
+
+	  ASTERISK-25931
+
+	  Change-Id: Icc4321a88f5c93ff809da3f372eebbf69c6a8549
+
+2016-05-04 03:17 +0000 [02f4ca1079]  Chris Trobridge <christ.trobridge@ultra-aep.com>
+
+	* config_options.c: Expand #ifdef to contain whole if statement.
+
+	  ASTERISK-25956 #close
+
+	  Change-Id: If6961ec54be276d5ab4f012ee7e7b420cb45de38
+
+2016-05-02 16:08 +0000 [380ac201ac]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_fax: add FAXMODE variable
+
+	  The app_fax set FAXMODE variable, but res_fax missing this feature.
+	  This patch add FAXMODE variable which is set to either "audio" or "T38".
+
+	  ASTERISK-25980
+
+	  Change-Id: Ie3dcbfb72cc681e9e267a60202f7fb8723a51b6b
+
+2016-05-02 05:56 +0000 [0c9faaee47]  Jean Aunis <jean.aunis@prescom.fr>
+
+	* app_chanspy: fix audiohook options in non read-only mode
+
+	  When option 'o' was not set, ChanSpy created its audiohook with the flag
+	  AST_AUDIOHOOK_MUTE_WRITE, which caused ChanSpy to listen audio from one
+	  direction only.
+
+	  ASTERISK-25866 #close
+
+	  Change-Id: I5c745855eea29a3fbc4e4aed0b0c0f53580535e0
+
+2016-04-07 16:33 +0000 [a4cfcda036]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip/AMI: add contact.updated event
+
+	  With the old SIP module AMI sends PeerStatus event on every
+	  successfully REGISTER requests, ie, on start registration,
+	  update registration and stop registration.
+
+	  With PJSIP AMI sends ContactStatus only when status is changed.
+	  Regarding registration:
+	  on start registration - Created
+	  on stop registration - Removed
+	  but on update registration nothing
+
+	  This patch added contact.updated event.
+
+	  ASTERISK-25904
+
+	  Change-Id: I8fad8aae9305481469c38d2146e1ba3a56d3108f
+
+2016-04-30 17:52 +0000 [e61716b774]  gtjoseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Various fixes discovered during testing of OSes
+
+	  For all OSes:
+	  * Disabled third-party codecs in pjproject and added
+	    '--disable-speex-codec --disable-speex-aec --disable-gsm-codec' to the
+	    configure options since we don't use the pjsip codec capability.
+
+	  FreeBSD:
+	  * Added FreeBSD support to install_prereq.
+	  * Changed pjproject/configure.m4 to use $GNU_MAKE instead of hardcoding "make".
+	  * Added __progname and environ to asterisk.exports.in.
+	  * Reverted the use of ldconfig to create shared library symlinks to ln.
+	  * Only enable epoll in pjproject if `uname -s` is Linux.
+	  * Added a patch to pjproject to take the name of the 'make' command from
+	    an environment variable if supplied.  This is needed for the python bindings.
+	    (merged by Teluu into pjproject trunk 5/3/2016)
+	  FreeBSD support isn't complete.  Still some general issues regarding
+	  make/gmake having nothing to do with pjproject.  With some handholding it DOES
+	  build successfully.
+
+	  CentOS:
+	  Added 'patch' and 'bzip2' to install_prereq PACKAGES_RH.
+	  CentOS 6/7 32/64 build and run the pjsip testsuite successfully.
+
+	  Ubuntu:
+	  No changes required.
+	  Ubuntu 15/16 32/64 build and run the pjsip testsuite successfully.
+
+	  Debian:
+	  No changes required.
+	  Debian 6/7/8 32/64 build and run the pjsip testsuite successfully.
+
+	  There will utimately be a follow-up patch to create an install_prereq for
+	  the testsuite as I've discovered a few missing requirements.
+
+	  ASTERISK-25968 #close
+
+	  Change-Id: I5756a07facfc63798115a5e73a8709382fe9259c
+
+2016-03-17 14:29 +0000 [080c6216b6]  Andrew Nagy <andrew.nagy@the159.com>
+
+	* app_voicemail: always copy dynamic struct to avoid race condition
+
+	  Voicemail email addresses can be corrupt or voicemail
+	  emails can end up being sent to the wrong email address if asterisk is
+	  reading voicemail.conf during a reload and processing an email at the
+	  same time. This patch always copies the struct that would otherwise only
+	  be copied once.
+
+	  ASTERISK-24463 #close
+	  Reported by: John Campbell
+	  Tested by: Etienne Lessard
+	  Tested by: Andrew Nagy
+	  Change-Id: I3a0643813116da84e2617291903d0d489b7425fb
+
+2016-04-15 14:26 +0000 [2b1edee772]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* pjsip: Added "reg_server" to contacts.
+
+	  If the Asterisk system name is set in asterisk.conf, it will be stored
+	  into the "reg_server" field in the ps_contacts table to facilitate
+	  multi-server setups.
+
+	  ASTERISK-25931
+
+	  Change-Id: Ia8f6bd2267809c78753b52bcf21835b9b59f4cb8
+
+2016-05-01 02:21 +0000 [bf13b59062]  Diederik de Groot <dkgroot@talon.nl>
+
+	* configs/basic-pbx/asterisk.conf: contains incorrect path separator
+
+	  Note: When packagers use these files (as an example) the paths are never
+	  really used when they are split using '='.
+
+	  Note: Thirdparty applications will also have trouble parsing the file when
+	  expecting '=>'.
+
+	  Change-Id: I0ada647f588e81f023fb1333ca15a1a333fd6004
+
+2016-04-27 17:19 +0000 [2c46063d54]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_exten_state: Create PUBLISH messages.
+
+	  Create PUBLISH messages to update a third party when an extension state
+	  changes because of either a device or presence state change.
+
+	  A configuration example:
+
+	  [exten-state-publisher]
+	  type=outbound-publish
+	  server_uri=sip:instance1@172.16.10.2
+	  event=presence
+	  ; Optional regex for context filtering, if specified only extension state
+	  ; for contexts matching the regex will cause a PUBLISH to be sent.
+	  @context=^users
+	  ; Optional regex for extension filtering, if specified only extension
+	  ; state for extensions matching the regex will cause a PUBLISH to be sent.
+	  @exten=^[0-9]*
+	  ; Required body type for the PUBLISH message.
+	  ;
+	  ; Supported values are:
+	  ; application/pidf+xml
+	  ; application/xpidf+xml
+	  ; application/cpim-pidf+xml
+	  ; application/dialog-info+xml (Planned support but not yet)
+	  @body=application/pidf+xml
+
+	  The '@' extended variables are used because the implementation can't
+	  extend the outbound publish type as it is provided by the outbound publish
+	  module.  That means you either have to use extended variables, or
+	  implement some sort of custom extended variable thing in the outbound
+	  publish module.  Another option would be to refactor that stuff to have an
+	  option which specifies the use of an alternate implementation's
+	  configuration and then have that passed to the implementation.  JColp
+	  opted for the extended variables method originally.
+
+	  ASTERISK-25972 #close
+
+	  Change-Id: Ic0dab4022f5cf59302129483ed38398764ee3cca
+
+2016-04-26 16:10 +0000 [0b5292525c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_exten_state: Check if body generator is available.
+
+	  When starting the extension state publishers, check if the requested
+	  message body generator is available.  If not available give error message
+	  and skip starting that publisher.
+
+	  * res_pjsip_pubsub.c: Create new API if type/subtype generator
+	  registered.
+
+	  * res_pjsip_exten_state.c: Use new body generator API for validation.
+
+	  ASTERISK-25922
+
+	  Change-Id: I4ad69200666e3cc909d4619e3c81042d7f9db25c
+
+2016-04-28 11:35 +0000 [369182d084]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Start body generator users after suppliers.
+
+	  Change-Id: I8f0b57841feaab56c8a4e821b5ccb4e05e5fbadb
+
+2016-04-28 16:06 +0000 [3af83ea2fb]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Add useful information to some messages.
+
+	  Change-Id: Ia0b2e15773894c599e5c5748bbc70e99f434192a
+
+2016-04-26 15:58 +0000 [8e1b663b87]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Fix body generator registration race.
+
+	  Change-Id: Id8752073ef06472a2fd96080f4009fac42843e67
+
+2016-04-28 16:54 +0000 [30415944a8]  gtjoseph <gjoseph@digium.com>
+
+	* pjproject_bundled:  Disable PJSIP_UNESCAPE_IN_PLACE
+
+	  When pjsip_parse_uri is called with PJSIP_UNESCAPE_IN_PLACE enabled,
+	  the input uri string will become corrupted if it contains escape sequences.
+	  It's not possible to automatically strdup or strdupa the input string because
+	  the output uri pj_str_t's will have pointers to chunks of the input string.
+	  Getting around this would require more memory management code and wouldn't
+	  be worth the savings of doing the unescape in place.
+
+	  ASTERISK-25970 #close
+	  Reported-by: Dmitriy Serov
+
+	  Change-Id: I28dc0e599b5108f7959b9c46dc8278371b372f88
+
+2016-04-26 15:13 +0000 [906ea2c43f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.h: Fix doxygen association.
+
+	  Change-Id: I110d3e3572598289fcd4215d966cf0c858f98632
+
+2016-04-25 16:00 +0000 [76ea4cfaae]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_publish.c: Remove redundant flag check.
+
+	  Change-Id: I0da80a3c3e0eae0c52ff27e7412ba027d6f52353
+
+2016-03-07 18:34 +0000 [4ebf9a938d]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Add ability to identify by Authorization username
+
+	  A feature of chan_sip that service providers relied upon was the ability to
+	  identify by the Authorization username.  This is most often used when customers
+	  have a PBX that needs to register rather than identify by IP address.  From my
+	  own experiance, this is pretty common with small businesses who otherwise
+	  don't need a static IP.
+
+	  In this scenario, a register from the customer's PBX may succeed because From
+	  will usually contain the PBXs account id but an INVITE will contain the caller
+	  id.  With nothing recognizable in From, the service provider's Asterisk can
+	  never match to an endpoint and the INVITE just stays unauthorized.
+
+	  The fixes:
+
+	  A new value "auth_username" has been added to endpoint/identify_by that
+	  will use the username and digest fields in the Authorization header
+	  instead of username and domain in the the From header to match an endpoint,
+	  or the To header to match an aor.  This code as added to
+	  res_pjsip_endpoint_identifier_user rather than creating a new module.
+
+	  Although identify_by was always a comma-separated list, there was only
+	  1 choice so order wasn't preserved.  So to keep the order, a vector was added
+	  to the end of ast_sip_endpoint.  This is only used by res_pjsip_registrar
+	  to find the aor.  The res_pjsip_endpoint_identifier_* modules are called in
+	  globals/endpoint_identifier_order.
+
+	  Along the way, the logic in res_pjsip_registrar was corrected to match
+	  most-specific to least-specific as res_pjsip_endpoint_identifier_user does.
+
+	  The order is:
+
+	  username@domain
+	  username@domain_alias
+	  username
+
+	  Auth by username does present 1 problem however, the first INVITE won't have
+	  an Authorization header so the distributor, not finding a match on anything,
+	  sends a securty_alert.  It still sends a 401 with a challenge so the next
+	  INVITE will have the Authorization header and presumably succeed.  As a result
+	  though, that first security alert is actually a false alarm.
+
+	  To address this, a new feature has been added to pjsip_distributor that keeps
+	  track of unidentified requests and only sends the security alert if a
+	  configurable number of unidentified requests come from the same IP in a
+	  configurable amout of time.  Those configuration options have been added to
+	  the global config object.  This feature is only used when auth_username
+	  is enabled.
+
+	  Finally, default_realm was added to the globals object to replace the hard
+	  coded "asterisk" used when an endpoint is not yet identified.
+
+	  The testsuite tests all pass but new tests are forthcoming for this new
+	  feature.
+
+	  ASTERISK-25835 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: I30ba62d208e6f63439600916fcd1c08a365ed69d
+
+2016-04-27 13:23 +0000 [2b150f0b80]  Mark Michelson <mmichelson@digium.com>
+
+	* func_odbc: Check connection status before executing queries.
+
+	  A recent change to func_odbc made it so that a single connection was
+	  maintained per DSN. The problem was that the code was optimistic about
+	  the health of the connection after initially opening it and did nothing
+	  to re-connect in case the connection had died.
+
+	  This change adds a check before executing a query to ensure that the
+	  connection to the database is still up and running.
+
+	  ASTERISK-25963 #close
+	  Reported by Ross Beer
+
+	  Change-Id: Id33c86eb04ff48ca088bb2e3086c27b3b683491d
+
+2016-04-15 11:59 +0000 [860b135c88]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: disable multi domain to improve realtime performace
+
+	  This patch added new global pjsip option 'disable_multi_domain'.
+	  Disabling Multi Domain can improve Realtime performance by reducing
+	  number of database requests.
+
+	  ASTERISK-25930 #close
+
+	  Change-Id: I2e7160f3aae68475d52742107949a799aa2c7dc7
+
+2016-04-01 07:50 +0000 [7281770710]  Jean Aunis <jean.aunis@prescom.fr>
+
+	* app_chanspy: reduce audio loss on the spying channel.
+
+	  ChanSpy was creating its audiohook with the flags AST_AUDIOHOOK_TRIGGER_SYNC
+	  and AST_AUDIOHOOK_SMALL_QUEUE, which caused audio frames to be lost when
+	  queues grow too large or when read and write queues go out of sync.
+	  Now these flags are set conditionally:
+	  - AST_AUDIOHOOK_TRIGGER_SYNC is not set if the option "o" is set
+	  - a new option "l" is created: if set, AST_AUDIOHOOK_SMALL_QUEUE will not
+	  be set on the audiohook
+
+	  ASTERISK-25866
+
+	  Change-Id: I9c7652f41d9fa72c8691e4e70ec4fd16b047a4dd
+
+2016-04-14 07:03 +0000 [81ea80b74c]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_exten_state: Add config support for exten state publishers.
+
+	  This change adds the ability to configure outbound publishing of
+	  extension state. Right now stuff is merely set up to store the
+	  configuration and to register a global extension state callback. The
+	  act of constructing the body and sending is not yet complete.
+
+	  Configurable elements right now are a regex for filtering the context,
+	  a regex for filtering the extension, and the body type to publish.
+
+	  ASTERISK-25922 #close
+
+	  Change-Id: Ia7e630136dfc355073c1cadff8ad394a08523d78
+
+2016-04-26 11:13 +0000 [c480159045]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip: Give more time for TCP/TLS threads to stop.
+
+	  The unload process currently tells each TCP/TLS to terminate but
+	  does not wait for them to do so. This introduces a race condition
+	  where the container holding the threads may be destroyed before
+	  the threads are able to remove themselves from it. When they
+	  finally do the container is invalid and can't be used causing a
+	  crash.
+
+	  A previous change existed which waited a bit to wait for any
+	  stranglers to finish. This change extends this and waits longer.
+
+	  ASTERISK-25961 #close
+
+	  Change-Id: Idc6262b670ca49ede32061159e323b7b63c6f3c6
+
+2016-04-26 05:48 +0000 [8ae69cffef]  Joshua Colp <jcolp@digium.com>
+
+	* app_queue: Fix crash when unloading module.
+
+	  When unloading the app_queue module the members in each queue are
+	  destroyed and as part of this they are removed from the pending
+	  members container. Unfortunately a crash would occur as the container
+	  was destroyed before the members were removed.
+
+	  This change tweaks ordering so the container destruction occurs
+	  after the members are destroyed.
+
+	  ASTERISK-16115
+
+	  Change-Id: I48c728668c55aee3d05b751a5d450fb57e87f44b
+
+2016-04-24 22:51 +0000 [284bb814ac]  gtjoseph <gjoseph@digium.com>
+
+	* config:  Fix ast_config_text_file_save2 writability check for missing files
+
+	  A patch I did back in 2014 modified ast_config_text_file_save2 to check the
+	  writability of the main file and include files before truncating and re-writing
+	  them.  An unintended side-effect of this was that if a file doesn't exist,
+	  the check fails and the write is aborted.
+
+	  This patch causes ast_config_text_file_save2 to check the writability of the
+	  parent directory of missing files instead of checking the file itself.  This
+	  allows missing files to be created again.  A unit test was also added to
+	  test_config to test saving of config files.
+
+	  The regression was discovered when app_voicemail's passwordlocation=spooldir
+	  feature stopped working.
+
+	  ASTERISK-25917 #close
+	  Reported-by: Jonathan Rose
+
+	  Change-Id: Ic4dbe58c277a47b674679e49daed5fc6de349f80
+
+2016-04-25 08:11 +0000 [f99ec857c8]  Javier Acosta <javier.acosta@beeonline.es>
+
+	* Fix case sensitive actions in AMI QueueSummary and QueueStatus
+
+	  ASTERISK-25954 #close
+	  Reported by: Javier Acosta
+
+	  Change-Id: I00be83d45cc7e8385de2523012bd196aafeeb256
+	  (cherry picked from commit c0688a6398f27296ff849848a2e416e036d794e3)
+
+2016-04-21 14:23 +0000 [30ab21d5fa]  Kevin Harwell <kharwell@digium.com>
+
+	* app_queue: queue members can receive multiple calls
+
+	  It was possible for a queue member that is a member of at least 2 or more
+	  queues to receive mulitiple calls at the same time. This happened because
+	  of a race between when a member was being rung and when the device state
+	  notified the other queue(s) member object of the state change.
+
+	  This patch makes it so when a queue member is being rung it gets added to
+	  a global pool of queue members. If that same member is tried again, e.g.
+	  from another queue, and it is found to already exist in the pending member
+	  container then it will not ring that member.
+
+	  ASTERISK-16115 #close
+
+	  Change-Id: I546dd474776d158c2b6be44205353dee5bac7e48
+
+2016-04-22 17:53 +0000 [99fcf2a791]  gtjoseph <gjoseph@digium.com>
+
+	* res_agi:  Prevent run_agi from eating frames it shouldn't
+
+	  The run_agi function is eating control frames when it shouldn't be. This is
+	  causing issues when an AGI is run from CONNECTED_LINE_SEND_SUB in a blond
+	  transfer.
+
+	  Alice calls Bob. Bob attended transfers to Charlie but hangs up before Charlie
+	  answers.
+
+	  Alice gets the COLP UPDATE indicating Charlie but Charlie never gets an UPDATE
+	  and is left thinking he's connected to Bob.
+
+	  In this case, when CONNECTED_LINE_SEND_SUB runs on Alice's channel and it calls
+	  an AGI, the extra eaten frames prevent CONNECTED_LINE_SEND_SUB from running on
+	  Charlie's channel.
+
+	  The fix was to accumulate deferrable frames in the "forever" loop instead of
+	  dropping them, and re-queue them just before running the actual agi command
+	  or exiting.
+
+	  ASTERISK-25951 #close
+
+	  Change-Id: I0f4bbfd72fc1126c2aaba41da3233a33d0433645
+
+2016-04-22 15:25 +0000 [757ec6172b]  Richard Mudgett <rmudgett@digium.com>
+
+	* test_message.c: Wait longer in case dialplan also processes the test message.
+
+	  Bumped the wait from 1 second to 5 seconds.  The test message was hitting my
+	  default call handler and failing the test because it took longer.
+
+	  Change-Id: I3a03737f25e92983de00548fcc7bbc50dd7544ba
+
+2016-04-21 23:53 +0000 [41ecf22587]  Kirill Katsnelson <kkm@smartaction.com>
+
+	* chan_sip: Make autocreated peers send PeerStatus events
+
+	  Since Stasis has been introduced, an attempt to send AMI messages by an
+	  autocreated peer caused a crash, and all events from autocreated peers were
+	  semi-inadvertently disabled altogether in 0b83761. This change restores the
+	  disabled functionality.
+
+	  ASTERISK-25950
+
+	  Change-Id: Iecc350f23db603fadb2f302064643ebe9664e974
+
+2016-04-13 17:09 +0000 [b3cc74fda9]  Richard Mudgett <rmudgett@digium.com>
+
+	* manager_channels.c: Fix allocation failure crash.
+
+	  An earlier allocation failure failed to create a channel snapshot for the
+	  AMI HangupRequest/SoftHangupRequest event which resulted in a crash in
+	  channel_hangup_request_cb().  Where the stasis message gets generated
+	  cannot tell if the NULL snapshot returned was because of an allocation
+	  failure or the channel was a dummy channel.
+
+	  * Made channel_hangup_request_cb() check if the channel blob has a
+	  snapshot and exit if it doesn't.
+
+	  * Eliminated the RAII_VAR usage in channel_hangup_request_cb().
+
+	  Change-Id: I0b6a1c4e95cbb7d80b2a7054c6eadecc169dfd24
+
+2016-04-13 13:50 +0000 [a63656b419]  Richard Mudgett <rmudgett@digium.com>
+
+	* Bridge system: Fix memory leaks and double frees on impart failure.
+
+	  You cannot reference the passed in features struct after calling
+	  ast_bridge_impart().  Even if the call fails.
+
+	  Change-Id: I902b88ba0d5d39520e670fb635078a367268ea21
+
+2016-04-13 13:20 +0000 [71dfa35540]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_softmix.c: Fix crash if channel fails to join mixing tech.
+
+	  softmix_bridge_join() failed because of an allocation failure.  To address
+	  this, the softmix bridge technology now checks if the channel failed to
+	  join softmix successfully.  In addition, the bridge now begins the process
+	  of kicking the channel out of the bridge so we don't have channels
+	  partially in the bridge for very long.
+
+	  * Fix the test_channel_feature_hooks.c unit tests.  The test channel must
+	  have a valid codec to join the simple_bridge technology.  This patch makes
+	  joining a bridge more strict by not allowing partially joined channels to
+	  remain in the bridge.
+
+	  Change-Id: I97e2ade6a2bcd1214f24fb839fda948825b61a2b
+
+2016-04-12 15:29 +0000 [06632a0d11]  Richard Mudgett <rmudgett@digium.com>
+
+	* Manager: Short circuit AMI message processing.
+
+	  Improve AMI message processing performance if there are no consumers
+	  listening for the messages.  We now skip creating the AMI event message
+	  text strings.
+
+	  Change-Id: I7b22fc5ec4e500d00635c1a467aa8ea68a1bb2b3
+
+2016-04-13 17:54 +0000 [6ddd856b86]  Richard Mudgett <rmudgett@digium.com>
+
+	* manager.c: Eliminate most RAII_VAR usage.
+
+	  * Made ast_manager_event_blob_create() not allocate the ao2 event object
+	  with a lock as it is not needed.
+
+	  Change-Id: I8e11bfedd22c21316012e0b9dd79f5918f644b7c
+
+2016-04-22 13:49 +0000 [924738e950]  Mark Michelson <mmichelson@digium.com>
+
+	* func_odbc: Use one connection per DSN.
+
+	  res_odbc was changed in Asterisk 13.8.0 to remove connection management,
+	  opting instead to let unixodbc maintain open connections and return
+	  those to Asterisk as requested.
+
+	  This was a boon for realtime, since it meant that multiple threads could
+	  potentially run parallel queries since they could each be using their
+	  own database connections.
+
+	  However, on the user-facing side, func_odbc, there were some inherent
+	  behaviors being relied on that no longer hold true after the change.
+	  One such reported behavior was that MySQL's LAST_INSERTED_ID() works
+	  per-connection. This means that if Asterisk uses separate connections
+	  for every database operation, whereas before it used one connection for
+	  everything, we have broken expectations and functionality.
+
+	  The fix provided in this patch is to make func_odbc use a single
+	  database connection per DSN. This way, user-facing database usage will
+	  have the same behavior as it did pre-13.8.0. However, realtime, which is
+	  the real workhorse of database interaction, will continue to let
+	  unixodbc manage connections.
+
+	  ASTERISK-25938 #close
+	  Reported by Edwin Vandamme
+
+	  Change-Id: Iac961fe79154c6211569afcdfec843c0c24c46dc
+
+2016-04-22 13:02 +0000 [6ede210c98]  Leif Madsen <leif@leifmadsen.com>
+
+	* Remove reference to non-existent sip.conf option
+
+	  Option was removed in commit 7f883ef495b57ae9182e47213d01d5e8009dbf3f
+
+	  ASTERISK-25927 #close
+
+	  Change-Id: I92f9b0196d9fc41d1d58354c07340c465ef1fcf8
+
+2016-04-21 08:26 +0000 [c991e5472e]  Diederik de Groot <dkgroot@talon.nl>
+
+	* lock.c: Check *lt before dereferencing it
+
+	  *lt is NULL if t->tracking == 0
+
+	  ASTERISK-25948 #close
+
+	  Change-Id: I4a81af28f9c82a74aa82413d772a7dc8fa6f45ba
+
+2016-04-15 14:36 +0000 [6b1a632290]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_stasis: Handle re-enter stasis bridge with swap channel.
+
+	  We lose the fact that there is a swap channel if there is one.  We
+	  currently wind up rejoining the stasis bridge as a normal join after the
+	  swap channel has already been kicked from the bridge.
+
+	  This patch preserves the swap channel so the AMI/ARI events can note that
+	  the channel joining the bridge is swapping with another channel.  Another
+	  benefit to swaqpping in one operation is if there are any channels that
+	  get lonely (MOH, bridge playback, and bridge record channels).  The lonely
+	  channels won't leave before the joining channel has a chance to come back
+	  in under stasis if the swap channel is the only reason the lonely channels
+	  are staying in the bridge.
+
+	  ASTERISK-25947 #close
+	  Reported by: Richard Mudgett
+
+	  ASTERISK-24649
+	  Reported by: John Bigelow
+
+	  ASTERISK-24782
+	  Reported by: John Bigelow
+
+	  Change-Id: If37ea508831d1fed6dbfac2f191c638fc0a850ee
+
+2016-04-19 16:58 +0000 [1c5248c383]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge: Hold off more than one imparting channel at a time.
+
+	  An earlier patch blocked the ast_bridge_impart() call until the channel
+	  either entered the target bridge or it failed.  Unfortuantely, if the
+	  target bridge is stasis and the imprted channel is not a stasis channel,
+	  stasis bounces the channel out of the bridge to come back into the bridge
+	  as a proper stasis channel.  When the channel is bounced out, that
+	  released the block on ast_bridge_impart() to continue.  If the impart was
+	  a result of a transfer, then it became a race to see if the swap channel
+	  would get hung up before the imparted channel could come back into the
+	  stasis bridge.  If the imparted channel won then everything is fine.  If
+	  the swap channel gets hung up first then the transfer will fail because
+	  the swap channel is leaving the bridge.
+
+	  * Allow a chain of ast_bridge_impart()'s to happen before any are
+	  unblocked to prevent the race condition described above.  When the channel
+	  finally joins the bridge or completely fails to join the bridge then the
+	  ast_bridge_impart() instances are unblocked.
+
+	  ASTERISK-25947
+	  Reported by: Richard Mudgett
+
+	  ASTERISK-24649
+	  Reported by: John Bigelow
+
+	  ASTERISK-24782
+	  Reported by: John Bigelow
+
+	  Change-Id: I8fef369171f295f580024ab4971e95c799d0dde1
+
+2016-04-19 17:52 +0000 [70e860ec49]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_callerid:  Clear out display name if id->name is not valid
+
+	  When create_new_id_hdr creates a new RPID or PAI header, it starts by cloning
+	  the From header, then it overwrites the display name and uri from the channel's
+	  connected.id.  If the connected.id.name wasn't valid, create_new_id_hdr was
+	  leaving the display name from the From header in the new RPID or PAI header.
+	  On an attended transfer where the originator had a caller id number set but not
+	  a display name, the re-INVITE to the final transferee had the number of the
+	  originator but the display name of the transferer.
+
+	  Added a check to clear out the display name in the new header if
+	  connected.id.name was invalid.
+
+	  ASTERISK-25942 #close
+
+	  Change-Id: I60b4bf7a7ece9b7425eba74151c0b4969cd2738b
+
+2016-04-19 13:02 +0000 [d95512a7dd]  Joshua Colp <jcolp@digium.com>
+
+	* app_talkdetect: Make the module core supported.
+
+	  This module is used as part of testsuite tests to confirm
+	  stuff works. I'm accordingly marking it as core as it is
+	  required by those tests.
+
+	  Change-Id: I558e7af7679b22b8ed641d7dd37ee4ca35b11e88
+
+2016-04-18 12:12 +0000 [0235a66532]  Mark Michelson <mmichelson@digium.com>
+
+	* PJSIP: Remove PJSIP parsing functions from uri length validation.
+
+	  The PJSIP parsing functions provide a nice concise way to check the
+	  length of a hostname in a SIP URI. The problem is that in order to use
+	  those parsing functions, it's required to use them from a thread that
+	  has registered with PJLib.
+
+	  On startup, when parsing AOR configuration, the permanent URI handler
+	  may not be run from a PJLib-registered thread. Specifically, this could
+	  happen when Asterisk was started in daemon mode rather than
+	  console-mode. If PJProject were compiled with assertions enabled, then
+	  this would cause Asterisk to crash on startup.
+
+	  The solution presented here is to do our own parsing of the contact URI
+	  in order to ensure that the hostname in the URI is not too long. The
+	  parsing does not attempt to perform a full SIP URI parse/validation,
+	  since the hostname in the URI is what is important.
+
+	  ASTERISK-25928 #close
+	  Reported by Joshua Colp
+
+	  Change-Id: Ic3d6c20ff3502507c17244a8b7e2ca761dc7fb60
+
+2016-04-18 17:00 +0000 [b8b60135ec]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_registrar: Fix bad memory-ness with user_agent.
+
+	  Recent changes to the PJSIP registrar resulted in tests failing due to
+	  missing AOR_CONTACT_ADDED test events. The reason for this was that the
+	  user_agent string had junk values in it, resulting in being unable to
+	  generate the event.
+
+	  I'm going to be honest here, I have no idea why this was happening. Here
+	  are the steps needed for the user_agent variable to get messed up:
+	  * REGISTER is received
+	  * First contact in the REGISTER results in a contact being removed
+	  * Second contact in the REGISTER results in a contact being added
+	  * The contact, AOR, expiration, and user agent all have to be passed as
+	    format parameters to the creation of a string. Any subset of those
+	    parameters would not be enough to cause the problem.
+
+	  Looking into what was happening, the thing that struck me as odd was
+	  that the user_agent variable was meant to be set to the value of the
+	  User-Agent SIP header in the incoming REGISTER. However, when removing a
+	  contact, the user_agent variable would be set (via ast_strdupa inside a
+	  loop) to the stored contact's user_agent. This means that the
+	  user_agent's value would be incorrect when attempting to process further
+	  contacts in the incoming REGISTER.
+
+	  The fix here is to use a different variable for the stored user agent
+	  when removing a contact. Correcting the behavior to be correct also
+	  means the memory usage is less weird, and the issue no longer occurs.
+
+	  ASTERISK-25929 #close
+	  Reported by Joshua Colp
+
+	  Change-Id: I7cd24c86a38dec69ebcc94150614bc25f46b8c08
+
+2016-04-18 13:41 +0000 [6cfa02394f]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_transport_management: Allow unload to occur.
+
+	  At shutdown it is possible for modules to be unloaded that wouldn't
+	  normally be unloaded. This allows the environment to be cleaned up.
+
+	  The res_pjsip_transport_management module did not have the unload
+	  logic in it to clean itself up causing the res_pjsip module to not
+	  get unloaded. As a result the res_pjsip monitor thread kept going
+	  processing traffic and timers when it shouldn't.
+
+	  Change-Id: Ic8cadee131e3b2c436a81d3ae8bb5775999ae00a
+
+2016-04-15 11:41 +0000 [6365f0018f]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_channel.c: Ignore role setup failure in channel push.
+
+	  We have to setup the channel roles after the bridge class push is called
+	  because the bridge class push callback may have set roles on the incoming
+	  channel.  Since we have already partially pushed the channel into the
+	  bridge and reversing what we have already done could be problematic, the
+	  only thing we can do is press on to complete pushing the channel into the
+	  bridge.
+
+	  * Ignore any channel role setup errors after pushing the channel into a
+	  bridge.  The channel may behave incorrectly in the bridge but we can no
+	  longer abort the push at this time.
+
+	  Change-Id: I08a97082b729052ee65cdca6bb730cf1289ede00
+
+2016-04-17 15:37 +0000 [f06ce7f90a]  Jaco Kroon <jaco@uls.co.za>
+
+	* chan_sip: Don't verify table if rtupdate=no
+
+	  If rtupdate=no do not verify sipregs/peers table has updatable fields.
+
+	  ASTERISK-25934 #close
+
+	  Change-Id: Iaa2c53037b93daccc7e7333c40d61861847b856d
+
+2016-04-18 04:53 +0000 [dbb47e0a47]  ibercom <ibercom123@gmail.com>
+
+	* app_queue: Frequent segfaults in function can_ring_entry()
+
+	  ASTERISK-25888 #close
+
+	  Change-Id: I007a2f2dd99823e04fb5be3ff01f02b0a2956117
+
+2016-04-15 16:51 +0000 [af114edb8b]  Richard Mudgett <rmudgett@digium.com>
+
+	* stasis_bridge.c: Update stasis bridge push diagnostic messages.
+
+	  Change-Id: I195b14994c9dcccb9452491ca20a885d2a54605a
+
+2016-04-12 14:55 +0000 [5e64d7e7a3]  Mark Michelson <mmichelson@digium.com>
+
+	* Dial: Combine frame handling functions.
+
+	  There is a good amount of repetition in the two frame handling routines
+	  in the Dial API. This commit combines the two functions into one.
+
+	  This is in preparation for an upcoming commit that adds the ability to
+	  handle frames for a channel in a bridge.
+
+	  ASTERISK-25925
+	  Reported by Mark Michelson
+
+	  Change-Id: Iaae2f174e3058e774cb44e10659fcdfb85345c58
+
+2016-04-11 16:20 +0000 [a6e2ba187a]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* Codecs: strip codec name while parsing allow/disallow options
+
+	  Failed registration using PJSIP/Realtime if one of the codec name
+	  in allow/disallow option is wrong or contains space.
+
+	  This patch strip codec name.
+
+	  ASTERISK-25914
+
+	  Change-Id: Ifdf02de94e5ddbce305640f6f0666084a3b9283d
+
+2016-04-14 13:49 +0000 [be4333ddad]  Mark Michelson <mmichelson@digium.com>
+
+	* transport management: Register thread with PJProject.
+
+	  The scheduler thread that kills idle TCP connections was not registering
+	  with PJProject properly and causing assertions if PJProject was built in
+	  debug mode.
+
+	  This change registers the thread with PJProject the first time that the
+	  scheduler callback executes.
+
+	  AST-2016-005
+
+	  Change-Id: I5f7a37e2c80726a99afe9dc2a4a69bdedf661283
+
+2016-03-17 12:28 +0000 [e83499df56]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Add serialized scheduler (res_pjsip/pjsip_scheduler.c)
+
+	  There are several places that do scheduled tasks or periodic housecleaning,
+	  each with its own implementation:
+
+	  * res_pjsip_keepalive has a thread that sends keepalives.
+	  * pjsip_distributor has a thread that cleans up expired unidentified requests.
+	  * res_pjsip_registrar_expire has a thread that cleans up expired contacts.
+	  * res_pjsip_pubsub uses ast_sched directly and then calls ast_sip_push_task.
+	  * res_pjsip_sdp_rtp also uses ast_sched to send keepalives.
+
+	  There are also places where we should be doing scheduled work but aren't.
+	  A good example are the places we have sorcery observers to start registration
+	  or qualify.  These don't work when changes are made to a backend database
+	  without a pjsip reload.  We need to check periodically.
+
+	  As a first step to solving these issues, a new ast_sip_sched facility has
+	  been created.
+
+	  ast_sip_sched wraps ast_sched but only uses ast_sched as a scheduled queue.
+	  When a task is ready to run, ast_sip_task_pusk is called for it. This ensures
+	  that the task is executed in a PJLIB registered thread and doesn't hold up the
+	  ast_sched thread so it can immediately continue processing the queue.  The
+	  serializer used by ast_sip_sched is one of your choosing or a random one from
+	  the res_pjsip pool if you don't choose one.
+
+	  Another feature is the ability to automatically clean up the task_data when the
+	  task expires (if ever).  If it's an ao2 object, it will be dereferenced, if
+	  it's a malloc'd object it will be freed.  This is selectable when the task is
+	  scheduled.  Even if you choose to not auto dereference an ao2 task data object,
+	  the scheduler itself maintains a reference to it while the task is under it's
+	  control.  This prevents the data from disappearing out from under the task.
+
+	  There are two scheduling models.
+
+	  AST_SIP_SCHED_TASK_PERIODIC specifies that the invocations of the task occur at
+	  the specific interval.  That is, every "interval" milliseconds, regardless of
+	  how long the task takes.  If the task takes longer than the interval, it will
+	  be scheduled at the next available multiple of interval.  For exmaple: If the
+	  task has an interval of 60 secs and the task takes 70 secs (it better not),
+	  the next invocation will happen at 120 seconds.
+
+	  AST_SIP_SCHED_TASK_DELAY specifies that the next invocation of the task should
+	  start "interval" milliseconds after the current invocation has finished.
+
+	  Also, the same ast_sched facility for fixed or variable intervals exists.  The
+	  task's return code in conjunction with the AST_SIP_SCHED_TASK_FIXED or
+	  AST_SIP_SCHED_TASK_VARIABLE flags controls the next invocation start time.
+
+	  One res_pjsip.h housekeeping change was made.  The pjsip header files were
+	  added to the top.  There have been a few cases lately where I've needed
+	  res_pjsip.h just for ast_sip calls and had compiles fail spectacularly because
+	  I didn't add the pjsip header files to my source even though I never referenced
+	  any pjsip calls.
+
+	  Finally, a few new convenience APIs were added to astobj2 to make things a
+	  little easier in the scheduler.  ao2_ref_and_lock() calls ao2_ref() and
+	  ao2_lock() in one go.  ao2_unlock_and_unref() does the reverse. A few macros
+	  were also copied from res_phoneprov because I got tired of having to duplicate
+	  the same hash, sort and compare functions over and over again. The
+	  AO2_STRING_FIELD_(HASH|SORT|CMP)_FN macros will insert functions suitable for
+	  aor_container_alloc into your source.
+
+	  This facility can be used immediately for the situations where we already have
+	  a thread that wakes up periodically or do some scheduled work.  For the
+	  registration and qualify issues, additional sorcery and schema changes would
+	  need to be made so that we can easily detect changed objects on a periodic
+	  basis without having to pull the entire database back to check.  I'm thinking
+	  of a last-updated timestamp on the rows but more on this later.
+
+	  Change-Id: I7af6ad2b2d896ea68e478aa1ae201d6dd016ba1c
+
+2016-03-08 12:12 +0000 [216f22fd0f]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_transport_management: Kill idle TCP connections.
+
+	  "Idle" here means that someone connects to us and does not send a SIP
+	  request. PJProject will not automatically time out such connections, so
+	  it's up to Asterisk to do it instead.
+
+	  When we receive an incoming TCP connection, we will start a timer
+	  (equivalent to transaction timer D) waiting to receive an incoming
+	  request. If we do not receive a request in that timeframe, then we will
+	  shut down the TCP connection.
+
+	  ASTERISK-25796 #close
+	  Reported by George Joseph
+
+	  AST-2016-005
+
+	  Change-Id: I7b0d303e5d140d0ccaf2f7af562071e3d1130ac6
+
+2016-03-08 10:52 +0000 [d9fba46016]  Mark Michelson <mmichelson@digium.com>
+
+	* Rename res_pjsip_keepalive res_pjsip_transport_management
+
+	  ASTERISK-25796
+	  Reported by George Joseph
+
+	  AST-2016-005
+
+	  Change-Id: Id322a05f927392293570599730050bc677d99433
+
+2016-04-14 07:23 +0000 [7b8b6e2e4f]  Mark Michelson <mmichelson@digium.com>
+
+	* AST-2016-004: Fix crash on REGISTER with long URI.
+
+	  Due to some ignored return values, Asterisk could crash if processing an
+	  incoming REGISTER whose contact URI was above a certain length.
+
+	  ASTERISK-25707 #close
+	  Reported by George Joseph
+
+	  Patches:
+	      0001-res_pjsip-Validate-that-URIs-don-t-exceed-pjproject-.patch
+
+	  AST-2016-004
+
+	  Change-Id: I3ea7cee16f29c8088794de3085ca7523c1c4833d
+
+2016-04-12 13:10 +0000 [ff3af764de]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_softmix.c: Fix crash if could not allocate the dsp.
+
+	  Fix off nominal crash where we could not setup the channel to process
+	  frames for the softmix bridge technology because of allocation failure.
+
+	  Change-Id: Ic307a8386e46bf551e48fcd1eb97276714d56372
+
+2016-04-13 13:38 +0000 [caa416d5f3]  gtjoseph <george.joseph@fairview5.com>
+
+	* stringfields:  Update extended string fields for master only.
+
+	  In 13, the new ast_string_field_header structure had to be dynamically
+	  allocated and assigned to a pointer in ast_string_field_mgr to preserve ABI
+	  compatability.  In master, it can be converted to being a structure-in-place in
+	  ast_string_field_mgr to eliminate the extra alloc and free calls.
+
+	  Change-Id: Ia97c5345eec68717a15dc16fe2e6746ff2a926f4
+
+2016-04-12 15:41 +0000 [bd3671b397]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject:  Add patch for removing strip of '[]' from header params
+
+	  From the patch submitted to Teluu on 4/12/2016
+	  <<<<<<<<<
+	  The wholesale stripping of '[]' from header parameters causes issues if
+	  something (like a port) occurs after the final ']'.
+
+	  '[2001:a::b]' will correctly parse to '2001:a::b'
+	  '[2001:a::b]:8080' will correctly parse to '2001:a::b' but the scanner is left
+	  with ':8080' and parsing stops with a syntax error.
+
+	  I can't even find a case where stripping the '[]' is a good thing anyway.  Even
+	  if you continued to parse and resulted in a string that looks like this...
+	  '2001:a::b:8080', it's not valid.
+
+	  This came up in Asterisk because Kamailio sends us a Contact with an alias
+	  URI parameter that has an IPv6 address in it like this:
+	  Contact: <sip:1171@127.0.0.1:5080;alias=[2001:1:2::3]~43691~6>
+	  which should be legal but causes a syntax error because of the characters
+	  after the final ']'.  Even if it didn't, the '[]' should still not be stripped.
+
+	  I've run the Asterisk Test Suite for PJSIP (252 tests) many of which are IPv6
+	  enabled.  No issues were caused by removing the code that strips the '[]'.
+	  >>>>>>>>>>>
+
+	  ASTERISK-25123 #close
+	  Reported-by: Anthony Messina
+
+	  Change-Id: I5cb33f4ebf07ee1f2b26d07caae715e2ec65595a
+
+2016-04-12 09:10 +0000 [5a0534dc62]  Joshua Colp <jcolp@digium.com>
+
+	* app_voicemail: Fix test_voicemail_notify_endl test.
+
+	  The test_voicemail_notify_endl test checks the end-of-line
+	  characters of an email message to confirm that they are consistent.
+	  The test wrongfully assumed that reading from the email message
+	  into a buffer will always result in more than 1 character being
+	  read. This is incorrect. If only 1 character was read the test
+	  would go outside of the buffer and access other memory causing
+	  a crash.
+
+	  The test now checks to ensure that 2 or more characters are read
+	  in ensuring the test stays within the buffer.
+
+	  ASTERISK-25874 #close
+
+	  Change-Id: Ic2c89cea6e90f2c0bc2d8138306ebbffd4f8b710
+
+2016-04-07 12:02 +0000 [c00c298a0e]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* app_voicemail/IMAP: function 'save_to_folder' creates wrong folder
+
+	  If try to move message to Cust1 (number 5)
+	  the function 'save_to_folder' tries to create Greeting folder instead of Cust1.
+
+	  This patch fixed it by setting GREETINGS_FOLDER = -1
+
+	  ASTERISK-24927 #close
+
+	  Change-Id: I03d1a761894bcc2d130ec9b003bbcddc28e25c51
+
+2016-04-07 16:18 +0000 [49813bc9e5]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip: Add headers to AMI Event ContactStatusDetail
+
+	  * Added Useragent and RegExpire headers to AMI Event
+	  ContactStatusDetail with associated documentation.
+
+	  ASTERISK-25903 #close
+
+	  Change-Id: If3d121e943e588d016ba51d4eb9c6a421a562239
+
+2016-04-05 16:56 +0000 [4e00e31ef1]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_outbound_publish: Add transport for outbound PUBLISH
+
+	  The first available transport of the appropriate type is used now.
+	  This patch adds new config option 'transport' for outbound-publish.
+	  If transport is set then outbound PUBLISH requests will use this transport.
+
+	  ASTERISK-25901 #close
+
+	  Change-Id: Ib389130489b70e36795b0003fa5fd386e2680151
+
+2016-04-11 14:26 +0000 [2cc56573de]  Jaco Kroon <jaco@uls.co.za>
+
+	* core_unreal: Fix hangupcauses not getting set on Local channels
+
+	  ASTERISK-25912 #close
+
+	  Change-Id: I8e72e6894feaf36c9450f2788d205d07baec23aa
+
+2016-04-01 13:30 +0000 [a621dd5e96]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip contact:  Lock expiration/addition of contacts
+
+	  Contact expiration can occur in several places:  res_pjsip_registrar,
+	  res_pjsip_registrar_expire, and automatically when anyone calls
+	  ast_sip_location_retrieve_aor_contact.  At the same time, res_pjsip_registrar
+	  may also be attempting to renew or add a contact.  Since none of this was locked
+	  it was possible for one thread to be renewing a contact and another thread to
+	  expire it immediately because it was working off of stale data.  This was the
+	  casue of intermittent registration/inbound/nominal/multiple_contacts test
+	  failures.
+
+	  Now, the new named lock functionality is used to lock the aor during contact
+	  expire and add operations and res_pjsip_registrar_expire now checks the
+	  expiration with the lock held before deleting the contact.
+
+	  ASTERISK-25885 #close
+	  Reported-by: Josh Colp
+
+	  Change-Id: I83d413c46a47796f3ab052ca3b349f21cca47059
+
+2016-04-10 14:16 +0000 [8637f29d24]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject:  Add patch to fix Via IPv6 parsing
+
+	  There's a bug in pjproject's sip_parser where the ":" wasn't correctly
+	  interpreted. This is causing IPv6 addresses in the "received" parameter of the
+	  Via header to cause a syntax check failure.
+
+	  This patch was submitted to Teluu on 4/10/2016.
+
+	  ASTERISK-25910 #close
+	  Reported-by: Anthony Messina
+
+	  Change-Id: Ic7e4c4aa14ded61860401ec349f5177568c4d922
+
+2016-03-31 20:04 +0000 [216abb0ae7]  gtjoseph <george.joseph@fairview5.com>
+
+	* lock:  Add named lock capability
+
+	  Locking some objects like sorcery objects can be tricky because the underlying
+	  ao2 object may not be the same for all callers.  For instance, two threads that
+	  call ast_sorcery_retrieve_by_id on the same aor name might actually get 2
+	  different ao2 objects if the underlying wizard had to rehydrate the aor from a
+	  database. Locking one ao2 object doesn't have any effect on the other even if
+	  those objects had locks in the first place.
+
+	  Named locks allow access control by keyspace and key strings.  Now an "aor"
+	  named "1000" can be locked and any other thread attempting to lock "aor" "1000"
+	  will wait regardless of whether the underlying ao2 object is the same or not.
+	  Mutex and rwlocks are supported.
+
+	  This capability will initially be used to lock an aor when multiple threads may
+	  be attempting to prune expired contacts from it.
+
+	  Change-Id: If258c0b7f92b02d07243ce70e535821a1ea7fb45
+
+2016-04-07 11:37 +0000 [f9dab80816]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* app_voicemail/IMAP: IMAP access FATAL error: Out of memory
+
+	  Sometimes uw-imap function 'mail_fetchbody' returns huge len
+	  which then pass to uw-imap function 'rfc822_base64'.
+	  uw-imap tries to allocate huge memory and abort() on fail.
+
+	  This patch check the len.
+	  If the len more than max size (128 Mbytes) log error.
+	  This patch also set variables len, newlen to avoid uninizialezed len.
+	  This patch also check pointer returned by rfc822_base64.
+
+	  ASTERISK-25899 #close
+
+	  Change-Id: I4a0e7d655f11abef6a5224e2169df6d5c1f1caca
+
+2016-04-07 16:39 +0000 [b3be945415]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_dialog_info: Add missing "direction" attribute in NOTIFY event
+
+	  BLF pickup isn't working on Cisco SPA and Snom phones
+	  if the direction="recipient" attribute is missing in 'dialog' tag.
+
+	  This patch adds direction="recipient" if extension state is
+	  Ringing.
+
+	  ASTERISK-24601 #close
+
+	  Change-Id: I5b2c097ca29fd59e92ba237ca5d397cb1b0bcd8c
+
+2016-04-06 17:57 +0000 [6138a75e8e]  Richard Mudgett <rmudgett@digium.com>
+
+	* pbx.h: Make ast_state_cb_type take more const.
+
+	  This eliminates some casts that I made a note saying v10 and above
+	  would no longer need them.
+
+	  Better late than never :)
+
+	  Change-Id: I346cdb3032b6478ceb40eb6fe732978b54035572
+
+2016-04-07 10:59 +0000 [72c19f7dc5]  Richard Mudgett <rmudgett@digium.com>
+
+	* pbx.c: Minor code rearangements.
+
+	  * Pull out a loop invariant.
+
+	  * Convert an else-if ladder to a switch statement.
+
+	  Change-Id: I0a95cfa9474a4600b9865f7b444534d275b37e95
+
+2016-04-07 12:26 +0000 [28cefc3e88]  Richard Mudgett <rmudgett@digium.com>
+
+	* pbx: Update doxygen for extension state watchers.
+
+	  Change-Id: Id1403b12136de62a272c01bb355aef65fd2c2d1e
+
+2016-04-07 11:49 +0000 [751d7a5a49]  gtjoseph <george.joseph@fairview5.com>
+
+	* alembic:  Remove batch operations (and sqlite support)
+
+	  Because SQLite doesn't support full ALTER capabilities, alembic scripts
+	  require batch operations.  However, that capability wasn't available until
+	  0.7.0 which some distributions haven't reached yet.  Therefore, the batch
+	  operations introduced in commit 86d6e44cc (review 2319) have been reverted
+	  and SQLite is unsupported again, for now anyway.
+
+	  Tested the full upgrade and downgrade on MySQL/Mariadb and Postgresql.
+
+	  ASTERISK-25890 #close
+	  Reported-by: Harley Peters
+
+	  Change-Id: I82eba5456736320256f6775f5b0b40133f4d1c80
+
+2016-04-07 11:05 +0000 [2eaeea690d]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_registrar_expire: Fix race condition at shutdown.
+
+	  When shutting down, the PJSIP sorcery is destroyed. The registrar
+	  expiration module queries the PJSIP sorcery to determine what
+	  to expire. As there was no synchronization between termination
+	  of the expiration thread and the unloading of the module it was
+	  possible for the thread to try to access the PJSIP sorcery after
+	  it had been destroyed.
+
+	  This change ensures that the thread is shut down before allowing
+	  the module to be considered unloaded.
+
+	  Change-Id: I69fd239edbaaf160c2d37ae00d3ac06e5596fe8b
+
+2016-04-06 16:28 +0000 [3e5672d843]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Fix configuration setting of "regcontext".
+
+	  Due to a merge problem two options were swapped causing the
+	  regcontext setting to not get set.
+
+	  Change-Id: Icb33edc668e7357bacbaec2861a6b5ac64edaff1
+
+2016-04-06 08:01 +0000 [8ed5f61152]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+
+	* frame.c: Copy the whole subclass in ast_frdup().
+
+	  The problem is ast_frdup() does not copy whole frame.subclass for voice,
+	  video and image frames, only the format is copied.  For video frames, the
+	  subclass structure contains the .frame_ending flag used to put the RTP
+	  marker where it needs to be.
+
+	  ASTERISK-25894 #close
+
+	  Change-Id: I812ca90e84ed5d4f473b997d0dd0d3c5a915fe33
+
+2016-03-30 17:18 +0000 [abbb2edd4c]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Add method to Dial a created channel.
+
+	  This adds a new ARI method that allows for you to dial a channel that
+	  you previously created in ARI.
+
+	  By combining this with the create method for channels, it allows for a
+	  workflow where a channel can be created, manipulated, and then dialed.
+	  The channel is under control of the ARI application during all stages of
+	  the Dial and can even be manipulated based on channel state changes
+	  observed within an ARI application.
+
+	  The overarching goal for this is to eventually be able to add a dialed
+	  channel to a Stasis bridge earlier than the "Up" state. However, at the
+	  moment more work is needed in the Dial and Bridge APIs in order to
+	  facilitate that.
+
+	  ASTERISK-25889 #close
+
+	  Change-Id: Ic6c399c791e66c4aa52454222fe4f8b02483a205
+
+2016-03-30 17:01 +0000 [dd48d60c5b]  Mark Michelson <mmichelson@digium.com>
+
+	* ARI: Add method to create a new channel.
+
+	  This adds a new ARI method to the channels resource that allows for the
+	  creation of a new channel. The channel is created and then placed into
+	  the specified Stasis application.
+
+	  This is different from the existing originate method that creates a
+	  channel, dials it, and then places the answered channel into the
+	  dialplan or a Stasis application. This method does not attempt to call
+	  the channel at all. Dialing is left as a later step after channel
+	  creation. This allows for pre-dialing channel manipulation if desired.
+
+	  ASTERISK-25889
+
+	  Change-Id: I3c96a0aba914b08e39f6256371a5bd4c92cbded8
+
+2016-03-28 11:31 +0000 [1dc5e28624]  Joshua Colp <jcolp@digium.com>
+
+	* pbx: Add support for autohints.
+
+	  This change introduces the concept of autohints. These are hints
+	  which are created as a result of device state changes occurring within
+	  the core. When this happens a hint will be created (if it does not
+	  exist already) using the device name as the extension.
+
+	  For example if a device state change is received for "PJSIP/bob"
+	  and autohints are enabled on a context then a hint will exist in
+	  that context for "bob" with a device of "PJSIP/bob".
+
+	  For virtual or custom device states the name after the type will
+	  be used. For example if the device state of "Custom:bob" changes
+	  then a hint will exist in that context for "bob" with a device of
+	  "Custom:bob".
+
+	  This functionality can be enabled in extensions.conf by placing
+	  "autohints=yes" in a context.
+
+	  ASTERISK-25881 #close
+
+	  Change-Id: I7e444c7da41b7b7d33374420fec658beeb18584e
+
+2016-04-05 14:23 +0000 [a098251e7e]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Handle deferred SDP hold/unhold properly.
+
+	  Some SIP devices indicate hold/unhold using deferred SDP reinvites. In
+	  other words, they provide no SDP in the reinvite.
+
+	  A typical transaction that starts hold might look something like this:
+
+	  * Device sends reinvite with no SDP
+	  * Asterisk sends 200 OK with SDP indicating sendrecv on streams.
+	  * Device sends ACK with SDP indicating sendonly on streams.
+
+	  At this point, PJMedia's SDP negotiator saves Asterisk's local state as
+	  being recvonly.
+
+	  Now, when the device attempts to unhold, it again uses a deferred SDP
+	  reinvite, so we end up doing the following:
+
+	  * Device sends reinvite with no SDP
+	  * Asterisk sends 200 OK with SDP indicating recvonly on streams
+	  * Device sends ACK with SDP indicating sendonly on streams
+
+	  The problem here is that Asterisk offered recvonly, and by RFC 3264's
+	  rules, if an offer is recvonly, the answer has to be sendonly. The
+	  result is that the device is not taken off hold.
+
+	  What is supposed to happen is that Asterisk should indicate sendrecv in
+	  the 200 OK that it sends. This way, the device has the freedom to
+	  indicate sendrecv if it wants the stream taken off hold, or it can
+	  continue to respond with sendonly if the purpose of the reinvite was
+	  something else (like a session timer refresher).
+
+	  The fix here is to alter the SDP negotiator's state when we receive a
+	  reinvite with no SDP. If the negotiator's state is currently in the
+	  recvonly or inactive state, then we alter our local state to be
+	  sendrecv. This way, we allow the device to indicate the stream state as
+	  desired.
+
+	  ASTERISK-25854 #close
+	  Reported by Robert McGilvray
+
+	  Change-Id: I7615737276165eef3a593038413d936247dcc6ed
+
+2016-03-30 16:47 +0000 [ef4d3f1328]  Mark Michelson <mmichelson@digium.com>
+
+	* Dial: Add function to append already-created channel.
+
+	  The Dial API takes responsiblity for creating an outbound channel when
+	  calling ast_dial_append(). This commit adds a new function,
+	  ast_dial_append_channel(), which allows us to create the channel outside
+	  the Dial API and then to append the channel to the ast_dial structure.
+
+	  This is useful for situations where the channel's creation and dialing
+	  are distinct operations. Upcoming ARI early bridge work will illustrate
+	  its usage.
+
+	  ASTERISK-25889
+
+	  Change-Id: Id8179f64f8f99132f80dead8d5db2030fd2c0509
+
+2016-03-27 23:33 +0000 [984d6fd95c]  gtjoseph <george.joseph@fairview5.com>
+
+	* config:  Allow filters when appending to a category
+
+	  In sorcery based config files where there are multiple categories with the same
+	  name, you can't use the (+) operator to reliably append to a category because
+	  config.c stops looking when it finds the first one with the same name.
+
+	  Example:
+
+	  [1000]
+	  type = endpoint
+
+	  [1000]
+	  type = aor
+
+	  [1000](+)
+	  authenticate_qualify = yes
+
+	  This config will fail because config.c appends authenticate_qualify to the
+	  first category it finds, the endpoint, and that's not valid for endpoint.
+
+	  Solution:
+
+	  The capability to find a category that contains a certain variable already
+	  exists so the only real change was to parse anything after the '+' that's not a
+	  comma, as a filter string.
+
+	  [1000]
+	  type = endpoint
+
+	  [1000]
+	  type = aor
+
+	  [1000](+type=aor)
+	  authenticate_qualify = yes
+
+	  This now works as expected.
+
+	  Although the following example doesn't make any sense for pjsip, you can even
+	  specify multiple filters:
+
+	  [1000](+type=aor&qualify_frequency=10)
+
+	  ASTERISK-25868 #close
+	  Reported-by: Nick Repin
+
+	  Change-Id: I10773da4c79db36fbf1993961992af63d3441580
+
+2016-04-05 10:21 +0000 [784fb43f43]  Joshua Colp <jcolp@digium.com>
+
+	* res_http_websocket: Make core supported.
+
+	  Websockets are a core part of ARI support and as such this
+	  module should also be core supported.
+
+	  Change-Id: I8f9283c6a167152761b92984779bb39e3db51a9c
+
+2016-03-25 23:22 +0000 [4d40b161c3]  gtjoseph <george.joseph@fairview5.com>
+
+	* stringfields:  Refactor to allow fields to be added to the end of structures
+
+	  String fields are great, except that you can't add new ones without breaking
+	  ABI compatibility because it shifts down everything else in the structure.
+	  The only alternative is to add your own char * field to the end of the
+	  structure and manage the memory yourself which isn't ideal, especially since
+	  you then can't use the OPT_STRINGFIELD_T type.
+
+	  Background:
+
+	  The reason string fields had to be declared inside the
+	  AST_DECLARE_STRING_FIELDS block was to facilitate iteration over all declared
+	  fields for initialization, compare and copy.  Since AST_DECLARE_STRING_FIELDS
+	  declared the pool, then the fields, then the manager, you could use the offsets
+	  of the pool and manager and iterate over the sequential addresses in between to
+	  access the fields. The actual pool, field allocation and field set operations
+	  don't actually care where the field is.  It's just iteration over the fields
+	  that was the problem.
+
+	  Solution: Extended String Fields
+
+	  An extended string field is one that is declared outside the
+	  AST_DECLARE_STRING_FIELDS block but still (anywhere) inside the parent
+	  structure.  Other than using AST_STRING_FIELD_EXTENDED instead of
+	  AST_STRING_FIELD, it looks the same as other string fields.  It's storage comes
+	  from the pool and it participates in string field compare and copy operations
+	  peformed on the parent structure. It's also a valid target for the
+	  OPT_STRINGFIELD_T aco option type.
+
+	  Implementation:
+
+	  To keep track of the extended fields and make sure that ABI isn't broken, the
+	  existing embedded_pool pointer in the manager structure was repurposed to be a
+	  pointer to a separate header structure that contains the embedded_pool pointer
+	  plus a vector of fields.  The length of the manager structure didn't change and
+	  the embedded_pool pointer isn't used in the macros, only the stringfields C
+	  code.  A side benefit of this is that changing the header structure in the
+	  future won't break ABI.
+
+	  ast_string_fields_init initializes the normal string fields and appends them to
+	  the vector, and subsequent calls to ast_string_field_init_extended initialize
+	  and append the extended fields. Cleanup, ast_string_fields_cmp, and
+	  ast_string_fields_copy can now work on the vector instead of sequentially
+	  traversing the addresses between the pool and manager.
+
+	  The total size of a structure using string fields didn't change, whether using
+	  extended fields or not, nor have the offsets of any structure members, either
+	  inside the original block or outside.  Adding an extended field to the end of a
+	  structure is the same as adding a char *.
+
+	  Details:
+
+	  The stringfield C code was pulled out from utils.c and into stringfields.c.
+	  It just made sense.
+
+	  Additional work was done in ast_string_field_init and
+	  ast_calloc_with_stringfields to handle the allocation of the new header
+	  structure and the vector, and the associated cleanup.  In the process some
+	  additional NULL pointer checking was added.
+
+	  A lot of work was done in stringfields.h since the logic for compare and copy
+	  is there.  Documentation was added as well as somne additional NULL checking.
+
+	  The ability to call ast_calloc_with_stringfields with a number of structures
+	  greater than 1 never really worked.  Well, the calloc worked but there was no
+	  way to access the additional structures or clean them up.  It was agreed that
+	  there was no use case for requesting more than 1 structure so an ast_assert
+	  was added to prevent it and the iteration code removed.
+
+	  Testing:
+
+	  The stringfield unit tests were updated to test both normal and extended
+	  fields.  Tests for ast_string_field_ptr_set_by_fields and
+	  ast_calloc_with_stringfields were also added.
+
+	  As an ABI test, 13 was compiled from git and the res_pjsip_* modules, except
+	  res_pjsip itself, saved off.  The patch was then added and a full compile and
+	  install was performed.  Then the older res_pjsip_* moduled were copied over the
+	  installed versions so res_pjsip was new and the rest were old.  No issues.
+
+	  contact->aor, which is a char * at the end of contact, was then changed to an
+	  extended string field and a recompile and reinstall was performed, again
+	  leaving stock versions of the the res_pjsip_* modules.  Again, no issues with
+	  the res_pjsip_* modules using the old stringfield implementation and with
+	  contact->aor as a char *, and res_pjsip itself using the new stringfield
+	  implementation and contact->aor being an extended string field.
+
+	  Finally, several existing string fields were converted to extended string
+	  fields to test OPT_STRINGFIELD_T.  Again, no issues.
+
+	  Change-Id: I235db338c5b178f5a13b7946afbaa5d4a0f91d61
+
+2016-04-04 18:02 +0000 [c07e1190ec]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_mwi:  Fix segv caused by 16c7d8e74a9af13f98c3c22aa9c43ce39965f6b7
+
+	  I forgot the new voicemail_extension wasn't a stringfield and didn't check
+	  for NULL where I should have.
+
+	  Change-Id: I029482d5c2ab72474838750461bd46b0809c90fb
+
+2016-04-03 11:47 +0000 [060b7b83bc]  gtjoseph <george.joseph@fairview5.com>
+
+	* install_prereq:  Fix check_installed_debs remove subversion
+
+	  check_installed_debs wasn't handling virtual packages like libsrtp-dev and
+	  libresample-dev and on multiarch systems it was accidentally filtering out all
+	  packages if any :i386 packages were found instead of just filtering out the
+	  :i386 packages themselves.
+
+	  Change-Id: Ifd68da0d1ee30cc84df14de3f9b9079d7c3cecda
+
+2016-04-01 13:09 +0000 [433d2c4bbf]  gtjoseph <george.joseph@fairview5.com>
+
+	* utils.c:  Fix typo in handle_show_locks
+
+	  ast_cli_allow_on_shutdown(e) should have been ast_cli_allow_at_shutdown(e).
+
+	  Change-Id: I4f092495c0b2bfd85c2651e0b5877bf4d05d9faf
+
+2016-03-30 18:34 +0000 [304f81780d]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject_bundled:  Fix use of LDCONFIG for shared library link creation
+
+	  LDCONFIG apparently isn't set to something sane on all systems so the creation
+	  of the shared library links fails.  Instead of just testing for non-blank,
+	  main/Makefile now checks that LDCONFIG is actually executable and reverts to
+	  LN if it isn't.
+
+	  This applies to both libasteriskpj and libasteriskssl.
+
+	  Thanks to 'abelbeck' for pointing out that the issue was LDCONFIG.
+
+	  ASTERISK-25873 #close
+	  Reported-by: Hans van Eijsden
+
+	  Change-Id: I25b76379bc637726ec044b2c0e709b56b3701729
+
+2016-03-29 13:47 +0000 [0ea742d33a]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_stasis: Add control ref to playback and recording structs.
+
+	  The stasis_app_playback and stasis_app_recording structs need to have a
+	  struct stasis_app_control ref.  Other threads can get a reference to the
+	  playback and recording structs from their respective global container.
+	  These other threads can then use the control pointer they contain after
+	  the control struct has gone.
+
+	  * Add control ref to stasis_app_playback and stasis_app_recording structs.
+
+	  With the refs added, the control command queue can now have a circular
+	  control reference which will cause the control struct to never get
+	  released if the control's command queue is not flushed when the channel
+	  leaves the Stasis application.  Also the command queue needs better
+	  protection from adding commands if the control->is_done flag is set.
+
+	  * Flush the control command queue on exit.
+
+	  ASTERISK-25882 #close
+
+	  Change-Id: I3cf1fb59cbe6f50f20d9e35a2c07ac07d7f4320d
+
+2016-03-28 18:10 +0000 [53f63ad770]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_stasis: Fix crash on a hanging up channel.
+
+	  * Give the struct stasis_app_control ao2 object a ref to the channel held
+	  in the object.  Now the channel will still be around if a thread needs to
+	  post a stasis message instead of crash because the topic was destroyed.
+
+	  * Moved stopping any lingering silence generator out of the struct
+	  stasis_app_control destructor and made it a part of exiting the Stasis
+	  application.  Who knows which thread the destructor will be called under
+	  so it cannot affect the channel's silence generator.  Not only was the
+	  channel unprotected when the silence generator was stopped, stasis may no
+	  longer even control the channel.
+
+	  ASTERISK-25882
+
+	  Change-Id: I21728161b5fe638cef7976fa36a605043a7497e4
+
+2016-03-30 13:31 +0000 [2fab4d7da8]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_stasis.c: Protect channel datastore list from stasis end.
+
+	  Change-Id: Ifadc469590bd4d5368e19d3763db3bd1f80fdb95
+
+2016-03-29 18:06 +0000 [ece2edaa04]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_ari: Cannot get control also means channel is unavailable.
+
+	  The only caller of ari_bridges_play_found() has this note:
+
+	  If ari_bridges_play_found fails because the channel is unavailable for
+	  playback, The channel will be removed from the playback list soon.  We can
+	  keep trying to get channels from the list until we either get one that
+	  will work or else there isn't a channel for this bridge anymore, in which
+	  case we'll revert to ari_bridges_play_new.
+
+	  Change-Id: Ib068141b367ccaa17be0dab4181c98e26c5127d6
+
+2016-03-29 14:29 +0000 [2f36cba4b5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_stasis_recording.c: Cleanup stasis_app_recording_find_by_name().
+
+	  Change-Id: Ic7d93c402c498677a122505558859c853d4e5ac7
+
+2016-03-28 14:23 +0000 [34457dd9db]  Richard Mudgett <rmudgett@digium.com>
+
+	* core_unreal.c: Add clarification comment about channel ref.
+
+	  Change-Id: I0be0627260cd8d6b6c3cc345949dcfdf32eff1f3
+
+2016-03-30 12:38 +0000 [2b3261cd36]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_mwi:  Allow subscribe to vm access extension as an alias
+
+	  Background:
+
+	  If your extension is 1000 and the voicemail access extension is 1571 and you
+	  dial 1571, usually a dialplan rule calls voicemailmain with your extension and
+	  you are placed directly in your mailbox.  Therefore most admins program the
+	  voicemail (or other speed dial) button on their phones to the access extension.
+	  Some phones (Snom at least) use whatever is programmed there to also subscribe
+	  for MWI and so can't dial one number and subscribe to another.  This works fine
+	  in chan_sip because chan_sip completely ignores the user portion of the
+	  SUBSCRIBE message request URI.  If it can match the peer, is subscribes to the
+	  peer's mailbox.  The user could be set to anything or nothing and you'd still
+	  get subscribed to your mailbox.
+
+	  Issue:
+
+	  chan_pjsip actually uses the user portion of the URI to find an aor and its
+	  mailboxes.  Therefore a subscribe to 1571 results in a 404.  Sure, you can
+	  create an aor for 1571 but you certainly can't add your entire voicemail
+	  system's mailboxes to it and everyone would get notified of every MWI.
+
+	  Solution:
+
+	  When an MWI subscribe comes in and an aor can't be found that matches the
+	  resource directly, check the resource against the endpoint's aors.  If an aor
+	  is found that has a voicemail_extension that matches the resource, use it.
+
+	  ASTERISK-25865
+	  Reported-by: Ross Beer
+
+	  Change-Id: I770ea185f751f1ada888fafb4b452115f1c06e9e
+
+2016-03-24 22:55 +0000 [e2524fcee3]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_mwi:  Add voicemail extension and mwi_subscribe_replaces_unsolicited
+
+	  res_pjsip_mwi was missing the chan_sip "vmexten" functionality which adds
+	  the Message-Account header to the MWI NOTIFY.  Also, specifying mailboxes
+	  on endpoints for unsolicited mwi and on aors for subscriptions required
+	  that the admin know in advance which the client wanted.  If you specified
+	  mailboxes on the endpoint, subscriptions were rejected even if you also
+	  specified mailboxes on the aor.
+
+	  Voicemail extension:
+	  * Added a global default_voicemail_extension which defaults to "".
+	  * Added voicemail_extension to both endpoint and aor.
+	  * Added ast_sip_subscription_get_dialog for support.
+	  * Added ast_sip_subscription_get_sip_uri for support.
+
+	  When an unsolicited NOTIFY is constructed, the From header is parsed, the
+	  voicemail extension from the endpoint is substituted for the user, and the
+	  result placed in the Message-Account field in the body.
+
+	  When a subscribed NOTIFY is constructed, the subscription dialog local uri
+	  is parsed, the voicemail_extension from the aor (looked up from the
+	  subscription resource name) is substituted for the user, and the result
+	  placed in the Message-Account field in the body.
+
+	  If no voicemail extension was defined, the Message-Account field is not added
+	  to the NOTIFY body.
+
+	  mwi_subscribe_replaces_unsolicited:
+	  * Added mwi_subscribe_replaces_unsolicited to endpoint.
+
+	  The previous behavior was to reject a subscribe if a previous internal
+	  subscription for unsolicited MWI was found for the mailbox.  That remains the
+	  default.  However, if there are mailboxes also set on the aor and the client
+	  subscribes and mwi_subscribe_replaces_unsolicited is set, the existing internal
+	  subscription is removed and replaced with the external subscription.  This
+	  allows an admin to configure mailboxes on both the endpoint and aor and allows
+	  the client to select which to use.
+
+	  ASTERISK-25865 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: Ic15a9415091760539c7134a5ba3dc4a6a1217cea
+
+2016-03-30 09:46 +0000 [724b9ab28f]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_rtp_asterisk:  Fix placement of txcount increment
+
+	  Commit 1bce690ccb36a4744a327c07af23a9a3a0fa20cd was incrementing txcount
+	  for rtcp packets as well as rtp packets and that was causing sender reports
+	  to be generated instead of receiver reports in cases where no rtp was actually
+	  being sent.
+
+	  Moved the txcount increment from __rtp_sento, which handles both rtp and rtcp,
+	  to rtp_sento which only handles rtp packets.
+
+	  Discovered by the hep/rtcp-receiver test.
+
+	  Change-Id: Ie442e4bb947a68847a676497021ba10ffaf376d5
+
+2016-03-26 22:33 +0000 [c4064727d2]  gtjoseph <george.joseph@fairview5.com>
+
+	* chan_pjsip:  Add 'pjsip show channelstats'
+
+	  Added the ability to show channel statistics to chan_pjsip (cli_functions.c)
+
+	  Moved the existing 'pjsip show channel(s)' functionality from
+	  pjsip_configuration to cli_functions.c.  The stats needed chan_pjsip's
+	  private header so it made sense to move the existing channel commands as well.
+
+	  Now using stasis_cache_dump to get the channel snapshots rather than retrieving
+	  all endpoints, then getting each one's channel snapshots.  Much more efficient.
+
+	  Change-Id: I03b114522126d27434030b285bf6d531ddd79869
+
+2016-03-25 10:59 +0000 [970803efcb]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+
+	* res_rtp_asterisk: Use separate SRTP session for RTCP with DTLS
+
+	  Asterisk uses separate UDP ports for RTP and RTCP traffic and RFC 5764
+	  explicitly states:
+
+	    There MUST be a separate DTLS-SRTP session for each distinct pair of
+	    source and destination ports used by a media session
+
+	  This means RTP keying material cannot be used for DTLS RTCP, which was
+	  the reason why RTCP encryption would fail.
+
+	  ASTERISK-25642
+
+	  Change-Id: I7e8779d8b63e371088081bb113131361b2847e3a
+
+2016-03-25 10:42 +0000 [9785e8d090]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+
+	* app_echo: forward and generate VIDUPDATE frames
+
+	  When using app_echo via WebRTC with VP8 video the video would appear
+	  only after a few minutes, because there would be nothing to request
+	  a full reference frame.
+
+	  This fixes the problem in both ways:
+	  - echos any VIDUPDATE frames received on the channel
+	  - sends one such frame when first video frame is to be forwarded
+
+	  This makes the echo work with Firefox and Chrome WebRTC implementation.
+
+	  ASTERISK-25867 #close
+
+	  Change-Id: I73bda87bf7532ee8bfb28d917045a21034908c1e
+
+2016-03-27 12:53 +0000 [44ffb5105a]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_rtp_asterisk:  Fix packet stats on bridged connection
+
+	  rxcount, txcount, rxoctetcount and txoctetcount weren't being calculated
+	  for bridged streams because the calulations were being done after the
+	  bridged short-circuit.  Actually, rxoctetcount wasn't ever being calculated.
+
+	  Moved the calculations so they occur for all valid received packets and
+	  all transmitted packets.  Also added rxoctetcount and txoctetcount to
+	  ast_rtp_instance_stat.
+
+	  Change-Id: I08fb06011a82d38c3b4068867a615068fbe59cbb
+
+2016-03-10 19:52 +0000 [c971a64366]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/pjsip_options:  Fix From generation on outgoing OPTIONS
+
+	  No one seemed to notice but every time an OPTIONS goes out, it goes
+	  out with a From of "asterisk" (or whatever the default from_user is set to),
+	  even if you specify an endpoint.
+
+	  The issue had several causes...
+	  qualify_contact is only called with an endpoint if called from the CLI.
+	  If the endpoint is NULL, qualify_contact only looks up the endpoint if
+	  authenticate_qualify=yes. Even then, it never passes it on to
+	  ast_sip_create_request where the From header is set.  Therefore From
+	  is always "asterisk" (or whatever the default from_user is set to).
+	  Even if ast_sip_create_request were to get an endpoint, it only sets
+	  the From if endpoint->from_user is set.
+
+	  The fix is 4 parts...
+
+	  First, create_out_of_dialog_request was modified to use the endpoint id
+	  if endpoint was specified and from_user is not set.
+
+	  Second, qualify_contact was modified to always look up an endpoint if
+	  one wasn't specified regardless of authenticate_qualify.  It then passes
+	  the endpoint on to create_out_of_dialog_request.
+
+	  Third (and most importantly), find_an_endpoint was modified to find
+	  an endpoint by using an "aors LIKE %contact->aor%" predicate with
+	  ast_sorcery_retrieve_by_fields.  As such, this patch will only work
+	  if the sorcery realtime optimizations patch goes in.  Otherwise we'd
+	  be pulling the entire endpoints database every time we send an OPTIONS.
+	  Since we already know the contact's aor, the on_endpoint callback was also
+	  modified to just check if the contact->aor is an exact match to one of
+	  the endpoint's.
+
+	  Finally, since we now have an endpoint for every OPTIONS request,
+	  res_pjsip/endpt_send_request (which handles out-of-dialog reqests) was
+	  updated to get the transport from the endpoint and set it on tdata.
+	  Now the correct transport is used.
+
+	  Change-Id: I2207e12bb435e373bd1e03ad091d82e5aba011af
+
+2016-03-08 15:55 +0000 [c948ce9651]  gtjoseph <george.joseph@fairview5.com>
+
+	* sorcery/res_pjsip:  Refactor for realtime performance
+
+	  There were a number of places in the res_pjsip stack that were getting
+	  all endpoints or all aors, and then filtering them locally.
+
+	  A good example is pjsip_options which, on startup, retrieves all
+	  endpoints, then the aors for those endpoints, then tests the aors to see
+	  if the qualify_frequency is > 0.  One issue was that it never did
+	  anything with the endpoints other than retrieve the aors so we probably
+	  could have skipped a step and just retrieved all aors. But nevermind.
+
+	  This worked reasonably well with local config files but with a realtime
+	  backend and thousands of objects, this was a nightmare.  The issue
+	  really boiled down to the fact that while realtime supports predicates
+	  that are passed to the database engine, the non-realtime sorcery
+	  backends didn't.
+
+	  They do now.
+
+	  The realtime engines have a scheme for doing simple comparisons. They
+	  take in an ast_variable (or list) for matching, and the name of each
+	  variable can contain an operator.  For instance, a name of
+	  "qualify_frequency >" and a value of "0" would create a SQL predicate
+	  that looks like "where qualify_frequency > '0'".  If there's no operator
+	  after the name, the engines add an '=' so a simple name of
+	  "qualify_frequency" and a value of "10" would return exact matches.
+
+	  The non-realtime backends decide whether to include an object in a
+	  result set by calling ast_sorcery_changeset_create on every object in
+	  the internal container.  However, ast_sorcery_changeset_create only does
+	  exact string matches though so a name of "qualify_frequency >" and a
+	  value of "0" returns nothing because the literal "qualify_frequency >"
+	  doesn't match any name in the objset set.
+
+	  So, the real task was to create a generic string matcher that can take a
+	  left value, operator and a right value and perform the match. To that
+	  end, strings.c has a new ast_strings_match(left, operator, right)
+	  function.  Left and right are the strings to operate on and the operator
+	  can be a string containing any of the following: = (or NULL or ""), !=,
+	  >, >=, <, <=, like or regex.  If the operator is like or regex, the
+	  right string should be a %-pattern or a regex expression.  If both left
+	  and right can be converted to float, then a numeric comparison is
+	  performed, otherwise a string comparison is performed.
+
+	  To use this new function on ast_variables, 2 new functions were added to
+	  config.c.  One that compares 2 ast_variables, and one that compares 2
+	  ast_variable lists.  The former is useful when you want to compare 2
+	  ast_variables that happen to be in a list but don't want to traverse the
+	  list.  The latter will traverse the right list and return true if all
+	  the variables in it match the left list.
+
+	  Now, the backends' fields_cmp functions call ast_variable_lists_match
+	  instead of ast_sorcery_changeset_create and they can now process the
+	  same syntax as the realtime engines.  The realtime backend just passes
+	  the variable list unaltered to the engine.  The only gotcha is that
+	  there's no common realtime engine support for regex so that's been noted
+	  in the api docs for ast_sorcery_retrieve_by_fields.
+
+	  Only one more change to sorcery was done...  A new config flag
+	  "allow_unqualified_fetch" was added to reg_sorcery_realtime.
+	  "no": ignore fetches if no predicate fields were supplied.
+	  "error": same as no but emit an error. (good for testing)
+	  "yes": allow (the default);
+	  "warn": allow but emit a warning. (good for testing)
+
+	  Now on to res_pjsip...
+
+	  pjsip_options was modified to retrieve aors with qualify_frequency > 0
+	  rather than all endpoints then all aors.  Not only was this a big
+	  improvement in realtime retrieval but even for config files there's an
+	  improvement because we're not going through endpoints anymore.
+
+	  res_pjsip_mwi was modified to retieve only endpoints with something in
+	  the mailboxes field instead of all endpoints then testing mailboxes.
+
+	  res_pjsip_registrar_expire was completely refactored.  It was retrieving
+	  all contacts then setting up scheduler entries to check for expiration.
+	  Now, it's a single thread (like keepalive) that periodically retrieves
+	  only contacts whose expiration time is < now and deletes them.  A new
+	  contact_expiration_check_interval was added to global with a default of
+	  30 seconds.
+
+	  Ross Beer reports that with this patch, his Asterisk startup time dropped
+	  from around an hour to under 30 seconds.
+
+	  There are still objects that can't be filtered at the database like
+	  identifies, transports, and registrations.  These are not going to be
+	  anywhere near as numerous as endpoints, aors, auths, contacts however.
+
+	  Back to allow_unqualified_fetch.  If this is set to yes and you have a
+	  very large number of objects in the database, the pjsip CLI commands
+	  will attempt to retrive ALL of them if not qualified with a LIKE.
+	  Worse, if you type "pjsip show endpoint <tab>" guess what's going to
+	  happen? :)  Having a cache helps but all the objects will have to be
+	  retrieved at least once to fill the cache.  Setting
+	  allow_unqualified_fetch=no prevents the mass retrieve and should be used
+	  on endpoints, auths, aors, and contacts.  It should NOT be used for
+	  identifies, registrations and transports since these MUST be
+	  retrieved in bulk.
+
+	  Example sorcery.conf:
+
+	  [res_pjsip]
+	  endpoint=config,pjsip.conf,criteria=type=endpoint
+	  endpoint=realtime,ps_endpoints,allow_unqualified_fetch=error
+
+	  ASTERISK-25826 #close
+	  Reported-by: Ross Beer
+	  Tested-by: Ross Beer
+
+	  Change-Id: Id2691e447db90892890036e663aaf907b2dc1c67
+
+2016-03-25 23:19 +0000 [8e8cf80cea]  Philip Correia
+
+	* res_parking: Fix blind transfer dynamic lots creation.
+
+	  Blind transfers to a recognized parking extension need to use the parker's
+	  channel variable values to create the dynamic parking lot.  This is
+	  because there is always only one parker while the parkee may actually be a
+	  multi-party bridge.  A multi-party bridge can never supply the needed
+	  channel variables to create the dynamic parking lot.  In the multi-party
+	  bridge blind transfer scenario, the parker's CHANNEL(parkinglot) value and
+	  channel variables are inherited by the local channel used to park the
+	  bridge.
+
+	  * In park_common_setup(), make use the parker instead of the parkee to
+	  supply the dynamic parking lot channel variable values.  In all but one
+	  case, the parkee is the same as the parker.  However, in the recognized
+	  parking extension blind transfer scenario for a two party bridge they are
+	  different channels.  For consistency, we need to use the parker channel.
+
+	  * In park_local_transfer(), pass the CHANNEL(parkinglot) value to the
+	  local channel when blind transferring a multi-party bridge to a recognized
+	  parking extension.
+
+	  * When a local channel starts a call, the Local;2 side needs to inherit
+	  the CHANNEL(parkinglot) value from Local;1.
+
+	  The DTMF one-touch parking case wasn't even trying to create dynamic
+	  parking lots before it aborted the attempt.
+
+	  * In parking_park_call(), add missing code to create a dynamic parking
+	  lot.
+
+	  A DTMF bridge hook is documented as returning -1 to remove the hook.
+	  Though the hook caller is really coded to accept non-zero.  See the
+	  ast_bridge_hook_callback typedef.
+
+	  * In feature_park_call(), don't remove the DTMF one-touch parking hook
+	  because of an error.
+
+	  ASTERISK-24605 #close
+	  Reported by:  Philip Correia
+	  Patches:
+	        call_park.patch (license #6672) patch uploaded by Philip Correia
+
+	  Change-Id: I221d3a8fcc181877a1158d17004474d35d8016c9
+
+2016-03-23 14:24 +0000 [3cf714031c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_parking: Cleanup find_channel_parking_lot_name() usage.
+
+	  Change-Id: I8f7a8890aef27824301c642d4d15407ac83e6f02
+
+2016-03-18 14:01 +0000 [13e75ee04f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_parking: Misc fixes.
+
+	  res/parking/parking_applications.c:
+
+	  * Add malloc fail checks in setup_park_common_datastore().
+
+	  * Fix playing parking failed announcement to only happen on non-blind
+	  transfers in park_app_exec().  It could never go out before because a test
+	  was provedly always false.
+
+	  res/parking/parking_bridge.c:
+
+	  * Fix NULL tolerance in generate_parked_user() because
+	  bridge_parking_push() can theoretically pass a NULL parker channel if the
+	  parker channel went away for some reason.
+
+	  * Clarify some weird code dealing with blind_transfer in
+	  bridge_parking_push().
+
+	  res/parking/parking_bridge_features.c:
+
+	  * Made park_local_transfer() set BLINDTRANSFER on the Local;1 channel
+	  which will be bulk copied to the Local;2 channel on the subsequent
+	  ast_call().  The additional advantage is if the parker channel has the
+	  BLINDTRANSFER and ATTENDEDTRANSFER variables set they are now guaranteed
+	  to be overridden.
+
+	  res/parking/parking_manager.c:
+
+	  * Fix AMI Park action input range checking of the Timeout header in
+	  manager_park().
+
+	  * Reduced locking scope to where needed in manager_park().
+
+	  res/res_parking.c:
+
+	  * Fix some off nominal missing unlocks by eliminating the returns.
+
+	  Change-Id: Ib64945bc285acb05a306dc12e6f16854898915ca
+
+2014-12-15 05:23 +0000 [e2853ae337]  Philip Correia
+
+	* res_parking: Update parking documentation for dynamic parking lots.
+
+	  * Remove duplicate res_parking.conf courtesytone config option
+	  documentation.
+
+	  ASTERISK-24596 #close
+	  Reported by:  Philip Correia
+
+	  ASTERISK-24605
+	  Reported by:  Philip Correia
+	  Patches:
+	        call_park_app_doc.patch (license #6672) patch uploaded by Philip Correia
+
+	  Change-Id: I90a92a891c6494dc08173e675856afcc4764c5b5
+
+2016-03-25 06:02 +0000 [72a897c534]  Joshua Colp <jcolp@digium.com>
+
+	* media_cache: Demote warning to debug as it may occur often.
+
+	  The file playback system will now query the media cache and then
+	  the old file functionality. Under normal conditions this will result
+	  in the cache failing to retrieve a file causing a warning message
+	  to get output each time a file is played back.
+
+	  This change demotes this warning to a debug message.
+
+	  Change-Id: Ib72246ba300b5cce32774bfb3c26634bfb708624
+2016-03-10 16:58 +0000 [89e94e886c]  Mark Michelson <mmichelson@digium.com>
+
+	* Restrict CLI/AMI commands on shutdown.
+
+	  During stress testing, we have frequently seen crashes occur because a
+	  CLI or AMI command attempts to access information that is in the process
+	  of being destroyed.
+
+	  When addressing how to fix this issue, we initially considered fixing
+	  individual crashes we observed. However, the changes required to fix
+	  those problems would introduce considerable overhead to the nominal
+	  case. This is not reasonable in order to prevent a crash from occurring
+	  while Asterisk is already shutting down.
+
+	  Instead, this change makes it so AMI and CLI commands cannot be executed
+	  if Asterisk is being shut down. For AMI, this is absolute. For CLI,
+	  though, certain commands can be registered so that they may be run
+	  during Asterisk shutdown.
+
+	  ASTERISK-25825 #close
+
+	  Change-Id: I8887e215ac352fadf7f4c1e082da9089b1421990
+
+2016-03-24 14:08 +0000 [3f720155b7]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Do not send all codecs on INVITE. Do not break on Session-Timers.
+
+	  Asterisk 13.7.0 included a fix for ASTERISK-24543, not to send all those
+	  codecs, which the caller did not request/support. That fix was not complete
+	  because on the second Session Timer all codecs were sent again. Some VoIP/SIP
+	  clients interpreted that complete codec-list as a change in the SIP session.
+	  Because of that, Asterisk did not send the RTP audio via NAT anymore which
+	  created a non-audio scenario after the second Session Timer fired.
+
+	  ASTERISK-24543 #close
+
+	  Change-Id: I1881827816ab7fd47eb4287a95961179b34a0b66
+
+2016-03-19 07:34 +0000 [894071ea2c]  Gianluca Merlo <gianluca.merlo@gmail.com>
+
+	* config: fix flags in uint option handler
+
+	  The configuration unsigned integer option handler sets flags for the
+	  parser as if the option should be a signed integer (PARSE_INT32),
+	  leading to errors on "out of range" values. Fix flags (PARSE_UINT32).
+
+	  A fix to res_pjsip is also present which stops invalid flags from
+	  being passed when registering sorcery object fields for qualify
+	  status.
+
+	  ASTERISK-25612 #close
+
+	  Change-Id: I96b539336275e0e72a8e8033487d2c3344debd3e
+
+2016-03-24 07:51 +0000 [13cdf3e8a1]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* musiconhold: Only warn if music class is not found in memory and database.
+
+	  The log message when a MusicOnHold music class was not found was changed
+	  from debug level to WARNING level in Asterisk 11.19 and 13.5.  For those
+	  using realtime musiconhold, this message is wrong because it warns
+	  before checking the database.
+
+	  This changeset delays the warning until after the database has been
+	  checked.
+
+	  Reported-by: Conrad de Wet
+	  ASTERISK-25444 #close
+
+	  Change-Id: I6cfb2db2f9cfbd2bb3d30566ecae361c4abf6dbf
+
+2016-03-24 05:48 +0000 [87c9ab97ea]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* core/logging: Fix broken syslog levels on older glibc.
+
+	  The fix to ASTERISK-25407 introduced the usage of LOG_MAKEPRI. However
+	  this macro is broken in older glibc (< 2.17); it would left-shift the
+	  facility a second time, causing the resultant priority to become
+	  invalid.
+
+	  The syslog manpage mentions nothing about LOG_MAKEPRI and suggests this:
+
+	      The priority argument is formed by ORing the facility and the level
+	      values [...].
+
+	  ASTERISK-25510 #close
+	  Reported by: Michael Newton
+
+	  Change-Id: Ia89debe7fac5ad090c7ef595c0707f31bb1e3d03
+2016-03-24 06:18 +0000 [a72f3b5bb4]  Joshua Colp <jcolp@digium.com>
+
+	* tests/test_http_media_cache: Fix file descriptor leak in test.
+
+	  Change-Id: Ie8a9ae3d13bdeaacafc8d28271adc6707f633a5f
+
+2016-02-28 19:05 +0000 [13efea24f7]  Matt Jordan <mjordan@digium.com>
+
+	* main/app: Only look to end of file if ':end' is specified, and not just ':'
+
+	  There is a little known feature in app_controlplayback that will cause the
+	  specified offset to be used relative to the end of a file if a ':end' is
+	  detected within the filename.
+
+	  This feature is pretty bad, but okay.
+
+	  However, a bug exists in this code where a ':' detected in the filename
+	  will cause the end pointer to be non-NULL, even if the full ':end' isn't
+	  specified. This causes us to treat an unspecified offset (0) as being
+	  "start playing from the end of the file", resulting in no file playback
+	  occurring.
+
+	  This patch fixes this bug by resetting the end pointer if ':end' is not
+	  found in the filename.
+
+	  Change-Id: Ib4c7b1b45283e4effd622a970055c51146892f35
+
+2015-12-26 15:29 +0000 [ca14b99e6e]  Matt Jordan <mjordan@digium.com>
+
+	* main/file: Add the ability to play media in the media cache
+
+	  This patch allows applications/APIs that access media through the core file
+	  APIs to play media in the media cache. Prior to determining if a 'filename'
+	  exists, the filename is passed to the media cache's retrieve API call. If
+	  that call succeeds, the local file specified passed back by the API is
+	  opened for streaming. When used in this fashion, the 'filename' is actually
+	  a URI that the media cache process and understand.
+
+	  ASTERISK-25654 #close
+
+	  Change-Id: I73b6e2e90c3e91b8500581c45cdf9c0dc785f5f0
+
+2015-12-30 10:52 +0000 [01962a3932]  Matt Jordan <mjordan@digium.com>
+
+	* tests/test_http_media_cache: Add unit tests for res_http_media_cache
+
+	  This patch adds unit tests for res_http_media cache, that covers nominal
+	  creation and retrieval - and through them as well, staleness and deletion
+	  checks. In addition, this patch adds tests that covers the interaction of
+	  various HTTP headers, including Expires, Etag, and Cache-Control.
+
+	  ASTERISK-25654
+
+	  Change-Id: I2db101e307c863857fe416d6f5bf4cace9ac7cf5
+
+2015-01-29 08:38 +0000 [22e2340813]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_http_media_cache: Add an HTTP(S) backend for the core media cache
+
+	  This patch adds a bucket backend for the core media cache that interfaces to a
+	  remote HTTP server. When a media item is requested in the cache, the cache will
+	  query its bucket backends to see if they can provide the media item. If that
+	  media item has a scheme of HTTP or HTTPS, this backend will be invoked.
+
+	  The backend provides callbacks for the following:
+	   * create - this will always retrieve the URI specified by the provided
+	              bucket_file, and store it in the file specified by the object.
+	   * retrieve - this will pull the URI specified and store it in a temporary
+	                file. It is then up to the media cache to move/rename this file
+	                if desired.
+	   * delete - destroys the file associated with the bucket_file.
+	   * stale - if the bucket_file has expired, based on received HTTP headers from
+	             the remote server, or if the ETag on the server no longer matches
+	             the ETag stored on the bucket_file, the resource is determined to be
+	             stale.
+
+	  Note that the backend respects the ETag, Expires, and Cache-Control headers
+	  provided by the HTTP server it is querying.
+
+	  ASTERISK-25654
+
+	  Change-Id: Ie201c2b34cafc0c90a7ee18d7c8359afaccc5250
+
+2015-12-26 15:31 +0000 [791b4c9f81]  Matt Jordan <mjordan@digium.com>
+
+	* main/media_cache: Provide an extension on the local file associated with a URI
+
+	  This patch does the following:
+
+	  First, it addresses file extension handling in the media cache. The media core
+	  in Asterisk is a bit interesting in that it wants:
+	   * A file to have an extension on it. That extension is used to associate the
+	     file with a defined format module.
+	   * The filename passed to the core to not have an extension on it. This allows
+	     the core to match the available file formats with the format a channel
+	     is capable of handling.
+
+	  Unfortunately, this makes the current implementation a bit lacking in the media
+	  cache. By default, we do not store the extension of a retrieved URI on the
+	  local file that is created. As a result, the media core does not know what
+	  format the file is, and the file is ignored. Modifying the file outside of the
+	  media core is bad, as we would not be able to update the internal
+	  ast_bucket_file's path.
+
+	  At the same time, we do not want to pass the extension out in the file_path
+	  parameter in ast_media_cache_retrieve. This parameter is intended to be fed
+	  into the media core; if we passed the extension, all callers would have to
+	  strip it off.
+
+	  Thus, this patch does the following:
+	  * If there is an extension specified in the URL, we append it to the local
+	    file name (if a preferred file name isn't specified), and we store that
+	    in the local file path.
+	  * The extension, however, is stripped off of the file_path parameter passed
+	    back out of ast_media_cache_retrieve.
+
+	  Second, this patch causes stale items to be completely removed from the system.
+	  Prior to this patch, sound files could be orphaned due to the bucket
+	  referencing the file being deleted, but the file itself not being removed. This
+	  is now addressed by explicitly calling ast_bucket_file_delete on the
+	  bucket_file when it is deemed to be stale. Note that this only happen when we
+	  know we will attempt to retrieve the resource again.
+
+	  Finally, this patch changes the AO2 container holding media items to just use
+	  a regular mutex. The usage for this container already assumed it was a plain
+	  mutex, and - given that retrieval of an item can cause it to be replaced in
+	  the container - a mutex makes more sense than a read/write lock.
+
+	  Change-Id: I51667fff86ae8d2e4a663555dfa85b11e935fe0f
+
+2014-10-25 20:21 +0000 [6bbcfb34bd]  Matt Jordan <mjordan@digium.com>
+
+	* funcs/func_curl: Add the ability for CURL to download and store files
+
+	  This patch adds a write option to the CURL dialplan function, allowing it to
+	  CURL files and store them locally. The value 'written' to the CURL URL
+	  specifies the location on disk to store the file. As an example:
+
+	  same => n,Set(CURL(http://1.1.1.1/foo.wav)=/tmp/foo.wav)
+
+	  Would retrieve the file foo.wav from the remote server and store it in the
+	  /tmp directory.
+
+	  Due to the potentially dangerous nature of this function call, APIs are
+	  forbidden from using the write functionality unless live_dangerously is set
+	  to True in asterisk.conf.
+
+	  ASTERISK-25652 #close
+
+	  Change-Id: I44f4ad823d7d20f04ceaad3698c5c7f653c41b0d
+
+2016-03-23 08:59 +0000 [392341ba37]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject-bundled:  Cleanups for reported issues
+
+	  PortAudio should no longer be required
+	  PJSIP_MAX_PKT_LEN is now 6000
+	  Older autoconf issue fixed. (CentOS 6)
+
+	  Change-Id: I463fa9586cbe7c6b3b603289f535bd8e361611dd
+
+2015-11-20 08:02 +0000 [ac66999971]  Francesco Castellano <francesco.castellano@messagenet.it>
+
+	* chan_sip.c: Space after port causes unnecessary resolution attempt
+
+	  check_via() already skips leading blanks where the sent-by address (with the
+	  optional port) should be placed.
+
+	  Since RFC 3261 allows for blanks between the port ant the Via parameters:
+	  > https://tools.ietf.org/html/rfc3261#section-20.42
+	  (actually it allows a lot of blanks more ;-)). I just switched from
+	  ast_skip_blanks() to ast_strip() on the local copy of the string.
+
+	  ASTERISK-21301 #close
+
+	  Change-Id: Ie5b8fe5a07067b7c0dc9bcdd1707e99b23b02b06
+2016-03-19 17:49 +0000 [1d3191b118]  gtjoseph <george.joseph@fairview5.com>
+
+	* progdocs:  Exclude ./third-party from documentation generation
+
+	  We don't need pjproject's documentation embedded in Asterisk's.
+
+	  Change-Id: Iea6f5a621c0f4e3168dda3321eaab258d9f24a17
+
+2016-03-18 20:32 +0000 [8f94f947f5]  Gianluca Merlo <gianluca.merlo@gmail.com>
+
+	* func_aes: fix misuse of strlen on binary data
+
+	  The encryption code for AES_ENCRYPT evaluates the length of the data to
+	  be encoded in base64 using strlen. The data is binary, thus the length
+	  of it can be underestimated at the first NULL character.
+	  Reuse the write pointer offset to evaluate it, instead.
+
+	  ASTERISK-25857 #close
+
+	  Change-Id: If686b5d570473eb926693c73461177b35b13b186
+2016-03-18 14:31 +0000 [a3c9a74a02]  Kevin Harwell <kharwell@digium.com>
+
+	* chan_pjsip: ref leak when checking direct_media_glare
+
+	  Fix the reference leak introduced in the following commit:
+
+	  c534bd58075e2e1a1e4f3b23c435186c71b155fd
+
+	  ASTERISK-25849
+
+	  Change-Id: I5cfefd5ee6c1c3a1715c050330aaa10e4d2a5e85
+2016-03-16 12:37 +0000 [c534bd5807]  Kevin Harwell <kharwell@digium.com>
+
+	* chan_pjsip: transfers with direct media reinvite has wrong address/port
+
+	  During a transfer involving direct media a race occurs between when the
+	  transferer channel is swapped out, initiating rtp changes/updates, and the
+	  subsequent reinvites.
+
+	  When Alice, after speaking with Charlie (Bob is on hold), connects Bob and
+	  Charlie invites are sent to each in order to establish the call between them.
+	  Bob is taken off hold and Charlie is told to have his media flow through
+	  Asterisk. However, if before those invites go out the bridge updates Bob's
+	  and/or Charlie's rtp information with direct media data (i.e. address, port)
+	  then the invite(s) will contain the remote data in the SDP instead of the
+	  Asterisk data.
+
+	  The race occurs in the native bridge glue code when updating the peer. The
+	  direct_media_address can get set twice before sending out the first invite
+	  during call connection. This can happen because the checking/setting of the
+	  direct_media_address happened in one thread while the sending of the invite(s)
+	  happened in another thread.
+
+	  This fix removes the race condition by moving the checking/setting of the
+	  direct_media_address to be in the same thread as the sending of the invites(s).
+	  This serializes the checking/setting and sending so they can no longer happen
+	  out of order.
+
+	  ASTERISK-25849 #close
+
+	  Change-Id: Idfea590175e74f401929a601dba0c91ca1a7f873
+
+2016-03-03 04:43 +0000 [bdccb81157]  Sergio Medina Toledo <lumasepa@gmail.com>
+
+	* res_pjsip_refer.c: Fix seg fault in process of Refer-to header.
+
+	  The "Refer-to" header of an incoming REFER request is parsed by
+	  pjsip_parse_uri().  That function requires the URI parameter to be NULL
+	  terminated.  Unfortunately, the previous code added the NULL terminator by
+	  overwriting memory that may not be safe.  The overwritten memory results
+	  could be benign, memory corruption, or a segmentation fault.  Now the URI
+	  is NULL terminated safely by copying the URI to a new chunk of memory with
+	  the correct size to be NULL terminated.
+
+	  ASTERISK-25814 #close
+
+	  Change-Id: I32565496684a5a49c3278fce06474b8c94b37342
+
+2016-02-25 10:29 +0000 [0da36fca6b]  Leif Madsen <leif@leifmadsen.com>
+
+	* Add initial support to build Docker images
+
+	  This work-in-progress is the first step to being able to reliably
+	  build Asterisk containers from the Asterisk source. I'm submitting
+	  this based on feedback gained at AstriDevCon 2015.
+
+	  Information about how to use this is provided in contrib/docker/README.md
+	  and will result in a local Asterisk container being built right from
+	  your source. I believe this can eventually be automated via
+	  hub.docker.com.
+
+	  Change-Id: Ifa070706d40e56755797097b6ed72c1e243bd0d1
+
+2016-03-11 12:22 +0000 [810f92c9dc]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix mwi resub deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023 #close
+
+	  Change-Id: I96d429c57a48861fd8bde63dd93db4e92dc3adb6
+
+2016-03-10 17:01 +0000 [72c444ba37]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix registration timeout and expire deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023
+
+	  Change-Id: I2e40de89efc8ae6e8850771d089ca44bc604b508
+
+2016-03-09 16:26 +0000 [7ea1e181dc]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix waitid deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  * Made always run check_pendings() under the scheduler thread so scheduler
+	  ids can be checked safely.
+
+	  ASTERISK-25023
+
+	  Change-Id: Ia834d6edd5bdb47c163e4ecf884428a4a8b17d52
+
+2016-03-10 12:17 +0000 [fbf8e04aed]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix t38id deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023
+
+	  Change-Id: If595e4456cd059d7171880c7f354e844c21b5f5f
+
+2016-03-08 15:08 +0000 [02458cc6fd]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix session timers deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023
+
+	  Change-Id: I6d65269151ba95e0d8fe4e9e611881cde2ab4900
+
+2016-03-09 16:34 +0000 [c7fdff2e37]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix reinviteid deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023
+
+	  Change-Id: I9c11b9d597468f63916c99e1dabff9f4a46f84c1
+
+2016-03-07 13:21 +0000 [69810b306d]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix autokillid deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  * Fix clearing autokillid in __sip_autodestruct() even though we could
+	  reschedule.
+
+	  ASTERISK-25023
+
+	  Change-Id: I450580dbf26e2e3952ee6628c735b001565c368f
+
+2016-03-09 16:32 +0000 [f484ddbdfe]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix packet retransid deadlock potential.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  * Fix retrans_pkt() to call check_pendings() with both the owner channel
+	  and the private objects locked as required.
+
+	  * Refactor dialog retransmission packet list to safely remove packet
+	  nodes.  The list nodes are now ao2 objects.  The list has a ref and the
+	  scheduled entry has a ref.
+
+	  ASTERISK-25023
+
+	  Change-Id: I50926d81be53f4cd3d572a3292cd25f563f59641
+
+2016-03-07 18:28 +0000 [67c79c326d]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix provisional_keepalive_sched_id deadlock.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Stopping a scheduled event can result in a deadlock if the scheduled event
+	  is running when you try to stop the event.  If you hold a lock needed by
+	  the scheduled event while trying to stop the scheduled event then a
+	  deadlock can happen.  The general strategy for resolving the deadlock
+	  potential is to push the actual starting and stopping of the scheduled
+	  events off onto the scheduler/do_monitor() thread by scheduling an
+	  immediate one shot scheduled event.  Some restructuring may be needed
+	  because the code may assume that the start/stop of the scheduled events is
+	  immediate.
+
+	  ASTERISK-25023
+
+	  Change-Id: I98a694fd42bc81436c83aa92de03226e6e4e3f48
+
+2016-03-09 11:22 +0000 [76be7093cd]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Adjust how dialog_unlink_all() stops scheduled events.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  * Make dialog_unlink_all() unschedule all items at once in the sched
+	  thread.
+
+	  ASTERISK-25023
+
+	  Change-Id: I7743072fb228836e8228b72f6dc46c8cc50b3fb4
+
+2016-03-10 21:54 +0000 [52f0932e4c]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Clear scheduled immediate events on unload.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  The reordering of chan_sip's shutdown is to handle any immediate events
+	  that get put onto the scheduler so resources aren't leaked.  The typical
+	  immediate events at this time are going to be concerned with stopping
+	  other scheduled events.
+
+	  ASTERISK-25023
+
+	  Change-Id: I3f6540717634f6f2e84d8531a054976f2bbb9d20
+
+2016-03-15 14:51 +0000 [0987a11cce]  Richard Mudgett <rmudgett@digium.com>
+
+	* sip/dialplan_functions.c: Fix /channels/chan_sip/test_sip_rtpqos crash.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  Delaying destruction of the chan_sip sip_pvt structures caused the
+	  /channels/chan_sip/test_sip_rtpqos unit test to crash.  That test
+	  registers a special test ast_rtp_engine with the rtp engine module.  When
+	  the unit test completes it cleans up by unregistering the test
+	  ast_rtp_engine and exits.  Since the delayed destruction of the sip_pvt
+	  happens after the unit test returns, the destructor tries to call the rtp
+	  engine destroy callback of the test ast_rtp_engine auto variable which no
+	  longer exists on the stack.
+
+	  * Change the test ast_rtp_engine auto variable to a static variable.  Now
+	  the variable can still exist after the unit test exits so the delayed
+	  sip_pvt destruction can complete successfully.
+
+	  ASTERISK-25023
+
+	  Change-Id: I61e34a12d425189ef7e96fc69ae14993f82f3f13
+
+2016-03-07 15:50 +0000 [9a7cfa2b61]  Richard Mudgett <rmudgett@digium.com>
+
+	* sched.c: Ensure oldest expiring entry runs first.
+
+	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+
+	  * Updated sched unit test to check new behavior.
+
+	  ASTERISK-25023
+
+	  Change-Id: Ib69437327b3cda5e14c4238d9ff91b2531b34ef3
+
+2016-03-15 13:31 +0000 [7964e260d3]  Andrew Nagy <andrew.nagy@the159.com>
+
+	* app_stasis: Don't hang up if app is not registered
+
+	  This prevents pbx_core from hanging up the channel if the app isn't
+	  registered.
+
+	  ASTERISK-25846 #close
+
+	  Change-Id: I63216a61f30706d5362bc0906b50b6f0544aebce
+2016-03-07 18:56 +0000 [cb97198ca6]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Simplify sip_pvt destructor call levels.
+
+	  Remove destructor calling destroy_it calling really_destroy_it
+	  for no benefit.  Just make the destructor the really_destroy_it
+	  function.
+
+	  Change-Id: Idea0d47b27dd74f2488db75bcc7f353d8fdc614a
+
+2016-03-04 18:25 +0000 [8be01398d9]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Made sip_reinvite_retry() call sip_pvt_lock_full().
+
+	  Change-Id: I90f04208a089f95488a2460185a8dbc3f6acca12
+
+2016-03-14 08:59 +0000 [4df7b3ae80]  Joshua Colp <jcolp@digium.com>
+
+	* build: Add configure check for proto field of PJSIP TLS transport setting.
+
+	  Older versions of PJSIP do not have the proto field on the TLS transport
+	  setting structure. This change adds a configure check so even if it is
+	  not present we will still be able to build.
+
+	  Change-Id: Ibf3f47befb91ed1b8194bf63888baa6fee05aba9
+
+2016-03-12 16:02 +0000 [0af6b5de62]  gtjoseph <george.joseph@fairview5.com>
+
+	* build_system:  Split COMPILE_DOUBLE from DONT_OPTIMIZE
+
+	  I can't ever recall actually needing the intermediate files or the checking
+	  that a double compile produces.  What I CAN remember is every DONT_OPTIMIZE
+	  build needing 3 invocations of gcc instead of 1 just to do the checks and
+	  produce those intermediate files.
+
+	  Having said that, Richard pointed out that the reason for the double compile
+	  was that there were cases in the past where a submitted patch failed to compile
+	  because the submitter never tried it with the optimizations turned on.
+
+	  To get the best of both worlds, COMPILE_DOUBLE has been split into its own
+	  option.  If DONT_OPTIMIZE is turned on, COMPILE_DOUBLE will also be selected
+	  BUT you can then turn it off if all you need are the debugging symbols.  This
+	  way you have to make an informed decision about disabling COMPILE_DOUBLE.
+
+	  To allow COMPILE_DOUBLE to be both auto-selected and turned off, a new feature
+	  was added to menuselect.  The <use> element can now contain an "autoselect"
+	  attribute which will turn the used member on but not create a hard dependency.
+	  The cflags.xml implementation for COMPILE_DOUBLE looks like this...
+
+	  <member name="DONT_OPTIMIZE" displayname="Disable Optimizations ...">
+	  	<use autoselect="yes">COMPILE_DOUBLE</use>
+	  	<support_level>core</support_level>
+	  </member>
+	  <member name="COMPILE_DOUBLE" displayname="Pre-compile with ...>
+	  	<depend>DONT_OPTIMIZE</depend>
+	  	<support_level>core</support_level>
+	  </member>
+
+	  When DONT_OPTIMIZE is turned on, COMPILE_DOUBLE is turned on because
+	  of the use.
+	  When DONT_OPTIMIZE is turned off, COMPILE_DOUBLE is turned off because
+	  of the depend.
+	  When COMPILE_DOUBLE is turned on, DONT_OPTIMIZE is turned on because
+	  of the depend.
+	  When COMPILE_DOUBLE is turned off, DONT_OPTIMIZE is left as is because
+	  it only uses COMPILE_DOUBLE, it doesn't depend on it.
+
+	  I also made a few tweaks to the ncurses implementation to move things
+	  left a bit to allow longer descriptions.
+
+	  Change-Id: Id49ca930ac4b5ec4fc2d8141979ad888da7b1611
+
+2016-03-10 13:09 +0000 [638133131a]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject:  Pass (dont_)optimize flags to pjproject and fix pjsua
+
+	  The pjproject Makefile now uses the Asterisk optimization flags which
+	  are determined by the setting of the DONT_OPTMIZE menuselect flag.
+	  The Makefile was also restructured so a change to the top level
+	  menuselect.makeopts will result in a rebuild of pjproject.
+
+	  Also, "--disable-resample" was removed from the pjproject configure
+	  options.  Without resample, pjsua (which is used by the testsuite)
+	  can't make audio calls.  When it can't, it segfaults.
+
+	  Change-Id: I24b0a4d0872acef00ed89b3c527a713ee4c2ccd4
+
+2016-03-11 16:03 +0000 [dcb25bb057]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* app_chanspy: Fix occasional deadlock with ChanSpy and Local channels.
+
+	  Channel masquerading had a conflict with autochannel locking.
+
+	  When locking autochannel->channel, the channel is fetched from the
+	  autochannel and then locked. During the fetch, the autochannel -- which
+	  has no locks itself -- can be modified by someone who owns the channel
+	  lock. That means that the value of autochan->channel cannot be trusted
+	  until you hold the lock.
+
+	  In practice, this caused problems with Local channels getting
+	  masqueraded away while the ChanSpy attempted to get info from that
+	  channel. The old channel which was about to get removed got locked, but
+	  the new (replaced) channel got unlocked (no-op). Because the replaced
+	  channel was now locked (and would never get unlocked), it couldn't get
+	  removed from the channel list in a timely manner, and would now cause
+	  deadlocks when iterating over the channel list.
+
+	  This change checks the autochannel after locking the channel for changes
+	  to the autochannel. If the channel had been changed, the lock is
+	  reobtained on the new channel.
+
+	  In theory it seems possible that after this fix, the lock attempt on the
+	  old (wrong) channel can be on an already destroyed lock, maybe causing
+	  a crash. But that hasn't been observed in the wild and is harder induce
+	  than the current deadlock.
+
+	  Thanks go to Filip Frank for suggesting a fix similar to this and
+	  especially to IRC user hexanol for pointing out why this deadlock was
+	  possible and testing this fix. And to Richard for catching my rookie
+	  while loop mistake ;)
+
+	  ASTERISK-25321 #close
+
+	  Change-Id: I293ae0014e531cd0e675c3f02d1d118a98683def
+
+2016-03-07 21:34 +0000 [fb28049de2]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjproject_bundled: Remove --with-external-pa from configure options.
+
+	  Not sure why it was there in the first place as we already specify
+	  --disable-sound.
+
+	  Change-Id: Ia80a40e8b1e1acc287955ab11ba1fbd0c7d4cff9
+
+2016-03-06 14:38 +0000 [d2eb65f71e]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Strip spaces from items parsed from comma-separated lists
+
+	  Configurations like "aors = a, b, c" were either ignoring everything after "a"
+	  or trying to look up " b".  Same for mailboxes,  ciphers, contacts and a few
+	  others.
+
+	  To fix, all the strsep(&copy, ",") calls have been wrapped in ast_strip.  To
+	  facilitate this, ast_strip, ast_skip_blanks and ast_skip_nonblanks were
+	  updated to handle null pointers.
+
+	  In some cases, an ast_strlen_zero() test was added to skip consecutive commas.
+
+	  There was also an attempt to ast_free an ast_strdupa'd string in
+	  ast_sip_for_each_aor which was causing a SEGV.  I removed it.
+
+	  Although this issue was reported for realtime, the issue was in the res_pjsip
+	  modules so all config mechanisms were affected.
+
+	  ASTERISK-25829 #close
+	  Reported-by: Mateusz Kowalski
+
+	  Change-Id: I0b22a2cf22a7c1c50d4ecacbfa540155bec0e7a2
+
+2016-03-07 02:02 +0000 [f690c105f3]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* res_odbc_transaction: fix some format tab
+
+	  Change-Id: I265e4ac47c629c9a63dd86b59df82a7ab3c64384
+
+2016-02-17 22:58 +0000 [0ec9fe5421]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* main/cli.c: Refactor function to print seconds formatted
+
+	  Refactor and created function ast_cli_print_timestr_fromseconds to print
+	  seconds formatted:  year(s) week(s) day(s) hour(s) second(s)
+
+	  This function now is used in addons/cdr_mysql.c,cdr_pgsql.c, main/cli.c,
+	  res_config_ldap.c, res_config_pgsql.c.
+
+	  Change-Id: Ibeb8634102cd11d3f8623398b279cb731bcde36c
+
+2016-03-04 20:37 +0000 [471ff375fd]  gtjoseph <george.joseph@fairview5.com>
+
+	* install_prereq: Add packages for bundled pjproject
+
+	  RedHat/CentOS needs python-devel
+	  Debian/Ubuntu needs automake, libsrtp-dev and python-dev
+
+	  Ubuntu also needed libncurses5-dev for cmenuselect so while not
+	  needed for pjproject, I adedd it anyway.
+
+	  Change-Id: Idf5fa16e2d87c687439621507e122cb9461d7089
+
+2016-02-24 17:25 +0000 [2b9849625c]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_caller_id: Anonymize 'From' when caller id presentation is prohibited
+
+	  Per RFC3325, the 'From' header is now anonymized on outgoing calls when
+	  caller id presentation is prohibited.
+
+	  TID = trust_id_outbound
+	  PRO = Set(CALLERID(pres)=prohib)
+	  USR = endpoint/from_user
+	  DOM = endpoint/from_domain
+	  PAI = YES(privacy=off), NO(not sent), PRI(privacy=full) (assumes send_pai=yes)
+
+	  Conditions          |Result
+	  --------------------|----------------------------------------------------
+	  TID PRO USR DOM     |PAI    FROM
+	  --------------------|----------------------------------------------------
+	  Y   Y   abc def.ghi |PRI    "Anonymous" <sip:abc@def.ghi>
+	  Y   Y   abc         |PRI    "Anonymous" <sip:abc@anonymous.invalid>
+	  Y   Y       def.ghi |PRI    "Anonymous" <sip:anonymous@def.ghi>
+	  Y   Y               |PRI    "Anonymous" <sip:anonymous@anonymous.invalid>
+
+	  Y   N   abc def.ghi |YES    <sip:abc@def.ghi>
+	  Y   N   abc         |YES    <sip:abc@<ip_address>>
+	  Y   N       def.ghi |YES    "Caller Name" <sip:<caller_exten>@def.ghi>
+	  Y   N               |YES    "Caller Name" <sip:<caller_exten>@<ip_address>>
+
+	  N   Y   abc def.ghi |NO     "Anonymous" <sip:abc@def.ghi>
+	  N   Y   abc         |NO     "Anonymous" <sip:abc@anonymous.invalid>
+	  N   Y       def.ghi |NO     "Anonymous" <sip:anonymous@def.ghi>
+	  N   Y               |NO     "Anonymous" <sip:anonymous@anonymous.invalid>
+
+	  N   N   abc def.ghi |YES    <sip:abc@def.ghi>
+	  N   N   abc         |YES    <sip:abc@<ip_address>>
+	  N   N       def.ghi |YES    "Caller Name" <sip:<caller_exten>@def.ghi>
+	  N   N               |YES    "Caller Name" <sip:<caller_exten>@<ip_address>>
+
+	  ASTERISK-25791 #close
+	  Reported-by: Anthony Messina
+
+	  Change-Id: I2c82a5ca1413c2c00fb62ea95b0ae8e97af54dc9
+
+2016-03-03 17:34 +0000 [37472f7398]  gtjoseph <george.joseph@fairview5.com>
+
+	* third_party/Makefile.rules:  Replace unsupported != operator with $(shell ...)
+
+	  Apparently the != operator is fairly new so I've replaced it with
+	  the old $(shell ...) syntax.
+
+	  Change-Id: I16b2e1878a4f91e7e9740abd427f9639f933c479
+	  Reported-by: Richard Mudgett
+
+2016-01-23 15:50 +0000 [195100e770]  gtjoseph <george.joseph@fairview5.com>
+
+	* loader: Retry dlopen when loading fails
+
+	  Although we use the RTLD_LAZY flag when calling dlopen
+	  the first time on a module, this only defers resolution
+	  for function calls.  Pointer references to functions are
+	  determined at link time so dlopen expects them to be there.
+	  Since we don't cross-module link, pointers to functions
+	  in other modules won't be available and dlopen will fail.
+
+	  Doing a "hardened" build also causes problems because it
+	  typically sets "-z now" on the ld command line which
+	  overrides RTLD_LAZY at run time.
+
+	  If the failing module isn't a GLOBAL_SYMBOLS module, then
+	  dlopen will be called again after all the GLOBAL_SYMBOLS
+	  modules have been loaded and they'll eventually resolve.
+
+	  If the calling module IS a GLOBAL_SYMBOLS module itself
+	  and a third module depends on it, then there's an issue
+	  because the second time through the dlopen loop,
+	  GLOBAL_SYMBOLS modules aren't given any special treatment
+	  and since the order in which dlopen is called isn't
+	  deterministic, the dependent may again be tried before the
+	  module it needs is loaded.
+
+	  Simple solution:  Save modules that fail load_resource
+	  because of a dlopen error in a list and retry them
+	  immediately after the first pass. Keep retrying until
+	  the failed list is empty or we reach a #defined max
+	  retries. Error messages are suppressed until the final
+	  pass which also gets rid of those confusing error messages
+	  about module failures that are later corrected.
+
+	  Change-Id: Iddae1d97cd2f00b94e61662447432765755f64bb
+
+2016-03-01 16:18 +0000 [15c5743ac1]  Kevin Harwell <kharwell@digium.com>
+
+	* bridge.c: Crash during attended transfer when missing a local channel half
+
+	  It's possible for the transferer channel to get hung up early during the
+	  attended transfer process. For instance, a phone may send a "bye" immediately
+	  upon receiving a sip notify that contains a sip frag 100 (I'm looking at you
+	  Jitsi). When this occurs a race begins between the transferer being hung up
+	  and completion of the transfer code.
+
+	  If the channel hangs up too early during a transfer involving stasis bridging
+	  for instance, then when the created local channel goes to look up its swap
+	  channel (and associated datastore) it can't find it (since it is no longer in
+	  the bridge) thus it fails to enter the stasis application. Consequently, the
+	  created local channel(s) hang up as well. If the timing is just right then the
+	  bridging code attempts to add the message link with missing local channel(s).
+	  Hence the crash.
+
+	  Unfortunately, there is no great way to solve the problem of the unexpected
+	  "bye". While we can't guarantee we won't receive an early hangup, and in this
+	  case still fail to enter the stasis application, we can make it so asterisk
+	  does not crash.
+
+	  This patch does just that by locking the local channel structure, checking
+	  that the local channel's peer has not been lost, and then continuing. This
+	  keeps the local channel's peer from being ripped out from underneath it by
+	  the local/unreal hangup code while attempting to set the stasis message link.
+
+	  ASTERISK-25771
+
+	  Change-Id: Ie6d6061e34c7c95f07116fffac9a09e5d225c880
+
+2016-03-01 18:08 +0000 [0d2ccbca62]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_refer.c: Delay sending the initial SIP Notify with frag 100
+
+	  During the transfer process, some phones (okay it was the Jitsi softphone,
+	  but maybe others are out there) send a "bye" immediately after receiving a
+	  SIP Notify. When a "bye" is received early for some types of transfers the
+	  transferer channel may no longer be available during late stage transfer
+	  processing.
+
+	  For instance, during an attended transfer involving stasis bridging at one
+	  point the created local channel looks for an associated swap channel in
+	  order to retrieve the stasis application name. If the transferer has hung
+	  up then the local channel will fail to find it. The local channel then has
+	  no way to know which stasis app to enter, so it fails and hangs up as well.
+	  Thus the transfer does not complete as expected.
+
+	  This patch delays the sending of the initial notify in order to give the
+	  transfer process enough time to gather the necessary data for a successful
+	  transfer.
+
+	  ASTERISK-25771
+
+	  Change-Id: I09cfc9a5d6ed4c007bc70625e0972b470393bf16
+
+2016-03-03 08:26 +0000 [6af7fc4c37]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_dtmf_info: NULL terminate the message body.
+
+	  PJSIP does not ensure that when printing the message body the
+	  buffer will be NULL terminated. This is problematic when searching
+	  for the signal and duration values of the DTMF.
+
+	  This change ensures the buffer is always NULL terminated.
+
+	  Change-Id: I52653a1a60c93092d06af31a27408d569cc98968
+
+2016-03-01 20:03 +0000 [b8b7c2e428]  gtjoseph <george.joseph@fairview5.com>
+
+	* alembic: Fix downgrade and tweak for sqlite
+
+	  Downgrade had a few issues.  First there was an errant 'update' statement in
+	  add_auto_dtmf_mode that looks like it was a copy/paste error.  Second, we
+	  weren't cleaning up the ENUMs so subsequent upgrades on postgres failed
+	  because the types already existed.
+
+	  For sqlite...  sqlite doesn't support ALTER or DROP COLUMN directly.
+	  Fortunately alembic batch_operations takes care of this for us if we
+	  use it so the alter and drops were converted to use batch operations.
+
+	  Here's an example downgrade:
+
+	      with op.batch_alter_table('ps_endpoints') as batch_op:
+	          batch_op.drop_column('tos_audio')
+	          batch_op.drop_column('tos_video')
+	          batch_op.add_column(sa.Column('tos_audio', yesno_values))
+	          batch_op.add_column(sa.Column('tos_video', yesno_values))
+	          batch_op.drop_column('cos_audio')
+	          batch_op.drop_column('cos_video')
+	          batch_op.add_column(sa.Column('cos_audio', yesno_values))
+	          batch_op.add_column(sa.Column('cos_video', yesno_values))
+
+	      with op.batch_alter_table('ps_transports') as batch_op:
+	          batch_op.drop_column('tos')
+	          batch_op.add_column(sa.Column('tos', yesno_values))
+	      # Can't cast integers to YESNO_VALUES, so dropping and adding is required
+	          batch_op.drop_column('cos')
+	          batch_op.add_column(sa.Column('cos', yesno_values))
+
+	  Upgrades from base to head and downgrades from head to base were tested
+	  repeatedly for postgresql, mysql/mariadb, and sqlite3.
+
+	  Change-Id: I862b0739eb3fd45ec3412dcc13c2340e1b7baef8
+
+2016-03-02 15:55 +0000 [7b71bca8a4]  gtjoseph <george.joseph@fairview5.com>
+
+	* config_transport:  Fix objects returned by ast_sip_get_transport_states
+
+	  ast_sip_get_transport_states was returning a container of internal_state
+	  objects instead of ast_sip_transport_state objects.  This was causing
+	  transport lookups to fail, most noticably in res_pjsip_nat, which
+	  couldn't find the correct external addresses.  This was causing contacts
+	  to go out with internal ip addresses.
+
+	  ASTERISK-25830 #close
+	  Reported-by: Sean Bright
+
+	  Change-Id: I1aee6a2fd46c42e8dd0af72498d17de459ac750e
+
+2016-03-02 11:17 +0000 [0a3f0e85ac]  Scott Griepentrog <scott@griepentrog.com>
+
+	* CHAOS: cleanup possible null vars on msg alloc failure
+
+	  In message.c, if msg_alloc fails to init the string field,
+	  vars may be null, so use a null tolerant cleanup.
+
+	  In res_pjsip_messaging.c, if msg_data_create fails, mdata
+	  will be null, so use a null tolerant cleanup.
+
+	  ASTERISK-25323
+
+	  Change-Id: Ic2d55c2c3750d5616e2a05ea92a19c717507ff56
+
+2016-03-02 09:34 +0000 [60aa871be3]  Scott Griepentrog <scott@griepentrog.com>
+
+	* CHAOS: prevent crash on failed strdup
+
+	  This patch avoids crashing on a null pointer
+	  if the strdup() allocation fails.
+
+	  ASTERISK-25323
+
+	  Change-Id: I3f67434820ba53b53663efd6cbb42749f4f6c0f5
+
+2016-02-29 18:11 +0000 [0bdbf0d882]  Richard Mudgett <rmudgett@digium.com>
+
+	* func_callerid.c: Update REDIRECTING reason documentation.
+
+	  Change-Id: I6e8d39b0711110a4bceafa652e58b30465e28386
+
+2016-02-26 18:57 +0000 [25de01f301]  Richard Mudgett <rmudgett@digium.com>
+
+	* SIP diversion: Fix REDIRECTING(reason) value inconsistencies.
+
+	  Previous chan_sip behavior:
+
+	  Before this patch chan_sip would always strip any quotes from an incoming
+	  reason and pass that value up as the REDIRECTING(reason).  For an outgoing
+	  reason value, chan_sip would check the value against known values and
+	  quote any it didn't recognize.  Incoming 480 response message reason text
+	  was just assigned to the REDIRECTING(reason).
+
+	  Previous chan_pjsip behavior:
+
+	  Before this patch chan_pjsip would always pass the incoming reason value
+	  up as the REDIRECTING(reason).  For an outgoing reason value, chan_pjsip
+	  would send the reason value as passed down.
+
+	  With this patch:
+
+	  Both channel drivers match incoming reason values with values documented
+	  by REDIRECTING(reason) and values documented by RFC5806 regardless of
+	  whether they are quoted or not.  RFC5806 values are mapped to the
+	  equivalent REDIRECTING(reason) documented value and is set in
+	  REDIRECTING(reason).  e.g., an incoming RFC5806 'unconditional' value or a
+	  quoted string version ('"unconditional"') is converted to
+	  REDIRECTING(reason)'s 'cfu' value.  The user's dialplan only needs to deal
+	  with 'cfu' instead of any of the aliases.
+
+	  The incoming 480 response reason text supported by chan_sip checks for
+	  known reason values and if not matched then puts quotes around the reason
+	  string and assigns that to REDIRECTING(reason).
+
+	  Both channel drivers send outgoing known REDIRECTING(reason) values as the
+	  unquoted RFC5806 equivalent.  User custom values are either sent as is or
+	  with added quotes if SIP doesn't allow a character within the value as
+	  part of a RFC3261 Section 25.1 token.  Note that there are still
+	  limitations on what characters can be put in a custom user value.  e.g.,
+	  embedding quotes in the middle of the reason string is silly and just
+	  going to cause you grief.
+
+	  * Setting a REDIRECTING(reason) value now recognizes RFC5806 aliases.
+	  e.g., Setting REDIRECTING(reason) to 'unconditional' is converted to the
+	  'cfu' value.
+
+	  * Added missing malloc() NULL return check in res_pjsip_diversion.c
+	  set_redirecting_reason().
+
+	  * Fixed potential read from a stale pointer in res_pjsip_diversion.c
+	  add_diversion_header().  The reason string needed to be copied into the
+	  tdata memory pool to ensure that the string would always be available.
+	  Otherwise, if the reason string returned by reason_code_to_str() was a
+	  user's reason string then the string could be freed later by another
+	  thread.
+
+	  Change-Id: Ifba83d23a195a9f64d55b9c681d2e62476b68a87
+
+2016-02-26 18:54 +0000 [8c8ef4efb0]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_send_to_voicemail.c: Allow either quoted or not send_to_vm reason.
+
+	  Change-Id: Id6350b3c7d4ec8df7ec89863566645e2b0f441fd
+
+2016-02-29 20:41 +0000 [75ec137e91]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_send_to_voicemail.c: Fix off-nominal double channel unref.
+
+	  * Fix double unref of other_party channel in off nominal path.
+
+	  * This is unlikely to be a real problem.  However, for safety,
+	  in handle_incoming_request() keep the datastore ref with the
+	  other_party channel ref until we are finished with the other_party
+	  channel.
+
+	  Change-Id: I78f22547bf0bb99fb20814ceab75952bd857f821
+
+2016-01-18 21:54 +0000 [3173e91bab]  gtjoseph <george.joseph@fairview5.com>
+
+	* build-system: Allow building with static pjproject
+
+	  Background here:
+	  http://lists.digium.com/pipermail/asterisk-dev/2016-January/075266.html
+
+	  From CHANGES:
+	   * To help insure that Asterisk is compiled and run with the same known
+	     version of pjproject, a new option (--with-pjproject-bundled) has been
+	     added to ./configure.  When specified, the version of pjproject specified
+	     in third-party/versions.mak will be downloaded and configured.  When you
+	     make Asterisk, the build process will also automatically build pjproject
+	     and Asterisk will be statically linked to it.  Once a particular version
+	     of pjproject is configured and built, it won't be configured or built
+	     again unless you run a 'make distclean'.
+
+	     To facilitate testing, when 'make install' is run, the pjsua and pjsystest
+	     utilities and the pjproject python bindings will be installed in
+	     ASTDATADIR/third-party/pjproject.
+
+	     The default behavior remains building with the shared pjproject
+	     installation, if any.
+
+	  Building:
+
+	     All you have to do is include the --with-pjproject-bundled option on
+	     the ./configure command line (and remove any existing --with-pjproject
+	     option if specified).  Everything else is automatic.
+
+	  Behind the scenes:
+
+	     The top-level Makefile was modified to include 'third-party' in the
+	     list of MOD_SUBDIRS.
+
+	     The third-party directory was created to contain any third party
+	     packages that may be needed in the future.  Its Makefile automatically
+	     iterates over any subdirectories passing on targets.
+
+	     The third-party/pjproject directory was created to house the pjproject
+	     source distribution.  Its Makefile contains targets to download, patch
+	     configure, generate dependencies, compile libs, apps and python bindings,
+	     sanitized build.mak and generate a symbols list.
+
+	     When bootstrap.sh is run, it automatically includes the configure.m4
+	     file in third-party/pjproject.  This file has a macro to download and
+	     conifgure pjproject and get and set PJPROJECT_INCLUDE, PJPROJECT_DIR
+	     and PJPROJECT_BUNDLED.  It also tests for the capabilities like
+	     PJ_TRANSACTION_GRP_LOCK by parsing preprocessor output as opposed to
+	     trying to compile.  Of course, bootstrap.sh is only run once and the
+	     configure file is incldued in the patch.
+
+	     When configure is run with the new options, the macro in configure.m4
+	     triggers the download, patch, conifgure and tests.  No compilation is
+	     performed at this time.  The downloaded tarball is cached in /tmp so
+	     it doesn't get downloaded again on a distclean.
+
+	     When make is run in the top-level Asterisk source directory, it will
+	     automatically descend all the subdirectories in third_party just as it
+	     does for addons, apps, etc.  The top-level Makefile makes sure that
+	     the 'third-party' is built before 'main' so that dependencies from the
+	     other directories are built first.
+
+	     When main does build, a new shared library (libasteriskpj) is created that
+	     links statically to the pjproject .a files and exports all their symbols.
+	     The asterisk binary links to that, just as it does with libasteriskssl.
+
+	     When Asterisk is installed, the pjsua and pjsystest apps, and the pjproject
+	     python bindings are installed in ASTDATADIR/third-party/pjproject.  This
+	     will facilitate testing, including running the testsuite which will be
+	     updated to check that directory for the pjsua module ahead of the system
+	     python library.
+
+	  Modules should continue to depend on pjproject if they use pjproject APIs
+	  directly.  They should not care about the implementation.  No changes to any
+	  res_pjsip modules were made.
+
+	  Change-Id: Ia7a60c28c2e9ba9537c5570f933c1ebcb20a3103
+
+2016-02-22 16:59 +0000 [2dae4a1ccf]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Fix T.38 issues caused by leaving a bridge.
+
+	  chan_sip could not handle AST_T38_TERMINATED frames being sent to it when
+	  the channel left the bridge.  The action resulted in overlapping outgoing
+	  reINVITEs.  The testsuite tests/fax/sip/directmedia_reinvite_t38 was not
+	  happy.
+
+	  * Force T.38 to be remembered as locally bridged.  Now when the channel
+	  leaves the native RTP bridge after T.38, the channel remembers that it has
+	  already reINVITEed the media back to Asterisk.  It just needs to terminate
+	  T.38 when the AST_T38_TERMINATED arrives.
+
+	  * Prevent redundant AST_T38_TERMINATED from causing problems.  Redundant
+	  AST_T38_TERMINATED frames could cause overlapping outgoing reINVITEs if
+	  they happen before the T.38 state changes to disabled.  Now the T.38 state
+	  is set to disabled before the reINVITE is sent.
+
+	  ASTERISK-25582 #close
+
+	  Change-Id: I53f5c6ce7d90b3f322a942af1a9bcab6d967b7ce
+
+2016-02-18 18:27 +0000 [bf29a4e2e6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_t38.c: Back out part of an earlier fix attempt.
+
+	  This backs out item 4 of the 4875e5ac32f5ccad51add6a4216947bfb385245d
+	  commit.  Item 4 added the t38_bye_supplement.  Unfortunately, the frame
+	  that it puts into the bridge may or may not be processed by the time the
+	  bridged peer is kicked out of the bridge.  If it is processed then all is
+	  well.  However, if it is not processed then that channel is stuck in fax
+	  mode until it hangs up or maybe if it joins another bridge for T.38
+	  faxing.
+
+	  ASTERISK-25582
+
+	  Change-Id: Ib20a03ecadf1bf8a0dcadfadf6c2f2e60919a9f7
+
+2016-02-22 13:54 +0000 [c7d45b84f9]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge core: Add owed T.38 terminate when channel leaves a bridge.
+
+	  The channel is now going to get T.38 terminated when it leaves the
+	  bridging system and the bridged peers are going to get T.38 terminated as
+	  well.
+
+	  ASTERISK-25582
+
+	  Change-Id: I77a9205979910210e3068e1ddff400dbf35c4ca7
+
+2016-02-19 16:01 +0000 [0e296563d7]  Richard Mudgett <rmudgett@digium.com>
+
+	* channel api: Create is_t38_active accessor functions.
+
+	  ASTERISK-25582
+
+	  Change-Id: I69451920b122de7ee18d15bb231c80ea7067a22b
+
+2016-02-19 19:06 +0000 [86f7336c91]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_channel: Don't settle owed events on an optimization.
+
+	  Local channel optimization could cause DTMF digits to be duplicated.
+	  Pending DTMF end events would be posted to a bridge when the local channel
+	  optimizes out and is replaced by the channel further down the chain.  When
+	  the real digit ends, the channel would get another DTMF end posted to the
+	  bridge.
+
+	  A -- LocalA;1/n -- LocalA;2/n -- LocalB;1 -- LocalB;2 -- B
+
+	  1) LocalA has the /n flag to prevent optimization.
+	  2) B is sending DTMF to A through the local channel chain.
+	  3) When LocalB optimizes out it can move B to the position of LocalB;1
+	  4) Without this patch, when B swaps with LocalB;1 then LocalB;1 would
+	  settle an owed DTMF end to the bridge toward LocalA;2.
+	  5) When B finally ends its DTMF it sends the DTMF end down the chain.
+	  6) Without this patch, A would hear the DTMF digit end when LocalB
+	  optimizes out and when B ends the original digit.
+
+	  ASTERISK-25582
+
+	  Change-Id: I1bbd28b8b399c0fb54985a5747f330a4cd2aa251
+
+2016-02-22 12:15 +0000 [128c96456c]  Richard Mudgett <rmudgett@digium.com>
+
+	* channel.c: Route all control frames to a channel through the same code.
+
+	  Frame hooks can conceivably return a control frame in exchange for an
+	  audio frame inside ast_write().  Those returned control frames were not
+	  handled quite the same as if they were sent to ast_indicate().  Now it
+	  doesn't matter if you use ast_write() to send an AST_FRAME_CONTROL to a
+	  channel or ast_indicate().
+
+	  ASTERISK-25582
+
+	  Change-Id: I5775f41421aca2b510128198e9b827bf9169629b
+
+2016-02-25 15:13 +0000 [4422905218]  gtjoseph <george.joseph@fairview5.com>
+
+	* sorcery:  Refactor create, update and delete to better deal with caches
+
+	  The ast_sorcery_create, update and delete function have been refactored
+	  to better deal with caches and errors.
+
+	  The action is now called on all non-caching wizards first. If ANY succeed,
+	  the action is called on all caching wizards and the observers are notified.
+	  This way we don't put something in the cache (or update or delete) before
+	  knowing the action was performed in at least 1 backend and we only call the
+	  observers once even if there were multiple writable backends.
+
+	  ast_sorcery_create was never adding to caches in the first place which
+	  was preventing contacts from getting added to a memory_cache when they
+	  were created.  In turn this was causing memory_cache to emit errors if
+	  the contact was deleted before being retrieved (which would have
+	  populated the cache).
+
+	  ASTERISK-25811 #close
+	  Reported-by: Ross Beer
+
+	  Change-Id: Id5596ce691685a79886e57b0865888458d6e7b46
+2016-02-25 15:39 +0000 [acf329a3c7]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_mwi:  Turn some NOTICEs and WARNINGs into debug 1s.
+
+	  There are a few cases where we're emitting notices or warnings
+	  for things that really need neither, like a client retrying to subscribe
+	  to mwi when they're not conifgured for it.  They get a 404 so there's no
+	  need for non-debug messages.
+
+	  Change-Id: I05e38a7ff6c2f2521146f4be6a79731b9864e61f
+2016-02-25 14:17 +0000 [7e3e1ddf7e]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_sorcery_memory_cache:  Fix SEGV in some CLI commands
+
+	  A few of the CLI commands weren't checking for enough arguments
+	  and were SEGVing.
+
+	  Change-Id: Ie6494132ad2fe54b4f014bcdc112a37c36a9b413
+
+2016-02-22 19:31 +0000 [803a2fc2d5]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.h: Remove extraneous semicolons.
+
+	  Change-Id: Ib462633d396fa941379dfef648dcd2245e350084
+
+2016-02-23 14:57 +0000 [886ee09471]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Suppress T.38 SDP c= line if addr is the same.
+
+	  Use the correct comparison function since we only care if the address
+	  without the port is the same.
+
+	  Change-Id: Ibf6c485f843a1be6dee58a47b33d81a7a8cbe3b0
+
+2016-02-16 08:14 +0000 [b7970cabfa]  Christof Lauber <christof.lauber@annax.ch>
+
+	* res_config_sqlite3: Fix crashes when reading peers from sqlite3 tables
+
+	  Introduced realloaction of ast_str buf in sqlite3_escape functions in case
+	  the returned buffer from threadstorage was actually too small.
+
+	  Change-Id: I3c5eb43aaade93ee457943daddc651781954c445
+
+2016-02-11 11:01 +0000 [ba8adb4ce3]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/config_transport: Allow reloading transports.
+
+	  The 'reload' mechanism actually involves closing the underlying
+	  socket and calling the appropriate udp, tcp or tls start functions
+	  again.  Only outbound_registration, pubsub and session needed work
+	  to reset the transport before sending requests to insure that the
+	  pjsip transport didn't get pulled out from under them.
+
+	  In my testing, no calls were dropped when a transport was changed
+	  for any of the 3 transport types even if ip addresses or ports were
+	  changed. To be on the safe side however, a new transport option was
+	  added (allow_reload) which defaults to 'no'.  Unless it's explicitly
+	  set to 'yes' for a transport, changes to that transport will be ignored
+	  on a reload of res_pjsip.  This should preserve the current behavior.
+
+	  Change-Id: I5e759850e25958117d4c02f62ceb7244d7ec9edf
+
+2016-02-19 04:30 +0000 [c00082329e]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* chan_sip: Optionally supply fromuser/fromdomain in SIP dial string.
+
+	  Previously you could add [!dnid] to the SIP dial string to alter the To:
+	  header. This change allows you to alter the From header as well.
+
+	  SIP dial string extra options now look like this:
+
+	      [![touser[@todomain]][![fromuser][@fromdomain]]]
+
+	  INCOMPATIBLE CHANGE: If you were using an exclamation mark in your To:
+	  header, that is no longer possible.
+
+	  ASTERISK-25803 #close
+
+	  Change-Id: I2457e9ba7a89eb1da22084bab5a4d4328e189db7
+
+2016-02-07 17:34 +0000 [f8767a8804]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjproject:  Add ability to map pjproject log levels to Asterisk log levels
+
+	  Warnings and errors in the pjproject libraries are generally handled by
+	  Asterisk.  In many cases, Asterisk wouldn't even consider them to be warnings
+	  or errors so the messages emitted by pjproject directly are either superfluous
+	  or misleading.  A good exampe of this are the level-0 errors pjproject emits
+	  when it can't open a TCP/TLS socket to a client to send an OPTIONS.  We don't
+	  consider a failure to qualify a UDP client an "ERROR", why should a TCP/TLS
+	  client be treated any differently?
+
+	  A config file for res_pjproject has bene added (pjproject.conf) and a new
+	  log_mappings object allows mapping pjproject levels to Asterisk levels
+	  (or nothing).  The defaults if no pjproject.conf file is found are the same
+	  as those that were hard-coded into res_pjproject initially: 0,1 = LOG_ERROR,
+	  2 = LOG_WARNING, 3,4,5 = LOG_DEBUG<level>
+
+	  Change-Id: Iba7bb349c70397586889b8f45b8c3d6c6c8c3898
+
+2016-02-18 10:55 +0000 [14886643c6]  Alexei Gradinari <alex2grad@gmail.com>
+
+	* res_pjsip_outbound_publish: Fix processing 412 response
+
+	  When Asterisk receives a 412 (Conditional Request Failed) response
+	  it has to recreate publish session.
+	  There is bug in res_pjsip_outbound_publish.c
+	  The function sip_outbound_publish_client_alloc is called with wrong object
+	  while processing 412 (Conditional Request Failed) response.
+	  This patch fixes it.
+
+	  ASTERISK-25229 #close
+
+	  Change-Id: I3b62f2debf6bb1e5817cde7b13ea39ef2bf14359
+
+2016-02-18 11:15 +0000 [8055d080cd]  Mark Michelson <mmichelson@digium.com>
+
+	* Fix failing threadpool_auto_increment test.
+
+	  The threadpool_auto_increment test fails infrequently for a couple of
+	  reasons
+	  * The threadpool listener was notified of fewer tasks being pushed than
+	    were actually pushed
+	  * The "was_empty" flag was set to an unexpected value.
+
+	  The problem is that the test pushes three tasks into the threadpool.
+	  Test expects the threadpool to essentially gather those three tasks, and
+	  then distribute those to the threadpool threads. It also expects that as
+	  the tasks are pushed in, the threadpool listener is alerted immediately
+	  that the tasks have been pushed. In reality, a task can be distributed
+	  to the threadpool threads quicker than expected, meaning that the
+	  threadpool has already emptied by the time each subsequent task is
+	  pushed. In addition, the internal threadpool queue can be delayed so
+	  that the threadpool listener is not alerted that a task has been pushed
+	  even after the task has been executed.
+
+	  From the test's point of view, there's no way to be able to predict
+	  exactly the order that task execution/listener notifications will occur,
+	  and there is no way to know which listener notifications will indicate
+	  that the threadpool was previously empty.
+
+	  For this reason, the test has been updated to only check the things it
+	  can check. It ensures that all tasks get executed, that the threads go
+	  idle after the tasks are executed, and that the listener is told the
+	  proper number of tasks that were pushed.
+
+	  Change-Id: I7673120d74adad64ae6894594a606e102d9a1f2c
+
+2016-02-17 13:30 +0000 [30a49b8a6a]  Richard Mudgett <rmudgett@digium.com>
+
+	* cel.c: Fix mismatch in ast_cel_track_event() return type.
+
+	  The return type of ast_cel_track_event() is not large enough to return all
+	  64 potential bits of the event enable mask.  Fortunately, the defined CEL
+	  events do not really need all 64 bits and the return value is only used to
+	  determine if the requested CEL event is enabled.
+
+	  * Made the ast_cel_track_event() return 0 or 1 only so the return value
+	  can fit inside an int type instead of zero or a truncated 64 bit non-zero
+	  value.
+
+	  Change-Id: I783d932320db11a95c7bf7636a72b6fe2566904c
+
+2016-02-16 23:37 +0000 [15aeb78c66]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: fix Calculate talktime when is first call answered
+
+	  Fix calculate of average time for talktime is wrong when is completed the
+	  first call beacuse the time for talked would be that call.
+
+	  ASTERISK-25800 #close
+
+	  Change-Id: I94f79028935913cd9174b090b52bb300b91b9492
+
+2016-02-16 16:37 +0000 [62282bb8ce]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_odbc: Fix exports.in for missing symbols
+
+	  res_odbc.exports.in was missing a few symbols.
+	  Changed to wildcards.
+
+	  Change-Id: Ieadd76df24e43ea92577f651d478a0f7b742c30c
+
+2016-02-16 12:20 +0000 [49203628f9]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_statsd:  Fix exports.in for missing symbols
+
+	  res_statsd.export.in was missing the _va variations of the log
+	  functions causing Asterisk to crash in res_pjsip if OPTIONAL_API
+	  wasn't enabled.
+
+	  ASTERISK-25727 #close
+	  Reported-by: Gergely Dömsödi
+
+	  Change-Id: I395729f9f51bdd33c5ca757f5f96ebedad74077b
+
+2016-02-15 21:31 +0000 [4f08e9fb64]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_config_wizard:  Add command to export primitive objects
+
+	  A new command (pjsip export config_wizard primitives) has been added that
+	  will export all the pjsip objects it created to the console or a file
+	  suitable for reuse in a pjsip.conf file.
+
+	  ASTERISK-24919 #close
+	  Reported-by: Ray Crumrine
+
+	  Change-Id: Ica2a5f494244b4f8345b0437b16d06aa0484452b
+
+2016-02-15 15:37 +0000 [be811c4be1]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_caller_id: Fix segfault when replacing rpid or pai header
+
+	  If the PJSIP_HEADER dialplan function adds a PAI or RPID header and send_rpid
+	  or send_pai is set, res_pjsip_caller_id attemps to retrieve, parse and modify
+	  the header added by the dialplan function.  Since the header added by the
+	  dialplan function is generic string, there are no virtual functions to parse
+	  the uri and we get a segfault when we try.  Since the modify, was really only
+	  an overwrite, we now just delete the old header if it was type PJSIP_H_OTHER
+	  and recreate it.
+
+	  This raises a question for another time though:  What should happen with
+	  duplicate headers?  Right now res_pjsip_header_funcs doesn't check for dups
+	  so if it's session supplement is loaded after res_pjsip_caller_id's (or any
+	  other module that adds headers), there'll be dups in the message.
+
+	  ASTERISK-25337 #close
+
+	  Change-Id: I5e296b52d30f106b822c0eb27c4c2b0e0f71c7fa
+
+2016-02-15 13:08 +0000 [13b6c02945]  Mark Michelson <mmichelson@digium.com>
+
+	* Fix creation race of contact_status structures.
+
+	  It is possible when processing a SIP REGISTER request to have two
+	  threads end up creating contact_status structures in sorcery.
+	  contact_status is created using a "find or create" function. If two
+	  threads call into this at the same time, each thread will fail to find
+	  an existing contact_status, and so both will end up creating a new
+	  contact status.
+
+	  During testing, we would see sporadic failures because the
+	  PJSIP_CONTACT() dialplan function would operate on a different
+	  contact_status than what had been updated by res_pjsip/pjsip_options.
+
+	  The fix here is two-fold:
+	  1) The "find or create" function for contact_status now has a lock
+	  around the entire operation. This way, if two threads attempt the
+	  operation simultaneously, the first to get there will create the object,
+	  and the second will find the object created by the first thread.
+
+	  2) res_sorcery_memory has had its create callback updated so that it
+	  will not allow for objects with duplicate IDs to be created.
+
+	  Change-Id: I55b1460ff1eb0af0a3697b82d7c2bac9f6af5b97
+
+2016-02-15 12:52 +0000 [5c400a0fed]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_pubsub: Move where the subscription is stored to after initialized.
+
+	  A problem arose when testing the AMI subscription listing actions where it
+	  was possible for a subscription that had not been fully initialized to be
+	  listed. This was problematic as the underlying listing code would crash.
+
+	  This change makes it so the subscription tree is fully set up before it is
+	  added to the list of subscriptions. This ensures that when the listing actions
+	  get the subscription it is valid.
+
+	  ASTERISK-25738 #close
+
+	  Change-Id: Iace2b13641c31bbcc0d43a39f99aba1f340c0f48
+
+2016-02-09 17:34 +0000 [b37555cc94]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Refactor load_module/unload_module
+
+	  load_module was just too hairy with every step having to clean up all
+	  previous steps on failure.
+
+	  Some of the pjproject init calls have now been moved to a separate
+	  load_pjsip function and the unload_pjsip function was enhanced to clean
+	  up everything if an error happened at any stage of the load process.
+
+	  In the process, a bunch of missing pj_shutdowns, serializer_pool_shutdowns
+	  and ast_threadpool_shutdowns were also corrected.
+
+	  Change-Id: I5eec711b437c35b56605ed99537ebbb30463b302
+
+2016-02-09 22:42 +0000 [c4d9f46878]  Badalyan Vyacheslav <slavon.net@gmail.com>
+
+	* Resources/res_phoneprov: fix memory leak and heap-use-after-free
+
+	  * heap-use-after-free happens when we free "cfg"
+	  but then use "value" which refers to it
+
+	  * A memory leak occurs because in some cases
+	  it is not released "defaults"
+
+	  ASTERISK-25721 #close
+	  Reported by: Badalyan Vyacheslav
+	  Tested by: Badalyan Vyacheslav
+
+	  Change-Id: I3807d3f4726df6864430ec144cf6265d3f538469
+
+2016-02-11 11:21 +0000 [e5fd972d24]  Etienne Lessard (license #6394)
+
+	* func_iconv: Ensure output strings are properly terminated.
+
+	  ASTERISK-25272 #close
+	  Reported by: Etienne Lessard
+	  patches:
+	   AST-25272.patch submitted by Etienne Lessard (license #6394)
+
+	  Change-Id: Id75ad202300960a1e91afe15e319d992936ecc17
+
+2016-02-10 16:16 +0000 [168c18737f]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Handle pjsip_dlg_create_uas deprecation
+
+	  Pjproject has deprecated pjsip_dlg_create_uas in 2.5 and replaced it with
+	  pjsip_dlg_create_uas_and_inc_lock which, as the name implies, automatically
+	  increments the lock on the returned dialog.  To account for this, configure.ac
+	  now detects the presence of pjsip_dlg_create_uas_and_inc_lock and res_pjsip.c
+	  has an #ifdef HAVE_PJSIP_DLG_CREATE_UAS_AND_INC_LOCK to decide whether to use
+	  the original call or the new one.  If the new one was used, the ref count is
+	  decremented before returning.
+
+	  ASTERISK-25751 #close
+	  Reported-by Josh Colp
+
+	  Change-Id: I1be776b94761df03bd0693bc7795a75682615ca8
+
+2016-02-09 20:13 +0000 [fd668670b5]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* res_config_pgsql: Show error message in reload if not connected.
+
+	  Change-Id: I9290115a1aaadb589eb1d02eaeb502eec01b31fa
+
+2016-02-09 23:40 +0000 [a23d01e943]  Badalyan Vyacheslav <slavon.net@gmail.com>
+
+	* Build: Added testing compiler to support the system sanitizes
+
+	  In older versions of the compiler was not sanitizes.
+	  Compilers other than GCC can not support the Usan and TSAN
+	  or have other options for *FLAGS.
+
+	  ASTERISK-25767 #close
+	  Reported by: Badalyan Vyacheslav
+	  Tested by: Badalyan Vyacheslav
+
+	  Change-Id: Iefce6608221fa87884b82ae3cb5649b7b1804916
+
+2016-02-09 20:57 +0000 [c7186c7f0a]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
+
+	* Build: Fix menuselect USAN conflicts
+
+	  USAN can be used together with other sanitizers.
+
+	  Reported by: Badalyan Vyacheslav
+	  Tested by: Badalyan Vyacheslav
+
+	  Change-Id: I3bffa350d70965c3026651dba3a12414d0aaa45f
+
+2016-02-09 14:21 +0000 [68643f83cd]  Corey Farrell <git@cfware.com>
+
+	* Simplify and fix conditional in FD_SET.
+
+	  FD_SET contains a conditional statement to protect against buffer
+	  overruns.  The statement was overly complicated and prevented use
+	  of the last array element of ast_fdset.  We now just verify the fd
+	  is less than ast_FDMAX.
+
+	  Change-Id: I41895c0b497b052aef5bf49d75c817c48b326f40
+
+2016-02-09 07:11 +0000 [e40fddbeb5]  Joshua Colp <jcolp@digium.com>
+
+	* tests/test_sorcery_memory_cache_thrash: Improve termination process.
+
+	  When terminating the threads thrashing a sorcery memory cache each
+	  would be told to stop and then we would wait on them. During at
+	  least one thrashing test this was problematic due to the specific
+	  usage pattern in use. It would take some time for termination of the
+	  thread to occur.
+
+	  This would occur due to contention between the threads retrieving
+	  and the threads updating the cache. As the retrieving threads are
+	  given priority it may be some time before the updating threads
+	  are able to proceed.
+
+	  This change makes it so all threads are told to stop and then each
+	  are joined to ensure they stop. This way all the threads should
+	  stop at around the same time instead of waiting for one to stop,
+	  the next to stop, then the next, and so on. As a result of this
+	  the execution time for each thrash test is much closer to their
+	  expected value than previously seen as well.
+
+	  Change-Id: I04a53470b0ea4170b8819180b0bd7475f3642827
+2016-01-29 17:56 +0000 [bbf3ace682]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Fix infinite recursion when loading transports from realtime
+
+	  Attempting to load a transport from realtime was forcing asterisk into an
+	  infinite recursion loop.  The first thing transport_apply did was to do a
+	  sorcery retrieve by id for an existing transport of the same name. For files,
+	  this just returns the previous object from res_sorcery_config's internal
+	  container, if any.  For realtime, the res_sourcery_realtime driver looks in the
+	  database and finds the existing row but now it has to rehydrate it into a
+	  sorcery object which means calling... transport_apply.  And so it goes.
+
+	  The main issue with loading from realtime (apart from the loop) was that
+	  transport stores structures and pointers directly in the ast_sip_transport
+	  structure instead of the separate ast_transport_state structure.  This patch
+	  separates those items into the ast_sip_transport_state structure.  The pattern
+	  is roughly the same as res_pjsip_outbound_registration.
+
+	  Although all current usages of ast_sip_transport and ast_sip_transport_state
+	  were modified to use the new ast_sip_get_transport_state API, the original
+	  items are left in ast_sip_transport and kept updated to maintain ABI
+	  compatability for third-party modules.  They are marked as deprecated and
+	  noted that they're now in ast_sip_transport_state.
+
+	  ASTERISK-25606 #close
+	  Reported-by: Martin Moučka
+
+	  Change-Id: Ic7a836ea8e786e8def51fe3f8cce855ea54f5f19
+
+2016-02-07 13:00 +0000 [72bf53eea5]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* res_config_pgsql: Add message on cli failed command status
+
+	  In case failed of command "realtime show pgsql status" show a message the data
+	  of connection to more clear information in error.
+
+	  Change-Id: Ia8e9e2400466606e7118f52a46e05df0719b6a29
+
+2016-02-05 10:29 +0000 [b69729dde5]  gtjoseph <george.joseph@fairview5.com>
+
+	* chan_misdn: Fix a few issues causing compile errors
+
+	  Change-Id: I54b48c24d7ca88ed80496fdfd142d08772a7ab98
+
+2016-01-25 17:36 +0000 [1bc54aee80]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_confbridge: Only use b_profile options from the conference.
+
+	  A user cannot set new bridge options after the conference is created by
+	  the first user.  Attempting to do so is documented as undefined behavior.
+
+	  This patch ensures that the bridge profile options used are from the
+	  conference and not what a subsequent user may have tried to set.
+
+	  Change-Id: I1b6383eba654679e5739d5a8de98199cf074a266
+
+2016-02-04 16:17 +0000 [3b426a8b09]  Mark Michelson <mmichelson@digium.com>
+
+	* Check for OpenSSL defines before trying to use them.
+
+	  The SSL_OP_NO_TLSv1_1 and SSL_OP_NO_TLSv1_2 defines did not exist prior
+	  to OpenSSL version 1.0.1. A recent commit attempts to, by default, set
+	  these options, which can cause problems on systems with older OpenSSL
+	  installations.
+
+	  This commit adds a configure script check for those defines and will not
+	  attempt to make use of those if they do not exist. We will print a
+	  warning urging the user to upgrade their OpenSSL installation if those
+	  defines are not present.
+
+	  Change-Id: I6a2eb9a43fd0738b404d8f6f2cf4b5c22d9d752d
+2016-02-03 14:25 +0000 [9b13ab6a63]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip/alembic:  Add missing columns to system and registration
+
+	  ps_systems needed disable_tcp_switch
+	  ps_registrations needed line and endpoint
+
+	  ASTERISK-25737 #close
+
+	  Change-Id: Iaf9c2d69e62243d9fa53104c28c5339c47d4ac19
+
+2016-02-04 11:39 +0000 [82e2938fa8]  Mark Michelson <mmichelson@digium.com>
+
+	* res_stasis_device_state: Fix refcounting error.
+
+	  Device state subscription lifetimes were governed by when the
+	  subscription was established and unsubscribed from. However, it is
+	  possible that at the time of unsubscription, there could be device state
+	  events still in flight. When those device state events occur, the device
+	  state callback could attempt to dereference a freed pointer. Crash.
+
+	  This change ensures that the lifetime of the device state subscription
+	  does not end until the underlying stasis subscription has confirmed that
+	  its final message has been sent.
+
+	  Change-Id: I25a0f1472894c1a562252fb7129671478e25e9b2
+
+2016-01-27 10:44 +0000 [d83dba7099]  Sean Bright <sean.bright@gmail.com>
+
+	* res_rtp_asterisk: Allow ICE host candidates to be overriden
+
+	  During ICE negotiation the IPs of the local interfaces are sent to the remote
+	  peer as host candidates. In many cases Asterisk is behind a static one-to-one
+	  NAT, so these host addresses will be internal IP addresses.
+
+	  To help in hiding the topology of the internal network, this patch adds the
+	  ability to override the host candidates by matching them against a
+	  user-defined list of replacements.
+
+	  Change-Id: I1c9541af97b83a4c690c8150d19bf7202c8bff1f
+
+2016-02-03 12:05 +0000 [0de74fad55]  Joshua Colp <jcolp@digium.com>
+
+	* AST-2016-001 http: Provide greater control of TLS and set modern defaults.
+
+	  This change exposes the configuration of various aspects of the TLS
+	  support and sets the default to the modern standards.
+
+	  The TLS cipher is now set to the best values according to the
+	  Mozilla OpSec team, different TLS versions can now be disabled, and
+	  the cipher order can be forced to be that of the server instead of
+	  the client.
+
+	  ASTERISK-24972 #close
+
+	  Change-Id: I0a10f2883f7559af5e48dee0901251dbf30d45b8
+2015-12-07 12:46 +0000 [e67b445e8d]  Richard Mudgett <rmudgett@digium.com>
+
+	* AST-2016-003 udptl.c: Fix uninitialized values.
+
+	  Sending UDPTL packets to Asterisk with the right amount of missing
+	  sequence numbers and enough redundant 0-length IFP packets, can make
+	  Asterisk crash.
+
+	  ASTERISK-25603 #close
+	  Reported by: Walter Doekes
+
+	  ASTERISK-25742 #close
+	  Reported by: Torrey Searle
+
+	  Change-Id: I97df8375041be986f3f266ac1946a538023a5255
+2015-09-28 17:07 +0000 [a877e0d94b]  Richard Mudgett <rmudgett@digium.com>
+
+	* AST-2016-002 chan_sip.c: Fix retransmission timeout integer overflow.
+
+	  Setting the sip.conf timert1 value to a value higher than 1245 can cause
+	  an integer overflow and result in large retransmit timeout times.  These
+	  large timeout times hold system file descriptors hostage and can cause the
+	  system to run out of file descriptors.
+
+	  NOTE: The default sip.conf timert1 value is 500 which does not expose the
+	  vulnerability.
+
+	  * The overflow is now detected and the previous timeout time is
+	  calculated.
+
+	  ASTERISK-25397 #close
+	  Reported by: Alexander Traud
+
+	  Change-Id: Ia7231f2f415af1cbf90b923e001b9219cff46290
+2016-02-03 14:07 +0000 [dcbedf9ab1]  gtjoseph <george.joseph@fairview5.com>
+
+	* logging: Remove/fix some message annoyances
+
+	  test_dlinklists doesn't need to NOTICE everyone that every macro worked.
+
+	  res_phoneprov doesn't need to VERBOSE everyone that a phoneprov extension or
+	  provider was registered.
+
+	  res_odbc was missing a newline at the end of one message.
+
+	  Change-Id: I6c06361518ef3711821795e535acd439782a995e
+
+2016-02-02 10:52 +0000 [6522361871]  Alexei Gradinari License #5691
+
+	* res_sorcery_realtime: Fix regex regression.
+
+	  A regression was introduced where searching for realtime PJSIP objects
+	  by regex by starting the regex with a leading "^" would cause no items
+	  to be returned.
+
+	  This was due to a change which attempted to drop the requirement for a
+	  leading "^" to be present due to how some CLI commands formulate their
+	  regexes. However, the change, rather than simply eliminating the
+	  requirement, caused any regexes that did begin with "^" to end up not
+	  returning the expected results.
+
+	  This change fixes the problem by inspecting the regex and formulating
+	  the realtime query differently depending on if it begins with "^".
+
+	  ASTERISK-25702 #close
+	  Reported by Nic Colledge
+
+	  Patches:
+	      realtime_retrieve_regex.patch submitted by Alexei Gradinari License #5691
+
+	  Change-Id: I055df608a6e6a10732044fa737a9fe8dca602693
+
+2016-02-02 04:05 +0000 [2a6f18cd55]  Karsten Wemheuer <kwe-digium@iptam.com>
+
+	* res_xmpp: Does not connect in component mode
+
+	  The module res_xmpp does not accept usernames in the form used in component
+	  mode (XEP-0114). In component mode there is no @something in the name.
+	  In component mode the connection is now not dropped anymore.
+
+	  If the xmpp server sends out a "stream" tag before handshake is finished,
+	  the connection gets dropped in res_xmpp. Now this tag will be ignored and
+	  the connection will be established.
+
+	  After connecting there will be an exchange of presence states. This does
+	  not work as expected in component mode. The responsible function
+	  "xmpp_pak_presence" is left before the states get sent out. Sending
+	  presence states in component mode is now moved to the top of the function.
+
+	  ASTERISK-25735 #close
+
+	  Change-Id: I70e036f931c3124ebb2ad1e56f93ed35cfdd9d5c
+2016-02-01 13:04 +0000 [40da6434c1]  gtjoseph <george.joseph@fairview5.com>
+
+	* build_system:  Fix some warnings highlighted by clang
+
+	  Fix some warnings found with clang.
+
+	  Change-Id: I5195b6189b148c2ee3ed4a19d015a6d4ef3e77bd
+
+2016-01-31 20:13 +0000 [52b29f9b4c]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip/alembic: Fix definition of qualify_timeout
+
+	  A recent commit set qualify_timeout to Decimal which isn't supported.
+	  This path corrects it to Float.
+
+	  Change-Id: I038f5274ba8cb60f8518a5845ce448d49306aadf
+
+2016-01-29 07:39 +0000 [55a7367ad4]  Stefan Engström <stefanen@kth.se>
+
+	* chan_sip.c: AMI & CLI notify methods get different values of asterisk's own ip.
+
+	  When I ask asterisk to send a SIP NOTIFY message to a sip peer using either a)
+	  AMI action: SIPnotify or b) cli command: sip notify <cmd> <peer>, I expect
+	  asterisk to include the same value for its own ip in both cases a) and b),
+	  but it seems a) produces a contact header like Contact:
+	  <sip:asterisk@192.168.1.227:8060> whereas b) produces a contact header like
+	  <sip:asterisk@127.0.0.1:8060>. 0.0.0.0:8060 is my udpbindaddr in sip.conf
+
+	  My guess is that manager_sipnotify should call
+	  ast_sip_ouraddrfor(&p->sa, &p->ourip, p) the same way sip_cli_notify does,
+	  because after applying this patch, both cases a) and b) produce
+	  the contact header that I expect: <sip:asterisk@192.168.1.227:8060>
+
+	  Reported by: Stefan Engström
+	  Tested by: Stefan Engström
+
+	  Change-Id: I86af5e209db64aab82c25417de6c768fb645f476
+2016-01-28 12:44 +0000 [d2397f028f]  Richard Mudgett <rmudgett@digium.com>
+
+	* config_options.c: Fix warning message wording.
+
+	  Change-Id: I915ea437936320393afde0e7552cf0a980a6b2e4
+
+2016-01-25 17:34 +0000 [af6b15976d]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_confbridge.c: Replace inlined code with existing function.
+
+	  Change-Id: Ida5594e9f8d7c1fc18eeb733a11f8fb96326da51
+
+2016-01-25 16:05 +0000 [7932336a3d]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_confbridge: Add ability to get the muted conference state.
+
+	  * Added CONFBRIDGE_INFO(muted,) for querying the muted conference state.
+
+	  * Added Muted header to AMI ConfbridgeListRooms action response list
+	  events to indicate the muted conference state.
+
+	  * Added Muted column to CLI "confbridge list" output to indicate the muted
+	  conference state and made the locked column a yes/no value instead of a
+	  locked/unlocked value.
+
+	  ASTERISK-20987
+	  Reported by: hristo
+
+	  Change-Id: I4076bd8ea1c23a3afd4f5833e9291b49a0c448b1
+
+2016-01-26 17:59 +0000 [894045e7cf]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_confbridge.c: Update CONFBRIDGE and CONFBRIDGE_INFO documentation.
+
+	  Change-Id: Ic1f9e22ba1f2ff3b3f5cb017c5ddcd9bd48eccc7
+
+2016-01-25 15:48 +0000 [12c93e8f81]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_confbridge: Make non-admin users join a muted conference muted.
+
+	  ASTERISK-20987 #close
+	  Reported by: hristo
+
+	  Change-Id: Ic61a2b524ab3a4cfadf227fc6b3506527bc03f38
+
+2016-01-27 13:08 +0000 [f19bf7a321]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Add res_pjproject dependency to samples
+
+	  Since res_pjsip now depends on res_pjproject, this has been added to
+	  basic-pbx modules.conf.
+
+	  Change-Id: I42826597d5e10f08e518208860c44c96e52f1b2d
+2016-01-27 10:29 +0000 [c53903d447]  gtjoseph <george.joseph@fairview5.com>
+
+	* build_system: Prevent goals needing makeopts from running when it's missing
+
+	  The Makefile only optionally includes makeopts so when goals like uninstall that
+	  dont depend on anything else are run after a distclean, rules like
+	  'rm -f "$(DESTDIR)$(ASTMODDIR)/"*' get run as 'rm -f ""/*' which attempts
+	  to remove everything in the root directory.
+
+	  Although there's a rule defined for makeopts which prints a message and does
+	  an 'exit 1', since '-include makepopts' was specified (with the -), the exit
+	  was ignored letting the rest of the rules run.
+
+	  This patch makes makeopts required unless the goal has the string 'clean' in it.
+
+	  ASTERISK-25730 #close
+	  Reported-by: George Joseph
+
+	  Change-Id: I1bce59a7ea4f48e7a468e22b2abbb13c63417ac7
+
+2016-01-25 09:35 +0000 [1dfd104a27]  Joshua Colp <jcolp@digium.com>
+
+	* config: Allow options to register when documentation is unavailable.
+
+	  The config options framework is strict in that configuration options must
+	  be documented unless XML documentation support is not available. In
+	  practice this is useful as it ensures documentation exists however in
+	  off-nominal cases this can cause strange problems.
+
+	  If it is expected that a config option has a non-zero or non-empty
+	  default value but the config option documentation is unavailable
+	  this reasonable expectation will not be met. This can cause obscure
+	  crashes and weirdness depending on how the code handles it.
+
+	  This change tweaks the behavior to ensure that the config option
+	  is still allowed to register, apply default values, and be set when
+	  devmode is not enabled. If devmode is enabled then the option can
+	  NOT be set.
+
+	  This also does not remove the initial documentation error message that
+	  is output on load when registering the configuration option.
+
+	  ASTERISK-25725 #close
+
+	  Change-Id: Iec42fca6b35f31326c33fcdc25473f6fd7bc8af8
+
+2016-01-25 10:23 +0000 [a706ad44e6]  Mark Michelson <mmichelson@digium.com>
+
+	* Stasis: Use custom structure when setting variables.
+
+	  A recent change to queue channel variable setting to the Stasis control
+	  queue caused a regression. When setting channel variables, it is
+	  possible to give a NULL channel variable value in order to unset the
+	  variable (i.e. remove it from the channel variable list). The change
+	  introduced a call to ast_variable_new(), which is not tolerant of NULL
+	  channel variable values.
+
+	  This new change switches from using ast_variable to using a custom
+	  channel variable struct that is lighter weight and NULL value-tolerant.
+
+	  Change-Id: I784d7beaaa3c036ea936d103e7caf0bb1562162d
+
+2016-01-25 16:56 +0000 [289daca9e8]  Rusty Newton <rnewton@digium.com>
+
+	* sounds/Makefile: Incremented core and extra sounds versions to 1.5
+
+	  Core and extra sounds 1.5 was recently released! The tarballs contain
+	  change descriptions however I figure more people will see this one so
+	  I'll try to be a bit detailed. Approximately 60 sounds were moved from Extra
+	  to Core for en, en_GB, fr and added for languages that didn't already
+	  have Extra sound sets (it,ja,ru).
+
+	  In addition all of the English and Russian sounds have been completely
+	  re-recorded.
+
+	  Sounds moved and added:
+	  activated,added,all-circuits-busy-now,astcc-followed-by-pound
+	  at-tone-time-exactly,call-forwarding,call-fwd-no-ans,call-fwd-on-busy
+	  ,call-fwd-unconditional,calling,call-waiting,cancelled,
+	  cannot-complete-as-dialed,check-number-dial-again,conf-full,de-activated
+	  ,disabled,do-not-disturb,enabled,enter-num-blacklist,entr-num-rmv-blklist
+	  ,extension,feature-not-avail-line,for,from-unknown-caller,goodbye,hello
+	  ,if-correct-press,im-sorry,info-about-last-call,is,is-in-use,is-set-to
+	  ,location,number,number-not-answering,num-was-successfully,one-moment-please
+	  ,please-try-again,pls-hold-while-try,pls-try-call-later,pm-invalid-option
+	  ,privacy-to-blacklist-last-caller,removed,simul-call-limit-reached
+	  ,something-terribly-wrong,sorry,sorry-youre-having-problems,speed-dial
+	  ,speed-dial-empty,telephone-number,time,to-call-this-number,to-extension
+	  ,to-listen-to-it,to-rerecord-it,unidentified-no-callback,with,you-entered
+	  ,your
+
+	  There were also a few random fixes here and there to file names for a few
+	  of the languages.
+
+	  ASTERISK-25068 #close
+
+	  Change-Id: I2b594344ec585d7dfd922b40c1af43b1508828b3
+2016-01-25 16:51 +0000 [b073244c51]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Prevent crash from AMI command on freed subscription.
+
+	  A test recently uncovered that running an ill-timed AMI command to show
+	  inbound subscriptions could cause a crash since Asterisk will try to
+	  operate on a freed subscription.
+
+	  The fix for this is to remove the subscription tree from the list of
+	  subscriptions at the time that we are sending our final NOTIFY request
+	  out. This way, as the subscription is in the process of dying, it is
+	  inaccessible from AMI.
+
+	  Change-Id: Ic0239003d8d73e04c47c12dd2a7e23867e5b5b23
+
+2016-01-25 11:03 +0000 [830f8933c2]  Corey Farrell <git@cfware.com>
+
+	* chan_sip: Fix buffer overrun in sip_sipredirect.
+
+	  sip_sipredirect uses sscanf to copy up to 256 characters to a stacked buffer
+	  of 256 characters.  This patch reduces the copy to 255 characters to leave
+	  room for the string null terminator.
+
+	  ASTERISK-25722 #close
+
+	  Change-Id: Id6c3a629a609e94153287512c59aa1923e8a03ab
+
+2016-01-23 16:45 +0000 [f299dc0d76]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: Add  Lastpause field of queue member
+
+	  Add time when started a the last pause for a queue member for
+	  QueueMemberStatus ami event.
+
+	  Also show accumulate time in seconds when started a pause for a queue
+	  member to CLI command 'queue show'.
+
+	  ASTERISK-16394 #close
+
+	  Change-Id: I4b12aa3b2efa8d02939db3e13712510b4879865c
+
+2016-01-23 12:34 +0000 [8c664da0ff]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: fix some tab format
+
+	  Change-Id: I2734392b131f1fb0949515d538f83f30fbc15d8c
+
+2016-01-23 11:41 +0000 [2fb45c7801]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* cdr_pgsql.cl: REFACTOR Macro LENGTHEN_BUF
+
+	  Remove repeated code on macro of assigned buffer to SQL vars.
+
+	  Add table and connection name to log error message when is not possible
+	  allocate memory.
+
+	  Change-Id: I1fbf37d286a032d38fdda72a9f736356956c9ffe
+
+2016-01-22 15:08 +0000 [959f7436cc]  Mark Michelson <mmichelson@digium.com>
+
+	* Stasis: Fix potential memory leak of control data.
+
+	  When queuing tasks onto the Stasis control queue, you can pass an
+	  arbitrary data pointer and a function to free that data. All ARI
+	  commands that use the Stasis control queue made the assumption that the
+	  destructor function would be called in all paths, whether the task was
+	  queued successfully or not. However, this was not correct. If a task was
+	  queued onto a control structure that was already completed, the
+	  allocated data would not be freed properly.
+
+	  This patch corrects this by making sure that all return paths call the
+	  data destructor.
+
+	  Change-Id: Ibf06522094f8e5c4cce652537dc5d7222b1c4fcb
+
+2016-01-21 10:58 +0000 [a45eacebf3]  Mark Michelson <mmichelson@digium.com>
+
+	* Stasis: Use control queue to prevent crash.
+
+	  A crash occurred when attempting to set a channel variable on a channel
+	  that had already been hung up. This is because there is a small window
+	  between when a control is grabbed and when the channel variable is set
+	  that the channel can be hung up.
+
+	  The fix here is to queue the setting of the channel variable onto the
+	  control queue. This way, the manipulation of the channel happens in a
+	  thread where it is safe to be done.
+
+	  In this change, I also noticed that the setting of bridge roles on
+	  channels was being done outside of the control queue, so I also changed
+	  those operations to be done in the control queue.
+
+	  ASTERISK-25709 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I2a0a4d51bce6fba6f1d9954e40935e42f366ea78
+
+2016-01-22 11:48 +0000 [7866806fc3]  Richard Mudgett <rmudgett@digium.com>
+
+	* logger.c: Fix buffer overrun found by address sanitizer.
+
+	  The null terminator of the tail struct member was not being allocated
+	  when no logger.conf config file is installed.
+
+	  ASTERISK-25714 #close
+	  Reported by: Badalian Vyacheslav
+
+	  Change-Id: I45770fdd08af39506a3bc33ba279c4f16e047a30
+
+2015-12-23 15:07 +0000 [9714da7aa4]  Mark Michelson <mmichelson@digium.com>
+
+	* res_odbc: Remove connection management
+
+	  Asterisk by default will create a single database connection and share
+	  it among all threads that attempt to access the database. In previous
+	  versions of Asterisk, this was tolerable, because the most used channel
+	  driver, chan_sip, mostly accessed the database from a single thread.
+	  With PJSIP, however, many threads may be attempting to perform database
+	  operations, and there is the potential for many more database accesses,
+	  meaning the concurrency is a horrible bottleneck if only one connection
+	  is shared.
+
+	  Asterisk has a connection pooling facility built into it, but the
+	  implementation has flaws. For one, there is a strict limit on the number
+	  of simultaneous connections that could be made to the database. Anything
+	  beyond the maximum would result in a failed operation. Attempting to
+	  predict what the maximum should be is nearly impossible even for someone
+	  intimately familiar with Asterisk's threading model. In addition, use of
+	  transactions in the dialplan can cause some severe bugs if connection
+	  pooling is enabled.
+
+	  This commit seeks to fix the concurrency problem by removing all
+	  connection management code from Asterisk and leaving that to the
+	  underlying unixODBC code instead. Now, Asterisk does not share a single
+	  connection, nor does it try to maintain a connection pool. Instead, all
+	  Asterisk ever does is request a connection from unixODBC and allow
+	  unixODBC to either allocate those connections or retrieve them from a
+	  pool.
+
+	  Doing this has a bit of a ripple effect. For one, since connections are
+	  not long-lived objects, several of the safeguards that previously
+	  existed have been removed. We don't have to worry about trying to use a
+	  connection that has gone stale. In every case, when we request a
+	  connection, it has just been made and we don't need to perform any
+	  sanity checks to be sure it's still active.
+
+	  Another major player affected by this change is transactions.
+	  Transactions and their respective connections were so tightly coupled
+	  that it was almost pornographic. This code change moves
+	  transaction-related code to its own file separate from the core ODBC
+	  functionality. This way, the core of ODBC does not even have to know
+	  that transactions exist.
+
+	  In making this large change, I had to look at a lot of code and
+	  understand it. When making this change, I discovered several places
+	  where the behavior is definitely not ideal, but it seemed outside the
+	  scope of this change to be fixing it. Instead, any place where I saw
+	  some sort of room for improvement has had a XXX comment added explaining
+	  what could be altered to improve it.
+
+	  Change-Id: I37a84def5ea4ddf93868ce8105f39de078297fbf
+
+2016-01-22 11:18 +0000 [d3969d09ae]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue.c: remove include for core_unreal.h not used in code.
+
+	  Change-Id: Idc2ae8a6bd869a66544916906744a5678622262d
+
+2016-01-21 16:40 +0000 [5dde111719]  Corey Farrell <git@cfware.com>
+
+	* Build System: Add support for checking alembic branches.
+
+	  * Add 'check-alembic' target to root Makefile.
+	  * Create build_tools/make_check_alembic to do the actual checks.
+
+	  ASTERISK-25685
+
+	  Change-Id: Ibb3cae7d1202ac23dc70b0f3b5801571ad46b004
+
+2016-01-19 18:20 +0000 [04078f43b5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res/res_pjsip/presence_xml.c: Add missing 2nd call presence state case.
+
+	  ASTERISK-25712 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I70634df24f8c6c3a2c66c45af61d021e4999253f
+
+2016-01-13 16:49 +0000 [5615db3714]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Add CLI "pjsip dump endpt [details]"
+
+	  Dump the res_pjsip endpt internals.
+
+	  In non-developer mode we will not document or make easily accessible the
+	  "details" option even though it is still available.  The user has to know
+	  it exists to use it.  Presumably they would also be aware of the potential
+	  crash warning below.
+
+	  Warning: PJPROJECT documents that the function used by this CLI command
+	  may cause a crash when asking for details because it tries to access all
+	  active memory pools.
+
+	  Change-Id: If2d98a3641c9873364d1daaad971376311aef3cb
+
+2016-01-18 03:49 +0000 [b259ac95ac]  Diederik de Groot <ddegroot@talon.nl>
+
+	* main/asterisk.c: ast_el_read_char
+
+	  Make sure buf[res] is not accessed at res=-1 (buffer underrun).
+	  Address Sanitizer will complain about this quite loudly.
+
+	  ASTERISK-24801 #close
+
+	  Change-Id: Ifcd7f691310815a31756b76067c56fba299d3ae9
+
+2016-01-18 19:27 +0000 [dd5c063934]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjproject:  Add module providing pjproject logging and utils
+
+	  res_pjsip_log_forwarder has been renamed to res_pjproject
+	  and enhanced as follows:
+
+	  As a follow-on to the recent 'Add CLI "pjsip show buildopts"' patch,
+	  a new ast_pjproject_get_buildopt function has been added.  It
+	  allows the caller to get the value of one of the buildopts.
+
+	  The initial use case is retrieving the runtime value of
+	  PJ_MAX_HOSTNAME to insure we don't send a hostname greater
+	  than pjproject can handle.  Since it can differ between
+	  the version of pjproject that Asterisk was compiled against
+	  and the version of pjproject that Asterisk is running against,
+	  we can't use the PJ_MAX_HOSTNAME macro directly in Asterisk
+	  source code.
+
+	  Change-Id: Iab6e82fec3d7cf00c1cf6185c42be3e7569dee1e
+
+2016-01-18 17:16 +0000 [3b9cba4294]  Matt Jordan <mjordan@digium.com>
+
+	* funcs/func_cdr: Correctly report high precision values for duration and billsec
+
+	  When CDRs were refactored, func_cdr's ability to report high precision values
+	  for duration and billsec (the 'f' option) was broken. This was due to func_cdr
+	  incorrectly interpreting the duration/billsec values provided by the CDR engine
+	  in milliseconds, as opposed to seconds. Since the CDR engine only provides
+	  duration and billsec in seconds, and does not expose either attribute with
+	  sufficient precision to merely pass back the underlying value, this patch fixes
+	  the bug by re-calculating duration and billsec with microsecond precision based
+	  on the start/answer/end times on the CDR.
+
+	  ASTERISK-25179 #close
+
+	  Change-Id: I8bc63822b496537a5bf80baf6102c06206bee841
+
+2016-01-20 07:52 +0000 [479cc99acd]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* README: Update year in copyright
+
+	  Change-Id: I56240f537fb3205672cdb2a74f0591ae7bb73dbc
+
+2016-01-19 17:15 +0000 [9fa76ba215]  Joshua Colp <jcolp@digium.com>
+
+	* test_threadpool: Wait for each task to complete and fix memory leak.
+
+	  This change makes the thread_timeout_thrash unit test wait for
+	  each task to complete. This fixes the problem where the test would
+	  prematurely end when all threads were gone and a new one had to be
+	  started to handle the last task. It also increases the thrasing as
+	  it is now more likely for each task to encounter the above scenario.
+
+	  This also fixes a memory leak where the data for each task was not
+	  being freed.
+
+	  ASTERISK-25611 #close
+
+	  Change-Id: I5017d621a4dc911f509074c16229b86bff2fb3c6
+
+2016-01-18 19:44 +0000 [c9f7269b2e]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Increase CLI "core ping taskprocessor" timeout.
+
+	  Change-Id: I4892d6acbb580d6c207d006341eaf5e0f8f2a029
+
+2016-01-18 19:43 +0000 [6e2a867716]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Fix some taskprocessor unrefs.
+
+	  You have to call ast_taskprocessor_unref() outside of the taskprocessor
+	  implementation code.  Taskprocessor use since v12 has become more
+	  transient than just the singleton uses in earlier versions.
+
+	  Change-Id: If7675299924c0cc65f2a43a85254e6f06f2d61bb
+
+2016-01-19 14:16 +0000 [a4dcbdf50f]  Richard Mudgett <rmudgett@digium.com>
+
+	* Fix alembic branches on master.
+
+	  Change-Id: I64ed21fec50eb833641ca49d92184f6aaabd86e8
+
+2016-01-05 17:12 +0000 [35a3e8cc7f]  Corey Farrell <git@cfware.com>
+
+	* Refactor init_logger_chain locking.
+
+	  This removes logchannels locking from init_logger_chain, puts the
+	  responsibility on the caller.  Adds locking around the one call that was
+	  missing it.
+
+	  ASTERISK-24833
+
+	  Change-Id: I6cc42117338bf9575650a67bcb78ab1a33d7bad8
+
+2016-01-18 22:10 +0000 [378fed4900]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: Fix preserved reason of pause when Asterisk is restared
+
+	  When the Asterisk is restared is not preseved reason paused of members.
+	  This patch fixed this cases, retain data on astdb and set when Asterisk
+	  is started.
+
+	  ASTERISK-25732 #close
+
+	  Report by: Rodrigo Ramírez Norambuena
+
+	  Change-Id: Id3fb744c579e006d27cda4a02334ac0e4bed9eb5
+
+2016-01-18 19:01 +0000 [130aa1427e]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip_loging_refactor: Rename res_pjsip_log_forwarder to res_pjproject
+
+	  Change-Id: I5387821f29e5caa0cba0b7d62b0fc0d341e7e20b
+
+2016-01-16 13:18 +0000 [eaf2b5052e]  Daniel Journo <dan@keshercommunications.com>
+
+	* Update version number in features.conf.sample
+
+	  Update the version number in the comments from Asterisk 12 to Asterisk 12+
+
+	  Change-Id: Ie692ac8cda3c993c3bf10f27f51a1cca3317ec7b
+
+2016-01-13 15:58 +0000 [c60d6c0162]  Daniel Journo <dan@keshercommunications.com>
+
+	* pjsip/alembic:  Fix qualify_timeout column definition
+
+	  Corrects the qualify_timeout column type from Integer to Decimal
+
+	  ASTERISK-25686 #close
+	  Reported-by: Marcelo Terres
+
+	  Change-Id: I757d0e3c011ee9be6cd5abd48bc92441a405d3c8
+
+2016-01-15 19:52 +0000 [480ccfcc97]  Corey Farrell <git@cfware.com>
+
+	* main/config: Clean config maps on shutdown.
+
+	  ASTERISK-25700 #close
+
+	  Change-Id: I096da84f9c62c6095f68bcf98eac4b7c7868e808
+
+2016-01-14 14:42 +0000 [a5b38b604c]  Kevin Harwell <kharwell@digium.com>
+
+	* bridge_basic: don't cache xferfailsound during an attended transfer
+
+	  The xferfailsound was read from the channel at the beginning of the transfer,
+	  and that value is "cached" for the duration of the transfer. Therefore, changing
+	  the xferfailsound on the channel using the FEATURE() dialplan function does
+	  nothing once the transfer is under way.
+
+	  This makes it so the transfer code instead gets the xferfailsound configuration
+	  options from the channel when it is actually going to be used.
+
+	  This patch also fixes a potential memory leak of the props object as well as
+	  making sure the condition variable gets initialized before being destroyed.
+
+	  ASTERISK-25696 #close
+
+	  Change-Id: Ic726b0f54ef588bd9c9c67f4b0e4d787934f85e4
+
+2015-07-10 10:37 +0000 [d36c4d0b01]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Simplify ast_taskprocessor_get() return code.
+
+	  Change-Id: Id5bd18ef1f60ef8be453e677e98478298358a9d1
+
+2016-01-13 18:20 +0000 [0a878020dc]  Richard Mudgett <rmudgett@digium.com>
+
+	* astmm.c: Add more stats to CLI "memory show" commands.
+
+	  * Add freed regions totals to allocations and summary.
+
+	  * Add totals for all allocations and not just the selected allocations.
+
+	  Change-Id: I61d5a5112617b0733097f2545a3006a344b4032a
+
+2016-01-14 16:00 +0000 [84b30c5e18]  Kevin Harwell <kharwell@digium.com>
+
+	* bridge_basic: don't play an attended transfer fail sound after target hangs up
+
+	  If the attended transfer destination answers (picks call up or goes to
+	  voicemail) and then hangs up on the transferer then transferer hears the
+	  fail sound.
+
+	  This patch makes it so the fail sound is not played when the transfer
+	  destination/target hangs up after answering.
+
+	  ASTERISK-25697 #close
+
+	  Change-Id: I97f142fe4fc2805d1a24b7c16143069dc03d9ded
+
+2016-01-14 14:36 +0000 [c7caee6c4b]  Corey Farrell <git@cfware.com>
+
+	* Remove *.gcna / *.gcno files from added module sources.
+
+	  Asterisk uses a Makefile macro to associate additional sources with a
+	  module.  This macro is responsible for creating clean targets but
+	  previously left behind *.gcna and *.gcno files.
+
+	  ASTERISK-25683 #close
+	  Reported by yaron nahum
+
+	  Change-Id: Idc0823fe80a25c42cefae901fde875e9fc38d8ea
+
+2016-01-14 09:26 +0000 [68cad96ffd]  Rusty Newton <rnewton@digium.com>
+
+	* func_channel: Add help text for undocumented CHANNEL function arguments
+
+	  Adding help text documentation for:
+	  * hangupsource
+	  * appname
+	  * appdata
+	  * exten
+	  * context
+	  * channame
+	  * uniqueid
+	  * linkedid
+
+	  ASTERISK-24097 #close
+	  Reported by: Steven T. Wheeler
+	  Tested by: Rusty Newton
+
+	  Change-Id: Ib94b00568b0433987df87d5b67ea529b5905754d
+
+2016-01-10 16:22 +0000 [8182146e85]  Daniel Journo <dan@keshercommunications.com>
+
+	* pjsip:  Add option global/regcontext
+
+	  Added new global option (regcontext) to pjsip. When set, Asterisk will
+	  dynamically create and destroy a NoOp priority 1 extension
+	  for a given endpoint who registers or unregisters with us.
+
+	  ASTERISK-25670 #close
+	  Reported-by: Daniel Journo
+
+	  Change-Id: Ib1530c5b45340625805c057f8ff1fb240a43ea62
+
+2016-01-12 11:14 +0000 [022423b98b]  Joshua Colp <jcolp@digium.com>
+
+	* app: Queue hangup if channel is hung up during sub or macro execution.
+
+	  This issue was exposed when executing a connected line subroutine.
+	  When connected or redirected subroutines or macros are executed it is
+	  expected that the underlying applications and logic invoked are fast
+	  and do not consume frames. In practice this constraint is not enforced
+	  and if not adhered to will cause channels to continue when they shouldn't.
+	  This is because each caller of the connected or redirected logic does not
+	  check whether the channel has been hung up on return. As a result the
+	  the hung up channel continues.
+
+	  This change makes it so when the API to execute a subroutine or
+	  macro is invoked the channel is checked to determine if it has hung up.
+	  If it has then a hangup is queued again so the caller will see it
+	  and stop.
+
+	  ASTERISK-25690 #close
+
+	  Change-Id: I1f9a8ceb1487df0389f0d346ce0f6dcbcaf476ea
+
+2016-01-13 07:20 +0000 [79a7321a47]  Sean Bright <sean.bright@gmail.com>
+
+	* res_musiconhold: Prevent multiple simultaneous reloads.
+
+	  There are two ways in which the reload() function in res_musiconhold can be
+	  called from the CLI:
+
+	    * module reload res_musiconhold.so
+	    * moh reload
+
+	  In the former case, the module loader holds a lock that prevents multiple
+	  concurrent calls, but in the latter there is no such protection.
+
+	  This patch changes the 'moh reload' CLI command to invoke the module loader
+	  directly, rather than call reload() explicitly.
+
+	  ASTERISK-25687 #close
+
+	  Change-Id: I408968b4c8932864411b7f9ad88cfdc7b9ba711c
+2016-01-12 14:25 +0000 [1fffe71f77]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_log_forwarder.c: Add CLI "pjsip show buildopts".
+
+	  PJPROJECT has a function available to dump the compile time
+	  options used when building the library.
+
+	  * Add CLI "pjsip show buildopts" command.
+
+	  * Update contrib/scripts/autosupport to get pjproject information.
+
+	  Change-Id: Id93a6a916d765b2a2e5a1aeb54caaf83206be748
+
+2016-01-12 10:36 +0000 [01c5e2a07e]  Mark Michelson <mmichelson@digium.com>
+
+	* res_sorcery_realtime: Remove leading ^ requirement.
+
+	  res_sorcery_realtime's search-by-regex callback performed a check to
+	  ensure that the passed-in regex began with a caret (^). If it did not,
+	  then no results would be returned.
+
+	  This callback only started to become used when "like" support was added
+	  to PJSIP CLI commands. The CLI command for listing objects would pass an
+	  empty regex ("") to the sorcery backend if no "like" statement was
+	  present. For most sorcery backends, this resulted in returning all
+	  objects. However, for realtime, this resulted in returning no objects.
+
+	  This commit seeks to fix the regression by removing the requirement from
+	  res_sorcery_realtime for the passed-in-regex to begin with a caret.
+
+	  ASTERISK-25689 #close
+	  Reported by Marcelo Terres
+
+	  Change-Id: I22b4dc5d7f3f11bb29ac2e42ef94682e9bab3b20
+
+2016-01-07 11:57 +0000 [a41aab477a]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip_sdp_rtp:  Add option endpoint/bind_rtp_to_media_address
+
+	  On a system with multiple ip addresses in the same subnet, if a
+	  transport is bound to a specific ip address and endpoint/media_address
+	   is set, the SIP/SDP will have the correct address in all fields but
+	  the rtp stream MAY still originate from one of the other ip addresses,
+	  most probably the "primary" ip address.  This happens because
+	   res_pjsip_sdp_rtp/create_rtp always calls ast_instance_new with
+	  the "all" ip address (0.0.0.0 or ::).
+
+	  The new option causes res_pjsip_sdp_rtp/create_rtp to call
+	  ast_rtp_instance_new with the endpoint's media_address (if specified)
+	  instead of the "all" address.  This causes the packets to originate from
+	  the specified address.
+
+	  ASTERISK-25632
+	  ASTERISK-25637
+	  Reported-by: Olivier Krief
+	  Reported-by: Dan Journo
+
+	  Change-Id: I3dfaa079e54ba7fb7c4fd1f5f7bd9509bbf8bd88
+
+2016-01-08 16:59 +0000 [7760029f19]  Kevin Harwell <kharwell@digium.com>
+
+	* pbx: Deadlock between contexts container and context_merge locks
+
+	  Recent changes (ASTERISK-25394 commit 2bd27d12223fe33b58c453965ed5c6ed3af7c4f5)
+	  introduced the possibility of a deadlock. Due to the mentioned modifications
+	  ast_change_hints now needs to keep both merge/delete and state callbacks from
+	  occurring while it executes. Unfortunately, sometimes ast_change_hints can be
+	  called with the contexts container locked. When this happens it's possible for
+	  another thread to grab the context_merge_lock before the thread calling into
+	  ast_change_hints does and then try to obtain the contexts container lock. This
+	  of course causes a deadlock between the two threads. The thread calling into
+	  ast_change_hints waits for the other thread to release context_merge_lock and
+	  the other thread is waiting on that one to release the contexts container lock.
+
+	  Unfortunately, there is not a great way to fix this problem. When hints change,
+	  the subsequent state callbacks cannot run at the same time as a merge/delete,
+	  nor when the usual state callbacks do. This patch alleviates the problem by
+	  having those particular callbacks (the ones run after a hint change) occur in a
+	  serialized task. By moving the context_merge_lock to a task it can now safely be
+	  attempted or held without a deadlock occurring.
+
+	  ASTERISK-25640 #close
+	  Reported by: Krzysztof Trempala
+
+	  Change-Id: If2210ea241afd1585dc2594c16faff84579bf302
+
+2016-01-10 17:08 +0000 [e9c2c1dc67]  Corey Farrell <git@cfware.com>
+
+	* devicestate: Cleanup engine thread during graceful shutdown.
+
+	  ASTERISK-25681 #close
+
+	  Change-Id: I64337c70f0ebd8c77f70792042684607c950c8f1
+
+2016-01-10 13:51 +0000 [90c0dcaee4]  Corey Farrell <git@cfware.com>
+
+	* manager: Cleanup manager_channelvars during shutdown.
+
+	  ASTERISK-25680 #close
+
+	  Change-Id: I3251d781cbc3f48a6a7e1b969ac4983f552b2446
+
+2016-01-10 13:27 +0000 [a868a381f0]  Corey Farrell <git@cfware.com>
+
+	* res_calendar: Cleanup scheduler context at unload.
+
+	  ASTERISK-25679 #close
+
+	  Change-Id: I839159bf6882cccc1b23494c7aa2bc2a2624613f
+
+2016-01-08 11:49 +0000 [a1c43022d2]  Joshua Colp <jcolp@digium.com>
+
+	* res_rtp_asterisk: Revert DTLS negotiation changes.
+
+	  Due to locking issues within pjnath these changes are being
+	  reverted until pjnath can be changed.
+
+	  ASTERISK-25645
+
+	  Revert "res_rtp_asterisk.c: Fix DTLS negotiation delays."
+
+	  This reverts commit 24ae124e4f7310cfa64c187b944b2ffc060da28d.
+
+	  Change-Id: I2986cfb2c43dc14455c1bcaf92c3804f9da49705
+
+	  Revert "res_rtp_asterisk: Resolve further timing issues with DTLS negotiation"
+
+	  This reverts commit 965a0eee46d24321f74c244e23c5a5f45e67e12b.
+
+	  Change-Id: Ie68fafde27dad4b03cb7a1e27ce2a8502c3f7bbe
+
+2016-01-09 17:57 +0000 [220ba979cf]  gtjoseph <george.joseph@fairview5.com>
+
+	* Revert "pjsip_location: Delete contact_status object when contact is deleted"
+
+	  This reverts commit 0a9941de9d24093b5ff44096d1d7406f29d11e45.
+
+	  Matt,
+
+	  This patch causes another problem and should not have been needed.
+	  Before this patch, persistent_endpoint_contact_deleted_observer WAS
+	  deleting the contact_status when ast_sip_location_delete_contact was
+	  called.  By deleting it yourself in ast_sip_location_delete_contact
+	  it was gone before the observer could run and the observer therefore
+	  was throwing an error and not sending stasis/AMI/statsd messages.
+
+	  So, I don't think this was the cause of your original issue.  I also
+	  had verified the contact AMI and statsd lifecycle and it was working.
+	  I'll double check now though.
+
+	  ASTERISK-25675
+	  Reported-by: Daniel Journo
+
+	  Change-Id: Ib586a6b7f90acb641b0c410f659743ab90e84f1a
+
+2016-01-09 18:04 +0000 [26e0e113dc]  Corey Farrell <git@cfware.com>
+
+	* pbx_dundi: Run cleanup on failed load.
+
+	  During failed startup of pbx_dundi no cleanup was performed.  Add a call
+	  to unload_module before returning AST_MODULE_LOAD_DECLINE.
+
+	  ASTERISK-25677 #close
+
+	  Change-Id: I8ffa226fda4365ee7068ac1f464473f1a4ebbb29
+
+2016-01-09 13:28 +0000 [dc2c000fd5]  Corey Farrell <git@cfware.com>
+
+	* res_crypto: Perform cleanup at shutdown.
+
+	  This change causes res_crypto to unregister CLI at shutdown while still
+	  preventing the module from being unloaded.
+
+	  ASTERISK-25673 #close
+
+	  Change-Id: Ie5d57338dc2752abfc0dd05d0eec86413f2304fc
+
+2016-01-06 19:10 +0000 [0bca2a5c26]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Create human friendly serializer names.
+
+	  PJSIP name formats:
+	  pjsip/aor/<aor>-<seq> -- registrar thread pool serializer
+	  pjsip/default-<seq> -- default thread pool serializer
+	  pjsip/messaging -- messaging thread pool serializer
+	  pjsip/outreg/<registration>-<seq> -- outbound registration thread pool
+	  serializer
+	  pjsip/pubsub/<endpoint>-<seq> -- pubsub thread pool serializer
+	  pjsip/refer/<endpoint>-<seq> -- REFER thread pool serializer
+	  pjsip/session/<endpoint>-<seq> -- session thread pool serializer
+	  pjsip/websocket-<seq> -- websocket thread pool serializer
+
+	  Change-Id: Iff9df8da3ddae1132cb2ef65f64df0c465c5e084
+
+2016-01-06 19:09 +0000 [f0f5fbbc01]  Richard Mudgett <rmudgett@digium.com>
+
+	* Sorcery: Create human friendly serializer names.
+
+	  Sorcery name formats:
+	  sorcery/<type>-<seq> -- Sorcery thread pool serializer
+
+	  Change-Id: Idc2e5d3dbab15c825b97c38c028319a0d2315c47
+
+2016-01-06 19:09 +0000 [b1c7ae9afc]  Richard Mudgett <rmudgett@digium.com>
+
+	* Stasis: Create human friendly taskprocessor/serializer names.
+
+	  Stasis name formats:
+	  subm:<topic>-<seq> -- Stasis subscription mailbox task processor
+	  subp:<topic>-<seq> -- Stasis subscription thread pool serializer
+
+	  Change-Id: Id19234b306e3594530bb040bc95d977f18ac7bfd
+
+2016-01-07 16:15 +0000 [3e857bb347]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: New API for human friendly taskprocessor names.
+
+	  * Add new API call to get a sequence number for use in human friendly
+	  taskprocessor names.
+
+	  * Add new API call to create a taskprocessor name in a given buffer and
+	  append a sequence number.
+
+	  Change-Id: Iac458f05b45232315ed64aa31b1df05b875537a9
+
+2016-01-06 17:19 +0000 [84c245d38c]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Fix CLI "core show taskprocessors" output format.
+
+	  Update the CLI "core show taskprocessors" output format to not be
+	  distorted because UUID names are longer than previously used taskprocessor
+	  names.
+
+	  Change-Id: I1a5c82ce3e8f765a0627796aba87f8f7be077601
+
+2016-01-07 21:07 +0000 [7d86979ea0]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Fix CLI "core show taskprocessors" unref.
+
+	  Change-Id: I1d9f4e532caa6dfabe034745dd16d06134efdce5
+
+2016-01-06 19:00 +0000 [1fb39aa8a0]  Richard Mudgett <rmudgett@digium.com>
+
+	* ccss.c: Replace space in taskprocessor name.
+
+	  The CLI "core ping taskprocessor" command does not work very
+	  well with taskprocessor names that have spaces in them.  You
+	  have to put quotes around the name so using tab completion
+	  becomes awkward.
+
+	  Change-Id: I29e806dd0a8a0256f4e2e0a7ab88c9e19ab0eda0
+
+2016-01-07 20:44 +0000 [71bb7b9c40]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Sort CLI "core show taskprocessors" output.
+
+	  Change-Id: I71e7bf57c7b908c8b8c71f1816348ed7c5a5d51e
+
+2016-01-05 16:54 +0000 [b025e1982f]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Add CLI "core ping taskprocessor" missing unlock.
+
+	  Change-Id: I78247e0faf978bf850b5ba4e9f4933ab3c59d17b
+
+2015-12-16 11:25 +0000 [c5e16fe33a]  Mark Michelson <mmichelson@digium.com>
+
+	* Alembic: Add PJSIP global keep_alive_interval.
+
+	  The keep_alive_interval option was added about a year ago, but no
+	  alembic revision was created to add the appropriate column to the
+	  database.
+
+	  This commit fixes the problem and adds the column. This was discovered
+	  by running the testsuite with automatic conversion to realtime enabled.
+
+	  Change-Id: If3ef92a7c4f4844d08f8aae170d2178aec5c4c1a
+
+2016-01-07 03:21 +0000 [6745cd6529]  Diederik de Groot <ddegroot@talon.nl>
+
+	* include/asterisk/time.h: Renamed global declaration:tv
+
+	  Renamed global declaration:tv to dummy_tv_var_for_types,
+	  which would oltherwise cause 'shadow' warnings when 'tv'
+	  was declared as a local variable elsewhere.
+
+	  Added comment to note that dummy_tv_var_for_types is never
+	  really exported and only used as a place holder.
+
+	  ASTERISK-25627 #close
+
+	  Change-Id: I9a6e17995006584f3627efe8988e3f8aa0f5dc28
+
+2016-01-07 15:37 +0000 [1afc8432dc]  Mark Michelson <mmichelson@digium.com>
+
+	* PJSIP: Prevent deadlock due to dialog/transaction lock inversion.
+
+	  A deadlock was observed where the monitor thread was stuck, therefore
+	  resulting in no incoming SIP traffic being processed.
+
+	  The problem occurred when two 200 OK responses arrived in response to a
+	  terminating NOTIFY request sent from Asterisk. The first 200 OK was
+	  dispatched to a threadpool worker, who locked the corresponding
+	  transaction. The second 200 OK arrived, resulting in the monitor thread
+	  locking the dialog. At this point, the two threads are at odds, because
+	  the monitor thread attempts to lock the transaction, and the threadpool
+	  thread loops attempting to try to lock the dialog.
+
+	  In this case, the fix is to not have the monitor thread attempt to hold
+	  both the dialog and transaction locks at the same time. Instead, we
+	  release the dialog lock before attempting to lock the transaction.
+
+	  There have also been some debug messages added to the process in an
+	  attempt to make it more clear what is going on in the process.
+
+	  ASTERISK-25668 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I4db0705f1403737b4360e33a8e6276805d086d4a
+
+2016-01-07 09:39 +0000 [5d8c42c6d3]  Corey Farrell <git@cfware.com>
+
+	* ast_format_cap_append_by_type: Resolve codec reference leak.
+
+	  This resolves a reference leak caused by ASTERISK-25535.  The pointer
+	  returned by ast_format_get_codec is saved so it can be released.
+
+	  ASTERISK-25664 #close
+
+	  Change-Id: If9941b1bf4320b2c59056546d6bce9422726d1ec
+
+2016-01-07 03:33 +0000 [7856762f2f]  Diederik de Groot <ddegroot@talon.nl>
+
+	* main: Use ast_strdup instead of strdup
+
+	  Fix compile error in main/utils.c because strdup was used in dummy_start
+
+	  Change-Id: Id61a6cf4f3cbf235450441e10e7da101a6335793
+
+2016-01-06 07:12 +0000 [64b2046f3d]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* Add sipp-sendfax.xml and spandspflow2pcap.py to contrib/scripts.
+
+	  The spandspflow2pcap.py creates pcap files from fax.log files, generated
+	  through 'fax set debug on' when receiving a fax. An example fax.log is
+	  included as spandspflow2pcap.log.
+
+	  The sipp-sendfax.xml SIPp scenario can be used to replay that fax with a
+	  recent version of SIPp.
+
+	  ASTERISK-25660 #close
+
+	  Change-Id: I4de8f28b084055b482ab8a5b28d28b605b0ed526
+
+2016-01-04 04:26 +0000 [084563e136]  Aaron An <anjb@ti-net.com.cn>
+
+	* cel/cel_radius: Fix wrong pointer.
+
+	  The macro ADD_VENDOR_CODE defined in the cel_radius.c should use the parameter
+	  y not the address of y.
+
+	  I capture the radius UDP packet via tcpdump, and the AV pairs are not correct,
+	  then i review the source code and compare it with cdr/cdr_radius.c. Fix it and
+	   it works.
+
+	  ASTERISK-25647 #close
+	  Reported by: Aaron An
+	  Tested by: Aaron An
+
+	  Change-Id: I72889bccd8fde120d47aa659edc0e7e6d4d019f0
+
+2016-01-04 20:23 +0000 [36f1eaf0b5]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move hangup handler routines to pbx_hangup_handler.c.
+
+	  This is the sixth patch in a series meant to reduce the bulk of pbx.c.
+	  This moves hangup handler management functions to their own source.
+
+	  Change-Id: Ib25a75aa57fc7d5c4294479e5cc46775912fb104
+
+2015-12-21 11:07 +0000 [90b06d1a3c]  Martin Tomec <tomec.martin@gmail.com>
+
+	* app_queue: Add member flag "in_call" to prevent reading wrong lastcall time
+
+	  Member lastcall time is updated later than member status. There was chance to
+	  check wrapuptime for available member with wrong (old) lastcall time.
+	  New boolean flag "in_call" is set to true right before connecting call, and
+	  reset to false after update of lastcall time. Members with "in_call" set to true
+	  are treat as unavailable.
+
+	  ASTERISK-19820 #close
+
+	  Change-Id: I1923230cf9859ee51563a8ed420a0628b4d2e500
+
+2016-01-04 19:46 +0000 [3507494b8a]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move dialplan application management routines to pbx_app.c.
+
+	  This is the sixth patch in a series meant to reduce the bulk of pbx.c.
+	  This moves dialplan application management functions to their own source.
+
+	  Change-Id: I444c10fb90a3cdf9f3047605d6a8aad49c22c44c
+
+2016-01-04 18:20 +0000 [54a8f1a396]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move switch routines to pbx_switch.c.
+
+	  This is the fifth patch in a series meant to reduce the bulk of pbx.c.
+	  This moves ast_switch functions to their own source.
+
+	  Change-Id: Ic2592a18a5c4d8a3c2dcf9786c9a6f650a8c628e
+
+2016-01-04 18:00 +0000 [c3c8b8e41d]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move timing routines to pbx_timing.c.
+
+	  This is the fourth patch in a series meant to reduce the bulk of pbx.c.
+	  This moves pbx timing functions to their own source.
+
+	  Change-Id: I05c45186cb11edfc901e95f6be4e6a8abf129cd6
+
+2015-12-30 10:49 +0000 [6d18fe151c]  gtjoseph <george.joseph@fairview5.com>
+
+	* voicemail: Move app_voicemail / res_mwi_external conflict to runtime
+
+	  The menuselect conflict between app_voicemail and res_mwi_external
+	  makes it hard to package 1 version of Asterisk.  There no actual
+	  build dependencies between the 2 so moving this check to runtime
+	  seems like a better solution.
+
+	  The ast_vm_register and ast_vm_greeter_register functions in app.c
+	  were modified to return AST_MODULE_LOAD_DECLINE instead of -1 if there
+	  is already a voicemail module registered. The modules' load_module
+	  functions were then modified to return DECLINE instead of -1 to the
+	  loader.  Since -1 is interpreted by the loader as AST_MODULE_LOAD_FAILURE,
+	  the modules were incorrectly causing Asterisk to stop so this needed
+	  to be cleaned up anyway.
+
+	  Now you can build both and use modules.conf to decide which voicemail
+	  implementation to load.
+
+	  The default menuselect options still build app_voicemail and not
+	  res_mwi_external but if both ARE built, res_mwi_external will load
+	  first and become the voicemail provider unless modules.conf rules
+	  prevent it.  This is noted in CHANGES.
+
+	  Change-Id: I7d98d4e8a3b87b8df9e51c2608f0da6ddfb89247
+
+2016-01-04 16:15 +0000 [5ee5c3739e]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move variable routines to pbx_variables.c.
+
+	  This is the third patch in a series meant to reduce the bulk of pbx.c.
+	  This moves channel and global variable routines to their own source.
+
+	  Change-Id: Ibe8fb4647db11598591d443a99e3f99200a56bc6
+
+2015-12-04 17:22 +0000 [f88b952093]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_dial: Immediately exit dial if the caller is already hung up.
+
+	  If a caller hangs up before dial is executed within an AGI then the AGI
+	  has likely eaten all queued frames before executing the dial in DeadAGI
+	  mode.  With the caller hung up and no pending frames from the caller's
+	  read queue, dial would not know that the call has hung up until a called
+	  channel answers.  It is rather annoying to whoever just answered the
+	  non-existent call.
+
+	  Dial should not continue execution in DeadAGI mode, hangup handlers, or
+	  the h exten.
+
+	  * Added a check early in dial to abort dialing if the caller has hungup.
+
+	  ASTERISK-25307 #close
+	  Reported by: David Cunningham
+
+	  Change-Id: Icd1bc0764726ef8c809f76743ca008d0f102f418
+
+2016-01-02 10:26 +0000 [e9dd16364e]  Matt Jordan <mjordan@digium.com>
+
+	* main/cdr: Allow setting properties on a finalized CDR if it is the last one
+
+	  Prior to this patch, we explicitly disallowed setting any properties on a
+	  finalized CDR. This seemed like a good idea at the time; in practice, it was
+	  more restrictive.
+
+	  There are weird and strange scenarios where setting a property on a finalized
+	  CDR is definitely wrong. For example, we may Fork a CDR, finalizing the
+	  previous one, then change a property. In said case, the old CDR is supposed
+	  to now be 'immutable' (so to speak), and should not be updated. From the
+	  perspective of the code, a forked CDR that is finalized is just finalized.
+	  Hence why we decided these should not be updated.
+
+	  In practice, it is much more common to want to set a property on a CDR in
+	  the h extension or in a hangup handler. Disallowing a common scenario to make
+	  an esoteric behaviour work isn't good. This patch fixes this by allowing
+	  callers to set a property IF we are the last CDR in the chain. This preserves
+	  the finalized CDR if it was forked, while allowing the more common case to
+	  function.
+
+	  ASTERISK-25458 #close
+
+	  Change-Id: Icf3553c607b9f561152a41e6d8381d594ccdf4b9
+
+2016-01-02 10:23 +0000 [153547a9b1]  Matt Jordan <mjordan@digium.com>
+
+	* main/cdr: Set the end time on a CDR if endbeforehexten is Yes
+
+	  Prior to this patch, the CDR engine attempted to set the end time on a CDR
+	  that was executing hangup logic and with endbeforehexten set to Yes by
+	  calling a function that inspects the properties on the Party A snapshot to
+	  determine if we are ready to set the end time. That always failed. This is
+	  because a Party A snapshot is not updated for CDRs that are executing hangup
+	  logic with endbeforehexten=Yes.
+
+	  Instead of calling a function that looks at the Party A snapshot, we just
+	  simply set the end time on the CDR. This is safe to call multiple times, and is
+	  safe to call at this point as we know that (a) we are executing hangup logic,
+	  and (b) we are supposed to set the end time at this point.
+
+	  ASTERISK-25458
+
+	  Change-Id: I0c27b493861f9c13c43addbbb21257f79047a3b3
+
+2015-12-30 20:51 +0000 [f9bfc2450e]  Corey Farrell <git@cfware.com>
+
+	* main/pbx: Move custom function routines to pbx_functions.c.
+
+	  This is the second patch in a series meant to reduce the bulk of pbx.c.
+	  This moves custom function management routines to their own source.
+
+	  Change-Id: I34a6190282f781cdbbd3ce9d3adeac3c3805e177
+
+2016-01-01 05:25 +0000 [3fd528dddf]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* Happy new year 2016.
+
+	  Change-Id: I22d3c90f6f27df82e915bbf81c1d91221f7a945e
+
+2015-12-13 13:09 +0000 [9cdf3ec19d]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip_history: Add a module that provides PJSIP history for debugging
+
+	  This patch adds a new module, res_pjsip_history, that provides a slightly
+	  better way of debugging SIP message traffic on a busy Asterisk system. The
+	  existing mechanisms all rely on passively dumping a SIP message to the CLI.
+	  While this is perfectly fine for logging purposes and well controlled
+	  environments, on many installations, the amount of SIP messages Asterisk
+	  receives will quickly swamp the CLI. This makes it difficult to view/capture
+	  those messages that you want to diagnose in real time.
+
+	  This patch provides another way of handling this. When enabled, the module
+	  will store SIP message traffic in memory. This traffic can then be queried
+	  at leisure.
+
+	  In order to make the querying useful, a CLI command has been implemented,
+	  'pjsip show history', that supports a basic expression syntax similar to
+	  SQL or other query languages. A small number of useful fields have been
+	  added in this initial patch; additional fields can easily be added in
+	  later improvements. Those fields are:
+	   - number: The entry index in the history
+	   - timestamp: The time the message was recieved
+	   - addr: The source/destination address of the message
+	   - sip.msg.request.method: The request method
+	   - sip.msg.call-id: The Call-ID header
+
+	  Note - this is a resurrection of the module initially proposed on Review Board
+	  here: https://reviewboard.asterisk.org/r/4053/
+
+	  Change-Id: I39bd74ce998e99ad5ebc0aab3e84df3a150f8e36
+
+2015-12-28 19:18 +0000 [5e67e51c6a]  gtjoseph <george.joseph@fairview5.com>
+
+	* main/pbx: Move pbx_builtin dialplan applications to pbx_builtins.c
+
+	  We joked about splitting pbx.c into multiple files but this first step was
+	  fairly easy.  All of the pbx_builtin dialplan applications have been moved
+	  into pbx_builtins.c and a new pbx_private.h file was added. load_pbx_builtins()
+	  is called by asterisk.c just after load_pbx().
+
+	  A few functions were renamed and are cross-exposed between the 2 source files.
+
+	  Change-Id: I87066be3dbf7f5822942ac1449d98cc43fc7561a
+
+2015-12-28 14:02 +0000 [a05bb258b1]  Joshua Colp <jcolp@digium.com>
+
+	* test_time: Provide a timeout when waiting.
+
+	  The test_timezone_watch unit test is written to expect a
+	  condition to be signaled when the inotify daemon thread runs.
+	  There exists a small window where the test_timezone_watch
+	  thread can signal the inotify daemon thread while it is not
+	  reading on the underlying file descriptor. If this occurs
+	  the test_timezone_watch thread will wait indefinitely for a
+	  signal that will never arrive.
+
+	  This change adds a timeout to the condition so it will return
+	  regardless after a period of time.
+
+	  Change-Id: Ifed981879df6de3d93acd3ee0a70f92546517390
+
+2015-12-24 20:26 +0000 [96b32e0321]  Matt Jordan <mjordan@digium.com>
+
+	* tests/test_stasis_endpoints: Remove expected duplicate events
+
+	  The cache_clear test was written to expect duplicate Stasis messages
+	  sent from the technology endpoint to the all caching topic. This patch
+	  fixes the test to no longer expect these duplicate messages.
+
+	  ASTERISK-25137
+
+	  Change-Id: I58075d70d6cdf42e792e0fb63ba624720bfce981
+
+2015-12-24 22:19 +0000 [3bddcc0219]  Dade Brandon <dade@xencall.com>
+
+	* res_http_websocket.c: prevent avoidable disconnections caused by write errors
+
+	  Updated ast_websocket_write to encode the entire frame in to one
+	  write operation, to ensure that we don't end up with a situation
+	  where the websocket header has been sent, while the body can not
+	  be written.
+
+	  Previous to August's patch in commit b9bd3c14, certain network
+	  conditions could cause the header to be written, and then the
+	  sub-sequent body to fail - which would cause the next successful
+	  write to contain a new header, and a new body (resulting in
+	  the peer receiving two headers - the second of which would be
+	  read as part of the body for the first header).
+
+	  This was patched to have both write operations individually fail
+	  by closing the websocket.
+
+	  In a case available to the submitter of this patch, the same
+	  body which would consistently fail to write, would succeed
+	  if written at the same time as the header.
+
+	  This update merges the two operations in to one, adds debug messages
+	  indicating the reason for a websocket connection being closed during
+	  a write operation, and clarifies some variable names for code legibility.
+
+	  Change-Id: I4db7a586af1c7a57184c31d3d55bf146f1a40598
+
+2015-05-27 13:22 +0000 [22db16fa81]  gtjoseph <george.joseph@fairview5.com>
+
+	* endpoint/stasis: Eliminate duplicate events on endpoint status change
+
+	  When an endpoint is created, its messages are forwarded to both the tech
+	  endpoint topic and the all endpoints topic. This is done so that various
+	  parties interested in endpoint messages can subscribe to just the tech
+	  endpoint and receive all messages associated with that particular technology,
+	  as opposed to subscribing to the all endpoints topic. Unfortunately, when the
+	  tech endpoint is created, it also forwards all of its messages to the all
+	  topic. This results in duplicate messages whenever an endpoint publishes its
+	  messages.
+
+	  This patch resolves the duplicate message issue by creating a new function
+	  for Stasis caching topics, stasis_cp_sink_create. In most respects, this acts
+	  as a normal caching topic, save that it no longer forwards messages it receives
+	  to the all endpoints topic. This allows it to act as an aggregation "sink",
+	  while preserving the necessary caching behaviour.
+
+	  ASTERISK-25137 #close
+	  Reported-by: Vitezslav Novy
+
+	  ASTERISK-25116 #close
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+
+	  Change-Id: Ie47784adfb973ab0063e59fc18f390d7dd26d17b
+
+2015-12-27 22:38 +0000 [6b08f01c60]  Corey Farrell <git@cfware.com>
+
+	* Remove res_jabber file that was left behind.
+
+	  Change-Id: I9d88fac0394d5bbaff0900a2ee911c4e4478846b
+
+2015-12-26 09:24 +0000 [d4b10cfb3e]  Ward van Wanrooij <ward@ward.nu>
+
+	* chan_sip: option 'notifyringing' change and doc fix
+
+	  In the sample sip.conf this is written with regard to notifyringing:
+	  ;notifyringing = no ; Control whether subscriptions already INUSE get sent
+	  RINGING when another call is sent (default: yes)
+
+	  However, this setting changes whether or not any RINGING indications are sent
+	  to subscriptions. There is no separate configurable setting that allows
+	  to control whether INUSE subscriptions also get sent RINGING. This is however
+	  a useful option, to see (using BLF) if somebody else is able to handle an
+	  incoming call or if everybody is busy.
+
+	  This patch corrects the documentation for notifyringing (so the documentation
+	  matches the functionality) and make notifyringing a tri-state option, by adding
+	  the value 'notinuse' (in addition to 'yes' and 'no'). When notifyringing =
+	  notinuse, only subscriptions that are not INUSE are sent the RINGING signal.
+
+	  The default setting for notifyringing remains set to yes, so the default
+	  behaviour is not affected.
+
+	  ASTERISK-25558
+
+	  Change-Id: I88f7036ee084bb3f43b74f15612695c6708f74aa
+
+2015-12-25 09:56 +0000 [6dc21bbf00]  Dade Brandon <dade@xencall.com>
+
+	* chan_sip.c: fix websocket_write_timeout default value
+
+	  websocket_write_timeout was not being set to its default value
+	  during sip config reload, which meant that prior to this commit,
+	  1) the default value of 100 was not used, unless an invalid value
+	  (or 1) was specified in sip.conf for websocket_write_timeout, and
+	  2) if the websocket_write_timeout directive was removed from sip.conf
+	  without a full restart of asterisk, then the previous value would
+	  continue to be used indefinitely.
+
+	  This essentially lead to a 0ms write timeout (the first write attempt
+	  in ast_careful_fwrite must have succeeded) in websocket write requests
+	  from chan_sip, unless websocket_write_timeout was explicitely set in sip.conf.
+
+	  Changes to websocket_write_timeout still only apply to new websocket
+	  sessions, after the sip reload -- timeouts on existing sessions are
+	  not adjusted during sip reload.
+
+	  Change-Id: Ibed3816ed29cc354af6564c5ab3e75eab72cb953
+
+2015-12-23 17:40 +0000 [8eb5da0679]  Richard Mudgett <rmudgett@digium.com>
+
+	* bridge_basic.c: Fix GOTO_ON_BLINDXFR
+
+	  Use of GOTO_ON_BLINDXFR would not work at all.  The target location would
+	  never be executed by the transferring channel.
+
+	  * Made feature_blind_transfer() call ast_bridge_set_after_go_on() with
+	  valid context, exten, and priority parameters from the transferring
+	  channel.
+
+	  * Renamed some feature_blind_transfer() local variables for clarity.
+
+	  ASTERISK-25641 #close
+	  Reported by Dmitry Melekhov
+
+	  Change-Id: I19bead9ffdc4aee8d58c654ca05a198da1e4b7ac
+
+2015-12-24 12:19 +0000 [2df4ad647c]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_location: Delete contact_status object when contact is deleted
+
+	  In 450579e908, a change was made that removed the deletion of the
+	  'contact_status' object when a 'contact' object is deleted in sorcery.
+	  This unfortunately means that the 'contact_status' object persists, even when
+	  something has explicitly removed a contact. The result is that the state of
+	  the contact will not be regenerated if that contact is re-created, and the
+	  stale state will be reported/used for that contact. It also results in
+	  no ContactStatusChanged events being generated for either ARI or AMI.
+
+	  This patch restores the deletion logic that was removed. Doing so now
+	  results in the expected events being generated again.
+
+	  Change-Id: I28789a112e845072308b5b34522690e3faf58f07
+
+2015-12-24 10:18 +0000 [b8876711f3]  Kevin Harwell <kharwell@digium.com>
+
+	* res_rtp_asterisk: rtp->ice check not wrapped in HAVE_PJPROJECT ifdef
+
+	  Change-Id: I19b49112e1b630bd04e859f14ccf96f8ebd6b151
+
+2015-12-20 21:33 +0000 [ca394161cf]  Dade Brandon <dade@xencall.com>
+
+	* app_amd: Correct maximum_number_of_words functionality & documentation
+
+	  - The maximum_number_of_words was previously documented as being
+	  the number of words that when exceeded, would result in the AMD
+	  application returning that the audio represents a machine.
+
+	  This was inconsistent with its actual functionality - it was
+	  a number of words that when REACHED, would result in determination
+	  as a machine.
+
+	  This update corrects the functionality to match the previously
+	  documented functionality.  This is a backwards incompatible change
+	  in configuration file, and has been added to UPGRADE.txt as a result.
+
+	  The sample configuration file and application defaults have been updated
+	  so that the default value is now 2, which reflects the same default
+	  functionality as previous versions.
+
+	  - Update documentation for silence_threshold, which previously implied
+	  that it was measuring time, rather than noise averages in the sample.
+
+	  - Update the comments in amd.conf.sample.
+
+	  ASTERISK-25639 #close
+	  Change-Id: I4b1451e5dc9cb3cb06d59b6ab872f5275ba79093
+
+2015-12-17 19:05 +0000 [648ca2b1b8]  Dade Brandon <dade@xencall.com>
+
+	* res_rtp_asterisk: Resolve further timing issues with DTLS negotiation
+
+	  Resolves an edge case dtls negotiation delay for certain networks which
+	  somehow manage to drop the rtcp side's packet when these are both sent
+	  ast_rtp_remote_address_set, causing it to have to time-out and restart
+	  the handshake.
+
+	  Move dtls pending bio flush in to it's own function, and call it from
+	  ast_rtp_on_ice_complete, when we're rtp->ice, rather than when
+	  ast_rtp_remote_address_set.
+
+	  Keep the existing flush from the recent change to res_rtp_remote_address_set
+	  if ice is not being used.
+
+	  ASTERISK-25614 #close
+	  Reported-by: XenCALL
+	  Tested by: XenCALL
+
+	  Change-Id: Ie2caedbdee1783159f375589b6fd3845c8577ba5
+
+2015-12-05 10:01 +0000 [902309fd04]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Add support for a full backend cache.
+
+	  This change introduces the configuration option 'full_backend_cache'
+	  which changes the cache to be a full mirror of the backend instead
+	  of a per-object cache. This allows all sorcery retrieval operations
+	  to be carried out against it and is useful for object types which
+	  are used in a "retrieve all" or "retrieve some" pattern.
+
+	  ASTERISK-25625 #close
+
+	  Change-Id: Ie2993487e9c19de563413ad5561c7403b48caab5
+
+2015-12-17 10:25 +0000 [a2431f83ef]  Joshua Colp <jcolp@digium.com>
+
+	* rtp_engine: Ignore empty filenames in DTLS configuration.
+
+	  When applying an empty DTLS configuration the filenames in the
+	  configuration will be empty. This is actually valid to do and
+	  each filename should simply be ignored.
+
+	  Change-Id: Ib761dc235638a3fb701df337952f831fc3e69539
+
+2015-12-17 08:10 +0000 [d2c8614122]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip: Enable WebSocket support by default.
+
+	  Per the documentation the WebSocket support in chan_sip is
+	  supposed to be enabled by default but is not. This change
+	  corrects that.
+
+	  Change-Id: Icb02bbcad47b11a795c14ce20a9bf29649a54423
+
+2015-12-14 12:04 +0000 [d17d9a9288]  Joshua Colp <jcolp@digium.com>
+
+	* json: Audit ast_json_* usage for thread safety.
+
+	  The JSON library Asterisk uses, jansson, is not thread
+	  safe for us in a few ways. To help with this wrappers for JSON
+	  object reference count increasing and decreasing were added
+	  which use a global lock to ensure they don't clobber over
+	  each other. This does not extend to reference count manipulation
+	  within the jansson library itself. This means you can't safely
+	  use the object borrowing specifier (O) in ast_json_pack and
+	  you can't share JSON instances between objects.
+
+	  This change removes uses of the O specifier and replaces them
+	  with the o specifier and an explicit ast_json_ref. Some cases
+	  of instance sharing have also been removed.
+
+	  ASTERISK-25601 #close
+
+	  Change-Id: I06550d8b0cc1bfeb56cab580a4e608ae4f1ec7d1
+
+2015-12-16 11:28 +0000 [cfb34adb83]  Mark Michelson <mmichelson@digium.com>
+
+	* Alembic: Increase column size of PJSIP AOR "contact".
+
+	  When running the PJSIP AMI "show_endpoint" test with automatic
+	  conversion to realtime, the test would fail. This was because the AOR
+	  "contact" column was sized at 40, and the configured contact was larger
+	  than that.
+
+	  This commit increases the size of the contact column to 255 characters.
+
+	  Change-Id: Ia65bc7fd37699b7c0eaef9629a1a31eab9a24ba1
+
+2015-12-14 13:53 +0000 [32ec83f37f]  server-pandora <server-pandora@xencall.com>
+
+	* res_rtp_asterisk.c: Fix DTLS negotiation delays.
+
+	  - Trigger pending DTLS packets to send out, once the RTP instance's remote
+	    address is set.
+	  - Avoids locking the DTLS structure unnecessarily by only doing this if
+	    DTLS is passive.
+	  - Add DTLS locks around the structurally sensitive calls in the SSL
+	    portion of __rtp_recvfrom, since dtls_srtp_check_pending does not lock
+	    inside of itself, and we're dealing with the SSL BIO in at least two
+	    threads.
+
+	  WebRTC channels may receive a DTLS handshake before
+	  ast_rtp_remote_address_set is called, which causes there to be a pending
+	  response to send out.   Previous to 1ad827, this was handled by calling
+	  dtls_srtp_check_pending on receipt of any RTP packet - a STUN or RTP
+	  packet could trigger the pending handshake response.  Since that was
+	  rightfully removed, whenever the DTLS handshake is received before the
+	  remote address is set, we would have to wait until another SSL packet
+	  arrives.
+
+	  As of Chrome M47's optimizations to their handshake process, WebRTC
+	  conversations between Chrome M47+ and Asterisk, where Asterisk is passive,
+	  experience a 1 second delay without this patch, because the SSL handshake
+	  is received before ICE negotation stores the remote_address, and the next
+	  SSL packet isn't received until after a 1 second timeout in Chrome, which
+	  causes a new handshake request.
+
+	  ASTERISK-25614 #close
+
+	  Change-Id: I547f1be7e302dbf71f6553dd8cbc0657b1d0b908
+
+2015-12-08 13:04 +0000 [52ca6fb94a]  sungtae kim <pchero21@gmail.com>
+
+	* AMI: Fixed OriginateResponse message
+
+	  When the asterisk sending OriginateResponse message,
+	  it doesn't set the "Uniqueid".
+	  And it didn't support correct response message for
+	  Application originate.
+
+	  ASTERISK-25624 #close
+
+	  Change-Id: I26f54f677ccfb0b7cfd4967a844a1657fd69b74d
+
+2015-12-14 15:25 +0000 [eccdf2250b]  Richard Mudgett <rmudgett@digium.com>
+
+	* Fix sscanf() format string type mismatch.
+
+	  ASTERISK-25615
+	  Reported by: George Joseph
+
+	  Change-Id: Ieff35307254ca193f3d473cff2e396ca57c7ce0b
+
+2015-12-14 06:26 +0000 [3e7522533c]  Carlos Oliva <carlos.oliva@invoxcontact.com>
+
+	* app_queue: update RT members when the 1st call joins a queue with no agents
+
+	  If a call enters on a queue and the members on that queue are updated in
+	  realtime (ex: using mysql inserting a new agent) the queue members are
+	  never refreshed and the call will stay in the queue until other event occurs.
+	  This happens only if this is the first call of the queue and there is no
+	  agents servicing.
+	  This patch prevent this issue, ensuring realtime members are updated if
+	  there is one call in the queue and no available agents
+
+	  ASTERISK-25442 #close
+
+	  Change-Id: If1e036d013a5c1d8b0bf60d71d48fe98694a8682
+
+2015-12-13 13:13 +0000 [9a96a86e2d]  Matt Jordan <mjordan@digium.com>
+
+	* main/utils: Don't emit an ERROR message if the read end of a pipe closes
+
+	  An ERROR or WARNING message should generally indicate that something has gone
+	  wrong in Asterisk. In the case of writing to a file descriptor, Asterisk is not
+	  in control of when the far end closes its reading on a file descriptor. If the
+	  far end does close the file descriptor in an unclean fashion, this isn't a bug
+	  or error in Asterisk, particularly when the situation can be gracefully
+	  handled in Asterisk.
+
+	  Currently, when this happens, a user would see the following somewhat cryptic
+	  ERROR message:
+
+	    "utils.c: write() returned error: Broken pipe"
+
+	  There's a few problems with this:
+	  (1) It doesn't provide any context, other than 'something broke a pipe'
+	  (2) As noted, it isn't actually an error in Asterisk
+	  (3) It can get rather spammy if the thing breaking the pipe occurs often, such
+	      as a FastAGI server
+	  (4) Spammy ERROR messages make Asterisk appear to be having issues, or can even
+	      mask legitimate issues
+
+	  This patch changes ast_carefulwrite to only log an ERROR if we actually had one
+	  that was reasonably under our control. For debugging purposes, we still emit
+	  a debug message if we detect that the far side has stopped reading.
+
+	  Change-Id: Ia503bb1efcec685fa6f3017bedf98061f8e1b566
+
+2015-12-12 11:08 +0000 [3e6637feb5]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip/config_transport: Check pjproject version at runtime for async ops
+
+	  pjproject < 2.5.0 will segfault on a tls transport if async_operations
+	  is greater than 1.  A runtime version check has been added to throw
+	  an error if the version is < 2.5.0 and async_operations > 1.
+
+	  To assist in the check, a new api "ast_compare_versions" was added
+	  to utils which compares 2 major.minor.patch.extra version strings.
+
+	  ASTERISK-25615 #close
+
+	  Change-Id: I8e88bb49cbcfbca88d9de705496d6f6a8c938a98
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+2015-12-10 11:44 +0000 [ceebdfce40]  Jonathan Rose <jrose@digium.com>
+
+	* chan_sip: Add TCP/TLS keepalive to TCP/TLS server
+
+	  Adds the TCP Keep Alive option to TCP and TLS server sockets. Previously
+	  this option was only being set on session sockets.
+	  http://www.tldp.org/HOWTO/html_single/TCP-Keepalive-HOWTO/
+	  According to the link above, the SO_KEEPALIVE option is useful for knowing
+	  when a TCP connected endpoint has severed communication without indicating
+	  it or has become unreachable for some reason. Without this patch, keep
+	  alive is not set on the socket listening for incoming TCP sessions and
+	  in Komatsu's report this resulted in the thread listening for TCP becoming
+	  stuck in a waiting state.
+
+	  ASTERISK-25364 #close
+	  Reported by: Hiroaki Komatsu
+
+	  Change-Id: I7ed7bcfa982b367dc64b4b73fbd962da49b9af36
+2015-12-07 13:07 +0000 [fcaebb0e43]  Corey Farrell <git@cfware.com>
+
+	* app_meetme: Set default value for audio_buffers.
+
+	  The default value was never set for audio_buffers, causing bad
+	  audio quality.  This ensures the default is always set.
+
+	  ASTERISK-25569 #close
+
+	  Change-Id: I2d2ee3e644120b0f9f6ea6ab9286d7d590942a44
+2015-12-09 09:48 +0000 [5790700497]  Tyler Cambron <tcambron@digium.com>
+
+	* res_chan_stats: Fix bug to send correct statistics to StatsD
+
+	  Fixed a bug that originally would show a negative number of
+	  active calls occuring in Asterisk. A gauge is persistent so
+	  incrementing and decrementing it results in a more consistent
+	  performance. Also changed to the call to StatsD to use
+	  ast_statsd_log_string() so that a "+" could be sent to StatsD.
+
+	  ASTERISK-25619 #close
+
+	  Change-Id: Iaaeff5c4c6a46535366b4d16ea0ed0ee75ab2ee7
+2015-12-08 17:49 +0000 [a987434564]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Add existence and readablity checks for tls related files
+
+	  Both transport and endpoint now check for the existence and readability
+	  of tls certificate and key files before passing them on to pjproject.
+	  This will cause the object to not load rather than waiting for pjproject
+	  to discover that there's a problem when a session is attempted.
+
+	  NOTE: chan_sip also uses ast_rtp_dtls_cfg_parse but it's located
+	  in build_peer which is gigantic and I didn't want to disturb it.
+	  Error messages will emit but it won't interrupt chan_sip loading.
+
+	  ASTERISK-25618 #close
+
+	  Change-Id: Ie43f2c1d653ac1fda6a6f6faecb7c2ebadaf47c9
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+2015-12-02 12:42 +0000 [be693539c3]  Eugene Voityuk <eugene@thirdlane.com>
+
+	* chan_sip.c: Start ICE negotiation when response is sent or received.
+
+	  The current logic for ICE negotiation starts it
+	  when receiving an SDP with ICE candidates. This is
+	  incorrect as ICE negotiation can only start when each 
+	  call party have at least one pair of local and remote 
+	  candidate. Starting ICE negotiation early would result 
+	  in negotiation failure and ultimately no audio.
+
+	  This change makes it so ICE negotiation is only started
+	  when a response with SDP is received or when a response
+	  with SDP is sent.
+
+	  ASTERISK-24146
+
+	  Change-Id: I55a632bde9e9827871b09141d82747e08379a8ca
+
+2015-12-08 01:57 +0000 [59a91c350a]  Filip Jenicek <phill@janevim.cz>
+
+	* chan_sip: Check sip_pvt pointer in ast_channel_get_t38_state(c)
+
+	  Asterisk may crash when calling ast_channel_get_t38_state(c)
+	  on a locked channel which is being hung up.
+
+	  ASTERISK-25609 #close
+
+	  Change-Id: Ifaa707c04b865a290ffab719bd2e5c48ff667c7b
+2015-12-08 11:03 +0000 [28ab03fbf7]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/config_transport: Prevent async_operations > 1 when protocol = tls
+
+	  See ASTERISK-25615.
+	  If the transport protocol is tls and async_operations > 1, pjproject
+	  will segfault if more than one operation is attempted on the same socket.
+	  Until this is fixed upstream, a check has been added to throw an error
+	  if a tls transport config has async_operations set to > 1.
+
+	  ASTERISK-25615
+
+	  Change-Id: I76b9a5b2a5a0054fe71ca5851e635f2dca7685a6
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+2015-12-08 08:39 +0000 [55dd7125b3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* codec_resample: Increase buffer for Opus Codec with FEC.
+
+	  ASTERISK-25599 #close
+
+	  Change-Id: Idbd187f711b2ec63dda949ca0f79aa0c1a0a0b6e
+
+2015-12-08 03:46 +0000 [64f899e5f3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* translate: Avoid a warning message when doing FEC within Opus Codec.
+
+	  ASTERISK-25616 #close
+
+	  Change-Id: Ibe729aaf2e6e25506cff247cec5149ec1e589319
+
+2015-12-04 15:36 +0000 [65c8147952]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip: Fix crash involving the bogus peer during sip reload.
+
+	  A crash happens sometimes when performing a CLI "sip reload".  The bogus
+	  peer gets refreshed while it is in use by a new call which can cause the
+	  crash.
+
+	  * Protected the global bogus peer object with an ao2 global object
+	  container.
+
+	  ASTERISK-25610 #close
+
+	  Change-Id: I5b528c742195681abcf713c6e1011ea65354eeed
+
+2015-11-13 07:58 +0000 [48c065e46d]  Christof Lauber <christof.lauber@annax.ch>
+
+	* chan_sip: Support parsing of Q.850 reason header in SIP BYE and CANCEL requests.
+
+	  Current support for reason header did work only in SIP responses.
+	  According to RFC3336 the reason header might appear in any SIP request.
+	  But it seems to make most sence in BYE and CANCEL so parasing is done
+	  there too (if use_q850_reason=yes).
+
+	  Change-Id: Ib6be7b34c23a76d0e98dfd0816c89931000ac790
+
+2015-12-06 16:35 +0000 [75c800eb28]  Matt Jordan <mjordan@digium.com>
+
+	* Revert "bridges/bridge_t38: Add a bridging module for managing T.38 state"
+
+	  This reverts commit f42d22d3a1ca5c8ea73df99a50c6a28caa8f8749.
+
+	  Unfortunately, using a bridge to manage T.38 state will cause severe deadlocks
+	  in core_unreal/chan_local. Local channels attempt to reach across both their
+	  peer and the peer's bridge to inspect T.38 state. Given the propensity of
+	  Local channel chains, managing the locking situation in such a scenario is
+	  practically infeasible.
+
+	  Change-Id: I932107387c13aad2c75a7a4c1e94197a9d6d8a51
+
+2015-12-04 16:23 +0000 [4be231e82f]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/contacts/statsd:  Make contact lifecycle events more consistent
+
+	  It will never be perfect or even pretty, mostly because of the differences
+	  between static and dynamic contacts.
+
+	  Created:
+
+	  Can't use the contact or contact_status alloc functions
+	  because the objects come and go regardless of the actual state.
+
+	  Can't use the contact_apply_handler, ast_sip_location_add_contact or
+	  a sorcery created handler because they only get called for dynamic
+	  contacts.  Similarly, permanent_uri_handler only gets called for
+	  static contacts.
+
+	  So, Matt had it right. :)  ast_res_pjsip_find_or_create_contact_status is
+	  the only place it can go and not have duplicated code.  Both
+	  permanent_uri_handler and contact_apply_handler call find_or_create.
+
+	  Removed:
+
+	  Can't use the destructors for the same reason as above.  The only
+	  place to put this is in persistent_endpoint_contact_deleted_observer
+	  which I believe is the "correct" place but even that will handle only
+	  dynamic contacts.  This doesn't called on shutdown however.  There is
+	  no hook to use for static contacts that may be removed because of a
+	  config change while asterisk is in operation.
+
+	  I moved the cleanup of contact_status from ast_sip_location_delete_contact
+	  to the handler as well.
+
+	  Status Change and RTT:
+
+	  Although they worked fine where they were (in update_contact_status) I
+	  moved them to persistent_endpoint_contact_status_observer to make it
+	  more consistent with removed.  There was logic there already to detect
+	  a state change.
+
+	  Finally, fixed a nit in permanent_uri_handler rmudgett reported
+	  eralier.
+
+	  ASTERISK-25608 #close
+
+	  Change-Id: I4b56e7dfc3be3baaaf6f1eac5b2068a0b79e357d
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+2015-11-21 06:08 +0000 [63c6d39a3e]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_format_attr_vp8: In SDP, forward max-fr and max-fs for video-codec VP8.
+
+	  ASTERISK-25584 #close
+
+	  Change-Id: Iae00071b4ff1ae76f24995aeac4d00284fd14f91
+
+2015-11-28 08:46 +0000 [f42d22d3a1]  Matt Jordan <mjordan@digium.com>
+
+	* bridges/bridge_t38: Add a bridging module for managing T.38 state
+
+	  When 4875e5ac32 was merged, it fixed several issues with a direct media bridge
+	  transitioning to handling a T.38 fax. However, it uncovered a race condition
+	  caused by the bridging core. When a channel involved in a T.38 fax leaves a
+	  bridge, the frame queued by the channel driver that should inform the far side
+	  that it is no longer in a T.38 fax may not make it across the bridge. The
+	  bridging framework is *extremely* aggressive in tearing down the bridge, and
+	  control frames that are currently in flight *may* get dropped.
+
+	  This patch adds a new module to the bridging framework, bridge_t38. This module
+	  maintains some notion of the T.38 state for the two channels in a bridge. When
+	  the bridge detects that it is being torn down or when one of the two channels
+	  leaves, it informs the respective channel(s) that they should stop faxing. This
+	  ensures that channels switch back to audio if they survive and are ejected out
+	  of a bridge while faxing.
+
+	  ASTERISK-25582
+
+	  Change-Id: If5b0bb478eb01c4607c9f4a7fc17c7957d260ea0
+
+2015-11-21 05:35 +0000 [dcc01bc0a7]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_format_attr_opus: Update to latest RFC 7587.
+
+	  Beside that, the format-attribute module sends only non-default values in the
+	  line fmtp, now. This avoids unnecessary overhead in SDP messages. Furthermore,
+	  previously the parameter stereo was not parsed when being the first parameter.
+
+	  ASTERISK-25583 #close
+
+	  Change-Id: Iae85ba3e5960bfd5d51cf65bcffad00dd4875a73
+2015-12-02 14:11 +0000 [69457b8d61]  Jonathan Rose <jrose@digium.com>
+
+	* Fix crash in audiohook translate to slin
+
+	  This patch fixes a crash which would occur when an audiohook was
+	  applied to a channel using an audio codec that could not be translated
+	  to signed linear (such as when using pass-through codecs like OPUS or
+	  when the codec translator module for the format in use is not loaded).
+
+	  ASTERISK-25498 #close
+	  Reported by: Ben Langfeld
+
+	  Change-Id: Ib6ea7373fcc22e537cad373996136636201f4384
+2015-12-03 12:07 +0000 [5959186017]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip: Use a MD5 hash for static Contact IDs
+
+	  When 90d9a70789 was merged, it mostly tested dynamic contacts created as
+	  a result of registering a PJSIP endpoint. Contacts generated in this
+	  fashion typically have a long alphanumeric string as their object identifier,
+	  which maps reasonably well for StatsD. Unfortunately, this doesn't work in the
+	  general case. StatsD treats both '.' and ':' characters as special characters.
+	  In particular, having a ':' appear in the middle of a StatsD metric will
+	  result in the metric being rejected.
+
+	  This causes some obvious issues with SIP URIs.
+
+	  The StatsD API should not be responsible for escaping the metric name passed
+	  to it. The metric is treated as a single long string, and it would be
+	  challenging to know what to escape in the string passed to the function.
+	  Likewise, we don't want to escape the metric in PJSIP, as that involves
+	  overhead that is wasted when either res_statsd isn't loaded or enabled.
+
+	  This patch takes an alternative approach. The Contact ID has been changed
+	  to be "aor@@uri_hash" instead of "aor@@uri". This (a) won't contain any of the
+	  aforementioned special characters, (b) can be done on Contact creation,
+	  which has minimal impact on run-time performance, and (c) also conforms to an
+	  earlier commit that changed the ID for dynamic contacts.
+
+	  The downside of this is that StatsD users will have to map SHA1 hashes back to
+	  the Contacts that are emitting the statistics. To that end, the CLI commands
+	  have been updated to include the first 10 characters of the MD5 hash, which
+	  should be enough to match what is shown in Graphite (or some other StatsD
+	  backend).
+
+	  ASTERISK-25595 #close
+
+	  Change-Id: Ic674a3307280365b4a45864a3571c295b48a01e2
+	  Reported-by: Matt Jordan
+	  Tested-by: George Joseph
+
+2015-11-30 22:19 +0000 [bd265a90be]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Update logging to show contact->uri in messages
+
+	  An earlier commit changed the id of dynamic contacts to contain
+	  a hash instead of the uri.  This patch updates status change
+	  logging to show the aor/uri instead of the id.  This required
+	  adding the aor id to contact and contact_status and adding
+	  uri to contact_status.  The aor id gets added to contact and
+	  contact_status in their allocators and the uri gets added to
+	  contact_status in pjsip_options when the contact_status is
+	  created or updated.
+
+	  ASTERISK-25598 #close
+
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+	  Change-Id: I56cbec1d2ddbe8461367dd8b6da8a6f47f6fe511
+
+2015-12-01 16:11 +0000 [b5281b74e0]  Jonathan Rose <jrose@digium.com>
+
+	* Unset BRIDGEPEER when leaving a bridge
+
+	  Currently if a channel is transferred out of a bridge, the BRIDGEPEER
+	  variable (also BRIDGEPVTCALLID) remain set even once the channel is
+	  out of the bridge. This patch removes these variables when leaving
+	  the bridge.
+
+	  ASTERISK-25600 #close
+	  Reported by: Mark Michelson
+
+	  Change-Id: I753ead2fffbfc65427ed4e9244c7066610e546da
+
+2015-11-30 14:22 +0000 [59ba84e5cd]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Fix off nominal ref leak.
+
+	  Change-Id: If83d63cf11cbc6df9b15251848b01feb570ade49
+
+2015-11-30 16:42 +0000 [ef77439e39]  Richard Mudgett <rmudgett@digium.com>
+
+	* sched.c: Make not return a sched id of 0.
+
+	  According to the API doxygen a sched ID of 0 is valid.  Unfortunately, 0
+	  was never returned historically and several users incorrectly coded usage
+	  of the returned sched ID assuming that 0 was invalid.
+
+	  ASTERISK-25476
+
+	  Change-Id: Ib19c7ebb44ec9fd393ef6646dea806d4f34e3a20
+
+2015-11-25 12:23 +0000 [145d10a5d0]  Richard Mudgett <rmudgett@digium.com>
+
+	* Audit improper usage of scheduler exposed by 5c713fdf18f. (v13 additions)
+
+	  chan_sip.c:
+	  * Initialize mwi subscription scheduler ids earlier because of ASTOBJ to
+	  ao2 conversion.
+
+	  * Initialize register scheduler ids earlier because of ASTOBJ to ao2
+	  conversion.
+
+	  chan_skinny.c:
+	  * Fix more scheduler usage for the valid 0 id value.
+
+	  ASTERISK-25476
+
+	  Change-Id: If9f0e5d99638b2f9d102d1ebc9c5a14b2d706e95
+
+2015-11-24 12:44 +0000 [fa20729032]  Richard Mudgett <rmudgett@digium.com>
+
+	* Audit improper usage of scheduler exposed by 5c713fdf18f.
+
+	  channels/chan_iax2.c:
+	  * Initialize struct chan_iax2_pvt scheduler ids earlier because of
+	  iax2_destroy_helper().
+
+	  channels/chan_sip.c:
+	  channels/sip/config_parser.c:
+	  * Fix initialization of scheduler id struct members.  Some off nominal
+	  paths had 0 as a scheduler id to be destroyed when it was never started.
+
+	  chan_skinny.c:
+	  * Fix some scheduler id comparisons that excluded the valid 0 id.
+
+	  channel.c:
+	  * Fix channel initialization of the video stream scheduler id.
+
+	  pbx_dundi.c:
+	  * Fix channel initialization of the packet retransmission scheduler id.
+
+	  ASTERISK-25476
+
+	  Change-Id: I07a3449f728f671d326a22fcbd071f150ba2e8c8
+
+2015-12-01 07:55 +0000 [b24f2f4c2e]  Alexander Traud <pabstraud@compuserve.com>
+
+	* codec_resample: Increase buffer for Opus Codec.
+
+	  ASTERISK-25599 #close
+
+	  Change-Id: I1f88a88c59fb4e1e62bbdbb100c7152d48e73f10
+
+2015-11-30 11:13 +0000 [e5723d2776]  gtjoseph <george.joseph@fairview5.com>
+
+	* dns: Change lookup failures from LOG_ERROR to debug 1.
+
+	  dns.c and dns_system_resolver.c were spitting out errors for lookup
+	  failures for things like not finding a SRV record even though
+	  there was an A record.  Those have been changed to debug messages.
+	  Logging not finding ANY record is left to the higher level caller.
+
+	  Also, dns_system_resolver was using Windows line endings so I
+	  converted them to Unix style.  The actual log changes are on lines
+	  156 and 159.
+
+	  Change-Id: I65be16ea15304b96f9dcb4d289dbd3e2286fc094
+
+2015-11-25 10:42 +0000 [270f7be54f]  Alexander Traud <pabstraud@compuserve.com>
+
+	* Build System: Support include-what-you-use.
+
+	  ASTERISK-25591 #close
+
+	  Change-Id: I8d3efa0826142ece9cbed2fd0d46f3b607fee6ae
+
+2015-11-08 23:49 +0000 [f2a84b500d]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: Show reason of pause on CLI
+
+	  Add value of pause reason when is paused on CLI command "queue show"
+
+	  ASTERISK-25581 #close
+
+	  Report by: Rodrigo Ramírez Norambuena
+
+	  Change-Id: I887028a40cd97b350da9a3bb2719616b7fec9864
+
+2015-11-27 07:39 +0000 [7cb8f2f33e]  Niklas Larsson <niklas@tese.se>
+
+	* CHANGES: Fix a typo
+
+	  Change-Id: Iceb3d9bb78140c376174a7bee197dfcf8ef9cda7
+
+2015-11-25 15:26 +0000 [9014f1f4a5]  Kevin Harwell <kharwell@digium.com>
+
+	* fastagi: record file closed after sending result
+
+	  The fastagi record-file testsuite test sometimes fails reporting an empty
+	  recorded file. This was happening because Asterisk was sending the agi result
+	  notification prior to actually closing the file and the data, being buffered,
+	  had not been written to the file yet when the test attempts to check the file
+	  size.
+
+	  This patch makes it so the record file stream is closed prior to sending the
+	  agi result notification.
+
+	  ASTERISK-25593 #close
+
+	  Change-Id: I6b2b3be3ae37f7c7b18e672c419a89b3b8513cde
+
+2015-11-25 13:29 +0000 [03759c5587]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* main: Slight refactor of main. Improve color situation.
+
+	  Several issues are addressed here:
+	  - main() is large, and half of it is only used if we're not rasterisk;
+	    fixed by spliting up the daemon part into a separate function.
+	  - Call ast_term_init from rasterisk as well.
+	  - Remove duplicate code reading/writing asterisk history file.
+	  - Attempt to tackle background color issues and color changes that
+	    occur. Tested by starting asterisk -c until the colors stopped
+	    changing at odd locations.
+	  - Remove unused term_prep() and term_prompt() functions.
+
+	  ASTERISK-25585 #close
+
+	  Change-Id: Ib641a0964c59ef9fe6f59efa8ccb481a9580c52f
+
+2015-11-24 13:54 +0000 [91346b9fb7]  David M. Lee <dlee@respoke.io>
+
+	* Fixed some typos
+
+	  Fixes some minor typos in the CHANGES file, plus an embarrasing typo in
+	  the StatsD API.
+
+	  Change-Id: I9ca4858c64a4a07d2643b81baa64baebb27a4eb7
+
+2015-11-24 13:07 +0000 [fb45130476]  Corey Farrell <git@cfware.com>
+
+	* res_pjsip_notify: Fix CLI usage info
+
+	  The usage info for 'pjsip send notify' previously referenced the
+	  chan_sip configuration sip_notify.conf.  Fix this to reference
+	  the correct configuration pjsip_notify.conf.
+
+	  ASTERISK-25590 #close
+
+	  Change-Id: I3898271a8e8a8b1db201741e790ebe2c6bf5cdea
+
+2015-11-18 09:43 +0000 [ee9c114747]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_endpoint_stats: Add module to emit endpoint StatsD statistics
+
+	  This patch adds a module that emits StatsD statistics about Asterisk
+	  endpoints. This includes:
+	   * A GAUGE statistic for endpoint states, tracking how many endpoints are in
+	     a particular state.
+	   * A GAUGE statistic for each endpoint, counting the number of channels
+	     currently associated with an endpoint.
+
+	  ASTERISK-25572
+
+	  Change-Id: If7e1333c5aeda8d136850b30c2101c0ee1c97305
+2015-11-23 14:27 +0000 [9ca652f1b9]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_realtime.c: Fix crash from NULL sorcery object type.
+
+	  If the sorcery object type is not found a NULL is returned.
+	  Unfortunately, sorcery_realtime_filter_objectset() will crash after
+	  complaining about not finding the object type and saying to expect errors.
+
+	  * Use ao2_cleanup() instead of ao2_ref() to prevent the crash.
+
+	  ASTERISK-25165
+	  Reported by Corey Farrell
+
+	  Change-Id: Ic3b64453ea3058cb68d5c26d97d4fe7b8eea2e97
+
+2015-11-18 10:07 +0000 [75d90a9951]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip/pjsip_options: Add StatsD statistics for PJSIP contacts
+
+	  This patch adds the ability to send StatsD statistics related to the
+	  state of PJSIP contacts. This includes:
+	   * A GUAGE statistic measuring the count of contacts in a particular state.
+	     This measures how many contacts are reachable, unreachable, etc.
+	   * The RTT time for each contact, if those contacts are qualified. This
+	     provides StatsD engines useful time-based data about each contact.
+
+	  ASTERISK-25571
+
+	  Change-Id: Ib8378d73afedfc622be0643b87c542557e0b332c
+
+2015-11-13 10:34 +0000 [482f2fc5ff]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_outbound_registration: Add registration statistics for StatsD
+
+	  This patch adds outbound registration statistics for StatsD. This includes
+	  the following:
+	   * A GUAGE metric for the overall count of outbound registrations.
+	   * A GUAGE metric for each state an outbound registration can be in. As the
+	     outbound registrations change state, the overall count of how many
+	     outbound registrations are in the particular state is changed.
+
+	  These statistics are particularly useful for systems with a large number of
+	  SIP trunks, and where measuring the change in state of the trunks is useful
+	  for monitoring.
+
+	  ASTERISK-25571
+
+	  Change-Id: Iba6ff248f5d1c1e01acbb63e9f0da1901692eb37
+
+2015-11-18 10:05 +0000 [97d7b344de]  Matt Jordan <mjordan@digium.com>
+
+	* res_statsd: Add functions that support variable arguments
+
+	  Often, the metric names of statistics we are generating for StatsD have some
+	  dynamic component to them. This can be the name of a particular resource, or
+	  some internal status label in Asterisk. With the current set of functions,
+	  callers of the statsd API must first build the metric name themselves, then
+	  pass this to the API functions. This results in a large amount of boilerplate
+	  code and usage of either fixed length static buffers or dynamic memory
+	  allocation, neither of which is desireable.
+
+	  This patch adds two new functions to the StatsD API that support a printf
+	  style format specifier for constructing the metric name. A dynamic string,
+	  allocated in threadstorage, is used to build the metric name. This eases
+	  the burden on users of the StatsD API.
+
+	  Change-Id: If533c72d1afa26d807508ea48b4d8c7b32f414ea
+
+2015-11-20 21:08 +0000 [726ee873a6]  Matt Jordan <mjordan@digium.com>
+
+	* chan_pjsip: Handle T.38 faxes with direct media bridges
+
+	  When a channel is in a direct media bridge, a re-INVITE may arrive that forces
+	  Asterisk to re-negotiate the media to a T.38 fax. When this occurs, the bridge
+	  must change its technology to a simple bridge, and re-INVITE the media back
+	  to Asterisk.
+
+	  Generally, this logic mostly already exists in Asterisk. However, prior to this
+	  patch, there were a few bugs:
+	  (1) The T.38 framehook currently prevents a channel capable of T.38 faxes from
+	      ever entering into a direct media bridge. This applies even when the only
+	      media being passed over the channel is audio. This patch fixes this bug
+	      by having the framehook specify that it defers caring about any frame type.
+	      This allows the channels to enter into a direct media bridge, which will
+	      be broken when a re-INVITE is received.
+	  (2) When a re-INVITE is received, nothing instructed the bridging layer to
+	      re-inspect the allowed bridging technology. This now occurs when either
+	      a re-INVITE is received from a peer, or when a response is received from
+	      the far end (that is, when the T.38 state changes to either
+	      T38_PEER_REINVITE or T38_LOCAL_REINVITE).
+	  (3) chan_pjsip needs to do a small amount of work to prevent a direct media
+	      bridge from being chosen when a T.38 session is in progress. When a T.38
+	      session supplement has a t38 datastore - which is added when we detect
+	      we should start thinking about T.38 on a channel - we now refuse a native
+	      RTP bridge.
+	  (4) When a BYE request is received, we don't terminate the T.38 session. If
+	      the other side of a T.38 fax survives the hangup (due to the 'g' flag
+	      in Dial, for example), we don't currently re-INVITE the media on the
+	      other channel back to audio. This patch now has res_pjsip_t38 intercept
+	      BYE requests and inform the far side that the T.38 session is terminated.
+	      This naturally causes the correct re-INVITEs to be sent.
+
+	  ASTERISK-25582
+
+	  Change-Id: Iabd6aa578e633d16e6b9f342091264e4324a79eb
+
+2015-10-22 09:44 +0000 [9315a93757]  Matt Jordan <mjordan@digium.com>
+
+	* main/cli: Use proper string methods to check existence of context/exten/app
+
+	  Because the context, extension, and application are stored in stringfields,
+	  checking for them being NULL doesn't work so well. This patch uses the
+	  appropriate string library call, ast_strlen_zero, to see if there is a value
+	  in the context/exten/app values.
+
+	  Change-Id: Ie09623bfdf35f5a8d3b23dd596647fe3c97b9a23
+
+2015-11-20 21:07 +0000 [d2b141c79f]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_t38: Add debug statements
+
+	  This patch adds some debug statements to res_pjsip_t38. These statements help
+	  to determine which SDP negotiation callbacks are being executed, and, when
+	  a particular callback exits, why a callback may not have applied its logic
+	  to the local or remote SDP.
+
+	  Change-Id: I61b3fb9183b7ebbb5da8e9f48b59a5d9d7042d77
+
+2015-11-19 09:40 +0000 [1bca90fcbe]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_outbound_registration: Apply configuration on object type load
+
+	  When Asterisk is configured to use a dynamic sorcery backend (such as
+	  res_sorcery_astdb) with 'registration' objects, it will fail to create the
+	  internal state objects associated with the registration objects on module
+	  load. This is due to nothing actually querying for the specific objects
+	  and calling their sorcery apply handler during module load.
+
+	  This patch fixes that by calling get_registrations in the sorcery observer's
+	  object_type_loaded handler. Doing this causes the sorcery backends to be
+	  asked for the current state of all registration objects, which causes the
+	  apply handler to be called and the internal run-time state to be created.
+
+	  ASTERISK-25575 #close
+
+	  Change-Id: Ie9306e797098c6d4da7bcf4a5434a15891508b23
+
+2015-11-11 06:29 +0000 [8ccb1d2bed]  Alexander Traud <pabstraud@compuserve.com>
+
+	* translate: Provide translation modules the result of SDP negotiation.
+
+	  Previously, a trancoding module did not have access to the joint but cached
+	  format. Therefore, the module did not have access to the attributes negotiated
+	  via SDP (line fmtp). Now, a translation module receives the joint format.
+
+	  ASTERISK-25545 #close
+
+	  Change-Id: Id6878a989b50573298dab115d3371ea369e1a718
+
+2015-11-19 01:03 +0000 [92ea46ba94]  Alexander Traud <pabstraud@compuserve.com>
+
+	* res_format_attr_h264: Do not reset string buffer.
+
+	  When no parameter is present, Asterisk does not generate the line fmtp, as
+	  expected. However, because a buffer was reset, even rtpmap and fmtp of previous
+	  media codecs got removed. Now, Asterisk does not reset other codecs in case of
+	  no parameter for H.264.
+
+	  ASTERISK-25573 #close
+
+	  Change-Id: I93811331f4a28c45418a9e14ee46c0debd47a286
+
+2015-11-18 02:25 +0000 [8c14b91651]  Alec Davis <sivad.a@paradise.net.nz>
+
+	* app_bridgeaddchan: ability to barge into existing call
+
+	  To be able to barge into a call by dialling a prefix+extension that maps
+	  to the extensions device.
+
+	  Senario is that DECT headset users may be away from their desks and need
+	  to transfer the call, the goal is that from any phone they dial a prefix
+	  then their extension and are added to the bridge that they are in, from
+	  there they can drop the headset call, as it's also on the handset,
+	  and transfer the caller.
+
+	  The dialplan would look like, where prefix=73, extension = 8512;
+	  exten => _738512,1,BridgeAdd(SIP/cisco0001)
+
+	  ASTERISK-25551 #close
+	  Reported By: Alec Davis
+
+	  Change-Id: I8eb5096a02168dcc8d7aeea416ef36ba4ed10540
+
+2015-11-05 15:37 +0000 [05addf3d8f]  Tyler Cambron <tcambron@digium.com>
+
+	* StatsD: Add sample rate compatibility
+
+	  Implemented support for the StatsD sample rate parameter,
+	  which is a parameter for determining when to send computed
+	  statistics to a client.
+
+	  Valid sample rate values are:
+	  Less than or equal to 0.0 will never be sent.
+	  Between 0.0 and 1.0 will randomly be sent.
+	  Greater than or equal to 1.0 will always be sent.
+
+	  ASTERISK-25419
+	  Reported By: Ashley Sanders
+
+	  Change-Id: I11d315d0a5034fffeae1178e650aa8264485ed52
+
+2015-11-17 14:53 +0000 [3dbaf696e9]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Be tolerant of short registration timeouts.
+
+	  Change-Id: Ie16f5053ebde0dc6507845393709b4d6a3ea526d
+
+2015-11-17 14:53 +0000 [eaf898ac88]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Fix 423 response handling.
+
+	  Receiving a 423 Interval Too Brief response after authentication for an
+	  outbound registration attempt results in assuming that the registrar has
+	  rejected the registration permanently.  If there are no configured retries
+	  for fatal responses then the outbound registration is stopped for that
+	  endpoint.
+
+	  For registrations, PJSIP/PJPROJECT intercepts the handling of 423
+	  responses and does not include any authentication in the updated
+	  registration request.  When the updated request is challenged then the
+	  Asterisk code assumes that we were challenged again because the peer
+	  rejected the authentication we sent earlier.
+
+	  * Made registration challenges keep track of the CSeq number to determine
+	  if the received challenge response was for the request we thought we sent.
+	  If the response's CSeq number differs from the CSeq number we last sent
+	  with authentication then authenticate again because it is a challenge to a
+	  different request.
+
+	  Change-Id: I81b4bd36d1be095bab606e34b8b44e6302971b09
+
+2015-11-18 00:20 +0000 [4013f9d577]  Alec Davis <sivad.a@paradise.net.nz>
+
+	* app_queue: (try_calling): mutex 'qe->chan' freed more times than we've locked!
+
+	  commit aae45acbd (Mark Michelson 2015-04-15 10:38:02 -0500 6525)
+	  refer ASTERISK-24958
+
+	  above commit removed ast_channel_lock(qe->chan);
+	  but failed to remove corresponding ast_channel_unlock(qe->chan);
+
+	  ASTERISK-25561 #close
+	  Reported Alec Davis
+
+	  Change-Id: Ie05f4e2d08912606178bf1fded57cc022c7a2e1a
+
+2015-11-16 16:10 +0000 [6919daab61]  gtjoseph <george.joseph@fairview5.com>
+
+	* dns: Fix pointer increment in dns_parse_answer_ex
+
+	  When dns_parse_answer_ex was iterating over the answers it
+	  wasn't incrementing the answer pointer correctly after the first
+	  answer.  The result was that no answers after the first
+	  were being returned.  For results where multiple records should
+	  have been sorted by priority, weight, etc., there was nothing
+	  to sort so the only the first record was returned even if it
+	  wouldn't have been the correct record based on the sort.
+
+	  ASTERISK-25565 #close
+	  Reported-by: Daniel Tryba
+	  Tested-by George Joseph
+
+	  Change-Id: I8622604fefdcd3c11e2c5609a6382e53b1467b0b
+
+2015-11-13 14:03 +0000 [ed13732188]  Mark Michelson <mmichelson@digium.com>
+
+	* Confbridge: Add a user timeout option
+
+	  This option adds the ability to specify a timeout, in seconds, for a
+	  participant in a ConfBridge. When the user's timeout has been reached,
+	  the user is ejected from the conference with the CONFBRIDGE_RESULT
+	  channel variable set to "TIMEOUT".
+
+	  The rationale for this change is that there have been times where we
+	  have seen channels get "stuck" in ConfBridge because a network issue
+	  results in a SIP BYE not being received by Asterisk. While these
+	  channels can be hung up manually via CLI/AMI/ARI, adding some sort of
+	  automatic cleanup of the channels is a nice feature to have.
+
+	  ASTERISK-25549 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I2996b6c5e16a3dda27595f8352abad0bda9c2d98
+
+2015-11-16 13:56 +0000 [a83e426e91]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip: Fix off nominal crash with requests that fail and have a timer
+
+	  When a request is sent using pjsip_endpt_send_request and fails, a condition
+	  exists where the request wrapper, which is an AO2 object, may be de-ref'd
+	  more times than it should. This occurs when the request's callback is called,
+	  and, in the callback, the timer on the PJSIP heap is cancelled. When that
+	  occurs, the request wrapper's lifetime is decremented. When
+	  pjsip_endpt_send_request fails, we unilaterally decrement the lifetime of
+	  the request wrapper again, even though we've already cancelled the reference
+	  associated with the timer.
+
+	  This patch checks the return result of pj_timer_heap_cancel_if_active before
+	  removing the reference associated with the timer. We now only decrement it
+	  in this case if a timer is cancelled as a result of the function call.
+
+	  Change-Id: I21332343a1a019c1117076f9bf2df27be2850102
+
+2015-11-14 07:02 +0000 [a1fcf6f7b2]  Joshua Colp <jcolp@digium.com>
+
+	* hashtab: Add NULL check when destroying iterator.
+
+	  The hashtab API is pretty NULL tolerant which has resulted
+	  in remaining callers not doing much checks themselves.
+	  Unfortunately the function to destroy an iterator does not
+	  do a NULL check and will result in a crash if passed NULL.
+	  This change fixes that.
+
+	  ASTERISK-25552 #close
+
+	  Change-Id: Ic1bf8eec3639e5a440f1c941d3ae3893ac6ed619
+
+2015-11-13 14:32 +0000 [436023a322]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_rfc3326.c: Fix crash when channel goes away.
+
+	  If an authenticated incoming caller does not respond to our 200 OK INVITE
+	  response with an ACK then PJSIP will hangup the call.  Unfortunately,
+	  there is a chance that the session's channel will go away between one use
+	  of the channel pointer and another when building the BYE request because
+	  the BYE is being built by the monitor thread and not the call's serializer
+	  thread.
+
+	  * Added a check to ensure that the thread trying to add the Reason header
+	  is the call's serializer thread.  This ensures that the channel will not
+	  go away on us.
+
+	  Change-Id: I866388d2b97ea2032eaae3f3ab3f1ca6cbd2df89
+
+2015-11-13 14:19 +0000 [e8881e1770]  Mark Michelson <mmichelson@digium.com>
+
+	* Taskprocessors: Increase high-water mark
+
+	  In practical tests, we have seen certain taskprocessors, specifically
+	  Stasis subscription taskprocessors, cross the recently-added high-water
+	  mark and emit a warning. This high-water mark warning is only intended
+	  to be emitted when things have tanked on the system and things are
+	  heading south quickly. In the practical tests, the Stasis taskprocessors
+	  sometimes had a max depth of 180 tasks in them, and Asterisk wasn't in
+	  any danger at all.
+
+	  As such, this ups the high-water mark to 500 tasks instead. It also
+	  redefines the SIP threadpool request denial number to be a multiple of
+	  the taskprocessor high-water mark.
+
+	  Change-Id: Ic8d3e9497452fecd768ac427bb6f58aa616eebce
+
+2015-11-11 07:00 +0000 [fd23d423d8]  Alexander Traud <pabstraud@compuserve.com>
+
+	* format: Register format-attribute module with cached formats.
+
+	  In Asterisk 13, cached formats are created before their corresponding format-
+	  attribute module is registered. Cached formats are involved when a local
+	  extension is called. Therefore, ast_format_generate_sdp_fmtp did not work
+	  on local extensions. This change affects the Opus Codec, H.263 (Plus), H.264,
+	  and format-attribute modules provided externally.
+
+	  ASTERISK-25160 #close
+
+	  Change-Id: I1ea1f0483e5261e2a050112e4ebdfc22057d1354
+
+2015-11-12 11:17 +0000 [40b58a5d2b]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip distributor: Don't send 503 response to responses.
+
+	  When the SIP threadpool is backed up with tasks, we send 503 responses
+	  to ensure that we don't try to overload ourselves. The problem is that
+	  we were not insuring that we were not trying to send a 503 to an
+	  incoming SIP response.
+
+	  This change makes it so that we only send the 503 on incoming requests.
+
+	  Change-Id: Ie2b418d89c0e453cc6c2b5c7d543651c981e1404
+
+2015-11-11 17:11 +0000 [264c74aa22]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Deny requests when threadpool queue is backed up.
+
+	  We have observed situations where the SIP threadpool may become
+	  deadlocked. However, because incoming traffic is still arriving, the SIP
+	  threadpool's queue can continue to grow, eventually running the system
+	  out of memory.
+
+	  This change makes it so that incoming traffic gets rejected with a 503
+	  response if the queue is backed up too much.
+
+	  Change-Id: I4e736d48a2ba79fd1f8056c0dcd330e38e6a3816
+
+2015-11-12 06:24 +0000 [a159747660]  Joshua Colp <jcolp@digium.com>
+
+	* format_cap: Don't append the 'none' format when appending all.
+
+	  When appending all formats of a type all the codecs are iterated
+	  and added. This operation was incorrectly adding the ast_format_none
+	  format which is special in that it is supposed to be used when no
+	  format is present. It shouldn't be appended.
+
+	  ASTERISK-25535
+
+	  Change-Id: I7b00f3bdf4a5f3022e483d6ece602b1e8b12827c
+
+2015-11-11 04:16 +0000 [d982b99e71]  Steve Davies <steve@one47.co.uk>
+
+	* Further fixes to improper usage of scheduler
+
+	  When ASTERISK-25449 was closed, a number of scheduler issues mentioned in
+	  the comments were missed. These have since beed raised in ASTERISK-25476
+	  and elsewhere.
+
+	  This patch attempts to collect all of the scheduler issues discovered so
+	  far and address them sensibly.
+
+	  ASTERISK-25476 #close
+
+	  Change-Id: I87a77d581e2e0d91d33b4b2fbff80f64a566d05b
+
+2015-11-11 11:04 +0000 [2954354404]  Joshua Colp <jcolp@digium.com>
+
+	* threadpool: Handle worker thread transitioning to dead when going active.
+
+	  This change adds handling of dead worker threads when moving them
+	  to be active. When this happens the worker thread is removed from
+	  both the active and idle threads container. If no threads are able
+	  to be moved to active then the pool grows as configured.
+
+	  A unit test has also been added which thrashes the idle timeout
+	  and thread activation to exploit any race conditions between the
+	  two.
+
+	  ASTERISK-25546 #close
+
+	  Change-Id: I6c455f9a40de60d9e86458d447b548fb52ba1143
+
+2015-11-10 09:24 +0000 [525c7ab780]  Alexander Traud <pabstraud@compuserve.com>
+
+	* rtp_engine: Init a format-attribute module to its RFC defaults.
+
+	  Previously, format-attribute modules relied on an existing fmtp line in SDP
+	  negotiation. However, fmtp is optional for several formats like the Opus Codec.
+	  Now, the format-attribute module is called with an empty fmtp, which allows the
+	  module to initialise itself to RFC defaults. Furthermore now, Asterisk is able
+	  to differentiate between internally and externally created formats.
+
+	  ASTERISK-25537 #close
+
+	  Change-Id: I28f680cef7fdf51c0969ff8da71548edad72ec52
+
+2015-11-09 18:19 +0000 [be93036a4e]  Corey Farrell <git@cfware.com>
+
+	* Remove ABI compatibility stub functions.
+
+	  ABI compatibility stubs existed for ast_app_separate_args and ast_verbose,
+	  this is not needed in master.
+
+	  Change-Id: I07b4d2c16079da3c2c6efa55df4a74368e0bd453
+
+2015-11-10 07:51 +0000 [02a124eda5]  Corey Farrell <git@cfware.com>
+
+	* Remove execute permission from dns_system_resolver.c
+
+	  Change-Id: I3185735db42064bab00d3e073aed703385a00bf4
+
+2015-11-09 03:01 +0000 [cf79b62778]  Alexander Traud <pabstraud@compuserve.com>
+
+	* ast_format_cap_get_names: To display all formats, the buffer was increased.
+
+	  ASTERISK-25533 #close
+
+	  Change-Id: Ie1a9d1a6511b3f1a56b93d04475fbf8a4e40010a
+
+2015-11-09 07:04 +0000 [e85f0c81af]  Alexander Traud <pabstraud@compuserve.com>
+
+	* ast_format_cap: Avoid format creation on module load, use cache instead.
+
+	  Since Asterisk 13, formats are immutable and cached. However while loading a
+	  module like chan_sip, some formats were created instead using cached ones.
+
+	  ASTERISK-25535 #close
+
+	  Change-Id: I479cdc220d5617c840a98f3389b3bd91e91fbd9b
+
+2015-11-06 07:54 +0000 [7dd8f89a50]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* func_callerid: Document that CALLERID(pres) is available.
+
+	  CALLERPRES() says that it's deprecated in favor of CALLERID(num-pres)
+	  and CALLERID(name-pres).  But for channel driver that don't make a
+	  distinction between the two (e.g. SIP), it makes more sense to get/set
+	  both at once.  This change reveals the availability of CALLERID(pres),
+	  CONNECTEDLINE(pres), REDIRECTING(orig-pres), REDIRECTING(to-pres) and
+	  REDIRECTING(from-pres).
+
+	  ASTERISK-25373 #close
+
+	  Change-Id: I5614ae4ab7d3bbe9c791c1adf147e10de8698d7a
+2015-11-06 07:52 +0000 [39daf9f066]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* docs: Fix a few typo's in app docs (more then, resourse).
+
+	  Change-Id: Iba57efadf6c0b822e762c7a001bc89611d98afd7
+
+2015-11-06 14:19 +0000 [d82a4b098f]  gtjoseph <george.joseph@fairview5.com>
+
+	* dns: Use ntohl for ans->ttl in dns_parse_answer_ex
+
+	  dns_parse_answer_ex was not converting ans->ttl from network
+	  by order to host byte order which was causing certain ttls
+	  it to go negative. In turn this was causing answer edit checks
+	  to fail.
+
+	  ASTERISK-25528 #close
+	  Reported-by: Daniel Tryba
+	  Tested-by: George Joseph
+
+	  Change-Id: I31505132d6321c46d2f39fd06c20ee808a864037
+
+2015-11-06 07:36 +0000 [74e7333317]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* xmldoc: Improve xmldoc wrapping of 'core show ...' output.
+
+	  Previously, the wrapping did both lookahead and lookback, which,
+	  together with color escape sequences, caused some lines to be wrapped
+	  way earlier than other lines.  This led to inconsistent output.
+
+	  This simplifies the wrapping code and makes it more sane: if maxcolumns
+	  is hit, we simply jump back to the last space and wrap there.
+
+	  ASTERISK-25527 #close
+
+	  Change-Id: I56d01c6f9a812642b1b05535c98d4db48d17c957
+
+2015-11-06 06:57 +0000 [9d6e917349]  Sean Bright (license #5060)
+
+	* res_pjsip_sdp_rtp: Enable Opus to be negotiated via SIP/SDP.
+
+	  In SIP/SDP, Opus has two channels always (see RFC 7587 section 7). The actual
+	  amount of channels is negotiated in-band. Therefore now, the Opus codec and its
+	  attribute rtpmap are registered with two channels.
+
+	  ASTERISK-24779 #close
+	  Reported by: PowerPBX
+	  Tested by: Alexander Traud
+	  patches:
+	    asterisk-24779.patch submitted by Sean Bright (license #5060)
+
+	  Change-Id: Ic7ac13cafa1d3450b4fa4987350924b42cbb657b
+
+2015-11-03 16:19 +0000 [a2c2a8e1bb]  Jonathan Rose <jrose@digium.com>
+
+	* taskprocessor: Add high water mark warnings
+
+	  If a taskprocessor's queue grows large, this can indicate that there
+	  may be a problem with tasks not leaving the processor or else that
+	  the number of available task processors for a given type of task is
+	  too low. This patch makes it so that if a taskprocessor's task queue
+	  grows above 100 queued tasks that it will emit a warning message.
+	  Warning messages are emitted only once per task processor.
+
+	  ASTERISK-25518 #close
+	  Reported by: Jonathan Rose
+
+	  Change-Id: Ib1607c35d18c1d6a0575b3f0e3ff5d932fd6600c
+
+2015-11-02 20:11 +0000 [cd5ae02812]  Corey Farrell <git@cfware.com>
+
+	* Increase account code maximum length to 80.
+
+	  This increases the maximum length of account code's to match
+	  extensions.  This ensures it is always possible to set an
+	  accountcode to ${EXTEN} without truncation.
+
+	  ASTERISK-23904
+	  Reported by: Ben Merrills
+
+	  Change-Id: If122602304ce03362722eb213a3111b32da5eeb9
+
+2015-11-03 14:36 +0000 [379c041038]  Tyler Cambron <tcambron@digium.com>
+
+	* StatsD: Add res_statsd compatibility
+
+	  Added a new api to res_statsd.c to allow it to receive a
+	  character pointer for the value argument. This allows for a
+	  '+' and a '-' to easily be sent with the value.
+
+	  ASTERISK-25419
+	  Reported By: Ashley Sanders
+
+	  Change-Id: Id6bb53600943d27347d2bcae26c0bd5643567611
+
+2015-11-04 14:31 +0000 [9c293b5104]  Matt Jordan <mjordan@digium.com>
+
+	* main/dial: Protect access to the format_cap structure of the requesting channel
+
+	  When a dial attempt is made that involves a requesting channel, we previously
+	  were not:
+	  a) Protecting access to the native format capabilities structure on the
+	     requesting channel. That is inherently unsafe.
+	  b) Reference bumping the lifetime of the format capabilities structure.
+
+	  In both cases, something else could sneak in, blow away the format
+	  capabilities, and we'd be holding onto an invalid format_cap structure. When
+	  the newly created channel attempts to construct its format capabilities, things
+	  go poorly.
+
+	  This patch:
+	  a) Ensures that we get a reference to the native format capabilities while
+	     the requesting channel is locked
+	  b) Holds a reference to the native format capabilities during the creation
+	     of the new channel.
+
+	  ASTERISK-25522 #close
+
+	  Change-Id: I0bfb7ba8b9711f4158cbeaae96edf9626e88a54f
+
+2015-10-30 22:57 +0000 [b0bf189908]  Corey Farrell <git@cfware.com>
+
+	* Fix cli display of build options.
+
+	  A previous commit reduced the AST_BUILDOPTS compiler define to
+	  only include options that affected ABI.  This included some options
+	  that were previously displayed by cli "core show settings".  This
+	  change corrects the CLI display while still restricting buildopts.h
+	  to ABI effecting options only.
+
+	  ASTERISK-25434 #close
+	  Reported by: Rusty Newton
+
+	  Change-Id: Id07af6bedd1d7d325878023e403fbd9d3607e325
+
+2015-11-03 10:58 +0000 [63e02b45c6]  Matt Jordan <mjordan@digium.com>
+
+	* pjsip_configuration: On delete, remove the persistent version of an endpoint
+
+	  When an endpoint is deleted (such as through an API), the persistent endpoint
+	  currently continues to lurk around. While this isn't harmful from a memory
+	  consumption perspective - as all persistent endpoints are reclaimed on
+	  shutdown - it does cause Stasis endpoint related operations to continue
+	  to believe that the endpoint may or may not exist.
+
+	  This patch causes the persistent endpoint related to a PJSIP endpoint to be
+	  destroyed if the PJSIP endpoint is deleted.
+
+	  Change-Id: I85ac707b4d5e6aad882ac275b0c2e2154affa5bb
+2015-11-03 11:15 +0000 [d33a1682e3]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip/location: Destroy contact_status objects on contact deletion
+
+	  The contact_status Sorcery objects are currently not destroyed when a contact
+	  is deleted. This causes the contact's last known RTT/status to be 'sticky'
+	  when the contact itself may no longer exist. This patch causes the
+	  contact_status objects associated with both dynamic and static contacts to
+	  be destroyed if the AoR holding those contacts is also destroyed (or via
+	  other paths where a contact may be deleted.)
+
+	  Change-Id: I7feec8b9278cac3c5263a4c0483f4a0f3b62426e
+
+2015-11-03 08:15 +0000 [e26a06c1da]  Matt Jordan <mjordan@digium.com>
+
+	* main/stasis_endpoints: Fix ContactStatusChange JSON for roundtrip_usec field
+
+	  The JSON packing for the ContactStatusChange event forgot to include the
+	  roundtrip_usec field. As a result, the field never showed up in any event,
+	  even when the data was available. This patch corrects that error by properly
+	  packing the JSON blob with the data.
+
+	  Change-Id: I8df80da659a44010afbd48f645967518ff5daa17
+
+2015-11-02 20:24 +0000 [40574a2ea3]  Corey Farrell <git@cfware.com>
+
+	* chan_sip: Allow websockets to be disabled.
+
+	  This patch adds a new setting "websockets_enabled" to sip.conf.
+	  Setting this to false allows chan_sip to be used without causing
+	  conflicts with res_pjsip_transport_websocket.
+
+	  ASTERISK-24106 #close
+	  Reported by: Andrew Nagy
+
+	  Change-Id: I04fe8c4f2d57b2d7375e0e25826c91a72e93bea7
+
+2015-11-02 17:19 +0000 [f80a0ae49b]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Set threadpool max size default to 50.
+
+	  During a stress test of subscriptions, a huge blast of
+	  subscription-related traffic resulted in the threadpool expanding to a
+	  ridiculous number of threads. The balooning of threads resulted in an
+	  increase of memory, which led to a crash due to being out of memory.
+
+	  An easy fix for the particular test was to limit the size of the
+	  threadpool, thus reining in the amount of memory that would be used. It
+	  was decided that there really is no downside to having a non-infinite
+	  default value for the maximum size of the threadpool, so this change
+	  introduces 50 threads as the maximum threadpool size for the SIP
+	  threadpool.
+
+	  ASTERISK-25513 #close
+	  Reported by John Bigelow
+
+	  Change-Id: If0b9514f1d9b172540ce1a6e2f2ffa1f2b6119be
+
+2015-10-29 15:25 +0000 [c5093b21ad]  Tyler Cambron <tcambron@digium.com>
+
+	* StatsD: Send stuff to the StatsD server and test
+
+	  Added code to allow the StatsD dialplan application to
+	  send data to the server specified in statsd.conf.
+
+	  ASTERISK-25419
+
+	  Change-Id: I400db2f37c6ddf61515ff5a019646e36dcd0f922
+
+2015-11-02 06:57 +0000 [014e3d426b]  Matt Jordan <mjordan@digium.com>
+
+	* pjsip_options: Schedule/unschedule qualifies on AoR creation/destruction
+
+	  When an AoR is created or destroyed dynamically, the scheduled OPTIONS
+	  requests that qualify the contacts on the AoR are not necessarily started
+	  or destroyed, particularly for persistent contacts created for that AoR.
+	  This patch adds create/update/delete sorcery observers for an AoR, which
+	  schedule/unschedule the qualifies as expected.
+
+	  Change-Id: Ic287ed2e2952a7808ee068776fe966f9554bdf7d
+
+2015-10-30 13:22 +0000 [80cf4960ff]  Matt Jordan <mjordan@digium.com>
+
+	* Makefile: Add a rule 'basic-pbx' that installs the Basic PBX configs
+
+	  This patch adds a rule for installing the Super Awesome Company based 'Basic
+	  PBX' configuration files. As part of adding this rule, a bit of the content
+	  that makes up installing the configuration files under the 'samples' target
+	  was refactored into a make subroutine for usage by additional later config
+	  make targets.
+
+	  Change-Id: I6c2e27906f73e2919a2b691da0be20ae70302404
+2015-10-29 08:28 +0000 [b522a5e30f]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_pubsub: Fix assertion when UAS dialog creation fails.
+
+	  When compiled with assertions enabled one will occur when destroying
+	  the subscription tree when UAS dialog creation fails. This is because
+	  the code assumes that a dialog will always exist on a subscription
+	  tree when in reality during this specific scenario it won't.
+
+	  This change makes it so a dialog is not removed from the subscription
+	  tree if it is not present.
+
+	  ASTERISK-25505 #close
+
+	  Change-Id: Id5c182b055aacc5e66c80546c64804ce19218dee
+
+2015-10-08 11:50 +0000 [fdfd0fb488]  Tyler Cambron <tcambron@digium.com>
+
+	* StatsD: Add user input validation to the application
+
+	  Added code to accept user input and validate it before
+	  allowing it to be sent to the StatsD server.
+
+	  ASTERISK-25419
+	  Reported By: Ashley Sanders
+
+	  Change-Id: I55c7ce44326a68ad6c5c1514b9575ac50f25bbc3
+
+2015-10-26 11:42 +0000 [d343a25173]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Do not send all codecs on INVITE.
+
+	  Since version 13, Asterisk sent all allowed codecs as callee, even when the
+	  caller did not request/support them. In case of dynamic RTP payloads, this led
+	  to the same ID for different codecs, which is not allowed by SIP/SDP. Now, the
+	  intersection between the requested and the supported codecs is send again.
+
+	  ASTERISK-24543 #close
+
+	  Change-Id: Ie90cb8bf893b0895f8d505e77343de3ba152a287
+
+2015-10-19 07:11 +0000 [88f3dbaec9]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* install_prereq: Update repositories before install on Debian systems
+
+	  When to install packages the indexed local is more old of the
+	  version of software on the repository they have been upgraded by security
+	  update then get the package will give 404 not found.
+
+	  The patch prevent by update local index to repository for aptitude before
+	  install.
+
+	  ASTERISK-25495 #close
+
+	  Reporte by: Rodrigo Ramírez Norambuena
+
+	  Change-Id: I645959e553aac542805ced394cac2dca964051fa
+
+2015-10-24 13:08 +0000 [4328d320c2]  gtjoseph <george.joseph@fairview5.com>
+
+	* build: GCC 5.1.x catches some new const, array bounds and missing paren issues
+
+	  Fixed 1 issue in each of the affected files.
+
+	  ASTERISK-25494 #close
+	  Reported-by: George Joseph
+	  Tested-by: George Joseph
+
+	  Change-Id: I818f149cd66a93b062df421e1c73c7942f5a4a77
+
+2015-10-20 16:02 +0000 [a8aee0bbdb]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Add "like" processing to pjsip list and show commands
+
+	  Add the ability to filter output from pjsip list and show commands
+	  using the "like" predicate like chan_sip.
+
+	  For endpoints, aors, auths, registrations, identifyies and transports,
+	  the modification was a simple change of an ast_sorcery_retrieve_by_fields
+	  call to ast_sorcery_retrieve_by_regex.  For channels and contacts a
+	  little more work had to be done because neither of those objects are
+	  true sorcery objects.  That was just removing the non-matching object
+	  from the final container.  Of course, a little extra plumbing in the
+	  common pjsip_cli code was needed to parse the "like" and pass the regex
+	  to the get_container callbacks.
+
+	  Some of the get_container code in res_pjsip_endpoint_identifier was also
+	  refactored for simplicity.
+
+	  ASTERISK-25477 #close
+	  Reported by: Bryant Zimmerman
+	  Tested by: George Joseph
+
+	  Change-Id: I646d9326b778aac26bb3e2bcd7fa1346d24434f1
+
+2015-10-21 12:22 +0000 [691c0e0b31]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip_outbound_registration: registration stops due to fatal 4xx response
+
+	  During outbound registration it is possible to receive a fatal (any permanent/
+	  non-temporary 4xx, 5xx, 6xx) response from the registrar that is simply due
+	  to a problem with the registrar itself. Upon receiving the failure response
+	  Asterisk terminates outbound registration for the given endpoint.
+
+	  This patch adds an option, 'fatal_retry_interval', that when set continues
+	  outbound registration at the given interval up to 'max_retries' upon receiving
+	  a fatal response.
+
+	  ASTERISK-25485 #close
+
+	  Change-Id: Ibc2c7b47164ac89cc803433c0bbe7063bfa143a2
+
+2015-10-22 17:07 +0000 [5dd9e1938a]  Mark Michelson <mmichelson@digium.com>
+
+	* format_cap: Detect vector allocation failures.
+
+	  A crash was seen on a system that ran out of memory due to Asterisk not
+	  checking for vector allocation failures in format_cap.c. With this
+	  change, if either of the AST_VECTOR_INIT calls fail, we will return a
+	  value indicating failure.
+
+	  Change-Id: Ieb9c59f39dfde6d11797a92b45e0cf8ac5722bc8
+
+2015-10-02 15:32 +0000 [7f9823ff57]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Prevent sending NOTIFY on destroyed dialog.
+
+	  A certain situation can result in our attempting to send a NOTIFY on a
+	  destroyed dialog. Say we attempt to send a NOTIFY to a subscriber, but
+	  that subscriber has dropped off the network. We end up retransmitting
+	  that NOTIFY until the appropriate SIP timer says to destroy the NOTIFY
+	  transaction. When the pjsip evsub code is told that the transaction has
+	  been terminated, it responds in kind by alerting us that the
+	  subscription has been terminated, destroying the subscription, and then
+	  removing its reference to the dialog, thus destroying the dialog.
+
+	  The problem is that when we get told that the subscription is being
+	  terminated, we detect that we have not sent a terminating NOTIFY
+	  request, so we queue up such a NOTIFY to be sent out. By the time that
+	  queued NOTIFY gets sent, the dialog has been destroyed, so attempting to
+	  send that NOTIFY can result in a crash.
+
+	  The fix being introduced here is actually a reintroduction of something
+	  the pubsub code used to employ. We hold a reference to the dialog and
+	  wait to decrement our reference to the dialog until our subscription
+	  tree object is destroyed. This way, we can send messages on the dialog
+	  even if the PJSIP evsub code wants to terminate earlier than we would
+	  like.
+
+	  In doing this, some NULL checks for subscription tree dialogs have been
+	  removed since NULL dialogs are no longer actually possible.
+
+	  Change-Id: I013f43cddd9408bb2a31b77f5db87a7972bfe1e5
+
+2015-09-29 14:53 +0000 [e9e4bc9ece]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Ensure dialog lock balance.
+
+	  When sending a NOTIFY, we lock the dialog and then unlock the dialog
+	  when finished. A recent change made it so that the subscription tree's
+	  dialog pointer will be set NULL when sending the final NOTIFY request
+	  out. This means that when we attempt to unlock the dialog, we pass a
+	  NULL pointer to pjsip_dlg_dec_lock(). The result is that the dialog
+	  remains locked after we think we have unlocked it. When a response to
+	  the NOTIFY arrives, the monitor thread attempts to lock the dialog, but
+	  it cannot because we never released the dialog lock. This results in
+	  Asterisk being unable to process incoming SIP traffic any longer.
+
+	  The fix in this patch is to use a local pointer to save off the pointer
+	  value of the subscription tree's dialog when locking and unlocking the
+	  dialog. This way, if the subscription tree's dialog pointer is NULLed
+	  out, the local pointer will still have point to the proper place and the
+	  dialog lock will be unlocked as we expect.
+
+	  Change-Id: I7ddb3eaed7276cceb9a65daca701c3d5e728e63a
+
+2015-09-28 16:36 +0000 [b96267f7a3]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Prevent crashes on final NOTIFY.
+
+	  The SIP dialog is removed from the subscription tree when the final
+	  NOTIFY is sent. However, after the final NOTIFY is sent, the persistence
+	  update function still attempts to access the cseq from the dialog,
+	  resulting in a crash.
+
+	  This fix removes the subscription persistence at the same time that the
+	  dialog is removed from the subscription tree. This way, there is no
+	  attempt to update persistence when the subscription is being destroyed.
+
+	  Change-Id: Ibb46977a6cef9c51dc95f40f43446e3d11eed5bb
+
+2015-09-17 17:28 +0000 [386cd7b2b0]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Remove serializer when sending final NOTIFY.
+
+	  There have been crashes seen where a taskprocessor's listener is NULL
+	  unexpectedly.
+
+	  Looking at backtraces, the problem was specifically seen in PJSIP
+	  serializers.
+
+	  Subscriptions make the mistake of removing a serializer from a dialog
+	  during subscription tree destruction. Since subscription trees are
+	  reference-counted, guaranteeing the circumstances behind the destruction
+	  are not possible. This makes it so that the dialog serializer can be
+	  removed while not holding the dialog lock. This makes it possible for
+	  the distributor to get a pointer to the dialog serializer and have that
+	  serializer get freed out from under it.
+
+	  The fix for this is to remove the serializer from a subscription dialog
+	  when sending the final NOTIFY. This guarantees that the serializer is
+	  removed with the dialog lock held. By doing this, we guarantee that if
+	  the distributor gains access to the dialog's serializer, it will not be
+	  possible for the serializer to get freed by another thread.
+
+	  Change-Id: I21f5dac33529f65cec45679bdace60670800ff66
+
+2015-09-02 09:14 +0000 [0b63d011c9]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Fix crash on destruction of empty subscription tree.
+
+	  If an old persistent subscription is recreated but then immediately
+	  destroyed because it is out of date, the subscription tree will have no
+	  leaf subscriptions on it. This was resulting in a crash when attempting
+	  to destroy the subscription tree.
+
+	  A simple NULL check fixes this problem.
+
+	  Change-Id: I85570b9e2bcc7260a3fe0ad85904b2a9bf36d2ac
+
+2015-09-01 15:47 +0000 [ac0194dad6]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Solidify lifetime and ownership of objects.
+
+	  There have been crashes and general instability seen in the pubsub code,
+	  so this patch introduces three changes to increase the stability.
+
+	  First, the ownership model for subscriptions has been modified. Due to
+	  RLS, subscriptions are stored in memory as a tree structure. Prior to my
+	  patch, the PJSIP subscription was the owner of the subscription tree.
+	  When the PJSIP subscription told us that it was terminating, we started
+	  destroying the subscription tree along with all of the individual leaf
+	  subscriptions that belong to the tree. The problem with this model is
+	  that the two actors in play here, the PJSIP subscription and the
+	  individual leaf subscriptions, need to have joint ownership of the
+	  subscription tree. So now, the PJSIP subscription and the individual
+	  leaf subscriptions each have a reference to the subscription tree. This
+	  way, we will not actually free memory until no players are left that
+	  care. The PJSIP subscription is a bigger stakeholder, in that if the
+	  PJSIP subscription's reference to the subscription tree is removed, the
+	  subscription tree instructs the leaf subscriptions to shut down and drop
+	  their references to the subscription tree when possible. The individual
+	  leaf subscriptions, upon being told to shut down, can drop their stasis
+	  subscriptions or whatever they use to learn of new state, and then drop
+	  their reference to the subscription tree once they are ready to die.
+
+	  Second, the lifetime of a PJSIP subscription's reference to our
+	  subscription tree has been altered. As I learned from doing a deep dive,
+	  the PJSIP evsub code can tell Asterisk multiple times that the
+	  subscription has been terminated, and not all of these times
+	  are especially helpful. I have altered the message flow that we use for
+	  SIP subscriptions such that we will always drop the PJSIP subscription's
+	  reference to the subscription tree when we send the NOTIFY that
+	  terminates a SIP subscription. This also means that we will now queue
+	  NOTIFY requests to be sent after responding to incoming SUBSCRIBEs so
+	  that we can have predictable state changes from the PJSIP evsub code.
+
+	  Third, the synchronization of operations has been improved. PJSIP can
+	  call into our code from a serializer thread (e.g. upon receiving an
+	  incoming request) or from the monitor thread (e.g. when a subscription
+	  times out). Because of this, there is the possibility of competing
+	  threads stepping on each other. PJSIP attempts to do some
+	  synchronization on its own by always keeping the dialog lock held when
+	  it calls into us. However, since we end up pushing tasks into the
+	  serializer, the result was that serialized operations were not grabbing
+	  the dialog lock and could, as a result, step on something that was being
+	  attempted by a different thread. Now we ensure that serialized
+	  operations grab the dialog lock, then check for extenuating
+	  circumstances, then proceed with their operation if they can.
+
+	  Change-Id: Iff2990c40178dad9cc5f6a5c7f76932ec644b2e5
+
+2015-10-19 15:28 +0000 [1ce62b2545]  Richard Mudgett <rmudgett@digium.com>
+
+	* strings.c: Fix __ast_str_helper() to always return a terminated string.
+
+	  Users of functions which call __ast_str_helper() such as the ones listed
+	  below are likely to not check the return value for failure so ensuring
+	  that the string is always nil terminated is a good safety measure.
+
+	  ast_str_set_va()
+	  ast_str_append_va()
+	  ast_str_set()
+	  ast_str_append()
+
+	  Change-Id: I36ab2d14bb6015868b49329dda8639d70fbcae07
+
+2015-10-19 15:27 +0000 [a04d946eaa]  Richard Mudgett <rmudgett@digium.com>
+
+	* Add missing failure checks to ast_str_set_va() callers.
+
+	  Change-Id: I0c2cdcd53727bdc6634095c61294807255bd278f
+
+2015-10-21 11:44 +0000 [64c172deba]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Move URI validation to use time.
+
+	  In a realtime based system with a limited number of threadpool threads
+	  it is possible for a deadlock to occur. This happens when permanent
+	  endpoint state is updated, which will cause database queries to be done.
+	  These queries may result in URI validation being done which is done
+	  synchronously using a PJSIP thread. If all PJSIP threads are in use
+	  processing traffic they themselves may be blocked waiting to get the
+	  permanent endpoint container lock when identifying an endpoint.
+
+	  This change moves URI validation to occur at use time instead of
+	  configuration time. While this comes at a cost of not seeing a problem
+	  until you use it it does solve the underlying deadlock problem.
+
+	  ASTERISK-25486 #close
+
+	  Change-Id: I2d7d167af987d23b3e8199e4a68f3359eba4c76a
+
+2015-10-21 08:08 +0000 [f9cbac7321]  Alexander Traud <pabstraud@compuserve.com>
+
+	* format: Update the maximum packetization time for iLBC 30.
+
+	  In September 2006, the maximum packetization time (ptime) were set to such a
+	  low value, packetization was disabled for many codecs actually. This was fixed
+	  for many codecs but not for iLBC 30. This enables packetization for iLBC which
+	  can be enabled for example via allow=ilbc:60,gsm,alaw,ulaw in the file sip.conf.
+
+	  ASTERISK-7803
+
+	  Change-Id: I2ef90023d35efb7cb8fe96ed74f53f6846ffad12
+2015-10-21 09:51 +0000 [f3b2b3d1b3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Fix autoframing=yes.
+
+	  With Asterisk 13, the structures ast_format and ast_codec changed. Because of
+	  that, the paketization timing (framing) of the RTP channel moved away from the
+	  formats/codecs. In the course of that change, the ptime of the callee was not
+	  honored anymore, when the optional autoframing was enabled.
+
+	  ASTERISK-25484 #close
+
+	  Change-Id: Ic600ccaa125e705922f89c72212c698215d239b4
+
+2015-10-20 22:24 +0000 [b425850f8b]  Matt Jordan <mjordan@digium.com>
+
+	* rest-api-templates: Wikify error code response reasons
+
+	  Error response code descriptions may contain wiki markup that need to be
+	  escaped. Without this patch, Confluence will reject the document being sent
+	  and the responsible script will raise an exception.
+
+	  Change-Id: I21fcb66fee7f6332381f2b99b1b0195dff215ee5
+
+2015-10-20 12:06 +0000 [7be6194d6f]  Matt Jordan <mjordan@digium.com>
+
+	* funcs/func_holdintercept: Actually add the HOLD_INTERCEPT function
+
+	  When ab803ec342 was committed, it accidentally forgot to actually *add* the
+	  HOLD_INTERCEPT function. This highlights two interesting points:
+	  * Gerrit forces you to put the patch as it is going to into the repo up for
+	    review, which Review Board did not. Yay Gerrit.
+	  * No one apparently bothered to use this feature, or else they don't know about
+	    it. I'm going to go with the latter explanation.
+
+	  ASTERISK-24922
+
+	  Change-Id: Ida38278f259dd07c334a36f9b7d5475b5db72396
+
+2015-10-19 14:14 +0000 [77780790e0]  Jonh Wendell <jonh.wendell@gmail.com>
+
+	* main/cdr: Allow modules to modify CDR fields before dispatching them
+
+	  This patch adds the functions
+
+	  	ast_cdr_modifier_register()
+	  	ast_cdr_modifier_unregister()
+
+	  That work much like ast_cdr_register() and ast_cdr_unregister().
+
+	  Modules registered will be given a chance to modify (or to do whatever
+	  they want) CDR fields just before they are passed to registered engines.
+
+	  Thus, for instance, if a module change the "userfield" field of a CDR,
+	  the modified value will be passed to every registered CDR backend for
+	  logging.
+
+	  ASTERISK-25479 #close
+
+	  Change-Id: If11d8fd19ef89b1a66ecacf1201e10fcf86ccd56
+2015-10-19 19:59 +0000 [b9bd249a85]  Matt Jordan <mjordan@digium.com>
+
+	* contrib/scripts/autosupport: Update for Asterisk 13
+
+	  This patch adds some minor tweaks for autosupport to update it for Asterisk 13.
+	  This includes:
+	  * Finally removing most references to Zaptel
+	  * Adding support for some additional 'core' commands, and fixing nomenclature
+	    that generally hasn't been used for some time
+	  * Adding some PJSIP/SIP commands to gather endpoints/peers and active channels
+
+	  Change-Id: Ic997b418cbd9313588b6608e50f47b0ce6f4f1f1
+	  (cherry picked from commit 9fc9777fa34753fb38991d42d8dbed516e907ca2)
+
+2015-10-18 18:22 +0000 [92fa8d1e0e]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* app_queue: Added reason pause of member
+
+	  In app_queue added value Paused Reason on QueueMemberStatus when a member
+	  on queue is paused and the reason was set.
+
+	  ASTERISK-25480 #close
+	  Reporte by: Rodrigo Ramírez Norambuena
+
+	  Change-Id: Ia5db503482f50764c15e2020196c785f59d4a68e
+
+2015-10-16 22:01 +0000 [b19860c03a]  Corey Farrell <git@cfware.com>
+
+	* res_ari_events: Fix memory leak in mustache template.
+
+	  ASTERISK-25308 fixed a memory leak in res_ari_events.c, but
+	  this file is regenerated by a template and the template was
+	  not fixed.
+
+	  Change-Id: Ied4c6deae89d21f87f9cf99676b1d055aa83b38b
+
+2015-10-14 14:15 +0000 [d799bcf361]  mdu113 <mulitskiy@acedsl.com>
+
+	* res_config_pgsql.c: Fix deadlock loading realtime configuration.
+
+	  On v13, loading several thousand PJSIP endpoints on Asterisk start causes
+	  a deadlock most of the time.
+
+	  Thanks to mdu113 for discovering that there was a call to pgsql_exec() not
+	  protected by the pgsql_lock reentrancy lock.
+
+	  {quote}
+	  I believe a code path exists that attempts to use pgsql connection without
+	  locking pgsql_lock.  I believe what happens during that deadlock that I
+	  see is two concurrent threads are both attempting to send query to pgsql,
+	  one of the thread is using a code path without locking pgsql_lock.  If
+	  they managed to send queries at the same time, it seems postgres ignores
+	  one of the queries and replies only to the one of them.  If it happens so
+	  that the thread holding the lock didn't receive the reply it will wait for
+	  it (and hold the lock) forever (or at least for very long time), thus
+	  completely blocking all access to db.
+	  {quote}
+
+	  * Added missing reentrancy locking around pgsql_exec() in find_table().
+
+	  * Moved unlock of pgsql_lock in unload_module() to avoid locking inversion
+	  between the psql_tables list lock and the pgsql_lock.
+
+	  ASTERISK-25455 #close
+	  Reported by:  mdu113
+	  Patches:
+	        res_config_pgsql.c-connlock2.diff (license #5543) patch uploaded by mdu113
+
+	  Change-Id: Id9e7cdf8a3b65ff19964b0cf942ace567938c4e2
+
+2015-10-13 14:13 +0000 [13229037d1]  Olle Johansson (License 5267)
+
+	* channels/chan_sip: Set cause code to 44 on RTP timeout
+
+	  To quote Olle:
+
+	  "When issuing a hangup due to RTP timeouts the cause code is not set. I have
+	  selected 44 based on Cisco's implementation..."
+
+	  ASTERISK-25135 #close
+	  Reported by: Olle Johansson
+	  patches:
+	    rtp-timeout-cause-1.8.diff uploaded by Olle Johansson (License 5267)
+
+	  Change-Id: Ia62100c55077d77901caee0bcae299f8dc7375fc
+
+2015-10-12 11:21 +0000 [984f100dab]  Richard Mudgett <rmudgett@digium.com>
+
+	* config.c: Fix off-nominal memory leak.
+
+	  Change-Id: I06e346e9a5c63cc5071e7eda537310c4b43bffe0
+
+2015-10-12 11:20 +0000 [9951255775]  Richard Mudgett <rmudgett@digium.com>
+
+	* config.c: Fix potential memory corruption after [section](+).
+
+	  The memory corruption could happen if the [section](+) is the last section
+	  in the file with trailing comments.  In this case process_text_line() has
+	  left *last_cat is set to newcat and newcat is destroyed.
+
+	  Change-Id: I0d1d999f553986f591becd000e7cc6ddfb978d93
+
+2015-10-12 11:21 +0000 [c1ed11ee31]  Richard Mudgett <rmudgett@digium.com>
+
+	* config.c: Fix #include after [section](+).
+
+	  An #include right after a [section](+) would associate any variable
+	  assignments before a new section in the #include with the wrong section.
+
+	  * Fix section association by setting the current section to the appended
+	  section.
+
+	  * Fix '+' and '!' section flag interaction corner case depending upon
+	  which flag came first.  If the '!' came first then it would be ignored.
+	  If the '!' came after then it would affect the appended section.  The '!'
+	  will now no longer be ignored.
+
+	  ASTERISK-25461 #close
+	  Reported by: Sean Pimental
+
+	  Change-Id: Ic9d3191c8758048e2cbce6432f854b32531731c3
+
+2015-10-10 15:20 +0000 [a12eb89ea4]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* Build: Add menuselect options for using compiler sanitizers
+
+	  This patch adds menuselect options for building Asterisk with
+	  various sanitizers provided by gcc and clang.
+
+	  When one of *SANITIZER flags is set in menuselect, the appropriate
+	  option is added to CFLAGS ad LDFLAGS for the build.
+
+	  Information on sanitizers in the project wiki:
+	  https://github.com/google/sanitizers/wiki
+
+	  GCC Manual:
+	  https://gcc.gnu.org/onlinedocs/gcc/Debugging-Options.html
+
+	  Clang Compiler User's Manual:
+	  http://clang.llvm.org/docs/UsersManual.html#controlling-code-generation
+
+	  ASTERISK-24718 #close
+	  Reported by: Badalian Vyacheslav
+
+	  Change-Id: Iafa51b792b7bcb20e848b99d16cf362d08590fa0
+
+2015-10-08 16:43 +0000 [ca030845ff]  Richard Mudgett <rmudgett@digium.com>
+
+	* configure: Fix check for libunbound to require v1.5.0 as minimum.
+
+	  Versions of libunbound before v1.4.21 do not compile with Asterisk.
+	  However, since v1.4.21 has a configure script bug that fails to detect the
+	  ldns library (which is fixed in v1.4.22) and v1.4.22 is not an easily
+	  detectable version we will require v1.5.0 as a minimum version of the
+	  library to work with Asterisk.
+
+	  ASTERISK-25108 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Ieb228bfb01467573fc121c7356a9dde27128894d
+
+2015-10-08 11:50 +0000 [2fe9f09705]  Tyler Cambron <tcambron@digium.com>
+
+	* StatsD: Write skeleton Asterisk application
+
+	  Wrote the skeleton framework for the Asterisk StatsD dialplan
+	  application. This includes a load function, unload function, a
+	  callback for execution, and XML documentation.
+
+	  ASTERISK-25419
+	  Reported By: Ashley Sanders
+
+	  Change-Id: I9597730e134c6e82c8a55ef4d5334b62dd473363
+
+2015-10-06 18:01 +0000 [34d7fa6c4a]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Fix deadlock when sending out-of-dialog requests.
+
+	  The struct send_request_wrapper has a pjsip lock associated with it that
+	  is created non-recursive.  There is a code path for the struct
+	  send_request_wrapper lock that will attempt to lock it recursively.  The
+	  reporter's deadlock showed that the thread calling endpt_send_request()
+	  deadlocked itself right after the wrapper object got created.
+
+	  Out-of-dialog requests such as MESSAGE, qualify OPTIONS, and unsolicited
+	  MWI NOTIFY messages can hit this deadlock.
+
+	  * Replaced the struct send_request_wrapper pjsip lock with the mutex lock
+	  that can come with an ao2 object since all of Asterisk's mutexes are
+	  recursive.  Benefits include removal of code maintaining the pjsip
+	  non-recursive lock since ao2 objects already know how to maintain their
+	  own lock and the lock will show up in the CLI "core show locks" output.
+
+	  ASTERISK-25435 #close
+	  Reported by: Dmitriy Serov
+
+	  Change-Id: I458e131dd1b9816f9e963f796c54136e9e84322d
+
+2015-10-06 11:05 +0000 [cc131832aa]  Stefan Engström <stefanen@kth.se>
+
+	* res/res_rtp_asterisk.c: Fix incorrect assignment of frame->subclass.frame_ending
+
+	  In ast_rtp_read, the value of the variable 'mark' which we try to assign to a
+	  frame->subclass.frame_ending may be 0, 1 or (1<<23), but we should translate
+	  it to 0 or 1.
+
+	  ASTERISK-25451 #close
+	  Change-Id: I53bdf5c026041730184a6a809009c028549ce626
+
+2015-10-07 01:24 +0000 [c944263e36]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* func_presencestate: Return "not_set" when no data is set in AstDB
+
+	  Return AST_PRESENCE_NOT_SET when CustomPresence AstDB key does not
+	  exist, i.e. when a new CustomPresence is added in the dialplan.
+
+	  ASTERISK-25400 #close
+	  Reported by: Andrew Nagy
+
+	  Change-Id: I6fb17b16591b5a55fbffe96f3994ec26b1b1723a
+
+2015-10-06 20:43 +0000 [4bf395e81e]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_rtp_asterisk: Fix assignment after ao2 decrement
+
+	  When we decide we will no longer schedule an RTCP write, we remove the
+	  reference to the RTP instance, then assign -1 to the stored scheduler ID
+	  in case something else comes along and wants to see if anything is scheduled.
+
+	  That scheduler ID is on the RTP instance. After 60a9172d7ef2 was merged to
+	  fix the regression introduced by 3cf0f29310, this improper assignment on a
+	  potentially destroyed object started getting tripped on the build agents.
+
+	  Frankly, this should have been crashing a lot more often earlier. I can only
+	  assume that the timing was changed just enough by both changes to start
+	  actually hitting this problem.
+
+	  As it is, simply moving the assignment prior to the ao2 deference is sufficient
+	  to keep the RTP instance from being referenced when it is very, truly,
+	  aboslutely dead.
+
+	  (Note that it is still good practice to assign -1 to the scheduler ID when we
+	  know we won't be scheduling it again, as the ao2 deref *may* not always destroy
+	  the ao2 object.)
+
+	  ASTERISK-25449
+
+	  Change-Id: Ie6d3cb4adc7b1a6c078b1c38c19fc84cf787cda7
+
+2015-10-06 12:40 +0000 [3ec9cf7d6a]  Florian Sauerteig <ffs@ccn.net>
+
+	* chan_sip: Fix port parsing for IPv6 addresses in SIP Via headers.
+
+	  If a Via header containes an IPv6 address and a port number is ommitted,
+	  as it is the standard port, we now leave the port empty and to not set it
+	  to the value after the first colon of the IPv6 address.
+
+	  ASTERISK-25443 #close
+
+	  Change-Id: Ie3c2f05471cd006bf04ed15598589c09577b1e70
+
+2015-10-05 16:53 +0000 [8fe9350b68]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_pjsip: Fix crash on reINVITE before initial INVITE completes.
+
+	  Apparently some endpoints attempt to send a reINVITE before completing the
+	  initial INVITE transaction.  In this case PJSIP responds appropriately to
+	  the reINVITE with a 491 INVITE request pending.  Unfortunately chan_pjsip
+	  is using the initial INVITE transaction state to determine if an INVITE is
+	  the initial INVITE or a reINVITE.  Since the initial INVITE transaction
+	  has not been confirmed yet chan_pjsip thinks the reINVITE is an initial
+	  INVITE and starts another PBX thread on the channel.  The extra PBX thread
+	  ensures that hilarity ensues.
+
+	  * Fix checks for a reINVITE on incoming requests to look for the presence
+	  of a to-tag instead of the initial INVITE transaction state.
+
+	  * Made caller_id_incoming_request() determine what to do if there is a
+	  channel on the session or not.  After a channel is created it is too late
+	  to just store the new party id on the session because the session's party
+	  id has already been copied to the channel's caller id.
+
+	  ASTERISK-25404 #close
+	  Reported by: Chet Stevens
+
+	  Change-Id: Ie78201c304a2b13226f3a4ce59908beecc2c68be
+
+2015-10-05 21:34 +0000 [8cb614fe20]  Matt Jordan <mjordan@digium.com>
+
+	* Fix improper usage of scheduler exposed by 5c713fdf18f
+
+	  When 5c713fdf18f was merged, it allowed for scheduled items to have an ID of
+	  '0' returned. While this was valid per the documentation for the API, it was
+	  apparently never returned previously. As a result, several users of the
+	  scheduler API viewed the result as being invalid, causing them to reschedule
+	  already scheduled items or otherwise fail in interesting ways.
+
+	  This patch corrects the users such that they view '0' as valid, and a returned
+	  ID of -1 as being invalid.
+
+	  Note that the failing HEP RTCP tests now pass with this patch. These tests
+	  failed due to a duplicate scheduling of the RTCP transmissions.
+
+	  ASTERISK-25449 #close
+
+	  Change-Id: I019a9aa8b6997584f66876331675981ac9e07e39
+2015-08-26 16:58 +0000 [c6b0d60264]  Debian Amtelco <dan@amtelco.com>
+
+	* chan_pjsip: Add Referred-By header to the PJSIP REFER packet.
+
+	  Some systems require the REFER packet to include a Referred-By header.
+	  If the channel variable SIPREFERREDBYHDR is set, it passes that value as the
+	  Referred-By header value.  Otherwise, it adds the current dialog’s local info.
+
+	  Reported by: Dan Cropp
+	  Tested by: Dan Cropp
+
+	  Change-Id: I3d17912ce548667edf53cb549e88a25475eda245
+
+2015-10-03 06:27 +0000 [89dec7675d]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* manager: Fix GetConfigJSON returning invalid JSON
+
+	  When GetConfigJSON was introduced back in 1.6, it returned each
+	  section as an array of strings: ["key=value", "key2=value2"].
+	  Afterwards, it was changed a few times and became
+	  ["key": "value", "key2": "value2"], which is not a correct JSON.
+	  This patch fixes that by constructing a JSON object {} instead of
+	  an array [].
+
+	  Also, the keys "istemplate" and "tempates" that are used to
+	  indicate templates and their inherited categories are now wrapped in
+	  quotes.
+
+	  ASTERISK-25391 #close
+	  Reported by: Bojan Nemčić
+
+	  Change-Id: Ibbe93c6a227dff14d4a54b0d152341857bcf6ad8
+
+2015-09-30 17:28 +0000 [1b80dbeb60]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Fix deadlock with scheduler.
+
+	  A deadlock can happen when a sorcery object is being expired from the
+	  memory cache when at the same time another object is being placed into the
+	  memory cache.  There are a couple other variations on this theme that
+	  could cause the deadlock.  Basically if an object is being expired from
+	  the sorcery memory cache at the same time as another thread tries to
+	  update the next object expiration timer the deadlock can happen.
+
+	  * Add a deadlock avoidance loop in expire_objects_from_cache() to check if
+	  someone is trying to remove the scheduler callback from the scheduler.
+
+	  ASTERISK-25441 #close
+
+	  Change-Id: Iec7b0bdb81a72b39477727b1535b2539ad0cf4dc
+
+2015-10-01 14:30 +0000 [9c1ca287a4]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Replace inline code with function.
+
+	  Make sorcery_memory_cache_close() call remove_all_from_cache() instead of
+	  partially inlining it.
+
+	  ASTERISK-25441
+
+	  Change-Id: I1aa6cb425b1a4307096f3f914d17af8ec179a74c
+
+2015-10-01 14:27 +0000 [6554a3b25e]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Shutdown in a less crash potential order.
+
+	  Basically you should shutdown in the opposite order of how you setup since
+	  later setup pieces likely depend on earlier setup pieces.  e.g.,
+	  Registering your external API with the rest of the system should be the
+	  last thing setup and the first thing unregistered during shutdown.
+
+	  Change-Id: I5715765b723100c8d3c2642e9e72cc7ad5ad115e
+
+2015-09-30 17:27 +0000 [359394cc29]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Misc tweaks.
+
+	  Change-Id: I8cd32dffbb4f33bb0c39518d6e4c991e73573160
+
+2015-09-30 17:27 +0000 [7942d1c2ff]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_sorcery_memory_cache.c: Made use OBJ_SEARCH_MASK.
+
+	  Change-Id: Ibca6574dc3c213b29cc93486e01ccd51f5caa46c
+
+2015-09-30 13:42 +0000 [9f229d6a49]  Joshua Colp <jcolp@digium.com>
+
+	* res_rtp_asterisk: Move "Set role" warning to be debug.
+
+	  In practice the set_role API callback can be invoked even
+	  when no ICE is present on an RTP instance. This can occur
+	  if ICE has not been enabled on it.
+
+	  ASTERISK-25438 #close
+
+	  Change-Id: I0e17e4316f0f0d7f095c78c3d4fd73a913b6ba69
+
+2015-09-28 15:31 +0000 [9bc7386b7c]  Richard Mudgett <rmudgett@digium.com>
+
+	* sched.c: Add warning about negative time interval request.
+
+	  Change-Id: Ib91435fb45b7f5f7c0fc83d0eec20b88098707bc
+
+2015-09-25 18:37 +0000 [12feec0bf7]  Richard Mudgett <rmudgett@digium.com>
+
+	* res/ari/config.c: Fix user sort compare function.
+
+	  Made use the ao2 sort compare template function and OBJ_SEARCH_xxx
+	  identifiers.
+
+	  Change-Id: Ic53005dc5aafa7a36c72300dd89b75fb63c92f4c
+
+2015-09-25 17:26 +0000 [3f4fa245e5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res/ari/config.c: Optimize conf_alloc() object init.
+
+	  * Now conf_alloc() has more off nominal error checking.
+
+	  * Eliminated RAII_VAR() use in conf_alloc().
+
+	  * Eliminated a dubius shortcut when destroying cfg->general in
+	  conf_destructor() that would cause a crash if cfg->general failed to get
+	  allocated.
+
+	  * Add some ACO registration section comments.
+
+	  Change-Id: Ia40c2b1b2d0777d641605118ae019c5a73865e1a
+
+2015-09-25 16:48 +0000 [aa00df62ee]  Richard Mudgett <rmudgett@digium.com>
+
+	* res/ari/config.c: Fix conf_alloc() object init.
+
+	  Need to finish initializing the string fields in the ao2 object before
+	  putting any default strings into them.
+
+	  ASTERISK-25383 #close
+	  Reported by:  yaron nahum
+
+	  Change-Id: I9f7f3a03f0c4991a01593abf8697b9a587c0ea84
+
+2015-09-21 07:26 +0000 [2d7a4a3357]  Matt Jordan <mjordan@digium.com>
+
+	* main/logger: Add log formatters and JSON structured logs
+
+	  When Asterisk is part of a larger distributed system, log files are often
+	  gathered using tools (such as logstash) that prefer to consume information
+	  and have it rendered using other tools (such as Kibana) that prefer a
+	  structured format, e.g., JSON. This patch adds support for JSON formatted
+	  logs by adding support for an optional log format specifier in Asterisk's
+	  logging subsystem. By adding a format specifier of '[json]':
+
+	  full => [json]debug,verbose,notice,warning,error
+
+	  Log messages will be output to the 'full' channel in the following
+	  format:
+
+	  {
+	    "hostname": Hostname or name specified in asterisk.conf
+	    "timestamp": Date/Time
+	    "identifiers": {
+	      "lwp": Thread ID,
+	      "callid": Call Identifier
+	    }
+	    "logmsg": {
+	      "location": {
+	        "filename": Name of the file that generated the log statement
+	        "function": Function that generated the log statement
+	        "line": Line number that called the logging function
+	      }
+	      "level": Log level, e.g., DEBUG, VERBOSE, etc.
+	      "message": Actual text of the log message
+	    }
+	  }
+
+	  ASTERISK-25425 #close
+
+	  Change-Id: I8649bfedf3fb7bf3138008cc11565553209cc238
+
+2015-09-27 20:45 +0000 [9402f80726]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_stasis: Fix accidental subscription to 'all' bridge topic
+
+	  When b99a7052621700a1aa641a1c24308f5873275fc8 was merged, subscribing to a
+	  NULL bridge will now cause app_subscribe_bridge to implicitly subscribe to
+	  all bridges. Unfortunately, the res_stasis control loop did not check that
+	  a bridge changing on a channel's control object was actually also non-NULL.
+	  As a result, app_subscribe_bridge will be called with a NULL bridge when a
+	  channel leaves a bridge. This causes a new subscription to be made to the
+	  bridge. If an application has also subscribed to the bridge, the application
+	  will now have two subscriptions:
+	  (1) The explicit one created by the app
+	  (2) The implicit one accidentally created by the control structure
+
+	  As a result, the 'BridgeDestroyed' event can be sent multiple times. This
+	  patch corrects the control loop such that it only subscribes an application
+	  to a new bridge if the bridge pointer is non-NULL.
+
+	  ASTERISK-24870
+
+	  Change-Id: I3510e55f6bc36517c10597ead857b964463c9f4f
+
+2015-09-04 13:51 +0000 [d6472d96b3]  Scott Griepentrog <scott@griepentrog.com>
+
+	* Scripts: check file versions of Asterisk and dependencies
+
+	  To help in diagnosing mismatched modules and libraries, this
+	  script scans for version, repository, and source information
+	  and reports what is found.
+
+	  ASTERISK-25376 #close
+	  Reported by: Ashley Sanders
+
+	  Change-Id: Ib0642d0fb96712476f59760d6d137a24633fe2d6
+
+2015-09-24 14:56 +0000 [7c7a7ddd27]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Force COLP update if outgoing channel name changed.
+
+	  * When a call is answered and the outgoing channel name has changed then
+	  force a connected line update because the channel is no longer the same.
+	  The channel was masqueraded into by another channel.  This is usually
+	  because of a call pickup.
+
+	  Note: Forwarded calls are handled in a controlled manner so the original
+	  channel name is replaced with the forwarded channel.
+
+	  ASTERISK-25423 #close
+	  Reported by: John Hardin
+
+	  Change-Id: Ie275ea9e99c092ad369db23e0feb08c44498c172
+
+2015-09-24 14:20 +0000 [145608bd81]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Factor out a connected line update routine.
+
+	  Replace inlined code with update_connected_line_from_peer().
+
+	  ASTERISK-25423
+	  Reported by: John Hardin
+
+	  Change-Id: I33bbd033596fcb0208d41d8970369b4e87b806f3
+
+2015-09-24 13:27 +0000 [1d394774b2]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_dial.c: Make 'A' option pass COLP updates.
+
+	  While the 'A' option is playing the announcement file allow the caller and
+	  peer to exchange COLP update frames.
+
+	  ASTERISK-25423
+	  Reported by: John Hardin
+
+	  Change-Id: Iac6cf89b56d26452c6bb88e9363622bbf23895f9
+
+2015-09-24 12:59 +0000 [680b76eb25]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_dial.c: Force COLP update if outgoing channel name changed.
+
+	  * When a call is answered and the outgoing channel name has changed then
+	  force a connected line update because the channel is no longer the same.
+	  The channel was masqueraded into by another channel.  This is usually
+	  because of a call pickup.
+
+	  Note: Forwarded calls are handled in a controlled manner so the original
+	  channel name is replaced with the forwarded channel.
+
+	  ASTERISK-25423
+	  Reported by: John Hardin
+
+	  Change-Id: I2e01f7a698fbbc8c26344a59c2be40c6cd98b00c
+
+2015-09-24 12:37 +0000 [fdf0bcb04a]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_dial.c: Factor out a connected line update routine.
+
+	  Replace inlined code with update_connected_line_from_peer().
+
+	  ASTERISK-25423
+	  Reported by: John Hardin
+
+	  Change-Id: Ia14f18def417645cd7fb453e1bdac682630a5091
+
+2015-09-23 17:41 +0000 [c285879845]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_dial.c: Remove some no-op code.
+
+	  Change-Id: Ice1884a94315d3cb7e3bbd47a9fba76a27276c54
+
+2015-09-23 14:02 +0000 [3eefa07a39]  Mark Michelson <mmichelson@digium.com>
+
+	* logger: Prevent duplicate dynamic channels from being added.
+
+	  There was a problem observed where the "logger add channel" CLI command
+	  would allow for a channel with the same name to be added multiple times.
+	  This would result in each message being written out to the same file
+	  multiple times.
+
+	  The problem was due to the difference in how logger channel filenames
+	  are stored versus the format they are allowed to be presented when they
+	  are added. For instance, if adding the logger channel "foo" through the
+	  CLI, the result would be a logger channel with the file name
+	  /var/log/asterisk/foo being stored. So when trying to add another "foo"
+	  channel, "foo" would not match "/var/log/asterisk/foo" so we'd happily
+	  add the duplicate channel.
+
+	  The fix presented here is to introduce two new methods in the logger
+	  code:
+	   * make_filename(): given a logger channel name, this creates the
+	     filename for that logger channel.
+	   * find_logchannel(): given a logger channel name, this calls
+	     make_filename() and then traverses the list of logchannels in order
+	     to find a match.
+
+	  This change has made use of make_filename() and find_logchannel()
+	  throughout to more consistently behave.
+
+	  ASTERISK-25305 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I892d52954d6007d8bc453c3cbdd9235dec9c4a36
+
+2015-09-24 14:49 +0000 [f42084be09]  Mark Michelson <mmichelson@digium.com>
+
+	* Do not swallow frames on channels leaving bridges.
+
+	  When leaving a bridge, indications on a channel could be swallowed by
+	  the internal indication logic because it appears that the channel is on
+	  its way to be hung up anyway. One such situation where this is
+	  detrimental is when channels on hold are redirected out of a bridge. The
+	  AST_CONTROL_UNHOLD indication from the bridging code is swallowed,
+	  leaving the channel in question to still appear to be on hold.
+
+	  The fix here is to modify the logic inside ast_indicate_data() to not
+	  drop the indication if the channel is simply leaving a bridge. This way,
+	  channels on hold redirected out of a bridge revert to their expected "in
+	  use" state after the redirection.
+
+	  ASTERISK-25418 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: If6115204dfa0551c050974ee138fabd15f978949
+
+2015-09-22 17:08 +0000 [06f4f80a63]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_page.c: Fix crash when forwarding with a predial handler.
+
+	  Page uses the async method of dialing with the dial API.  When a call gets
+	  forwarded there is no calling channel available.  If the predial handler
+	  was set then the calling channel could not be put into auto-service
+	  for the forwarded call because it doesn't exist.  A crash is the result.
+
+	  * Moved the callee predial parameter string processing to before the
+	  string is passed to the dial API rather than having the dial API do it.
+	  There are a few benefits do doing this.  The first is the predial
+	  parameter string processing doesn't need to be done for each channel
+	  called by the dial API.  The second is in async mode and the forwarded
+	  channel is to have the predial handler executed on it then the
+	  non-existent calling channel does not need to be present to process the
+	  predial parameter string.
+
+	  * Don't start auto-service on a non-existent calling channel to execute
+	  the predial handler when the dial API is in async mode and forwarding a
+	  call.
+
+	  ASTERISK-25384 #close
+	  Reported by: Chet Stevens
+
+	  Change-Id: If53892b286d29f6cf955e2545b03dcffa2610981
+
+2015-09-04 12:25 +0000 [b99a705262]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Add the ability to subscribe to all events
+
+	  This patch adds the ability to subscribe to all events. There are two possible
+	  ways to accomplish this:
+	  (1) On initial WebSocket connection. This patch adds a new query parameter,
+	      'subscribeAll'. If present and True, Asterisk will subscribe the
+	      applications to all ARI events.
+	  (2) Via the applications resource. When subscribing in this manner, an ARI
+	      client should merely specify a blank resource name, i.e., 'channels:'
+	      instead of 'channels:12354'. This will subscribe the application to all
+	      resources of the 'channels' type.
+
+	  ASTERISK-24870 #close
+
+	  Change-Id: I4a943b4db24442cf28bc64b24bfd541249790ad6
+
+2015-09-21 18:06 +0000 [c74101509d]  Kevin Harwell <kharwell@digium.com>
+
+	* app_record: RECORDED_FILE variable not being populated
+
+	  The RECORDED_FILE variable is empty unless a '%d' is specified in the filename.
+	  This patch makes it so the variable is always set to the filename.
+
+	  ASTERISK-25410 #close
+
+	  Change-Id: I4ec826d8eb582ae2ad184e717be8668b74d37653
+
+2015-09-21 08:16 +0000 [a29cf45c76]  Elazar Broad <elazar@thebroadfamily.com>
+
+	* core/logging: Fix logging to more than one syslog channel
+
+	  Currently, Asterisk will log to the last configured syslog
+	  channel in logger.conf. This is due to the fact that the
+	  final call to openlog() supersedes all of the previous calls.
+	  This commit removes the call to openlog() and passes the
+	  facility to ast_log_vsyslog(), along with utilizing the
+	  LOG_MAKEPRI macro to ensure that the message is routed to
+	  the correct facility and with the correct priority.
+
+	  ASTERISK-25407 #close
+	  Reported by: Elazar Broad
+	  Tested by: Elazar Broad
+
+	  Change-Id: Ie2a2416bc00cce1b04e99ef40917c2011953ddd2
+2015-09-04 12:24 +0000 [47813cc51c]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_stasis_device_state: Allow for subscribing to 'all' device state
+
+	  This patch adds support for subscribing to all device state changes. This is
+	  done either by subscribing to an empty device, e.g., 'eventSource=deviceState:',
+	  or by the WebSocket connection specifying that it wants all state in the
+	  system.
+
+	  ASTERISK-24870
+
+	  Change-Id: I9cfeca1c9e2231bd7ea73e45919111d44d2eda32
+
+2015-09-03 21:19 +0000 [5206aa9d30]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Add events for Contact and Peer Status changes
+
+	  This patch adds support for receiving events regarding Peer status changes
+	  and Contact status changes. This is particularly useful in scenarios where
+	  we are subscribed to all endpoints and channels, where we often want to know
+	  more about the state of channel technology specific items than a single
+	  endpoint's state.
+
+	  ASTERISK-24870
+
+	  Change-Id: I6137459cdc25ce27efc134ad58abf065653da4e9
+
+2015-09-19 12:49 +0000 [9200ad03a3]  Alexander Traud <pabstraud@compuserve.com>
+
+	* astfd: Adds a timestamp for each entry.
+
+	  Now with menuselect "DEBUG_FD_LEAKS" and CLI "core show fd", a timestamp is
+	  shown with each file descriptor. This helps to debug leaked UDP/TCP ports on
+	  long-lived servers, for example.
+
+	  ASTERISK-25405 #close
+
+	  Change-Id: I968339e5155a512eba1032a5263f1ec8b5e1f80b
+
+2015-09-16 08:22 +0000 [42a897c4c3]  Joshua Colp <jcolp@digium.com>
+
+	* pbx: Update device and presence state when changing a hint extension.
+
+	  When changing a hint extension without removing the hint first the
+	  device state and presence state is not updated. This causes the state
+	  of the hint to be that of the previous extension and not the current
+	  one. This state is kept until a state change occurs as a result of
+	  something (presence state change, device state change).
+
+	  This change updates the hint with the current device and presence
+	  state of the new extension when it is changed. Any state callbacks
+	  which may have been added before the hint extension is changed are
+	  also informed of the new device and presence state if either have
+	  changed.
+
+	  ASTERISK-25394 #close
+
+	  Change-Id: If268f1110290e502c73dd289c9e7e7b27bc8432f
+
+2015-09-17 16:34 +0000 [d9723d242a]  Scott Griepentrog <scott@griepentrog.com>
+
+	* CHAOS: avoid crash if string create fails
+
+	  Validate string buffer allocation before using them.
+
+	  ASTERISK-25323
+
+	  Change-Id: Ib9c338bdc1e53fb8b81366f0b39482b83ef56ce0
+
+2015-09-11 01:52 +0000 [99aa7cb26e]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* dr_adaptive_odbc.c, cel_odbc.c, cel_pgsql.c: REFACTOR Macro LENGTHEN_BUF
+
+	  Remove repeated code on macro of assigned buffer to SQL vars
+
+	  Change-Id: Icb19ad013124498e172ea1d0b29ccd0ed17deef0
+
+2015-09-17 04:52 +0000 [e4df271a3e]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* chan_sip: Fix From header truncation for extremely long CALLERID(name).
+
+	  The CALLERID(num) and CALLERID(name) and other info are placed into the
+	  `char from[256]` in initreqprep. If the name was too long, the addr-spec
+	  and params wouldn't fit.
+
+	  Code is moved around so the addr-spec with params is placed there first,
+	  and then fitting in as much of the display-name as possible.
+
+	  ASTERISK-25396 #close
+
+	  Change-Id: I33632baf024f01b6a00f8c7f35c91e5f68c40260
+
+2015-09-17 16:59 +0000 [e1927915bc]  Richard Mudgett <rmudgett@digium.com>
+
+	* CHAOS: res_pjsip_diversion avoid crash if allocation fails
+
+	  Validate ast_malloc buffer returned before using it in
+	  set_redirecting_value().
+
+	  ASTERISK-25323
+
+	  Change-Id: I15d2ed7cb0546818264c0bf251aa40adeae83253
+
+2015-09-17 16:47 +0000 [729a4325da]  Kevin Harwell <kharwell@digium.com>
+
+	* app_queue: AgentComplete event has wrong reason
+
+	  When a queued caller transfers an agent to another extension sometimes the
+	  raised AgentComplete event has a reason of "caller" and sometimes "transfer".
+	  Since a transfer has taken place this should always be transfer. This occurs
+	  because sometimes the stasis hangup event arrives before the transfer event
+	  thus writing a different reason out.
+
+	  With this patch, when a hangup event is received during a transfer it will
+	  check to see if the channel that is hanging up is part of a transfer. If so
+	  it will return and let the subsequently received transfer event handler take
+	  care of the cleanup.
+
+	  ASTERISK-25399 #close
+
+	  Change-Id: Ic63c49bd9a5ed463ea7a032fd2ea3d63bc81a50d
+
+2015-09-17 13:09 +0000 [87f04d5acf]  Scott Griepentrog <scott@griepentrog.com>
+
+	* PJSIP: avoid crash when getting rtp peer
+
+	  Although unlikely, if the tech private is returned as
+	  a NULL, chan_pjsip_get_rtp_peer() would crash.
+
+	  ASTERISK-25323
+
+	  Change-Id: Ie231369bfa7da926fb2b9fdaac228261a3152e6a
+
+2015-09-17 11:31 +0000 [63ede41227]  Kevin Harwell <kharwell@digium.com>
+
+	* app_queue: Crash when transferring
+
+	  During some transfer scenarios involving queues Asterisk would sometimes
+	  crash when trying to obtain a channel snapshot (could happen on caller or
+	  member channels). This occurred because the underlying channel had already
+	  disappeared when trying to obtain the latest snapshot.
+
+	  This patch adds a reference to both the member and caller channels that
+	  extends to the lifetime of the queue'd call, thus making sure the channels
+	  will always exist when retrieving the latest snapshots.
+
+	  ASTERISK-25185 #close
+	  Reported by: Etienne Lessard
+
+	  Change-Id: Ic397fa68fb4ff35fbc378e745da9246a7b552128
+
+2015-09-16 17:36 +0000 [e47396721f]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Eliminate race during initial NOTIFY.
+
+	  There is a slim chance of a race condition occurring where two threads
+	  can both attempt to manipulate the same area.
+
+	  Thread A can be handling an incoming initial SUBSCRIBE request. Thread A
+	  lets the specific subscription handler know that the subscription has
+	  been established.
+
+	  At this point, Thread B may detect a state change on the subscribed
+	  resource and queue up a notification task on Thread C, the subscription
+	  serializer thread.
+
+	  Now Thread A attempts to generate the initial NOTIFY request to send to
+	  the subscriber at the same time that Thread C attempts to generate a
+	  state change NOTIFY request to send to the subscriber.
+
+	  The result is that Threads A and C can step on the same memory area,
+	  resulting in a crash. The crash has been observed as happening when
+	  attempting to allocate more space to hold the body for the NOTIFY.
+
+	  The solution presented here is to queue the subscription establishment
+	  and initial NOTIFY generation onto the subscription serializer thread
+	  (Thread C in the above scenario). This way, there is no way that a state
+	  change notification can occur before the initial NOTIFY is sent, and if
+	  there is a quick succession of NOTIFYs, we can guarantee that the two
+	  NOTIFY requests will be sent in succession.
+
+	  Change-Id: I5a89a77b5f2717928c54d6efb9955e5f6f5cf815
+
+2015-08-28 15:42 +0000 [077adf48b8]  Alexander Traud <pabstraud@compuserve.com>
+
+	* translate: Fix transcoding while different in frame size.
+
+	  When Asterisk translates between codecs, each with a different frame size (for
+	  example between iLBC 30 and Speex-WB), too large frames were created by
+	  ast_trans_frameout. Now, ast_trans_frameout is called with the correct frame
+	  length, creating several frames when necessary. Affects all transcoding modules
+	  which used ast_trans_frameout: GSM, iLBC, LPC10, and Speex.
+
+	  ASTERISK-25353 #close
+
+	  Change-Id: I2e229569d73191d66a4e43fef35432db24000212
+
+2015-09-10 17:19 +0000 [0a74c80300]  Mark Michelson <mmichelson@digium.com>
+
+	* scheduler: Use queue for allocating sched IDs.
+
+	  It has been observed that on long-running busy systems, a scheduler
+	  context can eventually hit INT_MAX for its assigned IDs and end up
+	  overflowing into a very low negative number. When this occurs, this can
+	  result in odd behaviors, because a negative return is interpreted by
+	  callers as being a failure. However, the item actually was successfully
+	  scheduled. The result may be that a freed item remains in the scheduler,
+	  resulting in a crash at some point in the future.
+
+	  The scheduler can overflow because every time that an item is added to
+	  the scheduler, a counter is bumped and that counter's current value is
+	  assigned as the new item's ID.
+
+	  This patch introduces a new method for assigning scheduler IDs. Instead
+	  of assigning from a counter, a queue of available IDs is maintained.
+	  When assigning a new ID, an ID is pulled from the queue. When a
+	  scheduler item is released, its ID is pushed back onto the queue. This
+	  way, IDs may be reused when they become available, and the growth of ID
+	  numbers is directly related to concurrent activity within a scheduler
+	  context rather than the uptime of the system.
+
+	  Change-Id: I532708eef8f669d823457d7fefdad9a6078b99b2
+
+2015-09-03 21:15 +0000 [45cf79665c]  Matt Jordan <mjordan@digium.com>
+
+	* main/config_options: Check for existance of internal object before derefing
+
+	  Asterisk can load and register an object type while still having an invalid
+	  sorcery mapping. This can cause an issue when a creation call is invoked.
+	  For example, mis-configuring PJSIP's endpoint identifier by IP address mapping
+	  in sorcery.conf will cause the sorcery mechanism to be invalidated; however, a
+	  subsequent ARI invocation to create the object will cause a crash, as the
+	  internal type may not be registered as sorcery expects.
+
+	  Merely checking for a NULL pointer here solves the issue.
+
+	  Change-Id: I54079fb94a1440992f4735a9a1bbf1abb1c601ac
+
+2015-08-21 21:50 +0000 [34aa96bef4]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* chan_sip.c: Validation on module reload
+
+	  Change validation on reload module because now used the cli function for
+	  reload. The sip_reload() function never fail and ever return NULL for this
+	  reason on reload() now use the call the sip_reload() and return
+	  AST_MODULE_LOAD_SUCCESS.
+
+	  This problem is dectected on reload by PUT method on ARI, getting always
+	  404 http code when the module is reloaded.
+
+	  ASTERISK-25325 #close
+	  Reporte by: Rodrigo Ramírez Norambuena
+
+	  Change-Id: I41215877fb2cfc589e0d4d464000cf6825f4d7fb
+
+2015-08-21 17:39 +0000 [69824fdfbf]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Mark ast_sip_create_subscription() as not used.
+
+	  Change-Id: I2b8db18eac36c01a5c7eb9467699124e203fd093
+
+2015-09-09 12:24 +0000 [2526659432]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Add some notification comments.
+
+	  Change-Id: Ie62ff1f4b7adc1a12fa0303f53926af249b25e20
+
+2015-08-21 18:01 +0000 [9b290dfe2f]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Set dlg_status code instead of sending SIP response.
+
+	  We should not try to send a SIP response message because we may be
+	  restoring a persistent subscription where we are not responding to a SIP
+	  request.
+
+	  Change-Id: Id89167ef90320c5563f37e632db0dda6cb9e7dec
+
+2015-08-21 17:40 +0000 [73eb132012]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Fix off-nominal memory leak.
+
+	  Fix off-nominal visited vector leak in build_resource_tree().
+
+	  Change-Id: If0399c7941c9c0b1038bcfb7b9a371760977831c
+
+2015-08-21 15:26 +0000 [2b30fc2b2d]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Fix one byte buffer overrun error.
+
+	  ast_sip_pubsub_register_body_generator() did not account for the null
+	  terminator set by sprintf() in the allocated output buffer.
+
+	  Change-Id: I388688a132e479bca6ad1c19275eae0070969ae2
+
+2015-08-21 15:25 +0000 [08a182c8e6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Use ast_alloca() instead of alloca().
+
+	  Change-Id: Ia396096b4fedc2874649ca11137612c3f55e83e3
+
+2015-08-21 11:04 +0000 [61f30db877]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_pubsub.c: Add missing error return in load_module().
+
+	  Change-Id: I15debd0f717f16ee2f78e7f56151c3b3b97b72fc
+
+2015-08-21 11:03 +0000 [b8f07527b2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip/location.c: Use the builtin ao2_callback() match function instead.
+
+	  Change-Id: I364906d6d2bad3472929986704a0286b9a2cbe3f
+
+2015-09-10 09:49 +0000 [f1a2e82d49]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Copy default_from_user to avoid crash.
+
+	  The default_from_user retrieval function was pulling the
+	  default_from_user from the global configuration struct in an unsafe way.
+	  If using a database as a backend configuration store, the global
+	  configuration struct is short-lived, so grabbing a pointer from it
+	  results in referencing freed memory.
+
+	  The fix here is to copy the default_from_user value out of the global
+	  configuration struct.
+
+	  Thanks go to John Hardin for discovering this problem and proposing the
+	  patch on which this fix is based.
+
+	  ASTERISK-25390 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I6b96067a495c1259da768f4012d44e03e7c6148c
+
+2015-09-10 08:39 +0000 [bd71dcd1da]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_nat: Ignore REGISTER requests when looking for a Record-Route
+
+	  We will only rewrite the Contact header if there is no Record-Route header in
+	  the received request. If a malfunctioning proxy places a Record-Route header
+	  into a REGISTER request, we will decide that we shouldn't update the IP/port
+	  in the Contact header, and we will end up storing a contact with an AoR that
+	  contains the NAT'd IP address.
+
+	  While it is nice to have the proxy *not* send a Record-Route in a REGISTER
+	  request, it's also a good idea to not process the header in a non-dialog
+	  message. This patch updates the code to explicitly ignore the Record-Route
+	  header in REGISTER requests.
+
+	  ASTERISK-25387 #close
+
+	  Change-Id: I4bd3bcccc4003d460cc354d986b0dea2e433ef3f
+
+2015-09-09 16:46 +0000 [5bd363010e]  Alexander Anikin <may213@yandex.ru>
+
+	* chan_ooh323: Add ProgressIndicator IE with inband info available
+
+	  Add ProgressIndicator IE with inband info present to Progress and
+	  Alerting Q.931 message
+
+	  ASTERISK-25227 #close
+	  Reported by: Alexandr Dranchuk
+
+	  Change-Id: I326ad13cb1db9a72b3fd902bafed3c28a3684203
+2015-09-08 10:35 +0000 [fcea6910f6]  Scott Griepentrog <scott@griepentrog.com>
+
+	* pjsip: avoid possible crash req_caps allocation failure
+
+	  Make certain that the pjsip session has not failed to
+	  allocate the format capabilities structure, which can
+	  otherwise cause a crash when referenced.
+
+	  ASTERISK-25323
+
+	  Change-Id: I602790ba12714741165e441cc64a3ecde4cb5750
+
+2015-09-04 16:33 +0000 [8e5ed27a16]  David M. Lee <dlee@respoke.io>
+
+	* res_rtp_asterisk: Add more ICE debugging
+
+	  In working through a recent ICE negotiation bug, I found the debug
+	  logging in res_rtp_asterisk to be lacking. This patch adds a number of
+	  debug and warning statements that were helpful.
+
+	  Change-Id: I950c6d8f13a41f14b3d6334b4cafe7d4e997be80
+2015-09-08 07:21 +0000 [3628e380b8]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Use hash for contact object identity instead of Contact URI.
+
+	  In the wild it is possible for Contact URIs to be quite long as
+	  parameters can exist on them. This can present a problem when storing
+	  them in the AstDB as the URI is used as part of the object name and
+	  there is a fixed length limit for the AstDB. This will cause
+	  the contact to not get stored.
+
+	  This change uses the MD5 hash of the Contact URI as part of the
+	  object name instead. This has a fixed length which is guaranteed
+	  to not exceed the AstDB length limit.
+
+	  ASTERISK-25295 #close
+
+	  Change-Id: Ie8252a75331ca00b41b9f308f42cc1fbdf701a02
+
+2015-09-07 13:19 +0000 [d2106c0b21]  Alexander Anikin <may213@yandex.ru>
+
+	* chan_ooh323: call ast_rtp_instance_stop on ooh323_destroy
+
+	      Call ast_rtp_instance_stop on ooh323_destroy to free resources
+	      allocated by rtp instance
+
+	      ASTERISK-25299 #close
+	      Report by: Alexandr Dranchuk
+
+	  Change-Id: I455096bd7da016b871afe90af86067c2c7c9f33f
+
+2015-09-07 11:15 +0000 [ef3358d0c0]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip: Purge contacts when an AoR is deleted
+
+	  When an AoR is deleted by an external mechanism, such as through ARI, we
+	  currently do not remove dynamic contacts that were created for that AoR as a
+	  result of a received REGISTER request. As a result, re-creating the AoR will
+	  cause the dynamic contact to be interpreted as a persistent contact, leading
+	  to some rather strange state being created for the contacts/endpoints.
+
+	  This patch adds a sorcery observer for the 'aor' object. When a delete is
+	  issued on the underlying sorcery object, the observer is called, and all
+	  contacts created and persisted in sorcery for that AoR are also removed. Note
+	  that we don't want to perform this action when an AO2 object that is an AoR is
+	  destroyed, as the AoR can still exist in the backing storage (and we would
+	  thus be removing valid contacts from an AoR that still "exists".)
+
+	  ASTERISK-25381 #close
+
+	  Change-Id: I6697e51ef6b2858b5d63401f35dc378bb0f90328
+
+2015-09-05 14:58 +0000 [86b02228f5]  Matt Jordan <mjordan@digium.com>
+
+	* channels/pjsip/dialplan_functions: Add an option for extracting the SIP call-id
+
+	  This patch adds a new option to the CHANNEL function that allows for the
+	  extraction of the SIP call-id. It is used in conjunction with the 'pjsip'
+	  option, and will return the Call-ID of the INVITE request that established
+	  the PJSIP channel.
+
+	  ASTERISK-25352
+
+	  Change-Id: I278d1f8bcfe3a53c5aa1dadebc14e92b0abd476a
+
+2015-09-04 16:06 +0000 [27c89053b0]  David M. Lee <dlee@respoke.io>
+
+	* Fix when remote candidates exceed PJ_ICE_MAX_CAND
+
+	  We were passing the wrong count into pj_ice_sess_create_check_list(),
+	  causing the create to fail if we ever received more than PJ_ICE_MAX_CAND
+	  candidates.
+
+	  Change-Id: I0303d8e1ecb20a8de9fe629a3209d216c4028378
+
+2015-09-04 14:40 +0000 [993ae9a669]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Change default from user value.
+
+	  When Asterisk sends an outbound SIP request, if there is no direct
+	  reason to place a specific value for the username in the From header,
+	  Asterisk would generate a UUID. For example, this would happen when
+	  sending outbound OPTIONS requests when qualifying or when sending
+	  outbound INVITE requests when originating (if no explicit caller ID were
+	  provided). The issue is that some SIP providers reject these sorts of
+	  requests with a "Name too long" error response.
+
+	  This patch aims to fix this by changing the default outbound username in
+	  From headers to "asterisk". This value can be overridden by changing the
+	  default_from_user option in the global options if desired.
+
+	  ASTERISK-25377 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I6a4d34a56ff73ff4f661b0075aeba5461b7f3190
+
+2015-09-03 14:07 +0000 [7d981b787c]  Jonathan Rose <jrose@digium.com>
+
+	* ParkAndAnnounce: Add variable inheritance
+
+	  In Asterisk 11, the announcer channel would receive channel variables
+	  from the channel being parked by means of normal channel inheritance.
+	  This functionality was lost during the big res_parking project in
+	  Asterisk 12. This patch restores that functionality.
+
+	  ASTERISK-25369 #close
+	  Review: https://gerrit.asterisk.org/#/c/1180/
+
+	  Change-Id: Ie47e618330114ad2ea91e2edcef1cb6f341eed6e
+
+2015-09-04 09:26 +0000 [7691035312]  Scott Griepentrog <scott@griepentrog.com>
+
+	* endpoint snapshot: avoid second cleanup on alloc failure
+
+	  In ast_endpoint_snapshot_create(), a failure to init the
+	  string fields results in two attempts to ao2_cleanup the
+	  same pointer.  Removed RAII_VAR to eliminate problem.
+
+	  ASTERISK-25375 #close
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: If4d9dfb1bbe3836b623642ec690b6d49b25e8979
+
+2015-09-04 05:33 +0000 [be31747db8]  Martin Tomec <tomec.martin@gmail.com>
+
+	* res/pjsip: Mark WSS transport as secure
+
+	  Pjsip is refusing to use unsecure transport with "sips" in url.
+	  WSS should be considered as secure transport.
+
+	  ASTERISK-24602 #comment Partially fixed by setting WSS as secure
+
+	  Change-Id: Iddac406c6deba6240c41a603b8859dfefe1a5353
+
+2015-09-01 10:16 +0000 [fbdb42c9fc]  Guido Falsi <madpilot@freebsd.org>
+
+	* Core/General: Add #ifdef needed on FreeBSD.
+
+	  pthread_attr_init() defaults to PTHREAD_EXPLICIT_SCHED on FreeBSD
+	  too.
+
+	  ASTERISK-25310 #close
+	  Reported by: Guido Falsi
+
+	  Change-Id: Iae6befac9028b5b9795f86986a4a08a1ae6ab7c4
+2015-09-02 17:26 +0000 [c15d8cc0ed]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Fix contact refleak on stateful responses.
+
+	  When sending a stateful response, creation of the transaction can fail,
+	  most commonly because we are trying to create a transaction from a
+	  retransmitted request. When creation of the transaction fails, we end up
+	  leaking a reference to a contact that was bumped when the response was
+	  created.
+
+	  This patch adds the missing deref and fixes the reference leak.
+
+	  Change-Id: I2f97ad512aeb1b17e87ca29ae0abacb4d6395f07
+
+2015-09-02 12:41 +0000 [b51cf1e712]  Joshua Colp <jcolp@digium.com>
+
+	* pbx: Fix crash when issuing "core show hints" with long pattern match.
+
+	  When issuing the "core show hints" CLI command a combination of both
+	  the hint extension and context is created. This uses a fixed size
+	  buffer expecting that the extension will not exceed maximum extension
+	  length. When the extension is actually a pattern match this constraint
+	  does not hold true, and the extension may exceed the maximum extension
+	  length. In this case extra characters are written past the end of the
+	  fixed size buffer.
+
+	  This change makes it so the construction of the combined hint extension
+	  and context can not exceed the size of the buffer.
+
+	  ASTERISK-25367 #close
+
+	  Change-Id: Idfa1b95d0d4dc38e675be7c1de8900b3f981f499
+
+2015-09-01 09:05 +0000 [beb568e51c]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: re-re-fix persistent subscription storage.
+
+	  A recent change to res_pjsip_pubsub switched to using pjsip_msg_print as
+	  a means of writing an appropriate packet to persistent storage. While
+	  this partially solved the issue, it had its own problems.
+	  pjsip_msg_print will always add a Content-Length header to the message
+	  it prints. Frequent restarts of Asterisk can result in persistent
+	  subscriptions being written with five or more Content-Length headers. In
+	  addition, sometimes some apparent corruption of individual headers could
+	  be seen.
+
+	  This aims to fix the problem by not running a parsed message through an
+	  interpreter but rather by taking the raw message and saving it. The
+	  logic for what to save is going to be different depending on whether a
+	  SUBSCRIBE was received from the wire or if it was pulled from
+	  persistence. When receiving a packet from the wire, when using a
+	  streaming transport, the rdata->pkt_info.packet may contain multiple SIP
+	  messages or fragments. However, the rdata->msg_info.msg_buf will always
+	  contain the current SIP message to be processed. When pulling from
+	  persistence, though, the rdata->msg_info.msg_buf will be NULL since no
+	  transport actually handled the packet. However, since we know that we
+	  will always ever pull one SIP message from persistence, we are free to
+	  save directly from rdata->pkt_info.packet instead.
+
+	  ASTERISK-25365 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I33153b10d0b4dc8e3801aaaee2f48173b867855b
+
+2015-08-29 10:36 +0000 [fc4d4f5379]  Joshua Colp <jcolp@digium.com>
+
+	* taskprocessor: Fix race condition between unreferencing and finding.
+
+	  When unreferencing a taskprocessor its reference count is checked
+	  to determine if it should be unlinked from the taskprocessors
+	  container and its listener shut down. In between the time when the
+	  reference count is checked and unlinking it is possible for
+	  another thread to jump in, find it, and get a reference to it. If
+	  the thread then uses the taskprocessor it may find that it is not
+	  in the state it expects.
+
+	  This change locks the taskprocessors container during almost the
+	  entire unreference operation to ensure that any other thread which
+	  may attempt to find the taskprocessor has to wait.
+
+	  ASTERISK-25295
+
+	  Change-Id: Icb842db82fe1cf238da55df92e95938a4419377c
+
+2015-08-28 20:22 +0000 [bb38010c67]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_sdp_rtp: Fix multiple keepalive scheduled items.
+
+	  The keepalive support in res_pjsip_sdp_rtp currently assumes
+	  that a stream will only be negotiated once. This is false.
+	  If the stream is replaced and later added back it can be
+	  negotiated again causing multiple keepalive scheduled items
+	  to exist. This change explicitly deletes the existing
+	  keepalive scheduled item before adding the new one.
+
+	  The res_pjsip_sdp_rtp module also does not stop RTP
+	  keepalives or timeout timer if the stream has been
+	  replaced. This change adds a callback to the session media
+	  interface to allow a media stream to be stopped without
+	  the resources being destroyed. This allows the scheduled
+	  items and RTP to be stopped when the stream no longer
+	  exists.
+
+	  ASTERISK-25356 #close
+
+	  Change-Id: Ibe6a7cc0927c87326fd5f1c0d4ad889dbfbea1de
+
+2015-08-28 19:57 +0000 [c036e50fbe]  Joshua Colp <jcolp@digium.com>
+
+	* sched: ast_sched_del may return prematurely due to spurious wakeup
+
+	  When deleting a scheduled item if the item in question is currently
+	  executing the ast_sched_del function waits until it has completed.
+	  This is accomplished using ast_cond_wait. Unfortunately the
+	  ast_cond_wait function can suffer from spurious wakeups so the
+	  predicate needs to be checked after it returns to make sure it has
+	  really woken up as a result of being signaled.
+
+	  This change adds a loop around the ast_cond_wait to make sure that
+	  it only exits when the executing task has really completed.
+
+	  ASTERISK-25355 #close
+
+	  Change-Id: I51198270eb0b637c956c61aa409f46283432be61
+
+2015-08-27 12:26 +0000 [229b95d253]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_session: Don't invoke session supplements twice for BYE requests.
+
+	  When a BYE request is received the PJSIP invite session implementation
+	  creates and sends a 200 OK response before we are aware of it. This
+	  causes the INVITE session state callback to be called into and ultimately
+	  the session supplements run on the BYE request. Once this response has
+	  been sent the normal transaction state callback is invoked which
+	  invokes the session supplements on the BYE request again. This can
+	  be problematic in particular with res_pjsip_rfc3326 as it may
+	  attempt to update the hangup cause code on the channel while it is
+	  in the process of being hung up.
+
+	  This change makes it so the session supplements are only invoked
+	  once by the INVITE session state callback.
+
+	  ASTERISK-25318 #close
+
+	  Change-Id: I69c17df55ccbb61ef779ac38cc8c6b411376c19a
+
+2015-08-26 15:26 +0000 [6bfa14bdad]  Scott Griepentrog <scott@griepentrog.com>
+
+	* Chaos: handle failed allocation in get_media_encryption_type
+
+	  If the ast_strndup() call fails to allocate a copy of the
+	  transport string for parsing, fail gracefully.
+
+	  ASTERISK-25323
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: Ia4b905ce6d03da53fea526224455c1044b1a5a28
+
+2015-08-26 14:25 +0000 [490db8ba94]  Scott Griepentrog <scott@griepentrog.com>
+
+	* Chaos: make hangup NULL tolerant
+
+	  In chan_pjsip_new, if allocation of the pvt
+	  structure fails, ast_hangup is called.  But
+	  it was written to assume pvt was valid, and
+	  this change corrects that.
+
+	  ASTERISK-25323
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: I5f47860fe9cee4cd56abd3f79b108678ab72cc87
+2015-08-26 05:40 +0000 [d03d09aad3]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip: Allow call pickup to set the hangup cause.
+
+	  The call pickup implementation in chan_sip currently sets the channel
+	  hangup cause to "normal clearing" if call pickup is successfully
+	  performed. This action overwrites the "answered elsewhere" hangup cause
+	  set by the call pickup code and can result in the SIP device in
+	  question showing a missed call when it should not.
+
+	  This change sets the hangup cause to "normal clearing" as a
+	  default initially but allows the call pickup to change it as
+	  needed.
+
+	  ASTERISK-25346 #close
+
+	  Change-Id: I00ac2c269cee9e29586ee2c65e83c70e52a02cff
+
+2015-08-25 07:17 +0000 [d013ecf748]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Add common ast_sip_get_host_ip API.
+
+	  Modules commonly used the pj_gethostip function for retrieving the
+	  IP address of the host. This function does not cache the result and may
+	  result in a DNS lookup occurring, or additional work. If the DNS
+	  server is unreachable or network issues arise this can cause the
+	  pj_gethostip function to block for a period of time.
+
+	  This change adds an ast_sip_get_host_ip and ast_sip_get_host_ip_string
+	  function which does the same thing but caches the host IP address at
+	  module load time. This results in no additional work being done each
+	  time the local host IP address is needed.
+
+	  ASTERISK-25342 #close
+
+	  Change-Id: I3205deb679b01fa5ac05a94b623bfd620a2abe1e
+
+2015-08-24 06:21 +0000 [98d089fb9a]  Joshua Colp <jcolp@digium.com>
+
+	* bridge: Kick channel from bridge if hung up during action.
+
+	  When executing an action in a bridge it is possible for the
+	  channel to be hung up without the bridge becoming aware of it.
+	  This is most easily reproducible by hanging up when the bridge
+	  is streaming DTMF due to a feature timeout. This change makes
+	  it so after action execution the channel is checked to determine
+	  if it has been hung up and if it has it is kicked from the bridge.
+
+	  ASTERISK-25341 #close
+
+	  Change-Id: I6dd8b0c3f5888da1c57afed9e8a802ae0a053062
+
+2015-08-24 11:04 +0000 [a408369bac]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_pubsub: On recreated notify fail deleted sub_tree is referenced
+
+	  When recreating a subscription it is possible for a freed sub_tree
+	  to be referenced when the initial NOTIFY fails to be created.
+
+	  Change-Id: I681c215309aad01b21d611c2de47b3b0a6022788
+
+2015-08-23 18:26 +0000 [3af34441eb]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip/pjsip_configuration: Disregard empty auth values
+
+	  When an endpoint is backed by a non-static conf file backend (such as
+	  the AstDB or Realtime), the 'auth' object may be returned as being an
+	  empty string. Currently, res_pjsip will interpret that as being a valid
+	  auth object, and will attempt to authenticate inbound requests. This
+	  isn't desired; is an auth value is empty (which the name of an auth
+	  object cannot be), we should instead interpret that as being an invalid
+	  auth object and skip it.
+
+	  ASTERISK-25339 #close
+
+	  Change-Id: Ic32b0c6eb5575107d5164a8c40099e687cd722c7
+
+2015-08-21 23:37 +0000 [89003ea320]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+
+	* README*: Remove trailing whitespace
+
+	  Change-Id: I18b7d75187548a9ed55b4f258d21aaaf29d08874
+
+2015-07-28 13:47 +0000 [857923d9c7]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Set preferred rx payload type mapping on incoming offers.
+
+	  ASTERISK-25166
+	  Reported by: Kevin Harwell
+
+	  ASTERISK-17410
+	  Reported by: Boris Fox
+
+	  Change-Id: I7f04d5c8bee1126fee5fe6afbc39e45104469f4e
+
+2015-07-24 18:46 +0000 [d643b206c6]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_sdp_rtp.c: Set preferred rx payload type mapping on incoming offers.
+
+	  ASTERISK-25166
+	  Reported by: Kevin Harwell
+
+	  ASTERISK-17410
+	  Reported by: Boris Fox
+
+	  Change-Id: I97ecebc1ab9b5654fb918bf1f4c98c956b852369
+
+2015-07-27 19:19 +0000 [f7df3e1a01]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Get current or create a needed rx payload type mapping.
+
+	  * Make ast_rtp_codecs_payload_code() get the current mapping or create a
+	  rx payload type mapping.
+
+	  ASTERISK-25166
+	  Reported by: Kevin Harwell
+
+	  ASTERISK-17410
+	  Reported by: Boris Fox
+
+	  Change-Id: Ia4b2d45877a8f004f6ce3840e3d8afe533384e56
+
+2015-07-27 19:15 +0000 [38854a9f7b]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Extract rtp_codecs_payload_replace_rx().
+
+	  ASTERISK-25166
+	  Reported by: Kevin Harwell
+
+	  ASTERISK-17410
+	  Reported by: Boris Fox
+
+	  Change-Id: I34e23bf5b084c8570f9c3e6ccd19b95fe85af239
+
+2015-07-23 19:24 +0000 [1a549ed134]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Initial split of payload types into rx and tx mappings.
+
+	  There are numerous problems with the current implementation of the RTP
+	  payload type mapping in Asterisk.  It uses only one mapping structure to
+	  associate payload types to codecs.  The single mapping is overkill if all
+	  of the payload type values are well known values.  Dynamic payload type
+	  mappings do not work as well with the single mapping because RFC3264
+	  allows each side of the link to negotiate different dynamic mappings for
+	  what they want to receive.  Not only could you have the same codec mapped
+	  for sending and receiving on different payload types you could wind up
+	  with the same payload type mapped to different codecs for each direction.
+
+	  1) An independent payload type mapping is needed for sending and
+	  receiving.
+
+	  2) The receive mapping needs to keep track of previous mappings because of
+	  the slack to when negotiation happens and current packets in flight using
+	  the old mapping arrive.
+
+	  3) The transmit mapping only needs to keep track of the current negotiated
+	  values since we are sending the packets and know when the switchover takes
+	  place.
+
+	  * Needed to create ast_rtp_codecs_payload_code_tx() and make some callers
+	  use the new function because ast_rtp_codecs_payload_code() was used for
+	  mappings in both directions.
+
+	  * Needed to create ast_rtp_codecs_payloads_xover() for cases where we need
+	  to pass preferred codec mappings to the peer channel for early media
+	  bridging or when we need to prefer the offered mapping that RFC3264 says
+	  we SHOULD use.
+
+	  * ast_rtp_codecs_payloads_xover() and ast_rtp_codecs_payload_code_tx() are
+	  the only new public functions created.  All the others were only used for
+	  the tx or rx mapping direction so the function doxygen now reflects which
+	  direction the function operates.
+
+	  * chan_mgcp.c: Removed call to ast_rtp_codecs_payloads_clear() as doing
+	  that makes no sense when processing an incoming SDP.  We would be wiping
+	  out any mappings that we set for the possible outgoing SDP we sent
+	  earlier.
+
+	  ASTERISK-25166
+	  Reported by: Kevin Harwell
+
+	  ASTERISK-17410
+	  Reported by: Boris Fox
+
+	  Change-Id: Iaf6c227bca68cb7c414cf2fd4108a8ac98bd45ac
+
+2015-08-19 12:10 +0000 [21d419e4fc]  Richard Mudgett <rmudgett@digium.com>
+
+	* ari/ari_websockets.c: Fix ast_debug parameter type mismatch.
+
+	  This is a type mismatch fix of the debugging commit
+	  c63316eec10e1990a88bf4712238d6deb375bfa9 made to find out why
+	  a testsuite test was failing only on one of the continuous
+	  integration build agents.
+
+	  Change-Id: Iba34f6e87cec331f6ac80e4daff6476ea6f00a75
+
+2015-08-19 10:30 +0000 [53e2a6a829]  Scott Griepentrog <scott@griepentrog.com>
+
+	* contrib: script install_prereq should install sqlite3
+
+	  Asterisk needs the sqlite 3 library, which is package
+	  sqlite-devel in CentOS. By adding this package to the
+	  script, a problem with configure failing is resolved.
+
+	  ASTERISK-25331 #close
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I90efaf6a01914fea03f21e5cdbd91c348f44b0ec
+
+2015-08-18 15:07 +0000 [03eb6cbc10]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_ari_events: Fix shutdown ref leak.
+
+	  ASTERISK-25308 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: I592785bf70ff4b63d00e535b482f40da8e82a082
+
+2015-08-18 14:24 +0000 [e1e7e205bc]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_http_websocket.c: Add missing unref on an off nominal path.
+
+	  Change-Id: I228df6adecd4cb450d03e09e9a38c86bb566e811
+
+2015-08-18 16:06 +0000 [59253a2262]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_http_websocket.c: Fix some off nominal path cleanup.
+
+	  * Remove extraneous unlock on off-nominal path.
+	  * Add missing HTTP error reply.
+
+	  Change-Id: I1f402bfe448fba8696b507477cab5f060ccd9b2b
+
+2015-08-18 14:46 +0000 [1f0a9f8a76]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_ari.c: Add missing off nominal unlock and remove a RAII_VAR().
+
+	  Change-Id: I0c5e7b34057f26dadb39489c4dac3015c52f5dbf
+
+2015-08-14 12:55 +0000 [9fb4a96e15]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Fix setting QUEUE_MEMBER 'paused' and 'ringinuse'.
+
+	  Setting the 'paused' and 'ringinuse' options on a queue member using the
+	  dialplan function QUEUE_MEMBER did not behave the same way as the
+	  equivalent dialplan applications or AMI actions.
+
+	  * Made queue_function_mem_write() call the set_member_paused() and
+	  set_member_value() for the 'paused' and 'ringinuse' options respectively.
+	  A beneficial side effect is that the queue name is now optional and sets
+	  the value in all queues the interface is a member.
+
+	  * Update QUEUE_MEMBER XML documentation.
+
+	  * Fix error checking in QUEUE_MEMBER() write.
+
+	  ASTERISK-25215 #close
+	  Reported by: Lorne Gaetz
+
+	  Change-Id: I3a016be8dc94d63a9cc155295ff9c9afa5f707cb
+
+2015-08-17 16:41 +0000 [87b22969a4]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Extract some functions for simpler code.
+
+	  * Extract set_queue_member_pause() from set_member_paused() for simpler
+	  and more consistent code.
+
+	  * Extract set_queue_member_ringinuse() from
+	  set_member_ringinuse_help_members() for simpler code.
+
+	  Change-Id: Iecc1f4119c63347341d7ea6b65f5fc4963706306
+
+2015-08-17 13:34 +0000 [5cf98e2459]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_queue.c: Fix error checking in QUEUE_MEMBER() read.
+
+	  Change-Id: I7294e13d27875851c2f4ef6818adba507509d224
+
+2015-08-17 11:00 +0000 [178e1adffb]  Scott Griepentrog <scott@griepentrog.com>
+
+	* CHAOS: prevent sorcery object with null id
+
+	  When allocating a sorcery object, fail if the
+	  id value was not allocated.
+
+	  ASTERISK-25323
+	  Reported by: Scott Griepentrog
+
+	  Change-Id: I152133fb7545a4efcf7a0080ada77332d038669e
+
+2015-08-14 15:46 +0000 [5a85711568]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_sdp_rtp: Restore removed NULL check.
+
+	  When sending an RTP keepalive, we need to be sure we're not dealing with
+	  a NULL RTP instance. There had been a NULL check, but the commit that
+	  added the rtp_timeout and rtp_hold_timeout options removed the NULL
+	  check.
+
+	  Change-Id: I2d7dcd5022697cfc6bf3d9e19245419078e79b64
+
+2015-08-13 12:30 +0000 [7c4cb8618d]  Richard Mudgett <rmudgett@digium.com>
+
+	* audiohook.c: Simplify variable usage in audiohook_read_frame_both().
+
+	  Change-Id: I58bed58631a94295b267991c5b61a3a93c167f0c
+
+2015-08-13 12:22 +0000 [bb37473234]  Richard Mudgett <rmudgett@digium.com>
+
+	* audiohook.c: Fix MixMonitor crash when using the r() or t() options.
+
+	  The built frame format in audiohook_read_frame_both() is now set to a
+	  signed linear format before the rx and tx frames are duplicated instead of
+	  only for the mixed audio frame duplication.
+
+	  ASTERISK-25322 #close
+	  Reported by Sean Pimental
+
+	  Change-Id: I86f85b5c48c49e4e2d3b770797b9d484250a1538
+
+2015-08-12 12:59 +0000 [43bdddfc26]  Kevin Harwell <kharwell@digium.com>
+
+	* chan_sip.c: wrong peer searched in sip_report_security_event
+
+	  In chan_sip, after handling an incoming invite a security event is raised
+	  describing authorization (success, failure, etc...). However, it was doing
+	  a lookup of the peer by extension. This is fine for register messages, but
+	  in the case of an invite it may search and find the wrong peer, or a non
+	  existent one (for instance, in the case of call pickup). Also, if the peers
+	  are configured through realtime this may cause an unnecessary database lookup
+	  when caching is enabled.
+
+	  This patch makes it so that sip_report_security_event searches by IP address
+	  when looking for a peer instead of by extension after an invite is processed.
+
+	  ASTERISK-25320 #close
+
+	  Change-Id: I9b3f11549efb475b6561c64f0e6da1a481d98bc4
+2015-08-13 05:26 +0000 [495dfb24b7]  Joshua Colp <jcolp@digium.com>
+
+	* res_http_websocket: When shutting down a session don't close closed socket
+
+	  Due to the use of ast_websocket_close in session termination it is
+	  possible for the underlying socket to already be closed when the
+	  session is terminated. This occurs when the close frame is attempted
+	  to be written out but fails.
+
+	  Change-Id: I7572583529a42a7dc911ea77a974d8307d5c0c8b
+2015-08-11 05:24 +0000 [7e65be4ecd]  Joshua Colp <jcolp@digium.com>
+
+	* res_http_websocket: Forcefully terminate on write errors.
+
+	  The res_http_websocket module will currently attempt to close
+	  the WebSocket connection if fatal cases occur, such as when
+	  attempting to write out data and being unable to. When the
+	  fatal cases occur the code attempts to write a WebSocket close
+	  frame out to have the remote side close the connection. If
+	  writing this fails then the connection is not terminated.
+
+	  This change forcefully terminates the connection if the
+	  WebSocket is to be closed but is unable to send the close frame.
+
+	  ASTERISK-25312 #close
+
+	  Change-Id: I10973086671cc192a76424060d9ec8e688602845
+
+2015-08-09 18:42 +0000 [a87e2dd254]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_format_attr_silk: Expose format attributes to other modules
+
+	  This patch adds the .get callback to the format attribute module, such
+	  that the Asterisk core or other third party modules can query for the
+	  negotiated format attributes.
+
+	  Change-Id: Ia24f55cf9b661d651ce89b4f4b023d921380f19c
+
+2015-08-10 13:43 +0000 [87c92d2aee]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi.c: Flush the DAHDI write buffer after starting DTMF.
+
+	  Pressing DTMF digits on a phone to go out on a DAHDI channel can result in
+	  the digit not being recognized or even heard by the peer.
+
+	  Phone -> Asterisk -> DAHDI/channel
+
+	  Turns out the DAHDI behavior with DTMF generation (and any other generated
+	  tones) is exposed by the "buffers=" setting in chan_dahdi.conf.  When
+	  Asterisk requests to start sending DTMF then DAHDI waits until its write
+	  buffer is empty before generating any samples for the DTMF tones.  When
+	  Asterisk subsequently requests DAHDI to stop sending DTMF then DAHDI
+	  immediately stops generating the DTMF samples.  As a result, the more
+	  samples there are in the DAHDI write buffer the shorter the time DTMF
+	  actually gets sent on the wire.  If there are more samples in the write
+	  buffer than the time DTMF is supposed to be sent then no DTMF gets sent on
+	  the wire.  With the "buffers=12,half" setting and each buffer representing
+	  20 ms of samples then the DAHDI write buffer is going to contain around
+	  120 ms of samples.  For DTMF to be recognized by the peer the actual sent
+	  DTMF duration needs to be a minimum of 40 ms.  Therefore, the intended
+	  duration needs to be a minimum of 160 ms for the peer to receive the
+	  minimum DTMF digit duration to recognize it.
+
+	  A simple and effective solution to work around the DAHDI behavior is for
+	  Asterisk to flush the DAHDI write buffer when sending DTMF so the full
+	  duration of DTMF is actually sent on the wire.  When someone is going to
+	  send DTMF they are not likely to be talking before sending the tones so
+	  the flushed write samples are expected to just contain silence.
+
+	  * Made dahdi_digit_begin() flush the DAHDI write buffer after requesting
+	  to send a DTMF digit.
+
+	  ASTERISK-25315 #close
+	  Reported by John Hardin
+
+	  Change-Id: Ib56262c708cb7858082156bfc70ebd0a220efa6a
+
+2015-08-05 14:21 +0000 [b9b957d4e9]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi.c: Lock private struct for ast_write().
+
+	  There is a window of opportunity for DTMF to not go out if an audio frame
+	  is in the process of being written to DAHDI while another thread starts
+	  sending DTMF.  The thread sending the audio frame could be past the
+	  currently dialing check before being preempted by another thread starting
+	  a DTMF generation request.  When the thread sending the audio frame
+	  resumes it will then cause DAHDI to stop the DTMF tone generation.  The
+	  result is no DTMF goes out.
+
+	  * Made dahdi_write() lock the private struct before writing to the DAHDI
+	  file descriptor.
+
+	  ASTERISK-25315
+	  Reported by John Hardin
+
+	  Change-Id: Ib4e0264cf63305ed5da701188447668e72ec9abb
+
+2015-08-10 18:23 +0000 [f3f5b45d57]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.c: Fix crash from corrupt saved SUBSCRIBE message.
+
+	  If the saved SUBSCRIBE message is not parseable for whatever reason then
+	  Asterisk could crash when libpjsip tries to parse the message and adds an
+	  error message to the parse error list.
+
+	  * Made ast_sip_create_rdata() initialize the parse error rdata list.  The
+	  list is checked after parsing to see that it remains empty for the
+	  function to return successful.
+
+	  ASTERISK-25306
+	  Reported by Mark Michelson
+
+	  Change-Id: Ie0677f69f707503b1a37df18723bd59418085256
+
+2015-08-10 07:40 +0000 [991d4da1eb]  Alexander Traud <pabstraud@compuserve.com>
+
+	* chan_sip: Fix negotiation of iLBC 30.
+
+	  iLBC 20 was advertised in a SIP/SDP negotiation. However, only iLBC 30 is
+	  supported. Removes "a=fmtp:x mode=y" from SDP. Because of RFC 3952 section 5,
+	  only iLBC 30 is negotiated now.
+
+	  ASTERISK-25309 #close
+
+	  Change-Id: I92d724600a183eec3114da0ac607b994b1a793da
+
+2015-08-09 17:56 +0000 [e188192ad1]  Matt Jordan <mjordan@digium.com>
+
+	* main/format: Add an API call for retrieving format attributes
+
+	  Some codecs that may be a third party library to Asterisk need to have
+	  knowledge of the format attributes that were negotiated. Unfortunately,
+	  when the great format migration of Asterisk 13 occurred, that ability
+	  was lost.
+
+	  This patch adds an API call, ast_format_attribute_get, to the core
+	  format API, along with updates to the unit test to check the new API
+	  call. A new callback is also now available for format attribute modules,
+	  such that they can provide the format attribute values they manage.
+
+	  Note that the API returns a void *. This is done as the format attribute
+	  modules themselves may store format attributes in any particular manner
+	  they like. Care should be taken by consumers of the API to check the
+	  return value before casting and dereferencing. Consumers will obviously
+	  need to have a priori knowledge of the type of the format attribute as
+	  well.
+
+	  Change-Id: Ieec76883dfb46ecd7aff3dc81a52c81f4dc1b9e3
+
+2015-08-07 22:11 +0000 [d5f0c27122]  David M. Lee <dlee@respoke.io>
+
+	* Replace htobe64 with htonll
+
+	  We don't have a compatability function to fill in a missing htobe64; but
+	  we already have one for the identical htonll.
+
+	  Change-Id: Ic0a95db1c5b0041e14e6b127432fb533b97e4cac
+
+2015-07-24 17:04 +0000 [40caf0ad9b]  David M. Lee <dlee@respoke.io>
+
+	* Replaces clock_gettime() with ast_tsnow()
+
+	  clock_gettime() is, unfortunately, not portable. But I did like that
+	  over our usual `ts.tv_nsec = tv.tv_usec * 1000` copy/paste code we
+	  usually do when we want a timespec and all we have is ast_tvnow().
+
+	  This patch adds ast_tsnow(), which mimics ast_tvnow(), but returns a
+	  timespec. If clock_gettime() is available, it will use that. Otherwise
+	  ast_tsnow() falls back to using ast_tvnow().
+
+	  Change-Id: Ibb1ee67ccf4826b9b76d5a5eb62e90b29b6c456e
+
+2015-08-07 14:20 +0000 [12e6f5ac01]  Scott Emidy <jemidy@digium.com>
+
+	* ARI: Retrieve existing log channels
+
+	  An http request can be sent to get the existing Asterisk logs.
+
+	  The command "curl -v -u user:pass -X GET 'http://localhost:8088
+	  /ari/asterisk/logging'" can be run in the terminal to access the
+	  newly implemented functionality.
+
+	  * Retrieve all existing log channels
+
+	  ASTERISK-25252
+
+	  Change-Id: I7bb08b93e3b938c991f3f56cc5d188654768a808
+
+2015-08-07 11:14 +0000 [b91ca7ba49]  Scott Emidy <jemidy@digium.com>
+
+	* ARI: Creating log channels
+
+	  An http request can be sent to create a log channel
+	  in Asterisk.
+
+	  The command "curl -v -u user:pass -X POST
+	  'http://localhost:088/ari/asterisk/logging/mylog?
+	  configuration=notice,warning'" can be run in the terminal
+	  to access the newly implemented functionality for ARI.
+
+	  * Ability to create log channels using ARI
+
+	  ASTERISK-25252
+
+	  Change-Id: I9a20e5c75716dfbb6b62fd3474faf55be20bd782
+
+2015-08-06 15:18 +0000 [f19c4930c2]  Scott Emidy <jemidy@digium.com>
+
+	* ARI: Deleting log channels
+
+	  An http request can be sent to delete a log channel
+	  in Asterisk.
+
+	  The command "curl -v -u user:pass -X DELETE 'http://localhost:8088
+	  /ari/asterisk/logging/mylog'" can be run in the terminal
+	  to access the newly implemented functionally for ARI.
+
+	  * Able to delete log channels using ARI
+
+	  ASTERISK-25252
+
+	  Change-Id: Id6eeb54ebcc511595f0418d586ff55914bc3aae6
+
+2015-08-06 12:48 +0000 [382334cc06]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: More accurately persist packet.
+
+	  The pjsip_rx_data structure has a pkt_info.packet field on it that is
+	  the packet that was read from the transport. For datagram transports,
+	  the packet read from the transport will correspond to the SIP message
+	  that arrived. For streamed transports, however, it is possible to read
+	  multiple SIP messages in one packet.
+
+	  In a recent case, Asterisk crashed on a system where TCP was being used.
+	  This is because at some point, a read from the TCP socket resulted in a
+	  200 OK response as well as an incoming SUBSCRIBE request being stored in
+	  rdata->pkt_info.packet. When the SUBSCRIBE was processed, the
+	  combination 200 OK and SUBSCRIBE was saved in persistent storage. Later,
+	  a restart of Asterisk resulted in the crash because the persistent
+	  subscription recreation code ended up building the 200 OK response
+	  instead of a SUBSCRIBE request, and we attempted to access
+	  request-specific data.
+
+	  The fix here is to use the pjsip_msg_print() function in order to
+	  persist SUBSCRIBE requests. This way, rather than using the raw socket
+	  data, we use the parsed SIP message that PJSIP has given us. If we
+	  receive multiple SIP messages from a single read, we will be sure only
+	  to save off the relevant SIP message. There also is a safeguard put in
+	  place to make sure that if we do end up reconstructing a SIP response,
+	  it will not cause a crash.
+
+	  ASTERISK-25306 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I4bf16f7b76a2541d10b55de82bcd14c6e542afb2
+
+2015-08-04 16:12 +0000 [4b6c657a82]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip: Ensure sanitized XML is NULL terminated.
+
+	  The ast_sip_sanitize_xml function is used to sanitize
+	  a string for placement into XML. This is done by examining
+	  an input string and then appending values to an output
+	  buffer. The function used by its implementation, strncat,
+	  has specific behavior that was not taken into account.
+	  If the size of the input string exceeded the available
+	  output buffer size it was possible for the sanitization
+	  function to write past the output buffer itself causing
+	  a crash. The crash would either occur because it was
+	  writing into memory it shouldn't be or because the resulting
+	  string was not NULL terminated.
+
+	  This change keeps count of how much remaining space is
+	  available in the output buffer for text and only allows
+	  strncat to use that amount.
+
+	  Since this was exposed by the res_pjsip_pidf_digium_body_supplement
+	  module attempting to send a large message the maximum allowed
+	  message size has also been increased in it.
+
+	  A unit test has also been added which confirms that the
+	  ast_sip_sanitize_xml function is providing NULL terminated
+	  output even when the input length exceeds the output
+	  buffer size.
+
+	  ASTERISK-25304 #close
+
+	  Change-Id: I743dd9809d3e13d722df1b0509dfe34621398302
+
+2015-08-05 05:23 +0000 [7351d33a1f]  Joshua Colp <jcolp@digium.com>
+
+	* res_rtp_asterisk: Don't leak temporary key when enabling PFS.
+
+	  A change recently went in which enabled perfect forward secrecy for
+	  DTLS in res_rtp_asterisk. This was accomplished two different ways
+	  depending on the availability of a feature in OpenSSL. The fallback
+	  method created a temporary instance of a key but did not free it.
+	  This change fixes that.
+
+	  ASTERISK-25265
+
+	  Change-Id: Iadc031b67a91410bbefb17ffb4218d615d051396
+2015-08-04 09:47 +0000 [c63316eec1]  Mark Michelson <mmichelson@digium.com>
+
+	* res_http_websocket: Debug write lengths.
+
+	  Commit 39cc28f6ea2140ad6d561fd4c9e9a66f065cecee attempted to fix a
+	  test failure observed on 32 bit test agents by ensuring that a cast from
+	  a 32 bit unsigned integer to a 64 bit unsigned integer was happening in
+	  a predictable place. As it turns out, this did not cause test runs to
+	  succeed.
+
+	  This commit adds several redundant debug messages that print the payload
+	  lengths of websocket frames. The idea here is that this commit will not
+	  cause tests to succeed for the faulty test agent, but we might deduce
+	  where the fault lies more easily this way by observing at what point the
+	  expected value (537) changes to some ungangly huge number.
+
+	  If you are wondering why something like this is being committed to the
+	  branch, keep in mind that in commit
+	  39cc28f6ea2140ad6d561fd4c9e9a66f065cecee I noted that the observed test
+	  failures only happen when automated tests are run. Attempts to run the
+	  tests by hand manually on the test agent result in the tests passing.
+
+	  Change-Id: I14a65c19d8af40dadcdbd52348de3b0016e1ae8d
+
+2015-08-03 11:06 +0000 [35a98161df]  Mark Michelson <mmichelson@digium.com>
+
+	* res_http_websocket: Avoid passing strlen() to ast_websocket_write().
+
+	  We have seen a rash of test failures on a 32-bit build agent. Commit
+	  48698a5e21d7307f61b5fb2bd39fd593bc1423ca solved an obvious problem where
+	  we were not encoding a 64-bit value correctly over the wire. This
+	  commit, however, did not solve the test failures.
+
+	  In the failing tests, ARI is attempting to send a 537 byte text frame
+	  over a websocket. When sending a frame this small, 16 bits are all that
+	  is required in order to encode the payload length on the websocket
+	  frame. However, ast_websocket_write() thinks that the payload length is
+	  greater than 65535 and therefore writes out a 64 bit payload length.
+	  Inspecting this payload length, the lower 32 bits are exactly what we
+	  would expect it to be, 537 in hex. The upper 32 bits, are junk values
+	  that are not expected to be there.
+
+	  In the failure, we are passing the result of strlen() to a function that
+	  expects a uint64_t parameter to be passed in. strlen() returns a size_t,
+	  which on this 32-bit machine is 32 bits wide. Normally, passing a 32-bit
+	  unsigned value to somewhere where a 64-bit unsigned value is expected
+	  would cause no problems. In fact, in manual runs of failing tests, this
+	  works just fine. However, ast_websocket_write() uses the Asterisk
+	  optional API, which means that rather than a simple function call, there
+	  are a series of macros that are used for its declaration and
+	  implementation. These macros may be causing some sort of error to occur
+	  when converting from a 32 bit quantity to a 64 bit quantity.
+
+	  This commit changes the logic by making existing ast_websocket_write()
+	  calls use ast_websocket_write_string() instead. Within
+	  ast_websocket_write_string(), the 64-bit converted strlen is saved in a
+	  local variable, and that variable is passed to ast_websocket_write()
+	  instead.
+
+	  Note that this commit message is full of speculation rather than
+	  certainty. This is because the observed test failures, while always
+	  present in automated test runs, never occur when tests are manually
+	  attempted on the same test agent. The idea behind this commit is to fix
+	  a theoretical issue by performing changes that should, at the least,
+	  cause no harm. If it turns out that this change does not fix the failing
+	  tests, then this commit should be reverted.
+
+	  Change-Id: I4458dd87d785ca322b89c152b223a540a3d23e67
+
+2015-07-29 14:17 +0000 [1f02d20da4]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Rotate log channels.
+
+	  An http request can be sent to rotate a specified log channel.
+	  If the channel does not exist, an error response will be
+	  returned.
+
+	  The command "curl -v -u user:pass -X PUT 'http://localhost:8088
+	  /ari/asterisk/logging/logChannelName/rotate'" can be run in the
+	  terminal to access this new functionality.
+
+	  * Added the ability to rotate log files through ARI
+
+	  ASTERISK-25252
+
+	  Change-Id: Iaefa21cbbc1b29effb33004ee3d89c977e76ab01
+
+2015-07-31 11:27 +0000 [fe804b09b3]  Ashley Sanders <asanders@digium.com>
+
+	* ARI: Channels added to Stasis application during WebSocket creation ...
+
+	  Prior to ASTERISK-24988, the WebSocket handshake was resolved before Stasis
+	  applications were registered. This was done such that the WebSocket would be
+	  ready when an application is registered. However, by creating the WebSocket
+	  first, the client had the ability to make requests for the Stasis application
+	  it thought had been created with the initial handshake request. The inevitable
+	  conclusion of this scenario was the cart being put before the horse.
+
+	  ASTERISK-24988 resolved half of the problem by ensuring that the applications
+	  were created and registered with Stasis prior to completing the handshake
+	  with the client. While this meant that Stasis was ready when the client
+	  received the green-light from Asterisk, it also meant that the WebSocket was
+	  not yet ready for Stasis to dispatch messages.
+
+	  This patch introduces a message queuing mechanism for delaying messages from
+	  Stasis applications while the WebSocket is being constructed. When the ARI
+	  event processor receives the message from the WebSocket that it is being
+	  created, the event processor instantiates an event session which contains a
+	  message queue. It then tries to create and register the requested applications
+	  with Stasis. Messages that are dispatched from Stasis between this point and
+	  the point at which the event processor is notified the WebSocket is ready, are
+	  stashed in the queue. Once the WebSocket has been built, the queue's messages
+	  are dispatched in the order in which they were originally received and the
+	  queue is concurrently cleared.
+
+	  ASTERISK-25181 #close
+	  Reported By: Matt Jordan
+
+	  Change-Id: Iafef7b85a2e0bf78c114db4c87ffc3d16d671a17
+
+2015-07-29 12:58 +0000 [86034227ca]  Mark Michelson <mmichelson@digium.com>
+
+	* dns_core: Allow zero-length DNS responses.
+
+	  A testsuite test recently failed due to a crash that occurred in the DNS
+	  core. The problem was that the test could not resolve an address, did
+	  not set a result on the DNS query, and then indicated the query was
+	  completed. The DNS core does not handle the case of a query with no
+	  result gracefully, and so there is a crash.
+
+	  This changeset makes the DNS system resolver set a result with a
+	  zero-length answer in the case that a DNS resolution failure occurs
+	  early. The DNS core now also will accept such a response without
+	  treating it as invalid input. A unit test was updated to no longer treat
+	  setting a zero-length response as off-nominal.
+
+	  Change-Id: Ie56641e22debdaa61459e1c9a042e23b78affbf6
+
+2015-07-29 13:49 +0000 [f49bef08a2]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Fix performance issue with several channel drivers that use RTP.
+
+	  ast_rtp_codecs_get_payload() gets called once or twice for every received
+	  RTP frame so it would be nice to not allocate an ao2 object to then have
+	  it destroyed shortly thereafter.  The ao2 object gets allocated only if
+	  the payload type is not set by the channel driver as a negotiated value.
+	  The issue affects chan_skinny, chan_unistim, chan_rtp, and chan_ooh323.
+
+	  * Made static_RTP_PT[] an array of ao2 objects that
+	  ast_rtp_codecs_get_payload() can return instead of an array of structs
+	  that must be copied into a created ao2 object.
+
+	  ASTERISK-25296 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Icb6de5cd90bfae07d44403a1352963db9109dac0
+
+2015-07-29 17:00 +0000 [33a465249b]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_rtp_asterisk.c: Fix off-nominal crash potential.
+
+	  ASTERISK-25296
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I08549fb7c3ab40a559f41a3940f3732a4059b55b
+
+2015-07-29 13:48 +0000 [5f925d48b7]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Must protect mime_types_len with mime_types_lock.
+
+	  Change-Id: I44220dd369cc151ebf5281d5119d84bb9e54d54e
+
+2015-07-24 18:38 +0000 [ba7dd38470]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_sdp_rtp.c: Fixup some whitespace.
+
+	  Change-Id: Ib4eb7ef7dcaf93ddc26538f0a498aaf110d7a973
+
+2015-07-24 18:42 +0000 [3751bf0971]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_sdp_rtp.c: Fix processing wrong SDP media list.
+
+	  Change-Id: I7c076826c2d3c6ae8c923ca73b7a71980cca11f2
+
+2015-07-27 19:10 +0000 [e2d5d4db35]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.h: No sense allowing payload types larger than RFC allows.
+
+	  * Tweaked add_static_payload() to not use magic numbers.
+
+	  Change-Id: I1719ff0f6d3ce537a91572501eae5bcd912a420b
+
+2015-07-23 14:04 +0000 [bc1eae55cb]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.c: Minor tweaks.
+
+	  * Fix off nominial ref leak of new_type in
+	  ast_rtp_codecs_payloads_set_m_type().
+
+	  * No need to lock static_RTP_PT_lock in
+	  ast_rtp_codecs_payloads_set_m_type() and
+	  ast_rtp_codecs_payloads_set_rtpmap_type_rate() before the payload type
+	  parameter sanity check.
+
+	  * No need to create ast_rtp_payload_type ao2 objects with a lock since the
+	  lock is not used.
+
+	  Change-Id: I64dd1bb4dfabdc7e981e3f61448beac9bb7504d4
+
+2015-07-17 16:23 +0000 [d122c1e50b]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_sip.c: Tweak glue->update_peer() parameter nil value.
+
+	  Change glue->update_peer() parameter from 0 to NULL to better indicate it
+	  is a pointer.
+
+	  Change-Id: I8ff2e5087f0e19f6998e3488a712a2470cc823bd
+
+2015-07-23 12:41 +0000 [d12dc97fc9]  Richard Mudgett <rmudgett@digium.com>
+
+	* rtp_engine.h: Misc comment fixes.
+
+	  Change-Id: If98139264d5d97427b4685ecbdc54518f725bc43
+
+2015-07-30 17:05 +0000 [077c58cd5c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Fix crashes seen when call cancelled.
+
+	  Two testsuite tests crashed in the same place as a result of an INVITE
+	  being CANCELed.
+
+	  tests/channels/pjsip/resolver/srv/failover/in_dialog/transport_unspecified
+	  tests/channels/pjsip/resolver/srv/failover/in_dialog/transport_tcp
+
+	  The session pointer is no longer in the inv->mod_data[session_module.id]
+	  location because the INVITE transaction has reached the terminated state.
+
+	  ASTERISK-25297 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Idb75fdca0321f5447d5dac737a632a5f03614427
+
+2015-07-29 14:35 +0000 [5fcd1bc556]  Mark Michelson <mmichelson@digium.com>
+
+	* res_http_websocket: Properly encode 64 bit payload
+
+	  A test agent was continuously failing all ARI tests when run against
+	  Asterisk 13. As it turns out, the reason for this is that on those test
+	  runs, for some reason we decided to use the super extended 64 bit
+	  payload length for websocket text frames instead of the extended 16 bit
+	  payload length. For 64-bit payloads, the expected byte order over the
+	  network is
+
+	  7, 6, 5, 4, 3, 2, 1, 0
+
+	  However, we were sending the payload as
+
+	  3, 2, 1, 0, 7, 6, 5, 4
+
+	  This meant that we were saying to expect an absolutely MASSIVE payload
+	  to arrive. Since we did not follow through on this expected payload
+	  size, the client would sit patiently waiting for the rest of the payload
+	  to arrive until the test would time out.
+
+	  With this change, we use the htobe64() function instead of htonl() so
+	  that a 64-bit byte-swap is performed instead of a 32 bit byte-swap.
+
+	  Change-Id: Ibcd8552392845fbcdd017a8c8c1043b7fe35964a
+
+2015-07-29 12:23 +0000 [8fb8988fd4]  Mark Michelson <mmichelson@digium.com>
+
+	* Add a test event for inband ringing.
+
+	  This event is necessary for the bridge_wait_e_options test to be able to
+	  confirm that ringing is being played on the local channel that runs the
+	  BridgeWait() application with the e(r) option.
+
+	  ASTERISK-25292 #close
+	  Reported by Kevin Harwell
+
+	  Change-Id: Ifd3d3d2bebc73344d4b5310d0d55c7675359d72e
+
+2015-07-28 05:33 +0000 [1d081ec970]  Mark Duncan <mark@syon.co.jp>
+
+	* res/res_rtp_asterisk: Add ECDH support
+
+	  This will add ECDH support to Asterisk. It will
+	  detect auto ECDH support in OpenSSL
+	  (1.0.2b and above) during ./configure. If this is
+	  available, it will use it,
+	  otherwise it will fall back to prime256v1 (this
+	  behavior is consistent with
+	  other projects such as Apache and nginx).
+
+	  This fixes WebRTC being broken in Firefox 38+ due
+	  to Firefox now only supporting
+	  ciphers with perfect forward secrecy.
+
+	  ASTERISK-25265 #close
+
+	  Change-Id: I8c13b33a2a79c0bde2e69e4ba6afa5ab9351465b
+
+2015-07-16 12:16 +0000 [687597ca8c]  Jonathan Rose <jrose@digium.com>
+
+	* holding_bridge: ensure moh participants get frames
+
+	  Currently, if a blank musiconhold.conf is used, musiconhold will fail
+	  to start for a channel going into a holding bridge with an anticipation
+	  of getting music on hold. That being the case, no frames will be written
+	  to the channel and that can pose a problem for blind transfers in PJSIP
+	  which may rely on frames being written to get past the REFER framehook.
+	  This patch makes holding bridges start a silence generator if starting
+	  music on hold fails and makes it so that if no music on hold functions
+	  are installed that the ast_moh_start function will report a failure so
+	  that consumers of that function will be able to respond appropriately.
+
+	  ASTERISK-25271 #close
+
+	  Change-Id: I06f066728604943cba0bb0b39fa7cf658a21cd99
+	  (cherry picked from commit 8458b8d441c2f4143ff135163ff3da4f88fe14c8)
+
+2015-07-18 11:16 +0000 [309dd2a409]  Joshua Colp <jcolp@digium.com>
+
+	* pjsip: Add rtp_timeout and rtp_timeout_hold endpoint options.
+
+	  This change adds support for the 'rtp_timeout' and 'rtp_timeout_hold'
+	  endpoint options. These allow the channel to be hung up if RTP
+	  is not received from the remote endpoint for a specified number of
+	  seconds.
+
+	  ASTERISK-25259 #close
+
+	  Change-Id: I3f39daaa7da2596b5022737b77799d16204175b9
+
+2015-07-24 09:46 +0000 [a0c31c7a05]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Add rtp_keepalive to sample config file.
+
+	  Change-Id: I5f62d0c5684f8b2335f9f8ac2d79ee04fbdafb19
+
+2015-07-23 13:11 +0000 [d97bed46b7]  Mark Michelson <mmichelson@digium.com>
+
+	* Local channels: Alternate solution to ringback problem.
+
+	  Commit 54b25c80c8387aea9eb20f9f4f077486cbdf3e5d solved an issue where a
+	  specific scenario involving local channels and a native local RTP bridge
+	  could result in ringback still being heard on a calling channel even
+	  after the call is bridged.
+
+	  That commit caused many tests in the testsuite to fail with alarming
+	  consequences, such as not sending DialBegin and DialEnd events, and
+	  giving incorrect hangup causes during calls.
+
+	  This commit reverts the previous commit and implements and alternate
+	  solution. This new solution involves only passing AST_CONTROL_RINGING
+	  frames across local channels if the local channel is in AST_STATE_RING.
+	  Otherwise, the frame does not traverse the local channels. By doing
+	  this, we can ensure that a playtones generator does not get started on
+	  the calling channel but rather is started on the local channel on which
+	  the ringing frame was initially indicated.
+
+	  ASTERISK-25250 #close
+	  Reported by Etienne Lessard
+
+	  Change-Id: I3bc87a18a38eb2b68064f732d098edceb5c19f39
+
+2015-07-22 12:24 +0000 [1cc99ba8b6]  Joshua Colp <jcolp@digium.com>
+
+	* audiohook: Use manipulated frame instead of dropping it.
+
+	  Previous changes to sample rate support in audiohooks accidentally
+	  removed code responsible for allowing the manipulate audiohooks
+	  to work. Without this code the manipulated frame would be dropped
+	  and not used. This change restores it.
+
+	  ASTERISK-25253 #close
+
+	  Change-Id: I3ff50664cd82faac8941f976fcdcb3918a50fe13
+
+2015-07-22 09:46 +0000 [0b7148e262]  Mark Michelson <mmichelson@digium.com>
+
+	* Local channels: Do not block control -1 payloads.
+
+	  Control frames with a -1 payload are used as a special signal to stop
+	  playtones generators on channels. This indication is sent both by
+	  app_dial as well as by ast_answer() when a call is answered in case any
+	  tones were being generated on a calling channel.
+
+	  This control frame type was made to stop traversing local channel pairs
+	  as an optimization, because it was thought that it was unnecessary to
+	  send these indications, and allowing such unnecessary control frames to
+	  traverse the local channels would cause the local channels to optimize
+	  away less quickly.
+
+	  As it turns out, through some special magic dialplan code, it is
+	  possible to have a tones being played on a non-local channel, and it is
+	  important for the local channel to convey that the tones should be
+	  stopped. The result of having tones continue to be played on the
+	  non-local channel is that the tones play even once the channel has been
+	  bridged. By not blocking the -1 control frame type, we can ensure that
+	  this situation does not happen.
+
+	  ASTERISK-25250 #close
+	  Reported by Etienne Lessard
+
+	  Change-Id: I0bcaac3d70b619afdbd0ca8a8dd708f33fd2f815
+
+2015-07-22 05:16 +0000 [e5fe8d40c8]  Joshua Colp <jcolp@digium.com>
+
+	* audiohook: Read the correct number of samples based on audiohook format.
+
+	  Due to changes in audiohooks to support different sample rates the
+	  underlying storage of samples is in the format of the audiohook
+	  itself and not of the format being requested. This means that if a
+	  channel is using G722 the samples stored will be at 16kHz. If
+	  something subsequently reads from the audiohook at a format which
+	  is not the same sample rate as the audiohook the number of samples
+	  needs to be adjusted.
+
+	  Given the following example:
+	  1. Channel writing into audiohook at 16kHz (as it is using G722).
+	  2. Chanspy reading from audiohook at 8kHz.
+
+	  The original code would read 160 samples from the audiohook for
+	  each 20ms of audio. This is incorrect. Since the audio in the
+	  audiohook is at 16kHz the actual number needing to be read is 320.
+	  Failure to read this much would cause the audiohook to reset
+	  itself constantly as the buffer became full.
+
+	  This change adjusts the requested number of samples by determining
+	  the duration of audio requested and then calculating how many
+	  samples that would be in the audiohook format.
+
+	  ASTERISK-25247 #close
+
+	  Change-Id: Ia91ce516121882387a315fd8ee116b118b90653d
+
+2015-07-20 15:59 +0000 [293c9f6894]  Elazar Broad <elazar@thebroadfamily.com>
+
+	* cdr/cdr_adaptive_odbc.c: Fix quoted identifier usage when inserting CDR records
+
+	  Commit a24ce38 added support for the use of quoted indentifiers when inserting
+	  CDR records into the database. However, the if statement logic responsible for
+	  determining whether to use those identifiers is reversed, resulting in a
+	  reference to the quoted identifier character buffer which will be null, hence
+	  null terminating the SQL query, resulting in a truncated statement which
+	  fails to execute.
+
+	  ASTERISK-25263 #close
+	  Reported by: Elazar Broad
+	  Tested by: Elazar Broad
+
+	  Change-Id: I40da47309b67cc1572207b1515dcc08ec9b1f644
+2015-07-20 12:39 +0000 [d02196448b]  Rusty Newton <rnewton@digium.com>
+
+	* Documentation: A couple of trivial fixes in sip.conf.sample and func_cdr.c
+
+	   * In sip.conf.sample fix sentence where we said that WS or WSS are supported
+	     transports for use in an outbound register definition. They are not
+	     supported in that case.
+	   * In func_cdr.c made it clear that the Disable option for CDR_PROP can be used
+	     to enable CDR on a channel.
+
+	  ASTERISK-24867 #close
+	  Reported by: Rusty Newton
+
+	  ASTERISK-24853 #close
+	  Reported by: PSDK
+
+	  Change-Id: I3d698bc6302b9d00a0a995b5c4ad9a42d69b48ca
+
+2015-07-09 14:17 +0000 [2b42264e66]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Add rtp_keepalive endpoint option.
+
+	  This adds an "rtp_keepalive" option for PJSIP endpoints. Similar to the
+	  chan_sip option, this specifies an interval, in seconds, at which we
+	  will send RTP comfort noise frames. This can be useful for keeping RTP
+	  sessions alive as well as keeping NAT associations alive during lulls.
+
+	  ASTERISK-25242 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I3b9903d99e35fe5d0b53ecc46df82c750776bc8d
+
+2015-07-16 09:13 +0000 [8b503f2a10]  Michael Cargile <mikec@vicidial.com>
+
+	* res/res_musiconhold: Add a warning when MOH does not exist
+
+	  Change-Id: Ifdfbd0b97cf31478d29923ec30aabce28d01740b
+
+2015-07-19 09:11 +0000 [9475dc9492]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_sorcery_config: Prevent crash from misconfigured sorcery.conf
+
+	  Misconfiguring sorcery.conf with a 'config' wizard with no extra data
+	  will currently crash Asterisk on startup, as the wizard requires a comma
+	  delineated list to parse. This patch updates res_sorcery_config to check
+	  for the presence of the data before it starts manipulating it.
+
+	  Change-Id: I4c97512e8258bc82abe190627a9206c28f5d3847
+
+2015-07-16 09:46 +0000 [649460aa44]  Joshua Colp <jcolp@digium.com>
+
+	* chan_pjsip: Don't change formats when frame of unsupported format is received.
+
+	  Receipt of an RTP packet currently causes the formats on an PJSIP channel to
+	  change to the format of the RTP packet. In some off-nominal cases it's possible
+	  for this to be a format that has not been configured or negotiated. This change
+	  makes it so only formats explicitly configured on the endpoint are allowed.
+
+	  ASTERISK-25258 #close
+
+	  Change-Id: If93d641fb6418a285928839300d7854cab8c1020
+
+2015-07-14 16:55 +0000 [4a875e8082]  Richard Mudgett <rmudgett@digium.com>
+
+	* pbx.c: Post AMI VarSet event if delete a non-empty dialplan variable.
+
+	  ASTERISK-25256 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I0b6be720b66fa956f6a798cd22ef8934eb0c0ff3
+
+2015-07-17 04:59 +0000 [7908ae4934]  Patric Marschall <patric.marschall@1und1.de>
+
+	* sig_pri.h: force_restart_unavailable_chans in wrong scope
+
+	  In channels/sig_pri.h, struct sig_pri_span, the field
+	  force_restart_unavailable_chans is only defined if
+
+	  #if defined(HAVE_PRI_MCID) is true.
+
+	  All other occurences of force_restart_unavailable_chans are outside of the
+
+	  #if defined(HAVE_PRI_MCID)
+	  endif
+
+	  scope.
+
+	  ASTERISK-25257 #close
+	  Reported by: Patric Marschall
+
+	  Change-Id: I071de89cc2cd0d85927a013036e235851f672549
+
+2015-07-08 16:39 +0000 [254d07b15b]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Add support for push configuration of dynamic object
+
+	  This patch adds support for push configuration of dynamic, i.e.,
+	  sorcery, objects in Asterisk. It adds three new REST API calls to the
+	  'asterisk' resource:
+	   * GET /asterisk/{configClass}/{objectType}/{id}: retrieve the current
+	     object given its ID. This returns back a list of ConfigTuples, which
+	     define the fields and their present values that make up the object.
+	   * PUT /asterisk/{configClass}/{objectType}/{id}: create or update an
+	     object. A body may be passed with the request that contains fields to
+	     populate in the object. The same format as what is retrieved using
+	     the GET operation is used for the body, save that we specify that the
+	     list of fields to update are contained in the "fields" attribute.
+	   * DELETE /asterisk/{configClass}/{objectType}/{id}: remove a dynamic
+	     object from its backing storage.
+
+	  Note that the success/failure of these operations is somewhat
+	  configuration dependent, i.e., you must be using a sorcery wizard that
+	  supports the operation in question. If a sorcery wizard does not support
+	  the create or delete mechanisms, then the REST API call will fail with a
+	  403 forbidden.
+
+	  ASTERISK-25238 #close
+
+	  Change-Id: I28cd5c7bf6f67f8e9e437ff097f8fd171d30ff5c
+
+2015-07-15 15:40 +0000 [b34c4528ab]  Richard Mudgett <rmudgett@digium.com>
+
+	* strings.h: Fix issues with escape string functions.
+
+	  Fixes for issues with the ASTERISK-24934 patch.
+
+	  * Fixed ast_escape_alloc() and ast_escape_c_alloc() if the s parameter is
+	  an empty string.  If it were an empty string the functions returned NULL
+	  as if there were a memory allocation failure.  This failure caused the AMI
+	  VarSet event to not get posted if the new value was an empty string.
+
+	  * Fixed dest buffer overwrite potential in ast_escape() and
+	  ast_escape_c().  If the dest buffer size is smaller than the space needed
+	  by the escaped s parameter string then the dest buffer would be written
+	  beyond the end by the nul string terminator.  The num parameter was really
+	  the dest buffer size parameter so I renamed it to size.
+
+	  * Made nul terminate the dest buffer if the source string parameter s was
+	  an empty string in ast_escape() and ast_escape_c().
+
+	  * Updated ast_escape() and ast_escape_c() doxygen function description
+	  comments to reflect reality.
+
+	  * Added some more unit test cases to /main/strings/escape to cover the
+	  empty source string issues.
+
+	  ASTERISK-25255 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Id77fc704600ebcce81615c1200296f74de254104
+
+2015-07-14 14:29 +0000 [097c15ac51]  Richard Mudgett <rmudgett@digium.com>
+
+	* parking_applications.c: Fix ast_verb() line terminator.
+
+	  Change-Id: I8797238c71563e243c48c6145b4f1ae58f91f775
+
+2015-07-14 14:36 +0000 [8b620c555b]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_parking: Fix crash if ATTENDEDTRANSFER set empty before Park.
+
+	  setup_park_common_datastore() was assuming that a non-NULL string returned
+	  for the ATTENDEDTRANSFER and BLINDTRANSFER channel variables are not empty
+	  strings.  Things got crashy as a result.
+
+	  * Made setup_park_common_datastore() treat the channel variable values the
+	  same whether they are NULL or empty for ATTENDEDTRANSFER and
+	  BLINDTRANSFER.
+
+	  ASTERISK-25254 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I9a9c174b33f354f35f82cc6b7cea8303adbaf9c2
+
+2015-07-10 18:01 +0000 [4af24ec74b]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Extract sip_session_defer_termination_stop_timer().
+
+	  Change-Id: I9e115dee74bd72e06081d0ee73ecdeb886caa5fb
+
+2015-07-10 10:42 +0000 [71b3bcf5e0]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Add some helpful comments and minor tweaks.
+
+	  Change-Id: I742aeeaf5f760593f323a00fb691affe22e35743
+
+2015-07-10 10:43 +0000 [53c91737a5]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Fix off nominal crash potential in debug message.
+
+	  Change-Id: I09928297927ee85f7655289acee3a586816466bc
+
+2015-07-15 10:31 +0000 [eff6a88a88]  Matt Jordan <mjordan@digium.com>
+
+	* apps/app_dictate: Fix typo in attribution
+
+	  Last time I checked, it's "Sangoma", not "Samgoma". Thanks to Brian
+	  (GameGamer43) for pointing that out.
+
+	  Change-Id: I43d7b196f6d7a2b2517b84915e3a8dfbc2894106
+
+2015-07-15 10:28 +0000 [e01d93e092]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Fixed unload mode for unload module.
+
+	  Changed the unload mode to AST_FORCE_SOFT from AST_FORCE_FIRM,
+	  which would unload a module even if it was in use.
+
+	  * Changed unload mode to proper mode
+
+	  ASTERISK-25173
+
+	  Change-Id: If2402487b5bce05d9770f25f65f5c8e292ad5533
+
+2015-07-10 18:17 +0000 [1b666549f3]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session.c: Fix crash on call disconnect.
+
+	  The crash fix for ASTERISK-25183 backported some code from master to try
+	  to make sure that a BYE response is processed by the same serializer used
+	  by the BYE request.  The identified race condition causing that backport
+	  was the BYE request code had not finished processing after sending the BYE
+	  before the BYE response came in for processing under a different thread.
+	  Unfortunately, there is still a race condition.  Now the race condition is
+	  between destroying the call session's serializer in
+	  ast_taskprocessor_unreference() and using ast_taskprocessor_get() to get a
+	  reference to the serializer for a BYE response.  Even worse, the new race
+	  condition is a design limitation of the taskprocessor implementation that
+	  didn't matter in versions before v12.  Back then, taskprocessors were only
+	  destroyed when a module unloaded.  Now res_pjsip can destroy them when a
+	  call ends.
+
+	  However, as noted on the ASTERISK-25183 commit,
+	  session_inv_on_state_changed() is disassociating the dialog from the
+	  session when the invite dialog state becomes PJSIP_INV_STATE_DISCONNECTED.
+	  This is a tad too soon because our BYE request transaction has not
+	  completed yet.
+
+	  * Split session_end() that is called by session_inv_on_state_changed() to
+	  hold off session destruction until the BYE transaction timeout occurs or a
+	  failed initial INVITE transaction timeout occurs in
+	  session_inv_on_tsx_state_changed().
+
+	  ASTERISK-25201 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: Iaf8dc8485fd8392a2a3ee4ad3b7f7f04a0dcc961
+
+2015-07-14 13:12 +0000 [9d458b8311]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Added new functionality to reload a single module.
+
+	  An http request can be sent to reload an Asterisk module. If the
+	  module can not be reloaded or is not already loaded, an error
+	  response will be returned.
+
+	  The command "curl -v -u user:pass -X PUT 'http://localhost:8088
+	  /ari/asterisk/modules/{moduleName}'" (or something similar, based
+	  on configuration) can be run in the terminal to access this new
+	  functionality.
+
+	  For more information, see:
+	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+
+	  * Added new ARI functionality
+	  * Asterisk modules can be reloaded through http requests
+
+	  ASTERISK-25173
+
+	  Change-Id: I289188bcae182b2083bdbd9ebfffd50b62f58ae1
+
+2015-07-14 08:55 +0000 [f64f1c2772]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Added new functionality to unload a single module.
+
+	  An http request can be sent to unload an Asterisk module. If the
+	  module can not be unloaded or is already unloaded, an error response
+	  will be returned.
+
+	  The command "curl -v -u user:pass -X DELETE 'http://localhost:8088
+	  /ari/asterisk/modules/{moduleName}'" (or something similar, depending
+	  on configuration) can be run in the terminal to access this new
+	  functionality.
+
+	  For more information, see:
+	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+
+	  * Added new ARI functionality
+	  * Asterisk modules can be unloaded through http requests
+
+	  ASTERISK-25173
+
+	  Change-Id: I535a95f5676deb02651522761ecbdc0b00b5ac57
+
+2015-07-13 16:00 +0000 [aa5707b889]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Added new functionality to load a single module.
+
+	  An http request can be sent to load an Asterisk module. If the
+	  module can not be loaded or is loaded already, an error response
+	  will be returned.
+
+	  The command curl -v -u user:pass -X POST 'http://localhost:8088/ari
+	  /asterisk/modules/{moduleName}'" (or something similar, depending on
+	  configuration) can be run in the terminal to access this new
+	  functionality.
+
+	  For more information, see:
+	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+
+	  * Added new ARI functionality
+	  * Asterisk modules can be loaded through http requests
+
+	  ASTERISK-25173
+
+	  Change-Id: I9e05d5b8c5c666ecfef341504f9edc1aa84fda33
+
+2015-07-13 10:54 +0000 [6a764db370]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Added new functionality to get information on a single module.
+
+	  An http request can be sent to retrieve information on a single
+	  module, including the resource name, description, use count, status,
+	  and support level.
+
+	  The command "curl -v -u user:pass -X GET 'http://localhost:8088/ari
+	  /asterisk/modules/{moduleName}'" (or something similar, depending on
+	  configuration) can be run in the terminal to access this new
+	  functionality.
+
+	  For more information, see:
+	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+
+	  * Added new ARI functionality
+	  * Information on a single module can now be retrieved
+
+	  ASTERISK-25173
+
+	  Change-Id: Ibce5a94e70ecdf4e90329cf0ba66c33a62d37463
+
+2015-07-08 14:56 +0000 [c855523519]  Kevin Harwell <kharwell@digium.com>
+
+	* bridge.c: Fixed race condition during attended transfer
+
+	  During an attended transfer a thread is started that handles imparting the
+	  bridge channel. From the start of the thread to when the bridge channel is
+	  ready exists a gap that can potentially cause problems (for instance, the
+	  channel being swapped is hung up before the replacement channel enters the
+	  bridge thus stopping the transfer). This patch adds a condition that waits
+	  for the impart thread to get to a point of acceptable readiness before
+	  allowing the initiating thread to continue.
+
+	  ASTERISK-24782
+	  Reported by: John Bigelow
+
+	  Change-Id: I08fe33a2560da924e676df55b181e46fca604577
+
+2015-05-13 16:22 +0000 [ef82190804]  Matt Jordan <mjordan@digium.com>
+
+	* media cache: Add CLI commands
+
+	  This patch adds five CLI commands for the media cache:
+	   * 'media cache show all' - display a summary of all items in the media
+	     cache.
+	   * 'media cache show <uri>' - display detailed information about a
+	     single item in the media cache.
+	   * 'media cache delete <uri>' - remove an item from the media cache, and
+	     inform the bucket backend for the URI scheme to remove the item as
+	     well.
+	   * 'media cache refresh <uri>' - refresh a URI. If the item does not
+	     exist in the media cache, the bucket backend will pull down the media
+	     associated with the URI and create the item in the cache.
+	   * 'media cache create <uri>' - create an item in the media cache from
+	     some local media storage. Note that the bucket backend for the URI
+	     scheme must still permit the item creation.
+
+	  Change-Id: Id1c5707a3b8e2d96b56e4691a46a936cd171f4ae
+
+2015-01-29 08:38 +0000 [3ea0d38396]  Matt Jordan <mjordan@digium.com>
+
+	* media cache: Add a core API and facade for a backend agnostic media cache
+
+	  This patch adds a new API to the Asterisk core that acts as a media
+	  cache. The core API itself is mostly a thin wrapper around some bucket
+	  API provided implementation that itself acts as the mechanism of
+	  retrieval for media. The media cache API in the core provides the
+	  following:
+	   * A very thin in-memory cache of the active bucket_file items. Unlike a
+	     more traditional cache, it provides no expiration mechanisms. Most
+	     queries that hit the in-memory cache will also call into the bucket
+	     implementations as well. The bucket implementations are responsible
+	     for determining whether or not the active record is active and valid.
+	     This makes sense for the most likely implementation of a media cache
+	     backend, i.e., HTTP. The HTTP layer itself is the actual arbiter of
+	     whether or not a record is truly active; as such, the in-memory cache
+	     in the core has to defer to it.
+	   * The ability to create new items in the media cache from local
+	     resources. This allows for re-creation of items in the cache on
+	     restart.
+	   * Synchronization of items in the media cache to the AstDB. This
+	     also includes various pieces of important metadata.
+
+	  The API provides sufficient access that higher level APIs, such as the
+	  file or app APIs, do not have to worry about the semantics of the bucket
+	  APIs when needing to playback a resource.
+
+	  In addition, this patch provides unit tests for the media cache API. The
+	  unit tests use a fake bucket backend to verify correctness.
+
+	  Change-Id: I11227abbf14d8929eeb140ddd101dd5c3820391e
+
+2015-07-11 20:25 +0000 [887945d410]  Matt Jordan <mjordan@digium.com>
+
+	* main/bucket: Add a callback function for ast_bucket_file objects
+
+	  This patch adds a new function to the bucket API for ast_bucket_file
+	  objects, ast_bucket_file_metadata_callback. It will call ao2_callback on
+	  the ast_bucket_file's ao2_container of metadata, calling the provided
+	  ao2_callback_fn callback on each piece of metadata associated with the
+	  file.
+
+	  This is particularly useful when a bucket backend has added metadata,
+	  and a higher level API wants to be aware of/access said metadata,
+	  without knowing for sure what the key is.
+
+	  Change-Id: I96f6757717f47b650df91a437f7df16406227466
+
+2015-07-08 16:28 +0000 [458715d088]  Matt Jordan <mjordan@digium.com>
+
+	* main/sorcery: Don't fail object set creation from JSON if field fails
+
+	  Some individual fields may fail their conversion due to their default
+	  values being invalid for their custom handlers. In particular,
+	  configuration values that depend on others being enabled (and thus have
+	  an empty default value) are notorious for tripping this routine up. An
+	  example of this are any of the DTLS options for endpoints. Any of the
+	  DTLS options will fail to be applied (as DTLS is not enabled), causing
+	  the entire object set to be aborted.
+
+	  This patch makes it so that we log a debug message when skipping a
+	  field, and rumble on anyway.
+
+	  ASTERISK-25238
+
+	  Change-Id: I0bea13de79f66bf9f9ae6ece0e94a2dc1c026a76
+
+2015-07-08 16:21 +0000 [6ed58014f5]  Matt Jordan <mjordan@digium.com>
+
+	* main/format_cap: Parse capabilities generated by ast_format_cap_get_names
+
+	  We have a strange relationship between the parsing of format
+	  capabilities from a string and their representation as a string. We
+	  expect the format capabilities to be expressed as a string in the
+	  following format:
+
+	  allow = !all,ulaw,alaw
+	  disallow = g722
+
+	  While we would generate the string representation of those formats as:
+
+	  allow = (ulaw|alaw)
+	  disallow = (ulaw|alaw|g729...)
+
+	  When the configuration framework needs to store values as a string, it
+	  generates the format capabilities using the second representation; this
+	  representation however cannot be parsed when the entry is rehydrated.
+	  This patch fixes that by updating
+	  ast_format_cap_update_by_allow_disallow to parse an entry as if it were
+	  in the generated format if it has a leading '(' and a trailing ')'.
+
+	  ASTERISK-25238
+
+	  Change-Id: I904d43caf4cf45af06f6aee0c9e58556eb91d6ca
+
+2015-07-08 16:38 +0000 [e64e586900]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_sorcery_astdb: Add a debugging message for when retrieval by ID fails
+
+	  Having a debug message tell us that we attempted to look up an item but
+	  failed is nice in circumstances when it isn't clear if the wizard was
+	  queried correctly or not.
+
+	  Change-Id: I2600c3bbea87f252196358f62e73f4c7da8632f7
+
+2015-07-08 16:37 +0000 [7c14dfdc61]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_outbound_registration: Fix WARNING message
+
+	  Newlines are nice.
+
+	  Change-Id: Icf0d915db02882e47cd9077ed9009f5d44140d42
+
+2015-07-08 16:35 +0000 [3e286e6b51]  Matt Jordan <mjordan@digium.com>
+
+	* res_pjsip/configuration: Fix a variety of default value problems
+
+	  This patch fixes some bad default value handling in the following
+	  settings:
+
+	  * The 'message_context' and 'accountcode' settings are not mandatory. As
+	    such, we can allow their stringfield values to be empty.
+	  * The 'media_encryption' setting applies a default value of 'none' to
+	    the setting, which it then can't parse or understand. Since the value
+	    is documented to be 'no', this will now apply that as the default
+	    value.
+
+	  Change-Id: Ib9be7f97a7a5b9bc7aee868edf5acf38774cff83
+
+2015-07-08 16:32 +0000 [ffadb5f1de]  Matt Jordan <mjordan@digium.com>
+
+	* main/sorcery: Provide log messages when a wizard does not support an operation
+
+	  If a sorcery wizard does not support one of the 'optional' CRUD
+	  operations (namely the CUD), log a WARNING message so we are aware of
+	  why the operation failed. This also removes an assert in this case, as
+	  the CUD operation may have been triggered by an external system, in
+	  which case it is not a programming error but a configuration error.
+
+	  Change-Id: Ifecd9df946d9deaa86235257b49c6e5e24423b53
+
+2015-06-27 17:53 +0000 [5266796432]  Matt Jordan <mjordan@digium.com>
+
+	* tests/test_devicestate: Add additional tests for the device state API
+
+	  This patch adds more tests that exercise the device state API. This includes:
+
+	  * Tests that cover adding a device state provider, as well as deleting a
+	    device state provider. This also verifies that you cannot add an
+	    already added device state provider, and cannot delete an already
+	    deleted device state provider.
+	  * A test that covers changing device state and receiving said updates
+	    from a device state subscriber. This also covers hitting both the
+	    device state cache as well as a custom device state provider.
+	  * A test that covers converting device state to channel state and device
+	    state values to a string representation and back.
+	  * A test that covers obtaining device state from an active channel and a
+	    channel driver that provides its own device state.
+
+	  Change-Id: I2adca67ffb405cd8625a5d6df1e3f9b3d945c08d
+
+2015-06-27 17:51 +0000 [f77e688f20]  Matt Jordan <mjordan@digium.com>
+
+	* main/devicestate: Prevent duplicate registration of device state providers
+
+	  Currently, the device state provider API will allow you to register a
+	  device state provider with the same case insensitive name more than
+	  once. This could cause strange issues, as the duplicate device state
+	  providers will not be queried when a device's state has to be polled.
+	  This patch updates the API such that a device state provider with the
+	  same name as one that has already registered will be rejected.
+
+	  Change-Id: I4a418a12280b7b6e4960bd44f302e27cd036ceb2
+
+2015-06-26 10:57 +0000 [1b7760a8aa]  Benjamin Ford <bford@digium.com>
+
+	* ARI: Added new functionality to get all module information.
+
+	  An http request can be sent to retrieve a list of all existing modules,
+	  including the resource name, description, use count, status, and
+	  support level.
+
+	  The command "curl -v -u user:pass -X GET 'http://localhost:8088/ari/
+	  asterisk/modules" (or something similar, depending on configuration)
+	  can be run in the terminal to access this new functionality.
+
+	  For more information, see:
+	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+
+	  * Added new ARI functionality
+	  * Information on modules can now be retrieved
+
+	  Change-Id: I63cbbf0ec0c3544cc45ed2a588dceabe91c5e0b0
+
+2015-07-09 09:18 +0000 [4a25d55416]  Joshua Colp <jcolp@digium.com>
+
+	* bridge_native_rtp.c: Don't start native RTP bridging after attended transfer.
+
+	  The bridge_native_rtp module adds a frame hook to channels which are in
+	  a native RTP bridge. This frame hook is used to intercept when a hold
+	  or unhold frame traverses the bridge so native RTP can be stopped or
+	  started as appropriate. This is expected but exposes a specific bug
+	  when attended transfers are involved.
+
+	  Upon completion of an attended transfer an unhold frame is queued up
+	  to take one of the channels involved off hold. After this is done
+	  the channel is moved between bridges.
+
+	  When the frame hook is involved in this case for the unhold it
+	  releases the channel lock and acquires the bridge lock. This
+	  allows the bridge core to step in and move the channel
+	  (potentially changing the bridging techology) from another thread.
+	  Once completed the bridge lock is released by the bridge core.
+	  The frame hook is then able to acquire the bridge lock and
+	  wrongfully starts native RTP again, despite the channel no longer
+	  being in the bridge or needing to start native RTP. In fact at
+	  this point the frame hook is no longer attached to the channel.
+
+	  This change makes it so the native RTP bridge data is available to
+	  the frame hook when it is invoked. Whether the frame hook has
+	  been detached or not is stored on the native RTP bridge data and
+	  is checked by the frame hook before starting or stopping native
+	  RTP bridging. If the frame hook has been detached it does nothing.
+
+	  ASTERISK-25240 #close
+
+	  Change-Id: I13a73186a05f4e5a764f81e5cd0ccec1ed1891d2
+
+2015-07-08 04:21 +0000 [9276415f65]  Joshua Colp <jcolp@digium.com>
+
+	* res_rtp_asterisk: Ensure DTLS timeout timer is -1 if DTLS is not used.
+
+	  This change fixes a bug where the DTLS timeout timer would be
+	  initialized to 0 if DTLS was not used for an RTP session.
+
+	  ASTERISK-25103
+
+	  Change-Id: If8d26bb054f1d300838850da5b8db9044c2fe2ac
+
+2015-07-07 15:03 +0000 [3cdfd39af7]  Ashley Sanders <asanders@digium.com>
+
+	* DNS: Create a system-level DNS resolver
+
+	  Prior to this patch, the DNS core present in master had no default system-level
+	  resolver implementation. Therefore, it was not possible for the DNS core to
+	  perform resolutions unless the libunbound library was installed and the
+	  res_resolver_unbound module was loaded.
+
+	  This patch introduces a system-level DNS resolver implementation that will
+	  register itself with the lowest consideration priority available (to ensure
+	  that it is to be used only as a last resort). The resolver relies on low-level
+	  DNS search functions to perform a rudimentary DNS search based on a provided
+	  query and then supplies the search results to the DNS core.
+
+	  ASTERISK-25146 #close
+	  Reported By: Joshua Colp
+
+	  Change-Id: I3b36ea17b889a98df4f8d80d50bb7ee175afa077
+
+2015-07-01 07:55 +0000 [5717340ab3]  Joshua Colp <jcolp@digium.com>
+
+	* res_rtp_asterisk: Prevent simultaneous access to DTLS SSL context.
+
+	  This change moves logic for setting up the DTLS SSL contexts to
+	  when the SDP is done being processed instead of when ICE negotiation
+	  completes. It also stops handshakes from being initiated when we
+	  are acting as a server.
+
+	  Manipulating the SSL context when ICE negotiation has completed
+	  is problematic as the SSL context is not protected and if acting
+	  as a client the remote side may have started DTLS negotiation
+	  already.
+
+	  The retransmission timeout timer code has also been split up
+	  and simplified some. Both RTP and RTCP now have their own timers
+	  and the points at which the timer is stopped and started is now
+	  more specific. When a packet is sent the timer is started. When
+	  a response is received but before it is processed the timer is
+	  stopped. This provides a guarantee that the timeout is not
+	  occurring while the response is processed.
+
+	  ASTERISK-22805 #close
+	  ASTERISK-24550 #close
+	  ASTERISK-24651 #close
+	  ASTERISK-24832 #close
+	  ASTERISK-25103 #close
+	  ASTERISK-25127 #close
+
+	  Change-Id: Ib75ea2546f29d6efc3d2d37c58df6986c7bd9b91
+
+2015-06-26 18:48 +0000 [189841ddb7]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_mwi.c: Fix MWI subscription memory corruption crash.
+
+	  MWI subscriptions can crash or corrupt memory when using the subscription
+	  datastore to access the MWI subscription object because the datastore is
+	  not holding a reference to the object.
+
+	  * Give the subscription datastore a ref to the MWI subscription object.
+	  It is unfortunate that the ref causes a circular ref chain that must be
+	  explicitly broken to allow the memory to get released.  The loop is broken
+	  when the subscription is shutdown and if the subscription setup fails.
+
+	  ASTERISK-25168 #close
+	  Reported by: Carl Fortin
+
+	  Change-Id: Ice4fa823f138ff10a6c74d280699c41a82836d4f
+
+2015-07-02 14:51 +0000 [7cd99be534]  Richard Mudgett <rmudgett@digium.com>
+
+	* PJSIP XML, XPIDF: Fix buffer size overwrite memory corruption error.
+
+	  When res_pjsip body generator modules were generating XML or XPIDF
+	  response bodies, there was a chance that the generated body would be the
+	  exact size of the supplied buffer.  Adding the nul string terminator would
+	  then write beyond the end of the buffer and potentially corrupt memory.
+
+	  * Fix MALLOC_DEBUG high fence violations caused by adding a nul string
+	  terminator on the end of a buffer for XML or XPIDF response bodies.
+
+	  * Made calls to pj_xml_print() safer if the XML prolog is requested.  Due
+	  to a bug in pjproject, the return value could be -1 _or_
+	  AST_PJSIP_XML_PROLOG_LEN if the supplied buffer is not large enough.
+
+	  * Updated the doxygen comment of AST_PJSIP_XML_PROLOG_LEN to describe the
+	  return value of pj_xml_print() when the supplied buffer is not large
+	  enough.
+
+	  ASTERISK-25168
+	  Reported by: Carl Fortin
+
+	  Change-Id: Id70e1d373a6a2b2bd9e678b5cbc5e55b308981de
+
+2015-06-26 10:36 +0000 [792ed7ce93]  Richard Mudgett <rmudgett@digium.com>
+
+	* PJSIP FAX: Fix T.38 automatic reject timer NULL channel pointer dereferences.
+
+	  When a caller calls a FAX number and then hangs up right after the call is
+	  answered then the T.38 re-INVITE automatic reject timer may still be
+	  running after the channel goes away.
+
+	  * Added session NULL channel checks on the code paths that get executed by
+	  t38_automatic_reject() to prevent a crash when the T.38 re-INVITE
+	  automatic reject timer expires.
+
+	  ASTERISK-25168
+	  Reported by: Carl Fortin
+
+	  Change-Id: I07b6cd23815aedce5044f8f32543779e2f7a2403
+
+2015-06-30 11:17 +0000 [030e8339dd]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_mwi.c: Use safer loop coding in mwi_subscription_mailboxes_str().
+
+	  Change-Id: I6f39d809a6d1b47b35bb32b298f5a12f35d6f907
+
+2015-06-30 11:14 +0000 [453d7b8d69]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_mwi.c: Eliminate a simple RAII_VAR.
+
+	  Change-Id: Ib1843f81e826a6c760c424c88eb70c350d9d61da
+
+2015-06-30 11:11 +0000 [786c6d42ef]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_mwi.c: Fix mid-line log message line breaks.
+
+	  * Add create_mwi_subscriptions_for_endpoint() doxygen comment.
+
+	  Change-Id: I3c3f921f4ec749fb65b62d2f6fa0d4d1888b94e2
+
+2015-06-26 16:10 +0000 [1b91094edd]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_t38.c: Fix always false if test.
+
+	  Calling t38_change_state() sets the t38 state so it makes little sense to
+	  then check the state right after the call for something else.
+
+	  * Made the code in t38_interpret_parameters() reject or exit T.38 mode as
+	  intended but not implemented.
+
+	  Change-Id: Ib281263a6ed44da9448132c4e6df1e183b8a3df2
+
+2015-06-30 15:19 +0000 [74135c8efa]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip: Failover when server is not available
+
+	  Previously Asterisk did not properly failover to the next resolved DNS
+	  address when a endpoint could not be reached. With this patch, and while
+	  using res_pjsip, SIP requests (both in/out of dialog) now attempt to use
+	  the next address in the list of resolved addresses until a proper response
+	  is received or no more addresses are left.
+
+	  ASTERISK-25076 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: Ief14f4ebd82474881f72f4538f4577f30af2a764
+
+2015-07-06 09:24 +0000 [38a3c27a09]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Execute stale unit test last.
+
+	  In Jenkins there is currently a sporadic test failure of a
+	  variable number of sorcery memory cache unit tests. I have not
+	  been able to reproduce this on the build agents themselves or
+	  on my development machine.
+
+	  My working theory is that the stale unit test is causing a
+	  sorcery instance to persist longer than expected, causing subsequent
+	  tests to fail when setting up and initializing the next
+	  sorcery instance.
+
+	  To see if this is the case this change moves the stale unit test
+	  to execute last so no subsequent unit tests can have issues
+	  initializing their sorcery instance.
+
+	  Change-Id: Ifd6550a949613be774b75fa5db12c02110f82c4a
+
+2015-06-20 13:54 +0000 [ef8d3f6506]  Matt Jordan <mjordan@digium.com>
+
+	* bucket: Add clone/staleness operations for ast_bucket/ast_bucket_file
+
+	  This patch enhances the bucket API in two ways.
+
+	  First, since ast_bucket and ast_bucket_file instances are immutable, a 'clone'
+	  operation has been added that provides a 'clone' of an existing
+	  ast_bucket/ast_bucket_file object. Note that this makes use of the
+	  ast_sorcery_copy operation, along with the copy callback handler on the
+	  "bucket" and "file" object types for the bucket sorcery instance.
+
+	  Second, there is a need for the bucket API to ask a wizard if an object
+	  is stale. This is particularly useful with the upcoming media cache
+	  enhancements, where we want to ask the backing data storage if the
+	  object we are currently operating on has known updates. This patch adds
+	  API calls for ast_bucket and ast_bucket_file objects, which callback
+	  into their respective sorcery wizards via the sorcery API.
+
+	  Unit tests have also been added to cover the respective
+	  ast_bucket/ast_bucket_file clone and staleness operations.
+
+	  Change-Id: Ib0240ba915ece313f1678a085a716021d75d6b4a
+
+2015-07-04 10:03 +0000 [b178f8701b]  Matt Jordan <mjordan@digium.com>
+
+	* sorcery: Add support for object staleness
+
+	  This patch enhances the sorcery API to allow for sorcery wizards to
+	  determine if an object is stale. This includes the following:
+
+	  * Sorcery objects now have a timestamp that is set on creation. Since
+	    sorcery objects are immutable, this can be used by sorcery wizards to
+	    determine if an object is stale.
+
+	  * A new API call has been added, ast_sorcery_is_stale. This API call
+	    queries the wizards associated with the object, calling a new callback
+	    function 'is_stale'. Note that if a wizard does not support the new
+	    callback, objects are always assumed to not be stale.
+
+	  * Unit tests have been added that cover the new API call.
+
+	  Change-Id: Ica93c6a4e8a06c0376ea43e00cf702920b806064
+
+2015-07-04 18:22 +0000 [f35a4b8525]  Joshua Colp <jcolp@digium.com>
+
+	* res/res_http_websocket: Don't send HTTP response fragmented.
+
+	  This change makes it so that when accepting a WebSocket
+	  connection the HTTP response is sent as one packet instead of
+	  fragmented. Browsers don't like it when you send it fragmented.
+
+	  ASTERISK-25103
+
+	  Change-Id: I9b82c4ec2949b0bce692ad0bf6f7cea9709e7f69
+
+2015-06-27 18:47 +0000 [2c17515f3c]  Matt Jordan <mjordan@digium.com>
+
+	* Makefile: Remove coverage files on 'make clean'
+
+	  This patch updates a variety of Makefiles in Asterisk's build system to
+	  remove .gcda and .gcno files when 'make clean' is executed. These files
+	  are generated when '--enable-coverage' is passed to the Asterisk
+	  configure script.
+
+	  Change-Id: Ib70b41eea2ee2908885bff02e80faf9f40c84602
+
+2015-07-02 09:08 +0000 [34323f9f95]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* chan_sip: Fix early call pickup channel leak.
+
+	  When handle_invite_replaces() was called, and either ast_bridge_impart()
+	  failed or there was no bridge (because the channel we're picking up was
+	  still ringing), chan_sip would leak a channel.
+
+	  Thanks Matt and Corey for checking the bridge path.
+
+	  ASTERISK-25226 #close
+
+	  Change-Id: Ie736bb182170a73eef5bcef0ab0376f645c260c8
+
+2015-07-01 16:04 +0000 [ef74ccb18d]  Matt Jordan <mjordan@digium.com>
+
+	* sorcery/realtime: Add a bit of debug and warning messages for bad configs
+
+	  When a mapping does not exist between a sorcery.conf defined object and
+	  a realtime mapping in extconf, currently, the user will receive a slew
+	  of ERROR messages that don't really tell what is happening. Some ERROR
+	  messages may even be misleading, as they occur after the sorcery API has
+	  already given up on the attempt to load and create the sorcery object.
+
+	  This patch adds a bit of debug and a useful WARNING message for when a
+	  wizard's open callback fails for a particular object type. In the bad
+	  configurations that resulted in this patch, this provided a 'root cause'
+	  WARNING message that pointed in the right direction of the configuration
+	  problem.
+
+	  Change-Id: I1cc7344f2b015b8b9c85a7e6ebc8cb4753a8f80b
+
+2015-07-02 06:54 +0000 [f18436642b]  Joshua Colp <jcolp@digium.com>
+
+	* dns: Fix crash when invoking cancel in DNS recurring unit test.
+
+	  The recurring unit test expects the user data on a DNS query
+	  created as a result of a recurring DNS query to be the recurring
+	  structure itself. This is true, mostly. When invoking the user
+	  provided callback this user data is changed to the user provided
+	  data. This presents a race condition where the data may or may
+	  not point to the recurring data.
+
+	  This change simplifies the callback of the user provided callback
+	  by creating a new query and populating it with the expected values.
+	  This leaves the recurring DNS query alone and fixes the race
+	  condition. This is more in line with how the API should be used
+	  overall.
+
+	  ASTERISK-25222 #close
+
+	  Change-Id: I10fb6deec025dff097157e7ec17e6e4921778478
+
+2015-07-02 06:19 +0000 [6fbb58c7f7]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* chan_mgcp: Don't call close on fd -1.
+
+	  ASTERISK-25220 #close
+
+	  Change-Id: Ic48f3a82f51ada87f2fb0e016c9efe0ad56f1ee3
+
+2015-07-02 06:10 +0000 [13a318bbb1]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* rtp_engine: Skip useless self-assignment in ast_rtp_engine_unload_format.
+
+	  When running valgrind on Asterisk, it complained about:
+
+	      ==32423== Source and destination overlap in memcpy(0x85a920, 0x85a920, 304)
+	      ==32423==    at 0x4C2F71C: memcpy@@GLIBC_2.14 (in /usr/lib/valgrind/...)
+	      ==32423==    by 0x55BA91: ast_rtp_engine_unload_format (rtp_engine.c:2292)
+	      ==32423==    by 0x4EEFB7: ast_format_attr_unreg_interface (format.c:1437)
+
+	  The code in question is a struct assignment, which may be performed by
+	  memcpy as a compiler optimization. It is changed to only copy the struct
+	  contents if source and destination are different.
+
+	  ASTERISK-25219 #close
+
+	  Change-Id: I6d3546c326b03378ca8e9b8cefd41c16e0088b9a
+
+2015-07-02 05:16 +0000 [40274e3652]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* astfd: Fix buffer overflow in DEBUG_FD_LEAKS.
+
+	  If DEBUG_FD_LEAKS was used and more file descriptors than the default of
+	  1024 were available, some DEBUG_FD_LEAKS-patched functions would
+	  overwrite memory past the fixed-size (1024) fdleaks buffer.
+
+	  This change:
+	  - adds bounds checks to __ast_fdleak_fopen and __ast_fdleak_pipe
+	  - consistently uses ARRAY_LEN() instead of sizeof() or 1023 or 1024
+	  - stores pointers to constants instead of copying the contents
+	  - reorders the fdleaks struct for possibly tighter packing
+	  - adds a tiny bit of documentation
+
+	  ASTERISK-25212 #close
+
+	  Change-Id: Iacb69e7701c0f0a113786bd946cea5b6335a85e5
+
+2015-07-02 04:57 +0000 [3fab8212e3]  Walter Doekes <walter+asterisk@wjd.nu>
+
+	* res_timing: Don't close FD 0 when out of open files.
+
+	  This fixes so a failure to get a timer file descriptor does not cascade
+	  to closing FD 0.
+
+	  On error, both res_timing_kqueue and res_timing_timerfd would call the
+	  destructor before setting the file handle. The file handle had been
+	  initialized to 0, causing FD 0 to be closed. This in turn, resulted in
+	  floods of "CLI>" messages and an unusable terminal.
+
+	  ASTERISK-19277 #close
+	  Reported by: Barry Chern
+
+	  For the master branch, this was already fixed. This patch only ensures
+	  that we do not attempt to close a negative file descriptor.
+
+	  Change-Id: I147d7e33726c6e5a2751928d56561494f5800350
+
+2015-07-01 17:25 +0000 [41610df8d5]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_vpb.cc: Fix compiler warning Jenkins found.
+
+	  Change-Id: I0ec7fd10d56d90d5a60b12b5a7d6807f265ac5e0
+
+2015-07-01 13:34 +0000 [537df26f9c]  Scott Griepentrog <scott@griepentrog.com>
+
+	* Channel alert pipe: improve diagnostic error return
+
+	  When a frame is queued on a channel, any failure in
+	  ast_channel_alert_write is logged along with errno.
+
+	  This change improves the diagnostic message through
+	  aligning the errno value with actual failure cases.
+
+	  ASTERISK-25224
+	  Reported by: Andrey Biglari
+
+	  Change-Id: I1bf7b3337ad392789a9f02c650589cd065d20b5b
+
+2015-06-29 12:45 +0000 [58d18324f0]  Mark Michelson <mmichelson@digium.com>
+
+	* res_sorcery_realtime: Fix leak of sorcery object type.
+
+	  This prevents a leak of a sorcery object type when realtime sorcery
+	  objects are retrieved by fields or when multiple objects are retrieved.
+
+	  The extent of this leak is that sorcery object types would be leaked.
+	  These are allocated whenever an object type is registered with sorcery,
+	  meaning that on module shutdown, these objects would be leaked. This
+	  could be problematic if many reloads were performed, but it is not as
+	  severe as if every sorcery object retrieved from realtime were being
+	  leaked.
+
+	  ASTERISK-25165 #close
+	  Reported by Corey Farrell
+
+	  Change-Id: I625c3b50eee4576670b7eeb013c81ad043b4b4f8
+2015-06-26 22:02 +0000 [80d97290bb]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_corosync: Always decline module load, instead of failing
+
+	  Returns a 'failure' from the module load routine indicates to Asterisk
+	  that it should abort loading completely. This is rarely - in fact,
+	  really, never - a good option. Aborting load of Asterisk from a dynamic
+	  module implies that the core, and the rest of the dynamic modules, don't
+	  matter: we should abandon all processing.
+
+	  res_corosync is really not that important.
+
+	  This patch updates the module such that, if it fails to load, it
+	  politely declines (emitting ERROR messages along the way), and allows
+	  Asterisk to continue to function.
+
+	  Note that this issue was keeping Asterisk unit tests from running on
+	  certain build agents.
+
+	  Change-Id: I252249e81fb9b1a68e0da873f54f47e21d648f0f
+
+2015-06-26 20:38 +0000 [892cc5625f]  Matt Jordan <mjordan@digium.com>
+
+	* main/pbx: Resolve case sensitivity regression in PBX hints
+
+	  When 8297136f was merged for ASTERISK-25040, a regression was introduced
+	  surrounding the case sensitivity of device names within hints.
+	  Previously, device names - such as 'sip/foo' - were compared in a case
+	  insensitive fashion. Thus, 'sip/foo' was equivalent to 'SIP/foo'. After
+	  that patch, only the case sensitive name would match, i.e., 'SIP/foo'.
+	  As a result, some dialplan hints stopped working.
+
+	  This patch re-introduces case insensitive matching for device names in
+	  hints.
+
+	  ASTERISK-25040
+
+	  ASTERISK-25202 #close
+
+	  Change-Id: If5046a7d14097e1e3c12b63092b9584bb1e9cb4c
+	  (cherry picked from commit 96bbcf495a1da9e607d9b04a44b5c4f49e83cc03)
+
+2015-06-26 16:12 +0000 [e18b22a806]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_nat: Adjust when contact should be rewritten.
+
+	  A previous change made the contact only get rewritten if the dialog's
+	  route set was not marked frozen. Unfortunately, while the intent of this
+	  is correct, the dialog's route set actually gets marked as frozen
+	  earlier than expected, especially for UAS dialogs.
+
+	  Instead, the idea is that the contact needs to not be rewritten if there
+	  is a pre-existing route set on the dialog. This is now accomplished by
+	  checking the dialog's route set list instead of checking if the route
+	  set is frozen.
+
+	  Doing this causes some broken tests to begin passing again.
+
+	  ASTERISK-25196
+	  Reported by Mark Michelson
+
+	  Change-Id: I525ab251fd40a52ede327a52a2810a56deb0529e
+
+2015-06-19 18:27 +0000 [99b1aa6d26]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Add a serializer shutdown group.
+
+	  The client_state objects contain a serializer used to send the outbound
+	  REGISTER messages.  Once all those message transactions are complete then
+	  the module can shutdown.
+
+	  ASTERISK-24907 #close
+	  Reported by: Kevin Harwell
+
+	  Change-Id: Ibb2fe558f98190f2a06da830e0fadfa25516f547
+
+2015-06-26 10:41 +0000 [f536e9b59c]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_refer: Prevent sending duplicate headers.
+
+	  res_pjsip_refer will attempt to add Referred-By or Replaces headers to
+	  outbound INVITEs at times. If the INVITE gets challenged for
+	  authentication, then we will resend the INVITE. Prior to this patch, the
+	  Referred-By or Replaces header would be re-added to the outbound INVITE,
+	  resulting in duplicated headers.
+
+	  ASTERISK-25204 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I59fb5c08b4d253c0dba9ee3d3950b5025358222d
+
+2015-06-23 14:34 +0000 [c2d48a2a28]  Richard Mudgett <rmudgett@digium.com>
+
+	* AMI: Add Linkedid to the standard channel snapshot AMI event headers.
+
+	  ASTERISK-25189 #close
+	  Reported by: John Hardin
+
+	  Change-Id: I2b1778c3fdc1dca0ed55db4e3a639eddfb16c2ac
+2015-06-23 17:43 +0000 [700606a659]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_nat: Rewrite route set when required.
+
+	  When performing some provider testing, the rewrite_contact option was
+	  interfering with proper construction of a route set when sending an ACK
+	  after receiving a 200 OK response to an INVITE.
+
+	  The initial INVITE was sent to address sip:foo. The 200 OK had a Contact
+	  header with URI sip:bar. In addition, the 200 OK had Record-Route
+	  headers for sip:baz and sip:foo, in that order. Since the Record-Route
+	  headers had the lr parameter, the result should have been:
+
+	  * Set R-URI of the ACK to sip:bar.
+	  * Add Route headers for sip:foo and sip:baz, in that order.
+
+	  However, the rewrite_contact option resulted in our rewriting the
+	  Contact header on the 200 OK to sip:foo. The result was:
+
+	  * R-URI remained sip:foo.
+	  * We added Route headers for sip:foo and sip:baz, in that order.
+
+	  The result was that sip:bar was not indicated in the ACK at all, so the
+	  far end never received our ACK. The call eventually dropped.
+
+	  The intention of rewrite_contact is to rewrite the most immediate
+	  destination of our SIP request to be the same address on which we
+	  received a request or response. In the case of processing a SIP response
+	  with Record-Route headers, this means that instead of rewriting the
+	  Contact header, we should instead rewrite the bottom-most Record-Route
+	  header. In the case of processing a SIP request with Record-Route
+	  headers, this means we rewrite the top-most Record-route header.
+	  Like when we rewrite the Contact header, we also ensure to update
+	  the dialog's route set if it exists.
+
+	  ASTERISK-25196 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I9702157c3603a2d0bd8a8215ac27564d366b666f
+2015-06-19 16:16 +0000 [af4ae3095e]  Richard Mudgett <rmudgett@digium.com>
+
+	* threadpool, res_pjsip: Add serializer group shutdown API calls.
+
+	  A module trying to unload needs to wait for all serializers it creates and
+	  uses to complete processing before unloading.
+
+	  ASTERISK-24907
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I8c80b90f2f82754e8dbb02ddf3c9121e5e966059
+
+2015-06-16 15:06 +0000 [4c133d81cd]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Fix handle_client_state_destruction() refs
+
+	  * handle_client_state_destruction() must always be passed a ref to
+	  client_state because it will always unref client_state.
+	  handle_registration_response() was not passing a client_state ref.
+
+	  * Made the final un-REGISTER message get sent normally using the pjproject
+	  register control structure in handle_client_state_destruction().  The
+	  previous code attempted to short circuit the response handling for the
+	  module to unload.  That doesn't work for a couple reasons.  One,
+	  pjsip_regc_send() may call the registered callback before it returns and
+	  unbalance the client_state ref count.  Two, the registered callback
+	  handles any authentication for the un-REGISTER message.
+
+	  * Made the distinction between internal registration state and external
+	  registration status with sip_outbound_registration_status_str().  This is
+	  necessary to avoid altering documented AMI messages with internal
+	  changes.
+
+	  * Removed references to client_state->client outside of the serializer
+	  thread.  When handle_client_state_destruction() destroys the pjproject
+	  register control structure that memory is freed and cannot be referenced
+	  anymore.  These accesses were to provide information for debug and
+	  off-nominal warning messages.
+
+	  * In sip_outbound_registration_timer_cb() you should not access entry->id
+	  after unrefing client_state because the passed in entry is normally
+	  pointing to the timer entry in the client_state object.
+
+	  ASTERISK-24907
+	  Reported by: Kevin Harwell
+
+	  Change-Id: Ia7b446d8644b6b4550ef5bea49527671de65183f
+
+2015-06-15 15:28 +0000 [dc63377c60]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Use ast_sorcery_object_unregister() API
+
+	  The sorcery pjsip 'registration' config object needs to be destroyed on
+	  module unload.  Otherwise, a reload of res_pjsip could try to use
+	  callbacks for a previously unloaded instance of the module provided by
+	  ast_sorcery_object_register() or one of the variants.  Also, if
+	  res_pjsip_outbound_registration were subsequently reloaded, the sorcery
+	  config field objects would be registered in sorcery twice.
+
+	  ASTERISK-24907
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I304fad13dece2604af48353f6c6d9d5c7b064697
+
+2015-06-15 15:28 +0000 [9ec8a0f3cc]  Richard Mudgett <rmudgett@digium.com>
+
+	* sorcery: Add ast_sorcery_object_unregister() API call.
+
+	  Find and unlink the specified sorcery object type to complement
+	  ast_sorcery_object_register().  Without this function you cannot
+	  completely unload individual modules that use sorcery for configuration.
+
+	  ASTERISK-24907
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I1c04634fe9a90921bf676725c7d6bb2aeaab1c88
+
+2015-06-15 13:38 +0000 [77ff7325a2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Reorder load_module() and unload_module().
+
+	  It is best if the loading code creates and initializes the module's
+	  infrastructure before letting the system know of its existence.  The
+	  unloading code needs to reverse the actions of the loading code and in the
+	  reverse order.
+
+	  ASTERISK-24907
+	  Reported by: Kevin Harwell
+
+	  Change-Id: I5d151383e9787b5b60aa5e1627b10f040acdded4
+
+2015-06-25 06:42 +0000 [8d6cf667dc]  Joshua Colp <jcolp@digium.com>
+
+	* channel: Remove ignore of answer on non-outgoing channels.
+
+	  Due to the way that channels can now be moved around inside of
+	  Asterisk it is possible for the outgoing flag of a channel to get
+	  cleared before it has been answered. This results in the bridge
+	  not receiving notification that the outgoing leg has been answered.
+
+	  This most easily exhibits itself with DTMF based blond transfers.
+	  Since the answer of the outgoing leg is ignored the other party
+	  continues to receive both a locally generated ringing and the
+	  media stream of the outgoing leg upon its answer. This results
+	  in no media being heard.
+
+	  This change removes the ignore of the answer and allows it
+	  to pass through.
+
+	  ASTERISK-25171 #close
+
+	  Change-Id: I82aedcec4f89f34a2e5472086dfc9a6c775bca8e
+
+2015-06-24 14:30 +0000 [daaa551c92]  Richard Mudgett <rmudgett@digium.com>
+
+	* test.c: Add unit test registration checks for summary and description.
+
+	  Added checks when a unit test is registered to see that the summary and
+	  description strings do not end with a new-line '\n' for consistency.
+
+	  The check generates a warning message and will cause the
+	  /main/test/registrations unit test to fail.
+
+	  * Updated struct ast_test_info member doxygen comments.
+
+	  Change-Id: I295909b6bc013ed9b6882e85c05287082497534d
+
+2015-06-24 16:39 +0000 [71a4d1a033]  Richard Mudgett <rmudgett@digium.com>
+
+	* Unit tests: Fix more unit test description strings.
+
+	  Analyzing the code shows that the unit test summary and description
+	  strings should not end with a new-line character.  Where these strings are
+	  used in the code a new-line is provided for output.
+
+	  Change-Id: I2f4f37988ec363c8d1c5077a2fc8ca841c5cd30c
+
+2015-06-24 14:39 +0000 [9c6d72e30d]  Richard Mudgett <rmudgett@digium.com>
+
+	* Unit tests: Fix unit test description strings.
+
+	  Analyzing the code shows that the unit test summary and description
+	  strings should not end with a new-line character.  Where these strings are
+	  used in the code a new-line is provided for output.
+
+	  Change-Id: I129284f5e7ca93d82532334076da4c462d3d9fba
+
+2015-06-24 16:37 +0000 [a0c2d2089d]  Richard Mudgett <rmudgett@digium.com>
+
+	* DNS unit tests: Fix extraneous description string commas.
+
+	  Change-Id: Icf5f13c8e1c2c92a4473bb573ed2dd856ce1b64e
+
+2015-06-23 11:21 +0000 [3b2b004d69]  Joshua Colp <jcolp@digium.com>
+
+	* app_dial: Hold reference to calling channel formats when dialing outbound.
+
+	  Currently when requesting a channel the native formats of the
+	  calling channel are provided to the core for usage when dialing
+	  the outbound channel. This occurs without holding the channel lock
+	  or keeping a reference to the formats. This is problematic as
+	  the channel driver may end up changing the formats during this time.
+	  In the case of chan_sip this happens when an SDP negotiation
+	  completes.
+
+	  This change makes it so app_dial keeps a reference to the native
+	  formats of the calling channel which guarantees that they will
+	  remain valid for the period of time needed.
+
+	  ASTERISK-25172 #close
+
+	  Change-Id: I2f0a67bd0d5d14c3bdbaae552b4b1613a283f0db
+
+2015-06-17 16:23 +0000 [af66b0f3f7]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Add missing line endings to CLI commands
+
+	  Change-Id: I39ae612746d892d2dbe86f3ff2d7027fa1da57f7
+
+2015-06-12 14:29 +0000 [3f0708e5fe]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Eliminate simple RAII_VAR() usage.
+
+	  Change-Id: I399cb9d61bbba706b48c98e0bf75e98984cd9a9e
+
+2015-06-12 13:33 +0000 [9ceb848242]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Misc code cleanups.
+
+	  * Break some long lines.
+
+	  * Fix doxygen comment.
+
+	  Change-Id: I8f12ba6822f84d5e7bb575280270cd7e2fefb305
+
+2015-06-22 15:11 +0000 [44c3c392e3]  Kevin Harwell <kharwell@digium.com>
+
+	* bridge.c: Hangup attended transfer target if bridged
+
+	  After completing an attended transfer the transfer target channel was not being
+	  hung up after leaving the bridge. Added an explicit softhangup to hangup said
+	  channel, but only if it was previously bridged.
+
+	  ASTERISK-24782 #close
+	  Reported by: John Bigelow
+
+	  Change-Id: Idde9543d56842369384a5e8c00d72a22bbc39ada
+
+2015-06-17 05:04 +0000 [7846f73432]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_mwi: Set up unsolicited MWI upon registration.
+
+	  The res_pjsip_mwi previously required a reload to set up the proper
+	  subscriptions to allow unsolicited MWI to work. This change
+	  makes it so the act of registering will also cause this to occur.
+	  This is particularly useful if realtime is involved as no reload
+	  needs to occur within Asterisk to cause the MWI information
+	  to get sent.
+
+	  ASTERISK-25180 #close
+
+	  Change-Id: Id847b47de4b8b3ab8858455ccc2f07b0f915f252
+
+2015-06-22 13:57 +0000 [096b27d9d2]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_outbound_registration.c: Fix whitespace conflict potential.
+
+	  Change-Id: I82e6e388e3688aebe0783f16c9e0800a747584b5
+
+2015-06-22 09:26 +0000 [1ad9a6b6b6]  Alexander Traud (License 6520)
+
+	* chan_sip: Reload peer without its old capabilities.
+
+	  On reload, previously allowed codecs were not removed. Therefore, it was not
+	  possible to remove codecs while Asterisk was running. Furthermore, newly added
+	  codecs got appended behind the previous codecs. Therefore, it was not possible
+	  to add a codec with a priority of #1. This change removes the old capabilities
+	  before the current ones are added.
+
+	  ASTERISK-25182 #close
+	  Reported by: Alexander Traud
+	  patches:
+	   asterisk_13_allow_codec_reload.patch uploaded by Alexander Traud (License 6520)
+
+	  Change-Id: I62a06bcf15e08e8c54a35612195f97179ebe5802
+
+2015-06-20 19:38 +0000 [5caefc98a1]  Joshua Colp <jcolp@digium.com>
+
+	* chan_sip: Destroy peers without holding peers container lock.
+
+	  Due to the use of stasis_unsubscribe_and_join in the peer destructor
+	  it is possible for a deadlock to occur when an event callback is
+	  occurring at the same time.
+
+	  This happens because the peer may be destroyed while holding the
+	  peers container lock. If this occurs the event callback will never
+	  be able to acquire the container lock and the unsubscribe will
+	  never complete.
+
+	  This change makes it so the peers that have been removed from the
+	  peers container are not destroyed with the container lock held.
+
+	  ASTERISK-25163 #close
+
+	  Change-Id: Ic6bf1d9da4310142a4d196c45ddefb99317d9a33
+
+2015-06-18 13:16 +0000 [d7a1e84a1e]  Mark Michelson <mmichelson@digium.com>
+
+	* Resolve race conditions involving Stasis bridges.
+
+	  This resolves two observed race conditions.
+
+	  First, a bit of background on what the Stasis application does:
+
+	  1a Creates a stasis_app_control structure. This structure is linked into
+	     a global container and can be looked up using a channel's unique ID.
+	  2a Puts the channel in an event loop. The event loop can exit either
+	     because the stasis_app_control structure has been marked done, or
+	     because of some other factor, such as a hangup. In the event loop, the
+	     stasis_app_control determines if any specific ARI commands need to be
+	     run on the channel and will run them from this thread.
+	  3a Checks if the channel is bridged. If the channel is bridged, then
+	     ast_bridge_depart() is called since channels that are added to Stasis
+	     bridges are always imparted as departable.
+	  4a Unlink the stasis_app_control from the container.
+
+	  When an ARI command is received by Asterisk, the following occurs
+	  1b A thread is spawned to handle the HTTP request
+	  2b The stasis_app_control(s) that corresponds to the channel(s) in the
+	     request is/are retrieved. If the stasis_app_control cannot be
+	     retrieved, then it is assumed that the channel in question has exited
+	     the Stasis app or perhaps was never in Stasis in the first place.
+	  3b A command is queued onto the stasis_app_control, and the channel's
+	     event loop thread is signaled to run the command.
+	  4b While most ARI commands do nothing further, some, such as adding or
+	     removing channels from a bridge, will block until the command they
+	     issued has been completed by the channel's event loop.
+
+	  The first race condition that is solved by this patch involves a crash
+	  that can occur due to faulty detection of the channel's bridged status
+	  in step 3a. What can happen is that in step 2a, the event loop may run
+	  the ast_bridge_impart() function to asynchronously place the channel
+	  into a bridge, then immediately exit the event loop because the channel
+	  has hung up. In step 3a, we would detect that the channel was not
+	  bridged and would not call ast_bridge_depart(). The reason that the
+	  channel did not appear to be bridged was that the depart_thread that is
+	  spawned by ast_bridge_impart() had not yet started. That is the thread
+	  where the channel is marked as being bridged. Since we did not call
+	  ast_bridge_depart(), the Stasis application would exit, and then the
+	  channel would be destroyed Then the depart_thread would start up and
+	  try to manipulate the destroyed channel, causing a crash.
+
+	  The fix for this is to switch from using ast_channel_is_bridged() to
+	  checking the NULLity of ast_channel_internal_bridge_channel() to
+	  determine if ast_bridge_depart() needs to be called. The channel's
+	  internal bridge_channel is set when ast_bridge_impart() is called and
+	  is NULLed by the call to ast_bridge_depart(). If the channel's internal
+	  bridge_channel is non-NULL, then the channel must have been imparted
+	  into the bridge and needs to be departed, even if the actual bridging
+	  operation has not yet started. By departing the channel when necessary,
+	  the thread that is running the Stasis application will block until the
+	  bridge gives the okay that the depart_thread has exited.
+
+	  The second race condition that is solved by this patch involves a leak
+	  of HTTP handler threads. The problem was that step 2b would successfully
+	  retrieve a stasis_app_control structure. Then step 2a would exit the
+	  channel from the event loop due to a hangup. Steps 3a and 4a would
+	  execute, and then finally steps 3b and 4b would. The problem is that at
+	  step 4b, when attempting to add a channel to a bridge, the thread would
+	  block forever since the channel would never execute the queued command
+	  since it was finished with the event loop. This meant that the HTTP
+	  handling thread would be leaked, along with any references that thread
+	  may have owned (in my case, I was seeing bridges leaked).
+
+	  The fix for this is to hone in better on when the channel has exited the
+	  event loop. The stasis_app_control structure has an is_done field that
+	  is now set at each point where the channel may exit the event loop. If
+	  step 2b retrieves a valid stasis_app_control structure but the control
+	  is marked as done, then the attempted operation exits immediately since
+	  there will be nothing to service the attempted command.
+
+	  ASTERISK-25091 #close
+	  Reported by Ilya Trikoz
+
+	  Change-Id: If66265b73b4c9f8f58599124d777fedc54576628
+2015-06-17 07:00 +0000 [9668a1acb5]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Remove 'prefetch' option.
+
+	  To prevent confusion I am removing the prefetch option until such
+	  time as it is implemented. All other functionality, however, has
+	  been implemented.
+
+	  ASTERISK-25067
+
+	  Change-Id: I9ce6aa3e5c6c5bc3c5baa8ff90fa036d73939895
+
+2015-06-16 11:13 +0000 [59552c2d08]  Mark Michelson <mmichelson@digium.com>
+
+	* Parking: Add documentation for AMI ParkedCallSwap event.
+
+	  This event was added some time ago in order to clarify when a channel
+	  took the place of another channel in a parking lot. However, there was
+	  no XML documentation added for the event. This patch adds the XML
+	  documentation.
+
+	  ASTERISK-24900 #close
+	  Reported by Rusty Newton
+
+	  Change-Id: I4cfe7777c4b94bbff91c9221c6096a7a02a92eac
+2015-06-15 16:40 +0000 [ea9d5f155e]  Corey Farrell <git@cfware.com>
+
+	* func_pjsip_aor: Fix leaked contact from iterator.
+
+	  ASTERISK-25162 #close
+
+	  Change-Id: Id79aa3c6fe490016ee98efc97ac4c1d3f461f97e
+
+2015-06-12 16:58 +0000 [93ac45d3bd]  Kevin Harwell <kharwell@digium.com>
+
+	* res_pjsip: Add option to force G.726 to be treated as AAL2 packed.
+
+	  Some phones send g.726 audio packed for AAL2, which differs from what is
+	  recommended by RFC 3351. If Asterisk receives audio formatted as such when
+	  negotiating g.726 then it sounds a bit distorted. Added an option to
+	  res_pjsip_endpoint that allows g.726 negotiated audio to be treated as g.726
+	  AAL2 packed.
+
+	  ASTERISK-25158 #close
+	  Reported by: Steve Pitts
+
+	  Change-Id: Ie7e21f75493d7fe53e75e12c971e72f5afa33615
+
+2015-06-14 19:48 +0000 [15c2208701]  Matt Jordan <mjordan@digium.com>
+
+	* main/cdr: Carry over the disable flag when 'disable all' is specified
+
+	  The CDR_PROP function (as well as the NoCDR application) set the
+	  'disable all' flag (AST_CDR_FLAG_DISABLE_ALL) on the current CDR. This
+	  flag is supposed to be applied to all CDRs that are currently in the
+	  chain, as well as all CDRs that may be created in the future. Currently,
+	  however, the flag is only applied to the existing CDRs in the chain; new
+	  CDRs do not receive the 'disable all' flag. In particular, this affects
+	  parallel dials, which generate new CDRs for each pair of channels in
+	  the dial attempt.
+
+	  This patch carries over the 'disable all' flag when it is specified on a
+	  CDR and a new CDR is generated for the chain.
+
+	  ASTERISK-24344 #close
+
+	  Change-Id: I91a0f0031e4d147bdf8a68ecd08304d506fb6a0e
+2015-06-12 14:28 +0000 [b8bc15286f]  Matt Jordan <mjordan@digium.com>
+
+	* main/cdr: Copy context/exten on chained CDRs for parallel dials in subroutines
+
+	  When a parallel dial occurs, a new CDR will be created for each dial
+	  attempt that is made. In most circumstances, the act of creating each
+	  CDR in the chain will include a step that updates the Party A snapshot,
+	  which causes the context/extension of the Party A to be copied onto the
+	  CDR object.
+
+	  However, when the Party A is in a subroutine, we explicitly do *not*
+	  copy the context/extension onto the CDR. This prevents the Macro or
+	  GoSub routine name from blowing away the context/extension that the
+	  channel was originally executing in. For the original CDR, this is not a
+	  problem: the original CDR already recorded the last known 'good' state
+	  of the channel just prior to it going into the subroutine. However, for
+	  newly generated CDRs in a chain, there is no context/extension set on
+	  them. Since we are in a subroutine, we will never set the Party A's
+	  context/extension on the CDR, and we end up with a CDR with no
+	  destination recorded on it.
+
+	  This patch updates the creation of a chained CDR such that it copies
+	  over the original CDR's context/extension. This is the last known "good"
+	  state of the CDR, and is a reasonable starting point for the newly
+	  generated CDR. In the case where we are not in a subroutine, subsequent
+	  code will update the location of the CDR from the Party A information;
+	  in the case where we are in a subroutine, the context/extension on the
+	  original CDR is the correct information.
+
+	  ASTERISK-24443 #close
+
+	  Change-Id: I6a3ef0d6e458d3b9b30572feaec70f2964f3bc2a
+
+2015-06-11 08:18 +0000 [19f60d9412]  Damian Ivereigh <damo@launtel.net.au>
+
+	* chan_sip.c: Update dialog fromtag after request with auth
+
+	  If a client sends and INVITE which is 401 rejected, then subsequently
+	  sends a new INVITE with the auth info and uses a different fromtag
+	  from the first INVITE, Asterisk will accept the new INVITE as part of
+	  the original dialog - match_req_to_dialog() specifically ignores the
+	  fromtag. However it does not update the stored dialog with the new
+	  fromtag.
+
+	  This results in Asterisk being unable to match future packets that are
+	  part of this dialog (such as the ACK to the OK or the OK to the BYE),
+	  and the call is dropped.
+
+	  This problem was originally found when using an NEC-i SV8100-GE (NEC SIP
+	  Card).
+
+	  * After a successful match of a packet to the dialog, if the packet is
+	    not a SIP_RESPONSE, authentication is present and the fromtags are
+	    different, the stored fromtag is updated with the one from the recent
+	    INVITE.
+
+	  ASTERISK-25154 #close
+	  Reported by: Damian Ivereigh
+	  Tested by: Damian Ivereigh
+
+	  Change-Id: I5c16cf3b409e5ef9f2b2fe974b6bd2a45a6aa17e
+
+2015-06-11 18:52 +0000 [bb00b26f35]  Matt Jordan <mjordan@digium.com>
+
+	* chan_pjsip: Set the context and extension on the channel when created
+
+	  Prior to this patch, chan_pjsip was failing to pass the endpoint's
+	  context and the desired extension to the ast_channel_alloc_* routine.
+	  This caused a new channel snapshot to be issued without a context and
+	  extension, which can cause some reporting issues for users of AMI, CEL,
+	  and other APIs. The channel driver would later set the context and
+	  extension on the channel such that the channel would start in the
+	  correct location in the dialplan, but the information reported in the
+	  initial event would be incorrect.
+
+	  This patch modifies the channel driver such that it now passes the
+	  context and extension directly into the allocation routine. This
+	  provides the information in the new channel snapshot published over
+	  Stasis.
+
+	  ASTERISK-25156 #close
+	  Reported by: cloos
+
+	  Change-Id: Ic6f8542836e596db8f662071d118e8f934fdf25e
+
+2015-06-10 18:28 +0000 [7230ee2efe]  Joshua Colp <jcolp@digium.com>
+
+	* bridge: When performing a blonde transfer update connected line information.
+
+	  When performing a blonde transfer the code uses the old masquerade
+	  mechanism to move a channel around. As a result of this certain information,
+	  such as connected line, is moved between the channels involved. Upon
+	  completion of the move a frame is queued which is supposed to update the
+	  connected line information on the channel. This does not occur as the
+	  code considers it a redundant update since the masquerade operation
+	  updated the channel (but did not inform it of the new connected line
+	  information). The code also does not queue a connected line update
+	  to be handled by the thread handling the channel. Without this any
+	  other channel that may be loosely involved does not know it is
+	  talking to a different caller.
+
+	  This change does the following to resolve this:
+
+	  1. The indicated connected line information is cleared upon
+	  completion of the masquerade operation when doing a blonde transfer.
+	  This prevents the connected line update from being considered
+	  redundant.
+
+	  2. A connected line update frame is now queued upon the completion
+	  of the masquerade operation so any other channel loosely involved
+	  knows that there is a different caller.
+
+	  ASTERISK-25157 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: Ibb8798184a1dab3ecd35299faecc420034adbf20
+
+2015-06-11 14:39 +0000 [a657ab12f9]  Richard Mudgett <rmudgett@digium.com>
+
+	* app_directory: Fix crash when using the alias option 'a'.
+
+	  The voicemail.conf mailbox key/value pair is defined as:
+	  <mailbox>=[<password>[,<full-name>[,<email>[,<pager>[,<options>]]]]]
+	  Where all fields in the value including the field values are optional.
+
+	  Since the parsing code for the mailbox key/value pair is sloppy, this
+	  patch tightens the parsing for the directory information.
+
+	  * Renamed the 'pos' and 'bufptr' variables to 'name' and 'options'
+	  respectively in search_directory_sub().  Those names make more sense.
+
+	  * Made sure that search_directory_sub() is dealing with the voicemail.conf
+	  mailbox options field if it even exists when looking for the 'hidefromdir'
+	  and 'alias' options.
+
+	  * Fix crash if a voicemail.conf mailbox is just
+	  <mailbox>=<password>,<name> when the 'a' option is used.  If there were no
+	  fields after the name then the 'options' pointer was not checked for NULL.
+
+	  * Fix users.conf alias processing if the 'a' option is used.  The wrong
+	  variable was used.
+
+	  ASTERISK-25087 #close
+	  Reported by: Chet Stevens
+
+	  Change-Id: I86052ea77307beddddba5279824d39dc0d593374
+
+2015-06-05 15:37 +0000 [30cd559345]  Richard Mudgett <rmudgett@digium.com>
+
+	* DNS: Need to use the same serializer for a pjproject SIP transaction.
+
+	  All send/receive processing for a SIP transaction needs to be done under
+	  the same threadpool serializer to prevent reentrancy problems inside
+	  pjproject when using an external DNS resolver to process messages for the
+	  transaction.
+
+	  * Add threadpool API call to get the current serializer associated with
+	  the worker thread.
+
+	  * Pick a serializer from a pool of default serializers if the caller of
+	  res_pjsip.c:ast_sip_push_task() does not provide one.
+
+	  This is a simple way to ensure that all outgoing SIP request messages are
+	  processed under a serializer.  Otherwise, any place where a pushed task is
+	  done that would result in an outgoing out-of-dialog request would need to
+	  be modified to supply a serializer.  Serializers from the default
+	  serializer pool are picked in a round robin sequence for simplicity.
+
+	  A side effect is that the default serializer pool will limit the growth of
+	  the thread pool from random tasks.  This is not necessarily a bad thing.
+
+	  * Made pjsip_resolver.c use the requesting thread's serializer to execute
+	  the async callback.
+
+	  * Made pjsip_distributor.c save the thread's serializer name on the
+	  outgoing request tdata struct so the response can be processed under the
+	  same serializer.
+
+	  ASTERISK-25115 #close
+	  Reported by: John Bigelow
+
+	  Change-Id: Iea71c16ce1132017b5791635e198b8c27973f40a
+
+2015-06-05 12:16 +0000 [b23f33e7e5]  Richard Mudgett <rmudgett@digium.com>
+
+	* DNS: Fix some corner cases.
+
+	  * Fix query_set destruction before we are done kicking the queries off.
+
+	  * Fixed no queries requested handling.
+
+	  * Add empty queries request unit test.
+
+	  * Added missing allocation check in ast_dns_query_set_add().
+
+	  * Made initial pjsip resolving query vector slightly larger.
+
+	  ASTERISK-25115
+	  Reported by: John Bigelow
+
+	  Change-Id: Ie8be8347d0992e93946d72b6e7b1299727b038f2
+
+2015-06-10 17:51 +0000 [ae589da466]  Richard Mudgett <rmudgett@digium.com>
+
+	* DNS: Remove trailing newline from summary and descriptions.
+
+	  Those trailing newlines mess up test formatting.
+
+	  Change-Id: I5e3f3a55b82c9d7acb9661201d4993d1958f1185
+
+2015-06-05 11:43 +0000 [83bc9d366d]  Richard Mudgett <rmudgett@digium.com>
+
+	* pjsip_resolver.c: Fix debug code to only execute at acceptable debug level.
+
+	  Change-Id: I1716c93d6e097ad28128ecb9e806aac7a4180c8a
+
+2015-06-05 11:41 +0000 [6d49dccd85]  Richard Mudgett <rmudgett@digium.com>
+
+	* DNS: Fix doxygen comments.
+
+	  Change-Id: Icafea3fb4ea64ac027561b23cbfe2b17997dc549
+
+2015-06-09 15:31 +0000 [b705c09dbb]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip.h: Fix some doxygen comments.
+
+	  Change-Id: I4615771077c3c6a0a7273da6d7b5f77af7e8d976
+
+2015-06-05 13:46 +0000 [aa8479778e]  Richard Mudgett <rmudgett@digium.com>
+
+	* taskprocessor.c: Remove extra unref from off-nominal path.
+
+	  Change-Id: Iee3bd8c8a528776056972066698fe735f0f6cf60
+
+2015-05-31 12:37 +0000 [07f5f45e5a]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* res_pjsip_transport_websocket: Fix use-after-free bugs.
+
+	  This patch fixes use-after-free bugs caught by AddressSanitizer.
+
+	  1. PJSIP transport manager may decide to destroy transport on its own.
+	  For example, when the contact registered via websocket has not renewed
+	  its registration in time. The transport was destoyed, but the websocket
+	  listener thread was still active until the socket closes, and then tried
+	  to call transport_shutdown on transport that has been freed.
+
+	  Also, the transport destructor accessed wstransport->rdata.tp_info.pool
+	  right after freeing memory that contained wstransport itself.
+
+	  This patch converts transport to an ao2 object, allowing it to be
+	  refcounted, so that it is available until both websocket listener and
+	  pjsip transport manager are finished with it.
+
+	  2. The websocket listener deletes the last reference on websocket session
+	  when the tcp connection is closed, and it gets destroyed, but
+	  the transport manager may still use it, for example when disconnect
+	  happens in the middle of a SIP transaction.
+
+	  A new reference to websocket session has been added that is released
+	  with the transport to prevent this.
+
+	  ASTERISK-25096 #close
+	  Reported by: Josh Kitchens
+
+	  ASTERISK-24963 #close
+	  Reported by: Badalian Vyacheslav
+
+	  Change-Id: Idc0b63eb6e459c1ddfb2430127d34b3c4d8d373b
+
+2015-06-09 13:41 +0000 [f897f36721]  ibercom <ibercom123@gmail.com>
+
+	* weakref attribute detection broken with gcc 4.6 and higher
+
+	  GCC 4.7 Manual:
+	  http://gcc.gnu.org/onlinedocs/gcc-4.7.4/gcc/Function-Attributes.html
+
+	  weakref ("target")
+
+	  A weak reference is an alias that does not by itself require a definition
+	  to be given for the target symbol.
+
+	  ASTERISK-22559 #close
+	  Reported by: Ibercom
+
+	  Change-Id: I36a136cae947b65187a697533416f9ff9a0b8cdf
+
+2015-06-08 10:09 +0000 [80621ce3c5]  Corey Farrell <git@cfware.com>
+
+	* Fix unsafe uses of ast_context pointers.
+
+	  Although ast_context_find, ast_context_find_or_create and
+	  ast_context_destroy perform locking of the contexts table,
+	  any context pointer can become invalid at any time that the
+	  contexts table is unlocked. This change adds locking around
+	  all complete operations involving these functions.
+
+	  Places where ast_context_find was followed by ast_context_destroy
+	  have been replaced with calls ast_context_destroy_by_name.
+
+	  ASTERISK-25094 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I1866b6787730c9c4f3f836b6133ffe9c820734fa
+
+2015-06-08 09:44 +0000 [53c1126090]  Kevin Harwell <kharwell@digium.com>
+
+	* AMI: Escape string values.
+
+	  So this issue is a bit complicated. Since it is possible to pass values to AMI
+	  that contain a '\r\n' (or other similar sequences) these values need to be
+	  escaped. One way to solve this is to escape the values and then pass the escaped
+	  values to the AMI variable parameter string building function. However, this
+	  puts the onus on the pre-build function to escape all string values. This
+	  potentially requires a fair amount of changes along with a lot of string
+	  allocations/freeing for all values.
+
+	  Surely there is a way to push this complexity down a level into the string
+	  building function itself? This of course is possible, but ends up requiring a
+	  way to distinguish between strings that need to be escaped and those that don't.
+	  The best way to handle this is by introducing a new format specifier in the
+	  format string. For instance a %s (no escape) and %S (escape). However, that is
+	  a bit weird and unexpected.
+
+	  So faced with those possibilities this patch implements a limited version of the
+	  first option. Instead of attempting to escape all string values this patch only
+	  escapes those values that make sense. This approach limits the number of changes
+	  and doesn't suffer from the odd format specifier problem.
+
+	  ASTERISK-24934 #close
+	  Reported by: warren smith
+
+	  Change-Id: Ib55a5b84fe0481b0f2caaaab68c566f392c0aac0
+
+2015-06-02 15:07 +0000 [9fca378b36]  David M. Lee <dlee@respoke.io>
+
+	* Fixes for OS X
+
+	   * Add some type casting so tv_usec can really be a long, instead of
+	     some strange platform specific type.
+
+	   * Add some .dylib style files to .gitignore.
+
+	   * Switch from using -Xlinker to -Wl,. For [reasons unknown][], newer
+	     versions of GCC, when compiling the Homebrew formula for Asterisk,
+	     are not properly passing the -Xlinker options to the linker. Given
+	     that -Wl, does exactly the [same thing][], and does it properly, this
+	     patch changes the -Xlinker options to use -Wl, instead.
+
+	   [reasons unknown]: http://bit.ly/1SUbEYx
+	   [same thing]: https://gcc.gnu.org/onlinedocs/gcc/Link-Options.html
+
+	  Change-Id: Id5e6b3c6cc86282ea5fca630dc3991137c5bf4dd
+
+2015-06-04 07:14 +0000 [d463bac574]  ibercom <ibercom123@gmail.com>
+
+	* CLI: Cosmetic issue - core show uptime
+
+	  Show uptime information ends with an unnecessary space.
+
+	  Now NEEDCOMMA is better defined.
+
+	  Change-Id: I11b360504a0703309ff51772ff8f672287f3c5a1
+
+2015-06-04 13:11 +0000 [128fe4cee8]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Implement expire_on_reload option.
+
+	  This change implements the expire_on_reload option for memory caches.
+	  If enabled and a reload is performed all objects within the cache
+	  will be expired and the cache emptied.
+
+	  ASTERISK-25067
+	  Reported by: Matt Jordan
+
+	  Change-Id: Id46aa1957d660556700e689e195eed57c989b85e
+
+2015-06-02 10:20 +0000 [028edae82e]  Joshua Colp <jcolp@digium.com>
+
+	* test_sorcery_memory_cache_thrash: Add unit tests for thrashing the memory cache.
+
+	  This change adds a CLI command which can perform memory cache thrashing as well
+	  as unit tests which perform thrashing under the following configurations:
+
+	  1. Low number of unique objects that go stale after 1 second
+	  2. Low number of unique objects that expire after 1 second
+	  3. Low number of unique objects which are constantly updated
+	  4. Large number of unique objects which exceed a defined cache size
+	  5. Large number of unique objects which exceed a defined cache size
+	     that also expire and go stale rapidly
+	  6. Large number of unique objects which expire and go stale rapidly
+	  7. Large number of unique objects
+
+	  For all of the above there are a large number of threads constantly
+	  attempting to retrieve random objects and each test runs for a few
+	  seconds.
+
+	  ASTERISK-25067
+	  Reported by: Matt Jordan
+
+	  Change-Id: I8c8ceff977332c80ed4a31f10d694d48552b2f78
+
+2015-06-04 05:33 +0000 [19de2bbc5f]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Add test event when a refresh occurs.
+
+	  This change adds a testsuite event for when a refresh occurs.
+	  This is useful as it provides a guaranteed mechanism of knowing when
+	  it has occurred instead of waiting an arbitrary amount of time.
+
+	  ASTERISK-25067
+	  Reported by: Matt Jordan
+
+	  Change-Id: Iaa6b8d2d6bab7f99ee08e1c8908b8272a8987e65
+
+2015-06-03 20:12 +0000 [6737ded058]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* install_prereq: Check if is installed aptitude otherwise to install.
+
+	  If in Debian or system based, dont have aptitude installed the script do
+	  nothing. This patch checked if aptitude  installed, if not installed.
+
+	  Also, if execute script with all packages installed yet, the script not show
+	  nothing and return exit 1 because the command 'grep' get nothing from pipe from
+	  'awk'.
+
+	  ASTERISK-25113 #close
+	  Reported By: Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	  Change-Id: Iebdff55805d3917166e5e08e0a1e2176f36ff27f
+
+2015-06-03 17:41 +0000 [92ccffd9e6]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip: Prevent access of NULL channels.
+
+	  It is possible to receive incoming requests or responses after the channel
+	  on an ast_sip_session has been destroyed and NULLed out. Handlers of these
+	  sorts of requests or responses need to be prepared for the possibility
+	  that the channel is NULL or else they could cause a crash.
+
+	  While several places have been amended to deal with NULL channels, there
+	  were still a couple of places that needed updating.
+
+	  res_pjsip_dtmf_info.c: When handling incoming INFO requests, we need to
+	  return early if there is no channel on the session.
+
+	  res_pjsip_session.c: When handling a 302 response, we need to stop the
+	  redirecting attempt if there is no channel on the session.
+
+	  ASTERISK-25148 #close
+	  reported by Mark Michelson
+
+	  Change-Id: Id1a75ffc3d0eaa168b0b28188fb54d6cf9fc47a9
+
+2015-06-03 13:17 +0000 [d355ee7ff3]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/location: Fix ref leak in contact_apply_handler
+
+	  contact_apply_handler calls ast_res_pjsip_find_or_create_contact_status
+	  to force the creation of a contact_status object whenever a new
+	  contact is added but it didn't unref the returned object.
+
+	  Added an ao2_cleanup(status) to plug the leak.
+
+	  ASTERISK-25141
+
+	  Change-Id: Icc1401cae142855a1abc86ab5179dfb3ee861c40
+	  Reported-by: Corey Farrell
+
+2015-06-02 13:02 +0000 [6d8dc9bb5c]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip: Remove outgoing authentication code no longer needed.
+
+	  Associated with ASTERISK-25131
+
+	  Change-Id: Iefa3b2066cfd8b108a90d2dd4a64d92c3a195d33
+
+2015-06-02 12:55 +0000 [00a47ffc7e]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session: Fix cherry pick to master compile error.
+
+	  ASTERISK-25131
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I87c9c96ae4a8fe2bc8a0ddea6958a2ad9cefd8e3
+
+2015-06-02 12:27 +0000 [9472bbaa95]  Joerg Sonnenberger <joerg@bec.de>
+
+	* Remove const cast from leaf functions.
+
+	  app_control_register_rule and app_control_unregister_rule lock/unlock
+	  the queue, which is a mutating operation according to the
+	  ao2_lock/_unlock prototype. Depending on the specific (implicit) casts
+	  in SCOPED_LOCK and RAII_VAR, the compiler may warn or not. As the only
+	  callers of those functions do not have the const, get consistent results
+	  by just dropping it.
+
+	  Change-Id: Ib9e6296155a39bc5d627142a3828180c3cfe8fbb
+
+2015-06-02 11:35 +0000 [5f712e82ac]  Joerg Sonnenberger <joerg@bec.de>
+
+	* tcptls.c: Don't use OpenSSL functions when no SSL support is present.
+
+	  Change-Id: I68a85a7fcbdb282140ff333c6274b6763d5f82a3
+2015-06-01 12:08 +0000 [2cd40c2bd7]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_csv.c: Set file name for csv master to the module when (re)loaded.
+
+	  Compute the location for the csv master file when the module is
+	  loaded or reload.  Before it was calculated every time a log
+	  entry was written.
+
+	  Change-Id: I3ed9f6a8f965308099db70b71128f43d4d3f5585
+2015-05-26 13:56 +0000 [5cdcae5240]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_pjsip_session: Fix in-dialog authentication.
+
+	  When the remote peer requires authentication for in-dialog requests then
+	  re-INVITEs to the peer cause the call to be disconnected and other
+	  in-dialog requests to the peer like MESSAGE just don't go through.
+
+	  * Made session_inv_on_tsx_state_changed() handle in-dialog authentication
+	  for re-INVITEs and other methods.  Initial INVITEs cannot be handled here
+	  because the INVITE transaction must be restarted earlier.
+
+	  * Pulled needed code from res/res_pjsip/pjsip_outbound_auth.c in
+	  preparation for removing the file.  The generic outbound authentication
+	  code did not work as well as anticipated.
+
+	  * Created outbound_invite_auth() to only handle initial outbound INVITEs.
+	  Re-INVITEs cannot be handled here.  The re-INVITE transaction is still in
+	  progress and the PJSIP library cannot handle the overlapping INVITE
+	  transactions.  Other method types should not be handled here as this code
+	  only works on outgoing calls and we need to handle incoming and outgoing
+	  calls.
+
+	  ASTERISK-25131 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I12bdd7ddccc819b4ce4b091e826d1e26334601b0
+
+2015-05-30 20:22 +0000 [9f1939ee27]  Corey Farrell <git@cfware.com>
+
+	* pjsip_configuration: Fix leak in persistent_endpoint_update_state.
+
+	  The loop to find the first available contact of an endpoint grabbed
+	  contact from the iterator, then checked for offline state.  This
+	  caused the first contact after the state was found to leak a reference.
+
+	  ASTERISK-25141
+
+	  Change-Id: Id0f1d87410fc63742db0594eb4b18b36e99aec08
+
+2015-05-31 11:33 +0000 [0a5f8c0d73]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* Fix buffer overflow in slin sample frames generation.
+
+	  The length of frames retured by sample functions was twice as large as
+	  real, what caused global buffer overflow caught by AddressSanitizer.
+
+	  ASTERISK-24717 #close
+	  Reported by: Badalian Vyacheslav
+
+	  Change-Id: Iec2fe682aef13e556684912f906bedf7c18229c6
+
+2015-05-29 16:19 +0000 [bef000dd7c]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip/location:  Fix memory leak in permanent_uri_handler
+
+	  When permanent_uri_handler was creating the contact status
+	  object for each contact, it wasn't unreffing it at the
+	  end of the loop.
+
+	  ASTERISK-25141 #close
+	  Reported-by: Corey Farrell
+
+	  Change-Id: I7bb127994677bb3d459f87952f8425c9b9967b12
+
+2015-05-29 14:52 +0000 [82716410a4]  gtjoseph <george.joseph@fairview5.com>
+
+	* Revert "endpoint/stasis: Eliminate duplicate events on endpoint status change"
+
+	  This reverts commit 6fca75bb628dfff2ab112e80b0228cf3ac0b8a05.
+
+	  Change-Id: Ifee026cc63e22c5ac5717c37867a9f036373ae5a
+
+2015-05-26 07:34 +0000 [dfc45254d1]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Add CLI commands and AMI actions.
+
+	  This change adds the following CLI commands and AMI actions:
+
+	  sorcery memory cache show
+	  sorcery memory cache dump
+	  sorcery memory cache expire
+	  sorcery memory cache stale
+
+	  SorceryMemoryCacheExpire
+	  SorceryMemoryCacheExpireObject
+	  SorceryMemoryCacheStale
+	  SorceryMemoryCacheStaleObject
+
+	  These allow both examination and manipulation of sorcery memory
+	  caches from external sources.
+
+	  Cached objects can be explicitly expired from a cache or marked
+	  as stale. If expired they are immediately removed. If marked as
+	  stale they will be background refreshed when next retrieved.
+
+	  ASTERISK-25067
+	  Reported by Matt Jordan
+
+	  Change-Id: I68e03cfd8c34b5e07f4b6ee4fd93a3f4a00a3d9e
+
+2015-05-27 13:22 +0000 [6fca75bb62]  gtjoseph <george.joseph@fairview5.com>
+
+	* endpoint/stasis: Eliminate duplicate events on endpoint status change
+
+	  When an endpoint was created, it's messages were being forwarded to
+	  both the tech endpoint topic and the all endpoints topic.  Since
+	  the tech topic was also forwarded to all, this was resulting in
+	  duplicate messages whenever an endpoint published.  This patch
+	  causes the endpoint to only forward to the tech topic and lets
+	  the tech topic forward to all.
+
+	  To accomplish this, the existing stasis_cp_single_create function
+	  (which both creates and forwards) was cloned and split into 2
+	  functions, one that creates the topic and one that sets up the
+	  forwarding.  This allows endpoint_internal_create to create
+	  the topic from the endpoint_all cache without forwarding it there,
+	  then allows it to do the forward to the tech's topic.
+
+	  ASTERISK-25137 #close
+	  Reported-by: Vitezslav Novy
+	  ASTERISK-25116 #close
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+
+	  Change-Id: I26d7d4926a0861748fd3bdffe316b75b549a801c
+
+2015-05-26 13:01 +0000 [2e54e7227c]  Mark Michelson <mmichelson@digium.com>
+
+	* res_sorcery_memory_cache: Add support for refreshing stale objects.
+
+	  This change introduces a check of object_lifetime_stale when retrieving
+	  cached objects. If the amount of time the object has been in the cache
+	  exceeds the lifetime, then a task is scheduled to update the cached
+	  object based on an object retrieved from other sorcery wizards instead.
+
+	  To prevent the cached object from being retrieved during a refresh,
+	  thread-local storage is used to mark the thread as being a stale object
+	  update. This results in the cache returning no object, leading to
+	  sorcery querying other wizards for the object instead.
+
+	  A test has been added for stale objects as well. This test ensures that
+	  stale objects are retrieved the same as freshly-cached objects. The test
+	  also ensures that after an object is stale, changes in the backend are
+	  reflected in the cache, to include if the object has been deleted from
+	  the backend.
+
+	  ASTERISK-25067
+	  Reported by Matt Jordan
+
+	  Change-Id: I9bd7c049adf6939bfe2899f393c2bfbbf412d217
+2015-05-21 17:21 +0000 [b8ac683822]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip: Add AMI events for chan_pjsip contact lifecycle changes
+
+	  Add a new ContactStatus AMI event.
+	  Publish the following status/state changes:
+	  Created
+	  Removed
+	  Reachable
+	  Unreachable
+	  Unknown
+
+	  Contact URI, new status/state, aor and endpoint names, and the
+	  last qualify rtt result are included in the event.
+
+	  ASTERISK-25114 #close
+
+	  Change-Id: Id25aae5f7122facba183273efb3e8f36c20fb61e
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+
+2015-05-07 11:18 +0000 [95b186a174]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* res/res_config_pgsql.c: Use PQescapeStringConn for escaping names.
+
+	  Use function PQescapeStringConn for escaping the name of the table and
+	  schema instead of doing it manually.
+
+	  ASTERISK-25132 #close
+	  Reported By: Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	  Change-Id: I302a263f7210d20925f14716b508b081998b7608
+
+2015-05-26 07:44 +0000 [a7af6bca3c]  Joshua Colp <jcolp@digium.com>
+
+	* sorcery: Fix cache creation callback.
+
+	  The cache creation callback function expects to receive a sorcery_details
+	  structure and not just a standalone object.
+
+	  Change-Id: I3e4a5a137cb25292eb52d7a14cbb6daa09213450
+
+2015-05-24 13:47 +0000 [23a798fecc]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* Astobj2: Correctly treat hash_fn returning INT_MIN
+
+	  The code in astobj2_hash.c wrongly assumed that abs(int) is always > 0.
+	  However, abs(INT_MIN) = INT_MIN and is still negative, as well as
+	  abs(INT_MIN) % num_buckets, and as a result this led to a crash.
+
+	  One way to trigger the bug is using host=::80 or 0.0.0.128 in peer
+	  configuration section in chan_sip or chan_iax.
+
+	  This patch takes the remainder before applying abs, so that bucket
+	  number is always in range.
+
+	  ASTERISK-25100 #close
+	  Reported by: Mark Petersen
+
+	  Change-Id: Id6981400ad526f47e10bcf7b847b62bd2785e899
+2015-05-23 04:36 +0000 [70d54ab6c4]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* res_pjsip_transport_websocket: Fix crash on receiving large SIP packets
+
+	  Incoming SIP packets larger than PJSIP_MAX_PKT_LEN were themselves
+	  truncated before passing to pjsip_tpmgr_receive_packet, but the length
+	  was passed unaltered, thus causing memory corruption and segfault.
+
+	  ASTERISK-25122 #close
+
+	  Change-Id: I608a6b6b7f229eacc33a0a7d771d18e27e5b08ab
+
+2015-05-22 21:50 +0000 [50044fdc15]  Corey Farrell <git@cfware.com>
+
+	* Stasis: Fix unsafe use of stasis_unsubscribe in modules.
+
+	  Many uses of stasis_unsubscribe in modules can be reached through unload.
+	  These have been switched to stasis_unsubscribe_and_join.
+
+	  Some subscription callbacks do nothing, for these I've created a noop
+	  callback function in stasis.c.  This is used by some modules that monitor
+	  MWI topics in order to enable cache, since the callback does not become
+	  invalid after dlclose it is safe to use stasis_unsubscribe on these, even
+	  during module unload.
+
+	  ASTERISK-25121 #close
+
+	  Change-Id: Ifc2549fbd8eef7d703c222978e8f452e2972189c
+
+2015-05-22 16:52 +0000 [5a1f2a5884]  Corey Farrell <git@cfware.com>
+
+	* Astobj2: Run weakproxy subscription callbacks in reverse order.
+
+	  Modify ao2_weakproxy_subscribe so each new subscription is added
+	  to the head of the list.  This ensures that when other objects
+	  are allocated and use a subscription to the weakproxy for cleanup,
+	  cleanup will occur in the correct order.
+
+	  ASTERISK-25120 #close
+
+	  Change-Id: Ie0476f08ec21330de1b3f5a2dd3d9eb683df3d3d
+
+2015-05-22 12:22 +0000 [f66c41e668]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_pubsub: Note that 'dialog' is also a valid event type for RLS
+
+	  In addition to specifying lists of 'presence' and 'message-summary',
+	  users can also create lists of type 'dialog'. These should be treated in
+	  the same fashion as 'presence'.
+
+	  Change-Id: I583bb69cd9f88b0b29bf09ddaddeac4e84189f6e
+
+2015-05-22 12:18 +0000 [ad7192a8fd]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_pjsip_exten_state: Fix confusing NOTICE message
+
+	  When a SUBSCRIBE request is made to a dialplan hint that doesn't exist,
+	  the current NOTICE message informing users of this swaps the context and
+	  extension parameters. This can cause a bit of confusion.
+
+	  Thanks to CptBurger in #asterisk for helping to point this out.
+
+	  Change-Id: Ie584d1a58ae217385c87a450ca25b55ca0e36e43
+
+2015-05-17 20:36 +0000 [9cffcca5f9]  Matt Jordan <mjordan@digium.com>
+
+	* res/ari: Register Stasis application on WebSocket attempt
+
+	  Prior to this patch, when a WebSocket connection is made, ARI would not
+	  be informed of the connection until after the WebSocket layer had
+	  accepted the connection. This created a brief race condition where the
+	  ARI client would be notified that it was connected, a channel would be
+	  sent into the Stasis dialplan application, but ARI would not yet have
+	  registered the Stasis application presented in the HTTP request that
+	  established the WebSocket.
+
+	  This patch resolves this issue by doing the following:
+	   * When a WebSocket attempt is made, a callback is made into the ARI
+	     application layer, which verifies and registers the apps presented in
+	     the HTTP request. Because we do not yet have a WebSocket, we cannot
+	     have an event session for the corresponding applications. Some
+	     defensive checks were thus added to make the application objects
+	     tolerant to a NULL event session.
+	   * When a WebSocket connection is made, the registered application is
+	     updated with the newly created event session that wraps the WebSocket
+	     connection.
+
+	  ASTERISK-24988 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: Ia5dc60dc2b6bee76cd5aff0f69dd53b36e83f636
+
+2015-05-20 11:11 +0000 [29ef6571cb]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip: Refactor endpt_send_transaction (qualify_timeout)
+
+	  This patch refactors the transaction timeout processing to eliminate
+	  calling the lower level public pjsip functions and reverts to calling
+	  pjsip_endpt_send_request again.  This is the result of me noticing
+	  a possible incompatibility with pjproject-2.4 which was causing
+	  contact status flapping.
+
+	  The original version of this feature used the lower level calls to
+	  get access to the tsx structure in order to cancel the transaction
+	  when our own timer expires. Since we no longer have that access,
+	  if our own timer expires before the pjsip timer, we call the callbacks
+	  and just let the pjsip transaction take it's own course.  When the
+	  transaction ends, it discovers the callbacks have already been run
+	  and just cleans itself up.
+
+	  A few messages in pjsip_configuration were also added/cleaned up.
+
+	  ASTERISK-25105 #close
+
+	  Change-Id: I0810f3999cf63f3a72607bbecac36af0a957f33e
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+2015-05-20 17:35 +0000 [81d375baad]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Add support for object_lifetime_maximum.
+
+	  This makes the "object_lifetime_maximum" option operational.
+
+	  On the addition of an object to an empty memory cache a scheduled
+	  task is created which, when invoked, expires objects from the cache
+	  which have exceeded their lifetime. If more objects have been added
+	  the remaining life of the oldest object is used to schedule the
+	  next invocation of the scheduled task.
+
+	  If the oldest object is removed from the cache before it can be
+	  expired automatically the scheduled task is cancelled, if possible,
+	  and the lifetime of the next oldest is used to schedule the task.
+
+	  If during these two operations no additional objects exist in the
+	  cache then no task is scheduled.
+
+	  An additional unit test has been added which verifies this
+	  functionality.
+
+	  ASTERISK-25067
+	  Reported by: Matt Jordan
+
+	  Change-Id: I87409674674a508e7717ee20739ca15cec6ba7b6
+
+2015-05-20 00:45 +0000 [9e2a582d2d]  demon-ru <serov.d.p@gmail.com>
+
+	* res_pjsip_outbound_registration: Check request URI for line.
+
+	  When an inbound call is received the To header is checked
+	  for the "line" option. Some remote servers will place this
+	  in the request URI instead. This adds an additional check for
+	  the option in the request URI.
+
+	  ASTERISK-25072 #close
+	  Reported by: Dmitriy Serov
+
+	  Change-Id: Id4e44debbb80baad623b914a88574371575353c8
+
+2015-05-20 15:19 +0000 [071b3d43cb]  Mark Michelson <mmichelson@digium.com>
+
+	* res_sorcery_memory_cache: Add support for maximum_objects.
+
+	  This makes the "maximum_objects" option operational.
+
+	  A heap has been added alongside the hash table in the cache. When
+	  objects are added to the cache, they are also added to the heap.
+	  Similarly, when objects are removed from the cache, they are removed
+	  from the heap.
+
+	  The heap's use comes into play when an item is to be added to a "full"
+	  cache. When the cache is full, the oldest item is removed from the
+	  cache, using the heap to determine the oldest item.
+
+	  A unit test has been added that verifies that the maximum_objects option
+	  works as expected and that the oldest object is removed from the cache
+	  when an object beyond the maximum is added.
+
+	  ASTERISK-25067 #close
+	  Reported by Matt Jordan
+
+	  Change-Id: I490658830e9c4cbf0b3051e4cdc4913cf9f1b73a
+
+2015-05-16 17:02 +0000 [f2cc766d81]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_memory_cache: Add basic module implementation.
+
+	  This change adds a basic res_sorcery_memory_cache module which implements
+	  configuration option parsing, configuration file parsing for threading,
+	  sorcery interface implementation, and unit tests.
+
+	  Objects can be added, updated, deleted, and retrieved from the memory
+	  cache. Automatic expiration and stale handling will be added in the
+	  future.
+
+	  Note that unit tests exist within the module itself in case the
+	  threading done as a result of expiration results in asynchronous
+	  actions (which it likely will). Providing access and a notification
+	  mechanism for an external test module would be complicated and
+	  not worth it.
+
+	  ASTERISK-25067 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: Id8a6a357ef5a83d466f81eee56a67d13eeb118b9
+
+2015-05-21 17:51 +0000 [36e5402885]  Corey Farrell <git@cfware.com>
+
+	* res_mwi_external_ami: Use module version of AMI registration.
+
+	  Use ast_manager_register_xml for res_mwi_external_ami manager
+	  actions.  This ensures the module is held open while any of
+	  the actions are being run.
+
+	  ASTERISK-25117 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: Iececfdc2da498b2c32b9e09042f5f12292007ac7
+
+2015-05-21 13:05 +0000 [3e2a994c71]  Matt Jordan <mjordan@digium.com>
+
+	* ARI: Update version to 1.7.0
+
+	  This patch updates the version of ARI to 1.7.0 to reflect the backwards
+	  compatible changes that will be introduced in 13.4.0.
+
+	  Change-Id: I6c36e6144da426412f25828a868e4df916bff60a
+	  (cherry picked from commit 9d8a462356a938eea82e8424242d89a682495b57)
+
+2015-05-20 20:53 +0000 [d067847695]  Corey Farrell <git@cfware.com>
+
+	* Logger: Reset defaults before processing config.
+
+	  Reset options to default values before reloading config.  This ensures
+	  that if a setting is removed or commented out of the configuration file
+	  it is unset on reload.
+
+	  ASTERISK-25112 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: Id24bb1fb0885c2c14cf8bd6f69a0c2ee7cd6c5bd
+
+2015-05-20 19:05 +0000 [31f0d78d7b]  gtjoseph <george.joseph@fairview5.com>
+
+	* app_playback: Suppress warnings on playback if channel hung up
+
+	  If a channel hangs up while an audio file is playing, there's
+	  no need to clutter up the logs with a warning so suppress it
+	  if ast_check_hangup returns true.
+
+	  Also, change warning to debug/2 in file.c if writing a frame
+	  fails.  Same reasoning.
+
+	  Change-Id: I2e66191af3c5b6e951c98e8f1c3fe3cf2cf7ed89
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+
+2015-04-20 16:00 +0000 [83ff268b9e]  Yousf Ateya <y.ateya@starkbits.com>
+
+	* chan_iax2: Prevent deadlock between hangup and sending lagrq/ping
+
+	  channels/chan_iax.c: Prevent the deadlock between iax2_hangup and send_lagrq/
+	  send_ping. This deadlock happens because the scheduled task send_lagrq(or
+	  send_ping) starts execution after the call hangup procedure starts but before
+	  it deletes the tasks in the scheduler.
+
+	  The solution is to delete scheduled lagrq (and ping) task asynchronously
+	  (i.e. schedule AST_SCHED_DEL for these tasks); By this, AST_SCHED_DEL will
+	  be called in a new context (doesn't have callno locked).
+
+	  This commit also cleans up the procedure of sending LAGRQ and PING.
+
+	  main/sched.c: Do not assert when deleting non existant entry from scheduler.
+	  This assert seems to be the reason for a lot of awkward code to avoid it.
+
+	  ASTERISK-24983 #close
+	  Reported by: Y Ateya
+
+	  Change-Id: I03bec1fc8faacb89630269e935fa667c6d6c080c
+
+2015-05-14 15:21 +0000 [7bf88eb60d]  Kevin Harwell <kharwell@digium.com>
+
+	* audiohook.c: Difference in read/write rates caused continuous buffer resets
+
+	  Currently, everytime a sample rate change occurs (on read or write) the
+	  associated factory buffers are reset. If the requested sample rate on a
+	  read differed from that of a write then the buffers are continually reset
+	  on every read and write. This has the side effect of emptying the buffer,
+	  thus there being no data to read and then write to a file in the case of
+	  call recording.
+
+	  This patch fixes it so that an audiohook_list's rate always maintains the
+	  maximum sample rate among hooks and formats. Audiohook sample rates are
+	  only overwritten by this value when slin native compatibility is turned on.
+	  Also, the audiohook sample rate can only overwrite the list's sample rate
+	  when its rate is greater than that of the list or if compatibility is
+	  turned off. This keeps the rate from constantly switching/resetting.
+
+	  ASTERISK-24944 #close
+	  Reported by: Ronald Raikes
+
+	  Change-Id: Idab4dfef068a7922c09cc631dda27bc920a6c76f
+
+2015-05-13 09:55 +0000 [5ce54ed74a]  Matt Jordan <mjordan@digium.com>
+
+	* res/res_http_websocket: Add a pre-session established callback
+
+	  This patch updates http_websocket and its corresponding implementation
+	  with a pre-session established callback. This callback allows for
+	  WebSocket server consumers to be notified when a WebSocket connection is
+	  attempted, but before we accept it. Consumers can choose to reject the
+	  connection, if their application specific logic allows for it.
+
+	  As a result, this patch pulls out the previously private
+	  websocket_protocol struct and makes it public, as
+	  ast_websocket_protocol. In order to preserve backwards compatibility
+	  with existing modules, the existing APIs were left as-is, and new APIs
+	  were added for the creation of the ast_websocket_protocol as well as for
+	  adding a sub-protocol to a WebSocket server.
+
+	  In particular, the following new API calls were added:
+	  * ast_websocket_add_protocol2 - add a protocol to the core WebSocket
+	    server
+	  * ast_websocket_server_add_protocol2 - add a protocol to a specific
+	    WebSocket server
+	  * ast_websocket_sub_protocol_alloc - allocate a sub-protocol object.
+	    Consumers can populate this with whatever callbacks they wish to
+	    support, then add it to the core server or a specified server.
+
+	  ASTERISK-24988
+	  Reported by: Joshua Colp
+
+	  Change-Id: Ibe0bbb30c17eec6b578071bdbd197c911b620ab2
+
+2015-05-20 12:55 +0000 [ddb7cbef8e]  John Bigelow <jbigelow@digium.com>
+
+	* res/res_resolver_unbound.c: Add missing include of signal.h
+
+	  ASTERISK-25110 #close
+	  Reported by: John Bigelow
+
+	  Change-Id: I99a9d93f066f265357b647b8e99a75e45da5a39f
+
+2015-05-06 21:18 +0000 [9c3c7797e5]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cel, cdr: Assigned separator for column name and values.
+
+	  Use a separator string between column names and values for SQL sentences
+	  instead of evaluating the separator to use each time.
+
+	  This change adds a space after the comma in constructing SQL sentences.
+	  Before the SQL was created like "INSERT INTO cdr(calldate,clid,dst"
+	  without spaces between column name and values.
+
+	  The files applied this change are cdr/cdr_adaptive_odbc.c, cdr/cdr_pgsql.c,
+	  cel/cel_odbc.c
+
+	  ASTERISK-25109 #close
+	  Reported By: Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	  Change-Id: Ia5a1a161f5e26e1643703b30f8cc9cf0860cc7ea
+
+2015-05-17 07:15 +0000 [d8698b7f3f]  Matt Jordan <mjordan@digium.com>
+
+	* doxygen: Fix doxygen errors
+
+	  This patch fixes a number of errors and warning messages in the doxygen
+	  log. Specifically, it addresses:
+	  * A number of files incorrectly places a '\brief' tag immediately after
+	    a '\file' tag. Doing so emits a warning, as '\file' takes an optional
+	    argument specifying which file the doxygen comment is for. As '\brief'
+	    is not a file, doxygen was unamused.
+	  * A grouping of Stasis Topics and Messages in rtp_engine.h was
+	    incorrectly terminated. We now correctly terminate the grouping, which
+	    prevents members of rtp_engine.h from showing up in the wrong group.
+	  * Group indicators which are not part of the Stasis Topics and Messages
+	    group were removed. Group indicators without an \addtogroup or
+	    \ingroup have no meaning.
+
+	  Change-Id: Ia1415ffec6767e27233ae1cae5ed5970de5656d4
+
+2015-05-19 13:01 +0000 [d2e998cd68]  Corey Edwards <tensai@zmonkey.org>
+
+	* main/sdp_srtp.c: allow SDP crypto tag to be up to 9 digits
+
+	  ASTERISK-24887 #close
+	  Reported by: Makoto Dei
+	  Tested by: tensai
+
+	  Change-Id: I6a96f572adb17f76b3acafe503a01c48eb5dd9bf
+2015-05-14 22:05 +0000 [17129d2c29]  snuffy <snuffy22@gmail.com>
+
+	* chan_pjsip: Fix crash during off-nominal when no endpoint specified.
+
+	  Add missing return -1 when no endpoint name is specified.
+
+	  ASTERISK-25086 #close
+	  Reported by: snuffy
+
+	  Change-Id: I9de76c2935a1f4e3f0cffe97a670106f5605e89e
+2015-05-14 18:01 +0000 [5d93928175]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip_config_wizard/config: Fix template processing
+
+	  The config wizard was always pulling the first occurrence of
+	  a variable from an ast_variable list but this gets the template
+	  value from the list instead of any overridden value.  This patch
+	  creates ast_variable_find_last_in_list() in config.c and updates
+	  res_pjsip_config_wizard to use it instead of
+	  ast_variable_find_in_list.  Now the overridden values, where they
+	  exist, are used instead of template variables.
+
+	  Updated test_config to test the new API.
+
+	  ASTERISK-25089 #close
+
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  Change-Id: Ifa7ddefc956a463923ee6839dd1ebe021c299de4
+
+2015-05-15 01:54 +0000 [e48d29054f]  snuffy <snuffy22@gmail.com>
+
+	* cdr: Fix 'core show channel' CDR variable truncation.
+
+	  When the new Bridging API was implemented, the workspace variable
+	  changed to a malloc'd string, causing sizeof() to always be 8 (char).
+
+	  Revert back to stored on stack string for workspace.
+
+	  ASTERISK-25090 #close
+
+	  Change-Id: I51e610ae87371df771ce7693a955510efb90f8f7
+2015-05-10 09:55 +0000 [8f3f414d8c]  Alexander Traud (License 6520)
+
+	* tcptls: Enable multiple TLS certificate chains (RSA+ECC+DSA) for server socket.
+
+	  When a client connects to a server via SSL/TLS, the server commonly utilizes an
+	  RSA key-pair. However, other such algorithms exist (i.e. DSA and ECDSA), and if
+	  the server socket is configured with a certificate for either one of those, it
+	  would lose its compatibility with RSA-only clients.
+
+	  Now, the server socket can be configured with up to one RSA, ECDSA and DSA key
+	  each. For example, if a client is not compatible with SHA-2 hashed certificates
+	  like Nokia mobile phones, the server socket still can use RSA/SHA-1 for legacy
+	  clients and ECDSA/SHA-2 for everyone else.
+
+	  ASTERISK-24815 #close
+	  Reported by: Alexander Traud
+	  patches:
+	    tls_rsa_ecc_dsa.patch uploaded by Alexander Traud (License 6520)
+
+	  Change-Id: Iada5e00d326db5ef86e0af7069b4dfa1b979da9a
+
+2015-05-14 17:12 +0000 [2415a14ce9]  Maciej Szmigiero <mail@maciej.szmigiero.name>
+
+	* Add X.509 subject alternative name support to TLS certificate
+	  verification.
+
+	  This way one X.509 certificate can be used for hosts that
+	  can be reached under multiple DNS names or for multiple hosts.
+
+	  Signed-off-by: Maciej Szmigiero <mail@maciej.szmigiero.name>
+
+	  ASTERISK-25063 #close
+
+	  Change-Id: I13302c80490a0b44c43f1b45376c9bd7b15a538f
+
+2015-05-13 15:41 +0000 [3e89f01b55]  Jonathan Rose <jrose@digium.com>
+
+	* Message.c: Clear message channel frames on cleanup
+
+	  The message channel is a special channel that doesn't actually process frames.
+	  However, certain actions can cause frames to be placed in the channel's read
+	  queue including the Hangup application which is called on the channel after
+	  each message is processed. Since the channel will continually be reused for
+	  many messages, it's necessary to flush these frames at some point.
+
+	  ASTERISK-25083 #close
+	  Reported by: Jonathan Rose
+
+	  Change-Id: Idf18df73ccd8c220be38743335b5c79c2a4c0d0f
+
+2015-05-14 00:06 +0000 [0a46d43b9c]  Corey Farrell <git@cfware.com>
+
+	* Fix potential crash after unload of func_periodic_hook or test_message.
+
+	  These modules save a pointer to the context they create on load, and
+	  use that pointer to destroy the context at unload.  It is not safe
+	  to save this pointer, it is replaced during load of pbx_config,
+	  pbx_lua or pbx_ael.
+
+	  This change causes the modules to pass NULL to ast_context_destroy,
+	  a safer way to perform the unregistration since it does not use
+	  a pointer that could become invalid.
+
+	  ASTERISK-25085 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I6a00ec8e38046058f97dc703e1adcde9bf517835
+
+2015-05-12 08:58 +0000 [478fb4a388]  Corey Farrell <git@cfware.com>
+
+	* MALLOC_DEBUG: Replace WRAP_LIBC_MALLOC with ASTMM_LIBC.
+
+	  There are 3 ways that calls directly to standard allocator functions can
+	  be dealt with:
+	  1. Block their use, cause them to generate an error.  This is the default.
+	  2. Replace them with the Asterisk equivalent function calls.
+	  3. Leave them alone.
+
+	  This change allows one of these 3 options to be selected by any source.
+	  The source just needs to define ASTMM_LIBC to ASTMM_BLOCK, ASTMM_REDIRECT,
+	  or ASTMM_IGNORE to use option 1, 2 or 3 respectively.  Normally ASTMM_BLOCK
+	  is the correct option, so it is default when ASTMM_LIBC is not defined.
+	  In some cases when building 3rd party code it is desirable to have it use
+	  Asterisk functions, without changing the whole source - ASTMM_REDIRECT
+	  accomplishes this.  When using 3rd party libraries sometimes a static
+	  inline function will make use of malloc or free.  In these cases it may
+	  be unsafe to replace the allocator in the header, as it's possible the
+	  memory could be freed by the library using standard allocators.  For
+	  those cases ASTMM_IGNORE is needed.
+
+	  Change-Id: I8afef4bc7f3b93914263ae27d3a5858b69663fc7
+
+2015-05-05 19:49 +0000 [eec010829a]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* AST_MODULE_INFO: Format corrections to the usages of AST_MODULE_INFO macro.
+
+	  Change-Id: Icf88f9f861c6b2a16e5f626ff25795218a6f2723
+2015-05-06 05:28 +0000 [46bb8449e8]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cel/cel_pgsql.c: Use the 'SEP' macro when appending a column name
+
+	  When appending a column name to the sql buffer, the predicate, "if first is
+	  non-null, use empty string; else, use comma", is identical to the 'SEP' macro
+	  definition. Since they are the same, this patch replaces the redundant
+	  predicate statement with the 'SEP' macro.
+
+	  Change-Id: Ib8b6138b06a48381723108a05ab8752cb8700509
+2015-05-12 17:45 +0000 [0d97d7cb94]  Jonathan Rose <jrose@digium.com>
+
+	* app_voicemail: fix moving when old messages full
+
+	  When completing voicemail playback of a message in the 'INBOX', the
+	  message gets moved to the 'Old' messages folder. Without this patch, if
+	  the 'Old' folder is already at its set limit, then the 'INBOX' message will
+	  simply be deleted. With this patch, the flag to delete the message will be
+	  removed if the save_to_folder function indicates that the message could
+	  not be moved due to a full folder.
+
+	  ASTERISK-25082 #close
+	  Reported by: Jonathan Rose
+	  Review: https://gerrit.asterisk.org/#/c/448/
+
+	  Change-Id: I2be440a09f42e2d06d50975c40d1ad7f836ecb3f
+2015-05-12 17:34 +0000 [0bb0d4a603]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi/sig_pri: Fix crash on ISDN call hangup collision.
+
+	  If an ISDN call is hungup by both sides at the same time a crash could
+	  happen.
+
+	  * Added missing NULL checks for the owner channel after calling
+	  pri_queue_pvt_cause_data() in two places.  Code after those calls need to
+	  check the owner channel pointer for NULL before use because
+	  pri_queue_pvt_cause_data() needs to do deadlock avoidance to lock the
+	  owner and the owner may get hung up.
+
+	  ASTERISK-21893 #close
+	  Reported by:  Alexandr Gordeev
+
+	  Change-Id: Ica3e266ebc7a894b41d762326f08653e1904bb9a
+
+2015-05-06 08:31 +0000 [57386dcb67]  Corey Farrell <git@cfware.com>
+
+	* Allow command-line options to override asterisk.conf.
+
+	  Previous versions of Asterisk processed command-line options before
+	  processing asterisk.conf.  This meant that if an option was set in
+	  asterisk.conf, it could not be overridden with the equivelent command
+	  line option.  This change causes Asterisk to process the command-line
+	  twice.  First it processes options that are needed to load asterisk.conf,
+	  then it processes the remaining options after the config is read.
+
+	  This changes the function of -X slightly.  Previously using -X without
+	  disabling execincludes in asterisk.conf caused #exec to be usable in any
+	  config.  Now -X only enables #exec for the load of asterisk.conf, if it
+	  is wanted in the rest of the system it must be enabled with execincludes
+	  in asterisk.conf.  Updated 'asterisk -h' and 'man asterisk' to reflect
+	  the limited function of -X.
+
+	  ASTERISK-25042 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I1450d45c15b4467274b871914d893ed4f6564cd7
+
+2015-05-05 15:32 +0000 [52407088f8]  gtjoseph <george.joseph@fairview5.com>
+
+	* sorcery: Add API to insert/remove a wizard to/from an object type's list
+
+	  Currently you can 'apply' a wizard to an object type but the wizard
+	  always goes at the end of the object type's wizard list.  This patch
+	  adds a new ast_sorcery_insert_wizard_mapping function that allows
+	  you to insert a wizard anyplace in the list.  I.E.  You could
+	  add a caching wizard to an object type and place it before all
+	  wizards.
+
+	  ast_sorcery_get_wizard_mapping_count and
+	  ast_sorcery_get_wizard_mapping were added to allow examination
+	  of the mapping list.
+
+	  ast_sorcery_remove_mapping was added to remove a mapping by name.
+
+	  As part of this patch, the object type's wizard list was converted
+	  from an ao2_container to an AST_VECTOR_RW.
+
+	  A new test was added to test_sorcery for this capability.
+
+	  ASTERISK-25044 #close
+
+	  Change-Id: I9d2469a9296b2698082c0989e25e6848dc403b57
+
+2015-05-12 01:31 +0000 [cc853dcf90]  Corey Farrell <git@cfware.com>
+
+	* Fix processing of asterisk.conf debug=yes.
+
+	  The code which reads asterisk.conf supports processing the debug
+	  option with ast_true, but ast_true returns -1.  This causes debug
+	  to still be off, convert to 1 so debug will be on as requested.
+
+	  ASTERISK-25042
+	  Reported by: Corey Farrell
+
+	  Change-Id: I3c898b7d082d914b057e111b9357fde46bad9ed6
+
+2015-05-10 02:26 +0000 [c624e4bae1]  Sebastian Kemper <sebastian_ml@gmx.net>
+
+	* General: Fix recent menuselect-related cross compile regression
+
+	  MAKE_MENUSELECT currently sets CC to CC, which is the compiler for the
+	  target platform. But menuselect is to be run on the build system, so
+	  BUILD_CC needs to be used instead - like it was in the past, before the
+	  recent changes (https://reviewboard.asterisk.org/r/4370/). This is the
+	  patch for ASTERISK-25074.
+
+	  ASTERISK-25074 #close
+	  Reported by: Sebastian Kemper
+	  Tested by: Sebastian Kemper
+
+	  Change-Id: I8a2b1fc5deb6ad2b80f49baca35b1b13d468ebf8
+2015-05-01 12:22 +0000 [e6daafb8a6]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr_pgsql, cel_pgsql: Store maximum buffer size to prevent reallocation
+
+	  The code previously used a fixed size of 512 for the SQL
+	  queries. Depending on the size this may require it to grow.
+
+	  This change makes it so if the buffer size does grow the size
+	  is stored and next time the buffer will be large enough.
+
+	  Change-Id: I55385899f1c06dee47e4274c2d21538037b2d895
+2015-05-09 16:58 +0000 [87d8b36755]  gtjoseph <george.joseph@fairview5.com>
+
+	* vector:  Add REMOVE, ADD_SORTED and RESET macros
+
+	  Based on feedback from Corey Farrell and Y Ateya, a few new
+	  macros have been added...
+
+	  AST_VECTOR_REMOVE which takes a parameter to indicate if
+	  order should be preserved.
+
+	  AST_VECTOR_ADD_SORTED which adds an element to
+	  a sorted vector.
+
+	  AST_VECTOR_RESET which cleans all elements from the vector
+	  leaving the storage intact.
+
+	  Change-Id: I41d32dbdf7137e0557134efeff9f9f1064b58d14
+
+2015-05-11 07:07 +0000 [e6ebddd9ae]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* pbx/pbx_spool: Fix issue when call files were executed too early
+
+	  pbx_spool used to delete/move the call file upon successful outgoing
+	  call completion, but did not delete it from in-memory list of files
+	  (dirlist, used only when compiled with inotify/kqueue support).
+	  That resulted in an extra attempt to process that filename after
+	  retrytime seconds.
+	  Then, if a new file with the same name appears that is scheduled
+	  in future further than the completed one plus its retrytime,
+	  then it gets executed earlier than expected.
+
+	  This patch fixes remove_from_queue function to also remove the entry
+	  from the dirlist.
+
+	  ASTERISK-17069 #close
+	  Reported by: Jeremy Kister
+
+	  ASTERISK-24442 #close
+	  Reported by: tootai
+
+	  Change-Id: If9ec9b88073661ce485d6b008fd0b2612e49a28b
+
+2015-05-01 23:43 +0000 [c61b146238]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr_pgsql: Use PQescapeStringConn for escaping names.
+
+	  Use function PQescapeStringConn for escaping the name
+	  of the table and schema instead of doing it manually.
+
+	  Change-Id: I6709165e2d00463e9c813d24f17830ad4910b599
+2015-05-10 07:37 +0000 [2ab5d22c0d]  Yousf Ateya <y.ateya@starkbits.com>
+
+	* res_rtp_asterisk: Correction for the limit which detects that a packet is DTLS.
+
+	  First byte of DTLS packet shall be in range 20-63, not 20-64. Refer to RFC
+	  https://tools.ietf.org/html/rfc5764#section-5.1.2 for correct values.
+
+	  Change-Id: Iae6fa0d72b37c36a27fe40686e0ae6fba3afec31
+
+2015-05-10 08:36 +0000 [f82bd76e3c]  Joshua Colp <jcolp@digium.com>
+
+	* dns_srv: Fix SRV sorting when records with priority zero exist with non-zero.
+
+	  The DNS SRV sorting code currently has an issue when records with a priority
+	  of zero exist with records of a non-zero priority. This occurs because the
+	  sorting code considers zero to mean unset when in reality is a valid
+	  value. If the current priority is zero it will get replaced with any remaining
+	  record that has a priority of non-zero, until no records of those exist after
+	  which the records of priority zero are handled.
+
+	  This change makes it so that the priority of the first remaining record is
+	  the current starting priority. There is also a small optimization to prevent
+	  iterating records when the starting priority is already zero.
+
+	  Change-Id: I103511f35b50428f770bd4db3ffef70fb6f82d35
+
+2015-05-08 18:01 +0000 [1503d0c14c]  Alexandre Fournier <alexandre.fournier@kiplink.fr>
+
+	* res_config_mysql: Fix broken column type checking
+
+	  MySQL configuration engine contains a bug in require_mysql(). This
+	  function is used for column type checking in tables. This bug only
+	  affects DATETIME, DATE and FLOAT types.
+
+	  It came from mixing the first condition (switch-case-like
+	  if/then/else), to check the expected column type, with the second
+	  condition, to check the actual column type against the expected column
+	  type. Both conditions must be checked separately in order to avoid the
+	  execution of the wrong block.
+
+	  ASTERISK-18252 #comment This patch might fix the issue
+	  Reported by: Gareth Blades
+
+	  ASTERISK-25041 #close
+	  Reported by: Alexandre Fournier
+	  Tested by: Alexandre Fournier
+
+	  Change-Id: I0b8bf7e68ab938be8e6525a249260cb648cb0bfa
+
+2015-05-08 14:47 +0000 [5e361e1476]  Rusty Newton <rnewton@digium.com>
+
+	* configs/basic-pbx: Modified main IVR to play new Allison prompt.
+
+	  The main IVR was playing demo-congrats. I've switched it over to the
+	  basic-pbx-ivr-main file that we added in core sounds 1.4.27. This prompt
+	  has Allison prompting the user with the actual IVR menu.
+
+	  ASTERISK-24892 #close
+
+	  Change-Id: Ifb749616ff8e156a1031ddaddfcc9244767a095d
+
+2015-05-08 12:30 +0000 [2d4dc0c963]  Corey Farrell <git@cfware.com>
+
+	* Fix error's produced by astmm.h when standard allocators are used.
+
+	  astmm.h includes defines that are meant to cause error's when standard
+	  allocators (malloc, calloc, free, etc) are used.  It actually only
+	  causes a warning, which is not always caught on certain sources.  In
+	  modules this unknown symbol is not detected until runtime, where the
+	  module fails to load.  This modifies the define's so that using one
+	  of the blocked functions will cause a compile error regardless of
+	  CFLAGS.
+
+	  Moved spandsp header includes to before asterisk.h so the static inline
+	  functions can continue using malloc and free.  Although these functions
+	  are never called and optimized away, the updated replacement macro's
+	  would still cause a failure.
+
+	  Change-Id: I532640aca0913ba9da3b18c04a0f010ca1715af5
+
+2015-05-08 10:39 +0000 [63c71c9f4a]  Sean Bright <sean@malleable.com>
+
+	* res_rtp_asterisk: Issue ERROR if res_srtp is not found.
+
+	  While trying to get WebRTC working with chan_pjsip, I was running
+	  into the following error:
+
+	      Attempted to set an invalid DTLS-SRTP configuration on RTP
+	      instance...
+
+	  Josh helpfully pointed out that res_srtp.so might not be loaded, and
+	  sure enough, it wasn't. This patch adds a ERROR indiciating as much
+	  to hopefully help others having a similar problem.
+
+	  Change-Id: I13aa477b47b299876728a21b130998a0ea6cd19f
+
+2015-05-07 17:49 +0000 [60bf9ed91a]  Rusty Newton <rnewton@digium.com>
+
+	* sounds: Add Swedish sounds to Makefile and XML
+
+	  Added the necessary lines to the Makefile and sounds.xml so we'll have the
+	  Swedish sounds in all available formats in menuselect.
+
+	  See also: Swedish sounds were added into the core sounds release 1.4.27.
+
+	  ASTERISK-24744 #close
+
+	  Reported by: Tove Hjelm
+	  Tested by: Rusty Newton
+
+	  Change-Id: Ib6f4fd177afd1667b2402735034001d4d055a908
+
+2015-05-08 10:30 +0000 [f93b3a22d6]  Corey Farrell <git@cfware.com>
+
+	* Fix crash in codec_lpc10 when MALLOC_DEBUG is enabled.
+
+	  This switches codecs/lpc10/lpcini.c back to including "asterisk.h"
+	  instead of <stdlib.h>.  lpcini.c allocates memory that is freed by
+	  codec_lpc10.c, so it is important to use MALLOC_DEBUG allocator.
+	  Added #define WRAP_LIBC_MALLOC to the start of the source to prevent
+	  runtime symbol link error's.
+
+	  Change-Id: I74f63fd09fdeb673ee7753122c3bb4722ab6e1ac
+
+2015-05-07 14:54 +0000 [cf637f2510]  gtjoseph <george.joseph@fairview5.com>
+
+	* doc: Make progdocs play nice with git
+
+	  Moved contrib/asterisk-ng-doxygen to doc/asterisk-ng-doxygen.in
+
+	  Changed /Makefile to copy asterisk-ng-doxygen.in to
+	  asterisk-ng-doxygen then modify it with version instead of
+	  modifying asterisk-ng-doxygen directly.  Updated clean
+	  targets as well.
+
+	  Updated /.gitignore and doc/.gitignore.
+
+	  Change-Id: I38712d3e334fa4baec19d30d05de8c6f28137622
+
+2015-05-04 14:43 +0000 [b885f719bf]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* contrib/editors: Fix vim syntax highlighting of comments in config files
+
+	   * Added a lookbehind to one-line comment matcher to skip escaped
+	     semicolons.
+	   * Added support for block comments.
+
+	  Change-Id: Id17dfaeda8ed4be572e8107a0c010066584aaee7
+
+2015-05-06 13:24 +0000 [e33682cae2]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_exten_state: Fix race condition between sending NOTIFY and termination
+
+	  The res_pjsip_exten_state module currently has a race condition between
+	  processing the extension state callback from the PBX core and processing
+	  the subscription shutdown callback from res_pjsip_pubsub. There is currently
+	  no synchronization between the two. This can present a problem as while
+	  the SIP subscription will remain valid the tree it points to may not.
+	  This is in particular a problem as a task to send a NOTIFY may get queued
+	  which will try to use the tree that may no longer be valid.
+
+	  This change does the following to fix this problem:
+
+	  1. All access to the subscription tree is done within the task that
+	  sends the NOTIFY to ensure that no other thread is modifying or
+	  destroying the tree. This task executes on the serializer for the
+	  subscriptions.
+
+	  2. A reference to the subscription serializer is kept to ensure it
+	  remains valid for the lifetime of the extension state subscription.
+
+	  3. The NOTIFY task has been changed so it will no longer attempt
+	  to send a NOTIFY if the subscription has already been terminated.
+
+	  ASTERISK-25057 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: I0b3cd2fac5be8d9b3dc5e693aaa79846eeaf5643
+
+2015-05-05 20:22 +0000 [c886be5df2]  gtjoseph <george.joseph@fairview5.com>
+
+	* vector:  Additional enhancements and fixes
+
+	  After using the new vector stuff for real I found...
+
+	  A bug in AST_VECTOR_INSERT_AT that could cause a seg fault.
+
+	  The callbacks needed to be closer to ao2_callback in behavior
+	  WRT to CMP_MATCH and CMP_STOP behavior and the ability to return
+	  a vector of matched entries.
+
+	  A pre-existing issue with APPEND and REPLACE was also fixed.
+
+	  I also added a new macro to test.h that acts like ast_test_validate
+	  but also accepts a return code variable and a cleanup label.  As well
+	  as printing the error, it sets the rc variable to AST_TEST_FAIL and
+	  does a goto to the specified label on error.  I had a local version
+	  of this in test_vector so I just moved it.
+
+	  ASTERISK-25045
+
+	  Change-Id: I05e5e47fd02f61964be13b7e8942bab5d61b29cc
+
+2015-05-06 17:37 +0000 [1f5db1c7e3]  Kevin Harwell <kharwell@digium.com>
+
+	* res_stasis_snoop: Spying on a single direction continually increases CPU
+
+	  Creating a snoop channel in ARI and spying only on a single direction (in or
+	  out) results in CPU utilization continually increasing until the CPU is fully
+	  consumed. This occurs because frames are being put in the opposing direction's
+	  slin factory queue, but not being removed.
+
+	  Fixed the problem by always reading and disposing of frames from the opposite
+	  queue of the direction selected.
+
+	  ASTERISK-24938 #closes
+
+	  Change-Id: I935bfd15f1db958f364d9d6b3b45582c0113dd60
+
+2015-05-06 16:00 +0000 [7103b374ef]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi: Improve force_restart_unavailable_chans option description.
+
+	  ASTERISK-25034
+	  Reported by: Richard Mudgett
+
+	  Change-Id: I1ff8f02124d2f4abd632a050da52c64285bb7f30
+
+2015-05-06 04:32 +0000 [d2e2271874]  Joshua Colp <jcolp@digium.com>
+
+	* manager: Fix build due to missing variable usage.
+
+	  Change-Id: I26d4d2cb9cee924632ff59ef0b30a7e6a1e2b00d
+
+2015-05-04 20:11 +0000 [6b40bbf5bb]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* main/manager.c: Bugfix sort action_manager by alphabetically
+
+	  Fix the alphabetic order added on ast_manager_register_struct. The order
+	  for struct manager_action added is not working, this change fixes the
+	  problem.
+
+	  Change-Id: I149da0cd06c3c4445d7516cc303358e9f26f8b4b
+
+2015-05-05 18:17 +0000 [6c4d1c3223]  Richard Mudgett <rmudgett@digium.com>
+
+	* features: Fix crash when transferee hangs up during DTMF attended transfer.
+
+	  A crash happens with this sequence of steps:
+	  1) Party A is connected to party B.
+	  2) Party B starts a DTMF attended transfer.
+	  3) Party A hangs up while party B is dialing party C.
+
+	  When party A hangs up the bridge that party A and party B are in is
+	  dissolved and party B is kicked out of the bridge.  When party B finishes
+	  dialing party C he attempts to move to the new bridge with party C.  Since
+	  party B is no longer in a bridge the attempted move dereferences a NULL
+	  bridge_channel pointer and crashes.
+
+	  * Made the hold(), unhold(), ringing(), and the bridge_move() functions
+	  tolerant of the channel not being in a bridge.  The assertion that party B
+	  is always in a bridge is not true if the bridged peer of party B hangs up
+	  and dissolves the bridge.  Being tolerant of not being in a bridge allows
+	  the peer hangup stimulus to be processed by the FSM.
+
+	  * Made the bridge_move() function return void since where the return value
+	  for a failed move was checked generated a FSM coding ERROR message for a
+	  normal off-nominal condition.
+
+	  * Eliminated most uses of RAII_VAR in bridge_basic.c.
+
+	  ASTERISK-25003 #close
+	  Reported by: Artem Volodin
+
+	  Change-Id: Ie2c1b14e5e647d4ea6de300bf56d69805d7bcada
+
+2015-05-05 14:48 +0000 [90bfc02e84]  Ivan Poddubny <ivan.poddubny@gmail.com>
+
+	* app_queue: Fix queue_log EXITWITHTIMEOUT containing only 1 parameter
+
+	  This patch fixes EXITWITHTIMEOUT queue_log entry to always come with 3
+	  parameters: position, original position and waiting time.
+
+	  ASTERISK-25038 #close
+	  Reported by: Etienne Lessard
+
+	  Change-Id: I0c62045922e26bee2125e93aee1dee17eee79618
+
+2015-05-05 13:34 +0000 [bebf0b9b27]  Joshua Colp <jcolp@digium.com>
+
+	* chan_unistim: Fix build failure due to ACL changes.
+
+	  Change-Id: I57081045c72b9fcf12d5c84493278f9272c31b32
+
+2015-05-05 11:35 +0000 [247fef6653]  Alexander Traud (License 6520)
+
+	* tcptls: Avoiding ERR_remove_state in OpenSSL.
+
+	  ERR_remove_state was deprecated with OpenSSL 1.0.0 and was replaced by 
+	  ERR_remove_thread_state. ERR_load_SSL_strings and ERR_load_BIO_strings were 
+	  called by SSL_load_error_strings already and got removed. These changes allow 
+	  OpenSSL forks like BoringSSL to be used with Asterisk.
+
+	  ASTERISK-25043 #close
+	  Reported by: Alexander Traud
+	  patches:
+	    asterisk_with_BoringSSL.patch uploaded by Alexander Traud (License 6520)
+
+	  Change-Id: If1c0871ece21a7e0763fafbd2fa023ae49d4d629
+2015-05-05 09:47 +0000 [c541923ac3]  Corey Farrell <git@cfware.com>
+
+	* res_ari_bridges: Add missing dependencies.
+
+	  Missed this module in the previous commit.  res_ari_bridges uses symbols
+	  from res_stasis_playback and res_stasis_recording.
+
+	  ASTERISK-25027 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I90bf756abd25adfc4920d2869ebe7feb636b8c5f
+
+2015-05-05 09:27 +0000 [8a3e93a349]  Corey Farrell <git@cfware.com>
+
+	* pbx_config: Register manager actions with module version of macro.
+
+	  Switch manager actions in pbx_config to use the registration macro that
+	  passes the module pointer, allowing pbx_config reference to be bumped
+	  while the manager actions run.
+
+	  ASTERISK-25061 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I422c50dd74814616ac10c5e9c6598a0b1bc2c44e
+
+2015-05-01 22:14 +0000 [cb79b8ab80]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cel_pgsql: Add support for setting schema
+
+	  Add feature to set optional schema parameter on configuration file via
+	  'schema' setting.
+
+	  Fix query to get columns from table while considering schema. If in
+	  the database there exists two tables with same name in distinct schemas
+	  it will return an error when inserting record.
+
+	  ASTERISK-24967 #close
+
+	  Change-Id: I691fd2cbc277fcba10e615f5884f8de5d8152f2c
+
+2015-05-04 12:16 +0000 [11f650c6ac]  Joshua Colp <jcolp@digium.com>
+
+	* stasis: Fix dial masquerade datastore lifetime
+
+	  A recent change went into Asterisk which added reference counts to the
+	  channels stored in a dial masquerade datastore. Unfortunately this
+	  included a reference to the caller in a dialing operation. While all
+	  of the dialed targets have the datastore removed from them upon dialing
+	  completion this did not occur for the caller, causing it to have a
+	  reference to itself that could go never go away (as it depended on
+	  the destruction of the datastore which only happened when the channel
+	  was destroyed). This resulted in the caller channel remaining on the
+	  system despite it having hung up.
+
+	  This change does the following to fix this issue:
+
+	  1. The dial masquerade datastore is now removed from the caller upon
+	  dialing completion, just like the dialed targets.
+	  2. Upon destruction of the caller all the dialed targets are also
+	  removed from the dial masquerade datastore (just in case).
+	  3. The reference to the caller has been removed as it should not be
+	  possible for the datastore to now be valid/useful after the lifetime
+	  of the caller has ended.
+
+	  ASTERISK-25025 #close
+
+	  Change-Id: I1ef4ca5ca04980028604cc2af5d2992ac3431b3f
+
+2015-04-21 17:27 +0000 [a24ce38e5e]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr_adaptive_odbc: Add ability to set character for quoted identifiers.
+
+	  Added the ability to set the character to quote identifiers. This
+	  allows adding the character at the start and end of table and column
+	  names. This setting is configurable for cdr_adaptive_odbc via the
+	  quoted_identifiers in configuration file cdr_adaptive_odbc.conf.
+
+	  ASTERISK-25006
+
+	  Change-Id: I0b9a56b79ca13a727a803d88ed3b8643e37632b8
+
+2015-05-04 22:57 +0000 [39cf642d40]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr: standardizes tab for options of AST_MODULE_INFO
+
+	  Change-Id: I3c6de30b4859717873100092a7c06e206713a301
+
+2015-05-04 16:41 +0000 [df6c1d755f]  Corey Farrell <git@cfware.com>
+
+	* CLI: Enable automatic references to modules.
+
+	  * Pass module to ast_cli_register and ast_cli_register_multiple.
+	  * Add a module reference before executing any CLI callback, remove
+	    the reference when complete.
+
+	  ASTERISK-25049 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I7aafc7c9f2b912918f28fe51d51e9e8a755750e3
+
+2015-05-04 14:26 +0000 [a8bfa9e104]  Corey Farrell <git@cfware.com>
+
+	* Modules: Make ast_module_info->self available to auxiliary sources.
+
+	  ast_module_info->self is often needed to register items with the core.  Many
+	  modules have ad-hoc code to make this pointer available to auxiliary sources.
+	  This change updates the module build process to make the needed information
+	  available to all sources in a module.
+
+	  ASTERISK-25056 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I18c8cd58fbcb1b708425f6757becaeca9fa91815
+
+2015-05-01 19:25 +0000 [6d5941297b]  gtjoseph <george.joseph@fairview5.com>
+
+	* vector:  Traversal, retrieval, insert and locking enhancements
+
+	  Renamed AST_VECTOR_INSERT to AST_VECTOR_REPLACE because it really
+	  does replace not insert.  The few users of AST_VECTOR_INSERT were
+	  refactored.  Because these are macros, there should be no ABI
+	  compatibility issues.
+
+	  Added AST_VECTOR_INSERT_AT that actually inserts an element into the
+	  vector at a specific index pushing existing elements to the right.
+
+	  Added AST_VECTOR_GET_CMP that can retrieve from the vector based
+	  on a user-provided compare function.
+
+	  Added AST_VECTOR_CALLBACK function that will execute a function
+	  for each element in the vector.  Similar to ao2_callback and
+	  ao2_callback_data functions although the vector callback can take
+	  a variable number of arguments.  This should allow easy migration
+	  to a vector where a container might be too heavy.
+
+	  Added read/write locked vector and lock manipulation macros.
+
+	  Added unit tests.
+
+	  ASTERISK-25045 #close
+
+	  Change-Id: I2e07ecc709d2f5f91bcab8904e5e9340609b00e0
+
+2015-05-03 13:55 +0000 [4f4aaa0c30]  Corey Farrell <git@cfware.com>
+
+	* main/test.c: Add test to verify there were no registration errors.
+
+	  This adds a test that will fail if any test failed to register. Also fail
+	  if any test registration produced a warning about missing a leading or
+	  trailing slash.
+
+	  ASTERISK-25053 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I93e50b8fcbcfa7f1f5b41b2c44a51685c09529c3
+
+2015-04-21 11:52 +0000 [ebe371357e]  Martin Tomec <tomec.martin@gmail.com>
+
+	* res_odbc: Use negative connection cache for all connections
+
+	  Apply the negative connection cache setting to all connections,
+	  even those that are not pooled. This ensures that the connection
+	  will not be  re-established before the negative connection cache
+	  time is met.
+
+	  ASTERISK-22708 #close
+
+	  Change-Id: I431cc2e8584ab0b6908b3523d0a0e18c9a527271
+2015-05-03 21:03 +0000 [981084f08c]  Corey Farrell <git@cfware.com>
+
+	* Format Interfaces: Prevent unload except by shutdown.
+
+	  Format interfaces cannot be unregistered, so the modules that provide them
+	  need to be held open except by shutdown.
+
+	  ASTERISK-25054 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: Iadbd9675bf0d30b8fded5a739b163db3ea2db8f3
+
+2015-05-03 20:28 +0000 [75c0aa6979]  Matt Jordan <mjordan@digium.com>
+
+	* contrib/ast-db-manage: Add Postgres ENUM type support in auto DTMF mode update
+
+	  The upgrade script for auto DTMF mode (31cd4f4891ec) added in 88b0fa7755
+	  failed to add ENUM support for Postgres databases. This requires a
+	  specific import from the sqlalchemy.dialects.postgresql package. This
+	  patch corrects this error, which allows for Postgres update scripts to
+	  be generated.
+
+	  ASTERISK-24706
+
+	  Change-Id: I4742ac8efa533cd6f18e0bdd907b339a9aedf015
+
+2015-05-03 13:36 +0000 [1368dae773]  Corey Farrell <git@cfware.com>
+
+	* main/presencestate.c: Add trailing slash to test category.
+
+	  ASTERISK-25053
+	  Reported by: Corey Farrell
+
+	  Change-Id: I8c0375dd0818747b2d2e1ceaea87bfbeb2daf8d4
+
+2015-04-20 13:03 +0000 [305ce3defd]  Diederik de Groot <ddegroot@talon.nl>
+
+	* Update configure.ac/Makefile for clang
+
+	  Created autoconf/ast_check_raii.m4: contains AST_CHECK_RAII which
+	  checks compiler requirements for RAII:
+	  gcc: -fnested-functions support
+	  clang: -fblocks (and if required -lBlocksRuntime)
+	  The original check was implemented in configure.ac and now has it's
+	  own file. This function also sets C_COMPILER_FAMILY to either gcc or
+	  clang for use by makefile
+
+	  Created autoconf/ast_check_strsep_array_bounds.m4 (contains
+	  AST_CHECK_STRSEP_ARRAY_BOUNDS):
+	  which checks if clang is able to handle the optimized strsep & strcmp
+	  functions (linux). If not, the standard libc implementation should be
+	  used instead. Clang + the optimized macro's work with:
+	  strsep(char *, char []), but not with strsepo(char *, char *).
+	  Instead of replacing all the occurences throughout the source code,
+	  not using the optimized macro version seemed easier
+
+	  See 'define __strcmp_gc(s1, s2, l2) in bits/string2.h':
+	  llvm-comment: Normally, this array-bounds warning are suppressed for
+	  macros, so that unused paths like the one that accesses __s1[3] are
+	  not warned about.  But if you preprocess manually, and feed the
+	  result to another instance of clang, it will warn about all the
+	  possible forks of this particular if statement. Instead of switching
+	  of this optimization, another solution would be to run the preproces-
+	  sing step with -frewrite-includes, which should preserve enough
+	  information so that clang should still be able to suppress the diag-
+	  nostic at the compile step later on.
+
+	  See also "https://llvm.org/bugs/show_bug.cgi?id=20144"
+	  See also "https://llvm.org/bugs/show_bug.cgi?id=11536"
+
+	  Makefile.rules: If C_COMPILER_FAMILY=clang then add two warning
+	  suppressions:
+	  -Wno-unused-value
+	  -Wno-parentheses-equality
+	  In an earlier review (reviewboard: 4550 and 4554), they were deemed a
+	  nuisace and less than benefitial.
+
+	  configure.ac:
+	  Added AST_CHECK_RAII() see earlier
+	  Added AST_CHECK_STRSEP_ARRAY_BOUNDS() see earlier
+	  Removed moved content
+
+	  ASTERISK-24917
+	  Change-Id: I12ea29d3bda2254ad3908e279b7effbbac6a97cb
+
+2015-04-28 04:49 +0000 [8886b724ae]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_csv.c: Add a new option to enable columns added in Asterisk 1.8
+
+	  This patch adds a new option to cdr.conf, 'newcdrcolumns', that will handle CDR
+	  columns added in Asterisk 1.8. The columns are:
+	   * peeraccount
+	   * linkedid
+	   * sequence
+	  When enabled, the columns in the database entry will be populated with the data
+	  from the CDR.
+
+	  ASTERISK-24976 #close
+
+	  Change-Id: I51a57063f4ae5e194a9d933a8df45dc8a4534f0b
+2015-05-03 04:39 +0000 [94532b2c22]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* main/asterisk.c: Update Asterisk copyright year
+
+	  Change-Id: I5e75d7f7e2c096d74edd9e8735268a894f4b93ab
+
+2015-05-03 04:09 +0000 [2ed5e6a9ba]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* utils: Remove trailing whitespace
+
+	  Change-Id: I4644f43a6a1ca9b5130cd2a6746772b888eb4f7a
+
+2015-05-02 18:58 +0000 [c3ec5da156]  Corey Farrell <git@cfware.com>
+
+	* Remove unneeded uses of optional_api providers.
+
+	  A few cases exist where headers of optional_api provders are included but
+	  not needed.  This causes unneeded calls to ast_optional_api_use.
+
+	  * Don't include optional_api.h from sip_api.h.
+	  * Move 'struct ast_channel_monitor' to channel.h.
+	  * Don't include monitor.h from chan_sip.c, channel.c or features.c.
+
+	  The move of struct ast_channel_monitor is needed since channel.c depends on
+	  it.  This has no effect on users of monitor.h since channel.h is included
+	  from monitor.h.
+
+	  ASTERISK-25051 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I53ea65a9fc9693c89f8bcfd6120649bfcfbc3478
+
+2015-05-02 02:15 +0000 [44bbdbe3a4]  Corey Farrell <git@cfware.com>
+
+	* res_pjsip_dlg_options: Fix MODULEINFO section.
+
+	  Removed the extra space before "MODULEINFO" in res_pjsip_dlg_options.
+	  This extra space prevented any of the dependencies from being seen by
+	  menuselect, so building with default options would fail if PJSIP was
+	  not installed.
+
+	  This also makes the tool that extracts information for menuselect
+	  tolerant of multiple spaces in the future.
+
+	  ASTERISK-25033 #close
+	  Reported by: Peter Whisker
+
+	  Change-Id: Iccd54846f70c4a7a50cb5bf70b7bb5cb4bab3698
+
+2015-05-01 19:50 +0000 [e4f0a55f7f]  D Tucny <d@tucny.com>
+
+	* term: send proper reset sequence when black background is forced
+
+	  When using the force black background command-line option or configuration
+	  option an invalid reset sequence is sent following a coloured output item 
+	  in the CLI, the result is that the colour is not 'turned off' and continues
+	  until the next non-default coloured text output.
+
+	  A reset sequence is already defined in term.c, but the ast_term_reset
+	  function doesn't use it, instead building it's own invalid sequence and 
+	  returning that.
+
+	  This patch changes that behaviour, removing the building of a reset sequence
+	  and instead using the pre-built constant 'enddata' which is a suitable reset
+	  sequence for this purpose.
+
+	  ASTERISK-24896 #close
+	  Reported by: Dan Tucny
+
+	  Change-Id: I56323899123ae3264900389cae1f5b252aa3bf43
+2015-05-01 13:22 +0000 [8f3cee1258]  Corey Farrell <git@cfware.com>
+
+	* Astobj2: Fix initialization order of refdebug and AO2_DEBUG.
+
+	  This ensures that refdebug is initialized before AO2_DEBUG if
+	  both are enabled, since AO2_DEBUG allocates a container.
+
+	  This change also makes AO2_DEBUG initialization critical, a
+	  failure will abort Asterisk startup.  This is needed since
+	  the failure would be caused by reg_containers allocation
+	  failure, and that would result in a segmentation fault by
+	  ao2_container_register later in startup.
+
+	  ASTERISK-25048 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I9a243ea3fc5653b48b931ba6d61971cb2e530244
+
+2015-04-29 14:49 +0000 [7ac28be04b]  Matt Jordan <mjordan@digium.com>
+
+	* main/pbx: Improve performance of dialplan reloads with a large number of hints
+
+	  The PBX core maintains two hash tables for hints: a container of the
+	  actual hints (hints), along with a container of devices that are watching that
+	  hint (hintdevices). When a dialplan reload occurs, each hint in the hints
+	  container is destroyed; this requires a lookup in the container of devices to
+	  find the device => hint mapping object. In the current code, this performs an
+	  ao2_callback, iterating over each of the device to hint objects in the
+	  hintdevices container. For a large number of hints, this is extremely
+	  expensive: dialplan reloads with 20000 hints could take several minutes
+	  in just this phase.
+
+	  This patch improves the performance of this step in the dialplan reloads
+	  by caching which devices are watching a hint on the hint object itself.
+	  Since we don't want to create a circular reference, we just cache the
+	  name of the device. This allows us to perform a smarter ao2_callback on
+	  the hintdevices container during hint removal, hashing on the name of the
+	  device and returning an iterator to the matching names. The overall
+	  performance improvement is rather large, taking this step down to a number of
+	  seconds as opposed to minutes.
+
+	  In addition, this patch also registers the hint containers in the PBX
+	  core with the astobj2 library. This allows for reasonable debugging to
+	  hash collisions in those containers.
+
+	  ASTERISK-25040 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: Iedfc97a69d21070c50fca42275d7b3e714e59360
+
+2015-04-30 15:54 +0000 [6b208d8c3b]  Corey Farrell <git@cfware.com>
+
+	* Sample Configs: Fix syntax error in pjsip.conf
+
+	  The sample pjsip.conf has a few comment lines that are missing the
+	  semicolons at the start of the comment, causing the config to fail
+	  load.
+
+	  Change-Id: I776a38c916a7df7ee3e072fd0b21dbf4cc457352
+
+2015-04-30 15:20 +0000 [dc23204aca]  Mark Michelson <mmichelson@digium.com>
+
+	* Prevent potential crash on blond transfer.
+
+	  Scenario:
+	  Alice calls Bob. Bob performs a blond transfer to Carol. Carol rejects
+	  the incoming call (or some other immediate circumstance causes Carol not
+	  to answer the call)
+
+	  What occurs in this case is that when the bridge between Alice and Bob
+	  breaks, Alice is told to masquerade into Bob's channel that had placed
+	  the call to Carol. The actual masquerade goes down without a hitch.
+	  However, a channel fixup callback that attempts to publish dial events
+	  over Stasis has a crash. The reason for this crash is that the datastore
+	  on Bob's channel that placed the outbound call to Carol only had a bare
+	  pointer to Carol's channel. Since Carol rejected the incoming call,
+	  Carol's channel has been hung up and freed, meaning accessing her
+	  channel results in a crash.
+
+	  The fix here is simple. The dial fixup code has been altered to hold
+	  references to the involved channels and to drop those references when
+	  freeing data.
+
+	  ASTERISK-25025 #close
+	  Reported by Chet Stevens
+
+	  Change-Id: I54eedda207b8ec7a69263353b43abe5746aea197
+
+2015-04-30 14:40 +0000 [47fa2ad10b]  Corey Farrell <git@cfware.com>
+
+	* Build System: Fix issue with addons moduleinfo.
+
+	  The build system now scans additional sources when generating
+	  moduleinfo for menuselect.  Unfortunately the extra sources
+	  for format_mp3 only exist if downloaded.
+
+	  Use the Makefile macro 'wildcard' to allow moduleinfo generator
+	  to ignore sources that do not exist.
+
+	  Change-Id: I596604713b7345ce994f32197f8f6bfd9bcf4170
+
+2015-04-30 13:42 +0000 [bb6ddb3dc8]  Joshua Colp <jcolp@digium.com>
+
+	* res_ari_device_states: Fix dependency on res_stasis_device_state.
+
+	  The res_ari_device_states module depends on res_stasis_device_state,
+	  not res_stasis_device_states.
+
+	  Change-Id: I26e02ad37f9e36bcc859867e2fad1b90452ec3de
+
+2015-04-28 17:00 +0000 [11ffcf662f]  Mark Michelson <mmichelson@digium.com>
+
+	* Restrict functionality when ACLs are misconfigured.
+
+	  This patch has two main purposes:
+
+	  1) Improve warning messages when ACLs are configured improperly.
+	  2) Prevent misconfigured ACLs from allowing potentially unwanted
+	  traffic.
+
+	  To acomplish point (2) in most cases, whatever configuration object that
+	  the ACL belonged to was not allowed to load.
+
+	  The one exception is res_pjsip_acl. In that case, ACLs are their own
+	  configuration object. Furthermore, the module loading code has no
+	  indication that a ACL configuration had a failure. So the tactic taken
+	  here is to create an ACL that just blocks everything.
+
+	  ASTERISK-24969
+	  Reported by Corey Farrell
+
+	  Change-Id: I2ebcb6959cefad03cea4d81401be946203fcacae
+
+2015-04-29 14:29 +0000 [03c51cf525]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi: Add the chan_dahdi.conf force_restart_unavailable_chans option.
+
+	  Some telco switches occasionally ignore ISDN RESTART requests.  The fix
+	  for ASTERISK-19608 added an escape clause for B channels in the restarting
+	  state if the telco ignores a RESTART request.  If the telco fails to
+	  acknowledge the RESTART then Asterisk will assume the telco acknowledged
+	  the RESTART on the second call attempt requesting the B channel by the
+	  telco.  The escape clause is good for dealing with RESTART requests in
+	  general but it does cause the next call for the restarting B channel to be
+	  rejected if the telco insists the call must go on that B channel.
+
+	  chan_dahdi doesn't really need to issue a RESTART request in response to
+	  receiving a cause 44 (Requested channel not available) code.  Sending the
+	  RESTART in such a situation is not required (nor prohibited) by the
+	  standards.  I think chan_dahdi does this for historical reasons to deal
+	  with buggy peers to get channels unstuck in a similar fashion as the
+	  chan_dahdi.conf resetinterval option.
+
+	  * Add the chan_dahdi.conf force_restart_unavailable_chans compatability
+	  option that when disabled will prevent chan_dahdi from trying to RESTART
+	  the channel in response to a cause 44 code.
+
+	  ASTERISK-25034 #close
+	  Reported by: Richard Mudgett
+
+	  Change-Id: Ib8b17a438799920f4a2038826ff99a1884042f65
+2015-04-29 21:54 +0000 [556653d937]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_csv.c: Refactor, function to write content of csv file.
+
+	  Create a function for write content of CDR on csv files. Before used same
+	  code for write two distinct files (account and master cdr) instead use a
+	  function for thats.
+
+	  Reduced to one lock when files are written.
+
+	  Change-Id: Idce707f4c108083252e0aeb948f421d924953e65
+
+2015-04-30 06:04 +0000 [80aa9aee5d]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_outbound_registration: Fix double unref on error return.
+
+	  When the PJSIP pjsip_regc_send function is invoked and an error
+	  status returned the caller currently decrements the reference count
+	  of the client state that it just incremented, assuming the
+	  registration callback would not have been invoked. In practice
+	  this is not correct. If the failure happens after the transaction
+	  has been set up the callback will still be invoked. This will
+	  cause the reference count to be incorrectly decremented twice, once
+	  by the registration callback and second by the caller of
+	  pjsip_regc_send.
+
+	  This change makes it so that whether the callback is invoked or
+	  not is known by the caller of pjsip_regc_send. Depending on
+	  this it can know whether it is responsible for decrementing the
+	  reference count of the client state or not.
+
+	  ASTERISK-25037 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: I749dc12f3a22115c49c5d7d95ff42a5fa45319de
+
+2015-04-30 02:07 +0000 [7ff3b2d479]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* include/asterisk/channel.h: Fix typo
+
+	  Change-Id: Ie584b85e16a94c255e60d0b1732ef9686464fef3
+
+2015-04-29 16:15 +0000 [39d3e1ef6e]  Matt Jordan <mjordan@digium.com>
+
+	* main/rtp_engine: Fix DTLS double-free introduced by 0b6410c4f8
+
+	  The patch in 0b6410c4f8 did correctly fix a memory leak of the DTLS
+	  structures in the RTP engine. However, when a 'core reload' is issued, a
+	  double free of the memory pointed to by the char *'s in the DTLS
+	  configuration struct can occur, as ast_rtp_dtls_cfg_free does not set
+	  the pointers to NULL when they are freed.
+
+	  This patch sets those pointers to NULL, preventing a second call to
+	  ast_rtp_dtls_cfg_free from corrupting memory.
+
+	  ASTERISK-25022
+
+	  Change-Id: I820471e6070a37e3c26f760118c86770e12f6115
+
+2015-04-29 13:05 +0000 [5d0c182885]  Kevin Harwell <kharwell@digium.com>
+
+	* res_fax: allow 2400 transmission rate according to v.27ter standard
+
+	  A previous set of patches (see: ASTERISK-22790 & ASTERISK-23231) made it so
+	  a v.27 modem was not allowed to have a minimum transmission rate of 2400 bits
+	  per second. This reverts all or some of those patches since according to the
+	  v.27ter standard a rate of 2400 bits per second is also supported.
+
+	  One of the original patches also added 9600 bits per second support for v.27.
+	  This patch also removes that since v.27ter only supports 2400/4800 bits per
+	  second.
+
+	  Also, since Asterisk specifically supports v.27ter the enum was renamed to
+	  better reflect this.
+
+	  ASTERISK-24955 #close
+	  Reported by: Matt Jordan
+
+	  Change-Id: I4b9dfb6bf7eff08463ab47ee1a74224f27cae733
+
+2015-04-28 23:35 +0000 [c9c03998cc]  Corey Farrell <git@cfware.com>
+
+	* Astobj2: Add ao2_weakproxy_ref_object function.
+
+	  This function allows code to run ao2_ref against the real
+	  object associated with a weakproxy.  It is useful when
+	  all of the following conditions are true:
+	  * You have a pointer to weakproxy.
+	  * You do not have or need a pointer to the real object.
+	  * You need to ensure the real object exists and is not
+	    destroyed during a process.
+
+	  In this case it's wasteful to store a pointer to the real
+	  object just for the sake of releasing it later.
+
+	  Change-Id: I38a319b83314de75be74207a8771aab269bcca46
+
+2015-04-27 16:13 +0000 [4f1db2070d]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_outbound_registration: Don't fail on delayed processing.
+
+	  Odd behaviors have been observed during outbound registrations. The most
+	  common problem witnessed has been one where a request with
+	  authentication credentials cannot be created after receiving a 401
+	  response. Other behaviors include apparently processing an incorrect SIP
+	  response.
+
+	  Inspecting the code led to an apparent issue with regards to how we
+	  handle transactions in outbound registration code. When a response to a
+	  REGISTER arrives, we save a pointer to the transaction and then push a
+	  task onto the registration serializer. Between the time that we save the
+	  pointer and push the task, it's possible for the transaction to be
+	  destroyed due to a timeout. It's also possible for the address to be
+	  reused by the transaction layer for a new transaction.
+
+	  To allow for authentication of a REGISTER request to be authenticated
+	  after the transaction has timed out, we now hold a reference to the
+	  original REGISTER request instead of the transaction. The function for
+	  creating a request with authentication has been altered to take the
+	  original request instead of the transaction where the original request
+	  was sent.
+
+	  ASTERISK-25020
+	  Reported by Mark Michelson
+
+	  Change-Id: I756c19ab05ada5d0503175db9676acf87c686d0a
+2015-04-29 10:46 +0000 [ed5715eb39]  Joshua Colp <jcolp@digium.com>
+
+	* res_sorcery_config: Fix build issue due to syntax error.
+
+	  Change-Id: Ic8322f04e37842848ad72cf2871bd0378f67c4ac
+
+2015-04-29 06:46 +0000 [f226bd6f60]  Corey Farrell <git@cfware.com>
+
+	* ARI: Fix missing dependencies.
+
+	  ARI modules that are generated by 'make ari-stubs' are all dependent on
+	  res_ari_model.  Additionally some of the same modules depend on one or more
+	  res_stasis_* modules.
+
+	  ASTERISK-25027 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I8e07fe7e81fedacb87232f2b6f8b5f47927b4153
+
+2015-04-29 06:26 +0000 [881844297a]  Corey Farrell <git@cfware.com>
+
+	* res_pjsip: Remove incorrect MODULEINFO from presence_xml.c.
+
+	  Remove incorrect MODULEINFO block and unneeded header includes
+	  from presence_xml.c.
+
+	  ASTERISK-25027
+	  Reported by: Corey Farrell
+
+	  Change-Id: I977c609ab9d1fe05373027c4138900f6985990eb
+
+2015-04-29 06:17 +0000 [c232ff3af0]  Corey Farrell <git@cfware.com>
+
+	* Git Migration: Create doc/rest-api when needed.
+
+	  Create the directory './doc/rest-api' at the start of 'make ari-stubs'
+	  to prevent an error when documentation is generated.  The directory is
+	  also added to git ignores.
+
+	  ASTERISK-25027
+	  Reported by: Corey Farrell
+
+	  Change-Id: Iaccc7f0138501c23aa78feaca2f3cce9e68cbc1b
+
+2015-04-29 03:03 +0000 [5d997ecc83]  Corey Farrell <git@cfware.com>
+
+	* Build System: Prevent unneeded changes to asterisk/buildopts.h.
+
+	  * Add AST_DEVMODE to BUILDOPTS
+	  * Use BUILDOPTS to generate AST_BUILDOPT_SUM.
+	  * Remove loop that defined AST_MODULE_*
+
+	  These changes ensure that only ABI effecting options are considered for
+	  AST_BUILDOPT_SUM.  This also reduces unneeded full system rebuilds caused
+	  by enabling or disabling one module that another is dependent on.
+
+	  ASTERISK-25028 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I2c516d93df9f6aaa09ae079a8168c887a6ff93a2
+
+2015-04-29 00:02 +0000 [55a780d211]  Corey Farrell <git@cfware.com>
+
+	* Git Conversion: Switch Non-C files to ASTERISK_REGISTER_FILE.
+
+	  This switches files used to generate other sources to use the new
+	  ASTERISK_REGISTER_FILE macro.
+
+	  ASTERISK-25026 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: Ieb2537b83421cad07c8955e5f90c405ccf079740
+
+2015-04-28 13:28 +0000 [5ebfed8ef3]  Yousf Ateya <y.ateya@starkbits.com>
+
+	* chan_iax2: Ensure that IAX flags are 64 bits.
+
+	  Flags are 64 bits.  Without LLU suffix the value of 1<<31 is negative.
+	  Although it doesn't have an effect on the current implementation, it will
+	  be problem if more flags are added.
+
+	  Change-Id: Ic290c81cfbbbf062872392d99d3322932cc49487
+2015-04-28 00:29 +0000 [46cf643c75]  Ashley Sanders <asanders@digium.com>
+
+	* chan_pjsip: Creating Channel Causes Asterisk to Crash When Duplicate AOR
+	              Sections Exist in pjsip.conf
+
+	  This patch modifies the current loading strategy of the pjsip configuration. If
+	  duplicate sections (e.g. sections containing the same [id/type]) are defined in
+	  [pjsip.conf], the loader will consider the configuration for the given type as
+	  invalid when the duplicate section is encountered. The entire configuration
+	  (including what was previously loaded) for the duplicate [id/type] sections
+	  will be rejected and destroyed, an error message is logged and the load
+	  processing for the given stops.
+
+	  ASTERISK-24996
+	  Reported By: Ashley Sanders
+
+	  Change-Id: I35090ca4cd40f1f34881dfe701a329145c347aef
+2015-04-28 11:50 +0000 [0bbe2c35cf]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_vpb: Fix compile error due to use of ASTERISK_FILE_VERSION.
+
+	  Change-Id: I51179e2a83937423676da522b766f1126de4059e
+2015-04-27 14:44 +0000 [f47fed2e12]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_outbound_registration: Add debugging messages.
+
+	  When problems occur regarding outbound registrations, it currently
+	  is difficult to debug. Most off-nominal paths had warning messages,
+	  but sometimes we want to know what's going on before hitting the
+	  off-nominal path. This patch adds lots of debugging output that
+	  should give a clearer picture of what is happening with regards
+	  to outbound registrations.
+
+	  ASTERISK-25020
+	  Reported by Mark Michelson
+
+	  Change-Id: I577bde7860be0a6c872b5bcb4d5047340bf45d45
+
+2015-04-28 05:38 +0000 [5e96584829]  Steve Davies <steve@one47.co.uk>
+
+	* res_rtp_asterisk: Resolve 2 discrete memory leaks in DTLS
+
+	  ao2 ref leak in res_rtp_asterisk.c when a DTLS policy is created.
+	  The resources are linked into a table, but the original alloc refs
+	  are never released. ast_strdup leak in rtp_engine.c. If
+	  ast_rtp_dtls_cfg_copy() is called twice on the same destination struct,
+	  a pointer to an alloc'd string is overwritten before the string is free'd.
+
+	  ASTERISK-25022
+	  Reported by: one47
+
+	  Change-Id: I62a8ceb8679709f6c3769136dc6aa9a68202ff9b
+
+2015-04-28 04:28 +0000 [d6a2d92353]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_csv.c: Add missing space after comma.
+
+	  Change-Id: I3866a20019b1a3a2f10fe36640053929330b0fcb
+
+2015-04-27 22:01 +0000 [542bfee881]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* CHANGES: Add missing spaces.
+
+	  Change-Id: I534ea0f22759e3633585dfa9b145b4a284efe67f
+
+2015-04-17 02:16 +0000 [5c1d07baf0]  Corey Farrell <git@cfware.com>
+
+	* Astobj2: Allow reference debugging to be enabled/disabled by config.
+
+	  * The REF_DEBUG compiler flag no longer has any effect on code that uses
+	    Astobj2.  It is used to determine if reference debugging is enabled by
+	    default.  Reference debugging can be enabled or disabled in asterisk.conf.
+	  * Caller information is provided in logger errors for ao2 bad magic numbers.
+	  * Optimizes AO2 by merging internal functions with the public counterpart.
+	    This was possible now that we no longer require a dual ABI.
+
+	  ASTERISK-24974 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: Icf3552721fe999365ba8a8cf00a965aa6b897cc1
+
+2015-04-27 12:11 +0000 [356568dc7f]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip: Fix SEGV on pending-qualify contacts
+
+	  Permanent contacts that hadn't been qualified yet were missing
+	  their contact_status entries causing SEGVs when running CLI
+	  commands.
+
+	  This patch makes sure that contact_statuses are created for
+	  both dynamic and permanent contacts when they are created.
+	  It also adds checks in the CLI code to make sure there's a
+	  contact_status, just in case.
+
+	  ASTERISK-25018 #close
+	  Reported-by: Ivan Poddubny
+	  Tested-by: Ivan Poddubny
+	  Tested-by: George Joseph
+
+	  Change-Id: I3cc13e5cedcafb24c400368b515b02d7fb81e029
+
+2015-04-15 18:55 +0000 [358080e86e]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_odbc.c: Added to record new columns add on CDR 1.8 Asterisk Version
+
+	  Add new column to INSERT new columns added in cdr 1.8 version. The columns are:
+	   * peeraccount
+	   * linkedid
+	   * sequence
+	  This feature is configurable in cdr_odbc.conf using a new configuration
+	  option, 'newcdrcolumns'.
+
+	  ASTERISK-24976 #close
+
+	  Change-Id: Ibe0c7540a88305c6012786f438a0813ad8b19127
+2015-04-26 17:21 +0000 [d7f4788341]  Matt Jordan <mjordan@digium.com>
+
+	* channels/chan_skinny: Fix compilation error introduced in f8e21a1adf
+
+	  A typo in commit f8e21a1adf resulted in a compilation error in
+	  chan_skinny. This patch fixes the typo.
+
+	  ASTERISK-24917
+
+	  Change-Id: Id7f4ad1fe948eb2408622e80c27936ce4516c33c
+
+2015-04-23 17:29 +0000 [9f65ea482e]  Kevin Harwell <kharwell@digium.com>
+
+	* app_confbridge: Default the template option to a compatible default profile.
+
+	  Confbridge dynamic profiles did not have a default profile unless you
+	  explicitly used Set(CONFBRIDGE(bridge,template)=default_bridge). If a
+	  template was not set prior to the bridge being created then some
+	  options were left with no default values set. This patch makes it so
+	  the default templates are set to the default bridge and user profiles.
+
+	  ASTERISK-24749 #close
+	  Reported by: philippebolduc
+
+	  Change-Id: I1bd6e94b38701ac2112d842db68de63d46f60e0a
+
+2015-04-23 07:31 +0000 [cafdb7a049]  Olle E. Johansson <oej@edvina.net>
+
+	* CREDITS: Update credits for Olle Johansson
+
+	  Change-Id: I8f3d0a6c3f1075a1f7d8308593394611a96749de
+2015-04-24 09:17 +0000 [bd61c9300c]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_outbound_authenticator: Increase CSeq on authed requests.
+
+	  The way PJSIP generates an authenticated request is to use a previous
+	  request as a template. This means that the authenticated request will
+	  have the same Call-ID, From header (including tag), and CSeq as the
+	  original request. PJSIP generates a new branch on the Via header to
+	  indicate that this is a new transaction, though.
+
+	  There are some SIP implementations, though, that do not notice the
+	  change in the branch and therefore will match the authed request to the
+	  original request's transaction. Since the CSeq is the same, the server
+	  will repeat the response it sent to the original request.
+
+	  This patch aids interoperability by increasing the CSeq of the authed
+	  request by one.
+
+	  ASTERISK-24845 #close
+	  Reported by: Carl Fortin
+	  Tested by: Carl Fortin
+
+	  Change-Id: I39c4ca52e688a9f83bcc1878371334becdc5be01
+
+2015-04-22 04:17 +0000 [f8e21a1adf]  Diederik de Groot <ddegroot@talon.nl>
+
+	* Clang: Fix some more tautological-compare warnings.
+
+	  clang can warn about a so called tautological-compare, when it finds
+	  comparisons which are logically always true, and are therefor deemed
+	  unnecessary.
+
+	  Exanple:
+	  unsigned int x = 4;
+	  if (x > 0)    // x is always going to be bigger than 0
+
+	  Enum Case:
+	  Each enumeration is its own type. Enums are an integer type but they
+	  do not have to be *signed*. C leaves it up to the compiler as an
+	  implementation option what to consider the integer type of a particu-
+	  lar enumeration is. Gcc treats an enum without negative values as
+	  an int while clang treats this enum as an unsigned int.
+
+	  rmudgett & mmichelson: cast the enum to (unsigned int) in assert.
+	  The cast does have an effect. For gcc, which seems to treat all enums
+	  as int, the cast to unsigned int will eliminate the possibility of
+	  negative values being allowed. For clang, which seems to treat enums
+	  without any negative members as unsigned int, the cast will have no
+	  effect. If for some reason in the future a negative value is ever
+	  added to the enum the assert will still catch the negative value.
+
+	  ASTERISK-24917
+	  Change-Id: Ief23ef68916192b9b72dabe702b543ecfeca0b62
+
+2015-04-20 13:06 +0000 [1e74793061]  Diederik de Groot <ddegroot@talon.nl>
+
+	* Example script for scan-build (the llvm static analyzer)
+
+	   - Added Pre-amble (Options / Flags / Usage Example / GNU License)
+	   - Extended Configurability
+	   - Made Executable
+
+	  ASTERISK-24917
+	  Change-Id: I70405fe54e4be7dbfbcb62e291690069b88617a8
+
+2015-04-23 12:54 +0000 [89a3fc0572]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_t38: Don't crash on authenticated reinvite after originated T.38 FAX.
+
+	  When Asterisk originates a channel to an application, the channel is
+	  hung up once the application finishes executing. When the application
+	  in question is SendFax, the Asterisk PJSIP code will attempt to reinvite
+	  the T.38 session to audio after the FAX completes. The hangup of the
+	  channel happens in the midst of this reinvite transaction. In most
+	  circumstances, this works out okay because the BYE is delayed until the
+	  reinvite transaction can complete.
+
+	  However, if the reinvite that Asterisk sends receives a 401/407
+	  response, then Asterisk's attempt to re-send the reinvite with
+	  authentication will fail. This is because the session supplement in
+	  res_pjsip_t38 makes the assumption that the channel on the session will
+	  always be non-NULL. Since the channel has been hung up, though, the
+	  channel is now NULL. Attempting to operate on the channel causes a
+	  crash.
+
+	  This patch fixes the issue by ensuring that the channel on the session
+	  is not NULL before attempting to mess with the T.38 framehook.
+
+	  This patch also contains some corrections for comments that were
+	  incorrect and really confused me when I first started looking at the
+	  code.
+
+	  ASTERISK-25004 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: Ic5a1230668369dda4bb13524098aed9306ab45a0
+
+2015-04-23 09:16 +0000 [75666ad7c6]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_pjsip:  Validate that contact uris start with sip: or sips:
+
+	  Currently we use pjsip_parse_hdr to validate contact uris but it
+	  appears that it allows uris without a scheme if there's a port
+	  supplied.  I.E myexample.com will fail but myexample.com:5060 will
+	  pass even though it has no scheme.  This causes SEGVs later on
+	  whenever the uri is used.
+
+	  To prevent this, permanent_contact_validate has been updated to check
+	  that the scheme is either 'sip' or 'sips'.
+
+	  2 uses of possibly-null endpoint have also been fixed in
+	  create_out_of_dialog_request.
+
+	  ASTERISK-24999
+
+	  Change-Id: Ifc17d16a4923e1045d37fe51e43bbe29fa556ca2
+	  Reported-by: Brad Latus
+
+2015-04-23 08:00 +0000 [ca7193167e]  Diederik de Groot <ddegroot@talon.nl>
+
+	* Clang: change previous tautological-compare fixes.
+
+	  clang can warn about a so called tautological-compare, when it finds
+	  comparisons which are logically always true, and are therefor deemed
+	  unnecessary.
+
+	  Exanple:
+	  unsigned int x = 4;
+	  if (x > 0)    // x is always going to be bigger than 0
+
+	  Enum Case:
+	  Each enumeration is its own type. Enums are an integer type but they
+	  do not have to be *signed*. C leaves it up to the compiler as an
+	  implementation option what to consider the integer type of a particu-
+	  lar enumeration is. Gcc treats an enum without negative values as
+	  an int while clang treats this enum as an unsigned int.
+
+	  rmudgett & mmichelson: cast the enum to (unsigned int) in assert.
+	  The cast does have an effect. For gcc, which seems to treat all enums
+	  as int, the cast to unsigned int will eliminate the possibility of
+	  negative values being allowed. For clang, which seems to treat enums
+	  without any negative members as unsigned int, the cast will have no
+	  effect. If for some reason in the future a negative value is ever
+	  added to the enum the assert will still catch the negative value.
+
+	  ASTERISK-24917
+
+	  Change-Id: I0557ae0154a0b7de68883848a609309cdf0aee6a
+
+2015-04-22 16:22 +0000 [cc77440deb]  gtjoseph <george.joseph@fairview5.com>
+
+	* res_corosync: Add check for config file before calling corosync apis
+
+	  On some systems, res_corosync isn't compatible with the installed version of
+	  corosync so corosync_cfg_initialize fails, load_module returns LOAD_FAILURE,
+	  and Asterisk terminates.  The work around has been to remember to add
+	  res_corosync as a noload in modules.conf.  A better solution though is to have
+	  res_corosync check for its config file before attempting to call corosync apis
+	  and return LOAD_DECLINE if there's no config file.  This lets Asterisk loading
+	  continue.
+
+	  If you have a res_corosync.conf file and res_corosync fails, you get the same
+	  behavior as today and the fatal error tells you something is wrong with the
+	  install.
+
+	  ASTERISK-24998
+
+	  Change-Id: Iaf94a9431a4922ec4ec994003f02135acfdd3889
+2015-04-22 15:17 +0000 [c231c85ea4]  Corey Farrell <git@cfware.com>
+
+	* Astobj2: Ensure all calls to __adjust_lock pass a valid object.
+
+	  __adjust_lock doesn't check for invalid objects, and doesn't have an
+	  appropriate return value for invalid objects.  Most callers of
+	  __adjust_lock pass objects that have already been confirmed valid,
+	  this change adds checks before the remaining calls.
+
+	  ASTERISK-24997 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I669100f87937cc3f867cec56a27ae9c01292908f
+
+2015-04-22 16:32 +0000 [0722e11f26]  gtjoseph <george.joseph@fairview5.com>
+
+	* .gitignore:  Add .gcno and .gcda
+
+	  Products of --enable-coverage
+
+	  Change-Id: Ie20882d64b60692e2c941ea8872ab82a86ce77a3
+
+2015-04-22 11:28 +0000 [7216e3c608]  Joshua Colp <jcolp@digium.com>
+
+	* dns: Make query sets hold on to queries for their lifetime.
+
+	  The query set documentation states that upon completion queries can be
+	  retrieved for the lifetime of the query set. This is a reasonable
+	  expectation but does not currently occur. This was originally done
+	  to resolve a circular reference between queries and query sets, but
+	  in practice the query can be kept.
+
+	  This change makes it so a query does not have a reference to the
+	  query set until it begins resolving. It also makes it so that the
+	  reference is given up upon the query being completed. This allows
+	  the queries to remain for the lifetime of the query set. As the
+	  query set on the query is only useful to the query set functionality
+	  and only for the lifetime that the query is resolving this is safe
+	  to do.
+
+	  ASTERISK-24994 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: I54e09c0cb45475896654e7835394524e816d1aa0
+
+2015-04-20 13:01 +0000 [09c7c678a3]  Diederik de Groot <ddegroot@talon.nl>
+
+	* Fix/Update clang-RAII macro implementation
+
+	  - When you need to refer to 'variable XXX' outside a block, it needs
+	  to be declared as '__block XXX', otherwise it will not be available with-
+	  in the block, making updating that variable hard to do, and ast_free
+	  lead to issues.
+
+	  - Removed the #error message
+	  because it creates complications when compiling external projects
+	  against asterisk For example when using a different compiler than the
+	  one used to compile asterisk. The warning/error should be generated
+	  during the configure process not the compilation process
+
+	  ASTERISK-24917
+	  Change-Id: I12091228090e90831bf2b498293858f46ea7a8c2
+
+2015-04-14 14:04 +0000 [190fa4f333]  Joshua Colp <jcolp@digium.com>
+
+	* res_pjsip_mwi: Send unsolicited MWI NOTIFY on startup and when endpoint registers.
+
+	  Currently the res_pjsip_mwi module only sends an unsolicited MWI NOTIFY upon
+	  a mailbox state change (such as a new message being left, or one being deleted).
+	  In practice this is not sufficient to keep clients aware of the current MWI status.
+
+	  This change makes the module send unsolicited MWI NOTIFY on startup so that
+	  clients are guaranteed to have the most up to date MWI information. It also makes
+	  clients receive an unsolicited MWI NOTIFY upon registration so if they are unaware
+	  of the current MWI status they receive it.
+
+	  ASTERISK-24982 #close
+	  Reported by: Joshua Colp
+
+	  Change-Id: I043f20230227e91218f18a82c7d5bb2aa62b1d58
+
+2015-04-21 17:45 +0000 [2a36bb5d9a]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* CHANGES remove tab space
+
+	  Change-Id: I6b43e43474bf6fb77b8227eadb036036f8e90521
+
+2015-04-21 15:17 +0000 [5757d2d30d]  Corey Farrell <git@cfware.com>
+
+	* Check for ao2_alloc failure in __ast_channel_internal_alloc.
+
+	  Fix a crash that could occur in __ast_channel_internal_alloc if
+	  ao2_alloc fails.
+
+	  ASTERISK-24991 #close
+
+	  Change-Id: I4ca89189eb22f907408cb87d0a1645cfe1314a90
+
+2015-04-20 14:30 +0000 [6331be0638]  Mark Michelson <mmichelson@digium.com>
+
+	* res_pjsip_pubsub: Set the endpoint on SUBSCRIBE dialogs.
+
+	  When SUBSCRIBE dialogs were established, we never associated
+	  the endpoint that created the subscription with the dialog
+	  we end up creating. In most cases, this ended up not causing
+	  any problems.
+
+	  The actual bug that was observed was that when a device that
+	  was behind NAT established a subscription with Asterisk, Asterisk
+	  would end up sending in-dialog NOTIFY requests to the device's
+	  private IP addres instead of the public address of the NAT router.
+
+	  When Asterisk receives the initial SUBSCRIBE from the device,
+	  res_pjsip_nat rewrites the contact to the public address on which the
+	  SUBSCRIBE was received. This allows for the dialog to have its target
+	  address set to the proper public address. Asterisk then would send a 200
+	  OK response to the SUBSCRIBE, then a NOTIFY with the initial
+	  subscription state. The device would then send a 200 OK response to
+	  Asterisk's NOTIFY.
+
+	  Here's where things went wrong. When the 200 OK arrived, res_pjsip_nat
+	  did not rewrite the address in the Contact header. Then, when the PJSIP
+	  dialog layer processed the 200 OK, PJSIP would perform a comparison
+	  between the IP address in the Contact header and its saved target
+	  address for the dialog. Since they differed, PJSIP would update the
+	  target dialog address to be the address in the Contact header. From this
+	  point, if Asterisk needed to send a NOTIFY to the device, the result was
+	  that the NOTIFY would be sent to the private address that the device
+	  placed in the Contact header.
+
+	  The reason why res_pjsip_nat did not rewrite the address when it
+	  received the 200 OK response was that it could not associate the
+	  incoming response with a configured endpoint. This is because on a
+	  response, the only way to associate the response to an endpoint is by
+	  finding the dialog that the response is associated with and then finding
+	  the endpoint that is associated with that dialog. We do not perform
+	  endpoint lookups on responses. res_pjsip_pubsub skipped the step of
+	  associating the endpoint with the dialog we created, so res_pjsip_nat
+	  could not find the associated endpoint and therefore couldn't rewrite
+	  the contact.
+
+	  This commit message is like 50x longer than the actual fix.
+
+	  ASTERISK 24981 #close
+	  Reported by Mark Michelson
+
+	  Change-Id: I2b963c58c063bae293e038406f7d044a8a5377cd
+2015-04-16 22:34 +0000 [2f418c052e]  Gareth Palmer <gareth@acsdata.co.nz>
+
+	* New AMI Command Output Format
+
+	  This change modifies how the the output from a CLI command is sent
+	  to a client over AMI.
+
+	  Output from the CLI command is now sent as a series of zero-or-more
+	  Output: headers.
+
+	  Additionally, commands that fail to execute (eg: no such command,
+	  invalid syntax etc.) now cause an Error response instead of Success.
+
+	  If the command executed successfully, but the manager unable to
+	  provide the output the reason will be included in the Message:
+	  header. Otherwise it will contain 'Command output follows'.
+
+	  Depends on a new version of starpy (> 1.0.2) that supports the new
+	  output format.
+
+	  See pull-request https://github.com/asterisk/starpy/pull/34
+
+	  ASTERISK-24730
+
+	  Change-Id: I6718d95490f0a6b3f171c1a5cdad9207f9a44888
+2015-04-20 18:00 +0000 [614f506690]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_dahdi/sig_pri: Make post AMI HangupRequest events on PRI channels.
+
+	  The chan_dahdi channel driver is a very old driver.  The ability for it to
+	  support ISDN was added well after the initial analog support.  Setting the
+	  softhangup flags is a carry over from the original analog code.  The
+	  driver was not updated to call ast_queue_hangup() which will post the AMI
+	  HangupRequest event.
+
+	  * Changed sig_pri.c to call ast_queue_hangup() instead of setting the
+	  softhangup flag when the remote party initiates a hangup.
+
+	  ASTERISK-24895 #close
+	  Reported by: Andrew Zherdin
+
+	  Change-Id: I5fe2e48556507785fd8ab8e1c960683fd5d20325
+
+2015-04-20 13:40 +0000 [bff3064578]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+
+	* cdr/cdr_adaptive_odbc.c: Refactor concatenate columns name.
+
+	  The concatenate for columns name to INSERT INTO is always the same. It is
+	  possible to do it on one line.
+
+	  ASTERISK-24980
+
+	  Change-Id: Ib8bb53c42535378581d4ef729cc5ebbb22b067ac
+2015-04-20 09:53 +0000 [06ba1e59cb]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip_options:  Fix format specifier for int64_t rtt.
+
+	  Contact status rtt is an int64_t and needs the PRId64 macro to
+	  properly create the format specifier on 32-bit systems.
+
+	  Change-Id: I4b8ab958fc1e9a179556a9b4ffa49673ba9fdec7
+
+2015-04-18 13:36 +0000 [298faf7c50]  gtjoseph <george.joseph@fairview5.com>
+
+	* pjsip_options:  Fix non-qualified contacts showing as unavailable
+
+	  The "Add qualify_timeout processing and eventing" patch introduced
+	  an issue where contacts that had qualify_frequency set to 0 were
+	  showing Unavailable instead Unknown.  This patch checks for
+	  qualify_frequency=0 and create an "Unknown"  contact_status
+	  with an RTT = 0.
+
+	  Previously, the lack of contact_status implied Unknown but since
+	  we're now changing endpoint state based on contact_status, I've
+	  had to add new UNKNOWN status so that changes could trigger the
+	  appropriate contact_status observers.
+
+	  ASTERISK-24977: #close
+
+	  Change-Id: Ifcbc01533ce57f0e4e584b89a395326e098b8fe7
+
+2015-04-19 15:49 +0000 [8e903b17ea]  Matt Jordan <mjordan@digium.com>
+
+	* main/pbx: Don't attempt to destroy a previously destroyed exten/priority tuple
+
+	  When a PBX registrar is unloaded, it will fail to remove its extension from
+	  the context root_table if a dialplan application used by that extension is
+	  still loaded. This can be the case for AGI, which can be unloaded after several
+	  of the standard PBX providers. Often, this is harmless; however, if the
+	  extension's priorities are removed during the failed unloading *and* the
+	  dialplan application later unregisters, it leaves a ticking timebomb for the
+	  next PBX provider that attempts to iterate over the extensions. When that
+	  occurs, the peer_table pointer on the extension will already be set to NULL.
+	  The current code does not check to see if the pointer is NULL before passing
+	  it to a hashtab function this is not NULL tolerant.
+
+	  Since it is possible for the peer_table to be NULL when we normally would not
+	  expect that to be the case, the solution in this patch is to simply skip over
+	  processing an extension's priorities if peer_table is NULL.
+
+	  Prior to this patch, the tests/pbx/callerid_match test would crash during
+	  module unload. With this patch, the test no longer crashes after running.
+
+	  ASTERISK-24774 #close
+	  Reported by: Corey Farrell
+
+	  Change-Id: I2bbeecb7e0f77bac303a1b9135e4cdb4db6d4c40
+
+2015-04-17 18:05 +0000 [1269dd06bc]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_fax: Fix latent bug exposed by ASTERISK-24841 changes.
+
+	  Three fax related tests started failing as a result of changes made for
+	  ASTERISK-24841:
+	  tests/fax/pjsip/gateway_t38_g711
+	  tests/fax/sip/gateway_mix1
+	  tests/fax/sip/gateway_mix3
+
+	  Historically, ast_channel_make_compatible() did nothing if the channels
+	  were already "compatible" even if they had a sub-optimal translation path
+	  already setup.  With the changes from ASTERISK-24841 this is no longer
+	  true in order to allow the best translation paths to always be picked.  In
+	  res_fax.c:fax_gateway_framehook() code manually setup the channels to go
+	  through slin and then called ast_channel_make_compatible().  With the
+	  previous version of ast_channel_make_compatible() this was always a
+	  no-operation.
+
+	  * Remove call to ast_channel_make_compatible() in fax_gateway_framehook()
+	  that now undoes what was just setup when the framehook is attached.
+
+	  * Fixed locking around saving the channel formats in
+	  fax_gateway_framehook() to ensure that the formats that are saved are
+	  consistent.
+
+	  * Fix copy pasta errors in fax_gateway_framehook() that confuses read and
+	  write when dealing with saved channel formats.
+
+	  ASTERISK-24841
+	  Reported by: Matt Jordan
+
+	  Change-Id: I6fda0877104a370af586a5e8cf9e161a484da78d
+
+2015-04-17 16:19 +0000 [c1d44ff043]  Corey Farrell <git@cfware.com>
+
+	* Fix issue with AST_THREADSTORAGE_RAW when DEBUG_THREADLOCALS is enabled.
+
+	  When DEBUG_THREADLOCALS is enabled it causes the threadlocal cleanup to be
+	  called as a function.  This causes a compile error with raw threadstorage as
+	  it uses NULL for cleanup.  This fix uses a macro that provides NULL when
+	  DEBUG_THREADLOCALS is disabled, and replaces the call to "c_cleanup(data);"
+	  with "{};" when DEBUG_THREADLOCALS is enabled.
+
+	  ASTERISK-24975 #close
+	  Reported by: Ashley Sanders
+
+	  Change-Id: I3ef7428ee402816d9fcefa1b3b95830c00d5c402
+
+2015-04-15 10:38 +0000 [aae45acbda]  Mark Michelson <mmichelson@digium.com>
+
+	* Detect potential forwarding loops based on count.
+
+	  A potential problem that can arise is the following:
+
+	  * Bob's phone is programmed to automatically forward to Carol.
+	  * Carol's phone is programmed to automatically forward to Bob.
+	  * Alice calls Bob.
+
+	  If left unchecked, this results in an endless loops of call forwards
+	  that would eventually result in some sort of fiery crash.
+
+	  Asterisk's method of solving this issue was to track which interfaces
+	  had been dialed. If a destination were dialed a second time, then
+	  the attempt to call that destination would fail since a loop was
+	  detected.
+
+	  The problem with this method is that call forwarding has evolved. Some
+	  SIP phones allow for a user to manually forward an incoming call to an
+	  ad-hoc destination. This can mean that:
+
+	  * There are legitimate use cases where a device may be dialed multiple
+	  times, or
+	  * There can be human error when forwarding calls.
+
+	  This change removes the old method of detecting forwarding loops in
+	  favor of keeping a count of the number of destinations a channel has
+	  dialed on a particular branch of a call. If the number exceeds the
+	  set number of max forwards, then the call fails. This approach has
+	  the following advantages over the old:
+
+	  * It is much simpler.
+	  * It can detect loops involving local channels.
+	  * It is user configurable.
+
+	  The only disadvantage it has is that in the case where there is a
+	  legitimate forwarding loop present, it takes longer to detect it.
+	  However, the forwarding loop is still properly detected and the
+	  call is cleaned up as it should be.
 
-2016-07-13 17:45 +0000 [8cef8f35e7]  Kevin Harwell <kharwell@digium.com>
+	  Address review feedback on gerrit.
 
-	* translate: explicit format destination not properly set
+	  * Correct "mfgium" to "Digium"
+	  * Decrement max forwards by one in the case where allocation of the
+	    max forwards datastore is required.
+	  * Remove irrelevant code change from pjsip_global_headers.c
 
-	  If the destination format's name differed from the codec name then the
-	  translator's explict_dst field would be improperly set. In some circumstances
-	  it would end up setting it to a newly created format that has the same name
-	  as the codec when it actually needed to be the given destination codec.
+	  ASTERISK-24958 #close
 
-	  This could cause the translation path to use the wrong format. For instance,
-	  if an endpoint had specified 'myulaw' as a format the translator could end up
-	  using a 'ulaw' format (with whatever/default settings) instead. If the format
-	  attribute settings differed between the two then there may unexpected results
-	  during processing.
+	  Change-Id: Ia7e4b7cd3bccfbd34d9a859838356931bba56c23
+2015-04-16 10:51 +0000 [56a2baa21d]  Kevin Harwell <kharwell@digium.com>
 
-	  This patch removes the name check when building the translation path. This
-	  should make it always set the translator's explicit_dst to the given destination
-	  format as long as the sample rate and types match.
+	* bridge.c: NULL app causes crash during attended transfer
 
-	  Change-Id: Iaf8a03831d68e657d89569d54b505074efbefab5
+	  Due to a race condition there was a chance that during an attended transfer the
+	  channel's application would return NULL. This, of course, would cause a crash
+	  when attempting to access the memory. This patch retrieves the channel's app
+	  at an earlier time in processing in hopes that the app name is available.
+	  However, if it is not then "unknown" is used instead. Since some string value
+	  is now always present the crash can no longer occur.
 
-2016-07-08 11:46 +0000 [afbd10b0c5]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24869 #close
+	  Reported by: viniciusfontes
+	  Review: https://gerrit.asterisk.org/#/c/133/
 
-	* stasis_endpoint.c: Fix contactstatus_to_json().
+	  Change-Id: I5134b84c4524906d8148817719d76ffb306488ac
 
-	  The roundtrip_usec json member is optional.  If it isn't present then
-	  don't put it into the converted json structure where ast_json_pack()
-	  will choke on it.
+2015-04-11 17:04 +0000 [c6ed681638]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: I39bb2f86154ef54591270c58bfda8635070f9ea0
+	* res_pjsip: Add global option to limit the maximum time for initial qualifies
 
-2016-07-13 13:45 +0000 [2be13d62fd]  Corey Farrell <git@cfware.com>
+	  Currently when Asterisk starts initial qualifies of contacts are spread out
+	  randomly between 0 and qualify_timeout to prevent network and system overload.
+	  If a contact's qualify_frequency is 5 minutes however, that contact may be
+	  unavailable to accept calls for the entire 5 minutes after startup.  So while
+	  staggering the initial qualifies is a good idea, basing the time on
+	  qualify_timeout could leave contacts unavailable for too long.
 
-	* chan_sip: Fix reference leak in mwi_event_cb
+	  This patch adds a new global parameter "max_initial_qualify_time" that sets the
+	  maximum time for the initial qualifies.  This way you could make sure that all
+	  your contacts are initialy, randomly qualified within say 30 seconds but still
+	  have the contact's ongoing qualifies at a 5 minute interval.
 
-	  Cleanup the peer reference when stasis_subscription_final_message is
-	  true.  Also free peer_name even if peer exists, after reload a new
-	  peer_name will be allocated.
+	  If max_initial_qualify_time is > 0, the formula is initial_interval =
+	  min(max_initial_interval, qualify_timeout * random().  If not set,
+	  qualify_timeout is used.
 
-	  ASTERISK-26193 #close
+	  The default is "0" (disabled).
 
-	  Change-Id: If7ecd52facdc5c227f701c760841e3f6ca53cc69
+	  ASTERISK-24863 #close
 
-2016-06-22 07:13 +0000 [332beb27d8]  Eugene Voityuk <eugene@thirdlane.com>,Alexander Traud <pabstraud@compuserve.com>
+	  Change-Id: Ib80498aa1ea9923277bef51d6a9015c9c79740f4
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
 
-	* res_rtp_asterisk: Enable Forward Secrecy (PFS) for DTLS.
+2015-04-16 13:20 +0000 [664d3263e4]  Scott Griepentrog <scott@griepentrog.com>
 
-	  Since July 2014, TLS based protocols (SIP over TLS, Secure WebSockets, HTTPS)
-	  support PFS thanks to ASTERISK-23905. In July 2015, the same feature was added
-	  for DTLS. The source code from main/tcptls.c should have been re-used to ease
-	  security audits. Therefore, this change rolls back the change from July 2015 and
-	  re-uses the code from July 2014. This has the additional benefits to work under
-	  CentOS 7 and enabling not just ECDHE but DHE based cipher suites as well.
+	* res_pjsip_pubsub: On notify fail deleted sub_tree is then referenced
 
-	  ASTERISK-25659 #close
-	  Reported by: StefanEng86, urbaniak, pay123
-	  Tested by: sarumjanuch, traud
-	  patches:
-	  res_rtp_asterisk.patch submitted by sarumjanuch
-	  dtls_centos_step_1.patch submitted by traud
-	  dtls_centos_step_2.patch submitted by traud
+	  This change makes the send_notify of the sub_tree
+	  not happen when the sub_tree has been deleted due
+	  to the notify call failing, which avoids a crash.
 
-	  Change-Id: I537cadf4421f092a613146b230f2c0ee1be28d5c
+	  ASTERISK-24970 #close
 
-2016-07-13 11:30 +0000 [672a64bda3]  Corey Farrell <git@cfware.com>
+	  Change-Id: I1f20ffc08b192f59c457293b218025a693992cbf
+2015-04-11 16:56 +0000 [51886c68dc]  gtjoseph <george.joseph@fairview5.com>
 
-	* threadpool: Fix leak in ast_threadpool_serializer_group error path.
+	* pjsip_options: Add qualify_timeout processing and eventing
 
-	  ast_threadpool_serializer_group leaks a reference to ser when listener
-	  is allocated but tps is not.  Although listener takes the reference to
-	  ser cleanup functions are not run without tps.
+	  This is the second follow-on to https://reviewboard.asterisk.org/r/4572/ and the
+	  discussion at
+	  http://lists.digium.com/pipermail/asterisk-dev/2015-March/073921.html
 
-	  ASTERISK-26191 #close
+	  The basic issues are that changes in contact status don't cause events to be
+	  emitted for the associated endpoint.  Only dynamic contact add/delete actions
+	  update the endpoint.  Also, the qualify timeout is fixed by pjsip at 32 seconds
+	  which is a long time.
 
-	  Change-Id: Ie3ccf69a3f1e676c2ef62a77067c0cb57dc9a585
+	  This patch makes use of the new transaction timeout feature in r4585 and
+	  provides the following capabilities...
 
-2016-07-11 10:22 +0000 [fea201f7e6]  Richard Mudgett <rmudgett@digium.com>
+	  1.  A new aor/contact variable 'qualify_timeout' has been added that allows the
+	  user to specify the maximum time in milliseconds to wait for a response to an
+	  OPTIONS message.  The default is 3000ms.  When the timer expires, the contact is
+	  marked unavailable.
 
-	* pjsip_options.c: Fix container operation.
+	  2.  Contact status changes are now propagated up to the endpoint as follows...
+	  When any contact is 'Available', the endpoint is marked as 'Reachable'.  When
+	  all contacts are 'Unavailable', the endpoint is marked as 'Unreachable'.  The
+	  existing endpoint events are generated appropriately.
 
-	  aor_observer_deleted() needs to operate on all contacts found for the
-	  deleted AOR instead of only the first one found.  This is really only a
-	  problem if there is more than one contact for the AOR.
+	  ASTERISK-24863 #close
 
-	  Change-Id: Id24ac0d5e8c931330231fb45dd2a331a84339dc1
+	  Change-Id: Id0ce0528e58014da1324856ea537e7765466044a
+	  Tested-by: Dmitriy Serov
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
 
-2016-07-11 10:21 +0000 [02877b4b4f]  Richard Mudgett <rmudgett@digium.com>
+2015-04-11 16:39 +0000 [ab6382cafd]  gtjoseph <george.joseph@fairview5.com>
 
-	* pjsip_configuration.c: Misc cleanups.
+	* res_pjsip: Refactor endpt_send_request to include transaction timeout
 
-	  * Fix some whitespace in various routines.
+	  This is the first follow-on to https://reviewboard.asterisk.org/r/4572/ and the
+	  discussion at
+	  http://lists.digium.com/pipermail/asterisk-dev/2015-March/073921.html
 
-	  * Rename i to iter in persistent_endpoint_update_state().
+	  Since we currently have no control over pjproject transaction timeout, this
+	  patch pulls the pjsip_endpt_send_request function out of pjproject and into
+	  res_pjsip/endpt_send_transaction in order to implement that capability.
 
-	  * Fix off-nominal copy/paste message wording in
-	  persistent_endpoint_contact_deleted_observer()
+	  Now when the transaction is initiated, we also schedule our own pj_timer with
+	  our own desired timeout.
 
-	  Change-Id: Id8e34f5d09e7eebac3af22501c44c1110a3e29d8
+	  If the transaction completes before either timeout, pjproject cancels its timer,
+	  and calls our tsx callback where we cancel our timer and run the app callback.
 
-2016-07-13 08:57 +0000 [148cd1b319]  Alexander Traud <pabstraud@compuserve.com>
+	  If the pjproject timer times out first, pjproject calls our tsx callback where
+	  we cancel our timer and run the app callback.
 
-	* BuildSystem: Avoid obsolete warning with pthread.m4 on autoconf.
+	  If our timer times out first, we terminate the transaction which causes
+	  pjproject to cancel its timer and call our tsx callback where we run the app
+	  callback.
 
-	  Updated the macro-set autoconf/ax_pthread.m4 to its latest upstream version.
+	  Regardless of the scenario, pjproject is calling the tsx callback inside the
+	  group_lock and there are checks in the callback to make sure it doesn't run
+	  twice.
 
-	  ASTERISK-26046 #close
+	  As part of this patch ast_sip_send_out_of_dialog_request was created to replace
+	  its similarly named private function.  It takes a new timeout argument in
+	  milliseconds (<= 0 to disable the timeout).
 
-	  Change-Id: I11abc11d17acd2b6a8a5a5be8ae8e0949dab9cc7
+	  ASTERISK-24863 #close
+	  Reported-by: George Joseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
 
-2016-07-11 10:25 +0000 [97b4c7a5b4]  Richard Mudgett <rmudgett@digium.com>
+	  Change-Id: I0778dc730d9689c5147a444a04aee3c1026bf747
+2015-04-15 16:08 +0000 [043c38f6de]  gtjoseph <george.joseph@fairview5.com>
 
-	* res_pjsip: Fix statsd regression.
+	* More .gitignore updates
 
-	  The ASTERISK-25904 change-id I8fad8aae9305481469c38d2146e1ba3a56d3108f
-	  patch introduced several regressions when the newly created "Updated"
-	  state goes out for each endpoint registration refresh.
+	  Added .pyc and .sha1 to the top-level .gitignore.
 
-	  1) It restarted any OPTIONS RTT ping cycle.
+	  Change-Id: I7dfc4f554d54d22947b38140d3305007503cc16a
+	  Tested-by: George Joseph <george.joseph@fairview5.com>
 
-	  2) It would interfere with a currently active ping and throw off that
-	  ping's resulting RTT calculation.
+2015-04-14 02:36 +0000 [abf10a1d4c]  Corey Farrell <git@cfware.com>
 
-	  3) It cleared the RTT time each time the endpoint was refreshed.
+	* Build System: Enable use of ~/.asterisk.makeopts and /etc/asterisk.makeopts.
 
-	  4) The cleared RTT time was sent out as a statsd update each time.
+	  The Makefile claims that you can set default menuselect options by creating
+	  ~/.asterisk.makeopts or /etc/asterisk.makeopts, but they are never read.
+	  The rule for menuselect.makeopts is only allowed to run if the active target
+	  is 'menuselect', but the menuselect target doesn't depend on
+	  menuselect.makeopts.  A dot (wildcard character) was added so the rule will
+	  be active for the targets that cause it to run: nmenuselect, cmenuselect,
+	  and gmenuselect.
 
-	  5) It created two AMI events for each update.
+	  ASTERISK-13271 #close
+	  Reported by: John Nemeth
 
-	  * Revert the original patch and reimplement it.  Now the current contact
-	  status state is re-sent instead of the state being momentarily toggled
-	  every time the endpoint refreshes its registration.  The statsd events are
-	  not created for the re-sent refresh because they are sent after every
-	  OPTIONS ping.
+	  Change-Id: Ibde804ff196283def49ccb9432fbf224a22586e2
+2015-04-13 08:47 +0000 [a3cec44a0a]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-26160 #close
-	  Reported by: Matt Jordan
+	* res_pjsip: Add external PJSIP resolver implementation using core DNS API.
 
-	  Change-Id: Ie072be790fbb2a8f5c1c874266e4143fa31f66d1
+	  This change adds the following:
 
-2016-07-12 03:50 +0000 [3be6fa1e4b]  Alexander Traud <pabstraud@compuserve.com>
+	  1. A query set implementation. This is an API that allows queries to be executed in parallel and once all have completed a callback is invoked.
+	  2. Unit tests for the query set implementation.
+	  3. An external PJSIP resolver which uses the DNS core API to do NAPTR, SRV, AAAA, and A lookups.
 
-	* BuildSystem: Allow own CFLAGS on ./configure.
+	  For the resolver it will do NAPTR, SRV, and AAAA/A lookups in parallel. If NAPTR or SRV
+	  are available it will then do more queries. And so on. Preference is NAPTR > SRV > AAAA/A,
+	  with IPv6 preferred over IPv4. For transport it will prefer TLS > TCP > UDP if no explicit
+	  transport has been provided. Configured transports on the system are taken into account to
+	  eliminate resolved addresses which have no hope of completing.
 
-	  Before this change, make failed with the error
-	  Unknown value '' found in build_tools/menuselect-deps for NATIVE_ARCH
-	  when CFLAGS were supplied to the configure script. This was introduced with
-	  <https://reviewboard.asterisk.org/r/1852/> which disabled BUILD_NATIVE when
-	  CFLAGS were supplied. Those who need different -march= values, please, go for
-	  ./configure
-	  make menuselect.makeopts or make menuselect
-	  ./menuselect/menuselect --disable BUILD_NATIVE
+	  ASTERISK-24947 #close
+	  Reported by: Joshua Colp
 
-	  ASTERISK-25289 #close
+	  Change-Id: I56cb03ce4f9d3d600776f36928e0b3e379b5d71e
 
-	  Change-Id: Ic6365d5a97bb9b3556858f06432a8d1cfa83eebc
+2015-04-14 13:16 +0000 [33a319ae73]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
 
-2016-07-11 13:42 +0000 [5ee205d8bb]  Richard Mudgett <rmudgett@digium.com>
+	* cel_pgsql: Fix name string for log on unable allocate memory.
 
-	* ast_expr2: Fix off-nominal memory leak.
+	  The LOG_ERROR has reference to CDR instead of CEL  for LENGTHEN_BUF1 and
+	  LENGTHEN_BUF2.
 
-	  Thanks to ibercom for pointing out a memory leak that was missed
-	  in the earlier patch for the issue.
+	  ASTERISK-24965 #close
+	  Reported by: Rodrigo Ramirez Norambuena
 
-	  ASTERISK-26119
-	  Reported by: Alexei Gradinari
+	  Change-Id: Icc818697d7d66d34bfe3048cdd15ca2b06c89744
+2015-04-14 15:59 +0000 [f89481e39c]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I9a151f5c4725d97fb82a9e938bc73dc659532b71
+	* test_astobj2_weaken: Fix source file registration.
 
-2016-07-11 10:17 +0000 [f5e9872016]  Alexander Traud <pabstraud@compuserve.com>
+	  Update test_astobj2_weaken to use the new AST_REGISTER_FILE macro.
 
-	* install_prereq: Checkout of libSRTP 1.5.x.
+	  Change-Id: Ieedadf16610f2e042f393e0501a36447cd07f83d
 
-	  Since 5th November 2014, the master branch of libSRTP changed the prefix of
-	  several member names and is not compatible with the source code in Asterisk
-	  anymore. Therefore instead, this change checks out the latest version of the
-	  libSRTP 1.5.x branch. Furthermore now, libSRTP is compiled with OpenSSL as
-	  backend. This makes AES-GCM and AES-IN possible.
+2015-04-13 05:28 +0000 [62508d6891]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-22131 #close
+	* Build System: Create Makefile macro MOD_ADD_SOURCE.
 
-	  Change-Id: I2e396cdc01da0ff610686e398ed210ca7408f7d6
+	  This new macro allows a single line to add all additional
+	  sources to a module.  This helps prevent modules from
+	  missing steps, and makes future changes easier since
+	  they can be made in a single place.
 
-2016-07-10 19:08 +0000 [17efed6cf7]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24960 #close
+	  Reported by: Corey Farrell
 
-	* func_odbc: Fix connection deadlock.
+	  Change-Id: I38f12d8b72c5e7bb37a879b2fb51761a2855eb4b
 
-	  The func_odbc module was modified to ensure that the
-	  previous behavior of using a single database connection
-	  was maintained. This was done by getting a single database
-	  connection and holding on to it. With the new multiple
-	  connection support in res_odbc this will actually starve
-	  every other thread from getting access to the database as
-	  it also maintains the previous behavior of having only
-	  a single database connection.
+2015-04-12 09:08 +0000 [23a180cade]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
 
-	  This change disables the func_odbc specific behavior if
-	  the res_odbc module is running with only a single database
-	  connection active. The connection is only kept for the
-	  duration of the request.
+	* cdr_pgsql: Fix CLI "cdr show pgsql status" command.
 
-	  ASTERISK-26177 #close
+	  The command always showed the usage information.
 
-	  Change-Id: I9bdbd8a300fb3233877735ad3fd07bce38115b7f
+	  * Fix the error in command validation for CLI_SHOWUSAGE.
 
-2016-07-09 13:32 +0000 [06ba533bc7]  Corey Farrell <git@cfware.com>
+	  ASTERISK-24959 #close
+	  Reported by: Rodrigo Ramirez Norambuena
 
-	* chan_sip: Fix reference leaks in error paths.
+	  Change-Id: I584f0936bb01001336a468a55c1d05d79fe795d5
+2015-04-13 19:06 +0000 [bf46ef35ca]  gtjoseph <george.joseph@fairview5.com>
 
-	  * get_sip_pvt_from_replaces leaks sip_pvt_ptr on any error.
-	  * build_peer leaks peer on failure to allocate the endpoint.
+	* .gitignore updates for master/13
 
-	  This patch fixes get_sip_pvt by using an RAII_VAR, build_peer is fixed
-	  with an unref in the appropriate place.
+	  Added products of ./bootstrap
 
-	  ASTERISK-26184 #close
+	  Added nmenuselect and gmenuselect to menuselect/
 
-	  Change-Id: I728b424648ad041409f7d90880f4c28b3ce2ca12
+	  Change-Id: Ied658463958bafc04a9aff9ebc28e40c116a6e35
 
-2016-07-07 12:41 +0000 [9d4e664f62]  Corey Farrell <git@cfware.com>
+2015-04-13 06:52 +0000 [62e95065d6]  Corey Farrell <git@cfware.com>
 
-	* REF_DEBUG: Prevent logging of container node objects.
+	* AMI: Fix improper handling of lines that are exactly 1025 bytes long.
 
-	  Using AO2_CONTAINER_ALLOC_OPT_DUPS_REPLACE can result in an unref being
-	  recorded to the refs log for the node being replaced.  This prevents
-	  logging of those unrefs since they would produce errors in
-	  refcounter.py.
+	  When AMI receives a line that is 1025 bytes long, it sends two error
+	  messages.  Copy the last byte in the buffer to the first postiion,
+	  set the length to 1.
 
-	  ASTERISK-26181 #close
+	  ASTERISK-20524 #close
+	  Reported by: David M. Lee
 
-	  Change-Id: Ie4fded84e8a1a58b3a59ce59dfd7eb0da3ddc5d4
+	  Change-Id: Ifda403e2713b59582c715229814fd64a0733c5ea
 
-2016-07-07 10:55 +0000 [e26bd15e7a]  Scott Griepentrog <scott@griepentrog.com>
+2015-04-12 03:22 +0000 [cb6bf3094e]  Corey Farrell <git@cfware.com>
 
-	* PJSIP: provide valid tcp nodelay option for reuse
+	* astobj2: Add support for weakproxy objects.
 
-	  When using TCP transport with chan_pjsip, the TCP_NODELAY
-	  option value was allocated on the stack, then passed as a
-	  pointer to the tcp transport configuration structure, and
-	  later re-used on subsequently created sockets when it was
-	  no longer valid.  This patch changes the allocation to be
-	  a static.
+	  This implements "weak" references.  The weakproxy object is a real ao2 with
+	  normal reference counting of its own.  When a weakproxy is pointed to a normal
+	  object they hold references to each other.  The normal object is automatically
+	  freed when a single reference remains (the weakproxy).  The weakproxy also
+	  supports subscriptions that will notify callbacks when it does not point
+	  to any real object.
 
-	  ASTERISK-26180 #close
-	  Reported by: Scott Griepentrog
+	  ASTERISK-24936 #close
+	  Reported by: Corey Farrell
 
-	  Change-Id: I3251164c7f710dbdab031282f00e30a9770626a0
+	  Change-Id: Ib9f73c02262488d314d9d9d62f58165b9ec43c67
 
-2016-07-07 10:38 +0000 [77b0145a25]  Joshua Colp <jcolp@digium.com>
+2015-04-13 14:41 +0000 [a573b77f78]  David M. Lee <dlee@respoke.io>
 
-	* chan_sip/res_pjsip_t38: Handle a request to negotiate T.38 after it is enabled.
+	* Fixing extconf compile
 
-	  Some T.38 implementations may send another re-invite after the initial
-	  one which adds additional negotiation details (such as the max bitrate).
-	  Currently this will fail when passthrough is being done in chan_sip as we
-	  do nothing if T.38 is already active.
+	  During the mass code deletion for clang support, a stray backslash was
+	  left behind that was causing utils to fail to compile.
 
-	  Other handlers of T.38 inside of Asterisk (such as res_fax) handle this
-	  scenario so this change adds support for it to chan_sip and res_pjsip_t38.
-	  If a request to negotiate is received while T.38 is already enabled a
-	  new re-INVITE is sent and negotiation is done again.
+	  Change-Id: I60e5fa58c9a5b248bde23aaada79ff663f87a2a1
 
-	  ASTERISK-26179 #close
+2015-04-13 09:54 +0000 [3f9aa29945]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I0298494d3da6df3219bbfa4be9aa04015043145c
+	* build_tools/make_version: Update version parsing for Git migration
 
-2016-07-04 16:38 +0000 [b4a9fa2c9e]  Alexei Gradinari <alex2grad@gmail.com>
+	  External systems - such as the Asterisk Test Suite - require knowledge of the
+	  upstream branch. Unfortunately, after moving to Git, the Asterisk version
+	  currently consists of only a 'GIT" prefix followed by an object blob,
+	  e.g., GIT-as08d7. This makes it difficult for such systems to know what
+	  features are available in a particular check out of Asterisk.
 
-	* res_sorcery_realtime: fix bug when successful UPDATE is treated as failed
+	  This patch fixes this by hardcoding the branch in a variable in the
+	  make_version script. Since the mainline branches are not changed often -
+	  typically only once a year - this is a reasonable approach to solving
+	  the problem, and is more reliable than parsing the output of 'git branch
+	  -vv'. Branches that track off of an upstream primary branch will then get the
+	  benefit of knowing which mainline branch they are currently based off
+	  of.
 
-	  If the SQL UPDATE statement changes nothing then SQLRowCount returns 0.
-	  This value should be treated as success.
-	  But the function sorcery_realtime_update treats it as failed.
+	  ASTERISK-24954 #close
 
-	  This bug was found using stress tests on PJSIP.
-	  If there are 2 consecutive SIP REGISTER requests with the same contact data
-	  during 1 second then res_pjsip_registrar adds contact location on 1st request
-	  and tries to update contact location on 2nd.
-	  The update fails and res_pjsip_registrar even removes correct contact location.
+	  Change-Id: I8090d5d548b6d19e917157ed530b914b7eaf9799
 
-	  The test "object_update_uncreated" was removed from test_sorcery_realtime.c
-	  because it's now a valid situation.
+2015-04-13 05:57 +0000 [fbc8ddfe63]  Corey Farrell <git@cfware.com>
 
-	  This patch also adds missing debug of extra SQL parameter.
+	* Optional API: Fix handling of sources that are both provider and user.
 
-	  ASTERISK-26172 #close
+	  OPTIONAL_API has conditionals to define AST_OPTIONAL_API and
+	  AST_OPTIONAL_API_ATTR differently based on if AST_API_MODULE is defined.
+	  Unfortunately this is inside the include protection block, so only the
+	  first status of AST_API_MODULE is respected.  For example res_monitor
+	  is an optional API provider, but uses func_periodic_hook.  This makes
+	  func_periodic_hook non-optional to res_monitor.
 
-	  Change-Id: I05a7f3051455336c9dda29efc229decf86071303
+	  This changes optional_api.h so that AST_OPTIONAL_API and
+	  AST_OPTIONAL_API_ATTR is redefined every time the header is included.
 
-2016-06-24 19:55 +0000 [1dfd3fc995]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-17608 #close
+	  Reported by: Warren Selby
 
-	* res/res_pjsip_session: Check for presence of an active negotiator
+	  Change-Id: I8fcf2a5e7b481893e17484ecde4f172c9ffb5679
 
-	  It is possible in a hypothetical situation for a session refresh to be
-	  invoked on a PJSIP when the negotiatior on the INVITE session has not
-	  yet been established. While this shouldn't occur with existing uses of
-	  ast_sip_session_refresh, the crashes that occur due to improperly
-	  calling PJSIP functions that expect a non-NULL negotiatior are
-	  avoidable. PJSIP will create the negotiator in pjsip_inv_reinvite; this
-	  means that simply checking for the presence of the negotiator before
-	  passing it to other PJSIP functions that use it is allowable. As such,
-	  this patch adds checks for the presence of the negotiator before calling
-	  PJSIP functions that assume it is non-NULL.
+2015-04-11 21:38 +0000 [4a58261694]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I1028323e7e01b0a531865e5412a71b6f6ec4276d
+	* git migration: Refactor the ASTERISK_FILE_VERSION macro
 
-2015-10-19 18:55 +0000 [9dd0aeeb44]  Matt Jordan <mjordan@digium.com>
+	  Git does not support the ability to replace a token with a version
+	  string during check-in. While it does have support for replacing a
+	  token on clone, this is somewhat sub-optimal: the token is replaced
+	  with the object hash, which is not particularly easy for human
+	  consumption. What's more, in practice, the source file version was often
+	  not terribly useful. Generally, when triaging bugs, the overall version
+	  of Asterisk is far more useful than an individual SVN version of a file. As a
+	  result, this patch removes Asterisk's support for showing source file
+	  versions.
 
-	* res/res_pjsip_pubsub: Add additional debug statements
+	  Specifically, it does the following:
 
-	  When something very sad and wrong occurs, it's challenging sometimes to
-	  figure out why. This patch adds some additional debug statements on
-	  off-nominal paths to try and make debugging easier.
+	  * Rename ASTERISK_FILE_VERSION macro to ASTERISK_REGISTER_FILE, and
+	    remove passing the version in with the macro. Other facilities
+	    than 'core show file version' make use of the file names, such as
+	    setting a debug level only on a specific file. As such, the act of
+	    registering source files with the Asterisk core still has use. The
+	    macro rename now reflects the new macro purpose.
 
-	  Change-Id: I7bffb73cc733b6f80193a23340881db4a102b640
+	  * main/asterisk:
+	    - Refactor the file_version structure to reflect that it no longer
+	      tracks a version field.
+	    - Remove the "core show file version" CLI command. Without the file
+	      version, it is no longer useful.
+	    - Remove the ast_file_version_find function. The file version is no
+	      longer tracked.
+	    - Rename ast_register_file_version/ast_unregister_file_version to
+	      ast_register_file/ast_unregister_file, respectively.
+
+	  * main/manager: Remove value from the Version key of the ModuleCheck
+	    Action. The actual key itself has not been removed, as doing so would
+	    absolutely constitute a backwards incompatible change. However, since
+	    the file version is no longer tracked, there is no need to attempt to
+	    include it in the Version key.
 
-2015-10-19 18:55 +0000 [1ec4f8dd00]  Matt Jordan <mjordan@digium.com>
+	  * UPGRADE: Add notes for:
+	    - Modification to the ModuleCheck AMI Action
+	    - Removal of the "core show file version" CLI command
 
-	* res/res_corosync: Raise a Stasis message on node join/leave events
+	  Change-Id: I6cf0ff280e1668bf4957dc21f32a5ff43444a40e
 
-	  When res_corosync detects that a node leaves or joins, it currently is
-	  informed of this via Corosync callbacks. However, there are a few
-	  limitations with the information presented:
-	  (1) While we have information that Corosync is aware of - such as the
-	      Corosync nodeid - that information is really only useful inside of
-	      Corosync or res_corosync. There's no way to translate a Corosync
-	      nodeid to some other internally useful unique identifier for the
-	      Asterisk instance that just joined or left the cluster.
-	  (2) While res_corosync is notified of the instance joining or leaving
-	      the cluster, it has no mechanism to inform the Asterisk core or
-	      other modules of this event. This limits the usefulness of res_corosync
-	      as a heartbeat mechanism for other modules.
+2015-04-12 06:12 +0000 [5d34bce635]  Corey Farrell <git@cfware.com>
 
-	  This patch addresses both issues.
+	* main/editline: Add .gitignore.
 
-	  First, it adds the notion of a cluster discovery message both within the
-	  Stasis message bus, as well as the binary event messages that
-	  res_corosync uses to transmit data back and forth within the cluster.
-	  When Asterisk joins the cluster, it sends a discovery message to the other
-	  nodes in the cluster, which correlates the Corosync nodeid along with
-	  the Asterisk EID. res_corosync now maintains a hash of Corosync nodeids
-	  to Asterisk EIDs, such that it can map changes in cluster state with the
-	  Asterisk instance that has that nodeid. Likewise, when an Asterisk
-	  instance receives a discovery message from a node in the cluster, it now
-	  sends its own discovery message back to the originating node with the
-	  local Asterisk EID. This lets Asterisk instances within the cluster
-	  build a complete picture of the other Asterisk instances within the
-	  cluster.
+	  This patch adds a .gitignore for main/editline to ignore all build results.
 
-	  Second, it publishes the discovery messages onto the Stasis message bus.
-	  Said messages are published whenever a node joins or leaves the cluster.
-	  Interested modules can subscribe for the ast_cluster_discovery_type()
-	  message under the ast_system_topic() and be notified when changes in
-	  cluster state occur.
+	  Change-Id: I68c7bf375ea46282689e5a706534b69fca233b5d
 
-	  Change-Id: I9015f418d6ae7f47e4994e04e18948df4d49b465
+2015-04-11 23:22 +0000 [d6605b3c10]  Matt Jordan <mjordan@digium.com>
+
+	* .gitignore: Ignore tarballs (*.gz)
 
-2016-07-04 13:54 +0000 [2c16a81dd5]  Alexei Gradinari <alex2grad@gmail.com>
+	  This patch updates the root .gitignore file to ignore files with a .gz
+	  extension. This will cause git to ignore downloaded sound tarballs in
+	  the the sounds/ directory.
 
-	* res_pjsip: Added "subscribe_context" to endpoint
+	  Change-Id: Ie84f085cc0fa51262209e7bfc1b1ba8c04a1ef59
 
-	  If specified, incoming SUBSCRIBE requests will be searched for the matching
-	  extension in the indicated context. If no "subscribe_context" is specified,
-	  then the "context" setting is used.
+2015-04-11 13:20 +0000 [b35e184d41]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25471 #close
+	* Add .gitignore and .gitreview files
 
-	  Change-Id: I3fb7a15f5bc154079bd348c08b7ad1cdd2d5e514
+	  Add the .gitignore and .gitreview files to the asterisk repo.
 
-2016-07-04 05:58 +0000 [a1bd57884d]  Alexander Traud <pabstraud@compuserve.com>
+	  NB:  You can add local ignores to the .git/info/exclude file
+	  without having to do a commit.
 
-	* BuildSystem: Avoid obsolete warning with libcurl.m4 on autoconf.
+	  Common ignore patterns are in the top-level .gitignore file.
+	  Subdirectory-specific ignore patterns are in their own .gitignore
+	  files.
 
-	  Updated the macro-set autoconf/libcurl.m4 to its latest upstream version. This
-	  avoids a warning about an obsolete macro on AC_HELP_STRING, because Asterisk is
-	  using AS_HELP_STRING everywhere else already.
+	  Change-Id: I842a1588ff27d8a0189f12d597f0a7af033d6c69
+	  Tested-by: George Joseph
 
-	  ASTERISK-26046
+2015-04-11 10:27 +0000 [356b770632]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  Change-Id: I8299faf504ceaeee3e39930c59293809e116c631
+	* clang compiler warnings: Fix various warnings for tests
+
+	  This patch fixes a variety of clang compiler warnings for unit tests. This
+	  includes autological comparison issues, ignored return values, and
+	  interestingly enough, one embedded function. Fun!
 
-2016-06-30 15:17 +0000 [640fbbbe28]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4555
 
-	* features: Fix channel datastore access.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4555.patch submitted by dkdegroot (License 6600)
+	  ........
+
+	  Merged revisions 434705 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
+
+	  Merged revisions 434706 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434707 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2015-04-11 10:11 +0000 [5f181bcccd]  Juergen Spies (License 6698)
+
+	* res/res_pjsip_t38: Add missing initialization of t38faxmaxdatagram
+
+	  Prior to this patch, the far_max_datagram value on the UDPTL structure would
+	  remain -1 if the remote endpoint fails to provide the SDP media attribute
+	  T38FaxMaxDatagram. This can result in the INVITE request being rejected. With
+	  this patch, we will now properly initialize the value with either the default
+	  value or with the value provided by pjsip.conf's t38_udptl_maxdatagram
+	  parameter.
+
+	  Review: https://reviewboard.asterisk.org/r/4589
+
+	  ASTERISK-24928 #close
+	  Reported by: Juergen Spies
+	  Tested by: Juergen Spies
+	  patches:
+	    pjsipT38patch20150331.txt submitted by Juergen Spies (License 6698)
+	  ........
+
+	  Merged revisions 434688 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434689 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2015-04-10 18:37 +0000 [c499cabf53]  Richard Mudgett <rmudgett@digium.com>
+
+	* chan_pjsip/res_pjsip/bridge_softmix/core: Improve translation path choices.
+
+	  With this patch, chan_pjsip/res_pjsip now sets the native formats to the
+	  codecs negotiated by a call.
+
+	  * The changes in chan_pjsip.c and res_pjsip_sdp_rtp.c set the native
+	  formats to include all the negotiated audio codecs instead of only the
+	  initial preferred audio codec and later the currently received audio
+	  codec.
+
+	  * The audio frame handling in channel.c:ast_read() is more streamlined and
+	  will automatically adjust to changes in received frame formats.  The new
+	  policy is to remove translation and pass the new frame format to the
+	  receiver except if the translation was to a signed linear format.  A more
+	  long winded version is commented in ast_read() along with some caveats.
+
+	  * The audio frame handling in channel.c:ast_write() is more streamlined
+	  and will automatically adjust any needed translation to changes in the
+	  frame formats sent.  Frame formats sent can change for many reasons such
+	  as a recording is being played back or the bridged peer changed the format
+	  it sends.  Since it is a normal expectation that sent formats can change,
+	  the codec mismatch warning message is demoted to a debug message.
+
+	  * Removed the short circuit check in
+	  channel.c:ast_channel_make_compatible_helper().  Two party bridges need to
+	  make channels compatible with each other.  However, transfers and moving
+	  channels among bridges can result in otherwise compatible channels having
+	  sub-optimal translation paths if the make compatible check is short
+	  circuited.  A result of forcing the reevaluation of channel compatibility
+	  is that the asterisk.conf:transcode_via_slin and codecs.conf:genericplc
+	  options take effect consistently now.  It is unfortunate that these two
+	  options are enabled by default and negate some of the benefits to the
+	  changes in channel.c:ast_read() by forcing translation through signed
+	  linear on a two party bridge.
+
+	  * Improved the softmix bridge technology to better control the translation
+	  of frames to the bridge.  All of the incoming translation is now normally
+	  handled by ast_read() instead of splitting any translation steps between
+	  ast_read() and the slin factory.  If any frame comes in with an unexpected
+	  format then the translation path in ast_read() is updated for the next
+	  frame and the slin factory handles the current frame translation.
+
+	  This is the final patch in a series of patches aimed at improving
+	  translation path choices.  The other patches are on the following reviews:
+	  https://reviewboard.asterisk.org/r/4600/
+	  https://reviewboard.asterisk.org/r/4605/
+
+	  ASTERISK-24841 #close
+	  Reported by: Matt Jordan
+
+	  Review: https://reviewboard.asterisk.org/r/4609/
+	  ........
+
+	  Merged revisions 434671 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434672 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2015-04-10 16:06 +0000 [66f3fd0028]  Kevin Harwell <kharwell@digium.com>
+
+	* chan_sip: make progressinband default to no
+
+	  After the "progressinband" value setting of "never" was updated to never send a
+	  183 this separated its use from the "no" value. Since "never" was the default,
+	  but most users probably expect "no" this patch updates the default for the
+	  "progressinband" setting to "no."
 
-	  Found as a result of the testsuite tests/callparking test crashing.
+	  ASTERISK-24835 #close
+	  Reported by: Andrew Nagy
+	  Review: https://reviewboard.asterisk.org/r/4606/
+	  ........
 
-	  Several calls to ast_get_chan_featuremap_config() and
-	  ast_get_chan_features_xfer_config() did not lock the channel before
-	  calling so the channel's datastore list was accessed without the lock's
-	  protection.  Apparently another thread deleted a datastore on the
-	  channel's list while the crashing thread was walking the list.  Crash at
-	  0xdeaddead due to MALLOC_DEBUG's memory filler value as a result.
+	  Merged revisions 434654 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Add missing channel locks to calls that were not already protected
-	  as the doxygen for those calls indicates.
 
-	  Change-Id: Id273b3d305cc616406c353cbc841b2b7655efaa1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434655 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-22 17:26 +0000 [359134c8d3]  Richard Mudgett <rmudgett@digium.com>
+2015-04-10 12:56 +0000 [8bae18ab93]  yaron nahum (License 6676)
 
-	* res_pjsip_session.c: Don't send extra BYE if SDP invalid.
+	* res_pjsip: Add an 'auto' option for DTMF Mode
 
-	  When an answer SDP is invalid we were disconnecting the outgoing call and
-	  sending two BYE requests.  The first BYE was sent by PJPROJECT because of
-	  the invalid SDP answer.  The second BYE was sent by Asterisk because it
-	  thought the canceled call was the result of the RFC5407 section 3.1.2 race
-	  condition.
+	  This patch adds support for automatically detecting the type of DTMF that a
+	  PJSIP endpoint supports. When the 'dtmf_mode' endpoint option is set to 'auto',
+	  the channel created for an endpoint will attempt to determine if RFC 4733
+	  DTMF is supported. If so, it will use that DTMF type. If not, the DTMF type
+	  for the channel will be set to inband.
 
-	  * Made not send the BYE on a canceled session if the SDP negotiation is
-	  incomplete because PJPROJECT has already sent a BYE for the failed
-	  negotiation.
+	  Review: https://reviewboard.asterisk.org/r/4438
 
-	  ASTERISK-25772 #close
-	  Reported by:  Dmitriy Serov
+	  ASTERISK-24706 #close
+	  Reported by: yaron nahum
+	  patches:
+	    yaron_patch_3_Feb.diff submitted by yaron nahum (License 6676)
+	  ........
 
-	  Change-Id: I44ad0bd0605e8eeb7035c890d6f97a1331f1a836
+	  Merged revisions 434637 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-27 17:19 +0000 [5fabcf2ca1]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_session.c: End call on initial invalid SDP negotiation.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434638 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When an incoming call defers SDP negotiation and then sends us an invalid
-	  SDP in the ACK, we need to send a BYE to disconnect the call.  In this
-	  case SDP negotiation has failed and we don't have valid media streams
-	  negotiated.
+2015-04-10 12:00 +0000 [f69e46de25]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25772
+	* res_pjsip_config_wizard: Cleanup load unload
 
-	  Change-Id: Ia358516b0fc1e6c4c139b78246f10b9da7a2dfb8
+	  While investigating other unload issues I realized that the load/unload process 
+	  for the config wizard was pretty ugly so I've refactored it as follows...
 
-2016-06-23 15:13 +0000 [38a4e983dc]  Richard Mudgett <rmudgett@digium.com>
+	  When the res_pjsip sorcery instance is created the config_wizard bumps it's own 
+	  module reference to prevent it from unloading while the sorcery instance is 
+	  still active.  When res_pjsip unloads and it's sorcery instance is destroyed, 
+	  the config wizard unrefs itself which then allows itself to unload cleanly.  
+	  Since the config wizard now can't load after res_pjsip or unload before it 
+	  (which should have been the correct behavior all along), I was able to remove 
+	  the chunks of code in both load_module and unload_module that handled that case.
 
-	* res_pjsip.c: Register PJMEDIA error code decoder.
+	  Ran the testsuite tests to insure there were no functional changes and REF_DEBUG 
+	  to insure that Asterisk was shutting down cleanly with no FRACKs or leaks.
 
-	  Registering the PJMEDIA error codes allows errors found when parsing an
-	  incoming SDP to be easier to figure out.
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/4610/
+	  ........
 
-	  "Missing SDP rtpmap for dynamic payload type (PJMEDIA_SDP_EMISSINGRTPMAP)"
-	  is much easier to understand than "Unknown error 220030".
+	  Merged revisions 434619 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25772
 
-	  Change-Id: I44b2dcea656fedd7593171be9e845880a2c70ca0
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434620 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-27 16:56 +0000 [1952434df5]  Richard Mudgett <rmudgett@digium.com>
+2015-04-10 11:38 +0000 [6f1a7fe05f]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_session.c: Remove unused parameter from handle_incoming().
+	* bridge_softmix.c,channel.c: Minor code simplification and cleanup.
 
-	  Change-Id: Iedd182d189ec947c42edc2c66c4bda3c22060daa
+	  * Made code easier to follow in bridge_softmix.c:analyse_softmix_stats()
+	  and made some debug messages more helpful.
 
-2016-06-22 18:02 +0000 [28928ba5c4]  Richard Mudgett <rmudgett@digium.com>
+	  * Made some debug and warning messages more helpful in
+	  channel.c:set_format().
 
-	* res_pjsip: Add missing NULL checks when using pjsip_inv_end_session().
+	  Review: https://reviewboard.asterisk.org/r/4607/
+	  ........
 
-	  pjsip_inv_end_session() is documented as being able to return the
-	  passed in tdata parameter set to NULL on success.
+	  Merged revisions 434617 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I09d53725c49b7183c41bfa1be3ff225f3a8d3047
 
-2016-06-30 08:25 +0000 [43a78100c0]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434618 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* configure:  Fix HAVE_PJSIP_EVSUB_GRP_LOCK not set with external pjproject
+2015-04-10 11:32 +0000 [0b805cb875]  Richard Mudgett <rmudgett@digium.com>
 
-	  There was a typo in configure.ac preventing HAVE_PJSIP_EVSUB_GRP_LOCK
-	  from getting set when using an external pjproject.
+	* translate.c: Only select audio codecs to determine the best translation choice.
 
-	  ASTERISK-26099 #close
-	  Reported-by: Ross Beer
+	  Given a source capability of h264 and ulaw, a destination capability of
+	  h264 and g722 then ast_translator_best_choice() would pick h264 as the
+	  best choice even though h264 is a video codec and Asterisk only supports
+	  translation of audio codecs.  When the audio starts flowing, there are
+	  warnings about a codec mismatch when the channel tries to write a frame to
+	  the peer.
 
-	  Change-Id: I709af70428e125fb5ccd44b171d25dd29141f0ae
+	  * Made ast_translator_best_choice() only select audio codecs.
 
-2016-06-29 15:31 +0000 [99eff80e76]  Matt Jordan <mjordan@digium.com>
+	  * Restore a check in channel.c:set_format() lost after v1.8 to prevent
+	  trying to set a non-audio codec.
 
-	* hep.conf.sample: Default 'enabled' to 'no'
+	  This is an intermediate patch for a series of patches aimed at improving
+	  translation path choices for ASTERISK-24841.
 
-	  Following the principle of least surprise, we should not be sending
-	  massive numbers of PJSIP and RTCP HEP packets out into the ether to some
-	  only-slightly-random IP address. Having 'enabled' set to 'no' in the
-	  sample configuration file should prevent this from happening for those
-	  who run 'make samples'.
+	  This patch is a complete enough fix for ASTERISK-21777 as the v11 version
+	  of ast_translator_best_choice() does the same thing.  However, chan_sip.c
+	  still somehow tries to call ast_codec_choose() which then calls
+	  ast_best_codec() with a capability set that doesn't contain any audio
+	  formats for the incoming call.  The remaining warning message seems to be
+	  a benign transient.
 
-	  ASTERISK-26159 #close
+	  ASTERISK-21777 #close
+	  Reported by: Nick Ruggles
 
-	  Change-Id: I1753a64ca83a3442a6ebdc31061f8185c062d9b1
+	  ASTERISK-24380 #close
+	  Reported by: Matt Jordan
 
-2016-06-29 15:09 +0000 [78960975f2]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4605/
+	  ........
 
-	* pjproject/patches/config_site: Increase the max number of ICE candidates
+	  Merged revisions 434614 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  When negotiating ICE candidates with WebRTC capable endpoints, many
-	  networks will result in a browser offering ICE candidates that exceeds
-	  the default number of max candidates, 16. This patch bumps the max
-	  candidates to 32, with the max checks at twice the number of candidates.
-	  In practice, this has shown to be sufficient for browser/WebRTC
-	  negotiation.
+	  Merged revisions 434615 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ifd8da8b315f5ae14814d4ce20e10d2e6355020e5
 
-2016-06-28 09:00 +0000 [d07c8a0504]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434616 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* codecs:  Fix ABI incompatibility created by adding format_name to ast_codec
+2015-04-10 09:56 +0000 [894153b8b1]  Matt Jordan <mjordan@digium.com>
 
-	  Adding format_name even to the end of ast_codec caused issued with
-	  binary codec modules because the pointer would be garbage in asterisk
-	  when they registered.  So, the ast_codec structure was reverted and an
-	  internal_ast_codec structure was created just for use in codec.c.  A new
-	  internal-only API was also added (__ast_codec_register_with_format) so
-	  that codec_builtin could register codecs with the format_name in a
-	  separate parameter rather than in the ast_codec structure.
+	* res/ari: Fix model validation for ChannelHold event
 
-	  ASTERISK-26144 #close
-	  Reported-by: Alexei Gradinari
+	  When the ChannelHold event was added, the 'musicclass' parameter was
+	  erroneously removed. This caused the ChannelHold events to be rejected as
+	  they failed model validation. This patch updates the Swagger schema such that
+	  it now properly reflects the event that is being created.
 
-	  Change-Id: I6df1b08f6a6ae089db23adfe1ebc8636330265ba
+	  Hooray for tests that catch things like this.
+	  ........
 
-2016-06-28 08:22 +0000 [f3d236ca7f]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 434597 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* BuildSystem:  Fix a few issues hightlighted by gcc 6.x
 
-	  gcc 6.1.1 caught a few more issues.
-	  Made sure the unit tests still pass for the func_env and stdtime
-	  issues.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434598 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26157 #close
+2015-04-10 08:32 +0000 [02a0a4d65f]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I6664d8f34a45bc1481d2a854481c7878b0c1cf8e
+	* dns: Fix build when TEST_FRAMEWORK is not defined.
 
-2016-06-28 10:33 +0000 [9d5b0934d9]  Matt Jordan <mjordan@digium.com>
 
-	* configs/basic-pbx/modules.conf: Remove 'bad' modules
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434583 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch removes the following modules:
-	   - pbx_functions: It never existed.
-	   - res_pjsip_log_forwarder: It no longer exists.
-	   - res_hep_pjsip: The base HEP module wasn't loaded, and most basic PBXs
-	                    aren't going to be installing HOMER
-	   - res_pjsip_phoneprov_provider: The basic res_phoneprov module isn't
-	                    loaded, and we aren't configured to make use of the
-	                    module
+2015-04-10 07:40 +0000 [80c443bea4]  Y Ateya (License 6693)
 
-	  Change-Id: Id91f68cae7c9c8c3d370029fe1268cb51e4ff5a5
+	* channels/chan_iax2: Improve POKE expiration time calculation for lossy networks
 
-2016-06-22 11:19 +0000 [1dfc286418]  Joshua Colp <jcolp@digium.com>
+	  POKE is used to check for peer availability; however, in networks with packet
+	  loss, the current calculations may result in POKE expiration times that are too
+	  short. This patch alters the expiration/retry time logic to take into account
+	  the last known qualify round trip time, as opposed to always using a static
+	  value for each peer.
 
-	* siren: Add format attribute modules for Siren7 and Siren14.
+	  Review: https://reviewboard.asterisk.org/r/4536
 
-	  This change removes hardcoded SDP parsing and generation for
-	  Siren7 and Siren14 from chan_sip and moves it to format attribute
-	  modules so it can also be used by chan_pjsip.
+	  ASTERISK-22352 #close
+	  Reported by: Frederic Van Espen
 
-	  With this the fmtp lines for both are added with the bitrate
-	  information.
+	  ASTERISK-24894 #close
+	  Reported by: Y Ateya
+	  patches:
+	    poke_noanswer_duration.diff submitted by Y Ateya (License 6693)
+	  ........
 
-	  ASTERISK-26021
+	  Merged revisions 434564 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ibb004eda37a14c0a35ef0613f6237977fc800037
+	  Merged revisions 434565 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-23 04:33 +0000 [5f0a098243]  Alexander Traud <pabstraud@compuserve.com>
 
-	* BuildSystem: Avoid obsolete warning with AC_TYPE_SIGNAL on autoconf.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434566 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Removed the obsolete macro AC_TYPE_SIGNAL because Asterisk does not use K&R C
-	  but requires ANSI C anyway.
+2015-04-10 07:23 +0000 [b3d01f1fbf]  Y Ateya (License 6693)
 
-	  ASTERISK-26046
+	* channels/chan_iax2: Add a configuration parameter for call token expiration
 
-	  Change-Id: I914c014385e1862102d90fe7650621def78db02e
+	  This patch adds a new configuration parameter, 'calltokenexpiration', that
+	  controls how long before an authentication call token is expired. The default
+	  maintains the RFC specified 10 seconds. Setting it to a higher value may be
+	  useful in lossy networks.
 
-2016-06-02 17:26 +0000 [b3c787d1dd]  Alexei Gradinari <alex2grad@gmail.com>
+	  Review: https://reviewboard.asterisk.org/r/4588
 
-	* res_pjsip: improve realtime performance #2
+	  ASTERISK-24939 #close
+	  Reported by: Y Ateya
+	  patches:
+	    ctoken_configuration.diff submitted by Y Ateya (License 6693)
 
-	  The patch removes updating all Endpoints' status on startup.
-	  Instead, only non-qualified aors with static contact
-	  and non-qualified non-expired contacts are retrieved from the realtime to
-	  update the endpoint status to ONLINE.
-	  The endpoint name was added to the contact object to simply find the endpoint
-	  that created this contact.
 
-	  The status of endpoints with qualified aors will be updated by 'qualify'
-	  functions.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434563 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26061 #close
+2015-04-09 18:12 +0000 [ed6b6e3c03]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Id324c1776fa55d3741e0c5457ecac0304cb1a0df
+	* res_pjsip_phoneprov_provider: Fix reference leak on unload
 
-2016-06-23 13:47 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  res_pjsip_phoneprov_provider was leaking references to phoneprov objects due to 
+	  a missing OBJ_NODATA in an ao2_callback in load_users().  Rather than adding the 
+	  OBJ_NODATA, I changed load_users to use a more straightforward ao2_iterator.  
+	  This plugged the leak but exposed an unload order issue between 
+	  res_pjsip_phoneprov_provider, res_phoneprov and res_pjsip.
 
-	* asterisk 13.10.0-rc1 Released.
+	  res_pjsip_phoneprov_provider unloads first, then res_phoneprov, then res_pjsip.  
+	  Since res_pjsip_phoneprov_provider uses res_pjsip's sorcery instance, when it 
+	  unloads, it's objects are still in the sorcery instance.  When res_pjsip 
+	  unloads, it destroys all its objects including res_pjsip_phoneprov_provider's.  
+	  The phoneprov destructor then attempts to unregister the extension from 
+	  res_phoneprov but because res_phoneprov is already cleaned up, its users 
+	  container is gone and we get a FRACK.
 
-2016-06-23 08:42 +0000 [62349ac1b4]  Mark Michelson <mmichelson@digium.com>
+	  Simple solution, check for the NULL users container before attempting to remove 
+	  the entry. Duh.
 
-	* Release summaries: Add summaries for 13.10.0-rc1
+	  Ran tests/res_phoneprov/res_phoneprov_provider.  No leaks in 
+	  res_pjsip_phoneprov_provider and no FRACKs.
 
-2016-06-23 08:38 +0000 [8da6ba4328]  Mark Michelson <mmichelson@digium.com>
+	  Reported-by: Corey Farrell
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/4608/
+	  ASTERISK-24935 #close
+	  ........
 
-	* .version: Update for 13.10.0-rc1
+	  Merged revisions 434545 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-23 08:38 +0000 [170b85e3ae]  Mark Michelson <mmichelson@digium.com>
 
-	* .lastclean: Update for 13.10.0-rc1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434547 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-23 08:38 +0000 [4af7049b8f]  Mark Michelson <mmichelson@digium.com>
+2015-04-09 18:08 +0000 [9a63ada03a]  gtjoseph <george.joseph@fairview5.com>
 
-	* realtime: Add database scripts for 13.10.0-rc1
+	* loader/main: Don't set ast_fully_booted until deferred reloads are processed
 
-2016-06-22 15:04 +0000 [3d904659ec]  Corey Farrell <git@cfware.com>
+	  Until we have a true module management facility it's sometimes necessary for one 
+	  module to force a reload on another before its own load is complete.  If 
+	  Asterisk isn't fully booted yet, these reloads are deferred.  The problem is 
+	  that asterisk reports fully booted before processing the deferred reloads which 
+	  means Asterisk really isn't quite ready when it says it is.
 
-	* res_fax: Fix reference leak in fax_v21_session_new.
+	  This patch moves the report of fully booted after the processing of the deferred 
+	  reloads is complete.
 
-	  fax_v21_session_new created a session details object but only released
-	  the allocation reference during error conditions.  fax_session_new adds
-	  it's own reference to details if needed so the caller is always
-	  responsible for cleaning it's own reference.
+	  Since the pjsip stack has the most number of related modules, I ran the 
+	  channels/pjsip testsuite to make sure there aren't any issues.  All tests 
+	  passed.
 
-	  ASTERISK-26141 #close
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/4604/
+	  ........
 
-	  Change-Id: Ie7fc52a83b6596ce9ce2d5a2bd9f3e204f48fc88
+	  Merged revisions 434544 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-22 13:41 +0000 [48db4c2159]  gtjoseph <gjoseph@digium.com>
 
-	* res_rtp_asterisk:  Fix a self-comparison identified by gcc 6
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434546 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  gcc 6 caught a previously unidentified self-comparison in
-	  ice_candidate_cmp.  Fixed it and re-ordered the predicates for better
-	  short-circuiting.
+2015-04-09 17:07 +0000 [520b9f2174]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-26140 #close
+	* res_pjsip: add CLI command to show global and system configuration
 
-	  Change-Id: I3da713c568e24064430257b3502fbdafd35af7a7
+	  Added a new CLI command for res_pjsip that shows both global and system
+	  configuration settings: pjsip show settings
 
-2016-06-22 10:37 +0000 [bc69b03316]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-24918 #close
+	  Reported by: Scott Griepentrog
+	  Review: https://reviewboard.asterisk.org/r/4597/
+	  ........
 
-	* chan_unistim:  Fix memcpy in get_to_address
+	  Merged revisions 434527 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  A code block only enabled when HAVE_PKTINFO is not defined (FreeBSD)
-	  was using a pointer to a pointer as the destination of a memcpy and a
-	  '&' instead of '*' in the sizeof.
 
-	  ASTERISK-26138 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434528 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id4927ff256c0e470bdf7bcfc025146a2f656e708
+2015-04-09 11:09 +0000 [b2b1f24af6]  Richard Mudgett <rmudgett@digium.com>
 
-2016-06-20 13:18 +0000 [1b79e2deff]  Mark Michelson <mmichelson@digium.com>
+	* chan_iax2.c: Fix ref leak in iax2_request().
 
-	* Fix Alembic upgrades.
+	  * Increased warning message format capability string buffer size in
+	  iax2_request().
 
-	  A non-existent constraint was being referenced in the upgrade script.
-	  This patch corrects the problem by removing the reference.
+	  Review: https://reviewboard.asterisk.org/r/4601/
+	  ........
 
-	  This patch fixes another realtime problem as well. Our Alembic scripts
-	  store booleans as yes or no values. However, Sorcery tries to insert
-	  "true" or "false" instead. This patch updates Sorcery to use "yes" and
-	  "no"
+	  Merged revisions 434510 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26128 #close
 
-	  Change-Id: I366dbbf91418a9cb160b3ca74b0e59b5ac284bec
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434511 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-22 10:55 +0000 [e30602587c]  Alexander Traud <pabstraud@compuserve.com>
+2015-04-09 11:05 +0000 [459171be12]  Richard Mudgett <rmudgett@digium.com>
 
-	* BuildSystem: Avoid obsolete warning with AC_FUNC_SETVBUF_REVERSED on autoconf.
+	* bridge_native_rtp.c: Defer allocation and check if it fails in native_rtp_bridge_compatible().
 
-	  Removed the obsolete macro AC_FUNC_SETVBUF_REVERSED because Asterisk does not
-	  support the platform SVR2 from the year 1987 anymore.
+	  Review: https://reviewboard.asterisk.org/r/4601/
+	  ........
 
-	  ASTERISK-26046
+	  Merged revisions 434508 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I28161b037feb2d29ab46ed20e785928460226c22
 
-2016-06-22 10:51 +0000 [77da168e58]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434509 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* test_res_pjsip_scheduler: Add 'depends' on pjproject in MODULEINFO
+2015-04-09 10:43 +0000 [3ef0a17b1f]  yaron nahum (License 6676)
 
-	  Since the file was missing the depends on pjproject, it wasn't
-	  picking up the pjproject related include path.  If there was no
-	  system installed pjproject and pjproject-bundled was used, a compile
-	  would fail because pjsip.h wasn't found.
+	* res/res_pjsip_dlg_options: Add a module to handle in-dialog OPTIONS requests
 
-	  ASTERISK-26139 #close
+	  This patch adds a new session supplement that handles in-dialog OPTIONS
+	  requests. Said OPTIONS requests are sent a 200 OK, as an endpoint lookup
+	  for the OPTIONS request would already have been done by the time the
+	  session supplement receives the inbound request.
 
-	  Change-Id: I2ee64a999051452bc198c4e2c168c70769cd3757
+	  ASTERISK-24862 #close
+	  Reported by: yaron nahum
+	  patches:
+	    res_pjsip_dlg_options.c submitted by yaron nahum (License 6676)
+	  ........
 
-2016-06-21 06:52 +0000 [dfcd466bf0]  Torrey Searle <torrey@voxbone.com>
+	  Merged revisions 434506 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_rtp_asterisk: fix memory leak in dtls
 
-	  ensure that cert bios get freed after creating the fingerprint
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434507 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26129 #close
+2015-04-09 09:58 +0000 [c08ebc6eeb]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I44d23aea07dce80176ca1ff877c5ace9452ef451
+	* Reduce duplication of common DNS code.
 
-2016-06-21 17:42 +0000 [c982da0641]  Richard Mudgett <rmudgett@digium.com>
+	  The NAPTR and SRV branches were worked on independently and
+	  resulted in some code being duplicated in each. Since both
+	  have been merged into trunk now, this patch reduces the
+	  duplication by factoring out common code into its own
+	  source files.
 
-	* res_pjproject.c: Replace inlined DEBUG_ATLEAST() with macro.
 
-	  Change-Id: I8799fb0a347ad76e747dafd0eacf1ea1086b9a8c
 
-2016-06-12 11:19 +0000 [6a568bcc66]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434490 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_pubsub: Address SEGV when attempting to terminate a subscription
+2015-04-09 07:57 +0000 [ea0098724e]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  Occasionally under load we'll attempt to send a final NOTIFY on a
-	  subscription that's already been terminated and a SEGV will occur
-	  down in pjproject's evsub_destroy function.  This is a result of a
-	  race condition between all the paths that can generate a notify
-	  and/or destroy the underlying pjproject evsub object:
+	* clang compiler warnings: Fix autological comparisons
 
-	   * The client can send a SUBSCRIBE with Expires: 0.
-	   * The client can send a SUBSCRIBE/refresh.
-	   * The subscription timer can expire.
-	   * An extension state can change.
-	   * An MWI event can be generated.
-	   * The pjproject transaction timer (timer_b) can expire.
+	  This fixes autological comparison warnings in the following:
+	   * chan_skinny: letohl may return a signed or unsigned value, depending on the
+	     macro chosen
+	   * func_curl: Provide a specific cast to CURLoption to prevent mismatch
+	   * cel: Fix enum comparisons where the enum can never be negative
+	   * enum: Fix comparison of return result of dn_expand, which returns a signed
+	     int value
+	   * event: Fix enum comparisons where the enum can never be negative
+	   * indications: tone_data.freq1 and freq2 are unsigned, and hence can never be
+	     negative
+	   * presencestate: Use the actual enum value for INVALID state
+	   * security_events: Fix enum comparisons where the enum can never be negative
+	   * udptl: Don't bother to check if the return value from encode_length is less
+	     than 0, as it returns an unsigned int
+	   * translate: Since the parameters are unsigned int, don't bother checking
+	     to see if they are negative. The cast to unsigned int would already blow
+	     past the matrix bounds.
+	   * res_pjsip_exten_state: Use a temporary value to cache the return of
+	     ast_hint_presence_state
+	   * res_stasis_playback: Fix enum comparisons where the enum can never be
+	     negative
+	   * res_stasis_recording: Add an enum value for the case where the recording
+	     operation is in error; fix enum comparisons
+	   * resource_bridges: Use enum value as opposed to -1
+	   * resource_channels: Use enum value as opposed to -1
 
-	  Normally when our pubsub_on_evsub_state is called with a terminate,
-	  we push a task to the serializer and return at which point the dialog
-	  is unlocked.  This is usually not a problem because the task runs
-	  immediately and locks the dialog again.  When the system is heavily
-	  loaded though, there may be a delay between the unlock and relock
-	  during which another event may occur such as the subscription timer
-	  or timer_b expiring, an extension state change, etc.  These may also
-	  cause a terminate to be processed and if so, we could cause pjproject
-	  to try to destroy the evsub structure twice.  There's no way for us to
-	  tell that the evsub was already destroyed and the evsub's group lock
-	  can't tolerate this and SEGVs.
+	  Review: https://reviewboard.asterisk.org/r/4533
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4533.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  The remedy is twofold.
+	  Merged revisions 434469 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	   * A patch has been submitted to Teluu and added to the bundled
-	     pjproject which adds add/decrement operations on evsub's group lock.
+	  Merged revisions 434470 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	   * In res_pjsip_pubsub:
-	     * configure.ac and pjproject-bundled's configure.m4 were updated
-	       to check for the new evsub group lock APIs.
-	     * We now add a reference to the evsub group lock when we create
-	       the subscription and remove the reference when we clean up the
-	       subscription.  This prevents evsub from being destroyed before
-	       we're done with it.
-	     * A state has been added to the subscription tree structure so
-	       termination progress can be tracked through the asyncronous tasks.
-	     * The pubsub_on_evsub_state callback has been split so it's not doing
-	       double duty.  It now only handles the final cleanup of the
-	       subscription tree.  pubsub_on_rx_refresh now handles both client
-	       refreshes and client terminates.  It was always being called for
-	       both anyway.
-	     * The serialized_on_server_timeout task was removed since
-	       serialized_pubsub_on_rx_refresh was almost identical.
-	     * Missing state checks and ao2_cleanups were added.
-	     * Some debug levels were adjusted to make seeing only off-nominal
-	       things at level 1 and nominal or progress things at level 2+.
 
-	  ASTERISK-26099 #close
-	  Reported-by: Ross Beer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434471 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I779d11802cf672a51392e62a74a1216596075ba1
+2015-04-08 21:05 +0000 [2201e27340]  Stefan Engström (License 6691)
 
-2016-06-21 07:05 +0000 [ef97911a1c]  Alexander Traud <pabstraud@compuserve.com>
+	* apps/app_queue: Prevent possible crash when evaluating queue penalty rules
 
-	* res_rtp_asterisk: Use latest DTLS version available by underlying platform.
+	  Although it only occurred once, a crash occurred when a queue attempted to
+	  evaluate a queue penalty rule that appeared to have already been destroyed.
+	  In many locations in app_queue, a test is done to see if qe->pr is NULL;
+	  however, when we dispose of a queue's penalty rules, we don't set the pointer
+	  to NULL after free'ing it. This patch does that to prevent any dangling
+	  pointers from lingering on the queue object.
 
-	  Do not use DTLSv1_method() but DTLS_method() when available in OpenSSL of the
-	  underlying platform. This change enables DTLS 1.2 since OpenSSL 1.0.2, for
-	  WebRTC (DTLS-SRTP via SIP-over-WebSockets). This change enables AEAD-based
-	  cipher-suites.
+	  Review: https://reviewboard.asterisk.org/r/4522
 
-	  ASTERISK-26130 #close
+	  ASTERISK-23319 #close
+	  Reported by: Vadim
+	  patches:
+	    rb4552.patch submitted by Stefan Engström (License 6691)
+	  ........
 
-	  Change-Id: I41f24448d6d2953e8bdb97c9f4a6bc8a8f055fd0
+	  Merged revisions 434448 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-06-21 10:53 +0000 [69d58a1e37]  Scott Griepentrog <scott@griepentrog.com>
+	  Merged revisions 434449 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* PJSIP: provide transport type with received messages
 
-	  The receipt of a SIP MESSAGE may occur over any transport including TCP
-	  and TLS. When the message is received, the original URI is added to the
-	  message in the field PJSIP_RECVADDR, but this is insufficient to ensure
-	  a reply message can reach the originating endpoint. This patch adds the
-	  PJSIP_TRANSPORT field populated with the transport type.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434450 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26132 #close
+2015-04-08 13:32 +0000 [a759714101]  Jonathan Rose <jrose@digium.com>
 
-	  Change-Id: I28c4b1e40d573a056c81deb213ecf53e968f725e
+	* res_pjsip_t38: Fix FAX failures when using PJSIP with authentication
 
-2016-06-21 08:01 +0000 [cbfa9f771e]  Alexander Traud <pabstraud@compuserve.com>
+	  Without this patch, if a PJSIP endpoint with udptl enabled and authentication
+	  set attempted to use sendFax, the FAX session would fail during setup. This
+	  was because the invite issued in response to being auth challenged would cause
+	  the PJSIP channel performing the FAX to receive a second T38 framehook and
+	  this would cause frames to be consumed in an inappropriate manner.
 
-	* BuildSystem: Avoid obsolete warning with HELP_STRING on autoconf.
+	  ASTERISK-24933 #close
+	  Reported by: Jonathan Rose
+	  Review: https://reviewboard.asterisk.org/r/4577/
+	  ........
 
-	  Some configure scripts used both AC_HELP_STRING and its replacement
-	  AS_HELP_STRING. For consistency and to avoid obsolete warnings, those were
-	  changed to AS_HELP_STRING.
+	  Merged revisions 434425 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26046
 
-	  Change-Id: I8aad4fd2bdee40aa2a31ce3339a1eb33ff4f5b0f
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434431 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-20 10:29 +0000 [ba0d9e7f7a]  Joshua Colp <jcolp@digium.com>
+2015-04-08 13:20 +0000 [09df34d880]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_session: Handle race condition at shutdown with timer.
+	* Bridging: Eliminate the unnecessary make channel compatible with bridge operation.
 
-	  When shutting down res_pjsip_session will get unloaded before res_pjsip.
-	  The act of unloading unregisters all the PJSIP services and sets
-	  their module IDs to -1. In some cases it is possible for a timer to
-	  occur after this happens which calls into res_pjsip_session. The
-	  res_pjsip_session module can then try to get the session from the
-	  INVITE session using the module ID. Since the module ID is now -1
-	  this fails.
+	  When a channel enters the bridging system it is first made compatible with
+	  the bridge and then the bridge technology makes the channel compatible
+	  with the technology.  For all but the DAHDI native and softmix bridge
+	  technologies the make channel compatible with the bridge step is an
+	  effective noop because the other technologies allow all audio formats.
+	  For the DAHDI native bridge technology it doesn't matter because it is not
+	  an initial bridge technology and chan_dahdi allows only one native format
+	  per channel.  For the softmix bridge technology, it is a noop at best and
+	  harmful at worst because the wrong translation path could be setup if the
+	  channel's native formats allow more than one audio format.
 
-	  This change stores a copy of the module ID and uses it for the timer
-	  callback scenario. If the module ID is -1 the callback immediately
-	  returns but if the module ID is valid then it continues as normal.
+	  This is an intermediate patch for a series of patches aimed at improving
+	  translation path choices.
 
-	  This works as the original ID of the module is guaranteed to still
-	  be valid when used with the INVITE session.
+	  * Removed code dealing with the unnecessary step of making the channel
+	  compatible with the bridge.
 
-	  ASTERISK-26127 #close
+	  ASTERISK-24841
+	  Reported by: Matt Jordan
 
-	  Change-Id: I88df72525c4e9ef9f19c13aedddd3ac4a335c573
+	  Review: https://reviewboard.asterisk.org/r/4600/
+	  ........
 
-2016-06-20 12:13 +0000 [c1512f4108]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 434424 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* app_voicemail.c: Fix IMAP compile error.
 
-	  Fix compile error introduced by the patch for
-	  ASTERISK-26045
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434430 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I5b02876266f2824f4cec2b54d6ff4db5de5778d3
+2015-04-08 11:49 +0000 [8ec9a82b9a]  Maciej Szmigiero <mail@maciej.szmigiero.name> (license 6085)
 
-2016-06-16 15:56 +0000 [5134a8043a]  Alexei Gradinari <alex2grad@gmail.com>
+	* Security/tcptls: MitM Attack potential from certificate with NULL byte in CN.
 
-	* fix: memory leaks, resource leaks, out of bounds and bugs
+	  When registering to a SIP server with TLS, Asterisk will accept CA signed
+	  certificates with a common name that was signed for a domain other than the
+	  one requested if it contains a null character in the common name portion of
+	  the cert. This patch fixes that by checking that the common name length
+	  matches the the length of the content we actually read from the common name
+	  segment. Some certificate authorities automatically sign CA requests when
+	  the requesting CN isn't already taken, so an attacker could potentially
+	  register a CN with something like www.google.com\x00www.secretlyevil.net
+	  and have their certificate signed and Asterisk would accept that certificate
+	  as though it had been for www.google.com - this is a security fix and is
+	  noted in AST-2015-003.
 
-	  ASTERISK-26119 #close
+	  ASTERISK-24847 #close
+	  Reported by: Maciej Szmigiero
+	  Patches:
+	   asterisk-null-in-cn.patch submitted by mhej (license 6085)
+	  ........
 
-	  Change-Id: Iecbf7d0f360a021147344c4e83ab242fd1e7512c
+	  Merged revisions 434337 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2016-06-13 17:40 +0000 [cfebe3b94a]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 434338 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* ARI: Ensure announcer channels are destroyed.
+	  Merged revisions 434384 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Announcer channels were not being destroyed because the
-	  stasis_app_control structure that referenced them was not being
-	  destroyed. The control structure was not being destroyed because it was
-	  not being unlinked from its container. It was not being unlinked from
-	  its container because the after bridge callback for the announcer
-	  channel was not being run. The after bridge callback was not being run
-	  because the after bridge datastore was not being removed from the
-	  channel on destruction. The channel was not being destroyed because the
-	  hangup that used to destroy the channel was now only reducing the
-	  reference count to one. The reference count of the channel was only
-	  being reduced to one because the stasis_app_control structure was
-	  holding the final reference...
 
-	  The control structure used to not keep a reference to the channel, so
-	  that loop described above did not happen.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434385 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The solution is to manually remove the control structure from its
-	  container when the playback on a bridge is complete.
+2015-04-08 11:31 +0000 [2bd9e008a7]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26083 #close
-	  Reported by Joshua Colp
+	* format_cache.c: Add missing slin12 format to ast_format_cache_is_slinear().
+	  ........
 
-	  Change-Id: I0ddc0f64484ea0016245800b409b567dfe85cfb4
+	  Merged revisions 434357 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-20 08:05 +0000 [76516bd79d]  Alexander Traud <pabstraud@compuserve.com>
 
-	* http: leverage 'bindaddr' for TLS in http.conf
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434383 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The internal HTTP/WebSocket server supports both TCP and TLS, which can be
-	  activated separately via the file http.conf. The source code intends to re-use
-	  the TCP parameter 'bindaddr' for TLS, even if 'tlsbindaddr' is not specified
-	  explicitly. This did not work because of a typo. This change resolves this typo.
+2015-04-08 07:02 +0000 [3f54af689f]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-26126 #close
+	* chan_iax2: Fix compilation issue due to funky merge
 
-	  Change-Id: I5efb0409ae12044dfb3495b6b97b6d40a8c9c51f
+	  Don't mix declarations and code!
 
-2016-05-31 09:10 +0000 [89cc86fc38]  Vasil Kolev <vasil.kolev@securax.org>
 
-	* chan_sip: bigger buffers for headers, better failure mode
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434294 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Currently chan_sip can give weird messages if the contacts don't
-	  fit in the From: or To: headers. This fix changes the from,to and
-	  invite variables to use ast_str, allocates and deallocates them and
-	  resizes them if needed.
+2015-04-08 07:00 +0000 [a9b6a62461]  Jaco Kroon (License 5671)
 
-	  ASTERISK-26069 #close
+	* chan_iax2: Fix crash caused by unprotected access to iaxs[peer->callno]
 
-	  Change-Id: I1b68fcbddca6f6cc7d7a92fe1cb0d5430282b2b3
+	  This patch fixes an access to the peer callnumber that is unprotected by a
+	  corresponding mutex. The peer->callno value can be changed by multiple threads,
+	  and all data inside the iaxs array must be procted by a corresponding lock
+	  of iaxsl.
 
-2016-05-18 17:37 +0000 [d53a36ff33]  Richard Mudgett <rmudgett@digium.com>
+	  The patch moves the unprotected access to a location where the mutex is
+	  safely obtained.
 
-	* res_pjsip_transport_management.c: Misc cleanups to survive shutdown.
+	  Review: https://reviewboard.asterisk.org/r/4599/
 
-	  * In unload_module(), reordered destroying things to minimize the window
-	  that the global transports container could be used by other threads on
-	  shutdown.  When shutting down you need to stop things in the opposite
-	  order of creation.
+	  ASTERISK-21211 #close
+	  Reported by: Jaco Kroon
+	  patches:
+	    asterisk-11.2.1-iax2_poke-segfault.diff submitted by Jaco Kroon (License 5671)
+	  ........
 
-	  * Put the global transports container into an AO2_GLOBAL_OBJ_STATIC to
-	  eliminate the crash potential by other threads using the container on
-	  shutdown.
+	  Merged revisions 434291 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * Made struct monitored_transport.sip_received not use
-	  ast_atomic_fetchadd_int() since it is used as a boolean value that is only
-	  set TRUE.  It was previously incremented for every received SIP message
-	  and could theoretically overflow.
+	  Merged revisions 434292 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * In monitored_transport_state_callback(), allocated the monitored
-	  transport object without a lock since the lock was unused.
 
-	  * In keepalive_global_loaded(), removed releasing the transports container
-	  if the keepalive_thread could not be started.  I set it up to be tried
-	  again if the user reloads the configuration.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434293 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I8d12d16ef564290fa6d25a32334bb5ce8fdf87ff
+2015-04-08 06:54 +0000 [477536ef25]  Valentin Vidić (License 6697)
 
-2016-01-05 19:08 +0000 [03953d8034]  Richard Mudgett <rmudgett@digium.com>
+	* chan_sip: Handle IPv4 mapped IPv6 clients when NAT is enabled
 
-	* res_pjsip.c: Add check that timer actually got scheduled.
+	  When udpbindaddr is set to the IPv6 bind all address of '::', Asterisk will
+	  attempt to handle both IPv4 and IPv6 addresses, although the information will
+	  be stored in a struct with an AF_INET6 address type. However, the current
+	  NAT handling code won't handle the IPv4 mapped IPv6 addresses correctly.
+	  This patch adds an additional check for the mapped address case, allowing
+	  the NAT code to handle clients even when the address is IPv6.
 
-	  Change-Id: Iabaa2e5dccf0762c258101ea0eb1487cf6959ad1
+	  Review: https://reviewboard.asterisk.org/r/4563/
 
-2016-06-13 13:33 +0000 [32ab98116e]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-18032 #close
+	  Reported by: Christoph Timm
+	  patches:
+	    nat_with_ipv6.diff submitted by Valentin Vidić (License 6697)
+	  ........
 
-	* res_rtp_multicast.c: Fix warning message typo.
+	  Merged revisions 434288 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ic9928208b9957e09866abe3d9649030942ec52b3
+	  Merged revisions 434289 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-02-11 18:15 +0000 [0429c53368]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_session.c: Reorganize ast_sip_session_terminate().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434290 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I68a2128bcba4830985d2d441e70dfd1ac5bd712b
+2015-04-08 06:45 +0000 [b8fa8aa775]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-06-10 12:35 +0000 [5823f279f3]  Richard Mudgett <rmudgett@digium.com>
+	* clang compiler warnings: Fix pointer-bool-converesion warnings
 
-	* chan_rtp: Backport changes from master.
+	  This patch fixes several warnings pointed out by the clang compiler.
+	  * chan_pjsip: Removed check for data->text, as it will always be non-NULL.
+	  * app_minivm: Fixed evaluation of etemplate->locale, which will always
+	    evaluate to 'true'. This patch changes the evaluation to use
+	    ast_strlen_zero.
+	  * app_queue:
+	    - Fixed evaluation of qe->parent->monfmt, which always evaluates to
+	      true. Instead, we just check to see if the dereferenced pointer
+	      evaluates to true.
+	    - Fixed evaluation of mem->state_interface, wrapping it with a call to
+	      ast_strlen_zero.
+	  * res_smdi: Wrapped search_msg->mesg_desk_term with calls to ast_strlen_zero.
 
-	  * Deprecate chan_multicast_rtp.
+	  Review: https://reviewboard.asterisk.org/r/4541
 
-	  Change-Id: Ib5a45e58c75ee8abd0b4f9575379b5321feb853e
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4541.patch submitted by dkdegroot (License 6600)
+	  ........
 
-2016-06-10 16:13 +0000 [dde58df318]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 434285 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* chan_rtp.c: Copy file from chan_multicast_rtp.c
+	  Merged revisions 434286 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I1119b53f2152ab1cbec74b5be7ea44844dbda8ef
 
-2016-06-08 06:15 +0000 [ca38a3cbb4]  Alexander Traud <pabstraud@compuserve.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434287 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* core: Not the configured but granted number of possible file descriptors.
+2015-04-08 06:35 +0000 [016fba12e2]  Rodrigo Ramirez Norambuena (License 6577)
 
-	  With CLI "core show settings", simply the parameter maxfiles of the file
-	  asterisk.conf was shown. If that parameter was not set, nothing was displayed
-	  although the environment might have set a default number itself. Or if maxfiles
-	  were not granted (completely), still maxfiles was shown. Now, the maximum number
-	  of possible file descriptors in the environment is shown.
+	* cel_pgsl: Add support for GMT timestamps
 
-	  ASTERISK-26097
+	  This patch adds a new option to cel_pgsl, "usegmtime", which causes timestamps
+	  to be logged in GMT.
 
-	  Change-Id: I2df5c58863b5007b34b77adbe28b885dfcdf7e0b
+	  Review: https://reviewboard.asterisk.org/r/4571/
 
-2016-06-07 18:45 +0000 [caf6cccc5c]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-23186 #close
+	  Reported by: Rodrigo Ramirez Norambuena
+	  patches:
+	    cel_pgsql.c_add_usegmtime2.patch submitted by Rodrigo Ramirez Norambuena (License 6577)
 
-	* cel: Ensure only one dial status per channel exists.
 
-	  CEL wrongly assumed that a channel would only have a single dial
-	  event on it. This is incorrect. Particularly in a queue each
-	  call attempt to a member will result in a dial event, adding
-	  a new dial status in CEL without removing the old one. This
-	  would cause the container to grow with only one dial status
-	  being removed when the channel went away. The other dial status
-	  entries would remain leaking memory.
 
-	  This change fixes the memory leak by ensuring that only one dial
-	  status will only ever exist for each channel.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434284 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The behavior during the scenario where multiple events are received
-	  has also been improved. For failure cases the first failure will
-	  be the dial status. If an answer dial status is received, though,
-	  it will take priority and the dial status for the channel will be
-	  answer.
+2015-04-07 14:40 +0000 [d923ec80b9]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  Memory usage has also been decreased by storing the minimal
-	  amount of information and the code has been cleaned up slightly.
+	* pjsip: resolve compatibility problem with ast_sip_session
 
-	  ASTERISK-25262 #close
+	  A change in r430179 inserted a variable near the top of a
+	  structure caused a problem when running DPMA in a version
+	  of Asterisk compiled across the change.  This patch moves
+	  the new variable to the end of the structure, eliminating
+	  the problem.
 
-	  Change-Id: I5944eb923db17b6a0faa7317ff6abc9307c009fe
+	  Review: https://reviewboard.asterisk.org/r/4574/
+	  ........
 
-2016-06-09 10:37 +0000 [715ef071a1]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 433944 from http://svn.asterisk.org/svn/asterisk/branches/13
+	  ........
 
-	* chan_pjsip: Lock channel when checking for RTP changes.
+	  Merged revisions 434261 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  bridge_native_rtp can call into an RTP-capable channel driver in order
-	  for the driver to update information about who the channel is
-	  communicating with. For SIP channel drivers, this means deactivating
-	  RTCP and sending a reinvite so that the endpoints can communicate
-	  directly.
 
-	  bridge_native_rtp does the right thing and has the channel locked when
-	  calling into the channel driver. chan_pjsip can't alter session
-	  properties in this thread, though. chan_pjsip queues a task on the
-	  session serializer in order to update properties there.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434263 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The problem is that this queued task was not locking the channel. This
-	  meant that the queued task could attempt to deactivate RTCP at the same
-	  time that the channel thread was attempting to process an incoming RTCP
-	  packet. This could lead to a crash.
+2015-04-07 11:42 +0000 [153c4044e4]  Kevin Harwell <kharwell@digium.com>
 
-	  This patch fixes the issue by locking the channel in the queued task
-	  when altering RTP properties.
+	* bridge.c: Hangup attended transfer target after it has been swapped out
 
-	  ASTERISK-26092 #close
-	  Reported by Niklas Larsson
+	  After completing an attended transfer the transfer target channel (the one that
+	  gets swapped out) was not being hung up after leaving the bridge. This resulted
+	  in a channel possibly being left around. Added an explicit softhangup for the
+	  channel in question after the transfer is successfully completed in order to
+	  make sure the channel is hung up.
 
-	  Change-Id: I3464e226a3c41f6b915f97891e07fa1599e2a159
+	  ASTERISK-24782 #close
+	  Reported by: John Bigelow
+	  Review: https://reviewboard.asterisk.org/r/4575/
+	  ........
 
-2016-06-09 09:20 +0000 [a99ddc6a0d]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 434240 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* build:  Fix ast_sockaddr initialization to be more portable
 
-	  A change to glibc 2.22 changed the order of the sockadddr_storage
-	  members which caused the places where we do an initialization of
-	  ast_sockaddr with '{ { 0, 0, } }' to fail compilation.  Those
-	  initializers (which we shouldn't have been using anyway) have been
-	  replaced with memsets.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434241 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Idd1b3b320903d8771bfe221f0b015685de628fa4
+2015-04-07 10:34 +0000 [1eba6abae5]  Mark Michelson <mmichelson@digium.com>
 
-2016-06-08 12:26 +0000 [eabb398d71]  Matt Jordan <mjordan@digium.com>
+	* Do not queue message requests that we do not respond to.
 
-	* res_hep_{pjsip|rtcp}: Decline module loads if res_hep had not loaded
+	  If we receive a MESSAGE request that we cannot send a response
+	  to, we should not send the incoming MESSAGE to the dialplan.
 
-	  A crash can occur in res_hep_pjsip or res_hep_rtcp if res_hep has not
-	  loaded and does not have a configuration file. Previously when this
-	  occurred, checks were put in to see if the configuration was loaded
-	  successfully. While this is a good idea - and has been added to the
-	  offending function in res_hep - the reality is res_hep_pjsip and
-	  res_hep_rtcp have no business running if res_hep isn't also running.
+	  This commit should help the bouncing message_retrans test to
+	  pass consistently.
+	  ........
 
-	  As such, this patch also adds a function to res_hep that returns whether
-	  or not it successfully loaded. Oddly enough, ast_module_check returns
-	  "everything is peachy" even if a module declined its load - so it cannot
-	  be solely relied on. res_hep_pjsip and res_hep_rtcp now also check this
-	  function to see if they should continue to load; if it fails, they
-	  decline their load as well.
+	  Merged revisions 434218 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26096 #close
 
-	  Change-Id: I007e535fcc2e51c2ca48534f48c5fc2ac38935ea
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434219 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-08 05:58 +0000 [0d84421f93]  Alexander Traud <pabstraud@compuserve.com>
+2015-04-07 10:22 +0000 [c2f50ba6f4]  Matt Jordan <mjordan@digium.com>
 
-	* astfd: Not maximum size of a single file but maximum file descriptors.
+	* ARI: Add the ability to intercept hold and raise an event
 
-	  With menuselect "DEBUG_FD_LEAKS" and CLI "core show fd", the maximum size of a
-	  single file was shown. Now, the maximum number of possible file descriptors is
-	  shown.
+	  For some applications - such as SLA - a phone pressing hold should not behave
+	  in the fashion that the Asterisk core would like it to. Instead, the hold
+	  action has some application specific behaviour associated with it - such as
+	  disconnecting the channel that initiated the hold; only playing MoH to channels
+	  in the bridge if the channels are of a particular type, etc.
 
-	  ASTERISK-26097
+	  One way of accomplishing this is to use a framehook to intercept the
+	  hold/unhold frames, raise an event, and eat the frame. Tasty. This patch
+	  accomplishes that using a new dialplan function, HOLD_INTERCEPT.
 
-	  Change-Id: Icf98d145774b38cac144ca76d19eaef42ce659a3
+	  In addition, some general cleanup of raising hold/unhold Stasis messages was
+	  done, including removing some RAII_VAR usage.
 
-2016-06-02 14:53 +0000 [9c5a0b814b]  Timo Teräs <timo.teras@iki.fi>
+	  Review: https://reviewboard.asterisk.org/r/4549/
 
-	* Fix #include poll.h and sys/cdefs.h
+	  ASTERISK-24922 #close
+	  ........
 
-	  POSIX defines poll.h, sys/poll.h should not be used at is c-library
-	  internal header which may or may not exist. Notable in musl it
-	  generates warning of being incorrect. And add explict include of
-	  sys/cdefs.h where needed.
+	  Merged revisions 434216 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I142930df53fe7585a06b854b6faddc5301e024be
 
-2016-06-03 22:44 +0000 [9c35f34301]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434217 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_registrar.c: Eliminate rx REGISTER request race condition.
+2015-04-06 21:10 +0000 [af4d802773]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  This patch fixes a race condition processing received REGISTER requests
-	  and their retransmissions caused by REGISTER requests being processed by
-	  two threads.  The "sip_transaction Unable to register REGISTER transaction
-	  (key exists)" message is a notable symptom of this issue.
+	* clang compiler warnings: Fix sometimes-initialized warning in func_math
 
-	  This issue was more likely to happen before the pjsip/distributor
-	  serializers were created.  Instead of steps one and two below placing the
-	  REGISTER messages into the same pjsip/distributor they were placed in
-	  random pjsip/default serializers.
+	  This patch fixes a bug in a unit test in func_math where a variable could be
+	  passed to ast_free that wasn't allocated. This patch corrects the issue and
+	  ensures that we only attempt to free a variable if we previously allocated
+	  it.
 
-	  1) REGISTER requests come in and get placed on the pjsip/distributor
-	  serializer.
+	  Review: https://reviewboard.asterisk.org/r/4552
 
-	  2) Before the first request is processed a retransmission comes in and is
-	  placed on the same pjsip/distributor serializer.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4552.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  3) The first request goes up the pjsip stack and is then shunted off to
-	  the pjsip/aor/<aor> serializer.
+	  Merged revisions 434190 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  4) Before the first request is completed processing in the pjsip/aor/<aor>
-	  serializer, the second request goes up the pjsip stack and is also shunted
-	  off to the pjsip/aor/<aor> serializer.
+	  Merged revisions 434191 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  5) The first request completes processing and sends out its response.
 
-	  6) The second request completes processing and tries to send out its
-	  response but pjlib complains that the REGISTER transaction key already
-	  exists.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434192 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  7) Sadness ensues.
+2015-04-06 21:03 +0000 [c1cfe3fae2]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  * The race is eliminated by removing the pjsip/aor/<aor> serializer and
-	  continuing the processing in the pjsip/distributor serializer.  Now any
-	  retransmissions queued in the pjsip/distributor serializer will be
-	  processed after the first message is completely processed.
+	* clang compiler warnings: Fix non-literal-null-conversion warnings
 
-	  ASTERISK-26088 #close
-	  Reported by:  Richard Mudgett
+	  Clang will flag errors when a char pointer is set to '\0', as opposed to a
+	  value that the char pointer points to. This patch fixes this warning
+	  in a variety of locations.
 
-	  Change-Id: I842d714346088bf717ea27437f1dd85bff0bab5a
+	  Review: https://reviewboard.asterisk.org/r/4551
 
-2016-06-03 11:35 +0000 [557333ea4c]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4551.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	* stasis: Add setting subscription congestion levels.
+	  Merged revisions 434187 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Stasis subscriptions and message routers create taskprocessors to process
-	  the event messages.  API calls are needed to be able to set the congestion
-	  levels of these taskprocessors for selected subscriptions and message
-	  routers.
+	  Merged revisions 434188 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Updated CDR, CEL, and manager's stasis subscription congestion levels
-	  based upon stress testing.  Increased the congestion levels to reduce the
-	  potential for bursty call setup/teardown activity from triggering the
-	  taskprocessor overload alert.  CDRs in particular need an extra high
-	  congestion level because they can take awhile to process the stasis
-	  messages.
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434189 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id0a716394b4eee746dd158acc63d703902450244
+2015-04-06 16:54 +0000 [79fb8c32a6]  Mark Michelson <mmichelson@digium.com>
 
-2016-06-02 18:19 +0000 [110d772467]  Richard Mudgett <rmudgett@digium.com>
+	* Uncomment test case.
 
-	* sorcery: Add setting object type congestion levels.
 
-	  Sorcery creates taskprocessors for object types to process object observer
-	  callbacks.  An API call is needed to be able to set the congestion levels
-	  of these taskprocessors for selected object types.
 
-	  * Updated PJSIP's contact and contact_status sorcery object type observer
-	  default congestion levels based upon stress testing.  Increased the
-	  congestion levels to reduce the potential for bursty register/unregister
-	  and subscribe/unsubscribe activity from triggering the taskprocessor
-	  overload alert.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434170 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+2015-04-06 16:13 +0000 [fc314cb43f]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I4542e83b556f0714009bfeff89505c801f1218c6
+	* Add missing DNS NAPTR test file.
 
-2016-06-02 16:08 +0000 [610eee2a36]  Richard Mudgett <rmudgett@digium.com>
 
-	* taskprocessors: Implement high/low water mark alerts.
 
-	  When taskprocessors get backed up, there is a good chance that we are
-	  being overloaded and need to defer adding new work to the system.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434154 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Implemented a high/low water alert mechanism for modules to check if the
-	  system is being overloaded and take appropriate action.  When a
-	  taskprocessor is created it has default congestion levels set.  A
-	  taskprocessor can later have those congestion levels altered for specific
-	  needs if stress testing shows that the taskprocessor is a symptom of
-	  overloading or needs to handle bursty activity without triggering an
-	  overload alert.
+2015-04-06 14:23 +0000 [87d7c90e4e]  Kevin Harwell <kharwell@digium.com>
 
-	  * Add CLI "core show taskprocessor" low/high water columns.
+	* res_pjsip: config option 'timers' can't be set to 'no'
 
-	  * Fixed __allocate_taskprocessor() to not use RAII_VAR().  RAII_VAR() was
-	  never a good thing to use when creating a taskprocessor because of the
-	  nature of how its references needed to be cleaned up on a partial
-	  creation.
+	  When setting the configuration option 'timers' equal to 'no' the bit flag was
+	  not properly negated. This patch clears all associated flags and only sets the
+	  specified one. pjsip will handle any necessary flag combinations. Also went
+	  ahead and did similar for the '100rel' option.
 
-	  * Made res_pjsip's distributor check if the taskprocessor overload alert
-	  is active before placing a message representing brand new work onto a
-	  distributor serializer.
+	  ASTERISK-24910 #close
+	  Reported by: Ray Crumrine
+	  Review: https://reviewboard.asterisk.org/r/4582/
+	  ........
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  Merged revisions 434131 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I182f1be603529cd665958661c4c05ff9901825fa
 
-2016-05-27 17:31 +0000 [26e3492246]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434132 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_session: Use distributor serializer for incoming calls.
+2015-04-06 14:04 +0000 [e48f2e7897]  gtjoseph <george.joseph@fairview5.com>
 
-	  We must continue using the serializer that the original INVITE came in on
-	  for the dialog.  There may be retransmissions already enqueued in the
-	  original serializer that can result in reentrancy and message sequencing
-	  problems.
+	* build: Fixes for gcc 5 compilation
 
-	  Outgoing call legs create the pjsip/outsess/<endpoint> serializers for
-	  their dialogs.
+	  These are fixes for compilation under gcc 5.0...
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  chan_sip.c:    In parse_request needed to make 'lim' unsigned.
+	  inline_api.h:  Needed to add a check for '__GNUC_STDC_INLINE__' to detect C99 
+	                 inline semantics (same as clang).
+	  ccss.c:        In ast_cc_set_parm, needed to fix weird comparison.
+	  dsp.c:         Needed to work around a possible compiler bug.  It was throwing 
+	                 an array-bounds error but neither
+	                 sgriepentrog, rmudgett nor I could figure out why.
+	  manager.c:     In action_atxfer, needed to correct an array allocation.
 
-	  Change-Id: I24d7948749c582b8045d5389ba3f6588508adbbc
+	  This patch will go to 11, 13, trunk.
 
-2016-05-27 16:28 +0000 [ceb1007ed7]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4581/
+	  Reported-by: Jeffrey Ollie
+	  Tested-by: George Joseph
+	  ASTERISK-24932 #close
+	  ........
 
-	* res_pjsip_pubsub.c: Recreate subscriptions using distributor serializer.
+	  Merged revisions 434113 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * Resolves potential reentrancy problems if system restarted in the middle
-	  of subscription message transactions.
+	  Merged revisions 434114 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Fixes memory leak recreating persistent subscriptions when the
-	  subscription resource tree could not be created.
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434115 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I71e34d7ae8ed35a694f1030e820e2548c48697be
+2015-04-06 13:18 +0000 [0543879228]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-05-27 12:50 +0000 [27bafc3a8b]  Richard Mudgett <rmudgett@digium.com>
+	* clang compiler warnings: Remove large chunks of unused code from extconf
 
-	* res_pjsip_pubsub.c: Use distributor serializer for incoming subscriptions.
+	  This patch fixes a warning caught by clang, in which it detected that large
+	  chunks of extconf were unused. Frankly, I wish we could pretend that all of
+	  extconf was unused, but alas, that is not yet the case.
 
-	  We must continue using the serializer that the original SUBSCRIBE came in
-	  on for the dialog.  There may be retransmissions already enqueued in the
-	  original serializer that can result in reentrancy and message sequencing
-	  problems.  The "sip_transaction Unable to register SUBSCRIBE transaction
-	  (key exists)" message is a notable symptom of this issue.
+	  A few extraneous functions in the parking tests were removed as well, for
+	  the same reason.
 
-	  Outgoing subscriptions still create the pjsip/pubsub/<endpoint>
-	  serializers for their dialogs.
+	  Review: https://reviewboard.asterisk.org/r/4553
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4553.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  Change-Id: I18b00bb74a56747b2c8c29543a82440b110bf0b0
+	  Merged revisions 434093 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-26 17:35 +0000 [16b08444da]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 434097 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* pjsip_distributor.c: Consistently pick a serializer for messages.
 
-	  Incoming messages that are not part of a dialog or a recognized response
-	  to one of our requests need to be sent to a consistent serializer.  Under
-	  load we may be queueing retransmissions before we can process the original
-	  message.  We don't need to throw these messages onto random serializers
-	  and cause reentrancy and message sequencing problems.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434099 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Created a pool of pjsip/distributor serializers that get picked by
-	  hashing the call-id and remote tag strings of the received messages.
+2015-04-06 13:03 +0000 [e309a91e2d]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  * Made ast_sip_destroy_distributor() destroy items in the reverse order of
-	  creation.
+	* clang compiler warnings: Fix sometimes-uninitialized warning in pbx_config
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  This patch fixes a warning caught by clang, in which a char pointer could be
+	  assigned to before it was initialized. The patch re-organizes the code to
+	  ensure that the pointer is always initialized, even on off nominal paths.
 
-	  Change-Id: I2ce769389fc060d9f379977f559026fbcb632407
+	  Review: https://reviewboard.asterisk.org/r/4529
 
-2016-06-02 12:51 +0000 [993b769524]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4529.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	* pjsip_distributor.c: Ignore messages until fully booted.
+	  Merged revisions 434090 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  We should not be processing any incoming messages until we are fully
-	  booted.  We may not have dialplan or other needed configuration loaded
-	  yet.
+	  Merged revisions 434091 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26089 #close
-	  Reported by: Scott Griepentrog
 
-	  ASTERISK-26088
-	  Reported by:  Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434092 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I584aefb4f34b885a8927e1f13a2c64babd606264
+2015-04-06 12:52 +0000 [ed3cf8761b]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-06-02 12:04 +0000 [321a9b128f]  Joshua Colp <jcolp@digium.com>
+	* clang compiler warnings: Fix format specified in framehook
 
-	* res_odbc: Implement a connection pool.
+	  This patch fixes an invalid format specifier used in the formatting of an
+	  ERROR message in the framehook code. The format specifier specifies a
+	  type of 'unsigned short', but the argument passed to it is of type 'int'.
+	  The patch changes the format specifier to 'i'.
 
-	  Testing has shown that our usage of UnixODBC is problematic
-	  due to bugs within UnixODBC itself as well as the heavy weight
-	  cost of connecting and disconnecting database connections, even
-	  when pooling is enabled.
+	  Review: https://reviewboard.asterisk.org/r/4540
 
-	  For users of UnixODBC 2.3.1 and earlier crashes would occur due
-	  to insufficient protection of the disconnect operation. This was
-	  fixed in UnixODBC 2.3.2 and above.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4535.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  For users of UnixODBC 2.3.3 and higher a slow-down would occur
-	  under heavy database use due to repeated connection establishment.
-	  A regression is present where on each connection the database
-	  configuration is cached again, with the cache growing out of
-	  control.
+	  Merged revisions 434087 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The connection pool implementation present in this change helps
-	  to mitigate these issues by reducing how much we connect and
-	  disconnect database connections. We also solve the issue of
-	  crashes under UnixODBC 2.3.1 by defaulting the maximum number of
-	  connections to 1, returning us to the previous working behavior.
-	  For users who may have a fixed version the maximum concurrent
-	  connection limit can be increased helping with performance.
+	  Merged revisions 434088 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The connection pool works by keeping a list of active connections.
-	  If the connection limit has not been reached a new connection is
-	  established. If the connection limit has been reached then the
-	  request waits until a connection becomes available before
-	  continuing.
 
-	  ASTERISK-26074 #close
-	  ASTERISK-26054 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434089 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I6774bf4bac49a0b30242c76a09c403d2e856ecff
+2015-04-06 12:05 +0000 [0a26602b8c]  Mark Michelson <mmichelson@digium.com>
 
-2016-06-07 05:45 +0000 [c6ee4a0f44]  Alexander Traud <pabstraud@compuserve.com>
+	* Merge NAPTR support into trunk.
 
-	* res_srtp: Instead of libSRTP use OpenSSL as random source.
+	  This adds NAPTR record allocation and sorting, as well as
+	  unit tests that verify that NAPTR records are parsed and
+	  sorted correctly.
 
-	  Since libSRTP 1.5, its Random Number Generator (RNG) is not maintained anymore.
-	  Therefore, the symbol RAND_bytes is used instead of crypto_get_random.
+	  Review: https://reviewboard.asterisk.org/r/4542
 
-	  ASTERISK-24436 #close
 
-	  Change-Id: Iea0bae4d4e3c9aa0926ea442b6484b5159789d96
 
-2016-06-07 02:16 +0000 [d38b8e6399]  Alexander Traud <pabstraud@compuserve.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434068 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* BuildSystem: Avoid 'ar cru' and use 'ar cr' instead.
+2015-04-06 11:02 +0000 [edf9da4365]  Mark Michelson <mmichelson@digium.com>
 
-	  In several internal library projects, the files are archived with the help of
-	  'ar cr'. Only the projects editline and the Objective Open H.323 stack
-	  implementation in C (ooh323c) use 'ar cru' instead. Recently, some platforms
-	  changed the default parameters of AR which creates "/usr/bin/ar: `u' modifier
-	  ignored since `D' is the default (see `U')". For consistency and to avoid this
-	  message all projects use 'ar cr' now.
+	* Ensure that a non-zero sample rate is returned for all formats.
 
-	  ASTERISK-26091 #close
+	  Versions of Asterisk prior to 12 defaulted to 8000 as a sample rate
+	  if one was not provided by a format. In Asterisk 13, this was removed.
+	  The result was that some calculations which involve dividing by the
+	  sample rate resulted in dividing by 0. The fix being put in place
+	  here is to have the same default fallback that was present in previous
+	  versions of Asterisk.
 
-	  Change-Id: I710a9b1c01c1b5a1931a646098c044c8161ead40
+	  Asterisk-24914 #close
+	  Reported by Marcello Ceschia
+	  ........
 
-2016-05-27 14:49 +0000 [c27c232057]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 434046 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* ari/resource_channels:  Add 'formats' to channel create/originate
 
-	  If you create a local channel and don't specify an originator channel
-	  to take capabilities from, we automatically add all audio formats to
-	  the new channel's capabilities. When we try to make the channel
-	  compatible with another, the "best format" functions pick the best
-	  format available, which in this case will be slin192.  While this is
-	  great for preserving quality, it's the worst for performance and
-	  overkill for the vast majority of applications.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434047 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In the absense of any other information, adding all formats is the
-	  correct thing to do and it's not always possible to supply an
-	  originator so a new parameter 'formats' has been added to the channel
-	  create/originate functions. It's just a comma separated list of formats
-	  to make availalble for the channel. Example: "ulaw,slin,slin16".
-	  'formats' and 'originator' are mutually exclusive.
+2015-04-06 10:17 +0000 [ffd7319df3]  Corey Farrell <git@cfware.com>
+
+	* res_pjsip_phoneprov_provider: Revert 433996 / 433997.
+
+	  res_pjsip_phoneprov_provider is using ao2_callback with OBJ_MULTIPLE, then
+	  ignoring the return.  OBJ_NODATA flag was to prevent a reference leak, but
+	  this caused the module to FRACK on unload.  Revert change until this can
+	  be investigated further.
+
+	  ASTERISK-24935
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4578/
+	  ........
 
-	  To facilitate determination of format names, the format name has been
-	  added to "core show codecs".
+	  Merged revisions 434025 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26070 #close
 
-	  Change-Id: I091b23ecd41c1b4128d85028209772ee139f604b
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434026 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-06-02 04:59 +0000 [cda3385409]  Joshua Colp <jcolp@digium.com>
+2015-04-06 09:51 +0000 [53af579d4c]  Mark Michelson (license #5049)
 
-	* alembic: Fix migration.
+	* ParkedCall: Don't allow dialplan fallthrough after retrieving parked call.
 
-	  The 81b01a191a46_pjsip_add_contact_reg_server.py script was attempting
-	  to use UniqueConstraint and failing. It was not imported and after
-	  importing it also continued to fail.
+	  This is a change to align behavior with that of Asterisk 11 and previous versions.
+	  In those versions, if a parked call were retrieved, and the call ended, the parked
+	  call retriever would be hung up after the ParkedCall application ran. Prior to this
+	  patch, in Asterisk 13, the same situation would result in the parked call retriever
+	  falling through to additional priorities in the extension where the ParkedCall
+	  application was called. With this patch, the behavior between Asterisk 11 and 13
+	  aligns.
 
-	  I've changed the script to use the explicit name of the constraint
-	  instead.
+	  ASTERISK-24899 #close
+	  Reported by Malcolm Davenport
+	  Patches:
+	  	ASTERISK-24899.patch uploaded by Mark Michelson(license #5049)
+	  ........
 
-	  Change-Id: I2438b0be90b7ce583b47dd27983c0c1a02cea5b9
+	  Merged revisions 434022 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-06-01 13:57 +0000 [e2132dd358]  Richard Mudgett <rmudgett@digium.com>
 
-	* logging,cdr,cel: Fix stringfield memory leak.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@434023 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The stringfields refactor to allow adding stringfields to the end of a
-	  structure (f6f4cf459f43f072604927209b39646f84aaa2e2) exposed some
-	  incomplete cleanup code by some stringfield users.
+2015-04-05 07:55 +0000 [e6f0410028]  Corey Farrell <git@cfware.com>
 
-	  The most noticeable leaker is the logging system where there is a leak for
-	  every log message generated.
+	* res_pjsip_phoneprov_provider: Fix leaked OBJ_MULTIPLE iterator.
 
-	  ASTERISK-26078 #close
-	  Reported by:  Etienne Lessard
-	  Patches:
-	        jira_asterisk_26078_v13.patch (license #5621) patch uploaded
-	        by Richard Mudgett
+	  res_pjsip_phoneprov_provider was using ao2_callback with OBJ_MULTIPLE, then
+	  ignoring the return.  Added OBJ_NODATA flag to prevent a reference leak.
 
-	  Change-Id: If6a08b31336b492c3de6f9dfd07c447f8d5a8782
+	  ASTERISK-24935 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4578/
+	  ........
 
-2016-05-25 10:34 +0000 [2de58c6d01]  Alexei Gradinari <alex2grad@gmail.com>
+	  Merged revisions 433996 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* core/dial: New channel variable FORWARDERNAME
 
-	  Added a new channel variable FORWARDERNAME which indicates which
-	  channel was responsible for a forwarding requests received on dial attempt.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433997 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Fixed a bug in the app_queue: FORWARD_CONTEXT is not used.
+2015-04-03 16:54 +0000 [3439487a81]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-26059 #close
+	* res_pjsip_messaging: Serialize outbound SIP MESSAGEs
 
-	  Change-Id: I34e93e8c1b5e17776a77b319703c48c8ca48e7b2
+	  Outbound SIP MESSAGEs had the potential to be sent out
+	  of order from how they were specified in a set of
+	  dialplan steps.
 
-2016-05-31 13:02 +0000 [b2ce0e354b]  Richard Mudgett <rmudgett@digium.com>
+	  This change creates a serializer for sending outbound
+	  MESSAGE requests on. This ensures that the MESSAGEs are
+	  sent by Asterisk in the same order that they were sent
+	  from the dialplan.
 
-	* pjsip_distributor.c: Use correct rdata info access method (Part 2).
+	  ASTERISK-24937 #close
+	  Reported by Mark Michelson
 
-	  The pjproject doxygen for rdata->msg_info.info says to call
-	  pjsip_rx_data_get_info() instead of accessing the struct member directly.
-	  You need to call the function mostly because the function will generate
-	  the struct member value if it is not already setup.
+	  Review: https://reviewboard.asterisk.org/r/4579
+	  ........
 
-	  Change-Id: I4d519385a577f3e9d9193a88125e493cf17fa799
+	  Merged revisions 433968 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-30 19:27 +0000 [fe305ccf01]  gtjoseph <gjoseph@digium.com>
 
-	* res_pjsip_mwi_body_generator:  Re-order the body items
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433969 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Re-ordered the body items so Message-Account is second.
+2015-04-02 09:56 +0000 [6e5efe04bd]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  Messages-Waiting: no
-	  Message-Account: sip:1571@<IP Removed>:5060
-	  Voice-Message: 0/0 (0/0)
+	* pjsip: resolve compatibility problem with ast_sip_session
 
-	  ASTERISK-26065 #close
-	  Reported-by: Ross Beer
+	  A change in r430179 inserted a variable near the top of a
+	  structure caused a problem when running DPMA in a version
+	  of Asterisk compiled across the change.  This patch moves
+	  the new variable to the end of the structure, eliminating
+	  the problem.
 
-	  Change-Id: If5d35a64656eac98c2dd5e490cc0b2807bed80c3
+	  Review: https://reviewboard.asterisk.org/r/4574/
+	  ........
 
-2016-05-30 10:58 +0000 [e8abfdcdc5]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 433944 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* pjproject_bundled:  Move to pjproject 2.5
 
-	  Although all the patches we had against 2.4.5 were applied by Teluu,
-	  a new bug was introduced preventing re-use of tcp and tls transports
-	  This patch removes all the previous patches against 2.4.5, updates
-	  the version to 2.5, and adds a new patch to correct the transport
-	  re-use problem.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433945 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I0dc6c438c3910f7887418a5832ca186aea23d068
+2015-04-02 05:38 +0000 [154ba47766]  Corey Farrell <git@cfware.com>
 
-2016-05-27 12:25 +0000 [37d039fdf3]  Rusty Newton <rnewton@digium.com>
+	* Tell menuselect that MALLOC_DEBUG conflicts with DEBUG_CHAOS.
 
-	* res_pjsip: Add clarifying documentation to PJSIP_HEADER help text
+	  DEBUG_CHAOS was marked as conflicting with MALLOC_DEBUG, but
+	  for this to work correctly MALLOC_DEBUG must also be marked
+	  as conflicting with DEBUG_CHAOS.
 
-	  Added notes about when you can read or write headers. Specifically
-	  about being able to read on the inbound channel and write on an
-	  outbound channel.
+	  Review: https://reviewboard.asterisk.org/r/4557/
+	  ........
 
-	  ASTERISK-26063 #close
-	  Reported by: Private Name
-	  Tested by: Rusty Newton
+	  Merged revisions 433923 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ibeb64af17d1f6451028b3c29855a3f151a01d8c5
 
-2016-05-25 18:30 +0000 [03d5b3ce5c]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433924 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip_distributor.c: Use correct rdata info access method.
+2015-04-01 11:30 +0000 [a217d2d1db]  Ashley Sanders <asanders@digium.com>
 
-	  The pjproject doxygen for rdata->msg_info.info says to call
-	  pjsip_rx_data_get_info() instead of accessing the struct member directly.
-	  You need to call the function mostly because the function will generate
-	  the struct member value if it is not already setup.
+	* stasis: set a channel variable on websocket disconnect error
 
-	  Change-Id: Iafe8b01242b7deb0ebfdc36685e21374a43936d2
+	  Resolve compile errors caused by r433863 by fixing the
+	  documentation xml to comply with the schema.
+	  ........
 
-2016-05-20 13:56 +0000 [859bbec09b]  Alexei Gradinari <alex2grad@gmail.com>
+	  Merged revisions 433888 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* app_voicemail: fix bugs, imap mm_status log change to debug
 
-	  Fixed some bugs:
-	  - create dirpath when save downloading message from IMAP storage.
-	  - create IMAP folder if not exists when saving to IMAP storage
-	  - check if file successfully opened before write to it
-	  - some IMAP checks
-	  - remove non-standard flag 'Unseen'
-	  etc
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433891 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change to debug IMAP mm_status log instead of verbose.
+2015-04-01 11:27 +0000 [39824e3d01]  Joshua Colp <jcolp@digium.com>
 
-	  Remove unused X-Asterisk-VM-Caller-channel message header
-	  for security reason. The clients should not know name of peer/endpoint.
+	* dns: Add support for SRV record parsing and sorting.
 
-	  ASTERISK-26045 #close
+	  This change adds support for parsing SRV records and consuming their values
+	  in an easy fashion. It also adds automatic sorting of SRV records according
+	  to RFC 2782.
 
-	  Change-Id: I7f83d88b69b36934e2539c114b9fb612deed971b
+	  Tests have also been included which cover parsing, sorting, and off-nominal
+	  cases where the record is corrupted.
 
-2016-05-19 14:56 +0000 [230686f4ec]  Alexei Gradinari <alex2grad@gmail.com>
+	  ASTERISK-24931 #close
+	  Reported by: Joshua Colp
 
-	* res_pjsip: add "via_addr", "via_port", "call_id" to contact
+	  Review: https://reviewboard.asterisk.org/r/4528/
 
-	  As res_pjsip_nat rewrites contact's address, only the last Via header
-	  can contain the source address of registered endpoint.
-	  Also Call-Id header may contain the source address of registered
-	  endpoint.
 
-	  Added "via_addr", "via_port", "call_id" to contact.
-	  Added new fields ViaAddress, CallID to AMI event ContactStatus.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433889 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-26011
+2015-04-01 08:35 +0000 [da13d15425]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I36bcc0bf422b3e0623680152d80486aeafe4c576
+	* stasis: set a channel variable on websocket disconnect error
 
-2016-05-24 16:56 +0000 [04c12561a7]  Alexei Gradinari <alex2grad@gmail.com>
+	  Resolve compile errors caused by r433839 by included the missing
+	  header file, pbx.h.
+	  ........
 
-	* res_pjsip: chatty verbose messages
+	  Merged revisions 433863 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  There are a lot of verbose messages about Endpoint and Contact status
-	  changes if there are many dynamic endpoints.
-	  The patch sets verbose level 2 for Endpoint status changes
-	  and verbose level 3 for Contact status changes.
 
-	  ASTERISK-26055 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433868 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ie64e261ddbbc41bfff0f0190241152cc123fe6d7
+2015-03-31 17:49 +0000 [06578ef407]  Ashley Sanders <asanders@digium.com>
 
-2016-02-12 09:59 +0000 [a42bea3314]  Corey Farrell <git@cfware.com>
+	* stasis: set a channel variable on websocket disconnect error
 
-	* threadpool: Fix potential data race.
+	  When an error occurs while writing to a web socket, the web socket is
+	  disconnected and the event is logged. A side-effect of this, however, is that
+	  any application on the other side waiting for a response from Stasis is left
+	  hanging indefinitely (as there is no mechanism presently available for
+	  notifying interested parties about web socket error states in Stasis).
 
-	  worker_start checked for ZOMBIE status without holding a lock.  All
-	  other read/write of worker status are performed with a lock, so this
-	  check should do the same.
+	  To remedy this scenario, this patch introduces a new channel variable:
+	  STASISSTATUS.
 
-	  ASTERISK-25777 #close
+	  The possible values for STASISSTATUS are:
+	  SUCCESS         - The channel has exited Stasis without any failures
+	  FAILED          - Something caused Stasis to croak. Some (not all) possible
+	                    reasons for this:
+	                      - The app registry is not instantiated;
+	                      - The app requested is not registered;
+	                      - The app requested is not active;
+	                      - Stasis couldn't send a start message
 
-	  Change-Id: I5e33685a5c26fdb300851989a3b82be8c4e03781
+	  ASTERISK-24802
+	  Reported By: Kevin Harwell
+	  Review: https://reviewboard.asterisk.org/r/4519/
+	  ........
 
-2016-05-18 10:58 +0000 [a32616d60c]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Merged revisions 433839 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Makefile: remove OSARCH check for init install
 
-	  There are more specific checks for the platform.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433845 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Specifically this allows installing OS/X init scripts.
+2015-03-31 12:04 +0000 [2d28fa678e]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-26038 #close
+	* chan_sip: Fix expression in unit test /channels/chan_sip/test_sip_rtpqos.
 
-	  Change-Id: If08933621145b10362a0cfe73c079301d9c13f50
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Fix misplaced parentheses in original fabs() expression.
+	  ........
 
-2016-05-21 05:42 +0000 [9ddaab789e]  Jesper (License 5518)
+	  Merged revisions 433816 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* func_curl: Don't trim response text on non-ASCII characters
+	  Merged revisions 433817 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The characters 0x80-0xFF were trimmed as well as 0x00-0x20 because of
-	  a signed comparison.
 
-	  ASTERISK-25669 #close
-	  Reported by: Jesper
-	  patches:
-	    strings.curl.trim.patch submitted by Jesper (License 5518)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433818 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ia51e169f24e3252a7ebbaab3728630138ec6f60a
+2015-03-31 06:55 +0000 [076fc12afb]  Corey Farrell <git@cfware.com>
 
-2016-05-20 16:59 +0000 [9453d1187a]  Richard Mudgett <rmudgett@digium.com>
+	* Blocked revisions 433795
 
-	* parking.h: Update ast_parking_park_call() doxygen to reality.
+	  ........
+	  Re-add _ast_mem_backtrace_buffer variable for ABI compatibility.
 
-	  ASTERISK-26029
+	  Modules built prior to commit of r4502 expect to link at runtime
+	  to the variable _ast_mem_backtrace_buffer.  This change re-adds
+	  the variable to the C file only.
 
-	  Change-Id: I2db14d102a48d3224010e6d1c69e856373cc1260
+	  Review: https://reviewboard.asterisk.org/r/4558/
 
-2016-05-10 14:30 +0000 [cd89501d48]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* func_odbc: single database connection should be optional
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433796 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  func_odbc was changed in Asterisk 13.9.0
-	  to make func_odbc use a single database connection per DSN
-	  because of reported bug ASTERISK-25938
-	  with MySQL/MariaDB LAST_INSERT_ID().
+2015-03-30 06:43 +0000 [8d12288d8a]  Corey Farrell <git@cfware.com>
 
-	  This is drawback in performance when func_odbc is used
-	  very often in dialplan.
+	* Fix an ABI compatibility issue with ast_log_safe for modules.
 
-	  Single database connection should be optional.
+	  Binary modules are sometimes built against the latest release of
+	  Asterisk in each branch, and need to be compatible with all
+	  releases of that branch.  This change ensures that utils.h only
+	  uses ast_log_safe from the core.  For modules and utilities ast_log
+	  is used instead.
 
-	  ASTERISK-26010
+	  Review: https://reviewboard.asterisk.org/r/4548/
+	  ........
 
-	  Change-Id: I57d990616c957dabf7597dea5d5c3148f459dfb6
+	  Merged revisions 433772 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-20 09:39 +0000 [c0b190dd9a]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 433773 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip: Match dialogs on responses better.
 
-	  When receiving an incoming response to a dialog-starting INVITE, we were
-	  not matching the response to the INVITE dialog. Since we had not
-	  recorded the to-tag to the dialog structure, the PJSIP-provided method
-	  to find the dialog did not match.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433774 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Most of the time, this was not a problem, because there is a fall-back
-	  that makes the response get routed to the same serializer that the
-	  request was sent on. However, in cases where an asynchronous DNS lookup
-	  occurs in the PJSIP core, the thread that sends the INVITE is not
-	  actually a threadpool serializer thread. This means we are unable to
-	  record a serializer to handle the incoming response.
+2015-03-29 21:45 +0000 [7bc2345fb1]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  Now, imagine what happens when an INVITE is sent on a non-serialized
-	  thread, and an error response (such as a 486) arrives. The 486 ends up
-	  getting put on some random threadpool thread. Eventually, a hangup task
-	  gets queued on the INVITE dialog serializer. Since the 486 is being
-	  handled on a different thread, the hangup task can execute at the same
-	  time that the 486 is being handled. The hangup task assumes that it is
-	  the sole owner of the INVITE session and channel, so it ends up
-	  potentially freeing the channel and NULLing the session's channel
-	  pointer. The thread handling the 486 can crash as a result.
+	* clang compiler warnings: Fix -Wabsolute-value warnings
 
-	  This change has the incoming response match the INVITE transaction, and
-	  then get the dialog from that transaction. It's the same method we had
-	  been using for matching incoming CANCEL requests. By doing this, we get
-	  the INVITE dialog and can ensure that the 486 response ends up being
-	  handled by the same thread as the hangup, ensuring that the hangup runs
-	  after the 486 has been completely handled.
+	  This patch fixes several warnings caught by clang - in this case, usage of the
+	  abs function on non-integer values. This patch uses labs and fabs, as
+	  appropriate, in the various affected files.
 
-	  ASTERISK-25941 #close
-	  Reported by Javier Riveros
+	  Review: https://reviewboard.asterisk.org/r/4525
 
-	  Change-Id: I0d4cc5d07e2a8d03e9db704d34bdef2ba60794a0
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4525.patch submitted by dkdegroot (License 6600)
+	  ........
 
-2016-05-19 11:41 +0000 [ddcf983e39]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 433749 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_sorcery_astdb: Filter fields to only the registered ones.
+	  Merged revisions 433750 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change introduces the same filtering that is done in res_sorcery_realtime
-	  to the res_sorcery_astdb module. This allows persisted sorcery objects
-	  that may contain unknown fields to still be read in from the AstDB
-	  and used. This is particularly useful when switching between different
-	  versions of Asterisk that may have introduced additional fields.
 
-	  ASTERISK-26014 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433751 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib655130485a3ccfd635b7ed5546010ca14690fb2
+2015-03-29 21:39 +0000 [ce59fabd5c]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-05-09 21:40 +0000 [39fedfa423]  snuffy <snuffy22@gmail.com>
+	* clang compiler warnings: Fix invalid enum conversion
 
-	* res_pjsip_empty_info: Respond to empty SIP INFO packets
+	  This patch fixes some invalid enum conversion warnings caught by clang. In
+	  particular:
+	  * chan_sip: Several functions mixed usage of the st_refresher_param
+	    enum and st_refresher enum. This patch corrects the functions to use the
+	    right enum.
+	  * chan_pjsip: Fixed mixed usage of ast_sip_session_t38state and ast_t38_state.
+	  * strings: Fixed incorrect usage of AO2 flags with strings container.
+	  * res_stasis: Change a return enumeration to stasis_app_user_event_res.
 
-	  Some SBCs require responses to empty SIP INFO packets
-	  after establishing call via INVITE, if not responded to
-	  they may drop your call after unspecified timeout of X minutes.
+	  Review: https://reviewboard.asterisk.org/r/4535
 
-	  They are identified by having no Content-Type, check for this
-	  and respond with 200 - OK message.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4535.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  ASTERISK-24986 #close
-	  Reported-by: Ilya Trikoz, Federico Santulli
+	  Merged revisions 433746 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ib27e4f07151e5aef28fa587e4ead36c5b87c43e0
+	  Merged revisions 433747 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-18 07:54 +0000 [935e0496c4]  gtjoseph <gjoseph@digium.com>
 
-	* udptl:  Don't eat sequence numbers until OK is received
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433748 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Scenario:
-	  Local fax -> Asterisk w/ firewall -> Provider -> Remote fax
+2015-03-29 21:29 +0000 [61577cbee6]  Matt Jordan <mjordan@digium.com>
 
-	  * Local fax starts rtp call to remote fax
-	  * Remote fax starts t38 call back to local fax.
-	  * Local fax sends t38 no-signal to Asterisk before sending an OK.
-	  * udptl processes the frame and increments the expected sequence number.
-	  * chan_sip drops the frame because the call isn't up so nothing goes out
-	    the external interface to open the port for incoming packets.
-	  * Local fax sends OK and Asterisk sends OK to the remote fax.
-	  * Remote fax sends t38 packets which are dropped by the firewall.
-	  * Local fax re-sends t38 no-signal with the same sequence number.
-	  * udptl drops the frame because it thinks it's a dup.
-	  * Still no outgoing packets to open the firewall.
-	  * t38 negotiation fails.
+	* main/stdtime/localtime: Fix warning introduced in r433720
 
-	  The patch drops frames t38 received before udptl sequence processing
-	  when the call hasn't been answered yet.  The second no-signal frame
-	  is then seen as new and is relayed out the external interface which
-	  opens the port and allows negotiation to continue.
+	  The patch in r433720 caused a warning to be kicked back by gcc. It occurred
+	  due to this check in unistd.h:
 
-	  ASTERISK-26034 #close
+	      if (__nbytes > __bos0 (__buf))
+	          return __read_chk_warn (__fd, __buf, __nbytes, __bos0 (__buf));
 
-	  Change-Id: I11744b39748bd2ecbbe8ea84cdb4f3c5943c5af9
+	  That is, if __nbytes is greater than the result of GCC's built-in object size
+	  for the struct, we'll kick back a warning.
 
-2016-05-17 11:14 +0000 [77e8ec162b]  gtjoseph <gjoseph@digium.com>
+	  As it turns out, this is because there is an error in the code in the patch.
+	  We are passing the address of the pointer to the struct, not iev, which is a
+	  pointer to the struct. Hence, the number of bytes is probably going to be lot
+	  larger than the number of bytes that make up a pointer! This patch changes
+	  the code just read from the pointer to the struct - which fixes the warning.
 
-	* chan_sip:  Prevent extra Session-Expires headers from being added
+	  ASTERISK-24917
+	  ........
 
-	  When chan_sip does a re-INVITE to refresh a session and authentication
-	  is required, the INVITE with the Authorization header containes a
-	  second Session-Expires header without the ";refersher=" parameter.
-	  This is causing some proxies to return a 400.  Also, when Asterisk is
-	  the uas and the refresher, it is including the Session-Expires and
-	  Min-SE headers in OPTIONS messages which is not allowed per RFC4028.
+	  Merged revisions 433743 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch (based on the reporter's) Checks to see if a Session-Expires
-	  header is already in the message before adding another one.  It also
-	  checks that the method is INVITE or UPDATE.
+	  Merged revisions 433744 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26030 #close
 
-	  Change-Id: I58a7b07bab5a3177748d8a7034fb8ad8e11ce1d9
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433745 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-05-16 15:29 +0000 [3f6ef63099]  gtjoseph <gjoseph@digium.com>
+2015-03-29 20:57 +0000 [072734692e]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	* res_pjsip_outbound_registration:  Clean up state when registration is deleted
+	* clang compiler warnings: Ignore -Wunused-command-line-argument
 
-	  Nothing was cleaning up the registration state object when ast_sorcery_delete
-	  was called on a registration.  So, the registration was deleted from sorcery
-	  but the state object went right on refreshing the registration (or failing
-	  to refresh the registration) with the peer.
+	  Asterisk's build system has a tendency to pass include directives for libraries
+	  to everything compiled within a particular group of source files. This means
+	  we pass the header for libxml2 to things that don't necessarily need it. As a
+	  result, we ignore this particular warning.
 
-	  * Added a 'deleted' observer on registration that removes the state object.
+	  Review: https://reviewboard.asterisk.org/r/4545/
 
-	  ASTERISK-25964 #close
-	  Reported-by Matt Jordan
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4545.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  Change-Id: I2db792145cdb1f72ebbf57dd9099596dbbf12c23
+	  Merged revisions 433720 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-15 19:05 +0000 [b6f9392a12]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 433721 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip:  Set TCP_NODELAY on TCP transports
 
-	  Although it's perfectly legal to place multiple SIP messages in the same packet,
-	  it can cause problems because the Linux default is to enable Path MTU Discovery
-	  which sets the Don't Fragment bit on the packets. If adding a second message to
-	  the packet causes the MTU to be exceeded, and the destination isn't equipped to
-	  send a FRAGMENTATION NEEDED response to a large packet, the packet will just be
-	  dropped.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433722 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  We can't specifically tell the stack to send only 1 message per packet, but we
-	  can turn on TCP_NODELAY when we create the transport. This will at least tell
-	  the stack to send packets as soon as possible.
+2015-03-29 20:53 +0000 [1cf949c489]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  ASTERISK-26005 #close
-	  Reported-by: Ross Beer
+	* clang compiler warnings: Fix warning for -Wgnu-variable-sized-type-not-at-end
 
-	  Change-Id: I820f23227183f2416ca5e393bec510e8fe1c8fbd
+	  This patch fixes a warning caught by clang, wherein a variable sized struct is
+	  not located at the end of a struct. While the code in question actually
+	  expected this, this is a good warning to watch for. Hence, this patch refactors
+	  the code in question to not have two variable length elements in the same
+	  struct.
 
-2016-05-14 21:48 +0000 [361a16f316]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4530/
 
-	* configs/samples/pjsip.conf.sample: Fix typo
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4530.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  A ':' is not a valid token for starting a comment.
+	  Merged revisions 433717 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I123592d93a83d1bdde3e352822881eb9da85e5ad
+	  Merged revisions 433718 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-12 07:08 +0000 [f91a7dc993]  Matt Jordan <mjordan@digium.com>
 
-	* res/res_hep_pjsip: Fix reported local IP address when bound to 'any'
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433719 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When bound to an 'any' address, e.g., 0.0.0.0, PJSIP reports as its
-	  local address the 'any' address, as opposed to the IP address we
-	  actually received the packet on. This can cause some confusion in Homer,
-	  as it will dutifully report what we send it.
+2015-03-28 07:56 +0000 [d2776d4d45]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  This patch uses the PJSIP inspection routines to determine which IP
-	  address we probably received the packet on based on the remote party's
-	  IP address. In the event that this fails, it falls back to the IP
-	  address natively reported by the transport.
+	* clang compiler warnings: Fix a variety of "unused" warnings
 
-	  Change-Id: I076f835d2aef489e1ee1d01595b211eb2ce62da3
+	  This patch fixes the -Wunused-value -Wunused-variable -Wunused-const-variable
+	  errors caught by clang. Specifically:
 
-2016-05-14 12:29 +0000 [9de5cd209e]  Sean Bright <sean.bright@gmail.com>
+	  * apps/app_queue.c: removed unused qpm_cmd_usage[], qum_cmd_usage[],
+	                      qsmp_cmd_usage[]
+	  * cel/cel_sqlite3_custom.c: removed unused name[] = "cel_sqlite3_custom"
+	  * channels/chan_pjsip.c: removed unused desc[] = "PJSIP Channel"
+	  * codecs/gsm/src/gsm_create.c: removed unused ident[] = "$Header$"
+	  * funcs/func_env.c:729: Fixed ast_str_append_substr.
+	  * main/editline/np/strlcat.c: removed unused rcsid variable
+	  * main/editline/np/strlcpy.c: removed unused rcsid variable
+	  * main/security_events.c: removed unused TIMESTAMP_STR_LEN
+	  * utils/conf2ael.c: removed unused cfextension_states
+	  * utils/extconf.c: removed unused cfextension_states
 
-	* res_ari: Correct Location headers returned by some ARI resources
+	  Review: https://reviewboard.asterisk.org/r/4526
 
-	  The Location headers returned by:
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4526.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	   * /bridges/{bridgeId}/play
-	   * /bridges/{bridgeId}/record
-	   * /channels/{channelId}/play
-	   * /channels/{channelId}/record
+	  Merged revisions 433693 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Did not have the '/ari' prefix, and in the case of the 'play' resources, were
-	  using 'playback' instead of 'playbacks.'
+	  Merged revisions 433694 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I957c58a3a1471bf477dae7c67faa1b74fcd9241c
 
-2016-05-13 11:38 +0000 [524a302974]  Alexei Gradinari <alex2grad@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433695 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Endpoint IP Access Controls
+2015-03-28 07:48 +0000 [cb7b6bc4be]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  With the old SIP module we can use IP access controls per peer.
-	  PJSIP module missing this feature.
+	* clang compiler warnings: Fix -Wself-assign
 
-	  This patch added next configuration Endpoint options:
-	      "acl" - list of IP ACL section names in acl.conf
-	      "deny" - List of IP addresses to deny access from
-	      "permit" - List of IP addresses to permit access from
-	      "contact_acl" - List of Contact ACL section names in acl.conf
-	      "contact_deny" - List of Contact header addresses to deny
-	      "contact_permit" - List of Contact header addresses to permit
+	  Assigning a variable to itself isn't super useful. However, the WAV format
+	  modules make use of this in order to perform byte endian checks. This patch
+	  works around the warning by only performing the self assignment if we are
+	  going to do more than just assign it to ourselves. Which is odd, but true.
 
-	  This patch also better logging failed request:
-	      add custom message instead of "No matching endpoint found"
-	      add SIP method to logging
+	  Review: https://reviewboard.asterisk.org/r/4544/
 
-	  ASTERISK-25900
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4544.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  Change-Id: I456dea3909d929d413864fb347d28578415ebf02
+	  Merged revisions 433690 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-11 20:17 +0000 [89ae4466ea]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 433691 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_hep: Provide an option to pick the UUID type
 
-	  At one point in time, it seemed like a good idea to use the Asterisk
-	  channel name as the HEP correlation UUID. In particular, it felt like
-	  this would be a useful identifier to tie PJSIP messages and RTCP
-	  messages together, along with whatever other data we may eventually send
-	  to Homer. This also had the benefit of keeping the correlation UUID
-	  channel technology agnostic.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433692 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In practice, it isn't as useful as hoped, for two reasons:
-	  1) The first INVITE request received doesn't have a channel. As a
-	     result, there is always an 'odd message out', leading it to be
-	     potentially uncorrelated in Homer.
-	  2) Other systems sending capture packets (Kamailio) use the SIP Call-ID.
-	     This causes RTCP information to be uncorrelated to the SIP message
-	     traffic seen by those capture nodes.
+2015-03-28 07:41 +0000 [e9520dbe0d]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  In order to support both (in case someone is trying to use res_hep_rtcp
-	  with a non-PJSIP channel), this patch adds a new option, uuid_type, with
-	  two valid values - 'call-id' and 'channel'. The uuid_type option is used
-	  by a module to determine the preferred UUID type. When available, that
-	  source of a correlation UUID is used; when not, the more readily available
-	  source is used.
+	* clang compiler warnings: Fix -Wparantheses-equality warnings
 
-	  For res_hep_pjsip:
-	   - uuid_type = call-id: the module uses the SIP Call-ID header value
-	   - uuid_type = channel: the module uses the channel name if available,
-	                          falling back to SIP Call-ID if not
-	  For res_hep_rtcp:
-	   - uuid_type = call-id: the module uses the SIP Call-ID header if the
-	                          channel type is PJSIP and we have a channel,
-	                          falling back to the Stasis event provided
-	                          channel name if not
-	   - uuid_type = channel: the module uses the channel name
+	  Clang will treat ((a == b)) as a warning, as it reasonably expects that the
+	  developer may have intended to write (a == b) or ((a = b)). This patch cleans
+	  up all instances where equality, not assignment, was intended between two
+	  parantheses.
+
+	  Review: https://reviewboard.asterisk.org/r/4531/
+
+	  ASTERISK-24917
+	  Repoted by: dkdegroot
+	  patches:
+	    rb4531.patch submitted by dkdegroot (License 6600)
+	  ........
+
+	  Merged revisions 433687 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25352 #close
+	  Merged revisions 433688 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ide67e59a52d9c806e3cc0a797ea1a4b88a00122c
 
-2016-05-10 02:56 +0000 [a73d79c22f]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433689 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* basic-cfg: asterisk.conf: remove [directories]
+2015-03-28 07:33 +0000 [fd50e5bfb5]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-	  A minimal configuration does not need to explicitly spell out the
-	  directories. The built-in defaults will do just fine. In many cases
-	  they are wrong.
+	* clang compiler warnings: Fix -Wbitfield-constant-conversion warning
 
-	  Change-Id: Id1a671e5c5e9923765a4156b57f9f7e263fdd26c
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  In chan_iax2, we attempt to assign a -1 to a bitfield. This gets caught by
+	  clang, as it will truncate the -1 to a 1 implicitly.
 
-2016-05-10 03:06 +0000 [1c56de9453]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Instead, we just assign the value a '1'.
 
-	* basic-cfg: asterisk.conf: defaults of options
+	  Review: https://reviewboard.asterisk.org/r/4537/
 
-	  Note the default of remmed-out options. To clarify that those values are
-	  not the defaults.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4537.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  Change-Id: I849c29b7a710f0abc37355fcb5bfee335ae30738
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Merged revisions 433683 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-10 03:08 +0000 [d7af591c59]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Merged revisions 433684 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* basic-cfg: asterisk.conf: debug level 5 spams
 
-	  Don't suggest users to use debug level 5, which spews (usually
-	  non-useful) debug information. Reduce the suggestion to (an
-	  arbitrarily-selected) level 2.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433686 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib53195f78945970956ff59ef13fa89b90e0fcd60
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+2015-03-28 07:32 +0000 [c747b3b12a]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-05-10 03:10 +0000 [9b7db18fc1]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	* clang compiler warnings: Fix -Winitializer-overrides
 
-	* basic-cfg: asterisk.conf: don't set languages
+	  This patch fixes clange compiler warnings for initializer overrides.
+	  Specifically:
 
-	  * No need to set language in a miniml configuration. 'en' will do just
-	    fine.
-	  * It would be useful to have an example of setting it to a different
-	    language.
-	  * Setting the documentation language explicitly is likewise not
-	    required. Setting it to a different value is not common. At least
-	    until there is a set of translated documentation.
+	  res_pjsip/config_transport maps PJSIP_TLSV1_METHOD to the same enumeration
+	  value as PJSIP_SSL_DEFAULT_METHOD. When initializing an array containing
+	  those enum values, we therefore initialize the value twice to two different
+	  values, "tlsv1" and "default". This patch changes it to just initialize
+	  the index in the array to "tlsv1".
 
-	  Change-Id: I94d91ea34e129925f25af81ef8dc0906fb568cb7
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  Review: https://reviewboard.asterisk.org/r/4539/
 
-2016-05-10 08:17 +0000 [eec539a46e]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4539.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	* followme: delete the right recorded name file
+	  Merged revisions 433682 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  FollowMe with the option a records the name of the caller and plays it
-	  to the callee. However it has failed to clean up that recorded file
-	  as it tried to delete the file name without the '.sln' extension.
 
-	  ASTERISK-26008 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433685 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I79d7b1be7d5cde57bf076d9389e2a8a4422776ec
-	  Signed-off-by: Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+2015-03-28 07:20 +0000 [d6173cd1d0]  Diederik de Groot <dkgroot@talon.nl> (License 6600)
 
-2016-05-12 14:36 +0000 [02d30e171e]  Mark Michelson <mmichelson@digium.com>
+	* clang compiler warnings: Fix -Wunused-function; make inline function static
 
-	* Use doubles instead of floats for conversions when comparing strings.
+	  This patch fixes clang compilers warnings for unused functions. Specifically:
+	   * channels/chan_iax2: removed user_ref function
+	   * main/dsp.c: removed goertzel_update function
+	   * main/config.c: made variable_list_switch static
 
-	  In 13.9.0, there was an issue where PJSIP contacts added to an AOR would
-	  be deleted at seemingly random times.
+	  Review: https://reviewboard.asterisk.org/r/4527
 
-	  One reason this was happening was because of an operation to retrieve
-	  the contacts whose expiration time was less than or equal to the current
-	  time. When retrieving existing contacts, the contact's expiration time
-	  and the current time were converted from a string to a float, and those
-	  two floats were compared.
+	  ASTERISK-24917
+	  Reported by: dkdegroot
+	  patches:
+	    rb4527.patch submitted by dkdegroot (License 6600)
+	  ........
 
-	  On some systems, including mine, this conversion was horribly off. For
-	  instance, I could regularly see the string "1463079214" get converted
-	  into 1463079168.000000. When switching from using a float to using a
-	  double, the conversion was as expected.
+	  Merged revisions 433678 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Why was the conversion to float off? My best guess is that the
-	  conversion to float was attempting to store the entire value in the 23
-	  bit significand of the IEEE-754 floating point number. In particular, if
-	  you take only the 23 most significant bits of 1463079214, you get the
-	  messed up 1463079168 that we were seeing in the conversion. It likely
-	  was possible to get a more precise value by composing the number using
-	  an exponent, but the conversion did not work that way. With a double,
-	  you have a 52 bit significand, allowing the entire value to fit there,
-	  and thereby allowing an accurate conversion.
+	  Merged revisions 433680 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-26007 #close
-	  Reported by Greg Siemon
 
-	  Change-Id: I83ca7944aae8b7cd994b254c78ec02411d321070
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433681 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-05-12 09:13 +0000 [e2df15bae9]  gtjoseph <gjoseph@digium.com>
+2015-03-27 17:26 +0000 [b56592e3ae]  Jonathan Rose <jrose@digium.com>
 
-	* pjsip_distributor:  Add missing newline to NOTICE
+	* SAC: Add conferencing extensions and configuration
 
-	  There was a newline missing from the end of the "no matching endpoint" notice.
+	  Review: https://reviewboard.asterisk.org/r/4504/
+	  ........
 
-	  Change-Id: Idc11fe5bc0354072291663dbffe648c471e39181
+	  Merged revisions 433656 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-10 10:19 +0000 [a94a12bbf7]  Sebastian Damm <damm@sipgate.de>
 
-	* res_pjsip_outbound_registration: generate correct Contact URI for TLS
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433657 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  There are two types of SIP URIs indicating a secure transport:
-	  * sips:user@example.org
-	  * sip:user@example.org;transport=tls
+2015-03-27 16:21 +0000 [c21e2e45a8]  Rusty Newton <rnewton@digium.com>
 
-	  When using a sips URI, Asterisk checks incoming INVITEs and answers from
-	  the other side for sips URIs, and rejects the packet if there are only
-	  sip URIs. So Asterisk should only generate a sips Contact URI if the
-	  other side supports it.
+	* configs/basic-pbx - Super Awesome Company example configs Phase 1, Patch 2
 
-	  This patch makes Asterisk generate either a sip or sips Contact URI
-	  depending on the format of the server URI.
+	  Example configuration files for a "basic PBX" deployment for the fictitious
+	  Super Awesome Company. Details at https://reviewboard.asterisk.org/r/4488/
+	  and https://wiki.asterisk.org/wiki/display/AST/Super+Awesome+Company
 
-	  If you want a sip URI, use:
-	  server_uri=sip:example.org\;transport=tls
+	  Patch 4488 includes all functionality needed for SAC's outside connectivity
+	  and some externally accessed features, as well as outbound dialing.
 
-	  If you want a sips URI, use:
-	  server_uri=sips:example.org
+	  Reported by: Malcolm Davenport
+	  Tested by: Rusty Newton
 
-	  ASTERISK-25990 #close
-	  Reported-by: Sebastian Damm
+	  Review: https://reviewboard.asterisk.org/r/4488/
+	  ........
 
-	  Change-Id: I5ae57d6531ce940b5fc64d5cd2673e60db0f9ba2
+	  Merged revisions 433624 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-05 16:41 +0000 [36d66a23e0]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* logger: Add PID to syslog messages.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433637 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  During refactoring of this support the addition of
-	  the PID to messages was removed. This change adds it
-	  back in.
+2015-03-27 16:06 +0000 [2659e48d9d]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25538 #close
+	* res_pjsip_registrar_expire.c: Made use ao2 container template routines and eliminated some RAII_VAR() usage.
 
-	  Change-Id: Ie2d43b0652e59b7ac319a7dba94501540d70ba36
+	  * Converted the contact_autoexpire container to use the ao2 template hash
+	  and cmp functions.  Also made use the OBJ_SEARCH_xxx names instead of the
+	  deprecated names.
 
-2016-05-11 14:07 +0000 [37214b0bdf]  Matt Jordan <mjordan@digium.com>
+	  * Eliminates several unnecessary uses of RAII_VAR().
 
-	* configure: Fix errors with AST_UNDEFINED_SANITIZER/AST_LEAK_SANITIZER
+	  Review: https://reviewboard.asterisk.org/r/4524/
+	  ........
 
-	  When running on a system that does not support or use AST_UNDEFINED_SANITIZER
-	  or AST_LEAK_SANITIZER, the configure script would incorrectly set those
-	  constants to a blank value, e.g., 'AST_UNDEFINED_SANITIZER='. This would
-	  cause menuselect to error out, complaining that a blank value is not a
-	  valid option. This patch corrects the issue by setting the value to 0 if
-	  the options that those constants enable/disable is not found.
+	  Merged revisions 433622 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ib39814aaf940f308d500c1e026edb3d70de47fba
 
-2016-05-03 15:43 +0000 [49b25a0956]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433623 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_outbound_publish: state potential dropped on reloads/realtime fetches
+2015-03-27 15:46 +0000 [0b62e41654]  Mark Michelson <mmichelson@digium.com>
 
-	  When reloading, or fetching realtime data, if the "apply" failed for any
-	  numerous reasons the current state object would not be maintained. This
-	  potentially resulted in publishes being stopped for some states/clients when
-	  they should not have been.
+	* Add stateful PJSIP response API call, and use it for out-of-dialog responses.
 
-	  This patch makes it so the current state object is kept upon any type of reload/
-	  fetch failures.
+	  Asterisk had an issue where retransmissions of MESSAGE requests resulted in
+	  Asterisk processing the retransmission as if it were a new MESSAGE request.
 
-	  Change-Id: Iab6020c116d628ed2ae81183e987e2eaa3c90b30
+	  This patch fixes the issue by creating a transaction in PJSIP on the incoming
+	  request. This way, if a retransmission arrives, the PJSIP transaction layer
+	  will resend the response and Asterisk will not ever see the retransmission.
 
-2016-05-03 15:31 +0000 [1b5c91b7be]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24920 #close
+	  Reported by Mark Michelson
 
-	* res_pjsip_outbound_publish: Potential crash due to off nominal path
+	  Review: https://reviewboard.asterisk.org/r/4532/
+	  ........
 
-	  It was possible for the explicit publish destroy function to be called without
-	  the pjsip client ever being initialized. This fix checks to make sure there is
-	  a client to destroy before attempting.
+	  Merged revisions 433619 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I8eea1bfa3bd472149bfc255310be2a6248688f5c
 
-2016-05-03 15:35 +0000 [10de553c9d]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433620 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_outbound_publishing: After unloading the library won't load again
+2015-03-27 15:23 +0000 [a18da4eaf2]  Richard Mudgett <rmudgett@digium.com>
 
-	  The same thing was happening in res_pjsip_publish_asterisk. When the library
-	  was unloaded it did not unregister the object type from sorcery. Subsequent
-	  loads resulted in a failed load due to the sorcery type already existing.
+	* res_pjsip_registrar_expire.c: Cleanup scheduler leaks on unload/shutdown.
 
-	  Change-Id: Ifdc25e94e4cd40bc5a19eb4d0a00b86c2e9fedc9
+	  Contact expiration object refs were leaked when the module was unloaded.
 
-2016-05-03 14:59 +0000 [1a833b9739]  Kevin Harwell <kharwell@digium.com>
+	  * Made empty the scheduler of entries before destroying it to release the
+	  object ref held by the scheduler entry.
 
-	* res_pjsip_outbound_publish: Ref leak in off nominal callback paths
+	  Review: https://reviewboard.asterisk.org/r/4523/
+	  ........
 
-	  There were a few spots where the client object's reference was being leaked in
-	  sip_outbound_publish_callback. This patch cleans up those leaks.
+	  Merged revisions 433596 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I485d0bc9335090f373026f77c548042e258461df
 
-2016-05-03 15:39 +0000 [4752ef02e0]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433617 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_outbound_publish: Won't unload if condition wait times out
+2015-03-27 12:58 +0000 [cb1c639817]  Richard Mudgett <rmudgett@digium.com>
 
-	  When res_pjsip_outbound_publish unloads it has to wait for all current
-	  publishing objects to get done. However if the wait condition times out
-	  then it does not fail the unload. This sometimes results in an infinite
-	  loop check while unloading. This patch now fails the unload operation if
-	  the condition times out.
+	* Add missing file.  ASTERISK-24781
 
-	  Change-Id: Id57b8cbed9d61222690fcba1e4f18e259df4c7ec
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433597 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-05-05 11:37 +0000 [4d063814ba]  Kevin Harwell <kharwell@digium.com>
+2015-03-27 09:41 +0000 [a024af1156]  Justin T. Gibbs <gibbs@scsiguy.org> (License 6692)
 
-	* res_pjsip_authenticator_digest: Don't use source port in nonce verification
+	* res/res_timing_kqueue: Update the module to conform to current timer API
 
-	  From the issue reporter:
-	  "res_pjsip_outbound_authenticator_digest builds a nonce that is a hash of
-	  the timestamp, the source address, the source port, a server UUID that is
-	  calculated at startup, and the authentication realm.
+	  This patch updates the kqueue timing module to conform to current timer API.
 
-	  Rather than caching nonces that we create, we instead attempt to re-calculate
-	  the nonce when receiving an incoming request with authentication. We then
-	  compare the re-calculated nonce to the incoming nonce, and if they don't match,
-	  then authentication has failed early.
+	  This fixes issues with using the kqueue timing source on Asterisk 13 on
+	  FreeBSD 10. These issues include:
 
-	  The problem is that it is possible, especially when using TCP, to receive two
-	  requests from the same endpoint but have differing source ports for those
-	  requests. Asterisk itself commonly will use different source ports for
-	  outbound TCP requests."
+	  - Remove support for kevent64().  The values used to support Asterisk timers
+	    fit within 32bits and so can be handled on all platforms via kevent().
 
-	  This patch removes the source port dependency when building the nonce.
+	  - Provide debug logging for, but do not track, unacked events.  This matches
+	    the behavior of all other timer implementations.
 
-	  ASTERISK-25978 #close
+	  - Implement continuous mode by triggering and leaving active, a user event.
+	    This ensures that the file descriptor for the timer returns immediately from
+	    poll(), without placing the load of a high speed timer on the kernel.
 
-	  Change-Id: I871b5f4adce102df1c4988066283095ec509dffe
+	  - In kqueue_timer_get_max_rate(), don't overstate the capability of the timer.
+	    On some platforms, UINT_MAX is greater than INTPTR_MAX, the largest integer
+	    type kqueue supports for timers.
 
-2016-05-07 14:39 +0000 [fb6227a372]  gtjoseph <gjoseph@digium.com>
+	  - In kqueue_timer_get_event(), assume the caller woke up from poll() and just
+	    return the mode the timer is currently in. This matches all other timer
+	    implementations.
 
-	* config_transport:  Tell pjproject to allow all SSL/TLS protocols
+	  - Adjust the test code now that unacked events are not tracked.
 
-	  The default tls settings for pjproject only allow TLS 1, TLS 1.1 and TLS 1.2.
-	  SSL is not allowed.   So, even if you specify "sslv3" for a transport method,
-	  it's silently ignored and one of the TLS protocols is used.  This was a new
-	  behavior of pjsip_tls_setting_default() in 2.4 (when tls.proto was added) that
-	  we never caught.
+	  Review: https://reviewboard.asterisk.org/r/4465/
 
-	  Now we need to set tls.proto = 0 after we call pjsip_tls_setting_default().
-	  This tells pjproject to set the socket protocol to match the method.
+	  ASTERISK-24857 #close
+	  Reported by: scsiguy
+	  Tested by: Ed Hynan
+	  patches:
+	    rb4465.patch submitted by scsiguy (License 6692)
+	  ........
 
-	  ASTERISK-26004 #close
+	  Merged revisions 433574 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Icfb55c1ebe921298dedb4b1a1d3bdc3ca41dd078
 
-2016-05-04 02:40 +0000 [2db17a793c]  Jaco Kroon <jaco@uls.co.za>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433575 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* app_confbridge: Add a regcontext option for confbridge bridge profiles.
+2015-03-27 07:27 +0000 [10458d2878]  Corey Farrell <git@cfware.com>
 
-	  This patch allows for having app_confbridge register the name of the
-	  conference as an extension into a specific context, similar to
-	  regcontext for chan_sip.  This variant is not quite as involved as the
-	  one in chan_sip and doesn't allow for multiple contexts or custom
-	  extensions, you can only specify the context and the conference name
-	  will always be used as the extension to register.
+	* Fix link error for utils/aelparse.
 
-	  ASTERISK-25989 #close
+	  Use the standard ast_log instead of ast_log_safe for STANDALONE programs.
 
-	  Change-Id: Icacf94d9f2b5dfd31ef36f6cb702392619a7902f
+	  Review: https://reviewboard.asterisk.org/r/4538/
+	  ........
 
-2016-05-08 20:19 +0000 [2a7130b8b0]  gtjoseph <gjoseph@digium.com>
+	  Merged revisions 433549 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* pjproject_bundled:  Check for python-dev and TEST_FRAMEWORK
+	  Merged revisions 433550 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The pjsua and pjsystest apps are now built only if TEST_FRAMEWORK is set.
-	  The python bindings are now built only if TEST_FRAMEWORK is set and a
-	  python development package is installed.
 
-	  libresample was also disabled.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433551 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25993 #close
-	  Reported-by: Joshua Colp
+2015-03-27 02:12 +0000 [28e3bd0af7]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: If4e91c503a02f113d5b71bc8b972081fa3ff6f03
+	* Improved and portable ast_log recursion avoidance
 
-2016-05-04 15:16 +0000 [72eb7c8301]  Alexei Gradinari <alex2grad@gmail.com>
+	  This introduces a new logger routine ast_log_safe.  This routine should be
+	  used for all error messages in code that can be run as a result of ast_log.
+	  ast_log_safe does nothing if run recursively.  All error logging in
+	  astobj2.c, strings.c and utils.h have been switched to ast_log_safe.
 
-	* res_pjsip: module load priority
+	  This required adding support for raw threadstorage.  This provides direct
+	  access to the void* pointer in threadstorage.  In ast_log_safe, NULL is used
+	  to signify that this thread is not already running ast_log_safe, (void*)1 when
+	  it is already running.  This was done since it's critical that ast_log_safe
+	  do nothing that could log during recursion checking.
 
-	  The res_pjsip_authenticator_digest, res_pjsip_endpoint_identifier_*
-	  and res_pjsip_registrar modules should load ASAP
-	  to avoid "No matching endpoint found" for legitimate endpoint.
+	  ASTERISK-24155 #close
+	  Reported by: Timo Teräs
+	  Review: https://reviewboard.asterisk.org/r/4502/
+	  ........
 
-	  ASTERISK-25994
+	  Merged revisions 433522 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Iac95d95ad031e0be104189d29e923a2ad7c24a1b
+	  Merged revisions 433523 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-04 03:17 +0000 [dd00c71aae]  Chris Trobridge <christ.trobridge@ultra-aep.com>
 
-	* config_options.c: Expand #ifdef to contain whole if statement.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433524 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25956 #close
+2015-03-26 18:09 +0000 [554eb74516]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: If6961ec54be276d5ab4f012ee7e7b420cb45de38
+	* Fix compile errors caused by r4500 / r4501.
 
-2016-05-05 15:16 +0000 [e6eb17efd9]  Alexei Gradinari <alex2grad@gmail.com>
+	  * Add ast_register_cleanup to utils/clicompat.c to deal with
+	    any utils that copy sources from main.
+	  * Asterisk 13+: remove unused variables from core_local.c.
 
-	* stasis_endpoints: Add new Status and Headers to ContactStatus
+	  Review: https://reviewboard.asterisk.org/r/4534/
+	  ........
 
-	  ASTERISK-25903 added a new headers to AMI Event ContactStatusDetail.
-	  ASTERISK-25904 added a new Status to AMI Event ContactStatusDetail.
-	  These additions should be also in stasis_endpoints
-	  to include in command "manager show event ContactStatus"
+	  Merged revisions 433499 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I7610ad02a998e1f26c20caa27aa50279d0164f6a
+	  Merged revisions 433500 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-05 05:07 +0000 [fa11f4c920]  Joshua Colp <jcolp@digium.com>
 
-	* file: Ensure nativeformats remains valid for lifetime of use.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433501 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  It is possible for the nativeformats of a channel to change
-	  throughout its lifetime. As a result a user of it needs to either
-	  ensure the channel is locked when accessing the formats or keep
-	  a reference to the nativeformats themselves.
+2015-03-26 17:24 +0000 [3ddd92902a]  Corey Farrell <git@cfware.com>
 
-	  This change fixes the file playback support so it keeps a
-	  reference to the nativeformats when accessing things.
+	* Replace most uses of ast_register_atexit with ast_register_cleanup.
 
-	  ASTERISK-25998 #close
+	  Since 'core stop now' and 'core restart now' do not stop modules,
+	  it is unsafe for most of the core to run cleanups.  Originally all
+	  cleanups used ast_register_atexit, and were only changed when it
+	  was shown to be unsafe.  ast_register_atexit is now used only when
+	  absolutely required to prevent corruption and close child processes.
 
-	  Change-Id: Ie45b65475e1481ddf05b874ee48f63e39fff8915
+	  Exceptions that need to use ast_register_atexit:
+	  * CDR: Flush records.
+	  * res_musiconhold: Kill external applications.
+	  * AstDB: Close the DB.
+	  * canary_exit: Kill canary process.
 
-2016-04-15 09:32 +0000 [9c2032240e]  Alexei Gradinari <alex2grad@gmail.com>
+	  ASTERISK-24142 #close
+	  Reported by: David Brillert
 
-	* res_pjsip: improve realtime performance
+	  ASTERISK-24683 #close
+	  Reported by: Peter Katzmann
 
-	  This patch modified pjsip_options to retrieve only
-	  permament contacts for aor if the qualify_frequency is > 0
-	  and persisted contacts if the qualify_frequency is > 0.
+	  ASTERISK-24805 #close
+	  Reported by: Badalian Vyacheslav
 
-	  This patch also fixed a bug in res_sorcery_astdb.
-	  res_sorcery_astdb doesn't save object data retrived from astdb.
+	  ASTERISK-24881 #close
+	  Reported by: Corey Farrell
 
-	  ASTERISK-25826
+	  Review: https://reviewboard.asterisk.org/r/4500/
+	  Review: https://reviewboard.asterisk.org/r/4501/
+	  ........
 
-	  Change-Id: I1831fa46c4578eae5a3e574ee3362fddf08a1f05
+	  Merged revisions 433495 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-05-04 16:11 +0000 [fe38d21c2a]  Alexei Gradinari <alex2grad@gmail.com>
+	  Merged revisions 433497 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* pjsip: Added "reg_server" to contacts (fixed alembic)
 
-	  ASTERISK-25931
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433498 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Icc4321a88f5c93ff809da3f372eebbf69c6a8549
+2015-03-26 12:47 +0000 [d7fc85e69d]  Corey Farrell <git@cfware.com>
 
-2016-04-07 16:33 +0000 [7a14e669f0]  Alexei Gradinari <alex2grad@gmail.com>
+	* res_pjsip: Enable unload of all modules at shutdown.
 
-	* res_pjsip/AMI: add contact.updated event
+	  * Move most of res_pjsip:module_unload to unload_pjsip to resolve crashes
+	    caused by running PJSIP functions from non-PJSIP threads.
+	  * Remove call to pjsip_endpt_destroy(ast_pjsip_endpoint), it was causing
+	    crashes in some cases.  In theory pj_shutdown() should take care of this.
+	  * Mark res_pjsip_keepalive and res_pjsip_session as allowed to unload at
+	    shutdown.
+	  * Resolve leaked config global in res_pjsip_notify.
+	  * Unregister pubsub pjsip service module.
+	  * Implement cleanup for res_pjsip_session.
 
-	  With the old SIP module AMI sends PeerStatus event on every
-	  successfully REGISTER requests, ie, on start registration,
-	  update registration and stop registration.
+	  ASTERISK-24731 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4498/
+	  ........
 
-	  With PJSIP AMI sends ContactStatus only when status is changed.
-	  Regarding registration:
-	  on start registration - Created
-	  on stop registration - Removed
-	  but on update registration nothing
+	  Merged revisions 433469 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch added contact.updated event.
 
-	  ASTERISK-25904
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433470 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I8fad8aae9305481469c38d2146e1ba3a56d3108f
+2015-03-26 12:13 +0000 [ab674f67b5]  Kevin Harwell <kharwell@digium.com>
 
-2016-05-02 16:08 +0000 [06d4ac0355]  Alexei Gradinari <alex2grad@gmail.com>
+	* app_confbridge: file playback blocks dtmf
 
-	* res_fax: add FAXMODE variable
+	  Attempting to execute DTMF in a confbridge while file playback (prompt,
+	  announcement, etc) is occurring is not allowed. You have to wait until
+	  the sound file has completed before entering DTMF. This patch fixes it
+	  so that app_confbridge now monitors for dtmf key presses during menu
+	  driven file playback. If a key is pressed playback stops and it executes
+	  the matched menu option.
 
-	  The app_fax set FAXMODE variable, but res_fax missing this feature.
-	  This patch add FAXMODE variable which is set to either "audio" or "T38".
+	  ASTERISK-24864 #close
+	  Reported by: Steve Pitts
+	  Review: https://reviewboard.asterisk.org/r/4510/
+	  ........
 
-	  ASTERISK-25980
+	  Merged revisions 433445 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ie3dcbfb72cc681e9e267a60202f7fb8723a51b6b
+	  Merged revisions 433446 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-05-02 16:52 +0000 [2d17fe06c5]  Alexei Gradinari <alex2grad@gmail.com>
 
-	* res_fax/t38_gateway: Peer V.21 session is created on wrong channel
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433447 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The channel and peer V.21 sessions are created on the same channel now.
-	  The peer V.21 session should be created only on peer channel
-	  when one of channel can handle T.38.
+2015-03-25 13:37 +0000 [e953d15223]  Richard Mudgett <rmudgett@digium.com>
 
-	  Also this patch enable debug for T.38 gateway session
-	  if global fax debug enabled.
+	* A couple minor cleanup tweaks.
 
-	  ASTERISK-25982
+	  * In res/res_sorcery_realtime.c: Broke long line.
 
-	  Change-Id: I78387156ea521a77eb0faf170179ddd37a50430e
+	  * In main/bucket.c: Eliminated unnecessary NULL check as
+	  ast_sorcery_unref() is NULL tolerant and set the global object to NULL
+	  after unref in the system shutdown bucket_cleanup().
+	  ........
 
-2016-05-01 02:21 +0000 [a2f19d82a8]  Diederik de Groot <dkgroot@talon.nl>
+	  Merged revisions 433420 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* configs/basic-pbx/asterisk.conf: contains incorrect path separator
 
-	  Note: When packagers use these files (as an example) the paths are never
-	  really used when they are split using '='.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433421 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Note: Thirdparty applications will also have trouble parsing the file when
-	  expecting '=>'.
+2015-03-25 10:31 +0000 [47156aab92]  Simon Arlott (License 5756)
 
-	  Change-Id: I0ada647f588e81f023fb1333ca15a1a333fd6004
+	* res_xmpp: Buddies are always auto-registered when processing the roster
 
-2016-04-30 17:52 +0000 [f39089f17c]  gtjoseph <gjoseph@digium.com>
+	  Due to a quirk in the configuration handling of res_xmpp, the 'autoregister'
+	  setting was never actually processed. This was due to not properly copying
+	  over the global settings to the client settings when applying the
+	  configuration to the run-time object.
 
-	* pjproject_bundled:  Various fixes discovered during testing of OSes
+	  Review: https://reviewboard.asterisk.org/r/4496/
 
-	  For all OSes:
-	  * Disabled third-party codecs in pjproject and added
-	    '--disable-speex-codec --disable-speex-aec --disable-gsm-codec' to the
-	    configure options since we don't use the pjsip codec capability.
+	  ASTERISK-14233
+	  ASTERISK-24780 #close
+	  Reported by: Simon Arlott
+	  patches:
+	    asterisk-13.1.0-24780 uploaded by Simon Arlott (License 5756)
+	  ........
 
-	  FreeBSD:
-	  * Added FreeBSD support to install_prereq.
-	  * Changed pjproject/configure.m4 to use $GNU_MAKE instead of hardcoding "make".
-	  * Added __progname and environ to asterisk.exports.in.
-	  * Reverted the use of ldconfig to create shared library symlinks to ln.
-	  * Only enable epoll in pjproject if `uname -s` is Linux.
-	  * Added a patch to pjproject to take the name of the 'make' command from
-	    an environment variable if supplied.  This is needed for the python bindings.
-	    (merged by Teluu into pjproject trunk 5/3/2016)
-	  FreeBSD support isn't complete.  Still some general issues regarding
-	  make/gmake having nothing to do with pjproject.  With some handholding it DOES
-	  build successfully.
+	  Merged revisions 433395 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  CentOS:
-	  Added 'patch' and 'bzip2' to install_prereq PACKAGES_RH.
-	  CentOS 6/7 32/64 build and run the pjsip testsuite successfully.
+	  Merged revisions 433396 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Ubuntu:
-	  No changes required.
-	  Ubuntu 15/16 32/64 build and run the pjsip testsuite successfully.
 
-	  Debian:
-	  No changes required.
-	  Debian 6/7/8 32/64 build and run the pjsip testsuite successfully.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433397 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  There will utimately be a follow-up patch to create an install_prereq for
-	  the testsuite as I've discovered a few missing requirements.
+2015-03-25 07:32 +0000 [abf3e40902]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25968 #close
+	* dns: Add core DNS API + unit tests and res_resolver_unbound module + unit tests.
 
-	  Change-Id: I5756a07facfc63798115a5e73a8709382fe9259c
+	  This change adds an abstracted core DNS API which resembles the API described
+	  here[1]. The API provides a pluggable mechanism for resolvers and also a
+	  consistent view for records. Both synchronous and asynchronous queries are
+	  supported.
 
-2016-03-17 14:29 +0000 [8028fc7585]  Andrew Nagy <andrew.nagy@the159.com>
+	  This change also adds a res_resolver_unbound module which uses the libunbound
+	  library to provide resolution.
 
-	* app_voicemail: always copy dynamic struct to avoid race condition
+	  Unit tests have also been written for all of the above to confirm the API and
+	  functionality.
 
-	  Voicemail email addresses can be corrupt or voicemail
-	  emails can end up being sent to the wrong email address if asterisk is
-	  reading voicemail.conf during a reload and processing an email at the
-	  same time. This patch always copies the struct that would otherwise only
-	  be copied once.
+	  ASTERISK-24834 #close
+	  Reported by: Matt Jordan
 
-	  ASTERISK-24463 #close
-	  Reported by: John Campbell
-	  Tested by: Etienne Lessard
-	  Tested by: Andrew Nagy
-	  Change-Id: I3a0643813116da84e2617291903d0d489b7425fb
+	  ASTERISK-24836 #close
+	  Reported by: Matt Jordan
 
-2016-04-15 14:26 +0000 [3cb8934de0]  Alexei Gradinari <alex2grad@gmail.com>
+	  Review: https://reviewboard.asterisk.org/r/4474/
+	  Review: https://reviewboard.asterisk.org/r/4512/
 
-	* pjsip: Added "reg_server" to contacts.
+	  [1] https://wiki.asterisk.org/wiki/display/AST/Asterisk+DNS+API
 
-	  If the Asterisk system name is set in asterisk.conf, it will be stored
-	  into the "reg_server" field in the ps_contacts table to facilitate
-	  multi-server setups.
 
-	  ASTERISK-25931
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433370 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ia8f6bd2267809c78753b52bcf21835b9b59f4cb8
+2015-03-24 14:41 +0000 [4c2fc5b811]  Richard Mudgett <rmudgett@digium.com>
 
-2016-04-28 11:35 +0000 [7992923c70]  Richard Mudgett <rmudgett@digium.com>
+	* chan_pjsip: Add "rpid_immediate" option to prevent unnecessary "180 Ringing" messages.
 
-	* res_pjsip: Start body generator users after suppliers.
+	  Incoming PJSIP call legs that have not been answered yet send unnecessary
+	  "180 Ringing" or "183 Progress" messages every time a connected line
+	  update happens.  If the outgoing channel is also PJSIP then the incoming
+	  channel will always send a "180 Ringing" or "183 Progress" message when
+	  the outgoing channel sends the INVITE.
 
-	  Change-Id: I8f0b57841feaab56c8a4e821b5ccb4e05e5fbadb
+	  Consequences of these unnecessary messages:
+
+	  * The caller can start hearing ringback before the far end even gets the
+	  call.
+
+	  * Many phones tend to grab the first connected line information and refuse
+	  to update the display if it changes.  The first information is not likely
+	  to be correct if the call goes to an endpoint not under the control of the
+	  first Asterisk box.
+
+	  When connected line first went into Asterisk in v1.8, chan_sip received an
+	  undocumented option "rpid_immediate" that defaults to disabled.  When
+	  enabled, the option immediately passes connected line update information
+	  to the caller in "180 Ringing" or "183 Progress" messages as described
+	  above.
+
+	  * Added "rpid_immediate" option to prevent unnecessary "180 Ringing" or
+	  "183 Progress" messages.  The default is "no" to disable sending the
+	  unnecessary messages.
+
+	  ASTERISK-24781 #close
+	  Reported by: Richard Mudgett
 
-2016-04-28 16:06 +0000 [5dc0e082b2]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4473/
+	  ........
 
-	* res_pjsip_pubsub.c: Add useful information to some messages.
+	  Merged revisions 433338 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ia0b2e15773894c599e5c5748bbc70e99f434192a
 
-2016-04-26 15:58 +0000 [f9e416f053]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433339 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_pubsub.c: Fix body generator registration race.
+2015-03-22 19:05 +0000 [60f01520e7]  snuffy <snuffy22@gmail.com> (License 5024)
 
-	  Change-Id: Id8752073ef06472a2fd96080f4009fac42843e67
+	* Fix compilations errors on 64-bit OpenBSD systems
 
-2016-04-26 15:13 +0000 [b1b2019046]  Richard Mudgett <rmudgett@digium.com>
+	  In versiong 5.5, OpenBSD went to 64-bit time values. This requires a cast to
+	  (long) when printing members of certain time structs.
 
-	* res_pjsip_pubsub.h: Fix doxygen association.
+	  Review: https://reviewboard.asterisk.org/r/4507
 
-	  Change-Id: I110d3e3572598289fcd4215d966cf0c858f98632
+	  ASTERISK-24879 #close
+	  Reported by: snuffy
+	  Tested by: snuffy
+	  patches:
+	    openbsd-time64.diff uploaded by snuffy (License 5024)
+	  ........
 
-2016-04-25 16:00 +0000 [b7f07fdff5]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 433268 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_pjsip_outbound_publish.c: Remove redundant flag check.
+	  Merged revisions 433269 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I0da80a3c3e0eae0c52ff27e7412ba027d6f52353
 
-2016-04-28 16:54 +0000 [719ece5659]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433270 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjproject_bundled:  Disable PJSIP_UNESCAPE_IN_PLACE
+2015-03-22 18:11 +0000 [66670f02e6]  snuffy <snuffy22@gmail.com> (License 5024)
 
-	  When pjsip_parse_uri is called with PJSIP_UNESCAPE_IN_PLACE enabled,
-	  the input uri string will become corrupted if it contains escape sequences.
-	  It's not possible to automatically strdup or strdupa the input string because
-	  the output uri pj_str_t's will have pointers to chunks of the input string.
-	  Getting around this would require more memory management code and wouldn't
-	  be worth the savings of doing the unescape in place.
+	* Fix compilation issues for OpenBSD
 
-	  ASTERISK-25970 #close
-	  Reported-by: Dmitriy Serov
+	  This patch addresses compilation issues for OpenBSD. Specifically, it
+	  addresses:
+	   * It allows including <sys/vmmeter.h> in asterisk.c
+	   * Provides a needed (size_t) cast in xmldoc.c
 
-	  Change-Id: I28dc0e599b5108f7959b9c46dc8278371b372f88
+	  In 13+, it also addresses a conditional inclusion in loader.c.
 
-2016-03-07 18:34 +0000 [38bed4515d]  gtjoseph <george.joseph@fairview5.com>
+	  Review: https://reviewboard.asterisk.org/r/4506
 
-	* res_pjsip:  Add ability to identify by Authorization username
+	  ASTERISK-24880 #close
+	  Reported by: snuffy
+	  Tested by: snuffy
+	  patches:
+	    misc-openbsd.diff uploaded by snuffy (License 5024)
+	  ........
 
-	  A feature of chan_sip that service providers relied upon was the ability to
-	  identify by the Authorization username.  This is most often used when customers
-	  have a PBX that needs to register rather than identify by IP address.  From my
-	  own experiance, this is pretty common with small businesses who otherwise
-	  don't need a static IP.
+	  Merged revisions 433245 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  In this scenario, a register from the customer's PBX may succeed because From
-	  will usually contain the PBXs account id but an INVITE will contain the caller
-	  id.  With nothing recognizable in From, the service provider's Asterisk can
-	  never match to an endpoint and the INVITE just stays unauthorized.
+	  Merged revisions 433247 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The fixes:
 
-	  A new value "auth_username" has been added to endpoint/identify_by that
-	  will use the username and digest fields in the Authorization header
-	  instead of username and domain in the the From header to match an endpoint,
-	  or the To header to match an aor.  This code as added to
-	  res_pjsip_endpoint_identifier_user rather than creating a new module.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433248 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Although identify_by was always a comma-separated list, there was only
-	  1 choice so order wasn't preserved.  So to keep the order, a vector was added
-	  to the end of ast_sip_endpoint.  This is only used by res_pjsip_registrar
-	  to find the aor.  The res_pjsip_endpoint_identifier_* modules are called in
-	  globals/endpoint_identifier_order.
+2015-03-20 14:54 +0000 [7e097bce86]  Richard Mudgett <rmudgett@digium.com>
 
-	  Along the way, the logic in res_pjsip_registrar was corrected to match
-	  most-specific to least-specific as res_pjsip_endpoint_identifier_user does.
+	* Audit ast_pjsip_rdata_get_endpoint() usage for ref leaks.
 
-	  The order is:
+	  Valgrind found some memory leaks associated with
+	  ast_pjsip_rdata_get_endpoint().  The leaks would manifest when sending
+	  responses to OPTIONS requests, processing MESSAGE requests, and
+	  res_pjsip supplements implementing the incoming_request callback.
 
-	  username@domain
-	  username@domain_alias
-	  username
+	  * Fix ast_pjsip_rdata_get_endpoint() endpoint ref leaks in
+	  res/res_pjsip.c:supplement_on_rx_request(),
+	  res/res_pjsip/pjsip_options.c:send_options_response(),
+	  res/res_pjsip_messaging.c:rx_data_to_ast_msg(), and
+	  res/res_pjsip_messaging.c:send_response().
 
-	  Auth by username does present 1 problem however, the first INVITE won't have
-	  an Authorization header so the distributor, not finding a match on anything,
-	  sends a securty_alert.  It still sends a 401 with a challenge so the next
-	  INVITE will have the Authorization header and presumably succeed.  As a result
-	  though, that first security alert is actually a false alarm.
+	  * Eliminated RAII_VAR() use with ast_pjsip_rdata_get_endpoint() in
+	  res/res_pjsip_nat.c:nat_on_rx_message().
 
-	  To address this, a new feature has been added to pjsip_distributor that keeps
-	  track of unidentified requests and only sends the security alert if a
-	  configurable number of unidentified requests come from the same IP in a
-	  configurable amout of time.  Those configuration options have been added to
-	  the global config object.  This feature is only used when auth_username
-	  is enabled.
+	  * Fixed inconsistent but benign return value in
+	  res/res_pjsip/pjsip_options.c:options_on_rx_request().
 
-	  Finally, default_realm was added to the globals object to replace the hard
-	  coded "asterisk" used when an endpoint is not yet identified.
+	  Review: https://reviewboard.asterisk.org/r/4511/
+	  ........
 
-	  The testsuite tests all pass but new tests are forthcoming for this new
-	  feature.
+	  Merged revisions 433222 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25835 #close
-	  Reported-by: Ross Beer
 
-	  Change-Id: I30ba62d208e6f63439600916fcd1c08a365ed69d
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433223 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-27 13:23 +0000 [677d5b5151]  Mark Michelson <mmichelson@digium.com>
+2015-03-20 13:27 +0000 [148e8799fe]  Richard Mudgett <rmudgett@digium.com>
 
-	* func_odbc: Check connection status before executing queries.
+	* res_pjsip_sdp_rtp,sorcery: Fix invalid access and memory leak respectively.
 
-	  A recent change to func_odbc made it so that a single connection was
-	  maintained per DSN. The problem was that the code was optimistic about
-	  the health of the connection after initially opening it and did nothing
-	  to re-connect in case the connection had died.
+	  Valgrind found a memory leak and invalid access.
 
-	  This change adds a check before executing a query to ensure that the
-	  connection to the database is still up and running.
+	  * Fix invalid access by sscanf() being fed a non-nul terminated string of
+	  digits in res/res_pjsip_sdp_rtp.c:get_codecs().
 
-	  ASTERISK-25963 #close
-	  Reported by Ross Beer
+	  * Fix memory leak in main/sorcery.c:sorcery_object_field_destructor().
 
-	  Change-Id: Id33c86eb04ff48ca088bb2e3086c27b3b683491d
+	  * Fix potential NULL pointer dereference in
+	  main/xmldoc.c:xmldoc_get_syntax_config_option().
 
-2016-04-15 11:59 +0000 [df3639700a]  Alexei Gradinari <alex2grad@gmail.com>
+	  Review: https://reviewboard.asterisk.org/r/4513/
+	  ........
 
-	* res_pjsip: disable multi domain to improve realtime performace
+	  Merged revisions 433199 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch added new global pjsip option 'disable_multi_domain'.
-	  Disabling Multi Domain can improve Realtime performance by reducing
-	  number of database requests.
 
-	  ASTERISK-25930 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433200 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I2e7160f3aae68475d52742107949a799aa2c7dc7
+2015-03-19 14:20 +0000 [627cc16a8d]  Matt Jordan <mjordan@digium.com>
 
-2016-04-26 11:13 +0000 [949bf6b282]  Joshua Colp <jcolp@digium.com>
+	* funcs/func_env: Fix regression caused in FILE read operation
 
-	* chan_sip: Give more time for TCP/TLS threads to stop.
+	  When r432935 was merged, it did correctly fix a situation where a FILE read
+	  operation on the middle of a file buffer would not read the requested length
+	  in the parameters passed to the FILE function. Unfortunately, it would also
+	  allow the FILE function to append more bytes than what was available in the
+	  buffer if the length exceeded the end of the buffer length.
 
-	  The unload process currently tells each TCP/TLS to terminate but
-	  does not wait for them to do so. This introduces a race condition
-	  where the container holding the threads may be destroyed before
-	  the threads are able to remove themselves from it. When they
-	  finally do the container is invalid and can't be used causing a
-	  crash.
+	  This patch takes the minimum of the remaining bytes in the buffer along with
+	  the calculated length to append provided by the original patch, and uses
+	  that as the length to append in the return result. This patch also updates
+	  the unit tests with the scenarios that were originally pointed out in
+	  ASTERISK-21765 that the original implementation treated incorrectly.
 
-	  A previous change existed which waited a bit to wait for any
-	  stranglers to finish. This change extends this and waits longer.
+	  ASTERISK-21765
+	  ........
 
-	  ASTERISK-25961 #close
+	  Merged revisions 433173 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Idc6262b670ca49ede32061159e323b7b63c6f3c6
+	  Merged revisions 433174 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-26 05:48 +0000 [6959f5484b]  Joshua Colp <jcolp@digium.com>
 
-	* app_queue: Fix crash when unloading module.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433175 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When unloading the app_queue module the members in each queue are
-	  destroyed and as part of this they are removed from the pending
-	  members container. Unfortunately a crash would occur as the container
-	  was destroyed before the members were removed.
+2015-03-19 10:27 +0000 [79a81fed59]  Kevin Harwell <kharwell@digium.com>
 
-	  This change tweaks ordering so the container destruction occurs
-	  after the members are destroyed.
+	* alemebic scripts: endpoint identifier order option
 
-	  ASTERISK-16115
+	  The script was added in 13, but when committed to trunk it caused a branch to
+	  occur due to some trunk only alemebic changes. This fixes it so that the new
+	  'add_pjsip_endpoint_identifier_order script points to the correct down revision.
 
-	  Change-Id: I48c728668c55aee3d05b751a5d450fb57e87f44b
 
-2016-04-24 22:51 +0000 [b38f1146e5]  gtjoseph <gjoseph@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433152 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* config:  Fix ast_config_text_file_save2 writability check for missing files
+2015-03-19 05:21 +0000 [3aa0a869c2]  Corey Farrell <git@cfware.com>
 
-	  A patch I did back in 2014 modified ast_config_text_file_save2 to check the
-	  writability of the main file and include files before truncating and re-writing
-	  them.  An unintended side-effect of this was that if a file doesn't exist,
-	  the check fails and the write is aborted.
+	* logger: Apply default console logging when configuration cannot be loaded.
 
-	  This patch causes ast_config_text_file_save2 to check the writability of the
-	  parent directory of missing files instead of checking the file itself.  This
-	  allows missing files to be created again.  A unit test was also added to
-	  test_config to test saving of config files.
+	  When logger.conf is missing or invalid enable console logging and display
+	  an error message.
 
-	  The regression was discovered when app_voicemail's passwordlocation=spooldir
-	  feature stopped working.
+	  ASTERISK-24817 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4497/
+	  ........
 
-	  ASTERISK-25917 #close
-	  Reported-by: Jonathan Rose
+	  Merged revisions 433122 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ic4dbe58c277a47b674679e49daed5fc6de349f80
+	  Merged revisions 433126 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-21 23:53 +0000 [29bab0d1a4]  Kirill Katsnelson <kkm@smartaction.com>
 
-	* chan_sip: Make autocreated peers send PeerStatus events
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433130 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Since Stasis has been introduced, an attempt to send AMI messages by an
-	  autocreated peer caused a crash, and all events from autocreated peers were
-	  semi-inadvertently disabled altogether in 0b83761. This change restores the
-	  disabled functionality.
+2015-03-19 04:57 +0000 [d486659502]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25950
+	* chan_sip: Simplify dialog/peer references, improve REF_DEBUG output.
 
-	  Change-Id: Iecc350f23db603fadb2f302064643ebe9664e974
+	  * Replace functions for ref/undef of dialogs and peers with macro's
+	    to call ao2_t_bump/ao2_t_cleanup.
+	  * Enable passthough of REF_DEBUG caller information to sip_alloc and
+	    find_call.
 
-2016-04-21 14:23 +0000 [c345e530f4]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24882 #close 
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4189/
+	  ........
 
-	* app_queue: queue members can receive multiple calls
+	  Merged revisions 433115 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  It was possible for a queue member that is a member of at least 2 or more
-	  queues to receive mulitiple calls at the same time. This happened because
-	  of a race between when a member was being rung and when the device state
-	  notified the other queue(s) member object of the state change.
 
-	  This patch makes it so when a queue member is being rung it gets added to
-	  a global pool of queue members. If that same member is tried again, e.g.
-	  from another queue, and it is found to already exist in the pending member
-	  container then it will not ring that member.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433116 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-16115 #close
+2015-03-19 04:46 +0000 [2c83ac4364]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I546dd474776d158c2b6be44205353dee5bac7e48
+	* chan_sip: Fix dialog reference leaked to scheduler for reinvite_timeout.
 
-2016-04-25 08:11 +0000 [c0688a6398]  Javier Acosta <javier.acosta@beeonline.es>
+	  Release the scheduler reference to the dialog for reinvite timeout during
+	  dialog_unlink_all.
 
-	* Fix case sensitive actions in AMI QueueSummary and QueueStatus
+	  ASTERISK-24876 #close 
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4491/
+	  ........
 
-	  ASTERISK-25954 #close
-	  Reported by: Javier Acosta
+	  Merged revisions 433112 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I00be83d45cc7e8385de2523012bd196aafeeb256
+	  Merged revisions 433113 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-22 15:25 +0000 [ebf0724a83]  Richard Mudgett <rmudgett@digium.com>
 
-	* test_message.c: Wait longer in case dialplan also processes the test message.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433114 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Bumped the wait from 1 second to 5 seconds.  The test message was hitting my
-	  default call handler and failing the test because it took longer.
+2015-03-17 21:42 +0000 [e0ea490a11]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I3a03737f25e92983de00548fcc7bbc50dd7544ba
+	* res_pjsip_session: Fix off-nominal extra unref of session.
+	  ........
 
-2016-04-12 15:29 +0000 [ba63aa7c9e]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 433088 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Manager: Short circuit AMI message processing.
 
-	  Improve AMI message processing performance if there are no consumers
-	  listening for the messages.  We now skip creating the AMI event message
-	  text strings.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433089 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I7b22fc5ec4e500d00635c1a467aa8ea68a1bb2b3
+2015-03-17 17:15 +0000 [8c65c9167e]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2016-04-13 17:54 +0000 [d5ee6acf28]  Richard Mudgett <rmudgett@digium.com>
+	* Various: bugfixes found via chaos
 
-	* manager.c: Eliminate most RAII_VAR usage.
+	  Using DEBUG_CHAOS several instances of a null
+	  pointer crash, and one uninitialized variable
+	  were uncovered and fixed.  Also added details
+	  on why Asterisk failed to initialize.
 
-	  * Made ast_manager_event_blob_create() not allocate the ao2 event object
-	  with a lock as it is not needed.
+	  Review: https://reviewboard.asterisk.org/r/4468/
+	  ........
 
-	  Change-Id: I8e11bfedd22c21316012e0b9dd79f5918f644b7c
+	  Merged revisions 433064 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-13 17:09 +0000 [7303e3dc96]  Richard Mudgett <rmudgett@digium.com>
 
-	* manager_channels.c: Fix allocation failure crash.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433065 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  An earlier allocation failure failed to create a channel snapshot for the
-	  AMI HangupRequest/SoftHangupRequest event which resulted in a crash in
-	  channel_hangup_request_cb().  Where the stasis message gets generated
-	  cannot tell if the NULL snapshot returned was because of an allocation
-	  failure or the channel was a dummy channel.
+2015-03-17 17:03 +0000 [f25b265329]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  * Made channel_hangup_request_cb() check if the channel blob has a
-	  snapshot and exit if it doesn't.
+	* core: Introduce chaos into memory allocations
 
-	  * Eliminated the RAII_VAR usage in channel_hangup_request_cb().
+	  Locate potential crashes by exercising seldom
+	  used code paths.  This patch introduces a new
+	  define DEBUG_CHAOS, and mechanism to randomly
+	  return an error condition from functions that
+	  will seldom do so.  Functions that handle the
+	  allocation of memory get the first treatment.
 
-	  Change-Id: I0b6a1c4e95cbb7d80b2a7054c6eadecc169dfd24
+	  Review: https://reviewboard.asterisk.org/r/4463/
+	  ........
 
-2016-04-13 13:50 +0000 [1e93f3d723]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 433060 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Bridge system: Fix memory leaks and double frees on impart failure.
 
-	  You cannot reference the passed in features struct after calling
-	  ast_bridge_impart().  Even if the call fails.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433063 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I902b88ba0d5d39520e670fb635078a367268ea21
+2015-03-17 17:03 +0000 [62cf2a2c02]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2016-04-13 13:20 +0000 [5e388d4188]  Richard Mudgett <rmudgett@digium.com>
+	* Reverting accidental ci of wrong change in r433061
 
-	* bridge_softmix.c: Fix crash if channel fails to join mixing tech.
 
-	  softmix_bridge_join() failed because of an allocation failure.  To address
-	  this, the softmix bridge technology now checks if the channel failed to
-	  join softmix successfully.  In addition, the bridge now begins the process
-	  of kicking the channel out of the bridge so we don't have channels
-	  partially in the bridge for very long.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433062 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Fix the test_channel_feature_hooks.c unit tests.  The test channel must
-	  have a valid codec to join the simple_bridge technology.  This patch makes
-	  joining a bridge more strict by not allowing partially joined channels to
-	  remain in the bridge.
+2015-03-17 17:00 +0000 [cb6c7eecfd]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  Change-Id: I97e2ade6a2bcd1214f24fb839fda948825b61a2b
+	* various: cleanup issues found during leak hunt
 
-2016-03-17 12:28 +0000 [9740277713]  gtjoseph <george.joseph@fairview5.com>
+	  In this collection of small patches to prevent
+	  Valgrind errors are: fixes for reference leaks
+	  in config hooks, evaluating a parameter beyond
+	  bounds, and accessing a structure after a lock
+	  where it could have been already free'd.
 
-	* res_pjsip:  Add serialized scheduler (res_pjsip/pjsip_scheduler.c)
+	  Review: https://reviewboard.asterisk.org/r/4407/
+	  ........
 
-	  There are several places that do scheduled tasks or periodic housecleaning,
-	  each with its own implementation:
+	  Merged revisions 431583 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * res_pjsip_keepalive has a thread that sends keepalives.
-	  * pjsip_distributor has a thread that cleans up expired unidentified requests.
-	  * res_pjsip_registrar_expire has a thread that cleans up expired contacts.
-	  * res_pjsip_pubsub uses ast_sched directly and then calls ast_sip_push_task.
-	  * res_pjsip_sdp_rtp also uses ast_sched to send keepalives.
 
-	  There are also places where we should be doing scheduled work but aren't.
-	  A good example are the places we have sorcery observers to start registration
-	  or qualify.  These don't work when changes are made to a backend database
-	  without a pjsip reload.  We need to check periodically.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433061 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  As a first step to solving these issues, a new ast_sip_sched facility has
-	  been created.
+2015-03-17 16:52 +0000 [c41dd32b94]  Richard Mudgett <rmudgett@digium.com>
 
-	  ast_sip_sched wraps ast_sched but only uses ast_sched as a scheduled queue.
-	  When a task is ready to run, ast_sip_task_pusk is called for it. This ensures
-	  that the task is executed in a PJLIB registered thread and doesn't hold up the
-	  ast_sched thread so it can immediately continue processing the queue.  The
-	  serializer used by ast_sip_sched is one of your choosing or a random one from
-	  the res_pjsip pool if you don't choose one.
+	* Audit ast_sockaddr_resolve() usage for memory leaks.
 
-	  Another feature is the ability to automatically clean up the task_data when the
-	  task expires (if ever).  If it's an ao2 object, it will be dereferenced, if
-	  it's a malloc'd object it will be freed.  This is selectable when the task is
-	  scheduled.  Even if you choose to not auto dereference an ao2 task data object,
-	  the scheduler itself maintains a reference to it while the task is under it's
-	  control.  This prevents the data from disappearing out from under the task.
+	  Valgrind found some memory leaks associated with ast_sockaddr_resolve().
+	  Most of the leaks had already been fixed by earlier memory leak hunt
+	  patches.  This patch performs an audit of ast_sockaddr_resolve() and found
+	  one more.
 
-	  There are two scheduling models.
+	  * Fix ast_sockaddr_resolve() memory leak in
+	  apps/app_externalivr.c:app_exec().
 
-	  AST_SIP_SCHED_TASK_PERIODIC specifies that the invocations of the task occur at
-	  the specific interval.  That is, every "interval" milliseconds, regardless of
-	  how long the task takes.  If the task takes longer than the interval, it will
-	  be scheduled at the next available multiple of interval.  For exmaple: If the
-	  task has an interval of 60 secs and the task takes 70 secs (it better not),
-	  the next invocation will happen at 120 seconds.
+	  * Made main/netsock2.c:ast_sockaddr_resolve() always set the addrs
+	  parameter for safety so the pointer will never be uninitialized on return.
+	  The same goes for res/res_pjsip_acl.c:extract_contact_addr().
 
-	  AST_SIP_SCHED_TASK_DELAY specifies that the next invocation of the task should
-	  start "interval" milliseconds after the current invocation has finished.
+	  * Made functions that call ast_sockaddr_resolve() with RAII_VAR()
+	  controlling the addrs variable use ast_free instead of ast_free_ptr to
+	  provide better MALLOC_DEBUG information.
 
-	  Also, the same ast_sched facility for fixed or variable intervals exists.  The
-	  task's return code in conjunction with the AST_SIP_SCHED_TASK_FIXED or
-	  AST_SIP_SCHED_TASK_VARIABLE flags controls the next invocation start time.
+	  Review: https://reviewboard.asterisk.org/r/4509/
+	  ........
 
-	  One res_pjsip.h housekeeping change was made.  The pjsip header files were
-	  added to the top.  There have been a few cases lately where I've needed
-	  res_pjsip.h just for ast_sip calls and had compiles fail spectacularly because
-	  I didn't add the pjsip header files to my source even though I never referenced
-	  any pjsip calls.
+	  Merged revisions 433056 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Finally, a few new convenience APIs were added to astobj2 to make things a
-	  little easier in the scheduler.  ao2_ref_and_lock() calls ao2_ref() and
-	  ao2_lock() in one go.  ao2_unlock_and_unref() does the reverse. A few macros
-	  were also copied from res_phoneprov because I got tired of having to duplicate
-	  the same hash, sort and compare functions over and over again. The
-	  AO2_STRING_FIELD_(HASH|SORT|CMP)_FN macros will insert functions suitable for
-	  aor_container_alloc into your source.
+	  Merged revisions 433057 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This facility can be used immediately for the situations where we already have
-	  a thread that wakes up periodically or do some scheduled work.  For the
-	  registration and qualify issues, additional sorcery and schema changes would
-	  need to be made so that we can easily detect changed objects on a periodic
-	  basis without having to pull the entire database back to check.  I'm thinking
-	  of a last-updated timestamp on the rows but more on this later.
 
-	  Change-Id: I7af6ad2b2d896ea68e478aa1ae201d6dd016ba1c
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433058 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-25 21:43 +0000  Asterisk Development Team <asteriskteam@digium.com>
+2015-03-17 13:35 +0000 [803a916334]  Kevin Harwell <kharwell@digium.com>
 
-	* asterisk 13.9.0-rc1 Released.
+	* res_pjsip: Allow configuration of endpoint identifier query order
 
-2016-04-25 16:42 +0000 [5237b7cd47]  Joshua Colp <jcolp@digium.com>
+	  Updated some documentation stating that endpoint identifiers registered without
+	  a name are place at the front of the lookup list. Also renamed register method
+	  'ast_sip_register_endpoint_identifier_by_name' to
+	  'ast_sip_register_endpoint_identifier_with_name'
 
-	* Release summaries: Add summaries for 13.9.0-rc1
+	  ASTERISK-24840
+	  Reported by: Mark Michelson
+	  ........
 
-2016-04-25 16:40 +0000 [5a3850ecba]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 433031 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* .version: Update for 13.9.0-rc1
 
-2016-04-25 16:40 +0000 [60b39040e4]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433032 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* .lastclean: Update for 13.9.0-rc1
+2015-03-17 13:22 +0000 [aef7278af6]  Kevin Harwell <kharwell@digium.com>
 
-2016-04-25 16:40 +0000 [5078454464]  Joshua Colp <jcolp@digium.com>
+	* res_pjsip: Allow configuration of endpoint identifier query order
 
-	* realtime: Add database scripts for 13.9.0-rc1
+	  This patch fixes previously reverted code that caused binary incompatibility
+	  problems with some modules. And like the original patch it makes sure that
+	  no matter what order the endpoint identifier modules were loaded, priority is
+	  given based on the ones specified in the new global 'endpoint_identifier_order'
+	  option.
 
-2016-04-22 17:53 +0000 [eb7c581806]  gtjoseph <gjoseph@digium.com>
+	  ASTERISK-24840
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4489/
+	  ........
 
-	* res_agi:  Prevent run_agi from eating frames it shouldn't
+	  Merged revisions 433028 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The run_agi function is eating control frames when it shouldn't be. This is
-	  causing issues when an AGI is run from CONNECTED_LINE_SEND_SUB in a blond
-	  transfer.
 
-	  Alice calls Bob. Bob attended transfers to Charlie but hangs up before Charlie
-	  answers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433029 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Alice gets the COLP UPDATE indicating Charlie but Charlie never gets an UPDATE
-	  and is left thinking he's connected to Bob.
+2015-03-17 11:11 +0000 [259e833e88]  Richard Mudgett <rmudgett@digium.com>
 
-	  In this case, when CONNECTED_LINE_SEND_SUB runs on Alice's channel and it calls
-	  an AGI, the extra eaten frames prevent CONNECTED_LINE_SEND_SUB from running on
-	  Charlie's channel.
+	* res_pjsip: Add reason comment.
+	  ........
 
-	  The fix was to accumulate deferrable frames in the "forever" loop instead of
-	  dropping them, and re-queue them just before running the actual agi command
-	  or exiting.
+	  Merged revisions 433005 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25951 #close
 
-	  Change-Id: I0f4bbfd72fc1126c2aaba41da3233a33d0433645
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@433006 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-22 13:49 +0000 [068ae54c76]  Mark Michelson <mmichelson@digium.com>
+2015-03-13 21:29 +0000 [e89f83b3ad]  Matt Jordan <mjordan@digium.com>
 
-	* func_odbc: Use one connection per DSN.
+	* main/frame: Don't report empty disallow values as an error
 
-	  res_odbc was changed in Asterisk 13.8.0 to remove connection management,
-	  opting instead to let unixodbc maintain open connections and return
-	  those to Asterisk as requested.
+	  In realtime, it is normal to have a database with both 'allow' and 'disallow'
+	  columns in the schema. It is perfectly valid to have an 'allow' value of
+	  '!all,g722,ulaw,alaw' and no 'disallow' value. Unlike in static conf files,
+	  you can't *not* provide the disallow value. Thus, the empty disallow value
+	  causes a spurious WARNING message, which is kind of annoying.
 
-	  This was a boon for realtime, since it meant that multiple threads could
-	  potentially run parallel queries since they could each be using their
-	  own database connections.
+	  This patch makes it so that a 'disallow' value with no ... value ... is
+	  ignored. Granted, you can still screw this up as well, as technically
+	  specifying 'disallow=all,!ulaw' allows only ulaw, and then you would have no
+	  'allow' value in your database. But really, why would you do that? WHY?
 
-	  However, on the user-facing side, func_odbc, there were some inherent
-	  behaviors being relied on that no longer hold true after the change.
-	  One such reported behavior was that MySQL's LAST_INSERTED_ID() works
-	  per-connection. This means that if Asterisk uses separate connections
-	  for every database operation, whereas before it used one connection for
-	  everything, we have broken expectations and functionality.
+	  ASTERISK-16779 #close
+	  Reported by: Atis Lezdins
+	  ........
 
-	  The fix provided in this patch is to make func_odbc use a single
-	  database connection per DSN. This way, user-facing database usage will
-	  have the same behavior as it did pre-13.8.0. However, realtime, which is
-	  the real workhorse of database interaction, will continue to let
-	  unixodbc manage connections.
+	  Merged revisions 432970 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25938 #close
-	  Reported by Edwin Vandamme
+	  Merged revisions 432971 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Iac961fe79154c6211569afcdfec843c0c24c46dc
 
-2016-04-22 13:02 +0000 [6aeefa89bc]  Leif Madsen <leif@leifmadsen.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432972 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Remove reference to non-existent sip.conf option
+2015-03-13 21:01 +0000 [0d52907d2b]  Joshua Colp <jcolp@digium.com>
 
-	  Option was removed in commit 7f883ef495b57ae9182e47213d01d5e8009dbf3f
+	* func_curl: Don't hold exclusive lock when performing HTTP request.
 
-	  ASTERISK-25927 #close
+	  This code originally kept a lock held when performing the HTTP
+	  request to ensure that the options provided to curl remain valid.
+	  This doesn't seem to be necessary these days and holding the lock
+	  caused requests to happen sequentially instead of in parallel.
 
-	  Change-Id: I92f9b0196d9fc41d1d58354c07340c465ef1fcf8
+	  ASTERISK-18708 #close
+	  Reported by: Dave Cabot
+	  ........
 
-2016-04-21 08:26 +0000 [e750ea9b5b]  Diederik de Groot <dkgroot@talon.nl>
+	  Merged revisions 432948 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* lock.c: Check *lt before dereferencing it
+	  Merged revisions 432949 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  *lt is NULL if t->tracking == 0
 
-	  ASTERISK-25948 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432950 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I4a81af28f9c82a74aa82413d772a7dc8fa6f45ba
+2015-03-13 20:53 +0000 [ac1214d9d4]  Jan Juergens (License 6538)
 
-2016-04-15 14:36 +0000 [a036c35903]  Richard Mudgett <rmudgett@digium.com>
+	* apps/app_sms: Add an option to prevent SMS content from being logged
 
-	* res_stasis: Handle re-enter stasis bridge with swap channel.
+	  In some countries, privacy laws specify that SMS content cannot be saved by a
+	  provider. This patch adds a new option to the SMS application, 'n', which
+	  prevents the SMS content from being written to the SMS log.
 
-	  We lose the fact that there is a swap channel if there is one.  We
-	  currently wind up rejoining the stasis bridge as a normal join after the
-	  swap channel has already been kicked from the bridge.
+	  ASTERISK-22591 #close
+	  Reported by: Jan Juergens
+	  patches:
+	    DisableSmsContentLoggingByParam.patch uploaded by Jan Juergens (License 6538)
 
-	  This patch preserves the swap channel so the AMI/ARI events can note that
-	  the channel joining the bridge is swapping with another channel.  Another
-	  benefit to swaqpping in one operation is if there are any channels that
-	  get lonely (MOH, bridge playback, and bridge record channels).  The lonely
-	  channels won't leave before the joining channel has a chance to come back
-	  in under stasis if the swap channel is the only reason the lonely channels
-	  are staying in the bridge.
 
-	  ASTERISK-25947 #close
-	  Reported by: Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432947 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24649
-	  Reported by: John Bigelow
+2015-03-13 20:37 +0000 [b3fa35786f]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-24782
-	  Reported by: John Bigelow
+	* core: Fix tab completion of "core set debug channel" CLI command.
 
-	  Change-Id: If37ea508831d1fed6dbfac2f191c638fc0a850ee
+	  The "core set debug channel" CLI command mistakenly had source filenames
+	  added to its tab completion. This occurred because the CLI generator fell back
+	  to the "core set debug" command which permits setting debug at a source
+	  filename level.
 
-2016-04-19 16:58 +0000 [9942d50aa5]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-21038 #close
+	  Reported by: Richard Kenner
+	  ........
 
-	* bridge: Hold off more than one imparting channel at a time.
+	  Merged revisions 432944 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  An earlier patch blocked the ast_bridge_impart() call until the channel
-	  either entered the target bridge or it failed.  Unfortuantely, if the
-	  target bridge is stasis and the imprted channel is not a stasis channel,
-	  stasis bounces the channel out of the bridge to come back into the bridge
-	  as a proper stasis channel.  When the channel is bounced out, that
-	  released the block on ast_bridge_impart() to continue.  If the impart was
-	  a result of a transfer, then it became a race to see if the swap channel
-	  would get hung up before the imparted channel could come back into the
-	  stasis bridge.  If the imparted channel won then everything is fine.  If
-	  the swap channel gets hung up first then the transfer will fail because
-	  the swap channel is leaving the bridge.
+	  Merged revisions 432945 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Allow a chain of ast_bridge_impart()'s to happen before any are
-	  unblocked to prevent the race condition described above.  When the channel
-	  finally joins the bridge or completely fails to join the bridge then the
-	  ast_bridge_impart() instances are unblocked.
 
-	  ASTERISK-25947
-	  Reported by: Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432946 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24649
-	  Reported by: John Bigelow
+2015-03-13 20:22 +0000 [b4cc056067]  Di-Shi Sun (License 5076)
 
-	  ASTERISK-24782
-	  Reported by: John Bigelow
+	* FILE: fix retrieval of file contents when offset is specified
 
-	  Change-Id: I8fef369171f295f580024ab4971e95c799d0dde1
+	  The loop that reads in a file was not correctly using the offset when
+	  determining what bytes to append to the output. This patch corrects
+	  the logic such that the correct portion of the file is extracted when an
+	  offset is specified.
 
-2016-04-19 17:52 +0000 [516c626a7d]  gtjoseph <george.joseph@fairview5.com>
+	  ASTERISK-21765
+	  Reported by: John Zhong
+	  Tested by: Matt Jordan, Di-Shi Sun
+	  patches:
+	    file_read_390821.patch uploaded by Di-Shi Sun (License 5076)
+	  ........
 
-	* res_pjsip_callerid:  Clear out display name if id->name is not valid
+	  Merged revisions 432935 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  When create_new_id_hdr creates a new RPID or PAI header, it starts by cloning
-	  the From header, then it overwrites the display name and uri from the channel's
-	  connected.id.  If the connected.id.name wasn't valid, create_new_id_hdr was
-	  leaving the display name from the From header in the new RPID or PAI header.
-	  On an attended transfer where the originator had a caller id number set but not
-	  a display name, the re-INVITE to the final transferee had the number of the
-	  originator but the display name of the transferer.
+	  Merged revisions 432938 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Added a check to clear out the display name in the new header if
-	  connected.id.name was invalid.
 
-	  ASTERISK-25942 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432940 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I60b4bf7a7ece9b7425eba74151c0b4969cd2738b
+2015-03-13 19:24 +0000 [dc752f515b]  Matt Jordan <mjordan@digium.com>
 
-2016-04-19 13:02 +0000 [ded3794fc6]  Joshua Colp <jcolp@digium.com>
+	* apps/app_amd: Document maximum_word_length option; fix AMDCAUSE documentation
 
-	* app_talkdetect: Make the module core supported.
+	  This patch corrects the documentation for the AMD application. Specifically:
+	  * It documents the maximum_word_length option, which limits the maximum allowed
+	    length of a single utterance.
+	  * It clarifies the AMDCAUSE values MAXWORDS and MAXWORDLENGTH. MAXWORDLENGTH
+	    was documented as MAXWORDS, while MAXWORDS was undocumented.
 
-	  This module is used as part of testsuite tests to confirm
-	  stuff works. I'm accordingly marking it as core as it is
-	  required by those tests.
+	  Thanks to the issue reporter, Frank DiGennaro, for pointing out the issues.
 
-	  Change-Id: I558e7af7679b22b8ed641d7dd37ee4ca35b11e88
+	  ASTERISK-19470 #close
+	  Reported by: Frank DiGennaro
+	  ........
 
-2016-04-18 12:12 +0000 [efae187217]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 432918 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* PJSIP: Remove PJSIP parsing functions from uri length validation.
+	  Merged revisions 432920 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The PJSIP parsing functions provide a nice concise way to check the
-	  length of a hostname in a SIP URI. The problem is that in order to use
-	  those parsing functions, it's required to use them from a thread that
-	  has registered with PJLib.
 
-	  On startup, when parsing AOR configuration, the permanent URI handler
-	  may not be run from a PJLib-registered thread. Specifically, this could
-	  happen when Asterisk was started in daemon mode rather than
-	  console-mode. If PJProject were compiled with assertions enabled, then
-	  this would cause Asterisk to crash on startup.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432921 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The solution presented here is to do our own parsing of the contact URI
-	  in order to ensure that the hostname in the URI is not too long. The
-	  parsing does not attempt to perform a full SIP URI parse/validation,
-	  since the hostname in the URI is what is important.
+2015-03-13 12:06 +0000 [c52adca396]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25928 #close
-	  Reported by Joshua Colp
+	* chan_pjsip: AMI action PJSIPShowEndpoint closes AMI connection on error.
 
-	  Change-Id: Ic3d6c20ff3502507c17244a8b7e2ca761dc7fb60
+	  Also fixed similar problem with AMI action PJSIPShowEndpoints.
 
-2016-04-18 17:00 +0000 [f436b9ab11]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24872 #close
+	  Reported by: Dmitriy Serov
 
-	* res_pjsip_registrar: Fix bad memory-ness with user_agent.
+	  Review: https://reviewboard.asterisk.org/r/4487/
+	  ........
 
-	  Recent changes to the PJSIP registrar resulted in tests failing due to
-	  missing AOR_CONTACT_ADDED test events. The reason for this was that the
-	  user_agent string had junk values in it, resulting in being unable to
-	  generate the event.
+	  Merged revisions 432894 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  I'm going to be honest here, I have no idea why this was happening. Here
-	  are the steps needed for the user_agent variable to get messed up:
-	  * REGISTER is received
-	  * First contact in the REGISTER results in a contact being removed
-	  * Second contact in the REGISTER results in a contact being added
-	  * The contact, AOR, expiration, and user agent all have to be passed as
-	    format parameters to the creation of a string. Any subset of those
-	    parameters would not be enough to cause the problem.
 
-	  Looking into what was happening, the thing that struck me as odd was
-	  that the user_agent variable was meant to be set to the value of the
-	  User-Agent SIP header in the incoming REGISTER. However, when removing a
-	  contact, the user_agent variable would be set (via ast_strdupa inside a
-	  loop) to the stored contact's user_agent. This means that the
-	  user_agent's value would be incorrect when attempting to process further
-	  contacts in the incoming REGISTER.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432895 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The fix here is to use a different variable for the stored user agent
-	  when removing a contact. Correcting the behavior to be correct also
-	  means the memory usage is less weird, and the issue no longer occurs.
+2015-03-13 11:37 +0000 [636d82f4d8]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25929 #close
-	  Reported by Joshua Colp
+	* chan_pjsip/res_pjsip_callerid: Make Party ID handling simpler and consistent.
 
-	  Change-Id: I7cd24c86a38dec69ebcc94150614bc25f46b8c08
+	  The res_pjsip modules were manually checking both name and number
+	  presentation values when there is a function that determines the combined
+	  presentation for a party ID struct.  The function takes into account if
+	  the name or number components are valid while the manual code rarely
+	  checked if the data was even valid.
 
-2016-04-18 13:41 +0000 [49bfdc9ac0]  Joshua Colp <jcolp@digium.com>
+	  * Made use ast_party_id_presentation() rather than manually checking party
+	  ID presentation values.
 
-	* res_pjsip_transport_management: Allow unload to occur.
+	  * Ensure that set_id_from_pai() and set_id_from_rpid() will not return
+	  presentation values other than what is pulled out of the SIP headers.  It
+	  is best if the code doesn't assume that AST_PRES_ALLOWED and
+	  AST_PRES_USER_NUMBER_UNSCREENED are zero.
 
-	  At shutdown it is possible for modules to be unloaded that wouldn't
-	  normally be unloaded. This allows the environment to be cleaned up.
+	  * Fixed copy paste error in add_privacy_params() dealing with RPID
+	  privacy.
 
-	  The res_pjsip_transport_management module did not have the unload
-	  logic in it to clean itself up causing the res_pjsip module to not
-	  get unloaded. As a result the res_pjsip monitor thread kept going
-	  processing traffic and timers when it shouldn't.
+	  * Pulled the id->number.valid test from add_privacy_header() and
+	  add_privacy_params() up into the parent function add_id_headers() to skip
+	  adding PAI/RPID headers earlier.
 
-	  Change-Id: Ic8cadee131e3b2c436a81d3ae8bb5775999ae00a
+	  * Made update_connected_line_information() not send out connected line
+	  updates if the connected line number is invalid.  Lower level code would
+	  not add the party ID information and thus the sent message would be
+	  unnecessary.
 
-2016-04-15 11:41 +0000 [f4693d1897]  Richard Mudgett <rmudgett@digium.com>
+	  * Eliminated RAII_VAR usage in send_direct_media_request().
 
-	* bridge_channel.c: Ignore role setup failure in channel push.
+	  Review: https://reviewboard.asterisk.org/r/4472/
+	  ........
 
-	  We have to setup the channel roles after the bridge class push is called
-	  because the bridge class push callback may have set roles on the incoming
-	  channel.  Since we have already partially pushed the channel into the
-	  bridge and reversing what we have already done could be problematic, the
-	  only thing we can do is press on to complete pushing the channel into the
-	  bridge.
+	  Merged revisions 432892 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Ignore any channel role setup errors after pushing the channel into a
-	  bridge.  The channel may behave incorrectly in the bridge but we can no
-	  longer abort the push at this time.
 
-	  Change-Id: I08a97082b729052ee65cdca6bb730cf1289ede00
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432893 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-17 15:37 +0000 [22335fe18a]  Jaco Kroon <jaco@uls.co.za>
+2015-03-13 09:55 +0000 [d42c6adb1a]  Kevin Harwell <kharwell@digium.com>
 
-	* chan_sip: Don't verify table if rtupdate=no
+	* Revert - res_pjsip: Allow configuration of endpoint identifier query order
 
-	  If rtupdate=no do not verify sipregs/peers table has updatable fields.
+	  Due to a break in binary compatibility with some other modules these changes
+	  are being reverted until the issue can be resolved.
 
-	  ASTERISK-25934 #close
+	  ASTERISK-24840
+	  Reported by: Mark Michelson
+	  ........
 
-	  Change-Id: Iaa2c53037b93daccc7e7333c40d61861847b856d
+	  Merged revisions 432868 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-18 04:53 +0000 [3b9d8b60b2]  ibercom <ibercom123@gmail.com>
 
-	* app_queue: Frequent segfaults in function can_ring_entry()
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432869 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25888 #close
+2015-03-12 21:10 +0000 [f2c21ead1f]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I007a2f2dd99823e04fb5be3ff01f02b0a2956117
+	* Logger: Fix MALLOC_DEBUG build error.
 
-2016-04-15 16:51 +0000 [724acb6ce7]  Richard Mudgett <rmudgett@digium.com>
+	  Revision 432834 introduced a build error when MALLOC_DEBUG
+	  is used.  Switch callid threadstorage to simple
+	  AST_THREADSTORAGE since we no longer need custom cleanup.
 
-	* stasis_bridge.c: Update stasis bridge push diagnostic messages.
+	  Reported by: Corey Farrell
 
-	  Change-Id: I195b14994c9dcccb9452491ca20a885d2a54605a
 
-2016-04-14 13:49 +0000 [5f78801859]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432851 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* transport management: Register thread with PJProject.
+2015-03-12 20:12 +0000 [c08fd275bf]  Corey Farrell <git@cfware.com>
 
-	  The scheduler thread that kills idle TCP connections was not registering
-	  with PJProject properly and causing assertions if PJProject was built in
-	  debug mode.
+	* Logger: Convert 'struct ast_callid' to unsigned int.
 
-	  This change registers the thread with PJProject the first time that the
-	  scheduler callback executes.
+	  Switch logger callid's from AO2 objects to simple integers.
+	  This helps in two ways.  Copying integers is faster than
+	  referencing AO2 objects, so this will result in a small
+	  reduction in logger overhead.  This also erases the possibility
+	  of an infinate loop caused by an invalid callid in
+	  threadstorage.
 
-	  AST-2016-005
+	  ASTERISK-24833 #comment Committed callid conversion to trunk. 
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4466/
 
-	  Change-Id: I5f7a37e2c80726a99afe9dc2a4a69bdedf661283
 
-2016-03-08 12:12 +0000 [7fb3724a77]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432834 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_transport_management: Kill idle TCP connections.
+2015-03-12 07:58 +0000 [38ee441ea7]  Matt Jordan <mjordan@digium.com>
 
-	  "Idle" here means that someone connects to us and does not send a SIP
-	  request. PJProject will not automatically time out such connections, so
-	  it's up to Asterisk to do it instead.
+	* main/audiohook: Update internal sample rate on reads
 
-	  When we receive an incoming TCP connection, we will start a timer
-	  (equivalent to transaction timer D) waiting to receive an incoming
-	  request. If we do not receive a request in that timeframe, then we will
-	  shut down the TCP connection.
+	  When an audiohook is created (which is used by the various Spy applications
+	  and Snoop channel in Asterisk 13+), it initially is given a sample rate of
+	  8kHz. It is expected, however, that this rate may change based on the media
+	  that passes through the audiohook. However, the read/write operations on the
+	  audiohook behave very differently.
 
-	  ASTERISK-25796 #close
-	  Reported by George Joseph
+	  When a frame is written to the audiohook, the format of the frame is checked
+	  against the internal sample rate. If the rate of the format does not match
+	  the internal sample rate, the internal sample rate is updated and a new SLIN
+	  format is chosen based on that sample rate. This works just fine.
 
-	  AST-2016-005
+	  When a frame is read, however, we do something quite different. If the format
+	  rate matches the internal sample rate, all is fine. However, if the rates
+	  don't match, the audiohook attempts to "fix up" the number of samples that
+	  were requested. This can result in some seriously large number of samples
+	  being requested from the read/write factories.
 
-	  Change-Id: I7b0d303e5d140d0ccaf2f7af562071e3d1130ac6
+	  Consider the worst case - 192kHz SLIN. If we attempt to read 20ms worth of
+	  audio produced at that rate, we'd request 3840 samples (192000 / (1000 / 20)).
+	  However, if the audiohook is still expecting an internal sample rate of 8000,
+	  we'll attempt to "fix up" the requested samples to:
 
-2016-03-08 10:52 +0000 [707fd4dcd0]  Mark Michelson <mmichelson@digium.com>
+	    samples_converted = samples * (ast_format_get_sample_rate(format) /
+	                                   (float) audiohook->hook_internal_samp_rate);
 
-	* Rename res_pjsip_keepalive res_pjsip_transport_management
+	    which is:
 
-	  ASTERISK-25796
-	  Reported by George Joseph
+	    92160 = 3840 * (192000 / 8000)
 
-	  AST-2016-005
+	  This results in us attempting to read 92160 samples from our factories, as
+	  opposed to the 3840 that we actually wanted. On a 64-bit machine, this
+	  miraculously survives - despite allocating up to two buffers of length 92160
+	  on the stack. The 32-bit machines aren't quite so lucky. Even in the case where
+	  this works, we will either (a) get way more samples than we wanted; or (b) get
+	  about 3840 samples, assuming the timing is pretty good on the machine.
 
-	  Change-Id: Id322a05f927392293570599730050bc677d99433
+	  Either way, the calculation being performed is wrong, based on the API users
+	  expectations.
 
-2016-04-14 07:15 +0000 [0b4bb19e0b]  Mark Michelson <mmichelson@digium.com>
+	  My first inclination was to allocate the buffers on the heap. As it is,
+	  however, there's at least two drawbacks with doing this:
+	  (1) It's a bit complicated, as the size of the buffers may change during the
+	      lifetime of the audiohook (ew).
+	  (2) The stack is faster (yay); the heap is slower (boo).
 
-	* AST-2016-004: Fix crash on REGISTER with long URI.
+	  Since our calculation is flat out wrong in the first place, this patch fixes
+	  this issue by instead updating the internal sample rate based on the format
+	  passed into the read operation. This causes us to read the correct number of
+	  samples, and has the added benefit of setting the audihook with the right
+	  SLIN format.
 
-	  Due to some ignored return values, Asterisk could crash if processing an
-	  incoming REGISTER whose contact URI was above a certain length.
+	  Note that this issue was caught by the Asterisk Test Suite as a result of
+	  r432195 in the 13 branch. Because this issue is also theoretically possible
+	  in Asterisk 11, the change is being made here as well.
 
-	  ASTERISK-25707 #close
-	  Reported by George Joseph
+	  Review: https://reviewboard.asterisk.org/r/4475/
+	  ........
 
-	  Patches:
-	  	0001-res_pjsip-Validate-that-URIs-don-t-exceed-pjproject-.patch
+	  Merged revisions 432810 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  AST-2016-004
+	  Merged revisions 432811 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I0ed3898fe7ab10121b76c8c79046692de3a1be55
 
-2016-04-12 13:10 +0000 [f6e080c6a4]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432812 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* bridge_softmix.c: Fix crash if could not allocate the dsp.
+2015-03-12 07:40 +0000 [29304d10a0]  Diederik de Groot (License 6600)
 
-	  Fix off nominal crash where we could not setup the channel to process
-	  frames for the softmix bridge technology because of allocation failure.
+	* Add support for the clang compiler; update RAII_VAR to use BlocksRuntime
 
-	  Change-Id: Ic307a8386e46bf551e48fcd1eb97276714d56372
+	  RAII_VAR, which is used extensively in Asterisk to manage reference counted
+	  resources, uses a GCC extension to automatically invoke a cleanup function
+	  when a variable loses scope. While this functionality is incredibly useful
+	  and has prevented a large number of memory leaks, it also prevents Asterisk
+	  from being compiled with clang.
 
-2016-04-12 15:41 +0000 [cf15a2f2d3]  gtjoseph <george.joseph@fairview5.com>
+	  This patch updates the RAII_VAR macro such that it can be compiled with clang.
+	  It makes use of the BlocksRuntime, which allows for a closure to be created
+	  that performs the actual cleanup.
 
-	* pjproject:  Add patch for removing strip of '[]' from header params
+	  Note that this does not attempt to address the numerous warnings that the clang
+	  compiler catches in Asterisk.
 
-	  From the patch submitted to Teluu on 4/12/2016
-	  <<<<<<<<<
-	  The wholesale stripping of '[]' from header parameters causes issues if
-	  something (like a port) occurs after the final ']'.
+	  Much thanks for this patch goes to:
+	  * The folks on StackOverflow who asked this question and Leushenko for
+	    providing the answer that formed the basis of this code:
+	    http://stackoverflow.com/questions/24959440/rewrite-gcc-cleanup-macro-with-nested-function-for-clang
+	  * Diederik de Groot, who has been extremely patient in working on getting this
+	    patch into Asterisk.
 
-	  '[2001:a::b]' will correctly parse to '2001:a::b'
-	  '[2001:a::b]:8080' will correctly parse to '2001:a::b' but the scanner is left
-	  with ':8080' and parsing stops with a syntax error.
+	  Review: https://reviewboard.asterisk.org/r/4370/
 
-	  I can't even find a case where stripping the '[]' is a good thing anyway.  Even
-	  if you continued to parse and resulted in a string that looks like this...
-	  '2001:a::b:8080', it's not valid.
+	  ASTERISK-24133
+	  ASTERISK-23666
+	  ASTERISK-20399
+	  ASTERISK-20850 #close
+	  Reported by: Diederik de Groot
+	  patches:
+	    RAII_CLANG.patch uploaded by Diederik de Groot (License 6600)
+	  ........
 
-	  This came up in Asterisk because Kamailio sends us a Contact with an alias
-	  URI parameter that has an IPv6 address in it like this:
-	  Contact: <sip:1171@127.0.0.1:5080;alias=[2001:1:2::3]~43691~6>
-	  which should be legal but causes a syntax error because of the characters
-	  after the final ']'.  Even if it didn't, the '[]' should still not be stripped.
+	  Merged revisions 432807 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  I've run the Asterisk Test Suite for PJSIP (252 tests) many of which are IPv6
-	  enabled.  No issues were caused by removing the code that strips the '[]'.
-	  >>>>>>>>>>>
+	  Merged revisions 432808 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25123 #close
-	  Reported-by: Anthony Messina
 
-	  Change-Id: I5cb33f4ebf07ee1f2b26d07caae715e2ec65595a
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432809 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-12 09:10 +0000 [daa086fae4]  Joshua Colp <jcolp@digium.com>
+2015-03-11 11:39 +0000 [4115e327ac]  Richard Mudgett <rmudgett@digium.com>
 
-	* app_voicemail: Fix test_voicemail_notify_endl test.
+	* res_pjsip: Move internal init/destroy prototypes to private header file.
 
-	  The test_voicemail_notify_endl test checks the end-of-line
-	  characters of an email message to confirm that they are consistent.
-	  The test wrongfully assumed that reading from the email message
-	  into a buffer will always result in more than 1 character being
-	  read. This is incorrect. If only 1 character was read the test
-	  would go outside of the buffer and access other memory causing
-	  a crash.
+	  Done as a separate commit from a finding in
+	  https://reviewboard.asterisk.org/r/4467/
+	  ........
 
-	  The test now checks to ensure that 2 or more characters are read
-	  in ensuring the test stays within the buffer.
+	  Merged revisions 432787 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25874 #close
 
-	  Change-Id: Ic2c89cea6e90f2c0bc2d8138306ebbffd4f8b710
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432788 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-07 12:02 +0000 [f896136460]  Alexei Gradinari <alex2grad@gmail.com>
+2015-03-11 10:26 +0000 [89b65f5dda]  Richard Mudgett <rmudgett@digium.com>
 
-	* app_voicemail/IMAP: function 'save_to_folder' creates wrong folder
+	* res_pjsip: Fix pjsip.conf type=global object default value handling.
 
-	  If try to move message to Cust1 (number 5)
-	  the function 'save_to_folder' tries to create Greeting folder instead of Cust1.
+	  When a type=global section is not defined in pjsip.conf the global
+	  defaults are not applied.  As a result the mandatory Max-Forwards header
+	  is not added to SIP messages for res_pjsip/chan_pjsip.
 
-	  This patch fixed it by setting GREETINGS_FOLDER = -1
+	  The handling of pjsip.conf type=global objects has several problems:
 
-	  ASTERISK-24927 #close
+	  1) If the global object is missing the defaults are not applied.
 
-	  Change-Id: I03d1a761894bcc2d130ec9b003bbcddc28e25c51
+	  2) If the global object is missing the default_outbound_endpoint's default
+	  value is not returned by ast_sip_global_default_outbound_endpoint().
 
-2016-04-07 16:18 +0000 [70b7673f09]  Alexei Gradinari <alex2grad@gmail.com>
+	  3) Defines are needed so default values only need to be changed in one
+	  place.
 
-	* res_pjsip: Add headers to AMI Event ContactStatusDetail
+	  * Added a sorcery instance observer callback to check if there were any
+	  type=global sections loaded.  If there were more than one then issue an
+	  error message.  If there were none then apply the global defaults.
 
-	  * Added Useragent and RegExpire headers to AMI Event
-	  ContactStatusDetail with associated documentation.
+	  * Fixed ast_sip_global_default_outbound_endpoint() to return the
+	  documented default when no type=global object is defined.
 
-	  ASTERISK-25903 #close
+	  * Made defines for the global default values.
 
-	  Change-Id: If3d121e943e588d016ba51d4eb9c6a421a562239
+	  * Increased the default_useragent[] size because SVN version strings can
+	  get lengthy and 128 characters may not be enough.
 
-2016-04-11 16:20 +0000 [64ecd41c8f]  Alexei Gradinari <alex2grad@gmail.com>
+	  * Fixed an off-nominal code path ref leak in global_alloc() if the string
+	  fields fail to initialize.
 
-	* Codecs: strip codec name while parsing allow/disallow options
+	  * Eliminated RAII_VAR in get_global_cfg() and
+	  ast_sip_global_default_outbound_endpoint().
 
-	  Failed registration using PJSIP/Realtime if one of the codec name
-	  in allow/disallow option is wrong or contains space.
+	  ASTERISK-24807 #close
+	  Reported by: Anatoli
 
-	  This patch strip codec name.
+	  Review: https://reviewboard.asterisk.org/r/4467/
+	  ........
 
-	  ASTERISK-25914
+	  Merged revisions 432766 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ifdf02de94e5ddbce305640f6f0666084a3b9283d
 
-2016-04-11 14:26 +0000 [3f6c4667b8]  Jaco Kroon <jaco@uls.co.za>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432767 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* core_unreal: Fix hangupcauses not getting set on Local channels
+2015-03-11 10:22 +0000 [185d2e082a]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25912 #close
+	* res_pjsip: Fixed invalid empty Server and User-Agent SIP headers.
 
-	  Change-Id: I8e72e6894feaf36c9450f2788d205d07baec23aa
+	  Setting pjsip.conf useragent to an empty string results in an empty SIP
+	  header being sent.
 
-2016-04-01 13:30 +0000 [fe7e48db03]  gtjoseph <george.joseph@fairview5.com>
+	  * Made not add an empty SIP header item to the global SIP headers list.
 
-	* res_pjsip contact:  Lock expiration/addition of contacts
+	  Review: https://reviewboard.asterisk.org/r/4467/
+	  ........
 
-	  Contact expiration can occur in several places:  res_pjsip_registrar,
-	  res_pjsip_registrar_expire, and automatically when anyone calls
-	  ast_sip_location_retrieve_aor_contact.  At the same time, res_pjsip_registrar
-	  may also be attempting to renew or add a contact.  Since none of this was locked
-	  it was possible for one thread to be renewing a contact and another thread to
-	  expire it immediately because it was working off of stale data.  This was the
-	  casue of intermittent registration/inbound/nominal/multiple_contacts test
-	  failures.
+	  Merged revisions 432764 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Now, the new named lock functionality is used to lock the aor during contact
-	  expire and add operations and res_pjsip_registrar_expire now checks the
-	  expiration with the lock held before deleting the contact.
 
-	  ASTERISK-25885 #close
-	  Reported-by: Josh Colp
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432765 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I83d413c46a47796f3ab052ca3b349f21cca47059
+2015-03-10 18:09 +0000 [2889f074a0]  Joshua Colp <jcolp@digium.com>
 
-2016-04-10 14:16 +0000 [0c414eaf35]  gtjoseph <george.joseph@fairview5.com>
+	* core: Don't create snapshots with locks.
 
-	* pjproject:  Add patch to fix Via IPv6 parsing
+	  Snapshots are immutable and are never changed. Allocating them
+	  with a lock is wasteful.
 
-	  There's a bug in pjproject's sip_parser where the ":" wasn't correctly
-	  interpreted. This is causing IPv6 addresses in the "received" parameter of the
-	  Via header to cause a syntax check failure.
+	  Review: https://reviewboard.asterisk.org/r/4469/
+	  ........
 
-	  This patch was submitted to Teluu on 4/10/2016.
+	  Merged revisions 432742 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25910 #close
-	  Reported-by: Anthony Messina
 
-	  Change-Id: Ic7e4c4aa14ded61860401ec349f5177568c4d922
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432743 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-31 20:04 +0000 [772ff3048f]  gtjoseph <george.joseph@fairview5.com>
+2015-03-10 16:33 +0000 [15d266bf85]  Javier Acosta (License 6690)
 
-	* lock:  Add named lock capability
+	* res/res_config_odbc: Fix improper escaping of backslashes with MySQL
 
-	  Locking some objects like sorcery objects can be tricky because the underlying
-	  ao2 object may not be the same for all callers.  For instance, two threads that
-	  call ast_sorcery_retrieve_by_id on the same aor name might actually get 2
-	  different ao2 objects if the underlying wizard had to rehydrate the aor from a
-	  database. Locking one ao2 object doesn't have any effect on the other even if
-	  those objects had locks in the first place.
+	  When escaping backslashes with MySQL, the proper way to escape the characters
+	  in a LIKE clause is to escape the '\' four times, i.e., '\\\\'. To quote the
+	  MySQL manual:
 
-	  Named locks allow access control by keyspace and key strings.  Now an "aor"
-	  named "1000" can be locked and any other thread attempting to lock "aor" "1000"
-	  will wait regardless of whether the underlying ao2 object is the same or not.
-	  Mutex and rwlocks are supported.
+	  "Because MySQL uses C escape syntax in strings (for example, “\n” to represent
+	  a newline character), you must double any “\” that you use in LIKE strings.
+	  For example, to search for “\n”, specify it as “\\n”. To search for “\”,
+	  specify it as “\\\\”; this is because the backslashes are stripped once by the
+	  parser and again when the pattern match is made, leaving a single backslash to
+	  be matched against."
 
-	  This capability will initially be used to lock an aor when multiple threads may
-	  be attempting to prune expired contacts from it.
+	  ASTERISK-24808 #close
+	  Reported by: Javier Acosta
+	  patches:
+	    res_config_odbc.diff uploaded by Javier Acosta (License 6690)
+	  ........
 
-	  Change-Id: If258c0b7f92b02d07243ce70e535821a1ea7fb45
+	  Merged revisions 432720 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-04-05 16:56 +0000 [fd601f26f7]  Alexei Gradinari <alex2grad@gmail.com>
+	  Merged revisions 432721 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_outbound_publish: Add transport for outbound PUBLISH
 
-	  The first available transport of the appropriate type is used now.
-	  This patch adds new config option 'transport' for outbound-publish.
-	  If transport is set then outbound PUBLISH requests will use this transport.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432722 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25901 #close
+2015-03-10 13:13 +0000 [ab6e2c93f3]  Graham Barnett (License 6685)
 
-	  Change-Id: Ib389130489b70e36795b0003fa5fd386e2680151
+	* app_voicemail: Fix crash with IMAP backends when greetings aren't present
 
-2016-04-07 16:39 +0000 [5f768d2a9c]  Alexei Gradinari <alex2grad@gmail.com>
+	  When an IMAP backend is in use and greetings are set to be used, but aren't
+	  present for a user in their IMAP folder, Asterisk will crash. This occurs
+	  due to the mailstream being set to the 'greetings' folder and being left
+	  in that particular state, regardless of the success/failure of the attempt
+	  to access the folder the mailstream points to. Later access of the mailstream
+	  assumes that it points to the 'INBOX' (or some other folder), resulting in
+	  either a crash (if the greetings folder didn't exist and the mailstream is
+	  invalid) or an inability to read messages from the 'INBOX' folder.
 
-	* res_pjsip_dialog_info: Add missing "direction" attribute in NOTIFY event
+	  This patch restores the mailstream to its correct state after accessing the
+	  greetings. This fixes the crash, and sets the mailstream to the state that
+	  VoiceMailMain expects.
 
-	  BLF pickup isn't working on Cisco SPA and Snom phones
-	  if the direction="recipient" attribute is missing in 'dialog' tag.
+	  Note that while ASTERISK-23390 also contained a patch for this issue, the
+	  patch on ASTERISK-24786 is the one being merged here.
 
-	  This patch adds direction="recipient" if extension state is
-	  Ringing.
+	  Review: https://reviewboard.asterisk.org/r/4459/
 
-	  ASTERISK-24601 #close
+	  ASTERISK-23390 #close
+	  Reported by: Ben Smithurst
 
-	  Change-Id: I5b2c097ca29fd59e92ba237ca5d397cb1b0bcd8c
+	  ASTERISK-24786 #close
+	  Reported by: Graham Barnett
+	  Tested by: Graham Barnett
+	  patches:
+	    app_voicemail.c.patch.SIGSEGV3rev2 uploaded by Graham Barnett (License 6685)
+	  ........
 
-2016-04-07 10:59 +0000 [82638fb0c7]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 432695 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* pbx.c: Minor code rearangements.
+	  Merged revisions 432696 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Pull out a loop invariant.
 
-	  * Convert an else-if ladder to a switch statement.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432697 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I0a95cfa9474a4600b9865f7b444534d275b37e95
+2015-03-10 13:05 +0000 [79e9b37ad0]  Ed Hynan (Licnese 6680)
 
-2016-04-07 11:37 +0000 [bc320df173]  Alexei Gradinari <alex2grad@gmail.com>
+	* localtime: Fix file descriptor leak on kqueue(2) systems
 
-	* app_voicemail/IMAP: IMAP access FATAL error: Out of memory
+	  The localtime management in the Asterisk core contains a thread that watches
+	  for changes in the local timezone. On systems where the directory containing
+	  /etc/localtime is modified frequently, the thread monitoring the changes will
+	  be woken up to determine if any changes in timezone have occurred. When using
+	  kqueue(2), this can cause a leak of file descriptors due to some improper
+	  management of resources.
 
-	  Sometimes uw-imap function 'mail_fetchbody' returns huge len
-	  which then pass to uw-imap function 'rfc822_base64'.
-	  uw-imap tries to allocate huge memory and abort() on fail.
+	  This patch updates the kqueue(2) handling in localtime, such that is no longer
+	  leaks resources.
 
-	  This patch check the len.
-	  If the len more than max size (128 Mbytes) log error.
-	  This patch also set variables len, newlen to avoid uninizialezed len.
-	  This patch also check pointer returned by rfc822_base64.
+	  Review: https://reviewboard.asterisk.org/r/4450/
 
-	  ASTERISK-25899 #close
+	  ASTERISK-24739 #close
+	  Reported by: Ed Hynan
+	  patches:
+	    11.15.0-u.diff uploaded by Ed Hynan (Licnese 6680)
+	    11.7.0-u.diff uploaded by Ed Hynan (License 6680)
+	    svn-trunk-Jan-26-2015-u.diff uploaded by Ed Hynan (License 6680)
+	  ........
 
-	  Change-Id: I4a0e7d655f11abef6a5224e2169df6d5c1f1caca
+	  Merged revisions 432691 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-04-07 12:26 +0000 [2ef8a954b3]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 432693 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* pbx: Update doxygen for extension state watchers.
 
-	  Change-Id: Id1403b12136de62a272c01bb355aef65fd2c2d1e
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432694 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-07 11:49 +0000 [d312fdeb1b]  gtjoseph <george.joseph@fairview5.com>
+2015-03-10 11:08 +0000 [e7ee83ea90]  Richard Mudgett <rmudgett@digium.com>
 
-	* alembic:  Remove batch operations (and sqlite support)
+	* res_pjsip_refer: Fix occasional unexpected BYE sent after receiving a REFER.
 
-	  Because SQLite doesn't support full ALTER capabilities, alembic scripts
-	  require batch operations.  However, that capability wasn't available until
-	  0.7.0 which some distributions haven't reached yet.  Therefore, the batch
-	  operations introduced in commit 86d6e44cc (review 2319) have been reverted
-	  and SQLite is unsupported again, for now anyway.
+	  A race condition happened between initiating a transfer and requesting
+	  that a dialog termination be delayed.  Occasionally, the transferrer
+	  channels would exit the bridge and hangup before the dialog termination
+	  delay was requested.
 
-	  Tested the full upgrade and downgrade on MySQL/Mariadb and Postgresql.
+	  * Made request dialog termination delay before initiating the transfer
+	  action.  If the transfer fails then cancel the delayed dialog termination
+	  request.
 
-	  ASTERISK-25890 #close
-	  Reported-by: Harley Peters
+	  ASTERISK-24755 #close
+	  Reported by: John Bigelow
 
-	  Change-Id: I82eba5456736320256f6775f5b0b40133f4d1c80
+	  Review: https://reviewboard.asterisk.org/r/4460/
+	  ........
 
-2016-04-07 11:05 +0000 [901e8d78c4]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 432668 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_registrar_expire: Fix race condition at shutdown.
 
-	  When shutting down, the PJSIP sorcery is destroyed. The registrar
-	  expiration module queries the PJSIP sorcery to determine what
-	  to expire. As there was no synchronization between termination
-	  of the expiration thread and the unloading of the module it was
-	  possible for the thread to try to access the PJSIP sorcery after
-	  it had been destroyed.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432669 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This change ensures that the thread is shut down before allowing
-	  the module to be considered unloaded.
+2015-03-09 11:13 +0000 [1ce529d30e]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I69fd239edbaaf160c2d37ae00d3ac06e5596fe8b
+	* res_pjsip: allow configuration of endpoint identifier query order
 
-2016-04-06 16:28 +0000 [8207372e66]  Joshua Colp <jcolp@digium.com>
+	  It's possible to have a scenario that will create a conflict between endpoint
+	  identifiers. For instance an incoming call could be identified by two different
+	  endpoint identifiers and the one chosen depended upon which identifier module
+	  loaded first. This of course causes problems when, for example, the incoming
+	  call is expected to be identified by username, but instead is identified by ip.
+	  This patch adds a new 'global' option to res_pjsip called
+	  'endpoint_identifier_order'. It is a comma separated list of endpoint
+	  identifier names that specifies the order by which identifiers are processed
+	  and checked.
 
-	* res_pjsip: Fix configuration setting of "regcontext".
+	  ASTERISK-24840 #close
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4455/
+	  ........
 
-	  Due to a merge problem two options were swapped causing the
-	  regcontext setting to not get set.
+	  Merged revisions 432638 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Icb33edc668e7357bacbaec2861a6b5ac64edaff1
 
-2016-04-06 08:01 +0000 [0735a4d6d7]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432639 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* frame.c: Copy the whole subclass in ast_frdup().
+2015-03-07 19:47 +0000 [a5f80f1781]  Joshua Colp <jcolp@digium.com>
 
-	  The problem is ast_frdup() does not copy whole frame.subclass for voice,
-	  video and image frames, only the format is copied.  For video frames, the
-	  subclass structure contains the .frame_ending flag used to put the RTP
-	  marker where it needs to be.
+	* res_rtp_asterisk: Fix wrongful use of USE_PJPROJECT define.
 
-	  ASTERISK-25894 #close
+	  As pjproject is now used as a shared library a different define,
+	  HAVE_PJPROJECT, is used to specify if pjproject is present.
 
-	  Change-Id: I812ca90e84ed5d4f473b997d0dd0d3c5a915fe33
+	  ASTERISK-24830 #close
+	  Reported by: Stefan Engström
+	  ........
 
-2016-04-05 14:23 +0000 [c61dca6419]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 432614 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip: Handle deferred SDP hold/unhold properly.
 
-	  Some SIP devices indicate hold/unhold using deferred SDP reinvites. In
-	  other words, they provide no SDP in the reinvite.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432615 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A typical transaction that starts hold might look something like this:
+2015-03-06 16:59 +0000 [affcf1d766]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Device sends reinvite with no SDP
-	  * Asterisk sends 200 OK with SDP indicating sendrecv on streams.
-	  * Device sends ACK with SDP indicating sendonly on streams.
+	* res_pjsip_refer: Make safely get the context for a blind transfer.
 
-	  At this point, PJMedia's SDP negotiator saves Asterisk's local state as
-	  being recvonly.
+	  Made safely get the TRANSFER_CONTEXT channel value while the channel is
+	  locked in refer_incoming_attended_request() and
+	  refer_incoming_blind_request().  The pointer returned by
+	  pbx_builtin_getvar_helper() is only valid while the channel is locked.
+	  ........
 
-	  Now, when the device attempts to unhold, it again uses a deferred SDP
-	  reinvite, so we end up doing the following:
+	  Merged revisions 432594 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Device sends reinvite with no SDP
-	  * Asterisk sends 200 OK with SDP indicating recvonly on streams
-	  * Device sends ACK with SDP indicating sendonly on streams
 
-	  The problem here is that Asterisk offered recvonly, and by RFC 3264's
-	  rules, if an offer is recvonly, the answer has to be sendonly. The
-	  result is that the device is not taken off hold.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432595 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  What is supposed to happen is that Asterisk should indicate sendrecv in
-	  the 200 OK that it sends. This way, the device has the freedom to
-	  indicate sendrecv if it wants the stream taken off hold, or it can
-	  continue to respond with sendonly if the purpose of the reinvite was
-	  something else (like a session timer refresher).
+2015-03-06 16:18 +0000 [090ab1735b]  Richard Mudgett <rmudgett@digium.com>
 
-	  The fix here is to alter the SDP negotiator's state when we receive a
-	  reinvite with no SDP. If the negotiator's state is currently in the
-	  recvonly or inactive state, then we alter our local state to be
-	  sendrecv. This way, we allow the device to indicate the stream state as
-	  desired.
+	* res_pjsip_refer: Made refer_attended_alloc() not create the ao2 object with a lock.
 
-	  ASTERISK-25854 #close
-	  Reported by Robert McGilvray
+	  The lock is unused.
+	  ........
 
-	  Change-Id: I7615737276165eef3a593038413d936247dcc6ed
+	  Merged revisions 432574 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-27 23:33 +0000 [50b0922a22]  gtjoseph <george.joseph@fairview5.com>
 
-	* config:  Allow filters when appending to a category
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432579 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In sorcery based config files where there are multiple categories with the same
-	  name, you can't use the (+) operator to reliably append to a category because
-	  config.c stops looking when it finds the first one with the same name.
+2015-03-06 15:38 +0000 [b85cb7ea1b]  Jonathan Rose <jrose@digium.com>
 
-	  Example:
+	* app: Add functions to swap voicemail function table for testing purposes
+	  ........
 
-	  [1000]
-	  type = endpoint
+	  Merged revisions 432556 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  [1000]
-	  type = aor
 
-	  [1000](+)
-	  authenticate_qualify = yes
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432573 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This config will fail because config.c appends authenticate_qualify to the
-	  first category it finds, the endpoint, and that's not valid for endpoint.
+2015-03-06 14:24 +0000 [c7cc1b3059]  Richard Mudgett <rmudgett@digium.com>
 
-	  Solution:
+	* chan_dahdi/sig_analog: Fix distinctive ring detection to suck less.
 
-	  The capability to find a category that contains a certain variable already
-	  exists so the only real change was to parse anything after the '+' that's not a
-	  comma, as a filter string.
+	  The distinctive ring feature interferes with detecting Caller ID and
+	  appears to have been broken for years.  What happens is if you have a
+	  ring-ring cadence as used in the UK you get too many DAHDI events for the
+	  distinctive ring pattern array and Caller ID detection is aborted.  I
+	  think when Zapata/DAHDI added the ring begin event it broke distinctive
+	  ring.  More events happen than before and the code does no filtering of
+	  which event times are recorded in the pattern array.
 
-	  [1000]
-	  type = endpoint
+	  * Made distinctive ring only record the ringt count when the ring ends
+	  instead of on just any DAHDI event.  Distinctive ring can be ring,
+	  ring-ring, ring-ring-ring, or different ring durations for the up to three
+	  rings.
 
-	  [1000]
-	  type = aor
+	  * Fixed the distinctive ring detection enable (chan_dahdi.conf option
+	  usedistinctiveringdetection) to be per port instead of somewhat per port
+	  and somewhat global.  This has been broken since v1.8.
 
-	  [1000](+type=aor)
-	  authenticate_qualify = yes
+	  * Fixed using the default distinctive ring context when the detected
+	  pattern does not match any configured dringX patterns.  The default
+	  context did not get set when the previous call was a matched distinctive
+	  ring pattern and the current call is not matched.  This has been broken
+	  since v1.8.
 
-	  This now works as expected.
+	  * Made distinctive ring have no effect on Caller ID detection when it is
+	  disabled.  Caller ID detection just monitors for 10 seconds before giving
+	  up.
 
-	  Although the following example doesn't make any sense for pjsip, you can even
-	  specify multiple filters:
+	  * Fixed leak of struct callerid_state memory when a polarity reversal
+	  during Caller ID detection causes the incoming call to be aborted.
 
-	  [1000](+type=aor&qualify_frequency=10)
+	  DAHDI-1143
+	  AST-1545
+	  ASTERISK-24825 #close
+	  Reported by: Richard Mudgett
 
-	  ASTERISK-25868 #close
-	  Reported-by: Nick Repin
+	  ASTERISK-17588
+	  Reported by: Daniel Flounders
 
-	  Change-Id: I10773da4c79db36fbf1993961992af63d3441580
+	  Review: https://reviewboard.asterisk.org/r/4444/
+	  ........
 
-2016-04-05 10:21 +0000 [cb56ef8069]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 432530 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_http_websocket: Make core supported.
+	  Merged revisions 432534 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Websockets are a core part of ARI support and as such this
-	  module should also be core supported.
 
-	  Change-Id: I8f9283c6a167152761b92984779bb39e3db51a9c
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432551 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-25 23:22 +0000 [f6f4cf459f]  gtjoseph <george.joseph@fairview5.com>
+2015-03-06 13:34 +0000 [f1ab2c5e8b]  Richard Mudgett <rmudgett@digium.com>
 
-	* stringfields:  Refactor to allow fields to be added to the end of structures
+	* chan_sip: Fix realtime locking inversion when poking a just built peer.
 
-	  String fields are great, except that you can't add new ones without breaking
-	  ABI compatibility because it shifts down everything else in the structure.
-	  The only alternative is to add your own char * field to the end of the
-	  structure and manage the memory yourself which isn't ideal, especially since
-	  you then can't use the OPT_STRINGFIELD_T type.
+	  When a realtime peer is built it can cause a locking inversion when the
+	  just built peer is poked.  If the CLI command "sip show channels" is
+	  periodically executed then a deadlock can happen because of the locking
+	  inversion.
 
-	  Background:
+	  * Push the peer poke off onto the scheduler thread to avoid the locking
+	  inversion of the just built realtime peer.
 
-	  The reason string fields had to be declared inside the
-	  AST_DECLARE_STRING_FIELDS block was to facilitate iteration over all declared
-	  fields for initialization, compare and copy.  Since AST_DECLARE_STRING_FIELDS
-	  declared the pool, then the fields, then the manager, you could use the offsets
-	  of the pool and manager and iterate over the sequential addresses in between to
-	  access the fields. The actual pool, field allocation and field set operations
-	  don't actually care where the field is.  It's just iteration over the fields
-	  that was the problem.
+	  AST-1540
+	  ASTERISK-24838 #close
+	  Reported by: Richard Mudgett
 
-	  Solution: Extended String Fields
+	  Review: https://reviewboard.asterisk.org/r/4454/
+	  ........
 
-	  An extended string field is one that is declared outside the
-	  AST_DECLARE_STRING_FIELDS block but still (anywhere) inside the parent
-	  structure.  Other than using AST_STRING_FIELD_EXTENDED instead of
-	  AST_STRING_FIELD, it looks the same as other string fields.  It's storage comes
-	  from the pool and it participates in string field compare and copy operations
-	  peformed on the parent structure. It's also a valid target for the
-	  OPT_STRINGFIELD_T aco option type.
+	  Merged revisions 432526 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Implementation:
+	  Merged revisions 432528 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  To keep track of the extended fields and make sure that ABI isn't broken, the
-	  existing embedded_pool pointer in the manager structure was repurposed to be a
-	  pointer to a separate header structure that contains the embedded_pool pointer
-	  plus a vector of fields.  The length of the manager structure didn't change and
-	  the embedded_pool pointer isn't used in the macros, only the stringfields C
-	  code.  A side benefit of this is that changing the header structure in the
-	  future won't break ABI.
 
-	  ast_string_fields_init initializes the normal string fields and appends them to
-	  the vector, and subsequent calls to ast_string_field_init_extended initialize
-	  and append the extended fields. Cleanup, ast_string_fields_cmp, and
-	  ast_string_fields_copy can now work on the vector instead of sequentially
-	  traversing the addresses between the pool and manager.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432529 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The total size of a structure using string fields didn't change, whether using
-	  extended fields or not, nor have the offsets of any structure members, either
-	  inside the original block or outside.  Adding an extended field to the end of a
-	  structure is the same as adding a char *.
+2015-03-05 10:40 +0000 [5c3e33b3ca]  gtjoseph <george.joseph@fairview5.com>
 
-	  Details:
+	* app_voicemail: Fix compile breaking in app_voicemail with IMAP_STORAGE.
 
-	  The stringfield C code was pulled out from utils.c and into stringfields.c.
-	  It just made sense.
+	  There is a leftover "assert" in app_voicemail/__messagecount that references 
+	  variables that don't exist.  This causes the compile to fail when 
+	  --enable-dev-mode and IMAP_STORAGE are selected.
 
-	  Additional work was done in ast_string_field_init and
-	  ast_calloc_with_stringfields to handle the allocation of the new header
-	  structure and the vector, and the associated cleanup.  In the process some
-	  additional NULL pointer checking was added.
+	  This patch removes the assert.
 
-	  A lot of work was done in stringfields.h since the logic for compare and copy
-	  is there.  Documentation was added as well as somne additional NULL checking.
+	  Tested-by: George Joseph
 
-	  The ability to call ast_calloc_with_stringfields with a number of structures
-	  greater than 1 never really worked.  Well, the calloc worked but there was no
-	  way to access the additional structures or clean them up.  It was agreed that
-	  there was no use case for requesting more than 1 structure so an ast_assert
-	  was added to prevent it and the iteration code removed.
+	  Review: https://reviewboard.asterisk.org/r/4461/
+	  ........
 
-	  Testing:
+	  Merged revisions 432484 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The stringfield unit tests were updated to test both normal and extended
-	  fields.  Tests for ast_string_field_ptr_set_by_fields and
-	  ast_calloc_with_stringfields were also added.
+	  Merged revisions 432485 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  As an ABI test, 13 was compiled from git and the res_pjsip_* modules, except
-	  res_pjsip itself, saved off.  The patch was then added and a full compile and
-	  install was performed.  Then the older res_pjsip_* moduled were copied over the
-	  installed versions so res_pjsip was new and the rest were old.  No issues.
 
-	  contact->aor, which is a char * at the end of contact, was then changed to an
-	  extended string field and a recompile and reinstall was performed, again
-	  leaving stock versions of the the res_pjsip_* modules.  Again, no issues with
-	  the res_pjsip_* modules using the old stringfield implementation and with
-	  contact->aor as a char *, and res_pjsip itself using the new stringfield
-	  implementation and contact->aor being an extended string field.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432486 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Finally, several existing string fields were converted to extended string
-	  fields to test OPT_STRINGFIELD_T.  Again, no issues.
+2015-03-04 12:55 +0000 [41ba8fd7c0]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I235db338c5b178f5a13b7946afbaa5d4a0f91d61
+	* translate: Prevent invalid memory accesses on fast shutdown
 
-2016-04-04 18:02 +0000 [fe448ac8a7]  gtjoseph <george.joseph@fairview5.com>
+	  When a 'core restart now' or 'core stop now' is executed and a channel is
+	  currently in a media operation, the translator matrix can be destroyed while a
+	  channel is currently blocked on getting the best translation choice
+	  (see ast_translator_best_choice). When the channel gets the mutex, the
+	  translation matrix now has invalid memory, and Asterisk crashes.
 
-	* res_pjsip_mwi:  Fix segv caused by 16c7d8e74a9af13f98c3c22aa9c43ce39965f6b7
+	  This patch does two things:
+	  (1) We now only clean up the translation matrix on a graceful shutdown. In that
+	      case, there are no channels, and so there is no risk of this occurring.
+	  (2) We also now set the __matrix and __indextable to NULL. In some initial
+	      backtraces when this occurred, it looked as if there was a memory corruption
+	      occurring, and it wasn't until we determined that something had restarted
+	      Asterisk that the issue became clear. By setting these to NULL on shutdown,
+	      it becomes a bit easier to determine why a crash is occurring.
+
+	  Note that we could litter the code with NULL checks on the __matrix, but the
+	  act of making the translation matrix cleaned up on shutdown should preclude
+	  this issue from occurring in the first place, and this part of the code needs
+	  to be as fast as possible.
 
-	  I forgot the new voicemail_extension wasn't a stringfield and didn't check
-	  for NULL where I should have.
+	  Review: https://reviewboard.asterisk.org/r/4457/
+	  ........
 
-	  Change-Id: I029482d5c2ab72474838750461bd46b0809c90fb
+	  Merged revisions 432453 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-03 11:47 +0000 [9d4318f798]  gtjoseph <george.joseph@fairview5.com>
 
-	* install_prereq:  Fix check_installed_debs remove subversion
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432455 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  check_installed_debs wasn't handling virtual packages like libsrtp-dev and
-	  libresample-dev and on multiarch systems it was accidentally filtering out all
-	  packages if any :i386 packages were found instead of just filtering out the
-	  :i386 packages themselves.
+2015-03-02 13:15 +0000 [278ea2f468]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ifd68da0d1ee30cc84df14de3f9b9079d7c3cecda
+	* res/res_pjsip_sdp_rtp: Revert portion of r432195
 
-2016-04-01 13:09 +0000 [566601837e]  gtjoseph <george.joseph@fairview5.com>
+	  Unfortunately, while initial testing with ConfBridge did not reproduce the
+	  audio problem alluded to in the comment in res_pjsip_sdp_rtp, further testing
+	  did show that bridge_softmix and/or ConfBridge has a severe problem bridging
+	  two or more participants at different sampling rates. Sometimes, it even picks
+	  odd sampling rates that cause hideous audio problems.
 
-	* utils.c:  Fix typo in handle_show_locks
+	  This patch backs out the offending portion of the code until the issues in
+	  the affected bridging modules can be more properly analyzed.
 
-	  ast_cli_allow_on_shutdown(e) should have been ast_cli_allow_at_shutdown(e).
+	  ASTERISK-24841
+	  ........
 
-	  Change-Id: I4f092495c0b2bfd85c2651e0b5877bf4d05d9faf
+	  Merged revisions 432423 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-30 18:34 +0000 [964f54bd5d]  gtjoseph <george.joseph@fairview5.com>
 
-	* pjproject_bundled:  Fix use of LDCONFIG for shared library link creation
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432425 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  LDCONFIG apparently isn't set to something sane on all systems so the creation
-	  of the shared library links fails.  Instead of just testing for non-blank,
-	  main/Makefile now checks that LDCONFIG is actually executable and reverts to
-	  LN if it isn't.
+2015-02-27 12:31 +0000 [9e841e4fb6]  Richard Mudgett <rmudgett@digium.com>
 
-	  This applies to both libasteriskpj and libasteriskssl.
+	* ARI: Fix crash if integer values used in JSON payload 'variables' object.
 
-	  Thanks to 'abelbeck' for pointing out that the issue was LDCONFIG.
+	  Sending the following ARI commands caused Asterisk to crash if the JSON
+	  body 'variables' object passes values of types other than strings.
 
-	  ASTERISK-25873 #close
-	  Reported-by: Hans van Eijsden
+	  POST /ari/channels
+	  POST /ari/channels/{channelid}
+	  PUT /ari/endpoints/sendMessage
+	  PUT /ari/endpoints/{tech}/{resource}/sendMessage
 
-	  Change-Id: I25b76379bc637726ec044b2c0e709b56b3701729
+	  * Eliminated RAII_VAR usage in ast_ari_channels_originate_with_id(),
+	  ast_ari_channels_originate(), ast_ari_endpoints_send_message(), and
+	  ast_ari_endpoints_send_message_to_endpoint().
 
-2016-03-30 13:31 +0000 [5f73c2ef0a]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24751 #close
+	  Reported by:  jeffrey putnam
 
-	* res_stasis.c: Protect channel datastore list from stasis end.
+	  Review: https://reviewboard.asterisk.org/r/4447/
+	  ........
 
-	  Change-Id: Ifadc469590bd4d5368e19d3763db3bd1f80fdb95
+	  Merged revisions 432404 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-29 18:06 +0000 [74d63f56ee]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_ari: Cannot get control also means channel is unavailable.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432405 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The only caller of ari_bridges_play_found() has this note:
+2015-02-26 12:53 +0000 [d79670b269]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  If ari_bridges_play_found fails because the channel is unavailable for
-	  playback, The channel will be removed from the playback list soon.  We can
-	  keep trying to get channels from the list until we either get one that
-	  will work or else there isn't a channel for this bridge anymore, in which
-	  case we'll revert to ari_bridges_play_new.
+	* Dial API: add self destruct option when complete
 
-	  Change-Id: Ib068141b367ccaa17be0dab4181c98e26c5127d6
+	  This patch adds a self-destruction option to the
+	  dial api.  The usefulness of this is mostly when
+	  using async mode to spawn a separate thread used
+	  to handle the new call, while the calling thread
+	  is allowed to go on about other business.
 
-2016-03-29 14:29 +0000 [cf49b44090]  Richard Mudgett <rmudgett@digium.com>
+	  The only alternative to this option would be the
+	  calling thread spawning a new thread, or hanging
+	  around itself waiting to destroy the dial struct
+	  after completion.
 
-	* res_stasis_recording.c: Cleanup stasis_app_recording_find_by_name().
+	  Example of use (minus error checking):
 
-	  Change-Id: Ic7d93c402c498677a122505558859c853d4e5ac7
+	    struct ast_dial *dial = ast_dial_create();
 
-2016-03-28 14:23 +0000 [7f53f1d89e]  Richard Mudgett <rmudgett@digium.com>
+	    ast_dial_append(dial, "PJSIP", "200", NULL);
 
-	* core_unreal.c: Add clarification comment about channel ref.
+	    ast_dial_option_global_enable(dial, AST_DIAL_OPTION_ANSWER_EXEC, "Echo");
+	    ast_dial_option_global_enable(dial, AST_DIAL_OPTION_SELF_DESTROY, NULL);
 
-	  Change-Id: I0be0627260cd8d6b6c3cc345949dcfdf32eff1f3
+	    ast_dial_run(dial, NULL, 1);
 
-2016-03-29 13:47 +0000 [ecf4102d02]  Richard Mudgett <rmudgett@digium.com>
+	  The dial_run call will return almost immediately
+	  after spawning the new thread to run and monitor
+	  the dial.  If the call is answered, it is placed
+	  into the echo app.  When completed, it will call
+	  ast_dial_destroy() on the dial structure.
 
-	* res_stasis: Add control ref to playback and recording structs.
+	  Note that any allocations made to pass values to
+	  ast_dial_set_user_data() or dial options must be
+	  free'd in a state callback function on any of:
+	    AST_DIAL_RESULT_UNASWERED,
+	    AST_DIAL_RESULT_ANSWERED,
+	    AST_DIAL_RESULT_HANGUP, or 
+	    AST_DIAL_RESULT_TIMEOUT.
 
-	  The stasis_app_playback and stasis_app_recording structs need to have a
-	  struct stasis_app_control ref.  Other threads can get a reference to the
-	  playback and recording structs from their respective global container.
-	  These other threads can then use the control pointer they contain after
-	  the control struct has gone.
+	  Review: https://reviewboard.asterisk.org/r/4443/
+	  ........
 
-	  * Add control ref to stasis_app_playback and stasis_app_recording structs.
+	  Merged revisions 432385 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  With the refs added, the control command queue can now have a circular
-	  control reference which will cause the control struct to never get
-	  released if the control's command queue is not flushed when the channel
-	  leaves the Stasis application.  Also the command queue needs better
-	  protection from adding commands if the control->is_done flag is set.
 
-	  * Flush the control command queue on exit.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432386 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25882 #close
+2015-02-26 11:12 +0000 [d04fbb0f9d]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I3cf1fb59cbe6f50f20d9e35a2c07ac07d7f4320d
+	* app_chanspy, channel: fix frame leaks
 
-2016-03-28 18:10 +0000 [a179aba65e]  Richard Mudgett <rmudgett@digium.com>
+	  Fixed a couple of frame leaks that were found during testing.
 
-	* res_stasis: Fix crash on a hanging up channel.
+	  ASTERISK-24828 #close
+	  Reported by: John Hardin
+	  Review: https://reviewboard.asterisk.org/r/4445/
+	  ........
 
-	  * Give the struct stasis_app_control ao2 object a ref to the channel held
-	  in the object.  Now the channel will still be around if a thread needs to
-	  post a stasis message instead of crash because the topic was destroyed.
+	  Merged revisions 432362 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * Moved stopping any lingering silence generator out of the struct
-	  stasis_app_control destructor and made it a part of exiting the Stasis
-	  application.  Who knows which thread the destructor will be called under
-	  so it cannot affect the channel's silence generator.  Not only was the
-	  channel unprotected when the silence generator was stopped, stasis may no
-	  longer even control the channel.
+	  Merged revisions 432363 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25882
 
-	  Change-Id: I21728161b5fe638cef7976fa36a605043a7497e4
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432364 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-30 12:38 +0000 [16c7d8e74a]  gtjoseph <george.joseph@fairview5.com>
+2015-02-25 22:58 +0000 [8a16c2f0c2]  Matt Jordan <mjordan@digium.com>
 
-	* res_pjsip_mwi:  Allow subscribe to vm access extension as an alias
+	* make: Remove 'res_features' from libraries to link against with cygwin/mingw32
 
-	  Background:
+	  Both the apps and channels Makefiles still listed 'res_features' as modules to
+	  link against when compiling for cygwin or mingw32. This module hasn't existed
+	  for quite some time.
 
-	  If your extension is 1000 and the voicemail access extension is 1571 and you
-	  dial 1571, usually a dialplan rule calls voicemailmain with your extension and
-	  you are placed directly in your mailbox.  Therefore most admins program the
-	  voicemail (or other speed dial) button on their phones to the access extension.
-	  Some phones (Snom at least) use whatever is programmed there to also subscribe
-	  for MWI and so can't dial one number and subscribe to another.  This works fine
-	  in chan_sip because chan_sip completely ignores the user portion of the
-	  SUBSCRIBE message request URI.  If it can match the peer, is subscribes to the
-	  peer's mailbox.  The user could be set to anything or nothing and you'd still
-	  get subscribed to your mailbox.
+	  ASTERISK-18105 #close
+	  Reported by: feyfre
+	  ........
 
-	  Issue:
+	  Merged revisions 432341 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  chan_pjsip actually uses the user portion of the URI to find an aor and its
-	  mailboxes.  Therefore a subscribe to 1571 results in a 404.  Sure, you can
-	  create an aor for 1571 but you certainly can't add your entire voicemail
-	  system's mailboxes to it and everyone would get notified of every MWI.
+	  Merged revisions 432342 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Solution:
 
-	  When an MWI subscribe comes in and an aor can't be found that matches the
-	  resource directly, check the resource against the endpoint's aors.  If an aor
-	  is found that has a voicemail_extension that matches the resource, use it.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432343 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25865
-	  Reported-by: Ross Beer
+2015-02-25 21:03 +0000 [3725173b9e]  Makoto Dei (License 5027)
 
-	  Change-Id: I770ea185f751f1ada888fafb4b452115f1c06e9e
+	* channels/chan_sip: Don't send a BYE after final response when PBX thread fails
 
-2016-03-24 22:55 +0000 [d8f0bc3572]  gtjoseph <george.joseph@fairview5.com>
+	  When Asterisk fails to start a PBX thread for a new channel - for example, when
+	  the maxcalls setting in asterisk.conf is exceeded - we currently send a final
+	  response, and then attempt to send a BYE request to the UA. Since that's all
+	  sorts of wrong, this patch fixes that by setting sipalreadygone on the sip_pvt
+	  such that we don't get stuck sending BYE requests to something that does not
+	  want it.
 
-	* res_pjsip_mwi:  Add voicemail extension and mwi_subscribe_replaces_unsolicited
+	  Note that this patch is a slight modification of the one on ASTERISK-15434.
+	  For clarity, it explicitly calls sipalreadygone with the calls to transmit a
+	  final response.
 
-	  res_pjsip_mwi was missing the chan_sip "vmexten" functionality which adds
-	  the Message-Account header to the MWI NOTIFY.  Also, specifying mailboxes
-	  on endpoints for unsolicited mwi and on aors for subscriptions required
-	  that the admin know in advance which the client wanted.  If you specified
-	  mailboxes on the endpoint, subscriptions were rejected even if you also
-	  specified mailboxes on the aor.
+	  ASTERISK-21845
+	  ASTERISK-15434 #close
+	  Reported by: Makoto Dei
+	  Tested by: Matt Jordan
+	  patches:
+	    sip-pbxstart-failed.patch uploaded by Makoto Dei (License 5027)
+	  ........
 
-	  Voicemail extension:
-	  * Added a global default_voicemail_extension which defaults to "".
-	  * Added voicemail_extension to both endpoint and aor.
-	  * Added ast_sip_subscription_get_dialog for support.
-	  * Added ast_sip_subscription_get_sip_uri for support.
+	  Merged revisions 432320 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  When an unsolicited NOTIFY is constructed, the From header is parsed, the
-	  voicemail extension from the endpoint is substituted for the user, and the
-	  result placed in the Message-Account field in the body.
+	  Merged revisions 432321 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When a subscribed NOTIFY is constructed, the subscription dialog local uri
-	  is parsed, the voicemail_extension from the aor (looked up from the
-	  subscription resource name) is substituted for the user, and the result
-	  placed in the Message-Account field in the body.
 
-	  If no voicemail extension was defined, the Message-Account field is not added
-	  to the NOTIFY body.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432322 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  mwi_subscribe_replaces_unsolicited:
-	  * Added mwi_subscribe_replaces_unsolicited to endpoint.
+2015-02-25 17:49 +0000 [e484140aed]  Rusty Newton <rnewton@digium.com>
 
-	  The previous behavior was to reject a subscribe if a previous internal
-	  subscription for unsolicited MWI was found for the mailbox.  That remains the
-	  default.  However, if there are mailboxes also set on the aor and the client
-	  subscribes and mwi_subscribe_replaces_unsolicited is set, the existing internal
-	  subscription is removed and replaced with the external subscription.  This
-	  allows an admin to configure mailboxes on both the endpoint and aor and allows
-	  the client to select which to use.
+	* configs/basic-pbx - Super Awesome Company example configs Phase 1, Patch 1
 
-	  ASTERISK-25865 #close
-	  Reported-by: Ross Beer
+	  Example configuration files for a "basic PBX" deployment for the fictitious
+	  Super Awesome Company. Details at https://reviewboard.asterisk.org/r/4379/
+	  and https://wiki.asterisk.org/wiki/display/AST/Super+Awesome+Company
 
-	  Change-Id: Ic15a9415091760539c7134a5ba3dc4a6a1217cea
+	  Reported by: Malcolm Davenport
+	  Tested by: Rusty Newton
 
-2016-03-30 09:46 +0000 [8dc8d6ceb8]  gtjoseph <george.joseph@fairview5.com>
+	  Review: https://reviewboard.asterisk.org/r/4379/
+	  ........
 
-	* res_rtp_asterisk:  Fix placement of txcount increment
+	  Merged revisions 432301 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Commit 1bce690ccb36a4744a327c07af23a9a3a0fa20cd was incrementing txcount
-	  for rtcp packets as well as rtp packets and that was causing sender reports
-	  to be generated instead of receiver reports in cases where no rtp was actually
-	  being sent.
 
-	  Moved the txcount increment from __rtp_sento, which handles both rtp and rtcp,
-	  to rtp_sento which only handles rtp packets.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432302 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Discovered by the hep/rtcp-receiver test.
+2015-02-25 17:09 +0000 [ced84d7e62]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ie442e4bb947a68847a676497021ba10ffaf376d5
+	* configure: Promote SQLite3 "not installed" warning to error
 
-2016-03-26 22:33 +0000 [c7eb18d865]  gtjoseph <george.joseph@fairview5.com>
+	  Since Asterisk won't build without the library, not having it is definitely
+	  an error. Thanks to Kyle Kurz for pointing this out.
+	  ........
 
-	* chan_pjsip:  Add 'pjsip show channelstats'
+	  Merged revisions 432280 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Added the ability to show channel statistics to chan_pjsip (cli_functions.c)
+	  Merged revisions 432281 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Moved the existing 'pjsip show channel(s)' functionality from
-	  pjsip_configuration to cli_functions.c.  The stats needed chan_pjsip's
-	  private header so it made sense to move the existing channel commands as well.
 
-	  Now using stasis_cache_dump to get the channel snapshots rather than retrieving
-	  all endpoints, then getting each one's channel snapshots.  Much more efficient.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432282 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I03b114522126d27434030b285bf6d531ddd79869
+2015-02-25 17:05 +0000 [4b63da7f7d]  Matt Jordan <mjordan@digium.com>
 
-2016-03-10 19:52 +0000 [1583559a06]  gtjoseph <george.joseph@fairview5.com>
+	* channels/chan_sip: Clarify WARNING message in mismatched SRTP scenario
 
-	* res_pjsip/pjsip_options:  Fix From generation on outgoing OPTIONS
+	  When we receive an SDP as part of an offer/answer for a peer/friend has been
+	  configured to require encryption, and that SDP offer/answer failed to provide
+	  acceptable crypto attributes, we currently issue a WARNING that uses the phrase
+	  "we" and "requested". In this case, both of those terms are ambiguous - the
+	  user will probably think "we" is Asterisk (it most likely isn't) and it may
+	  not be a "request", so much as an SDP that was received in some fashion.
 
-	  No one seemed to notice but every time an OPTIONS goes out, it goes
-	  out with a From of "asterisk" (or whatever the default from_user is set to),
-	  even if you specify an endpoint.
+	  This patch makes the WARNING messages slightly less bad and a bit more
+	  accurate as well.
 
-	  The issue had several causes...
-	  qualify_contact is only called with an endpoint if called from the CLI.
-	  If the endpoint is NULL, qualify_contact only looks up the endpoint if
-	  authenticate_qualify=yes. Even then, it never passes it on to
-	  ast_sip_create_request where the From header is set.  Therefore From
-	  is always "asterisk" (or whatever the default from_user is set to).
-	  Even if ast_sip_create_request were to get an endpoint, it only sets
-	  the From if endpoint->from_user is set.
+	  ASTERISK-23214 #close
+	  Reported by: Rusty Newton
+	  ........
 
-	  The fix is 4 parts...
+	  Merged revisions 432277 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  First, create_out_of_dialog_request was modified to use the endpoint id
-	  if endpoint was specified and from_user is not set.
+	  Merged revisions 432278 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Second, qualify_contact was modified to always look up an endpoint if
-	  one wasn't specified regardless of authenticate_qualify.  It then passes
-	  the endpoint on to create_out_of_dialog_request.
 
-	  Third (and most importantly), find_an_endpoint was modified to find
-	  an endpoint by using an "aors LIKE %contact->aor%" predicate with
-	  ast_sorcery_retrieve_by_fields.  As such, this patch will only work
-	  if the sorcery realtime optimizations patch goes in.  Otherwise we'd
-	  be pulling the entire endpoints database every time we send an OPTIONS.
-	  Since we already know the contact's aor, the on_endpoint callback was also
-	  modified to just check if the contact->aor is an exact match to one of
-	  the endpoint's.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432279 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Finally, since we now have an endpoint for every OPTIONS request,
-	  res_pjsip/endpt_send_request (which handles out-of-dialog reqests) was
-	  updated to get the transport from the endpoint and set it on tdata.
-	  Now the correct transport is used.
+2015-02-25 15:42 +0000 [d68012d1a3]  Olle Johansson <oej@edvina.net> (License 5267)
 
-	  Change-Id: I2207e12bb435e373bd1e03ad091d82e5aba011af
-2016-03-25 10:59 +0000 [0cfab30b28]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+	* channels/sip/sdp_crypto: Handle SRTP keys negotiated with key lifetime/MKI
+
+	  Prior to this patch, SDP offers negotiating SDES-SRTP crypto attributes would
+	  be rejected if those crypto attributes contained either a key lifetime or a
+	  MKI parameter. While from a theoretical point of view this was defensible -
+	  Asterisk does not support key lifetimes or multiple crypto keys - from a
+	  practical point of view, this is quite a problem. A large number of endpoints
+	  offer lifetimes/MKI, which Asterisk can tolerate so long as it doesn't actually
+	  have to support anything more than a single key or refresh the key.
+	  In reality, this is (so far as we've seen) always the case.
+
+	  This patch is a forward port of Olle's work in the lingon-srtp-key-lifetime-1.8
+	  branch. To quote Olle from ASTERISK-17721, it handles lifetime/MKI parameters
+	  in the following fashion:
+
+	  > The Lingon branch now handle lifetime and MKI parameters.
+	  >
+	  > We only accept lifetimes up to max for the crypto and higher than 10 hours
+	  > for packetization of 20 ms (50 pps).
+	  >
+	  > We only handle MKI with index 1.
+	  >
+	  > We do not really bother with counting packets and reinviting at end of
+	  > lifetime, so the min of 10 hours kind of takes care of most calls. If there
+	  > are longer ones, we rely on the other side for re-invites.
+	  >
+	  > It's still not perfect, but I personally think this is an improvement. A
+	  > configuration option for minimum lifetime accepted could be added.
+
+	  When the patch was ported forward, I decided against adding a configuration
+	  option as Olle's handling was more than sufficient for every case I've seen
+	  come through the issue tracker or through interoperability testing. We can
+	  revisit that decision if it proves to be false.
+
+	  A few small other tweaks were made to the surrounding code to reduce
+	  indentation and provide better type safety for the 'tag' parameter.
 
-	* res_rtp_asterisk: Use separate SRTP session for RTCP with DTLS
+	  Review: https://reviewboard.asterisk.org/r/4419/
+	  Review: https://reviewboard.asterisk.org/r/4418/
 
-	  Asterisk uses separate UDP ports for RTP and RTCP traffic and RFC 5764
-	  explicitly states:
+	  ASTERISK-17721 #close
+	  Reported by: Terry Wilson
 
-	    There MUST be a separate DTLS-SRTP session for each distinct pair of
-	    source and destination ports used by a media session
+	  ASTERISK-17899 #close
+	  Reported by: Dwayne Hubbard
+	  patches:
+	    lingon-srtp-key-lifetime-1.8.diff uploaded by oej (License 5267)
 
-	  This means RTP keying material cannot be used for DTLS RTCP, which was
-	  the reason why RTCP encryption would fail.
+	  ASTERISK-20233
+	  Reported by: tootai
 
-	  ASTERISK-25642
+	  ASTERISK-22748
+	  Reported by: Alejandro Mejia
+	  ........
 
-	  Change-Id: I7e8779d8b63e371088081bb113131361b2847e3a
+	  Merged revisions 432239 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-03-25 10:42 +0000 [6a9c18fb59]  Jacek Konieczny <jkonieczny@eggsoft.pl>
+	  Merged revisions 432258 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* app_echo: forward and generate VIDUPDATE frames
 
-	  When using app_echo via WebRTC with VP8 video the video would appear
-	  only after a few minutes, because there would be nothing to request
-	  a full reference frame.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432259 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This fixes the problem in both ways:
-	  - echos any VIDUPDATE frames received on the channel
-	  - sends one such frame when first video frame is to be forwarded
+2015-02-25 14:47 +0000 [ff642289f4]  David M. Lee <dlee@digium.com>
 
-	  This makes the echo work with Firefox and Chrome WebRTC implementation.
+	* Increase WebSocket frame size and improve large read handling
 
-	  ASTERISK-25867 #close
+	  Some WebSocket applications, like [chan_respoke][], require a larger
+	  frame size than the default 8k; this patch bumps the default to 16k.
+	  This patch also fixes some problems exacerbated by large frames.
 
-	  Change-Id: I73bda87bf7532ee8bfb28d917045a21034908c1e
+	  The sanity counter was decremented on every fread attempt in
+	  ws_safe_read(), regardless of whether data was read from the socket or
+	  not. For large frames, this could result in loss of sanity prior to
+	  reading the entire frame. (16k frame / 1448 bytes per segment = 12
+	  segments).
 
-2016-03-27 12:53 +0000 [1bce690ccb]  gtjoseph <george.joseph@fairview5.com>
+	  This patch changes the sanity counter so that it only decrements when
+	  fread() doesn't read any bytes. This more closely matches the original
+	  intention of ws_safe_read(), given that the error message is
+	  "Websocket seems unresponsive".
 
-	* res_rtp_asterisk:  Fix packet stats on bridged connection
+	  This patch also properly logs EOF conditions, so disconnects are no
+	  longer confused with unresponsive connections.
 
-	  rxcount, txcount, rxoctetcount and txoctetcount weren't being calculated
-	  for bridged streams because the calulations were being done after the
-	  bridged short-circuit.  Actually, rxoctetcount wasn't ever being calculated.
+	   [chan_respoke]: https://github.com/respoke/chan_respoke
 
-	  Moved the calculations so they occur for all valid received packets and
-	  all transmitted packets.  Also added rxoctetcount and txoctetcount to
-	  ast_rtp_instance_stat.
+	  Review: https://reviewboard.asterisk.org/r/4431/
+	  ........
 
-	  Change-Id: I08fb06011a82d38c3b4068867a615068fbe59cbb
+	  Merged revisions 432236 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-03-25 23:19 +0000 [50f90d4099]  Philip Correia
+	  Merged revisions 432237 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_parking: Fix blind transfer dynamic lots creation.
 
-	  Blind transfers to a recognized parking extension need to use the parker's
-	  channel variable values to create the dynamic parking lot.  This is
-	  because there is always only one parker while the parkee may actually be a
-	  multi-party bridge.  A multi-party bridge can never supply the needed
-	  channel variables to create the dynamic parking lot.  In the multi-party
-	  bridge blind transfer scenario, the parker's CHANNEL(parkinglot) value and
-	  channel variables are inherited by the local channel used to park the
-	  bridge.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432238 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * In park_common_setup(), make use the parker instead of the parkee to
-	  supply the dynamic parking lot channel variable values.  In all but one
-	  case, the parkee is the same as the parker.  However, in the recognized
-	  parking extension blind transfer scenario for a two party bridge they are
-	  different channels.  For consistency, we need to use the parker channel.
+2015-02-24 17:00 +0000 [57525c3cf2]  Richard Mudgett <rmudgett@digium.com>
 
-	  * In park_local_transfer(), pass the CHANNEL(parkinglot) value to the
-	  local channel when blind transferring a multi-party bridge to a recognized
-	  parking extension.
+	* config.h: Use real parameter names for ast_variable_new() define.
 
-	  * When a local channel starts a call, the Local;2 side needs to inherit
-	  the CHANNEL(parkinglot) value from Local;1.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432220 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The DTMF one-touch parking case wasn't even trying to create dynamic
-	  parking lots before it aborted the attempt.
+2015-02-24 16:14 +0000 [8574c4d197]  Matt Jordan <mjordan@digium.com>
 
-	  * In parking_park_call(), add missing code to create a dynamic parking
-	  lot.
+	* channels/chan_sip: Fix crash when transmitting packet after thread shutdown
 
-	  A DTMF bridge hook is documented as returning -1 to remove the hook.
-	  Though the hook caller is really coded to accept non-zero.  See the
-	  ast_bridge_hook_callback typedef.
+	  When the monitor thread is stopped, its pthread ID is set to a specific value
+	  (AST_PTHREADT_STOP) so that later portions of the code can determine whether
+	  or not it is safe to manipulate the thread. Unfortunately, __sip_reliable_xmit
+	  failed to check for that value, checking instead only for AST_PTHREAD_STOP.
+	  Passing the invalid yet very specific value to pthread_kill causes a crash.
 
-	  * In feature_park_call(), don't remove the DTMF one-touch parking hook
-	  because of an error.
+	  This patch adds a check for AST_PTHREADT_STOP in __sip_reliable_xmit such that
+	  it doesn't attempt to poke the thread if the thread has already been stopped.
 
-	  ASTERISK-24605 #close
-	  Reported by:  Philip Correia
-	  Patches:
-	        call_park.patch (license #6672) patch uploaded by Philip Correia
+	  ASTERISK-24800 #close
+	  Reported by: JoshE
+	  ........
 
-	  Change-Id: I221d3a8fcc181877a1158d17004474d35d8016c9
+	  Merged revisions 432198 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-03-08 15:55 +0000 [5aa5c49413]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 432199 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* sorcery/res_pjsip:  Refactor for realtime performance
 
-	  There were a number of places in the res_pjsip stack that were getting
-	  all endpoints or all aors, and then filtering them locally.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432200 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A good example is pjsip_options which, on startup, retrieves all
-	  endpoints, then the aors for those endpoints, then tests the aors to see
-	  if the qualify_frequency is > 0.  One issue was that it never did
-	  anything with the endpoints other than retrieve the aors so we probably
-	  could have skipped a step and just retrieved all aors. But nevermind.
+2015-02-24 16:00 +0000 [a528dfc9a7]  Matt Jordan <mjordan@digium.com>
 
-	  This worked reasonably well with local config files but with a realtime
-	  backend and thousands of objects, this was a nightmare.  The issue
-	  really boiled down to the fact that while realtime supports predicates
-	  that are passed to the database engine, the non-realtime sorcery
-	  backends didn't.
+	* ARI/PJSIP: Apply requesting channel's format cap to created channels
 
-	  They do now.
+	  This patch addresses the following problems:
+	  * ari/resource_channels: In ARI, we currently create a format capability
+	    structure of SLIN and apply it to the new channel being created. This was
+	    originally done when the PBX core was used to create the channel, as there
+	    was a condition where a newly created channel could be created without any
+	    formats. Unfortunately, now that the Dial API is being used, this has two
+	    drawbacks:
+	    (a) SLIN, while it will ensure audio will flows, can cause a lot of
+	        needless transcodings to occur, particularly when a Local channel is
+	        created to the dialplan. When no format capabilities are available, the
+	        Dial API handles this better by handing all audio formats to the requsted
+	        channels. As such, we defer to that API to provide the format
+	        capabilities.
+	    (b) If a channel (requester) is causing this channel to be created, we
+	        currently don't use its format capabilities as we are passing in our own.
+	        However, the Dial API will use the requester channel's formats if none
+	        are passed into it, and the requester channel exists and has format
+	        capabilities. This is the "best" scenario, as it is the most likely to
+	        create a media path that minimizes transcoding.
+	    Fixing this simply entails removing the providing of the format capabilities
+	    structure to the Dial API.
+
+	  * chan_pjsip: Rather than blindly picking the first format in the format
+	    capability structure - which actually *can* be a video or text format - we
+	    select an audio format, and only pick the first format if that fails. That
+	    minimizes the weird scenario where we attempt to transcode between video/audio.
+
+	  * res_pjsip_sdp_rtp: Applied the joint capapbilites to the format structure.
+	    Since ast_request already limits us down to one format capability once the
+	    format capabilities are passed along, there's no reason to squelch it here.
+
+	  * channel: Fixed a comment. The reason we have to minimize our requested
+	    format capabilities down to a single format is due to Asterisk's inability
+	    to convey the format to be used back "up" a channel chain. Consider the
+	    following:
+
+	      PJSIP/A => L;1 <=> L;2 => PJSIP/B
+	      g,u,a     g,u,a    g,u,a      u
+
+	    That is, we have PJSIP/A dialing a Local channel, where the Local;2 dials
+	    PJSIP/B. PJSIP/A has native format capabilities g722,ulaw,alaw; the Local
+	    channel has inherited those format capabilities down the line; PJSIP/B
+	    supports only ulaw. According to these format capabilities, ulaw is
+	    acceptable and should be selected across all the channels, and no
+	    transcoding should occur. However, there is no way to convey this: when L;2
+	    and PJSIP/B are put into a bridge, we will select ulaw, but that is not
+	    conveyed to PJSIP/A and L;1. Thus, we end up with:
+
+	      PJSIP/A <=> L;1 <=> L;2 <=> PJSIP/B
+	        g          g   X   u        u
+
+	    Which causes g722 to be written to PJSIP/B.
+
+	    Even if we can convey the 'ulaw' choice back up the chain (which through
+	    some severe hacking in Local channels was accomplished), such that the chain
+	    looks like:
+
+	      PJSIP/A <=> L;1 <=> L;2 <=> PJSIP/B
+	        u          u       u         u
+
+	    We have no way to tell PJSIP/A's *channel driver* to Answer in the SDP back
+	    with only 'ulaw'. This results in all the channel structures being set up
+	    correctly, but PJSIP/A *still* sending g722 and causing the chain to fall
+	    apart.
+
+	    There's a lot of difficulty just in setting this up, as there are numerous
+	    race conditions in the act of bridging, and no clean mechanism to pass the
+	    selected format backwards down an established channel chain. As such, the
+	    best that can be done at this point in time is clarifying the comment.
 
-	  The realtime engines have a scheme for doing simple comparisons. They
-	  take in an ast_variable (or list) for matching, and the name of each
-	  variable can contain an operator.  For instance, a name of
-	  "qualify_frequency >" and a value of "0" would create a SQL predicate
-	  that looks like "where qualify_frequency > '0'".  If there's no operator
-	  after the name, the engines add an '=' so a simple name of
-	  "qualify_frequency" and a value of "10" would return exact matches.
+	  Review: https://reviewboard.asterisk.org/r/4434/
 
-	  The non-realtime backends decide whether to include an object in a
-	  result set by calling ast_sorcery_changeset_create on every object in
-	  the internal container.  However, ast_sorcery_changeset_create only does
-	  exact string matches though so a name of "qualify_frequency >" and a
-	  value of "0" returns nothing because the literal "qualify_frequency >"
-	  doesn't match any name in the objset set.
+	  ASTERISK-24812 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  So, the real task was to create a generic string matcher that can take a
-	  left value, operator and a right value and perform the match. To that
-	  end, strings.c has a new ast_strings_match(left, operator, right)
-	  function.  Left and right are the strings to operate on and the operator
-	  can be a string containing any of the following: = (or NULL or ""), !=,
-	  >, >=, <, <=, like or regex.  If the operator is like or regex, the
-	  right string should be a %-pattern or a regex expression.  If both left
-	  and right can be converted to float, then a numeric comparison is
-	  performed, otherwise a string comparison is performed.
+	  Merged revisions 432195 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  To use this new function on ast_variables, 2 new functions were added to
-	  config.c.  One that compares 2 ast_variables, and one that compares 2
-	  ast_variable lists.  The former is useful when you want to compare 2
-	  ast_variables that happen to be in a list but don't want to traverse the
-	  list.  The latter will traverse the right list and return true if all
-	  the variables in it match the left list.
 
-	  Now, the backends' fields_cmp functions call ast_variable_lists_match
-	  instead of ast_sorcery_changeset_create and they can now process the
-	  same syntax as the realtime engines.  The realtime backend just passes
-	  the variable list unaltered to the engine.  The only gotcha is that
-	  there's no common realtime engine support for regex so that's been noted
-	  in the api docs for ast_sorcery_retrieve_by_fields.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432196 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Only one more change to sorcery was done...  A new config flag
-	  "allow_unqualified_fetch" was added to reg_sorcery_realtime.
-	  "no": ignore fetches if no predicate fields were supplied.
-	  "error": same as no but emit an error. (good for testing)
-	  "yes": allow (the default);
-	  "warn": allow but emit a warning. (good for testing)
+2015-02-24 12:38 +0000 [91733b5d15]  Kevin Harwell <kharwell@digium.com>
 
-	  Now on to res_pjsip...
+	* bridge_softmix: G.729 codec license held
 
-	  pjsip_options was modified to retrieve aors with qualify_frequency > 0
-	  rather than all endpoints then all aors.  Not only was this a big
-	  improvement in realtime retrieval but even for config files there's an
-	  improvement because we're not going through endpoints anymore.
+	  When more than one call using the same codec type enters into a softmix bridge
+	  and no audio is present for a channel the bridge optimizes the out frame by
+	  using the same one for all channels with the same codec type. Unfortunately,
+	  when that number (channels with same codec type) dropped to <= 1 the codec
+	  was not dereferenced. At least not until all parties left the bridge. Thus in
+	  the case of G.729 the license was not released. This patch ensures that the
+	  codec is dereferenced immediately when the optimization no longer applies.
 
-	  res_pjsip_mwi was modified to retieve only endpoints with something in
-	  the mailboxes field instead of all endpoints then testing mailboxes.
+	  ASTERISK-24797 #close
+	  Reported by: Luke Hulsey
+	  Review: https://reviewboard.asterisk.org/r/4429/
+	  ........
 
-	  res_pjsip_registrar_expire was completely refactored.  It was retrieving
-	  all contacts then setting up scheduler entries to check for expiration.
-	  Now, it's a single thread (like keepalive) that periodically retrieves
-	  only contacts whose expiration time is < now and deletes them.  A new
-	  contact_expiration_check_interval was added to global with a default of
-	  30 seconds.
+	  Merged revisions 432174 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Ross Beer reports that with this patch, his Asterisk startup time dropped
-	  from around an hour to under 30 seconds.
+	  Merged revisions 432175 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  There are still objects that can't be filtered at the database like
-	  identifies, transports, and registrations.  These are not going to be
-	  anywhere near as numerous as endpoints, aors, auths, contacts however.
 
-	  Back to allow_unqualified_fetch.  If this is set to yes and you have a
-	  very large number of objects in the database, the pjsip CLI commands
-	  will attempt to retrive ALL of them if not qualified with a LIKE.
-	  Worse, if you type "pjsip show endpoint <tab>" guess what's going to
-	  happen? :)  Having a cache helps but all the objects will have to be
-	  retrieved at least once to fill the cache.  Setting
-	  allow_unqualified_fetch=no prevents the mass retrieve and should be used
-	  on endpoints, auths, aors, and contacts.  It should NOT be used for
-	  identifies, registrations and transports since these MUST be
-	  retrieved in bulk.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432176 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Example sorcery.conf:
+2015-02-21 14:48 +0000 [bedf51b2ce]  Joshua Colp <jcolp@digium.com>
 
-	  [res_pjsip]
-	  endpoint=config,pjsip.conf,criteria=type=endpoint
-	  endpoint=realtime,ps_endpoints,allow_unqualified_fetch=error
+	* res_ari_channels: Return a 404 response when a requested channel variable does not exist.
 
-	  ASTERISK-25826 #close
-	  Reported-by: Ross Beer
-	  Tested-by: Ross Beer
+	  This change makes it so that if a channel variable is requested and it does not exist
+	  a 404 response will be returned instead of an allocation failed response. This makes
+	  it easier to debug and figure out what is going on for a user.
 
-	  Change-Id: Id2691e447db90892890036e663aaf907b2dc1c67
+	  ASTERISK-24677 #close
+	  Reported by: Joshua Colp
+	  ........
 
-2016-03-23 14:24 +0000 [05fc3a96d1]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 432154 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_parking: Cleanup find_channel_parking_lot_name() usage.
 
-	  Change-Id: I8f7a8890aef27824301c642d4d15407ac83e6f02
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432155 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-18 14:01 +0000 [a4189763ab]  Richard Mudgett <rmudgett@digium.com>
+2015-02-21 13:28 +0000 [87b7060f36]  Joshua Colp <jcolp@digium.com>
 
-	* res_parking: Misc fixes.
+	* res_pjsip_registrar: Add Expires header to 200 OK if present in REGISTER.
 
-	  res/parking/parking_applications.c:
+	  Some implementations don't pay attention to the expires for individual contacts.
+	  In this case they may consider the lack of an Expires header in the 200 OK as
+	  unregistered. This change makes it so if an Expires header is present in the REGISTER
+	  we will add one in the 200 OK.
 
-	  * Add malloc fail checks in setup_park_common_datastore().
+	  ASTERISK-24785 #close
+	  Reported by: Ross Beer
+	  ........
 
-	  * Fix playing parking failed announcement to only happen on non-blind
-	  transfers in park_app_exec().  It could never go out before because a test
-	  was provedly always false.
+	  Merged revisions 432136 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  res/parking/parking_bridge.c:
 
-	  * Fix NULL tolerance in generate_parked_user() because
-	  bridge_parking_push() can theoretically pass a NULL parker channel if the
-	  parker channel went away for some reason.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432137 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Clarify some weird code dealing with blind_transfer in
-	  bridge_parking_push().
+2015-02-21 12:53 +0000 [283bb15c16]  Joshua Colp <jcolp@digium.com>
 
-	  res/parking/parking_bridge_features.c:
+	* res_pjsip: Add a log message when creating a UAC dialog to a target URI that is invalid.
 
-	  * Made park_local_transfer() set BLINDTRANSFER on the Local;1 channel
-	  which will be bulk copied to the Local;2 channel on the subsequent
-	  ast_call().  The additional advantage is if the parker channel has the
-	  BLINDTRANSFER and ATTENDEDTRANSFER variables set they are now guaranteed
-	  to be overridden.
+	  ASTERISK-24499 #close
+	  Reported by: Rusty Newton
+	  ........
 
-	  res/parking/parking_manager.c:
+	  Merged revisions 432118 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Fix AMI Park action input range checking of the Timeout header in
-	  manager_park().
 
-	  * Reduced locking scope to where needed in manager_park().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432119 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  res/res_parking.c:
+2015-02-21 11:36 +0000 [b3c1ad5d73]  Graham Barnett (License 6685)
 
-	  * Fix some off nominal missing unlocks by eliminating the returns.
+	* apps/app_voicemail: Demote an ERROR message to a WARNING message
 
-	  Change-Id: Ib64945bc285acb05a306dc12e6f16854898915ca
+	  When using IMAP voicemail with FreePBX, you will often get ERROR messages
+	  complaining about not being able to find a mailbox. This is due to how FreePBX
+	  handles voicemail mailboxes. Unfortunately, app_voicemail has to consider this
+	  a configuration error, as in any other system it would be indicative of
+	  someone misconfiguring their system.
 
-2014-12-15 05:23 +0000 [6f95b5eda1]  Philip Correia
+	  Regardless, a misconfiguration is a WARNING, and not an ERROR. This patch
+	  demotes the message so that system administrators can hopefully reduce some
+	  of the noise in their log files.
 
-	* res_parking: Update parking documentation for dynamic parking lots.
+	  Note that in the original patch this was made into a NOTICE, but that's a
+	  too forgiving.
 
-	  * Remove duplicate res_parking.conf courtesytone config option
-	  documentation.
+	  ASTERISK-24790 #close
+	  Reported by: Graham Barnett
+	  patches:
+	    app_voicemail.c.patch_noise uploaded by Graham Barnett (License 6685)
+	  ........
 
-	  ASTERISK-24596 #close
-	  Reported by:  Philip Correia
+	  Merged revisions 432098 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-24605
-	  Reported by:  Philip Correia
-	  Patches:
-	        call_park_app_doc.patch (license #6672) patch uploaded by Philip Correia
+	  Merged revisions 432099 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I90a92a891c6494dc08173e675856afcc4764c5b5
 
-2016-03-24 14:08 +0000 [81ce60f6d4]  Alexander Traud <pabstraud@compuserve.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432100 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: Do not send all codecs on INVITE. Do not break on Session-Timers.
+2015-02-21 08:06 +0000 [2ea7ccbf70]  Joshua Colp <jcolp@digium.com>
 
-	  Asterisk 13.7.0 included a fix for ASTERISK-24543, not to send all those
-	  codecs, which the caller did not request/support. That fix was not complete
-	  because on the second Session Timer all codecs were sent again. Some VoIP/SIP
-	  clients interpreted that complete codec-list as a change in the SIP session.
-	  Because of that, Asterisk did not send the RTP audio via NAT anymore which
-	  created a non-audio scenario after the second Session Timer fired.
+	* http: Add missing html tag to 'httpstatus' functionality.
 
-	  ASTERISK-24543 #close
+	  ASTERISK-24724 #close
+	  Reported by: Ashley Sanders
+	  ........
 
-	  Change-Id: I1881827816ab7fd47eb4287a95961179b34a0b66
+	  Merged revisions 432078 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-03-19 07:34 +0000 [c6e4c48e67]  Gianluca Merlo <gianluca.merlo@gmail.com>
+	  Merged revisions 432079 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* config: fix flags in uint option handler
 
-	  The configuration unsigned integer option handler sets flags for the
-	  parser as if the option should be a signed integer (PARSE_INT32),
-	  leading to errors on "out of range" values. Fix flags (PARSE_UINT32).
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432080 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A fix to res_pjsip is also present which stops invalid flags from
-	  being passed when registering sorcery object fields for qualify
-	  status.
+2015-02-20 20:58 +0000 [e66b874f5d]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25612 #close
+	* Allow shutdown to unload modules that register bucket scheme's or codec's.
 
-	  Change-Id: I96b539336275e0e72a8e8033487d2c3344debd3e
+	  * Change __ast_module_shutdown_ref to be NULL safe (11+).
+	  * Allow modules that call ast_bucket_scheme_register or ast_codec_register
+	    to be unloaded during graceful shutdown only (13+ only).
 
-2016-03-10 16:58 +0000 [59c8e189fd]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24796 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4428/
+	  ........
 
-	* Restrict CLI/AMI commands on shutdown.
+	  Merged revisions 432058 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  During stress testing, we have frequently seen crashes occur because a
-	  CLI or AMI command attempts to access information that is in the process
-	  of being destroyed.
+	  Merged revisions 432059 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When addressing how to fix this issue, we initially considered fixing
-	  individual crashes we observed. However, the changes required to fix
-	  those problems would introduce considerable overhead to the nominal
-	  case. This is not reasonable in order to prevent a crash from occurring
-	  while Asterisk is already shutting down.
 
-	  Instead, this change makes it so AMI and CLI commands cannot be executed
-	  if Asterisk is being shut down. For AMI, this is absolute. For CLI,
-	  though, certain commands can be registered so that they may be run
-	  during Asterisk shutdown.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432060 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25825 #close
+2015-02-20 20:51 +0000 [bb71672a47]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I8887e215ac352fadf7f4c1e082da9089b1421990
-2016-03-24 07:45 +0000 [ff3eebf454]  Walter Doekes <walter+asterisk@wjd.nu>
+	* main/asterisk.c: Reverse #if statement in listener() to fix code folding.
 
-	* musiconhold: Only warn if music class is not found in memory and database.
+	  listener() opens the same code block in two places (#if and #else).  This
+	  confuses some folding editors causing it to think that an extra code block
+	  was opened.  Folding in 'geany' causes all code after listener() to be
+	  folded as if it were part of that procedure.
 
-	  The log message when a MusicOnHold music class was not found was changed
-	  from debug level to WARNING level in Asterisk 11.19 and 13.5.  For those
-	  using realtime musiconhold, this message is wrong because it warns
-	  before checking the database.
+	  ASTERISK-24813 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4435/
 
-	  This changeset delays the warning until after the database has been
-	  checked.
 
-	  Reported-by: Conrad de Wet
-	  ASTERISK-25444 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432057 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I6cfb2db2f9cfbd2bb3d30566ecae361c4abf6dbf
+2015-02-20 20:47 +0000 [ce50fa314a]  Corey Farrell <git@cfware.com>
 
-2016-03-24 05:38 +0000 [82e55e4883]  Walter Doekes <walter+asterisk@wjd.nu>
+	* asterisk/lock.h: Fix syntax errors for non-gcc OSX with 64-bit integers.
 
-	* core/logging: Fix broken syslog levels on older glibc.
+	  Add a couple of missing closing brackets / parenthesis.
 
-	  The fix to ASTERISK-25407 introduced the usage of LOG_MAKEPRI. However
-	  this macro is broken in older glibc (< 2.17); it would left-shift the
-	  facility a second time, causing the resultant priority to become
-	  invalid.
+	  ASTERISK-24814 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4436/
+	  ........
 
-	  The syslog manpage mentions nothing about LOG_MAKEPRI and suggests this:
+	  Merged revisions 432054 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	      The priority argument is formed by ORing the facility and the level
-	      values [...].
+	  Merged revisions 432055 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25510 #close
-	  Reported by: Michael Newton
 
-	  Change-Id: Ia89debe7fac5ad090c7ef595c0707f31bb1e3d03
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432056 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-23 08:59 +0000 [d963a33749]  gtjoseph <george.joseph@fairview5.com>
+2015-02-20 11:55 +0000 [bb06603d5f]  Richard Mudgett <rmudgett@digium.com>
 
-	* pjproject-bundled:  Cleanups for reported issues
+	* chan_dahdi/sig_analog: Put log message strings on one line.
 
-	  PortAudio should no longer be required
-	  PJSIP_MAX_PKT_LEN is now 6000
-	  Older autoconf issue fixed. (CentOS 6)
+	  With the log messages on one line, you can search for the log message seen
+	  in the log and expect to find it.
+	  ........
 
-	  Change-Id: I463fa9586cbe7c6b3b603289f535bd8e361611dd
+	  Merged revisions 432032 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-11-20 08:02 +0000 [c5170677e7]  Francesco Castellano <francesco.castellano@messagenet.it>
+	  Merged revisions 432034 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip.c: Space after port causes unnecessary resolution attempt
 
-	  check_via() already skips leading blanks where the sent-by address (with the
-	  optional port) should be placed.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432036 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Since RFC 3261 allows for blanks between the port ant the Via parameters:
-	  > https://tools.ietf.org/html/rfc3261#section-20.42
-	  (actually it allows a lot of blanks more ;-)). I just switched from
-	  ast_skip_blanks() to ast_strip() on the local copy of the string.
+2015-02-20 11:53 +0000 [340818ad12]  Matt Hoskins (license 6688)
 
-	  ASTERISK-21301 #close
+	* ASTERISK-24811: Add ast_sorcery_apply_config() to res_pjsip_publish_asterisk.
 
-	  Change-Id: Ie5b8fe5a07067b7c0dc9bcdd1707e99b23b02b06
+	  Matt Hoskins reported that res_pjsip_publish_asterisk wouldn't pull config from 
+	  realtime.  Turns out it was just missing a call ast_sorcery_apply_config().
 
-2016-03-19 17:49 +0000 [51deadee38]  gtjoseph <george.joseph@fairview5.com>
+	  res_pjsip_acl was missing it as well, so I added it.  The other pjsip modules 
+	  looked OK.
 
-	* progdocs:  Exclude ./third-party from documentation generation
+	  ASTERISK-24811 #close
+	  Reported-by: Matt Hoskins
+	  Tested-by: George Joseph
+	  Tested-by: Matt Hoskins
+	  patches:
+	  	res_pjsip_publish_asterisk.c.patch submitted by Matt Hoskins (license 6688)
 
-	  We don't need pjproject's documentation embedded in Asterisk's.
+	  Review: https://reviewboard.asterisk.org/r/4433/
+	  ........
 
-	  Change-Id: Iea6f5a621c0f4e3168dda3321eaab258d9f24a17
+	  Merged revisions 432033 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-18 20:32 +0000 [aa2fcd244e]  Gianluca Merlo <gianluca.merlo@gmail.com>
 
-	* func_aes: fix misuse of strlen on binary data
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432035 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The encryption code for AES_ENCRYPT evaluates the length of the data to
-	  be encoded in base64 using strlen. The data is binary, thus the length
-	  of it can be underestimated at the first NULL character.
-	  Reuse the write pointer offset to evaluate it, instead.
+2015-02-20 09:47 +0000 [4dab71831f]  Graham Barnett (License 6685)
 
-	  ASTERISK-25857 #close
+	* apps/app_voicemail: Fix IMAP header compatibility issue with Microsoft Exchange
 
-	  Change-Id: If686b5d570473eb926693c73461177b35b13b186
+	  When interfacing with Microsoft Exchange, custom headers will be returned as
+	  all lower case. Currently, the IMAP header code will fail to parse the returned
+	  custom headers, as it will be performing a case sensitive comparison. This can
+	  cause playback of messages to fail, as needed information - such as origtime -
+	  will not be present.
 
-2016-04-20 10:46 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  This patch updates app_voicemail's header parsing code to perform a case
+	  insensitive lookup for the requested custom headers. Since the headers are
+	  specific to Asterisk, e.g., 'x-asterisk-vm-orig-time', and headers should be
+	  unique in an IMAP message, this should cause no issues with other systems.
 
-	* asterisk 13.8.2 Released.
+	  ASTERISK-24787 #close
+	  Reported by: Graham Barnett
+	  patches:
+	    app_voicemail.c.patch_MSExchange uploaded by Graham Barnett (License 6685)
+	  ........
 
-2016-04-20 05:45 +0000 [26d67ce885]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 432012 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* Release summaries: Remove previous versions
+	  Merged revisions 432013 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-20 05:45 +0000 [d9909232ed]  Joshua Colp <jcolp@digium.com>
 
-	* .version: Update for 13.8.2
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@432014 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-20 05:45 +0000 [fc57bb9b15]  Joshua Colp <jcolp@digium.com>
+2015-02-19 15:26 +0000 [05cc6d6d55]  Richard Mudgett <rmudgett@digium.com>
 
-	* .lastclean: Update for 13.8.2
+	* chan_dahdi: Remove some dead code.
+	  ........
 
-2016-04-20 05:45 +0000 [ac04474f38]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 431992 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* realtime: Add database scripts for 13.8.2
+	  Merged revisions 431993 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-18 17:00 +0000 [91a3e1184f]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip_registrar: Fix bad memory-ness with user_agent.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431994 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Recent changes to the PJSIP registrar resulted in tests failing due to
-	  missing AOR_CONTACT_ADDED test events. The reason for this was that the
-	  user_agent string had junk values in it, resulting in being unable to
-	  generate the event.
+2015-02-19 12:26 +0000 [252aee4228]  Richard Mudgett <rmudgett@digium.com>
 
-	  I'm going to be honest here, I have no idea why this was happening. Here
-	  are the steps needed for the user_agent variable to get messed up:
-	  * REGISTER is received
-	  * First contact in the REGISTER results in a contact being removed
-	  * Second contact in the REGISTER results in a contact being added
-	  * The contact, AOR, expiration, and user agent all have to be passed as
-	    format parameters to the creation of a string. Any subset of those
-	    parameters would not be enough to cause the problem.
+	* ISDN AOC: Fix crash from an AOC-E message that doesn't have a channel association.
 
-	  Looking into what was happening, the thing that struck me as odd was
-	  that the user_agent variable was meant to be set to the value of the
-	  User-Agent SIP header in the incoming REGISTER. However, when removing a
-	  contact, the user_agent variable would be set (via ast_strdupa inside a
-	  loop) to the stored contact's user_agent. This means that the
-	  user_agent's value would be incorrect when attempting to process further
-	  contacts in the incoming REGISTER.
+	  Processing an AOC-E event that does not or no longer has a channel
+	  association causes a crash.
 
-	  The fix here is to use a different variable for the stored user agent
-	  when removing a contact. Correcting the behavior to be correct also
-	  means the memory usage is less weird, and the issue no longer occurs.
+	  The problem with posting AOC events to the channel topic is that AOC-E
+	  events don't always have a channel association and posting the event to
+	  the all channels topic is just wrong.  AOC-E events do however have their
+	  own charging association method to refer to the agreement with the
+	  charging entity.
 
-	  ASTERISK-25929 #close
-	  Reported by Joshua Colp
+	  * Changed the AOC events to post to the AMI manager topic instead of the
+	  channel topics.  If a channel is associated with the event then channel
+	  snapshot information is supplied with the AMI event.
 
-	  Change-Id: I7cd24c86a38dec69ebcc94150614bc25f46b8c08
-	  (cherry picked from commit f436b9ab111f1ff57c6dd3970051f123b42c1103)
+	  * Eliminated RAII_VAR() usage in aoc_to_ami() and ast_aoc_manager_event().
 
-2016-04-18 13:41 +0000 [70e25ced60]  Joshua Colp <jcolp@digium.com>
+	  This patch supercedes the patch on Review: https://reviewboard.asterisk.org/r/4427/
 
-	* res_pjsip_transport_management: Allow unload to occur.
+	  ASTERISK-22670 #close
+	  Reported by: klaus3000
 
-	  At shutdown it is possible for modules to be unloaded that wouldn't
-	  normally be unloaded. This allows the environment to be cleaned up.
+	  ASTERISK-24689 #close
+	  Reported by: Marcel Manz
 
-	  The res_pjsip_transport_management module did not have the unload
-	  logic in it to clean itself up causing the res_pjsip module to not
-	  get unloaded. As a result the res_pjsip monitor thread kept going
-	  processing traffic and timers when it shouldn't.
+	  ASTERISK-24740 #close
+	  Reported by: Panos Gkikakis
 
-	  Change-Id: Ic8cadee131e3b2c436a81d3ae8bb5775999ae00a
-	  (cherry picked from commit 49bfdc9ac029e0ef17cd8e85d8d7b7731387a34e)
+	  Review: https://reviewboard.asterisk.org/r/4430/
+	  ........
 
-2016-04-18 12:12 +0000 [856931edc2]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 431974 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* PJSIP: Remove PJSIP parsing functions from uri length validation.
 
-	  The PJSIP parsing functions provide a nice concise way to check the
-	  length of a hostname in a SIP URI. The problem is that in order to use
-	  those parsing functions, it's required to use them from a thread that
-	  has registered with PJLib.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431975 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  On startup, when parsing AOR configuration, the permanent URI handler
-	  may not be run from a PJLib-registered thread. Specifically, this could
-	  happen when Asterisk was started in daemon mode rather than
-	  console-mode. If PJProject were compiled with assertions enabled, then
-	  this would cause Asterisk to crash on startup.
+2015-02-19 11:37 +0000 [6992b2e8fa]  Richard Mudgett <rmudgett@digium.com>
 
-	  The solution presented here is to do our own parsing of the contact URI
-	  in order to ensure that the hostname in the URI is not too long. The
-	  parsing does not attempt to perform a full SIP URI parse/validation,
-	  since the hostname in the URI is what is important.
+	* res_pjsip_refer: Handle INVITE with Replaces failure after answer.
 
-	  ASTERISK-25928 #close
-	  Reported by Joshua Colp
+	  * Fixed hangup handling of the session->channel after answer if the
+	  ast_channel_move() or ast_bridge_impart() fails.  We are still the thread
+	  controlling the session->channel so we need to call ast_hangup() to kill
+	  the channel.
 
-	  Change-Id: Ic3d6c20ff3502507c17244a8b7e2ca761dc7fb60
-	  (cherry picked from commit efae187217750e322cd6167705a33f888b631427)
+	  * Fixed debug messages in refer_incoming_invite_request() referencing
+	  incorrect channnels on success.  Code comments now say why the
+	  session->channel cannot be used.
 
-2016-04-14 20:26 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4422/
+	  ........
 
-	* asterisk 13.8.1 Released.
+	  Merged revisions 431956 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-14 15:26 +0000 [18e6f12e83]  Kevin Harwell <kharwell@digium.com>
 
-	* Release summaries: Remove previous versions
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431957 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-04-14 15:26 +0000 [625c07711a]  Kevin Harwell <kharwell@digium.com>
+2015-02-19 09:28 +0000 [e3fd826cdb]  Alexander Traud (License 6520)
 
-	* .version: Update for 13.8.1
+	* tcptls: Handle new OpenSSL compile time option to disable SSLv3
 
-2016-04-14 15:26 +0000 [584f1fb3c7]  Kevin Harwell <kharwell@digium.com>
+	  Some distributions are going to disable SSLv3 at compile time. This option can
+	  be checked using the directive OPENSSL_NO_SSL3_METHOD. This patch updates the
+	  TCP/TLS handling in Asterisk to look for that directive before attempting to
+	  use the SSLv3 specific methods.
 
-	* .lastclean: Update for 13.8.1
+	  ASTERISK-24799 #close
+	  Reported by: Alexander Traud
+	  patches:
+	    no-ssl3-method.patch uploaded by Alexander Traud (License 6520)
+	  ........
 
-2016-04-14 15:26 +0000 [1e37a63379]  Kevin Harwell <kharwell@digium.com>
+	  Merged revisions 431936 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* realtime: Add database scripts for 13.8.1
+	  Merged revisions 431937 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-04-14 13:49 +0000 [dcf1b3c098]  Mark Michelson <mmichelson@digium.com>
 
-	* transport management: Register thread with PJProject.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431938 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The scheduler thread that kills idle TCP connections was not registering
-	  with PJProject properly and causing assertions if PJProject was built in
-	  debug mode.
+2015-02-18 20:03 +0000 [a4774ceaa5]  Corey Farrell <git@cfware.com>
 
-	  This change registers the thread with PJProject the first time that the
-	  scheduler callback executes.
+	* Create work around for scheduler leaks during shutdown.
 
-	  AST-2016-005
+	  * Added ast_sched_clean_by_callback for cleanup of scheduled events
+	    that have not yet fired.
+	  * Run all pending peercnt_remove_cb and replace_callno events in chan_iax2.
+	    Cleanup of replace_callno events is only run 11, since it no longer
+	    releases any references or allocations in 13+.
 
-	  Change-Id: I5f7a37e2c80726a99afe9dc2a4a69bdedf661283
+	  ASTERISK-24451 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4425/
+	  ........
 
-2016-03-08 12:12 +0000 [efafbb1319]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 431916 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_pjsip_transport_management: Kill idle TCP connections.
+	  Merged revisions 431917 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  "Idle" here means that someone connects to us and does not send a SIP
-	  request. PJProject will not automatically time out such connections, so
-	  it's up to Asterisk to do it instead.
 
-	  When we receive an incoming TCP connection, we will start a timer
-	  (equivalent to transaction timer D) waiting to receive an incoming
-	  request. If we do not receive a request in that timeframe, then we will
-	  shut down the TCP connection.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431918 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25796 #close
-	  Reported by George Joseph
+2015-02-17 09:34 +0000 [09bfe4b208]  Richard Mudgett <rmudgett@digium.com>
 
-	  AST-2016-005
+	* res_pjsip_refer: Fix crash from a REFER and BYE collision.
 
-	  Change-Id: I7b0d303e5d140d0ccaf2f7af562071e3d1130ac6
+	  Analyzing a one-off crash on a busy system showed that processing a REFER
+	  request had a NULL session channel pointer.  The only way I can think of
+	  that could cause this is if an outgoing BYE transaction overlapped the
+	  incoming REFER transaction in a collision.  Asterisk sends a BYE while the
+	  phone sends a REFER to complete an attended transfer.
 
-2016-03-08 10:52 +0000 [159f639770]  Mark Michelson <mmichelson@digium.com>
+	  * Made check the session channel pointer before processing an incoming
+	  REFER request in res_pjsip_refer.
 
-	* Rename res_pjsip_keepalive res_pjsip_transport_management
+	  * Fixed similar crash potential for res_pjsip supplement incoming request
+	  processing for res_pjsip_sdp_rtp INFO, res_pjsip_caller_id INVITE/UPDATE,
+	  res_pjsip_messaging MESSAGE, and res_pjsip_send_to_voicemail REFER
+	  messages.
 
-	  ASTERISK-25796
-	  Reported by George Joseph
+	  * Made res_pjsip_messaging respond to a message body too large with a 413
+	  instead of ignoring it.
 
-	  AST-2016-005
+	  ASTERISK-24700 #close
+	  Reported by: Zane Conkle
 
-	  Change-Id: Id322a05f927392293570599730050bc677d99433
+	  Review: https://reviewboard.asterisk.org/r/4417/
+	  ........
 
-2016-04-14 07:15 +0000 [c164ff004d]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 431898 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* AST-2016-004: Fix crash on REGISTER with long URI.
 
-	  Due to some ignored return values, Asterisk could crash if processing an
-	  incoming REGISTER whose contact URI was above a certain length.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431899 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25707 #close
-	  Reported by George Joseph
+2015-02-16 15:29 +0000 [d808eace5c]  Matt Jordan <mjordan@digium.com>
 
-	  Patches:
-	  	0001-res_pjsip-Validate-that-URIs-don-t-exceed-pjproject-.patch
+	* res/res_rtp_asterisk: Fix crash in debug from RTCP reports without report block
 
-	  AST-2016-004
+	  When RTCP debugging was enabled, an RTCP report without a report block would
+	  cause a crash. This was due to the verbose output not checking to see if the
+	  report_block pointer was NULl before dereferencing it.
 
-	  Change-Id: I0ed3898fe7ab10121b76c8c79046692de3a1be55
+	  This patch adds the necessary check to prevent printing any verbose output
+	  if the far side hasn't provided us the information they should have.
 
-2016-03-29 19:39 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  ASTERISK-24791 #close
+	  Reported by: JoshE
+	  Tested by: JoshE
+	  ........
 
-	* asterisk 13.8.0 Released.
+	  Merged revisions 431879 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-29 14:39 +0000 [0f885f0076]  Mark Michelson <mmichelson@digium.com>
 
-	* Release summaries: Add summaries for 13.8.0
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431880 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-29 14:34 +0000 [a1fa37aebd]  Mark Michelson <mmichelson@digium.com>
+2015-02-15 13:01 +0000 [55eb8fc068]  Joshua Colp <jcolp@digium.com>
 
-	* Release summaries: Remove previous versions
+	* pjsip: Remove "contact" type from pjsip.conf.sample
 
-2016-03-29 14:34 +0000 [e7de5fd439]  Mark Michelson <mmichelson@digium.com>
+	  The "contact" object is not meant to be configured from the pjsip.conf
+	  configuration file. It is meant to be created as a result of a registration
+	  and stored elsewhere.
 
-	* .version: Update for 13.8.0
+	  ASTERISK-24085 #close
+	  Reported by: Rusty Newton
+	  ........
 
-2016-03-29 14:34 +0000 [8baf813848]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 431860 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* .lastclean: Update for 13.8.0
 
-2016-03-29 14:34 +0000 [42469df205]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431861 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* realtime: Add database scripts for 13.8.0
+2015-02-15 12:00 +0000 [55709bc1f7]  Joshua Colp <jcolp@digium.com>
 
-2016-03-22 18:31 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* install_prereq: Tweak flags when configuring pjproject.
 
-	* asterisk 13.8.0-rc1 Released.
+	  This change does two things:
+	  1. Disables debugging so assertions which can return an error do,
+	  instead of asserting.
+	  2. Enables IPv6 support.
 
-2016-03-22 13:26 +0000 [a698424678]  Mark Michelson <mmichelson@lunkwill>
+	  ASTERISK-24632 #close
+	  Reported by: Rusty Newton
+	  ........
 
-	* Release summaries: Add summaries for 13.8.0-rc1
+	  Merged revisions 431843 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-22 13:21 +0000 [e395a0b973]  Mark Michelson <mmichelson@lunkwill>
 
-	* .version: Update for 13.8.0-rc1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431844 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-22 13:21 +0000 [38a86b2dbf]  Mark Michelson <mmichelson@lunkwill>
+2015-02-15 11:43 +0000 [e78dd39885]  Joshua Colp <jcolp@digium.com>
 
-	* .lastclean: Update for 13.8.0-rc1
+	* res_sorcery_config: Improve object lookup times.
 
-2016-03-22 13:21 +0000 [e0c8c8bf4a]  Mark Michelson <mmichelson@lunkwill>
+	  The res_sorcery_config module currently uses a fixed bucket
+	  size of 53. This means that depending on the number of objects
+	  you either end up with excess buckets or a lot of collisions.
+	  Due to the way that res_sorcery_config is implemented it's actually
+	  possible to make the bucket size dynamic based on the number of
+	  objects. This is due to the fact that each loading of the config file
+	  produces a new container and does not modify the existing one.
+	  This change uses the number of expected objects and finds a prime
+	  number near it. In practice depending on the number of objects this
+	  can speed up lookups anywhere from 2X to 15X. This change also removes
+	  the lock from the container as it is not needed.
 
-	* realtime: Add database scripts for 13.8.0-rc1
+	  Review: https://reviewboard.asterisk.org/r/4423/
+	  ........
 
-2016-03-18 14:31 +0000 [6a40520fe9]  Kevin Harwell <kharwell@digium.com>
+	  Merged revisions 431841 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_pjsip: ref leak when checking direct_media_glare
 
-	  Fix the reference leak introduced in the following commit:
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431842 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  9444ddadf8525d1ce66a1faf1db97f9f6c265ca4
+2015-02-15 10:01 +0000 [e6fe69b76c]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25849
+	* res_pjsip: Add "pjsip show version" CLI command.
 
-	  Change-Id: I5cfefd5ee6c1c3a1715c050330aaa10e4d2a5e85
-2016-03-16 12:37 +0000 [9444ddadf8]  Kevin Harwell <kharwell@digium.com>
+	  When debugging things it can be useful to know absolutely what
+	  version of pjproject res_pjsip is running against. This change
+	  adds a "pjsip show version" CLI command which can be used to
+	  query for this.
 
-	* chan_pjsip: transfers with direct media reinvite has wrong address/port
+	  ASTERISK-24685 #close
+	  Reported by: Joshua Colp
 
-	  During a transfer involving direct media a race occurs between when the
-	  transferer channel is swapped out, initiating rtp changes/updates, and the
-	  subsequent reinvites.
+	  Review: https://reviewboard.asterisk.org/r/4424/
+	  ........
 
-	  When Alice, after speaking with Charlie (Bob is on hold), connects Bob and
-	  Charlie invites are sent to each in order to establish the call between them.
-	  Bob is taken off hold and Charlie is told to have his media flow through
-	  Asterisk. However, if before those invites go out the bridge updates Bob's
-	  and/or Charlie's rtp information with direct media data (i.e. address, port)
-	  then the invite(s) will contain the remote data in the SDP instead of the
-	  Asterisk data.
+	  Merged revisions 431824 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The race occurs in the native bridge glue code when updating the peer. The
-	  direct_media_address can get set twice before sending out the first invite
-	  during call connection. This can happen because the checking/setting of the
-	  direct_media_address happened in one thread while the sending of the invite(s)
-	  happened in another thread.
 
-	  This fix removes the race condition by moving the checking/setting of the
-	  direct_media_address to be in the same thread as the sending of the invites(s).
-	  This serializes the checking/setting and sending so they can no longer happen
-	  out of order.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431825 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25849 #close
+2015-02-15 06:41 +0000 [17f9e0cacc]  Matthias Urlichs (license 5508)
 
-	  Change-Id: Idfea590175e74f401929a601dba0c91ca1a7f873
+	* res_timing_pthread: Fix leaky pipes.
 
-2015-10-19 07:11 +0000 [88240f98d9]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  During some refactoring the way private information for timers
+	  was stored was changed. As a result of this the action which normally
+	  removed the timer upon closure in res_timing_pthread was also removed
+	  causing the timer to remain after it should using up resources.
+	  This change ensures that the timer is removed upon closure.
 
-	* install_prereq: Update repositories before install on Debian systems
+	  ASTERISK-24768 #close
+	  Reported by: Matthias Urlichs
+	  patches:
+	   timer.patch submitted by Matthias Urlichs (license 5508)
+	  ........
 
-	  When to install packages the indexed local is more old of the
-	  version of software on the repository they have been upgraded by security
-	  update then get the package will give 404 not found.
+	  Merged revisions 431807 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The patch prevent by update local index to repository for aptitude before
-	  install.
 
-	  ASTERISK-25495 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431808 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Reporte by: Rodrigo Ramírez Norambuena
+2015-02-14 18:33 +0000 [d1bd8b091b]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I645959e553aac542805ced394cac2dca964051fa
-	  (cherry picked from commit 88f3dbaec9509bfba8bc1de7799aa0dc65304bb5)
+	* apps/app_mixmonitor: Move Test Event for MIXMONITOR_END to after it finishes
 
-2015-06-03 20:12 +0000 [efcf9a96db]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  The Test Event for MIXMONITOR_END - which signals that a MixMonitor has
+	  completed - technically fired before the filestream was closed. If a test
+	  used this to trigger a condition to verify that the file was written, it
+	  could result in a race condition where the file size would not be what the
+	  test expected.
 
-	* install_prereq: Check if is installed aptitude otherwise to install.
+	  Luckily, no tests were using this (although they should have been). Since the
+	  test event needed to be moved after the point where the MixMonitor autochan has
+	  been destroyed, the test event no longer emits the channel name. Luckily,
+	  nothing needs it.
+	  ........
 
-	  If in Debian or system based, dont have aptitude installed the script do
-	  nothing. This patch checked if aptitude  installed, if not installed.
+	  Merged revisions 431788 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Also, if execute script with all packages installed yet, the script not show
-	  nothing and return exit 1 because the command 'grep' get nothing from pipe from
-	  'awk'.
+	  Merged revisions 431789 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25113 #close
-	  Reported By: Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
 
-	  Change-Id: Iebdff55805d3917166e5e08e0a1e2176f36ff27f
-	  (cherry picked from commit 6737ded0581a9e1256bdfe30c1d747e7ca93f8b3)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431790 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-03 04:43 +0000 [2b1b8e382a]  Sergio Medina Toledo <lumasepa@gmail.com>
+2015-02-14 13:46 +0000 [455a98a2f8]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_refer.c: Fix seg fault in process of Refer-to header.
+	* sorcery: Output an error message if a wizard is specified for an object type and it isn't found.
 
-	  The "Refer-to" header of an incoming REFER request is parsed by
-	  pjsip_parse_uri().  That function requires the URI parameter to be NULL
-	  terminated.  Unfortunately, the previous code added the NULL terminator by
-	  overwriting memory that may not be safe.  The overwritten memory results
-	  could be benign, memory corruption, or a segmentation fault.  Now the URI
-	  is NULL terminated safely by copying the URI to a new chunk of memory with
-	  the correct size to be NULL terminated.
+	  ASTERISK-24612 #close
+	  Reported by: Joshua Colp
+	  ........
 
-	  ASTERISK-25814 #close
+	  Merged revisions 431771 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I32565496684a5a49c3278fce06474b8c94b37342
 
-2016-03-11 12:22 +0000 [de04308ae4]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431772 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip.c: Fix mwi resub deadlock potential.
+2015-02-14 12:31 +0000 [fae6bf8ace]  Joshua Colp <jcolp@digium.com>
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	* res_pjsip_exten_state: Improve log message when a subscription is attempted to a non-existent extension.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  ASTERISK-24716 #close
+	  Reported by: Rusty Newton
+	  ........
 
-	  ASTERISK-25023 #close
+	  Merged revisions 431754 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I96d429c57a48861fd8bde63dd93db4e92dc3adb6
 
-2016-03-10 17:01 +0000 [5f6627a8a4]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431755 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip.c: Fix registration timeout and expire deadlock potential.
+2015-02-14 12:21 +0000 [cc96e4a7ef]  Joshua Colp <jcolp@digium.com>
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	* Multiple revisions 431751-431752
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  ........
+	    r431751 | file | 2015-02-14 14:19:07 -0400 (Sat, 14 Feb 2015) | 5 lines
+	    
+	    chan_pjsip: Fix crash when CHANNEL dialplan function is invoked with pjsip argument and no type.
+	    
+	    ASTERISK-24771 #close
+	    Reported by: Niklas Larsson
+	  ........
+	    r431752 | file | 2015-02-14 14:20:27 -0400 (Sat, 14 Feb 2015) | 2 lines
+	    
+	    'information' ends with an 'n'.
+	  ........
 
-	  ASTERISK-25023
+	  Merged revisions 431751-431752 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2e40de89efc8ae6e8850771d089ca44bc604b508
 
-2016-03-10 12:17 +0000 [32bd7a64f9]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431753 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip.c: Fix t38id deadlock potential.
+2015-02-13 11:24 +0000 [f00ebf0a2d]  Richard Mudgett <rmudgett@digium.com>
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	* res_pjsip_session: Fix double re-INVITE collision crash.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  A multi-asterisk box setup with direct media enabled would occasionally
+	  crash when two re-INVITE collisions on a call leg happen in a row.
 
-	  ASTERISK-25023
+	  The re-INVITE logic only had one timer struct to defer the re-INVITE.
+	  When the second collision happens the timer struct is overwritten and put
+	  into the timer heap again.  Resources for the first timer are leaked and
+	  the heap has two positions occupied by the same timer struct.  Now the
+	  heap ordering is potentially corrupted, the timer will fire twice, and any
+	  resources allocated for the second timer will be released twice.
 
-	  Change-Id: If595e4456cd059d7171880c7f354e844c21b5f5f
+	  * The solution is to put the collided re-INVITE into the delayed requests
+	  queue with all the other delayed requests and cherry pick the next request
+	  that can come off the queue when an event happens.
 
-2016-03-09 16:34 +0000 [43556b800b]  Richard Mudgett <rmudgett@digium.com>
+	  * Changed to put delayed BYE requests at the head of the delayed queue.
+	  There is no sense in processing delayed UPDATEs and re-INVITEs when a BYE
+	  has been requested.
 
-	* chan_sip.c: Fix reinviteid deadlock potential.
+	  * Made the start of a BYE request flush the delayed requests queue to
+	  prevent a delayed request from overlapping the BYE transaction.  I saw a
+	  few cases where a delayed re-INVITE got started after the BYE transaction
+	  started.
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  * Changed the delayed_request struct to use an enum instead of a string
+	  for the request method.  Cherry picking the queue is easier with an enum
+	  than string comparisons and the compiler can warn if a switch statement
+	  does not cover all defined enum values.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  * Improved the debug output to give more information.  It helps to know
+	  which channel is involved with an endpoint.  Trunks can have many channels
+	  associated with the endpoint at the same time.
 
-	  ASTERISK-25023
+	  ASTERISK-24727 #close
+	  Reported by: Mark Michelson
 
-	  Change-Id: I9c11b9d597468f63916c99e1dabff9f4a46f84c1
+	  Review: https://reviewboard.asterisk.org/r/4414/
+	  ........
 
-2016-03-09 16:32 +0000 [38c1cdab2c]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 431734 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip.c: Fix packet retransid deadlock potential.
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431735 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+2015-02-12 14:34 +0000 [29f66b0429]  Matt Jordan <mjordan@digium.com>
 
-	  * Fix retrans_pkt() to call check_pendings() with both the owner channel
-	  and the private objects locked as required.
+	* ARI/PJSIP: Add the ability to redirect (transfer) a channel in a Stasis app
 
-	  * Refactor dialog retransmission packet list to safely remove packet
-	  nodes.  The list nodes are now ao2 objects.  The list has a ref and the
-	  scheduled entry has a ref.
+	  This patch adds a new feature to ARI to redirect a channel to another server,
+	  and fixes a few bugs in PJSIP's handling of the Transfer dialplan
+	  application/ARI redirect capability.
+
+	  *New Feature*
+	  A new operation has been added to the ARI channels resource, redirect. With
+	  this, a channel in a Stasis application can be redirected to another endpoint
+	  of the same underlying channel technology.
+
+	  *Bug fixes*
+	  In the process of writing this new feature, two bugs were fixed in the PJSIP
+	  stack:
+	  (1) The existing .transfer channel callback had the limitation that it could
+	      only transfer channels to a SIP URI, i.e., you had to pass
+	      'PJSIP/sip:foo@my_provider.com' to the dialplan application. While this is
+	      still supported, it is somewhat unintuitive - particularly in a world full
+	      of endpoints. As such, we now also support specifying the PJSIP endpoint to
+	      transfer to.
+	  (2) res_pjsip_multihomed was, unfortunately, trying to 'help' a 302 redirect by
+	      updating its Contact header. Alas, that resulted in the forwarding
+	      destination set by the dialplan application/ARI resource/whatever being
+	      rewritten with very incorrect information. Hence, we now don't bother
+	      updating an outgoing response if it is a 302. Since this took a looong time
+	      to find, some additional debug statements have been added to those modules
+	      that update the Contact headers.
 
-	  ASTERISK-25023
+	  Review: https://reviewboard.asterisk.org/r/4316/
 
-	  Change-Id: I50926d81be53f4cd3d572a3292cd25f563f59641
+	  ASTERISK-24015 #close
+	  Reported by: Private Name
 
-2016-03-09 16:26 +0000 [e4ad55c888]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24703 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	* chan_sip.c: Fix waitid deadlock potential.
+	  Merged revisions 431717 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431718 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Made always run check_pendings() under the scheduler thread so scheduler
-	  ids can be checked safely.
+2015-02-11 12:03 +0000 [9d081ed06c]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-25023
+	* res_pjsip: dtls_handler causes Asterisk to crash
 
-	  Change-Id: Ia834d6edd5bdb47c163e4ecf884428a4a8b17d52
+	  There have been a couple of times where a crash occurred in the dtls_handler
+	  section of the code for res_pjsip. Unfortunately, in working this issue the
+	  problem was unable to be reproduced. After looking at the backtraces and
+	  through the code the current best guess as to why this happened might be due
+	  to a reentrance problem and the strtok function. So, the current fix is to
+	  convert the strtok function into the reentrant version of the function,
+	  strtok_r.
 
-2016-03-08 15:08 +0000 [98d5669c28]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24741 #close
+	  Reported by: Zane Conkle
+	  Review: https://reviewboard.asterisk.org/r/4409/
+	  ........
 
-	* chan_sip.c: Fix session timers deadlock potential.
+	  Merged revisions 431698 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431699 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25023
+2015-02-11 11:45 +0000 [cc85e55d88]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: I6d65269151ba95e0d8fe4e9e611881cde2ab4900
+	* ari_websockets: removed extra check on websocket session read
 
-2016-03-07 13:21 +0000 [9cb8f73226]  Richard Mudgett <rmudgett@digium.com>
+	  When merging the websocket timeout issue (ASTERISK-24701) an extra, almost
+	  duplicate, check was left in the code that should not have been. This removes
+	  it.
 
-	* chan_sip.c: Fix autokillid deadlock potential.
+	  ASTERISK-24701 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4412/
+	  ........
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  Merged revisions 431693 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
 
-	  * Fix clearing autokillid in __sip_autodestruct() even though we could
-	  reschedule.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431695 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25023
+2015-02-11 11:39 +0000 [e2d3215b83]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I450580dbf26e2e3952ee6628c735b001565c368f
+	* HTTP: Stop accepting requests on final system shutdown.
 
-2016-03-07 18:28 +0000 [c5c7f48a15]  Richard Mudgett <rmudgett@digium.com>
+	  There are three CLI commands to stop and restart Asterisk each.
 
-	* chan_sip.c: Fix provisional_keepalive_sched_id deadlock.
+	  1) core stop/restart now - Hangup all calls and stop or restart Asterisk.
+	  New channels are prevented while the shutdown request is pending.
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  2) core stop/restart gracefully - Stop or restart Asterisk when there are
+	  no calls remaining in the system.  New channels are prevented while the
+	  shutdown request is pending.
 
-	  Stopping a scheduled event can result in a deadlock if the scheduled event
-	  is running when you try to stop the event.  If you hold a lock needed by
-	  the scheduled event while trying to stop the scheduled event then a
-	  deadlock can happen.  The general strategy for resolving the deadlock
-	  potential is to push the actual starting and stopping of the scheduled
-	  events off onto the scheduler/do_monitor() thread by scheduling an
-	  immediate one shot scheduled event.  Some restructuring may be needed
-	  because the code may assume that the start/stop of the scheduled events is
-	  immediate.
+	  3) core stop/restart when convenient - Stop or restart Asterisk when there
+	  are no calls in the system.  New calls are not prevented while the
+	  shutdown request is pending.
 
-	  ASTERISK-25023
+	  ARI has made stopping/restarting Asterisk more problematic.  While a
+	  shutdown request is pending it is desirable to continue to process ARI
+	  HTTP requests for current calls.  To handle the current calls while a
+	  shutdown request is pending, a new committed to shutdown phase is needed
+	  so ARI applications can deal with the calls until the system is fully
+	  committed to shutdown.
 
-	  Change-Id: I98a694fd42bc81436c83aa92de03226e6e4e3f48
+	  * Added a new shutdown committed phase so ARI applications can deal with
+	  calls until the final committed to shutdown phase is reached.
 
-2016-03-09 11:22 +0000 [f959d84dfd]  Richard Mudgett <rmudgett@digium.com>
+	  * Made refuse new HTTP requests when the system has reached the final
+	  system shutdown phase.  Starting anything while the system is actively
+	  releasing resources and unloading modules is not a good thing.
 
-	* chan_sip.c: Adjust how dialog_unlink_all() stops scheduled events.
+	  * Split the bridging framework shutdown to not cleanup the global bridging
+	  containers when shutting down in a hurry.  This is similar to how other
+	  modules prevent crashes on rapid system shutdown.
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  * Moved ast_begin_shutdown(), ast_cancel_shutdown(), and
+	  ast_shutting_down().  You should not have to include channel.h just to
+	  access these system functions.
 
-	  * Make dialog_unlink_all() unschedule all items at once in the sched
-	  thread.
+	  ASTERISK-24752 #close
+	  Reported by: Matthew Jordan
 
-	  ASTERISK-25023
+	  Review: https://reviewboard.asterisk.org/r/4399/
+	  ........
 
-	  Change-Id: I7743072fb228836e8228b72f6dc46c8cc50b3fb4
+	  Merged revisions 431692 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-10 21:54 +0000 [5f3225ddcc]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_sip.c: Clear scheduled immediate events on unload.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431694 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+2015-02-11 11:13 +0000 [5a17ed7a38]  Richard Miller (License 5685)
 
-	  The reordering of chan_sip's shutdown is to handle any immediate events
-	  that get put onto the scheduler so resources aren't leaked.  The typical
-	  immediate events at this time are going to be concerned with stopping
-	  other scheduled events.
+	* channels/chan_sip: Fix RealTime error during SIP unregistration with MariaDB
 
-	  ASTERISK-25023
+	  When a SIP device that has its registration stored in RealTime unregisters,
+	  the entry for that device is updated with blank values, i.e., "", indicating
+	  that it is no longer registered. Unfortunately, one of those values that is
+	  'blanked' is the device's port. If the column type for the port is not a
+	  string datatype (the recommended type is integer), an ODBC or database error
+	  will be thrown. MariaDB does not coerce empty strings to a valid integer value.
 
-	  Change-Id: I3f6540717634f6f2e84d8531a054976f2bbb9d20
+	  This patch updates the query run from chan_sip such that it replaces the port
+	  value with a value of '0', as opposed to a blank value. This is the value that
+	  other database backends coerce the empty string ("") to already, and the
+	  handling of reading a RealTime registration value from a backend already
+	  anticipates receiving a port of '0' from the backends.
 
-2016-03-15 14:51 +0000 [7a74971771]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24772 #close
+	  Reported by: Richard Miller
+	  patches:
+	    chan_sip.diff uploaded by Richard Miller (License 5685)
+	  ........
 
-	* sip/dialplan_functions.c: Fix /channels/chan_sip/test_sip_rtpqos crash.
+	  Merged revisions 431673 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  Merged revisions 431674 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Delaying destruction of the chan_sip sip_pvt structures caused the
-	  /channels/chan_sip/test_sip_rtpqos unit test to crash.  That test
-	  registers a special test ast_rtp_engine with the rtp engine module.  When
-	  the unit test completes it cleans up by unregistering the test
-	  ast_rtp_engine and exits.  Since the delayed destruction of the sip_pvt
-	  happens after the unit test returns, the destructor tries to call the rtp
-	  engine destroy callback of the test ast_rtp_engine auto variable which no
-	  longer exists on the stack.
 
-	  * Change the test ast_rtp_engine auto variable to a static variable.  Now
-	  the variable can still exist after the unit test exits so the delayed
-	  sip_pvt destruction can complete successfully.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431675 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25023
+2015-02-11 11:03 +0000 [8cc50b1ebc]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I61e34a12d425189ef7e96fc69ae14993f82f3f13
+	* Enable REF_DEBUG for ast_module_ref / ast_module_unref.
 
-2016-03-15 13:31 +0000 [d2c09ed73b]  Andrew Nagy <andrew.nagy@the159.com>
+	  Add ast_module_shutdown_ref for use by modules that can
+	  only be unloaded during graceful shutdown.
 
-	* app_stasis: Don't hang up if app is not registered
+	  When REF_DEBUG is enabled:
+	  * Add an empty ao2 object to struct ast_module.
+	  * Allocate ao2 object when the module is loaded.
+	  * Perform an ao2_ref in each place where mod->usecount is manipulated.
+	  * ao2_cleanup on module unload.
 
-	  This prevents pbx_core from hanging up the channel if the app isn't
-	  registered.
+	  ASTERISK-24479 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4141/
+	  ........
 
-	  ASTERISK-25846 #close
+	  Merged revisions 431662 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I63216a61f30706d5362bc0906b50b6f0544aebce
-2016-03-07 15:50 +0000 [b2d2906445]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 431663 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* sched.c: Ensure oldest expiring entry runs first.
 
-	  This patch is part of a series to resolve deadlocks in chan_sip.c.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431672 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Updated sched unit test to check new behavior.
+2015-02-11 10:52 +0000 [137c4b0778]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-25023
+	* res_http_websocket: websocket write timeout fails to fully disconnect
 
-	  Change-Id: Ib69437327b3cda5e14c4238d9ff91b2531b34ef3
+	  When writing to a websocket if a timeout occurred the underlying socket did not
+	  get closed/disconnected. This patch makes sure the websocket gets disconnected
+	  on a write timeout. Also a notice is logged stating that the websocket was
+	  disconnected.
 
-2016-03-04 18:25 +0000 [9ae21b510f]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24701 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4412/
+	  ........
 
-	* chan_sip.c: Made sip_reinvite_retry() call sip_pvt_lock_full().
+	  Merged revisions 431669 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I90f04208a089f95488a2460185a8dbc3f6acca12
+	  Merged revisions 431670 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-03-07 18:56 +0000 [56bcb97a3c]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_sip.c: Simplify sip_pvt destructor call levels.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431671 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Remove destructor calling destroy_it calling really_destroy_it
-	  for no benefit.  Just make the destructor the really_destroy_it
-	  function.
+2015-02-10 17:17 +0000 [49161d8df8]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Idea0d47b27dd74f2488db75bcc7f353d8fdc614a
+	* res_pjsip_config_wizard: Add ability to auto-create hints.
 
-2016-03-14 08:59 +0000 [677a65fcbb]  Joshua Colp <jcolp@digium.com>
+	  Looking at the Super Awesome Company sample reminded me that creating hints is 
+	  just plain gruntwork.  So you can now have the pjsip conifg wizard auto-create 
+	  them for you.
 
-	* build: Add configure check for proto field of PJSIP TLS transport setting.
+	  Specifying 'hint_exten' in the wizard will create 
+	  'exten => <hint_exten>,hint/PJSIP/<wizard_id>'
+	  in whatever is specified for 'hint_context'.
 
-	  Older versions of PJSIP do not have the proto field on the TLS transport
-	  setting structure. This change adds a configure check so even if it is
-	  not present we will still be able to build.
+	  Specifying 'hint_application' in the wizard will create
+	  'exten => <hint_exten>,1,<hint_application>'
+	  in whatever is specified for 'hint_context'.
 
-	  Change-Id: Ibf3f47befb91ed1b8194bf63888baa6fee05aba9
+	  The default for 'hint_context' is the endpoint's context.
+	  There's no default for 'hint_application'.  If not specified, no app is added.
+	  There's no default for 'hint_exten'.  If not specified, neither the hint itself 
+	  nor the application will be created.
 
-2016-03-12 16:02 +0000 [32f0a3d52a]  gtjoseph <george.joseph@fairview5.com>
+	  Some may think this is the slippery slope to users.conf but hints are a basic 
+	  necessity for phones unlike voicemail, manager, etc that users.conf creates.
 
-	* build_system:  Split COMPILE_DOUBLE from DONT_OPTIMIZE
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/4383/
+	  ........
 
-	  I can't ever recall actually needing the intermediate files or the checking
-	  that a double compile produces.  What I CAN remember is every DONT_OPTIMIZE
-	  build needing 3 invocations of gcc instead of 1 just to do the checks and
-	  produce those intermediate files.
+	  Merged revisions 431643 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Having said that, Richard pointed out that the reason for the double compile
-	  was that there were cases in the past where a submitted patch failed to compile
-	  because the submitter never tried it with the optimizations turned on.
 
-	  To get the best of both worlds, COMPILE_DOUBLE has been split into its own
-	  option.  If DONT_OPTIMIZE is turned on, COMPILE_DOUBLE will also be selected
-	  BUT you can then turn it off if all you need are the debugging symbols.  This
-	  way you have to make an informed decision about disabling COMPILE_DOUBLE.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431644 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  To allow COMPILE_DOUBLE to be both auto-selected and turned off, a new feature
-	  was added to menuselect.  The <use> element can now contain an "autoselect"
-	  attribute which will turn the used member on but not create a hard dependency.
-	  The cflags.xml implementation for COMPILE_DOUBLE looks like this...
+2015-02-08 21:12 +0000 [858e825568]  Ben Merrills (License 6678)
 
-	  <member name="DONT_OPTIMIZE" displayname="Disable Optimizations ...">
-	  	<use autoselect="yes">COMPILE_DOUBLE</use>
-	  	<support_level>core</support_level>
-	  </member>
-	  <member name="COMPILE_DOUBLE" displayname="Pre-compile with ...>
-	  	<depend>DONT_OPTIMIZE</depend>
-	  	<support_level>core</support_level>
-	  </member>
+	* res/ari/resource_channels: Add missing 'no_answer' reason to DELETE /channels
 
-	  When DONT_OPTIMIZE is turned on, COMPILE_DOUBLE is turned on because
-	  of the use.
-	  When DONT_OPTIMIZE is turned off, COMPILE_DOUBLE is turned off because
-	  of the depend.
-	  When COMPILE_DOUBLE is turned on, DONT_OPTIMIZE is turned on because
-	  of the depend.
-	  When COMPILE_DOUBLE is turned off, DONT_OPTIMIZE is left as is because
-	  it only uses COMPILE_DOUBLE, it doesn't depend on it.
+	  One of the canonical reasons for hanging up a channel is because the far end
+	  failed to answer - or because someone else answered, and we want to get rid of
+	  this channel. This patch adds the missing value to the 'reason' query parameter
+	  for the DELETE /channels operation.
 
-	  I also made a few tweaks to the ncurses implementation to move things
-	  left a bit to allow longer descriptions.
+	  Review: https://reviewboard.asterisk.org/r/4400
 
-	  Change-Id: Id49ca930ac4b5ec4fc2d8141979ad888da7b1611
+	  ASTERISK-24745 #close
+	  Reported by: Ben Merrills
+	  patches:
+	    add_no_answer_ari_hangup_cause.diff uploaded by Ben Merrills (License 6678)
+	  ........
 
-2016-03-10 13:09 +0000 [38499e7125]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 431622 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* pjproject:  Pass (dont_)optimize flags to pjproject and fix pjsua
 
-	  The pjproject Makefile now uses the Asterisk optimization flags which
-	  are determined by the setting of the DONT_OPTMIZE menuselect flag.
-	  The Makefile was also restructured so a change to the top level
-	  menuselect.makeopts will result in a rebuild of pjproject.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431623 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Also, "--disable-resample" was removed from the pjproject configure
-	  options.  Without resample, pjsua (which is used by the testsuite)
-	  can't make audio calls.  When it can't, it segfaults.
+2015-02-08 20:35 +0000 [17247daae6]  ibercom <ibercom123@gmail.com> (License 6599)
 
-	  Change-Id: I24b0a4d0872acef00ed89b3c527a713ee4c2ccd4
+	* res/res_odbc: Remove unneeded queries when determining if a table exists
 
-2016-03-11 16:03 +0000 [336cae73cc]  Walter Doekes <walter+asterisk@wjd.nu>
+	  This patch modifies the ast_odbc_find_table function such that it only performs
+	  a lookup of the requested table if the table is not already known. Prior to
+	  this patch, a queries would be executed against the database even if the table
+	  was already known and cached.
 
-	* app_chanspy: Fix occasional deadlock with ChanSpy and Local channels.
+	  Review: https://reviewboard.asterisk.org/r/4405/
 
-	  Channel masquerading had a conflict with autochannel locking.
+	  ASTERISK-24742 #close
+	  Reported by: ibercom
+	  patches:
+	    patch.diff uploaded by ibercom (License 6599)
+	  ........
 
-	  When locking autochannel->channel, the channel is fetched from the
-	  autochannel and then locked. During the fetch, the autochannel -- which
-	  has no locks itself -- can be modified by someone who owns the channel
-	  lock. That means that the value of autochan->channel cannot be trusted
-	  until you hold the lock.
+	  Merged revisions 431617 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  In practice, this caused problems with Local channels getting
-	  masqueraded away while the ChanSpy attempted to get info from that
-	  channel. The old channel which was about to get removed got locked, but
-	  the new (replaced) channel got unlocked (no-op). Because the replaced
-	  channel was now locked (and would never get unlocked), it couldn't get
-	  removed from the channel list in a timely manner, and would now cause
-	  deadlocks when iterating over the channel list.
+	  Merged revisions 431618 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change checks the autochannel after locking the channel for changes
-	  to the autochannel. If the channel had been changed, the lock is
-	  reobtained on the new channel.
 
-	  In theory it seems possible that after this fix, the lock attempt on the
-	  old (wrong) channel can be on an already destroyed lock, maybe causing
-	  a crash. But that hasn't been observed in the wild and is harder induce
-	  than the current deadlock.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431619 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Thanks go to Filip Frank for suggesting a fix similar to this and
-	  especially to IRC user hexanol for pointing out why this deadlock was
-	  possible and testing this fix. And to Richard for catching my rookie
-	  while loop mistake ;)
+2015-02-08 11:24 +0000 [2ebe811d80]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25321 #close
+	* res/res_pjsip_sdp_rtp: Fix leak of local ICE candidates when applying to SDP
 
-	  Change-Id: I293ae0014e531cd0e675c3f02d1d118a98683def
+	  When an SDP is created for an outgoing request/response, the ICE candidates
+	  obtained from the RTP instance are currently leaked. This causes the ao2
+	  container that holds the candidates to never properly be reclaimed when the
+	  RTP instance is destroyed.
 
-2016-03-07 21:34 +0000 [875d5e9872]  gtjoseph <george.joseph@fairview5.com>
+	  This patch properly decrements the ICE candidates' container if it is
+	  successfully obtained.
 
-	* pjproject_bundled: Remove --with-external-pa from configure options.
+	  ASTERISK-24769 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  Not sure why it was there in the first place as we already specify
-	  --disable-sound.
+	  Merged revisions 431600 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ia80a40e8b1e1acc287955ab11ba1fbd0c7d4cff9
 
-2016-03-06 14:38 +0000 [530cff5f5f]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431601 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip:  Strip spaces from items parsed from comma-separated lists
+2015-02-06 15:26 +0000 [7ca1a0da04]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  Configurations like "aors = a, b, c" were either ignoring everything after "a"
-	  or trying to look up " b".  Same for mailboxes,  ciphers, contacts and a few
-	  others.
+	* various: cleanup issues found during leak hunt
 
-	  To fix, all the strsep(&copy, ",") calls have been wrapped in ast_strip.  To
-	  facilitate this, ast_strip, ast_skip_blanks and ast_skip_nonblanks were
-	  updated to handle null pointers.
+	  In this collection of small patches to prevent
+	  Valgrind errors are: fixes for reference leaks
+	  in config hooks, evaluating a parameter beyond
+	  bounds, and accessing a structure after a lock
+	  where it could have been already free'd.
 
-	  In some cases, an ast_strlen_zero() test was added to skip consecutive commas.
+	  Review: https://reviewboard.asterisk.org/r/4407/
+	  ........
 
-	  There was also an attempt to ast_free an ast_strdupa'd string in
-	  ast_sip_for_each_aor which was causing a SEGV.  I removed it.
+	  Merged revisions 431583 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Although this issue was reported for realtime, the issue was in the res_pjsip
-	  modules so all config mechanisms were affected.
 
-	  ASTERISK-25829 #close
-	  Reported-by: Mateusz Kowalski
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431584 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I0b22a2cf22a7c1c50d4ecacbfa540155bec0e7a2
+2015-02-03 19:27 +0000 [a79c920aa1]  Joshua Colp <jcolp@digium.com>
 
-2016-03-04 20:37 +0000 [3c8076a83b]  gtjoseph <george.joseph@fairview5.com>
+	* res_pjsip_keepalive: Don't crash if PJSIP module is not loaded.
+	  ........
 
-	* install_prereq: Add packages for bundled pjproject
+	  Merged revisions 431555 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  RedHat/CentOS needs python-devel
-	  Debian/Ubuntu needs automake, libsrtp-dev and python-dev
 
-	  Ubuntu also needed libncurses5-dev for cmenuselect so while not
-	  needed for pjproject, I adedd it anyway.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431556 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Idf5fa16e2d87c687439621507e122cb9461d7089
+2015-02-03 18:59 +0000 [03ce56d6c5]  Joshua Colp <jcolp@digium.com>
 
-2016-02-24 17:25 +0000 [27f32cd0a6]  gtjoseph <george.joseph@fairview5.com>
+	* sorcery: Don't try to load object types which haven't been defined.
 
-	* res_pjsip_caller_id: Anonymize 'From' when caller id presentation is prohibited
+	  The act of defining wizards for an object type in sorcery.conf will
+	  create a minimal object type. This can cause a problem when a module
+	  has multiple sorcery instances (which all get the wizards from sorcery.conf
+	  applied) but the sorcery instances do not all contain full information
+	  about the object types. Upon loading errors will occur stating that
+	  the objects can not be created. This is confusing and is actually
+	  perfectly fine.
 
-	  Per RFC3325, the 'From' header is now anonymized on outgoing calls when
-	  caller id presentation is prohibited.
+	  This change makes it so that only object types which have been fully
+	  defined will be loaded.
 
-	  TID = trust_id_outbound
-	  PRO = Set(CALLERID(pres)=prohib)
-	  USR = endpoint/from_user
-	  DOM = endpoint/from_domain
-	  PAI = YES(privacy=off), NO(not sent), PRI(privacy=full) (assumes send_pai=yes)
+	  ASTERISK-24748 #close
+	  Reported by: Joshua Colp
+	  ........
 
-	  Conditions          |Result
-	  --------------------|----------------------------------------------------
-	  TID PRO USR DOM     |PAI    FROM
-	  --------------------|----------------------------------------------------
-	  Y   Y   abc def.ghi |PRI    "Anonymous" <sip:abc@def.ghi>
-	  Y   Y   abc         |PRI    "Anonymous" <sip:abc@anonymous.invalid>
-	  Y   Y       def.ghi |PRI    "Anonymous" <sip:anonymous@def.ghi>
-	  Y   Y               |PRI    "Anonymous" <sip:anonymous@anonymous.invalid>
+	  Merged revisions 431538 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Y   N   abc def.ghi |YES    <sip:abc@def.ghi>
-	  Y   N   abc         |YES    <sip:abc@<ip_address>>
-	  Y   N       def.ghi |YES    "Caller Name" <sip:<caller_exten>@def.ghi>
-	  Y   N               |YES    "Caller Name" <sip:<caller_exten>@<ip_address>>
 
-	  N   Y   abc def.ghi |NO     "Anonymous" <sip:abc@def.ghi>
-	  N   Y   abc         |NO     "Anonymous" <sip:abc@anonymous.invalid>
-	  N   Y       def.ghi |NO     "Anonymous" <sip:anonymous@def.ghi>
-	  N   Y               |NO     "Anonymous" <sip:anonymous@anonymous.invalid>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431539 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  N   N   abc def.ghi |YES    <sip:abc@def.ghi>
-	  N   N   abc         |YES    <sip:abc@<ip_address>>
-	  N   N       def.ghi |YES    "Caller Name" <sip:<caller_exten>@def.ghi>
-	  N   N               |YES    "Caller Name" <sip:<caller_exten>@<ip_address>>
+2015-01-31 10:28 +0000 [14a57782a6]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25791 #close
-	  Reported-by: Anthony Messina
+	* res_format_attr_h264: Fix crash when determining joint capability.
 
-	  Change-Id: I2c82a5ca1413c2c00fb62ea95b0ae8e97af54dc9
+	  The res_format_attr_h264 module currently incorrectly attempts to
+	  copy SPS and PPS information from the wrong attribute. This change
+	  fixes that.
 
-2016-03-03 17:34 +0000 [7cf7b0a4f9]  gtjoseph <george.joseph@fairview5.com>
+	  ASTERISK-24616 #close
+	  Reported by: Yura Kocyuba
 
-	* third_party/Makefile.rules:  Replace unsupported != operator with $(shell ...)
+	  Review: https://reviewboard.asterisk.org/r/4392/
+	  ........
 
-	  Apparently the != operator is fairly new so I've replaced it with
-	  the old $(shell ...) syntax.
+	  Merged revisions 431521 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I16b2e1878a4f91e7e9740abd427f9639f933c479
-	  Reported-by: Richard Mudgett
-2016-01-23 15:50 +0000 [53f57001f2]  gtjoseph <george.joseph@fairview5.com>
 
-	* loader: Retry dlopen when loading fails
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431522 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Although we use the RTLD_LAZY flag when calling dlopen
-	  the first time on a module, this only defers resolution
-	  for function calls.  Pointer references to functions are
-	  determined at link time so dlopen expects them to be there.
-	  Since we don't cross-module link, pointers to functions
-	  in other modules won't be available and dlopen will fail.
+2015-01-30 11:49 +0000 [23bb5f6a73]  Richard Mudgett <rmudgett@digium.com>
 
-	  Doing a "hardened" build also causes problems because it
-	  typically sets "-z now" on the ld command line which
-	  overrides RTLD_LAZY at run time.
+	* app_agent_pool: Fix initial module load agent device state reporting.
 
-	  If the failing module isn't a GLOBAL_SYMBOLS module, then
-	  dlopen will be called again after all the GLOBAL_SYMBOLS
-	  modules have been loaded and they'll eventually resolve.
+	  When the app_agent_pool module initially loads there is a race condition
+	  between the thread loading agents.conf and the device state internal
+	  processing thread.  If the device state internal processing thread handles
+	  the agent creation state updates before the thread that loaded agents.conf
+	  registers the device state provider callback then the cached agent state
+	  is "Invalid".  When a consumer module like app_queue asks for the agent state
+	  it gets the cached "Invalid" state instead of the real state from the provider.
 
-	  If the calling module IS a GLOBAL_SYMBOLS module itself
-	  and a third module depends on it, then there's an issue
-	  because the second time through the dlopen loop,
-	  GLOBAL_SYMBOLS modules aren't given any special treatment
-	  and since the order in which dlopen is called isn't
-	  deterministic, the dependent may again be tried before the
-	  module it needs is loaded.
+	  * Moved loading the agents.conf configuration to the last thing setup by
+	  app_agent_pool in load_module().  Now the device state provider callback
+	  is registered before the config is loaded so the agent creation state
+	  updates are guaranteed to get the initial device state.
 
-	  Simple solution:  Save modules that fail load_resource
-	  because of a dlopen error in a list and retry them
-	  immediately after the first pass. Keep retrying until
-	  the failed list is empty or we reach a #defined max
-	  retries. Error messages are suppressed until the final
-	  pass which also gets rid of those confusing error messages
-	  about module failures that are later corrected.
+	  * Removed some now redundant config cleanup on error in load_config().
 
-	  Change-Id: Iddae1d97cd2f00b94e61662447432765755f64bb
+	  * Added lock protection when accessing the device state in
+	  agent_pvt_devstate_get() and eliminated the RAII_VAR() usage.
 
-2016-03-01 16:18 +0000 [40d9e9e238]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24737 #close
+	  Reported by: Steve Pitts
 
-	* bridge.c: Crash during attended transfer when missing a local channel half
+	  Review: https://reviewboard.asterisk.org/r/4390/
+	  ........
 
-	  It's possible for the transferer channel to get hung up early during the
-	  attended transfer process. For instance, a phone may send a "bye" immediately
-	  upon receiving a sip notify that contains a sip frag 100 (I'm looking at you
-	  Jitsi). When this occurs a race begins between the transferer being hung up
-	  and completion of the transfer code.
+	  Merged revisions 431492 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  If the channel hangs up too early during a transfer involving stasis bridging
-	  for instance, then when the created local channel goes to look up its swap
-	  channel (and associated datastore) it can't find it (since it is no longer in
-	  the bridge) thus it fails to enter the stasis application. Consequently, the
-	  created local channel(s) hang up as well. If the timing is just right then the
-	  bridging code attempts to add the message link with missing local channel(s).
-	  Hence the crash.
 
-	  Unfortunately, there is no great way to solve the problem of the unexpected
-	  "bye". While we can't guarantee we won't receive an early hangup, and in this
-	  case still fail to enter the stasis application, we can make it so asterisk
-	  does not crash.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431493 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch does just that by locking the local channel structure, checking
-	  that the local channel's peer has not been lost, and then continuing. This
-	  keeps the local channel's peer from being ripped out from underneath it by
-	  the local/unreal hangup code while attempting to set the stasis message link.
+2015-01-30 11:41 +0000 [5c9f1b3f51]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-25771
+	* res_pjsip_outbound_publish: eventually crashes when no response is ever received
 
-	  Change-Id: Ie6d6061e34c7c95f07116fffac9a09e5d225c880
+	  When Asterisk attempts to send SIP outbound publish information and no response
+	  is ever received (no 200 okay, 412, 423) the system eventually crashes. A
+	  response is never received because the system Asterisk is attempting to send
+	  publish information to is not available. The underlying pjsip framework attempts
+	  to send publish information. After several attempts it calls back into the
+	  Asterisk outbound publish code. At this point if the "client->queue" is empty
+	  Asterisk attempts to schedule a refresh which utilizes "rdata" and since no
+	  response was received the given "rdata" struture is NULL. Attempting to
+	  dereference a NULL object of course results in a crash.
 
-2016-03-01 18:08 +0000 [ff3da61c35]  Kevin Harwell <kharwell@digium.com>
+	  The fix here removes the dependency on rdata for schedule_publish_refresh.
+	  Instead param->expiration is now passed to it as this is set to -1 if no
+	  response is received. Also added a notification when no response is received.
 
-	* res_pjsip_refer.c: Delay sending the initial SIP Notify with frag 100
+	  ASTERISK-24635 #close
+	  Reported by: Marco Paland
+	  Review: https://reviewboard.asterisk.org/r/4384/
+	  ........
 
-	  During the transfer process, some phones (okay it was the Jitsi softphone,
-	  but maybe others are out there) send a "bye" immediately after receiving a
-	  SIP Notify. When a "bye" is received early for some types of transfers the
-	  transferer channel may no longer be available during late stage transfer
-	  processing.
+	  Merged revisions 431490 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  For instance, during an attended transfer involving stasis bridging at one
-	  point the created local channel looks for an associated swap channel in
-	  order to retrieve the stasis application name. If the transferer has hung
-	  up then the local channel will fail to find it. The local channel then has
-	  no way to know which stasis app to enter, so it fails and hangs up as well.
-	  Thus the transfer does not complete as expected.
 
-	  This patch delays the sending of the initial notify in order to give the
-	  transfer process enough time to gather the necessary data for a successful
-	  transfer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431491 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25771
+2015-01-30 11:21 +0000 [6a76740b83]  Ashley Sanders <asanders@digium.com>
 
-	  Change-Id: I09cfc9a5d6ed4c007bc70625e0972b470393bf16
+	* HTTP: For httpd server, need option to define server name for security purposes
 
-2016-03-03 08:26 +0000 [26b8f2692e]  Joshua Colp <jcolp@digium.com>
+	  Added a new config property [servername] to the http.conf file; updated the http server to use the new property when sending responses, for showing http status through the CLI and when reporting status through the 'httpstatus' webpage. In this version, [servername] is uncommented by default.
 
-	* res_pjsip_dtmf_info: NULL terminate the message body.
+	  ASTERISK-24316 #close
+	  Reported By: Andrew Nagy
+	  Review: https://reviewboard.asterisk.org/r/4374/
+	  ........
 
-	  PJSIP does not ensure that when printing the message body the
-	  buffer will be NULL terminated. This is problematic when searching
-	  for the signal and duration values of the DTMF.
+	  Merged revisions 431471 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change ensures the buffer is always NULL terminated.
 
-	  Change-Id: I52653a1a60c93092d06af31a27408d569cc98968
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431484 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-03-01 20:03 +0000 [86d6e44cc1]  gtjoseph <george.joseph@fairview5.com>
+2015-01-30 10:49 +0000 [bd0bdf1e41]  Mark Michelson <mmichelson@digium.com>
 
-	* alembic: Fix downgrade and tweak for sqlite
+	* Fix some memory leaks.
 
-	  Downgrade had a few issues.  First there was an errant 'update' statement in
-	  add_auto_dtmf_mode that looks like it was a copy/paste error.  Second, we
-	  weren't cleaning up the ENUMs so subsequent upgrades on postgres failed
-	  because the types already existed.
+	  These memory leaks were found and fixed by John Hardin. I'm just
+	  committing them for him.
 
-	  For sqlite...  sqlite doesn't support ALTER or DROP COLUMN directly.
-	  Fortunately alembic batch_operations takes care of this for us if we
-	  use it so the alter and drops were converted to use batch operations.
+	  ASTERISK-24736 #close
+	  Reported by Mark Michelson
 
-	  Here's an example downgrade:
+	  Review: https://reviewboard.asterisk.org/r/4389
+	  ........
 
-	      with op.batch_alter_table('ps_endpoints') as batch_op:
-	          batch_op.drop_column('tos_audio')
-	          batch_op.drop_column('tos_video')
-	          batch_op.add_column(sa.Column('tos_audio', yesno_values))
-	          batch_op.add_column(sa.Column('tos_video', yesno_values))
-	          batch_op.drop_column('cos_audio')
-	          batch_op.drop_column('cos_video')
-	          batch_op.add_column(sa.Column('cos_audio', yesno_values))
-	          batch_op.add_column(sa.Column('cos_video', yesno_values))
+	  Merged revisions 431468 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	      with op.batch_alter_table('ps_transports') as batch_op:
-	          batch_op.drop_column('tos')
-	          batch_op.add_column(sa.Column('tos', yesno_values))
-	      # Can't cast integers to YESNO_VALUES, so dropping and adding is required
-	          batch_op.drop_column('cos')
-	          batch_op.add_column(sa.Column('cos', yesno_values))
 
-	  Upgrades from base to head and downgrades from head to base were tested
-	  repeatedly for postgresql, mysql/mariadb, and sqlite3.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431469 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I862b0739eb3fd45ec3412dcc13c2340e1b7baef8
+2015-01-29 17:03 +0000 [388d691f34]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2016-03-02 15:55 +0000 [6f0d7ce9db]  gtjoseph <george.joseph@fairview5.com>
+	* stasis transfer: fix stasis bridge push race part two
 
-	* config_transport:  Fix objects returned by ast_sip_get_transport_states
+	  When swapping a Local channel in place of one already
+	  in a bridge (to complete a bridge attended transfer),
+	  the channel that was swapped out can actually be hung
+	  up before the stasis bridge push callback executes on
+	  the independant transfer thread.  This results in the
+	  stasis app loop dropping out and removing the control
+	  that has the the app name which the local replacement
+	  channel needs so it can re-enter stasis.
+
+	  To avoid this race condition a new push_peek callback
+	  has been added, and called from the ast_bridge_impart
+	  thread before it launches the independant thread that
+	  will complete the transfer.  Now the stasis push_peek
+	  callback can copy the stasis app name before the swap
+	  channel can hang up.
 
-	  ast_sip_get_transport_states was returning a container of internal_state
-	  objects instead of ast_sip_transport_state objects.  This was causing
-	  transport lookups to fail, most noticably in res_pjsip_nat, which
-	  couldn't find the correct external addresses.  This was causing contacts
-	  to go out with internal ip addresses.
+	  ASTERISK-24649
+	  Review: https://reviewboard.asterisk.org/r/4382/
+	  ........
 
-	  ASTERISK-25830 #close
-	  Reported-by: Sean Bright
+	  Merged revisions 431450 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I1aee6a2fd46c42e8dd0af72498d17de459ac750e
 
-2016-03-02 11:17 +0000 [1ea7a5a774]  Scott Griepentrog <scott@griepentrog.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431451 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* CHAOS: cleanup possible null vars on msg alloc failure
+2015-01-29 15:20 +0000 [f61c80a8f7]  Mark Michelson <mmichelson@digium.com>
 
-	  In message.c, if msg_alloc fails to init the string field,
-	  vars may be null, so use a null tolerant cleanup.
+	* Allow disabling of 100rel support on PJSIP endpoints.
 
-	  In res_pjsip_messaging.c, if msg_data_create fails, mdata
-	  will be null, so use a null tolerant cleanup.
+	  Due to an inversion error, setting 100rel=no would not actually
+	  change the current value of the setting (which defaulted to "yes").
+	  With this fix, the inversion is corrected.
+	  ........
 
-	  ASTERISK-25323
+	  Merged revisions 431420 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ic2d55c2c3750d5616e2a05ea92a19c717507ff56
 
-2016-03-02 09:34 +0000 [3c37c7071f]  Scott Griepentrog <scott@griepentrog.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431436 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* CHAOS: prevent crash on failed strdup
+2015-01-29 15:02 +0000 [034798e37e]  Mark Michelson <mmichelson@digium.com>
 
-	  This patch avoids crashing on a null pointer
-	  if the strdup() allocation fails.
+	* Use SIPS URIs in Contact headers when appropriate.
 
-	  ASTERISK-25323
+	  RFC 3261 sections 8.1.1.8 and 12.1.1 dictate specific
+	  scenarios when we are required to use SIPS URIs in Contact
+	  headers. Asterisk's non-compliance with this could actually
+	  cause calls to get dropped when communicating with clients
+	  that are strict about checking the Contact header.
 
-	  Change-Id: I3f67434820ba53b53663efd6cbb42749f4f6c0f5
+	  Both of the SIP stacks in Asterisk suffered from this issue.
+	  This changeset corrects the behavior in res_pjsip/chan_pjsip.c
 
-2016-02-29 18:11 +0000 [9633be9d25]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4345
+	  ........
 
-	* func_callerid.c: Update REDIRECTING reason documentation.
+	  Merged revisions 431426 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I6e8d39b0711110a4bceafa652e58b30465e28386
 
-2016-02-26 18:57 +0000 [4165ea7778]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431427 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* SIP diversion: Fix REDIRECTING(reason) value inconsistencies.
+2015-01-29 14:54 +0000 [fe76d4829f]  Mark Michelson <mmichelson@digium.com>
 
-	  Previous chan_sip behavior:
+	* Use SIPS URIs in Contact headers when appropriate.
 
-	  Before this patch chan_sip would always strip any quotes from an incoming
-	  reason and pass that value up as the REDIRECTING(reason).  For an outgoing
-	  reason value, chan_sip would check the value against known values and
-	  quote any it didn't recognize.  Incoming 480 response message reason text
-	  was just assigned to the REDIRECTING(reason).
+	  RFC 3261 sections 8.1.1.8 and 12.1.1 dictate specific
+	  scenarios when we are required to use SIPS URIs in Contact
+	  headers. Asterisk's non-compliance with this could actually
+	  cause calls to get dropped when communicating with clients
+	  that are strict about checking the Contact header.
 
-	  Previous chan_pjsip behavior:
+	  Both of the SIP stacks in Asterisk suffered from this issue.
+	  This changeset corrects the behavior in chan_sip.
 
-	  Before this patch chan_pjsip would always pass the incoming reason value
-	  up as the REDIRECTING(reason).  For an outgoing reason value, chan_pjsip
-	  would send the reason value as passed down.
+	  ASTERISK-24646 #close
+	  Reported by Stephan Eisvogel
 
-	  With this patch:
+	  Review: https://reviewboard.asterisk.org/r/4346
+	  ........
 
-	  Both channel drivers match incoming reason values with values documented
-	  by REDIRECTING(reason) and values documented by RFC5806 regardless of
-	  whether they are quoted or not.  RFC5806 values are mapped to the
-	  equivalent REDIRECTING(reason) documented value and is set in
-	  REDIRECTING(reason).  e.g., an incoming RFC5806 'unconditional' value or a
-	  quoted string version ('"unconditional"') is converted to
-	  REDIRECTING(reason)'s 'cfu' value.  The user's dialplan only needs to deal
-	  with 'cfu' instead of any of the aliases.
+	  Merged revisions 431423 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The incoming 480 response reason text supported by chan_sip checks for
-	  known reason values and if not matched then puts quotes around the reason
-	  string and assigns that to REDIRECTING(reason).
+	  Merged revisions 431424 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Both channel drivers send outgoing known REDIRECTING(reason) values as the
-	  unquoted RFC5806 equivalent.  User custom values are either sent as is or
-	  with added quotes if SIP doesn't allow a character within the value as
-	  part of a RFC3261 Section 25.1 token.  Note that there are still
-	  limitations on what characters can be put in a custom user value.  e.g.,
-	  embedding quotes in the middle of the reason string is silly and just
-	  going to cause you grief.
 
-	  * Setting a REDIRECTING(reason) value now recognizes RFC5806 aliases.
-	  e.g., Setting REDIRECTING(reason) to 'unconditional' is converted to the
-	  'cfu' value.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431425 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Added missing malloc() NULL return check in res_pjsip_diversion.c
-	  set_redirecting_reason().
+2015-01-29 10:47 +0000 [8357ffab9c]  gtjoseph <george.joseph@fairview5.com>
 
-	  * Fixed potential read from a stale pointer in res_pjsip_diversion.c
-	  add_diversion_header().  The reason string needed to be copied into the
-	  tdata memory pool to ensure that the string would always be available.
-	  Otherwise, if the reason string returned by reason_code_to_str() was a
-	  user's reason string then the string could be freed later by another
-	  thread.
+	* res_pjsip_exten_state: Reduce log clutter... change a WARNING to a VERBOSE/2
 
-	  Change-Id: Ifba83d23a195a9f64d55b9c681d2e62476b68a87
+	  Reduce log clutter by changing the "Watcher for hint %s (removed|deactivated)"
+	  message from WARNING to VERBOSE/2.
 
-2016-02-26 18:54 +0000 [41f4af4ce5]  Richard Mudgett <rmudgett@digium.com>
+	  Tested-by: George Joseph
 
-	* res_pjsip_send_to_voicemail.c: Allow either quoted or not send_to_vm reason.
+	  Review: https://reviewboard.asterisk.org/r/4387/
+	  ........
 
-	  Change-Id: Id6350b3c7d4ec8df7ec89863566645e2b0f441fd
+	  Merged revisions 431403 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-02-29 20:41 +0000 [4c5998ff55]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_send_to_voicemail.c: Fix off-nominal double channel unref.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431404 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Fix double unref of other_party channel in off nominal path.
+2015-01-29 06:09 +0000 [9893ba7ffb]  Joshua Colp <jcolp@digium.com>
 
-	  * This is unlikely to be a real problem.  However, for safety,
-	  in handle_incoming_request() keep the datastore ref with the
-	  other_party channel ref until we are finished with the other_party
-	  channel.
+	* res_rtp_asterisk: Fix DTLS when used with OpenSSL 1.0.1k
 
-	  Change-Id: I78f22547bf0bb99fb20814ceab75952bd857f821
+	  A recent security fix for OpenSSL broke DTLS negotiation for many
+	  applications. This was caused by read ahead not being enabled when it
+	  should be. While a commit has gone into OpenSSL to force read ahead
+	  on for DTLS it may take some time for a release to be made and the
+	  change to be present in distributions (if at all). As enabling read
+	  ahead is a simple one line change this commit does that and fixes
+	  the issue.
+
+	  ASTERISK-24711 #close
+	  Reported by: Jared Biel
+	  ........
 
-2016-01-18 21:54 +0000 [b59956a875]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 431384 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* build-system: Allow building with static pjproject
+	  Merged revisions 431385 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Background here:
-	  http://lists.digium.com/pipermail/asterisk-dev/2016-January/075266.html
 
-	  From CHANGES:
-	   * To help insure that Asterisk is compiled and run with the same known
-	     version of pjproject, a new option (--with-pjproject-bundled) has been
-	     added to ./configure.  When specified, the version of pjproject specified
-	     in third-party/versions.mak will be downloaded and configured.  When you
-	     make Asterisk, the build process will also automatically build pjproject
-	     and Asterisk will be statically linked to it.  Once a particular version
-	     of pjproject is configured and built, it won't be configured or built
-	     again unless you run a 'make distclean'.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431386 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	     To facilitate testing, when 'make install' is run, the pjsua and pjsystest
-	     utilities and the pjproject python bindings will be installed in
-	     ASTDATADIR/third-party/pjproject.
+2015-01-28 11:42 +0000 [b3ff43a4e8]  Mark Michelson <mmichelson@digium.com>
 
-	     The default behavior remains building with the shared pjproject
-	     installation, if any.
+	* Fix file descriptor leak in RTP code.
 
-	  Building:
+	  SIP requests that offered codecs incompatible with configured values
+	  could result in the allocation of RTP and RTCP ports that would not get
+	  reclaimed later.
 
-	     All you have to do is include the --with-pjproject-bundled option on
-	     the ./configure command line (and remove any existing --with-pjproject
-	     option if specified).  Everything else is automatic.
+	  ASTERISK-24666 #close
+	  Reported by Y Ateya
 
-	  Behind the scenes:
+	  Review: https://reviewboard.asterisk.org/r/4323
 
-	     The top-level Makefile was modified to include 'third-party' in the
-	     list of MOD_SUBDIRS.
+	  AST-2015-001
+	  ........
 
-	     The third-party directory was created to contain any third party
-	     packages that may be needed in the future.  Its Makefile automatically
-	     iterates over any subdirectories passing on targets.
+	  Merged revisions 431300 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	     The third-party/pjproject directory was created to house the pjproject
-	     source distribution.  Its Makefile contains targets to download, patch
-	     configure, generate dependencies, compile libs, apps and python bindings,
-	     sanitized build.mak and generate a symbols list.
+	  Merged revisions 431303 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	     When bootstrap.sh is run, it automatically includes the configure.m4
-	     file in third-party/pjproject.  This file has a macro to download and
-	     conifgure pjproject and get and set PJPROJECT_INCLUDE, PJPROJECT_DIR
-	     and PJPROJECT_BUNDLED.  It also tests for the capabilities like
-	     PJ_TRANSACTION_GRP_LOCK by parsing preprocessor output as opposed to
-	     trying to compile.  Of course, bootstrap.sh is only run once and the
-	     configure file is incldued in the patch.
 
-	     When configure is run with the new options, the macro in configure.m4
-	     triggers the download, patch, conifgure and tests.  No compilation is
-	     performed at this time.  The downloaded tarball is cached in /tmp so
-	     it doesn't get downloaded again on a distclean.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431304 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	     When make is run in the top-level Asterisk source directory, it will
-	     automatically descend all the subdirectories in third_party just as it
-	     does for addons, apps, etc.  The top-level Makefile makes sure that
-	     the 'third-party' is built before 'main' so that dependencies from the
-	     other directories are built first.
+2015-01-28 11:34 +0000 [3cccfac399]  Mark Michelson <mmichelson@digium.com>
 
-	     When main does build, a new shared library (libasteriskpj) is created that
-	     links statically to the pjproject .a files and exports all their symbols.
-	     The asterisk binary links to that, just as it does with libasteriskssl.
+	* Multiple revisions 431297-431298
 
-	     When Asterisk is installed, the pjsua and pjsystest apps, and the pjproject
-	     python bindings are installed in ASTDATADIR/third-party/pjproject.  This
-	     will facilitate testing, including running the testsuite which will be
-	     updated to check that directory for the pjsua module ahead of the system
-	     python library.
+	  ........
+	    r431297 | mmichelson | 2015-01-28 11:05:26 -0600 (Wed, 28 Jan 2015) | 17 lines
+	    
+	    Mitigate possible HTTP injection attacks using CURL() function in Asterisk.
+	    
+	    CVE-2014-8150 disclosed a vulnerability in libcURL where HTTP request injection
+	    can be performed given properly-crafted URLs.
+	    
+	    Since Asterisk makes use of libcURL, and it is possible that users of Asterisk may
+	    get cURL URLs from user input or remote sources, we have made a patch to Asterisk
+	    to prevent such HTTP injection attacks from originating from Asterisk.
+	    
+	    ASTERISK-24676 #close
+	    Reported by Matt Jordan
+	    
+	    Review: https://reviewboard.asterisk.org/r/4364
+	    
+	    AST-2015-002
+	  ........
+	    r431298 | mmichelson | 2015-01-28 11:12:49 -0600 (Wed, 28 Jan 2015) | 3 lines
+	    
+	    Fix compilation error from previous patch.
+	  ........
 
-	  Modules should continue to depend on pjproject if they use pjproject APIs
-	  directly.  They should not care about the implementation.  No changes to any
-	  res_pjsip modules were made.
+	  Merged revisions 431297-431298 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ia7a60c28c2e9ba9537c5570f933c1ebcb20a3103
+	  Merged revisions 431299 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2016-02-22 16:59 +0000 [18a323e542]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 431301 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip.c: Fix T.38 issues caused by leaving a bridge.
 
-	  chan_sip could not handle AST_T38_TERMINATED frames being sent to it when
-	  the channel left the bridge.  The action resulted in overlapping outgoing
-	  reINVITEs.  The testsuite tests/fax/sip/directmedia_reinvite_t38 was not
-	  happy.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431302 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Force T.38 to be remembered as locally bridged.  Now when the channel
-	  leaves the native RTP bridge after T.38, the channel remembers that it has
-	  already reINVITEed the media back to Asterisk.  It just needs to terminate
-	  T.38 when the AST_T38_TERMINATED arrives.
+2015-01-28 06:19 +0000 [f080ca6536]  Sean Bright <sean@malleable.com>
 
-	  * Prevent redundant AST_T38_TERMINATED from causing problems.  Redundant
-	  AST_T38_TERMINATED frames could cause overlapping outgoing reINVITEs if
-	  they happen before the T.38 state changes to disabled.  Now the T.38 state
-	  is set to disabled before the reINVITE is sent.
+	* media formats: update res_format_attr_opus & silk
 
-	  ASTERISK-25582 #close
+	  In r419044, we changed how formats were handled, but the return value
+	  of the format_parse_sdp_fmtp functions in res_format_attr_opus and
+	  res_format_attr_silk were not updated, causing calls to fail.  Ran
+	  into this when getting codec_opus working with Asterisk 13.
 
-	  Change-Id: I53f5c6ce7d90b3f322a942af1a9bcab6d967b7ce
+	  Once the return value was corrected, we were crashing in opus_getjoint
+	  because of NULL format attributes.  I've fixed this as well in this
+	  patch.
 
-2016-02-18 18:27 +0000 [263a39f2cc]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4371/
+	  ........
 
-	* res_pjsip_t38.c: Back out part of an earlier fix attempt.
+	  Merged revisions 431267 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This backs out item 4 of the 4875e5ac32f5ccad51add6a4216947bfb385245d
-	  commit.  Item 4 added the t38_bye_supplement.  Unfortunately, the frame
-	  that it puts into the bridge may or may not be processed by the time the
-	  bridged peer is kicked out of the bridge.  If it is processed then all is
-	  well.  However, if it is not processed then that channel is stuck in fax
-	  mode until it hangs up or maybe if it joins another bridge for T.38
-	  faxing.
 
-	  ASTERISK-25582
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431268 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib20a03ecadf1bf8a0dcadfadf6c2f2e60919a9f7
+2015-01-27 22:29 +0000 [69e107b24e]  Richard Mudgett <rmudgett@digium.com>
 
-2016-02-22 13:54 +0000 [221422be50]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_outbound_registration: Fix reload race condition.
 
-	* bridge core: Add owed T.38 terminate when channel leaves a bridge.
+	  Performing a CLI "module reload" command when there are new pjsip.conf
+	  registration objects defined frequently failed to load them correctly.
 
-	  The channel is now going to get T.38 terminated when it leaves the
-	  bridging system and the bridged peers are going to get T.38 terminated as
-	  well.
+	  What happens is a race condition between res_pjsip pushing its reload into
+	  an asynchronous task processor task and the thread that does the rest of
+	  the reloads when it gets to reloading the res_pjsip_outbound_registration
+	  module.  A similar race condition happens between a reload and the CLI/AMI
+	  show registrations commands.  The reload updates the current_states
+	  container and the CLI/AMI commands call get_registrations() which builds a
+	  new current_states container.
 
-	  ASTERISK-25582
+	  * Made res_pjsip.c reload_module() use ast_sip_push_task_synchronous()
+	  instead of ast_sip_push_task() to eliminate two threads processing config
+	  reloads at the same time.
 
-	  Change-Id: I77a9205979910210e3068e1ddff400dbf35c4ca7
+	  * Made get_registrations() not replace the global current_states container
+	  so the CLI/AMI show registrations command cannot interfere with reloading.
+	  You could never add/remove objects in the container without the
+	  possibility of the container being replaced out from under you by
+	  get_registrations().
 
-2016-02-19 16:01 +0000 [0a5bc64491]  Richard Mudgett <rmudgett@digium.com>
+	  * Added a registration loaded sorcery instance observer to purge any dead
+	  registration objects since get_registrations() cannot do this job anymore.
+	  The struct ast_sorcery_instance_observer callbacks must be used because
+	  the callback happens inline with the load process.  The struct
+	  ast_sorcery_observer callbacks are pushed to a different thread.
 
-	* channel api: Create is_t38_active accessor functions.
+	  * Added some global current_states NULL pointer checks in case the
+	  container disappears because of unload_module().
 
-	  ASTERISK-25582
+	  * Made sorcery's struct ast_sorcery_instance_observer.object_type_loaded
+	  callbacks guaranteed to be called before any struct
+	  ast_sorcery_observer.loaded callbacks will be called.
 
-	  Change-Id: I69451920b122de7ee18d15bb231c80ea7067a22b
+	  * Moved the check for non-reloadable objects to before the sorcery
+	  instance loading callbacks happen to short circuit unnecessary work.
+	  Previously with non-reloadable objects, the sorcery instance
+	  loading/loaded callbacks would always happen, the individual wizard
+	  loading/loaded would be prevented, and the non-reloadable type logging
+	  message would be logged for each associated wizard.
 
-2016-02-19 19:06 +0000 [513638a5f4]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24729 #close
+	  Review: https://reviewboard.asterisk.org/r/4381/
+	  ........
 
-	* bridge_channel: Don't settle owed events on an optimization.
+	  Merged revisions 431243 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Local channel optimization could cause DTMF digits to be duplicated.
-	  Pending DTMF end events would be posted to a bridge when the local channel
-	  optimizes out and is replaced by the channel further down the chain.  When
-	  the real digit ends, the channel would get another DTMF end posted to the
-	  bridge.
 
-	  A -- LocalA;1/n -- LocalA;2/n -- LocalB;1 -- LocalB;2 -- B
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431251 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  1) LocalA has the /n flag to prevent optimization.
-	  2) B is sending DTMF to A through the local channel chain.
-	  3) When LocalB optimizes out it can move B to the position of LocalB;1
-	  4) Without this patch, when B swaps with LocalB;1 then LocalB;1 would
-	  settle an owed DTMF end to the bridge toward LocalA;2.
-	  5) When B finally ends its DTMF it sends the DTMF end down the chain.
-	  6) Without this patch, A would hear the DTMF digit end when LocalB
-	  optimizes out and when B ends the original digit.
+2015-01-27 16:58 +0000 [c7591ef6bc]  Kevin Harwell <kharwell@digium.com>
 
-	  ASTERISK-25582
+	* tcptls: Bad file descriptor error when reloading chan_sip
 
-	  Change-Id: I1bbd28b8b399c0fb54985a5747f330a4cd2aa251
+	  While running through some scenarios using chan_sip and tcp a problem would
+	  occur that resulted in a flood of bad file descriptor messages on the cli:
 
-2016-02-22 12:15 +0000 [7c4495cb70]  Richard Mudgett <rmudgett@digium.com>
+	  tcptls.c:712 ast_tcptls_server_root: Accept failed: Bad file descriptor
 
-	* channel.c: Route all control frames to a channel through the same code.
+	  The message is received because the underlying socket has been closed, so is
+	  valid. This is probably happening because unloading of chan_sip is not atomic.
+	  That however is outside the scope of this patch. This patch simply stops the
+	  logging of multiple occurrences of that message.
 
-	  Frame hooks can conceivably return a control frame in exchange for an
-	  audio frame inside ast_write().  Those returned control frames were not
-	  handled quite the same as if they were sent to ast_indicate().  Now it
-	  doesn't matter if you use ast_write() to send an AST_FRAME_CONTROL to a
-	  channel or ast_indicate().
+	  ASTERISK-24728 #close
+	  Reported by: Thomas Thompson
+	  Review: https://reviewboard.asterisk.org/r/4380/
+	  ........
 
-	  ASTERISK-25582
+	  Merged revisions 431218 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I5775f41421aca2b510128198e9b827bf9169629b
+	  Merged revisions 431219 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-02-25 15:13 +0000 [48d713a832]  gtjoseph <george.joseph@fairview5.com>
 
-	* sorcery:  Refactor create, update and delete to better deal with caches
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431220 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The ast_sorcery_create, update and delete function have been refactored
-	  to better deal with caches and errors.
+2015-01-27 13:31 +0000 [e826cb8a26]  Jonathan Rose <jrose@digium.com>
 
-	  The action is now called on all non-caching wizards first. If ANY succeed,
-	  the action is called on all caching wizards and the observers are notified.
-	  This way we don't put something in the cache (or update or delete) before
-	  knowing the action was performed in at least 1 backend and we only call the
-	  observers once even if there were multiple writable backends.
+	* Manager: Fix Manager Action ModuleLoad to give correct response when reloading
 
-	  ast_sorcery_create was never adding to caches in the first place which
-	  was preventing contacts from getting added to a memory_cache when they
-	  were created.  In turn this was causing memory_cache to emit errors if
-	  the contact was deleted before being retrieved (which would have
-	  populated the cache).
+	  Prior to this patch, ModuleLoad would respond with an error indicating that
+	  the requested module wasn't found in spite of finding and reloading the
+	  module.
 
-	  ASTERISK-25811 #close
-	  Reported-by: Ross Beer
+	  Review: https://reviewboard.asterisk.org/r/4373/
+	  ASTERISK-24721 #close
+	  ........
 
-	  Change-Id: Id5596ce691685a79886e57b0865888458d6e7b46
-2016-02-25 15:39 +0000 [ee947d4a7a]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 431153 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_mwi:  Turn some NOTICEs and WARNINGs into debug 1s.
 
-	  There are a few cases where we're emitting notices or warnings
-	  for things that really need neither, like a client retrying to subscribe
-	  to mwi when they're not conifgured for it.  They get a 404 so there's no
-	  need for non-debug messages.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431201 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I05e38a7ff6c2f2521146f4be6a79731b9864e61f
-2016-02-25 14:17 +0000 [6e70e8ccdb]  gtjoseph <george.joseph@fairview5.com>
+2015-01-27 13:22 +0000 [3b0f03ef7b]  Kevin Harwell <kharwell@digium.com>
 
-	* res_sorcery_memory_cache:  Fix SEGV in some CLI commands
+	* chan_sip: stale nonce causes failure
 
-	  A few of the CLI commands weren't checking for enough arguments
-	  and were SEGVing.
+	  When refreshing (with a small expiration) a registration that was sent to
+	  chan_sip the nonce would be considered stale and reject the registration.
+	  What was happening was that the initial registration's "dialog" still existed
+	  in the dialogs container and upon refresh the dialog match algorithm would
+	  choose that as the "dialog" instead of the newly created one. This occurred
+	  because the algorithm did not check to see if the from tag matched if
+	  authentication info was available after the 401. So, it ended up assuming
+	  the original "dialog" was a match and stopped the search. The old "dialog"
+	  of course had an old nonce, thus the stale nonce message.
 
-	  Change-Id: Ie6494132ad2fe54b4f014bcdc112a37c36a9b413
+	  This fix attempts to leave the original functionality alone except in the case
+	  of a REGISTER. If a REGISTER is received if searches for an existing "dialog"
+	  matching only on the callid. If the expires value is low enough it will reuse
+	  dialog that is there, otherwise it will create a new one.
 
-2016-02-25 10:29 +0000 [4417f64d83]  Leif Madsen <leif@leifmadsen.com>
+	  ASTERISK-24715 #close
+	  Reported by: John Bigelow
+	  Review: https://reviewboard.asterisk.org/r/4367/
+	  ........
 
-	* Add initial support to build Docker images
+	  Merged revisions 431187 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This work-in-progress is the first step to being able to reliably
-	  build Asterisk containers from the Asterisk source. I'm submitting
-	  this based on feedback gained at AstriDevCon 2015.
+	  Merged revisions 431194 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Information about how to use this is provided in contrib/docker/README.md
-	  and will result in a local Asterisk container being built right from
-	  your source. I believe this can eventually be automated via
-	  hub.docker.com.
 
-	  Change-Id: Ifa070706d40e56755797097b6ed72c1e243bd0d1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431197 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-02-22 19:31 +0000 [e7a6abbbd3]  Richard Mudgett <rmudgett@digium.com>
+2015-01-27 13:12 +0000 [e62bd46511]  Corey Farrell <git@cfware.com> (license 5909)
 
-	* rtp_engine.h: Remove extraneous semicolons.
+	* res_pjsip: make it unloadable (take 2)
 
-	  Change-Id: Ib462633d396fa941379dfef648dcd2245e350084
+	  Due to the original patch causing memory corruptions it was removed until the
+	  problem could be resolved. This patch is the original patch plus some added
+	  locking around stasis router subcription that was needed to avoid the memory
+	  corruption.
 
-2016-02-23 14:57 +0000 [6656afffa0]  Richard Mudgett <rmudgett@digium.com>
+	  Description of the original problem and patch (still applicable):
 
-	* chan_sip.c: Suppress T.38 SDP c= line if addr is the same.
+	  The res_pjsip module was previously unloadable. With this patch it can now
+	  be unloaded.
 
-	  Use the correct comparison function since we only care if the address
-	  without the port is the same.
+	  This patch is based off the original patch on the issue (listed below) by Corey
+	  Farrell with a few modifications. Namely, removed a few changes not required to
+	  make the module unloadable and also fixed a bug that would cause asterisk to
+	  crash on unloading.
+
+	  This patch is the first step (should hopefully be followed by another/others at
+	  some point) in allowing res_pjsip and the modules that depend on it to be
+	  unloadable. At this time, res_pjsip and some of the modules that depend on
+	  res_pjsip cannot be unloaded without causing problems of some sort.
+
+	  The goal of this patch is to get res_pjsip and only res_pjsip to be able to
+	  unload successfully and/or shutdown without incident (crashes, leaks, etc...).
+	  Other dependent modules may still cause problems on unload.
+
+	  Basically made sure, with the patch applied, that res_pjsip (with no other
+	  dependent modules loaded) could be succesfully unloaded and Asterisk could
+	  shutdown without any leaks or crashes that pertained directly to res_pjsip.
 
-	  Change-Id: Ibf6c485f843a1be6dee58a47b33d81a7a8cbe3b0
+	  ASTERISK-24485 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4363/
+	  patches:
+	    pjsip_unload-broken-r1.patch submitted by Corey Farrell (license 5909)
+	  ........
 
-2016-02-16 08:14 +0000 [ea9deff996]  Christof Lauber <christof.lauber@annax.ch>
+	  Merged revisions 431179 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_config_sqlite3: Fix crashes when reading peers from sqlite3 tables
 
-	  Introduced realloaction of ast_str buf in sqlite3_escape functions in case
-	  the returned buffer from threadstorage was actually too small.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431180 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I3c5eb43aaade93ee457943daddc651781954c445
+2015-01-27 11:48 +0000 [94eebd5ba5]  Richard Mudgett <rmudgett@digium.com>
 
-2016-02-11 11:01 +0000 [d2a1457e0b]  gtjoseph <george.joseph@fairview5.com>
+	* app_confbridge: Repeatedly starting and stopping recording ref leaks the recording channel.
 
-	* res_pjsip/config_transport: Allow reloading transports.
+	  Starting and stopping conference recording more than once causes the
+	  recording channels to be leaked.  For v13 the channels also show up in the
+	  CLI "core show channels" output.
 
-	  The 'reload' mechanism actually involves closing the underlying
-	  socket and calling the appropriate udp, tcp or tls start functions
-	  again.  Only outbound_registration, pubsub and session needed work
-	  to reset the transport before sending requests to insure that the
-	  pjsip transport didn't get pulled out from under them.
+	  * Reworked and simplified the recording channel code to use
+	  ast_bridge_impart() instead of managing the recording thread in the
+	  ConfBridge code.  The recording channel's ref handling easily falls into
+	  place and other off nominal code paths get handled better as a result.
 
-	  In my testing, no calls were dropped when a transport was changed
-	  for any of the 3 transport types even if ip addresses or ports were
-	  changed. To be on the safe side however, a new transport option was
-	  added (allow_reload) which defaults to 'no'.  Unless it's explicitly
-	  set to 'yes' for a transport, changes to that transport will be ignored
-	  on a reload of res_pjsip.  This should preserve the current behavior.
+	  ASTERISK-24719 #close
+	  Reported by: John Bigelow
 
-	  Change-Id: I5e759850e25958117d4c02f62ceb7244d7ec9edf
+	  Review: https://reviewboard.asterisk.org/r/4368/
+	  Review: https://reviewboard.asterisk.org/r/4369/
+	  ........
 
-2016-02-07 17:34 +0000 [6b921f706d]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 431135 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_pjproject:  Add ability to map pjproject log levels to Asterisk log levels
+	  Merged revisions 431160 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Warnings and errors in the pjproject libraries are generally handled by
-	  Asterisk.  In many cases, Asterisk wouldn't even consider them to be warnings
-	  or errors so the messages emitted by pjproject directly are either superfluous
-	  or misleading.  A good exampe of this are the level-0 errors pjproject emits
-	  when it can't open a TCP/TLS socket to a client to send an OPTIONS.  We don't
-	  consider a failure to qualify a UDP client an "ERROR", why should a TCP/TLS
-	  client be treated any differently?
 
-	  A config file for res_pjproject has bene added (pjproject.conf) and a new
-	  log_mappings object allows mapping pjproject levels to Asterisk levels
-	  (or nothing).  The defaults if no pjproject.conf file is found are the same
-	  as those that were hard-coded into res_pjproject initially: 0,1 = LOG_ERROR,
-	  2 = LOG_WARNING, 3,4,5 = LOG_DEBUG<level>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431161 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iba7bb349c70397586889b8f45b8c3d6c6c8c3898
+2015-01-27 11:34 +0000 [a43d24a9d3]  Joshua Colp <jcolp@digium.com>
 
-2016-02-18 10:55 +0000 [f295088764]  Alexei Gradinari <alex2grad@gmail.com>
+	* bridge / res_pjsip_sdp_rtp: Fix issues with media not being reinvited during direct media.
 
-	* res_pjsip_outbound_publish: Fix processing 412 response
+	  This change fixes two issues:
 
-	  When Asterisk receives a 412 (Conditional Request Failed) response
-	  it has to recreate publish session.
-	  There is bug in res_pjsip_outbound_publish.c
-	  The function sip_outbound_publish_client_alloc is called with wrong object
-	  while processing 412 (Conditional Request Failed) response.
-	  This patch fixes it.
+	  1. During a swap operation bridging added the new channel before having the swap channel
+	  leave. This was not handled in bridge_native_rtp and could result in a channel not getting
+	  reinvited back to Asterisk. After this change the swap channel will leave first and the
+	  new channel will then join.
 
-	  ASTERISK-25229 #close
+	  2. If a re-invite was received after a session had been established any upstream elements
+	  (such as bridge_native_rtp) were not notified that they may want to re-evaluate things.
+	  After this change an UPDATE_RTP_PEER control frame is queued when this situation occurs
+	  and upstream can react.
 
-	  Change-Id: I3b62f2debf6bb1e5817cde7b13ea39ef2bf14359
-2016-02-18 11:15 +0000 [f1f79812c1]  Mark Michelson <mmichelson@digium.com>
+	  AST-1524 #close
 
-	* Fix failing threadpool_auto_increment test.
+	  Review: https://reviewboard.asterisk.org/r/4378/
+	  ........
 
-	  The threadpool_auto_increment test fails infrequently for a couple of
-	  reasons
-	  * The threadpool listener was notified of fewer tasks being pushed than
-	    were actually pushed
-	  * The "was_empty" flag was set to an unexpected value.
+	  Merged revisions 431157 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The problem is that the test pushes three tasks into the threadpool.
-	  Test expects the threadpool to essentially gather those three tasks, and
-	  then distribute those to the threadpool threads. It also expects that as
-	  the tasks are pushed in, the threadpool listener is alerted immediately
-	  that the tasks have been pushed. In reality, a task can be distributed
-	  to the threadpool threads quicker than expected, meaning that the
-	  threadpool has already emptied by the time each subsequent task is
-	  pushed. In addition, the internal threadpool queue can be delayed so
-	  that the threadpool listener is not alerted that a task has been pushed
-	  even after the task has been executed.
 
-	  From the test's point of view, there's no way to be able to predict
-	  exactly the order that task execution/listener notifications will occur,
-	  and there is no way to know which listener notifications will indicate
-	  that the threadpool was previously empty.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431158 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  For this reason, the test has been updated to only check the things it
-	  can check. It ensures that all tasks get executed, that the threads go
-	  idle after the tasks are executed, and that the listener is told the
-	  proper number of tasks that were pushed.
+2015-01-27 11:21 +0000 [fb8a2e0399]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I7673120d74adad64ae6894594a606e102d9a1f2c
+	* ARI: Improve wiki documentation
 
-2016-02-16 23:37 +0000 [79dc5e2f00]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  This patch improves the documentation of ARI on the wiki. Specifically, it
+	  addresses the following:
+	  * Allowed values and allowed ranges weren't documented. This was particularly
+	    frustrating, as Asterisk would reject query parameters with disallowed values
+	    - but we didn't tell anyone what the allowed values were.
+	  * The /play/id operation on /channels and /bridges failed to document all of
+	    the added media resource types.
+	  * Documentation for creating a channel into a Stasis application failed to
+	    note when it occurred, and that creating a channel into Stasis conflicts with
+	    creating a channel into the dialplan.
+	  * Some other minor tweaks in the mustache templates, including italicizing the
+	    parameter type, putting the default value on its own sub-bullet, and some
+	    other nicities.
 
-	* app_queue: fix Calculate talktime when is first call answered
+	  Review: https://reviewboard.asterisk.org/r/4351
+	  ........
 
-	  Fix calculate of average time for talktime is wrong when is completed the
-	  first call beacuse the time for talked would be that call.
+	  Merged revisions 431145 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25800 #close
 
-	  Change-Id: I94f79028935913cd9174b090b52bb300b91b9492
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431148 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-02-17 13:30 +0000 [5a3a857dd6]  Richard Mudgett <rmudgett@digium.com>
+2015-01-27 11:16 +0000 [aa8fd7d1b9]  Matt Jordan <mjordan@digium.com>
 
-	* cel.c: Fix mismatch in ast_cel_track_event() return type.
+	* app_confbridge: Restore user's menu name to CLI output of 'confbridge list'
 
-	  The return type of ast_cel_track_event() is not large enough to return all
-	  64 potential bits of the event enable mask.  Fortunately, the defined CEL
-	  events do not really need all 64 bits and the return value is only used to
-	  determine if the requested CEL event is enabled.
+	  When issuing a 'confbridge list XXXX' CLI command, the resulting output no
+	  longer displays the menu associated with a ConfBridge participant.
 
-	  * Made the ast_cel_track_event() return 0 or 1 only so the return value
-	  can fit inside an int type instead of zero or a truncated 64 bit non-zero
-	  value.
+	  The issue was caused by ASTERISK-22760. When that patch was done, it removed
+	  the copying of the menu name associated with the user from the actual user
+	  profile.
 
-	  Change-Id: I783d932320db11a95c7bf7636a72b6fe2566904c
+	  This patch fixes the issue by copying the menu name over to the user profile
+	  when the menu hooks are applied to the user. Since that function now does a
+	  little bit more than just apply the hooks, the name of the function has been
+	  changed to cover the copying of the menu name over as well.
 
-2016-02-16 16:37 +0000 [87ab65c557]  gtjoseph <george.joseph@fairview5.com>
+	  In addition, there is a disparity between the menu name length as it is stored
+	  on the conf_menu structure and the confbridge_user structure; this patch makes
+	  the lengths match so that a strcpy can be used.
 
-	* res_odbc: Fix exports.in for missing symbols
+	  Review: https://reviewboard.asterisk.org/r/4372/
 
-	  res_odbc.exports.in was missing a few symbols.
-	  Changed to wildcards.
+	  ASTERISK-24723 #close
+	  Reported by: Steve Pitts
+	  ........
 
-	  Change-Id: Ieadd76df24e43ea92577f651d478a0f7b742c30c
+	  Merged revisions 431134 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-02-16 12:20 +0000 [c0f3062031]  gtjoseph <george.joseph@fairview5.com>
 
-	* res_statsd:  Fix exports.in for missing symbols
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431136 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  res_statsd.export.in was missing the _va variations of the log
-	  functions causing Asterisk to crash in res_pjsip if OPTIONAL_API
-	  wasn't enabled.
+2015-01-27 05:47 +0000 [2504f97b01]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25727 #close
-	  Reported-by: Gergely Dömsödi
+	* res_parking: Fix crash due to race condition when unloading.
 
-	  Change-Id: I395729f9f51bdd33c5ca757f5f96ebedad74077b
+	  There is currently a race condition when unloading the res_parking
+	  module. Depending on the will of the universe the subscription
+	  invocation may occur AFTER the module is unloaded. This is because
+	  the module does NOT use stasis_unsubscribe_and_join when terminating
+	  the subscription. It merely uses stasis_unsubscribe.
 
-2016-02-15 21:31 +0000 [5e848dae7b]  gtjoseph <george.joseph@fairview5.com>
+	  This change makes it use stasis_unsubscribe_and_join which is documented
+	  for usage in this exact scenario.
 
-	* res_pjsip_config_wizard:  Add command to export primitive objects
+	  AST-1520 #close
 
-	  A new command (pjsip export config_wizard primitives) has been added that
-	  will export all the pjsip objects it created to the console or a file
-	  suitable for reuse in a pjsip.conf file.
+	  Review: https://reviewboard.asterisk.org/r/4375/
+	  ........
 
-	  ASTERISK-24919 #close
-	  Reported-by: Ray Crumrine
+	  Merged revisions 431114 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ica2a5f494244b4f8345b0437b16d06aa0484452b
 
-2016-02-15 15:37 +0000 [34c64707d1]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431115 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_caller_id: Fix segfault when replacing rpid or pai header
+2015-01-26 08:50 +0000 [965777ccfc]  David M. Lee <dlee@digium.com>
 
-	  If the PJSIP_HEADER dialplan function adds a PAI or RPID header and send_rpid
-	  or send_pai is set, res_pjsip_caller_id attemps to retrieve, parse and modify
-	  the header added by the dialplan function.  Since the header added by the
-	  dialplan function is generic string, there are no virtual functions to parse
-	  the uri and we get a segfault when we try.  Since the modify, was really only
-	  an overwrite, we now just delete the old header if it was type PJSIP_H_OTHER
-	  and recreate it.
+	* Various fixes for OS X
 
-	  This raises a question for another time though:  What should happen with
-	  duplicate headers?  Right now res_pjsip_header_funcs doesn't check for dups
-	  so if it's session supplement is loaded after res_pjsip_caller_id's (or any
-	  other module that adds headers), there'll be dups in the message.
+	  This patch addresses compilation errors on OS X. It's been a while, so
+	  there's quite a few things.
 
-	  ASTERISK-25337 #close
+	   * Fixed __attribute__ decls in route.h to be portable.
+	   * Fixed htonll and ntohll to work when they are defined as macros.
+	   * Replaced sem_t usage with our ast_sem wrapper.
+	   * Added ast_sem_timedwait to our ast_sem wrapper.
+	   * Fixed some GCC 4.9 warnings using sig*set() functions.
+	   * Fixed some format strings for portability.
+	   * Fixed compilation issues with res_timing_kqueue (although tests still fail
+	     on OS X).
+	   * Fixed menuconfig /sbin/launchd detection, which disables res_timing_kqueue
+	     on OS X).
 
-	  Change-Id: I5e296b52d30f106b822c0eb27c4c2b0e0f71c7fa
+	  ASTERISK-24539 #close
+	  Reported by: George Joseph
 
-2016-02-15 13:08 +0000 [ebe167f792]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24544 #close
+	  Reported by: George Joseph
 
-	* Fix creation race of contact_status structures.
+	  Review: https://reviewboard.asterisk.org/r/4327/
+	  ........
 
-	  It is possible when processing a SIP REGISTER request to have two
-	  threads end up creating contact_status structures in sorcery.
-	  contact_status is created using a "find or create" function. If two
-	  threads call into this at the same time, each thread will fail to find
-	  an existing contact_status, and so both will end up creating a new
-	  contact status.
+	  Merged revisions 431092 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  During testing, we would see sporadic failures because the
-	  PJSIP_CONTACT() dialplan function would operate on a different
-	  contact_status than what had been updated by res_pjsip/pjsip_options.
 
-	  The fix here is two-fold:
-	  1) The "find or create" function for contact_status now has a lock
-	  around the entire operation. This way, if two threads attempt the
-	  operation simultaneously, the first to get there will create the object,
-	  and the second will find the object created by the first thread.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431093 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  2) res_sorcery_memory has had its create callback updated so that it
-	  will not allow for objects with duplicate IDs to be created.
+2015-01-25 07:43 +0000 [a8ae5a7bcb]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I55b1460ff1eb0af0a3697b82d7c2bac9f6af5b97
+	* dynamic realtime: Updates fail to work due to update fields being passed over
 
-2016-02-15 12:52 +0000 [1c4f2a920d]  Joshua Colp <jcolp@digium.com>
+	  When a crash was fixed due to usage of the REALTIME function in r423003, a
+	  regression was introduced into ast_update2_realtime where the update fields
+	  passed to the function would be skipped and the lookup field processed twice.
 
-	* res_pjsip_pubsub: Move where the subscription is stored to after initialized.
+	  The use of this function is a bit interesting: A variable argument list is
+	  used with two sentinel values - the first marks the end of the lookup
+	  fields/values; the second marks the end of the update fields/values.
+	  Unfortunately, ast_update2_realtime parses over the lookup fields twice, as
+	  opposed to parsing over the update fields. This causes the lookups to succeed,
+	  but the updates itself to have no effect.
 
-	  A problem arose when testing the AMI subscription listing actions where it
-	  was possible for a subscription that had not been fully initialized to be
-	  listed. This was problematic as the underlying listing code would crash.
+	  Note that the most common instance of this problem occurred in app_voicemail
+	  during the updating of a mailbox password.
 
-	  This change makes it so the subscription tree is fully set up before it is
-	  added to the list of subscriptions. This ensures that when the listing actions
-	  get the subscription it is valid.
+	  Thanks to the issue reporter, Paddy Grice, for pointing out the problem.
 
-	  ASTERISK-25738 #close
+	  Review: https://reviewboard.asterisk.org/r/4356/
 
-	  Change-Id: Iace2b13641c31bbcc0d43a39f99aba1f340c0f48
+	  ASTERISK-24231
 
-2015-02-20 20:51 +0000 [ac00c6bc2d]  Corey Farrell <git@cfware.com>
+	  ASTERISK-24626 #close
+	  Reported by: Paddy Grice
+	  ........
 
-	* main/asterisk.c: Reverse #if statement in listener() to fix code folding.
+	  Merged revisions 431072 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  listener() opens the same code block in two places (#if and #else).  This
-	  confuses some folding editors causing it to think that an extra code block
-	  was opened.  Folding in 'geany' causes all code after listener() to be
-	  folded as if it were part of that procedure.
 
-	  ASTERISK-24813 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431073 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I4b8c766e6c91e327dd445e8c18f8a6f268acd961
+2015-01-23 14:17 +0000 [b69b0d12ee]  Richard Mudgett <rmudgett@digium.com>
 
-2016-02-09 17:34 +0000 [b1b797e0e7]  gtjoseph <george.joseph@fairview5.com>
+	* app_confbridge: Shorten CBRec channel names to CBRec/<conf_name>-<seq-num>
 
-	* res_pjsip:  Refactor load_module/unload_module
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431055 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  load_module was just too hairy with every step having to clean up all
-	  previous steps on failure.
+2015-01-23 14:14 +0000 [c780223507]  Richard Mudgett <rmudgett@digium.com>
 
-	  Some of the pjproject init calls have now been moved to a separate
-	  load_pjsip function and the unload_pjsip function was enhanced to clean
-	  up everything if an error happened at any stage of the load process.
+	* app_confbridge: Make CBRec channel names more unique.
 
-	  In the process, a bunch of missing pj_shutdowns, serializer_pool_shutdowns
-	  and ast_threadpool_shutdowns were also corrected.
+	  Channel names should be different from other channels in the system while
+	  the channel exists.
 
-	  Change-Id: I5eec711b437c35b56605ed99537ebbb30463b302
+	  * Use a sequence number for CBRec channels instead of a random number
+	  because the same random number could be picked again for the next CBRec
+	  channel.
+	  ........
 
-2016-02-09 22:42 +0000 [20e9792fbc]  Badalyan Vyacheslav <slavon.net@gmail.com>
+	  Merged revisions 431052 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Resources/res_phoneprov: fix memory leak and heap-use-after-free
 
-	  * heap-use-after-free happens when we free "cfg"
-	  but then use "value" which refers to it
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431053 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * A memory leak occurs because in some cases
-	  it is not released "defaults"
+2015-01-23 13:51 +0000 [b38be992b1]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25721 #close
-	  Reported by: Badalyan Vyacheslav
-	  Tested by: Badalyan Vyacheslav
+	* app_confbridge: Whitespace
 
-	  Change-Id: I3807d3f4726df6864430ec144cf6265d3f538469
+	  Because there is sometimes no sence to any whitespace.
+	  ........
 
-2016-02-11 11:21 +0000 [962a9d61f8]  Etienne Lessard (license #6394)
+	  Merged revisions 431049 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* func_iconv: Ensure output strings are properly terminated.
+	  Merged revisions 431050 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25272 #close
-	  Reported by: Etienne Lessard
-	  patches:
-	   AST-25272.patch submitted by Etienne Lessard (license #6394)
 
-	  Change-Id: Id75ad202300960a1e91afe15e319d992936ecc17
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431051 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-02-10 16:16 +0000 [c1bf014ea0]  gtjoseph <george.joseph@fairview5.com>
+2015-01-23 12:46 +0000 [89610adda5]  David M. Lee <dlee@digium.com>
 
-	* res_pjsip:  Handle pjsip_dlg_create_uas deprecation
+	* Add depend on pjproject to res_pjsip_config_wizard.c
+	  ........
 
-	  Pjproject has deprecated pjsip_dlg_create_uas in 2.5 and replaced it with
-	  pjsip_dlg_create_uas_and_inc_lock which, as the name implies, automatically
-	  increments the lock on the returned dialog.  To account for this, configure.ac
-	  now detects the presence of pjsip_dlg_create_uas_and_inc_lock and res_pjsip.c
-	  has an #ifdef HAVE_PJSIP_DLG_CREATE_UAS_AND_INC_LOCK to decide whether to use
-	  the original call or the new one.  If the new one was used, the ref count is
-	  decremented before returning.
+	  Merged revisions 431030 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25751 #close
-	  Reported-by Josh Colp
 
-	  Change-Id: I1be776b94761df03bd0693bc7795a75682615ca8
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431034 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-02-09 23:40 +0000 [bd07b6f0dd]  Badalyan Vyacheslav <slavon.net@gmail.com>
+2015-01-23 09:21 +0000 [ca02121ef7]  Kevin Harwell <kharwell@digium.com>
 
-	* Build: Added testing compiler to support the system sanitizes
+	* Investigate and fix memory leaks in Asterisk
 
-	  In older versions of the compiler was not sanitizes.
-	  Compilers other than GCC can not support the Usan and TSAN
-	  or have other options for *FLAGS.
+	  Fixed memory leaks that were found in Asterisk.
 
-	  ASTERISK-25767 #close
-	  Reported by: Badalyan Vyacheslav
-	  Tested by: Badalyan Vyacheslav
+	  ASTERISK-24693 #close
+	  Reported by:  Kevin Harwell
+	  Review: https://reviewboard.asterisk.org/r/4347/
+	  ........
 
-	  Change-Id: Iefce6608221fa87884b82ae3cb5649b7b1804916
+	  Merged revisions 430999 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-02-09 20:57 +0000 [e9e896abd1]  Badalyan Vyacheslav <v.badalyan@open-bs.ru>
 
-	* Build: Fix menuselect USAN conflicts
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431010 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  USAN can be used together with other sanitizers.
+2015-01-23 09:13 +0000 [49cbfa7de6]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  Reported by: Badalyan Vyacheslav
-	  Tested by: Badalyan Vyacheslav
+	* Fix typo's (retrieve, specified, address).
+	  ........
 
-	  Change-Id: I3bffa350d70965c3026651dba3a12414d0aaa45f
+	  Merged revisions 430996 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-02-09 14:21 +0000 [93e8ed0154]  Corey Farrell <git@cfware.com>
+	  Merged revisions 430998 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Simplify and fix conditional in FD_SET.
 
-	  FD_SET contains a conditional statement to protect against buffer
-	  overruns.  The statement was overly complicated and prevented use
-	  of the last array element of ast_fdset.  We now just verify the fd
-	  is less than ast_FDMAX.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@431000 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I41895c0b497b052aef5bf49d75c817c48b326f40
+2015-01-23 08:39 +0000 [874cb5615d]  HZMI8gkCvPpom0tM (License 6658)
 
-2016-02-09 07:11 +0000 [a7c8d4cd6b]  Joshua Colp <jcolp@digium.com>
+	* chan_sip: Case insensitive comparison of "defaultuser" parameter.
 
-	* tests/test_sorcery_memory_cache_thrash: Improve termination process.
+	  All the other configuration options are case insensitive, so this one
+	  should be too.
 
-	  When terminating the threads thrashing a sorcery memory cache each
-	  would be told to stop and then we would wait on them. During at
-	  least one thrashing test this was problematic due to the specific
-	  usage pattern in use. It would take some time for termination of the
-	  thread to occur.
+	  ASTERISK-24355 #close
+	  Reported by: HZMI8gkCvPpom0tM
+	  patches:
+	    ast.patch uploaded by HZMI8gkCvPpom0tM (License 6658)
+	  ........
 
-	  This would occur due to contention between the threads retrieving
-	  and the threads updating the cache. As the retrieving threads are
-	  given priority it may be some time before the updating threads
-	  are able to proceed.
+	  Merged revisions 430993 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This change makes it so all threads are told to stop and then each
-	  are joined to ensure they stop. This way all the threads should
-	  stop at around the same time instead of waiting for one to stop,
-	  the next to stop, then the next, and so on. As a result of this
-	  the execution time for each thrash test is much closer to their
-	  expected value than previously seen as well.
+	  Merged revisions 430994 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I04a53470b0ea4170b8819180b0bd7475f3642827
-2016-01-29 17:56 +0000 [2451d4e455]  gtjoseph <george.joseph@fairview5.com>
 
-	* res_pjsip:  Fix infinite recursion when loading transports from realtime
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430995 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Attempting to load a transport from realtime was forcing asterisk into an
-	  infinite recursion loop.  The first thing transport_apply did was to do a
-	  sorcery retrieve by id for an existing transport of the same name. For files,
-	  this just returns the previous object from res_sorcery_config's internal
-	  container, if any.  For realtime, the res_sourcery_realtime driver looks in the
-	  database and finds the existing row but now it has to rehydrate it into a
-	  sorcery object which means calling... transport_apply.  And so it goes.
+2015-01-22 13:30 +0000 [9bff4eeca3]  Richard Mudgett <rmudgett@digium.com>
 
-	  The main issue with loading from realtime (apart from the loop) was that
-	  transport stores structures and pointers directly in the ast_sip_transport
-	  structure instead of the separate ast_transport_state structure.  This patch
-	  separates those items into the ast_sip_transport_state structure.  The pattern
-	  is roughly the same as res_pjsip_outbound_registration.
+	* Bridge core: Pass a ref with the swap channel when joining a bridge.
 
-	  Although all current usages of ast_sip_transport and ast_sip_transport_state
-	  were modified to use the new ast_sip_get_transport_state API, the original
-	  items are left in ast_sip_transport and kept updated to maintain ABI
-	  compatability for third-party modules.  They are marked as deprecated and
-	  noted that they're now in ast_sip_transport_state.
+	  When code imparts a channel into a bridge to swap with another channel, a
+	  ref needs to be held on the swap channel to ensure that it cannot
+	  dissapear before finding it in the bridge.
 
-	  ASTERISK-25606 #close
-	  Reported-by: Martin Moučka
+	  * The ast_bridge_join() swap channel parameter now always steals a ref for
+	  the swap channel.  This is the only change to the bridge framework's
+	  public API semantics.
 
-	  Change-Id: Ic7a836ea8e786e8def51fe3f8cce855ea54f5f19
+	  * bridge_channel_internal_join() now requires the bridge_channel->swap
+	  channel to pass in a ref.
 
-2016-01-25 17:36 +0000 [6f978fbfe5]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24649
+	  Reported by: John Bigelow
 
-	* app_confbridge: Only use b_profile options from the conference.
+	  Review: https://reviewboard.asterisk.org/r/4354/
+	  ........
 
-	  A user cannot set new bridge options after the conference is created by
-	  the first user.  Attempting to do so is documented as undefined behavior.
+	  Merged revisions 430975 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch ensures that the bridge profile options used are from the
-	  conference and not what a subsequent user may have tried to set.
 
-	  Change-Id: I1b6383eba654679e5739d5a8de98199cf074a266
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430976 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-02-05 10:29 +0000 [ec8fd6714d]  gtjoseph <george.joseph@fairview5.com>
+2015-01-22 13:14 +0000 [e67ca431ee]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_misdn: Fix a few issues causing compile errors
+	* res_pjsip_outbound_registration.c: Minor code cleanup.
 
-	  Change-Id: I54b48c24d7ca88ed80496fdfd142d08772a7ab98
+	  * Add an allocation failure check and assert in
+	  sip_outbound_registration_response_cb().
 
-2016-02-04 16:17 +0000 [6a799cd78f]  Mark Michelson <mmichelson@digium.com>
+	  * Made sip_outbound_registration_state_destroy() handle partially created
+	  state objects from sip_outbound_registration_state_alloc().
 
-	* Check for OpenSSL defines before trying to use them.
+	  Review: https://reviewboard.asterisk.org/r/4366/
+	  ........
 
-	  The SSL_OP_NO_TLSv1_1 and SSL_OP_NO_TLSv1_2 defines did not exist prior
-	  to OpenSSL version 1.0.1. A recent commit attempts to, by default, set
-	  these options, which can cause problems on systems with older OpenSSL
-	  installations.
+	  Merged revisions 430957 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This commit adds a configure script check for those defines and will not
-	  attempt to make use of those if they do not exist. We will print a
-	  warning urging the user to upgrade their OpenSSL installation if those
-	  defines are not present.
 
-	  Change-Id: I6a2eb9a43fd0738b404d8f6f2cf4b5c22d9d752d
-2016-02-03 14:25 +0000 [953d1cc11a]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430958 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip/alembic:  Add missing columns to system and registration
+2015-01-22 12:10 +0000 [49f405fe4c]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  ps_systems needed disable_tcp_switch
-	  ps_registrations needed line and endpoint
+	* stasis transfer: fix a race condition on stasis bridge push
 
-	  ASTERISK-25737 #close
+	  After a bridge transfer completes where a local replacement
+	  channel is used, a stasis transfer message with the details
+	  of the transfer is sent.  This is processed by stasis which
+	  then sets the stasis app name and replaced channel snapshot
+	  on the replacement channel.
 
-	  Change-Id: Iaf9c2d69e62243d9fa53104c28c5339c47d4ac19
+	  However, since a separate thread was already started to run
+	  stasis on the new replacement channel, a race was on to see
+	  if the message processing would be completed before the app
+	  name was needed, otherwise the channel would be hung up.
+
+	  This change moves the calls used to set the stasis app name
+	  and the replace snapshot to the bridge_stasis_push function
+	  callback from the bridge transfer logic, allowing the steps
+	  to be completed earlier and more deterministically, and the
+	  race elimianted.
+
+	  NOTE: the swap channel parameter to bridge_stasis_push (and
+	  thus all bridge push callbacks) must always be present when
+	  performing a swap with another channel.
 
-2016-02-04 11:39 +0000 [23829b3253]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24649 #close
+	  Reported by: John Bigelow
+	  Review: https://reviewboard.asterisk.org/r/4341/
+	  ........
 
-	* res_stasis_device_state: Fix refcounting error.
+	  Merged revisions 430939 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Device state subscription lifetimes were governed by when the
-	  subscription was established and unsubscribed from. However, it is
-	  possible that at the time of unsubscription, there could be device state
-	  events still in flight. When those device state events occur, the device
-	  state callback could attempt to dereference a freed pointer. Crash.
 
-	  This change ensures that the lifetime of the device state subscription
-	  does not end until the underlying stasis subscription has confirmed that
-	  its final message has been sent.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430940 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I25a0f1472894c1a562252fb7129671478e25e9b2
+2015-01-22 08:23 +0000 [7fcc9ce8bc]  Gareth Palmer (License 5169)
 
-2016-01-27 10:44 +0000 [4e8e6d3922]  Sean Bright <sean.bright@gmail.com>
+	* apps/app_voicemail: Trigger MWI notification with MixMonitor m() option
 
-	* res_rtp_asterisk: Allow ICE host candidates to be overriden
+	  The MixMonitor m() option allows a recording to be pushed to a specific
+	  voicemail mailbox. If the message is delivered to the mailbox's INBOX, however,
+	  no MWI notification is currently raised.
 
-	  During ICE negotiation the IPs of the local interfaces are sent to the remote
-	  peer as host candidates. In many cases Asterisk is behind a static one-to-one
-	  NAT, so these host addresses will be internal IP addresses.
+	  This patch corrects the issue by properly calling notify_new_state from the
+	  msg_create_from_file function. This will cause MWI to be triggered if the
+	  message was placed in the mailbox's INBOX.
 
-	  To help in hiding the topology of the internal network, this patch adds the
-	  ability to override the host candidates by matching them against a
-	  user-defined list of replacements.
+	  ASTERISK-24709 #close
+	  Reported by: Gareth Palmer
+	  patches:
+	    app_voicemail-430919.patch uploaded by Gareth Palmer (License 5169)
+	  ........
 
-	  Change-Id: I1c9541af97b83a4c690c8150d19bf7202c8bff1f
+	  Merged revisions 430920 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-12-07 12:46 +0000 [c6b1b2b1c8]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 430921 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* AST-2016-003 udptl.c: Fix uninitialized values.
 
-	  Sending UDPTL packets to Asterisk with the right amount of missing
-	  sequence numbers and enough redundant 0-length IFP packets, can make
-	  Asterisk crash.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430922 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25603 #close
-	  Reported by: Walter Doekes
+2015-01-21 15:57 +0000 [38738a7316]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25742 #close
-	  Reported by: Torrey Searle
+	* res_pjsip_outbound_registration.c: Move unref to a better place.
 
-	  Change-Id: I97df8375041be986f3f266ac1946a538023a5255
-2016-02-03 12:05 +0000 [f8acadde2c]  Joshua Colp <jcolp@digium.com>
+	  Move an unconditional unref of client_state so it doesn't look like it
+	  could be used after the last ref has destroyed it.
+	  ........
 
-	* AST-2016-001 http: Provide greater control of TLS and set modern defaults.
+	  Merged revisions 430902 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change exposes the configuration of various aspects of the TLS
-	  support and sets the default to the modern standards.
 
-	  The TLS cipher is now set to the best values according to the
-	  Mozilla OpSec team, different TLS versions can now be disabled, and
-	  the cipher order can be forced to be that of the server instead of
-	  the client.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430903 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24972 #close
+2015-01-21 07:36 +0000 [5835bf7a7f]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I0a10f2883f7559af5e48dee0901251dbf30d45b8
-2015-09-28 17:07 +0000 [3c81a052c8]  Richard Mudgett <rmudgett@digium.com>
+	* channels/chan_sip: Fix registration leak during reload
 
-	* AST-2016-002 chan_sip.c: Fix retransmission timeout integer overflow.
+	  When the SIP registrations were migrated to using ao2 in what was then trunk,
+	  the explicit destruction of the registrations on module reload was removed and
+	  not replaced with an ao2 equivalent. Debugging done by Stefan Engström, the
+	  issue reporter, on ASTERISK-24673 confirmed that the reference in the
+	  registry_list container was being leaked.
 
-	  Setting the sip.conf timert1 value to a value higher than 1245 can cause
-	  an integer overflow and result in large retransmit timeout times.  These
-	  large timeout times hold system file descriptors hostage and can cause the
-	  system to run out of file descriptors.
+	  Since the purpose of cleanup_all_regs is to prep a registration for
+	  destruction, this function now calls an ao2_callback function callback with the
+	  OBJ_MULTIPLE | OBJ_NODATA | OBJ_UNLINK flags used to remove the registrations.
+	  This cleans up each registration, and also removes it from the registration
+	  container registry_list.
 
-	  NOTE: The default sip.conf timert1 value is 500 which does not expose the
-	  vulnerability.
+	  Review: https://reviewboard.asterisk.org/r/4355/
 
-	  * The overflow is now detected and the previous timeout time is
-	  calculated.
+	  ASTERISK-24640 #close
+	  Reported by: Max Man
 
-	  ASTERISK-25397 #close
-	  Reported by: Alexander Traud
+	  ASTERISK-24673 #close
+	  Reported by: Stefan Engström
+	  Tested by: Stefan Engström
+	  ........
 
-	  Change-Id: Ia7231f2f415af1cbf90b923e001b9219cff46290
-2016-02-03 14:07 +0000 [2a6ee8caeb]  gtjoseph <george.joseph@fairview5.com>
+	  Merged revisions 430864 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* logging: Remove/fix some message annoyances
 
-	  test_dlinklists doesn't need to NOTICE everyone that every macro worked.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430866 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  res_phoneprov doesn't need to VERBOSE everyone that a phoneprov extension or
-	  provider was registered.
+2015-01-21 07:27 +0000 [958a41a884]  Matt Jordan <mjordan@digium.com>
 
-	  res_odbc was missing a newline at the end of one message.
+	* AMI: Add documentation for the missing Cdr/CEL events.
 
-	  Change-Id: I6c06361518ef3711821795e535acd439782a995e
+	  This patch adds AMI event documentation for the Cdr and CEL AMI events.
 
-2016-02-02 10:52 +0000 [32fc784284]  Alexei Gradinari License #5691
+	  Note that while these events do share fields with each other and with other
+	  channel related events, they do not contain all of the fields in a standard
+	  channel snapshot, nor is the description of the fields identical. As such,
+	  the patch opts for documentation for each field, for each event.
 
-	* res_sorcery_realtime: Fix regex regression.
+	  Review: https://reviewboard.asterisk.org/r/4350/
 
-	  A regression was introduced where searching for realtime PJSIP objects
-	  by regex by starting the regex with a leading "^" would cause no items
-	  to be returned.
+	  ASTERISK-24671 #close
+	  Reported by: Dan Jenkins
+	  ........
 
-	  This was due to a change which attempted to drop the requirement for a
-	  leading "^" to be present due to how some CLI commands formulate their
-	  regexes. However, the change, rather than simply eliminating the
-	  requirement, caused any regexes that did begin with "^" to end up not
-	  returning the expected results.
+	  Merged revisions 430862 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change fixes the problem by inspecting the regex and formulating
-	  the realtime query differently depending on if it begins with "^".
 
-	  ASTERISK-25702 #close
-	  Reported by Nic Colledge
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430863 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Patches:
-	      realtime_retrieve_regex.patch submitted by Alexei Gradinari License #5691
+2015-01-21 07:12 +0000 [4740ef50f4]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I055df608a6e6a10732044fa737a9fe8dca602693
+	* apps/app_dial: Don't publish DialEnd twice on unexpected GoSub/Macro values
 
-2016-02-02 04:05 +0000 [0405c31756]  Karsten Wemheuer <kwe-digium@iptam.com>
+	  The Dial application has some interesting options with the mid-call Macro (M)
+	  and GoSub (U) options. If the MACRO_RESULT/GOSUB_RESULT returns specific
+	  values, the Dial application will take some action upon the channels involved
+	  in the dial operation (such as hanging up a particular party, etc.) The Dial
+	  application ensures that a Stasis message is published in the event that
+	  MACRO_RESULT/GOSUB_RESULT returns a value that kills the dial operation, so
+	  that there is a corresponding DialEnd event published in AMI/ARI for the
+	  DialBegin event that preceeded it.
 
-	* res_xmpp: Does not connect in component mode
+	  A bug exists where that same DialEnd event will be published on Stasis even if
+	  the value returned in MACRO_RESULT/GOSUB_RESULT is not one that the Dial
+	  application cares about. This causes two DialEnd events to be published - one
+	  with the MACRO_RESULT/GOSUB_RESULT and another with "ANSWERED" - which is all
+	  sorts of wrong.
 
-	  The module res_xmpp does not accept usernames in the form used in component
-	  mode (XEP-0114). In component mode there is no @something in the name.
-	  In component mode the connection is now not dropped anymore.
+	  This patch fixes the bug by ensuring that we only publish a DialEnd message to
+	  Stasis if the Dial application's mid-call Macro/GoSub returns something that
+	  Dial cares about.
 
-	  If the xmpp server sends out a "stream" tag before handshake is finished,
-	  the connection gets dropped in res_xmpp. Now this tag will be ignored and
-	  the connection will be established.
+	  Review: https://reviewboard.asterisk.org/r/4336
 
-	  After connecting there will be an exchange of presence states. This does
-	  not work as expected in component mode. The responsible function
-	  "xmpp_pak_presence" is left before the states get sent out. Sending
-	  presence states in component mode is now moved to the top of the function.
+	  ASTERISK-24682 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  ASTERISK-25735 #close
+	  Merged revisions 430842 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I70e036f931c3124ebb2ad1e56f93ed35cfdd9d5c
-2016-02-01 13:04 +0000 [8804d0973c]  gtjoseph <george.joseph@fairview5.com>
 
-	* build_system:  Fix some warnings highlighted by clang
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430844 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Fix some warnings found with clang.
+2015-01-21 07:06 +0000 [228fdb3f4e]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I5195b6189b148c2ee3ed4a19d015a6d4ef3e77bd
+	* main/rtp_engine: Format NTP timestamps as unsigned longs
 
-2016-02-01 13:16 +0000 [109b0aff6b]  gtjoseph <george.joseph@fairview5.com>
+	  When the RTCP reports are created, the NTP timestamps are stored as strings,
+	  as JSON does not have an integer type long enough to store the value. However,
+	  on 32-bit systems, a signed long may overflow for some portion of the
+	  timestamp.
 
-	* res/Makefile: Fix bug in "clean" target for ari
+	  This patch corrects the overflow by formatting the timestamps as unsigned
+	  longs.
+	  ........
 
-	  The "clean" target was attempting to clean res/ari from inside
-	  the res directory which doesn't remove anything.  Removed the res/
-	  prefix.
+	  Merged revisions 430840 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ib1a518d54efa81b9fd5a42742d43cc3767435bf6
 
-2016-01-31 20:13 +0000 [a85fab7c44]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430841 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip/alembic: Fix definition of qualify_timeout
+2015-01-20 11:15 +0000 [804ab70f9d]  Ashley Sanders <asanders@digium.com>
 
-	  A recent commit set qualify_timeout to Decimal which isn't supported.
-	  This path corrects it to Float.
+	* ARI: Fixed crash that occurred when updating a bridge when the optional query parameter 'name' was not supplied.
 
-	  Change-Id: I038f5274ba8cb60f8518a5845ce448d49306aadf
+	  Prior to this changeset, posting to the: /ari/bridges/{bridgeId} endpoint without specifying a value for the [name] query parameter, would crash Asterisk if the bridge you are attempting to create (or update) had the same ID as an existing bridge. The internal mechanism of the POST operation interpreted a null value for name, thus resulting in an error condition that crashed Asterisk.
 
-2016-01-29 07:39 +0000 [aa9348ab9a]  Stefan Engström <stefanen@kth.se>
+	  ASTERISK-24560 #close
+	  Reported By: Kinsey Moore
 
-	* chan_sip.c: AMI & CLI notify methods get different values of asterisk's own ip.
+	  Review: https://reviewboard.asterisk.org/r/4349/
+	  ........
 
-	  When I ask asterisk to send a SIP NOTIFY message to a sip peer using either a)
-	  AMI action: SIPnotify or b) cli command: sip notify <cmd> <peer>, I expect
-	  asterisk to include the same value for its own ip in both cases a) and b),
-	  but it seems a) produces a contact header like Contact:
-	  <sip:asterisk@192.168.1.227:8060> whereas b) produces a contact header like
-	  <sip:asterisk@127.0.0.1:8060>. 0.0.0.0:8060 is my udpbindaddr in sip.conf
+	  Merged revisions 430818 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  My guess is that manager_sipnotify should call
-	  ast_sip_ouraddrfor(&p->sa, &p->ourip, p) the same way sip_cli_notify does,
-	  because after applying this patch, both cases a) and b) produce
-	  the contact header that I expect: <sip:asterisk@192.168.1.227:8060>
 
-	  Reported by: Stefan Engström
-	  Tested by: Stefan Engström
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430820 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I86af5e209db64aab82c25417de6c768fb645f476
-2015-12-23 15:07 +0000 [65bd4fcc3f]  Mark Michelson <mmichelson@digium.com>
+2015-01-20 10:59 +0000 [e4738a59eb]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_odbc: Remove connection management
+	* CHANNEL(peer), chan_iax2, res_fax, SNMP agent: Fix deadlock from reaching across a bridge.
 
-	  Asterisk by default will create a single database connection and share
-	  it among all threads that attempt to access the database. In previous
-	  versions of Asterisk, this was tolerable, because the most used channel
-	  driver, chan_sip, mostly accessed the database from a single thread.
-	  With PJSIP, however, many threads may be attempting to perform database
-	  operations, and there is the potential for many more database accesses,
-	  meaning the concurrency is a horrible bottleneck if only one connection
-	  is shared.
+	  Calling ast_channel_bridge_peer() cannot be done while holding any channel
+	  locks.  The reported issue hit the deadlock in chan_iax2, but an audit of
+	  the ast_channel_bridge_peer() calls found three more locations where the
+	  same deadlock can occur.
 
-	  Asterisk has a connection pooling facility built into it, but the
-	  implementation has flaws. For one, there is a strict limit on the number
-	  of simultaneous connections that could be made to the database. Anything
-	  beyond the maximum would result in a failed operation. Attempting to
-	  predict what the maximum should be is nearly impossible even for someone
-	  intimately familiar with Asterisk's threading model. In addition, use of
-	  transactions in the dialplan can cause some severe bugs if connection
-	  pooling is enabled.
+	  * Made CHANNEL(peer), res_fax, and the SNMP agent not call
+	  ast_channel_bridge_peer() with any channel locked.  For CHANNEL(peer) I
+	  had to rework the logic to not hold the channel lock.
 
-	  This commit seeks to fix the concurrency problem by removing all
-	  connection management code from Asterisk and leaving that to the
-	  underlying unixODBC code instead. Now, Asterisk does not share a single
-	  connection, nor does it try to maintain a connection pool. Instead, all
-	  Asterisk ever does is request a connection from unixODBC and allow
-	  unixODBC to either allocate those connections or retrieve them from a
-	  pool.
+	  * Made chan_iax2 no longer call ast_channel_bridge_peer().  It was done
+	  for legacy reasons that no longer apply.
 
-	  Doing this has a bit of a ripple effect. For one, since connections are
-	  not long-lived objects, several of the safeguards that previously
-	  existed have been removed. We don't have to worry about trying to use a
-	  connection that has gone stale. In every case, when we request a
-	  connection, it has just been made and we don't need to perform any
-	  sanity checks to be sure it's still active.
+	  * Removed the iax.conf forcejitterbuffer option.  It is now always enabled
+	  when the jitterbuffer option is enabled.  If you put a jitter buffer on a
+	  channel it will be on the channel.
 
-	  Another major player affected by this change is transactions.
-	  Transactions and their respective connections were so tightly coupled
-	  that it was almost pornographic. This code change moves
-	  transaction-related code to its own file separate from the core ODBC
-	  functionality. This way, the core of ODBC does not even have to know
-	  that transactions exist.
+	  ASTERISK-24600 #close
+	  Reported by: Jeff Collell
 
-	  In making this large change, I had to look at a lot of code and
-	  understand it. When making this change, I discovered several places
-	  where the behavior is definitely not ideal, but it seemed outside the
-	  scope of this change to be fixing it. Instead, any place where I saw
-	  some sort of room for improvement has had a XXX comment added explaining
-	  what could be altered to improve it.
+	  Review: https://reviewboard.asterisk.org/r/4342/
+	  ........
 
-	  Change-Id: I37a84def5ea4ddf93868ce8105f39de078297fbf
+	  Merged revisions 430817 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-28 12:44 +0000 [2a9e623ff9]  Richard Mudgett <rmudgett@digium.com>
 
-	* config_options.c: Fix warning message wording.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430819 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I915ea437936320393afde0e7552cf0a980a6b2e4
+2015-01-19 20:41 +0000 [14b8e03dad]  Ben Klang (License 5876)
 
-2016-01-25 17:34 +0000 [ed3c9c1512]  Richard Mudgett <rmudgett@digium.com>
+	* contrib/scripts/install_prereq: Don't install 32-bit packages on 64-bit hosts
 
-	* app_confbridge.c: Replace inlined code with existing function.
+	  On Debian based systems, the install_prereq tool uses a search command on
+	  Debian that results in selecting both 64-bit and 32-bit packages. Besides the
+	  waste of disk space, this can actually cause aptitude use 100% of memory on a
+	  VM with 1GB of RAM as it tried to work out all of the 32-bit package
+	  dependencies.
 
-	  Change-Id: Ida5594e9f8d7c1fc18eeb733a11f8fb96326da51
+	  This patch filters out the 32-bit packages on a 64-bit machine, and leaves
+	  32-bit machines alone.
 
-2016-01-25 16:05 +0000 [1d0abf86e7]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24048 #close
+	  Reported by: Ben Klang
+	  Tested by: Ben Klang, Matt Jordan
+	  patches:
+	    install_prereq_64-bit_compat.patch uploaded by Ben Klang (License 5876)
+	  ........
 
-	* app_confbridge: Add ability to get the muted conference state.
+	  Merged revisions 430798 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * Added CONFBRIDGE_INFO(muted,) for querying the muted conference state.
+	  Merged revisions 430799 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Added Muted header to AMI ConfbridgeListRooms action response list
-	  events to indicate the muted conference state.
 
-	  * Added Muted column to CLI "confbridge list" output to indicate the muted
-	  conference state and made the locked column a yes/no value instead of a
-	  locked/unlocked value.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430800 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-20987
-	  Reported by: hristo
+2015-01-19 20:33 +0000 [112bf1597e]  LEI FU (License 6640)
 
-	  Change-Id: I4076bd8ea1c23a3afd4f5833e9291b49a0c448b1
+	* app_voicemail: Temp message left after review/hangup with ODBC/IMAP backend
 
-2016-01-26 17:59 +0000 [f0d40afa69]  Richard Mudgett <rmudgett@digium.com>
+	  When using ODBC or IMAP storage, temporary files created on the file system
+	  must be disposed of using the DISPOSE macro. The DELETE macro will map to a
+	  deletion function for the backend storage, but does not clean up any local
+	  files created as a result of the operation.
 
-	* app_confbridge.c: Update CONFBRIDGE and CONFBRIDGE_INFO documentation.
+	  When using voicemail with the operator and review options enabled, pressing
+	  0 to enter the menu, followed by 1 to save the message, followed by any
+	  other DTMF press to delete the message, will result in the temporary file
+	  lingering on the file system.
 
-	  Change-Id: Ic1f9e22ba1f2ff3b3f5cb017c5ddcd9bd48eccc7
+	  This patch properly calls DISPOSE after the DELETE. This causes the local
+	  file to be disposed of.
 
-2016-01-25 15:48 +0000 [3e51e5c7fd]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24288 #close
+	  Reported by: LEI FU
+	  patches:
+	    voicemail_odbc_review_fix.diff uploaded by LEI FU (License 6640)
+	  ........
 
-	* app_confbridge: Make non-admin users join a muted conference muted.
+	  Merged revisions 430795 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-20987 #close
-	  Reported by: hristo
+	  Merged revisions 430796 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ic61a2b524ab3a4cfadf227fc6b3506527bc03f38
 
-2016-01-27 13:02 +0000 [9da18af992]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430797 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip:  Add res_pjproject dependency to UPGRADE.txt and samples
+2015-01-19 12:15 +0000 [7dc784ffa9]  Mark Michelson <mmichelson@digium.com>
 
-	  Since res_pjsip now depends on res_pjproject, this is now mentioned
-	  in UPGRADE.txt and the basic-pbx modules.conf has been updated.
+	* Call extension state callbacks at hint creation.
 
-	  Change-Id: I42826597d5e10f08e518208860c44c96e52f1b2d
-2016-01-27 10:29 +0000 [aee8448bc2]  gtjoseph <george.joseph@fairview5.com>
+	  When a hint gets created, any subsequent device or presence
+	  state changes result in extension status events getting sent
+	  out to interested parties. However, at the time of hint creation,
+	  no such event gets sent out, so watchers of extension state are
+	  potentially left in the dark until the first state change after
+	  hint creation.
 
-	* build_system: Prevent goals needing makeopts from running when it's missing
+	  Patch contributed by John Hardin (License #6512)
+	  ........
 
-	  The Makefile only optionally includes makeopts so when goals like uninstall that
-	  dont depend on anything else are run after a distclean, rules like
-	  'rm -f "$(DESTDIR)$(ASTMODDIR)/"*' get run as 'rm -f ""/*' which attempts
-	  to remove everything in the root directory.
+	  Merged revisions 430776 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Although there's a rule defined for makeopts which prints a message and does
-	  an 'exit 1', since '-include makepopts' was specified (with the -), the exit
-	  was ignored letting the rest of the rules run.
 
-	  This patch makes makeopts required unless the goal has the string 'clean' in it.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430777 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25730 #close
-	  Reported-by: George Joseph
+2015-01-19 07:19 +0000 [e43912f3f3]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I1bce59a7ea4f48e7a468e22b2abbb13c63417ac7
+	* res_pjsip / res_pjsip_multihomed: Use the correct transport and addressing information on UAS sessions.
 
-2016-01-25 09:35 +0000 [f22074e5d9]  Joshua Colp <jcolp@digium.com>
+	  The first thing this patch fixes is UAS dialogs. Previously if a transport was
+	  configured on an endpoint and an inbound session was created there was no guarantee
+	  that requests sent on the dialog would use the correct transport and address
+	  information. This has now been fixed so an explicitly configured transport
+	  is taken into account.
 
-	* config: Allow options to register when documentation is unavailable.
+	  The second thing this patch fixes is res_pjsip_multihomed. The res_pjsip_multihomed
+	  module attempts to determine what transport a message should go out on and what
+	  addressing information should go into the message itself. In a scenario where
+	  multiple transports exist bound to the same IP address but a different port the
+	  code would incorrectly alter the transport and change the message to the wrong
+	  transport. This change makes the res_pjsip_multihomed module smarter so it will
+	  only change the transport and address information in the message when it is
+	  possible and makes sense.
 
-	  The config options framework is strict in that configuration options must
-	  be documented unless XML documentation support is not available. In
-	  practice this is useful as it ensures documentation exists however in
-	  off-nominal cases this can cause strange problems.
+	  ASTERISK-24615 #close
+	  Reported by: David Justl
 
-	  If it is expected that a config option has a non-zero or non-empty
-	  default value but the config option documentation is unavailable
-	  this reasonable expectation will not be met. This can cause obscure
-	  crashes and weirdness depending on how the code handles it.
+	  Review: https://reviewboard.asterisk.org/r/4331/
+	  ........
 
-	  This change tweaks the behavior to ensure that the config option
-	  is still allowed to register, apply default values, and be set when
-	  devmode is not enabled. If devmode is enabled then the option can
-	  NOT be set.
+	  Merged revisions 430755 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This also does not remove the initial documentation error message that
-	  is output on load when registering the configuration option.
 
-	  ASTERISK-25725 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430756 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iec42fca6b35f31326c33fcdc25473f6fd7bc8af8
+2015-01-16 18:35 +0000 [07e2a48ab1]  Kevin Harwell <kharwell@digium.com>
 
-2016-01-25 10:23 +0000 [4a3275abb9]  Mark Michelson <mmichelson@digium.com>
+	* REVERTING res_pjsip: make it unloadable
 
-	* Stasis: Use custom structure when setting variables.
+	  Due to the original patch causing memory corruptions the patch is
+	  being removed until the problem can be resolved.
+	  ........
 
-	  A recent change to queue channel variable setting to the Stasis control
-	  queue caused a regression. When setting channel variables, it is
-	  possible to give a NULL channel variable value in order to unset the
-	  variable (i.e. remove it from the channel variable list). The change
-	  introduced a call to ast_variable_new(), which is not tolerant of NULL
-	  channel variable values.
+	  Merged revisions 430734 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This new change switches from using ast_variable to using a custom
-	  channel variable struct that is lighter weight and NULL value-tolerant.
 
-	  Change-Id: I784d7beaaa3c036ea936d103e7caf0bb1562162d
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430735 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-25 16:56 +0000 [b2c8a99f9e]  Rusty Newton <rnewton@digium.com>
+2015-01-16 16:14 +0000 [1111944afb]  Mark Michelson <mmichelson@digium.com>
 
-	* sounds/Makefile: Incremented core and extra sounds versions to 1.5
+	* Change PJProject version requirement for ca_list_path transport option in CHANGES file.
+	  ........
 
-	  Core and extra sounds 1.5 was recently released! The tarballs contain
-	  change descriptions however I figure more people will see this one so
-	  I'll try to be a bit detailed. Approximately 60 sounds were moved from Extra
-	  to Core for en, en_GB, fr and added for languages that didn't already
-	  have Extra sound sets (it,ja,ru).
+	  Merged revisions 430716 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  In addition all of the English and Russian sounds have been completely
-	  re-recorded.
 
-	  Sounds moved and added:
-	  activated,added,all-circuits-busy-now,astcc-followed-by-pound
-	  at-tone-time-exactly,call-forwarding,call-fwd-no-ans,call-fwd-on-busy
-	  ,call-fwd-unconditional,calling,call-waiting,cancelled,
-	  cannot-complete-as-dialed,check-number-dial-again,conf-full,de-activated
-	  ,disabled,do-not-disturb,enabled,enter-num-blacklist,entr-num-rmv-blklist
-	  ,extension,feature-not-avail-line,for,from-unknown-caller,goodbye,hello
-	  ,if-correct-press,im-sorry,info-about-last-call,is,is-in-use,is-set-to
-	  ,location,number,number-not-answering,num-was-successfully,one-moment-please
-	  ,please-try-again,pls-hold-while-try,pls-try-call-later,pm-invalid-option
-	  ,privacy-to-blacklist-last-caller,removed,simul-call-limit-reached
-	  ,something-terribly-wrong,sorry,sorry-youre-having-problems,speed-dial
-	  ,speed-dial-empty,telephone-number,time,to-call-this-number,to-extension
-	  ,to-listen-to-it,to-rerecord-it,unidentified-no-callback,with,you-entered
-	  ,your
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430717 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  There were also a few random fixes here and there to file names for a few
-	  of the languages.
+2015-01-16 16:13 +0000 [831acba826]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-25068 #close
+	* Fix problem where a hung channel could occur on a failed blind transfer.
 
-	  Change-Id: I2b594344ec585d7dfd922b40c1af43b1508828b3
-2016-01-25 16:51 +0000 [8261bda1bf]  Mark Michelson <mmichelson@digium.com>
+	  Different clients react differently to being told that a blind transfer
+	  has failed. Some will simply send a BYE and be done with it. Others will
+	  attempt to reinvite themselves back onto the call.
 
-	* res_pjsip_pubsub: Prevent crash from AMI command on freed subscription.
+	  In the latter case, we were creating a new channel and then leaving it to
+	  sit forever doing nothing. With this code change, that new channel will
+	  not be created and the dialog with the transferring channel will be cleaned
+	  up properly.
 
-	  A test recently uncovered that running an ill-timed AMI command to show
-	  inbound subscriptions could cause a crash since Asterisk will try to
-	  operate on a freed subscription.
+	  ASTERISK-24624 #close
+	  Reported by Zane Conkle
 
-	  The fix for this is to remove the subscription tree from the list of
-	  subscriptions at the time that we are sending our final NOTIFY request
-	  out. This way, as the subscription is in the process of dying, it is
-	  inaccessible from AMI.
+	  Review: https://reviewboard.asterisk.org/r/4339
+	  ........
 
-	  Change-Id: Ic0239003d8d73e04c47c12dd2a7e23867e5b5b23
+	  Merged revisions 430714 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-25 11:03 +0000 [a6823bb0c4]  Corey Farrell <git@cfware.com>
 
-	* chan_sip: Fix buffer overrun in sip_sipredirect.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430715 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  sip_sipredirect uses sscanf to copy up to 256 characters to a stacked buffer
-	  of 256 characters.  This patch reduces the copy to 255 characters to leave
-	  room for the string null terminator.
+2015-01-16 15:46 +0000 [023fa0f9e8]  cloos <cloos@jhcloos.com> (License #5956)
 
-	  ASTERISK-25722 #close
+	* Add support for the ca_list_path option for PJSIP transports.
 
-	  Change-Id: Id6c3a629a609e94153287512c59aa1923e8a03ab
+	  This allows for a path to be specified that has a collection of CA
+	  certificates in it.
 
-2016-01-22 15:08 +0000 [1003c2eb05]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24575 #close
+	  Reported by cloos
+	  Patches:
+	  	pj-ca-path-trunk.diff uploaded by cloos (License #5956)
 
-	* Stasis: Fix potential memory leak of control data.
+	  Review: https://reviewboard.asterisk.org/r/4344
+	  ........
 
-	  When queuing tasks onto the Stasis control queue, you can pass an
-	  arbitrary data pointer and a function to free that data. All ARI
-	  commands that use the Stasis control queue made the assumption that the
-	  destructor function would be called in all paths, whether the task was
-	  queued successfully or not. However, this was not correct. If a task was
-	  queued onto a control structure that was already completed, the
-	  allocated data would not be freed properly.
+	  Merged revisions 430709 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch corrects this by making sure that all return paths call the
-	  data destructor.
 
-	  Change-Id: Ibf06522094f8e5c4cce652537dc5d7222b1c4fcb
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430713 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-21 10:58 +0000 [eedd77fda0]  Mark Michelson <mmichelson@digium.com>
+2015-01-15 11:36 +0000 [a8ea2f9287]  Richard Mudgett <rmudgett@digium.com>
 
-	* Stasis: Use control queue to prevent crash.
+	* res_fax.c, res_fax_spandsp.c: Remove redundant locking.
 
-	  A crash occurred when attempting to set a channel variable on a channel
-	  that had already been hung up. This is because there is a small window
-	  between when a control is grabbed and when the channel variable is set
-	  that the channel can be hung up.
+	  When FAX was developed, apparently the faxregistry.container used to be a
+	  linked list that was converted to an ao2 container.  Some of the
+	  replacement ao2 container operations still had explicit lock/unlocks
+	  around them.
 
-	  The fix here is to queue the setting of the channel variable onto the
-	  control queue. This way, the manipulation of the channel happens in a
-	  thread where it is safe to be done.
+	  Three off nominal code paths in res_fax.c and res_fax_spandsp.c unlock the
+	  channel even though the routine did not lock the channel and other code
+	  paths in the routine do not unlock the channel.
 
-	  In this change, I also noticed that the setting of bridge roles on
-	  channels was being done outside of the control queue, so I also changed
-	  those operations to be done in the control queue.
+	  Review: https://reviewboard.asterisk.org/r/4340/
+	  ........
 
-	  ASTERISK-25709 #close
-	  Reported by Mark Michelson
+	  Merged revisions 430687 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2a0a4d51bce6fba6f1d9954e40935e42f366ea78
 
-2016-01-22 11:48 +0000 [1c95b211a0]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430688 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* logger.c: Fix buffer overrun found by address sanitizer.
+2015-01-15 11:28 +0000 [9b1c36d3fa]  Richard Mudgett <rmudgett@digium.com>
 
-	  The null terminator of the tail struct member was not being allocated
-	  when no logger.conf config file is installed.
+	* res_fax.c, res_fax_spandsp.c: Fix some curlies on the end of function definitions.
+	  ........
 
-	  ASTERISK-25714 #close
-	  Reported by: Badalian Vyacheslav
+	  Merged revisions 430685 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I45770fdd08af39506a3bc33ba279c4f16e047a30
 
-2016-01-21 16:40 +0000 [6ff945ab87]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430686 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Build System: Add support for checking alembic branches.
+2015-01-15 06:10 +0000 [1e605d950b]  Joshua Colp <jcolp@digium.com>
 
-	  * Add 'check-alembic' target to root Makefile.
-	  * Create build_tools/make_check_alembic to do the actual checks.
+	* res_pjsip_outbound_registration: Fix race condition when reloading and listing registrations.
 
-	  ASTERISK-25685
+	  Due to the split of outbound registration state from configuration it is possible during
+	  a reload for a "pjsip show registrations" CLI command to be executed which gets an older
+	  snapshot of the configuration. This configuration may include outbound registrations which
+	  have been removed due to a reload operation occurring at the same time. The code for
+	  printing the outbound registration did not take this into account but now it does.
 
-	  Change-Id: Ibb3cae7d1202ac23dc70b0f3b5801571ad46b004
+	  AST-1506 #close
 
-2016-01-19 18:20 +0000 [02035212de]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4338/
+	  ........
 
-	* res/res_pjsip/presence_xml.c: Add missing 2nd call presence state case.
+	  Merged revisions 430664 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25712 #close
-	  Reported by: Richard Mudgett
 
-	  Change-Id: I70634df24f8c6c3a2c66c45af61d021e4999253f
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430665 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-18 03:49 +0000 [c68c66c61f]  Diederik de Groot <ddegroot@talon.nl>
+2015-01-14 20:19 +0000 [f11fb76205]  abelbeck <lonnie@abelbeck.com> (License 5903)
 
-	* main/asterisk.c: ast_el_read_char
+	* configure: If cross-compiling, assume we have working semaphores
 
-	  Make sure buf[res] is not accessed at res=-1 (buffer underrun).
-	  Address Sanitizer will complain about this quite loudly.
+	  The Asterisk 13 configure.ac checks for HAS_WORKING_SEMAPHORE but does not have
+	  an option for cross-compiling so it fails with an exit. Since we're cross-
+	  compiling, we can't exactly go looking for the header. The semaphore.h header
+	  is relatively common:
+	  * It's part of the POSIX standard
+	  * It's part of GNU C Library
+	  As such, we assume that it will be present when cross-compiling.
 
-	  ASTERISK-24801 #close
+	  As such, this patch defaults "HAS_WORKING_SEMAPHORE" to "1" if cross-compiling
+	  is detected.
 
-	  Change-Id: Ifcd7f691310815a31756b76067c56fba299d3ae9
+	  If you're cross-compiling to a platform that doesn't support this, then make
+	  sure you re-define this to 0.
 
-2016-01-13 16:49 +0000 [f87c3275cc]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24663 #close
+	  Reported by: abelbeck
+	  patches:
+	    asterisk-13-anonymous-semaphores.patch uploaded by abelbeck (License 5903)
+	  ........
 
-	* res_pjsip: Add CLI "pjsip dump endpt [details]"
+	  Merged revisions 430646 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Dump the res_pjsip endpt internals.
 
-	  In non-developer mode we will not document or make easily accessible the
-	  "details" option even though it is still available.  The user has to know
-	  it exists to use it.  Presumably they would also be aware of the potential
-	  crash warning below.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430647 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Warning: PJPROJECT documents that the function used by this CLI command
-	  may cause a crash when asking for details because it tries to access all
-	  active memory pools.
+2015-01-14 17:15 +0000 [49542a794b]  Corey Farrell <git@cfware.com> (license 5909)
 
-	  Change-Id: If2d98a3641c9873364d1daaad971376311aef3cb
+	* res_pjsip: make it unloadable
 
-2016-01-18 17:16 +0000 [46b2de55f9]  Matt Jordan <mjordan@digium.com>
+	  The res_pjsip module was previously unloadable. With this patch it can now
+	  be unloaded.
 
-	* funcs/func_cdr: Correctly report high precision values for duration and billsec
+	  This patch is based off the original patch on the issue (listed below) by Corey
+	  Farrell with a few modifications. Namely, removed a few changes not required to
+	  make the module unloadable and also fixed a bug that would cause asterisk to
+	  crash on unloading.
 
-	  When CDRs were refactored, func_cdr's ability to report high precision values
-	  for duration and billsec (the 'f' option) was broken. This was due to func_cdr
-	  incorrectly interpreting the duration/billsec values provided by the CDR engine
-	  in milliseconds, as opposed to seconds. Since the CDR engine only provides
-	  duration and billsec in seconds, and does not expose either attribute with
-	  sufficient precision to merely pass back the underlying value, this patch fixes
-	  the bug by re-calculating duration and billsec with microsecond precision based
-	  on the start/answer/end times on the CDR.
+	  This patch is the first step (should hopefully be followed by another/others at
+	  some point) in allowing res_pjsip and the modules that depend on it to be
+	  unloadable. At this time, res_pjsip and some of the modules that depend on
+	  res_pjsip cannot be unloaded without causing problems of some sort.
 
-	  ASTERISK-25179 #close
+	  The goal of this patch is to get res_pjsip and only res_pjsip to be able to
+	  unload successfully and/or shutdown without incident (crashes, leaks, etc...).
+	  Other dependent modules may still cause problems on unload.
 
-	  Change-Id: I8bc63822b496537a5bf80baf6102c06206bee841
+	  Basically made sure, with the patch applied, that res_pjsip (with no other
+	  dependent modules loaded) could be succesfully unloaded and Asterisk could
+	  shutdown without any leaks or crashes that pertained directly to res_pjsip.
 
-2016-01-18 19:20 +0000 [137fe5ae01]  gtjoseph <george.joseph@fairview5.com>
+	  ASTERISK-24485 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4311/
+	  patches:
+	    pjsip_unload-broken-r1.patch submitted by Corey Farrell (license 5909)
+	  ........
 
-	* res_pjproject:  Add module providing pjproject logging and utils
+	  Merged revisions 430628 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  res_pjsip_log_forwarder has been renamed to res_pjproject
-	  and enhanced as follows:
 
-	  As a follow-on to the recent 'Add CLI "pjsip show buildopts"' patch,
-	  a new ast_pjproject_get_buildopt function has been added.  It
-	  allows the caller to get the value of one of the buildopts.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430629 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The initial use case is retrieving the runtime value of
-	  PJ_MAX_HOSTNAME to insure we don't send a hostname greater
-	  than pjproject can handle.  Since it can differ between
-	  the version of pjproject that Asterisk was compiled against
-	  and the version of pjproject that Asterisk is running against,
-	  we can't use the PJ_MAX_HOSTNAME macro directly in Asterisk
-	  source code.
+2015-01-14 14:39 +0000 [67234b3ee2]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: Iab6e82fec3d7cf00c1cf6185c42be3e7569dee1e
+	* Prevent slow graceful shutdown when outbound publications never started.
 
-2016-01-19 17:15 +0000 [b5c13c1545]  Joshua Colp <jcolp@digium.com>
+	  The code was missing the case for explicitly destroying an outbound publication
+	  when Asterisk had never actually published anything. The result was that Asterisk
+	  would hang for a while on a graceful shutdown.
 
-	* test_threadpool: Wait for each task to complete and fix memory leak.
+	  With this change, the case is taken into account, and on a graceful shutdown, these
+	  publications are destroyed without the need to actually send a PUBLISH request.
 
-	  This change makes the thread_timeout_thrash unit test wait for
-	  each task to complete. This fixes the problem where the test would
-	  prematurely end when all threads were gone and a new one had to be
-	  started to handle the last task. It also increases the thrasing as
-	  it is now more likely for each task to encounter the above scenario.
+	  ASTERISK-24655 #close
+	  Reported by Kevin Harwell
 
-	  This also fixes a memory leak where the data for each task was not
-	  being freed.
+	  Review: https://reviewboard.asterisk.org/r/4325
+	  ........
 
-	  ASTERISK-25611 #close
+	  Merged revisions 430608 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I5017d621a4dc911f509074c16229b86bff2fb3c6
 
-2016-01-18 19:44 +0000 [0ab89182d9]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430609 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* taskprocessor.c: Increase CLI "core ping taskprocessor" timeout.
+2015-01-14 09:40 +0000 [3eec8e4c44]  Diederik de Groot (License 6600)
 
-	  Change-Id: I4892d6acbb580d6c207d006341eaf5e0f8f2a029
+	* build_tools/mkpkgconfig: Fix Cflags concatenation error in asterisk.pc
 
-2016-01-18 19:43 +0000 [a2a8ea3330]  Richard Mudgett <rmudgett@digium.com>
+	  The mkpkgconfig script incorrectly concatenates Cflags options together. As an
+	  example, the following:
+	  Cflags: -I/usr/include/libxml2 -g3
 
-	* taskprocessor.c: Fix some taskprocessor unrefs.
+	  Is instead generated as:
+	  Cflags: -I/usr/include/libxml2-g3
 
-	  You have to call ast_taskprocessor_unref() outside of the taskprocessor
-	  implementation code.  Taskprocessor use since v12 has become more
-	  transient than just the singleton uses in earlier versions.
+	  This patch corrects the generation of Cflags in mkpkgconfig such that the
+	  Cflags options are output correctly.
 
-	  Change-Id: If7675299924c0cc65f2a43a85254e6f06f2d61bb
+	  Review: https://reviewboard.asterisk.org/r/3707/
 
-2016-01-19 13:44 +0000 [d604a9afc8]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-23991 #close
+	  Reported by: Diederik de Groot
+	  patches:
+	    fix_mkpkgconfig.diff uploaded by Diederik de Groot (License 6600)
+	  ........
 
-	* Fix alembic branches on v13.
+	  Merged revisions 430589 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I313449b609ede18ad1e1763a655dd23b9210a8e0
+	  Merged revisions 430590 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-18 18:45 +0000 [a0c79f3a4f]  gtjoseph <george.joseph@fairview5.com>
 
-	* pjsip_loging_refactor: Rename res_pjsip_log_forwarder to res_pjproject
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430591 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I5387821f29e5caa0cba0b7d62b0fc0d341e7e20b
+2015-01-13 12:17 +0000 [1780de95e4]  Richard Mudgett <rmudgett@digium.com>
 
-2016-01-14 09:26 +0000 [018ccf680b]  Rusty Newton <rnewton@digium.com>
+	* app_macro: Don't restore the calling location on a channel redirect.
 
-	* func_channel: Add help text for undocumented CHANNEL function arguments
+	  v11: If a channel redirect to a macro exten of a macro that is active
+	  happens, the redirect location doesn't get executed.  Instead the original
+	  macro location is restored and gets reexecuted.
 
-	  Adding help text documentation for:
-	  * hangupsource
-	  * appname
-	  * appdata
-	  * exten
-	  * context
-	  * channame
-	  * uniqueid
-	  * linkedid
+	  v13: An additional effect happens if a parked call times out to an
+	  extension in the macro that parked the call then the macro is reexecuted
+	  instead of the expected park return location.
 
-	  ASTERISK-24097 #close
-	  Reported by: Steven T. Wheeler
-	  Tested by: Rusty Newton
+	  * Made not restore the macro calling location on an
+	  AST_SOFTHANGUP_ASYNCGOTO.
 
-	  Change-Id: Ib94b00568b0433987df87d5b67ea529b5905754d
+	  * Increased the locked channel range when setting up the macro execution
+	  environment to cover things that should be done while the channel is
+	  locked.
 
-2016-01-16 13:18 +0000 [5644bca9f9]  Daniel Journo <dan@keshercommunications.com>
+	  * Removed unnecessary NULL tests before calling ast_free() in
+	  _macro_exec().
 
-	* Update version number in features.conf.sample
+	  ASTERISK-23850 #close
+	  Reported by: Andrew Nagy
 
-	  Update the version number in the comments from Asterisk 12 to Asterisk 12+
+	  Review: https://reviewboard.asterisk.org/r/4292/
+	  ........
 
-	  Change-Id: Ie692ac8cda3c993c3bf10f27f51a1cca3317ec7b
+	  Merged revisions 430564 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2016-01-15 19:52 +0000 [3f5f30cf82]  Corey Farrell <git@cfware.com>
+	  Merged revisions 430565 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/config: Clean config maps on shutdown.
 
-	  ASTERISK-25700 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430567 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I096da84f9c62c6095f68bcf98eac4b7c7868e808
+2015-01-13 06:09 +0000 [0e631a541d]  Joshua Colp <jcolp@digium.com>
 
-2016-01-14 14:42 +0000 [660fedecb7]  Kevin Harwell <kharwell@digium.com>
+	* chan_pjsip: Add configure check for 'pjsip_get_dest_info' function.
 
-	* bridge_basic: don't cache xferfailsound during an attended transfer
+	  The 'pjsip_get_dest_info' function is used to determine if the signaling transport
+	  of the dialog is secure or not. This function was added in PJSIP 2.3 and does not
+	  exist in earlier versions.
 
-	  The xferfailsound was read from the channel at the beginning of the transfer,
-	  and that value is "cached" for the duration of the transfer. Therefore, changing
-	  the xferfailsound on the channel using the FEATURE() dialplan function does
-	  nothing once the transfer is under way.
+	  This configure check allows Asterisk to build and run with older versions at the
+	  loss of the 'secure' argument for the PJSIP CHANNEL dialplan function. Usage of
+	  this argument will require upgrading to PJSIP 2.3.
 
-	  This makes it so the transfer code instead gets the xferfailsound configuration
-	  options from the channel when it is actually going to be used.
+	  ASTERISK-24665 #close
+	  Reported by: Mark Michelson
 
-	  This patch also fixes a potential memory leak of the props object as well as
-	  making sure the condition variable gets initialized before being destroyed.
+	  Review: https://reviewboard.asterisk.org/r/4329/
+	  ........
 
-	  ASTERISK-25696 #close
+	  Merged revisions 430546 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ic726b0f54ef588bd9c9c67f4b0e4d787934f85e4
 
-2015-07-10 10:37 +0000 [9cda1de34d]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430547 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* taskprocessor.c: Simplify ast_taskprocessor_get() return code.
+2015-01-12 13:13 +0000 [4dd6b6ff59]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Id5bd18ef1f60ef8be453e677e98478298358a9d1
+	* AMI: Revert non-backwards compatible changes from earlier commit.
 
-2016-01-13 18:20 +0000 [a79af2b312]  Richard Mudgett <rmudgett@digium.com>
+	  * Reverted the change to astman_send_listack() to not use the listflag
+	  parameter and always set the value to "Start" so the start capitalization
+	  is consistent.  Unfortunately changing the case of a returned value is not
+	  a backward compatible change so for now FAXSessions is going to have to
+	  remain inconsistent with all of the other AMI list actions.
 
-	* astmm.c: Add more stats to CLI "memory show" commands.
+	  * Reverted the minor protocol error fix in action_getconfig() when no
+	  requested categories are found.  Each line needs to be formatted as
+	  "Header: text".
 
-	  * Add freed regions totals to allocations and summary.
+	  Caught by the testsuite.
 
-	  * Add totals for all allocations and not just the selected allocations.
+	  ASTERISK-24049
+	  ........
 
-	  Change-Id: I61d5a5112617b0733097f2545a3006a344b4032a
+	  Merged revisions 430528 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-14 16:00 +0000 [83feb7db3b]  Kevin Harwell <kharwell@digium.com>
 
-	* bridge_basic: don't play an attended transfer fail sound after target hangs up
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430529 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  If the attended transfer destination answers (picks call up or goes to
-	  voicemail) and then hangs up on the transferer then transferer hears the
-	  fail sound.
+2015-01-12 12:28 +0000 [aa7e06f797]  Niklas Larsson (License 5068)
 
-	  This patch makes it so the fail sound is not played when the transfer
-	  destination/target hangs up after answering.
+	* configs/samples/features.conf.sample: Document attended transfer DTMF options
 
-	  ASTERISK-25697 #close
+	  The sample config was missing the configuration options for DTMF attended
+	  transfer completion scenarios. The configuration options 'atxferabort',
+	  'atxfercomplete', 'atxferthreeway', and 'atxferswap' are now documented in the
+	  appropriate configuration file.
 
-	  Change-Id: I97f142fe4fc2805d1a24b7c16143069dc03d9ded
+	  ASTERISK-24678 #close
+	  Reported by: Niklas Larsson
+	  patches:
+	    features.conf.sample.diff uploaded by Niklas Larsson (License 5068)
+	  ........
 
-2016-01-14 13:22 +0000 [935d641f3b]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 430526 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Remove res/ari/* content during 'make clean'.
 
-	  'make clean' and 'make distclean' can leave behind .o files in the
-	  res/ari/ directory. One observed consequence of this is that running
-	  Asterisk with MALLOC_DEBUG can cause Asterisk to crash immediately on
-	  startup sometimes.
-
-	  By ensuring that we are making a clean build, we can be sure that stale
-	  files are not being included in the build and causing problems when
-	  build options should have caused files to be re-built.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430527 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25683 #close
-	  Reported by yaron nahum
+2015-01-12 12:09 +0000 [c7ea108e02]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I1f48baa904d2468eddeefb42ee68a56af7adc7b7
+	* Revert -r430452 It needs to be redone for the next major AMI version change instead.
 
-2016-01-13 15:58 +0000 [46f21df302]  Daniel Journo <dan@keshercommunications.com>
+	  ASTERISK-24049
 
-	* pjsip/alembic:  Fix qualify_timeout column definition
 
-	  Corrects the qualify_timeout column type from Integer to Decimal
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430509 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25686 #close
-	  Reported-by: Marcelo Terres
+2015-01-12 12:01 +0000 [9065488ddd]  Michael L. Young (license 5026)
 
-	  Change-Id: I757d0e3c011ee9be6cd5abd48bc92441a405d3c8
+	* main/syslog: Allow dynamic logs, such as security events, to log to the syslog
 
-2016-01-12 11:14 +0000 [32b29d7b02]  Joshua Colp <jcolp@digium.com>
+	  The security event log uses a dynamic log level (SECURITY) that is registered
+	  with the Asterisk logging core. Unfortunately, the syslog would ignore log
+	  statements that had a dynamic log level associated with them. Because the
+	  syslog cannot handle ad hoc dynamic log levels, this patch treats any dynamic
+	  log entries sent to the syslog as logs with a level of NOTICE.
 
-	* app: Queue hangup if channel is hung up during sub or macro execution.
+	  ASTERISK-20744 #close
+	  Reported by: Michael Keuter
+	  Tested by: Michael L. Young, Jacek Konieczny
+	  patches:
+	    asterisk-20744-syslog-dynamic-logging_trunk.diff uploaded by Michael L. Young (license 5026)
+	  ........
 
-	  This issue was exposed when executing a connected line subroutine.
-	  When connected or redirected subroutines or macros are executed it is
-	  expected that the underlying applications and logic invoked are fast
-	  and do not consume frames. In practice this constraint is not enforced
-	  and if not adhered to will cause channels to continue when they shouldn't.
-	  This is because each caller of the connected or redirected logic does not
-	  check whether the channel has been hung up on return. As a result the
-	  the hung up channel continues.
+	  Merged revisions 430506 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This change makes it so when the API to execute a subroutine or
-	  macro is invoked the channel is checked to determine if it has hung up.
-	  If it has then a hangup is queued again so the caller will see it
-	  and stop.
+	  Merged revisions 430507 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25690 #close
 
-	  Change-Id: I1f9a8ceb1487df0389f0d346ce0f6dcbcaf476ea
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430508 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-13 07:20 +0000 [e7cfda0b38]  Sean Bright <sean.bright@gmail.com>
+2015-01-12 09:18 +0000 [b38acbce6e]  Kristian Hogh (License 6639)
 
-	* res_musiconhold: Prevent multiple simultaneous reloads.
+	* funcs/func_curl: Fix memory leak when CURLOPT channel datastore is destroyed
 
-	  There are two ways in which the reload() function in res_musiconhold can be
-	  called from the CLI:
+	  When the channel datastore associated with the usage of CURLOPT on a specific
+	  channel is freed, the underlying structure holding the list of options is not
+	  disposed of. This patch properly frees the structure in the datastore .destroy
+	  callback.
 
-	    * module reload res_musiconhold.so
-	    * moh reload
+	  ASTERISK-24672 #close
+	  Reported by: Kristian Hogh
+	  patches:
+	    func_curl-memory-leak.diff uploaded by Kristian Hogh (License 6639)
+	  ........
 
-	  In the former case, the module loader holds a lock that prevents multiple
-	  concurrent calls, but in the latter there is no such protection.
+	  Merged revisions 430487 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch changes the 'moh reload' CLI command to invoke the module loader
-	  directly, rather than call reload() explicitly.
+	  Merged revisions 430488 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25687 #close
 
-	  Change-Id: I408968b4c8932864411b7f9ad88cfdc7b9ba711c
-2016-01-12 14:25 +0000 [5586abc957]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430489 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_log_forwarder.c: Add CLI "pjsip show buildopts".
+2015-01-09 16:09 +0000 [fba836cc02]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  PJPROJECT has a function available to dump the compile time
-	  options used when building the library.
+	* sip_to_pjsip: improve ability to parse input files
 
-	  * Add CLI "pjsip show buildopts" command.
+	  General improvements to SIP to PJSIP conversion utility:
 
-	  * Update contrib/scripts/autosupport to get pjproject information.
+	  1) track default section of input file to allow parsing
+	     an include file that doesn't specify a [section]
 
-	  Change-Id: Id93a6a916d765b2a2e5a1aeb54caaf83206be748
+	  2) informatively handle case of assignment without [section]
 
-2016-01-12 10:36 +0000 [4cd58c3b20]  Mark Michelson <mmichelson@digium.com>
+	  3) correctly handle getting sections from included files
+	     - [section]'s are inherited by included file
 
-	* res_sorcery_realtime: Remove leading ^ requirement.
+	  4) provide null string as default transport bind ip
 
-	  res_sorcery_realtime's search-by-regex callback performed a check to
-	  ensure that the passed-in regex began with a caret (^). If it did not,
-	  then no results would be returned.
+	  5) gracefully handle missing portions of registration string
 
-	  This callback only started to become used when "like" support was added
-	  to PJSIP CLI commands. The CLI command for listing objects would pass an
-	  empty regex ("") to the sorcery backend if no "like" statement was
-	  present. For most sorcery backends, this resulted in returning all
-	  objects. However, for realtime, this resulted in returning no objects.
+	  6) denote steps of operation during conversion and confirm
+	     top level files as a convenience
 
-	  This commit seeks to fix the regression by removing the requirement from
-	  res_sorcery_realtime for the passed-in-regex to begin with a caret.
+	  ASTERISK-24474 #close
+	  Review: https://reviewboard.asterisk.org/r/4280/
+	  Reported by: John Kiniston
+	  ........
 
-	  ASTERISK-25689 #close
-	  Reported by Marcelo Terres
+	  Merged revisions 430469 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I22b4dc5d7f3f11bb29ac2e42ef94682e9bab3b20
 
-2016-01-07 11:57 +0000 [219c204a41]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430470 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip_sdp_rtp:  Add option endpoint/bind_rtp_to_media_address
+2015-01-09 15:45 +0000 [5b30938394]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  On a system with multiple ip addresses in the same subnet, if a
-	  transport is bound to a specific ip address and endpoint/media_address
-	   is set, the SIP/SDP will have the correct address in all fields but
-	  the rtp stream MAY still originate from one of the other ip addresses,
-	  most probably the "primary" ip address.  This happens because
-	   res_pjsip_sdp_rtp/create_rtp always calls ast_instance_new with
-	  the "all" ip address (0.0.0.0 or ::).
+	* app_bridge: return to the next dialplan priority
 
-	  The new option causes res_pjsip_sdp_rtp/create_rtp to call
-	  ast_rtp_instance_new with the endpoint's media_address (if specified)
-	  instead of the "all" address.  This causes the packets to originate from
-	  the specified address.
+	  When app_bridge grabs a channel and puts it into
+	  a bridge, the channel should then continue where
+	  it left off in the dialplan after the bridge has
+	  ended.   Although it stores the current dialplan
+	  location as an after bridge goto on the channel,
+	  it was executing the same priority again instead
+	  of going to the next priority.   By swapping the
+	  "specific" version of bridge_set_after_goto with
+	  bridge_set_after_go_on, the next priority in the
+	  dialplan is executed instead.
 
-	  ASTERISK-25632
-	  ASTERISK-25637
-	  Reported-by: Olivier Krief
-	  Reported-by: Dan Journo
+	  ASTERISK-24637 #close
+	  Review: https://reviewboard.asterisk.org/r/4322/
+	  Reported by: John Bigelow
+	  ........
 
-	  Change-Id: I3dfaa079e54ba7fb7c4fd1f5f7bd9509bbf8bd88
+	  Merged revisions 430467 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-10 16:22 +0000 [22801a06ee]  Daniel Journo <dan@keshercommunications.com>
 
-	* pjsip:  Add option global/regcontext
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430468 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Added new global option (regcontext) to pjsip. When set, Asterisk will
-	  dynamically create and destroy a NoOp priority 1 extension
-	  for a given endpoint who registers or unregisters with us.
+2015-01-09 12:53 +0000 [ef34a05f21]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25670 #close
-	  Reported-by: Daniel Journo
+	* AMI: Remove no longer used parameter from astman_send_listack().
 
-	  Change-Id: Ib1530c5b45340625805c057f8ff1fb240a43ea62
+	  Follow-up issue to -r430435 from reviewboard review.
+
+	  ASTERISK-24049
+	  Review: https://reviewboard.asterisk.org/r/4315/
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430452 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2015-01-09 12:16 +0000 [52a7cdb101]  Richard Mudgett <rmudgett@digium.com>
+
+	* AMI: Make AMI actions that generate event lists consistent.
+
+	  * Made the following AMI actions use list API calls for consistency:
+	  Agents
+	  BridgeInfo
+	  BridgeList
+	  BridgeTechnologyList
+	  ConfbridgeLIst
+	  ConfbridgeLIstRooms
+	  CoreShowChannels
+	  DAHDIShowChannels
+	  DBGet
+	  DeviceStateList
+	  ExtensionStateList
+	  FAXSessions
+	  Hangup
+	  IAXpeerlist
+	  IAXpeers
+	  IAXregistry
+	  MeetmeList
+	  MeetmeListRooms
+	  MWIGet
+	  ParkedCalls
+	  Parkinglots
+	  PJSIPShowEndpoint
+	  PJSIPShowEndpoints
+	  PJSIPShowRegistrationsInbound
+	  PJSIPShowRegistrationsOutbound
+	  PJSIPShowResourceLists
+	  PJSIPShowSubscriptionsInbound
+	  PJSIPShowSubscriptionsOutbound
+	  PresenceStateList
+	  PRIShowSpans
+	  QueueStatus
+	  QueueSummary
+	  ShowDialPlan
+	  SIPpeers
+	  SIPpeerstatus
+	  SIPshowregistry
+	  SKINNYdevices
+	  SKINNYlines
+	  Status
+	  VoicemailUsersList
+
+	  * Incremented the AMI version to 2.7.0.
+
+	  * Changed astman_send_listack() to not use the listflag parameter and
+	  always set the value to "Start" so the start capitalization is consistent.
+	  i.e., The FAXSessions used "Start" while the rest of the system used
+	  "start".  The corresponding complete event always used "Complete".
+
+	  * Fixed ami_show_resource_lists() "PJSIPShowResourceLists" to output the
+	  AMI ActionID for all of its list events.
+
+	  * Fixed off-nominal AMI protocol error in manager_bridge_info(),
+	  manager_parking_status_single_lot(), and
+	  manager_parking_status_all_lots().  Use of astman_send_error() after
+	  responding to the original AMI action request violates the action response
+	  pattern by sending two responses.
 
-2016-01-08 15:22 +0000 [1600ebca7d]  Kevin Harwell <kharwell@digium.com>
+	  * Fixed minor protocol error in action_getconfig() when no requested
+	  categories are found.  Each line needs to be formatted as "Header: text".
 
-	* pbx: Deadlock between contexts container and context_merge locks
+	  * Fixed off-nominal memory leak in manager_build_parked_call_string().
 
-	  Recent changes (ASTERISK-25394 commit 2bd27d12223fe33b58c453965ed5c6ed3af7c4f5)
-	  introduced the possibility of a deadlock. Due to the mentioned modifications
-	  ast_change_hints now needs to keep both merge/delete and state callbacks from
-	  occurring while it executes. Unfortunately, sometimes ast_change_hints can be
-	  called with the contexts container locked. When this happens it's possible for
-	  another thread to grab the context_merge_lock before the thread calling into
-	  ast_change_hints does and then try to obtain the contexts container lock. This
-	  of course causes a deadlock between the two threads. The thread calling into
-	  ast_change_hints waits for the other thread to release context_merge_lock and
-	  the other thread is waiting on that one to release the contexts container lock.
+	  * Eliminated unnecessary use of RAII_VAR() in ami_subscription_detail().
 
-	  Unfortunately, there is not a great way to fix this problem. When hints change,
-	  the subsequent state callbacks cannot run at the same time as a merge/delete,
-	  nor when the usual state callbacks do. This patch alleviates the problem by
-	  having those particular callbacks (the ones run after a hint change) occur in a
-	  serialized task. By moving the context_merge_lock to a task it can now safely be
-	  attempted or held without a deadlock occurring.
+	  ASTERISK-24049 #close
+	  Reported by: Jonathan Rose
+
+	  Review: https://reviewboard.asterisk.org/r/4315/
+	  ........
 
-	  ASTERISK-25640 #close
-	  Reported by: Krzysztof Trempala
+	  Merged revisions 430434 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: If2210ea241afd1585dc2594c16faff84579bf302
 
-2016-01-10 17:08 +0000 [0fc3dad965]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430435 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* devicestate: Cleanup engine thread during graceful shutdown.
+2015-01-09 08:53 +0000 [77ee23210d]  Kinsey Moore <kmoore@digium.com>
 
-	  ASTERISK-25681 #close
+	* res_fax: Add T.38 negotiation timeout option
 
-	  Change-Id: I64337c70f0ebd8c77f70792042684607c950c8f1
+	  This change makes the T.38 negotiation timeout configurable via
+	  't38timeout' in res_fax.conf or FAXOPT(t38timeout). It was previously
+	  hard coded to be 5000 milliseconds.
 
-2016-01-10 13:51 +0000 [f34dd10495]  Corey Farrell <git@cfware.com>
+	  This change also handles T.38 switch failures by aborting the fax since
+	  in the case where this can happen, both sides have agreed to switch to
+	  T.38 and Asterisk is unable to do so.
 
-	* manager: Cleanup manager_channelvars during shutdown.
+	  Review: https://reviewboard.asterisk.org/r/4320/
+	  ........
 
-	  ASTERISK-25680 #close
+	  Merged revisions 430415 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I3251d781cbc3f48a6a7e1b969ac4983f552b2446
+	  Merged revisions 430416 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-10 13:27 +0000 [1d3a1167fc]  Corey Farrell <git@cfware.com>
 
-	* res_calendar: Cleanup scheduler context at unload.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430417 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25679 #close
+2015-01-08 15:41 +0000 [8786fe13a4]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: I839159bf6882cccc1b23494c7aa2bc2a2624613f
+	* res_pjsip_pubsub: Fix persistent subscriptions not surviving graceful shutdown
 
-2016-01-08 11:49 +0000 [3a160cdbf6]  Joshua Colp <jcolp@digium.com>
+	  If you do a 'core (shutdown|restart) graceful' persistent subscriptions won't 
+	  survive.  If you do a 'core (shutdown|restart) now' or asterisk terminates for 
+	  some reason, they do.  Here's why...
 
-	* res_rtp_asterisk: Revert DTLS negotiation changes.
+	  When asterisk shuts down gracefully, it sends a 'NOTIFY/terminated' to 
+	  subscribers for each subscription.  This not only tells the subscribers that the 
+	  dialog/state machine is done, it also frees the last reference to the 
+	  subscription tree which causes the persistent subscription to get deleted from 
+	  astdb.  When asterisk restarts, nothing's left.  Just preventing the delete from 
+	  astdb doesn't work because we already told the subscriber to terminate the 
+	  dialog so we can't restart it even if it was still in astdb.  Everything works 
+	  OK if asterisk terminates unexpectedly because we never send the 'terminated' 
+	  message so on restart, the subscription is still in astdb and the subscriber is 
+	  none the wiser.
 
-	  Due to locking issues within pjnath these changes are being
-	  reverted until pjnath can be changed.
+	  This patch suppresses the sending of 'NOTIFY/terminated' on shutdown for 
+	  persistent connections.
 
-	  ASTERISK-25645
+	  Tested-by: George Joseph
 
-	  Revert "res_rtp_asterisk.c: Fix DTLS negotiation delays."
+	  Review: https://reviewboard.asterisk.org/r/4318/
+	  ........
 
-	  This reverts commit 24ae124e4f7310cfa64c187b944b2ffc060da28d.
+	  Merged revisions 430397 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2986cfb2c43dc14455c1bcaf92c3804f9da49705
 
-	  Revert "res_rtp_asterisk: Resolve further timing issues with DTLS negotiation"
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430398 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This reverts commit 965a0eee46d24321f74c244e23c5a5f45e67e12b.
+2015-01-08 15:38 +0000 [c55f86c69d]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Ie68fafde27dad4b03cb7a1e27ce2a8502c3f7bbe
+	* res_pjsip_outbound_registration: Fix reference leak.
 
-2016-01-09 17:57 +0000 [4b10fc9173]  gtjoseph <george.joseph@fairview5.com>
+	  Every time a registration started, sip_outbound_registration_response_cb bumps 
+	  the ref count on client_state then pushes a handle_registration_response task.  
+	  handle_registration_response never unreffed it though.  So every time a 
+	  registration goes out, the ref count goes up by one.
 
-	* Revert "pjsip_location: Delete contact_status object when contact is deleted"
+	  This patch adds the unreffs to handle_registration_response.
 
-	  This reverts commit 0a9941de9d24093b5ff44096d1d7406f29d11e45.
+	  Tested-by: George Joseph
 
-	  Matt,
+	  Review: https://reviewboard.asterisk.org/r/4303/
+	  ........
 
-	  This patch causes another problem and should not have been needed.
-	  Before this patch, persistent_endpoint_contact_deleted_observer WAS
-	  deleting the contact_status when ast_sip_location_delete_contact was
-	  called.  By deleting it yourself in ast_sip_location_delete_contact
-	  it was gone before the observer could run and the observer therefore
-	  was throwing an error and not sending stasis/AMI/statsd messages.
+	  Merged revisions 430395 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  So, I don't think this was the cause of your original issue.  I also
-	  had verified the contact AMI and statsd lifecycle and it was working.
-	  I'll double check now though.
 
-	  ASTERISK-25675
-	  Reported-by: Daniel Journo
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430396 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib586a6b7f90acb641b0c410f659743ab90e84f1a
+2015-01-08 11:51 +0000 [030facce94]  gtjoseph <george.joseph@fairview5.com>
 
-2016-01-09 18:04 +0000 [79b4309881]  Corey Farrell <git@cfware.com>
+	* res_pjsip_outbound_registration: Fix several reload issues
 
-	* pbx_dundi: Run cleanup on failed load.
+	  There are 2 issues with reloading registrations...
 
-	  During failed startup of pbx_dundi no cleanup was performed.  Add a call
-	  to unload_module before returning AST_MODULE_LOAD_DECLINE.
+	  1.  The 'can_reuse_registration' test wasn't considering the intervals or 
+	  expiration in its determination of whether a registration changed or not so if 
+	  you changed any of the intervals or the expiration and reloaded, the object 
+	  would get reloaded but the actual timers wouldn't change.  
+	  can_reuse_registration now does a sorcery diff on the old and new objects 
+	  instead of discretely testing certain fields.  Now if you change expiration for 
+	  instance, and reload, the timer is updated and re-registration will occur on the 
+	  new value.
 
-	  ASTERISK-25677 #close
+	  2.  If you mung up your password on an outbound registration you get a permanent 
+	  failure.  If you fix the password (on the outbound_auth object) and reload, 
+	  nothing tells outbound_registration to try again because the registration itself 
+	  didn't change.  This patch adds an observer on the "auth" object type and if any 
+	  auth changes, existing registration states are searched and those in a 
+	  REJECTED_PERMANENT state are retried.
 
-	  Change-Id: I8ffa226fda4365ee7068ac1f464473f1a4ebbb29
+	  Tested-by: George Joseph
 
-2016-01-09 13:28 +0000 [a5406b1f9e]  Corey Farrell <git@cfware.com>
+	  Review: https://reviewboard.asterisk.org/r/4304/
+	  ........
 
-	* res_crypto: Perform cleanup at shutdown.
+	  Merged revisions 430373 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change causes res_crypto to unregister CLI at shutdown while still
-	  preventing the module from being unloaded.
 
-	  ASTERISK-25673 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430374 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ie5d57338dc2752abfc0dd05d0eec86413f2304fc
+2015-01-07 15:26 +0000 [f8c4909eb7]  Kinsey Moore <kmoore@digium.com>
 
-2016-01-06 19:10 +0000 [cf8e7a580b]  Richard Mudgett <rmudgett@digium.com>
+	* ARI: Allow usage of ASYNCGOTO with Stasis()
 
-	* res_pjsip: Create human friendly serializer names.
+	  When the AMI Redirect action is used with a channel bridged inside
+	  Stasis() and not running a pbx, the channel is hung up instead of
+	  proceeding to the desired location in dialplan. This change allows
+	  such channels to be Redirected properly by detecting the operation
+	  used by Redirect (ASYNCGOTO) and using the code already established
+	  for functionality of the ARI channel continue operation.
 
-	  PJSIP name formats:
-	  pjsip/aor/<aor>-<seq> -- registrar thread pool serializer
-	  pjsip/default-<seq> -- default thread pool serializer
-	  pjsip/messaging -- messaging thread pool serializer
-	  pjsip/outreg/<registration>-<seq> -- outbound registration thread pool
-	  serializer
-	  pjsip/pubsub/<endpoint>-<seq> -- pubsub thread pool serializer
-	  pjsip/refer/<endpoint>-<seq> -- REFER thread pool serializer
-	  pjsip/session/<endpoint>-<seq> -- session thread pool serializer
-	  pjsip/websocket-<seq> -- websocket thread pool serializer
+	  ASTERISK-24591 #close
+	  Review: https://reviewboard.asterisk.org/r/4271/
+	  ........
 
-	  Change-Id: Iff9df8da3ddae1132cb2ef65f64df0c465c5e084
+	  Merged revisions 430355 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-06 19:09 +0000 [4276f185f0]  Richard Mudgett <rmudgett@digium.com>
 
-	* Sorcery: Create human friendly serializer names.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430356 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Sorcery name formats:
-	  sorcery/<type>-<seq> -- Sorcery thread pool serializer
+2015-01-07 12:54 +0000 [7f836c1c15]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: Idc2e5d3dbab15c825b97c38c028319a0d2315c47
+	* Add the ability to continue and originate using priority labels.
 
-2016-01-06 19:09 +0000 [f02ac1b7f9]  Richard Mudgett <rmudgett@digium.com>
+	  With this patch, the following two ARI commands
 
-	* Stasis: Create human friendly taskprocessor/serializer names.
+	  POST /channels
+	  POST /channels/{id}/continue
 
-	  Stasis name formats:
-	  subm:<topic>-<seq> -- Stasis subscription mailbox task processor
-	  subp:<topic>-<seq> -- Stasis subscription thread pool serializer
+	  Accept a new parameter, label, that can be used to continue to or originate
+	  to a priority label in the dialplan.
 
-	  Change-Id: Id19234b306e3594530bb040bc95d977f18ac7bfd
+	  Because this is adding a new parameter to ARI commands, the API version of
+	  ARI has been bumped from 1.6.0 to 1.7.0.
 
-2016-01-07 16:15 +0000 [ec1f1c6742]  Richard Mudgett <rmudgett@digium.com>
+	  This patch comes courtesy of Nir Simionovich from Greenfield Tech. Thanks!
 
-	* taskprocessor.c: New API for human friendly taskprocessor names.
+	  ASTERISK-24412 #close
+	  Reported by Nir Simionovich
 
-	  * Add new API call to get a sequence number for use in human friendly
-	  taskprocessor names.
+	  Review: https://reviewboard.asterisk.org/r/4285
+	  ........
 
-	  * Add new API call to create a taskprocessor name in a given buffer and
-	  append a sequence number.
+	  Merged revisions 430337 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Iac458f05b45232315ed64aa31b1df05b875537a9
 
-2016-01-06 17:19 +0000 [d8bc3e0c8b]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430338 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* taskprocessor.c: Fix CLI "core show taskprocessors" output format.
+2015-01-07 12:17 +0000 [e83853eebc]  gtjoseph <george.joseph@fairview5.com>
 
-	  Update the CLI "core show taskprocessors" output format to not be
-	  distorted because UUID names are longer than previously used taskprocessor
-	  names.
+	* res_pjsip_exten_state: Change 'does not exist' warning to notice
 
-	  Change-Id: I1a5c82ce3e8f765a0627796aba87f8f7be077601
+	  The 'new_subscribe: Extension <> does not exist or has no associated hint'
+	  is a config issue and doesn't need to clutter up logs with warnings.
+	  Changed to notice.
 
-2016-01-07 21:07 +0000 [2c4b7502de]  Richard Mudgett <rmudgett@digium.com>
+	  Tested-by: George Joseph
 
-	* taskprocessor.c: Fix CLI "core show taskprocessors" unref.
+	  Review: https://reviewboard.asterisk.org/r/4307/
+	  ........
 
-	  Change-Id: I1d9f4e532caa6dfabe034745dd16d06134efdce5
+	  Merged revisions 430319 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-07 20:44 +0000 [3b33ac7a46]  Richard Mudgett <rmudgett@digium.com>
 
-	* taskprocessor.c: Sort CLI "core show taskprocessors" output.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430321 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I71e7bf57c7b908c8b8c71f1816348ed7c5a5d51e
+2015-01-07 12:15 +0000 [8cde7443c2]  gtjoseph <george.joseph@fairview5.com>
 
-2016-01-06 19:00 +0000 [0fc32c4dd3]  Richard Mudgett <rmudgett@digium.com>
+	* res_pjsip_mwi: Change "MWI Subscription failed" message from warning to notice
 
-	* ccss.c: Replace space in taskprocessor name.
+	  The "MWI Subscription failed" message means the client is trying to subscribe
+	  to a mailbox that doesn't exist.  There's no need to clutter up logs with
+	  warnings for a client misconfiguration so I changed it to a notice.
 
-	  The CLI "core ping taskprocessor" command does not work very
-	  well with taskprocessor names that have spaces in them.  You
-	  have to put quotes around the name so using tab completion
-	  becomes awkward.
+	  Tested-by: George Joseph
 
-	  Change-Id: I29e806dd0a8a0256f4e2e0a7ab88c9e19ab0eda0
+	  Review: https://reviewboard.asterisk.org/r/4306/
+	  ........
 
-2016-01-05 16:54 +0000 [0e0c24ad78]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 430317 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* taskprocessor.c: Add CLI "core ping taskprocessor" missing unlock.
 
-	  Change-Id: I78247e0faf978bf850b5ba4e9f4933ab3c59d17b
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430318 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-07 03:33 +0000 [0f79c8839b]  Diederik de Groot <ddegroot@talon.nl>
+2015-01-07 11:54 +0000 [685f7ef924]  gtjoseph <george.joseph@fairview5.com>
 
-	* main: Use ast_strdup instead of strdup
+	* func_config: Add ability to retrieve specific occurrence of a variable
 
-	  Fix compile error in main/utils.c because strdup was used in dummy_start
+	  I guess nobody uses templates with AST_CONFIG because today if you have a
+	  context that inherits from a template and you call AST_CONFIG on the context,
+	  you'll get the value from the template even if you've overridden it in the
+	  context.  This is because AST_CONFIG only gets the first occurrence which is
+	  always from the template.
 
-	  Change-Id: Id61a6cf4f3cbf235450441e10e7da101a6335793
+	  This patch adds an optional 'index' parameter to AST_CONFIG which lets you
+	  specify the exact occurrence to retrieve, or '-1' to retrieve the last.
+	  The default behavior is the current behavior.
 
-2016-01-07 03:21 +0000 [4285dee778]  Diederik de Groot <ddegroot@talon.nl>
+	  Tested-by: George Joseph
 
-	* include/asterisk/time.h: Renamed global declaration:tv
+	  Review: https://reviewboard.asterisk.org/r/4313/
+	  ........
 
-	  Renamed global declaration:tv to dummy_tv_var_for_types,
-	  which would oltherwise cause 'shadow' warnings when 'tv'
-	  was declared as a local variable elsewhere.
+	  Merged revisions 430315 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Added comment to note that dummy_tv_var_for_types is never
-	  really exported and only used as a place holder.
 
-	  ASTERISK-25627 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430316 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I9a6e17995006584f3627efe8988e3f8aa0f5dc28
+2015-01-07 11:45 +0000 [464647d8f8]  Mark Michelson <mmichelson@digium.com>
 
-2016-01-07 15:37 +0000 [96094feab6]  Mark Michelson <mmichelson@digium.com>
+	* Fix ability to perform a remote attended transfer with PJSIP.
 
-	* PJSIP: Prevent deadlock due to dialog/transaction lock inversion.
+	  This fix has two parts:
 
-	  A deadlock was observed where the monitor thread was stuck, therefore
-	  resulting in no incoming SIP traffic being processed.
+	  * Corrected an error message to properly state that external_replaces is an extension. The
+	    error message also prints what dialplan context the external_replaces extension was being
+	    looked for in.
+	  * Corrected the printing of the Replaces: header in an INVITE request. We were duplicating
+	    "Replaces: " in the header.
 
-	  The problem occurred when two 200 OK responses arrived in response to a
-	  terminating NOTIFY request sent from Asterisk. The first 200 OK was
-	  dispatched to a threadpool worker, who locked the corresponding
-	  transaction. The second 200 OK arrived, resulting in the monitor thread
-	  locking the dialog. At this point, the two threads are at odds, because
-	  the monitor thread attempts to lock the transaction, and the threadpool
-	  thread loops attempting to try to lock the dialog.
+	  ASTERISK-24376 #close
+	  Reported by Matt Jordan
 
-	  In this case, the fix is to not have the monitor thread attempt to hold
-	  both the dialog and transaction locks at the same time. Instead, we
-	  release the dialog lock before attempting to lock the transaction.
+	  Review: https://reviewboard.asterisk.org/r/4296
+	  ........
 
-	  There have also been some debug messages added to the process in an
-	  attempt to make it more clear what is going on in the process.
+	  Merged revisions 430313 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25668 #close
-	  Reported by Mark Michelson
 
-	  Change-Id: I4db0705f1403737b4360e33a8e6276805d086d4a
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430314 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2016-01-07 09:39 +0000 [52e9de0016]  Corey Farrell <git@cfware.com>
+2015-01-07 10:56 +0000 [56de48107f]  gtjoseph <george.joseph@fairview5.com>
 
-	* ast_format_cap_append_by_type: Resolve codec reference leak.
+	* config: Add option to NOT preserve effective context when changing a template
 
-	  This resolves a reference leak caused by ASTERISK-25535.  The pointer
-	  returned by ast_format_get_codec is saved so it can be released.
+	  Let's say you have a template T with variable VAR1 = ON and you have a
+	  context C(T) that doesn't specify VAR1.  If you read C, the effective value
+	  of VAR1 is ON.  Now you change T VAR1 to OFF and call
+	  ast_config_text_file_save.  The current behavior is that the file gets
+	  re-written with T/VAR1=OFF but C/VAR1=ON is added.  Personally, I think this
+	  is a bug. It's preserving the effective state of C even though I didn't
+	  specify C/VAR1 in th first place.  I believe the behavior should be that if
+	  I didn't specify C/VAR1 originally, then the effective value of C/VAR1 should
+	  continue to follow the inherited state.  Now, if I DID explicitly specify
+	  C/VAR1, the it should be preserved even if the template changes.
+
+	  Even though I think the existing behavior is a bug, it's been that way forever
+	  so I'm not changing it.  Instead, I've created ast_config_text_file_save2()
+	  that takes a bitmask of flags, one of which is to preserve the effective context
+	  (the current behavior).  The original ast_config_text_file_save calls *2 with
+	  the preserve flag.  If you want the new behavior, call *2 directly without a
+	  flag.
+
+	  I've also updated Manager UpdateConfig with a new parameter
+	  'PreserveEffectiveContext' whose default is 'yes'.  If you want the new behavior
+	  with UpdateConfig, set 'PreserveEffectiveContext: no'.
 
-	  ASTERISK-25664 #close
+	  Tested-by: George Joseph
 
-	  Change-Id: If9941b1bf4320b2c59056546d6bce9422726d1ec
+	  Review: https://reviewboard.asterisk.org/r/4297/
+	  ........
 
-2016-01-04 04:26 +0000 [86eae38d7e]  Aaron An <anjb@ti-net.com.cn>
+	  Merged revisions 430295 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* cel/cel_radius: Fix wrong pointer.
 
-	  The macro ADD_VENDOR_CODE defined in the cel_radius.c should use the parameter
-	  y not the address of y.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430296 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  I capture the radius UDP packet via tcpdump, and the AV pairs are not correct,
-	  then i review the source code and compare it with cdr/cdr_radius.c. Fix it and
-	   it works.
+2015-01-06 21:01 +0000 [0c5234f12a]  Kinsey Moore <kmoore@digium.com>
 
-	  ASTERISK-25647 #close
-	  Reported by: Aaron An
-	  Tested by: Aaron An
+	* Fix dev-mode build on recent gcc
+	  ........
 
-	  Change-Id: I72889bccd8fde120d47aa659edc0e7e6d4d019f0
+	  Merged revisions 430274 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2016-01-05 14:52 +0000 [881dc862e0]  gtjoseph <george.joseph@fairview5.com>
 
-	* asterisk.h: Add ASTERISK_REGISTER_FILE macro
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430275 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The 11/13 branches and master use 2 different file version macros. 11/13
-	  uses ASTERISK_FILE_VERSION but master uses ASTERISK_REGISTER_FILE. This
-	  means a new file added to 11/13 can't just be cherry-picked to master
-	  because the macro has to be changed.
+2015-01-06 16:46 +0000 [220df246d9]  Matt Jordan <mjordan@digium.com>
 
-	  To make cherry-picking possible, ASTERISK_REGISTER_FILE was added
-	  to asterisk.h as a simple alias for ASTERISK_FILE_VERSION(__FILE__, NULL)
-	  The "$Revision$" tag doesn't do anything since Asterisk moved to git so
-	  just passing NULL as the verison works fine.  asterisk.h was also
-	  annotated to deprecate ASTERISK_FILE_VERSION and suggest using
-	  ASTERISK_REGISTER_FILE for all new files.
+	* Blocked revisions 430252
 
-	  Finally, 2 recent file additions, pbx_builtins.c and pbx_functions.c,
-	  were modified to use the new macro to make sure it actually worked.
-	  'core show file version' showed the correct output.
+	  ........
+	  contrib/ast-db-manage: Correct down_revision path for user_eq_phone
 
-	  Change-Id: I5867ed898818d26ee49bb6e5c7d4c1a45d4789a5
+	  When the user_eq_phone patch was backported to 13, it referenced the downward
+	  revision that the PJSIP optimistic encryption option also references. This
+	  creates a multi-path upgrade Exception when generating the SQL files.
 
-2016-01-05 11:06 +0000 [d228b62fd4]  gtjoseph <george.joseph@fairview5.com>
+	  This patch corrects this in the 13 branch. Note that trunk, which already
+	  contained both of these features, is unaffected by this problem.
 
-	* stasis_cache_pattern:  Backport to 13
 
-	  Somehow stasis_cache_pattern got out of sync between 13 and master
-	  and it was causing duplicate channel message issues in 13 when
-	  related to a specific endpoint. I.E. from statsd,
-	  'endpoints.PJSIP.1174.channels 0|g' was being emitted twice.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430254 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Backporting stasis_cache_pattern from master to 13 solved
-	  the issue and running the unit and testsuite tests confirmed
-	  that no new ones were created.
+2015-01-06 11:53 +0000 [8b5bde3e5a]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25317 #close
+	* res_pjsip_mwi: Change warning to notice
 
-	  Change-Id: Ia8707462f62d15eed14541c37f332a7bbbceb548
-2016-01-04 20:23 +0000 [e462f0063f]  Corey Farrell <git@cfware.com>
+	  When res_pjsip loads and an endpoint auto-subscribes a mailbox for mwi,
+	  if a contact hasn't registered yet, res_pjsip_mwi spits out a warning.
+	  This is a perfectly normal situation though and doesn't require something
+	  as serious as a warning.  It's also self correcting. The device will start
+	  getting mwi as soon as it registers.
 
-	* main/pbx: Move hangup handler routines to pbx_hangup_handler.c.
+	  This patch changes the warning to a notice.
 
-	  This is the sixth patch in a series meant to reduce the bulk of pbx.c.
-	  This moves hangup handler management functions to their own source.
+	  Tested-by: George Joseph
 
-	  Change-Id: Ib25a75aa57fc7d5c4294479e5cc46775912fb104
+	  Review: https://reviewboard.asterisk.org/r/4314/
+	  ........
 
-2016-01-04 19:46 +0000 [ab191d124c]  Corey Farrell <git@cfware.com>
+	  Merged revisions 430227 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/pbx: Move dialplan application management routines to pbx_app.c.
 
-	  This is the sixth patch in a series meant to reduce the bulk of pbx.c.
-	  This moves dialplan application management functions to their own source.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430228 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I444c10fb90a3cdf9f3047605d6a8aad49c22c44c
+2015-01-06 11:49 +0000 [5f60ebc004]  gtjoseph <george.joseph@fairview5.com>
 
-2016-01-04 18:20 +0000 [09a9b93896]  Corey Farrell <git@cfware.com>
+	* bridge_native_rtp: Change local/remote message from debug/2 to verb/4
 
-	* main/pbx: Move switch routines to pbx_switch.c.
+	  Change the "Locally bridged"/"Remotely bridged" messages from dbg/2 to verb/4.
 
-	  This is the fifth patch in a series meant to reduce the bulk of pbx.c.
-	  This moves ast_switch functions to their own source.
+	  Tested-by: George Joseph
 
-	  Change-Id: Ic2592a18a5c4d8a3c2dcf9786c9a6f650a8c628e
+	  Review: https://reviewboard.asterisk.org/r/4300/
+	  ........
 
-2016-01-04 18:00 +0000 [c608274a39]  Corey Farrell <git@cfware.com>
+	  Merged revisions 430225 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/pbx: Move timing routines to pbx_timing.c.
 
-	  This is the fourth patch in a series meant to reduce the bulk of pbx.c.
-	  This moves pbx timing functions to their own source.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430226 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I05c45186cb11edfc901e95f6be4e6a8abf129cd6
+2015-01-06 11:43 +0000 [fb3c8e3424]  gtjoseph <george.joseph@fairview5.com>
 
-2015-12-29 04:31 +0000 [338a8ffed6]  Martin Tomec <tomec.martin@gmail.com>
+	* outbound_registration: Add 'pjsip send register' and update 'send unregister'
 
-	* app_queue: Add member flag "in_call" to prevent reading wrong lastcall time
+	  The current behavior of 'pjsip send unregister' is to send the unregister
+	  (REGISTER with 0 exp) but let the next scheduled register proceed normally.
+	  I don't think that's a good idea.  If you unregister, it should stay
+	  unregistered until you decide to start registrations again.  So this patch
+	  just adds a cancel_registration call to the current unregister_task to
+	  cancel the timer.
 
-	  Member lastcall time is updated later than member status. There was chance to
-	  check wrapuptime for available member with wrong (old) lastcall time.
-	  New boolean flag "in_call" is set to true right before connecting call, and
-	  reset to false after update of lastcall time. Members with "in_call" set to true
-	  are treat as unavailable.
+	  Of course, now you need  a way to start registration again so I've added
+	  a 'pjsip send register' command that unregisters and cancels any existing
+	  registration (the same as send unregister), then sends an immediate
+	  registration and starts the timer back up again.
 
-	  ASTERISK-19820 #close
+	  Both changes also ripple to AMI.  There's a new PJSIPRegister command.
 
-	  Change-Id: I1923230cf9859ee51563a8ed420a0628b4d2e500
+	  There's no harm in calling either command repeatedly.  They don't care
+	  about the actual state.
 
-2015-12-28 17:23 +0000 [e13719bff1]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  Tested-by: George Joseph
 
-	* app_queue: Added reason pause of member
+	  Review: https://reviewboard.asterisk.org/r/4301/
+	  ........
 
-	  In app_queue added value Paused Reason on QueueMemberStatus when a member
-	  on queue is paused and the reason was set.
+	  Merged revisions 430223 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25480 #close
-	  Reporte by: Rodrigo Ramírez Norambuena
 
-	  Change-Id: Ia5db503482f50764c15e2020196c785f59d4a68e
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430224 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-30 10:49 +0000 [4ec85a9f07]  gtjoseph <george.joseph@fairview5.com>
+2015-01-06 11:29 +0000 [7dc0c88fc6]  gtjoseph <george.joseph@fairview5.com>
 
-	* voicemail: Move app_voicemail / res_mwi_external conflict to runtime
+	* pjsip cli: Fix sorting of contacts for 'pjsip list contacts'
 
-	  The menuselect conflict between app_voicemail and res_mwi_external
-	  makes it hard to package 1 version of Asterisk.  There no actual
-	  build dependencies between the 2 so moving this check to runtime
-	  seems like a better solution.
+	  For some reason I was using a hash container instead of a list to gather the
+	  contacts for 'pjsip list/show contacts' so even though I had a sort function,
+	  the output wasn't sorted.  This patch just changes the hash container to a
+	  list container and the contacts now appear sorted in the CLI.
 
-	  The ast_vm_register and ast_vm_greeter_register functions in app.c
-	  were modified to return AST_MODULE_LOAD_DECLINE instead of -1 if there
-	  is already a voicemail module registered. The modules' load_module
-	  functions were then modified to return DECLINE instead of -1 to the
-	  loader.  Since -1 is interpreted by the loader as AST_MODULE_LOAD_FAILURE,
-	  the modules were incorrectly causing Asterisk to stop so this needed
-	  to be cleaned up anyway.
+	  Tested-by: George Joseph
 
-	  Now you can build both and use modules.conf to decide which voicemail
-	  implementation to load.
+	  Review: https://reviewboard.asterisk.org/r/4305/
+	  ........
 
-	  The default menuselect options still build app_voicemail and not
-	  res_mwi_external but if both ARE built, res_mwi_external will load
-	  first and become the voicemail provider unless modules.conf rules
-	  prevent it.  This is noted in CHANGES.
+	  Merged revisions 430221 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I7d98d4e8a3b87b8df9e51c2608f0da6ddfb89247
 
-2016-01-04 16:22 +0000 [7fdcfd7724]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430222 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* main/pbx: Move variable routines to pbx_variables.c.
+2015-01-05 16:50 +0000 [0b8fbf9238]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  This is the third patch in a series meant to reduce the bulk of pbx.c.
-	  This moves channel and global variable routines to their own source.
+	* bridge: avoid leaking channel during blond transfer pt2
 
-	  Change-Id: Ibe8fb4647db11598591d443a99e3f99200a56bc6
+	  A blond transfer to a failed destination, when followed
+	  by a recall attempt, lead to a leak of the reference to
+	  the destination channel.  In addition to correcting the
+	  regression on the previous attempt (r429826) this fixes
+	  the leak and two additional reference leaks on failures
+	  of bridge_import.
 
-2015-12-04 17:22 +0000 [80a8b2a4cd]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24513 #close
+	  Review: https://reviewboard.asterisk.org/r/4302/
+	  ........
 
-	* app_dial: Immediately exit dial if the caller is already hung up.
+	  Merged revisions 430199 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  If a caller hangs up before dial is executed within an AGI then the AGI
-	  has likely eaten all queued frames before executing the dial in DeadAGI
-	  mode.  With the caller hung up and no pending frames from the caller's
-	  read queue, dial would not know that the call has hung up until a called
-	  channel answers.  It is rather annoying to whoever just answered the
-	  non-existent call.
+	  Merged revisions 430200 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Dial should not continue execution in DeadAGI mode, hangup handlers, or
-	  the h exten.
 
-	  * Added a check early in dial to abort dialing if the caller has hungup.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430201 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25307 #close
-	  Reported by: David Cunningham
+2015-01-05 11:57 +0000 [e0bd2ca104]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Icd1bc0764726ef8c809f76743ca008d0f102f418
+	* pjsip: Document addition of 'PJSIP_AOR' and 'PJSIP_CONTACT' in CHANGES file.
+	  ........
 
-2016-01-02 10:26 +0000 [1087b0c6ed]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 430181 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/cdr: Allow setting properties on a finalized CDR if it is the last one
 
-	  Prior to this patch, we explicitly disallowed setting any properties on a
-	  finalized CDR. This seemed like a good idea at the time; in practice, it was
-	  more restrictive.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430182 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  There are weird and strange scenarios where setting a property on a finalized
-	  CDR is definitely wrong. For example, we may Fork a CDR, finalizing the
-	  previous one, then change a property. In said case, the old CDR is supposed
-	  to now be 'immutable' (so to speak), and should not be updated. From the
-	  perspective of the code, a forked CDR that is finalized is just finalized.
-	  Hence why we decided these should not be updated.
+2015-01-05 11:53 +0000 [f7cf988a82]  Joshua Colp <jcolp@digium.com>
 
-	  In practice, it is much more common to want to set a property on a CDR in
-	  the h extension or in a hangup handler. Disallowing a common scenario to make
-	  an esoteric behaviour work isn't good. This patch fixes this by allowing
-	  callers to set a property IF we are the last CDR in the chain. This preserves
-	  the finalized CDR if it was forked, while allowing the more common case to
-	  function.
+	* pjsip: Add 'PJSIP_AOR' and 'PJSIP_CONTACT' dialplan functions.
 
-	  ASTERISK-25458 #close
+	  The PJSIP_AOR dialplan function allows inspection of configured AORs including
+	  what contacts are currently bound to them.
 
-	  Change-Id: Icf3553c607b9f561152a41e6d8381d594ccdf4b9
+	  The PJSIP_CONTACT dialplan function allows inspection of contacts in existence.
+	  These can include both externally added (by way of registration) or permanent
+	  ones.
 
-2016-01-02 10:23 +0000 [1f23e65b89]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24341
+	  Reported by: xrobau
 
-	* main/cdr: Set the end time on a CDR if endbeforehexten is Yes
+	  Review: https://reviewboard.asterisk.org/r/4308/
+	  ........
 
-	  Prior to this patch, the CDR engine attempted to set the end time on a CDR
-	  that was executing hangup logic and with endbeforehexten set to Yes by
-	  calling a function that inspects the properties on the Party A snapshot to
-	  determine if we are ready to set the end time. That always failed. This is
-	  because a Party A snapshot is not updated for CDRs that are executing hangup
-	  logic with endbeforehexten=Yes.
+	  Merged revisions 430179 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Instead of calling a function that looks at the Party A snapshot, we just
-	  simply set the end time on the CDR. This is safe to call multiple times, and is
-	  safe to call at this point as we know that (a) we are executing hangup logic,
-	  and (b) we are supposed to set the end time at this point.
 
-	  ASTERISK-25458
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430180 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I0c27b493861f9c13c43addbbb21257f79047a3b3
+2014-12-31 12:54 +0000 [8d059c3808]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2015-12-30 20:51 +0000 [2ffade4574]  Corey Farrell <git@cfware.com>
+	* rtp_engine: keep payload types in correct range
 
-	* main/pbx: Move custom function routines to pbx_functions.c.
+	  In r428708 additional codecs were added including
+	  a payload type of 128 which is outside of nominal
+	  range of 0-127.  This change moves changes 128 to
+	  96 to avoid causing a pjsip assertion when making
+	  a call to an endpoint configured with allow=all.
 
-	  This is the second patch in a series meant to reduce the bulk of pbx.c.
-	  This moves custom function management routines to their own source.
+	  ASTERISK-24367 #close
+	  Review: https://reviewboard.asterisk.org/r/4286/
 
-	  Change-Id: I34a6190282f781cdbbd3ce9d3adeac3c3805e177
 
-2015-12-28 19:18 +0000 [20b8474f20]  gtjoseph <george.joseph@fairview5.com>
 
-	* main/pbx: Move pbx_builtin dialplan applications to pbx_builtins.c
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430164 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  We joked about splitting pbx.c into multiple files but this first step was
-	  fairly easy.  All of the pbx_builtin dialplan applications have been moved
-	  into pbx_builtins.c and a new pbx_private.h file was added. load_pbx_builtins()
-	  is called by asterisk.c just after load_pbx().
+2014-12-29 07:14 +0000 [cb6a737359]  Kinsey Moore <kmoore@digium.com>
 
-	  A few functions were renamed and are cross-exposed between the 2 source files.
+	* PJSIP: Update transport method documentation
 
-	  Change-Id: I87066be3dbf7f5822942ac1449d98cc43fc7561a
+	  This updates the documentation for the 'method' configuration option to
+	  be more verbose about the behaviors of values 'unspecified' and
+	  'default'. They do exactly the same thing which is to select the
+	  default as defined by PJSIP which is currently TLSv1.
 
-2015-12-24 20:26 +0000 [e4a566918a]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4264/
+	  ........
 
-	* tests/test_stasis_endpoints: Remove expected duplicate events
+	  Merged revisions 430145 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The cache_clear test was written to expect duplicate Stasis messages
-	  sent from the technology endpoint to the all caching topic. This patch
-	  fixes the test to no longer expect these duplicate messages.
 
-	  ASTERISK-25137
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430146 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I58075d70d6cdf42e792e0fb63ba624720bfce981
+2014-12-24 15:28 +0000 [91becf952a]  Kevin Harwell <kharwell@digium.com>
 
-2015-12-28 14:02 +0000 [a280400758]  Joshua Colp <jcolp@digium.com>
+	* app_queue: Update sample conf documenation
 
-	* test_time: Provide a timeout when waiting.
+	  Updated the queues.conf.sample file to explicitly state which channel queue
+	  variables are propagated to.
 
-	  The test_timezone_watch unit test is written to expect a
-	  condition to be signaled when the inotify daemon thread runs.
-	  There exists a small window where the test_timezone_watch
-	  thread can signal the inotify daemon thread while it is not
-	  reading on the underlying file descriptor. If this occurs
-	  the test_timezone_watch thread will wait indefinitely for a
-	  signal that will never arrive.
+	  ASTERISK-24267
+	  Reported by: Mitch Claborn
+	  ........
 
-	  This change adds a timeout to the condition so it will return
-	  regardless after a period of time.
+	  Merged revisions 430126 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ifed981879df6de3d93acd3ee0a70f92546517390
+	  Merged revisions 430127 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-05-27 13:22 +0000 [3a1c4885be]  gtjoseph <george.joseph@fairview5.com>
 
-	* endpoint/stasis: Eliminate duplicate events on endpoint status change
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430128 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When an endpoint is created, its messages are forwarded to both the tech
-	  endpoint topic and the all endpoints topic. This is done so that various
-	  parties interested in endpoint messages can subscribe to just the tech
-	  endpoint and receive all messages associated with that particular technology,
-	  as opposed to subscribing to the all endpoints topic. Unfortunately, when the
-	  tech endpoint is created, it also forwards all of its messages to the all
-	  topic. This results in duplicate messages whenever an endpoint publishes its
-	  messages.
+2014-12-24 10:59 +0000 [3a73c6c90e]  Matt Jordan <mjordan@digium.com>
 
-	  This patch resolves the duplicate message issue by creating a new function
-	  for Stasis caching topics, stasis_cp_sink_create. In most respects, this acts
-	  as a normal caching topic, save that it no longer forwards messages it receives
-	  to the all endpoints topic. This allows it to act as an aggregation "sink",
-	  while preserving the necessary caching behaviour.
+	* main/pbx.c: Fix double lock of contexts lock introduced by r429967
 
-	  ASTERISK-25137 #close
-	  Reported-by: Vitezslav Novy
+	  We only need to hold the context_merge_lock once. Locking it twice will make
+	  many other parts of Asterisk very sad.
 
-	  ASTERISK-25116 #close
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  ASTERISK-24641 #close
 
-	  Change-Id: Ie47784adfb973ab0063e59fc18f390d7dd26d17b
-2015-12-24 22:19 +0000 [136c537695]  Dade Brandon <dade@xencall.com>
 
-	* res_http_websocket.c: prevent avoidable disconnections caused by write errors
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430111 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Updated ast_websocket_write to encode the entire frame in to one
-	  write operation, to ensure that we don't end up with a situation
-	  where the websocket header has been sent, while the body can not
-	  be written.
+2014-12-23 17:19 +0000 [7ea4156a5e]  gtjoseph <george.joseph@fairview5.com>
 
-	  Previous to August's patch in commit b9bd3c14, certain network
-	  conditions could cause the header to be written, and then the
-	  sub-sequent body to fail - which would cause the next successful
-	  write to contain a new header, and a new body (resulting in
-	  the peer receiving two headers - the second of which would be
-	  read as part of the body for the first header).
+	* pjsip_options: Fix continued qualifies after endpoint/aor deletion
 
-	  This was patched to have both write operations individually fail
-	  by closing the websocket.
+	  If you remove an endpoint/aor from pjsip.conf then do a core reload,
+	  qualifies will continue even though the object are gone.  This happens
+	  because nothing clears out the qualify tasks.
 
-	  In a case available to the submitter of this patch, the same
-	  body which would consistently fail to write, would succeed
-	  if written at the same time as the header.
+	  This patch unschedules all existing qualify tasks before scheduling
+	  new ones on reload.
 
-	  This update merges the two operations in to one, adds debug messages
-	  indicating the reason for a websocket connection being closed during
-	  a write operation, and clarifies some variable names for code legibility.
+	  Tested-by: George Joseph
 
-	  Change-Id: I4db7a586af1c7a57184c31d3d55bf146f1a40598
+	  Review: https://reviewboard.asterisk.org/r/4290/
+	  ........
 
-2015-12-27 22:38 +0000 [f2efbb5d75]  Corey Farrell <git@cfware.com>
+	  Merged revisions 430064 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Remove res_jabber file that was left behind.
 
-	  Change-Id: I9d88fac0394d5bbaff0900a2ee911c4e4478846b
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430067 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-13 13:09 +0000 [dde7f3c1c4]  Matt Jordan <mjordan@digium.com>
+2014-12-23 17:16 +0000 [62d1dba271]  gtjoseph <george.joseph@fairview5.com>
 
-	* res_pjsip_history: Add a module that provides PJSIP history for debugging
+	* test_astobj2: Fix warning for missing trailing slash in category
 
-	  This patch adds a new module, res_pjsip_history, that provides a slightly
-	  better way of debugging SIP message traffic on a busy Asterisk system. The
-	  existing mechanisms all rely on passively dumping a SIP message to the CLI.
-	  While this is perfectly fine for logging purposes and well controlled
-	  environments, on many installations, the amount of SIP messages Asterisk
-	  receives will quickly swamp the CLI. This makes it difficult to view/capture
-	  those messages that you want to diagnose in real time.
+	  This patch adds a trailing slash to the category for this test.
+	  No more warning.
 
-	  This patch provides another way of handling this. When enabled, the module
-	  will store SIP message traffic in memory. This traffic can then be queried
-	  at leisure.
+	  Tested-by: George Joseph
 
-	  In order to make the querying useful, a CLI command has been implemented,
-	  'pjsip show history', that supports a basic expression syntax similar to
-	  SQL or other query languages. A small number of useful fields have been
-	  added in this initial patch; additional fields can easily be added in
-	  later improvements. Those fields are:
-	   - number: The entry index in the history
-	   - timestamp: The time the message was recieved
-	   - addr: The source/destination address of the message
-	   - sip.msg.request.method: The request method
-	   - sip.msg.call-id: The Call-ID header
+	  Review: https://reviewboard.asterisk.org/r/4295/
+	  ........
 
-	  Note - this is a resurrection of the module initially proposed on Review Board
-	  here: https://reviewboard.asterisk.org/r/4053/
+	  Merged revisions 430059 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I39bd74ce998e99ad5ebc0aab3e84df3a150f8e36
 
-2015-12-25 09:56 +0000 [be050f2638]  Dade Brandon <dade@xencall.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430060 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip.c: fix websocket_write_timeout default value
+2014-12-22 15:20 +0000 [1c0604e905]  Richard Mudgett <rmudgett@digium.com>
 
-	  websocket_write_timeout was not being set to its default value
-	  during sip config reload, which meant that prior to this commit,
-	  1) the default value of 100 was not used, unless an invalid value
-	  (or 1) was specified in sip.conf for websocket_write_timeout, and
-	  2) if the websocket_write_timeout directive was removed from sip.conf
-	  without a full restart of asterisk, then the previous value would
-	  continue to be used indefinitely.
+	* DTMF atxfer: Setup recall channels as if the transferee initiated the call.
 
-	  This essentially lead to a 0ms write timeout (the first write attempt
-	  in ast_careful_fwrite must have succeeded) in websocket write requests
-	  from chan_sip, unless websocket_write_timeout was explicitely set in sip.conf.
+	  After the initial DTMF atxfer call attempt to the transfer target fails to
+	  answer during a blonde transfer, the recall callback channels do not get
+	  setup with information from the initial transferrer channel.  As a result,
+	  the recall callback to the transferrer does not have callid, channel
+	  variables, datastores, accountcode, peeraccount, COLP, and CLID setup.  A
+	  similar situation happens with the recall callback to the transfer target
+	  but it is less visible.  The recall callback to the transfer target does
+	  not have callid, channel variables, datastores, accountcode, peeraccount,
+	  and COLP setup.
 
-	  Changes to websocket_write_timeout still only apply to new websocket
-	  sessions, after the sip reload -- timeouts on existing sessions are
-	  not adjusted during sip reload.
+	  * Added missing information to the recall callback channels before
+	  initiating the call.  callid, channel variables, datastores, accountcode,
+	  peeraccount, COLP, and CLID
 
-	  Change-Id: Ibed3816ed29cc354af6564c5ab3e75eab72cb953
+	  * Set callid of the transferrer channel on the DTMF atxfer controller
+	  thread attended_transfer_monitor_thread().
 
-2015-12-23 17:40 +0000 [b3024cad10]  Richard Mudgett <rmudgett@digium.com>
+	  * Added missing channel unlocks and props unref to off nominal paths in
+	  attended_transfer_properties_alloc().
 
-	* bridge_basic.c: Fix GOTO_ON_BLINDXFR
+	  ASTERISK-23841 #close
+	  Reported by: Richard Mudgett
 
-	  Use of GOTO_ON_BLINDXFR would not work at all.  The target location would
-	  never be executed by the transferring channel.
+	  Review: https://reviewboard.asterisk.org/r/4259/
+	  ........
 
-	  * Made feature_blind_transfer() call ast_bridge_set_after_go_on() with
-	  valid context, exten, and priority parameters from the transferring
-	  channel.
+	  Merged revisions 430034 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Renamed some feature_blind_transfer() local variables for clarity.
 
-	  ASTERISK-25641 #close
-	  Reported by Dmitry Melekhov
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430041 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I19bead9ffdc4aee8d58c654ca05a198da1e4b7ac
+2014-12-22 14:25 +0000 [7d954f4cb1]  Richard Mudgett <rmudgett@digium.com>
 
-2015-12-24 12:19 +0000 [0a9941de9d]  Matt Jordan <mjordan@digium.com>
+	* Fix compilation since the patch for ASTERISK-24363 went in.
 
-	* res/res_pjsip_location: Delete contact_status object when contact is deleted
 
-	  In 450579e908, a change was made that removed the deletion of the
-	  'contact_status' object when a 'contact' object is deleted in sorcery.
-	  This unfortunately means that the 'contact_status' object persists, even when
-	  something has explicitly removed a contact. The result is that the state of
-	  the contact will not be regenerated if that contact is re-created, and the
-	  stale state will be reported/used for that contact. It also results in
-	  no ContactStatusChanged events being generated for either ARI or AMI.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430028 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch restores the deletion logic that was removed. Doing so now
-	  results in the expected events being generated again.
+2014-12-22 14:08 +0000 [bbd9ff122e]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I28789a112e845072308b5b34522690e3faf58f07
+	* queue_log: Post QUEUESTART entry when Asterisk fully boots.
 
-2015-12-24 10:18 +0000 [1e24a0ca8a]  Kevin Harwell <kharwell@digium.com>
+	  The QUEUESTART log entry has historically acted like a fully booted event
+	  for the queue_log file.  When the QUEUESTART entry was posted to the log
+	  was broken by the change made by ASTERISK-15863.
 
-	* res_rtp_asterisk: rtp->ice check not wrapped in HAVE_PJPROJECT ifdef
+	  * Made post the QUEUESTART queue_log entry when Asterisk fully boots.
+	  This restores the intent of that log entry and happens after realtime has
+	  had a chance to load.
 
-	  Change-Id: I19b49112e1b630bd04e859f14ccf96f8ebd6b151
+	  AST-1444 #close
+	  Reported by: Denis Martinez
 
-2015-12-20 21:33 +0000 [1d3d20dd68]  Dade Brandon <dade@xencall.com>
+	  Review: https://reviewboard.asterisk.org/r/4282/
+	  ........
 
-	* app_amd: Correct documentation to reflect functionality
+	  Merged revisions 430009 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Update documentation to reflect that maximum_number_of_words
-	  has functionality inconsistent with the variable name (and inconsistent
-	  with prior documentation.)
+	  Merged revisions 430010 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Update documentation for silence_threshold, which previously implied
-	  that it was measuring time, rather than noise averages in the sample.
 
-	  Update the comments in amd.conf.sample.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@430011 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25639 #close
-	  Change-Id: I4b1451e5dc9cb3cb06d59b6ab872f5275ba79093
+2014-12-22 09:40 +0000 [264a50c52a]  Karsten Wemheuer (License 5930)
 
-2015-12-17 19:05 +0000 [965a0eee46]  Dade Brandon <dade@xencall.com>
+	* chan_sip: Send CANCEL via original INVITE destination even after UPDATE request
 
-	* res_rtp_asterisk: Resolve further timing issues with DTLS negotiation
+	  Given the following scenario:
+	  * Three SIP phones (A, B, C), all communicating via a proxy with Asterisk
+	  * A call is established between A and B. B performs a SIP attended transfer of
+	    A to C. B sets the call on hold (A is hearing MOH) and dials the extension of
+	    C. While phone C is ringing, B transfers the call (that is, what we typically
+	    call a 'blond transfer').
+	  * When the transfer completes, A hears the ringing of phone C, while B is idle.
+
+	  In the SIP messaging for the above scenario, a REFER request is sent to
+	  transfer the call. When "sendrpid=yes" is set in sip.conf, Asterisk may send an
+	  UPDATE request to phone C to update party information. This update is sent
+	  directly to phone C, not through the intervening proxy. This has the unfortunate
+	  side effect of providing route information, which is then set on the sip_pvt
+	  structure for C. If someone (e.g. B) is trying to get the call back (through a
+	  directed pickup), Asterisk will send a CANCEL request to C. However, since we
+	  have now updated the route set, the CANCEL request will be sent directly to C
+	  and not through the proxy. The phone ignores this CANCEL according to RFC3261
+	  (Section 9.1).
+
+	  This patch updates reqprep such that the route is not updated if an UPDATE
+	  request is being sent while the INVITE state is INV_PROCEEDING or
+	  INV_EARLY_MEDIA. This ensures that a subsequent CANCEL request is still sent
+	  to the correct location.
 
-	  Resolves an edge case dtls negotiation delay for certain networks which
-	  somehow manage to drop the rtcp side's packet when these are both sent
-	  ast_rtp_remote_address_set, causing it to have to time-out and restart
-	  the handshake.
+	  Review: https://reviewboard.asterisk.org/r/4279
 
-	  Move dtls pending bio flush in to it's own function, and call it from
-	  ast_rtp_on_ice_complete, when we're rtp->ice, rather than when
-	  ast_rtp_remote_address_set.
+	  ASTERISK-24628 #close
+	  Reported by: Karsten Wemheuer
+	  patches:
+	    issue.patch uploaded by Karsten Wemheuer (License 5930)
+	  ........
 
-	  Keep the existing flush from the recent change to res_rtp_remote_address_set
-	  if ice is not being used.
+	  Merged revisions 429982 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25614 #close
-	  Reported-by: XenCALL
-	  Tested by: XenCALL
+	  Merged revisions 429983 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ie2caedbdee1783159f375589b6fd3845c8577ba5
 
-2015-12-18 09:54 +0000 [ae428d8460]  Carlos Oliva <carlos.oliva@invoxcontact.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429984 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* app_queue: update RT members when the 1st call joins a queue with no agents
+2014-12-22 08:33 +0000 [0c38276d6e]  Gareth Palmer (License 5169)
 
-	  If a call enters on a queue and the members on that queue are updated in
-	  realtime (ex: using mysql inserting a new agent) the queue members are
-	  never refreshed and the call will stay in the queue until other event occurs.
-	  This happens only if this is the first call of the queue and there is no
-	  agents servicing.
-	  This patch prevent this issue, ensuring realtime members are updated if
-	  there is one call in the queue and no available agents
+	* presencestate: Allow channel drivers to provide presence state information
 
-	  ASTERISK-25442 #close
+	  This patch adds the ability for channel drivers to supply presence information
+	  in a similar manner to device state. The patch does not provide any channel
+	  driver implementations, but it does provide the core infrastructure necessary
+	  for channel drivers to provide such information.
 
-	  Change-Id: If1e036d013a5c1d8b0bf60d71d48fe98694a8682
+	  The core handles multiple providers of presence state information. Ordering
+	  of presence state is as follows:
+	   INVALID < NOT_SET < AVAILABLE < UNAVAILABLE < CHAT < AWAY < XA < DND
 
-2015-12-05 10:01 +0000 [59d5bb0613]  Joshua Colp <jcolp@digium.com>
+	  Each provider can trump the previous if it provides a presence state that
+	  supercedes a previous one.
 
-	* res_sorcery_memory_cache: Add support for a full backend cache.
+	  Review: https://reviewboard.asterisk.org/r/4050
 
-	  This change introduces the configuration option 'full_backend_cache'
-	  which changes the cache to be a full mirror of the backend instead
-	  of a per-object cache. This allows all sorcery retrieval operations
-	  to be carried out against it and is useful for object types which
-	  are used in a "retrieve all" or "retrieve some" pattern.
+	  ASTERISK-24363 #close
+	  Reported by: Gareth Palmer
+	  patches:
+	    chan_presencestate-428146.patch uploaded by Gareth Palmer (License 5169)
 
-	  ASTERISK-25625 #close
 
-	  Change-Id: Ie2993487e9c19de563413ad5561c7403b48caab5
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429967 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-17 10:25 +0000 [0cefcabd58]  Joshua Colp <jcolp@digium.com>
+2014-12-22 06:16 +0000 [2afeadcc84]  Matt Jordan <mjordan@digium.com>
 
-	* rtp_engine: Ignore empty filenames in DTLS configuration.
+	* app_confbridge: Fix build error caused by XML validation errors
 
-	  When applying an empty DTLS configuration the filenames in the
-	  configuration will be empty. This is actually valid to do and
-	  each filename should simply be ignored.
+	  Summaries can't contain XML nodes, as they are defined to contain only text
+	  data.
 
-	  Change-Id: Ib761dc235638a3fb701df337952f831fc3e69539
 
-2015-12-17 08:10 +0000 [158a0a5422]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429952 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: Enable WebSocket support by default.
+2014-12-21 20:35 +0000 [b79a4a464f]  Gareth Palmer (License 5169)
 
-	  Per the documentation the WebSocket support in chan_sip is
-	  supposed to be enabled by default but is not. This change
-	  corrects that.
+	* app_confbridge: Add the ability to pass options/command to MixMonitor
 
-	  Change-Id: Icb02bbcad47b11a795c14ce20a9bf29649a54423
+	  This patch adds the ability to pass options and a command to MixMontor when
+	  recording a conference using ConfBridge.
 
-2015-12-14 12:04 +0000 [a9d6fc571d]  Joshua Colp <jcolp@digium.com>
+	  New options are -
 
-	* json: Audit ast_json_* usage for thread safety.
+	  * record_options: Options to MixMontor, eg: m(), W() etc.
+	  * record_command: The command to execute when recording is over.
+	  * record_file_timestamp: Append the start time to the file name.
 
-	  The JSON library Asterisk uses, jansson, is not thread
-	  safe for us in a few ways. To help with this wrappers for JSON
-	  object reference count increasing and decreasing were added
-	  which use a global lock to ensure they don't clobber over
-	  each other. This does not extend to reference count manipulation
-	  within the jansson library itself. This means you can't safely
-	  use the object borrowing specifier (O) in ast_json_pack and
-	  you can't share JSON instances between objects.
+	  These options can also be used with the CONFBRIDGE function, e.g.,
+	  Set(CONFBRIDGE(bridge,record_command)=/path/to/command ^{MIXMONITOR_FILENAME}))
 
-	  This change removes uses of the O specifier and replaces them
-	  with the o specifier and an explicit ast_json_ref. Some cases
-	  of instance sharing have also been removed.
+	  Review: https://reviewboard.asterisk.org/r/4023
 
-	  ASTERISK-25601 #close
+	  ASTERISK-24351 #close
+	  Reported by: Gareth Palmer
+	  patches:
+	    record_command-428838.patch uploaded by Gareth Palmer (License 5169)
 
-	  Change-Id: I06550d8b0cc1bfeb56cab580a4e608ae4f1ec7d1
 
-2015-12-16 11:28 +0000 [53bd5a539a]  Mark Michelson <mmichelson@digium.com>
 
-	* Alembic: Increase column size of PJSIP AOR "contact".
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429934 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When running the PJSIP AMI "show_endpoint" test with automatic
-	  conversion to realtime, the test would fail. This was because the AOR
-	  "contact" column was sized at 40, and the configured contact was larger
-	  than that.
+2014-12-21 18:17 +0000 [b137a92aef]  gtjoseph <george.joseph@fairview5.com>
 
-	  This commit increases the size of the contact column to 255 characters.
+	* res_pjsip_phoneprovi_provider: Fix reload
 
-	  Change-Id: Ia65bc7fd37699b7c0eaef9629a1a31eab9a24ba1
+	  Reloading wasn't working correctly because on a reload, the sorcery apply
+	  handler was never being called for unchanged users.  So, instead of using
+	  an apply handler, I'm now iterating over all users.  Works much more reliably.
 
-2015-12-16 11:25 +0000 [da17dc4d75]  Mark Michelson <mmichelson@digium.com>
+	  Tested-by: George Joseph
 
-	* Alembic: Add PJSIP global keep_alive_interval.
+	  Review: https://reviewboard.asterisk.org/r/4288/
+	  ........
 
-	  The keep_alive_interval option was added about a year ago, but no
-	  alembic revision was created to add the appropriate column to the
-	  database.
+	  Merged revisions 429914 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This commit fixes the problem and adds the column. This was discovered
-	  by running the testsuite with automatic conversion to realtime enabled.
 
-	  Change-Id: If3ef92a7c4f4844d08f8aae170d2178aec5c4c1a
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429915 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-08 13:04 +0000 [fe8011cc50]  sungtae kim <pchero21@gmail.com>
+2014-12-20 14:57 +0000 [ba403e83bd]  Joshua Colp <jcolp@digium.com>
 
-	* AMI: Fixed OriginateResponse message
+	* acl: Fix reloading of configuration if configuration file does not exist at startup.
 
-	  When the asterisk sending OriginateResponse message,
-	  it doesn't set the "Uniqueid".
-	  And it didn't support correct response message for
-	  Application originate.
+	  The named ACL code incorrectly destroyed the config options information if loading
+	  of the configuration file failed at startup. This would result in reloading
+	  also failing even if a valid configuration file was put in place.
 
-	  ASTERISK-25624 #close
+	  ASTERISK-23733 #close
+	  Reported by: Richard Kenner
+	  ........
 
-	  Change-Id: I26f54f677ccfb0b7cfd4967a844a1657fd69b74d
+	  Merged revisions 429893 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
+
+	  Merged revisions 429894 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-12-15 18:01 +0000  Asterisk Development Team <asteriskteam@digium.com>
 
-	* asterisk 13.7.0-rc1 Released.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429895 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-15 11:57 +0000 [0370acecfc]  Kevin Harwell <kharwell@lunkwill>
+2014-12-19 14:56 +0000 [54bd1c9683]  Richard Mudgett <rmudgett@digium.com>
 
-	* Release summaries: Add summaries for 13.7.0-rc1
+	* res_http_websocket.c: Fix incorrect use of sizeof in ast_websocket_write().
 
-2015-12-15 11:54 +0000 [d1bb33fe0b]  Kevin Harwell <kharwell@lunkwill>
+	  This won't fix the reported issue but it is an incorrect use of sizeof.
 
-	* .version: Update for 13.7.0-rc1
+	  ASTERISK-24566
+	  Reported by:  Badalian Vyacheslav
+	  ........
 
-2015-12-15 11:54 +0000 [d06a65de01]  Kevin Harwell <kharwell@lunkwill>
+	  Merged revisions 429867 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* .lastclean: Update for 13.7.0-rc1
+	  Merged revisions 429868 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-12-15 11:54 +0000 [fb37b44660]  Kevin Harwell <kharwell@lunkwill>
 
-	* realtime: Add database scripts for 13.7.0-rc1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429870 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-15 11:48 +0000 [20b7164b8c]  Kevin Harwell <kharwell@lunkwill>
+2014-12-19 11:34 +0000 [b508b3474e]  Richard Mudgett <rmudgett@digium.com>
 
-	* .version: Update for 13.7.0-rc1
+	* chan_dahdi: Don't ignore setvar when using configuration section scheme.
 
-2015-12-15 11:48 +0000 [6cbf2414c3]  Kevin Harwell <kharwell@lunkwill>
+	  When the configuration section scheme of chan_dahdi.conf is used (keyword
+	  dahdichan instead of channel) all setvar= options are completely ignored.
+	  No variable defined this way appears in the created DAHDI channels.
 
-	* .lastclean: Update for 13.7.0-rc1
+	  * Move the clearing of setvar values to after the deferred processing of
+	  dahdichan.
 
-2015-12-15 11:48 +0000 [ba1794464d]  Kevin Harwell <kharwell@lunkwill>
+	  AST-1378 #close
+	  Reported by: Guenther Kelleter
+	  Patch by: Guenther Kelleter
+	  ........
 
-	* realtime: Add database scripts for 13.7.0-rc1
+	  Merged revisions 429825 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-12-15 11:39 +0000 [b3e9753a23]  Kevin Harwell <kharwell@lunkwill>
+	  Merged revisions 429829 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* .version: Update for 13.7.0-rc1
 
-2015-12-15 11:39 +0000 [b0df64b5f0]  Kevin Harwell <kharwell@lunkwill>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429830 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* .lastclean: Update for 13.7.0-rc1
+2014-12-19 11:27 +0000 [07d1012383]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2015-12-15 11:39 +0000 [ce9a59faf6]  Kevin Harwell <kharwell@lunkwill>
+	* bridge: avoid leaking channel during blond transfer
 
-	* realtime: Add database scripts for 13.7.0-rc1
+	  After a blond transfer (start attended and hang up)
+	  to a destination that also hangs up without answer,
+	  the Local;1 channel was leaked and would show up on
+	  core show channels.  This was happening because the
+	  attended state blond_nonfinal_enter() resetting the
+	  props->transfer_target to null while releasing it's
+	  own reference, which would later prevent props from
+	  releasing another reference during destruction. The
+	  change made here is simply to not assign the target
+	  to NULL.
 
-2015-12-15 11:28 +0000 [2e26bef5bb]  Kevin Harwell <kharwell@lunkwill>
+	  ASTERISK-24513 #close
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4262/
+	  ........
 
-	* .version: Update for 13.7.0-rc1
+	  Merged revisions 429826 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-12-15 11:28 +0000 [5e9b47516d]  Kevin Harwell <kharwell@lunkwill>
+	  Merged revisions 429827 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* .lastclean: Update for 13.7.0-rc1
 
-2015-12-15 11:28 +0000 [034112c574]  Kevin Harwell <kharwell@lunkwill>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429828 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* realtime: Add database scripts for 13.7.0-rc1
+2014-12-18 16:40 +0000 [2cbfafa8c1]  Richard Mudgett <rmudgett@digium.com>
 
-2015-12-15 11:19 +0000 [d1f8ff1789]  Kevin Harwell <kharwell@lunkwill>
+	* chan_dahdi.c, res_rtp_asterisk.c: Change some spammy debug messages to level 5.
 
-	* .version: Update for 13.7.0-rc1
+	  ASTERISK-24337 #close
+	  Reported by: Rusty Newton
+	  ........
 
-2015-12-15 11:19 +0000 [9376488bef]  Kevin Harwell <kharwell@lunkwill>
+	  Merged revisions 429804 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* .lastclean: Update for 13.7.0-rc1
+	  Merged revisions 429805 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-12-15 11:19 +0000 [a894c9e7a9]  Kevin Harwell <kharwell@lunkwill>
 
-	* realtime: Add database scripts for 13.7.0-rc1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429806 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-15 11:12 +0000 [52afb0f112]  Kevin Harwell <kharwell@lunkwill>
+2014-12-18 14:09 +0000 [eacbb4ceb5]  Richard Mudgett
 
-	* .version: Update for 13.7.0-rc1
+	* chan_dahdi: Populate CALLERID(ani2) for incoming calls in featdmf signaling mode.
 
-2015-12-15 11:12 +0000 [2de343eb85]  Kevin Harwell <kharwell@lunkwill>
+	  For the featdmf signaling mode the incoming MF Caller-ID information is
+	  formatted as follows: *${CALLERID(ani2)}${CALLERID(ani)}#*${EXTEN}#
 
-	* .lastclean: Update for 13.7.0-rc1
+	  Rather than discarding the ani2 digits, populate the CALLERID(ani2) value
+	  with what is received instead.
 
-2015-12-15 11:12 +0000 [184de2a160]  Kevin Harwell <kharwell@lunkwill>
+	  AST-1368 #close
+	  Reported by: Denis Martinez
+	  Patches:
+	        extract_ani2_for_featdmf_v11.patch (license #5621) patch uploaded by Richard Mudgett
+	  ........
 
-	* realtime: Add database scripts for 13.7.0-rc1
+	  Merged revisions 429783 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-12-14 13:53 +0000 [24ae124e4f]  server-pandora <server-pandora@xencall.com>
+	  Merged revisions 429784 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_rtp_asterisk.c: Fix DTLS negotiation delays.
 
-	  - Trigger pending DTLS packets to send out, once the RTP instance's remote
-	    address is set.
-	  - Avoids locking the DTLS structure unnecessarily by only doing this if
-	    DTLS is passive.
-	  - Add DTLS locks around the structurally sensitive calls in the SSL
-	    portion of __rtp_recvfrom, since dtls_srtp_check_pending does not lock
-	    inside of itself, and we're dealing with the SSL BIO in at least two
-	    threads.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429785 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  WebRTC channels may receive a DTLS handshake before
-	  ast_rtp_remote_address_set is called, which causes there to be a pending
-	  response to send out.   Previous to 1ad827, this was handled by calling
-	  dtls_srtp_check_pending on receipt of any RTP packet - a STUN or RTP
-	  packet could trigger the pending handshake response.  Since that was
-	  rightfully removed, whenever the DTLS handshake is received before the
-	  remote address is set, we would have to wait until another SSL packet
-	  arrives.
+2014-12-18 09:55 +0000 [546a54574f]  Kevin Harwell <kharwell@digium.com>
 
-	  As of Chrome M47's optimizations to their handshake process, WebRTC
-	  conversations between Chrome M47+ and Asterisk, where Asterisk is passive,
-	  experience a 1 second delay without this patch, because the SSL handshake
-	  is received before ICE negotation stores the remote_address, and the next
-	  SSL packet isn't received until after a 1 second timeout in Chrome, which
-	  causes a new handshake request.
+	* res_pjsip_sdp_rtp: wrong bridge chosen when the DTMF mode is not compatible
 
-	  ASTERISK-25614 #close
+	  A native rtp bridge was being chosen (it shouldn't have been) when using two
+	  pjsip channels with incompatible DTMF modes.  This patch sets the rtp instance
+	  property, AST_RTP_PROPERTY_DTMF, for the appropriate DTMF mode(s) for pjsip.
+	  It was not being set before, meaning all DTMF modes for pjsip were being treated
+	  as compatible, thus native bridging would be chosen as the bridge type when it
+	  shouldn't have been.
 
-	  Change-Id: I547f1be7e302dbf71f6553dd8cbc0657b1d0b908
+	  ASTERISK-24459 #close
+	  Reported by: Yaniv Simhi
+	  Review: https://reviewboard.asterisk.org/r/4265/
+	  ........
 
-2015-12-14 15:25 +0000 [36097a185d]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 429763 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Fix sscanf() format string type mismatch.
 
-	  ASTERISK-25615
-	  Reported by: George Joseph
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429764 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ieff35307254ca193f3d473cff2e396ca57c7ce0b
+2014-12-18 09:40 +0000 [2f3e5b494a]  Mark Michelson <mmichelson@digium.com>
 
-2015-12-13 13:13 +0000 [94f9927784]  Matt Jordan <mjordan@digium.com>
+	* Prevent potential infinite outbound authentication loops in registration.
 
-	* main/utils: Don't emit an ERROR message if the read end of a pipe closes
+	  Prior to this patch, Asterisk would always respond to 401 responses to
+	  registration attempts by trying to provide a registration with authentication
+	  credentials. Even if subsequent attempts were rejected with 401 responses,
+	  Asterisk would continue this behavior. If authentication credentials were
+	  incorrect, this could continue forever.
 
-	  An ERROR or WARNING message should generally indicate that something has gone
-	  wrong in Asterisk. In the case of writing to a file descriptor, Asterisk is not
-	  in control of when the far end closes its reading on a file descriptor. If the
-	  far end does close the file descriptor in an unclean fashion, this isn't a bug
-	  or error in Asterisk, particularly when the situation can be gracefully
-	  handled in Asterisk.
+	  With this patch, we keep track of whether we have attempted authentication
+	  on an outbound registration attempt. If we already have, we don not try
+	  again until the next attempt. This prevents the infinite loop scenario.
 
-	  Currently, when this happens, a user would see the following somewhat cryptic
-	  ERROR message:
+	  Review: https://reviewboard.asterisk.org/r/4273
+	  ........
 
-	    "utils.c: write() returned error: Broken pipe"
+	  Merged revisions 429761 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  There's a few problems with this:
-	  (1) It doesn't provide any context, other than 'something broke a pipe'
-	  (2) As noted, it isn't actually an error in Asterisk
-	  (3) It can get rather spammy if the thing breaking the pipe occurs often, such
-	      as a FastAGI server
-	  (4) Spammy ERROR messages make Asterisk appear to be having issues, or can even
-	      mask legitimate issues
 
-	  This patch changes ast_carefulwrite to only log an ERROR if we actually had one
-	  that was reasonably under our control. For debugging purposes, we still emit
-	  a debug message if we detect that the far side has stopped reading.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429762 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ia503bb1efcec685fa6f3017bedf98061f8e1b566
+2014-12-18 09:18 +0000 [2b1f2b5c1f]  Mark Michelson <mmichelson@digium.com>
 
-2015-12-12 11:08 +0000 [5b867fa904]  gtjoseph <george.joseph@fairview5.com>
+	* Prevent possible race condition on dual redirect of channels in the same bridge.
 
-	* pjsip/config_transport: Check pjproject version at runtime for async ops
+	  The AST_FLAG_BRIDGE_DUAL_REDIRECT_WAIT flag was created to prevent bridges from
+	  prematurely acting on orphaned channels in bridges. The problem with the AMI
+	  redirect action was that it was setting this flag on channels based on the presence
+	  of a PBX, not whether the channel was in a bridge. Whether a channel has a PBX
+	  is irrelevant, so the condition has been altered to check if the channel is in a
+	  bridge.
 
-	  pjproject < 2.5.0 will segfault on a tls transport if async_operations
-	  is greater than 1.  A runtime version check has been added to throw
-	  an error if the version is < 2.5.0 and async_operations > 1.
+	  ASTERISK-24536 #close
+	  Reported by Niklas Larsson
 
-	  To assist in the check, a new api "ast_compare_versions" was added
-	  to utils which compares 2 major.minor.patch.extra version strings.
+	  Review: https://reviewboard.asterisk.org/r/4268
+	  ........
 
-	  ASTERISK-25615 #close
+	  Merged revisions 429741 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I8e88bb49cbcfbca88d9de705496d6f6a8c938a98
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
 
-2015-12-10 11:44 +0000 [14b41115e3]  Jonathan Rose <jrose@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429745 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: Add TCP/TLS keepalive to TCP/TLS server
+2014-12-18 08:50 +0000 [cc1405bd38]  Mark Michelson <mmichelson@digium.com>
 
-	  Adds the TCP Keep Alive option to TCP and TLS server sockets. Previously
-	  this option was only being set on session sockets.
-	  http://www.tldp.org/HOWTO/html_single/TCP-Keepalive-HOWTO/
-	  According to the link above, the SO_KEEPALIVE option is useful for knowing
-	  when a TCP connected endpoint has severed communication without indicating
-	  it or has become unreachable for some reason. Without this patch, keep
-	  alive is not set on the socket listening for incoming TCP sessions and
-	  in Komatsu's report this resulted in the thread listening for TCP becoming
-	  stuck in a waiting state.
+	* Ensure the correct value is returned for CHANNEL(pjsip, secure)
 
-	  ASTERISK-25364 #close
-	  Reported by: Hiroaki Komatsu
+	  Prior to this patch, we were using the PJSIP dialog's secure flag
+	  to determine if a secure transport was being used. Unfortunately,
+	  the dialog's secure flag was only set if a SIPS URI were in use,
+	  as required by RFC 3261 sections 12.1.1 and 12.1.2. What we're interested
+	  in is not dialog security, but transport security. This code change
+	  switches to a model where we use the dialog's target URI to determine
+	  what transport would be used to communicate, and then check if that
+	  transport is secure.
 
-	  Change-Id: I7ed7bcfa982b367dc64b4b73fbd962da49b9af36
-2015-12-09 09:48 +0000 [cd119ed4a2]  Tyler Cambron <tcambron@digium.com>
+	  AST-1450 #close
+	  Reported by John Bigelow
 
-	* res_chan_stats: Fix bug to send correct statistics to StatsD
+	  Review: https://reviewboard.asterisk.org/r/4277
+	  ........
 
-	  Fixed a bug that originally would show a negative number of
-	  active calls occuring in Asterisk. A gauge is persistent so
-	  incrementing and decrementing it results in a more consistent
-	  performance. Also changed to the call to StatsD to use
-	  ast_statsd_log_string() so that a "+" could be sent to StatsD.
+	  Merged revisions 429739 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25619 #close
 
-	  Change-Id: Iaaeff5c4c6a46535366b4d16ea0ed0ee75ab2ee7
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429740 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-07 13:07 +0000 [ddf4dddf4f]  Corey Farrell <git@cfware.com>
+2014-12-17 18:11 +0000 [18b5a336ef]  gtjoseph <george.joseph@fairview5.com>
 
-	* app_meetme: Set default value for audio_buffers.
+	* res_pjsip_config_wizard: fix unload SEGV
 
-	  The default value was never set for audio_buffers, causing bad
-	  audio quality.  This ensures the default is always set.
+	  If certain pjsip modules aren't loaded, the wizard causes a SEGV
+	  when it unloads.  Added a check for the presense of the object
+	  type wizard before trying to clean it up.
 
-	  ASTERISK-25569 #close
+	  Tested-by: George Joseph
+	  ........
 
-	  Change-Id: I2d2ee3e644120b0f9f6ea6ab9286d7d590942a44
-2015-12-08 01:57 +0000 [142d4fefb8]  Filip Jenicek <phill@janevim.cz>
+	  Merged revisions 429719 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip: Check sip_pvt pointer in ast_channel_get_t38_state(c)
 
-	  Asterisk may crash when calling ast_channel_get_t38_state(c)
-	  on a locked channel which is being hung up.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429720 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25609 #close
+2014-12-17 17:06 +0000 [c4360796f7]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Ifaa707c04b865a290ffab719bd2e5c48ff667c7b
+	* res_pjsip_config_wizard: Change FILEUNCHANGED config_load2 flag determination
 
-2015-12-08 17:49 +0000 [21962dad93]  gtjoseph <george.joseph@fairview5.com>
+	  The module now applies the FILEUNCHANGED flag when both reloaded is
+	  specified AND there's no last_config for the object type.
 
-	* res_pjsip:  Add existence and readablity checks for tls related files
+	  Tested-by: George Joseph
 
-	  Both transport and endpoint now check for the existence and readability
-	  of tls certificate and key files before passing them on to pjproject.
-	  This will cause the object to not load rather than waiting for pjproject
-	  to discover that there's a problem when a session is attempted.
+	  Review: https://reviewboard.asterisk.org/r/4276/
+	  ........
 
-	  NOTE: chan_sip also uses ast_rtp_dtls_cfg_parse but it's located
-	  in build_peer which is gigantic and I didn't want to disturb it.
-	  Error messages will emit but it won't interrupt chan_sip loading.
+	  Merged revisions 429699 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25618 #close
 
-	  Change-Id: Ie43f2c1d653ac1fda6a6f6faecb7c2ebadaf47c9
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429700 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-12-02 12:42 +0000 [28d9243079]  Eugene Voityuk <eugene@thirdlane.com>
+2014-12-17 04:23 +0000 [8b6ecc449c]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* chan_sip.c: Start ICE negotiation when response is sent or received.
+	* Fix printf problems with high ascii characters after r413586 (1.8).
 
-	  The current logic for ICE negotiation starts it
-	  when receiving an SDP with ICE candidates. This is
-	  incorrect as ICE negotiation can only start when each 
-	  call party have at least one pair of local and remote 
-	  candidate. Starting ICE negotiation early would result 
-	  in negotiation failure and ultimately no audio.
+	  In r413586 (1.8) various casts were added to silence gcc 4.10 warnings.
+	  Those fixes included things like:
 
-	  This change makes it so ICE negotiation is only started
-	  when a response with SDP is received or when a response
-	  with SDP is sent.
+	      -out += sprintf(out, "%%%02X", (unsigned char) *ptr);
+	      +out += sprintf(out, "%%%02X", (unsigned) *ptr);
 
-	  ASTERISK-24146
+	  That works for low ascii characters, but for the high range that yields
+	  e.g. FFFFFFC3 when C3 is expected.
 
-	  Change-Id: I55a632bde9e9827871b09141d82747e08379a8ca
-2015-12-08 11:03 +0000 [e03582a1c2]  gtjoseph <george.joseph@fairview5.com>
+	  This changeset:
+	  - fixes those casts to use the 'hh' unsigned char modifier instead
+	  - consistently uses %02x instead of %2.2x (or other non-standard usage)
+	  - adds a few 'h' modifiers in various places
+	  - fixes a 'replcaes' typo
+	  - dev/urandon typo (in 13+ patch)
 
-	* res_pjsip/config_transport: Prevent async_operations > 1 when protocol = tls
+	  Review: https://reviewboard.asterisk.org/r/4263/
 
-	  See ASTERISK-25615.
-	  If the transport protocol is tls and async_operations > 1, pjproject
-	  will segfault if more than one operation is attempted on the same socket.
-	  Until this is fixed upstream, a check has been added to throw an error
-	  if a tls transport config has async_operations set to > 1.
+	  ASTERISK-24619 #close
+	  Reported by: Stefan27 (on IRC)
+	  ........
 
-	  ASTERISK-25615
+	  Merged revisions 429673 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I76b9a5b2a5a0054fe71ca5851e635f2dca7685a6
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
+	  Merged revisions 429674 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-12-08 08:39 +0000 [876600ce6e]  Alexander Traud <pabstraud@compuserve.com>
+	  Merged revisions 429675 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* codec_resample: Increase buffer for Opus Codec with FEC.
 
-	  ASTERISK-25599 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429683 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Idbd187f711b2ec63dda949ca0f79aa0c1a0a0b6e
+2014-12-16 11:53 +0000 [c4cc668ba9]  gtjoseph <george.joseph@fairview5.com>
 
-2015-12-08 03:46 +0000 [69e3d40ad7]  Alexander Traud <pabstraud@compuserve.com>
+	* res_pjsip_config_wizard: fix test breakage
 
-	* translate: Avoid a warning message when doing FEC within Opus Codec.
+	  Fix test breakage caused by not checking for res_pjsip before
+	  calling ast_sip_get_sorcery.
 
-	  ASTERISK-25616 #close
+	  Tested-by: George Joseph
 
-	  Change-Id: Ibe729aaf2e6e25506cff247cec5149ec1e589319
+	  Review: https://reviewboard.asterisk.org/r/4269/
+	  ........
 
-2015-12-04 15:36 +0000 [2b992014dc]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 429653 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip: Fix crash involving the bogus peer during sip reload.
 
-	  A crash happens sometimes when performing a CLI "sip reload".  The bogus
-	  peer gets refreshed while it is in use by a new call which can cause the
-	  crash.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429654 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Protected the global bogus peer object with an ao2 global object
-	  container.
+2014-12-16 10:39 +0000 [58095d2486]  Andreas Steinmetz (license 6523)
 
-	  ASTERISK-25610 #close
+	* chan_sip: Allow T.38 switch-over when SRTP is in use.
 
-	  Change-Id: I5b528c742195681abcf713c6e1011ea65354eeed
+	  Previously when SRTP was enabled on a channel it was not possible
+	  to switch to T.38 as no crypto attributes would be present.
 
-2015-12-06 16:32 +0000 [529535f0c2]  Matt Jordan <mjordan@digium.com>
+	  This change makes it so it is now possible. If a T.38 re-invite
+	  comes in SRTP is terminated since in practice you can't encrypt
+	  a UDPTL stream. Now... if we were doing T.38 over RTP (which
+	  does exist) then we'd have a chance but almost nobody does that so
+	  here we are.
 
-	* Revert "bridges/bridge_t38: Add a bridging module for managing T.38 state"
+	  ASTERISK-24449 #close
+	  Reported by: Andreas Steinmetz
+	  patches:
+	   udptl-ignore-srtp-v2.patch submitted by Andreas Steinmetz (license 6523)
+	  ........
 
-	  This reverts commit 6614babea27fbafbe11820ea03737dd5c4f9ecec.
+	  Merged revisions 429632 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Unfortunately, using a bridge to manage T.38 state will cause severe deadlocks
-	  in core_unreal/chan_local. Local channels attempt to reach across both their
-	  peer and the peer's bridge to inspect T.38 state. Given the propensity of
-	  Local channel chains, managing the locking situation in such a scenario is
-	  practically infeasible.
+	  Merged revisions 429633 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ic687397ffea08dfb899345a443bd990ec3d0416a
 
-2015-12-04 16:23 +0000 [450579e908]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429634 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip/contacts/statsd:  Make contact lifecycle events more consistent
+2014-12-16 09:44 +0000 [b5182a6795]  Joshua Colp <jcolp@digium.com>
 
-	  It will never be perfect or even pretty, mostly because of the differences
-	  between static and dynamic contacts.
+	* res_pjsip_t38: Fix T.38 failure when peer reinvites immediately.
 
-	  Created:
+	  If a remote endpoint reinvites to T.38 immediately the state machine
+	  will go into a peer reinvite state. If a T.38 capable application
+	  (such as ReceiveFax) queries it will receive this state. Normally
+	  the application will then indicate so that the channel driver will
+	  queue up the T.38 offer previously received. Once it receives this
+	  offer the application will act normally and negotiate.
 
-	  Can't use the contact or contact_status alloc functions
-	  because the objects come and go regardless of the actual state.
+	  The res_pjsip_t38 module incorrectly partially squashed this indication.
+	  This would cause the application to think the request had failed when
+	  in reality it had actually worked.
 
-	  Can't use the contact_apply_handler, ast_sip_location_add_contact or
-	  a sorcery created handler because they only get called for dynamic
-	  contacts.  Similarly, permanent_uri_handler only gets called for
-	  static contacts.
+	  This change makes it so that no T.38 control frames (or indications)
+	  are squashed.
+	  ........
 
-	  So, Matt had it right. :)  ast_res_pjsip_find_or_create_contact_status is
-	  the only place it can go and not have duplicated code.  Both
-	  permanent_uri_handler and contact_apply_handler call find_or_create.
+	  Merged revisions 429612 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Removed:
 
-	  Can't use the destructors for the same reason as above.  The only
-	  place to put this is in persistent_endpoint_contact_deleted_observer
-	  which I believe is the "correct" place but even that will handle only
-	  dynamic contacts.  This doesn't called on shutdown however.  There is
-	  no hook to use for static contacts that may be removed because of a
-	  config change while asterisk is in operation.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429613 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  I moved the cleanup of contact_status from ast_sip_location_delete_contact
-	  to the handler as well.
+2014-12-15 11:08 +0000 [39b54a21dc]  gtjoseph <george.joseph@fairview5.com>
 
-	  Status Change and RTT:
+	* res_pjsip_config_wizard: Allow streamlined config of common pjsip scenarios
 
-	  Although they worked fine where they were (in update_contact_status) I
-	  moved them to persistent_endpoint_contact_status_observer to make it
-	  more consistent with removed.  There was logic there already to detect
-	  a state change.
+	  res_pjsip_config_wizard
+	  ------------------
+	   * This is a new module that adds streamlined configuration capability for
+	     chan_pjsip.  It's targetted at users who have lots of basic configuration
+	     scenarios like 'phone' or 'agent' or 'trunk'.  Additional information
+	     can be found in the sample configuration file at
+	     config/samples/pjsip_wizard.conf.sample.
 
-	  Finally, fixed a nit in permanent_uri_handler rmudgett reported
-	  eralier.
+	  Tested-by: George Joseph
 
-	  ASTERISK-25608 #close
+	  Review: https://reviewboard.asterisk.org/r/4190/
+	  ........
 
-	  Change-Id: I4b56e7dfc3be3baaaf6f1eac5b2068a0b79e357d
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
+	  Merged revisions 429592 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-21 06:02 +0000 [5a18193dc0]  Alexander Traud <pabstraud@compuserve.com>
 
-	* res_format_attr_vp8: In SDP, forward max-fr and max-fs for video-codec VP8.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429593 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25584 #close
+2014-12-15 09:48 +0000 [53e5b377a0]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: Iae00071b4ff1ae76f24995aeac4d00284fd14f91
+	* Activate persistent subscriptions when they are recreated.
 
-2015-11-21 05:21 +0000 [3e2178c05e]  Alexander Traud <pabstraud@compuserve.com>
+	  Prior to this change, recreating persistent subscriptions would
+	  create the subscription but would not activate it. This led to subscriptions
+	  being listed in the "NULL" state by diagnostics and not sending NOTIFYs
+	  when expected.
 
-	* res_format_attr_opus: Update to latest RFC 7587.
+	  Review: https://reviewboard.asterisk.org/r/4261
+	  ........
 
-	  Beside that, the format-attribute module sends only non-default values in the
-	  line fmtp, now. This avoids unnecessary overhead in SDP messages. Furthermore,
-	  previously the parameter stereo was not parsed when being the first parameter.
+	  Merged revisions 429571 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25583 #close
 
-	  Change-Id: Iae85ba3e5960bfd5d51cf65bcffad00dd4875a73
-2015-12-02 14:11 +0000 [072d94183c]  Jonathan Rose <jrose@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429573 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Fix crash in audiohook translate to slin
+2014-12-12 17:57 +0000 [6472568bc6]  gtjoseph <george.joseph@fairview5.com>
 
-	  This patch fixes a crash which would occur when an audiohook was
-	  applied to a channel using an audio codec that could not be translated
-	  to signed linear (such as when using pass-through codecs like OPUS or
-	  when the codec translator module for the format in use is not loaded).
+	* loader: Move definition of ast_module_reload from _private.h to module.h
 
-	  ASTERISK-25498 #close
-	  Reported by: Ben Langfeld
+	  No functionality change.  Just move the definition of ast_module_reload
+	  from _private.h to module.h so it can be public.
 
-	  Change-Id: Ib6ea7373fcc22e537cad373996136636201f4384
+	  Also removed the include of _private.h from manager.c since ast_module_load
+	  was the only reason for including it.
 
-2015-12-03 12:07 +0000 [9184fbeb34]  gtjoseph <george.joseph@fairview5.com>
+	  Tested-by: George Joseph
 
-	* res_pjsip: Use a MD5 hash for static Contact IDs
+	  Review: https://reviewboard.asterisk.org/r/4251/
+	  ........
 
-	  When 90d9a70789 was merged, it mostly tested dynamic contacts created as
-	  a result of registering a PJSIP endpoint. Contacts generated in this
-	  fashion typically have a long alphanumeric string as their object identifier,
-	  which maps reasonably well for StatsD. Unfortunately, this doesn't work in the
-	  general case. StatsD treats both '.' and ':' characters as special characters.
-	  In particular, having a ':' appear in the middle of a StatsD metric will
-	  result in the metric being rejected.
+	  Merged revisions 429542 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This causes some obvious issues with SIP URIs.
 
-	  The StatsD API should not be responsible for escaping the metric name passed
-	  to it. The metric is treated as a single long string, and it would be
-	  challenging to know what to escape in the string passed to the function.
-	  Likewise, we don't want to escape the metric in PJSIP, as that involves
-	  overhead that is wasted when either res_statsd isn't loaded or enabled.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429543 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch takes an alternative approach. The Contact ID has been changed
-	  to be "aor@@uri_hash" instead of "aor@@uri". This (a) won't contain any of the
-	  aforementioned special characters, (b) can be done on Contact creation,
-	  which has minimal impact on run-time performance, and (c) also conforms to an
-	  earlier commit that changed the ID for dynamic contacts.
+2014-12-12 17:49 +0000 [308c1b41dd]  Richard Mudgett <rmudgett@digium.com>
 
-	  The downside of this is that StatsD users will have to map SHA1 hashes back to
-	  the Contacts that are emitting the statistics. To that end, the CLI commands
-	  have been updated to include the first 10 characters of the MD5 hash, which
-	  should be enough to match what is shown in Graphite (or some other StatsD
-	  backend).
+	* DEBUG_THREADS: Fix regression and lock tracking initialization problems.
 
-	  ASTERISK-25595 #close
+	  This patch started with David Lee's patch at
+	  https://reviewboard.asterisk.org/r/2826/ and includes a regression fix
+	  introduced by the ASTERISK-22455 patch.
 
-	  Change-Id: Ic674a3307280365b4a45864a3571c295b48a01e2
-	  Reported-by: Matt Jordan
-	  Tested-by: George Joseph
+	  The initialization of a mutex's lock tracking structure was not protected
+	  in a critical section.  This is fine for any mutex that is explicitly
+	  initialized, but a static mutex may have its lock tracking double
+	  initialized if multiple threads attempt the first lock simultaneously.
 
-2015-11-30 22:19 +0000 [ed9134282e]  gtjoseph <george.joseph@fairview5.com>
+	  * Added a global mutex to properly serialize initialization of the lock
+	  tracking structure.  The painful global lock can be mitigated by adding a
+	  double checked lock flag as discussed on the original review request.
 
-	* res_pjsip:  Update logging to show contact->uri in messages
+	  * Defer lock tracking initialization until first use.
 
-	  An earlier commit changed the id of dynamic contacts to contain
-	  a hash instead of the uri.  This patch updates status change
-	  logging to show the aor/uri instead of the id.  This required
-	  adding the aor id to contact and contact_status and adding
-	  uri to contact_status.  The aor id gets added to contact and
-	  contact_status in their allocators and the uri gets added to
-	  contact_status in pjsip_options when the contact_status is
-	  created or updated.
+	  * Don't be "helpful" and initialize an uninitialized lock when
+	  DEBUG_THREADS is enabled.  Debug code is not supposed to fix or change
+	  normal code behavior.  We don't need a lock initialization race that would
+	  force a re-setup of lock tracking.  Lock tracking already handles
+	  initialization on first use.
 
-	  ASTERISK-25598 #close
+	  * Properly handle allocation failures of the lock tracking structure.
 
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
+	  * No need to initialize tracking data in __ast_pthread_mutex_destroy()
+	  just to turn around and destroy it.
 
-	  Change-Id: I56cbec1d2ddbe8461367dd8b6da8a6f47f6fe511
 
-2015-12-01 16:11 +0000 [eadad24b59]  Jonathan Rose <jrose@digium.com>
+	  The regression introduced by ASTERISK-22455 is the result of manipulating
+	  a pthread_mutex_t struct outside of the pthread library code.  The
+	  pthread_mutex_t struct seems to have a global linked list pointer member
+	  that can get changed by other threads.  Therefore, saving and restoring
+	  the contents of a pthread_mutex_t struct is a bad thing.
 
-	* Unset BRIDGEPEER when leaving a bridge
+	  Thanks to Thomas Airmont for finding this obscure regression.
 
-	  Currently if a channel is transferred out of a bridge, the BRIDGEPEER
-	  variable (also BRIDGEPVTCALLID) remain set even once the channel is
-	  out of the bridge. This patch removes these variables when leaving
-	  the bridge.
+	  * Don't overwrite the struct ast_lock_track.reentr_mutex member to restore
+	  tracking data in __ast_cond_wait() and __ast_cond_timedwait().  The
+	  pthread_mutex_t struct must be treated as a read-only opaque variable.
 
-	  ASTERISK-25600 #close
-	  Reported by: Mark Michelson
 
-	  Change-Id: I753ead2fffbfc65427ed4e9244c7066610e546da
+	  Miscellaneous other items fixed by this patch:
 
-2015-11-30 14:22 +0000 [bb0b60619d]  Richard Mudgett <rmudgett@digium.com>
+	  * Match ast_suspend_lock_info() with ast_restore_lock_info() in
+	  __ast_cond_timedwait().
 
-	* res_sorcery_memory_cache.c: Fix off nominal ref leak.
+	  * Made some uninitialized lock sanity checks return EINVAL and try a
+	  DO_THREAD_CRASH.
 
-	  Change-Id: If83d63cf11cbc6df9b15251848b01feb570ade49
+	  * Fix bad canlog initialization expressions.
 
-2015-11-30 16:42 +0000 [e7c88e11aa]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24614 #close
+	  Reported by: Thomas Airmont
 
-	* sched.c: Make not return a sched id of 0.
+	  Review: https://reviewboard.asterisk.org/r/4247/
+	  Review: https://reviewboard.asterisk.org/r/2826/
+	  ........
 
-	  According to the API doxygen a sched ID of 0 is valid.  Unfortunately, 0
-	  was never returned historically and several users incorrectly coded usage
-	  of the returned sched ID assuming that 0 was invalid.
+	  Merged revisions 429539 from http://svn.asterisk.org/svn/asterisk/branches/11
 
-	  ASTERISK-25476
 
-	  Change-Id: Ib19c7ebb44ec9fd393ef6646dea806d4f34e3a20
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429541 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-11-25 12:23 +0000 [4aed349a7b]  Richard Mudgett <rmudgett@digium.com>
+2014-12-12 16:54 +0000 [901221ffae]  Matt Jordan <mjordan@digium.com>
 
-	* Audit improper usage of scheduler exposed by 5c713fdf18f. (v13 additions)
+	* res/res_agi: Make Verbose message for 'stream file' match other playbacks
 
-	  chan_sip.c:
-	  * Initialize mwi subscription scheduler ids earlier because of ASTOBJ to
-	  ao2 conversion.
+	  The Verbose message displayed when a file is played back via 'stream file'
+	  was formatted differently than other playbacks:
+	  * It didn't include the channel name
+	  * It didn't include the channel language
+	  It does, however, include the playback offset as well as any escape digits.
+	  That information was kept; however, this patch updates the formatting to more
+	  closely match the Verbose messages displayed when a file is played back by
+	  'control stream file', Playback, ControlPlayback, or any other file playback
+	  operation.
+	  ........
 
-	  * Initialize register scheduler ids earlier because of ASTOBJ to ao2
-	  conversion.
+	  Merged revisions 429519 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  chan_skinny.c:
-	  * Fix more scheduler usage for the valid 0 id value.
 
-	  ASTERISK-25476
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429520 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: If9f0e5d99638b2f9d102d1ebc9c5a14b2d706e95
+2014-12-12 11:01 +0000 [8d325be503]  Joshua Colp <jcolp@digium.com>
 
-2015-11-24 12:44 +0000 [6d9156d10f]  Richard Mudgett <rmudgett@digium.com>
+	* media: Fix crash when determining sample count of a frame during shutdown.
 
-	* Audit improper usage of scheduler exposed by 5c713fdf18f.
+	  When shutting down Asterisk the codecs are cleaned up. As a result anything
+	  attempting to get a codec based on ID or details will find that no codec
+	  exists. This currently occurs when determining the sample count of a frame.
+	  This code did not take this situation into account.
 
-	  channels/chan_iax2.c:
-	  * Initialize struct chan_iax2_pvt scheduler ids earlier because of
-	  iax2_destroy_helper().
+	  This change fixes this by getting the codec directly from the format and
+	  eliminates the lookup. This is both faster and also provides a guarantee
+	  that the codec will exist and will be valid.
 
-	  channels/chan_sip.c:
-	  channels/sip/config_parser.c:
-	  * Fix initialization of scheduler id struct members.  Some off nominal
-	  paths had 0 as a scheduler id to be destroyed when it was never started.
+	  ASTERISK-24604 #close
+	  Reported by: Matt Jordan
 
-	  chan_skinny.c:
-	  * Fix some scheduler id comparisons that excluded the valid 0 id.
+	  Review: https://reviewboard.asterisk.org/r/4260/
+	  ........
 
-	  channel.c:
-	  * Fix channel initialization of the video stream scheduler id.
+	  Merged revisions 429497 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  pbx_dundi.c:
-	  * Fix channel initialization of the packet retransmission scheduler id.
 
-	  ASTERISK-25476
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429498 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I07a3449f728f671d326a22fcbd071f150ba2e8c8
+2014-12-12 09:31 +0000 [72499dc697]  Kevin Harwell <kharwell@digium.com>
 
-2015-12-01 07:55 +0000 [b76c196e13]  Alexander Traud <pabstraud@compuserve.com>
+	* chan_pjsip: Race between channel answer and bridge setup when using direct media
 
-	* codec_resample: Increase buffer for Opus Codec.
+	  When direct media is enabled and a pjsip channel is answered a race would occur
+	  between the handling of the answer and bridge setup. Sometimes the media
+	  negotiation would take place after the native bridge was setup. This resulted
+	  in a NULL media address, which in turn resulted in Asterisk using its address
+	  as the remote media address when sending a reinvite.  This patch makes the
+	  chan_pjsip answer handler synchronous thus alleviating the race condition (the
+	  bridge won't start setting things up until after it returns).
 
-	  ASTERISK-25599 #close
+	  ASTERISK-24563 #close
+	  Reported by: Steve Pitts
+	  Review: https://reviewboard.asterisk.org/r/4257/
+	  ........
 
-	  Change-Id: I1f88a88c59fb4e1e62bbdbb100c7152d48e73f10
+	  Merged revisions 429477 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-28 08:46 +0000 [6614babea2]  Matt Jordan <mjordan@digium.com>
 
-	* bridges/bridge_t38: Add a bridging module for managing T.38 state
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429478 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When 4875e5ac32 was merged, it fixed several issues with a direct media bridge
-	  transitioning to handling a T.38 fax. However, it uncovered a race condition
-	  caused by the bridging core. When a channel involved in a T.38 fax leaves a
-	  bridge, the frame queued by the channel driver that should inform the far side
-	  that it is no longer in a T.38 fax may not make it across the bridge. The
-	  bridging framework is *extremely* aggressive in tearing down the bridge, and
-	  control frames that are currently in flight *may* get dropped.
+2014-12-12 09:03 +0000 [2e6d2b1484]  David M. Lee <dlee@digium.com>
 
-	  This patch adds a new module to the bridging framework, bridge_t38. This module
-	  maintains some notion of the T.38 state for the two channels in a bridge. When
-	  the bridge detects that it is being torn down or when one of the two channels
-	  leaves, it informs the respective channel(s) that they should stop faxing. This
-	  ensures that channels switch back to audio if they survive and are ejected out
-	  of a bridge while faxing.
+	* Fix crash for sorcery misconfigs
 
-	  ASTERISK-25582
+	  res_pjsip_outbound_publish was missing the CHECK_PJSIP_MODULE_LOADED()
+	  call in load_module, and would crash with a segfault if res_pjsip
+	  declined to load.
 
-	  Change-Id: If5b0bb478eb01c4607c9f4a7fc17c7957d260ea0
+	  Review: https://reviewboard.asterisk.org/r/4258/
+	  ........
 
-2015-11-27 07:39 +0000 [3fcf160fae]  Niklas Larsson <niklas@tese.se>
+	  Merged revisions 429457 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* CHANGES: Fix a typo
 
-	  Change-Id: Iceb3d9bb78140c376174a7bee197dfcf8ef9cda7
-2015-11-25 15:26 +0000 [45efbf8503]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429458 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* fastagi: record file closed after sending result
+2014-12-12 08:12 +0000 [a6cf13f2e9]  Kinsey Moore <kmoore@digium.com>
 
-	  The fastagi record-file testsuite test sometimes fails reporting an empty
-	  recorded file. This was happening because Asterisk was sending the agi result
-	  notification prior to actually closing the file and the data, being buffered,
-	  had not been written to the file yet when the test attempts to check the file
-	  size.
+	* PJSIP: Allow use of 'inactive' streams for hold
 
-	  This patch makes it so the record file stream is closed prior to sending the
-	  agi result notification.
+	  This allows use of the 'inactive' stream direction identifier to be
+	  used for hold where 'sendonly' is normally used. Some Seimens phones
+	  use 'inactive' and this change allows music on hold to operate
+	  properly.
 
-	  ASTERISK-25593 #close
+	  Review: https://reviewboard.asterisk.org/r/4252/
+	  Reported by: Steve Pitts
+	  ........
 
-	  Change-Id: I6b2b3be3ae37f7c7b18e672c419a89b3b8513cde
+	  Merged revisions 429432 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-11-25 13:29 +0000 [b2787876d6]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Merged revisions 429433 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main: Slight refactor of main. Improve color situation.
 
-	  Several issues are addressed here:
-	  - main() is large, and half of it is only used if we're not rasterisk;
-	    fixed by spliting up the daemon part into a separate function.
-	  - Call ast_term_init from rasterisk as well.
-	  - Remove duplicate code reading/writing asterisk history file.
-	  - Attempt to tackle background color issues and color changes that
-	    occur. Tested by starting asterisk -c until the colors stopped
-	    changing at odd locations.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429434 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25585 #close
+2014-12-12 08:04 +0000 [b99770d4fe]  Kinsey Moore <kmoore@digium.com>
 
-	  Change-Id: Ib641a0964c59ef9fe6f59efa8ccb481a9580c52f
+	* Sorcery: Log when old config remains in use
 
-2015-11-24 13:54 +0000 [59881fbb99]  David M. Lee <dlee@respoke.io>
+	  This adds a log message notifying the user that a stale configuration
+	  is in place upon reload when a config object fails to load. This
+	  situation can end up causing confusion when the object failed to load
+	  but exists from a previous config load especially when the old config
+	  is significantly different from the new config.
 
-	* Fixed some typos
+	  Review: https://reviewboard.asterisk.org/r/4250/
+	  Reported by: Thomas Thompson
+	  ........
 
-	  Fixes some minor typos in the CHANGES file, plus an embarrasing typo in
-	  the StatsD API.
+	  Merged revisions 429429 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I9ca4858c64a4a07d2643b81baa64baebb27a4eb7
+	  Merged revisions 429430 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-24 13:07 +0000 [b75f587d15]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_notify: Fix CLI usage info
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429431 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The usage info for 'pjsip send notify' previously referenced the
-	  chan_sip configuration sip_notify.conf.  Fix this to reference
-	  the correct configuration pjsip_notify.conf.
+2014-12-12 07:06 +0000 [74d43977cf]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25590 #close
+	* res_pjsip_session: Delay sending BYE if a re-INVITE transaction is in progress.
 
-	  Change-Id: I3898271a8e8a8b1db201741e790ebe2c6bf5cdea
+	  Given the scenario where a PJSIP channel is in a native RTP bridge with direct
+	  media and the channel is then hung up the code will currently re-INVITE the channel
+	  back to Asterisk and send a BYE at the same time. Many SIP implementations dislike
+	  this greatly.
 
-2015-11-23 14:27 +0000 [fc45f4040d]  Richard Mudgett <rmudgett@digium.com>
+	  This change makes it so that if a re-INVITE transaction is in progress the BYE
+	  is queued to occur after the completion of the transaction (be it through normal
+	  means or a timeout).
 
-	* res_sorcery_realtime.c: Fix crash from NULL sorcery object type.
+	  Review: https://reviewboard.asterisk.org/r/4248/
+	  ........
 
-	  If the sorcery object type is not found a NULL is returned.
-	  Unfortunately, sorcery_realtime_filter_objectset() will crash after
-	  complaining about not finding the object type and saying to expect errors.
+	  Merged revisions 429409 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Use ao2_cleanup() instead of ao2_ref() to prevent the crash.
 
-	  ASTERISK-25165
-	  Reported by Corey Farrell
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429410 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ic3b64453ea3058cb68d5c26d97d4fe7b8eea2e97
+2014-12-12 06:32 +0000 [8d384f3825]  Joshua Colp <jcolp@digium.com>
 
-2015-11-20 21:08 +0000 [4875e5ac32]  Matt Jordan <mjordan@digium.com>
+	* res_pjsip_session: Fix issue where a declined media stream in a re-INVITE would fail SDP negotiation.
 
-	* chan_pjsip: Handle T.38 faxes with direct media bridges
+	  In the past the SDP negotiation within res_pjsip_session was made more tolerant of
+	  certain situations. The only case where SDP negotiation will fail is when a major
+	  error occurs during negotiation. Receiving an already declined media stream is
+	  not considered a major error.
 
-	  When a channel is in a direct media bridge, a re-INVITE may arrive that forces
-	  Asterisk to re-negotiate the media to a T.38 fax. When this occurs, the bridge
-	  must change its technology to a simple bridge, and re-INVITE the media back
-	  to Asterisk.
+	  When producing the local SDP the logic took this into account so on the initial INVITE
+	  the declined media stream did not cause an SDP negotiation failure. Unfortunately
+	  the logic for handling media streams with a handler did not mirror this logic and
+	  considered an already declined media stream an error and thus failed the SDP
+	  negotiation.
 
-	  Generally, this logic mostly already exists in Asterisk. However, prior to this
-	  patch, there were a few bugs:
-	  (1) The T.38 framehook currently prevents a channel capable of T.38 faxes from
-	      ever entering into a direct media bridge. This applies even when the only
-	      media being passed over the channel is audio. This patch fixes this bug
-	      by having the framehook specify that it defers caring about any frame type.
-	      This allows the channels to enter into a direct media bridge, which will
-	      be broken when a re-INVITE is received.
-	  (2) When a re-INVITE is received, nothing instructed the bridging layer to
-	      re-inspect the allowed bridging technology. This now occurs when either
-	      a re-INVITE is received from a peer, or when a response is received from
-	      the far end (that is, when the T.38 state changes to either
-	      T38_PEER_REINVITE or T38_LOCAL_REINVITE).
-	  (3) chan_pjsip needs to do a small amount of work to prevent a direct media
-	      bridge from being chosen when a T.38 session is in progress. When a T.38
-	      session supplement has a t38 datastore - which is added when we detect
-	      we should start thinking about T.38 on a channel - we now refuse a native
-	      RTP bridge.
-	  (4) When a BYE request is received, we don't terminate the T.38 session. If
-	      the other side of a T.38 fax survives the hangup (due to the 'g' flag
-	      in Dial, for example), we don't currently re-INVITE the media on the
-	      other channel back to audio. This patch now has res_pjsip_t38 intercept
-	      BYE requests and inform the far side that the T.38 session is terminated.
-	      This naturally causes the correct re-INVITEs to be sent.
+	  This change makes the logic between both situations match so only under major
+	  errors will the SDP negotiation fail.
 
-	  ASTERISK-25582
+	  ASTERISK-24607 #close
+	  Reported by: Matt Jordan
 
-	  Change-Id: Iabd6aa578e633d16e6b9f342091264e4324a79eb
+	  Review: https://reviewboard.asterisk.org/r/4254/
+	  ........
 
-2015-11-20 21:07 +0000 [2b94d9a10d]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 429407 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_pjsip_t38: Add debug statements
 
-	  This patch adds some debug statements to res_pjsip_t38. These statements help
-	  to determine which SDP negotiation callbacks are being executed, and, when
-	  a particular callback exits, why a callback may not have applied its logic
-	  to the local or remote SDP.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429408 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I61b3fb9183b7ebbb5da8e9f48b59a5d9d7042d77
+2014-12-11 14:32 +0000 [63d3f0af95]  Kevin Harwell <kharwell@digium.com>
 
-2015-10-22 09:44 +0000 [af288b2d96]  Matt Jordan <mjordan@digium.com>
+	* ARI/AMI: Include language in standard channel snapshot output
 
-	* main/cli: Use proper string methods to check existence of context/exten/app
+	  The CHANGES verbiage for the "language" addition had been put under the wrong
+	  release. This moves it to be under 13.1 to 13.2 changes.
 
-	  Because the context, extension, and application are stored in stringfields,
-	  checking for them being NULL doesn't work so well. This patch uses the
-	  appropriate string library call, ast_strlen_zero, to see if there is a value
-	  in the context/exten/app values.
+	  ASTERISK-24553
+	  Reported by: Matt Jordan
+	  ........
 
-	  Change-Id: Ie09623bfdf35f5a8d3b23dd596647fe3c97b9a23
+	  Merged revisions 429387 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-18 09:43 +0000 [d27aac0a9d]  Matt Jordan <mjordan@digium.com>
 
-	* res/res_endpoint_stats: Add module to emit endpoint StatsD statistics
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429388 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch adds a module that emits StatsD statistics about Asterisk
-	  endpoints. This includes:
-	   * A GUAGE statistic for endpoint states, tracking how many endpoints are in
-	     a particular state.
-	   * A GUAGE statistic for each endpoint, counting the number of channels
-	     currently associated with an endpoint.
+2014-12-11 07:53 +0000 [d64b9904fd]  Kinsey Moore <kmoore@digium.com>
 
-	  ASTERISK-25572
+	* Stasis: Update unittest for channel snapshots
 
-	  Change-Id: If7e1333c5aeda8d136850b30c2101c0ee1c97305
+	  This adjusts the unit test for channel snapshots to take the new
+	  language key into account.
+	  ........
 
-2015-11-18 10:07 +0000 [90d9a70789]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 429352 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip/pjsip_options: Add StatsD statistics for PJSIP contacts
 
-	  This patch adds the ability to send StatsD statistics related to the
-	  state of PJSIP contacts. This includes:
-	   * A GUAGE statistic measuring the count of contacts in a particular state.
-	     This measures how many contacts are reachable, unreachable, etc.
-	   * The RTT time for each contact, if those contacts are qualified. This
-	     provides StatsD engines useful time-based data about each contact.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429353 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25571
+2014-12-10 09:43 +0000 [e890f9f653]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: Ib8378d73afedfc622be0643b87c542557e0b332c
+	* ARI/AMI: Include language in standard channel snapshot output
 
-2015-11-13 10:34 +0000 [75097a0955]  Matt Jordan <mjordan@digium.com>
+	  Adding information about including "language" in the standard channel snapshot
+	  output to the CHANGES file. Note the actual source changes have already been
+	  previously committed.
 
-	* res/res_pjsip_outbound_registration: Add registration statistics for StatsD
+	  ASTERISK-24553
+	  Reported by: Matt Jordan
+	  ........
 
-	  This patch adds outbound registration statistics for StatsD. This includes
-	  the following:
-	   * A GUAGE metric for the overall count of outbound registrations.
-	   * A GUAGE metric for each state an outbound registration can be in. As the
-	     outbound registrations change state, the overall count of how many
-	     outbound registrations are in the particular state is changed.
+	  Merged revisions 429325 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  These statistics are particularly useful for systems with a large number of
-	  SIP trunks, and where measuring the change in state of the trunks is useful
-	  for monitoring.
+	  Merged revisions 429326 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25571
 
-	  Change-Id: Iba6ff248f5d1c1e01acbb63e9f0da1901692eb37
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429327 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-11-19 09:40 +0000 [8f71263e72]  Matt Jordan <mjordan@digium.com>
+2014-12-10 07:35 +0000 [03c94ef761]  Joshua Colp <jcolp@digium.com>
 
-	* res/res_pjsip_outbound_registration: Apply configuration on object type load
+	* res_http_websocket: Fix crash due to double freeing memory when receiving a payload length of zero.
 
-	  When Asterisk is configured to use a dynamic sorcery backend (such as
-	  res_sorcery_astdb) with 'registration' objects, it will fail to create the
-	  internal state objects associated with the registration objects on module
-	  load. This is due to nothing actually querying for the specific objects
-	  and calling their sorcery apply handler during module load.
+	  Frames with a payload length of 0 were incorrectly handled in res_http_websocket.
+	  Provided a frame with a payload had been received prior it was possible for a double
+	  free to occur. The realloc operation would succeed (thus freeing the payload) but be
+	  treated as an error. When the session was then torn down the payload would be
+	  freed again causing a crash. The read function now takes this into account.
 
-	  This patch fixes that by calling get_registrations in the sorcery observer's
-	  object_type_loaded handler. Doing this causes the sorcery backends to be
-	  asked for the current state of all registration objects, which causes the
-	  apply handler to be called and the internal run-time state to be created.
+	  This change also fixes assumptions made by users of res_http_websocket. There is no
+	  guarantee that a frame received from it will be NULL terminated.
 
-	  ASTERISK-25575 #close
+	  ASTERISK-24472 #close
+	  Reported by: Badalian Vyacheslav
 
-	  Change-Id: Ie9306e797098c6d4da7bcf4a5434a15891508b23
+	  Review: https://reviewboard.asterisk.org/r/4220/
+	  Review: https://reviewboard.asterisk.org/r/4219/
+	  ........
 
-2015-11-11 11:51 +0000 [0b508789ab]  Alexander Traud <pabstraud@compuserve.com>
+	  Merged revisions 429270 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* translate: Provide translation modules the result of SDP negotiation.
+	  Merged revisions 429272 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Previously, a trancoding module did not have access to the joint but cached
-	  format. Therefore, the module did not have access to the attributes negotiated
-	  via SDP (line fmtp). Now, a translation module receives the joint format.
+	  Merged revisions 429273 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25545 #close
 
-	  Change-Id: Id6878a989b50573298dab115d3371ea369e1a718
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429274 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-11-19 01:14 +0000 [1aa552b2a2]  Alexander Traud <pabstraud@compuserve.com>
+2014-12-10 07:16 +0000 [0cba439c4d]  Kinsey Moore <kmoore@digium.com>
 
-	* res_format_attr_h264: Do not reset string buffer.
+	* PJSIP: Fix assert on initial mass qualify
 
-	  When no parameter is present, Asterisk does not generate the line fmtp, as
-	  expected. However, because a buffer was reset, even rtpmap and fmtp of previous
-	  media codecs got removed. Now, Asterisk does not reset other codecs in case of
-	  no parameter for H.264.
+	  This fixes the MWI test regressions caused by r429127 and ensures that
+	  contacts have non-zero qualify_frequency before attempting scheduling.
+	  ........
 
-	  ASTERISK-25573 #close
+	  Merged revisions 429245 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I93811331f4a28c45418a9e14ee46c0debd47a286
+	  Merged revisions 429246 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-18 10:05 +0000 [3354b325c6]  Matt Jordan <mjordan@digium.com>
 
-	* res_statsd: Add functions that support variable arguments
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429247 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Often, the metric names of statistics we are generating for StatsD have some
-	  dynamic component to them. This can be the name of a particular resource, or
-	  some internal status label in Asterisk. With the current set of functions,
-	  callers of the statsd API must first build the metric name themselves, then
-	  pass this to the API functions. This results in a large amount of boilerplate
-	  code and usage of either fixed length static buffers or dynamic memory
-	  allocation, neither of which is desireable.
+2014-12-09 14:47 +0000 [8fe45f0f0a]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  This patch adds two new functions to the StatsD API that support a printf
-	  style format specifier for constructing the metric name. A dynamic string,
-	  allocated in threadstorage, is used to build the metric name. This eases
-	  the burden on users of the StatsD API.
+	* core: avoid possible asterisk -r crash from long id
 
-	  Change-Id: If533c72d1afa26d807508ea48b4d8c7b32f414ea
+	  When connecting to the remote console, an id string
+	  is first provided that consts of the hostname, pid,
+	  and version.  This is parsed by the remote instance
+	  using a buffer that may be too short, and can allow
+	  a buffer overrun because it is not terminated. This
+	  patch adds termination and a larger buffer.
 
-2015-11-17 14:53 +0000 [d4a522d587]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4182/
+	  ........
 
-	* res_pjsip_outbound_registration.c: Be tolerant of short registration timeouts.
+	  Merged revisions 429223 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ie16f5053ebde0dc6507845393709b4d6a3ea526d
 
-2015-11-17 14:53 +0000 [e44ab3816c]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429224 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_outbound_registration.c: Fix 423 response handling.
+2014-12-09 14:20 +0000 [d673209abc]  Kevin Harwell <kharwell@digium.com>
 
-	  Receiving a 423 Interval Too Brief response after authentication for an
-	  outbound registration attempt results in assuming that the registrar has
-	  rejected the registration permanently.  If there are no configured retries
-	  for fatal responses then the outbound registration is stopped for that
-	  endpoint.
+	* ARI/AMI: Include language in standard channel snapshot output
 
-	  For registrations, PJSIP/PJPROJECT intercepts the handling of 423
-	  responses and does not include any authentication in the updated
-	  registration request.  When the updated request is challenged then the
-	  Asterisk code assumes that we were challenged again because the peer
-	  rejected the authentication we sent earlier.
+	  The channel "language" was already part of a channel snapshot, however is was
+	  not sent out over AMI or ARI. This patch makes it so the channel "language" is
+	  included in the appropriate AMI or ARI events.
 
-	  * Made registration challenges keep track of the CSeq number to determine
-	  if the received challenge response was for the request we thought we sent.
-	  If the response's CSeq number differs from the CSeq number we last sent
-	  with authentication then authenticate again because it is a challenge to a
-	  different request.
+	  ASTERISK-24553 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4245/
+	  ........
 
-	  Change-Id: I81b4bd36d1be095bab606e34b8b44e6302971b09
+	  Merged revisions 429204 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-11-03 14:36 +0000 [1e0040b88f]  Tyler Cambron <tcambron@digium.com>
+	  Merged revisions 429206 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* StatsD: Add res_statsd compatibility
 
-	  Added a new api to res_statsd.c to allow it to receive a
-	  character pointer for the value argument. This allows for a
-	  '+' and a '-' to easily be sent with the value.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429209 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25419
-	  Reported By: Ashley Sanders
+2014-12-09 14:03 +0000 [c17cef1c38]  Kevin Harwell <kharwell@digium.com>
 
-	  Change-Id: Id6bb53600943d27347d2bcae26c0bd5643567611
+	* Direct Media calls within private network sometimes get one way audio
 
-2015-11-16 13:56 +0000 [f62b642fe3]  Matt Jordan <mjordan@digium.com>
+	  When endpoints with direct_media enabled, behind a firewall (Asterisk on a
+	  separate network) and were bridged sometimes Asterisk would send the ip
+	  address of the firewall in the sdp to one of the phones in the reinvite
+	  resulting in one way audio. When sending the reinvite Asterisk will retrieve
+	  the media address from the associated rtp instance, but if frames were being
+	  read this can be overwritten with another address (in this case the
+	  firewall's).  This patch ensures that Asterisk uses the original device
+	  address when using direct media.
 
-	* res/res_pjsip: Fix off nominal crash with requests that fail and have a timer
+	  ASTERISK-24563
+	  Reported by: Steve Pitts
+	  Review: https://reviewboard.asterisk.org/r/4216/
+	  ........
 
-	  When a request is sent using pjsip_endpt_send_request and fails, a condition
-	  exists where the request wrapper, which is an AO2 object, may be de-ref'd
-	  more times than it should. This occurs when the request's callback is called,
-	  and, in the callback, the timer on the PJSIP heap is cancelled. When that
-	  occurs, the request wrapper's lifetime is decremented. When
-	  pjsip_endpt_send_request fails, we unilaterally decrement the lifetime of
-	  the request wrapper again, even though we've already cancelled the reference
-	  associated with the timer.
+	  Merged revisions 429195 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch checks the return result of pj_timer_heap_cancel_if_active before
-	  removing the reference associated with the timer. We now only decrement it
-	  in this case if a timer is cancelled as a result of the function call.
+	  Merged revisions 429196 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I21332343a1a019c1117076f9bf2df27be2850102
 
-2015-11-13 14:03 +0000 [fdd2afcd16]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429197 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Confbridge: Add a user timeout option
+2014-12-09 12:36 +0000 [7844266e21]  Kevin Harwell <kharwell@digium.com>
 
-	  This option adds the ability to specify a timeout, in seconds, for a
-	  participant in a ConfBridge. When the user's timeout has been reached,
-	  the user is ejected from the conference with the CONFBRIDGE_RESULT
-	  channel variable set to "TIMEOUT".
+	* res_pjsip_outbound_publish: stack overflow when using non-default sorcery wizard
 
-	  The rationale for this change is that there have been times where we
-	  have seen channels get "stuck" in ConfBridge because a network issue
-	  results in a SIP BYE not being received by Asterisk. While these
-	  channels can be hung up manually via CLI/AMI/ARI, adding some sort of
-	  automatic cleanup of the channels is a nice feature to have.
+	  When using a non-default sorcery wizard (in this instance realtime) for outbound
+	  publishes Asterisk will crash after a stack overflow occurs due to the code
+	  infinitely recursing.  The fix entails removing the outbound publish state
+	  dependency from the outbound publish sorcery object and instead keeping an in
+	  memory container that can be used to lookup the state when needed.
 
-	  ASTERISK-25549 #close
-	  Reported by Mark Michelson
+	  ASTERISK-24514 #close
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4178/
+	  ........
 
-	  Change-Id: I2996b6c5e16a3dda27595f8352abad0bda9c2d98
+	  Merged revisions 429175 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-16 04:29 +0000 [7debb986a5]  Alec Davis <sivad.a@paradise.net.nz>
 
-	* app_queue: (try_calling): mutex 'qe->chan' freed more times than we've locked!
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429176 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  commit aae45acbd (Mark Michelson 2015-04-15 10:38:02 -0500 6525)
-	  refer ASTERISK-24958
+2014-12-09 09:45 +0000 [60ab564ad2]  Joshua Colp <jcolp@digium.com>
 
-	  above commit removed ast_channel_lock(qe->chan);
-	  but failed to remove corresponding ast_channel_unlock(qe->chan);
+	* ari: Add support for specifying an originator channel when originating.
 
-	  ASTERISK-25561 #close
-	  Reported Alec Davis
+	  If an originator channel is specified when originating a channel the linked ID
+	  of it will be applied to the newly originated outgoing channel. This allows
+	  an association to be made between the two so it is known that the originator
+	  has dialed the originated channel.
 
-	  Change-Id: Ie05f4e2d08912606178bf1fded57cc022c7a2e1a
+	  ASTERISK-24552 #close
+	  Reported by: Matt Jordan
 
-2015-11-14 07:02 +0000 [afd9a89e5a]  Joshua Colp <jcolp@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4243/
+	  ........
 
-	* hashtab: Add NULL check when destroying iterator.
+	  Merged revisions 429153 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The hashtab API is pretty NULL tolerant which has resulted
-	  in remaining callers not doing much checks themselves.
-	  Unfortunately the function to destroy an iterator does not
-	  do a NULL check and will result in a crash if passed NULL.
-	  This change fixes that.
 
-	  ASTERISK-25552 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429154 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ic1bf8eec3639e5a440f1c941d3ae3893ac6ed619
+2014-12-09 08:01 +0000 [b6e18cae5c]  Kinsey Moore <kmoore@digium.com>
 
-2015-11-13 14:32 +0000 [c0f2f8de45]  Richard Mudgett <rmudgett@digium.com>
+	* PJSIP: Stagger outbound qualifies
 
-	* res_pjsip_rfc3326.c: Fix crash when channel goes away.
+	  This change staggers initiation of outbound qualify (OPTIONS) attempts
+	  to reduce instantaneous server load and prevent network congestion.
 
-	  If an authenticated incoming caller does not respond to our 200 OK INVITE
-	  response with an ACK then PJSIP will hangup the call.  Unfortunately,
-	  there is a chance that the session's channel will go away between one use
-	  of the channel pointer and another when building the BYE request because
-	  the BYE is being built by the monitor thread and not the call's serializer
-	  thread.
+	  Review: https://reviewboard.asterisk.org/r/4246/
+	  ASTERISK-24342 #close
+	  Reported by: Richard Mudgett
+	  ........
 
-	  * Added a check to ensure that the thread trying to add the Reason header
-	  is the call's serializer thread.  This ensures that the channel will not
-	  go away on us.
+	  Merged revisions 429127 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I866388d2b97ea2032eaae3f3ab3f1ca6cbd2df89
+	  Merged revisions 429128 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-13 14:19 +0000 [4f43b85c92]  Mark Michelson <mmichelson@digium.com>
 
-	* Taskprocessors: Increase high-water mark
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429129 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In practical tests, we have seen certain taskprocessors, specifically
-	  Stasis subscription taskprocessors, cross the recently-added high-water
-	  mark and emit a warning. This high-water mark warning is only intended
-	  to be emitted when things have tanked on the system and things are
-	  heading south quickly. In the practical tests, the Stasis taskprocessors
-	  sometimes had a max depth of 180 tasks in them, and Asterisk wasn't in
-	  any danger at all.
+2014-12-08 10:54 +0000 [fe6cbf455a]  Matt Jordan <mjordan@digium.com>
 
-	  As such, this ups the high-water mark to 500 tasks instead. It also
-	  redefines the SIP threadpool request denial number to be a multiple of
-	  the taskprocessor high-water mark.
+	* AMI/ARI: Update version to 2.6.0/1.6.0 respectively for new features
 
-	  Change-Id: Ic8d3e9497452fecd768ac427bb6f58aa616eebce
+	  AMI/ARI are getting a few enhancements in the next release of Asterisk 13. Per
+	  semantic versioning, that warrants a bump in the minor version number, as it
+	  reflects a backwards compatible change. Hence, this commit.
+	  ........
 
-2015-11-11 11:46 +0000 [d8d3991390]  Alexander Traud <pabstraud@compuserve.com>
+	  Merged revisions 429091 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* format: Register format-attribute module with cached formats.
 
-	  In Asterisk 13, cached formats are created before their corresponding format-
-	  attribute module is registered. Cached formats are involved when a local
-	  extension is called. Therefore, ast_format_generate_sdp_fmtp did not work
-	  on local extensions. This change affects the Opus Codec, H.263 (Plus), H.264,
-	  and format-attribute modules provided externally.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429092 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25160 #close
+2014-12-08 10:43 +0000 [bba1763f47]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I1ea1f0483e5261e2a050112e4ebdfc22057d1354
+	* Fix a crash that would occur when receiving a 491 response to a reinvite.
 
-2015-11-12 11:17 +0000 [367972e42d]  Mark Michelson <mmichelson@digium.com>
+	  The reviewboard description does a fine job of summarizing this, so here it is:
 
-	* res_pjsip distributor: Don't send 503 response to responses.
+	  A reporter discovered that Asterisk would crash when attempting to retransmit
+	  a reinvite that had previously received a 491 response. The crash occurred
+	  because a pjsip_tx_data structure was being saved for reuse, but its reference
+	  count was not being increased. The result was that the pjsip_tx_data was being
+	  freed before we were actually done with it. When we attempted to re-use the
+	  structure when re-sending the reinvite, Asterisk would crash.
 
-	  When the SIP threadpool is backed up with tasks, we send 503 responses
-	  to ensure that we don't try to overload ourselves. The problem is that
-	  we were not insuring that we were not trying to send a 503 to an
-	  incoming SIP response.
+	  The fix implemented here is not to try holding onto the pjsip_tx_data at all.
+	  Instead, when we reschedule sending the reinvite, we create a brand new
+	  pjsip_tx_data and send that instead. Because of this change, there is no need
+	  for an ast_sip_session_delayed_request structure to have a pjsip_tx_data on
+	  it any more. So any code referencing its use has been removed.
 
-	  This change makes it so that we only send the 503 on incoming requests.
+	  When this initial fix was introduced, I encountered a second crash when
+	  processing a subsequent 200 OK on a rescheduled reinvite. The reason was
+	  that when rescheduling the reinvite, we gave the wrong location for a
+	  response callback. This has been fixed in this patch as well.
 
-	  Change-Id: Ie2b418d89c0e453cc6c2b5c7d543651c981e1404
+	  ASTERISK-24556 #close
+	  Reported by Abhay Gupta
 
-2015-11-11 17:11 +0000 [2f9cb7d62b]  Mark Michelson <mmichelson@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4233
+	  ........
 
-	* res_pjsip: Deny requests when threadpool queue is backed up.
+	  Merged revisions 429089 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  We have observed situations where the SIP threadpool may become
-	  deadlocked. However, because incoming traffic is still arriving, the SIP
-	  threadpool's queue can continue to grow, eventually running the system
-	  out of memory.
 
-	  This change makes it so that incoming traffic gets rejected with a 503
-	  response if the queue is backed up too much.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429090 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I4e736d48a2ba79fd1f8056c0dcd330e38e6a3816
+2014-12-08 10:24 +0000 [fe7671fee6]  Mark Michelson <mmichelson@digium.com>
 
-2015-11-12 06:24 +0000 [4e5bf12b33]  Joshua Colp <jcolp@digium.com>
+	* Add new AMI and ARI events for connected line changes on a channel.
 
-	* format_cap: Don't append the 'none' format when appending all.
+	  The AMI event is called NewConnectedLine and the ARI event is called
+	  ChannelConnectedLine.
 
-	  When appending all formats of a type all the codecs are iterated
-	  and added. This operation was incorrectly adding the ast_format_none
-	  format which is special in that it is supposed to be used when no
-	  format is present. It shouldn't be appended.
+	  ASTERISK-24554 #close
+	  Reported by Matt Jordan
 
-	  ASTERISK-25535
+	  Review: https://reviewboard.asterisk.org/r/4231
+	  ........
 
-	  Change-Id: I7b00f3bdf4a5f3022e483d6ece602b1e8b12827c
+	  Merged revisions 429064 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-11 04:16 +0000 [07583c2888]  Steve Davies <steve@one47.co.uk>
 
-	* Further fixes to improper usage of scheduler
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429084 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When ASTERISK-25449 was closed, a number of scheduler issues mentioned in
-	  the comments were missed. These have since beed raised in ASTERISK-25476
-	  and elsewhere.
+2014-12-08 09:45 +0000 [4bb556a847]  Kinsey Moore <kmoore@digium.com>
 
-	  This patch attempts to collect all of the scheduler issues discovered so
-	  far and address them sensibly.
+	* Stasis: Fix StasisStart/End order and missing events
 
-	  ASTERISK-25476 #close
+	  This corrects several bugs that currently exist in the stasis
+	  application code.
+
+	  * After a masquerade, the resulting channels have channel topics that
+	    do not match their uniqueids
+	  ** Masquerades now swap channel topics appropriately
+	  * StasisStart and StasisEnd messages are leaked to observer
+	    applications due to being published on channel topics
+	  ** StasisStart and StasisEnd publishing is now properly restricted
+	     to controlling apps via app topics
+	  * Race conditions exist where StasisStart and StasisEnd messages due to
+	    a masquerade may be received out of order due to being published on
+	    different topics
+	  ** These messages are now published directly on the app topic so this
+	     is now a non-issue
+	  * StasisEnds are sometimes missing when sent due to masquerades and
+	    bridge swaps into and out of Stasis()
+	  ** This was due to StasisEnd processing adjusting message-sent flags
+	     after Stasis() had already exited and Stasis() had been re-entered
+	  ** This was corrected by adjusting these flags prior to sending the
+	     message while the initial Stasis() application was still shutting
+	     down
+
+	  Review: https://reviewboard.asterisk.org/r/4213/
+	  ASTERISK-24537 #close
+	  Reported by: Matt DiMeo
+	  ........
 
-	  Change-Id: I87a77d581e2e0d91d33b4b2fbff80f64a566d05b
+	  Merged revisions 429061 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-11-11 11:04 +0000 [b818d70533]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 429062 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* threadpool: Handle worker thread transitioning to dead when going active.
 
-	  This change adds handling of dead worker threads when moving them
-	  to be active. When this happens the worker thread is removed from
-	  both the active and idle threads container. If no threads are able
-	  to be moved to active then the pool grows as configured.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429063 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A unit test has also been added which thrashes the idle timeout
-	  and thread activation to exploit any race conditions between the
-	  two.
+2014-12-06 12:16 +0000 [49aa87e17c]  Nuno Borges (License 6116)
 
-	  ASTERISK-25546 #close
+	* res/res_monitor: Reset in/out sample counts on Monitor start
 
-	  Change-Id: I6c455f9a40de60d9e86458d447b548fb52ba1143
+	  When repeatedly starting/stopping a Monitor on a channel, the accumulated
+	  in/out sample counts are never reset to 0. This can cause inadvertent jumps
+	  in the recordings, as the code in the channel core will determine incorrectly
+	  that a jump in the recorded file position should occur. Setting the sample
+	  counts to 0 simply reflects the initial state a Monitor should be in when it
+	  is started, as this is the initial count that would be on the channels at that
+	  time.
 
-2015-11-10 09:27 +0000 [4bf84459c7]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-24573 #close
+	  Reported by: Nuno Borges
+	  patches:
+	    24573.patch uploaded by Nuno Borges (License 6116)
+	  ........
 
-	* rtp_engine: Init a format-attribute module to its RFC defaults.
+	  Merged revisions 429031 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Previously, format-attribute modules relied on an existing fmtp line in SDP
-	  negotiation. However, fmtp is optional for several formats like the Opus Codec.
-	  Now, the format-attribute module is called with an empty fmtp, which allows the
-	  module to initialise itself to RFC defaults. Furthermore now, Asterisk is able
-	  to differentiate between internally and externally created formats.
+	  Merged revisions 429032 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25537 #close
+	  Merged revisions 429033 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I28f680cef7fdf51c0969ff8da71548edad72ec52
 
-2015-11-09 03:04 +0000 [1bff400df7]  Alexander Traud <pabstraud@compuserve.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429034 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* ast_format_cap_get_names: To display all formats, the buffer was increased.
+2014-12-06 11:36 +0000 [0cdb71aae9]  Nuno Borges (License 6116)
 
-	  ASTERISK-25533 #close
+	* apps/app_meetme: Apply default values on initial load with no config file
 
-	  Change-Id: Ie1a9d1a6511b3f1a56b93d04475fbf8a4e40010a
+	  When the app_meetme module is loaded without its configuration file, the
+	  module settings aren't initialized. In particular, this impacts the use
+	  of logging realtime members. This patch guarantees that we always set the
+	  default module settings on initial load.
 
-2015-11-09 07:04 +0000 [f3ac4d8090]  Alexander Traud <pabstraud@compuserve.com>
+	  Review: https://reviewboard.asterisk.org/r/4242/
 
-	* ast_format_cap: Avoid format creation on module load, use cache instead.
+	  ASTERISK-24572 #close
+	  Reported by: Nuno Borges
+	  patches:
+	    24572.patch uploaded by Nuno Borges (License 6116)
+	  ........
 
-	  Since Asterisk 13, formats are immutable and cached. However while loading a
-	  module like chan_sip, some formats were created instead using cached ones.
+	  Merged revisions 429027 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25535 #close
+	  Merged revisions 429028 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I479cdc220d5617c840a98f3389b3bd91e91fbd9b
+	  Merged revisions 429029 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-06 07:54 +0000 [6d1bdb9d3b]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* func_callerid: Document that CALLERID(pres) is available.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429030 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  CALLERPRES() says that it's deprecated in favor of CALLERID(num-pres)
-	  and CALLERID(name-pres).  But for channel driver that don't make a
-	  distinction between the two (e.g. SIP), it makes more sense to get/set
-	  both at once.  This change reveals the availability of CALLERID(pres),
-	  CONNECTEDLINE(pres), REDIRECTING(orig-pres), REDIRECTING(to-pres) and
-	  REDIRECTING(from-pres).
+2014-12-05 11:08 +0000 [d04445c24a]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25373 #close
+	* sorcery: Add additional observer capabilities.
 
-	  Change-Id: I5614ae4ab7d3bbe9c791c1adf147e10de8698d7a
-2015-11-06 07:52 +0000 [8410336681]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Add new global, instance and wizard observers.
+	  instance_created
+	  wizard_registered
+	  wizard_unregistered
+	  instance_destroying
+	  instance_loading
+	  instance_loaded
+	  wizard_mapped
+	  object_type_registered
+	  object_type_loading
+	  object_type_loaded
+	  wizard_loading
+	  wizard_loaded
 
-	* docs: Fix a few typo's in app docs (more then, resourse).
+	  Tested-by: George Joseph
 
-	  Change-Id: Iba57efadf6c0b822e762c7a001bc89611d98afd7
+	  Review: https://reviewboard.asterisk.org/r/4215/
+	  ........
 
-2015-11-06 07:36 +0000 [0d425f2eb4]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Merged revisions 428999 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* xmldoc: Improve xmldoc wrapping of 'core show ...' output.
+	  Merged revisions 429000 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Previously, the wrapping did both lookahead and lookback, which,
-	  together with color escape sequences, caused some lines to be wrapped
-	  way earlier than other lines.  This led to inconsistent output.
 
-	  This simplifies the wrapping code and makes it more sane: if maxcolumns
-	  is hit, we simply jump back to the last space and wrap there.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@429001 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25527 #close
+2014-12-04 11:13 +0000 [19992844be]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I56d01c6f9a812642b1b05535c98d4db48d17c957
+	* main/test: Fix compilation issue on 32-bit systems
 
-2015-11-06 06:57 +0000 [33752e0837]  Sean Bright (license #5060)
+	  On a 32-bit system, a type of intmax_t will result in a compilation warning
+	  when formatted as a 'long int'. Use the format specifier of %jd (which was
+	  what was used originally in manager.c) to format the JSON extracted integer
+	  on both 32-/64-bit systems.
+	  ........
 
-	* res_pjsip_sdp_rtp: Enable Opus to be negotiated via SIP/SDP.
+	  Merged revisions 428972 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  In SIP/SDP, Opus has two channels always (see RFC 7587 section 7). The actual
-	  amount of channels is negotiated in-band. Therefore now, the Opus codec and its
-	  attribute rtpmap are registered with two channels.
+	  Merged revisions 428973 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24779 #close
-	  Reported by: PowerPBX
-	  Tested by: Alexander Traud
-	  patches:
-	    asterisk-24779.patch submitted by Sean Bright (license #5060)
 
-	  Change-Id: Ic7ac13cafa1d3450b4fa4987350924b42cbb657b
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428974 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-11-03 16:19 +0000 [6ff48319d9]  Jonathan Rose <jrose@digium.com>
+2014-12-04 09:48 +0000 [343a83d7d8]  Matt Jordan <mjordan@digium.com>
 
-	* taskprocessor: Add high water mark warnings
+	* main/test: Fix race condition between AMI topic and Test Suite topic
 
-	  If a taskprocessor's queue grows large, this can indicate that there
-	  may be a problem with tasks not leaving the processor or else that
-	  the number of available task processors for a given type of task is
-	  too low. This patch makes it so that if a taskprocessor's task queue
-	  grows above 100 queued tasks that it will emit a warning message.
-	  Warning messages are emitted only once per task processor.
+	  This patch fixes a race condition between the raising of test AMI events (which
+	  drive many tests in the Asterisk Test Suite) and other AMI events. Prior to
+	  this patch, the Stasis messages published to the test topic were not forwarded
+	  to the AMI topic. Instead, the code in manager had a dedicated handler for test
+	  messages that was independent of the topics forwarded to the AMI topic. This
+	  results in no synchronization between the test messages and the rest of the
+	  Stasis messages published out over AMI. In some test with very tight timing
+	  constraints, this can result in out of order messages and spurious test
+	  failures. Properly forwarding the Test Suite topic to the AMI topic ensures
+	  that the messages are synchronized properly.
 
-	  ASTERISK-25518 #close
-	  Reported by: Jonathan Rose
+	  This patch does that, and moves the message handling to the Stasis definition
+	  of the Test Suite message in test.c as well.
 
-	  Change-Id: Ib1607c35d18c1d6a0575b3f0e3ff5d932fd6600c
+	  Review: https://reviewboard.asterisk.org/r/4221/
+	  ........
 
-2015-11-04 14:31 +0000 [506aea26e6]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 428945 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* main/dial: Protect access to the format_cap structure of the requesting channel
+	  Merged revisions 428946 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When a dial attempt is made that involves a requesting channel, we previously
-	  were not:
-	  a) Protecting access to the native format capabilities structure on the
-	     requesting channel. That is inherently unsafe.
-	  b) Reference bumping the lifetime of the format capabilities structure.
 
-	  In both cases, something else could sneak in, blow away the format
-	  capabilities, and we'd be holding onto an invalid format_cap structure. When
-	  the newly created channel attempts to construct its format capabilities, things
-	  go poorly.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428947 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch:
-	  a) Ensures that we get a reference to the native format capabilities while
-	     the requesting channel is locked
-	  b) Holds a reference to the native format capabilities during the creation
-	     of the new channel.
+2014-12-03 14:59 +0000 [7cb2c446b4]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25522 #close
+	* tests/test_cel: Add test_cel_attended_transfer_bridges_link to racey tests
 
-	  Change-Id: I0bfb7ba8b9711f4158cbeaae96edf9626e88a54f
+	  Despite failing less often, the ordering of the ATTENDEDTRANSFER event and the
+	  BRIDGE_EXIT event for the Alice and David channels is not defined. This makes
+	  the test still fail.
+	  ........
 
-2015-10-30 22:57 +0000 [d098d00424]  Corey Farrell <git@cfware.com>
+	  Merged revisions 428918 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Fix cli display of build options.
+	  Merged revisions 428919 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  A previous commit reduced the AST_BUILDOPTS compiler define to
-	  only include options that affected ABI.  This included some options
-	  that were previously displayed by cli "core show settings".  This
-	  change corrects the CLI display while still restricting buildopts.h
-	  to ABI effecting options only.
 
-	  ASTERISK-25434 #close
-	  Reported by: Rusty Newton
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428920 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id07af6bedd1d7d325878023e403fbd9d3607e325
+2014-12-03 13:49 +0000 [7475e1c948]  Matt Jordan <mjordan@digium.com>
 
-2015-11-03 11:15 +0000 [afec1b1b64]  Matt Jordan <mjordan@digium.com>
+	* tests/test_cel: Fix CEL unit test failures caused by attended transfer changes
 
-	* res_pjsip/location: Destroy contact_status objects on contact deletion
+	  When the publication of attended transfer messages were pushed to another
+	  thread, some subtle race conditions were introduced with the CEL unit tests.
+	  This patch fixes one of them, and pushes the other to ASTERISK-22367, which
+	  already exists to fix another bouncy CEL unit test.
 
-	  The contact_status Sorcery objects are currently not destroyed when a contact
-	  is deleted. This causes the contact's last known RTT/status to be 'sticky'
-	  when the contact itself may no longer exist. This patch causes the
-	  contact_status objects associated with both dynamic and static contacts to
-	  be destroyed if the AoR holding those contacts is also destroyed (or via
-	  other paths where a contact may be deleted.)
+	  In particular, this patch fixes the test_cel_attended_transfer_bridges_link
+	  test, and defers the test_cel_attended_transfer_bridges_swap test to the
+	  aforementioned JIRA issue.
 
-	  Change-Id: I7feec8b9278cac3c5263a4c0483f4a0f3b62426e
+	  ASTERISK-22367
+	  ........
 
-2015-11-03 10:58 +0000 [715f770c9f]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 428891 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* pjsip_configuration: On delete, remove the persistent version of an endpoint
+	  Merged revisions 428892 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When an endpoint is deleted (such as through an API), the persistent endpoint
-	  currently continues to lurk around. While this isn't harmful from a memory
-	  consumption perspective - as all persistent endpoints are reclaimed on
-	  shutdown - it does cause Stasis endpoint related operations to continue
-	  to believe that the endpoint may or may not exist.
 
-	  This patch causes the persistent endpoint related to a PJSIP endpoint to be
-	  destroyed if the PJSIP endpoint is deleted.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428893 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I85ac707b4d5e6aad882ac275b0c2e2154affa5bb
-2015-11-03 08:15 +0000 [f0f190af08]  Matt Jordan <mjordan@digium.com>
+2014-12-03 10:45 +0000 [6d4ef7ddf4]  David Duncan Ross Palmer (License 6660)
 
-	* main/stasis_endpoints: Fix ContactStatusChange JSON for roundtrip_usec field
+	* apps/app_voicemail: Fix crash with IMAP when streams are opened simultaneously
 
-	  The JSON packing for the ContactStatusChange event forgot to include the
-	  roundtrip_usec field. As a result, the field never showed up in any event,
-	  even when the data was available. This patch corrects that error by properly
-	  packing the JSON blob with the data.
+	  The UW IMAP library is instrinsically not thread-safe, and relies upon higher
+	  level applications to guarantee thread safety. For the most part, this is
+	  provided by the vms object, which provides locking for individual streams.
+	  Unfortunately, this is not sufficient for calls to mail_open which create the
+	  IMAP stream. mail_open can, on some systems, call into a UW IMAP specific
+	  function for determining the address of a system based on a hostname,
+	  ip_nametoaddr.
 
-	  Change-Id: I8df80da659a44010afbd48f645967518ff5daa17
+	  In the ip6_unix implementation of this function, static variables are used
+	  to hold parsing buffers. This can cause a crash if multiple threads attempt
+	  to convert a hostname to an address at the same time. Locking on a single
+	  mail stream is not sufficient to prevent simultaneous access to these static
+	  variables.
 
-2015-11-02 20:24 +0000 [0393bd6bed]  Corey Farrell <git@cfware.com>
+	  In the IMAP library, this function can be called from the mail_open and
+	  imap_status functions. As the imap_status function is not used by
+	  app_voicemail, locking on access to mail_open is sufficient to prevent
+	  any mangling of the buffers.
+
+	  Review: https://reviewboard.asterisk.org/r/4188/
+
+	  ASTERISK-24516 #close
+	  Reported by: David Duncan Ross Palmer
+	  Tested by: David Duncan Ross Palmer
+	  patches:
+	    ASTERISK-24516.diff uploaded by David Duncan Ross Palmer (License 6660)
+	  ........
 
-	* chan_sip: Allow websockets to be disabled.
+	  Merged revisions 428863 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch adds a new setting "websockets_enabled" to sip.conf.
-	  Setting this to false allows chan_sip to be used without causing
-	  conflicts with res_pjsip_transport_websocket.
+	  Merged revisions 428864 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24106 #close
-	  Reported by: Andrew Nagy
+	  Merged revisions 428865 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I04fe8c4f2d57b2d7375e0e25826c91a72e93bea7
 
-2015-11-02 17:19 +0000 [6fbffe42e1]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428866 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Set threadpool max size default to 50.
+2014-12-02 15:54 +0000 [63cbd28999]  gtjoseph <george.joseph@fairview5.com>
 
-	  During a stress test of subscriptions, a huge blast of
-	  subscription-related traffic resulted in the threadpool expanding to a
-	  ridiculous number of threads. The balooning of threads resulted in an
-	  increase of memory, which led to a crash due to being out of memory.
+	* CHANGES: Add item for new 'pjsip show identif(y|ies) commands
 
-	  An easy fix for the particular test was to limit the size of the
-	  threadpool, thus reining in the amount of memory that would be used. It
-	  was decided that there really is no downside to having a non-infinite
-	  default value for the maximum size of the threadpool, so this change
-	  introduces 50 threads as the maximum threadpool size for the SIP
-	  threadpool.
+	  Tested-by: George Joseph
+	  ........
 
-	  ASTERISK-25513 #close
-	  Reported by John Bigelow
+	  Merged revisions 428836 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: If0b9514f1d9b172540ce1a6e2f2ffa1f2b6119be
+	  Merged revisions 428837 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-11-02 06:57 +0000 [11e54b1932]  Matt Jordan <mjordan@digium.com>
 
-	* pjsip_options: Schedule/unschedule qualifies on AoR creation/destruction
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428838 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When an AoR is created or destroyed dynamically, the scheduled OPTIONS
-	  requests that qualify the contacts on the AoR are not necessarily started
-	  or destroyed, particularly for persistent contacts created for that AoR.
-	  This patch adds create/update/delete sorcery observers for an AoR, which
-	  schedule/unschedule the qualifies as expected.
+2014-12-02 13:04 +0000 [dd00e80cbe]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ic287ed2e2952a7808ee068776fe966f9554bdf7d
+	* tests/test_stasis: Resolve compilation issues from Asterisk 12 merge
 
-2015-10-30 13:22 +0000 [118d628e08]  Matt Jordan <mjordan@digium.com>
+	  When merging the changes up stream in r428687, I missed the fact that the
+	  signature for stasis_message_type_create was changed. This patch fixes
+	  the compilation issues introduced by that merge.
+	  ........
 
-	* Makefile: Add a rule 'basic-pbx' that installs the Basic PBX configs
+	  Merged revisions 428815 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch adds a rule for installing the Super Awesome Company based 'Basic
-	  PBX' configuration files. As part of adding this rule, a bit of the content
-	  that makes up installing the configuration files under the 'samples' target
-	  was refactored into a make subroutine for usage by additional later config
-	  make targets.
 
-	  Change-Id: I6c2e27906f73e2919a2b691da0be20ae70302404
-2015-10-29 08:28 +0000 [9a021a42ad]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428816 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_pubsub: Fix assertion when UAS dialog creation fails.
+2014-12-02 11:10 +0000 [08636aadec]  Birger Harzenetter (License 5870)
 
-	  When compiled with assertions enabled one will occur when destroying
-	  the subscription tree when UAS dialog creation fails. This is because
-	  the code assumes that a dialog will always exist on a subscription
-	  tree when in reality during this specific scenario it won't.
+	* pbx/pbx_loopback: Speed up switches by avoiding unneeded lookups
 
-	  This change makes it so a dialog is not removed from the subscription
-	  tree if it is not present.
+	  This patch makes a small rearrangement to only do dialplan lookups during
+	  loopback switches if the pattern matches. Prior to this patch, the dialplan
+	  lookups were always performed, even when the result would be discarded.
+	  Dialplan lookups can be very costly if remote switches - like DUNDi - are
+	  present. In those cases extension matching is sped up considerably, making
+	  the issue of lost digits more manageable.
 
-	  ASTERISK-25505 #close
+	  As collateral damage, 6 trailing spaces were killed.
 
-	  Change-Id: Id5c182b055aacc5e66c80546c64804ce19218dee
+	  Review: https://reviewboard.asterisk.org/r/4211
 
-2015-10-26 11:42 +0000 [1256aedf66]  Alexander Traud <pabstraud@compuserve.com>
+	  ASTERISK-24577 #close
+	  Reported by: Birger Harzenetter
+	  patches:
+	    ast-loopback.patch uploaded by Birger Harzenetter (License 5870)
+	  ........
 
-	* chan_sip: Do not send all codecs on INVITE.
+	  Merged revisions 428787 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Since version 13, Asterisk sent all allowed codecs as callee, even when the
-	  caller did not request/support them. In case of dynamic RTP payloads, this led
-	  to the same ID for different codecs, which is not allowed by SIP/SDP. Now, the
-	  intersection between the requested and the supported codecs is send again.
+	  Merged revisions 428788 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24543 #close
+	  Merged revisions 428789 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ie90cb8bf893b0895f8d505e77343de3ba152a287
 
-2015-10-24 13:08 +0000 [5f593e7c38]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428790 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* build: GCC 5.1.x catches some new const, array bounds and missing paren issues
+2014-12-02 06:21 +0000 [0c1aaa7da5]  Joshua Colp <jcolp@digium.com>
 
-	  Fixed 1 issue in each of the affected files.
+	* res_pjsip_refer: Fix issue where native bridge may not occur upon completion of a transfer.
 
-	  ASTERISK-25494 #close
-	  Reported-by: George Joseph
-	  Tested-by: George Joseph
+	  There are two methods within res_pjsip_refer for keeping track of the state of a transfer.
+	  The first is a framehook which looks at frames passing by to determine the state. The second
+	  subscribes to know when the channel joins a bridge. In the case when the channel joins the
+	  bridge the framehook is *NOT* removed and this prevents the native RTP bridging technology
+	  from getting used.
 
-	  Change-Id: I818f149cd66a93b062df421e1c73c7942f5a4a77
+	  This change gets the channel and if it still exists remove the framehook.
 
-2015-10-20 16:02 +0000 [162acd45f7]  gtjoseph <george.joseph@fairview5.com>
+	  Review: https://reviewboard.asterisk.org/r/4218/
+	  ........
 
-	* res_pjsip:  Add "like" processing to pjsip list and show commands
+	  Merged revisions 428760 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Add the ability to filter output from pjsip list and show commands
-	  using the "like" predicate like chan_sip.
+	  Merged revisions 428761 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  For endpoints, aors, auths, registrations, identifyies and transports,
-	  the modification was a simple change of an ast_sorcery_retrieve_by_fields
-	  call to ast_sorcery_retrieve_by_regex.  For channels and contacts a
-	  little more work had to be done because neither of those objects are
-	  true sorcery objects.  That was just removing the non-matching object
-	  from the final container.  Of course, a little extra plumbing in the
-	  common pjsip_cli code was needed to parse the "like" and pass the regex
-	  to the get_container callbacks.
 
-	  Some of the get_container code in res_pjsip_endpoint_identifier was also
-	  refactored for simplicity.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428762 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25477 #close
-	  Reported by: Bryant Zimmerman
-	  Tested by: George Joseph
+2014-12-01 18:38 +0000 [f128ff61ab]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: I646d9326b778aac26bb3e2bcd7fa1346d24434f1
+	* config: Create ast_variable_find_in_list()
 
-2015-10-21 11:51 +0000 [c58091737d]  Kevin Harwell <kharwell@digium.com>
+	  Add
+	  const char *ast_variable_find_in_list(const struct ast_variable *list,
+	     const char *variable);
 
-	* res_pjsip_outbound_registration: registration stops due to fatal 4xx response
+	  ast_variable_find() requires a config category to search whereas
+	  ast_variable_find_in_list() just needs the root list element which is
+	  useful if you don't have a category.
 
-	  During outbound registration it is possible to receive a fatal (any permanent/
-	  non-temporary 4xx, 5xx, 6xx) response from the registrar that is simply due
-	  to a problem with the registrar itself. Upon receiving the failure response
-	  Asterisk terminates outbound registration for the given endpoint.
+	  Tested-by: George Joseph
 
-	  This patch adds an option, 'fatal_retry_interval', that when set continues
-	  outbound registration at the given interval up to 'max_retries' upon receiving
-	  a fatal response.
+	  Review: https://reviewboard.asterisk.org/r/4217/
+	  ........
 
-	  ASTERISK-25485 #close
+	  Merged revisions 428733 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Ibc2c7b47164ac89cc803433c0bbe7063bfa143a2
+	  Merged revisions 428734 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-10-22 17:07 +0000 [ebe69dee0d]  Mark Michelson <mmichelson@digium.com>
 
-	* format_cap: Detect vector allocation failures.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428735 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A crash was seen on a system that ran out of memory due to Asterisk not
-	  checking for vector allocation failures in format_cap.c. With this
-	  change, if either of the AST_VECTOR_INIT calls fail, we will return a
-	  value indicating failure.
+2014-12-01 18:31 +0000 [f418f25c44]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Ieb9c59f39dfde6d11797a92b45e0cf8ac5722bc8
+	* res_pjsip_endpoint_identifier_ip: Add 'show identify(ies)' cli commands
 
-2015-10-02 15:32 +0000 [3b19efefef]  Mark Michelson <mmichelson@digium.com>
+	  While troubleshooting other things I realized there were no pjsip cli
+	  commands for identify.  This patch adds them.  It also also fixes a
+	  reference leak when a 'show endpoint' displayed identifies and properly
+	  sets the return code if load_module can't allocate a cli formatter structure.
 
-	* res_pjsip_pubsub: Prevent sending NOTIFY on destroyed dialog.
+	  Tested-by: George Joseph
 
-	  A certain situation can result in our attempting to send a NOTIFY on a
-	  destroyed dialog. Say we attempt to send a NOTIFY to a subscriber, but
-	  that subscriber has dropped off the network. We end up retransmitting
-	  that NOTIFY until the appropriate SIP timer says to destroy the NOTIFY
-	  transaction. When the pjsip evsub code is told that the transaction has
-	  been terminated, it responds in kind by alerting us that the
-	  subscription has been terminated, destroying the subscription, and then
-	  removing its reference to the dialog, thus destroying the dialog.
+	  Review: https://reviewboard.asterisk.org/r/4212/
+	  ........
 
-	  The problem is that when we get told that the subscription is being
-	  terminated, we detect that we have not sent a terminating NOTIFY
-	  request, so we queue up such a NOTIFY to be sent out. By the time that
-	  queued NOTIFY gets sent, the dialog has been destroyed, so attempting to
-	  send that NOTIFY can result in a crash.
+	  Merged revisions 428725 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The fix being introduced here is actually a reintroduction of something
-	  the pubsub code used to employ. We hold a reference to the dialog and
-	  wait to decrement our reference to the dialog until our subscription
-	  tree object is destroyed. This way, we can send messages on the dialog
-	  even if the PJSIP evsub code wants to terminate earlier than we would
-	  like.
+	  Merged revisions 428731 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  In doing this, some NULL checks for subscription tree dialogs have been
-	  removed since NULL dialogs are no longer actually possible.
 
-	  Change-Id: I013f43cddd9408bb2a31b77f5db87a7972bfe1e5
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428732 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-29 14:53 +0000 [0a346f095f]  Mark Michelson <mmichelson@digium.com>
+2014-12-01 12:51 +0000 [4ff6bd831f]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_pubsub: Ensure dialog lock balance.
+	* rtp_engine: Add support for transporting signed linear at 12kHz, 24kHz, 32kHz, 44kHz, 48kHz, 96kHz, and 192kHz over RTP.
 
-	  When sending a NOTIFY, we lock the dialog and then unlock the dialog
-	  when finished. A recent change made it so that the subscription tree's
-	  dialog pointer will be set NULL when sending the final NOTIFY request
-	  out. This means that when we attempt to unlock the dialog, we pass a
-	  NULL pointer to pjsip_dlg_dec_lock(). The result is that the dialog
-	  remains locked after we think we have unlocked it. When a response to
-	  the NOTIFY arrives, the monitor thread attempts to lock the dialog, but
-	  it cannot because we never released the dialog lock. This results in
-	  Asterisk being unable to process incoming SIP traffic any longer.
+	  This change adds mappings in the RTP engine layer for the remaining signed linear formats.
 
-	  The fix in this patch is to use a local pointer to save off the pointer
-	  value of the subscription tree's dialog when locking and unlocking the
-	  dialog. This way, if the subscription tree's dialog pointer is NULLed
-	  out, the local pointer will still have point to the proper place and the
-	  dialog lock will be unlocked as we expect.
+	  ASTERISK-24274 #close
+	  Reported by: Frankie Chin
 
-	  Change-Id: I7ddb3eaed7276cceb9a65daca701c3d5e728e63a
+	  Review: https://reviewboard.asterisk.org/r/4093/
 
-2015-09-28 16:36 +0000 [ad39508095]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip_pubsub: Prevent crashes on final NOTIFY.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428708 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The SIP dialog is removed from the subscription tree when the final
-	  NOTIFY is sent. However, after the final NOTIFY is sent, the persistence
-	  update function still attempts to access the cseq from the dialog,
-	  resulting in a crash.
+2014-12-01 11:59 +0000 [1106e8fd0f]  Matt Jordan <mjordan@digium.com>
 
-	  This fix removes the subscription persistence at the same time that the
-	  dialog is removed from the subscription tree. This way, there is no
-	  attempt to update persistence when the subscription is being destroyed.
+	* main/stasis: Allow subscriptions to use a threadpool for message delivery
 
-	  Change-Id: Ibb46977a6cef9c51dc95f40f43446e3d11eed5bb
+	  Prior to this patch, all Stasis subscriptions would receive a dedicated
+	  thread for servicing published messages. In contrast, prior to r400178
+	  (see review https://reviewboard.asterisk.org/r/2881/), the subscriptions
+	  shared a thread pool. It was discovered during some initial work on Stasis
+	  that, for a low subscription count with high message throughput, the
+	  threadpool was not as performant as simply having a dedicated thread per
+	  subscriber.
 
-2015-09-17 17:28 +0000 [067f408760]  Mark Michelson <mmichelson@digium.com>
+	  For situations where a subscriber receives a substantial number of messages
+	  and is always present, the model of having a dedicated thread per subscriber
+	  makes sense. While we still have plenty of subscriptions that would follow
+	  this model, e.g., AMI, CDRs, CEL, etc., there are plenty that also fall into
+	  the following two categories:
+	  * Large number of subscriptions, specifically those tied to endpoints/peers.
+	  * Low number of messages. Some subscriptions exist specifically to coordinate
+	    a single message - the subscription is created, a message is published, the
+	    delivery is synchronized, and the subscription is destroyed.
+	  In both of the latter two cases, creating a dedicated thread is wasteful (and
+	  in the case of a large number of peers/endpoints, harmful). In those cases,
+	  having shared delivery threads is far more performant.
 
-	* res_pjsip_pubsub: Remove serializer when sending final NOTIFY.
+	  This patch adds the ability of a subscriber to Stasis to choose whether or not
+	  their messages are dispatched on a dedicated thread or on a threadpool. The
+	  threadpool is configurable through stasis.conf.
 
-	  There have been crashes seen where a taskprocessor's listener is NULL
-	  unexpectedly.
+	  Review: https://reviewboard.asterisk.org/r/4193
 
-	  Looking at backtraces, the problem was specifically seen in PJSIP
-	  serializers.
+	  ASTERISK-24533 #close
+	  Reported by: xrobau
+	  Tested by: xrobau
+	  ........
 
-	  Subscriptions make the mistake of removing a serializer from a dialog
-	  during subscription tree destruction. Since subscription trees are
-	  reference-counted, guaranteeing the circumstances behind the destruction
-	  are not possible. This makes it so that the dialog serializer can be
-	  removed while not holding the dialog lock. This makes it possible for
-	  the distributor to get a pointer to the dialog serializer and have that
-	  serializer get freed out from under it.
+	  Merged revisions 428681 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The fix for this is to remove the serializer from a subscription dialog
-	  when sending the final NOTIFY. This guarantees that the serializer is
-	  removed with the dialog lock held. By doing this, we guarantee that if
-	  the distributor gains access to the dialog's serializer, it will not be
-	  possible for the serializer to get freed by another thread.
+	  Merged revisions 428687 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I21f5dac33529f65cec45679bdace60670800ff66
 
-2015-09-02 09:14 +0000 [1bcc592765]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428688 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_pubsub: Fix crash on destruction of empty subscription tree.
+2014-12-01 07:41 +0000 [ef9ca8bc32]  Ben Smithurst (license 6529)
 
-	  If an old persistent subscription is recreated but then immediately
-	  destroyed because it is out of date, the subscription tree will have no
-	  leaf subscriptions on it. This was resulting in a crash when attempting
-	  to destroy the subscription tree.
+	* app_record: Fix bug where using the 'k' option and hanging up would trim 1/4 of a second of the recording.
 
-	  A simple NULL check fixes this problem.
+	  The Record dialplan function trims 1/4 of a second from the end of recordings in case
+	  they are terminated because of DTMF. When hanging up, however, you don't want this to happen.
+	  This change makes it so on hangup this does not occur.
 
-	  Change-Id: I85570b9e2bcc7260a3fe0ad85904b2a9bf36d2ac
+	  ASTERISK-24530 #close
+	  Reported by: Ben Smithurst
+	  patches:
+	   app_record_v2.diff submitted by Ben Smithurst (license 6529)
 
-2015-09-01 15:47 +0000 [b3cc2bd7df]  Mark Michelson <mmichelson@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4201/
+	  ........
 
-	* res_pjsip_pubsub: Solidify lifetime and ownership of objects.
+	  Merged revisions 428653 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  There have been crashes and general instability seen in the pubsub code,
-	  so this patch introduces three changes to increase the stability.
+	  Merged revisions 428654 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  First, the ownership model for subscriptions has been modified. Due to
-	  RLS, subscriptions are stored in memory as a tree structure. Prior to my
-	  patch, the PJSIP subscription was the owner of the subscription tree.
-	  When the PJSIP subscription told us that it was terminating, we started
-	  destroying the subscription tree along with all of the individual leaf
-	  subscriptions that belong to the tree. The problem with this model is
-	  that the two actors in play here, the PJSIP subscription and the
-	  individual leaf subscriptions, need to have joint ownership of the
-	  subscription tree. So now, the PJSIP subscription and the individual
-	  leaf subscriptions each have a reference to the subscription tree. This
-	  way, we will not actually free memory until no players are left that
-	  care. The PJSIP subscription is a bigger stakeholder, in that if the
-	  PJSIP subscription's reference to the subscription tree is removed, the
-	  subscription tree instructs the leaf subscriptions to shut down and drop
-	  their references to the subscription tree when possible. The individual
-	  leaf subscriptions, upon being told to shut down, can drop their stasis
-	  subscriptions or whatever they use to learn of new state, and then drop
-	  their reference to the subscription tree once they are ready to die.
+	  Merged revisions 428655 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Second, the lifetime of a PJSIP subscription's reference to our
-	  subscription tree has been altered. As I learned from doing a deep dive,
-	  the PJSIP evsub code can tell Asterisk multiple times that the
-	  subscription has been terminated, and not all of these times
-	  are especially helpful. I have altered the message flow that we use for
-	  SIP subscriptions such that we will always drop the PJSIP subscription's
-	  reference to the subscription tree when we send the NOTIFY that
-	  terminates a SIP subscription. This also means that we will now queue
-	  NOTIFY requests to be sent after responding to incoming SUBSCRIBEs so
-	  that we can have predictable state changes from the PJSIP evsub code.
 
-	  Third, the synchronization of operations has been improved. PJSIP can
-	  call into our code from a serializer thread (e.g. upon receiving an
-	  incoming request) or from the monitor thread (e.g. when a subscription
-	  times out). Because of this, there is the possibility of competing
-	  threads stepping on each other. PJSIP attempts to do some
-	  synchronization on its own by always keeping the dialog lock held when
-	  it calls into us. However, since we end up pushing tasks into the
-	  serializer, the result was that serialized operations were not grabbing
-	  the dialog lock and could, as a result, step on something that was being
-	  attempted by a different thread. Now we ensure that serialized
-	  operations grab the dialog lock, then check for extenuating
-	  circumstances, then proceed with their operation if they can.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428656 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iff2990c40178dad9cc5f6a5c7f76932ec644b2e5
+2014-12-01 07:08 +0000 [7db3d1642b]  snuffy <snuffy22@gmail.com> (license 5024)
 
-2015-10-19 15:28 +0000 [c8c65dfa41]  Richard Mudgett <rmudgett@digium.com>
+	* channel: Extend size of buffer for codecs in "core show channeltype" CLI command.
 
-	* strings.c: Fix __ast_str_helper() to always return a terminated string.
+	  The static buffer for codecs when invoking the "core show channeltype" CLI command
+	  did not have enough room for all codecs. This has been extended so it does.
 
-	  Users of functions which call __ast_str_helper() such as the ones listed
-	  below are likely to not check the return value for failure so ensuring
-	  that the string is always nil terminated is a good safety measure.
+	  ASTERISK-24542 #close
+	  Reported by: snuffy
+	  patches:
+	   channeltype-tech.diff submitted by snuffy (license 5024)
 
-	  ast_str_set_va()
-	  ast_str_append_va()
-	  ast_str_set()
-	  ast_str_append()
+	  Review: https://reviewboard.asterisk.org/r/4204/
+	  ........
 
-	  Change-Id: I36ab2d14bb6015868b49329dda8639d70fbcae07
+	  Merged revisions 428632 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-10-19 15:27 +0000 [b271d4a28a]  Richard Mudgett <rmudgett@digium.com>
 
-	* Add missing failure checks to ast_str_set_va() callers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428633 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I0c2cdcd53727bdc6634095c61294807255bd278f
+2014-11-24 14:39 +0000 [3e08619faf]  Richard Mudgett <rmudgett@digium.com>
 
-2015-10-21 11:44 +0000 [f2725c8b77]  Joshua Colp <jcolp@digium.com>
+	* test_channel_feature_hooks.c: Fix unit test for DTMF hooks.
 
-	* res_pjsip: Move URI validation to use time.
+	  Fix the failing /channels/features/test_features_channel_dtmf unit test.
 
-	  In a realtime based system with a limited number of threadpool threads
-	  it is possible for a deadlock to occur. This happens when permanent
-	  endpoint state is updated, which will cause database queries to be done.
-	  These queries may result in URI validation being done which is done
-	  synchronously using a PJSIP thread. If all PJSIP threads are in use
-	  processing traffic they themselves may be blocked waiting to get the
-	  permanent endpoint container lock when identifying an endpoint.
+	  DTMF emulation does not work without a stream of packets to prod the
+	  emulation code.
 
-	  This change moves URI validation to occur at use time instead of
-	  configuration time. While this comes at a cost of not seeing a problem
-	  until you use it it does solve the underlying deadlock problem.
+	  Review: https://reviewboard.asterisk.org/r/4199/
+	  ........
 
-	  ASTERISK-25486 #close
+	  Merged revisions 428604 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2d7d167af987d23b3e8199e4a68f3359eba4c76a
 
-2015-10-21 08:08 +0000 [84ff075d41]  Alexander Traud <pabstraud@compuserve.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428605 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* format: Update the maximum packetization time for iLBC 30.
+2014-11-24 14:32 +0000 [c38ffca9a1]  Richard Mudgett <rmudgett@digium.com>
 
-	  In September 2006, the maximum packetization time (ptime) were set to such a
-	  low value, packetization was disabled for many codecs actually. This was fixed
-	  for many codecs but not for iLBC 30. This enables packetization for iLBC which
-	  can be enabled for example via allow=ilbc:60,gsm,alaw,ulaw in the file sip.conf.
+	* DTMF hooks: Leaving channels need to push any collected digits into the bridge.
 
-	  ASTERISK-7803
+	  Any partially collected DTMF digits for a DTMF hook need to be pushed into
+	  the bridge when a channel leaves the bridging system as if there were a
+	  timeout.
 
-	  Change-Id: I2ef90023d35efb7cb8fe96ed74f53f6846ffad12
-2015-10-21 09:51 +0000 [869ef2a8ee]  Alexander Traud <pabstraud@compuserve.com>
+	  Review: https://reviewboard.asterisk.org/r/4199/
+	  ........
 
-	* chan_sip: Fix autoframing=yes.
+	  Merged revisions 428601 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  With Asterisk 13, the structures ast_format and ast_codec changed. Because of
-	  that, the paketization timing (framing) of the RTP channel moved away from the
-	  formats/codecs. In the course of that change, the ptime of the callee was not
-	  honored anymore, when the optional autoframing was enabled.
+	  Merged revisions 428602 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25484 #close
 
-	  Change-Id: Ic600ccaa125e705922f89c72212c698215d239b4
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428603 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-10-20 22:24 +0000 [9fd2adc204]  Matt Jordan <mjordan@digium.com>
+2014-11-21 13:16 +0000 [3576ae47f4]  Richard Mudgett <rmudgett@digium.com>
 
-	* rest-api-templates: Wikify error code response reasons
+	* manager: Fix could not extend string messages.
 
-	  Error response code descriptions may contain wiki markup that need to be
-	  escaped. Without this patch, Confluence will reject the document being sent
-	  and the responsible script will raise an exception.
+	  When shutting down Asterisk that has an active AMI connection, you get
+	  several "failed to extend from %d to %d" messages because use of the
+	  EVENT_FLAG_SHUTDOWN attempts to add all AMI permission strings to the
+	  event.
 
-	  Change-Id: I21fcb66fee7f6332381f2b99b1b0195dff215ee5
+	  * Created MAX_AUTH_PERM_STRING to use when creating stack based struct
+	  ast_str variables used with the authority_to_str() and
+	  user_authority_to_str() functions instead of a variety of magic numbers
+	  that could be too small.
 
-2015-10-20 12:06 +0000 [72cbb6df55]  Matt Jordan <mjordan@digium.com>
+	  * Added a special check for EVENT_FLAG_SHUTDOWN to authority_to_str() so
+	  it will not attempt to add all permission level strings.
 
-	* funcs/func_holdintercept: Actually add the HOLD_INTERCEPT function
+	  Review: https://reviewboard.asterisk.org/r/4200/
+	  ........
 
-	  When ab803ec342 was committed, it accidentally forgot to actually *add* the
-	  HOLD_INTERCEPT function. This highlights two interesting points:
-	  * Gerrit forces you to put the patch as it is going to into the repo up for
-	    review, which Review Board did not. Yay Gerrit.
-	  * No one apparently bothered to use this feature, or else they don't know about
-	    it. I'm going to go with the latter explanation.
+	  Merged revisions 428570 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-24922
+	  Merged revisions 428571 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Ida38278f259dd07c334a36f9b7d5475b5db72396
+	  Merged revisions 428572 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-10-19 19:59 +0000 [9fc9777fa3]  Matt Jordan <mjordan@digium.com>
 
-	* contrib/scripts/autosupport: Update for Asterisk 13
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428573 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch adds some minor tweaks for autosupport to update it for Asterisk 13.
-	  This includes:
-	  * Finally removing most references to Zaptel
-	  * Adding support for some additional 'core' commands, and fixing nomenclature
-	    that generally hasn't been used for some time
-	  * Adding some PJSIP/SIP commands to gather endpoints/peers and active channels
+2014-11-21 11:49 +0000 [4394e0431c]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Ic997b418cbd9313588b6608e50f47b0ce6f4f1f1
+	* sorcery: Make is_object_field_registered handle field names that are regexes.
 
-2015-10-14 14:15 +0000 [dc6ec661b3]  mdu113 <mulitskiy@acedsl.com>
+	  As a result of https://reviewboard.asterisk.org/r/3305, res_sorcery_realtime
+	  was tossing database fields that didn't have an exact match to a sorcery
+	  registered field.  This broke the ability to use regexes as field names which
+	  manifested itself as a failure of res_pjsip_phoneprov_provider which uses
+	  this capability.  It also broke handling of fields that start with '@' in
+	  realtime but I don't think anyone noticed.
 
-	* res_config_pgsql.c: Fix deadlock loading realtime configuration.
+	  This patch does the following...
+	  * Modifies ast_sorcery_fields_register to pre-compile the name regex.
+	  * Modifies ast_sorcery_is_object_field_registered to test the regex if it
+	    exists instead of doing an exact strcmp.
+	  * Modifies res_pjsip_phoneprov_provider with a few tweaks to get it to work
+	    with realtime.
 
-	  On v13, loading several thousand PJSIP endpoints on Asterisk start causes
-	  a deadlock most of the time.
+	  Tested-by: George Joseph
 
-	  Thanks to mdu113 for discovering that there was a call to pgsql_exec() not
-	  protected by the pgsql_lock reentrancy lock.
+	  Review: https://reviewboard.asterisk.org/r/4185/
+	  ........
 
-	  {quote}
-	  I believe a code path exists that attempts to use pgsql connection without
-	  locking pgsql_lock.  I believe what happens during that deadlock that I
-	  see is two concurrent threads are both attempting to send query to pgsql,
-	  one of the thread is using a code path without locking pgsql_lock.  If
-	  they managed to send queries at the same time, it seems postgres ignores
-	  one of the queries and replies only to the one of them.  If it happens so
-	  that the thread holding the lock didn't receive the reply it will wait for
-	  it (and hold the lock) forever (or at least for very long time), thus
-	  completely blocking all access to db.
-	  {quote}
+	  Merged revisions 428543 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Added missing reentrancy locking around pgsql_exec() in find_table().
+	  Merged revisions 428544 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Moved unlock of pgsql_lock in unload_module() to avoid locking inversion
-	  between the psql_tables list lock and the pgsql_lock.
 
-	  ASTERISK-25455 #close
-	  Reported by:  mdu113
-	  Patches:
-	        res_config_pgsql.c-connlock2.diff (license #5543) patch uploaded by mdu113
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428545 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id9e7cdf8a3b65ff19964b0cf942ace567938c4e2
+2014-11-21 07:59 +0000 [d663e045f5]  Olle Johansson <oej@edvina.net>
 
-2015-10-13 14:13 +0000 [f8707ae9a5]  Olle Johansson (License 5267)
+	* sip.conf.sample - note that media_address does not change listen address, just the SDP
 
-	* channels/chan_sip: Set cause code to 44 on RTP timeout
 
-	  To quote Olle:
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428526 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  "When issuing a hangup due to RTP timeouts the cause code is not set. I have
-	  selected 44 based on Cisco's implementation..."
+2014-11-20 20:17 +0000 [2be984fb11]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25135 #close
-	  Reported by: Olle Johansson
-	  patches:
-	    rtp-timeout-cause-1.8.diff uploaded by Olle Johansson (License 5267)
+	* main/bridge_basic: Fix features regressions introduced by r428165
 
-	  Change-Id: Ia62100c55077d77901caee0bcae299f8dc7375fc
+	  In r428165, two bugs were introduced:
 
-2015-10-10 15:20 +0000 [486b172b50]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  * Prior to entering the features retry loop, the buffer that holds the
+	    collected digits is wiped. However, this inadvertently wipes out the
+	    first collected digit on the first pass through, which is obtained
+	    in ast_stream_and_wait. This caused all of the features tests to fail.
+	  * If ast_app_dtget returns a hangup (-1), the loop would retry incorrectly.
+	    If we detect a hangup, we have to stop trying the feature.
 
-	* Build: Add menuselect options for using compiler sanitizers
+	  This patch fixes both issues.
 
-	  This patch adds menuselect options for building Asterisk with
-	  various sanitizers provided by gcc and clang.
+	  Review: https://reviewboard.asterisk.org/r/4196/
+	  ........
 
-	  When one of *SANITIZER flags is set in menuselect, the appropriate
-	  option is added to CFLAGS ad LDFLAGS for the build.
+	  Merged revisions 428505 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Information on sanitizers in the project wiki:
-	  https://github.com/google/sanitizers/wiki
 
-	  GCC Manual:
-	  https://gcc.gnu.org/onlinedocs/gcc/Debugging-Options.html
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428506 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Clang Compiler User's Manual:
-	  http://clang.llvm.org/docs/UsersManual.html#controlling-code-generation
+2014-11-20 10:37 +0000 [2f78fde10f]  Matt Jordan <mjordan@digium.com> (License #6283)
 
-	  ASTERISK-24718 #close
-	  Reported by: Badalian Vyacheslav
+	* Fix error with mixed address family ACLs.
 
-	  Change-Id: Iafa51b792b7bcb20e848b99d16cf362d08590fa0
+	  Prior to this commit, the address family of the first item in an ACL
+	  was used to compare all incoming traffic. This could lead to traffic
+	  of other IP address families bypassing ACLs.
 
-2015-10-12 11:21 +0000 [e14023ca35]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24469 #close
 
-	* config.c: Fix off-nominal memory leak.
+	  Reported by Matt Jordan
+	  Patches:
+	  	ASTERISK-24469-11.diff uploaded by Matt Jordan (License #6283)
 
-	  Change-Id: I06e346e9a5c63cc5071e7eda537310c4b43bffe0
+	  AST-2014-012
+	  ........
 
-2015-10-12 11:20 +0000 [a99e821520]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428402 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* config.c: Fix potential memory corruption after [section](+).
+	  Merged revisions 428417 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The memory corruption could happen if the [section](+) is the last section
-	  in the file with trailing comments.  In this case process_text_line() has
-	  left *last_cat is set to newcat and newcat is destroyed.
+	  Merged revisions 428422 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I0d1d999f553986f591becd000e7cc6ddfb978d93
+	  Merged revisions 428425 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-10-12 11:21 +0000 [8d31d2526b]  Richard Mudgett <rmudgett@digium.com>
 
-	* config.c: Fix #include after [section](+).
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428426 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  An #include right after a [section](+) would associate any variable
-	  assignments before a new section in the #include with the wrong section.
+2014-11-20 10:35 +0000 [2486b48cec]  Gareth Palmer (license 5169)
 
-	  * Fix section association by setting the current section to the appended
-	  section.
+	* AST-2014-018 - func_db: DB Dialplan function permission escalation via AMI.
 
-	  * Fix '+' and '!' section flag interaction corner case depending upon
-	  which flag came first.  If the '!' came first then it would be ignored.
-	  If the '!' came after then it would affect the appended section.  The '!'
-	  will now no longer be ignored.
+	  The DB dialplan function when executed from an external protocol (for instance
+	  AMI), could result in a privilege escalation.
 
-	  ASTERISK-25461 #close
-	  Reported by: Sean Pimental
+	  Asterisk now inhibits the DB function from being executed from an external
+	  interface if the live_dangerously option is set to no.
 
-	  Change-Id: Ic9d3191c8758048e2cbce6432f854b32531731c3
+	  ASTERISK-24534
+	  Reported by: Gareth Palmer
+	  patches: submitted by Gareth Palmer (license 5169)
+	  ........
 
-2015-10-06 18:01 +0000 [3329c714f7]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428331 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* res_pjsip: Fix deadlock when sending out-of-dialog requests.
+	  Merged revisions 428363 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The struct send_request_wrapper has a pjsip lock associated with it that
-	  is created non-recursive.  There is a code path for the struct
-	  send_request_wrapper lock that will attempt to lock it recursively.  The
-	  reporter's deadlock showed that the thread calling endpt_send_request()
-	  deadlocked itself right after the wrapper object got created.
+	  Merged revisions 428409 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Out-of-dialog requests such as MESSAGE, qualify OPTIONS, and unsolicited
-	  MWI NOTIFY messages can hit this deadlock.
+	  Merged revisions 428413 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Replaced the struct send_request_wrapper pjsip lock with the mutex lock
-	  that can come with an ao2 object since all of Asterisk's mutexes are
-	  recursive.  Benefits include removal of code maintaining the pjsip
-	  non-recursive lock since ao2 objects already know how to maintain their
-	  own lock and the lock will show up in the CLI "core show locks" output.
 
-	  ASTERISK-25435 #close
-	  Reported by: Dmitriy Serov
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428418 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I458e131dd1b9816f9e963f796c54136e9e84322d
+2014-11-20 10:25 +0000 [2f97486d43]  Jonathan Rose <jrose@digium.com>
 
-2015-10-06 11:05 +0000 [a1435aa3fa]  Stefan Engström <stefanen@kth.se>
+	* PJSIP ACLs: Fix ACLs not loading on startup and apply/acl issues on contact
 
-	* res/res_rtp_asterisk.c: Fix incorrect assignment of frame->subclass.frame_ending
+	  The biggest problem this patch fixes is that ACLs weren't previously being
+	  loaded when the res_pjsip_acl module was loaded. Yikes. In addition, the
+	  ACL options contact_permit and contact_acl were effectively interpreted as
+	  contact_deny and this patch fixes that as well.
 
-	  In ast_rtp_read, the value of the variable 'mark' which we try to assign to a
-	  frame->subclass.frame_ending may be 0, 1 or (1<<23), but we should translate
-	  it to 0 or 1.
+	  AST-1418 #close
+	  Reported by: Thomas Thompson
+	  Review: https://reviewboard.asterisk.org/r/4120/
 
-	  ASTERISK-25451 #close
-	  Change-Id: I53bdf5c026041730184a6a809009c028549ce626
+	  ASTERISK-24531 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4171/
+	  ........
 
-2015-10-07 01:24 +0000 [3357678b94]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Merged revisions 428333 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* func_presencestate: Return "not_set" when no data is set in AstDB
+	  Merged revisions 428343 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Return AST_PRESENCE_NOT_SET when CustomPresence AstDB key does not
-	  exist, i.e. when a new CustomPresence is added in the dialplan.
 
-	  ASTERISK-25400 #close
-	  Reported by: Andrew Nagy
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428376 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I6fb17b16591b5a55fbffe96f3994ec26b1b1723a
+2014-11-20 09:57 +0000 [a389f2d7a0]  Kevin Harwell <kharwell@digium.com>
 
-2015-10-06 20:43 +0000 [b714b2152d]  Matt Jordan <mjordan@digium.com>
+	* AST-2014-017 - app_confbridge: permission escalation/ class authorization.
 
-	* res/res_rtp_asterisk: Fix assignment after ao2 decrement
+	  Confbridge dialplan function permission escalation via AMI and inappropriate
+	  class authorization on the ConfbridgeStartRecord action. The CONFBRIDGE dialplan
+	  function when executed from an external protocol (for instance AMI), could
+	  result in a privilege escalation. Also, the AMI action “ConfbridgeStartRecord”
+	  could also be used to execute arbitrary system commands without first checking
+	  for system access. The AMI “ConfbridgeStopRecord” has also been updated to
+	  only run under a system authorization.
 
-	  When we decide we will no longer schedule an RTCP write, we remove the
-	  reference to the RTP instance, then assign -1 to the stored scheduler ID
-	  in case something else comes along and wants to see if anything is scheduled.
+	  Asterisk now inhibits the CONFBRIDGE function from being executed from an
+	  external interface if the live_dangerously option is set to no.  Also, the
+	  “ConfbridgeStartRecord” AMI action is now only allowed to execute under a
+	  user with system level access.
 
-	  That scheduler ID is on the RTP instance. After 60a9172d7ef2 was merged to
-	  fix the regression introduced by 3cf0f29310, this improper assignment on a
-	  potentially destroyed object started getting tripped on the build agents.
+	  ASTERISK-24490
+	  Reported by: Gareth Palmer
+	  ........
 
-	  Frankly, this should have been crashing a lot more often earlier. I can only
-	  assume that the timing was changed just enough by both changes to start
-	  actually hitting this problem.
+	  Merged revisions 428332 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  As it is, simply moving the assignment prior to the ao2 deference is sufficient
-	  to keep the RTP instance from being referenced when it is very, truly,
-	  aboslutely dead.
+	  Merged revisions 428334 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  (Note that it is still good practice to assign -1 to the scheduler ID when we
-	  know we won't be scheduling it again, as the ao2 deref *may* not always destroy
-	  the ao2 object.)
+	  Merged revisions 428339 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25449
 
-	  Change-Id: Ie6d3cb4adc7b1a6c078b1c38c19fc84cf787cda7
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428342 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-10-06 12:40 +0000 [f939e2bd48]  Florian Sauerteig <ffs@ccn.net>
+2014-11-20 08:56 +0000 [1c88ca9d31]  Joshua Colp <jcolp@digium.com>
 
-	* chan_sip: Fix port parsing for IPv6 addresses in SIP Via headers.
+	* AST-2014-016: Fix crash when receiving an in-dialog INVITE with Replaces in res_pjsip_refer.
 
-	  If a Via header containes an IPv6 address and a port number is ommitted,
-	  as it is the standard port, we now leave the port empty and to not set it
-	  to the value after the first colon of the IPv6 address.
+	  The implementation of INVITE with Replaces in res_pjsip_refer did not expect them to
+	  occur in-dialog. As a result it would incorrectly attempt to hang up a channel it
+	  thought was under its control. In reality the channel would be under the control of
+	  another thread. When the other thread accessed the channel it would be accessing freed
+	  memory and could crash.
 
-	  ASTERISK-25443 #close
+	  This change makes res_pjsip_refer not act on an in-dialog INVITE with Replaces.
 
-	  Change-Id: Ie3c2f05471cd006bf04ed15598589c09577b1e70
+	  ASTERISK-24528 #close
+	  Reported by: Joshua Colp
+	  ........
 
-2015-10-05 16:53 +0000 [426263a64d]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428304 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* chan_pjsip: Fix crash on reINVITE before initial INVITE completes.
+	  Merged revisions 428305 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Apparently some endpoints attempt to send a reINVITE before completing the
-	  initial INVITE transaction.  In this case PJSIP responds appropriately to
-	  the reINVITE with a 491 INVITE request pending.  Unfortunately chan_pjsip
-	  is using the initial INVITE transaction state to determine if an INVITE is
-	  the initial INVITE or a reINVITE.  Since the initial INVITE transaction
-	  has not been confirmed yet chan_pjsip thinks the reINVITE is an initial
-	  INVITE and starts another PBX thread on the channel.  The extra PBX thread
-	  ensures that hilarity ensues.
 
-	  * Fix checks for a reINVITE on incoming requests to look for the presence
-	  of a to-tag instead of the initial INVITE transaction state.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428306 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Made caller_id_incoming_request() determine what to do if there is a
-	  channel on the session or not.  After a channel is created it is too late
-	  to just store the new party id on the session because the session's party
-	  id has already been copied to the channel's caller id.
+2014-11-20 08:49 +0000 [d25eda5fb2]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25404 #close
-	  Reported by: Chet Stevens
+	* AST-2014-015: Fix race condition in chan_pjsip when sending responses after a CANCEL has been received.
 
-	  Change-Id: Ie78201c304a2b13226f3a4ce59908beecc2c68be
+	  Due to the serialized architecture of chan_pjsip there exists a race condition where a CANCEL may
+	  be received and processed before responses (such as 180 Ringing, 183 Session Progress, and 200 OK)
+	  are sent. Since the session is in an unexpected state PJSIP will assert when this is attempted.
 
-2015-10-05 21:34 +0000 [50fa9ff997]  Matt Jordan <mjordan@digium.com>
+	  This change makes it so that these responses are not sent on disconnected sessions.
 
-	* Fix improper usage of scheduler exposed by 5c713fdf18f
+	  ASTERISK-24471 #close
+	  Reported by: yaron nahum
+	  ........
 
-	  When 5c713fdf18f was merged, it allowed for scheduled items to have an ID of
-	  '0' returned. While this was valid per the documentation for the API, it was
-	  apparently never returned previously. As a result, several users of the
-	  scheduler API viewed the result as being invalid, causing them to reschedule
-	  already scheduled items or otherwise fail in interesting ways.
+	  Merged revisions 428301 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch corrects the users such that they view '0' as valid, and a returned
-	  ID of -1 as being invalid.
+	  Merged revisions 428302 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Note that the failing HEP RTCP tests now pass with this patch. These tests
-	  failed due to a duplicate scheduling of the RTCP transmissions.
 
-	  ASTERISK-25449 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428303 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I019a9aa8b6997584f66876331675981ac9e07e39
-2015-08-26 16:58 +0000 [8f777ab584]  Debian Amtelco <dan@amtelco.com>
+2014-11-19 13:32 +0000 [57c6f89bf0]  Corey Farrell <git@cfware.com>
 
-	* chan_pjsip: Add Referred-By header to the PJSIP REFER packet.
+	* stringfields: Fix bug in ast_string_fields_copy.
 
-	  Some systems require the REFER packet to include a Referred-By header.
-	  If the channel variable SIPREFERREDBYHDR is set, it passes that value as the
-	  Referred-By header value.  Otherwise, it adds the current dialog’s local info.
+	  ast_string_fields_copy relies on the fact that
+	  __ast_string_field_release_active never previously
+	  zeroed pool->used, so keeping the existing pointer
+	  was "ok".  Now that existing pools can be reset to
+	  'empty', it is important to set each field to
+	  __ast_string_field_empty after releasing the memory.
 
-	  Reported by: Dan Cropp
-	  Tested by: Dan Cropp
+	  ASTERISK-24535 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4186/
+	  ........
 
-	  Change-Id: I3d17912ce548667edf53cb549e88a25475eda245
+	  Merged revisions 428272 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-10-03 06:27 +0000 [74635b5638]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Merged revisions 428273 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* manager: Fix GetConfigJSON returning invalid JSON
 
-	  When GetConfigJSON was introduced back in 1.6, it returned each
-	  section as an array of strings: ["key=value", "key2=value2"].
-	  Afterwards, it was changed a few times and became
-	  ["key": "value", "key2": "value2"], which is not a correct JSON.
-	  This patch fixes that by constructing a JSON object {} instead of
-	  an array [].
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428274 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Also, the keys "istemplate" and "tempates" that are used to
-	  indicate templates and their inherited categories are now wrapped in
-	  quotes.
+2014-11-19 11:22 +0000 [a7c9f4c668]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25391 #close
-	  Reported by: Bojan Nemčić
+	* ast_str: Fix improper member access to struct ast_str members.
 
-	  Change-Id: Ibbe93c6a227dff14d4a54b0d152341857bcf6ad8
+	  Accessing members of struct ast_str outside of the string manipulation API
+	  routines is invalid since struct ast_str is supposed to be treated as
+	  opaque.
 
-2015-09-30 17:28 +0000 [40c69e78f5]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4194/
+	  ........
 
-	* res_sorcery_memory_cache.c: Fix deadlock with scheduler.
+	  Merged revisions 428244 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  A deadlock can happen when a sorcery object is being expired from the
-	  memory cache when at the same time another object is being placed into the
-	  memory cache.  There are a couple other variations on this theme that
-	  could cause the deadlock.  Basically if an object is being expired from
-	  the sorcery memory cache at the same time as another thread tries to
-	  update the next object expiration timer the deadlock can happen.
+	  Merged revisions 428245 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Add a deadlock avoidance loop in expire_objects_from_cache() to check if
-	  someone is trying to remove the scheduler callback from the scheduler.
+	  Merged revisions 428246 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25441 #close
 
-	  Change-Id: Iec7b0bdb81a72b39477727b1535b2539ad0cf4dc
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428255 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-10-01 14:30 +0000 [dfeb513e85]  Richard Mudgett <rmudgett@digium.com>
+2014-11-19 06:50 +0000 [7f8b7ace72]  Joshua Colp <jcolp@digium.com>
 
-	* res_sorcery_memory_cache.c: Replace inline code with function.
+	* res_pjsip_sdp_rtp: Add support for optimistic SRTP.
 
-	  Make sorcery_memory_cache_close() call remove_all_from_cache() instead of
-	  partially inlining it.
+	  Optimistic SRTP is the ability to enable SRTP but not have it be
+	  a fatal requirement. If SRTP can be used it will be, if not it won't be.
+	  This gives you a better chance of using it without having your sessions
+	  fail when it can't be.
 
-	  ASTERISK-25441
+	  Encrypt all the things!
 
-	  Change-Id: I1aa6cb425b1a4307096f3f914d17af8ec179a74c
+	  Review: https://reviewboard.asterisk.org/r/3992/
+	  ........
 
-2015-10-01 14:27 +0000 [ced0a2d71b]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428222 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_sorcery_memory_cache.c: Shutdown in a less crash potential order.
 
-	  Basically you should shutdown in the opposite order of how you setup since
-	  later setup pieces likely depend on earlier setup pieces.  e.g.,
-	  Registering your external API with the rest of the system should be the
-	  last thing setup and the first thing unregistered during shutdown.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428224 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I5715765b723100c8d3c2642e9e72cc7ad5ad115e
+2014-11-19 06:45 +0000 [b2e766a6b7]  Joshua Colp <jcolp@digium.com>
 
-2015-09-30 17:27 +0000 [cc279eea11]  Richard Mudgett <rmudgett@digium.com>
+	* alembic: Fix alembic migration for 'moh_passthrough' option in res_pjsip.
 
-	* res_sorcery_memory_cache.c: Misc tweaks.
 
-	  Change-Id: I8cd32dffbb4f33bb0c39518d6e4c991e73573160
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428223 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-30 17:27 +0000 [9af3b613f6]  Richard Mudgett <rmudgett@digium.com>
+2014-11-19 05:51 +0000 [3119c3737f]  Joshua Colp <jcolp@digium.com>
 
-	* res_sorcery_memory_cache.c: Made use OBJ_SEARCH_MASK.
+	* res_pjsip_refer: Ensure Refer-To is NULL terminated and parse it as a URI.
 
-	  Change-Id: Ibca6574dc3c213b29cc93486e01ccd51f5caa46c
+	  There is no guarantee that when we get a Refer-To that it will be NULL terminated.
+	  As the URI parsing function requires it to be we now NULL terminate it.
 
-2015-09-30 13:42 +0000 [56ed7b9dd5]  Joshua Colp <jcolp@digium.com>
+	  Additionally parsing the Refer-To as a 'To' header is needless and it can
+	  simply be done as a URI. This also fixes a problem where certain Refer-To headers
+	  would not be parsed as a 'To' header causing the REFER to fail.
 
-	* res_rtp_asterisk: Move "Set role" warning to be debug.
+	  ASTERISK-24508 #close
+	  Reported by: Beppo Mazzucato
 
-	  In practice the set_role API callback can be invoked even
-	  when no ICE is present on an RTP instance. This can occur
-	  if ICE has not been enabled on it.
+	  Review: https://reviewboard.asterisk.org/r/4187/
+	  ........
 
-	  ASTERISK-25438 #close
+	  Merged revisions 428195 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I0e17e4316f0f0d7f095c78c3d4fd73a913b6ba69
+	  Merged revisions 428196 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-28 15:31 +0000 [ddebb217f0]  Richard Mudgett <rmudgett@digium.com>
 
-	* sched.c: Add warning about negative time interval request.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428197 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib91435fb45b7f5f7c0fc83d0eec20b88098707bc
+2014-11-18 13:12 +0000 [a94efa239c]  Richard Mudgett <rmudgett@digium.com>
 
-2015-09-29 21:15 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* parking_tests.c: Add missing newline on a unit test message.
+	  ........
 
-	* asterisk 13.6.0-rc1 Released.
+	  Merged revisions 428168 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-09-29 16:12 +0000 [bba1c4066b]  Kevin Harwell <kharwell@lunkwill>
+	  Merged revisions 428169 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Release summaries: Add summaries for 13.6.0-rc1
 
-2015-09-29 16:08 +0000 [82c4aecdbb]  Kevin Harwell <kharwell@lunkwill>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428170 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* .version: Update for 13.6.0-rc1
+2014-11-17 10:58 +0000 [2e750db120]  Mark Michelson <mmichelson@digium.com>
 
-2015-09-29 16:08 +0000 [bc18db7388]  Kevin Harwell <kharwell@lunkwill>
+	* Allow for transferer to retry when dialing an invalid extension.
 
-	* .lastclean: Update for 13.6.0-rc1
+	  This allows for a configurable number of attempts for a transferer
+	  to dial an extension to transfer the call to. For Asterisk 13, the
+	  default values are such that upgrading between versions will not
+	  cause a behaivour change. For trunk, though, the defaults will be
+	  changed to be more user-friendly.
 
-2015-09-29 16:08 +0000 [b9c53f95e3]  Kevin Harwell <kharwell@lunkwill>
+	  Review: https://reviewboard.asterisk.org/r/4167
+	  ........
 
-	* realtime: Add database scripts for 13.6.0-rc1
+	  Merged revisions 428145 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-29 14:53 +0000 [d30939b6e8]  Kevin Harwell <kharwell@digium.com>
 
-	* ARI: Changed version from 1.8.0 to 1.9.0
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428146 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I510991c60d28d171f47c4b58bba4947f7fc71b13
+2014-11-17 10:02 +0000 [4cea5fd4ba]  Corey Farrell <git@cfware.com>
 
-2015-09-25 18:37 +0000 [5f19c9bade]  Richard Mudgett <rmudgett@digium.com>
+	* chan_sip: Fix theoretical leak of p->refer.
 
-	* res/ari/config.c: Fix user sort compare function.
+	  If transmit_refer is called when p->refer is already allocated,
+	  it leaks the previous allocation.  Updated code to always free
+	  previous allocation during a new allocation.  Also instead of
+	  checking if we have a previous allocation, always create a
+	  clean record.
 
-	  Made use the ao2 sort compare template function and OBJ_SEARCH_xxx
-	  identifiers.
+	  ASTERISK-15242 #close
+	  Reported by: David Woolley
+	  Review: https://reviewboard.asterisk.org/r/4160/
+	  ........
 
-	  Change-Id: Ic53005dc5aafa7a36c72300dd89b75fb63c92f4c
+	  Merged revisions 428117 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-09-25 17:26 +0000 [3a85764039]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428118 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res/ari/config.c: Optimize conf_alloc() object init.
+	  Merged revisions 428119 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Now conf_alloc() has more off nominal error checking.
 
-	  * Eliminated RAII_VAR() use in conf_alloc().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428120 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Eliminated a dubius shortcut when destroying cfg->general in
-	  conf_destructor() that would cause a crash if cfg->general failed to get
-	  allocated.
+2014-11-17 09:27 +0000 [948af7fd79]  Matt Jordan <mjordan@digium.com>
 
-	  * Add some ACO registration section comments.
+	* apps/app_confbridge: Ensure 'normal' users hear message when last marked leaves
 
-	  Change-Id: Ia40c2b1b2d0777d641605118ae019c5a73865e1a
+	  When r428077 was made for ASTERISK-24522, it failed to take into account users
+	  who are neither wait_marked nor end_marked. These users are *also* supposed to
+	  hear the 'leader has left the conference' message. Granted, this behaviour is
+	  a bit odd; however, that is how it used to work... and behaviour changes are
+	  not good.
 
-2015-09-25 16:48 +0000 [028033e5a8]  Richard Mudgett <rmudgett@digium.com>
+	  This patch ensures that if there are any 'normal' users present when the last
+	  marked user leaves the conference, the message will still be played to them.
 
-	* res/ari/config.c: Fix conf_alloc() object init.
+	  Note that this regression was caught by the Asterisk Test Suite's
+	  confbridge_nominal test, which has a quirky combination of users.
+	  ........
 
-	  Need to finish initializing the string fields in the ao2 object before
-	  putting any default strings into them.
+	  Merged revisions 428113 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25383 #close
-	  Reported by:  yaron nahum
+	  Merged revisions 428114 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I9f7f3a03f0c4991a01593abf8697b9a587c0ea84
+	  Merged revisions 428115 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-27 20:45 +0000 [90165e306d]  Matt Jordan <mjordan@digium.com>
 
-	* res/res_stasis: Fix accidental subscription to 'all' bridge topic
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428116 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When b99a7052621700a1aa641a1c24308f5873275fc8 was merged, subscribing to a
-	  NULL bridge will now cause app_subscribe_bridge to implicitly subscribe to
-	  all bridges. Unfortunately, the res_stasis control loop did not check that
-	  a bridge changing on a channel's control object was actually also non-NULL.
-	  As a result, app_subscribe_bridge will be called with a NULL bridge when a
-	  channel leaves a bridge. This causes a new subscription to be made to the
-	  bridge. If an application has also subscribed to the bridge, the application
-	  will now have two subscriptions:
-	  (1) The explicit one created by the app
-	  (2) The implicit one accidentally created by the control structure
+2014-11-16 21:08 +0000 [fc2279afea]  Matt Jordan <mjordan@digium.com>
 
-	  As a result, the 'BridgeDestroyed' event can be sent multiple times. This
-	  patch corrects the control loop such that it only subscribes an application
-	  to a new bridge if the bridge pointer is non-NULL.
+	* app_confbridge: Don't play leader leaving prompt if no one will hear it
 
-	  ASTERISK-24870
+	  Consider the following:
+	  - A marked user in a conference
+	  - One or more end_marked only users in the conference
 
-	  Change-Id: I3510e55f6bc36517c10597ead857b964463c9f4f
+	  When the marked users leaves, we will be in the conf_state_multi_marked state.
+	  This currently will traverse the users, kicking out any who have the end_marked
+	  flags. When they are kicked, a full ast_bridge_remove is immediately called on
+	  the channels. At this time, we also unilaterally set the need_prompt flag.
 
-2015-09-04 13:51 +0000 [e1223ff6db]  Scott Griepentrog <scott@griepentrog.com>
+	  When the need_prompt flag is set, we then playback a sound to the bridge
+	  informing everyone that the leader has left; however, no one is left in the
+	  bridge. This causes some odd behaviour for the end_marked users - they are
+	  stuck waiting for the bridge to be unlocked. This results in them waiting for
+	  5 or 6 seconds of dead air before hearing that they've been kicked.
 
-	* Scripts: check file versions of Asterisk and dependencies
+	  Unfortunately, we do have to keep the bridge locked while we're playing back
+	  the 'leader-has-left' prompt. If there are any wait_marked users in the
+	  conference, this behaviour can't be easily changed - but we do make the case
+	  of the end_marked users better with this patch.
 
-	  To help in diagnosing mismatched modules and libraries, this
-	  script scans for version, repository, and source information
-	  and reports what is found.
+	  Review: https://reviewboard.asterisk.org/r/4184/
 
-	  ASTERISK-25376 #close
-	  Reported by: Ashley Sanders
+	  ASTERISK-24522 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  Change-Id: Ib0642d0fb96712476f59760d6d137a24633fe2d6
+	  Merged revisions 428077 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-09-24 14:56 +0000 [6b1e7583c1]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 428078 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* app_queue.c: Force COLP update if outgoing channel name changed.
+	  Merged revisions 428079 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * When a call is answered and the outgoing channel name has changed then
-	  force a connected line update because the channel is no longer the same.
-	  The channel was masqueraded into by another channel.  This is usually
-	  because of a call pickup.
 
-	  Note: Forwarded calls are handled in a controlled manner so the original
-	  channel name is replaced with the forwarded channel.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428080 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25423 #close
-	  Reported by: John Hardin
+2014-11-16 15:13 +0000 [656601d8c4]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ie275ea9e99c092ad369db23e0feb08c44498c172
+	* chan_pjsip: Remove AOR check when dialing and one is specified.
 
-2015-09-24 14:20 +0000 [6bf304bf25]  Richard Mudgett <rmudgett@digium.com>
+	  The AOR value may contain the name of an AOR or a full SIP URI.
+	  Checking if the AOR exists can't be done as a result of this.
+	  ........
 
-	* app_queue.c: Factor out a connected line update routine.
+	  Merged revisions 428051 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Replace inlined code with update_connected_line_from_peer().
+	  Merged revisions 428052 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25423
-	  Reported by: John Hardin
 
-	  Change-Id: I33bbd033596fcb0208d41d8970369b4e87b806f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428053 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-24 13:27 +0000 [e36b5f1e8e]  Richard Mudgett <rmudgett@digium.com>
+2014-11-16 06:12 +0000 [bc02cbabd9]  Joshua Colp <jcolp@digium.com>
 
-	* app_dial.c: Make 'A' option pass COLP updates.
+	* chan_sip: Fix bug where DTLS configuration from general would copy dtlsenable.
 
-	  While the 'A' option is playing the announcement file allow the caller and
-	  peer to exchange COLP update frames.
 
-	  ASTERISK-25423
-	  Reported by: John Hardin
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428034 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iac6cf89b56d26452c6bb88e9363622bbf23895f9
+2014-11-15 15:52 +0000 [6993743b1f]  Etienne Lessard (License 6394)
 
-2015-09-24 12:59 +0000 [747bfac895]  Richard Mudgett <rmudgett@digium.com>
+	* cel/cel_odbc: Provide microsecond precision in 'eventtime' column when possible
 
-	* app_dial.c: Force COLP update if outgoing channel name changed.
+	  This patch adds microsecond precision when inserting a CEL record into a table
+	  with an "eventtime" column of type timestamp, instead of second precision. The
+	  documentation (configs/cel_odbc.conf.sample) was already saying that the
+	  eventtime column included microseconds precision, but that was not the case.
 
-	  * When a call is answered and the outgoing channel name has changed then
-	  force a connected line update because the channel is no longer the same.
-	  The channel was masqueraded into by another channel.  This is usually
-	  because of a call pickup.
+	  Also, without this patch, if you had a table with an "eventtime" column of
+	  type varchar, you had millisecond precision. With this patch, you also get
+	  microsecond precision in this case.
 
-	  Note: Forwarded calls are handled in a controlled manner so the original
-	  channel name is replaced with the forwarded channel.
+	  Review: https://reviewboard.asterisk.org/r/3980
 
-	  ASTERISK-25423
-	  Reported by: John Hardin
+	  ASTERISK-24283 #close
+	  Reported by: Etienne Lessard
+	  patches:
+	    cel_odbc_time_precision.patch uploaded by Etienne Lessard (License 6394)
+	  ........
 
-	  Change-Id: I2e01f7a698fbbc8c26344a59c2be40c6cd98b00c
+	  Merged revisions 427952 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-09-24 12:37 +0000 [14481d9aa0]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427953 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* app_dial.c: Factor out a connected line update routine.
+	  Merged revisions 427954 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Replace inlined code with update_connected_line_from_peer().
 
-	  ASTERISK-25423
-	  Reported by: John Hardin
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428010 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ia14f18def417645cd7fb453e1bdac682630a5091
+2014-11-15 15:36 +0000 [ece61f5ed1]  Joshua Colp <jcolp@digium.com>
 
-2015-09-23 17:41 +0000 [bbeda190c3]  Richard Mudgett <rmudgett@digium.com>
+	* chan_pjsip: Add additional log message when an AOR is specified when dialing and it does not exist.
 
-	* app_dial.c: Remove some no-op code.
+	  ASTERISK-24499 #close
+	  Reported by: Rusty Newton
+	  ........
 
-	  Change-Id: Ice1884a94315d3cb7e3bbd47a9fba76a27276c54
+	  Merged revisions 428007 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-09-23 14:02 +0000 [f050fa76eb]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 428008 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* logger: Prevent duplicate dynamic channels from being added.
 
-	  There was a problem observed where the "logger add channel" CLI command
-	  would allow for a channel with the same name to be added multiple times.
-	  This would result in each message being written out to the same file
-	  multiple times.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@428009 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The problem was due to the difference in how logger channel filenames
-	  are stored versus the format they are allowed to be presented when they
-	  are added. For instance, if adding the logger channel "foo" through the
-	  CLI, the result would be a logger channel with the file name
-	  /var/log/asterisk/foo being stored. So when trying to add another "foo"
-	  channel, "foo" would not match "/var/log/asterisk/foo" so we'd happily
-	  add the duplicate channel.
+2014-11-15 13:01 +0000 [49e63a191d]  Joshua Colp <jcolp@digium.com>
 
-	  The fix presented here is to introduce two new methods in the logger
-	  code:
-	   * make_filename(): given a logger channel name, this creates the
-	     filename for that logger channel.
-	   * find_logchannel(): given a logger channel name, this calls
-	     make_filename() and then traverses the list of logchannels in order
-	     to find a match.
+	* chan_motif / chan_pjsip: Fix incorrect "No such module" messages when reloading.
 
-	  This change has made use of make_filename() and find_logchannel()
-	  throughout to more consistently behave.
+	  For chan_motif the direct return value of the underlying config options framework
+	  was passed back. This can relay various states which the module loader would not
+	  interpet as success. It has been changed so only on errors will it report back
+	  an error.
 
-	  ASTERISK-25305 #close
-	  Reported by Mark Michelson
+	  For chan_pjsip the code implemented a dummy reload function which always
+	  returned an error. This has been removed as all configuration is held within
+	  res_pjsip instead.
 
-	  Change-Id: I892d52954d6007d8bc453c3cbdd9235dec9c4a36
+	  ASTERISK-23651 #close
+	  Reported by: Rusty Newton
+	  ........
 
-2015-09-24 14:49 +0000 [629458d349]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 427981 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Do not swallow frames on channels leaving bridges.
+	  Merged revisions 427982 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When leaving a bridge, indications on a channel could be swallowed by
-	  the internal indication logic because it appears that the channel is on
-	  its way to be hung up anyway. One such situation where this is
-	  detrimental is when channels on hold are redirected out of a bridge. The
-	  AST_CONTROL_UNHOLD indication from the bridging code is swallowed,
-	  leaving the channel in question to still appear to be on hold.
 
-	  The fix here is to modify the logic inside ast_indicate_data() to not
-	  drop the indication if the channel is simply leaving a bridge. This way,
-	  channels on hold redirected out of a bridge revert to their expected "in
-	  use" state after the redirection.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427983 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25418 #close
-	  Reported by Mark Michelson
+2014-11-15 12:29 +0000 [9d2882d274]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: If6115204dfa0551c050974ee138fabd15f978949
+	* res_pjsip: Enforce requirements for session timer minimum expiration period and normal expiration period.
 
-2015-09-22 17:08 +0000 [5f15cd93f0]  Richard Mudgett <rmudgett@digium.com>
+	  This change enforces the requirements in PJSIP for session timer configuration. The minimum
+	  expiration period must be 90 seconds or higher and the normal expiration period can not
+	  be lower than the minimum expiration period. If either of these were done the code would
+	  assert at session setup time.
 
-	* app_page.c: Fix crash when forwarding with a predial handler.
+	  ASTERISK-24336 #close
+	  Reported by: Leon Rowland
+	  ........
 
-	  Page uses the async method of dialing with the dial API.  When a call gets
-	  forwarded there is no calling channel available.  If the predial handler
-	  was set then the calling channel could not be put into auto-service
-	  for the forwarded call because it doesn't exist.  A crash is the result.
+	  Merged revisions 427978 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Moved the callee predial parameter string processing to before the
-	  string is passed to the dial API rather than having the dial API do it.
-	  There are a few benefits do doing this.  The first is the predial
-	  parameter string processing doesn't need to be done for each channel
-	  called by the dial API.  The second is in async mode and the forwarded
-	  channel is to have the predial handler executed on it then the
-	  non-existent calling channel does not need to be present to process the
-	  predial parameter string.
+	  Merged revisions 427979 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Don't start auto-service on a non-existent calling channel to execute
-	  the predial handler when the dial API is in async mode and forwarding a
-	  call.
 
-	  ASTERISK-25384 #close
-	  Reported by: Chet Stevens
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427980 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: If53892b286d29f6cf955e2545b03dcffa2610981
+2014-11-15 10:31 +0000 [d0523b4b3c]  Michael K. (license 6621)
 
-2015-09-03 21:19 +0000 [b50e372394]  Matt Jordan <mjordan@digium.com>
+	* chan_sip: Add support for setting DTLS configuration in the general section.
 
-	* ARI: Add events for Contact and Peer Status changes
+	  Configuration of DTLS in the general section will be applied to any users
+	  or peers. If configuration exists at their level it overrides the general
+	  section values.
 
-	  This patch adds support for receiving events regarding Peer status changes
-	  and Contact status changes. This is particularly useful in scenarios where
-	  we are subscribed to all endpoints and channels, where we often want to know
-	  more about the state of channel technology specific items than a single
-	  endpoint's state.
+	  ASTERISK-24128 #close
+	  Reported by: Michael K.
+	  patches:
+	    dtls_default_settings.patch submitted by Michael K. (license 6621)
 
-	  ASTERISK-24870
+	  Review: https://reviewboard.asterisk.org/r/3867/
 
-	  Change-Id: I6137459cdc25ce27efc134ad58abf065653da4e9
 
-2015-09-04 12:24 +0000 [3502c0431d]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427950 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/res_stasis_device_state: Allow for subscribing to 'all' device state
+2014-11-14 15:51 +0000 [3268544907]  Matt Jordan <mjordan@digium.com>
 
-	  This patch adds support for subscribing to all device state changes. This is
-	  done either by subscribing to an empty device, e.g., 'eventSource=deviceState:',
-	  or by the WebSocket connection specifying that it wants all state in the
-	  system.
+	* tests/test_cel: Unlock bridge on off nominal paths
 
-	  ASTERISK-24870
+	  If the test fails due to memory allocation errors, we may as well attempt to
+	  unlock the bridge on the way out.
+	  ........
 
-	  Change-Id: I9cfeca1c9e2231bd7ea73e45919111d44d2eda32
+	  Merged revisions 427927 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-04 12:25 +0000 [4c9f613309]  Matt Jordan <mjordan@digium.com>
 
-	* ARI: Add the ability to subscribe to all events
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427932 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch adds the ability to subscribe to all events. There are two possible
-	  ways to accomplish this:
-	  (1) On initial WebSocket connection. This patch adds a new query parameter,
-	      'subscribeAll'. If present and True, Asterisk will subscribe the
-	      applications to all ARI events.
-	  (2) Via the applications resource. When subscribing in this manner, an ARI
-	      client should merely specify a blank resource name, i.e., 'channels:'
-	      instead of 'channels:12354'. This will subscribe the application to all
-	      resources of the 'channels' type.
+2014-11-14 12:12 +0000 [df2090b931]  Jonathan Rose <jrose@digium.com>
+
+	* Documentation: Revise explanation of cdr.conf option 'Unanswered'
+
+	  ASTERISK-24279 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4109/
+	  ........
+
+	  Merged revisions 427901 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
+
+	  Merged revisions 427902 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427903 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-11-14 09:52 +0000 [ba811ae1c3]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  ASTERISK-24870 #close
+	* stun: correct attribute string padding to match rfc
 
-	  Change-Id: I4a943b4db24442cf28bc64b24bfd541249790ad6
+	  When sending the USERNAME attribute in an RTP STUN
+	  response, the implementation in append_attr_string
+	  passed the actual length, instead of padding it up
+	  to a multiple of four bytes as required by the RFC
+	  3489.  This change adds separate variables for the
+	  string and padded attributed lengths, and performs
+	  padding correctly.
 
-2015-09-21 08:16 +0000 [ec514ad64d]  Elazar Broad <elazar@thebroadfamily.com>
+	  Reported by: Thomas Arimont
+	  Review: https://reviewboard.asterisk.org/r/4139/
+	  ........
 
-	* core/logging: Fix logging to more than one syslog channel
+	  Merged revisions 427874 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Currently, Asterisk will log to the last configured syslog
-	  channel in logger.conf. This is due to the fact that the
-	  final call to openlog() supersedes all of the previous calls.
-	  This commit removes the call to openlog() and passes the
-	  facility to ast_log_vsyslog(), along with utilizing the
-	  LOG_MAKEPRI macro to ensure that the message is routed to
-	  the correct facility and with the correct priority.
+	  Merged revisions 427875 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25407 #close
-	  Reported by: Elazar Broad
-	  Tested by: Elazar Broad
+	  Merged revisions 427876 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ie2a2416bc00cce1b04e99ef40917c2011953ddd2
 
-2015-09-21 18:06 +0000 [aeddee39fb]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427877 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* app_record: RECORDED_FILE variable not being populated
+2014-11-14 09:28 +0000 [2d9471ab1f]  Mark Michelson <mmichelson@digium.com>
 
-	  The RECORDED_FILE variable is empty unless a '%d' is specified in the filename.
-	  This patch makes it so the variable is always set to the filename.
+	* Fix race condition that could result in ARI transfer messages not being sent.
 
-	  ASTERISK-25410 #close
+	  From reviewboard:
+
+	  "During blind transfer testing, it was noticed that tests were failing
+	  occasionally because the ARI blind transfer event was not being sent.
+	  After investigating, I detected a race condition in the blind transfer
+	  code. When blind transferring a single channel, the actual transfer
+	  operation (i.e. removing the transferee from the bridge and directing
+	  them to the proper dialplan location) is queued onto the transferee
+	  bridge channel. After queuing the transfer operation, the blind transfer
+	  Stasis message is published. At the time of publication, snapshots of
+	  the channels and bridge involved are created. The ARI subscriber to the
+	  blind transfer Stasis message then attempts to determine if the bridge
+	  or any of the involved channels are subscribed to by ARI applications.
+	  If so, then the blind transfer message is sent to the applications. The
+	  way that the ARI blind transfer message handler works is to first see
+	  if the transferer channel is subscribed to. If not, then iterate over
+	  all the channel IDs in the bridge snapshot and determine if any of
+	  those are subscribed to. In the test we were running, the lone
+	  transferee channel was subscribed to, so an ARI event should have been
+	  sent to our application. Occasionally, though, the bridge snapshot did
+	  not have any channels IDs on it at all. Why?
+
+	  The problem is that since the blind transfer operation is handled by a
+	  separate thread, it is possible that the transfer will have completed and
+	  the channels removed from the bridge before we publish the blind transfer
+	  Stasis message. Since the blind transfer has completed, the bridge on
+	  which the transfer occurred no longer has any channels on it, so the
+	  resulting bridge snapshot has no channels on it. Through investigation of
+	  the code, I found that attended transfers can have this issue too for the
+	  case where a transferee is transferred to an application."
+
+	  The fix employed here is to decouple the creation of snapshots for the transfer
+	  messages from the publication of the transfer messages. This way, snapshots
+	  can be created to reflect what they are at the time of the transfer operation.
 
-	  Change-Id: I4ec826d8eb582ae2ad184e717be8668b74d37653
+	  Review: https://reviewboard.asterisk.org/r/4135
+	  ........
 
-2015-09-16 08:22 +0000 [2bd27d1222]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 427848 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* pbx: Update device and presence state when changing a hint extension.
+	  Merged revisions 427870 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When changing a hint extension without removing the hint first the
-	  device state and presence state is not updated. This causes the state
-	  of the hint to be that of the previous extension and not the current
-	  one. This state is kept until a state change occurs as a result of
-	  something (presence state change, device state change).
 
-	  This change updates the hint with the current device and presence
-	  state of the new extension when it is changed. Any state callbacks
-	  which may have been added before the hint extension is changed are
-	  also informed of the new device and presence state if either have
-	  changed.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427873 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25394 #close
+2014-11-14 08:56 +0000 [737b811749]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: If268f1110290e502c73dd289c9e7e7b27bc8432f
+	* app_confbridge: Play "leader has left" sound even when musiconhold is enabled.
 
-2015-09-17 16:34 +0000 [c94f46080f]  Scott Griepentrog <scott@griepentrog.com>
+	  Currently if the leader of a conference bridge leaves any participant
+	  that has musiconhold enabled will not hear the "leader has left" sound.
+	  This is because musiconhold is started and THEN the sound is played.
 
-	* CHAOS: avoid crash if string create fails
+	  This change makes it so that the sound is played and THEN musiconhold
+	  is started. This provides a better experience for users as they may not
+	  have known previously why they went back to musiconhold.
 
-	  Validate string buffer allocation before using them.
+	  Review: https://reviewboard.asterisk.org/r/4177/
+	  ........
 
-	  ASTERISK-25323
+	  Merged revisions 427844 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ib9c338bdc1e53fb8b81366f0b39482b83ef56ce0
+	  Merged revisions 427845 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-09-17 04:52 +0000 [b59c4d82b5]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Merged revisions 427846 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip: Fix From header truncation for extremely long CALLERID(name).
 
-	  The CALLERID(num) and CALLERID(name) and other info are placed into the
-	  `char from[256]` in initreqprep. If the name was too long, the addr-spec
-	  and params wouldn't fit.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427847 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Code is moved around so the addr-spec with params is placed there first,
-	  and then fitting in as much of the display-name as possible.
+2014-11-14 08:40 +0000 [2454505d5a]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-25396 #close
+	* Fix race condition where duplicated requests may be handled by multiple threads.
 
-	  Change-Id: I33632baf024f01b6a00f8c7f35c91e5f68c40260
+	  This is the Asterisk 13 version of the patch. The main difference is in the pubsub
+	  code since it was completely refactored between Asterisk 12 and 13.
 
-2015-09-17 16:59 +0000 [4cc59533b9]  Richard Mudgett <rmudgett@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4175
+	  ........
 
-	* CHAOS: res_pjsip_diversion avoid crash if allocation fails
+	  Merged revisions 427841 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Validate ast_malloc buffer returned before using it in
-	  set_redirecting_value().
 
-	  ASTERISK-25323
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427842 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I15d2ed7cb0546818264c0bf251aa40adeae83253
+2014-11-13 16:26 +0000 [49b7a1cbaf]  Kevin Harwell <kharwell@digium.com>
 
-2015-09-17 16:47 +0000 [4fb95bbc4e]  Kevin Harwell <kharwell@digium.com>
+	* res_pjsip_exten_state: PJSIPShowSubscriptionsInbound causes crash
 
-	* app_queue: AgentComplete event has wrong reason
+	  When using a non-default sorcery wizard (in this instance realtime) for
+	  outbound registrations and after adding in an appropriate call to
+	  ast_sorcery_apply_config() (since it is missing) Asterisk will crash after
+	  a stack overflow occurs due to the code infinitely recursing.  The fix entails
+	  removing the outbound registration state dependency from the outbound
+	  registration sorcery object and instead keeping an in memory container that
+	  can be used to lookup the state when needed.
 
-	  When a queued caller transfers an agent to another extension sometimes the
-	  raised AgentComplete event has a reason of "caller" and sometimes "transfer".
-	  Since a transfer has taken place this should always be transfer. This occurs
-	  because sometimes the stasis hangup event arrives before the transfer event
-	  thus writing a different reason out.
+	  ASTERISK-24514
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4164/
+	  ........
 
-	  With this patch, when a hangup event is received during a transfer it will
-	  check to see if the channel that is hanging up is part of a transfer. If so
-	  it will return and let the subsequently received transfer event handler take
-	  care of the cleanup.
+	  Merged revisions 427814 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25399 #close
+	  Merged revisions 427815 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ic63c49bd9a5ed463ea7a032fd2ea3d63bc81a50d
 
-2015-09-17 13:09 +0000 [fb6b5c684b]  Scott Griepentrog <scott@griepentrog.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427823 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* PJSIP: avoid crash when getting rtp peer
+2014-11-13 09:46 +0000 [74e706878b]  Kinsey Moore <kmoore@digium.com>
 
-	  Although unlikely, if the tech private is returned as
-	  a NULL, chan_pjsip_get_rtp_peer() would crash.
+	* Stasis: Fix StasisEnd message ordering
 
-	  ASTERISK-25323
+	  This change corrects message ordering in cases where a channel-related
+	  message can be received after a Stasis/ARI application has received the
+	  StasisEnd message. The StasisEnd message was being passed to
+	  applications directly without waiting for the channel topic to empty.
 
-	  Change-Id: Ie231369bfa7da926fb2b9fdaac228261a3152e6a
+	  As a result of this fix, other bugs were also identified and fixed:
+	  * StasisStart messages were also being sent directly to apps and are
+	    now routed through the stasis message bus properly
+	  * Masquerade monitor datastores were being removed at the incorrect
+	    time in some cases and were causing StasisEnd messages to not be sent
+	  * General refactoring where necessary for the above
+	  * Unsubscription on StasisEnd timing changes to prevent additional
+	    messages from following the StasisEnd when they shouldn't
 
-2015-09-17 11:31 +0000 [6409e7b11a]  Kevin Harwell <kharwell@digium.com>
+	  A channel sanitization function pointer was added to reduce processing
+	  and AO2 lookups.
 
-	* app_queue: Crash when transferring
+	  Review: https://reviewboard.asterisk.org/r/4163/
+	  ASTERISK-24501 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  During some transfer scenarios involving queues Asterisk would sometimes
-	  crash when trying to obtain a channel snapshot (could happen on caller or
-	  member channels). This occurred because the underlying channel had already
-	  disappeared when trying to obtain the latest snapshot.
+	  Merged revisions 427788 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch adds a reference to both the member and caller channels that
-	  extends to the lifetime of the queue'd call, thus making sure the channels
-	  will always exist when retrieving the latest snapshots.
+	  Merged revisions 427789 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25185 #close
-	  Reported by: Etienne Lessard
 
-	  Change-Id: Ic397fa68fb4ff35fbc378e745da9246a7b552128
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427790 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-16 17:36 +0000 [fe5077b1f8]  Mark Michelson <mmichelson@digium.com>
+2014-11-12 18:23 +0000 [cc4c396647]  Matt Jordan <mjordan@digium.com>
 
-	* res_pjsip_pubsub: Eliminate race during initial NOTIFY.
+	* main/rtp_engine: Fix crash when processing more than one RTCP report info block
 
-	  There is a slim chance of a race condition occurring where two threads
-	  can both attempt to manipulate the same area.
+	  Asterisk - in res_rtp_asterisk - only understands a single RTCP report info
+	  block. When the RTCP information was refactored in the RTP Engine to be pushed
+	  over the Stasis message bus, I put in the hooks into the engine to handle
+	  multiple RTCP report info blocks, in the hope that a future RTP implementation
+	  would be able to provide that data. Unfortunately, res_rtp_asterisk has a
+	  tendency to "lie":
+	  (1) It will send RTCP reports with a reception_report_count greater than 1
+	      (which is pulled directly from the RTCP packet itself, so that part is
+	      correct)
+	  (2) It will only provide a single report block
+
+	  When the rtp_engine goes to convert this to a JSON blob, hilarity ensues as it
+	  looks for a report block that doesn't exist.
+
+	  This patch updates the rtp_engine to be a bit more skeptical about what it is
+	  presented with. While this could also be fixed in res_rtp_asterisk, this patch
+	  prefers to fix it in the engine for two reasons:
+	  (1) The engine is designed to work with multiple RTP implementation, and hence
+	      having it be more robust is a good thing (tm)
+	  (2) res_rtp_asterisk's handling of RTCP information is "fun". It should report
+	      the correct reception_report_count; ideally it should also be giving us all
+	      of the blocks - but it is *definitely* not designed to do that. Going down
+	      that road is a non-trivial effort.
+
+	  Review: https://reviewboard.asterisk.org/r/4158/
+
+	  ASTERISK-24489 #close
+	  Reported by: Gregory Malsack
+	  Tested by: Gregory Malsack
+
+	  ASTERISK-24498 #close
+	  Reported by: Beppo Mazzucato
+	  Tested by: Beppo Maazucato
+	  ........
 
-	  Thread A can be handling an incoming initial SUBSCRIBE request. Thread A
-	  lets the specific subscription handler know that the subscription has
-	  been established.
+	  Merged revisions 427762 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  At this point, Thread B may detect a state change on the subscribed
-	  resource and queue up a notification task on Thread C, the subscription
-	  serializer thread.
+	  Merged revisions 427763 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Now Thread A attempts to generate the initial NOTIFY request to send to
-	  the subscriber at the same time that Thread C attempts to generate a
-	  state change NOTIFY request to send to the subscriber.
 
-	  The result is that Threads A and C can step on the same memory area,
-	  resulting in a crash. The crash has been observed as happening when
-	  attempting to allocate more space to hold the body for the NOTIFY.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427771 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The solution presented here is to queue the subscription establishment
-	  and initial NOTIFY generation onto the subscription serializer thread
-	  (Thread C in the above scenario). This way, there is no way that a state
-	  change notification can occur before the initial NOTIFY is sent, and if
-	  there is a quick succession of NOTIFYs, we can guarantee that the two
-	  NOTIFY requests will be sent in succession.
+2014-11-12 14:40 +0000 [ec1a7654f3]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I5a89a77b5f2717928c54d6efb9955e5f6f5cf815
+	* Fix leak in AMI Action Bridge
 
-2015-08-28 15:42 +0000 [b88c54fa4b]  Alexander Traud <pabstraud@compuserve.com>
+	  Add missing reference cleanup for newly created bridge.
 
-	* translate: Fix transcoding while different in frame size.
+	  ASTERISK-24281
+	  Reported by: Stefan Engström
+	  Review: https://reviewboard.asterisk.org/r/4154/
+	  ........
 
-	  When Asterisk translates between codecs, each with a different frame size (for
-	  example between iLBC 30 and Speex-WB), too large frames were created by
-	  ast_trans_frameout. Now, ast_trans_frameout is called with the correct frame
-	  length, creating several frames when necessary. Affects all transcoding modules
-	  which used ast_trans_frameout: GSM, iLBC, LPC10, and Speex.
+	  Merged revisions 427736 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25353 #close
+	  Merged revisions 427737 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2e229569d73191d66a4e43fef35432db24000212
 
-2015-09-10 17:19 +0000 [5c713fdf18]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427738 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* scheduler: Use queue for allocating sched IDs.
+2014-11-12 10:13 +0000 [dbb8f0a935]  Joshua Colp <jcolp@digium.com>
 
-	  It has been observed that on long-running busy systems, a scheduler
-	  context can eventually hit INT_MAX for its assigned IDs and end up
-	  overflowing into a very low negative number. When this occurs, this can
-	  result in odd behaviors, because a negative return is interpreted by
-	  callers as being a failure. However, the item actually was successfully
-	  scheduled. The result may be that a freed item remains in the scheduler,
-	  resulting in a crash at some point in the future.
+	* pbx: Fix off-nominal case where a freed extension may still be used.
 
-	  The scheduler can overflow because every time that an item is added to
-	  the scheduler, a counter is bumped and that counter's current value is
-	  assigned as the new item's ID.
+	  If during the operation of adding an extension a priority is added but
+	  fails it is possible for the extension to be freed but still exist in
+	  the PBX core. If this occurs subsequent lookups may try to access the
+	  extension and end up in freed memory.
 
-	  This patch introduces a new method for assigning scheduler IDs. Instead
-	  of assigning from a counter, a queue of available IDs is maintained.
-	  When assigning a new ID, an ID is pulled from the queue. When a
-	  scheduler item is released, its ID is pushed back onto the queue. This
-	  way, IDs may be reused when they become available, and the growth of ID
-	  numbers is directly related to concurrent activity within a scheduler
-	  context rather than the uptime of the system.
+	  This change removes the extension from the PBX core when the priority
+	  addition fails and then frees the extension.
 
-	  Change-Id: I532708eef8f669d823457d7fefdad9a6078b99b2
+	  ASTERISK-24444 #close
+	  Reported by: Leandro Dardini
 
-2015-08-21 21:50 +0000 [865377fc38]  Rodrigo Ramírez Norambuena <a@rodrigoramirez.com>
+	  Review: https://reviewboard.asterisk.org/r/4162/
+	  ........
 
-	* chan_sip.c: Validation on module reload
+	  Merged revisions 427709 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change validation on reload module because now used the cli function for
-	  reload. The sip_reload() function never fail and ever return NULL for this
-	  reason on reload() now use the call the sip_reload() and return
-	  AST_MODULE_LOAD_SUCCESS.
+	  Merged revisions 427710 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This problem is dectected on reload by PUT method on ARI, getting always
-	  404 http code when the module is reloaded.
+	  Merged revisions 427711 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25325 #close
-	  Reporte by: Rodrigo Ramírez Norambuena
 
-	  Change-Id: I41215877fb2cfc589e0d4d464000cf6825f4d7fb
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427712 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-21 17:39 +0000 [e75aff53e6]  Richard Mudgett <rmudgett@digium.com>
+2014-11-12 07:47 +0000 [9f89b83269]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_pubsub.c: Mark ast_sip_create_subscription() as not used.
+	* Fix compiler error when using ./configure --enable-dev-mode --enable-coverage
 
-	  Change-Id: I2b8db18eac36c01a5c7eb9467699124e203fd093
+	  When DONT_OPTIMIZE is enabled with dev-mode, it causes a shadow compilation
+	  to be done with output to /dev/null.  This can cause errors with coverage
+	  when GCC attempts to write to /dev/null.gcno.  This change disables
+	  coverage for the shadow compilation.
 
-2015-09-09 12:24 +0000 [4d91d01df1]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24502 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4151/
+	  ........
 
-	* res_pjsip_pubsub.c: Add some notification comments.
+	  Merged revisions 427682 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Ie62ff1f4b7adc1a12fa0303f53926af249b25e20
+	  Merged revisions 427683 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-08-21 18:01 +0000 [f36a9d1221]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427684 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_pubsub.c: Set dlg_status code instead of sending SIP response.
 
-	  We should not try to send a SIP response message because we may be
-	  restoring a persistent subscription where we are not responding to a SIP
-	  request.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427685 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id89167ef90320c5563f37e632db0dda6cb9e7dec
+2014-11-09 02:01 +0000 [21c41e4542]  Corey Farrell <git@cfware.com>
 
-2015-08-21 17:40 +0000 [94582f8fab]  Richard Mudgett <rmudgett@digium.com>
+	* manager: Fix HTTP connection reference leaks.
 
-	* res_pjsip_pubsub.c: Fix off-nominal memory leak.
+	  Fix reference leak that happens if (session && !blastaway).
 
-	  Fix off-nominal visited vector leak in build_resource_tree().
+	  ASTERISK-24505 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4153/
+	  ........
 
-	  Change-Id: If0399c7941c9c0b1038bcfb7b9a371760977831c
+	  Merged revisions 427641 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-08-21 15:26 +0000 [8b3ed52239]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427642 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res_pjsip_pubsub.c: Fix one byte buffer overrun error.
+	  Merged revisions 427643 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ast_sip_pubsub_register_body_generator() did not account for the null
-	  terminator set by sprintf() in the allocated output buffer.
 
-	  Change-Id: I388688a132e479bca6ad1c19275eae0070969ae2
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427644 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-21 15:25 +0000 [4329bd1e4c]  Richard Mudgett <rmudgett@digium.com>
+2014-11-08 18:38 +0000 [f4392c4b6d]  Xavier Hienne (License 6657)
 
-	* res_pjsip_pubsub.c: Use ast_alloca() instead of alloca().
+	* channels/chan_mgcp: Fix regression which causes gateways to be skipped
 
-	  Change-Id: Ia396096b4fedc2874649ca11137612c3f55e83e3
+	  In r227276, a while loop was turned into a for loop. Unfortunately, a portion
+	  of the while loop was left in the code such that, when a static gateway is
+	  encountered in the list of MGCP gateways, the next gateway would be skipped.
+	  At best, we would simply flip past a gateway; at worst, this could lead to a
+	  crash.
 
-2015-08-21 11:04 +0000 [a456a20ecf]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24500 #close
+	  Reported by: Xavier Hienne
+	  patches:
+	    chan_mgcp.patch uploaded by Xavier Hienne (License 6657)
+	  ........
 
-	* res_pjsip_pubsub.c: Add missing error return in load_module().
+	  Merged revisions 427613 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I15debd0f717f16ee2f78e7f56151c3b3b97b72fc
+	  Merged revisions 427614 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-08-21 11:03 +0000 [f58f4c6e27]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427615 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip/location.c: Use the builtin ao2_callback() match function instead.
 
-	  Change-Id: I364906d6d2bad3472929986704a0286b9a2cbe3f
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427616 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-10 09:49 +0000 [9d1f176e29]  Mark Michelson <mmichelson@digium.com>
+2014-11-08 18:26 +0000 [d773f9d03e]  Dmitriy Bubnov (License 6651),Dmitry Bubnov (License 6651)
 
-	* res_pjsip: Copy default_from_user to avoid crash.
+	* addons/chan_mobile: Increase buffer size of UCS2 encoded SMS messages
 
-	  The default_from_user retrieval function was pulling the
-	  default_from_user from the global configuration struct in an unsafe way.
-	  If using a database as a backend configuration store, the global
-	  configuration struct is short-lived, so grabbing a pointer from it
-	  results in referencing freed memory.
+	  When UCS2 character encoding is used, one symbol in national language can be
+	  expanded to 4 bytes. The current buffer used for receiving message in
+	  do_monitor_phone is 256 bytes, which is not large enough for incoming messages.
 
-	  The fix here is to copy the default_from_user value out of the global
-	  configuration struct.
+	  For example:
+	  * AT+CMGR phone response prefix
+	    '+CMGR: "REC UNREAD","+7**********",,"14/10/29,13:31:39+12"\r\n' - 60 bytes
+	  * SMS body with UCS2 encoding (max) - 280 bytes
+	  * AT+CMGR phone response suffix '\r\n\r\nOK\r\n' - 8 bytes
+	  * Terminating null character - 1 byte
 
-	  Thanks go to John Hardin for discovering this problem and proposing the
-	  patch on which this fix is based.
+	  This results in a needed buffer size of 349 bytes. Hence, this patch opts for a
+	  350 byte buffer.
 
-	  ASTERISK-25390 #close
-	  Reported by Mark Michelson
+	  ASTERISK-24468 #close
+	  Reported by: Dmitriy Bubnov
+	  patches:
+	    chan_mobile-1_8.diff uploaded by Dmitriy Bubnov (License 6651)
+	    chan_mobile-trunk.diff uploaded by Dmitry Bubnov (License 6651)
+	  ........
 
-	  Change-Id: I6b96067a495c1259da768f4012d44e03e7c6148c
+	  Merged revisions 427607 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-09-10 08:39 +0000 [1dd0e220bf]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 427610 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res/res_pjsip_nat: Ignore REGISTER requests when looking for a Record-Route
+	  Merged revisions 427611 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  We will only rewrite the Contact header if there is no Record-Route header in
-	  the received request. If a malfunctioning proxy places a Record-Route header
-	  into a REGISTER request, we will decide that we shouldn't update the IP/port
-	  in the Contact header, and we will end up storing a contact with an AoR that
-	  contains the NAT'd IP address.
 
-	  While it is nice to have the proxy *not* send a Record-Route in a REGISTER
-	  request, it's also a good idea to not process the header in a non-dialog
-	  message. This patch updates the code to explicitly ignore the Record-Route
-	  header in REGISTER requests.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427612 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25387 #close
+2014-11-08 18:14 +0000 [08d773532b]  abelbeck <lonnie@abelbeck.com> (License 5903)
 
-	  Change-Id: I4bd3bcccc4003d460cc354d986b0dea2e433ef3f
+	* app_voicemail: Fix enhancement that allowed multiple recipients in To: header
 
-2015-09-03 21:15 +0000 [4eedd9ef9d]  Matt Jordan <mjordan@digium.com>
+	  An issue existed in r420577, which added multiple recipients to voicemail
+	  emails. The patch, when looking at the intended recipients, looked ahead for
+	  the '|' character inside a while loop which already had pulled out the
+	  appropriate field parsing on the '|' character. This would cause it to skip
+	  the recipients.
 
-	* main/config_options: Check for existance of internal object before derefing
+	  This patch fixes it such that it relies completely on the while loop to parse
+	  through the e-mail fields.
 
-	  Asterisk can load and register an object type while still having an invalid
-	  sorcery mapping. This can cause an issue when a creation call is invoked.
-	  For example, mis-configuring PJSIP's endpoint identifier by IP address mapping
-	  in sorcery.conf will cause the sorcery mechanism to be invalidated; however, a
-	  subsequent ARI invocation to create the object will cause a crash, as the
-	  internal type may not be registered as sorcery expects.
+	  Note that the original author of the patch looked at this fix and approved it.
 
-	  Merely checking for a NULL pointer here solves the issue.
+	  ASTERISK-24250 #close
+	  Reported by: abelbeck
+	  patches:
+	    voicemail-420577-to-comma-fix.diff uploaded by abelbeck (License 5903)
+	  ........
 
-	  Change-Id: I54079fb94a1440992f4735a9a1bbf1abb1c601ac
-2015-09-09 16:46 +0000 [71408df2b8]  Alexander Anikin <may213@yandex.ru>
+	  Merged revisions 427585 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_ooh323: Add ProgressIndicator IE with inband info available
 
-	  Add ProgressIndicator IE with inband info present to Progress and
-	  Alerting Q.931 message
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427586 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25227 #close
-	  Reported by: Alexandr Dranchuk
+2014-11-08 18:04 +0000 [9a1ab5d548]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I326ad13cb1db9a72b3fd902bafed3c28a3684203
-2015-09-08 10:35 +0000 [f72f9ceefc]  Scott Griepentrog <scott@griepentrog.com>
+	* bridge_native_rtp: Fix T.38 issues with remote bridges
 
-	* pjsip: avoid possible crash req_caps allocation failure
+	  After r425242 the fax/sip/directmedia_reinvite_t38 test started failing due to
+	  the surviving channel not being re-INVITEd back from T.38 to audio. This patch
+	  fixes that bug - a deeper explanation of what happened follows.
+
+	  When two RTP channels are in a native bridge, the bridging layer will
+	  investigate each via the get_rtp_info glue callback. This callback returns the
+	  native bridge preference of the channel *at that moment in time* (that part is
+	  key). At different points during the bridging, the native bridging layer will
+	  inform the RTP capable channels of the status of the bridge via the update_peer
+	  glue callback.
+
+	  In a T.38 scenario with audio direct media, the sequence of events will often
+	  look like the following:
+	   * SIP/A and SIP/B both have audio and enter a native bridge.
+	   * Asterisk re-INVITEs audio between SIP/A and SIP/B directly (via an
+	     update_peer callback).
+	   * SIP/A sends a re-INVITE to T.38, which causes Asterisk to send a re-INVITE
+	     to T.38 to SIP/B. Assuming everyone 200 OKs the process, the UDPTL stack
+	     receives UDPTL packets in Asterisk from both endpoints. From the perspective
+	     of the channels, we are now in a local bridge for T.38, even though we are
+	     technically still in a remote bridge in bridge_native_rtp. (YAY!)
+	   * When one side hangs up, bridge_native_rtp is told to stop bridging. It then
+	     re-evaluates the channels and asks them how they are bridged - and since
+	     T.38 is enabled, they reply with a Local bridge (which is correct), but is
+	     wrong because the audio portion is still technically in a remote bridge.
+	   * Asterisk releases the surviving channel, whose audio is *not* re-INVITED
+	     back to Asterisk as bridge_native_rtp incorrectly assumes that it was in a
+	     local bridge.
+
+	  Ironically, prior to r425242, this used to work mostly due to a fluke in the
+	  bridging layer.
+
+	  The purpose of the get_rtp_info callback shouldn't be modified: it should tell
+	  the bridging layer what kind of bridge the channel prefers at that moment in
+	  time. If you have T.38 enabled, that *must* be a local bridge, as the UDPTPL
+	  stack must be in the media path. As such, this patch does not modify that
+	  part of the code.
+
+	  However, we have to tell the channels to re-evaluate themselves when they come
+	  out of a native bridge, since we can no longer trust the get_rtp_info callbacks
+	  when the native bridge is being stopped. Something else may have changed in the
+	  channels, and they may now be lying to us. As such, this patch makes it so that
+	  we unilaterally tell the channels that they are no longer bridged via the
+	  update_peer callback. This is actually what the channels expect anyway: code in
+	  both chan_sip and chan_pjsip's callbacks look at the T.38 state and - if they
+	  were in T.38 - send a re-INVITE to get the audio back to Asterisk.
 
-	  Make certain that the pjsip session has not failed to
-	  allocate the format capabilities structure, which can
-	  otherwise cause a crash when referenced.
+	  Review: https://reviewboard.asterisk.org/r/4157/
+	  ........
 
-	  ASTERISK-25323
+	  Merged revisions 427582 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I602790ba12714741165e441cc64a3ecde4cb5750
+	  Merged revisions 427583 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-03 14:07 +0000 [fbf720db91]  Jonathan Rose <jrose@digium.com>
 
-	* ParkAndAnnounce: Add variable inheritance
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427584 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In Asterisk 11, the announcer channel would receive channel variables
-	  from the channel being parked by means of normal channel inheritance.
-	  This functionality was lost during the big res_parking project in
-	  Asterisk 12. This patch restores that functionality.
+2014-11-08 12:20 +0000 [d4fd0774f4]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25369 #close
-	  Review: https://gerrit.asterisk.org/#/c/1180/
+	* chan_console: Fix reference leaks to pvt.
 
-	  Change-Id: Ie47e618330114ad2ea91e2edcef1cb6f341eed6e
+	  Fix a bunch of calls to get_active_pvt
+	  where the reference is never released.
 
-2015-09-04 16:33 +0000 [695f26cbb7]  David M. Lee <dlee@respoke.io>
+	  ASTERISK-24504 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4152/
+	  ........
 
-	* res_rtp_asterisk: Add more ICE debugging
+	  Merged revisions 427554 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  In working through a recent ICE negotiation bug, I found the debug
-	  logging in res_rtp_asterisk to be lacking. This patch adds a number of
-	  debug and warning statements that were helpful.
+	  Merged revisions 427555 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I950c6d8f13a41f14b3d6334b4cafe7d4e997be80
-2015-09-01 10:16 +0000 [4ed9c9a280]  Guido Falsi <madpilot@freebsd.org>
+	  Merged revisions 427557 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Core/General: Add #ifdef needed on FreeBSD.
 
-	  pthread_attr_init() defaults to PTHREAD_EXPLICIT_SCHED on FreeBSD
-	  too.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427566 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25310 #close
-	  Reported by: Guido Falsi
+2014-11-06 13:26 +0000 [7571bae5ab]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Iae6befac9028b5b9795f86986a4a08a1ae6ab7c4
+	* app_agent_pool: Made agent alert interruptable by DTMF.
 
-2015-09-08 07:21 +0000 [5469caa9dd]  Joshua Colp <jcolp@digium.com>
+	  Made agent able to interrupt the alerting beep playback with DTMF.  Any
+	  digit can interrupt if the call does not need to be acknowledged.  Only
+	  the first digit of the acknowledgement can interrupt if the call needs to
+	  be acknowledged.  The agent interrupting the alerting playback builds on
+	  the ASTERISK-24447 patch because it knows what digit interrupted the
+	  playback and needs to be able to pass that digit to the DTMF hook digit
+	  collection code.
 
-	* res_pjsip: Use hash for contact object identity instead of Contact URI.
+	  ASTERISK-24257 #close
+	  Reported by: Steve Pitts
 
-	  In the wild it is possible for Contact URIs to be quite long as
-	  parameters can exist on them. This can present a problem when storing
-	  them in the AstDB as the URI is used as part of the object name and
-	  there is a fixed length limit for the AstDB. This will cause
-	  the contact to not get stored.
+	  Review: https://reviewboard.asterisk.org/r/4123/
+	  ........
 
-	  This change uses the MD5 hash of the Contact URI as part of the
-	  object name instead. This has a fixed length which is guaranteed
-	  to not exceed the AstDB length limit.
+	  Merged revisions 427508 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25295 #close
+	  Merged revisions 427512 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ie8252a75331ca00b41b9f308f42cc1fbdf701a02
 
-2015-09-07 13:19 +0000 [480c443e26]  Alexander Anikin <may213@yandex.ru>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427519 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_ooh323: call ast_rtp_instance_stop on ooh323_destroy
+2014-11-06 13:12 +0000 [a68baad74f]  Richard Mudgett <rmudgett@digium.com>
 
-	      Call ast_rtp_instance_stop on ooh323_destroy to free resources
-	      allocated by rtp instance
+	* Bridge DTMF hooks: Made audio pass from the bridge while waiting for more matching digits.
 
-	      ASTERISK-25299 #close
-	      Report by: Alexandr Dranchuk
+	  * Made collecting DTMF digits for the DTMF feature hooks pass frames from
+	  the bridge.
 
-	  Change-Id: I455096bd7da016b871afe90af86067c2c7c9f33f
+	  * Made collecting DTMF digits possible by other bridge hooks if there is a
+	  need.
 
-2015-09-07 11:15 +0000 [c3e6debdb9]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24447 #close
+	  Reported by: Richard Mudgett
 
-	* res/res_pjsip: Purge contacts when an AoR is deleted
+	  Review: https://reviewboard.asterisk.org/r/4123/
+	  ........
 
-	  When an AoR is deleted by an external mechanism, such as through ARI, we
-	  currently do not remove dynamic contacts that were created for that AoR as a
-	  result of a received REGISTER request. As a result, re-creating the AoR will
-	  cause the dynamic contact to be interpreted as a persistent contact, leading
-	  to some rather strange state being created for the contacts/endpoints.
+	  Merged revisions 427493 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch adds a sorcery observer for the 'aor' object. When a delete is
-	  issued on the underlying sorcery object, the observer is called, and all
-	  contacts created and persisted in sorcery for that AoR are also removed. Note
-	  that we don't want to perform this action when an AO2 object that is an AoR is
-	  destroyed, as the AoR can still exist in the backing storage (and we would
-	  thus be removing valid contacts from an AoR that still "exists".)
+	  Merged revisions 427494 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25381 #close
 
-	  Change-Id: I6697e51ef6b2858b5d63401f35dc378bb0f90328
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427495 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-09-05 14:58 +0000 [78d0b9d97e]  Matt Jordan <mjordan@digium.com>
+2014-11-06 12:21 +0000 [47074f4bfd]  Joshua Colp <jcolp@digium.com>
 
-	* channels/pjsip/dialplan_functions: Add an option for extracting the SIP call-id
+	* res_pjsip: Ensure in-dialog responses have an endpoint associated.
 
-	  This patch adds a new option to the CHANNEL function that allows for the
-	  extraction of the SIP call-id. It is used in conjunction with the 'pjsip'
-	  option, and will return the Call-ID of the INVITE request that established
-	  the PJSIP channel.
+	  When handling incoming messages we determine if it is associated with
+	  a dialog. If so we use that to determine what serializer and endpoint
+	  to use for the message. Previously this would pass the endpoint to the
+	  endpoint lookup module to actually place the endpoint completely on the
+	  message. For in-dialog responses, however, this did not occur as
+	  dialog processing took over and the endpoint lookup did not occur.
 
-	  ASTERISK-25352
+	  This change just places the endpoint in the expected spot immediately
+	  instead of relying on the endpoint lookup module. In-dialog responses
+	  thus have the expected endpoint.
 
-	  Change-Id: I278d1f8bcfe3a53c5aa1dadebc14e92b0abd476a
+	  AST-1459 #close
 
-2015-09-04 16:06 +0000 [61c6c6aa6c]  David M. Lee <dlee@respoke.io>
+	  Review: https://reviewboard.asterisk.org/r/4146/
+	  ........
 
-	* Fix when remote candidates exceed PJ_ICE_MAX_CAND
+	  Merged revisions 427490 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  We were passing the wrong count into pj_ice_sess_create_check_list(),
-	  causing the create to fail if we ever received more than PJ_ICE_MAX_CAND
-	  candidates.
+	  Merged revisions 427491 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I0303d8e1ecb20a8de9fe629a3209d216c4028378
 
-2015-09-04 14:40 +0000 [ac62928d6b]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427492 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Change default from user value.
+2014-11-06 06:15 +0000 [4d80f223af]  Corey Farrell <git@cfware.com>
 
-	  When Asterisk sends an outbound SIP request, if there is no direct
-	  reason to place a specific value for the username in the From header,
-	  Asterisk would generate a UUID. For example, this would happen when
-	  sending outbound OPTIONS requests when qualifying or when sending
-	  outbound INVITE requests when originating (if no explicit caller ID were
-	  provided). The issue is that some SIP providers reject these sorts of
-	  requests with a "Name too long" error response.
+	* main/file.c: fix possible extra ast_module_unref to format modules.
 
-	  This patch aims to fix this by changing the default outbound username in
-	  From headers to "asterisk". This value can be overridden by changing the
-	  default_from_user option in the global options if desired.
+	  fn_wrapper only adds a reference to the format's module if the file
+	  was able to be opened.  If not this causes an unmatched
+	  ast_module_unref in filestream_destructor.  Move ast_module_ref to
+	  get_stream.
 
-	  ASTERISK-25377 #close
-	  Reported by Mark Michelson
+	  ASTERISK-24492 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4149/
+	  ........
 
-	  Change-Id: I6a4d34a56ff73ff4f661b0075aeba5461b7f3190
+	  Merged revisions 427464 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-09-04 09:26 +0000 [6002472a62]  Scott Griepentrog <scott@griepentrog.com>
+	  Merged revisions 427465 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* endpoint snapshot: avoid second cleanup on alloc failure
+	  Merged revisions 427466 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  In ast_endpoint_snapshot_create(), a failure to init the
-	  string fields results in two attempts to ao2_cleanup the
-	  same pointer.  Removed RAII_VAR to eliminate problem.
 
-	  ASTERISK-25375 #close
-	  Reported by: Scott Griepentrog
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427467 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: If4d9dfb1bbe3836b623642ec690b6d49b25e8979
+2014-11-06 03:24 +0000 [c46664305a]  Corey Farrell <git@cfware.com>
 
-2015-09-04 05:33 +0000 [d32e516c7c]  Martin Tomec <tomec.martin@gmail.com>
+	* res_hep: fix major leak that occurs when config is missing or enabled=no.
 
-	* res/pjsip: Mark WSS transport as secure
+	  Add missing unreference in hepv3_send_packet.
 
-	  Pjsip is refusing to use unsecure transport with "sips" in url.
-	  WSS should be considered as secure transport.
+	  ASTERISK-24491 #close
+	  Reported by: Zane Conkle
+	  Tested by: Zane Conkle
+	  Review: https://reviewboard.asterisk.org/r/4150/
+	  ........
 
-	  ASTERISK-24602 #comment Partially fixed by setting WSS as secure
+	  Merged revisions 427400 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Iddac406c6deba6240c41a603b8859dfefe1a5353
+	  Merged revisions 427405 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-09-02 17:26 +0000 [ad9cb6c2ce]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip: Fix contact refleak on stateful responses.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427408 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When sending a stateful response, creation of the transaction can fail,
-	  most commonly because we are trying to create a transaction from a
-	  retransmitted request. When creation of the transaction fails, we end up
-	  leaking a reference to a contact that was bumped when the response was
-	  created.
+2014-11-06 03:18 +0000 [7e2369310c]  Corey Farrell <git@cfware.com>
 
-	  This patch adds the missing deref and fixes the reference leak.
+	* Fix unintential memory retention in stringfields.
 
-	  Change-Id: I2f97ad512aeb1b17e87ca29ae0abacb4d6395f07
+	  * Fix missing / unreachable calls to __ast_string_field_release_active.
+	  * Reset pool->used to zero when the current pool->active reaches zero.
 
-2015-09-02 12:41 +0000 [cc1363209e]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24307 #close
+	  Reported by: Etienne Lessard
+	  Tested by: ibercom, Etienne Lessard
+	  Review: https://reviewboard.asterisk.org/r/4114/
+	  ........
 
-	* pbx: Fix crash when issuing "core show hints" with long pattern match.
+	  Merged revisions 427380 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  When issuing the "core show hints" CLI command a combination of both
-	  the hint extension and context is created. This uses a fixed size
-	  buffer expecting that the extension will not exceed maximum extension
-	  length. When the extension is actually a pattern match this constraint
-	  does not hold true, and the extension may exceed the maximum extension
-	  length. In this case extra characters are written past the end of the
-	  fixed size buffer.
+	  Merged revisions 427381 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This change makes it so the construction of the combined hint extension
-	  and context can not exceed the size of the buffer.
+	  Merged revisions 427382 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25367 #close
+	  Merged revisions 427384 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Idfa1b95d0d4dc38e675be7c1de8900b3f981f499
 
-2015-09-01 09:05 +0000 [d58c8d73af]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427388 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_pubsub: re-re-fix persistent subscription storage.
+2014-11-05 20:41 +0000 [362dde2229]  gtjoseph <george.joseph@fairview5.com>
 
-	  A recent change to res_pjsip_pubsub switched to using pjsip_msg_print as
-	  a means of writing an appropriate packet to persistent storage. While
-	  this partially solved the issue, it had its own problems.
-	  pjsip_msg_print will always add a Content-Length header to the message
-	  it prints. Frequent restarts of Asterisk can result in persistent
-	  subscriptions being written with five or more Content-Length headers. In
-	  addition, sometimes some apparent corruption of individual headers could
-	  be seen.
+	* test_strings:  Remove string tests that exercise asserts.
 
-	  This aims to fix the problem by not running a parsed message through an
-	  interpreter but rather by taking the raw message and saving it. The
-	  logic for what to save is going to be different depending on whether a
-	  SUBSCRIBE was received from the wire or if it was pulled from
-	  persistence. When receiving a packet from the wire, when using a
-	  streaming transport, the rdata->pkt_info.packet may contain multiple SIP
-	  messages or fragments. However, the rdata->msg_info.msg_buf will always
-	  contain the current SIP message to be processed. When pulling from
-	  persistence, though, the rdata->msg_info.msg_buf will be NULL since no
-	  transport actually handled the packet. However, since we know that we
-	  will always ever pull one SIP message from persistence, we are free to
-	  save directly from rdata->pkt_info.packet instead.
+	  Since unit tests are run with DO_CRASH, those tests were causing
+	  the test to fail.
 
-	  ASTERISK-25365 #close
-	  Reported by Mark Michelson
+	  Tested-by: George Joseph
+	  ........
 
-	  Change-Id: I33153b10d0b4dc8e3801aaaee2f48173b867855b
+	  Merged revisions 427354 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-08-31 15:24 +0000 [03fe79f29e]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 427355 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Fix deadlock on presence state changes.
+	  Merged revisions 427356 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  A deadlock was observed where three threads were competing for different
-	  locks:
 
-	  * One thread held the hints lock and was attempting to lock a specific
-	    hint.
-	  * One thread was holding the specific hint's lock and was attempting to
-	    lock the contexts lock
-	  * One thread was holding the contexts lock and attempting to lock the
-	    hints lock.
-
-	  Clearly the second thread was doing the wrong thing here. The fix for
-	  this is to make sure that the hint's lock is not held on presence state
-	  changes. Something similar is already done (and commented about) for
-	  device state changes.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427357 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25362 #close
-	  Reported by Mark Michelson
+2014-11-05 13:53 +0000 [69f29e627f]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I15ec2416b92978a4c0c08273b2d46cb21aff97e2
+	* Make the disable_tcp_switch PJSIP system object enabled by default.
 
-2015-08-29 10:36 +0000 [a676ba2aad]  Joshua Colp <jcolp@digium.com>
+	  Testing has shown repeatedly that PJSIP's default behavior of switching
+	  automatically to TCP for large messages can cause issues. The most common
+	  issues are that devices that we are communicating with do not handle the
+	  switch to TCP gracefully, thus causing situations such as broken calls or
+	  broken subscriptions. Now, in order to have this behavior happen, you must
+	  opt into it. The sample file has been updated to warn that enabling the
+	  TCP switch behavior may cause issues for you, so use at your own risk.
+	  ........
 
-	* taskprocessor: Fix race condition between unreferencing and finding.
+	  Merged revisions 427334 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When unreferencing a taskprocessor its reference count is checked
-	  to determine if it should be unlinked from the taskprocessors
-	  container and its listener shut down. In between the time when the
-	  reference count is checked and unlinking it is possible for
-	  another thread to jump in, find it, and get a reference to it. If
-	  the thread then uses the taskprocessor it may find that it is not
-	  in the state it expects.
 
-	  This change locks the taskprocessors container during almost the
-	  entire unreference operation to ensure that any other thread which
-	  may attempt to find the taskprocessor has to wait.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427335 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25295
+2014-11-05 06:19 +0000 [b06078880b]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Icb842db82fe1cf238da55df92e95938a4419377c
+	* res_pjsip_multihomed: Add logging during startup to aid debugging if local DNS is misbehaving.
 
-2015-08-28 20:22 +0000 [1b1561f4c8]  Joshua Colp <jcolp@digium.com>
+	  This change adds a bit of logging so if the local DNS is misbehaving it is easier
+	  to track down what is going on and where Asterisk may be hanging.
 
-	* res_pjsip_sdp_rtp: Fix multiple keepalive scheduled items.
+	  ASTERISK-24438 #close
+	  Reported by: Melissa Shepherd
 
-	  The keepalive support in res_pjsip_sdp_rtp currently assumes
-	  that a stream will only be negotiated once. This is false.
-	  If the stream is replaced and later added back it can be
-	  negotiated again causing multiple keepalive scheduled items
-	  to exist. This change explicitly deletes the existing
-	  keepalive scheduled item before adding the new one.
+	  Review: https://reviewboard.asterisk.org/r/4148/
+	  ........
 
-	  The res_pjsip_sdp_rtp module also does not stop RTP
-	  keepalives or timeout timer if the stream has been
-	  replaced. This change adds a callback to the session media
-	  interface to allow a media stream to be stopped without
-	  the resources being destroyed. This allows the scheduled
-	  items and RTP to be stopped when the stream no longer
-	  exists.
+	  Merged revisions 427300 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25356 #close
+	  Merged revisions 427303 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ibe6a7cc0927c87326fd5f1c0d4ad889dbfbea1de
 
-2015-08-28 19:57 +0000 [85e1cb51b2]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427306 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* sched: ast_sched_del may return prematurely due to spurious wakeup
+2014-11-04 18:17 +0000 [d5de94201e]  gtjoseph <george.joseph@fairview5.com>
 
-	  When deleting a scheduled item if the item in question is currently
-	  executing the ast_sched_del function waits until it has completed.
-	  This is accomplished using ast_cond_wait. Unfortunately the
-	  ast_cond_wait function can suffer from spurious wakeups so the
-	  predicate needs to be checked after it returns to make sure it has
-	  really woken up as a result of being signaled.
+	* config: Make text_file_save and 'dialplan save' escape semicolons in values.
 
-	  This change adds a loop around the ast_cond_wait to make sure that
-	  it only exits when the executing task has really completed.
+	  When a config file is read, an unescaped semicolon signals comments which are
+	  stripped from the value before it's stored.  Escaped semicolons are then
+	  unescaped and become part of the value.  Both of these behaviors are normal
+	  and expected.  When the config is serialized either by 'dialplan save' or
+	  AMI/UpdateConfig however, the now unescaped semicolons are written as-is.
+	  If you actually reload the file just saved, the unescaped semicolons are
+	  now treated as start of comments.
 
-	  ASTERISK-25355 #close
+	  Since true comments are stripped on read, any semicolons in
+	  ast_variable.value must have been escaped originally.  This patch
+	  re-escapes semicolons in ast_variable.values before they're written to
+	  file either by 'dialplan save' or config/ast_config_text_file_save which
+	  is called by AMI/UpdateConfig. I also fixed a few pre-existing formatting
+	  issues nearby in pbx_config.c
 
-	  Change-Id: I51198270eb0b637c956c61aa409f46283432be61
+	  Tested-by: George Joseph
+	  ASTERISK-20127 #close
 
-2015-08-27 12:26 +0000 [c2c7319082]  Joshua Colp <jcolp@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4132/
+	  ........
 
-	* res_pjsip_session: Don't invoke session supplements twice for BYE requests.
+	  Merged revisions 427275 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  When a BYE request is received the PJSIP invite session implementation
-	  creates and sends a 200 OK response before we are aware of it. This
-	  causes the INVITE session state callback to be called into and ultimately
-	  the session supplements run on the BYE request. Once this response has
-	  been sent the normal transaction state callback is invoked which
-	  invokes the session supplements on the BYE request again. This can
-	  be problematic in particular with res_pjsip_rfc3326 as it may
-	  attempt to update the hangup cause code on the channel while it is
-	  in the process of being hung up.
+	  Merged revisions 427276 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change makes it so the session supplements are only invoked
-	  once by the INVITE session state callback.
 
-	  ASTERISK-25318 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427277 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I69c17df55ccbb61ef779ac38cc8c6b411376c19a
+2014-11-04 16:51 +0000 [c77a71ad2f]  Joshua Colp <jcolp@digium.com>
 
-2015-08-26 15:26 +0000 [6862c2a167]  Scott Griepentrog <scott@griepentrog.com>
+	* res_pjsip: Apply the 'user_eq_phone' setting to the To header as well.
 
-	* Chaos: handle failed allocation in get_media_encryption_type
 
-	  If the ast_strndup() call fails to allocate a copy of the
-	  transport string for parsing, fail gracefully.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427259 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25323
-	  Reported by: Scott Griepentrog
+2014-11-04 16:31 +0000 [5e43d68717]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ia4b905ce6d03da53fea526224455c1044b1a5a28
+	* res_pjsip: Allow + at the beginning of a phone number when user_eq_phone is enabled.
 
-2015-08-26 14:25 +0000 [f1cd636658]  Scott Griepentrog <scott@griepentrog.com>
 
-	* Chaos: make hangup NULL tolerant
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427257 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In chan_pjsip_new, if allocation of the pvt
-	  structure fails, ast_hangup is called.  But
-	  it was written to assume pvt was valid, and
-	  this change corrects that.
+2014-11-04 14:49 +0000 [bd42a09d7f]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25323
-	  Reported by: Scott Griepentrog
+	* config: BUG: Restore ability for non-templ to be used as base objs in config.
 
-	  Change-Id: I5f47860fe9cee4cd56abd3f79b108678ab72cc87
+	  My recent refactor of config.c accidentally removed the capability for an
+	  object to inherit from a non-template object.
 
-2015-08-26 05:40 +0000 [c01111223f]  Joshua Colp <jcolp@digium.com>
+	  This patch restores the capability to inherit from both template and
+	  non-template objects.
 
-	* chan_sip: Allow call pickup to set the hangup cause.
+	  Tested-by: George Joseph
+	  Reported-by: Scott Griepentrog
+	  ASTERISK-24487 #close
 
-	  The call pickup implementation in chan_sip currently sets the channel
-	  hangup cause to "normal clearing" if call pickup is successfully
-	  performed. This action overwrites the "answered elsewhere" hangup cause
-	  set by the call pickup code and can result in the SIP device in
-	  question showing a missed call when it should not.
+	  Review: https://reviewboard.asterisk.org/r/4147/
+	  ........
 
-	  This change sets the hangup cause to "normal clearing" as a
-	  default initially but allows the call pickup to change it as
-	  needed.
+	  Merged revisions 427227 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25346 #close
+	  Merged revisions 427228 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I00ac2c269cee9e29586ee2c65e83c70e52a02cff
 
-2015-08-25 07:17 +0000 [2a4eee0cd9]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427229 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Add common ast_sip_get_host_ip API.
+2014-11-04 13:46 +0000 [97e1c7f3a9]  Corey Farrell <git@cfware.com>
 
-	  Modules commonly used the pj_gethostip function for retrieving the
-	  IP address of the host. This function does not cache the result and may
-	  result in a DNS lookup occurring, or additional work. If the DNS
-	  server is unreachable or network issues arise this can cause the
-	  pj_gethostip function to block for a period of time.
+	* func_talkdetect: Fix stasis message leak in audiohook callback.
+
+	  ASTERISK-24482 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4142/
+	  ........
+
+	  Merged revisions 427203 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change adds an ast_sip_get_host_ip and ast_sip_get_host_ip_string
-	  function which does the same thing but caches the host IP address at
-	  module load time. This results in no additional work being done each
-	  time the local host IP address is needed.
+	  Merged revisions 427204 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25342 #close
 
-	  Change-Id: I3205deb679b01fa5ac05a94b623bfd620a2abe1e
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427205 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-24 11:04 +0000 [7c4d0c3506]  Joshua Colp <jcolp@digium.com>
+2014-11-04 13:33 +0000 [9f2874639d]  Corey Farrell <git@cfware.com>
 
-	* res_pjsip_pubsub: On recreated notify fail deleted sub_tree is referenced
+	* res_http_websockets: Fix extra unref of module
 
-	  When recreating a subscription it is possible for a freed sub_tree
-	  to be referenced when the initial NOTIFY fails to be created.
+	  In websocket_add_protocol_internal is used to add the "echo"
+	  protocol, but ast_websocket_remove_protocol is used to remove
+	  it.  This causes an extra call to ast_module_unref.
 
-	  Change-Id: I681c215309aad01b21d611c2de47b3b0a6022788
+	  ASTERISK-24480 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4140/
+	  ........
 
-2015-08-24 06:21 +0000 [6c2dab1e88]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 427200 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* bridge: Kick channel from bridge if hung up during action.
+	  Merged revisions 427201 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When executing an action in a bridge it is possible for the
-	  channel to be hung up without the bridge becoming aware of it.
-	  This is most easily reproducible by hanging up when the bridge
-	  is streaming DTMF due to a feature timeout. This change makes
-	  it so after action execution the channel is checked to determine
-	  if it has been hung up and if it has it is kicked from the bridge.
 
-	  ASTERISK-25341 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427202 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I6dd8b0c3f5888da1c57afed9e8a802ae0a053062
+2014-11-04 08:11 +0000 [bdc35c77b9]  Corey Farrell <git@cfware.com>
 
-2015-08-23 18:26 +0000 [bc6fe07f5c]  Matt Jordan <mjordan@digium.com>
+	* Fix crash caused by merge error on review 4138
 
-	* res_pjsip/pjsip_configuration: Disregard empty auth values
+	  When merging from 12 to 13 there were conflicts,
+	  I mistakenly had the loop run ast_closestream(others[0])
+	  when it should be ast_closestream(others[x]).
+	  ........
 
-	  When an endpoint is backed by a non-static conf file backend (such as
-	  the AstDB or Realtime), the 'auth' object may be returned as being an
-	  empty string. Currently, res_pjsip will interpret that as being a valid
-	  auth object, and will attempt to authenticate inbound requests. This
-	  isn't desired; is an auth value is empty (which the name of an auth
-	  object cannot be), we should instead interpret that as being an invalid
-	  auth object and skip it.
+	  Merged revisions 427181 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25339 #close
 
-	  Change-Id: Ic32b0c6eb5575107d5164a8c40099e687cd722c7
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427182 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-19 12:10 +0000 [0582776f7f]  Richard Mudgett <rmudgett@digium.com>
+2014-11-04 06:03 +0000 [d159885e50]  Joshua Colp <jcolp@digium.com>
 
-	* ari/ari_websockets.c: Fix ast_debug parameter type mismatch.
+	* res_pjsip_outbound_registration: Add virtual line support.
 
-	  This is a type mismatch fix of the debugging commit
-	  c63316eec10e1990a88bf4712238d6deb375bfa9 made to find out why
-	  a testsuite test was failing only on one of the continuous
-	  integration build agents.
+	  Virtual line support establishes a relationship between messages
+	  related to an outbound registration and a local endpoint. This is
+	  accomplished by attaching a parameter to the Contact of the outbound
+	  registration and looking for it on any received requests. If the
+	  parameter exists and can be matched to an outbound registration
+	  the configured endpoint is associated with the request.
 
-	  Change-Id: Iba34f6e87cec331f6ac80e4daff6476ea6f00a75
+	  Review: https://reviewboard.asterisk.org/r/2964/
 
-2015-08-19 10:30 +0000 [504213f542]  Scott Griepentrog <scott@griepentrog.com>
 
-	* contrib: script install_prereq should install sqlite3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427165 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Asterisk needs the sqlite 3 library, which is package
-	  sqlite-devel in CentOS. By adding this package to the
-	  script, a problem with configure failing is resolved.
+2014-11-03 12:22 +0000 [33f0251b6c]  Richard Mudgett <rmudgett@digium.com>
 
-	  ASTERISK-25331 #close
-	  Reported by: Kevin Harwell
+	* res_pjsip: Add disable_tcp_switch option.
 
-	  Change-Id: I90efaf6a01914fea03f21e5cdbd91c348f44b0ec
+	  When a packet exceeds the MTU, pjproject will switch from UDP to TCP.  In
+	  some circumstances (on some networks), this can cause some issues with
+	  messages not getting sent to the correct destination - and can also cause
+	  connections to get dropped due to quirks in pjproject deciding to
+	  terminate TCP connections with no messages.
 
-2015-08-18 16:06 +0000 [77518d5434]  Richard Mudgett <rmudgett@digium.com>
+	  While fixing the routing/messaging issues is important, having a
+	  configuration option in Asterisk that tells pjproject to not switch over
+	  to TCP would be useful.  That way, if some glitch is discovered on some
+	  other network/site, we can at least disable the behavior until a fix is
+	  put into place.
 
-	* res_http_websocket.c: Fix some off nominal path cleanup.
+	  AFS-197 #close
 
-	  * Remove extraneous unlock on off-nominal path.
-	  * Add missing HTTP error reply.
+	  Review: https://reviewboard.asterisk.org/r/4137/
+	  ........
 
-	  Change-Id: I1f402bfe448fba8696b507477cab5f060ccd9b2b
+	  Merged revisions 427129 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-08-18 14:46 +0000 [c61547fee6]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427130 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_ari.c: Add missing off nominal unlock and remove a RAII_VAR().
 
-	  Change-Id: I0c5e7b34057f26dadb39489c4dac3015c52f5dbf
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427137 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-17 16:41 +0000 [bd867cd078]  Richard Mudgett <rmudgett@digium.com>
+2014-11-03 09:03 +0000 [b9aeff9580]  Joshua Colp <jcolp@digium.com>
 
-	* app_queue.c: Extract some functions for simpler code.
+	* chan_pjsip: Update CHANGES file to include 'moh_passthrough' setting
 
-	  * Extract set_queue_member_pause() from set_member_paused() for simpler
-	  and more consistent code.
 
-	  * Extract set_queue_member_ringinuse() from
-	  set_member_ringinuse_help_members() for simpler code.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427113 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iecc1f4119c63347341d7ea6b65f5fc4963706306
+2014-11-03 08:45 +0000 [ac091d4184]  Joshua Colp <jcolp@digium.com>
 
-2015-08-14 12:55 +0000 [e5f5b9f384]  Richard Mudgett <rmudgett@digium.com>
+	* chan_pjsip: Add support for passing hold and unhold requests through.
 
-	* app_queue.c: Fix setting QUEUE_MEMBER 'paused' and 'ringinuse'.
+	  This change adds an option, moh_passthrough, that when enabled will pass
+	  hold and unhold requests through using a SIP re-invite. When placing on
+	  hold a re-invite with sendonly will be sent and when taking off hold a
+	  re-invite with sendrecv will be sent. This allows remote servers to handle
+	  the musiconhold instead of the local Asterisk instance being responsible.
 
-	  Setting the 'paused' and 'ringinuse' options on a queue member using the
-	  dialplan function QUEUE_MEMBER did not behave the same way as the
-	  equivalent dialplan applications or AMI actions.
+	  Review: https://reviewboard.asterisk.org/r/4103/
 
-	  * Made queue_function_mem_write() call the set_member_paused() and
-	  set_member_value() for the 'paused' and 'ringinuse' options respectively.
-	  A beneficial side effect is that the queue name is now optional and sets
-	  the value in all queues the interface is a member.
 
-	  * Update QUEUE_MEMBER XML documentation.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427112 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Fix error checking in QUEUE_MEMBER() write.
+2014-11-02 20:36 +0000 [285be15aaf]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25215 #close
-	  Reported by: Lorne Gaetz
+	* Fix compile error caused by review 4138
 
-	  Change-Id: I3a016be8dc94d63a9cc155295ff9c9afa5f707cb
+	  There is no procedure called ast_closeframe, fix code to use
+	  ast_closestream.
 
-2015-08-17 13:34 +0000 [ded51e3d77]  Richard Mudgett <rmudgett@digium.com>
+	  Reported By: Matt Jordan
+	  ........
 
-	* app_queue.c: Fix error checking in QUEUE_MEMBER() read.
+	  Merged revisions 427087 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I7294e13d27875851c2f4ef6818adba507509d224
+	  Merged revisions 427088 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-08-17 11:00 +0000 [ab373f2cef]  Scott Griepentrog <scott@griepentrog.com>
+	  Merged revisions 427089 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* CHAOS: prevent sorcery object with null id
 
-	  When allocating a sorcery object, fail if the
-	  id value was not allocated.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427090 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25323
-	  Reported by: Scott Griepentrog
+2014-11-02 02:13 +0000 [509c04ef38]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: I152133fb7545a4efcf7a0080ada77332d038669e
+	* Fix ast_writestream leaks
 
-2015-08-14 15:46 +0000 [b719f56c72]  Mark Michelson <mmichelson@digium.com>
+	  Fix cleanup in __ast_play_and_record where others[x] may be leaked.
+	  This was caught where prepend != NULL && outmsg != NULL, once
+	  realfile[x] == NULL any further others[x] would be leaked. A cleanup
+	  block was also added for prepend != NULL && outmsg == NULL.
 
-	* res_pjsip_sdp_rtp: Restore removed NULL check.
+	  11+: Fix leak of ast_writestream recording_fs in
+	  app_voicemail:leave_voicemail.
 
-	  When sending an RTP keepalive, we need to be sure we're not dealing with
-	  a NULL RTP instance. There had been a NULL check, but the commit that
-	  added the rtp_timeout and rtp_hold_timeout options removed the NULL
-	  check.
+	  ASTERISK-24476 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4138/
+	  ........
 
-	  Change-Id: I2d7dcd5022697cfc6bf3d9e19245419078e79b64
+	  Merged revisions 427023 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-08-13 12:30 +0000 [cea5dc7b8a]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 427024 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* audiohook.c: Simplify variable usage in audiohook_read_frame_both().
+	  Merged revisions 427025 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I58bed58631a94295b267991c5b61a3a93c167f0c
+	  Merged revisions 427026 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-08-13 12:22 +0000 [b3a56bee83]  Richard Mudgett <rmudgett@digium.com>
 
-	* audiohook.c: Fix MixMonitor crash when using the r() or t() options.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427027 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The built frame format in audiohook_read_frame_both() is now set to a
-	  signed linear format before the rx and tx frames are duplicated instead of
-	  only for the mixed audio frame duplication.
+2014-11-02 01:40 +0000 [85c1822a9d]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25322 #close
-	  Reported by Sean Pimental
+	* func_jitterbuffer: fix frame leaks.
 
-	  Change-Id: I86f85b5c48c49e4e2d3b770797b9d484250a1538
+	  Fix code paths where it is possible for frames to leak.
+	  Fix uninitialized variable in jb_get_fixed and jb_get_adaptive.
 
-2015-08-12 12:59 +0000 [25af2d71c8]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-22409 #related
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4128/
+	  ........
 
-	* chan_sip.c: wrong peer searched in sip_report_security_event
+	  Merged revisions 427019 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  In chan_sip, after handling an incoming invite a security event is raised
-	  describing authorization (success, failure, etc...). However, it was doing
-	  a lookup of the peer by extension. This is fine for register messages, but
-	  in the case of an invite it may search and find the wrong peer, or a non
-	  existent one (for instance, in the case of call pickup). Also, if the peers
-	  are configured through realtime this may cause an unnecessary database lookup
-	  when caching is enabled.
+	  Merged revisions 427020 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch makes it so that sip_report_security_event searches by IP address
-	  when looking for a peer instead of by extension after an invite is processed.
+	  Merged revisions 427021 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25320 #close
 
-	  Change-Id: I9b3f11549efb475b6561c64f0e6da1a481d98bc4
-2015-08-13 05:26 +0000 [e18c300550]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@427022 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_http_websocket: When shutting down a session don't close closed socket
+2014-11-01 20:01 +0000 [5db1c978e3]  Matt Jordan <mjordan@digium.com>
 
-	  Due to the use of ast_websocket_close in session termination it is
-	  possible for the underlying socket to already be closed when the
-	  session is terminated. This occurs when the close frame is attempted
-	  to be written out but fails.
+	* res/res_stasis: Fix crash on module unload while performing operation
 
-	  Change-Id: I7572583529a42a7dc911ea77a974d8307d5c0c8b
-2015-08-11 05:24 +0000 [b4e9416138]  Joshua Colp <jcolp@digium.com>
+	  When the res_stasis module is unloaded, it will dispose of the apps_registry
+	  container. This is a problem if an ARI operation is in flight that attempts
+	  to use the registry, as the shutdown occurs in a separate thread. This patch
+	  adds some sanity checks to the various routines that access the registry which
+	  cause the operations to fail if the apps_registry does not exist.
 
-	* res_http_websocket: Forcefully terminate on write errors.
+	  Crash caught by the Asterisk Test Suite.
+	  ........
 
-	  The res_http_websocket module will currently attempt to close
-	  the WebSocket connection if fatal cases occur, such as when
-	  attempting to write out data and being unable to. When the
-	  fatal cases occur the code attempts to write a WebSocket close
-	  frame out to have the remote side close the connection. If
-	  writing this fails then the connection is not terminated.
+	  Merged revisions 426995 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change forcefully terminates the connection if the
-	  WebSocket is to be closed but is unable to send the close frame.
+	  Merged revisions 426996 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25312 #close
 
-	  Change-Id: I10973086671cc192a76424060d9ec8e688602845
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426997 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-10 13:43 +0000 [256bc52b66]  Richard Mudgett <rmudgett@digium.com>
+2014-10-31 11:52 +0000 [4219c40775]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	* chan_dahdi.c: Flush the DAHDI write buffer after starting DTMF.
+	* install init.d files on GNU/kFreeBSD
 
-	  Pressing DTMF digits on a phone to go out on a DAHDI channel can result in
-	  the digit not being recognized or even heard by the peer.
+	  Review: https://reviewboard.asterisk.org/r/4118/
+	  ........
 
-	  Phone -> Asterisk -> DAHDI/channel
+	  Merged revisions 426926 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Turns out the DAHDI behavior with DTMF generation (and any other generated
-	  tones) is exposed by the "buffers=" setting in chan_dahdi.conf.  When
-	  Asterisk requests to start sending DTMF then DAHDI waits until its write
-	  buffer is empty before generating any samples for the DTMF tones.  When
-	  Asterisk subsequently requests DAHDI to stop sending DTMF then DAHDI
-	  immediately stops generating the DTMF samples.  As a result, the more
-	  samples there are in the DAHDI write buffer the shorter the time DTMF
-	  actually gets sent on the wire.  If there are more samples in the write
-	  buffer than the time DTMF is supposed to be sent then no DTMF gets sent on
-	  the wire.  With the "buffers=12,half" setting and each buffer representing
-	  20 ms of samples then the DAHDI write buffer is going to contain around
-	  120 ms of samples.  For DTMF to be recognized by the peer the actual sent
-	  DTMF duration needs to be a minimum of 40 ms.  Therefore, the intended
-	  duration needs to be a minimum of 160 ms for the peer to receive the
-	  minimum DTMF digit duration to recognize it.
+	  Merged revisions 426927 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  A simple and effective solution to work around the DAHDI behavior is for
-	  Asterisk to flush the DAHDI write buffer when sending DTMF so the full
-	  duration of DTMF is actually sent on the wire.  When someone is going to
-	  send DTMF they are not likely to be talking before sending the tones so
-	  the flushed write samples are expected to just contain silence.
+	  Merged revisions 426933 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Made dahdi_digit_begin() flush the DAHDI write buffer after requesting
-	  to send a DTMF digit.
+	  Merged revisions 426934 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25315 #close
-	  Reported by John Hardin
 
-	  Change-Id: Ib56262c708cb7858082156bfc70ebd0a220efa6a
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426935 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-05 14:21 +0000 [800e0ea48d]  Richard Mudgett <rmudgett@digium.com>
+2014-10-31 11:41 +0000 [28173ddf05]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	* chan_dahdi.c: Lock private struct for ast_write().
+	* pjsip: clarify tls cert and key file usage
 
-	  There is a window of opportunity for DTMF to not go out if an audio frame
-	  is in the process of being written to DAHDI while another thread starts
-	  sending DTMF.  The thread sending the audio frame could be past the
-	  currently dialing check before being preempted by another thread starting
-	  a DTMF generation request.  When the thread sending the audio frame
-	  resumes it will then cause DAHDI to stop the DTMF tone generation.  The
-	  result is no DTMF goes out.
+	  A question arose as to whether a .pem file
+	  could be provided in place of the .crt and
+	  .key files in a PJSIP TLS configuration. I
+	  tested this and discovered that although a
+	  cert will be read from the pem file, a key
+	  will not, and thus the priv_key_file entry
+	  is still required. This update to the fine
+	  documentation clarifies the option usage.
 
-	  * Made dahdi_write() lock the private struct before writing to the DAHDI
-	  file descriptor.
+	  AST-1448 #close
+	  Review: https://reviewboard.asterisk.org/r/4129/
+	  Reported by: John Bigelow
+	  ........
 
-	  ASTERISK-25315
-	  Reported by John Hardin
+	  Merged revisions 426928 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Ib4e0264cf63305ed5da701188447668e72ec9abb
+	  Merged revisions 426930 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-08-10 18:23 +0000 [c126afe18f]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip.c: Fix crash from corrupt saved SUBSCRIBE message.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426932 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  If the saved SUBSCRIBE message is not parseable for whatever reason then
-	  Asterisk could crash when libpjsip tries to parse the message and adds an
-	  error message to the parse error list.
+2014-10-31 11:24 +0000 [f59db388a7]  John Bigelow (License 5091)
 
-	  * Made ast_sip_create_rdata() initialize the parse error rdata list.  The
-	  list is checked after parsing to see that it remains empty for the
-	  function to return successful.
+	* pjsip: Handle outbound unregister correctly
 
-	  ASTERISK-25306
-	  Reported by Mark Michelson
+	  This updates the status of the outbound registration
+	  to reflect when it has been unregistered.  Since the
+	  registration is unregistered but is not stopped, the
+	  registration schedule remains active as before.  The
+	  patch also updates the documentation of both the AMI
+	  and CLI commands.
 
-	  Change-Id: Ie0677f69f707503b1a37df18723bd59418085256
+	  ASTERISK-24411 #close
+	  Review: https://reviewboard.asterisk.org/r/4119/
+	  Reported by: John Bigelow
+	  patches:
+	    unregister-patch1.txt uploaded by John Bigelow (License 5091)
+	  ........
 
-2015-08-10 07:40 +0000 [f68c995bc9]  Alexander Traud <pabstraud@compuserve.com>
+	  Merged revisions 426923 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* chan_sip: Fix negotiation of iLBC 30.
+	  Merged revisions 426924 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  iLBC 20 was advertised in a SIP/SDP negotiation. However, only iLBC 30 is
-	  supported. Removes "a=fmtp:x mode=y" from SDP. Because of RFC 3952 section 5,
-	  only iLBC 30 is negotiated now.
 
-	  ASTERISK-25309 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426925 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I92d724600a183eec3114da0ac607b994b1a793da
+2014-10-30 22:26 +0000 [d88282af40]  Matt Jordan <mjordan@digium.com>
 
-2015-08-09 18:42 +0000 [8e194047ac]  Matt Jordan <mjordan@digium.com>
+	* channels/sip/reqresp_parser: Fix unit tests for r426594
 
-	* res/res_format_attr_silk: Expose format attributes to other modules
+	  When r426594 was made, it did not take into account a unit test that verified
+	  that the function properly populated the unsupported buffer. The function
+	  would previously memset the buffer if it detected it had any contents; since
+	  this function can now be called iteratively on successive headers, the unit
+	  tests would now fail. This patch updates the unit tests to reset the buffer
+	  themselves between successive calls, and updates the documentation of the
+	  function to note that this is now required.
+	  ........
 
-	  This patch adds the .get callback to the format attribute module, such
-	  that the Asterisk core or other third party modules can query for the
-	  negotiated format attributes.
+	  Merged revisions 426858 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Change-Id: Ia24f55cf9b661d651ce89b4f4b023d921380f19c
+	  Merged revisions 426860 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-08-09 17:56 +0000 [a0f451c35e]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 426863 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* main/format: Add an API call for retrieving format attributes
+	  Merged revisions 426865 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Some codecs that may be a third party library to Asterisk need to have
-	  knowledge of the format attributes that were negotiated. Unfortunately,
-	  when the great format migration of Asterisk 13 occurred, that ability
-	  was lost.
 
-	  This patch adds an API call, ast_format_attribute_get, to the core
-	  format API, along with updates to the unit test to check the new API
-	  call. A new callback is also now available for format attribute modules,
-	  such that they can provide the format attribute values they manage.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426868 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Note that the API returns a void *. This is done as the format attribute
-	  modules themselves may store format attributes in any particular manner
-	  they like. Care should be taken by consumers of the API to check the
-	  return value before casting and dereferencing. Consumers will obviously
-	  need to have a priori knowledge of the type of the format attribute as
-	  well.
+2014-10-30 22:09 +0000 [bf684b63a3]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Ieec76883dfb46ecd7aff3dc81a52c81f4dc1b9e3
+	* REF_DEBUG: Install refcounter.py to $(ASTDATADIR)/scripts
 
-2015-08-07 22:11 +0000 [26f0559a94]  David M. Lee <dlee@respoke.io>
+	  This change ensures refcounter.py is installed to a place where it
+	  can be found by the Asterisk testsuite if REF_DEBUG is enabled.
 
-	* Replace htobe64 with htonll
+	  ASTERISK-24432 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4094/
+	  ........
 
-	  We don't have a compatability function to fill in a missing htobe64; but
-	  we already have one for the identical htonll.
+	  Merged revisions 426830 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Change-Id: Ic0a95db1c5b0041e14e6b127432fb533b97e4cac
+	  Merged revisions 426831 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-08-07 14:20 +0000 [df9ce36366]  Scott Emidy <jemidy@digium.com>
+	  Merged revisions 426832 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* ARI: Retrieve existing log channels
+	  Merged revisions 426833 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  An http request can be sent to get the existing Asterisk logs.
 
-	  The command "curl -v -u user:pass -X GET 'http://localhost:8088
-	  /ari/asterisk/logging'" can be run in the terminal to access the
-	  newly implemented functionality.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426834 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Retrieve all existing log channels
+2014-10-30 18:56 +0000 [e4374a3abe]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25252
+	* app_queue: fix a couple leaks to struct call_queue in set_member_value
 
-	  Change-Id: I7bb08b93e3b938c991f3f56cc5d188654768a808
+	  set_member_value has a couple leaks to references in the variable q
+	  found through testsuite tests/queues/set_penalty.  Also remove the
+	  REF_DEBUG_ONLY_QUEUES compiler declaration, this is no longer possible
+	  with the updated REF_DEBUG code.
 
-2015-08-07 11:14 +0000 [e9f1bc08cb]  Scott Emidy <jemidy@digium.com>
+	  ASTERISK-24466 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4125/
+	  ........
 
-	* ARI: Creating log channels
+	  Merged revisions 426805 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  An http request can be sent to create a log channel
-	  in Asterisk.
+	  Merged revisions 426806 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The command "curl -v -u user:pass -X POST
-	  'http://localhost:088/ari/asterisk/logging/mylog?
-	  configuration=notice,warning'" can be run in the terminal
-	  to access the newly implemented functionality for ARI.
+	  Merged revisions 426807 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Ability to create log channels using ARI
 
-	  ASTERISK-25252
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426808 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I9a20e5c75716dfbb6b62fd3474faf55be20bd782
+2014-10-30 18:45 +0000 [ced81afff2]  Corey Farrell <git@cfware.com>
 
-2015-08-06 15:18 +0000 [78364132ce]  Scott Emidy <jemidy@digium.com>
+	* audiohooks: Clean references to formats
 
-	* ARI: Deleting log channels
+	  Cleanup references to in_translate[x].format and
+	  out_translate[x].format in ast_audiohook_detach_list.
 
-	  An http request can be sent to delete a log channel
-	  in Asterisk.
+	  ASTERISK-24465 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4124/
+	  ........
 
-	  The command "curl -v -u user:pass -X DELETE 'http://localhost:8088
-	  /ari/asterisk/logging/mylog'" can be run in the terminal
-	  to access the newly implemented functionally for ARI.
+	  Merged revisions 426803 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Able to delete log channels using ARI
 
-	  ASTERISK-25252
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426804 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Id6eeb54ebcc511595f0418d586ff55914bc3aae6
+2014-10-30 16:14 +0000 [a537e314d1]  Kevin Harwell <kharwell@digium.com>
 
-2015-08-06 12:48 +0000 [e25569ef95]  Mark Michelson <mmichelson@digium.com>
+	* res_pjsip_exten_state: PJSIPShowSubscriptionsInbound causes crash
 
-	* res_pjsip_pubsub: More accurately persist packet.
+	  Currently, it is possible for some subscriptions to get into a NULL state. When
+	  this occurs and the PJSIPShowSubscriptionsInbound ami action is issued and a
+	  device is subscribed for extension state then the associated subscription state
+	  object can't be located.  The code then attempts to dereference a NULL object.
+	  Added a NULL check to avoid the problem.
 
-	  The pjsip_rx_data structure has a pkt_info.packet field on it that is
-	  the packet that was read from the transport. For datagram transports,
-	  the packet read from the transport will correspond to the SIP message
-	  that arrived. For streamed transports, however, it is possible to read
-	  multiple SIP messages in one packet.
+	  Reported by: John Bigelow
+	  ........
 
-	  In a recent case, Asterisk crashed on a system where TCP was being used.
-	  This is because at some point, a read from the TCP socket resulted in a
-	  200 OK response as well as an incoming SUBSCRIBE request being stored in
-	  rdata->pkt_info.packet. When the SUBSCRIBE was processed, the
-	  combination 200 OK and SUBSCRIBE was saved in persistent storage. Later,
-	  a restart of Asterisk resulted in the crash because the persistent
-	  subscription recreation code ended up building the 200 OK response
-	  instead of a SUBSCRIBE request, and we attempted to access
-	  request-specific data.
+	  Merged revisions 426779 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The fix here is to use the pjsip_msg_print() function in order to
-	  persist SUBSCRIBE requests. This way, rather than using the raw socket
-	  data, we use the parsed SIP message that PJSIP has given us. If we
-	  receive multiple SIP messages from a single read, we will be sure only
-	  to save off the relevant SIP message. There also is a safeguard put in
-	  place to make sure that if we do end up reconstructing a SIP response,
-	  it will not cause a crash.
+	  Merged revisions 426780 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25306 #close
-	  Reported by Mark Michelson
 
-	  Change-Id: I4bf16f7b76a2541d10b55de82bcd14c6e542afb2
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426781 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-04 16:12 +0000 [8521a86367]  Joshua Colp <jcolp@digium.com>
+2014-10-30 12:18 +0000 [cd52456ea1]  Kevin Harwell <kharwell@digium.com>
 
-	* res_pjsip: Ensure sanitized XML is NULL terminated.
+	* res_pjsip: incorrect qualify statistics after disabling for contact
 
-	  The ast_sip_sanitize_xml function is used to sanitize
-	  a string for placement into XML. This is done by examining
-	  an input string and then appending values to an output
-	  buffer. The function used by its implementation, strncat,
-	  has specific behavior that was not taken into account.
-	  If the size of the input string exceeded the available
-	  output buffer size it was possible for the sanitization
-	  function to write past the output buffer itself causing
-	  a crash. The crash would either occur because it was
-	  writing into memory it shouldn't be or because the resulting
-	  string was not NULL terminated.
+	  When removing the qualify_frequency from an AoR or a contact the statistics
+	  shown when issuing "pjsip show aors" from the CLI are incorrect. This patch
+	  deletes the contact's status object from sorcery, disassociating it from the
+	  contact, if the qualify_freqency is removed from configuration.
 
-	  This change keeps count of how much remaining space is
-	  available in the output buffer for text and only allows
-	  strncat to use that amount.
+	  ASTERISK-24462 #close
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4116/
+	  ........
 
-	  Since this was exposed by the res_pjsip_pidf_digium_body_supplement
-	  module attempting to send a large message the maximum allowed
-	  message size has also been increased in it.
+	  Merged revisions 426755 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  A unit test has also been added which confirms that the
-	  ast_sip_sanitize_xml function is providing NULL terminated
-	  output even when the input length exceeds the output
-	  buffer size.
+	  Merged revisions 426757 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25304 #close
 
-	  Change-Id: I743dd9809d3e13d722df1b0509dfe34621398302
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426761 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-08-05 05:23 +0000 [9a12804e59]  Joshua Colp <jcolp@digium.com>
+2014-10-30 04:21 +0000 [5d8d90c402]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* res_rtp_asterisk: Don't leak temporary key when enabling PFS.
+	* app_voicemail: Fix unchecked bounds of myArray in IMAP_STORAGE.
 
-	  A change recently went in which enabled perfect forward secrecy for
-	  DTLS in res_rtp_asterisk. This was accomplished two different ways
-	  depending on the availability of a feature in OpenSSL. The fallback
-	  method created a temporary instance of a key but did not free it.
-	  This change fixes that.
+	  In update_messages_by_imapuser(), messages were appended to a finite
+	  array which resulted in a crash when an IMAP mailbox contained more
+	  than 256 entries. This memory is now dynamically increased as needed.
 
-	  ASTERISK-25265
+	  Observe that this patch adds a bunch of XXX's to questionable code. See
+	  the review (url below) for more information.
 
-	  Change-Id: Iadc031b67a91410bbefb17ffb4218d615d051396
-2015-08-04 09:47 +0000 [27dc2094e9]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24190 #close
+	  Reported by: Nick Adams
+	  Tested by: Nick Adams
 
-	* res_http_websocket: Debug write lengths.
+	  Review: https://reviewboard.asterisk.org/r/4126/
+	  ........
 
-	  Commit 39cc28f6ea2140ad6d561fd4c9e9a66f065cecee attempted to fix a
-	  test failure observed on 32 bit test agents by ensuring that a cast from
-	  a 32 bit unsigned integer to a 64 bit unsigned integer was happening in
-	  a predictable place. As it turns out, this did not cause test runs to
-	  succeed.
+	  Merged revisions 426691 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  This commit adds several redundant debug messages that print the payload
-	  lengths of websocket frames. The idea here is that this commit will not
-	  cause tests to succeed for the faulty test agent, but we might deduce
-	  where the fault lies more easily this way by observing at what point the
-	  expected value (537) changes to some ungangly huge number.
+	  Merged revisions 426692 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  If you are wondering why something like this is being committed to the
-	  branch, keep in mind that in commit
-	  39cc28f6ea2140ad6d561fd4c9e9a66f065cecee I noted that the observed test
-	  failures only happen when automated tests are run. Attempts to run the
-	  tests by hand manually on the test agent result in the tests passing.
+	  Merged revisions 426696 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I14a65c19d8af40dadcdbd52348de3b0016e1ae8d
+	  Merged revisions 426702 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-08-03 11:06 +0000 [39cc28f6ea]  Mark Michelson <mmichelson@digium.com>
 
-	* res_http_websocket: Avoid passing strlen() to ast_websocket_write().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426706 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  We have seen a rash of test failures on a 32-bit build agent. Commit
-	  48698a5e21d7307f61b5fb2bd39fd593bc1423ca solved an obvious problem where
-	  we were not encoding a 64-bit value correctly over the wire. This
-	  commit, however, did not solve the test failures.
+2014-10-30 01:15 +0000 [c866ced76b]  Igor Goncharovskiy <igor.goncharovsky@gmail.com>
 
-	  In the failing tests, ARI is attempting to send a 537 byte text frame
-	  over a websocket. When sending a frame this small, 16 bits are all that
-	  is required in order to encode the payload length on the websocket
-	  frame. However, ast_websocket_write() thinks that the payload length is
-	  greater than 65535 and therefore writes out a 64 bit payload length.
-	  Inspecting this payload length, the lower 32 bits are exactly what we
-	  would expect it to be, 537 in hex. The upper 32 bits, are junk values
-	  that are not expected to be there.
+	* 
+	  Add additional checks for NULL pointers to fix several crashes reported.
 
-	  In the failure, we are passing the result of strlen() to a function that
-	  expects a uint64_t parameter to be passed in. strlen() returns a size_t,
-	  which on this 32-bit machine is 32 bits wide. Normally, passing a 32-bit
-	  unsigned value to somewhere where a 64-bit unsigned value is expected
-	  would cause no problems. In fact, in manual runs of failing tests, this
-	  works just fine. However, ast_websocket_write() uses the Asterisk
-	  optional API, which means that rather than a simple function call, there
-	  are a series of macros that are used for its declaration and
-	  implementation. These macros may be causing some sort of error to occur
-	  when converting from a 32 bit quantity to a 64 bit quantity.
+	  ASTERISK-24304 #close
+	  Reported by: dhanapathy sathya
+	  ........
 
-	  This commit changes the logic by making existing ast_websocket_write()
-	  calls use ast_websocket_write_string() instead. Within
-	  ast_websocket_write_string(), the 64-bit converted strlen is saved in a
-	  local variable, and that variable is passed to ast_websocket_write()
-	  instead.
+	  Merged revisions 426666 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Note that this commit message is full of speculation rather than
-	  certainty. This is because the observed test failures, while always
-	  present in automated test runs, never occur when tests are manually
-	  attempted on the same test agent. The idea behind this commit is to fix
-	  a theoretical issue by performing changes that should, at the least,
-	  cause no harm. If it turns out that this change does not fix the failing
-	  tests, then this commit should be reverted.
+	  Merged revisions 426667 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I4458dd87d785ca322b89c152b223a540a3d23e67
+	  Merged revisions 426668 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-28 05:33 +0000 [aed068844c]  Mark Duncan <mark@syon.co.jp>
 
-	* res/res_rtp_asterisk: Add ECDH support
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426669 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This will add ECDH support to Asterisk. It will
-	  detect auto ECDH support in OpenSSL
-	  (1.0.2b and above) during ./configure. If this is
-	  available, it will use it,
-	  otherwise it will fall back to prime256v1 (this
-	  behavior is consistent with
-	  other projects such as Apache and nginx).
+2014-10-29 20:59 +0000 [0ddc3bde24]  Olle Johansson <oej@edvina.net> (License 5267)
 
-	  This fixes WebRTC being broken in Firefox 38+ due
-	  to Firefox now only supporting
-	  ciphers with perfect forward secrecy.
+	* channels/chan_sip: Add improved support for 4xx error codes
 
-	  ASTERISK-25265 #close
+	  This patch adds support for 414, 493, 479, and a stray 400 response in REGISTER
+	  response handling. This helps interoperability in a number of scenarios.
 
-	  Change-Id: I8c13b33a2a79c0bde2e69e4ba6afa5ab9351465b
+	  Review: https://reviewboard.asterisk.org/r/3437
 
-2015-07-29 14:17 +0000 [1ae762634c]  Benjamin Ford <bford@digium.com>
+	  patches:
+	    rb3437.patch uploaded by oej (License 5267)
+	  ........
 
-	* ARI: Rotate log channels.
+	  Merged revisions 426599 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  An http request can be sent to rotate a specified log channel.
-	  If the channel does not exist, an error response will be
-	  returned.
+	  Merged revisions 426600 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The command "curl -v -u user:pass -X PUT 'http://localhost:8088
-	  /ari/asterisk/logging/logChannelName/rotate'" can be run in the
-	  terminal to access this new functionality.
+	  Merged revisions 426601 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Added the ability to rotate log files through ARI
+	  Merged revisions 426602 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25252
 
-	  Change-Id: Iaefa21cbbc1b29effb33004ee3d89c977e76ab01
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426603 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-29 13:49 +0000 [aeeb170fc4]  Richard Mudgett <rmudgett@digium.com>
+2014-10-29 20:48 +0000 [ff83ff564c]  Olle Johansson <oej@edvina.net> (License 5267)
 
-	* rtp_engine.c: Fix performance issue with several channel drivers that use RTP.
+	* channels/chan_sip: Support mutltiple Supported and Required headers
 
-	  ast_rtp_codecs_get_payload() gets called once or twice for every received
-	  RTP frame so it would be nice to not allocate an ao2 object to then have
-	  it destroyed shortly thereafter.  The ao2 object gets allocated only if
-	  the payload type is not set by the channel driver as a negotiated value.
-	  The issue affects chan_skinny, chan_unistim, chan_rtp, and chan_ooh323.
+	  A SIP request may contain multiple Supported: and Required: headers. Currently,
+	  chan_sip only parses the first Supported/Required header it finds. This patch
+	  adds support for multiple Supported/Required headers for INVITE requests.
 
-	  * Made static_RTP_PT[] an array of ao2 objects that
-	  ast_rtp_codecs_get_payload() can return instead of an array of structs
-	  that must be copied into a created ao2 object.
+	  Review: https://reviewboard.asterisk.org/r/2478
 
-	  ASTERISK-25296 #close
-	  Reported by: Richard Mudgett
+	  ASTERISK-21721 #close
+	  Reported by: Olle Johansson
+	  patches:
+	    rb2478.patch uploaded by oej (License 5267)
+	  ........
 
-	  Change-Id: Icb6de5cd90bfae07d44403a1352963db9109dac0
+	  Merged revisions 426594 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-07-29 17:00 +0000 [84262749d2]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 426595 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_rtp_asterisk.c: Fix off-nominal crash potential.
+	  Merged revisions 426596 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25296
-	  Reported by: Richard Mudgett
+	  Merged revisions 426597 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I08549fb7c3ab40a559f41a3940f3732a4059b55b
 
-2015-07-29 13:48 +0000 [1519eb44a7]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426598 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* rtp_engine.c: Must protect mime_types_len with mime_types_lock.
+2014-10-29 08:02 +0000 [8a69aedd17]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
 
-	  Change-Id: I44220dd369cc151ebf5281d5119d84bb9e54d54e
+	* Fix building chan_phone on big endian systems
 
-2015-07-24 18:42 +0000 [a93b7a927c]  Richard Mudgett <rmudgett@digium.com>
+	  A left over from the formats conversion (Corey Farrell).
 
-	* res_pjsip_sdp_rtp.c: Fix processing wrong SDP media list.
+	  ASTERISK-24458 #close
+	  Review: https://reviewboard.asterisk.org/r/4117/
 
-	  Change-Id: I7c076826c2d3c6ae8c923ca73b7a71980cca11f2
+	  ........
 
-2015-07-24 18:38 +0000 [741fa0d26d]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 426570 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_sdp_rtp.c: Fixup some whitespace.
 
-	  Change-Id: Ib4eb7ef7dcaf93ddc26538f0a498aaf110d7a973
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426573 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-27 19:10 +0000 [89b21fd9a3]  Richard Mudgett <rmudgett@digium.com>
+2014-10-28 16:35 +0000 [0ed8aebda9]  Richard Mudgett <rmudgett@digium.com>
 
-	* rtp_engine.h: No sense allowing payload types larger than RFC allows.
+	* bridge_builtin_features: Add missing channel locks around ast_get_chan_features_general_config().
 
-	  * Tweaked add_static_payload() to not use magic numbers.
+	  The feature_automonitor() and feature_automixmonitor() functions were not
+	  locking the channel around ast_get_chan_features_general_config().
+	  Accessing the channel datastore list without the channel locked is a good
+	  way to corrupt the list or follow the pointer chain into oblivion.
+	  ........
 
-	  Change-Id: I1719ff0f6d3ce537a91572501eae5bcd912a420b
+	  Merged revisions 426531 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-07-23 14:04 +0000 [7427c7f13b]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 426552 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* rtp_engine.c: Minor tweaks.
 
-	  * Fix off nominial ref leak of new_type in
-	  ast_rtp_codecs_payloads_set_m_type().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426553 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * No need to lock static_RTP_PT_lock in
-	  ast_rtp_codecs_payloads_set_m_type() and
-	  ast_rtp_codecs_payloads_set_rtpmap_type_rate() before the payload type
-	  parameter sanity check.
+2014-10-28 16:10 +0000 [7205d76d7d]  Corey Farrell <git@cfware.com>
 
-	  * No need to create ast_rtp_payload_type ao2 objects with a lock since the
-	  lock is not used.
+	* res_fax: Resolve T38 gateway frame leak.
 
-	  Change-Id: I64dd1bb4dfabdc7e981e3f61448beac9bb7504d4
+	  When frames are translated by a fax gateway they need to be freed.  The
+	  existing call to ast_frfree was unreachable.  This change reorganizes
+	  fax_gateway_framehook to ensure that ast_frfree is called when needed.
 
-2015-07-23 12:41 +0000 [e20f435b60]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24457 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4115/
+	  ........
 
-	* rtp_engine.h: Misc comment fixes.
+	  Merged revisions 426527 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: If98139264d5d97427b4685ecbdc54518f725bc43
+	  Merged revisions 426528 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-07-17 16:23 +0000 [bc5d7f9c37]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 426529 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_sip.c: Tweak glue->update_peer() parameter nil value.
 
-	  Change glue->update_peer() parameter from 0 to NULL to better indicate it
-	  is a pointer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426530 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I8ff2e5087f0e19f6998e3488a712a2470cc823bd
+2014-10-28 15:44 +0000 [67e496c275]  Corey Farrell <git@cfware.com>
 
-2015-07-30 17:05 +0000 [13eb491e35]  Richard Mudgett <rmudgett@digium.com>
+	* manager: Unsubscribe from acl_change_sub at shutdown.
 
-	* res_pjsip_session.c: Fix crashes seen when call cancelled.
+	  ASTERISK-24453 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4110/
+	  ........
 
-	  Two testsuite tests crashed in the same place as a result of an INVITE
-	  being CANCELed.
+	  Merged revisions 426524 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  tests/channels/pjsip/resolver/srv/failover/in_dialog/transport_unspecified
-	  tests/channels/pjsip/resolver/srv/failover/in_dialog/transport_tcp
+	  Merged revisions 426525 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The session pointer is no longer in the inv->mod_data[session_module.id]
-	  location because the INVITE transaction has reached the terminated state.
 
-	  ASTERISK-25297 #close
-	  Reported by: Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426526 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Idb75fdca0321f5447d5dac737a632a5f03614427
+2014-10-28 13:09 +0000 [1fe22c411d]  Malcolm Davenport <malcolmd@digium.com>
 
-2015-07-29 14:35 +0000 [48698a5e21]  Mark Michelson <mmichelson@digium.com>
+	* ASTERISK-23512, correct inaccurate comment in manager.conf.sample
 
-	* res_http_websocket: Properly encode 64 bit payload
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426462 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A test agent was continuously failing all ARI tests when run against
-	  Asterisk 13. As it turns out, the reason for this is that on those test
-	  runs, for some reason we decided to use the super extended 64 bit
-	  payload length for websocket text frames instead of the extended 16 bit
-	  payload length. For 64-bit payloads, the expected byte order over the
-	  network is
+2014-10-28 11:41 +0000 [8e9f593e3a]  Matt Jordan <mjordan@digium.com>
+
+	* main/bridge: Destroy features struct on off nominal path during bridge impart
+
+	  When a channel is imparted to a bridge, the invocation of the function may
+	  provide an ast_bridge_features struct. Upon passing this to ast_bridge_impart,
+	  the caller must assume that ownership has passed to the function, as in all
+	  paths the function destroys the struct prior to returning (as its purpose is
+	  to configure the behavior of the channel while in the bridge). On one off
+	  nominal path - where the channel already has a PBX thread - the struct was not
+	  being destroyed.
 
-	  7, 6, 5, 4, 3, 2, 1, 0
+	  This patch fixes that glitch.
 
-	  However, we were sending the payload as
+	  ASTERISK-24437 #close
+	  Reported by: Scott Griepentrog
+	  ........
 
-	  3, 2, 1, 0, 7, 6, 5, 4
+	  Merged revisions 426431 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This meant that we were saying to expect an absolutely MASSIVE payload
-	  to arrive. Since we did not follow through on this expected payload
-	  size, the client would sit patiently waiting for the rest of the payload
-	  to arrive until the test would time out.
+	  Merged revisions 426432 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  With this change, we use the htobe64() function instead of htonl() so
-	  that a 64-bit byte-swap is performed instead of a 32 bit byte-swap.
 
-	  Change-Id: Ibcd8552392845fbcdd017a8c8c1043b7fe35964a
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426433 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-29 12:23 +0000 [10ba72a927]  Mark Michelson <mmichelson@digium.com>
+2014-10-28 09:59 +0000 [f4b4d42630]  Matt Jordan <mjordan@digium.com>
 
-	* Add a test event for inband ringing.
+	* main/manager: Fix typo in AMI event documentation of "OriginateResponse"
 
-	  This event is necessary for the bridge_wait_e_options test to be able to
-	  confirm that ringing is being played on the local channel that runs the
-	  BridgeWait() application with the e(r) option.
+	  The parameter name is "Response", not "Resonse".
 
-	  ASTERISK-25292 #close
-	  Reported by Kevin Harwell
+	  ASTERISK-24430 #close
+	  Reported by: Dafi Ni
+	  ........
 
-	  Change-Id: Ifd3d3d2bebc73344d4b5310d0d55c7675359d72e
+	  Merged revisions 426366 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-07-16 12:16 +0000 [8458b8d441]  Jonathan Rose <jrose@digium.com>
+	  Merged revisions 426367 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* holding_bridge: ensure moh participants get frames
+	  Merged revisions 426368 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Currently, if a blank musiconhold.conf is used, musiconhold will fail
-	  to start for a channel going into a holding bridge with an anticipation
-	  of getting music on hold. That being the case, no frames will be written
-	  to the channel and that can pose a problem for blind transfers in PJSIP
-	  which may rely on frames being written to get past the REFER framehook.
-	  This patch makes holding bridges start a silence generator if starting
-	  music on hold fails and makes it so that if no music on hold functions
-	  are installed that the ast_moh_start function will report a failure so
-	  that consumers of that function will be able to respond appropriately.
 
-	  ASTERISK-25271 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426369 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I06f066728604943cba0bb0b39fa7cf658a21cd99
+2014-10-28 09:57 +0000 [68d9872f58]  Malcolm Davenport <malcolmd@digium.com>
 
-2015-07-24 22:20 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	* ASTERISK-24323, fix bug in documentation of AGI STREAM FILE CONTROL
 
-	* asterisk 13.5.0-rc1 Released.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426365 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-24 17:15 +0000 [a4b527393b]  Matt Jordan <mjordan@digium.com>
+2014-10-28 08:13 +0000 [684b8762a9]  Malcolm Davenport <malcolmd@digium.com>
 
-	* Release summaries: Add summaries for 13.5.0-rc1
+	* ASTERISK-24419, fix incorrect syntax for setting language in extensions.conf.sample
 
-2015-07-24 17:11 +0000 [158b0b8ebf]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426297 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* .version: Update for 13.5.0-rc1
+2014-10-28 06:22 +0000 [2290393273]  Corey Farrell <git@cfware.com>
 
-2015-07-24 17:11 +0000 [a0a7650e34]  Matt Jordan <mjordan@digium.com>
+	* app_queue: Cleanup ao2_iterator
 
-	* .lastclean: Update for 13.5.0-rc1
+	  Clean ao2_iterator, resolving reference leak to queue members.
 
-2015-07-24 17:11 +0000 [4d238af086]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24454 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4111/
+	  ........
 
-	* realtime: Add database scripts for 13.5.0-rc1
+	  Merged revisions 426255 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-07-24 12:56 +0000 [f78a4b52b8]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 426260 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Bump the ARI version to 1.8.0
+	  Merged revisions 426266 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Due to backwards compatible changes, the ARI version should be bumped to
-	  1.8.0 prior to the release of 13.5.0. Note that a previous patch already
-	  bumped the version of AMI for this release.
 
-	  Change-Id: I419033bfbbc0d3533a29ccb32b2981f39e0883e7
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426272 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-18 11:16 +0000 [2749721791]  Joshua Colp <jcolp@digium.com>
+2014-10-28 06:12 +0000 [ab16f46139]  Corey Farrell <git@cfware.com>
 
-	* pjsip: Add rtp_timeout and rtp_timeout_hold endpoint options.
+	* func_cdr: Fix CDR_PROP payload leak
 
-	  This change adds support for the 'rtp_timeout' and 'rtp_timeout_hold'
-	  endpoint options. These allow the channel to be hung up if RTP
-	  is not received from the remote endpoint for a specified number of
-	  seconds.
+	  Remove duplicate allocation of payload, preventing leak.
 
-	  ASTERISK-25259 #close
+	  ASTERISK-24455 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4113/
+	  ........
 
-	  Change-Id: I3f39daaa7da2596b5022737b77799d16204175b9
+	  Merged revisions 426252 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-24 09:46 +0000 [b4e19e414a]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip: Add rtp_keepalive to sample config file.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426253 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I5f62d0c5684f8b2335f9f8ac2d79ee04fbdafb19
+2014-10-27 12:55 +0000 [ef8cdd40e5]  Sean Bright <sean@malleable.com>
 
-2015-07-23 13:11 +0000 [f635520527]  Mark Michelson <mmichelson@digium.com>
+	* configure: Add autoconf check for libopus.
 
-	* Local channels: Alternate solution to ringback problem.
+	  Because opus transcoding support cannot be included in the standard Asterisk
+	  distribution, a few codec_opus implementations have popped up.  To make it
+	  easier for people to drop in opus support in their own installations, this
+	  patch adds configure checks for libopus.
 
-	  Commit 54b25c80c8387aea9eb20f9f4f077486cbdf3e5d solved an issue where a
-	  specific scenario involving local channels and a native local RTP bridge
-	  could result in ringback still being heard on a calling channel even
-	  after the call is bridged.
+	  Review: https://reviewboard.asterisk.org/r/4106/
+	  ........
 
-	  That commit caused many tests in the testsuite to fail with alarming
-	  consequences, such as not sending DialBegin and DialEnd events, and
-	  giving incorrect hangup causes during calls.
+	  Merged revisions 426234 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This commit reverts the previous commit and implements and alternate
-	  solution. This new solution involves only passing AST_CONTROL_RINGING
-	  frames across local channels if the local channel is in AST_STATE_RING.
-	  Otherwise, the frame does not traverse the local channels. By doing
-	  this, we can ensure that a playtones generator does not get started on
-	  the calling channel but rather is started on the local channel on which
-	  the ringing frame was initially indicated.
 
-	  ASTERISK-25250 #close
-	  Reported by Etienne Lessard
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426235 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I3bc87a18a38eb2b68064f732d098edceb5c19f39
+2014-10-26 21:47 +0000 [5a17878085]  Matt Jordan <mjordan@digium.com>
 
-2015-07-22 12:24 +0000 [f509730cb9]  Joshua Colp <jcolp@digium.com>
+	* res/res_http_websocket: Fix minor nits found by wdoekes on r409681
 
-	* audiohook: Use manipulated frame instead of dropping it.
+	  When Moises committed the fixes for WSS (which was a great patch), wdoekes had
+	  a few style nits that were on the review that got missed. This patch resolves
+	  what I *think* were all of the ones that were still on the review.
 
-	  Previous changes to sample rate support in audiohooks accidentally
-	  removed code responsible for allowing the manipulate audiohooks
-	  to work. Without this code the manipulated frame would be dropped
-	  and not used. This change restores it.
+	  Thanks to both moy for the patch, and wdoekes for the reviews.
 
-	  ASTERISK-25253 #close
+	  Review: https://reviewboard.asterisk.org/r/3248/
+	  ........
 
-	  Change-Id: I3ff50664cd82faac8941f976fcdcb3918a50fe13
+	  Merged revisions 426209 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-07-22 09:46 +0000 [54b25c80c8]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 426210 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Local channels: Do not block control -1 payloads.
+	  Merged revisions 426211 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Control frames with a -1 payload are used as a special signal to stop
-	  playtones generators on channels. This indication is sent both by
-	  app_dial as well as by ast_answer() when a call is answered in case any
-	  tones were being generated on a calling channel.
 
-	  This control frame type was made to stop traversing local channel pairs
-	  as an optimization, because it was thought that it was unnecessary to
-	  send these indications, and allowing such unnecessary control frames to
-	  traverse the local channels would cause the local channels to optimize
-	  away less quickly.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426212 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  As it turns out, through some special magic dialplan code, it is
-	  possible to have a tones being played on a non-local channel, and it is
-	  important for the local channel to convey that the tones should be
-	  stopped. The result of having tones continue to be played on the
-	  non-local channel is that the tones play even once the channel has been
-	  bridged. By not blocking the -1 control frame type, we can ensure that
-	  this situation does not happen.
+2014-10-26 21:27 +0000 [62bee9b327]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25250 #close
-	  Reported by Etienne Lessard
+	* res/res_phoneprov: Fix crash on shutdown caused by container cleanup
 
-	  Change-Id: I0bcaac3d70b619afdbd0ca8a8dd708f33fd2f815
+	  In res_phoneprov, unloading the module first destroys the http_routes
+	  container, followed by the users. However, users may have a route in
+	  the http_routes container; the validity of this container is not checked
+	  in the users destructor. Hence, we hit an assert as the container has already
+	  been set to NULL.
 
-2015-07-22 05:16 +0000 [f1493f900e]  Joshua Colp <jcolp@digium.com>
+	  This patch does two things:
+	  (1) It adds a sanity check in the user destructor (because why not)
+	  (2) It switches the order of destruction, so that users are disposed of prior
+	      to the HTTP routes they may hold a reference to.
 
-	* audiohook: Read the correct number of samples based on audiohook format.
+	  Note that this crash was caught by the Test Suite (go go testing!)
+	  ........
 
-	  Due to changes in audiohooks to support different sample rates the
-	  underlying storage of samples is in the format of the audiohook
-	  itself and not of the format being requested. This means that if a
-	  channel is using G722 the samples stored will be at 16kHz. If
-	  something subsequently reads from the audiohook at a format which
-	  is not the same sample rate as the audiohook the number of samples
-	  needs to be adjusted.
+	  Merged revisions 426174 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Given the following example:
-	  1. Channel writing into audiohook at 16kHz (as it is using G722).
-	  2. Chanspy reading from audiohook at 8kHz.
+	  Merged revisions 426176 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The original code would read 160 samples from the audiohook for
-	  each 20ms of audio. This is incorrect. Since the audio in the
-	  audiohook is at 16kHz the actual number needing to be read is 320.
-	  Failure to read this much would cause the audiohook to reset
-	  itself constantly as the buffer became full.
 
-	  This change adjusts the requested number of samples by determining
-	  the duration of audio requested and then calculating how many
-	  samples that would be in the audiohook format.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426179 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25247 #close
+2014-10-26 20:47 +0000 [130a3fcd7f]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ia91ce516121882387a315fd8ee116b118b90653d
+	* res/res_srtp: Fix include issue for libsrtp 1.5.0
 
-2015-07-20 12:39 +0000 [62c64c3bd1]  Rusty Newton <rnewton@digium.com>
+	  In libsrtp 1.5.0, crypto_get_random is no longer resolved simply by including
+	  srtp.h. Now, one must include crypto_kernel.h as well. As it turns out, this
+	  header file has been provided by the library since 2006, so this is a
+	  relatively benign change.
 
-	* Documentation: A couple of trivial fixes in sip.conf.sample and func_cdr.c
+	  ASTERISK-24436 #close
+	  Reported by: Patrick Laimbock
+	  ........
 
-	   * In sip.conf.sample fix sentence where we said that WS or WSS are supported
-	     transports for use in an outbound register definition. They are not
-	     supported in that case.
-	   * In func_cdr.c made it clear that the Disable option for CDR_PROP can be used
-	     to enable CDR on a channel.
+	  Merged revisions 426140 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  ASTERISK-24867 #close
-	  Reported by: Rusty Newton
+	  Merged revisions 426141 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-24853 #close
-	  Reported by: PSDK
+	  Merged revisions 426142 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I3d698bc6302b9d00a0a995b5c4ad9a42d69b48ca
+	  Merged revisions 426143 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-09 14:17 +0000 [d9094ddd73]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip: Add rtp_keepalive endpoint option.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426144 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This adds an "rtp_keepalive" option for PJSIP endpoints. Similar to the
-	  chan_sip option, this specifies an interval, in seconds, at which we
-	  will send RTP comfort noise frames. This can be useful for keeping RTP
-	  sessions alive as well as keeping NAT associations alive during lulls.
+2014-10-24 10:32 +0000 [c084728690]  Jonathan Rose <jrose@digium.com>
 
-	  ASTERISK-25242 #close
-	  Reported by Mark Michelson
+	* Documentation: Improve documentation for ExtensionStatus AMI events
 
-	  Change-Id: I06660ba672c0a343814af4cec838e6025cafd54b
+	  Review: https://reviewboard.asterisk.org/r/4085/
+	  ........
 
-2015-07-16 09:13 +0000 [a23adcca3d]  Michael Cargile <mikec@vicidial.com>
+	  Merged revisions 426120 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_musiconhold: Add a warning when MOH does not exist
 
-	  Change-Id: Ifdfbd0b97cf31478d29923ec30aabce28d01740b
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426121 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-19 09:11 +0000 [03064daeb2]  Matt Jordan <mjordan@digium.com>
+2014-10-22 16:41 +0000 [c4d7e7e270]  Shaun Ruffell <sruffell@digium.com>
 
-	* res/res_sorcery_config: Prevent crash from misconfigured sorcery.conf
+	* codec_dahdi: Cannot use struct ast_translator.core_{src,src}_codec.
 
-	  Misconfiguring sorcery.conf with a 'config' wizard with no extra data
-	  will currently crash Asterisk on startup, as the wizard requires a comma
-	  delineated list to parse. This patch updates res_sorcery_config to check
-	  for the presence of the data before it starts manipulating it.
+	  This fixes a Segmentation fault introduced in r419044 "media formats: re-architect
+	  handling of media for performance improvements".
 
-	  Change-Id: I4c97512e8258bc82abe190627a9206c28f5d3847
+	  The problem is that codec_dahdi was using core_src_codec and core_dst_codec in the
+	  ast_translator structure when these fields were never set. Now instead of trying to map
+	  the new core codec descriptions to the way DAHDI defines different codecs, we will store
+	  the DAHDI specific formats in 'struct translator' directly so we can refer to them without
+	  mapping.
 
-2015-07-16 09:46 +0000 [2c626ceb64]  Joshua Colp <jcolp@digium.com>
+	  This also allows us to remove the "global_format_map" structure, since we can now query
+	  the list of translators directly to make sure we do not ever register a DAHDI based
+	  translator for a specific path more than once and eliminate the need to keep the list and
+	  the map in sync.
 
-	* chan_pjsip: Don't change formats when frame of unsupported format is received.
+	  ASTERISK-24435 #close
+	  Reported by: Marian Koniuszko
 
-	  Receipt of an RTP packet currently causes the formats on an PJSIP channel to
-	  change to the format of the RTP packet. In some off-nominal cases it's possible
-	  for this to be a format that has not been configured or negotiated. This change
-	  makes it so only formats explicitly configured on the endpoint are allowed.
+	  Review: https://reviewboard.asterisk.org/r/4105/
+	  ........
 
-	  ASTERISK-25258 #close
+	  Merged revisions 426097 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: If93d641fb6418a285928839300d7854cab8c1020
 
-2015-07-17 04:59 +0000 [abb14ac5b8]  Patric Marschall <patric.marschall@1und1.de>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426099 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* sig_pri.h: force_restart_unavailable_chans in wrong scope
+2014-10-21 13:04 +0000 [2165868be7]  Richard Mudgett <rmudgett@digium.com>
 
-	  In channels/sig_pri.h, struct sig_pri_span, the field
-	  force_restart_unavailable_chans is only defined if
+	* translage.c: Fix regression when generating translation path strings.
 
-	  #if defined(HAVE_PRI_MCID) is true.
+	  Fix the AMI Status action read and write translation path strings from
+	  growing for each channel in the status event list by reseting the ast
+	  string given to ast_translate_path_to_str() to fill in the given
+	  translation path.
+	  ........
 
-	  All other occurences of force_restart_unavailable_chans are outside of the
+	  Merged revisions 426079 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  #if defined(HAVE_PRI_MCID)
-	  endif
 
-	  scope.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426080 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25257 #close
-	  Reported by: Patric Marschall
+2014-10-20 09:20 +0000 [dad0334cf1]  abelbeck <lonnie@abelbeck.com> (License 5903),Matt Jordan <mjordan@digium.com> (License 6283)
 
-	  Change-Id: I071de89cc2cd0d85927a013036e235851f672549
-2015-07-14 16:55 +0000 [875aee4c09]  Richard Mudgett <rmudgett@digium.com>
+	* AST-2014-011: Fix POODLE security issues
 
-	* pbx.c: Post AMI VarSet event if delete a non-empty dialplan variable.
+	  There are two aspects to the vulnerability:
+	  (1) res_jabber/res_xmpp use SSLv3 only. This patch updates the module to use
+	      TLSv1+. At this time, it does not refactor res_jabber/res_xmpp to use the
+	      TCP/TLS core, which should be done as an improvement at a latter date.
+	  (2) The TCP/TLS core, when tlsclientmethod/sslclientmethod is left unspecified,
+	      will default to the OpenSSL SSLv23_method. This method allows for all
+	      ecnryption methods, including SSLv2/SSLv3. A MITM can exploit this by
+	      forcing a fallback to SSLv3, which leaves the server vulnerable to POODLE.
+	      This patch adds WARNINGS if a user uses SSLv2/SSLv3 in their configuration,
+	      and explicitly disables SSLv2/SSLv3 if using SSLv23_method.
 
-	  ASTERISK-25256 #close
-	  Reported by: Richard Mudgett
+	  For TLS clients, Asterisk will default to TLSv1+ and WARN if SSLv2 or SSLv3 is
+	  explicitly chosen. For TLS servers, Asterisk will no longer support SSLv2 or
+	  SSLv3.
 
-	  Change-Id: I0b6be720b66fa956f6a798cd22ef8934eb0c0ff3
+	  Much thanks to abelbeck for reporting the vulnerability and providing a patch
+	  for the res_jabber/res_xmpp modules.
 
-2015-07-08 16:39 +0000 [8bcf6d2801]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4096/
 
-	* ARI: Add support for push configuration of dynamic object
+	  ASTERISK-24425 #close
+	  Reported by: abelbeck
+	  Tested by: abelbeck, opsmonitor, gtjoseph
+	  patches:
+	    asterisk-1.8-jabber-tls.patch uploaded by abelbeck (License 5903)
+	    asterisk-11-jabber-xmpp-tls.patch uploaded by abelbeck (License 5903)
+	    AST-2014-011-1.8.diff uploaded by mjordan (License 6283)
+	    AST-2014-011-11.diff uploaded by mjordan (License 6283)
+	  ........
 
-	  This patch adds support for push configuration of dynamic, i.e.,
-	  sorcery, objects in Asterisk. It adds three new REST API calls to the
-	  'asterisk' resource:
-	   * GET /asterisk/{configClass}/{objectType}/{id}: retrieve the current
-	     object given its ID. This returns back a list of ConfigTuples, which
-	     define the fields and their present values that make up the object.
-	   * PUT /asterisk/{configClass}/{objectType}/{id}: create or update an
-	     object. A body may be passed with the request that contains fields to
-	     populate in the object. The same format as what is retrieved using
-	     the GET operation is used for the body, save that we specify that the
-	     list of fields to update are contained in the "fields" attribute.
-	   * DELETE /asterisk/{configClass}/{objectType}/{id}: remove a dynamic
-	     object from its backing storage.
+	  Merged revisions 425987 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Note that the success/failure of these operations is somewhat
-	  configuration dependent, i.e., you must be using a sorcery wizard that
-	  supports the operation in question. If a sorcery wizard does not support
-	  the create or delete mechanisms, then the REST API call will fail with a
-	  403 forbidden.
+	  Merged revisions 425991 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25238 #close
 
-	  Change-Id: I28cd5c7bf6f67f8e9e437ff097f8fd171d30ff5c
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@426003 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-15 15:40 +0000 [e31cb6b248]  Richard Mudgett <rmudgett@digium.com>
+2014-10-19 12:09 +0000 [5e10e369b1]  gtjoseph <george.joseph@fairview5.com>
 
-	* strings.h: Fix issues with escape string functions.
+	* build: Force -fsigned-char on platforms where the default for char is unsigned
 
-	  Fixes for issues with the ASTERISK-24934 patch.
+	  gcc on the ARM platform defaults 'char' to 'unsigned char' whereas Intel and
+	  SPARC default to 'signed char'.  This is only an issue in the rare cases where
+	  negative values are assigned to a 'char' but this this patch insures
+	  compatibility by detecting platforms that default to 'unsigned' and adding an
+	  '-fsigned-char' flag to _ASTCFLAGS.
 
-	  * Fixed ast_escape_alloc() and ast_escape_c_alloc() if the s parameter is
-	  an empty string.  If it were an empty string the functions returned NULL
-	  as if there were a memory allocation failure.  This failure caused the AMI
-	  VarSet event to not get posted if the new value was an empty string.
+	  If compiling for ARM (native or cross-compile) be sure to run ./bootstrap.sh
+	  and ./configure to regenerate the build files.  You shouldn't have to do this
+	  for Intel or SPARC.
 
-	  * Fixed dest buffer overwrite potential in ast_escape() and
-	  ast_escape_c().  If the dest buffer size is smaller than the space needed
-	  by the escaped s parameter string then the dest buffer would be written
-	  beyond the end by the nul string terminator.  The num parameter was really
-	  the dest buffer size parameter so I renamed it to size.
+	  Tested-by: George Joseph
 
-	  * Made nul terminate the dest buffer if the source string parameter s was
-	  an empty string in ast_escape() and ast_escape_c().
+	  Review: https://reviewboard.asterisk.org/r/4091/
+	  ........
 
-	  * Updated ast_escape() and ast_escape_c() doxygen function description
-	  comments to reflect reality.
+	  Merged revisions 425964 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Added some more unit test cases to /main/strings/escape to cover the
-	  empty source string issues.
+	  Merged revisions 425965 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25255 #close
-	  Reported by: Richard Mudgett
 
-	  Change-Id: Id77fc704600ebcce81615c1200296f74de254104
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425966 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-14 14:29 +0000 [243c0d1609]  Richard Mudgett <rmudgett@digium.com>
+2014-10-18 23:03 +0000 [404b6ab3ab]  Matt Jordan <mjordan@digium.com>
 
-	* parking_applications.c: Fix ast_verb() line terminator.
+	* res/res_pjsip_sdp_rtp: Revert 425924
 
-	  Change-Id: I8797238c71563e243c48c6145b4f1ae58f91f775
+	  This patch for r425924 introduced a bug, wherein sending an INVITE request
+	  with no SDP would cause Asterisk to not send an SDP Offer in the 200
+	  OK. The current structure of res_pjsip_sdp_rtp is a bit hard to deal with
+	  to fix this, as create_outgoing_sdp has no knowledge of whether or not it is
+	  creating an SDP as a new Offer or an Answer. This is something of an oversight
+	  in the callback definition, as the caller of it does have this information.
 
-2015-07-14 14:36 +0000 [c782320c68]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_parking: Fix crash if ATTENDEDTRANSFER set empty before Park.
 
-	  setup_park_common_datastore() was assuming that a non-NULL string returned
-	  for the ATTENDEDTRANSFER and BLINDTRANSFER channel variables are not empty
-	  strings.  Things got crashy as a result.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425945 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Made setup_park_common_datastore() treat the channel variable values the
-	  same whether they are NULL or empty for ATTENDEDTRANSFER and
-	  BLINDTRANSFER.
+2014-10-18 19:56 +0000 [b263c8bdae]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25254 #close
-	  Reported by: Richard Mudgett
+	* res/res_pjsip_sdp_rtp: Remove left over reference to override_prefs
 
-	  Change-Id: I9a9c174b33f354f35f82cc6b7cea8303adbaf9c2
+	  The usage of the local override_prefs variable in create_outgoing_sdp_stream
+	  was previously to track an override format preference set by PJSIP_MEDIA_OFFER.
+	  Now, however, that function simply sets the joint capabilities structure,
+	  session->req_caps. During the media format rework, the override_prefs was
+	  instead used to check if there were any formats in session->req_caps.
 
-2015-07-10 18:01 +0000 [2735dd5b2d]  Richard Mudgett <rmudgett@digium.com>
+	  However, this usage isn't useful in create_outgoing_sdp_stream.
+	  session->req_caps contains the negotiated formats for *all* streams, not just
+	  the current one being created. Thus, so long as any stream of any type has
+	  provided a format, override_prefs will be non-zero. Hence, its usage in
+	  checking whether or not we should look at the formats on the endpoint or
+	  the joint capabilities is generally useless.
 
-	* res_pjsip_session.c: Extract sip_session_defer_termination_stop_timer().
+	  There's only two things useful to check:
+	  (1) Does the endpoint have a format for the media type?
+	  (2) Did we negotiate a format for the media type?
 
-	  Change-Id: I9e115dee74bd72e06081d0ee73ecdeb886caa5fb
+	  If either of those is a 'no', then we must kill the media stream.
 
-2015-07-10 10:42 +0000 [3d0ca343ca]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_session.c: Add some helpful comments and minor tweaks.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425924 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I742aeeaf5f760593f323a00fb691affe22e35743
+2014-10-17 17:45 +0000 [b8f687f27c]  Jonathan Rose <jrose@digium.com>
 
-2015-07-10 10:43 +0000 [8d08bb179c]  Richard Mudgett <rmudgett@digium.com>
+	* Sample Configurations: make 'pjsip reload' reload all reloadable pjsip modules
 
-	* res_pjsip_session.c: Fix off nominal crash potential in debug message.
+	  AST-1432 #close
+	  Reported by: John Bigelow
+	  ........
 
-	  Change-Id: I09928297927ee85f7655289acee3a586816466bc
+	  Merged revisions 425905 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-15 10:31 +0000 [0a1a550593]  Matt Jordan <mjordan@digium.com>
 
-	* apps/app_dictate: Fix typo in attribution
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425906 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Last time I checked, it's "Sangoma", not "Samgoma". Thanks to Brian
-	  (GameGamer43) for pointing that out.
+2014-10-17 08:35 +0000 [8f58592252]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I43d7b196f6d7a2b2517b84915e3a8dfbc2894106
+	* res_pjsip_session/res_pjsip_sdp_rtp: Be more tolerant of offers
 
-2015-07-15 10:28 +0000 [3384e64ef6]  Benjamin Ford <bford@digium.com>
+	  When an inbound SDP offer is received, Asterisk currently makes a few
+	  incorrection assumptions:
 
-	* ARI: Fixed unload mode for unload module.
+	  (1) If the offer contains more than a single audio/video stream, Asterisk will
+	      reject the entire stream with a 488. This is an overly strict response;
+	      generally, Asterisk should accept the media streams that it can accept and
+	      decline the others.
+	  (2) If the offer contains a declined media stream, Asterisk will attempt to
+	      process it anyway. This can result in attempting to match format
+	      capabilities on a declined media stream, leading to a 488. Asterisk should
+	      simply ignore declined media streams.
+	  (3) Asterisk will currently attempt to handle offers with AVPF with
+	      use_avpf=No/AVP with use_avpf=Yes. This mismatch results in invalid SDP
+	      answers being sent in response. If there is a mismatch between the media
+	      type being offered and the configuration, Asterisk must reject the offer
+	      with a 488.
 
-	  Changed the unload mode to AST_FORCE_SOFT from AST_FORCE_FIRM,
-	  which would unload a module even if it was in use.
+	  This patch does the following:
+	  * Asterisk will accept SDP offers with at least one media stream that it can
+	    use. Some WARNING messages have been dropped to NOTICEs as a result.
+	  * Asterisk will not accept an offer with a media type that doesn't match its
+	    configuration.
+	  * Asterisk will ignore declined media streams properly.
 
-	  * Changed unload mode to proper mode
+	  #SIPit31
 
-	  ASTERISK-25173
+	  Review: https://reviewboard.asterisk.org/r/4063/
 
-	  Change-Id: If2402487b5bce05d9770f25f65f5c8e292ad5533
+	  ASTERISK-24122 #close
+	  Reported by: James Van Vleet
 
-2015-07-08 16:38 +0000 [0b6ff77afb]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24381 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	* res/res_sorcery_astdb: Add a debugging message for when retrieval by ID fails
+	  Merged revisions 425868 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Having a debug message tell us that we attempted to look up an item but
-	  failed is nice in circumstances when it isn't clear if the wizard was
-	  queried correctly or not.
+	  Merged revisions 425879 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2600c3bbea87f252196358f62e73f4c7da8632f7
 
-2015-07-08 16:37 +0000 [2f0d6d346c]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425881 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/res_pjsip_outbound_registration: Fix WARNING message
+2014-10-17 08:17 +0000 [0d0e38a0e1]  Joshua Colp <jcolp@digium.com>
 
-	  Newlines are nice.
+	* res_pjsip_keepalive: Add runtime configurable keepalive module for connection-oriented transports.
 
-	  Change-Id: Icf0d915db02882e47cd9077ed9009f5d44140d42
+	  This change adds a module which is configurable using the keep_alive_interval setting in the
+	  global section that will send a CRLF keep alive to all active connection-oriented transports at
+	  the provided interval. This is useful because it can help keep connections open through NATs.
+	  This functionality also exists within PJSIP but can not be controlled at runtime and requires
+	  recompiling it.
 
-2015-07-08 16:35 +0000 [cd2213f1ae]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4084/
 
-	* res_pjsip/configuration: Fix a variety of default value problems
 
-	  This patch fixes some bad default value handling in the following
-	  settings:
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425825 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * The 'message_context' and 'accountcode' settings are not mandatory. As
-	    such, we can allow their stringfield values to be empty.
-	  * The 'media_encryption' setting applies a default value of 'none' to
-	    the setting, which it then can't parse or understand. Since the value
-	    is documented to be 'no', this will now apply that as the default
-	    value.
+2014-10-17 08:11 +0000 [86eea19c8f]  Damian Ivereigh (License 6632)
 
-	  Change-Id: Ib9be7f97a7a5b9bc7aee868edf5acf38774cff83
+	* channels/chan_sip: Respect outboundproxy setting when sending qualify requests
 
-2015-07-08 16:32 +0000 [2e4bdbd78a]  Matt Jordan <mjordan@digium.com>
+	  The outboundproxy setting is currently ignored when sending OPTIONS requests
+	  as a result of the qualify setting. This means that if an Asterisk server is
+	  unable to send the packet directly to a peer, it is unable to qualify any
+	  non-inbound registered peer (e.g. a peer SIP Trunk).
 
-	* main/sorcery: Provide log messages when a wizard does not support an operation
+	  This patch grabs the outboundproxy information for a peer when a qualify
+	  attempt is being constructed and, if it finds the information, uses it
+	  when sending the OPTIONS request.
 
-	  If a sorcery wizard does not support one of the 'optional' CRUD
-	  operations (namely the CUD), log a WARNING message so we are aware of
-	  why the operation failed. This also removes an assert in this case, as
-	  the CUD operation may have been triggered by an external system, in
-	  which case it is not a programming error but a configuration error.
+	  Review: https://reviewboard.asterisk.org/r/3948
 
-	  Change-Id: Ifecd9df946d9deaa86235257b49c6e5e24423b53
+	  ASTERISK-24063 #close
+	  Reported by: Damian Ivereigh
+	  patches:
+	    outboundproxy-dai.patch uploaded by Damian Ivereigh (License 6632)
+	  ........
 
-2015-07-10 18:17 +0000 [653f2087e0]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 425818 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* res_pjsip_session.c: Fix crash on call disconnect.
+	  Merged revisions 425819 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The crash fix for ASTERISK-25183 backported some code from master to try
-	  to make sure that a BYE response is processed by the same serializer used
-	  by the BYE request.  The identified race condition causing that backport
-	  was the BYE request code had not finished processing after sending the BYE
-	  before the BYE response came in for processing under a different thread.
-	  Unfortunately, there is still a race condition.  Now the race condition is
-	  between destroying the call session's serializer in
-	  ast_taskprocessor_unreference() and using ast_taskprocessor_get() to get a
-	  reference to the serializer for a BYE response.  Even worse, the new race
-	  condition is a design limitation of the taskprocessor implementation that
-	  didn't matter in versions before v12.  Back then, taskprocessors were only
-	  destroyed when a module unloaded.  Now res_pjsip can destroy them when a
-	  call ends.
+	  Merged revisions 425820 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  However, as noted on the ASTERISK-25183 commit,
-	  session_inv_on_state_changed() is disassociating the dialog from the
-	  session when the invite dialog state becomes PJSIP_INV_STATE_DISCONNECTED.
-	  This is a tad too soon because our BYE request transaction has not
-	  completed yet.
+	  Merged revisions 425821 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Split session_end() that is called by session_inv_on_state_changed() to
-	  hold off session destruction until the BYE transaction timeout occurs or a
-	  failed initial INVITE transaction timeout occurs in
-	  session_inv_on_tsx_state_changed().
 
-	  ASTERISK-25201 #close
-	  Reported by: Matt Jordan
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425822 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iaf8dc8485fd8392a2a3ee4ad3b7f7f04a0dcc961
+2014-10-17 06:30 +0000 [7144c739e9]  Joshua Colp <jcolp@digium.com>
 
-2015-07-14 13:12 +0000 [1aafadf814]  Benjamin Ford <bford@digium.com>
+	* res_pjsip: Add 'user_eq_phone' option to add a 'user=phone' parameter when applicable.
 
-	* ARI: Added new functionality to reload a single module.
+	  This change adds a configuration option which adds a 'user=phone' parameter if the user
+	  portion of the request URI or the From URI is determined to be a number.
 
-	  An http request can be sent to reload an Asterisk module. If the
-	  module can not be reloaded or is not already loaded, an error
-	  response will be returned.
+	  Review: https://reviewboard.asterisk.org/r/4073/
 
-	  The command "curl -v -u user:pass -X PUT 'http://localhost:8088
-	  /ari/asterisk/modules/{moduleName}'" (or something similar, based
-	  on configuration) can be run in the terminal to access this new
-	  functionality.
 
-	  For more information, see:
-	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425804 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Added new ARI functionality
-	  * Asterisk modules can be reloaded through http requests
+2014-10-16 21:49 +0000 [f91cb1207c]  Richard Mudgett
 
-	  ASTERISK-25173
+	* AMI: Add missing VarSet events when a channel inherits variables.
 
-	  Change-Id: I289188bcae182b2083bdbd9ebfffd50b62f58ae1
+	  There should be AMI VarSet events when channel variables are inherited by
+	  an outgoing channel.  Also local;2 should generate VarSet events when it
+	  gets all of its channel variables from channel local;1.
 
-2015-07-14 08:55 +0000 [9dcae23cfc]  Benjamin Ford <bford@digium.com>
+	  ASTERISK-24415 #close
+	  Reported by: Richard Mudgett
+	  Patches:
+	        jira_asterisk_24415_v12.patch (license #5621) patch uploaded by Richard Mudgett
 
-	* ARI: Added new functionality to unload a single module.
+	  Review: https://reviewboard.asterisk.org/r/4074/
+	  ........
 
-	  An http request can be sent to unload an Asterisk module. If the
-	  module can not be unloaded or is already unloaded, an error response
-	  will be returned.
+	  Merged revisions 425782 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The command "curl -v -u user:pass -X DELETE 'http://localhost:8088
-	  /ari/asterisk/modules/{moduleName}'" (or something similar, depending
-	  on configuration) can be run in the terminal to access this new
-	  functionality.
+	  Merged revisions 425783 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  For more information, see:
-	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
 
-	  * Added new ARI functionality
-	  * Asterisk modules can be unloaded through http requests
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425784 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25173
+2014-10-16 21:01 +0000 [df59a71b83]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: I535a95f5676deb02651522761ecbdc0b00b5ac57
+	* bridge_native_rtp: Fix audio issues when moving from remote bridge to softmix
 
-2015-07-13 16:00 +0000 [c219a98d2b]  Benjamin Ford <bford@digium.com>
+	  When a native RTP bridge that is remotely bridging its participants switches
+	  to a softmix bridge, it may not properly re-INVITE the media for one or both
+	  participants back to Asterisk. This is due to the current bridge_native_rtp
+	  code only re-INVITEs if it believes the channel will survive the bridge
+	  operation. Currently, that code is failing, as it expects the channels to
+	  have a soft hangup flag set on it indicating that a redirect has occurred
+	  or that the channel is going to leave the bridge. (The code did not take into
+	  account a smart bridge operation).
 
-	* ARI: Added new functionality to load a single module.
+	  This patch also renames a few things to be more reflective of the underlying
+	  types.
 
-	  An http request can be sent to load an Asterisk module. If the
-	  module can not be loaded or is loaded already, an error response
-	  will be returned.
+	  Review: https://reviewboard.asterisk.org/r/3997/
 
-	  The command curl -v -u user:pass -X POST 'http://localhost:8088/ari
-	  /asterisk/modules/{moduleName}'" (or something similar, depending on
-	  configuration) can be run in the terminal to access this new
-	  functionality.
+	  ASTERISK-24327 #close
+	  ........
 
-	  For more information, see:
-	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+	  Merged revisions 425760 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Added new ARI functionality
-	  * Asterisk modules can be loaded through http requests
+	  Merged revisions 425761 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25173
 
-	  Change-Id: I9e05d5b8c5c666ecfef341504f9edc1aa84fda33
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425762 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-13 10:54 +0000 [73e35d20de]  Benjamin Ford <bford@digium.com>
+2014-10-16 20:46 +0000 [2ccbdd2624]  Matt Jordan <mjordan@digium.com>
 
-	* ARI: Added new functionality to get information on a single module.
+	* test_cel: Update pickup test to expect CANCEL instead of ANSWSER
 
-	  An http request can be sent to retrieve information on a single
-	  module, including the resource name, description, use count, status,
-	  and support level.
+	  The CEL pickup test previously looked for a disposition of ANSWER between the
+	  original caller/peer when the call is picked up. This is actually incorrect:
+	  the disposition should, at the very least, not be ANSWER as the call was
+	  never ANSWERed. The disposition is now CANCEL; this patch updates the test
+	  accordingly.
+	  ........
 
-	  The command "curl -v -u user:pass -X GET 'http://localhost:8088/ari
-	  /asterisk/modules/{moduleName}'" (or something similar, depending on
-	  configuration) can be run in the terminal to access this new
-	  functionality.
+	  Merged revisions 425757 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  For more information, see:
-	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+	  Merged revisions 425758 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Added new ARI functionality
-	  * Information on a single module can now be retrieved
 
-	  ASTERISK-25173
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425759 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ibce5a94e70ecdf4e90329cf0ba66c33a62d37463
+2014-10-16 16:21 +0000 [873d956144]  Matt Jordan <mjordan@digium.com>
 
-2015-07-08 14:56 +0000 [97ee0ee6c6]  Kevin Harwell <kharwell@digium.com>
+	* main/cdr: Use 'time' when rescheduling batched CDRs as opposed to 'size'
 
-	* bridge.c: Fixed race condition during attended transfer
+	  When refactoring CDRs to use the configuration framework, a 'whoops' was
+	  introduced where the CDR batch size was used when rescheduling a batch,
+	  as opposed to the time duration. This patch corrects that obvious mistake.
 
-	  During an attended transfer a thread is started that handles imparting the
-	  bridge channel. From the start of the thread to when the bridge channel is
-	  ready exists a gap that can potentially cause problems (for instance, the
-	  channel being swapped is hung up before the replacement channel enters the
-	  bridge thus stopping the transfer). This patch adds a condition that waits
-	  for the impart thread to get to a point of acceptable readiness before
-	  allowing the initiating thread to continue.
+	  ASTERISK-24426 #close
+	  Reported by: Shane Blaser
+	  ........
 
-	  ASTERISK-24782
-	  Reported by: John Bigelow
+	  Merged revisions 425735 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I08fe33a2560da924e676df55b181e46fca604577
+	  Merged revisions 425736 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-08 16:28 +0000 [bb76b88baf]  Matt Jordan <mjordan@digium.com>
 
-	* main/sorcery: Don't fail object set creation from JSON if field fails
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425744 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Some individual fields may fail their conversion due to their default
-	  values being invalid for their custom handlers. In particular,
-	  configuration values that depend on others being enabled (and thus have
-	  an empty default value) are notorious for tripping this routine up. An
-	  example of this are any of the DTLS options for endpoints. Any of the
-	  DTLS options will fail to be applied (as DTLS is not enabled), causing
-	  the entire object set to be aborted.
+2014-10-16 12:32 +0000 [c2ec5f0f6f]  gtjoseph <george.joseph@fairview5.com>
 
-	  This patch makes it so that we log a debug message when skipping a
-	  field, and rumble on anyway.
+	* config: Fix inf loop using ast_category_browse and ast_variable_retrieve
 
-	  ASTERISK-25238
+	  Fix infinite loop when calling ast_variable_retrieve inside an
+	  ast_category_browse loop when there is more than 1 category with
+	  the same name.
 
-	  Change-Id: I0bea13de79f66bf9f9ae6ece0e94a2dc1c026a76
+	  Tested-by: George Joseph
 
-2015-07-08 16:21 +0000 [5f13c2226a]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4089/
+	  ........
 
-	* main/format_cap: Parse capabilities generated by ast_format_cap_get_names
+	  Merged revisions 425713 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  We have a strange relationship between the parsing of format
-	  capabilities from a string and their representation as a string. We
-	  expect the format capabilities to be expressed as a string in the
-	  following format:
+	  Merged revisions 425714 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  allow = !all,ulaw,alaw
-	  disallow = g722
 
-	  While we would generate the string representation of those formats as:
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425715 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  allow = (ulaw|alaw)
-	  disallow = (ulaw|alaw|g729...)
+2014-10-16 11:32 +0000 [86a4ce4957]  Kinsey Moore <kmoore@digium.com>
 
-	  When the configuration framework needs to store values as a string, it
-	  generates the format capabilities using the second representation; this
-	  representation however cannot be parsed when the entry is rehydrated.
-	  This patch fixes that by updating
-	  ast_format_cap_update_by_allow_disallow to parse an entry as if it were
-	  in the generated format if it has a leading '(' and a trailing ')'.
+	* PJSIP: Enforce module load dependencies
 
-	  ASTERISK-25238
+	  This enforces that res_pjsip, res_pjsip_session, and res_pjsip_pubsub
+	  have loaded properly before attempting to load any modules that depend
+	  on them since the module loader system is not currently capable of
+	  resolving module dependencies on its own.
 
-	  Change-Id: I904d43caf4cf45af06f6aee0c9e58556eb91d6ca
+	  ASTERISK-24312 #close
+	  Reported by: Dafi Ni
+	  Review: https://reviewboard.asterisk.org/r/4062/
+	  ........
 
-2015-06-27 17:53 +0000 [2325b106fd]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 425690 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* tests/test_devicestate: Add additional tests for the device state API
+	  Merged revisions 425691 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch adds more tests that exercise the device state API. This includes:
 
-	  * Tests that cover adding a device state provider, as well as deleting a
-	    device state provider. This also verifies that you cannot add an
-	    already added device state provider, and cannot delete an already
-	    deleted device state provider.
-	  * A test that covers changing device state and receiving said updates
-	    from a device state subscriber. This also covers hitting both the
-	    device state cache as well as a custom device state provider.
-	  * A test that covers converting device state to channel state and device
-	    state values to a string representation and back.
-	  * A test that covers obtaining device state from an active channel and a
-	    channel driver that provides its own device state.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425700 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I2adca67ffb405cd8625a5d6df1e3f9b3d945c08d
+2014-10-16 01:22 +0000 [a770ca168d]  Igor Goncharovskiy <igor.goncharovsky@gmail.com>
 
-2015-06-27 17:51 +0000 [328f0be806]  Matt Jordan <mjordan@digium.com>
+	* 
+	  Fix loss of voice after second call drops (on a second line) in case using multiple lines on unistim phones. There is regression was introduced in r391379.
 
-	* main/devicestate: Prevent duplicate registration of device state providers
+	  Reported by: Rustam Khankishyiev
+	  (closes issue ASTERISK-23846)
+	  ........
 
-	  Currently, the device state provider API will allow you to register a
-	  device state provider with the same case insensitive name more than
-	  once. This could cause strange issues, as the duplicate device state
-	  providers will not be queried when a device's state has to be polled.
-	  This patch updates the API such that a device state provider with the
-	  same name as one that has already registered will be rejected.
+	  Merged revisions 425667 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I4a418a12280b7b6e4960bd44f302e27cd036ceb2
+	  Merged revisions 425668 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-07-10 22:25 +0000 [bee41eec62]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 425669 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_sorcery_memory_cache: Fix test registration issues
 
-	  Again, tests now need to not end with a newline. This patch makes it so
-	  the tests can register again, unit tests will actually pass, and we can
-	  stop wasting time trying to figure out why builds are failing when they
-	  really aren't failing.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425677 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ide519fbeba89f413c733446c5ff7b224fc4ce840
+2014-10-15 20:26 +0000 [bfee1b4bc5]  Joshua Colp <jcolp@digium.com>
 
-2015-07-10 21:42 +0000 [4d738e9026]  Matt Jordan <mjordan@digium.com>
+	* res_rtp_asterisk: Fix a bug where ICE state would get reset when it shouldn't.
 
-	* tests/test_sorcery_memory_cache_thrash: Fix test loading problems
+	  In the case where the ICE negotiation had not yet started current state would
+	  get wiped when it shouldn't.
 
-	  Because unit tests now want descriptions to not end with a newline, the
-	  sorcery memory cache thrash tests failed to register. This patch
-	  corrects their descriptions.
+	  This also removes channel binding as in practice this does not work well with
+	  other implementations.
+	  ........
 
-	  Change-Id: Id004b1becfdeed8ee3c846f49beab76a5c0f68b6
+	  Merged revisions 425644 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-06-26 10:57 +0000 [47ea312b24]  Benjamin Ford <bford@digium.com>
+	  Merged revisions 425645 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* ARI: Added new functionality to get all module information.
+	  Merged revisions 425646 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  An http request can be sent to retrieve a list of all existing modules,
-	  including the resource name, description, use count, status, and
-	  support level.
 
-	  The command "curl -v -u user:pass -X GET 'http://localhost:8088/ari/
-	  asterisk/modules" (or something similar, depending on configuration)
-	  can be run in the terminal to access this new functionality.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425647 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  For more information, see:
-	  https://wiki.asterisk.org/wiki.display/~bford/Asterisk+ARI+Resource
+2014-10-15 14:39 +0000 [28c11fff78]  Richard Mudgett <rmudgett@digium.com>
 
-	  * Added new ARI functionality
-	  * Information on modules can now be retrieved
+	* chan_motif: Cleanup jingle_tech.capabilities only once.
+	  ........
 
-	  Change-Id: I63cbbf0ec0c3544cc45ed2a588dceabe91c5e0b0
+	  Merged revisions 425627 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-09 09:18 +0000 [d558b00c85]  Joshua Colp <jcolp@digium.com>
 
-	* bridge_native_rtp.c: Don't start native RTP bridging after attended transfer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425628 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The bridge_native_rtp module adds a frame hook to channels which are in
-	  a native RTP bridge. This frame hook is used to intercept when a hold
-	  or unhold frame traverses the bridge so native RTP can be stopped or
-	  started as appropriate. This is expected but exposes a specific bug
-	  when attended transfers are involved.
+2014-10-15 14:17 +0000 [3d58066de9]  Jonathan Rose <jrose@digium.com>
 
-	  Upon completion of an attended transfer an unhold frame is queued up
-	  to take one of the channels involved off hold. After this is done
-	  the channel is moved between bridges.
+	* parking_tests: Fix assertions and possibly crashes in res_parking unit tests
 
-	  When the frame hook is involved in this case for the unhold it
-	  releases the channel lock and acquires the bridge lock. This
-	  allows the bridge core to step in and move the channel
-	  (potentially changing the bridging techology) from another thread.
-	  Once completed the bridge lock is released by the bridge core.
-	  The frame hook is then able to acquire the bridge lock and
-	  wrongfully starts native RTP again, despite the channel no longer
-	  being in the bridge or needing to start native RTP. In fact at
-	  this point the frame hook is no longer attached to the channel.
+	  Assertions were caused by attempting to play music on hold to a channel with
+	  no formats. Parking unit test channels were given formats and a technology so
+	  that they would be able to pretend to read/write frames.
 
-	  This change makes it so the native RTP bridge data is available to
-	  the frame hook when it is invoked. Whether the frame hook has
-	  been detached or not is stored on the native RTP bridge data and
-	  is checked by the frame hook before starting or stopping native
-	  RTP bridging. If the frame hook has been detached it does nothing.
+	  ASTERISK-24413 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/4075/
+	  ........
 
-	  ASTERISK-25240 #close
+	  Merged revisions 425611 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I13a73186a05f4e5a764f81e5cd0ccec1ed1891d2
 
-2015-05-16 17:02 +0000 [b74b071369]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425613 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_sorcery_memory_cache: Backport to 13
+2014-10-15 05:03 +0000 [90c98d384b]  Alexandr Anikin <may@telecom-service.ru>
 
-	  Gerrit is complaining of conflicts when trying to create a patch series
-	  of all of the cherry-picked master commits, so I have instead squashed
-	  it all into one commit.
+	* chan_ooh323: fix rtptimeout general value checking
 
-	  ASTERISK-25067 #close
-	  Reported by: Matt Jordan
+	  correct condition to check rtptimeout in [general] config section
 
-	  Change-Id: I6dda90343fae24a75dc5beec84980024e8d61eb9
+	  ASTERISK-24393 #close
+	  Reported by:  Dmitry Melekhov
+	  Tested by:  Dmitry Melekhov
+	  Patches:
+	    ASTERISK-24393.patch
+	  ........
 
-2015-07-08 04:21 +0000 [7ff1ac8797]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 425547 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* res_rtp_asterisk: Ensure DTLS timeout timer is -1 if DTLS is not used.
+	  Merged revisions 425548 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This change fixes a bug where the DTLS timeout timer would be
-	  initialized to 0 if DTLS was not used for an RTP session.
+	  Merged revisions 425589 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25103
+	  Merged revisions 425590 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: If8d26bb054f1d300838850da5b8db9044c2fe2ac
 
-2015-07-01 07:55 +0000 [05e8e14982]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425591 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-10-14 15:48 +0000 [104fca5001]  gtjoseph <george.joseph@fairview5.com>
+
+	* config: Fix SEGV in unit test with MALLOC_DEBUG
 
-	* res_rtp_asterisk: Prevent simultaneous access to DTLS SSL context.
+	  With MALLOC_DEBUG the /main/config config_basic_ops test was causing a
+	  SEGV while doing an ast_category_delete in an ast_category_browse loop.
+	  Apparently this never worked but was also never tested.  I removed the
+	  test, added 2 notes to config.h indicating that it's not supported and
+	  added a few lines of code to ast_category_delete to prevent the SEGV
+	  should someone attempt it in the future.
 
-	  This change moves logic for setting up the DTLS SSL contexts to
-	  when the SDP is done being processed instead of when ICE negotiation
-	  completes. It also stops handshakes from being initiated when we
-	  are acting as a server.
+	  Tested-by: George Joseph
 
-	  Manipulating the SSL context when ICE negotiation has completed
-	  is problematic as the SSL context is not protected and if acting
-	  as a client the remote side may have started DTLS negotiation
-	  already.
+	  Review: https://reviewboard.asterisk.org/r/4078/
+	  ........
 
-	  The retransmission timeout timer code has also been split up
-	  and simplified some. Both RTP and RTCP now have their own timers
-	  and the points at which the timer is stopped and started is now
-	  more specific. When a packet is sent the timer is started. When
-	  a response is received but before it is processed the timer is
-	  stopped. This provides a guarantee that the timeout is not
-	  occurring while the response is processed.
+	  Merged revisions 425525 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-22805 #close
-	  ASTERISK-24550 #close
-	  ASTERISK-24651 #close
-	  ASTERISK-24832 #close
-	  ASTERISK-25103 #close
-	  ASTERISK-25127 #close
+	  Merged revisions 425526 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ib75ea2546f29d6efc3d2d37c58df6986c7bd9b91
 
-2015-06-26 16:10 +0000 [38bace4fbb]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425527 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_t38.c: Fix always false if test.
+2014-10-14 14:12 +0000 [87b5006ff0]  Jonathan Rose <jrose@digium.com>
 
-	  Calling t38_change_state() sets the t38 state so it makes little sense to
-	  then check the state right after the call for something else.
+	* Scheduler: Fix a nasty scheduler caching bug which makes new tasks not execute
 
-	  * Made the code in t38_interpret_parameters() reject or exit T.38 mode as
-	  intended but not implemented.
+	  Tasks that were marked for pending deletion in the scheduler would be moved to
+	  the cache for later reuse, but after being recycled the deleted mark wouldn't
+	  be removed resulting in fresh tasks being deleted without reason... and
+	  immediately moved back into the cache where they could be reused again. This
+	  could cause horrendous things to happen in just about anything that used a
+	  scheduler.
 
-	  Change-Id: Ib281263a6ed44da9448132c4e6df1e183b8a3df2
+	  ASTERISK-24321 #close
+	  Reported by: Steve Pitts
+	  Review: https://reviewboard.asterisk.org/r/4071/
+	  ........
 
-2015-06-30 11:17 +0000 [2f7688c788]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 425503 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res_pjsip_mwi.c: Use safer loop coding in mwi_subscription_mailboxes_str().
+	  Merged revisions 425504 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I6f39d809a6d1b47b35bb32b298f5a12f35d6f907
 
-2015-06-30 11:14 +0000 [74be3a50d7]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425505 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_mwi.c: Eliminate a simple RAII_VAR.
+2014-10-14 13:13 +0000 [527b58aeb7]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: Ib1843f81e826a6c760c424c88eb70c350d9d61da
+	* res_phoneprov: Create accessor for ast_phoneprov_std_variable_lookup
 
-2015-06-30 11:11 +0000 [589e93617a]  Richard Mudgett <rmudgett@digium.com>
+	  Based on feedback from Richard, I created an accessor for
+	  res_phoneprov/ast_phoneprov_std_variable_lookup and added
+	  load priority to AST_MODULE_INFO.
 
-	* res_pjsip_mwi.c: Fix mid-line log message line breaks.
+	  Tested-by: George Joseph
+	  Tested-by: Richard Mudgett
 
-	  * Add create_mwi_subscriptions_for_endpoint() doxygen comment.
+	  Review: https://reviewboard.asterisk.org/r/4076/
+	  ........
 
-	  Change-Id: I3c3f921f4ec749fb65b62d2f6fa0d4d1888b94e2
+	  Merged revisions 425480 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-06-26 18:48 +0000 [0d67e04359]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 425481 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_mwi.c: Fix MWI subscription memory corruption crash.
 
-	  MWI subscriptions can crash or corrupt memory when using the subscription
-	  datastore to access the MWI subscription object because the datastore is
-	  not holding a reference to the object.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425482 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Give the subscription datastore a ref to the MWI subscription object.
-	  It is unfortunate that the ref causes a circular ref chain that must be
-	  explicitly broken to allow the memory to get released.  The loop is broken
-	  when the subscription is shutdown and if the subscription setup fails.
+2014-10-14 11:47 +0000 [fbb19db0c8]  Corey Farrell <git@cfware.com>
 
-	  ASTERISK-25168 #close
-	  Reported by: Carl Fortin
+	* res_fax: Fix reference leak caused by gateway sessions
 
-	  Change-Id: Ice4fa823f138ff10a6c74d280699c41a82836d4f
+	  Fax gateway session objects can be re-used, causing the
+	  same gateway session to be added to faxregistry.container
+	  more than once.  This change causes fax_session_new to
+	  remove the reserved session from the container before
+	  it's id is changed, ensuring it's possible for the
+	  session to be freed.
 
-2015-07-02 14:51 +0000 [0422433f47]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24392 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4049/
+	  ........
 
-	* PJSIP XML, XPIDF: Fix buffer size overwrite memory corruption error.
+	  Merged revisions 425457 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  When res_pjsip body generator modules were generating XML or XPIDF
-	  response bodies, there was a chance that the generated body would be the
-	  exact size of the supplied buffer.  Adding the nul string terminator would
-	  then write beyond the end of the buffer and potentially corrupt memory.
+	  Merged revisions 425458 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Fix MALLOC_DEBUG high fence violations caused by adding a nul string
-	  terminator on the end of a buffer for XML or XPIDF response bodies.
+	  Merged revisions 425459 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Made calls to pj_xml_print() safer if the XML prolog is requested.  Due
-	  to a bug in pjproject, the return value could be -1 _or_
-	  AST_PJSIP_XML_PROLOG_LEN if the supplied buffer is not large enough.
 
-	  * Updated the doxygen comment of AST_PJSIP_XML_PROLOG_LEN to describe the
-	  return value of pj_xml_print() when the supplied buffer is not large
-	  enough.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425460 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25168
-	  Reported by: Carl Fortin
+2014-10-14 11:43 +0000 [c61b66e107]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Id70e1d373a6a2b2bd9e678b5cbc5e55b308981de
+	* stasis_channels.c: Resolve unfinished Dials when doing masquerades (Part 2)
 
-2015-06-26 10:36 +0000 [8ea214aed7]  Richard Mudgett <rmudgett@digium.com>
+	  Masquerades into and out of channels that are involved in a dial operation
+	  don't create the expected dial end event.  The missing dial end event goes
+	  against the model for things like CDRs and generating Dial end manager
+	  actions and such.
 
-	* PJSIP FAX: Fix T.38 automatic reject timer NULL channel pointer dereferences.
+	  There are four cases:
 
-	  When a caller calls a FAX number and then hangs up right after the call is
-	  answered then the T.38 re-INVITE automatic reject timer may still be
-	  running after the channel goes away.
+	  1) A channel masquerades into the caller channel.  The case happens when
+	  performing a blonde transfer using the channel driver's protocol.
 
-	  * Added session NULL channel checks on the code paths that get executed by
-	  t38_automatic_reject() to prevent a crash when the T.38 re-INVITE
-	  automatic reject timer expires.
+	  2) A channel masquerades into a callee channel.  The case happens when
+	  performing a directed call pickup.
 
-	  ASTERISK-25168
-	  Reported by: Carl Fortin
+	  3) The caller channel masquerades out of dial.  The case happens when
+	  using the Bridge application on the caller channel.
 
-	  Change-Id: I07b6cd23815aedce5044f8f32543779e2f7a2403
+	  4) A callee channel masquerades out of dial.  The case happens when using
+	  the Bridge application on a peer channel.
 
-2015-06-05 15:37 +0000 [ada7346792]  Richard Mudgett <rmudgett@digium.com>
+	  As it turned out, all four cases need to be handled instead of just the
+	  first one.
 
-	* res_pjsip: Need to use the same serializer for a pjproject SIP transaction.
+	  ASTERISK-24237
+	  Reported by: Richard Mudgett
 
-	  All send/receive processing for a SIP transaction needs to be done under
-	  the same threadpool serializer to prevent reentrancy problems inside
-	  pjproject and res_pjsip.
+	  ASTERISK-24394 #close
+	  Reported by: Richard Mudgett
 
-	  * Add threadpool API call to get the current serializer associated with
-	  the worker thread.
+	  Review: https://reviewboard.asterisk.org/r/4066/
+	  ........
 
-	  * Pick a serializer from a pool of default serializers if the caller of
-	  res_pjsip.c:ast_sip_push_task() does not provide one.
+	  Merged revisions 425430 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This is a simple way to ensure that all outgoing SIP request messages are
-	  processed under a serializer.  Otherwise, any place where a pushed task is
-	  done that would result in an outgoing out-of-dialog request would need to
-	  be modified to supply a serializer.  Serializers from the default
-	  serializer pool are picked in a round robin sequence for simplicity.
+	  Merged revisions 425455 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  A side effect is that the default serializer pool will limit the growth of
-	  the thread pool from random tasks.  This is not necessarily a bad thing.
 
-	  * Made pjsip_distributor.c save the thread's serializer name on the
-	  outgoing request tdata struct so the response can be processed under the
-	  same serializer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425456 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This is a cherry-pick from master.
+2014-10-14 11:20 +0000 [01bdc80475]  Corey Farrell <git@cfware.com>
 
-	  **** ASTERISK-25115 Change-Id: Iea71c16ce1132017b5791635e198b8c27973f40a
+	* res_fax: Resolve module reference leak caused by reserved sessions
 
-	  NOTE: session_inv_on_state_changed() is disassociating the dialog from the
-	  session when the invite dialog becomes PJSIP_INV_STATE_DISCONNECTED.
-	  Unfortunately this is a tad too soon because our BYE request transaction
-	  has not completed yet.
+	  Remove reference to module providing reserved session after
+	  adding a reference to the final module.  This re-reference
+	  is done to ensure that module references are correct even
+	  if the final session selects a different module than the
+	  reserved session.
 
-	  ASTERISK-25183 #close
-	  Reported by: Matt Jordan
+	  ASTERISK-18923 #close
+	  Reported by: Grigoriy Puzankin
+	  Review: https://reviewboard.asterisk.org/r/4048/
+	  ........
 
-	  Change-Id: I8bad0ae1daf18d75b8c9e55874244b7962df2d0a
+	  Merged revisions 425405 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-07-04 18:22 +0000 [55137c3d12]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 425407 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res/res_http_websocket: Don't send HTTP response fragmented.
+	  Merged revisions 425411 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change makes it so that when accepting a WebSocket
-	  connection the HTTP response is sent as one packet instead of
-	  fragmented. Browsers don't like it when you send it fragmented.
+	  Merged revisions 425415 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25103
 
-	  Change-Id: I9b82c4ec2949b0bce692ad0bf6f7cea9709e7f69
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425419 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-27 18:47 +0000 [49f81ddb85]  Matt Jordan <mjordan@digium.com>
+2014-10-13 11:12 +0000 [c7e6b6ba3d]  gtjoseph <george.joseph@fairview5.com>
 
-	* Makefile: Remove coverage files on 'make clean'
+	* manager/config: Support templates and non-unique category names via AMI
 
-	  This patch updates a variety of Makefiles in Asterisk's build system to
-	  remove .gcda and .gcno files when 'make clean' is executed. These files
-	  are generated when '--enable-coverage' is passed to the Asterisk
-	  configure script.
+	  This patch provides the capability to manipulate templates and categories
+	  with non-unique names via AMI.
+
+	  Summary of changes:
+
+	  GetConfig and GetConfigJSON: Added "Filter" parameter:  A comma separated list
+	  of name_regex=value_regex expressions which will cause only categories whose
+	  variables match all expressions to be considered.  The special variable name
+	  TEMPLATES can be used to control whether templates are included.  Passing
+	  'include' as the value will include templates along with normal categories.
+	  Passing 'restrict' as the value will restrict the operation to ONLY templates.
+	  Not specifying a TEMPLATES expression results in the current default behavior
+	  which is to not include templates.
+
+	  UpdateConfig: NewCat now includes options for allowing duplicate category
+	  names, indicating if the category should be created as a template, and
+	  specifying templates the category should inherit from.  The rest of the
+	  actions now accept a filter string as defined above.  If there are non-unique
+	  category names, you can now update specific ones based on variable values.
+
+	  To facilitate the new capabilities in manager, corresponding changes had to be
+	  made to config, most notably the addition of filter criteria to many of the
+	  APIs.  In some cases it was easy to change the references to use the new
+	  prototype but others would have required touching too many files for this
+	  patch so a wrapper with the original prototype was created.  Macros couldn't
+	  be used in this case because it would break binary compatibility with modules
+	  such as res_digium_phone that are linked to real symbols.
 
-	  Change-Id: Ib70b41eea2ee2908885bff02e80faf9f40c84602
+	  Tested-by: George Joseph
 
-2015-07-02 09:08 +0000 [e0f565663b]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Review: https://reviewboard.asterisk.org/r/4033/
+	  ........
 
-	* chan_sip: Fix early call pickup channel leak.
+	  Merged revisions 425383 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  When handle_invite_replaces() was called, and either ast_bridge_impart()
-	  failed or there was no bridge (because the channel we're picking up was
-	  still ringing), chan_sip would leak a channel.
+	  Merged revisions 425384 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Thanks Matt and Corey for checking the bridge path.
 
-	  ASTERISK-25226 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425385 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ie736bb182170a73eef5bcef0ab0376f645c260c8
+2014-10-12 16:09 +0000 [8d6f1d763c]  Joshua Colp <jcolp@digium.com>
 
-2015-07-02 06:19 +0000 [a5a262be78]  Walter Doekes <walter+asterisk@wjd.nu>
+	* res_rtp_asterisk: Make the ICE transport check case insensitive as some implementations use 'udp'.
+	  ........
 
-	* chan_mgcp: Don't call close on fd -1.
+	  Merged revisions 425360 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25220 #close
+	  Merged revisions 425361 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Ic48f3a82f51ada87f2fb0e016c9efe0ad56f1ee3
+	  Merged revisions 425362 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-02 06:10 +0000 [b835312b4c]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* rtp_engine: Skip useless self-assignment in ast_rtp_engine_unload_format.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425363 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When running valgrind on Asterisk, it complained about:
+2014-10-12 03:17 +0000 [9e72c74db5]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	      ==32423== Source and destination overlap in memcpy(0x85a920, 0x85a920, 304)
-	      ==32423==    at 0x4C2F71C: memcpy@@GLIBC_2.14 (in /usr/lib/valgrind/...)
-	      ==32423==    by 0x55BA91: ast_rtp_engine_unload_format (rtp_engine.c:2292)
-	      ==32423==    by 0x4EEFB7: ast_format_attr_unreg_interface (format.c:1437)
+	* chan_sip: Fix so asterisk won't send reINVITE after a BYE.
 
-	  The code in question is a struct assignment, which may be performed by
-	  memcpy as a compiler optimization. It is changed to only copy the struct
-	  contents if source and destination are different.
+	  After a reINVITE glare situation, Asterisk would re-send the reINVITE
+	  even though the call had been hung up in the mean time.  This patch
+	  unschedules the reinvite when handling the BYE.
 
-	  ASTERISK-25219 #close
+	  ASTERISK-22791 #close
+	  Reported by: Paolo Compagnini
+	  Tested by: Paolo Compagnini
 
-	  Change-Id: I6d3546c326b03378ca8e9b8cefd41c16e0088b9a
+	  Review: https://reviewboard.asterisk.org/r/4056/
+	  (testcase is in review r4055)
+	  ........
 
-2015-07-02 05:16 +0000 [6551e16e03]  Walter Doekes <walter+asterisk@wjd.nu>
+	  Merged revisions 425296 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* astfd: Fix buffer overflow in DEBUG_FD_LEAKS.
+	  Merged revisions 425297 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  If DEBUG_FD_LEAKS was used and more file descriptors than the default of
-	  1024 were available, some DEBUG_FD_LEAKS-patched functions would
-	  overwrite memory past the fixed-size (1024) fdleaks buffer.
+	  Merged revisions 425298 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change:
-	  - adds bounds checks to __ast_fdleak_fopen and __ast_fdleak_pipe
-	  - consistently uses ARRAY_LEN() instead of sizeof() or 1023 or 1024
-	  - stores pointers to constants instead of copying the contents
-	  - reorders the fdleaks struct for possibly tighter packing
-	  - adds a tiny bit of documentation
+	  Merged revisions 425299 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25212 #close
 
-	  Change-Id: Iacb69e7701c0f0a113786bd946cea5b6335a85e5
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425300 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-07-02 04:57 +0000 [f4dd9560cf]  Walter Doekes <walter+asterisk@wjd.nu>
+2014-10-12 02:57 +0000 [c0ac874106]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* res_timing: Don't close FD 0 when out of open files.
+	* build: Relax badshell tilde test to allow for ~ in middle of DESTDIR.
 
-	  This fixes so a failure to get a timer file descriptor does not cascade
-	  to closing FD 0.
+	  The main Makefile has a target test called 'badshell' that tests if
+	  DESTDIR does not happen to have an an-expanded tilde (~).  This might
+	  be the case if you run: make install DESTDIR=~/somewhere/
 
-	  On error, both res_timing_kqueue and res_timing_timerfd would call the
-	  destructor before setting the file handle. The file handle had been
-	  initialized to 0, causing FD 0 to be closed. This in turn, resulted in
-	  floods of "CLI>" messages and an unusable terminal.
+	  That test also disallowed valid tildes in directory names. The test is
+	  now changed to only trigger on a tilde at the start of the path.
 
-	  ASTERISK-19277 #close
-	  Reported by: Barry Chern
+	  ASTERISK-13797 #close
+	  Reported by: Tzafrir Cohen
 
-	  For the 13 branch, this was already fixed. This patch only ensures that
-	  we do not attempt to close a negative file descriptor.
+	  Review: https://reviewboard.asterisk.org/r/4064/
+	  ........
 
-	  Change-Id: I147d7e33726c6e5a2751928d56561494f5800350
+	  Merged revisions 425291 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-07-01 17:25 +0000 [78a1f4aa46]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 425292 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* chan_vpb.cc: Fix compiler warning Jenkins found.
+	  Merged revisions 425293 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I0ec7fd10d56d90d5a60b12b5a7d6807f265ac5e0
+	  Merged revisions 425294 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-07-01 13:34 +0000 [6b16fbfc22]  Scott Griepentrog <scott@griepentrog.com>
 
-	* Channel alert pipe: improve diagnostic error return
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425295 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When a frame is queued on a channel, any failure in
-	  ast_channel_alert_write is logged along with errno.
+2014-10-12 02:47 +0000 [2a03efdbae]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  This change improves the diagnostic message through
-	  aligning the errno value with actual failure cases.
+	* res_calendar_ews: Relax neon version check to work with 0.30 too.
 
-	  ASTERISK-25224
-	  Reported by: Andrey Biglari
+	  Allow res_calendar_ews to work not only with libneon-0.29 but also
+	  with 0.30.
 
-	  Change-Id: I1bf7b3337ad392789a9f02c650589cd065d20b5b
+	  ASTERISK-24325 #close
+	  Reported by: Tzafrir Cohen
 
-2015-07-01 16:04 +0000 [8e07ab145d]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/4068/
+	  ........
 
-	* sorcery/realtime: Add a bit of debug and warning messages for bad configs
+	  Merged revisions 425286 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  When a mapping does not exist between a sorcery.conf defined object and
-	  a realtime mapping in extconf, currently, the user will receive a slew
-	  of ERROR messages that don't really tell what is happening. Some ERROR
-	  messages may even be misleading, as they occur after the sorcery API has
-	  already given up on the attempt to load and create the sorcery object.
+	  Merged revisions 425287 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch adds a bit of debug and a useful WARNING message for when a
-	  wizard's open callback fails for a particular object type. In the bad
-	  configurations that resulted in this patch, this provided a 'root cause'
-	  WARNING message that pointed in the right direction of the configuration
-	  problem.
+	  Merged revisions 425288 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I1cc7344f2b015b8b9c85a7e6ebc8cb4753a8f80b
-2015-06-29 12:45 +0000 [156395e743]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 425289 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_sorcery_realtime: Fix leak of sorcery object type.
 
-	  This prevents a leak of a sorcery object type when realtime sorcery
-	  objects are retrieved by fields or when multiple objects are retrieved.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425290 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The extent of this leak is that sorcery object types would be leaked.
-	  These are allocated whenever an object type is registered with sorcery,
-	  meaning that on module shutdown, these objects would be leaked. This
-	  could be problematic if many reloads were performed, but it is not as
-	  severe as if every sorcery object retrieved from realtime were being
-	  leaked.
+2014-10-11 16:09 +0000 [6a3c11c75b]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25165 #close
-	  Reported by Corey Farrell
+	* res_phoneprov: Cleanup module load error handling
 
-	  Change-Id: I625c3b50eee4576670b7eeb013c81ad043b4b4f8
+	  Tested module load/reload interaction between res_phoneprov and
+	  res_pjsip_phoneprov_provider in cases where res_phoneprov didn't
+	  load correctly (usually misconfiguration or missing phoneprov.conf)
 
-2015-06-26 22:02 +0000 [a5e9c4e9b2]  Matt Jordan <mjordan@digium.com>
+	  Tested-by: George Joseph
 
-	* res/res_corosync: Always decline module load, instead of failing
+	  Review: https://reviewboard.asterisk.org/r/4069/
+	  ........
 
-	  Returns a 'failure' from the module load routine indicates to Asterisk
-	  that it should abort loading completely. This is rarely - in fact,
-	  really, never - a good option. Aborting load of Asterisk from a dynamic
-	  module implies that the core, and the rest of the dynamic modules, don't
-	  matter: we should abandon all processing.
+	  Merged revisions 425264 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  res_corosync is really not that important.
+	  Merged revisions 425265 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch updates the module such that, if it fails to load, it
-	  politely declines (emitting ERROR messages along the way), and allows
-	  Asterisk to continue to function.
 
-	  Note that this issue was keeping Asterisk unit tests from running on
-	  certain build agents.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425266 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I252249e81fb9b1a68e0da873f54f47e21d648f0f
+2014-10-10 15:48 +0000 [98d5b7090d]  Joshua Colp <jcolp@digium.com>
 
-2015-06-26 20:38 +0000 [399cd8bcd9]  Matt Jordan <mjordan@digium.com>
+	* bridge: During a smart bridge operation provide a more complete bridge to the old technology.
 
-	* main/pbx: Resolve case sensitivity regression in PBX hints
+	  When a smart bridge operation occurs and a bridge transitions from one
+	  technology to another the old technology is provided the channels formerly
+	  in it and told that they are leaving. Unfortunately the bridge provided
+	  along with them is incomplete. The bridge, despite there being channels in it,
+	  contains none. This forces technology implementations to have additional
+	  logic when channels are leaving or to store their own duplicated
+	  state.
 
-	  When 8297136f was merged for ASTERISK-25040, a regression was introduced
-	  surrounding the case sensitivity of device names within hints.
-	  Previously, device names - such as 'sip/foo' - were compared in a case
-	  insensitive fashion. Thus, 'sip/foo' was equivalent to 'SIP/foo'. After
-	  that patch, only the case sensitive name would match, i.e., 'SIP/foo'.
-	  As a result, some dialplan hints stopped working.
+	  This change makes the bridge more complete so it contains the expected
+	  channels. Now that the bridge is complete special logic within
+	  bridge_native_rtp is no longer needed and has been removed.
 
-	  This patch re-introduces case insensitive matching for device names in
-	  hints.
+	  Review: https://reviewboard.asterisk.org/r/4057/
+	  ........
 
-	  ASTERISK-25040
+	  Merged revisions 425242 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25202 #close
+	  Merged revisions 425243 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: If5046a7d14097e1e3c12b63092b9584bb1e9cb4c
-	  (cherry picked from commit 96bbcf495a1da9e607d9b04a44b5c4f49e83cc03)
 
-2015-06-26 16:12 +0000 [24eec5a10b]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425244 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_nat: Adjust when contact should be rewritten.
+2014-10-10 09:31 +0000 [c3ff212cae]  Matt Jordan <mjordan@digium.com>
 
-	  A previous change made the contact only get rewritten if the dialog's
-	  route set was not marked frozen. Unfortunately, while the intent of this
-	  is correct, the dialog's route set actually gets marked as frozen
-	  earlier than expected, especially for UAS dialogs.
+	* res/res_phoneprov: Bail on registration if res_phoneprov didn't load
 
-	  Instead, the idea is that the contact needs to not be rewritten if there
-	  is a pre-existing route set on the dialog. This is now accomplished by
-	  checking the dialog's route set list instead of checking if the route
-	  set is frozen.
+	  If res_phoneprov failed to fully load (due to not being configured), the
+	  providers container will be NULL. If a module attempts to register a phone
+	  provisioning provider, it should check for the presence of the container.
+	  If there is no providers container, it should return an error.
 
-	  Doing this causes some broken tests to begin passing again.
+	  This patch makes the ast_phoneprov_provider_register function do that...
+	  otherwise this would be a silly commit message.
+	  ........
 
-	  ASTERISK-25196
-	  Reported by Mark Michelson
+	  Merged revisions 425220 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I525ab251fd40a52ede327a52a2810a56deb0529e
+	  Merged revisions 425221 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-19 18:27 +0000 [0ec461a637]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_outbound_registration.c: Add a serializer shutdown group.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425222 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The client_state objects contain a serializer used to send the outbound
-	  REGISTER messages.  Once all those message transactions are complete then
-	  the module can shutdown.
+2014-10-10 09:24 +0000 [c46100ad5f]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-24907 #close
-	  Reported by: Kevin Harwell
+	* res_pjsip_phoneprov_provider: Add missing dependency on pjproject.
+	  ........
 
-	  Change-Id: Ibb2fe558f98190f2a06da830e0fadfa25516f547
+	  Merged revisions 425216 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-06-26 10:41 +0000 [05a2cc1293]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 425217 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_refer: Prevent sending duplicate headers.
 
-	  res_pjsip_refer will attempt to add Referred-By or Replaces headers to
-	  outbound INVITEs at times. If the INVITE gets challenged for
-	  authentication, then we will resend the INVITE. Prior to this patch, the
-	  Referred-By or Replaces header would be re-added to the outbound INVITE,
-	  resulting in duplicated headers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425218 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25204 #close
-	  Reported by Mark Michelson
+2014-10-10 08:03 +0000 [37b5f52da7]  Kinsey Moore
 
-	  Change-Id: I59fb5c08b4d253c0dba9ee3d3950b5025358222d
+	* CallerID: Fix parsing regression
 
-2015-06-23 17:43 +0000 [028fa54620]  Mark Michelson <mmichelson@digium.com>
+	  This fixes a regression in callerid parsing introduced when another bug
+	  was fixed. This bug occurred when the name was composed entirely of
+	  DTMF keys and quoted without a number section (<>). 
 
-	* res_pjsip_nat: Rewrite route set when required.
+	  ASTERISK-24406 #close
+	  Reported by: Etienne Lessard
+	  Tested by: Etienne Lessard
+	  Patches:
+	      callerid_fix.diff uploaded by Kinsey Moore
+	  Review: https://reviewboard.asterisk.org/r/4067/
+	  ........
 
-	  When performing some provider testing, the rewrite_contact option was
-	  interfering with proper construction of a route set when sending an ACK
-	  after receiving a 200 OK response to an INVITE.
+	  Merged revisions 425152 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  The initial INVITE was sent to address sip:foo. The 200 OK had a Contact
-	  header with URI sip:bar. In addition, the 200 OK had Record-Route
-	  headers for sip:baz and sip:foo, in that order. Since the Record-Route
-	  headers had the lr parameter, the result should have been:
+	  Merged revisions 425153 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * Set R-URI of the ACK to sip:bar.
-	  * Add Route headers for sip:foo and sip:baz, in that order.
+	  Merged revisions 425154 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  However, the rewrite_contact option resulted in our rewriting the
-	  Contact header on the 200 OK to sip:foo. The result was:
+	  Merged revisions 425155 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * R-URI remained sip:foo.
-	  * We added Route headers for sip:foo and sip:baz, in that order.
 
-	  The result was that sip:bar was not indicated in the ACK at all, so the
-	  far end never received our ACK. The call eventually dropped.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425156 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The intention of rewrite_contact is to rewrite the most immediate
-	  destination of our SIP request to be the same address on which we
-	  received a request or response. In the case of processing a SIP response
-	  with Record-Route headers, this means that instead of rewriting the
-	  Contact header, we should instead rewrite the bottom-most Record-Route
-	  header. In the case of processing a SIP request with Record-Route
-	  headers, this means we rewrite the top-most Record-route header.
-	  Like when we rewrite the Contact header, we also ensure to update
-	  the dialog's route set if it exists.
+2014-10-10 07:10 +0000 [0ef680cff0]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25196 #close
-	  Reported by Mark Michelson
+	* res_pjsip_nat: Place source port into rport of responses if 'force_rport' is on.
 
-	  Change-Id: I9702157c3603a2d0bd8a8215ac27564d366b666f
-2015-06-19 16:16 +0000 [84c12f9e0c]  Richard Mudgett <rmudgett@digium.com>
+	  When the 'force_rport' option is enabled the behavior should be the same
+	  as if the remote side placed rport into the message themselves. Therefore
+	  any responses we send should include the source port of the request in the
+	  rport of the Via header.
 
-	* threadpool, res_pjsip: Add serializer group shutdown API calls.
+	  #SIPit31
 
-	  A module trying to unload needs to wait for all serializers it creates and
-	  uses to complete processing before unloading.
+	  ASTERISK-24387 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  ASTERISK-24907
-	  Reported by: Kevin Harwell
+	  Merged revisions 425131 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I8c80b90f2f82754e8dbb02ddf3c9121e5e966059
+	  Merged revisions 425132 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-16 15:06 +0000 [602c4b74b5]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_outbound_registration.c: Fix handle_client_state_destruction() refs
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425133 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * handle_client_state_destruction() must always be passed a ref to
-	  client_state because it will always unref client_state.
-	  handle_registration_response() was not passing a client_state ref.
+2014-10-10 02:34 +0000 [d3f525fd8f]  Torrey Searle (License #5334),Nitesh Bansal (License #6418)
 
-	  * Made the final un-REGISTER message get sent normally using the pjproject
-	  register control structure in handle_client_state_destruction().  The
-	  previous code attempted to short circuit the response handling for the
-	  module to unload.  That doesn't work for a couple reasons.  One,
-	  pjsip_regc_send() may call the registered callback before it returns and
-	  unbalance the client_state ref count.  Two, the registered callback
-	  handles any authentication for the un-REGISTER message.
+	* chan_sip: Fix dialog leak resulting from missing ACK to re-INVITE.
 
-	  * Made the distinction between internal registration state and external
-	  registration status with sip_outbound_registration_status_str().  This is
-	  necessary to avoid altering documented AMI messages with internal
-	  changes.
+	  If a device re-INVITEs at the same time as the dialog is hung up, and
+	  if then the ACK to the re-INVITE never reaches Asterisk, chan_sip would
+	  fail to destroy the dialog after a while.  This resulted in (most
+	  prominently) file handle leaks.
 
-	  * Removed references to client_state->client outside of the serializer
-	  thread.  When handle_client_state_destruction() destroys the pjproject
-	  register control structure that memory is freed and cannot be referenced
-	  anymore.  These accesses were to provide information for debug and
-	  off-nominal warning messages.
+	  (Patch reindented by me.)
 
-	  * In sip_outbound_registration_timer_cb() you should not access entry->id
-	  after unrefing client_state because the passed in entry is normally
-	  pointing to the timer entry in the client_state object.
+	  ASTERISK-20784 #close
+	  ASTERISK-15879 #close
+	  Reported by: Torrey Searle, Nitesh Bansal
+	  Patches:
+	    reinvite_ack_timeout.patch uploaded by Torrey Searle (License #5334)
+	    patch_asterisk_20784.txt uploaded by Nitesh Bansal (License #6418)
 
-	  ASTERISK-24907
-	  Reported by: Kevin Harwell
+	  Reviewboard: https://reviewboard.asterisk.org/r/4052/
+	  (testcase can be found at r4051)
+	  ........
 
-	  Change-Id: Ia7b446d8644b6b4550ef5bea49527671de65183f
+	  Merged revisions 425068 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-06-15 15:28 +0000 [8c6a95a9ac]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 425069 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_pjsip_outbound_registration.c: Use ast_sorcery_object_unregister() API
+	  Merged revisions 425070 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The sorcery pjsip 'registration' config object needs to be destroyed on
-	  module unload.  Otherwise, a reload of res_pjsip could try to use
-	  callbacks for a previously unloaded instance of the module provided by
-	  ast_sorcery_object_register() or one of the variants.  Also, if
-	  res_pjsip_outbound_registration were subsequently reloaded, the sorcery
-	  config field objects would be registered in sorcery twice.
+	  Merged revisions 425071 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24907
-	  Reported by: Kevin Harwell
 
-	  Change-Id: I304fad13dece2604af48353f6c6d9d5c7b064697
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425072 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-25 06:42 +0000 [e4a2ef9e4e]  Joshua Colp <jcolp@digium.com>
+2014-10-09 18:37 +0000 [aef63118da]  gtjoseph <george.joseph@fairview5.com>
 
-	* channel: Remove ignore of answer on non-outgoing channels.
+	* res_pjsip_phoneprov_provider: fix compile breakage on AST_VECTOR
 
-	  Due to the way that channels can now be moved around inside of
-	  Asterisk it is possible for the outgoing flag of a channel to get
-	  cleared before it has been answered. This results in the bridge
-	  not receiving notification that the outgoing leg has been answered.
+	  endpoint->inbound_auths was changed to a vector in 13 and I
+	  committed the 12 patch instead of the 13 patch.
 
-	  This most easily exhibits itself with DTMF based blond transfers.
-	  Since the answer of the outgoing leg is ignored the other party
-	  continues to receive both a locally generated ringing and the
-	  media stream of the outgoing leg upon its answer. This results
-	  in no media being heard.
+	  Tested-by: George Joseph
+	  ........
 
-	  This change removes the ignore of the answer and allows it
-	  to pass through.
+	  Merged revisions 425052 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25171 #close
 
-	  Change-Id: I82aedcec4f89f34a2e5472086dfc9a6c775bca8e
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425053 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-15 15:28 +0000 [20f3d77ab9]  Richard Mudgett <rmudgett@digium.com>
+2014-10-09 16:39 +0000 [6fc4df7279]  Kevin Harwell <kharwell@digium.com>
 
-	* sorcery: Add ast_sorcery_object_unregister() API call.
+	* res_rtp_asterisk: Crash if no candidates received for component
 
-	  Find and unlink the specified sorcery object type to complement
-	  ast_sorcery_object_register().  Without this function you cannot
-	  completely unload individual modules that use sorcery for configuration.
+	  When starting ice if there is not at least one remote ice candidate with an RTP
+	  component asterisk will crash. This is due to an assertion in pjnath as it
+	  expects at least one candidate with an RTP component. Added a check to make
+	  sure at least one candidate contains an RTP component and at least one candidate
+	  has an RTCP component.
 
-	  ASTERISK-24907
-	  Reported by: Kevin Harwell
+	  ASTERISK-24383 #close
+	  Review: https://reviewboard.asterisk.org/r/4039/
+	  ........
 
-	  Change-Id: I1c04634fe9a90921bf676725c7d6bb2aeaab1c88
+	  Merged revisions 425031 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-15 13:38 +0000 [4313f32969]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_outbound_registration.c: Reorder load_module() and unload_module().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425032 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  It is best if the loading code creates and initializes the module's
-	  infrastructure before letting the system know of its existence.  The
-	  unloading code needs to reverse the actions of the loading code and in the
-	  reverse order.
+2014-10-09 15:55 +0000 [c6837c236f]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-24907
-	  Reported by: Kevin Harwell
+	* res_pjsip_phoneprov_provider: Provides pjsip integration with res_phoneprov
 
-	  Change-Id: I5d151383e9787b5b60aa5e1627b10f040acdded4
+	  This module allows res_pjsip to integrate with res_phoneprov.  It handles
+	  the pjsip 'phoneprov' object type.
 
-2015-06-23 14:34 +0000 [890c923786]  Richard Mudgett <rmudgett@digium.com>
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3976/
+	  ........
 
-	* AMI: Add Linkedid to the standard channel snapshot AMI event headers.
+	  Merged revisions 425007 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * The AMI version is bumped to 2.8.0.
+	  Merged revisions 425008 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25189 #close
-	  Reported by: John Hardin
 
-	  Change-Id: I2b1778c3fdc1dca0ed55db4e3a639eddfb16c2ac
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@425009 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-24 14:30 +0000 [2602a7484b]  Richard Mudgett <rmudgett@digium.com>
+2014-10-09 13:44 +0000 [3a187aa14a]  Matt Jordan <mjordan@digium.com>
 
-	* test.c: Add unit test registration checks for summary and description.
+	* res/res_phoneprov: Don't cancel Asterisk load on module load failure
+	  ........
 
-	  Added checks when a unit test is registered to see that the summary and
-	  description strings do not end with a new-line '\n' for consistency.
+	  Merged revisions 424985 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The check generates a warning message and will cause the
-	  /main/test/registrations unit test to fail.
+	  Merged revisions 424986 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Updated struct ast_test_info member doxygen comments.
 
-	  Change-Id: I295909b6bc013ed9b6882e85c05287082497534d
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424987 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-24 14:39 +0000 [2b0482d699]  Richard Mudgett <rmudgett@digium.com>
+2014-10-09 12:46 +0000 [cc595f7353]  gtjoseph <george.joseph@fairview5.com>
 
-	* Unit tests: Fix unit test description strings.
+	* res_phoneprov: Refactor phoneprov to allow pluggable config providers
 
-	  Analyzing the code shows that the unit test summary and description
-	  strings should not end with a new-line character.  Where these strings are
-	  used in the code a new-line is provided for output.
+	  This patch makes res_phoneprov more modular so other modules (like pjsip)
+	  can provide configuration information instead of res_phoneprov relying solely
+	  on users.conf and sip.conf.  To accomplish this a new ast_phoneprov public API
+	  is now exposed which allows config providers to register themselves, set
+	  defaults (server profile, etc) and add user extensions.
 
-	  Change-Id: I129284f5e7ca93d82532334076da4c462d3d9fba
+	  * ast_phoneprov_provider_register registers the provider and provides callbacks
+	    for loading default settings and loading users.
+	  * ast_phoneprov_provider_unregister clears the defaults and users.
+	  * ast_phoneprov_add_extension should be called once for each user/extension
+	    by the provider's load_users callback to add them.
+	  * ast_phoneprov_delete_extension deletes one extension.
+	  * ast_phoneprov_delete_extensions deletes all extensions for the provider.
 
-2015-06-23 11:21 +0000 [e99e654d75]  Joshua Colp <jcolp@digium.com>
+	  Tested-by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3970/
+	  ........
 
-	* app_dial: Hold reference to calling channel formats when dialing outbound.
+	  Merged revisions 424963 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Currently when requesting a channel the native formats of the
-	  calling channel are provided to the core for usage when dialing
-	  the outbound channel. This occurs without holding the channel lock
-	  or keeping a reference to the formats. This is problematic as
-	  the channel driver may end up changing the formats during this time.
-	  In the case of chan_sip this happens when an SDP negotiation
-	  completes.
+	  Merged revisions 424964 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change makes it so app_dial keeps a reference to the native
-	  formats of the calling channel which guarantees that they will
-	  remain valid for the period of time needed.
 
-	  ASTERISK-25172 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424965 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I2f0a67bd0d5d14c3bdbaae552b4b1613a283f0db
-2015-06-17 05:04 +0000 [80e82dc97f]  Joshua Colp <jcolp@digium.com>
+2014-10-09 11:38 +0000 [0f50e8856b]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_mwi: Set up unsolicited MWI upon registration.
+	* cdr.c: Make turning on CDR debug a one step process instead of two.
 
-	  The res_pjsip_mwi previously required a reload to set up the proper
-	  subscriptions to allow unsolicited MWI to work. This change
-	  makes it so the act of registering will also cause this to occur.
-	  This is particularly useful if realtime is involved as no reload
-	  needs to occur within Asterisk to cause the MWI information
-	  to get sent.
+	  Now "cdr set debug on" doesn't also require "core set verbose 1" to see
+	  CDR debug output.
+	  ........
 
-	  ASTERISK-25180 #close
+	  Merged revisions 424941 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Id847b47de4b8b3ab8858455ccc2f07b0f915f252
+	  Merged revisions 424942 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-22 15:11 +0000 [35a99b6394]  Kevin Harwell <kharwell@digium.com>
 
-	* bridge.c: Hangup attended transfer target if bridged
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424943 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  After completing an attended transfer the transfer target channel was not being
-	  hung up after leaving the bridge. Added an explicit softhangup to hangup said
-	  channel, but only if it was previously bridged.
+2014-10-09 03:10 +0000 [d0255c4a46]  Michael Myles (License #6626)
 
-	  ASTERISK-24782 #close
-	  Reported by: John Bigelow
+	* safe_asterisk: Don't automatically exceed MAXFILES value of 2^20.
 
-	  Change-Id: Idde9543d56842369384a5e8c00d72a22bbc39ada
+	  On systems with lots of RAM (e.g. 24GB) /proc/sys/fs/file-max divided
+	  by two can exceed the per-process file limit of 2^20. This patch
+	  ensures the value is capped.
 
-2015-06-17 16:23 +0000 [036bc0012f]  Richard Mudgett <rmudgett@digium.com>
+	  (Patch cleaned up by me.)
 
-	* res_pjsip_outbound_registration.c: Add missing line endings to CLI commands
+	  ASTERISK-24011 #close
+	  Reported by: Michael Myles
+	  Patches:
+	    safe_asterisk-ulimit.diff uploaded by Michael Myles (License #6626)
+	  ........
 
-	  Change-Id: I39ae612746d892d2dbe86f3ff2d7027fa1da57f7
+	  Merged revisions 424875 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-2015-06-12 14:29 +0000 [bec7435945]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 424878 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_pjsip_outbound_registration.c: Eliminate simple RAII_VAR() usage.
+	  Merged revisions 424879 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I399cb9d61bbba706b48c98e0bf75e98984cd9a9e
+	  Merged revisions 424880 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-12 13:33 +0000 [c2519fdf1c]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_outbound_registration.c: Misc code cleanups.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424881 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Break some long lines.
+2014-10-08 13:47 +0000 [8b0089ea1d]  Joshua Colp <jcolp@digium.com>
 
-	  * Fix doxygen comment.
+	* res_rtp_asterisk: Allow only UDP ICE candidates.
 
-	  Change-Id: I8f12ba6822f84d5e7bb575280270cd7e2fefb305
+	  The underlying library, pjnath, that res_rtp_asterisk uses for ICE
+	  support does not have support for ICE-TCP. As candidates are
+	  passed through directly to it this can cause error messages to occur
+	  when it receives something unexpected (such as a TCP candidate).
+	  This change merely ignores all non-UDP candidates so they never
+	  reach pjnath.
 
-2015-06-22 09:26 +0000 [a419c69def]  Alexander Traud (License 6520)
+	  ASTERISK-24326 #close
+	  Reported by: Joshua Colp
+	  ........
 
-	* chan_sip: Reload peer without its old capabilities.
+	  Merged revisions 424852 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  On reload, previously allowed codecs were not removed. Therefore, it was not
-	  possible to remove codecs while Asterisk was running. Furthermore, newly added
-	  codecs got appended behind the previous codecs. Therefore, it was not possible
-	  to add a codec with a priority of #1. This change removes the old capabilities
-	  before the current ones are added.
+	  Merged revisions 424853 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25182 #close
-	  Reported by: Alexander Traud
-	  patches:
-	   asterisk_13_allow_codec_reload.patch uploaded by Alexander Traud (License 6520)
+	  Merged revisions 424854 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I62a06bcf15e08e8c54a35612195f97179ebe5802
 
-2015-06-20 19:38 +0000 [74616ae43d]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424855 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: Destroy peers without holding peers container lock.
+2014-10-08 13:24 +0000 [5e50638539]  Kinsey Moore <kmoore@digium.com>
 
-	  Due to the use of stasis_unsubscribe_and_join in the peer destructor
-	  it is possible for a deadlock to occur when an event callback is
-	  occurring at the same time.
+	* Stasis: Relegate log message to dev-mode
 
-	  This happens because the peer may be destroyed while holding the
-	  peers container lock. If this occurs the event callback will never
-	  be able to acquire the container lock and the unsubscribe will
-	  never complete.
+	  This error message primarily applies to development tasks and will now
+	  only show up when dev-mode is enabled via configure.
+	  ........
 
-	  This change makes it so the peers that have been removed from the
-	  peers container are not destroyed with the container lock held.
+	  Merged revisions 424850 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25163 #close
 
-	  Change-Id: Ic6bf1d9da4310142a4d196c45ddefb99317d9a33
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424851 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-18 13:16 +0000 [9015bb4c8c]  Mark Michelson <mmichelson@digium.com>
+2014-10-08 09:54 +0000 [3dfc485e35]  Kinsey Moore <kmoore@digium.com>
 
-	* Resolve race conditions involving Stasis bridges.
+	* Indexer: Format message types may not exist
 
-	  This resolves two observed race conditions.
+	  In Asterisk 13+, any given message type is not guaranteed to exist even
+	  if Asterisk comes up correctly since creation of the message type could
+	  be declined. The indexer should not prevent Asterisk from starting
+	  under these conditions.
+	  ........
 
-	  First, a bit of background on what the Stasis application does:
+	  Merged revisions 424833 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  1a Creates a stasis_app_control structure. This structure is linked into
-	     a global container and can be looked up using a channel's unique ID.
-	  2a Puts the channel in an event loop. The event loop can exit either
-	     because the stasis_app_control structure has been marked done, or
-	     because of some other factor, such as a hangup. In the event loop, the
-	     stasis_app_control determines if any specific ARI commands need to be
-	     run on the channel and will run them from this thread.
-	  3a Checks if the channel is bridged. If the channel is bridged, then
-	     ast_bridge_depart() is called since channels that are added to Stasis
-	     bridges are always imparted as departable.
-	  4a Unlink the stasis_app_control from the container.
 
-	  When an ARI command is received by Asterisk, the following occurs
-	  1b A thread is spawned to handle the HTTP request
-	  2b The stasis_app_control(s) that corresponds to the channel(s) in the
-	     request is/are retrieved. If the stasis_app_control cannot be
-	     retrieved, then it is assumed that the channel in question has exited
-	     the Stasis app or perhaps was never in Stasis in the first place.
-	  3b A command is queued onto the stasis_app_control, and the channel's
-	     event loop thread is signaled to run the command.
-	  4b While most ARI commands do nothing further, some, such as adding or
-	     removing channels from a bridge, will block until the command they
-	     issued has been completed by the channel's event loop.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424834 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The first race condition that is solved by this patch involves a crash
-	  that can occur due to faulty detection of the channel's bridged status
-	  in step 3a. What can happen is that in step 2a, the event loop may run
-	  the ast_bridge_impart() function to asynchronously place the channel
-	  into a bridge, then immediately exit the event loop because the channel
-	  has hung up. In step 3a, we would detect that the channel was not
-	  bridged and would not call ast_bridge_depart(). The reason that the
-	  channel did not appear to be bridged was that the depart_thread that is
-	  spawned by ast_bridge_impart() had not yet started. That is the thread
-	  where the channel is marked as being bridged. Since we did not call
-	  ast_bridge_depart(), the Stasis application would exit, and then the
-	  channel would be destroyed Then the depart_thread would start up and
-	  try to manipulate the destroyed channel, causing a crash.
+2014-10-07 15:33 +0000 [d8bbf1ec1d]  Kinsey Moore <kmoore@digium.com>
 
-	  The fix for this is to switch from using ast_channel_is_bridged() to
-	  checking the NULLity of ast_channel_internal_bridge_channel() to
-	  determine if ast_bridge_depart() needs to be called. The channel's
-	  internal bridge_channel is set when ast_bridge_impart() is called and
-	  is NULLed by the call to ast_bridge_depart(). If the channel's internal
-	  bridge_channel is non-NULL, then the channel must have been imparted
-	  into the bridge and needs to be departed, even if the actual bridging
-	  operation has not yet started. By departing the channel when necessary,
-	  the thread that is running the Stasis application will block until the
-	  bridge gives the okay that the depart_thread has exited.
+	* Stasis: Only log errors for non-declined types
 
-	  The second race condition that is solved by this patch involves a leak
-	  of HTTP handler threads. The problem was that step 2b would successfully
-	  retrieve a stasis_app_control structure. Then step 2a would exit the
-	  channel from the event loop due to a hangup. Steps 3a and 4a would
-	  execute, and then finally steps 3b and 4b would. The problem is that at
-	  step 4b, when attempting to add a channel to a bridge, the thread would
-	  block forever since the channel would never execute the queued command
-	  since it was finished with the event loop. This meant that the HTTP
-	  handling thread would be leaked, along with any references that thread
-	  may have owned (in my case, I was seeing bridges leaked).
+	  When message type creation is declined via stasis.conf, certain
+	  operations log errors assuming that the declined type is being used
+	  before initialization or after destruction. These error messages get
+	  quite spammy for oft used message types and should not be logged in the
+	  first place since the message type is validly NULL.
 
-	  The fix for this is to hone in better on when the channel has exited the
-	  event loop. The stasis_app_control structure has an is_done field that
-	  is now set at each point where the channel may exit the event loop. If
-	  step 2b retrieves a valid stasis_app_control structure but the control
-	  is marked as done, then the attempted operation exits immediately since
-	  there will be nothing to service the attempted command.
+	  Reported by: Matt DiMeo
+	  ........
 
-	  ASTERISK-25091 #close
-	  Reported by Ilya Trikoz
+	  Merged revisions 424769 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: If66265b73b4c9f8f58599124d777fedc54576628
-2015-06-16 11:13 +0000 [723a9d4225]  Mark Michelson <mmichelson@digium.com>
 
-	* Parking: Add documentation for AMI ParkedCallSwap event.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424770 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This event was added some time ago in order to clarify when a channel
-	  took the place of another channel in a parking lot. However, there was
-	  no XML documentation added for the event. This patch adds the XML
-	  documentation.
+2014-10-07 13:34 +0000 [f7225da08a]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-24900 #close
-	  Reported by Rusty Newton
+	* data: Properly access formats in capabilities structure when adding codecs.
 
-	  Change-Id: I4cfe7777c4b94bbff91c9221c6096a7a02a92eac
-2015-06-15 16:40 +0000 [79bf56c78a]  Corey Farrell <git@cfware.com>
+	  Formats within a capabilities structure are addressed starting at 0, not 1.
+	  Assuming 1 causes it to exceed an array.
 
-	* func_pjsip_aor: Fix leaked contact from iterator.
+	  ASTERISK-24389 #close
+	  Reported by: Kevin Harwell
+	  ........
 
-	  ASTERISK-25162 #close
+	  Merged revisions 424752 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Id79aa3c6fe490016ee98efc97ac4c1d3f461f97e
 
-2015-06-12 16:58 +0000 [31c77b157b]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424753 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Add option to force G.726 to be treated as AAL2 packed.
+2014-10-07 12:44 +0000 [a9011106b6]  Matt Jordan <mjordan@digium.com>
 
-	  Some phones send g.726 audio packed for AAL2, which differs from what is
-	  recommended by RFC 3351. If Asterisk receives audio formatted as such when
-	  negotiating g.726 then it sounds a bit distorted. Added an option to
-	  res_pjsip_endpoint that allows g.726 negotiated audio to be treated as g.726
-	  AAL2 packed.
+	* res/res_pjsip_outbound_registration: Initialize auth_reject_permanent parameter
 
-	  ASTERISK-25158 #close
-	  Reported by: Steve Pitts
+	  Prior to this patch, the auth_reject_permanent parameter was not initialized on
+	  the registration client state, leading to the parameter being disabled
+	  regardless of the value specified in pjsip.conf.
 
-	  Change-Id: Ie7e21f75493d7fe53e75e12c971e72f5afa33615
+	  This patch initialized the setting on the registration client state to the
+	  provided configuration value.
 
-2015-06-14 19:48 +0000 [de8c7f46ed]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24398 #close
+	  ........
 
-	* main/cdr: Carry over the disable flag when 'disable all' is specified
+	  Merged revisions 424730 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The CDR_PROP function (as well as the NoCDR application) set the
-	  'disable all' flag (AST_CDR_FLAG_DISABLE_ALL) on the current CDR. This
-	  flag is supposed to be applied to all CDRs that are currently in the
-	  chain, as well as all CDRs that may be created in the future. Currently,
-	  however, the flag is only applied to the existing CDRs in the chain; new
-	  CDRs do not receive the 'disable all' flag. In particular, this affects
-	  parallel dials, which generate new CDRs for each pair of channels in
-	  the dial attempt.
+	  Merged revisions 424731 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch carries over the 'disable all' flag when it is specified on a
-	  CDR and a new CDR is generated for the chain.
 
-	  ASTERISK-24344 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424732 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I91a0f0031e4d147bdf8a68ecd08304d506fb6a0e
-2015-06-12 14:28 +0000 [78ea356e78]  Matt Jordan <mjordan@digium.com>
+2014-10-07 09:09 +0000 [523da7d1b3]  Matt Jordan <mjordan@digium.com>
 
-	* main/cdr: Copy context/exten on chained CDRs for parallel dials in subroutines
+	* res/res_pjsip_pubsub: Fix typo in WARNING message
+	  ........
 
-	  When a parallel dial occurs, a new CDR will be created for each dial
-	  attempt that is made. In most circumstances, the act of creating each
-	  CDR in the chain will include a step that updates the Party A snapshot,
-	  which causes the context/extension of the Party A to be copied onto the
-	  CDR object.
+	  Merged revisions 424713 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  However, when the Party A is in a subroutine, we explicitly do *not*
-	  copy the context/extension onto the CDR. This prevents the Macro or
-	  GoSub routine name from blowing away the context/extension that the
-	  channel was originally executing in. For the original CDR, this is not a
-	  problem: the original CDR already recorded the last known 'good' state
-	  of the channel just prior to it going into the subroutine. However, for
-	  newly generated CDRs in a chain, there is no context/extension set on
-	  them. Since we are in a subroutine, we will never set the Party A's
-	  context/extension on the CDR, and we end up with a CDR with no
-	  destination recorded on it.
 
-	  This patch updates the creation of a chained CDR such that it copies
-	  over the original CDR's context/extension. This is the last known "good"
-	  state of the CDR, and is a reasonable starting point for the newly
-	  generated CDR. In the case where we are not in a subroutine, subsequent
-	  code will update the location of the CDR from the Party A information;
-	  in the case where we are in a subroutine, the context/extension on the
-	  original CDR is the correct information.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424714 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24443 #close
+2014-10-06 13:39 +0000 [39bd5b7a70]  Peter Katzmann (License 5968)
 
-	  Change-Id: I6a3ef0d6e458d3b9b30572feaec70f2964f3bc2a
+	* message: Don't close an AMI connection on SendMessage action error
 
-2015-06-11 08:18 +0000 [3f57f3f8ec]  Damian Ivereigh <damo@launtel.net.au>
+	  If SendMessage encounters an error (such as incorrect input provided to the
+	  action), it will currently return -1. Actions should only return -1 if the
+	  connection to the AMI client should be closed. In this case, SendMessage
+	  causing the client to disconnect is inappropriate.
 
-	* chan_sip.c: Update dialog fromtag after request with auth
+	  This patch causes the action to return 0, which simply causes the action to
+	  fail.
 
-	  If a client sends and INVITE which is 401 rejected, then subsequently
-	  sends a new INVITE with the auth info and uses a different fromtag
-	  from the first INVITE, Asterisk will accept the new INVITE as part of
-	  the original dialog - match_req_to_dialog() specifically ignores the
-	  fromtag. However it does not update the stored dialog with the new
-	  fromtag.
+	  Review: https://reviewboard.asterisk.org/r/4024
 
-	  This results in Asterisk being unable to match future packets that are
-	  part of this dialog (such as the ACK to the OK or the OK to the BYE),
-	  and the call is dropped.
+	  ASTERISK-24354 #close
+	  Reported by: Peter Katzmann
+	  patches:
+	    sendMessage.patch uploaded by Peter Katzmann (License 5968)
+	  ........
 
-	  This problem was originally found when using an NEC-i SV8100-GE (NEC SIP
-	  Card).
+	  Merged revisions 424690 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  * After a successful match of a packet to the dialog, if the packet is
-	    not a SIP_RESPONSE, authentication is present and the fromtags are
-	    different, the stored fromtag is updated with the one from the recent
-	    INVITE.
+	  Merged revisions 424691 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25154 #close
-	  Reported by: Damian Ivereigh
-	  Tested by: Damian Ivereigh
+	  Merged revisions 424692 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I5c16cf3b409e5ef9f2b2fe974b6bd2a45a6aa17e
 
-2015-06-11 18:52 +0000 [30a0f2d9ac]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424693 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_pjsip: Set the context and extension on the channel when created
+2014-10-06 10:41 +0000 [c384532aa4]  Richard Mudgett <rmudgett@digium.com>
 
-	  Prior to this patch, chan_pjsip was failing to pass the endpoint's
-	  context and the desired extension to the ast_channel_alloc_* routine.
-	  This caused a new channel snapshot to be issued without a context and
-	  extension, which can cause some reporting issues for users of AMI, CEL,
-	  and other APIs. The channel driver would later set the context and
-	  extension on the channel such that the channel would start in the
-	  correct location in the dialplan, but the information reported in the
-	  initial event would be incorrect.
+	* features.c: Fix lingering channel ref while Bridge() application is active.
 
-	  This patch modifies the channel driver such that it now passes the
-	  context and extension directly into the allocation routine. This
-	  provides the information in the new channel snapshot published over
-	  Stasis.
+	  Using the Bridge application to bridge a channel that is executing an
+	  applicaiton such as Wait results in a lingering Surrogate channel in the
+	  CLI "core show channels" output even though it has already hungup.
 
-	  ASTERISK-25156 #close
-	  Reported by: cloos
+	  * Fix bridge_exec() to not hold onto the current_dest_chan ref once it has
+	  been put into the bridge.
 
-	  Change-Id: Ic6f8542836e596db8f662071d118e8f934fdf25e
+	  * Eliminated bridge_exec()'s use of RAII_VAR().
 
-2015-06-10 18:28 +0000 [dbb067279e]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24224 #close
+	  Reported by: Mark Michelson
 
-	* bridge: When performing a blonde transfer update connected line information.
+	  Review: https://reviewboard.asterisk.org/r/4041/
+	  ........
 
-	  When performing a blonde transfer the code uses the old masquerade
-	  mechanism to move a channel around. As a result of this certain information,
-	  such as connected line, is moved between the channels involved. Upon
-	  completion of the move a frame is queued which is supposed to update the
-	  connected line information on the channel. This does not occur as the
-	  code considers it a redundant update since the masquerade operation
-	  updated the channel (but did not inform it of the new connected line
-	  information). The code also does not queue a connected line update
-	  to be handled by the thread handling the channel. Without this any
-	  other channel that may be loosely involved does not know it is
-	  talking to a different caller.
+	  Merged revisions 424668 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change does the following to resolve this:
+	  Merged revisions 424669 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  1. The indicated connected line information is cleared upon
-	  completion of the masquerade operation when doing a blonde transfer.
-	  This prevents the connected line update from being considered
-	  redundant.
 
-	  2. A connected line update frame is now queued upon the completion
-	  of the masquerade operation so any other channel loosely involved
-	  knows that there is a different caller.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424670 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25157 #close
-	  Reported by: Joshua Colp
+2014-10-06 07:39 +0000 [3a87f32dc0]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Ibb8798184a1dab3ecd35299faecc420034adbf20
+	* sdp_srtp: Add new lines to some WARNING messages
+	  ........
 
-2015-06-11 14:39 +0000 [a2f4d03c87]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 424646 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* app_directory: Fix crash when using the alias option 'a'.
+	  Merged revisions 424647 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The voicemail.conf mailbox key/value pair is defined as:
-	  <mailbox>=[<password>[,<full-name>[,<email>[,<pager>[,<options>]]]]]
-	  Where all fields in the value including the field values are optional.
 
-	  Since the parsing code for the mailbox key/value pair is sloppy, this
-	  patch tightens the parsing for the directory information.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424648 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Renamed the 'pos' and 'bufptr' variables to 'name' and 'options'
-	  respectively in search_directory_sub().  Those names make more sense.
+2014-10-05 20:01 +0000 [cce3d99ec8]  Matt Jordan <mjordan@digium.com>
 
-	  * Made sure that search_directory_sub() is dealing with the voicemail.conf
-	  mailbox options field if it even exists when looking for the 'hidefromdir'
-	  and 'alias' options.
+	* res_pjsip/pjsip_options: Do not 404 an OPTIONS request not sent to an endpoint
 
-	  * Fix crash if a voicemail.conf mailbox is just
-	  <mailbox>=<password>,<name> when the 'a' option is used.  If there were no
-	  fields after the name then the 'options' pointer was not checked for NULL.
+	  An OPTIONS request that is sent to Asterisk but not to a specific endpoint is
+	  currently sent a 404 in response. This is because, not surprisingly, an empty
+	  extension is never going to be found in the dialplan.
 
-	  * Fix users.conf alias processing if the 'a' option is used.  The wrong
-	  variable was used.
+	  This patch makes it so that we only attempt to look up the endpoint in the
+	  dialplan if it is specified in the OPTIONS request URI.
 
-	  ASTERISK-25087 #close
-	  Reported by: Chet Stevens
+	  #SIPit31
 
-	  Change-Id: I86052ea77307beddddba5279824d39dc0d593374
+	  ASTERISK-24370 #close
+	  Reported by: Matt Jordan
+	  ........
 
-2015-06-09 15:31 +0000 [a2b718f4f6]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 424624 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res_pjsip.h: Fix some doxygen comments.
+	  Merged revisions 424625 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I4615771077c3c6a0a7273da6d7b5f77af7e8d976
 
-2015-06-05 13:46 +0000 [32ddf6d86b]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424626 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* taskprocessor.c: Remove extra unref from off-nominal path.
+2014-10-05 19:53 +0000 [c013916869]  Matt Jordan <mjordan@digium.com>
 
-	  Change-Id: Iee3bd8c8a528776056972066698fe735f0f6cf60
+	* pjsip/dialplan_functions: Handle PJSIP_MEDIA_OFFER called on non-PJSIP channels
 
-2015-04-20 16:00 +0000 [cf98c744d5]  Yousf Ateya <y.ateya@starkbits.com>
+	  Calling PJSIP_MEDIA_OFFER on a non-PJSIP channel is hazardous to your health.
+	  It will treat the channels as a PJSIP channel, eventually hitting an ao2 error,
+	  FRACKing on assertion error, and quite likely crashing.
 
-	* chan_iax2: Prevent deadlock between hangup and sending lagrq/ping
+	  This patch adds checks to the read/write callbacks that ensure that the channel
+	  technology is of type 'PJSIP' before attempting to operate on the channel.
 
-	  channels/chan_iax.c: Prevent the deadlock between iax2_hangup and send_lagrq/
-	  send_ping. This deadlock happens because the scheduled task send_lagrq(or
-	  send_ping) starts execution after the call hangup procedure starts but before
-	  it deletes the tasks in the scheduler.
+	  #SIPit31
 
-	  The solution is to delete scheduled lagrq (and ping) task asynchronously
-	  (i.e. schedule AST_SCHED_DEL for these tasks); By this, AST_SCHED_DEL will
-	  be called in a new context (doesn't have callno locked).
+	  ASTERISK-24382 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  This commit also cleans up the procedure of sending LAGRQ and PING.
+	  Merged revisions 424621 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  main/sched.c: Do not assert when deleting non existant entry from scheduler.
-	  This assert seems to be the reason for a lot of awkward code to avoid it.
+	  Merged revisions 424622 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24983 #close
-	  Reported by: Y Ateya
 
-	  Change-Id: I03bec1fc8faacb89630269e935fa667c6d6c080c
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424623 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-31 12:37 +0000 [8af6c9cf6b]  Ivan Poddubny <ivan.poddubny@gmail.com>
+2014-10-05 19:31 +0000 [45b7b474ac]  Matt Jordan <mjordan@digium.com>
 
-	* res_pjsip_transport_websocket: Fix use-after-free bugs.
+	* res_pjsip: Prevent crashes when PJPROJECT presents an rdata with no message
 
-	  This patch fixes use-after-free bugs caught by AddressSanitizer.
+	  When a message that exceeds the PJ_MAX_PKT_SIZE is sent over a reliable
+	  transport, it is possible (although it shouldn't occur) for pjproject to pass
+	  up an rdata object with a NULL msg in the msg_info. Needless to say, things
+	  that attempt to dereference this are in for a rough ride.
 
-	  1. PJSIP transport manager may decide to destroy transport on its own.
-	  For example, when the contact registered via websocket has not renewed
-	  its registration in time. The transport was destoyed, but the websocket
-	  listener thread was still active until the socket closes, and then tried
-	  to call transport_shutdown on transport that has been freed.
+	  In particular, this caused crashes in three different locations, all of which
+	  are 'low level' enough to intercept an rdata object early in processing:
 
-	  Also, the transport destructor accessed wstransport->rdata.tp_info.pool
-	  right after freeing memory that contained wstransport itself.
+	  (1) res_pjsip_logger
+	  (2) res_hep_pjsip
+	  (3) res_pjsip/distributor
 
-	  This patch converts transport to an ao2 object, allowing it to be
-	  refcounted, so that it is available until both websocket listener and
-	  pjsip transport manager are finished with it.
+	  Anything that can intercept an rdata object before res_pjsip/distributor should
+	  be defensive when looking at the received packet.
 
-	  2. The websocket listener deletes the last reference on websocket session
-	  when the tcp connection is closed, and it gets destroyed, but
-	  the transport manager may still use it, for example when disconnect
-	  happens in the middle of a SIP transaction.
+	  #SIPit31
 
-	  A new reference to websocket session has been added that is released
-	  with the transport to prevent this.
+	  ASTERISK-24369 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  ASTERISK-25096 #close
-	  Reported by: Josh Kitchens
+	  Merged revisions 424618 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24963 #close
-	  Reported by: Badalian Vyacheslav
+	  Merged revisions 424619 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Idc0b63eb6e459c1ddfb2430127d34b3c4d8d373b
 
-2015-06-09 13:41 +0000 [3046bc17ed]  ibercom <ibercom123@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424620 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* weakref attribute detection broken with gcc 4.6 and higher
+2014-10-05 19:13 +0000 [f27f41a288]  Matt Jordan <mjordan@digium.com>
 
-	  GCC 4.7 Manual:
-	  http://gcc.gnu.org/onlinedocs/gcc-4.7.4/gcc/Function-Attributes.html
+	* res/res_pjsip_pubsub: Gracefully handle errors when re-creating subscriptions
 
-	  weakref ("target")
+	  A subscription that has been persisted can - for various reasons - fail to be
+	  re-created on startup. This patch resolves a number of crashes that occurred
+	  when a subscription cannot be re-created on several off-nominal paths.
 
-	  A weak reference is an alias that does not by itself require a definition
-	  to be given for the target symbol.
+	  #SIPit31
 
-	  ASTERISK-22559 #close
-	  Reported by: Ibercom
+	  ASTERISK-24368 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  Change-Id: I36a136cae947b65187a697533416f9ff9a0b8cdf
+	  Merged revisions 424601 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-06-08 10:09 +0000 [55c8daf88b]  Corey Farrell <git@cfware.com>
 
-	* Fix unsafe uses of ast_context pointers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424602 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Although ast_context_find, ast_context_find_or_create and
-	  ast_context_destroy perform locking of the contexts table,
-	  any context pointer can become invalid at any time that the
-	  contexts table is unlocked. This change adds locking around
-	  all complete operations involving these functions.
+2014-10-04 19:49 +0000 [9611ef4f1e]  Corey Farrell <git@cfware.com>
 
-	  Places where ast_context_find was followed by ast_context_destroy
-	  have been replaced with calls ast_context_destroy_by_name.
+	* Release AMI connections on shutdown.
 
-	  ASTERISK-25094 #close
+	  ASTERISK-24378 #close
 	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4037/
+	  ........
 
-	  Change-Id: I1866b6787730c9c4f3f836b6133ffe9c820734fa
+	  Merged revisions 424578 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-06-04 07:14 +0000 [e0090216db]  ibercom <ibercom123@gmail.com>
+	  Merged revisions 424579 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* CLI: Cosmetic issue - core show uptime
+	  Merged revisions 424580 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Show uptime information ends with an unnecessary space.
 
-	  Now NEEDCOMMA is better defined.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424581 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I11b360504a0703309ff51772ff8f672287f3c5a1
+2014-10-04 19:15 +0000 [1b0902caa4]  Corey Farrell <git@cfware.com>
 
-2015-06-03 17:41 +0000 [88212ccb7f]  Mark Michelson <mmichelson@digium.com>
+	* chan_motif: Correct last commit to use ao2_cleanup to free format cap
 
-	* res_pjsip: Prevent access of NULL channels.
+	  This fix applies to 13 and trunk.
 
-	  It is possible to receive incoming requests or responses after the channel
-	  on an ast_sip_session has been destroyed and NULLed out. Handlers of these
-	  sorts of requests or responses need to be prepared for the possibility
-	  that the channel is NULL or else they could cause a crash.
+	  ASTERISK-24384 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4043/
+	  ........
 
-	  While several places have been amended to deal with NULL channels, there
-	  were still a couple of places that needed updating.
+	  Merged revisions 424554 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  res_pjsip_dtmf_info.c: When handling incoming INFO requests, we need to
-	  return early if there is no channel on the session.
 
-	  res_pjsip_session.c: When handling a 302 response, we need to stop the
-	  redirecting attempt if there is no channel on the session.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424555 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25148 #close
-	  reported by Mark Michelson
+2014-10-04 19:02 +0000 [0cea12b9e8]  Corey Farrell <git@cfware.com>
 
-	  Change-Id: Id1a75ffc3d0eaa168b0b28188fb54d6cf9fc47a9
+	* chan_motif: Release format capabilities and config on module load error
 
-2015-06-01 11:45 +0000 [f5d5aa67dc]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24384 #close
+	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4043/
+	  ........
 
-	* AMI: Escape string values.
+	  Merged revisions 424550 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  So this issue is a bit complicated. Since it is possible to pass values to AMI
-	  that contain a '\r\n' (or other similar sequences) these values need to be
-	  escaped. One way to solve this is to escape the values and then pass the escaped
-	  values to the AMI variable parameter string building function. However, this
-	  puts the onus on the pre-build function to escape all string values. This
-	  potentially requires a fair amount of changes along with a lot of string
-	  allocations/freeing for all values.
+	  Merged revisions 424551 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Surely there is a way to push this complexity down a level into the string
-	  building function itself? This of course is possible, but ends up requiring a
-	  way to distinguish between strings that need to be escaped and those that don't.
-	  The best way to handle this is by introducing a new format specifier in the
-	  format string. For instance a %s (no escape) and %S (escape). However, that is
-	  a bit weird and unexpected.
+	  Merged revisions 424552 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  So faced with those possibilities this patch implements a limited version of the
-	  first option. Instead of attempting to escape all string values this patch only
-	  escapes those values that make sense. This approach limits the number of changes
-	  and doesn't suffer from the odd format specifier problem.
 
-	  ASTERISK-24934 #close
-	  Reported by: warren smith
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424553 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ib55a5b84fe0481b0f2caaaab68c566f392c0aac0
+2014-10-03 16:58 +0000 [24ded9d9eb]  Richard Mudgett <rmudgett@digium.com>
 
-2015-06-03 13:17 +0000 [5dc9fb4198]  gtjoseph <george.joseph@fairview5.com>
+	* res_pjsip: Fix XML typo and update CHANGES.
 
-	* res_pjsip/location: Fix ref leak in contact_apply_handler
+	  ASTERISK-24199
+	  ........
 
-	  contact_apply_handler calls ast_res_pjsip_find_or_create_contact_status
-	  to force the creation of a contact_status object whenever a new
-	  contact is added but it didn't unref the returned object.
+	  Merged revisions 424528 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Added an ao2_cleanup(status) to plug the leak.
+	  Merged revisions 424529 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25141
 
-	  Change-Id: Icc1401cae142855a1abc86ab5179dfb3ee861c40
-	  Reported-by: Corey Farrell
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424530 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-06-02 15:07 +0000 [d908272b7e]  David M. Lee <dlee@respoke.io>
+2014-10-03 14:42 +0000 [70301b0438]  Richard Mudgett <rmudgett@digium.com>
 
-	* Fixes for OS X
+	* audiohooks: Reevaluate the bridge technology when an audiohook is added or removed.
 
-	   * Add some type casting so tv_usec can really be a long, instead of
-	     some strange platform specific type.
+	  Adding a mixmonitor to a channel causes the bridge to change technologies
+	  from native to simple_bridge so the call can be recorded.  However, when
+	  the mixmonitor is stopped the bridge does not switch back to the native
+	  technology.
 
-	   * Add some .dylib style files to .gitignore.
+	  * Added unbridge requests to reevaluate the bridge when a channel
+	  audiohook is removed.
 
-	   * Switch from using -Xlinker to -Wl,. For [reasons unknown][], newer
-	     versions of GCC, when compiling the Homebrew formula for Asterisk,
-	     are not properly passing the -Xlinker options to the linker. Given
-	     that -Wl, does exactly the [same thing][], and does it properly, this
-	     patch changes the -Xlinker options to use -Wl, instead.
+	  * Moved the unbridge request into ast_audiohook_attach() ensure that the
+	  bridge reevaluates whenever an audiohook is attached.  This simplified the
+	  mixmonitor and chan_spy start code as well.
 
-	   [reasons unknown]: http://bit.ly/1SUbEYx
-	   [same thing]: https://gcc.gnu.org/onlinedocs/gcc/Link-Options.html
+	  * Added defensive code to stop_mixmonitor_full() in case additional
+	  arguments are ever added to the StopMixMonitor application.
 
-	  Change-Id: Id5e6b3c6cc86282ea5fca630dc3991137c5bf4dd
+	  * Made ast_framehook_detach() not do an unbridge request if the framehook
+	  does not exist.
 
-2015-05-30 20:22 +0000 [9e7827e3ac]  Corey Farrell <git@cfware.com>
+	  * Made ast_framehook_list_fixup() do an unbridge request if there are any
+	  framehooks.  Also simplified the loop.
 
-	* pjsip_configuration: Fix leak in persistent_endpoint_update_state.
+	  ASTERISK-24195 #close
+	  Reported by: Jonathan Rose
 
-	  The loop to find the first available contact of an endpoint grabbed
-	  contact from the iterator, then checked for offline state.  This
-	  caused the first contact after the state was found to leak a reference.
+	  Review: https://reviewboard.asterisk.org/r/4046/
+	  ........
 
-	  ASTERISK-25141
+	  Merged revisions 424506 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: Id0f1d87410fc63742db0594eb4b18b36e99aec08
-2015-05-31 11:33 +0000 [888bb49618]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Merged revisions 424507 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Fix buffer overflow in slin sample frames generation.
 
-	  The length of frames retured by sample functions was twice as large as
-	  real, what caused global buffer overflow caught by AddressSanitizer.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424508 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24717 #close
-	  Reported by: Badalian Vyacheslav
+2014-10-03 13:54 +0000 [cc11a78869]  Kristian Hogh
 
-	  Change-Id: Iec2fe682aef13e556684912f906bedf7c18229c6
+	* app_queue: Add dialplan function to get the channel name at the specified position in a queue.
 
-2015-05-29 16:19 +0000 [857166b5e5]  gtjoseph <george.joseph@fairview5.com>
+	  The QUEUE_GET_CHANNEL function returns the caller's channel name at the
+	  specified position in a queue.
 
-	* res_pjsip/location:  Fix memory leak in permanent_uri_handler
+	  QUEUE_GET_CHANNEL(<queuename>[,<position>])
 
-	  When permanent_uri_handler was creating the contact status
-	  object for each contact, it wasn't unreffing it at the
-	  end of the loop.
+	  The queue position parameter defaults to 1 if not specified.
 
-	  ASTERISK-25141 #close
-	  Reported-by: Corey Farrell
+	  Noop(${QUEUE_GET_CHANNEL(queuename, 2)})
+	  "SIP/peer-00000002", if queue exist and have at least 2 callers
 
-	  Change-Id: I7bb127994677bb3d459f87952f8425c9b9967b12
+	  Noop(${QUEUE_GET_CHANNEL(queuename, 1)})
+	  Noop(${QUEUE_GET_CHANNEL(queuename)})
+	  "SIP/peer-00000000", if queue exist and have at least 1 caller
 
-2015-05-29 14:52 +0000 [1558a89129]  gtjoseph <george.joseph@fairview5.com>
+	  ASTERISK-24365 #close
+	  Reported by: Kristian Hogh
+	  Patches:
+	        queue_get_firstchannel.patch (license #6639) patch uploaded by Kristian Hogh
+	        rb4035.patch (license #6639) patch uploaded by Kristian Hogh
+	        Patch morphed from QUEUE_GET_FIRSTCHANEL to the more general QUEUE_GET_CHANNEL
+	        on reviewbord.
 
-	* Revert "endpoint/stasis: Eliminate duplicate events on endpoint status change"
+	  Review: https://reviewboard.asterisk.org/r/4035/
 
-	  This reverts commit 35c699086ae2fd81b2473307ccb2ae79ad32375a.
 
-	  Change-Id: Ia98c2b4820cf579a5b9bb75e9e05d7a233205fb7
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424493 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-27 13:22 +0000 [35c699086a]  gtjoseph <george.joseph@fairview5.com>
+2014-10-03 12:47 +0000 [0165c5f95a]  Richard Mudgett <rmudgett@digium.com>
 
-	* endpoint/stasis: Eliminate duplicate events on endpoint status change
+	* chan_pjsip: Fix deadlock when masquerading PJSIP channels.
 
-	  When an endpoint was created, it's messages were being forwarded to
-	  both the tech endpoint topic and the all endpoints topic.  Since
-	  the tech topic was also forwarded to all, this was resulting in
-	  duplicate messages whenever an endpoint published.  This patch
-	  causes the endpoint to only forward to the tech topic and lets
-	  the tech topic forward to all.
+	  Performing a directed call pickup resulted in a deadlock when PJSIP
+	  channels were involved.
 
-	  To accomplish this, the existing stasis_cp_single_create function
-	  (which both creates and forwards) was cloned and split into 2
-	  functions, one that creates the topic and one that sets up the
-	  forwarding.  This allows endpoint_internal_create to create
-	  the topic from the endpoint_all cache without forwarding it there,
-	  then allows it to do the forward to the tech's topic.
+	  A masquerade needs to hold onto the channel locks while it swaps channel
+	  information between the two channels involved in the masquerade.  With
+	  PJSIP channels, the fixup routine needed to push a fixup task onto the
+	  PJSIP channel's serializer.  Unfortunately, if the serializer was also
+	  processing a task that needed to lock the channel, you get deadlock.
 
-	  ASTERISK-25137 #close
-	  Reported-by: Vitezslav Novy
-	  ASTERISK-25116 #close
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  * Added a new control frame that is used to notify the channels that a
+	  masquerade is about to start and when it has completed.
 
-	  Change-Id: I26d7d4926a0861748fd3bdffe316b75b549a801c
+	  * Added the ability to query taskprocessors if the current thread is the
+	  taskprocessor thread.
 
-2015-05-26 13:56 +0000 [fe21f2e52f]  Richard Mudgett <rmudgett@digium.com>
+	  * Added the ability to suspend/unsuspend the PJSIP serializer thread so a
+	  masquerade could fixup the PJSIP channel without using the serializer.
 
-	* res_pjsip_session: Fix in-dialog authentication.
+	  ASTERISK-24356 #close
+	  Reported by: rmudgett
 
-	  When the remote peer requires authentication for in-dialog requests then
-	  re-INVITEs to the peer cause the call to be disconnected and other
-	  in-dialog requests to the peer like MESSAGE just don't go through.
+	  Review: https://reviewboard.asterisk.org/r/4034/
+	  ........
 
-	  * Made session_inv_on_tsx_state_changed() handle in-dialog authentication
-	  for re-INVITEs and other methods.  Initial INVITEs cannot be handled here
-	  because the INVITE transaction must be restarted earlier.
+	  Merged revisions 424471 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Pulled needed code from res/res_pjsip/pjsip_outbound_auth.c in
-	  preparation for removing the file.  The generic outbound authentication
-	  code did not work as well as anticipated.
+	  Merged revisions 424472 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Created outbound_invite_auth() to only handle initial outbound INVITEs.
-	  Re-INVITEs cannot be handled here.  The re-INVITE transaction is still in
-	  progress and the PJSIP library cannot handle the overlapping INVITE
-	  transactions.  Other method types should not be handled here as this code
-	  only works on outgoing calls and we need to handle incoming and outgoing
-	  calls.
 
-	  ASTERISK-25131 #close
-	  Reported by: Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424473 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I12bdd7ddccc819b4ce4b091e826d1e26334601b0
+2014-10-03 10:55 +0000 [4967478d18]  gtjoseph <george.joseph@fairview5.com>
 
-2015-05-21 17:21 +0000 [262d590819]  gtjoseph <george.joseph@fairview5.com>
+	* sorcery: Prevent SEGV in sorcery_wizard_create when there's no create function
 
-	* res_pjsip: Add AMI events for chan_pjsip contact lifecycle changes
+	  When you call ast_sorcery_create() you don't necessarily know which wizard is
+	  going to be invoked.  If it happens to be a wizard like 'config' that doesn't
+	  have a 'create' virtual function you get a segfault in the
+	  sorcery_wizard_create callback.  This patch catches the null function pointer,
+	  does an ast_assert, and logs an error.
 
-	  Add a new ContactStatus AMI event.
-	  Publish the following status/state changes:
-	  Created
-	  Removed
-	  Reachable
-	  Unreachable
-	  Unknown
+	  Review: https://reviewboard.asterisk.org/r/4044/
+	  ........
 
-	  Contact URI, new status/state, aor and endpoint names, and the
-	  last qualify rtt result are included in the event.
+	  Merged revisions 424447 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25114 #close
+	  Merged revisions 424448 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Id25aae5f7122facba183273efb3e8f36c20fb61e
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
 
-2015-05-26 07:44 +0000 [5a42397018]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424449 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* sorcery: Fix cache creation callback.
+2014-10-03 08:59 +0000 [b1f8eba178]  Kinsey Moore <kmoore@digium.com>
 
-	  The cache creation callback function expects to receive a sorcery_details
-	  structure and not just a standalone object.
+	* PJSIP: Restore functional default for callerid_privacy
 
-	  Change-Id: I3e4a5a137cb25292eb52d7a14cbb6daa09213450
+	  The pjsip config option default fixups from r424263 altered the
+	  functional default from "allowed_not_screened" to "allowed". This
+	  change restores the functional default value when none is provided.
+	  ........
 
-2015-05-24 13:47 +0000 [97a6ce1717]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Merged revisions 424426 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Astobj2: Correctly treat hash_fn returning INT_MIN
+	  Merged revisions 424427 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The code in astobj2_hash.c wrongly assumed that abs(int) is always > 0.
-	  However, abs(INT_MIN) = INT_MIN and is still negative, as well as
-	  abs(INT_MIN) % num_buckets, and as a result this led to a crash.
 
-	  One way to trigger the bug is using host=::80 or 0.0.0.128 in peer
-	  configuration section in chan_sip or chan_iax.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424428 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch takes the remainder before applying abs, so that bucket
-	  number is always in range.
+2014-10-03 08:33 +0000 [4246652603]  Kinsey Moore <kmoore@digium.com>
 
-	  ASTERISK-25100 #close
-	  Reported by: Mark Petersen
+	* Manager: Add missing fields and documentation for CoreShowChannels
 
-	  Change-Id: Id6981400ad526f47e10bcf7b847b62bd2785e899
-2015-05-23 04:36 +0000 [554bd1e39c]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  This corrects some issues introduced in the responses to the
+	  CoreShowChannels AMI command as well as adding documentation for the
+	  responses. The command in Asterisk 12 was missing the following fields:
+	  Duration, Application, ApplicationData, and BridgedChannel and
+	  BridgedUniqueID (replaced with BridgeId).
 
-	* res_pjsip_transport_websocket: Fix crash on receiving large SIP packets
+	  ASTERISK-24262 #close
+	  Reported by: Mitch Claborn
+	  Review: https://reviewboard.asterisk.org/r/4040/
+	  ........
 
-	  Incoming SIP packets larger than PJSIP_MAX_PKT_LEN were themselves
-	  truncated before passing to pjsip_tpmgr_receive_packet, but the length
-	  was passed unaltered, thus causing memory corruption and segfault.
+	  Merged revisions 424423 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25122 #close
+	  Merged revisions 424424 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I608a6b6b7f229eacc33a0a7d771d18e27e5b08ab
 
-2015-05-22 21:50 +0000 [0d266cbe02]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424425 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Stasis: Fix unsafe use of stasis_unsubscribe in modules.
+2014-10-02 16:55 +0000 [2b0777c017]  Richard Mudgett <rmudgett@digium.com>
 
-	  Many uses of stasis_unsubscribe in modules can be reached through unload.
-	  These have been switched to stasis_unsubscribe_and_join.
+	* res_pjsip: Make transport cipher option accept a comma separated list of cipher names.
 
-	  Some subscription callbacks do nothing, for these I've created a noop
-	  callback function in stasis.c.  This is used by some modules that monitor
-	  MWI topics in order to enable cache, since the callback does not become
-	  invalid after dlclose it is safe to use stasis_unsubscribe on these, even
-	  during module unload.
+	  Improvements to the res_pjsip transport cipher option.
 
-	  ASTERISK-25121 #close
+	  * Made the cipher option accept a comma separated list of OpenSSL cipher
+	  names.  Users of realtime will be glad if they have more than one name to
+	  list.
 
-	  Change-Id: Ifc2549fbd8eef7d703c222978e8f452e2972189c
+	  * Added the CLI command 'pjsip list ciphers' so a user can know what
+	  OpenSSL names are available for the cipher option.
 
-2015-05-22 12:22 +0000 [51ffed5e61]  Matt Jordan <mjordan@digium.com>
+	  * Updated the cipher option online XML documentation to specify what is
+	  expected for the value.
 
-	* res/res_pjsip_pubsub: Note that 'dialog' is also a valid event type for RLS
+	  * Updated pjsip.conf.sample to not indicate that ALL is acceptable since
+	  ALL does not imply a preference order for the ciphers and PJSIP does not
+	  simply pass the string to OpenSSL for interpretation.
 
-	  In addition to specifying lists of 'presence' and 'message-summary',
-	  users can also create lists of type 'dialog'. These should be treated in
-	  the same fashion as 'presence'.
+	  ASTERISK-24199 #close
+	  Reported by: Joshua Colp
 
-	  Change-Id: I583bb69cd9f88b0b29bf09ddaddeac4e84189f6e
+	  Review: https://reviewboard.asterisk.org/r/4018/
+	  ........
 
-2015-05-22 12:18 +0000 [7950b65e4f]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 424393 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res/res_pjsip_exten_state: Fix confusing NOTICE message
+	  Merged revisions 424394 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When a SUBSCRIBE request is made to a dialplan hint that doesn't exist,
-	  the current NOTICE message informing users of this swaps the context and
-	  extension parameters. This can cause a bit of confusion.
 
-	  Thanks to CptBurger in #asterisk for helping to point this out.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424395 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Ie584d1a58ae217385c87a450ca25b55ca0e36e43
+2014-10-02 15:23 +0000 [b15cd42b5b]  Jonathan Rose <jrose@digium.com>
 
-2015-05-17 20:36 +0000 [5ac65ddfb4]  Matt Jordan <mjordan@digium.com>
+	* Alembic: Add enumerator value to sippeers -> directmedia - 'outgoing'
 
-	* res/ari: Register Stasis application on WebSocket attempt
+	  The 'outgoing' value was left off of the enumerator when first creating the
+	  column. This patch adds it, and should gracefully upgrade keeping the existing
+	  data in tact.
 
-	  Prior to this patch, when a WebSocket connection is made, ARI would not
-	  be informed of the connection until after the WebSocket layer had
-	  accepted the connection. This created a brief race condition where the
-	  ARI client would be notified that it was connected, a channel would be
-	  sent into the Stasis dialplan application, but ARI would not yet have
-	  registered the Stasis application presented in the HTTP request that
-	  established the WebSocket.
+	  ASTERISK-23781 #close
+	  Reported by: Stephen More
+	  Review: https://reviewboard.asterisk.org/r/4013/
+	  ........
 
-	  This patch resolves this issue by doing the following:
-	   * When a WebSocket attempt is made, a callback is made into the ARI
-	     application layer, which verifies and registers the apps presented in
-	     the HTTP request. Because we do not yet have a WebSocket, we cannot
-	     have an event session for the corresponding applications. Some
-	     defensive checks were thus added to make the application objects
-	     tolerant to a NULL event session.
-	   * When a WebSocket connection is made, the registered application is
-	     updated with the newly created event session that wraps the WebSocket
-	     connection.
+	  Merged revisions 424372 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24988 #close
-	  Reported by: Joshua Colp
+	  Merged revisions 424373 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ia5dc60dc2b6bee76cd5aff0f69dd53b36e83f636
 
-2015-05-20 11:11 +0000 [60e2fbfe62]  gtjoseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424380 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Refactor endpt_send_transaction (qualify_timeout)
+2014-10-02 10:33 +0000 [2f570094b7]  Jonathan Rose <jrose@digium.com>
 
-	  This patch refactors the transaction timeout processing to eliminate
-	  calling the lower level public pjsip functions and reverts to calling
-	  pjsip_endpt_send_request again.  This is the result of me noticing
-	  a possible incompatibility with pjproject-2.4 which was causing
-	  contact status flapping.
+	* chan_pjsip: Fix an assertion for channels that lack formats on creation
 
-	  The original version of this feature used the lower level calls to
-	  get access to the tsx structure in order to cancel the transaction
-	  when our own timer expires. Since we no longer have that access,
-	  if our own timer expires before the pjsip timer, we call the callbacks
-	  and just let the pjsip transaction take it's own course.  When the
-	  transaction ends, it discovers the callbacks have already been run
-	  and just cleans itself up.
+	  ASTERISK-24222 #close
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/4017/
+	  ........
 
-	  A few messages in pjsip_configuration were also added/cleaned up.
+	  Merged revisions 424333 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25105 #close
 
-	  Change-Id: I0810f3999cf63f3a72607bbecac36af0a957f33e
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
-2015-05-20 00:45 +0000 [42476e6633]  demon-ru <serov.d.p@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424358 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_outbound_registration: Check request URI for line.
+2014-10-02 08:36 +0000 [aa5458d6ab]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  When an inbound call is received the To header is checked
-	  for the "line" option. Some remote servers will place this
-	  in the request URI instead. This adds an additional check for
-	  the option in the request URI.
+	* res_pjsip: document use of rewrite_contact in sample conf
 
-	  ASTERISK-25072 #close
-	  Reported by: Dmitriy Serov
+	  Without setting rewrite_contact, an invite to an endpoint
+	  behind NAT will not reach it - unless the endpoint itself
+	  uses STUN or TURN to discover it's public URI.  Thus, the
+	  use of this should be in the sample documentation.
 
-	  Change-Id: Id4e44debbb80baad623b914a88574371575353c8
+	  Review: https://reviewboard.asterisk.org/r/4036/
+	  ........
+
+	  Merged revisions 424337 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-05-21 17:51 +0000 [e7edb59db6]  Corey Farrell <git@cfware.com>
+	  Merged revisions 424338 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_mwi_external_ami: Use module version of AMI registration.
 
-	  Use ast_manager_register_xml for res_mwi_external_ami manager
-	  actions.  This ensures the module is held open while any of
-	  the actions are being run.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424339 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25117 #close
+2014-10-01 15:37 +0000 [a752ca00bd]  Corey Farrell <git@cfware.com>
+
+	* res_hep: Release allocation reference to configuration.
+
+	  ASTERISK-24362 #close
 	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/4026/
+	  ........
 
-	  Change-Id: Iececfdc2da498b2c32b9e09042f5f12292007ac7
-2015-05-21 19:59 +0000  Asterisk Development Team <asteriskteam@digium.com>
+	  Merged revisions 424312 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* asterisk 13.4.0-rc1 Released.
+	  Merged revisions 424313 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-05-21 14:56 +0000 [3fb2b375fe]  Matt Jordan <mjordan@digium.com>
 
-	* Release summaries: Remove previous versions
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424314 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-21 14:56 +0000 [9d9ae03842]  Matt Jordan <mjordan@digium.com>
+2014-10-01 11:39 +0000 [adba2a8d7f]  Joshua Colp <jcolp@digium.com>
 
-	* .version: Update for 13.4.0-rc1
+	* res_pjsip: Add 'dtls_fingerprint' option to configure DTLS fingerprint hash.
 
-2015-05-21 14:56 +0000 [53a39083e5]  Matt Jordan <mjordan@digium.com>
+	  During the latest update to DTLS-SRTP support the ability to configure
+	  the hash used for fingerprints was added. This gave us two supported ones:
+	  SHA-1 and SHA-256. The default was accordingly updated to SHA-256.
+	  Unfortunately this configuration ability was not exposed within res_pjsip.
+	  This change adds a dtls_fingerprint option that controls it.
 
-	* .lastclean: Update for 13.4.0-rc1
+	  #SIPit31
+	  ........
 
-2015-05-21 14:56 +0000 [7af8ef9346]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 424290 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* realtime: Add database scripts for 13.4.0-rc1
+	  Merged revisions 424291 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-05-21 14:52 +0000 [20982c68d4]  Matt Jordan <mjordan@digium.com>
 
-	* Release summaries: Correct summaries for 13.4.0-rc1
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424292 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-21 13:17 +0000 [1bb62b037f]  mjordan <mjordan@lunkwill>
+2014-10-01 11:20 +0000 [9233b1cf44]  Joshua Colp <jcolp@digium.com>
 
-	* ChangeLog: Updated for 13.4.0-rc1
+	* res_pjsip_sdp_rtp: Accept DTLS attributes in top level, not just media session.
 
-2015-05-21 13:17 +0000 [1e98a36699]  mjordan <mjordan@lunkwill>
+	  #SIPit31
+	  ........
 
-	* Release summaries: Add summaries for 13.4.0-rc1
+	  Merged revisions 424287 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-05-21 13:15 +0000 [5c12e5ba72]  mjordan <mjordan@lunkwill>
+	  Merged revisions 424288 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* .version: Update for 13.4.0-rc1
 
-2015-05-21 13:15 +0000 [69292a9f11]  mjordan <mjordan@lunkwill>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424289 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* .lastclean: Update for 13.4.0-rc1
+2014-10-01 07:28 +0000 [4d2c7c23f8]  Kinsey Moore <kmoore@digium.com>
 
-2015-05-21 13:15 +0000 [628680803a]  mjordan <mjordan@lunkwill>
+	* PJSIP: Handle defaults properly
 
-	* realtime: Add database scripts for 13.4.0-rc1
+	  This updates the code behind PJSIP configuration options with custom
+	  handlers to deal with the assigned default values properly where it
+	  makes sense and adjusting the default value where it doesn't. Before
+	  applying this patch, there were several cases where the default value
+	  for an option would prevent that config section from loading properly.
 
-2015-05-21 13:05 +0000 [9d8a462356]  Matt Jordan <mjordan@digium.com>
+	  Reported by: Thomas Thompson
+	  Review: https://reviewboard.asterisk.org/r/4019/
+	  ........
 
-	* ARI: Update version to 1.7.0
+	  Merged revisions 424263 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch updates the version of ARI to 1.7.0 to reflect the backwards
-	  compatible changes that will be introduced in 13.4.0.
+	  Merged revisions 424266 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I6c36e6144da426412f25828a868e4df916bff60a
 
-2015-05-21 07:22 +0000 [620054c527]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424267 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "audiohook.c: Difference in read/write rates caused continuous buffer resets" into 13
-2015-05-21 07:21 +0000 [f5e195b44e]  Matt Jordan <mjordan@digium.com>
+2014-10-01 07:15 +0000 [122cc050d0]  Kinsey Moore <kmoore@digium.com>
 
-	* Merge "Logger: Reset defaults before processing config." into 13
-2015-05-21 07:20 +0000 [e8a4e01c32]  Matt Jordan <mjordan@digium.com>
+	* PJSIP: Force transport on contact rewrite
 
-	* Merge "res/res_http_websocket: Add a pre-session established callback" into 13
-2015-05-21 05:15 +0000 [3c98544543]  Joshua Colp <jcolp@digium.com>
+	  If contact rewriting is enabled but the contact differs in transport
+	  from what is actually being used, messages after the initial INVITE
+	  transaction can be sent to an incorrect transport/port combination. In
+	  the case where this bug occurred the remote party never received a BYE
+	  since it was sent to the remote party's TCP port over UDP.
 
-	* Merge "main/sdp_srtp.c: allow SDP crypto tag to be up to 9 digits" into 13
-2015-05-20 20:53 +0000 [9b6e228419]  Corey Farrell <git@cfware.com>
+	  Review: https://reviewboard.asterisk.org/r/4032/
+	  ........
 
-	* Logger: Reset defaults before processing config.
+	  Merged revisions 424244 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Reset options to default values before reloading config.  This ensures
-	  that if a setting is removed or commented out of the configuration file
-	  it is unset on reload.
+	  Merged revisions 424245 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25112 #close
-	  Reported by: Corey Farrell
 
-	  Change-Id: Id24bb1fb0885c2c14cf8bd6f69a0c2ee7cd6c5bd
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424246 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-20 19:05 +0000 [7fcf0a97b8]  George Joseph <george.joseph@fairview5.com>
+2014-10-01 05:10 +0000 [c3a7524457]  ibercom <ibercom123@gmail.com> (License #6599)
 
-	* app_playback: Suppress warnings on playback if channel hung up
+	* chan_sip: Simplify some unref code by removing unlink_peer_from_tables.
 
-	  If a channel hangs up while an audio file is playing, there's
-	  no need to clutter up the logs with a warning so suppress it
-	  if ast_check_hangup returns true.
+	  ASTERISK-22945 #related
+	  Reported by: ibercom
+	  Patches:
+	    asterisk11-chan_sip-simplifies.patch uploaded by ibercom (License #6599)
+	  ........
 
-	  Also, change warning to debug/2 in file.c if writing a frame
-	  fails.  Same reasoning.
+	  Merged revisions 424181 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Change-Id: I2e66191af3c5b6e951c98e8f1c3fe3cf2cf7ed89
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 424182 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-05-14 15:21 +0000 [b1e8c0b9eb]  Kevin Harwell <kharwell@digium.com>
+	  Merged revisions 424183 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* audiohook.c: Difference in read/write rates caused continuous buffer resets
+	  Merged revisions 424184 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Currently, everytime a sample rate change occurs (on read or write) the
-	  associated factory buffers are reset. If the requested sample rate on a
-	  read differed from that of a write then the buffers are continually reset
-	  on every read and write. This has the side effect of emptying the buffer,
-	  thus there being no data to read and then write to a file in the case of
-	  call recording.
 
-	  This patch fixes it so that an audiohook_list's rate always maintains the
-	  maximum sample rate among hooks and formats. Audiohook sample rates are
-	  only overwritten by this value when slin native compatibility is turned on.
-	  Also, the audiohook sample rate can only overwrite the list's sample rate
-	  when its rate is greater than that of the list or if compatibility is
-	  turned off. This keeps the rate from constantly switching/resetting.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424185 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24944 #close
-	  Reported by: Ronald Raikes
+2014-10-01 04:55 +0000 [841d978a30]  ibercom <ibercom123@gmail.com> (License #6599)
 
-	  Change-Id: Idab4dfef068a7922c09cc631dda27bc920a6c76f
+	* chan_sip: Remove excess ref of realtime peer before sip_poke_peer.
 
-2015-05-20 15:22 +0000 [4a450f863b]  Matt Jordan <mjordan@digium.com>
+	  The peer is referenced at the end of sip_poke_peer, it should not get
+	  an extra ref before the call to sip_poke_peer. This fixes a memory
+	  leak.
 
-	* Merge "Fix potential crash after unload of func_periodic_hook or test_message." into 13
-2015-05-19 13:01 +0000 [17d6ede337]  Corey Edwards <tensai@zmonkey.org>
+	  ASTERISK-22945 #close
+	  Reported by: ibercom
+	  Tested by: Yuriy Gorlichenko
+	  Patches:
+	    asterisk11.patch uploaded by ibercom (License #6599)
 
-	* main/sdp_srtp.c: allow SDP crypto tag to be up to 9 digits
+	  Review: https://reviewboard.asterisk.org/r/4031/
+	  ........
 
-	  ASTERISK-24887 #close
-	  Reported by: Makoto Dei
-	  Tested by: tensai
+	  Merged revisions 424176 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Change-Id: I6a96f572adb17f76b3acafe503a01c48eb5dd9bf
+	  Merged revisions 424177 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-05-13 09:55 +0000 [31cc24aad6]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 424178 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res/res_http_websocket: Add a pre-session established callback
+	  Merged revisions 424179 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch updates http_websocket and its corresponding implementation
-	  with a pre-session established callback. This callback allows for
-	  WebSocket server consumers to be notified when a WebSocket connection is
-	  attempted, but before we accept it. Consumers can choose to reject the
-	  connection, if their application specific logic allows for it.
 
-	  As a result, this patch pulls out the previously private
-	  websocket_protocol struct and makes it public, as
-	  ast_websocket_protocol. In order to preserve backwards compatibility
-	  with existing modules, the existing APIs were left as-is, and new APIs
-	  were added for the creation of the ast_websocket_protocol as well as for
-	  adding a sub-protocol to a WebSocket server.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424180 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In particular, the following new API calls were added:
-	  * ast_websocket_add_protocol2 - add a protocol to the core WebSocket
-	    server
-	  * ast_websocket_server_add_protocol2 - add a protocol to a specific
-	    WebSocket server
-	  * ast_websocket_sub_protocol_alloc - allocate a sub-protocol object.
-	    Consumers can populate this with whatever callbacks they wish to
-	    support, then add it to the core server or a specified server.
+2014-09-30 06:42 +0000 [d7c29885ad]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-24988
-	  Reported by: Joshua Colp
+	* res_pjsip_sdp_rtp: Don't place an extra whitespace before 'rport' and don't put IPv6 addresses in brackets.
 
-	  Change-Id: Ibe0bbb30c17eec6b578071bdbd197c911b620ab2
+	  #SIPit31
+	  ........
 
-2015-05-14 22:05 +0000 [f9114179e6]  snuffy <snuffy22@gmail.com>
+	  Merged revisions 424155 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* chan_pjsip: Fix crash during off-nominal when no endpoint specified.
+	  Merged revisions 424156 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Add missing return -1 when no endpoint name is specified.
 
-	  ASTERISK-25086 #close
-	  Reported by: snuffy
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424157 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I9de76c2935a1f4e3f0cffe97a670106f5605e89e
-2015-05-14 18:01 +0000 [dd78ab42e4]  George Joseph <george.joseph@fairview5.com>
+2014-09-30 06:36 +0000 [3641ebcf96]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_config_wizard/config: Fix template processing
+	* res_rtp_asterisk: Ensure that the base and mapped address for candidates is present in SDP.
 
-	  The config wizard was always pulling the first occurrence of
-	  a variable from an ast_variable list but this gets the template
-	  value from the list instead of any overridden value.  This patch
-	  creates ast_variable_find_last_in_list() in config.c and updates
-	  res_pjsip_config_wizard to use it instead of
-	  ast_variable_find_in_list.  Now the overridden values, where they
-	  exist, are used instead of template variables.
+	  This change fixes an issue where ICE candidates put into the SDP did not contain
+	  the 'raddr' and 'rport' information for server reflexive and relay candidates.
 
-	  Updated test_config to test the new API.
+	  #SIPit31
+	  ........
 
-	  ASTERISK-25089 #close
+	  Merged revisions 424151 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
-	  Change-Id: Ifa7ddefc956a463923ee6839dd1ebe021c299de4
+	  Merged revisions 424152 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-05-15 01:54 +0000 [091b436007]  snuffy <snuffy22@gmail.com>
+	  Merged revisions 424153 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* cdr: Fix 'core show channel' CDR variable truncation.
 
-	  When the new Bridging API was implemented, the workspace variable
-	  changed to a malloc'd string, causing sizeof() to always be 8 (char).
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424154 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Revert back to stored on stack string for workspace.
+2014-09-29 17:00 +0000 [27396a6b59]  gtjoseph <george.joseph@fairview5.com>
 
-	  ASTERISK-25090 #close
+	* pjsip_cli: Suppress header print on error or no objects
 
-	  Change-Id: I51e610ae87371df771ce7693a955510efb90f8f7
-2015-05-14 15:20 +0000 [8697a49ef9]  Joshua Colp <jcolp@digium.com>
+	  If there's an error on the pjsip command line or there are no objects, don't
+	  print the column headers.
 
-	* Merge "sorcery: Add API to insert/remove a wizard to/from an object type's list" into 13
-2015-05-14 15:19 +0000 [aea349a87e]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24350 #close
+	  Reported-by: Brad Latus
+	  Tested-by: George Joseph
+	  Tested-by: Brad Latus
 
-	* Merge "Message.c: Clear message channel frames on cleanup" into 13
-2015-05-14 00:06 +0000 [6b7282ca40]  Corey Farrell <git@cfware.com>
+	  Review: https://reviewboard.asterisk.org/r/4025/
+	  ........
 
-	* Fix potential crash after unload of func_periodic_hook or test_message.
+	  Merged revisions 424128 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  These modules save a pointer to the context they create on load, and
-	  use that pointer to destroy the context at unload.  It is not safe
-	  to save this pointer, it is replaced during load of pbx_config,
-	  pbx_lua or pbx_ael.
+	  Merged revisions 424129 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This change causes the modules to pass NULL to ast_context_destroy,
-	  a safer way to perform the unregistration since it does not use
-	  a pointer that could become invalid.
 
-	  ASTERISK-25085 #close
-	  Reported by: Corey Farrell
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424130 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I6a00ec8e38046058f97dc703e1adcde9bf517835
-2015-05-14 05:02 +0000 [8f8d54a18e]  Joshua Colp <jcolp@digium.com>
+2014-09-29 16:32 +0000 [b56dfb78c5]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	* Merge "main/manager.c: Bugfix sort action_manager by alphabetically" into 13
-2015-05-13 15:41 +0000 [02c5130589]  Jonathan Rose <jrose@digium.com>
+	* autosupport: Fix bashism.
 
-	* Message.c: Clear message channel frames on cleanup
+	  '==' is bashism (bashspecific, fails when dash is /bin/sh). Anyway, a
+	  'case' works better there.
 
-	  The message channel is a special channel that doesn't actually process frames.
-	  However, certain actions can cause frames to be placed in the channel's read
-	  queue including the Hangup application which is called on the channel after
-	  each message is processed. Since the channel will continually be reused for
-	  many messages, it's necessary to flush these frames at some point.
+	  Originally committed in r375059 and r375060 on 2012-10-16 21:13:08.
 
-	  ASTERISK-25083 #close
-	  Reported by: Jonathan Rose
+	  ASTERISK-20567 #close
+	  Reported by: Tzafrir Cohen
+	  ........
 
-	  Change-Id: Idf18df73ccd8c220be38743335b5c79c2a4c0d0f
+	  Merged revisions 424117 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-05-13 15:44 +0000 [586da882bc]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 424125 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "app_voicemail: fix moving when old messages full" into 13
-2015-05-12 17:45 +0000 [d49d64b79c]  Jonathan Rose <jrose@digium.com>
+	  Merged revisions 424126 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* app_voicemail: fix moving when old messages full
 
-	  When completing voicemail playback of a message in the 'INBOX', the
-	  message gets moved to the 'Old' messages folder. Without this patch, if
-	  the 'Old' folder is already at its set limit, then the 'INBOX' message will
-	  simply be deleted. With this patch, the flag to delete the message will be
-	  removed if the save_to_folder function indicates that the message could
-	  not be moved due to a full folder.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424127 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25082 #close
-	  Reported by: Jonathan Rose
-	  Review: https://gerrit.asterisk.org/#/c/448/
+2014-09-29 16:18 +0000 [270932635d]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: I2be440a09f42e2d06d50975c40d1ad7f836ecb3f
-2015-05-13 14:20 +0000 [51478575e4]  Joshua Colp <jcolp@digium.com>
+	* Simplify UUID generation in several places.
 
-	* Merge "General: Fix recent menuselect-related cross compile regression" into 13
-2015-05-13 12:26 +0000 [5fcaf727cc]  Joshua Colp <jcolp@digium.com>
+	  Replace code using ast_uuid_generate() with simpler and faster code using
+	  ast_uuid_generate_str().  The new code avoids a malloc(), free(), and
+	  copy.
+	  ........
 
-	* Merge "res_config_mysql: Fix broken column type checking" into 13
-2015-05-13 12:24 +0000 [6a12b0634b]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 424103 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "chan_dahdi/sig_pri: Fix crash on ISDN call hangup collision." into 13
-2015-05-04 20:11 +0000 [9b13536fed]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  Merged revisions 424105 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/manager.c: Bugfix sort action_manager by alphabetically
 
-	  Fix the alphabetic order added on ast_manager_register_struct. The order
-	  for struct manager_action added is not working, this change fixes the
-	  problem.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424109 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I149da0cd06c3c4445d7516cc303358e9f26f8b4b
+2014-09-29 15:28 +0000 [9d2bc0675a]  Richard Mudgett <rmudgett@digium.com>
 
-2015-05-08 18:01 +0000 [e67e8d5c7f]  Alexandre Fournier <alexandre.fournier@kiplink.fr>
+	* threadpool.c: Minor cleanup fixes.
 
-	* res_config_mysql: Fix broken column type checking
+	  * Fix threadpool_alloc() prototype.
 
-	  MySQL configuration engine contains a bug in require_mysql(). This
-	  function is used for column type checking in tables. This bug only
-	  affects DATETIME, DATE and FLOAT types.
+	  * Add missing off-nominal NULL check of pool in threadpool_alloc().
 
-	  It came from mixing the first condition (switch-case-like
-	  if/then/else), to check the expected column type, with the second
-	  condition, to check the actual column type against the expected column
-	  type. Both conditions must be checked separately in order to avoid the
-	  execution of the wrong block.
+	  * searializer_create() does not need to create the object with a lock as
+	  the lock is not used.
+	  ........
 
-	  ASTERISK-18252 #comment This patch might fix the issue
-	  Reported by: Gareth Blades
+	  Merged revisions 424096 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25041 #close
-	  Reported by: Alexandre Fournier
-	  Tested by: Alexandre Fournier
+	  Merged revisions 424097 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I0b8bf7e68ab938be8e6525a249260cb648cb0bfa
 
-2015-05-10 07:37 +0000 [16f602f5c2]  Yousf Ateya <y.ateya@starkbits.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424098 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_rtp_asterisk: Correction for the limit which detects that a packet is DTLS.
+2014-09-27 12:29 +0000 [2eef53c465]  Joshua Colp <jcolp@digium.com>
 
-	  First byte of DTLS packet shall be in range 20-63, not 20-64. Refer to RFC
-	  https://tools.ietf.org/html/rfc5764#section-5.1.2 for correct values.
+	* res_pjsip_session: Reduce SDP size by removing duplicate connection lines.
 
-	  Change-Id: Iae6fa0d72b37c36a27fe40686e0ae6fba3afec31
+	  Due to the architecture of how media streams are handled each individual
+	  handler adds connection details (IP address) for it. The first media stream
+	  is then used as the top level SDP connection line. In practice each
+	  line ends up being the same so to reduce the SDP size stream-level connection
+	  information is also added to the SDP if it differs from the top level SDP
+	  connection line.
 
-2015-05-13 04:35 +0000 [62422712f7]  Joshua Colp <jcolp@digium.com>
 
-	* Merge "cdr_pgsql: Use PQescapeStringConn for escaping names." into 13
-2015-05-12 17:34 +0000 [c780b6e431]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424077 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_dahdi/sig_pri: Fix crash on ISDN call hangup collision.
+2014-09-27 07:44 +0000 [76744543b4]  Joshua Colp <jcolp@digium.com>
 
-	  If an ISDN call is hungup by both sides at the same time a crash could
-	  happen.
+	* res_pjsip_session: Add additional checks for delaying session refreshes.
 
-	  * Added missing NULL checks for the owner channel after calling
-	  pri_queue_pvt_cause_data() in two places.  Code after those calls need to
-	  check the owner channel pointer for NULL before use because
-	  pri_queue_pvt_cause_data() needs to do deadlock avoidance to lock the
-	  owner and the owner may get hung up.
+	  There are certain situations which no checks existed for which need to prevent
+	  session refreshes. This includes sending a session refresh with SDP before SDP
+	  negotiation has completed and sending a session refresh before the dialog itself
+	  has been established. Checks for these have been added.
 
-	  ASTERISK-21893 #close
-	  Reported by:  Alexandr Gordeev
+	  Additionally COLP related UPDATEs were including SDP when it is not needed.
 
-	  Change-Id: Ica3e266ebc7a894b41d762326f08653e1904bb9a
+	  Review: https://reviewboard.asterisk.org/r/4008/
+	  ........
 
-2015-05-10 02:26 +0000 [6627de830b]  Sebastian Kemper <sebastian_ml@gmx.net>
+	  Merged revisions 424056 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* General: Fix recent menuselect-related cross compile regression
+	  Merged revisions 424057 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  MAKE_MENUSELECT currently sets CC to CC, which is the compiler for the
-	  target platform. But menuselect is to be run on the build system, so
-	  BUILD_CC needs to be used instead - like it was in the past, before the
-	  recent changes (https://reviewboard.asterisk.org/r/4370/). This is the
-	  patch for ASTERISK-25074.
 
-	  ASTERISK-25074 #close
-	  Reported by: Sebastian Kemper
-	  Tested by: Sebastian Kemper
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424058 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I8a2b1fc5deb6ad2b80f49baca35b1b13d468ebf8
+2014-09-26 10:51 +0000 [3c1804eb0d]  Richard Mudgett <rmudgett@digium.com>
 
-2015-05-05 15:32 +0000 [637c8f065e]  George Joseph <george.joseph@fairview5.com>
+	* format_mp3: Made the get script conditionally apply patch if not already there.
 
-	* sorcery: Add API to insert/remove a wizard to/from an object type's list
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424039 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Currently you can 'apply' a wizard to an object type but the wizard
-	  always goes at the end of the object type's wizard list.  This patch
-	  adds a new ast_sorcery_insert_wizard_mapping function that allows
-	  you to insert a wizard anyplace in the list.  I.E.  You could
-	  add a caching wizard to an object type and place it before all
-	  wizards.
+2014-09-26 10:43 +0000 [e0abb82ab8]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  ast_sorcery_get_wizard_mapping_count and
-	  ast_sorcery_get_wizard_mapping were added to allow examination
-	  of the mapping list.
+	* core: Ouch, forgot to undo a test free() in r423978.
 
-	  ast_sorcery_remove_mapping was added to remove a mapping by name.
 
-	  As part of this patch, the object type's wizard list was converted
-	  from an ao2_container to an AST_VECTOR_RW.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424038 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A new test was added to test_sorcery for this capability.
+2014-09-26 10:28 +0000 [d07b9af24b]  Jeremy Laine
 
-	  ASTERISK-25044 #close
+	* res_fax: Fix out of bounds error in update_modem_bits().
 
-	  Change-Id: I9d2469a9296b2698082c0989e25e6848dc403b57
+	  ASTERISK-24357 #close
+	  Reported by: Jeremy Laine
+	  Patches:
+	        res_fax_bounds.patch (license #6561) patch uploaded by Jeremy Laine
+	  	  Modified patch to not use magic numbers.
+	  ........
 
-2015-05-12 01:31 +0000 [3cdb7950f0]  Corey Farrell <git@cfware.com>
+	  Merged revisions 423979 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* Fix processing of asterisk.conf debug=yes.
+	  Merged revisions 423983 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  The code which reads asterisk.conf supports processing the debug
-	  option with ast_true, but ast_true returns -1.  This causes debug
-	  to still be off, convert to 1 so debug will be on as requested.
+	  Merged revisions 423987 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25042
-	  Reported by: Corey Farrell
+	  Merged revisions 423992 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I3c898b7d082d914b057e111b9357fde46bad9ed6
 
-2015-05-01 23:43 +0000 [6553a00770]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@424016 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* cdr_pgsql: Use PQescapeStringConn for escaping names.
+2014-09-26 09:41 +0000 [37179a2b1f]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  Use function PQescapeStringConn for escaping the name
-	  of the table and schema instead of doing it manually.
+	* core: Don't allow free to mean ast_free (and malloc, etc..).
 
-	  Change-Id: I6709165e2d00463e9c813d24f17830ad4910b599
+	  This gets rid of most old libc free/malloc/realloc and replaces them
+	  with ast_free and friends. When compiling with MALLOC_DEBUG you'll
+	  notice it when you're mistakenly using one of the libc variants. For
+	  the legacy cases you can define WRAP_LIBC_MALLOC before including
+	  asterisk.h.
 
-2015-05-12 05:38 +0000 [8523a5ed09]  Joshua Colp <jcolp@digium.com>
+	  Even better would be if the errors were also enabled when compiling
+	  without MALLOC_DEBUG, but that's a slightly more invasive header
+	  file change.
 
-	* Merge "vector:  Add REMOVE, ADD_SORTED and RESET macros" into 13
-2015-05-09 16:58 +0000 [ea917fefaf]  George Joseph <george.joseph@fairview5.com>
+	  Those compiling addons/format_mp3 will need to rerun
+	  ./contrib/scripts/get_mp3_source.sh.
 
-	* vector:  Add REMOVE, ADD_SORTED and RESET macros
+	  ASTERISK-24348 #related
+	  Review: https://reviewboard.asterisk.org/r/4015/
 
-	  Based on feedback from Corey Farrell and Y Ateya, a few new
-	  macros have been added...
 
-	  AST_VECTOR_REMOVE which takes a parameter to indicate if
-	  order should be preserved.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423978 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  AST_VECTOR_ADD_SORTED which adds an element to
-	  a sorted vector.
+2014-09-26 03:26 +0000 [b8c1130ed1]  Jeremy Lainé (License #6561)
 
-	  AST_VECTOR_RESET which cleans all elements from the vector
-	  leaving the storage intact.
+	* docs: Escape unescaped minus sign in asterisk.8 manpage.
 
-	  Change-Id: I41d32dbdf7137e0557134efeff9f9f1064b58d14
+	  ASTERISK-23768 #close
+	  Reported by: Jeremy Lainé
+	  Patches:
+	    escape_manpage_hyphen.patch uploaded by Jeremy Lainé (License #6561)
+	  ........
 
-2015-05-11 07:07 +0000 [d5864a358c]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Merged revisions 423915 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* pbx/pbx_spool: Fix issue when call files were executed too early
+	  Merged revisions 423916 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  pbx_spool used to delete/move the call file upon successful outgoing
-	  call completion, but did not delete it from in-memory list of files
-	  (dirlist, used only when compiled with inotify/kqueue support).
-	  That resulted in an extra attempt to process that filename after
-	  retrytime seconds.
-	  Then, if a new file with the same name appears that is scheduled
-	  in future further than the completed one plus its retrytime,
-	  then it gets executed earlier than expected.
+	  Merged revisions 423917 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch fixes remove_from_queue function to also remove the entry
-	  from the dirlist.
+	  Merged revisions 423918 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-17069 #close
-	  Reported by: Jeremy Kister
 
-	  ASTERISK-24442 #close
-	  Reported by: tootai
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423919 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: If9ec9b88073661ce485d6b008fd0b2612e49a28b
+2014-09-25 16:03 +0000 [fa0c33ebc1]  Richard Mudgett <rmudgett@digium.com>
 
-2015-05-08 14:47 +0000 [4dbd4021c9]  Rusty Newton <rnewton@digium.com>
+	* res_pjsip.c: Add missing off nominal cleanup in ast_sip_push_task_synchronous().
 
-	* configs/basic-pbx: Modified main IVR to play new Allison prompt.
+	  * Made memset the std struct in ast_sip_push_task_synchronous() because if
+	  DEBUG_THREADS is enabled then uninitialized lock tracking data is used.
+	  ........
 
-	  The main IVR was playing demo-congrats. I've switched it over to the
-	  basic-pbx-ivr-main file that we added in core sounds 1.4.27. This prompt
-	  has Allison prompting the user with the actual IVR menu.
+	  Merged revisions 423894 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24892 #close
+	  Merged revisions 423895 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ifb749616ff8e156a1031ddaddfcc9244767a095d
 
-2015-05-08 15:55 +0000 [7111ba6df4]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423896 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "tcptls: Avoiding ERR_remove_state in OpenSSL." into 13
-2015-05-08 10:39 +0000 [613a461c3d]  Sean Bright <sean@malleable.com>
+2014-09-25 15:49 +0000 [d172d84fe1]  Kristian Høgh (License #6639)
 
-	* res_rtp_asterisk: Issue ERROR if res_srtp is not found.
+	* musiconhold: Add preferchannelclass=no option to prefer app class.
 
-	  While trying to get WebRTC working with chan_pjsip, I was running
-	  into the following error:
+	  The new option 'preferchannelclass' is added to musiconhold.conf. If yes
+	  (the default) the CHANNEL(musicclass) is preferred when choosing the
+	  hold music. If it is no, the class suggested by the application that
+	  calls the MoH (e.g. the Queue() app) gets preferred (new behaviour).
 
-	      Attempted to set an invalid DTLS-SRTP configuration on RTP
-	      instance...
+	  This way you set a different hold-music from the Queue-music by setting
+	  both the CHANNEL(musicclass) and the queue-context musicclass.
 
-	  Josh helpfully pointed out that res_srtp.so might not be loaded, and
-	  sure enough, it wasn't. This patch adds a ERROR indiciating as much
-	  to hopefully help others having a similar problem.
+	  ASTERISK-24276 #close
+	  Reported by: Kristian Høgh
+	  Patches:
+	    app_override_channel_moh.patch uploaded by Kristian Høgh (License #6639)
 
-	  Change-Id: I13aa477b47b299876728a21b130998a0ea6cd19f
+	  Review: https://reviewboard.asterisk.org/r/4010/
 
-2015-05-07 17:49 +0000 [394fcb5eab]  Rusty Newton <rnewton@digium.com>
 
-	* sounds: Add Swedish sounds to Makefile and XML
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423893 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Added the necessary lines to the Makefile and sounds.xml so we'll have the
-	  Swedish sounds in all available formats in menuselect.
+2014-09-24 13:35 +0000 [68077634fe]  Richard Mudgett <rmudgett@digium.com>
 
-	  See also: Swedish sounds were added into the core sounds release 1.4.27.
+	* pjsip_options.c: Fix race condition stopping periodic out of dialog OPTIONS request.
 
-	  ASTERISK-24744 #close
+	  The crash on the issues is a result of an invalid transport configuration
+	  change when asterisk is restarted.  The attempt to send the qualify
+	  request fails and we cleaned up.  However, the callback is also called
+	  which results in a double unref of the objects involved.
 
-	  Reported by: Tove Hjelm
-	  Tested by: Rusty Newton
+	  * Put a wrapper around pjsip_endpt_send_request() to detect when the
+	  passed in callback is called because of an error so callers can know to
+	  not cleanup.
 
-	  Change-Id: Ib6f4fd177afd1667b2402735034001d4d055a908
+	  * Made send_request_cb() able to handle repeated challenges (Up to 10).
 
-2015-05-08 09:54 +0000 [30c3b254c5]  Joshua Colp <jcolp@digium.com>
+	  * Fix periodic endpoint qualify OPTIONS sched deletion race by avoiding
+	  it.  The sched entry will no longer self stop and must be externally
+	  stopped.
 
-	* Merge "doc: Make progdocs play nice with git" into 13
-2015-05-05 11:35 +0000 [2115f11b54]  Alexander Traud (License 6520)
+	  * Added REF_DEBUG description tags to struct sched_data in
+	  pjsip_options.c.
 
-	* tcptls: Avoiding ERR_remove_state in OpenSSL.
+	  * Fix some off-nominal ref leaks in schedule_qualify(),
+	  qualify_and_schedule().
 
-	  ERR_remove_state was deprecated with OpenSSL 1.0.0 and was replaced by 
-	  ERR_remove_thread_state. ERR_load_SSL_strings and ERR_load_BIO_strings were 
-	  called by SSL_load_error_strings already and got removed. These changes allow 
-	  OpenSSL forks like BoringSSL to be used with Asterisk.
+	  * Reordered pjsip_options.c module start/stop code to cleanup better on
+	  error.
 
-	  ASTERISK-25043 #close
-	  Reported by: Alexander Traud
-	  patches:
-	    asterisk_with_BoringSSL.patch uploaded by Alexander Traud (License 6520)
+	  ASTERISK-24295 #close
+	  Reported by: Rogger Padilla
 
-	  Change-Id: If1c0871ece21a7e0763fafbd2fa023ae49d4d629
-	  (cherry picked from commit 247fef66537b59649e7571d64e2c574a106dbd65)
+	  Review: https://reviewboard.asterisk.org/r/3954/
+	  ........
 
-2015-05-07 14:54 +0000 [5392e970d0]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 423866 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* doc: Make progdocs play nice with git
+	  Merged revisions 423867 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Moved contrib/asterisk-ng-doxygen to doc/asterisk-ng-doxygen.in
 
-	  Changed /Makefile to copy asterisk-ng-doxygen.in to
-	  asterisk-ng-doxygen then modify it with version instead of
-	  modifying asterisk-ng-doxygen directly.  Updated clean
-	  targets as well.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423868 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Updated /.gitignore and doc/.gitignore.
+2014-09-24 03:55 +0000 [39fada4dc9]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  Change-Id: I38712d3e334fa4baec19d30d05de8c6f28137622
+	* chan_sip: Unref outbound proxy structure on dialog/pvt destruction.
 
-2015-05-07 15:10 +0000 [1e44d1bef9]  Joshua Colp <jcolp@digium.com>
+	  Make sure outbound proxy refs are always unreffed on dialog destruction.
 
-	* Merge "res_pjsip_exten_state: Fix race condition between sending NOTIFY and termination" into 13
-2015-05-04 14:43 +0000 [608f0a94ee]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  Review: https://reviewboard.asterisk.org/r/4016/
+	  ........
 
-	* contrib/editors: Fix vim syntax highlighting of comments in config files
+	  Merged revisions 423800 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	   * Added a lookbehind to one-line comment matcher to skip escaped
-	     semicolons.
-	   * Added support for block comments.
+	  Merged revisions 423801 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: Id17dfaeda8ed4be572e8107a0c010066584aaee7
+	  Merged revisions 423802 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-05-07 13:30 +0000 [22c6c12af2]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423803 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "vector:  Additional enhancements and fixes" into 13
-2015-05-06 13:24 +0000 [d649d682c4]  Joshua Colp <jcolp@digium.com>
 
-	* res_pjsip_exten_state: Fix race condition between sending NOTIFY and termination
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423804 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The res_pjsip_exten_state module currently has a race condition between
-	  processing the extension state callback from the PBX core and processing
-	  the subscription shutdown callback from res_pjsip_pubsub. There is currently
-	  no synchronization between the two. This can present a problem as while
-	  the SIP subscription will remain valid the tree it points to may not.
-	  This is in particular a problem as a task to send a NOTIFY may get queued
-	  which will try to use the tree that may no longer be valid.
+2014-09-23 09:36 +0000 [a89964a510]  Mark Michelson <mmichelson@digium.com>
 
-	  This change does the following to fix this problem:
+	* Make CDR and CEL unit tests less FRACKy.
 
-	  1. All access to the subscription tree is done within the task that
-	  sends the NOTIFY to ensure that no other thread is modifying or
-	  destroying the tree. This task executes on the serializer for the
-	  subscriptions.
+	  Prior to this commit, CDR and CEL tests were expected to trigger
+	  FRACKs (i.e. assertions) due to the fact that the channels they
+	  create have no formats on them. Some code was independently added
+	  recently that attempts to prevent FRACKs from occurring by failing
+	  early when attempting to set up translation paths if one or both
+	  channels support no formats. Unfortunately, this attempt to be helpful
+	  made the CDR and CEL tests go from simply FRACKing to outright
+	  failing and in some cases, failing so badly as to crash Asterisk.
 
-	  2. A reference to the subscription serializer is kept to ensure it
-	  remains valid for the lifetime of the extension state subscription.
+	  This commit seeks to correct past mistakes by adding the ulaw format
+	  to channels created by the CDR and CEL unit tests. This makes setting
+	  up translation paths succeed, eliminates previously-seen FRACKs, and
+	  ultimately causes the unit tests to succeed again.
 
-	  3. The NOTIFY task has been changed so it will no longer attempt
-	  to send a NOTIFY if the subscription has already been terminated.
+	  Review: https://reviewboard.asterisk.org/r/4014
+	  ........
 
-	  ASTERISK-25057 #close
-	  Reported by: Matt Jordan
+	  Merged revisions 423783 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I0b3cd2fac5be8d9b3dc5e693aaa79846eeaf5643
 
-2015-05-07 07:02 +0000 [9322bc6ff6]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423784 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "chan_dahdi: Improve force_restart_unavailable_chans option description." into 13
-2015-05-07 06:39 +0000 [b1514362ef]  Matt Jordan <mjordan@digium.com>
+2014-09-22 14:49 +0000 [593455621b]  Torrey Searle (License #5334)
 
-	* Merge "res_stasis_snoop: Spying on a single direction continually increases CPU" into 13
-2015-05-07 06:28 +0000 [652ee2ff83]  Joshua Colp <jcolp@digium.com>
+	* chan_sip: On INVITE retransmission, don't add an extra 503 response.
 
-	* Merge "features: Fix crash when transferee hangs up during DTMF attended transfer." into 13
-2015-05-05 20:22 +0000 [5f9aea8e3c]  George Joseph <george.joseph@fairview5.com>
+	  INVITE arrives to asterisk, asterisk responds Busy(). If the INVITE is
+	  retransmitted, asterisk would generate a 503 in addition to the 486.
 
-	* vector:  Additional enhancements and fixes
+	  Thanks Torrey Searle for providing a working regression test.
 
-	  After using the new vector stuff for real I found...
+	  ASTERISK-24335 #close
 
-	  A bug in AST_VECTOR_INSERT_AT that could cause a seg fault.
+	  Review: https://reviewboard.asterisk.org/r/4003/
+	  Patches:
+	    retrans_486_invite.patch uploaded by Torrey Searle (License #5334)
+	  ........
 
-	  The callbacks needed to be closer to ao2_callback in behavior
-	  WRT to CMP_MATCH and CMP_STOP behavior and the ability to return
-	  a vector of matched entries.
+	  Merged revisions 423720 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  A pre-existing issue with APPEND and REPLACE was also fixed.
+	  Merged revisions 423721 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  I also added a new macro to test.h that acts like ast_test_validate
-	  but also accepts a return code variable and a cleanup label.  As well
-	  as printing the error, it sets the rc variable to AST_TEST_FAIL and
-	  does a goto to the specified label on error.  I had a local version
-	  of this in test_vector so I just moved it.
+	  Merged revisions 423722 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25045
+	  Merged revisions 423723 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I05e5e47fd02f61964be13b7e8942bab5d61b29cc
 
-2015-05-04 17:28 +0000 [68513e00f7]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423724 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_stasis_snoop: Spying on a single direction continually increases CPU
+2014-09-22 12:42 +0000 [63a4da4a0d]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  Creating a snoop channel in ARI and spying only on a single direction (in or
-	  out) results in CPU utilization continually increasing until the CPU is fully
-	  consumed. This occurs because frames are being put in the opposing direction's
-	  slin factory queue, but not being removed.
+	* cli.c: Fix tab completion "module load" when MALLOC_DEBUG is enabled.
 
-	  Fixed the problem by always reading and disposing of frames from the opposite
-	  queue of the direction selected.
+	  r421600 conflicted with r155763.
 
-	  ASTERISK-24938 #closes
+	  ASTERISK-24348 #close
+	  ........
 
-	  Change-Id: I935bfd15f1db958f364d9d6b3b45582c0113dd60
-2015-05-06 16:00 +0000 [904f5d98f6]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 423657 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* chan_dahdi: Improve force_restart_unavailable_chans option description.
+	  Merged revisions 423658 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  ASTERISK-25034
-	  Reported by: Richard Mudgett
+	  Merged revisions 423659 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I1ff8f02124d2f4abd632a050da52c64285bb7f30
+	  Merged revisions 423660 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-05-06 07:42 +0000 [d6ffbe39b0]  Joshua Colp <jcolp@digium.com>
 
-	* Merge "app_queue: Fix queue_log EXITWITHTIMEOUT containing only 1 parameter" into 13
-2015-05-06 06:13 +0000 [dfb292ce3e]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423661 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "res_ari_bridges: Add missing dependencies." into 13
-2015-05-05 21:05 +0000 [50e90f9121]  Matt Jordan <mjordan@digium.com>
+2014-09-20 20:16 +0000 [64a9e5f001]  Matt Jordan <mjordan@digium.com>
 
-	* Merge "pbx_config: Register manager actions with module version of macro." into 13
-2015-05-05 18:17 +0000 [be1260a35f]  Richard Mudgett <rmudgett@digium.com>
+	* main/channel: Unlock channel in off-nominal path
 
-	* features: Fix crash when transferee hangs up during DTMF attended transfer.
+	  In r423414 (13) / r423415 (trunk), an API call that determines if a format
+	  capability structure is empty was added. This returns true if the format
+	  capability structure is completely empty or "none". A check for this was added
+	  in channel.c's set_format call. Unfortunately, when this check was true, it
+	  returned from the function while still holding the channel lock. This caused
+	  the CDR unit tests - which have a tendency to create channels with no formats -
+	  to deadlock. Whoops.
 
-	  A crash happens with this sequence of steps:
-	  1) Party A is connected to party B.
-	  2) Party B starts a DTMF attended transfer.
-	  3) Party A hangs up while party B is dialing party C.
+	  This patch unlocks the channel on the off-nominal path.
+	  ........
 
-	  When party A hangs up the bridge that party A and party B are in is
-	  dissolved and party B is kicked out of the bridge.  When party B finishes
-	  dialing party C he attempts to move to the new bridge with party C.  Since
-	  party B is no longer in a bridge the attempted move dereferences a NULL
-	  bridge_channel pointer and crashes.
+	  Merged revisions 423641 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Made the hold(), unhold(), ringing(), and the bridge_move() functions
-	  tolerant of the channel not being in a bridge.  The assertion that party B
-	  is always in a bridge is not true if the bridged peer of party B hangs up
-	  and dissolves the bridge.  Being tolerant of not being in a bridge allows
-	  the peer hangup stimulus to be processed by the FSM.
 
-	  * Made the bridge_move() function return void since where the return value
-	  for a failed move was checked generated a FSM coding ERROR message for a
-	  normal off-nominal condition.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423642 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Eliminated most uses of RAII_VAR in bridge_basic.c.
+2014-09-20 18:55 +0000 [9bf039346a]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-25003 #close
-	  Reported by: Artem Volodin
+	* rest-api/api-docs/events.json: Remove non-compliant 'extends' attribute
 
-	  Change-Id: Ie2c1b14e5e647d4ea6de300bf56d69805d7bcada
+	  Prior to the release of Swagger 1.2, the attribute 'extends' was being
+	  promoted as a possible way to show that a particular object extends an existing
+	  object. Instead, the Swagger specification went with the 'subTypes' attribute
+	  in the base object. This patch removes the unsupported attribute; the object
+	  that the offending objects proposed to extend already lists them in its
+	  'subTypes' attribute.
 
-2015-05-05 15:40 +0000 [8b0f85ac06]  George Joseph <george.joseph@fairview5.com>
+	  ASTERISK-24300 #close
+	  Reported by: Bradley Watkins
+	  ........
 
-	* test_vector: Fix build breakage caused by ASTERISK_REGISTER_FILE
+	  Merged revisions 423620 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  My 13 version of test_vector had an ASTERISK_REGISTER_FILE() macro
-	  call at the top which is only supported in master.  Once removed
-	  builds are successful.
+	  Merged revisions 423621 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I7cac8b669bed6de543bbf4e2eec3cffc9741acdd
 
-2015-05-05 14:48 +0000 [87263b47b5]  Ivan Poddubny <ivan.poddubny@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423622 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* app_queue: Fix queue_log EXITWITHTIMEOUT containing only 1 parameter
+2014-09-20 18:41 +0000 [de6e467db7]  Matt Jordan <mjordan@digium.com>
 
-	  This patch fixes EXITWITHTIMEOUT queue_log entry to always come with 3
-	  parameters: position, original position and waiting time.
+	* rest-api/api-docs: Correct basePath in resources to match top resources file
 
-	  ASTERISK-25038 #close
-	  Reported by: Etienne Lessard
+	  The resources.json file that defines the resource JSON files used with ARI
+	  references a basePath of 'http://localhost:8088/ari'. This does not match what
+	  is defined in the resource files themselves, 'http://localhost:8088/stasis'.
+	  The correct base path is the one that includes 'ari' in the URL; this patch
+	  updates the various resource JSON files to have the correct basePath.
 
-	  Change-Id: I0c62045922e26bee2125e93aee1dee17eee79618
+	  ASTERISK-24339 #close
+	  Reported by: Bradley Watkins
+	  ........
 
-2015-05-05 13:13 +0000 [2d9081b5ec]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423617 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "stasis: Fix dial masquerade datastore lifetime" into 13
-2015-05-05 12:45 +0000 [8ca25dfd7e]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423618 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "vector:  Traversal, retrieval, insert and locking enhancements" into 13
-2015-05-05 09:47 +0000 [366ea63438]  Corey Farrell <git@cfware.com>
 
-	* res_ari_bridges: Add missing dependencies.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423619 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Missed this module in the previous commit.  res_ari_bridges uses symbols
-	  from res_stasis_playback and res_stasis_recording.
+2014-09-19 14:51 +0000 [354fff327d]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25027 #close
-	  Reported by: Corey Farrell
+	* res_pjsip_notify: Fix crash on unload/load and don't say the module doesn't exist on reload.
 
-	  Change-Id: I90bf756abd25adfc4920d2869ebe7feb636b8c5f
+	  When unloading the module did not unregister the CLI commands causing a crash upon
+	  load when they were registered again.
 
-2015-05-05 09:27 +0000 [69ae8cf0a4]  Corey Farrell <git@cfware.com>
+	  When reloading the module the return value from the config options framework was not
+	  checked to determine if an error occurred or not. This caused a message to be output
+	  saying the module did not exist when reloading if no changes were present.
 
-	* pbx_config: Register manager actions with module version of macro.
+	  AST-1433 #close
+	  AST-1434 #close
+	  ........
 
-	  Switch manager actions in pbx_config to use the registration macro that
-	  passes the module pointer, allowing pbx_config reference to be bumped
-	  while the manager actions run.
+	  Merged revisions 423579 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25061 #close
-	  Reported by: Corey Farrell
+	  Merged revisions 423580 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I422c50dd74814616ac10c5e9c6598a0b1bc2c44e
 
-2015-05-04 12:16 +0000 [181ae3b8d9]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423581 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* stasis: Fix dial masquerade datastore lifetime
+2014-09-19 12:16 +0000 [ec0313c411]  Richard Mudgett <rmudgett@digium.com>
 
-	  A recent change went into Asterisk which added reference counts to the
-	  channels stored in a dial masquerade datastore. Unfortunately this
-	  included a reference to the caller in a dialing operation. While all
-	  of the dialed targets have the datastore removed from them upon dialing
-	  completion this did not occur for the caller, causing it to have a
-	  reference to itself that could go never go away (as it depended on
-	  the destruction of the datastore which only happened when the channel
-	  was destroyed). This resulted in the caller channel remaining on the
-	  system despite it having hung up.
+	* res_pjsip_sdp_rtp.c: Fix native formats containing formats that were not negotiated.
 
-	  This change does the following to fix this issue:
+	  Outgoing PJSIP calls can result in non-negotiated formats listed in the
+	  channel's native formats if video formats are listed in the endpoint's
+	  configuration.  The resulting call could then use a non-negotiated format
+	  resulting in one way audio.
 
-	  1. The dial masquerade datastore is now removed from the caller upon
-	  dialing completion, just like the dialed targets.
-	  2. Upon destruction of the caller all the dialed targets are also
-	  removed from the dial masquerade datastore (just in case).
-	  3. The reference to the caller has been removed as it should not be
-	  possible for the datastore to now be valid/useful after the lifetime
-	  of the caller has ended.
+	  * Simplified the update of session->req_caps in set_caps().  Why do
+	  something in five steps when only one is needed?
 
-	  ASTERISK-25025 #close
+	  AFS-162 #close
 
-	  Change-Id: I1ef4ca5ca04980028604cc2af5d2992ac3431b3f
+	  Review: https://reviewboard.asterisk.org/r/4000/
+	  ........
 
-2015-05-01 19:25 +0000 [7a7e9733c2]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 423561 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* vector:  Traversal, retrieval, insert and locking enhancements
 
-	  Renamed AST_VECTOR_INSERT to AST_VECTOR_REPLACE because it really
-	  does replace not insert.  The few users of AST_VECTOR_INSERT were
-	  refactored.  Because these are macros, there should be no ABI
-	  compatibility issues.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423563 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Added AST_VECTOR_INSERT_AT that actually inserts an element into the
-	  vector at a specific index pushing existing elements to the right.
+2014-09-19 10:54 +0000 [6dae345674]  Jonathan Rose <jrose@digium.com>
 
-	  Added AST_VECTOR_GET_CMP that can retrieve from the vector based
-	  on a user-provided compare function.
+	* Stasis_channels: Resolve unfinished Dials when doing masquerades
 
-	  Added AST_VECTOR_CALLBACK function that will execute a function
-	  for each element in the vector.  Similar to ao2_callback and
-	  ao2_callback_data functions although the vector callback can take
-	  a variable number of arguments.  This should allow easy migration
-	  to a vector where a container might be too heavy.
+	  Masquerades into channels that are in the dialing state don't end their dial
+	  and this goes against the model for things like CDRs and generating Dial end
+	  manager actions and such.
 
-	  Added read/write locked vector and lock manipulation macros.
+	  ASTERISK-24237 #close
+	  Reported by: Richard Mudgett
+	  Review: https://reviewboard.asterisk.org/r/3990/
+	  ........
 
-	  Added unit tests.
+	  Merged revisions 423525 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-25045 #close
+	  Merged revisions 423530 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I2e07ecc709d2f5f91bcab8904e5e9340609b00e0
 
-2015-05-03 13:55 +0000 [040d2f8558]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423546 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* main/test.c: Add test to verify there were no registration errors.
+2014-09-19 10:11 +0000 [7e602175ff]  Jonathan Rose <jrose@digium.com>
 
-	  This adds a test that will fail if any test failed to register. Also fail
-	  if any test registration produced a warning about missing a leading or
-	  trailing slash.
+	* chan_iax2: Fix a crash when using chan_iax2 jitterbuffer settings
 
-	  ASTERISK-25053 #close
-	  Reported by: Corey Farrell
+	  Caused by format changes in Asterisk 13
 
-	  Change-Id: I93e50b8fcbcfa7f1f5b41b2c44a51685c09529c3
+	  ASTERISK-24265 #close
+	  Reported by: Dafi Ni
+	  Review: https://reviewboard.asterisk.org/r/3999/
+	  ........
 
-2015-05-04 09:26 +0000 [626bffc4c2]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423524 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "contrib/ast-db-manage: Add Postgres ENUM type support in auto DTMF mode update" into 13
-2015-05-04 09:26 +0000 [87fb7fc165]  Matt Jordan <mjordan@digium.com>
 
-	* Merge "cdr/cdr_csv.c: Add a new option to enable columns added in Asterisk 1.8" into 13
-2015-05-04 09:25 +0000 [81c27127aa]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423526 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "Format Interfaces: Prevent unload except by shutdown." into 13
-2015-05-04 07:46 +0000 [743fed71fc]  Matt Jordan <mjordan@digium.com>
+2014-09-19 07:50 +0000 [7f2623a26f]  Kinsey Moore <kmoore@digium.com>
 
-	* Merge "res_odbc: Use negative connection cache for all connections" into 13
-2015-04-21 11:52 +0000 [3dcec04ab5]  Martin Tomec <tomec.martin@gmail.com>
+	* PJSIP: Prevent T38 framehook being put on wrong channel
 
-	* res_odbc: Use negative connection cache for all connections
+	  This change gives framehooks a reverse-direction masquerade callback in
+	  addition to chan_fixup_cb similar to the callback added to datastores
+	  to handle the same situation. The new callback provides the same
+	  parameters as the fixup callback, but is called on the new channel's
+	  framehooks before moving framehooks from the old channel to the new
+	  channel. This gives the framehooks an oppurtunity to decide whether
+	  they should remain on the new channel or be removed.
+
+	  This new callback is used to prevent the PJSIP T.38 framehook from
+	  remaining on a masqueraded channel if the new channel is not also a
+	  PJSIP channel. This was causing a crash when a local channel was
+	  masqueraded into a PJSIP channel and the framehook was executed on the
+	  local channel since the channel's tech private data was not structured
+	  as expected.
+
+	  Review: https://reviewboard.asterisk.org/r/4001/
+	  ........
+
+	  Merged revisions 423503 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
+
+	  Merged revisions 423504 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423505 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Apply the negative connection cache setting to all connections,
-	  even those that are not pooled. This ensures that the connection
-	  will not be re-established before the negative connection cache
-	  time is met.
+2014-09-18 14:31 +0000 [40e033a6b6]  Sean Bright <sean@malleable.com>
 
-	  ASTERISK-22708 #close
+	* res_pjsip: Don't require a password when doing userpass authentication.
 
-	  Change-Id: I431cc2e8584ab0b6908b3523d0a0e18c9a527271
-2015-05-04 04:03 +0000 [74799b3fe2]  Matt Jordan <mjordan@digium.com>
+	  An empty password is valid for username/password authentication so we should
+	  allow password to be empty/not supplied.
 
-	* Merge "Remove unneeded uses of optional_api providers." into 13
-2015-05-04 04:03 +0000 [78c02f8e88]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3988
+	  ........
 
-	* Merge "Update configure.ac/Makefile for clang" into 13
-2015-05-03 21:03 +0000 [f38066fcad]  Corey Farrell <git@cfware.com>
+	  Merged revisions 423481 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Format Interfaces: Prevent unload except by shutdown.
+	  Merged revisions 423482 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Format interfaces cannot be unregistered, so the modules that provide them
-	  need to be held open except by shutdown.
 
-	  ASTERISK-25054 #close
-	  Reported by: Corey Farrell
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423483 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iadbd9675bf0d30b8fded5a739b163db3ea2db8f3
+2014-09-18 14:23 +0000 [ad8ef9175a]  gtjoseph <george.joseph@fairview5.com>
 
-2015-05-03 20:28 +0000 [e76a6a97bf]  Matt Jordan <mjordan@digium.com>
+	* utils: Create ast_strsep function that ignores separators inside quotes
 
-	* contrib/ast-db-manage: Add Postgres ENUM type support in auto DTMF mode update
+	  This function acts like strsep with three exceptions...
+	  * The separator is a single character instead of a string.
+	  * Separators inside quotes are treated literally instead of like separators.
+	  * You can elect to have leading and trailing whitespace and quotes
+	  stripped from the result and have '\' sequences unescaped.
 
-	  The upgrade script for auto DTMF mode (31cd4f4891ec) added in 88b0fa7755
-	  failed to add ENUM support for Postgres databases. This requires a
-	  specific import from the sqlalchemy.dialects.postgresql package. This
-	  patch corrects this error, which allows for Postgres update scripts to
-	  be generated.
+	  Like strsep, ast_strsep maintains no internal state and you can call it
+	  recursively using different separators on the same storage.
 
-	  ASTERISK-24706
+	  Also like strsep, for consistent results, consecutive separators are not
+	  collapsed so you may get an empty string as a valid result.
 
-	  Change-Id: I4742ac8efa533cd6f18e0bdd907b339a9aedf015
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3989/
+	  ........
 
-2015-05-01 19:50 +0000 [92120247e9]  D Tucny <d@tucny.com>
+	  Merged revisions 423476 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* term: send proper reset sequence when black background is forced
+	  Merged revisions 423478 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When using the force black background command-line option or configuration
-	  option an invalid reset sequence is sent following a coloured output item 
-	  in the CLI, the result is that the colour is not 'turned off' and continues
-	  until the next non-default coloured text output.
 
-	  A reset sequence is already defined in term.c, but the ast_term_reset
-	  function doesn't use it, instead building it's own invalid sequence and 
-	  returning that.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423480 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch changes that behaviour, removing the building of a reset sequence
-	  and instead using the pre-built constant 'enddata' which is a suitable reset
-	  sequence for this purpose.
+2014-09-18 13:56 +0000 [de72f3edbc]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-24896 #close
-	  Reported by: Dan Tucny
+	* Add subscription state test events.
 
-	  Change-Id: I56323899123ae3264900389cae1f5b252aa3bf43
+	  These are needed for a set of batched notification RLS tests that are
+	  about to be committed to the testsuite.
 
-2015-05-03 09:20 +0000 [13819a34c4]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3967
+	  ........
 
-	* Merge "Build System: Prevent unneeded changes to asterisk/buildopts.h." into 13
-2015-05-03 09:19 +0000 [b518ba1c6c]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423462 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "res_pjsip_dlg_options: Fix MODULEINFO section." into 13
-2015-05-02 18:58 +0000 [ad6ea29697]  Corey Farrell <git@cfware.com>
 
-	* Remove unneeded uses of optional_api providers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423463 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A few cases exist where headers of optional_api provders are included but
-	  not needed.  This causes unneeded calls to ast_optional_api_use.
+2014-09-18 12:22 +0000 [ac46240b62]  Jonathan Rose <jrose@digium.com>
 
-	  * Don't include optional_api.h from sip_api.h.
-	  * Move 'struct ast_channel_monitor' to channel.h.
-	  * Don't include monitor.h from chan_sip.c, channel.c or features.c.
+	* res_pjsip_endpoint_identifier_ip: Fix parsing of match value with CIDR
 
-	  The move of struct ast_channel_monitor is needed since channel.c depends on
-	  it.  This has no effect on users of monitor.h since channel.h is included
-	  from monitor.h.
+	  Also fixes comma separates match lists
 
-	  ASTERISK-25051 #close
-	  Reported by: Corey Farrell
+	  ASTERISK-24290 #close
+	  Reported by: Ray Crumrine
+	  Review: https://reviewboard.asterisk.org/r/3995/
+	  ........
 
-	  Change-Id: I53ea65a9fc9693c89f8bcfd6120649bfcfbc3478
+	  Merged revisions 423417 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-05-02 10:19 +0000 [9888562c8c]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423425 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "include/asterisk/channel.h: Fix typo" into 13
-2015-05-02 10:17 +0000 [b4000f2d44]  Matt Jordan <mjordan@digium.com>
 
-	* Merge "Astobj2: Fix initialization order of refdebug and AO2_DEBUG." into 13
-2015-04-30 02:07 +0000 [525c8c8689]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423442 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* include/asterisk/channel.h: Fix typo
+2014-09-18 12:10 +0000 [02cf1835e3]  Richard Mudgett <rmudgett@digium.com>
 
-	  Change-Id: Ie584b85e16a94c255e60d0b1732ef9686464fef3
+	* bridge_softmix.c: Made use ao2_replace() instead of the inline equivalent.
 
-2015-05-02 02:15 +0000 [63196a8256]  Corey Farrell <git@cfware.com>
+	  * Clarified some read/write format comments.
 
-	* res_pjsip_dlg_options: Fix MODULEINFO section.
+	  * Fixed a doxygen tag typo.
+	  ........
 
-	  Removed the extra space before "MODULEINFO" in res_pjsip_dlg_options.
-	  This extra space prevented any of the dependencies from being seen by
-	  menuselect, so building with default options would fail if PJSIP was
-	  not installed.
+	  Merged revisions 423423 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This also makes the tool that extracts information for menuselect
-	  tolerant of multiple spaces in the future.
 
-	  ASTERISK-25033 #close
-	  Reported by: Peter Whisker
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423424 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iccd54846f70c4a7a50cb5bf70b7bb5cb4bab3698
+2014-09-18 11:56 +0000 [a7add3a257]  Richard Mudgett <rmudgett@digium.com>
 
-2015-04-29 03:03 +0000 [ac1f0090eb]  Corey Farrell <git@cfware.com>
+	* astobj2.c/refcounter.py: Fix to deal with invalid object refs.
 
-	* Build System: Prevent unneeded changes to asterisk/buildopts.h.
+	  * Make astob2 REF_DEBUG output an invalid object line when an invalid ao2
+	  object ref/unref is attempted.  This is similar to the
+	  constructor/destructor lines.
 
-	  * Add AST_DEVMODE to BUILDOPTS
-	  * Remove CFLAGS that do not effect ABI from BUILDOPTS.
-	  * Use BUILDOPTS to generate AST_BUILDOPT_SUM.
-	  * Remove loop that defined AST_MODULE_*
+	  * Fixed refcounter.py to handle skewed objects that have
+	  constructor/destructor states.
 
-	  These changes ensure that only ABI effecting options are considered for
-	  AST_BUILDOPT_SUM.  This also reduces unneeded full system rebuilds caused
-	  by enabling or disabling one module that another is dependent on.
+	  * Made refcounter.py highlight the invalid ao2 object refs by putting them
+	  in their own section of the processed output file.
 
-	  ASTERISK-25028
-	  Reported by: Corey Farrell
+	  * Made refcounter.py highlight unreffing an object by more than one that
+	  results in a negative ref count and the object being destroyed.  The
+	  abnormally destroyed object is reported in the invalid and finalized
+	  object sections of the output.
 
-	  Change-Id: I2c516d93df9f6aaa09ae079a8168c887a6ff93a2
+	  Review: https://reviewboard.asterisk.org/r/3971/
+	  ........
 
-2015-05-01 13:22 +0000 [5875bf183c]  Corey Farrell <git@cfware.com>
+	  Merged revisions 423349 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* Astobj2: Fix initialization order of refdebug and AO2_DEBUG.
+	  Merged revisions 423400 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This ensures that refdebug is initialized before AO2_DEBUG if
-	  both are enabled, since AO2_DEBUG allocates a container.
+	  Merged revisions 423416 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This change also makes AO2_DEBUG initialization critical, a
-	  failure will abort Asterisk startup.  This is needed since
-	  the failure would be caused by reg_containers allocation
-	  failure, and that would result in a segmentation fault by
-	  ao2_container_register later in startup.
+	  Merged revisions 423418 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25048 #close
-	  Reported by: Corey Farrell
 
-	  Change-Id: I9a243ea3fc5653b48b931ba6d61971cb2e530244
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423422 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-29 14:49 +0000 [1b19c15f17]  Matt Jordan <mjordan@digium.com>
+2014-09-18 11:38 +0000 [fa6313ad29]  Mark Michelson <mmichelson@digium.com>
 
-	* main/pbx: Improve performance of dialplan reloads with a large number of hints
+	* Add API call to determine if format capability structure is "empty".
 
-	  The PBX core maintains two hash tables for hints: a container of the
-	  actual hints (hints), along with a container of devices that are watching that
-	  hint (hintdevices). When a dialplan reload occurs, each hint in the hints
-	  container is destroyed; this requires a lookup in the container of devices to
-	  find the device => hint mapping object. In the current code, this performs an
-	  ao2_callback, iterating over each of the device to hint objects in the
-	  hintdevices container. For a large number of hints, this is extremely
-	  expensive: dialplan reloads with 20000 hints could take several minutes
-	  in just this phase.
+	  Empty here means that there are no formats in the format_cap structure
+	  or the only format in it is the "none" format.
 
-	  This patch improves the performance of this step in the dialplan reloads
-	  by caching which devices are watching a hint on the hint object itself.
-	  Since we don't want to create a circular reference, we just cache the
-	  name of the device. This allows us to perform a smarter ao2_callback on
-	  the hintdevices container during hint removal, hashing on the name of the
-	  device and returning an iterator to the matching names. The overall
-	  performance improvement is rather large, taking this step down to a number of
-	  seconds as opposed to minutes.
+	  I've added calls to check the emptiness of a format_cap in a few places
+	  in order to short-circuit operations that would otherwise be pointless
+	  as well as to prevent some assertions from being triggered in cases
+	  where channels with no formats are used.
+	  ........
 
-	  In addition, this patch also registers the hint containers in the PBX
-	  core with the astobj2 library. This allows for reasonable debugging to
-	  hash collisions in those containers.
+	  Merged revisions 423414 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25040 #close
-	  Reported by: Matt Jordan
 
-	  Change-Id: Iedfc97a69d21070c50fca42275d7b3e714e59360
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423415 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-05-01 06:55 +0000 [ec0f80b6e8]  Matt Jordan <mjordan@digium.com>
+2014-09-18 11:24 +0000 [389db2b720]  Mark Michelson (License #5049)
 
-	* Merge "res_pjsip_outbound_authenticator_digest: Add missing outbound authenticator callback." into 13
-2015-05-01 06:55 +0000 [ed51fbbe9c]  Matt Jordan <mjordan@digium.com>
+	* res_fax_spandsp: Properly handle cleanup before starting FAXes.
 
-	* Merge "Prevent potential crash on blond transfer." into 13
-2015-04-30 15:54 +0000 [3efe0df044]  Corey Farrell <git@cfware.com>
+	  If faxing fails at a very early stage, then it is possible for
+	  us to pass a NULL t30 state pointer to spandsp, which spandsp
+	  is none too pleased with.
 
-	* Sample Configs: Fix syntax error in pjsip.conf
+	  This patch ensures that we pass the correct pointer to spandsp
+	  in the situation where we have not yet set our local t30 state
+	  pointer.
 
-	  The sample pjsip.conf has a few comment lines that are missing the
-	  semicolons at the start of the comment, causing the config to fail
-	  load.
+	  ASTERISK-24301 #close
+	  Reported by Matt Jordan
+	  Patches:
+	  	ASTERISK-24301-fax.diff Uploaded by Mark Michelson (License #5049)
+	  ........
 
-	  Change-Id: I776a38c916a7df7ee3e072fd0b21dbf4cc457352
+	  Merged revisions 423360 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-04-30 15:20 +0000 [077979618b]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 423365 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Prevent potential crash on blond transfer.
+	  Merged revisions 423372 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Scenario:
-	  Alice calls Bob. Bob performs a blond transfer to Carol. Carol rejects
-	  the incoming call (or some other immediate circumstance causes Carol not
-	  to answer the call)
 
-	  What occurs in this case is that when the bridge between Alice and Bob
-	  breaks, Alice is told to masquerade into Bob's channel that had placed
-	  the call to Carol. The actual masquerade goes down without a hitch.
-	  However, a channel fixup callback that attempts to publish dial events
-	  over Stasis has a crash. The reason for this crash is that the datastore
-	  on Bob's channel that placed the outbound call to Carol only had a bare
-	  pointer to Carol's channel. Since Carol rejected the incoming call,
-	  Carol's channel has been hung up and freed, meaning accessing her
-	  channel results in a crash.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423380 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The fix here is simple. The dial fixup code has been altered to hold
-	  references to the involved channels and to drop those references when
-	  freeing data.
+2014-09-18 11:09 +0000 [79eac1ffca]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-25025 #close
-	  Reported by Chet Stevens
+	* res_pjsip_pubsub: Add some type safety when generating NOTIFY bodies.
 
-	  Change-Id: I54eedda207b8ec7a69263353b43abe5746aea197
+	  res_pjsip_pubsub has two separate checks that it makes when a SUBSCRIBE
+	  arrives.
+	  * It checks that there is a subscription handler for the Event
+	  * It checks that there are body generators for the types in the Accept header
 
-2015-04-30 14:09 +0000 [4b8cddfb36]  Mark Michelson <mmichelson@digium.com>
+	  The problem is, there's nothing that ensures that these two things will
+	  actually mesh with each other. For instance, Asterisk will accept a subscription
+	  to MWI that accepts pidf+xml bodies. That doesn't make sense.
+
+	  With this commit, we add some type information to the mix. Subscription
+	  handlers state they generate data of type X, and body generators state
+	  that they consume data of type X. This way, Asterisk doesn't end up in
+	  some hilariously mismatched situation like the one in the previous paragraph.
 
-	* res_pjsip_outbound_authenticator_digest: Add missing outbound authenticator callback.
+	  ASTERISK-24136 #close
+	  Reported by Mark Michelson
 
-	  The Asterisk 13 version of the fix for outbound registration was missing
-	  a key component that set the outbound authenticator's callback that
-	  creates an authenticated request based on an old request. This was
-	  picked up by some outbound registration tests failing in the testsuite.
+	  Review: https://reviewboard.asterisk.org/r/3877
+	  Review: https://reviewboard.asterisk.org/r/3878
+	  ........
 
-	  Change-Id: I5ca9379698c606da36bc38eaffccedaf64211ce3
-2015-04-30 13:42 +0000 [415a0d0745]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 423344 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res_ari_device_states: Fix dependency on res_stasis_device_state.
+	  Merged revisions 423348 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The res_ari_device_states module depends on res_stasis_device_state,
-	  not res_stasis_device_states.
 
-	  Change-Id: I26e02ad37f9e36bcc859867e2fad1b90452ec3de
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423350 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-30 11:11 +0000 [e0c6f88010]  Mark Michelson <mmichelson@digium.com>
+2014-09-18 10:14 +0000 [126334a7aa]  gtjoseph <george.joseph@fairview5.com>
 
-	* Merge "chan_dahdi: Add the chan_dahdi.conf force_restart_unavailable_chans option." into 13
-2015-04-30 10:53 +0000 [d1bc86fc99]  Matt Jordan <mjordan@digium.com>
+	* res_pjsip: ami: Fix error in AMI output when an endpoint has no transport
 
-	* Merge "res_pjsip_outbound_registration: Add virtual line support." into 13
-2015-04-29 14:29 +0000 [d3c310a28c]  Richard Mudgett <rmudgett@digium.com>
+	  When no transport is associated to an endpoint, the AMI output for
+	  PJSIPShowEndpoint indicates an error instead of silently ignoring the
+	  missing transport.
 
-	* chan_dahdi: Add the chan_dahdi.conf force_restart_unavailable_chans option.
+	  This patch causes the error to appear only if a transport was specified
+	  on the endpoint and the transport doesn't exist.  It also fixes an issue
+	  with counting the objects that were actually found.
 
-	  Some telco switches occasionally ignore ISDN RESTART requests.  The fix
-	  for ASTERISK-19608 added an escape clause for B channels in the restarting
-	  state if the telco ignores a RESTART request.  If the telco fails to
-	  acknowledge the RESTART then Asterisk will assume the telco acknowledged
-	  the RESTART on the second call attempt requesting the B channel by the
-	  telco.  The escape clause is good for dealing with RESTART requests in
-	  general but it does cause the next call for the restarting B channel to be
-	  rejected if the telco insists the call must go on that B channel.
+	  ASTERISK-24161 #close
+	  ASTERISK-24331 #close
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3998/
+	  ........
 
-	  chan_dahdi doesn't really need to issue a RESTART request in response to
-	  receiving a cause 44 (Requested channel not available) code.  Sending the
-	  RESTART in such a situation is not required (nor prohibited) by the
-	  standards.  I think chan_dahdi does this for historical reasons to deal
-	  with buggy peers to get channels unstuck in a similar fashion as the
-	  chan_dahdi.conf resetinterval option.
+	  Merged revisions 423282 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Add the chan_dahdi.conf force_restart_unavailable_chans compatability
-	  option that when disabled will prevent chan_dahdi from trying to RESTART
-	  the channel in response to a cause 44 code.
+	  Merged revisions 423284 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25034 #close
-	  Reported by: Richard Mudgett
 
-	  Change-Id: Ib8b17a438799920f4a2038826ff99a1884042f65
-2015-04-30 06:38 +0000 [7f611fa0e8]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423285 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* cdr/cdr_csv.c: Add a new option to enable columns added in Asterisk 1.8
+2014-09-18 10:07 +0000 [b89491e39c]  David M. Lee <dlee@digium.com>
 
-	  This patch adds a new option to cdr.conf, 'newcdrcolumns', that will handle CDR
-	  columns added in Asterisk 1.8. The columns are:
-	   * peeraccount
-	   * linkedid
-	   * sequence
-	  When enabled, the columns in the database entry will be populated with the data
-	  from the CDR.
+	* Only install dahdi_span_config_hook if DAHDI is enabled
 
-	  ASTERISK-24976 #close
+	  This patch changes the install to only install the hook script if
+	  DAHDI is enabled. It also adds the script to the uninstall task, and
+	  moves the DAHDI_UDEV_HOOK_DIR variable so that it's not between the
+	  _MAKEOPTS variables and their comment.
 
-	  Change-Id: I51a57063f4ae5e194a9d933a8df45dc8a4534f0b
+	  This allows installs which specify a --prefix to work normally, as
+	  long as they don't enable DAHDI.
 
-2015-04-30 06:04 +0000 [e332c7ed5e]  Joshua Colp <jcolp@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3972/
+	  ........
 
-	* res_pjsip_outbound_registration: Fix double unref on error return.
+	  Merged revisions 423281 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When the PJSIP pjsip_regc_send function is invoked and an error
-	  status returned the caller currently decrements the reference count
-	  of the client state that it just incremented, assuming the
-	  registration callback would not have been invoked. In practice
-	  this is not correct. If the failure happens after the transaction
-	  has been set up the callback will still be invoked. This will
-	  cause the reference count to be incorrectly decremented twice, once
-	  by the registration callback and second by the caller of
-	  pjsip_regc_send.
 
-	  This change makes it so that whether the callback is invoked or
-	  not is known by the caller of pjsip_regc_send. Depending on
-	  this it can know whether it is responsible for decrementing the
-	  reference count of the client state or not.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423283 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25037 #close
-	  Reported by: Joshua Colp
+2014-09-18 09:46 +0000 [d120e40309]  gtjoseph <george.joseph@fairview5.com>
 
-	  Change-Id: I749dc12f3a22115c49c5d7d95ff42a5fa45319de
+	* config: bug: Fix SEGV in ast_category_insert when matching category isn't found
 
-2015-04-20 13:03 +0000 [9c3ed42875]  Diederik de Groot <ddegroot@talon.nl>
+	  If you call ast_category_insert with a match category that doesn't exist, the
+	  list traverse runs out of 'next' categories and you get a SEGV.  This patch
+	  adds check for the end-of-list condition and changes the signature to return
+	  an int for success/failure indication instead of a void.
 
-	* Update configure.ac/Makefile for clang
+	  The only consumer of this function is manager and it was also changed to use
+	  the return value.
 
-	  Created autoconf/ast_check_raii.m4: contains AST_CHECK_RAII which
-	  checks compiler requirements for RAII:
-	  gcc: -fnested-functions support
-	  clang: -fblocks (and if required -lBlocksRuntime)
-	  The original check was implemented in configure.ac and now has it's
-	  own file. This function also sets C_COMPILER_FAMILY to either gcc or
-	  clang for use by makefile
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3993/
+	  ........
 
-	  Created autoconf/ast_check_strsep_array_bounds.m4 (contains
-	  AST_CHECK_STRSEP_ARRAY_BOUNDS):
-	  which checks if clang is able to handle the optimized strsep & strcmp
-	  functions (linux). If not, the standard libc implementation should be
-	  used instead. Clang + the optimized macro's work with:
-	  strsep(char *, char []), but not with strsepo(char *, char *).
-	  Instead of replacing all the occurences throughout the source code,
-	  not using the optimized macro version seemed easier
+	  Merged revisions 423276 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  See 'define __strcmp_gc(s1, s2, l2) in bits/string2.h':
-	  llvm-comment: Normally, this array-bounds warning are suppressed for
-	  macros, so that unused paths like the one that accesses __s1[3] are
-	  not warned about.  But if you preprocess manually, and feed the
-	  result to another instance of clang, it will warn about all the
-	  possible forks of this particular if statement. Instead of switching
-	  of this optimization, another solution would be to run the preproces-
-	  sing step with -frewrite-includes, which should preserve enough
-	  information so that clang should still be able to suppress the diag-
-	  nostic at the compile step later on.
+	  Merged revisions 423277 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  See also "https://llvm.org/bugs/show_bug.cgi?id=20144"
-	  See also "https://llvm.org/bugs/show_bug.cgi?id=11536"
+	  Merged revisions 423278 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Makefile.rules: If C_COMPILER_FAMILY=clang then add two warning
-	  suppressions:
-	  -Wno-unused-value
-	  -Wno-parentheses-equality
-	  In an earlier review (reviewboard: 4550 and 4554), they were deemed a
-	  nuisace and less than benefitial.
+	  Merged revisions 423279 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  configure.ac:
-	  Added AST_CHECK_RAII() see earlier
-	  Added AST_CHECK_STRSEP_ARRAY_BOUNDS() see earlier
-	  Removed moved content
 
-	  ASTERISK-24917
-	  Change-Id: I12ea29d3bda2254ad3908e279b7effbbac6a97cb
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423280 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-29 16:43 +0000 [37a193da18]  Matt Jordan <mjordan@digium.com>
+2014-09-17 13:06 +0000 [8839ba3727]  Joshua Colp <jcolp@digium.com>
 
-	* Merge "ARI: Fix missing dependencies." into 13
-2015-04-29 16:42 +0000 [6a86b3555b]  Matt Jordan <mjordan@digium.com>
+	* res_rtp_asterisk: Ensure that the thread terminating pj stuff is registered.
+	  ........
 
-	* Merge "res_fax: allow 2400 transmission rate according to v.27ter standard" into 13
-2015-04-29 16:15 +0000 [d4e207e27e]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 423253 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* main/rtp_engine: Fix DTLS double-free introduced by 0b6410c4f8
+	  Merged revisions 423254 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The patch in 0b6410c4f8 did correctly fix a memory leak of the DTLS
-	  structures in the RTP engine. However, when a 'core reload' is issued, a
-	  double free of the memory pointed to by the char *'s in the DTLS
-	  configuration struct can occur, as ast_rtp_dtls_cfg_free does not set
-	  the pointers to NULL when they are freed.
+	  Merged revisions 423255 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch sets those pointers to NULL, preventing a second call to
-	  ast_rtp_dtls_cfg_free from corrupting memory.
 
-	  ASTERISK-25022
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423256 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I820471e6070a37e3c26f760118c86770e12f6115
+2014-09-16 17:46 +0000 [fcc09fd0de]  Matt Jordan <mjordan@digium.com>
 
-2015-04-29 13:05 +0000 [3fb6daeb55]  Kevin Harwell <kharwell@digium.com>
+	* pbx/Makefile: Revert r423237
 
-	* res_fax: allow 2400 transmission rate according to v.27ter standard
+	  This patch was supposed to go into a team branch, but I was a bit fast on the
+	  gun before 'svn switch' had apparently moved the target branch over.
 
-	  A previous set of patches (see: ASTERISK-22790 & ASTERISK-23231) made it so
-	  a v.27 modem was not allowed to have a minimum transmission rate of 2400 bits
-	  per second. This reverts all or some of those patches since according to the
-	  v.27ter standard a rate of 2400 bits per second is also supported.
 
-	  One of the original patches also added 9600 bits per second support for v.27.
-	  This patch also removes that since v.27ter only supports 2400/4800 bits per
-	  second.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423238 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Also, since Asterisk specifically supports v.27ter the enum was renamed to
-	  better reflect this.
+2014-09-16 17:42 +0000 [712b4195ef]  Matt Jordan <mjordan@digium.com>
 
-	  ASTERISK-24955 #close
-	  Reported by: Matt Jordan
+	* Add some pbx python stuff
 
-	  Change-Id: I4b9dfb6bf7eff08463ab47ee1a74224f27cae733
 
-2015-04-29 10:46 +0000 [49ef81c15c]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423237 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_sorcery_config: Fix build issue due to syntax error.
+2014-09-16 16:06 +0000 [618b46d8f0]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: Ic8322f04e37842848ad72cf2871bd0378f67c4ac
+	* Multiple revisions 423209,423212
 
-2015-04-28 00:29 +0000 [3278fe5327]  Ashley Sanders <asanders@digium.com>
+	  ........
+	    r423209 | file | 2014-09-16 17:35:34 -0300 (Tue, 16 Sep 2014) | 8 lines
+	    
+	    res_rtp_asterisk: Fix building when pjproject is not used.
+	    ........
+	    
+	    Merged revisions 423207 from http://svn.asterisk.org/svn/asterisk/branches/11
+	    ........
+	    
+	    Merged revisions 423208 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
+	    r423212 | file | 2014-09-16 18:03:59 -0300 (Tue, 16 Sep 2014) | 10 lines
+	    
+	    res_rtp_asterisk: Fix 100% CPU usage due to timer heap thread spinning.
+	    
+	    Side note: I need a vacation.
+	    ........
+	    
+	    Merged revisions 423210 from http://svn.asterisk.org/svn/asterisk/branches/11
+	    ........
+	    
+	    Merged revisions 423211 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* chan_pjsip: Creating Channel Causes Asterisk to Crash When Duplicate AOR
-	              Sections Exist in pjsip.conf
+	  Merged revisions 423209,423212 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch modifies the current loading strategy of the pjsip configuration. If
-	  duplicate sections (e.g. sections containing the same [id/type]) are defined in
-	  [pjsip.conf], the loader will consider the configuration for the given type as
-	  invalid when the duplicate section is encountered. The entire configuration
-	  (including what was previously loaded) for the duplicate [id/type] sections
-	  will be rejected and destroyed, an error message is logged and the load
-	  processing for the given stops.
 
-	  ASTERISK-24996
-	  Reported By: Ashley Sanders
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423213 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I35090ca4cd40f1f34881dfe701a329145c347aef
+2014-09-16 11:33 +0000 [662b687dbe]  Scott Griepentrog <sgriepentrog@digium.com>
 
-2014-11-04 06:03 +0000 [89f6719f7a]  Joshua Colp <jcolp@digium.com>
+	* Voicemail: get correct duration when copying file to vm
 
-	* res_pjsip_outbound_registration: Add virtual line support.
+	  Changes made during format improvements resulted in the
+	  recording to voicemail option 'm' of the MixMonitor app
+	  writing a zero length duration in the msgXXXX.txt file.
 
-	  Virtual line support establishes a relationship between messages
-	  related to an outbound registration and a local endpoint. This is
-	  accomplished by attaching a parameter to the Contact of the outbound
-	  registration and looking for it on any received requests. If the
-	  parameter exists and can be matched to an outbound registration
-	  the configured endpoint is associated with the request.
+	  This change introduces a new function ast_ratestream(),
+	  which provides the sample rate of the format associated
+	  with the stream, and updates the app_voicemail function
+	  for ast_app_copy_recording_to_vm to calculate the right
+	  duration.
 
-	  ASTERISK-24949 #close
-	  Reported by: Joshua Colp
+	  Review: https://reviewboard.asterisk.org/r/3996/
+	  ASTERISK-24328 #close
+	  ........
 
-	  Change-Id: I7df909d2625479110a83fdd354c21ac539e8615d
+	  Merged revisions 423192 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-29 06:39 +0000 [d61f03c4f9]  Corey Farrell <git@cfware.com>
 
-	* ARI: Fix missing dependencies.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423193 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ARI modules that are generated by 'make ari-stubs' are all dependent on
-	  res_ari_model.  Additionally some of the same modules depend on one or more
-	  res_stasis_* modules.
+2014-09-16 07:12 +0000 [ceedf44edd]  Joshua Colp <jcolp@digium.com>
 
-	  ASTERISK-25027 #close
-	  Reported by: Corey Farrell
+	* res_pjsip_session: Fix usage of wrong memory pool when creating local SDP.
+	  ........
 
-	  Change-Id: I8e07fe7e81fedacb87232f2b6f8b5f47927b4153
+	  Merged revisions 423172 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-29 06:26 +0000 [3e4624ad21]  Corey Farrell <git@cfware.com>
+	  Merged revisions 423173 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip: Remove incorrect MODULEINFO from presence_xml.c.
 
-	  Remove incorrect MODULEINFO block and unneeded header includes
-	  from presence_xml.c.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423174 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25027
-	  Reported by: Corey Farrell
+2014-09-16 06:12 +0000 [e977425bc8]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I977c609ab9d1fe05373027c4138900f6985990eb
+	* res_rtp_asterisk: Fix a myriad of TURN client issues.
 
-2015-04-29 06:17 +0000 [fed9faab8d]  Corey Farrell <git@cfware.com>
+	  1. The number of file descriptors an ioqueue instance can handle is fixed, so we
+	  now spawn the required number to handle the load.
+	  2. Our transport identifiers were exceeding the range supported by pjnath.
+	  3. The TURN client did not set up client binding causing needless bandwidth usage.
+	  4. The code no longer updates address information on each packet.
+	  5. STUN traffic was getting looped back to Asterisk instead of going through the
+	  TURN server.
+	  6. Synchronization now ensures things are completely setup or destroyed.
+	  7. Logging now reflects the target the TURN server is sending to/receiving from
+	  on our behalf.
 
-	* Git Migration: Create doc/rest-api when needed.
+	  ASTERISK-23577 #close
+	  Reported by: Jay Jideliov
 
-	  Create the directory './doc/rest-api' at the start of 'make ari-stubs'
-	  to prevent an error when documentation is generated.  The directory is
-	  also added to git ignores.
+	  ASTERISK-23634 #close
+	  Reported by: Roman Skvirsky
 
-	  ASTERISK-25027
-	  Reported by: Corey Farrell
+	  Review: https://reviewboard.asterisk.org/r/3982/
+	  ........
 
-	  Change-Id: Iaccc7f0138501c23aa78feaca2f3cce9e68cbc1b
+	  Merged revisions 423150 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-04-29 05:17 +0000 [df23c8a86b]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 423151 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* res_pjsip_outbound_registration: Fix build due to removal of transaction.
+	  Merged revisions 423152 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I7a8a7beec3334cec304943f2dd7597eabe2e3150
 
-2015-04-28 19:18 +0000 [95ab9fdb1a]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423153 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "res_pjsip_outbound_registration: Add debugging messages." into 13
-2015-04-28 19:18 +0000 [0e70dc0dc8]  Joshua Colp <jcolp@digium.com>
+2014-09-15 05:50 +0000 [77834b72d3]  Zogot, cleaned up by me.
 
-	* Merge "res_pjsip_outbound_registration: Don't fail on delayed processing: 13." into 13
-2015-04-27 16:56 +0000 [e39bd6ba46]  Mark Michelson <mmichelson@digium.com>
+	* contrib: Fix verifyi typo in alembic DB script ps_transport table.
 
-	* res_pjsip_outbound_registration: Don't fail on delayed processing: 13.
+	  Reported by: Zogot (on IRC)
+	  Patches:
+	    tmp.diff uploaded by Zogot, cleaned up by me.
+	  ........
 
-	  This is the Asterisk 13 version of a change to master that allows for
-	  registration responses to be processed successfully potentially after
-	  the original transaction has timed out. The main difference between this
-	  and the master change is that the master version has API changes that
-	  are unacceptable for 13. For 13, this is worked around by adding a new
-	  API call that the outbound registration code uses instead.
+	  Merged revisions 423128 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The following is the text from the master version of this commit:
+	  Merged revisions 423129 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Odd behaviors have been observed during outbound registrations. The most
-	  common problem witnessed has been one where a request with
-	  authentication credentials cannot be created after receiving a 401
-	  response. Other behaviors include apparently processing an incorrect SIP
-	  response.
 
-	  Inspecting the code led to an apparent issue with regards to how we
-	  handle transactions in outbound registration code. When a response to a
-	  REGISTER arrives, we save a pointer to the transaction and then push a
-	  task onto the registration serializer. Between the time that we save the
-	  pointer and push the task, it's possible for the transaction to be
-	  destroyed due to a timeout. It's also possible for the address to be
-	  reused by the transaction layer for a new transaction.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423130 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  To allow for authentication of a REGISTER request to be authenticated
-	  after the transaction has timed out, we now also hold a reference to the
-	  original REGISTER request instead of the transaction. The function for
-	  creating a request with authentication has been altered to take the
-	  original request instead of the transaction where the original request
-	  was sent.
+2014-09-14 10:54 +0000 [a62fedf0cb]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  ASTERISK-25020
-	  Reported by Mark Michelson
+	* chan_sip: Clarify that sipdebug=yes cannot be undone by the CLI.
 
-	  Change-Id: If1ee5f601be839479a219424f0358a229f358f7c
-2015-04-27 14:44 +0000 [1bf008fc76]  Mark Michelson <mmichelson@digium.com>
+	  Document it in sip.conf.
 
-	* res_pjsip_outbound_registration: Add debugging messages.
+	  ASTERISK-24249 #close
+	  Reported by: Avinash Mohod
 
-	  When problems occur regarding outbound registrations, it currently
-	  is difficult to debug. Most off-nominal paths had warning messages,
-	  but sometimes we want to know what's going on before hitting the
-	  off-nominal path. This patch adds lots of debugging output that
-	  should give a clearer picture of what is happening with regards
-	  to outbound registrations.
+	  Review: https://reviewboard.asterisk.org/r/3926/
+	  ........
 
-	  ASTERISK-25020
-	  Reported by Mark Michelson
+	  Merged revisions 423066 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  Change-Id: I577bde7860be0a6c872b5bcb4d5047340bf45d45
+	  Merged revisions 423067 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-04-28 07:13 +0000 [7ee05892d6]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 423068 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "Example script for scan-build (the llvm static analyzer)" into 13
-2015-04-28 05:38 +0000 [0b6410c4f8]  Steve Davies <steve@one47.co.uk>
+	  Merged revisions 423069 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_rtp_asterisk: Resolve 2 discrete memory leaks in DTLS
 
-	  ao2 ref leak in res_rtp_asterisk.c when a DTLS policy is created.
-	  The resources are linked into a table, but the original alloc refs
-	  are never released. ast_strdup leak in rtp_engine.c. If
-	  ast_rtp_dtls_cfg_copy() is called twice on the same destination struct,
-	  a pointer to an alloc'd string is overwritten before the string is free'd.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423070 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25022
-	  Reported by: one47
+2014-09-14 10:41 +0000 [9c1f34c7e9]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  Change-Id: I62a8ceb8679709f6c3769136dc6aa9a68202ff9b
+	* musiconhold: Add sort=randstart, and deprecate old stuff.
 
-2015-04-28 06:55 +0000 [427209603d]  Joshua Colp <jcolp@digium.com>
+	  - adds sort=randstart (next to sort=, sort=random, sort=alpha)
+	  - combines duplicate moh option parsing code into a single function
+	  - adds deprecationwarnings for application=r to sort randomly
+	  - adds deprecationwarnings for random=yes to sort randomly
+	  - removes invisible code that was supposed to stay until 1.8 
 
-	* Merge "cdr/cdr_odbc.c: Added to record new columns add on CDR 1.8 Asterisk Version" into 13
-2015-04-27 12:11 +0000 [99fb87ae13]  George Joseph <george.joseph@fairview5.com>
+	  The sort=randstart works like sort=alpha, except we start at a random
+	  position.
 
-	* res_pjsip: Fix SEGV on pending-qualify contacts
+	  Review: https://reviewboard.asterisk.org/r/3991/
 
-	  Permanent contacts that hadn't been qualified yet were missing
-	  their contact_status entries causing SEGVs when running CLI
-	  commands.
 
-	  This patch makes sure that contact_statuses are created for
-	  both dynamic and permanent contacts when they are created.
-	  It also adds checks in the CLI code to make sure there's a
-	  contact_status, just in case.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423065 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-25018 #close
-	  Reported-by: Ivan Poddubny
-	  Tested-by: Ivan Poddubny
-	  Tested-by: George Joseph
+2014-09-12 12:42 +0000 [02295456ef]  Joshua Colp <jcolp@digium.com>
 
-	  Change-Id: I3cc13e5cedcafb24c400368b515b02d7fb81e029
+	* chan_rtp: Add unicast RTP support.
 
-2015-04-15 18:55 +0000 [d5dd43856e]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  This module supports sending both unicast and multicast RTP
+	  to a specified target. Multicast functionality is the same as
+	  chan_multicast_rtp was. In the case of unicast a specific
+	  IP address and port can be specified, along with optional RTP
+	  engine and format in the form of:
 
-	* cdr/cdr_odbc.c: Added to record new columns add on CDR 1.8 Asterisk Version
+	  UnicastRTP/<ip address>:<port>/<engine>/<format>
 
-	  Add new column to INSERT new columns added in cdr 1.8 version. The columns are:
-	   * peeraccount
-	   * linkedid
-	   * sequence
-	  This feature is configurable in cdr_odbc.conf using a new configuration
-	  option, 'newcdrcolumns'.
+	  This can be useful for sending a copy of a media stream to
+	  another application for processing.
 
-	  ASTERISK-24976 #close
+	  Review: https://reviewboard.asterisk.org/r/3981/
 
-	  Change-Id: Ibe0c7540a88305c6012786f438a0813ad8b19127
-2015-04-26 17:21 +0000 [e9788056e9]  Matt Jordan <mjordan@digium.com>
 
-	* channels/chan_skinny: Fix compilation error introduced in f8e21a1adf
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@423004 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A typo in commit f8e21a1adf resulted in a compilation error in
-	  chan_skinny. This patch fixes the typo.
+2014-09-12 11:19 +0000 [dd6bdede7d]  Jonathan Rose <jrose@digium.com>
 
-	  ASTERISK-24917
+	* Realtime: Fix a bug that caused realtime destroy command to crash
 
-	  Change-Id: Id7f4ad1fe948eb2408622e80c27936ce4516c33c
+	  Also has could affect with anything that goes through ast_destroy_realtime.
+	  If a CLI user used the command 'realtime destroy <family>' with only a single
+	  column/value pair, Asterisk would crash when trying to create a variable list
+	  from a NULL value.
 
-2015-04-26 15:53 +0000 [2d277996b7]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24231 #close
+	  Reported by: Niklas Larsson
+	  Review: https://reviewboard.asterisk.org/r/3985/
+	  ........
 
-	* Merge "Clang: Fix some more tautological-compare warnings." into 13
-2015-04-24 13:07 +0000 [145f65598c]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422984 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "app_confbridge: Default the template option to a compatible default profile." into 13
-2015-04-23 15:11 +0000 [7e5056b393]  Kevin Harwell <kharwell@digium.com>
+	  Merged revisions 422985 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* app_confbridge: Default the template option to a compatible default profile.
 
-	  Confbridge dynamic profiles did not have a default profile unless you
-	  explicitly used Set(CONFBRIDGE(bridge,template)=default_bridge). If a
-	  template was not set prior to the bridge being created then some
-	  options were left with no default values set. This patch makes it so
-	  the default templates are set to the default bridge and user profiles.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422991 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24749 #close
-	  Reported by: philippebolduc
+2014-09-11 17:17 +0000 [c212a71f0b]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: I1bd6e94b38701ac2112d842db68de63d46f60e0a
+	* Remove undocumented default behavior of ast_play_and_record_full acceptdtmf.
 
-2015-04-24 09:17 +0000 [1da9ec969d]  Mark Michelson <mmichelson@digium.com>
+	  ast_play_and_record_full() has a parameter called "acceptdtmf" that is a
+	  string of acceptable DTMF digits that may be pressed by a caller to end
+	  and accept the recording.
 
-	* res_pjsip_outbound_authenticator: Increase CSeq on authed requests.
+	  ARI uses this function in order to perform recording, and it provides
+	  options for what is passed as acceptdtmf to ast_play_and_record_full().
+	  By default, ARI passes an empty string, with the intention that no DTMF
+	  can be used to end the recording.
 
-	  The way PJSIP generates an authenticated request is to use a previous
-	  request as a template. This means that the authenticated request will
-	  have the same Call-ID, From header (including tag), and CSeq as the
-	  original request. PJSIP generates a new branch on the Via header to
-	  indicate that this is a new transaction, though.
+	  The problem is that ast_play_and_record_full() attempts to be "helpful"
+	  by setting "#" as the acceptdtmf if an empty string or NULL pointer
+	  has been passed in. With ARI, this results in unexpected behavior
+	  occurring if you have attempted to intercept "#" yourself in order
+	  to perform some other manipulation of the live recording.
 
-	  There are some SIP implementations, though, that do not notice the
-	  change in the branch and therefore will match the authed request to the
-	  original request's transaction. Since the CSeq is the same, the server
-	  will repeat the response it sent to the original request.
+	  This change removes the "helpful" behavior by no longer accepting
+	  "#" as a default acceptdtmf if none is specified by the caller of
+	  ast_play_and_record_full(). This makes the ARI scenario work as
+	  expected.
 
-	  This patch aids interoperability by increasing the CSeq of the authed
-	  request by one.
+	  The other callers of ast_play_and_record_full() are app_voicemail
+	  and app_minivm, and in both cases, they pass an explicit "#" to
+	  ast_play_and_record_full() as acceptdtmf, so they are unaffected
+	  by this change.
+	  ........
 
-	  ASTERISK-24845 #close
-	  Reported by: Carl Fortin
-	  Tested by: Carl Fortin
+	  Merged revisions 422964 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Change-Id: I39c4ca52e688a9f83bcc1878371334becdc5be01
+	  Merged revisions 422965 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-24 09:24 +0000 [bf3d9db4a6]  Matt Jordan <mjordan@digium.com>
 
-	* Merge "res_pjsip_t38: Don't crash on authenticated reinvite after originated T.38 FAX." into 13
-2015-04-20 13:06 +0000 [cb318f3960]  Diederik de Groot <ddegroot@talon.nl>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422967 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Example script for scan-build (the llvm static analyzer)
+2014-09-10 11:07 +0000 [93894d53c4]  gtjoseph <george.joseph@fairview5.com>
 
-	   - Added Pre-amble (Options / Flags / Usage Example / GNU License)
-	   - Extended Configurability
-	   - Made Executable
+	* config: bug: fix truncation of included config files on permissions error
 
-	  ASTERISK-24917
-	  Change-Id: I70405fe54e4be7dbfbcb62e291690069b88617a8
+	  ast_config_text_file_save() currently truncates include files as they
+	  are processed.  If a subsequent include file or the main config file has
+	  a permissions error that prevents writing, earlier include files are left
+	  truncated resulting in a frantic search for backups.
 
-2015-04-23 17:23 +0000 [b3cd5bc77f]  Mark Michelson <mmichelson@digium.com>
+	  This patch causes ast_config_text_file_save to check for write access
+	  on all files before it truncates any of them.
 
-	* Merge "Clang: change previous tautological-compare fixes." into 13
-2015-04-23 12:54 +0000 [eabf3b5a3c]  Mark Michelson <mmichelson@digium.com>
+	  Will be applied 1.8 > trunk.
 
-	* res_pjsip_t38: Don't crash on authenticated reinvite after originated T.38 FAX.
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3986/
+	  ........
 
-	  When Asterisk originates a channel to an application, the channel is
-	  hung up once the application finishes executing. When the application
-	  in question is SendFax, the Asterisk PJSIP code will attempt to reinvite
-	  the T.38 session to audio after the FAX completes. The hangup of the
-	  channel happens in the midst of this reinvite transaction. In most
-	  circumstances, this works out okay because the BYE is delayed until the
-	  reinvite transaction can complete.
+	  Merged revisions 422900 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  However, if the reinvite that Asterisk sends receives a 401/407
-	  response, then Asterisk's attempt to re-send the reinvite with
-	  authentication will fail. This is because the session supplement in
-	  res_pjsip_t38 makes the assumption that the channel on the session will
-	  always be non-NULL. Since the channel has been hung up, though, the
-	  channel is now NULL. Attempting to operate on the channel causes a
-	  crash.
+	  Merged revisions 422903 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  This patch fixes the issue by ensuring that the channel on the session
-	  is not NULL before attempting to mess with the T.38 framehook.
+	  Merged revisions 422904 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch also contains some corrections for comments that were
-	  incorrect and really confused me when I first started looking at the
-	  code.
+	  Merged revisions 422905 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-25004 #close
-	  Reported by Mark Michelson
 
-	  Change-Id: Ic5a1230668369dda4bb13524098aed9306ab45a0
-2015-04-23 09:16 +0000 [f70d21b2cf]  George Joseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422906 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip:  Validate that contact uris start with sip: or sips:
+2014-09-10 11:00 +0000 [7bd3287a11]  Sean Bright <sean@malleable.com>
 
-	  Currently we use pjsip_parse_hdr to validate contact uris but it
-	  appears that it allows uris without a scheme if there's a port
-	  supplied.  I.E myexample.com will fail but myexample.com:5060 will
-	  pass even though it has no scheme.  This causes SEGVs later on
-	  whenever the uri is used.
+	* pjsip/config_auth.c: Add missing whitespace to log messages.
 
-	  To prevent this, permanent_contact_validate has been updated to check
-	  that the scheme is either 'sip' or 'sips'.
+	  The errors generated when validating 'auth' settings are missing a space which
+	  makes the messages a little confusing.
+	  ........
 
-	  2 uses of possibly-null endpoint have also been fixed in
-	  create_out_of_dialog_request.
+	  Merged revisions 422899 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24999
+	  Merged revisions 422901 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ifc17d16a4923e1045d37fe51e43bbe29fa556ca2
-	  Reported-by: Brad Latus
 
-2015-04-23 08:00 +0000 [1bb16bedc7]  Diederik de Groot <ddegroot@talon.nl>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422902 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Clang: change previous tautological-compare fixes.
+2014-09-09 15:15 +0000 [a47873168a]  Richard Mudgett <rmudgett@digium.com>
 
-	  clang can warn about a so called tautological-compare, when it finds
-	  comparisons which are logically always true, and are therefor deemed
-	  unnecessary.
+	* Update CHANGES for CHANNEL(onhold).
 
-	  Exanple:
-	  unsigned int x = 4;
-	  if (x > 0)    // x is always going to be bigger than 0
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422885 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Enum Case:
-	  Each enumeration is its own type. Enums are an integer type but they
-	  do not have to be *signed*. C leaves it up to the compiler as an
-	  implementation option what to consider the integer type of a particu-
-	  lar enumeration is. Gcc treats an enum without negative values as
-	  an int while clang treats this enum as an unsigned int.
+2014-09-09 15:11 +0000 [51f082af34]  Rusty Newton <rnewton@digium.com>
 
-	  rmudgett & mmichelson: cast the enum to (unsigned int) in assert.
-	  The cast does have an effect. For gcc, which seems to treat all enums
-	  as int, the cast to unsigned int will eliminate the possibility of
-	  negative values being allowed. For clang, which seems to treat enums
-	  without any negative members as unsigned int, the cast will have no
-	  effect. If for some reason in the future a negative value is ever
-	  added to the enum the assert will still catch the negative value.
+	* Sounds/BuildSystem: Modifications to include new releases and Japanese language.
 
-	  ASTERISK-24917
+	  Modifying Makefile and sounds.xml to include new core 1.4.26 and extra 1.4.15
+	  sound prompt releases, plus the new Japanese core sound prompts contributed
+	  by QLOOG.
 
-	  Change-Id: I0557ae0154a0b7de68883848a609309cdf0aee6a
+	  ASTERISK-23324
+	  Reported by: Kevin McCoy
+	  Tested by: Rusty Newton
+	  ........
 
-2015-04-23 06:50 +0000 [a06924e9d9]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422789 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* Merge "Astobj2: Ensure all calls to __adjust_lock pass a valid object." into 13
-2015-04-22 16:22 +0000 [1474bb05f6]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 422790 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	* res_corosync: Add check for config file before calling corosync apis
+	  Merged revisions 422791 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  On some systems, res_corosync isn't compatible with the installed version of
-	  corosync so corosync_cfg_initialize fails, load_module returns LOAD_FAILURE,
-	  and Asterisk terminates.  The work around has been to remember to add
-	  res_corosync as a noload in modules.conf.  A better solution though is to have
-	  res_corosync check for its config file before attempting to call corosync apis
-	  and return LOAD_DECLINE if there's no config file.  This lets Asterisk loading
-	  continue.
+	  Merged revisions 422883 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  If you have a res_corosync.conf file and res_corosync fails, you get the same
-	  behavior as today and the fatal error tells you something is wrong with the
-	  install.
 
-	  ASTERISK-24998
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422884 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: Iaf94a9431a4922ec4ec994003f02135acfdd3889
-2015-04-22 15:17 +0000 [73efb093b8]  Corey Farrell <git@cfware.com>
+2014-09-09 11:14 +0000 [9183416fe2]  Richard Mudgett <rmudgett@digium.com>
 
-	* Astobj2: Ensure all calls to __adjust_lock pass a valid object.
+	* func_channel: Add CHANNEL(onhold) item to get the current hold status of the channel.
 
-	  __adjust_lock doesn't check for invalid objects, and doesn't have an
-	  appropriate return value for invalid objects.  Most callers of
-	  __adjust_lock pass objects that have already been confirmed valid,
-	  this change adds checks before the remaining calls.
+	  It would be useful to get the current hold status of a channel.
 
-	  ASTERISK-24997 #close
-	  Reported by: Corey Farrell
+	  Added CHANNEL(onhold) item that returns 1 (onhold) and 0 (not-onhold) for
+	  the hold status of a channel.
 
-	  Change-Id: I669100f87937cc3f867cec56a27ae9c01292908f
+	  ASTERISK-24038
+	  Reported by: Matt Jordan
 
-2015-04-22 16:32 +0000 [b0e929219b]  George Joseph <george.joseph@fairview5.com>
+	  AFS-113 #close
+	  Reported by: Mark Michelson
 
-	* .gitignore:  Add .gcno and .gcda
+	  Review: https://reviewboard.asterisk.org/r/3983/
 
-	  Products of --enable-coverage
 
-	  Change-Id: Ie20882d64b60692e2c941ea8872ab82a86ce77a3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422870 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-22 14:25 +0000 [5a3948a66f]  Matt Jordan <mjordan@digium.com>
+2014-09-08 13:04 +0000 [baf99dffac]  Mark Michelson <mmichelson@digium.com>
 
-	* Merge "Fix/Update clang-RAII macro implementation" into 13
-2015-04-22 14:07 +0000 [2ef1e1fc68]  Mark Michelson <mmichelson@digium.com>
+	* Add note about configuring list_items on a single line.
+	  ........
 
-	* Merge "res_pjsip_mwi: Send unsolicited MWI NOTIFY on startup and when endpoint registers." into 13
-2015-04-22 04:17 +0000 [d6dfc85666]  Diederik de Groot <ddegroot@talon.nl>
+	  Merged revisions 422855 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Clang: Fix some more tautological-compare warnings.
 
-	  clang can warn about a so called tautological-compare, when it finds
-	  comparisons which are logically always true, and are therefor deemed
-	  unnecessary.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422856 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Exanple:
-	  unsigned int x = 4;
-	  if (x > 0)    // x is always going to be bigger than 0
+2014-09-08 12:53 +0000 [5ad0edacb6]  Mark Michelson <mmichelson@digium.com>
 
-	  Enum Case:
-	  Each enumeration is its own type. Enums are an integer type but they
-	  do not have to be *signed*. C leaves it up to the compiler as an
-	  implementation option what to consider the integer type of a particu-
-	  lar enumeration is. Gcc treats an enum without negative values as
-	  an int while clang treats this enum as an unsigned int.
+	* Add sample configuration for resource lists.
 
-	  rmudgett & mmichelson: cast the enum to (unsigned int) in assert.
-	  The cast does have an effect. For gcc, which seems to treat all enums
-	  as int, the cast to unsigned int will eliminate the possibility of
-	  negative values being allowed. For clang, which seems to treat enums
-	  without any negative members as unsigned int, the cast will have no
-	  effect. If for some reason in the future a negative value is ever
-	  added to the enum the assert will still catch the negative value.
+	  On review /r/3977, it was recommended to note in the
+	  sample configuration about the size limitation for
+	  resource lists. However, since there was no section in
+	  the sample configuration at all for resource list
+	  subscriptions, I decided to make a separate commit
+	  where I have added the necessary sample configuration
+	  as well as the size limitation warning.
+	  ........
 
-	  ASTERISK-24917
-	  Change-Id: Ief23ef68916192b9b72dabe702b543ecfeca0b62
+	  Merged revisions 422853 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-22 05:45 +0000 [edd9e54818]  Joshua Colp <jcolp@digium.com>
 
-	* Merge "Check for ao2_alloc failure in __ast_channel_internal_alloc." into 13
-2015-04-14 14:04 +0000 [7b57116833]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422854 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip_mwi: Send unsolicited MWI NOTIFY on startup and when endpoint registers.
+2014-09-08 12:35 +0000 [c6bc44f700]  Mark Michelson <mmichelson@digium.com>
 
-	  Currently the res_pjsip_mwi module only sends an unsolicited MWI NOTIFY upon
-	  a mailbox state change (such as a new message being left, or one being deleted).
-	  In practice this is not sufficient to keep clients aware of the current MWI status.
+	* Pre-allocate transmission data buffer for RLS NOTIFY requests.
 
-	  This change makes the module send unsolicited MWI NOTIFY on startup so that
-	  clients are guaranteed to have the most up to date MWI information. It also makes
-	  clients receive an unsolicited MWI NOTIFY upon registration so if they are unaware
-	  of the current MWI status they receive it.
+	  PJSIP, unless a constant is modified at compilation time, limits
+	  SIP requests to 4000 bytes. Full-state RLS notifications can easily
+	  exceed this limit with moderately small lists.
 
-	  ASTERISK-24982 #close
-	  Reported by: Joshua Colp
+	  This changeset allows for Asterisk to work around this size limit by
+	  performing its own allocation of the transmission data buffer. This
+	  way, Asterisk can allocate a buffer that exceeds the built-in maximum.
 
-	  Change-Id: I043f20230227e91218f18a82c7d5bb2aa62b1d58
+	  We still impose our own limit of 64000 bytes, mainly because making
+	  allocations larger than that is a bit absurd.
 
-2015-04-22 05:29 +0000 [4423d5f755]  Joshua Colp <jcolp@digium.com>
+	  ASTERISK-24181 #close
+	  Reported by Mark Michelson
 
-	* Merge "res_pjsip_pubsub: Set the endpoint on SUBSCRIBE dialogs." into 13
-2015-04-21 15:17 +0000 [ad1a118632]  Corey Farrell <git@cfware.com>
+	  Review: https://reviewboard.asterisk.org/r/3977
+	  ........
 
-	* Check for ao2_alloc failure in __ast_channel_internal_alloc.
+	  Merged revisions 422851 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Fix a crash that could occur in __ast_channel_internal_alloc if
-	  ao2_alloc fails.
 
-	  ASTERISK-24991 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422852 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I4ca89189eb22f907408cb87d0a1645cfe1314a90
+2014-09-08 10:58 +0000 [ef5f7a0e32]  Jonathan Rose <jrose@digium.com>
 
-2015-04-20 14:30 +0000 [3327560cb2]  Mark Michelson <mmichelson@digium.com>
+	* res_pjsip_pubsub: Check supported headers for eventlist when subscribing to
+	  resource list
 
-	* res_pjsip_pubsub: Set the endpoint on SUBSCRIBE dialogs.
+	  https://wiki.asterisk.org/wiki/display/AST/Resource+List+Subscription+Test+Plan
+	  According to the off-nominal plan, if evenlist support is not specified in a
+	  SUBSCRIBE's supported header(s), that subscription should be rejected with an
+	  error.
 
-	  When SUBSCRIBE dialogs were established, we never associated
-	  the endpoint that created the subscription with the dialog
-	  we end up creating. In most cases, this ended up not causing
-	  any problems.
+	  ASTERISK-23871
+	  Reported by: Mark Michelson
+	  Review: https://reviewboard.asterisk.org/r/3960/diff/#index_header
+	  ........
 
-	  The actual bug that was observed was that when a device that
-	  was behind NAT established a subscription with Asterisk, Asterisk
-	  would end up sending in-dialog NOTIFY requests to the device's
-	  private IP addres instead of the public address of the NAT router.
+	  Merged revisions 422836 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  When Asterisk receives the initial SUBSCRIBE from the device,
-	  res_pjsip_nat rewrites the contact to the public address on which the
-	  SUBSCRIBE was received. This allows for the dialog to have its target
-	  address set to the proper public address. Asterisk then would send a 200
-	  OK response to the SUBSCRIBE, then a NOTIFY with the initial
-	  subscription state. The device would then send a 200 OK response to
-	  Asterisk's NOTIFY.
 
-	  Here's where things went wrong. When the 200 OK arrived, res_pjsip_nat
-	  did not rewrite the address in the Contact header. Then, when the PJSIP
-	  dialog layer processed the 200 OK, PJSIP would perform a comparison
-	  between the IP address in the Contact header and its saved target
-	  address for the dialog. Since they differed, PJSIP would update the
-	  target dialog address to be the address in the Contact header. From this
-	  point, if Asterisk needed to send a NOTIFY to the device, the result was
-	  that the NOTIFY would be sent to the private address that the device
-	  placed in the Contact header.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422837 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The reason why res_pjsip_nat did not rewrite the address when it
-	  received the 200 OK response was that it could not associate the
-	  incoming response with a configured endpoint. This is because on a
-	  response, the only way to associate the response to an endpoint is by
-	  finding the dialog that the response is associated with and then finding
-	  the endpoint that is associated with that dialog. We do not perform
-	  endpoint lookups on responses. res_pjsip_pubsub skipped the step of
-	  associating the endpoint with the dialog we created, so res_pjsip_nat
-	  could not find the associated endpoint and therefore couldn't rewrite
-	  the contact.
+2014-09-06 17:50 +0000 [71acca4de2]  Matt Jordan <mjordan@digium.com>
 
-	  This commit message is like 50x longer than the actual fix.
+	* main/cdr: Copy over location information during a fork
 
-	  ASTERISK 24981 #close
-	  Reported by Mark Michelson
+	  When a CDR is forked, a new CDR is created and appended to the CDR chain for
+	  the Party A. The forked CDR starts life off as a clone of the last
+	  non-finalized for the particular Party A. In the past, merely copying over
+	  the snapshots for Party A/Party B would be sufficient. However, as the CDRs
+	  now contain cached information from Party A - specifically application/data,
+	  context, and extension - we need to copy that over during a fork as well.
 
-	  Change-Id: I2b963c58c063bae293e038406f7d044a8a5377cd
-2015-04-20 18:00 +0000 [d08446ec36]  Richard Mudgett <rmudgett@digium.com>
+	  Huzzah for unit tests catching this when the context/extension were derived
+	  from a cached value on the CDR instead of on Party A.
+	  ........
 
-	* chan_dahdi/sig_pri: Make post AMI HangupRequest events on PRI channels.
+	  Merged revisions 422769 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The chan_dahdi channel driver is a very old driver.  The ability for it to
-	  support ISDN was added well after the initial analog support.  Setting the
-	  softhangup flags is a carry over from the original analog code.  The
-	  driver was not updated to call ast_queue_hangup() which will post the AMI
-	  HangupRequest event.
+	  Merged revisions 422770 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Changed sig_pri.c to call ast_queue_hangup() instead of setting the
-	  softhangup flag when the remote party initiates a hangup.
 
-	  ASTERISK-24895 #close
-	  Reported by: Andrew Zherdin
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422771 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I5fe2e48556507785fd8ab8e1c960683fd5d20325
+2014-09-06 17:22 +0000 [e4591f98b1]  Matt Jordan <mjordan@digium.com>
 
-2015-04-20 17:23 +0000 [96e18453f4]  Joshua Colp <jcolp@digium.com>
+	* main/rtp_engine: Format NTP timestamps as unsigned ints
 
-	* Merge "pjsip_options:  Fix non-qualified contacts showing as unavailable" into 13
-2015-04-20 13:01 +0000 [2be9cc2643]  Diederik de Groot <ddegroot@talon.nl>
+	  On some systems, a timeval's tv_sec/tv_usec will be unsigned lont ints, as
+	  opposed to long ints. When the RTP engine formats these as strings, it was
+	  previously formatting them as signed integers, which can result in some
+	  odd negative timestamp values (particularly on 32-bit systems). This patch
+	  formats the values as unsigned long integers.
+	  ........
 
-	* Fix/Update clang-RAII macro implementation
+	  Merged revisions 422766 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  - When you need to refer to 'variable XXX' outside a block, it needs
-	  to be declared as '__block XXX', otherwise it will not be available with-
-	  in the block, making updating that variable hard to do, and ast_free
-	  lead to issues.
+	  Merged revisions 422767 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  - Removed the #error message
-	  because it creates complications when compiling external projects
-	  against asterisk For example when using a different compiler than the
-	  one used to compile asterisk. The warning/error should be generated
-	  during the configure process not the compilation process
 
-	  ASTERISK-24917
-	  Change-Id: I12091228090e90831bf2b498293858f46ea7a8c2
-2015-04-20 09:53 +0000 [b74b2cdcda]  George Joseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422768 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip_options:  Fix format specifier for int64_t rtt.
+2014-09-06 14:13 +0000 [fd8010de2b]  Joshua Colp <jcolp@digium.com>
 
-	  Contact status rtt is an int64_t and needs the PRId64 macro to
-	  properly create the format specifier on 32-bit systems.
+	* res_pjsip_sdp_rtp: Fix retrieval of "ice-pwd" attribute if in session and not media stream.
+	  ........
 
-	  Change-Id: I4b8ab958fc1e9a179556a9b4ffa49673ba9fdec7
+	  Merged revisions 422746 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-20 06:29 +0000 [27a122af66]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422747 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "main/pbx: Don't attempt to destroy a previously destroyed exten/priority tuple" into 13
-2015-04-20 05:54 +0000 [9581a0ebf3]  Joshua Colp <jcolp@digium.com>
 
-	* Merge "Fix issue with AST_THREADSTORAGE_RAW when DEBUG_THREADLOCALS is enabled." into 13
-2015-04-18 13:36 +0000 [63169e00ff]  George Joseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422748 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* pjsip_options:  Fix non-qualified contacts showing as unavailable
+2014-09-05 17:04 +0000 [d42b116925]  Matt Jordan <mjordan@digium.com>
 
-	  The "Add qualify_timeout processing and eventing" patch introduced
-	  an issue where contacts that had qualify_frequency set to 0 were
-	  showing Unavailable instead Unknown.  This patch checks for
-	  qualify_frequency=0 and create an "Unknown"  contact_status
-	  with an RTT = 0.
+	* main/cdrs: Preserve context/extension when executing a Macro or GoSub
 
-	  Previously, the lack of contact_status implied Unknown but since
-	  we're now changing endpoint state based on contact_status, I've
-	  had to add new UNKNOWN status so that changes could trigger the
-	  appropriate contact_status observers.
+	  The context/extension in a CDR is generally considered the destination of a
+	  call. When looking at a 2-party call CDR, users will typically be presented
+	  with the following:
 
-	  ASTERISK-24977: #close
+	  context    exten      channel     dest_channel app  data
+	  default    1000       SIP/8675309 SIP/1000     Dial SIP/1000,,20
 
-	  Change-Id: Ifcbc01533ce57f0e4e584b89a395326e098b8fe7
+	  However, if the Dial actually takes place in a Macro, the current behaviour
+	  in 12 will result in the following CDR:
 
-2015-04-19 15:49 +0000 [f0c82a173a]  Matt Jordan <mjordan@digium.com>
+	  context    exten      channel     dest_channel app  data
+	  macro-dial s          SIP/8675309 SIP/1000     Dial SIP/1000,,20
 
-	* main/pbx: Don't attempt to destroy a previously destroyed exten/priority tuple
+	  The same is true of a GoSub:
 
-	  When a PBX registrar is unloaded, it will fail to remove its extension from
-	  the context root_table if a dialplan application used by that extension is
-	  still loaded. This can be the case for AGI, which can be unloaded after several
-	  of the standard PBX providers. Often, this is harmless; however, if the
-	  extension's priorities are removed during the failed unloading *and* the
-	  dialplan application later unregisters, it leaves a ticking timebomb for the
-	  next PBX provider that attempts to iterate over the extensions. When that
-	  occurs, the peer_table pointer on the extension will already be set to NULL.
-	  The current code does not check to see if the pointer is NULL before passing
-	  it to a hashtab function this is not NULL tolerant.
+	  context    exten      channel     dest_channel app  data
+	  subs       dial_stuff SIP/8675309 SIP/1000     Dial SIP/1000,,20
 
-	  Since it is possible for the peer_table to be NULL when we normally would not
-	  expect that to be the case, the solution in this patch is to simply skip over
-	  processing an extension's priorities if peer_table is NULL.
+	  This generally makes the context/exten fields less than useful.
 
-	  Prior to this patch, the tests/pbx/callerid_match test would crash during
-	  module unload. With this patch, the test no longer crashes after running.
+	  It isn't hard to preserve these values in the CDR state machine; however, we
+	  need to have something that informs us when a channel is executing a
+	  subroutine. Prior to this patch, there isn't anything that does this.
 
-	  ASTERISK-24774 #close
-	  Reported by: Corey Farrell
+	  This patch solves this problem by adding a new channel flag,
+	  AST_FLAG_SUBROUTINE_EXEC. This flag is set on a channel when it executes a
+	  Macro or a GoSub. The CDR engine looks for this value when updating a Party A
+	  snapshot; if the flag is present, we don't override the context/exten on the
+	  main CDR object. In a funny quirk, executing a hangup handler must *not* abide
+	  by this logic, as the endbeforehexten logic assumes that the user wants to see
+	  data that occurs in hangup logic, which includes those subroutines. Since
+	  those execute outside of a typical Dial operation (and will typically have
+	  their own dedicated CDR anyway), this is unlikely to cause any heartburn.
 
-	  Change-Id: I2bbeecb7e0f77bac303a1b9135e4cdb4db6d4c40
+	  Review: https://reviewboard.asterisk.org/r/3962/
 
-2015-04-17 18:05 +0000 [82bc0fd3ad]  Richard Mudgett <rmudgett@digium.com>
+	  ASTERISK-24254 #close
+	  Reported by: tm1000, Tony Lewis
+	  Tested by: Tony Lewis
+	  ........
 
-	* res_fax: Fix latent bug exposed by ASTERISK-24841 changes.
+	  Merged revisions 422718 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Three fax related tests started failing as a result of changes made for
-	  ASTERISK-24841:
-	  tests/fax/pjsip/gateway_t38_g711
-	  tests/fax/sip/gateway_mix1
-	  tests/fax/sip/gateway_mix3
+	  Merged revisions 422719 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Historically, ast_channel_make_compatible() did nothing if the channels
-	  were already "compatible" even if they had a sub-optimal translation path
-	  already setup.  With the changes from ASTERISK-24841 this is no longer
-	  true in order to allow the best translation paths to always be picked.  In
-	  res_fax.c:fax_gateway_framehook() code manually setup the channels to go
-	  through slin and then called ast_channel_make_compatible().  With the
-	  previous version of ast_channel_make_compatible() this was always a
-	  no-operation.
 
-	  * Remove call to ast_channel_make_compatible() in fax_gateway_framehook()
-	  that now undoes what was just setup when the framehook is attached.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422720 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Fixed locking around saving the channel formats in
-	  fax_gateway_framehook() to ensure that the formats that are saved are
-	  consistent.
+2014-09-05 16:56 +0000 [4499eb05d8]  Matt Jordan <mjordan@digium.com>
 
-	  * Fix copy pasta errors in fax_gateway_framehook() that confuses read and
-	  write when dealing with saved channel formats.
+	* main/cdr: Fix crash/memory consumption in CDRs in multi-party bridge scenarios
 
-	  ASTERISK-24841
-	  Reported by: Matt Jordan
+	  This patch fixes an issue where CDRs would get stuck generating an infinite
+	  number of CDRs, eventually crashing Asterisk (and consuming a lot of memory
+	  along the way).
 
-	  Change-Id: I6fda0877104a370af586a5e8cf9e161a484da78d
+	  When a channel enters into a multi-party bridge, the CDR engine creates
+	  mappings of each participant to each other participant, picking the 'A' party
+	  as it goes. So, if we have four channels in a multi-party bridge (Alice, Bob,
+	  Charlie, Denise), we would have something like:
 
-2015-04-17 16:19 +0000 [c59a800707]  Corey Farrell <git@cfware.com>
+	  Alice => Bob
+	  Alice => Charlie
+	  Alice => Denise
+	  Bob => Charlie
+	  Bob => Denise
+	  Charlie => Denise
 
-	* Fix issue with AST_THREADSTORAGE_RAW when DEBUG_THREADLOCALS is enabled.
+	  This works fine when participants enter the bridge a single time.
 
-	  When DEBUG_THREADLOCALS is enabled it causes the threadlocal cleanup to be
-	  called as a function.  This causes a compile error with raw threadstorage as
-	  it uses NULL for cleanup.  This fix uses a macro that provides NULL when
-	  DEBUG_THREADLOCALS is disabled, and replaces the call to "c_cleanup(data);"
-	  with "{};" when DEBUG_THREADLOCALS is enabled.
+	  When a participant leaves a bridge, the CDRs for that channel are transitioned
+	  to a finalized state.
 
-	  ASTERISK-24975 #close
-	  Reported by: Ashley Sanders
+	  The bug occurs if Bob rejoins. When the CDR engine creates mappings between the
+	  channels, it walks through all the participants currently in the bridge, and
+	  realizes that no one in the bridge can create a CDR with the channel (Bob).
+	  As such it creates a new CDR for the candidate and appends it to that
+	  candidate's chain. Unfortunately, on this particular code path, it doesn't
+	  stop traversing the candidate's chain. Since we just added ourselves to the
+	  chain, this causes the loop to keep going, constantly adding new CDRs.
 
-	  Change-Id: I3ef7428ee402816d9fcefa1b3b95830c00d5c402
+	  This patch makes it so the engine bails when it creates a CDR match in this
+	  case.
 
-2015-04-17 15:57 +0000 [e05b076827]  Matt Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3964/
 
-	* Merge "Detect potential forwarding loops based on count." into 13
-2015-04-15 10:38 +0000 [4f1a8dbe92]  Mark Michelson <mmichelson@digium.com>
+	  ASTERISK-24241 #close
+	  Reported by: Deepak Singh Rawat
+	  Tested by: Deepak Singh Rawat
 
-	* Detect potential forwarding loops based on count.
+	  ASTERISK-24208
+	  Reported by: Frankie Chin
+	  ........
 
-	  A potential problem that can arise is the following:
+	  Merged revisions 422715 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  * Bob's phone is programmed to automatically forward to Carol.
-	  * Carol's phone is programmed to automatically forward to Bob.
-	  * Alice calls Bob.
+	  Merged revisions 422716 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  If left unchecked, this results in an endless loops of call forwards
-	  that would eventually result in some sort of fiery crash.
 
-	  Asterisk's method of solving this issue was to track which interfaces
-	  had been dialed. If a destination were dialed a second time, then
-	  the attempt to call that destination would fail since a loop was
-	  detected.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422717 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The problem with this method is that call forwarding has evolved. Some
-	  SIP phones allow for a user to manually forward an incoming call to an
-	  ad-hoc destination. This can mean that:
+2014-09-05 15:38 +0000 [025bd1bf3f]  Richard Mudgett <rmudgett@digium.com>
 
-	  * There are legitimate use cases where a device may be dialed multiple
-	  times, or
-	  * There can be human error when forwarding calls.
+	* func_channel.c: Add missing locking to some CHANNEL() requests.
 
-	  This change removes the old method of detecting forwarding loops in
-	  favor of keeping a count of the number of destinations a channel has
-	  dialed on a particular branch of a call. If the number exceeds the
-	  set number of max forwards, then the call fails. This approach has
-	  the following advantages over the old:
+	  * The CHANNEL() audionativeformat, videonativeformat, audioreadformat, and
+	  audiowriteformat now need locking since the media format rework when
+	  accessing the channel's format pointers.
 
-	  * It is much simpler.
-	  * It can detect loops involving local channels.
-	  * It is user configurable.
+	  * Increased the buffer size for CHANNEL() audionativeformat and
+	  videonativeformat output strings since the allow=all can be a lengthy
+	  list.
 
-	  The only disadvantage it has is that in the case where there is a
-	  legitimate forwarding loop present, it takes longer to detect it.
-	  However, the forwarding loop is still properly detected and the
-	  call is cleaned up as it should be.
+	  * Tweaked the CHANNEL() XML documentation for secure_bridge_signaling,
+	  secure_bridge_media, and state.
 
-	  Address review feedback on gerrit.
+	  * Ensured the output buffer is initialized for secure_bridge_signaling and
+	  secure_bridge_media.
 
-	  * Correct "mfgium" to "Digium"
-	  * Decrement max forwards by one in the case where allocation of the
-	    max forwards datastore is required.
-	  * Remove irrelevant code change from pjsip_global_headers.c
+	  * Made use the locked_copy_string() macro instead of inlining it for trace
+	  and checkhangup.
+	  ........
 
-	  ASTERISK-24958 #close
+	  Merged revisions 422700 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ia7e4b7cd3bccfbd34d9a859838356931bba56c23
-2015-04-11 16:56 +0000 [674b18bdf0]  George Joseph <george.joseph@fairview5.com>
 
-	* pjsip_options: Add qualify_timeout processing and eventing
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422701 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This is the second follow-on to https://reviewboard.asterisk.org/r/4572/ and the
-	  discussion at
-	  http://lists.digium.com/pipermail/asterisk-dev/2015-March/073921.html
+2014-09-05 15:22 +0000 [85878c4dd8]  Jonathan Rose <jrose@digium.com>
 
-	  The basic issues are that changes in contact status don't cause events to be
-	  emitted for the associated endpoint.  Only dynamic contact add/delete actions
-	  update the endpoint.  Also, the qualify timeout is fixed by pjsip at 32 seconds
-	  which is a long time.
+	* Dial API: Add a dial option to indicate the dialed channel will replace dialer
 
-	  This patch makes use of the new transaction timeout feature in r4585 and
-	  provides the following capabilities...
+	  Adds an option to the dial API that marks an outgoing dial as replacing the dialing channel for the purpose of propagating accountcode. When it is used, AST_CHANNEL_REQUESTOR_REPLACEMENT is used instead of AST_CHANNEL_REQUESTOR_BRIDGE_PEER when setting accountcodes on the involved channels with ast_channel_req_accountcodes.
 
-	  1.  A new aor/contact variable 'qualify_timeout' has been added that allows the
-	  user to specify the maximum time in milliseconds to wait for a response to an
-	  OPTIONS message.  The default is 3000ms.  When the timer expires, the contact is
-	  marked unavailable.
+	  Review: https://reviewboard.asterisk.org/r/3968/
+	  ........
 
-	  2.  Contact status changes are now propagated up to the endpoint as follows...
-	  When any contact is 'Available', the endpoint is marked as 'Reachable'.  When
-	  all contacts are 'Unavailable', the endpoint is marked as 'Unreachable'.  The
-	  existing endpoint events are generated appropriately.
+	  Merged revisions 422684 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24863 #close
 
-	  Change-Id: Id0ce0528e58014da1324856ea537e7765466044a
-	  Tested-by: Dmitriy Serov
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422697 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-17 15:29 +0000 [f1abf51b73]  Matt Jordan <mjordan@digium.com>
+2014-09-05 14:39 +0000 [e19017fc00]  Jonathan Rose <jrose@digium.com>
 
-	* Merge "res_pjsip: Refactor endpt_send_request to include transaction timeout" into 13
-2015-04-17 10:30 +0000 [ab5b38e434]  Matt Jordan <mjordan@digium.com>
+	* Call IDs: Fix appearance of call ID in core show channels when NULL
 
-	* Merge "res_pjsip: Add global option to limit the maximum time for initial qualifies" into 13
-2015-04-17 10:25 +0000 [ec77b6148f]  Joshua Colp <jcolp@digium.com>
+	  NULL call IDs were meant to appear as '(none)' but instead were showing
+	  the contents of an uninitialized character buffer.
 
-	* Merge "res_pjsip_pubsub: On notify fail deleted sub_tree is then referenced" into 13
-2015-04-16 10:51 +0000 [b56c1914fa]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24223
+	  Review: https://reviewboard.asterisk.org/r/3979/
+	  ........
 
-	* bridge.c: NULL app causes crash during attended transfer
+	  Merged revisions 422664 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Due to a race condition there was a chance that during an attended transfer the
-	  channel's application would return NULL. This, of course, would cause a crash
-	  when attempting to access the memory. This patch retrieves the channel's app
-	  at an earlier time in processing in hopes that the app name is available.
-	  However, if it is not then "unknown" is used instead. Since some string value
-	  is now always present the crash can no longer occur.
+	  Merged revisions 422665 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24869 #close
-	  Reported by: viniciusfontes
-	  Review:
 
-	  Change-Id: I5134b84c4524906d8148817719d76ffb306488ac
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422683 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-16 13:20 +0000 [8d4ce7cc2b]  Scott Griepentrog <scott@griepentrog.com>
+2014-09-05 12:45 +0000 [5a1de68b9a]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_pubsub: On notify fail deleted sub_tree is then referenced
+	* devicestate.c: Minor tweaks
 
-	  This change makes the send_notify of the sub_tree
-	  not happen when the sub_tree has been deleted due
-	  to the notify call failing, which avoids a crash.
+	  * In ast_state_chan2dev() use ARRAY_LEN() instead of a sentinel value in
+	  chan2dev[].
 
-	  ASTERISK-24970 #close
+	  * Fix some comments in chan_iax2.c.
+	  ........
 
-	  Change-Id: I1f20ffc08b192f59c457293b218025a693992cbf
-2015-04-11 16:39 +0000 [bf46799f0e]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 422661 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip: Refactor endpt_send_request to include transaction timeout
 
-	  This is the first follow-on to https://reviewboard.asterisk.org/r/4572/ and the
-	  discussion at
-	  http://lists.digium.com/pipermail/asterisk-dev/2015-March/073921.html
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422663 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Since we currently have no control over pjproject transaction timeout, this
-	  patch pulls the pjsip_endpt_send_request function out of pjproject and into
-	  res_pjsip/endpt_send_transaction in order to implement that capability.
+2014-09-05 08:29 +0000 [2362d88a18]  Kinsey Moore <kmoore@digium.com>
 
-	  Now when the transaction is initiated, we also schedule our own pj_timer with
-	  our own desired timeout.
+	* Menuselect: Fix incorrect enabling on failed deps
 
-	  If the transaction completes before either timeout, pjproject cancels its timer,
-	  and calls our tsx callback where we cancel our timer and run the app callback.
+	  This corrects a situation where menuselect can incorrectly enable a
+	  module by default that has defaultenabled set to "no" and has
+	  failed/non-selected dependencies. The bug is due to an inverted test
+	  when checking for whether the given module should be set to enabled by
+	  default on load.
 
-	  If the pjproject timer times out first, pjproject calls our tsx callback where
-	  we cancel our timer and run the app callback.
+	  Review: https://reviewboard.asterisk.org/r/3975/
+	  Reported by: John Bigelow
+	  ........
 
-	  If our timer times out first, we terminate the transaction which causes
-	  pjproject to cancel its timer and call our tsx callback where we run the app
-	  callback.
+	  Merged revisions 422646 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Regardless of the scenario, pjproject is calling the tsx callback inside the
-	  group_lock and there are checks in the callback to make sure it doesn't run
-	  twice.
 
-	  As part of this patch ast_sip_send_out_of_dialog_request was created to replace
-	  its similarly named private function.  It takes a new timeout argument in
-	  milliseconds (<= 0 to disable the timeout).
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422647 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24863 #close
-	  Reported-by: George Joseph <george.joseph@fairview5.com>
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+2014-09-04 17:05 +0000 [af75e45da1]  Jonathan Rose <jrose@digium.com>
 
-	  Change-Id: I0778dc730d9689c5147a444a04aee3c1026bf747
-2015-04-11 17:04 +0000 [1b6f6ff841]  George Joseph <george.joseph@fairview5.com>
+	* Manager: Require read permission for SYSTEM in order to send FullyBooted
 
-	* res_pjsip: Add global option to limit the maximum time for initial qualifies
+	  Review: https://reviewboard.asterisk.org/r/3969/
+	  ........
 
-	  Currently when Asterisk starts initial qualifies of contacts are spread out
-	  randomly between 0 and qualify_timeout to prevent network and system overload.
-	  If a contact's qualify_frequency is 5 minutes however, that contact may be
-	  unavailable to accept calls for the entire 5 minutes after startup.  So while
-	  staggering the initial qualifies is a good idea, basing the time on
-	  qualify_timeout could leave contacts unavailable for too long.
+	  Merged revisions 422584 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  This patch adds a new global parameter "max_initial_qualify_time" that sets the
-	  maximum time for the initial qualifies.  This way you could make sure that all
-	  your contacts are initialy, randomly qualified within say 30 seconds but still
-	  have the contact's ongoing qualifies at a 5 minute interval.
+	  Merged revisions 422625 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  If max_initial_qualify_time is > 0, the formula is initial_interval =
-	  min(max_initial_interval, qualify_timeout * random().  If not set,
-	  qualify_timeout is used.
+	  Merged revisions 422626 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The default is "0" (disabled).
+	  Merged revisions 422631 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24863 #close
 
-	  Change-Id: Ib80498aa1ea9923277bef51d6a9015c9c79740f4
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422632 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-15 16:08 +0000 [5d218cde87]  George Joseph <george.joseph@fairview5.com>
+2014-09-03 09:05 +0000 [3cd36d0e10]  Joshua Colp <jcolp@digium.com>
 
-	* More .gitignore updates
+	* res_pjsip_transport_websocket: Fix crash when the Contact header is not a URI.
 
-	  Added .pyc and .sha1 to the top-level .gitignore.
+	  The code for changing the Contact header wrongly assumed that the Contact
+	  would always contain a URI. This is incorrect.
 
-	  Change-Id: I7dfc4f554d54d22947b38140d3305007503cc16a
-	  Tested-by: George Joseph <george.joseph@fairview5.com>
+	  ASTERISK-24271
+	  Reported by: Dafi Ni
+	  ........
 
-2015-04-15 13:36 +0000 [97f83c4c53]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422557 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "Build System: Replace comment about setting menuselect defaults." into 13
-2015-04-14 13:16 +0000 [abd56db3e0]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  Merged revisions 422558 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* cel_pgsql: Fix name string for log on unable allocate memory.
 
-	  The LOG_ERROR has reference to CDR instead of CEL  for LENGTHEN_BUF1 and
-	  LENGTHEN_BUF2.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422559 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24965 #close
-	  Reported by: Rodrigo Ramirez Norambuena
+2014-09-02 15:29 +0000 [1b64f353f1]  Mark Michelson <mmichelson@digium.com>
 
-	  Change-Id: Icc818697d7d66d34bfe3048cdd15ca2b06c89744
-2015-04-14 13:48 +0000 [222fbe1d9a]  Corey Farrell <git@cfware.com>
+	* Resolve race condition where channels enter dialplan application before media has been negotiated.
 
-	* Build System: Replace comment about setting menuselect defaults.
+	  Testsuite tests will occasionally fail because on reception of a 200 OK SIP response,
+	  an AST_CONTROL_ANSWER frame is queued prior to when media has finished being
+	  negotiated. This is because session supplements are called into before PJSIP's
+	  inv_session code has told us that media has been updated. Sometimes the queued answer
+	  frame is handled by the PBX thread before the ensuing media negotiations occur, causing
+	  a test failure.
 
-	  The Makefile claims that you can set default menuselect options by creating
-	  ~/.asterisk.makeopts or /etc/asterisk.makeopts, but those files have never
-	  been respected in Asterisk 11 or 13.  This changes the comment to accurately
-	  reflect that these files are not automatically used by the build system.
+	  As it turns out, there is another place that session supplements could be called into, which is
+	  after media has finished getting negotiated. What this commit introduces is a means for session
+	  supplements to indicate when they wish to be called into when handling an incoming SIP response.
+	  By default, all session supplements will be run at the same point that they were prior to this
+	  commit. However, session supplements may indicate that they wish to be handled earlier than
+	  normal on redirects, or they may indicate they wish to be handled after media has been negotiated.
 
-	  ASTERISK-13721 #close
-	  Reported by: pj
+	  In this changeset, two session supplements have been updated to indicate a preference for when
+	  they should be run: res_pjsip_diversion executes before handling redirection in order to get
+	  information from the Diversion header, and chan_pjsip now handles responses to INVITEs after
+	  media negotiation to fix the race condition mentioned previously.
 
-	  Change-Id: Ibde804ff196283def49ccb9432fbf224a22586e2
+	  ASTERISK-24212 #close
+	  Reported by Matt Jordan
 
-2015-04-12 09:08 +0000 [07e729cc7b]  Rodrigo Ramírez Norambuena <decipher.hk@gmail.com>
+	  Review: https://reviewboard.asterisk.org/r/3930
+	  ........
 
-	* cdr_pgsql: Fix CLI "cdr show pgsql status" command.
+	  Merged revisions 422536 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  The command always showed the usage information.
+	  Merged revisions 422542 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Fix the error in command validation for CLI_SHOWUSAGE.
 
-	  ASTERISK-24959 #close
-	  Reported by: Rodrigo Ramirez Norambuena
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422543 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Change-Id: I584f0936bb01001336a468a55c1d05d79fe795d5
-	  (cherry picked from commit 23a180cade51e84b9def65b05759c3cb9feba225)
+2014-09-01 10:25 +0000 [897cbf6a4f]  Matt Jordan <mjordan@digium.com>
 
-2015-04-13 19:06 +0000 [7d43d85bea]  George Joseph <george.joseph@fairview5.com>
+	* main/cli: Do not attempt to show CDR data for internal channels
 
-	* .gitignore updates for master/13
+	  Internal channels don't have CDRs. Querying the CDR engine for their variables
+	  will make it cranky.
+	  ........
 
-	  Added products of ./bootstrap
+	  Merged revisions 422506 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Added nmenuselect and gmenuselect to menuselect/
+	  Merged revisions 422507 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: Ied658463958bafc04a9aff9ebc28e40c116a6e35
 
-2015-04-13 14:41 +0000 [3d27c223a5]  David M. Lee <dlee@respoke.io>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422524 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Fixing extconf compile
+2014-09-01 09:15 +0000 [df5dbbd878]  Matt Jordan <mjordan@digium.com>
 
-	  During the mass code deletion for clang support, a stray backslash was
-	  left behind that was causing utils to fail to compile.
+	* res_stasis: Don't play MoH to channels by default when added to holding bridges
 
-	  Change-Id: I60e5fa58c9a5b248bde23aaada79ff663f87a2a1
+	  When ARI manipulates a bridge, it generally doesn't care what the mixing
+	  technology is. Operations on a bridge initiated through ARI should perform
+	  their action in generally the same way, regardless of the bridge's mixing
+	  technology. While the mixing technology may determine how media flows to
+	  channels, the actual operations on a bridge themselves should be the same.
+
+	  Currently, this isn't the case with holding bridges. When a channel joins
+	  without a role, MoH is started on that channel automatically. Subsequent bridge
+	  operations that would stop MoH would fail (as there is no Announcer channel
+	  playing MoH to the bridge). Starting MoH on the bridge will also create two
+	  MoH streams: one from the MoH being played on the participant channel, and one
+	  from the announcer channel. From the perspective of ARI users, this is
+	  counter-intuitive - I would not expect MoH to be started for me. The mixing
+	  technology determines how media is shared between participants, not the
+	  application experience.
+
+	  This patch does the following:
+	   * The Stasis bridge class now inspects channels as they are going into a
+	     bridge. If the bridge has a holding capability, and the channel has no
+	     roles, we give it a participant role and mark the default behaviour to have
+	     no entertainment. This allows addChannel operations to continue to set a
+	     participant role with an entertainment option if it felt like it (or could
+	     do it).
+	   * The music on hold channel is now Stasis approved (tm)
+
+	  Review: https://reviewboard.asterisk.org/r/3929/
+
+	  ASTERISK-24264 #close
+	  Reported by: Samuel Galarneau
+	  Tested by: Samuel Galarneau 
+	  ........
 
-2015-04-13 12:03 +0000 [30045b4e67]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422503 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* Merge "build_tools/make_version: Update version parsing for Git migration" into 13
-2015-04-13 10:47 +0000 [88dbf6653e]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 422504 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Merge "res_monitor: Add dependency on func_periodic_hook." into 13
-2015-04-13 09:54 +0000 [e996d8f728]  Matt Jordan <mjordan@digium.com>
 
-	* build_tools/make_version: Update version parsing for Git migration
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422505 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  External systems - such as the Asterisk Test Suite - require knowledge of the
-	  upstream branch. Unfortunately, after moving to Git, the Asterisk version
-	  currently consists of only a 'GIT" prefix followed by an object blob,
-	  e.g., GIT-as08d7. This makes it difficult for such systems to know what
-	  features are available in a particular check out of Asterisk.
+2014-08-30 12:33 +0000 [5aefecd81e]  gtjoseph <george.joseph@fairview5.com>
 
-	  This patch fixes this by hardcoding the branch in a variable in the
-	  make_version script. Since the mainline branches are not changed often -
-	  typically only once a year - this is a reasonable approach to solving
-	  the problem, and is more reliable than parsing the output of 'git branch
-	  -vv'. Branches that track off of an upstream primary branch will then get the
-	  benefit of knowing which mainline branch they are currently based off
-	  of.
+	* confbridge: Add Duration to ConfbridgeList event
 
-	  ASTERISK-24954 #close
+	  The ConfbridgeList event doesn't include how long the user has been a
+	  member of the conference.  This patch adds Duration (seconds) which
+	  is based on user->chan->answertime.
 
-	  Change-Id: I8090d5d548b6d19e917157ed530b914b7eaf9799
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3955/
+	  ........
+
+	  Merged revisions 422444 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-12 12:59 +0000 [d1a6f1a9f9]  Matt Jordan <mjordan@digium.com>
+	  Merged revisions 422445 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* git migration: Remove support for file versions
 
-	  Git does not support the ability to replace a token with a version
-	  string during check-in. While it does have support for replacing a
-	  token on clone, this is somewhat sub-optimal: the token is replaced
-	  with the object hash, which is not particularly easy for human
-	  consumption. What's more, in practice, the source file version was often
-	  not terribly useful. Generally, when triaging bugs, the overall version
-	  of Asterisk is far more useful than an individual SVN version of a file.
-	  As a result, this patch removes Asterisk's support for showing source file
-	  versions.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422446 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Specifically, it does the following:
-	  * main/asterisk:
-	    - Refactor the file_version structure to reflect that it no longer
-	      tracks a version field.
-	    - Alter the "core show file version" CLI command such that it always
-	      reports the version of Asterisk. The file version is no longer
-	      available.
+2014-08-30 12:24 +0000 [59d4dbd3d0]  gtjoseph <george.joseph@fairview5.com>
 
-	  * main/manager: The Version key now always reports the Asterisk version.
+	* manager: Make WaitEvent action respect eventfilters
 
-	  * UPGRADE: Add notes for:
-	    - Modification to the ModuleCheck AMI Action.
-	    - Modification of the "core show file version" CLI command.
+	  A WaitEvent issued via an http session isn't respecting eventfilters defined
+	  for the user. I just added a match_filter to the predicate that controls
+	  astman_append.
 
-	  Change-Id: Ia932d3c64cd18a14a3c894109baa657ec0a85d28
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3958/
+	  ........
 
-2015-04-13 06:19 +0000 [0e4b997cd7]  Corey Farrell <git@cfware.com>
+	  Merged revisions 422439 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	* res_monitor: Add dependency on func_periodic_hook.
+	  Merged revisions 422440 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  OPTIONAL_API has conditionals to define AST_OPTIONAL_API and
-	  AST_OPTIONAL_API_ATTR differently based on if AST_API_MODULE is defined.
-	  Unfortunately this is inside the include protection block, so only the
-	  first status of AST_API_MODULE is respected.  For example res_monitor
-	  is an optional API provider, but uses func_periodic_hook.  This makes
-	  func_periodic_hook non-optional to res_monitor.
+	  Merged revisions 422441 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-17608 #close
-	  Reported by: Warren Selby
+	  Merged revisions 422442 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Change-Id: I8fcf2a5e7b481893e17484ecde4f172c9ffb5679
 
-2015-04-12 15:27 +0000 [91c1ed7ef6]  Matt Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422443 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Merge "main/editline: Add .gitignore." into 13
-2015-04-12 06:12 +0000 [a77c31b99c]  Corey Farrell <git@cfware.com>
+2014-08-29 14:40 +0000 [664f83a03b]  Jeremy Laine (License 6561)
 
-	* main/editline: Add .gitignore.
+	* doc: Add a manpage for the smsq utility
 
-	  This patch adds a .gitignore for main/editline to ignore all build results.
+	  This patch adds a manpage for the smsq utility. Note that this is one of
+	  the patches the Debian distro applies for the Asterisk project, as per
+	  ASTERISK-24191.
 
-	  Change-Id: I68c7bf375ea46282689e5a706534b69fca233b5d
+	  Review: https://reviewboard.asterisk.org/r/3895/
 
-2015-04-11 23:22 +0000 [d918c3b78e]  Matt Jordan <mjordan@digium.com>
+	  ASTERISK-24171 #close
+	  Reported by: Jeremy Laine
+	  patches:
+	    smsq.8 uploaded by Jeremy Laine (License 6561)
+	  ........
 
-	* .gitignore: Ignore tarballs (*.gz)
+	  Merged revisions 422376 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  This patch updates the root .gitignore file to ignore files with a .gz
-	  extension. This will cause git to ignore downloaded sound tarballs in
-	  the the sounds/ directory.
+	  Merged revisions 422377 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Change-Id: I1e42fbfa02a8884231507b683e8e49ac3e278aaa
+	  Merged revisions 422378 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-11 13:20 +0000 [555b5f5d30]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 422379 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Add .gitignore and .gitreview files
 
-	  Add the .gitignore and .gitreview files to the asterisk repo.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422380 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  NB:  You can add local ignores to the .git/info/exclude file
-	  without having to do a commit.
+2014-08-29 14:35 +0000 [81598fa082]  Jeremy Laine (License 6561)
 
-	  Common ignore patterns are in the top-level .gitignore file.
-	  Subdirectory-specific ignore patterns are in their own .gitignore
-	  files.
+	* doc: Add a manpage for the aelparse utility
 
-	  Change-Id: I4c8af3b8e3739957db545f7368ac53f38e99f696
-	  Tested-by: George Joseph
+	  This patch adds a manpage for the aelparse utility. Note that this is one of
+	  the patches the Debian distro applies for the Asterisk project, as per
+	  ASTERISK-24191.
 
-2015-04-11 10:35 +0000 [5807ca519c]  Matthew Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3896/
 
-	* Blocked revisions 434708
+	  ASTERISK-24171 #close
+	  Reported by: Jeremy Laine
+	  patches:
+	    aelparse.8 uploaded by Jeremy Laine (License 6561)
+	  ........
 
+	  Merged revisions 422371 from http://svn.asterisk.org/svn/asterisk/branches/1.8
 	  ........
-	  main/event: Remove unnecessary assignment of negative value to enum
 
-	  When cleaning up some clang compiler warnings, the comparison of a negative
-	  value to an unsigned enum was removed. However, the initial assignment of a
-	  negative value to said enum remained in the variable declaration. This patch
-	  removes that assignment.
+	  Merged revisions 422372 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Thanks to ibercom in #asterisk-bugs for pointing it out.
+	  Merged revisions 422373 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 422374 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434709 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-11 10:26 +0000 [d0d78d5732]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422375 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix various warnings for tests
+2014-08-29 13:46 +0000 [2df2d785b7]  Scott Griepentrog <sgriepentrog@digium.com>
 
-	  This patch fixes a variety of clang compiler warnings for unit tests. This
-	  includes autological comparison issues, ignored return values, and
-	  interestingly enough, one embedded function. Fun!
+	* The assertion that peer was not found on final event
+	  message was being triggered on configuration reload.
+	  This patch changes that case to just return instead.
 
-	  Review: https://reviewboard.asterisk.org/r/4555
+	  Review: https://reviewboard.asterisk.org/r/3953/
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4555.patch submitted by dkdegroot (License 6600)
-	  ........
 
-	  Merged revisions 434705 from http://svn.asterisk.org/svn/asterisk/branches/11
 
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422358 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434706 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+2014-08-28 16:54 +0000 [3194892ea2]  Matt Jordan <mjordan@digium.com>
 
-2015-04-11 10:10 +0000 [4cf7d0bf01]  Juergen Spies (License 6698)
+	* LICENSE: Clarify language in Asterisk's LICENSE to allow for linking to UniMRCP
 
-	* res/res_pjsip_t38: Add missing initialization of t38faxmaxdatagram
+	  The UniMRCP project distributes Asterisk modules that integrate Asterisk with
+	  UniMRCP, and other Asterisk users use the UniMRCP library as well.
+	  Unfortunately, the UniMRCP license is Apache 2.0, which per the Free Software
+	  Foundation, is not a compatible license with the GPLv2.
 
-	  Prior to this patch, the far_max_datagram value on the UDPTL structure would
-	  remain -1 if the remote endpoint fails to provide the SDP media attribute
-	  T38FaxMaxDatagram. This can result in the INVITE request being rejected. With
-	  this patch, we will now properly initialize the value with either the default
-	  value or with the value provided by pjsip.conf's t38_udptl_maxdatagram
-	  parameter.
+	  "Please note that this license is not compatible with GPL version 2, because it
+	  has some requirements that are not in that GPL version. These include certain
+	  patent termination and indemnification provisions. The patent termination
+	  provision is a good thing, which is why we recommend the Apache 2.0 license for
+	  substantial programs over other lax permissive licenses."
 
-	  Review: https://reviewboard.asterisk.org/r/4589
+	  On the other hand, UniMRCP is a great project and we'd like to let people use
+	  it with Asterisk.
 
-	  ASTERISK-24928 #close
-	  Reported by: Juergen Spies
-	  Tested by: Juergen Spies
-	  patches:
-	    pjsipT38patch20150331.txt submitted by Juergen Spies (License 6698)
+	  This patch updates the LICENSE text to allow users to link Asterisk with
+	  UniMRCP and distribute the resulting binaries.
+	  ........
 
+	  Merged revisions 422293 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434688 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 422294 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-04-10 18:29 +0000 [13cd99682d]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 422295 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* chan_pjsip/res_pjsip/bridge_softmix/core: Improve translation path choices.
+	  Merged revisions 422296 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  With this patch, chan_pjsip/res_pjsip now sets the native formats to the
-	  codecs negotiated by a call.
 
-	  * The changes in chan_pjsip.c and res_pjsip_sdp_rtp.c set the native
-	  formats to include all the negotiated audio codecs instead of only the
-	  initial preferred audio codec and later the currently received audio
-	  codec.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422297 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * The audio frame handling in channel.c:ast_read() is more streamlined and
-	  will automatically adjust to changes in received frame formats.  The new
-	  policy is to remove translation and pass the new frame format to the
-	  receiver except if the translation was to a signed linear format.  A more
-	  long winded version is commented in ast_read() along with some caveats.
+2014-08-28 15:31 +0000 [c5916fb39f]  Michael L. Young (license 5026)
 
-	  * The audio frame handling in channel.c:ast_write() is more streamlined
-	  and will automatically adjust any needed translation to changes in the
-	  frame formats sent.  Frame formats sent can change for many reasons such
-	  as a recording is being played back or the bridged peer changed the format
-	  it sends.  Since it is a normal expectation that sent formats can change,
-	  the codec mismatch warning message is demoted to a debug message.
+	* chan_iax2: Fix Dynamic IAX2 Registrations After Temporary DNS Failure
 
-	  * Removed the short circuit check in
-	  channel.c:ast_channel_make_compatible_helper().  Two party bridges need to
-	  make channels compatible with each other.  However, transfers and moving
-	  channels among bridges can result in otherwise compatible channels having
-	  sub-optimal translation paths if the make compatible check is short
-	  circuited.  A result of forcing the reevaluation of channel compatibility
-	  is that the asterisk.conf:transcode_via_slin and codecs.conf:genericplc
-	  options take effect consistently now.  It is unfortunate that these two
-	  options are enabled by default and negate some of the benefits to the
-	  changes in channel.c:ast_read() by forcing translation through signed
-	  linear on a two party bridge.
+	  The reporter on the issue found some issues when upgrading from version 10 to 11
+	  on 55 hosts.
 
-	  * Improved the softmix bridge technology to better control the translation
-	  of frames to the bridge.  All of the incoming translation is now normally
-	  handled by ast_read() instead of splitting any translation steps between
-	  ast_read() and the slin factory.  If any frame comes in with an unexpected
-	  format then the translation path in ast_read() is updated for the next
-	  frame and the slin factory handles the current frame translation.
+	  Two situations that can occur with dynamic registrations.
 
-	  This is the final patch in a series of patches aimed at improving
-	  translation path choices.  The other patches are on the following reviews:
-	  https://reviewboard.asterisk.org/r/4600/
-	  https://reviewboard.asterisk.org/r/4605/
+	  1.  With dnsmgr disabled, if the host is not resolvable we are not trying to
+	      resolve the host again when it is time to attempt to register again.  This
+	      results in never registering to the host.
+	  2.  With dnsmgr enabled, when the host is temporarily not resolvable the
+	      address is set to 0.0.0.0:0 and then when the host is resolvable the port
+	      is not being restored and stays set to 0.
 
-	  ASTERISK-24841 #close
-	  Reported by: Matt Jordan
+	  This patch resolves these two issues by:
 
-	  Review: https://reviewboard.asterisk.org/r/4609/
+	  * Storing the hostname so that it can be used for resolving with DNS.
+	  * Resolve the hostname on the next scheduled attempt to register.
+	  * Storing the port used to reach the host so that when the hostname is
+	    resolvable again, we can set the port again if the port is still unset after
+	    looking up the host.
+
+	  ASTERISK-23767 #close
+	  Reported by: David Herselman
+	  Tested by: David Herselman, Michael L. Young
+	  Patches:
+	      asterisk-23767-dns_reg_retry_and_set_port_11_v3.diff
+	                                       uploaded by Michael L. Young (license 5026)
+
+	  Review: https://reviewboard.asterisk.org/r/3856/
+	  ........
+
+	  Merged revisions 422274 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
+
+	  Merged revisions 422275 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 422276 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434671 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-10 16:03 +0000 [af458e2e60]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422277 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: make progressinband default to no
+2014-08-28 12:29 +0000 [4e750a26fd]  Richard Mudgett <rmudgett@digium.com>
 
-	  After the "progressinband" value setting of "never" was updated to never send a
-	  183 this separated its use from the "no" value. Since "never" was the default,
-	  but most users probably expect "no" this patch updates the default for the
-	  "progressinband" setting to "no."
+	* Added ConfBridge AMI event note to UPGRADE.txt.
+	  ........
 
-	  ASTERISK-24835 #close
-	  Reported by: Andrew Nagy
-	  Review: https://reviewboard.asterisk.org/r/4606/
+	  Merged revisions 422255 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 422256 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434654 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-10 12:53 +0000 [88b0fa7755]  yaron nahum (License 6676)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422257 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res_pjsip: Add an 'auto' option for DTMF Mode
+2014-08-28 11:06 +0000 [ef28cc0d43]  Paul Belanger
 
-	  This patch adds support for automatically detecting the type of DTMF that a
-	  PJSIP endpoint supports. When the 'dtmf_mode' endpoint option is set to 'auto',
-	  the channel created for an endpoint will attempt to determine if RFC 4733
-	  DTMF is supported. If so, it will use that DTMF type. If not, the DTMF type
-	  for the channel will be set to inband.
+	* chan_sip.c: Add 'rtpbindaddr' setting
 
-	  Review: https://reviewboard.asterisk.org/r/4438
+	  Users now have the ability to bind the rtpengine instance to a specific IP
+	  address.  For example, you want chan_sip (call control) on eth0 but rtp (media)
+	  on eth1.
 
-	  ASTERISK-24706 #close
-	  Reported by: yaron nahum
-	  patches:
-	    yaron_patch_3_Feb.diff submitted by yaron nahum (License 6676)
+	  ASTERISK-24280 #close
+	  Reported by: Paul Belanger
+	  Tested by: Paul Belanger
+	  Review: https://reviewboard.asterisk.org/r/3952/
+	  Patches:
+	      rtpengine.diff uploaded by Paul Belanger
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434637 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422241 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-10 11:59 +0000 [16afee4651]  George Joseph <george.joseph@fairview5.com>
+2014-08-28 10:50 +0000 [327d67270f]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip_config_wizard: Cleanup load unload
+	* Fix bug that did not allow for multiple batched RLS notifications to be sent.
 
-	  While investigating other unload issues I realized that the load/unload process 
-	  for the config wizard was pretty ugly so I've refactored it as follows...
+	  A misunderstanding of how the scheduler worked caused further batched notifications
+	  beyond the first not to get scheduled. Now we reset our scheduler ID to -1 after
+	  the batched notification is sent. This way, further notifications can be scheduled
+	  when they arise.
+	  ........
 
-	  When the res_pjsip sorcery instance is created the config_wizard bumps it's own 
-	  module reference to prevent it from unloading while the sorcery instance is 
-	  still active.  When res_pjsip unloads and it's sorcery instance is destroyed, 
-	  the config wizard unrefs itself which then allows itself to unload cleanly.  
-	  Since the config wizard now can't load after res_pjsip or unload before it 
-	  (which should have been the correct behavior all along), I was able to remove 
-	  the chunks of code in both load_module and unload_module that handled that case.
+	  Merged revisions 422239 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Ran the testsuite tests to insure there were no functional changes and REF_DEBUG 
-	  to insure that Asterisk was shutting down cleanly with no FRACKs or leaks.
 
-	  Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4610/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422240 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-27 19:44 +0000 [94e1b4a8a4]  Richard Mudgett <rmudgett@digium.com>
 
+	* res/res_pjsip/pjsip_options.c: Eliminate excessive RAII_VAR usage.
 
+	  * Fix off nominal ref leak in find_or_create_contact_status().
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434619 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  * Add missing NULL check of status in update_contact_status() and
+	  init_start_time().
+	  ........
 
-2015-04-10 11:37 +0000 [125acc52fe]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 422214 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* bridge_softmix.c,channel.c: Minor code simplification and cleanup.
+	  Merged revisions 422215 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Made code easier to follow in bridge_softmix.c:analyse_softmix_stats()
-	  and made some debug messages more helpful.
 
-	  * Made some debug and warning messages more helpful in
-	  channel.c:set_format().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422216 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4607/
+2014-08-27 19:16 +0000 [4728c05957]  Richard Mudgett <rmudgett@digium.com>
 
+	* sched: Fix typo and whitespace change.
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434617 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 422200 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-10 11:28 +0000 [a63f7ad04a]  Richard Mudgett <rmudgett@digium.com>
 
-	* translate.c: Only select audio codecs to determine the best translation choice.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422201 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Given a source capability of h264 and ulaw, a destination capability of
-	  h264 and g722 then ast_translator_best_choice() would pick h264 as the
-	  best choice even though h264 is a video codec and Asterisk only supports
-	  translation of audio codecs.  When the audio starts flowing, there are
-	  warnings about a codec mismatch when the channel tries to write a frame to
-	  the peer.
+2014-08-27 12:30 +0000 [7c1a22fba7]  gtjoseph <george.joseph@fairview5.com>
 
-	  * Made ast_translator_best_choice() only select audio codecs.
+	* confbridge: Add 'Admin' param to join, leave, mute, unmute and talking events
 
-	  * Restore a check in channel.c:set_format() lost after v1.8 to prevent
-	  trying to set a non-audio codec.
+	  Currently there's no way to tell if a user is an admin or not when receiving
+	  the join, leave, mute, unmute and talking events.  This patch adds that
+	  capability.
 
-	  This is an intermediate patch for a series of patches aimed at improving
-	  translation path choices for ASTERISK-24841.
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3950/
+	  ........
 
-	  This patch is a complete enough fix for ASTERISK-21777 as the v11 version
-	  of ast_translator_best_choice() does the same thing.  However, chan_sip.c
-	  still somehow tries to call ast_codec_choose() which then calls
-	  ast_best_codec() with a capability set that doesn't contain any audio
-	  formats for the incoming call.  The remaining warning message seems to be
-	  a benign transient.
+	  Merged revisions 422176 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-21777 #close
-	  Reported by: Nick Ruggles
+	  Merged revisions 422177 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24380 #close
-	  Reported by: Matt Jordan
 
-	  Review: https://reviewboard.asterisk.org/r/4605/
-	  ........
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422178 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Merged revisions 434614 from http://svn.asterisk.org/svn/asterisk/branches/11
+2014-08-27 10:39 +0000 [bf85018107]  Kinsey Moore <kmoore@digium.com>
 
+	* CallerID: Fix parsing of malformed callerid
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434615 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  This allows the callerid parsing function to handle malformed input
+	  strings and strings containing escaped and unescaped double quotes.
+	  This also adds a unittest to cover many of the cases where the parsing
+	  algorithm previously failed.
 
-2015-04-10 09:55 +0000 [c9791dba1f]  Matthew Jordan <mjordan@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3923/
+	  Review: https://reviewboard.asterisk.org/r/3933/
+	  ........
 
-	* res/ari: Fix model validation for ChannelHold event
+	  Merged revisions 422112 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  When the ChannelHold event was added, the 'musicclass' parameter was
-	  erroneously removed. This caused the ChannelHold events to be rejected as
-	  they failed model validation. This patch updates the Swagger schema such that
-	  it now properly reflects the event that is being created.
+	  Merged revisions 422113 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Hooray for tests that catch things like this.
+	  Merged revisions 422114 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 422154 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434597 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-10 07:39 +0000 [c39faa4729]  Y Ateya (License 6693)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422158 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* channels/chan_iax2: Improve POKE expiration time calculation for lossy networks
+2014-08-26 18:30 +0000 [d199536a04]  gtjoseph <george.joseph@fairview5.com>
 
-	  POKE is used to check for peer availability; however, in networks with packet
-	  loss, the current calculations may result in POKE expiration times that are too
-	  short. This patch alters the expiration/retry time logic to take into account
-	  the last known qualify round trip time, as opposed to always using a static
-	  value for each peer.
+	* confbridge: Make kick, mute and unmute handle channel targets consistently.
 
-	  Review: https://reviewboard.asterisk.org/r/4536
+	  Kick, mute and unmute were a little inconsistent in their handling of channel
+	  targets.  This patch cleans that up by insuring they all handle the 'all'
+	  target consistently and adds the 'participants' target which acts on
+	  non-admins.  Documentation for kick was also cleaned up as it never
+	  supported partial channel names.
 
-	  ASTERISK-22352 #close
-	  Reported by: Frederic Van Espen
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3944/
+	  ........
 
-	  ASTERISK-24894 #close
-	  Reported by: Y Ateya
-	  patches:
-	    poke_noanswer_duration.diff submitted by Y Ateya (License 6693)
+	  Merged revisions 422090 from http://svn.asterisk.org/svn/asterisk/branches/12
 	  ........
 
-	  Merged revisions 434564 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 422091 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434565 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422092 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-09 17:35 +0000 [75c2c85962]  George Joseph <george.joseph@fairview5.com>
+2014-08-26 17:14 +0000 [c5ab4adf17]  Mark Michelson <mmichelson@digium.com>
 
-	* res_pjsip_phoneprov_provider: Fix reference leak on unload
+	* Fix race condition in the scheduler when deleting a running entry.
 
-	  res_pjsip_phoneprov_provider was leaking references to phoneprov objects due to 
-	  a missing OBJ_NODATA in an ao2_callback in load_users().  Rather than adding the 
-	  OBJ_NODATA, I changed load_users to use a more straightforward ao2_iterator.  
-	  This plugged the leak but exposed an unload order issue between 
-	  res_pjsip_phoneprov_provider, res_phoneprov and res_pjsip.
+	  When scheduled tasks run, they are removed from the heap (or hashtab).
+	  When a scheduled task is deleted, if the task can't be found in the
+	  heap (or hashtab), an assertion is triggered. If DO_CRASH is enabled,
+	  this assertion causes a crash.
 
-	  res_pjsip_phoneprov_provider unloads first, then res_phoneprov, then res_pjsip.  
-	  Since res_pjsip_phoneprov_provider uses res_pjsip's sorcery instance, when it 
-	  unloads, it's objects are still in the sorcery instance.  When res_pjsip 
-	  unloads, it destroys all its objects including res_pjsip_phoneprov_provider's.  
-	  The phoneprov destructor then attempts to unregister the extension from 
-	  res_phoneprov but because res_phoneprov is already cleaned up, its users 
-	  container is gone and we get a FRACK.
+	  The problem is, sometimes it just so happens that someone attempts
+	  to delete a scheduled task at the time that it is running, leading
+	  to a crash. This change corrects the issue by tracking which task
+	  is currently running. If that task is attempted to be deleted,
+	  then we mark the task, and then wait for the task to complete.
+	  This way, we can be sure to coordinate task deletion and memory
+	  freeing.
 
-	  Simple solution, check for the NULL users container before attempting to remove 
-	  the entry. Duh.
+	  ASTERISK-24212
+	  Reported by Matt Jordan
 
-	  Ran tests/res_phoneprov/res_phoneprov_provider.  No leaks in 
-	  res_pjsip_phoneprov_provider and no FRACKs.
+	  Review: https://reviewboard.asterisk.org/r/3927
+	  ........
 
-	  Reported-by: Corey Farrell
-	  Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4608/
-	  ASTERISK-24935 #close
+	  Merged revisions 422070 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 422071 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434545 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422072 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-09 17:31 +0000 [73c286a393]  George Joseph <george.joseph@fairview5.com>
+2014-08-25 11:45 +0000 [fefa6fba82]  Richard Mudgett <rmudgett@digium.com>
 
-	* loader/main: Don't set ast_fully_booted until deferred reloads are processed
+	* res_musiconhold.c: Release any format refs before memset().
 
-	  Until we have a true module management facility it's sometimes necessary for one 
-	  module to force a reload on another before its own load is complete.  If 
-	  Asterisk isn't fully booted yet, these reloads are deferred.  The problem is 
-	  that asterisk reports fully booted before processing the deferred reloads which 
-	  means Asterisk really isn't quite ready when it says it is.
+	  * Clear the channel music_state pointer before destroying the music_state
+	  object for safety.
+	  ........
 
-	  This patch moves the report of fully booted after the processing of the deferred 
-	  reloads is complete.
+	  Merged revisions 422037 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Since the pjsip stack has the most number of related modules, I ran the 
-	  channels/pjsip testsuite to make sure there aren't any issues.  All tests 
-	  passed.
 
-	  Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4604/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@422038 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-25 11:16 +0000 [2b19d94a71]  Richard Mudgett <rmudgett@digium.com>
 
+	* res_musiconhold: Fix MOH restarting where it left off from the last hold.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434544 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Restore code removed by https://reviewboard.asterisk.org/r/3536/ that
+	  introduced a regression that prevents MOH from restarting were it left off
+	  the last time.
 
-2015-04-09 17:03 +0000 [5737650a67]  Kevin Harwell <kharwell@digium.com>
+	  ASTERISK-24019 #close
+	  Reported by: Jason Richards
+	  Patches:
+	        jira_asterisk_24019_v1.8.patch (license #5621) patch uploaded by rmudgett
 
-	* res_pjsip: add CLI command to show global and system configuration
+	  Review: https://reviewboard.asterisk.org/r/3928/
+	  ........
 
-	  Added a new CLI command for res_pjsip that shows both global and system
-	  configuration settings: pjsip show settings
+	  Merged revisions 421976 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  ASTERISK-24918 #close
-	  Reported by: Scott Griepentrog
-	  Review: https://reviewboard.asterisk.org/r/4597/
+	  Merged revisions 421977 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421978 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434527 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421979 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-09 11:07 +0000 [1695a5b85f]  Richard Mudgett <rmudgett@digium.com>
 
-	* chan_iax2.c: Fix ref leak in iax2_request().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421980 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Increased warning message format capability string buffer size in
-	  iax2_request().
+2014-08-24 14:37 +0000 [497a92d079]  Joshua Colp <jcolp@digium.com>
 
-	  Review: https://reviewboard.asterisk.org/r/4601/
+	* res_pjsip_transport_websocket: Attach the Websocket module on outgoing INVITEs.
 
+	  In order to alter the Contact header on in-dialog requests and responses the
+	  Websocket module must be attached on outgoing INVITEs. The Contact header is
+	  modified so that the PJSIP transport layer can find and use the existing
+	  Websocket connection based on the source IP address, port, and transport.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434510 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  ASTERISK-24143 #close
+	  Reported by: Aleksei Kulakov
+	  ........
 
-2015-04-09 10:54 +0000 [92c1688edb]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 421955 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* bridge_native_rtp.c: Defer allocation and check if it fails in native_rtp_bridge_compatible().
+	  Merged revisions 421956 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Review: https://reviewboard.asterisk.org/r/4601/
 
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421957 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434508 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+2014-08-24 14:21 +0000 [477e2e6edb]  Joshua Colp <jcolp@digium.com>
 
-2015-04-09 10:42 +0000 [2679d0100a]  yaron nahum (License 6676)
+	* res_pjsip_transport_websocket: Fix a progressive memory growth.
 
-	* res/res_pjsip_dlg_options: Add a module to handle in-dialog OPTIONS requests
+	  The packet structure used to receive messages was using the transport
+	  pool. This meant that for each parsing the pool would grow accordingly.
+	  Since memory can not be reclaimed without resetting it this would
+	  cause the memory pool to grow and grow.
 
-	  This patch adds a new session supplement that handles in-dialog OPTIONS
-	  requests. Said OPTIONS requests are sent a 200 OK, as an endpoint lookup
-	  for the OPTIONS request would already have been done by the time the
-	  session supplement receives the inbound request.
+	  This change uses a specific memory pool for the packet structure and
+	  resets it to a fresh state after the message has been received and
+	  handled.
+	  ........
 
-	  ASTERISK-24862 #close
-	  Reported by: yaron nahum
-	  patches:
-	    res_pjsip_dlg_options.c submitted by yaron nahum (License 6676)
+	  Merged revisions 421939 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421945 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434506 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-09 07:56 +0000 [6ba6e3dffd]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421950 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix autological comparisons
+2014-08-24 13:54 +0000 [2c0cbf8e64]  Joshua Colp <jcolp@digium.com>
 
-	  This fixes autological comparison warnings in the following:
-	   * chan_skinny: letohl may return a signed or unsigned value, depending on the
-	     macro chosen
-	   * func_curl: Provide a specific cast to CURLoption to prevent mismatch
-	   * cel: Fix enum comparisons where the enum can never be negative
-	   * enum: Fix comparison of return result of dn_expand, which returns a signed
-	     int value
-	   * event: Fix enum comparisons where the enum can never be negative
-	   * indications: tone_data.freq1 and freq2 are unsigned, and hence can never be
-	     negative
-	   * presencestate: Use the actual enum value for INVALID state
-	   * security_events: Fix enum comparisons where the enum can never be negative
-	   * udptl: Don't bother to check if the return value from encode_length is less
-	     than 0, as it returns an unsigned int
-	   * translate: Since the parameters are unsigned int, don't bother checking
-	     to see if they are negative. The cast to unsigned int would already blow
-	     past the matrix bounds.
-	   * res_pjsip_exten_state: Use a temporary value to cache the return of
-	     ast_hint_presence_state
-	   * res_stasis_playback: Fix enum comparisons where the enum can never be
-	     negative
-	   * res_stasis_recording: Add an enum value for the case where the recording
-	     operation is in error; fix enum comparisons
-	   * resource_bridges: Use enum value as opposed to -1
-	   * resource_channels: Use enum value as opposed to -1
+	* res_pjsip_transport_websocket: Ensure secure Websocket clients can be called.
 
-	  Review: https://reviewboard.asterisk.org/r/4533
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4533.patch submitted by dkdegroot (License 6600)
+	  This change enforces the transport in the Contact header for Websocket clients.
+	  Previously a client may provide a transport of 'ws' when it is actually using
+	  a transport of 'wss'. This would cause outgoing calls to fail as the existing
+	  connection could not be found.
 	  ........
 
-	  Merged revisions 434469 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421931 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421932 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434470 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-08 21:05 +0000 [e05c8ae68e]  Stefan Engström (License 6691)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421933 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* apps/app_queue: Prevent possible crash when evaluating queue penalty rules
+2014-08-24 12:22 +0000 [cee660dadf]  Badalian Vyacheslav (license 5249)
 
-	  Although it only occurred once, a crash occurred when a queue attempted to
-	  evaluate a queue penalty rule that appeared to have already been destroyed.
-	  In many locations in app_queue, a test is done to see if qe->pr is NULL;
-	  however, when we dispose of a queue's penalty rules, we don't set the pointer
-	  to NULL after free'ing it. This patch does that to prevent any dangling
-	  pointers from lingering on the queue object.
+	* chan_sip: Use the server reflexive ICE candidate RTCP port as provided.
 
-	  Review: https://reviewboard.asterisk.org/r/4522
+	  This code originally worked around an issue within res_rtp_asterisk itself.
+	  The wrong socket was being used for the STUN check for RTCP, causing the
+	  port to be the same as RTP. This was subsequently fixed and the RTCP port
+	  provided for the ICE candidate is correct and does not need to be incremented.
 
-	  ASTERISK-23319 #close
-	  Reported by: Vadim
-	  patches:
-	    rb4552.patch submitted by Stefan Engström (License 6691)
+	  ASTERISK-23997 #close
+	  Reported by: Badalian Vyacheslav
+	  Patches:
+	   plus1.diff submitted by Badalian Vyacheslav (license 5249)
 	  ........
 
-	  Merged revisions 434448 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421909 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421910 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434449 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421911 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-08 13:15 +0000 [f21b45db49]  Jonathan Rose <jrose@digium.com>
 
-	* res_pjsip_t38: Fix FAX failures when using PJSIP with authentication
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421912 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Without this patch, if a PJSIP endpoint with udptl enabled and authentication
-	  set attempted to use sendFax, the FAX session would fail during setup. This
-	  was because the invite issued in response to being auth challenged would cause
-	  the PJSIP channel performing the FAX to receive a second T38 framehook and
-	  this would cause frames to be consumed in an inappropriate manner.
+2014-08-22 11:56 +0000 [dcfffce66d]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-24933 #close
-	  Reported by: Jonathan Rose
-	  Review: https://reviewboard.asterisk.org/r/4577/
+	* Fix a locking inversion in MixMonitor.
 
+	  We need to unlock the audiohook before trying to lock
+	  the channel, since the correct locking order is channel
+	  then audiohook.
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434425 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421882 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-08 13:14 +0000 [4441bb6a25]  Richard Mudgett <rmudgett@digium.com>
 
-	* Bridging: Eliminate the unnecessary make channel compatible with bridge operation.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421883 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  When a channel enters the bridging system it is first made compatible with
-	  the bridge and then the bridge technology makes the channel compatible
-	  with the technology.  For all but the DAHDI native and softmix bridge
-	  technologies the make channel compatible with the bridge step is an
-	  effective noop because the other technologies allow all audio formats.
-	  For the DAHDI native bridge technology it doesn't matter because it is not
-	  an initial bridge technology and chan_dahdi allows only one native format
-	  per channel.  For the softmix bridge technology, it is a noop at best and
-	  harmful at worst because the wrong translation path could be setup if the
-	  channel's native formats allow more than one audio format.
+2014-08-22 11:52 +0000 [33835e17a0]  Jonathan Rose <jrose@digium.com>
 
-	  This is an intermediate patch for a series of patches aimed at improving
-	  translation path choices.
+	* ARI: Fix a crash caused by hanging during playback to a channel in a bridge
 
-	  * Removed code dealing with the unnecessary step of making the channel
-	  compatible with the bridge.
+	  ASTERISK-24147 #close
+	  Reported by: Edvin Vidmar
+	  Review: https://reviewboard.asterisk.org/r/3908/
+	  ........
 
-	  ASTERISK-24841
-	  Reported by: Matt Jordan
+	  Merged revisions 421879 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Review: https://reviewboard.asterisk.org/r/4600/
+	  Merged revisions 421880 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434424 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421881 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-08 11:40 +0000 [f767440906]  mhej (license 6085)
+2014-08-22 09:09 +0000 [1498ae0830]  Matt Jordan <mjordan@digium.com>
 
-	* Security/tcptls: MitM Attack potential from certificate with NULL byte in CN.
+	* main/message: Add a new-line to a DEBUG message
+	  ........
 
-	  When registering to a SIP server with TLS, Asterisk will accept CA signed
-	  certificates with a common name that was signed for a domain other than the
-	  one requested if it contains a null character in the common name portion of
-	  the cert. This patch fixes that by checking that the common name length
-	  matches the the length of the content we actually read from the common name
-	  segment. Some certificate authorities automatically sign CA requests when
-	  the requesting CN isn't already taken, so an attacker could potentially
-	  register a CN with something like www.google.com\x00www.secretlyevil.net
-	  and have their certificate signed and Asterisk would accept that certificate
-	  as though it had been for www.google.com - this is a security fix and is
-	  noted in AST-2015-003.
+	  Merged revisions 421859 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24847 #close
-	  Reported by: Maciej Szmigiero
-	  Patches:
-	   asterisk-null-in-cn.patch submitted by mhej (license 6085)
+	  Merged revisions 421860 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421861 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-08-21 17:09 +0000 [f8c4fc1121]  Richard Mudgett <rmudgett@digium.com>
+
+	* res_musiconhold.c: Remove obsolete REF_DEBUG code.
+
+	  Remove unneeded code that writes to the wrong file location in an obsolete
+	  format.
 	  ........
 
-	  Merged revisions 434337 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  Merged revisions 421799 from http://svn.asterisk.org/svn/asterisk/branches/1.8
 	  ........
 
-	  Merged revisions 434338 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421800 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421801 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434384 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421802 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-08 11:23 +0000 [1712d16825]  Richard Mudgett <rmudgett@digium.com>
 
-	* format_cache.c: Add missing slin12 format to ast_format_cache_is_slinear().
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421803 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434357 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+2014-08-21 16:43 +0000 [644e693645]  Mark Michelson <mmichelson@digium.com>
 
-2015-04-08 07:33 +0000 [ae39dd1f46]  Matthew Jordan <mjordan@digium.com>
+	* Switch from hostname to an IP address in the SDP origin line.
 
-	* chan_iax2: Fix compilation issue due to funky merge
+	  Using the hostname in the SDP origin line may not satisfy the requirement
+	  of RFC 4566 that we use a FQDN or IP address. This change has us use the
+	  same information from the SDP connection line if possible. If not possible,
+	  we'll use the configured media address. And if that's not possible, we use
+	  the result of a PJLIB call to get the IP address of ourself.
 
-	  Don't mix declarations and code
+	  ASTERISK-23994 #close
+	  Reported by Private Name
 
+	  Review: https://reviewboard.asterisk.org/r/3925
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434314 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421796 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-08 07:00 +0000 [05397ad01e]  Jaco Kroon (License 5671)
+	  Merged revisions 421797 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_iax2: Fix crash caused by unprotected access to iaxs[peer->callno]
 
-	  This patch fixes an access to the peer callnumber that is unprotected by a
-	  corresponding mutex. The peer->callno value can be changed by multiple threads,
-	  and all data inside the iaxs array must be procted by a corresponding lock
-	  of iaxsl.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421798 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  The patch moves the unprotected access to a location where the mutex is
-	  safely obtained.
+2014-08-21 16:37 +0000 [56a1d4930a]  Mark Michelson <mmichelson@digium.com>
 
-	  Review: https://reviewboard.asterisk.org/r/4599/
+	* Ensure after-bridge behavior is correct when moving from Stasis to a non-Stasis bridge.
 
-	  ASTERISK-21211 #close
-	  Reported by: Jaco Kroon
-	  patches:
-	    asterisk-11.2.1-iax2_poke-segfault.diff submitted by Jaco Kroon (License 5671)
+	  Because of the departable state of channels that enter Stasis bridges, Stasis has to
+	  take responsibility for directing the channel to its intended after-bridge destination
+	  if the channel moves from a Stasis bridge to a non-Stasis bridge. This change ensures
+	  that when such a move occurs, when the channel leaves the bridging system, any after
+	  bridge gotos are honored.
+
+	  Review: https://reviewboard.asterisk.org/r/3920
 	  ........
 
-	  Merged revisions 434291 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421792 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421794 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434292 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-08 06:53 +0000 [be13c72142]  Valentin Vidić (License 6697)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421795 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* chan_sip: Handle IPv4 mapped IPv6 clients when NAT is enabled
+2014-08-21 16:35 +0000 [4946981646]  Jonathan Rose (license 6182)
 
-	  When udpbindaddr is set to the IPv6 bind all address of '::', Asterisk will
-	  attempt to handle both IPv4 and IPv6 addresses, although the information will
-	  be stored in a struct with an AF_INET6 address type. However, the current
-	  NAT handling code won't handle the IPv4 mapped IPv6 addresses correctly.
-	  This patch adds an additional check for the mapped address case, allowing
-	  the NAT code to handle clients even when the address is IPv6.
+	* res_musiconhold: Fix reference leaks caused when reloading with REF_DEBUG set
 
-	  Review: https://reviewboard.asterisk.org/r/4563/
+	  Due to a faulty function for debugging reference decrementing, it was possible
+	  to reduce the refcount on the wrong object if two moh classes of the same name
+	  were in the moh class container.
 
-	  ASTERISK-18032 #close
-	  Reported by: Christoph Timm
-	  patches:
-	    nat_with_ipv6.diff submitted by Valentin Vidić (License 6697)
+	  (closes issue ASTERISK-22252)
+	  Reported by: Walter Doekes
+	  Patches:
+	      18_moh_debug_ref_patch.diff Uploaded by Jonathan Rose (license 6182)
 	  ........
 
-	  Merged revisions 434288 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 398937 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 421777 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434289 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421779 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-08 06:44 +0000 [f324870dab]  dkdegroot (License 6600)
+	  Merged revisions 421788 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* clang compiler warnings: Fix pointer-bool-converesion warnings
 
-	  This patch fixes several warnings pointed out by the clang compiler.
-	  * chan_pjsip: Removed check for data->text, as it will always be non-NULL.
-	  * app_minivm: Fixed evaluation of etemplate->locale, which will always
-	    evaluate to 'true'. This patch changes the evaluation to use
-	    ast_strlen_zero.
-	  * app_queue:
-	    - Fixed evaluation of qe->parent->monfmt, which always evaluates to
-	      true. Instead, we just check to see if the dereferenced pointer
-	      evaluates to true.
-	    - Fixed evaluation of mem->state_interface, wrapping it with a call to
-	      ast_strlen_zero.
-	  * res_smdi: Wrapped search_msg->mesg_desk_term with calls to ast_strlen_zero.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421793 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4541
+2014-08-21 16:28 +0000 [12d34bb12f]  Mark Michelson <mmichelson@digium.com>
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4541.patch submitted by dkdegroot (License 6600)
+	* Let's try checking the name and number, instead of the name twice.
 	  ........
 
-	  Merged revisions 434285 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421789 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421790 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434286 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-07 14:38 +0000 [a6aed7f6f6]  Scott Griepentrog <sgriepentrog@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421791 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Revert accidental change in r434261
+2014-08-21 16:19 +0000 [2150daf748]  Mark Michelson <mmichelson@digium.com>
 
+	* Improve consistency of party ID privacy usage.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434262 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Prior to this change, the Remote-Party-ID header took the position of
+	  "If caller name and number are not explicitly allowed, then they are private"
+	  and P-Asserted-Identity took the position of
+	  "Caller name and number are only private if marked explicitly so"
 
-2015-04-07 14:35 +0000 [0584e29300]  Scott Griepentrog <sgriepentrog@digium.com>
+	  Now both mechanisms of conveying party identification use the former approach.
+	  ........
 
-	* pjsip: resolve compatibility problem with ast_sip_session
+	  Merged revisions 421778 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  A change in r430179 inserted a variable near the top of a
-	  structure caused a problem when running DPMA in a version
-	  of Asterisk compiled across the change.  This patch moves
-	  the new variable to the end of the structure, eliminating
-	  the problem.
+	  Merged revisions 421783 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Review: https://reviewboard.asterisk.org/r/4574/
-	  ........
 
-	  Merged revisions 433944 from http://svn.asterisk.org/svn/asterisk/branches/13
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421785 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-21 12:35 +0000 [77ddc5b713]  Elazar Broad (License 5835)
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434261 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	* chan_sip: Don't use port derived from fromdomain if it isn't set
 
-2015-04-07 11:40 +0000 [d754f70239]  Kevin Harwell <kharwell@digium.com>
+	  If a user does not provide a port in the fromdomain setting, chan_sip will set
+	  the fromdomainport to STANDARD_SIP_PORT (5060). The fromdomainport value will
+	  then get used unilaterally in certain places. This causes issues with TLS,
+	  where the default port is expected to be 5061.
 
-	* bridge.c: Hangup attended transfer target after it has been swapped out
+	  This patch modifies chan_sip such that fromdomainport is only used if it is
+	  not the standard SIP port; otherwise, the port from the SIP pvt's recorded
+	  self IP address is used.
 
-	  After completing an attended transfer the transfer target channel (the one that
-	  gets swapped out) was not being hung up after leaving the bridge. This resulted
-	  in a channel possibly being left around. Added an explicit softhangup for the
-	  channel in question after the transfer is successfully completed in order to
-	  make sure the channel is hung up.
+	  Review: https://reviewboard.asterisk.org/r/3893/
 
-	  ASTERISK-24782 #close
-	  Reported by: John Bigelow
-	  Review: https://reviewboard.asterisk.org/r/4575/
+	  ASTERISK-24178 #close
+	  Reported by: Elazar Broad
+	  patches:
+	    fromdomainport_fix.diff uploaded by Elazar Broad (License 5835)
+	  ........
 
+	  Merged revisions 421717 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 421718 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434240 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421719 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-07 10:33 +0000 [c516981dc7]  Mark Michelson <mmichelson@digium.com>
+	  Merged revisions 421720 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Do not queue message requests that we do not respond to.
 
-	  If we receive a MESSAGE request that we cannot send a response
-	  to, we should not send the incoming MESSAGE to the dialplan.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421721 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This commit should help the bouncing message_retrans test to
-	  pass consistently.
+2014-08-21 10:25 +0000 [f3a525e9a6]  Matt Jordan <mjordan@digium.com>
 
+	* ARI: Fix implicit answer when playback is initiated on unanswered channel
 
+	  When issuing a POST /channels/{channel_id}/play on a channel that is not
+	  yet answered, ARI is supposed to:
+	  * Queue up an AST_CONTROL_PROGRESS on the channel
+	  * Start up the playback of the media
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434218 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Instead, we sneak an answer on the channel right before starting playing media.
 
-2015-04-07 10:21 +0000 [ab803ec342]  Matthew Jordan <mjordan@digium.com>
+	  This is due to ARI's usage of control_streamfile. This function implicitly
+	  answers the channel (and doesn't give ARI the option to stop it). The answering
+	  of the channel here is probably unnecessary:
+	  * app_voicemail, by far the biggest consumer of this function, always answers
+	    the channels anyway
+	  * control stream file (in res_agi) and ControlPlayback probably shouldn't be
+	    implicitly answering the channel. Answering should not be tied directly to
+	    playing back media.
 
-	* ARI: Add the ability to intercept hold and raise an event
+	  As it turns out, the answering of the channel here is pretty old:
+	  356042    twilson       if (ast_channel_state(chan) != AST_STATE_UP) {
+	    3087      anthm               res = ast_answer(chan);
+	  180259   tilghman       }
 
-	  For some applications - such as SLA - a phone pressing hold should not behave
-	  in the fashion that the Asterisk core would like it to. Instead, the hold
-	  action has some application specific behaviour associated with it - such as
-	  disconnecting the channel that initiated the hold; only playing MoH to channels
-	  in the bridge if the channels are of a particular type, etc.
+	  (As in, ancient?)
 
-	  One way of accomplishing this is to use a framehook to intercept the
-	  hold/unhold frames, raise an event, and eat the frame. Tasty. This patch
-	  accomplishes that using a new dialplan function, HOLD_INTERCEPT.
+	  Note that others ran into this problem and commented about it on various
+	  mailing lists.
 
-	  In addition, some general cleanup of raising hold/unhold Stasis messages was
-	  done, including removing some RAII_VAR usage.
+	  Review: https://reviewboard.asterisk.org/r/3907/
 
-	  Review: https://reviewboard.asterisk.org/r/4549/
+	  ASTERISK-24229 #close
+	  Reported by: Matt Jordan
+	  ........
 
-	  ASTERISK-24922 #close
+	  Merged revisions 421695 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421696 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434216 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 21:09 +0000 [488f093e97]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421699 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix sometimes-initialized warning in func_math
+2014-08-21 09:52 +0000 [085d5a2629]  Shaun Ruffell (License 5417)
 
-	  This patch fixes a bug in a unit test in func_math where a variable could be
-	  passed to ast_free that wasn't allocated. This patch corrects the issue and
-	  ensures that we only attempt to free a variable if we previously allocated
-	  it.
+	* Clean up files that do not end with newlines
 
-	  Review: https://reviewboard.asterisk.org/r/4552
+	  Trivial patch to add new lines to several files missing them. This fixes
+	  warnings when compiling with gcc 4.1.2 on CentOS 5.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
+	  ASTERISK-24245 #close
+	  Reported by: Shaun Ruffell
 	  patches:
-	    rb4552.patch submitted by dkdegroot (License 6600)
+	    0002-Trivial-addition-of-newlines-at-end-of-three-files.patch uploaded by Shaun Ruffell (License 5417)
 	  ........
 
-	  Merged revisions 434190 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421677 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421678 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434191 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 21:03 +0000 [c027133f6d]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421679 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix non-literal-null-conversion warnings
+2014-08-21 09:42 +0000 [da91946df7]  Shaun Ruffell (License 5417)
 
-	  Clang will flag errors when a char pointer is set to '\0', as opposed to a
-	  value that the char pointer points to. This patch fixes this warning
-	  in a variety of locations.
+	* uri: Quiet warning about type qualifiers ignored on function return type
 
-	  Review: https://reviewboard.asterisk.org/r/4551
+	  This patch fixes gcc warnings that occur due to the type qualifier 'const'
+	  being ignored on a return type of int.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
+	  ASTERISK-24246 #close
+	  Reported by: Shaun Ruffell
 	  patches:
-	    rb4551.patch submitted by dkdegroot (License 6600)
+	    0001-main-uri-Quiet-warning-about-ignored-attribute-on-re.patch uploaded by Shaun Ruffell (License 5417)
 	  ........
 
-	  Merged revisions 434187 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421675 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434188 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421676 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 14:23 +0000 [2270c40d33]  Kevin Harwell <kharwell@digium.com>
+2014-08-20 17:52 +0000 [b7f98c3da4]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip: config option 'timers' can't be set to 'no'
+	* chan_pjsip: Update media translation paths when new SDP negotiated.
 
-	  When setting the configuration option 'timers' equal to 'no' the bit flag was
-	  not properly negated. This patch clears all associated flags and only sets the
-	  specified one. pjsip will handle any necessary flag combinations. Also went
-	  ahead and did similar for the '100rel' option.
+	  On a SIP reinvite that changes media strams, the PJSIP channel driver was
+	  flooding the log with "Asked to transmit frame type %s, while native
+	  formats is %s" warnings.
 
-	  ASTERISK-24910 #close
-	  Reported by: Ray Crumrine
-	  Review: https://reviewboard.asterisk.org/r/4582/
+	  * Fixes PJSIP not setting up translation paths when the formats change on
+	  a reinvite.  AFS-63 was effectively reintroduced because of the media
+	  formats work.  res_pjsip_sdp_rtp.c:set_caps()
 
+	  * Improved the unexpected frame format WARNING message to include more
+	  information.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434131 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  * Added protective locking while altering formats on a channel.  Reworked
+	  set_format() to simplify and protect the formats under manipulation.
 
-2015-04-06 14:02 +0000 [95de71f247]  George Joseph <george.joseph@fairview5.com>
+	  * Restored some code that got lost in the media_formats work.
+	  (channel.c:set_format() and res_pjsip_sdp_rtp.c:set_caps())
 
-	* build: Fixes for gcc 5 compilation
+	  AFS-137 #close
+	  Reported by: Mark Michelson
 
-	  These are fixes for compilation under gcc 5.0...
+	  Review: https://reviewboard.asterisk.org/r/3906/
+	  ........
 
-	  chan_sip.c:    In parse_request needed to make 'lim' unsigned.
-	  inline_api.h:  Needed to add a check for '__GNUC_STDC_INLINE__' to detect C99 
-	                 inline semantics (same as clang).
-	  ccss.c:        In ast_cc_set_parm, needed to fix weird comparison.
-	  dsp.c:         Needed to work around a possible compiler bug.  It was throwing 
-	                 an array-bounds error but neither
-	                 sgriepentrog, rmudgett nor I could figure out why.
-	  manager.c:     In action_atxfer, needed to correct an array allocation.
+	  Merged revisions 421645 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  This patch will go to 11, 13, trunk.
 
-	  Review: https://reviewboard.asterisk.org/r/4581/
-	  Reported-by: Jeffrey Ollie
-	  Tested-by: George Joseph
-	  ASTERISK-24932 #close
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421646 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-08-20 17:23 +0000 [4672c139dd]  Richard Mudgett <rmudgett@digium.com>
+
+	* cli.c: Fix tab completion of "module load" when MALLOC_DEBUG is enabled.
+
+	  filename_completion_function() returns memory that was not allocated by
+	  the MALLOC_DEBUG allocation tracker so the memory must be freed by
+	  ast_std_free().
 	  ........
 
-	  Merged revisions 434113 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421600 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 421602 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434114 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421608 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-06 13:18 +0000 [d54ccda3b1]  dkdegroot (License 6600)
+	  Merged revisions 421616 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* clang compiler warnings: Remove large chunks of unused code from extconf
 
-	  This patch fixes a warning caught by clang, in which it detected that large
-	  chunks of extconf were unused. Frankly, I wish we could pretend that all of
-	  extconf was unused, but alas, that is not yet the case.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421623 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  A few extraneous functions in the parking tests were removed as well, for
-	  the same reason.
+2014-08-20 15:41 +0000 [49f8bd4ad4]  Mark Michelson <mmichelson@digium.com>
 
-	  Review: https://reviewboard.asterisk.org/r/4553
+	* Set the role for inbound subscriptions correctly.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4553.patch submitted by dkdegroot (License 6600)
+	  This was causing the AMI show_subscriptions test in
+	  the testsuite to fail since all subscriptions were being
+	  seen as subscribers instead of notifiers.
 	  ........
 
-	  Merged revisions 434093 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421585 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434097 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421586 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 13:03 +0000 [0ecd472e4f]  dkdegroot (License 6600)
+2014-08-20 15:04 +0000 [d0640ad7df]  Mark Michelson <mmichelson@digium.com>
 
-	* clang compiler warnings: Fix sometimes-uninitialized warning in pbx_config
+	* Move evaluation of set_var options in pjsip to the end of channel initialization.
 
-	  This patch fixes a warning caught by clang, in which a char pointer could be
-	  assigned to before it was initialized. The patch re-organizes the code to
-	  ensure that the pointer is always initialized, even on off nominal paths.
+	  This allows for set_var to override certain defaults such as caller ID and codec
+	  values. This also fixes a test suite regression. The "set_var" test suite test attempted
+	  to use set_var to override caller ID, but a recent change caused that to no longer work.
+	  ........
 
-	  Review: https://reviewboard.asterisk.org/r/4529
+	  Merged revisions 421565 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4529.patch submitted by dkdegroot (License 6600)
+	  Merged revisions 421566 from http://svn.asterisk.org/svn/asterisk/branches/13
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421567 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-08-20 08:06 +0000 [36f4bff943]  Kinsey Moore <kmoore@digium.com>
+
+	* Stasis: Add information to blind transfer event
+
+	  When a blind transfer occurs that is forced to create a local channel
+	  pair to satisfy the transfer request, information about the local
+	  channel pair is not published. This adds a field to describe that
+	  channel to the blind transfer message struct so that this information
+	  is conveyed properly to consumers of the blind transfer message.
+
+	  This also fixes a bug in which Stasis() was unable to properly identify
+	  the channel that was replacing an existing Stasis-controlled channel
+	  due to a blind transfer.
+
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/3921/
 	  ........
 
-	  Merged revisions 434090 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421537 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421538 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434091 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 12:52 +0000 [4e7be5b2dc]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421539 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix format specified in framehook
+2014-08-20 07:39 +0000 [01f1ff1f77]  Kinsey Moore <kmoore@digium.com>
 
-	  This patch fixes an invalid format specifier used in the formatting of an
-	  ERROR message in the framehook code. The format specifier specifies a
-	  type of 'unsigned short', but the argument passed to it is of type 'int'.
-	  The patch changes the format specifier to 'i'.
+	* AMI: Add AllVariables parameter to Status
 
-	  Review: https://reviewboard.asterisk.org/r/4540
+	  This adds the AllVariables parameter to the Status AMI action such that
+	  if defined and set to "true", all channel variables will be reported in
+	  the subsequent Status event(s). This parameter does not negate the
+	  functionality of the "Variables" parameter so that global variables and
+	  dialplan functions can be requested.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4535.patch submitted by dkdegroot (License 6600)
+	  Review: https://reviewboard.asterisk.org/r/3915/
+
+
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421534 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+
+2014-08-19 15:28 +0000 [76290adf50]  Mark Michelson <mmichelson@digium.com>
+
+	* Alter documentation for callerid_privacy to use correct values.
 	  ........
 
-	  Merged revisions 434087 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421485 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421488 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434088 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 11:02 +0000 [2443b40341]  Mark Michelson <mmichelson@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421490 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Ensure that a non-zero sample rate is returned for all formats.
+2014-08-19 14:55 +0000 [28a89e7685]  Mark Michelson <mmichelson@digium.com>
 
-	  Versions of Asterisk prior to 12 defaulted to 8000 as a sample rate
-	  if one was not provided by a format. In Asterisk 13, this was removed.
-	  The result was that some calculations which involve dividing by the
-	  sample rate resulted in dividing by 0. The fix being put in place
-	  here is to have the same default fallback that was present in previous
-	  versions of Asterisk.
+	* Fix compilation error on certain versions of GCC.
+	  ........
 
-	  Asterisk-24914 #close
-	  Reported by Marcello Ceschia
+	  Merged revisions 421447 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421448 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434046 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421449 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-04-06 10:16 +0000 [b1102cd642]  Corey Farrell <git@cfware.com>
+2014-08-19 14:43 +0000 [a85a483fcd]  Kinsey Moore <kmoore@digium.com>
 
-	* res_pjsip_phoneprov_provider: Revert 433996 / 433997.
+	* AMI Docs: Fix Status channel parameter optionality
+	  ........
 
-	  res_pjsip_phoneprov_provider is using ao2_callback with OBJ_MULTIPLE, then
-	  ignoring the return.  OBJ_NODATA flag was to prevent a reference leak, but
-	  this caused the module to FRACK on unload.  Revert change until this can
-	  be investigated further.
+	  Merged revisions 421442 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  ASTERISK-24935
-	  Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4578/
+	  Merged revisions 421443 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421444 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434025 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421445 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-04-06 09:50 +0000 [0f25076f67]  Mark Michelson (license #5049)
 
-	* ParkedCall: Don't allow dialplan fallthrough after retrieving parked call.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421446 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This is a change to align behavior with that of Asterisk 11 and previous versions.
-	  In those versions, if a parked call were retrieved, and the call ended, the parked
-	  call retriever would be hung up after the ParkedCall application ran. Prior to this
-	  patch, in Asterisk 13, the same situation would result in the parked call retriever
-	  falling through to additional priorities in the extension where the ParkedCall
-	  application was called. With this patch, the behavior between Asterisk 11 and 13
-	  aligns.
+2014-08-19 11:36 +0000 [222b5cd036]  Krandon Bruse (license 6631)
 
-	  ASTERISK-24899 #close
-	  Reported by Malcolm Davenport
-	  Patches:
-	  	ASTERISK-24899.patch uploaded by Mark Michelson(license #5049)
+	* ARI: Fix a bug where /channels/{channelID}/continue doesn't execute PBX
 
+	  If /channels/{channelID}/continue is called on a channel that was originated
+	  without a PBX (such as the ARI command POST channel with a stasis application
+	  argument), the channel will not start dialplan execution. This patch will now
+	  run the PBX out of the stasis execution if the channel doesn't currently have
+	  an active PBX upon continuing.
 
+	  ASTERISK-24043 #close
+	  Reported by: Krandon Bruse
+	  Review: https://reviewboard.asterisk.org/r/3917/
+	  Patches:
+	      stasis-continue.diff submitted by Krandon Bruse (license 6631)
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@434022 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421416 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-04-05 07:53 +0000 [709fa14b44]  Corey Farrell <git@cfware.com>
+	  Merged revisions 421423 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_pjsip_phoneprov_provider: Fix leaked OBJ_MULTIPLE iterator.
 
-	  res_pjsip_phoneprov_provider was using ao2_callback with OBJ_MULTIPLE, then
-	  ignoring the return.  Added OBJ_NODATA flag to prevent a reference leak.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421424 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24935 #close
-	  Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4578/
+2014-08-19 11:16 +0000 [83a9b91da9]  Richard Mudgett <rmudgett@digium.com>
 
+	* chan_pjsip: Fix attended transfer connected line name update.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433996 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  A calls B
+	  B answers
+	  B SIP attended transfers to C
+	  C answers, B and C can see each other's connected line information
+	  B completes the transfer
+	  A has number but no name connected line information about C
+	    while C has the full information about A
 
-2015-04-03 16:53 +0000 [1ee8424f27]  Mark Michelson <mmichelson@digium.com>
+	  I examined the incoming and outgoing party id information handling of
+	  chan_pjsip and found several issues:
 
-	* res_pjsip_messaging: Serialize outbound SIP MESSAGEs
+	  * Fixed ast_sip_session_create_outgoing() not setting up the configured
+	  endpoint id as the new channel's caller id.  This is why party A got
+	  default connected line information.
 
-	  Outbound SIP MESSAGEs had the potential to be sent out
-	  of order from how they were specified in a set of
-	  dialplan steps.
+	  * Made update_initial_connected_line() use the channel's CALLERID(id)
+	  information.  The core, app_dial, or predial routine may have filled in or
+	  changed the endpoint caller id information.
 
-	  This change creates a serializer for sending outbound
-	  MESSAGE requests on. This ensures that the MESSAGEs are
-	  sent by Asterisk in the same order that they were sent
-	  from the dialplan.
+	  * Fixed chan_pjsip_new() not setting the full party id information
+	  available on the caller id and ANI party id.  This includes the configured
+	  callerid_tag string and other party id fields.
 
-	  ASTERISK-24937 #close
-	  Reported by Mark Michelson
+	  * Fixed accessing channel party id information without the channel lock
+	  held.
 
-	  Review: https://reviewboard.asterisk.org/r/4579
+	  * Fixed using the effective connected line id without doing a deep copy
+	  outside of holding the channel lock.  Shallow copy string pointers can
+	  become stale if the channel lock is not held.
 
+	  * Made queue_connected_line_update() also update the channel's
+	  CALLERID(id) information.  Moving the channel to another bridge would need
+	  the information there for the new bridge peer.
 
+	  * Fixed off nominal memory leak in update_incoming_connected_line().
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433968 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  * Added pjsip.conf callerid_tag string to party id information from
+	  enabled trust_inbound endpoint in caller_id_incoming_request().
 
-2015-04-02 09:56 +0000 [169e57d2e0]  Scott Griepentrog <sgriepentrog@digium.com>
+	  AFS-98 #close
+	  Reported by: Mark Michelson
 
-	* pjsip: resolve compatibility problem with ast_sip_session
+	  Review: https://reviewboard.asterisk.org/r/3913/
+	  ........
 
-	  A change in r430179 inserted a variable near the top of a
-	  structure caused a problem when running DPMA in a version
-	  of Asterisk compiled across the change.  This patch moves
-	  the new variable to the end of the structure, eliminating
-	  the problem.
+	  Merged revisions 421400 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Review: https://reviewboard.asterisk.org/r/4574/
+	  Merged revisions 421403 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421404 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433944 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+2014-08-18 16:18 +0000 [c4c9d4ad6c]  Damien Wedhorn <voip@facts.com.au>
 
-2015-04-02 05:31 +0000 [1eb0c5f4e8]  Corey Farrell <git@cfware.com>
+	* Skinny: Fixup compile warning for non dev-mode.
+	  ........
 
-	* Tell menuselect that MALLOC_DEBUG conflicts with DEBUG_CHAOS.
+	  Merged revisions 421376 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  DEBUG_CHAOS was marked as conflicting with MALLOC_DEBUG, but
-	  for this to work correctly MALLOC_DEBUG must also be marked
-	  as conflicting with DEBUG_CHAOS.
 
-	  Review: https://reviewboard.asterisk.org/r/4557/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421380 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-18 15:20 +0000 [1de8b8035e]  gtjoseph <george.joseph@fairview5.com>
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433923 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	* func_config: Change 'Not Found' message from ERROR to DEBUG
 
-2015-04-01 11:25 +0000 [e301185983]  Ashley Sanders <asanders@digium.com>
+	  When you call the CONFIG dialplan function with the name of a variable that
+	  doesn't exist in the target context you get an ERROR.  This does nothing but
+	  clutter up the logs with messages that may be perfectly acceptable.  Just
+	  because a variable wasn't in the context doesn't mean it's an error.  Maybei
+	  t's optional or just needs to be defaulted or ignored.
 
-	* stasis: set a channel variable on websocket disconnect error
+	  This patch changes the log level from ERROR to DEBUG.  If a dialplan developer
+	  wants to debug their dialplan they still canby setting the console debug level 
+	  as needed.
 
-	  Resolve compile errors caused by r433863 by fixing the
-	  documentation xml to comply with the schema.
+	  Tested by: George Joseph
+	  Review: https://reviewboard.asterisk.org/r/3919/
+	  ........
 
+	  Merged revisions 421327 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433888 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421328 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-03-31 22:26 +0000 [a1f12d9231]  Ashley Sanders <asanders@digium.com>
+	  Merged revisions 421329 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* stasis: set a channel variable on websocket disconnect error
+	  Merged revisions 421337 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Resolve compile errors caused by r433839 by included the missing
-	  header file, pbx.h.
 
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421341 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-17 20:14 +0000 [bb494067a5]  Matt Jordan <mjordan@digium.com>
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433863 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	* Multiple revisions 421311-421312
 
-2015-03-31 17:00 +0000 [7293ecd90b]  Ashley Sanders <asanders@digium.com>
+	  ........
+	    r421311 | mjordan | 2014-08-17 20:11:28 -0500 (Sun, 17 Aug 2014) | 9 lines
+	    
+	    res/ari/resource_channels: Don't return allocation failure on failed function
+	    
+	    If a function fails to execute, it is most likely due to one of two reasons:
+	    (1) The function doesn't exist or can't be read from
+	    (2) The function is dangerous and is restricted based on the user's permissions
+	    
+	    Currently we return allocation failure, which is incorrect. This updates the
+	    reason code to more accurately reflect why the request failed.
 
-	* stasis: set a channel variable on websocket disconnect error
+	    ASTERISK-24215
+	  ........
+	    r421312 | mjordan | 2014-08-17 20:13:41 -0500 (Sun, 17 Aug 2014) | 4 lines
+	    
+	    res/ari/resource_channels: Fix compilation issue
+	    
+	    Forgot a parameter. Whoops.
+	  ........
 
-	  When an error occurs while writing to a web socket, the web socket is
-	  disconnected and the event is logged. A side-effect of this, however, is that
-	  any application on the other side waiting for a response from Stasis is left
-	  hanging indefinitely (as there is no mechanism presently available for
-	  notifying interested parties about web socket error states in Stasis).
+	  Merged revisions 421311-421312 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  To remedy this scenario, this patch introduces a new channel variable:
-	  STASISSTATUS.
 
-	  The possible values for STASISSTATUS are:
-	  SUCCESS         - The channel has exited Stasis without any failures
-	  FAILED          - Something caused Stasis to croak. Some (not all) possible
-	                    reasons for this:
-	                      - The app registry is not instantiated;
-	                      - The app requested is not registered;
-	                      - The app requested is not active;
-	                      - Stasis couldn't send a start message
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421313 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  ASTERISK-24802
-	  Reported By: Kevin Harwell
-	  Review: https://reviewboard.asterisk.org/r/4519/
+2014-08-17 19:57 +0000 [ba5d5da60b]  Matt Jordan <mjordan@digium.com> (License #6283)
 
+	* Improve call forwarding reporting, especially with regards to ARI.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433839 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  This patch addresses a few issues:
 
-2015-03-31 11:55 +0000 [94949e7f2f]  Richard Mudgett <rmudgett@digium.com>
+	  1) The order of Dial events have been changed when performing a call forward.
+	     The order has now been altered to
+	      1) Dial begins dialing channel A.
+	      2) When A forwards the call to B, we issue the dial end event to channel
+	         A, indicating the dial is being canceled due to a forward to B.
+	      3) When the call to channel B occurs, we then issue a new dial begin to
+	         channel B.
 
-	* chan_sip: Fix expression in unit test /channels/chan_sip/test_sip_rtpqos.
+	  2) Call forwards are now reported on the calling channel, not the peer channel.
 
-	  Fix misplaced parentheses in original fabs() expression.
-	  ........
+	  3) AMI DialEnd events have been altered to display the extension the call is
+	     being forwarded to when relevant.
 
-	  Merged revisions 433816 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  4) You can now get the values of channel variables for channels that are not
+	     currently in the Stasis application. This brings the retrieval of channel
+	     variables more in line with the rest of channel read operations since they
+	     may be performed on channels not in Stasis.
 
+	  ASTERISK-24134 #close
+	  Reported by Matt Jordan
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433817 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  ASTERISK-24138 #close
+	  Reported by Matt Jordan
 
-2015-03-31 06:47 +0000 [9967739669]  Corey Farrell <git@cfware.com>
+	  Patches:
+	  	forward-shenanigans.diff uploaded by Matt Jordan (License #6283)
 
-	* Re-add _ast_mem_backtrace_buffer variable for ABI compatibility.
+	  Review: https://reviewboard.asterisk.org/r/3899
+	  ........
 
-	  Modules built prior to commit of r4502 expect to link at runtime
-	  to the variable _ast_mem_backtrace_buffer.  This change re-adds
-	  the variable to the C file only.
+	  Merged revisions 420794 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Review: https://reviewboard.asterisk.org/r/4558/
 
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421310 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433795 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+2014-08-17 18:29 +0000 [6525f374db]  Matt Jordan <mjordan@digium.com>
 
-2015-03-30 06:42 +0000 [2d39bc5528]  Corey Farrell <git@cfware.com>
+	* apps/app_meetme: Fix crash when publishing MeetMe messages with no channel
 
-	* Fix an ABI compatibility issue with ast_log_safe for modules.
+	  The same function, meetme_stasis_generate_msg, handles creating and publishing
+	  Stasis message both when there are channels in the MeetMe conference and when
+	  there are no channels in the conference. When the performance improvement was
+	  made to use cached snapshots, this created a situation where Asterisk would
+	  crash: obtaining a cached snapshot is not NULL tolerant.
 
-	  Binary modules are sometimes built against the latest release of
-	  Asterisk in each branch, and need to be compatible with all
-	  releases of that branch.  This change ensures that utils.h only
-	  uses ast_log_safe from the core.  For modules and utilities ast_log
-	  is used instead.
+	  This patch restores the previous implementation, which used a NULL safe set
+	  of routines to produce a blob containing the channel snapshot (if available)
+	  and information about the MeetMe conference.
 
-	  Review: https://reviewboard.asterisk.org/r/4548/
+	  ASTERISK-24234 #close
+	  Reported by: Shaun Ruffell
+	  Tested by: Shaun Ruffell
 	  ........
 
-	  Merged revisions 433772 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421270 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421273 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433773 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-29 21:44 +0000 [5f8faf16af]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421276 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix -Wabsolute-value warnings
+2014-08-17 18:10 +0000 [44fc6ea6ff]  Richard Mudgett <rmudgett@digium.com> (License 5621)
 
-	  This patch fixes several warnings caught by clang - in this case, usage of the
-	  abs function on non-integer values. This patch uses labs and fabs, as
-	  appropriate, in the various affected files.
+	* apps/app_dial: Fix Dial 'z' option
 
-	  Review: https://reviewboard.asterisk.org/r/4525
+	  The 'z' option is supposed to disable the dial timeout in the case of a call
+	  forward. Unfortunately, the wrong timeout timer was passed to the do_forward
+	  function, resulting in the option not working.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
+	  ASTERISK-24225 #close
+	  Reported by: dimitripietro
+	  Tested by: dimitripietro
 	  patches:
-	    rb4525.patch submitted by dkdegroot (License 6600)
+	    jira_asterisk_24225_v1.8.patch uploaded by rmudgett (License 5621)
+	    jira_asterisk_24225_v11.patch uploaded by rmudgett (License 5621)
 	  ........
 
-	  Merged revisions 433749 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421232 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 421233 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433750 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421234 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-03-29 21:39 +0000 [09b681e344]  dkdegroot (License 6600)
+	  Merged revisions 421235 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* clang compiler warnings: Fix invalid enum conversion
 
-	  This patch fixes some invalid enum conversion warnings caught by clang. In
-	  particular:
-	  * chan_sip: Several functions mixed usage of the st_refresher_param
-	    enum and st_refresher enum. This patch corrects the functions to use the
-	    right enum.
-	  * chan_pjsip: Fixed mixed usage of ast_sip_session_t38state and ast_t38_state.
-	  * strings: Fixed incorrect usage of AO2 flags with strings container.
-	  * res_stasis: Change a return enumeration to stasis_app_user_event_res.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421236 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4535
+2014-08-17 17:35 +0000 [98ca5c0b5f]  cloos <cloos@jhcloos.com> (License 5956)
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
+	* configure: Undefine FORTIFY_SOURCE prior to defining it for patched gcc
+
+	  Some distributions of Linux patch gcc to define FORTIFY_SOURCE when gcc is
+	  executed with optimization. This "help" unfortunately results in re-definition
+	  warnings when FORTIFY_SOURCE is later defined in Asterisk's build system. This
+	  patch undefines FORTIFY_SOURCE prior to defining it to prevent this warning.
+
+	  Review: https://reviewboard.asterisk.org/r/3912/
+
+	  ASTERISK-24032 #close
+	  Reported by: Kilburn
+	  Tested by: Kilburn, wdoekes
 	  patches:
-	    rb4535.patch submitted by dkdegroot (License 6600)
+	    1.8.diff uploaded by cloos (License 5956)
+	    10.diff uploaded by cloos (License 5956)
+	    11.diff uploaded by cloos (License 5956)
+	    12.diff uploaded by cloos (License 5956)
+	    13.diff uploaded by cloos (License 5956)
 	  ........
 
-	  Merged revisions 433746 from http://svn.asterisk.org/svn/asterisk/branches/11
-
+	  Merged revisions 421227 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433747 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421228 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-2015-03-29 21:29 +0000 [7f33abb827]  Matthew Jordan <mjordan@digium.com>
+	  Merged revisions 421229 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	* main/stdtime/localtime: Fix warning introduced in r433720
+	  Merged revisions 421230 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  The patch in r433720 caused a warning to be kicked back by gcc. It occurred
-	  due to this check in unistd.h:
 
-	      if (__nbytes > __bos0 (__buf))
-	          return __read_chk_warn (__fd, __buf, __nbytes, __bos0 (__buf));
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421231 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  That is, if __nbytes is greater than the result of GCC's built-in object size
-	  for the struct, we'll kick back a warning.
+2014-08-17 11:11 +0000 [952da298ce]  Joshua Colp <jcolp@digium.com>
 
-	  As it turns out, this is because there is an error in the code in the patch.
-	  We are passing the address of the pointer to the struct, not iev, which is a
-	  pointer to the struct. Hence, the number of bytes is probably going to be lot
-	  larger than the number of bytes that make up a pointer! This patch changes
-	  the code just read from the pointer to the struct - which fixes the warning.
+	* res_http_websocket: Include query parameters in client connection requests.
 
-	  ASTERISK-24917
+	  Review: https://reviewboard.asterisk.org/r/3914/
 	  ........
 
-	  Merged revisions 433743 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421210 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433744 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421211 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-29 20:56 +0000 [47eeb67e14]  dkdegroot (License 6600)
+2014-08-15 12:26 +0000 [9b658b7c60]  Jonathan Rose <jrose@digium.com>
 
-	* clang compiler warnings: Ignore -Wunused-command-line-argument
+	* Bridging: Fix a behavioral change when checking if a channel is leaving a bridge
 
-	  Asterisk's build system has a tendency to pass include directives for libraries
-	  to everything compiled within a particular group of source files. This means
-	  we pass the header for libxml2 to things that don't necessarily need it. As a
-	  result, we ignore this particular warning.
+	  r420934 introduced some failures in the test suite.  Upon investigating, it was
+	  discovered that differences in the way we were evaluating whether a channel was in
+	  the process of leaving a bridge were causing some reinvites not to occur (mostly
+	  reinvites back to Asterisk when ending a call). This patch fixes that behavioral
+	  change.
 
-	  Review: https://reviewboard.asterisk.org/r/4545/
+	  ASTERISK-24027 #close
+	  Reported by: Matt Jordan
+	  Review: https://reviewboard.asterisk.org/r/3910/
+	  ........
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4545.patch submitted by dkdegroot (License 6600)
+	  Merged revisions 421186 from http://svn.asterisk.org/svn/asterisk/branches/12
 	  ........
 
-	  Merged revisions 433720 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421187 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433721 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421195 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-29 20:52 +0000 [dbb4d6f9e7]  dkdegroot (License 6600)
+2014-08-15 10:50 +0000 [0d0a616e1a]  Matt Jordan <mjordan@digium.com>
 
-	* clang compiler warnings: Fix warning for -Wgnu-variable-sized-type-not-at-end
+	* app_voicemail/app: Remove test events that were duplicated by r421059
 
-	  This patch fixes a warning caught by clang, wherein a variable sized struct is
-	  not located at the end of a struct. While the code in question actually
-	  expected this, this is a good warning to watch for. Hence, this patch refactors
-	  the code in question to not have two variable length elements in the same
-	  struct.
+	  Moving the test event raised when a file is played back (which occurred in
+	  r421059) broke the ever loving snot out of the voicemail tests. This caused
+	  duplicate test events to get raised, as app_voicemail and main/app were raising
+	  events prior to call ast_streamfile. The voicemail tests did not enjoy getting
+	  multiple events.
 
-	  Review: https://reviewboard.asterisk.org/r/4530/
+	  Since raising the playback event in ast_streamfile is far more useful to the
+	  vast majority of tests, this patch keeps the call there and simply removes the
+	  extraneous calls that duplicated the event.
+	  ........
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4530.patch submitted by dkdegroot (License 6600)
+	  Merged revisions 421125 from http://svn.asterisk.org/svn/asterisk/branches/1.8
 	  ........
 
-	  Merged revisions 433717 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421164 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421165 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433718 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421166 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-28 07:56 +0000 [e126ab9eeb]  dkdegroot (License 6600)
 
-	* clang compiler warnings: Fix a variety of "unused" warnings
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421167 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch fixes the -Wunused-value -Wunused-variable -Wunused-const-variable
-	  errors caught by clang. Specifically:
+2014-08-14 16:16 +0000 [980e49614c]  Matt Jordan <mjordan@digium.com>
 
-	  * apps/app_queue.c: removed unused qpm_cmd_usage[], qum_cmd_usage[],
-	                      qsmp_cmd_usage[]
-	  * cel/cel_sqlite3_custom.c: removed unused name[] = "cel_sqlite3_custom"
-	  * channels/chan_pjsip.c: removed unused desc[] = "PJSIP Channel"
-	  * codecs/gsm/src/gsm_create.c: removed unused ident[] = "$Header$"
-	  * funcs/func_env.c:729: Fixed ast_str_append_substr.
-	  * main/editline/np/strlcat.c: removed unused rcsid variable
-	  * main/editline/np/strlcpy.c: removed unused rcsid variable
-	  * main/security_events.c: removed unused TIMESTAMP_STR_LEN
-	  * utils/conf2ael.c: removed unused cfextension_states
-	  * utils/extconf.c: removed unused cfextension_states
+	* res/res_hep_rtcp: Remove dependency on PJSIP
 
-	  Review: https://reviewboard.asterisk.org/r/4526
+	  The res_hep_rtcp module was incorrectly including <pjsip.h>. This didn't need
+	  to be included, as the module does not using PJPROJECT any fashion.
+	  Unfortunately, because res_hep_rtcp did not include pjsip in its MODULEINFO as
+	  a dependency, this also meant that res_hep_rtcp will fail to compile on a
+	  system without PJPROJECT.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4526.patch submitted by dkdegroot (License 6600)
+	  This patch removes the include.
+
+	  Thanks to Damien Wedhorn for pointing this out in #asterisk-dev.
+
+	  ASTERISK-24236 #close
+	  Reported by: Damien Wedhorn, Matt Jordan
+	  Tested by: Damien Wedhorn
 	  ........
 
-	  Merged revisions 433693 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421064 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 421065 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433694 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-28 07:48 +0000 [2f6534527d]  dkdegroot (License 6600)
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421066 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* clang compiler warnings: Fix -Wself-assign
+2014-08-14 15:59 +0000 [513981c89d]  Matt Jordan <mjordan@digium.com>
 
-	  Assigning a variable to itself isn't super useful. However, the WAV format
-	  modules make use of this in order to perform byte endian checks. This patch
-	  works around the warning by only performing the self assignment if we are
-	  going to do more than just assign it to ourselves. Which is odd, but true.
+	* main/file: Move test event to emit PLAYBACK event more consistently
 
-	  Review: https://reviewboard.asterisk.org/r/4544/
+	  This is being done in advance of the test for ASTERISK-23953
+	  ........
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4544.patch submitted by dkdegroot (License 6600)
+	  Merged revisions 421059 from http://svn.asterisk.org/svn/asterisk/branches/1.8
 	  ........
 
-	  Merged revisions 433690 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 421060 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
+	  Merged revisions 421061 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433691 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421062 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-28 07:40 +0000 [eb70993a50]  dkdegroot (License 6600)
 
-	* clang compiler warnings: Fix -Wparantheses-equality warnings
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421063 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Clang will treat ((a == b)) as a warning, as it reasonably expects that the
-	  developer may have intended to write (a == b) or ((a = b)). This patch cleans
-	  up all instances where equality, not assignment, was intended between two
-	  parantheses.
+2014-08-14 14:21 +0000 [0b11c48522]  Matt Jordan <mjordan@digium.com>
 
-	  Review: https://reviewboard.asterisk.org/r/4531/
+	* cel: Make sure channels in extra fields include their unique IDs as well
 
-	  ASTERISK-24917
-	  Repoted by: dkdegroot
-	  patches:
-	    rb4531.patch submitted by dkdegroot (License 6600)
-	  ........
+	  CEL typically tracks a lot of information using the unique ID of the channel.
+	  This is typically needed due to tying events together using the linked ID of
+	  the various channels involved in a "call", which is derived from the channel ID
+	  of the oldest channel involved in a bridge (or in the case of a Dial, the
+	  parent channel).
 
-	  Merged revisions 433687 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Previously, we had updated the extra fields to include the involved channel
+	  names, but forgot to put in the unique ID. This patch corrects that error.
+	  ........
 
+	  Merged revisions 421037 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433688 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421042 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-28 07:31 +0000 [c0ff16036a]  dkdegroot (License 6600)
 
-	* clang compiler warnings: Fix -Wbitfield-constant-conversion warning
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421043 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  In chan_iax2, we attempt to assign a -1 to a bitfield. This gets caught by
-	  clang, as it will truncate the -1 to a 1 implicitly.
+2014-08-14 11:33 +0000 [79c5c08db9]  Richard Mudgett <rmudgett@digium.com>
 
-	  Instead, we just assign the value a '1'.
+	* ARI: Originate to app local channel subscription code optimization.
 
-	  Review: https://reviewboard.asterisk.org/r/4537/
+	  Reduce the scope of local_peer and only get it if the ARI originate is
+	  subscribing to the channels.
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4537.patch submitted by dkdegroot (License 6600)
+	  Review: https://reviewboard.asterisk.org/r/3905/
 	  ........
 
-	  Merged revisions 433683 from http://svn.asterisk.org/svn/asterisk/branches/11
-
+	  Merged revisions 421009 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433684 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 421010 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-28 07:27 +0000 [844bc76bef]  dkdegroot (License 6600)
 
-	* clang compiler warnings: Fix -Winitializer-overrides
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@421012 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch fixes clange compiler warnings for initializer overrides.
-	  Specifically:
+2014-08-14 11:01 +0000 [e4b32731b9]  Richard Mudgett <rmudgett@digium.com>
 
-	  res_pjsip/config_transport maps PJSIP_TLSV1_METHOD to the same enumeration
-	  value as PJSIP_SSL_DEFAULT_METHOD. When initializing an array containing
-	  those enum values, we therefore initialize the value twice to two different
-	  values, "tlsv1" and "default". This patch changes it to just initialize
-	  the index in the array to "tlsv1".
+	* channel_internal_api.c: Replace some code with ao2_replace().
 
-	  Review: https://reviewboard.asterisk.org/r/4539/
+	  Use ao2_replace() instead of ao2_cleanup(); ao2_bump().
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4539.patch submitted by dkdegroot (License 6600)
+	  ao2_replace() has the advantange of not altering the ref count if the
+	  replaced pointer is the same.
 
+	  Review: https://reviewboard.asterisk.org/r/3904/
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433682 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 420992 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-28 07:19 +0000 [5e204042d9]  dkdegroot (License 6600)
 
-	* clang compiler warnings: Fix -Wunused-function; make inline function static
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420993 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This patch fixes clang compilers warnings for unused functions. Specifically:
-	   * channels/chan_iax2: removed user_ref function
-	   * main/dsp.c: removed goertzel_update function
-	   * main/config.c: made variable_list_switch static
+2014-08-13 12:05 +0000 [dd41d0ff01]  Richard Mudgett <rmudgett@digium.com>
 
-	  Review: https://reviewboard.asterisk.org/r/4527
+	* res_pjsip_send_to_voicemail.c: Fix svn file properties.
+	  ........
 
-	  ASTERISK-24917
-	  Reported by: dkdegroot
-	  patches:
-	    rb4527.patch submitted by dkdegroot (License 6600)
+	  Merged revisions 420956 from http://svn.asterisk.org/svn/asterisk/branches/12
 	  ........
 
-	  Merged revisions 433678 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 420957 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433680 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420958 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-27 17:34 +0000 [cfbf5fbe91]  Jonathan Rose <jrose@digium.com>
+2014-08-13 11:56 +0000 [6aa510b41f]  Kinsey Moore <kmoore@digium.com>
 
-	* SAC: Add a few basic queues
+	* PJSIP: Prevent crash no-URI contacts
 
-	  Review: https://reviewboard.asterisk.org/r/4503/
+	  This prevents a crash from occurring when a contact with no URI is used
+	  for the creation of an outbound out-of-dialog request with no
+	  associated endpoint.
+	  ........
 
+	  Merged revisions 420949 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433658 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 420950 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-27 17:25 +0000 [1a50d8d4c2]  Jonathan Rose <jrose@digium.com>
 
-	* SAC: Add conferencing extensions and configuration
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420953 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4504/
+2014-08-13 11:24 +0000 [d4695774e7]  Jonathan Rose <jrose@digium.com>
 
+	* Bridges: Fix feature interruption/unintended kick caused by external actions
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433656 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  If a manager or CLI user attached a mixmonitor to a call running a dynamic
+	  bridge feature while in a bridge, the feature would be interrupted and the
+	  channel would be forcibly kicked out of the bridge (usually ending the call
+	  during a simple 1 to 1 call). This would also occur during any similar action
+	  that could set the unbridge soft hangup flag, so the fix for this was to
+	  remove unbridge from the soft hangup flags and make it a separate thing all
+	  together.
 
-2015-03-27 16:15 +0000 [c6c08d755d]  Rusty Newton <rnewton@digium.com>
+	  ASTERISK-24027 #close
+	  Reported by: mjordan
+	  Review: https://reviewboard.asterisk.org/r/3900/
+	  ........
 
-	* configs/basic-pbx - Super Awesome Company example configs Phase 1, Patch 2
+	  Merged revisions 420934 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  Example configuration files for a "basic PBX" deployment for the fictitious
-	  Super Awesome Company. Details at https://reviewboard.asterisk.org/r/4488/
-	  and https://wiki.asterisk.org/wiki/display/AST/Super+Awesome+Company
+	  Merged revisions 420940 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Patch 4488 includes all functionality needed for SAC's outside connectivity
-	  and some externally accessed features, as well as outbound dialing.
 
-	  Reported by: Malcolm Davenport
-	  Tested by: Rusty Newton
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420947 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4488/
+2014-08-13 09:31 +0000 [6a6702bb0f]  Kinsey Moore <kmoore@digium.com>
 
+	* AMI: Improve documentation for Status action
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433624 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 420919 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-03-27 16:04 +0000 [13557675d4]  Richard Mudgett <rmudgett@digium.com>
 
-	* res_pjsip_registrar_expire.c: Made use ao2 container template routines and eliminated some RAII_VAR() usage.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420921 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  * Converted the contact_autoexpire container to use the ao2 template hash
-	  and cmp functions.  Also made use the OBJ_SEARCH_xxx names instead of the
-	  deprecated names.
+2014-08-13 02:54 +0000 [52c94d3af4]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  * Eliminates several unnecessary uses of RAII_VAR().
+	* logger: Don't store verbose-magic in the log files.
 
-	  Review: https://reviewboard.asterisk.org/r/4524/
+	  In r399267, the verbose2magic stuff was edited. This time it results
+	  in magic characters in the log files for multiline messages.
 
+	  In trunk (and 13) this was fixed by the "stripping" of those
+	  characters from multiline messages (in r414798).
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433622 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  This fix is altered to actually strip the characters and not replace
+	  them with blanks.
 
-2015-03-27 15:30 +0000 [85feac857c]  Mark Michelson <mmichelson@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3901/
+	  Review: https://reviewboard.asterisk.org/r/3902/
+	  ........
 
-	* Add stateful PJSIP response API call, and use it for out-of-dialog responses.
+	  Merged revisions 420897 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  Asterisk had an issue where retransmissions of MESSAGE requests resulted in
-	  Asterisk processing the retransmission as if it were a new MESSAGE request.
+	  Merged revisions 420898 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  This patch fixes the issue by creating a transaction in PJSIP on the incoming
-	  request. This way, if a retransmission arrives, the PJSIP transaction layer
-	  will resend the response and Asterisk will not ever see the retransmission.
+	  Merged revisions 420899 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24920 #close
-	  Reported by Mark Michelson
 
-	  Review: https://reviewboard.asterisk.org/r/4532/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420900 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-12 18:45 +0000 [969982b878]  Richard Mudgett <rmudgett@digium.com>
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433619 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	* chan_sip: Fix type mismatch when the format is changed.
 
-2015-03-27 12:50 +0000 [dc2cf21144]  Richard Mudgett <rmudgett@digium.com>
+	  Symptom is most likely an invalid ao2 object bad magic number message or a
+	  less likely crash.
+	  ........
 
-	* res_pjsip_registrar_expire.c: Cleanup scheduler leaks on unload/shutdown.
+	  Merged revisions 420881 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Contact expiration object refs were leaked when the module was unloaded.
 
-	  * Made empty the scheduler of entries before destroying it to release the
-	  object ref held by the scheduler entry.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420882 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4523/
+2014-08-12 18:36 +0000 [8526d967c9]  Richard Mudgett <rmudgett@digium.com>
 
+	* res_stasis_snoop.c: Fix off nominial exit path leaving Snoop channel locked and not hungup.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433596 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  * Made use ast_copy_string() instead of strcpy() for snoop uniqueid for
+	  safety.  There is no guarantee that the max channel uniqueid length will
+	  remain the same as the snoop uniqueid space.
+	  ........
 
-2015-03-27 09:41 +0000 [6e6f5b3a1f]  scsiguy (License 6692)
+	  Merged revisions 420879 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_timing_kqueue: Update the module to conform to current timer API
 
-	  This patch updates the kqueue timing module to conform to current timer API.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420880 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  This fixes issues with using the kqueue timing source on Asterisk 13 on
-	  FreeBSD 10. These issues include:
+2014-08-12 06:18 +0000 [ca61f8ac82]  Joshua Colp <jcolp@digium.com>
 
-	  - Remove support for kevent64().  The values used to support Asterisk timers
-	    fit within 32bits and so can be handled on all platforms via kevent().
+	* app_voicemail: Fix the "test_voicemail_vm_info" unit test.
+	  ........
 
-	  - Provide debug logging for, but do not track, unacked events.  This matches
-	    the behavior of all other timer implementations.
+	  Merged revisions 420856 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  - Implement continuous mode by triggering and leaving active, a user event.
-	    This ensures that the file descriptor for the timer returns immediately from
-	    poll(), without placing the load of a high speed timer on the kernel.
 
-	  - In kqueue_timer_get_max_rate(), don't overstate the capability of the timer.
-	    On some platforms, UINT_MAX is greater than INTPTR_MAX, the largest integer
-	    type kqueue supports for timers.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420858 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  - In kqueue_timer_get_event(), assume the caller woke up from poll() and just
-	    return the mode the timer is currently in. This matches all other timer
-	    implementations.
+2014-08-11 16:04 +0000 [aba07a0f6e]  Richard Mudgett <rmudgett@digium.com>
 
-	  - Adjust the test code now that unacked events are not tracked.
+	* res/stasis/command.c: Fix recent commit using spaces instead of tabs.
+	  ........
 
-	  Review: https://reviewboard.asterisk.org/r/4465/
+	  Merged revisions 420836 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-	  ASTERISK-24857 #close
-	  Reported by: scsiguy
-	  Tested by: Ed Hynan
-	  patches:
-	    rb4465.patch submitted by scsiguy (License 6692)
+	  Merged revisions 420837 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433574 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420838 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-27 07:26 +0000 [b0df413fb2]  Corey Farrell <git@cfware.com>
+2014-08-11 13:51 +0000 [ffccae8269]  Matt Jordan <mjordan@digium.com>
 
-	* Fix link error for utils/aelparse.
+	* AMI/ARI: Update version to 2.5.0/1.5.0 respectively
 
-	  Use the standard ast_log instead of ast_log_safe for STANDALONE programs.
+	  This is to support the backwards compatible changes made in the next version
+	  of Asterisk.
+	  ........
 
-	  Review: https://reviewboard.asterisk.org/r/4538/
+	  Merged revisions 420805 from http://svn.asterisk.org/svn/asterisk/branches/12
 	  ........
 
-	  Merged revisions 433549 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 420808 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433550 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420811 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-27 02:09 +0000 [d01706ce1e]  Corey Farrell <git@cfware.com>
+2014-08-11 13:46 +0000 [7a4691b425]  Kinsey Moore <kmoore@digium.com>
 
-	* Improved and portable ast_log recursion avoidance
+	* Stasis: Use the correct return value
 
-	  This introduces a new logger routine ast_log_safe.  This routine should be
-	  used for all error messages in code that can be run as a result of ast_log.
-	  ast_log_safe does nothing if run recursively.  All error logging in
-	  astobj2.c, strings.c and utils.h have been switched to ast_log_safe.
+	  Return the correct value instead of always returning 0 when setting
+	  internal status on unreal channels.
 
-	  This required adding support for raw threadstorage.  This provides direct
-	  access to the void* pointer in threadstorage.  In ast_log_safe, NULL is used
-	  to signify that this thread is not already running ast_log_safe, (void*)1 when
-	  it is already running.  This was done since it's critical that ast_log_safe
-	  do nothing that could log during recursion checking.
+	  Reported by: Richard Mudgett
+	  ........
 
-	  ASTERISK-24155 #close
-	  Reported by: Timo Teräs
-	  Review: https://reviewboard.asterisk.org/r/4502/
+	  Merged revisions 420802 from http://svn.asterisk.org/svn/asterisk/branches/12
 	  ........
 
-	  Merged revisions 433522 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 420803 from http://svn.asterisk.org/svn/asterisk/branches/13
 
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433523 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420804 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-26 18:07 +0000 [4b225e2104]  Corey Farrell <git@cfware.com>
+2014-08-11 13:38 +0000 [6f735288b0]  Kinsey Moore <kmoore@digium.com>
 
-	* Fix compile errors caused by r4500 / r4501.
+	* Stasis: Allow internal channels directly into bridges
 
-	  * Add ast_register_cleanup to utils/clicompat.c to deal with
-	    any utils that copy sources from main.
-	  * Asterisk 13+: remove unused variables from core_local.c.
+	  The patch to catch channels being shoehorned into Stasis() via external
+	  mechanisms also happens to catch Announcer and Recorder channels
+	  because they aren't known to be stasis-controlled channels in the usual
+	  sense. This marks those channels as Stasis()-internal channels and
+	  allows them directly into bridges.
 
-	  Review: https://reviewboard.asterisk.org/r/4534/
+	  Review: https://reviewboard.asterisk.org/r/3903/
 	  ........
 
-	  Merged revisions 433499 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 420795 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
+	  Merged revisions 420796 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433500 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-03-26 17:19 +0000 [6adf26f14d]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420797 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* Replace most uses of ast_register_atexit with ast_register_cleanup.
+2014-08-11 12:40 +0000 [db0a97f8ce]  Mark Michelson <mmichelson@digium.com>
 
-	  Since 'core stop now' and 'core restart now' do not stop modules,
-	  it is unsafe for most of the core to run cleanups.  Originally all
-	  cleanups used ast_register_atexit, and were only changed when it
-	  was shown to be unsafe.  ast_register_atexit is now used only when
-	  absolutely required to prevent corruption and close child processes.
+	* Fix crashing unit tests with regards to RLS.
 
-	  Exceptions that need to use ast_register_atexit:
-	  * CDR: Flush records.
-	  * res_musiconhold: Kill external applications.
-	  * AstDB: Close the DB.
-	  * canary_exit: Kill canary process.
+	  The unit tests require a sorcery.conf file that has been
+	  set up to store resource lists in memory rather than retrieving
+	  from configuration.
 
-	  ASTERISK-24142 #close
-	  Reported by: David Brillert
+	  With a setup that is not conducive to running the tests, a fault
+	  in sorcery currently causes Asterisk to crash when attempting to
+	  run any of the tests.
 
-	  ASTERISK-24683 #close
-	  Reported by: Peter Katzmann
+	  To get around the crash, this adds a function that verifies the
+	  current environment and marks the tests as "not run" if the setup
+	  is not correct.
+	  ........
 
-	  ASTERISK-24805 #close
-	  Reported by: Badalian Vyacheslav
+	  Merged revisions 420779 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  ASTERISK-24881 #close
-	  Reported by: Corey Farrell
 
-	  Review: https://reviewboard.asterisk.org/r/4500/
-	  Review: https://reviewboard.asterisk.org/r/4501/
-	  ........
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420780 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Merged revisions 433495 from http://svn.asterisk.org/svn/asterisk/branches/11
+2014-08-11 11:03 +0000 [b4e33c81e3]  Mark Michelson <mmichelson@digium.com>
 
+	* Fix crash encountered by the testsuite.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433497 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Running testsuite tests locally produced no errors, but when
+	  run using the continuous integration framework, crashes occurred.
 
-2015-03-26 12:46 +0000 [d0df545a44]  Corey Farrell <git@cfware.com>
+	  The crashes occurred due to a refcounting error that had been fixed
+	  for a similar situation.
+	  ........
 
-	* res_pjsip: Enable unload of all modules at shutdown.
+	  Merged revisions 420758 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Move most of res_pjsip:module_unload to unload_pjsip to resolve crashes
-	    caused by running PJSIP functions from non-PJSIP threads.
-	  * Remove call to pjsip_endpt_destroy(ast_pjsip_endpoint), it was causing
-	    crashes in some cases.  In theory pj_shutdown() should take care of this.
-	  * Mark res_pjsip_keepalive and res_pjsip_session as allowed to unload at
-	    shutdown.
-	  * Resolve leaked config global in res_pjsip_notify.
-	  * Unregister pubsub pjsip service module.
-	  * Implement cleanup for res_pjsip_session.
 
-	  ASTERISK-24731 #close
-	  Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4498/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420759 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
+2014-08-11 08:57 +0000 [becf7c7003]  Matt Jordan <mjordan@digium.com>
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433469 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	* res_hep: Remove disabling of modules
 
-2015-03-26 12:04 +0000 [fd434a210f]  Kevin Harwell <kharwell@digium.com>
+	  These modules were originally specified as being disabled, as they were
+	  introduced midstream in Asterisk 12. That makes it nicer for folks who are
+	  upgrading to a new release in the middle of Asterisk 12. That's not the case
+	  for Asterisk 13: it's a brand new release. There's no reason to have the
+	  modules disabled by default in that case.
+	  ........
 
-	* app_confbridge: file playback blocks dtmf
+	  Merged revisions 420742 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  Attempting to execute DTMF in a confbridge while file playback (prompt,
-	  announcement, etc) is occurring is not allowed. You have to wait until
-	  the sound file has completed before entering DTMF. This patch fixes it
-	  so that app_confbridge now monitors for dtmf key presses during menu
-	  driven file playback. If a key is pressed playback stops and it executes
-	  the matched menu option.
 
-	  ASTERISK-24864 #close
-	  Reported by: Steve Pitts
-	  Review: https://reviewboard.asterisk.org/r/4510/
-	  ........
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420743 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Merged revisions 433445 from http://svn.asterisk.org/svn/asterisk/branches/11
+2014-08-11 05:41 +0000 [1e0846167b]  Walter Doekes <walter+asterisk@wjd.nu>
 
+	* general: Fix memory Corruption in __ast_string_field_ptr_build_va.
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433446 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  If the space left in a stringfield is between 0 and
+	  (alignof(ast_string_field_allocation)-1) adding new data would cause
+	  memory corruption, because we would assume enough space (unsigned
+	  underrun).
 
-2015-03-25 13:37 +0000 [dea885a607]  Richard Mudgett <rmudgett@digium.com>
+	  Thanks Arnd Schmitter for reporting and finding out the cause!
 
-	* A couple minor cleanup tweaks.
+	  ASTERISK-23508 #close
+	  Reported by: Arnd Schmitter
+	  Tested by: Arnd Schmitter, JoshE
 
-	  * In res/res_sorcery_realtime.c: Broke long line.
+	  Review: https://reviewboard.asterisk.org/r/3898/
+	  ........
 
-	  * In main/bucket.c: Eliminated unnecessary NULL check as
-	  ast_sorcery_unref() is NULL tolerant and set the global object to NULL
-	  after unref in the system shutdown bucket_cleanup().
+	  Merged revisions 420680 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 420715 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433420 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 420716 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-03-25 10:30 +0000 [05de9082a5]  Simon Arlott (License 5756)
+	  Merged revisions 420717 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res_xmpp: Buddies are always auto-registered when processing the roster
 
-	  Due to a quirk in the configuration handling of res_xmpp, the 'autoregister'
-	  setting was never actually processed. This was due to not properly copying
-	  over the global settings to the client settings when applying the
-	  configuration to the run-time object.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420718 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4496/
+2014-08-11 04:55 +0000 [b2afbc48e4]  Walter Doekes <walter+asterisk@wjd.nu>
 
-	  ASTERISK-14233
-	  ASTERISK-24780 #close
-	  Reported by: Simon Arlott
-	  patches:
-	    asterisk-13.1.0-24780 uploaded by Simon Arlott (License 5756)
+	* tcptls: Avoid compiler warning on non-dev-mode.
 	  ........
 
-	  Merged revisions 433395 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  Merged revisions 420654 from http://svn.asterisk.org/svn/asterisk/branches/1.8
+	  ........
 
+	  Merged revisions 420655 from http://svn.asterisk.org/svn/asterisk/branches/11
+	  ........
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433396 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  Merged revisions 420656 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2015-03-24 14:26 +0000 [b1e9552b08]  Richard Mudgett <rmudgett@digium.com>
+	  Merged revisions 420657 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* chan_pjsip: Add "rpid_immediate" option to prevent unnecessary "180 Ringing" messages.
 
-	  Incoming PJSIP call legs that have not been answered yet send unnecessary
-	  "180 Ringing" or "183 Progress" messages every time a connected line
-	  update happens.  If the outgoing channel is also PJSIP then the incoming
-	  channel will always send a "180 Ringing" or "183 Progress" message when
-	  the outgoing channel sends the INVITE.
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420658 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Consequences of these unnecessary messages:
+2014-08-10 20:31 +0000 [6650704414]  Matt Jordan <mjordan@digium.com>
 
-	  * The caller can start hearing ringback before the far end even gets the
-	  call.
+	* funcs/func_jitterbuffer: Tweak documentation
 
-	  * Many phones tend to grab the first connected line information and refuse
-	  to update the display if it changes.  The first information is not likely
-	  to be correct if the call goes to an endpoint not under the control of the
-	  first Asterisk box.
+	  This patch merely reformats and cleans up a bit of the jitterbuffer
+	  documentation for the wiki.
+	  ........
 
-	  When connected line first went into Asterisk in v1.8, chan_sip received an
-	  undocumented option "rpid_immediate" that defaults to disabled.  When
-	  enabled, the option immediately passes connected line update information
-	  to the caller in "180 Ringing" or "183 Progress" messages as described
-	  above.
+	  Merged revisions 420639 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	  * Added "rpid_immediate" option to prevent unnecessary "180 Ringing" or
-	  "183 Progress" messages.  The default is "no" to disable sending the
-	  unnecessary messages.
 
-	  ASTERISK-24781 #close
-	  Reported by: Richard Mudgett
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420640 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	  Review: https://reviewboard.asterisk.org/r/4473/
+2014-08-10 19:14 +0000 [add46fd27c]  Michael K (License 6621)
 
+	* app_queue: Add RealTime support for queue rules
 
-	  git-svn-id: https://origsvn.digium.com/svn/asterisk/branches/13@433338 65c4cc65-6c06-0410-ace0-fbb531ad65f3
+	  This patch gives the optional ability to keep queue rules in RealTime. It is
+	  important to note that with this patch:
+	   (a) Queue rules in RealTime are only examined on module load/reload
+	   (b) Queue rules are loaded both from the queuerules.conf file as well as the
+	       RealTime backend
+	  To inform app_queue to examine RealTime for queue rules, a new setting has been
+	  added to queuerules.conf's general section "realtime_rules". RealTime queue
+	  rules will only be used when this setting is set to "yes".
+
+	  The schema for the database table supports a rule_name, time, min_penalty, and
+	  max_penalty columns. min_penalty and max_penalty can be relative, if a '-' or
+	  '+' literal is provided. Otherwise, the penalties are treated as constants.
+
+	  For example:
+	  rule_name, time, min_penalty, max_penalty
+	  'default', '10', '20', '30'
+	  'test2', '20', '30', '55'
+	  'test2', '25', '-11', '+1111'
+	  'test2', '400', '112', '333'
+	  'test3', '0', '4564', '46546'
+	  'test_rule', '40', '15', '50'
+
+	  which would result in :
+
+	  Rule: default
+	   - After 10 seconds, adjust QUEUE_MAX_PENALTY to 30 and adjust
+	     QUEUE_MIN_PENALTY to 20
+	  Rule: test2
+	   - After 20 seconds, adjust QUEUE_MAX_PENALTY to 55 and adjust
+	     QUEUE_MIN_PENALTY to 30
+	   - After 25 seconds, adjust QUEUE_MAX_PENALTY by 1111 and adjust
+	     QUEUE_MIN_PENALTY by -11
+	   - After 400 seconds, adjust QUEUE_MAX_PENALTY to 333 and adjust
+	     QUEUE_MIN_PENALTY to 112
+	  Rule: test3
+	   - After 0 seconds, adjust QUEUE_MAX_PENALTY to 46546 and adjust
+	     QUEUE_MIN_PENALTY to 4564
+	  Rule: test_rule
+	   - After 40 seconds, adjust QUEUE_MAX_PENALTY to 50 and adjust
+	     QUEUE_MIN_PENALTY to 15
+
+	  If you use RealTime, the queue rules will be always reloaded on a module
+	  reload, even if the underlying file did not change. With the option disabled,
+	  the rules will only be reloaded if the file was modified.
 
-2015-03-23  Asterisk Development Team <asteriskteam@digium.com>
+	  Review: https://reviewboard.asterisk.org/r/3607/
 
-	* Asterisk 13.3.0-rc1 Released.
+	  ASTERISK-23823 #close
+	  Reported by: Michael K
+	  patches:
+	    app_queue.c_realtime_trunk.patch uploaded by Michael K (License 6621)
+	  ........
 
-2015-03-22 23:58 +0000 [r433247-433269]  Matthew Jordan <mjordan@digium.com>
+	  Merged revisions 420624 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* apps/app_queue.c, main/cli.c, main/cdr.c, main/manager.c,
-	  main/rtp_engine.c, /, funcs/func_cdr.c: Fix compilations errors
-	  on 64-bit OpenBSD systems In versiong 5.5, OpenBSD went to 64-bit
-	  time values. This requires a cast to (long) when printing members
-	  of certain time structs. Review:
-	  https://reviewboard.asterisk.org/r/4507 ASTERISK-24879 #close
-	  Reported by: snuffy Tested by: snuffy patches:
-	  openbsd-time64.diff uploaded by snuffy (License 5024) ........
-	  Merged revisions 433268 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* main/asterisk.c, main/loader.c, main/xmldoc.c, /: Fix compilation
-	  issues for OpenBSD This patch addresses compilation issues for
-	  OpenBSD. Specifically, it addresses: * It allows including
-	  <sys/vmmeter.h> in asterisk.c * Provides a needed (size_t) cast
-	  in xmldoc.c In 13+, it also addresses a conditional inclusion in
-	  loader.c. Review: https://reviewboard.asterisk.org/r/4506
-	  ASTERISK-24880 #close Reported by: snuffy Tested by: snuffy
-	  patches: misc-openbsd.diff uploaded by snuffy (License 5024)
-	  ........ Merged revisions 433245 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-20 19:52 +0000 [r433199-433222]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_messaging.c, res/res_pjsip/pjsip_options.c,
-	  res/res_pjsip.c, res/res_pjsip_nat.c: Audit
-	  ast_pjsip_rdata_get_endpoint() usage for ref leaks. Valgrind
-	  found some memory leaks associated with
-	  ast_pjsip_rdata_get_endpoint(). The leaks would manifest when
-	  sending responses to OPTIONS requests, processing MESSAGE
-	  requests, and res_pjsip supplements implementing the
-	  incoming_request callback. * Fix ast_pjsip_rdata_get_endpoint()
-	  endpoint ref leaks in res/res_pjsip.c:supplement_on_rx_request(),
-	  res/res_pjsip/pjsip_options.c:send_options_response(),
-	  res/res_pjsip_messaging.c:rx_data_to_ast_msg(), and
-	  res/res_pjsip_messaging.c:send_response(). * Eliminated
-	  RAII_VAR() use with ast_pjsip_rdata_get_endpoint() in
-	  res/res_pjsip_nat.c:nat_on_rx_message(). * Fixed inconsistent but
-	  benign return value in
-	  res/res_pjsip/pjsip_options.c:options_on_rx_request(). Review:
-	  https://reviewboard.asterisk.org/r/4511/
-
-	* res/res_pjsip_sdp_rtp.c, main/sorcery.c, main/xmldoc.c:
-	  res_pjsip_sdp_rtp,sorcery: Fix invalid access and memory leak
-	  respectively. Valgrind found a memory leak and invalid access. *
-	  Fix invalid access by sscanf() being fed a non-nul terminated
-	  string of digits in res/res_pjsip_sdp_rtp.c:get_codecs(). * Fix
-	  memory leak in main/sorcery.c:sorcery_object_field_destructor().
-	  * Fix potential NULL pointer dereference in
-	  main/xmldoc.c:xmldoc_get_syntax_config_option(). Review:
-	  https://reviewboard.asterisk.org/r/4513/
 
-2015-03-19 19:19 +0000 [r433174]  Matthew Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420625 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* funcs/func_env.c, tests/test_func_file.c, /: funcs/func_env: Fix
-	  regression caused in FILE read operation When r432935 was merged,
-	  it did correctly fix a situation where a FILE read operation on
-	  the middle of a file buffer would not read the requested length
-	  in the parameters passed to the FILE function. Unfortunately, it
-	  would also allow the FILE function to append more bytes than what
-	  was available in the buffer if the length exceeded the end of the
-	  buffer length. This patch takes the minimum of the remaining
-	  bytes in the buffer along with the calculated length to append
-	  provided by the original patch, and uses that as the length to
-	  append in the return result. This patch also updates the unit
-	  tests with the scenarios that were originally pointed out in
-	  ASTERISK-21765 that the original implementation treated
-	  incorrectly. ASTERISK-21765 ........ Merged revisions 433173 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-19 10:20 +0000 [r433113-433126]  Corey Farrell <git@cfware.com>
-
-	* main/logger.c, /: logger: Apply default console logging when
-	  configuration cannot be loaded. When logger.conf is missing or
-	  invalid enable console logging and display an error message.
-	  ASTERISK-24817 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4497/ ........ Merged
-	  revisions 433122 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* channels/sip/include/dialog.h, channels/chan_sip.c,
-	  channels/sip/include/sip.h: chan_sip: Simplify dialog/peer
-	  references, improve REF_DEBUG output. * Replace functions for
-	  ref/undef of dialogs and peers with macro's to call
-	  ao2_t_bump/ao2_t_cleanup. * Enable passthough of REF_DEBUG caller
-	  information to sip_alloc and find_call. ASTERISK-24882 #close
-	  Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4189/
-
-	* /, channels/chan_sip.c: chan_sip: Fix dialog reference leaked to
-	  scheduler for reinvite_timeout. Release the scheduler reference
-	  to the dialog for reinvite timeout during dialog_unlink_all.
-	  ASTERISK-24876 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4491/ ........ Merged
-	  revisions 433112 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-18 02:34 +0000 [r433088]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_session.c: res_pjsip_session: Fix off-nominal extra
-	  unref of session.
-
-2015-03-17 22:15 +0000 [r433060-433064]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* main/asterisk.c, main/config.c, main/xmldoc.c, main/manager.c,
-	  include/asterisk/config.h, main/utils.c, main/codec_builtin.c,
-	  main/endpoints.c: Various: bugfixes found via chaos Using
-	  DEBUG_CHAOS several instances of a null pointer crash, and one
-	  uninitialized variable were uncovered and fixed. Also added
-	  details on why Asterisk failed to initialize. Review:
-	  https://reviewboard.asterisk.org/r/4468/
-
-	* build_tools/cflags.xml, include/asterisk/utils.h: core: Introduce
-	  chaos into memory allocations Locate potential crashes by
-	  exercising seldom used code paths. This patch introduces a new
-	  define DEBUG_CHAOS, and mechanism to randomly return an error
-	  condition from functions that will seldom do so. Functions that
-	  handle the allocation of memory get the first treatment. Review:
-	  https://reviewboard.asterisk.org/r/4463/
-
-2015-03-17 21:49 +0000 [r433057]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/netsock2.c, /, res/res_pjsip_sdp_rtp.c, res/res_pjsip_t38.c,
-	  apps/app_externalivr.c, res/res_pjsip_acl.c: Audit
-	  ast_sockaddr_resolve() usage for memory leaks. Valgrind found
-	  some memory leaks associated with ast_sockaddr_resolve(). Most of
-	  the leaks had already been fixed by earlier memory leak hunt
-	  patches. This patch performs an audit of ast_sockaddr_resolve()
-	  and found one more. * Fix ast_sockaddr_resolve() memory leak in
-	  apps/app_externalivr.c:app_exec(). * Made
-	  main/netsock2.c:ast_sockaddr_resolve() always set the addrs
-	  parameter for safety so the pointer will never be uninitialized
-	  on return. The same goes for
-	  res/res_pjsip_acl.c:extract_contact_addr(). * Made functions that
-	  call ast_sockaddr_resolve() with RAII_VAR() controlling the addrs
-	  variable use ast_free instead of ast_free_ptr to provide better
-	  MALLOC_DEBUG information. Review:
-	  https://reviewboard.asterisk.org/r/4509/ ........ Merged
-	  revisions 433056 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-17 18:34 +0000 [r433028-433031]  Kevin Harwell <kharwell@digium.com>
-
-	* include/asterisk/res_pjsip.h,
-	  res/res_pjsip_endpoint_identifier_anonymous.c,
-	  res/res_pjsip_endpoint_identifier_ip.c, res/res_pjsip.c,
-	  res/res_pjsip_endpoint_identifier_user.c: res_pjsip: Allow
-	  configuration of endpoint identifier query order Updated some
-	  documentation stating that endpoint identifiers registered
-	  without a name are place at the front of the lookup list. Also
-	  renamed register method
-	  'ast_sip_register_endpoint_identifier_by_name' to
-	  'ast_sip_register_endpoint_identifier_with_name' ASTERISK-24840
-	  Reported by: Mark Michelson
+2014-08-10 17:02 +0000 [f7bb772804]  Matt Jordan <mjordan@digium.com>
 
-	* configs/samples/pjsip.conf.sample, CHANGES, res/res_pjsip.c,
-	  res/res_pjsip_endpoint_identifier_user.c,
-	  include/asterisk/res_pjsip.h, res/res_pjsip/config_global.c,
-	  res/res_pjsip_endpoint_identifier_anonymous.c,
-	  res/res_pjsip_endpoint_identifier_ip.c,
-	  contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py:
-	  res_pjsip: Allow configuration of endpoint identifier query order
-	  This patch fixes previously reverted code that caused binary
-	  incompatibility problems with some modules. And like the original
-	  patch it makes sure that no matter what order the endpoint
-	  identifier modules were loaded, priority is given based on the
-	  ones specified in the new global 'endpoint_identifier_order'
-	  option. ASTERISK-24840 Reported by: Mark Michelson Review:
-	  https://reviewboard.asterisk.org/r/4489/
-
-2015-03-17 16:10 +0000 [r433005]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip.c: res_pjsip: Add reason comment.
-
-2015-03-14 02:28 +0000 [r432971]  Matthew Jordan <mjordan@digium.com>
-
-	* /, main/format_cap.c: main/frame: Don't report empty disallow
-	  values as an error In realtime, it is normal to have a database
-	  with both 'allow' and 'disallow' columns in the schema. It is
-	  perfectly valid to have an 'allow' value of '!all,g722,ulaw,alaw'
-	  and no 'disallow' value. Unlike in static conf files, you can't
-	  *not* provide the disallow value. Thus, the empty disallow value
-	  causes a spurious WARNING message, which is kind of annoying.
-	  This patch makes it so that a 'disallow' value with no ... value
-	  ... is ignored. Granted, you can still screw this up as well, as
-	  technically specifying 'disallow=all,!ulaw' allows only ulaw, and
-	  then you would have no 'allow' value in your database. But
-	  really, why would you do that? WHY? ASTERISK-16779 #close
-	  Reported by: Atis Lezdins ........ Merged revisions 432970 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-14 02:00 +0000 [r432945-432949]  Joshua Colp <jcolp@digium.com>
-
-	* funcs/func_curl.c, /: func_curl: Don't hold exclusive lock when
-	  performing HTTP request. This code originally kept a lock held
-	  when performing the HTTP request to ensure that the options
-	  provided to curl remain valid. This doesn't seem to be necessary
-	  these days and holding the lock caused requests to happen
-	  sequentially instead of in parallel. ASTERISK-18708 #close
-	  Reported by: Dave Cabot ........ Merged revisions 432948 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* main/cli.c, /: core: Fix tab completion of "core set debug
-	  channel" CLI command. The "core set debug channel" CLI command
-	  mistakenly had source filenames added to its tab completion. This
-	  occurred because the CLI generator fell back to the "core set
-	  debug" command which permits setting debug at a source filename
-	  level. ASTERISK-21038 #close Reported by: Richard Kenner ........
-	  Merged revisions 432944 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-14 01:21 +0000 [r432920-432938]  Matthew Jordan <mjordan@digium.com>
-
-	* /, funcs/func_env.c: FILE: fix retrieval of file contents when
-	  offset is specified The loop that reads in a file was not
-	  correctly using the offset when determining what bytes to append
-	  to the output. This patch corrects the logic such that the
-	  correct portion of the file is extracted when an offset is
-	  specified. ASTERISK-21765 Reported by: John Zhong Tested by: Matt
-	  Jordan, Di-Shi Sun patches: file_read_390821.patch uploaded by
-	  Di-Shi Sun (License 5076) ........ Merged revisions 432935 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* apps/app_amd.c, /, configs/samples/amd.conf.sample: apps/app_amd:
-	  Document maximum_word_length option; fix AMDCAUSE documentation
-	  This patch corrects the documentation for the AMD application.
-	  Specifically: * It documents the maximum_word_length option,
-	  which limits the maximum allowed length of a single utterance. *
-	  It clarifies the AMDCAUSE values MAXWORDS and MAXWORDLENGTH.
-	  MAXWORDLENGTH was documented as MAXWORDS, while MAXWORDS was
-	  undocumented. Thanks to the issue reporter, Frank DiGennaro, for
-	  pointing out the issues. ASTERISK-19470 #close Reported by: Frank
-	  DiGennaro ........ Merged revisions 432918 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-13 17:04 +0000 [r432892-432894]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip/pjsip_configuration.c: chan_pjsip: AMI action
-	  PJSIPShowEndpoint closes AMI connection on error. Also fixed
-	  similar problem with AMI action PJSIPShowEndpoints.
-	  ASTERISK-24872 #close Reported by: Dmitriy Serov Review:
-	  https://reviewboard.asterisk.org/r/4487/
-
-	* channels/chan_pjsip.c, res/res_pjsip_caller_id.c:
-	  chan_pjsip/res_pjsip_callerid: Make Party ID handling simpler and
-	  consistent. The res_pjsip modules were manually checking both
-	  name and number presentation values when there is a function that
-	  determines the combined presentation for a party ID struct. The
-	  function takes into account if the name or number components are
-	  valid while the manual code rarely checked if the data was even
-	  valid. * Made use ast_party_id_presentation() rather than
-	  manually checking party ID presentation values. * Ensure that
-	  set_id_from_pai() and set_id_from_rpid() will not return
-	  presentation values other than what is pulled out of the SIP
-	  headers. It is best if the code doesn't assume that
-	  AST_PRES_ALLOWED and AST_PRES_USER_NUMBER_UNSCREENED are zero. *
-	  Fixed copy paste error in add_privacy_params() dealing with RPID
-	  privacy. * Pulled the id->number.valid test from
-	  add_privacy_header() and add_privacy_params() up into the parent
-	  function add_id_headers() to skip adding PAI/RPID headers
-	  earlier. * Made update_connected_line_information() not send out
-	  connected line updates if the connected line number is invalid.
-	  Lower level code would not add the party ID information and thus
-	  the sent message would be unnecessary. * Eliminated RAII_VAR
-	  usage in send_direct_media_request(). Review:
-	  https://reviewboard.asterisk.org/r/4472/
-
-2015-03-13 14:48 +0000 [r432868]  Kevin Harwell <kharwell@digium.com>
-
-	* include/asterisk/res_pjsip.h, res/res_pjsip/config_global.c,
-	  res/res_pjsip_endpoint_identifier_anonymous.c,
-	  res/res_pjsip_endpoint_identifier_ip.c,
-	  contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py,
-	  configs/samples/pjsip.conf.sample, CHANGES, res/res_pjsip.c,
-	  res/res_pjsip_endpoint_identifier_user.c: Revert - res_pjsip:
-	  Allow configuration of endpoint identifier query order Due to a
-	  break in binary compatibility with some other modules these
-	  changes are being reverted until the issue can be resolved.
-	  ASTERISK-24840 Reported by: Mark Michelson
-
-2015-03-12 12:58 +0000 [r432808-432811]  Matthew Jordan <mjordan@digium.com>
-
-	* /, main/audiohook.c: main/audiohook: Update internal sample rate
-	  on reads When an audiohook is created (which is used by the
-	  various Spy applications and Snoop channel in Asterisk 13+), it
-	  initially is given a sample rate of 8kHz. It is expected,
-	  however, that this rate may change based on the media that passes
-	  through the audiohook. However, the read/write operations on the
-	  audiohook behave very differently. When a frame is written to the
-	  audiohook, the format of the frame is checked against the
-	  internal sample rate. If the rate of the format does not match
-	  the internal sample rate, the internal sample rate is updated and
-	  a new SLIN format is chosen based on that sample rate. This works
-	  just fine. When a frame is read, however, we do something quite
-	  different. If the format rate matches the internal sample rate,
-	  all is fine. However, if the rates don't match, the audiohook
-	  attempts to "fix up" the number of samples that were requested.
-	  This can result in some seriously large number of samples being
-	  requested from the read/write factories. Consider the worst case
-	  - 192kHz SLIN. If we attempt to read 20ms worth of audio produced
-	  at that rate, we'd request 3840 samples (192000 / (1000 / 20)).
-	  However, if the audiohook is still expecting an internal sample
-	  rate of 8000, we'll attempt to "fix up" the requested samples to:
-	  samples_converted = samples * (ast_format_get_sample_rate(format)
-	  / (float) audiohook->hook_internal_samp_rate); which is: 92160 =
-	  3840 * (192000 / 8000) This results in us attempting to read
-	  92160 samples from our factories, as opposed to the 3840 that we
-	  actually wanted. On a 64-bit machine, this miraculously survives
-	  - despite allocating up to two buffers of length 92160 on the
-	  stack. The 32-bit machines aren't quite so lucky. Even in the
-	  case where this works, we will either (a) get way more samples
-	  than we wanted; or (b) get about 3840 samples, assuming the
-	  timing is pretty good on the machine. Either way, the calculation
-	  being performed is wrong, based on the API users expectations. My
-	  first inclination was to allocate the buffers on the heap. As it
-	  is, however, there's at least two drawbacks with doing this: (1)
-	  It's a bit complicated, as the size of the buffers may change
-	  during the lifetime of the audiohook (ew). (2) The stack is
-	  faster (yay); the heap is slower (boo). Since our calculation is
-	  flat out wrong in the first place, this patch fixes this issue by
-	  instead updating the internal sample rate based on the format
-	  passed into the read operation. This causes us to read the
-	  correct number of samples, and has the added benefit of setting
-	  the audihook with the right SLIN format. Note that this issue was
-	  caught by the Asterisk Test Suite as a result of r432195 in the
-	  13 branch. Because this issue is also theoretically possible in
-	  Asterisk 11, the change is being made here as well. Review:
-	  https://reviewboard.asterisk.org/r/4475/ ........ Merged
-	  revisions 432810 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* Makefile, include/asterisk/utils.h, /, configure, main/Makefile,
-	  configure.ac, include/asterisk/inline_api.h, makeopts.in: Add
-	  support for the clang compiler; update RAII_VAR to use
-	  BlocksRuntime RAII_VAR, which is used extensively in Asterisk to
-	  manage reference counted resources, uses a GCC extension to
-	  automatically invoke a cleanup function when a variable loses
-	  scope. While this functionality is incredibly useful and has
-	  prevented a large number of memory leaks, it also prevents
-	  Asterisk from being compiled with clang. This patch updates the
-	  RAII_VAR macro such that it can be compiled with clang. It makes
-	  use of the BlocksRuntime, which allows for a closure to be
-	  created that performs the actual cleanup. Note that this does not
-	  attempt to address the numerous warnings that the clang compiler
-	  catches in Asterisk. Much thanks for this patch goes to: * The
-	  folks on StackOverflow who asked this question and Leushenko for
-	  providing the answer that formed the basis of this code:
-	  http://stackoverflow.com/questions/24959440/rewrite-gcc-cleanup-macro-with-nested-function-for-clang
-	  * Diederik de Groot, who has been extremely patient in working on
-	  getting this patch into Asterisk. Review:
-	  https://reviewboard.asterisk.org/r/4370/ ASTERISK-24133
-	  ASTERISK-23666 ASTERISK-20399 ASTERISK-20850 #close Reported by:
-	  Diederik de Groot patches: RAII_CLANG.patch uploaded by Diederik
-	  de Groot (License 6600) ........ Merged revisions 432807 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-11 16:38 +0000 [r432764-432787]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip/config_domain_aliases.c,
-	  res/res_pjsip/include/res_pjsip_private.h,
-	  include/asterisk/res_pjsip.h: res_pjsip: Move internal
-	  init/destroy prototypes to private header file. Done as a
-	  separate commit from a finding in
-	  https://reviewboard.asterisk.org/r/4467/
+	* Update CHANGES file
+	  ........
 
-	* include/asterisk/res_pjsip.h, res/res_pjsip/config_global.c,
-	  res/res_pjsip/pjsip_configuration.c: res_pjsip: Fix pjsip.conf
-	  type=global object default value handling. When a type=global
-	  section is not defined in pjsip.conf the global defaults are not
-	  applied. As a result the mandatory Max-Forwards header is not
-	  added to SIP messages for res_pjsip/chan_pjsip. The handling of
-	  pjsip.conf type=global objects has several problems: 1) If the
-	  global object is missing the defaults are not applied. 2) If the
-	  global object is missing the default_outbound_endpoint's default
-	  value is not returned by
-	  ast_sip_global_default_outbound_endpoint(). 3) Defines are needed
-	  so default values only need to be changed in one place. * Added a
-	  sorcery instance observer callback to check if there were any
-	  type=global sections loaded. If there were more than one then
-	  issue an error message. If there were none then apply the global
-	  defaults. * Fixed ast_sip_global_default_outbound_endpoint() to
-	  return the documented default when no type=global object is
-	  defined. * Made defines for the global default values. *
-	  Increased the default_useragent[] size because SVN version
-	  strings can get lengthy and 128 characters may not be enough. *
-	  Fixed an off-nominal code path ref leak in global_alloc() if the
-	  string fields fail to initialize. * Eliminated RAII_VAR in
-	  get_global_cfg() and ast_sip_global_default_outbound_endpoint().
-	  ASTERISK-24807 #close Reported by: Anatoli Review:
-	  https://reviewboard.asterisk.org/r/4467/
+	  Merged revisions 420609 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_pjsip/pjsip_global_headers.c: res_pjsip: Fixed invalid
-	  empty Server and User-Agent SIP headers. Setting pjsip.conf
-	  useragent to an empty string results in an empty SIP header being
-	  sent. * Made not add an empty SIP header item to the global SIP
-	  headers list. Review: https://reviewboard.asterisk.org/r/4467/
-
-2015-03-10 23:09 +0000 [r432742]  Joshua Colp <jcolp@digium.com>
-
-	* main/stasis_channels.c, main/endpoints.c, main/stasis_bridges.c:
-	  core: Don't create snapshots with locks. Snapshots are immutable
-	  and are never changed. Allocating them with a lock is wasteful.
-	  Review: https://reviewboard.asterisk.org/r/4469/
 
-2015-03-10 21:33 +0000 [r432693-432721]  Matthew Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420610 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/res_config_odbc.c, /: res/res_config_odbc: Fix improper
-	  escaping of backslashes with MySQL When escaping backslashes with
-	  MySQL, the proper way to escape the characters in a LIKE clause
-	  is to escape the '\' four times, i.e., '\\\\'. To quote the MySQL
-	  manual: "Because MySQL uses C escape syntax in strings (for
-	  example, “\n” to represent a newline character), you must double
-	  any “\” that you use in LIKE strings. For example, to search for
-	  “\n”, specify it as “\\n”. To search for “\”, specify it as
-	  “\\\\”; this is because the backslashes are stripped once by the
-	  parser and again when the pattern match is made, leaving a single
-	  backslash to be matched against." ASTERISK-24808 #close Reported
-	  by: Javier Acosta patches: res_config_odbc.diff uploaded by
-	  Javier Acosta (License 6690) ........ Merged revisions 432720
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* apps/app_voicemail.c, /: app_voicemail: Fix crash with IMAP
-	  backends when greetings aren't present When an IMAP backend is in
-	  use and greetings are set to be used, but aren't present for a
-	  user in their IMAP folder, Asterisk will crash. This occurs due
-	  to the mailstream being set to the 'greetings' folder and being
-	  left in that particular state, regardless of the success/failure
-	  of the attempt to access the folder the mailstream points to.
-	  Later access of the mailstream assumes that it points to the
-	  'INBOX' (or some other folder), resulting in either a crash (if
-	  the greetings folder didn't exist and the mailstream is invalid)
-	  or an inability to read messages from the 'INBOX' folder. This
-	  patch restores the mailstream to its correct state after
-	  accessing the greetings. This fixes the crash, and sets the
-	  mailstream to the state that VoiceMailMain expects. Note that
-	  while ASTERISK-23390 also contained a patch for this issue, the
-	  patch on ASTERISK-24786 is the one being merged here. Review:
-	  https://reviewboard.asterisk.org/r/4459/ ASTERISK-23390 #close
-	  Reported by: Ben Smithurst ASTERISK-24786 #close Reported by:
-	  Graham Barnett Tested by: Graham Barnett patches:
-	  app_voicemail.c.patch.SIGSEGV3rev2 uploaded by Graham Barnett
-	  (License 6685) ........ Merged revisions 432695 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, main/stdtime/localtime.c: localtime: Fix file descriptor leak
-	  on kqueue(2) systems The localtime management in the Asterisk
-	  core contains a thread that watches for changes in the local
-	  timezone. On systems where the directory containing
-	  /etc/localtime is modified frequently, the thread monitoring the
-	  changes will be woken up to determine if any changes in timezone
-	  have occurred. When using kqueue(2), this can cause a leak of
-	  file descriptors due to some improper management of resources.
-	  This patch updates the kqueue(2) handling in localtime, such that
-	  is no longer leaks resources. Review:
-	  https://reviewboard.asterisk.org/r/4450/ ASTERISK-24739 #close
-	  Reported by: Ed Hynan patches: 11.15.0-u.diff uploaded by Ed
-	  Hynan (Licnese 6680) 11.7.0-u.diff uploaded by Ed Hynan (License
-	  6680) svn-trunk-Jan-26-2015-u.diff uploaded by Ed Hynan (License
-	  6680) ........ Merged revisions 432691 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-10 16:04 +0000 [r432668]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_refer.c, res/res_pjsip_session.c,
-	  include/asterisk/res_pjsip_session.h,
-	  res/res_pjsip_session.exports.in: res_pjsip_refer: Fix occasional
-	  unexpected BYE sent after receiving a REFER. A race condition
-	  happened between initiating a transfer and requesting that a
-	  dialog termination be delayed. Occasionally, the transferrer
-	  channels would exit the bridge and hangup before the dialog
-	  termination delay was requested. * Made request dialog
-	  termination delay before initiating the transfer action. If the
-	  transfer fails then cancel the delayed dialog termination
-	  request. ASTERISK-24755 #close Reported by: John Bigelow Review:
-	  https://reviewboard.asterisk.org/r/4460/
-
-2015-03-09 16:12 +0000 [r432638]  Kevin Harwell <kharwell@digium.com>
-
-	* include/asterisk/res_pjsip.h, res/res_pjsip/config_global.c,
-	  res/res_pjsip_endpoint_identifier_anonymous.c,
-	  res/res_pjsip_endpoint_identifier_ip.c,
-	  contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py
-	  (added), configs/samples/pjsip.conf.sample, CHANGES,
-	  res/res_pjsip.c, res/res_pjsip_endpoint_identifier_user.c:
-	  res_pjsip: Allow configuration of endpoint identifier query order
-	  It's possible to have a scenario that will create a conflict
-	  between endpoint identifiers. For instance an incoming call could
-	  be identified by two different endpoint identifiers and the one
-	  chosen depended upon which identifier module loaded first. This
-	  of course causes problems when, for example, the incoming call is
-	  expected to be identified by username, but instead is identified
-	  by ip. This patch adds a new 'global' option to res_pjsip called
-	  'endpoint_identifier_order'. It is a comma separated list of
-	  endpoint identifier names that specifies the order by which
-	  identifiers are processed and checked. ASTERISK-24840 #close
-	  Reported by: Mark Michelson Review:
-	  https://reviewboard.asterisk.org/r/4455/
-
-2015-03-08 01:46 +0000 [r432614]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_rtp_asterisk.c: res_rtp_asterisk: Fix wrongful use of
-	  USE_PJPROJECT define. As pjproject is now used as a shared
-	  library a different define, HAVE_PJPROJECT, is used to specify if
-	  pjproject is present. ASTERISK-24830 #close Reported by: Stefan
-	  Engström
-
-2015-03-06 22:50 +0000 [r432574-432594]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_refer.c: res_pjsip_refer: Make safely get the
-	  context for a blind transfer. Made safely get the
-	  TRANSFER_CONTEXT channel value while the channel is locked in
-	  refer_incoming_attended_request() and
-	  refer_incoming_blind_request(). The pointer returned by
-	  pbx_builtin_getvar_helper() is only valid while the channel is
-	  locked.
+2014-08-10 16:35 +0000 [455243cdd4]  Matt Jordan <mjordan@digium.com>
 
-	* res/res_pjsip_refer.c: res_pjsip_refer: Made
-	  refer_attended_alloc() not create the ao2 object with a lock. The
-	  lock is unused.
-
-2015-03-06 21:11 +0000 [r432556]  Jonathan Rose <jrose@digium.com>
-
-	* include/asterisk/app.h, main/app.c: app: Add functions to swap
-	  voicemail function table for testing purposes
-
-2015-03-06 20:18 +0000 [r432528-432534]  Richard Mudgett <rmudgett@digium.com>
-
-	* channels/chan_dahdi.c, channels/sig_analog.c, /,
-	  channels/chan_dahdi.h, channels/sig_analog.h, UPGRADE.txt:
-	  chan_dahdi/sig_analog: Fix distinctive ring detection to suck
-	  less. The distinctive ring feature interferes with detecting
-	  Caller ID and appears to have been broken for years. What happens
-	  is if you have a ring-ring cadence as used in the UK you get too
-	  many DAHDI events for the distinctive ring pattern array and
-	  Caller ID detection is aborted. I think when Zapata/DAHDI added
-	  the ring begin event it broke distinctive ring. More events
-	  happen than before and the code does no filtering of which event
-	  times are recorded in the pattern array. * Made distinctive ring
-	  only record the ringt count when the ring ends instead of on just
-	  any DAHDI event. Distinctive ring can be ring, ring-ring,
-	  ring-ring-ring, or different ring durations for the up to three
-	  rings. * Fixed the distinctive ring detection enable
-	  (chan_dahdi.conf option usedistinctiveringdetection) to be per
-	  port instead of somewhat per port and somewhat global. This has
-	  been broken since v1.8. * Fixed using the default distinctive
-	  ring context when the detected pattern does not match any
-	  configured dringX patterns. The default context did not get set
-	  when the previous call was a matched distinctive ring pattern and
-	  the current call is not matched. This has been broken since v1.8.
-	  * Made distinctive ring have no effect on Caller ID detection
-	  when it is disabled. Caller ID detection just monitors for 10
-	  seconds before giving up. * Fixed leak of struct callerid_state
-	  memory when a polarity reversal during Caller ID detection causes
-	  the incoming call to be aborted. DAHDI-1143 AST-1545
-	  ASTERISK-24825 #close Reported by: Richard Mudgett ASTERISK-17588
-	  Reported by: Daniel Flounders Review:
-	  https://reviewboard.asterisk.org/r/4444/ ........ Merged
-	  revisions 432530 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, channels/chan_sip.c: chan_sip: Fix realtime locking inversion
-	  when poking a just built peer. When a realtime peer is built it
-	  can cause a locking inversion when the just built peer is poked.
-	  If the CLI command "sip show channels" is periodically executed
-	  then a deadlock can happen because of the locking inversion. *
-	  Push the peer poke off onto the scheduler thread to avoid the
-	  locking inversion of the just built realtime peer. AST-1540
-	  ASTERISK-24838 #close Reported by: Richard Mudgett Review:
-	  https://reviewboard.asterisk.org/r/4454/ ........ Merged
-	  revisions 432526 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-05 16:38 +0000 [r432485]  George Joseph <george.joseph@fairview5.com>
-
-	* apps/app_voicemail.c, /: app_voicemail: Fix compile breaking in
-	  app_voicemail with IMAP_STORAGE. There is a leftover "assert" in
-	  app_voicemail/__messagecount that references variables that don't
-	  exist. This causes the compile to fail when --enable-dev-mode and
-	  IMAP_STORAGE are selected. This patch removes the assert.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4461/ ........ Merged
-	  revisions 432484 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-03-04 18:52 +0000 [r432453]  Matthew Jordan <mjordan@digium.com>
-
-	* main/translate.c: translate: Prevent invalid memory accesses on
-	  fast shutdown When a 'core restart now' or 'core stop now' is
-	  executed and a channel is currently in a media operation, the
-	  translator matrix can be destroyed while a channel is currently
-	  blocked on getting the best translation choice (see
-	  ast_translator_best_choice). When the channel gets the mutex, the
-	  translation matrix now has invalid memory, and Asterisk crashes.
-	  This patch does two things: (1) We now only clean up the
-	  translation matrix on a graceful shutdown. In that case, there
-	  are no channels, and so there is no risk of this occurring. (2)
-	  We also now set the __matrix and __indextable to NULL. In some
-	  initial backtraces when this occurred, it looked as if there was
-	  a memory corruption occurring, and it wasn't until we determined
-	  that something had restarted Asterisk that the issue became
-	  clear. By setting these to NULL on shutdown, it becomes a bit
-	  easier to determine why a crash is occurring. Note that we could
-	  litter the code with NULL checks on the __matrix, but the act of
-	  making the translation matrix cleaned up on shutdown should
-	  preclude this issue from occurring in the first place, and this
-	  part of the code needs to be as fast as possible. Review:
-	  https://reviewboard.asterisk.org/r/4457/
-
-2015-03-02 19:14 +0000 [r432423]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_pjsip_sdp_rtp.c: res/res_pjsip_sdp_rtp: Revert portion of
-	  r432195 Unfortunately, while initial testing with ConfBridge did
-	  not reproduce the audio problem alluded to in the comment in
-	  res_pjsip_sdp_rtp, further testing did show that bridge_softmix
-	  and/or ConfBridge has a severe problem bridging two or more
-	  participants at different sampling rates. Sometimes, it even
-	  picks odd sampling rates that cause hideous audio problems. This
-	  patch backs out the offending portion of the code until the
-	  issues in the affected bridging modules can be more properly
-	  analyzed. ASTERISK-24841
-
-2015-02-27 18:23 +0000 [r432404]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/json.c, rest-api/api-docs/endpoints.json,
-	  res/ari/resource_endpoints.c, res/res_ari_endpoints.c,
-	  include/asterisk/json.h, res/ari/resource_channels.c: ARI: Fix
-	  crash if integer values used in JSON payload 'variables' object.
-	  Sending the following ARI commands caused Asterisk to crash if
-	  the JSON body 'variables' object passes values of types other
-	  than strings. POST /ari/channels POST /ari/channels/{channelid}
-	  PUT /ari/endpoints/sendMessage PUT
-	  /ari/endpoints/{tech}/{resource}/sendMessage * Eliminated
-	  RAII_VAR usage in ast_ari_channels_originate_with_id(),
-	  ast_ari_channels_originate(), ast_ari_endpoints_send_message(),
-	  and ast_ari_endpoints_send_message_to_endpoint(). ASTERISK-24751
-	  #close Reported by: jeffrey putnam Review:
-	  https://reviewboard.asterisk.org/r/4447/
-
-2015-02-26 18:52 +0000 [r432385]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* include/asterisk/dial.h, main/dial.c: Dial API: add self destruct
-	  option when complete This patch adds a self-destruction option to
-	  the dial api. The usefulness of this is mostly when using async
-	  mode to spawn a separate thread used to handle the new call,
-	  while the calling thread is allowed to go on about other
-	  business. The only alternative to this option would be the
-	  calling thread spawning a new thread, or hanging around itself
-	  waiting to destroy the dial struct after completion. Example of
-	  use (minus error checking): struct ast_dial *dial =
-	  ast_dial_create(); ast_dial_append(dial, "PJSIP", "200", NULL);
-	  ast_dial_option_global_enable(dial, AST_DIAL_OPTION_ANSWER_EXEC,
-	  "Echo"); ast_dial_option_global_enable(dial,
-	  AST_DIAL_OPTION_SELF_DESTROY, NULL); ast_dial_run(dial, NULL, 1);
-	  The dial_run call will return almost immediately after spawning
-	  the new thread to run and monitor the dial. If the call is
-	  answered, it is placed into the echo app. When completed, it will
-	  call ast_dial_destroy() on the dial structure. Note that any
-	  allocations made to pass values to ast_dial_set_user_data() or
-	  dial options must be free'd in a state callback function on any
-	  of: AST_DIAL_RESULT_UNASWERED, AST_DIAL_RESULT_ANSWERED,
-	  AST_DIAL_RESULT_HANGUP, or AST_DIAL_RESULT_TIMEOUT. Review:
-	  https://reviewboard.asterisk.org/r/4443/
-
-2015-02-26 17:07 +0000 [r432363]  Kevin Harwell <kharwell@digium.com>
-
-	* /, apps/app_chanspy.c, main/channel.c: app_chanspy, channel: fix
-	  frame leaks Fixed a couple of frame leaks that were found during
-	  testing. ASTERISK-24828 #close Reported by: John Hardin Review:
-	  https://reviewboard.asterisk.org/r/4445/ ........ Merged
-	  revisions 432362 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-26 04:58 +0000 [r432321-432342]  Matthew Jordan <mjordan@digium.com>
-
-	* /, apps/Makefile, channels/Makefile: make: Remove 'res_features'
-	  from libraries to link against with cygwin/mingw32 Both the apps
-	  and channels Makefiles still listed 'res_features' as modules to
-	  link against when compiling for cygwin or mingw32. This module
-	  hasn't existed for quite some time. ASTERISK-18105 #close
-	  Reported by: feyfre ........ Merged revisions 432341 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, channels/chan_sip.c: channels/chan_sip: Don't send a BYE after
-	  final response when PBX thread fails When Asterisk fails to start
-	  a PBX thread for a new channel - for example, when the maxcalls
-	  setting in asterisk.conf is exceeded - we currently send a final
-	  response, and then attempt to send a BYE request to the UA. Since
-	  that's all sorts of wrong, this patch fixes that by setting
-	  sipalreadygone on the sip_pvt such that we don't get stuck
-	  sending BYE requests to something that does not want it. Note
-	  that this patch is a slight modification of the one on
-	  ASTERISK-15434. For clarity, it explicitly calls sipalreadygone
-	  with the calls to transmit a final response. ASTERISK-21845
-	  ASTERISK-15434 #close Reported by: Makoto Dei Tested by: Matt
-	  Jordan patches: sip-pbxstart-failed.patch uploaded by Makoto Dei
-	  (License 5027) ........ Merged revisions 432320 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-25 23:48 +0000 [r432301]  Rusty Newton <rnewton@digium.com>
-
-	* configs/basic-pbx/README (added), configs/basic-pbx (added),
-	  configs/basic-pbx/extensions.conf (added),
-	  configs/basic-pbx/logger.conf (added),
-	  configs/basic-pbx/indications.conf (added),
-	  configs/basic-pbx/musiconhold.conf (added),
-	  configs/basic-pbx/asterisk.conf (added),
-	  configs/basic-pbx/pjsip.conf (added),
-	  configs/basic-pbx/modules.conf (added),
-	  configs/basic-pbx/voicemail.conf (added): configs/basic-pbx -
-	  Super Awesome Company example configs Phase 1, Patch 1 Example
-	  configuration files for a "basic PBX" deployment for the
-	  fictitious Super Awesome Company. Details at
-	  https://reviewboard.asterisk.org/r/4379/ and
-	  https://wiki.asterisk.org/wiki/display/AST/Super+Awesome+Company
-	  Reported by: Malcolm Davenport Tested by: Rusty Newton Review:
-	  https://reviewboard.asterisk.org/r/4379/
-
-2015-02-25 23:09 +0000 [r432258-432281]  Matthew Jordan <mjordan@digium.com>
-
-	* /, configure, configure.ac: configure: Promote SQLite3 "not
-	  installed" warning to error Since Asterisk won't build without
-	  the library, not having it is definitely an error. Thanks to Kyle
-	  Kurz for pointing this out. ........ Merged revisions 432280 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, channels/chan_sip.c: channels/chan_sip: Clarify WARNING
-	  message in mismatched SRTP scenario When we receive an SDP as
-	  part of an offer/answer for a peer/friend has been configured to
-	  require encryption, and that SDP offer/answer failed to provide
-	  acceptable crypto attributes, we currently issue a WARNING that
-	  uses the phrase "we" and "requested". In this case, both of those
-	  terms are ambiguous - the user will probably think "we" is
-	  Asterisk (it most likely isn't) and it may not be a "request", so
-	  much as an SDP that was received in some fashion. This patch
-	  makes the WARNING messages slightly less bad and a bit more
-	  accurate as well. ASTERISK-23214 #close Reported by: Rusty Newton
-	  ........ Merged revisions 432277 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* main/sdp_srtp.c, /: channels/sip/sdp_crypto: Handle SRTP keys
-	  negotiated with key lifetime/MKI Prior to this patch, SDP offers
-	  negotiating SDES-SRTP crypto attributes would be rejected if
-	  those crypto attributes contained either a key lifetime or a MKI
-	  parameter. While from a theoretical point of view this was
-	  defensible - Asterisk does not support key lifetimes or multiple
-	  crypto keys - from a practical point of view, this is quite a
-	  problem. A large number of endpoints offer lifetimes/MKI, which
-	  Asterisk can tolerate so long as it doesn't actually have to
-	  support anything more than a single key or refresh the key. In
-	  reality, this is (so far as we've seen) always the case. This
-	  patch is a forward port of Olle's work in the
-	  lingon-srtp-key-lifetime-1.8 branch. To quote Olle from
-	  ASTERISK-17721, it handles lifetime/MKI parameters in the
-	  following fashion: > The Lingon branch now handle lifetime and
-	  MKI parameters. > > We only accept lifetimes up to max for the
-	  crypto and higher than 10 hours > for packetization of 20 ms (50
-	  pps). > > We only handle MKI with index 1. > > We do not really
-	  bother with counting packets and reinviting at end of > lifetime,
-	  so the min of 10 hours kind of takes care of most calls. If there
-	  > are longer ones, we rely on the other side for re-invites. > >
-	  It's still not perfect, but I personally think this is an
-	  improvement. A > configuration option for minimum lifetime
-	  accepted could be added. When the patch was ported forward, I
-	  decided against adding a configuration option as Olle's handling
-	  was more than sufficient for every case I've seen come through
-	  the issue tracker or through interoperability testing. We can
-	  revisit that decision if it proves to be false. A few small other
-	  tweaks were made to the surrounding code to reduce indentation
-	  and provide better type safety for the 'tag' parameter. Review:
-	  https://reviewboard.asterisk.org/r/4419/ Review:
-	  https://reviewboard.asterisk.org/r/4418/ ASTERISK-17721 #close
-	  Reported by: Terry Wilson ASTERISK-17899 #close Reported by:
-	  Dwayne Hubbard patches: lingon-srtp-key-lifetime-1.8.diff
-	  uploaded by oej (License 5267) ASTERISK-20233 Reported by: tootai
-	  ASTERISK-22748 Reported by: Alejandro Mejia ........ Merged
-	  revisions 432239 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-25 20:44 +0000 [r432237]  David M. Lee <dlee@digium.com>
-
-	* res/res_http_websocket.c, /: Increase WebSocket frame size and
-	  improve large read handling Some WebSocket applications, like
-	  [chan_respoke][], require a larger frame size than the default
-	  8k; this patch bumps the default to 16k. This patch also fixes
-	  some problems exacerbated by large frames. The sanity counter was
-	  decremented on every fread attempt in ws_safe_read(), regardless
-	  of whether data was read from the socket or not. For large
-	  frames, this could result in loss of sanity prior to reading the
-	  entire frame. (16k frame / 1448 bytes per segment = 12 segments).
-	  This patch changes the sanity counter so that it only decrements
-	  when fread() doesn't read any bytes. This more closely matches
-	  the original intention of ws_safe_read(), given that the error
-	  message is "Websocket seems unresponsive". This patch also
-	  properly logs EOF conditions, so disconnects are no longer
-	  confused with unresponsive connections. [chan_respoke]:
-	  https://github.com/respoke/chan_respoke Review:
-	  https://reviewboard.asterisk.org/r/4431/ ........ Merged
-	  revisions 432236 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-24 22:14 +0000 [r432195-432199]  Matthew Jordan <mjordan@digium.com>
-
-	* /, channels/chan_sip.c: channels/chan_sip: Fix crash when
-	  transmitting packet after thread shutdown When the monitor thread
-	  is stopped, its pthread ID is set to a specific value
-	  (AST_PTHREADT_STOP) so that later portions of the code can
-	  determine whether or not it is safe to manipulate the thread.
-	  Unfortunately, __sip_reliable_xmit failed to check for that
-	  value, checking instead only for AST_PTHREAD_STOP. Passing the
-	  invalid yet very specific value to pthread_kill causes a crash.
-	  This patch adds a check for AST_PTHREADT_STOP in
-	  __sip_reliable_xmit such that it doesn't attempt to poke the
-	  thread if the thread has already been stopped. ASTERISK-24800
-	  #close Reported by: JoshE ........ Merged revisions 432198 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* channels/chan_pjsip.c, main/channel.c, res/res_pjsip_sdp_rtp.c,
-	  res/ari/resource_channels.c: ARI/PJSIP: Apply requesting
-	  channel's format cap to created channels This patch addresses the
-	  following problems: * ari/resource_channels: In ARI, we currently
-	  create a format capability structure of SLIN and apply it to the
-	  new channel being created. This was originally done when the PBX
-	  core was used to create the channel, as there was a condition
-	  where a newly created channel could be created without any
-	  formats. Unfortunately, now that the Dial API is being used, this
-	  has two drawbacks: (a) SLIN, while it will ensure audio will
-	  flows, can cause a lot of needless transcodings to occur,
-	  particularly when a Local channel is created to the dialplan.
-	  When no format capabilities are available, the Dial API handles
-	  this better by handing all audio formats to the requsted
-	  channels. As such, we defer to that API to provide the format
-	  capabilities. (b) If a channel (requester) is causing this
-	  channel to be created, we currently don't use its format
-	  capabilities as we are passing in our own. However, the Dial API
-	  will use the requester channel's formats if none are passed into
-	  it, and the requester channel exists and has format capabilities.
-	  This is the "best" scenario, as it is the most likely to create a
-	  media path that minimizes transcoding. Fixing this simply entails
-	  removing the providing of the format capabilities structure to
-	  the Dial API. * chan_pjsip: Rather than blindly picking the first
-	  format in the format capability structure - which actually *can*
-	  be a video or text format - we select an audio format, and only
-	  pick the first format if that fails. That minimizes the weird
-	  scenario where we attempt to transcode between video/audio. *
-	  res_pjsip_sdp_rtp: Applied the joint capapbilites to the format
-	  structure. Since ast_request already limits us down to one format
-	  capability once the format capabilities are passed along, there's
-	  no reason to squelch it here. * channel: Fixed a comment. The
-	  reason we have to minimize our requested format capabilities down
-	  to a single format is due to Asterisk's inability to convey the
-	  format to be used back "up" a channel chain. Consider the
-	  following: PJSIP/A => L;1 <=> L;2 => PJSIP/B g,u,a g,u,a g,u,a u
-	  That is, we have PJSIP/A dialing a Local channel, where the
-	  Local;2 dials PJSIP/B. PJSIP/A has native format capabilities
-	  g722,ulaw,alaw; the Local channel has inherited those format
-	  capabilities down the line; PJSIP/B supports only ulaw. According
-	  to these format capabilities, ulaw is acceptable and should be
-	  selected across all the channels, and no transcoding should
-	  occur. However, there is no way to convey this: when L;2 and
-	  PJSIP/B are put into a bridge, we will select ulaw, but that is
-	  not conveyed to PJSIP/A and L;1. Thus, we end up with: PJSIP/A
-	  <=> L;1 <=> L;2 <=> PJSIP/B g g X u u Which causes g722 to be
-	  written to PJSIP/B. Even if we can convey the 'ulaw' choice back
-	  up the chain (which through some severe hacking in Local channels
-	  was accomplished), such that the chain looks like: PJSIP/A <=>
-	  L;1 <=> L;2 <=> PJSIP/B u u u u We have no way to tell PJSIP/A's
-	  *channel driver* to Answer in the SDP back with only 'ulaw'. This
-	  results in all the channel structures being set up correctly, but
-	  PJSIP/A *still* sending g722 and causing the chain to fall apart.
-	  There's a lot of difficulty just in setting this up, as there are
-	  numerous race conditions in the act of bridging, and no clean
-	  mechanism to pass the selected format backwards down an
-	  established channel chain. As such, the best that can be done at
-	  this point in time is clarifying the comment. Review:
-	  https://reviewboard.asterisk.org/r/4434/ ASTERISK-24812 #close
-	  Reported by: Matt Jordan
+	* Update UPGRADE-13.txt file
 
-2015-02-24 18:32 +0000 [r432175]  Kevin Harwell <kharwell@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420608 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* /, bridges/bridge_softmix.c: bridge_softmix: G.729 codec license
-	  held When more than one call using the same codec type enters
-	  into a softmix bridge and no audio is present for a channel the
-	  bridge optimizes the out frame by using the same one for all
-	  channels with the same codec type. Unfortunately, when that
-	  number (channels with same codec type) dropped to <= 1 the codec
-	  was not dereferenced. At least not until all parties left the
-	  bridge. Thus in the case of G.729 the license was not released.
-	  This patch ensures that the codec is dereferenced immediately
-	  when the optimization no longer applies. ASTERISK-24797 #close
-	  Reported by: Luke Hulsey Review:
-	  https://reviewboard.asterisk.org/r/4429/ ........ Merged
-	  revisions 432174 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-21 20:47 +0000 [r432118-432154]  Joshua Colp <jcolp@digium.com>
-
-	* rest-api/api-docs/channels.json, res/ari/resource_channels.c,
-	  res/res_ari_channels.c: res_ari_channels: Return a 404 response
-	  when a requested channel variable does not exist. This change
-	  makes it so that if a channel variable is requested and it does
-	  not exist a 404 response will be returned instead of an
-	  allocation failed response. This makes it easier to debug and
-	  figure out what is going on for a user. ASTERISK-24677 #close
-	  Reported by: Joshua Colp
+2014-08-08 15:08 +0000 [3e452fa4d9]  Jason Parker <jparker@digium.com>
 
-	* res/res_pjsip_registrar.c: res_pjsip_registrar: Add Expires
-	  header to 200 OK if present in REGISTER. Some implementations
-	  don't pay attention to the expires for individual contacts. In
-	  this case they may consider the lack of an Expires header in the
-	  200 OK as unregistered. This change makes it so if an Expires
-	  header is present in the REGISTER we will add one in the 200 OK.
-	  ASTERISK-24785 #close Reported by: Ross Beer
+	* Fix build in devmode.
+	  ........
 
-	* res/res_pjsip.c: res_pjsip: Add a log message when creating a UAC
-	  dialog to a target URI that is invalid. ASTERISK-24499 #close
-	  Reported by: Rusty Newton
+	  Merged revisions 420592 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-2015-02-21 17:35 +0000 [r432099]  Matthew Jordan <mjordan@digium.com>
 
-	* apps/app_voicemail.c, /: apps/app_voicemail: Demote an ERROR
-	  message to a WARNING message When using IMAP voicemail with
-	  FreePBX, you will often get ERROR messages complaining about not
-	  being able to find a mailbox. This is due to how FreePBX handles
-	  voicemail mailboxes. Unfortunately, app_voicemail has to consider
-	  this a configuration error, as in any other system it would be
-	  indicative of someone misconfiguring their system. Regardless, a
-	  misconfiguration is a WARNING, and not an ERROR. This patch
-	  demotes the message so that system administrators can hopefully
-	  reduce some of the noise in their log files. Note that in the
-	  original patch this was made into a NOTICE, but that's a too
-	  forgiving. ASTERISK-24790 #close Reported by: Graham Barnett
-	  patches: app_voicemail.c.patch_noise uploaded by Graham Barnett
-	  (License 6685) ........ Merged revisions 432098 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-21 14:05 +0000 [r432079]  Joshua Colp <jcolp@digium.com>
-
-	* main/http.c, /: http: Add missing html tag to 'httpstatus'
-	  functionality. ASTERISK-24724 #close Reported by: Ashley Sanders
-	  ........ Merged revisions 432078 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-21 02:56 +0000 [r432055-432059]  Corey Farrell <git@cfware.com>
-
-	* /, main/bucket.c, main/codec.c, main/loader.c: Allow shutdown to
-	  unload modules that register bucket scheme's or codec's. * Change
-	  __ast_module_shutdown_ref to be NULL safe (11+). * Allow modules
-	  that call ast_bucket_scheme_register or ast_codec_register to be
-	  unloaded during graceful shutdown only (13+ only). ASTERISK-24796
-	  #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4428/ ........ Merged
-	  revisions 432058 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, include/asterisk/lock.h: asterisk/lock.h: Fix syntax errors
-	  for non-gcc OSX with 64-bit integers. Add a couple of missing
-	  closing brackets / parenthesis. ASTERISK-24814 #close Reported
-	  by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4436/ ........ Merged
-	  revisions 432054 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-20 17:51 +0000 [r432034]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, channels/sig_analog.c: chan_dahdi/sig_analog: Put log message
-	  strings on one line. With the log messages on one line, you can
-	  search for the log message seen in the log and expect to find it.
-	  ........ Merged revisions 432032 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-20 17:46 +0000 [r432033]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_publish_asterisk.c, res/res_pjsip_acl.c:
-	  ASTERISK-24811: Add ast_sorcery_apply_config() to
-	  res_pjsip_publish_asterisk. Matt Hoskins reported that
-	  res_pjsip_publish_asterisk wouldn't pull config from realtime.
-	  Turns out it was just missing a call ast_sorcery_apply_config().
-	  res_pjsip_acl was missing it as well, so I added it. The other
-	  pjsip modules looked OK. ASTERISK-24811 #close Reported-by: Matt
-	  Hoskins Tested-by: George Joseph Tested-by: Matt Hoskins patches:
-	  res_pjsip_publish_asterisk.c.patch submitted by Matt Hoskins
-	  (license 6688) Review: https://reviewboard.asterisk.org/r/4433/
-
-2015-02-20 15:47 +0000 [r432013]  Matthew Jordan <mjordan@digium.com>
-
-	* apps/app_voicemail.c, /: apps/app_voicemail: Fix IMAP header
-	  compatibility issue with Microsoft Exchange When interfacing with
-	  Microsoft Exchange, custom headers will be returned as all lower
-	  case. Currently, the IMAP header code will fail to parse the
-	  returned custom headers, as it will be performing a case
-	  sensitive comparison. This can cause playback of messages to
-	  fail, as needed information - such as origtime - will not be
-	  present. This patch updates app_voicemail's header parsing code
-	  to perform a case insensitive lookup for the requested custom
-	  headers. Since the headers are specific to Asterisk, e.g.,
-	  'x-asterisk-vm-orig-time', and headers should be unique in an
-	  IMAP message, this should cause no issues with other systems.
-	  ASTERISK-24787 #close Reported by: Graham Barnett patches:
-	  app_voicemail.c.patch_MSExchange uploaded by Graham Barnett
-	  (License 6685) ........ Merged revisions 432012 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-19 21:25 +0000 [r431956-431993]  Richard Mudgett <rmudgett@digium.com>
-
-	* channels/chan_dahdi.c, channels/sig_analog.c, /: chan_dahdi:
-	  Remove some dead code. ........ Merged revisions 431992 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* main/aoc.c: ISDN AOC: Fix crash from an AOC-E message that
-	  doesn't have a channel association. Processing an AOC-E event
-	  that does not or no longer has a channel association causes a
-	  crash. The problem with posting AOC events to the channel topic
-	  is that AOC-E events don't always have a channel association and
-	  posting the event to the all channels topic is just wrong. AOC-E
-	  events do however have their own charging association method to
-	  refer to the agreement with the charging entity. * Changed the
-	  AOC events to post to the AMI manager topic instead of the
-	  channel topics. If a channel is associated with the event then
-	  channel snapshot information is supplied with the AMI event. *
-	  Eliminated RAII_VAR() usage in aoc_to_ami() and
-	  ast_aoc_manager_event(). This patch supercedes the patch on
-	  Review: https://reviewboard.asterisk.org/r/4427/ ASTERISK-22670
-	  #close Reported by: klaus3000 ASTERISK-24689 #close Reported by:
-	  Marcel Manz ASTERISK-24740 #close Reported by: Panos Gkikakis
-	  Review: https://reviewboard.asterisk.org/r/4430/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420593 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/res_pjsip_refer.c: res_pjsip_refer: Handle INVITE with
-	  Replaces failure after answer. * Fixed hangup handling of the
-	  session->channel after answer if the ast_channel_move() or
-	  ast_bridge_impart() fails. We are still the thread controlling
-	  the session->channel so we need to call ast_hangup() to kill the
-	  channel. * Fixed debug messages in
-	  refer_incoming_invite_request() referencing incorrect channnels
-	  on success. Code comments now say why the session->channel cannot
-	  be used. Review: https://reviewboard.asterisk.org/r/4422/
-
-2015-02-19 15:28 +0000 [r431937]  Matthew Jordan <mjordan@digium.com>
-
-	* main/tcptls.c, /: tcptls: Handle new OpenSSL compile time option
-	  to disable SSLv3 Some distributions are going to disable SSLv3 at
-	  compile time. This option can be checked using the directive
-	  OPENSSL_NO_SSL3_METHOD. This patch updates the TCP/TLS handling
-	  in Asterisk to look for that directive before attempting to use
-	  the SSLv3 specific methods. ASTERISK-24799 #close Reported by:
-	  Alexander Traud patches: no-ssl3-method.patch uploaded by
-	  Alexander Traud (License 6520) ........ Merged revisions 431936
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-19 02:01 +0000 [r431917]  Corey Farrell <git@cfware.com>
-
-	* main/sched.c, /, include/asterisk/sched.h, channels/chan_iax2.c:
-	  Create work around for scheduler leaks during shutdown. * Added
-	  ast_sched_clean_by_callback for cleanup of scheduled events that
-	  have not yet fired. * Run all pending peercnt_remove_cb and
-	  replace_callno events in chan_iax2. Cleanup of replace_callno
-	  events is only run 11, since it no longer releases any references
-	  or allocations in 13+. ASTERISK-24451 #close Reported by: Corey
-	  Farrell Review: https://reviewboard.asterisk.org/r/4425/ ........
-	  Merged revisions 431916 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-17 15:31 +0000 [r431898]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_sdp_rtp.c, res/res_pjsip_messaging.c,
-	  res/res_pjsip_caller_id.c, res/res_pjsip_refer.c,
-	  res/res_pjsip_send_to_voicemail.c: res_pjsip_refer: Fix crash
-	  from a REFER and BYE collision. Analyzing a one-off crash on a
-	  busy system showed that processing a REFER request had a NULL
-	  session channel pointer. The only way I can think of that could
-	  cause this is if an outgoing BYE transaction overlapped the
-	  incoming REFER transaction in a collision. Asterisk sends a BYE
-	  while the phone sends a REFER to complete an attended transfer. *
-	  Made check the session channel pointer before processing an
-	  incoming REFER request in res_pjsip_refer. * Fixed similar crash
-	  potential for res_pjsip supplement incoming request processing
-	  for res_pjsip_sdp_rtp INFO, res_pjsip_caller_id INVITE/UPDATE,
-	  res_pjsip_messaging MESSAGE, and res_pjsip_send_to_voicemail
-	  REFER messages. * Made res_pjsip_messaging respond to a message
-	  body too large with a 413 instead of ignoring it. ASTERISK-24700
-	  #close Reported by: Zane Conkle Review:
-	  https://reviewboard.asterisk.org/r/4417/
-
-2015-02-16 21:29 +0000 [r431879]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_rtp_asterisk.c: res/res_rtp_asterisk: Fix crash in debug
-	  from RTCP reports without report block When RTCP debugging was
-	  enabled, an RTCP report without a report block would cause a
-	  crash. This was due to the verbose output not checking to see if
-	  the report_block pointer was NULl before dereferencing it. This
-	  patch adds the necessary check to prevent printing any verbose
-	  output if the far side hasn't provided us the information they
-	  should have. ASTERISK-24791 #close Reported by: JoshE Tested by:
-	  JoshE
-
-2015-02-15 19:00 +0000 [r431807-431860]  Joshua Colp <jcolp@digium.com>
-
-	* configs/samples/pjsip.conf.sample: pjsip: Remove "contact" type
-	  from pjsip.conf.sample The "contact" object is not meant to be
-	  configured from the pjsip.conf configuration file. It is meant to
-	  be created as a result of a registration and stored elsewhere.
-	  ASTERISK-24085 #close Reported by: Rusty Newton
-
-	* contrib/scripts/install_prereq: install_prereq: Tweak flags when
-	  configuring pjproject. This change does two things: 1. Disables
-	  debugging so assertions which can return an error do, instead of
-	  asserting. 2. Enables IPv6 support. ASTERISK-24632 #close
-	  Reported by: Rusty Newton
+2014-08-08 14:16 +0000 [5ce4ad8031]  Jason Parker <jparker@digium.com>
 
-	* res/res_sorcery_config.c: res_sorcery_config: Improve object
-	  lookup times. The res_sorcery_config module currently uses a
-	  fixed bucket size of 53. This means that depending on the number
-	  of objects you either end up with excess buckets or a lot of
-	  collisions. Due to the way that res_sorcery_config is implemented
-	  it's actually possible to make the bucket size dynamic based on
-	  the number of objects. This is due to the fact that each loading
-	  of the config file produces a new container and does not modify
-	  the existing one. This change uses the number of expected objects
-	  and finds a prime number near it. In practice depending on the
-	  number of objects this can speed up lookups anywhere from 2X to
-	  15X. This change also removes the lock from the container as it
-	  is not needed. Review: https://reviewboard.asterisk.org/r/4423/
-
-	* res/res_pjsip/pjsip_cli.c: res_pjsip: Add "pjsip show version"
-	  CLI command. When debugging things it can be useful to know
-	  absolutely what version of pjproject res_pjsip is running
-	  against. This change adds a "pjsip show version" CLI command
-	  which can be used to query for this. ASTERISK-24685 #close
-	  Reported by: Joshua Colp Review:
-	  https://reviewboard.asterisk.org/r/4424/
+	* app_voicemail: Add the ability to specify multiple email addresses.
 
-	* res/res_timing_pthread.c: res_timing_pthread: Fix leaky pipes.
-	  During some refactoring the way private information for timers
-	  was stored was changed. As a result of this the action which
-	  normally removed the timer upon closure in res_timing_pthread was
-	  also removed causing the timer to remain after it should using up
-	  resources. This change ensures that the timer is removed upon
-	  closure. ASTERISK-24768 #close Reported by: Matthias Urlichs
-	  patches: timer.patch submitted by Matthias Urlichs (license 5508)
-
-2015-02-15 00:32 +0000 [r431789]  Matthew Jordan <mjordan@digium.com>
-
-	* /, apps/app_mixmonitor.c: apps/app_mixmonitor: Move Test Event
-	  for MIXMONITOR_END to after it finishes The Test Event for
-	  MIXMONITOR_END - which signals that a MixMonitor has completed -
-	  technically fired before the filestream was closed. If a test
-	  used this to trigger a condition to verify that the file was
-	  written, it could result in a race condition where the file size
-	  would not be what the test expected. Luckily, no tests were using
-	  this (although they should have been). Since the test event
-	  needed to be moved after the point where the MixMonitor autochan
-	  has been destroyed, the test event no longer emits the channel
-	  name. Luckily, nothing needs it. ........ Merged revisions 431788
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-14 19:45 +0000 [r431751-431771]  Joshua Colp <jcolp@digium.com>
-
-	* main/sorcery.c: sorcery: Output an error message if a wizard is
-	  specified for an object type and it isn't found. ASTERISK-24612
-	  #close Reported by: Joshua Colp
-
-	* res/res_pjsip_exten_state.c: res_pjsip_exten_state: Improve log
-	  message when a subscription is attempted to a non-existent
-	  extension. ASTERISK-24716 #close Reported by: Rusty Newton
-
-	* channels/pjsip/dialplan_functions.c: 'information' ends with an
-	  'n'.
-
-	* channels/pjsip/dialplan_functions.c: chan_pjsip: Fix crash when
-	  CHANNEL dialplan function is invoked with pjsip argument and no
-	  type. ASTERISK-24771 #close Reported by: Niklas Larsson
-
-2015-02-13 17:21 +0000 [r431734]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_session.c: res_pjsip_session: Fix double re-INVITE
-	  collision crash. A multi-asterisk box setup with direct media
-	  enabled would occasionally crash when two re-INVITE collisions on
-	  a call leg happen in a row. The re-INVITE logic only had one
-	  timer struct to defer the re-INVITE. When the second collision
-	  happens the timer struct is overwritten and put into the timer
-	  heap again. Resources for the first timer are leaked and the heap
-	  has two positions occupied by the same timer struct. Now the heap
-	  ordering is potentially corrupted, the timer will fire twice, and
-	  any resources allocated for the second timer will be released
-	  twice. * The solution is to put the collided re-INVITE into the
-	  delayed requests queue with all the other delayed requests and
-	  cherry pick the next request that can come off the queue when an
-	  event happens. * Changed to put delayed BYE requests at the head
-	  of the delayed queue. There is no sense in processing delayed
-	  UPDATEs and re-INVITEs when a BYE has been requested. * Made the
-	  start of a BYE request flush the delayed requests queue to
-	  prevent a delayed request from overlapping the BYE transaction. I
-	  saw a few cases where a delayed re-INVITE got started after the
-	  BYE transaction started. * Changed the delayed_request struct to
-	  use an enum instead of a string for the request method. Cherry
-	  picking the queue is easier with an enum than string comparisons
-	  and the compiler can warn if a switch statement does not cover
-	  all defined enum values. * Improved the debug output to give more
-	  information. It helps to know which channel is involved with an
-	  endpoint. Trunks can have many channels associated with the
-	  endpoint at the same time. ASTERISK-24727 #close Reported by:
-	  Mark Michelson Review: https://reviewboard.asterisk.org/r/4414/
-
-2015-02-12 20:32 +0000 [r431717]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_pjsip_multihomed.c, res/stasis/control.c,
-	  include/asterisk/stasis_app.h, rest-api/api-docs/channels.json,
-	  res/ari/resource_channels.c, CHANGES, res/res_ari_channels.c,
-	  channels/chan_pjsip.c, res/res_pjsip_nat.c,
-	  res/res_pjsip_transport_websocket.c, res/ari/resource_channels.h:
-	  ARI/PJSIP: Add the ability to redirect (transfer) a channel in a
-	  Stasis app This patch adds a new feature to ARI to redirect a
-	  channel to another server, and fixes a few bugs in PJSIP's
-	  handling of the Transfer dialplan application/ARI redirect
-	  capability. *New Feature* A new operation has been added to the
-	  ARI channels resource, redirect. With this, a channel in a Stasis
-	  application can be redirected to another endpoint of the same
-	  underlying channel technology. *Bug fixes* In the process of
-	  writing this new feature, two bugs were fixed in the PJSIP stack:
-	  (1) The existing .transfer channel callback had the limitation
-	  that it could only transfer channels to a SIP URI, i.e., you had
-	  to pass 'PJSIP/sip:foo@my_provider.com' to the dialplan
-	  application. While this is still supported, it is somewhat
-	  unintuitive - particularly in a world full of endpoints. As such,
-	  we now also support specifying the PJSIP endpoint to transfer to.
-	  (2) res_pjsip_multihomed was, unfortunately, trying to 'help' a
-	  302 redirect by updating its Contact header. Alas, that resulted
-	  in the forwarding destination set by the dialplan application/ARI
-	  resource/whatever being rewritten with very incorrect
-	  information. Hence, we now don't bother updating an outgoing
-	  response if it is a 302. Since this took a looong time to find,
-	  some additional debug statements have been added to those modules
-	  that update the Contact headers. Review:
-	  https://reviewboard.asterisk.org/r/4316/ ASTERISK-24015 #close
-	  Reported by: Private Name ASTERISK-24703 #close Reported by: Matt
-	  Jordan
-
-2015-02-11 18:02 +0000 [r431693-431698]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip/pjsip_configuration.c: res_pjsip: dtls_handler
-	  causes Asterisk to crash There have been a couple of times where
-	  a crash occurred in the dtls_handler section of the code for
-	  res_pjsip. Unfortunately, in working this issue the problem was
-	  unable to be reproduced. After looking at the backtraces and
-	  through the code the current best guess as to why this happened
-	  might be due to a reentrance problem and the strtok function. So,
-	  the current fix is to convert the strtok function into the
-	  reentrant version of the function, strtok_r. ASTERISK-24741
-	  #close Reported by: Zane Conkle Review:
-	  https://reviewboard.asterisk.org/r/4409/
-
-	* res/ari/ari_websockets.c: ari_websockets: removed extra check on
-	  websocket session read When merging the websocket timeout issue
-	  (ASTERISK-24701) an extra, almost duplicate, check was left in
-	  the code that should not have been. This removes it.
-	  ASTERISK-24701 #close Reported by: Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4412/
-
-2015-02-11 17:28 +0000 [r431692]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/bridge.c, main/http.c, apps/app_confbridge.c,
-	  include/asterisk/channel.h, res/res_pjsip/pjsip_options.c,
-	  res/res_pjsip_pubsub.c, main/asterisk.c, main/channel.c,
-	  include/asterisk.h, channels/chan_sip.c: HTTP: Stop accepting
-	  requests on final system shutdown. There are three CLI commands
-	  to stop and restart Asterisk each. 1) core stop/restart now -
-	  Hangup all calls and stop or restart Asterisk. New channels are
-	  prevented while the shutdown request is pending. 2) core
-	  stop/restart gracefully - Stop or restart Asterisk when there are
-	  no calls remaining in the system. New channels are prevented
-	  while the shutdown request is pending. 3) core stop/restart when
-	  convenient - Stop or restart Asterisk when there are no calls in
-	  the system. New calls are not prevented while the shutdown
-	  request is pending. ARI has made stopping/restarting Asterisk
-	  more problematic. While a shutdown request is pending it is
-	  desirable to continue to process ARI HTTP requests for current
-	  calls. To handle the current calls while a shutdown request is
-	  pending, a new committed to shutdown phase is needed so ARI
-	  applications can deal with the calls until the system is fully
-	  committed to shutdown. * Added a new shutdown committed phase so
-	  ARI applications can deal with calls until the final committed to
-	  shutdown phase is reached. * Made refuse new HTTP requests when
-	  the system has reached the final system shutdown phase. Starting
-	  anything while the system is actively releasing resources and
-	  unloading modules is not a good thing. * Split the bridging
-	  framework shutdown to not cleanup the global bridging containers
-	  when shutting down in a hurry. This is similar to how other
-	  modules prevent crashes on rapid system shutdown. * Moved
-	  ast_begin_shutdown(), ast_cancel_shutdown(), and
-	  ast_shutting_down(). You should not have to include channel.h
-	  just to access these system functions. ASTERISK-24752 #close
-	  Reported by: Matthew Jordan Review:
-	  https://reviewboard.asterisk.org/r/4399/
-
-2015-02-11 17:12 +0000 [r431674]  Matthew Jordan <mjordan@digium.com>
-
-	* /, channels/chan_sip.c: channels/chan_sip: Fix RealTime error
-	  during SIP unregistration with MariaDB When a SIP device that has
-	  its registration stored in RealTime unregisters, the entry for
-	  that device is updated with blank values, i.e., "", indicating
-	  that it is no longer registered. Unfortunately, one of those
-	  values that is 'blanked' is the device's port. If the column type
-	  for the port is not a string datatype (the recommended type is
-	  integer), an ODBC or database error will be thrown. MariaDB does
-	  not coerce empty strings to a valid integer value. This patch
-	  updates the query run from chan_sip such that it replaces the
-	  port value with a value of '0', as opposed to a blank value. This
-	  is the value that other database backends coerce the empty string
-	  ("") to already, and the handling of reading a RealTime
-	  registration value from a backend already anticipates receiving a
-	  port of '0' from the backends. ASTERISK-24772 #close Reported by:
-	  Richard Miller patches: chan_sip.diff uploaded by Richard Miller
-	  (License 5685) ........ Merged revisions 431673 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-11 16:51 +0000 [r431670]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_http_websocket.c, res/ari/ari_websockets.c, /:
-	  res_http_websocket: websocket write timeout fails to fully
-	  disconnect When writing to a websocket if a timeout occurred the
-	  underlying socket did not get closed/disconnected. This patch
-	  makes sure the websocket gets disconnected on a write timeout.
-	  Also a notice is logged stating that the websocket was
-	  disconnected. ASTERISK-24701 #close Reported by: Matt Jordan
-	  Review: https://reviewboard.asterisk.org/r/4412/ ........ Merged
-	  revisions 431669 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-11 15:51 +0000 [r431663]  Corey Farrell <git@cfware.com>
-
-	* include/asterisk/module.h, main/loader.c, /,
-	  bridges/bridge_builtin_features.c: Enable REF_DEBUG for
-	  ast_module_ref / ast_module_unref. Add ast_module_shutdown_ref
-	  for use by modules that can only be unloaded during graceful
-	  shutdown. When REF_DEBUG is enabled: * Add an empty ao2 object to
-	  struct ast_module. * Allocate ao2 object when the module is
-	  loaded. * Perform an ao2_ref in each place where mod->usecount is
-	  manipulated. * ao2_cleanup on module unload. ASTERISK-24479
-	  #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4141/ ........ Merged
-	  revisions 431662 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-02-10 23:16 +0000 [r431643]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_config_wizard.c,
-	  configs/samples/pjsip_wizard.conf.sample:
-	  res_pjsip_config_wizard: Add ability to auto-create hints.
-	  Looking at the Super Awesome Company sample reminded me that
-	  creating hints is just plain gruntwork. So you can now have the
-	  pjsip conifg wizard auto-create them for you. Specifying
-	  'hint_exten' in the wizard will create 'exten =>
-	  <hint_exten>,hint/PJSIP/<wizard_id>' in whatever is specified for
-	  'hint_context'. Specifying 'hint_application' in the wizard will
-	  create 'exten => <hint_exten>,1,<hint_application>' in whatever
-	  is specified for 'hint_context'. The default for 'hint_context'
-	  is the endpoint's context. There's no default for
-	  'hint_application'. If not specified, no app is added. There's no
-	  default for 'hint_exten'. If not specified, neither the hint
-	  itself nor the application will be created. Some may think this
-	  is the slippery slope to users.conf but hints are a basic
-	  necessity for phones unlike voicemail, manager, etc that
-	  users.conf creates. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4383/
-
-2015-02-09 03:10 +0000 [r431600-431622]  Matthew Jordan <mjordan@digium.com>
-
-	* rest-api/api-docs/channels.json, res/ari/resource_channels.c:
-	  res/ari/resource_channels: Add missing 'no_answer' reason to
-	  DELETE /channels One of the canonical reasons for hanging up a
-	  channel is because the far end failed to answer - or because
-	  someone else answered, and we want to get rid of this channel.
-	  This patch adds the missing value to the 'reason' query parameter
-	  for the DELETE /channels operation. Review:
-	  https://reviewboard.asterisk.org/r/4400 ASTERISK-24745 #close
-	  Reported by: Ben Merrills patches:
-	  add_no_answer_ari_hangup_cause.diff uploaded by Ben Merrills
-	  (License 6678)
-
-	* /, res/res_odbc.c: res/res_odbc: Remove unneeded queries when
-	  determining if a table exists This patch modifies the
-	  ast_odbc_find_table function such that it only performs a lookup
-	  of the requested table if the table is not already known. Prior
-	  to this patch, a queries would be executed against the database
-	  even if the table was already known and cached. Review:
-	  https://reviewboard.asterisk.org/r/4405/ ASTERISK-24742 #close
-	  Reported by: ibercom patches: patch.diff uploaded by ibercom
-	  (License 6599) ........ Merged revisions 431617 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* res/res_pjsip_sdp_rtp.c: res/res_pjsip_sdp_rtp: Fix leak of local
-	  ICE candidates when applying to SDP When an SDP is created for an
-	  outgoing request/response, the ICE candidates obtained from the
-	  RTP instance are currently leaked. This causes the ao2 container
-	  that holds the candidates to never properly be reclaimed when the
-	  RTP instance is destroyed. This patch properly decrements the ICE
-	  candidates' container if it is successfully obtained.
-	  ASTERISK-24769 #close Reported by: Matt Jordan
-
-2015-02-06 21:26 +0000 [r431583]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* main/utils.c, res/res_pjsip.c, main/config.c: various: cleanup
-	  issues found during leak hunt In this collection of small patches
-	  to prevent Valgrind errors are: fixes for reference leaks in
-	  config hooks, evaluating a parameter beyond bounds, and accessing
-	  a structure after a lock where it could have been already free'd.
-	  Review: https://reviewboard.asterisk.org/r/4407/
+	  ASTERISK-24045
+	  Reported by: Jacob Barber
+	  Review: https://reviewboard.asterisk.org/r/3833/
+	  ........
 
-2015-02-04 01:27 +0000 [r431538-431555]  Joshua Colp <jcolp@digium.com>
+	  Merged revisions 420577 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_pjsip_keepalive.c: res_pjsip_keepalive: Don't crash if
-	  PJSIP module is not loaded.
 
-	* main/sorcery.c: sorcery: Don't try to load object types which
-	  haven't been defined. The act of defining wizards for an object
-	  type in sorcery.conf will create a minimal object type. This can
-	  cause a problem when a module has multiple sorcery instances
-	  (which all get the wizards from sorcery.conf applied) but the
-	  sorcery instances do not all contain full information about the
-	  object types. Upon loading errors will occur stating that the
-	  objects can not be created. This is confusing and is actually
-	  perfectly fine. This change makes it so that only object types
-	  which have been fully defined will be loaded. ASTERISK-24748
-	  #close Reported by: Joshua Colp
-
-2015-01-31 16:27 +0000 [r431521]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_format_attr_h264.c: res_format_attr_h264: Fix crash when
-	  determining joint capability. The res_format_attr_h264 module
-	  currently incorrectly attempts to copy SPS and PPS information
-	  from the wrong attribute. This change fixes that. ASTERISK-24616
-	  #close Reported by: Yura Kocyuba Review:
-	  https://reviewboard.asterisk.org/r/4392/
-
-2015-02-06  Asterisk Development Team <asteriskteam@digium.com>
-
-	* Asterisk 13.2.0 Released.
-
-2015-01-30  Asterisk Development Team <asteriskteam@digium.com>
-
-	* Asterisk 13.2.0-rc1 Released.
-
-2015-01-30 17:44 +0000 [r431492]  Richard Mudgett <rmudgett@digium.com>
-
-	* apps/app_agent_pool.c: app_agent_pool: Fix initial module load
-	  agent device state reporting. When the app_agent_pool module
-	  initially loads there is a race condition between the thread
-	  loading agents.conf and the device state internal processing
-	  thread. If the device state internal processing thread handles
-	  the agent creation state updates before the thread that loaded
-	  agents.conf registers the device state provider callback then the
-	  cached agent state is "Invalid". When a consumer module like
-	  app_queue asks for the agent state it gets the cached "Invalid"
-	  state instead of the real state from the provider. * Moved
-	  loading the agents.conf configuration to the last thing setup by
-	  app_agent_pool in load_module(). Now the device state provider
-	  callback is registered before the config is loaded so the agent
-	  creation state updates are guaranteed to get the initial device
-	  state. * Removed some now redundant config cleanup on error in
-	  load_config(). * Added lock protection when accessing the device
-	  state in agent_pvt_devstate_get() and eliminated the RAII_VAR()
-	  usage. ASTERISK-24737 #close Reported by: Steve Pitts Review:
-	  https://reviewboard.asterisk.org/r/4390/
-
-2015-01-30 17:38 +0000 [r431490]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip_outbound_publish.c: res_pjsip_outbound_publish:
-	  eventually crashes when no response is ever received When
-	  Asterisk attempts to send SIP outbound publish information and no
-	  response is ever received (no 200 okay, 412, 423) the system
-	  eventually crashes. A response is never received because the
-	  system Asterisk is attempting to send publish information to is
-	  not available. The underlying pjsip framework attempts to send
-	  publish information. After several attempts it calls back into
-	  the Asterisk outbound publish code. At this point if the
-	  "client->queue" is empty Asterisk attempts to schedule a refresh
-	  which utilizes "rdata" and since no response was received the
-	  given "rdata" struture is NULL. Attempting to dereference a NULL
-	  object of course results in a crash. The fix here removes the
-	  dependency on rdata for schedule_publish_refresh. Instead
-	  param->expiration is now passed to it as this is set to -1 if no
-	  response is received. Also added a notification when no response
-	  is received. ASTERISK-24635 #close Reported by: Marco Paland
-	  Review: https://reviewboard.asterisk.org/r/4384/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420578 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2015-01-30 16:52 +0000 [r431471]  asanders <asanders@localhost>:
+2014-08-08 12:53 +0000 [91f7b66183]  Matt Jordan <mjordan@digium.com>
 
-	* include/asterisk/http.h, configs/samples/http.conf.sample,
-	  main/http.c: HTTP: For httpd server, need option to define server
-	  name for security purposes Added a new config property
-	  [servername] to the http.conf file; updated the http server to
-	  use the new property when sending responses, for showing http
-	  status through the CLI and when reporting status through the
-	  'httpstatus' webpage. ASTERISK-24316 #close Reported By: Andrew
-	  Nagy Review: https://reviewboard.asterisk.org/r/4374/
-
-2015-01-30 16:47 +0000 [r431468]  Mark Michelson <mmichelson@digium.com>
-
-	* main/stasis_channels.c, channels/chan_pjsip.c, main/xmldoc.c,
-	  res/res_pjsip_refer.c, main/pbx.c, main/manager.c,
-	  pbx/pbx_spool.c, main/bridge_after.c: Fix some memory leaks.
-	  These memory leaks were found and fixed by John Hardin. I'm just
-	  committing them for him. ASTERISK-24736 #close Reported by Mark
-	  Michelson Review: https://reviewboard.asterisk.org/r/4389
+	* chan_sip: Mark chan_sip and its files as extended support
+	  ........
 
-2015-01-29 23:02 +0000 [r431450]  Scott Griepentrog <sgriepentrog@digium.com>
+	  Merged revisions 420562 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* include/asterisk/bridge.h, main/bridge.c,
-	  res/stasis/stasis_bridge.c: stasis transfer: fix stasis bridge
-	  push race part two When swapping a Local channel in place of one
-	  already in a bridge (to complete a bridge attended transfer), the
-	  channel that was swapped out can actually be hung up before the
-	  stasis bridge push callback executes on the independant transfer
-	  thread. This results in the stasis app loop dropping out and
-	  removing the control that has the the app name which the local
-	  replacement channel needs so it can re-enter stasis. To avoid
-	  this race condition a new push_peek callback has been added, and
-	  called from the ast_bridge_impart thread before it launches the
-	  independant thread that will complete the transfer. Now the
-	  stasis push_peek callback can copy the stasis app name before the
-	  swap channel can hang up. ASTERISK-24649 Review:
-	  https://reviewboard.asterisk.org/r/4382/
-
-2015-01-29 20:58 +0000 [r431420-431426]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip.c, res/res_pjsip_sips_contact.c (added): Use SIPS
-	  URIs in Contact headers when appropriate. RFC 3261 sections
-	  8.1.1.8 and 12.1.1 dictate specific scenarios when we are
-	  required to use SIPS URIs in Contact headers. Asterisk's
-	  non-compliance with this could actually cause calls to get
-	  dropped when communicating with clients that are strict about
-	  checking the Contact header. Both of the SIP stacks in Asterisk
-	  suffered from this issue. This changeset corrects the behavior in
-	  res_pjsip/chan_pjsip.c Review:
-	  https://reviewboard.asterisk.org/r/4345
-
-	* /, channels/chan_sip.c: Use SIPS URIs in Contact headers when
-	  appropriate. RFC 3261 sections 8.1.1.8 and 12.1.1 dictate
-	  specific scenarios when we are required to use SIPS URIs in
-	  Contact headers. Asterisk's non-compliance with this could
-	  actually cause calls to get dropped when communicating with
-	  clients that are strict about checking the Contact header. Both
-	  of the SIP stacks in Asterisk suffered from this issue. This
-	  changeset corrects the behavior in chan_sip. ASTERISK-24646
-	  #close Reported by Stephan Eisvogel Review:
-	  https://reviewboard.asterisk.org/r/4346 ........ Merged revisions
-	  431423 from http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* res/res_pjsip/pjsip_configuration.c: Allow disabling of 100rel
-	  support on PJSIP endpoints. Due to an inversion error, setting
-	  100rel=no would not actually change the current value of the
-	  setting (which defaulted to "yes"). With this fix, the inversion
-	  is corrected.
-
-2015-01-29 16:46 +0000 [r431403]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_exten_state.c: res_pjsip_exten_state: Reduce log
-	  clutter... change a WARNING to a VERBOSE/2 Reduce log clutter by
-	  changing the "Watcher for hint %s (removed|deactivated)" message
-	  from WARNING to VERBOSE/2. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4387/
-
-2015-01-29 12:09 +0000 [r431385]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_rtp_asterisk.c, /: res_rtp_asterisk: Fix DTLS when used
-	  with OpenSSL 1.0.1k A recent security fix for OpenSSL broke DTLS
-	  negotiation for many applications. This was caused by read ahead
-	  not being enabled when it should be. While a commit has gone into
-	  OpenSSL to force read ahead on for DTLS it may take some time for
-	  a release to be made and the change to be present in
-	  distributions (if at all). As enabling read ahead is a simple one
-	  line change this commit does that and fixes the issue.
-	  ASTERISK-24711 #close Reported by: Jared Biel ........ Merged
-	  revisions 431384 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-28 17:37 +0000 [r431301-431303]  Mark Michelson <mmichelson@digium.com>
-
-	* /, res/res_pjsip_sdp_rtp.c, res/res_pjsip_t38.c,
-	  res/res_pjsip_session.c: Fix file descriptor leak in RTP code.
-	  SIP requests that offered codecs incompatible with configured
-	  values could result in the allocation of RTP and RTCP ports that
-	  would not get reclaimed later. ASTERISK-24666 #close Reported by
-	  Y Ateya Review: https://reviewboard.asterisk.org/r/4323
-	  AST-2015-001 ........ Merged revisions 431300 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* funcs/func_curl.c, /: Multiple revisions 431297-431298 ........
-	  r431297 | mmichelson | 2015-01-28 11:05:26 -0600 (Wed, 28 Jan
-	  2015) | 17 lines Mitigate possible HTTP injection attacks using
-	  CURL() function in Asterisk. CVE-2014-8150 disclosed a
-	  vulnerability in libcURL where HTTP request injection can be
-	  performed given properly-crafted URLs. Since Asterisk makes use
-	  of libcURL, and it is possible that users of Asterisk may get
-	  cURL URLs from user input or remote sources, we have made a patch
-	  to Asterisk to prevent such HTTP injection attacks from
-	  originating from Asterisk. ASTERISK-24676 #close Reported by Matt
-	  Jordan Review: https://reviewboard.asterisk.org/r/4364
-	  AST-2015-002 ........ r431298 | mmichelson | 2015-01-28 11:12:49
-	  -0600 (Wed, 28 Jan 2015) | 3 lines Fix compilation error from
-	  previous patch. ........ Merged revisions 431297-431298 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 431299 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2015-01-28 12:18 +0000 [r431267]  Sean Bright <sean@malleable.com>
-
-	* res/res_format_attr_silk.c, res/res_format_attr_opus.c: media
-	  formats: update res_format_attr_opus & silk In r419044, we
-	  changed how formats were handled, but the return value of the
-	  format_parse_sdp_fmtp functions in res_format_attr_opus and
-	  res_format_attr_silk were not updated, causing calls to fail. Ran
-	  into this when getting codec_opus working with Asterisk 13. Once
-	  the return value was corrected, we were crashing in opus_getjoint
-	  because of NULL format attributes. I've fixed this as well in
-	  this patch. Review: https://reviewboard.asterisk.org/r/4371/
-
-2015-01-28 04:09 +0000 [r431243]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/sorcery.c, res/res_pjsip_outbound_registration.c,
-	  res/res_pjsip.c: res_pjsip_outbound_registration: Fix reload race
-	  condition. Performing a CLI "module reload" command when there
-	  are new pjsip.conf registration objects defined frequently failed
-	  to load them correctly. What happens is a race condition between
-	  res_pjsip pushing its reload into an asynchronous task processor
-	  task and the thread that does the rest of the reloads when it
-	  gets to reloading the res_pjsip_outbound_registration module. A
-	  similar race condition happens between a reload and the CLI/AMI
-	  show registrations commands. The reload updates the
-	  current_states container and the CLI/AMI commands call
-	  get_registrations() which builds a new current_states container.
-	  * Made res_pjsip.c reload_module() use
-	  ast_sip_push_task_synchronous() instead of ast_sip_push_task() to
-	  eliminate two threads processing config reloads at the same time.
-	  * Made get_registrations() not replace the global current_states
-	  container so the CLI/AMI show registrations command cannot
-	  interfere with reloading. You could never add/remove objects in
-	  the container without the possibility of the container being
-	  replaced out from under you by get_registrations(). * Added a
-	  registration loaded sorcery instance observer to purge any dead
-	  registration objects since get_registrations() cannot do this job
-	  anymore. The struct ast_sorcery_instance_observer callbacks must
-	  be used because the callback happens inline with the load
-	  process. The struct ast_sorcery_observer callbacks are pushed to
-	  a different thread. * Added some global current_states NULL
-	  pointer checks in case the container disappears because of
-	  unload_module(). * Made sorcery's struct
-	  ast_sorcery_instance_observer.object_type_loaded callbacks
-	  guaranteed to be called before any struct
-	  ast_sorcery_observer.loaded callbacks will be called. * Moved the
-	  check for non-reloadable objects to before the sorcery instance
-	  loading callbacks happen to short circuit unnecessary work.
-	  Previously with non-reloadable objects, the sorcery instance
-	  loading/loaded callbacks would always happen, the individual
-	  wizard loading/loaded would be prevented, and the non-reloadable
-	  type logging message would be logged for each associated wizard.
-	  ASTERISK-24729 #close Review:
-	  https://reviewboard.asterisk.org/r/4381/
-
-2015-01-27 22:56 +0000 [r431179-431219]  Kevin Harwell <kharwell@digium.com>
-
-	* /, main/tcptls.c: tcptls: Bad file descriptor error when
-	  reloading chan_sip While running through some scenarios using
-	  chan_sip and tcp a problem would occur that resulted in a flood
-	  of bad file descriptor messages on the cli: tcptls.c:712
-	  ast_tcptls_server_root: Accept failed: Bad file descriptor The
-	  message is received because the underlying socket has been
-	  closed, so is valid. This is probably happening because unloading
-	  of chan_sip is not atomic. That however is outside the scope of
-	  this patch. This patch simply stops the logging of multiple
-	  occurrences of that message. ASTERISK-24728 #close Reported by:
-	  Thomas Thompson Review: https://reviewboard.asterisk.org/r/4380/
-	  ........ Merged revisions 431218 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, channels/chan_sip.c: chan_sip: stale nonce causes failure When
-	  refreshing (with a small expiration) a registration that was sent
-	  to chan_sip the nonce would be considered stale and reject the
-	  registration. What was happening was that the initial
-	  registration's "dialog" still existed in the dialogs container
-	  and upon refresh the dialog match algorithm would choose that as
-	  the "dialog" instead of the newly created one. This occurred
-	  because the algorithm did not check to see if the from tag
-	  matched if authentication info was available after the 401. So,
-	  it ended up assuming the original "dialog" was a match and
-	  stopped the search. The old "dialog" of course had an old nonce,
-	  thus the stale nonce message. This fix attempts to leave the
-	  original functionality alone except in the case of a REGISTER. If
-	  a REGISTER is received if searches for an existing "dialog"
-	  matching only on the callid. If the expires value is low enough
-	  it will reuse dialog that is there, otherwise it will create a
-	  new one. ASTERISK-24715 #close Reported by: John Bigelow Review:
-	  https://reviewboard.asterisk.org/r/4367/ ........ Merged
-	  revisions 431187 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* res/res_pjsip/pjsip_outbound_auth.c, res/res_pjsip/config_auth.c,
-	  main/stasis_message_router.c, res/res_pjsip/location.c,
-	  res/res_pjsip/pjsip_configuration.c,
-	  res/res_pjsip/pjsip_distributor.c,
-	  res/res_pjsip/include/res_pjsip_private.h,
-	  res/res_pjsip/pjsip_global_headers.c,
-	  res/res_pjsip/pjsip_options.c, res/res_pjsip.c,
-	  res/res_pjsip/config_transport.c: res_pjsip: make it unloadable
-	  (take 2) Due to the original patch causing memory corruptions it
-	  was removed until the problem could be resolved. This patch is
-	  the original patch plus some added locking around stasis router
-	  subcription that was needed to avoid the memory corruption.
-	  Description of the original problem and patch (still applicable):
-	  The res_pjsip module was previously unloadable. With this patch
-	  it can now be unloaded. This patch is based off the original
-	  patch on the issue (listed below) by Corey Farrell with a few
-	  modifications. Namely, removed a few changes not required to make
-	  the module unloadable and also fixed a bug that would cause
-	  asterisk to crash on unloading. This patch is the first step
-	  (should hopefully be followed by another/others at some point) in
-	  allowing res_pjsip and the modules that depend on it to be
-	  unloadable. At this time, res_pjsip and some of the modules that
-	  depend on res_pjsip cannot be unloaded without causing problems
-	  of some sort. The goal of this patch is to get res_pjsip and only
-	  res_pjsip to be able to unload successfully and/or shutdown
-	  without incident (crashes, leaks, etc...). Other dependent
-	  modules may still cause problems on unload. Basically made sure,
-	  with the patch applied, that res_pjsip (with no other dependent
-	  modules loaded) could be succesfully unloaded and Asterisk could
-	  shutdown without any leaks or crashes that pertained directly to
-	  res_pjsip. ASTERISK-24485 #close Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4363/ patches:
-	  pjsip_unload-broken-r1.patch submitted by Corey Farrell (license
-	  5909)
-
-2015-01-27 17:36 +0000 [r431160]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, apps/confbridge/include/confbridge.h, apps/app_confbridge.c:
-	  app_confbridge: Repeatedly starting and stopping recording ref
-	  leaks the recording channel. Starting and stopping conference
-	  recording more than once causes the recording channels to be
-	  leaked. For v13 the channels also show up in the CLI "core show
-	  channels" output. * Reworked and simplified the recording channel
-	  code to use ast_bridge_impart() instead of managing the recording
-	  thread in the ConfBridge code. The recording channel's ref
-	  handling easily falls into place and other off nominal code paths
-	  get handled better as a result. ASTERISK-24719 #close Reported
-	  by: John Bigelow Review: https://reviewboard.asterisk.org/r/4368/
-	  Review: https://reviewboard.asterisk.org/r/4369/ ........ Merged
-	  revisions 431135 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-27 17:32 +0000 [r431157]  Joshua Colp <jcolp@digium.com>
-
-	* main/bridge_channel.c, res/res_pjsip_sdp_rtp.c: bridge /
-	  res_pjsip_sdp_rtp: Fix issues with media not being reinvited
-	  during direct media. This change fixes two issues: 1. During a
-	  swap operation bridging added the new channel before having the
-	  swap channel leave. This was not handled in bridge_native_rtp and
-	  could result in a channel not getting reinvited back to Asterisk.
-	  After this change the swap channel will leave first and the new
-	  channel will then join. 2. If a re-invite was received after a
-	  session had been established any upstream elements (such as
-	  bridge_native_rtp) were not notified that they may want to
-	  re-evaluate things. After this change an UPDATE_RTP_PEER control
-	  frame is queued when this situation occurs and upstream can
-	  react. AST-1524 #close Review:
-	  https://reviewboard.asterisk.org/r/4378/
-
-2015-01-27 17:22 +0000 [r431153]  Jonathan Rose <jrose@digium.com>
-
-	* main/manager.c: Manager: Fix Manager Action ModuleLoad to give
-	  correct response when reloading Prior to this patch, ModuleLoad
-	  would respond with an error indicating that the requested module
-	  wasn't found in spite of finding and reloading the module.
-	  Review: https://reviewboard.asterisk.org/r/4373/ ASTERISK-24721
-	  #close
-
-2015-01-27 17:20 +0000 [r431134-431145]  Matthew Jordan <mjordan@digium.com>
-
-	* res/ari/resource_bridges.c,
-	  rest-api-templates/asterisk_processor.py,
-	  res/ari/resource_channels.h, res/res_ari_bridges.c,
-	  res/ari/resource_bridges.h, rest-api-templates/api.wiki.mustache,
-	  rest-api/api-docs/channels.json,
-	  rest-api-templates/swagger_model.py,
-	  rest-api/api-docs/bridges.json: ARI: Improve wiki documentation
-	  This patch improves the documentation of ARI on the wiki.
-	  Specifically, it addresses the following: * Allowed values and
-	  allowed ranges weren't documented. This was particularly
-	  frustrating, as Asterisk would reject query parameters with
-	  disallowed values - but we didn't tell anyone what the allowed
-	  values were. * The /play/id operation on /channels and /bridges
-	  failed to document all of the added media resource types. *
-	  Documentation for creating a channel into a Stasis application
-	  failed to note when it occurred, and that creating a channel into
-	  Stasis conflicts with creating a channel into the dialplan. *
-	  Some other minor tweaks in the mustache templates, including
-	  italicizing the parameter type, putting the default value on its
-	  own sub-bullet, and some other nicities. Review:
-	  https://reviewboard.asterisk.org/r/4351
-
-	* apps/confbridge/conf_config_parser.c,
-	  apps/confbridge/include/confbridge.h: app_confbridge: Restore
-	  user's menu name to CLI output of 'confbridge list' When issuing
-	  a 'confbridge list XXXX' CLI command, the resulting output no
-	  longer displays the menu associated with a ConfBridge
-	  participant. The issue was caused by ASTERISK-22760. When that
-	  patch was done, it removed the copying of the menu name
-	  associated with the user from the actual user profile. This patch
-	  fixes the issue by copying the menu name over to the user profile
-	  when the menu hooks are applied to the user. Since that function
-	  now does a little bit more than just apply the hooks, the name of
-	  the function has been changed to cover the copying of the menu
-	  name over as well. In addition, there is a disparity between the
-	  menu name length as it is stored on the conf_menu structure and
-	  the confbridge_user structure; this patch makes the lengths match
-	  so that a strcpy can be used. Review:
-	  https://reviewboard.asterisk.org/r/4372/ ASTERISK-24723 #close
-	  Reported by: Steve Pitts
 
-2015-01-27 11:47 +0000 [r431114]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420563 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/parking/parking_manager.c: res_parking: Fix crash due to race
-	  condition when unloading. There is currently a race condition
-	  when unloading the res_parking module. Depending on the will of
-	  the universe the subscription invocation may occur AFTER the
-	  module is unloaded. This is because the module does NOT use
-	  stasis_unsubscribe_and_join when terminating the subscription. It
-	  merely uses stasis_unsubscribe. This change makes it use
-	  stasis_unsubscribe_and_join which is documented for usage in this
-	  exact scenario. AST-1520 #close Review:
-	  https://reviewboard.asterisk.org/r/4375/
-
-2015-01-26 14:49 +0000 [r431092]  David M. Lee <dlee@digium.com>
-
-	* channels/sip/include/route.h, funcs/func_presencestate.c,
-	  main/rtp_engine.c, configure, include/asterisk/autoconfig.h.in,
-	  include/asterisk/sem.h, configure.ac, main/app.c,
-	  main/bridge_channel.c, main/sem.c, res/res_timing_kqueue.c,
-	  main/asterisk.c: Various fixes for OS X This patch addresses
-	  compilation errors on OS X. It's been a while, so there's quite a
-	  few things. * Fixed __attribute__ decls in route.h to be
-	  portable. * Fixed htonll and ntohll to work when they are defined
-	  as macros. * Replaced sem_t usage with our ast_sem wrapper. *
-	  Added ast_sem_timedwait to our ast_sem wrapper. * Fixed some GCC
-	  4.9 warnings using sig*set() functions. * Fixed some format
-	  strings for portability. * Fixed compilation issues with
-	  res_timing_kqueue (although tests still fail on OS X). * Fixed
-	  menuconfig /sbin/launchd detection, which disables
-	  res_timing_kqueue on OS X). ASTERISK-24539 #close Reported by:
-	  George Joseph ASTERISK-24544 #close Reported by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4327/
+2014-08-08 07:40 +0000 [86e927a714]  Matt Jordan <mjordan@digium.com>
 
-2015-01-25 13:42 +0000 [r431072]  Matthew Jordan <mjordan@digium.com>
+	* make_ari_stubs: Update wiki prefix to '13'
+	  ........
 
-	* main/config.c: dynamic realtime: Updates fail to work due to
-	  update fields being passed over When a crash was fixed due to
-	  usage of the REALTIME function in r423003, a regression was
-	  introduced into ast_update2_realtime where the update fields
-	  passed to the function would be skipped and the lookup field
-	  processed twice. The use of this function is a bit interesting: A
-	  variable argument list is used with two sentinel values - the
-	  first marks the end of the lookup fields/values; the second marks
-	  the end of the update fields/values. Unfortunately,
-	  ast_update2_realtime parses over the lookup fields twice, as
-	  opposed to parsing over the update fields. This causes the
-	  lookups to succeed, but the updates itself to have no effect.
-	  Note that the most common instance of this problem occurred in
-	  app_voicemail during the updating of a mailbox password. Thanks
-	  to the issue reporter, Paddy Grice, for pointing out the problem.
-	  Review: https://reviewboard.asterisk.org/r/4356/ ASTERISK-24231
-	  ASTERISK-24626 #close Reported by: Paddy Grice
-
-2015-01-23 20:13 +0000 [r431050-431052]  Richard Mudgett <rmudgett@digium.com>
-
-	* apps/confbridge/conf_chan_record.c: app_confbridge: Make CBRec
-	  channel names more unique. Channel names should be different from
-	  other channels in the system while the channel exists. * Use a
-	  sequence number for CBRec channels instead of a random number
-	  because the same random number could be picked again for the next
-	  CBRec channel.
-
-	* /, apps/app_confbridge.c: app_confbridge: Whitespace Because
-	  there is sometimes no sence to any whitespace. ........ Merged
-	  revisions 431049 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-23 17:08 +0000 [r431030]  David M. Lee <dlee@digium.com>
-
-	* res/res_pjsip_config_wizard.c: Add depend on pjproject to
-	  res_pjsip_config_wizard.c
-
-2015-01-23 15:12 +0000 [r430999]  Kevin Harwell <kharwell@digium.com>
-
-	* res/parking/parking_applications.c, channels/chan_iax2.c,
-	  res/res_pjsip/pjsip_global_headers.c, res/res_pjsip_pubsub.c,
-	  res/res_ari_channels.c, res/res_stasis.c,
-	  rest-api-templates/param_parsing.mustache,
-	  res/res_ari_endpoints.c, res/res_ari_events.c,
-	  include/asterisk/stasis_app.h, res/res_pjsip_mwi.c: Investigate
-	  and fix memory leaks in Asterisk Fixed memory leaks that were
-	  found in Asterisk. ASTERISK-24693 #close Reported by: Kevin
-	  Harwell Review: https://reviewboard.asterisk.org/r/4347/
-
-2015-01-23 15:03 +0000 [r430994-430998]  Walter Doekes <walter+asterisk@wjd.nu>
-
-	* apps/app_voicemail.c, channels/chan_unistim.c,
-	  funcs/func_hangupcause.c, main/manager_bridges.c,
-	  channels/chan_misdn.c, funcs/func_groupcount.c, /,
-	  addons/ooh323c/src/ooh245.c, channels/chan_sip.c, res/res_fax.c,
-	  res/res_pjsip_outbound_registration.c, apps/app_minivm.c,
-	  apps/app_alarmreceiver.c, include/asterisk/channel.h,
-	  contrib/utils/eagi_proxy.c: Fix typo's (retrieve, specified,
-	  address). ........ Merged revisions 430996 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* /, channels/chan_sip.c: chan_sip: Case insensitive comparison of
-	  "defaultuser" parameter. All the other configuration options are
-	  case insensitive, so this one should be too. ASTERISK-24355
-	  #close Reported by: HZMI8gkCvPpom0tM patches: ast.patch uploaded
-	  by HZMI8gkCvPpom0tM (License 6658) ........ Merged revisions
-	  430993 from http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-22 19:24 +0000 [r430957-430975]  Richard Mudgett <rmudgett@digium.com>
-
-	* include/asterisk/bridge.h,
-	  include/asterisk/bridge_channel_internal.h, main/bridge.c,
-	  include/asterisk/bridge_internal.h, main/bridge_channel.c: Bridge
-	  core: Pass a ref with the swap channel when joining a bridge.
-	  When code imparts a channel into a bridge to swap with another
-	  channel, a ref needs to be held on the swap channel to ensure
-	  that it cannot dissapear before finding it in the bridge. * The
-	  ast_bridge_join() swap channel parameter now always steals a ref
-	  for the swap channel. This is the only change to the bridge
-	  framework's public API semantics. *
-	  bridge_channel_internal_join() now requires the
-	  bridge_channel->swap channel to pass in a ref. ASTERISK-24649
-	  Reported by: John Bigelow Review:
-	  https://reviewboard.asterisk.org/r/4354/
-
-	* res/res_pjsip_outbound_registration.c:
-	  res_pjsip_outbound_registration.c: Minor code cleanup. * Add an
-	  allocation failure check and assert in
-	  sip_outbound_registration_response_cb(). * Made
-	  sip_outbound_registration_state_destroy() handle partially
-	  created state objects from
-	  sip_outbound_registration_state_alloc(). Review:
-	  https://reviewboard.asterisk.org/r/4366/
-
-2015-01-22 18:09 +0000 [r430939]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* res/stasis/app.c, res/stasis/stasis_bridge.c: stasis transfer:
-	  fix a race condition on stasis bridge push After a bridge
-	  transfer completes where a local replacement channel is used, a
-	  stasis transfer message with the details of the transfer is sent.
-	  This is processed by stasis which then sets the stasis app name
-	  and replaced channel snapshot on the replacement channel.
-	  However, since a separate thread was already started to run
-	  stasis on the new replacement channel, a race was on to see if
-	  the message processing would be completed before the app name was
-	  needed, otherwise the channel would be hung up. This change moves
-	  the calls used to set the stasis app name and the replace
-	  snapshot to the bridge_stasis_push function callback from the
-	  bridge transfer logic, allowing the steps to be completed earlier
-	  and more deterministically, and the race elimianted. NOTE: the
-	  swap channel parameter to bridge_stasis_push (and thus all bridge
-	  push callbacks) must always be present when performing a swap
-	  with another channel. ASTERISK-24649 #close Reported by: John
-	  Bigelow Review: https://reviewboard.asterisk.org/r/4341/
-
-2015-01-22 14:23 +0000 [r430921]  Matthew Jordan <mjordan@digium.com>
-
-	* /, apps/app_voicemail.c: apps/app_voicemail: Trigger MWI
-	  notification with MixMonitor m() option The MixMonitor m() option
-	  allows a recording to be pushed to a specific voicemail mailbox.
-	  If the message is delivered to the mailbox's INBOX, however, no
-	  MWI notification is currently raised. This patch corrects the
-	  issue by properly calling notify_new_state from the
-	  msg_create_from_file function. This will cause MWI to be
-	  triggered if the message was placed in the mailbox's INBOX.
-	  ASTERISK-24709 #close Reported by: Gareth Palmer patches:
-	  app_voicemail-430919.patch uploaded by Gareth Palmer (License
-	  5169) ........ Merged revisions 430920 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-21 21:53 +0000 [r430902]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_pjsip_outbound_registration.c:
-	  res_pjsip_outbound_registration.c: Move unref to a better place.
-	  Move an unconditional unref of client_state so it doesn't look
-	  like it could be used after the last ref has destroyed it.
-
-2015-01-21 13:33 +0000 [r430840-430864]  Matthew Jordan <mjordan@digium.com>
-
-	* channels/chan_sip.c: channels/chan_sip: Fix registration leak
-	  during reload When the SIP registrations were migrated to using
-	  ao2 in what was then trunk, the explicit destruction of the
-	  registrations on module reload was removed and not replaced with
-	  an ao2 equivalent. Debugging done by Stefan Engström, the issue
-	  reporter, on ASTERISK-24673 confirmed that the reference in the
-	  registry_list container was being leaked. Since the purpose of
-	  cleanup_all_regs is to prep a registration for destruction, this
-	  function now calls an ao2_callback function callback with the
-	  OBJ_MULTIPLE | OBJ_NODATA | OBJ_UNLINK flags used to remove the
-	  registrations. This cleans up each registration, and also removes
-	  it from the registration container registry_list. Review:
-	  https://reviewboard.asterisk.org/r/4355/ ASTERISK-24640 #close
-	  Reported by: Max Man ASTERISK-24673 #close Reported by: Stefan
-	  Engström Tested by: Stefan Engström
-
-	* cdr/cdr_manager.c, cel/cel_manager.c: AMI: Add documentation for
-	  the missing Cdr/CEL events. This patch adds AMI event
-	  documentation for the Cdr and CEL AMI events. Note that while
-	  these events do share fields with each other and with other
-	  channel related events, they do not contain all of the fields in
-	  a standard channel snapshot, nor is the description of the fields
-	  identical. As such, the patch opts for documentation for each
-	  field, for each event. Review:
-	  https://reviewboard.asterisk.org/r/4350/ ASTERISK-24671 #close
-	  Reported by: Dan Jenkins
+	  Merged revisions 420538 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* apps/app_dial.c: apps/app_dial: Don't publish DialEnd twice on
-	  unexpected GoSub/Macro values The Dial application has some
-	  interesting options with the mid-call Macro (M) and GoSub (U)
-	  options. If the MACRO_RESULT/GOSUB_RESULT returns specific
-	  values, the Dial application will take some action upon the
-	  channels involved in the dial operation (such as hanging up a
-	  particular party, etc.) The Dial application ensures that a
-	  Stasis message is published in the event that
-	  MACRO_RESULT/GOSUB_RESULT returns a value that kills the dial
-	  operation, so that there is a corresponding DialEnd event
-	  published in AMI/ARI for the DialBegin event that preceeded it. A
-	  bug exists where that same DialEnd event will be published on
-	  Stasis even if the value returned in MACRO_RESULT/GOSUB_RESULT is
-	  not one that the Dial application cares about. This causes two
-	  DialEnd events to be published - one with the
-	  MACRO_RESULT/GOSUB_RESULT and another with "ANSWERED" - which is
-	  all sorts of wrong. This patch fixes the bug by ensuring that we
-	  only publish a DialEnd message to Stasis if the Dial
-	  application's mid-call Macro/GoSub returns something that Dial
-	  cares about. Review: https://reviewboard.asterisk.org/r/4336
-	  ASTERISK-24682 #close Reported by: Matt Jordan
-
-	* main/rtp_engine.c: main/rtp_engine: Format NTP timestamps as
-	  unsigned longs When the RTCP reports are created, the NTP
-	  timestamps are stored as strings, as JSON does not have an
-	  integer type long enough to store the value. However, on 32-bit
-	  systems, a signed long may overflow for some portion of the
-	  timestamp. This patch corrects the overflow by formatting the
-	  timestamps as unsigned longs.
-
-2015-01-20 16:51 +0000 [r430818]  asanders <asanders@localhost>:
-
-	* res/ari/resource_bridges.c: ARI: Fixed crash that occurred when
-	  updating a bridge when the optional query parameter 'name' was
-	  not supplied. Prior to this changeset, posting to the:
-	  /ari/bridges/{bridgeId} endpoint without specifying a value for
-	  the [name] query parameter, would crash Asterisk if the bridge
-	  you are attempting to create (or update) had the same ID as an
-	  existing bridge. The internal mechanism of the POST operation
-	  interpreted a null value for name, thus resulting in an error
-	  condition that crashed Asterisk. ASTERISK-24560 #close Reported
-	  By: Kinsey Moore Review: https://reviewboard.asterisk.org/r/4349/
-
-2015-01-20 16:46 +0000 [r430817]  Richard Mudgett <rmudgett@digium.com>
-
-	* configs/samples/iax.conf.sample, res/res_fax.c,
-	  funcs/func_channel.c, UPGRADE.txt, res/snmp/agent.c,
-	  channels/chan_iax2.c: CHANNEL(peer), chan_iax2, res_fax, SNMP
-	  agent: Fix deadlock from reaching across a bridge. Calling
-	  ast_channel_bridge_peer() cannot be done while holding any
-	  channel locks. The reported issue hit the deadlock in chan_iax2,
-	  but an audit of the ast_channel_bridge_peer() calls found three
-	  more locations where the same deadlock can occur. * Made
-	  CHANNEL(peer), res_fax, and the SNMP agent not call
-	  ast_channel_bridge_peer() with any channel locked. For
-	  CHANNEL(peer) I had to rework the logic to not hold the channel
-	  lock. * Made chan_iax2 no longer call ast_channel_bridge_peer().
-	  It was done for legacy reasons that no longer apply. * Removed
-	  the iax.conf forcejitterbuffer option. It is now always enabled
-	  when the jitterbuffer option is enabled. If you put a jitter
-	  buffer on a channel it will be on the channel. ASTERISK-24600
-	  #close Reported by: Jeff Collell Review:
-	  https://reviewboard.asterisk.org/r/4342/
-
-2015-01-20 02:39 +0000 [r430796-430799]  Matthew Jordan <mjordan@digium.com>
-
-	* contrib/scripts/install_prereq, /:
-	  contrib/scripts/install_prereq: Don't install 32-bit packages on
-	  64-bit hosts On Debian based systems, the install_prereq tool
-	  uses a search command on Debian that results in selecting both
-	  64-bit and 32-bit packages. Besides the waste of disk space, this
-	  can actually cause aptitude use 100% of memory on a VM with 1GB
-	  of RAM as it tried to work out all of the 32-bit package
-	  dependencies. This patch filters out the 32-bit packages on a
-	  64-bit machine, and leaves 32-bit machines alone. ASTERISK-24048
-	  #close Reported by: Ben Klang Tested by: Ben Klang, Matt Jordan
-	  patches: install_prereq_64-bit_compat.patch uploaded by Ben Klang
-	  (License 5876) ........ Merged revisions 430798 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* apps/app_voicemail.c, /: app_voicemail: Temp message left after
-	  review/hangup with ODBC/IMAP backend When using ODBC or IMAP
-	  storage, temporary files created on the file system must be
-	  disposed of using the DISPOSE macro. The DELETE macro will map to
-	  a deletion function for the backend storage, but does not clean
-	  up any local files created as a result of the operation. When
-	  using voicemail with the operator and review options enabled,
-	  pressing 0 to enter the menu, followed by 1 to save the message,
-	  followed by any other DTMF press to delete the message, will
-	  result in the temporary file lingering on the file system. This
-	  patch properly calls DISPOSE after the DELETE. This causes the
-	  local file to be disposed of. ASTERISK-24288 #close Reported by:
-	  LEI FU patches: voicemail_odbc_review_fix.diff uploaded by LEI FU
-	  (License 6640) ........ Merged revisions 430795 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-19 18:05 +0000 [r430776]  Mark Michelson <mmichelson@digium.com>
-
-	* main/pbx.c: Call extension state callbacks at hint creation. When
-	  a hint gets created, any subsequent device or presence state
-	  changes result in extension status events getting sent out to
-	  interested parties. However, at the time of hint creation, no
-	  such event gets sent out, so watchers of extension state are
-	  potentially left in the dark until the first state change after
-	  hint creation. Patch contributed by John Hardin (License #6512)
 
-2015-01-19 13:18 +0000 [r430755]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420539 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* res/res_pjsip_multihomed.c, res/res_pjsip.c: res_pjsip /
-	  res_pjsip_multihomed: Use the correct transport and addressing
-	  information on UAS sessions. The first thing this patch fixes is
-	  UAS dialogs. Previously if a transport was configured on an
-	  endpoint and an inbound session was created there was no
-	  guarantee that requests sent on the dialog would use the correct
-	  transport and address information. This has now been fixed so an
-	  explicitly configured transport is taken into account. The second
-	  thing this patch fixes is res_pjsip_multihomed. The
-	  res_pjsip_multihomed module attempts to determine what transport
-	  a message should go out on and what addressing information should
-	  go into the message itself. In a scenario where multiple
-	  transports exist bound to the same IP address but a different
-	  port the code would incorrectly alter the transport and change
-	  the message to the wrong transport. This change makes the
-	  res_pjsip_multihomed module smarter so it will only change the
-	  transport and address information in the message when it is
-	  possible and makes sense. ASTERISK-24615 #close Reported by:
-	  David Justl Review: https://reviewboard.asterisk.org/r/4331/
-
-2015-01-17 00:31 +0000 [r430734]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip/config_transport.c,
-	  res/res_pjsip/pjsip_outbound_auth.c, res/res_pjsip/config_auth.c,
-	  main/stasis_message_router.c, res/res_pjsip/location.c,
-	  res/res_pjsip/pjsip_configuration.c,
-	  res/res_pjsip/pjsip_distributor.c,
-	  res/res_pjsip/include/res_pjsip_private.h,
-	  res/res_pjsip/pjsip_global_headers.c,
-	  res/res_pjsip/pjsip_options.c, res/res_pjsip.c: REVERTING
-	  res_pjsip: make it unloadable Due to the original patch causing
-	  memory corruptions the patch is being removed until the problem
-	  can be resolved.
-
-2015-01-16 22:13 +0000 [r430709-430716]  Mark Michelson <mmichelson@digium.com>
-
-	* CHANGES: Change PJProject version requirement for ca_list_path
-	  transport option in CHANGES file.
-
-	* channels/chan_pjsip.c, res/res_pjsip_session.c: Fix problem where
-	  a hung channel could occur on a failed blind transfer. Different
-	  clients react differently to being told that a blind transfer has
-	  failed. Some will simply send a BYE and be done with it. Others
-	  will attempt to reinvite themselves back onto the call. In the
-	  latter case, we were creating a new channel and then leaving it
-	  to sit forever doing nothing. With this code change, that new
-	  channel will not be created and the dialog with the transferring
-	  channel will be cleaned up properly. ASTERISK-24624 #close
-	  Reported by Zane Conkle Review:
-	  https://reviewboard.asterisk.org/r/4339
-
-	* include/asterisk/res_pjsip.h, configure,
-	  include/asterisk/autoconfig.h.in, configure.ac,
-	  configs/samples/pjsip.conf.sample, CHANGES, res/res_pjsip.c,
-	  res/res_pjsip/config_transport.c: Add support for the
-	  ca_list_path option for PJSIP transports. This allows for a path
-	  to be specified that has a collection of CA certificates in it.
-	  ASTERISK-24575 #close Reported by cloos Patches:
-	  pj-ca-path-trunk.diff uploaded by cloos (License #5956) Review:
-	  https://reviewboard.asterisk.org/r/4344
-
-2015-01-15 17:35 +0000 [r430685-430687]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_fax_spandsp.c, res/res_fax.c: res_fax.c,
-	  res_fax_spandsp.c: Remove redundant locking. When FAX was
-	  developed, apparently the faxregistry.container used to be a
-	  linked list that was converted to an ao2 container. Some of the
-	  replacement ao2 container operations still had explicit
-	  lock/unlocks around them. Three off nominal code paths in
-	  res_fax.c and res_fax_spandsp.c unlock the channel even though
-	  the routine did not lock the channel and other code paths in the
-	  routine do not unlock the channel. Review:
-	  https://reviewboard.asterisk.org/r/4340/
-
-	* res/res_fax_spandsp.c, res/res_fax.c: res_fax.c,
-	  res_fax_spandsp.c: Fix some curlies on the end of function
-	  definitions.
-
-2015-01-15 12:09 +0000 [r430664]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_outbound_registration.c:
-	  res_pjsip_outbound_registration: Fix race condition when
-	  reloading and listing registrations. Due to the split of outbound
-	  registration state from configuration it is possible during a
-	  reload for a "pjsip show registrations" CLI command to be
-	  executed which gets an older snapshot of the configuration. This
-	  configuration may include outbound registrations which have been
-	  removed due to a reload operation occurring at the same time. The
-	  code for printing the outbound registration did not take this
-	  into account but now it does. AST-1506 #close Review:
-	  https://reviewboard.asterisk.org/r/4338/
-
-2015-01-15 02:18 +0000 [r430646]  Matthew Jordan <mjordan@digium.com>
-
-	* configure, configure.ac: configure: If cross-compiling, assume we
-	  have working semaphores The Asterisk 13 configure.ac checks for
-	  HAS_WORKING_SEMAPHORE but does not have an option for
-	  cross-compiling so it fails with an exit. Since we're cross-
-	  compiling, we can't exactly go looking for the header. The
-	  semaphore.h header is relatively common: * It's part of the POSIX
-	  standard * It's part of GNU C Library As such, we assume that it
-	  will be present when cross-compiling. As such, this patch
-	  defaults "HAS_WORKING_SEMAPHORE" to "1" if cross-compiling is
-	  detected. If you're cross-compiling to a platform that doesn't
-	  support this, then make sure you re-define this to 0.
-	  ASTERISK-24663 #close Reported by: abelbeck patches:
-	  asterisk-13-anonymous-semaphores.patch uploaded by abelbeck
-	  (License 5903)
-
-2015-01-14 23:14 +0000 [r430628]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip/pjsip_configuration.c,
-	  res/res_pjsip/pjsip_distributor.c,
-	  res/res_pjsip/include/res_pjsip_private.h,
-	  res/res_pjsip/pjsip_global_headers.c,
-	  res/res_pjsip/pjsip_options.c, res/res_pjsip.c,
-	  res/res_pjsip/config_transport.c,
-	  res/res_pjsip/pjsip_outbound_auth.c, res/res_pjsip/config_auth.c,
-	  main/stasis_message_router.c, res/res_pjsip/location.c:
-	  res_pjsip: make it unloadable The res_pjsip module was previously
-	  unloadable. With this patch it can now be unloaded. This patch is
-	  based off the original patch on the issue (listed below) by Corey
-	  Farrell with a few modifications. Namely, removed a few changes
-	  not required to make the module unloadable and also fixed a bug
-	  that would cause asterisk to crash on unloading. This patch is
-	  the first step (should hopefully be followed by another/others at
-	  some point) in allowing res_pjsip and the modules that depend on
-	  it to be unloadable. At this time, res_pjsip and some of the
-	  modules that depend on res_pjsip cannot be unloaded without
-	  causing problems of some sort. The goal of this patch is to get
-	  res_pjsip and only res_pjsip to be able to unload successfully
-	  and/or shutdown without incident (crashes, leaks, etc...). Other
-	  dependent modules may still cause problems on unload. Basically
-	  made sure, with the patch applied, that res_pjsip (with no other
-	  dependent modules loaded) could be succesfully unloaded and
-	  Asterisk could shutdown without any leaks or crashes that
-	  pertained directly to res_pjsip. ASTERISK-24485 #close Reported
-	  by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4311/ patches:
-	  pjsip_unload-broken-r1.patch submitted by Corey Farrell (license
-	  5909)
-
-2015-01-14 20:27 +0000 [r430608]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip_outbound_publish.c: Prevent slow graceful shutdown
-	  when outbound publications never started. The code was missing
-	  the case for explicitly destroying an outbound publication when
-	  Asterisk had never actually published anything. The result was
-	  that Asterisk would hang for a while on a graceful shutdown. With
-	  this change, the case is taken into account, and on a graceful
-	  shutdown, these publications are destroyed without the need to
-	  actually send a PUBLISH request. ASTERISK-24655 #close Reported
-	  by Kevin Harwell Review: https://reviewboard.asterisk.org/r/4325
-
-2015-01-14 15:39 +0000 [r430590]  Matthew Jordan <mjordan@digium.com>
-
-	* build_tools/mkpkgconfig, /: build_tools/mkpkgconfig: Fix Cflags
-	  concatenation error in asterisk.pc The mkpkgconfig script
-	  incorrectly concatenates Cflags options together. As an example,
-	  the following: Cflags: -I/usr/include/libxml2 -g3 Is instead
-	  generated as: Cflags: -I/usr/include/libxml2-g3 This patch
-	  corrects the generation of Cflags in mkpkgconfig such that the
-	  Cflags options are output correctly. Review:
-	  https://reviewboard.asterisk.org/r/3707/ ASTERISK-23991 #close
-	  Reported by: Diederik de Groot patches: fix_mkpkgconfig.diff
-	  uploaded by Diederik de Groot (License 6600) ........ Merged
-	  revisions 430589 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-13 18:16 +0000 [r430565]  Richard Mudgett <rmudgett@digium.com>
-
-	* apps/app_macro.c, /: app_macro: Don't restore the calling
-	  location on a channel redirect. v11: If a channel redirect to a
-	  macro exten of a macro that is active happens, the redirect
-	  location doesn't get executed. Instead the original macro
-	  location is restored and gets reexecuted. v13: An additional
-	  effect happens if a parked call times out to an extension in the
-	  macro that parked the call then the macro is reexecuted instead
-	  of the expected park return location. * Made not restore the
-	  macro calling location on an AST_SOFTHANGUP_ASYNCGOTO. *
-	  Increased the locked channel range when setting up the macro
-	  execution environment to cover things that should be done while
-	  the channel is locked. * Removed unnecessary NULL tests before
-	  calling ast_free() in _macro_exec(). ASTERISK-23850 #close
-	  Reported by: Andrew Nagy Review:
-	  https://reviewboard.asterisk.org/r/4292/ ........ Merged
-	  revisions 430564 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-13 12:06 +0000 [r430546]  Joshua Colp <jcolp@digium.com>
-
-	* channels/pjsip/dialplan_functions.c, configure,
-	  include/asterisk/autoconfig.h.in, configure.ac: chan_pjsip: Add
-	  configure check for 'pjsip_get_dest_info' function. The
-	  'pjsip_get_dest_info' function is used to determine if the
-	  signaling transport of the dialog is secure or not. This function
-	  was added in PJSIP 2.3 and does not exist in earlier versions.
-	  This configure check allows Asterisk to build and run with older
-	  versions at the loss of the 'secure' argument for the PJSIP
-	  CHANNEL dialplan function. Usage of this argument will require
-	  upgrading to PJSIP 2.3. ASTERISK-24665 #close Reported by: Mark
-	  Michelson Review: https://reviewboard.asterisk.org/r/4329/
-
-2015-01-12 18:34 +0000 [r430528]  Richard Mudgett <rmudgett@digium.com>
-
-	* include/asterisk/manager.h, main/manager.c: AMI: Revert
-	  non-backwards compatible changes from earlier commit. * Reverted
-	  the change to astman_send_listack() to not use the listflag
-	  parameter and always set the value to "Start" so the start
-	  capitalization is consistent. Unfortunately changing the case of
-	  a returned value is not a backward compatible change so for now
-	  FAXSessions is going to have to remain inconsistent with all of
-	  the other AMI list actions. * Reverted the minor protocol error
-	  fix in action_getconfig() when no requested categories are found.
-	  Each line needs to be formatted as "Header: text". Caught by the
-	  testsuite. ASTERISK-24049
-
-2015-01-12 18:28 +0000 [r430488-430526]  Matthew Jordan <mjordan@digium.com>
-
-	* configs/samples/features.conf.sample:
-	  configs/samples/features.conf.sample: Document attended transfer
-	  DTMF options The sample config was missing the configuration
-	  options for DTMF attended transfer completion scenarios. The
-	  configuration options 'atxferabort', 'atxfercomplete',
-	  'atxferthreeway', and 'atxferswap' are now documented in the
-	  appropriate configuration file. ASTERISK-24678 #close Reported
-	  by: Niklas Larsson patches: features.conf.sample.diff uploaded by
-	  Niklas Larsson (License 5068)
-
-	* main/syslog.c, include/asterisk/syslog.h, /: main/syslog: Allow
-	  dynamic logs, such as security events, to log to the syslog The
-	  security event log uses a dynamic log level (SECURITY) that is
-	  registered with the Asterisk logging core. Unfortunately, the
-	  syslog would ignore log statements that had a dynamic log level
-	  associated with them. Because the syslog cannot handle ad hoc
-	  dynamic log levels, this patch treats any dynamic log entries
-	  sent to the syslog as logs with a level of NOTICE. ASTERISK-20744
-	  #close Reported by: Michael Keuter Tested by: Michael L. Young,
-	  Jacek Konieczny patches:
-	  asterisk-20744-syslog-dynamic-logging_trunk.diff uploaded by
-	  Michael L. Young (license 5026) ........ Merged revisions 430506
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* funcs/func_curl.c, /: funcs/func_curl: Fix memory leak when
-	  CURLOPT channel datastore is destroyed When the channel datastore
-	  associated with the usage of CURLOPT on a specific channel is
-	  freed, the underlying structure holding the list of options is
-	  not disposed of. This patch properly frees the structure in the
-	  datastore .destroy callback. ASTERISK-24672 #close Reported by:
-	  Kristian Hogh patches: func_curl-memory-leak.diff uploaded by
-	  Kristian Hogh (License 6639) ........ Merged revisions 430487
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-09 22:08 +0000 [r430467-430469]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* contrib/scripts/sip_to_pjsip/sip_to_pjsip.py,
-	  contrib/scripts/sip_to_pjsip/astconfigparser.py: sip_to_pjsip:
-	  improve ability to parse input files General improvements to SIP
-	  to PJSIP conversion utility: 1) track default section of input
-	  file to allow parsing an include file that doesn't specify a
-	  [section] 2) informatively handle case of assignment without
-	  [section] 3) correctly handle getting sections from included
-	  files - [section]'s are inherited by included file 4) provide
-	  null string as default transport bind ip 5) gracefully handle
-	  missing portions of registration string 6) denote steps of
-	  operation during conversion and confirm top level files as a
-	  convenience ASTERISK-24474 #close Review:
-	  https://reviewboard.asterisk.org/r/4280/ Reported by: John
-	  Kiniston
-
-	* main/features.c: app_bridge: return to the next dialplan priority
-	  When app_bridge grabs a channel and puts it into a bridge, the
-	  channel should then continue where it left off in the dialplan
-	  after the bridge has ended. Although it stores the current
-	  dialplan location as an after bridge goto on the channel, it was
-	  executing the same priority again instead of going to the next
-	  priority. By swapping the "specific" version of
-	  bridge_set_after_goto with bridge_set_after_go_on, the next
-	  priority in the dialplan is executed instead. ASTERISK-24637
-	  #close Review: https://reviewboard.asterisk.org/r/4322/ Reported
-	  by: John Bigelow
-
-2015-01-09 17:54 +0000 [r430434]  Richard Mudgett <rmudgett@digium.com>
-
-	* UPGRADE.txt, res/res_mwi_external_ami.c, CHANGES,
-	  include/asterisk/manager.h, channels/chan_iax2.c,
-	  apps/app_queue.c, apps/app_agent_pool.c,
-	  res/res_manager_devicestate.c, main/manager_bridges.c,
-	  channels/chan_dahdi.c, main/manager.c, channels/chan_skinny.c,
-	  res/res_pjsip_outbound_registration.c,
-	  res/res_manager_presencestate.c,
-	  res/res_pjsip/pjsip_configuration.c, apps/app_confbridge.c,
-	  res/res_pjsip_pubsub.c, main/db.c, res/parking/parking_manager.c,
-	  res/res_pjsip_registrar.c, apps/app_voicemail.c, main/pbx.c,
-	  channels/chan_sip.c, apps/app_meetme.c, main/bridge.c,
-	  res/res_fax.c: AMI: Make AMI actions that generate event lists
-	  consistent. * Made the following AMI actions use list API calls
-	  for consistency: Agents BridgeInfo BridgeList
-	  BridgeTechnologyList ConfbridgeLIst ConfbridgeLIstRooms
-	  CoreShowChannels DAHDIShowChannels DBGet DeviceStateList
-	  ExtensionStateList FAXSessions Hangup IAXpeerlist IAXpeers
-	  IAXregistry MeetmeList MeetmeListRooms MWIGet ParkedCalls
-	  Parkinglots PJSIPShowEndpoint PJSIPShowEndpoints
-	  PJSIPShowRegistrationsInbound PJSIPShowRegistrationsOutbound
-	  PJSIPShowResourceLists PJSIPShowSubscriptionsInbound
-	  PJSIPShowSubscriptionsOutbound PresenceStateList PRIShowSpans
-	  QueueStatus QueueSummary ShowDialPlan SIPpeers SIPpeerstatus
-	  SIPshowregistry SKINNYdevices SKINNYlines Status
-	  VoicemailUsersList * Incremented the AMI version to 2.7.0. *
-	  Changed astman_send_listack() to not use the listflag parameter
-	  and always set the value to "Start" so the start capitalization
-	  is consistent. i.e., The FAXSessions used "Start" while the rest
-	  of the system used "start". The corresponding complete event
-	  always used "Complete". * Fixed ami_show_resource_lists()
-	  "PJSIPShowResourceLists" to output the AMI ActionID for all of
-	  its list events. * Fixed off-nominal AMI protocol error in
-	  manager_bridge_info(), manager_parking_status_single_lot(), and
-	  manager_parking_status_all_lots(). Use of astman_send_error()
-	  after responding to the original AMI action request violates the
-	  action response pattern by sending two responses. * Fixed minor
-	  protocol error in action_getconfig() when no requested categories
-	  are found. Each line needs to be formatted as "Header: text". *
-	  Fixed off-nominal memory leak in
-	  manager_build_parked_call_string(). * Eliminated unnecessary use
-	  of RAII_VAR() in ami_subscription_detail(). ASTERISK-24049 #close
-	  Reported by: Jonathan Rose Review:
-	  https://reviewboard.asterisk.org/r/4315/
-
-2015-01-09 14:51 +0000 [r430416]  Kinsey Moore <kmoore@digium.com>
-
-	* /, res/res_fax.c, include/asterisk/res_fax.h,
-	  configs/samples/res_fax.conf.sample, CHANGES: res_fax: Add T.38
-	  negotiation timeout option This change makes the T.38 negotiation
-	  timeout configurable via 't38timeout' in res_fax.conf or
-	  FAXOPT(t38timeout). It was previously hard coded to be 5000
-	  milliseconds. This change also handles T.38 switch failures by
-	  aborting the fax since in the case where this can happen, both
-	  sides have agreed to switch to T.38 and Asterisk is unable to do
-	  so. Review: https://reviewboard.asterisk.org/r/4320/ ........
-	  Merged revisions 430415 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2015-01-08 21:40 +0000 [r430373-430397]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_pubsub.c: res_pjsip_pubsub: Fix persistent
-	  subscriptions not surviving graceful shutdown If you do a 'core
-	  (shutdown|restart) graceful' persistent subscriptions won't
-	  survive. If you do a 'core (shutdown|restart) now' or asterisk
-	  terminates for some reason, they do. Here's why... When asterisk
-	  shuts down gracefully, it sends a 'NOTIFY/terminated' to
-	  subscribers for each subscription. This not only tells the
-	  subscribers that the dialog/state machine is done, it also frees
-	  the last reference to the subscription tree which causes the
-	  persistent subscription to get deleted from astdb. When asterisk
-	  restarts, nothing's left. Just preventing the delete from astdb
-	  doesn't work because we already told the subscriber to terminate
-	  the dialog so we can't restart it even if it was still in astdb.
-	  Everything works OK if asterisk terminates unexpectedly because
-	  we never send the 'terminated' message so on restart, the
-	  subscription is still in astdb and the subscriber is none the
-	  wiser. This patch suppresses the sending of 'NOTIFY/terminated'
-	  on shutdown for persistent connections. Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4318/
+2014-08-08 07:38 +0000 [1f35fccda1]  Matt Jordan <mjordan@digium.com>
 
-	* res/res_pjsip_outbound_registration.c:
-	  res_pjsip_outbound_registration: Fix reference leak. Every time a
-	  registration started, sip_outbound_registration_response_cb bumps
-	  the ref count on client_state then pushes a
-	  handle_registration_response task. handle_registration_response
-	  never unreffed it though. So every time a registration goes out,
-	  the ref count goes up by one. This patch adds the unreffs to
-	  handle_registration_response. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4303/
-
-	* res/res_pjsip_outbound_registration.c:
-	  res_pjsip_outbound_registration: Fix several reload issues There
-	  are 2 issues with reloading registrations... 1. The
-	  'can_reuse_registration' test wasn't considering the intervals or
-	  expiration in its determination of whether a registration changed
-	  or not so if you changed any of the intervals or the expiration
-	  and reloaded, the object would get reloaded but the actual timers
-	  wouldn't change. can_reuse_registration now does a sorcery diff
-	  on the old and new objects instead of discretely testing certain
-	  fields. Now if you change expiration for instance, and reload,
-	  the timer is updated and re-registration will occur on the new
-	  value. 2. If you mung up your password on an outbound
-	  registration you get a permanent failure. If you fix the password
-	  (on the outbound_auth object) and reload, nothing tells
-	  outbound_registration to try again because the registration
-	  itself didn't change. This patch adds an observer on the "auth"
-	  object type and if any auth changes, existing registration states
-	  are searched and those in a REJECTED_PERMANENT state are retried.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4304/
-
-2015-01-07 21:25 +0000 [r430355]  Kinsey Moore <kmoore@digium.com>
-
-	* res/res_stasis.c: ARI: Allow usage of ASYNCGOTO with Stasis()
-	  When the AMI Redirect action is used with a channel bridged
-	  inside Stasis() and not running a pbx, the channel is hung up
-	  instead of proceeding to the desired location in dialplan. This
-	  change allows such channels to be Redirected properly by
-	  detecting the operation used by Redirect (ASYNCGOTO) and using
-	  the code already established for functionality of the ARI channel
-	  continue operation. ASTERISK-24591 #close Review:
-	  https://reviewboard.asterisk.org/r/4271/
-
-2015-01-07 18:53 +0000 [r430337]  Mark Michelson <mmichelson@digium.com>
-
-	* rest-api/api-docs/channels.json, rest-api/resources.json,
-	  res/ari/resource_channels.c, CHANGES, res/res_ari_channels.c,
-	  res/ari/resource_channels.h: Add the ability to continue and
-	  originate using priority labels. With this patch, the following
-	  two ARI commands POST /channels POST /channels/{id}/continue
-	  Accept a new parameter, label, that can be used to continue to or
-	  originate to a priority label in the dialplan. Because this is
-	  adding a new parameter to ARI commands, the API version of ARI
-	  has been bumped from 1.6.0 to 1.7.0. This patch comes courtesy of
-	  Nir Simionovich from Greenfield Tech. Thanks! ASTERISK-24412
-	  #close Reported by Nir Simionovich Review:
-	  https://reviewboard.asterisk.org/r/4285
-
-2015-01-07 18:17 +0000 [r430315-430319]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_exten_state.c: res_pjsip_exten_state: Change 'does
-	  not exist' warning to notice The 'new_subscribe: Extension <>
-	  does not exist or has no associated hint' is a config issue and
-	  doesn't need to clutter up logs with warnings. Changed to notice.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4307/
-
-	* res/res_pjsip_mwi.c: res_pjsip_mwi: Change "MWI Subscription
-	  failed" message from warning to notice The "MWI Subscription
-	  failed" message means the client is trying to subscribe to a
-	  mailbox that doesn't exist. There's no need to clutter up logs
-	  with warnings for a client misconfiguration so I changed it to a
-	  notice. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4306/
-
-	* funcs/func_config.c, tests/test_config.c: func_config: Add
-	  ability to retrieve specific occurrence of a variable I guess
-	  nobody uses templates with AST_CONFIG because today if you have a
-	  context that inherits from a template and you call AST_CONFIG on
-	  the context, you'll get the value from the template even if
-	  you've overridden it in the context. This is because AST_CONFIG
-	  only gets the first occurrence which is always from the template.
-	  This patch adds an optional 'index' parameter to AST_CONFIG which
-	  lets you specify the exact occurrence to retrieve, or '-1' to
-	  retrieve the last. The default behavior is the current behavior.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4313/
-
-2015-01-07 17:35 +0000 [r430313]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip_refer.c: Fix ability to perform a remote attended
-	  transfer with PJSIP. This fix has two parts: * Corrected an error
-	  message to properly state that external_replaces is an extension.
-	  The error message also prints what dialplan context the
-	  external_replaces extension was being looked for in. * Corrected
-	  the printing of the Replaces: header in an INVITE request. We
-	  were duplicating "Replaces: " in the header. ASTERISK-24376
-	  #close Reported by Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4296
-
-2015-01-07 16:55 +0000 [r430295]  George Joseph <george.joseph@fairview5.com>
-
-	* include/asterisk/config.h, main/config.c, main/manager.c: config:
-	  Add option to NOT preserve effective context when changing a
-	  template Let's say you have a template T with variable VAR1 = ON
-	  and you have a context C(T) that doesn't specify VAR1. If you
-	  read C, the effective value of VAR1 is ON. Now you change T VAR1
-	  to OFF and call ast_config_text_file_save. The current behavior
-	  is that the file gets re-written with T/VAR1=OFF but C/VAR1=ON is
-	  added. Personally, I think this is a bug. It's preserving the
-	  effective state of C even though I didn't specify C/VAR1 in th
-	  first place. I believe the behavior should be that if I didn't
-	  specify C/VAR1 originally, then the effective value of C/VAR1
-	  should continue to follow the inherited state. Now, if I DID
-	  explicitly specify C/VAR1, the it should be preserved even if the
-	  template changes. Even though I think the existing behavior is a
-	  bug, it's been that way forever so I'm not changing it. Instead,
-	  I've created ast_config_text_file_save2() that takes a bitmask of
-	  flags, one of which is to preserve the effective context (the
-	  current behavior). The original ast_config_text_file_save calls
-	  *2 with the preserve flag. If you want the new behavior, call *2
-	  directly without a flag. I've also updated Manager UpdateConfig
-	  with a new parameter 'PreserveEffectiveContext' whose default is
-	  'yes'. If you want the new behavior with UpdateConfig, set
-	  'PreserveEffectiveContext: no'. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4297/
-
-2015-01-07 02:52 +0000 [r430274]  Kinsey Moore <kmoore@digium.com>
-
-	* res/res_pjsip/pjsip_options.c, res/res_pjsip.c,
-	  main/rtp_engine.c: Fix dev-mode build on recent gcc
-
-2015-01-06 22:46 +0000 [r430252]  Matthew Jordan <mjordan@digium.com>
-
-	* contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py:
-	  contrib/ast-db-manage: Correct down_revision path for
-	  user_eq_phone When the user_eq_phone patch was backported to 13,
-	  it referenced the downward revision that the PJSIP optimistic
-	  encryption option also references. This creates a multi-path
-	  upgrade Exception when generating the SQL files. This patch
-	  corrects this in the 13 branch. Note that trunk, which already
-	  contained both of these features, is unaffected by this problem.
+	* res_ari_resource.c.mustache: Update template to emit module support level
+	  ........
 
-2015-01-06 17:52 +0000 [r430221-430227]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 420536 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* res/res_pjsip_mwi.c: res_pjsip_mwi: Change warning to notice When
-	  res_pjsip loads and an endpoint auto-subscribes a mailbox for
-	  mwi, if a contact hasn't registered yet, res_pjsip_mwi spits out
-	  a warning. This is a perfectly normal situation though and
-	  doesn't require something as serious as a warning. It's also self
-	  correcting. The device will start getting mwi as soon as it
-	  registers. This patch changes the warning to a notice. Tested-by:
-	  George Joseph Review: https://reviewboard.asterisk.org/r/4314/
-
-	* bridges/bridge_native_rtp.c: bridge_native_rtp: Change
-	  local/remote message from debug/2 to verb/4 Change the "Locally
-	  bridged"/"Remotely bridged" messages from dbg/2 to verb/4.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4300/
-
-	* res/res_pjsip_outbound_registration.c, CHANGES:
-	  outbound_registration: Add 'pjsip send register' and update 'send
-	  unregister' The current behavior of 'pjsip send unregister' is to
-	  send the unregister (REGISTER with 0 exp) but let the next
-	  scheduled register proceed normally. I don't think that's a good
-	  idea. If you unregister, it should stay unregistered until you
-	  decide to start registrations again. So this patch just adds a
-	  cancel_registration call to the current unregister_task to cancel
-	  the timer. Of course, now you need a way to start registration
-	  again so I've added a 'pjsip send register' command that
-	  unregisters and cancels any existing registration (the same as
-	  send unregister), then sends an immediate registration and starts
-	  the timer back up again. Both changes also ripple to AMI. There's
-	  a new PJSIPRegister command. There's no harm in calling either
-	  command repeatedly. They don't care about the actual state.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4301/
-
-	* res/res_pjsip/location.c: pjsip cli: Fix sorting of contacts for
-	  'pjsip list contacts' For some reason I was using a hash
-	  container instead of a list to gather the contacts for 'pjsip
-	  list/show contacts' so even though I had a sort function, the
-	  output wasn't sorted. This patch just changes the hash container
-	  to a list container and the contacts now appear sorted in the
-	  CLI. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4305/
-
-2015-01-05 22:49 +0000 [r430200]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* /, main/bridge_basic.c: bridge: avoid leaking channel during
-	  blond transfer pt2 A blond transfer to a failed destination, when
-	  followed by a recall attempt, lead to a leak of the reference to
-	  the destination channel. In addition to correcting the regression
-	  on the previous attempt (r429826) this fixes the leak and two
-	  additional reference leaks on failures of bridge_import.
-	  ASTERISK-24513 #close Review:
-	  https://reviewboard.asterisk.org/r/4302/ ........ Merged
-	  revisions 430199 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2015-01-05 17:56 +0000 [r430179-430181]  Joshua Colp <jcolp@digium.com>
-
-	* CHANGES: pjsip: Document addition of 'PJSIP_AOR' and
-	  'PJSIP_CONTACT' in CHANGES file.
-
-	* funcs/func_pjsip_contact.c (added), res/res_pjsip_session.c,
-	  include/asterisk/res_pjsip.h,
-	  channels/pjsip/dialplan_functions.c,
-	  include/asterisk/res_pjsip_session.h, funcs/func_pjsip_aor.c
-	  (added), res/res_pjsip/location.c: pjsip: Add 'PJSIP_AOR' and
-	  'PJSIP_CONTACT' dialplan functions. The PJSIP_AOR dialplan
-	  function allows inspection of configured AORs including what
-	  contacts are currently bound to them. The PJSIP_CONTACT dialplan
-	  function allows inspection of contacts in existence. These can
-	  include both externally added (by way of registration) or
-	  permanent ones. ASTERISK-24341 Reported by: xrobau Review:
-	  https://reviewboard.asterisk.org/r/4308/
-
-2014-12-29 13:10 +0000 [r430145]  Kinsey Moore <kmoore@digium.com>
-
-	* res/res_pjsip.c: PJSIP: Update transport method documentation
-	  This updates the documentation for the 'method' configuration
-	  option to be more verbose about the behaviors of values
-	  'unspecified' and 'default'. They do exactly the same thing which
-	  is to select the default as defined by PJSIP which is currently
-	  TLSv1. Review: https://reviewboard.asterisk.org/r/4264/
-
-2014-12-24 21:27 +0000 [r430127]  Kevin Harwell <kharwell@digium.com>
-
-	* /, configs/samples/queues.conf.sample: app_queue: Update sample
-	  conf documenation Updated the queues.conf.sample file to
-	  explicitly state which channel queue variables are propagated to.
-	  ASTERISK-24267 Reported by: Mitch Claborn ........ Merged
-	  revisions 430126 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-24 15:26 +0000 [r430083-430092]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_pjsip.c,
-	  contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py
-	  (added): res_pjsip: Backport missing commits for user_eq_phone
-	  This backports the following from trunk, which were missed:
-	  r427257 | file | 2014-11-04 16:31:16 -0600 (Tue, 04 Nov 2014) | 2
-	  lines res_pjsip: Allow + at the beginning of a phone number when
-	  user_eq_phone is enabled. r427259 | file | 2014-11-04 16:51:32
-	  -0600 (Tue, 04 Nov 2014) | 2 lines res_pjsip: Apply the
-	  'user_eq_phone' setting to the To header as well. It also adds
-	  the Alembic script for the option. ASTERISK-24643
-
-	* CHANGES, res/res_pjsip.c, include/asterisk/res_pjsip.h,
-	  res/res_pjsip/config_global.c, res/res_pjsip_keepalive.c (added),
-	  configs/samples/pjsip.conf.sample: res_pjsip_keepalive: Add
-	  runtime configurable keepalive module for connection-oriented
-	  transports. Note that this is backport from trunk of r425825.
-	  This change adds a module which is configurable using the
-	  keep_alive_interval setting in the global section that will send
-	  a CRLF keep alive to all active connection-oriented transports at
-	  the provided interval. This is useful because it can help keep
-	  connections open through NATs. This functionality also exists
-	  within PJSIP but can not be controlled at runtime and requires
-	  recompiling it. Review: https://reviewboard.asterisk.org/r/4084/
-	  ASTERISK-24644 #close
-
-	* res/res_pjsip/pjsip_configuration.c, res/res_pjsip_caller_id.c,
-	  CHANGES, res/res_pjsip.c, include/asterisk/res_pjsip.h:
-	  res_pjsip: Add 'user_eq_phone' option to add a 'user=phone'
-	  parameter when applicable. Note that this is a backport of
-	  r425804 from trunk. This change adds a configuration option which
-	  adds a 'user=phone' parameter if the user portion of the request
-	  URI or the From URI is determined to be a number. Review:
-	  https://reviewboard.asterisk.org/r/4073/ ASTERISK-24643 #close
-
-2014-12-23 23:18 +0000 [r430059-430064]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip/pjsip_options.c: pjsip_options: Fix continued
-	  qualifies after endpoint/aor deletion If you remove an
-	  endpoint/aor from pjsip.conf then do a core reload, qualifies
-	  will continue even though the object are gone. This happens
-	  because nothing clears out the qualify tasks. This patch
-	  unschedules all existing qualify tasks before scheduling new ones
-	  on reload. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4290/
-
-	* tests/test_astobj2.c: test_astobj2: Fix warning for missing
-	  trailing slash in category This patch adds a trailing slash to
-	  the category for this test. No more warning. Tested-by: George
-	  Joseph Review: https://reviewboard.asterisk.org/r/4295/
-
-2014-12-22 21:18 +0000 [r430010-430034]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/bridge_basic.c: DTMF atxfer: Setup recall channels as if the
-	  transferee initiated the call. After the initial DTMF atxfer call
-	  attempt to the transfer target fails to answer during a blonde
-	  transfer, the recall callback channels do not get setup with
-	  information from the initial transferrer channel. As a result,
-	  the recall callback to the transferrer does not have callid,
-	  channel variables, datastores, accountcode, peeraccount, COLP,
-	  and CLID setup. A similar situation happens with the recall
-	  callback to the transfer target but it is less visible. The
-	  recall callback to the transfer target does not have callid,
-	  channel variables, datastores, accountcode, peeraccount, and COLP
-	  setup. * Added missing information to the recall callback
-	  channels before initiating the call. callid, channel variables,
-	  datastores, accountcode, peeraccount, COLP, and CLID * Set callid
-	  of the transferrer channel on the DTMF atxfer controller thread
-	  attended_transfer_monitor_thread(). * Added missing channel
-	  unlocks and props unref to off nominal paths in
-	  attended_transfer_properties_alloc(). ASTERISK-23841 #close
-	  Reported by: Richard Mudgett Review:
-	  https://reviewboard.asterisk.org/r/4259/
-
-	* /, main/logger.c, include/asterisk/_private.h, main/asterisk.c:
-	  queue_log: Post QUEUESTART entry when Asterisk fully boots. The
-	  QUEUESTART log entry has historically acted like a fully booted
-	  event for the queue_log file. When the QUEUESTART entry was
-	  posted to the log was broken by the change made by
-	  ASTERISK-15863. * Made post the QUEUESTART queue_log entry when
-	  Asterisk fully boots. This restores the intent of that log entry
-	  and happens after realtime has had a chance to load. AST-1444
-	  #close Reported by: Denis Martinez Review:
-	  https://reviewboard.asterisk.org/r/4282/ ........ Merged
-	  revisions 430009 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-22 15:40 +0000 [r429983]  Matthew Jordan <mjordan@digium.com>
-
-	* /, channels/chan_sip.c: chan_sip: Send CANCEL via original INVITE
-	  destination even after UPDATE request Given the following
-	  scenario: * Three SIP phones (A, B, C), all communicating via a
-	  proxy with Asterisk * A call is established between A and B. B
-	  performs a SIP attended transfer of A to C. B sets the call on
-	  hold (A is hearing MOH) and dials the extension of C. While phone
-	  C is ringing, B transfers the call (that is, what we typically
-	  call a 'blond transfer'). * When the transfer completes, A hears
-	  the ringing of phone C, while B is idle. In the SIP messaging for
-	  the above scenario, a REFER request is sent to transfer the call.
-	  When "sendrpid=yes" is set in sip.conf, Asterisk may send an
-	  UPDATE request to phone C to update party information. This
-	  update is sent directly to phone C, not through the intervening
-	  proxy. This has the unfortunate side effect of providing route
-	  information, which is then set on the sip_pvt structure for C. If
-	  someone (e.g. B) is trying to get the call back (through a
-	  directed pickup), Asterisk will send a CANCEL request to C.
-	  However, since we have now updated the route set, the CANCEL
-	  request will be sent directly to C and not through the proxy. The
-	  phone ignores this CANCEL according to RFC3261 (Section 9.1).
-	  This patch updates reqprep such that the route is not updated if
-	  an UPDATE request is being sent while the INVITE state is
-	  INV_PROCEEDING or INV_EARLY_MEDIA. This ensures that a subsequent
-	  CANCEL request is still sent to the correct location. Review:
-	  https://reviewboard.asterisk.org/r/4279 ASTERISK-24628 #close
-	  Reported by: Karsten Wemheuer patches: issue.patch uploaded by
-	  Karsten Wemheuer (License 5930) ........ Merged revisions 429982
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-22 00:17 +0000 [r429914]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_phoneprov_provider.c:
-	  res_pjsip_phoneprovi_provider: Fix reload Reloading wasn't
-	  working correctly because on a reload, the sorcery apply handler
-	  was never being called for unchanged users. So, instead of using
-	  an apply handler, I'm now iterating over all users. Works much
-	  more reliably. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4288/
-
-2014-12-20 20:57 +0000 [r429894]  Joshua Colp <jcolp@digium.com>
-
-	* main/named_acl.c, /: acl: Fix reloading of configuration if
-	  configuration file does not exist at startup. The named ACL code
-	  incorrectly destroyed the config options information if loading
-	  of the configuration file failed at startup. This would result in
-	  reloading also failing even if a valid configuration file was put
-	  in place. ASTERISK-23733 #close Reported by: Richard Kenner
-	  ........ Merged revisions 429893 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-19 20:54 +0000 [r429829-429868]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, res/res_http_websocket.c: res_http_websocket.c: Fix incorrect
-	  use of sizeof in ast_websocket_write(). This won't fix the
-	  reported issue but it is an incorrect use of sizeof.
-	  ASTERISK-24566 Reported by: Badalian Vyacheslav ........ Merged
-	  revisions 429867 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* channels/chan_dahdi.c, /: chan_dahdi: Don't ignore setvar when
-	  using configuration section scheme. When the configuration
-	  section scheme of chan_dahdi.conf is used (keyword dahdichan
-	  instead of channel) all setvar= options are completely ignored.
-	  No variable defined this way appears in the created DAHDI
-	  channels. * Move the clearing of setvar values to after the
-	  deferred processing of dahdichan. AST-1378 #close Reported by:
-	  Guenther Kelleter Patch by: Guenther Kelleter ........ Merged
-	  revisions 429825 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-19 17:26 +0000 [r429827]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* /, main/bridge_basic.c: bridge: avoid leaking channel during
-	  blond transfer After a blond transfer (start attended and hang
-	  up) to a destination that also hangs up without answer, the
-	  Local;1 channel was leaked and would show up on core show
-	  channels. This was happening because the attended state
-	  blond_nonfinal_enter() resetting the props->transfer_target to
-	  null while releasing it's own reference, which would later
-	  prevent props from releasing another reference during
-	  destruction. The change made here is simply to not assign the
-	  target to NULL. ASTERISK-24513 #close Reported by: Mark Michelson
-	  Review: https://reviewboard.asterisk.org/r/4262/ ........ Merged
-	  revisions 429826 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-18 22:38 +0000 [r429784-429805]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_rtp_asterisk.c, channels/chan_dahdi.c, /: chan_dahdi.c,
-	  res_rtp_asterisk.c: Change some spammy debug messages to level 5.
-	  ASTERISK-24337 #close Reported by: Rusty Newton ........ Merged
-	  revisions 429804 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* UPGRADE.txt, channels/sig_analog.c, /: chan_dahdi: Populate
-	  CALLERID(ani2) for incoming calls in featdmf signaling mode. For
-	  the featdmf signaling mode the incoming MF Caller-ID information
-	  is formatted as follows:
-	  *${CALLERID(ani2)}${CALLERID(ani)}#*${EXTEN}# Rather than
-	  discarding the ani2 digits, populate the CALLERID(ani2) value
-	  with what is received instead. AST-1368 #close Reported by: Denis
-	  Martinez Patches: extract_ani2_for_featdmf_v11.patch (license
-	  #5621) patch uploaded by Richard Mudgett ........ Merged
-	  revisions 429783 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-18 15:50 +0000 [r429763]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip_sdp_rtp.c: res_pjsip_sdp_rtp: wrong bridge chosen
-	  when the DTMF mode is not compatible A native rtp bridge was
-	  being chosen (it shouldn't have been) when using two pjsip
-	  channels with incompatible DTMF modes. This patch sets the rtp
-	  instance property, AST_RTP_PROPERTY_DTMF, for the appropriate
-	  DTMF mode(s) for pjsip. It was not being set before, meaning all
-	  DTMF modes for pjsip were being treated as compatible, thus
-	  native bridging would be chosen as the bridge type when it
-	  shouldn't have been. ASTERISK-24459 #close Reported by: Yaniv
-	  Simhi Review: https://reviewboard.asterisk.org/r/4265/
-
-2014-12-18 15:34 +0000 [r429739-429761]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip_outbound_registration.c: Prevent potential infinite
-	  outbound authentication loops in registration. Prior to this
-	  patch, Asterisk would always respond to 401 responses to
-	  registration attempts by trying to provide a registration with
-	  authentication credentials. Even if subsequent attempts were
-	  rejected with 401 responses, Asterisk would continue this
-	  behavior. If authentication credentials were incorrect, this
-	  could continue forever. With this patch, we keep track of whether
-	  we have attempted authentication on an outbound registration
-	  attempt. If we already have, we don not try again until the next
-	  attempt. This prevents the infinite loop scenario. Review:
-	  https://reviewboard.asterisk.org/r/4273
-
-	* main/manager.c: Prevent possible race condition on dual redirect
-	  of channels in the same bridge. The
-	  AST_FLAG_BRIDGE_DUAL_REDIRECT_WAIT flag was created to prevent
-	  bridges from prematurely acting on orphaned channels in bridges.
-	  The problem with the AMI redirect action was that it was setting
-	  this flag on channels based on the presence of a PBX, not whether
-	  the channel was in a bridge. Whether a channel has a PBX is
-	  irrelevant, so the condition has been altered to check if the
-	  channel is in a bridge. ASTERISK-24536 #close Reported by Niklas
-	  Larsson Review: https://reviewboard.asterisk.org/r/4268
-
-	* channels/pjsip/dialplan_functions.c: Ensure the correct value is
-	  returned for CHANNEL(pjsip, secure) Prior to this patch, we were
-	  using the PJSIP dialog's secure flag to determine if a secure
-	  transport was being used. Unfortunately, the dialog's secure flag
-	  was only set if a SIPS URI were in use, as required by RFC 3261
-	  sections 12.1.1 and 12.1.2. What we're interested in is not
-	  dialog security, but transport security. This code change
-	  switches to a model where we use the dialog's target URI to
-	  determine what transport would be used to communicate, and then
-	  check if that transport is secure. AST-1450 #close Reported by
-	  John Bigelow Review: https://reviewboard.asterisk.org/r/4277
-
-2014-12-18 00:10 +0000 [r429699-429719]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_config_wizard.c: res_pjsip_config_wizard: fix
-	  unload SEGV If certain pjsip modules aren't loaded, the wizard
-	  causes a SEGV when it unloads. Added a check for the presense of
-	  the object type wizard before trying to clean it up. Tested-by:
-	  George Joseph
-
-	* res/res_pjsip_config_wizard.c: res_pjsip_config_wizard: Change
-	  FILEUNCHANGED config_load2 flag determination The module now
-	  applies the FILEUNCHANGED flag when both reloaded is specified
-	  AND there's no last_config for the object type. Tested-by: George
-	  Joseph Review: https://reviewboard.asterisk.org/r/4276/
-
-2014-12-17 09:54 +0000 [r429675]  Walter Doekes <walter+asterisk@wjd.nu>
-
-	* addons/ooh323c/src/printHandler.c, apps/app_adsiprog.c,
-	  channels/chan_unistim.c, main/udptl.c, res/res_rtp_asterisk.c, /,
-	  channels/chan_sip.c, channels/vcodecs.c, res/res_crypto.c,
-	  utils/astman.c, utils/smsq.c, main/utils.c, pbx/dundi-parser.c,
-	  apps/app_getcpeid.c, channels/chan_iax2.c, channels/sig_pri.c,
-	  res/res_pktccops.c, main/loader.c, channels/iax2/parser.c,
-	  main/uuid.c, main/manager.c, channels/chan_misdn.c,
-	  apps/app_osplookup.c, channels/misdn/ie.c, main/http.c,
-	  apps/app_sms.c: Fix printf problems with high ascii characters
-	  after r413586 (1.8). In r413586 (1.8) various casts were added to
-	  silence gcc 4.10 warnings. Those fixes included things like: -out
-	  += sprintf(out, "%%%02X", (unsigned char) *ptr); +out +=
-	  sprintf(out, "%%%02X", (unsigned) *ptr); That works for low ascii
-	  characters, but for the high range that yields e.g. FFFFFFC3 when
-	  C3 is expected. This changeset: - fixes those casts to use the
-	  'hh' unsigned char modifier instead - consistently uses %02x
-	  instead of %2.2x (or other non-standard usage) - adds a few 'h'
-	  modifiers in various places - fixes a 'replcaes' typo -
-	  dev/urandon typo (in 13+ patch) Review:
-	  https://reviewboard.asterisk.org/r/4263/ ASTERISK-24619 #close
-	  Reported by: Stefan27 (on IRC) ........ Merged revisions 429673
-	  from http://svn.asterisk.org/svn/asterisk/branches/11 ........
-	  Merged revisions 429674 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-16 17:53 +0000 [r429653]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_config_wizard.c: res_pjsip_config_wizard: fix test
-	  breakage Fix test breakage caused by not checking for res_pjsip
-	  before calling ast_sip_get_sorcery. Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4269/
 
-2014-12-16 16:38 +0000 [r429612-429633]  Joshua Colp <jcolp@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420537 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* /, channels/chan_sip.c: chan_sip: Allow T.38 switch-over when
-	  SRTP is in use. Previously when SRTP was enabled on a channel it
-	  was not possible to switch to T.38 as no crypto attributes would
-	  be present. This change makes it so it is now possible. If a T.38
-	  re-invite comes in SRTP is terminated since in practice you can't
-	  encrypt a UDPTL stream. Now... if we were doing T.38 over RTP
-	  (which does exist) then we'd have a chance but almost nobody does
-	  that so here we are. ASTERISK-24449 #close Reported by: Andreas
-	  Steinmetz patches: udptl-ignore-srtp-v2.patch submitted by
-	  Andreas Steinmetz (license 6523) ........ Merged revisions 429632
-	  from http://svn.asterisk.org/svn/asterisk/branches/11
-
-	* res/res_pjsip_t38.c: res_pjsip_t38: Fix T.38 failure when peer
-	  reinvites immediately. If a remote endpoint reinvites to T.38
-	  immediately the state machine will go into a peer reinvite state.
-	  If a T.38 capable application (such as ReceiveFax) queries it
-	  will receive this state. Normally the application will then
-	  indicate so that the channel driver will queue up the T.38 offer
-	  previously received. Once it receives this offer the application
-	  will act normally and negotiate. The res_pjsip_t38 module
-	  incorrectly partially squashed this indication. This would cause
-	  the application to think the request had failed when in reality
-	  it had actually worked. This change makes it so that no T.38
-	  control frames (or indications) are squashed.
-
-2014-12-15 17:07 +0000 [r429592]  George Joseph <george.joseph@fairview5.com>
-
-	* res/res_pjsip_phoneprov_provider.c,
-	  configs/samples/pjsip_wizard.conf.sample (added), CHANGES,
-	  res/res_pjsip_config_wizard.c (added): res_pjsip_config_wizard:
-	  Allow streamlined config of common pjsip scenarios
-	  res_pjsip_config_wizard ------------------ * This is a new module
-	  that adds streamlined configuration capability for chan_pjsip.
-	  It's targetted at users who have lots of basic configuration
-	  scenarios like 'phone' or 'agent' or 'trunk'. Additional
-	  information can be found in the sample configuration file at
-	  config/samples/pjsip_wizard.conf.sample. Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4190/
+2014-08-08 07:33 +0000 [008c1ad9bf]  Matt Jordan <mjordan@digium.com>
 
-2014-12-15 15:36 +0000 [r429571]  Mark Michelson <mmichelson@digium.com>
+	* main/message: remove debug message
+	  ........
 
-	* res/res_pjsip_pubsub.c: Activate persistent subscriptions when
-	  they are recreated. Prior to this change, recreating persistent
-	  subscriptions would create the subscription but would not
-	  activate it. This led to subscriptions being listed in the "NULL"
-	  state by diagnostics and not sending NOTIFYs when expected.
-	  Review: https://reviewboard.asterisk.org/r/4261
+	  Merged revisions 420533 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2014-12-12 23:54 +0000 [r429542]  George Joseph <george.joseph@fairview5.com>
+	  Merged revisions 420534 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* main/manager.c, include/asterisk/module.h,
-	  include/asterisk/_private.h: loader: Move definition of
-	  ast_module_reload from _private.h to module.h No functionality
-	  change. Just move the definition of ast_module_reload from
-	  _private.h to module.h so it can be public. Also removed the
-	  include of _private.h from manager.c since ast_module_load was
-	  the only reason for including it. Tested-by: George Joseph
-	  Review: https://reviewboard.asterisk.org/r/4251/
 
-2014-12-12 23:40 +0000 [r429540]  Richard Mudgett <rmudgett@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420535 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* main/lock.c, /, include/asterisk/lock.h: DEBUG_THREADS: Fix
-	  regression and lock tracking initialization problems. This patch
-	  started with David Lee's patch at
-	  https://reviewboard.asterisk.org/r/2826/ and includes a
-	  regression fix introduced by the ASTERISK-22455 patch. The
-	  initialization of a mutex's lock tracking structure was not
-	  protected in a critical section. This is fine for any mutex that
-	  is explicitly initialized, but a static mutex may have its lock
-	  tracking double initialized if multiple threads attempt the first
-	  lock simultaneously. * Added a global mutex to properly serialize
-	  initialization of the lock tracking structure. The painful global
-	  lock can be mitigated by adding a double checked lock flag as
-	  discussed on the original review request. * Defer lock tracking
-	  initialization until first use. * Don't be "helpful" and
-	  initialize an uninitialized lock when DEBUG_THREADS is enabled.
-	  Debug code is not supposed to fix or change normal code behavior.
-	  We don't need a lock initialization race that would force a
-	  re-setup of lock tracking. Lock tracking already handles
-	  initialization on first use. * Properly handle allocation
-	  failures of the lock tracking structure. * No need to initialize
-	  tracking data in __ast_pthread_mutex_destroy() just to turn
-	  around and destroy it. The regression introduced by
-	  ASTERISK-22455 is the result of manipulating a pthread_mutex_t
-	  struct outside of the pthread library code. The pthread_mutex_t
-	  struct seems to have a global linked list pointer member that can
-	  get changed by other threads. Therefore, saving and restoring the
-	  contents of a pthread_mutex_t struct is a bad thing. Thanks to
-	  Thomas Airmont for finding this obscure regression. * Don't
-	  overwrite the struct ast_lock_track.reentr_mutex member to
-	  restore tracking data in __ast_cond_wait() and
-	  __ast_cond_timedwait(). The pthread_mutex_t struct must be
-	  treated as a read-only opaque variable. Miscellaneous other items
-	  fixed by this patch: * Match ast_suspend_lock_info() with
-	  ast_restore_lock_info() in __ast_cond_timedwait(). * Made some
-	  uninitialized lock sanity checks return EINVAL and try a
-	  DO_THREAD_CRASH. * Fix bad canlog initialization expressions.
-	  ASTERISK-24614 #close Reported by: Thomas Airmont Review:
-	  https://reviewboard.asterisk.org/r/4247/ Review:
-	  https://reviewboard.asterisk.org/r/2826/ ........ Merged
-	  revisions 429539 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11
-
-2014-12-12 22:53 +0000 [r429518-429519]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_agi.c: res/res_agi: Make Verbose message for 'stream
-	  file' match other playbacks The Verbose message displayed when a
-	  file is played back via 'stream file' was formatted differently
-	  than other playbacks: * It didn't include the channel name * It
-	  didn't include the channel language It does, however, include the
-	  playback offset as well as any escape digits. That information
-	  was kept; however, this patch updates the formatting to more
-	  closely match the Verbose messages displayed when a file is
-	  played back by 'control stream file', Playback, ControlPlayback,
-	  or any other file playback operation.
-
-	* /: Add 11 merge properties
-
-2014-12-12 16:57 +0000 [r429497]  Joshua Colp <jcolp@digium.com>
-
-	* main/format.c, main/codec.c, include/asterisk/format.h: media:
-	  Fix crash when determining sample count of a frame during
-	  shutdown. When shutting down Asterisk the codecs are cleaned up.
-	  As a result anything attempting to get a codec based on ID or
-	  details will find that no codec exists. This currently occurs
-	  when determining the sample count of a frame. This code did not
-	  take this situation into account. This change fixes this by
-	  getting the codec directly from the format and eliminates the
-	  lookup. This is both faster and also provides a guarantee that
-	  the codec will exist and will be valid. ASTERISK-24604 #close
-	  Reported by: Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4260/
-
-2014-12-12 15:30 +0000 [r429477]  Kevin Harwell <kharwell@digium.com>
-
-	* channels/chan_pjsip.c: chan_pjsip: Race between channel answer
-	  and bridge setup when using direct media When direct media is
-	  enabled and a pjsip channel is answered a race would occur
-	  between the handling of the answer and bridge setup. Sometimes
-	  the media negotiation would take place after the native bridge
-	  was setup. This resulted in a NULL media address, which in turn
-	  resulted in Asterisk using its address as the remote media
-	  address when sending a reinvite. This patch makes the chan_pjsip
-	  answer handler synchronous thus alleviating the race condition
-	  (the bridge won't start setting things up until after it
-	  returns). ASTERISK-24563 #close Reported by: Steve Pitts Review:
-	  https://reviewboard.asterisk.org/r/4257/
-
-2014-12-12 15:00 +0000 [r429457]  David M. Lee <dlee@digium.com>
-
-	* res/res_pjsip_outbound_publish.c: Fix crash for sorcery
-	  misconfigs res_pjsip_outbound_publish was missing the
-	  CHECK_PJSIP_MODULE_LOADED() call in load_module, and would crash
-	  with a segfault if res_pjsip declined to load. Review:
-	  https://reviewboard.asterisk.org/r/4258/
-
-2014-12-12 14:12 +0000 [r429430-429433]  Kinsey Moore <kmoore@digium.com>
-
-	* /, res/res_pjsip_sdp_rtp.c: PJSIP: Allow use of 'inactive'
-	  streams for hold This allows use of the 'inactive' stream
-	  direction identifier to be used for hold where 'sendonly' is
-	  normally used. Some Seimens phones use 'inactive' and this change
-	  allows music on hold to operate properly. Review:
-	  https://reviewboard.asterisk.org/r/4252/ Reported by: Steve Pitts
-	  ........ Merged revisions 429432 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* res/res_sorcery_config.c, /: Sorcery: Log when old config remains
-	  in use This adds a log message notifying the user that a stale
-	  configuration is in place upon reload when a config object fails
-	  to load. This situation can end up causing confusion when the
-	  object failed to load but exists from a previous config load
-	  especially when the old config is significantly different from
-	  the new config. Review: https://reviewboard.asterisk.org/r/4250/
-	  Reported by: Thomas Thompson ........ Merged revisions 429429
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-12 13:05 +0000 [r429407-429409]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_session.exports.in, channels/chan_pjsip.c,
-	  res/res_pjsip_session.c, include/asterisk/res_pjsip_session.h:
-	  res_pjsip_session: Delay sending BYE if a re-INVITE transaction
-	  is in progress. Given the scenario where a PJSIP channel is in a
-	  native RTP bridge with direct media and the channel is then hung
-	  up the code will currently re-INVITE the channel back to Asterisk
-	  and send a BYE at the same time. Many SIP implementations dislike
-	  this greatly. This change makes it so that if a re-INVITE
-	  transaction is in progress the BYE is queued to occur after the
-	  completion of the transaction (be it through normal means or a
-	  timeout). Review: https://reviewboard.asterisk.org/r/4248/
-
-	* res/res_pjsip_session.c: res_pjsip_session: Fix issue where a
-	  declined media stream in a re-INVITE would fail SDP negotiation.
-	  In the past the SDP negotiation within res_pjsip_session was made
-	  more tolerant of certain situations. The only case where SDP
-	  negotiation will fail is when a major error occurs during
-	  negotiation. Receiving an already declined media stream is not
-	  considered a major error. When producing the local SDP the logic
-	  took this into account so on the initial INVITE the declined
-	  media stream did not cause an SDP negotiation failure.
-	  Unfortunately the logic for handling media streams with a handler
-	  did not mirror this logic and considered an already declined
-	  media stream an error and thus failed the SDP negotiation. This
-	  change makes the logic between both situations match so only
-	  under major errors will the SDP negotiation fail. ASTERISK-24607
-	  #close Reported by: Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4254/
-
-2014-12-11 20:31 +0000 [r429387]  Kevin Harwell <kharwell@digium.com>
-
-	* CHANGES: ARI/AMI: Include language in standard channel snapshot
-	  output The CHANGES verbiage for the "language" addition had been
-	  put under the wrong release. This moves it to be under 13.1 to
-	  13.2 changes. ASTERISK-24553 Reported by: Matt Jordan
-
-2014-12-11 17:21 +0000 [r429352-429379]  Kinsey Moore <kmoore@digium.com>
-
-	* /: Recorded merge of revisions 429378 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12 ........ Fix
-	  incorrect patch applied in r429354 The patch that was applied was
-	  another pending patch. This swaps them out.
-
-	* /: Recorded merge of revisions 429354 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12 ........ Stasis:
-	  Update unittest for channel snapshots This adjusts the unit test
-	  for channel snapshots to take the new language key into account.
-
-	* tests/test_stasis_channels.c: Stasis: Update unittest for channel
-	  snapshots This adjusts the unit test for channel snapshots to
-	  take the new language key into account.
-
-2014-12-10 15:42 +0000 [r429326]  Kevin Harwell <kharwell@digium.com>
-
-	* /, CHANGES: ARI/AMI: Include language in standard channel
-	  snapshot output Adding information about including "language" in
-	  the standard channel snapshot output to the CHANGES file. Note
-	  the actual source changes have already been previously committed.
-	  ASTERISK-24553 Reported by: Matt Jordan ........ Merged revisions
-	  429325 from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-10 13:34 +0000 [r429273]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_http_websocket.c, res/res_pjsip_transport_websocket.c, /,
-	  channels/chan_sip.c: res_http_websocket: Fix crash due to double
-	  freeing memory when receiving a payload length of zero. Frames
-	  with a payload length of 0 were incorrectly handled in
-	  res_http_websocket. Provided a frame with a payload had been
-	  received prior it was possible for a double free to occur. The
-	  realloc operation would succeed (thus freeing the payload) but be
-	  treated as an error. When the session was then torn down the
-	  payload would be freed again causing a crash. The read function
-	  now takes this into account. This change also fixes assumptions
-	  made by users of res_http_websocket. There is no guarantee that a
-	  frame received from it will be NULL terminated. ASTERISK-24472
-	  #close Reported by: Badalian Vyacheslav Review:
-	  https://reviewboard.asterisk.org/r/4220/ Review:
-	  https://reviewboard.asterisk.org/r/4219/ ........ Merged
-	  revisions 429270 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 429272 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-10 13:14 +0000 [r429246]  Kinsey Moore <kmoore@digium.com>
-
-	* /, res/res_pjsip/pjsip_options.c: PJSIP: Fix assert on initial
-	  mass qualify This fixes the MWI test regressions caused by
-	  r429127 and ensures that contacts have non-zero qualify_frequency
-	  before attempting scheduling. ........ Merged revisions 429245
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-09 20:46 +0000 [r429223]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* main/asterisk.c: core: avoid possible asterisk -r crash from long
-	  id When connecting to the remote console, an id string is first
-	  provided that consts of the hostname, pid, and version. This is
-	  parsed by the remote instance using a buffer that may be too
-	  short, and can allow a buffer overrun because it is not
-	  terminated. This patch adds termination and a larger buffer.
-	  Review: https://reviewboard.asterisk.org/r/4182/
+2014-08-07 22:07 +0000 [c94fef6f36]  Kinsey Moore <kmoore@digium.com>
 
-2014-12-09 20:19 +0000 [r429175-429206]  Kevin Harwell <kharwell@digium.com>
+	* CEL: Update unit tests for additional information
 
-	* res/ari/ari_model_validators.h, /, main/stasis_channels.c,
-	  rest-api/api-docs/channels.json, res/ari/ari_model_validators.c,
-	  main/manager_channels.c: ARI/AMI: Include language in standard
-	  channel snapshot output The channel "language" was already part
-	  of a channel snapshot, however is was not sent out over AMI or
-	  ARI. This patch makes it so the channel "language" is included in
-	  the appropriate AMI or ARI events. ASTERISK-24553 #close Reported
-	  by: Matt Jordan Review: https://reviewboard.asterisk.org/r/4245/
-	  ........ Merged revisions 429204 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* include/asterisk/rtp_engine.h, res/res_rtp_asterisk.c,
-	  main/rtp_engine.c, /, channels/chan_sip.c: Direct Media calls
-	  within private network sometimes get one way audio When endpoints
-	  with direct_media enabled, behind a firewall (Asterisk on a
-	  separate network) and were bridged sometimes Asterisk would send
-	  the ip address of the firewall in the sdp to one of the phones in
-	  the reinvite resulting in one way audio. When sending the
-	  reinvite Asterisk will retrieve the media address from the
-	  associated rtp instance, but if frames were being read this can
-	  be overwritten with another address (in this case the
-	  firewall's). This patch ensures that Asterisk uses the original
-	  device address when using direct media. ASTERISK-24563 Reported
-	  by: Steve Pitts Review: https://reviewboard.asterisk.org/r/4216/
-	  ........ Merged revisions 429195 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* res/res_pjsip_outbound_publish.c: res_pjsip_outbound_publish:
-	  stack overflow when using non-default sorcery wizard When using a
-	  non-default sorcery wizard (in this instance realtime) for
-	  outbound publishes Asterisk will crash after a stack overflow
-	  occurs due to the code infinitely recursing. The fix entails
-	  removing the outbound publish state dependency from the outbound
-	  publish sorcery object and instead keeping an in memory container
-	  that can be used to lookup the state when needed. ASTERISK-24514
-	  #close Reported by: Mark Michelson Review:
-	  https://reviewboard.asterisk.org/r/4178/
-
-2014-12-09 15:44 +0000 [r429153]  Joshua Colp <jcolp@digium.com>
-
-	* res/ari/resource_channels.h, rest-api/api-docs/channels.json,
-	  res/ari/resource_channels.c, CHANGES, res/res_ari_channels.c:
-	  ari: Add support for specifying an originator channel when
-	  originating. If an originator channel is specified when
-	  originating a channel the linked ID of it will be applied to the
-	  newly originated outgoing channel. This allows an association to
-	  be made between the two so it is known that the originator has
-	  dialed the originated channel. ASTERISK-24552 #close Reported by:
-	  Matt Jordan Review: https://reviewboard.asterisk.org/r/4243/
-
-2014-12-09 14:00 +0000 [r429128]  Kinsey Moore <kmoore@digium.com>
-
-	* /, res/res_pjsip/pjsip_options.c: PJSIP: Stagger outbound
-	  qualifies This change staggers initiation of outbound qualify
-	  (OPTIONS) attempts to reduce instantaneous server load and
-	  prevent network congestion. Review:
-	  https://reviewboard.asterisk.org/r/4246/ ASTERISK-24342 #close
-	  Reported by: Richard Mudgett ........ Merged revisions 429127
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-15  Asterisk Development Team <asteriskteam@digium.com>
-
-	* Asterisk 13.1.0 Released.
-
-2014-12-10  Asterisk Development Team <asteriskteam@digium.com>
-
-	* Asterisk 13.1.0-rc2 Released.
-
-	* AST-2014-019: Fix crash when receiving a WebSocket packet with a
-	  payload length of zero.
-
-	  Frames with a payload length of 0 were incorrectly handled in
-	  res_http_websocket. Provided a frame with a payload had been
-	  received prior it was possible for a double free to occur. The
-	  realloc operation would succeed (thus freeing the payload) but be
-	  treated as an error. When the session was then torn down the payload
-	  would be freed again causing a crash. The read function now takes
-	  this into account.
-
-	  This change also fixes assumptions made by users of
-	  res_http_websocket. There is no guarantee that a frame received from
-	  it will be NULL terminated.
+	  This updates the CEL unit tests for the new information contained in
+	  the attended transfer CEL extra field.
+	  ........
 
-	  ASTERISK-24472 #close
-	  Reported by: Badalian Vyacheslav
+	  Merged revisions 420513 from http://svn.asterisk.org/svn/asterisk/branches/12
+	  ........
 
-2014-12-08  Asterisk Development Team <asteriskteam@digium.com>
+	  Merged revisions 420514 from http://svn.asterisk.org/svn/asterisk/branches/13
 
-	* Asterisk 13.1.0-rc1 Released.
 
-2014-12-08 16:53 +0000 [r429091]  Matthew Jordan <mjordan@digium.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420515 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* rest-api/api-docs/playbacks.json, UPGRADE.txt,
-	  rest-api/api-docs/channels.json, rest-api/api-docs/sounds.json,
-	  rest-api/resources.json, CHANGES, include/asterisk/manager.h,
-	  rest-api/api-docs/bridges.json,
-	  rest-api/api-docs/recordings.json,
-	  rest-api/api-docs/deviceStates.json,
-	  rest-api/api-docs/endpoints.json,
-	  rest-api/api-docs/mailboxes.json, rest-api/api-docs/events.json,
-	  rest-api/api-docs/asterisk.json,
-	  rest-api/api-docs/applications.json: AMI/ARI: Update version to
-	  2.6.0/1.6.0 respectively for new features AMI/ARI are getting a
-	  few enhancements in the next release of Asterisk 13. Per semantic
-	  versioning, that warrants a bump in the minor version number, as
-	  it reflects a backwards compatible change. Hence, this commit.
-
-2014-12-08 16:41 +0000 [r429064-429089]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip_session.c: Fix a crash that would occur when
-	  receiving a 491 response to a reinvite. The reviewboard
-	  description does a fine job of summarizing this, so here it is: A
-	  reporter discovered that Asterisk would crash when attempting to
-	  retransmit a reinvite that had previously received a 491
-	  response. The crash occurred because a pjsip_tx_data structure
-	  was being saved for reuse, but its reference count was not being
-	  increased. The result was that the pjsip_tx_data was being freed
-	  before we were actually done with it. When we attempted to re-use
-	  the structure when re-sending the reinvite, Asterisk would crash.
-	  The fix implemented here is not to try holding onto the
-	  pjsip_tx_data at all. Instead, when we reschedule sending the
-	  reinvite, we create a brand new pjsip_tx_data and send that
-	  instead. Because of this change, there is no need for an
-	  ast_sip_session_delayed_request structure to have a pjsip_tx_data
-	  on it any more. So any code referencing its use has been removed.
-	  When this initial fix was introduced, I encountered a second
-	  crash when processing a subsequent 200 OK on a rescheduled
-	  reinvite. The reason was that when rescheduling the reinvite, we
-	  gave the wrong location for a response callback. This has been
-	  fixed in this patch as well. ASTERISK-24556 #close Reported by
-	  Abhay Gupta Review: https://reviewboard.asterisk.org/r/4233
-
-	* main/stasis_channels.c, CHANGES, res/ari/ari_model_validators.c,
-	  main/manager_channels.c, main/channel.c,
-	  res/ari/ari_model_validators.h,
-	  include/asterisk/stasis_channels.h,
-	  rest-api/api-docs/events.json, res/stasis/app.c: Add new AMI and
-	  ARI events for connected line changes on a channel. The AMI event
-	  is called NewConnectedLine and the ARI event is called
-	  ChannelConnectedLine. ASTERISK-24554 #close Reported by Matt
-	  Jordan Review: https://reviewboard.asterisk.org/r/4231
-
-2014-12-08 15:43 +0000 [r429062]  Kinsey Moore <kmoore@digium.com>
-
-	* /, res/stasis/app.c, main/channel_internal_api.c,
-	  res/stasis/stasis_bridge.c, res/stasis/app.h,
-	  include/asterisk/channel.h, res/res_stasis.c, main/channel.c:
-	  Stasis: Fix StasisStart/End order and missing events This
-	  corrects several bugs that currently exist in the stasis
-	  application code. * After a masquerade, the resulting channels
-	  have channel topics that do not match their uniqueids **
-	  Masquerades now swap channel topics appropriately * StasisStart
-	  and StasisEnd messages are leaked to observer applications due to
-	  being published on channel topics ** StasisStart and StasisEnd
-	  publishing is now properly restricted to controlling apps via app
-	  topics * Race conditions exist where StasisStart and StasisEnd
-	  messages due to a masquerade may be received out of order due to
-	  being published on different topics ** These messages are now
-	  published directly on the app topic so this is now a non-issue *
-	  StasisEnds are sometimes missing when sent due to masquerades and
-	  bridge swaps into and out of Stasis() ** This was due to
-	  StasisEnd processing adjusting message-sent flags after Stasis()
-	  had already exited and Stasis() had been re-entered ** This was
-	  corrected by adjusting these flags prior to sending the message
-	  while the initial Stasis() application was still shutting down
-	  Review: https://reviewboard.asterisk.org/r/4213/ ASTERISK-24537
-	  #close Reported by: Matt DiMeo ........ Merged revisions 429061
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-06 18:16 +0000 [r429029-429033]  Matthew Jordan <mjordan@digium.com>
-
-	* res/res_monitor.c, /: res/res_monitor: Reset in/out sample counts
-	  on Monitor start When repeatedly starting/stopping a Monitor on a
-	  channel, the accumulated in/out sample counts are never reset to
-	  0. This can cause inadvertent jumps in the recordings, as the
-	  code in the channel core will determine incorrectly that a jump
-	  in the recorded file position should occur. Setting the sample
-	  counts to 0 simply reflects the initial state a Monitor should be
-	  in when it is started, as this is the initial count that would be
-	  on the channels at that time. ASTERISK-24573 #close Reported by:
-	  Nuno Borges patches: 24573.patch uploaded by Nuno Borges (License
-	  6116) ........ Merged revisions 429031 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 429032 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, apps/app_meetme.c: apps/app_meetme: Apply default values on
-	  initial load with no config file When the app_meetme module is
-	  loaded without its configuration file, the module settings aren't
-	  initialized. In particular, this impacts the use of logging
-	  realtime members. This patch guarantees that we always set the
-	  default module settings on initial load. Review:
-	  https://reviewboard.asterisk.org/r/4242/ ASTERISK-24572 #close
-	  Reported by: Nuno Borges patches: 24572.patch uploaded by Nuno
-	  Borges (License 6116) ........ Merged revisions 429027 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 429028 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-05 17:06 +0000 [r429000]  George Joseph <george.joseph@fairview5.com>
-
-	* tests/test_sorcery.c, main/sorcery.c, include/asterisk/test.h, /,
-	  include/asterisk/sorcery.h: sorcery: Add additional observer
-	  capabilities. Add new global, instance and wizard observers.
-	  instance_created wizard_registered wizard_unregistered
-	  instance_destroying instance_loading instance_loaded
-	  wizard_mapped object_type_registered object_type_loading
-	  object_type_loaded wizard_loading wizard_loaded Tested-by: George
-	  Joseph Review: https://reviewboard.asterisk.org/r/4215/ ........
-	  Merged revisions 428999 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-04 17:13 +0000 [r428865-428973]  Matthew Jordan <mjordan@digium.com>
-
-	* /, main/test.c: main/test: Fix compilation issue on 32-bit
-	  systems On a 32-bit system, a type of intmax_t will result in a
-	  compilation warning when formatted as a 'long int'. Use the
-	  format specifier of %jd (which was what was used originally in
-	  manager.c) to format the JSON extracted integer on both
-	  32-/64-bit systems. ........ Merged revisions 428972 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/manager.c, /, main/test.c: main/test: Fix race condition
-	  between AMI topic and Test Suite topic This patch fixes a race
-	  condition between the raising of test AMI events (which drive
-	  many tests in the Asterisk Test Suite) and other AMI events.
-	  Prior to this patch, the Stasis messages published to the test
-	  topic were not forwarded to the AMI topic. Instead, the code in
-	  manager had a dedicated handler for test messages that was
-	  independent of the topics forwarded to the AMI topic. This
-	  results in no synchronization between the test messages and the
-	  rest of the Stasis messages published out over AMI. In some test
-	  with very tight timing constraints, this can result in out of
-	  order messages and spurious test failures. Properly forwarding
-	  the Test Suite topic to the AMI topic ensures that the messages
-	  are synchronized properly. This patch does that, and moves the
-	  message handling to the Stasis definition of the Test Suite
-	  message in test.c as well. Review:
-	  https://reviewboard.asterisk.org/r/4221/ ........ Merged
-	  revisions 428945 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* tests/test_cel.c, /: tests/test_cel: Add
-	  test_cel_attended_transfer_bridges_link to racey tests Despite
-	  failing less often, the ordering of the ATTENDEDTRANSFER event
-	  and the BRIDGE_EXIT event for the Alice and David channels is not
-	  defined. This makes the test still fail. ........ Merged
-	  revisions 428918 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* tests/test_cel.c, /: tests/test_cel: Fix CEL unit test failures
-	  caused by attended transfer changes When the publication of
-	  attended transfer messages were pushed to another thread, some
-	  subtle race conditions were introduced with the CEL unit tests.
-	  This patch fixes one of them, and pushes the other to
-	  ASTERISK-22367, which already exists to fix another bouncy CEL
-	  unit test. In particular, this patch fixes the
-	  test_cel_attended_transfer_bridges_link test, and defers the
-	  test_cel_attended_transfer_bridges_swap test to the
-	  aforementioned JIRA issue. ASTERISK-22367 ........ Merged
-	  revisions 428891 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* apps/app_voicemail.c, /: apps/app_voicemail: Fix crash with IMAP
-	  when streams are opened simultaneously The UW IMAP library is
-	  instrinsically not thread-safe, and relies upon higher level
-	  applications to guarantee thread safety. For the most part, this
-	  is provided by the vms object, which provides locking for
-	  individual streams. Unfortunately, this is not sufficient for
-	  calls to mail_open which create the IMAP stream. mail_open can,
-	  on some systems, call into a UW IMAP specific function for
-	  determining the address of a system based on a hostname,
-	  ip_nametoaddr. In the ip6_unix implementation of this function,
-	  static variables are used to hold parsing buffers. This can cause
-	  a crash if multiple threads attempt to convert a hostname to an
-	  address at the same time. Locking on a single mail stream is not
-	  sufficient to prevent simultaneous access to these static
-	  variables. In the IMAP library, this function can be called from
-	  the mail_open and imap_status functions. As the imap_status
-	  function is not used by app_voicemail, locking on access to
-	  mail_open is sufficient to prevent any mangling of the buffers.
-	  Review: https://reviewboard.asterisk.org/r/4188/ ASTERISK-24516
-	  #close Reported by: David Duncan Ross Palmer Tested by: David
-	  Duncan Ross Palmer patches: ASTERISK-24516.diff uploaded by David
-	  Duncan Ross Palmer (License 6660) ........ Merged revisions
-	  428863 from http://svn.asterisk.org/svn/asterisk/branches/11
-	  ........ Merged revisions 428864 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-02 21:53 +0000 [r428837]  George Joseph <george.joseph@fairview5.com>
-
-	* CHANGES, /: CHANGES: Add item for new 'pjsip show identif(y|ies)
-	  commands Tested-by: George Joseph ........ Merged revisions
-	  428836 from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-02 19:03 +0000 [r428789-428815]  Matthew Jordan <mjordan@digium.com>
-
-	* tests/test_stasis.c: tests/test_stasis: Resolve compilation
-	  issues from Asterisk 12 merge When merging the changes up stream
-	  in r428687, I missed the fact that the signature for
-	  stasis_message_type_create was changed. This patch fixes the
-	  compilation issues introduced by that merge.
-
-	* pbx/pbx_loopback.c, /: pbx/pbx_loopback: Speed up switches by
-	  avoiding unneeded lookups This patch makes a small rearrangement
-	  to only do dialplan lookups during loopback switches if the
-	  pattern matches. Prior to this patch, the dialplan lookups were
-	  always performed, even when the result would be discarded.
-	  Dialplan lookups can be very costly if remote switches - like
-	  DUNDi - are present. In those cases extension matching is sped up
-	  considerably, making the issue of lost digits more manageable. As
-	  collateral damage, 6 trailing spaces were killed. Review:
-	  https://reviewboard.asterisk.org/r/4211 ASTERISK-24577 #close
-	  Reported by: Birger Harzenetter patches: ast-loopback.patch
-	  uploaded by Birger Harzenetter (License 5870) ........ Merged
-	  revisions 428787 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428788 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-02 12:20 +0000 [r428761]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_refer.c, /: res_pjsip_refer: Fix issue where native
-	  bridge may not occur upon completion of a transfer. There are two
-	  methods within res_pjsip_refer for keeping track of the state of
-	  a transfer. The first is a framehook which looks at frames
-	  passing by to determine the state. The second subscribes to know
-	  when the channel joins a bridge. In the case when the channel
-	  joins the bridge the framehook is *NOT* removed and this prevents
-	  the native RTP bridging technology from getting used. This change
-	  gets the channel and if it still exists remove the framehook.
-	  Review: https://reviewboard.asterisk.org/r/4218/ ........ Merged
-	  revisions 428760 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-02 00:38 +0000 [r428731-428734]  George Joseph <george.joseph@fairview5.com>
-
-	* /, include/asterisk/config.h, main/config.c: config: Create
-	  ast_variable_find_in_list() Add const char
-	  *ast_variable_find_in_list(const struct ast_variable *list, const
-	  char *variable); ast_variable_find() requires a config category
-	  to search whereas ast_variable_find_in_list() just needs the root
-	  list element which is useful if you don't have a category.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4217/ ........ Merged
-	  revisions 428733 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, res/res_pjsip_endpoint_identifier_ip.c,
-	  res/res_pjsip/pjsip_cli.c: res_pjsip_endpoint_identifier_ip: Add
-	  'show identify(ies)' cli commands While troubleshooting other
-	  things I realized there were no pjsip cli commands for identify.
-	  This patch adds them. It also also fixes a reference leak when a
-	  'show endpoint' displayed identifies and properly sets the return
-	  code if load_module can't allocate a cli formatter structure.
-	  Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4212/ ........ Merged
-	  revisions 428725 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-01 17:57 +0000 [r428687]  Matthew Jordan <mjordan@digium.com>
-
-	* channels/chan_skinny.c, res/res_pjsip_mwi.c, tests/test_stasis.c,
-	  res/res_pjsip_pubsub.c, res/res_pjsip_refer.c,
-	  channels/chan_mgcp.c, main/stasis_cache.c, channels/chan_sip.c,
-	  include/asterisk/stasis_internal.h, /, include/asterisk/stasis.h,
-	  UPGRADE.txt, configs/samples/stasis.conf.sample,
-	  res/parking/parking_applications.c, res/res_xmpp.c,
-	  channels/chan_iax2.c, apps/app_queue.c,
-	  res/res_stasis_device_state.c, channels/sig_pri.c,
-	  include/asterisk/stasis_message_router.h, main/endpoints.c,
-	  res/parking/parking_bridge_features.c, main/stasis.c,
-	  channels/chan_dahdi.c, main/stasis_message_router.c: main/stasis:
-	  Allow subscriptions to use a threadpool for message delivery
-	  Prior to this patch, all Stasis subscriptions would receive a
-	  dedicated thread for servicing published messages. In contrast,
-	  prior to r400178 (see review
-	  https://reviewboard.asterisk.org/r/2881/), the subscriptions
-	  shared a thread pool. It was discovered during some initial work
-	  on Stasis that, for a low subscription count with high message
-	  throughput, the threadpool was not as performant as simply having
-	  a dedicated thread per subscriber. For situations where a
-	  subscriber receives a substantial number of messages and is
-	  always present, the model of having a dedicated thread per
-	  subscriber makes sense. While we still have plenty of
-	  subscriptions that would follow this model, e.g., AMI, CDRs, CEL,
-	  etc., there are plenty that also fall into the following two
-	  categories: * Large number of subscriptions, specifically those
-	  tied to endpoints/peers. * Low number of messages. Some
-	  subscriptions exist specifically to coordinate a single message -
-	  the subscription is created, a message is published, the delivery
-	  is synchronized, and the subscription is destroyed. In both of
-	  the latter two cases, creating a dedicated thread is wasteful
-	  (and in the case of a large number of peers/endpoints, harmful).
-	  In those cases, having shared delivery threads is far more
-	  performant. This patch adds the ability of a subscriber to Stasis
-	  to choose whether or not their messages are dispatched on a
-	  dedicated thread or on a threadpool. The threadpool is
-	  configurable through stasis.conf. Review:
-	  https://reviewboard.asterisk.org/r/4193 ASTERISK-24533 #close
-	  Reported by: xrobau Tested by: xrobau ........ Merged revisions
-	  428681 from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-12-01 13:41 +0000 [r428632-428655]  Joshua Colp <jcolp@digium.com>
-
-	* /, apps/app_record.c: app_record: Fix bug where using the 'k'
-	  option and hanging up would trim 1/4 of a second of the
-	  recording. The Record dialplan function trims 1/4 of a second
-	  from the end of recordings in case they are terminated because of
-	  DTMF. When hanging up, however, you don't want this to happen.
-	  This change makes it so on hangup this does not occur.
-	  ASTERISK-24530 #close Reported by: Ben Smithurst patches:
-	  app_record_v2.diff submitted by Ben Smithurst (license 6529)
-	  Review: https://reviewboard.asterisk.org/r/4201/ ........ Merged
-	  revisions 428653 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428654 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/channel.c: channel: Extend size of buffer for codecs in
-	  "core show channeltype" CLI command. The static buffer for codecs
-	  when invoking the "core show channeltype" CLI command did not
-	  have enough room for all codecs. This has been extended so it
-	  does. ASTERISK-24542 #close Reported by: snuffy patches:
-	  channeltype-tech.diff submitted by snuffy (license 5024) Review:
-	  https://reviewboard.asterisk.org/r/4204/
-
-2014-11-24 20:37 +0000 [r428602-428604]  Richard Mudgett <rmudgett@digium.com>
-
-	* tests/test_channel_feature_hooks.c: test_channel_feature_hooks.c:
-	  Fix unit test for DTMF hooks. Fix the failing
-	  /channels/features/test_features_channel_dtmf unit test. DTMF
-	  emulation does not work without a stream of packets to prod the
-	  emulation code. Review: https://reviewboard.asterisk.org/r/4199/
-
-	* /, main/bridge.c, main/bridge_channel.c: DTMF hooks: Leaving
-	  channels need to push any collected digits into the bridge. Any
-	  partially collected DTMF digits for a DTMF hook need to be pushed
-	  into the bridge when a channel leaves the bridging system as if
-	  there were a timeout. Review:
-	  https://reviewboard.asterisk.org/r/4199/ ........ Merged
-	  revisions 428601 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-21 19:09 +0000 [r428572]  Richard Mudgett <rmudgett@digium.com>
-
-	* main/manager.c, /: manager: Fix could not extend string messages.
-	  When shutting down Asterisk that has an active AMI connection,
-	  you get several "failed to extend from %d to %d" messages because
-	  use of the EVENT_FLAG_SHUTDOWN attempts to add all AMI permission
-	  strings to the event. * Created MAX_AUTH_PERM_STRING to use when
-	  creating stack based struct ast_str variables used with the
-	  authority_to_str() and user_authority_to_str() functions instead
-	  of a variety of magic numbers that could be too small. * Added a
-	  special check for EVENT_FLAG_SHUTDOWN to authority_to_str() so it
-	  will not attempt to add all permission level strings. Review:
-	  https://reviewboard.asterisk.org/r/4200/ ........ Merged
-	  revisions 428570 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428571 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-21 17:45 +0000 [r428544]  George Joseph <george.joseph@fairview5.com>
-
-	* main/sorcery.c, /, res/res_pjsip_phoneprov_provider.c,
-	  tests/test_sorcery.c: sorcery: Make is_object_field_registered
-	  handle field names that are regexes. As a result of
-	  https://reviewboard.asterisk.org/r/3305, res_sorcery_realtime was
-	  tossing database fields that didn't have an exact match to a
-	  sorcery registered field. This broke the ability to use regexes
-	  as field names which manifested itself as a failure of
-	  res_pjsip_phoneprov_provider which uses this capability. It also
-	  broke handling of fields that start with '@' in realtime but I
-	  don't think anyone noticed. This patch does the following... *
-	  Modifies ast_sorcery_fields_register to pre-compile the name
-	  regex. * Modifies ast_sorcery_is_object_field_registered to test
-	  the regex if it exists instead of doing an exact strcmp. *
-	  Modifies res_pjsip_phoneprov_provider with a few tweaks to get it
-	  to work with realtime. Tested-by: George Joseph Review:
-	  https://reviewboard.asterisk.org/r/4185/ ........ Merged
-	  revisions 428543 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-21 02:16 +0000 [r428505]  Matthew Jordan <mjordan@digium.com>
-
-	* main/bridge_basic.c: main/bridge_basic: Fix features regressions
-	  introduced by r428165 In r428165, two bugs were introduced: *
-	  Prior to entering the features retry loop, the buffer that holds
-	  the collected digits is wiped. However, this inadvertently wipes
-	  out the first collected digit on the first pass through, which is
-	  obtained in ast_stream_and_wait. This caused all of the features
-	  tests to fail. * If ast_app_dtget returns a hangup (-1), the loop
-	  would retry incorrectly. If we detect a hangup, we have to stop
-	  trying the feature. This patch fixes both issues. Review:
-	  https://reviewboard.asterisk.org/r/4196/
-
-2014-11-20 16:36 +0000 [r428425]  Mark Michelson <mmichelson@digium.com>
-
-	* main/acl.c, /: Fix error with mixed address family ACLs. Prior to
-	  this commit, the address family of the first item in an ACL was
-	  used to compare all incoming traffic. This could lead to traffic
-	  of other IP address families bypassing ACLs. ASTERISK-24469
-	  #close Reported by Matt Jordan Patches: ASTERISK-24469-11.diff
-	  uploaded by Matt Jordan (License #6283) AST-2014-012 ........
-	  Merged revisions 428402 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 428417 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428422 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-20 16:34 +0000 [r428413]  Kevin Harwell <kharwell@digium.com>
-
-	* funcs/func_db.c, /: AST-2014-018 - func_db: DB Dialplan function
-	  permission escalation via AMI. The DB dialplan function when
-	  executed from an external protocol (for instance AMI), could
-	  result in a privilege escalation. Asterisk now inhibits the DB
-	  function from being executed from an external interface if the
-	  live_dangerously option is set to no. ASTERISK-24534 Reported by:
-	  Gareth Palmer patches: submitted by Gareth Palmer (license 5169)
-	  ........ Merged revisions 428331 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 428363 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428409 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-20 16:13 +0000 [r428343]  Jonathan Rose <jrose@digium.com>
-
-	* res/res_pjsip_acl.c, /: PJSIP ACLs: Fix ACLs not loading on
-	  startup and apply/acl issues on contact The biggest problem this
-	  patch fixes is that ACLs weren't previously being loaded when the
-	  res_pjsip_acl module was loaded. Yikes. In addition, the ACL
-	  options contact_permit and contact_acl were effectively
-	  interpreted as contact_deny and this patch fixes that as well.
-	  AST-1418 #close Reported by: Thomas Thompson Review:
-	  https://reviewboard.asterisk.org/r/4120/ ASTERISK-24531 #close
-	  Reported by: Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4171/ ........ Merged
-	  revisions 428333 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-20 15:50 +0000 [r428339]  Kevin Harwell <kharwell@digium.com>
-
-	* apps/app_confbridge.c, /: AST-2014-017 - app_confbridge:
-	  permission escalation/ class authorization. Confbridge dialplan
-	  function permission escalation via AMI and inappropriate class
-	  authorization on the ConfbridgeStartRecord action. The CONFBRIDGE
-	  dialplan function when executed from an external protocol (for
-	  instance AMI), could result in a privilege escalation. Also, the
-	  AMI action “ConfbridgeStartRecord” could also be used to execute
-	  arbitrary system commands without first checking for system
-	  access. Asterisk now inhibits the CONFBRIDGE function from being
-	  executed from an external interface if the live_dangerously
-	  option is set to no. Also, the “ConfbridgeStartRecord” AMI action
-	  is now only allowed to execute under a user with system level
-	  access. ASTERISK-24490 Reported by: Gareth Palmer ........ Merged
-	  revisions 428332 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428334 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-20 14:55 +0000 [r428302-428305]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_refer.c, /: AST-2014-016: Fix crash when receiving
-	  an in-dialog INVITE with Replaces in res_pjsip_refer. The
-	  implementation of INVITE with Replaces in res_pjsip_refer did not
-	  expect them to occur in-dialog. As a result it would incorrectly
-	  attempt to hang up a channel it thought was under its control. In
-	  reality the channel would be under the control of another thread.
-	  When the other thread accessed the channel it would be accessing
-	  freed memory and could crash. This change makes res_pjsip_refer
-	  not act on an in-dialog INVITE with Replaces. ASTERISK-24528
-	  #close Reported by: Joshua Colp ........ Merged revisions 428304
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* channels/chan_pjsip.c, /: AST-2014-015: Fix race condition in
-	  chan_pjsip when sending responses after a CANCEL has been
-	  received. Due to the serialized architecture of chan_pjsip there
-	  exists a race condition where a CANCEL may be received and
-	  processed before responses (such as 180 Ringing, 183 Session
-	  Progress, and 200 OK) are sent. Since the session is in an
-	  unexpected state PJSIP will assert when this is attempted. This
-	  change makes it so that these responses are not sent on
-	  disconnected sessions. ASTERISK-24471 #close Reported by: yaron
-	  nahum ........ Merged revisions 428301 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-19 19:31 +0000 [r428273]  Corey Farrell <git@cfware.com>
-
-	* include/asterisk/stringfields.h, /: stringfields: Fix bug in
-	  ast_string_fields_copy. ast_string_fields_copy relies on the fact
-	  that __ast_string_field_release_active never previously zeroed
-	  pool->used, so keeping the existing pointer was "ok". Now that
-	  existing pools can be reset to 'empty', it is important to set
-	  each field to __ast_string_field_empty after releasing the
-	  memory. ASTERISK-24535 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4186/ ........ Merged
-	  revisions 428272 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-19 17:13 +0000 [r428246]  Richard Mudgett <rmudgett@digium.com>
-
-	* res/res_calendar.c, main/manager.c, /, channels/chan_sip.c,
-	  channels/sip/security_events.c: ast_str: Fix improper member
-	  access to struct ast_str members. Accessing members of struct
-	  ast_str outside of the string manipulation API routines is
-	  invalid since struct ast_str is supposed to be treated as opaque.
-	  Review: https://reviewboard.asterisk.org/r/4194/ ........ Merged
-	  revisions 428244 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428245 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-19 12:40 +0000 [r428196-428222]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_session.c, include/asterisk/res_pjsip.h,
-	  include/asterisk/res_pjsip_session.h, res/res_pjsip_sdp_rtp.c,
-	  res/res_pjsip/pjsip_configuration.c,
-	  configs/samples/pjsip.conf.sample,
-	  contrib/ast-db-manage/config/versions/eb88a14f2a_add_media_encryption_optimistic_to_pjsip.py
-	  (added), CHANGES, res/res_pjsip.c: res_pjsip_sdp_rtp: Add support
-	  for optimistic SRTP. Optimistic SRTP is the ability to enable
-	  SRTP but not have it be a fatal requirement. If SRTP can be used
-	  it will be, if not it won't be. This gives you a better chance of
-	  using it without having your sessions fail when it can't be.
-	  Encrypt all the things! Review:
-	  https://reviewboard.asterisk.org/r/3992/
-
-	* res/res_pjsip_refer.c, /: res_pjsip_refer: Ensure Refer-To is
-	  NULL terminated and parse it as a URI. There is no guarantee that
-	  when we get a Refer-To that it will be NULL terminated. As the
-	  URI parsing function requires it to be we now NULL terminate it.
-	  Additionally parsing the Refer-To as a 'To' header is needless
-	  and it can simply be done as a URI. This also fixes a problem
-	  where certain Refer-To headers would not be parsed as a 'To'
-	  header causing the REFER to fail. ASTERISK-24508 #close Reported
-	  by: Beppo Mazzucato Review:
-	  https://reviewboard.asterisk.org/r/4187/ ........ Merged
-	  revisions 428195 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-18 18:54 +0000 [r428169]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, res/parking/parking_tests.c: parking_tests.c: Add missing
-	  newline on a unit test message. ........ Merged revisions 428168
-	  from http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-17 16:51 +0000 [r428145]  Mark Michelson <mmichelson@digium.com>
-
-	* CHANGES, main/features_config.c,
-	  configs/samples/features.conf.sample,
-	  include/asterisk/features_config.h, main/bridge_basic.c: Allow
-	  for transferer to retry when dialing an invalid extension. This
-	  allows for a configurable number of attempts for a transferer to
-	  dial an extension to transfer the call to. For Asterisk 13, the
-	  default values are such that upgrading between versions will not
-	  cause a behaivour change. For trunk, though, the defaults will be
-	  changed to be more user-friendly. Review:
-	  https://reviewboard.asterisk.org/r/4167
+2014-08-07 20:37 +0000 [96be6b2228]  Matt Jordan <mjordan@digium.com>
 
-2014-11-17 16:00 +0000 [r428119]  Corey Farrell <git@cfware.com>
+	* Initialize svnmerge from branches/13
 
-	* /, channels/chan_sip.c: chan_sip: Fix theoretical leak of
-	  p->refer. If transmit_refer is called when p->refer is already
-	  allocated, it leaks the previous allocation. Updated code to
-	  always free previous allocation during a new allocation. Also
-	  instead of checking if we have a previous allocation, always
-	  create a clean record. ASTERISK-15242 #close Reported by: David
-	  Woolley Review: https://reviewboard.asterisk.org/r/4160/ ........
-	  Merged revisions 428117 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428118 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-17 15:27 +0000 [r428079-428115]  Matthew Jordan <mjordan@digium.com>
-
-	* /, apps/confbridge/conf_state_multi_marked.c:
-	  apps/app_confbridge: Ensure 'normal' users hear message when last
-	  marked leaves When r428077 was made for ASTERISK-24522, it failed
-	  to take into account users who are neither wait_marked nor
-	  end_marked. These users are *also* supposed to hear the 'leader
-	  has left the conference' message. Granted, this behaviour is a
-	  bit odd; however, that is how it used to work... and behaviour
-	  changes are not good. This patch ensures that if there are any
-	  'normal' users present when the last marked user leaves the
-	  conference, the message will still be played to them. Note that
-	  this regression was caught by the Asterisk Test Suite's
-	  confbridge_nominal test, which has a quirky combination of users.
-	  ........ Merged revisions 428113 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428114 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, apps/confbridge/conf_state_multi_marked.c: app_confbridge:
-	  Don't play leader leaving prompt if no one will hear it Consider
-	  the following: - A marked user in a conference - One or more
-	  end_marked only users in the conference When the marked users
-	  leaves, we will be in the conf_state_multi_marked state. This
-	  currently will traverse the users, kicking out any who have the
-	  end_marked flags. When they are kicked, a full ast_bridge_remove
-	  is immediately called on the channels. At this time, we also
-	  unilaterally set the need_prompt flag. When the need_prompt flag
-	  is set, we then playback a sound to the bridge informing everyone
-	  that the leader has left; however, no one is left in the bridge.
-	  This causes some odd behaviour for the end_marked users - they
-	  are stuck waiting for the bridge to be unlocked. This results in
-	  them waiting for 5 or 6 seconds of dead air before hearing that
-	  they've been kicked. Unfortunately, we do have to keep the bridge
-	  locked while we're playing back the 'leader-has-left' prompt. If
-	  there are any wait_marked users in the conference, this behaviour
-	  can't be easily changed - but we do make the case of the
-	  end_marked users better with this patch. Review:
-	  https://reviewboard.asterisk.org/r/4184/ ASTERISK-24522 #close
-	  Reported by: Matt Jordan ........ Merged revisions 428077 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 428078 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-16 21:12 +0000 [r427979-428052]  Joshua Colp <jcolp@digium.com>
-
-	* channels/chan_pjsip.c, /: chan_pjsip: Remove AOR check when
-	  dialing and one is specified. The AOR value may contain the name
-	  of an AOR or a full SIP URI. Checking if the AOR exists can't be
-	  done as a result of this. ........ Merged revisions 428051 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, channels/chan_pjsip.c: chan_pjsip: Add additional log message
-	  when an AOR is specified when dialing and it does not exist.
-	  ASTERISK-24499 #close Reported by: Rusty Newton ........ Merged
-	  revisions 428007 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* channels/chan_motif.c, channels/chan_pjsip.c, /: chan_motif /
-	  chan_pjsip: Fix incorrect "No such module" messages when
-	  reloading. For chan_motif the direct return value of the
-	  underlying config options framework was passed back. This can
-	  relay various states which the module loader would not interpet
-	  as success. It has been changed so only on errors will it report
-	  back an error. For chan_pjsip the code implemented a dummy reload
-	  function which always returned an error. This has been removed as
-	  all configuration is held within res_pjsip instead.
-	  ASTERISK-23651 #close Reported by: Rusty Newton ........ Merged
-	  revisions 427981 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, res/res_pjsip/pjsip_configuration.c: res_pjsip: Enforce
-	  requirements for session timer minimum expiration period and
-	  normal expiration period. This change enforces the requirements
-	  in PJSIP for session timer configuration. The minimum expiration
-	  period must be 90 seconds or higher and the normal expiration
-	  period can not be lower than the minimum expiration period. If
-	  either of these were done the code would assert at session setup
-	  time. ASTERISK-24336 #close Reported by: Leon Rowland ........
-	  Merged revisions 427978 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-15 16:56 +0000 [r427927-427954]  Matthew Jordan <mjordan@digium.com>
-
-	* cel/cel_odbc.c, /: cel/cel_odbc: Provide microsecond precision in
-	  'eventtime' column when possible This patch adds microsecond
-	  precision when inserting a CEL record into a table with an
-	  "eventtime" column of type timestamp, instead of second
-	  precision. The documentation (configs/cel_odbc.conf.sample) was
-	  already saying that the eventtime column included microseconds
-	  precision, but that was not the case. Also, without this patch,
-	  if you had a table with an "eventtime" column of type varchar,
-	  you had millisecond precision. With this patch, you also get
-	  microsecond precision in this case. Review:
-	  https://reviewboard.asterisk.org/r/3980 ASTERISK-24283 #close
-	  Reported by: Etienne Lessard patches:
-	  cel_odbc_time_precision.patch uploaded by Etienne Lessard
-	  (License 6394) ........ Merged revisions 427952 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427953 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* tests/test_cel.c: tests/test_cel: Unlock bridge on off nominal
-	  paths If the test fails due to memory allocation errors, we may
-	  as well attempt to unlock the bridge on the way out.
-
-2014-11-14 17:45 +0000 [r427902]  Jonathan Rose <jrose@digium.com>
-
-	* configs/samples/cdr.conf.sample, main/cdr.c, /: Documentation:
-	  Revise explanation of cdr.conf option 'Unanswered' ASTERISK-24279
-	  #close Reported by: Matt Jordan Review:
-	  https://reviewboard.asterisk.org/r/4109/ ........ Merged
-	  revisions 427901 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-14 15:51 +0000 [r427876]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* /, main/stun.c: stun: correct attribute string padding to match
-	  rfc When sending the USERNAME attribute in an RTP STUN response,
-	  the implementation in append_attr_string passed the actual
-	  length, instead of padding it up to a multiple of four bytes as
-	  required by the RFC 3489. This change adds separate variables for
-	  the string and padded attributed lengths, and performs padding
-	  correctly. Reported by: Thomas Arimont Review:
-	  https://reviewboard.asterisk.org/r/4139/ ........ Merged
-	  revisions 427874 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427875 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-14 15:24 +0000 [r427870]  Mark Michelson <mmichelson@digium.com>
-
-	* main/bridge.c, main/bridge_basic.c,
-	  include/asterisk/stasis_bridges.h, tests/test_cel.c,
-	  apps/app_queue.c, main/cel.c, main/stasis_bridges.c, /,
-	  res/stasis/app.c: Fix race condition that could result in ARI
-	  transfer messages not being sent. From reviewboard: "During blind
-	  transfer testing, it was noticed that tests were failing
-	  occasionally because the ARI blind transfer event was not being
-	  sent. After investigating, I detected a race condition in the
-	  blind transfer code. When blind transferring a single channel,
-	  the actual transfer operation (i.e. removing the transferee from
-	  the bridge and directing them to the proper dialplan location) is
-	  queued onto the transferee bridge channel. After queuing the
-	  transfer operation, the blind transfer Stasis message is
-	  published. At the time of publication, snapshots of the channels
-	  and bridge involved are created. The ARI subscriber to the blind
-	  transfer Stasis message then attempts to determine if the bridge
-	  or any of the involved channels are subscribed to by ARI
-	  applications. If so, then the blind transfer message is sent to
-	  the applications. The way that the ARI blind transfer message
-	  handler works is to first see if the transferer channel is
-	  subscribed to. If not, then iterate over all the channel IDs in
-	  the bridge snapshot and determine if any of those are subscribed
-	  to. In the test we were running, the lone transferee channel was
-	  subscribed to, so an ARI event should have been sent to our
-	  application. Occasionally, though, the bridge snapshot did not
-	  have any channels IDs on it at all. Why? The problem is that
-	  since the blind transfer operation is handled by a separate
-	  thread, it is possible that the transfer will have completed and
-	  the channels removed from the bridge before we publish the blind
-	  transfer Stasis message. Since the blind transfer has completed,
-	  the bridge on which the transfer occurred no longer has any
-	  channels on it, so the resulting bridge snapshot has no channels
-	  on it. Through investigation of the code, I found that attended
-	  transfers can have this issue too for the case where a transferee
-	  is transferred to an application." The fix employed here is to
-	  decouple the creation of snapshots for the transfer messages from
-	  the publication of the transfer messages. This way, snapshots can
-	  be created to reflect what they are at the time of the transfer
-	  operation. Review: https://reviewboard.asterisk.org/r/4135
-	  ........ Merged revisions 427848 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-14 14:56 +0000 [r427846]  Joshua Colp <jcolp@digium.com>
-
-	* /, apps/confbridge/conf_state_multi_marked.c: app_confbridge:
-	  Play "leader has left" sound even when musiconhold is enabled.
-	  Currently if the leader of a conference bridge leaves any
-	  participant that has musiconhold enabled will not hear the
-	  "leader has left" sound. This is because musiconhold is started
-	  and THEN the sound is played. This change makes it so that the
-	  sound is played and THEN musiconhold is started. This provides a
-	  better experience for users as they may not have known previously
-	  why they went back to musiconhold. Review:
-	  https://reviewboard.asterisk.org/r/4177/ ........ Merged
-	  revisions 427844 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427845 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-14 14:24 +0000 [r427841]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip.c, res/res_pjsip_pubsub.c, res/res_pjsip_session.c,
-	  include/asterisk/res_pjsip.h: Fix race condition where duplicated
-	  requests may be handled by multiple threads. This is the Asterisk
-	  13 version of the patch. The main difference is in the pubsub
-	  code since it was completely refactored between Asterisk 12 and
-	  13. Review: https://reviewboard.asterisk.org/r/4175
-
-2014-11-13 22:03 +0000 [r427815]  Kevin Harwell <kharwell@digium.com>
-
-	* /, res/res_pjsip_outbound_registration.c: res_pjsip_exten_state:
-	  PJSIPShowSubscriptionsInbound causes crash When using a
-	  non-default sorcery wizard (in this instance realtime) for
-	  outbound registrations and after adding in an appropriate call to
-	  ast_sorcery_apply_config() (since it is missing) Asterisk will
-	  crash after a stack overflow occurs due to the code infinitely
-	  recursing. The fix entails removing the outbound registration
-	  state dependency from the outbound registration sorcery object
-	  and instead keeping an in memory container that can be used to
-	  lookup the state when needed. ASTERISK-24514 Reported by: Mark
-	  Michelson Review: https://reviewboard.asterisk.org/r/4164/
-	  ........ Merged revisions 427814 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-13 15:44 +0000 [r427789]  Kinsey Moore <kmoore@digium.com>
-
-	* include/asterisk/stasis.h, include/asterisk/stasis_app.h,
-	  res/stasis/app.h, res/res_stasis.c, /, res/stasis/app.c,
-	  res/stasis/stasis_bridge.c: Stasis: Fix StasisEnd message
-	  ordering This change corrects message ordering in cases where a
-	  channel-related message can be received after a Stasis/ARI
-	  application has received the StasisEnd message. The StasisEnd
-	  message was being passed to applications directly without waiting
-	  for the channel topic to empty. As a result of this fix, other
-	  bugs were also identified and fixed: * StasisStart messages were
-	  also being sent directly to apps and are now routed through the
-	  stasis message bus properly * Masquerade monitor datastores were
-	  being removed at the incorrect time in some cases and were
-	  causing StasisEnd messages to not be sent * General refactoring
-	  where necessary for the above * Unsubscription on StasisEnd
-	  timing changes to prevent additional messages from following the
-	  StasisEnd when they shouldn't A channel sanitization function
-	  pointer was added to reduce processing and AO2 lookups. Review:
-	  https://reviewboard.asterisk.org/r/4163/ ASTERISK-24501 #close
-	  Reported by: Matt Jordan ........ Merged revisions 427788 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-13 00:00 +0000 [r427763]  Matthew Jordan <mjordan@digium.com>
-
-	* main/rtp_engine.c, /: main/rtp_engine: Fix crash when processing
-	  more than one RTCP report info block Asterisk - in
-	  res_rtp_asterisk - only understands a single RTCP report info
-	  block. When the RTCP information was refactored in the RTP Engine
-	  to be pushed over the Stasis message bus, I put in the hooks into
-	  the engine to handle multiple RTCP report info blocks, in the
-	  hope that a future RTP implementation would be able to provide
-	  that data. Unfortunately, res_rtp_asterisk has a tendency to
-	  "lie": (1) It will send RTCP reports with a
-	  reception_report_count greater than 1 (which is pulled directly
-	  from the RTCP packet itself, so that part is correct) (2) It will
-	  only provide a single report block When the rtp_engine goes to
-	  convert this to a JSON blob, hilarity ensues as it looks for a
-	  report block that doesn't exist. This patch updates the
-	  rtp_engine to be a bit more skeptical about what it is presented
-	  with. While this could also be fixed in res_rtp_asterisk, this
-	  patch prefers to fix it in the engine for two reasons: (1) The
-	  engine is designed to work with multiple RTP implementation, and
-	  hence having it be more robust is a good thing (tm) (2)
-	  res_rtp_asterisk's handling of RTCP information is "fun". It
-	  should report the correct reception_report_count; ideally it
-	  should also be giving us all of the blocks - but it is
-	  *definitely* not designed to do that. Going down that road is a
-	  non-trivial effort. Review:
-	  https://reviewboard.asterisk.org/r/4158/ ASTERISK-24489 #close
-	  Reported by: Gregory Malsack Tested by: Gregory Malsack
-	  ASTERISK-24498 #close Reported by: Beppo Mazzucato Tested by:
-	  Beppo Maazucato ........ Merged revisions 427762 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-12 20:39 +0000 [r427737]  Corey Farrell <git@cfware.com>
-
-	* /, main/features.c: Fix leak in AMI Action Bridge Add missing
-	  reference cleanup for newly created bridge. ASTERISK-24281
-	  Reported by: Stefan Engström Review:
-	  https://reviewboard.asterisk.org/r/4154/ ........ Merged
-	  revisions 427736 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-12 16:12 +0000 [r427711]  Joshua Colp <jcolp@digium.com>
-
-	* main/pbx.c, /: pbx: Fix off-nominal case where a freed extension
-	  may still be used. If during the operation of adding an extension
-	  a priority is added but fails it is possible for the extension to
-	  be freed but still exist in the PBX core. If this occurs
-	  subsequent lookups may try to access the extension and end up in
-	  freed memory. This change removes the extension from the PBX core
-	  when the priority addition fails and then frees the extension.
-	  ASTERISK-24444 #close Reported by: Leandro Dardini Review:
-	  https://reviewboard.asterisk.org/r/4162/ ........ Merged
-	  revisions 427709 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427710 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-12 13:46 +0000 [r427684]  Corey Farrell <git@cfware.com>
-
-	* codecs/ilbc, /, tests, codecs/speex, apps/confbridge,
-	  Makefile.rules: Fix compiler error when using ./configure
-	  --enable-dev-mode --enable-coverage When DONT_OPTIMIZE is enabled
-	  with dev-mode, it causes a shadow compilation to be done with
-	  output to /dev/null. This can cause errors with coverage when GCC
-	  attempts to write to /dev/null.gcno. This change disables
-	  coverage for the shadow compilation. ASTERISK-24502 #close
-	  Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4151/ ........ Merged
-	  revisions 427682 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427683 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420499 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-2014-11-09 08:00 +0000 [r427643]  Corey Farrell <git@cfware.com>
+2014-08-07 20:36 +0000 [38a0df95b1]  Matt Jordan <mjordan@digium.com>
 
-	* main/manager.c, /: manager: Fix HTTP connection reference leaks.
-	  Fix reference leak that happens if (session && !blastaway).
-	  ASTERISK-24505 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4153/ ........ Merged
-	  revisions 427641 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427642 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-09 00:38 +0000 [r427583-427615]  Matthew Jordan <mjordan@digium.com>
-
-	* channels/chan_mgcp.c, /: channels/chan_mgcp: Fix regression which
-	  causes gateways to be skipped In r227276, a while loop was turned
-	  into a for loop. Unfortunately, a portion of the while loop was
-	  left in the code such that, when a static gateway is encountered
-	  in the list of MGCP gateways, the next gateway would be skipped.
-	  At best, we would simply flip past a gateway; at worst, this
-	  could lead to a crash. ASTERISK-24500 #close Reported by: Xavier
-	  Hienne patches: chan_mgcp.patch uploaded by Xavier Hienne
-	  (License 6657) ........ Merged revisions 427613 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427614 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, addons/chan_mobile.c: addons/chan_mobile: Increase buffer size
-	  of UCS2 encoded SMS messages When UCS2 character encoding is
-	  used, one symbol in national language can be expanded to 4 bytes.
-	  The current buffer used for receiving message in do_monitor_phone
-	  is 256 bytes, which is not large enough for incoming messages.
-	  For example: * AT+CMGR phone response prefix '+CMGR: "REC
-	  UNREAD","+7**********",,"14/10/29,13:31:39+12"\r\n' - 60 bytes *
-	  SMS body with UCS2 encoding (max) - 280 bytes * AT+CMGR phone
-	  response suffix '\r\n\r\nOK\r\n' - 8 bytes * Terminating null
-	  character - 1 byte This results in a needed buffer size of 349
-	  bytes. Hence, this patch opts for a 350 byte buffer.
-	  ASTERISK-24468 #close Reported by: Dmitriy Bubnov patches:
-	  chan_mobile-1_8.diff uploaded by Dmitriy Bubnov (License 6651)
-	  chan_mobile-trunk.diff uploaded by Dmitry Bubnov (License 6651)
-	  ........ Merged revisions 427607 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427610 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* apps/app_voicemail.c: app_voicemail: Fix enhancement that allowed
-	  multiple recipients in To: header An issue existed in r420577,
-	  which added multiple recipients to voicemail emails. The patch,
-	  when looking at the intended recipients, looked ahead for the '|'
-	  character inside a while loop which already had pulled out the
-	  appropriate field parsing on the '|' character. This would cause
-	  it to skip the recipients. This patch fixes it such that it
-	  relies completely on the while loop to parse through the e-mail
-	  fields. Note that the original author of the patch looked at this
-	  fix and approved it. ASTERISK-24250 #close Reported by: abelbeck
-	  patches: voicemail-420577-to-comma-fix.diff uploaded by abelbeck
-	  (License 5903)
-
-	* /, bridges/bridge_native_rtp.c: bridge_native_rtp: Fix T.38
-	  issues with remote bridges After r425242 the
-	  fax/sip/directmedia_reinvite_t38 test started failing due to the
-	  surviving channel not being re-INVITEd back from T.38 to audio.
-	  This patch fixes that bug - a deeper explanation of what happened
-	  follows. When two RTP channels are in a native bridge, the
-	  bridging layer will investigate each via the get_rtp_info glue
-	  callback. This callback returns the native bridge preference of
-	  the channel *at that moment in time* (that part is key). At
-	  different points during the bridging, the native bridging layer
-	  will inform the RTP capable channels of the status of the bridge
-	  via the update_peer glue callback. In a T.38 scenario with audio
-	  direct media, the sequence of events will often look like the
-	  following: * SIP/A and SIP/B both have audio and enter a native
-	  bridge. * Asterisk re-INVITEs audio between SIP/A and SIP/B
-	  directly (via an update_peer callback). * SIP/A sends a re-INVITE
-	  to T.38, which causes Asterisk to send a re-INVITE to T.38 to
-	  SIP/B. Assuming everyone 200 OKs the process, the UDPTL stack
-	  receives UDPTL packets in Asterisk from both endpoints. From the
-	  perspective of the channels, we are now in a local bridge for
-	  T.38, even though we are technically still in a remote bridge in
-	  bridge_native_rtp. (YAY!) * When one side hangs up,
-	  bridge_native_rtp is told to stop bridging. It then re-evaluates
-	  the channels and asks them how they are bridged - and since T.38
-	  is enabled, they reply with a Local bridge (which is correct),
-	  but is wrong because the audio portion is still technically in a
-	  remote bridge. * Asterisk releases the surviving channel, whose
-	  audio is *not* re-INVITED back to Asterisk as bridge_native_rtp
-	  incorrectly assumes that it was in a local bridge. Ironically,
-	  prior to r425242, this used to work mostly due to a fluke in the
-	  bridging layer. The purpose of the get_rtp_info callback
-	  shouldn't be modified: it should tell the bridging layer what
-	  kind of bridge the channel prefers at that moment in time. If you
-	  have T.38 enabled, that *must* be a local bridge, as the UDPTPL
-	  stack must be in the media path. As such, this patch does not
-	  modify that part of the code. However, we have to tell the
-	  channels to re-evaluate themselves when they come out of a native
-	  bridge, since we can no longer trust the get_rtp_info callbacks
-	  when the native bridge is being stopped. Something else may have
-	  changed in the channels, and they may now be lying to us. As
-	  such, this patch makes it so that we unilaterally tell the
-	  channels that they are no longer bridged via the update_peer
-	  callback. This is actually what the channels expect anyway: code
-	  in both chan_sip and chan_pjsip's callbacks look at the T.38
-	  state and - if they were in T.38 - send a re-INVITE to get the
-	  audio back to Asterisk. Review:
-	  https://reviewboard.asterisk.org/r/4157/ ........ Merged
-	  revisions 427582 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-08 18:17 +0000 [r427557]  Corey Farrell <git@cfware.com>
-
-	* /, channels/chan_console.c: chan_console: Fix reference leaks to
-	  pvt. Fix a bunch of calls to get_active_pvt where the reference
-	  is never released. ASTERISK-24504 #close Reported by: Corey
-	  Farrell Review: https://reviewboard.asterisk.org/r/4152/ ........
-	  Merged revisions 427554 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427555 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-06 19:22 +0000 [r427494-427512]  Richard Mudgett <rmudgett@digium.com>
-
-	* apps/app_agent_pool.c, /: app_agent_pool: Made agent alert
-	  interruptable by DTMF. Made agent able to interrupt the alerting
-	  beep playback with DTMF. Any digit can interrupt if the call does
-	  not need to be acknowledged. Only the first digit of the
-	  acknowledgement can interrupt if the call needs to be
-	  acknowledged. The agent interrupting the alerting playback builds
-	  on the ASTERISK-24447 patch because it knows what digit
-	  interrupted the playback and needs to be able to pass that digit
-	  to the DTMF hook digit collection code. ASTERISK-24257 #close
-	  Reported by: Steve Pitts Review:
-	  https://reviewboard.asterisk.org/r/4123/ ........ Merged
-	  revisions 427508 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, include/asterisk/bridge_channel.h, main/bridge_channel.c:
-	  Bridge DTMF hooks: Made audio pass from the bridge while waiting
-	  for more matching digits. * Made collecting DTMF digits for the
-	  DTMF feature hooks pass frames from the bridge. * Made collecting
-	  DTMF digits possible by other bridge hooks if there is a need.
-	  ASTERISK-24447 #close Reported by: Richard Mudgett Review:
-	  https://reviewboard.asterisk.org/r/4123/ ........ Merged
-	  revisions 427493 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-06 18:20 +0000 [r427491]  Joshua Colp <jcolp@digium.com>
-
-	* /, res/res_pjsip/pjsip_distributor.c: res_pjsip: Ensure in-dialog
-	  responses have an endpoint associated. When handling incoming
-	  messages we determine if it is associated with a dialog. If so we
-	  use that to determine what serializer and endpoint to use for the
-	  message. Previously this would pass the endpoint to the endpoint
-	  lookup module to actually place the endpoint completely on the
-	  message. For in-dialog responses, however, this did not occur as
-	  dialog processing took over and the endpoint lookup did not
-	  occur. This change just places the endpoint in the expected spot
-	  immediately instead of relying on the endpoint lookup module.
-	  In-dialog responses thus have the expected endpoint. AST-1459
-	  #close Review: https://reviewboard.asterisk.org/r/4146/ ........
-	  Merged revisions 427490 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-06 12:13 +0000 [r427384-427466]  Corey Farrell <git@cfware.com>
-
-	* main/file.c, /: main/file.c: fix possible extra ast_module_unref
-	  to format modules. fn_wrapper only adds a reference to the
-	  format's module if the file was able to be opened. If not this
-	  causes an unmatched ast_module_unref in filestream_destructor.
-	  Move ast_module_ref to get_stream. ASTERISK-24492 #close Reported
-	  by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4149/ ........ Merged
-	  revisions 427464 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427465 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* res/res_hep.c, /: res_hep: fix major leak that occurs when config
-	  is missing or enabled=no. Add missing unreference in
-	  hepv3_send_packet. ASTERISK-24491 #close Reported by: Zane Conkle
-	  Tested by: Zane Conkle Review:
-	  https://reviewboard.asterisk.org/r/4150/ ........ Merged
-	  revisions 427400 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, main/utils.c, include/asterisk/stringfields.h: Fix unintential
-	  memory retention in stringfields. * Fix missing / unreachable
-	  calls to __ast_string_field_release_active. * Reset pool->used to
-	  zero when the current pool->active reaches zero. ASTERISK-24307
-	  #close Reported by: Etienne Lessard Tested by: ibercom, Etienne
-	  Lessard Review: https://reviewboard.asterisk.org/r/4114/ ........
-	  Merged revisions 427380 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 427381 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427382 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-06 02:37 +0000 [r427356]  George Joseph <george.joseph@fairview5.com>
-
-	* tests/test_strings.c, /: test_strings: Remove string tests that
-	  exercise asserts. Since unit tests are run with DO_CRASH, those
-	  tests were causing the test to fail. Tested-by: George Joseph
-	  ........ Merged revisions 427354 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427355 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-05 19:52 +0000 [r427334]  Mark Michelson <mmichelson@digium.com>
-
-	* res/res_pjsip/config_system.c, configs/samples/pjsip.conf.sample,
-	  res/res_pjsip.c: Make the disable_tcp_switch PJSIP system object
-	  enabled by default. Testing has shown repeatedly that PJSIP's
-	  default behavior of switching automatically to TCP for large
-	  messages can cause issues. The most common issues are that
-	  devices that we are communicating with do not handle the switch
-	  to TCP gracefully, thus causing situations such as broken calls
-	  or broken subscriptions. Now, in order to have this behavior
-	  happen, you must opt into it. The sample file has been updated to
-	  warn that enabling the TCP switch behavior may cause issues for
-	  you, so use at your own risk.
-
-2014-11-05 12:18 +0000 [r427303]  Joshua Colp <jcolp@digium.com>
-
-	* res/res_pjsip_multihomed.c, /: res_pjsip_multihomed: Add logging
-	  during startup to aid debugging if local DNS is misbehaving. This
-	  change adds a bit of logging so if the local DNS is misbehaving
-	  it is easier to track down what is going on and where Asterisk
-	  may be hanging. ASTERISK-24438 #close Reported by: Melissa
-	  Shepherd Review: https://reviewboard.asterisk.org/r/4148/
-	  ........ Merged revisions 427300 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-05 00:15 +0000 [r427228-427276]  George Joseph <george.joseph@fairview5.com>
-
-	* pbx/pbx_config.c, main/config.c, tests/test_strings.c,
-	  include/asterisk/utils.h, /, main/utils.c: config: Make
-	  text_file_save and 'dialplan save' escape semicolons in values.
-	  When a config file is read, an unescaped semicolon signals
-	  comments which are stripped from the value before it's stored.
-	  Escaped semicolons are then unescaped and become part of the
-	  value. Both of these behaviors are normal and expected. When the
-	  config is serialized either by 'dialplan save' or
-	  AMI/UpdateConfig however, the now unescaped semicolons are
-	  written as-is. If you actually reload the file just saved, the
-	  unescaped semicolons are now treated as start of comments. Since
-	  true comments are stripped on read, any semicolons in
-	  ast_variable.value must have been escaped originally. This patch
-	  re-escapes semicolons in ast_variable.values before they're
-	  written to file either by 'dialplan save' or
-	  config/ast_config_text_file_save which is called by
-	  AMI/UpdateConfig. I also fixed a few pre-existing formatting
-	  issues nearby in pbx_config.c Tested-by: George Joseph
-	  ASTERISK-20127 #close Review:
-	  https://reviewboard.asterisk.org/r/4132/ ........ Merged
-	  revisions 427275 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/config.c, /: config: BUG: Restore ability for non-templ to
-	  be used as base objs in config. My recent refactor of config.c
-	  accidentally removed the capability for an object to inherit from
-	  a non-template object. This patch restores the capability to
-	  inherit from both template and non-template objects. Tested-by:
-	  George Joseph Reported-by: Scott Griepentrog ASTERISK-24487
-	  #close Review: https://reviewboard.asterisk.org/r/4147/ ........
-	  Merged revisions 427227 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-04 19:44 +0000 [r427181-427204]  Corey Farrell <git@cfware.com>
-
-	* funcs/func_talkdetect.c, /: func_talkdetect: Fix stasis message
-	  leak in audiohook callback. ASTERISK-24482 #close Reported by:
-	  Corey Farrell Review: https://reviewboard.asterisk.org/r/4142/
-	  ........ Merged revisions 427203 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, res/res_http_websocket.c: res_http_websockets: Fix extra unref
-	  of module In websocket_add_protocol_internal is used to add the
-	  "echo" protocol, but ast_websocket_remove_protocol is used to
-	  remove it. This causes an extra call to ast_module_unref.
-	  ASTERISK-24480 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4140/ ........ Merged
-	  revisions 427200 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/app.c: Fix crash caused by merge error on review 4138 When
-	  merging from 12 to 13 there were conflicts, I mistakenly had the
-	  loop run ast_closestream(others[0]) when it should be
-	  ast_closestream(others[x]).
-
-2014-11-03 18:15 +0000 [r427130]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, res/res_pjsip/config_system.c, UPGRADE.txt,
-	  configs/samples/pjsip.conf.sample, res/res_pjsip.c: res_pjsip:
-	  Add disable_tcp_switch option. When a packet exceeds the MTU,
-	  pjproject will switch from UDP to TCP. In some circumstances (on
-	  some networks), this can cause some issues with messages not
-	  getting sent to the correct destination - and can also cause
-	  connections to get dropped due to quirks in pjproject deciding to
-	  terminate TCP connections with no messages. While fixing the
-	  routing/messaging issues is important, having a configuration
-	  option in Asterisk that tells pjproject to not switch over to TCP
-	  would be useful. That way, if some glitch is discovered on some
-	  other network/site, we can at least disable the behavior until a
-	  fix is put into place. AFS-197 #close Review:
-	  https://reviewboard.asterisk.org/r/4137/ ........ Merged
-	  revisions 427129 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
+	* Remove 12 merge properties
 
-2014-11-03 02:34 +0000 [r427021-427089]  Corey Farrell <git@cfware.com>
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420498 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
-	* apps/app_voicemail.c, /: Fix compile error caused by review 4138
-	  There is no procedure called ast_closeframe, fix code to use
-	  ast_closestream. Reported By: Matt Jordan ........ Merged
-	  revisions 427087 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427088 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/app.c, apps/app_voicemail.c, /: Fix ast_writestream leaks
-	  Fix cleanup in __ast_play_and_record where others[x] may be
-	  leaked. This was caught where prepend != NULL && outmsg != NULL,
-	  once realfile[x] == NULL any further others[x] would be leaked. A
-	  cleanup block was also added for prepend != NULL && outmsg ==
-	  NULL. 11+: Fix leak of ast_writestream recording_fs in
-	  app_voicemail:leave_voicemail. ASTERISK-24476 #close Reported by:
-	  Corey Farrell Review: https://reviewboard.asterisk.org/r/4138/
-	  ........ Merged revisions 427023 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 427024 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427025 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, main/abstract_jb.c: func_jitterbuffer: fix frame leaks. Fix
-	  code paths where it is possible for frames to leak. Fix
-	  uninitialized variable in jb_get_fixed and jb_get_adaptive.
-	  ASTERISK-22409 #related Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4128/ ........ Merged
-	  revisions 427019 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 427020 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-11-02 01:01 +0000 [r426996]  Matthew Jordan <mjordan@digium.com>
-
-	* /, res/res_stasis.c: res/res_stasis: Fix crash on module unload
-	  while performing operation When the res_stasis module is
-	  unloaded, it will dispose of the apps_registry container. This is
-	  a problem if an ARI operation is in flight that attempts to use
-	  the registry, as the shutdown occurs in a separate thread. This
-	  patch adds some sanity checks to the various routines that access
-	  the registry which cause the operations to fail if the
-	  apps_registry does not exist. Crash caught by the Asterisk Test
-	  Suite. ........ Merged revisions 426995 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-31 16:50 +0000 [r426934]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
-
-	* Makefile, /: install init.d files on GNU/kFreeBSD Review:
-	  https://reviewboard.asterisk.org/r/4118/ ........ Merged
-	  revisions 426926 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426927 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426933 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-31 16:40 +0000 [r426924-426930]  Scott Griepentrog <sgriepentrog@digium.com>
-
-	* /, configs/samples/pjsip.conf.sample, res/res_pjsip.c: pjsip:
-	  clarify tls cert and key file usage A question arose as to
-	  whether a .pem file could be provided in place of the .crt and
-	  .key files in a PJSIP TLS configuration. I tested this and
-	  discovered that although a cert will be read from the pem file, a
-	  key will not, and thus the priv_key_file entry is still required.
-	  This update to the fine documentation clarifies the option usage.
-	  AST-1448 #close Review: https://reviewboard.asterisk.org/r/4129/
-	  Reported by: John Bigelow ........ Merged revisions 426928 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, res/res_pjsip_outbound_registration.c: pjsip: Handle outbound
-	  unregister correctly This updates the status of the outbound
-	  registration to reflect when it has been unregistered. Since the
-	  registration is unregistered but is not stopped, the registration
-	  schedule remains active as before. The patch also updates the
-	  documentation of both the AMI and CLI commands. ASTERISK-24411
-	  #close Review: https://reviewboard.asterisk.org/r/4119/ Reported
-	  by: John Bigelow patches: unregister-patch1.txt uploaded by John
-	  Bigelow (License 5091) ........ Merged revisions 426923 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-31 03:26 +0000 [r426865]  Matthew Jordan <mjordan@digium.com>
-
-	* /, channels/sip/reqresp_parser.c,
-	  channels/sip/include/reqresp_parser.h:
-	  channels/sip/reqresp_parser: Fix unit tests for r426594 When
-	  r426594 was made, it did not take into account a unit test that
-	  verified that the function properly populated the unsupported
-	  buffer. The function would previously memset the buffer if it
-	  detected it had any contents; since this function can now be
-	  called iteratively on successive headers, the unit tests would
-	  now fail. This patch updates the unit tests to reset the buffer
-	  themselves between successive calls, and updates the
-	  documentation of the function to note that this is now required.
-	  ........ Merged revisions 426858 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426860 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426863 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-31 03:08 +0000 [r426803-426833]  Corey Farrell <git@cfware.com>
-
-	* contrib/Makefile (added), Makefile, /: REF_DEBUG: Install
-	  refcounter.py to $(ASTDATADIR)/scripts This change ensures
-	  refcounter.py is installed to a place where it can be found by
-	  the Asterisk testsuite if REF_DEBUG is enabled. ASTERISK-24432
-	  #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4094/ ........ Merged
-	  revisions 426830 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426831 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426832 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, apps/app_queue.c: app_queue: fix a couple leaks to struct
-	  call_queue in set_member_value set_member_value has a couple
-	  leaks to references in the variable q found through testsuite
-	  tests/queues/set_penalty. Also remove the REF_DEBUG_ONLY_QUEUES
-	  compiler declaration, this is no longer possible with the updated
-	  REF_DEBUG code. ASTERISK-24466 #close Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4125/ ........ Merged
-	  revisions 426805 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426806 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/audiohook.c: audiohooks: Clean references to formats Cleanup
-	  references to in_translate[x].format and out_translate[x].format
-	  in ast_audiohook_detach_list. ASTERISK-24465 #close Reported by:
-	  Corey Farrell Review: https://reviewboard.asterisk.org/r/4124/
-
-2014-10-30 21:13 +0000 [r426757-426780]  Kevin Harwell <kharwell@digium.com>
-
-	* res/res_pjsip_exten_state.c, /: res_pjsip_exten_state:
-	  PJSIPShowSubscriptionsInbound causes crash Currently, it is
-	  possible for some subscriptions to get into a NULL state. When
-	  this occurs and the PJSIPShowSubscriptionsInbound ami action is
-	  issued and a device is subscribed for extension state then the
-	  associated subscription state object can't be located. The code
-	  then attempts to dereference a NULL object. Added a NULL check to
-	  avoid the problem. Reported by: John Bigelow ........ Merged
-	  revisions 426779 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* res/res_pjsip/pjsip_options.c, /: res_pjsip: incorrect qualify
-	  statistics after disabling for contact When removing the
-	  qualify_frequency from an AoR or a contact the statistics shown
-	  when issuing "pjsip show aors" from the CLI are incorrect. This
-	  patch deletes the contact's status object from sorcery,
-	  disassociating it from the contact, if the qualify_freqency is
-	  removed from configuration. ASTERISK-24462 #close Reported by:
-	  Mark Michelson Review: https://reviewboard.asterisk.org/r/4116/
-	  ........ Merged revisions 426755 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-30 09:20 +0000 [r426702]  Walter Doekes <walter+asterisk@wjd.nu>
-
-	* apps/app_voicemail.c, /: app_voicemail: Fix unchecked bounds of
-	  myArray in IMAP_STORAGE. In update_messages_by_imapuser(),
-	  messages were appended to a finite array which resulted in a
-	  crash when an IMAP mailbox contained more than 256 entries. This
-	  memory is now dynamically increased as needed. Observe that this
-	  patch adds a bunch of XXX's to questionable code. See the review
-	  (url below) for more information. ASTERISK-24190 #close Reported
-	  by: Nick Adams Tested by: Nick Adams Review:
-	  https://reviewboard.asterisk.org/r/4126/ ........ Merged
-	  revisions 426691 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426692 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426696 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-30 06:09 +0000 [r426668]  Igor Goncharovskiy <igor.goncharovsky@gmail.com>
-
-	* channels/chan_unistim.c, /: Add additional checks for NULL
-	  pointers to fix several crashes reported. ASTERISK-24304 #close
-	  Reported by: dhanapathy sathya ........ Merged revisions 426666
-	  from http://svn.asterisk.org/svn/asterisk/branches/11 ........
-	  Merged revisions 426667 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-30 01:59 +0000 [r426597-426602]  Matthew Jordan <mjordan@digium.com>
-
-	* /, channels/chan_sip.c: channels/chan_sip: Add improved support
-	  for 4xx error codes This patch adds support for 414, 493, 479,
-	  and a stray 400 response in REGISTER response handling. This
-	  helps interoperability in a number of scenarios. Review:
-	  https://reviewboard.asterisk.org/r/3437 patches: rb3437.patch
-	  uploaded by oej (License 5267) ........ Merged revisions 426599
-	  from http://svn.asterisk.org/svn/asterisk/branches/1.8 ........
-	  Merged revisions 426600 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426601 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* channels/sip/reqresp_parser.c, /, channels/chan_sip.c:
-	  channels/chan_sip: Support mutltiple Supported and Required
-	  headers A SIP request may contain multiple Supported: and
-	  Required: headers. Currently, chan_sip only parses the first
-	  Supported/Required header it finds. This patch adds support for
-	  multiple Supported/Required headers for INVITE requests. Review:
-	  https://reviewboard.asterisk.org/r/2478 ASTERISK-21721 #close
-	  Reported by: Olle Johansson patches: rb2478.patch uploaded by oej
-	  (License 5267) ........ Merged revisions 426594 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426595 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426596 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-29 10:33 +0000 [r426570]  Tzafrir Cohen <tzafrir.cohen@xorcom.com>
-
-	* channels/chan_phone.c: Fix building chan_phone on big endian
-	  systems A left over from the formats conversion (Corey Farrell).
-	  ASTERISK-24458 #close Review:
-	  https://reviewboard.asterisk.org/r/4117/
-
-2014-10-28 21:26 +0000 [r426552]  Richard Mudgett <rmudgett@digium.com>
-
-	* /, bridges/bridge_builtin_features.c: bridge_builtin_features:
-	  Add missing channel locks around
-	  ast_get_chan_features_general_config(). The feature_automonitor()
-	  and feature_automixmonitor() functions were not locking the
-	  channel around ast_get_chan_features_general_config(). Accessing
-	  the channel datastore list without the channel locked is a good
-	  way to corrupt the list or follow the pointer chain into
-	  oblivion. ........ Merged revisions 426531 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-28 21:05 +0000 [r426525-426529]  Corey Farrell <git@cfware.com>
-
-	* /, res/res_fax.c: res_fax: Resolve T38 gateway frame leak. When
-	  frames are translated by a fax gateway they need to be freed. The
-	  existing call to ast_frfree was unreachable. This change
-	  reorganizes fax_gateway_framehook to ensure that ast_frfree is
-	  called when needed. ASTERISK-24457 #close Reported by: Corey
-	  Farrell Review: https://reviewboard.asterisk.org/r/4115/ ........
-	  Merged revisions 426527 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426528 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/manager.c, /: manager: Unsubscribe from acl_change_sub at
-	  shutdown. ASTERISK-24453 #close Reported by: Corey Farrell
-	  Review: https://reviewboard.asterisk.org/r/4110/ ........ Merged
-	  revisions 426524 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-28 18:09 +0000 [r426459]  mdavenport <mdavenport@localhost>:
-
-	* configs/samples/manager.conf.sample: ASTERISK-23512, correct
-	  inaccurate comment in manager.conf.sample
-
-2014-10-28 16:40 +0000 [r426368-426432]  Matthew Jordan <mjordan@digium.com>
-
-	* /, main/bridge.c: main/bridge: Destroy features struct on off
-	  nominal path during bridge impart When a channel is imparted to a
-	  bridge, the invocation of the function may provide an
-	  ast_bridge_features struct. Upon passing this to
-	  ast_bridge_impart, the caller must assume that ownership has
-	  passed to the function, as in all paths the function destroys the
-	  struct prior to returning (as its purpose is to configure the
-	  behavior of the channel while in the bridge). On one off nominal
-	  path - where the channel already has a PBX thread - the struct
-	  was not being destroyed. This patch fixes that glitch.
-	  ASTERISK-24437 #close Reported by: Scott Griepentrog ........
-	  Merged revisions 426431 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* main/manager.c, /: main/manager: Fix typo in AMI event
-	  documentation of "OriginateResponse" The parameter name is
-	  "Response", not "Resonse". ASTERISK-24430 #close Reported by:
-	  Dafi Ni ........ Merged revisions 426366 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426367 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-28 14:56 +0000 [r426294-426362]  mdavenport <mdavenport@localhost>:
-
-	* res/res_agi.c: ASTERISK-24323, fix bug in documentation of AGI
-	  STREAM FILE CONTROL
-
-	* configs/samples/extensions.conf.sample: ASTERISK-24419, fix
-	  incorrect syntax for setting language in extensions.conf.sample
-
-2014-10-28 11:20 +0000 [r426252-426266]  Corey Farrell <git@cfware.com>
-
-	* apps/app_queue.c, /: app_queue: Cleanup ao2_iterator Clean
-	  ao2_iterator, resolving reference leak to queue members.
-	  ASTERISK-24454 #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4111/ ........ Merged
-	  revisions 426255 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426260 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* funcs/func_cdr.c: func_cdr: Fix CDR_PROP payload leak Remove
-	  duplicate allocation of payload, preventing leak. ASTERISK-24455
-	  #close Reported by: Corey Farrell Review:
-	  https://reviewboard.asterisk.org/r/4113/
-
-2014-10-27 17:54 +0000 [r426234]  Sean Bright <sean@malleable.com>
-
-	* build_tools/menuselect-deps.in, configure,
-	  include/asterisk/autoconfig.h.in, configure.ac, makeopts.in:
-	  configure: Add autoconf check for libopus. Because opus
-	  transcoding support cannot be included in the standard Asterisk
-	  distribution, a few codec_opus implementations have popped up. To
-	  make it easier for people to drop in opus support in their own
-	  installations, this patch adds configure checks for libopus.
-	  Review: https://reviewboard.asterisk.org/r/4106/
+2014-08-07 20:33 +0000 [5760526f69]  Matt Jordan <mjordan@digium.com>
 
-2014-10-27 02:46 +0000 [r426143-426211]  Matthew Jordan <mjordan@digium.com>
+	* Update UPGRADE.txt for 13 branch
 
-	* res/res_http_websocket.c, /: res/res_http_websocket: Fix minor
-	  nits found by wdoekes on r409681 When Moises committed the fixes
-	  for WSS (which was a great patch), wdoekes had a few style nits
-	  that were on the review that got missed. This patch resolves what
-	  I *think* were all of the ones that were still on the review.
-	  Thanks to both moy for the patch, and wdoekes for the reviews.
-	  Review: https://reviewboard.asterisk.org/r/3248/ ........ Merged
-	  revisions 426209 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426210 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* /, res/res_phoneprov.c: res/res_phoneprov: Fix crash on shutdown
-	  caused by container cleanup In res_phoneprov, unloading the
-	  module first destroys the http_routes container, followed by the
-	  users. However, users may have a route in the http_routes
-	  container; the validity of this container is not checked in the
-	  users destructor. Hence, we hit an assert as the container has
-	  already been set to NULL. This patch does two things: (1) It adds
-	  a sanity check in the user destructor (because why not) (2) It
-	  switches the order of destruction, so that users are disposed of
-	  prior to the HTTP routes they may hold a reference to. Note that
-	  this crash was caught by the Test Suite (go go testing!) ........
-	  Merged revisions 426174 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-	* res/res_srtp.c, /: res/res_srtp: Fix include issue for libsrtp
-	  1.5.0 In libsrtp 1.5.0, crypto_get_random is no longer resolved
-	  simply by including srtp.h. Now, one must include crypto_kernel.h
-	  as well. As it turns out, this header file has been provided by
-	  the library since 2006, so this is a relatively benign change.
-	  ASTERISK-24436 #close Reported by: Patrick Laimbock ........
-	  Merged revisions 426140 from
-	  http://svn.asterisk.org/svn/asterisk/branches/1.8 ........ Merged
-	  revisions 426141 from
-	  http://svn.asterisk.org/svn/asterisk/branches/11 ........ Merged
-	  revisions 426142 from
-	  http://svn.asterisk.org/svn/asterisk/branches/12
-
-2014-10-24 15:17 +0000 [r426120]  Jonathan Rose <jrose@digium.com>
-
-	* main/manager.c: Documentation: Improve documentation for
-	  ExtensionStatus AMI events Review:
-	  https://reviewboard.asterisk.org/r/4085/
+	  git-svn-id: https://origsvn.digium.com/svn/asterisk/trunk@420497 65c4cc65-6c06-0410-ace0-fbb531ad65f3
 
 2014-10-24  Asterisk Development Team <asteriskteam@digium.com>
 
diff -pruN 1:13.23.1~dfsg-2/CHANGES 1:16.1.1~dfsg-1/CHANGES
--- 1:13.23.1~dfsg-2/CHANGES	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/CHANGES	2018-12-26 18:45:13.000000000 +0000
@@ -9,7 +9,182 @@
 ==============================================================================
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.22.0 to Asterisk 13.23.0 ----------
+--- Functionality changes from Asterisk 16.0.0 to Asterisk 16.1.0 ------------
+------------------------------------------------------------------------------
+
+pbx_config
+------------------
+ * pbx_config will now find and process multiple 'globals' sections from
+   extensions.conf.  Variables are processed in the order they are found
+   and duplicate variables overwrite the previous value.
+
+chan_pjsip
+------------------
+ * New dialplan function PJSIP_PARSE_URI added to parse an URI and return
+   a specified part of the URI.
+
+Core
+------------------
+ * ast_bt_get_symbols() now returns a vector of strings instead of an
+   array of strings.  This must be freed with ast_bt_free_symbols.
+
+res_pjsip
+------------------
+ * New options 'trust_connected_line' and 'send_connected_line' have been
+   added to the endpoint. The option 'trust_connected_line' is to control
+   if connected line updates are accepted from this endpoint.
+   The option 'send_connected_line' is to control if connected line updates
+   can be sent to this endpoint.
+   The default value is 'yes' for both options.
+
+res_rtp_asterisk
+------------------
+ * The existing strictrtp option in rtp.conf has a new choice availabe, called
+   'seqno', which behaves the same way as setting strictrtp to 'yes', but will
+   ignore the time interval during learning so that bursts of packets can still
+   trigger learning our source.
+
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 15 to Asterisk 16 --------------------
+------------------------------------------------------------------------------
+
+app_fax
+------------------
+ * The app_fax module is now deprecated, users should migrate to the
+   replacement module res_fax.
+
+app_originate
+------------------
+ * An 'a' option has been added to the Originate dialplan application which
+   will execute the originate in an asynchronous fashion. If set then the
+   application will return immediately without waiting for the originated
+   channel to answer.
+
+Build System
+------------------
+ * MALLOC_DEBUG no longer has an effect on Asterisk's ABI.  Asterisk built
+   with MALLOC_DEBUG can now successfully load binary modules built without
+   MALLOC_DEBUG and vice versa.  Third-party pre-compiled modules no longer
+   need to have a special build with it enabled.
+
+ * Asterisk now depends on libjansson >= 2.11.  If this version is not
+   available on your distro you can use `./configure --with-jansson-bundled`.
+
+app_macro
+------------------
+ * The app_macro module is now deprecated and by default it is no longer
+   built.  Users should migrate to app_stack (Gosub).  A warning is logged
+   the first time any Macro is used.
+
+app_setcallerid
+------------------
+ * The app_setcallerid module has been removed. The CALLERID dialplan function
+   should be used instead.
+
+chan_sip
+------------------
+ * New function SIP_HEADERS() enumerates all headers in the incoming INVITE.
+
+ * The variable GET_TRANSFERRER_DATA set in the peer channel causes matching
+   headers be retrieved from the REFER message and made accessible to the
+   dialplan in the hash TRANSFER_DATA.
+
+chan_dahdi
+------------------
+ * Timeouts for reading digits from analog phones are now configurable in
+   chan_dahdi.conf: firstdigit_timeout, interdigit_timeout, matchdigit_timeout.
+
+AMI
+------------------
+ * The ContactStatus and Status fields for the manager events ContactStatus
+   and ContactStatusDetail are now set to "NonQualified" when a contact exists
+   but has not been qualified.
+
+ * The "Newexten" event is now part of the "dialplan" class. The documentation
+   for Asterisk 15 already specified this, but the implementation was actually
+   using the "call" class instead.
+
+ARI
+------------------
+ * The ContactInfo event's contact_status field is now set to "NonQualified"
+   when a contact exists but has not been qualified.
+
+app_queue
+------------------
+ * Added the ability to set the wrapuptime in the configuration of member.
+   When set the wrapuptime on the member is used instead of the wrapuptime
+   defined for the queue itself.
+
+ * Added predial handler support for caller and callee channels with the
+   B and b options respectively.  This is similar to the predial support
+   in app_dial.
+
+res_config_sqlite
+------------------
+ * The res_config_sqlite module is now deprecated, users should migrate to the
+   replacement module res_config_sqlite3.
+
+res_monitor
+------------------
+ * The res_monitor module is now deprecated, users should migrate to the
+   replacement module app_mixmonitor.
+
+res_pjsip
+------------------
+ * A new AMI action, PJSIPShowAors, has been added which displays information
+   about all configured PJSIP AORs.
+
+ * A new AMI action, PJSIPShowAuths, has been added which displays information
+   about all configured PJSIP Auths.
+
+ * A new AMI action, PJSIPShowContacts, has been added which displays information
+   about all configured PJSIP Contacts.
+
+res_pjsip_registrar_expire
+------------------
+ * The res_pjsip_registrar_expire module has been removed.  The functionality has
+   been moved into res_pjsip_registrar.
+
+func_audiohookinherit
+------------------
+ * The func_audiohookinherit module has been removed. Due to architectural changes
+   in Asterisk 12, audiohook inheritance is performed automatically and this
+   function now lacks function.
+
+cdr_syslog
+------------------
+ * The cdr_syslog module is now deprecated and by default it is no longer
+   built.
+
+cdr_sqlite
+------------------
+ * The cdr_sqlite module has been removed. Users should move to using the
+   cdr_sqlite3_custom module instead.
+
+format_jpeg
+------------------
+ * The format_jpeg module has been removed.
+
+pbx_dundi
+------------------
+ * DUNDi now supports IPv6
+
+Core:
+------------------
+ * libedit is no longer available as an embedded library and must be provided
+   by the system.
+ * The STATIC_BUILD functionality has been removed as it has not been maintained
+   and has not worked in quite some time.
+ * The module loader now enforces inter-module dependencies.  This ensures that
+   a module is not started before another it depends on, even if preload is used.
+   If a dependency is not available or fails to startup this will block any
+   dependants from startup.
+ * Parts of the Asterisk core which can load configuration from realtime are now
+   built-in modules.  It is no longer necessary to preload realtime drivers as
+   they are always initialized before the built-in modules.
+
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 15.5.0 to Asterisk 15.6.0 ------------
 ------------------------------------------------------------------------------
 
 res_pjsip
@@ -27,7 +202,7 @@ res_pjsip_endpoint_identifier_ip
    match_header = SIPHeader: /regex/
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.21.0 to Asterisk 13.22.0 ----------
+--- Functionality changes from Asterisk 15.4.0 to Asterisk 15.5.0 ------------
 ------------------------------------------------------------------------------
 
 Core
@@ -41,16 +216,22 @@ Core
 
 app_sendtext
 ------------------
-    Support Enhanced Messaging.  SendText now accepts new channel variables
-    that can be used to override the To and From display names and set the
-    Content-Type of a message.  Since you can now set Content-Type, other
-    text/* content types are now valid.
+ * Support Enhanced Messaging.  SendText now accepts new channel variables
+   that can be used to override the To and From display names and set the
+   Content-Type of a message.  Since you can now set Content-Type, other
+   text/* content types are now valid.
 
 app_confbridge
 ------------------
-  * ConfbridgeList now shows talking status. This utilizes the same voice
-    detection as the ConfbridgeTalking event, so bridges must be configured
-    with "talk_detection_events=yes" for this flag to have meaning.
+ * ConfbridgeList now shows talking status. This utilizes the same voice
+   detection as the ConfbridgeTalking event, so bridges must be configured
+   with "talk_detection_events=yes" for this flag to have meaning.
+
+ * ConfBridge can now send events to participants via in-dialog MESSAGEs.
+   All current Confbridge events are supported, such as ConfbridgeJoin,
+   ConfbridgeLeave, etc.  In addition to those events, a new event
+   ConfbridgeWelcome has been added that will send a list of all
+   current participants to a new participant.
 
 res_pjsip
 ------------------
@@ -77,15 +258,9 @@ res_pjsip
     functionality.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.20.0 to Asterisk 13.21.0 ----------
+--- Functionality changes from Asterisk 15.3.0 to Asterisk 15.4.0 ------------
 ------------------------------------------------------------------------------
 
-Build System
-------------------
- * RADIUS backends for CEL and CDR can now also be built using the radcli
-   client library, in addition to the existing support for building them
-   using either freeradius or radiusclient-ng.
-
 Core
 ------------------
  * A new configuration option "genericplc_on_equal_codecs" was added to the
@@ -106,7 +281,7 @@ res_pjsip_notify
    channel.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.19.0 to Asterisk 13.20.0 ----------
+--- Functionality changes from Asterisk 15.2.0 to Asterisk 15.3.0 ------------
 ------------------------------------------------------------------------------
 
 Core
@@ -138,7 +313,7 @@ res_pjsip
    identifier method split into the "ip" and "header" endpoint identifier
    methods.
 
- * The pjsip_transport_event feature introduced in 13.18.0 has been refactored.
+ * The pjsip_transport_event feature introduced in 15.1.0 has been refactored.
    Any external modules that may have used that feature (highly unlikey) will
    need to be changed as the API has been altered slightly.
 
@@ -156,12 +331,6 @@ res_pjsip_endpoint_identifier_ip
    you can now predict which endpoint is matched when a request comes in that
    matches both.
 
-res_pjsip_registrar_expire
-------------------
- * The res_pjsip_registrar_expire module is now deprecated and will be removed
-   from Asterisk 16.  The existing functionality was moved into
-   res_pjsip_registrar.
-
 res_pjsip_pubsub
 ------------------
  * In an earlier release, inbound registrations on a reliable transport
@@ -181,7 +350,7 @@ res_pjsip_transport_management
    be available if res_pjsip is loaded.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.18.0 to Asterisk 13.19.0 ----------
+--- Functionality changes from Asterisk 15.1.0 to Asterisk 15.2.0 ------------
 ------------------------------------------------------------------------------
 
 Core
@@ -198,6 +367,14 @@ chan_sip
  * Calls to invalid extensions are now reported as an ACL failure security event
    "no_extension_match".
 
+res_rtp_asterisk
+------------------
+ * The X.509 certificate used for DTLS negotation can now be automatically
+   generated. This is supported by res_pjsip by specifying
+   "dtls_auto_generate_cert = yes" on a PJSIP endpoint. For chan_sip, you
+   would set "dtlsautogeneratecert = yes" either in the [general] section of
+   sip.conf or on a specific peer.
+
 res_pjsip
 ------------------
  * The "identify_by" on endpoints can now be set to "ip" to restrict an endpoint
@@ -205,21 +382,50 @@ res_pjsip
    default has been changed to "username,ip".
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.17.0 to Asterisk 13.18.0 ----------
+--- Functionality changes from Asterisk 15.0.0 to Asterisk 15.1.0 ------------
 ------------------------------------------------------------------------------
 
-Core
+res_pjsip
+------------------
+ * The "remove_existing" option now allows a registration to succeed by
+   displacing any existing contacts that now exceed the "max_contacts" count.
+   Any removed contacts are the next to expire.  The behaviour change is
+   beneficial when "rewrite_contact" is enabled and "max_contacts" is greater
+   than one.  The removed contact is likely the old contact created by
+   "rewrite_contact" that the device is refreshing.
+
+AMI
+------------------
+ * Added a new CancelAtxfer action that cancels an attended transfer.
+
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 14 to Asterisk 15 --------------------
+------------------------------------------------------------------------------
+
+app_queue
 ------------------
- * VP9 is now a supported passthrough video codec and it can be used by
-   specifying "vp9" in the allow line.
+ * PAUSEALL/UNPAUSEALL now sets the pause reason in the queue_log if it has
+   been defined.
+
+ * A new option, "announce-position-only-up," has been added that, when set to
+   yes, causes position announcements to only be played when the caller's
+   queue position has improved since the last time that we annouced their
+   position. This default is no.
 
 Build System
 ------------------
+ * '--with-pjproject-bundled' is now the default when running ./configure
+   It can be disabled with '--without-pjproject-bundled'.
+
  * A '--with-download-cache' option is now available which is equivalent to
    setting '--with-sounds-cache' and '--with-externals-cache' to the same
    value.  The download cache can also be set via the AST_DOWNLOAD_CACHE
    environment variable.
 
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 14.6.0 to Asterisk 14.7.0 ------------
+------------------------------------------------------------------------------
+
 res_pjsip
 ------------------
  * The "external_media_address" on transports is now resolved using dnsmgr and
@@ -230,13 +436,6 @@ res_pjsip
    unsolicited MWI NOTIFY requests and make them available to other modules via
    the stasis message bus.
 
- * The "remove_existing" option now allows a registration to succeed by
-   displacing any existing contacts that now exceed the "max_contacts" count.
-   Any removed contacts are the next to expire.  The behaviour change is
-   beneficial when "rewrite_contact" is enabled and "max_contacts" is greater
-   than one.  The removed contact is likely the old contact created by
-   "rewrite_contact" that the device is refreshing.
-
 res_musiconhold
 ------------------
  * By default, when res_musiconhold reloads or unloads, it sends a HUP signal
@@ -253,8 +452,6 @@ res_musiconhold
    which sends signals to the application and its descendants directly, or
    "process" which sends signals only to the application itself.
 
-res_pjsip
-------------------
  * New dialplan function PJSIP_DTMF_MODE added to get or change the DTMF mode
    of a channel on a per-call basis.
 
@@ -264,16 +461,8 @@ res_xmpp
    instructions in xmpp.conf.sample to retrieve and configure the necessary
    tokens.
 
-app_queue
-------------------
- * Add priority to callers in AMI QueueStatus response.
-
-AMI
-------------------
- * Added a new CancelAtxfer action that cancels an attended transfer.
-
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.16.0 to Asterisk 13.17.0 ----------
+--- Functionality changes from Asterisk 14.5.0 to Asterisk 14.6.0 ------------
 ------------------------------------------------------------------------------
 
 app_voicemail
@@ -327,7 +516,7 @@ Build System
    options to match those supplied for the asterisk configure.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.15.0 to Asterisk 13.16.0 ----------
+--- Functionality changes from Asterisk 14.4.0 to Asterisk 14.5.0 ------------
 ------------------------------------------------------------------------------
 
 res_rtp_asterisk
@@ -367,7 +556,89 @@ res_hep_rtcp
    SIP Call-ID instead of the Asterisk channel name.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.14.0 to Asterisk 13.15.0 ----------
+--- Functionality changes from Asterisk 14.3.0 to Asterisk 14.4.0 ------------
+------------------------------------------------------------------------------
+
+Build System
+------------------
+ * LOW_MEMORY no longer has an effect on Asterisk ABI.  Symbols that were
+   previously suppressed by LOW_MEMORY are now replaced by stub functions.
+   Asterisk built with LOW_MEMORY can now successfully load binary modules
+   built without LOW_MEMORY and vice versa.
+
+ * RADIUS backends for CEL and CDR can now also be built using the radcli
+   client library, in addition to the existing support for building them
+   using either freeradius or radiusclient-ng.
+
+Core
+------------------
+ * ASTERISK_REGISTER_FILE was no longer useful and has been removed.  Sources
+   which use mtx_prof must now manually declare and initialize the variable.
+
+chan_sip
+------------------
+ * If an offer is received with optional SRTP (a media stream with RTP/AVP but
+   which contains a crypto line) chan_sip will now accept it and enable SRTP.
+   If you would like to do optional SRTP on outbound you will need to create
+   a dialplan that dials with it enabled initially and if it fails fall back to
+   without.
+
+res_pjsip
+------------------
+ * Added endpoint configuration parameter "preferred_codec_only".
+   This allow asterisk response to a SIP invite with the single most
+   preferred codec rather than advertising all joint codec capabilities.
+   This limits the other side's codec choice to exactly what we prefer.
+
+cdr_radius
+------------------
+ * To fix a memory leak the syslog channel is now empty if it has not been set
+   and used by a syslog channel in the logger.
+
+cel_radius
+------------------
+ * To fix a memory leak the syslog channel is now empty if it has not been set
+   and used by a syslog channel in the logger.
+
+RTP
+------------------
+ * New setting "rtp_pt_dynamic = 35" in asterisk.conf:
+   Normally the Dynamic RTP Payload Type numbers are 96-127, which allow just 32
+   formats. To avoid the message "No Dynamic RTP mapping available", the range
+   was changed to 35-63,96-127. This is allowed by RFC 3551 section 3. However,
+   when you use more than 32 formats and calls are not accepted by a remote
+   implementation, please report this and go back to rtp_pt_dynamic = 96.
+
+ * A new setting, "rtp_use_dynamic", has been added in asterisk.conf". When set
+   to "yes" RTP dynamic payload types are assigned dynamically per RTP instance.
+   When set to "no" RTP dynamic payload types are globally initialized to pre-
+   designated numbers and function similar to static payload types.
+
+app_originate
+------------------
+ * Added support to gosub predial routines on both original channel and on the
+   created channel using options parameter (like app_dial) B() and b().  This
+   allows for adding variables to newly created channel or, e.g. setting callerid.
+
+CLI Commands
+------------------
+ * 'dialplan show' output will now show [config_file:line_number] instead of
+   [registrar] when that information is available. Currently only extensions
+   registered by pbx_config when loading/reloading will use this format.
+
+app_queue
+------------------
+ * Add 'QueueUpdate' application which can be used to track outbound calls
+   using app_queue.
+
+pbx_spool
+------------------
+ * Asterisk will now set the AST_OUTGOING_ATTEMPT channel variable so that
+   attempt-specific behavior is possible. This is a 1-based number that
+   simply increases by 1 for each attempt.
+
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 14.3.0 to Asterisk 14.4.0 ------------
 ------------------------------------------------------------------------------
 
 AMI
@@ -456,7 +727,7 @@ res_rtp_asterisk
    a PJSIP endpoint in pjsip.conf to enable the feature.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.13.0 to Asterisk 13.14.0 ----------
+--- Functionality changes from Asterisk 14.2.0 to Asterisk 14.3.0 ------------
 ------------------------------------------------------------------------------
 
 res_pjproject
@@ -506,7 +777,7 @@ ARI
  * 'ari set debug' now displays requests and responses as well as events.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.12.0 to Asterisk 13.13.0 ----------
+--- Functionality changes from Asterisk 14.1.0 to Asterisk 14.2.0 ------------
 ------------------------------------------------------------------------------
 
 AMI
@@ -540,6 +811,12 @@ ARI
    Applications subscribed to a bridge will receive this event when the source
    of video changes in a mixing bridge.
 
+ * The ARI major version has been bumped. There are not any known breaking changes
+   in ARI. The major version has been bumped because otherwise we can end up with
+   overlapping version numbers between different Asterisk versions. Now each major
+   version of Asterisk will bring with it a change in the major version of ARI.
+   The ARI version in Asterisk 14 is now 2.0.0.
+
 res_pjsip
 ------------------
  * Automatic dual stack support is now implemented. Depending on DNS resolution
@@ -567,29 +844,20 @@ CLI Commands
       Enable/disable debugging of an ARI application. When debugged, verbose
       information will be sent to the Asterisk CLI.
 
-RTP
-------------------
- * New setting "rtp_pt_dynamic = 96" in asterisk.conf:
-   Normally the Dynamic RTP Payload Type numbers are 96-127, which allow 32
-   formats. When you use more and receive the message "No Dynamic RTP mapping
-   available", extend the dynamic range by going for rtp_pt_dynamic = 35 (or 0)
-   instead of 96. This allows 29 (or 64) additional formats. On default this is
-   disabled and the range is 96-127 because any number below might be rejected
-   by a remote implementation; although no such broken implementation is known.
 
 Queue
 ------------------
  * A new dialplan variable, ABANDONED, is set when the call is not answered
    by an agent.
 
-Core
+res_ari
 ------------------
- * The TLS core in Asterisk now supports X.509 certificate subject alternative
-   names. This way one X.509 certificate can be used for hosts that can be
-   reached under multiple DNS names or for multiple hosts.
+ * The configuration file ari.conf now supports a channelvars option, which
+   specifies a list of channel variables to include in each channel-oriented
+   ARI event.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.11.0 to Asterisk 13.12.0 ----------
+--- Functionality changes from Asterisk 14.0.0 to Asterisk 14.1.0 ------------
 ------------------------------------------------------------------------------
 
 Build System
@@ -662,12 +930,6 @@ app_confbridge
     instance, allows a channel to immediately exit the ConfBridge without having
     to wait for a leave announcement to play.
 
-Core
-------------------
-  * If Asterisk is built with systemd support, and run under systemd, it will
-    notify systemd of its state using sd_notify. Use 'Type=notify' in
-    asterisk.service.
-
 app_dial
 ------------------
  * Added the "Q" option which sets the Q.850/Q.931 cause on unanswered channels
@@ -682,17 +944,182 @@ res_ari
    setting in asterisk.conf.
 
 ------------------------------------------------------------------------------
---- Functionality changes from Asterisk 13.10.0 to Asterisk 13.11.0 ----------
+--- Functionality changes from Asterisk 13 to Asterisk 14 --------------------
 ------------------------------------------------------------------------------
 
+AMI
+-----------------
+ * A new event, "DialState" has been added. This is similar to "DialBegin" and
+ "DialEnd" in that it tracks the state of a dialed call. The difference is that
+ this indicates some intermediate state change in the dial attempt, such as
+ "RINGING", "PROGRESS", or "PROCEEDING".
+
+ARI
+-----------------
+ * A new ARI method has been added to the channels resource. "create" allows for
+   you to create a new channel and place that channel into a Stasis application.
+   This is similar to origination except that the specified channel is not
+   dialed. This allows for an application writer to create a channel, perform
+   manipulations on it, and then delay dialing the channel until later.
+
+ * To complement the "create" method, a "dial" method has been added to the
+   channels resource in order to place a call to a created channel.
+
+ * All operations that initiate playback of media on a resource now support
+   a list of media URIs. The list of URIs are played in the order they are
+   presented to the resource. A new event, "PlaybackContinuing", is raised when
+   a media URI finishes but before the next media URI starts. When a list is
+   played, the "Playback" model will contain the optional attribute
+   "next_media_uri", which specifies the next media URI in the list to be played
+   back to the resource. The "PlaybackFinished" event is raised when all media
+   URIs are done.
+
+ * Stored recordings now allow for the media associated with a stored recording
+   to be retrieved. The new route, GET /recordings/stored/{name}/file, will
+   transmit the raw media file to the requester as binary.
+
+
+ * "Dial" events have been modified to not only be sent when dialing begins and ends.
+ They now are also sent for intermediate states, such as "RINGING", "PROGRESS", and
+ "PROCEEDING".
+
+Applications
+------------------
+
+BridgeAdd
+------------------
+ * A new application in Asterisk, this will join the calling channel
+   to an existing bridge containing the named channel prefix.
+
+ChanSpy
+------------------
+ * Added the 'l' option, which forces ChanSpy's audiohook to use a long queue
+   to store the audio frames. This option is useful if audio loss is
+   experienced when using ChanSpy, but may introduce some delay in the audio
+   feed on the listening channel.
+
+Codecs
+------------------
+ * Added format attribute negotiation for the iLBC audio codec. Format attribute
+   negotiation is provided by the res_format_attr_ilbc module. iLBC 20 is the
+   default now. Falls back to iLBC 30, when the remote party requests this.
+
+ConfBridge
+------------------
+ * Added the ability to pass options to MixMonitor when recording is used with
+   ConfBridge. This includes the addition of the following configuration
+   parameters for the 'bridge' object:
+   - record_file_timestamp: whether or not to append the start time to the
+     recorded file name
+   - record_options: the options to pass to the MixMonitor application
+   - record_command: a command to execute when recording is finished
+   Note that these options may also be with the CONFBRIDGE function.
+
+ControlPlayback
+------------------
+ * Remote files can now be retrieved and played back. See the Playback
+   dialplan application for more details.
+
+FollowMe
+------------------
+ * It is now possible to disable the prompt from a callee by setting
+   'enable_callee_prompt = no' in followme.conf.
+
+Playback
+------------------
+ * Remote files can now be retrieved and played back via the Playback and other
+   media playback dialplan applications. This is done by directly providing
+   the URL to play to the dialplan application:
+     same => n,Playback(http://1.1.1.1/howler-monkeys-fl.wav)
+   Note that unlike 'normal' media files, the entire URI to the file must be
+   provided, including the file extension. Currently, on HTTP and HTTPS URI
+   schemes are supported.
+
+Queue
+-------------------
+ * Added field ReasonPause on QueueMemberStatus if set when paused, the reason
+   the queue member was paused.
+
+ * Added field LastPause on QueueMemberStatus for time when started the last
+   pause for a queue member.
+
+ * Show the time when started the last pause for queue member on CLI for command
+   'queue show'.
+
+SMS
+------------------
+ * Added the 'n' option, which prevents the SMS from being written to the log
+   file. This is needed for those countries with privacy laws that require
+   providers to not log SMS content.
+
+
+Channel Drivers
+------------------
+
 chan_dahdi
 ------------------
- * Added "faxdetect_timeout" option.
-   The option determines how many seconds into a call before faxdetect
-   is disabled for the call.  Setting the value to zero disables the timeout.
+ * The CALLERID(ani2) value for incoming calls is now populated in featdmf
+   signaling mode.  The information was previously discarded.
+
+ * Added the force_restart_unavailable_chans compatibility option.  When
+   enabled it causes Asterisk to restart the ISDN B channel if an outgoing
+   call receives cause 44 (Requested channel not available).
+
+chan_iax2
+------------------
+ * The iax.conf forcejitterbuffer option has been removed.  It is now always
+   forced if you set iax.conf jitterbuffer=yes.  If you put a jitter buffer
+   on a channel it will be on the channel.
+
+ * A new configuration parameters, 'calltokenexpiration', has been added that
+   controls the duration before a call token expires. Default duration is 10
+   seconds. Setting this to a higher value may help in lagged networks or those
+   experiencing high packet loss.
+
+ * Plaintext auth mode is deprecated and removed from possible default modes.
+
+chan_rtp (was chan_multicast_rtp)
+------------------
+ * Added unicast RTP support and renamed chan_multicast_rtp to chan_rtp.
+
+ * The format for dialing a unicast RTP channel is:
+   UnicastRTP/<destination-addr>[/[<options>]]
+   Where <destination-addr> is something like '127.0.0.1:5060'.
+   Where <options> are in standard Asterisk flag options format:
+   c(<codec>) - Specify which codec/format to use such as 'ulaw'.
+   e(<engine>) - Specify which RTP engine to use such as 'asterisk'.
+
+ * New options were added for a multicast RTP channel.  The format for
+   dialing a multicast RTP channel is:
+   MulticastRTP/<type>/<destination-addr>[/[<control-addr>][/[<options>]]]
+   Where <type> can be either 'basic' or 'linksys'.
+   Where <destination-addr> is something like '224.0.0.3:5060'.
+   Where <control-addr> is something like '127.0.0.1:5060'.
+   Where <options> are in standard Asterisk flag options format:
+   c(<codec>) - Specify which codec/format to use such as 'ulaw'.
+   i(<address>) - Specify the interface address from which multicast RTP
+     is sent.
+   l(<enable>) - Set whether packets are looped back to the sender.  The
+     enable value can be 0 to set looping to off and non-zero to set
+     looping on.
+   t(<ttl>) - Set the time-to-live (TTL) value for multicast packets.
 
 chan_sip
 ------------------
+ * New 'rtpbindaddr' global setting. This allows a user to define which
+   ipaddress to bind the rtpengine to. For example, chan_sip might bind
+   to eth0 (10.0.0.2) but rtpengine to eth1 (192.168.1.10).
+
+ * DTLS related configuration options can now be set at a general level.
+   Enabling DTLS support, though, requires enabling it at the user
+   or peer level.
+
+ * Added the possibility to set the From: header through the the SIP dial
+   string (populating the fromuser/fromdomain fields), complementing the
+   [!dnid] option for the To: header that has existed since 1.6.0 (1d6b192).
+   NOTE: This is again separated by an exclamation mark, so the To: header may
+   not contain one of those.
+
  * Session-Timers (RFC 4028) work for TCP (and TLS) transports as well now.
    Previously Asterisk dropped calls only with UDP transports. However with
    longer international calls via TCP, the SIP channel might break, because
@@ -701,6 +1128,201 @@ chan_sip
    enabled at default), you might see additional dropped calls. Consequently
    please, consider to go for session-timers=refuse in your sip.conf.
 
+chan_pjsip
+------------------
+ * New 'user_eq_phone' endpoint setting. This adds a 'user=phone' parameter
+   to the request URI and From URI if the user is determined to be a phone
+   number.
+
+ * New 'moh_passthrough' endpoint setting. This will pass hold and unhold
+   requests through using SIP re-invites with sendonly and sendrecv accordingly.
+
+ * Added the pjsip.conf system type disable_tcp_switch option.  The option
+   allows the user to disable switching from UDP to TCP transports described
+   by RFC 3261 section 18.1.1.
+
+ * New 'line' and 'endpoint' options added on outbound registrations. This
+   allows some identifying information to be added to the Contact of the
+   outbound registration. If this information is present on messages received
+   from the remote server the message will automatically be associated with the
+   configured endpoint on the outbound registration.
+
+
+Core
+------------------
+ * The core of Asterisk uses a message bus called "Stasis" to distribute
+   information to internal components. For performance reasons, the message
+   distribution was modified to make use of a thread pool instead of a
+   dedicated thread per consumer in certain cases. The initial settings for
+   the thread pool can now be configured in 'stasis.conf'.
+
+ * A new core DNS API has been implemented which provides a common interface
+   for DNS functionality. Modules that use this functionality will require that
+   a DNS resolver module is loaded and available.
+
+ * Modified processing of command-line options to first parse only what
+   is necessary to read asterisk.conf. Once asterisk.conf is fully loaded,
+   the remaining options are processed.  The -X option now applies to
+   asterisk.conf only.  To enable #exec for other config files you must
+   set execincludes=yes in asterisk.conf.  Any other option set on the
+   command-line will now override the equivalent setting from asterisk.conf.
+
+ * The TLS core in Asterisk now supports X.509 certificate subject alternative
+   names. This way one X.509 certificate can be used for hosts that can be
+   reached under multiple DNS names or for multiple hosts.
+
+ * The Asterisk logging system now supports JSON structured logging. Log
+   channels specified in logger.conf or added dynamically via CLI commands now
+   support an optional specifier prior to their levels that determines their
+   formatting. To set a log channel to format its entries as JSON, a formatter
+   of '[json]' can be set, e.g.,
+      full => [json]debug,verbose,notice,warning,error
+
+ * The core now supports a 'media cache', which stores temporary media files
+   retrieved from external sources. CLI commands have been added to manipulate
+   and display the cached files, including:
+   - 'media cache show <all>' - show all cached media files, or details about
+     one particular cached media file
+   - 'media cache refresh <item>' - force a refresh of a particular media file
+     in the cache
+   - 'media cache delete <item>' - remove an item from the cache
+   - 'media cache create <uri>' - retrieve a URI and store it in the cache
+
+ * The ability for device state hints to be automatically created as a result of
+   device state changes now exists in the PBX. This functionality is referred to
+   as "autohints" and is configurable in extensions.conf by placing "autohints=yes"
+   in the context. If enabled a device state hint will be automatically created
+   with the name of the device.
+
+* If Asterisk is built with systemd support, and run under systemd, it will
+  notify systemd of its state using sd_notify. Use 'Type=notify' in
+  asterisk.service.
+
+Functions
+------------------
+ * The func_odbc global option "single_db_connection" default value has been
+   changed to 'no'.
+
+
+Formats
+------------------
+ * New module format_ogg_speex added which supports Speex codec inside
+   Ogg containers (filename extension .spx).
+
+
+CHANNEL
+------------------
+ * Added CHANNEL(onhold) item that returns 1 (onhold) and 0 (not-onhold) for
+   the hold status of a channel.
+
+CURL
+------------------
+ * The CURL function now supports a write option, which will save the retrieved
+   file to a location on disk. As an example:
+     same => n,Set(CURL(https://1.1.1.1/foo.wav)=/tmp/foo.wav)
+   will save 'foo.wav' to /tmp.
+
+DTMF Features
+------------------
+ * The transferdialattempts default value has been changed from 1 to 3. The
+   transferinvalidsound has been changed from "pbx-invalid" to
+   "privacy-incorrect". These were changed to make DTMF transfers be more
+   user-friendly by default.
+
+
+Resources
+------------------
+
+res_http_media_cache
+------------------
+ * A backend for the core media cache, this module retrieves media files from
+   a remote HTTP(S) server and stores them in the core media cache for later
+   playback.
+
+res_musiconhold
+------------------
+ * Added sort=randstart to the sort options. It sorts the files by name and
+   then chooses the first file to play at random.
+ * Added preferchannelclass=no option to prefer the application-passed class
+   over the channel-set musicclass. This allows separate hold-music from
+   application (e.g. Queue or Dial) specified music.
+
+res_resolver_unbound
+------------------
+ * Added a res_resolver_unbound module which uses the libunbound resolver library
+   to perform DNS resolution. This module requires the libunbound library to be
+   installed in order to be used.
+
+res_pjsip
+------------------
+ * A new SIP resolver using the core DNS API has been implemented. This relies on
+   external SIP resolver support in PJSIP which is only available as of PJSIP
+   2.4. If this support is unavailable the existing built-in PJSIP SIP resolver
+   will be used instead. The new SIP resolver provides NAPTR support, improved
+   SRV support, and AAAA record support.
+
+res_pjsip_info_empty
+--------------------
+ * A new module that can respond to empty Content-Type INFO packets during call.
+   Some SBCs will terminate a call if their empty INFO packets are not responded
+   to within a predefined time.
+
+res_pjsip_outbound_registration
+-------------------------------
+* A new 'fatal_retry_interval' option has been added to outbound registration.
+  When set (default is zero), and upon receiving a failure response to an
+  outbound registration, registration is retried at the given interval up to
+  'max_retries'.
+
+res_pjsip_outbound_publish
+------------------
+ * Added a new multi_user option that when set to 'yes' allows a given configuration
+   to be used for multiple users.
+
+
+CEL Backends
+------------------
+
+cel_pgsql
+------------------
+ * Added a new option, 'usegmtime', which causes timestamps in CEL events
+   to be logged in GMT.
+
+ * Added support to set schema where located the table cel. This settings is
+   configurable for cel_pgsql via the 'schema' in configuration file
+   cel_pgsql.conf.
+
+
+CDR Backends
+------------------
+
+cdr_adaptive_odbc
+------------------
+ * Added the ability to set the character to quote identifiers. This
+   allows adding the character at the start and end of table and column
+   names. This setting is configurable for cdr_adaptive_odbc via the
+   quoted_identifiers in configuration file cdr_adaptive_odbc.conf.
+
+cdr_odbc
+------------------
+ * Added a new configuration option, "newcdrcolumns", which enables use of the
+   post-1.8 CDR columns 'peeraccount', 'linkedid', and 'sequence'.
+
+cdr_csv
+------------------
+ * Added a new configuration option, "newcdrcolumns", which enables use of the
+   post-1.8 CDR columns 'peeraccount', 'linkedid', and 'sequence'.
+
+------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 13.10.0 to Asterisk 13.11.0 ----------
+------------------------------------------------------------------------------
+
+chan_dahdi
+------------------
+ * Added "faxdetect_timeout" option.
+   The option determines how many seconds into a call before faxdetect
+   is disabled for the call.  Setting the value to zero disables the timeout.
+
 res_pjsip
 ------------------
  * Added "fax_detect_timeout" to endpoint.
@@ -739,36 +1361,6 @@ Core
  * A channel variable FORWARDERNAME is now set which indicates which channel
    was responsible for a forwarding requests received on dial attempt.
 
-chan_multicast_rtp
-------------------
- * Deprecated in favor of chan_rtp which is basically chan_multicast_rtp
-   renamed to chan_rtp with UnicastRTP channels added and some internal code
-   improvements.
-
-chan_rtp
-------------------
- * The format for dialing a unicast RTP channel is:
-   UnicastRTP/<destination-addr>[/[<options>]]
-   Where <destination-addr> is something like '127.0.0.1:5060'.
-   Where <options> are in standard Asterisk flag options format:
-   c(<codec>) - Specify which codec/format to use such as 'ulaw'.
-   e(<engine>) - Specify which RTP engine to use such as 'asterisk'.
-
- * More options are available over what chan_multicast_rtp supports.
-   The format for dialing a multicast RTP channel is:
-   MulticastRTP/<type>/<destination-addr>[/[<control-addr>][/[<options>]]]
-   Where <type> can be either 'basic' or 'linksys'.
-   Where <destination-addr> is something like '224.0.0.3:5060'.
-   Where <control-addr> is something like '127.0.0.1:5060'.
-   Where <options> are in standard Asterisk flag options format:
-   c(<codec>) - Specify which codec/format to use such as 'ulaw'.
-   i(<address>) - Specify the interface address from which multicast RTP
-     is sent.
-   l(<enable>) - Set whether packets are looped back to the sender.  The
-     enable value can be 0 to set looping to off and non-zero to set
-     looping on.
-   t(<ttl>) - Set the time-to-live (TTL) value for multicast packets.
-
 func_odbc
 ------------------
  * Added new global option "single_db_connection".
@@ -815,12 +1407,6 @@ res_hep
    valid value using the specified 'uuid_type', the module may fallback to a
    more readily available source for the correlation UUID.
 
-res_pjsip_info_empty
---------------------
- * A new module that can respond to empty Content-Type INFO packets during call.
-   Some SBCs will terminate a call if their empty INFO packets are not responded
-   to within a predefined time.
-
 res_odbc
 ------------------
  * A new option has been added, 'max_connections', which sets the maximum number
@@ -851,6 +1437,15 @@ res_ari_channels
 --- Functionality changes from Asterisk 13.8.0 to Asterisk 13.9.0 ------------
 ------------------------------------------------------------------------------
 
+res_parking:
+ - The dynamic parking lot creation channel variables PARKINGDYNAMIC,
+   PARKINGDYNCONTEXT, PARKINGDYNEXTEN, and PARKINGDYNPOS are now looked
+   for in the parker's channel instead of the parked channel.  This is only
+   of significance if the parker uses blind transfer or the DTMF one-step
+   parking feature.  You need to use the double underscore '__' inheritance
+   for these variables.  The indefinite inheritance is also recommended
+   for the PARKINGEXTEN variable.
+
 res_pjsip
 ------------------
  * Added new global option (disable_multi_domain) to pjsip.
@@ -1012,11 +1607,6 @@ Voicemail
    app_voicemail will be skipped.  Use 'preload=app_voicemail.so' in
    modules.conf to force app_voicemail to be the voicemail provider.
 
-Queue
--------------------
- * Added field ReasonPause on QueueMemberStatus if set when paused, the reason
-   the queue member was paused.
-
 res_pjsip_sdp_rtp
 ------------------
  * A new option (bind_rtp_to_media_address) has been added to endpoint which
@@ -1065,11 +1655,6 @@ Dialplan Functions
 
 res_pjsip_outbound_registration
 -------------------------------
- * A new 'fatal_retry_interval' option has been added to outbound registration.
-   When set (default is zero), and upon receiving a failure response to an
-   outbound registration, registration is retried at the given interval up to
-   'max_retries'.
-
  * If res_statsd is loaded and a StatsD server is configured, basic statistics
    regarding the state of outbound registrations will now be emitted. This
    includes:
@@ -1139,7 +1724,6 @@ ARI
    ContactStatusChange. In particular, these events are useful when subscribing
    to all event sources, as they provide additional endpoint related
    information beyond the addition/removal of channels from an endpoint.
-   The ARI version has been bumped to 1.9.0 as a result.
 
  * Added the ability to subscribe to all ARI events in Asterisk, regardless
    of whether the application 'controls' the resource. This is useful for
@@ -1213,22 +1797,12 @@ chan_pjsip
  * The configuration setting 'progressinband' now defaults to 'no', which
    matches the actual behavior of previous versions.
 
- * New 'line' and 'endpoint' options added on outbound registrations. This allows some
-   identifying information to be added to the Contact of the outbound registration.
-   If this information is present on messages received from the remote server
-   the message will automatically be associated with the configured endpoint on the
-   outbound registration.
-
 res_pjsip
 ------------------
  * A new CLI command has been added: "pjsip show settings", which shows
    both the global and system configuration settings.
 
- * A new global option has been added: "max_initial_qualify_time", which
-   sets the maximum amount of time from startup that qualifies should be
-   attempted on all contacts.
-
-* A new aor option has been added: "qualify_timeout", which sets the timeout
+ * A new aor option has been added: "qualify_timeout", which sets the timeout
    in seconds for a qualify.  The default is 3 seconds.  This overrides the
    hard coded 32 seconds in pjproject.
 
@@ -1236,6 +1810,10 @@ res_pjsip
    unavailable.  When any contact becomes available, the endpoint will status
    will change back to "Reachable".
 
+ * A new global option has been added: "max_initial_qualify_time", which
+   sets the maximum amount of time from startup that qualifies should be
+   attempted on all contacts.
+
 res_ari_channels
 ------------------
  * Two new events, 'ChannelHold' and 'ChannelUnhold', have been added to the
@@ -1279,6 +1857,9 @@ res_pjsip
 --- Functionality changes from Asterisk 13.1.0 to Asterisk 13.2.0 ------------
 ------------------------------------------------------------------------------
 
+ * New 'PJSIP_AOR' and 'PJSIP_CONTACT' dialplan functions have been added which
+   allow examining PJSIP AORs or contacts from the dialplan.
+
 res_pjsip_outbound_registration
 ------------------
  * The 'pjsip send unregister' command now stops further registrations.
@@ -1287,22 +1868,10 @@ res_pjsip_outbound_registration
    start or restart periodic registration.  It can be used after a
    'send unregister' or after a 401 permanent error.
 
-chan_pjsip
-------------------
- * New 'user_eq_phone' endpoint setting. This adds a 'user=phone' parameter
-   to the request URI and From URI if the user is determined to be a phone number.
-
- * New global setting 'keep_alive_interval'. Setting this to a non-zero value
-   will cause keep alive messages to be sent periodically over connection
-   oriented transports.
-
- * New 'PJSIP_AOR' and 'PJSIP_CONTACT' dialplan functions have been added which
-   allow examining PJSIP AORs or contacts from the dialplan.
-
 res_pjsip_config_wizard
 ------------------
  * This is a new module that adds streamlined configuration capability for
-   chan_pjsip.  It's targetted at users who have lots of basic configuration
+   chan_pjsip.  It's targeted at users who have lots of basic configuration
    scenarios like 'phone' or 'agent' or 'trunk'.  Additional information
    can be found in the sample configuration file at
    config/samples/pjsip_wizard.conf.sample.
@@ -1353,26 +1922,20 @@ AMI
  * Event NewConnectedLine is emitted when the connected line information on
    a channel changes.
 
- * The version of AMI has been bumped to 2.6.0 to account for the backwards
-   compatible feature listed above.
-
 ARI
 ------------------
  * Event ChannelConnectedLine is emitted when the connected line information
    on a channel changes.
 
- * The version of ARI has been bumped to 1.6.0 to account for the backwards
-   compatible feature listed above.
-
 Core Transfers
 -----------------
 
 The features.conf general section has three new configurable options:
     * transferdialattempts
-	* transferretrysound
-	* transferinvalidsound
+    * transferretrysound
+    * transferinvalidsound
 For more information on what these options do, see the Asterisk wiki:
-	https://wiki.asterisk.org/wiki/x/W4fAAQ
+ https://wiki.asterisk.org/wiki/x/W4fAAQ
 
 Channel Drivers
 ------------------
@@ -1432,6 +1995,7 @@ included in Asterisk 13.
 Finally, all users upgrading to Asterisk 13 should read the UPGRADE.txt file
 delivered with this release.
 
+
 Build System
 ------------------
  * Sample config files have been moved from configs/ to a sub-folder of that
@@ -1490,6 +2054,8 @@ Say
    language. The 'language' parameter in say.conf now recognizes a setting of
    'ja', which will enable Japanese language specific mechanisms for playing
    back numbers, dates, and other items.
+ * Counting, enumeration and dates now supports Icelandic grammar with the
+   'language' parameter set to 'is'.
 
 SayCountPL
 ------------------
@@ -5643,6 +6209,7 @@ Voicemail Changes
      the message as urgent after he has recorded a voicemail by following the voice instructions.
     When listening to voicemails using VoiceMailMain urgent messages will be presented before other
      messages
+  * Added "is" language support
 
 Queue changes
 -------------
diff -pruN 1:13.23.1~dfsg-2/channels/chan_alsa.c 1:16.1.1~dfsg-1/channels/chan_alsa.c
--- 1:13.23.1~dfsg-2/channels/chan_alsa.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_alsa.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <errno.h>
 #ifndef ESTRPIPE
 #define ESTRPIPE EPIPE
@@ -1038,8 +1036,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "ALSA Console Channel Driver",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_bridge_media.c 1:16.1.1~dfsg-1/channels/chan_bridge_media.c
--- 1:13.23.1~dfsg-2/channels/chan_bridge_media.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_bridge_media.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/bridge.h"
 #include "asterisk/core_unreal.h"
@@ -118,8 +116,8 @@ static struct ast_channel *media_request
 	const struct ast_channel *requestor, const char *data, struct ast_channel_tech *tech, const char *role)
 {
 	struct ast_channel *chan;
+	ast_callid callid;
 
-	RAII_VAR(struct ast_callid *, callid, NULL, ast_callid_cleanup);
 	RAII_VAR(struct ast_unreal_pvt *, pvt, NULL, ao2_cleanup);
 
 	if (!(pvt = ast_unreal_alloc(sizeof(*pvt), ast_unreal_destructor, cap))) {
@@ -216,6 +214,6 @@ static int load_module(void)
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Bridge Media Channel Driver",
 	.support_level = AST_MODULE_SUPPORT_CORE,
-    .load = load_module,
-    .unload = unload_module,
+	.load = load_module,
+	.unload = unload_module,
 );
diff -pruN 1:13.23.1~dfsg-2/channels/chan_console.c 1:16.1.1~dfsg-1/channels/chan_console.c
--- 1:13.23.1~dfsg-2/channels/chan_console.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_console.c	2018-12-26 18:45:13.000000000 +0000
@@ -62,9 +62,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include <sys/signal.h>  /* SIGURG */
+#include <signal.h>  /* SIGURG */
 
 #include <portaudio.h>
 
@@ -871,7 +869,7 @@ static char *cli_console_dial(struct ast
 	} else
 		ast_cli(a->fd, "No such extension '%s' in context '%s'\n", mye, myc);
 
-	free(s);
+	ast_free(s);
 
 	unref_pvt(pvt);
 
@@ -1536,7 +1534,9 @@ static int load_module(void)
 
 	init_pvt(&globals, NULL);
 
-	if (!(pvts = ao2_container_alloc(NUM_PVT_BUCKETS, pvt_hash_cb, pvt_cmp_cb)))
+	pvts = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NUM_PVT_BUCKETS,
+		pvt_hash_cb, NULL, pvt_cmp_cb);
+	if (!pvts)
 		goto return_error;
 
 	if (load_config(0))
@@ -1582,9 +1582,9 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Console Channel Driver",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
 );
diff -pruN 1:13.23.1~dfsg-2/channels/chan_dahdi.c 1:16.1.1~dfsg-1/channels/chan_dahdi.c
--- 1:13.23.1~dfsg-2/channels/chan_dahdi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_dahdi.c	2018-12-26 18:45:13.000000000 +0000
@@ -54,15 +54,12 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #if defined(__NetBSD__) || defined(__FreeBSD__)
 #include <pthread.h>
-#include <signal.h>
 #else
-#include <sys/signal.h>
 #include <sys/sysmacros.h>
 #endif
+#include <signal.h>
 #include <sys/stat.h>
 #include <math.h>
 
@@ -124,7 +121,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/devicestate.h"
 #include "asterisk/paths.h"
 #include "asterisk/ccss.h"
-#include "asterisk/data.h"
 #include "asterisk/features_config.h"
 #include "asterisk/bridge.h"
 #include "asterisk/stasis_channels.h"
@@ -619,15 +615,6 @@ static int pridebugfd = -1;
 static char pridebugfilename[1024] = "";
 #endif
 
-/*! \brief Wait up to 16 seconds for first digit (FXO logic) */
-static int firstdigittimeout = 16000;
-
-/*! \brief How long to wait for following digits (FXO logic) */
-static int gendigittimeout = 8000;
-
-/*! \brief How long to wait for an extra digit, if there is an ambiguous match */
-static int matchdigittimeout = 3000;
-
 /*! \brief Protect the interface list (of dahdi_pvt's) */
 AST_MUTEX_DEFINE_STATIC(iflock);
 
@@ -796,78 +783,6 @@ const char * const subnames[] = {
 	"Threeway"
 };
 
-#define DATA_EXPORT_DAHDI_PVT(MEMBER)					\
-	MEMBER(dahdi_pvt, cid_rxgain, AST_DATA_DOUBLE)			\
-	MEMBER(dahdi_pvt, rxgain, AST_DATA_DOUBLE)			\
-	MEMBER(dahdi_pvt, txgain, AST_DATA_DOUBLE)			\
-	MEMBER(dahdi_pvt, txdrc, AST_DATA_DOUBLE)			\
-	MEMBER(dahdi_pvt, rxdrc, AST_DATA_DOUBLE)			\
-	MEMBER(dahdi_pvt, adsi, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, answeronpolarityswitch, AST_DATA_BOOLEAN)	\
-	MEMBER(dahdi_pvt, busydetect, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, callreturn, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, callwaiting, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, callwaitingcallerid, AST_DATA_BOOLEAN)	\
-	MEMBER(dahdi_pvt, cancallforward, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, canpark, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, confirmanswer, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, destroy, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, didtdd, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, dialednone, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, dialing, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, digital, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, dnd, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, echobreak, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, echocanbridged, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, echocanon, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, faxhandled, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, usefaxbuffers, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, bufferoverrideinuse, AST_DATA_BOOLEAN)	\
-	MEMBER(dahdi_pvt, firstradio, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, hanguponpolarityswitch, AST_DATA_BOOLEAN)	\
-	MEMBER(dahdi_pvt, hardwaredtmf, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, hidecallerid, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, hidecalleridname, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, ignoredtmf, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, immediate, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, inalarm, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mate, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, outgoing, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, permcallwaiting, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, priindication_oob, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, priexclusive, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, pulse, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, pulsedial, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, restartpending, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, restrictcid, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, threewaycalling, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, transfer, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, use_callerid, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, use_callingpres, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, usedistinctiveringdetection, AST_DATA_BOOLEAN)	\
-	MEMBER(dahdi_pvt, dahditrcallerid, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, transfertobusy, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mwimonitor_neon, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mwimonitor_fsk, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mwimonitor_rpas, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mwimonitoractive, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, mwisendactive, AST_DATA_BOOLEAN)			\
-	MEMBER(dahdi_pvt, inservice, AST_DATA_BOOLEAN)				\
-	MEMBER(dahdi_pvt, locallyblocked, AST_DATA_UNSIGNED_INTEGER)		\
-	MEMBER(dahdi_pvt, remotelyblocked, AST_DATA_UNSIGNED_INTEGER)		\
-	MEMBER(dahdi_pvt, manages_span_alarms, AST_DATA_BOOLEAN)		\
-	MEMBER(dahdi_pvt, use_smdi, AST_DATA_BOOLEAN)				\
-	MEMBER(dahdi_pvt, context, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, defcontext, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, description, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, exten, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, language, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, mohinterpret, AST_DATA_STRING)			\
-	MEMBER(dahdi_pvt, mohsuggest, AST_DATA_STRING)				\
-	MEMBER(dahdi_pvt, parkinglot, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(dahdi_pvt, DATA_EXPORT_DAHDI_PVT);
-
 static struct dahdi_pvt *iflist = NULL;	/*!< Main interface list start */
 static struct dahdi_pvt *ifend = NULL;	/*!< Main interface list end */
 
@@ -966,7 +881,6 @@ static struct dahdi_chan_conf dahdi_chan
 			.privateprefix = "",
 			.unknownprefix = "",
 			.colp_send = SIG_PRI_COLP_UPDATE,
-			.force_restart_unavailable_chans = 1,
 			.resetinterval = -1,
 		},
 #endif
@@ -1054,6 +968,9 @@ static struct dahdi_chan_conf dahdi_chan
 			.buf_no = numbufs,
 			.usefaxbuffers = 0,
 			.cc_params = ast_cc_config_params_init(),
+			.firstdigit_timeout = ANALOG_FIRST_DIGIT_TIMEOUT,
+			.interdigit_timeout = ANALOG_INTER_DIGIT_TIMEOUT,
+			.matchdigit_timeout = ANALOG_MATCH_DIGIT_TIMEOUT,
 		},
 		.timing = {
 			.prewinktime = -1,
@@ -2282,13 +2199,13 @@ static void my_swap_subchannels(void *pv
  *
  * \param callid_created value returned from ast_callid_threadstorage_auto()
  */
-static struct ast_channel *dahdi_new_callid_clean(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, struct ast_callid *callid, int callid_created);
+static struct ast_channel *dahdi_new_callid_clean(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, ast_callid callid, int callid_created);
 
-static struct ast_channel *dahdi_new(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, struct ast_callid *callid);
+static struct ast_channel *dahdi_new(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, ast_callid callid);
 
 static struct ast_channel *my_new_analog_ast_channel(void *pvt, int state, int startpbx, enum analog_sub sub, const struct ast_channel *requestor)
 {
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 	struct dahdi_pvt *p = pvt;
 	int dsub = analogsub_to_dahdisub(sub);
@@ -2315,7 +2232,7 @@ static struct ast_channel *my_new_pri_as
 	struct dahdi_pvt *p = pvt;
 	int audio;
 	int newlaw = -1;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 
 	switch (p->sig) {
@@ -3242,7 +3159,7 @@ static struct ast_channel *my_new_ss7_as
 	struct dahdi_pvt *p = pvt;
 	int audio;
 	int newlaw;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 
 	/* Set to audio mode at this point */
@@ -3403,6 +3320,19 @@ static int my_have_progressdetect(void *
 	}
 }
 
+#define gen_pvt_field_callback(type, field) \
+	static type my_get_##field(void *pvt) \
+	{ \
+		struct dahdi_pvt *p = pvt; \
+		return p->field; \
+	}
+
+gen_pvt_field_callback(int, firstdigit_timeout);
+gen_pvt_field_callback(int, interdigit_timeout);
+gen_pvt_field_callback(int, matchdigit_timeout);
+
+#undef gen_pvt_field_callback
+
 struct analog_callback analog_callbacks =
 {
 	.play_tone = my_play_tone,
@@ -3471,6 +3401,9 @@ struct analog_callback analog_callbacks
 	.answer_polarityswitch = my_answer_polarityswitch,
 	.hangup_polarityswitch = my_hangup_polarityswitch,
 	.have_progressdetect = my_have_progressdetect,
+	.get_firstdigit_timeout = my_get_firstdigit_timeout,
+	.get_matchdigit_timeout = my_get_matchdigit_timeout,
+	.get_interdigit_timeout = my_get_interdigit_timeout,
 };
 
 /*! Round robin search locations. */
@@ -3726,7 +3659,7 @@ static void dahdi_r2_on_call_offered(ope
 {
 	struct dahdi_pvt *p;
 	struct ast_channel *c;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 	ast_verbose("MFC/R2 call offered on chan %d. ANI = %s, DNIS = %s, Category = %s\n",
 			openr2_chan_get_number(r2chan), ani ? ani : "(restricted)", dnis,
@@ -3795,7 +3728,7 @@ static void dahdi_r2_on_call_accepted(op
 {
 	struct dahdi_pvt *p = NULL;
 	struct ast_channel *c = NULL;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 	p = openr2_chan_get_client_data(r2chan);
 	dahdi_ec_enable(p);
@@ -7964,7 +7897,7 @@ static struct ast_frame *dahdi_handle_ev
 					p->subs[SUB_REAL].needflash = 1;
 					goto winkflashdone;
 				} else if (!check_for_conference(p)) {
-					struct ast_callid *callid = NULL;
+					ast_callid callid = 0;
 					int callid_created;
 					char cid_num[256];
 					char cid_name[256];
@@ -9100,7 +9033,7 @@ static struct ast_str *create_channel_na
 	return chan_name;
 }
 
-static struct ast_channel *dahdi_new_callid_clean(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, struct ast_callid *callid, int callid_created)
+static struct ast_channel *dahdi_new_callid_clean(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, ast_callid callid, int callid_created)
 {
 	struct ast_channel *new_channel = dahdi_new(i, state, startpbx, idx, law, assignedids, requestor, callid);
 
@@ -9109,7 +9042,7 @@ static struct ast_channel *dahdi_new_cal
 	return new_channel;
 }
 
-static struct ast_channel *dahdi_new(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, struct ast_callid *callid)
+static struct ast_channel *dahdi_new(struct dahdi_pvt *i, int state, int startpbx, int idx, int law, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, ast_callid callid)
 {
 	struct ast_channel *tmp;
 	struct ast_format_cap *caps;
@@ -9632,9 +9565,9 @@ static void *analog_ss_thread(void *data
 				dtmfbuf[len] = '\0';
 				while ((len < AST_MAX_EXTENSION-1) && ast_matchmore_extension(chan, ast_channel_context(chan), dtmfbuf, 1, p->cid_num)) {
 					if (ast_exists_extension(chan, ast_channel_context(chan), dtmfbuf, 1, p->cid_num)) {
-						timeout = matchdigittimeout;
+						timeout = p->matchdigit_timeout;
 					} else {
-						timeout = gendigittimeout;
+						timeout = p->interdigit_timeout;
 					}
 					res = ast_waitfordigit(chan, timeout);
 					if (res < 0) {
@@ -9802,7 +9735,7 @@ static void *analog_ss_thread(void *data
 	case SIG_FXOGS:
 	case SIG_FXOKS:
 		/* Read the first digit */
-		timeout = firstdigittimeout;
+		timeout = p->firstdigit_timeout;
 		/* If starting a threeway call, never timeout on the first digit so someone
 		   can use flash-hook as a "hold" feature */
 		if (p->subs[SUB_THREEWAY].owner)
@@ -9877,8 +9810,8 @@ static void *analog_ss_thread(void *data
 					}
 				} else {
 					/* It's a match, but they just typed a digit, and there is an ambiguous match,
-					   so just set the timeout to matchdigittimeout and wait some more */
-					timeout = matchdigittimeout;
+					   so just set the timeout to matchdigit_timeout and wait some more */
+					timeout = p->matchdigit_timeout;
 				}
 			} else if (res == 0) {
 				ast_debug(1, "not enough digits (and no ambiguous match)...\n");
@@ -9898,7 +9831,7 @@ static void *analog_ss_thread(void *data
 				len = 0;
 				ioctl(p->subs[idx].dfd,DAHDI_CONFDIAG,&len);
 				memset(exten, 0, sizeof(exten));
-				timeout = firstdigittimeout;
+				timeout = p->firstdigit_timeout;
 
 			} else if (!strcmp(exten, pickupexten)) {
 				/* Scan all channels and see if there are any
@@ -9943,7 +9876,7 @@ static void *analog_ss_thread(void *data
 				}
 				len = 0;
 				memset(exten, 0, sizeof(exten));
-				timeout = firstdigittimeout;
+				timeout = p->firstdigit_timeout;
 			} else if (p->callreturn && !strcmp(exten, "*69")) {
 				res = tone_zone_play_tone(p->subs[idx].dfd, DAHDI_TONE_DIALRECALL);
 				break;
@@ -10015,7 +9948,7 @@ static void *analog_ss_thread(void *data
 				}
 				len = 0;
 				memset(exten, 0, sizeof(exten));
-				timeout = firstdigittimeout;
+				timeout = p->firstdigit_timeout;
 			} else if (!strcmp(exten, "*0")) {
 				struct ast_channel *nbridge =
 					p->subs[SUB_THREEWAY].owner;
@@ -10063,7 +9996,7 @@ static void *analog_ss_thread(void *data
 				break;
 			}
 			if (!timeout)
-				timeout = gendigittimeout;
+				timeout = p->interdigit_timeout;
 			if (len && !ast_ignore_pattern(ast_channel_context(chan), exten))
 				tone_zone_play_tone(p->subs[idx].dfd, -1);
 		}
@@ -10711,7 +10644,7 @@ static void *mwi_thread(void *data)
 
 		if (i & DAHDI_IOMUX_SIGEVENT) {
 			struct ast_channel *chan;
-			struct ast_callid *callid = NULL;
+			ast_callid callid = 0;
 			int callid_created;
 
 			/* If we get an event, screen out events that we do not act on.
@@ -11145,7 +11078,7 @@ static struct dahdi_pvt *handle_init_eve
 	int res;
 	pthread_t threadid;
 	struct ast_channel *chan;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created;
 
 	/* Handle an event on a given channel for the monitor thread. */
@@ -11631,7 +11564,7 @@ static void *do_monitor(void *data)
 										doomed = analog_handle_init_event(i->sig_pvt, ANALOG_EVENT_DTMFCID);
 										i->dtmfcid_holdoff_state = 1;
 									} else {
-										struct ast_callid *callid = NULL;
+										ast_callid callid = 0;
 										int callid_created = ast_callid_threadstorage_auto(&callid);
 										chan = dahdi_new(i, AST_STATE_PRERING, 0, SUB_REAL, 0, NULL, NULL, callid);
 										if (!chan) {
@@ -11987,38 +11920,6 @@ static int sigtype_to_signalling(int sig
 
 /*!
  * \internal
- * \brief Get file name and channel number from (subdir,number)
- *
- * \param subdir name of the subdirectory under /dev/dahdi/
- * \param channel name of device file under /dev/dahdi/<subdir>/
- * \param path buffer to put file name in
- * \param pathlen maximal length of path
- *
- * \retval minor number of dahdi channel.
- * \retval -errno on error.
- */
-static int device2chan(const char *subdir, int channel, char *path, int pathlen)
-{
-	struct stat	stbuf;
-	int		num;
-
-	snprintf(path, pathlen, "/dev/dahdi/%s/%d", subdir, channel);
-	if (stat(path, &stbuf) < 0) {
-		ast_log(LOG_ERROR, "stat(%s) failed: %s\n", path, strerror(errno));
-		return -errno;
-	}
-	if (!S_ISCHR(stbuf.st_mode)) {
-		ast_log(LOG_ERROR, "%s: Not a character device file\n", path);
-		return -EINVAL;
-	}
-	num = minor(stbuf.st_rdev);
-	ast_debug(1, "%s -> %d\n", path, num);
-	return num;
-
-}
-
-/*!
- * \internal
  * \brief Initialize/create a channel interface.
  *
  * \param channel Channel interface number to initialize/create.
@@ -12614,6 +12515,9 @@ static struct dahdi_pvt *mkintf(int chan
 		tmp->waitfordialtone = conf->chan.waitfordialtone;
 		tmp->dialtone_detect = conf->chan.dialtone_detect;
 		tmp->faxdetect_timeout = conf->chan.faxdetect_timeout;
+		tmp->firstdigit_timeout = conf->chan.firstdigit_timeout;
+		tmp->interdigit_timeout = conf->chan.interdigit_timeout;
+		tmp->matchdigit_timeout = conf->chan.matchdigit_timeout;
 		tmp->cancallforward = conf->chan.cancallforward;
 		tmp->dtmfrelax = conf->chan.dtmfrelax;
 		tmp->callwaiting = tmp->permcallwaiting;
@@ -12690,6 +12594,8 @@ static struct dahdi_pvt *mkintf(int chan
 				 * knows that we care about it.  Then, chan_dahdi will get the MWI from the
 				 * event cache instead of checking the mailbox directly. */
 				tmp->mwi_event_sub = stasis_subscribe_pool(mailbox_specific_topic, stasis_subscription_cb_noop, NULL);
+				stasis_subscription_accept_message_type(tmp->mwi_event_sub, ast_mwi_state_type());
+				stasis_subscription_set_filter(tmp->mwi_event_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 			}
 		}
 #ifdef HAVE_DAHDI_LINEREVERSE_VMWI
@@ -13439,7 +13345,7 @@ static struct ast_channel *dahdi_request
 	int transcapdigital = 0;
 #endif	/* defined(HAVE_PRI) || defined(HAVE_SS7) */
 	struct dahdi_starting_point start;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 
 	ast_mutex_lock(&iflock);
@@ -13761,12 +13667,12 @@ static void dahdi_ss7_message(struct ss7
 	if (ss7) {
 		for (i = 0; i < NUM_SPANS; i++) {
 			if (linksets[i].ss7.ss7 == ss7) {
-				ast_verbose_callid(NULL, "[%d] %s", i + 1, s);
+				ast_verbose_callid(0, "[%d] %s", i + 1, s);
 				return;
 			}
 		}
 	}
-	ast_verbose_callid(NULL, "%s", s);
+	ast_verbose_callid(0, "%s", s);
 }
 #endif	/* defined(HAVE_SS7) */
 
@@ -13778,12 +13684,12 @@ static void dahdi_ss7_error(struct ss7 *
 	if (ss7) {
 		for (i = 0; i < NUM_SPANS; i++) {
 			if (linksets[i].ss7.ss7 == ss7) {
-				ast_log_callid(LOG_ERROR, NULL, "[%d] %s", i + 1, s);
+				ast_log_callid(LOG_ERROR, 0, "[%d] %s", i + 1, s);
 				return;
 			}
 		}
 	}
-	ast_log_callid(LOG_ERROR, NULL, "%s", s);
+	ast_log_callid(LOG_ERROR, 0, "%s", s);
 }
 #endif	/* defined(HAVE_SS7) */
 
@@ -13893,22 +13799,22 @@ static void dahdi_pri_message(struct pri
 		}
 		if (-1 < span) {
 			if (1 < dchancount) {
-				ast_verbose_callid(NULL, "[PRI Span: %d D-Channel: %d] %s", span + 1, dchan, s);
+				ast_verbose_callid(0, "[PRI Span: %d D-Channel: %d] %s", span + 1, dchan, s);
 			} else {
-				ast_verbose_callid(NULL, "PRI Span: %d %s", span + 1, s);
+				ast_verbose_callid(0, "PRI Span: %d %s", span + 1, s);
 			}
 		} else {
-			ast_verbose_callid(NULL, "PRI Span: ? %s", s);
+			ast_verbose_callid(0, "PRI Span: ? %s", s);
 		}
 	} else {
-		ast_verbose_callid(NULL, "PRI Span: ? %s", s);
+		ast_verbose_callid(0, "PRI Span: ? %s", s);
 	}
 
 	ast_mutex_lock(&pridebugfdlock);
 
 	if (pridebugfd >= 0) {
 		if (write(pridebugfd, s, strlen(s)) < 0) {
-			ast_log_callid(LOG_WARNING, NULL, "write() failed: %s\n", strerror(errno));
+			ast_log_callid(LOG_WARNING, 0, "write() failed: %s\n", strerror(errno));
 		}
 	}
 
@@ -13944,22 +13850,22 @@ static void dahdi_pri_error(struct pri *
 		}
 		if (-1 < span) {
 			if (1 < dchancount) {
-				ast_log_callid(LOG_ERROR, NULL, "[PRI Span: %d D-Channel: %d] %s", span + 1, dchan, s);
+				ast_log_callid(LOG_ERROR, 0, "[PRI Span: %d D-Channel: %d] %s", span + 1, dchan, s);
 			} else {
-				ast_log_callid(LOG_ERROR, NULL, "PRI Span: %d %s", span + 1, s);
+				ast_log_callid(LOG_ERROR, 0, "PRI Span: %d %s", span + 1, s);
 			}
 		} else {
-			ast_log_callid(LOG_ERROR, NULL, "PRI Span: ? %s", s);
+			ast_log_callid(LOG_ERROR, 0, "PRI Span: ? %s", s);
 		}
 	} else {
-		ast_log_callid(LOG_ERROR, NULL, "PRI Span: ? %s", s);
+		ast_log_callid(LOG_ERROR, 0, "PRI Span: ? %s", s);
 	}
 
 	ast_mutex_lock(&pridebugfdlock);
 
 	if (pridebugfd >= 0) {
 		if (write(pridebugfd, s, strlen(s)) < 0) {
-			ast_log_callid(LOG_WARNING, NULL, "write() failed: %s\n", strerror(errno));
+			ast_log_callid(LOG_WARNING, 0, "write() failed: %s\n", strerror(errno));
 		}
 	}
 
@@ -17355,7 +17261,6 @@ static int __unload_module(void)
 	ast_manager_unregister("PRIDebugFileSet");
 	ast_manager_unregister("PRIDebugFileUnset");
 #endif	/* defined(HAVE_PRI) */
-	ast_data_unregister(NULL);
 	ast_channel_unregister(&dahdi_tech);
 
 	/* Hangup all interfaces if they have an owner */
@@ -17434,33 +17339,9 @@ static int unload_module(void)
 	return __unload_module();
 }
 
-static void string_replace(char *str, int char1, int char2)
-{
-	for (; *str; str++) {
-		if (*str == char1) {
-			*str = char2;
-		}
-	}
-}
-
-static char *parse_spanchan(char *chanstr, char **subdir)
-{
-	char *p;
-
-	if ((p = strrchr(chanstr, '!')) == NULL) {
-		*subdir = NULL;
-		return chanstr;
-	}
-	*p++ = '\0';
-	string_replace(chanstr, '!', '/');
-	*subdir = chanstr;
-	return p;
-}
-
 static int build_channels(struct dahdi_chan_conf *conf, const char *value, int reload, int lineno)
 {
 	char *c, *chan;
-	char *subdir;
 	int x, start, finish;
 	struct dahdi_pvt *tmp;
 
@@ -17470,7 +17351,6 @@ static int build_channels(struct dahdi_c
 	}
 
 	c = ast_strdupa(value);
-	c = parse_spanchan(c, &subdir);
 
 	while ((chan = strsep(&c, ","))) {
 		if (sscanf(chan, "%30d-%30d", &start, &finish) == 2) {
@@ -17492,39 +17372,22 @@ static int build_channels(struct dahdi_c
 		}
 
 		for (x = start; x <= finish; x++) {
-			char fn[PATH_MAX];
-			int real_channel = x;
-
-			if (!ast_strlen_zero(subdir)) {
-				real_channel = device2chan(subdir, x, fn, sizeof(fn));
-				if (real_channel < 0) {
-					if (conf->ignore_failed_channels) {
-						ast_log(LOG_WARNING, "Failed configuring %s!%d, (got %d). But moving on to others.\n",
-								subdir, x, real_channel);
-						continue;
-					} else {
-						ast_log(LOG_ERROR, "Failed configuring %s!%d, (got %d).\n",
-								subdir, x, real_channel);
-						return -1;
-					}
-				}
-			}
 			if (conf->wanted_channels_start &&
-				(real_channel < conf->wanted_channels_start ||
-				 real_channel > conf->wanted_channels_end)
+				(x < conf->wanted_channels_start ||
+				 x > conf->wanted_channels_end)
 			   ) {
 				continue;
 			}
-			tmp = mkintf(real_channel, conf, reload);
+			tmp = mkintf(x, conf, reload);
 
 			if (tmp) {
-				ast_verb(3, "%s channel %d, %s signalling\n", reload ? "Reconfigured" : "Registered", real_channel, sig2str(tmp->sig));
+				ast_verb(3, "%s channel %d, %s signalling\n", reload ? "Reconfigured" : "Registered", x, sig2str(tmp->sig));
 			} else {
 				ast_log(LOG_ERROR, "Unable to %s channel '%s'\n",
 						(reload == 1) ? "reconfigure" : "register", value);
 				return -1;
 			}
-			if (real_channel == CHAN_PSEUDO) {
+			if (x == CHAN_PSEUDO) {
 				has_pseudo = 1;
 			}
 		}
@@ -17869,6 +17732,21 @@ static int process_dahdi(struct dahdi_ch
 			if (sscanf(v->value, "%30u", &confp->chan.faxdetect_timeout) != 1) {
 				confp->chan.faxdetect_timeout = 0;
 			}
+		} else if (!strcasecmp(v->name, "firstdigit_timeout")) {
+			if (sscanf(v->value, "%30d", &confp->chan.firstdigit_timeout) != 1
+				|| confp->chan.firstdigit_timeout <= 0) {
+				confp->chan.firstdigit_timeout = ANALOG_FIRST_DIGIT_TIMEOUT;
+			}
+		} else if (!strcasecmp(v->name, "interdigit_timeout")) {
+			if (sscanf(v->value, "%30d", &confp->chan.interdigit_timeout) != 1
+				|| confp->chan.interdigit_timeout <= 0) {
+				confp->chan.interdigit_timeout = ANALOG_INTER_DIGIT_TIMEOUT;
+			}
+		} else if (!strcasecmp(v->name, "matchdigit_timeout")) {
+			if (sscanf(v->value, "%30d", &confp->chan.matchdigit_timeout) != 1
+				|| confp->chan.matchdigit_timeout <= 0) {
+				confp->chan.matchdigit_timeout = ANALOG_MATCH_DIGIT_TIMEOUT;
+			}
 		} else if (!strcasecmp(v->name, "echocancel")) {
 			process_echocancel(confp, v->value, v->lineno);
 		} else if (!strcasecmp(v->name, "echotraining")) {
@@ -19440,163 +19318,6 @@ static int setup_dahdi(int reload)
 }
 
 /*!
- * \internal
- * \brief Callback used to generate the dahdi status tree.
- * \param[in] search The search pattern tree.
- * \retval NULL on error.
- * \retval non-NULL The generated tree.
- */
-static int dahdi_status_data_provider_get(const struct ast_data_search *search,
-		struct ast_data *data_root)
-{
-	int ctl, res, span;
-	struct ast_data *data_span, *data_alarms;
-	struct dahdi_spaninfo s;
-
-	ctl = open("/dev/dahdi/ctl", O_RDWR);
-	if (ctl < 0) {
-		ast_log(LOG_ERROR, "No DAHDI found. Unable to open /dev/dahdi/ctl: %s\n", strerror(errno));
-		return -1;
-	}
-	for (span = 1; span < DAHDI_MAX_SPANS; ++span) {
-		s.spanno = span;
-		res = ioctl(ctl, DAHDI_SPANSTAT, &s);
-		if (res) {
-			continue;
-		}
-
-		data_span = ast_data_add_node(data_root, "span");
-		if (!data_span) {
-			continue;
-		}
-		ast_data_add_str(data_span, "description", s.desc);
-
-		/* insert the alarms status */
-		data_alarms = ast_data_add_node(data_span, "alarms");
-		if (!data_alarms) {
-			continue;
-		}
-
-		ast_data_add_bool(data_alarms, "BLUE", s.alarms & DAHDI_ALARM_BLUE);
-		ast_data_add_bool(data_alarms, "YELLOW", s.alarms & DAHDI_ALARM_YELLOW);
-		ast_data_add_bool(data_alarms, "RED", s.alarms & DAHDI_ALARM_RED);
-		ast_data_add_bool(data_alarms, "LOOPBACK", s.alarms & DAHDI_ALARM_LOOPBACK);
-		ast_data_add_bool(data_alarms, "RECOVER", s.alarms & DAHDI_ALARM_RECOVER);
-		ast_data_add_bool(data_alarms, "NOTOPEN", s.alarms & DAHDI_ALARM_NOTOPEN);
-
-		ast_data_add_int(data_span, "irqmisses", s.irqmisses);
-		ast_data_add_int(data_span, "bpviol", s.bpvcount);
-		ast_data_add_int(data_span, "crc4", s.crc4count);
-		ast_data_add_str(data_span, "framing",	s.lineconfig & DAHDI_CONFIG_D4 ? "D4" :
-							s.lineconfig & DAHDI_CONFIG_ESF ? "ESF" :
-							s.lineconfig & DAHDI_CONFIG_CCS ? "CCS" :
-							"CAS");
-		ast_data_add_str(data_span, "coding",	s.lineconfig & DAHDI_CONFIG_B8ZS ? "B8ZS" :
-							s.lineconfig & DAHDI_CONFIG_HDB3 ? "HDB3" :
-							s.lineconfig & DAHDI_CONFIG_AMI ? "AMI" :
-							"Unknown");
-		ast_data_add_str(data_span, "options",	s.lineconfig & DAHDI_CONFIG_CRC4 ?
-							s.lineconfig & DAHDI_CONFIG_NOTOPEN ? "CRC4/YEL" : "CRC4" :
-							s.lineconfig & DAHDI_CONFIG_NOTOPEN ? "YEL" : "");
-		ast_data_add_str(data_span, "lbo", lbostr[s.lbo]);
-
-		/* if this span doesn't match remove it. */
-		if (!ast_data_search_match(search, data_span)) {
-			ast_data_remove_node(data_root, data_span);
-		}
-	}
-	close(ctl);
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Callback used to generate the dahdi channels tree.
- * \param[in] search The search pattern tree.
- * \retval NULL on error.
- * \retval non-NULL The generated tree.
- */
-static int dahdi_channels_data_provider_get(const struct ast_data_search *search,
-		struct ast_data *data_root)
-{
-	struct dahdi_pvt *tmp;
-	struct ast_data *data_channel;
-
-	ast_mutex_lock(&iflock);
-	for (tmp = iflist; tmp; tmp = tmp->next) {
-		data_channel = ast_data_add_node(data_root, "channel");
-		if (!data_channel) {
-			continue;
-		}
-
-		ast_data_add_structure(dahdi_pvt, data_channel, tmp);
-
-		/* if this channel doesn't match remove it. */
-		if (!ast_data_search_match(search, data_channel)) {
-			ast_data_remove_node(data_root, data_channel);
-		}
-	}
-	ast_mutex_unlock(&iflock);
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Callback used to generate the dahdi channels tree.
- * \param[in] search The search pattern tree.
- * \retval NULL on error.
- * \retval non-NULL The generated tree.
- */
-static int dahdi_version_data_provider_get(const struct ast_data_search *search,
-		struct ast_data *data_root)
-{
-	int pseudo_fd = -1;
-	struct dahdi_versioninfo vi = {
-		.version = "Unknown",
-		.echo_canceller = "Unknown"
-	};
-
-	if ((pseudo_fd = open("/dev/dahdi/ctl", O_RDONLY)) < 0) {
-		ast_log(LOG_ERROR, "Failed to open control file to get version.\n");
-		return -1;
-	}
-
-	if (ioctl(pseudo_fd, DAHDI_GETVERSION, &vi)) {
-		ast_log(LOG_ERROR, "Failed to get DAHDI version: %s\n", strerror(errno));
-	}
-
-	close(pseudo_fd);
-
-	ast_data_add_str(data_root, "value", vi.version);
-	ast_data_add_str(data_root, "echocanceller", vi.echo_canceller);
-
-	return 0;
-}
-
-static const struct ast_data_handler dahdi_status_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = dahdi_status_data_provider_get
-};
-
-static const struct ast_data_handler dahdi_channels_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = dahdi_channels_data_provider_get
-};
-
-static const struct ast_data_handler dahdi_version_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = dahdi_version_data_provider_get
-};
-
-static const struct ast_data_entry dahdi_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/channel/dahdi/status", &dahdi_status_data_provider),
-	AST_DATA_ENTRY("asterisk/channel/dahdi/channels", &dahdi_channels_data_provider),
-	AST_DATA_ENTRY("asterisk/channel/dahdi/version", &dahdi_version_data_provider)
-};
-
-/*!
  * \brief Load the module
  *
  * Module loading including tests for configuration or dependencies.
@@ -19624,7 +19345,7 @@ static int load_module(void)
 	ast_format_cap_append(dahdi_tech.capabilities, ast_format_ulaw, 0);
 	ast_format_cap_append(dahdi_tech.capabilities, ast_format_alaw, 0);
 
-	if (dahdi_native_load(ast_module_info->self, &dahdi_tech)) {
+	if (dahdi_native_load(&dahdi_tech)) {
 		ao2_ref(dahdi_tech.capabilities, -1);
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -19692,8 +19413,6 @@ static int load_module(void)
 #endif
 
 	ast_cli_register_multiple(dahdi_cli, ARRAY_LEN(dahdi_cli));
-	/* register all the data providers */
-	ast_data_register_multiple(dahdi_data_providers, ARRAY_LEN(dahdi_data_providers));
 	memset(round_robin, 0, sizeof(round_robin));
 	ast_manager_register_xml("DAHDITransfer", 0, action_transfer);
 	ast_manager_register_xml("DAHDIHangup", 0, action_transferhangup);
@@ -19857,5 +19576,6 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	.nonoptreq = "res_smdi",
-	);
+	.requires = "ccss",
+	.optional_modules = "res_smdi",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_dahdi.h 1:16.1.1~dfsg-1/channels/chan_dahdi.h
--- 1:13.23.1~dfsg-2/channels/chan_dahdi.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_dahdi.h	2018-12-26 18:45:13.000000000 +0000
@@ -617,6 +617,21 @@ struct dahdi_pvt {
 	 * \note Set from the "faxdetect_timeout" value read in from chan_dahdi.conf
 	 */
 	unsigned int faxdetect_timeout;
+	/*!
+	 * \brief Time (ms) to detect first digit (in an analog phone)
+	 * \note Set from the "firstdigit_timeout" value read in from chan_dahdi.conf
+	 */
+	int firstdigit_timeout;
+	/*!
+	 * \brief Time (ms) to detect following digits (in an analog phone)
+	 * \note Set from the "interdigit_timeout" value read in from chan_dahdi.conf
+	 */
+	int interdigit_timeout;
+	/*!
+	 * \brief Time (ms) to wait, in case of ambiguous match (in an analog phone)
+	 * \note Set from the "matchdigit_timeout" value read in from chan_dahdi.conf
+	 */
+	int matchdigit_timeout;
 	struct timeval waitingfordt;			/*!< Time we started waiting for dialtone */
 	struct timeval flashtime;			/*!< Last flash-hook time */
 	/*! \brief Opaque DSP configuration structure. */
diff -pruN 1:13.23.1~dfsg-2/channels/chan_iax2.c 1:16.1.1~dfsg-1/channels/chan_iax2.c
--- 1:13.23.1~dfsg-2/channels/chan_iax2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_iax2.c	2018-12-26 18:45:13.000000000 +0000
@@ -59,8 +59,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/mman.h>
 #include <dirent.h>
 #include <sys/socket.h>
@@ -69,7 +67,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <sys/time.h>
-#include <sys/signal.h>
 #include <signal.h>
 #include <strings.h>
 #include <netdb.h>
@@ -103,14 +100,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/localtime.h"
 #include "asterisk/dnsmgr.h"
 #include "asterisk/devicestate.h"
-#include "asterisk/netsock.h"
 #include "asterisk/stringfields.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/timing.h"
 #include "asterisk/taskprocessor.h"
 #include "asterisk/test.h"
-#include "asterisk/data.h"
 #include "asterisk/security_events.h"
 #include "asterisk/stasis_endpoints.h"
 #include "asterisk/bridge.h"
@@ -127,6 +122,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "iax2/include/provision.h"
 #include "iax2/include/codec_pref.h"
 #include "iax2/include/format_compatibility.h"
+#include "iax2/include/netsock.h"
 
 #include "jitterbuf.h"
 
@@ -453,37 +449,37 @@ struct iax2_context {
 };
 
 
-#define	IAX_HASCALLERID         (uint64_t)(1 << 0)    /*!< CallerID has been specified */
-#define IAX_DELME               (uint64_t)(1 << 1)    /*!< Needs to be deleted */
-#define IAX_TEMPONLY            (uint64_t)(1 << 2)    /*!< Temporary (realtime) */
-#define IAX_TRUNK               (uint64_t)(1 << 3)    /*!< Treat as a trunk */
-#define IAX_NOTRANSFER          (uint64_t)(1 << 4)    /*!< Don't native bridge */
-#define IAX_USEJITTERBUF        (uint64_t)(1 << 5)    /*!< Use jitter buffer */
-#define IAX_DYNAMIC             (uint64_t)(1 << 6)    /*!< dynamic peer */
-#define IAX_SENDANI             (uint64_t)(1 << 7)    /*!< Send ANI along with CallerID */
-#define IAX_RTSAVE_SYSNAME      (uint64_t)(1 << 8)    /*!< Save Systname on Realtime Updates */
-#define IAX_ALREADYGONE         (uint64_t)(1 << 9)    /*!< Already disconnected */
-#define IAX_PROVISION           (uint64_t)(1 << 10)   /*!< This is a provisioning request */
-#define IAX_QUELCH              (uint64_t)(1 << 11)   /*!< Whether or not we quelch audio */
-#define IAX_ENCRYPTED           (uint64_t)(1 << 12)   /*!< Whether we should assume encrypted tx/rx */
-#define IAX_KEYPOPULATED        (uint64_t)(1 << 13)   /*!< Whether we have a key populated */
-#define IAX_CODEC_USER_FIRST    (uint64_t)(1 << 14)   /*!< are we willing to let the other guy choose the codec? */
-#define IAX_CODEC_NOPREFS       (uint64_t)(1 << 15)   /*!< Force old behaviour by turning off prefs */
-#define IAX_CODEC_NOCAP         (uint64_t)(1 << 16)   /*!< only consider requested format and ignore capabilities*/
-#define IAX_RTCACHEFRIENDS      (uint64_t)(1 << 17)   /*!< let realtime stay till your reload */
-#define IAX_RTUPDATE            (uint64_t)(1 << 18)   /*!< Send a realtime update */
-#define IAX_RTAUTOCLEAR         (uint64_t)(1 << 19)   /*!< erase me on expire */
-#define IAX_RTIGNOREREGEXPIRE   (uint64_t)(1 << 21)   /*!< When using realtime, ignore registration expiration */
-#define IAX_TRUNKTIMESTAMPS     (uint64_t)(1 << 22)   /*!< Send trunk timestamps */
-#define IAX_TRANSFERMEDIA       (uint64_t)(1 << 23)   /*!< When doing IAX2 transfers, transfer media only */
-#define IAX_MAXAUTHREQ          (uint64_t)(1 << 24)   /*!< Maximum outstanding AUTHREQ restriction is in place */
-#define IAX_DELAYPBXSTART       (uint64_t)(1 << 25)   /*!< Don't start a PBX on the channel until the peer sends us a response, so that we've achieved a three-way handshake with them before sending voice or anything else */
-#define IAX_ALLOWFWDOWNLOAD     (uint64_t)(1 << 26)   /*!< Allow the FWDOWNL command? */
-#define IAX_IMMEDIATE           (uint64_t)(1 << 27)   /*!< Allow immediate off-hook to extension s */
-#define IAX_SENDCONNECTEDLINE   (uint64_t)(1 << 28)   /*!< Allow sending of connected line updates */
-#define IAX_RECVCONNECTEDLINE   (uint64_t)(1 << 29)   /*!< Allow receiving of connected line updates */
-#define IAX_FORCE_ENCRYPT       (uint64_t)(1 << 30)   /*!< Forces call encryption, if encryption not possible hangup */
-#define IAX_SHRINKCALLERID      (uint64_t)(1 << 31)   /*!< Turn on and off caller id shrinking */
+#define IAX_HASCALLERID         (uint64_t)(1LLU << 0)    /*!< CallerID has been specified */
+#define IAX_DELME               (uint64_t)(1LLU << 1)    /*!< Needs to be deleted */
+#define IAX_TEMPONLY            (uint64_t)(1LLU << 2)    /*!< Temporary (realtime) */
+#define IAX_TRUNK               (uint64_t)(1LLU << 3)    /*!< Treat as a trunk */
+#define IAX_NOTRANSFER          (uint64_t)(1LLU << 4)    /*!< Don't native bridge */
+#define IAX_USEJITTERBUF        (uint64_t)(1LLU << 5)    /*!< Use jitter buffer */
+#define IAX_DYNAMIC             (uint64_t)(1LLU << 6)    /*!< dynamic peer */
+#define IAX_SENDANI             (uint64_t)(1LLU << 7)    /*!< Send ANI along with CallerID */
+#define IAX_RTSAVE_SYSNAME      (uint64_t)(1LLU << 8)    /*!< Save Systname on Realtime Updates */
+#define IAX_ALREADYGONE         (uint64_t)(1LLU << 9)    /*!< Already disconnected */
+#define IAX_PROVISION           (uint64_t)(1LLU << 10)   /*!< This is a provisioning request */
+#define IAX_QUELCH              (uint64_t)(1LLU << 11)   /*!< Whether or not we quelch audio */
+#define IAX_ENCRYPTED           (uint64_t)(1LLU << 12)   /*!< Whether we should assume encrypted tx/rx */
+#define IAX_KEYPOPULATED        (uint64_t)(1LLU << 13)   /*!< Whether we have a key populated */
+#define IAX_CODEC_USER_FIRST    (uint64_t)(1LLU << 14)   /*!< are we willing to let the other guy choose the codec? */
+#define IAX_CODEC_NOPREFS       (uint64_t)(1LLU << 15)   /*!< Force old behaviour by turning off prefs */
+#define IAX_CODEC_NOCAP         (uint64_t)(1LLU << 16)   /*!< only consider requested format and ignore capabilities*/
+#define IAX_RTCACHEFRIENDS      (uint64_t)(1LLU << 17)   /*!< let realtime stay till your reload */
+#define IAX_RTUPDATE            (uint64_t)(1LLU << 18)   /*!< Send a realtime update */
+#define IAX_RTAUTOCLEAR         (uint64_t)(1LLU << 19)   /*!< erase me on expire */
+#define IAX_RTIGNOREREGEXPIRE   (uint64_t)(1LLU << 21)   /*!< When using realtime, ignore registration expiration */
+#define IAX_TRUNKTIMESTAMPS     (uint64_t)(1LLU << 22)   /*!< Send trunk timestamps */
+#define IAX_TRANSFERMEDIA       (uint64_t)(1LLU << 23)   /*!< When doing IAX2 transfers, transfer media only */
+#define IAX_MAXAUTHREQ          (uint64_t)(1LLU << 24)   /*!< Maximum outstanding AUTHREQ restriction is in place */
+#define IAX_DELAYPBXSTART       (uint64_t)(1LLU << 25)   /*!< Don't start a PBX on the channel until the peer sends us a response, so that we've achieved a three-way handshake with them before sending voice or anything else */
+#define IAX_ALLOWFWDOWNLOAD     (uint64_t)(1LLU << 26)   /*!< Allow the FWDOWNL command? */
+#define IAX_IMMEDIATE           (uint64_t)(1LLU << 27)   /*!< Allow immediate off-hook to extension s */
+#define IAX_SENDCONNECTEDLINE   (uint64_t)(1LLU << 28)   /*!< Allow sending of connected line updates */
+#define IAX_RECVCONNECTEDLINE   (uint64_t)(1LLU << 29)   /*!< Allow receiving of connected line updates */
+#define IAX_FORCE_ENCRYPT       (uint64_t)(1LLU << 30)   /*!< Forces call encryption, if encryption not possible hangup */
+#define IAX_SHRINKCALLERID      (uint64_t)(1LLU << 31)   /*!< Turn on and off caller id shrinking */
 static int global_rtautoclear = 120;
 
 static int reload_config(int forced_reload);
@@ -701,7 +697,7 @@ struct chan_iax2_pvt {
 	/*! Socket to send/receive on for this call */
 	int sockfd;
 	/*! ast_callid bound to dialog */
-	struct ast_callid *callid;
+	ast_callid callid;
 	/*! Last received voice format */
 	iax2_format voiceformat;
 	/*! Last received video format */
@@ -938,7 +934,7 @@ static struct ast_taskprocessor *transmi
 
 static int randomcalltokendata;
 
-static const time_t MAX_CALLTOKEN_DELAY = 10;
+static time_t max_calltoken_delay = 10;
 
 /*!
  * This module will get much higher performance when doing a lot of
@@ -1127,30 +1123,22 @@ static void signal_condition(ast_mutex_t
  */
 static struct chan_iax2_pvt *iaxs[IAX_MAX_CALLS];
 
-static struct ast_callid *iax_pvt_callid_get(int callno)
+static ast_callid iax_pvt_callid_get(int callno)
 {
-	if (iaxs[callno]->callid) {
-		return ast_callid_ref(iaxs[callno]->callid);
-	}
-	return NULL;
+	return iaxs[callno]->callid;
 }
 
-static void iax_pvt_callid_set(int callno, struct ast_callid *callid)
+static void iax_pvt_callid_set(int callno, ast_callid callid)
 {
-	if (iaxs[callno]->callid) {
-		ast_callid_unref(iaxs[callno]->callid);
-	}
-	ast_callid_ref(callid);
 	iaxs[callno]->callid = callid;
 }
 
 static void iax_pvt_callid_new(int callno)
 {
-	struct ast_callid *callid = ast_create_callid();
+	ast_callid callid = ast_create_callid();
 	char buffer[AST_CALLID_BUFFER_LENGTH];
 	ast_callid_strnprint(buffer, sizeof(buffer), callid);
 	iax_pvt_callid_set(callno, callid);
-	ast_callid_unref(callid);
 }
 
 /*!
@@ -1442,6 +1430,11 @@ static int iax2_is_control_frame_allowed
 		/* Intended only for the sending machine's local channel structure. */
 	case AST_CONTROL_MASQUERADE_NOTIFY:
 		/* Intended only for masquerades when calling ast_indicate_data(). */
+	case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+		/* Intended only for internal stream topology manipulation. */
+	case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+		/* Intended only for internal stream topology change notification. */
+	case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
 	case AST_CONTROL_STREAM_STOP:
 	case AST_CONTROL_STREAM_SUSPEND:
 	case AST_CONTROL_STREAM_RESTART:
@@ -1463,6 +1456,8 @@ static void network_change_stasis_subscr
 	if (!network_change_sub) {
 		network_change_sub = stasis_subscribe(ast_system_topic(),
 			network_change_stasis_cb, NULL);
+		stasis_subscription_accept_message_type(network_change_sub, ast_network_change_type());
+		stasis_subscription_set_filter(network_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 }
 
@@ -1476,6 +1471,8 @@ static void acl_change_stasis_subscribe(
 	if (!acl_change_sub) {
 		acl_change_sub = stasis_subscribe(ast_security_topic(),
 			acl_change_stasis_cb, NULL);
+		stasis_subscription_accept_message_type(acl_change_sub, ast_named_acl_change_type());
+		stasis_subscription_set_filter(acl_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 }
 
@@ -1958,19 +1955,6 @@ static int iax2_parse_allow_disallow(str
 	return res;
 }
 
-static int iax2_data_add_codecs(struct ast_data *root, const char *node_name, iax2_format formats)
-{
-	int res;
-	struct ast_format_cap *cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-	if (!cap) {
-		return -1;
-	}
-	iax2_format_compatibility_bitfield2cap(formats, cap);
-	res = ast_data_add_codecs(root, node_name, cap);
-	ao2_ref(cap, -1);
-	return res;
-}
-
 /*!
  * \note The only member of the peer passed here guaranteed to be set is the name field
  */
@@ -2253,11 +2237,6 @@ static void pvt_destructor(void *obj)
 		jb_destroy(pvt->jb);
 		ast_string_field_free_memory(pvt);
 	}
-
-	if (pvt->callid) {
-		ast_callid_unref(pvt->callid);
-	}
-
 }
 
 static struct chan_iax2_pvt *new_iax(struct ast_sockaddr *addr, const char *host)
@@ -5002,7 +4981,7 @@ static int handle_call_token(struct ast_
 		if (strcmp(hash, rec_hash)) {
 			ast_log(LOG_WARNING, "Address %s failed CallToken hash inspection\n", ast_sockaddr_stringify(addr));
 			goto reject; /* received hash does not match ours, reject */
-		} else if ((t < rec_time) || ((t - rec_time) >= MAX_CALLTOKEN_DELAY)) {
+		} else if ((t < rec_time) || ((t - rec_time) >= max_calltoken_delay)) {
 			ast_log(LOG_WARNING, "Too much delay in IAX2 calltoken timestamp from address %s\n", ast_sockaddr_stringify(addr));
 			goto reject; /* too much delay, reject */
 		}
@@ -5847,7 +5826,7 @@ static struct ast_channel *ast_iax2_new(
 	struct ast_variable *v = NULL;
 	struct ast_format_cap *native;
 	struct ast_format *tmpfmt;
-	struct ast_callid *callid;
+	ast_callid callid;
 	char *peer_name = NULL;
 
 	if (!(i = iaxs[callno])) {
@@ -8012,7 +7991,7 @@ static int check_access(int callno, stru
 		  * Set authmethods to the last known authmethod used by the system
 		  * Set a fake secret, it's not looked at, just required to attempt authentication.
 		  * Set authrej so the AUTHREP is rejected without even looking at its contents */
-		iaxs[callno]->authmethods = last_authmethod ? last_authmethod : (IAX_AUTH_MD5 | IAX_AUTH_PLAINTEXT);
+		iaxs[callno]->authmethods = last_authmethod ? last_authmethod : IAX_AUTH_MD5;
 		ast_string_field_set(iaxs[callno], secret, "badsecret");
 		iaxs[callno]->authrej = 1;
 		if (!ast_strlen_zero(iaxs[callno]->username)) {
@@ -9207,7 +9186,7 @@ static int registry_authrequest(int call
 	 * peer does not exist, and vice-versa.
 	 * Therefore, we use whatever the last peer used (which may vary over the
 	 * course of a server, which should leak minimal information). */
-	sentauthmethod = p ? p->authmethods : last_authmethod ? last_authmethod : (IAX_AUTH_MD5 | IAX_AUTH_PLAINTEXT);
+	sentauthmethod = p ? p->authmethods : last_authmethod ? last_authmethod : IAX_AUTH_MD5;
 	if (!p) {
 		iaxs[callno]->authmethods = sentauthmethod;
 	}
@@ -10269,12 +10248,11 @@ static int socket_process_helper(struct
 	}
 
 	if (fr->callno > 0) {
-		struct ast_callid *mount_callid;
+		ast_callid mount_callid;
 		ast_mutex_lock(&iaxsl[fr->callno]);
 		if (iaxs[fr->callno] && ((mount_callid = iax_pvt_callid_get(fr->callno)))) {
 			/* Bind to thread */
 			ast_callid_threadassoc_add(mount_callid);
-			ast_callid_unref(mount_callid);
 		}
 	}
 
@@ -11971,11 +11949,9 @@ immediatedial:
 
 static int socket_process(struct iax2_thread *thread)
 {
-	struct ast_callid *callid;
 	int res = socket_process_helper(thread);
-	if ((callid = ast_read_threadstorage_callid())) {
+	if (ast_read_threadstorage_callid()) {
 		ast_callid_threadassoc_remove();
-		callid = ast_callid_unref(callid);
 	}
 	return res;
 }
@@ -12470,7 +12446,7 @@ static struct ast_channel *iax2_request(
 	struct parsed_dial_string pds;
 	struct create_addr_info cai;
 	char *tmpstr;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	memset(&pds, 0, sizeof(pds));
 	tmpstr = ast_strdupa(data);
@@ -12575,9 +12551,6 @@ static struct ast_channel *iax2_request(
 		ao2_ref(format, -1);
 	}
 
-	if (callid) {
-		ast_callid_unref(callid);
-	}
 	return c;
 }
 
@@ -12897,6 +12870,9 @@ static struct iax2_peer *build_peer(cons
 				}
 			} else if (!strcasecmp(v->name, "auth")) {
 				peer->authmethods = get_auth_methods(v->value);
+				if (peer->authmethods & IAX_AUTH_PLAINTEXT) {
+					ast_log(LOG_WARNING, "Auth method for peer '%s' is set to deprecated 'plaintext' at line %d of iax.conf\n", peer->name, v->lineno);
+				}
 			} else if (!strcasecmp(v->name, "encryption")) {
 				peer->encmethods |= get_encrypt_methods(v->value);
 				if (!peer->encmethods) {
@@ -13073,7 +13049,7 @@ static struct iax2_peer *build_peer(cons
 			}
 		}
 		if (!peer->authmethods)
-			peer->authmethods = IAX_AUTH_MD5 | IAX_AUTH_PLAINTEXT;
+			peer->authmethods = IAX_AUTH_MD5;
 		ast_clear_flag64(peer, IAX_DELME);
 	}
 
@@ -13100,6 +13076,8 @@ static struct iax2_peer *build_peer(cons
 			 * mailboxes.  However, we just grab the events out of the cache when it
 			 * is time to send MWI, since it is only sent with a REGACK. */
 			peer->mwi_event_sub = stasis_subscribe_pool(mailbox_specific_topic, stasis_subscription_cb_noop, NULL);
+			stasis_subscription_accept_message_type(peer->mwi_event_sub, ast_mwi_state_type());
+			stasis_subscription_set_filter(peer->mwi_event_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 	}
 
@@ -13222,6 +13200,9 @@ static struct iax2_user *build_user(cons
 				}
 			} else if (!strcasecmp(v->name, "auth")) {
 				user->authmethods = get_auth_methods(v->value);
+				if (user->authmethods & IAX_AUTH_PLAINTEXT) {
+					ast_log(LOG_WARNING, "Auth method for user '%s' is set to deprecated 'plaintext' at line %d of iax.conf\n", user->name, v->lineno);
+				}
 			} else if (!strcasecmp(v->name, "encryption")) {
 				user->encmethods |= get_encrypt_methods(v->value);
 				if (!user->encmethods) {
@@ -13354,13 +13335,13 @@ static struct iax2_user *build_user(cons
 		}
 		if (!user->authmethods) {
 			if (!ast_strlen_zero(user->secret)) {
-				user->authmethods = IAX_AUTH_MD5 | IAX_AUTH_PLAINTEXT;
+				user->authmethods = IAX_AUTH_MD5;
 				if (!ast_strlen_zero(user->inkeys))
 					user->authmethods |= IAX_AUTH_RSA;
 			} else if (!ast_strlen_zero(user->inkeys)) {
 				user->authmethods = IAX_AUTH_RSA;
 			} else {
-				user->authmethods = IAX_AUTH_MD5 | IAX_AUTH_PLAINTEXT;
+				user->authmethods = IAX_AUTH_MD5;
 			}
 		}
 		ast_clear_flag64(user, IAX_DELME);
@@ -13835,8 +13816,17 @@ static int set_config(const char *config
 		} else if (!strcasecmp(v->name, "calltokenoptional")) {
 			if (add_calltoken_ignore(v->value)) {
 				ast_log(LOG_WARNING, "Invalid calltokenoptional address range - '%s' line %d\n", v->value, v->lineno);
+				return -1;
 			}
-		} else if (!strcasecmp(v->name, "subscribe_network_change_event")) {
+		} else if (!strcasecmp(v->name, "calltokenexpiration")) {
+			int temp = -1;
+			sscanf(v->value, "%u", &temp);
+			if( temp <= 0 ){
+				ast_log(LOG_WARNING, "Invalid calltokenexpiration value %s. Should be integer greater than 0.\n", v->value);
+			} else {
+				max_calltoken_delay = temp;
+			}
+		}  else if (!strcasecmp(v->name, "subscribe_network_change_event")) {
 			if (ast_true(v->value)) {
 				subscribe_network_change = 1;
 			} else if (ast_false(v->value)) {
@@ -14565,129 +14555,6 @@ static struct ast_cli_entry cli_iax2[] =
 #endif /* IAXTESTS */
 };
 
-#ifdef TEST_FRAMEWORK
-AST_TEST_DEFINE(test_iax2_peers_get)
-{
-	struct ast_data_query query = {
-		.path = "/asterisk/channel/iax2/peers",
-		.search = "peers/peer/name=test_peer_data_provider"
-	};
-	struct ast_data *node;
-	struct iax2_peer *peer;
-
-	switch (cmd) {
-		case TEST_INIT:
-			info->name = "iax2_peers_get_data_test";
-			info->category = "/main/data/iax2/peers/";
-			info->summary = "IAX2 peers data providers unit test";
-			info->description =
-				"Tests whether the IAX2 peers data provider implementation works as expected.";
-			return AST_TEST_NOT_RUN;
-		case TEST_EXECUTE:
-			break;
-	}
-
-	/* build a test peer */
-	peer = build_peer("test_peer_data_provider", NULL, NULL, 0);
-	if (!peer) {
-		return AST_TEST_FAIL;
-	}
-	peer->expiry= 1010;
-	ao2_link(peers, peer);
-
-	node = ast_data_get(&query);
-	if (!node) {
-		ao2_unlink(peers, peer);
-		peer_unref(peer);
-		return AST_TEST_FAIL;
-	}
-
-	/* check returned data node. */
-	if (strcmp(ast_data_retrieve_string(node, "peer/name"), "test_peer_data_provider")) {
-		ao2_unlink(peers, peer);
-		peer_unref(peer);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	if (ast_data_retrieve_int(node, "peer/expiry") != 1010) {
-		ao2_unlink(peers, peer);
-		peer_unref(peer);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	/* release resources */
-	ast_data_free(node);
-
-	ao2_unlink(peers, peer);
-	peer_unref(peer);
-
-	return AST_TEST_PASS;
-}
-
-AST_TEST_DEFINE(test_iax2_users_get)
-{
-	struct ast_data_query query = {
-		.path = "/asterisk/channel/iax2/users",
-		.search = "users/user/name=test_user_data_provider"
-	};
-	struct ast_data *node;
-	struct iax2_user *user;
-
-	switch (cmd) {
-		case TEST_INIT:
-			info->name = "iax2_users_get_data_test";
-			info->category = "/main/data/iax2/users/";
-			info->summary = "IAX2 users data providers unit test";
-			info->description =
-				"Tests whether the IAX2 users data provider implementation works as expected.";
-			return AST_TEST_NOT_RUN;
-		case TEST_EXECUTE:
-			break;
-	}
-
-	user = build_user("test_user_data_provider", NULL, NULL, 0);
-	if (!user) {
-		ast_test_status_update(test, "Failed to build a test user\n");
-		return AST_TEST_FAIL;
-	}
-	user->amaflags = 1010;
-	ao2_link(users, user);
-
-	node = ast_data_get(&query);
-	if (!node) {
-		ast_test_status_update(test, "The data query to find our test user failed\n");
-		ao2_unlink(users, user);
-		user_unref(user);
-		return AST_TEST_FAIL;
-	}
-
-	if (strcmp(ast_data_retrieve_string(node, "user/name"), "test_user_data_provider")) {
-		ast_test_status_update(test, "Our data results did not return the test user created in the previous step.\n");
-		ao2_unlink(users, user);
-		user_unref(user);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	if (ast_data_retrieve_int(node, "user/amaflags/value") != 1010) {
-		ast_test_status_update(test, "The amaflags field in our test user was '%d' not the expected value '1010'\n", ast_data_retrieve_int(node, "user/amaflags/value"));
-		ao2_unlink(users, user);
-		user_unref(user);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	ast_data_free(node);
-
-	ao2_unlink(users, user);
-	user_unref(user);
-
-	return AST_TEST_PASS;
-}
-#endif
-
 static void cleanup_thread_list(void *head)
 {
 	AST_LIST_HEAD(iax2_thread_list, iax2_thread);
@@ -14753,11 +14620,6 @@ static int __unload_module(void)
 	ast_manager_unregister( "IAXnetstats" );
 	ast_manager_unregister( "IAXregistry" );
 	ast_unregister_application(papp);
-#ifdef TEST_FRAMEWORK
-	AST_TEST_UNREGISTER(test_iax2_peers_get);
-	AST_TEST_UNREGISTER(test_iax2_users_get);
-#endif
-	ast_data_unregister(NULL);
 	ast_cli_unregister_multiple(cli_iax2, ARRAY_LEN(cli_iax2));
 	ast_unregister_switch(&iax2_switch);
 	ast_channel_unregister(&iax2_tech);
@@ -14852,23 +14714,54 @@ static int load_objects(void)
 	peers = users = iax_peercallno_pvts = iax_transfercallno_pvts = NULL;
 	peercnts = callno_limits = calltoken_ignores = NULL;
 
-	if (!(peers = ao2_container_alloc(MAX_PEER_BUCKETS, peer_hash_cb, peer_cmp_cb))) {
+	peers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_PEER_BUCKETS,
+		peer_hash_cb, NULL, peer_cmp_cb);
+	if (!peers) {
 		goto container_fail;
-	} else if (!(users = ao2_container_alloc(MAX_USER_BUCKETS, user_hash_cb, user_cmp_cb))) {
+	}
+
+	users = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_USER_BUCKETS,
+		user_hash_cb, NULL, user_cmp_cb);
+	if (!users) {
 		goto container_fail;
-	} else if (!(iax_peercallno_pvts = ao2_container_alloc(IAX_MAX_CALLS, pvt_hash_cb, pvt_cmp_cb))) {
+	}
+
+	iax_peercallno_pvts = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		IAX_MAX_CALLS, pvt_hash_cb, NULL, pvt_cmp_cb);
+	if (!iax_peercallno_pvts) {
 		goto container_fail;
-	} else if (!(iax_transfercallno_pvts = ao2_container_alloc(IAX_MAX_CALLS, transfercallno_pvt_hash_cb, transfercallno_pvt_cmp_cb))) {
+	}
+
+	iax_transfercallno_pvts = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		IAX_MAX_CALLS, transfercallno_pvt_hash_cb, NULL, transfercallno_pvt_cmp_cb);
+	if (!iax_transfercallno_pvts) {
 		goto container_fail;
-	} else if (!(peercnts = ao2_container_alloc(MAX_PEER_BUCKETS, peercnt_hash_cb, peercnt_cmp_cb))) {
+	}
+
+	peercnts = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_PEER_BUCKETS,
+		peercnt_hash_cb, NULL, peercnt_cmp_cb);
+	if (!peercnts) {
 		goto container_fail;
-	} else if (!(callno_limits = ao2_container_alloc(MAX_PEER_BUCKETS, addr_range_hash_cb, addr_range_cmp_cb))) {
+	}
+
+	callno_limits = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_PEER_BUCKETS, addr_range_hash_cb, NULL, addr_range_cmp_cb);
+	if (!callno_limits) {
 		goto container_fail;
-	} else if (!(calltoken_ignores = ao2_container_alloc(MAX_PEER_BUCKETS, addr_range_hash_cb, addr_range_cmp_cb))) {
+	}
+
+	calltoken_ignores = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_PEER_BUCKETS, addr_range_hash_cb, NULL, addr_range_cmp_cb);
+	if (!calltoken_ignores) {
 		goto container_fail;
-	} else if (create_callno_pools()) {
+	}
+
+	if (create_callno_pools()) {
 		goto container_fail;
-	} else if  (!(transmit_processor = ast_taskprocessor_get("iax2_transmit", TPS_REF_DEFAULT))) {
+	}
+
+	transmit_processor = ast_taskprocessor_get("iax2_transmit", TPS_REF_DEFAULT);
+	if (!transmit_processor) {
 		goto container_fail;
 	}
 
@@ -14899,191 +14792,6 @@ container_fail:
 	return -1;
 }
 
-
-#define DATA_EXPORT_IAX2_PEER(MEMBER)				\
-	MEMBER(iax2_peer, name, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, username, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, secret, AST_DATA_PASSWORD)		\
-	MEMBER(iax2_peer, dbsecret, AST_DATA_PASSWORD)		\
-	MEMBER(iax2_peer, outkey, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, regexten, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, context, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, peercontext, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, mailbox, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, mohinterpret, AST_DATA_STRING)	\
-	MEMBER(iax2_peer, mohsuggest, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, inkeys, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, cid_num, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, cid_name, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, zonetag, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, parkinglot, AST_DATA_STRING)		\
-	MEMBER(iax2_peer, expiry, AST_DATA_SECONDS)		\
-	MEMBER(iax2_peer, callno, AST_DATA_INTEGER)		\
-	MEMBER(iax2_peer, lastms, AST_DATA_MILLISECONDS)	\
-	MEMBER(iax2_peer, maxms, AST_DATA_MILLISECONDS)		\
-	MEMBER(iax2_peer, pokefreqok, AST_DATA_MILLISECONDS)	\
-	MEMBER(iax2_peer, pokefreqnotok, AST_DATA_MILLISECONDS)	\
-	MEMBER(iax2_peer, historicms, AST_DATA_INTEGER)		\
-	MEMBER(iax2_peer, smoothing, AST_DATA_BOOLEAN)		\
-        MEMBER(iax2_peer, maxcallno, AST_DATA_INTEGER)
-
-AST_DATA_STRUCTURE(iax2_peer, DATA_EXPORT_IAX2_PEER);
-
-static int peers_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ast_data *data_peer;
-	struct iax2_peer *peer;
-	struct ao2_iterator i;
-	char status[20];
-	struct ast_str *encmethods = ast_str_alloca(256);
-
-	i = ao2_iterator_init(peers, 0);
-	while ((peer = ao2_iterator_next(&i))) {
-		data_peer = ast_data_add_node(data_root, "peer");
-		if (!data_peer) {
-			peer_unref(peer);
-			continue;
-		}
-
-		ast_data_add_structure(iax2_peer, data_peer, peer);
-
-		iax2_data_add_codecs(data_peer, "codecs", peer->capability);
-
-		peer_status(peer, status, sizeof(status));
-		ast_data_add_str(data_peer, "status", status);
-
-		ast_data_add_str(data_peer, "host", ast_sockaddr_stringify_host(&peer->addr));
-
-		ast_data_add_str(data_peer, "mask", ast_sockaddr_stringify_addr(&peer->mask));
-
-		ast_data_add_int(data_peer, "port", ast_sockaddr_port(&peer->addr));
-
-		ast_data_add_bool(data_peer, "trunk", ast_test_flag64(peer, IAX_TRUNK));
-
-		ast_data_add_bool(data_peer, "dynamic", ast_test_flag64(peer, IAX_DYNAMIC));
-
-		encmethods_to_str(peer->encmethods, &encmethods);
-		ast_data_add_str(data_peer, "encryption", peer->encmethods ? ast_str_buffer(encmethods) : "no");
-
-		peer_unref(peer);
-
-		if (!ast_data_search_match(search, data_peer)) {
-			ast_data_remove_node(data_root, data_peer);
-		}
-	}
-	ao2_iterator_destroy(&i);
-
-	return 0;
-}
-
-#define DATA_EXPORT_IAX2_USER(MEMBER)					\
-        MEMBER(iax2_user, name, AST_DATA_STRING)			\
-        MEMBER(iax2_user, dbsecret, AST_DATA_PASSWORD)			\
-        MEMBER(iax2_user, accountcode, AST_DATA_STRING)			\
-        MEMBER(iax2_user, mohinterpret, AST_DATA_STRING)		\
-        MEMBER(iax2_user, mohsuggest, AST_DATA_STRING)			\
-        MEMBER(iax2_user, inkeys, AST_DATA_STRING)			\
-        MEMBER(iax2_user, language, AST_DATA_STRING)			\
-        MEMBER(iax2_user, cid_num, AST_DATA_STRING)			\
-        MEMBER(iax2_user, cid_name, AST_DATA_STRING)			\
-        MEMBER(iax2_user, parkinglot, AST_DATA_STRING)			\
-        MEMBER(iax2_user, maxauthreq, AST_DATA_INTEGER)			\
-        MEMBER(iax2_user, curauthreq, AST_DATA_INTEGER)
-
-AST_DATA_STRUCTURE(iax2_user, DATA_EXPORT_IAX2_USER);
-
-static int users_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ast_data *data_user, *data_authmethods, *data_enum_node;
-	struct iax2_user *user;
-	struct ao2_iterator i;
-	struct ast_str *auth;
-	char *pstr = "";
-
-	if (!(auth = ast_str_create(90))) {
-		ast_log(LOG_ERROR, "Unable to create temporary string for storing 'secret'\n");
-		return 0;
-	}
-
-	i = ao2_iterator_init(users, 0);
-	for (; (user = ao2_iterator_next(&i)); user_unref(user)) {
-		data_user = ast_data_add_node(data_root, "user");
-		if (!data_user) {
-			continue;
-		}
-
-		ast_data_add_structure(iax2_user, data_user, user);
-
-		iax2_data_add_codecs(data_user, "codecs", user->capability);
-
-		if (!ast_strlen_zero(user->secret)) {
-			ast_str_set(&auth, 0, "%s", user->secret);
-		} else if (!ast_strlen_zero(user->inkeys)) {
-			ast_str_set(&auth, 0, "Key: %s", user->inkeys);
-		} else {
-			ast_str_set(&auth, 0, "no secret");
-		}
-		ast_data_add_password(data_user, "secret", ast_str_buffer(auth));
-
-		ast_data_add_str(data_user, "context", user->contexts ? user->contexts->context : DEFAULT_CONTEXT);
-
-		/* authmethods */
-		data_authmethods = ast_data_add_node(data_user, "authmethods");
-		if (!data_authmethods) {
-			ast_data_remove_node(data_root, data_user);
-			continue;
-		}
-		ast_data_add_bool(data_authmethods, "rsa", user->authmethods & IAX_AUTH_RSA);
-		ast_data_add_bool(data_authmethods, "md5", user->authmethods & IAX_AUTH_MD5);
-		ast_data_add_bool(data_authmethods, "plaintext", user->authmethods & IAX_AUTH_PLAINTEXT);
-
-		/* amaflags */
-		data_enum_node = ast_data_add_node(data_user, "amaflags");
-		if (!data_enum_node) {
-			ast_data_remove_node(data_root, data_user);
-			continue;
-		}
-		ast_data_add_int(data_enum_node, "value", user->amaflags);
-		ast_data_add_str(data_enum_node, "text", ast_channel_amaflags2string(user->amaflags));
-
-		ast_data_add_bool(data_user, "access-control", ast_acl_list_is_empty(user->acl) ? 0 : 1);
-
-		if (ast_test_flag64(user, IAX_CODEC_NOCAP)) {
-			pstr = "REQ only";
-		} else if (ast_test_flag64(user, IAX_CODEC_NOPREFS)) {
-			pstr = "disabled";
-		} else {
-			pstr = ast_test_flag64(user, IAX_CODEC_USER_FIRST) ? "caller" : "host";
-		}
-		ast_data_add_str(data_user, "codec-preferences", pstr);
-
-		if (!ast_data_search_match(search, data_user)) {
-			ast_data_remove_node(data_root, data_user);
-		}
-	}
-	ao2_iterator_destroy(&i);
-
-	ast_free(auth);
-	return 0;
-}
-
-static const struct ast_data_handler peers_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = peers_data_provider_get
-};
-
-static const struct ast_data_handler users_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = users_data_provider_get
-};
-
-static const struct ast_data_entry iax2_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/channel/iax2/peers", &peers_data_provider),
-	AST_DATA_ENTRY("asterisk/channel/iax2/users", &users_data_provider),
-};
-
 /*!
  * \brief Load the module
  *
@@ -15183,13 +14891,6 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-#ifdef TEST_FRAMEWORK
-	AST_TEST_REGISTER(test_iax2_peers_get);
-	AST_TEST_REGISTER(test_iax2_users_get);
-#endif
-
-	/* Register AstData providers */
-	ast_data_register_multiple(iax2_data_providers, ARRAY_LEN(iax2_data_providers));
 	ast_cli_register_multiple(cli_iax2, ARRAY_LEN(cli_iax2));
 
 	ast_register_application_xml(papp, iax2_prov_app);
@@ -15240,10 +14941,11 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Inter Asterisk eXchange (Ver 2)",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-		.nonoptreq = "res_crypto",
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "dnsmgr",
+	.optional_modules = "res_crypto",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_mgcp.c 1:16.1.1~dfsg-1/channels/chan_mgcp.c
--- 1:13.23.1~dfsg-2/channels/chan_mgcp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_mgcp.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,14 +40,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <net/if.h>
 #include <fcntl.h>
 #include <netdb.h>
-#include <sys/signal.h>
 #include <signal.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
@@ -2018,7 +2015,6 @@ static int process_sdp(struct mgcp_subch
 	ast_rtp_instance_set_remote_address(sub->rtp, &sin_tmp);
 	ast_debug(3, "Peer RTP is at port %s:%d\n", ast_inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
 	/* Scan through the RTP payload types specified in a "m=" line: */
-	ast_rtp_codecs_payloads_clear(ast_rtp_instance_get_codecs(sub->rtp), sub->rtp);
 	codecs = ast_strdupa(m + len);
 	while (!ast_strlen_zero(codecs)) {
 		if (sscanf(codecs, "%30d%n", &codec, &len) != 1) {
@@ -3799,7 +3795,7 @@ static int mgcp_prune_realtime_gateway(s
 			if (prune) {
 				ast_mutex_destroy(&s->lock);
 				ast_mutex_destroy(&s->cx_queue_lock);
-				free(s);
+				ast_free(s);
 			}
 		}
 		ast_mutex_unlock(&e->lock);
@@ -3809,7 +3805,7 @@ static int mgcp_prune_realtime_gateway(s
 			ast_mutex_destroy(&e->lock);
 			ast_mutex_destroy(&e->rqnt_queue_lock);
 			ast_mutex_destroy(&e->cmd_queue_lock);
-			free(e);
+			ast_free(e);
 		}
 	}
 	if (prune) {
@@ -3901,7 +3897,7 @@ static void *do_monitor(void *data)
 						}
 						ast_mutex_unlock(&g->msgs_lock);
 						ast_mutex_destroy(&g->msgs_lock);
-						free(g);
+						ast_free(g);
 					} else {
 						ast_mutex_unlock(&g->msgs_lock);
 						gprev = g;
@@ -4096,7 +4092,19 @@ static struct mgcp_gateway *build_gatewa
 			ast_sockaddr_to_sin(&tmp, &gw->defaddr);
 		} else if (!strcasecmp(v->name, "permit") ||
 			!strcasecmp(v->name, "deny")) {
-			gw->ha = ast_append_ha(v->name, v->value, gw->ha, NULL);
+			int acl_error = 0;
+			gw->ha = ast_append_ha(v->name, v->value, gw->ha, &acl_error);
+			if (acl_error) {
+				ast_log(LOG_ERROR, "Invalid ACL '%s' specified for MGCP gateway '%s' on line %d. Not creating.\n",
+						v->value, cat, v->lineno);
+				if (!gw_reload) {
+					ast_mutex_destroy(&gw->msgs_lock);
+					ast_free(gw);
+				} else {
+					gw->delme = 1;
+				}
+				return NULL;
+			}
 		} else if (!strcasecmp(v->name, "port")) {
 			gw->addr.sin_port = htons(atoi(v->value));
 		} else if (!strcasecmp(v->name, "context")) {
@@ -4234,6 +4242,8 @@ static struct mgcp_gateway *build_gatewa
 						 * knows that we care about it.  Then, chan_mgcp will get the MWI from the
 						 * event cache instead of checking the mailbox directly. */
 						e->mwi_event_sub = stasis_subscribe_pool(mailbox_specific_topic, stasis_subscription_cb_noop, NULL);
+						stasis_subscription_accept_message_type(e->mwi_event_sub, ast_mwi_state_type());
+						stasis_subscription_set_filter(e->mwi_event_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 					}
 				}
 				snprintf(e->rqnt_ident, sizeof(e->rqnt_ident), "%08lx", (unsigned long)ast_random());
@@ -4475,7 +4485,8 @@ static enum ast_rtp_glue_result mgcp_get
 	if (!(sub = ast_channel_tech_pvt(chan)) || !(sub->rtp))
 		return AST_RTP_GLUE_RESULT_FORBID;
 
-	*instance = sub->rtp ? ao2_ref(sub->rtp, +1), sub->rtp : NULL;
+	ao2_ref(sub->rtp, +1);
+	*instance = sub->rtp;
 
 	if (sub->parent->directmedia)
 		return AST_RTP_GLUE_RESULT_REMOTE;
@@ -5011,10 +5022,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Media Gateway Control Protocol (MGCP)",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-		.nonoptreq = "res_pktccops",
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.optional_modules = "res_pktccops",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_misdn.c 1:16.1.1~dfsg-1/channels/chan_misdn.c
--- 1:13.23.1~dfsg-2/channels/chan_misdn.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_misdn.c	2018-12-26 18:45:13.000000000 +0000
@@ -76,13 +76,12 @@
 	<depend>isdnnet</depend>
 	<depend>misdn</depend>
 	<depend>suppserv</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
+	<replacement>chan_dahdi</replacement>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pthread.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -12831,7 +12830,7 @@ static void chan_misdn_log(int level, in
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Channel driver for mISDN Support (BRI/PRI)",
-	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload,
diff -pruN 1:13.23.1~dfsg-2/channels/chan_motif.c 1:16.1.1~dfsg-1/channels/chan_motif.c
--- 1:13.23.1~dfsg-2/channels/chan_motif.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_motif.c	2018-12-26 18:45:13.000000000 +0000
@@ -44,14 +44,12 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <fcntl.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <iksemel.h>
 #include <pthread.h>
 
@@ -317,7 +315,7 @@ struct jingle_session {
 	struct ast_format_cap *peercap;       /*!< Peer codec capabilities */
 	unsigned int outgoing:1;              /*!< Whether this is an outgoing leg or not */
 	unsigned int gone:1;                  /*!< In the eyes of Jingle this session is already gone */
-	struct ast_callid *callid;            /*!< Bound session call-id */
+	ast_callid callid;                    /*!< Bound session call-id */
 };
 
 static const char channel_type[] = "Motif";
@@ -473,7 +471,9 @@ static struct jingle_endpoint_state *jin
 		return NULL;
 	}
 
-	if (!(state->sessions = ao2_container_alloc(SESSION_BUCKETS, jingle_session_hash, jingle_session_cmp))) {
+	state->sessions = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		SESSION_BUCKETS, jingle_session_hash, NULL, jingle_session_cmp);
+	if (!state->sessions) {
 		ao2_ref(state, -1);
 		return NULL;
 	}
@@ -584,10 +584,6 @@ static void jingle_session_destructor(vo
 	ao2_cleanup(session->jointcap);
 	ao2_cleanup(session->peercap);
 
-	if (session->callid) {
-		ast_callid_unref(session->callid);
-	}
-
 	ast_string_field_free_memory(session);
 }
 
@@ -607,7 +603,9 @@ static void *jingle_config_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->endpoints = ao2_container_alloc(ENDPOINT_BUCKETS, jingle_endpoint_hash, jingle_endpoint_cmp))) {
+	cfg->endpoints = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		ENDPOINT_BUCKETS, jingle_endpoint_hash, NULL, jingle_endpoint_cmp);
+	if (!cfg->endpoints) {
 		ao2_ref(cfg, -1);
 		return NULL;
 	}
@@ -703,7 +701,7 @@ static void jingle_enable_video(struct j
 static struct jingle_session *jingle_alloc(struct jingle_endpoint *endpoint, const char *from, const char *sid)
 {
 	struct jingle_session *session;
-	struct ast_callid *callid;
+	ast_callid callid;
 	struct ast_sockaddr tmp;
 
 	if (!(session = ao2_alloc(sizeof(*session), jingle_session_destructor))) {
@@ -2822,9 +2820,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Motif Jingle Channel Driver",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_xmpp",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_multicast_rtp.c 1:16.1.1~dfsg-1/channels/chan_multicast_rtp.c
--- 1:13.23.1~dfsg-2/channels/chan_multicast_rtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_multicast_rtp.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,224 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2009, Digium, Inc.
- *
- * Joshua Colp <jcolp@digium.com>
- * Andreas 'MacBrody' Brodmann <andreas.brodmann@gmail.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- *
- * \author Joshua Colp <jcolp@digium.com>
- * \author Andreas 'MacBrody' Broadmann <andreas.brodmann@gmail.com>
- *
- * \brief Multicast RTP Paging Channel
- *
- * \ingroup channel_drivers
- */
-
-/*** MODULEINFO
-	<support_level>deprecated</support_level>
-	<defaultenabled>no</defaultenabled>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include <fcntl.h>
-#include <sys/signal.h>
-
-#include "asterisk/lock.h"
-#include "asterisk/channel.h"
-#include "asterisk/config.h"
-#include "asterisk/module.h"
-#include "asterisk/pbx.h"
-#include "asterisk/sched.h"
-#include "asterisk/io.h"
-#include "asterisk/acl.h"
-#include "asterisk/callerid.h"
-#include "asterisk/file.h"
-#include "asterisk/cli.h"
-#include "asterisk/app.h"
-#include "asterisk/rtp_engine.h"
-#include "asterisk/causes.h"
-
-static const char tdesc[] = "Multicast RTP Paging Channel Driver";
-
-/* Forward declarations */
-static struct ast_channel *multicast_rtp_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause);
-static int multicast_rtp_call(struct ast_channel *ast, const char *dest, int timeout);
-static int multicast_rtp_hangup(struct ast_channel *ast);
-static struct ast_frame *multicast_rtp_read(struct ast_channel *ast);
-static int multicast_rtp_write(struct ast_channel *ast, struct ast_frame *f);
-
-/* Channel driver declaration */
-static struct ast_channel_tech multicast_rtp_tech = {
-	.type = "MulticastRTP",
-	.description = tdesc,
-	.requester = multicast_rtp_request,
-	.call = multicast_rtp_call,
-	.hangup = multicast_rtp_hangup,
-	.read = multicast_rtp_read,
-	.write = multicast_rtp_write,
-};
-
-/*! \brief Function called when we should read a frame from the channel */
-static struct ast_frame  *multicast_rtp_read(struct ast_channel *ast)
-{
-	return &ast_null_frame;
-}
-
-/*! \brief Function called when we should write a frame to the channel */
-static int multicast_rtp_write(struct ast_channel *ast, struct ast_frame *f)
-{
-	struct ast_rtp_instance *instance = ast_channel_tech_pvt(ast);
-
-	return ast_rtp_instance_write(instance, f);
-}
-
-/*! \brief Function called when we should actually call the destination */
-static int multicast_rtp_call(struct ast_channel *ast, const char *dest, int timeout)
-{
-	struct ast_rtp_instance *instance = ast_channel_tech_pvt(ast);
-
-	ast_queue_control(ast, AST_CONTROL_ANSWER);
-
-	return ast_rtp_instance_activate(instance);
-}
-
-/*! \brief Function called when we should hang the channel up */
-static int multicast_rtp_hangup(struct ast_channel *ast)
-{
-	struct ast_rtp_instance *instance = ast_channel_tech_pvt(ast);
-
-	ast_rtp_instance_destroy(instance);
-
-	ast_channel_tech_pvt_set(ast, NULL);
-
-	return 0;
-}
-
-/*! \brief Function called when we should prepare to call the destination */
-static struct ast_channel *multicast_rtp_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause)
-{
-	char *tmp = ast_strdupa(data), *multicast_type = tmp, *destination, *control;
-	struct ast_rtp_instance *instance;
-	struct ast_sockaddr control_address;
-	struct ast_sockaddr destination_address;
-	struct ast_channel *chan;
-	struct ast_format_cap *caps = NULL;
-	struct ast_format *fmt = NULL;
-
-	fmt = ast_format_cap_get_format(cap, 0);
-
-	ast_sockaddr_setnull(&control_address);
-
-	/* If no type was given we can't do anything */
-	if (ast_strlen_zero(multicast_type)) {
-		goto failure;
-	}
-
-	if (!(destination = strchr(tmp, '/'))) {
-		goto failure;
-	}
-	*destination++ = '\0';
-
-	if ((control = strchr(destination, '/'))) {
-		*control++ = '\0';
-		if (!ast_sockaddr_parse(&control_address, control,
-					PARSE_PORT_REQUIRE)) {
-			goto failure;
-		}
-	}
-
-	if (!ast_sockaddr_parse(&destination_address, destination,
-				PARSE_PORT_REQUIRE)) {
-		goto failure;
-	}
-
-	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-	if (!caps) {
-		goto failure;
-	}
-
-	if (!(instance = ast_rtp_instance_new("multicast", NULL, &control_address, multicast_type))) {
-		goto failure;
-	}
-
-	if (!(chan = ast_channel_alloc(1, AST_STATE_DOWN, "", "", "", "", "", assignedids, requestor, 0, "MulticastRTP/%p", instance))) {
-		ast_rtp_instance_destroy(instance);
-		goto failure;
-	}
-	ast_rtp_instance_set_channel_id(instance, ast_channel_uniqueid(chan));
-	ast_rtp_instance_set_remote_address(instance, &destination_address);
-
-	ast_channel_tech_set(chan, &multicast_rtp_tech);
-
-	ast_format_cap_append(caps, fmt, 0);
-	ast_channel_nativeformats_set(chan, caps);
-	ast_channel_set_writeformat(chan, fmt);
-	ast_channel_set_rawwriteformat(chan, fmt);
-	ast_channel_set_readformat(chan, fmt);
-	ast_channel_set_rawreadformat(chan, fmt);
-
-	ast_channel_tech_pvt_set(chan, instance);
-
-	ast_channel_unlock(chan);
-
-	ao2_ref(fmt, -1);
-	ao2_ref(caps, -1);
-
-	return chan;
-
-failure:
-	ao2_cleanup(fmt);
-	ao2_cleanup(caps);
-	*cause = AST_CAUSE_FAILURE;
-	return NULL;
-}
-
-/*! \brief Function called when our module is loaded */
-static int load_module(void)
-{
-	if (!(multicast_rtp_tech.capabilities = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
-		return AST_MODULE_LOAD_DECLINE;
-	}
-	ast_format_cap_append_by_type(multicast_rtp_tech.capabilities, AST_MEDIA_TYPE_UNKNOWN);
-	if (ast_channel_register(&multicast_rtp_tech)) {
-		ast_log(LOG_ERROR, "Unable to register channel class 'MulticastRTP'\n");
-		ao2_ref(multicast_rtp_tech.capabilities, -1);
-		multicast_rtp_tech.capabilities = NULL;
-		return AST_MODULE_LOAD_DECLINE;
-	}
-
-	return AST_MODULE_LOAD_SUCCESS;
-}
-
-/*! \brief Function called when our module is unloaded */
-static int unload_module(void)
-{
-	ast_channel_unregister(&multicast_rtp_tech);
-	ao2_ref(multicast_rtp_tech.capabilities, -1);
-	multicast_rtp_tech.capabilities = NULL;
-
-	return 0;
-}
-
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Multicast RTP Paging Channel (use chan_rtp instead)",
-	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
-	.load = load_module,
-	.unload = unload_module,
-	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_nbs.c 1:16.1.1~dfsg-1/channels/chan_nbs.c
--- 1:13.23.1~dfsg-2/channels/chan_nbs.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_nbs.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,13 +27,11 @@
 
 /*** MODULEINFO
 	<depend>nbs</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <arpa/inet.h>
@@ -272,4 +270,4 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Network Broadcast Sound Support");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Network Broadcast Sound Support");
diff -pruN 1:13.23.1~dfsg-2/channels/chan_oss.c 1:16.1.1~dfsg-1/channels/chan_oss.c
--- 1:13.23.1~dfsg-2/channels/chan_oss.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_oss.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,13 +40,11 @@
 
 /*** MODULEINFO
 	<depend>oss</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>		/* isalnum() used here */
 #include <math.h>
 #include <sys/ioctl.h>
@@ -1526,4 +1524,4 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "OSS Console Channel Driver");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "OSS Console Channel Driver");
diff -pruN 1:13.23.1~dfsg-2/channels/chan_phone.c 1:16.1.1~dfsg-1/channels/chan_phone.c
--- 1:13.23.1~dfsg-2/channels/chan_phone.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_phone.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,11 @@
 
 /*** MODULEINFO
 	<depend>ixjuser</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <sys/socket.h>
 #include <sys/time.h>
@@ -1516,4 +1514,4 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD_EXTENDED(ASTERISK_GPL_KEY, "Linux Telephony API Support");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Linux Telephony API Support");
diff -pruN 1:13.23.1~dfsg-2/channels/chan_pjsip.c 1:16.1.1~dfsg-1/channels/chan_pjsip.c
--- 1:13.23.1~dfsg-2/channels/chan_pjsip.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_pjsip.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,6 @@
 #include <pjsip_ua.h>
 #include <pjlib.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
@@ -67,6 +65,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #include "asterisk/res_pjsip.h"
 #include "asterisk/res_pjsip_session.h"
+#include "asterisk/stream.h"
 
 #include "pjsip/include/chan_pjsip.h"
 #include "pjsip/include/dialplan_functions.h"
@@ -81,17 +80,13 @@ static unsigned int chan_idx;
 
 static void chan_pjsip_pvt_dtor(void *obj)
 {
-	struct chan_pjsip_pvt *pvt = obj;
-	int i;
-
-	for (i = 0; i < SIP_MEDIA_SIZE; ++i) {
-		ao2_cleanup(pvt->media[i]);
-		pvt->media[i] = NULL;
-	}
 }
 
 /* \brief Asterisk core interaction functions */
 static struct ast_channel *chan_pjsip_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause);
+static struct ast_channel *chan_pjsip_request_with_stream_topology(const char *type,
+	struct ast_stream_topology *topology, const struct ast_assigned_ids *assignedids,
+	const struct ast_channel *requestor, const char *data, int *cause);
 static int chan_pjsip_sendtext_data(struct ast_channel *ast, struct ast_msg_data *msg);
 static int chan_pjsip_sendtext(struct ast_channel *ast, const char *text);
 static int chan_pjsip_digit_begin(struct ast_channel *ast, char digit);
@@ -99,8 +94,9 @@ static int chan_pjsip_digit_end(struct a
 static int chan_pjsip_call(struct ast_channel *ast, const char *dest, int timeout);
 static int chan_pjsip_hangup(struct ast_channel *ast);
 static int chan_pjsip_answer(struct ast_channel *ast);
-static struct ast_frame *chan_pjsip_read(struct ast_channel *ast);
+static struct ast_frame *chan_pjsip_read_stream(struct ast_channel *ast);
 static int chan_pjsip_write(struct ast_channel *ast, struct ast_frame *f);
+static int chan_pjsip_write_stream(struct ast_channel *ast, int stream_num, struct ast_frame *f);
 static int chan_pjsip_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen);
 static int chan_pjsip_transfer(struct ast_channel *ast, const char *target);
 static int chan_pjsip_fixup(struct ast_channel *oldchan, struct ast_channel *newchan);
@@ -113,6 +109,7 @@ struct ast_channel_tech chan_pjsip_tech
 	.type = channel_type,
 	.description = "PJSIP Channel Driver",
 	.requester = chan_pjsip_request,
+	.requester_with_stream_topology = chan_pjsip_request_with_stream_topology,
 	.send_text = chan_pjsip_sendtext,
 	.send_text_data = chan_pjsip_sendtext_data,
 	.send_digit_begin = chan_pjsip_digit_begin,
@@ -120,10 +117,10 @@ struct ast_channel_tech chan_pjsip_tech
 	.call = chan_pjsip_call,
 	.hangup = chan_pjsip_hangup,
 	.answer = chan_pjsip_answer,
-	.read = chan_pjsip_read,
+	.read_stream = chan_pjsip_read_stream,
 	.write = chan_pjsip_write,
-	.write_video = chan_pjsip_write,
-	.exception = chan_pjsip_read,
+	.write_stream = chan_pjsip_write_stream,
+	.exception = chan_pjsip_read_stream,
 	.indicate = chan_pjsip_indicate,
 	.transfer = chan_pjsip_transfer,
 	.fixup = chan_pjsip_fixup,
@@ -171,11 +168,20 @@ static struct ast_sip_session_supplement
 static enum ast_rtp_glue_result chan_pjsip_get_rtp_peer(struct ast_channel *chan, struct ast_rtp_instance **instance)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-	struct chan_pjsip_pvt *pvt;
 	struct ast_sip_endpoint *endpoint;
 	struct ast_datastore *datastore;
+	struct ast_sip_session_media *media;
 
-	if (!channel || !channel->session || !(pvt = channel->pvt) || !pvt->media[SIP_MEDIA_AUDIO]->rtp) {
+	if (!channel || !channel->session) {
+		return AST_RTP_GLUE_RESULT_FORBID;
+	}
+
+	/* XXX Getting the first RTP instance for direct media related stuff seems just
+	 * absolutely wrong. But the native RTP bridge knows no other method than single-stream
+	 * for direct media. So this is the best we can do.
+	 */
+	media = channel->session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
+	if (!media || !media->rtp) {
 		return AST_RTP_GLUE_RESULT_FORBID;
 	}
 
@@ -187,7 +193,7 @@ static enum ast_rtp_glue_result chan_pjs
 
 	endpoint = channel->session->endpoint;
 
-	*instance = pvt->media[SIP_MEDIA_AUDIO]->rtp;
+	*instance = media->rtp;
 	ao2_ref(*instance, +1);
 
 	ast_assert(endpoint != NULL);
@@ -206,16 +212,21 @@ static enum ast_rtp_glue_result chan_pjs
 static enum ast_rtp_glue_result chan_pjsip_get_vrtp_peer(struct ast_channel *chan, struct ast_rtp_instance **instance)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
 	struct ast_sip_endpoint *endpoint;
+	struct ast_sip_session_media *media;
 
-	if (!pvt || !channel->session || !pvt->media[SIP_MEDIA_VIDEO]->rtp) {
+	if (!channel || !channel->session) {
+		return AST_RTP_GLUE_RESULT_FORBID;
+	}
+
+	media = channel->session->active_media_state->default_session[AST_MEDIA_TYPE_VIDEO];
+	if (!media || !media->rtp) {
 		return AST_RTP_GLUE_RESULT_FORBID;
 	}
 
 	endpoint = channel->session->endpoint;
 
-	*instance = pvt->media[SIP_MEDIA_VIDEO]->rtp;
+	*instance = media->rtp;
 	ao2_ref(*instance, +1);
 
 	ast_assert(endpoint != NULL);
@@ -277,18 +288,43 @@ static int direct_media_mitigate_glare(s
 	return 0;
 }
 
+/*! \brief Helper function to find the position for RTCP */
+static int rtp_find_rtcp_fd_position(struct ast_sip_session *session, struct ast_rtp_instance *rtp)
+{
+	int index;
+
+	for (index = 0; index < AST_VECTOR_SIZE(&session->active_media_state->read_callbacks); ++index) {
+		struct ast_sip_session_media_read_callback_state *callback_state =
+			AST_VECTOR_GET_ADDR(&session->active_media_state->read_callbacks, index);
+
+		if (callback_state->fd != ast_rtp_instance_fd(rtp, 1)) {
+			continue;
+		}
+
+		return index;
+	}
+
+	return -1;
+}
+
 /*!
  * \pre chan is locked
  */
 static int check_for_rtp_changes(struct ast_channel *chan, struct ast_rtp_instance *rtp,
-		struct ast_sip_session_media *media, int rtcp_fd)
+		struct ast_sip_session_media *media, struct ast_sip_session *session)
 {
-	int changed = 0;
+	int changed = 0, position = -1;
+
+	if (media->rtp) {
+		position = rtp_find_rtcp_fd_position(session, media->rtp);
+	}
 
 	if (rtp) {
 		changed = ast_rtp_instance_get_and_cmp_remote_address(rtp, &media->direct_media_addr);
 		if (media->rtp) {
-			ast_channel_set_fd(chan, rtcp_fd, -1);
+			if (position != -1) {
+				ast_channel_set_fd(chan, position + AST_EXTENDED_FDS, -1);
+			}
 			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_RTCP, 0);
 		}
 	} else if (!ast_sockaddr_isnull(&media->direct_media_addr)){
@@ -296,7 +332,9 @@ static int check_for_rtp_changes(struct
 		changed = 1;
 		if (media->rtp) {
 			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_RTCP, 1);
-			ast_channel_set_fd(chan, rtcp_fd, ast_rtp_instance_fd(media->rtp, 1));
+			if (position != -1) {
+				ast_channel_set_fd(chan, position + AST_EXTENDED_FDS, ast_rtp_instance_fd(media->rtp, 1));
+			}
 		}
 	}
 
@@ -345,22 +383,27 @@ static int send_direct_media_request(voi
 {
 	struct rtp_direct_media_data *cdata = data;
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(cdata->chan);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
+	struct ast_sip_session *session;
 	int changed = 0;
 	int res = 0;
 
+	/* XXX In an ideal world each media stream would be direct, but for now preserve behavior
+	 * and connect only the default media sessions for audio and video.
+	 */
+
 	/* The channel needs to be locked when checking for RTP changes.
 	 * Otherwise, we could end up destroying an underlying RTCP structure
 	 * at the same time that the channel thread is attempting to read RTCP
 	 */
 	ast_channel_lock(cdata->chan);
-	if (pvt->media[SIP_MEDIA_AUDIO]) {
+	session = channel->session;
+	if (session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO]) {
 		changed |= check_for_rtp_changes(
-			cdata->chan, cdata->rtp, pvt->media[SIP_MEDIA_AUDIO], 1);
+			cdata->chan, cdata->rtp, session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO], session);
 	}
-	if (pvt->media[SIP_MEDIA_VIDEO]) {
+	if (session->active_media_state->default_session[AST_MEDIA_TYPE_VIDEO]) {
 		changed |= check_for_rtp_changes(
-			cdata->chan, cdata->vrtp, pvt->media[SIP_MEDIA_VIDEO], 3);
+			cdata->chan, cdata->vrtp, session->active_media_state->default_session[AST_MEDIA_TYPE_VIDEO], session);
 	}
 	ast_channel_unlock(cdata->chan);
 
@@ -380,7 +423,7 @@ static int send_direct_media_request(voi
 	if (changed) {
 		ast_debug(4, "RTP changed on %s; initiating direct media update\n", ast_channel_name(cdata->chan));
 		res = ast_sip_session_refresh(cdata->session, NULL, NULL, NULL,
-			cdata->session->endpoint->media.direct_media.method, 1);
+			cdata->session->endpoint->media.direct_media.method, 1, NULL);
 	}
 
 	ao2_ref(cdata, -1);
@@ -432,14 +475,53 @@ static struct ast_rtp_glue chan_pjsip_rt
 	.update_peer = chan_pjsip_set_rtp_peer,
 };
 
-static void set_channel_on_rtp_instance(struct chan_pjsip_pvt *pvt, const char *channel_id)
+static void set_channel_on_rtp_instance(const struct ast_sip_session *session,
+	const char *channel_id)
 {
-	if (pvt->media[SIP_MEDIA_AUDIO] && pvt->media[SIP_MEDIA_AUDIO]->rtp) {
-		ast_rtp_instance_set_channel_id(pvt->media[SIP_MEDIA_AUDIO]->rtp, channel_id);
+	int i;
+
+	for (i = 0; i < AST_VECTOR_SIZE(&session->active_media_state->sessions); ++i) {
+		struct ast_sip_session_media *session_media;
+
+		session_media = AST_VECTOR_GET(&session->active_media_state->sessions, i);
+		if (!session_media || !session_media->rtp) {
+			continue;
+		}
+
+		ast_rtp_instance_set_channel_id(session_media->rtp, channel_id);
 	}
-	if (pvt->media[SIP_MEDIA_VIDEO] && pvt->media[SIP_MEDIA_VIDEO]->rtp) {
-		ast_rtp_instance_set_channel_id(pvt->media[SIP_MEDIA_VIDEO]->rtp, channel_id);
+}
+
+/*!
+ * \brief Determine if a topology is compatible with format capabilities
+ *
+ * This will return true if ANY formats in the topology are compatible with the format
+ * capabilities.
+ *
+ * XXX When supporting true multistream, we will need to be sure to mark which streams from
+ * top1 are compatible with which streams from top2. Then the ones that are not compatible
+ * will need to be marked as "removed" so that they are negotiated as expected.
+ *
+ * \param top Topology
+ * \param cap Format capabilities
+ * \retval 1 The topology has at least one compatible format
+ * \retval 0 The topology has no compatible formats or an error occurred.
+ */
+static int compatible_formats_exist(struct ast_stream_topology *top, struct ast_format_cap *cap)
+{
+	struct ast_format_cap *cap_from_top;
+	int res;
+
+	cap_from_top = ast_format_cap_from_stream_topology(top);
+
+	if (!cap_from_top) {
+		return 0;
 	}
+
+	res = ast_format_cap_iscompatible(cap_from_top, cap);
+	ao2_ref(cap_from_top, -1);
+
+	return res;
 }
 
 /*! \brief Function called to create a new PJSIP Asterisk channel */
@@ -450,12 +532,9 @@ static struct ast_channel *chan_pjsip_ne
 	RAII_VAR(struct chan_pjsip_pvt *, pvt, NULL, ao2_cleanup);
 	struct ast_sip_channel_pvt *channel;
 	struct ast_variable *var;
+	struct ast_stream_topology *topology;
 
-	if (!(pvt = ao2_alloc(sizeof(*pvt), chan_pjsip_pvt_dtor))) {
-		return NULL;
-	}
-	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-	if (!caps) {
+	if (!(pvt = ao2_alloc_options(sizeof(*pvt), chan_pjsip_pvt_dtor, AO2_ALLOC_OPT_LOCK_NOLOCK))) {
 		return NULL;
 	}
 
@@ -469,31 +548,46 @@ static struct ast_channel *chan_pjsip_ne
 		ast_sorcery_object_get_id(session->endpoint),
 		(unsigned) ast_atomic_fetchadd_int((int *) &chan_idx, +1));
 	if (!chan) {
-		ao2_ref(caps, -1);
 		return NULL;
 	}
 
 	ast_channel_tech_set(chan, &chan_pjsip_tech);
 
 	if (!(channel = ast_sip_channel_pvt_alloc(pvt, session))) {
-		ao2_ref(caps, -1);
 		ast_channel_unlock(chan);
 		ast_hangup(chan);
 		return NULL;
 	}
 
-	ast_channel_stage_snapshot(chan);
-
 	ast_channel_tech_pvt_set(chan, channel);
 
-	if (!ast_format_cap_count(session->req_caps) ||
-		!ast_format_cap_iscompatible(session->req_caps, session->endpoint->media.codecs)) {
+	if (!ast_stream_topology_get_count(session->pending_media_state->topology) ||
+		!compatible_formats_exist(session->pending_media_state->topology, session->endpoint->media.codecs)) {
+		caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+		if (!caps) {
+			ast_channel_unlock(chan);
+			ast_hangup(chan);
+			return NULL;
+		}
 		ast_format_cap_append_from_cap(caps, session->endpoint->media.codecs, AST_MEDIA_TYPE_UNKNOWN);
+		topology = ast_stream_topology_clone(session->endpoint->media.topology);
 	} else {
-		ast_format_cap_append_from_cap(caps, session->req_caps, AST_MEDIA_TYPE_UNKNOWN);
+		caps = ast_format_cap_from_stream_topology(session->pending_media_state->topology);
+		topology = ast_stream_topology_clone(session->pending_media_state->topology);
 	}
 
+	if (!topology || !caps) {
+		ao2_cleanup(caps);
+		ast_stream_topology_free(topology);
+		ast_channel_unlock(chan);
+		ast_hangup(chan);
+		return NULL;
+	}
+
+	ast_channel_stage_snapshot(chan);
+
 	ast_channel_nativeformats_set(chan, caps);
+	ast_channel_set_stream_topology(chan, topology);
 
 	if (!ast_format_cap_empty(caps)) {
 		struct ast_format *fmt;
@@ -555,12 +649,7 @@ static struct ast_channel *chan_pjsip_ne
 	ast_channel_stage_snapshot_done(chan);
 	ast_channel_unlock(chan);
 
-	/* If res_pjsip_session is ever updated to create/destroy ast_sip_session_media
-	 * during a call such as if multiple same-type stream support is introduced,
-	 * these will need to be recaptured as well */
-	pvt->media[SIP_MEDIA_AUDIO] = ao2_find(session->media, "audio", OBJ_KEY);
-	pvt->media[SIP_MEDIA_VIDEO] = ao2_find(session->media, "video", OBJ_KEY);
-	set_channel_on_rtp_instance(pvt, ast_channel_uniqueid(chan));
+	set_channel_on_rtp_instance(session, ast_channel_uniqueid(chan));
 
 	return chan;
 }
@@ -716,44 +805,27 @@ static struct ast_frame *chan_pjsip_cng_
  *
  * \note The channel is already locked.
  */
-static struct ast_frame *chan_pjsip_read(struct ast_channel *ast)
+static struct ast_frame *chan_pjsip_read_stream(struct ast_channel *ast)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct ast_sip_session *session;
-	struct chan_pjsip_pvt *pvt = channel->pvt;
+	struct ast_sip_session *session = channel->session;
+	struct ast_sip_session_media_read_callback_state *callback_state;
 	struct ast_frame *f;
-	struct ast_sip_session_media *media = NULL;
-	int rtcp = 0;
-	int fdno = ast_channel_fdno(ast);
-
-	switch (fdno) {
-	case 0:
-		media = pvt->media[SIP_MEDIA_AUDIO];
-		break;
-	case 1:
-		media = pvt->media[SIP_MEDIA_AUDIO];
-		rtcp = 1;
-		break;
-	case 2:
-		media = pvt->media[SIP_MEDIA_VIDEO];
-		break;
-	case 3:
-		media = pvt->media[SIP_MEDIA_VIDEO];
-		rtcp = 1;
-		break;
-	}
+	int fdno = ast_channel_fdno(ast) - AST_EXTENDED_FDS;
 
-	if (!media || !media->rtp) {
+	if (fdno >= AST_VECTOR_SIZE(&session->active_media_state->read_callbacks)) {
 		return &ast_null_frame;
 	}
 
-	if (!(f = ast_rtp_instance_read(media->rtp, rtcp))) {
+	callback_state = AST_VECTOR_GET_ADDR(&session->active_media_state->read_callbacks, fdno);
+	f = callback_state->read_callback(session, callback_state->session);
+
+	if (!f) {
 		return f;
 	}
 
-	ast_rtp_instance_set_last_rx(media->rtp, time(NULL));
-
-	if (f->frametype != AST_FRAME_VOICE) {
+	if (f->frametype != AST_FRAME_VOICE ||
+		callback_state->session != session->active_media_state->default_session[callback_state->session->type]) {
 		return f;
 	}
 
@@ -833,22 +905,31 @@ static struct ast_frame *chan_pjsip_read
 	return f;
 }
 
-/*! \brief Function called by core to write frames */
-static int chan_pjsip_write(struct ast_channel *ast, struct ast_frame *frame)
+static int chan_pjsip_write_stream(struct ast_channel *ast, int stream_num, struct ast_frame *frame)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
-	struct ast_sip_session_media *media;
+	struct ast_sip_session *session = channel->session;
+	struct ast_sip_session_media *media = NULL;
 	int res = 0;
 
+	/* The core provides a guarantee that the stream will exist when we are called if stream_num is provided */
+	if (stream_num >= 0) {
+		/* What is not guaranteed is that a media session will exist */
+		if (stream_num < AST_VECTOR_SIZE(&channel->session->active_media_state->sessions)) {
+			media = AST_VECTOR_GET(&channel->session->active_media_state->sessions, stream_num);
+		}
+	}
+
 	switch (frame->frametype) {
 	case AST_FRAME_VOICE:
-		media = pvt->media[SIP_MEDIA_AUDIO];
-
 		if (!media) {
 			return 0;
-		}
-		if (ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), frame->subclass.format) == AST_FORMAT_CMP_NOT_EQUAL) {
+		} else if (media->type != AST_MEDIA_TYPE_AUDIO) {
+			ast_debug(3, "Channel %s stream %d is of type '%s', not audio!\n",
+				ast_channel_name(ast), stream_num, ast_codec_media_type2str(media->type));
+			return 0;
+		} else if (media == channel->session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO] &&
+			ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), frame->subclass.format) == AST_FORMAT_CMP_NOT_EQUAL) {
 			struct ast_str *cap_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
 			struct ast_str *write_transpath = ast_str_alloca(256);
 			struct ast_str *read_transpath = ast_str_alloca(256);
@@ -865,20 +946,47 @@ static int chan_pjsip_write(struct ast_c
 				ast_format_get_name(ast_channel_rawwriteformat(ast)),
 				ast_translate_path_to_str(ast_channel_writetrans(ast), &write_transpath));
 			return 0;
-		}
-		if (media->rtp) {
-			res = ast_rtp_instance_write(media->rtp, frame);
+		} else if (media->write_callback) {
+			res = media->write_callback(session, media, frame);
+
 		}
 		break;
 	case AST_FRAME_VIDEO:
-		if ((media = pvt->media[SIP_MEDIA_VIDEO]) && media->rtp) {
-			res = ast_rtp_instance_write(media->rtp, frame);
+		if (!media) {
+			return 0;
+		} else if (media->type != AST_MEDIA_TYPE_VIDEO) {
+			ast_debug(3, "Channel %s stream %d is of type '%s', not video!\n",
+				ast_channel_name(ast), stream_num, ast_codec_media_type2str(media->type));
+			return 0;
+		} else if (media->write_callback) {
+			res = media->write_callback(session, media, frame);
 		}
 		break;
 	case AST_FRAME_MODEM:
+		if (!media) {
+			return 0;
+		} else if (media->type != AST_MEDIA_TYPE_IMAGE) {
+			ast_debug(3, "Channel %s stream %d is of type '%s', not image!\n",
+				ast_channel_name(ast), stream_num, ast_codec_media_type2str(media->type));
+			return 0;
+		} else if (media->write_callback) {
+			res = media->write_callback(session, media, frame);
+		}
 		break;
 	case AST_FRAME_CNG:
 		break;
+	case AST_FRAME_RTCP:
+		/* We only support writing out feedback */
+		if (frame->subclass.integer != AST_RTP_RTCP_PSFB || !media) {
+			return 0;
+		} else if (media->type != AST_MEDIA_TYPE_VIDEO) {
+			ast_debug(3, "Channel %s stream %d is of type '%s', not video! Unable to write RTCP feedback.\n",
+				ast_channel_name(ast), stream_num, ast_codec_media_type2str(media->type));
+			return 0;
+		} else if (media->write_callback) {
+			res = media->write_callback(session, media, frame);
+		}
+		break;
 	default:
 		ast_log(LOG_WARNING, "Can't send %u type frames with PJSIP\n", frame->frametype);
 		break;
@@ -887,11 +995,15 @@ static int chan_pjsip_write(struct ast_c
 	return res;
 }
 
+static int chan_pjsip_write(struct ast_channel *ast, struct ast_frame *frame)
+{
+	return chan_pjsip_write_stream(ast, -1, frame);
+}
+
 /*! \brief Function called by core to change the underlying owner channel */
 static int chan_pjsip_fixup(struct ast_channel *oldchan, struct ast_channel *newchan)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(newchan);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
 
 	if (channel->session->channel != oldchan) {
 		return -1;
@@ -904,7 +1016,7 @@ static int chan_pjsip_fixup(struct ast_c
 	 */
 	channel->session->channel = newchan;
 
-	set_channel_on_rtp_instance(pvt, ast_channel_uniqueid(newchan));
+	set_channel_on_rtp_instance(channel->session, ast_channel_uniqueid(newchan));
 
 	return 0;
 }
@@ -1268,7 +1380,8 @@ static int is_colp_update_allowed(struct
 	struct ast_party_id connected_id;
 	int update_allowed = 0;
 
-	if (!session->endpoint->id.send_pai && !session->endpoint->id.send_rpid) {
+	if (!session->endpoint->send_connected_line
+		|| (!session->endpoint->id.send_pai && !session->endpoint->id.send_rpid)) {
 		return 0;
 	}
 
@@ -1318,9 +1431,9 @@ static int update_connected_line_informa
 			/* Only the INVITE method actually needs SDP, UPDATE can do without */
 			generate_new_sdp = (method == AST_SIP_SESSION_REFRESH_METHOD_INVITE);
 
-			ast_sip_session_refresh(session, NULL, NULL, NULL, method, generate_new_sdp);
+			ast_sip_session_refresh(session, NULL, NULL, NULL, method, generate_new_sdp, NULL);
 		}
-	} else if (session->endpoint->rpid_immediate
+	} else if (session->endpoint->id.rpid_immediate
 		&& session->inv_session->state != PJSIP_INV_STATE_DISCONNECTED
 		&& is_colp_update_allowed(session)) {
 		int response_code = 0;
@@ -1348,16 +1461,133 @@ static int update_connected_line_informa
 	return 0;
 }
 
+/*! \brief Callback which changes the value of locally held on the media stream */
+static void local_hold_set_state(struct ast_sip_session_media *session_media, unsigned int held)
+{
+	if (session_media) {
+		session_media->locally_held = held;
+	}
+}
+
+/*! \brief Update local hold state and send a re-INVITE with the new SDP */
+static int remote_send_hold_refresh(struct ast_sip_session *session, unsigned int held)
+{
+	AST_VECTOR_CALLBACK_VOID(&session->active_media_state->sessions, local_hold_set_state, held);
+	ast_sip_session_refresh(session, NULL, NULL, NULL, AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1, NULL);
+	ao2_ref(session, -1);
+
+	return 0;
+}
+
+/*! \brief Update local hold state to be held */
+static int remote_send_hold(void *data)
+{
+	return remote_send_hold_refresh(data, 1);
+}
+
+/*! \brief Update local hold state to be unheld */
+static int remote_send_unhold(void *data)
+{
+	return remote_send_hold_refresh(data, 0);
+}
+
+struct topology_change_refresh_data {
+	struct ast_sip_session *session;
+	struct ast_sip_session_media_state *media_state;
+};
+
+static void topology_change_refresh_data_free(struct topology_change_refresh_data *refresh_data)
+{
+	ao2_cleanup(refresh_data->session);
+
+	ast_sip_session_media_state_free(refresh_data->media_state);
+	ast_free(refresh_data);
+}
+
+static struct topology_change_refresh_data *topology_change_refresh_data_alloc(
+	struct ast_sip_session *session, const struct ast_stream_topology *topology)
+{
+	struct topology_change_refresh_data *refresh_data;
+
+	refresh_data = ast_calloc(1, sizeof(*refresh_data));
+	if (!refresh_data) {
+		return NULL;
+	}
+
+	refresh_data->session = ao2_bump(session);
+	refresh_data->media_state = ast_sip_session_media_state_alloc();
+	if (!refresh_data->media_state) {
+		topology_change_refresh_data_free(refresh_data);
+		return NULL;
+	}
+	refresh_data->media_state->topology = ast_stream_topology_clone(topology);
+	if (!refresh_data->media_state->topology) {
+		topology_change_refresh_data_free(refresh_data);
+		return NULL;
+	}
+
+	return refresh_data;
+}
+
+static int on_topology_change_response(struct ast_sip_session *session, pjsip_rx_data *rdata)
+{
+	if (PJSIP_IS_STATUS_IN_CLASS(rdata->msg_info.msg->line.status.code, 200)) {
+		/* The topology was changed to something new so give notice to what requested
+		 * it so it queries the channel and updates accordingly.
+		 */
+		if (session->channel) {
+			ast_queue_control(session->channel, AST_CONTROL_STREAM_TOPOLOGY_CHANGED);
+		}
+	} else if (300 <= rdata->msg_info.msg->line.status.code) {
+		/* The topology change failed, so drop the current pending media state */
+		ast_sip_session_media_state_reset(session->pending_media_state);
+	}
+
+	return 0;
+}
+
+static int send_topology_change_refresh(void *data)
+{
+	struct topology_change_refresh_data *refresh_data = data;
+	int ret;
+
+	ret = ast_sip_session_refresh(refresh_data->session, NULL, NULL, on_topology_change_response,
+		AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1, refresh_data->media_state);
+	refresh_data->media_state = NULL;
+	topology_change_refresh_data_free(refresh_data);
+
+	return ret;
+}
+
+static int handle_topology_request_change(struct ast_sip_session *session,
+	const struct ast_stream_topology *proposed)
+{
+	struct topology_change_refresh_data *refresh_data;
+	int res;
+
+	refresh_data = topology_change_refresh_data_alloc(session, proposed);
+	if (!refresh_data) {
+		return -1;
+	}
+
+	res = ast_sip_push_task(session->serializer, send_topology_change_refresh, refresh_data);
+	if (res) {
+		topology_change_refresh_data_free(refresh_data);
+	}
+	return res;
+}
+
 /*! \brief Function called by core to ask the channel to indicate some sort of condition */
 static int chan_pjsip_indicate(struct ast_channel *ast, int condition, const void *data, size_t datalen)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
 	struct ast_sip_session_media *media;
 	int response_code = 0;
 	int res = 0;
 	char *device_buf;
 	size_t device_buf_size;
+	int i;
+	const struct ast_stream_topology *topology;
 
 	switch (condition) {
 	case AST_CONTROL_RINGING:
@@ -1410,40 +1640,50 @@ static int chan_pjsip_indicate(struct as
 		ast_devstate_changed(AST_DEVICE_UNKNOWN, AST_DEVSTATE_CACHABLE, "PJSIP/%s", ast_sorcery_object_get_id(channel->session->endpoint));
 		break;
 	case AST_CONTROL_VIDUPDATE:
-		media = pvt->media[SIP_MEDIA_VIDEO];
-		if (media && media->rtp) {
-			/* FIXME: Only use this for VP8. Additional work would have to be done to
-			 * fully support other video codecs */
-
-			if (ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), ast_format_vp8) != AST_FORMAT_CMP_NOT_EQUAL ||
-				ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), ast_format_vp9) != AST_FORMAT_CMP_NOT_EQUAL) {
-				/* FIXME Fake RTP write, this will be sent as an RTCP packet. Ideally the
-				 * RTP engine would provide a way to externally write/schedule RTCP
-				 * packets */
-				struct ast_frame fr;
-				fr.frametype = AST_FRAME_CONTROL;
-				fr.subclass.integer = AST_CONTROL_VIDUPDATE;
-				res = ast_rtp_instance_write(media->rtp, &fr);
-			} else {
-				ao2_ref(channel->session, +1);
-#ifdef HAVE_PJSIP_INV_SESSION_REF
-				if (pjsip_inv_add_ref(channel->session->inv_session) != PJ_SUCCESS) {
-					ast_log(LOG_ERROR, "Can't increase the session reference counter\n");
-					ao2_cleanup(channel->session);
+		for (i = 0; i < AST_VECTOR_SIZE(&channel->session->active_media_state->sessions); ++i) {
+			media = AST_VECTOR_GET(&channel->session->active_media_state->sessions, i);
+			if (!media || media->type != AST_MEDIA_TYPE_VIDEO) {
+				continue;
+			}
+			if (media->rtp) {
+				/* FIXME: Only use this for VP8. Additional work would have to be done to
+				 * fully support other video codecs */
+
+				if (ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), ast_format_vp8) != AST_FORMAT_CMP_NOT_EQUAL ||
+					ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), ast_format_vp9) != AST_FORMAT_CMP_NOT_EQUAL ||
+					(channel->session->endpoint->media.webrtc &&
+					 ast_format_cap_iscompatible_format(ast_channel_nativeformats(ast), ast_format_h264) != AST_FORMAT_CMP_NOT_EQUAL)) {
+					/* FIXME Fake RTP write, this will be sent as an RTCP packet. Ideally the
+					 * RTP engine would provide a way to externally write/schedule RTCP
+					 * packets */
+					struct ast_frame fr;
+					fr.frametype = AST_FRAME_CONTROL;
+					fr.subclass.integer = AST_CONTROL_VIDUPDATE;
+					res = ast_rtp_instance_write(media->rtp, &fr);
 				} else {
-#endif
-					if (ast_sip_push_task(channel->session->serializer, transmit_info_with_vidupdate, channel->session)) {
+					ao2_ref(channel->session, +1);
+#ifdef HAVE_PJSIP_INV_SESSION_REF
+					if (pjsip_inv_add_ref(channel->session->inv_session) != PJ_SUCCESS) {
+						ast_log(LOG_ERROR, "Can't increase the session reference counter\n");
 						ao2_cleanup(channel->session);
-					}
+					} else {
+#endif
+						if (ast_sip_push_task(channel->session->serializer, transmit_info_with_vidupdate, channel->session)) {
+							ao2_cleanup(channel->session);
+						}
 #ifdef HAVE_PJSIP_INV_SESSION_REF
-				}
+					}
 #endif
+				}
+				ast_test_suite_event_notify("AST_CONTROL_VIDUPDATE", "Result: Success");
+			} else {
+				ast_test_suite_event_notify("AST_CONTROL_VIDUPDATE", "Result: Failure");
+				res = -1;
 			}
-			ast_test_suite_event_notify("AST_CONTROL_VIDUPDATE", "Result: Success");
-		} else {
-			ast_test_suite_event_notify("AST_CONTROL_VIDUPDATE", "Result: Failure");
-			res = -1;
 		}
+		/* XXX If there were no video streams, then this should set
+		 * res to -1
+		 */
 		break;
 	case AST_CONTROL_CONNECTED_LINE:
 		ao2_ref(channel->session, +1);
@@ -1490,7 +1730,15 @@ static int chan_pjsip_indicate(struct as
 		device_buf = alloca(device_buf_size);
 		ast_channel_get_device_name(ast, device_buf, device_buf_size);
 		ast_devstate_changed_literal(AST_DEVICE_ONHOLD, 1, device_buf);
-		ast_moh_start(ast, data, NULL);
+		if (!channel->session->endpoint->moh_passthrough) {
+			ast_moh_start(ast, data, NULL);
+		} else {
+			if (ast_sip_push_task(channel->session->serializer, remote_send_hold, ao2_bump(channel->session))) {
+				ast_log(LOG_WARNING, "Could not queue task to remotely put session '%s' on hold with endpoint '%s'\n",
+					ast_sorcery_object_get_id(channel->session), ast_sorcery_object_get_id(channel->session->endpoint));
+				ao2_ref(channel->session, -1);
+			}
+		}
 		break;
 	case AST_CONTROL_UNHOLD:
 		chan_pjsip_remove_hold(ast_channel_uniqueid(ast));
@@ -1498,7 +1746,15 @@ static int chan_pjsip_indicate(struct as
 		device_buf = alloca(device_buf_size);
 		ast_channel_get_device_name(ast, device_buf, device_buf_size);
 		ast_devstate_changed_literal(AST_DEVICE_UNKNOWN, 1, device_buf);
-		ast_moh_stop(ast);
+		if (!channel->session->endpoint->moh_passthrough) {
+			ast_moh_stop(ast);
+		} else {
+			if (ast_sip_push_task(channel->session->serializer, remote_send_unhold, ao2_bump(channel->session))) {
+				ast_log(LOG_WARNING, "Could not queue task to remotely take session '%s' off hold with endpoint '%s'\n",
+					ast_sorcery_object_get_id(channel->session), ast_sorcery_object_get_id(channel->session->endpoint));
+				ao2_ref(channel->session, -1);
+			}
+		}
 		break;
 	case AST_CONTROL_SRCUPDATE:
 		break;
@@ -1523,6 +1779,14 @@ static int chan_pjsip_indicate(struct as
 		}
 
 		break;
+	case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+		topology = data;
+		res = handle_topology_request_change(channel->session, topology);
+		break;
+	case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+		break;
+	case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
+		break;
 	case -1:
 		res = -1;
 		break;
@@ -1736,10 +2000,11 @@ static int chan_pjsip_transfer(struct as
 static int chan_pjsip_digit_begin(struct ast_channel *chan, char digit)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
-	struct ast_sip_session_media *media = pvt->media[SIP_MEDIA_AUDIO];
+	struct ast_sip_session_media *media;
 	int res = 0;
 
+	media = channel->session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
+
 	switch (channel->session->dtmf) {
 	case AST_SIP_DTMF_RFC_4733:
 		if (!media || !media->rtp) {
@@ -1856,10 +2121,11 @@ failure:
 static int chan_pjsip_digit_end(struct ast_channel *ast, char digit, unsigned int duration)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
-	struct ast_sip_session_media *media = pvt->media[SIP_MEDIA_AUDIO];
+	struct ast_sip_session_media *media;
 	int res = 0;
 
+	media = channel->session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
+
 	switch (channel->session->dtmf) {
 	case AST_SIP_DTMF_AUTO_INFO:
 	{
@@ -1956,7 +2222,6 @@ static int call(void *data)
 {
 	struct ast_sip_channel_pvt *channel = data;
 	struct ast_sip_session *session = channel->session;
-	struct chan_pjsip_pvt *pvt = channel->pvt;
 	pjsip_tx_data *tdata;
 
 	int res = ast_sip_session_create_invite(session, &tdata);
@@ -1965,7 +2230,7 @@ static int call(void *data)
 		ast_set_hangupsource(session->channel, ast_channel_name(session->channel), 0);
 		ast_queue_hangup(session->channel);
 	} else {
-		set_channel_on_rtp_instance(pvt, ast_channel_uniqueid(session->channel));
+		set_channel_on_rtp_instance(session, ast_channel_uniqueid(session->channel));
 		update_initial_connected_line(session);
 		ast_sip_session_send_request(session, tdata);
 	}
@@ -2063,10 +2328,10 @@ static struct hangup_data *hangup_data_a
 }
 
 /*! \brief Clear a channel from a session along with its PVT */
-static void clear_session_and_channel(struct ast_sip_session *session, struct ast_channel *ast, struct chan_pjsip_pvt *pvt)
+static void clear_session_and_channel(struct ast_sip_session *session, struct ast_channel *ast)
 {
 	session->channel = NULL;
-	set_channel_on_rtp_instance(pvt, "");
+	set_channel_on_rtp_instance(session, "");
 	ast_channel_tech_pvt_set(ast, NULL);
 }
 
@@ -2075,7 +2340,6 @@ static int hangup(void *data)
 	struct hangup_data *h_data = data;
 	struct ast_channel *ast = h_data->chan;
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct chan_pjsip_pvt *pvt = channel->pvt;
 	struct ast_sip_session *session = channel->session;
 	int cause = h_data->cause;
 
@@ -2085,7 +2349,7 @@ static int hangup(void *data)
 	 * afterwards.
 	 */
 	ast_sip_session_terminate(ao2_bump(session), cause);
-	clear_session_and_channel(session, ast, pvt);
+	clear_session_and_channel(session, ast);
 	ao2_cleanup(session);
 	ao2_cleanup(channel);
 	ao2_cleanup(h_data);
@@ -2096,7 +2360,6 @@ static int hangup(void *data)
 static int chan_pjsip_hangup(struct ast_channel *ast)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(ast);
-	struct chan_pjsip_pvt *pvt;
 	int cause;
 	struct hangup_data *h_data;
 
@@ -2104,7 +2367,6 @@ static int chan_pjsip_hangup(struct ast_
 		return -1;
 	}
 
-	pvt = channel->pvt;
 	cause = hangup_cause2sip(ast_channel_hangupcause(channel->session->channel));
 	h_data = hangup_data_alloc(cause, ast);
 
@@ -2123,7 +2385,7 @@ failure:
 	/* Go ahead and do our cleanup of the session and channel even if we're not going
 	 * to be able to send our SIP request/response
 	 */
-	clear_session_and_channel(channel->session, ast, pvt);
+	clear_session_and_channel(channel->session, ast);
 	ao2_cleanup(channel);
 	ao2_cleanup(h_data);
 
@@ -2132,7 +2394,7 @@ failure:
 
 struct request_data {
 	struct ast_sip_session *session;
-	struct ast_format_cap *caps;
+	struct ast_stream_topology *topology;
 	const char *dest;
 	int cause;
 };
@@ -2230,7 +2492,7 @@ static int request(void *obj)
 	}
 
 	session = ast_sip_session_create_outgoing(endpoint, NULL, args.aor, request_user,
-		req_data->caps);
+		req_data->topology);
 	ao2_ref(endpoint, -1);
 	if (!session) {
 		ast_log(LOG_ERROR, "Failed to create outgoing session to endpoint '%s'\n", endpoint_name);
@@ -2244,12 +2506,12 @@ static int request(void *obj)
 }
 
 /*! \brief Function called by core to create a new outgoing PJSIP session */
-static struct ast_channel *chan_pjsip_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause)
+static struct ast_channel *chan_pjsip_request_with_stream_topology(const char *type, struct ast_stream_topology *topology, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause)
 {
 	struct request_data req_data;
 	RAII_VAR(struct ast_sip_session *, session, NULL, ao2_cleanup);
 
-	req_data.caps = cap;
+	req_data.topology = topology;
 	req_data.dest = data;
 	/* Default failure value in case ast_sip_push_task_wait_servant() itself fails. */
 	req_data.cause = AST_CAUSE_FAILURE;
@@ -2269,6 +2531,23 @@ static struct ast_channel *chan_pjsip_re
 	return session->channel;
 }
 
+static struct ast_channel *chan_pjsip_request(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *data, int *cause)
+{
+	struct ast_stream_topology *topology;
+	struct ast_channel *chan;
+
+	topology = ast_stream_topology_create_from_format_cap(cap);
+	if (!topology) {
+		return NULL;
+	}
+
+	chan = chan_pjsip_request_with_stream_topology(type, topology, assignedids, requestor, data, cause);
+
+	ast_stream_topology_free(topology);
+
+	return chan;
+}
+
 struct sendtext_data {
 	struct ast_sip_session *session;
 	struct ast_msg_data *msg;
@@ -2773,6 +3052,11 @@ static struct ast_custom_function chan_p
 	.read = pjsip_acf_dial_contacts_read,
 };
 
+static struct ast_custom_function chan_pjsip_parse_uri_function = {
+	.name = "PJSIP_PARSE_URI",
+	.read = pjsip_acf_parse_uri_read,
+};
+
 static struct ast_custom_function media_offer_function = {
 	.name = "PJSIP_MEDIA_OFFER",
 	.read = pjsip_acf_media_offer_read,
@@ -2804,8 +3088,6 @@ static int load_module(void)
 {
 	struct ao2_container *endpoints;
 
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	if (!(chan_pjsip_tech.capabilities = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -2824,6 +3106,11 @@ static int load_module(void)
 		goto end;
 	}
 
+	if (ast_custom_function_register(&chan_pjsip_parse_uri_function)) {
+		ast_log(LOG_ERROR, "Unable to register PJSIP_PARSE_URI dialplan function\n");
+		goto end;
+	}
+
 	if (ast_custom_function_register(&media_offer_function)) {
 		ast_log(LOG_WARNING, "Unable to register PJSIP_MEDIA_OFFER dialplan function\n");
 		goto end;
@@ -2839,15 +3126,8 @@ static int load_module(void)
 		goto end;
 	}
 
-	if (ast_sip_session_register_supplement(&chan_pjsip_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register PJSIP supplement\n");
-		goto end;
-	}
-
-	if (ast_sip_session_register_supplement(&chan_pjsip_supplement_response)) {
-		ast_log(LOG_ERROR, "Unable to register PJSIP supplement\n");
-		goto end;
-	}
+	ast_sip_session_register_supplement(&chan_pjsip_supplement);
+	ast_sip_session_register_supplement(&chan_pjsip_supplement_response);
 
 	if (!(pjsip_uids_onhold = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK,
 			AO2_CONTAINER_ALLOC_OPT_DUPS_REJECT, 37, uid_hold_hash_fn,
@@ -2856,20 +3136,9 @@ static int load_module(void)
 		goto end;
 	}
 
-	if (ast_sip_session_register_supplement(&call_pickup_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register PJSIP call pickup supplement\n");
-		goto end;
-	}
-
-	if (ast_sip_session_register_supplement(&pbx_start_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register PJSIP pbx start supplement\n");
-		goto end;
-	}
-
-	if (ast_sip_session_register_supplement(&chan_pjsip_ack_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register PJSIP ACK supplement\n");
-		goto end;
-	}
+	ast_sip_session_register_supplement(&call_pickup_supplement);
+	ast_sip_session_register_supplement(&pbx_start_supplement);
+	ast_sip_session_register_supplement(&chan_pjsip_ack_supplement);
 
 	if (pjsip_channel_cli_register()) {
 		ast_log(LOG_ERROR, "Unable to register PJSIP Channel CLI\n");
@@ -2896,6 +3165,7 @@ end:
 	ast_custom_function_unregister(&dtmf_mode_function);
 	ast_custom_function_unregister(&media_offer_function);
 	ast_custom_function_unregister(&chan_pjsip_dial_contacts_function);
+	ast_custom_function_unregister(&chan_pjsip_parse_uri_function);
 	ast_custom_function_unregister(&session_refresh_function);
 	ast_channel_unregister(&chan_pjsip_tech);
 	ast_rtp_glue_unregister(&chan_pjsip_rtp_glue);
@@ -2920,6 +3190,7 @@ static int unload_module(void)
 	ast_custom_function_unregister(&dtmf_mode_function);
 	ast_custom_function_unregister(&media_offer_function);
 	ast_custom_function_unregister(&chan_pjsip_dial_contacts_function);
+	ast_custom_function_unregister(&chan_pjsip_parse_uri_function);
 	ast_custom_function_unregister(&session_refresh_function);
 
 	ast_channel_unregister(&chan_pjsip_tech);
@@ -2930,8 +3201,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Channel Driver",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_rtp.c 1:16.1.1~dfsg-1/channels/chan_rtp.c
--- 1:13.23.1~dfsg-2/channels/chan_rtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_rtp.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
@@ -434,4 +432,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_rtp_multicast",
 );
diff -pruN 1:13.23.1~dfsg-2/channels/chan_sip.c 1:16.1.1~dfsg-1/channels/chan_sip.c
--- 1:13.23.1~dfsg-2/channels/chan_sip.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_sip.c	2018-12-26 18:45:13.000000000 +0000
@@ -199,32 +199,35 @@
 	The session-timers parameter in sip.conf defines the mode of operation of SIP session-timers feature in
 	Asterisk. The Asterisk can be configured in one of the following three modes:
 
-	1. Accept :: In the "accept" mode, the Asterisk server honors session-timers requests
-		made by remote end-points. A remote end-point can request Asterisk to engage
-		session-timers by either sending it an INVITE request with a "Supported: timer"
-		header in it or by responding to Asterisk's INVITE with a 200 OK that contains
-		Session-Expires: header in it. In this mode, the Asterisk server does not
-		request session-timers from remote end-points. This is the default mode.
-	2. Originate :: In the "originate" mode, the Asterisk server requests the remote
-		end-points to activate session-timers in addition to honoring such requests
-		made by the remote end-pints. In order to get as much protection as possible
-		against hanging SIP channels due to network or end-point failures, Asterisk
-		resends periodic re-INVITEs even if a remote end-point does not support
-		the session-timers feature.
-	3. Refuse :: In the "refuse" mode, Asterisk acts as if it does not support session-
-		timers for inbound or outbound requests. If a remote end-point requests
-		session-timers in a dialog, then Asterisk ignores that request unless it's
-		noted as a requirement (Require: header), in which case the INVITE is
-		rejected with a 420 Bad Extension response.
+	1. Accept :: In the "accept" mode, the Asterisk server honors
+		session-timers requests made by remote end-points. A remote
+		end-point can request Asterisk to engage session-timers by either
+		sending it an INVITE request with a "Supported: timer" header in
+		it or by responding to Asterisk's INVITE with a 200 OK that
+		contains Session-Expires: header in it. In this mode, the Asterisk
+		server does not request session-timers from remote
+		end-points. This is the default mode.
+
+	2. Originate :: In the "originate" mode, the Asterisk server
+		requests the remote end-points to activate session-timers in
+		addition to honoring such requests made by the remote
+		end-points. In order to get as much protection as possible against
+		hanging SIP channels due to network or end-point failures,
+		Asterisk resends periodic re-INVITEs even if a remote end-point
+		does not support the session-timers feature.
+
+	3. Refuse :: In the "refuse" mode, Asterisk acts as if it does not
+		support session- timers for inbound or outbound requests. If a
+		remote end-point requests session-timers in a dialog, then
+		Asterisk ignores that request unless it's noted as a requirement
+		(Require: header), in which case the INVITE is rejected with a 420
+		Bad Extension response.
 
 */
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
-#include <sys/signal.h>
 #include <regex.h>
 #include <inttypes.h>
 
@@ -260,7 +263,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/threadstorage.h"
 #include "asterisk/translate.h"
 #include "asterisk/ast_version.h"
-#include "asterisk/data.h"
 #include "asterisk/aoc.h"
 #include "asterisk/message.h"
 #include "sip/include/sip.h"
@@ -285,6 +287,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/features_config.h"
 #include "asterisk/http_websocket.h"
 #include "asterisk/format_cache.h"
+#include "asterisk/linkedlists.h"	/* for AST_LIST_NEXT */
 
 /*** DOCUMENTATION
 	<application name="SIPDtmfMode" language="en_US">
@@ -374,9 +377,53 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<para>Since there are several headers (such as Via) which can occur multiple
 			times, SIP_HEADER takes an optional second argument to specify which header with
 			that name to retrieve. Headers start at offset <literal>1</literal>.</para>
+			<para>This function does not access headers from the REFER message if the call
+			was transferred. To obtain the REFER headers, set the dialplan variable
+			<variable>GET_TRANSFERRER_DATA</variable> to the prefix of the headers of the
+			REFER message that you need to access; for example, <literal>X-</literal> to
+			get all headers starting with <literal>X-</literal>. The variable must be set
+			before a call to the application that starts the channel that may eventually
+			transfer back into the dialplan, and must be inherited by that channel, so prefix
+			it with the <literal>_</literal> or <literal>__</literal> when setting (or
+			set it in the pre-dial handler executed on the new channel). To get all headers
+			of the REFER message, set the value to <literal>*</literal>. Headers
+			are returned in the form of a dialplan hash TRANSFER_DATA, and can be accessed
+			with the functions <variable>HASHKEYS(TRANSFER_DATA)</variable> and, e. g.,
+			<variable>HASH(TRANSFER_DATA,X-That-Special-Header)</variable>.</para>
+			<para>Please also note that contents of the SDP (an attachment to the
+			SIP request) can't be accessed with this function.</para>
+		</description>
+		<see-also>
+			<ref type="function">SIP_HEADERS</ref>
+		</see-also>
+	</function>
+	<function name="SIP_HEADERS" language="en_US">
+		<synopsis>
+			Gets the list of SIP header names from an incoming INVITE message.
+		</synopsis>
+		<syntax>
+			<parameter name="prefix">
+				<para>If specified, only the headers matching the given prefix are returned.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Returns a comma-separated list of header names (without values) from the
+			INVITE message that originated the current channel. Multiple headers with the
+			same name are included in the list only once. The returned list can be iterated
+			over using the functions POP() and SIP_HEADER().</para>
+			<para>For example, <literal>${SIP_HEADERS(Co)}</literal> might return
+			<literal>Contact,Content-Length,Content-Type</literal>. As a practical example,
+			you may use <literal>${SIP_HEADERS(X-)}</literal> to enumerate optional extended
+			headers.</para>
+			<para>This function does not access headers from the incoming SIP REFER message;
+			see the documentation of the function SIP_HEADER for how to access them.</para>
 			<para>Please observe that contents of the SDP (an attachment to the
 			SIP request) can't be accessed with this function.</para>
 		</description>
+		<see-also>
+			<ref type="function">SIP_HEADER</ref>
+			<ref type="function">POP</ref>
+		</see-also>
 	</function>
 	<function name="SIPPEER" language="en_US">
 		<synopsis>
@@ -1080,6 +1127,7 @@ static struct ast_sockaddr internip;
  */
 static struct ast_sockaddr externaddr;      /*!< External IP address if we are behind NAT */
 static struct ast_sockaddr media_address; /*!< External RTP IP address if we are behind NAT */
+static struct ast_sockaddr rtpbindaddr;   /*!< RTP: The address we bind to */
 
 static char externhost[MAXHOSTNAMELEN];   /*!< External host name */
 static time_t externexpire;             /*!< Expiration counter for re-resolving external host name in dynamic DNS */
@@ -1209,7 +1257,7 @@ static void try_suggested_sip_codec(stru
 static const char *get_sdp_iterate(int* start, struct sip_request *req, const char *name);
 static char get_sdp_line(int *start, int stop, struct sip_request *req, const char **value);
 static int find_sdp(struct sip_request *req);
-static int process_sdp(struct sip_pvt *p, struct sip_request *req, int t38action);
+static int process_sdp(struct sip_pvt *p, struct sip_request *req, int t38action, int is_offer);
 static int process_sdp_o(const char *o, struct sip_pvt *p);
 static int process_sdp_c(const char *c, struct ast_sockaddr *addr);
 static int process_sdp_a_sendonly(const char *a, int *sendonly);
@@ -1274,7 +1322,7 @@ static void check_for_nat(const struct a
 
 /*--- Device monitoring and Device/extension state/event handling */
 static int extensionstate_update(const char *context, const char *exten, struct state_notify_data *data, struct sip_pvt *p, int force);
-static int cb_extensionstate(char *context, char *exten, struct ast_state_cb_info *info, void *data);
+static int cb_extensionstate(const char *context, const char *exten, struct ast_state_cb_info *info, void *data);
 static int sip_poke_noanswer(const void *data);
 static int sip_poke_peer(struct sip_peer *peer, int force);
 static void sip_poke_all_peers(void);
@@ -1332,8 +1380,6 @@ static int sip_dtmfmode(struct ast_chann
 static int sip_addheader(struct ast_channel *chan, const char *data);
 static int sip_do_reload(enum channelreloadreason reason);
 static char *sip_reload(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a);
-static int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
-				      const char *name, int flag, int family);
 static int ast_sockaddr_resolve_first(struct ast_sockaddr *addr,
 				      const char *name, int flag);
 static int ast_sockaddr_resolve_first_transport(struct ast_sockaddr *addr,
@@ -1404,6 +1450,7 @@ static char *remove_uri_parameters(char
 static int get_refer_info(struct sip_pvt *transferer, struct sip_request *outgoing_req);
 static int get_also_info(struct sip_pvt *p, struct sip_request *oreq);
 static int parse_ok_contact(struct sip_pvt *pvt, struct sip_request *req);
+static int use_reason_header(struct sip_pvt *pvt, struct sip_request *req);
 static int set_address_from_contact(struct sip_pvt *pvt);
 static void check_via(struct sip_pvt *p, const struct sip_request *req);
 static int get_rpid(struct sip_pvt *p, struct sip_request *oreq);
@@ -1485,7 +1532,7 @@ static void handle_response(struct sip_p
 
 /*------ SRTP Support -------- */
 static int process_crypto(struct sip_pvt *p, struct ast_rtp_instance *rtp, struct ast_sdp_srtp **srtp,
-		const char *a, int secure_transport);
+		const char *a);
 
 /*------ T38 Support --------- */
 static int transmit_response_with_t38_sdp(struct sip_pvt *p, char *msg, struct sip_request *req, int retrans);
@@ -1768,8 +1815,9 @@ static int initialize_escs(void)
 {
 	int i, res = 0;
 	for (i = 0; i < ARRAY_LEN(event_state_compositors); i++) {
-		if (!((event_state_compositors[i].compositor) =
-					ao2_container_alloc(ESC_MAX_BUCKETS, esc_hash_fn, esc_cmp_fn))) {
+		event_state_compositors[i].compositor = ao2_container_alloc_hash(
+			AO2_ALLOC_OPT_LOCK_MUTEX, 0, ESC_MAX_BUCKETS, esc_hash_fn, NULL, esc_cmp_fn);
+		if (!event_state_compositors[i].compositor) {
 			res = -1;
 		}
 	}
@@ -2058,7 +2106,7 @@ static int sip_cc_monitor_request_cc(str
 		return -1;
 	}
 
-	if (!(monitor_instance->subscription_pvt = sip_alloc(NULL, NULL, 0, SIP_SUBSCRIBE, NULL, NULL))) {
+	if (!(monitor_instance->subscription_pvt = sip_alloc(NULL, NULL, 0, SIP_SUBSCRIBE, NULL, 0))) {
 		return -1;
 	}
 
@@ -2325,6 +2373,9 @@ static struct ast_tls_config sip_tls_cfg
 /*! \brief Default TLS connection configuration */
 static struct ast_tls_config default_tls_cfg;
 
+/*! \brief Default DTLS connection configuration */
+static struct ast_rtp_dtls_cfg default_dtls_cfg;
+
 /*! \brief The TCP server definition */
 static struct ast_tcptls_session_args sip_tcp_desc = {
 	.accept_fd = -1,
@@ -2544,7 +2595,7 @@ static struct sip_threadinfo *sip_thread
 	}
 	ao2_t_ref(tcptls_session, +1, "tcptls_session ref for sip_threadinfo object");
 	th->tcptls_session = tcptls_session;
-	th->type = transport ? transport : (tcptls_session->ssl ? AST_TRANSPORT_TLS: AST_TRANSPORT_TCP);
+	th->type = transport ? transport : (ast_iostream_get_ssl(tcptls_session->stream) ? AST_TRANSPORT_TLS: AST_TRANSPORT_TCP);
 	ao2_t_link(threadt, th, "Adding new tcptls helper thread");
 	ao2_t_ref(th, -1, "Decrementing threadinfo ref from alloc, only table ref remains");
 	return th;
@@ -2567,7 +2618,7 @@ static int sip_tcptls_write(struct ast_t
 
 	ao2_lock(tcptls_session);
 
-	if ((tcptls_session->fd == -1) ||
+	if (!tcptls_session->stream ||
 		!(th = ao2_t_find(threadt, &tmp, OBJ_POINTER, "ao2_find, getting sip_threadinfo in tcp helper thread")) ||
 		!(packet = ao2_alloc(sizeof(*packet), tcptls_packet_destructor)) ||
 		!(packet->data = ast_str_create(len))) {
@@ -2881,7 +2932,7 @@ static int sip_tcptls_read(struct sip_re
 			} else {
 				timeout = -1;
 			}
-			res = ast_wait_for_input(tcptls_session->fd, timeout);
+			res = ast_wait_for_input(ast_iostream_get_fd(tcptls_session->stream), timeout);
 			if (res < 0) {
 				ast_debug(2, "SIP TCP/TLS server :: ast_wait_for_input returned %d\n", res);
 				return -1;
@@ -2890,7 +2941,7 @@ static int sip_tcptls_read(struct sip_re
 				return -1;
 			}
 
-			res = ast_tcptls_server_read(tcptls_session, readbuf, sizeof(readbuf) - 1);
+			res = ast_iostream_read(tcptls_session->stream, readbuf, sizeof(readbuf) - 1);
 			if (res < 0) {
 				if (errno == EAGAIN || errno == EINTR) {
 					continue;
@@ -2951,11 +3002,8 @@ static void *_sip_tcp_helper_thread(stru
 			goto cleanup;
 		}
 
-		if (ast_fd_set_flags(tcptls_session->fd, O_NONBLOCK)) {
-			goto cleanup;
-		}
-
-		if (!(me = sip_threadinfo_create(tcptls_session, tcptls_session->ssl ? AST_TRANSPORT_TLS : AST_TRANSPORT_TCP))) {
+		ast_iostream_nonblock(tcptls_session->stream);
+		if (!(me = sip_threadinfo_create(tcptls_session, ast_iostream_get_ssl(tcptls_session->stream) ? AST_TRANSPORT_TLS : AST_TRANSPORT_TCP))) {
 			goto cleanup;
 		}
 		me->threadid = pthread_self();
@@ -2978,15 +3026,15 @@ static void *_sip_tcp_helper_thread(stru
 	}
 
 	flags = 1;
-	if (setsockopt(tcptls_session->fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
+	if (setsockopt(ast_iostream_get_fd(tcptls_session->stream), SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
 		ast_log(LOG_ERROR, "error enabling TCP keep-alives on sip socket: %s\n", strerror(errno));
 		goto cleanup;
 	}
 
-	ast_debug(2, "Starting thread for %s server\n", tcptls_session->ssl ? "TLS" : "TCP");
+	ast_debug(2, "Starting thread for %s server\n", ast_iostream_get_ssl(tcptls_session->stream) ? "TLS" : "TCP");
 
 	/* set up pollfd to watch for reads on both the socket and the alert_pipe */
-	fds[0].fd = tcptls_session->fd;
+	fds[0].fd = ast_iostream_get_fd(tcptls_session->stream);
 	fds[1].fd = me->alert_pipe[0];
 	fds[0].events = fds[1].events = POLLIN | POLLPRI;
 
@@ -3006,9 +3054,9 @@ static void *_sip_tcp_helper_thread(stru
 	 * We cannot let the stream exclusively wait for data to arrive.
 	 * We have to wake up the task to send outgoing messages.
 	 */
-	ast_tcptls_stream_set_exclusive_input(tcptls_session->stream_cookie, 0);
+	ast_iostream_set_exclusive_input(tcptls_session->stream, 0);
 
-	ast_tcptls_stream_set_timeout_sequence(tcptls_session->stream_cookie, ast_tvnow(),
+	ast_iostream_set_timeout_sequence(tcptls_session->stream, ast_tvnow(),
 		tcptls_session->client ? -1 : (authtimeout * 1000));
 
 	for (;;) {
@@ -3016,7 +3064,7 @@ static void *_sip_tcp_helper_thread(stru
 
 		if (!tcptls_session->client && req.authenticated && !authenticated) {
 			authenticated = 1;
-			ast_tcptls_stream_set_timeout_disable(tcptls_session->stream_cookie);
+			ast_iostream_set_timeout_disable(tcptls_session->stream);
 			ast_atomic_fetchadd_int(&unauth_sessions, -1);
 		}
 
@@ -3027,7 +3075,7 @@ static void *_sip_tcp_helper_thread(stru
 			}
 
 			if (timeout == 0) {
-				ast_debug(2, "SIP %s server timed out\n", tcptls_session->ssl ? "TLS": "TCP");
+				ast_debug(2, "SIP %s server timed out\n", ast_iostream_get_ssl(tcptls_session->stream) ? "TLS": "TCP");
 				goto cleanup;
 			}
 		} else {
@@ -3037,11 +3085,11 @@ static void *_sip_tcp_helper_thread(stru
 		if (ast_str_strlen(tcptls_session->overflow_buf) == 0) {
 			res = ast_poll(fds, 2, timeout); /* polls for both socket and alert_pipe */
 			if (res < 0) {
-				ast_debug(2, "SIP %s server :: ast_wait_for_input returned %d\n", tcptls_session->ssl ? "TLS": "TCP", res);
+				ast_debug(2, "SIP %s server :: ast_wait_for_input returned %d\n", ast_iostream_get_ssl(tcptls_session->stream) ? "TLS": "TCP", res);
 				goto cleanup;
 			} else if (res == 0) {
 				/* timeout */
-				ast_debug(2, "SIP %s server timed out\n", tcptls_session->ssl ? "TLS": "TCP");
+				ast_debug(2, "SIP %s server timed out\n", ast_iostream_get_ssl(tcptls_session->stream) ? "TLS": "TCP");
 				goto cleanup;
 			}
 		}
@@ -3066,14 +3114,14 @@ static void *_sip_tcp_helper_thread(stru
 
 			memset(buf, 0, sizeof(buf));
 
-			if (tcptls_session->ssl) {
+			if (ast_iostream_get_ssl(tcptls_session->stream)) {
 				set_socket_transport(&req.socket, AST_TRANSPORT_TLS);
 				req.socket.port = htons(ourport_tls);
 			} else {
 				set_socket_transport(&req.socket, AST_TRANSPORT_TCP);
 				req.socket.port = htons(ourport_tcp);
 			}
-			req.socket.fd = tcptls_session->fd;
+			req.socket.fd = ast_iostream_get_fd(tcptls_session->stream);
 
 			res = sip_tcptls_read(&req, tcptls_session, authenticated, start);
 			if (res < 0) {
@@ -3093,7 +3141,7 @@ static void *_sip_tcp_helper_thread(stru
 
 			if (read(me->alert_pipe[0], &alert, sizeof(alert)) == -1) {
 				ast_log(LOG_ERROR, "read() failed: %s\n", strerror(errno));
-				continue;
+				goto cleanup;
 			}
 
 			switch (alert) {
@@ -3107,19 +3155,22 @@ static void *_sip_tcp_helper_thread(stru
 				ao2_unlock(me);
 
 				if (packet) {
-					if (ast_tcptls_server_write(tcptls_session, ast_str_buffer(packet->data), packet->len) == -1) {
+					if (ast_iostream_write(tcptls_session->stream, ast_str_buffer(packet->data), packet->len) == -1) {
 						ast_log(LOG_WARNING, "Failure to write to tcp/tls socket\n");
 					}
 					ao2_t_ref(packet, -1, "tcptls packet sent, this is no longer needed");
+				} else {
+					goto cleanup;
 				}
 				break;
 			default:
 				ast_log(LOG_ERROR, "Unknown tcptls thread alert '%u'\n", alert);
+				goto cleanup;
 			}
 		}
 	}
 
-	ast_debug(2, "Shutting down thread for %s server\n", tcptls_session->ssl ? "TLS" : "TCP");
+	ast_debug(2, "Shutting down thread for %s server\n", ast_iostream_get_ssl(tcptls_session->stream) ? "TLS" : "TCP");
 
 cleanup:
 	if (tcptls_session && !tcptls_session->client && !authenticated) {
@@ -5971,10 +6022,15 @@ static int dialog_initialize_rtp(struct
 		return 0;
 	}
 
+	if (!ast_sockaddr_isnull(&rtpbindaddr)) {
+		ast_sockaddr_copy(&bindaddr_tmp, &rtpbindaddr);
+	} else {
+		ast_sockaddr_copy(&bindaddr_tmp, &bindaddr);
+	}
+
 	/* Make sure previous RTP instances/FD's do not leak */
 	dialog_clean_rtp(dialog);
 
-	ast_sockaddr_copy(&bindaddr_tmp, &bindaddr);
 	if (!(dialog->rtp = ast_rtp_instance_new(dialog->engine, sched, &bindaddr_tmp, NULL))) {
 		return -1;
 	}
@@ -6733,11 +6789,6 @@ static void sip_pvt_dtor(void *vdoomed)
 		ao2_ref(p->last_device_state_info, -1);
 		p->last_device_state_info = NULL;
 	}
-
-	/* Lastly, kill the callid associated with the pvt */
-	if (p->logger_callid) {
-		ast_callid_unref(p->logger_callid);
-	}
 }
 
 /*! \brief  update_call_counter: Handle call_limit for SIP devices
@@ -8071,7 +8122,7 @@ static int sip_indicate(struct ast_chann
  *
  * \return New ast_channel locked.
  */
-static struct ast_channel *sip_new(struct sip_pvt *i, int state, const char *title, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, struct ast_callid *callid)
+static struct ast_channel *sip_new(struct sip_pvt *i, int state, const char *title, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, ast_callid callid)
 {
 	struct ast_format_cap *caps;
 	struct ast_channel *tmp;
@@ -8840,12 +8891,8 @@ static struct sip_st_dlg* sip_st_alloc(s
 	return p->stimer;
 }
 
-static void sip_pvt_callid_set(struct sip_pvt *pvt, struct ast_callid *callid)
+static void sip_pvt_callid_set(struct sip_pvt *pvt, ast_callid callid)
 {
-	if (pvt->logger_callid) {
-		ast_callid_unref(pvt->logger_callid);
-	}
-	ast_callid_ref(callid);
 	pvt->logger_callid = callid;
 }
 
@@ -8854,14 +8901,14 @@ static void sip_pvt_callid_set(struct si
  * remember to release the reference.
  */
 struct sip_pvt *__sip_alloc(ast_string_field callid, struct ast_sockaddr *addr,
-				 int useglobal_nat, const int intended_method, struct sip_request *req, struct ast_callid *logger_callid,
+				 int useglobal_nat, const int intended_method, struct sip_request *req, ast_callid logger_callid,
 				 const char *file, int line, const char *func)
 {
 	struct sip_pvt *p;
 
-	p = __ao2_alloc_debug(sizeof(*p), sip_pvt_dtor,
+	p = __ao2_alloc(sizeof(*p), sip_pvt_dtor,
 		AO2_ALLOC_OPT_LOCK_MUTEX, "allocate a dialog(pvt) struct",
-		file, line, func, 1);
+		file, line, func);
 	if (!p) {
 		return NULL;
 	}
@@ -9248,20 +9295,14 @@ static void forked_invite_init(struct si
 {
 	struct sip_pvt *p;
 	const char *callid;
-	struct ast_callid *logger_callid;
+	ast_callid logger_callid;
 
 	sip_pvt_lock(original);
 	callid = ast_strdupa(original->callid);
 	logger_callid = original->logger_callid;
-	if (logger_callid) {
-		ast_callid_ref(logger_callid);
-	}
 	sip_pvt_unlock(original);
 
 	p = sip_alloc(callid, addr, 1, SIP_INVITE, req, logger_callid);
-	if (logger_callid) {
-		ast_callid_unref(logger_callid);
-	}
 	if (!p)  {
 		return; /* alloc error */
 	}
@@ -9444,7 +9485,7 @@ static struct sip_pvt *__find_call(struc
 		struct sip_pvt tmp_dialog = {
 			.callid = callid,
 		};
-		sip_pvt_ptr = __ao2_find_debug(dialogs, &tmp_dialog, OBJ_POINTER,
+		sip_pvt_ptr = __ao2_find(dialogs, &tmp_dialog, OBJ_POINTER,
 			"find_call in dialogs", file, line, func);
 		if (sip_pvt_ptr) {  /* well, if we don't find it-- what IS in there? */
 			/* Found the call */
@@ -9459,7 +9500,7 @@ static struct sip_pvt *__find_call(struc
 		struct sip_pvt *fork_pvt = NULL;
 		struct match_req_args args = { 0, };
 		int found;
-		struct ao2_iterator *iterator = __ao2_callback_debug(dialogs,
+		struct ao2_iterator *iterator = __ao2_callback(dialogs,
 			OBJ_POINTER | OBJ_MULTIPLE,
 			dialog_find_multiple,
 			&tmp_dialog,
@@ -9518,7 +9559,7 @@ static struct sip_pvt *__find_call(struc
 				/* This is likely a forked Request that somehow resulted in us receiving multiple parts of the fork.
 			 	* RFC 3261 section 8.2.2.2, Indicate that we want to merge requests by sending a 482 response. */
 				transmit_response_using_temp(callid, addr, 1, intended_method, req, "482 (Loop Detected)");
-				__ao2_ref_debug(sip_pvt_ptr, -1, "pvt did not match incoming SIP msg, unref from search.",
+				__ao2_ref(sip_pvt_ptr, -1, "pvt did not match incoming SIP msg, unref from search.",
 					file, line, func);
 				ao2_iterator_destroy(iterator);
 				dialog_unref(fork_pvt, "unref fork_pvt");
@@ -9530,7 +9571,7 @@ static struct sip_pvt *__find_call(struc
 				/* fall through */
 			case SIP_REQ_NOT_MATCH:
 			default:
-				__ao2_ref_debug(sip_pvt_ptr, -1, "pvt did not match incoming SIP msg, unref from search",
+				__ao2_ref(sip_pvt_ptr, -1, "pvt did not match incoming SIP msg, unref from search",
 					file, line, func);
 				break;
 			}
@@ -9558,7 +9599,7 @@ static struct sip_pvt *__find_call(struc
 	/* See if the method is capable of creating a dialog */
 	if (sip_methods[intended_method].can_create == CAN_CREATE_DIALOG) {
 		struct sip_pvt *p = NULL;
-		struct ast_callid *logger_callid = NULL;
+		ast_callid logger_callid = 0;
 
 		if (intended_method == SIP_INVITE) {
 			logger_callid = ast_create_callid();
@@ -9577,10 +9618,6 @@ static struct sip_pvt *__find_call(struc
 			transmit_response_using_temp(callid, addr, 1, intended_method, req, "500 Server internal error");
 			ast_debug(4, "Failed allocating SIP dialog, sending 500 Server internal error and giving up\n");
 		}
-		/* If we created an ast_callid for an invite, we need to free it now. */
-		if (logger_callid) {
-			ast_callid_unref(logger_callid);
-		}
 		return p; /* can be NULL */
 	} else if( sip_methods[intended_method].can_create == CAN_CREATE_DIALOG_UNSUPPORTED_METHOD) {
 		/* A method we do not support, let's take it on the volley */
@@ -10163,7 +10200,7 @@ static int has_media_level_attribute(int
  	Return 0 on success, a negative value on errors.
 	Must be called after find_sdp().
 */
-static int process_sdp(struct sip_pvt *p, struct sip_request *req, int t38action)
+static int process_sdp(struct sip_pvt *p, struct sip_request *req, int t38action, int is_offer)
 {
 	int res = 0;
 
@@ -10745,9 +10782,13 @@ static int process_sdp(struct sip_pvt *p
 						}
 					} else if (process_sdp_a_sendonly(value, &sendonly)) {
 						processed = TRUE;
-					} else if (!processed_crypto && process_crypto(p, p->rtp, &p->srtp, value, secure_audio)) {
+					} else if (!processed_crypto && process_crypto(p, p->rtp, &p->srtp, value)) {
 						processed_crypto = TRUE;
 						processed = TRUE;
+						if (secure_audio == FALSE) {
+							ast_log(AST_LOG_NOTICE, "Processed audio crypto attribute without SAVP specified; accepting anyway\n");
+							secure_audio = TRUE;
+						}
 					} else if (process_sdp_a_audio(value, p, &newaudiortp, &last_rtpmap_codec)) {
 						processed = TRUE;
 					} else if (process_sdp_a_rtcp_mux(value, p, &remote_rtcp_mux_audio)) {
@@ -10764,9 +10805,13 @@ static int process_sdp(struct sip_pvt *p
 						if (p->vsrtp) {
 							ast_set_flag(p->vsrtp, AST_SRTP_CRYPTO_OFFER_OK);
 						}
-					} else if (!processed_crypto && process_crypto(p, p->vrtp, &p->vsrtp, value, secure_video)) {
+					} else if (!processed_crypto && process_crypto(p, p->vrtp, &p->vsrtp, value)) {
 						processed_crypto = TRUE;
 						processed = TRUE;
+						if (secure_video == FALSE) {
+							ast_log(AST_LOG_NOTICE, "Processed video crypto attribute without SAVP specified; accepting anyway\n");
+							secure_video = TRUE;
+						}
 					} else if (process_sdp_a_video(value, p, &newvideortp, &last_rtpmap_codec)) {
 						processed = TRUE;
 					} else if (process_sdp_a_rtcp_mux(value, p, &remote_rtcp_mux_video)) {
@@ -10779,7 +10824,7 @@ static int process_sdp(struct sip_pvt *p
 						processed = TRUE;
 					} else if (process_sdp_a_text(value, p, &newtextrtp, red_fmtp, &red_num_gen, red_data_pt, &last_rtpmap_codec)) {
 						processed = TRUE;
-					} else if (!processed_crypto && process_crypto(p, p->trtp, &p->tsrtp, value, 1)) {
+					} else if (!processed_crypto && process_crypto(p, p->trtp, &p->tsrtp, value)) {
 						processed_crypto = TRUE;
 						processed = TRUE;
 					}
@@ -10866,6 +10911,16 @@ static int process_sdp(struct sip_pvt *p
 		change_t38_state(p, T38_DISABLED);
 	}
 
+	if (is_offer) {
+		/*
+		 * Setup rx payload type mapping to prefer the mapping
+		 * from the peer that the RFC says we SHOULD use.
+		 */
+		ast_rtp_codecs_payloads_xover(&newaudiortp, &newaudiortp, NULL);
+		ast_rtp_codecs_payloads_xover(&newvideortp, &newvideortp, NULL);
+		ast_rtp_codecs_payloads_xover(&newtextrtp, &newtextrtp, NULL);
+	}
+
 	/* Now gather all of the codecs that we are asked for: */
 	ast_rtp_codecs_payload_formats(&newaudiortp, peercapability, &peernoncodeccapability);
 	ast_rtp_codecs_payload_formats(&newvideortp, vpeercapability, &vpeernoncodeccapability);
@@ -13123,7 +13178,7 @@ static void add_codec_to_sdp(const struc
 	/* Opus mandates 2 channels in rtpmap */
 	if (ast_format_cmp(format, ast_format_opus) == AST_FORMAT_CMP_EQUAL) {
 		ast_str_append(a_buf, 0, "a=rtpmap:%d %s/%u/2\r\n", rtp_code, mime, rate);
-	} else {
+	} else if ((AST_RTP_PT_LAST_STATIC < rtp_code) || !(sip_cfg.compactheaders)) {
 		ast_str_append(a_buf, 0, "a=rtpmap:%d %s/%u\r\n", rtp_code, mime, rate);
 	}
 
@@ -13362,21 +13417,34 @@ static void get_our_media_address(struct
 
 static char *crypto_get_attrib(struct ast_sdp_srtp *srtp, int dtls_enabled, int default_taglen_32)
 {
+	struct ast_sdp_srtp *tmp = srtp;
 	char *a_crypto;
-	const char *orig_crypto;
 
-	if (!srtp || dtls_enabled) {
+	if (!tmp || dtls_enabled) {
 		return NULL;
 	}
 
-	orig_crypto = ast_sdp_srtp_get_attrib(srtp, dtls_enabled, default_taglen_32);
-	if (ast_strlen_zero(orig_crypto)) {
+	a_crypto = ast_strdup("");
+	if (!a_crypto) {
 		return NULL;
 	}
 
-	if (ast_asprintf(&a_crypto, "a=crypto:%s\r\n", orig_crypto) == -1) {
-		return NULL;
-	}
+	do {
+		char *copy = a_crypto;
+		const char *orig_crypto = ast_sdp_srtp_get_attrib(tmp, dtls_enabled, default_taglen_32);
+
+		if (ast_strlen_zero(orig_crypto)) {
+			ast_free(copy);
+			return NULL;
+		}
+		if (ast_asprintf(&a_crypto, "%sa=crypto:%s\r\n", copy, orig_crypto) == -1) {
+			ast_free(copy);
+			return NULL;
+		}
+
+		ast_free(copy);
+	} while ((tmp = AST_LIST_NEXT(tmp, sdp_srtp_list)));
+
 	return a_crypto;
 }
 
@@ -14587,7 +14655,7 @@ static int transmit_publish(struct sip_e
 
 	epa_entry->publish_type = publish_type;
 
-	if (!(pvt = sip_alloc(NULL, NULL, 0, SIP_PUBLISH, NULL, NULL))) {
+	if (!(pvt = sip_alloc(NULL, NULL, 0, SIP_PUBLISH, NULL, 0))) {
 		return -1;
 	}
 
@@ -14966,7 +15034,7 @@ static int __sip_subscribe_mwi_do(struct
 	}
 
 	/* Create a dialog that we will use for the subscription */
-	if (!(mwi->call = sip_alloc(NULL, NULL, 0, SIP_SUBSCRIBE, NULL, NULL))) {
+	if (!(mwi->call = sip_alloc(NULL, NULL, 0, SIP_SUBSCRIBE, NULL, 0))) {
 		return -1;
 	}
 
@@ -15073,7 +15141,7 @@ static void state_notify_build_xml(struc
 
 	switch (data->state) {
 	case (AST_EXTENSION_RINGING | AST_EXTENSION_INUSE):
-		statestring = (sip_cfg.notifyringing) ? "early" : "confirmed";
+		statestring = (sip_cfg.notifyringing == NOTIFYRINGING_ENABLED) ? "early" : "confirmed";
 		local_state = NOTIFY_INUSE;
 		pidfstate = "busy";
 		pidfnote = "Ringing";
@@ -15566,13 +15634,13 @@ static int manager_sipnotify(struct mans
 			ast_variables_destroy(p->notify->headers);
 		}
 	} else {
-		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, NULL))) {
+		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, 0))) {
 			astman_send_error(s, m, "Unable to build sip pvt data for notify (memory/socket error)");
 			ast_variables_destroy(vars);
 			return 0;
 		}
 
-		if (create_addr(p, channame, NULL, 0)) {
+		if (create_addr(p, channame, NULL, 1)) {
 			/* Maybe they're not registered, etc. */
 			dialog_unlink_all(p);
 			dialog_unref(p, "unref dialog inside for loop" );
@@ -16036,7 +16104,7 @@ static int transmit_register(struct sip_
 			r->callid_valid = TRUE;
 		}
 		/* Allocate SIP dialog for registration */
-		if (!(p = sip_alloc( r->callid, NULL, 0, SIP_REGISTER, NULL, NULL))) {
+		if (!(p = sip_alloc( r->callid, NULL, 0, SIP_REGISTER, NULL, 0))) {
 			ast_log(LOG_WARNING, "Unable to allocate registration transaction (memory or socket error)\n");
 			return 0;
 		}
@@ -16706,6 +16774,49 @@ static int parse_ok_contact(struct sip_p
 	return TRUE;
 }
 
+/*!
+ * \brief Parses SIP reason header according to RFC3326 and sets channel's hangupcause if configured so
+ *  and header present
+ *
+ * \note This is used in BYE and CANCEL request and SIP response, but according to RFC3326 it could
+ *       appear in any request, but makes not a lot of sense in others than BYE or CANCEL.
+ *       Currently only implemented for Q.850 status codes.
+ * \retval 0 success
+ * \retval -1 on failure or if not configured
+ */
+static int use_reason_header(struct sip_pvt *pvt, struct sip_request *req)
+{
+	int ret, cause;
+	const char *rp, *rh;
+
+	if (!pvt->owner) {
+		return -1;
+	}
+
+	if (!ast_test_flag(&pvt->flags[1], SIP_PAGE2_Q850_REASON) ||
+		!(rh = sip_get_header(req, "Reason"))) {
+		return -1;
+	}
+
+	rh = ast_skip_blanks(rh);
+	if (strncasecmp(rh, "Q.850", 5)) {
+		return -1;
+	}
+
+	ret = -1;
+	cause = ast_channel_hangupcause(pvt->owner);
+	rp = strstr(rh, "cause=");
+	if (rp && sscanf(rp + 6, "%3d", &cause) == 1) {
+		ret = 0;
+		ast_channel_hangupcause_set(pvt->owner, cause & 0x7f);
+		if (req->debug) {
+			ast_verbose("Using Reason header for cause code: %d\n",
+						ast_channel_hangupcause(pvt->owner));
+		}
+	}
+	return ret;
+}
+
 /*! \brief parse uri in a way that allows semicolon stripping if legacy mode is enabled
  *
  * \note This calls parse_uri which has the unexpected property that passing more
@@ -17384,6 +17495,8 @@ static void network_change_stasis_subscr
 	if (!network_change_sub) {
 		network_change_sub = stasis_subscribe(ast_system_topic(),
 			network_change_stasis_cb, NULL);
+		stasis_subscription_accept_message_type(network_change_sub, ast_network_change_type());
+		stasis_subscription_set_filter(network_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 }
 
@@ -17397,6 +17510,8 @@ static void acl_change_stasis_subscribe(
 	if (!acl_change_sub) {
 		acl_change_sub = stasis_subscribe(ast_security_topic(),
 			acl_change_stasis_cb, NULL);
+		stasis_subscription_accept_message_type(acl_change_sub, ast_named_acl_change_type());
+		stasis_subscription_set_filter(acl_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 
 }
@@ -17520,7 +17635,7 @@ static int extensionstate_update(const c
 /*! \brief Callback for the devicestate notification (SUBSCRIBE) support subsystem
 \note	If you add an "hint" priority to the extension in the dial plan,
 	you will get notifications on device state changes */
-static int cb_extensionstate(char *context, char *exten, struct ast_state_cb_info *info, void *data)
+static int cb_extensionstate(const char *context, const char *exten, struct ast_state_cb_info *info, void *data)
 {
 	struct sip_pvt *p = data;
 	struct state_notify_data notify_data = {
@@ -18610,6 +18725,29 @@ static int get_sip_pvt_from_replaces(con
 	return 0;
 }
 
+static void extract_transferrer_headers(const char *prefix, struct ast_channel *peer, const struct sip_request *req)
+{
+	struct ast_str *pbxvar = ast_str_alloca(120);
+	int i;
+
+	/* The '*' alone matches all headers. */
+	if (strcmp(prefix, "*") == 0) {
+		prefix = "";
+	}
+
+	for (i = 0; i < req->headers; i++) {
+		const char *header = REQ_OFFSET_TO_STR(req, header[i]);
+		if (ast_begins_with(header, prefix)) {
+			int hdrlen = strcspn(header, " \t:");
+			const char *val = ast_skip_blanks(header + hdrlen);
+			if (hdrlen > 0 && *val == ':') {
+				ast_str_set(&pbxvar, -1, "~HASH~TRANSFER_DATA~%.*s~", hdrlen, header);
+				pbx_builtin_setvar_helper(peer, ast_str_buffer(pbxvar), ast_skip_blanks(val + 1));
+			}
+		}
+	}
+}
+
 /*! \brief Call transfer support (the REFER method)
  * 	Extracts Refer headers into pvt dialog structure
  *
@@ -18672,10 +18810,19 @@ static int get_refer_info(struct sip_pvt
 
 		peer = ast_channel_bridge_peer(owner_ref);
 		if (peer) {
+			const char *get_xfrdata;
+
 			pbx_builtin_setvar_helper(peer, "SIPREFERRINGCONTEXT",
 				S_OR(transferer->context, NULL));
 			pbx_builtin_setvar_helper(peer, "__SIPREFERREDBYHDR",
 				S_OR(p_referred_by, NULL));
+
+			ast_channel_lock(peer);
+			get_xfrdata = pbx_builtin_getvar_helper(peer, "GET_TRANSFERRER_DATA");
+			if (!ast_strlen_zero(get_xfrdata)) {
+				extract_transferrer_headers(get_xfrdata, peer, req);
+			}
+			ast_channel_unlock(peer);
 		}
 
 		owner_relock = sip_pvt_lock_full(transferer);
@@ -21632,6 +21779,10 @@ static char *sip_show_settings(struct as
 		default_tls_cfg.enabled != FALSE ?
 				ast_sockaddr_stringify(&sip_tls_desc.local_address) :
 				"Disabled");
+	ast_cli(a->fd, "  RTP Bindaddress:        %s\n",
+		!ast_sockaddr_isnull(&rtpbindaddr) ?
+				ast_sockaddr_stringify_addr(&rtpbindaddr) :
+				"Disabled");
 	ast_cli(a->fd, "  Videosupport:           %s\n", AST_CLI_YESNO(ast_test_flag(&global_flags[1], SIP_PAGE2_VIDEOSUPPORT)));
 	ast_cli(a->fd, "  Textsupport:            %s\n", AST_CLI_YESNO(ast_test_flag(&global_flags[1], SIP_PAGE2_TEXTSUPPORT)));
 	ast_cli(a->fd, "  Ignore SDP sess. ver.:  %s\n", AST_CLI_YESNO(ast_test_flag(&global_flags[1], SIP_PAGE2_IGNORESDPVERSION)));
@@ -21761,7 +21912,7 @@ static char *sip_show_settings(struct as
 	ast_cli(a->fd, "  Outbound reg. timeout:  %d secs\n", global_reg_timeout);
 	ast_cli(a->fd, "  Outbound reg. attempts: %d\n", global_regattempts_max);
 	ast_cli(a->fd, "  Outbound reg. retry 403:%s\n", AST_CLI_YESNO(global_reg_retry_403));
-	ast_cli(a->fd, "  Notify ringing state:   %s\n", AST_CLI_YESNO(sip_cfg.notifyringing));
+	ast_cli(a->fd, "  Notify ringing state:   %s%s\n", AST_CLI_YESNO(sip_cfg.notifyringing), sip_cfg.notifyringing == NOTIFYRINGING_NOTINUSE ? " (when not in use)" : "");
 	if (sip_cfg.notifyringing) {
 		ast_cli(a->fd, "    Include CID:          %s%s\n",
 				AST_CLI_YESNO(sip_cfg.notifycid),
@@ -22658,7 +22809,7 @@ static char *sip_cli_notify(struct ast_c
 		char buf[512];
 		struct ast_variable *header, *var;
 
-		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, NULL))) {
+		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, 0))) {
 			ast_log(LOG_WARNING, "Unable to build sip pvt data for notify (memory/socket error)\n");
 			return CLI_FAILURE;
 		}
@@ -22974,6 +23125,7 @@ static int func_header_read(struct ast_c
 {
 	struct sip_pvt *p;
 	const char *content = NULL;
+	char *mutable_data = ast_strdupa(data);
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(header);
 		AST_APP_ARG(number);
@@ -22997,7 +23149,7 @@ static int func_header_read(struct ast_c
 		return -1;
 	}
 
-	AST_STANDARD_APP_ARGS(args, data);
+	AST_STANDARD_APP_ARGS(args, mutable_data);
 	if (!args.number) {
 		number = 1;
 	} else {
@@ -23033,6 +23185,91 @@ static struct ast_custom_function sip_he
 	.read = func_header_read,
 };
 
+/*! \brief Read unique list of SIP headers (dialplan function) */
+static int func_headers_read2(struct ast_channel *chan, const char *function, char *data, struct ast_str **buf, ssize_t maxlen)
+{
+	int i;
+	struct sip_pvt *pvt;
+	char *mutable_data = ast_strdupa(data);
+	struct ast_str *token = ast_str_alloca(100);
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(pattern);
+	);
+
+	if (!chan) {
+		return -1;
+	}
+
+	ast_channel_lock(chan);
+
+	if (!IS_SIP_TECH(ast_channel_tech(chan))) {
+		ast_log(LOG_WARNING, "This function can only be used on SIP channels.\n");
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	pvt = ast_channel_tech_pvt(chan);
+	if (!pvt) {
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	AST_STANDARD_APP_ARGS(args, mutable_data);
+	if (!args.pattern || strcmp(args.pattern, "*") == 0) {
+		args.pattern = "";
+	}
+
+	for (i = 0; i < pvt->initreq.headers; i++) {
+		const char *header = REQ_OFFSET_TO_STR(&pvt->initreq, header[i]);
+		if (ast_begins_with(header, args.pattern)) {
+			int hdrlen = strcspn(header, " \t:,");  /* Comma will break our logic, and illegal per RFC. */
+			const char *term = ast_skip_blanks(header + hdrlen);
+			if (hdrlen > 0 && *term == ':') {  /* Header is malformed otherwise! */
+				const char *s = NULL;
+
+				/* Return short headers in full form always. */
+				if (hdrlen == 1) {
+					char short_hdr[2] = { header[0], '\0' };
+					s = find_full_alias(short_hdr, NULL);
+				}
+				if (s) {
+					/* Short header was found and expanded. */
+					ast_str_set(&token, -1, "%s,", s);
+				} else {
+					/* Return the header as is, whether 1-character or not. */
+					ast_str_set(&token, -1, "%.*s,", hdrlen, header);
+				}
+
+				/* Has the same header been already added? */
+				s = ast_str_buffer(*buf);
+				while ((s = strstr(s, ast_str_buffer(token))) != NULL) {
+					/* Found suffix, but is it the full token? */
+					if (s == ast_str_buffer(*buf) || s[-1] == ',')
+						break;
+					/* Only suffix matched, go on with the search after the comma. */
+					s += hdrlen + 1;
+				}
+
+				/* s is null iff not broken from the loop, hence header not yet added. */
+				if (s == NULL) {
+					ast_str_append(buf, maxlen, "%s", ast_str_buffer(token));
+				}
+			}
+		}
+	}
+
+	ast_str_truncate(*buf, -1);  /* Trim the last comma. Safe if empty. */
+
+	ast_channel_unlock(chan);
+	return 0;
+}
+
+static struct ast_custom_function sip_headers_function = {
+	.name = "SIP_HEADERS",
+	.read2 = func_headers_read2,
+};
+
+
 /*! \brief  Dial plan function to check if domain is local */
 static int func_check_sipdomain(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
 {
@@ -23767,7 +24004,7 @@ static void handle_response_invite(struc
 			if (p->invitestate != INV_CANCELLED) {
 				p->invitestate = INV_EARLY_MEDIA;
 			}
-			res = process_sdp(p, req, SDP_T38_NONE);
+			res = process_sdp(p, req, SDP_T38_NONE, FALSE);
 			if (!req->ignore && p->owner) {
 				/* Queue a progress frame only if we have SDP in 180 or 182 */
 				ast_queue_control(p->owner, AST_CONTROL_PROGRESS);
@@ -23853,7 +24090,7 @@ static void handle_response_invite(struc
 			if (p->invitestate != INV_CANCELLED) {
 				p->invitestate = INV_EARLY_MEDIA;
 			}
-			res = process_sdp(p, req, SDP_T38_NONE);
+			res = process_sdp(p, req, SDP_T38_NONE, FALSE);
 			if (!req->ignore && p->owner) {
 				/* Queue a progress frame */
 				ast_queue_control(p->owner, AST_CONTROL_PROGRESS);
@@ -23879,7 +24116,8 @@ static void handle_response_invite(struc
 		}
 		p->authtries = 0;
 		if (find_sdp(req)) {
-			if ((res = process_sdp(p, req, SDP_T38_ACCEPT)) && !req->ignore) {
+			res = process_sdp(p, req, SDP_T38_ACCEPT, FALSE);
+			if (res && !req->ignore) {
 				if (!reinvite) {
 					/* This 200 OK's SDP is not acceptable, so we need to ack, then hangup */
 					/* For re-invites, we try to recover */
@@ -24822,27 +25060,13 @@ static void handle_response(struct sip_p
 		msg = "";
 
 	sipmethod = find_sip_method(msg);
-
 	owner = p->owner;
 	if (owner) {
-		const char *rp = NULL, *rh = NULL;
-
 		ast_channel_hangupcause_set(owner, 0);
-		if (ast_test_flag(&p->flags[1], SIP_PAGE2_Q850_REASON) && (rh = sip_get_header(req, "Reason"))) {
-			rh = ast_skip_blanks(rh);
-			if (!strncasecmp(rh, "Q.850", 5)) {
-				int cause = ast_channel_hangupcause(owner);
-				rp = strstr(rh, "cause=");
-				if (rp && sscanf(rp + 6, "%30d", &cause) == 1) {
-					ast_channel_hangupcause_set(owner, cause & 0x7f);
-					if (req->debug)
-						ast_verbose("Using Reason header for cause code: %d\n", ast_channel_hangupcause(owner));
-				}
-			}
-		}
-
-		if (!ast_channel_hangupcause(owner))
+		if (use_reason_header(p, req)) {
+			/* Use the SIP cause */
 			ast_channel_hangupcause_set(owner, hangup_sip2cause(resp));
+		}
 	}
 
 	/* Acknowledge whatever it is destined for */
@@ -25168,7 +25392,7 @@ static void handle_response(struct sip_p
 						sip_cancel_destroy(p);
 					}
 					if (find_sdp(req))
-						process_sdp(p, req, SDP_T38_NONE);
+						process_sdp(p, req, SDP_T38_NONE, FALSE);
 					if (p->owner) {
 						/* Queue a progress frame */
 						ast_queue_control(p->owner, AST_CONTROL_PROGRESS);
@@ -25684,7 +25908,13 @@ static int handle_invite_replaces(struct
 		}
 		ao2_ref(bridge, -1);
 	} else {
-		ast_channel_move(replaces_chan, c);
+		int pickedup;
+		ast_channel_lock(replaces_chan);
+		pickedup = ast_can_pickup(replaces_chan) && !ast_do_pickup(c, replaces_chan);
+		ast_channel_unlock(replaces_chan);
+		if (!pickedup) {
+			ast_channel_move(replaces_chan, c);
+		}
 		ast_hangup(c);
 	}
 	ast_channel_unref(c);
@@ -26314,7 +26544,7 @@ static int handle_request_invite(struct
 			}
 			/* Handle SDP here if we already have an owner */
 			if (find_sdp(req)) {
-				if (process_sdp(p, req, SDP_T38_INITIATE)) {
+				if (process_sdp(p, req, SDP_T38_INITIATE, TRUE)) {
 					if (!ast_strlen_zero(sip_get_header(req, "Content-Encoding"))) {
 						/* Asterisk does not yet support any Content-Encoding methods.  Always
 						 * attempt to process the sdp, but return a 415 if a Content-Encoding header
@@ -26379,7 +26609,7 @@ static int handle_request_invite(struct
 
 		/* We have a successful authentication, process the SDP portion if there is one */
 		if (find_sdp(req)) {
-			if (process_sdp(p, req, SDP_T38_INITIATE)) {
+			if (process_sdp(p, req, SDP_T38_INITIATE, TRUE)) {
 				/* Asterisk does not yet support any Content-Encoding methods.  Always
 				 * attempt to process the sdp, but return a 415 if a Content-Encoding header
 				 * was present after processing fails. */
@@ -27197,6 +27427,8 @@ static int handle_request_cancel(struct
 		return 0;
 	}
 
+	use_reason_header(p, req);
+
 	/* At this point, we could have cancelled the invite at the same time
 	   as the other side sends a CANCEL. Our final reply with error code
 	   might not have been received by the other side before the CANCEL
@@ -27213,7 +27445,7 @@ static int handle_request_cancel(struct
 
 	stop_media_flows(p); /* Immediately stop RTP, VRTP and UDPTL as applicable */
 	if (p->owner) {
-		sip_queue_hangup_cause(p, 0);
+		sip_queue_hangup_cause(p, ast_channel_hangupcause(p->owner));
 	} else {
 		sip_scheddestroy(p, DEFAULT_TRANS_TIMEOUT);
 	}
@@ -27389,6 +27621,7 @@ static int handle_request_bye(struct sip
 		stop_session_timer(p); /* Stop Session-Timer */
 	}
 
+	use_reason_header(p, req);
 	if (!ast_strlen_zero(sip_get_header(req, "Also"))) {
 		ast_log(LOG_NOTICE, "Client '%s' using deprecated BYE/Also transfer method.  Ask vendor to support REFER instead\n",
 			ast_sockaddr_stringify(&p->recv));
@@ -27429,7 +27662,7 @@ static int handle_request_bye(struct sip
 				ast_queue_hangup_with_cause(p->owner, AST_CAUSE_PROTOCOL_ERROR);
 		}
 	} else if (p->owner) {
-		sip_queue_hangup_cause(p, 0);
+		sip_queue_hangup_cause(p, ast_channel_hangupcause(p->owner));
 		sip_scheddestroy_final(p, DEFAULT_TRANS_TIMEOUT);
 		ast_debug(3, "Received bye, issuing owner hangup\n");
 	} else {
@@ -27534,7 +27767,7 @@ static int sip_msg_send(const struct ast
 	struct ast_msg_var_iterator *iter;
 	struct sip_peer *peer_ptr;
 
-	if (!(pvt = sip_alloc(NULL, NULL, 0, SIP_MESSAGE, NULL, NULL))) {
+	if (!(pvt = sip_alloc(NULL, NULL, 0, SIP_MESSAGE, NULL, 0))) {
 		return -1;
 	}
 
@@ -28157,6 +28390,9 @@ static void add_peer_mwi_subs(struct sip
 		mailbox_specific_topic = ast_mwi_topic(mailbox->id);
 		if (mailbox_specific_topic) {
 			mailbox->event_sub = stasis_subscribe_pool(mailbox_specific_topic, mwi_event_cb, peer);
+			stasis_subscription_accept_message_type(mailbox->event_sub, ast_mwi_state_type());
+			stasis_subscription_accept_message_type(mailbox->event_sub, stasis_subscription_change_type());
+			stasis_subscription_set_filter(mailbox->event_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 	}
 }
@@ -28987,7 +29223,7 @@ static int handle_incoming(struct sip_pv
 			p->pendinginvite = 0;
 			acked = __sip_ack(p, seqno, 1 /* response */, 0);
 			if (p->owner && find_sdp(req)) {
-				if (process_sdp(p, req, SDP_T38_NONE)) {
+				if (process_sdp(p, req, SDP_T38_NONE, FALSE)) {
 					return -1;
 				}
 				if (ast_test_flag(&p->flags[0], SIP_DIRECT_MEDIA)) {
@@ -29241,9 +29477,8 @@ static int sip_prepare_socket(struct sip
 		return s->fd;
 	}
 	if ((s->type & (AST_TRANSPORT_TCP | AST_TRANSPORT_TLS)) &&
-			(s->tcptls_session) &&
-			(s->tcptls_session->fd != -1)) {
-		return s->tcptls_session->fd;
+			s->tcptls_session && s->tcptls_session->stream) {
+		return ast_iostream_get_fd(s->tcptls_session->stream);
 	}
 	if ((s->type & (AST_TRANSPORT_WS | AST_TRANSPORT_WSS))) {
 		return s->ws_session ? ast_websocket_fd(s->ws_session) : -1;
@@ -29273,7 +29508,7 @@ static int sip_prepare_socket(struct sip
 	/* 1.  check for existing threads */
 	ast_sockaddr_copy(&sa_tmp, sip_real_dst(p));
 	if ((tcptls_session = sip_tcp_locate(&sa_tmp))) {
-		s->fd = tcptls_session->fd;
+		s->fd = ast_iostream_get_fd(tcptls_session->stream);
 		if (s->tcptls_session) {
 			ao2_ref(s->tcptls_session, -1);
 			s->tcptls_session = NULL;
@@ -29331,7 +29566,7 @@ static int sip_prepare_socket(struct sip
 		goto create_tcptls_session_fail;
 	}
 
-	s->fd = s->tcptls_session->fd;
+	s->fd = ast_iostream_get_fd(s->tcptls_session->stream);
 
 	/* client connections need to have the sip_threadinfo object created before
 	 * the thread is detached.  This ensures the alert_pipe is up before it will
@@ -29456,7 +29691,7 @@ static int sip_send_mwi_to_peer(struct s
 	} else {
 		ao2_unlock(peer);
 		/* Build temporary dialog for this message */
-		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, NULL))) {
+		if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, 0))) {
 			update_peer_lastmsgssent(peer, -1, 0);
 			return -1;
 		}
@@ -30134,8 +30369,7 @@ static int sip_send_keepalive(const void
 	if ((peer->socket.fd != -1) && (peer->socket.type == AST_TRANSPORT_UDP)) {
 		res = ast_sendto(peer->socket.fd, keepalive, count, 0, &peer->addr);
 	} else if ((peer->socket.type & (AST_TRANSPORT_TCP | AST_TRANSPORT_TLS)) &&
-		   (peer->socket.tcptls_session) &&
-		   (peer->socket.tcptls_session->fd != -1)) {
+		   peer->socket.tcptls_session) {
 		res = sip_tcptls_write(peer->socket.tcptls_session, keepalive, count);
 	} else if (peer->socket.type == AST_TRANSPORT_UDP) {
 		res = ast_sendto(sipsock, keepalive, count, 0, &peer->addr);
@@ -30253,7 +30487,7 @@ static int sip_poke_peer(struct sip_peer
 		peer->call = dialog_unref(peer->call, "unref dialog peer->call");
 		/* peer->call = sip_destroy(peer->call); */
 	}
-	if (!(p = sip_alloc(NULL, NULL, 0, SIP_OPTIONS, NULL, NULL))) {
+	if (!(p = sip_alloc(NULL, NULL, 0, SIP_OPTIONS, NULL, 0))) {
 		return -1;
 	}
 	peer->call = dialog_ref(p, "copy sip alloc from p to peer->call");
@@ -30434,7 +30668,8 @@ static int sip_devicestate(const char *d
  *	or	SIP/devicename/extension/IPorHost
  *	or	SIP/username@domain//IPorHost
  *	and there is an optional [!dnid] argument you can append to alter the
- *	To: header.
+ *	To: header. And after that, a [![fromuser][@fromdomain]] argument.
+ *	Leave those blank to use the defaults.
  * \endverbatim
  */
 static struct ast_channel *sip_request_call(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *dest, int *cause)
@@ -30453,7 +30688,7 @@ static struct ast_channel *sip_request_c
 	char dialstring[256];
 	char *remote_address;
 	enum ast_transport transport = 0;
-	struct ast_callid *callid;
+	ast_callid callid;
 	AST_DECLARE_APP_ARGS(args,
 		AST_APP_ARG(peerorhost);
 		AST_APP_ARG(exten);
@@ -30486,9 +30721,6 @@ static struct ast_channel *sip_request_c
 	if (!(p = sip_alloc(NULL, NULL, 0, SIP_INVITE, NULL, callid))) {
 		ast_log(LOG_ERROR, "Unable to build sip pvt data for '%s' (Out of memory or socket error)\n", dest);
 		*cause = AST_CAUSE_SWITCH_CONGESTION;
-		if (callid) {
-			ast_callid_unref(callid);
-		}
 		return NULL;
 	}
 
@@ -30503,20 +30735,55 @@ static struct ast_channel *sip_request_c
 		/* sip_destroy(p); */
 		ast_log(LOG_ERROR, "Unable to build option SIP data structure - Out of memory\n");
 		*cause = AST_CAUSE_SWITCH_CONGESTION;
-		if (callid) {
-			ast_callid_unref(callid);
-		}
 		return NULL;
 	}
 
 	/* Save the destination, the SIP dial string */
 	ast_copy_string(tmp, dest, sizeof(tmp));
 
-	/* Find DNID and take it away */
+	/* Find optional DNID (SIP to-uri) and From-CLI (SIP from-uri)
+	 * and strip it from the dial string:
+	 *   [!touser[@todomain][![fromuser][@fromdomain]]]
+	 * For historical reasons, the touser@todomain is passed as dnid
+	 * while fromuser@fromdomain are split immediately. Passing a
+	 * todomain without touser will create an invalid SIP message. */
 	dnid = strchr(tmp, '!');
 	if (dnid != NULL) {
+		char *fromuser_and_domain;
+
 		*dnid++ = '\0';
-		ast_string_field_set(p, todnid, dnid);
+		if ((fromuser_and_domain = strchr(dnid, '!'))) {
+			char *forward_compat;
+			char *fromdomain;
+
+			*fromuser_and_domain++ = '\0';
+
+			/* Cut it at a trailing NUL or trailing '!' for
+			 * forward compatibility with extra arguments
+			 * in the future. */
+			if ((forward_compat = strchr(fromuser_and_domain, '!'))) {
+				/* Ignore the rest.. */
+				*forward_compat = '\0';
+			}
+
+			if ((fromdomain = strchr(fromuser_and_domain, '@'))) {
+				*fromdomain++ = '\0';
+				/* Set fromdomain. */
+				if (!ast_strlen_zero(fromdomain)) {
+					ast_string_field_set(p, fromdomain, fromdomain);
+				}
+			}
+
+			/* Set fromuser. */
+			if (!ast_strlen_zero(fromuser_and_domain)) {
+				ast_string_field_set(p, fromuser, fromuser_and_domain);
+			}
+		}
+
+		/* Set DNID (touser/todomain). */
+		if (!ast_strlen_zero(dnid)) {
+			ast_string_field_set(p, todnid, dnid);
+		}
 	}
 
 	/* If stripping the DNID left us with nothing, bail out */
@@ -30524,9 +30791,6 @@ static struct ast_channel *sip_request_c
 		dialog_unlink_all(p);
 		dialog_unref(p, "unref dialog p from bad destination");
 		*cause = AST_CAUSE_DESTINATION_OUT_OF_ORDER;
-		if (callid) {
-			ast_callid_unref(callid);
-		}
 		return NULL;
 	}
 
@@ -30598,9 +30862,6 @@ static struct ast_channel *sip_request_c
 		dialog_unlink_all(p);
 		dialog_unref(p, "unref dialog p UNREGISTERED");
 		/* sip_destroy(p); */
-		if (callid) {
-			ast_callid_unref(callid);
-		}
 		return NULL;
 	}
 	if (ast_strlen_zero(p->peername) && ext)
@@ -30662,9 +30923,6 @@ static struct ast_channel *sip_request_c
 	sip_pvt_lock(p);
 
 	tmpc = sip_new(p, AST_STATE_DOWN, host, assignedids, requestor, callid);	/* Place the call */
-	if (callid) {
-		callid = ast_callid_unref(callid);
-	}
 
 	sip_pvt_unlock(p);
 	if (!tmpc) {
@@ -31304,8 +31562,6 @@ static struct sip_peer *build_peer(const
 			firstpass = 0;
 		} else {
 			ast_format_cap_remove_by_type(peer->caps, AST_MEDIA_TYPE_UNKNOWN);
-			ast_rtp_dtls_cfg_free(&peer->dtls_cfg);
-			memset(&peer->dtls_cfg, 0, sizeof(peer->dtls_cfg));
 		}
 	} else {
 		if (!(peer = ao2_t_alloc(sizeof(*peer), sip_destroy_peer_fn, "allocate a peer struct"))) {
@@ -31387,6 +31643,11 @@ static struct sip_peer *build_peer(const
 	peer->named_callgroups = ast_unref_namedgroups(peer->named_callgroups);
 	peer->named_pickupgroups = ast_unref_namedgroups(peer->named_pickupgroups);
 
+	/* Set the default DTLS settings from default_tls_cfg */
+	ast_rtp_dtls_cfg_free(&peer->dtls_cfg);
+	ast_rtp_dtls_cfg_copy(&default_dtls_cfg, &peer->dtls_cfg);
+	peer->dtls_cfg.enabled = FALSE;
+
 	for (; v || ((v = alt) && !(alt=NULL)); v = v->next) {
 		if (!devstate_only) {
 			if (handle_common_options(&peerflags[0], &mask[0], v)) {
@@ -31530,7 +31791,9 @@ static struct sip_peer *build_peer(const
 					ast_append_acl(v->name, v->value, &peer->acl, &ha_error, &acl_change_subscription_needed);
 				}
 				if (ha_error) {
-					ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s\n", v->lineno, v->value);
+					ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s. Deleting peer\n", v->lineno, v->value);
+					sip_unref_peer(peer, "Removing peer due to bad ACL configuration");
+					return NULL;
 				}
 			} else if (!strcasecmp(v->name, "contactpermit") || !strcasecmp(v->name, "contactdeny") || !strcasecmp(v->name, "contactacl")) {
 				int ha_error = 0;
@@ -31538,13 +31801,17 @@ static struct sip_peer *build_peer(const
 					ast_append_acl(v->name + 7, v->value, &peer->contactacl, &ha_error, &acl_change_subscription_needed);
 				}
 				if (ha_error) {
-					ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s\n", v->lineno, v->value);
+					ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s. Deleting peer\n", v->lineno, v->value);
+					sip_unref_peer(peer, "Removing peer due to bad contact ACL configuration");
+					return NULL;
 				}
 			} else if (!strcasecmp(v->name, "directmediapermit") || !strcasecmp(v->name, "directmediadeny") || !strcasecmp(v->name, "directmediaacl")) {
 				int ha_error = 0;
 				ast_append_acl(v->name + 11, v->value, &peer->directmediaacl, &ha_error, &acl_change_subscription_needed);
 				if (ha_error) {
-					ast_log(LOG_ERROR, "Bad directmedia ACL entry in configuration line %d : %s\n", v->lineno, v->value);
+					ast_log(LOG_ERROR, "Bad directmedia ACL entry in configuration line %d : %s. Deleting peer\n", v->lineno, v->value);
+					sip_unref_peer(peer, "Removing peer due to bad direct media ACL configuration");
+					return NULL;
 				}
 			} else if (!strcasecmp(v->name, "port")) {
 				peer->portinuri = 1;
@@ -31758,6 +32025,14 @@ static struct sip_peer *build_peer(const
 			}
 		}
 
+		/* Validate DTLS configuration */
+		if (ast_rtp_dtls_cfg_validate(&peer->dtls_cfg)) {
+			sip_unref_peer(peer, "Removing peer due to bad DTLS configuration");
+			return NULL;
+		}
+
+		/* SRB */
+
 		/* Apply the encryption tag length to the DTLS configuration, in case DTLS is in use */
 		peer->dtls_cfg.suite = (ast_test_flag(&peer->flags[2], SIP_PAGE3_SRTP_TAG_32) ? AST_AES_CM_128_HMAC_SHA1_32 : AST_AES_CM_128_HMAC_SHA1_80);
 
@@ -32011,7 +32286,7 @@ static struct sip_peer *build_peer(const
 	olddirectmediaacl = ast_free_acl_list(olddirectmediaacl);
 	if (!ast_strlen_zero(peer->callback)) { /* build string from peer info */
 		char *reg_string;
-		if (ast_asprintf(&reg_string, "%s?%s:%s@%s/%s", peer->name, peer->username, !ast_strlen_zero(peer->remotesecret) ? peer->remotesecret : peer->secret, peer->tohost, peer->callback) >= 0) {
+		if (ast_asprintf(&reg_string, "%s?%s:%s:%s@%s/%s", peer->name, S_OR(peer->fromuser, peer->username), S_OR(peer->remotesecret, peer->secret), peer->username, peer->tohost, peer->callback) >= 0) {
 			sip_register(reg_string, 0); /* XXX TODO: count in registry_count */
 			ast_free(reg_string);
 		}
@@ -32188,6 +32463,7 @@ static int reload_config(enum channelrel
 	sip_cfg.contact_acl = ast_free_acl_list(sip_cfg.contact_acl);
 
 	default_tls_cfg.enabled = FALSE;		/* Default: Disable TLS */
+	default_dtls_cfg.enabled = FALSE;		/* Default: Disable DTLS too */
 
 	if (reason != CHANNEL_MODULE_LOAD) {
 		ast_debug(4, "--------------- SIP reload started\n");
@@ -32206,19 +32482,26 @@ static int reload_config(enum channelrel
 		ao2_t_callback(peers, OBJ_NODATA, peer_markall_func, NULL, "callback to mark all peers");
 	}
 
-	/* Reset certificate handling for TLS sessions */
+	/* Reset certificate handling for TLS and DTLS sessions */
 	if (reason != CHANNEL_MODULE_LOAD) {
 		ast_free(default_tls_cfg.certfile);
 		ast_free(default_tls_cfg.pvtfile);
 		ast_free(default_tls_cfg.cipher);
 		ast_free(default_tls_cfg.cafile);
 		ast_free(default_tls_cfg.capath);
+		ast_rtp_dtls_cfg_free(&default_dtls_cfg);
 	}
 	default_tls_cfg.certfile = ast_strdup(AST_CERTFILE); /*XXX Not sure if this is useful */
 	default_tls_cfg.pvtfile = ast_strdup("");
 	default_tls_cfg.cipher = ast_strdup("");
 	default_tls_cfg.cafile = ast_strdup("");
 	default_tls_cfg.capath = ast_strdup("");
+	/* Using the same idea fro DTLS as the code block above for TLS */
+	default_dtls_cfg.certfile = ast_strdup("");
+	default_dtls_cfg.pvtfile = ast_strdup("");
+	default_dtls_cfg.cipher = ast_strdup("");
+	default_dtls_cfg.cafile = ast_strdup("");
+	default_dtls_cfg.capath = ast_strdup("");
 
 	/* Initialize copy of current sip_cfg.regcontext for later use in removing stale contexts */
 	ast_copy_string(oldcontexts, sip_cfg.regcontext, sizeof(oldcontexts));
@@ -32240,6 +32523,7 @@ static int reload_config(enum channelrel
 	memset(&localaddr, 0, sizeof(localaddr));
 	memset(&externaddr, 0, sizeof(externaddr));
 	memset(&media_address, 0, sizeof(media_address));
+	memset(&rtpbindaddr, 0, sizeof(rtpbindaddr));
 	memset(&sip_cfg.outboundproxy, 0, sizeof(struct sip_proxy));
 	sip_cfg.outboundproxy.force = FALSE;		/*!< Don't force proxy usage, use route: headers */
 	default_transports = AST_TRANSPORT_UDP;
@@ -32390,6 +32674,9 @@ static int reload_config(enum channelrel
 			continue;
 		}
 
+		/* Load default dtls configuration */
+		ast_rtp_dtls_cfg_parse(&default_dtls_cfg, v->name, v->value);
+
 		/* handle tls conf, don't allow setting of tlsverifyclient as it isn't supported by chan_sip */
 		if (!strcasecmp(v->name, "tlsverifyclient")) {
 			ast_log(LOG_WARNING, "Ignoring unsupported option 'tlsverifyclient'\n");
@@ -32496,7 +32783,8 @@ static int reload_config(enum channelrel
 			int ha_error = 0;
 			ast_append_acl(v->name + 7, v->value, &sip_cfg.contact_acl, &ha_error, &acl_change_subscription_needed);
 			if (ha_error) {
-				ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s\n", v->lineno, v->value);
+				ast_log(LOG_ERROR, "Bad ACL entry in configuration line %d : %s. Failing to load chan_sip.so\n", v->lineno, v->value);
+				return -1;
 			}
 		} else if (!strcasecmp(v->name, "rtautoclear")) {
 			int i = atoi(v->value);
@@ -32536,7 +32824,11 @@ static int reload_config(enum channelrel
 		} else if (!strcasecmp(v->name, "directrtpsetup")) {
 			sip_cfg.directrtpsetup = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "notifyringing")) {
-			sip_cfg.notifyringing = ast_true(v->value);
+			if (!strcasecmp(v->value, "notinuse")) {
+				sip_cfg.notifyringing = NOTIFYRINGING_NOTINUSE;
+			} else {
+				sip_cfg.notifyringing = ast_true(v->value) ? NOTIFYRINGING_ENABLED : NOTIFYRINGING_DISABLED;
+			}
 		} else if (!strcasecmp(v->name, "notifyhold")) {
 			sip_cfg.notifyhold = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "notifycid")) {
@@ -32705,6 +32997,10 @@ static int reload_config(enum channelrel
 		} else if (!strcasecmp(v->name, "media_address")) {
 			if (ast_parse_arg(v->value, PARSE_ADDR, &media_address))
 				ast_log(LOG_WARNING, "Invalid address for media_address keyword: %s\n", v->value);
+		} else if (!strcasecmp(v->name, "rtpbindaddr")) {
+			if (ast_parse_arg(v->value, PARSE_ADDR, &rtpbindaddr)) {
+				ast_log(LOG_WARNING, "Invalid address for rtpbindaddr keyword: %s\n", v->value);
+			}
 		} else if (!strcasecmp(v->name, "externaddr") || !strcasecmp(v->name, "externip")) {
 			if (ast_parse_arg(v->value, PARSE_ADDR, &externaddr)) {
 				ast_log(LOG_WARNING,
@@ -32936,6 +33232,11 @@ static int reload_config(enum channelrel
 		}
 	}
 
+	/* Validate DTLS configuration */
+	if (ast_rtp_dtls_cfg_validate(&default_dtls_cfg)) {
+		return -1;
+	}
+
 	/* Override global defaults if setting found in general section */
 	ast_copy_flags(&global_flags[0], &setflags[0], mask[0].flags);
 	ast_copy_flags(&global_flags[1], &setflags[1], mask[1].flags);
@@ -33922,7 +34223,7 @@ static void sip_send_all_mwi_subscriptio
 }
 
 static int process_crypto(struct sip_pvt *p, struct ast_rtp_instance *rtp, struct ast_sdp_srtp **srtp,
-		const char *a, int secure_transport)
+		const char *a)
 {
 	struct ast_rtp_engine_dtls *dtls;
 
@@ -33938,23 +34239,6 @@ static int process_crypto(struct sip_pvt
 	/* skip "crypto:" */
 	a += strlen("crypto:");
 
-	if (!secure_transport) {
-		/* > The Secure Real-time Transport Protocol (SRTP)
-		 * > [RFC3711] provides security services for RTP media
-		 * > and is signaled by use of secure RTP transport (e.g.,
-		 * > "RTP/SAVP" or "RTP/SAVPF") in an SDP media (m=) line.
-		 * > ...
-		 * > The "crypto" attribute MUST only appear at the SDP
-		 * > media level (not at the session level).
-		 *
-		 * Ergo, we can trust RTP/(S)AVP to be read from the m=
-		 * line before we get here. If it was RTP/AVP, then this
-		 * is SNOM-specific optional SRTP. Ignore it.
-		 */
-		ast_log(LOG_WARNING, "Ignoring crypto attribute in SDP because RTP transport is insecure\n");
-		return FALSE;
-	}
-
 	if (!*srtp) {
 		if (ast_test_flag(&p->flags[0], SIP_OUTGOING)) {
 			ast_log(LOG_WARNING, "Ignoring unexpected crypto attribute in SDP answer\n");
@@ -34064,30 +34348,6 @@ static int reload(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-/*! \brief  Return the first entry from ast_sockaddr_resolve filtered by address family
- *
- * \warning Using this function probably means you have a faulty design.
- */
-static int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
-				      const char* name, int flag, int family)
-{
-	struct ast_sockaddr *addrs;
-	int addrs_cnt;
-
-	addrs_cnt = ast_sockaddr_resolve(&addrs, name, flag, family);
-	if (addrs_cnt <= 0) {
-		return 1;
-	}
-	if (addrs_cnt > 1) {
-		ast_debug(1, "Multiple addresses, using the first one only\n");
-	}
-
-	ast_sockaddr_copy(addr, &addrs[0]);
-
-	ast_free(addrs);
-	return 0;
-}
-
 /*! \brief  Return the first entry from ast_sockaddr_resolve filtered by family of binddaddr
  *
  * \warning Using this function probably means you have a faulty design.
@@ -34537,75 +34797,6 @@ AST_TEST_DEFINE(test_sip_mwi_subscribe_p
 	return res;
 }
 
-AST_TEST_DEFINE(test_sip_peers_get)
-{
-	struct sip_peer *peer;
-	struct ast_data *node;
-	struct ast_data_query query = {
-		.path = "/asterisk/channel/sip/peers",
-		.search = "peers/peer/name=test_peer_data_provider"
-	};
-
-	switch (cmd) {
-		case TEST_INIT:
-			info->name = "sip_peers_get_data_test";
-			info->category = "/main/data/sip/peers/";
-			info->summary = "SIP peers data providers unit test";
-			info->description =
-				"Tests whether the SIP peers data provider implementation works as expected.";
-			return AST_TEST_NOT_RUN;
-		case TEST_EXECUTE:
-			break;
-	}
-
-	/* Create the peer that we will retrieve. */
-	peer = build_peer("test_peer_data_provider", NULL, NULL, 0, 0);
-	if (!peer) {
-		return AST_TEST_FAIL;
-	}
-	peer->type = SIP_TYPE_USER;
-	peer->call_limit = 10;
-	ao2_link(peers, peer);
-
-	/* retrieve the chan_sip/peers tree and check the created peer. */
-	node = ast_data_get(&query);
-	if (!node) {
-		ao2_unlink(peers, peer);
-		ao2_ref(peer, -1);
-		return AST_TEST_FAIL;
-	}
-
-	/* compare item. */
-	if (strcmp(ast_data_retrieve_string(node, "peer/name"), "test_peer_data_provider")) {
-		ao2_unlink(peers, peer);
-		ao2_ref(peer, -1);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	if (strcmp(ast_data_retrieve_string(node, "peer/type"), "user")) {
-		ao2_unlink(peers, peer);
-		ao2_ref(peer, -1);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	if (ast_data_retrieve_int(node, "peer/call_limit") != 10) {
-		ao2_unlink(peers, peer);
-		ao2_ref(peer, -1);
-		ast_data_free(node);
-		return AST_TEST_FAIL;
-	}
-
-	/* release resources */
-	ast_data_free(node);
-
-	ao2_unlink(peers, peer);
-	ao2_ref(peer, -1);
-
-	return AST_TEST_PASS;
-}
-
 /*!
  * \brief Imitation TCP reception loop
  *
@@ -35138,170 +35329,6 @@ AST_TEST_DEFINE(get_in_brackets_const_te
 
 #endif
 
-#define DATA_EXPORT_SIP_PEER(MEMBER)				\
-	MEMBER(sip_peer, name, AST_DATA_STRING)			\
-	MEMBER(sip_peer, secret, AST_DATA_PASSWORD)		\
-	MEMBER(sip_peer, md5secret, AST_DATA_PASSWORD)		\
-	MEMBER(sip_peer, remotesecret, AST_DATA_PASSWORD)	\
-	MEMBER(sip_peer, context, AST_DATA_STRING)		\
-	MEMBER(sip_peer, subscribecontext, AST_DATA_STRING)	\
-	MEMBER(sip_peer, username, AST_DATA_STRING)		\
-	MEMBER(sip_peer, accountcode, AST_DATA_STRING)		\
-	MEMBER(sip_peer, tohost, AST_DATA_STRING)		\
-	MEMBER(sip_peer, regexten, AST_DATA_STRING)		\
-	MEMBER(sip_peer, fromuser, AST_DATA_STRING)		\
-	MEMBER(sip_peer, fromdomain, AST_DATA_STRING)		\
-	MEMBER(sip_peer, fullcontact, AST_DATA_STRING)		\
-	MEMBER(sip_peer, cid_num, AST_DATA_STRING)		\
-	MEMBER(sip_peer, cid_name, AST_DATA_STRING)		\
-	MEMBER(sip_peer, vmexten, AST_DATA_STRING)		\
-	MEMBER(sip_peer, language, AST_DATA_STRING)		\
-	MEMBER(sip_peer, mohinterpret, AST_DATA_STRING)		\
-	MEMBER(sip_peer, mohsuggest, AST_DATA_STRING)		\
-	MEMBER(sip_peer, parkinglot, AST_DATA_STRING)		\
-	MEMBER(sip_peer, useragent, AST_DATA_STRING)		\
-	MEMBER(sip_peer, mwi_from, AST_DATA_STRING)		\
-	MEMBER(sip_peer, engine, AST_DATA_STRING)		\
-	MEMBER(sip_peer, unsolicited_mailbox, AST_DATA_STRING)	\
-	MEMBER(sip_peer, is_realtime, AST_DATA_BOOLEAN)		\
-	MEMBER(sip_peer, host_dynamic, AST_DATA_BOOLEAN)	\
-	MEMBER(sip_peer, autoframing, AST_DATA_BOOLEAN)		\
-	MEMBER(sip_peer, inuse, AST_DATA_INTEGER)		\
-	MEMBER(sip_peer, ringing, AST_DATA_INTEGER)		\
-	MEMBER(sip_peer, onhold, AST_DATA_INTEGER)		\
-	MEMBER(sip_peer, call_limit, AST_DATA_INTEGER)		\
-	MEMBER(sip_peer, t38_maxdatagram, AST_DATA_INTEGER)	\
-	MEMBER(sip_peer, maxcallbitrate, AST_DATA_INTEGER)	\
-	MEMBER(sip_peer, rtptimeout, AST_DATA_SECONDS)		\
-	MEMBER(sip_peer, rtpholdtimeout, AST_DATA_SECONDS)	\
-	MEMBER(sip_peer, rtpkeepalive, AST_DATA_SECONDS)	\
-	MEMBER(sip_peer, lastms, AST_DATA_MILLISECONDS)		\
-	MEMBER(sip_peer, maxms, AST_DATA_MILLISECONDS)		\
-	MEMBER(sip_peer, qualifyfreq, AST_DATA_MILLISECONDS)	\
-	MEMBER(sip_peer, timer_t1, AST_DATA_MILLISECONDS)	\
-	MEMBER(sip_peer, timer_b, AST_DATA_MILLISECONDS)	\
-	MEMBER(sip_peer, description, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(sip_peer, DATA_EXPORT_SIP_PEER);
-
-static int peers_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct sip_peer *peer;
-	struct ao2_iterator i;
-	struct ast_data *data_peer, *data_peer_mailboxes = NULL, *data_peer_mailbox, *enum_node;
-	struct ast_data *data_sip_options;
-	int total_mailboxes, x;
-	struct sip_mailbox *mailbox;
-
-	i = ao2_iterator_init(peers, 0);
-	while ((peer = ao2_iterator_next(&i))) {
-		ao2_lock(peer);
-
-		data_peer = ast_data_add_node(data_root, "peer");
-		if (!data_peer) {
-			ao2_unlock(peer);
-			ao2_ref(peer, -1);
-			continue;
-		}
-
-		ast_data_add_structure(sip_peer, data_peer, peer);
-
-		/* transfer mode */
-		enum_node = ast_data_add_node(data_peer, "allowtransfer");
-		if (!enum_node) {
-			ao2_unlock(peer);
-			ao2_ref(peer, -1);
-			continue;
-		}
-		ast_data_add_str(enum_node, "text", transfermode2str(peer->allowtransfer));
-		ast_data_add_int(enum_node, "value", peer->allowtransfer);
-
-		/* transports */
-		ast_data_add_str(data_peer, "transports", get_transport_list(peer->transports));
-
-		/* peer type */
-		if ((peer->type & SIP_TYPE_USER) && (peer->type & SIP_TYPE_PEER)) {
-			ast_data_add_str(data_peer, "type", "friend");
-		} else if (peer->type & SIP_TYPE_PEER) {
-			ast_data_add_str(data_peer, "type", "peer");
-		} else if (peer->type & SIP_TYPE_USER) {
-			ast_data_add_str(data_peer, "type", "user");
-		}
-
-		/* mailboxes */
-		total_mailboxes = 0;
-		AST_LIST_TRAVERSE(&peer->mailboxes, mailbox, entry) {
-			if (!total_mailboxes) {
-				data_peer_mailboxes = ast_data_add_node(data_peer, "mailboxes");
-				if (!data_peer_mailboxes) {
-					break;
-				}
-				total_mailboxes++;
-			}
-
-			data_peer_mailbox = ast_data_add_node(data_peer_mailboxes, "mailbox");
-			if (!data_peer_mailbox) {
-				continue;
-			}
-			ast_data_add_str(data_peer_mailbox, "id", mailbox->id);
-		}
-
-		/* amaflags */
-		enum_node = ast_data_add_node(data_peer, "amaflags");
-		if (!enum_node) {
-			ao2_unlock(peer);
-			ao2_ref(peer, -1);
-			continue;
-		}
-		ast_data_add_int(enum_node, "value", peer->amaflags);
-		ast_data_add_str(enum_node, "text", ast_channel_amaflags2string(peer->amaflags));
-
-		/* sip options */
-		data_sip_options = ast_data_add_node(data_peer, "sipoptions");
-		if (!data_sip_options) {
-			ao2_unlock(peer);
-			ao2_ref(peer, -1);
-			continue;
-		}
-		for (x = 0 ; x < ARRAY_LEN(sip_options); x++) {
-			ast_data_add_bool(data_sip_options, sip_options[x].text, peer->sipoptions & sip_options[x].id);
-		}
-
-		/* callingpres */
-		enum_node = ast_data_add_node(data_peer, "callingpres");
-		if (!enum_node) {
-			ao2_unlock(peer);
-			ao2_ref(peer, -1);
-			continue;
-		}
-		ast_data_add_int(enum_node, "value", peer->callingpres);
-		ast_data_add_str(enum_node, "text", ast_describe_caller_presentation(peer->callingpres));
-
-		/* codecs */
-		ast_data_add_codecs(data_peer, "codecs", peer->caps);
-
-		if (!ast_data_search_match(search, data_peer)) {
-			ast_data_remove_node(data_root, data_peer);
-		}
-
-		ao2_unlock(peer);
-		ao2_ref(peer, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-	return 0;
-}
-
-static const struct ast_data_handler peers_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = peers_data_provider_get
-};
-
-static const struct ast_data_entry sip_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/channel/sip/peers", &peers_data_provider),
-};
-
 static const struct ast_sip_api_tech chan_sip_api_provider = {
 	.version = AST_SIP_API_VERSION,
 	.name = "chan_sip",
@@ -35347,12 +35374,18 @@ static int load_module(void)
 
 	/* the fact that ao2_containers can't resize automatically is a major worry! */
 	/* if the number of objects gets above MAX_XXX_BUCKETS, things will slow down */
-	peers = ao2_t_container_alloc(HASH_PEER_SIZE, peer_hash_cb, peer_cmp_cb, "allocate peers");
-	peers_by_ip = ao2_t_container_alloc(HASH_PEER_SIZE, peer_iphash_cb, NULL, "allocate peers_by_ip");
-	dialogs = ao2_t_container_alloc(HASH_DIALOG_SIZE, dialog_hash_cb, dialog_cmp_cb, "allocate dialogs");
-	dialogs_needdestroy = ao2_t_container_alloc(1, NULL, NULL, "allocate dialogs_needdestroy");
-	dialogs_rtpcheck = ao2_t_container_alloc(HASH_DIALOG_SIZE, dialog_hash_cb, dialog_cmp_cb, "allocate dialogs for rtpchecks");
-	threadt = ao2_t_container_alloc(HASH_DIALOG_SIZE, threadt_hash_cb, threadt_cmp_cb, "allocate threadt table");
+	peers = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_PEER_SIZE,
+		peer_hash_cb, NULL, peer_cmp_cb, "allocate peers");
+	peers_by_ip = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_PEER_SIZE,
+		peer_iphash_cb, NULL, NULL, "allocate peers_by_ip");
+	dialogs = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_DIALOG_SIZE,
+		dialog_hash_cb, NULL, dialog_cmp_cb, "allocate dialogs");
+	dialogs_needdestroy = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 1,
+		NULL, NULL, NULL, "allocate dialogs_needdestroy");
+	dialogs_rtpcheck = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_DIALOG_SIZE,
+		dialog_hash_cb, NULL, dialog_cmp_cb, "allocate dialogs for rtpchecks");
+	threadt = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_DIALOG_SIZE,
+		threadt_hash_cb, NULL, threadt_cmp_cb, "allocate threadt table");
 	if (!peers || !peers_by_ip || !dialogs || !dialogs_needdestroy || !dialogs_rtpcheck
 		|| !threadt) {
 		ast_log(LOG_ERROR, "Unable to create primary SIP container(s)\n");
@@ -35366,7 +35399,8 @@ static int load_module(void)
 	}
 	ast_format_cap_append_by_type(sip_tech.capabilities, AST_MEDIA_TYPE_AUDIO);
 
-	registry_list = ao2_t_container_alloc(HASH_REGISTRY_SIZE, registry_hash_cb, registry_cmp_cb, "allocate registry_list");
+	registry_list = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, HASH_REGISTRY_SIZE,
+		registry_hash_cb, NULL, registry_cmp_cb, "allocate registry_list");
 	subscription_mwi_list = ao2_t_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX,
 		AO2_CONTAINER_ALLOC_OPT_INSERT_BEGIN, NULL, NULL, "allocate subscription_mwi_list");
 
@@ -35422,15 +35456,11 @@ static int load_module(void)
 	}
 
 #ifdef TEST_FRAMEWORK
-	AST_TEST_REGISTER(test_sip_peers_get);
 	AST_TEST_REGISTER(test_sip_mwi_subscribe_parse);
 	AST_TEST_REGISTER(test_tcp_message_fragmentation);
 	AST_TEST_REGISTER(get_in_brackets_const_test);
 #endif
 
-	/* Register AstData providers */
-	ast_data_register_multiple(sip_data_providers, ARRAY_LEN(sip_data_providers));
-
 	/* Register all CLI functions for SIP */
 	ast_cli_register_multiple(cli_sip, ARRAY_LEN(cli_sip));
 
@@ -35447,6 +35477,7 @@ static int load_module(void)
 
 	/* Register dialplan functions */
 	ast_custom_function_register(&sip_header_function);
+	ast_custom_function_register(&sip_headers_function);
 	ast_custom_function_register(&sippeer_function);
 	ast_custom_function_register(&checksipdomain_function);
 
@@ -35487,7 +35518,9 @@ static int load_module(void)
 		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
-	if (!(sip_monitor_instances = ao2_container_alloc(37, sip_monitor_instance_hash_fn, sip_monitor_instance_cmp_fn))) {
+	sip_monitor_instances = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		sip_monitor_instance_hash_fn, NULL, sip_monitor_instance_cmp_fn);
+	if (!sip_monitor_instances) {
 		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -35547,6 +35580,7 @@ static int unload_module(void)
 
 	/* Unregister dial plan functions */
 	ast_custom_function_unregister(&sippeer_function);
+	ast_custom_function_unregister(&sip_headers_function);
 	ast_custom_function_unregister(&sip_header_function);
 	ast_custom_function_unregister(&checksipdomain_function);
 
@@ -35557,14 +35591,10 @@ static int unload_module(void)
 #ifdef TEST_FRAMEWORK
 	ast_unregister_application(app_sipsendcustominfo);
 
-	AST_TEST_UNREGISTER(test_sip_peers_get);
 	AST_TEST_UNREGISTER(test_sip_mwi_subscribe_parse);
 	AST_TEST_UNREGISTER(test_tcp_message_fragmentation);
 	AST_TEST_UNREGISTER(get_in_brackets_const_test);
 #endif
-	/* Unregister all the AstData providers */
-	ast_data_unregister(NULL);
-
 	/* Unregister CLI commands */
 	ast_cli_unregister_multiple(cli_sip, ARRAY_LEN(cli_sip));
 
@@ -35687,6 +35717,8 @@ static int unload_module(void)
 	ast_free(default_tls_cfg.cafile);
 	ast_free(default_tls_cfg.capath);
 
+	ast_rtp_dtls_cfg_free(&default_dtls_cfg);
+
 	ao2_cleanup(registry_list);
 	ao2_cleanup(subscription_mwi_list);
 
@@ -35735,10 +35767,11 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Session Initiation Protocol (SIP)",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-		.nonoptreq = "res_crypto,res_http_websocket",
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "ccss,dnsmgr,udptl",
+	.optional_modules = "res_crypto,res_http_websocket",
+);
diff -pruN 1:13.23.1~dfsg-2/channels/chan_skinny.c 1:16.1.1~dfsg-1/channels/chan_skinny.c
--- 1:13.23.1~dfsg-2/channels/chan_skinny.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_skinny.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
@@ -49,7 +47,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <fcntl.h>
 #include <netdb.h>
 #include <arpa/inet.h>
-#include <sys/signal.h>
 #include <signal.h>
 #include <ctype.h>
 
@@ -1671,7 +1668,7 @@ static struct ast_channel_tech skinny_te
 	.send_digit_end = skinny_senddigit_end,
 };
 
-static int skinny_extensionstate_cb(char *context, char *id, struct ast_state_cb_info *info, void *data);
+static int skinny_extensionstate_cb(const char *context, const char *exten, struct ast_state_cb_info *info, void *data);
 
 static struct skinny_line *skinny_line_alloc(void)
 {
@@ -3413,7 +3410,7 @@ static void transmit_serviceurlstat(stru
 	transmit_response(d, req);
 }
 
-static int skinny_extensionstate_cb(char *context, char *exten, struct ast_state_cb_info *info, void *data)
+static int skinny_extensionstate_cb(const char *context, const char *exten, struct ast_state_cb_info *info, void *data)
 {
 	struct skinny_container *container = data;
 	struct skinny_device *d = NULL;
@@ -8086,7 +8083,14 @@ static void config_parse_variables(int t
 			}
 		} else if (!strcasecmp(v->name, "permit") || !strcasecmp(v->name, "deny")) {
 			if (type & (TYPE_DEVICE)) {
-				CDEV->ha = ast_append_ha(v->name, v->value, CDEV->ha, NULL);
+				int acl_error = 0;
+
+				CDEV->ha = ast_append_ha(v->name, v->value, CDEV->ha, &acl_error);
+				if (acl_error) {
+					ast_log(LOG_ERROR, "Invalid ACL '%s' on line '%d'. Deleting device\n",
+							v->value, v->lineno);
+					CDEV->prune = 1;
+				}
 				continue;
 			}
 		} else if (!strcasecmp(v->name, "allow")) {
@@ -8330,6 +8334,8 @@ static struct skinny_line *config_line(c
 		mailbox_specific_topic = ast_mwi_topic(l->mailbox);
 		if (mailbox_specific_topic) {
 			l->mwi_event_sub = stasis_subscribe_pool(mailbox_specific_topic, mwi_event_cb, l);
+			stasis_subscription_accept_message_type(l->mwi_event_sub, ast_mwi_state_type());
+			stasis_subscription_set_filter(l->mwi_event_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 	}
 
@@ -8575,16 +8581,16 @@ static void delete_devices(void)
 		}
 		/* Delete all speeddials for this device */
 		while ((sd = AST_LIST_REMOVE_HEAD(&d->speeddials, list))) {
-			free(sd->container);
-			free(sd);
+			ast_free(sd->container);
+			ast_free(sd);
 		}
 		/* Delete all serviceurls for this device */
 		while ((surl = AST_LIST_REMOVE_HEAD(&d->serviceurls, list))) {
-			free(surl);
+			ast_free(surl);
 		}
 		/* Delete all addons for this device */
 		while ((a = AST_LIST_REMOVE_HEAD(&d->addons, list))) {
-			free(a);
+			ast_free(a);
 		}
 		d = skinny_device_destroy(d);
 	}
@@ -8638,11 +8644,11 @@ int skinny_reload(void)
 		}
 		/* Delete all speeddials for this device */
 		while ((sd = AST_LIST_REMOVE_HEAD(&d->speeddials, list))) {
-			free(sd);
+			ast_free(sd);
 		}
 		/* Delete all addons for this device */
 		while ((a = AST_LIST_REMOVE_HEAD(&d->addons, list))) {
-			free(a);
+			ast_free(a);
 		}
 		AST_LIST_REMOVE_CURRENT(list);
 		d = skinny_device_destroy(d);
@@ -8825,9 +8831,9 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Skinny Client Control Protocol (Skinny)",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
 );
diff -pruN 1:13.23.1~dfsg-2/channels/chan_unistim.c 1:16.1.1~dfsg-1/channels/chan_unistim.c
--- 1:13.23.1~dfsg-2/channels/chan_unistim.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_unistim.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 #include <signal.h>
 
@@ -815,7 +813,9 @@ static const char *ustmtext(const char *
 		char tmp[1024], *p, *p_orig = NULL, *p_trans = NULL;
 		FILE *f;
 
-		if (!(lang->trans = ao2_container_alloc(8, lang_hash_fn, lang_cmp_fn))) {
+		lang->trans = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 8,
+			lang_hash_fn, NULL, lang_cmp_fn);
+		if (!lang->trans) {
 			ast_log(LOG_ERROR, "Unable to allocate container for translation!\n");
 			return str;
 		}
@@ -2727,7 +2727,8 @@ static void send_start_rtp(struct unisti
 	}
 
 	pte = sub->parent->parent->session;
-	codec = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(sub->rtp), 1, ast_channel_readformat(sub->owner), 0);
+	codec = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(sub->rtp),
+		1, ast_channel_readformat(sub->owner), 0);
 	if ((ast_format_cmp(ast_channel_readformat(sub->owner), ast_format_ulaw) == AST_FORMAT_CMP_EQUAL) ||
 		(ast_format_cmp(ast_channel_readformat(sub->owner), ast_format_alaw) == AST_FORMAT_CMP_EQUAL)) {
 		if (unistimdebug) {
@@ -6423,6 +6424,85 @@ static struct unistim_line *find_line_by
 	return ret;
 }
 
+static void delete_device(struct unistim_device *d)
+{
+	struct unistim_line *l;
+	struct unistim_subchannel *sub;
+	struct unistimsession *s;
+
+	if (unistimdebug) {
+		ast_verb(0, "Removing device '%s'\n", d->name);
+	}
+	AST_LIST_LOCK(&d->subs);
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&d->subs, sub, list){
+		if (sub->subtype == SUB_REAL) {
+			if (sub->owner) {
+				ast_log(LOG_WARNING,
+						"Device '%s' was not deleted : a call is in progress. Try again later.\n",
+						d->name);
+				AST_LIST_UNLOCK(&d->subs);
+				return;
+			}
+		}
+		if (sub->subtype == SUB_THREEWAY) {
+			ast_log(LOG_WARNING,
+					"Device '%s' with threeway call subchannels allocated, aborting.\n",
+					d->name);
+			AST_LIST_UNLOCK(&d->subs);
+			return;
+		}
+		AST_LIST_REMOVE_CURRENT(list);
+		ast_mutex_destroy(&sub->lock);
+		ast_free(sub);
+	}
+	AST_LIST_TRAVERSE_SAFE_END
+	AST_LIST_UNLOCK(&d->subs);
+
+
+	AST_LIST_LOCK(&d->lines);
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&d->lines, l, list){
+		AST_LIST_REMOVE_CURRENT(list);
+		ast_mutex_destroy(&l->lock);
+		unistim_line_destroy(l);
+	}
+	AST_LIST_TRAVERSE_SAFE_END
+	AST_LIST_UNLOCK(&d->lines);
+
+	if (d->session) {
+		if (sessions == d->session) {
+			sessions = d->session->next;
+		} else {
+			s = sessions;
+			while (s) {
+				if (s->next == d->session) {
+					s->next = d->session->next;
+					break;
+				}
+				s = s->next;
+			}
+		}
+		ast_mutex_destroy(&d->session->lock);
+		ast_free(d->session);
+	}
+	if (devices == d) {
+		devices = d->next;
+	} else {
+		struct unistim_device *d2 = devices;
+		while (d2) {
+			if (d2->next == d) {
+				d2->next = d->next;
+				break;
+			}
+			d2 = d2->next;
+		}
+	}
+	if (d->tz) {
+		d->tz = ast_tone_zone_unref(d->tz);
+	}
+	ast_mutex_destroy(&d->lock);
+	ast_free(d);
+}
+
 static struct unistim_device *build_device(const char *cat, const struct ast_variable *v)
 {
 	struct unistim_device *d;
@@ -6514,7 +6594,14 @@ static struct unistim_device *build_devi
 		} else if (!strcasecmp(v->name, "tn")) {
 			ast_copy_string(d->extension_number, v->value, sizeof(d->extension_number));
 		} else if (!strcasecmp(v->name, "permit") || !strcasecmp(v->name, "deny")) {
-			d->ha = ast_append_ha(v->name, v->value, d->ha, NULL);
+			int acl_error = 0;
+			d->ha = ast_append_ha(v->name, v->value, d->ha, &acl_error);
+			if (acl_error) {
+				ast_log(LOG_ERROR, "Invalid ACL '%s' specified for device '%s' on line %d. Deleting device\n",
+						v->value, cat, v->lineno);
+				delete_device(d);
+				return NULL;
+			}
 		} else if (!strcasecmp(v->name, "context")) {
 			ast_copy_string(d->context, v->value, sizeof(d->context));
 		} else if (!strcasecmp(v->name, "maintext0")) {
@@ -6881,85 +6968,7 @@ static int reload_config(void)
 	d = devices;
 	while (d) {
 		if (d->to_delete) {
-			struct unistim_line *l;
-			struct unistim_subchannel *sub;
-
-			if (unistimdebug) {
-				ast_verb(0, "Removing device '%s'\n", d->name);
-			}
-			AST_LIST_LOCK(&d->subs);
-			AST_LIST_TRAVERSE_SAFE_BEGIN(&d->subs, sub, list){
-				if (sub->subtype == SUB_REAL) {
-					if (!sub) {
-						ast_log(LOG_ERROR, "Device '%s' without a subchannel !, aborting\n",
-								d->name);
-						ast_config_destroy(cfg);
-						return 0;
-					}
-					if (sub->owner) {
-						ast_log(LOG_WARNING,
-								"Device '%s' was not deleted : a call is in progress. Try again later.\n",
-								d->name);
-						d = d->next;
-						continue;
-					}
-				}
-				if (sub->subtype == SUB_THREEWAY) {
-					ast_log(LOG_WARNING,
-							"Device '%s' with threeway call subchannels allocated, aborting.\n",
-							d->name);
-					break;
-				}
-				AST_LIST_REMOVE_CURRENT(list);
-				ast_mutex_destroy(&sub->lock);
-				ast_free(sub);
-			}
-			AST_LIST_TRAVERSE_SAFE_END
-			AST_LIST_UNLOCK(&d->subs);
-
-
-			AST_LIST_LOCK(&d->lines);
-			AST_LIST_TRAVERSE_SAFE_BEGIN(&d->lines, l, list){
-				AST_LIST_REMOVE_CURRENT(list);
-				ast_mutex_destroy(&l->lock);
-				unistim_line_destroy(l);
-			}
-			AST_LIST_TRAVERSE_SAFE_END
-			AST_LIST_UNLOCK(&d->lines);
-
-			if (d->session) {
-				if (sessions == d->session) {
-					sessions = d->session->next;
-				} else {
-					s = sessions;
-					while (s) {
-						if (s->next == d->session) {
-							s->next = d->session->next;
-							break;
-						}
-						s = s->next;
-					}
-				}
-				ast_mutex_destroy(&d->session->lock);
-				ast_free(d->session);
-			}
-			if (devices == d) {
-				devices = d->next;
-			} else {
-				struct unistim_device *d2 = devices;
-				while (d2) {
-					if (d2->next == d) {
-						d2->next = d->next;
-						break;
-					}
-					d2 = d2->next;
-				}
-			}
-			if (d->tz) {
-				d->tz = ast_tone_zone_unref(d->tz);
-			}
-			ast_mutex_destroy(&d->lock);
-			ast_free(d);
+			delete_device(d);
 			d = devices;
 			continue;
 		}
@@ -7198,7 +7207,7 @@ int reload(void)
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "UNISTIM Protocol (USTM)",
 	.support_level = AST_MODULE_SUPPORT_EXTENDED,
-    .load = load_module,
-    .unload = unload_module,
-    .reload = reload,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
 );
diff -pruN 1:13.23.1~dfsg-2/channels/chan_vpb.cc 1:16.1.1~dfsg-1/channels/chan_vpb.cc
--- 1:13.23.1~dfsg-2/channels/chan_vpb.cc	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/chan_vpb.cc	2018-12-26 18:45:13.000000000 +0000
@@ -50,7 +50,7 @@
 /*** MODULEINFO
 	<depend>vpb</depend>
 	<defaultenabled>no</defaultenabled>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include <vpbapi.h>
@@ -59,8 +59,6 @@ extern "C" {
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/utils.h"
 #include "asterisk/channel.h"
@@ -366,14 +364,18 @@ static struct ast_channel_tech vpb_tech
 	.capabilities = NULL,
 	.properties = 0,
 	.requester = vpb_request,
+	.requester_with_stream_topology = NULL,
 	.devicestate = NULL,
+	.presencestate = NULL,
 	.send_digit_begin = vpb_digit_begin,
 	.send_digit_end = vpb_digit_end,
 	.call = vpb_call,
 	.hangup = vpb_hangup,
 	.answer = vpb_answer,
 	.read = vpb_read,
+	.read_stream = NULL,
 	.write = vpb_write,
+	.write_stream = NULL,
 	.send_text = NULL,
 	.send_image = NULL,
 	.send_html = NULL,
@@ -396,14 +398,18 @@ static struct ast_channel_tech vpb_tech_
 	.capabilities = NULL,
 	.properties = 0,
 	.requester = vpb_request,
+	.requester_with_stream_topology = NULL,
 	.devicestate = NULL,
+	.presencestate = NULL,
 	.send_digit_begin = vpb_digit_begin,
 	.send_digit_end = vpb_digit_end,
 	.call = vpb_call,
 	.hangup = vpb_hangup,
 	.answer = vpb_answer,
 	.read = vpb_read,
+	.read_stream = NULL,
 	.write = vpb_write,
+	.write_stream = NULL,
 	.send_text = NULL,
 	.send_image = NULL,
 	.send_html = NULL,
@@ -2869,4 +2875,4 @@ static enum ast_module_load_result load_
 #endif
 /**/
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Voicetronix API driver");
+AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Voicetronix API driver");
diff -pruN 1:13.23.1~dfsg-2/channels/console_board.c 1:16.1.1~dfsg-1/channels/console_board.c
--- 1:13.23.1~dfsg-2/channels/console_board.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/console_board.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,7 +42,6 @@
  ***/
 
 #include "asterisk.h"	/* ast_strdupa */
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/utils.h"	/* ast_strdupa */
 #include "console_video.h"	/* ast_strdupa */
 
diff -pruN 1:13.23.1~dfsg-2/channels/console_video.c 1:16.1.1~dfsg-1/channels/console_video.c
--- 1:13.23.1~dfsg-2/channels/console_video.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/console_video.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,7 +41,6 @@
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include <sys/ioctl.h>
 #include "asterisk/cli.h"
 #include "asterisk/file.h"
diff -pruN 1:13.23.1~dfsg-2/channels/dahdi/bridge_native_dahdi.c 1:16.1.1~dfsg-1/channels/dahdi/bridge_native_dahdi.c
--- 1:13.23.1~dfsg-2/channels/dahdi/bridge_native_dahdi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/dahdi/bridge_native_dahdi.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "../sig_analog.h"
 #if defined(HAVE_PRI)
 #include "../sig_pri.h"
@@ -903,11 +901,11 @@ void dahdi_native_unload(void)
  * \retval 0 on success.
  * \retval -1 on error.
  */
-int dahdi_native_load(struct ast_module *mod, const struct ast_channel_tech *tech)
+int dahdi_native_load(const struct ast_channel_tech *tech)
 {
 	dahdi_tech = tech;
 
-	if (__ast_bridge_technology_register(&native_bridge, mod)) {
+	if (ast_bridge_technology_register(&native_bridge)) {
 		dahdi_native_unload();
 		return -1;
 	}
diff -pruN 1:13.23.1~dfsg-2/channels/dahdi/bridge_native_dahdi.h 1:16.1.1~dfsg-1/channels/dahdi/bridge_native_dahdi.h
--- 1:13.23.1~dfsg-2/channels/dahdi/bridge_native_dahdi.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/dahdi/bridge_native_dahdi.h	2018-12-26 18:45:13.000000000 +0000
@@ -36,7 +36,7 @@ extern "C" {
 /* ------------------------------------------------------------------- */
 
 void dahdi_native_unload(void);
-int dahdi_native_load(struct ast_module *mod, const struct ast_channel_tech *tech);
+int dahdi_native_load(const struct ast_channel_tech *tech);
 
 /* ------------------------------------------------------------------- */
 
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/codec_pref.c 1:16.1.1~dfsg-1/channels/iax2/codec_pref.c
--- 1:13.23.1~dfsg-2/channels/iax2/codec_pref.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/codec_pref.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/firmware.c 1:16.1.1~dfsg-1/channels/iax2/firmware.c
--- 1:13.23.1~dfsg-2/channels/iax2/firmware.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/firmware.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/format_compatibility.c 1:16.1.1~dfsg-1/channels/iax2/format_compatibility.c
--- 1:13.23.1~dfsg-2/channels/iax2/format_compatibility.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/format_compatibility.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/include/astobj.h 1:16.1.1~dfsg-1/channels/iax2/include/astobj.h
--- 1:13.23.1~dfsg-2/channels/iax2/include/astobj.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/include/astobj.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,823 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 1999 - 2005, Digium, Inc.
+ *
+ * Mark Spencer <markster@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief Object Model for Asterisk
+ *
+ * \deprecated Use astobj2.h instead
+ */
+
+#ifndef _ASTERISK_ASTOBJ_H
+#define _ASTERISK_ASTOBJ_H
+
+#include "asterisk/lock.h"
+
+/*! \file
+ * \brief A set of macros implementing objects and containers.
+ * Macros are used for maximum performance, to support multiple inheritance,
+ * and to be easily integrated into existing structures without additional
+ * malloc calls, etc.
+ *
+ * These macros expect to operate on two different object types, ASTOBJs and
+ * ASTOBJ_CONTAINERs.  These are not actual types, as any struct can be
+ * converted into an ASTOBJ compatible object or container using the supplied
+ * macros.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_object {
+ *    ASTOBJ_COMPONENTS(struct sample_object);
+ * };
+ *
+ * struct sample_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_object);
+ * } super_container;
+ *
+ * void sample_object_destroy(struct sample_object *obj)
+ * {
+ *    free(obj);
+ * }
+ *
+ * int init_stuff()
+ * {
+ *    struct sample_object *obj1;
+ *    struct sample_object *found_obj;
+ *
+ *    obj1 = malloc(sizeof(struct sample_object));
+ *
+ *    ASTOBJ_CONTAINER_INIT(&super_container);
+ *
+ *    ASTOBJ_INIT(obj1);
+ *    ASTOBJ_WRLOCK(obj1);
+ *    ast_copy_string(obj1->name, "obj1", sizeof(obj1->name));
+ *    ASTOBJ_UNLOCK(obj1);
+ *
+ *    ASTOBJ_CONTAINER_LINK(&super_container, obj1);
+ *
+ *    found_obj = ASTOBJ_CONTAINER_FIND(&super_container, "obj1");
+ *
+ *    if(found_obj) {
+ *       printf("Found object: %s", found_obj->name);
+ *       ASTOBJ_UNREF(found_obj,sample_object_destroy);
+ *    }
+ *
+ *    ASTOBJ_CONTAINER_DESTROYALL(&super_container,sample_object_destroy);
+ *    ASTOBJ_CONTAINER_DESTROY(&super_container);
+ *
+ *    return 0;
+ * }
+ * \endcode
+ */
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+#define ASTOBJ_DEFAULT_NAMELEN 	80
+#define ASTOBJ_DEFAULT_BUCKETS	256
+#define ASTOBJ_DEFAULT_HASH		ast_strhash
+
+#define ASTOBJ_FLAG_MARKED	(1 << 0)		/* Object has been marked for future operation */
+
+/* C++ is simply a syntactic crutch for those who cannot think for themselves
+   in an object oriented way. */
+
+/*! \brief Lock an ASTOBJ for reading.
+ */
+#define ASTOBJ_RDLOCK(object) ast_mutex_lock(&(object)->_lock)
+
+/*! \brief Lock an ASTOBJ for writing.
+ */
+#define ASTOBJ_WRLOCK(object) ast_mutex_lock(&(object)->_lock)
+
+#define ASTOBJ_TRYWRLOCK(object) ast_mutex_trylock(&(object)->_lock)
+
+/*! \brief Unlock a locked object. */
+#define ASTOBJ_UNLOCK(object) ast_mutex_unlock(&(object)->_lock)
+
+#ifdef ASTOBJ_CONTAINER_HASHMODEL
+#define __ASTOBJ_HASH(type,hashes) \
+	type *next[hashes]
+#else
+#define __ASTOBJ_HASH(type,hashes) \
+	type *next[1]
+#endif
+
+/*! \brief Add ASTOBJ components to a struct (without locking support).
+ *
+ * \param type The datatype of the object.
+ * \param namelen The length to make the name char array.
+ * \param hashes The number of containers the object can be present in.
+ *
+ * This macro adds components to a struct to make it an ASTOBJ.  This macro
+ * differs from ASTOBJ_COMPONENTS_FULL in that it does not create a mutex for
+ * locking.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct {
+ *    ASTOBJ_COMPONENTS_NOLOCK_FULL(struct sample_struct,1,1);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_COMPONENTS_NOLOCK_FULL(type,namelen,hashes) \
+	char name[namelen]; \
+	unsigned int refcount; \
+	unsigned int objflags; \
+	__ASTOBJ_HASH(type,hashes)
+
+/*! \brief Add ASTOBJ components to a struct (without locking support).
+ *
+ * \param type The datatype of the object.
+ *
+ * This macro works like #ASTOBJ_COMPONENTS_NOLOCK_FULL() except it only accepts a
+ * type and uses default values for namelen and hashes.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_componets {
+ *    ASTOBJ_COMPONENTS_NOLOCK(struct sample_struct);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_COMPONENTS_NOLOCK(type) \
+	ASTOBJ_COMPONENTS_NOLOCK_FULL(type,ASTOBJ_DEFAULT_NAMELEN,1)
+
+/*! \brief Add ASTOBJ components to a struct (with locking support).
+ *
+ * \param type The datatype of the object.
+ *
+ * This macro works like #ASTOBJ_COMPONENTS_NOLOCK() except it includes locking
+ * support.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct {
+ *    ASTOBJ_COMPONENTS(struct sample_struct);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_COMPONENTS(type) \
+	ASTOBJ_COMPONENTS_NOLOCK(type); \
+	ast_mutex_t _lock;
+
+/*! \brief Add ASTOBJ components to a struct (with locking support).
+ *
+ * \param type The datatype of the object.
+ * \param namelen The length to make the name char array.
+ * \param hashes The number of containers the object can be present in.
+ *
+ * This macro adds components to a struct to make it an ASTOBJ and includes
+ * support for locking.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct {
+ *    ASTOBJ_COMPONENTS_FULL(struct sample_struct,1,1);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_COMPONENTS_FULL(type,namelen,hashes) \
+	ASTOBJ_COMPONENTS_NOLOCK_FULL(type,namelen,hashes); \
+	ast_mutex_t _lock;
+
+/*! \brief Increment an object reference count.
+ * \param object A pointer to the object to operate on.
+ * \return The object.
+ */
+#define ASTOBJ_REF(object) \
+	({ \
+		ASTOBJ_WRLOCK(object); \
+		(object)->refcount++; \
+		ASTOBJ_UNLOCK(object); \
+		(object); \
+	})
+
+/*! \brief Decrement the reference count on an object.
+ *
+ * \param object A pointer the object to operate on.
+ * \param destructor The destructor to call if the object is no longer referenced.  It will be passed the pointer as an argument.
+ *
+ * This macro unreferences an object and calls the specfied destructor if the
+ * object is no longer referenced.  The destructor should free the object if it
+ * was dynamically allocated.
+ */
+#define ASTOBJ_UNREF(object,destructor) \
+	do { \
+		int newcount = 0; \
+		ASTOBJ_WRLOCK(object); \
+		if (__builtin_expect((object)->refcount > 0, 1)) \
+			newcount = --((object)->refcount); \
+		else \
+			ast_log(AST_LOG_WARNING, "Unreferencing unreferenced (object)!\n"); \
+		ASTOBJ_UNLOCK(object); \
+		if (newcount == 0) { \
+			ast_mutex_destroy(&(object)->_lock); \
+			destructor((object)); \
+		} \
+		(object) = NULL; \
+	} while(0)
+
+/*! \brief Mark an ASTOBJ by adding the #ASTOBJ_FLAG_MARKED flag to its objflags mask.
+ * \param object A pointer to the object to operate on.
+ *
+ * This macro "marks" an object.  Marked objects can later be unlinked from a container using
+ * #ASTOBJ_CONTAINER_PRUNE_MARKED().
+ *
+ */
+#define ASTOBJ_MARK(object) \
+	do { \
+		ASTOBJ_WRLOCK(object); \
+		(object)->objflags |= ASTOBJ_FLAG_MARKED; \
+		ASTOBJ_UNLOCK(object); \
+	} while(0)
+
+/*! \brief Unmark an ASTOBJ by subtracting the #ASTOBJ_FLAG_MARKED flag from its objflags mask.
+ * \param object A pointer to the object to operate on.
+ */
+#define ASTOBJ_UNMARK(object) \
+	do { \
+		ASTOBJ_WRLOCK(object); \
+		(object)->objflags &= ~ASTOBJ_FLAG_MARKED; \
+		ASTOBJ_UNLOCK(object); \
+	} while(0)
+
+/*! \brief Initialize an object.
+ * \param object A pointer to the object to operate on.
+ *
+ * \note This should only be used on objects that support locking (objects
+ * created with #ASTOBJ_COMPONENTS() or #ASTOBJ_COMPONENTS_FULL())
+ */
+#define ASTOBJ_INIT(object) \
+	do { \
+		ast_mutex_init(&(object)->_lock); \
+		object->name[0] = '\0'; \
+		object->refcount = 1; \
+	} while(0)
+
+/* Containers for objects -- current implementation is linked lists, but
+   should be able to be converted to hashes relatively easily */
+
+/*! \brief Lock an ASTOBJ_CONTAINER for reading.
+ */
+#define ASTOBJ_CONTAINER_RDLOCK(container) ast_mutex_lock(&(container)->_lock)
+
+/*! \brief Lock an ASTOBJ_CONTAINER for writing.
+ */
+#define ASTOBJ_CONTAINER_WRLOCK(container) ast_mutex_lock(&(container)->_lock)
+
+/*! \brief Unlock an ASTOBJ_CONTAINER. */
+#define ASTOBJ_CONTAINER_UNLOCK(container) ast_mutex_unlock(&(container)->_lock)
+
+#ifdef ASTOBJ_CONTAINER_HASHMODEL
+#error "Hash model for object containers not yet implemented!"
+#else
+/* Linked lists */
+
+/*! \brief Create a container for ASTOBJs (without locking support).
+ *
+ * \param type The type of objects the container will hold.
+ * \param hashes Currently unused.
+ * \param buckets Currently unused.
+ *
+ * This macro is used to create a container for ASTOBJs without locking
+ * support.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_nolock_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(struct sample_struct,1,1);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(type,hashes,buckets) \
+	type *head
+
+/*! \brief Initialize a container.
+ *
+ * \param container A pointer to the container to initialize.
+ * \param hashes Currently unused.
+ * \param buckets Currently unused.
+ *
+ * This macro initializes a container.  It should only be used on containers
+ * that support locking.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS_FULL(struct sample_struct,1,1);
+ * } container;
+ *
+ * int func()
+ * {
+ *    ASTOBJ_CONTAINER_INIT_FULL(&container,1,1);
+ * }
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_INIT_FULL(container,hashes,buckets) \
+	do { \
+		ast_mutex_init(&(container)->_lock); \
+	} while(0)
+
+/*! \brief Destroy a container.
+ *
+ * \param container A pointer to the container to destroy.
+ * \param hashes Currently unused.
+ * \param buckets Currently unused.
+ *
+ * This macro frees up resources used by a container.  It does not operate on
+ * the objects in the container.  To unlink the objects from the container use
+ * #ASTOBJ_CONTAINER_DESTROYALL().
+ *
+ * \note This macro should only be used on containers with locking support.
+ */
+#define ASTOBJ_CONTAINER_DESTROY_FULL(container,hashes,buckets) \
+	do { \
+		ast_mutex_destroy(&(container)->_lock); \
+	} while(0)
+
+/*! \brief Iterate through the objects in a container.
+ *
+ * \param container A pointer to the container to traverse.
+ * \param continue A condition to allow the traversal to continue.
+ * \param eval A statement to evaluate in the iteration loop.
+ *
+ * This is macro is a little complicated, but it may help to think of it as a
+ * loop.  Basically it iterates through the specfied containter as long as the
+ * condition is met.  Two variables, iterator and next, are provided for use in
+ * your \p eval statement.  See the sample code for an example.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * ASTOBJ_CONTAINER_TRAVERSE(&sample_container,1, {
+ *    ASTOBJ_RDLOCK(iterator);
+ *    printf("Currently iterating over '%s'\n", iterator->name);
+ *    ASTOBJ_UNLOCK(iterator);
+ * } );
+ * \endcode
+ *
+ * \code
+ * ASTOBJ_CONTAINER_TRAVERSE(&sample_container,1, sample_func(iterator));
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_TRAVERSE(container,continue,eval) \
+	do { \
+		typeof((container)->head) iterator; \
+		typeof((container)->head) next; \
+		ASTOBJ_CONTAINER_RDLOCK(container); \
+		next = (container)->head; \
+		while((continue) && (iterator = next)) { \
+			next = iterator->next[0]; \
+			eval; \
+		} \
+		ASTOBJ_CONTAINER_UNLOCK(container); \
+	} while(0)
+
+/*! \brief Find an object in a container.
+ *
+ * \param container A pointer to the container to search.
+ * \param namestr The name to search for.
+ *
+ * Use this function to find an object with the specfied name in a container.
+ *
+ * \note When the returned object is no longer in use, #ASTOBJ_UNREF() should
+ * be used to free the additional reference created by this macro.
+ *
+ * \return A new reference to the object located or NULL if nothing is found.
+ */
+#define ASTOBJ_CONTAINER_FIND(container,namestr) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
+			if (!(strcasecmp(iterator->name, (namestr)))) \
+				found = ASTOBJ_REF(iterator); \
+		} while (0)); \
+		found; \
+	})
+
+/*! \brief Find an object in a container.
+ *
+ * \param container A pointer to the container to search.
+ * \param data The data to search for.
+ * \param field The field/member of the container's objects to search.
+ * \param hashfunc The hash function to use, currently not implemented.
+ * \param hashoffset The hash offset to use, currently not implemented.
+ * \param comparefunc The function used to compare the field and data values.
+ *
+ * This macro iterates through a container passing the specified field and data
+ * elements to the specified comparefunc.  The function should return 0 when a match is found.
+ *
+ * \note When the returned object is no longer in use, #ASTOBJ_UNREF() should
+ * be used to free the additional reference created by this macro.
+ *
+ * \return A pointer to the object located or NULL if nothing is found.
+ */
+#define ASTOBJ_CONTAINER_FIND_FULL(container,data,field,hashfunc,hashoffset,comparefunc) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
+			ASTOBJ_RDLOCK(iterator); \
+			if (!(comparefunc(iterator->field, (data)))) { \
+				found = ASTOBJ_REF(iterator); \
+			} \
+			ASTOBJ_UNLOCK(iterator); \
+		} while (0)); \
+		found; \
+	})
+
+/*! \brief Empty a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param destructor A destructor function to call on each object.
+ *
+ * This macro loops through a container removing all the items from it using
+ * #ASTOBJ_UNREF().  This does not destroy the container itself, use
+ * #ASTOBJ_CONTAINER_DESTROY() for that.
+ *
+ * \note If any object in the container is only referenced by the container,
+ * the destructor will be called for that object once it has been removed.
+ */
+#define ASTOBJ_CONTAINER_DESTROYALL(container,destructor) \
+	do { \
+		typeof((container)->head) iterator; \
+		ASTOBJ_CONTAINER_WRLOCK(container); \
+		while((iterator = (container)->head)) { \
+			(container)->head = (iterator)->next[0]; \
+			ASTOBJ_UNREF(iterator,destructor); \
+		} \
+		ASTOBJ_CONTAINER_UNLOCK(container); \
+	} while(0)
+
+/*! \brief Remove an object from a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param obj A pointer to the object to remove.
+ *
+ * This macro iterates through a container and removes the specfied object if
+ * it exists in the container.
+ *
+ * \note This macro does not destroy any objects, it simply unlinks
+ * them from the list.  No destructors are called.
+ *
+ * \return The container's reference to the removed object or NULL if no
+ * matching object was found.
+ */
+#define ASTOBJ_CONTAINER_UNLINK(container,obj) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		typeof((container)->head) prev = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
+			if (iterator == obj) { \
+				found = iterator; \
+				found->next[0] = NULL; \
+				ASTOBJ_CONTAINER_WRLOCK(container); \
+				if (prev) \
+					prev->next[0] = next; \
+				else \
+					(container)->head = next; \
+				ASTOBJ_CONTAINER_UNLOCK(container); \
+			} \
+			prev = iterator; \
+		} while (0)); \
+		found; \
+	})
+
+/*! \brief Find and remove an object from a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param namestr The name of the object to remove.
+ *
+ * This macro iterates through a container and removes the first object with
+ * the specfied name from the container.
+ *
+ * \note This macro does not destroy any objects, it simply unlinks
+ * them.  No destructors are called.
+ *
+ * \return The container's reference to the removed object or NULL if no
+ * matching object was found.
+ */
+#define ASTOBJ_CONTAINER_FIND_UNLINK(container,namestr) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		typeof((container)->head) prev = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
+			if (!(strcasecmp(iterator->name, (namestr)))) { \
+				found = iterator; \
+				found->next[0] = NULL; \
+				ASTOBJ_CONTAINER_WRLOCK(container); \
+				if (prev) \
+					prev->next[0] = next; \
+				else \
+					(container)->head = next; \
+				ASTOBJ_CONTAINER_UNLOCK(container); \
+			} \
+			prev = iterator; \
+		} while (0)); \
+		found; \
+	})
+
+/*! \brief Find and remove an object in a container.
+ *
+ * \param container A pointer to the container to search.
+ * \param data The data to search for.
+ * \param field The field/member of the container's objects to search.
+ * \param hashfunc The hash function to use, currently not implemented.
+ * \param hashoffset The hash offset to use, currently not implemented.
+ * \param comparefunc The function used to compare the field and data values.
+ *
+ * This macro iterates through a container passing the specified field and data
+ * elements to the specified comparefunc.  The function should return 0 when a match is found.
+ * If a match is found it is removed from the list.
+ *
+ * \note This macro does not destroy any objects, it simply unlinks
+ * them.  No destructors are called.
+ *
+ * \return The container's reference to the removed object or NULL if no match
+ * was found.
+ */
+#define ASTOBJ_CONTAINER_FIND_UNLINK_FULL(container,data,field,hashfunc,hashoffset,comparefunc) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		typeof((container)->head) prev = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
+			ASTOBJ_RDLOCK(iterator); \
+			if (!(comparefunc(iterator->field, (data)))) { \
+				found = iterator; \
+				found->next[0] = NULL; \
+				ASTOBJ_CONTAINER_WRLOCK(container); \
+				if (prev) \
+					prev->next[0] = next; \
+				else \
+					(container)->head = next; \
+				ASTOBJ_CONTAINER_UNLOCK(container); \
+			} \
+			ASTOBJ_UNLOCK(iterator); \
+			prev = iterator; \
+		} while (0)); \
+		found; \
+	})
+
+/*! \brief Add an object to the end of a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param newobj A pointer to the object to be added.
+ *
+ * This macro adds an object to the end of a container.
+ */
+#define ASTOBJ_CONTAINER_LINK_END(container,newobj) \
+	do { \
+		typeof((container)->head) iterator; \
+		typeof((container)->head) next; \
+		typeof((container)->head) prev; \
+		ASTOBJ_CONTAINER_RDLOCK(container); \
+		prev = NULL; \
+		next = (container)->head; \
+		while((iterator = next)) { \
+			next = iterator->next[0]; \
+			prev = iterator; \
+		} \
+		if(prev) { \
+			ASTOBJ_CONTAINER_WRLOCK((container)); \
+			prev->next[0] = ASTOBJ_REF(newobj); \
+			(newobj)->next[0] = NULL; \
+			ASTOBJ_CONTAINER_UNLOCK((container)); \
+		} else { \
+			ASTOBJ_CONTAINER_LINK_START((container),(newobj)); \
+		} \
+		ASTOBJ_CONTAINER_UNLOCK((container)); \
+	} while(0)
+
+/*! \brief Add an object to the front of a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param newobj A pointer to the object to be added.
+ *
+ * This macro adds an object to the start of a container.
+ */
+#define ASTOBJ_CONTAINER_LINK_START(container,newobj) \
+	do { \
+		ASTOBJ_CONTAINER_WRLOCK(container); \
+		(newobj)->next[0] = (container)->head; \
+		(container)->head = ASTOBJ_REF(newobj); \
+		ASTOBJ_CONTAINER_UNLOCK(container); \
+	} while(0)
+
+/*! \brief Remove an object from the front of a container.
+ *
+ * \param container A pointer to the container to operate on.
+ *
+ * This macro removes the first object in a container.
+ *
+ * \note This macro does not destroy any objects, it simply unlinks
+ * them from the list.  No destructors are called.
+ *
+ * \return The container's reference to the removed object or NULL if no
+ * matching object was found.
+ */
+#define ASTOBJ_CONTAINER_UNLINK_START(container) \
+	({ \
+		typeof((container)->head) found = NULL; \
+		ASTOBJ_CONTAINER_WRLOCK(container); \
+	 	if((container)->head) { \
+	 		found = (container)->head; \
+	 		(container)->head = (container)->head->next[0]; \
+	 		found->next[0] = NULL; \
+	 	} \
+		ASTOBJ_CONTAINER_UNLOCK(container); \
+	 	found; \
+	})
+
+/*! \brief Prune marked objects from a container.
+ *
+ * \param container A pointer to the container to prune.
+ * \param destructor A destructor function to call on each marked object.
+ *
+ * This macro iterates through the specfied container and prunes any marked
+ * objects executing the specfied destructor if necessary.
+ */
+#define ASTOBJ_CONTAINER_PRUNE_MARKED(container,destructor) \
+	do { \
+		typeof((container)->head) prev = NULL; \
+		ASTOBJ_CONTAINER_TRAVERSE(container, 1, do { \
+			ASTOBJ_RDLOCK(iterator); \
+			if (iterator->objflags & ASTOBJ_FLAG_MARKED) { \
+				ASTOBJ_CONTAINER_WRLOCK(container); \
+				if (prev) \
+					prev->next[0] = next; \
+				else \
+					(container)->head = next; \
+				ASTOBJ_CONTAINER_UNLOCK(container); \
+				ASTOBJ_UNLOCK(iterator); \
+				ASTOBJ_UNREF(iterator,destructor); \
+				continue; \
+			} \
+			ASTOBJ_UNLOCK(iterator); \
+			prev = iterator; \
+		} while (0)); \
+	} while(0)
+
+/*! \brief Add an object to a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param newobj A pointer to the object to be added.
+ * \param data Currently unused.
+ * \param field Currently unused.
+ * \param hashfunc Currently unused.
+ * \param hashoffset Currently unused.
+ * \param comparefunc Currently unused.
+ *
+ * Currently this function adds an object to the head of the list.  One day it
+ * will support adding objects atthe position specified using the various
+ * options this macro offers.
+ */
+#define ASTOBJ_CONTAINER_LINK_FULL(container,newobj,data,field,hashfunc,hashoffset,comparefunc) \
+	do { \
+		ASTOBJ_CONTAINER_WRLOCK(container); \
+		(newobj)->next[0] = (container)->head; \
+		(container)->head = ASTOBJ_REF(newobj); \
+		ASTOBJ_CONTAINER_UNLOCK(container); \
+	} while(0)
+
+#endif /* List model */
+
+/* Common to hash and linked list models */
+
+/*! \brief Create a container for ASTOBJs (without locking support).
+ *
+ * \param type The type of objects the container will hold.
+ *
+ * This macro is used to create a container for ASTOBJs without locking
+ * support.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_nolock_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(struct sample_struct);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(type) \
+	ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(type,1,ASTOBJ_DEFAULT_BUCKETS)
+
+
+/*! \brief Create a container for ASTOBJs (with locking support).
+ *
+ * \param type The type of objects the container will hold.
+ *
+ * This macro is used to create a container for ASTOBJs with locking support.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_struct);
+ * };
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_COMPONENTS(type) \
+	ast_mutex_t _lock; \
+	ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(type)
+
+/*! \brief Initialize a container.
+ *
+ * \param container A pointer to the container to initialize.
+ *
+ * This macro initializes a container.  It should only be used on containers
+ * that support locking.
+ *
+ * <b>Sample Usage:</b>
+ * \code
+ * struct sample_struct_container {
+ *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_struct);
+ * } container;
+ *
+ * int func()
+ * {
+ *    ASTOBJ_CONTAINER_INIT(&container);
+ * }
+ * \endcode
+ */
+#define ASTOBJ_CONTAINER_INIT(container) \
+	ASTOBJ_CONTAINER_INIT_FULL(container,1,ASTOBJ_DEFAULT_BUCKETS)
+
+/*! \brief Destroy a container.
+ *
+ * \param container A pointer to the container to destory.
+ *
+ * This macro frees up resources used by a container.  It does not operate on
+ * the objects in the container.  To unlink the objects from the container use
+ * #ASTOBJ_CONTAINER_DESTROYALL().
+ *
+ * \note This macro should only be used on containers with locking support.
+ */
+#define ASTOBJ_CONTAINER_DESTROY(container) \
+	ASTOBJ_CONTAINER_DESTROY_FULL(container,1,ASTOBJ_DEFAULT_BUCKETS)
+
+/*! \brief Add an object to a container.
+ *
+ * \param container A pointer to the container to operate on.
+ * \param newobj A pointer to the object to be added.
+ *
+ * Currently this macro adds an object to the head of a container.  One day it
+ * should add an object in alphabetical order.
+ */
+#define ASTOBJ_CONTAINER_LINK(container,newobj) \
+	ASTOBJ_CONTAINER_LINK_FULL(container,newobj,(newobj)->name,name,ASTOBJ_DEFAULT_HASH,0,strcasecmp)
+
+/*! \brief Mark all the objects in a container.
+ * \param container A pointer to the container to operate on.
+ */
+#define ASTOBJ_CONTAINER_MARKALL(container) \
+	ASTOBJ_CONTAINER_TRAVERSE(container, 1, ASTOBJ_MARK(iterator))
+
+/*! \brief Unmark all the objects in a container.
+ * \param container A pointer to the container to operate on.
+ */
+#define ASTOBJ_CONTAINER_UNMARKALL(container) \
+	ASTOBJ_CONTAINER_TRAVERSE(container, 1, ASTOBJ_UNMARK(iterator))
+
+/*! \brief Dump information about an object into a string.
+ *
+ * \param s A pointer to the string buffer to use.
+ * \param slen The length of s.
+ * \param obj A pointer to the object to dump.
+ *
+ * This macro dumps a text representation of the name, objectflags, and
+ * refcount fields of an object to the specfied string buffer.
+ */
+#define ASTOBJ_DUMP(s,slen,obj) \
+	snprintf((s),(slen),"name: %s\nobjflags: %u\nrefcount: %u\n\n", (obj)->name, (obj)->objflags, (obj)->refcount);
+
+/*! \brief Dump information about all the objects in a container to a file descriptor.
+ *
+ * \param fd The file descriptor to write to.
+ * \param s A string buffer, same as #ASTOBJ_DUMP().
+ * \param slen The length of s, same as #ASTOBJ_DUMP().
+ * \param container A pointer to the container to dump.
+ *
+ * This macro dumps a text representation of the name, objectflags, and
+ * refcount fields of all the objects in a container to the specified file
+ * descriptor.
+ */
+#define ASTOBJ_CONTAINER_DUMP(fd,s,slen,container) \
+	ASTOBJ_CONTAINER_TRAVERSE(container, 1, do { ASTOBJ_DUMP(s,slen,iterator); ast_cli(fd, "%s", s); } while(0))
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_ASTOBJ_H */
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/include/netsock.h 1:16.1.1~dfsg-1/channels/iax2/include/netsock.h
--- 1:13.23.1~dfsg-2/channels/iax2/include/netsock.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/include/netsock.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,74 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 1999 - 2005, Digium, Inc.
+ *
+ * Mark Spencer <markster@digium.com>
+ * Kevin P. Fleming <kpfleming@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief Network socket handling
+ *
+ * \deprecated Use netsock2.h instead
+ */
+
+#ifndef _ASTERISK_NETSOCK_H
+#define _ASTERISK_NETSOCK_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+#include "asterisk/network.h"
+#include "asterisk/io.h"
+#include "asterisk/netsock2.h"
+
+struct ast_netsock;
+
+struct ast_netsock_list;
+
+struct ast_netsock_list *ast_netsock_list_alloc(void);
+
+int ast_netsock_init(struct ast_netsock_list *list);
+
+struct ast_netsock *ast_netsock_bind(struct ast_netsock_list *list, struct io_context *ioc,
+				     const char *bindinfo, int defaultport, int tos, int cos, ast_io_cb callback, void *data);
+
+struct ast_netsock *ast_netsock_bindaddr(struct ast_netsock_list *list, struct io_context *ioc,
+					 struct ast_sockaddr *bindaddr, int tos, int cos, ast_io_cb callback, void *data);
+
+int ast_netsock_release(struct ast_netsock_list *list);
+
+struct ast_netsock *ast_netsock_find(struct ast_netsock_list *list,
+				     struct ast_sockaddr *addr);
+
+/*!
+ * \deprecated Use ast_seq_qos in netsock2.h which properly handles IPv4 and IPv6
+ *             sockets, instead.
+ */
+int ast_netsock_set_qos(int sockfd, int tos, int cos, const char *desc);
+
+int ast_netsock_sockfd(const struct ast_netsock *ns);
+
+const struct ast_sockaddr *ast_netsock_boundaddr(const struct ast_netsock *ns);
+
+void *ast_netsock_data(const struct ast_netsock *ns);
+
+void ast_netsock_unref(struct ast_netsock *ns);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_NETSOCK_H */
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/netsock.c 1:16.1.1~dfsg-1/channels/iax2/netsock.c
--- 1:13.23.1~dfsg-2/channels/iax2/netsock.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/netsock.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,201 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 1999 - 2005, Digium, Inc.
+ *
+ * Kevin P. Fleming <kpfleming@digium.com>
+ * Mark Spencer <markster@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Network socket handling
+ *
+ * \author Kevin P. Fleming <kpfleming@digium.com>
+ * \author Mark Spencer <markster@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#if !defined (__linux__) && !defined (__GNU__)
+#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__Darwin__) || defined(__GLIBC__)
+#include <net/if_dl.h>
+#endif
+#endif
+
+#if defined (SOLARIS)
+#include <sys/sockio.h>
+#elif defined(HAVE_GETIFADDRS)
+#include <ifaddrs.h>
+#endif
+
+#include "include/netsock.h"
+#include "asterisk/netsock2.h"
+#include "asterisk/utils.h"
+#include "include/astobj.h"
+
+struct ast_netsock {
+	ASTOBJ_COMPONENTS(struct ast_netsock);
+	struct ast_sockaddr bindaddr;
+	int sockfd;
+	int *ioref;
+	struct io_context *ioc;
+	void *data;
+};
+
+struct ast_netsock_list {
+	ASTOBJ_CONTAINER_COMPONENTS(struct ast_netsock);
+	struct io_context *ioc;
+};
+
+static void ast_netsock_destroy(struct ast_netsock *netsock)
+{
+	ast_io_remove(netsock->ioc, netsock->ioref);
+	close(netsock->sockfd);
+	ast_free(netsock);
+}
+
+struct ast_netsock_list *ast_netsock_list_alloc(void)
+{
+	return ast_calloc(1, sizeof(struct ast_netsock_list));
+}
+
+int ast_netsock_init(struct ast_netsock_list *list)
+{
+	memset(list, 0, sizeof(*list));
+	ASTOBJ_CONTAINER_INIT(list);
+
+	return 0;
+}
+
+int ast_netsock_release(struct ast_netsock_list *list)
+{
+	ASTOBJ_CONTAINER_DESTROYALL(list, ast_netsock_destroy);
+	ASTOBJ_CONTAINER_DESTROY(list);
+	ast_free(list);
+
+	return 0;
+}
+
+struct ast_netsock *ast_netsock_find(struct ast_netsock_list *list, struct ast_sockaddr *addr)
+{
+	struct ast_netsock *sock = NULL;
+
+	ASTOBJ_CONTAINER_TRAVERSE(list, !sock, {
+		ASTOBJ_RDLOCK(iterator);
+		if (!ast_sockaddr_cmp(&iterator->bindaddr, addr)) {
+			sock = iterator;
+		}
+		ASTOBJ_UNLOCK(iterator);
+	});
+
+	return sock;
+}
+
+struct ast_netsock *ast_netsock_bindaddr(struct ast_netsock_list *list, struct io_context *ioc, struct ast_sockaddr *bindaddr, int tos, int cos, ast_io_cb callback, void *data)
+{
+	int netsocket = -1;
+	int *ioref;
+
+	struct ast_netsock *ns;
+	const int reuseFlag = 1;
+
+	/* Make a UDP socket */
+	netsocket = socket(ast_sockaddr_is_ipv6(bindaddr) ? AST_AF_INET6 : AST_AF_INET, SOCK_DGRAM, IPPROTO_IP);
+
+	if (netsocket < 0) {
+		ast_log(LOG_ERROR, "Unable to create network socket: %s\n", strerror(errno));
+		return NULL;
+	}
+	if (setsockopt(netsocket, SOL_SOCKET, SO_REUSEADDR, (char *)&reuseFlag, sizeof reuseFlag) < 0) {
+		ast_log(LOG_WARNING, "Error setting SO_REUSEADDR on sockfd '%d'\n", netsocket);
+	}
+	if (ast_bind(netsocket, bindaddr)) {
+		ast_log(LOG_ERROR,
+			"Unable to bind to %s: %s\n",
+			ast_sockaddr_stringify(bindaddr),
+			strerror(errno));
+		close(netsocket);
+		return NULL;
+	}
+
+	ast_set_qos(netsocket, tos, cos, "IAX2");
+
+	ast_enable_packet_fragmentation(netsocket);
+
+	if (!(ns = ast_calloc(1, sizeof(*ns)))) {
+		close(netsocket);
+		return NULL;
+	}
+
+	/* Establish I/O callback for socket read */
+	if (!(ioref = ast_io_add(ioc, netsocket, callback, AST_IO_IN, ns))) {
+		close(netsocket);
+		ast_free(ns);
+		return NULL;
+	}
+	ASTOBJ_INIT(ns);
+	ns->ioref = ioref;
+	ns->ioc = ioc;
+	ns->sockfd = netsocket;
+	ns->data = data;
+	ast_sockaddr_copy(&ns->bindaddr, bindaddr);
+	ASTOBJ_CONTAINER_LINK(list, ns);
+
+	return ns;
+}
+
+int ast_netsock_set_qos(int sockfd, int tos, int cos, const char *desc)
+{
+	return ast_set_qos(sockfd, tos, cos, desc);
+}
+
+struct ast_netsock *ast_netsock_bind(struct ast_netsock_list *list, struct io_context *ioc, const char *bindinfo, int defaultport, int tos, int cos, ast_io_cb callback, void *data)
+{
+	struct ast_sockaddr addr;
+
+	if (ast_sockaddr_parse(&addr, bindinfo, 0)) {
+
+		if (!ast_sockaddr_port(&addr)) {
+			ast_sockaddr_set_port(&addr, defaultport);
+		}
+
+		return ast_netsock_bindaddr(list, ioc, &addr, tos, cos, callback, data);
+	}
+
+	return NULL;
+}
+
+int ast_netsock_sockfd(const struct ast_netsock *ns)
+{
+	return ns ? ns-> sockfd : -1;
+}
+
+const struct ast_sockaddr *ast_netsock_boundaddr(const struct ast_netsock *ns)
+{
+	return &ns->bindaddr;
+}
+
+void *ast_netsock_data(const struct ast_netsock *ns)
+{
+	return ns->data;
+}
+
+void ast_netsock_unref(struct ast_netsock *ns)
+{
+	ASTOBJ_UNREF(ns, ast_netsock_destroy);
+}
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/parser.c 1:16.1.1~dfsg-1/channels/iax2/parser.c
--- 1:13.23.1~dfsg-2/channels/iax2/parser.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/parser.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
diff -pruN 1:13.23.1~dfsg-2/channels/iax2/provision.c 1:16.1.1~dfsg-1/channels/iax2/provision.c
--- 1:13.23.1~dfsg-2/channels/iax2/provision.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/iax2/provision.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <netdb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
diff -pruN 1:13.23.1~dfsg-2/channels/Makefile 1:16.1.1~dfsg-1/channels/Makefile
--- 1:13.23.1~dfsg-2/channels/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -23,50 +23,20 @@ ifneq ($(findstring $(OSARCH), mingw32 c
   LIBS+= -lres_monitor.so
 endif
 
-clean::
-	$(MAKE) -C misdn clean
-	rm -f dahdi/*.o dahdi/*.i dahdi/*.gcda dahdi/*.gcno
-	rm -f sip/*.o sip/*.i sip/*.gcda sip/*.gcno
-	rm -f iax2/*.o iax2/*.i iax2/*.gcda iax2/*.gcno
-	rm -f pjsip/*.o pjsip/*.i pjsip/*.gcda pjsip/*.gcno
-
-chan_iax2.so: $(subst .c,.o,$(wildcard iax2/*.c))
-$(subst .c,.o,$(wildcard iax2/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_iax2)
-
-chan_sip.so: $(subst .c,.o,$(wildcard sip/*.c))
-$(subst .c,.o,$(wildcard sip/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_sip)
-
-chan_pjsip.so: $(subst .c,.o,$(wildcard pjsip/*.c))
-$(subst .c,.o,$(wildcard pjsip/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_pjsip)
-
-# Additional objects to combine with chan_dahdi.so
-CHAN_DAHDI_OBJS= \
-	$(subst .c,.o,$(wildcard dahdi/*.c))	\
-	sig_analog.o	\
-	sig_pri.o	\
-	sig_ss7.o	\
-
-chan_dahdi.so: $(CHAN_DAHDI_OBJS)
-$(CHAN_DAHDI_OBJS): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_dahdi)
+$(call MOD_ADD_C,chan_iax2,$(wildcard iax2/*.c))
+$(call MOD_ADD_C,chan_sip,$(wildcard sip/*.c))
+$(call MOD_ADD_C,chan_pjsip,$(wildcard pjsip/*.c))
+$(call MOD_ADD_C,chan_dahdi,$(wildcard dahdi/*.c) sig_analog.c sig_pri.c sig_ss7.c)
+$(call MOD_ADD_C,chan_misdn,misdn_config.c misdn/isdn_lib.c misdn/isdn_msg_parser.c)
 
+chan_dahdi.o: _ASTCFLAGS+=$(call get_menuselect_cflags,LOTS_OF_SPANS)
 chan_mgcp.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 chan_unistim.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 chan_phone.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 chan_sip.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 
 chan_misdn.o: _ASTCFLAGS+=-Imisdn
-
 misdn_config.o: _ASTCFLAGS+=-Imisdn
-
 misdn/isdn_lib.o: _ASTCFLAGS+=-Wno-strict-aliasing
 
-chan_misdn.so: misdn_config.o misdn/isdn_lib.o misdn/isdn_msg_parser.o
-misdn_config.o misdn/isdn_lib.o misdn/isdn_msg_parser.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_misdn)
-
-chan_oss.so: console_video.o vgrabbers.o console_board.o
-console_video.o vgrabbers.o console_board.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,chan_oss)
-
-chan_usbradio.o: ./xpmr/xpmr.c ./xpmr/xpmr.h ./xpmr/xpmr_coef.h
-
-chan_usbradio.so: LIBS+=-lusb -lasound
-chan_usbradio.so: _ASTCFLAGS+=-DNDEBUG
+$(call MOD_ADD_C,chan_oss,console_video.c vgrabbers.c console_board.c)
diff -pruN 1:13.23.1~dfsg-2/channels/misdn_config.c 1:16.1.1~dfsg-1/channels/misdn_config.c
--- 1:13.23.1~dfsg-2/channels/misdn_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/misdn_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "chan_misdn_config.h"
 
 #include "asterisk/config.h"
diff -pruN 1:13.23.1~dfsg-2/channels/pjsip/cli_commands.c 1:16.1.1~dfsg-1/channels/pjsip/cli_commands.c
--- 1:13.23.1~dfsg-2/channels/pjsip/cli_commands.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/pjsip/cli_commands.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <pjsip.h>
 #include <pjlib.h>
 #include <pjsip_ua.h>
@@ -344,8 +342,9 @@ static int cli_channelstats_print_body(v
 	const struct ast_channel_snapshot *snapshot = obj;
 	struct ast_channel *channel = ast_channel_get_by_name(snapshot->name);
 	struct ast_sip_channel_pvt *cpvt = channel ? ast_channel_tech_pvt(channel) : NULL;
-	struct chan_pjsip_pvt *pvt = cpvt ? cpvt->pvt : NULL;
-	struct ast_sip_session_media *media = pvt ? pvt->media[SIP_MEDIA_AUDIO] : NULL;
+	struct ast_sip_session *session;
+	struct ast_sip_session_media *media;
+	struct ast_rtp_instance *rtp;
 	struct ast_rtp_instance_stats stats;
 	char *print_name = NULL;
 	char *print_time = alloca(32);
@@ -353,29 +352,46 @@ static int cli_channelstats_print_body(v
 
 	ast_assert(context->output_buffer != NULL);
 
+	if (!channel) {
+		ast_str_append(&context->output_buffer, 0, " %s not valid\n", snapshot->name);
+		return -1;
+	}
+
+	ast_channel_lock(channel);
+
+	session = cpvt->session;
+	if (!session) {
+		ast_str_append(&context->output_buffer, 0, " %s not valid\n", snapshot->name);
+		ast_channel_unlock(channel);
+		ao2_cleanup(channel);
+		return -1;
+	}
+
+	media = session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
 	if (!media || !media->rtp) {
 		ast_str_append(&context->output_buffer, 0, " %s not valid\n", snapshot->name);
+		ast_channel_unlock(channel);
 		ao2_cleanup(channel);
 		return -1;
 	}
 
+	rtp = ao2_bump(media->rtp);
+
 	codec_in_use[0] = '\0';
 
-	if (channel) {
-		ast_channel_lock(channel);
-		if (ast_channel_rawreadformat(channel)) {
-			ast_copy_string(codec_in_use, ast_format_get_name(ast_channel_rawreadformat(channel)), sizeof(codec_in_use));
-		}
-		ast_channel_unlock(channel);
+	if (ast_channel_rawreadformat(channel)) {
+		ast_copy_string(codec_in_use, ast_format_get_name(ast_channel_rawreadformat(channel)), sizeof(codec_in_use));
 	}
 
+	ast_channel_unlock(channel);
+
 	print_name = ast_strdupa(snapshot->name);
 	/* Skip the PJSIP/.  We know what channel type it is and we need the space. */
 	print_name += 6;
 
 	ast_format_duration_hh_mm_ss(ast_tvnow().tv_sec - snapshot->creationtime.tv_sec, print_time, 32);
 
-	if (ast_rtp_instance_get_stats(media->rtp, &stats, AST_RTP_INSTANCE_STAT_ALL)) {
+	if (ast_rtp_instance_get_stats(rtp, &stats, AST_RTP_INSTANCE_STAT_ALL)) {
 		ast_str_append(&context->output_buffer, 0, "%s direct media\n", snapshot->name);
 	} else {
 		ast_str_append(&context->output_buffer, 0,
@@ -400,6 +416,7 @@ static int cli_channelstats_print_body(v
 		);
 	}
 
+	ao2_cleanup(rtp);
 	ao2_cleanup(channel);
 
 	return 0;
diff -pruN 1:13.23.1~dfsg-2/channels/pjsip/dialplan_functions.c 1:16.1.1~dfsg-1/channels/pjsip/dialplan_functions.c
--- 1:13.23.1~dfsg-2/channels/pjsip/dialplan_functions.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/pjsip/dialplan_functions.c	2018-12-26 18:45:13.000000000 +0000
@@ -121,6 +121,60 @@
 		<ref type="function">PJSIP_MEDIA_OFFER</ref>
 	</see-also>
 </function>
+<function name="PJSIP_PARSE_URI" language="en_US">
+	<synopsis>
+		Parse an uri and return a type part of the URI.
+	</synopsis>
+	<syntax>
+		<parameter name="uri" required="true">
+			<para>URI to parse</para>
+		</parameter>
+		<parameter name="type" required="true">
+			<para>The <literal>type</literal> parameter specifies which URI part to read</para>
+			<enumlist>
+				<enum name="display">
+					<para>Display name.</para>
+				</enum>
+				<enum name="scheme">
+					<para>URI scheme.</para>
+				</enum>
+				<enum name="user">
+					<para>User part.</para>
+				</enum>
+				<enum name="passwd">
+					<para>Password part.</para>
+				</enum>
+				<enum name="host">
+					<para>Host part.</para>
+				</enum>
+				<enum name="port">
+					<para>Port number, or zero.</para>
+				</enum>
+				<enum name="user_param">
+					<para>User parameter.</para>
+				</enum>
+				<enum name="method_param">
+					<para>Method parameter.</para>
+				</enum>
+				<enum name="transport_param">
+					<para>Transport parameter.</para>
+				</enum>
+				<enum name="ttl_param">
+					<para>TTL param, or -1.</para>
+				</enum>
+				<enum name="lr_param">
+					<para>Loose routing param, or zero.</para>
+				</enum>
+				<enum name="maddr_param">
+					<para>Maddr param.</para>
+				</enum>
+			</enumlist>
+		</parameter>
+	</syntax>
+	<description>
+		<para>Parse an URI and return a specified part of the URI.</para>
+	</description>
+</function>
 <info name="CHANNEL" language="en_US" tech="PJSIP">
 	<enumlist>
 		<enum name="rtp">
@@ -454,13 +508,12 @@
 #include <pjlib.h>
 #include <pjsip_ua.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
 #include "asterisk/acl.h"
 #include "asterisk/app.h"
 #include "asterisk/channel.h"
+#include "asterisk/stream.h"
 #include "asterisk/format.h"
 #include "asterisk/dsp.h"
 #include "asterisk/pbx.h"
@@ -486,8 +539,8 @@ static const char *t38state_to_string[T3
 static int channel_read_rtp(struct ast_channel *chan, const char *type, const char *field, char *buf, size_t buflen)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-	struct chan_pjsip_pvt *pvt;
-	struct ast_sip_session_media *media = NULL;
+	struct ast_sip_session *session;
+	struct ast_sip_session_media *media;
 	struct ast_sockaddr addr;
 
 	if (!channel) {
@@ -495,9 +548,9 @@ static int channel_read_rtp(struct ast_c
 		return -1;
 	}
 
-	pvt = channel->pvt;
-	if (!pvt) {
-		ast_log(AST_LOG_WARNING, "Channel %s has no chan_pjsip pvt!\n", ast_channel_name(chan));
+	session = channel->session;
+	if (!session) {
+		ast_log(AST_LOG_WARNING, "Channel %s has no session!\n", ast_channel_name(chan));
 		return -1;
 	}
 
@@ -507,9 +560,9 @@ static int channel_read_rtp(struct ast_c
 	}
 
 	if (ast_strlen_zero(field) || !strcmp(field, "audio")) {
-		media = pvt->media[SIP_MEDIA_AUDIO];
+		media = session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
 	} else if (!strcmp(field, "video")) {
-		media = pvt->media[SIP_MEDIA_VIDEO];
+		media = session->active_media_state->default_session[AST_MEDIA_TYPE_VIDEO];
 	} else {
 		ast_log(AST_LOG_WARNING, "Unknown media type field '%s' for 'rtp' information\n", field);
 		return -1;
@@ -538,7 +591,7 @@ static int channel_read_rtp(struct ast_c
 			snprintf(buf, buflen, "%d", 0);
 		}
 	} else if (!strcmp(type, "hold")) {
-		snprintf(buf, buflen, "%d", media->held ? 1 : 0);
+		snprintf(buf, buflen, "%d", media->remotely_held ? 1 : 0);
 	} else {
 		ast_log(AST_LOG_WARNING, "Unknown type field '%s' specified for 'rtp' information\n", type);
 		return -1;
@@ -553,17 +606,17 @@ static int channel_read_rtp(struct ast_c
 static int channel_read_rtcp(struct ast_channel *chan, const char *type, const char *field, char *buf, size_t buflen)
 {
 	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-	struct chan_pjsip_pvt *pvt;
-	struct ast_sip_session_media *media = NULL;
+	struct ast_sip_session *session;
+	struct ast_sip_session_media *media;
 
 	if (!channel) {
 		ast_log(AST_LOG_WARNING, "Channel %s has no pvt!\n", ast_channel_name(chan));
 		return -1;
 	}
 
-	pvt = channel->pvt;
-	if (!pvt) {
-		ast_log(AST_LOG_WARNING, "Channel %s has no chan_pjsip pvt!\n", ast_channel_name(chan));
+	session = channel->session;
+	if (!session) {
+		ast_log(AST_LOG_WARNING, "Channel %s has no session!\n", ast_channel_name(chan));
 		return -1;
 	}
 
@@ -573,9 +626,9 @@ static int channel_read_rtcp(struct ast_
 	}
 
 	if (ast_strlen_zero(field) || !strcmp(field, "audio")) {
-		media = pvt->media[SIP_MEDIA_AUDIO];
+		media = session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
 	} else if (!strcmp(field, "video")) {
-		media = pvt->media[SIP_MEDIA_VIDEO];
+		media = session->active_media_state->default_session[AST_MEDIA_TYPE_VIDEO];
 	} else {
 		ast_log(AST_LOG_WARNING, "Unknown media type field '%s' for 'rtcp' information\n", field);
 		return -1;
@@ -987,22 +1040,238 @@ int pjsip_acf_dial_contacts_read(struct
 	return 0;
 }
 
+/*! \brief Session refresh state information */
+struct session_refresh_state {
+	/*! \brief Created proposed media state */
+	struct ast_sip_session_media_state *media_state;
+};
+
+/*! \brief Destructor for session refresh information */
+static void session_refresh_state_destroy(void *obj)
+{
+	struct session_refresh_state *state = obj;
+
+	ast_sip_session_media_state_free(state->media_state);
+	ast_free(obj);
+}
+
+/*! \brief Datastore for attaching session refresh state information */
+static const struct ast_datastore_info session_refresh_datastore = {
+	.type = "pjsip_session_refresh",
+	.destroy = session_refresh_state_destroy,
+};
+
+/*! \brief Helper function which retrieves or allocates a session refresh state information datastore */
+static struct session_refresh_state *session_refresh_state_get_or_alloc(struct ast_sip_session *session)
+{
+	RAII_VAR(struct ast_datastore *, datastore, ast_sip_session_get_datastore(session, "pjsip_session_refresh"), ao2_cleanup);
+	struct session_refresh_state *state;
+
+	/* While the datastore refcount is decremented this is operating in the serializer so it will remain valid regardless */
+	if (datastore) {
+		return datastore->data;
+	}
+
+	if (!(datastore = ast_sip_session_alloc_datastore(&session_refresh_datastore, "pjsip_session_refresh"))
+		|| !(datastore->data = ast_calloc(1, sizeof(struct session_refresh_state)))
+		|| ast_sip_session_add_datastore(session, datastore)) {
+		return NULL;
+	}
+
+	state = datastore->data;
+	state->media_state = ast_sip_session_media_state_alloc();
+	if (!state->media_state) {
+		ast_sip_session_remove_datastore(session, "pjsip_session_refresh");
+		return NULL;
+	}
+	state->media_state->topology = ast_stream_topology_clone(session->endpoint->media.topology);
+	if (!state->media_state->topology) {
+		ast_sip_session_remove_datastore(session, "pjsip_session_refresh");
+		return NULL;
+	}
+
+	datastore->data = state;
+
+	return state;
+}
+
+/*! \brief Struct used to push PJSIP_PARSE_URI function arguments to task processor */
+struct parse_uri_args {
+	const char *uri;
+	const char *type;
+	char *buf;
+	size_t buflen;
+	int ret;
+};
+
+/*! \internal \brief Taskprocessor callback that handles the PJSIP_PARSE_URI on a PJSIP thread */
+static int parse_uri_cb(void *data)
+{
+	struct parse_uri_args *args = data;
+	pj_pool_t *pool;
+	pjsip_name_addr *uri;
+	pjsip_sip_uri *sip_uri;
+	pj_str_t tmp;
+
+	args->ret = 0;
+
+	pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "ParseUri", 128, 128);
+	if (!pool) {
+		ast_log(LOG_ERROR, "Failed to allocate ParseUri endpoint pool.\n");
+		args->ret = -1;
+		return 0;
+	}
+
+	pj_strdup2_with_null(pool, &tmp, args->uri);
+	uri = (pjsip_name_addr *)pjsip_parse_uri(pool, tmp.ptr, tmp.slen, PJSIP_PARSE_URI_AS_NAMEADDR);
+	if (!uri) {
+		ast_log(LOG_WARNING, "Failed to parse URI '%s'\n", args->uri);
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		args->ret = -1;
+		return 0;
+	}
+
+	if (!strcmp(args->type, "scheme")) {
+		ast_copy_pj_str(args->buf, pjsip_uri_get_scheme(uri), args->buflen);
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		return 0;
+	} else if (!strcmp(args->type, "display")) {
+		ast_copy_pj_str(args->buf, &uri->display, args->buflen);
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		return 0;
+	}
+
+	sip_uri = pjsip_uri_get_uri(uri);
+	if (!sip_uri) {
+		ast_log(LOG_ERROR, "Failed to get an URI object for '%s'\n", args->uri);
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		args->ret = -1;
+		return 0;
+	}
+
+	if (!strcmp(args->type, "user")) {
+		ast_copy_pj_str(args->buf, &sip_uri->user, args->buflen);
+	} else if (!strcmp(args->type, "passwd")) {
+		ast_copy_pj_str(args->buf, &sip_uri->passwd, args->buflen);
+	} else if (!strcmp(args->type, "host")) {
+		ast_copy_pj_str(args->buf, &sip_uri->host, args->buflen);
+	} else if (!strcmp(args->type, "port")) {
+		snprintf(args->buf, args->buflen, "%d", sip_uri->port);
+	} else if (!strcmp(args->type, "user_param")) {
+		ast_copy_pj_str(args->buf, &sip_uri->user_param, args->buflen);
+	} else if (!strcmp(args->type, "method_param")) {
+		ast_copy_pj_str(args->buf, &sip_uri->method_param, args->buflen);
+	} else if (!strcmp(args->type, "transport_param")) {
+		ast_copy_pj_str(args->buf, &sip_uri->transport_param, args->buflen);
+	} else if (!strcmp(args->type, "ttl_param")) {
+		snprintf(args->buf, args->buflen, "%d", sip_uri->ttl_param);
+	} else if (!strcmp(args->type, "lr_param")) {
+		snprintf(args->buf, args->buflen, "%d", sip_uri->lr_param);
+	} else if (!strcmp(args->type, "maddr_param")) {
+		ast_copy_pj_str(args->buf, &sip_uri->maddr_param, args->buflen);
+	} else {
+		ast_log(AST_LOG_WARNING, "Unknown type part '%s' specified\n", args->type);
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		args->ret = -1;
+		return 0;
+	}
+
+	pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+
+	return 0;
+}
+
+int pjsip_acf_parse_uri_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t buflen)
+{
+	struct parse_uri_args func_args = { 0, };
+
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(uri_str);
+		AST_APP_ARG(type);
+	);
+
+	AST_STANDARD_APP_ARGS(args, data);
+
+	if (ast_strlen_zero(args.uri_str)) {
+		ast_log(LOG_WARNING, "An URI must be specified when using the '%s' dialplan function\n", cmd);
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.type)) {
+		ast_log(LOG_WARNING, "A type part of the URI must be specified when using the '%s' dialplan function\n", cmd);
+		return -1;
+	}
+
+	memset(buf, 0, buflen);
+
+	func_args.uri = args.uri_str;
+	func_args.type = args.type;
+	func_args.buf = buf;
+	func_args.buflen = buflen;
+	if (ast_sip_push_task_wait_serializer(NULL, parse_uri_cb, &func_args)) {
+		ast_log(LOG_WARNING, "Unable to parse URI: failed to push task\n");
+		return -1;
+	}
+
+	return func_args.ret;
+}
+
 static int media_offer_read_av(struct ast_sip_session *session, char *buf,
 			       size_t len, enum ast_media_type media_type)
 {
+	struct ast_stream_topology *topology;
 	int idx;
+	struct ast_stream *stream = NULL;
+	struct ast_format_cap *caps;
 	size_t accum = 0;
 
+	if (session->inv_session->dlg->state == PJSIP_DIALOG_STATE_ESTABLISHED) {
+		struct session_refresh_state *state;
+
+		/* As we've already answered we need to store our media state until we are ready to send it */
+		state = session_refresh_state_get_or_alloc(session);
+		if (!state) {
+			return -1;
+		}
+		topology = state->media_state->topology;
+	} else {
+		/* The session is not yet up so we are initially answering or offering */
+		if (!session->pending_media_state->topology) {
+			session->pending_media_state->topology = ast_stream_topology_clone(session->endpoint->media.topology);
+			if (!session->pending_media_state->topology) {
+				return -1;
+			}
+		}
+		topology = session->pending_media_state->topology;
+	}
+
+	/* Find the first suitable stream */
+	for (idx = 0; idx < ast_stream_topology_get_count(topology); ++idx) {
+		stream = ast_stream_topology_get_stream(topology, idx);
+
+		if (ast_stream_get_type(stream) != media_type ||
+			ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+			stream = NULL;
+			continue;
+		}
+
+		break;
+	}
+
+	/* If no suitable stream then exit early */
+	if (!stream) {
+		buf[0] = '\0';
+		return 0;
+	}
+
+	caps = ast_stream_get_formats(stream);
+
 	/* Note: buf is not terminated while the string is being built. */
-	for (idx = 0; idx < ast_format_cap_count(session->req_caps); ++idx) {
+	for (idx = 0; idx < ast_format_cap_count(caps); ++idx) {
 		struct ast_format *fmt;
 		size_t size;
 
-		fmt = ast_format_cap_get_format(session->req_caps, idx);
-		if (ast_format_get_type(fmt) != media_type) {
-			ao2_ref(fmt, -1);
-			continue;
-		}
+		fmt = ast_format_cap_get_format(caps, idx);
 
 		/* Add one for a comma or terminator */
 		size = strlen(ast_format_get_name(fmt)) + 1;
@@ -1036,9 +1305,43 @@ struct media_offer_data {
 static int media_offer_write_av(void *obj)
 {
 	struct media_offer_data *data = obj;
+	struct ast_stream_topology *topology;
+	struct ast_stream *stream;
+	struct ast_format_cap *caps;
+
+	if (data->session->inv_session->dlg->state == PJSIP_DIALOG_STATE_ESTABLISHED) {
+		struct session_refresh_state *state;
+
+		/* As we've already answered we need to store our media state until we are ready to send it */
+		state = session_refresh_state_get_or_alloc(data->session);
+		if (!state) {
+			return -1;
+		}
+		topology = state->media_state->topology;
+	} else {
+		/* The session is not yet up so we are initially answering or offering */
+		if (!data->session->pending_media_state->topology) {
+			data->session->pending_media_state->topology = ast_stream_topology_clone(data->session->endpoint->media.topology);
+			if (!data->session->pending_media_state->topology) {
+				return -1;
+			}
+		}
+		topology = data->session->pending_media_state->topology;
+	}
 
-	ast_format_cap_remove_by_type(data->session->req_caps, data->media_type);
-	ast_format_cap_update_by_allow_disallow(data->session->req_caps, data->value, 1);
+	/* XXX This method won't work when it comes time to do multistream support. The proper way to do this
+	 * will either be to
+	 * a) Alter all media streams of a particular type.
+	 * b) Change the dialplan function to be able to specify which stream to alter and alter only that
+	 * one stream
+	 */
+	stream = ast_stream_topology_get_first_stream_by_type(topology, data->media_type);
+	if (!stream) {
+		return 0;
+	}
+	caps = ast_stream_get_formats(stream);
+	ast_format_cap_remove_by_type(caps, data->media_type);
+	ast_format_cap_update_by_allow_disallow(caps, data->value, 1);
 
 	return 0;
 }
@@ -1164,7 +1467,7 @@ static int dtmf_mode_refresh_cb(void *ob
 		ast_debug(3, "Changing DTMF mode on channel %s after OFFER/ANSWER completion. Sending session refresh\n", ast_channel_name(data->session->channel));
 
 		ast_sip_session_refresh(data->session, NULL, NULL,
-			sip_session_response_cb, data->method, 1);
+			sip_session_response_cb, data->method, 1, NULL);
 	} else if (data->session->inv_session->state == PJSIP_INV_STATE_INCOMING) {
 		ast_debug(3, "Changing DTMF mode on channel %s during OFFER/ANSWER exchange. Updating SDP answer\n", ast_channel_name(data->session->channel));
 		ast_sip_session_regenerate_answer(data->session, NULL);
@@ -1176,7 +1479,7 @@ static int dtmf_mode_refresh_cb(void *ob
 int pjsip_acf_dtmf_mode_write(struct ast_channel *chan, const char *cmd, char *data, const char *value)
 {
 	struct ast_sip_channel_pvt *channel;
-	struct chan_pjsip_pvt *pjsip_pvt;
+	struct ast_sip_session_media *media;
 	int dsp_features = 0;
 	int dtmf = -1;
 	struct refresh_data rdata = {
@@ -1215,30 +1518,31 @@ int pjsip_acf_dtmf_mode_write(struct ast
 
 	channel->session->dtmf = dtmf;
 
-	pjsip_pvt = channel->pvt;
-	if (pjsip_pvt->media[SIP_MEDIA_AUDIO]  && (pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp) {
+	media = channel->session->active_media_state->default_session[AST_MEDIA_TYPE_AUDIO];
+
+	if (media && media->rtp) {
 		if (channel->session->dtmf == AST_SIP_DTMF_RFC_4733) {
-			ast_rtp_instance_set_prop((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_PROPERTY_DTMF, 1);
-			ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_RFC2833);
+			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_DTMF, 1);
+			ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_RFC2833);
 		} else if (channel->session->dtmf == AST_SIP_DTMF_INFO) {
-			ast_rtp_instance_set_prop((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_PROPERTY_DTMF, 0);
-			ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_NONE);
+			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_DTMF, 0);
+			ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_NONE);
 		} else if (channel->session->dtmf == AST_SIP_DTMF_INBAND) {
-			ast_rtp_instance_set_prop((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_PROPERTY_DTMF, 0);
-			ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_INBAND);
+			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_DTMF, 0);
+			ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_INBAND);
 		} else if (channel->session->dtmf == AST_SIP_DTMF_NONE) {
-			ast_rtp_instance_set_prop((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_PROPERTY_DTMF, 0);
-			ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_NONE);
+			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_DTMF, 0);
+			ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_NONE);
 		} else if (channel->session->dtmf == AST_SIP_DTMF_AUTO) {
-			if (ast_rtp_instance_dtmf_mode_get((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp) != AST_RTP_DTMF_MODE_RFC2833) {
+			if (ast_rtp_instance_dtmf_mode_get(media->rtp) != AST_RTP_DTMF_MODE_RFC2833) {
 				/* no RFC4733 negotiated, enable inband */
-				ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_INBAND);
+				ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_INBAND);
 			}
 		} else if (channel->session->dtmf == AST_SIP_DTMF_AUTO_INFO) {
-			ast_rtp_instance_set_prop((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_PROPERTY_DTMF, 0);
-			if (ast_rtp_instance_dtmf_mode_get((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp) == AST_RTP_DTMF_MODE_INBAND) {
+			ast_rtp_instance_set_prop(media->rtp, AST_RTP_PROPERTY_DTMF, 0);
+			if (ast_rtp_instance_dtmf_mode_get(media->rtp) == AST_RTP_DTMF_MODE_INBAND) {
 				/* if inband, switch to INFO */
-				ast_rtp_instance_dtmf_mode_set((pjsip_pvt->media[SIP_MEDIA_AUDIO])->rtp, AST_RTP_DTMF_MODE_NONE);
+				ast_rtp_instance_dtmf_mode_set(media->rtp, AST_RTP_DTMF_MODE_NONE);
 			}
 		}
 	}
@@ -1273,9 +1577,18 @@ int pjsip_acf_dtmf_mode_write(struct ast
 static int refresh_write_cb(void *obj)
 {
 	struct refresh_data *data = obj;
+	struct session_refresh_state *state;
+
+	state = session_refresh_state_get_or_alloc(data->session);
+	if (!state) {
+		return -1;
+	}
 
 	ast_sip_session_refresh(data->session, NULL, NULL,
-		sip_session_response_cb, data->method, 1);
+		sip_session_response_cb, data->method, 1, state->media_state);
+
+	state->media_state = NULL;
+	ast_sip_session_remove_datastore(data->session, "pjsip_session_refresh");
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/channels/pjsip/include/chan_pjsip.h 1:16.1.1~dfsg-1/channels/pjsip/include/chan_pjsip.h
--- 1:13.23.1~dfsg-2/channels/pjsip/include/chan_pjsip.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/pjsip/include/chan_pjsip.h	2018-12-26 18:45:13.000000000 +0000
@@ -34,25 +34,12 @@ struct transport_info_data {
 	pj_sockaddr local_addr;
 };
 
-/*!
- * \brief Positions of various media
- */
-enum sip_session_media_position {
-	/*! \brief First is audio */
-	SIP_MEDIA_AUDIO = 0,
-	/*! \brief Second is video */
-	SIP_MEDIA_VIDEO,
-	/*! \brief Last is the size for media details */
-	SIP_MEDIA_SIZE,
-};
 
 /*!
  * \brief The PJSIP channel driver pvt, stored in the \ref ast_sip_channel_pvt
  * data structure
  */
 struct chan_pjsip_pvt {
-	/*! \brief The available media sessions */
-	struct ast_sip_session_media *media[SIP_MEDIA_SIZE];
 };
 
 #endif /* _CHAN_PJSIP_HEADER */
diff -pruN 1:13.23.1~dfsg-2/channels/pjsip/include/dialplan_functions.h 1:16.1.1~dfsg-1/channels/pjsip/include/dialplan_functions.h
--- 1:13.23.1~dfsg-2/channels/pjsip/include/dialplan_functions.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/pjsip/include/dialplan_functions.h	2018-12-26 18:45:13.000000000 +0000
@@ -110,4 +110,17 @@ int pjsip_acf_session_refresh_write(stru
  */
 int pjsip_acf_dial_contacts_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len);
 
+/*!
+ * \brief PJSIP_PARSE_URI function read callback
+ * \param chan The channel the function is called on
+ * \param cmd The name of the function
+ * \param data Arguments passed to the function
+ * \param buf Out buffer that should be populated with the data
+ * \param len Size of the buffer
+ *
+ * \retval 0 on success
+ * \retval -1 on failure
+ */
+int pjsip_acf_parse_uri_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len);
+
 #endif /* _PJSIP_DIALPLAN_FUNCTIONS */
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/channels/sig_analog.c 1:16.1.1~dfsg-1/channels/sig_analog.c
--- 1:13.23.1~dfsg-2/channels/sig_analog.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sig_analog.c	2018-12-26 18:45:13.000000000 +0000
@@ -62,9 +62,6 @@
 #define POLARITY_IDLE 0
 #define POLARITY_REV    1
 #define MIN_MS_SINCE_FLASH			( (2000) )	/*!< 2000 ms */
-static int analog_matchdigittimeout = 3000;
-static int analog_gendigittimeout = 8000;
-static int analog_firstdigittimeout = 16000;
 static char analog_defaultcic[64] = "";
 static char analog_defaultozz[64] = "";
 
@@ -219,6 +216,21 @@ static int analog_have_progressdetect(st
 	return 0;
 }
 
+#define gen_analog_field_callback(type, callback_name, def_value) \
+	static type analog_get_##callback_name(struct analog_pvt *p) \
+	{ \
+		if (!analog_callbacks.get_##callback_name) { \
+			return def_value; \
+		} \
+		return analog_callbacks.get_##callback_name(p->chan_pvt); \
+	}
+
+gen_analog_field_callback(int, firstdigit_timeout, ANALOG_FIRST_DIGIT_TIMEOUT);
+gen_analog_field_callback(int, interdigit_timeout, ANALOG_INTER_DIGIT_TIMEOUT);
+gen_analog_field_callback(int, matchdigit_timeout, ANALOG_MATCH_DIGIT_TIMEOUT);
+
+#undef gen_analog_field_callback
+
 enum analog_cid_start analog_str_to_cidstart(const char *value)
 {
 	if (!strcasecmp(value, "ring")) {
@@ -1715,7 +1727,7 @@ static void *__analog_ss_thread(void *da
 	int len = 0;
 	int res;
 	int idx;
-	struct ast_callid *callid;
+	ast_callid callid;
 	RAII_VAR(struct ast_features_pickup_config *, pickup_cfg, NULL, ao2_cleanup);
 	const char *pickupexten;
 
@@ -1730,7 +1742,6 @@ static void *__analog_ss_thread(void *da
 
 	if ((callid = ast_channel_callid(chan))) {
 		ast_callid_threadassoc_add(callid);
-		ast_callid_unref(callid);
 	}
 
 	/* in the bizarre case where the channel has become a zombie before we
@@ -1887,9 +1898,9 @@ static void *__analog_ss_thread(void *da
 				dtmfbuf[len] = '\0';
 				while ((len < AST_MAX_EXTENSION-1) && ast_matchmore_extension(chan, ast_channel_context(chan), dtmfbuf, 1, p->cid_num)) {
 					if (ast_exists_extension(chan, ast_channel_context(chan), dtmfbuf, 1, p->cid_num)) {
-						timeout = analog_matchdigittimeout;
+						timeout = analog_get_matchdigit_timeout(p);
 					} else {
-						timeout = analog_gendigittimeout;
+						timeout = analog_get_interdigit_timeout(p);
 					}
 					res = ast_waitfordigit(chan, timeout);
 					if (res < 0) {
@@ -2075,7 +2086,7 @@ static void *__analog_ss_thread(void *da
 	case ANALOG_SIG_FXOGS:
 	case ANALOG_SIG_FXOKS:
 		/* Read the first digit */
-		timeout = analog_firstdigittimeout;
+		timeout = analog_get_firstdigit_timeout(p);
 		/* If starting a threeway call, never timeout on the first digit so someone
 		   can use flash-hook as a "hold" feature */
 		if (p->subs[ANALOG_SUB_THREEWAY].owner) {
@@ -2156,7 +2167,7 @@ static void *__analog_ss_thread(void *da
 				} else {
 					/* It's a match, but they just typed a digit, and there is an ambiguous match,
 					   so just set the timeout to analog_matchdigittimeout and wait some more */
-					timeout = analog_matchdigittimeout;
+					timeout = analog_get_matchdigit_timeout(p);
 				}
 			} else if (res == 0) {
 				ast_debug(1, "not enough digits (and no ambiguous match)...\n");
@@ -2175,7 +2186,7 @@ static void *__analog_ss_thread(void *da
 				}
 				len = 0;
 				memset(exten, 0, sizeof(exten));
-				timeout = analog_firstdigittimeout;
+				timeout = analog_get_firstdigit_timeout(p);
 
 			} else if (!strcmp(exten, pickupexten)) {
 				/* Scan all channels and see if there are any
@@ -2220,7 +2231,7 @@ static void *__analog_ss_thread(void *da
 				}
 				len = 0;
 				memset(exten, 0, sizeof(exten));
-				timeout = analog_firstdigittimeout;
+				timeout = analog_get_firstdigit_timeout(p);
 			} else if (p->callreturn && !strcmp(exten, "*69")) {
 				res = 0;
 				if (!ast_strlen_zero(p->lastcid_num)) {
@@ -2306,7 +2317,7 @@ static void *__analog_ss_thread(void *da
 				}
 				len = 0;
 				memset(exten, 0, sizeof(exten));
-				timeout = analog_firstdigittimeout;
+				timeout = analog_get_firstdigit_timeout(p);
 			} else if (!strcmp(exten, "*0")) {
 				struct ast_channel *nbridge = p->subs[ANALOG_SUB_THREEWAY].owner;
 				struct analog_pvt *pbridge = NULL;
@@ -2349,7 +2360,7 @@ static void *__analog_ss_thread(void *da
 				break;
 			}
 			if (!timeout) {
-				timeout = analog_gendigittimeout;
+				timeout = analog_get_interdigit_timeout(p);
 			}
 			if (len && !ast_ignore_pattern(ast_channel_context(chan), exten)) {
 				analog_play_tone(p, idx, -1);
@@ -3198,7 +3209,7 @@ static struct ast_frame *__analog_handle
 					ast_queue_control(p->subs[ANALOG_SUB_REAL].owner, AST_CONTROL_FLASH);
 					goto winkflashdone;
 				} else if (!analog_check_for_conference(p)) {
-					struct ast_callid *callid = NULL;
+					ast_callid callid = 0;
 					int callid_created;
 					char cid_num[256];
 					char cid_name[256];
@@ -3661,7 +3672,7 @@ void *analog_handle_init_event(struct an
 	int res;
 	pthread_t threadid;
 	struct ast_channel *chan;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created;
 
 	ast_debug(1, "channel (%d) - signaling (%d) - event (%s)\n",
diff -pruN 1:13.23.1~dfsg-2/channels/sig_analog.h 1:16.1.1~dfsg-1/channels/sig_analog.h
--- 1:13.23.1~dfsg-2/channels/sig_analog.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sig_analog.h	2018-12-26 18:45:13.000000000 +0000
@@ -34,6 +34,13 @@
 #define READ_SIZE 160
 #define RING_PATTERNS 3
 
+/*! \brief Default time (ms) to detect first digit */
+#define ANALOG_FIRST_DIGIT_TIMEOUT	16000
+/*! \brief Default time (ms) to detect following digits */
+#define ANALOG_INTER_DIGIT_TIMEOUT	8000
+/*! \brief Default time (ms) to wait, in case of ambiguous match */
+#define ANALOG_MATCH_DIGIT_TIMEOUT	3000
+
 /* Signalling types supported */
 enum analog_sigtype {
 	ANALOG_SIG_NONE = -1,
@@ -237,6 +244,9 @@ struct analog_callback {
 
 	const char *(* const get_orig_dialstring)(void *pvt);
 	int (* const have_progressdetect)(void *pvt);
+	int (* const get_firstdigit_timeout)(void *pvt);
+	int (* const get_interdigit_timeout)(void *pvt);
+	int (* const get_matchdigit_timeout)(void *pvt);
 };
 
 /*! Global analog callbacks to the upper layer. */
diff -pruN 1:13.23.1~dfsg-2/channels/sig_pri.c 1:16.1.1~dfsg-1/channels/sig_pri.c
--- 1:13.23.1~dfsg-2/channels/sig_pri.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sig_pri.c	2018-12-26 18:45:13.000000000 +0000
@@ -2045,11 +2045,10 @@ static void *do_idle_thread(void *v_pvt)
 	int timeout_ms = 30000;
 	int ms;
 	struct timeval start;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	if ((callid = ast_channel_callid(chan))) {
 		ast_callid_threadassoc_add(callid);
-		callid = ast_callid_unref(callid);
 	}
 
 	ast_verb(3, "Initiating idle call on channel %s\n", ast_channel_name(chan));
@@ -2104,7 +2103,7 @@ static void *pri_ss_thread(void *data)
 	int res;
 	int len;
 	int timeout;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	if (!chan) {
 		/* We lost the owner before we could get started. */
@@ -2113,7 +2112,6 @@ static void *pri_ss_thread(void *data)
 
 	if ((callid = ast_channel_callid(chan))) {
 		ast_callid_threadassoc_add(callid);
-		ast_callid_unref(callid);
 	}
 
 	/*
@@ -5347,13 +5345,12 @@ static void sig_pri_moh_fsm_event(struct
  * \internal
  * \brief Set callid threadstorage for the pri_dchannel thread when a new call is created
  *
- * \return A new callid which has been bound to threadstorage. The return must be
- *         unreffed and the threadstorage should be unbound when the pri_dchannel
- *         primary loop wraps.
+ * \return A new callid which has been bound to threadstorage. The threadstorage
+ *         should be unbound when the pri_dchannel primary loop wraps.
  */
-static struct ast_callid *func_pri_dchannel_new_callid(void)
+static ast_callid func_pri_dchannel_new_callid(void)
 {
-	struct ast_callid *callid = ast_create_callid();
+	ast_callid callid = ast_create_callid();
 
 	if (callid) {
 		ast_callid_threadassoc_add(callid);
@@ -5372,20 +5369,18 @@ static struct ast_callid *func_pri_dchan
  * \note Assumes the pri->lock is already obtained.
  * \note Assumes the sig_pri_lock_private(pri->pvts[chanpos]) is already obtained.
  *
- * \return a reference to the callid bound to the channel which has also
- *         been bound to threadstorage if it exists. If this returns non-NULL,
- *         the callid must be unreffed and the threadstorage should be unbound
- *         when the pri_dchannel primary loop wraps.
+ * \return The callid which has also been bound to threadstorage if it exists.
+ *         The threadstorage should be unbound when the pri_dchannel primary loop wraps.
  */
-static struct ast_callid *func_pri_dchannel_chanpos_callid(struct sig_pri_span *pri, int chanpos)
+static ast_callid func_pri_dchannel_chanpos_callid(struct sig_pri_span *pri, int chanpos)
 {
 	if (chanpos < 0) {
-		return NULL;
+		return 0;
 	}
 
 	sig_pri_lock_owner(pri, chanpos);
 	if (pri->pvts[chanpos]->owner) {
-		struct ast_callid *callid;
+		ast_callid callid;
 		callid = ast_channel_callid(pri->pvts[chanpos]->owner);
 		ast_channel_unlock(pri->pvts[chanpos]->owner);
 		if (callid) {
@@ -5394,7 +5389,7 @@ static struct ast_callid *func_pri_dchan
 		}
 	}
 
-	return NULL;
+	return 0;
 }
 
 #if defined(HAVE_PRI_CALL_HOLD)
@@ -5417,7 +5412,7 @@ static int sig_pri_handle_hold(struct si
 	int chanpos_old;
 	int chanpos_new;
 	struct ast_channel *owner;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 
 	chanpos_old = pri_find_principle_by_call(pri, ev->hold.call);
 	if (chanpos_old < 0) {
@@ -5477,7 +5472,6 @@ done_with_private:;
 	}
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 
@@ -5501,7 +5495,7 @@ done_with_private:;
 static void sig_pri_handle_hold_ack(struct sig_pri_span *pri, pri_event *ev)
 {
 	int chanpos;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	/*
 	 * We were successfully put on hold by the remote party
@@ -5533,7 +5527,6 @@ static void sig_pri_handle_hold_ack(stru
 	sig_pri_span_devstate_changed(pri);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -5555,7 +5548,7 @@ static void sig_pri_handle_hold_ack(stru
 static void sig_pri_handle_hold_rej(struct sig_pri_span *pri, pri_event *ev)
 {
 	int chanpos;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	chanpos = pri_find_principle(pri, ev->hold_rej.channel, ev->hold_rej.call);
 	if (chanpos < 0) {
@@ -5583,7 +5576,6 @@ static void sig_pri_handle_hold_rej(stru
 	sig_pri_unlock_private(pri->pvts[chanpos]);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -5605,7 +5597,7 @@ static void sig_pri_handle_hold_rej(stru
 static void sig_pri_handle_retrieve(struct sig_pri_span *pri, pri_event *ev)
 {
 	int chanpos;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	if (!(ev->retrieve.channel & PRI_HELD_CALL)) {
 		/* The call is not currently held. */
@@ -5658,7 +5650,6 @@ static void sig_pri_handle_retrieve(stru
 	sig_pri_span_devstate_changed(pri);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -5680,7 +5671,7 @@ static void sig_pri_handle_retrieve(stru
 static void sig_pri_handle_retrieve_ack(struct sig_pri_span *pri, pri_event *ev)
 {
 	int chanpos;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	chanpos = pri_find_fixup_principle(pri, ev->retrieve_ack.channel,
 		ev->retrieve_ack.call);
@@ -5699,7 +5690,6 @@ static void sig_pri_handle_retrieve_ack(
 	sig_pri_span_devstate_changed(pri);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -5721,7 +5711,7 @@ static void sig_pri_handle_retrieve_ack(
 static void sig_pri_handle_retrieve_rej(struct sig_pri_span *pri, pri_event *ev)
 {
 	int chanpos;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	chanpos = pri_find_principle(pri, ev->retrieve_rej.channel, ev->retrieve_rej.call);
 	if (chanpos < 0) {
@@ -5750,7 +5740,6 @@ static void sig_pri_handle_retrieve_rej(
 	sig_pri_unlock_private(pri->pvts[chanpos]);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -5870,7 +5859,7 @@ static void sig_pri_handle_setup(struct
 	int need_dialtone;
 	enum sig_pri_law law;
 	int chanpos = -1;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	struct ast_channel *c;
 	char plancallingnum[AST_MAX_EXTENSION];
 	char plancallingani[AST_MAX_EXTENSION];
@@ -6224,7 +6213,6 @@ static void sig_pri_handle_setup(struct
 
 setup_exit:;
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 }
@@ -6272,7 +6260,7 @@ static void *pri_dchannel(void *vpri)
 			ast_log(LOG_WARNING, "Idle dial string '%s' lacks '@context'\n", pri->idleext);
 	}
 	for (;;) {
-		struct ast_callid *callid = NULL;
+		ast_callid callid = 0;
 
 		for (i = 0; i < SIG_PRI_NUM_DCHANS; i++) {
 			if (!pri->dchans[i])
@@ -7613,9 +7601,8 @@ static void *pri_dchannel(void *vpri)
 				break;
 			}
 
-			/* If a callid was set, we need to deref it and remove it from thread storage. */
+			/* If a callid was set, we need to remove it from thread storage. */
 			if (callid) {
-				callid = ast_callid_unref(callid);
 				ast_callid_threadassoc_remove();
 			}
 		}
@@ -9143,6 +9130,9 @@ int sig_pri_start_pri(struct sig_pri_spa
 		if (!pri->mbox[i].sub) {
 			ast_log(LOG_ERROR, "%s span %d could not subscribe to MWI events for %s(%s).\n",
 				sig_pri_cc_type_name, pri->span, pri->mbox[i].vm_box, mbox_id);
+		} else {
+			stasis_subscription_accept_message_type(pri->mbox[i].sub, ast_mwi_state_type());
+			stasis_subscription_set_filter(pri->mbox[i].sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 #if defined(HAVE_PRI_MWI_V2)
 		if (ast_strlen_zero(pri->mbox[i].vm_number)) {
@@ -10143,8 +10133,8 @@ int sig_pri_load(const char *cc_type_nam
 
 #if defined(HAVE_PRI_CCSS)
 	sig_pri_cc_type_name = cc_type_name;
-	sig_pri_cc_monitors = ao2_container_alloc(37, sig_pri_cc_monitor_instance_hash_fn,
-		sig_pri_cc_monitor_instance_cmp_fn);
+	sig_pri_cc_monitors = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		sig_pri_cc_monitor_instance_hash_fn, NULL, sig_pri_cc_monitor_instance_cmp_fn);
 	if (!sig_pri_cc_monitors) {
 		return -1;
 	}
diff -pruN 1:13.23.1~dfsg-2/channels/sig_ss7.c 1:16.1.1~dfsg-1/channels/sig_ss7.c
--- 1:13.23.1~dfsg-2/channels/sig_ss7.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sig_ss7.c	2018-12-26 18:45:13.000000000 +0000
@@ -976,7 +976,7 @@ static void ss7_start_call(struct sig_ss
 	struct ast_channel *c;
 	char tmp[256];
 	char *strp;
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	int callid_created = ast_callid_threadstorage_auto(&callid);
 
 	if (!(linkset->flags & LINKSET_FLAG_EXPLICITACM)) {
@@ -1315,14 +1315,13 @@ static int ss7_pres_scr2cid_pres(char pr
  * \note Assumes the ss7->lock is already obtained.
  * \note Assumes the sig_ss7_lock_private(ss7->pvts[chanpos]) is already obtained.
  *
- * \return a reference to the callid bound to the channel which has also
- *         been bound to threadstorage if it exists. If this returns non-NULL,
- *         the callid must be unreffed and the threadstorage should be unbound
+ * \return The callid bound to the channel which has also been bound to threadstorage
+ *         if it exists. If this returns non-zero, the threadstorage should be unbound
  *         before the while loop wraps in ss7_linkset.
  */
-static struct ast_callid *func_ss7_linkset_callid(struct sig_ss7_linkset *linkset, int chanpos)
+static ast_callid func_ss7_linkset_callid(struct sig_ss7_linkset *linkset, int chanpos)
 {
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	sig_ss7_lock_owner(linkset, chanpos);
 	if (linkset->pvts[chanpos]->owner) {
 		callid = ast_channel_callid(linkset->pvts[chanpos]->owner);
@@ -1460,7 +1459,7 @@ void *ss7_linkset(void *data)
 		}
 
 		while ((e = ss7_check_event(ss7))) {
-			struct ast_callid *callid = NULL;
+			ast_callid callid = 0;
 			int chanpos = -1;
 			char cause_str[30];
 
@@ -2342,7 +2341,6 @@ void *ss7_linkset(void *data)
 
 			/* Call ID stuff needs to be cleaned up here */
 			if (callid) {
-				callid = ast_callid_unref(callid);
 				ast_callid_threadassoc_remove();
 			}
 		}
diff -pruN 1:13.23.1~dfsg-2/channels/sip/config_parser.c 1:16.1.1~dfsg-1/channels/sip/config_parser.c
--- 1:13.23.1~dfsg-2/channels/sip/config_parser.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/config_parser.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "include/sip.h"
 #include "include/config_parser.h"
 #include "include/sip_utils.h"
diff -pruN 1:13.23.1~dfsg-2/channels/sip/dialplan_functions.c 1:16.1.1~dfsg-1/channels/sip/dialplan_functions.c
--- 1:13.23.1~dfsg-2/channels/sip/dialplan_functions.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/dialplan_functions.c	2018-12-26 18:45:13.000000000 +0000
@@ -110,8 +110,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 
 #include "asterisk/channel.h"
@@ -426,9 +424,9 @@ AST_TEST_DEFINE(test_sip_rtpqos_1)
 		break;
 	}
 
-	ast_rtp_engine_register2(&test_engine, NULL);
+	ast_rtp_engine_register(&test_engine);
 	/* Have to associate this with a SIP pvt and an ast_channel */
-	if (!(p = sip_alloc(NULL, NULL, 0, SIP_NOTIFY, NULL, NULL))) {
+	if (!(p = sip_alloc(0, NULL, 0, SIP_NOTIFY, NULL, 0))) {
 		res = AST_TEST_NOT_RUN;
 		goto done;
 	}
diff -pruN 1:13.23.1~dfsg-2/channels/sip/include/dialog.h 1:16.1.1~dfsg-1/channels/sip/include/dialog.h
--- 1:13.23.1~dfsg-2/channels/sip/include/dialog.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/include/dialog.h	2018-12-26 18:45:13.000000000 +0000
@@ -33,7 +33,7 @@
 #define dialog_unref(dialog, tag) ({ ao2_t_cleanup(dialog, tag); (NULL); })
 
 struct sip_pvt *__sip_alloc(ast_string_field callid, struct ast_sockaddr *sin,
-				 int useglobal_nat, const int intended_method, struct sip_request *req, struct ast_callid *logger_callid,
+				 int useglobal_nat, const int intended_method, struct sip_request *req, ast_callid logger_callid,
 				 const char *file, int line, const char *func);
 
 #define sip_alloc(callid, addr, useglobal_nat, intended_method, req, logger_callid) \
diff -pruN 1:13.23.1~dfsg-2/channels/sip/include/sip.h 1:16.1.1~dfsg-1/channels/sip/include/sip.h
--- 1:13.23.1~dfsg-2/channels/sip/include/sip.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/include/sip.h	2018-12-26 18:45:13.000000000 +0000
@@ -211,7 +211,7 @@
 #define DEFAULT_ALLOW_EXT_DOM  TRUE    /*!< Allow external domains */
 #define DEFAULT_REALM          "asterisk" /*!< Realm for HTTP digest authentication */
 #define DEFAULT_DOMAINSASREALM FALSE    /*!< Use the domain option to guess the realm for registration and invite requests */
-#define DEFAULT_NOTIFYRINGING  TRUE     /*!< Notify devicestate system on ringing state */
+#define DEFAULT_NOTIFYRINGING  NOTIFYRINGING_ENABLED /*!< Notify devicestate system on ringing state */
 #define DEFAULT_NOTIFYCID      DISABLED	/*!< Include CID with ringing notifications */
 #define DEFAULT_PEDANTIC       TRUE     /*!< Follow SIP standards for dialog matching */
 #define DEFAULT_AUTOCREATEPEER AUTOPEERS_DISABLED    /*!< Don't create peers automagically */
@@ -622,6 +622,13 @@ enum sipmethod {
 	SIP_PING,       /*!< Not supported at all, no standard but still implemented out there */
 };
 
+/*! \brief Setting for the 'notifyringing' option, see sip.conf.sample for details. */
+enum notifyringing_setting {
+	NOTIFYRINGING_DISABLED = 0,
+	NOTIFYRINGING_ENABLED = 1,
+	NOTIFYRINGING_NOTINUSE = 2,
+};
+
 /*! \brief Settings for the 'notifycid' option, see sip.conf.sample for details. */
 enum notifycid_setting {
 	DISABLED       = 0,
@@ -991,7 +998,7 @@ struct sip_msg_hdr {
 struct sip_pvt {
 	struct sip_pvt *next;                   /*!< Next dialog in chain */
 	enum invitestates invitestate;          /*!< Track state of SIP_INVITEs */
-	struct ast_callid *logger_callid;		/*!< Identifier for call used in log messages */
+	ast_callid logger_callid;               /*!< Identifier for call used in log messages */
 	int method;                             /*!< SIP method that opened this dialog */
 	AST_DECLARE_STRING_FIELDS(
 		AST_STRING_FIELD(callid);       /*!< Global CallID */
diff -pruN 1:13.23.1~dfsg-2/channels/sip/reqresp_parser.c 1:16.1.1~dfsg-1/channels/sip/reqresp_parser.c
--- 1:13.23.1~dfsg-2/channels/sip/reqresp_parser.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/reqresp_parser.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "include/sip.h"
 #include "include/sip_utils.h"
 #include "include/reqresp_parser.h"
diff -pruN 1:13.23.1~dfsg-2/channels/sip/route.c 1:16.1.1~dfsg-1/channels/sip/route.c
--- 1:13.23.1~dfsg-2/channels/sip/route.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/route.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 
 #include "include/route.h"
diff -pruN 1:13.23.1~dfsg-2/channels/sip/security_events.c 1:16.1.1~dfsg-1/channels/sip/security_events.c
--- 1:13.23.1~dfsg-2/channels/sip/security_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/sip/security_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "include/sip.h"
 #include "include/security_events.h"
 
diff -pruN 1:13.23.1~dfsg-2/channels/vgrabbers.c 1:16.1.1~dfsg-1/channels/vgrabbers.c
--- 1:13.23.1~dfsg-2/channels/vgrabbers.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/channels/vgrabbers.c	2018-12-26 18:45:13.000000000 +0000
@@ -49,7 +49,6 @@
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include <sys/ioctl.h>
 #include "asterisk/file.h"
 #include "asterisk/utils.h"	/* ast_calloc */
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_adpcm.c 1:16.1.1~dfsg-1/codecs/codec_adpcm.c
--- 1:13.23.1~dfsg-2/codecs/codec_adpcm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_adpcm.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/module.h"
@@ -355,7 +353,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Adaptive Differential PCM Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_alaw.c 1:16.1.1~dfsg-1/codecs/codec_alaw.c
--- 1:13.23.1~dfsg-2/codecs/codec_alaw.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_alaw.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/config.h"
 #include "asterisk/translate.h"
@@ -139,7 +137,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "A-law Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_a_mu.c 1:16.1.1~dfsg-1/codecs/codec_a_mu.c
--- 1:13.23.1~dfsg-2/codecs/codec_a_mu.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_a_mu.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/translate.h"
 #include "asterisk/alaw.h"
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_codec2.c 1:16.1.1~dfsg-1/codecs/codec_codec2.c
--- 1:13.23.1~dfsg-2/codecs/codec_codec2.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_codec2.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,222 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Alexander Traud
+ *
+ * Alexander Traud <pabstraud@compuserve.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Translate between signed linear and Codec 2
+ *
+ * \author Alexander Traud <pabstraud@compuserve.com>
+ *
+ * \note http://www.rowetel.com/codec2.html
+ *
+ * \ingroup codecs
+ */
+
+/*** MODULEINFO
+	<depend>codec2</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/codec.h"             /* for AST_MEDIA_TYPE_AUDIO       */
+#include "asterisk/frame.h"             /* for ast_frame                  */
+#include "asterisk/linkedlists.h"       /* for AST_LIST_NEXT, etc         */
+#include "asterisk/logger.h"            /* for ast_log, etc               */
+#include "asterisk/module.h"
+#include "asterisk/rtp_engine.h"        /* ast_rtp_engine_(un)load_format */
+#include "asterisk/translate.h"         /* for ast_trans_pvt, etc         */
+
+#include <codec2/codec2.h>
+
+#define BUFFER_SAMPLES    8000
+#define CODEC2_SAMPLES    160  /* consider codec2_samples_per_frame(.) */
+#define CODEC2_FRAME_LEN  6    /* consider codec2_bits_per_frame(.)    */
+
+/* Sample frame data */
+#include "asterisk/slin.h"
+#include "ex_codec2.h"
+
+struct codec2_translator_pvt {
+	struct CODEC2 *state; /* May be encoder or decoder */
+	int16_t buf[BUFFER_SAMPLES];
+};
+
+static int codec2_new(struct ast_trans_pvt *pvt)
+{
+	struct codec2_translator_pvt *tmp = pvt->pvt;
+
+	tmp->state = codec2_create(CODEC2_MODE_2400);
+
+	if (!tmp->state) {
+		ast_log(LOG_ERROR, "Error creating Codec 2 conversion\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+/*! \brief decode and store in outbuf. */
+static int codec2tolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
+{
+	struct codec2_translator_pvt *tmp = pvt->pvt;
+	int x;
+
+	for (x = 0; x < f->datalen; x += CODEC2_FRAME_LEN) {
+		unsigned char *src = f->data.ptr + x;
+		int16_t *dst = pvt->outbuf.i16 + pvt->samples;
+
+		codec2_decode(tmp->state, dst, src);
+
+		pvt->samples += CODEC2_SAMPLES;
+		pvt->datalen += CODEC2_SAMPLES * 2;
+	}
+
+	return 0;
+}
+
+/*! \brief store samples into working buffer for later decode */
+static int lintocodec2_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
+{
+	struct codec2_translator_pvt *tmp = pvt->pvt;
+
+	memcpy(tmp->buf + pvt->samples, f->data.ptr, f->datalen);
+	pvt->samples += f->samples;
+
+	return 0;
+}
+
+/*! \brief encode and produce a frame */
+static struct ast_frame *lintocodec2_frameout(struct ast_trans_pvt *pvt)
+{
+	struct codec2_translator_pvt *tmp = pvt->pvt;
+	struct ast_frame *result = NULL;
+	struct ast_frame *last = NULL;
+	int samples = 0; /* output samples */
+
+	while (pvt->samples >= CODEC2_SAMPLES) {
+		struct ast_frame *current;
+
+		/* Encode a frame of data */
+		codec2_encode(tmp->state, pvt->outbuf.uc, tmp->buf + samples);
+
+		samples += CODEC2_SAMPLES;
+		pvt->samples -= CODEC2_SAMPLES;
+
+		current = ast_trans_frameout(pvt, CODEC2_FRAME_LEN, CODEC2_SAMPLES);
+
+		if (!current) {
+			continue;
+		} else if (last) {
+			AST_LIST_NEXT(last, frame_list) = current;
+		} else {
+			result = current;
+		}
+		last = current;
+	}
+
+	/* Move the data at the end of the buffer to the front */
+	if (samples) {
+		memmove(tmp->buf, tmp->buf + samples, pvt->samples * 2);
+	}
+
+	return result;
+}
+
+static void codec2_destroy_stuff(struct ast_trans_pvt *pvt)
+{
+	struct codec2_translator_pvt *tmp = pvt->pvt;
+
+	if (tmp->state) {
+		codec2_destroy(tmp->state);
+	}
+}
+
+static struct ast_translator codec2tolin = {
+	.name = "codec2tolin",
+	.src_codec = {
+		.name = "codec2",
+		.type = AST_MEDIA_TYPE_AUDIO,
+		.sample_rate = 8000,
+	},
+	.dst_codec = {
+		.name = "slin",
+		.type = AST_MEDIA_TYPE_AUDIO,
+		.sample_rate = 8000,
+	},
+	.format = "slin",
+	.newpvt = codec2_new,
+	.framein = codec2tolin_framein,
+	.destroy = codec2_destroy_stuff,
+	.sample = codec2_sample,
+	.desc_size = sizeof(struct codec2_translator_pvt),
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = BUFFER_SAMPLES * 2,
+};
+
+static struct ast_translator lintocodec2 = {
+	.name = "lintocodec2",
+	.src_codec = {
+		.name = "slin",
+		.type = AST_MEDIA_TYPE_AUDIO,
+		.sample_rate = 8000,
+	},
+	.dst_codec = {
+		.name = "codec2",
+		.type = AST_MEDIA_TYPE_AUDIO,
+		.sample_rate = 8000,
+	},
+	.format = "codec2",
+	.newpvt = codec2_new,
+	.framein = lintocodec2_framein,
+	.frameout = lintocodec2_frameout,
+	.destroy = codec2_destroy_stuff,
+	.sample = slin8_sample,
+	.desc_size = sizeof(struct codec2_translator_pvt),
+	.buffer_samples = BUFFER_SAMPLES,
+	.buf_size = (BUFFER_SAMPLES * CODEC2_FRAME_LEN + CODEC2_SAMPLES - 1) / CODEC2_SAMPLES,
+};
+
+static int unload_module(void)
+{
+	int res = 0;
+
+	res |= ast_rtp_engine_unload_format(ast_format_codec2);
+	res |= ast_unregister_translator(&lintocodec2);
+	res |= ast_unregister_translator(&codec2tolin);
+
+	return res;
+}
+
+static int load_module(void)
+{
+	int res = 0;
+
+	res |= ast_register_translator(&codec2tolin);
+	res |= ast_register_translator(&lintocodec2);
+	res |= ast_rtp_engine_load_format(ast_format_codec2);
+
+	if (res) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Codec 2 Coder/Decoder");
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_dahdi.c 1:16.1.1~dfsg-1/codecs/codec_dahdi.c
--- 1:13.23.1~dfsg-2/codecs/codec_dahdi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_dahdi.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 #include "asterisk.h"
 #include <stdbool.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <poll.h>
 #include <fcntl.h>
 #include <netinet/in.h>
@@ -868,8 +866,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Generic DAHDI Transcoder Codec Translator",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_g722.c 1:16.1.1~dfsg-1/codecs/codec_g722.c
--- 1:13.23.1~dfsg-2/codecs/codec_g722.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_g722.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/linkedlists.h"
 #include "asterisk/module.h"
 #include "asterisk/config.h"
@@ -250,7 +248,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ITU G.722-64kbps G722 Transcoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_g726.c 1:16.1.1~dfsg-1/codecs/codec_g726.c
--- 1:13.23.1~dfsg-2/codecs/codec_g726.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_g726.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/module.h"
@@ -899,7 +897,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ITU G.726-32kbps G726 Transcoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_gsm.c 1:16.1.1~dfsg-1/codecs/codec_gsm.c
--- 1:13.23.1~dfsg-2/codecs/codec_gsm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_gsm.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/translate.h"
 #include "asterisk/config.h"
 #include "asterisk/module.h"
@@ -248,7 +246,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "GSM Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_ilbc.c 1:16.1.1~dfsg-1/codecs/codec_ilbc.c
--- 1:13.23.1~dfsg-2/codecs/codec_ilbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_ilbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,12 +32,13 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include "asterisk/translate.h"
+#include "asterisk/codec.h"             /* for AST_MEDIA_TYPE_AUDIO */
+#include "asterisk/format.h"            /* for ast_format_get_attribute_data */
+#include "asterisk/frame.h"             /* for ast_frame, etc */
+#include "asterisk/linkedlists.h"       /* for AST_LIST_NEXT, etc */
+#include "asterisk/logger.h"            /* for ast_log, ast_debug, etc */
 #include "asterisk/module.h"
-#include "asterisk/utils.h"
-#include "asterisk/linkedlists.h"
+#include "asterisk/translate.h"         /* for ast_trans_pvt, etc */
 
 #ifdef ILBC_WEBRTC
 #include <ilbc.h>
@@ -52,13 +53,10 @@ typedef float         ilbc_block;
 #define BUF_TYPE uc
 #endif
 
-#define USE_ILBC_ENHANCER	0
-#define ILBC_MS 			30
-/* #define ILBC_MS			20 */
-
-#define	ILBC_FRAME_LEN	50	/* apparently... */
-#define	ILBC_SAMPLES	240	/* 30ms at 8000 hz */
-#define	BUFFER_SAMPLES	8000
+#include "asterisk/ilbc.h"
+
+#define USE_ILBC_ENHANCER 0
+#define BUFFER_SAMPLES    8000
 
 /* Sample frame data */
 #include "asterisk/slin.h"
@@ -69,13 +67,16 @@ struct ilbc_coder_pvt {
 	iLBC_Dec_Inst_t dec;
 	/* Enough to store a full second */
 	int16_t buf[BUFFER_SAMPLES];
+	int16_t inited;
 };
 
 static int lintoilbc_new(struct ast_trans_pvt *pvt)
 {
 	struct ilbc_coder_pvt *tmp = pvt->pvt;
+	struct ilbc_attr *attr = pvt->explicit_dst ? ast_format_get_attribute_data(pvt->explicit_dst) : NULL;
+	const unsigned int mode = attr ? attr->mode : 30;
 
-	initEncode(&tmp->enc, ILBC_MS);
+	initEncode(&tmp->enc, mode);
 
 	return 0;
 }
@@ -84,7 +85,7 @@ static int ilbctolin_new(struct ast_tran
 {
 	struct ilbc_coder_pvt *tmp = pvt->pvt;
 
-	initDecode(&tmp->dec, ILBC_MS, USE_ILBC_ENHANCER);
+	tmp->inited = 0; /* we do not know the iLBC mode, yet */
 
 	return 0;
 }
@@ -93,13 +94,19 @@ static int ilbctolin_new(struct ast_tran
 static int ilbctolin_framein(struct ast_trans_pvt *pvt, struct ast_frame *f)
 {
 	struct ilbc_coder_pvt *tmp = pvt->pvt;
+	struct ilbc_attr *attr = ast_format_get_attribute_data(f->subclass.format);
+	const unsigned int mode = attr ? attr->mode : 30;
+	const unsigned int sample_rate = pvt->t->dst_codec.sample_rate;
+	const unsigned int samples_per_frame = mode * sample_rate / 1000;
+	const unsigned int octets_per_frame = (mode == 20) ? 38 : 50;
+
 	int plc_mode = 1; /* 1 = normal data, 0 = plc */
 	/* Assuming there's space left, decode into the current buffer at
 	   the tail location.  Read in as many frames as there are */
 	int x,i;
 	int datalen = f->datalen;
 	int16_t *dst = pvt->outbuf.i16;
-	ilbc_block tmpf[ILBC_SAMPLES];
+	ilbc_block tmpf[samples_per_frame];
 
 	if (!f->data.ptr && datalen) {
 		ast_debug(1, "issue 16070, ILIB ERROR. data = NULL datalen = %d src = %s\n", datalen, f->src ? f->src : "no src set");
@@ -108,27 +115,32 @@ static int ilbctolin_framein(struct ast_
 	}
 
 	if (datalen == 0) { /* native PLC, set fake datalen and clear plc_mode */
-		datalen = ILBC_FRAME_LEN;
-		f->samples = ILBC_SAMPLES;
+		datalen = octets_per_frame;
+		f->samples = samples_per_frame;
 		plc_mode = 0;	/* do native plc */
-		pvt->samples += ILBC_SAMPLES;
+		pvt->samples += samples_per_frame;
 	}
 
-	if (datalen % ILBC_FRAME_LEN) {
-		ast_log(LOG_WARNING, "Huh?  An ilbc frame that isn't a multiple of 50 bytes long from %s (%d)?\n", f->src, datalen);
+	if (datalen % octets_per_frame) {
+		ast_log(LOG_WARNING, "Huh?  An ilbc frame that isn't a multiple of %u bytes long from %s (%d)?\n", octets_per_frame, f->src, datalen);
 		return -1;
 	}
 
-	for (x=0; x < datalen ; x += ILBC_FRAME_LEN) {
-		if (pvt->samples + ILBC_SAMPLES > BUFFER_SAMPLES) {
+	if (!tmp->inited) {
+		initDecode(&tmp->dec, mode, USE_ILBC_ENHANCER);
+		tmp->inited = 1;
+	}
+
+	for (x = 0; x < datalen; x += octets_per_frame) {
+		if (pvt->samples + samples_per_frame > BUFFER_SAMPLES) {
 			ast_log(LOG_WARNING, "Out of buffer space\n");
 			return -1;
 		}
 		iLBC_decode(tmpf, plc_mode ? f->data.ptr + x : NULL, &tmp->dec, plc_mode);
-		for ( i=0; i < ILBC_SAMPLES; i++)
+		for (i = 0; i < samples_per_frame; i++)
 			dst[pvt->samples + i] = tmpf[i];
-		pvt->samples += ILBC_SAMPLES;
-		pvt->datalen += 2*ILBC_SAMPLES;
+		pvt->samples += samples_per_frame;
+		pvt->datalen += samples_per_frame * 2;
 	}
 	return 0;
 }
@@ -155,20 +167,26 @@ static struct ast_frame *lintoilbc_frame
 	struct ast_frame *last = NULL;
 	int samples = 0; /* output samples */
 
-	while (pvt->samples >= ILBC_SAMPLES) {
+	struct ilbc_attr *attr = ast_format_get_attribute_data(pvt->f.subclass.format);
+	const unsigned int mode = attr ? attr->mode : 30;
+	const unsigned int sample_rate = pvt->t->dst_codec.sample_rate;
+	const unsigned int samples_per_frame = mode * sample_rate / 1000;
+	const unsigned int octets_per_frame = (mode == 20) ? 38 : 50;
+
+	while (pvt->samples >= samples_per_frame) {
 		struct ast_frame *current;
-		ilbc_block tmpf[ILBC_SAMPLES];
+		ilbc_block tmpf[samples_per_frame];
 		int i;
 
 		/* Encode a frame of data */
-		for (i = 0 ; i < ILBC_SAMPLES ; i++)
+		for (i = 0; i < samples_per_frame; i++)
 			tmpf[i] = tmp->buf[samples + i];
 		iLBC_encode((ilbc_bytes *) pvt->outbuf.BUF_TYPE, tmpf, &tmp->enc);
 
-		samples += ILBC_SAMPLES;
-		pvt->samples -= ILBC_SAMPLES;
+		samples += samples_per_frame;
+		pvt->samples -= samples_per_frame;
 
-		current = ast_trans_frameout(pvt, ILBC_FRAME_LEN, ILBC_SAMPLES);
+		current = ast_trans_frameout(pvt, octets_per_frame, samples_per_frame);
 		if (!current) {
 			continue;
 		} else if (last) {
@@ -226,7 +244,8 @@ static struct ast_translator lintoilbc =
 	.frameout = lintoilbc_frameout,
 	.sample = slin8_sample,
 	.desc_size = sizeof(struct ilbc_coder_pvt),
-	.buf_size = (BUFFER_SAMPLES * ILBC_FRAME_LEN + ILBC_SAMPLES - 1) / ILBC_SAMPLES,
+	/* frame len (38 bytes), frame size (160 samples), ceil (+ 160 - 1) */
+	.buf_size = (BUFFER_SAMPLES * 38 + 160 - 1) / 160,
 };
 
 static int unload_module(void)
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_lpc10.c 1:16.1.1~dfsg-1/codecs/codec_lpc10.c
--- 1:13.23.1~dfsg-2/codecs/codec_lpc10.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_lpc10.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/translate.h"
 #include "asterisk/config.h"
 #include "asterisk/module.h"
@@ -281,7 +279,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "LPC10 2.4kbps Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_resample.c 1:16.1.1~dfsg-1/codecs/codec_resample.c
--- 1:13.23.1~dfsg-2/codecs/codec_resample.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_resample.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 #include "asterisk.h"
 #include "speex/speex_resampler.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/translate.h"
 #include "asterisk/slin.h"
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_speex.c 1:16.1.1~dfsg-1/codecs/codec_speex.c
--- 1:13.23.1~dfsg-2/codecs/codec_speex.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_speex.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <speex/speex.h>
 
 /* We require a post 1.1.8 version of Speex to enable preprocessing
@@ -57,6 +55,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/frame.h"
 #include "asterisk/linkedlists.h"
 
+/* For struct ast_rtp_rtcp_report and struct ast_rtp_rtcp_report_block */
+#include "asterisk/rtp_engine.h"
+
 /* codec variables */
 static int quality = 3;
 static int complexity = 2;
@@ -66,6 +67,7 @@ static int vbr = 0;
 static float vbr_quality = 4;
 static int abr = 0;
 static int dtx = 0;	/* set to 1 to enable silence detection */
+static int exp_rtcp_fb = 0;	/* set to 1 to use experimental RTCP feedback for changing bitrate */
 
 static int preproc = 0;
 static int pp_vad = 0;
@@ -93,6 +95,11 @@ struct speex_coder_pvt {
 	SpeexBits bits;
 	int framesize;
 	int silent_state;
+
+	int fraction_lost;
+	int quality;
+	int default_quality;
+
 #ifdef _SPEEX_TYPES_H
 	SpeexPreprocessState *pp;
 	spx_int16_t buf[BUFFER_SAMPLES];
@@ -139,6 +146,11 @@ static int speex_encoder_construct(struc
 		speex_encoder_ctl(tmp->speex, SPEEX_SET_DTX, &dtx);
 	tmp->silent_state = 0;
 
+	tmp->fraction_lost = 0;
+	tmp->default_quality = vbr ? vbr_quality : quality;
+	tmp->quality = tmp->default_quality;
+	ast_debug(3, "Default quality (%s): %d\n", vbr ? "vbr" : "cbr", tmp->default_quality);
+
 	return 0;
 }
 
@@ -344,6 +356,74 @@ static struct ast_frame *lintospeex_fram
 	return result;
 }
 
+/*! \brief handle incoming RTCP feedback and possibly edit encoder settings */
+static void lintospeex_feedback(struct ast_trans_pvt *pvt, struct ast_frame *feedback)
+{
+	struct speex_coder_pvt *tmp = pvt->pvt;
+
+	struct ast_rtp_rtcp_report *rtcp_report;
+	struct ast_rtp_rtcp_report_block *report_block;
+
+	int fraction_lost;
+	int percent;
+	int bitrate;
+	int q;
+
+	if(!exp_rtcp_fb)
+		return;
+
+	/* We only accept feedback information in the form of SR and RR reports */
+	if (feedback->subclass.integer != AST_RTP_RTCP_SR && feedback->subclass.integer != AST_RTP_RTCP_RR) {
+		return;
+	}
+
+	rtcp_report = (struct ast_rtp_rtcp_report *)feedback->data.ptr;
+	if (rtcp_report->reception_report_count == 0)
+		return;
+	report_block = rtcp_report->report_block[0];
+	fraction_lost = report_block->lost_count.fraction;
+	if (fraction_lost == tmp->fraction_lost)
+		return;
+	/* Per RFC3550, fraction lost is defined to be the number of packets lost
+	 * divided by the number of packets expected. Since it's a 8-bit value,
+	 * and we want a percentage value, we multiply by 100 and divide by 256. */
+	percent = (fraction_lost*100)/256;
+	bitrate = 0;
+	q = -1;
+	ast_debug(3, "Fraction lost changed: %d --> %d percent loss\n", fraction_lost, percent);
+	/* Handle change */
+	speex_encoder_ctl(tmp->speex, SPEEX_GET_BITRATE, &bitrate);
+	ast_debug(3, "Current bitrate: %d\n", bitrate);
+	ast_debug(3, "Current quality: %d/%d\n", tmp->quality, tmp->default_quality);
+	/* FIXME BADLY Very ugly example of how this could be handled: probably sucks */
+	if (percent < 10) {
+		/* Not that bad, default quality is fine */
+		q = tmp->default_quality;
+	} else if (percent < 20) {
+		/* Quite bad, let's go down a bit */
+		q = tmp->default_quality-1;
+	} else if (percent < 30) {
+		/* Very bad, let's go down even more */
+		q = tmp->default_quality-2;
+	} else {
+		/* Really bad, use the lowest quality possible */
+		q = 0;
+	}
+	if (q < 0)
+		q = 0;
+	if (q != tmp->quality) {
+		ast_debug(3, "  -- Setting to %d\n", q);
+		if (vbr) {
+			float vbr_q = q;
+			speex_encoder_ctl(tmp->speex, SPEEX_SET_VBR_QUALITY, &vbr_q);
+		} else {
+			speex_encoder_ctl(tmp->speex, SPEEX_SET_QUALITY, &q);
+		}
+		tmp->quality = q;
+	}
+	tmp->fraction_lost = fraction_lost;
+}
+
 static void speextolin_destroy(struct ast_trans_pvt *arg)
 {
 	struct speex_coder_pvt *pvt = arg->pvt;
@@ -402,6 +482,7 @@ static struct ast_translator lintospeex
 	.newpvt = lintospeex_new,
 	.framein = lintospeex_framein,
 	.frameout = lintospeex_frameout,
+	.feedback = lintospeex_feedback,
 	.destroy = lintospeex_destroy,
 	.sample = slin8_sample,
 	.desc_size = sizeof(struct speex_coder_pvt),
@@ -448,6 +529,7 @@ static struct ast_translator lin16tospee
 	.newpvt = lin16tospeexwb_new,
 	.framein = lintospeex_framein,
 	.frameout = lintospeex_frameout,
+	.feedback = lintospeex_feedback,
 	.destroy = lintospeex_destroy,
 	.sample = slin16_sample,
 	.desc_size = sizeof(struct speex_coder_pvt),
@@ -493,6 +575,7 @@ static struct ast_translator lin32tospee
 	.newpvt = lin32tospeexuwb_new,
 	.framein = lintospeex_framein,
 	.frameout = lintospeex_frameout,
+	.feedback = lintospeex_feedback,
 	.destroy = lintospeex_destroy,
 	.desc_size = sizeof(struct speex_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
@@ -588,6 +671,9 @@ static int parse_config(int reload)
 				pp_dereverb_level = res_f;
 			} else
 				ast_log(LOG_ERROR,"Error! Preprocessor Dereverb Level must be >= 0\n");
+		} else if (!strcasecmp(var->name, "experimental_rtcp_feedback")) {
+			exp_rtcp_fb = ast_true(var->value) ? 1 : 0;
+			ast_verb(3, "CODEC SPEEX: Experimental RTCP Feedback. [%s]\n",exp_rtcp_fb ? "on" : "off");
 		}
 	}
 	ast_config_destroy(cfg);
@@ -603,44 +689,42 @@ static int reload(void)
 
 static int unload_module(void)
 {
-	int res = 0;
-
-	res |= ast_unregister_translator(&speextolin);
-	res |= ast_unregister_translator(&lintospeex);
-	res |= ast_unregister_translator(&speexwbtolin16);
-	res |= ast_unregister_translator(&lin16tospeexwb);
-	res |= ast_unregister_translator(&speexuwbtolin32);
-	res |= ast_unregister_translator(&lin32tospeexuwb);
+	ast_unregister_translator(&speextolin);
+	ast_unregister_translator(&lintospeex);
+	ast_unregister_translator(&speexwbtolin16);
+	ast_unregister_translator(&lin16tospeexwb);
+	ast_unregister_translator(&speexuwbtolin32);
+	ast_unregister_translator(&lin32tospeexuwb);
 
-
-	return res;
+	return 0;
 }
 
 static int load_module(void)
 {
 	int res = 0;
 
-	if (parse_config(0))
+	if (parse_config(0)) {
 		return AST_MODULE_LOAD_DECLINE;
+	}
 
+	/* XXX It is most likely a bug in this module if we fail to register a translator */
 	res |= ast_register_translator(&speextolin);
 	res |= ast_register_translator(&lintospeex);
 	res |= ast_register_translator(&speexwbtolin16);
 	res |= ast_register_translator(&lin16tospeexwb);
 	res |= ast_register_translator(&speexuwbtolin32);
 	res |= ast_register_translator(&lin32tospeexuwb);
-
 	if (res) {
 		unload_module();
-		return res;
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	return res;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Speex Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/codecs.xml 1:16.1.1~dfsg-1/codecs/codecs.xml
--- 1:13.23.1~dfsg-2/codecs/codecs.xml	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codecs.xml	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 <member name="codec_opus" displayname="Download the Opus codec from Digium.  See http://downloads.digium.com/pub/telephony/codec_opus/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<depend>res_format_attr_opus</depend>
@@ -7,24 +8,28 @@
 </member>
 <member name="codec_silk" displayname="Download the SILK codec from Digium.  See http://downloads.digium.com/pub/telephony/codec_silk/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<defaultenabled>no</defaultenabled>
 </member>
 <member name="codec_siren7" displayname="Download the Siren7 codec from Digium.  See http://downloads.digium.com/pub/telephony/codec_siren7/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<defaultenabled>no</defaultenabled>
 </member>
 <member name="codec_siren14" displayname="Download the Siren14 codec from Digium.  See http://downloads.digium.com/pub/telephony/codec_siren14/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<defaultenabled>no</defaultenabled>
 </member>
 <member name="codec_g729a" displayname="Download the g729a codec from Digium.  A license must be purchased for this codec.  See http://downloads.digium.com/pub/telephony/codec_g729/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<defaultenabled>no</defaultenabled>
diff -pruN 1:13.23.1~dfsg-2/codecs/codec_ulaw.c 1:16.1.1~dfsg-1/codecs/codec_ulaw.c
--- 1:13.23.1~dfsg-2/codecs/codec_ulaw.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/codec_ulaw.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/config.h"
 #include "asterisk/translate.h"
@@ -190,7 +188,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "mu-Law Coder/Decoder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/codecs/ex_codec2.h 1:16.1.1~dfsg-1/codecs/ex_codec2.h
--- 1:13.23.1~dfsg-2/codecs/ex_codec2.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/ex_codec2.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,32 @@
+/*! \file
+ * \brief 8-bit raw data
+ *
+ * Copyright (C) 2016, Alexander Traud
+ *
+ * Distributed under the terms of the GNU General Public License
+ *
+ */
+
+#include "asterisk/format_cache.h"      /* for ast_format_codec2    */
+#include "asterisk/frame.h"             /* for ast_frame, etc       */
+
+static uint8_t ex_codec2[] = {
+	0xea, 0xca, 0x14, 0x85, 0x91, 0x78,
+};
+
+static struct ast_frame *codec2_sample(void)
+{
+	static struct ast_frame f = {
+		.frametype = AST_FRAME_VOICE,
+		.datalen = sizeof(ex_codec2),
+		.samples = CODEC2_SAMPLES,
+		.mallocd = 0,
+		.offset = 0,
+		.src = __PRETTY_FUNCTION__,
+		.data.ptr = ex_codec2,
+	};
+
+	f.subclass.format = ast_format_codec2;
+
+	return &f;
+}
diff -pruN 1:13.23.1~dfsg-2/codecs/ex_ilbc.h 1:16.1.1~dfsg-1/codecs/ex_ilbc.h
--- 1:13.23.1~dfsg-2/codecs/ex_ilbc.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/ex_ilbc.h	2018-12-26 18:45:13.000000000 +0000
@@ -7,6 +7,9 @@
  *
  */
 
+#include "asterisk/format_cache.h"      /* for ast_format_ilbc */
+#include "asterisk/frame.h"             /* for ast_frame, etc */
+
 static uint8_t ex_ilbc[] = {
 	0xff, 0xa0, 0xff, 0xfa, 0x0f, 0x60, 0x12, 0x11, 0xa2, 0x47,
 	0x22, 0x8c, 0x00, 0x00, 0x01, 0x02, 0x80, 0x43, 0xa0, 0x40,
@@ -20,8 +23,8 @@ static struct ast_frame *ilbc_sample(voi
 	static struct ast_frame f = {
 		.frametype = AST_FRAME_VOICE,
 		.datalen = sizeof(ex_ilbc),
-		/* All frames are 30 ms long */
-		.samples = ILBC_SAMPLES,
+		/* example frames are default long (30 ms) */
+		.samples = 240,
 		.mallocd = 0,
 		.offset = 0,
 		.src = __PRETTY_FUNCTION__,
diff -pruN 1:13.23.1~dfsg-2/codecs/lpc10/lpcini.c 1:16.1.1~dfsg-1/codecs/lpc10/lpcini.c
--- 1:13.23.1~dfsg-2/codecs/lpc10/lpcini.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/lpc10/lpcini.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,6 +34,7 @@ Some OSS fixes and a few lpc changes to
 	-lf2c -lm   (in that order)
 */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 #include "f2c.h"
 
diff -pruN 1:13.23.1~dfsg-2/codecs/lpc10/Makefile 1:16.1.1~dfsg-1/codecs/lpc10/Makefile
--- 1:13.23.1~dfsg-2/codecs/lpc10/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/lpc10/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -25,7 +25,7 @@ CFLAGS+= -fPIC -Wno-comment
 
 # The code here generates lots of warnings, so compiling with -Werror
 # fails miserably. Remove it for the time being.
-_ASTCFLAGS:=$(_ASTCFLAGS:-Werror=)
+_ASTCFLAGS:=$(_ASTCFLAGS:-Werror=) -DAST_MODULE_SELF_SYM=__internal_codec_lpc10_self
 
 LIB = $(LIB_TARGET_DIR)/liblpc10.a
 
diff -pruN 1:13.23.1~dfsg-2/codecs/Makefile 1:16.1.1~dfsg-1/codecs/Makefile
--- 1:13.23.1~dfsg-2/codecs/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/codecs/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -53,8 +53,6 @@ clean::
 	for dir in $(SUB_DIRS); do \
 		$(MAKE) -C $$dir clean; \
 	done
-	rm -f g722/*.[oai]
-	rm -f speex/*.[oai]
 
 .PHONY: $(SUB_DIRS)
 
@@ -81,9 +79,8 @@ $(SUB_ILBC):
 	@$(MAKE) -C $(SUB_ILBC) all _ASTCFLAGS="$(filter-out -Wmissing-prototypes -Wmissing-declarations -Wshadow,$(_ASTCFLAGS)) $(AST_NO_STRICT_OVERFLOW)"
 
 
-codec_g722.so: g722/g722_encode.o g722/g722_decode.o
 
-g722/g722_encode.o g722/g722_decode.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,codec_g722)
+$(call MOD_ADD_C,codec_g722,g722/g722_encode.c g722/g722_decode.c)
 
 
 ifeq ($(BUILD_CPU),x86_64)
@@ -92,6 +89,5 @@ else
 SPEEX_RESAMPLE_CFLAGS:=
 endif
 
-codec_resample.so: speex/resample.o
-
-speex/resample.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,codec_resample) $(SPEEX_RESAMPLE_CFLAGS)
+$(call MOD_ADD_C,codec_resample,speex/resample.c)
+speex/resample.o: _ASTCFLAGS+=$(SPEEX_RESAMPLE_CFLAGS)
diff -pruN 1:13.23.1~dfsg-2/configs/basic-pbx/extensions.conf 1:16.1.1~dfsg-1/configs/basic-pbx/extensions.conf
--- 1:13.23.1~dfsg-2/configs/basic-pbx/extensions.conf	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/basic-pbx/extensions.conf	2018-12-26 18:45:13.000000000 +0000
@@ -36,20 +36,6 @@ exten = 2565551234,1,Verbose(1, "User ${
  same = n,VoiceMailMain(example)
  same = n,Hangup()
 
-; Extension to queue for sales.
-; The queue has a 300 second timeout.
-exten = 2565551200,1,Verbose(1, "User ${CALLERID(num)} dialed the sales queue.")
- same = n,Answer()
- same = n,Queue(sales,,,,300)
- same = n,Goto(operator,1)
-
-; Extension to queue for a customer advocate.
-; The queue has a 1200 second timeout.
-exten = 2565551250,1,Verbose(1, "User ${CALLERID(num)} dialed the customer advocate queue.")
- same = n,Answer()
- same = n,Queue(customer_advocate,,,,1200)
- same = n,Goto(operator,1)
-
 [Dialing-Errors]
 ; Handle any extensions dialed internally that don't otherwise exist.
 ; Comment out or remove this extension if you would rather have the calls
diff -pruN 1:13.23.1~dfsg-2/configs/basic-pbx/modules.conf 1:16.1.1~dfsg-1/configs/basic-pbx/modules.conf
--- 1:13.23.1~dfsg-2/configs/basic-pbx/modules.conf	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/basic-pbx/modules.conf	2018-12-26 18:45:13.000000000 +0000
@@ -14,7 +14,6 @@ load = app_verbose.so
 load = app_voicemail.so
 load = app_directory.so
 load = app_confbridge.so
-load = app_queue.so
 
 ; Bridging
 
diff -pruN 1:13.23.1~dfsg-2/configs/basic-pbx/queues.conf 1:16.1.1~dfsg-1/configs/basic-pbx/queues.conf
--- 1:13.23.1~dfsg-2/configs/basic-pbx/queues.conf	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/basic-pbx/queues.conf	1970-01-01 00:00:00.000000000 +0000
@@ -1,18 +0,0 @@
-[general]
-monitor-type = MixMonitor
-
-;========================Sales Queue ==
-; Calls all sales persons in a ring-all fashion
-[sales]
-strategy=ringall
-member => PJSIP/1109 ; Terry Jules - Director of Sales
-member => PJSIP/1105 ; Garnet Claude - Sales Associate
-member => PJSIP/1112 ; Franny Ocean - Sales Associate
-
-;===================== Customer Advocate Queue ==
-; Calls all customer advocates in a ring-all fashion
-[customer_advocate]
-strategy=ringall
-member => PJSIP/1101 ; Maria Berny - Director of Customer Experience
-member => PJSIP/1115 ; Dusty Williams - Customer Advocate
-member => PJSIP/1102 ; Tommy Briar - Customer Advocate
diff -pruN 1:13.23.1~dfsg-2/configs/samples/amd.conf.sample 1:16.1.1~dfsg-1/configs/samples/amd.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/amd.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/amd.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -23,9 +23,5 @@ maximum_word_length = 5000  	; Maximum d
 between_words_silence = 50	; Minimum duration of silence after a word to consider
 				; the audio what follows as a new word
 
-maximum_number_of_words = 3	; Maximum number of words in the greeting
-				; If REACHED, then the result is detection as a MACHINE
-				; WARNING: Releases prior to January 1 2016 documented
-				; maximum_number_of_words as 'if exceeded, then MACHINE',
-				; which did not reflect the true functionality.  In Asterisk 14,
-				; this functionality will change to reflect the variables' name.
+maximum_number_of_words = 2	; Maximum number of words in the greeting
+				; If exceeded, then the result is detection as a MACHINE
diff -pruN 1:13.23.1~dfsg-2/configs/samples/ari.conf.sample 1:16.1.1~dfsg-1/configs/samples/ari.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/ari.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/ari.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -13,6 +13,11 @@ enabled = yes       ; When set to no, AR
 ; receiving clients are slow to process the received information. Value is in
 ; milliseconds; default is 100 ms.
 ;websocket_write_timeout = 100
+;
+; Display certain channel variables every time a channel-oriented
+; event is emitted:
+;
+;channelvars = var1,var2,var3
 
 ;[username]
 ;type = user        ; Specifies user configuration
diff -pruN 1:13.23.1~dfsg-2/configs/samples/asterisk.conf.sample 1:16.1.1~dfsg-1/configs/samples/asterisk.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/asterisk.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/asterisk.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -14,6 +14,7 @@ astsbindir => /usr/sbin
 [options]
 ;verbose = 3
 ;debug = 3
+;refdebug = yes			; Enable reference count debug logging.
 ;alwaysfork = yes		; Same as -F at startup.
 ;nofork = yes			; Same as -f at startup.
 ;quiet = yes			; Same as -q at startup.
@@ -105,15 +106,18 @@ documentation_language = en_US	; Set the
 				; This is currently is used by DUNDi and
 				; Exchanging Device and Mailbox State
 				; using protocols: XMPP, Corosync and PJSIP.
-;rtp_pt_dynamic = 96		; Normally the Dynamic RTP Payload Type numbers
-				; are 96-127, which allow 32 formats. When you
-				; use more and receive the message "No Dynamic
-				; RTP mapping available", extend the dynamic
-				; range by going for 35 (or 0) instead of 96.
-				; This allows 29 (or 64) more formats. 96 is the
-				; default because any number below might be
-				; rejected by a remote implementation; although
-				; no such broken implementation is known, yet.
+;rtp_use_dynamic = yes          ; When set to "yes" RTP dynamic payload types
+                                ; are assigned dynamically per RTP instance vs.
+                                ; allowing Asterisk to globally initialize them
+                                ; to pre-designated numbers (defaults to "yes").
+;rtp_pt_dynamic = 35		; Normally the Dynamic RTP Payload Type numbers
+				; are 96-127, which allow just 32 formats. The
+				; starting point 35 enables the range 35-63 and
+				; allows 29 additional formats. When you use
+				; more than 32 formats in the dynamic range and
+				; calls are not accepted by a remote
+				; implementation, please report this and go
+				; back to value 96.
 
 ; Changing the following lines may compromise your security.
 ;[files]
diff -pruN 1:13.23.1~dfsg-2/configs/samples/calendar.conf.sample 1:16.1.1~dfsg-1/configs/samples/calendar.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/calendar.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/calendar.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -6,6 +6,8 @@
 ;refresh = 15             ; refresh calendar every n minutes
 ;timeframe = 60           ; number of minutes of calendar data to pull for each refresh period
 ;                         ; should always be >= refresh
+;fetch_again_at_reload = no ; to reload the calendar content when the module is reloaded
+;
 ;
 ; You can set up res_calendar to execute a call upon an upcoming busy status
 ; The following fields are available from the ${CALENDAR_EVENT(<field>)} dialplan function:
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cdr_adaptive_odbc.conf.sample 1:16.1.1~dfsg-1/configs/samples/cdr_adaptive_odbc.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cdr_adaptive_odbc.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cdr_adaptive_odbc.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -57,3 +57,6 @@
 ; for this is to allow different sections to specify different values for
 ; a certain named column, presumably separated by filters.
 ;static "Some Special Value" => identifier_code
+;
+; Add quoted indentifiers for table and columns names.
+;quoted_identifiers=" ; Default to null
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cdr_beanstalkd.conf.sample 1:16.1.1~dfsg-1/configs/samples/cdr_beanstalkd.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cdr_beanstalkd.conf.sample	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cdr_beanstalkd.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,20 @@
+;
+; Asterisk Call Management CDR via Beanstalkd job queue
+;
+; Beanstalkd is a simple job queue server, that is highly versatile and simple to use.
+; Beanstalkd includes the capability of using multiple queues at the same time, with priorities.
+;
+; This module requires that your server has the beanstalk-client library installed. The library
+; can be downloaded from - https://github.com/deepfryed/beanstalk-client
+;
+
+[general]
+;enabled = yes
+
+;host = 127.0.0.1    ; Specify the remote IP address of the Beanstalkd server
+;port = 11300        ; Specify the remote PORT of the the Beanstalkd server
+;tube = asterisk-cdr ; Specify the default CDR job queue to use
+;priority = 99       ; Specify the default job priority for the queue. This parameter is useful when building
+                     ; platform with multiple Asterisk servers, that are used for different functions. For example,
+                     ; none billable CDR records can be inserted with a priority of 99, while billable ones be
+                     ; inserted with a priority of 1
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cdr_sqlite3_custom.conf.sample 1:16.1.1~dfsg-1/configs/samples/cdr_sqlite3_custom.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cdr_sqlite3_custom.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cdr_sqlite3_custom.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -5,6 +5,7 @@
 ;table => cdr
 ;columns => calldate, clid, dcontext, channel, dstchannel, lastapp, lastdata, duration, billsec, disposition, amaflags, accountcode, uniqueid, userfield, test
 ;values => '${CDR(start)}','${CDR(clid)}','${CDR(dcontext)}','${CDR(channel)}','${CDR(dstchannel)}','${CDR(lastapp)}','${CDR(lastdata)}','${CDR(duration)}','${CDR(billsec)}','${CDR(disposition)}','${CDR(amaflags)}','${CDR(accountcode)}','${CDR(uniqueid)}','${CDR(userfield)}','${CDR(test)}'
+;busy_timeout => 1000
 
 ;Enable High Resolution Times for billsec and duration fields
 ;values => '${CDR(start)}','${CDR(clid)}','${CDR(dcontext)}','${CDR(channel)}','${CDR(dstchannel)}','${CDR(lastapp)}','${CDR(lastdata)}','${CDR(duration,f)}','${CDR(billsec,f)}','${CDR(disposition)}','${CDR(amaflags)}','${CDR(accountcode)}','${CDR(uniqueid)}','${CDR(userfield)}','${CDR(test)}'
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cel_beanstalkd.conf.sample 1:16.1.1~dfsg-1/configs/samples/cel_beanstalkd.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cel_beanstalkd.conf.sample	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cel_beanstalkd.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,21 @@
+;
+; Beanstalkd Job Queue Server CEL Backend
+;
+[general]
+
+; Backend Activation
+;
+; Use the 'enabled' keyword to turn CEL logging
+; on or off.
+;
+; Accepted values: yes and no
+; Default value:   no
+;enabled = yes
+
+;host = 127.0.0.1    ; Specify the remote IP address of the Beanstalkd server
+;port = 11300        ; Specify the remote PORT of the the Beanstalkd server
+;tube = asterisk-cel ; Specify the default CDR job queue to use
+;priority = 99       ; Specify the default job priority for the queue. This parameter is useful when building
+                     ; platform with multiple Asterisk servers, that are used for different functions. For example,
+                     ; none billable CDR records can be inserted with a priority of 99, while billable ones be
+                     ; inserted with a priority of 1
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cel_pgsql.conf.sample 1:16.1.1~dfsg-1/configs/samples/cel_pgsql.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cel_pgsql.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cel_pgsql.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -59,10 +59,15 @@
 ;
 ;show_user_defined=yes
 
+; Log date/time in GMT. The default of this option is 'no'.
+;usegmtime=yes
+
 ;hostname=localhost
 ;port=5432
 ;dbname=asterisk
 ;password=password
 ;user=postgres
 ;table=cel		;SQL table where CEL's will be inserted
-;appname=asterisk   ; Postgres application_name support (optional). Whitespace not allowed.
\ No newline at end of file
+;schema=public ;Schema where CEL's table is located.  Optional parameter.
+               ;If schema support is present the default value used will be current_schema().
+;appname=asterisk   ; Postgres application_name support (optional). Whitespace not allowed.
diff -pruN 1:13.23.1~dfsg-2/configs/samples/cel_sqlite3_custom.conf.sample 1:16.1.1~dfsg-1/configs/samples/cel_sqlite3_custom.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/cel_sqlite3_custom.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/cel_sqlite3_custom.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -22,3 +22,4 @@
 ;table	=> cel
 ;columns	=> eventtype, eventtime, cidname, cidnum, cidani, cidrdnis, ciddnid, context, exten, channame, appname, appdata, amaflags, accountcode, uniqueid, userfield, peer, userdeftype, eventextra
 ;values	=> '${eventtype}','${eventtime}','${CALLERID(name)}','${CALLERID(num)}','${CALLERID(ANI)}','${CALLERID(RDNIS)}','${CALLERID(DNID)}','${CHANNEL(context)}','${CHANNEL(exten)}','${CHANNEL(channame)}','${CHANNEL(appname)}','${CHANNEL(appdata)}','${CHANNEL(amaflags)}','${CHANNEL(accountcode)}','${CHANNEL(uniqueid)}','${CHANNEL(userfield)}','${BRIDGEPEER}','${userdeftype}','${eventextra}'
+;busy_timeout	=> 1000
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/configs/samples/chan_dahdi.conf.sample 1:16.1.1~dfsg-1/configs/samples/chan_dahdi.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/chan_dahdi.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/chan_dahdi.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -208,7 +208,7 @@ context=public
 ; response to call collisions (glare) and buggy peers.  However, there
 ; are telco switches out there that ignore the RESTART and continue to
 ; send calls to the channel in the restarting state.
-; Default yes in current release branches for backward compatibility.
+; Default no.
 ;
 ;force_restart_unavailable_chans=yes
 ;
@@ -1137,6 +1137,21 @@ pickupgroup=1
 ;
 ;faxbuffers=>6,full
 ;
+; When FXO signalling (FXS device, e.g. analog phone) is used, overlap dialing
+; is typically used. Asterisk has several configurable (per-channel) timeouts
+; to know how long to wait for the next digit. All the values are in
+; milliseconds.
+; * firstdigit_timeout: a longer timeout before any digit is dialed.
+;   By default: 16 seconds.
+; * interdigit_timeout: timeout for next digits, if the current number dialed
+;   does not match a number in the current context. Default: 8 seconds.
+; * matchdigit_timeout: timeout for next digits, if the current number dialed
+;   matches a number in the current context. Default: 3 seconds.
+;
+;firstdigit_timeout=16000
+;interdigit_timeout=8000
+;matchdigit_timeout=3000
+;
 ; Configure the default number of DAHDI buffers and the transmit policy to use.
 ; This can be used to eliminate data drops when scheduling jitter prevents
 ; Asterisk from writing to a DAHDI channel regularly. Most users will probably
@@ -1330,24 +1345,6 @@ pickupgroup=1
 ; signalling = pri_cpe
 ; group = 2
 ; channel => 1-23
-;
-; Alternatively, the number of the channel may be replaced with a relative
-; path to a device file under /dev/dahdi . The final element of that file
-; must be a number, though. The directory separator is '!', as we can't
-; use '/' in a dial string. So if we have
-;
-;   /dev/dahdi/span-name/pstn/00/1
-;   /dev/dahdi/span-name/pstn/00/2
-;   /dev/dahdi/span-name/pstn/00/3
-;   /dev/dahdi/span-name/pstn/00/4
-;
-; we could use:
-;channel => span-name!pstn!00!1-4
-;
-; or:
-;channel => span-name!pstn!00!1,2,3,4
-;
-; See also ignore_failed_channels above.
 
 ;  Used for distinctive ring support for x100p.
 ;  You can see the dringX patterns is to set any one of the dringXcontext fields
diff -pruN 1:13.23.1~dfsg-2/configs/samples/codecs.conf.sample 1:16.1.1~dfsg-1/configs/samples/codecs.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/codecs.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/codecs.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -57,6 +57,9 @@ pp_dereverb => false
 pp_dereverb_decay => 0.4
 pp_dereverb_level => 0.3
 
+; experimental bitrate changes depending on RTCP feedback [true / false]
+experimental_rtcp_feedback => false
+
 
 [plc]
 ; for all codecs which do not support native PLC
diff -pruN 1:13.23.1~dfsg-2/configs/samples/confbridge.conf.sample 1:16.1.1~dfsg-1/configs/samples/confbridge.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/confbridge.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/confbridge.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -18,6 +18,18 @@
 [default_user]
 type=user
 ;admin=yes     ; Sets if the user is an admin or not. Off by default.
+
+;send_events=no  ; If events are enabled for this bridge and this option is
+                 ; set, users will receive events like join, leave, talking,
+                 ; etc. via text messages.  For users accessing the bridge
+                 ; via chan_pjsip, this means in-dialog MESSAGE messages.
+                 ; This is most useful for WebRTC participants where the
+                 ; browser application can use the messages to alter the user
+                 ; interface.
+;echo_events=yes ; If events are enabled for this user and this option is set,
+                 ; the user will receive events they trigger, talking, mute, etc.
+                 ; If not set, they will not receive their own events.
+
 ;marked=yes    ; Sets if this is a marked user or not. Off by default.
 ;startmuted=yes; Sets if all users should start out muted. Off by default
 ;music_on_hold_when_empty=yes  ; Sets whether MOH should be played when only
@@ -177,6 +189,12 @@ type=bridge
                                ; By default, the record_file is stored in Asterisk's spool/monitor directory
                                ; with a unique filename starting with the 'confbridge' prefix.
 
+;record_file_append=yes        ; Append record file when starting/stopping on same conference recording.
+;record_file_timestamp=yes     ; Append the start time to the record file name.
+
+;record_options=               ; Pass additional options to MixMonitor.
+;record_command=</path/to/command> ; Command to execute when recording finishes.
+
 ;internal_sample_rate=auto     ; Sets the internal native sample rate the
                                ; conference is mixed at.  This is set to automatically
                                ; adjust the sample rate to the best quality by default.
@@ -215,11 +233,35 @@ type=bridge
                            ; first_marked: The first marked user to join the conference with video capabilities
                            ;               is the single source of video distribution among all participants. If
                            ;               that user leaves, the marked user to join after them becomes the source.
+                           ;
+                           ; sfu: Selective Forwarding Unit - Sets multi-stream operation
+                           ;      for a multi-party video conference.
 
 ;language=en               ; Set the language used for announcements to the conference.
                            ; Default is en (English).
 
 ;regcontext=conferences    ; The name of the context into which to register conference names as extensions.
+;video_update_discard=2000 ; Amount of time (in milliseconds) to discard video update requests after sending a video
+                           ; update request. Default is 2000. A video update request is a request for a full video
+                           ; intra-frame. Clients can request this if they require a full frame in order to decode
+                           ; the video stream. Since a full frame can be large limiting how often they occur can
+                           ; reduce bandwidth usage at the cost of increasing how long it may take a newly joined
+                           ; channel to receive the video stream.
+;remb_send_interval=1000   ; Interval (in milliseconds) at which a combined REMB frame will be sent to sources of video.
+                           ; A REMB frame contains receiver estimated maximum bitrate information. By creating a combined
+                           ; frame and sending it to the sources of video the sender can be influenced on what bitrate
+                           ; they choose allowing a better experience for the receivers. This defaults to 0, or disabled.
+;remb_behavior=average     ; How the combined REMB report for an SFU video bridge is constructed. If set to "average" then
+                           ; the estimated maximum bitrate of each receiver is used to construct an average bitrate. If
+                           ; set to "lowest" the lowest maximum bitrate is forwarded to the sender. If set to "highest"
+                           ; the highest maximum bitrate is forwarded to the sender. This defaults to "average".
+
+;enable_events=no          ; If enabled, recipients who joined the bridge via a channel driver
+                           ; that supports Enhanced Messaging (currently only chan_pjsip) will
+                           ; receive in-dialog messages containing a JSON body describing the
+                           ; event.  The Content-Type header will be
+                           ; "text/x-ast-confbridge-event".
+                           ; This feature must also be enabled in user profiles.
 
 ; All sounds in the conference are customizable using the bridge profile options below.
 ; Simply state the option followed by the filename or full path of the filename after
@@ -261,6 +303,8 @@ type=bridge
 ;sound_unlocked_now; The sound played to an admin after toggling the conference to unlocked mode.
 ;sound_error_menu ; The sound played when an invalid menu option is entered.
 ;sound_begin ; The sound played to the conference when the first marked user enters the conference.
+;sound_binaural_on ; The sound played when binaural audio is turned on
+;sound_binaural_off ; The sound played when binaural audio is turned off
 
 ; --- ConfBridge Menu Options ---
 ; The ConfBridge application also has the ability to
@@ -299,6 +343,7 @@ type=bridge
                                        ; using the '&' character as a delimiter.
 ; toggle_mute      ; Toggle turning on and off mute.  Mute will make the user silent
                    ; to everyone else, but the user will still be able to listen in.
+; toggle_binaural  ; Toggle on or off binaural audio processing.
 
 ; no_op ; This action does nothing (No Operation). Its only real purpose exists for
         ; being able to reserve a sequence in the config as a menu exit sequence.
diff -pruN 1:13.23.1~dfsg-2/configs/samples/extensions.conf.sample 1:16.1.1~dfsg-1/configs/samples/extensions.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/extensions.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/extensions.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -197,6 +197,11 @@ TRUNKMSD=1					; MSD digits to strip (us
 ; before "B".
 ;
 ;[context]
+;
+;autohints = yes
+; If enabled for a context, a device state hint will be automatically created in
+; the context with the name of the device and updated with device state changes.
+;
 ;exten => someexten,{priority|label{+|-}offset}[(alias)],application(arg1,arg2,...)
 ;
 ; Timing list for includes is
diff -pruN 1:13.23.1~dfsg-2/configs/samples/extensions_minivm.conf.sample 1:16.1.1~dfsg-1/configs/samples/extensions_minivm.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/extensions_minivm.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/extensions_minivm.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,5 @@
 ; MINI-VOICEMAIL dialplan example
 ; ---------------------------------------------------------------------------------------
-; ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-;
 ;
 ; This is an example on how to use the Mini-Voicemail system to build
 ; voicemail systems.
diff -pruN 1:13.23.1~dfsg-2/configs/samples/followme.conf.sample 1:16.1.1~dfsg-1/configs/samples/followme.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/followme.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/followme.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -5,6 +5,11 @@ featuredigittimeout=>5000
 ; The number of ms to wait for a digit input for the callee on whether to take the call or
 ; not before we consider them "done" entering digits.
 ;
+enable_callee_prompt=>true
+; Enable prompting a callee to either accept or reject the forwarded call.
+; If disabled, the optional prompting for caller name (option 'a') is
+; likewise disabled. Enabled by default.
+;
 takecall=>1
 ; The global default keypress for the callee to take taking the current call. This can be
 ; a single digit or multiple digits. Default is "1".
@@ -58,6 +63,9 @@ number=>01233456,25
 ; step to make a choice on whether to take the call or not.  That being the case,
 ; you may want to make the timeout on the last step longer to give enough time to
 ; make the choice to accept or not.
+enable_callee_prompt=>true
+; Enable prompting the callee to accept the forwarded call. The default
+; is the global value.
 takecall=>1
 ; The keypress for the callee to take taking the current call. This can be
 ; a single digit or multiple digits. Default is the global default.
diff -pruN 1:13.23.1~dfsg-2/configs/samples/func_odbc.conf.sample 1:16.1.1~dfsg-1/configs/samples/func_odbc.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/func_odbc.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/func_odbc.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -11,8 +11,8 @@
 ; in a second func_odbc call.
 ; Note that you'll need additional dialplan locks for this behaviour to work.
 ; There are better ways: using stored procedures/functions instead.
-; This option is enabled by default.
-;single_db_connection=yes
+; This option is disabled by default.
+;single_db_connection=no
 ;
 ;
 ; Each context is a separately defined function.  By convention, all
diff -pruN 1:13.23.1~dfsg-2/configs/samples/http.conf.sample 1:16.1.1~dfsg-1/configs/samples/http.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/http.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/http.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -21,7 +21,7 @@
 ; of the Server header field from HTTP response message headers and the
 ; <address /> element from HTTP response message bodies.
 ;
-;servername=Asterisk
+servername=Asterisk
 ;
 ; Whether HTTP/HTTPS interface is enabled or not.  Default is no.
 ; This also affects manager/rawman/mxml access (see manager.conf)
diff -pruN 1:13.23.1~dfsg-2/configs/samples/iax.conf.sample 1:16.1.1~dfsg-1/configs/samples/iax.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/iax.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/iax.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -431,6 +431,10 @@ autokill=yes
 ;
 ;requirecalltoken=no
 ;
+; Maximum time allowed for call token authentication handshaking. Default is 10 seconds.
+; Use higher values in lagged or high packet loss networks.
+;
+;calltokenexpiration=10
 
 ;
 ; These options are used to limit the amount of call numbers allocated to a
diff -pruN 1:13.23.1~dfsg-2/configs/samples/logger.conf.sample 1:16.1.1~dfsg-1/configs/samples/logger.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/logger.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/logger.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -88,7 +88,35 @@
 ;
 [logfiles]
 ;
-; Format is "filename" and then "levels" of debugging to be included:
+; Format is:
+;
+; logger_name => [formatter]levels
+;
+; The name of the logger dictates not only the name of the logging
+; channel, but also its type. Valid types are:
+;   - 'console'  - The root console of Asterisk
+;   - 'syslog'   - Linux syslog, with facilities specified afterwards with
+;                  a period delimiter, e.g., 'syslog.local0'
+;   - 'filename' - The name of the log file to create. This is the default
+;                  for log channels.
+;
+; Filenames can either be relative to the standard Asterisk log directory
+; (see 'astlogdir' in asterisk.conf), or absolute paths that begin with
+; '/'.
+;
+; An optional formatter can be specified prior to the log levels sent
+; to the log channel. The formatter is defined immediately preceeding the
+; levels, and is enclosed in square brackets. Valid formatters are:
+;   - [default] - The default formatter, this outputs log messages using a
+;                 human readable format.
+;   - [json]    - Log the output in JSON. Note that JSON formatted log entries,
+;                 if specified for a logger type of 'console', will be formatted
+;                 per the 'default' formatter for log messages of type VERBOSE.
+;                 This is due to the remote consoles intepreting verbosity
+;                 outside of the logging subsystem.
+;
+; Log levels include the following, and are specified in a comma delineated
+; list:
 ;    debug
 ;    notice
 ;    warning
@@ -98,12 +126,6 @@
 ;    fax
 ;    security
 ;
-; Special filename "console" represents the root console
-;
-; Filenames can either be relative to the standard Asterisk log directory
-; (see 'astlogdir' in asterisk.conf), or absolute paths that begin with
-; '/'.
-;
 ; Verbose takes an optional argument, in the form of an integer level.
 ; Verbose messages with higher levels will not be logged to the file.  If
 ; the verbose level is not specified, it will log verbose messages following
@@ -135,7 +157,9 @@ console => notice,warning,error
 ;console => notice,warning,error,debug
 messages => notice,warning,error
 ;full => notice,warning,error,debug,verbose,dtmf,fax
-
+;
+;full-json => [json]debug,verbose,notice,warning,error,dtmf,fax
+;
 ;syslog keyword : This special keyword logs to syslog facility
 ;
 ;syslog.local0 => notice,warning,error
diff -pruN 1:13.23.1~dfsg-2/configs/samples/modules.conf.sample 1:16.1.1~dfsg-1/configs/samples/modules.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/modules.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/modules.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -8,33 +8,28 @@
 autoload=yes
 ;
 ; Any modules that need to be loaded before the Asterisk core has been
-; initialized (just after the logger has been initialized) can be loaded
-; using 'preload'. This will frequently be needed if you wish to map all
-; module configuration files into Realtime storage, since the Realtime
-; driver will need to be loaded before the modules using those configuration
-; files are initialized.
-;
-; An example of loading ODBC support would be:
-;preload => res_odbc.so
-;preload => res_config_odbc.so
+; initialized (just after the logger initialization) can be loaded
+; using 'preload'.  'preload' forces a module and the modules it
+; is known to depend upon to be loaded earlier than they normally get
+; loaded.
+;
+; NOTE: There is no good reason left to use 'preload' anymore.  It was
+; historically required to preload realtime driver modules so you could
+; map Asterisk core configuration files to Realtime storage.
+; This is no longer needed.
 ;
-; Uncomment the following if you wish to use the Speech Recognition API
-;preload => res_speech.so
+;preload => your_special_module.so
 ;
 ; If you want Asterisk to fail if a module does not load, then use
 ; the "require" keyword. Asterisk will exit with a status code of 2
 ; if a required module does not load.
 ;
-; require = chan_sip.so
-; If you want you can combine with preload
-; preload-require = res_odbc.so
-;
-; If you want, load the GTK console right away.
+; require = chan_pjsip.so
 ;
-noload => pbx_gtkconsole.so
-;load => pbx_gtkconsole.so
+; If you want you can combine with preload
+; preload-require = your_special_module.so
 ;
-load => res_musiconhold.so
+;load => res_musiconhold.so
 ;
 ; Load one of: chan_oss, alsa, or console (portaudio).
 ; By default, load chan_oss only (automatically).
diff -pruN 1:13.23.1~dfsg-2/configs/samples/musiconhold.conf.sample 1:16.1.1~dfsg-1/configs/samples/musiconhold.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/musiconhold.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/musiconhold.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 ; Music on Hold -- Sample Configuration
 ;
 [general]
-;cachertclasses=yes ; use 1 instance of moh class for all users who are using it,
-                    ; decrease consumable cpu cycles and memory
-                    ; disabled by default
-
+;cachertclasses=yes     ; use 1 instance of moh class for all users who are using it,
+                        ; decrease consumable cpu cycles and memory
+                        ; disabled by default
+
+;preferchannelclass=yes ; Prefer the musicclass as defined by CHANNEL(musicclass),
+                        ; over a class set by an application (e.g. a musicclass set on a queue).
+                        ; Defaults to yes.
 
 ; valid mode options:
 ; files		-- read files from a directory in any Asterisk supported
@@ -57,13 +60,16 @@ directory=moh
 ;               ; be played to them. Also, if using modes that Asterisk
 ;               ; controls the playlist for (files, mp3, etc), the same
 ;               ; sound will also be played between MOH songs.
-;sort=random    ; Sort the files in random order
+;sort=random    ; Sort the files in random order. Other sort options are:
+;               ; 'alpha' and 'randstart'. If 'alpha', the files are sorted
+;               ; in alphabetical order. If 'randstart', the files are sorted
+;               ; in alphabetical order as well, but the first file is chosen
+;               ; at random. If unspecified, the sort order is undefined.
 
 ;[native-alphabetical]
 ;mode=files
 ;directory=moh
-;sort=alpha     ; Sort the files in alphabetical order.  If this option is
-;               ; not specified, the sort order is undefined.
+;sort=alpha     ; Sort the files in alphabetical order.
 
 ; =========
 ; Other (non-native) playback methods
diff -pruN 1:13.23.1~dfsg-2/configs/samples/pjsip.conf.sample 1:16.1.1~dfsg-1/configs/samples/pjsip.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/pjsip.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/pjsip.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -608,6 +608,10 @@
 ;direct_media_glare_mitigation=none     ; Mitigation of direct media re INVITE
                                         ; glare (default: "none")
 ;direct_media_method=invite     ; Direct Media method type (default: "invite")
+;trust_connected_line=yes       ; Accept Connected Line updates from this endpoint
+                                ; (default: "yes")
+;send_connected_line=yes        ; Send Connected Line updates to this endpoint
+                                ; (default: "yes")
 ;connected_line_method=invite   ; Connected line method type.
                                 ; When set to "invite", check the remote's
                                 ; Allow header and if UPDATE is allowed, send
@@ -651,6 +655,8 @@
                       ; An MWI subscribe will replace unsoliticed NOTIFYs
                       ; (default: "no")
 ;moh_suggest=default    ; Default Music On Hold class (default: "default")
+;moh_passthrough=yes    ; Pass Music On Hold through using SIP re-invites with sendonly
+                        ; when placing on hold and sendrecv when taking off hold
 ;outbound_auth= ; Authentication object used for outbound requests (default:
                 ; "")
 ;outbound_proxy=        ; Proxy through which to send requests, a full SIP URI
@@ -750,10 +756,12 @@
                 ; "no")
 ;dtls_rekey=0   ; Interval at which to renegotiate the TLS session and rekey
                 ; the SRTP session (default: "0")
-;dtls_cert_file=        ; Path to certificate file to present to peer (default:
-                        ; "")
-;dtls_private_key=      ; Path to private key for certificate file (default:
-                        ; "")
+;dtls_auto_generate_cert= ; Enable ephemeral DTLS certificate generation (default:
+                          ; "no")
+;dtls_cert_file=          ; Path to certificate file to present to peer (default:
+                          ; "")
+;dtls_private_key=        ; Path to private key for certificate file (default:
+                          ; "")
 ;dtls_cipher=   ; Cipher to use for DTLS negotiation (default: "")
 ;dtls_ca_file=  ; Path to certificate authority certificate (default: "")
 ;dtls_ca_path=  ; Path to a directory containing certificate authority
@@ -777,6 +785,10 @@
                    ; "0" or not enabled)
 ;contact_user= ; On outgoing requests, force the user portion of the Contact
                ; header to this value (default: "")
+;preferred_codec_only=yes       ; Respond to a SIP invite with the single most preferred codec
+                                ; rather than advertising all joint codec capabilities. This
+                                ; limits the other side's codec choice to exactly what we prefer.
+                                ; default is no.
 ;asymmetric_rtp_codec= ; Allow the sending and receiving codec to differ and
                        ; not be automatically matched (default: "no")
 ;refer_blind_progress= ; Whether to notifies all the progress details on blind
@@ -788,6 +800,18 @@
                               ; The value "yes" is useful for some SIP phones
                               ; (Cisco SPA) to be able to indicate and pick up
                               ; ringing devices.
+;max_audio_streams= ; The maximum number of allowed negotiated audio streams
+                    ; (default: 1)
+;max_video_streams= ; The maximum number of allowed negotiated video streams
+                    ; (default: 1)
+;webrtc= ; When set to "yes" this also enables the following values that are needed
+         ; for webrtc: rtcp_mux, use_avpf, ice_support, and use_received_transport.
+         ; The following configuration settings also get defaulted as follows:
+         ;     media_encryption=dtls
+         ;     dtls_verify=fingerprint
+         ;     dtls_setup=actpass
+         ; A dtls_cert_file and a dtls_ca_file still need to be specified.
+         ; Default for this option is "no"
 ;incoming_mwi_mailbox = ; Mailbox name to use when incoming MWI NOTIFYs are
                         ; received.
                         ; If an MWI NOTIFY is received FROM this endpoint,
@@ -869,7 +893,13 @@
                 ; (default: "")
 ;cert_file=     ; Certificate file for endpoint TLS ONLY
                 ; Will read .crt or .pem file but only uses cert,
-                ; a .key file must be specified via priv_key_file
+                ; a .key file must be specified via priv_key_file.
+                ; Since PJProject version 2.5: If the file name ends in _rsa,
+                ; for example "asterisk_rsa.pem", the files "asterisk_dsa.pem"
+                ; and/or "asterisk_ecc.pem" are loaded (certificate, inter-
+                ; mediates, private key), to support multiple algorithms for
+                ; server authentication (RSA, DSA, ECDSA). If the chains are
+                ; different, at least OpenSSL 1.0.2 is required.
                 ; (default: "")
 ;cipher=        ; Preferred cryptography cipher names TLS ONLY (default: "")
 ;method=        ; Method of SSL transport TLS ONLY (default: "")
diff -pruN 1:13.23.1~dfsg-2/configs/samples/queuerules.conf.sample 1:16.1.1~dfsg-1/configs/samples/queuerules.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/queuerules.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/queuerules.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -9,22 +9,39 @@
 ; realtime_rules = yes
 ;
 
-; It is possible to change the value of the QUEUE_MAX_PENALTY and QUEUE_MIN_PENALTY
+; It is possible to change the value of the QUEUE_MAX_PENALTY, QUEUE_MIN_PENALTY and QUEUE_RAISE_PENALTY
 ; channel variables in mid-call by defining rules in the queue for when to do so. This can allow for
 ; a call to be opened to more members or potentially a different set of members.
 ; The advantage to changing members this way as opposed to inserting the caller into a
 ; different queue with more members or reinserting the caller into the same queue with a different
 ; QUEUE_MAX_PENALTY or QUEUE_MIN_PENALTY set is that the caller does not lose his place in the queue.
 ;
+; QUEUE_MAX_PENALTY, QUEUE_MIN_PENALTY and QUEUE_RAISE_PENALTY only apply to a queue call, and are only
+; modified by these rules if they are initially set in the dialplan.
+;
+; If QUEUE_MIN_PENALTY is set, agents with a lower penalty value will not be considered for the caller.
+; If QUEUE_MAX_PENALTY is set, agents with a higher penalty value will not be considered for the caller.
+; If QUEUE_RAISE_PENALTY is set, agents with a lower penalty will be treated as having a penalty = QUEUE_RAISE_PENALTY.
+;
+; QUEUE_RAISE_PENALTY example:
+;   - Agent 1 has penalty 1
+;   - Agent 2 has penalty 2
+;   - the queue rule is set to:
+;       penaltychange => 30,,,2
+;
+;   Prior to the 30 second mark, Agent 1 will take priority over Agent 2 for call distribution.
+;   After 30 seconds, Agent 1's priority is bumped to 2 by the penaltychange, so both agents are treated equally.
+;
 ; Note: There is a limitation to these rules; a caller will follow the penaltychange rules for
 ; the queue that were defined at the time the caller entered the queue. If an update to the rules is
 ; made during the caller's stay in the queue, these will not be reflected for that caller.
 ;
 ; The syntax for these rules is
-; penaltychange => <number of seconds into the call>,<absolute or relative change to QUEUE_MAX_PENALTY>[,absolute or relative change to QUEUE_MIN_PENALTY]
+; penaltychange => <number of seconds into the call>,<absolute or relative change to QUEUE_MAX_PENALTY>[,absolute or relative change to QUEUE_MIN_PENALTY][,absolute or relative change to QUEUE_RAISE_PENALTY]
 ;
 ; Example:
 ; [myrule]
 ; penaltychange => 30,+3   ; 30 seconds into the call increase the QUEUE_MAX_PENALTY by 3, no change to QUEUE_MIN_PENALTY
 ; penaltychange => 60,10,5 ; 60 seconds into the call increase the QUEUE_MAX_PENALTY to 10 and increase the QUEUE_MIN_PENALTY to 5
 ; penaltychange => 75,,7   ; 75 seconds into the call keep the QUEUE_MAX_PENALTY the same and increase the QUEUE_MIN_PENALTY to 7
+; penaltychange => 90,,,20 ; 90 seconds into the call leave QUEUE_MAX_PENALTY and QUEUE_MIN_PENALTY untouched and set QUEUE_RAISE_PENALTY to 20
diff -pruN 1:13.23.1~dfsg-2/configs/samples/queues.conf.sample 1:16.1.1~dfsg-1/configs/samples/queues.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/queues.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/queues.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -113,8 +113,10 @@ monitor-type = MixMonitor
 ;strategy = ringall
 ;
 ; Second settings for service level (default 0)
-; Used for service level statistics (calls answered within service level time
-; frame)
+; Used for service level statistics (calls answered within service level time frame)
+; There are 2 Service Level Performance Calculations SL and SL2
+; SL = (total calls answered within threshold / total calls answered) * 100
+; SL2 = ((calls answered within threshold + calls abandoned within threshold) / (answered calls + abandoned calls)) * 100
 ;servicelevel = 60
 ;
 ; A context may be specified, in which if the user types a SINGLE
@@ -343,6 +345,11 @@ monitor-type = MixMonitor
 ;
 ; announce-round-seconds = 10
 ;
+; Only announce the caller's position if it has improved since the last announcement.
+; The default value is no.
+;
+; announce-position-only-up = yes
+;
 ; Use these sound files in making position/holdtime announcements.  The
 ; defaults are as listed below -- change only if you need to.
 ;
@@ -544,7 +551,7 @@ monitor-type = MixMonitor
 ; must also preload pbx_config.so and chan_local.so (or pbx_ael.so, pbx_lua.so,
 ; or pbx_realtime.so, depending on how your dialplan is configured).
 ;
-; syntax: member => interface,[,penalty][,membername][,state_interface][,ringinuse]
+; syntax: member => interface,[,penalty][,membername][,state_interface][,ringinuse][,wrapuptime]
 ;
 ;member => DAHDI/1
 ;member => DAHDI/2,10
diff -pruN 1:13.23.1~dfsg-2/configs/samples/res_config_sqlite3.conf.sample 1:16.1.1~dfsg-1/configs/samples/res_config_sqlite3.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/res_config_sqlite3.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/res_config_sqlite3.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -28,3 +28,8 @@
 ;   to lock for writing.
 ;
 ;batch=1000
+;
+; busy timeout - timeout in miliseconds to stop waiting for locked database.
+;   More info at https://www.sqlite.org/c3ref/busy_timeout.html
+;
+;busy_timeout=1000
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/configs/samples/resolver_unbound.conf.sample 1:16.1.1~dfsg-1/configs/samples/resolver_unbound.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/resolver_unbound.conf.sample	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/resolver_unbound.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,23 @@
+; Unbound DNS Resolver Configuration
+;
+; This file serves as a reference for the configurable options within the
+; unbound DNS resolver.
+
+[general]
+;hosts = /etc/hosts        ; Full path to a hosts file which contains a mapping of
+;                          ; hostnames to addresses. If "system" is specified then
+;                          ; the system specific hosts file will be used. (default: system)
+;resolv = /etc/resolv.conf ; Full path to a resolv.conf which contains the nameservers
+;                          ; to use for resolution. If "system" is specified then the
+;                          ; system specific resolv.conf file will be used. (default: system)
+;nameserver = 127.0.0.1    ; An explicit nameserver to use for queries. If this option
+;                          ; is specified multiple times the first configured one will
+;                          ; be treated as the primary with each subsequent one being
+;                          ; a backup. If the resolv options is also specified the
+;                          ; nameservers from it will be tried after all nameserver
+;                          ; options.
+;debug = 99                ; The debug level to run the unbound resolver at. While
+;                          ; there is no explicit range the higher the number the more
+;                          ; debug is output.
+;ta_file = /etc/asterisk/dnssec_keys ; Full path to a trusted anchors key file. These keys are
+;                                    ; used to verify DNSSEC signed results.
diff -pruN 1:13.23.1~dfsg-2/configs/samples/res_parking.conf.sample 1:16.1.1~dfsg-1/configs/samples/res_parking.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/res_parking.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/res_parking.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -102,7 +102,7 @@ context => parkedcalls          ; Which
                                 ;         Dial() to 'SIP/0004F2040001'.
                                 ;
                                 ; During the timeout procedure, the following variables are set
-                                ; PARKINGSLOT - extension that the call was parked in prior to timing out
+                                ; PARKING_SPACE - extension that the call was parked in prior to timing out
                                 ; PARKEDLOT - name of the lot that the call was parked in prior to timing out
                                 ; PARKER - dial string to call the device that parked the call
 
diff -pruN 1:13.23.1~dfsg-2/configs/samples/rtp.conf.sample 1:16.1.1~dfsg-1/configs/samples/rtp.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/rtp.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/rtp.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -31,6 +31,10 @@ rtpend=20000
 ; seconds after starting learning mode.  Once learning mode completes the
 ; current stream is locked in and cannot change until the next
 ; renegotiation.
+; Valid options are "no" to disable strictrtp, "yes" to enable strictrtp,
+; and "seqno", which does the same thing as strictrtp=yes, but only checks
+; to make sure the sequence number is correct rather than checking the time
+; interval as well.
 ; This option is enabled by default.
 ; strictrtp=yes
 ;
diff -pruN 1:13.23.1~dfsg-2/configs/samples/sip.conf.sample 1:16.1.1~dfsg-1/configs/samples/sip.conf.sample
--- 1:13.23.1~dfsg-2/configs/samples/sip.conf.sample	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configs/samples/sip.conf.sample	2018-12-26 18:45:13.000000000 +0000
@@ -24,6 +24,9 @@
 ;        SIP/devicename/extension
 ;        SIP/devicename/extension/IPorHost
 ;        SIP/username@domain//IPorHost
+; And to alter the To: or the From: header, you can additionally append
+; the following to any of the above strings:
+;        [![touser[@todomain]][![fromuser][@fromdomain]]]
 ;
 ;
 ; Devicename
@@ -57,6 +60,14 @@
 ;
 ;         SIP/sales@mysipproxy!sales@edvina.net
 ;
+; (Specifying only @todomain without touser will create an invalid SIP
+; request.)
+;
+; Similarly, you can specify the From header as well, after a second
+; exclamation mark:
+;
+;         SIP/customer@mysipproxy!!customersupport@wearespindle.com
+;
 ; A new feature for 1.8 allows one to specify a host or IP address to use
 ; when routing the call. This is typically used in tandem with func_srv if
 ; multiple methods of reaching the same domain exist. The host or IP address
@@ -180,6 +191,9 @@ allowoverlap=no                 ; Disabl
 udpbindaddr=0.0.0.0             ; IP address to bind UDP listen socket to (0.0.0.0 binds to all)
                                 ; Optionally add a port number, 192.168.1.1:5062 (default is port 5060)
 
+;rtpbindaddr=172.16.42.1        ; IP address to bind RTP listen sock to (default is disabled). When
+                                ; disabled the udpbindaddr is used.
+
 ; When a dialog is started with another SIP endpoint, the other endpoint
 ; should include an Allow header telling us what SIP methods the endpoint
 ; implements. However, some endpoints either do not include an Allow header
@@ -562,7 +576,12 @@ srvlookup=yes                   ; Enable
 ; ----------------------- TLS settings ------------------------------------------------------------
 ;tlscertfile=</path/to/certificate.pem> ; Certificate chain (*.pem format only) to use for TLS connections
                                         ; The certificates must be sorted starting with the subject's certificate
-                                        ; and followed by intermediate CA certificates if applicable.
+                                        ; and followed by intermediate CA certificates if applicable. If the
+                                        ; file name ends in _rsa, for example "asterisk_rsa.pem", the files
+                                        ; "asterisk_dsa.pem" and/or "asterisk_ecc.pem" are loaded
+                                        ; (certificate, intermediates, private key), to support multiple
+                                        ; algorithms for server authentication (RSA, DSA, ECDSA). If the chains
+                                        ; are different, at least OpenSSL 1.0.2 is required.
                                         ; Default is to look for "asterisk.pem" in current directory
 
 ;tlsprivatekey=</path/to/private.pem> ; Private key file (*.pem format only) for TLS connections.
@@ -695,8 +714,12 @@ srvlookup=yes                   ; Enable
 ;subscribecontext = default     ; Set a specific context for SUBSCRIBE requests
                                 ; Useful to limit subscriptions to local extensions
                                 ; Settable per peer/user also
-;notifyringing = no             ; Control whether subscriptions already INUSE get sent
-                                ; RINGING when another call is sent (default: yes)
+;notifyringing = no             ; Control when subscriptions get notified of ringing state.
+                                ; Specify 'no' to not send any ringing notifications.
+                                ; Specify 'yes' to always send ringing notifications (default).
+                                ; Specify 'notinuse' to only send ringing notifications for
+                                ; extensions that are not currently in use. This is useful as a
+                                ; visual indication of who is available to pick up an incoming call
 ;notifyhold = yes               ; Notify subscriptions on HOLD state (default: no)
                                 ; Turning on notifyringing and notifyhold will add a lot
                                 ; more database transactions if you are using realtime.
@@ -772,7 +795,7 @@ srvlookup=yes                   ; Enable
 ; A similar effect can be achieved by adding a "callbackextension" option in a peer section.
 ; this is equivalent to having the following line in the general section:
 ;
-;        register => username:secret@host/callbackextension
+;        register => fromuser:secret:username@host/callbackextension
 ;
 ; and more readable because you don't have to write the parameters in two places
 ; (note that the "port" is ignored - this is a bug that should be fixed).
@@ -963,6 +986,10 @@ srvlookup=yes                   ; Enable
 ; the media_address configuration option. This is only applicable to the general section and
 ; can not be set per-user or per-peer.
 ;
+; Note that this does not change the listen address for RTP, it only changes the
+; advertised address in the SDP. The Asterisk RTP engine will still listen on
+; the standard IP address.
+;
 ; media_address = 172.16.42.1
 ;
 ; Through the use of the res_stun_monitor module, Asterisk has the ability to detect when the
@@ -1313,6 +1340,7 @@ srvlookup=yes                   ; Enable
 ; encryption
 ; description		; Used to provide a description of the peer in console output
 ; dtlsenable
+; dtlsautogeneratecert
 ; dtlsverify
 ; dtlsrekey
 ; dtlscertfile
@@ -1331,8 +1359,8 @@ srvlookup=yes                   ; Enable
 ;
 ; DTLS-SRTP support is available if the underlying RTP engine in use supports it.
 ;
-; Note: DTLS configuration must be set directly on a user, peer, or friend. Setting these
-;       options globally in the [general] section will have no effect.
+; Note that all configuration options except dtlsenable can be set at the general level.
+; If set they will be present on the user or peer unless overridden with a different value.
 ;
 ; dtlsenable = yes                   ; Enable or disable DTLS-SRTP support
 ; dtlsverify = yes                   ; Verify that provided peer certificate and fingerprint are valid
@@ -1342,6 +1370,7 @@ srvlookup=yes                   ; Enable
 ;				     ; A value of 'certificate' will perform ONLY certficiate verification
 ; dtlsrekey = 60                     ; Interval at which to renegotiate the TLS session and rekey the SRTP session
 ;                                    ; If this is not set or the value provided is 0 rekeying will be disabled
+; dtlsautogeneratecert = yes         ; Enable ephemeral DTLS certificate generation. The default is 'no.'
 ; dtlscertfile = file                ; Path to certificate file to present
 ; dtlsprivatekey = file              ; Path to private key for certificate file
 ; dtlscipher = <SSL cipher string>   ; Cipher to use for TLS negotiation
diff -pruN 1:13.23.1~dfsg-2/configure 1:16.1.1~dfsg-1/configure
--- 1:13.23.1~dfsg-2/configure	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configure	2018-12-26 18:45:13.000000000 +0000
@@ -708,10 +708,8 @@ JANSSON_LIBS
 JANSSON_CFLAGS
 UUID_LIB
 UUID_INCLUDE
-EDITLINE_LIB
 LIBEDIT_LIBS
 LIBEDIT_CFLAGS
-LIBEDIT_INTERNAL
 PKG_CONFIG_LIBDIR
 PKG_CONFIG_PATH
 PKG_CONFIG
@@ -736,22 +734,18 @@ PBX_UNIXODBC
 UNIXODBC_DIR
 UNIXODBC_INCLUDE
 UNIXODBC_LIB
+PBX_UNBOUND
+UNBOUND_DIR
+UNBOUND_INCLUDE
+UNBOUND_LIB
 PBX_TONEZONE
 TONEZONE_DIR
 TONEZONE_INCLUDE
 TONEZONE_LIB
-PBX_TINFO
-TINFO_DIR
-TINFO_INCLUDE
-TINFO_LIB
 PBX_TIMERFD
 TIMERFD_DIR
 TIMERFD_INCLUDE
 TIMERFD_LIB
-PBX_TERMCAP
-TERMCAP_DIR
-TERMCAP_INCLUDE
-TERMCAP_LIB
 PBX_FREETDS
 FREETDS_DIR
 FREETDS_INCLUDE
@@ -768,10 +762,26 @@ PBX_OPENSSL
 OPENSSL_DIR
 OPENSSL_INCLUDE
 OPENSSL_LIB
+PBX_SRTP_GET_VERSION
+SRTP_GET_VERSION_DIR
+SRTP_GET_VERSION_INCLUDE
+SRTP_GET_VERSION_LIB
 PBX_SRTP_SHUTDOWN
 SRTP_SHUTDOWN_DIR
 SRTP_SHUTDOWN_INCLUDE
 SRTP_SHUTDOWN_LIB
+PBX_SRTP_GCM
+SRTP_GCM_DIR
+SRTP_GCM_INCLUDE
+SRTP_GCM_LIB
+PBX_SRTP_192
+SRTP_192_DIR
+SRTP_192_INCLUDE
+SRTP_192_LIB
+PBX_SRTP_256
+SRTP_256_DIR
+SRTP_256_INCLUDE
+SRTP_256_LIB
 PBX_SRTP
 SRTP_DIR
 SRTP_INCLUDE
@@ -812,10 +822,18 @@ PBX_SDL
 SDL_DIR
 SDL_INCLUDE
 SDL_LIB
+PBX_SNDFILE
+SNDFILE_DIR
+SNDFILE_INCLUDE
+SNDFILE_LIB
 PBX_RESAMPLE
 RESAMPLE_DIR
 RESAMPLE_INCLUDE
 RESAMPLE_LIB
+PBX_FFTW3
+FFTW3_DIR
+FFTW3_INCLUDE
+FFTW3_LIB
 PBX_RADIUS
 RADIUS_DIR
 RADIUS_INCLUDE
@@ -912,6 +930,10 @@ PBX_POPT
 POPT_DIR
 POPT_INCLUDE
 POPT_LIB
+PBX_PJSIP_ENDPOINT_COMPACT_FORM
+PJSIP_ENDPOINT_COMPACT_FORM_DIR
+PJSIP_ENDPOINT_COMPACT_FORM_INCLUDE
+PJSIP_ENDPOINT_COMPACT_FORM_LIB
 PBX_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS
 PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS_DIR
 PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS_INCLUDE
@@ -960,6 +982,10 @@ PBX_PJSIP_DLG_CREATE_UAS_AND_INC_LOCK
 PJSIP_DLG_CREATE_UAS_AND_INC_LOCK_DIR
 PJSIP_DLG_CREATE_UAS_AND_INC_LOCK_INCLUDE
 PJSIP_DLG_CREATE_UAS_AND_INC_LOCK_LIB
+PBX_BEANSTALK
+BEANSTALK_DIR
+BEANSTALK_INCLUDE
+BEANSTALK_LIB
 PBX_PGSQL
 PGSQL_DIR
 PGSQL_INCLUDE
@@ -1004,10 +1030,6 @@ PBX_NEON
 NEON_DIR
 NEON_INCLUDE
 NEON_LIB
-PBX_NCURSES
-NCURSES_DIR
-NCURSES_INCLUDE
-NCURSES_LIB
 PBX_NBS
 NBS_DIR
 NBS_INCLUDE
@@ -1133,10 +1155,6 @@ PBX_CRYPT
 CRYPT_DIR
 CRYPT_INCLUDE
 CRYPT_LIB
-PBX_CURSES
-CURSES_DIR
-CURSES_INCLUDE
-CURSES_LIB
 PBX_COROSYNC_CFG_STATE_TRACK
 COROSYNC_CFG_STATE_TRACK_DIR
 COROSYNC_CFG_STATE_TRACK_INCLUDE
@@ -1145,6 +1163,10 @@ PBX_COROSYNC
 COROSYNC_DIR
 COROSYNC_INCLUDE
 COROSYNC_LIB
+PBX_CODEC2
+CODEC2_DIR
+CODEC2_INCLUDE
+CODEC2_LIB
 PBX_CAP
 CAP_DIR
 CAP_INCLUDE
@@ -1187,6 +1209,7 @@ AST_DOWNLOAD_CACHE
 AST_DEVMODE_STRICT
 AST_DEVMODE
 NOISY_BUILD
+PBX_NO_BINARY_MODULES
 PTHREAD_CFLAGS
 PTHREAD_LIBS
 PTHREAD_CC
@@ -1337,6 +1360,7 @@ ac_subst_files=''
 ac_user_opts='
 enable_option_checking
 with_gnu_ld
+enable_binary_modules
 enable_dev_mode
 with_download_cache
 with_sounds_cache
@@ -1349,8 +1373,8 @@ with_bfd
 with_execinfo
 with_bluetooth
 with_cap
+with_codec2
 with_cpg
-with_curses
 with_crypt
 with_crypto
 with_dahdi
@@ -1380,7 +1404,6 @@ with_lua
 with_misdn
 with_mysqlclient
 with_nbs
-with_ncurses
 with_neon
 with_neon29
 with_netsnmp
@@ -1392,12 +1415,15 @@ with_opusfile
 with_osptk
 with_oss
 with_postgres
+with_beanstalk
 with_pjproject
 with_popt
 with_portaudio
 with_pri
 with_radius
+with_fftw3
 with_resample
+with_sndfile
 with_sdl
 with_SDL_image
 with_spandsp
@@ -1410,10 +1436,9 @@ with_srtp
 with_ssl
 with_suppserv
 with_tds
-with_termcap
 with_timerfd
-with_tinfo
 with_tonezone
+with_unbound
 with_unixodbc
 with_vorbis
 with_vpb
@@ -2074,6 +2099,8 @@ Optional Features:
   --disable-option-checking  ignore unrecognized --enable/--with options
   --disable-FEATURE       do not include FEATURE (same as --enable-FEATURE=no)
   --enable-FEATURE[=ARG]  include FEATURE [ARG=yes]
+  --disable-binary-modules
+                          Block installation of binary modules.
   --enable-dev-mode       Turn on developer mode
   --enable-coverage       Turn on code coverage tracking (for gcov)
   --disable-xmldoc        Explicitly disable XML documentation
@@ -2095,14 +2122,14 @@ Optional Packages:
                           use cached external module tarfiles in PATH
   --with-jansson-bundled  Use bundled jansson library
   --with-pjproject-bundled
-                          Use bundled pjproject libraries
+                          Use bundled pjproject libraries (default)
   --with-asound=PATH      use Advanced Linux Sound Architecture files in PATH
   --with-bfd=PATH         use Debug symbol decoding files in PATH
   --with-execinfo=PATH    use Stack Backtrace files in PATH
   --with-bluetooth=PATH   use Bluetooth files in PATH
   --with-cap=PATH         use POSIX 1.e capabilities files in PATH
+  --with-codec2=PATH      use Codec 2 Audio Decoder/Encoder files in PATH
   --with-cpg=PATH         use Corosync files in PATH
-  --with-curses=PATH      use curses files in PATH
   --with-crypt=PATH       use password and data encryption files in PATH
   --with-crypto=PATH      use OpenSSL Cryptography files in PATH
   --with-dahdi=PATH       use DAHDI files in PATH
@@ -2128,15 +2155,13 @@ Optional Packages:
   --with-ldap=PATH        use OpenLDAP files in PATH
   --with-libcurl=PREFIX   look for the curl library in PREFIX/lib and headers
                           in PREFIX/include
-  --with-libedit=PATH     use NetBSD Editline library files in PATH, use
-                          'internal' Editline otherwise
+  --with-libedit=PATH     use NetBSD Editline library files in PATH
   --with-libxml2=PATH     use LibXML2 files in PATH
   --with-libxslt=PATH     use LibXSLT files in PATH
   --with-lua=PATH         use Lua files in PATH
   --with-misdn=PATH       use mISDN user files in PATH
   --with-mysqlclient=PATH use MySQL client files in PATH
   --with-nbs=PATH         use Network Broadcast Sound files in PATH
-  --with-ncurses=PATH     use ncurses files in PATH
   --with-neon=PATH        use neon files in PATH
   --with-neon29=PATH      use neon29 files in PATH
   --with-netsnmp=PATH     use Net-SNMP files in PATH
@@ -2148,12 +2173,15 @@ Optional Packages:
   --with-osptk=PATH       use OSP Toolkit files in PATH
   --with-oss=PATH         use Open Sound System files in PATH
   --with-postgres=PATH    use PostgreSQL files in PATH
+  --with-beanstalk=PATH   use Beanstalk Job Queue files in PATH
   --with-pjproject=PATH   use PJPROJECT files in PATH
   --with-popt=PATH        use popt files in PATH
   --with-portaudio=PATH   use PortAudio files in PATH
   --with-pri=PATH         use ISDN PRI files in PATH
   --with-radius=PATH      use Radius Client files in PATH
+  --with-fftw3=PATH       use LIBFFTW3 files in PATH
   --with-resample=PATH    use LIBRESAMPLE files in PATH
+  --with-sndfile=PATH     use libsndfile files in PATH
   --with-sdl=PATH         use Sdl files in PATH
   --with-SDL_image=PATH   use Sdl Image files in PATH
   --with-spandsp=PATH     use SPANDSP files in PATH
@@ -2167,10 +2195,9 @@ Optional Packages:
   --with-ssl=PATH         use OpenSSL Secure Sockets Layer files in PATH
   --with-suppserv=PATH    use mISDN Supplemental Services files in PATH
   --with-tds=PATH         use FreeTDS files in PATH
-  --with-termcap=PATH     use Termcap files in PATH
   --with-timerfd=PATH     use timerfd files in PATH
-  --with-tinfo=PATH       use Term Info files in PATH
   --with-tonezone=PATH    use tonezone files in PATH
+  --with-unbound=PATH     use unbound files in PATH
   --with-unixodbc=PATH    use unixODBC files in PATH
   --with-vorbis=PATH      use Vorbis files in PATH
   --with-vpb=PATH         use Voicetronix API files in PATH
@@ -4764,6 +4791,9 @@ case "${host_os}" in
      linux-gnu*)
      OSARCH=linux-gnu
      ;;
+     linux-musl*)
+     OSARCH=linux-musl
+     ;;
      kfreebsd*-gnu)
      OSARCH=kfreebsd-gnu
      ;;
@@ -8896,6 +8926,18 @@ ac_link='$CC -o conftest$ac_exeext $CFLA
 ac_compiler_gnu=$ac_cv_c_compiler_gnu
 
 
+PBX_NO_BINARY_MODULES=0
+# Check whether --enable-binary-modules was given.
+if test "${enable_binary_modules+set}" = set; then :
+  enableval=$enable_binary_modules; case "${enableval}" in
+	      y|ye|yes) PBX_NO_BINARY_MODULES=0 ;;
+	      n|no)  PBX_NO_BINARY_MODULES=1 ;;
+	      *) as_fn_error $? "bad value ${enableval} for --disable-binary-modules" "$LINENO" 5  ;;
+	esac
+fi
+
+
+
 # Check whether --enable-dev-mode was given.
 if test "${enable_dev_mode+set}" = set; then :
   enableval=$enable_dev_mode; case "${enableval}" in
@@ -9153,7 +9195,7 @@ if test "${with_jansson_bundled+set}" =
 fi
 
 
-PJPROJECT_BUNDLED=no
+PJPROJECT_BUNDLED=yes
 
 
 
@@ -9244,6 +9286,38 @@ $as_echo_n "checking for bundled jansson
 	JANSSON_LIB="-L${JANSSON_DIR}/dest/lib -ljansson"
 	PBX_JANSSON=1
 
+	# We haven't run install yet
+
+	# Define the ast_json_int_t (large integer type) to match jansson's
+	saved_cppflags="${CPPFLAGS}"
+	CPPFLAGS="${CPPFLAGS} ${JANSSON_INCLUDE}"
+	cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+#include <$JANSSON_DIR/source/src/jansson.h>
+int
+main ()
+{
+#if !JSON_INTEGER_IS_LONG_LONG
+		#error "not long long"
+		#endif
+
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_compile "$LINENO"; then :
+
+$as_echo "#define AST_JSON_INT_T long long" >>confdefs.h
+
+else
+
+$as_echo "#define AST_JSON_INT_T long" >>confdefs.h
+
+fi
+rm -f core conftest.err conftest.$ac_objext conftest.$ac_ext
+	CPPFLAGS="${saved_cppflags}"
+
+
 
 
 
@@ -9251,6 +9325,9 @@ $as_echo_n "checking for bundled jansson
 	{ $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
 $as_echo "yes" >&6; }
 
+$as_echo "#define HAVE_JANSSON_BUNDLED 1" >>confdefs.h
+
+
 	fi
 
 
@@ -9326,7 +9403,7 @@ $as_echo "configuring" >&6; }
 	fi
 
 	export TAR PATCH SED NM EXTERNALS_CACHE_DIR AST_DOWNLOAD_CACHE DOWNLOAD_TO_STDOUT DOWNLOAD_TIMEOUT DOWNLOAD MD5 CAT CUT GREP
-	export NOISY_BUILD
+	export NOISY_BUILD AST_DEVMODE
 	${GNU_MAKE} --quiet --no-print-directory -C ${PJPROJECT_DIR} \
 		PJPROJECT_CONFIGURE_OPTS="$PJPROJECT_CONFIGURE_OPTS" \
 		EXTERNALS_CACHE_DIR="${EXTERNALS_CACHE_DIR:-${AST_DOWNLOAD_CACHE}}" \
@@ -9390,6 +9467,9 @@ $as_echo "#define HAVE_PJSIP_TSX_LAYER_F
 $as_echo "#define HAVE_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS 1" >>confdefs.h
 
 
+$as_echo "#define HAVE_PJSIP_ENDPOINT_COMPACT_FORM 1" >>confdefs.h
+
+
 
 
 
@@ -9576,6 +9656,38 @@ fi
 
 
 
+    CODEC2_DESCRIP="Codec 2 Audio Decoder/Encoder"
+    CODEC2_OPTION="codec2"
+    PBX_CODEC2=0
+
+# Check whether --with-codec2 was given.
+if test "${with_codec2+set}" = set; then :
+  withval=$with_codec2;
+	case ${withval} in
+	n|no)
+	USE_CODEC2=no
+	# -1 is a magic value used by menuselect to know that the package
+	# was disabled, other than 'not found'
+	PBX_CODEC2=-1
+	;;
+	y|ye|yes)
+	ac_mandatory_list="${ac_mandatory_list} CODEC2"
+	;;
+	*)
+	CODEC2_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} CODEC2"
+	;;
+	esac
+
+fi
+
+
+
+
+
+
+
+
     COROSYNC_DESCRIP="Corosync"
     COROSYNC_OPTION="cpg"
     PBX_COROSYNC=0
@@ -9620,38 +9732,6 @@ PBX_COROSYNC_CFG_STATE_TRACK=0
 
 
 
-    CURSES_DESCRIP="curses"
-    CURSES_OPTION="curses"
-    PBX_CURSES=0
-
-# Check whether --with-curses was given.
-if test "${with_curses+set}" = set; then :
-  withval=$with_curses;
-	case ${withval} in
-	n|no)
-	USE_CURSES=no
-	# -1 is a magic value used by menuselect to know that the package
-	# was disabled, other than 'not found'
-	PBX_CURSES=-1
-	;;
-	y|ye|yes)
-	ac_mandatory_list="${ac_mandatory_list} CURSES"
-	;;
-	*)
-	CURSES_DIR="${withval}"
-	ac_mandatory_list="${ac_mandatory_list} CURSES"
-	;;
-	esac
-
-fi
-
-
-
-
-
-
-
-
     CRYPT_DESCRIP="password and data encryption"
     CRYPT_OPTION="crypt"
     PBX_CRYPT=0
@@ -10971,38 +11051,6 @@ fi
 
 
 
-    NCURSES_DESCRIP="ncurses"
-    NCURSES_OPTION="ncurses"
-    PBX_NCURSES=0
-
-# Check whether --with-ncurses was given.
-if test "${with_ncurses+set}" = set; then :
-  withval=$with_ncurses;
-	case ${withval} in
-	n|no)
-	USE_NCURSES=no
-	# -1 is a magic value used by menuselect to know that the package
-	# was disabled, other than 'not found'
-	PBX_NCURSES=-1
-	;;
-	y|ye|yes)
-	ac_mandatory_list="${ac_mandatory_list} NCURSES"
-	;;
-	*)
-	NCURSES_DIR="${withval}"
-	ac_mandatory_list="${ac_mandatory_list} NCURSES"
-	;;
-	esac
-
-fi
-
-
-
-
-
-
-
-
     NEON_DESCRIP="neon"
     NEON_OPTION="neon"
     PBX_NEON=0
@@ -11355,6 +11403,38 @@ fi
 
 
 
+    BEANSTALK_DESCRIP="Beanstalk Job Queue"
+    BEANSTALK_OPTION="beanstalk"
+    PBX_BEANSTALK=0
+
+# Check whether --with-beanstalk was given.
+if test "${with_beanstalk+set}" = set; then :
+  withval=$with_beanstalk;
+	case ${withval} in
+	n|no)
+	USE_BEANSTALK=no
+	# -1 is a magic value used by menuselect to know that the package
+	# was disabled, other than 'not found'
+	PBX_BEANSTALK=-1
+	;;
+	y|ye|yes)
+	ac_mandatory_list="${ac_mandatory_list} BEANSTALK"
+	;;
+	*)
+	BEANSTALK_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} BEANSTALK"
+	;;
+	esac
+
+fi
+
+
+
+
+
+
+
+
 if test "x${PBX_PJPROJECT}" != "x1" ; then
 
     PJPROJECT_DESCRIP="PJPROJECT"
@@ -11532,6 +11612,18 @@ PBX_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWER
 
 
 
+
+PJSIP_ENDPOINT_COMPACT_FORM_DESCRIP="PJSIP Compact Form Support on Endpoint"
+PJSIP_ENDPOINT_COMPACT_FORM_OPTION=pjsip
+PJSIP_ENDPOINT_COMPACT_FORM_DIR=${PJPROJECT_DIR}
+
+PBX_PJSIP_ENDPOINT_COMPACT_FORM=0
+
+
+
+
+
+
 fi
 
 
@@ -11907,6 +11999,38 @@ fi
 
 
 
+    FFTW3_DESCRIP="LIBFFTW3"
+    FFTW3_OPTION="fftw3"
+    PBX_FFTW3=0
+
+# Check whether --with-fftw3 was given.
+if test "${with_fftw3+set}" = set; then :
+  withval=$with_fftw3;
+	case ${withval} in
+	n|no)
+	USE_FFTW3=no
+	# -1 is a magic value used by menuselect to know that the package
+	# was disabled, other than 'not found'
+	PBX_FFTW3=-1
+	;;
+	y|ye|yes)
+	ac_mandatory_list="${ac_mandatory_list} FFTW3"
+	;;
+	*)
+	FFTW3_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} FFTW3"
+	;;
+	esac
+
+fi
+
+
+
+
+
+
+
+
     RESAMPLE_DESCRIP="LIBRESAMPLE"
     RESAMPLE_OPTION="resample"
     PBX_RESAMPLE=0
@@ -11939,6 +12063,38 @@ fi
 
 
 
+    SNDFILE_DESCRIP="libsndfile"
+    SNDFILE_OPTION="sndfile"
+    PBX_SNDFILE=0
+
+# Check whether --with-sndfile was given.
+if test "${with_sndfile+set}" = set; then :
+  withval=$with_sndfile;
+	case ${withval} in
+	n|no)
+	USE_SNDFILE=no
+	# -1 is a magic value used by menuselect to know that the package
+	# was disabled, other than 'not found'
+	PBX_SNDFILE=-1
+	;;
+	y|ye|yes)
+	ac_mandatory_list="${ac_mandatory_list} SNDFILE"
+	;;
+	*)
+	SNDFILE_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} SNDFILE"
+	;;
+	esac
+
+fi
+
+
+
+
+
+
+
+
     SDL_DESCRIP="Sdl"
     SDL_OPTION="sdl"
     PBX_SDL=0
@@ -12270,6 +12426,42 @@ fi
 
 
 
+SRTP_256_DESCRIP="SRTP Library AES-256 (ICM)"
+SRTP_256_OPTION=srtp
+SRTP_256_DIR=${SRTP_DIR}
+
+PBX_SRTP_256=0
+
+
+
+
+
+
+
+SRTP_192_DESCRIP="SRTP Library AES-192 (ICM)"
+SRTP_192_OPTION=srtp
+SRTP_192_DIR=${SRTP_DIR}
+
+PBX_SRTP_192=0
+
+
+
+
+
+
+
+SRTP_GCM_DESCRIP="SRTP Library AES-128 (GCM) and AES-256 (GCM)"
+SRTP_GCM_OPTION=srtp
+SRTP_GCM_DIR=${SRTP_DIR}
+
+PBX_SRTP_GCM=0
+
+
+
+
+
+
+
 SRTP_SHUTDOWN_DESCRIP="SRTP Library Shutdown Function"
 SRTP_SHUTDOWN_OPTION=srtp
 SRTP_SHUTDOWN_DIR=${SRTP_DIR}
@@ -12282,6 +12474,18 @@ PBX_SRTP_SHUTDOWN=0
 
 
 
+SRTP_GET_VERSION_DESCRIP="SRTP Library Version Function"
+SRTP_GET_VERSION_OPTION=srtp
+SRTP_GET_VERSION_DIR=${SRTP_DIR}
+
+PBX_SRTP_GET_VERSION=0
+
+
+
+
+
+
+
     OPENSSL_DESCRIP="OpenSSL Secure Sockets Layer"
     OPENSSL_OPTION="ssl"
     PBX_OPENSSL=0
@@ -12390,38 +12594,6 @@ fi
 
 
 
-    TERMCAP_DESCRIP="Termcap"
-    TERMCAP_OPTION="termcap"
-    PBX_TERMCAP=0
-
-# Check whether --with-termcap was given.
-if test "${with_termcap+set}" = set; then :
-  withval=$with_termcap;
-	case ${withval} in
-	n|no)
-	USE_TERMCAP=no
-	# -1 is a magic value used by menuselect to know that the package
-	# was disabled, other than 'not found'
-	PBX_TERMCAP=-1
-	;;
-	y|ye|yes)
-	ac_mandatory_list="${ac_mandatory_list} TERMCAP"
-	;;
-	*)
-	TERMCAP_DIR="${withval}"
-	ac_mandatory_list="${ac_mandatory_list} TERMCAP"
-	;;
-	esac
-
-fi
-
-
-
-
-
-
-
-
     TIMERFD_DESCRIP="timerfd"
     TIMERFD_OPTION="timerfd"
     PBX_TIMERFD=0
@@ -12454,26 +12626,26 @@ fi
 
 
 
-    TINFO_DESCRIP="Term Info"
-    TINFO_OPTION="tinfo"
-    PBX_TINFO=0
-
-# Check whether --with-tinfo was given.
-if test "${with_tinfo+set}" = set; then :
-  withval=$with_tinfo;
+    TONEZONE_DESCRIP="tonezone"
+    TONEZONE_OPTION="tonezone"
+    PBX_TONEZONE=0
+
+# Check whether --with-tonezone was given.
+if test "${with_tonezone+set}" = set; then :
+  withval=$with_tonezone;
 	case ${withval} in
 	n|no)
-	USE_TINFO=no
+	USE_TONEZONE=no
 	# -1 is a magic value used by menuselect to know that the package
 	# was disabled, other than 'not found'
-	PBX_TINFO=-1
+	PBX_TONEZONE=-1
 	;;
 	y|ye|yes)
-	ac_mandatory_list="${ac_mandatory_list} TINFO"
+	ac_mandatory_list="${ac_mandatory_list} TONEZONE"
 	;;
 	*)
-	TINFO_DIR="${withval}"
-	ac_mandatory_list="${ac_mandatory_list} TINFO"
+	TONEZONE_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} TONEZONE"
 	;;
 	esac
 
@@ -12486,26 +12658,26 @@ fi
 
 
 
-    TONEZONE_DESCRIP="tonezone"
-    TONEZONE_OPTION="tonezone"
-    PBX_TONEZONE=0
-
-# Check whether --with-tonezone was given.
-if test "${with_tonezone+set}" = set; then :
-  withval=$with_tonezone;
+    UNBOUND_DESCRIP="unbound"
+    UNBOUND_OPTION="unbound"
+    PBX_UNBOUND=0
+
+# Check whether --with-unbound was given.
+if test "${with_unbound+set}" = set; then :
+  withval=$with_unbound;
 	case ${withval} in
 	n|no)
-	USE_TONEZONE=no
+	USE_UNBOUND=no
 	# -1 is a magic value used by menuselect to know that the package
 	# was disabled, other than 'not found'
-	PBX_TONEZONE=-1
+	PBX_UNBOUND=-1
 	;;
 	y|ye|yes)
-	ac_mandatory_list="${ac_mandatory_list} TONEZONE"
+	ac_mandatory_list="${ac_mandatory_list} UNBOUND"
 	;;
 	*)
-	TONEZONE_DIR="${withval}"
-	ac_mandatory_list="${ac_mandatory_list} TONEZONE"
+	UNBOUND_DIR="${withval}"
+	ac_mandatory_list="${ac_mandatory_list} UNBOUND"
 	;;
 	esac
 
@@ -13036,6 +13208,10 @@ fi
 
 fi
 
+if test -z $ac_header_dirent -o "$ac_header_dirent" = "no"; then
+  as_fn_error $? "*** Could not find dirent header that defines 'DIR'." "$LINENO" 5
+fi
+
 { $as_echo "$as_me:${as_lineno-$LINENO}: checking for ANSI C header files" >&5
 $as_echo_n "checking for ANSI C header files... " >&6; }
 if ${ac_cv_header_stdc+:} false; then :
@@ -13148,6 +13324,10 @@ $as_echo "#define STDC_HEADERS 1" >>conf
 
 fi
 
+if test "$ac_cv_header_stdc" != "yes"; then
+  as_fn_error $? "*** ANSI C header files not found." "$LINENO" 5
+fi
+
 { $as_echo "$as_me:${as_lineno-$LINENO}: checking for sys/wait.h that is POSIX.1 compatible" >&5
 $as_echo_n "checking for sys/wait.h that is POSIX.1 compatible... " >&6; }
 if ${ac_cv_header_sys_wait_h+:} false; then :
@@ -13189,10 +13369,26 @@ $as_echo "#define HAVE_SYS_WAIT_H 1" >>c
 
 fi
 
-for ac_header in arpa/inet.h fcntl.h inttypes.h libintl.h limits.h locale.h malloc.h netdb.h netinet/in.h stddef.h stdint.h stdlib.h string.h strings.h sys/event.h sys/file.h sys/ioctl.h sys/param.h sys/socket.h sys/time.h syslog.h termios.h unistd.h utime.h arpa/nameser.h sys/io.h
+if test "$ac_cv_header_sys_wait_h" != "yes"; then
+  as_fn_error $? "*** POSIX.1 compatible sys/wait.h is required." "$LINENO" 5
+fi
+
+for ac_header in sys/types.h netinet/in.h arpa/nameser.h netdb.h resolv.h
 do :
   as_ac_Header=`$as_echo "ac_cv_header_$ac_header" | $as_tr_sh`
-ac_fn_c_check_header_mongrel "$LINENO" "$ac_header" "$as_ac_Header" "$ac_includes_default"
+ac_fn_c_check_header_compile "$LINENO" "$ac_header" "$as_ac_Header" "#ifdef HAVE_SYS_TYPES_H
+#  include <sys/types.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#  include <netinet/in.h>   /* inet_ functions / structs */
+#endif
+#ifdef HAVE_ARPA_NAMESER_H
+#  include <arpa/nameser.h> /* DNS HEADER struct */
+#endif
+#ifdef HAVE_NETDB_H
+#  include <netdb.h>
+#endif
+"
 if eval test \"x\$"$as_ac_Header"\" = x"yes"; then :
   cat >>confdefs.h <<_ACEOF
 #define `$as_echo "HAVE_$ac_header" | $as_tr_cpp` 1
@@ -13203,13 +13399,58 @@ fi
 done
 
 
+if test "$ac_cv_header_resolv_h" != "yes"; then
+  as_fn_error $? "*** resolv.h is required." "$LINENO" 5
+fi
+
+# Check for headers that are unconditionally required on all platforms.  Do not use
+# this to check for headers that are only needed by modules.
+for ac_header in  \
+  arpa/nameser.h assert.h ctype.h dlfcn.h errno.h fcntl.h float.h grp.h \
+  inttypes.h limits.h locale.h math.h pwd.h netinet/in.h regex.h \
+  sched.h stdarg.h stdint.h stdio.h stdlib.h string.h syslog.h \
+  sys/file.h sys/ioctl.h sys/param.h sys/resource.h sys/socket.h sys/stat.h \
+  sys/time.h sys/types.h sys/un.h \
+  termios.h time.h unistd.h \
+
+do :
+  as_ac_Header=`$as_echo "ac_cv_header_$ac_header" | $as_tr_sh`
+ac_fn_c_check_header_mongrel "$LINENO" "$ac_header" "$as_ac_Header" "$ac_includes_default"
+if eval test \"x\$"$as_ac_Header"\" = x"yes"; then :
+  cat >>confdefs.h <<_ACEOF
+#define `$as_echo "HAVE_$ac_header" | $as_tr_cpp` 1
+_ACEOF
+
+else
+
+  as_fn_error $? "*** A required header was not found." "$LINENO" 5
+
+fi
+
+done
+
+
+for ac_header in arpa/inet.h libintl.h malloc.h netdb.h stddef.h strings.h sys/event.h utime.h
+do :
+  as_ac_Header=`$as_echo "ac_cv_header_$ac_header" | $as_tr_sh`
+ac_fn_c_check_header_mongrel "$LINENO" "$ac_header" "$as_ac_Header" "$ac_includes_default"
+if eval test \"x\$"$as_ac_Header"\" = x"yes"; then :
+  cat >>confdefs.h <<_ACEOF
+#define `$as_echo "HAVE_$ac_header" | $as_tr_cpp` 1
+_ACEOF
+
+fi
+
+done
+
+
+
+
+
+
 
 
 
-
-
-
-
 if test "x$ac_cv_env_PKG_CONFIG_set" != "xset"; then
 	if test -n "$ac_tool_prefix"; then
   # Extract the first word of "${ac_tool_prefix}pkg-config", so it can be a program name with args.
@@ -13325,17 +13566,6 @@ $as_echo "no" >&6; }
 fi
 
 # Find required NetBSD Editline library (libedit).
-EDITLINE_LIB=""
-LIBEDIT_INTERNAL="yes"
-
-LIBEDIT_SYSTEM="yes"
-if test "${USE_LIBEDIT}" != "no"; then
-   if test "${LIBEDIT_DIR}" = "internal"; then
-      LIBEDIT_SYSTEM="no"
-   elif test "${LIBEDIT_DIR}" != ""; then
-      LIBEDIT_INTERNAL="no"
-   fi
-   if test "${LIBEDIT_SYSTEM}" = "yes"; then
 
    if test "x${PBX_LIBEDIT}" != "x1" -a "${USE_LIBEDIT}" != "no"; then
 
@@ -13424,7 +13654,7 @@ $as_echo "#define HAVE_LIBEDIT 1" >>conf
 fi
    fi
 
-      # some platforms do not list libedit via pkg-config, for example OpenBSD 6.2
+# some platforms do not list libedit via pkg-config, for example OpenBSD 6.2
 
 if test "x${PBX_LIBEDIT}" != "x1" -a "${USE_LIBEDIT}" != "no"; then
    pbxlibdir=""
@@ -13521,14 +13751,10 @@ _ACEOF
 fi
 
 
-      if test "$PBX_LIBEDIT" = "1"; then
-         LIBEDIT_INTERNAL="no"
-      fi
-   fi
-   if test "${LIBEDIT_INTERNAL}" = "yes"; then
-      PBX_LIBEDIT=1
-      LIBEDIT_IS_UNICODE=no
-   else
+if test "${PBX_LIBEDIT}" != 1; then
+	as_fn_error $? "*** Please install the 'libedit' development package." "$LINENO" 5
+	exit 1
+fi
 
     if test "x${PBX_LIBEDIT_IS_UNICODE}" != "x1" -a "${USE_LIBEDIT_IS_UNICODE}" != "no"; then
 	{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for Testing for libedit unicode support" >&5
@@ -13569,395 +13795,6 @@ rm -f core conftest.err conftest.$ac_obj
 	CPPFLAGS="${saved_cppflags}"
     fi
 
-   fi
-fi
-# If the Asterisk internal copy should be build (user said so) or
-# must be build (system does not have it), either -ltermcap,
-# -ltinfo, or -l{n}curses must be available.
-if test "${LIBEDIT_INTERNAL}" = "yes"; then
-
-if test "x${PBX_TERMCAP}" != "x1" -a "${USE_TERMCAP}" != "no"; then
-   pbxlibdir=""
-   # if --with-TERMCAP=DIR has been specified, use it.
-   if test "x${TERMCAP_DIR}" != "x"; then
-      if test -d ${TERMCAP_DIR}/lib; then
-         pbxlibdir="-L${TERMCAP_DIR}/lib"
-      else
-         pbxlibdir="-L${TERMCAP_DIR}"
-      fi
-   fi
-
-      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
-      CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for tgetent in -ltermcap" >&5
-$as_echo_n "checking for tgetent in -ltermcap... " >&6; }
-if ${ac_cv_lib_termcap_tgetent+:} false; then :
-  $as_echo_n "(cached) " >&6
-else
-  ac_check_lib_save_LIBS=$LIBS
-LIBS="-ltermcap ${pbxlibdir}  $LIBS"
-cat confdefs.h - <<_ACEOF >conftest.$ac_ext
-/* end confdefs.h.  */
-
-/* Override any GCC internal prototype to avoid an error.
-   Use char because int might match the return type of a GCC
-   builtin and then its argument prototype would still apply.  */
-#ifdef __cplusplus
-extern "C"
-#endif
-char tgetent ();
-int
-main ()
-{
-return tgetent ();
-  ;
-  return 0;
-}
-_ACEOF
-if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_termcap_tgetent=yes
-else
-  ac_cv_lib_termcap_tgetent=no
-fi
-rm -f core conftest.err conftest.$ac_objext \
-    conftest$ac_exeext conftest.$ac_ext
-LIBS=$ac_check_lib_save_LIBS
-fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_termcap_tgetent" >&5
-$as_echo "$ac_cv_lib_termcap_tgetent" >&6; }
-if test "x$ac_cv_lib_termcap_tgetent" = xyes; then :
-  AST_TERMCAP_FOUND=yes
-else
-  AST_TERMCAP_FOUND=no
-fi
-
-      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
-
-
-   # now check for the header.
-   if test "${AST_TERMCAP_FOUND}" = "yes"; then
-      TERMCAP_LIB="${pbxlibdir} -ltermcap "
-      # if --with-TERMCAP=DIR has been specified, use it.
-      if test "x${TERMCAP_DIR}" != "x"; then
-         TERMCAP_INCLUDE="-I${TERMCAP_DIR}/include"
-      fi
-      TERMCAP_INCLUDE="${TERMCAP_INCLUDE} "
-
-         # no header, assume found
-         TERMCAP_HEADER_FOUND="1"
-
-      if test "x${TERMCAP_HEADER_FOUND}" = "x0" ; then
-         TERMCAP_LIB=""
-         TERMCAP_INCLUDE=""
-      else
-
-         PBX_TERMCAP=1
-         cat >>confdefs.h <<_ACEOF
-#define HAVE_TERMCAP 1
-_ACEOF
-
-      fi
-   fi
-fi
-
-
-   if test "x$TERMCAP_LIB" != "x" ; then
-      EDITLINE_LIB="$TERMCAP_LIB"
-   else
-
-if test "x${PBX_TINFO}" != "x1" -a "${USE_TINFO}" != "no"; then
-   pbxlibdir=""
-   # if --with-TINFO=DIR has been specified, use it.
-   if test "x${TINFO_DIR}" != "x"; then
-      if test -d ${TINFO_DIR}/lib; then
-         pbxlibdir="-L${TINFO_DIR}/lib"
-      else
-         pbxlibdir="-L${TINFO_DIR}"
-      fi
-   fi
-
-      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
-      CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for tgetent in -ltinfo" >&5
-$as_echo_n "checking for tgetent in -ltinfo... " >&6; }
-if ${ac_cv_lib_tinfo_tgetent+:} false; then :
-  $as_echo_n "(cached) " >&6
-else
-  ac_check_lib_save_LIBS=$LIBS
-LIBS="-ltinfo ${pbxlibdir}  $LIBS"
-cat confdefs.h - <<_ACEOF >conftest.$ac_ext
-/* end confdefs.h.  */
-
-/* Override any GCC internal prototype to avoid an error.
-   Use char because int might match the return type of a GCC
-   builtin and then its argument prototype would still apply.  */
-#ifdef __cplusplus
-extern "C"
-#endif
-char tgetent ();
-int
-main ()
-{
-return tgetent ();
-  ;
-  return 0;
-}
-_ACEOF
-if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_tinfo_tgetent=yes
-else
-  ac_cv_lib_tinfo_tgetent=no
-fi
-rm -f core conftest.err conftest.$ac_objext \
-    conftest$ac_exeext conftest.$ac_ext
-LIBS=$ac_check_lib_save_LIBS
-fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_tinfo_tgetent" >&5
-$as_echo "$ac_cv_lib_tinfo_tgetent" >&6; }
-if test "x$ac_cv_lib_tinfo_tgetent" = xyes; then :
-  AST_TINFO_FOUND=yes
-else
-  AST_TINFO_FOUND=no
-fi
-
-      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
-
-
-   # now check for the header.
-   if test "${AST_TINFO_FOUND}" = "yes"; then
-      TINFO_LIB="${pbxlibdir} -ltinfo "
-      # if --with-TINFO=DIR has been specified, use it.
-      if test "x${TINFO_DIR}" != "x"; then
-         TINFO_INCLUDE="-I${TINFO_DIR}/include"
-      fi
-      TINFO_INCLUDE="${TINFO_INCLUDE} "
-
-         # no header, assume found
-         TINFO_HEADER_FOUND="1"
-
-      if test "x${TINFO_HEADER_FOUND}" = "x0" ; then
-         TINFO_LIB=""
-         TINFO_INCLUDE=""
-      else
-
-         PBX_TINFO=1
-         cat >>confdefs.h <<_ACEOF
-#define HAVE_TINFO 1
-_ACEOF
-
-      fi
-   fi
-fi
-
-
-      if test "x$TINFO_LIB" != "x" ; then
-         EDITLINE_LIB="$TINFO_LIB"
-      else
-
-if test "x${PBX_CURSES}" != "x1" -a "${USE_CURSES}" != "no"; then
-   pbxlibdir=""
-   # if --with-CURSES=DIR has been specified, use it.
-   if test "x${CURSES_DIR}" != "x"; then
-      if test -d ${CURSES_DIR}/lib; then
-         pbxlibdir="-L${CURSES_DIR}/lib"
-      else
-         pbxlibdir="-L${CURSES_DIR}"
-      fi
-   fi
-
-      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
-      CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for initscr in -lcurses" >&5
-$as_echo_n "checking for initscr in -lcurses... " >&6; }
-if ${ac_cv_lib_curses_initscr+:} false; then :
-  $as_echo_n "(cached) " >&6
-else
-  ac_check_lib_save_LIBS=$LIBS
-LIBS="-lcurses ${pbxlibdir}  $LIBS"
-cat confdefs.h - <<_ACEOF >conftest.$ac_ext
-/* end confdefs.h.  */
-
-/* Override any GCC internal prototype to avoid an error.
-   Use char because int might match the return type of a GCC
-   builtin and then its argument prototype would still apply.  */
-#ifdef __cplusplus
-extern "C"
-#endif
-char initscr ();
-int
-main ()
-{
-return initscr ();
-  ;
-  return 0;
-}
-_ACEOF
-if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_curses_initscr=yes
-else
-  ac_cv_lib_curses_initscr=no
-fi
-rm -f core conftest.err conftest.$ac_objext \
-    conftest$ac_exeext conftest.$ac_ext
-LIBS=$ac_check_lib_save_LIBS
-fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_curses_initscr" >&5
-$as_echo "$ac_cv_lib_curses_initscr" >&6; }
-if test "x$ac_cv_lib_curses_initscr" = xyes; then :
-  AST_CURSES_FOUND=yes
-else
-  AST_CURSES_FOUND=no
-fi
-
-      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
-
-
-   # now check for the header.
-   if test "${AST_CURSES_FOUND}" = "yes"; then
-      CURSES_LIB="${pbxlibdir} -lcurses "
-      # if --with-CURSES=DIR has been specified, use it.
-      if test "x${CURSES_DIR}" != "x"; then
-         CURSES_INCLUDE="-I${CURSES_DIR}/include"
-      fi
-      CURSES_INCLUDE="${CURSES_INCLUDE} "
-
-         # check for the header
-         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
-         CPPFLAGS="${CPPFLAGS} ${CURSES_INCLUDE}"
-         ac_fn_c_check_header_mongrel "$LINENO" "curses.h" "ac_cv_header_curses_h" "$ac_includes_default"
-if test "x$ac_cv_header_curses_h" = xyes; then :
-  CURSES_HEADER_FOUND=1
-else
-  CURSES_HEADER_FOUND=0
-fi
-
-
-         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
-
-      if test "x${CURSES_HEADER_FOUND}" = "x0" ; then
-         CURSES_LIB=""
-         CURSES_INCLUDE=""
-      else
-
-         PBX_CURSES=1
-         cat >>confdefs.h <<_ACEOF
-#define HAVE_CURSES 1
-_ACEOF
-
-      fi
-   fi
-fi
-
-
-         if test "x$CURSES_LIB" != "x" ; then
-            EDITLINE_LIB="$CURSES_LIB"
-         else
-
-if test "x${PBX_NCURSES}" != "x1" -a "${USE_NCURSES}" != "no"; then
-   pbxlibdir=""
-   # if --with-NCURSES=DIR has been specified, use it.
-   if test "x${NCURSES_DIR}" != "x"; then
-      if test -d ${NCURSES_DIR}/lib; then
-         pbxlibdir="-L${NCURSES_DIR}/lib"
-      else
-         pbxlibdir="-L${NCURSES_DIR}"
-      fi
-   fi
-
-      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
-      CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for initscr in -lncurses" >&5
-$as_echo_n "checking for initscr in -lncurses... " >&6; }
-if ${ac_cv_lib_ncurses_initscr+:} false; then :
-  $as_echo_n "(cached) " >&6
-else
-  ac_check_lib_save_LIBS=$LIBS
-LIBS="-lncurses ${pbxlibdir}  $LIBS"
-cat confdefs.h - <<_ACEOF >conftest.$ac_ext
-/* end confdefs.h.  */
-
-/* Override any GCC internal prototype to avoid an error.
-   Use char because int might match the return type of a GCC
-   builtin and then its argument prototype would still apply.  */
-#ifdef __cplusplus
-extern "C"
-#endif
-char initscr ();
-int
-main ()
-{
-return initscr ();
-  ;
-  return 0;
-}
-_ACEOF
-if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_ncurses_initscr=yes
-else
-  ac_cv_lib_ncurses_initscr=no
-fi
-rm -f core conftest.err conftest.$ac_objext \
-    conftest$ac_exeext conftest.$ac_ext
-LIBS=$ac_check_lib_save_LIBS
-fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_ncurses_initscr" >&5
-$as_echo "$ac_cv_lib_ncurses_initscr" >&6; }
-if test "x$ac_cv_lib_ncurses_initscr" = xyes; then :
-  AST_NCURSES_FOUND=yes
-else
-  AST_NCURSES_FOUND=no
-fi
-
-      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
-
-
-   # now check for the header.
-   if test "${AST_NCURSES_FOUND}" = "yes"; then
-      NCURSES_LIB="${pbxlibdir} -lncurses "
-      # if --with-NCURSES=DIR has been specified, use it.
-      if test "x${NCURSES_DIR}" != "x"; then
-         NCURSES_INCLUDE="-I${NCURSES_DIR}/include"
-      fi
-      NCURSES_INCLUDE="${NCURSES_INCLUDE} "
-
-         # check for the header
-         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
-         CPPFLAGS="${CPPFLAGS} ${NCURSES_INCLUDE}"
-         ac_fn_c_check_header_mongrel "$LINENO" "ncurses.h" "ac_cv_header_ncurses_h" "$ac_includes_default"
-if test "x$ac_cv_header_ncurses_h" = xyes; then :
-  NCURSES_HEADER_FOUND=1
-else
-  NCURSES_HEADER_FOUND=0
-fi
-
-
-         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
-
-      if test "x${NCURSES_HEADER_FOUND}" = "x0" ; then
-         NCURSES_LIB=""
-         NCURSES_INCLUDE=""
-      else
-
-         PBX_NCURSES=1
-         cat >>confdefs.h <<_ACEOF
-#define HAVE_NCURSES 1
-_ACEOF
-
-      fi
-   fi
-fi
-
-
-            if test "x$NCURSES_LIB" != "x" ; then
-               EDITLINE_LIB="$NCURSES_LIB"
-            else
-               as_fn_error $? "*** termcap support not found (on modern systems, this typically means the ncurses development package is missing)" "$LINENO" 5
-            fi
-         fi
-      fi
-   fi
-fi
-
 
 # Find required UUID support.
 #  * -luuid on Linux
@@ -14200,12 +14037,12 @@ if test -n "$JANSSON_CFLAGS"; then
     pkg_cv_JANSSON_CFLAGS="$JANSSON_CFLAGS"
  elif test -n "$PKG_CONFIG"; then
     if test -n "$PKG_CONFIG" && \
-    { { $as_echo "$as_me:${as_lineno-$LINENO}: \$PKG_CONFIG --exists --print-errors \"jansson\""; } >&5
-  ($PKG_CONFIG --exists --print-errors "jansson") 2>&5
+    { { $as_echo "$as_me:${as_lineno-$LINENO}: \$PKG_CONFIG --exists --print-errors \"jansson >= 2.11\""; } >&5
+  ($PKG_CONFIG --exists --print-errors "jansson >= 2.11") 2>&5
   ac_status=$?
   $as_echo "$as_me:${as_lineno-$LINENO}: \$? = $ac_status" >&5
   test $ac_status = 0; }; then
-  pkg_cv_JANSSON_CFLAGS=`$PKG_CONFIG --cflags "jansson" 2>/dev/null`
+  pkg_cv_JANSSON_CFLAGS=`$PKG_CONFIG --cflags "jansson >= 2.11" 2>/dev/null`
 		      test "x$?" != "x0" && pkg_failed=yes
 else
   pkg_failed=yes
@@ -14217,12 +14054,12 @@ if test -n "$JANSSON_LIBS"; then
     pkg_cv_JANSSON_LIBS="$JANSSON_LIBS"
  elif test -n "$PKG_CONFIG"; then
     if test -n "$PKG_CONFIG" && \
-    { { $as_echo "$as_me:${as_lineno-$LINENO}: \$PKG_CONFIG --exists --print-errors \"jansson\""; } >&5
-  ($PKG_CONFIG --exists --print-errors "jansson") 2>&5
+    { { $as_echo "$as_me:${as_lineno-$LINENO}: \$PKG_CONFIG --exists --print-errors \"jansson >= 2.11\""; } >&5
+  ($PKG_CONFIG --exists --print-errors "jansson >= 2.11") 2>&5
   ac_status=$?
   $as_echo "$as_me:${as_lineno-$LINENO}: \$? = $ac_status" >&5
   test $ac_status = 0; }; then
-  pkg_cv_JANSSON_LIBS=`$PKG_CONFIG --libs "jansson" 2>/dev/null`
+  pkg_cv_JANSSON_LIBS=`$PKG_CONFIG --libs "jansson >= 2.11" 2>/dev/null`
 		      test "x$?" != "x0" && pkg_failed=yes
 else
   pkg_failed=yes
@@ -14243,9 +14080,9 @@ else
         _pkg_short_errors_supported=no
 fi
         if test $_pkg_short_errors_supported = yes; then
-	        JANSSON_PKG_ERRORS=`$PKG_CONFIG --short-errors --print-errors --cflags --libs "jansson" 2>&1`
+	        JANSSON_PKG_ERRORS=`$PKG_CONFIG --short-errors --print-errors --cflags --libs "jansson >= 2.11" 2>&1`
         else
-	        JANSSON_PKG_ERRORS=`$PKG_CONFIG --print-errors --cflags --libs "jansson" 2>&1`
+	        JANSSON_PKG_ERRORS=`$PKG_CONFIG --print-errors --cflags --libs "jansson >= 2.11" 2>&1`
         fi
 	# Put the nasty error message in config.log where it belongs
 	echo "$JANSSON_PKG_ERRORS" >&5
@@ -14277,6 +14114,7 @@ $as_echo "#define HAVE_JANSSON 1" >>conf
 fi
    fi
 
+	# json_sprintf is available in 2.11+
 
 if test "x${PBX_JANSSON}" != "x1" -a "${USE_JANSSON}" != "no"; then
    pbxlibdir=""
@@ -14291,9 +14129,9 @@ if test "x${PBX_JANSSON}" != "x1" -a "${
 
       ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
       CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for json_dumps in -ljansson" >&5
-$as_echo_n "checking for json_dumps in -ljansson... " >&6; }
-if ${ac_cv_lib_jansson_json_dumps+:} false; then :
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for json_sprintf in -ljansson" >&5
+$as_echo_n "checking for json_sprintf in -ljansson... " >&6; }
+if ${ac_cv_lib_jansson_json_sprintf+:} false; then :
   $as_echo_n "(cached) " >&6
 else
   ac_check_lib_save_LIBS=$LIBS
@@ -14307,27 +14145,27 @@ cat confdefs.h - <<_ACEOF >conftest.$ac_
 #ifdef __cplusplus
 extern "C"
 #endif
-char json_dumps ();
+char json_sprintf ();
 int
 main ()
 {
-return json_dumps ();
+return json_sprintf ();
   ;
   return 0;
 }
 _ACEOF
 if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_jansson_json_dumps=yes
+  ac_cv_lib_jansson_json_sprintf=yes
 else
-  ac_cv_lib_jansson_json_dumps=no
+  ac_cv_lib_jansson_json_sprintf=no
 fi
 rm -f core conftest.err conftest.$ac_objext \
     conftest$ac_exeext conftest.$ac_ext
 LIBS=$ac_check_lib_save_LIBS
 fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_jansson_json_dumps" >&5
-$as_echo "$ac_cv_lib_jansson_json_dumps" >&6; }
-if test "x$ac_cv_lib_jansson_json_dumps" = xyes; then :
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_jansson_json_sprintf" >&5
+$as_echo "$ac_cv_lib_jansson_json_sprintf" >&6; }
+if test "x$ac_cv_lib_jansson_json_sprintf" = xyes; then :
   AST_JANSSON_FOUND=yes
 else
   AST_JANSSON_FOUND=no
@@ -14368,21 +14206,51 @@ fi
 #define HAVE_JANSSON 1
 _ACEOF
 
-      fi
-   fi
-fi
+      fi
+   fi
+fi
+
+
+
+	if test "${PBX_JANSSON}" != 1; then
+		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** Asterisk requires libjansson >= 2.11 and no system copy was found." >&5
+$as_echo "$as_me: *** Asterisk requires libjansson >= 2.11 and no system copy was found." >&6;}
+		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** Please install the 'libjansson' development package or" >&5
+$as_echo "$as_me: *** Please install the 'libjansson' development package or" >&6;}
+		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** use './configure --with-jansson-bundled'" >&5
+$as_echo "$as_me: *** use './configure --with-jansson-bundled'" >&6;}
+		exit 1
+	fi
+
+	# Define the ast_json_int_t (large integer type) to match jansson's
+	saved_cppflags="${CPPFLAGS}"
+	CPPFLAGS="${CPPFLAGS} ${JANSSON_INCLUDE}"
+	cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+#include <jansson.h>
+int
+main ()
+{
+#if !JSON_INTEGER_IS_LONG_LONG
+		#error "not long long"
+		#endif
+
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_compile "$LINENO"; then :
+
+$as_echo "#define AST_JSON_INT_T long long" >>confdefs.h
 
+else
 
+$as_echo "#define AST_JSON_INT_T long" >>confdefs.h
+
+fi
+rm -f core conftest.err conftest.$ac_objext conftest.$ac_ext
+	CPPFLAGS="${saved_cppflags}"
 
-	if test "${PBX_JANSSON}" != 1; then
-		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** Asterisk requires libjansson and no system copy was found." >&5
-$as_echo "$as_me: *** Asterisk requires libjansson and no system copy was found." >&6;}
-		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** Please install the 'libjansson' development package or" >&5
-$as_echo "$as_me: *** Please install the 'libjansson' development package or" >&6;}
-		{ $as_echo "$as_me:${as_lineno-$LINENO}: *** use './configure --with-jansson-bundled'" >&5
-$as_echo "$as_me: *** use './configure --with-jansson-bundled'" >&6;}
-		exit 1
-	fi
 else
 	PBX_JANSSON=1
 fi
@@ -16979,7 +16847,7 @@ fi
 done
 
 
-for ac_func in asprintf atexit closefrom dup2 eaccess endpwent euidaccess ffsll ftruncate getcwd gethostbyname gethostname getloadavg gettimeofday glob ioperm inet_ntoa isascii memchr memmove memset mkdir mkdtemp munmap newlocale ppoll putenv re_comp regcomp select setenv socket strcasecmp strcasestr strchr strcspn strdup strerror strlcat strlcpy strncasecmp strndup strnlen strrchr strsep strspn strstr strtod strtol strtold strtoq unsetenv uselocale utime vasprintf getpeereid sysctl swapctl
+for ac_func in asprintf atexit closefrom dup2 eaccess endpwent euidaccess ffsll ftruncate getcwd gethostbyname gethostname getloadavg gettimeofday glob ioperm inet_ntoa isascii memchr memmove memset mkdir mkdtemp munmap newlocale ppoll putenv re_comp regcomp select setenv socket strcasecmp strcasestr strchr strcspn strdup strerror strlcat strlcpy strncasecmp strndup strnlen strrchr strsep strspn strstr strtod strtol strtold strtoq unsetenv uselocale utime vasprintf getpeereid sysctl swapctl malloc_trim
 do :
   as_ac_var=`$as_echo "ac_cv_func_$ac_func" | $as_tr_sh`
 ac_fn_c_check_func "$LINENO" "$ac_func" "$as_ac_var"
@@ -17248,21 +17116,6 @@ fi
 
 
 
-# https support (in main/http.c) uses funopen on BSD systems,
-# fopencookie on linux
-for ac_func in funopen fopencookie
-do :
-  as_ac_var=`$as_echo "ac_cv_func_$ac_func" | $as_tr_sh`
-ac_fn_c_check_func "$LINENO" "$ac_func" "$as_ac_var"
-if eval test \"x\$"$as_ac_var"\" = x"yes"; then :
-  cat >>confdefs.h <<_ACEOF
-#define `$as_echo "HAVE_$ac_func" | $as_tr_cpp` 1
-_ACEOF
-
-fi
-done
-
-
 for ac_func in inet_aton
 do :
   ac_fn_c_check_func "$LINENO" "inet_aton" "ac_cv_func_inet_aton"
@@ -17976,8 +17829,8 @@ fi
 done
 
 
-{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for compiler atomic operations" >&5
-$as_echo_n "checking for compiler atomic operations... " >&6; }
+{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for compiler sync operations" >&5
+$as_echo_n "checking for compiler sync operations... " >&6; }
 cat confdefs.h - <<_ACEOF >conftest.$ac_ext
 /* end confdefs.h.  */
 
@@ -17992,6 +17845,7 @@ _ACEOF
 if ac_fn_c_try_link "$LINENO"; then :
   { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
 $as_echo "yes" >&6; }
+ax_cv_have_gcc_atomics=1
 
 $as_echo "#define HAVE_GCC_ATOMICS 1" >>confdefs.h
 
@@ -18003,6 +17857,38 @@ fi
 rm -f core conftest.err conftest.$ac_objext \
     conftest$ac_exeext conftest.$ac_ext
 
+{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for compiler atomic operations" >&5
+$as_echo_n "checking for compiler atomic operations... " >&6; }
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+int
+main ()
+{
+int foo1; int foo2 = __atomic_fetch_add(&foo1, 1, __ATOMIC_RELAXED);
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
+$as_echo "yes" >&6; }
+ax_cv_have_c_atomics=1
+
+$as_echo "#define HAVE_C_ATOMICS 1" >>confdefs.h
+
+else
+  { $as_echo "$as_me:${as_lineno-$LINENO}: result: no" >&5
+$as_echo "no" >&6; }
+
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+
+if test -z $ax_cv_have_c_atomics$ax_cv_have_gcc_atomics; then
+	as_fn_error $? "*** Atomic operations are not supported by your compiler." "$LINENO" 5
+fi
+
 # glibc, AFAIK, is the only C library that makes printing a NULL to a string safe.
 { $as_echo "$as_me:${as_lineno-$LINENO}: checking if your system printf is NULL-safe." >&5
 $as_echo_n "checking if your system printf is NULL-safe.... " >&6; }
@@ -19429,17 +19315,6 @@ rm -f core conftest.err conftest.$ac_obj
 
 
 
-ac_fn_c_check_header_mongrel "$LINENO" "libkern/OSAtomic.h" "ac_cv_header_libkern_OSAtomic_h" "$ac_includes_default"
-if test "x$ac_cv_header_libkern_OSAtomic_h" = xyes; then :
-
-cat >>confdefs.h <<_ACEOF
-#define HAVE_OSX_ATOMICS 1
-_ACEOF
-
-fi
-
-
-
 # The cast to long int works around a bug in the HP C Compiler
 # version HP92453-01 B.11.11.23709.GP, which incorrectly rejects
 # declarations like `int a3[[(sizeof (unsigned char)) >= 0]];'.
@@ -20157,7 +20032,8 @@ fi
 
 
 
-if test "x${OSARCH}" = "xlinux-gnu" ; then
+case "${OSARCH}" in
+linux*)
 
 if test "x${PBX_CAP}" != "x1" -a "${USE_CAP}" != "no"; then
    pbxlibdir=""
@@ -20254,7 +20130,8 @@ _ACEOF
 fi
 
 
-fi
+  ;;
+esac
 
 
     if test "x${PBX_DAHDI}" != "x1"; then
@@ -23679,6 +23556,161 @@ fi
 
 
 
+# Check that unbound is installed and the version code fragment compiles
+
+if test "x${PBX_UNBOUND}" != "x1" -a "${USE_UNBOUND}" != "no"; then
+   pbxlibdir=""
+   # if --with-UNBOUND=DIR has been specified, use it.
+   if test "x${UNBOUND_DIR}" != "x"; then
+      if test -d ${UNBOUND_DIR}/lib; then
+         pbxlibdir="-L${UNBOUND_DIR}/lib"
+      else
+         pbxlibdir="-L${UNBOUND_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for ub_ctx_delete in -lunbound" >&5
+$as_echo_n "checking for ub_ctx_delete in -lunbound... " >&6; }
+if ${ac_cv_lib_unbound_ub_ctx_delete+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lunbound ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char ub_ctx_delete ();
+int
+main ()
+{
+return ub_ctx_delete ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_unbound_ub_ctx_delete=yes
+else
+  ac_cv_lib_unbound_ub_ctx_delete=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_unbound_ub_ctx_delete" >&5
+$as_echo "$ac_cv_lib_unbound_ub_ctx_delete" >&6; }
+if test "x$ac_cv_lib_unbound_ub_ctx_delete" = xyes; then :
+  AST_UNBOUND_FOUND=yes
+else
+  AST_UNBOUND_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_UNBOUND_FOUND}" = "yes"; then
+      UNBOUND_LIB="${pbxlibdir} -lunbound "
+      # if --with-UNBOUND=DIR has been specified, use it.
+      if test "x${UNBOUND_DIR}" != "x"; then
+         UNBOUND_INCLUDE="-I${UNBOUND_DIR}/include"
+      fi
+      UNBOUND_INCLUDE="${UNBOUND_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${UNBOUND_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "unbound.h" "ac_cv_header_unbound_h" "$ac_includes_default"
+if test "x$ac_cv_header_unbound_h" = xyes; then :
+  UNBOUND_HEADER_FOUND=1
+else
+  UNBOUND_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+   fi
+   # Validate the package with the supplied code.
+   if test "x${UNBOUND_HEADER_FOUND}" = "x1" ; then
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for unbound version >= 1.5" >&5
+$as_echo_n "checking for unbound version >= 1.5... " >&6; }
+      ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+      CPPFLAGS="${CPPFLAGS} ${UNBOUND_INCLUDE}"
+      ast_ext_lib_check_saved_LIBS="${LIBS}"
+      LIBS="${UNBOUND_LIB}"
+      cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+      #include <unbound.h>
+int
+main ()
+{
+
+            #if (UNBOUND_VERSION_MAJOR < 1 || (UNBOUND_VERSION_MAJOR == 1 && UNBOUND_VERSION_MINOR < 5 ))
+            #error "Unbound version must be >= 1.5"
+            #endif
+
+
+  ;
+  return 0;
+}
+
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+
+            if test "x${cross_compiling}" = "xyes" ; then
+               UNBOUND_VALIDATED="1"
+               { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes (guessed for cross-compile)" >&5
+$as_echo "yes (guessed for cross-compile)" >&6; }
+            else
+               ./conftest$EXEEXT
+               if test $? -eq 0 ; then
+                  UNBOUND_VALIDATED="1"
+                  { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
+$as_echo "yes" >&6; }
+               else
+                  { $as_echo "$as_me:${as_lineno-$LINENO}: result: no" >&5
+$as_echo "no" >&6; }
+               fi
+            fi
+
+else
+
+            { $as_echo "$as_me:${as_lineno-$LINENO}: result: no" >&5
+$as_echo "no" >&6; }
+
+
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+      CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+      LIBS="${ast_ext_lib_check_saved_LIBS}"
+   fi
+
+   if test "x${UNBOUND_VALIDATED}" = "x1" ; then
+
+      PBX_UNBOUND=1
+      cat >>confdefs.h <<_ACEOF
+#define HAVE_UNBOUND 1
+_ACEOF
+
+   else
+      UNBOUND_LIB=""
+      UNBOUND_INCLUDE=""
+   fi
+fi
+
+
+
 
 if test "x${PBX_UNIXODBC}" != "x1" -a "${USE_UNIXODBC}" != "no"; then
    pbxlibdir=""
@@ -24076,19 +24108,116 @@ if test "x${PBX_BLUETOOTH}" != "x1" -a "
       if test -d ${BLUETOOTH_DIR}/lib; then
          pbxlibdir="-L${BLUETOOTH_DIR}/lib"
       else
-         pbxlibdir="-L${BLUETOOTH_DIR}"
+         pbxlibdir="-L${BLUETOOTH_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for ba2str in -lbluetooth" >&5
+$as_echo_n "checking for ba2str in -lbluetooth... " >&6; }
+if ${ac_cv_lib_bluetooth_ba2str+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lbluetooth ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char ba2str ();
+int
+main ()
+{
+return ba2str ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_bluetooth_ba2str=yes
+else
+  ac_cv_lib_bluetooth_ba2str=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_bluetooth_ba2str" >&5
+$as_echo "$ac_cv_lib_bluetooth_ba2str" >&6; }
+if test "x$ac_cv_lib_bluetooth_ba2str" = xyes; then :
+  AST_BLUETOOTH_FOUND=yes
+else
+  AST_BLUETOOTH_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_BLUETOOTH_FOUND}" = "yes"; then
+      BLUETOOTH_LIB="${pbxlibdir} -lbluetooth "
+      # if --with-BLUETOOTH=DIR has been specified, use it.
+      if test "x${BLUETOOTH_DIR}" != "x"; then
+         BLUETOOTH_INCLUDE="-I${BLUETOOTH_DIR}/include"
+      fi
+      BLUETOOTH_INCLUDE="${BLUETOOTH_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${BLUETOOTH_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "bluetooth/bluetooth.h" "ac_cv_header_bluetooth_bluetooth_h" "$ac_includes_default"
+if test "x$ac_cv_header_bluetooth_bluetooth_h" = xyes; then :
+  BLUETOOTH_HEADER_FOUND=1
+else
+  BLUETOOTH_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${BLUETOOTH_HEADER_FOUND}" = "x0" ; then
+         BLUETOOTH_LIB=""
+         BLUETOOTH_INCLUDE=""
+      else
+
+         PBX_BLUETOOTH=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_BLUETOOTH 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+
+if test "x${PBX_BEANSTALK}" != "x1" -a "${USE_BEANSTALK}" != "no"; then
+   pbxlibdir=""
+   # if --with-BEANSTALK=DIR has been specified, use it.
+   if test "x${BEANSTALK_DIR}" != "x"; then
+      if test -d ${BEANSTALK_DIR}/lib; then
+         pbxlibdir="-L${BEANSTALK_DIR}/lib"
+      else
+         pbxlibdir="-L${BEANSTALK_DIR}"
       fi
    fi
 
       ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
       CFLAGS="${CFLAGS} "
-      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for ba2str in -lbluetooth" >&5
-$as_echo_n "checking for ba2str in -lbluetooth... " >&6; }
-if ${ac_cv_lib_bluetooth_ba2str+:} false; then :
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for bs_version in -lbeanstalk" >&5
+$as_echo_n "checking for bs_version in -lbeanstalk... " >&6; }
+if ${ac_cv_lib_beanstalk_bs_version+:} false; then :
   $as_echo_n "(cached) " >&6
 else
   ac_check_lib_save_LIBS=$LIBS
-LIBS="-lbluetooth ${pbxlibdir}  $LIBS"
+LIBS="-lbeanstalk ${pbxlibdir}  $LIBS"
 cat confdefs.h - <<_ACEOF >conftest.$ac_ext
 /* end confdefs.h.  */
 
@@ -24098,65 +24227,65 @@ cat confdefs.h - <<_ACEOF >conftest.$ac_
 #ifdef __cplusplus
 extern "C"
 #endif
-char ba2str ();
+char bs_version ();
 int
 main ()
 {
-return ba2str ();
+return bs_version ();
   ;
   return 0;
 }
 _ACEOF
 if ac_fn_c_try_link "$LINENO"; then :
-  ac_cv_lib_bluetooth_ba2str=yes
+  ac_cv_lib_beanstalk_bs_version=yes
 else
-  ac_cv_lib_bluetooth_ba2str=no
+  ac_cv_lib_beanstalk_bs_version=no
 fi
 rm -f core conftest.err conftest.$ac_objext \
     conftest$ac_exeext conftest.$ac_ext
 LIBS=$ac_check_lib_save_LIBS
 fi
-{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_bluetooth_ba2str" >&5
-$as_echo "$ac_cv_lib_bluetooth_ba2str" >&6; }
-if test "x$ac_cv_lib_bluetooth_ba2str" = xyes; then :
-  AST_BLUETOOTH_FOUND=yes
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_beanstalk_bs_version" >&5
+$as_echo "$ac_cv_lib_beanstalk_bs_version" >&6; }
+if test "x$ac_cv_lib_beanstalk_bs_version" = xyes; then :
+  AST_BEANSTALK_FOUND=yes
 else
-  AST_BLUETOOTH_FOUND=no
+  AST_BEANSTALK_FOUND=no
 fi
 
       CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
 
 
    # now check for the header.
-   if test "${AST_BLUETOOTH_FOUND}" = "yes"; then
-      BLUETOOTH_LIB="${pbxlibdir} -lbluetooth "
-      # if --with-BLUETOOTH=DIR has been specified, use it.
-      if test "x${BLUETOOTH_DIR}" != "x"; then
-         BLUETOOTH_INCLUDE="-I${BLUETOOTH_DIR}/include"
+   if test "${AST_BEANSTALK_FOUND}" = "yes"; then
+      BEANSTALK_LIB="${pbxlibdir} -lbeanstalk "
+      # if --with-BEANSTALK=DIR has been specified, use it.
+      if test "x${BEANSTALK_DIR}" != "x"; then
+         BEANSTALK_INCLUDE="-I${BEANSTALK_DIR}/include"
       fi
-      BLUETOOTH_INCLUDE="${BLUETOOTH_INCLUDE} "
+      BEANSTALK_INCLUDE="${BEANSTALK_INCLUDE} "
 
          # check for the header
          ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
-         CPPFLAGS="${CPPFLAGS} ${BLUETOOTH_INCLUDE}"
-         ac_fn_c_check_header_mongrel "$LINENO" "bluetooth/bluetooth.h" "ac_cv_header_bluetooth_bluetooth_h" "$ac_includes_default"
-if test "x$ac_cv_header_bluetooth_bluetooth_h" = xyes; then :
-  BLUETOOTH_HEADER_FOUND=1
+         CPPFLAGS="${CPPFLAGS} ${BEANSTALK_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "beanstalk.h" "ac_cv_header_beanstalk_h" "$ac_includes_default"
+if test "x$ac_cv_header_beanstalk_h" = xyes; then :
+  BEANSTALK_HEADER_FOUND=1
 else
-  BLUETOOTH_HEADER_FOUND=0
+  BEANSTALK_HEADER_FOUND=0
 fi
 
 
          CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
 
-      if test "x${BLUETOOTH_HEADER_FOUND}" = "x0" ; then
-         BLUETOOTH_LIB=""
-         BLUETOOTH_INCLUDE=""
+      if test "x${BEANSTALK_HEADER_FOUND}" = "x0" ; then
+         BEANSTALK_LIB=""
+         BEANSTALK_INCLUDE=""
       else
 
-         PBX_BLUETOOTH=1
+         PBX_BEANSTALK=1
          cat >>confdefs.h <<_ACEOF
-#define HAVE_BLUETOOTH 1
+#define HAVE_BEANSTALK 1
 _ACEOF
 
       fi
@@ -25408,6 +25537,46 @@ rm -f core conftest.err conftest.$ac_obj
 	CPPFLAGS="${saved_cppflags}"
     fi
 
+
+    if test "x${PBX_PJSIP_ENDPOINT_COMPACT_FORM}" != "x1" -a "${USE_PJSIP_ENDPOINT_COMPACT_FORM}" != "no"; then
+	{ $as_echo "$as_me:${as_lineno-$LINENO}: checking if \"pjsip_cfg()->endpt.use_compact_form = PJ_TRUE;\" compiles using pjsip.h" >&5
+$as_echo_n "checking if \"pjsip_cfg()->endpt.use_compact_form = PJ_TRUE;\" compiles using pjsip.h... " >&6; }
+	saved_cppflags="${CPPFLAGS}"
+	if test "x${PJSIP_ENDPOINT_COMPACT_FORM_DIR}" != "x"; then
+	    PJSIP_ENDPOINT_COMPACT_FORM_INCLUDE="-I${PJSIP_ENDPOINT_COMPACT_FORM_DIR}/include"
+	fi
+	CPPFLAGS="${CPPFLAGS} ${PJSIP_ENDPOINT_COMPACT_FORM_INCLUDE}"
+
+	cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+ #include <pjsip.h>
+int
+main ()
+{
+ pjsip_cfg()->endpt.use_compact_form = PJ_TRUE;;
+
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_compile "$LINENO"; then :
+     { $as_echo "$as_me:${as_lineno-$LINENO}: result: yes" >&5
+$as_echo "yes" >&6; }
+		PBX_PJSIP_ENDPOINT_COMPACT_FORM=1
+
+$as_echo "#define HAVE_PJSIP_ENDPOINT_COMPACT_FORM 1" >>confdefs.h
+
+
+
+else
+         { $as_echo "$as_me:${as_lineno-$LINENO}: result: no" >&5
+$as_echo "no" >&6; }
+
+fi
+rm -f core conftest.err conftest.$ac_objext conftest.$ac_ext
+	CPPFLAGS="${saved_cppflags}"
+    fi
+
       LIBS="${saved_libs}"
       CPPFLAGS="${saved_cppflags}"
 
@@ -28290,6 +28459,198 @@ fi
 
 
 
+if test "x${PBX_FFTW3}" != "x1" -a "${USE_FFTW3}" != "no"; then
+   pbxlibdir=""
+   # if --with-FFTW3=DIR has been specified, use it.
+   if test "x${FFTW3_DIR}" != "x"; then
+      if test -d ${FFTW3_DIR}/lib; then
+         pbxlibdir="-L${FFTW3_DIR}/lib"
+      else
+         pbxlibdir="-L${FFTW3_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for fftw_malloc in -lfftw3" >&5
+$as_echo_n "checking for fftw_malloc in -lfftw3... " >&6; }
+if ${ac_cv_lib_fftw3_fftw_malloc+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lfftw3 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char fftw_malloc ();
+int
+main ()
+{
+return fftw_malloc ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_fftw3_fftw_malloc=yes
+else
+  ac_cv_lib_fftw3_fftw_malloc=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_fftw3_fftw_malloc" >&5
+$as_echo "$ac_cv_lib_fftw3_fftw_malloc" >&6; }
+if test "x$ac_cv_lib_fftw3_fftw_malloc" = xyes; then :
+  AST_FFTW3_FOUND=yes
+else
+  AST_FFTW3_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_FFTW3_FOUND}" = "yes"; then
+      FFTW3_LIB="${pbxlibdir} -lfftw3 "
+      # if --with-FFTW3=DIR has been specified, use it.
+      if test "x${FFTW3_DIR}" != "x"; then
+         FFTW3_INCLUDE="-I${FFTW3_DIR}/include"
+      fi
+      FFTW3_INCLUDE="${FFTW3_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${FFTW3_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "fftw3.h" "ac_cv_header_fftw3_h" "$ac_includes_default"
+if test "x$ac_cv_header_fftw3_h" = xyes; then :
+  FFTW3_HEADER_FOUND=1
+else
+  FFTW3_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${FFTW3_HEADER_FOUND}" = "x0" ; then
+         FFTW3_LIB=""
+         FFTW3_INCLUDE=""
+      else
+
+         PBX_FFTW3=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_FFTW3 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SNDFILE}" != "x1" -a "${USE_SNDFILE}" != "no"; then
+   pbxlibdir=""
+   # if --with-SNDFILE=DIR has been specified, use it.
+   if test "x${SNDFILE_DIR}" != "x"; then
+      if test -d ${SNDFILE_DIR}/lib; then
+         pbxlibdir="-L${SNDFILE_DIR}/lib"
+      else
+         pbxlibdir="-L${SNDFILE_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for sf_open in -lsndfile" >&5
+$as_echo_n "checking for sf_open in -lsndfile... " >&6; }
+if ${ac_cv_lib_sndfile_sf_open+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsndfile ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char sf_open ();
+int
+main ()
+{
+return sf_open ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_sndfile_sf_open=yes
+else
+  ac_cv_lib_sndfile_sf_open=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_sndfile_sf_open" >&5
+$as_echo "$ac_cv_lib_sndfile_sf_open" >&6; }
+if test "x$ac_cv_lib_sndfile_sf_open" = xyes; then :
+  AST_SNDFILE_FOUND=yes
+else
+  AST_SNDFILE_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SNDFILE_FOUND}" = "yes"; then
+      SNDFILE_LIB="${pbxlibdir} -lsndfile "
+      # if --with-SNDFILE=DIR has been specified, use it.
+      if test "x${SNDFILE_DIR}" != "x"; then
+         SNDFILE_INCLUDE="-I${SNDFILE_DIR}/include"
+      fi
+      SNDFILE_INCLUDE="${SNDFILE_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${SNDFILE_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "sndfile.h" "ac_cv_header_sndfile_h" "$ac_includes_default"
+if test "x$ac_cv_header_sndfile_h" = xyes; then :
+  SNDFILE_HEADER_FOUND=1
+else
+  SNDFILE_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${SNDFILE_HEADER_FOUND}" = "x0" ; then
+         SNDFILE_LIB=""
+         SNDFILE_INCLUDE=""
+      else
+
+         PBX_SNDFILE=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SNDFILE 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
 
     if test "x${PBX_SPANDSP}" != "x1" -a "${USE_SPANDSP}" != "no"; then
 	{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for minimum version of SpanDSP" >&5
@@ -29419,20 +29780,124 @@ fi
 if test "x$ac_cv_header_radcli_radcli_h" = xyes; then :
   RADIUS_HEADER_FOUND=1
 else
-  RADIUS_HEADER_FOUND=0
+  RADIUS_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${RADIUS_HEADER_FOUND}" = "x0" ; then
+         RADIUS_LIB=""
+         RADIUS_INCLUDE=""
+      else
+
+         PBX_RADIUS=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_RADIUS 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+		if test "x${PBX_RADIUS}" = "x1"; then
+
+$as_echo "#define RADIUS_HEADER_STR <radcli/radcli.h>" >>confdefs.h
+
+		fi
+	fi
+fi
+
+
+if test "x${PBX_CODEC2}" != "x1" -a "${USE_CODEC2}" != "no"; then
+   pbxlibdir=""
+   # if --with-CODEC2=DIR has been specified, use it.
+   if test "x${CODEC2_DIR}" != "x"; then
+      if test -d ${CODEC2_DIR}/lib; then
+         pbxlibdir="-L${CODEC2_DIR}/lib"
+      else
+         pbxlibdir="-L${CODEC2_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for codec2_create in -lcodec2" >&5
+$as_echo_n "checking for codec2_create in -lcodec2... " >&6; }
+if ${ac_cv_lib_codec2_codec2_create+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lcodec2 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char codec2_create ();
+int
+main ()
+{
+return codec2_create ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_codec2_codec2_create=yes
+else
+  ac_cv_lib_codec2_codec2_create=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_codec2_codec2_create" >&5
+$as_echo "$ac_cv_lib_codec2_codec2_create" >&6; }
+if test "x$ac_cv_lib_codec2_codec2_create" = xyes; then :
+  AST_CODEC2_FOUND=yes
+else
+  AST_CODEC2_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_CODEC2_FOUND}" = "yes"; then
+      CODEC2_LIB="${pbxlibdir} -lcodec2 "
+      # if --with-CODEC2=DIR has been specified, use it.
+      if test "x${CODEC2_DIR}" != "x"; then
+         CODEC2_INCLUDE="-I${CODEC2_DIR}/include"
+      fi
+      CODEC2_INCLUDE="${CODEC2_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${CODEC2_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "codec2/codec2.h" "ac_cv_header_codec2_codec2_h" "$ac_includes_default"
+if test "x$ac_cv_header_codec2_codec2_h" = xyes; then :
+  CODEC2_HEADER_FOUND=1
+else
+  CODEC2_HEADER_FOUND=0
 fi
 
 
          CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
 
-      if test "x${RADIUS_HEADER_FOUND}" = "x0" ; then
-         RADIUS_LIB=""
-         RADIUS_INCLUDE=""
+      if test "x${CODEC2_HEADER_FOUND}" = "x0" ; then
+         CODEC2_LIB=""
+         CODEC2_INCLUDE=""
       else
 
-         PBX_RADIUS=1
+         PBX_CODEC2=1
          cat >>confdefs.h <<_ACEOF
-#define HAVE_RADIUS 1
+#define HAVE_CODEC2 1
 _ACEOF
 
       fi
@@ -29440,13 +29905,6 @@ _ACEOF
 fi
 
 
-		if test "x${PBX_RADIUS}" = "x1"; then
-
-$as_echo "#define RADIUS_HEADER_STR <radcli/radcli.h>" >>confdefs.h
-
-		fi
-	fi
-fi
 
 
 if test "x${PBX_COROSYNC}" != "x1" -a "${USE_COROSYNC}" != "no"; then
@@ -30812,6 +31270,264 @@ fi
 if test "x$PBX_SRTP" = x1;
 then
 
+if test "x${PBX_SRTP_256}" != "x1" -a "${USE_SRTP_256}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_256=DIR has been specified, use it.
+   if test "x${SRTP_256_DIR}" != "x"; then
+      if test -d ${SRTP_256_DIR}/lib; then
+         pbxlibdir="-L${SRTP_256_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_256_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80 in -lsrtp2" >&5
+$as_echo_n "checking for srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80 in -lsrtp2... " >&6; }
+if ${ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp2 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80 ();
+int
+main ()
+{
+return srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80 ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80=yes
+else
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" >&5
+$as_echo "$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" >&6; }
+if test "x$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" = xyes; then :
+  AST_SRTP_256_FOUND=yes
+else
+  AST_SRTP_256_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_256_FOUND}" = "yes"; then
+      SRTP_256_LIB="${pbxlibdir} -lsrtp2 "
+      # if --with-SRTP_256=DIR has been specified, use it.
+      if test "x${SRTP_256_DIR}" != "x"; then
+         SRTP_256_INCLUDE="-I${SRTP_256_DIR}/include"
+      fi
+      SRTP_256_INCLUDE="${SRTP_256_INCLUDE} "
+
+         # no header, assume found
+         SRTP_256_HEADER_FOUND="1"
+
+      if test "x${SRTP_256_HEADER_FOUND}" = "x0" ; then
+         SRTP_256_LIB=""
+         SRTP_256_INCLUDE=""
+      else
+
+         PBX_SRTP_256=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_256 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SRTP_192}" != "x1" -a "${USE_SRTP_192}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_192=DIR has been specified, use it.
+   if test "x${SRTP_192_DIR}" != "x"; then
+      if test -d ${SRTP_192_DIR}/lib; then
+         pbxlibdir="-L${SRTP_192_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_192_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80 in -lsrtp2" >&5
+$as_echo_n "checking for srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80 in -lsrtp2... " >&6; }
+if ${ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp2 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80 ();
+int
+main ()
+{
+return srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80 ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80=yes
+else
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" >&5
+$as_echo "$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" >&6; }
+if test "x$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" = xyes; then :
+  AST_SRTP_192_FOUND=yes
+else
+  AST_SRTP_192_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_192_FOUND}" = "yes"; then
+      SRTP_192_LIB="${pbxlibdir} -lsrtp2 "
+      # if --with-SRTP_192=DIR has been specified, use it.
+      if test "x${SRTP_192_DIR}" != "x"; then
+         SRTP_192_INCLUDE="-I${SRTP_192_DIR}/include"
+      fi
+      SRTP_192_INCLUDE="${SRTP_192_INCLUDE} "
+
+         # no header, assume found
+         SRTP_192_HEADER_FOUND="1"
+
+      if test "x${SRTP_192_HEADER_FOUND}" = "x0" ; then
+         SRTP_192_LIB=""
+         SRTP_192_INCLUDE=""
+      else
+
+         PBX_SRTP_192=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_192 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SRTP_GCM}" != "x1" -a "${USE_SRTP_GCM}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_GCM=DIR has been specified, use it.
+   if test "x${SRTP_GCM_DIR}" != "x"; then
+      if test -d ${SRTP_GCM_DIR}/lib; then
+         pbxlibdir="-L${SRTP_GCM_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_GCM_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for srtp_crypto_policy_set_aes_gcm_128_8_auth in -lsrtp2" >&5
+$as_echo_n "checking for srtp_crypto_policy_set_aes_gcm_128_8_auth in -lsrtp2... " >&6; }
+if ${ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp2 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char srtp_crypto_policy_set_aes_gcm_128_8_auth ();
+int
+main ()
+{
+return srtp_crypto_policy_set_aes_gcm_128_8_auth ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth=yes
+else
+  ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth" >&5
+$as_echo "$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth" >&6; }
+if test "x$ac_cv_lib_srtp2_srtp_crypto_policy_set_aes_gcm_128_8_auth" = xyes; then :
+  AST_SRTP_GCM_FOUND=yes
+else
+  AST_SRTP_GCM_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_GCM_FOUND}" = "yes"; then
+      SRTP_GCM_LIB="${pbxlibdir} -lsrtp2 "
+      # if --with-SRTP_GCM=DIR has been specified, use it.
+      if test "x${SRTP_GCM_DIR}" != "x"; then
+         SRTP_GCM_INCLUDE="-I${SRTP_GCM_DIR}/include"
+      fi
+      SRTP_GCM_INCLUDE="${SRTP_GCM_INCLUDE} "
+
+         # no header, assume found
+         SRTP_GCM_HEADER_FOUND="1"
+
+      if test "x${SRTP_GCM_HEADER_FOUND}" = "x0" ; then
+         SRTP_GCM_LIB=""
+         SRTP_GCM_INCLUDE=""
+      else
+
+         PBX_SRTP_GCM=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_GCM 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
 if test "x${PBX_SRTP_SHUTDOWN}" != "x1" -a "${USE_SRTP_SHUTDOWN}" != "no"; then
    pbxlibdir=""
    # if --with-SRTP_SHUTDOWN=DIR has been specified, use it.
@@ -30877,29 +31593,125 @@ fi
       if test "x${SRTP_SHUTDOWN_DIR}" != "x"; then
          SRTP_SHUTDOWN_INCLUDE="-I${SRTP_SHUTDOWN_DIR}/include"
       fi
-      SRTP_SHUTDOWN_INCLUDE="${SRTP_SHUTDOWN_INCLUDE} "
+      SRTP_SHUTDOWN_INCLUDE="${SRTP_SHUTDOWN_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${SRTP_SHUTDOWN_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "srtp2/srtp.h" "ac_cv_header_srtp2_srtp_h" "$ac_includes_default"
+if test "x$ac_cv_header_srtp2_srtp_h" = xyes; then :
+  SRTP_SHUTDOWN_HEADER_FOUND=1
+else
+  SRTP_SHUTDOWN_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${SRTP_SHUTDOWN_HEADER_FOUND}" = "x0" ; then
+         SRTP_SHUTDOWN_LIB=""
+         SRTP_SHUTDOWN_INCLUDE=""
+      else
+
+         PBX_SRTP_SHUTDOWN=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_SHUTDOWN 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SRTP_GET_VERSION}" != "x1" -a "${USE_SRTP_GET_VERSION}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_GET_VERSION=DIR has been specified, use it.
+   if test "x${SRTP_GET_VERSION_DIR}" != "x"; then
+      if test -d ${SRTP_GET_VERSION_DIR}/lib; then
+         pbxlibdir="-L${SRTP_GET_VERSION_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_GET_VERSION_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for srtp_get_version_string in -lsrtp2" >&5
+$as_echo_n "checking for srtp_get_version_string in -lsrtp2... " >&6; }
+if ${ac_cv_lib_srtp2_srtp_get_version_string+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp2 ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char srtp_get_version_string ();
+int
+main ()
+{
+return srtp_get_version_string ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp2_srtp_get_version_string=yes
+else
+  ac_cv_lib_srtp2_srtp_get_version_string=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp2_srtp_get_version_string" >&5
+$as_echo "$ac_cv_lib_srtp2_srtp_get_version_string" >&6; }
+if test "x$ac_cv_lib_srtp2_srtp_get_version_string" = xyes; then :
+  AST_SRTP_GET_VERSION_FOUND=yes
+else
+  AST_SRTP_GET_VERSION_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_GET_VERSION_FOUND}" = "yes"; then
+      SRTP_GET_VERSION_LIB="${pbxlibdir} -lsrtp2 "
+      # if --with-SRTP_GET_VERSION=DIR has been specified, use it.
+      if test "x${SRTP_GET_VERSION_DIR}" != "x"; then
+         SRTP_GET_VERSION_INCLUDE="-I${SRTP_GET_VERSION_DIR}/include"
+      fi
+      SRTP_GET_VERSION_INCLUDE="${SRTP_GET_VERSION_INCLUDE} "
 
          # check for the header
          ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
-         CPPFLAGS="${CPPFLAGS} ${SRTP_SHUTDOWN_INCLUDE}"
+         CPPFLAGS="${CPPFLAGS} ${SRTP_GET_VERSION_INCLUDE}"
          ac_fn_c_check_header_mongrel "$LINENO" "srtp2/srtp.h" "ac_cv_header_srtp2_srtp_h" "$ac_includes_default"
 if test "x$ac_cv_header_srtp2_srtp_h" = xyes; then :
-  SRTP_SHUTDOWN_HEADER_FOUND=1
+  SRTP_GET_VERSION_HEADER_FOUND=1
 else
-  SRTP_SHUTDOWN_HEADER_FOUND=0
+  SRTP_GET_VERSION_HEADER_FOUND=0
 fi
 
 
          CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
 
-      if test "x${SRTP_SHUTDOWN_HEADER_FOUND}" = "x0" ; then
-         SRTP_SHUTDOWN_LIB=""
-         SRTP_SHUTDOWN_INCLUDE=""
+      if test "x${SRTP_GET_VERSION_HEADER_FOUND}" = "x0" ; then
+         SRTP_GET_VERSION_LIB=""
+         SRTP_GET_VERSION_INCLUDE=""
       else
 
-         PBX_SRTP_SHUTDOWN=1
+         PBX_SRTP_GET_VERSION=1
          cat >>confdefs.h <<_ACEOF
-#define HAVE_SRTP_SHUTDOWN 1
+#define HAVE_SRTP_GET_VERSION 1
 _ACEOF
 
       fi
@@ -30917,7 +31729,7 @@ $as_echo "$as_me: WARNING: ***" >&2;}
 $as_echo "$as_me: WARNING: *** OpenSSL required when using libsrtp2, checking for libsrtp instead." >&2;}
         { $as_echo "$as_me:${as_lineno-$LINENO}: WARNING: ***" >&5
 $as_echo "$as_me: WARNING: ***" >&2;}
-        PBX_SRTP=0
+	PBX_SRTP=0
     fi
 fi
 
@@ -31094,6 +31906,264 @@ fi
     if test "x$PBX_SRTP" = x1;
     then
 
+if test "x${PBX_SRTP_256}" != "x1" -a "${USE_SRTP_256}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_256=DIR has been specified, use it.
+   if test "x${SRTP_256_DIR}" != "x"; then
+      if test -d ${SRTP_256_DIR}/lib; then
+         pbxlibdir="-L${SRTP_256_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_256_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for crypto_policy_set_aes_cm_256_hmac_sha1_80 in -lsrtp" >&5
+$as_echo_n "checking for crypto_policy_set_aes_cm_256_hmac_sha1_80 in -lsrtp... " >&6; }
+if ${ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char crypto_policy_set_aes_cm_256_hmac_sha1_80 ();
+int
+main ()
+{
+return crypto_policy_set_aes_cm_256_hmac_sha1_80 ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80=yes
+else
+  ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" >&5
+$as_echo "$ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" >&6; }
+if test "x$ac_cv_lib_srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80" = xyes; then :
+  AST_SRTP_256_FOUND=yes
+else
+  AST_SRTP_256_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_256_FOUND}" = "yes"; then
+      SRTP_256_LIB="${pbxlibdir} -lsrtp "
+      # if --with-SRTP_256=DIR has been specified, use it.
+      if test "x${SRTP_256_DIR}" != "x"; then
+         SRTP_256_INCLUDE="-I${SRTP_256_DIR}/include"
+      fi
+      SRTP_256_INCLUDE="${SRTP_256_INCLUDE} "
+
+         # no header, assume found
+         SRTP_256_HEADER_FOUND="1"
+
+      if test "x${SRTP_256_HEADER_FOUND}" = "x0" ; then
+         SRTP_256_LIB=""
+         SRTP_256_INCLUDE=""
+      else
+
+         PBX_SRTP_256=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_256 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SRTP_192}" != "x1" -a "${USE_SRTP_192}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_192=DIR has been specified, use it.
+   if test "x${SRTP_192_DIR}" != "x"; then
+      if test -d ${SRTP_192_DIR}/lib; then
+         pbxlibdir="-L${SRTP_192_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_192_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for crypto_policy_set_aes_cm_192_hmac_sha1_80 in -lsrtp" >&5
+$as_echo_n "checking for crypto_policy_set_aes_cm_192_hmac_sha1_80 in -lsrtp... " >&6; }
+if ${ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char crypto_policy_set_aes_cm_192_hmac_sha1_80 ();
+int
+main ()
+{
+return crypto_policy_set_aes_cm_192_hmac_sha1_80 ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80=yes
+else
+  ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" >&5
+$as_echo "$ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" >&6; }
+if test "x$ac_cv_lib_srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80" = xyes; then :
+  AST_SRTP_192_FOUND=yes
+else
+  AST_SRTP_192_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_192_FOUND}" = "yes"; then
+      SRTP_192_LIB="${pbxlibdir} -lsrtp "
+      # if --with-SRTP_192=DIR has been specified, use it.
+      if test "x${SRTP_192_DIR}" != "x"; then
+         SRTP_192_INCLUDE="-I${SRTP_192_DIR}/include"
+      fi
+      SRTP_192_INCLUDE="${SRTP_192_INCLUDE} "
+
+         # no header, assume found
+         SRTP_192_HEADER_FOUND="1"
+
+      if test "x${SRTP_192_HEADER_FOUND}" = "x0" ; then
+         SRTP_192_LIB=""
+         SRTP_192_INCLUDE=""
+      else
+
+         PBX_SRTP_192=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_192 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
+if test "x${PBX_SRTP_GCM}" != "x1" -a "${USE_SRTP_GCM}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_GCM=DIR has been specified, use it.
+   if test "x${SRTP_GCM_DIR}" != "x"; then
+      if test -d ${SRTP_GCM_DIR}/lib; then
+         pbxlibdir="-L${SRTP_GCM_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_GCM_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for crypto_policy_set_aes_gcm_128_8_auth in -lsrtp" >&5
+$as_echo_n "checking for crypto_policy_set_aes_gcm_128_8_auth in -lsrtp... " >&6; }
+if ${ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char crypto_policy_set_aes_gcm_128_8_auth ();
+int
+main ()
+{
+return crypto_policy_set_aes_gcm_128_8_auth ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth=yes
+else
+  ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth" >&5
+$as_echo "$ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth" >&6; }
+if test "x$ac_cv_lib_srtp_crypto_policy_set_aes_gcm_128_8_auth" = xyes; then :
+  AST_SRTP_GCM_FOUND=yes
+else
+  AST_SRTP_GCM_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_GCM_FOUND}" = "yes"; then
+      SRTP_GCM_LIB="${pbxlibdir} -lsrtp "
+      # if --with-SRTP_GCM=DIR has been specified, use it.
+      if test "x${SRTP_GCM_DIR}" != "x"; then
+         SRTP_GCM_INCLUDE="-I${SRTP_GCM_DIR}/include"
+      fi
+      SRTP_GCM_INCLUDE="${SRTP_GCM_INCLUDE} "
+
+         # no header, assume found
+         SRTP_GCM_HEADER_FOUND="1"
+
+      if test "x${SRTP_GCM_HEADER_FOUND}" = "x0" ; then
+         SRTP_GCM_LIB=""
+         SRTP_GCM_INCLUDE=""
+      else
+
+         PBX_SRTP_GCM=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_GCM 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
+
 if test "x${PBX_SRTP_SHUTDOWN}" != "x1" -a "${USE_SRTP_SHUTDOWN}" != "no"; then
    pbxlibdir=""
    # if --with-SRTP_SHUTDOWN=DIR has been specified, use it.
@@ -31189,6 +32259,102 @@ _ACEOF
 fi
 
 
+
+if test "x${PBX_SRTP_GET_VERSION}" != "x1" -a "${USE_SRTP_GET_VERSION}" != "no"; then
+   pbxlibdir=""
+   # if --with-SRTP_GET_VERSION=DIR has been specified, use it.
+   if test "x${SRTP_GET_VERSION_DIR}" != "x"; then
+      if test -d ${SRTP_GET_VERSION_DIR}/lib; then
+         pbxlibdir="-L${SRTP_GET_VERSION_DIR}/lib"
+      else
+         pbxlibdir="-L${SRTP_GET_VERSION_DIR}"
+      fi
+   fi
+
+      ast_ext_lib_check_save_CFLAGS="${CFLAGS}"
+      CFLAGS="${CFLAGS} "
+      { $as_echo "$as_me:${as_lineno-$LINENO}: checking for srtp_get_version_string in -lsrtp" >&5
+$as_echo_n "checking for srtp_get_version_string in -lsrtp... " >&6; }
+if ${ac_cv_lib_srtp_srtp_get_version_string+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  ac_check_lib_save_LIBS=$LIBS
+LIBS="-lsrtp ${pbxlibdir}  $LIBS"
+cat confdefs.h - <<_ACEOF >conftest.$ac_ext
+/* end confdefs.h.  */
+
+/* Override any GCC internal prototype to avoid an error.
+   Use char because int might match the return type of a GCC
+   builtin and then its argument prototype would still apply.  */
+#ifdef __cplusplus
+extern "C"
+#endif
+char srtp_get_version_string ();
+int
+main ()
+{
+return srtp_get_version_string ();
+  ;
+  return 0;
+}
+_ACEOF
+if ac_fn_c_try_link "$LINENO"; then :
+  ac_cv_lib_srtp_srtp_get_version_string=yes
+else
+  ac_cv_lib_srtp_srtp_get_version_string=no
+fi
+rm -f core conftest.err conftest.$ac_objext \
+    conftest$ac_exeext conftest.$ac_ext
+LIBS=$ac_check_lib_save_LIBS
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_lib_srtp_srtp_get_version_string" >&5
+$as_echo "$ac_cv_lib_srtp_srtp_get_version_string" >&6; }
+if test "x$ac_cv_lib_srtp_srtp_get_version_string" = xyes; then :
+  AST_SRTP_GET_VERSION_FOUND=yes
+else
+  AST_SRTP_GET_VERSION_FOUND=no
+fi
+
+      CFLAGS="${ast_ext_lib_check_save_CFLAGS}"
+
+
+   # now check for the header.
+   if test "${AST_SRTP_GET_VERSION_FOUND}" = "yes"; then
+      SRTP_GET_VERSION_LIB="${pbxlibdir} -lsrtp "
+      # if --with-SRTP_GET_VERSION=DIR has been specified, use it.
+      if test "x${SRTP_GET_VERSION_DIR}" != "x"; then
+         SRTP_GET_VERSION_INCLUDE="-I${SRTP_GET_VERSION_DIR}/include"
+      fi
+      SRTP_GET_VERSION_INCLUDE="${SRTP_GET_VERSION_INCLUDE} "
+
+         # check for the header
+         ast_ext_lib_check_saved_CPPFLAGS="${CPPFLAGS}"
+         CPPFLAGS="${CPPFLAGS} ${SRTP_GET_VERSION_INCLUDE}"
+         ac_fn_c_check_header_mongrel "$LINENO" "srtp/srtp.h" "ac_cv_header_srtp_srtp_h" "$ac_includes_default"
+if test "x$ac_cv_header_srtp_srtp_h" = xyes; then :
+  SRTP_GET_VERSION_HEADER_FOUND=1
+else
+  SRTP_GET_VERSION_HEADER_FOUND=0
+fi
+
+
+         CPPFLAGS="${ast_ext_lib_check_saved_CPPFLAGS}"
+
+      if test "x${SRTP_GET_VERSION_HEADER_FOUND}" = "x0" ; then
+         SRTP_GET_VERSION_LIB=""
+         SRTP_GET_VERSION_INCLUDE=""
+      else
+
+         PBX_SRTP_GET_VERSION=1
+         cat >>confdefs.h <<_ACEOF
+#define HAVE_SRTP_GET_VERSION 1
+_ACEOF
+
+      fi
+   fi
+fi
+
+
     fi
 fi
 
@@ -33351,6 +34517,31 @@ else
 	touch makeopts.acbak
 fi
 
+# conf_bridge (binaural rendering): check if HRIRs are available
+{ $as_echo "$as_me:${as_lineno-$LINENO}: checking for bridges/bridge_softmix/include/hrirs.h" >&5
+$as_echo_n "checking for bridges/bridge_softmix/include/hrirs.h... " >&6; }
+if ${ac_cv_file_bridges_bridge_softmix_include_hrirs_h+:} false; then :
+  $as_echo_n "(cached) " >&6
+else
+  test "$cross_compiling" = yes &&
+  as_fn_error $? "cannot check for file existence when cross compiling" "$LINENO" 5
+if test -r "bridges/bridge_softmix/include/hrirs.h"; then
+  ac_cv_file_bridges_bridge_softmix_include_hrirs_h=yes
+else
+  ac_cv_file_bridges_bridge_softmix_include_hrirs_h=no
+fi
+fi
+{ $as_echo "$as_me:${as_lineno-$LINENO}: result: $ac_cv_file_bridges_bridge_softmix_include_hrirs_h" >&5
+$as_echo "$ac_cv_file_bridges_bridge_softmix_include_hrirs_h" >&6; }
+if test "x$ac_cv_file_bridges_bridge_softmix_include_hrirs_h" = xyes; then :
+
+cat >>confdefs.h <<_ACEOF
+#define HAVE_BRIDGES_BRIDGE_SOFTMIX_INCLUDE_HRIRS_H 1
+_ACEOF
+
+fi
+
+
 ac_config_files="$ac_config_files build_tools/menuselect-deps makeopts"
 
 
diff -pruN 1:13.23.1~dfsg-2/configure.ac 1:16.1.1~dfsg-1/configure.ac
--- 1:13.23.1~dfsg-2/configure.ac	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/configure.ac	2018-12-26 18:45:13.000000000 +0000
@@ -175,6 +175,9 @@ case "${host_os}" in
      linux-gnu*)
      OSARCH=linux-gnu
      ;;
+     linux-musl*)
+     OSARCH=linux-musl
+     ;;
      kfreebsd*-gnu)
      OSARCH=kfreebsd-gnu
      ;;
@@ -212,17 +215,13 @@ if test ! x"${UNAME}" = xNo; then
 fi
 AC_SUBST(PBX_OSREV)
 
-AH_TOP(
-#ifndef ASTERISK_AUTOCONFIG_H
+AH_TOP([#ifndef ASTERISK_AUTOCONFIG_H
 #define ASTERISK_AUTOCONFIG_H
+/* IWYU pragma: private, include "asterisk.h" */
 
-#include "asterisk/buildopts.h"
-
-)
+#include "asterisk/buildopts.h" /* IWYU pragma: export */])
 
-AH_BOTTOM(
-#endif
-)
+AH_BOTTOM([#endif	/* ASTERISK_AUTOCONFIG_H */])
 
 # cross-compile checks
 if test "${cross_compiling}" = "yes";
@@ -386,6 +385,17 @@ AX_PTHREAD
 
 AC_LANG(C)
 
+PBX_NO_BINARY_MODULES=0
+AC_ARG_ENABLE([binary-modules],
+	[AS_HELP_STRING([--disable-binary-modules],
+		[Block installation of binary modules.])],
+	[case "${enableval}" in
+	      y|ye|yes) PBX_NO_BINARY_MODULES=0 ;;
+	      n|no)  PBX_NO_BINARY_MODULES=1 ;;
+	      *) AC_MSG_ERROR(bad value ${enableval} for --disable-binary-modules)  ;;
+	esac])
+AC_SUBST(PBX_NO_BINARY_MODULES)
+
 AC_ARG_ENABLE([dev-mode],
 	[AS_HELP_STRING([--enable-dev-mode],
 		[Turn on developer mode])],
@@ -433,12 +443,12 @@ AC_ARG_WITH([jansson-bundled],
 		*) JANSSON_BUNDLED=no ;;
 	esac])
 
-PJPROJECT_BUNDLED=no
+PJPROJECT_BUNDLED=yes
 AH_TEMPLATE(m4_bpatsubst([[HAVE_PJPROJECT_BUNDLED]], [(.*)]), [Define to 1 when using the bundled pjproject.])
 
 AC_ARG_WITH([pjproject-bundled],
 	[AS_HELP_STRING([--with-pjproject-bundled],
-		[Use bundled pjproject libraries])],
+		[Use bundled pjproject libraries (default)])],
 	[case "${withval}" in
 	      n|no) PJPROJECT_BUNDLED=no ;;
 	      *) PJPROJECT_BUNDLED=yes ;;
@@ -464,9 +474,9 @@ AST_EXT_LIB_SETUP([BFD], [Debug symbol d
 AST_EXT_LIB_SETUP([BKTR], [Stack Backtrace], [execinfo])
 AST_EXT_LIB_SETUP([BLUETOOTH], [Bluetooth], [bluetooth])
 AST_EXT_LIB_SETUP([CAP], [POSIX 1.e capabilities], [cap])
+AST_EXT_LIB_SETUP([CODEC2], [Codec 2 Audio Decoder/Encoder], [codec2])
 AST_EXT_LIB_SETUP([COROSYNC], [Corosync], [cpg])
 AST_EXT_LIB_SETUP_OPTIONAL([COROSYNC_CFG_STATE_TRACK], [A callback only in corosync 1.x], [COROSYNC], [cfg])
-AST_EXT_LIB_SETUP([CURSES], [curses], [curses])
 AST_EXT_LIB_SETUP([CRYPT], [password and data encryption], [crypt])
 AST_EXT_LIB_SETUP([CRYPTO], [OpenSSL Cryptography], [crypto])
 AST_EXT_LIB_SETUP_OPTIONAL([OPENSSL_SRTP], [OpenSSL SRTP Extension Support], [CRYPTO], [crypto])
@@ -490,7 +500,7 @@ AST_EXT_LIB_SETUP([URIPARSER], [uriparse
 AST_EXT_LIB_SETUP([KQUEUE], [kqueue support], [kqueue])
 AST_EXT_LIB_SETUP([LDAP], [OpenLDAP], [ldap])
 AST_LIBCURL_CHECK_CONFIG([], [7.10.1])
-AST_EXT_LIB_SETUP([LIBEDIT], [NetBSD Editline library], [libedit], [, use 'internal' Editline otherwise])
+AST_EXT_LIB_SETUP([LIBEDIT], [NetBSD Editline library], [libedit])
 AST_EXT_LIB_SETUP_OPTIONAL([LIBEDIT_IS_UNICODE], [Libedit compiled for unicode], [LIBEDIT], [libedit])
 AST_EXT_LIB_SETUP([LIBXML2], [LibXML2], [libxml2])
 AST_EXT_LIB_SETUP([LIBXSLT], [LibXSLT], [libxslt])
@@ -499,7 +509,6 @@ AST_EXT_LIB_SETUP([LUA], [Lua], [lua])
 AST_EXT_LIB_SETUP([MISDN], [mISDN user], [misdn])
 AST_EXT_LIB_SETUP([MYSQLCLIENT], [MySQL client], [mysqlclient])
 AST_EXT_LIB_SETUP([NBS], [Network Broadcast Sound], [nbs])
-AST_EXT_LIB_SETUP([NCURSES], [ncurses], [ncurses])
 AST_EXT_LIB_SETUP([NEON], [neon], [neon])
 AST_EXT_LIB_SETUP([NEON29], [neon29], [neon29])
 AST_EXT_LIB_SETUP([NETSNMP], [Net-SNMP], [netsnmp])
@@ -511,6 +520,7 @@ AST_EXT_LIB_SETUP([OPUSFILE], [Opusfile]
 AST_EXT_LIB_SETUP([OSPTK], [OSP Toolkit], [osptk])
 AST_EXT_LIB_SETUP([OSS], [Open Sound System], [oss])
 AST_EXT_LIB_SETUP([PGSQL], [PostgreSQL], [postgres])
+AST_EXT_LIB_SETUP([BEANSTALK], [Beanstalk Job Queue], [beanstalk])
 
 if test "x${PBX_PJPROJECT}" != "x1" ; then
 AST_EXT_LIB_SETUP([PJPROJECT], [PJPROJECT], [pjproject])
@@ -526,6 +536,7 @@ AST_EXT_LIB_SETUP_OPTIONAL([PJSIP_INV_SE
 AST_EXT_LIB_SETUP_OPTIONAL([PJSIP_AUTH_CLT_DEINIT], [pjsip_auth_clt_deinit support], [PJPROJECT], [pjsip])
 AST_EXT_LIB_SETUP_OPTIONAL([PJSIP_TSX_LAYER_FIND_TSX2], [pjsip_tsx_layer_find_tsx2 support], [PJPROJECT], [pjsip])
 AST_EXT_LIB_SETUP_OPTIONAL([PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS], [PJSIP INVITE Accept Multiple SDP Answers], [PJPROJECT], [pjsip])
+AST_EXT_LIB_SETUP_OPTIONAL([PJSIP_ENDPOINT_COMPACT_FORM], [PJSIP Compact Form Support on Endpoint], [PJPROJECT], [pjsip])
 fi
 
 AST_EXT_LIB_SETUP([POPT], [popt], [popt])
@@ -556,7 +567,9 @@ AST_EXT_LIB_SETUP_OPTIONAL([PRI_SERVICE_
 AST_EXT_LIB_SETUP_OPTIONAL([PRI_REVERSE_CHARGE], [ISDN reverse charge], [PRI], [pri])
 # ------------------------------------^
 AST_EXT_LIB_SETUP([RADIUS], [Radius Client], [radius])
+AST_EXT_LIB_SETUP([FFTW3], [LIBFFTW3], [fftw3])
 AST_EXT_LIB_SETUP([RESAMPLE], [LIBRESAMPLE], [resample])
+AST_EXT_LIB_SETUP([SNDFILE], [libsndfile], [sndfile])
 AST_EXT_LIB_SETUP([SDL], [Sdl], [sdl])
 AST_EXT_LIB_SETUP([SDL_IMAGE], [Sdl Image], [SDL_image])
 AST_EXT_LIB_SETUP([SPANDSP], [SPANDSP], [spandsp])
@@ -568,15 +581,18 @@ AST_EXT_LIB_SETUP_DEPENDENT([SPEEX_PREPR
 AST_EXT_LIB_SETUP([SQLITE], [SQLite], [sqlite])
 AST_EXT_LIB_SETUP([SQLITE3], [SQLite], [sqlite3])
 AST_EXT_LIB_SETUP([SRTP], [Secure RTP], [srtp])
+AST_EXT_LIB_SETUP_OPTIONAL([SRTP_256], [SRTP Library AES-256 (ICM)], [SRTP], [srtp])
+AST_EXT_LIB_SETUP_OPTIONAL([SRTP_192], [SRTP Library AES-192 (ICM)], [SRTP], [srtp])
+AST_EXT_LIB_SETUP_OPTIONAL([SRTP_GCM], [SRTP Library AES-128 (GCM) and AES-256 (GCM)], [SRTP], [srtp])
 AST_EXT_LIB_SETUP_OPTIONAL([SRTP_SHUTDOWN], [SRTP Library Shutdown Function], [SRTP], [srtp])
+AST_EXT_LIB_SETUP_OPTIONAL([SRTP_GET_VERSION], [SRTP Library Version Function], [SRTP], [srtp])
 AST_EXT_LIB_SETUP([OPENSSL], [OpenSSL Secure Sockets Layer], [ssl])
 AST_EXT_LIB_SETUP_OPTIONAL([RT], [Realtime functions], [rt])
 AST_EXT_LIB_SETUP([SUPPSERV], [mISDN Supplemental Services], [suppserv])
 AST_EXT_LIB_SETUP([FREETDS], [FreeTDS], [tds])
-AST_EXT_LIB_SETUP([TERMCAP], [Termcap], [termcap])
 AST_EXT_LIB_SETUP([TIMERFD], [timerfd], [timerfd])
-AST_EXT_LIB_SETUP([TINFO], [Term Info], [tinfo])
 AST_EXT_LIB_SETUP([TONEZONE], [tonezone], [tonezone])
+AST_EXT_LIB_SETUP([UNBOUND], [unbound], [unbound])
 AST_EXT_LIB_SETUP([UNIXODBC], [unixODBC], [unixodbc])
 AST_EXT_LIB_SETUP([VORBIS], [Vorbis], [vorbis])
 AST_EXT_LIB_SETUP([VPB], [Voicetronix API], [vpb])
@@ -588,65 +604,51 @@ AST_EXT_LIB_SETUP([ZLIB], [zlib compress
 
 AC_FUNC_ALLOCA
 AC_HEADER_DIRENT
+if test -z $ac_header_dirent -o "$ac_header_dirent" = "no"; then
+  AC_MSG_ERROR([*** Could not find dirent header that defines 'DIR'.])
+fi
+
 AC_HEADER_STDC
+if test "$ac_cv_header_stdc" != "yes"; then
+  AC_MSG_ERROR([*** ANSI C header files not found.])
+fi
+
 AC_HEADER_SYS_WAIT
-AC_CHECK_HEADERS([arpa/inet.h fcntl.h inttypes.h libintl.h limits.h locale.h malloc.h netdb.h netinet/in.h stddef.h stdint.h stdlib.h string.h strings.h sys/event.h sys/file.h sys/ioctl.h sys/param.h sys/socket.h sys/time.h syslog.h termios.h unistd.h utime.h arpa/nameser.h sys/io.h])
+if test "$ac_cv_header_sys_wait_h" != "yes"; then
+  AC_MSG_ERROR([*** POSIX.1 compatible sys/wait.h is required.])
+fi
+
+AC_HEADER_RESOLV
+if test "$ac_cv_header_resolv_h" != "yes"; then
+  AC_MSG_ERROR([*** resolv.h is required.])
+fi
+
+# Check for headers that are unconditionally required on all platforms.  Do not use
+# this to check for headers that are only needed by modules.
+AC_CHECK_HEADERS([ \
+  arpa/nameser.h assert.h ctype.h dlfcn.h errno.h fcntl.h float.h grp.h \
+  inttypes.h limits.h locale.h math.h pwd.h netinet/in.h regex.h \
+  sched.h stdarg.h stdint.h stdio.h stdlib.h string.h syslog.h \
+  sys/file.h sys/ioctl.h sys/param.h sys/resource.h sys/socket.h sys/stat.h \
+  sys/time.h sys/types.h sys/un.h \
+  termios.h time.h unistd.h \
+], [], [
+  AC_MSG_ERROR([*** A required header was not found.])
+])
+
+AC_CHECK_HEADERS([arpa/inet.h libintl.h malloc.h netdb.h stddef.h strings.h sys/event.h utime.h])
 
 PKG_PROG_PKG_CONFIG()
 
 # Find required NetBSD Editline library (libedit).
-EDITLINE_LIB=""
-LIBEDIT_INTERNAL="yes"
-AC_SUBST(LIBEDIT_INTERNAL)
-LIBEDIT_SYSTEM="yes"
-if test "${USE_LIBEDIT}" != "no"; then
-   if test "${LIBEDIT_DIR}" = "internal"; then
-      LIBEDIT_SYSTEM="no"
-   elif test "${LIBEDIT_DIR}" != ""; then
-      LIBEDIT_INTERNAL="no"
-   fi
-   if test "${LIBEDIT_SYSTEM}" = "yes"; then
-      AST_PKG_CONFIG_CHECK(LIBEDIT, libedit)
-      # some platforms do not list libedit via pkg-config, for example OpenBSD 6.2
-      AST_EXT_LIB_CHECK([LIBEDIT], [edit], [history_init], [histedit.h], [-ltermcap])
-      if test "$PBX_LIBEDIT" = "1"; then
-         LIBEDIT_INTERNAL="no"
-      fi
-   fi
-   if test "${LIBEDIT_INTERNAL}" = "yes"; then
-      PBX_LIBEDIT=1
-      LIBEDIT_IS_UNICODE=no
-   else
-      AST_C_COMPILE_CHECK([LIBEDIT_IS_UNICODE], [el_rfunc_t *callback;], [histedit.h], [], [Testing for libedit unicode support])
-   fi
-fi
-# If the Asterisk internal copy should be build (user said so) or
-# must be build (system does not have it), either -ltermcap,
-# -ltinfo, or -l{n}curses must be available.
-if test "${LIBEDIT_INTERNAL}" = "yes"; then
-   AST_EXT_LIB_CHECK([TERMCAP], [termcap], [tgetent], [])
-   if test "x$TERMCAP_LIB" != "x" ; then
-      EDITLINE_LIB="$TERMCAP_LIB"
-   else
-      AST_EXT_LIB_CHECK([TINFO], [tinfo], [tgetent], [])
-      if test "x$TINFO_LIB" != "x" ; then
-         EDITLINE_LIB="$TINFO_LIB"
-      else
-         AST_EXT_LIB_CHECK([CURSES], [curses], [initscr], [curses.h])
-         if test "x$CURSES_LIB" != "x" ; then
-            EDITLINE_LIB="$CURSES_LIB"
-         else
-            AST_EXT_LIB_CHECK([NCURSES], [ncurses], [initscr], [ncurses.h])
-            if test "x$NCURSES_LIB" != "x" ; then
-               EDITLINE_LIB="$NCURSES_LIB"
-            else
-               AC_MSG_ERROR([*** termcap support not found (on modern systems, this typically means the ncurses development package is missing)])
-            fi
-         fi
-      fi
-   fi
+AST_PKG_CONFIG_CHECK(LIBEDIT, libedit)
+# some platforms do not list libedit via pkg-config, for example OpenBSD 6.2
+AST_EXT_LIB_CHECK([LIBEDIT], [edit], [history_init], [histedit.h], [-ltermcap])
+if test "${PBX_LIBEDIT}" != 1; then
+	AC_MSG_ERROR(*** Please install the 'libedit' development package.)
+	exit 1
 fi
-AC_SUBST(EDITLINE_LIB)
+AST_C_COMPILE_CHECK([LIBEDIT_IS_UNICODE], [el_rfunc_t *callback;], [histedit.h], [], [Testing for libedit unicode support])
 
 # Find required UUID support.
 #  * -luuid on Linux
@@ -676,15 +678,17 @@ AC_SUBST(UUID_LIB)
 
 # Find required JSON support if bundled is not enabled.
 if test "$JANSSON_BUNDLED" = "no" ; then
-	AST_PKG_CONFIG_CHECK([JANSSON], [jansson])
-	AST_EXT_LIB_CHECK([JANSSON], [jansson], [json_dumps], [jansson.h])
+	AST_PKG_CONFIG_CHECK([JANSSON], [jansson >= 2.11])
+	# json_sprintf is available in 2.11+
+	AST_EXT_LIB_CHECK([JANSSON], [jansson], [json_sprintf], [jansson.h])
 
 	if test "${PBX_JANSSON}" != 1; then
-		AC_MSG_NOTICE(*** Asterisk requires libjansson and no system copy was found.)
+		AC_MSG_NOTICE(*** Asterisk requires libjansson >= 2.11 and no system copy was found.)
 		AC_MSG_NOTICE(*** Please install the 'libjansson' development package or)
 		AC_MSG_NOTICE(*** use './configure --with-jansson-bundled')
 		exit 1
 	fi
+	JANSSON_DEFINE_JSON_INT()
 else
 	PBX_JANSSON=1
 fi
@@ -780,7 +784,7 @@ AC_FUNC_STRNLEN
 AC_FUNC_STRTOD
 AC_FUNC_UTIME_NULL
 AC_FUNC_VPRINTF
-AC_CHECK_FUNCS([asprintf atexit closefrom dup2 eaccess endpwent euidaccess ffsll ftruncate getcwd gethostbyname gethostname getloadavg gettimeofday glob ioperm inet_ntoa isascii memchr memmove memset mkdir mkdtemp munmap newlocale ppoll putenv re_comp regcomp select setenv socket strcasecmp strcasestr strchr strcspn strdup strerror strlcat strlcpy strncasecmp strndup strnlen strrchr strsep strspn strstr strtod strtol strtold strtoq unsetenv uselocale utime vasprintf getpeereid sysctl swapctl])
+AC_CHECK_FUNCS([asprintf atexit closefrom dup2 eaccess endpwent euidaccess ffsll ftruncate getcwd gethostbyname gethostname getloadavg gettimeofday glob ioperm inet_ntoa isascii memchr memmove memset mkdir mkdtemp munmap newlocale ppoll putenv re_comp regcomp select setenv socket strcasecmp strcasestr strchr strcspn strdup strerror strlcat strlcpy strncasecmp strndup strnlen strrchr strsep strspn strstr strtod strtol strtold strtoq unsetenv uselocale utime vasprintf getpeereid sysctl swapctl malloc_trim])
 
 AC_MSG_CHECKING(for htonll)
 AC_LINK_IFELSE(
@@ -874,10 +878,6 @@ AC_ARG_ENABLE([asteriskssl],
 	esac], [AST_ASTERISKSSL=yes])
 AC_SUBST(AST_ASTERISKSSL)
 
-# https support (in main/http.c) uses funopen on BSD systems,
-# fopencookie on linux
-AC_CHECK_FUNCS([funopen fopencookie])
-
 AC_CHECK_FUNCS([inet_aton])
 
 # check if we have IP_PKTINFO constant defined
@@ -1090,14 +1090,28 @@ AC_LINK_IFELSE(
 # for FreeBSD thr_self
 AC_CHECK_HEADERS([sys/thr.h])
 
-AC_MSG_CHECKING(for compiler atomic operations)
+AC_MSG_CHECKING(for compiler sync operations)
 AC_LINK_IFELSE(
 [AC_LANG_PROGRAM([], [int foo1; int foo2 = __sync_fetch_and_add(&foo1, 1);])],
 AC_MSG_RESULT(yes)
-AC_DEFINE([HAVE_GCC_ATOMICS], 1, [Define to 1 if your GCC C compiler provides atomic operations.]),
+ax_cv_have_gcc_atomics=1
+AC_DEFINE([HAVE_GCC_ATOMICS], 1, [Define to 1 if your GCC C compiler provides __sync atomic operations.]),
 AC_MSG_RESULT(no)
 )
 
+AC_MSG_CHECKING(for compiler atomic operations)
+AC_LINK_IFELSE(
+[AC_LANG_PROGRAM([], [int foo1; int foo2 = __atomic_fetch_add(&foo1, 1, __ATOMIC_RELAXED);])],
+AC_MSG_RESULT(yes)
+ax_cv_have_c_atomics=1
+AC_DEFINE([HAVE_C_ATOMICS], 1, [Define to 1 if your C compiler provides __atomic operations.]),
+AC_MSG_RESULT(no)
+)
+
+if test -z $ax_cv_have_c_atomics$ax_cv_have_gcc_atomics; then
+	AC_MSG_ERROR([*** Atomic operations are not supported by your compiler.])
+fi
+
 # glibc, AFAIK, is the only C library that makes printing a NULL to a string safe.
 AC_MSG_CHECKING([if your system printf is NULL-safe.])
 AC_RUN_IFELSE(
@@ -1438,9 +1452,6 @@ AST_C_DEFINE_CHECK([RTLD_NOLOAD], [RTLD_
 
 AST_C_DEFINE_CHECK([IP_MTU_DISCOVER], [IP_MTU_DISCOVER], [netinet/in.h])
 
-AC_CHECK_HEADER([libkern/OSAtomic.h],
-                [AC_DEFINE_UNQUOTED([HAVE_OSX_ATOMICS], 1, [Define to 1 if OSX atomic operations are supported.])])
-
 AC_CHECK_SIZEOF([int])
 AC_CHECK_SIZEOF([long])
 AC_CHECK_SIZEOF([long long])
@@ -1509,9 +1520,11 @@ AST_EXT_LIB_CHECK([BFD], [bfd], [bfd_ope
 # openSUSE requires -lz
 AST_EXT_LIB_CHECK([BFD], [bfd], [bfd_openr], [bfd.h], [-ldl -liberty -lz])
 
-if test "x${OSARCH}" = "xlinux-gnu" ; then
+case "${OSARCH}" in
+linux*)
   AST_EXT_LIB_CHECK([CAP], [cap], [cap_from_text], [sys/capability.h])
-fi
+  ;;
+esac
 
 AST_C_DEFINE_CHECK([DAHDI], [DAHDI_RESET_COUNTERS], [dahdi/user.h], [230])
 AST_C_DEFINE_CHECK([DAHDI], [DAHDI_DEFAULT_MTU_MRU], [dahdi/user.h], [220])
@@ -2248,6 +2261,18 @@ AST_EXT_TOOL_CHECK([NETSNMP], [net-snmp-
 
 AST_EXT_LIB_CHECK([NEWT], [newt], [newtBell], [newt.h])
 
+# Check that unbound is installed and the version code fragment compiles
+AST_EXT_LIB_EXTRA_CHECK([UNBOUND], [unbound], [ub_ctx_delete], [unbound.h],
+   [], [], [
+      AC_LANG_PROGRAM( [#include <unbound.h>],
+         [
+            #if (UNBOUND_VERSION_MAJOR < 1 || (UNBOUND_VERSION_MAJOR == 1 && UNBOUND_VERSION_MINOR < 5 ))
+            #error "Unbound version must be >= 1.5"
+            #endif
+         ]
+      )
+   ], [unbound version >= 1.5])
+
 AST_EXT_LIB_CHECK([UNIXODBC], [odbc], [SQLConnect], [sql.h])
 
 AST_EXT_LIB_CHECK([OGG], [ogg], [ogg_stream_init], [ogg/ogg.h])
@@ -2259,6 +2284,8 @@ AST_EXT_LIB_CHECK([BKTR], [c], [backtrac
 
 AST_EXT_LIB_CHECK([BLUETOOTH], [bluetooth], [ba2str], [bluetooth/bluetooth.h])
 
+AST_EXT_LIB_CHECK([BEANSTALK], [beanstalk], [bs_version], [beanstalk.h])
+
 # possible places for oss definitions
 AST_EXT_LIB_CHECK([OSS], [ossaudio], [], [linux/soundcard.h])
 AST_EXT_LIB_CHECK([OSS], [ossaudio], [], [sys/soundcard.h])
@@ -2348,6 +2375,7 @@ if test "$USE_PJPROJECT" != "no" ; then
       LIBS="${LIBS} ${PJPROJECT_LIB}"
       AST_C_COMPILE_CHECK([PJSIP_TLS_TRANSPORT_PROTO], [struct pjsip_tls_setting setting; int proto; proto = setting.proto;], [pjsip.h])
       AST_C_COMPILE_CHECK([PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS], [pjsip_cfg()->endpt.accept_multiple_sdp_answers = 0;], [pjsip.h])
+      AST_C_COMPILE_CHECK([PJSIP_ENDPOINT_COMPACT_FORM], [pjsip_cfg()->endpt.use_compact_form = PJ_TRUE;], [pjsip.h])
       LIBS="${saved_libs}"
       CPPFLAGS="${saved_cppflags}"
 
@@ -2395,6 +2423,8 @@ AST_EXT_LIB_CHECK([PRI_REVERSE_CHARGE],
 # ------------------------------------^
 
 AST_EXT_LIB_CHECK([RESAMPLE], [resample], [resample_open], [libresample.h], [-lm])
+AST_EXT_LIB_CHECK([FFTW3], [fftw3], [fftw_malloc], [fftw3.h])
+AST_EXT_LIB_CHECK([SNDFILE], [sndfile], [sf_open], [sndfile.h])
 
 AST_C_COMPILE_CHECK([SPANDSP], [
 		#if SPANDSP_RELEASE_DATE < 20080516
@@ -2461,6 +2491,8 @@ else
 	fi
 fi
 
+AST_EXT_LIB_CHECK([CODEC2], [codec2], [codec2_create], [codec2/codec2.h])
+
 AST_EXT_LIB_CHECK([COROSYNC], [cpg], [cpg_join], [corosync/cpg.h], [-lcpg -lcfg])
 AST_EXT_LIB_CHECK([COROSYNC_CFG_STATE_TRACK], [cfg], [corosync_cfg_state_track], [corosync/cfg.h], [-lcfg])
 
@@ -2543,7 +2575,11 @@ AST_EXT_LIB_CHECK_SHARED([SRTP], [srtp2]
 
 if test "x$PBX_SRTP" = x1;
 then
+    AST_EXT_LIB_CHECK([SRTP_256], [srtp2], [srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80])
+    AST_EXT_LIB_CHECK([SRTP_192], [srtp2], [srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80])
+    AST_EXT_LIB_CHECK([SRTP_GCM], [srtp2], [srtp_crypto_policy_set_aes_gcm_128_8_auth])
     AST_EXT_LIB_CHECK([SRTP_SHUTDOWN], [srtp2], [srtp_shutdown], [srtp2/srtp.h])
+    AST_EXT_LIB_CHECK([SRTP_GET_VERSION], [srtp2], [srtp_get_version_string], [srtp2/srtp.h])
 
     # libsrtp2 removed support for PRNG, so we require OpenSSL
     if test "x$PBX_OPENSSL" != x1;
@@ -2551,7 +2587,7 @@ then
         AC_MSG_WARN([***])
         AC_MSG_WARN([*** OpenSSL required when using libsrtp2, checking for libsrtp instead.])
         AC_MSG_WARN([***])
-        PBX_SRTP=0
+	PBX_SRTP=0
     fi
 fi
 
@@ -2574,7 +2610,11 @@ then
 
     if test "x$PBX_SRTP" = x1;
     then
+        AST_EXT_LIB_CHECK([SRTP_256], [srtp], [crypto_policy_set_aes_cm_256_hmac_sha1_80])
+        AST_EXT_LIB_CHECK([SRTP_192], [srtp], [crypto_policy_set_aes_cm_192_hmac_sha1_80])
+        AST_EXT_LIB_CHECK([SRTP_GCM], [srtp], [crypto_policy_set_aes_gcm_128_8_auth])
         AST_EXT_LIB_CHECK([SRTP_SHUTDOWN], [srtp], [srtp_shutdown], [srtp/srtp.h])
+        AST_EXT_LIB_CHECK([SRTP_GET_VERSION], [srtp], [srtp_get_version_string], [srtp/srtp.h])
     fi
 fi
 
@@ -2749,6 +2789,9 @@ else
 	touch makeopts.acbak
 fi
 
+# conf_bridge (binaural rendering): check if HRIRs are available
+AC_CHECK_FILES([bridges/bridge_softmix/include/hrirs.h])
+
 AC_CONFIG_FILES([build_tools/menuselect-deps makeopts])
 AST_CHECK_MANDATORY
 
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/cdr/versions/54cde9847798_expand_accountcode_to_80.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/cdr/versions/54cde9847798_expand_accountcode_to_80.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/cdr/versions/54cde9847798_expand_accountcode_to_80.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/cdr/versions/54cde9847798_expand_accountcode_to_80.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,26 @@
+"""expand accountcode to 80
+
+Revision ID: 54cde9847798
+Revises: 210693f3123d
+Create Date: 2015-11-05 09:54:06.815364
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '54cde9847798'
+down_revision = '210693f3123d'
+
+from alembic import op
+import sqlalchemy as sa
+
+
+def upgrade():
+    op.alter_column('cdr', 'accountcode', type_=sa.String(80))
+    op.alter_column('cdr', 'peeraccount', type_=sa.String(80))
+    pass
+
+
+def downgrade():
+    op.alter_column('cdr', 'accountcode', type_=sa.String(20))
+    op.alter_column('cdr', 'peeraccount', type_=sa.String(20))
+    pass
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/041c0d3d1857_add_dtls_auto_gen_cert.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/041c0d3d1857_add_dtls_auto_gen_cert.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/041c0d3d1857_add_dtls_auto_gen_cert.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/041c0d3d1857_add_dtls_auto_gen_cert.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,33 @@
+"""add_dtls_auto_generate_cert
+
+Revision ID: 041c0d3d1857
+Revises: de83fac997e2
+Create Date: 2017-10-30 14:28:10.548395
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '041c0d3d1857'
+down_revision = 'de83fac997e2'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('dtls_auto_generate_cert', yesno_values))
+
+
+def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_dtls_auto_generate_cert_yesno_values', 'ps_endpoints')
+    op.drop_column('ps_endpoints', 'dtls_auto_generate_cert')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/15b1430ad6f1_add_moh_passthrough_option_to_pjsip.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/15b1430ad6f1_add_moh_passthrough_option_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/15b1430ad6f1_add_moh_passthrough_option_to_pjsip.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/15b1430ad6f1_add_moh_passthrough_option_to_pjsip.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,33 @@
+"""add moh passthrough option to pjsip
+
+Revision ID: 15b1430ad6f1
+Revises: 371a3bf4143e
+Create Date: 2014-11-19 07:44:51.225703
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '15b1430ad6f1'
+down_revision = '371a3bf4143e'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('moh_passthrough', yesno_values))
+
+def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_moh_passthrough_yesno_values','ps_endpoints')
+    with op.batch_alter_table('ps_endpoints') as batch_op:
+        batch_op.drop_column('moh_passthrough')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/164abbd708c_add_auto_info_to_endpoint_dtmf_mode.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/164abbd708c_add_auto_info_to_endpoint_dtmf_mode.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/164abbd708c_add_auto_info_to_endpoint_dtmf_mode.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/164abbd708c_add_auto_info_to_endpoint_dtmf_mode.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
 """Add auto_info to endpoint dtmf_mode
 
 Revision ID: 164abbd708c
-Revises: 86bb1efa278d
+Revises: 39959b9c2566
 Create Date: 2017-06-19 13:55:15.354706
 
 """
 
 # revision identifiers, used by Alembic.
 revision = '164abbd708c'
-down_revision = 'd7983954dd96'
+down_revision = '39959b9c2566'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/189a235b3fd7_add_keep_alive_interval.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/189a235b3fd7_add_keep_alive_interval.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/189a235b3fd7_add_keep_alive_interval.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/189a235b3fd7_add_keep_alive_interval.py	2018-12-26 18:45:13.000000000 +0000
@@ -8,7 +8,7 @@ Create Date: 2015-12-16 11:23:16.994523
 
 # revision identifiers, used by Alembic.
 revision = '189a235b3fd7'
-down_revision = '28ce1e718f05'
+down_revision = '339a3bdf53fc'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/1ac563b350a8_add_pjsip_trust_send__connected_line.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/1ac563b350a8_add_pjsip_trust_send__connected_line.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/1ac563b350a8_add_pjsip_trust_send__connected_line.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/1ac563b350a8_add_pjsip_trust_send__connected_line.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,40 @@
+"""add pjsip trust/send _connected_line
+
+Revision ID: 1ac563b350a8
+Revises: 2bb1a85135ad
+Create Date: 2018-10-12 17:10:34.530282
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '1ac563b350a8'
+down_revision = '2bb1a85135ad'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+AST_BOOL_NAME = 'ast_bool_values'
+# We'll just ignore the n/y and f/t abbreviations as Asterisk does not write
+# those aliases.
+AST_BOOL_VALUES = [ '0', '1',
+                    'off', 'on',
+                    'false', 'true',
+                    'no', 'yes' ]
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # ast_bool_values has already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    ast_bool_values = ENUM(*AST_BOOL_VALUES, name=AST_BOOL_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('trust_connected_line', ast_bool_values))
+    op.add_column('ps_endpoints', sa.Column('send_connected_line', ast_bool_values))
+
+def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_trust_connected_line_ast_bool_values', 'ps_endpoints')
+        op.drop_constraint('ck_ps_endpoints_send_connected_line_ast_bool_values', 'ps_endpoints')
+    op.drop_column('ps_endpoints', 'trust_connected_line')
+    op.drop_column('ps_endpoints', 'send_connected_line')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/2bb1a85135ad_pjsip_add_use_callerid_contact.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/2bb1a85135ad_pjsip_add_use_callerid_contact.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/2bb1a85135ad_pjsip_add_use_callerid_contact.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/2bb1a85135ad_pjsip_add_use_callerid_contact.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,39 @@
+"""pjsip add use_callerid_contact
+
+Revision ID: 2bb1a85135ad
+Revises: 7f85dd44c775
+Create Date: 2018-10-18 15:13:40.462354
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '2bb1a85135ad'
+down_revision = '7f85dd44c775'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+AST_BOOL_NAME = 'ast_bool_values'
+# We'll just ignore the n/y and f/t abbreviations as Asterisk does not write
+# those aliases.
+AST_BOOL_VALUES = [ '0', '1',
+                    'off', 'on',
+                    'false', 'true',
+                    'no', 'yes' ]
+
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # ast_bool_values has already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    ast_bool_values = ENUM(*AST_BOOL_VALUES, name=AST_BOOL_NAME, create_type=False)
+
+    op.add_column('ps_globals', sa.Column('use_callerid_contact', ast_bool_values))
+
+
+def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_globals_use_callerid_contact_ast_bool_values', 'ps_globals')
+    op.drop_column('ps_globals', 'use_callerid_contact')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/339a3bdf53fc_expand_accountcode_to_80.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/339a3bdf53fc_expand_accountcode_to_80.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/339a3bdf53fc_expand_accountcode_to_80.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/339a3bdf53fc_expand_accountcode_to_80.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,34 @@
+"""expand accountcode to 80
+
+Revision ID: 339a3bdf53fc
+Revises: 28ce1e718f05
+Create Date: 2015-11-05 10:10:27.465794
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '339a3bdf53fc'
+down_revision = '28ce1e718f05'
+
+from alembic import op
+import sqlalchemy as sa
+
+
+def upgrade():
+    with op.batch_alter_table('ps_endpoints') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(80))
+    with op.batch_alter_table('sippeers') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(80))
+    with op.batch_alter_table('iaxfriends') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(80))
+    pass
+
+
+def downgrade():
+    with op.batch_alter_table('ps_endpoints') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(20))
+    with op.batch_alter_table('sippeers') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(40))
+    with op.batch_alter_table('iaxfriends') as batch_op:
+        batch_op.alter_column('accountcode', type_=sa.String(20))
+    pass
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/371a3bf4143e_add_user_eq_phone_option_to_pjsip.py	2018-12-26 18:45:13.000000000 +0000
@@ -8,7 +8,7 @@ Create Date: 2014-10-13 13:46:24.474675
 
 # revision identifiers, used by Alembic.
 revision = '371a3bf4143e'
-down_revision = 'eb88a14f2a'
+down_revision = '10aedae86a32'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/39959b9c2566_pjsip_stream_maximum.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/39959b9c2566_pjsip_stream_maximum.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/39959b9c2566_pjsip_stream_maximum.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/39959b9c2566_pjsip_stream_maximum.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,24 @@
+"""pjsip_stream_maximum
+
+Revision ID: 39959b9c2566
+Revises: d7983954dd96
+Create Date: 2017-06-15 13:18:12.372333
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '39959b9c2566'
+down_revision = 'd7983954dd96'
+
+from alembic import op
+import sqlalchemy as sa
+
+
+def upgrade():
+    op.add_column('ps_endpoints', sa.Column('max_audio_streams', sa.Integer))
+    op.add_column('ps_endpoints', sa.Column('max_video_streams', sa.Integer))
+
+
+def downgrade():
+    op.drop_column('ps_endpoints', 'max_audio_streams')
+    op.drop_column('ps_endpoints', 'max_video_streams')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/423f34ad36e2_fix_pjsip_qualify_ti.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/423f34ad36e2_fix_pjsip_qualify_ti.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/423f34ad36e2_fix_pjsip_qualify_ti.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/423f34ad36e2_fix_pjsip_qualify_ti.py	2018-12-26 18:45:13.000000000 +0000
@@ -13,6 +13,7 @@ down_revision = '136885b81223'
 from alembic import op
 import sqlalchemy as sa
 
+
 def upgrade():
     op.alter_column('ps_aors', 'qualify_timeout', type_=sa.Float)
     op.alter_column('ps_contacts', 'qualify_timeout', type_=sa.Float)
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/44ccced114ce_add_webrtc.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/44ccced114ce_add_webrtc.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/44ccced114ce_add_webrtc.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/44ccced114ce_add_webrtc.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,31 @@
+"""add webrtc option to ps_endpoints
+
+Revision ID: 44ccced114ce
+Revises: 164abbd708c
+Create Date: 2017-07-10 17:07:25.926150
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '44ccced114ce'
+down_revision = '164abbd708c'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('webrtc', yesno_values))
+
+
+def downgrade():
+    op.drop_column('ps_endpoints', 'webrtc')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/45e3f47c6c44_add_pjsip_endpoint_identifier_order.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
 """add pjsip endpoint_identifier_order
 
 Revision ID: 45e3f47c6c44
-Revises: 371a3bf4143e
+Revises: 945b1098bdd
 Create Date: 2015-03-02 09:32:20.632015
 
 """
 
 # revision identifiers, used by Alembic.
 revision = '45e3f47c6c44'
-down_revision = '371a3bf4143e'
+down_revision = '945b1098bdd'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/4c573e7135bd_fix_tos_field_types.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/4c573e7135bd_fix_tos_field_types.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/4c573e7135bd_fix_tos_field_types.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/4c573e7135bd_fix_tos_field_types.py	2018-12-26 18:45:13.000000000 +0000
@@ -11,7 +11,6 @@ revision = '4c573e7135bd'
 down_revision = '28887f25a46f'
 
 from alembic import op
-from alembic import context
 import sqlalchemy as sa
 from sqlalchemy.dialects.postgresql import ENUM
 
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/5139253c0423_make_q_member_uniqueid_autoinc.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/5139253c0423_make_q_member_uniqueid_autoinc.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/5139253c0423_make_q_member_uniqueid_autoinc.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/5139253c0423_make_q_member_uniqueid_autoinc.py	2018-12-26 18:45:13.000000000 +0000
@@ -33,7 +33,6 @@ import sqlalchemy as sa
 
 
 def upgrade():
-    context = op.get_context()
     # Was unable to find a way to use op.alter_column() to add the unique
     # index property.
     op.drop_column('queue_members', 'uniqueid')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/52798ad97bdf_add_pjsip_identify_by_header.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/52798ad97bdf_add_pjsip_identify_by_header.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/52798ad97bdf_add_pjsip_identify_by_header.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/52798ad97bdf_add_pjsip_identify_by_header.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
 """add pjsip identify by header
 
 Revision ID: 52798ad97bdf
-Revises: 20abce6d1e3c
+Revises: e2f04d309071
 Create Date: 2018-01-08 12:16:02.782277
 
 """
 
 # revision identifiers, used by Alembic.
 revision = '52798ad97bdf'
-down_revision = '20abce6d1e3c'
+down_revision = 'e2f04d309071'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/6d8c104e6184_res_pjsip_add_contact_via_addr_and_.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/6d8c104e6184_res_pjsip_add_contact_via_addr_and_.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/6d8c104e6184_res_pjsip_add_contact_via_addr_and_.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/6d8c104e6184_res_pjsip_add_contact_via_addr_and_.py	2018-12-26 18:45:13.000000000 +0000
@@ -8,7 +8,7 @@ Create Date: 2016-05-19 15:51:33.410852
 
 # revision identifiers, used by Alembic.
 revision = 'a845e4d8ade8'
-down_revision = 'bca7113d796f'
+down_revision = 'd7e3c73eb2bf'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/7f3e21abe318_add_preferred_codec_only_option_to_pjsip.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/7f3e21abe318_add_preferred_codec_only_option_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/7f3e21abe318_add_preferred_codec_only_option_to_pjsip.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/7f3e21abe318_add_preferred_codec_only_option_to_pjsip.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,30 @@
+"""add preferred_codec_only option to pjsip
+
+Revision ID: 7f3e21abe318
+Revises: 4e2493ef32e6
+Create Date: 2016-09-02 11:00:23.534748
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '7f3e21abe318'
+down_revision = '4e2493ef32e6'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('preferred_codec_only', yesno_values))
+
+def downgrade():
+    op.drop_column('ps_endpoints', 'preferred_codec_only')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/7f85dd44c775_fix_suppress_q850_reason_headers.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/7f85dd44c775_fix_suppress_q850_reason_headers.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/7f85dd44c775_fix_suppress_q850_reason_headers.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/7f85dd44c775_fix_suppress_q850_reason_headers.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,43 @@
+"""fix suppress_q850_reason_headers
+
+Revision ID: 7f85dd44c775
+Revises: fe6592859b85
+Create Date: 2018-09-18 16:16:29.304815
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '7f85dd44c775'
+down_revision = 'fe6592859b85'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+
+def upgrade():
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    if op.get_context().bind.dialect.name != 'mssql':
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_header', type_=yesno_values,
+                        new_column_name='suppress_q850_reason_headers')
+    else:
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_header', existing_type=yesno_values, type_=sa.String(3),
+                        new_column_name='suppress_q850_reason_headers')
+        yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=True)
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_headers', existing_type=sa.String(3), type_=yesno_values)
+
+
+def downgrade():
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+    if op.get_context().bind.dialect.name != 'mssql':
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_headers', type_=yesno_values,
+                        new_column_name='suppress_q850_reason_header')
+    else:
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_headers', existing_type=yesno_values, type_=sa.String(3),
+                        new_column_name='suppress_q850_reason_header')
+        yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=True)
+        op.alter_column('ps_endpoints', 'suppress_q850_reason_header', existing_type=sa.String(3), type_=yesno_values)
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/837aa67461fb_add_pjsip_endpoint_ip_access_control_.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/837aa67461fb_add_pjsip_endpoint_ip_access_control_.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/837aa67461fb_add_pjsip_endpoint_ip_access_control_.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/837aa67461fb_add_pjsip_endpoint_ip_access_control_.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,13 +1,13 @@
 """Add PJSIP Endpoint IP Access Control options
 
-Revision ID: bca7113d796f
+Revision ID: d7e3c73eb2bf
 Revises: 6be31516058d
-Create Date: 2016-05-13 12:37:03.786359
+Create Date: 2016-05-13 12:45:45.071871
 
 """
 
 # revision identifiers, used by Alembic.
-revision = 'bca7113d796f'
+revision = 'd7e3c73eb2bf'
 down_revision = '6be31516058d'
 
 from alembic import op
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/945b1098bdd_add_media_encryption_optimistic_to_pjsip.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/945b1098bdd_add_media_encryption_optimistic_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/945b1098bdd_add_media_encryption_optimistic_to_pjsip.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/945b1098bdd_add_media_encryption_optimistic_to_pjsip.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,33 @@
+"""add media encryption optimistic to pjsip
+
+Revision ID: 945b1098bdd
+Revises: 15b1430ad6f1
+Create Date: 2014-11-19 07:47:52.490388
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = '945b1098bdd'
+down_revision = '15b1430ad6f1'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+    ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('media_encryption_optimistic', yesno_values))
+
+
+def downgrade():
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_media_encryption_optimistic_yesno_values','ps_endpoints')
+    op.drop_column('ps_endpoints', 'media_encryption_optimistic')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/a6ef36f1309_ps_globals_add_ignore_uri_user_options.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/a6ef36f1309_ps_globals_add_ignore_uri_user_options.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/a6ef36f1309_ps_globals_add_ignore_uri_user_options.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/a6ef36f1309_ps_globals_add_ignore_uri_user_options.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
 """ps_globals add ignore_uri_user_options
 
 Revision ID: a6ef36f1309
-Revises: 4e2493ef32e6
+Revises: 7f3e21abe318
 Create Date: 2016-08-31 12:24:22.368956
 
 """
 
 # revision identifiers, used by Alembic.
 revision = 'a6ef36f1309'
-down_revision = '4e2493ef32e6'
+down_revision = '7f3e21abe318'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/b83645976fdd_add_dtls_fingerprint_to_ps_endpoints.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/b83645976fdd_add_dtls_fingerprint_to_ps_endpoints.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/b83645976fdd_add_dtls_fingerprint_to_ps_endpoints.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/b83645976fdd_add_dtls_fingerprint_to_ps_endpoints.py	2018-12-26 18:45:13.000000000 +0000
@@ -31,7 +31,7 @@ def upgrade():
 def downgrade():
     context = op.get_context()
 
-    if op.get_context().bind.dialect.name == 'mssql':
+    if context.bind.dialect.name == 'mssql':
         op.drop_constraint('ck_ps_endpoints_dtls_fingerprint_sha_hash_values', 'ps_endpoints')
     op.drop_column('ps_endpoints', 'dtls_fingerprint')
 
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/de83fac997e2_add_bundle_to_ps_endpoints.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/de83fac997e2_add_bundle_to_ps_endpoints.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/de83fac997e2_add_bundle_to_ps_endpoints.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/de83fac997e2_add_bundle_to_ps_endpoints.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,34 @@
+"""add bundle to ps_endpoints
+
+Revision ID: de83fac997e2
+Revises: 20abce6d1e3c
+Create Date: 2017-10-24 17:10:57.242020
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = 'de83fac997e2'
+down_revision = '20abce6d1e3c'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+YESNO_NAME = 'yesno_values'
+YESNO_VALUES = ['yes', 'no']
+
+def upgrade():
+     ############################# Enums ##############################
+
+    # yesno_values have already been created, so use postgres enum object
+    # type to get around "already created" issue - works okay with mysql
+    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
+
+    op.add_column('ps_endpoints', sa.Column('bundle', yesno_values))
+
+def downgrade():
+    context = op.get_context()
+
+    if context.bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_bundle_yesno_values', 'ps_endpoints')
+    op.drop_column('ps_endpoints', 'bundle')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/e2f04d309071_add_wrapuptime_to_queue_members.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/e2f04d309071_add_wrapuptime_to_queue_members.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/e2f04d309071_add_wrapuptime_to_queue_members.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/e2f04d309071_add_wrapuptime_to_queue_members.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,22 @@
+"""add wrapuptime to queue_members
+
+Revision ID: e2f04d309071
+Revises: 041c0d3d1857
+Create Date: 2017-12-07 08:32:45.360857
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = 'e2f04d309071'
+down_revision = '041c0d3d1857'
+
+from alembic import op
+import sqlalchemy as sa
+
+
+def upgrade():
+    op.add_column('queue_members', sa.Column('wrapuptime', sa.Integer))
+
+
+def downgrade():
+    op.drop_column('queue_members', 'wrapuptime')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/eb88a14f2a_add_media_encryption_optimistic_to_pjsip.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/eb88a14f2a_add_media_encryption_optimistic_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/eb88a14f2a_add_media_encryption_optimistic_to_pjsip.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/eb88a14f2a_add_media_encryption_optimistic_to_pjsip.py	1970-01-01 00:00:00.000000000 +0000
@@ -1,33 +0,0 @@
-"""add media encryption optimistic to pjsip
-
-Revision ID: eb88a14f2a
-Revises: 10aedae86a32
-Create Date: 2014-11-19 07:08:55.423018
-
-"""
-
-# revision identifiers, used by Alembic.
-revision = 'eb88a14f2a'
-down_revision = '10aedae86a32'
-
-from alembic import op
-import sqlalchemy as sa
-from sqlalchemy.dialects.postgresql import ENUM
-
-YESNO_NAME = 'yesno_values'
-YESNO_VALUES = ['yes', 'no']
-
-def upgrade():
-    ############################# Enums ##############################
-
-    # yesno_values have already been created, so use postgres enum object
-    # type to get around "already created" issue - works okay with mysql
-    yesno_values = ENUM(*YESNO_VALUES, name=YESNO_NAME, create_type=False)
-
-    op.add_column('ps_endpoints', sa.Column('media_encryption_optimistic', yesno_values))
-
-
-def downgrade():
-    if op.get_context().bind.dialect.name == 'mssql':
-        op.drop_constraint('ck_ps_endpoints_media_encryption_optimistic_yesno_values','ps_endpoints')
-    op.drop_column('ps_endpoints', 'media_encryption_optimistic')
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/f3d1c5d38b56_add_prune_on_boot.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/f3d1c5d38b56_add_prune_on_boot.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/f3d1c5d38b56_add_prune_on_boot.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/f3d1c5d38b56_add_prune_on_boot.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
 """add_prune_on_boot
 
 Revision ID: f3d1c5d38b56
-Revises: 164abbd708c
+Revises: 44ccced114ce
 Create Date: 2017-08-04 17:31:23.124767
 
 """
 
 # revision identifiers, used by Alembic.
 revision = 'f3d1c5d38b56'
-down_revision = '164abbd708c'
+down_revision = '44ccced114ce'
 
 from alembic import op
 import sqlalchemy as sa
diff -pruN 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/fe6592859b85_fix_mwi_subscribe_replaces_.py 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/fe6592859b85_fix_mwi_subscribe_replaces_.py
--- 1:13.23.1~dfsg-2/contrib/ast-db-manage/config/versions/fe6592859b85_fix_mwi_subscribe_replaces_.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/ast-db-manage/config/versions/fe6592859b85_fix_mwi_subscribe_replaces_.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,61 @@
+"""Fix mwi_subscribe_replaces_unsolicited
+
+Revision ID: fe6592859b85
+Revises: 19b00bc19b7b
+Create Date: 2018-08-06 15:50:44.405534
+
+"""
+
+# revision identifiers, used by Alembic.
+revision = 'fe6592859b85'
+down_revision = '1d3ed26d9978'
+
+from alembic import op
+import sqlalchemy as sa
+from sqlalchemy.dialects.postgresql import ENUM
+
+AST_BOOL_NAME = 'ast_bool_values'
+# We'll just ignore the n/y and f/t abbreviations as Asterisk does not write
+# those aliases.
+AST_BOOL_VALUES = [ '0', '1',
+                    'off', 'on',
+                    'false', 'true',
+                    'no', 'yes' ]
+
+
+def upgrade():
+    # Create the new enum
+    ast_bool_values = ENUM(*AST_BOOL_VALUES, name=AST_BOOL_NAME, create_type=False)
+    if op.get_context().bind.dialect.name == 'postgresql':
+        ast_bool_values.create(op.get_bind(), checkfirst=False)
+
+    # There is no direct way to convert from Integer to ENUM that is
+    # not database specific so we transition through a string type.
+    op.alter_column('ps_endpoints', 'mwi_subscribe_replaces_unsolicited',
+                    type_=sa.String(5))
+    op.alter_column('ps_endpoints', 'mwi_subscribe_replaces_unsolicited',
+                    type_=ast_bool_values)
+
+
+def downgrade():
+    # First we need to ensure the column is using only the 'numeric' bool enum values.
+    op.execute("UPDATE ps_endpoints SET mwi_subscribe_replaces_unsolicited='0'"
+               " WHERE mwi_subscribe_replaces_unsolicited='off'"
+               "  OR mwi_subscribe_replaces_unsolicited='false'"
+               "  OR mwi_subscribe_replaces_unsolicited='no'")
+    op.execute("UPDATE ps_endpoints SET mwi_subscribe_replaces_unsolicited='1'"
+               " WHERE mwi_subscribe_replaces_unsolicited='on'"
+               "  OR mwi_subscribe_replaces_unsolicited='true'"
+               "  OR mwi_subscribe_replaces_unsolicited='yes'")
+
+    # There is no direct way to convert from ENUM to Integer that is
+    # not database specific so we transition through a string type.
+    if op.get_context().bind.dialect.name == 'mssql':
+        op.drop_constraint('ck_ps_endpoints_mwi_subscribe_replaces_unsolicited_ast_bool_values', 'ps_endpoints')
+    op.alter_column('ps_endpoints', 'mwi_subscribe_replaces_unsolicited',
+                    type_=sa.String(5))
+    op.alter_column('ps_endpoints', 'mwi_subscribe_replaces_unsolicited',
+                    type_=sa.Integer)
+
+    if op.get_context().bind.dialect.name == 'postgresql':
+        ENUM(name=AST_BOOL_NAME).drop(op.get_bind(), checkfirst=False)
diff -pruN 1:13.23.1~dfsg-2/contrib/Makefile 1:16.1.1~dfsg-1/contrib/Makefile
--- 1:13.23.1~dfsg-2/contrib/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -25,9 +25,13 @@ install:
 	$(INSTALL) -m 755 scripts/ast_loggrabber "$(DESTDIR)$(ASTDATADIR)/scripts/ast_loggrabber"
 	$(INSTALL) -m 755 scripts/ast_coredumper "$(DESTDIR)$(ASTDATADIR)/scripts/ast_coredumper"
 	$(INSTALL) -m 755 scripts/refcounter.py "$(DESTDIR)$(ASTDATADIR)/scripts/refcounter.py"
+	$(INSTALL) -m 755 scripts/refstats.py "$(DESTDIR)$(ASTDATADIR)/scripts/refstats.py"
+	$(INSTALL) -m 755 scripts/reflocks.py "$(DESTDIR)$(ASTDATADIR)/scripts/reflocks.py"
 
 uninstall:
 	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/ast_logescalator"
 	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/ast_loggrabber"
 	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/ast_coredumper"
 	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/refcounter.py"
+	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/refstats.py"
+	-rm -f "$(DESTDIR)$(ASTDATADIR)/scripts/reflocks.py"
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_cdr.sql 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_cdr.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_cdr.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_cdr.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN TRANSACTION;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 GO
@@ -38,6 +39,20 @@ INSERT INTO alembic_version (version_num
 
 GO
 
+-- Running upgrade 210693f3123d -> 54cde9847798
+
+ALTER TABLE cdr ALTER COLUMN accountcode VARCHAR(80);
+
+GO
+
+ALTER TABLE cdr ALTER COLUMN peeraccount VARCHAR(80);
+
+GO
+
+UPDATE alembic_version SET version_num='54cde9847798' WHERE alembic_version.version_num = '210693f3123d';
+
+GO
+
 COMMIT;
 
 GO
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_config.sql 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_config.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_config.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_config.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN TRANSACTION;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 GO
@@ -1057,41 +1058,55 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade 10aedae86a32 -> eb88a14f2a
+-- Running upgrade 10aedae86a32 -> 371a3bf4143e
 
-ALTER TABLE ps_endpoints ADD media_encryption_optimistic VARCHAR(3) NULL;
+ALTER TABLE ps_endpoints ADD user_eq_phone VARCHAR(3) NULL;
 
 GO
 
-ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (media_encryption_optimistic IN ('yes', 'no'));
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (user_eq_phone IN ('yes', 'no'));
 
 GO
 
-UPDATE alembic_version SET version_num='eb88a14f2a' WHERE alembic_version.version_num = '10aedae86a32';
+UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = '10aedae86a32';
 
 GO
 
--- Running upgrade eb88a14f2a -> 371a3bf4143e
+-- Running upgrade 371a3bf4143e -> 15b1430ad6f1
 
-ALTER TABLE ps_endpoints ADD user_eq_phone VARCHAR(3) NULL;
+ALTER TABLE ps_endpoints ADD moh_passthrough VARCHAR(3) NULL;
 
 GO
 
-ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (user_eq_phone IN ('yes', 'no'));
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (moh_passthrough IN ('yes', 'no'));
 
 GO
 
-UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = 'eb88a14f2a';
+UPDATE alembic_version SET version_num='15b1430ad6f1' WHERE alembic_version.version_num = '371a3bf4143e';
 
 GO
 
--- Running upgrade 371a3bf4143e -> 45e3f47c6c44
+-- Running upgrade 15b1430ad6f1 -> 945b1098bdd
+
+ALTER TABLE ps_endpoints ADD media_encryption_optimistic VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (media_encryption_optimistic IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='945b1098bdd' WHERE alembic_version.version_num = '15b1430ad6f1';
+
+GO
+
+-- Running upgrade 945b1098bdd -> 45e3f47c6c44
 
 ALTER TABLE ps_globals ADD endpoint_identifier_order VARCHAR(40) NULL;
 
 GO
 
-UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '371a3bf4143e';
+UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '945b1098bdd';
 
 GO
 
@@ -1209,13 +1224,31 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade 28ce1e718f05 -> 189a235b3fd7
+-- Running upgrade 28ce1e718f05 -> 339a3bdf53fc
+
+ALTER TABLE ps_endpoints ALTER COLUMN accountcode VARCHAR(80);
+
+GO
+
+ALTER TABLE sippeers ALTER COLUMN accountcode VARCHAR(80);
+
+GO
+
+ALTER TABLE iaxfriends ALTER COLUMN accountcode VARCHAR(80);
+
+GO
+
+UPDATE alembic_version SET version_num='339a3bdf53fc' WHERE alembic_version.version_num = '28ce1e718f05';
+
+GO
+
+-- Running upgrade 339a3bdf53fc -> 189a235b3fd7
 
 ALTER TABLE ps_globals ADD keep_alive_interval INTEGER NULL;
 
 GO
 
-UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '28ce1e718f05';
+UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '339a3bdf53fc';
 
 GO
 
@@ -1403,7 +1436,7 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade 6be31516058d -> bca7113d796f
+-- Running upgrade 6be31516058d -> d7e3c73eb2bf
 
 ALTER TABLE ps_endpoints ADD [deny] VARCHAR(95) NULL;
 
@@ -1429,11 +1462,11 @@ ALTER TABLE ps_endpoints ADD contact_acl
 
 GO
 
-UPDATE alembic_version SET version_num='bca7113d796f' WHERE alembic_version.version_num = '6be31516058d';
+UPDATE alembic_version SET version_num='d7e3c73eb2bf' WHERE alembic_version.version_num = '6be31516058d';
 
 GO
 
--- Running upgrade bca7113d796f -> a845e4d8ade8
+-- Running upgrade d7e3c73eb2bf -> a845e4d8ade8
 
 ALTER TABLE ps_contacts ADD via_addr VARCHAR(40) NULL;
 
@@ -1447,7 +1480,7 @@ ALTER TABLE ps_contacts ADD call_id VARC
 
 GO
 
-UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'bca7113d796f';
+UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'd7e3c73eb2bf';
 
 GO
 
@@ -1543,7 +1576,21 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade 4e2493ef32e6 -> a6ef36f1309
+-- Running upgrade 4e2493ef32e6 -> 7f3e21abe318
+
+ALTER TABLE ps_endpoints ADD preferred_codec_only VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (preferred_codec_only IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='7f3e21abe318' WHERE alembic_version.version_num = '4e2493ef32e6';
+
+GO
+
+-- Running upgrade 7f3e21abe318 -> a6ef36f1309
 
 ALTER TABLE ps_globals ADD ignore_uri_user_options VARCHAR(3) NULL;
 
@@ -1553,7 +1600,7 @@ ALTER TABLE ps_globals ADD CONSTRAINT ye
 
 GO
 
-UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '4e2493ef32e6';
+UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '7f3e21abe318';
 
 GO
 
@@ -1755,7 +1802,21 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade d7983954dd96 -> 164abbd708c
+-- Running upgrade d7983954dd96 -> 39959b9c2566
+
+ALTER TABLE ps_endpoints ADD max_audio_streams INTEGER NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD max_video_streams INTEGER NULL;
+
+GO
+
+UPDATE alembic_version SET version_num='39959b9c2566' WHERE alembic_version.version_num = 'd7983954dd96';
+
+GO
+
+-- Running upgrade 39959b9c2566 -> 164abbd708c
 
 ALTER TABLE ps_endpoints DROP CONSTRAINT pjsip_dtmf_mode_values_v2;
 
@@ -1769,11 +1830,25 @@ ALTER TABLE ps_endpoints ADD CONSTRAINT
 
 GO
 
-UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96';
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = '39959b9c2566';
+
+GO
+
+-- Running upgrade 164abbd708c -> 44ccced114ce
+
+ALTER TABLE ps_endpoints ADD webrtc VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (webrtc IN ('yes', 'no'));
 
 GO
 
--- Running upgrade 164abbd708c -> f3d1c5d38b56
+UPDATE alembic_version SET version_num='44ccced114ce' WHERE alembic_version.version_num = '164abbd708c';
+
+GO
+
+-- Running upgrade 44ccced114ce -> f3d1c5d38b56
 
 ALTER TABLE ps_contacts ADD prune_on_boot VARCHAR(3) NULL;
 
@@ -1783,7 +1858,7 @@ ALTER TABLE ps_contacts ADD CONSTRAINT y
 
 GO
 
-UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c';
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '44ccced114ce';
 
 GO
 
@@ -1829,7 +1904,45 @@ UPDATE alembic_version SET version_num='
 
 GO
 
--- Running upgrade 20abce6d1e3c -> 52798ad97bdf
+-- Running upgrade 20abce6d1e3c -> de83fac997e2
+
+ALTER TABLE ps_endpoints ADD bundle VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (bundle IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='de83fac997e2' WHERE alembic_version.version_num = '20abce6d1e3c';
+
+GO
+
+-- Running upgrade de83fac997e2 -> 041c0d3d1857
+
+ALTER TABLE ps_endpoints ADD dtls_auto_generate_cert VARCHAR(3) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (dtls_auto_generate_cert IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='041c0d3d1857' WHERE alembic_version.version_num = 'de83fac997e2';
+
+GO
+
+-- Running upgrade 041c0d3d1857 -> e2f04d309071
+
+ALTER TABLE queue_members ADD wrapuptime INTEGER NULL;
+
+GO
+
+UPDATE alembic_version SET version_num='e2f04d309071' WHERE alembic_version.version_num = '041c0d3d1857';
+
+GO
+
+-- Running upgrade e2f04d309071 -> 52798ad97bdf
 
 ALTER TABLE ps_endpoints DROP CONSTRAINT ck_ps_endpoints_identify_by_pjsip_identify_by_values;
 
@@ -1839,7 +1952,7 @@ ALTER TABLE ps_endpoints ALTER COLUMN id
 
 GO
 
-UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '20abce6d1e3c';
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = 'e2f04d309071';
 
 GO
 
@@ -1919,6 +2032,86 @@ UPDATE alembic_version SET version_num='
 
 GO
 
+-- Running upgrade 1d3ed26d9978 -> fe6592859b85
+
+ALTER TABLE ps_endpoints ALTER COLUMN mwi_subscribe_replaces_unsolicited VARCHAR(5);
+
+GO
+
+ALTER TABLE ps_endpoints ALTER COLUMN mwi_subscribe_replaces_unsolicited VARCHAR(5);
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (mwi_subscribe_replaces_unsolicited IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'));
+
+GO
+
+UPDATE alembic_version SET version_num='fe6592859b85' WHERE alembic_version.version_num = '1d3ed26d9978';
+
+GO
+
+-- Running upgrade fe6592859b85 -> 7f85dd44c775
+
+ALTER TABLE ps_endpoints DROP CONSTRAINT yesno_values;
+
+GO
+
+ALTER TABLE ps_endpoints ALTER COLUMN suppress_q850_reason_header VARCHAR(3);
+
+GO
+
+EXEC sp_rename 'ps_endpoints.suppress_q850_reason_header', suppress_q850_reason_headers, 'COLUMN';
+
+GO
+
+ALTER TABLE ps_endpoints ALTER COLUMN suppress_q850_reason_headers VARCHAR(3);
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (suppress_q850_reason_headers IN ('yes', 'no'));
+
+GO
+
+UPDATE alembic_version SET version_num='7f85dd44c775' WHERE alembic_version.version_num = 'fe6592859b85';
+
+GO
+
+-- Running upgrade 7f85dd44c775 -> 2bb1a85135ad
+
+ALTER TABLE ps_globals ADD use_callerid_contact VARCHAR(5) NULL;
+
+GO
+
+ALTER TABLE ps_globals ADD CONSTRAINT ast_bool_values CHECK (use_callerid_contact IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'));
+
+GO
+
+UPDATE alembic_version SET version_num='2bb1a85135ad' WHERE alembic_version.version_num = '7f85dd44c775';
+
+GO
+
+-- Running upgrade 2bb1a85135ad -> 1ac563b350a8
+
+ALTER TABLE ps_endpoints ADD trust_connected_line VARCHAR(5) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (trust_connected_line IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'));
+
+GO
+
+ALTER TABLE ps_endpoints ADD send_connected_line VARCHAR(5) NULL;
+
+GO
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (send_connected_line IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'));
+
+GO
+
+UPDATE alembic_version SET version_num='1ac563b350a8' WHERE alembic_version.version_num = '2bb1a85135ad';
+
+GO
+
 COMMIT;
 
 GO
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_voicemail.sql 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_voicemail.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mssql/mssql_voicemail.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mssql/mssql_voicemail.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN TRANSACTION;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 GO
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_cdr.sql 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_cdr.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_cdr.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_cdr.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> 210693f3123d
@@ -30,3 +31,11 @@ CREATE TABLE cdr (
 
 INSERT INTO alembic_version (version_num) VALUES ('210693f3123d');
 
+-- Running upgrade 210693f3123d -> 54cde9847798
+
+ALTER TABLE cdr MODIFY accountcode VARCHAR(80) NULL;
+
+ALTER TABLE cdr MODIFY peeraccount VARCHAR(80) NULL;
+
+UPDATE alembic_version SET version_num='54cde9847798' WHERE alembic_version.version_num = '210693f3123d';
+
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_config.sql 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_config.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_config.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_config.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> 4da0c5f79a9c
@@ -698,23 +699,29 @@ ALTER TABLE sippeers MODIFY directmedia
 
 UPDATE alembic_version SET version_num='10aedae86a32' WHERE alembic_version.version_num = '5950038a6ead';
 
--- Running upgrade 10aedae86a32 -> eb88a14f2a
+-- Running upgrade 10aedae86a32 -> 371a3bf4143e
 
-ALTER TABLE ps_endpoints ADD COLUMN media_encryption_optimistic ENUM('yes','no');
+ALTER TABLE ps_endpoints ADD COLUMN user_eq_phone ENUM('yes','no');
 
-UPDATE alembic_version SET version_num='eb88a14f2a' WHERE alembic_version.version_num = '10aedae86a32';
+UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = '10aedae86a32';
 
--- Running upgrade eb88a14f2a -> 371a3bf4143e
+-- Running upgrade 371a3bf4143e -> 15b1430ad6f1
 
-ALTER TABLE ps_endpoints ADD COLUMN user_eq_phone ENUM('yes','no');
+ALTER TABLE ps_endpoints ADD COLUMN moh_passthrough ENUM('yes','no');
+
+UPDATE alembic_version SET version_num='15b1430ad6f1' WHERE alembic_version.version_num = '371a3bf4143e';
+
+-- Running upgrade 15b1430ad6f1 -> 945b1098bdd
+
+ALTER TABLE ps_endpoints ADD COLUMN media_encryption_optimistic ENUM('yes','no');
 
-UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = 'eb88a14f2a';
+UPDATE alembic_version SET version_num='945b1098bdd' WHERE alembic_version.version_num = '15b1430ad6f1';
 
--- Running upgrade 371a3bf4143e -> 45e3f47c6c44
+-- Running upgrade 945b1098bdd -> 45e3f47c6c44
 
 ALTER TABLE ps_globals ADD COLUMN endpoint_identifier_order VARCHAR(40);
 
-UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '371a3bf4143e';
+UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '945b1098bdd';
 
 -- Running upgrade 45e3f47c6c44 -> 23530d604b96
 
@@ -774,11 +781,21 @@ ALTER TABLE ps_registrations ADD COLUMN
 
 UPDATE alembic_version SET version_num='28ce1e718f05' WHERE alembic_version.version_num = '154177371065';
 
--- Running upgrade 28ce1e718f05 -> 189a235b3fd7
+-- Running upgrade 28ce1e718f05 -> 339a3bdf53fc
+
+ALTER TABLE ps_endpoints MODIFY accountcode VARCHAR(80) NULL;
+
+ALTER TABLE sippeers MODIFY accountcode VARCHAR(80) NULL;
+
+ALTER TABLE iaxfriends MODIFY accountcode VARCHAR(80) NULL;
+
+UPDATE alembic_version SET version_num='339a3bdf53fc' WHERE alembic_version.version_num = '28ce1e718f05';
+
+-- Running upgrade 339a3bdf53fc -> 189a235b3fd7
 
 ALTER TABLE ps_globals ADD COLUMN keep_alive_interval INTEGER;
 
-UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '28ce1e718f05';
+UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '339a3bdf53fc';
 
 -- Running upgrade 189a235b3fd7 -> 2d078ec071b7
 
@@ -872,7 +889,7 @@ ALTER TABLE ps_contacts ADD COLUMN authe
 
 UPDATE alembic_version SET version_num='6be31516058d' WHERE alembic_version.version_num = '81b01a191a46';
 
--- Running upgrade 6be31516058d -> bca7113d796f
+-- Running upgrade 6be31516058d -> d7e3c73eb2bf
 
 ALTER TABLE ps_endpoints ADD COLUMN deny VARCHAR(95);
 
@@ -886,9 +903,9 @@ ALTER TABLE ps_endpoints ADD COLUMN cont
 
 ALTER TABLE ps_endpoints ADD COLUMN contact_acl VARCHAR(40);
 
-UPDATE alembic_version SET version_num='bca7113d796f' WHERE alembic_version.version_num = '6be31516058d';
+UPDATE alembic_version SET version_num='d7e3c73eb2bf' WHERE alembic_version.version_num = '6be31516058d';
 
--- Running upgrade bca7113d796f -> a845e4d8ade8
+-- Running upgrade d7e3c73eb2bf -> a845e4d8ade8
 
 ALTER TABLE ps_contacts ADD COLUMN via_addr VARCHAR(40);
 
@@ -896,7 +913,7 @@ ALTER TABLE ps_contacts ADD COLUMN via_p
 
 ALTER TABLE ps_contacts ADD COLUMN call_id VARCHAR(255);
 
-UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'bca7113d796f';
+UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'd7e3c73eb2bf';
 
 -- Running upgrade a845e4d8ade8 -> ef7efc2d3964
 
@@ -944,11 +961,17 @@ ALTER TABLE ps_endpoints ADD COLUMN cont
 
 UPDATE alembic_version SET version_num='4e2493ef32e6' WHERE alembic_version.version_num = '3772f8f828da';
 
--- Running upgrade 4e2493ef32e6 -> a6ef36f1309
+-- Running upgrade 4e2493ef32e6 -> 7f3e21abe318
+
+ALTER TABLE ps_endpoints ADD COLUMN preferred_codec_only ENUM('yes','no');
+
+UPDATE alembic_version SET version_num='7f3e21abe318' WHERE alembic_version.version_num = '4e2493ef32e6';
+
+-- Running upgrade 7f3e21abe318 -> a6ef36f1309
 
 ALTER TABLE ps_globals ADD COLUMN ignore_uri_user_options ENUM('yes','no');
 
-UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '4e2493ef32e6';
+UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '7f3e21abe318';
 
 -- Running upgrade a6ef36f1309 -> 4468b4a91372
 
@@ -1062,17 +1085,31 @@ ALTER TABLE ps_endpoints ADD COLUMN noti
 
 UPDATE alembic_version SET version_num='d7983954dd96' WHERE alembic_version.version_num = '86bb1efa278d';
 
--- Running upgrade d7983954dd96 -> 164abbd708c
+-- Running upgrade d7983954dd96 -> 39959b9c2566
+
+ALTER TABLE ps_endpoints ADD COLUMN max_audio_streams INTEGER;
+
+ALTER TABLE ps_endpoints ADD COLUMN max_video_streams INTEGER;
+
+UPDATE alembic_version SET version_num='39959b9c2566' WHERE alembic_version.version_num = 'd7983954dd96';
+
+-- Running upgrade 39959b9c2566 -> 164abbd708c
 
 ALTER TABLE ps_endpoints MODIFY dtmf_mode ENUM('rfc4733','inband','info','auto','auto_info') NULL;
 
-UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96';
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = '39959b9c2566';
+
+-- Running upgrade 164abbd708c -> 44ccced114ce
+
+ALTER TABLE ps_endpoints ADD COLUMN webrtc ENUM('yes','no');
 
--- Running upgrade 164abbd708c -> f3d1c5d38b56
+UPDATE alembic_version SET version_num='44ccced114ce' WHERE alembic_version.version_num = '164abbd708c';
+
+-- Running upgrade 44ccced114ce -> f3d1c5d38b56
 
 ALTER TABLE ps_contacts ADD COLUMN prune_on_boot ENUM('yes','no');
 
-UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c';
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '44ccced114ce';
 
 -- Running upgrade f3d1c5d38b56 -> b83645976fdd
 
@@ -1092,11 +1129,29 @@ ALTER TABLE ps_endpoints MODIFY identify
 
 UPDATE alembic_version SET version_num='20abce6d1e3c' WHERE alembic_version.version_num = 'a1698e8bb9c5';
 
--- Running upgrade 20abce6d1e3c -> 52798ad97bdf
+-- Running upgrade 20abce6d1e3c -> de83fac997e2
+
+ALTER TABLE ps_endpoints ADD COLUMN bundle ENUM('yes','no');
+
+UPDATE alembic_version SET version_num='de83fac997e2' WHERE alembic_version.version_num = '20abce6d1e3c';
+
+-- Running upgrade de83fac997e2 -> 041c0d3d1857
+
+ALTER TABLE ps_endpoints ADD COLUMN dtls_auto_generate_cert ENUM('yes','no');
+
+UPDATE alembic_version SET version_num='041c0d3d1857' WHERE alembic_version.version_num = 'de83fac997e2';
+
+-- Running upgrade 041c0d3d1857 -> e2f04d309071
+
+ALTER TABLE queue_members ADD COLUMN wrapuptime INTEGER;
+
+UPDATE alembic_version SET version_num='e2f04d309071' WHERE alembic_version.version_num = '041c0d3d1857';
+
+-- Running upgrade e2f04d309071 -> 52798ad97bdf
 
 ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR(80) NULL;
 
-UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '20abce6d1e3c';
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = 'e2f04d309071';
 
 -- Running upgrade 52798ad97bdf -> d3e4284f8707
 
@@ -1128,3 +1183,31 @@ ALTER TABLE ps_contacts MODIFY uri VARCH
 
 UPDATE alembic_version SET version_num='1d3ed26d9978' WHERE alembic_version.version_num = '19b00bc19b7b';
 
+-- Running upgrade 1d3ed26d9978 -> fe6592859b85
+
+ALTER TABLE ps_endpoints MODIFY mwi_subscribe_replaces_unsolicited VARCHAR(5) NULL;
+
+ALTER TABLE ps_endpoints MODIFY mwi_subscribe_replaces_unsolicited ENUM('0','1','off','on','false','true','no','yes') NULL;
+
+UPDATE alembic_version SET version_num='fe6592859b85' WHERE alembic_version.version_num = '1d3ed26d9978';
+
+-- Running upgrade fe6592859b85 -> 7f85dd44c775
+
+ALTER TABLE ps_endpoints CHANGE suppress_q850_reason_header suppress_q850_reason_headers ENUM('yes','no') NULL;
+
+UPDATE alembic_version SET version_num='7f85dd44c775' WHERE alembic_version.version_num = 'fe6592859b85';
+
+-- Running upgrade 7f85dd44c775 -> 2bb1a85135ad
+
+ALTER TABLE ps_globals ADD COLUMN use_callerid_contact ENUM('0','1','off','on','false','true','no','yes');
+
+UPDATE alembic_version SET version_num='2bb1a85135ad' WHERE alembic_version.version_num = '7f85dd44c775';
+
+-- Running upgrade 2bb1a85135ad -> 1ac563b350a8
+
+ALTER TABLE ps_endpoints ADD COLUMN trust_connected_line ENUM('0','1','off','on','false','true','no','yes');
+
+ALTER TABLE ps_endpoints ADD COLUMN send_connected_line ENUM('0','1','off','on','false','true','no','yes');
+
+UPDATE alembic_version SET version_num='1ac563b350a8' WHERE alembic_version.version_num = '2bb1a85135ad';
+
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_voicemail.sql 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_voicemail.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/mysql/mysql_voicemail.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/mysql/mysql_voicemail.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> a2e9769475e
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_cdr.sql 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_cdr.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_cdr.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_cdr.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR2(32 CHAR) NOT NULL
+    version_num VARCHAR2(32 CHAR) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 )
 
 /
@@ -36,3 +37,17 @@ INSERT INTO alembic_version (version_num
 
 /
 
+-- Running upgrade 210693f3123d -> 54cde9847798
+
+ALTER TABLE cdr MODIFY accountcode VARCHAR2(80 CHAR)
+
+/
+
+ALTER TABLE cdr MODIFY peeraccount VARCHAR2(80 CHAR)
+
+/
+
+UPDATE alembic_version SET version_num='54cde9847798' WHERE alembic_version.version_num = '210693f3123d'
+
+/
+
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_config.sql 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_config.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_config.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_config.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR2(32 CHAR) NOT NULL
+    version_num VARCHAR2(32 CHAR) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 )
 
 /
@@ -1039,41 +1040,55 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade 10aedae86a32 -> eb88a14f2a
+-- Running upgrade 10aedae86a32 -> 371a3bf4143e
 
-ALTER TABLE ps_endpoints ADD media_encryption_optimistic VARCHAR(3 CHAR)
+ALTER TABLE ps_endpoints ADD user_eq_phone VARCHAR(3 CHAR)
 
 /
 
-ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (media_encryption_optimistic IN ('yes', 'no'))
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (user_eq_phone IN ('yes', 'no'))
 
 /
 
-UPDATE alembic_version SET version_num='eb88a14f2a' WHERE alembic_version.version_num = '10aedae86a32'
+UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = '10aedae86a32'
 
 /
 
--- Running upgrade eb88a14f2a -> 371a3bf4143e
+-- Running upgrade 371a3bf4143e -> 15b1430ad6f1
 
-ALTER TABLE ps_endpoints ADD user_eq_phone VARCHAR(3 CHAR)
+ALTER TABLE ps_endpoints ADD moh_passthrough VARCHAR(3 CHAR)
 
 /
 
-ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (user_eq_phone IN ('yes', 'no'))
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (moh_passthrough IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='15b1430ad6f1' WHERE alembic_version.version_num = '371a3bf4143e'
+
+/
+
+-- Running upgrade 15b1430ad6f1 -> 945b1098bdd
+
+ALTER TABLE ps_endpoints ADD media_encryption_optimistic VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (media_encryption_optimistic IN ('yes', 'no'))
 
 /
 
-UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = 'eb88a14f2a'
+UPDATE alembic_version SET version_num='945b1098bdd' WHERE alembic_version.version_num = '15b1430ad6f1'
 
 /
 
--- Running upgrade 371a3bf4143e -> 45e3f47c6c44
+-- Running upgrade 945b1098bdd -> 45e3f47c6c44
 
 ALTER TABLE ps_globals ADD endpoint_identifier_order VARCHAR2(40 CHAR)
 
 /
 
-UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '371a3bf4143e'
+UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '945b1098bdd'
 
 /
 
@@ -1191,13 +1206,31 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade 28ce1e718f05 -> 189a235b3fd7
+-- Running upgrade 28ce1e718f05 -> 339a3bdf53fc
+
+ALTER TABLE ps_endpoints MODIFY accountcode VARCHAR2(80 CHAR)
+
+/
+
+ALTER TABLE sippeers MODIFY accountcode VARCHAR2(80 CHAR)
+
+/
+
+ALTER TABLE iaxfriends MODIFY accountcode VARCHAR2(80 CHAR)
+
+/
+
+UPDATE alembic_version SET version_num='339a3bdf53fc' WHERE alembic_version.version_num = '28ce1e718f05'
+
+/
+
+-- Running upgrade 339a3bdf53fc -> 189a235b3fd7
 
 ALTER TABLE ps_globals ADD keep_alive_interval INTEGER
 
 /
 
-UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '28ce1e718f05'
+UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '339a3bdf53fc'
 
 /
 
@@ -1385,7 +1418,7 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade 6be31516058d -> bca7113d796f
+-- Running upgrade 6be31516058d -> d7e3c73eb2bf
 
 ALTER TABLE ps_endpoints ADD deny VARCHAR2(95 CHAR)
 
@@ -1411,11 +1444,11 @@ ALTER TABLE ps_endpoints ADD contact_acl
 
 /
 
-UPDATE alembic_version SET version_num='bca7113d796f' WHERE alembic_version.version_num = '6be31516058d'
+UPDATE alembic_version SET version_num='d7e3c73eb2bf' WHERE alembic_version.version_num = '6be31516058d'
 
 /
 
--- Running upgrade bca7113d796f -> a845e4d8ade8
+-- Running upgrade d7e3c73eb2bf -> a845e4d8ade8
 
 ALTER TABLE ps_contacts ADD via_addr VARCHAR2(40 CHAR)
 
@@ -1429,7 +1462,7 @@ ALTER TABLE ps_contacts ADD call_id VARC
 
 /
 
-UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'bca7113d796f'
+UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'd7e3c73eb2bf'
 
 /
 
@@ -1521,7 +1554,21 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade 4e2493ef32e6 -> a6ef36f1309
+-- Running upgrade 4e2493ef32e6 -> 7f3e21abe318
+
+ALTER TABLE ps_endpoints ADD preferred_codec_only VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (preferred_codec_only IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='7f3e21abe318' WHERE alembic_version.version_num = '4e2493ef32e6'
+
+/
+
+-- Running upgrade 7f3e21abe318 -> a6ef36f1309
 
 ALTER TABLE ps_globals ADD ignore_uri_user_options VARCHAR(3 CHAR)
 
@@ -1531,7 +1578,7 @@ ALTER TABLE ps_globals ADD CONSTRAINT ye
 
 /
 
-UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '4e2493ef32e6'
+UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '7f3e21abe318'
 
 /
 
@@ -1733,7 +1780,21 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade d7983954dd96 -> 164abbd708c
+-- Running upgrade d7983954dd96 -> 39959b9c2566
+
+ALTER TABLE ps_endpoints ADD max_audio_streams INTEGER
+
+/
+
+ALTER TABLE ps_endpoints ADD max_video_streams INTEGER
+
+/
+
+UPDATE alembic_version SET version_num='39959b9c2566' WHERE alembic_version.version_num = 'd7983954dd96'
+
+/
+
+-- Running upgrade 39959b9c2566 -> 164abbd708c
 
 ALTER TABLE ps_endpoints DROP CONSTRAINT pjsip_dtmf_mode_values_v2
 
@@ -1747,11 +1808,25 @@ ALTER TABLE ps_endpoints ADD CONSTRAINT
 
 /
 
-UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96'
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = '39959b9c2566'
 
 /
 
--- Running upgrade 164abbd708c -> f3d1c5d38b56
+-- Running upgrade 164abbd708c -> 44ccced114ce
+
+ALTER TABLE ps_endpoints ADD webrtc VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (webrtc IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='44ccced114ce' WHERE alembic_version.version_num = '164abbd708c'
+
+/
+
+-- Running upgrade 44ccced114ce -> f3d1c5d38b56
 
 ALTER TABLE ps_contacts ADD prune_on_boot VARCHAR(3 CHAR)
 
@@ -1761,7 +1836,7 @@ ALTER TABLE ps_contacts ADD CONSTRAINT y
 
 /
 
-UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c'
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '44ccced114ce'
 
 /
 
@@ -1803,13 +1878,51 @@ UPDATE alembic_version SET version_num='
 
 /
 
--- Running upgrade 20abce6d1e3c -> 52798ad97bdf
+-- Running upgrade 20abce6d1e3c -> de83fac997e2
+
+ALTER TABLE ps_endpoints ADD bundle VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (bundle IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='de83fac997e2' WHERE alembic_version.version_num = '20abce6d1e3c'
+
+/
+
+-- Running upgrade de83fac997e2 -> 041c0d3d1857
+
+ALTER TABLE ps_endpoints ADD dtls_auto_generate_cert VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (dtls_auto_generate_cert IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='041c0d3d1857' WHERE alembic_version.version_num = 'de83fac997e2'
+
+/
+
+-- Running upgrade 041c0d3d1857 -> e2f04d309071
+
+ALTER TABLE queue_members ADD wrapuptime INTEGER
+
+/
+
+UPDATE alembic_version SET version_num='e2f04d309071' WHERE alembic_version.version_num = '041c0d3d1857'
+
+/
+
+-- Running upgrade e2f04d309071 -> 52798ad97bdf
 
 ALTER TABLE ps_endpoints MODIFY identify_by VARCHAR2(80 CHAR)
 
 /
 
-UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '20abce6d1e3c'
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = 'e2f04d309071'
 
 /
 
@@ -1889,3 +2002,75 @@ UPDATE alembic_version SET version_num='
 
 /
 
+-- Running upgrade 1d3ed26d9978 -> fe6592859b85
+
+ALTER TABLE ps_endpoints MODIFY mwi_subscribe_replaces_unsolicited VARCHAR2(5 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints MODIFY mwi_subscribe_replaces_unsolicited VARCHAR(5 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (mwi_subscribe_replaces_unsolicited IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'))
+
+/
+
+UPDATE alembic_version SET version_num='fe6592859b85' WHERE alembic_version.version_num = '1d3ed26d9978'
+
+/
+
+-- Running upgrade fe6592859b85 -> 7f85dd44c775
+
+ALTER TABLE ps_endpoints MODIFY suppress_q850_reason_header VARCHAR(3 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints RENAME COLUMN suppress_q850_reason_header TO suppress_q850_reason_headers
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT yesno_values CHECK (suppress_q850_reason_header IN ('yes', 'no'))
+
+/
+
+UPDATE alembic_version SET version_num='7f85dd44c775' WHERE alembic_version.version_num = 'fe6592859b85'
+
+/
+
+-- Running upgrade 7f85dd44c775 -> 2bb1a85135ad
+
+ALTER TABLE ps_globals ADD use_callerid_contact VARCHAR(5 CHAR)
+
+/
+
+ALTER TABLE ps_globals ADD CONSTRAINT ast_bool_values CHECK (use_callerid_contact IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'))
+
+/
+
+UPDATE alembic_version SET version_num='2bb1a85135ad' WHERE alembic_version.version_num = '7f85dd44c775'
+
+/
+
+-- Running upgrade 2bb1a85135ad -> 1ac563b350a8
+
+ALTER TABLE ps_endpoints ADD trust_connected_line VARCHAR(5 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (trust_connected_line IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'))
+
+/
+
+ALTER TABLE ps_endpoints ADD send_connected_line VARCHAR(5 CHAR)
+
+/
+
+ALTER TABLE ps_endpoints ADD CONSTRAINT ast_bool_values CHECK (send_connected_line IN ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes'))
+
+/
+
+UPDATE alembic_version SET version_num='1ac563b350a8' WHERE alembic_version.version_num = '2bb1a85135ad'
+
+/
+
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_voicemail.sql 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_voicemail.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/oracle/oracle_voicemail.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/oracle/oracle_voicemail.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 CREATE TABLE alembic_version (
-    version_num VARCHAR2(32 CHAR) NOT NULL
+    version_num VARCHAR2(32 CHAR) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 )
 
 /
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_cdr.sql 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_cdr.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_cdr.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_cdr.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> 210693f3123d
@@ -32,5 +33,13 @@ CREATE TABLE cdr (
 
 INSERT INTO alembic_version (version_num) VALUES ('210693f3123d');
 
+-- Running upgrade 210693f3123d -> 54cde9847798
+
+ALTER TABLE cdr ALTER COLUMN accountcode TYPE VARCHAR(80);
+
+ALTER TABLE cdr ALTER COLUMN peeraccount TYPE VARCHAR(80);
+
+UPDATE alembic_version SET version_num='54cde9847798' WHERE alembic_version.version_num = '210693f3123d';
+
 COMMIT;
 
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_config.sql 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_config.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_config.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_config.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> 4da0c5f79a9c
@@ -764,23 +765,29 @@ DROP TYPE sip_directmedia_values;
 
 UPDATE alembic_version SET version_num='10aedae86a32' WHERE alembic_version.version_num = '5950038a6ead';
 
--- Running upgrade 10aedae86a32 -> eb88a14f2a
+-- Running upgrade 10aedae86a32 -> 371a3bf4143e
 
-ALTER TABLE ps_endpoints ADD COLUMN media_encryption_optimistic yesno_values;
+ALTER TABLE ps_endpoints ADD COLUMN user_eq_phone yesno_values;
 
-UPDATE alembic_version SET version_num='eb88a14f2a' WHERE alembic_version.version_num = '10aedae86a32';
+UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = '10aedae86a32';
 
--- Running upgrade eb88a14f2a -> 371a3bf4143e
+-- Running upgrade 371a3bf4143e -> 15b1430ad6f1
 
-ALTER TABLE ps_endpoints ADD COLUMN user_eq_phone yesno_values;
+ALTER TABLE ps_endpoints ADD COLUMN moh_passthrough yesno_values;
+
+UPDATE alembic_version SET version_num='15b1430ad6f1' WHERE alembic_version.version_num = '371a3bf4143e';
+
+-- Running upgrade 15b1430ad6f1 -> 945b1098bdd
+
+ALTER TABLE ps_endpoints ADD COLUMN media_encryption_optimistic yesno_values;
 
-UPDATE alembic_version SET version_num='371a3bf4143e' WHERE alembic_version.version_num = 'eb88a14f2a';
+UPDATE alembic_version SET version_num='945b1098bdd' WHERE alembic_version.version_num = '15b1430ad6f1';
 
--- Running upgrade 371a3bf4143e -> 45e3f47c6c44
+-- Running upgrade 945b1098bdd -> 45e3f47c6c44
 
 ALTER TABLE ps_globals ADD COLUMN endpoint_identifier_order VARCHAR(40);
 
-UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '371a3bf4143e';
+UPDATE alembic_version SET version_num='45e3f47c6c44' WHERE alembic_version.version_num = '945b1098bdd';
 
 -- Running upgrade 45e3f47c6c44 -> 23530d604b96
 
@@ -844,11 +851,21 @@ ALTER TABLE ps_registrations ADD COLUMN
 
 UPDATE alembic_version SET version_num='28ce1e718f05' WHERE alembic_version.version_num = '154177371065';
 
--- Running upgrade 28ce1e718f05 -> 189a235b3fd7
+-- Running upgrade 28ce1e718f05 -> 339a3bdf53fc
+
+ALTER TABLE ps_endpoints ALTER COLUMN accountcode TYPE VARCHAR(80);
+
+ALTER TABLE sippeers ALTER COLUMN accountcode TYPE VARCHAR(80);
+
+ALTER TABLE iaxfriends ALTER COLUMN accountcode TYPE VARCHAR(80);
+
+UPDATE alembic_version SET version_num='339a3bdf53fc' WHERE alembic_version.version_num = '28ce1e718f05';
+
+-- Running upgrade 339a3bdf53fc -> 189a235b3fd7
 
 ALTER TABLE ps_globals ADD COLUMN keep_alive_interval INTEGER;
 
-UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '28ce1e718f05';
+UPDATE alembic_version SET version_num='189a235b3fd7' WHERE alembic_version.version_num = '339a3bdf53fc';
 
 -- Running upgrade 189a235b3fd7 -> 2d078ec071b7
 
@@ -942,7 +959,7 @@ ALTER TABLE ps_contacts ADD COLUMN authe
 
 UPDATE alembic_version SET version_num='6be31516058d' WHERE alembic_version.version_num = '81b01a191a46';
 
--- Running upgrade 6be31516058d -> bca7113d796f
+-- Running upgrade 6be31516058d -> d7e3c73eb2bf
 
 ALTER TABLE ps_endpoints ADD COLUMN deny VARCHAR(95);
 
@@ -956,9 +973,9 @@ ALTER TABLE ps_endpoints ADD COLUMN cont
 
 ALTER TABLE ps_endpoints ADD COLUMN contact_acl VARCHAR(40);
 
-UPDATE alembic_version SET version_num='bca7113d796f' WHERE alembic_version.version_num = '6be31516058d';
+UPDATE alembic_version SET version_num='d7e3c73eb2bf' WHERE alembic_version.version_num = '6be31516058d';
 
--- Running upgrade bca7113d796f -> a845e4d8ade8
+-- Running upgrade d7e3c73eb2bf -> a845e4d8ade8
 
 ALTER TABLE ps_contacts ADD COLUMN via_addr VARCHAR(40);
 
@@ -966,7 +983,7 @@ ALTER TABLE ps_contacts ADD COLUMN via_p
 
 ALTER TABLE ps_contacts ADD COLUMN call_id VARCHAR(255);
 
-UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'bca7113d796f';
+UPDATE alembic_version SET version_num='a845e4d8ade8' WHERE alembic_version.version_num = 'd7e3c73eb2bf';
 
 -- Running upgrade a845e4d8ade8 -> ef7efc2d3964
 
@@ -1020,11 +1037,17 @@ ALTER TABLE ps_endpoints ADD COLUMN cont
 
 UPDATE alembic_version SET version_num='4e2493ef32e6' WHERE alembic_version.version_num = '3772f8f828da';
 
--- Running upgrade 4e2493ef32e6 -> a6ef36f1309
+-- Running upgrade 4e2493ef32e6 -> 7f3e21abe318
+
+ALTER TABLE ps_endpoints ADD COLUMN preferred_codec_only yesno_values;
+
+UPDATE alembic_version SET version_num='7f3e21abe318' WHERE alembic_version.version_num = '4e2493ef32e6';
+
+-- Running upgrade 7f3e21abe318 -> a6ef36f1309
 
 ALTER TABLE ps_globals ADD COLUMN ignore_uri_user_options yesno_values;
 
-UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '4e2493ef32e6';
+UPDATE alembic_version SET version_num='a6ef36f1309' WHERE alembic_version.version_num = '7f3e21abe318';
 
 -- Running upgrade a6ef36f1309 -> 4468b4a91372
 
@@ -1138,7 +1161,15 @@ ALTER TABLE ps_endpoints ADD COLUMN noti
 
 UPDATE alembic_version SET version_num='d7983954dd96' WHERE alembic_version.version_num = '86bb1efa278d';
 
--- Running upgrade d7983954dd96 -> 164abbd708c
+-- Running upgrade d7983954dd96 -> 39959b9c2566
+
+ALTER TABLE ps_endpoints ADD COLUMN max_audio_streams INTEGER;
+
+ALTER TABLE ps_endpoints ADD COLUMN max_video_streams INTEGER;
+
+UPDATE alembic_version SET version_num='39959b9c2566' WHERE alembic_version.version_num = 'd7983954dd96';
+
+-- Running upgrade 39959b9c2566 -> 164abbd708c
 
 CREATE TYPE pjsip_dtmf_mode_values_v3 AS ENUM ('rfc4733', 'inband', 'info', 'auto', 'auto_info');
 
@@ -1146,13 +1177,19 @@ ALTER TABLE ps_endpoints ALTER COLUMN dt
 
 DROP TYPE pjsip_dtmf_mode_values_v2;
 
-UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = 'd7983954dd96';
+UPDATE alembic_version SET version_num='164abbd708c' WHERE alembic_version.version_num = '39959b9c2566';
+
+-- Running upgrade 164abbd708c -> 44ccced114ce
+
+ALTER TABLE ps_endpoints ADD COLUMN webrtc yesno_values;
 
--- Running upgrade 164abbd708c -> f3d1c5d38b56
+UPDATE alembic_version SET version_num='44ccced114ce' WHERE alembic_version.version_num = '164abbd708c';
+
+-- Running upgrade 44ccced114ce -> f3d1c5d38b56
 
 ALTER TABLE ps_contacts ADD COLUMN prune_on_boot yesno_values;
 
-UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '164abbd708c';
+UPDATE alembic_version SET version_num='f3d1c5d38b56' WHERE alembic_version.version_num = '44ccced114ce';
 
 -- Running upgrade f3d1c5d38b56 -> b83645976fdd
 
@@ -1180,13 +1217,31 @@ DROP TYPE pjsip_identify_by_values_tmp;
 
 UPDATE alembic_version SET version_num='20abce6d1e3c' WHERE alembic_version.version_num = 'a1698e8bb9c5';
 
--- Running upgrade 20abce6d1e3c -> 52798ad97bdf
+-- Running upgrade 20abce6d1e3c -> de83fac997e2
+
+ALTER TABLE ps_endpoints ADD COLUMN bundle yesno_values;
+
+UPDATE alembic_version SET version_num='de83fac997e2' WHERE alembic_version.version_num = '20abce6d1e3c';
+
+-- Running upgrade de83fac997e2 -> 041c0d3d1857
+
+ALTER TABLE ps_endpoints ADD COLUMN dtls_auto_generate_cert yesno_values;
+
+UPDATE alembic_version SET version_num='041c0d3d1857' WHERE alembic_version.version_num = 'de83fac997e2';
+
+-- Running upgrade 041c0d3d1857 -> e2f04d309071
+
+ALTER TABLE queue_members ADD COLUMN wrapuptime INTEGER;
+
+UPDATE alembic_version SET version_num='e2f04d309071' WHERE alembic_version.version_num = '041c0d3d1857';
+
+-- Running upgrade e2f04d309071 -> 52798ad97bdf
 
 ALTER TABLE ps_endpoints ALTER COLUMN identify_by TYPE varchar(80) USING identify_by::text::pjsip_identify_by_values;
 
 DROP TYPE pjsip_identify_by_values;
 
-UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = '20abce6d1e3c';
+UPDATE alembic_version SET version_num='52798ad97bdf' WHERE alembic_version.version_num = 'e2f04d309071';
 
 -- Running upgrade 52798ad97bdf -> d3e4284f8707
 
@@ -1218,5 +1273,37 @@ ALTER TABLE ps_contacts ALTER COLUMN uri
 
 UPDATE alembic_version SET version_num='1d3ed26d9978' WHERE alembic_version.version_num = '19b00bc19b7b';
 
+-- Running upgrade 1d3ed26d9978 -> fe6592859b85
+
+CREATE TYPE ast_bool_values AS ENUM ('0', '1', 'off', 'on', 'false', 'true', 'no', 'yes');
+
+ALTER TABLE ps_endpoints ALTER COLUMN mwi_subscribe_replaces_unsolicited TYPE VARCHAR(5);
+
+ALTER TABLE ps_endpoints ALTER COLUMN mwi_subscribe_replaces_unsolicited TYPE ast_bool_values;
+
+UPDATE alembic_version SET version_num='fe6592859b85' WHERE alembic_version.version_num = '1d3ed26d9978';
+
+-- Running upgrade fe6592859b85 -> 7f85dd44c775
+
+ALTER TABLE ps_endpoints ALTER COLUMN suppress_q850_reason_header TYPE yesno_values;
+
+ALTER TABLE ps_endpoints RENAME suppress_q850_reason_header TO suppress_q850_reason_headers;
+
+UPDATE alembic_version SET version_num='7f85dd44c775' WHERE alembic_version.version_num = 'fe6592859b85';
+
+-- Running upgrade 7f85dd44c775 -> 2bb1a85135ad
+
+ALTER TABLE ps_globals ADD COLUMN use_callerid_contact ast_bool_values;
+
+UPDATE alembic_version SET version_num='2bb1a85135ad' WHERE alembic_version.version_num = '7f85dd44c775';
+
+-- Running upgrade 2bb1a85135ad -> 1ac563b350a8
+
+ALTER TABLE ps_endpoints ADD COLUMN trust_connected_line ast_bool_values;
+
+ALTER TABLE ps_endpoints ADD COLUMN send_connected_line ast_bool_values;
+
+UPDATE alembic_version SET version_num='1ac563b350a8' WHERE alembic_version.version_num = '2bb1a85135ad';
+
 COMMIT;
 
diff -pruN 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_voicemail.sql 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_voicemail.sql
--- 1:13.23.1~dfsg-2/contrib/realtime/postgresql/postgresql_voicemail.sql	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/realtime/postgresql/postgresql_voicemail.sql	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,8 @@
 BEGIN;
 
 CREATE TABLE alembic_version (
-    version_num VARCHAR(32) NOT NULL
+    version_num VARCHAR(32) NOT NULL, 
+    CONSTRAINT alembic_version_pkc PRIMARY KEY (version_num)
 );
 
 -- Running upgrade  -> a2e9769475e
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/ast_coredumper 1:16.1.1~dfsg-1/contrib/scripts/ast_coredumper
--- 1:13.23.1~dfsg-2/contrib/scripts/ast_coredumper	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/ast_coredumper	2018-12-26 18:45:13.000000000 +0000
@@ -235,8 +235,8 @@ declare -a ARGS_COREDUMPS
 
 readconf() {
 	while read line ; do
-		declare -n v=${line%%=*}
-		[ -z "${v}" ] && eval $line || :
+		v=${line%%=*}
+		[ -z "${!v}" ] && eval $line || :
 	done <<EOF
 $( sed -r -e "/\s*#/d" -e "/^\s*$/d" $1 )
 EOF
@@ -419,8 +419,12 @@ fi
 # Extract the gdb scripts from the end of this script
 # and save them to /tmp/.gdbinit
 
+gdbinit=${OUTPUTDIR:-/tmp}/.ast_coredumper.gdbinit
+
+trap "rm $gdbinit" EXIT
+
 ss=`egrep -n "^#@@@SCRIPTSTART@@@" $0 |cut -f1 -d:`
-tail -n +${ss} $0 >${OUTPUTDIR:-/tmp}/.ast_coredumper.gdbinit
+tail -n +${ss} $0 >$gdbinit
 
 # Now iterate over the coredumps and dump the debugging info
 for i in ${!COREDUMPS[@]} ; do
@@ -431,7 +435,7 @@ for i in ${!COREDUMPS[@]} ; do
 	cfname=`basename ${cf}`
 	outputdir=${OUTPUTDIR:-${cfdir}}
 
-	${GDB} -n --batch -q --ex "source ${OUTPUTDIR:-/tmp}/.ast_coredumper.gdbinit" "$asterisk_bin" "$cf" 2>/dev/null | (
+	${GDB} -n --batch -q --ex "source $gdbinit" "$asterisk_bin" "$cf" 2>/dev/null | (
 		of=/dev/null
 		while IFS= read line ; do
 			if [[ "$line" =~ !@!@!@!\ ([^\ ]+)\ !@!@!@! ]] ; then
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/get_mp3_source.sh 1:16.1.1~dfsg-1/contrib/scripts/get_mp3_source.sh
--- 1:13.23.1~dfsg-2/contrib/scripts/get_mp3_source.sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/get_mp3_source.sh	2018-12-26 18:45:13.000000000 +0000
@@ -6,9 +6,21 @@ if [ -f addons/mp3/mpg123.h ]; then
     echo "need to be downloaded."
     echo "***"
 
+    # Manually patch interface.c if not done yet.
+    if ! grep -q ASTMM_LIBC addons/mp3/interface.c; then
+        sed -i -e '/#include "asterisk.h"/i#define ASTMM_LIBC ASTMM_REDIRECT' \
+            addons/mp3/interface.c
+    fi
+
     exit 1
 fi
 
 svn export http://svn.digium.com/svn/thirdparty/mp3/trunk addons/mp3 $@
 
+# Manually patch interface.c if not done yet.
+if ! grep -q ASTMM_LIBC addons/mp3/interface.c; then
+    sed -i -e '/#include "asterisk.h"/i#define ASTMM_LIBC ASTMM_REDIRECT' \
+        addons/mp3/interface.c
+fi
+
 exit 0
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/install_prereq 1:16.1.1~dfsg-1/contrib/scripts/install_prereq
--- 1:13.23.1~dfsg-2/contrib/scripts/install_prereq	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/install_prereq	2018-12-26 18:45:13.000000000 +0000
@@ -28,6 +28,7 @@ PACKAGES_DEBIAN="$PACKAGES_DEBIAN libpq-
 PACKAGES_DEBIAN="$PACKAGES_DEBIAN libvpb-dev libmysqlclient-dev libbluetooth-dev libradcli-dev freetds-dev libosptk-dev libjack-jackd2-dev bash"
 PACKAGES_DEBIAN="$PACKAGES_DEBIAN libsnmp-dev libiksemel-dev libcorosync-common-dev libcpg-dev libcfg-dev libnewt-dev libpopt-dev libical-dev libspandsp-dev"
 PACKAGES_DEBIAN="$PACKAGES_DEBIAN libresample1-dev libc-client2007e-dev binutils-dev libsrtp0-dev libsrtp2-dev libgsm1-dev doxygen graphviz zlib1g-dev libldap2-dev"
+PACKAGES_DEBIAN="$PACKAGES_DEBIAN libcodec2-dev libfftw3-dev libsndfile1-dev libunbound-dev"
 # Asterisk: for the unpackaged below:
 PACKAGES_DEBIAN="$PACKAGES_DEBIAN wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -43,6 +44,7 @@ PACKAGES_RH="$PACKAGES_RH postgresql-dev
 PACKAGES_RH="$PACKAGES_RH mysql-devel bluez-libs-devel radcli-devel freetds-devel jack-audio-connection-kit-devel bash"
 PACKAGES_RH="$PACKAGES_RH net-snmp-devel iksemel-devel corosynclib-devel newt-devel popt-devel libical-devel spandsp-devel"
 PACKAGES_RH="$PACKAGES_RH libresample-devel uw-imap-devel binutils-devel libsrtp-devel gsm-devel doxygen graphviz zlib-devel openldap-devel hoard"
+PACKAGES_RH="$PACKAGES_RH codec2-devel fftw-devel libsndfile-devel unbound-devel"
 # Asterisk: for the unpackaged below:
 PACKAGES_RH="$PACKAGES_RH wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -58,6 +60,7 @@ PACKAGES_SUSE="$PACKAGES_SUSE postgresql
 PACKAGES_SUSE="$PACKAGES_SUSE libmysqlclient-devel bluez-devel freeradius-client-devel freetds-devel bash"
 PACKAGES_SUSE="$PACKAGES_SUSE net-snmp-devel iksemel-devel libcorosync-devel newt-devel popt-devel libical-devel spandsp-devel"
 PACKAGES_SUSE="$PACKAGES_SUSE imap-devel libsrtp2-devel libgsm-devel doxygen graphviz zlib-devel openldap2-devel"
+PACKAGES_SUSE="$PACKAGES_SUSE codec2-devel fftw3-devel libsndfile-devel unbound-devel"
 # Asterisk: for the unpackaged below:
 PACKAGES_SUSE="$PACKAGES_SUSE wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -73,6 +76,7 @@ PACKAGES_ARCH="$PACKAGES_ARCH postgresql
 PACKAGES_ARCH="$PACKAGES_ARCH libmariadbclient bluez-libs radcli freetds bash"
 PACKAGES_ARCH="$PACKAGES_ARCH net-snmp libnewt popt libical spandsp"
 PACKAGES_ARCH="$PACKAGES_ARCH c-client binutils libsrtp gsm doxygen graphviz zlib libldap"
+PACKAGES_ARCH="$PACKAGES_ARCH fftw libsndfile unbound"
 # Asterisk: for the unpackaged below:
 PACKAGES_ARCH="$PACKAGES_ARCH wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -88,6 +92,7 @@ PACKAGES_GENTOO="$PACKAGES_GENTOO dev-db
 PACKAGES_GENTOO="$PACKAGES_GENTOO virtual/libmysqlclient net-wireless/bluez net-dialup/radiusclient-ng dev-db/freetds app-shells/bash"
 PACKAGES_GENTOO="$PACKAGES_GENTOO net-analyzer/net-snmp dev-libs/iksemel sys-cluster/corosync dev-libs/newt dev-libs/popt dev-libs/libical media-libs/spandsp"
 PACKAGES_GENTOO="$PACKAGES_GENTOO net-libs/c-client sys-devel/binutils net-libs/libsrtp media-sound/gsm media-libs/libilbc app-doc/doxygen sys-libs/zlib net-nds/openldap"
+PACKAGES_GENTOO="$PACKAGES_GENTOO sci-libs/fftw media-libs/libsndfile net-dns/unbound"
 # Asterisk: for the unpackaged below:
 PACKAGES_GENTOO="$PACKAGES_GENTOO net-misc/wget dev-vcs/subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -103,6 +108,7 @@ PACKAGES_NBSD="$PACKAGES_NBSD postgresql
 PACKAGES_NBSD="$PACKAGES_NBSD mysql-client radiusclient-ng freetds bash"
 PACKAGES_NBSD="$PACKAGES_NBSD net-snmp iksemel popt libical spandsp"
 PACKAGES_NBSD="$PACKAGES_NBSD imap-uw srtp gsm doxygen graphviz libzip openldap-client"
+PACKAGES_NBSD="$PACKAGES_NBSD codec2 fftw libsndfile unbound"
 # Asterisk: for the unpackaged below:
 PACKAGES_NBSD="$PACKAGES_NBSD wget subversion-base"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -118,6 +124,7 @@ PACKAGES_OBSD="$PACKAGES_OBSD postgresql
 PACKAGES_OBSD="$PACKAGES_OBSD mariadb-client radcli freetds"
 PACKAGES_OBSD="$PACKAGES_OBSD net-snmp iksemel popt libical spandsp"
 PACKAGES_OBSD="$PACKAGES_OBSD c-client libsrtp gsm doxygen graphviz"
+PACKAGES_OBSD="$PACKAGES_OBSD fftw3 libsndfile"
 # Asterisk: for the unpackaged below:
 PACKAGES_OBSD="$PACKAGES_OBSD wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -133,6 +140,7 @@ PACKAGES_FBSD="$PACKAGES_FBSD postgresql
 PACKAGES_FBSD="$PACKAGES_FBSD mysql57-client radcli freetds"
 PACKAGES_FBSD="$PACKAGES_FBSD net-snmp iksemel corosync newt popt libical spandsp"
 PACKAGES_FBSD="$PACKAGES_FBSD cclient libbfd libsrtp gsm libilbc doxygen graphviz libzip openldap-client libhoard"
+PACKAGES_FBSD="$PACKAGES_FBSD codec2 fftw3 libsndfile unbound"
 # Asterisk: for the unpackaged below:
 PACKAGES_FBSD="$PACKAGES_FBSD wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -148,6 +156,7 @@ PACKAGES_DBSD="$PACKAGES_DBSD postgresql
 PACKAGES_DBSD="$PACKAGES_DBSD mariadb101-client radcli freetds"
 PACKAGES_DBSD="$PACKAGES_DBSD net-snmp iksemel corosync newt popt libical spandsp"
 PACKAGES_DBSD="$PACKAGES_DBSD cclient binutils libsrtp gsm libilbc doxygen graphviz libzip openldap-client libhoard"
+PACKAGES_DBSD="$PACKAGES_DBSD codec2 fftw3 libsndfile unbound"
 # Asterisk: for the unpackaged below:
 PACKAGES_DBSD="$PACKAGES_DBSD wget subversion"
 # Asterisk: for ./configure --with-pjproject-bundled:
@@ -155,7 +164,7 @@ PACKAGES_DBSD="$PACKAGES_DBSD bzip2 patc
 
 KVERS=`uname -r`
 
-JANSSON_VER=2.11
+JANSSON_VER=2.12
 
 case "$1" in
 test)
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/refcounter.py 1:16.1.1~dfsg-1/contrib/scripts/refcounter.py
--- 1:13.23.1~dfsg-2/contrib/scripts/refcounter.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/refcounter.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,8 +1,8 @@
 #!/usr/bin/env python
 """Process a ref debug log
 
- This file will process a log file created by the REF_DEBUG
- build option in Asterisk.
+ This file will process a log file created by enabling
+ the refdebug config option in asterisk.conf.
 
  See http://www.asterisk.org for more information about
  the Asterisk project. Please do not directly contact
@@ -108,6 +108,12 @@ def process_file(options):
             else:
                 current_objects[obj]['curcount'] += int(parsed_line['delta'])
 
+            # Suppress object sizes and lock-state from output logs.
+            if 'constructor' in parsed_line['state']:
+                parsed_line['state'] = '**constructor**'
+            elif 'destructor' in parsed_line['state']:
+                parsed_line['state'] = '**destructor**'
+
             current_objects[obj]['log'].append(
                 "[%s] %s:%s %s: %s %s - [%s]" % (
                     parsed_line['thread_id'],
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/reflocks.py 1:16.1.1~dfsg-1/contrib/scripts/reflocks.py
--- 1:13.23.1~dfsg-2/contrib/scripts/reflocks.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/reflocks.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,118 @@
+#!/usr/bin/env python
+"""Process a ref debug log for lock usage
+
+ This file will process a log file created by Asterisk
+ that was compiled with REF_DEBUG and DEBUG_THREADS.
+
+ See http://www.asterisk.org for more information about
+ the Asterisk project. Please do not directly contact
+ any of the maintainers of this project for assistance;
+ the project provides a web site, mailing lists and IRC
+ channels for your use.
+
+ This program is free software, distributed under the terms of
+ the GNU General Public License Version 2. See the LICENSE file
+ at the top of the source tree.
+
+ Copyright (C) 2018, CFWare, LLC
+ Corey Farrell <git@cfware.com>
+"""
+
+from __future__ import print_function
+import sys
+import os
+
+from optparse import OptionParser
+
+
+def process_file(options):
+    """The routine that kicks off processing a ref file"""
+
+    object_types = {}
+    objects = {}
+    filename = options.filepath
+
+    with open(filename, 'r') as ref_file:
+        for line in ref_file:
+            if 'constructor' not in line and 'destructor' not in line:
+                continue
+            # The line format is:
+            # addr,delta,thread_id,file,line,function,state,tag
+            # Only addr, file, line, function, state are used by reflocks.py
+            tokens = line.strip().split(',', 7)
+            addr = tokens[0]
+            state = tokens[6]
+            if 'constructor' in state:
+                obj_type = '%s:%s:%s' % (tokens[3], tokens[4], tokens[5])
+                if obj_type not in object_types:
+                    object_types[obj_type] = {
+                        'used': 0,
+                        'unused': 0,
+                        'none': 0
+                    }
+                objects[addr] = obj_type
+            elif 'destructor' in state:
+                if addr not in objects:
+                    # This error would be reported by refcounter.py.
+                    continue
+                obj_type = objects[addr]
+                del objects[addr]
+                if '**lock-state:unused**' in state:
+                    object_types[obj_type]['unused'] += 1
+                elif '**lock-state:used**' in state:
+                    object_types[obj_type]['used'] += 1
+                elif '**lock-state:none**' in state:
+                    object_types[obj_type]['none'] += 1
+
+    for (allocator, info) in object_types.items():
+        stats = [];
+        if info['used'] > 0:
+            if not options.used:
+                continue
+            stats.append("%d used" % info['used'])
+        if info['unused'] > 0:
+            stats.append("%d unused" % info['unused'])
+        if info['none'] > 0 and options.none:
+            stats.append("%d none" % info['none'])
+        if len(stats) == 0:
+            continue
+        print("%s: %s" % (allocator, ', '.join(stats)))
+
+
+def main(argv=None):
+    """Main entry point for the script"""
+
+    ret_code = 0
+
+    if argv is None:
+        argv = sys.argv
+
+    parser = OptionParser()
+
+    parser.add_option("-f", "--file", action="store", type="string",
+                      dest="filepath", default="/var/log/asterisk/refs",
+                      help="The full path to the refs file to process")
+    parser.add_option("-u", "--suppress-used", action="store_false",
+                      dest="used", default=True,
+                      help="Don't output types that have used locks.")
+    parser.add_option("-n", "--show-none", action="store_true",
+                      dest="none", default=False,
+                      help="Show counts of objects with no locking.")
+
+    (options, args) = parser.parse_args(argv)
+
+    if not os.path.isfile(options.filepath):
+        print("File not found: %s" % options.filepath, file=sys.stderr)
+        return -1
+
+    try:
+        process_file(options)
+    except (KeyboardInterrupt, SystemExit, IOError):
+        print("File processing cancelled", file=sys.stderr)
+        return -1
+
+    return ret_code
+
+
+if __name__ == "__main__":
+    sys.exit(main(sys.argv))
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/refstats.py 1:16.1.1~dfsg-1/contrib/scripts/refstats.py
--- 1:13.23.1~dfsg-2/contrib/scripts/refstats.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/refstats.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,180 @@
+#!/usr/bin/env python
+"""Process a ref debug log for memory usage
+
+ This will provide information about total and peak
+ allocations.
+
+ See http://www.asterisk.org for more information about
+ the Asterisk project. Please do not directly contact
+ any of the maintainers of this project for assistance;
+ the project provides a web site, mailing lists and IRC
+ channels for your use.
+
+ This program is free software, distributed under the terms of
+ the GNU General Public License Version 2. See the LICENSE file
+ at the top of the source tree.
+
+ Copyright (C) 2018, CFWare, LLC
+ Corey Farrell <git@cfware.com>
+"""
+
+from __future__ import print_function
+import sys
+import os
+
+from optparse import OptionParser
+
+
+def create_stats():
+    """Create statistics object"""
+    return {
+        'count': 0,
+        'overhead': 0,
+        'user_data': 0,
+        'totalmem': 0
+    }
+
+
+def update_stats(current, peak, total, key, direction, delta):
+    """Update statistics objects"""
+
+    if direction == 1:
+        total[key] += delta
+
+    delta *= direction
+    current[key] += delta
+    if current[key] > peak[key]:
+        peak[key] = current[key]
+
+
+def process_file(options):
+    """The routine that kicks off processing a ref file"""
+
+    current = create_stats()
+    total = create_stats()
+    peak = create_stats()
+    object_types = {}
+    objects = {}
+    filename = options.filepath
+
+    with open(filename, 'r') as ref_file:
+        for line in ref_file:
+            if 'constructor' not in line and 'destructor' not in line:
+                continue
+            # The line format is:
+            # addr,delta,thread_id,file,line,function,state,tag
+            # Only addr, file, line, function, state are used by reflocks.py
+            tokens = line.strip().split(',', 7)
+            addr = tokens[0]
+            state = tokens[6]
+            if 'constructor' in state:
+                split_state = state.split("**")
+                if len(split_state) < 4:
+                    print("File does not contain object size information", file=sys.stderr)
+                    sys.exit(1)
+
+                obj_type = '%s:%s:%s' % (tokens[3], tokens[4], tokens[5])
+                if obj_type not in object_types:
+                    object_types[obj_type] = {
+                        'used': 0,
+                        'unused': 0,
+                        'none': 0
+                    }
+                overhead = int(split_state[2])
+                user_data = int(split_state[3])
+                obj = objects[addr] = {
+                    'overhead': overhead,
+                    'user_data': user_data,
+                    'obj_type': obj_type
+                }
+
+                direction = 1
+            else:
+                if addr not in objects:
+                    # This error would be reported by refcounter.py.
+                    continue
+                obj = objects[addr]
+                del objects[addr]
+                direction = -1
+                obj_type = obj['obj_type']
+                if '**lock-state:unused**' in state:
+                    object_types[obj_type]['unused'] += 1
+                elif '**lock-state:used**' in state:
+                    object_types[obj_type]['used'] += 1
+
+            # Increment current and peak usage
+            update_stats(current, peak, total, 'count', direction, 1)
+            update_stats(current, peak, total, 'overhead', direction, obj['overhead'])
+            update_stats(current, peak, total, 'user_data', direction, obj['user_data'])
+            update_stats(current, peak, total, 'totalmem', direction, obj['overhead'] + obj['user_data'])
+
+    print("Total usage statistics:")
+    print("%20s: %d" % ("Count", total['count']))
+    print("%20s: %d" % ("Total Memory (k)", total['totalmem'] / 1024))
+    print("%20s: %d (%.2f%%)" % ("Overhead (k)", total['overhead'] / 1024, total['overhead'] * 100.0 / total['totalmem']))
+    print("%20s: %d" % ("User Data (k)", total['user_data'] / 1024))
+    print("")
+    print("Peak usage statistics:")
+    print("%20s: %d" % ("Count", peak['count']))
+    print("%20s: %d" % ("Total Memory (k)", peak['totalmem'] / 1024))
+    print("%20s: %d (%.2f%%)" % ("Overhead (k)", peak['overhead'] / 1024, peak['overhead'] * 100.0 / peak['totalmem']))
+    print("%20s: %d" % ("User Data (k)", peak['user_data'] / 1024))
+    print("")
+
+    lockbyobj = {'used': 0, 'total': 0}
+    lockbytype = {'used': 0, 'total': 0}
+    for (allocator, info) in object_types.items():
+        lockbyobj['used'] += info['used']
+        lockbyobj['total'] += info['used'] + info['unused']
+
+        if info['used'] != 0:
+            lockbytype['used'] += 1
+        elif info['unused'] == 0:
+            # This object type doesn't have locking.
+            continue
+        lockbytype['total'] += 1
+
+    print("Lock usage statistics:")
+    print("%20s: %d of %d used (%.2f%%)" % (
+        "By object",
+        lockbyobj['used'],
+        lockbyobj['total'],
+        lockbyobj['used'] * 100.0 / lockbyobj['total']))
+    print("%20s: %d of %d used (%.2f%%)" % (
+        "By type",
+        lockbytype['used'],
+        lockbytype['total'],
+        lockbytype['used'] * 100.0 / lockbytype['total']))
+
+
+def main(argv=None):
+    """Main entry point for the script"""
+
+    ret_code = 0
+
+    if argv is None:
+        argv = sys.argv
+
+    parser = OptionParser()
+
+    parser.add_option("-f", "--file", action="store", type="string",
+                      dest="filepath", default="/var/log/asterisk/refs",
+                      help="The full path to the refs file to process")
+
+    (options, args) = parser.parse_args(argv)
+
+    if not os.path.isfile(options.filepath):
+        print("File not found: %s" % options.filepath, file=sys.stderr)
+        return -1
+
+    try:
+        process_file(options)
+    except (KeyboardInterrupt, SystemExit, IOError):
+        print("File processing cancelled", file=sys.stderr)
+        return -1
+
+    return ret_code
+
+
+if __name__ == "__main__":
+    sys.exit(main(sys.argv))
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/sipp-sendfax.xml 1:16.1.1~dfsg-1/contrib/scripts/sipp-sendfax.xml
--- 1:13.23.1~dfsg-2/contrib/scripts/sipp-sendfax.xml	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/sipp-sendfax.xml	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,331 @@
+<?xml version="1.0" encoding="UTF-8" ?>
+<!DOCTYPE scenario SYSTEM "sipp.dtd">
+<scenario name="INVITE (optional auth), re-INVITE to T38 and send a Fax, Walter Doekes 2012-2013">
+
+  <!--
+
+  NOTE: Creating a sipp-sendfax.pcap is as easy as:
+  - receive a fax with asterisk
+  - get the incoming side of the spansdp.log (use 'fax set debug on',
+    check your logger.conf)
+  - feed it to spandspflow2pcap.py
+
+  NOTE: sipp-sendfax.xml requires image pcap play support in SIPp. This
+  means a version above 3.5.0, or the master git branch from
+  https://github.com/SIPp/sipp.
+
+  -->
+
+  <label id="invite"/>
+
+  <send retrans="500" start_txn="invite">
+    <![CDATA[
+
+      INVITE sip:[tel]@[remote_ip]:[remote_port] SIP/2.0
+      Via: SIP/2.0/[transport] [local_ip]:[local_port];branch=[branch]
+      From: sip:[service]@[local_ip]:[local_port];tag=[pid]SIPpTag00[call_number]
+      To: sip:[tel]@[remote_ip]:[remote_port]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] INVITE
+      Max-Forwards: 70
+      Content-Type: application/sdp
+      Content-Length: [len]
+
+      v=0
+      o=- 144969 144969 IN IP[local_ip_type] [local_ip]
+      s=-
+      c=IN IP[media_ip_type] [media_ip]
+      t=0 0
+      m=audio [media_port] RTP/AVP 8 0
+      a=rtpmap:8 PCMA/8000
+      a=rtpmap:0 PCMU/8000
+
+    ]]>
+  </send>
+
+  <recv response="100" optional="true" response_txn="invite"/>
+
+  <recv response="180" optional="true" response_txn="invite"/>
+
+  <recv response="181" optional="true" response_txn="invite"/>
+
+  <recv response="183" optional="true" response_txn="invite"/>
+
+  <recv response="200" optional="true" rrs="true" response_txn="invite" next="invite-ack"/>
+
+  <recv response="401" optional="true" rrs="true" next="invite-with-auth" auth="true" rrs="true" response_txn="invite"/>
+
+  <recv response="407" auth="true" rrs="true" response_txn="invite"/>
+
+  <label id="invite-with-auth"/>
+
+  <send ack_txn="invite">
+    <![CDATA[
+
+      ACK sip:[tel]@[remote_ip]:[remote_port] SIP/2.0
+      [last_Via:]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] ACK
+      Max-Forwards: 70
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+  <send retrans="500" start_txn="invite">
+    <![CDATA[
+
+      INVITE sip:[tel]@[remote_ip]:[remote_port] SIP/2.0
+      Via: SIP/2.0/[transport] [local_ip]:[local_port];branch=[branch]
+      From: sip:[service]@[local_ip]:[local_port];tag=[pid]SIPpTag00[call_number]
+      To: sip:[tel]@[remote_ip]:[remote_port]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      [authentication]
+      Call-ID: [call_id]
+      CSeq: [cseq] INVITE
+      Max-Forwards: 70
+      Content-Type: application/sdp
+      Content-Length: [len]
+
+      v=0
+      o=- 144969 144969 IN IP[local_ip_type] [local_ip]
+      s=-
+      c=IN IP[media_ip_type] [media_ip]
+      t=0 0
+      m=audio [media_port] RTP/AVP 8 0
+      a=rtpmap:8 PCMA/8000
+      a=rtpmap:0 PCMU/8000
+
+    ]]>
+  </send>
+
+  <recv response="100" response_txn="invite"/>
+
+  <recv response="180" optional="true" response_txn="invite"/>
+
+  <recv response="181" optional="true" response_txn="invite"/>
+
+  <recv response="183" optional="true" response_txn="invite"/>
+
+  <recv response="200" rrs="true" response_txn="invite"/>
+
+  <label id="invite-ack"/>
+
+  <send ack_txn="invite">
+    <![CDATA[
+
+      ACK [next_url] SIP/2.0
+      [last_Via:]
+      [routes]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] ACK
+      Max-Forwards: 70
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+  <!-- Send a bit of noise to start the RTP. You may need to use
+       the -i MY_IP command line option. -->
+  <pause milliseconds="500"/>
+
+  <nop>
+    <action>
+      <exec play_pcap_audio="g711a.pcap"/>
+    </action>
+  </nop>
+
+  <pause milliseconds="500"/>
+
+
+  <!-- *****************************************************************
+
+  Initiate re-INVITE to T38.
+
+  ****************************************************************** -->
+
+  <send start_txn="t38invite">
+    <![CDATA[
+
+      INVITE [next_url] SIP/2.0
+      Via: SIP/2.0/[transport] [local_ip]:[local_port];branch=[branch]
+      [routes]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] INVITE
+      Max-Forwards: 70
+      Content-Type: application/sdp
+      Content-Length: [len]
+
+      v=0
+      o=- 145280 145280 IN IP[local_ip_type] [local_ip]
+      s=-
+      c=IN IP[media_ip_type] [media_ip]
+      t=0 0
+      m=image [media_port] udptl t38
+      a=T38FaxVersion:0
+      a=T38MaxBitRate:14400
+      a=T38FaxRateManagement:transferredTCF
+      a=T38FaxMaxBuffer:200
+      a=T38FaxMaxDatagram:200
+      a=T38FaxUdpEC:t38UDPRedundancy
+
+    ]]>
+  </send>
+
+  <recv response="100" optional="true" response_txn="t38invite"/>
+
+  <recv response="488" optional="true" response_txn="t38invite" next="abort"/>
+
+  <recv response="200" response_txn="t38invite"/>
+
+  <send ack_txn="t38invite">
+    <![CDATA[
+
+      ACK [next_url] SIP/2.0
+      [last_Via:]
+      [routes]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] ACK
+      Max-Forwards: 70
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+  <!-- Enable the nop/action below to replay the FAX image. You may need to use
+       the -i MY_IP command line option. -->
+  <pause milliseconds="500"/>
+
+  <nop>
+    <action>
+      <exec play_pcap_image="sipp-sendfax.pcap"/>
+    </action>
+  </nop>
+
+
+  <!-- *****************************************************************
+
+  Wait for re-INVITE back to audio.
+
+  ****************************************************************** -->
+
+  <recv request="INVITE"/>
+
+  <send retrans="500">
+    <![CDATA[
+
+      SIP/2.0 200 OK
+      [last_Via:]
+      [last_Record-Route:]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      [last_Call-ID:]
+      [last_CSeq:]
+      Max-Forwards: 70
+      Content-Type: application/sdp
+      Content-Length: [len]
+
+      v=0
+      o=- 146312 146312 IN IP[local_ip_type] [local_ip]
+      s=-
+      c=IN IP[media_ip_type] [media_ip]
+      t=0 0
+      m=audio [media_port] RTP/AVP 8 0
+      a=rtpmap:8 PCMA/8000
+      a=rtpmap:0 PCMU/8000
+
+    ]]>
+  </send>
+
+  <recv request="ACK"/>
+
+  <recv request="BYE"/>
+
+  <send next="done">
+    <![CDATA[
+
+      SIP/2.0 200 OK
+      [last_Via:]
+      [last_Record-Route:]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      [last_Call-ID:]
+      [last_CSeq:]
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+
+  <!-- *****************************************************************
+
+  Abort the call ourselves
+
+  ****************************************************************** -->
+
+  <label id="abort"/>
+
+  <send ack_txn="t38invite">
+    <![CDATA[
+
+      ACK [next_url] SIP/2.0
+      [last_Via:]
+      [routes]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] ACK
+      Max-Forwards: 70
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+  <send start_txn="bye">
+    <![CDATA[
+
+      BYE [next_url] SIP/2.0
+      [last_Via:]
+      [routes]
+      [last_From:]
+      [last_To:]
+      Contact: sip:[service]@[local_ip]:[local_port]
+      Call-ID: [call_id]
+      CSeq: [cseq] BYE
+      Max-Forwards: 70
+      Content-Length: 0
+
+    ]]>
+  </send>
+
+  <recv response="200" response_txn="bye" next="done"/>
+
+
+  <!-- *****************************************************************
+
+  Finalize
+
+  ****************************************************************** -->
+
+  <label id="done"/>
+
+  <!-- Keep call open to be able to retransmit stuff -->
+  <timewait milliseconds="2000"/>
+
+</scenario><!-- vim: set ts=8 sw=2 sts=2 et ai: -->
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/sip_to_pjsip/sip_to_pjsip.py 1:16.1.1~dfsg-1/contrib/scripts/sip_to_pjsip/sip_to_pjsip.py
--- 1:13.23.1~dfsg-2/contrib/scripts/sip_to_pjsip/sip_to_pjsip.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/sip_to_pjsip/sip_to_pjsip.py	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,8 @@
 #!/usr/bin/env python
 
+from __future__ import print_function
+
+import sys
 import optparse
 import socket
 try:
@@ -10,6 +13,7 @@ import astdicts
 import astconfigparser
 
 PREFIX = 'pjsip_'
+QUIET = False
 
 ###############################################################################
 ### some utility functions
@@ -40,6 +44,11 @@ def section_by_type(section, pjsip, type
         return sect
 
 
+def ignore(key=None, val=None, section=None, pjsip=None,
+           nmapped=None, type='endpoint'):
+    """Ignore a key and mark it as mapped"""
+
+
 def set_value(key=None, val=None, section=None, pjsip=None,
               nmapped=None, type='endpoint'):
     """Sets the key to the value within the section in pjsip.conf"""
@@ -101,7 +110,7 @@ def merge_codec_value(key=None, val=None
             else:
                 merge_value(key, val, section, pjsip, nmapped, type, section_to, key_to)
         except LookupError:
-            print("lookup error")
+            print("lookup error", file=sys.stderr)
             merge_value(key, val, section, pjsip, nmapped, type, section_to, key_to)
             return
     elif key == 'disallow':
@@ -508,6 +517,7 @@ peer_map = [
     ['dtlscapath',         set_value('dtls_ca_path')],
     ['dtlssetup',          set_value('dtls_setup')],
     ['encryption_taglen',  from_encryption_taglen],
+    ['setvar',             ignore],
 
 ############################ maps to an aor ###################################
 
@@ -844,7 +854,7 @@ def create_tls(sip, pjsip, nmapped):
                   ' this was just for outbound client connections. In' \
                   ' chan_pjsip, this value is for client and server. Instead,' \
                   ' consider not to specify \'tlsclientmethod\' for chan_sip' \
-                  ' and \'method = sslv23\' for chan_pjsip.')
+                  ' and \'method = sslv23\' for chan_pjsip.', file=sys.stderr)
     except LookupError:
         """
         OpenSSL emerged during the 90s. SSLv2 and SSLv3 were the only
@@ -1117,6 +1127,19 @@ def map_registrations(sip, pjsip, nmappe
         reg.write(pjsip, nmapped)
 
 
+def map_setvars(sip, section, pjsip, nmapped):
+    """
+    Map all setvar in peer section to the appropriate endpoint set_var
+    """
+    try:
+        setvars = sip.section(section)[0].get('setvar')
+    except LookupError:
+        return
+
+    for setvar in setvars:
+        set_value('set_var', setvar, section, pjsip, nmapped)
+
+
 def map_peer(sip, section, pjsip, nmapped):
     """
     Map the options from a peer section in sip.conf into the appropriate
@@ -1218,6 +1241,7 @@ def convert(sip, filename, non_mappings,
             pass
         else:
             map_peer(sip, section, pjsip, nmapped)
+            map_setvars(sip, section, pjsip, nmapped)
 
     find_non_mapped(sip.defaults(), nmapped)
     find_non_mapped(sip.sections(), nmapped)
@@ -1247,7 +1271,7 @@ def write_pjsip(filename, pjsip, non_map
             pjsip.write(fp)
 
     except IOError:
-        print("Could not open file " + filename + " for writing")
+        print("Could not open file " + filename + " for writing", file=sys.stderr)
 
 ###############################################################################
 
@@ -1258,6 +1282,7 @@ def cli_options():
     print usage information
     """
     global PREFIX
+    global QUIET
     usage = "usage: %prog [options] [input-file [output-file]]\n\n" \
         "Converts the chan_sip configuration input-file to the chan_pjsip output-file.\n" \
         "The input-file defaults to 'sip.conf'.\n" \
@@ -1265,24 +1290,35 @@ def cli_options():
     parser = optparse.OptionParser(usage=usage)
     parser.add_option('-p', '--prefix', dest='prefix', default=PREFIX,
                       help='output prefix for include files')
+    parser.add_option('-q', '--quiet', dest='quiet', default=False, action='store_true',
+                      help="don't print messages to stdout")
 
     options, args = parser.parse_args()
     PREFIX = options.prefix
+    if options.quiet:
+        QUIET = True
 
     sip_filename = args[0] if len(args) else 'sip.conf'
     pjsip_filename = args[1] if len(args) == 2 else 'pjsip.conf'
 
     return sip_filename, pjsip_filename
 
+
+def info(msg):
+    if QUIET:
+        return
+    print(msg)
+
+
 if __name__ == "__main__":
     sip_filename, pjsip_filename = cli_options()
     # configuration parser for sip.conf
     sip = astconfigparser.MultiOrderedConfigParser()
-    print('Please, report any issue at:')
-    print('    https://issues.asterisk.org/')
-    print('Reading ' + sip_filename)
+    info('Please, report any issue at:')
+    info('    https://issues.asterisk.org/')
+    info('Reading ' + sip_filename)
     sip.read(sip_filename)
-    print('Converting to PJSIP...')
+    info('Converting to PJSIP...')
     pjsip, non_mappings = convert(sip, pjsip_filename, dict(), False)
-    print('Writing ' + pjsip_filename)
+    info('Writing ' + pjsip_filename)
     write_pjsip(pjsip_filename, pjsip, non_mappings)
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/spandspflow2pcap.log 1:16.1.1~dfsg-1/contrib/scripts/spandspflow2pcap.log
--- 1:13.23.1~dfsg-2/contrib/scripts/spandspflow2pcap.log	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/spandspflow2pcap.log	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,605 @@
+[2015-12-02 11:55:55] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     0: indicator no-signal
+[2015-12-02 11:55:55] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     1: indicator ced
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     2: IFP 00
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     2: indicator no-signal
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     3: IFP 06
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     3: indicator v21-preamble
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.30 A signal is present
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier up (-2) in state 1
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Framing OK (-6) in state 1
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_A_CED to T30_PHASE_B_RX
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 4
+[2015-12-02 11:55:58] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 0
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     4: IFP c0 01 80 00 00 ff
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     4: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     5: IFP c0 01 80 00 00 c0
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     5: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     6: IFP c0 01 80 00 00 c2
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     6: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     7: IFP c0 01 80 00 00 76
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     7: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     8: IFP c0 01 80 00 00 ee
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     8: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     9: IFP c0 01 80 00 00 f6
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx     9: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    10: IFP c0 01 80 00 00 76
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    10: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    11: IFP c0 01 80 00 00 d6
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    11: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    12: IFP c0 01 80 00 00 76
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    12: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    13: IFP c0 01 80 00 00 ae
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    13: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    14: IFP c0 01 80 00 00 04
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    14: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    15: IFP c0 01 80 00 00 04
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    15: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    16: IFP c0 01 80 00 00 04
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    16: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    17: IFP c0 01 80 00 00 04
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    17: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    18: IFP c0 01 80 00 00 04
+[2015-12-02 11:55:59] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    18: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    19: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    19: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    20: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    20: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    21: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    21: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    22: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    22: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    23: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    23: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    24: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    24: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    25: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    25: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    26: IFP c0 01 80 00 00 04
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    26: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    27: IFP c0 01 20
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    27: (0) data v21/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type TSI - CRC OK (clean)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  TSI without final frame tag
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 43 6e 77 6f 6e 6b 6e 75 20 20 20 20 20 20 20 20 20 20 20 20 20
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Remote gave TSI as: "unknown"
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    28: IFP c0 01 80 00 00 ff
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    28: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    29: IFP c0 01 80 00 00 c8
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    29: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    30: IFP c0 01 80 00 00 c1
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    30: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    31: IFP c0 01 80 00 00 00
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    31: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    32: IFP c0 01 80 00 00 46
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    32: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    33: IFP c0 01 80 00 00 1f
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    33: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    34: IFP c0 01 80 00 00 22
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    34: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    35: IFP c0 01 40
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    35: (0) data v21/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type DCS - CRC OK, sig end (clean)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  DCS with final frame tag
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 13 83 00 62 f8 44
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx final frame in state 1
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 DCS before DIS
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 DCS:
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... ...0= Store and forward Internet fax (T.37): Not set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... .0..= Real-time Internet fax (T.38): Not set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... 0...= 3G mobile network: Not set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... ..1.= Receive fax: Set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   ..10 00..= Selected data signalling rate: V.17 14400bps
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .1.. ....= R8x7.7lines/mm and/or 200x200pels/25.4mm: Set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   0... ....= 2-D coding: Not set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... ..00= Recording width: 215mm +- 1%
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... 10..= Recording length: Unlimited
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .111 ....= Minimum scan line time: 0ms
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   1... ....= Extension indicator: Set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... ..0.= Compressed/uncompressed mode: Compressed
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... .1..= Error correction mode (ECM): ECM
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .... 0...= Frame size: 256 octets
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   .1.. ....= T.6 coding: Set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30   0... ....= Extension indicator: Not set
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Selected compression T.6 (3)
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Get document at 14400bps, modem 7
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 1 to 7
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier down (-1) in state 7
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_B_RX to T30_PHASE_C_NON_ECM_RX
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 0
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 7
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 0
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    36: IFP 00
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    36: indicator no-signal
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    37: IFP 1e
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    37: indicator v17-14400-long-training
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 A signal is present
+[2015-12-02 11:56:00] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2 (55840 remaining)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    38: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    38: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Non-ECM signal status is Training succeeded (-4) in state 7
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    39: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    39: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    40: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    40: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    41: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    41: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    42: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    42: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    43: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    43: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    44: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    44: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    45: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    45: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    46: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    46: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    47: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    47: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    48: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    48: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    49: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    49: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    50: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    50: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    51: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    51: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    52: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    52: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    53: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    53: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    54: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    54: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    55: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    55: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    56: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    56: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    57: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    57: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    58: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    58: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    59: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    59: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    60: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    60: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    61: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    61: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    62: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:02] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    62: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    63: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    63: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    64: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    64: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    65: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    65: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    66: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    66: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    67: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    67: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    68: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    68: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    69: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    69: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    70: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    70: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    71: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    71: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    72: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    72: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    73: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    73: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    74: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    74: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    75: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    75: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    76: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    76: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    77: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    77: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    78: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    78: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    79: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    79: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    80: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    80: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    81: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    81: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    82: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    82: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    83: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    83: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    84: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    84: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    85: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    85: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    86: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    86: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    87: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    87: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    88: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    88: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    89: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    89: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    90: IFP d0 01 e0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    90: (0) data v17-14400/t4-non-ecm-data + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    91: IFP d0 01 f0 00 35 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    91: (0) data v17-14400/t4-non-ecm-sig-end + 54 byte(s)
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Receive complete in phase T30_PHASE_C_NON_ECM_RX, state 7
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Non-ECM signal status is Carrier down (-1) in state 7
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Trainability (TCF) test result - 23328 total bits. longest run of zeros was 23328
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_C_NON_ECM_RX to T30_PHASE_B_TX
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 0
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 4
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 7 to 8
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Tx:  CFR with final frame tag
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Tx:  ff 13 84
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    92: IFP 00
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    92: indicator no-signal
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     2: indicator no-signal
+[2015-12-02 11:56:03] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     3: indicator v21-preamble
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     4: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     4: IFP c0 01 80 00 00 ff
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     5: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     5: IFP c0 01 80 00 00 c8
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     6: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     6: IFP c0 01 80 00 00 21
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Send complete in phase T30_PHASE_B_TX, state 8
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     7: (0) data v21/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     7: IFP c0 01 40
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Send complete in phase T30_PHASE_B_TX, state 8
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 8 to 12
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_B_TX to T30_PHASE_C_ECM_RX
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 7
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 0
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:04] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     8: indicator no-signal
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    93: IFP 1c
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    93: indicator v17-14400-short-training
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 A signal is present
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2 (51520 remaining)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    94: IFP d0 01 80 00 35 ff c0 60 00 ff ff ff ff ff ff ff ff 2b 92 8a 4c c2 9d 8b 22 38 68 23 a8 22 87 0c 8e 90 60 81 21 10 c8 ea da b1 91 d5 06 e1 38 55 04 ed b6 b5 49 7b df c2 45 90 c9
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    94: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    95: IFP d0 01 80 00 35 17 45 d1 84 54 91 74 5d 18 45 2d 17 44 55 17 45 d1 74 5d 17 45 d1 74 5d 17 45 d1 74 76 24 8b a2 e8 c2 25 08 ba 2e 8b a2 e8 ba 2e 8b a2 e8 ba 2e 8b a3 b1 24 61 17
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    95: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    96: IFP d0 01 80 00 35 44 b1 17 45 d1 74 5d 17 45 d1 74 5d 17 45 d1 75 ed b2 ba 11 1c 8c 81 51 1c ce c1 51 1c 08 23 a2 2e 88 e1 9c 8e 8a 9a 08 49 98 28 71 11 11 11 11 11 06 76 a1 11 c3
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    96: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    97: IFP d0 01 80 00 35 39 1d 04 22 22 22 22 22 24 f6 a0 a1 c4 48 a3 94 06 50 e2 22 22 22 22 22 0f a4 08 a1 d0 41 46 10 51 a1 1c 47 1b c4 60 8a 1e 84 64 75 1f 77 8c 20 a2 d3 23 a4 1a 60
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    97: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    98: IFP d0 01 80 00 2b 8a 1d a0 dd 30 8a 1d f5 50 54 30 45 0e 30 45 0f 23 a0 a3 82 28 79 1d 61 06 3d db a2 3a b5 42 0e 23 50 8a 1f 11 84 50 ed 03 5a 54 11
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    98: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    99: IFP d0 01 20
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx    99: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 00 ff ff ff ff ff ff ff ff d4 49 51 32 43 b9 d1 44 1c 16 c4 15 44 e1 30 71 09 06 81 84 08 13 57 5b 8d 89 ab 60 87 1c aa 20 b7 6d ad 92 de fb 43 a2 09 93 e8 a2 8b 21 2a 89 2e ba 18 a2 b4 e8 22 aa e8 a2 8b 2e ba e8 a2 8b 2e ba e8 a2 8b 2e 6e 24 d1 45 17 43 a4 10 5d 74 d1 45 17 5d 74 d1 45 17 5d 74 d1 c5 8d 24 86 e8 22 8d e8 a2 8b 2e ba e8 a2 8b 2e ba e8 a2 8b ae b7 4d 5d 88 38 31 81 8a 38 73 83 8a 38 10 c4 45 74 11 87 39 71 51 59 10 92 19 14 8e 88 88 88 88 88 60 6e 85 88 c3 9c b8 20 44 44 44 44 44 24 6f 05 85 23 12 c5 29 60 0a 47 44 44 44 44 44 f0 25 10 85 0b 82 62 08 8a 85 38 e2 d8 23 06 51 78 21 26 ae f8 ee 31 04 45 cb c4 25 58 06 51 b8 05 bb 0c 51 b8 af 0a 2a 0c a2 70 0c a2 f0 c4 05 c5 41 14 9e b8 86 60 bc db 45 5c ad 42 70 c4 0a 51 f8 88 21 0a b7 c0 5a 2a 88
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 0, length 256
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   100: IFP d0 01 80 00 35 ff c0 60 80 43 c5 25 0a 10 61 5c 10 63 41 56 1a d9 0d 04 2f b0 8a 1d 84 50 fd be de d2 d2 2a 11 1d 2e 15 dd 42 84 50 f5 23 ae d1 74 f4 28 a5 f5 7c 34 19 02 40 af
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   100: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   101: IFP d0 01 80 00 35 0c 6c 22 87 61 14 3e 11 50 10 ac 23 08 48 e8 ba ad 42 a1 e1 42 51 0c 78 8b 74 08 10 88 df 0e 18 22 87 84 50 f8 6e 11 43 bf 4a 29 74 93 8e 96 e1 14 3f bb f7 60 bd
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   101: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   102: IFP d0 01 80 00 35 84 50 ef 1a af fe 6b 0d 30 8a 1d 63 09 48 e8 22 87 56 ea f0 c8 31 6c 22 87 21 b2 24 75 8c 83 ec a0 a1 c8 6c 89 1d 48 3e ca 88 6c 86 c8 60 c1 52 32 49 35 d8 20 45
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   102: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   103: IFP d0 01 80 00 35 5a 61 06 d5 23 e8 fa 60 83 69 1e 47 d0 45 0e 10 45 5e 8f a3 e8 f2 19 5c a9 18 45 d1 74 47 44 74 47 44 74 47 44 74 47 44 74 47 58 22 9d 97 45 d1 74 47 44 74 47 44
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   103: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   104: IFP d0 01 80 00 2b 74 5d 17 40 8a 79 74 61 02 28 70 cb a2 ea 88 e8 5e 10 6f a1 1a 23 a3 09 b6 c4 5a 16 1d 08 8c c8 c0 72 28 1c e8 1a 44 69 11 c3 0a 77
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   104: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   105: IFP d0 01 20
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   105: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 01 c2 a3 a4 50 08 86 3a 08 c6 82 6a 58 9b b0 20 f4 0d 51 b8 21 0a bf 7d 7b 4b 4b 54 88 b8 74 a8 bb 42 21 0a af c4 75 8b 2e 2f 14 a5 af 3e 2c 98 40 02 f5 30 36 44 e1 86 28 7c 88 0a 08 35 c4 10 12 17 5d b5 42 85 87 42 8a 30 1e d1 2e 10 08 11 fb 70 18 44 e1 21 0a 1f 76 88 c2 fd 52 94 2e c9 71 69 87 28 fc dd ef 06 bd 21 0a f7 58 f5 7f d6 b0 0c 51 b8 c6 90 12 17 44 e1 6a 57 0f 13 8c 36 44 e1 84 4d 24 ae 31 c1 37 05 85 13 36 91 b8 12 7c 53 11 36 61 13 06 83 4a 4c 92 ac 1b 04 a2 5a 86 60 ab c4 17 5f 06 c1 96 78 e2 0b a2 70 08 a2 7a f1 c5 17 4f 98 3a 95 18 a2 8b 2e e2 22 2e e2 22 2e e2 22 2e e2 22 2e e2 1a 44 b9 e9 a2 8b 2e e2 22 2e e2 22 2e ba e8 02 51 9e 2e 86 40 14 0e d3 45 57 11 17 7a 08 f6 85 58 c4 c5 90 6d 23 5a 68 b8 10 31 13 03 4e 14 38 17 58 22 96 88 c3 50 ee
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 1, length 256
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   106: IFP d0 01 80 00 35 ff c0 60 40 40 53 3a a2 3a 23 82 82 3a 29 03 90 60 e4 74 77 c0 a6 49 a2 38 30 47 44 59 11 c3 61 1d 12 e0 a6 8a 1d 0b 84 8b a0 45 45 54 8e 82 42 21 d2 08 a1 d2 28
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   106: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   107: IFP d0 01 80 00 35 74 b4 e1 c3 08 28 b6 d0 71 11 06 99 1d 21 a6 08 12 11 61 05 11 64 75 0c 58 20 8a 76 d0 88 45 3c 8e 9d c5 a0 8a 79 1d 3b 45 d0 b0 db ea 94 45 50 52 3a 4d 76 e2 35
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   107: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   108: IFP d0 01 80 00 35 41 18 42 d0 48 ba 1b 7c 24 5d 0d aa 42 d5 6f c1 14 3e db 6c 8e ae c2 64 74 08 a1 de 23 b8 b7 db 8b bd 58 45 0e eb f1 d0 41 91 d5 d2 40 87 1e 77 a2 23 ad 69 14 3d
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   108: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   109: IFP d0 01 80 00 35 25 c8 e9 57 1a bf 69 ec 6d 82 28 74 e9 91 d2 78 8b 7b 63 be e3 b6 df d6 81 14 3a 5b 76 b4 c2 5a 54 11 43 d9 0c 3d 68 22 87 6b 09 a5 ef bb 76 c3 4a ec 8e bf a1 78
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   109: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   110: IFP d0 01 80 00 2b 4e c3 a1 77 23 a0 8a 1e 11 43 dc 8e bd 6b 4a 92 a4 08 a1 d8 45 0f d5 48 e8 fa 23 a2 3a 08 a1 ed 17 5e 47 44 74 47 44 74 5d 29 a2 23
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   110: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   111: IFP d0 01 20
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   111: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 02 02 ca 5c 45 5c c4 41 41 5c 94 c0 09 06 27 2e ee 03 65 92 45 1c 0c e2 22 9a 88 c3 86 b8 48 07 65 51 b8 d0 21 d1 05 a2 a2 2a 71 41 42 84 4b 10 85 4b 14 2e 2d 87 c3 10 14 6d 0b 8e 88 60 99 b8 84 65 10 48 88 86 a0 88 26 ae 30 1a 04 51 6e 0b 11 a2 3c 71 b9 a3 05 51 9e b8 dc a2 0b 0d db 57 29 a2 0a 4a 5c b2 6e 47 ac 82 18 42 0b 12 5d d8 3e 24 ba b0 55 42 ab f6 83 28 7c db 36 71 75 43 26 2e 10 85 7b c4 1d ed db d1 bd 1a a2 70 d7 8f 0b 82 89 ab 4b 02 e1 78 ee 45 c4 b5 96 28 bc a4 13 97 ea 58 fd 96 37 b6 41 14 2e 97 89 4b 1e d1 de c6 7d c7 6d fb 6b 81 28 5c da 6e 2d 43 5a 2a 88 c2 9b 30 bc 16 44 e1 d6 90 a5 f7 dd 6e c3 52 37 71 fd 85 1e 72 c3 85 ee c4 05 51 78 88 c2 3b 71 bd d6 52 49 25 10 85 1b a2 f0 ab 12 17 5f c4 45 5c 10 85 b7 e8 7a e2 22 2e e2 22 2e ba 94 45 c4
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 2, length 256
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   112: IFP d0 01 80 00 35 ff c0 60 c0 a2 3a 2e 82 28 7d 97 44 74 47 44 74 5d 34 5d 1e 44 74 47 44 74 3a 19 1d 11 d1 1d 11 d1 75 b0 8a 1d d8 45 0e ed 86 1b 23 aa 23 a7 61 b7 db 42 10 4c 43
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   112: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   113: IFP d0 01 80 00 35 08 21 6c 22 9c 21 c2 28 71 11 10 cc 22 3a 68 ba 08 28 88 63 c4 44 58 91 d1 75 88 41 31 16 11 4e e2 22 21 97 44 75 48 6a 9a ae e8 70 90 4b 04 50 eb 6c 11 43 a1 12
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   113: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   114: IFP d0 01 80 00 35 3a c6 0c 71 6c 11 51 43 1a 6c 52 8c 6f ff 6d ad c3 7c 6b 23 a0 45 0e c5 ca 74 5d 0e 83 23 a2 ea c1 14 3b 08 a1 db 64 74 61 64 74 83 6e 0c 8e b9 1d 04 50 f2 3a c2
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   114: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   115: IFP d0 01 80 00 35 ee 8a 1e 47 41 14 3a a4 95 a0 aa 2d e2 3c 62 47 49 36 35 16 91 85 88 45 0e 9a b2 eb 89 c4 47 0a e2 db b4 47 41 14 3c 5b 8a 6a c8 ea ee 9f 08 a1 df 21 aa e5 6c 3d
+[2015-12-02 11:56:05] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   115: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   116: IFP d0 01 80 00 2b f1 0c 76 c2 28 7c 43 23 ae f9 ad 1d 01 70 a0 ca e2 04 0b e3 3b ad 1c 02 0d a3 68 da 36 82 28 77 08 19 56 50 e4 33 d9 56 55 95 65 6c
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   116: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   117: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   117: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 03 45 5c 74 41 14 be e9 22 2e e2 22 2e ba 2c ba 78 22 2e e2 22 2e 5c 98 b8 88 8b b8 88 8b ae 0d 51 b8 1b a2 70 b7 61 d8 c4 55 c4 e5 86 ed db 42 08 32 c2 10 84 36 44 39 84 43 14 8e 88 08 33 44 5c 16 5d 10 14 11 c6 23 22 1a 89 8b ae 11 82 8c 68 88 72 47 44 84 e9 22 ae 12 56 59 75 17 0e 09 d2 20 0a d7 36 88 c2 85 48 5c 63 30 8e 36 88 8a c2 58 36 4a 31 f6 ff b6 b5 c3 3e d6 c4 05 a2 70 a3 53 2e ba 70 c1 c4 45 57 83 28 dc 10 85 db 26 2e 86 26 2e c1 76 30 71 9d b8 20 0a 4f 5c 43 77 51 78 e2 82 28 5c 25 a9 05 55 b4 47 3c 46 e2 92 6c ac 68 89 a1 11 a2 70 59 4d d7 91 23 e2 50 47 db 2d e2 82 28 3c da 51 56 13 57 77 f9 10 85 fb 84 55 a7 36 bc 8f 30 6e 43 14 3e c2 c4 75 9f b5 b8 80 0e 05 53 47 20 d0 c7 dc b5 38 40 b0 c5 16 5b 6c 41 14 ee 10 98 6a 0a 27 cc 9b 6a aa a9 a6 36
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 3, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   118: IFP d0 01 80 00 35 ff c0 60 20 4c a1 d1 75 a7 5c 3a 5d 3f 1c 8e ae e4 74 9e 08 10 98 45 d1 74 11 4f 90 d1 46 08 10 84 54 23 68 ba 2e 8b a2 e8 ba 2e 91 50 74 ed c1 02 10 8a 74 08 10
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   118: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   119: IFP d0 01 80 00 35 88 89 74 c8 67 43 90 60 38 20 42 21 84 08 10 88 88 d0 ee fd 91 d5 f6 d9 1d 5d dd c2 28 78 84 50 f0 8a 1e 22 22 2e 91 74 8a 1d 08 d0 88 88 88 88 40 83 29 ca 20 a7
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   119: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   120: IFP d0 01 80 00 35 48 21 16 84 42 08 c2 0f 91 d0 45 42 23 a1 c4 ed 4d 05 49 42 28 7f 69 75 74 92 a6 47 51 86 39 1d 0d db 42 2f 62 22 2d 87 b7 10 8a 77 88 a2 85 ae f8 df 64 74 42 03
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   120: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   121: IFP d0 01 80 00 35 5b ee 11 43 86 47 44 18 35 ee ed d2 4d 05 54 5d 3d 06 9a a0 82 16 b1 2e 8b ac 11 43 85 e9 71 85 f5 1b 4f f8 77 76 f5 16 ec 22 87 61 14 eb 61 3e 47 42 3b 57 bf 91
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   121: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   122: IFP d0 01 80 00 2b d5 de dc 34 fb ef 91 d2 91 d1 84 92 c2 a4 87 1a 5c 8e 8d 49 3b c1 14 3a 4b d6 f4 b4 92 ad 2f b8 d8 ed ee e4 11 17 44 74 47 44 74 47
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   122: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   123: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   123: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 04 32 85 8b ae e5 3a 5c ba fc 38 71 75 27 2e 79 10 08 19 a2 8b 2e 88 f2 09 8b 62 10 08 21 2a c4 16 5d 74 d1 45 17 5d 74 89 0a 2e b7 83 40 08 51 2e 10 08 11 91 2e 13 e6 c2 09 06 1c 04 42 84 21 10 08 11 11 0b 77 bf 89 ab 6f 9b b8 ba bb 43 14 1e 21 0a 0f 51 78 44 44 74 89 2e 51 b8 10 0b 11 11 11 11 02 c1 94 53 04 e5 12 84 68 21 42 10 43 f0 89 0b a2 42 c4 85 23 b7 b2 a0 92 42 14 fe 96 ae 2e 49 65 e2 8a 61 9c b8 b0 db 42 f4 46 44 b4 e1 ed 08 51 ee 11 45 a1 75 1f fb 26 2e 42 c0 da 77 88 c2 61 e2 22 18 ac 77 b7 4b b2 a0 2a ba bc 60 59 05 41 68 8d 74 d1 35 88 c2 a1 97 8e a1 af d8 f2 1f ee 6e af 68 37 44 e1 86 28 d7 86 7c e2 42 dc ea fd 89 ab 7b 3b 2c df f7 89 4b 89 8b 21 49 43 25 e1 58 3a 71 b1 92 dc 83 28 5c d2 6b 2f 2d 49 b5 f4 1d 1b b7 77 27 88 e8 22 2e e2 22 2e e2
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 4, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   124: IFP d0 01 80 00 35 ff c0 60 a0 44 74 47 44 74 08 a1 ee 47 4d c4 19 1d 11 d1 1d 11 d1 1d 04 14 6c 86 c1 ca 72 8a ed b6 d0 45 0e dd bd b7 76 1f be e4 75 08 a1 e4 75 91 d5 17 55 45 d2
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   124: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   125: IFP d0 01 80 00 35 e0 81 31 11 11 11 11 a4 34 84 44 44 46 85 89 1d 60 8a 1c 2a c6 b4 50 f2 3a 5a 4b 7f d4 22 87 71 bb 08 a1 d8 dd 8b 76 21 43 db b0 7b 0b c1 14 fc 49 01 a4 de e1 14
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   125: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   126: IFP d0 01 80 00 35 3b 2a 03 4e f6 9c 48 61 a4 47 55 04 50 f1 a4 92 49 22 52 82 c2 49 58 64 75 82 28 74 a4 75 42 b9 02 46 11 38 66 52 b4 d1 ad 18 44 e1 97 5a 82 29 e6 a4 61 13 86 64
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   126: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   127: IFP d0 01 80 00 35 38 b9 1d 53 bb b6 10 4e ef 84 50 e3 2e b6 8b a6 c3 de 22 11 57 23 ad c8 e9 a2 3a c4 4f 24 54 07 76 1a 1a 11 45 47 69 84 50 f2 3a fe 9c 22 9d 78 86 32 3a c1 14 3c
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   127: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   128: IFP d0 01 80 00 2b 76 15 62 11 56 d6 c1 15 08 71 14 54 2e 83 09 48 e8 a8 42 8a 89 1d 42 28 77 b8 de 11 43 bb 04 50 f1 c8 ea f0 8a 71 91 d5 b0 b1 08 a1
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   128: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   129: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   129: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 05 22 2e e2 22 2e 10 85 77 e2 b2 23 98 b8 88 8b b8 88 8b b8 20 28 36 61 83 53 4e 51 b7 6d 0b a2 70 bb bd ed 6e f8 7d 27 ae 10 85 27 ae 89 ab e8 aa a2 4b 07 81 8c 88 88 88 88 25 2c 21 22 22 62 a1 91 b8 06 51 38 54 63 2d 0a 4f 5c 5a d2 fe 2b 44 e1 8e dd 10 85 1b bb d1 6e 84 c2 db 0d de d0 83 28 3f 92 80 25 7b 87 28 dc 54 c0 72 6f 39 12 86 25 e2 aa 20 0a 8f 25 49 92 44 4a 41 43 92 1a 26 ae 41 14 2e 25 ae 42 9d 40 62 88 1c 66 4a 2d 8b b5 18 22 87 e9 5a 41 94 67 25 86 c8 61 26 1c 9d b8 ca dd 6d 08 72 f7 21 0a c7 74 6d d1 65 c3 7b 44 88 ea c4 b5 13 97 45 5c 23 f2 24 2a e0 6e 58 58 88 a2 e2 96 21 0a 4f 5c 7f 39 44 b9 1e 61 4c 5c 83 28 3c 6e a8 46 88 6a 6b 83 a8 10 8e 28 2a 74 c1 90 12 17 15 42 51 91 b8 42 14 ee 1d 7b 88 c2 dd 20 0a 8f 13 57 0f 51 8e 89 ab 0d 8d 10 85
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 5, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   130: IFP d0 01 80 00 35 ff c0 60 60 f9 17 42 11 59 7c 42 2b 48 83 d6 11 47 2a 22 22 7c 33 c6 3d 2c 69 27 51 4b 1b a0 45 0e 94 cd 1b 43 e2 2f a8 9b 40 8a 1e 1f c8 e8 4d a1 57 b8 22 9e fe
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   130: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   131: IFP d0 01 80 00 35 e0 8a 7b 0d bb 6c 8e b6 9b 23 a1 1b c2 29 fb 91 d0 8b 61 14 ed bc 46 c2 29 f5 15 d5 71 50 45 0e 8c 2a a0 d9 75 82 28 74 5d 50 82 28 75 8b d6 34 34 95 84 50 e8 5e
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   131: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   132: IFP d0 01 80 00 35 df 6d b7 6a c3 1f 0d 8e 34 36 3e d8 68 8e a1 ab 61 85 64 74 38 45 3f a4 95 24 90 55 5c 11 58 c8 eb 04 50 e1 29 1d 6a 11 43 a1 14 81 14 3f 1a 10 45 0f 1f 16 f6 ed
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   132: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   133: IFP d0 01 80 00 35 bc 36 f1 0d ee d8 22 87 b8 ed 91 d0 ee d8 dd d2 f2 eb ac 22 87 55 41 15 0f 04 50 eb 1f f1 ae c2 55 7b b1 bb c2 09 b2 ea da 23 a1 08 ab 76 c2 61 af 76 a4 75 84 50
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   133: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   134: IFP d0 01 80 00 2b ed fe 96 95 22 1b 03 8a 62 b2 0b ee 9a a4 ab a7 51 08 a1 e3 4a bb 6e 1b ec 11 b4 61 3a 0d e8 c2 26 88 3e 11 43 dc 22 87 e1 14 3c 22
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   134: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   135: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   135: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 06 9f e8 42 88 9a 3e 42 d4 12 c1 6b 88 e2 54 44 44 3e cc 63 bc 34 96 e4 8a d2 d8 05 a2 70 29 b3 d8 c2 47 f4 15 d9 02 51 78 f8 13 17 b2 85 ea 1d 44 79 7f 07 51 de b0 dd 36 71 6d d9 c4 85 d8 43 94 df 89 0b d1 86 28 b7 3d 62 43 94 af a8 ab 8e 0a a2 70 31 54 05 9b ae 41 14 2e ba 0a 41 14 ae d1 6b 2c 2c a9 21 0a 17 7a fb b6 ed 56 c3 f8 b0 71 2c 6c 7c 1b 16 71 85 d5 86 a1 26 2e 1c a2 fc 25 a9 24 09 aa 3a 88 1a 13 d7 20 0a 87 94 b8 56 88 c2 85 28 81 28 fc 58 08 a2 f0 f8 68 6f b7 3d 6c 8f b0 77 1b 44 e1 1d b7 89 0b 77 1b bb 4b 4f d7 35 44 e1 aa 82 a8 f0 20 0a d7 f8 8f 75 43 aa de 8d dd 43 90 4d 57 5b c4 85 10 d5 6e 43 86 f5 6e 25 ae 21 0a b7 7f 69 a9 44 d8 c0 51 46 4d d0 77 59 25 d5 e5 8a 10 85 c7 52 dd 76 d8 37 88 2d 86 5c b0 17 43 64 11 7c 88 c2 3b 44 e1 87 28 3c 44
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 6, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   136: IFP d0 01 80 00 35 ff c0 60 e0 87 6e d4 3d ec 82 81 d5 6a a9 02 04 22 11 43 c2 28 75 a8 41 09 18 8b a2 e8 84 46 10 f6 d5 be e3 82 28 74 5d 12 11 26 82 91 d5 02 28 74 4b 10 58 45 0f
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   136: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   137: IFP d0 01 80 00 35 7d cb a6 db b1 1d b2 3a b2 3a 3a 22 e8 ba 63 38 85 84 25 d0 41 34 61 3e 47 47 44 5d 17 49 36 47 49 17 59 50 88 e8 8e 88 e8 83 41 5b 23 a6 c4 20 84 ba 2e 87 16 cb
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   137: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   138: IFP d0 01 80 00 35 a1 a3 a2 23 a2 3a 23 a0 82 78 d2 4b 0e aa b5 14 c5 21 60 c4 90 81 03 10 82 88 62 47 4f 62 29 04 50 e8 22 9d 82 28 74 38 20 4c 62 47 41 04 e9 82 28 70 81 02 68 4d
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   138: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   139: IFP d0 01 80 00 35 68 15 b1 3e 82 91 d0 20 4c 62 47 42 bc 39 1d 37 64 74 47 4b 6d 91 d7 21 88 6d 91 d1 a2 04 09 03 23 a0 8a 1c 33 08 ba 04 09 94 38 6c 31 27 41 14 3b 86 47 46 88 b9
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   139: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   140: IFP d0 01 80 00 2b 91 d3 44 74 30 c8 e9 c8 eb 65 0e 18 97 41 04 e1 b2 3a 08 27 14 19 1d 11 d1 74 10 43 c3 23 a6 84 42 0c 4b a3 a2 0b 5c 69 14 38 c4 8e
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   140: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   141: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   141: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 07 e1 76 2b bc 37 41 81 ab 56 95 40 20 44 88 c2 43 14 ae 15 82 90 18 d1 45 17 21 62 08 6f ab 7d c7 41 14 2e ba 48 88 64 41 89 ab 40 14 2e d2 08 1a a2 f0 be d3 65 db 8d b8 4d 5c 4d 5c 5c 44 17 5d c6 1c a1 21 a4 0b 82 2c 86 7c e2 e2 22 ba e8 92 6c e2 92 e8 9a 0a 11 17 71 11 17 c1 82 da c4 65 23 04 21 5d 74 e1 68 d3 85 c5 45 c4 45 5c c4 05 41 1e 4b d2 70 55 ad 28 a3 84 06 23 09 81 c0 08 41 11 46 e2 f2 46 94 20 0a 17 44 b9 41 14 2e 1c 04 32 46 e2 82 20 97 41 14 0e 81 40 16 b2 16 a8 8d 7c 41 89 0b 04 32 46 e2 42 3d 9c b8 ec 26 2e e2 d2 b6 89 eb 84 11 b6 89 8b 45 20 90 c0 c4 05 51 38 cc 10 5d 20 90 29 1c 36 8c e4 82 28 dc 61 e2 62 11 9d 89 cb 22 2e 0c 13 97 13 d7 a6 70 18 e9 82 20 87 4d 5c 10 e4 28 98 b8 88 8b 2e 08 c2 c3 c4 65 21 42 30 d2 c5 45 d0 3a 96 28 1c 23 71
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 7, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   142: IFP d0 01 80 00 35 ff c0 60 10 8b a5 49 0c 8e 8c 22 e8 ba 50 8a 1e e4 75 65 d1 74 08 13 16 2c 31 10 82 10 87 06 47 45 d0 41 0a 15 65 d1 74 08 14 5a 65 d1 74 47 45 d1 74 5d 17 45 d1
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   142: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   143: IFP d0 01 80 00 35 74 5d 26 5d 02 42 44 23 08 c2 04 50 f2 3a 2e 98 31 23 a2 3a 2e 8b a6 a3 04 09 a0 45 0e 23 48 ba 23 a2 3a 08 21 04 82 29 d9 74 08 a1 e0 8a 1c 18 91 d1 1d 11 d1 74
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   143: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   144: IFP d0 01 80 00 35 47 41 05 15 6d f1 62 23 6d 88 88 8d c2 29 dc 44 47 11 71 11 1b 11 11 71 11 11 11 11 10 c8 e8 31 16 10 88 88 89 16 65 0e 22 22 29 88 c8 e9 88 88 88 c8 e8 62 22 c6
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   144: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   145: IFP d0 01 80 00 35 22 22 22 2a d2 4a b5 82 28 74 2d 91 d6 47 48 a1 d0 45 0f 5b a5 68 ba 51 08 a1 d0 c2 09 70 d3 61 bf 76 ec 5c 76 ef bc 5a 69 dc ba 84 53 a7 23 aa a5 54 12 fa 54 ad
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   145: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   146: IFP d0 01 80 00 2b 61 14 3a 7d 20 90 c7 23 ac 6d db 0f 86 1b b7 7b fb 74 47 54 47 41 b7 a8 aa 4b 5a 54 e1 14 3a a4 50 f2 3a d1 4e 12 c2 68 42 28 74 2c
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   146: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   147: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   147: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 08 d1 a5 92 30 71 31 44 17 5d 0a 51 78 27 ae a6 8b 2e 10 c8 68 34 8c 08 41 08 e1 60 e2 a2 0b 82 50 a8 a6 8b 2e 10 28 5a a6 8b 2e e2 a2 8b 2e ba e8 a2 8b 2e ba 64 ba 40 42 22 c4 10 43 20 0a 4f 5c 74 19 8c c4 45 5c 74 d1 65 c5 20 90 05 a2 70 c4 12 5d c4 45 5c 10 84 20 41 94 9b 2e 10 85 07 51 38 18 89 8b b8 88 8b 2e e2 82 a0 a8 b6 8f 46 c4 b6 11 11 b1 43 94 3b 22 e2 88 8e 88 d8 88 88 8e 88 88 88 88 08 13 17 8c 68 08 11 11 91 68 a6 70 44 44 94 11 13 97 11 11 11 13 17 46 44 63 44 44 44 54 4b 52 ad 41 14 2e b4 89 6b e2 12 85 0b a2 f0 da a5 16 5d 8a 10 85 0b 43 90 0e cb 86 fd 6e 37 3a 6e f7 3d 5a 96 3b 5d 21 ca e5 c4 55 a5 2a 48 5f 2a b5 86 28 5c be 04 09 e3 c4 35 b6 db f0 61 d8 ed de df 2e e2 2a e2 82 ed 15 55 d2 5a 2a 87 28 5c 25 0a 4f 5c 8b 72 48 43 16 42 14 2e 34
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 8, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   148: IFP d0 01 80 00 35 ff c0 60 90 2a 87 69 c3 76 ee 47 4c 3b 10 f4 2d e2 d3 d0 45 45 da ad 3e d5 04 53 ca 12 42 8a 1d 04 53 ae b5 5d 04 35 09 14 3a 41 72 3a bb 7b 16 85 45 b2 3a 17 0d
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   148: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   149: IFP d0 01 80 00 35 b4 47 4d 38 6b 72 3a 69 a1 da 23 aa 42 8a 1d 24 ea 8a 1e 12 4d 5a 48 5d 5a 2e 95 84 28 a1 d2 4c 2a 1b 8e e1 15 09 c5 b6 88 e8 36 0e 11 50 81 84 db 86 10 bc 21 0c
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   149: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   150: IFP d0 01 80 00 35 20 c2 30 83 09 85 69 f1 0e 47 54 18 4a 3b 42 1a 2e 83 08 c2 08 a7 0b 0c 2d 14 38 46 13 09 82 10 d3 08 c2 e4 74 11 4e ad a1 b7 6a c2 29 d0 45 3a 08 21 08 a7 40 81
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   150: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   151: IFP d0 01 80 00 35 08 86 b0 8a 74 5d 62 1a 10 d1 74 11 4e 88 e9 a2 3a 08 a8 08 74 23 45 d2 41 15 01 34 08 ab 08 c2 a6 10 86 10 86 10 88 68 51 43 84 61 24 34 2d 22 9c 10 45 3a 11 06
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   151: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   152: IFP d0 01 80 00 2b 10 82 2a c2 11 45 38 45 d4 1a 06 08 5c 4c c1 c9 08 22 9c 21 45 79 06 0e 08 41 84 61 04 55 82 08 ae 20 88 28 21 53 20 9e 52 b2 bc 83
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   152: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   153: IFP d0 01 20
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   153: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 09 54 e1 96 c3 6e 77 e2 32 dc 08 2f b4 47 cb 0b a2 a2 5b b5 7c ab 20 ca 53 48 42 51 b8 20 ca 75 ad ba 20 ac 90 28 5c 82 4e 5c dd de 68 a1 a2 4d 5c e8 b0 2d e2 b2 1c d6 4e 5c 96 85 5b c4 55 42 51 b8 24 57 51 78 48 b2 5a 12 ba 5a 74 a9 21 14 85 4b 32 54 d8 71 87 a8 90 a3 6d 11 17 6c 70 88 0a 81 21 db 61 08 3d 84 30 04 43 0c c1 90 a1 96 8f 70 e2 2a 18 52 dc 42 58 74 c1 10 43 10 e5 d0 30 b4 28 1c 62 c8 90 41 08 cb 10 43 27 2e 88 72 b5 85 ed 56 43 94 0b a2 5c 10 84 10 e5 02 81 10 61 0d 51 2e ba 46 58 08 8b 2e 88 72 11 97 45 5c 10 15 10 2e c4 a2 4b 82 a8 80 2c 10 d5 10 43 65 08 61 08 61 08 11 16 8a c2 21 86 24 2c b4 44 39 08 a2 5c 88 60 08 41 54 43 88 a2 1c a2 2b 58 60 10 3a 32 83 93 10 44 39 84 a2 9e 60 70 10 82 21 86 20 aa 41 10 75 04 11 14 84 ca 04 79 4a 4d 3d c1
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 9, length 256
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   154: IFP d0 01 80 00 35 ff c0 60 50 ac 90 7d 83 08 8d 05 38 82 29 c2 12 70 c0 9f 08 46 d2 2a 64 1f 65 a5 61 08 58 43 0b d6 22 47 41 48 d2 08 aa 8a 82 a4 90 a2 e5 39 52 17 2a a2 a0 a1 32
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   154: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   155: IFP d0 01 80 00 35 ab 28 50 a4 e5 59 40 a5 56 55 94 df 8a 72 93 95 05 6c a0 ad 0a 82 84 8a 72 aa 2a 0a d9 51 41 95 05 0d 14 e5 27 2a 0a 5d 02 d8 88 88 88 88 88 88 88 88 88 88 88 88
+[2015-12-02 11:56:06] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   155: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   156: IFP d0 01 80 00 35 88 88 88 88 88 8c ae 28 8b a0 94 20 9b 23 a2 3a 7b 18 82 28 7c 11 43 86 47 45 d3 71 88 65 d0 4a 84 7a dd 05 6a 14 8e ad 0f 84 53 a8 d1 75 0c 21 86 8b a8 68 60 8a
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   156: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   157: IFP d0 01 80 00 35 b0 8b a8 84 52 61 58 9f 46 14 47 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   157: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   158: IFP d0 01 80 00 2b ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   158: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   159: IFP d0 01 20
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   159: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 0a 35 09 be c1 10 b1 a0 1c 41 94 43 48 0e 03 f9 10 62 4b 54 26 f8 a6 a5 86 10 1a c2 d0 6b 44 e2 82 12 4b 10 55 51 41 25 09 45 a7 9c 4a e8 54 45 05 85 4c d5 14 0a 25 a7 9a 02 a5 6a aa 29 fb 51 4e c9 a9 a0 36 05 b5 50 41 21 51 4e 55 54 50 9b 8a 82 a9 a0 b0 28 a7 e4 54 50 ba 40 1b 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 31 75 14 d1 05 29 04 d9 c4 45 5c de 18 41 14 3e 88 c2 61 e2 a2 cb 8e 11 a6 0b 52 21 5e bb a0 56 28 71 b5 f0 21 ca 15 8b ae 30 84 61 d1 15 16 06 51 0d d1 15 21 4a 86 1a f9 62 28 e2 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 10, length 256
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   160: IFP d0 01 80 00 35 ff c0 60 d0 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   160: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   161: IFP d0 01 80 00 35 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   161: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   162: IFP d0 01 80 00 35 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   162: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   163: IFP d0 01 80 00 35 ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff fe 00 20 03 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   163: (0) data v17-14400/hdlc-data + 54 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   164: IFP d0 01 80 00 2b 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   164: (0) data v17-14400/hdlc-data + 44 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   165: IFP d0 01 20
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   165: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type FCD - CRC OK (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  FCD without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 06 0b ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff ff 7f 00 04 c0 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Storing ECM frame 11, length 256
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   166: IFP d0 01 80 00 02 ff c0 61
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   166: (0) data v17-14400/hdlc-data + 3 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   167: IFP d0 01 20
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   167: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type RCP - CRC OK (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop none (0 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  RCP without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 86
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 12 to 13
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   168: IFP d0 01 80 00 02 ff c0 61
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   168: (0) data v17-14400/hdlc-data + 3 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   169: IFP d0 01 20
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   169: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type RCP - CRC OK (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2 (55840 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  RCP without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 86
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   170: IFP d0 01 80 00 02 ff c0 61
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   170: (0) data v17-14400/hdlc-data + 3 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   171: IFP d0 01 20
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   171: (0) data v17-14400/hdlc-fcs-OK + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type RCP - CRC OK (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2 (55840 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  RCP without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 86
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   172: IFP d0 01 80 00 02 ff c0 61
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   172: (0) data v17-14400/hdlc-data + 3 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   173: IFP d0 01 40
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   173: (0) data v17-14400/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type RCP - CRC OK, sig end (clean)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2 (55840 remaining)
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  RCP without final frame tag
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 03 86
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier down (-1) in state 13
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_C_ECM_RX to T30_PHASE_D_RX
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 4
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 0
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   174: IFP 00
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   174: indicator no-signal
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   175: IFP 06
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   175: indicator v21-preamble
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 A signal is present
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier up (-2) in state 13
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Framing OK (-6) in state 13
+[2015-12-02 11:56:07] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T2A
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   176: IFP c0 01 80 00 00 ff
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   176: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   177: IFP c0 01 80 00 00 c8
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   177: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   178: IFP c0 01 80 00 00 fd
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   178: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   179: IFP c0 01 80 00 00 f4
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   179: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   180: IFP c0 01 80 00 00 00
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   180: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   181: IFP c0 01 80 00 00 00
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   181: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   182: IFP c0 01 80 00 00 d0
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   182: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   183: IFP c0 01 40
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   183: (0) data v21/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type PPS - CRC OK, sig end (clean)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T2A (16960 remaining)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  PPS with final frame tag
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 13 bf 2f 00 00 0b
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx final frame in state 13
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Received PPS + EOP - page 0, block 0, 12 frames
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Partial page OK - committing block 0, 12 frames
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Page no = 1
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Image size = 1728 x 2292 pixels
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Image resolution = 8031/m x 7700/m
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Compression = T.6 (3)
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Compressed image size = 2998 bytes
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 13 to 14
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Tx:  MCF with final frame tag
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Tx:  ff 13 8c
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier down (-1) in state 14
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_D_RX to T30_PHASE_D_TX
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 0
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 4
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:56:08] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx     9: indicator v21-preamble
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   184: IFP 00
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   184: indicator no-signal
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    10: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    10: IFP c0 01 80 00 00 ff
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    11: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    11: IFP c0 01 80 00 00 c8
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    12: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    12: IFP c0 01 80 00 00 31
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Send complete in phase T30_PHASE_D_TX, state 14
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    13: (0) data v21/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    13: IFP c0 01 40
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Send complete in phase T30_PHASE_D_TX, state 14
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_D_TX to T30_PHASE_D_RX
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 4
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 0
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T4
+[2015-12-02 11:56:09] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Tx    14: indicator no-signal
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   185: IFP 06
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   185: indicator v21-preamble
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.30 A signal is present
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier up (-2) in state 14
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Framing OK (-6) in state 14
+[2015-12-02 11:56:10] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Start T4A
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   186: IFP c0 01 80 00 00 ff
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   186: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   187: IFP c0 01 80 00 00 c8
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   187: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   188: IFP c0 01 80 00 00 df
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   188: (0) data v21/hdlc-data + 1 byte(s)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   189: IFP c0 01 40
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38 Rx   189: (0) data v21/hdlc-fcs-OK-sig-end + 0 byte(s)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Type DCN - CRC OK, sig end (clean)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Stop T4A (15680 remaining)
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  DCN with final frame tag
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx:  ff 13 fb
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Rx final frame in state 14
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Disconnecting
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_D_RX to T30_PHASE_E
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 0
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 1
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 14 to 2
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 HDLC signal status is Carrier down (-1) in state 2
+[2015-12-02 11:56:11] FAX[13876][C-00000001] res_fax.c: FLOW T.30 No signal is present
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Send complete in phase T30_PHASE_E, state 2
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from state 2 to 32
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.30 Changing from phase T30_PHASE_E to T30_PHASE_CALL_FINISHED
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set rx type 9
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.38T Set tx type 9
+[2015-12-02 11:56:12] FAX[13876][C-00000001] res_fax.c: FLOW T.38T FAX exchange complete
diff -pruN 1:13.23.1~dfsg-2/contrib/scripts/spandspflow2pcap.py 1:16.1.1~dfsg-1/contrib/scripts/spandspflow2pcap.py
--- 1:13.23.1~dfsg-2/contrib/scripts/spandspflow2pcap.py	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/scripts/spandspflow2pcap.py	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,197 @@
+#!/usr/bin/env python
+# vim: set ts=8 sw=4 sts=4 et ai tw=79:
+'''
+Usage: ./spandspflow2pcap.py SPANDSP_LOG SENDFAX_PCAP
+
+Takes a log from Asterisk with SpanDSP, extracts the "received" data
+and puts it in a pcap file. Use 'fax set debug on' and configure
+logger.conf to get fax logs.
+
+Input data should look something like this::
+
+    [2013-08-07 15:17:34] FAX[23479] res_fax.c: FLOW T.38 Rx     5: IFP c0 01 ...
+
+Output data will look like a valid pcap file ;-)
+
+This allows you to reconstruct received faxes into replayable pcaps.
+
+Replaying is expected to be done by SIPp with sipp-sendfax.xml. The
+SIPp binary used for replaying must have image (fax) support. This means
+you'll need a version higher than 3.5.0 (unreleased when writing this),
+or the git master branch: https://github.com/SIPp/sipp
+
+
+Author: Walter Doekes, OSSO B.V. (2013,2015,2016)
+License: Public Domain
+'''
+from base64 import b16decode
+from datetime import datetime, timedelta
+from re import search
+from time import mktime
+from struct import pack
+import sys
+
+
+LOSSY = False
+EMPTY_RECOVERY = False
+
+
+def n2b(text):
+    return b16decode(text.replace(' ', '').replace('\n', '').upper())
+
+
+class FaxPcap(object):
+    PCAP_PREAMBLE = n2b('d4 c3 b2 a1 02 00 04 00'
+                        '00 00 00 00 00 00 00 00'
+                        'ff ff 00 00 71 00 00 00')
+
+    def __init__(self, outfile):
+        self.outfile = outfile
+        self.date = None
+        self.dateoff = timedelta(seconds=0)
+        self.seqno = None
+        self.udpseqno = 128
+        self.prev_data = None
+
+        # Only do this if at pos 0?
+        self.outfile.write(self.PCAP_PREAMBLE)
+
+    def data2packet(self, date, udpseqno, seqno, data, prev_data):
+        sum16 = '\x43\x21'  # checksum is irrelevant for sipp sending
+
+        new_prev = data  # without seqno..
+        data = '%s%s' % (pack('>H', seqno), data)
+        if prev_data:
+            if LOSSY and (seqno % 3) == 2:
+                return '', new_prev
+            if EMPTY_RECOVERY:
+                # struct ast_frame f[16], we have room for a few
+                # packets.
+                packets = 14
+                data += '\x00%c%s%s' % (
+                    chr(packets + 1), '\x00' * packets, prev_data)
+            else:
+                # Add 1 previous packet, without the seqno.
+                data += '\x00\x01' + prev_data
+
+        kwargs = {'udpseqno': pack('>H', udpseqno), 'sum16': sum16}
+
+        kwargs['data'] = data
+        kwargs['lenb16'] = pack('>H', len(kwargs['data']) + 8)
+        udp = '\x00\x01\x00\x02%(lenb16)s%(sum16)s%(data)s' % kwargs
+
+        kwargs['data'] = udp
+        kwargs['lenb16'] = pack('>H', len(kwargs['data']) + 20)
+        ip = ('\x45\xb8%(lenb16)s%(udpseqno)s\x00\x00\xf9\x11%(sum16)s\x01'
+              '\x01\x01\x01\x02\x02\x02\x02%(data)s') % kwargs
+
+        kwargs['data'] = ip
+        frame = ('\x00\x00\x00\x01\x00\x06\x00\x30\x48\xb1\x1c\x34\x00\x00'
+                 '\x08\x00%(data)s') % kwargs
+
+        kwargs['data'] = frame
+        sec = mktime(date.timetuple())
+        msec = date.microsecond
+        datalen = len(kwargs['data'])
+        kwargs['pre'] = pack('<IIII', sec, msec, datalen, datalen)
+        packet = '%(pre)s%(data)s' % kwargs
+
+        return (packet, new_prev)
+
+    def add(self, date, seqno, data):
+        if self.seqno is None:
+            self.seqno = 0
+            for i in range(seqno):
+                # In case the first zeroes were dropped, add them.
+                self.add(date, i, '\x00')
+        assert seqno == self.seqno, '%s != %s' % (seqno, self.seqno)
+
+        # Data is prepended by len(data).
+        data = chr(len(data)) + data
+
+        # Auto-increasing dates
+        if self.date is None or date > self.date:
+            # print 'date is larger', date, self.date
+            self.date = date
+        elif (date < self.date.replace(microsecond=0)):
+            assert False, ('We increased too fast.. decrease delta: %r/%r' %
+                           (date, self.date))
+        else:
+            self.date += timedelta(microseconds=9000)
+
+        print(seqno, '\t', self.date + self.dateoff)
+
+        # Make packet.
+        packet, prev_data = self.data2packet(self.date + self.dateoff,
+                                             self.udpseqno, self.seqno,
+                                             data, self.prev_data)
+        self.outfile.write(packet)
+
+        # Increase values.
+        self.udpseqno += 1
+        self.seqno += 1
+        self.prev_data = prev_data
+
+    def add_garbage(self, date):
+        if self.date is None or date > self.date:
+            self.date = date
+
+        packet, ignored = self.data2packet(self.date, self.udpseqno,
+                                           0xffff, 'GARBAGE', '')
+        self.udpseqno += 1
+
+        self.outfile.write(packet)
+
+
+with open(sys.argv[1], 'r') as infile:
+    with open(sys.argv[2], 'wb') as outfile:
+        first = True
+        p = FaxPcap(outfile)
+        # p.add(datetime.now(), 0, n2b('06'))
+        # p.add(datetime.now(), 1, n2b('c0 01 80 00 00 ff'))
+
+        for lineno, line in enumerate(infile):
+            # Look for lines like:
+            # [2013-08-07 15:17:34] FAX[23479] res_fax.c: \
+            #   FLOW T.38 Rx     5: IFP c0 01 80 00 00 ff
+            if 'FLOW T.38 Rx' not in line:
+                continue
+            if 'IFP' not in line:
+                continue
+
+            match = search(r'(\d{4})-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)', line)
+            assert match
+            date = datetime(*[int(i) for i in match.groups()])
+
+            match = search(r'Rx\s*(\d+):', line)
+            assert match
+            seqno = int(match.groups()[0])
+
+            match = search(r': IFP ([0-9a-f ]+)', line)
+            assert match
+            data = n2b(match.groups()[0])
+
+            # Have the file start a second early.
+            if first:
+                p.add_garbage(date)
+                first = False
+
+            # Add the packets.
+            #
+            # T.38 basic format of UDPTL payload section with redundancy:
+            #
+            # UDPTL_SEQNO
+            # - 2 sequence number (big endian)
+            # UDPTL_PRIMARY_PAYLOAD (T30?)
+            # - 1 subpacket length (excluding this byte)
+            # - 1 type of message (e.g. 0xd0 for data(?))
+            # - 1 items in data field (e.g. 0x01)
+            # - 2 length of data (big endian)
+            # - N data
+            # RECOVERY (optional)
+            # - 2 count of previous seqno packets (big endian)
+            # - N UDPTL_PRIMARY_PAYLOAD of (seqno-1)
+            # - N UDPTL_PRIMARY_PAYLOAD of (seqno-2)
+            # - ...
+            #
+            p.add(date, seqno, data)
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk-ami.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk-ami.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk-ami.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk-ami.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,10 @@
+[Unit]
+Description=Asterisk Manager Interface Socket
+
+[Socket]
+Service=asterisk.service
+ListenStream=0.0.0.0:5038
+
+[Install]
+WantedBy=sockets.target
+RequiredBy=asterisk.service
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk-amis.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk-amis.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk-amis.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk-amis.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,10 @@
+[Unit]
+Description=Asterisk Manager Interface TLS Socket
+
+[Socket]
+Service=asterisk.service
+ListenStream=0.0.0.0:5039
+
+[Install]
+WantedBy=sockets.target
+RequiredBy=asterisk.service
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk-cli.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk-cli.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk-cli.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk-cli.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,13 @@
+[Unit]
+Description=Asterisk Command-line Interface Socket
+
+[Socket]
+Service=asterisk.service
+ListenStream=/var/run/asterisk/asterisk.ctl
+SocketUser=asterisk
+SocketGroup=asterisk
+SocketMode=0660
+
+[Install]
+WantedBy=sockets.target
+RequiredBy=asterisk.service
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk-http.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk-http.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk-http.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk-http.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,11 @@
+[Unit]
+Description=Asterisk HTTP Socket
+
+[Socket]
+Service=asterisk.service
+FreeBind=true
+ListenStream=127.0.0.1:8088
+
+[Install]
+WantedBy=sockets.target
+RequiredBy=asterisk.service
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk-https.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk-https.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk-https.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk-https.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,11 @@
+[Unit]
+Description=Asterisk HTTPS Socket
+
+[Socket]
+Service=asterisk.service
+FreeBind=true
+ListenStream=127.0.0.1:8089
+
+[Install]
+WantedBy=sockets.target
+RequiredBy=asterisk.service
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk.service 1:16.1.1~dfsg-1/contrib/systemd/asterisk.service
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk.service	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk.service	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,27 @@
+[Unit]
+Description=Asterisk PBX and telephony daemon.
+After=network.target
+
+[Service]
+Type=notify
+Environment=HOME=/var/lib/asterisk
+WorkingDirectory=/var/lib/asterisk
+User=asterisk
+Group=asterisk
+ExecStart=/usr/sbin/asterisk -mqf -C /etc/asterisk/asterisk.conf
+ExecReload=/usr/sbin/asterisk -rx 'core reload'
+
+#Nice=0
+#UMask=0002
+LimitCORE=infinity
+#LimitNOFILE=
+Restart=always
+RestartSec=4
+
+# Prevent duplication of logs with color codes to /var/log/messages
+StandardOutput=null
+
+PrivateTmp=true
+
+[Install]
+WantedBy=multi-user.target
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/asterisk.socket 1:16.1.1~dfsg-1/contrib/systemd/asterisk.socket
--- 1:13.23.1~dfsg-2/contrib/systemd/asterisk.socket	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/asterisk.socket	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,26 @@
+[Unit]
+Description=Asterisk Sockets
+
+[Socket]
+FreeBind=true
+SocketUser=asterisk
+SocketGroup=asterisk
+SocketMode=0660
+
+# CLI
+ListenStream=/var/run/asterisk/asterisk.ctl
+# AMI
+ListenStream=0.0.0.0:5038
+# AMIS
+ListenStream=0.0.0.0:5039
+# HTTP
+ListenStream=127.0.0.1:8088
+# HTTPS
+ListenStream=127.0.0.1:8089
+# chan_sip TCP
+ListenStream=0.0.0.0:5060
+# chan_sip TLS
+ListenStream=0.0.0.0:5061
+
+[Install]
+WantedBy=sockets.target
diff -pruN 1:13.23.1~dfsg-2/contrib/systemd/README.txt 1:16.1.1~dfsg-1/contrib/systemd/README.txt
--- 1:13.23.1~dfsg-2/contrib/systemd/README.txt	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/contrib/systemd/README.txt	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,122 @@
+SystemD Socket Activation for Asterisk
+======================================
+
+This folder contains sample unit files which can be used as the basis of a
+socket activated Asterisk deployment.  Socket activation support currently
+extends to the following listeners:
+
+* Asterisk Command-line Interface
+* Asterisk Manager Interface (clear text and TLS)
+* Builtin HTTP / HTTPS server
+
+The primary use case of this feature is to allow Asterisk to be started by
+other services through use of AMI, CLI or REST API.
+
+The examples and documentation assume that Asterisk was linked to libsystemd
+when compiled.  This integration is required for `Type=notify` and socket
+activation to work.
+
+Security
+========
+
+Care must be take if enabling socket activation on any IP:PORT that is not
+protected by a firewall.  Any user that can reach any socket activation
+port can start Asterisk, even if they do not have valid credentials to sign
+into the service in question.  Enabling HTTP socket activation on a system
+which provides SIP over websockets would allow remote users to start Asterisk
+any time the HTTP socket is running.
+
+This functionality bypasses the normal restriction where only 'root' can start
+a service.  Enabling AMI socket activation allows any user on the local server
+to start Asterisk by running 'telnet localhost 5038'.
+
+CLI activation is secured by the combination of SocketUser, SocketGroup and
+SocketMode settings in the systemd socket.  Only local users with access will
+be able to start asterisk by using CLI.
+
+
+Separate .socket units or a single unit
+=======================================
+
+Asterisk is a complex system with many components which can be enabled or
+disabled individually.  Using socket activation requires deciding to use
+a single socket file or multiple separate socket files.
+
+The remainder of this README assumes separate socket units are used for each
+listener.
+
+
+Service and Socket files
+========================
+
+All .socket and .service examples in this folder use "reasonable" default
+paths for Linux.  Depending on your distribution and ./configure options
+you may need to modify these before installing.  The files are meant to
+be examples rather than files to be blindly installed.
+
+
+Installing and enabling socket units
+====================================
+
+Modify socket files as desired.  Install them to a location where systemd
+will find them.  pkg-config can be used to determine an appropriate location.
+
+For socket files to be managed directly by the local administrator:
+    pkg-config systemd --variable systemdsystemconfdir
+
+For socket files to be deployed by package manager:
+    pkg-config systemd --variable systemdsystemunitdir
+
+
+After installing socket files you must run 'systemctl daemon-reload' for
+systemd to read the added/modified units.  After this you can enable the
+desired sockets, for example to enable AMI:
+    systemctl enable asterisk-ami.socket
+
+
+Socket Selection
+================
+
+Asterisk configuration is unchanged by use of socket activation.  When a
+component that supports socket activation starts a listener in Asterisk,
+any sockets provided by systemd are iterated.  The systemd socket is used
+when the bound address configured by Asterisk is an exact match with the
+address given by the ListenStream setting in the systemd socket.
+
+
+Command-line Interface
+======================
+
+Symbolic links do not appear to be resolved when checking the CLI listener.
+This may be of concern since /var/run is often a symbolic link to /run. Both
+Asterisk and systemd must use /var/run, or both must use /run.  Mismatching
+will result in service startup failure.
+
+When socket activation is used for Asterisk CLI some asterisk.conf options
+are ignored.  The following options from the [files] section are ignored
+and must instead be set by the systemd socket file.
+* astctlowner - use SocketUser
+* astctlgroup - use SocketGroup
+* astctlpermissions - use SocketMode
+
+See asterisk-cli.socket for an example of these settings.
+
+
+Stopping Asterisk
+=================
+
+Some existing asterisk.service files use CLI 'core stop now' for the ExecStop
+command.  It is not recommended to use CLI to stop Asterisk on systems where
+CLI socket activation is enabled.  If Asterisk fails to start systemd still
+tries running the ExecStop command.  This can result in an loop where ExecStop
+causes CLI socket activation to start Asterisk again.  A better way to deal
+with shutdown is to use Type=notify and do not specify an ExecStop command.
+See the example asterisk.service.
+
+
+Unused Sockets
+==============
+
+Asterisk makes no attempt to check for sockets provided by systemd that are not
+used.  It is the users responsibility to only provide sockets which Asterisk is
+configured to use.
diff -pruN 1:13.23.1~dfsg-2/CREDITS 1:16.1.1~dfsg-1/CREDITS
--- 1:13.23.1~dfsg-2/CREDITS	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/CREDITS	2018-12-26 18:45:13.000000000 +0000
@@ -154,6 +154,7 @@
 		(dialog-info), QUEUE_EXISTS function, device state provider
 		architecture, multiparking (together with mvanbaak), meetme and
 		parking device states, MiniVM - the small voicemail system,
+		RTP improvements, RTCP enhancements, DTMF timing fixes,
 		many documentation updates/corrections, and many bug fixes.
 		oej(AT)edvina.net, http://edvina.net
 
diff -pruN 1:13.23.1~dfsg-2/debian/ast_config/modules.conf 1:16.1.1~dfsg-1/debian/ast_config/modules.conf
--- 1:13.23.1~dfsg-2/debian/ast_config/modules.conf	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/ast_config/modules.conf	2019-01-11 17:51:43.000000000 +0000
@@ -47,11 +47,12 @@ noload => chan_capi.so
 ;
 load => res_musiconhold.so
 ;
-; Load either OSS or ALSA, not both
-; By default, load OSS only (automatically) and do not load ALSA
+; Do not load load local channel drivers (using the system speaker) by default,
+; they are not used in most installations and might block the sound hardware
 ;
 noload => chan_alsa.so
-;noload => chan_oss.so
+noload => chan_console.so
+noload => chan_oss.so
 ;
 ; Disable CDR logging to SQLite by default since it writes unconditionally to 
 ; cdr.db without a way to rotate it.
diff -pruN 1:13.23.1~dfsg-2/debian/asterisk.init 1:16.1.1~dfsg-1/debian/asterisk.init
--- 1:13.23.1~dfsg-2/debian/asterisk.init	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/asterisk.init	2019-01-11 17:51:43.000000000 +0000
@@ -118,6 +118,10 @@ status() {
 	status_of_proc -p "$PIDFILE" "$NAME" "$DESC" && return 0 || return $?
 }
 
+safe_status() {
+	status_of_proc -p "$ASTSAFE_PIDFILE" "$NAME" "$DESC" && return 0 || return $?
+}
+
 asterisk_rx() {
 	if ! status >/dev/null; then return 0; fi
 
@@ -139,46 +143,53 @@ case "$1" in
 	exit 0
 	;;
   start)
-	if status > /dev/null; then
-		echo "$DESC is already running. Use restart."
-		exit 0
-	fi
-	echo -n "Starting $DESC: "
-	if [ "$RUNASTSAFE" != "yes" ];then
-		# TODO: what if we cought the wrapper just as its asterisk
-		# was killed? status should check for the wrapper if we're in
-		# "safe mode"
-		if status > /dev/null; then
+	if [ "$RUNASTSAFE" = "yes" ];then
+		if safe_status >/dev/null; then
 			echo "$DESC is already running. Use restart."
 			exit 0
 		fi
-		start-stop-daemon --start --group $GROUP --pidfile "$PIDFILE" \
-			$CHDIR_PARM \
-			--exec $REALDAEMON -- $PARAMS > /dev/null
-	else
+		echo -n "Starting $DESC: "
 		export ASTSAFE_FOREGROUND=1
 		start-stop-daemon --start --group $GROUP \
 			--background --make-pidfile \
 			$CHDIR_PARM --pidfile "$ASTSAFE_PIDFILE" \
 			--exec $REALDAEMON -- $PARAMS
+	else
+		if status >/dev/null; then
+			echo "$DESC is already running. Use restart."
+			exit 0
+		fi
+		echo -n "Starting $DESC: "
+		start-stop-daemon --start --group $GROUP --pidfile "$PIDFILE" \
+			$CHDIR_PARM \
+			--exec $REALDAEMON -- $PARAMS > /dev/null
 	fi
-		
-	
 	echo "$NAME."
 	;;
   stop)
 	echo -n "Stopping $DESC: $NAME"
 	# Try gracefully.
-	# this may hang in some cases. Specifically, when the asterisk
+	# This may hang in some cases. Specifically, when the asterisk
 	# processes is stopped. No bother to worry about cleanup: 
 	# it will either fail or die when asterisk dies.
 	( asterisk_rx 'core stop now' > /dev/null 2>&1 & ) &
 	if [ "$RUNASTSAFE" = "yes" ];then
+		# If you're switching back and forth between RUNASTSAFE
+		# you may get a warning about a stale pidfile. Ignore
+		# it.
 		start-stop-daemon --stop --quiet --oknodo \
 				  --pidfile $ASTSAFE_PIDFILE
 		rm -f $ASTSAFE_PIDFILE
 	fi
-	start-stop-daemon --stop --quiet --oknodo --exec $DAEMON --pidfile=$PIDFILE
+	# Sometimes during a quick restart cycle, the 'core stop now'
+	# from above won't reach the daemon -- perhaps it wasn't
+	# listening yet. At this point we want TERM to kick in.
+	# In any case, we must be certain that it is stopped before we
+	# exit the "stop" case; otherwise a "restart" might complete
+	# with no asterisk running at all (because of the status checks
+	# in "start").
+	start-stop-daemon --stop --quiet --oknodo --retry=8/TERM/30/KILL/5 \
+		--exec $DAEMON --pidfile=$PIDFILE
 	echo "."
 	;;
   reload)
diff -pruN 1:13.23.1~dfsg-2/debian/asterisk.install 1:16.1.1~dfsg-1/debian/asterisk.install
--- 1:13.23.1~dfsg-2/debian/asterisk.install	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/asterisk.install	2019-01-11 17:51:43.000000000 +0000
@@ -1,5 +1,6 @@
 usr/sbin
 usr/bin/asterisk-config-custom
+usr/lib/libasterisk*
 usr/share/asterisk/firmware
 usr/share/asterisk/keys
 usr/share/asterisk/conf
diff -pruN 1:13.23.1~dfsg-2/debian/asterisk.lintian-overrides 1:16.1.1~dfsg-1/debian/asterisk.lintian-overrides
--- 1:13.23.1~dfsg-2/debian/asterisk.lintian-overrides	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/asterisk.lintian-overrides	2019-01-11 17:51:43.000000000 +0000
@@ -1,3 +1,6 @@
 asterisk: package-contains-empty-directory
 # astcanary is not supposed to be invoked manually
 asterisk: binary-without-manpage usr/sbin/astcanary
+# private copy of pjproject
+asterisk: package-name-doesnt-match-sonames libasteriskpj2
+asterisk: non-dev-pkg-with-shlib-symlink usr/lib/libasteriskpj.so.2 usr/lib/libasteriskpj.so
diff -pruN 1:13.23.1~dfsg-2/debian/changelog 1:16.1.1~dfsg-1/debian/changelog
--- 1:13.23.1~dfsg-2/debian/changelog	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/changelog	2019-01-11 17:51:43.000000000 +0000
@@ -1,3 +1,28 @@
+asterisk (1:16.1.1~dfsg-1) unstable; urgency=medium
+
+  Upload new major version to unstable
+
+  [ Bernhard Schmidt ]
+  * New upstream version 16.1.1 (Closes: #886984, #917481)
+    - build with embedded pjproject 2.8 (dfsg-repacked)
+    - Add lintian overrides for bundled library
+    - Reenable app_macro, many dialplans need it
+  * Update d/watch for Asterisk 16.x
+  * Add signing key for Chris Savinovich <csavinovich@digium.com>
+  * New upstream version 16.1.0~dfsg
+  * README.Debian: Fix a typo found by lintian
+  * Drop libsqlite0-dev, deprecated
+  * Do not load any local channel drivers by default (Closes: #821392)
+  * asterisk.service: Attempt to run with realtime priority by default
+    (Closes: #801629)
+  * Improve/fix some raceconditions in sysv-initscript.
+    Thanks to Walter Doekes (Closes: #778746)
+
+  [ Rob Thomas ]
+  * Build-Depend on libunbound-dev for async DNS
+
+ -- Bernhard Schmidt <berni@debian.org>  Fri, 11 Jan 2019 18:51:43 +0100
+
 asterisk (1:13.23.1~dfsg-2) unstable; urgency=medium
 
   * Fix autopkgtest by parsing XML results (Closes: #909689)
diff -pruN 1:13.23.1~dfsg-2/debian/control 1:16.1.1~dfsg-1/debian/control
--- 1:13.23.1~dfsg-2/debian/control	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/control	2019-01-11 17:51:43.000000000 +0000
@@ -43,7 +43,6 @@ Build-Depends:
  libopenr2-dev [linux-any],
  libopus-dev,
  libopusfile-dev,
- libpjproject-dev (>= 2.7~),
  libpopt-dev,
  libpq-dev,
  libpri-dev,
@@ -54,7 +53,6 @@ Build-Depends:
  libspandsp-dev,
  libspeex-dev,
  libspeexdsp-dev,
- libsqlite0-dev,
  libsqlite3-dev,
  libsrtp2-dev,
  libss7-dev,
@@ -62,6 +60,7 @@ Build-Depends:
  libsystemd-dev [linux-any],
  libswscale-dev,
  libtonezone-dev [linux-any],
+ libunbound-dev,
  liburiparser-dev,
  libvorbis-dev,
  libvpb-dev [linux-any],
diff -pruN 1:13.23.1~dfsg-2/debian/copyright.pjproject 1:16.1.1~dfsg-1/debian/copyright.pjproject
--- 1:13.23.1~dfsg-2/debian/copyright.pjproject	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/copyright.pjproject	2019-01-11 17:51:43.000000000 +0000
@@ -0,0 +1,334 @@
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: PJPROJECT
+Source: https://github.com/asterisk/pjproject
+Comment: Currently packaging the Asterisk fork of PJProject, as it supports
+ shared libraries.
+ .
+ Upstream source repackaged to address DFSG concerns and other issues:
+  * Copyright-protected code without DFSG-free license
+    (g7221 ilbc milenage s60_g729_bitstream.h BaseClasses ApiLock.*)
+  * Copyright-protected graphics ICC profiles without DFSG-free license
+    (*.png)
+  * Binary Java files potentially with non-free parts
+    (*.jar)
+  * Convenience code copies
+    (gsm resample speex srtp)
+Files-Excluded:
+ third_party/g7221
+ third_party/gsm
+ third_party/ilbc
+ third_party/milenage
+ third_party/resample
+ third_party/speex
+ third_party/srtp
+ third_party/yuv
+ third_party/webrtc
+ third_party/BaseClasses
+ pjmedia/src/pjmedia-audiodev/s60_g729_bitstream.h
+ pjsip-apps/src/pjsua/bb10/assets/images/teluu-logo.png
+ pjsip-apps/src/pjsua/android/gradle/wrapper/gradle-wrapper.jar
+ pjsip-apps/src/pjsua/ios/ipjsua/Default@2x.png
+ pjsip-apps/src/pjsua/ios/ipjsua/Default-568h@2x.png
+ pjsip-apps/src/pjsua/ios/ipjsua/Default.png
+ pjsip-apps/src/pjsua/winrt/cli/uwp/Assets/teluu-logo.png
+ pjsip-apps/src/pjsua/winrt/cli/wp8/Assets/teluu-logo.png
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipBackEnd/ApiLock.cpp
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipBackEnd/ApiLock.h
+ pjsip-apps/src/swig/java/android/gradle/wrapper/gradle-wrapper.jar
+
+Files: *
+Copyright:
+  1993-2006, Douglas C. Schmidt <d.schmidt@vanderbilt.edu>
+  2003-2008, Benny Prijono <benny@prijono.org>
+  2007-2009, Keystream AB
+  2007-2009, Konftel AB
+  2008-2016, Teluu Inc. <http://www.teluu.com/>
+  2010-2012, Regis Montoya (r3gis) <http://www.r3gis.fr/>
+  2011 Dan Arrhenius <dan@keystream.se>
+License-Grant:
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the
+ Free Software Foundation; either version 2 of the License, or (at your
+ option) any later version.
+License: GPL-2+
+Comment:
+ Some files has additional alternative licensing ignored as irrelevant
+ for our redistribution.
+
+Files: pjsip-apps/src/pjsua/winrt/gui/uwp/Voip/Helpers/AppServiceHelper.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/Voip/Helpers/VoipCallHelper.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/Voip/MainPage.xaml.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipBackEnd/pch.cpp
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipBackEnd/pch.h
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipHost/VoipHost.cpp
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipTasks/AppService.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipTasks/BackgroundOperations/BackgroundOperations.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipTasks/CallRtcTask.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipTasks/Helpers/Current.cs
+ pjsip-apps/src/pjsua/winrt/gui/uwp/VoipTasks/Helpers/VccCallHelper.cs
+Copyright: Microsoft
+License-Grant:
+ This code is licensed under the MIT License (MIT).
+License: Expat
+Comment:
+ Assuming the Expat flavor of the MIT family of licenses.
+
+Files: pjlib/src/pj/compat/longjmp_i386.S
+ pjlib-util/src/pjlib-util/getopt.c
+Copyright: Mike Borella <mike_borella@mw.3com.com>
+  1987-1998, Free Software Foundation, Inc.
+License-Grant:
+ The GNU C Library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Library General Public License as
+ published by the Free Software Foundation; either version 2 of the
+ License, or (at your option) any later version.
+License: LGPL-2+
+Comment:
+ pjlib-util/include/pjlib-util/getopt.h has the same license boilerplate,
+ only it has a comment "this file has now become GPL" on top (comment
+ removed by Upstream).
+
+Files: third_party/threademulation/include/ThreadEmulation.h
+ third_party/threademulation/src/ThreadEmulation.cpp
+Copyright: Microsoft Corporation
+License: Ms-PL
+Comment:
+ Code lack licensing (contains only a standard disclaimer). File
+ "third_party/threademulation/Microsoft Permissive License.txt" is
+ assumed to apply.
+
+Files: pjsip-apps/src/pjsua/bb10/bar-descriptor.xml
+Copyright: 2011-2013, Research In Motion Limited
+License-Grant:
+ Licensed under the Apache License, Version 2.0 (the "License"); you may
+ not use this file except in compliance with the License. You may obtain
+ a copy of the License at <http://www.apache.org/licenses/LICENSE-2.0>
+License: Apache-2
+
+Files: pjlib/src/pj/sock_linux_kernel.c
+Copyright: 1983, 1993, The Regents of the University of California.
+  2003-2008, Benny Prijono <benny@prijono.org>
+  2008-2011, Teluu Inc. <http://www.teluu.com/>
+License-Grant:
+ This program is free software; you can redistribute it and/or modify it
+ under the terms of the GNU General Public License as published by the
+ Free Software Foundation; either version 2 of the License, or (at your
+ option) any later version.
+License: BSD-4-clause and GPL-2+
+
+Files: aconfigure
+Copyright: 1992-1996, 1998-2012, Free Software Foundation, Inc.
+License: FSFUL
+
+Files: pjlib/src/pj/compat/setjmp_i386.S
+Copyright: 1995-1997, 2000-2001, Free Software Foundation, Inc.
+License-Grant:
+ The GNU C Library is free software; you can redistribute it and/or
+ modify it under the terms of the GNU Lesser General Public License as
+ published by the Free Software Foundation; either version 2.1 of the
+ License, or (at your option) any later version.
+License: LGPL-2.1+
+
+Files: third_party/mp3/BladeMP3EncDLL.h
+Copyright: 1999-2002 A.L. Faber
+License-Grant:
+ This library is free software; you can redistribute it and/or modify it
+ under the terms of the GNU Lesser General Public License as published
+ by the Free Software Foundation; either version 2 of the License, or
+ (at your option) any later version.
+License: LGPL-2.1+
+
+Files: install-sh
+Copyright: 1991, the Massachusetts Institute of Technology
+License: NTP
+
+Files: pjmedia/src/pjmedia/alaw_ulaw.c
+Copyright: Sun Microsystems, Inc.
+License: Rdisc~code
+
+Files: pjlib-util/src/pjlib-util/crc32.c
+Copyright: None
+License: public-domain
+ This file is partly taken from Crypto++ library
+ (<http://www.cryptopp.com/>) and
+ <http://www.di-mgt.com.au/crypto.html#CRC>.
+ .
+ Since the original version of the code is put in public domain, this
+ file is put on public domain as well.
+
+Files: pjlib-util/src/pjlib-util/md5.c
+Copyright: None
+License: public-domain
+ This code is in the public domain; do with it what you wish.
+
+Files: pjlib-util/src/pjlib-util/sha1.c
+Copyright: None
+License: public-domain
+ Still 100% public domain
+
+Files: debian/*
+Copyright: 2013, Tzafrir Cohen <tzafrir.cohen@xorcom.com>
+License: GPL-2+
+
+Files: debian/copyright-check
+Copyright: 2016-2017, Jonas Smedegaard <dr@jones.dk>
+License: GPL-3+
+
+License: Apache-2
+License-Reference: /usr/share/common-licenses/Apache-2.0
+
+License: Expat
+ Permission is hereby granted, free of charge, to any person obtaining a
+ copy of this software and associated documentation files (the
+ "Software"), to deal in the Software without restriction, including
+ without limitation the rights to use, copy, modify, merge, publish,
+ distribute, sublicense, and/or sell copies of the Software, and to
+ permit persons to whom the Software is furnished to do so, subject to
+ the following conditions:
+ .
+ The above copyright notice and this permission notice shall be included
+ in all copies or substantial portions of the Software.
+ .
+ THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+ IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
+ CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
+ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+License: FSFUL
+ This configure script is free software; the Free Software Foundation
+ gives unlimited permission to copy, distribute and modify it.
+
+License: GPL-2+
+License-Reference: /usr/share/common-licenses/GPL-2
+
+License: GPL-3+
+License-Reference: /usr/share/common-licenses/GPL-3
+
+License: LGPL-2+
+License-Reference: /usr/share/common-licenses/LGPL-2
+
+License: LGPL-2.1+
+License-Reference: /usr/share/common-licenses/LGPL-2.1
+
+License: Ms-PL
+ Microsoft Permissive License (Ms-PL)
+ .
+ This license governs use of the accompanying software. If you use the
+ software, you accept this license. If you do not accept the license, do
+ not use the software.
+ .
+  1. Definitions
+     The terms 'reproduce,' 'reproduction,' 'derivative works,' and
+     'distribution' have the same meaning here as under U.S. copyright
+     law.
+     A 'contribution' is the original software, or any additions or
+     changes to the software.
+     A 'contributor' is any person that distributes its contribution
+     under this license.
+     'Licensed patents' are a contributor's patent claims that read
+     directly on its contribution.
+ .
+ 2. Grant of Rights
+    (A) Copyright Grant- Subject to the terms of this license, including
+        the license conditions and limitations in section 3, each
+        contributor grants you a non-exclusive, worldwide, royalty-free
+        copyright license to reproduce its contribution, prepare
+        derivative works of its contribution, and distribute its
+        contribution or any derivative works that you create.
+    (B) Patent Grant- Subject to the terms of this license, including
+        the license conditions and limitations in section 3, each
+        contributor grants you a non-exclusive, worldwide, royalty-free
+        license under its licensed patents to make, have made, use,
+        sell, offer for sale, import, and/or otherwise dispose of its
+        contribution in the software or derivative works of the
+        contribution in the software.
+ .
+ 3. Conditions and Limitations
+    (A) No Trademark License- This license does not grant you rights to
+        use any contributors' name, logo, or trademarks.
+    (B) If you bring a patent claim against any contributor over patents
+        that you claim are infringed by the software, your patent
+        license from such contributor to the software ends
+        automatically.
+    (C) If you distribute any portion of the software, you must retain
+        all copyright, patent, trademark, and attribution notices that
+        are present in the software.
+    (D) If you distribute any portion of the software in source code
+        form, you may do so only under this license by including a
+        complete copy of this license with your distribution. If you
+        distribute any portion of the software in compiled or object
+        code form, you may only do so under a license that complies with
+        this license.
+    (E) The software is licensed "as-is." You bear the risk of using it.
+        The contributors give no express warranties, guarantees or
+        conditions. You may have additional consumer rights under your
+        local laws which this license cannot change. To the extent
+        permitted under your local laws, the contributors exclude the
+        implied warranties of merchantability, fitness for a particular
+        purpose and non-infringement.
+
+License: NTP
+ Permission to use, copy, modify, distribute, and sell this software and
+ its documentation for any purpose is hereby granted without fee,
+ provided that the above copyright notice appear in all copies and that
+ both that copyright notice and this permission notice appear in
+ supporting documentation, and that the name of M.I.T. not be used in
+ advertising or publicity pertaining to distribution of the software
+ without specific, written prior permission.  M.I.T. makes no
+ representations about the suitability of this software for any purpose.
+ It is provided "as is" without express or implied warranty.
+
+License: BSD-4-clause
+ Redistribution and use in source and binary forms, with or without
+ modification, are permitted provided that the following conditions are
+ met:
+  1. Redistributions of source code must retain the above copyright
+     notice, this list of conditions and the following disclaimer.
+  2. Redistributions in binary form must reproduce the above copyright
+     notice, this list of conditions and the following disclaimer in the
+     documentation and/or other materials provided with the
+     distribution.
+  3. All advertising materials mentioning features or use of this
+     software must display the following acknowledgement: This product
+     includes software developed by the University of California,
+     Berkeley and its contributors.
+  4. Neither the name of the University nor the names of its
+     contributors may be used to endorse or promote products derived
+     from this software without specific prior written permission.
+ .
+ THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
+ BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+License: Rdisc~code
+ This source code is a product of Sun Microsystems, Inc. and is provided
+ for unrestricted use.  Users may copy or modify this source code
+ without charge.
+ .
+ SUN SOURCE CODE IS PROVIDED AS IS WITH NO WARRANTIES OF ANY KIND
+ INCLUDING THE WARRANTIES OF DESIGN, MERCHANTIBILITY AND FITNESS FOR A
+ PARTICULAR PURPOSE, OR ARISING FROM A COURSE OF DEALING, USAGE OR TRADE
+ PRACTICE.
+ .
+ Sun source code is provided with no support and without any obligation
+ on the part of Sun Microsystems, Inc. to assist in its use, correction,
+ modification or enhancement.
+ .
+ SUN MICROSYSTEMS, INC. SHALL HAVE NO LIABILITY WITH RESPECT TO THE
+ INFRINGEMENT OF COPYRIGHTS, TRADE SECRETS OR ANY PATENTS BY THIS
+ SOFTWARE OR ANY PART THEREOF.
+ .
+ In no event will Sun Microsystems, Inc. be liable for any lost revenue
+ or profits or other special, indirect and consequential damages, even
+ if Sun has been advised of the possibility of such damages.
diff -pruN 1:13.23.1~dfsg-2/debian/patches/aelparse_enable.patch 1:16.1.1~dfsg-1/debian/patches/aelparse_enable.patch
--- 1:13.23.1~dfsg-2/debian/patches/aelparse_enable.patch	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/aelparse_enable.patch	2019-01-11 17:51:43.000000000 +0000
@@ -2,7 +2,7 @@ Description: Enable the aelparse applica
 Author: Jeremy Lainé <jeremy.laine@m4x.org>
 Forwarded: no
 Bug-Debian: https://bugs.debian.org/747866
-Last-Update: 2016-03-19
+Last-Update: 2018-11-17
 
 --- a/utils/utils.xml
 +++ b/utils/utils.xml
@@ -11,6 +11,6 @@ Last-Update: 2016-03-19
    <member name="aelparse">
 -	<defaultenabled>no</defaultenabled>
 +	<defaultenabled>yes</defaultenabled>
- 	<support_level>extended</support_level>
+ 	<support_level>deprecated</support_level>
    </member>
    <member name="astcanary">
diff -pruN 1:13.23.1~dfsg-2/debian/patches/amr.patch 1:16.1.1~dfsg-1/debian/patches/amr.patch
--- 1:13.23.1~dfsg-2/debian/patches/amr.patch	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/amr.patch	2019-01-11 17:51:43.000000000 +0000
@@ -59,8 +59,8 @@ Last-Update: 2016-08-29
 +AMR_WB_DECODER=@PBX_AMR_WB_DECODER@
 +AMR_WB_ENCODER=@PBX_AMR_WB_ENCODER@
  BLUETOOTH=@PBX_BLUETOOTH@
+ BEANSTALK=@PBX_BEANSTALK@
  COROSYNC=@PBX_COROSYNC@
- CRYPTO=@PBX_CRYPTO@
 --- /dev/null
 +++ b/codecs/codec_amr.c
 @@ -0,0 +1,405 @@
@@ -523,7 +523,7 @@ Last-Update: 2016-08-29
 +}
 --- a/configure.ac
 +++ b/configure.ac
-@@ -448,6 +448,9 @@
+@@ -456,6 +456,9 @@
  # to make things easier for the users.
  
  AST_EXT_LIB_SETUP([ALSA], [Advanced Linux Sound Architecture], [asound])
@@ -533,7 +533,7 @@ Last-Update: 2016-08-29
  AST_EXT_LIB_SETUP([BFD], [Debug symbol decoding], [bfd])
  
  # BKTR is used for backtrace support on platforms that do not
-@@ -1487,6 +1490,10 @@
+@@ -1500,6 +1503,10 @@
  
  AST_EXT_LIB_CHECK([ALSA], [asound], [snd_pcm_open], [alsa/asoundlib.h])
  
@@ -568,7 +568,7 @@ Last-Update: 2016-08-29
 +#endif /* _AST_FORMAT_AMR_H */
 --- a/include/asterisk/format_cache.h
 +++ b/include/asterisk/format_cache.h
-@@ -237,6 +237,16 @@
+@@ -247,6 +247,16 @@
  extern struct ast_format *ast_format_silk24;
  
  /*!
@@ -587,7 +587,7 @@ Last-Update: 2016-08-29
   * \retval 0 success
 --- a/main/codec_builtin.c
 +++ b/main/codec_builtin.c
-@@ -861,6 +861,54 @@
+@@ -904,6 +904,54 @@
  	.samples_count = silk_samples
  };
  
@@ -642,19 +642,19 @@ Last-Update: 2016-08-29
  #define CODEC_REGISTER_AND_CACHE(codec) \
  	({ \
  		int __res_ ## __LINE__ = 0; \
-@@ -893,6 +941,8 @@
+@@ -936,6 +984,8 @@
  {
  	int res = 0;
  
 +	res |= CODEC_REGISTER_AND_CACHE(amr);
 +	res |= CODEC_REGISTER_AND_CACHE(amrwb);
+ 	res |= CODEC_REGISTER_AND_CACHE(codec2);
  	res |= CODEC_REGISTER_AND_CACHE(g723);
  	res |= CODEC_REGISTER_AND_CACHE(ulaw);
- 	res |= CODEC_REGISTER_AND_CACHE(alaw);
 --- a/main/format_cache.c
 +++ b/main/format_cache.c
-@@ -223,6 +223,16 @@
- struct ast_format *ast_format_opus;
+@@ -226,6 +226,16 @@
+ struct ast_format *ast_format_codec2;
  
  /*!
 + * \brief Built-in cached AMR format.
@@ -670,7 +670,7 @@ Last-Update: 2016-08-29
   * \brief Built-in cached t140 format.
   */
  struct ast_format *ast_format_t140;
-@@ -305,6 +315,8 @@
+@@ -313,6 +323,8 @@
  	ao2_cleanup(formats);
  	formats = NULL;
  
@@ -679,7 +679,7 @@ Last-Update: 2016-08-29
  	ao2_replace(ast_format_g723, NULL);
  	ao2_replace(ast_format_ulaw, NULL);
  	ao2_replace(ast_format_alaw, NULL);
-@@ -422,6 +434,10 @@
+@@ -434,6 +446,10 @@
  		ao2_replace(ast_format_g719, format);
  	} else if (!strcmp(name, "opus")) {
  		ao2_replace(ast_format_opus, format);
@@ -692,7 +692,7 @@ Last-Update: 2016-08-29
  	} else if (!strcmp(name, "png")) {
 --- a/main/rtp_engine.c
 +++ b/main/rtp_engine.c
-@@ -2732,6 +2732,9 @@
+@@ -3597,6 +3597,9 @@
  	set_next_mime_type(ast_format_vp8, 0,  "video", "VP8", 90000);
  	set_next_mime_type(ast_format_vp9, 0, "video", "VP9", 90000);
  
@@ -702,9 +702,9 @@ Last-Update: 2016-08-29
  	/* Define the static rtp payload mappings */
  	add_static_payload(0, ast_format_ulaw, 0);
  	#ifdef USE_DEPRECATED_G726
-@@ -2778,6 +2781,9 @@
- 	add_static_payload(100, ast_format_vp8, 0);
- 	add_static_payload(107, ast_format_opus, 0);
+@@ -3657,6 +3660,9 @@
+ 	add_static_payload(127, ast_format_slin96, 0);
+ 	/* payload types above 127 are not valid */
  
 +	add_static_payload(108, ast_format_amr, 0);
 +	add_static_payload(109, ast_format_amrwb, 0);
diff -pruN 1:13.23.1~dfsg-2/debian/patches/autoreconf-pjproject 1:16.1.1~dfsg-1/debian/patches/autoreconf-pjproject
--- 1:13.23.1~dfsg-2/debian/patches/autoreconf-pjproject	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/autoreconf-pjproject	2019-01-11 17:51:43.000000000 +0000
@@ -0,0 +1,22 @@
+Description: Update autoconf files for pjproject
+ config.guess and config.sub for pjproject are six years old, this
+ makes the build FTBFS on newer architectures like ppc64el.
+ .
+ Unfortunately the sources are only unpacked during the toplevel
+ ./configure run, so we cannot solve this with dh_autoreconf
+Author: Bernhard Schmidt <berni@debian.org>
+Last-Update: 2018-12-21
+--- a/third-party/pjproject/Makefile
++++ b/third-party/pjproject/Makefile
+@@ -100,6 +100,11 @@
+ 	@mv pjproject-$(PJPROJECT_VERSION) source
+ 	$(ECHO_PREFIX) Applying patches "$(realpath patches)" "$(realpath .)/source"
+ 	$(CMD_PREFIX) ../apply_patches $(QUIET_CONFIGURE) "$(realpath patches)" "$(realpath .)/source"
++	# Debian patch: Update autoconf files to support newer architectures
++	cd source && autoconf aconfigure.ac >aconfigure
++	cp /usr/share/misc/config.guess source/config.guess
++	cp /usr/share/misc/config.sub source/config.sub
++	# End Debian patch
+ 	-@touch source/.unpacked
+ 
+ source/version.mak: source/.unpacked
diff -pruN 1:13.23.1~dfsg-2/debian/patches/hack-multiple-app-voicemail 1:16.1.1~dfsg-1/debian/patches/hack-multiple-app-voicemail
--- 1:13.23.1~dfsg-2/debian/patches/hack-multiple-app-voicemail	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/hack-multiple-app-voicemail	2019-01-11 17:51:43.000000000 +0000
@@ -50,10 +50,10 @@ Last-Update: 2016-03-19
 +
  include $(ASTTOPDIR)/Makefile.moddir_rules
  
- clean::
+ $(call MOD_ADD_C,app_confbridge,$(wildcard confbridge/*.c))
 --- a/Makefile.moddir_rules
 +++ b/Makefile.moddir_rules
-@@ -37,7 +37,7 @@
+@@ -29,7 +29,7 @@
  # Use MODULE_EXCLUDE to specify additional modules to exclude.
  
  ifneq ($(MODULE_PREFIX),)
diff -pruN 1:13.23.1~dfsg-2/debian/patches/mpglib 1:16.1.1~dfsg-1/debian/patches/mpglib
--- 1:13.23.1~dfsg-2/debian/patches/mpglib	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/mpglib	2019-01-11 17:51:43.000000000 +0000
@@ -2,11 +2,11 @@ Description: mpglib code originally in a
  The package asterisk-addons originally included mpglib. After the merge
  with asterisk, that code is no longer included and needs to be fetched
  (contrib/scripts/get_mpg_source.sh). This patch includes that fetched
- source (rev. 180).
+ source (rev. 202).
  .
  TODO: get rid of this code and use libmpg123 or whatever.
 Origin:  http://svn.digium.com/svn/thirdparty/mp3/trunk
-Last-Update: 2016-03-19
+Last-Update: 2018-11-17
 
 --- /dev/null
 +++ b/addons/mp3/common.c
@@ -1331,7 +1331,9 @@ Last-Update: 2016-03-19
 +
 --- /dev/null
 +++ b/addons/mp3/interface.c
-@@ -0,0 +1,323 @@
+@@ -0,0 +1,325 @@
++#define WRAP_LIBC_MALLOC
++#define ASTMM_LIBC ASTMM_REDIRECT
 +#include "asterisk.h"
 +#include "asterisk/logger.h"
 +
diff -pruN 1:13.23.1~dfsg-2/debian/patches/reenable 1:16.1.1~dfsg-1/debian/patches/reenable
--- 1:13.23.1~dfsg-2/debian/patches/reenable	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/reenable	2019-01-11 17:51:43.000000000 +0000
@@ -4,12 +4,13 @@ Last-Update: 2016-03-19
 
 --- a/channels/chan_vpb.cc
 +++ b/channels/chan_vpb.cc
-@@ -49,7 +49,6 @@
+@@ -49,7 +49,7 @@
  
  /*** MODULEINFO
  	<depend>vpb</depend>
 -	<defaultenabled>no</defaultenabled>
- 	<support_level>extended</support_level>
++	<defaultenabled>yes</defaultenabled>
+ 	<support_level>deprecated</support_level>
   ***/
  
 --- a/channels/chan_mgcp.c
diff -pruN 1:13.23.1~dfsg-2/debian/patches/series 1:16.1.1~dfsg-1/debian/patches/series
--- 1:13.23.1~dfsg-2/debian/patches/series	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/series	2019-01-11 17:51:43.000000000 +0000
@@ -31,3 +31,4 @@ ffmpeg-detection.patch
 ffmpeg-includes.patch
 
 build-reproducibly
+autoreconf-pjproject
diff -pruN 1:13.23.1~dfsg-2/debian/patches/smsq_enable.patch 1:16.1.1~dfsg-1/debian/patches/smsq_enable.patch
--- 1:13.23.1~dfsg-2/debian/patches/smsq_enable.patch	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/smsq_enable.patch	2019-01-11 17:51:43.000000000 +0000
@@ -7,7 +7,7 @@ Last-Update: 2016-03-19
 --- a/utils/utils.xml
 +++ b/utils/utils.xml
 @@ -37,7 +37,7 @@
- 	<support_level>extended</support_level>
+ 	<support_level>deprecated</support_level>
    </member>
    <member name="smsq">
 -	<defaultenabled>no</defaultenabled>
diff -pruN 1:13.23.1~dfsg-2/debian/patches/systemd.patch 1:16.1.1~dfsg-1/debian/patches/systemd.patch
--- 1:13.23.1~dfsg-2/debian/patches/systemd.patch	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/patches/systemd.patch	2019-01-11 17:51:43.000000000 +0000
@@ -41,7 +41,7 @@ Last-Update: 2016-04-02
 +# Note: if you changed the parameters passed in the command-line (ExecStart)
 +# be sure to copy them over here as well.
 +[Service]
-+ExecStart = $BASE_DIR/asterisk g -f -U asterisk
++ExecStart = $BASE_DIR/asterisk g -f -p -U asterisk
 +ExecReload = $BASE_DIR/asterisk -rx 'core reload'
 +WorkingDirectory = $varlibdir
 +EOF
@@ -90,7 +90,7 @@ Last-Update: 2016-04-02
 +
 +[Service]
 +Type=notify
-+ExecStart=__ASTERISK_SBIN_DIR__/asterisk -g -f -U asterisk
++ExecStart=__ASTERISK_SBIN_DIR__/asterisk -g -f -p -U asterisk
 +ExecReload=__ASTERISK_SBIN_DIR__/asterisk -rx 'core reload'
 +Restart=on-failure
 +RestartSec=1
Binary files 1:13.23.1~dfsg-2/debian/pjproject_2.8~dfsg.orig.tar.bz2 and 1:16.1.1~dfsg-1/debian/pjproject_2.8~dfsg.orig.tar.bz2 differ
diff -pruN 1:13.23.1~dfsg-2/debian/README.Debian 1:16.1.1~dfsg-1/debian/README.Debian
--- 1:13.23.1~dfsg-2/debian/README.Debian	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/README.Debian	2019-01-11 17:51:43.000000000 +0000
@@ -260,7 +260,7 @@ set of configuration files installed to
 
 If you do need any of those protocols, copy the sample file from
 /usr/share/asterisk/conf/samples/FILE.conf.sample to
-/etc/asterisk/FILE.conf and resart asterisk (or the specific module).
+/etc/asterisk/FILE.conf and restart asterisk (or the specific module).
 
 
 Missing Documentation
diff -pruN 1:13.23.1~dfsg-2/debian/README.source 1:16.1.1~dfsg-1/debian/README.source
--- 1:13.23.1~dfsg-2/debian/README.source	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/README.source	2019-01-11 17:51:43.000000000 +0000
@@ -23,3 +23,22 @@ system, and from the same system, runnin
   ./debian/tests/asttestmods
 
 Note that it needs to be run as root.
+
+== pjproject repacking ==
+Upstream only supports building against their local copy of pjproject (later
+installed as libasteriskpj), which is downloaded, patched and built during
+the build process. Builds in Debian may not download anything during the
+build, in addition parts of the pjproject source archive may not be DFSG free.
+
+The Debian package includes a dfsg-repacked upstream tarball in debian/,
+which is generated with the command
+
+mk-origtargz --package pjproject --version 2.8 \
+	--copyright-file copyright.pjproject \
+	--compression bzip2 \
+	--repack-suffix '~dfsg' \
+	pjproject-2.8.tar.bz2
+
+It uses a local copy of the debian/copyright file of src:pjproject as input
+for deleted files. In the buildprocess the .tar.bz2 is copied into place
+and the checksum regenerated.
diff -pruN 1:13.23.1~dfsg-2/debian/rules 1:16.1.1~dfsg-1/debian/rules
--- 1:13.23.1~dfsg-2/debian/rules	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/rules	2019-01-11 17:51:43.000000000 +0000
@@ -74,6 +74,12 @@ override_dh_autoreconf:
 # ignore them.
 override_dh_auto_configure:
 	chmod 755 $(CURDIR)/debian/dummyprogs/fetch
+	# Copy DSFG-free version of pjproject source into /tmp where the
+	# upstream build will find it, update checksums because we repacked
+	# it
+	cp -a debian/pjproject_2.8~dfsg.orig.tar.bz2 /tmp/pjproject-2.8.tar.bz2
+	cp third-party/pjproject/pjproject-2.8.tar.bz2.md5 third-party/pjproject/pjproject-2.8.tar.bz2.md5.debian_sav
+	md5sum /tmp/pjproject-2.8.tar.bz2 | sed 's:/tmp/::' > third-party/pjproject/pjproject-2.8.tar.bz2.md5
 	$(FETCH_ENV) ./configure			\
 		CFLAGS="-fgnu89-inline $(CFLAGS) $(CPPFLAGS)" LDFLAGS="$(LDFLAGS) -Wl,--as-needed" \
 		--host=$(DEB_HOST_GNU_TYPE)		\
@@ -92,6 +98,7 @@ override_dh_auto_configure:
 
 override_dh_auto_build:
 	$(MAKE) menuselect.makeopts BUILD_CFLAGS="$(CFLAGS) $(CPPFLAGS)" BUILD_LDFLAGS="$(LDFLAGS)"
+	menuselect/menuselect --enable app_macro menuselect.makeopts
 	menuselect/menuselect --enable codec_opus_open_source menuselect.makeopts
 	@if [ "x${ENABLE_DEBUG}" != "x" ] ; then \
 		menuselect/menuselect --enable BETTER_BACKTRACES menuselect.makeopts ; \
@@ -116,6 +123,9 @@ override_dh_clean:
 
 	[ ! -f .version.debian_sav ] || mv .version.debian_sav .version
 	if [ -f configure.debian_sav ]; then mv configure.debian_sav configure; fi
+	if [ -f third-party/pjproject/pjproject-2.8.tar.bz2.md5.debian_sav ]; then \
+	  mv third-party/pjproject/pjproject-2.8.tar.bz2.md5.debian_sav third-party/pjproject/pjproject-2.8.tar.bz2.md5; \
+	fi
 
 	dh_clean
 
diff -pruN 1:13.23.1~dfsg-2/debian/source/include-binaries 1:16.1.1~dfsg-1/debian/source/include-binaries
--- 1:13.23.1~dfsg-2/debian/source/include-binaries	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/source/include-binaries	2019-01-11 17:51:43.000000000 +0000
@@ -0,0 +1 @@
+debian/pjproject_2.8~dfsg.orig.tar.bz2
diff -pruN 1:13.23.1~dfsg-2/debian/upstream/7C2C8A8BCB3F61BD.key 1:16.1.1~dfsg-1/debian/upstream/7C2C8A8BCB3F61BD.key
--- 1:13.23.1~dfsg-2/debian/upstream/7C2C8A8BCB3F61BD.key	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/upstream/7C2C8A8BCB3F61BD.key	2019-01-11 17:51:43.000000000 +0000
@@ -0,0 +1,56 @@
+pub   rsa4096 2018-04-19 [SC]
+      0F77FB5D216A02390B4C51DB7C2C8A8BCB3F61BD
+uid        [ unbekannt ] Chris Savinovich <csavinovich@digium.com>
+sub   rsa4096 2018-04-19 [E]
+
+-----BEGIN PGP PUBLIC KEY BLOCK-----
+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+=nOCz
+-----END PGP PUBLIC KEY BLOCK-----
diff -pruN 1:13.23.1~dfsg-2/debian/upstream/signing-key.asc 1:16.1.1~dfsg-1/debian/upstream/signing-key.asc
--- 1:13.23.1~dfsg-2/debian/upstream/signing-key.asc	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/upstream/signing-key.asc	2019-01-11 17:51:43.000000000 +0000
@@ -143,6 +143,62 @@ OEiiRIHfg2TH+ig7Y7yISQQYEQIACQUCTHvWEQIb
 My5iY8M7a9AHJFEBloY6W6NPMwCfboD0UcExki9nmTvN0pfmdfusu3k=
 =ZhBt
 -----END PGP PUBLIC KEY BLOCK-----
+pub   rsa4096 2018-04-19 [SC]
+      0F77FB5D216A02390B4C51DB7C2C8A8BCB3F61BD
+uid        [ unbekannt ] Chris Savinovich <csavinovich@digium.com>
+sub   rsa4096 2018-04-19 [E]
+
+-----BEGIN PGP PUBLIC KEY BLOCK-----
+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+=nOCz
+-----END PGP PUBLIC KEY BLOCK-----
 pub   dsa1024 2007-06-08 [SC]
       80CEBC345EC9FF529B4B7B808438CBA18D0CAA72
 uid        [ unbekannt ] Matthew Fredrickson <creslin@digium.com>
diff -pruN 1:13.23.1~dfsg-2/debian/watch 1:16.1.1~dfsg-1/debian/watch
--- 1:13.23.1~dfsg-2/debian/watch	2019-01-03 15:20:10.000000000 +0000
+++ 1:16.1.1~dfsg-1/debian/watch	2019-01-11 17:51:43.000000000 +0000
@@ -1,4 +1,4 @@
 # run "uscan --report" to check or "gpb import-orig --uscan" to update
 version=4
 opts="repack,compression=xz,dversionmangle=s/~dfsg\d*$//,repacksuffix=~dfsg,pgpsigurlmangle=s/$/.asc/" \
-	http://downloads.asterisk.org/pub/telephony/asterisk/releases/asterisk-(13.[0-9.]*).tar.gz debian
+	http://downloads.asterisk.org/pub/telephony/asterisk/releases/asterisk-(16.[0-9.]*).tar.gz debian
diff -pruN 1:13.23.1~dfsg-2/doc/asterisk.8 1:16.1.1~dfsg-1/doc/asterisk.8
--- 1:13.23.1~dfsg-2/doc/asterisk.8	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/doc/asterisk.8	2018-12-26 18:45:13.000000000 +0000
@@ -232,9 +232,8 @@ then terminating when the command execut
 supplied.
 .TP
 \-X
-Enables executing of includes via \fB#exec\fR directive.
-This can be useful if You want to do \fB#exec\fR inside
-\*(T<\fIasterisk.conf\fR\*(T>
+Enables executing of includes via \fB#exec\fR directive inside
+\*(T<\fIasterisk.conf\fR\*(T>.
 .SH EXAMPLES
 \fBasterisk\fR - Begin Asterisk as a daemon
 .PP
diff -pruN 1:13.23.1~dfsg-2/formats/format_g719.c 1:16.1.1~dfsg-1/formats/format_g719.c
--- 1:13.23.1~dfsg-2/formats/format_g719.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_g719.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_g723.c 1:16.1.1~dfsg-1/formats/format_g723.c
--- 1:13.23.1~dfsg-2/formats/format_g723.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_g723.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/format_cache.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_g726.c 1:16.1.1~dfsg-1/formats/format_g726.c
--- 1:13.23.1~dfsg-2/formats/format_g726.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_g726.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_g729.c 1:16.1.1~dfsg-1/formats/format_g729.c
--- 1:13.23.1~dfsg-2/formats/format_g729.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_g729.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_gsm.c 1:16.1.1~dfsg-1/formats/format_gsm.c
--- 1:13.23.1~dfsg-2/formats/format_gsm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_gsm.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_h263.c 1:16.1.1~dfsg-1/formats/format_h263.c
--- 1:13.23.1~dfsg-2/formats/format_h263.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_h263.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_h264.c 1:16.1.1~dfsg-1/formats/format_h264.c
--- 1:13.23.1~dfsg-2/formats/format_h264.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_h264.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_ilbc.c 1:16.1.1~dfsg-1/formats/format_ilbc.c
--- 1:13.23.1~dfsg-2/formats/format_ilbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_ilbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_jpeg.c 1:16.1.1~dfsg-1/formats/format_jpeg.c
--- 1:13.23.1~dfsg-2/formats/format_jpeg.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_jpeg.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,117 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Mark Spencer <markster@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- *
- * \brief JPEG File format support.
- *
- * \arg File name extension: jpeg, jpg
- * \ingroup formats
- */
-
-/*** MODULEINFO
-	<support_level>extended</support_level>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include "asterisk/mod_format.h"
-#include "asterisk/module.h"
-#include "asterisk/image.h"
-#include "asterisk/endian.h"
-#include "asterisk/format_cache.h"
-
-static struct ast_frame *jpeg_read_image(int fd, int len)
-{
-	struct ast_frame fr;
-	int res;
-	char buf[65536];
-	if (len > sizeof(buf) || len < 0) {
-		ast_log(LOG_WARNING, "JPEG image too large to read\n");
-		return NULL;
-	}
-	res = read(fd, buf, len);
-	if (res < len) {
-		ast_log(LOG_WARNING, "Only read %d of %d bytes: %s\n", res, len, strerror(errno));
-	}
-	memset(&fr, 0, sizeof(fr));
-	fr.frametype = AST_FRAME_IMAGE;
-	fr.subclass.format = ast_format_jpeg;
-	fr.data.ptr = buf;
-	fr.src = "JPEG Read";
-	fr.datalen = len;
-	return ast_frisolate(&fr);
-}
-
-static int jpeg_identify(int fd)
-{
-	char buf[10];
-	int res;
-	res = read(fd, buf, sizeof(buf));
-	if (res < sizeof(buf))
-		return 0;
-	if (memcmp(buf + 6, "JFIF", 4))
-		return 0;
-	return 1;
-}
-
-static int jpeg_write_image(int fd, struct ast_frame *fr)
-{
-	int res=0;
-	if (fr->datalen) {
-		res = write(fd, fr->data.ptr, fr->datalen);
-		if (res != fr->datalen) {
-			ast_log(LOG_WARNING, "Only wrote %d of %d bytes: %s\n", res, fr->datalen, strerror(errno));
-			return -1;
-		}
-	}
-	return res;
-}
-
-static struct ast_imager jpeg_format = {
-	.name = "jpg",
-	.desc = "JPEG (Joint Picture Experts Group)",
-	.exts = "jpg|jpeg",
-	.read_image = jpeg_read_image,
-	.identify = jpeg_identify,
-	.write_image = jpeg_write_image,
-};
-
-static int load_module(void)
-{
-	jpeg_format.format = ast_format_jpeg;
-	if (ast_image_register(&jpeg_format))
-		return AST_MODULE_LOAD_DECLINE;
-	return AST_MODULE_LOAD_SUCCESS;
-}
-
-static int unload_module(void)
-{
-	ast_image_unregister(&jpeg_format);
-
-	return 0;
-}
-
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "jpeg (joint picture experts group) image format",
-	.support_level = AST_MODULE_SUPPORT_EXTENDED,
-	.load = load_module,
-	.unload = unload_module,
-	.load_pri = AST_MODPRI_APP_DEPEND
-);
diff -pruN 1:13.23.1~dfsg-2/formats/format_ogg_speex.c 1:16.1.1~dfsg-1/formats/format_ogg_speex.c
--- 1:13.23.1~dfsg-2/formats/format_ogg_speex.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_ogg_speex.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,345 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2011-2016, Timo Teräs
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief OGG/Speex streams.
+ * \arg File name extension: spx
+ * \ingroup formats
+ */
+
+/*** MODULEINFO
+	<depend>speex</depend>
+	<depend>ogg</depend>
+	<support_level>extended</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/mod_format.h"
+#include "asterisk/module.h"
+#include "asterisk/format_cache.h"
+
+#include <speex/speex_header.h>
+#include <ogg/ogg.h>
+
+#define BLOCK_SIZE	4096		/* buffer size for feeding OGG routines */
+#define	BUF_SIZE	200
+
+struct speex_desc {	/* format specific parameters */
+	/* structures for handling the Ogg container */
+	ogg_sync_state oy;
+	ogg_stream_state os;
+	ogg_page og;
+	ogg_packet op;
+
+	int serialno;
+
+	/*! \brief Indicates whether an End of Stream condition has been detected. */
+	int eos;
+};
+
+static int read_packet(struct ast_filestream *fs)
+{
+	struct speex_desc *s = (struct speex_desc *)fs->_private;
+	char *buffer;
+	int result;
+	size_t bytes;
+
+	while (1) {
+		/* Get one packet */
+		result = ogg_stream_packetout(&s->os, &s->op);
+		if (result > 0) {
+			if (s->op.bytes >= 5 && !memcmp(s->op.packet, "Speex", 5)) {
+				s->serialno = s->os.serialno;
+			}
+			if (s->serialno == -1 || s->os.serialno != s->serialno) {
+				continue;
+			}
+			return 0;
+		}
+
+		if (result < 0) {
+			ast_log(LOG_WARNING,
+				"Corrupt or missing data at this page position; continuing...\n");
+		}
+
+		/* No more packets left in the current page... */
+		if (s->eos) {
+			/* No more pages left in the stream */
+			return -1;
+		}
+
+		while (!s->eos) {
+			/* See if OGG has any pages in it's internal buffers */
+			result = ogg_sync_pageout(&s->oy, &s->og);
+			if (result > 0) {
+				/* Read all streams. */
+				if (ogg_page_serialno(&s->og) != s->os.serialno) {
+					ogg_stream_reset_serialno(&s->os, ogg_page_serialno(&s->og));
+				}
+				/* Yes, OGG has more pages in it's internal buffers,
+				   add the page to the stream state */
+				result = ogg_stream_pagein(&s->os, &s->og);
+				if (result == 0) {
+					/* Yes, got a new, valid page */
+					if (ogg_page_eos(&s->og) &&
+					    ogg_page_serialno(&s->og) == s->serialno)
+						s->eos = 1;
+					break;
+				}
+				ast_log(LOG_WARNING,
+					"Invalid page in the bitstream; continuing...\n");
+			}
+
+			if (result < 0) {
+				ast_log(LOG_WARNING,
+					"Corrupt or missing data in bitstream; continuing...\n");
+			}
+
+			/* No, we need to read more data from the file descrptor */
+			/* get a buffer from OGG to read the data into */
+			buffer = ogg_sync_buffer(&s->oy, BLOCK_SIZE);
+			bytes = fread(buffer, 1, BLOCK_SIZE, fs->f);
+			ogg_sync_wrote(&s->oy, bytes);
+			if (bytes == 0) {
+				s->eos = 1;
+			}
+		}
+	}
+}
+
+/*!
+ * \brief Create a new OGG/Speex filestream and set it up for reading.
+ * \param fs File that points to on disk storage of the OGG/Speex data.
+ * \return The new filestream.
+ */
+static int ogg_speex_open(struct ast_filestream *fs)
+{
+	char *buffer;
+	size_t bytes;
+	struct speex_desc *s = (struct speex_desc *)fs->_private;
+	SpeexHeader *hdr = NULL;
+	int i, result, expected_rate;
+
+	expected_rate = ast_format_get_sample_rate(fs->fmt->format);
+	s->serialno = -1;
+	ogg_sync_init(&s->oy);
+
+	buffer = ogg_sync_buffer(&s->oy, BLOCK_SIZE);
+	bytes = fread(buffer, 1, BLOCK_SIZE, fs->f);
+	ogg_sync_wrote(&s->oy, bytes);
+
+	result = ogg_sync_pageout(&s->oy, &s->og);
+	if (result != 1) {
+		if(bytes < BLOCK_SIZE) {
+			ast_log(LOG_ERROR, "Run out of data...\n");
+		} else {
+			ast_log(LOG_ERROR, "Input does not appear to be an Ogg bitstream.\n");
+		}
+		ogg_sync_clear(&s->oy);
+		return -1;
+	}
+
+	ogg_stream_init(&s->os, ogg_page_serialno(&s->og));
+	if (ogg_stream_pagein(&s->os, &s->og) < 0) {
+		ast_log(LOG_ERROR, "Error reading first page of Ogg bitstream data.\n");
+		goto error;
+	}
+
+	if (read_packet(fs) < 0) {
+		ast_log(LOG_ERROR, "Error reading initial header packet.\n");
+		goto error;
+	}
+
+	hdr = speex_packet_to_header((char*)s->op.packet, s->op.bytes);
+	if (memcmp(hdr->speex_string, "Speex   ", 8)) {
+		ast_log(LOG_ERROR, "OGG container does not contain Speex audio!\n");
+		goto error;
+	}
+	if (hdr->frames_per_packet != 1) {
+		ast_log(LOG_ERROR, "Only one frame-per-packet OGG/Speex files are currently supported!\n");
+		goto error;
+	}
+	if (hdr->nb_channels != 1) {
+		ast_log(LOG_ERROR, "Only monophonic OGG/Speex files are currently supported!\n");
+		goto error;
+	}
+	if (hdr->rate != expected_rate) {
+		ast_log(LOG_ERROR, "Unexpected sampling rate (%d != %d)!\n",
+			hdr->rate, expected_rate);
+		goto error;
+	}
+
+	/* this packet is the comment */
+	if (read_packet(fs) < 0) {
+		ast_log(LOG_ERROR, "Error reading comment packet.\n");
+		goto error;
+	}
+	for (i = 0; i < hdr->extra_headers; i++) {
+		if (read_packet(fs) < 0) {
+			ast_log(LOG_ERROR, "Error reading extra header packet %d.\n", i+1);
+			goto error;
+		}
+	}
+	speex_header_free(hdr);
+
+	return 0;
+error:
+	if (hdr) {
+		speex_header_free(hdr);
+	}
+	ogg_stream_clear(&s->os);
+	ogg_sync_clear(&s->oy);
+	return -1;
+}
+
+/*!
+ * \brief Close a OGG/Speex filestream.
+ * \param fs A OGG/Speex filestream.
+ */
+static void ogg_speex_close(struct ast_filestream *fs)
+{
+	struct speex_desc *s = (struct speex_desc *)fs->_private;
+
+	ogg_stream_clear(&s->os);
+	ogg_sync_clear(&s->oy);
+}
+
+/*!
+ * \brief Read a frame full of audio data from the filestream.
+ * \param fs The filestream.
+ * \param whennext Number of sample times to schedule the next call.
+ * \return A pointer to a frame containing audio data or NULL ifthere is no more audio data.
+ */
+static struct ast_frame *ogg_speex_read(struct ast_filestream *fs,
+					 int *whennext)
+{
+	struct speex_desc *s = (struct speex_desc *)fs->_private;
+
+	if (read_packet(fs) < 0) {
+		return NULL;
+	}
+
+	AST_FRAME_SET_BUFFER(&fs->fr, fs->buf, AST_FRIENDLY_OFFSET, BUF_SIZE);
+	memcpy(fs->fr.data.ptr, s->op.packet, s->op.bytes);
+	fs->fr.datalen = s->op.bytes;
+	fs->fr.samples = *whennext = ast_codec_samples_count(&fs->fr);
+
+	return &fs->fr;
+}
+
+/*!
+ * \brief Trucate an OGG/Speex filestream.
+ * \param s The filestream to truncate.
+ * \return 0 on success, -1 on failure.
+ */
+
+static int ogg_speex_trunc(struct ast_filestream *s)
+{
+	ast_log(LOG_WARNING, "Truncation is not supported on OGG/Speex streams!\n");
+	return -1;
+}
+
+/*!
+ * \brief Seek to a specific position in an OGG/Speex filestream.
+ * \param s The filestream to truncate.
+ * \param sample_offset New position for the filestream, measured in 8KHz samples.
+ * \param whence Location to measure
+ * \return 0 on success, -1 on failure.
+ */
+static int ogg_speex_seek(struct ast_filestream *s, off_t sample_offset, int whence)
+{
+	ast_log(LOG_WARNING, "Seeking is not supported on OGG/Speex streams!\n");
+	return -1;
+}
+
+static off_t ogg_speex_tell(struct ast_filestream *s)
+{
+	ast_log(LOG_WARNING, "Telling is not supported on OGG/Speex streams!\n");
+	return -1;
+}
+
+static struct ast_format_def speex_f = {
+	.name = "ogg_speex",
+	.exts = "spx",
+	.open = ogg_speex_open,
+	.seek = ogg_speex_seek,
+	.trunc = ogg_speex_trunc,
+	.tell = ogg_speex_tell,
+	.read = ogg_speex_read,
+	.close = ogg_speex_close,
+	.buf_size = BUF_SIZE + AST_FRIENDLY_OFFSET,
+	.desc_size = sizeof(struct speex_desc),
+};
+
+static struct ast_format_def speex16_f = {
+	.name = "ogg_speex16",
+	.exts = "spx16",
+	.open = ogg_speex_open,
+	.seek = ogg_speex_seek,
+	.trunc = ogg_speex_trunc,
+	.tell = ogg_speex_tell,
+	.read = ogg_speex_read,
+	.close = ogg_speex_close,
+	.buf_size = BUF_SIZE + AST_FRIENDLY_OFFSET,
+	.desc_size = sizeof(struct speex_desc),
+};
+
+static struct ast_format_def speex32_f = {
+	.name = "ogg_speex32",
+	.exts = "spx32",
+	.open = ogg_speex_open,
+	.seek = ogg_speex_seek,
+	.trunc = ogg_speex_trunc,
+	.tell = ogg_speex_tell,
+	.read = ogg_speex_read,
+	.close = ogg_speex_close,
+	.buf_size = BUF_SIZE + AST_FRIENDLY_OFFSET,
+	.desc_size = sizeof(struct speex_desc),
+};
+
+static int unload_module(void)
+{
+	int res = 0;
+	res |= ast_format_def_unregister(speex_f.name);
+	res |= ast_format_def_unregister(speex16_f.name);
+	res |= ast_format_def_unregister(speex32_f.name);
+	return res;
+}
+
+static int load_module(void)
+{
+	speex_f.format = ast_format_speex;
+	speex16_f.format = ast_format_speex16;
+	speex32_f.format = ast_format_speex32;
+
+	if (ast_format_def_register(&speex_f) ||
+	    ast_format_def_register(&speex16_f) ||
+	    ast_format_def_register(&speex32_f)) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "OGG/Speex audio",
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND
+);
diff -pruN 1:13.23.1~dfsg-2/formats/format_ogg_vorbis.c 1:16.1.1~dfsg-1/formats/format_ogg_vorbis.c
--- 1:13.23.1~dfsg-2/formats/format_ogg_vorbis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_ogg_vorbis.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ogg/ogg.h>
 #include <vorbis/codec.h>
 #include <vorbis/vorbisenc.h>
diff -pruN 1:13.23.1~dfsg-2/formats/format_pcm.c 1:16.1.1~dfsg-1/formats/format_pcm.c
--- 1:13.23.1~dfsg-2/formats/format_pcm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_pcm.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
@@ -509,6 +507,7 @@ static struct ast_format_def alaw_f = {
 static struct ast_format_def pcm_f = {
 	.name = "pcm",
 	.exts = "pcm|ulaw|ul|mu|ulw",
+	.mime_types = "audio/basic",
 	.write = pcm_write,
 	.seek = pcm_seek,
 	.trunc = pcm_trunc,
diff -pruN 1:13.23.1~dfsg-2/formats/format_siren14.c 1:16.1.1~dfsg-1/formats/format_siren14.c
--- 1:13.23.1~dfsg-2/formats/format_siren14.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_siren14.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_siren7.c 1:16.1.1~dfsg-1/formats/format_siren7.c
--- 1:13.23.1~dfsg-2/formats/format_siren7.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_siren7.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_sln.c 1:16.1.1~dfsg-1/formats/format_sln.c
--- 1:13.23.1~dfsg-2/formats/format_sln.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_sln.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/formats/format_vox.c 1:16.1.1~dfsg-1/formats/format_vox.c
--- 1:13.23.1~dfsg-2/formats/format_vox.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_vox.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
@@ -130,6 +128,7 @@ static off_t vox_tell(struct ast_filestr
 static struct ast_format_def vox_f = {
 	.name = "vox",
 	.exts = "vox",
+	.mime_types = "audio/x-vox",
 	.write = vox_write,
 	.seek = vox_seek,
 	.trunc = vox_trunc,
diff -pruN 1:13.23.1~dfsg-2/formats/format_wav.c 1:16.1.1~dfsg-1/formats/format_wav.c
--- 1:13.23.1~dfsg-2/formats/format_wav.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_wav.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
@@ -534,6 +532,7 @@ static struct ast_format_def wav16_f = {
 static struct ast_format_def wav_f = {
 	.name = "wav",
 	.exts = "wav",
+	.mime_types = "audio/wav|audio/x-wav",
 	.open =	wav_open,
 	.rewrite = wav_rewrite,
 	.write = wav_write,
diff -pruN 1:13.23.1~dfsg-2/formats/format_wav_gsm.c 1:16.1.1~dfsg-1/formats/format_wav_gsm.c
--- 1:13.23.1~dfsg-2/formats/format_wav_gsm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/formats/format_wav_gsm.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/mod_format.h"
 #include "asterisk/module.h"
 #include "asterisk/endian.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_aes.c 1:16.1.1~dfsg-1/funcs/func_aes.c
--- 1:13.23.1~dfsg-2/funcs/func_aes.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_aes.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 #include "asterisk/app.h"
@@ -180,8 +178,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "AES dialplan functions",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.nonoptreq = "res_crypto",
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_crypto",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_audiohookinherit.c 1:16.1.1~dfsg-1/funcs/func_audiohookinherit.c
--- 1:13.23.1~dfsg-2/funcs/func_audiohookinherit.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_audiohookinherit.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,90 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2008, Digium, Inc.
- *
- * Mark Michelson <mmichelson@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- *
- * Please follow coding guidelines
- * http://svn.digium.com/view/asterisk/trunk/doc/CODING-GUIDELINES
- */
-
-/*! \file
- *
- * \brief Audiohook inheritance function
- *
- * \author Mark Michelson <mmichelson@digium.com>
- *
- * \ingroup functions
- */
-
-/*** MODULEINFO
-	<defaultenabled>no</defaultenabled>
-	<support_level>deprecated</support_level>
- ***/
-
-#include "asterisk.h"
-#include "asterisk/channel.h"
-#include "asterisk/logger.h"
-#include "asterisk/pbx.h"
-#include "asterisk/module.h"
-
-/*** DOCUMENTATION
-	<function name = "AUDIOHOOK_INHERIT" language="en_US">
-		<synopsis>
-			DEPRECATED: Used to set whether an audiohook may be inherited to another
-			channel. Due to architectural changes in Asterisk 12, audiohook inheritance
-			is performed automatically and this function now lacks function.
-		</synopsis>
-		<description>
-			<para>Prior to Asterisk 12, masquerades would occur under all sorts of
-			situations which were hard to predict.  In Asterisk 12, masquerades only
-			occur as a result of a small set of operations for which inheriting all
-			audiohooks from the original channel is now safe.  So in Asterisk 12.5+,
-			all audiohooks are inherited without needing other controls expressing
-			which audiohooks should be inherited under which conditions.</para>
-		</description>
-	</function>
- ***/
-
-static int func_inheritance_write(struct ast_channel *chan, const char *function, char *data, const char *value)
-{
-	static int warned = 0;
-
-	if (!warned) {
-		ast_log(LOG_NOTICE, "AUDIOHOOK_INHERIT is deprecated and now does nothing.\n");
-		warned++;
-	}
-
-	return 0;
-}
-
-static struct ast_custom_function inheritance_function = {
-	.name = "AUDIOHOOK_INHERIT",
-	.write = func_inheritance_write,
-};
-
-static int unload_module(void)
-{
-	return ast_custom_function_unregister(&inheritance_function);
-}
-
-static int load_module(void)
-{
-	if (ast_custom_function_register(&inheritance_function)) {
-		return AST_MODULE_LOAD_DECLINE;
-	} else {
-		return AST_MODULE_LOAD_SUCCESS;
-	}
-}
-AST_MODULE_INFO_STANDARD_DEPRECATED(ASTERISK_GPL_KEY, "Audiohook inheritance placeholder function");
diff -pruN 1:13.23.1~dfsg-2/funcs/func_base64.c 1:16.1.1~dfsg-1/funcs/func_base64.c
--- 1:13.23.1~dfsg-2/funcs/func_base64.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_base64.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"	/* function register/unregister */
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_blacklist.c 1:16.1.1~dfsg-1/funcs/func_blacklist.c
--- 1:13.23.1~dfsg-2/funcs/func_blacklist.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_blacklist.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_callcompletion.c 1:16.1.1~dfsg-1/funcs/func_callcompletion.c
--- 1:13.23.1~dfsg-2/funcs/func_callcompletion.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_callcompletion.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/ccss.h"
@@ -128,4 +126,9 @@ static int load_module(void)
 	return ast_custom_function_register(&cc_function) == 0 ? AST_MODULE_LOAD_SUCCESS : AST_MODULE_LOAD_DECLINE;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Call Control Configuration Function");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Call Control Configuration Function",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "ccss",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_callerid.c 1:16.1.1~dfsg-1/funcs/func_callerid.c
--- 1:13.23.1~dfsg-2/funcs/func_callerid.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_callerid.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_cdr.c 1:16.1.1~dfsg-1/funcs/func_cdr.c
--- 1:13.23.1~dfsg-2/funcs/func_cdr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_cdr.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -706,4 +704,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Call Detail Record (CDR) dialplan functions");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Call Detail Record (CDR) dialplan functions",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "cdr",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_channel.c 1:16.1.1~dfsg-1/funcs/func_channel.c
--- 1:13.23.1~dfsg-2/funcs/func_channel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_channel.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 #include <ctype.h>
 
@@ -160,6 +158,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						and exten are supplied by the channel pushing the handler
 						before it is pushed.</para>
 					</enum>
+					<enum name="onhold">
+						<para>R/O Whether or not the channel is onhold. (1/0)</para>
+					</enum>
 					<enum name="language">
 						<para>R/W language for sounds played.</para>
 					</enum>
@@ -339,7 +340,10 @@ static int func_channel_read(struct ast_
 		locked_copy_string(chan, buf, ast_channel_parkinglot(chan), len);
 	else if (!strcasecmp(data, "state"))
 		locked_copy_string(chan, buf, ast_state2str(ast_channel_state(chan)), len);
-	else if (!strcasecmp(data, "channeltype"))
+	else if (!strcasecmp(data, "onhold")) {
+		locked_copy_string(chan, buf,
+			ast_channel_hold_state(chan) == AST_CONTROL_HOLD ? "1" : "0", len);
+	} else if (!strcasecmp(data, "channeltype"))
 		locked_copy_string(chan, buf, ast_channel_tech(chan)->type, len);
 	else if (!strcasecmp(data, "accountcode"))
 		locked_copy_string(chan, buf, ast_channel_accountcode(chan), len);
@@ -446,14 +450,13 @@ static int func_channel_read(struct ast_
 		snprintf(buf, len, "%d", ast_max_forwards_get(chan));
 		ast_channel_unlock(chan);
 	} else if (!strcasecmp(data, "callid")) {
-		struct ast_callid *callid;
+		ast_callid callid;
 
 		buf[0] = '\0';
 		ast_channel_lock(chan);
 		callid = ast_channel_callid(chan);
 		if (callid) {
 			ast_callid_strnprint(buf, len, callid);
-			ast_callid_unref(callid);
 		}
 		ast_channel_unlock(chan);
 	} else if (!ast_channel_tech(chan) || !ast_channel_tech(chan)->func_channel_read || ast_channel_tech(chan)->func_channel_read(chan, function, data, buf, len)) {
diff -pruN 1:13.23.1~dfsg-2/funcs/func_config.c 1:16.1.1~dfsg-1/funcs/func_config.c
--- 1:13.23.1~dfsg-2/funcs/func_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_curl.c 1:16.1.1~dfsg-1/funcs/func_curl.c
--- 1:13.23.1~dfsg-2/funcs/func_curl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_curl.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,8 +37,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <curl/curl.h>
 
 #include "asterisk/lock.h"
@@ -58,15 +56,39 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			Retrieve content from a remote web or ftp server
 		</synopsis>
 		<syntax>
-			<parameter name="url" required="true" />
+			<parameter name="url" required="true">
+				<para>The full URL for the resource to retrieve.</para>
+			</parameter>
 			<parameter name="post-data">
+				<para><emphasis>Read Only</emphasis></para>
 				<para>If specified, an <literal>HTTP POST</literal> will be
 				performed with the content of
 				<replaceable>post-data</replaceable>, instead of an
 				<literal>HTTP GET</literal> (default).</para>
 			</parameter>
 		</syntax>
-		<description />
+		<description>
+			<para>When this function is read, a <literal>HTTP GET</literal>
+			(by default) will be used to retrieve the contents of the provided
+			<replaceable>url</replaceable>. The contents are returned as the
+			result of the function.</para>
+			<example title="Displaying contents of a page" language="text">
+			exten => s,1,Verbose(0, ${CURL(http://localhost:8088/static/astman.css)})
+			</example>
+			<para>When this function is written to, a <literal>HTTP GET</literal>
+			will be used to retrieve the contents of the provided
+			<replaceable>url</replaceable>. The value written to the function
+			specifies the destination file of the cURL'd resource.</para>
+			<example title="Retrieving a file" language="text">
+			exten => s,1,Set(CURL(http://localhost:8088/static/astman.css)=/var/spool/asterisk/tmp/astman.css))
+			</example>
+			<note>
+				<para>If <literal>live_dangerously</literal> in <literal>asterisk.conf</literal>
+				is set to <literal>no</literal>, this function can only be written to from the
+				dialplan, and not directly from external protocols. Read operations are
+				unaffected.</para>
+			</note>
+		</description>
 		<see-also>
 			<ref type="function">CURLOPT</ref>
 		</see-also>
@@ -196,7 +218,7 @@ static void curlds_free(void *data)
 		return;
 	}
 	while ((setting = AST_LIST_REMOVE_HEAD(list, list))) {
-		free(setting);
+		ast_free(setting);
 	}
 	AST_LIST_HEAD_DESTROY(list);
 	ast_free(list);
@@ -395,7 +417,7 @@ yuck:
 	AST_LIST_TRAVERSE_SAFE_BEGIN(list, cur, list) {
 		if (cur->key == new->key) {
 			AST_LIST_REMOVE_CURRENT(list);
-			free(cur);
+			ast_free(cur);
 			break;
 		}
 	}
@@ -526,16 +548,27 @@ static int acf_curlopt_read2(struct ast_
 	return acf_curlopt_helper(chan, cmd, data, NULL, buf, len);
 }
 
+/*! \brief Callback data passed to \ref WriteMemoryCallback */
+struct curl_write_callback_data {
+	/*! \brief If a string is being built, the string buffer */
+	struct ast_str *str;
+	/*! \brief The max size of \ref str */
+	ssize_t len;
+	/*! \brief If a file is being retrieved, the file to write to */
+	FILE *out_file;
+};
+
 static size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data)
 {
-	register int realsize = size * nmemb;
-	struct ast_str **pstr = (struct ast_str **)data;
-
-	ast_debug(3, "Called with data=%p, str=%p, realsize=%d, len=%zu, used=%zu\n", data, *pstr, realsize, ast_str_size(*pstr), ast_str_strlen(*pstr));
-
-	ast_str_append_substr(pstr, 0, ptr, realsize);
+	register int realsize = 0;
+	struct curl_write_callback_data *cb_data = data;
 
-	ast_debug(3, "Now, len=%zu, used=%zu\n", ast_str_size(*pstr), ast_str_strlen(*pstr));
+	if (cb_data->str) {
+		realsize = size * nmemb;
+		ast_str_append_substr(&cb_data->str, 0, ptr, realsize);
+	} else if (cb_data->out_file) {
+		realsize = fwrite(ptr, size, nmemb, cb_data->out_file);
+	}
 
 	return realsize;
 }
@@ -594,15 +627,16 @@ static int url_is_vulnerable(const char
 	return 0;
 }
 
-static int acf_curl_helper(struct ast_channel *chan, const char *cmd, char *info, char *buf, struct ast_str **input_str, ssize_t len)
+struct curl_args {
+	const char *url;
+	const char *postdata;
+	struct curl_write_callback_data cb_data;
+};
+
+static int acf_curl_helper(struct ast_channel *chan, struct curl_args *args)
 {
 	struct ast_str *escapebuf = ast_str_thread_get(&thread_escapebuf, 16);
-	struct ast_str *str = ast_str_create(16);
 	int ret = -1;
-	AST_DECLARE_APP_ARGS(args,
-		AST_APP_ARG(url);
-		AST_APP_ARG(postdata);
-	);
 	CURL **curl;
 	struct curl_settings *cur;
 	struct ast_datastore *store = NULL;
@@ -610,35 +644,17 @@ static int acf_curl_helper(struct ast_ch
 	AST_LIST_HEAD(global_curl_info, curl_settings) *list = NULL;
 	char curl_errbuf[CURL_ERROR_SIZE + 1]; /* add one to be safe */
 
-	if (buf) {
-		*buf = '\0';
-	}
-
-	if (!str) {
-		return -1;
-	}
-
 	if (!escapebuf) {
-		ast_free(str);
 		return -1;
 	}
 
-	if (ast_strlen_zero(info)) {
-		ast_log(LOG_WARNING, "CURL requires an argument (URL)\n");
-		ast_free(str);
-		return -1;
-	}
-
-	AST_STANDARD_APP_ARGS(args, info);
-
-	if (url_is_vulnerable(args.url)) {
-		ast_log(LOG_ERROR, "URL '%s' is vulnerable to HTTP injection attacks. Aborting CURL() call.\n", args.url);
+	if (!(curl = ast_threadstorage_get(&curl_instance, sizeof(*curl)))) {
+		ast_log(LOG_ERROR, "Cannot allocate curl structure\n");
 		return -1;
 	}
 
-	if (!(curl = ast_threadstorage_get(&curl_instance, sizeof(*curl)))) {
-		ast_log(LOG_ERROR, "Cannot allocate curl structure\n");
-		ast_free(str);
+	if (url_is_vulnerable(args->url)) {
+		ast_log(LOG_ERROR, "URL '%s' is vulnerable to HTTP injection attacks. Aborting CURL() call.\n", args->url);
 		return -1;
 	}
 
@@ -673,12 +689,12 @@ static int acf_curl_helper(struct ast_ch
 		}
 	}
 
-	curl_easy_setopt(*curl, CURLOPT_URL, args.url);
-	curl_easy_setopt(*curl, CURLOPT_FILE, (void *) &str);
+	curl_easy_setopt(*curl, CURLOPT_URL, args->url);
+	curl_easy_setopt(*curl, CURLOPT_FILE, (void *) &args->cb_data);
 
-	if (args.postdata) {
+	if (args->postdata) {
 		curl_easy_setopt(*curl, CURLOPT_POST, 1);
-		curl_easy_setopt(*curl, CURLOPT_POSTFIELDS, args.postdata);
+		curl_easy_setopt(*curl, CURLOPT_POSTFIELDS, args->postdata);
 	}
 
 	/* Temporarily assign a buffer for curl to write errors to. */
@@ -686,7 +702,7 @@ static int acf_curl_helper(struct ast_ch
 	curl_easy_setopt(*curl, CURLOPT_ERRORBUFFER, curl_errbuf);
 
 	if (curl_easy_perform(*curl) != 0) {
-		ast_log(LOG_WARNING, "%s ('%s')\n", curl_errbuf, args.url);
+		ast_log(LOG_WARNING, "%s ('%s')\n", curl_errbuf, args->url);
 	}
 
 	/* Reset buffer to NULL so curl doesn't try to write to it when the
@@ -699,19 +715,19 @@ static int acf_curl_helper(struct ast_ch
 		AST_LIST_UNLOCK(list);
 	}
 
-	if (args.postdata) {
+	if (args->postdata) {
 		curl_easy_setopt(*curl, CURLOPT_POST, 0);
 	}
 
-	if (ast_str_strlen(str)) {
-		ast_str_trim_blanks(str);
+	if (args->cb_data.str && ast_str_strlen(args->cb_data.str)) {
+		ast_str_trim_blanks(args->cb_data.str);
 
-		ast_debug(3, "str='%s'\n", ast_str_buffer(str));
+		ast_debug(3, "CURL returned str='%s'\n", ast_str_buffer(args->cb_data.str));
 		if (hashcompat) {
-			char *remainder = ast_str_buffer(str);
+			char *remainder = ast_str_buffer(args->cb_data.str);
 			char *piece;
-			struct ast_str *fields = ast_str_create(ast_str_strlen(str) / 2);
-			struct ast_str *values = ast_str_create(ast_str_strlen(str) / 2);
+			struct ast_str *fields = ast_str_create(ast_str_strlen(args->cb_data.str) / 2);
+			struct ast_str *values = ast_str_create(ast_str_strlen(args->cb_data.str) / 2);
 			int rowcount = 0;
 			while (fields && values && (piece = strsep(&remainder, "&"))) {
 				char *name = strsep(&piece, "=");
@@ -725,49 +741,93 @@ static int acf_curl_helper(struct ast_ch
 				rowcount++;
 			}
 			pbx_builtin_setvar_helper(chan, "~ODBCFIELDS~", ast_str_buffer(fields));
-			if (buf) {
-				ast_copy_string(buf, ast_str_buffer(values), len);
-			} else {
-				ast_str_set(input_str, len, "%s", ast_str_buffer(values));
-			}
+			ast_str_set(&args->cb_data.str, 0, "%s", ast_str_buffer(values));
 			ast_free(fields);
 			ast_free(values);
-		} else {
-			if (buf) {
-				ast_copy_string(buf, ast_str_buffer(str), len);
-			} else {
-				ast_str_set(input_str, len, "%s", ast_str_buffer(str));
-			}
 		}
 		ret = 0;
 	}
-	ast_free(str);
 
-	if (chan)
+	if (chan) {
 		ast_autoservice_stop(chan);
+	}
 
 	return ret;
 }
 
-static int acf_curl_exec(struct ast_channel *chan, const char *cmd, char *info, char *buf, size_t len)
+static int acf_curl_exec(struct ast_channel *chan, const char *cmd, char *info, struct ast_str **buf, ssize_t len)
 {
-	return acf_curl_helper(chan, cmd, info, buf, NULL, len);
+	struct curl_args curl_params = { 0, };
+	int res;
+
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(url);
+		AST_APP_ARG(postdata);
+	);
+
+	AST_STANDARD_APP_ARGS(args, info);
+
+	if (ast_strlen_zero(info)) {
+		ast_log(LOG_WARNING, "CURL requires an argument (URL)\n");
+		return -1;
+	}
+
+	curl_params.url = args.url;
+	curl_params.postdata = args.postdata;
+	curl_params.cb_data.str = ast_str_create(16);
+	if (!curl_params.cb_data.str) {
+		return -1;
+	}
+
+	res = acf_curl_helper(chan, &curl_params);
+	ast_str_set(buf, len, "%s", ast_str_buffer(curl_params.cb_data.str));
+	ast_free(curl_params.cb_data.str);
+
+	return res;
 }
 
-static int acf_curl2_exec(struct ast_channel *chan, const char *cmd, char *info, struct ast_str **buf, ssize_t len)
+static int acf_curl_write(struct ast_channel *chan, const char *cmd, char *name, const char *value)
 {
-	return acf_curl_helper(chan, cmd, info, NULL, buf, len);
+	struct curl_args curl_params = { 0, };
+	int res;
+	char *args_value = ast_strdupa(value);
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(file_path);
+	);
+
+	AST_STANDARD_APP_ARGS(args, args_value);
+
+	if (ast_strlen_zero(name)) {
+		ast_log(LOG_WARNING, "CURL requires an argument (URL)\n");
+		return -1;
+	}
+
+	if (ast_strlen_zero(args.file_path)) {
+		ast_log(LOG_WARNING, "CURL requires a file to write\n");
+		return -1;
+	}
+
+	curl_params.url = name;
+	curl_params.cb_data.out_file = fopen(args.file_path, "w");
+	if (!curl_params.cb_data.out_file) {
+		ast_log(LOG_WARNING, "Failed to open file %s: %s (%d)\n",
+			args.file_path,
+			strerror(errno),
+			errno);
+		return -1;
+	}
+
+	res = acf_curl_helper(chan, &curl_params);
+
+	fclose(curl_params.cb_data.out_file);
+
+	return res;
 }
 
 static struct ast_custom_function acf_curl = {
 	.name = "CURL",
-	.synopsis = "Retrieves the contents of a URL",
-	.syntax = "CURL(url[,post-data])",
-	.desc =
-	"  url       - URL to retrieve\n"
-	"  post-data - Optional data to send as a POST (GET is default action)\n",
-	.read = acf_curl_exec,
-	.read2 = acf_curl2_exec,
+	.read2 = acf_curl_exec,
+	.write = acf_curl_write,
 };
 
 static struct ast_custom_function acf_curlopt = {
@@ -865,14 +925,7 @@ static int load_module(void)
 {
 	int res;
 
-	if (!ast_module_check("res_curl.so")) {
-		if (ast_load_resource("res_curl.so") != AST_MODULE_LOAD_SUCCESS) {
-			ast_log(LOG_ERROR, "Cannot load res_curl, so func_curl cannot be loaded\n");
-			return AST_MODULE_LOAD_DECLINE;
-		}
-	}
-
-	res = ast_custom_function_register(&acf_curl);
+	res = ast_custom_function_register_escalating(&acf_curl, AST_CFE_WRITE);
 	res |= ast_custom_function_register(&acf_curlopt);
 
 	AST_TEST_REGISTER(vulnerable_url);
@@ -881,8 +934,9 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Load external URL",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_REALTIME_DEPEND2,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_REALTIME_DEPEND2,
+	.requires = "res_curl",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_cut.c 1:16.1.1~dfsg-1/funcs/func_cut.c
--- 1:13.23.1~dfsg-2/funcs/func_cut.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_cut.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_db.c 1:16.1.1~dfsg-1/funcs/func_db.c
--- 1:13.23.1~dfsg-2/funcs/func_db.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_db.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_devstate.c 1:16.1.1~dfsg-1/funcs/func_devstate.c
--- 1:13.23.1~dfsg-2/funcs/func_devstate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_devstate.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_dialgroup.c 1:16.1.1~dfsg-1/funcs/func_dialgroup.c
--- 1:13.23.1~dfsg-2/funcs/func_dialgroup.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_dialgroup.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include "asterisk/module.h"
@@ -223,7 +221,8 @@ static int dialgroup_write(struct ast_ch
 		grhead = ao2_alloc(sizeof(*grhead), group_destroy);
 		if (!grhead)
 			return -1;
-		grhead->entries = ao2_container_alloc(37, entry_hash_fn, entry_cmp_fn);
+		grhead->entries = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+			entry_hash_fn, NULL, entry_cmp_fn);
 		if (!grhead->entries) {
 			ao2_ref(grhead, -1);
 			return -1;
@@ -238,7 +237,9 @@ static int dialgroup_write(struct ast_ch
 
 		/* Remove all existing */
 		ao2_ref(grhead->entries, -1);
-		if (!(grhead->entries = ao2_container_alloc(37, entry_hash_fn, entry_cmp_fn))) {
+		grhead->entries = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+			entry_hash_fn, NULL, entry_cmp_fn);
+		if (!grhead->entries) {
 			ao2_unlink(group_container, grhead);
 			ao2_ref(grhead, -1);
 			return -1;
@@ -299,7 +300,9 @@ static int load_module(void)
 	struct ast_db_entry *dbtree, *tmp;
 	char groupname[AST_MAX_EXTENSION], *ptr;
 
-	if ((group_container = ao2_container_alloc(37, group_hash_fn, group_cmp_fn))) {
+	group_container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		group_hash_fn, NULL, group_cmp_fn);
+	if (group_container) {
 		/* Refresh groups from astdb */
 		if ((dbtree = ast_db_gettree("dialgroup", NULL))) {
 			for (tmp = dbtree; tmp; tmp = tmp->next) {
diff -pruN 1:13.23.1~dfsg-2/funcs/func_dialplan.c 1:16.1.1~dfsg-1/funcs/func_dialplan.c
--- 1:13.23.1~dfsg-2/funcs/func_dialplan.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_dialplan.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -198,4 +196,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_enum.c 1:16.1.1~dfsg-1/funcs/func_enum.c
--- 1:13.23.1~dfsg-2/funcs/func_enum.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_enum.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -474,4 +472,9 @@ static int load_module(void)
 	return res;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "ENUM related dialplan functions");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ENUM related dialplan functions",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "enum",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_env.c 1:16.1.1~dfsg-1/funcs/func_env.c
--- 1:13.23.1~dfsg-2/funcs/func_env.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_env.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>   /* stat(2) */
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_extstate.c 1:16.1.1~dfsg-1/funcs/func_extstate.c
--- 1:13.23.1~dfsg-2/funcs/func_extstate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_extstate.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_frame_trace.c 1:16.1.1~dfsg-1/funcs/func_frame_trace.c
--- 1:13.23.1~dfsg-2/funcs/func_frame_trace.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_frame_trace.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -340,6 +338,15 @@ static void print_frame(struct ast_frame
 			/* Should never happen. */
 			ast_assert(0);
 			break;
+		case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+			ast_verbose("SubClass: STREAM_TOPOLOGY_REQUEST_CHANGE\n");
+			break;
+		case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+			ast_verbose("SubClass: STREAM_TOPOLOGY_CHANGED\n");
+			break;
+		case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
+			ast_verbose("SubClass: STREAM_TOPOLOGY_SOURCE_CHANGED\n");
+			break;
 		case AST_CONTROL_STREAM_STOP:
 			ast_verbose("SubClass: STREAM_STOP\n");
 			break;
diff -pruN 1:13.23.1~dfsg-2/funcs/func_global.c 1:16.1.1~dfsg-1/funcs/func_global.c
--- 1:13.23.1~dfsg-2/funcs/func_global.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_global.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_groupcount.c 1:16.1.1~dfsg-1/funcs/func_groupcount.c
--- 1:13.23.1~dfsg-2/funcs/func_groupcount.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_groupcount.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_hangupcause.c 1:16.1.1~dfsg-1/funcs/func_hangupcause.c
--- 1:13.23.1~dfsg-2/funcs/func_hangupcause.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_hangupcause.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_holdintercept.c 1:16.1.1~dfsg-1/funcs/func_holdintercept.c
--- 1:13.23.1~dfsg-2/funcs/func_holdintercept.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_holdintercept.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_iconv.c 1:16.1.1~dfsg-1/funcs/func_iconv.c
--- 1:13.23.1~dfsg-2/funcs/func_iconv.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_iconv.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <iconv.h>
 
diff -pruN 1:13.23.1~dfsg-2/funcs/func_jitterbuffer.c 1:16.1.1~dfsg-1/funcs/func_jitterbuffer.c
--- 1:13.23.1~dfsg-2/funcs/func_jitterbuffer.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_jitterbuffer.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/framehook.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_lock.c 1:16.1.1~dfsg-1/funcs/func_lock.c
--- 1:13.23.1~dfsg-2/funcs/func_lock.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_lock.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/lock.h"
@@ -217,12 +215,6 @@ static void *lock_broker(void *unused)
 	return NULL;
 }
 
-static int ast_channel_hash_cb(const void *obj, const int flags)
-{
-	const struct ast_channel *chan = obj;
-	return ast_str_case_hash(ast_channel_name(chan));
-}
-
 static int ast_channel_cmp_cb(void *obj, void *arg, int flags)
 {
 	struct ast_channel *chan = obj, *cmp_args = arg;
@@ -298,7 +290,9 @@ static int get_lock(struct ast_channel *
 			AST_LIST_UNLOCK(&locklist);
 			return -1;
 		}
-		if (!(current->requesters = ao2_container_alloc(1, ast_channel_hash_cb, ast_channel_cmp_cb))) {
+		current->requesters = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+			NULL, ast_channel_cmp_cb);
+		if (!current->requesters) {
 			ast_mutex_destroy(&current->mutex);
 			ast_cond_destroy(&current->cond);
 			ast_free(current);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_logic.c 1:16.1.1~dfsg-1/funcs/func_logic.c
--- 1:13.23.1~dfsg-2/funcs/func_logic.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_logic.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_math.c 1:16.1.1~dfsg-1/funcs/func_math.c
--- 1:13.23.1~dfsg-2/funcs/func_math.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_math.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_md5.c 1:16.1.1~dfsg-1/funcs/func_md5.c
--- 1:13.23.1~dfsg-2/funcs/func_md5.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_md5.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 
diff -pruN 1:13.23.1~dfsg-2/funcs/func_module.c 1:16.1.1~dfsg-1/funcs/func_module.c
--- 1:13.23.1~dfsg-2/funcs/func_module.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_module.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 
diff -pruN 1:13.23.1~dfsg-2/funcs/func_odbc.c 1:16.1.1~dfsg-1/funcs/func_odbc.c
--- 1:13.23.1~dfsg-2/funcs/func_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -102,7 +100,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 static char *config = "func_odbc.conf";
 
-#define DEFAULT_SINGLE_DB_CONNECTION 1
+#define DEFAULT_SINGLE_DB_CONNECTION 0
 
 static int single_db_connection;
 
@@ -1795,7 +1793,8 @@ static int load_module(void)
 	dsns = NULL;
 
 	if (single_db_connection) {
-		dsns = ao2_container_alloc(DSN_BUCKETS, dsn_hash, dsn_cmp);
+		dsns = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, DSN_BUCKETS,
+			dsn_hash, NULL, dsn_cmp);
 		if (!dsns) {
 			ast_log(LOG_ERROR, "Could not initialize DSN container\n");
 			ast_rwlock_unlock(&single_db_connection_lock);
@@ -1893,7 +1892,8 @@ static int reload(void)
 	}
 
 	if (single_db_connection) {
-		dsns = ao2_container_alloc(DSN_BUCKETS, dsn_hash, dsn_cmp);
+		dsns = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, DSN_BUCKETS,
+			dsn_hash, NULL, dsn_cmp);
 		if (!dsns) {
 			ast_log(LOG_ERROR, "Could not initialize DSN container\n");
 			ast_rwlock_unlock(&single_db_connection_lock);
@@ -1941,8 +1941,8 @@ reload_out:
 /* XXX need to revise usecount - set if query_lock is set */
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ODBC lookups",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_periodic_hook.c 1:16.1.1~dfsg-1/funcs/func_periodic_hook.c
--- 1:13.23.1~dfsg-2/funcs/func_periodic_hook.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_periodic_hook.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -139,8 +137,6 @@ static void hook_datastore_destroy_callb
 	ast_free(state->context);
 	ast_free(state->exten);
 	ast_free(state);
-
-	ast_module_unref(ast_module_info->self);
 }
 
 static const struct ast_datastore_info hook_datastore = {
@@ -307,7 +303,7 @@ static int init_hook(struct ast_channel
 	if (!(datastore = ast_datastore_alloc(&hook_datastore, uid))) {
 		return -1;
 	}
-	ast_module_ref(ast_module_info->self);
+
 	if (!(state = hook_state_alloc(context, exten, interval, hook_id))) {
 		ast_datastore_free(datastore);
 		return -1;
@@ -450,7 +446,8 @@ static int unload_module(void)
 {
 	ast_context_destroy(NULL, AST_MODULE);
 
-	return ast_custom_function_unregister(&hook_function);
+	ast_custom_function_unregister(&hook_function);
+	return 0;
 }
 
 static int load_module(void)
@@ -465,37 +462,36 @@ static int load_module(void)
 	/*
 	 * Based on a handy recipe from the Asterisk Cookbook.
 	 */
-	ast_add_extension(context_name, 1, exten_name, 1, "", "",
+	res = ast_add_extension(context_name, 1, exten_name, 1, "", "",
 			"Set", "EncodedChannel=${CUT(HOOK_CHANNEL,-,1-2)}",
 			NULL, AST_MODULE);
-	ast_add_extension(context_name, 1, exten_name, 2, "", "",
+	res |= ast_add_extension(context_name, 1, exten_name, 2, "", "",
 			"Set", "GROUP_NAME=${EncodedChannel}${HOOK_ID}",
 			NULL, AST_MODULE);
-	ast_add_extension(context_name, 1, exten_name, 3, "", "",
+	res |= ast_add_extension(context_name, 1, exten_name, 3, "", "",
 			"Set", "GROUP(periodic-hook)=${GROUP_NAME}",
 			NULL, AST_MODULE);
-	ast_add_extension(context_name, 1, exten_name, 4, "", "", "ExecIf",
+	res |= ast_add_extension(context_name, 1, exten_name, 4, "", "", "ExecIf",
 			"$[${GROUP_COUNT(${GROUP_NAME}@periodic-hook)} > 1]?Hangup()",
 			NULL, AST_MODULE);
-	ast_add_extension(context_name, 1, exten_name, 5, "", "",
+	res |= ast_add_extension(context_name, 1, exten_name, 5, "", "",
 			"Set", "ChannelToSpy=${URIDECODE(${EncodedChannel})}",
 			NULL, AST_MODULE);
-	ast_add_extension(context_name, 1, exten_name, 6, "", "",
+	res |= ast_add_extension(context_name, 1, exten_name, 6, "", "",
 			"ChanSpy", "${ChannelToSpy},qEB", NULL, AST_MODULE);
 
-	res = ast_add_extension(context_name, 1, beep_exten, 1, "", "",
+	res |= ast_add_extension(context_name, 1, beep_exten, 1, "", "",
 			"Answer", "", NULL, AST_MODULE);
 	res |= ast_add_extension(context_name, 1, beep_exten, 2, "", "",
 			"Playback", "beep", NULL, AST_MODULE);
 
-	res = ast_custom_function_register_escalating(&hook_function, AST_CFE_BOTH);
+	res |= ast_custom_function_register_escalating(&hook_function, AST_CFE_BOTH);
 
-	if (!res) {
-		/* For Optional API. */
-		ast_module_shutdown_ref(ast_module_info->self);
+	if (res) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
 	}
-
-	return res ? AST_MODULE_LOAD_DECLINE : AST_MODULE_LOAD_SUCCESS;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
 int AST_OPTIONAL_API_NAME(ast_beep_start)(struct ast_channel *chan,
@@ -520,7 +516,7 @@ int AST_OPTIONAL_API_NAME(ast_beep_stop)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "Periodic dialplan hooks.",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_pitchshift.c 1:16.1.1~dfsg-1/funcs/func_pitchshift.c
--- 1:13.23.1~dfsg-2/funcs/func_pitchshift.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_pitchshift.c	2018-12-26 18:45:13.000000000 +0000
@@ -64,8 +64,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_pjsip_aor.c 1:16.1.1~dfsg-1/funcs/func_pjsip_aor.c
--- 1:13.23.1~dfsg-2/funcs/func_pjsip_aor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_pjsip_aor.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <pjlib.h>
 
@@ -183,4 +181,9 @@ static int load_module(void)
 	return ast_custom_function_register(&pjsip_aor_function);
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Get information about a PJSIP AOR");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Get information about a PJSIP AOR",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_pjsip_contact.c 1:16.1.1~dfsg-1/funcs/func_pjsip_contact.c
--- 1:13.23.1~dfsg-2/funcs/func_pjsip_contact.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_pjsip_contact.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <pjlib.h>
 
@@ -149,7 +147,7 @@ static int pjsip_contact_function_read(s
 	if (!strcmp(args.field_name, "status")) {
 		ast_str_set(buf, len, "%s", ast_sip_get_contact_status_label(contact_status ? contact_status->status : UNKNOWN));
 	} else if (!strcmp(args.field_name, "rtt")) {
-		if (!contact_status || contact_status->status == UNKNOWN) {
+		if (!contact_status || contact_status->status != AVAILABLE) {
 			ast_str_set(buf, len, "%s", "N/A");
 		} else {
 			ast_str_set(buf, len, "%" PRId64, contact_status->rtt);
@@ -200,4 +198,9 @@ static int load_module(void)
 	return ast_custom_function_register(&pjsip_contact_function);
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Get information about a PJSIP contact");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Get information about a PJSIP contact",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_pjsip_endpoint.c 1:16.1.1~dfsg-1/funcs/func_pjsip_endpoint.c
--- 1:13.23.1~dfsg-2/funcs/func_pjsip_endpoint.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_pjsip_endpoint.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <pjlib.h>
 
@@ -158,4 +156,9 @@ static int load_module(void)
 	return ast_custom_function_register(&pjsip_endpoint_function);
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Get information about a PJSIP endpoint");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Get information about a PJSIP endpoint",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/funcs/func_presencestate.c 1:16.1.1~dfsg-1/funcs/func_presencestate.c
--- 1:13.23.1~dfsg-2/funcs/func_presencestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_presencestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_rand.c 1:16.1.1~dfsg-1/funcs/func_rand.c
--- 1:13.23.1~dfsg-2/funcs/func_rand.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_rand.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_realtime.c 1:16.1.1~dfsg-1/funcs/func_realtime.c
--- 1:13.23.1~dfsg-2/funcs/func_realtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_realtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_sha1.c 1:16.1.1~dfsg-1/funcs/func_sha1.c
--- 1:13.23.1~dfsg-2/funcs/func_sha1.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_sha1.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 
diff -pruN 1:13.23.1~dfsg-2/funcs/func_shell.c 1:16.1.1~dfsg-1/funcs/func_shell.c
--- 1:13.23.1~dfsg-2/funcs/func_shell.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_shell.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_sorcery.c 1:16.1.1~dfsg-1/funcs/func_sorcery.c
--- 1:13.23.1~dfsg-2/funcs/func_sorcery.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_sorcery.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/app.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_speex.c 1:16.1.1~dfsg-1/funcs/func_speex.c
--- 1:13.23.1~dfsg-2/funcs/func_speex.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_speex.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <speex/speex_preprocess.h>
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_sprintf.c 1:16.1.1~dfsg-1/funcs/func_sprintf.c
--- 1:13.23.1~dfsg-2/funcs/func_sprintf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_sprintf.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_srv.c 1:16.1.1~dfsg-1/funcs/func_srv.c
--- 1:13.23.1~dfsg-2/funcs/func_srv.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_srv.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/srv.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_strings.c 1:16.1.1~dfsg-1/funcs/func_strings.c
--- 1:13.23.1~dfsg-2/funcs/func_strings.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_strings.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 #include <ctype.h>
 
@@ -1091,10 +1089,33 @@ static int array(struct ast_channel *cha
 	return 0;
 }
 
+static const char *get_key(const struct ast_str *prefix, const struct ast_var_t *var)
+{
+	const char *prefix_name = ast_str_buffer(prefix);
+	const char *var_name = ast_var_name(var);
+	int prefix_len;
+	int var_len;
+
+	if (ast_strlen_zero(var_name)) {
+		return NULL;
+	}
+
+	prefix_len = ast_str_strlen(prefix);
+	var_len = strlen(var_name);
+
+	/*
+	 * Make sure we only match on non-empty, hash function created keys. If valid
+	 * then return a pointer to the variable that's just after the prefix.
+	 */
+	return var_len > (prefix_len + 1) && var_name[var_len - 1] == '~' &&
+		strncmp(prefix_name, var_name, prefix_len) == 0 ? var_name + prefix_len : NULL;
+}
+
 static int hashkeys_read(struct ast_channel *chan, const char *cmd, char *data, char *buf, size_t len)
 {
 	struct ast_var_t *newvar;
 	struct ast_str *prefix = ast_str_alloca(80);
+	size_t buf_len;
 
 	if (!chan) {
 		ast_log(LOG_WARNING, "No channel was provided to %s function.\n", cmd);
@@ -1105,15 +1126,19 @@ static int hashkeys_read(struct ast_chan
 	memset(buf, 0, len);
 
 	AST_LIST_TRAVERSE(ast_channel_varshead(chan), newvar, entries) {
-		if (strncmp(ast_str_buffer(prefix), ast_var_name(newvar), ast_str_strlen(prefix)) == 0) {
-			/* Copy everything after the prefix */
-			strncat(buf, ast_var_name(newvar) + ast_str_strlen(prefix), len - strlen(buf) - 1);
-			/* Trim the trailing ~ */
+		const char *key = get_key(prefix, newvar);
+
+		if (key) {
+			strncat(buf, key, len - strlen(buf) - 1);
+			/* Replace the trailing ~ */
 			buf[strlen(buf) - 1] = ',';
 		}
 	}
 	/* Trim the trailing comma */
-	buf[strlen(buf) - 1] = '\0';
+	buf_len = strlen(buf);
+	if (buf_len) {
+		buf[buf_len - 1] = '\0';
+	}
 	return 0;
 }
 
@@ -1121,7 +1146,6 @@ static int hashkeys_read2(struct ast_cha
 {
 	struct ast_var_t *newvar;
 	struct ast_str *prefix = ast_str_alloca(80);
-	char *tmp;
 
 	if (!chan) {
 		ast_log(LOG_WARNING, "No channel was provided to %s function.\n", cmd);
@@ -1131,17 +1155,19 @@ static int hashkeys_read2(struct ast_cha
 	ast_str_set(&prefix, -1, HASH_PREFIX, data);
 
 	AST_LIST_TRAVERSE(ast_channel_varshead(chan), newvar, entries) {
-		if (strncmp(ast_str_buffer(prefix), ast_var_name(newvar), ast_str_strlen(prefix)) == 0) {
-			/* Copy everything after the prefix */
-			ast_str_append(buf, len, "%s", ast_var_name(newvar) + ast_str_strlen(prefix));
-			/* Trim the trailing ~ */
+		const char *key = get_key(prefix, newvar);
+
+		if (key) {
+			char *tmp;
+
+			ast_str_append(buf, len, "%s", key);
+			/* Replace the trailing ~ */
 			tmp = ast_str_buffer(*buf);
 			tmp[ast_str_strlen(*buf) - 1] = ',';
 		}
 	}
-	/* Trim the trailing comma */
-	tmp = ast_str_buffer(*buf);
-	tmp[ast_str_strlen(*buf) - 1] = '\0';
+
+	ast_str_truncate(*buf, -1);
 	return 0;
 }
 
diff -pruN 1:13.23.1~dfsg-2/funcs/func_sysinfo.c 1:16.1.1~dfsg-1/funcs/func_sysinfo.c
--- 1:13.23.1~dfsg-2/funcs/func_sysinfo.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_sysinfo.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #if defined(HAVE_SYSINFO)
 #include <sys/sysinfo.h>
 #endif
diff -pruN 1:13.23.1~dfsg-2/funcs/func_talkdetect.c 1:16.1.1~dfsg-1/funcs/func_talkdetect.c
--- 1:13.23.1~dfsg-2/funcs/func_talkdetect.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_talkdetect.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_timeout.c 1:16.1.1~dfsg-1/funcs/func_timeout.c
--- 1:13.23.1~dfsg-2/funcs/func_timeout.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_timeout.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_uri.c 1:16.1.1~dfsg-1/funcs/func_uri.c
--- 1:13.23.1~dfsg-2/funcs/func_uri.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_uri.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_version.c 1:16.1.1~dfsg-1/funcs/func_version.c
--- 1:13.23.1~dfsg-2/funcs/func_version.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_version.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_vmcount.c 1:16.1.1~dfsg-1/funcs/func_vmcount.c
--- 1:13.23.1~dfsg-2/funcs/func_vmcount.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_vmcount.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dirent.h>
 
 #include "asterisk/file.h"
diff -pruN 1:13.23.1~dfsg-2/funcs/func_volume.c 1:16.1.1~dfsg-1/funcs/func_volume.c
--- 1:13.23.1~dfsg-2/funcs/func_volume.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/funcs/func_volume.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/.gitreview 1:16.1.1~dfsg-1/.gitreview
--- 1:13.23.1~dfsg-2/.gitreview	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/.gitreview	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,5 @@
 [gerrit]
-defaultbranch=13
+defaultbranch=16
 #
 # Intentional padding to ensure it is possible to point a commit
 # to an alternative gerrit server/repository without breaking
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/acl.h 1:16.1.1~dfsg-1/include/asterisk/acl.h
--- 1:13.23.1~dfsg-2/include/asterisk/acl.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/acl.h	2018-12-26 18:45:13.000000000 +0000
@@ -382,24 +382,6 @@ const char *ast_tos2str(unsigned int tos
 struct ast_ha *ast_named_acl_find(const char *name, int *is_realtime, int *is_undefined);
 
 /*!
- * \brief Initialize and configure the named ACL system.
- *
- * \details
- * This function will prepare the named ACL system for use.
- * For this reason, it needs to be called before other things that use ACLs are initialized.
- */
-int ast_named_acl_init(void);
-
-/*!
- * \brief reload/reconfigure the named ACL system.
- *
- * \details
- * This function is designed to trigger an event upon a successful reload that may update
- * ACL consumers.
- */
-int ast_named_acl_reload(void);
-
-/*!
  * \brief a \ref stasis_message_type for changes against a named ACL or the set of all named ACLs
  * \since 12
  *
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/agi.h 1:16.1.1~dfsg-1/include/asterisk/agi.h
--- 1:13.23.1~dfsg-2/include/asterisk/agi.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/agi.h	2018-12-26 18:45:13.000000000 +0000
@@ -82,13 +82,12 @@ AST_OPTIONAL_API(int, ast_agi_register,
  *
  * Unregisters an AGI command.
  *
- * \param mod Pointer to the module_info structure for the module that is unregistering the command
  * \param cmd Pointer to the descriptor for the command
  * \return 1 on success, 0 if the command was not already registered
  *
  */
 AST_OPTIONAL_API(int, ast_agi_unregister,
-		 (struct ast_module *mod, agi_command *cmd),
+		 (agi_command *cmd),
 		 { return AST_OPTIONAL_API_UNAVAILABLE; });
 
 /*!
@@ -116,7 +115,6 @@ AST_OPTIONAL_API(int, ast_agi_register_m
  * Unregisters a group of AGI commands, provided as an array of struct agi_command
  * entries.
  *
- * \param mod Pointer to the module_info structure for the module that is unregistering the commands
  * \param cmd Pointer to the first entry in the array of command descriptors
  * \param len Length of the array (use the ARRAY_LEN macro to determine this easily)
  * \return 0 on success, -1 on failure, AST_OPTIONAL_API_UNAVAILABLE if res_agi is not loaded
@@ -125,7 +123,7 @@ AST_OPTIONAL_API(int, ast_agi_register_m
  * remaining commands in the array; it will not reregister the already-unregistered commands.
  */
 AST_OPTIONAL_API(int, ast_agi_unregister_multiple,
-		 (struct ast_module *mod, struct agi_command *cmd, unsigned int len),
+		 (struct agi_command *cmd, unsigned int len),
 		 { return AST_OPTIONAL_API_UNAVAILABLE; });
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/app.h 1:16.1.1~dfsg-1/include/asterisk/app.h
--- 1:13.23.1~dfsg-2/include/asterisk/app.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/app.h	2018-12-26 18:45:13.000000000 +0000
@@ -585,7 +585,7 @@ int ast_vm_is_registered(void);
 int __ast_vm_register(const struct ast_vm_functions *vm_table, struct ast_module *module);
 
 /*! \brief See \ref __ast_vm_register() */
-#define ast_vm_register(vm_table) __ast_vm_register(vm_table, ast_module_info ? ast_module_info->self : NULL)
+#define ast_vm_register(vm_table) __ast_vm_register(vm_table, AST_MODULE_SELF)
 
 /*!
  * \brief Unregister the specified voicemail provider
@@ -654,7 +654,7 @@ int ast_vm_greeter_is_registered(void);
 int __ast_vm_greeter_register(const struct ast_vm_greeter_functions *vm_table, struct ast_module *module);
 
 /*! \brief See \ref __ast_vm_greeter_register() */
-#define ast_vm_greeter_register(vm_table) __ast_vm_greeter_register(vm_table, ast_module_info ? ast_module_info->self : NULL)
+#define ast_vm_greeter_register(vm_table) __ast_vm_greeter_register(vm_table, AST_MODULE_SELF)
 
 /*!
  * \brief Unregister the specified voicemail greeter provider
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/ari.h 1:16.1.1~dfsg-1/include/asterisk/ari.h
--- 1:13.23.1~dfsg-2/include/asterisk/ari.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/ari.h	2018-12-26 18:45:13.000000000 +0000
@@ -96,6 +96,8 @@ struct ast_ari_response {
 	/*! HTTP response code.
 	 * See http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html */
 	int response_code;
+	/*! File descriptor for whatever file we want to respond with */
+	int fd;
 	/*! Corresponding text for the response code */
 	const char *response_text; /* Shouldn't http.c handle this? */
 	/*! Flag to indicate that no further response is needed */
@@ -266,14 +268,4 @@ void ast_ari_response_created(struct ast
  */
 void ast_ari_response_alloc_failed(struct ast_ari_response *response);
 
-/*! \brief Determines whether the res_ari module is loaded */
-#define CHECK_ARI_MODULE_LOADED()				\
-	do {							\
-		if (!ast_module_check("res_ari.so")		\
-			|| !ast_ari_oom_json()) {	\
-			return AST_MODULE_LOAD_DECLINE;		\
-		}						\
-	} while(0)
-
-
 #endif /* _ASTERISK_ARI_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/astmm.h 1:16.1.1~dfsg-1/include/asterisk/astmm.h
--- 1:13.23.1~dfsg-2/include/asterisk/astmm.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/astmm.h	2018-12-26 18:45:13.000000000 +0000
@@ -17,117 +17,296 @@
  */
 
 /*! \file
- * \brief Asterisk memory usage debugging
- * This file provides headers for MALLOC_DEBUG, a define used for tracking down
- * memory leaks.  It should never be \#included directly; always use the
- * MALLOC_DEBUG definition in menuselect to activate those functions.
+ * \brief Asterisk memory management routines
+ *
+ * This file should never be \#included directly, it is included
+ * by asterisk.h.
  */
 
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 #ifndef _ASTERISK_ASTMM_H
 #define _ASTERISK_ASTMM_H
+/* IWYU pragma: private, include "asterisk.h" */
 
-#ifndef STANDALONE
-
-#define __AST_DEBUG_MALLOC
-
-#include "asterisk.h"
-
-/* Include these now to prevent them from being needed later */
-#include <sys/types.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <stdarg.h>
-
-/* Undefine any macros */
-#undef malloc
-#undef calloc
-#undef realloc
-#undef strdup
-#undef strndup
-#undef asprintf
-#undef vasprintf
-#undef free
-
-void *ast_std_malloc(size_t size);
-void *ast_std_calloc(size_t nmemb, size_t size);
+void *ast_std_malloc(size_t size) attribute_malloc;
+void *ast_std_calloc(size_t nmemb, size_t size) attribute_malloc;
 void *ast_std_realloc(void *ptr, size_t size);
 void ast_std_free(void *ptr);
+
+/*!
+ * \brief free() wrapper
+ *
+ * ast_free_ptr should be used when a function pointer for free() needs to be passed
+ * as the argument to a function. Otherwise, astmm will cause seg faults.
+ */
 void ast_free_ptr(void *ptr);
 
-void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func);
-void *__ast_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func);
-void *__ast_malloc(size_t size, const char *file, int lineno, const char *func);
+void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func) attribute_malloc;
+void *__ast_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func) attribute_malloc;
+void *__ast_malloc(size_t size, const char *file, int lineno, const char *func) attribute_malloc;
 void __ast_free(void *ptr, const char *file, int lineno, const char *func);
 void *__ast_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func);
-char *__ast_strdup(const char *s, const char *file, int lineno, const char *func);
-char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func);
+char *__ast_strdup(const char *s, const char *file, int lineno, const char *func) attribute_malloc;
+char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func) attribute_malloc;
 int __ast_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
 	__attribute__((format(printf, 5, 6)));
 int __ast_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
 	__attribute__((format(printf, 2, 0)));
-void __ast_mm_init_phase_1(void);
-void __ast_mm_init_phase_2(void);
 
+/* The __ast_repl functions should not used from Asterisk sources, they are exposed
+ * for use by ASTMM_REDIRECT and bundled pjproject. */
+void *__ast_repl_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func) attribute_malloc;
+void *__ast_repl_malloc(size_t size, const char *file, int lineno, const char *func) attribute_malloc;
+void *__ast_repl_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func);
+char *__ast_repl_strdup(const char *s, const char *file, int lineno, const char *func) attribute_malloc;
+char *__ast_repl_strndup(const char *s, size_t n, const char *file, int lineno, const char *func) attribute_malloc;
+int __ast_repl_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
+	__attribute__((format(printf, 5, 6)));
+int __ast_repl_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
+	__attribute__((format(printf, 2, 0)));
 
-/* Provide our own definitions */
-#define calloc(a,b) \
-	__ast_calloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief ASTMM_LIBC can be defined to control the meaning of standard allocators.
+ *
+ * \note The standard allocators effected by this compiler define are:
+ *    malloc, calloc, realloc, strdup, strndup, asprintf, vasprintf and free.
+ *
+ * @{
+ */
 
-#define ast_calloc(a,b) \
-	__ast_calloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief Produce compiler errors if standard allocators are used.
+ *
+ * \note This is the default option, and in most cases the correct option.
+ * Any use of standard allocators will cause an error, even if those uses
+ * are in unused static inline header functions.
+ */
+#define ASTMM_BLOCK    0
 
-#define ast_calloc_cache(a,b) \
-	__ast_calloc_cache(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief Redirect standard allocators to use Asterisk functions.
+ *
+ * \note This option is used in some cases instead of changing the
+ * existing source to use Asterisk functions.  New code should
+ * generally avoid this option, except where it's needed to work
+ * with situations where switching the code is unreasonable, such
+ * as output from code generators that are hard coded to use
+ * standard functions.
+ */
+#define ASTMM_REDIRECT 1
 
-#define malloc(a) \
-	__ast_malloc(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief Standard allocators are used directly.
+ *
+ * \note This option is needed when including 3rd party headers with calls
+ * to standard allocators from inline functions.  Using ASTMM_REDIRECT in
+ * this situation could result in an object being allocated by malloc and
+ * freed by ast_free, or the reverse.
+ */
+#define ASTMM_IGNORE   2
 
-#define ast_malloc(a) \
-	__ast_malloc(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * }@
+ */
 
-#define free(a) \
-	__ast_free(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#if !defined(ASTMM_LIBC)
+/* BLOCK libc allocators by default. */
+#define ASTMM_LIBC ASTMM_BLOCK
+#endif
 
-#define ast_free(a) \
-	__ast_free(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#if ASTMM_LIBC == ASTMM_IGNORE
+/* Don't touch the libc functions. */
+#else
 
-#define realloc(a,b) \
-	__ast_realloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/* Undefine any macros */
+#undef malloc
+#undef calloc
+#undef realloc
+#undef strdup
+#undef strndup
+#undef asprintf
+#undef vasprintf
+#undef free
 
-#define ast_realloc(a,b) \
-	__ast_realloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#if ASTMM_LIBC == ASTMM_REDIRECT
 
+/* Redefine libc functions to our own versions */
+#define calloc(nmemb, size) \
+	__ast_repl_calloc(nmemb, size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define malloc(size) \
+	__ast_repl_malloc(size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define free(ptr) \
+	__ast_free(ptr, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define realloc(ptr, size) \
+	__ast_repl_realloc(ptr, size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define strdup(s) \
+	__ast_repl_strdup(s, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define strndup(s, n) \
+	__ast_repl_strndup(s, n, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define asprintf(strp, format, args...) \
+	__ast_repl_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, strp, format, args)
+#define vasprintf(strp, format, ap) \
+	__ast_repl_vasprintf(strp, format, ap, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#elif ASTMM_LIBC == ASTMM_BLOCK
+
+/* Redefine libc functions to cause compile errors */
+#define calloc(a, b) \
+	Do_not_use_calloc__use_ast_calloc->fail(a, b)
+#define malloc(a) \
+	Do_not_use_malloc__use_ast_malloc->fail(a)
+#define free(a) \
+	Do_not_use_free__use_ast_free_or_ast_std_free_for_remotely_allocated_memory->fail(a)
+#define realloc(a, b) \
+	Do_not_use_realloc__use_ast_realloc->fail(a, b)
 #define strdup(a) \
-	__ast_strdup(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+	Do_not_use_strdup__use_ast_strdup->fail(a)
+#define strndup(a, b) \
+	Do_not_use_strndup__use_ast_strndup->fail(a, b)
+#define asprintf(a, b, c...) \
+	Do_not_use_asprintf__use_ast_asprintf->fail(a, b, c)
+#define vasprintf(a, b, c) \
+	Do_not_use_vasprintf__use_ast_vasprintf->fail(a, b, c)
 
-#define ast_strdup(a) \
-	__ast_strdup(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#else
+#error "Unacceptable value for the macro ASTMM_LIBC"
+#endif
 
-#define strndup(a,b) \
-	__ast_strndup(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#endif
 
-#define ast_strndup(a,b) \
-	__ast_strndup(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/* Provide our own definition for ast_free */
 
-#define asprintf(a, b, c...) \
-	__ast_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, a, b, c)
+#define ast_free(a) \
+	__ast_free(a, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ast_asprintf(a, b, c...) \
-	__ast_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, a, b, c)
+/*!
+ * \brief A wrapper for malloc()
+ *
+ * ast_malloc() is a wrapper for malloc() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * The argument and return value are the same as malloc()
+ */
+#define ast_malloc(len) \
+	__ast_malloc((len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \brief A wrapper for calloc()
+ *
+ * ast_calloc() is a wrapper for calloc() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * The arguments and return value are the same as calloc()
+ */
+#define ast_calloc(num, len) \
+	__ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \brief A wrapper for calloc() for use in cache pools
+ *
+ * ast_calloc_cache() is a wrapper for calloc() that will generate an Asterisk log
+ * message in the case that the allocation fails. When memory debugging is in use,
+ * the memory allocated by this function will be marked as 'cache' so it can be
+ * distinguished from normal memory allocations.
+ *
+ * The arguments and return value are the same as calloc()
+ */
+#define ast_calloc_cache(num, len) \
+	__ast_calloc_cache((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define vasprintf(a,b,c) \
-	__ast_vasprintf(a,b,c,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief A wrapper for realloc()
+ *
+ * ast_realloc() is a wrapper for realloc() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * The arguments and return value are the same as realloc()
+ */
+#define ast_realloc(p, len) \
+	__ast_realloc((p), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ast_vasprintf(a,b,c) \
-	__ast_vasprintf(a,b,c,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*!
+ * \brief A wrapper for strdup()
+ *
+ * ast_strdup() is a wrapper for strdup() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * ast_strdup(), unlike strdup(), can safely accept a NULL argument. If a NULL
+ * argument is provided, ast_strdup will return NULL without generating any
+ * kind of error log message.
+ *
+ * The argument and return value are the same as strdup()
+ */
+#define ast_strdup(str) \
+	__ast_strdup((str), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#endif /* !STANDALONE */
+/*!
+ * \brief A wrapper for strndup()
+ *
+ * ast_strndup() is a wrapper for strndup() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * ast_strndup(), unlike strndup(), can safely accept a NULL argument for the
+ * string to duplicate. If a NULL argument is provided, ast_strdup will return
+ * NULL without generating any kind of error log message.
+ *
+ * The arguments and return value are the same as strndup()
+ */
+#define ast_strndup(str, len) \
+	__ast_strndup((str), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \brief A wrapper for asprintf()
+ *
+ * ast_asprintf() is a wrapper for asprintf() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * The arguments and return value are the same as asprintf()
+ */
+#define ast_asprintf(ret, fmt, ...) \
+	__ast_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, (ret), (fmt), __VA_ARGS__)
+
+/*!
+ * \brief A wrapper for vasprintf()
+ *
+ * ast_vasprintf() is a wrapper for vasprintf() that will generate an Asterisk log
+ * message in the case that the allocation fails.
+ *
+ * The arguments and return value are the same as vasprintf()
+ */
+#define ast_vasprintf(ret, fmt, ap) \
+	__ast_vasprintf((ret), (fmt), (ap), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+  \brief call __builtin_alloca to ensure we get gcc builtin semantics
+  \param size The size of the buffer we want allocated
+
+  This macro will attempt to allocate memory from the stack.  If it fails
+  you won't get a NULL returned, but a SEGFAULT if you're lucky.
+*/
+#define ast_alloca(size) __builtin_alloca(size)
+
+#if !defined(ast_strdupa) && defined(__GNUC__)
+/*!
+ * \brief duplicate a string in memory from the stack
+ * \param s The string to duplicate
+ *
+ * This macro will duplicate the given string.  It returns a pointer to the stack
+ * allocatted memory for the new string.
+ */
+#define ast_strdupa(s)                                                    \
+	(__extension__                                                    \
+	({                                                                \
+		const char *__old = (s);                                  \
+		size_t __len = strlen(__old) + 1;                         \
+		char *__new = __builtin_alloca(__len);                    \
+		memcpy (__new, __old, __len);                             \
+		__new;                                                    \
+	}))
+#endif
 
 #else
 #error "NEVER INCLUDE astmm.h DIRECTLY!!"
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/astobj2.h 1:16.1.1~dfsg-1/include/asterisk/astobj2.h
--- 1:13.23.1~dfsg-2/include/asterisk/astobj2.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/astobj2.h	2018-12-26 18:45:13.000000000 +0000
@@ -19,6 +19,7 @@
 
 #include "asterisk/compat.h"
 #include "asterisk/lock.h"
+#include "asterisk/linkedlists.h"
 #include "asterisk/inline_api.h"
 
 /*! \file
@@ -92,7 +93,8 @@ parameters. At the moment, this is done
 
     struct ao2_container *c;
 
-    c = ao2_container_alloc(MAX_BUCKETS, my_hash_fn, my_cmp_fn);
+    c = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_BUCKETS,
+        my_hash_fn, NULL, my_cmp_fn);
     \endcode
 
 where
@@ -108,7 +110,7 @@ A container knows little or nothing abou
 other than the fact that they have been created by ao2_alloc().
 All knowledge of the (user-defined) internals of the objects
 is left to the (user-supplied) functions passed as arguments
-to ao2_container_alloc().
+to ao2_container_alloc_hash().
 
 If we want to insert an object in a container, we should
 initialize its fields -- especially, those used by my_hash_fn() --
@@ -141,31 +143,11 @@ list. However there is no ordering among
 /*
 \note DEBUGGING REF COUNTS BIBLE:
 An interface to help debug refcounting is provided
-in this package. It is dependent on the REF_DEBUG macro being
-defined via menuselect and in using variants of the normal ao2_xxxx
-function that are named ao2_t_xxxx instead, with an extra argument,
-a string that will be printed out into the refs log file when the
-refcount for an object is changed.
-
-  these ao2_t_xxx variants are provided:
-
-ao2_t_alloc(arg1, arg2, arg3)
-ao2_t_ref(arg1,arg2,arg3)
-ao2_t_container_alloc(arg1,arg2,arg3,arg4)
-ao2_t_link(arg1, arg2, arg3)
-ao2_t_unlink(arg1, arg2, arg3)
-ao2_t_callback(arg1,arg2,arg3,arg4,arg5)
-ao2_t_find(arg1,arg2,arg3,arg4)
-ao2_t_iterator_next(arg1, arg2)
-
-If you study each argument list, you will see that these functions all have
-one extra argument than their ao2_xxx counterpart. The last argument in
-each case is supposed to be a string pointer, a "tag", that should contain
-enough of an explanation, that you can pair operations that increment the
-ref count, with operations that are meant to decrement the refcount.
+in this package. It is dependent on the refdebug being enabled in
+asterisk.conf.
 
-Each of these calls will generate at least one line of output in the refs
-log files. These lines look like this:
+Each of the reference manipulations will generate one line of output in the refs
+log file. These lines look like this:
 ...
 0x8756f00,+1,1234,chan_sip.c,22240,load_module,**constructor**,allocate users
 0x86e3408,+1,1234,chan_sip.c,22241,load_module,**constructor**,allocate peers
@@ -210,61 +192,29 @@ follows:
   context for the ref change. Note that any subsequent columns are
   considered to be part of this tag.
 
-Sometimes you have some helper functions to do object ref/unref
+Sometimes you have some helper functions to do object create/ref/unref
 operations. Using these normally hides the place where these
 functions were called. To get the location where these functions
-were called to appear in /refs, you can do this sort of thing:
+were called to appear in refs log, you can do this sort of thing:
 
-#ifdef REF_DEBUG
-#define dialog_ref(arg1,arg2) dialog_ref_debug((arg1),(arg2), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define dialog_unref(arg1,arg2) dialog_unref_debug((arg1),(arg2), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-static struct sip_pvt *dialog_ref_debug(struct sip_pvt *p, const char *tag, const char *file, int line, const char *func)
-{
-	if (p) {
-		ao2_ref_debug(p, 1, tag, file, line, func);
-	} else {
-		ast_log(LOG_ERROR, "Attempt to Ref a null pointer\n");
-	}
-	return p;
-}
+#define my_t_alloc(data,tag)  my_alloc_debug((data), tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define my_alloc(data)        my_t_alloc((data), "")
 
-static struct sip_pvt *dialog_unref_debug(struct sip_pvt *p, const char *tag, const char *file, int line, const char *func)
+static struct mydata *my_alloc_debug(void *data,
+	const char *tag, const char *file, int line, const char *func)
 {
-	if (p) {
-		ao2_ref_debug(p, -1, tag, file, line, func);
-	}
-	return NULL;
-}
-#else
-static struct sip_pvt *dialog_ref(struct sip_pvt *p, const char *tag)
-{
-	if (p) {
-		ao2_ref(p, 1);
-	} else {
-		ast_log(LOG_ERROR, "Attempt to Ref a null pointer\n");
-	}
-	return p;
-}
+	struct mydata *p;
 
-static struct sip_pvt *dialog_unref(struct sip_pvt *p, const char *tag)
-{
+	p = __ao2_alloc(sizeof(*p), NULL, AO2_ALLOC_OPT_LOCK_MUTEX, tag, file, line, func);
 	if (p) {
-		ao2_ref(p, -1);
+		p->data = data;
 	}
-	return NULL;
+	return p;
 }
-#endif
-
-In the above code, note that the "normal" helper funcs call ao2_ref() as
-normal, and the "helper" functions call ao2_ref_debug directly with the
-file, function, and line number info provided. You might find this
-well worth the effort to help track these function calls in the code.
 
 To find out why objects are not destroyed (a common bug), you can
-edit the source file to use the ao2_t_* variants, enable REF_DEBUG
-in menuselect, and add a descriptive tag to each call. Recompile,
-and run Asterisk, exit asterisk with "core stop gracefully", which should
-result in every object being destroyed.
+enable refdebug in asterisk.conf.  Run asterisk, exit with "core stop gracefully".
+This should result in every object being destroyed.
 
 Then, you can "sort -k 1 {AST_LOG_DIR}/refs > x1" to get a sorted list of
 all the objects, or you can use "contrib/script/refcounter.py" to scan
@@ -419,6 +369,13 @@ enum ao2_alloc_opts {
 	AO2_ALLOC_OPT_LOCK_NOLOCK = (2 << 0),
 	/*! The ao2 object locking option field mask. */
 	AO2_ALLOC_OPT_LOCK_MASK = (3 << 0),
+	/*!
+	 * \internal The ao2 object uses a separate object for locking.
+	 *
+	 * \note This option is used internally by ao2_alloc_with_lockobj and
+	 * should never be passed directly to ao2_alloc.
+	 */
+	AO2_ALLOC_OPT_LOCK_OBJ = AO2_ALLOC_OPT_LOCK_MASK,
 };
 
 /*!
@@ -439,52 +396,45 @@ enum ao2_alloc_opts {
  * - the returned pointer cannot be free()'d or realloc()'ed;
  *   rather, we just call ao2_ref(o, -1);
  *
+ * \note refdebug logging is skipped if debug_msg is NULL
+ *
  * @{
  */
 
-#if defined(REF_DEBUG)
-
 #define ao2_t_alloc_options(data_size, destructor_fn, options, debug_msg) \
-	__ao2_alloc_debug((data_size), (destructor_fn), (options), (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
+	__ao2_alloc((data_size), (destructor_fn), (options), (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_alloc_options(data_size, destructor_fn, options) \
-	__ao2_alloc_debug((data_size), (destructor_fn), (options), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
+	__ao2_alloc((data_size), (destructor_fn), (options), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 #define ao2_t_alloc(data_size, destructor_fn, debug_msg) \
-	__ao2_alloc_debug((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
+	__ao2_alloc((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_alloc(data_size, destructor_fn) \
-	__ao2_alloc_debug((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-
-#elif defined(__AST_DEBUG_MALLOC)
-
-#define ao2_t_alloc_options(data_size, destructor_fn, options, debug_msg) \
-	__ao2_alloc_debug((data_size), (destructor_fn), (options), (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-#define ao2_alloc_options(data_size, destructor_fn, options) \
-	__ao2_alloc_debug((data_size), (destructor_fn), (options), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#define ao2_t_alloc(data_size, destructor_fn, debug_msg) \
-	__ao2_alloc_debug((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, (debug_msg),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-#define ao2_alloc(data_size, destructor_fn) \
-	__ao2_alloc_debug((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#else
-
-#define ao2_t_alloc_options(data_size, destructor_fn, options, debug_msg) \
-	__ao2_alloc((data_size), (destructor_fn), (options))
-#define ao2_alloc_options(data_size, destructor_fn, options) \
-	__ao2_alloc((data_size), (destructor_fn), (options))
+	__ao2_alloc((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ao2_t_alloc(data_size, destructor_fn, debug_msg) \
-	__ao2_alloc((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX)
-#define ao2_alloc(data_size, destructor_fn) \
-	__ao2_alloc((data_size), (destructor_fn), AO2_ALLOC_OPT_LOCK_MUTEX)
+void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
-#endif
+/*! @} */
 
-void *__ao2_alloc_debug(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options, const char *tag,
-	const char *file, int line, const char *func, int ref_debug) attribute_warn_unused_result;
-void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options) attribute_warn_unused_result;
+/*!
+ * \since 14.1.0
+ * \brief Allocate and initialize an object with separate locking.
+ *
+ * \param data_size The sizeof() of the user-defined structure.
+ * \param destructor_fn The destructor function (can be NULL)
+ * \param lockobj A separate ao2 object that will provide locking.
+ * \param debug_msg An ao2 object debug tracing message.
+ * \return A pointer to user-data.
+ *
+ * \see \ref ao2_alloc for additional details.
+ *
+ * \note lockobj must be a valid AO2 object.
+ */
+#define ao2_alloc_with_lockobj(data_size, destructor_fn, lockobj, tag) \
+	__ao2_alloc_with_lockobj((data_size), (destructor_fn), (lockobj), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-/*! @} */
+void *__ao2_alloc_with_lockobj(size_t data_size, ao2_destructor_fn destructor_fn, void *lockobj,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
 /*! \brief
  * Reference/unreference an object and return the old refcount.
@@ -507,20 +457,12 @@ void *__ao2_alloc(size_t data_size, ao2_
  * can go away is when we release our reference, and it is
  * the last one in existence.
  *
+ * \note refdebug logging is skipped if tag is NULL
  * @{
  */
 
-#ifdef REF_DEBUG
-
-#define ao2_t_ref(o,delta,tag) __ao2_ref_debug((o), (delta), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_ref(o,delta)       __ao2_ref_debug((o), (delta), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#else
-
-#define ao2_t_ref(o,delta,tag) __ao2_ref((o), (delta))
-#define ao2_ref(o,delta)       __ao2_ref((o), (delta))
-
-#endif
+#define ao2_t_ref(o,delta,tag) __ao2_ref((o), (delta), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_ref(o,delta)       __ao2_ref((o), (delta), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Retrieve the ao2 options used to create the object.
@@ -550,8 +492,7 @@ unsigned int ao2_options_get(void *obj);
 #define ao2_bump(obj) \
 	ao2_t_bump((obj), "")
 
-int __ao2_ref_debug(void *o, int delta, const char *tag, const char *file, int line, const char *func);
-int __ao2_ref(void *o, int delta);
+int __ao2_ref(void *o, int delta, const char *tag, const char *file, int line, const char *func);
 
 /*!
  * \since 12.4.0
@@ -579,6 +520,184 @@ int __ao2_ref(void *o, int delta);
 
 /*! @} */
 
+/*! \brief ao2_weakproxy
+ *
+ * @{
+ */
+struct ao2_weakproxy_notification;
+typedef void (*ao2_weakproxy_notification_cb)(void *weakproxy, void *data);
+
+/*! \brief This struct should be opaque, but it's size is needed. */
+struct ao2_weakproxy {
+	AST_LIST_HEAD_NOLOCK(, ao2_weakproxy_notification) destroyed_cb;
+};
+
+/*! \brief Macro which must be used at the beginning of weakproxy capable objects.
+ *
+ * \note The primary purpose of user defined fields on weakproxy objects is to hold
+ *       immutable container keys for the real object.
+ */
+#define AO2_WEAKPROXY() struct ao2_weakproxy __weakproxy##__LINE__
+
+/*!
+ * \since 14.0.0
+ * \brief Allocate an ao2_weakproxy object
+ *
+ * \param data_size The sizeof() of the user-defined structure.
+ * \param destructor_fn The destructor function (can be NULL)
+ *
+ * \note "struct ao2_weakproxy" must be the first field of any object.
+ *       This can be done by using AO2_WEAKPROXY to declare your structure.
+ */
+void *__ao2_weakproxy_alloc(size_t data_size, ao2_destructor_fn destructor_fn,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
+
+#define ao2_weakproxy_alloc(data_size, destructor_fn) \
+	__ao2_weakproxy_alloc(data_size, destructor_fn, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#define ao2_t_weakproxy_alloc(data_size, destructor_fn, tag) \
+	__ao2_weakproxy_alloc(data_size, destructor_fn, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \since 14.0.0
+ * \brief Associate weakproxy with obj.
+ *
+ * \param weakproxy An object created by ao2_weakproxy_alloc.
+ * \param obj An ao2 object not created by ao2_weakproxy_alloc.
+ * \param flags OBJ_NOLOCK to avoid locking weakproxy.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ *
+ * \note obj must be newly created, this procedure is not thread safe
+ *       if any other code can reach obj before this procedure ends.
+ *
+ * \note weakproxy may be previously existing, but must not currently
+ *       have an object set.
+ *
+ * \note The only way to unset an object is for it to be destroyed.
+ *       Any call to this function while an object is already set will fail.
+ */
+int __ao2_weakproxy_set_object(void *weakproxy, void *obj, int flags,
+	const char *tag, const char *file, int line, const char *func);
+
+#define ao2_weakproxy_set_object(weakproxy, obj, flags) \
+	__ao2_weakproxy_set_object(weakproxy, obj, flags, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#define ao2_t_weakproxy_set_object(weakproxy, obj, flags, tag) \
+	__ao2_weakproxy_set_object(weakproxy, obj, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \since 14.0.0
+ * \brief Run ao2_t_ref on the object associated with weakproxy.
+ *
+ * \param weakproxy The weakproxy to read from.
+ * \param delta Value to add to the reference counter.
+ * \param flags OBJ_NOLOCK to avoid locking weakproxy.
+ *
+ * \retval -2 weakproxy is not a valid ao2_weakproxy.
+ * \retval -1 weakproxy has no associated object.
+ *
+ * \return The value of the reference counter before the operation.
+ */
+int __ao2_weakproxy_ref_object(void *weakproxy, int delta, int flags,
+	const char *tag, const char *file, int line, const char *func);
+
+#define ao2_t_weakproxy_ref_object(weakproxy, delta, flags, tag) \
+	__ao2_weakproxy_ref_object(weakproxy, delta, flags, \
+		tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#define ao2_weakproxy_ref_object(weakproxy, delta, flags) \
+	ao2_t_weakproxy_ref_object(weakproxy, delta, flags, "")
+
+/*!
+ * \since 14.0.0
+ * \brief Get the object associated with weakproxy.
+ *
+ * \param weakproxy The weakproxy to read from.
+ * \param flags OBJ_NOLOCK to avoid locking weakproxy.
+ *
+ * \return A reference to the object previously set by ao2_weakproxy_set_object.
+ * \retval NULL Either no object was set or the previously set object has been freed.
+ */
+void *__ao2_weakproxy_get_object(void *weakproxy, int flags,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
+
+#define ao2_weakproxy_get_object(weakproxy, flags) \
+	__ao2_weakproxy_get_object(weakproxy, flags, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#define ao2_t_weakproxy_get_object(weakproxy, flags, tag) \
+	__ao2_weakproxy_get_object(weakproxy, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \since 14.0.0
+ * \brief Request notification when weakproxy points to NULL.
+ *
+ * \param weakproxy The weak object
+ * \param cb Procedure to call when no real object is associated
+ * \param data Passed to cb
+ * \param flags OBJ_NOLOCK to avoid locking weakproxy.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ *
+ * \note Callbacks are run in the reverse order of subscriptions.
+ *
+ * \note This procedure will allow the same cb / data pair to be added to
+ *       the same weakproxy multiple times.
+ *
+ * \note It is the caller's responsibility to ensure that *data is valid
+ *       until after cb() is run or ao2_weakproxy_unsubscribe is called.
+ *
+ * \note If the weakproxy currently points to NULL the callback will be run immediately,
+ *       without being added to the subscriber list.
+ */
+int ao2_weakproxy_subscribe(void *weakproxy, ao2_weakproxy_notification_cb cb, void *data, int flags);
+
+/*!
+ * \since 14.0.0
+ * \brief Remove notification of real object destruction.
+ *
+ * \param weakproxy The weak object
+ * \param cb Callback to remove from destroy notification list
+ * \param data Data pointer to match
+ * \param flags OBJ_NOLOCK to avoid locking weakproxy.
+ *              OBJ_MULTIPLE to remove all copies of the same cb / data pair.
+ *
+ * \return The number of subscriptions removed.
+ * \retval 0 cb / data pair not found, nothing removed.
+ * \retval -1 Failure due to invalid parameters.
+ *
+ * \note Unless flags includes OBJ_MULTIPLE, this will only remove a single copy
+ *       of the cb / data pair.  If it was subscribed multiple times it must be
+ *       unsubscribed as many times.  The OBJ_MULTIPLE flag can be used to remove
+ *       matching subscriptions.
+ *
+ * \note When it's time to run callbacks they are copied to a temporary list so the
+ *       weakproxy can be unlocked before running.  That means it's possible for
+ *       this function to find nothing before the callback is run in another thread.
+ */
+int ao2_weakproxy_unsubscribe(void *weakproxy, ao2_weakproxy_notification_cb cb, void *data, int flags);
+
+/*!
+ * \since 14.0.0
+ * \brief Get the weakproxy attached to obj
+ *
+ * \param obj The object to retrieve a weakproxy from
+ *
+ * \return The weakproxy object
+ */
+void *__ao2_get_weakproxy(void *obj,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
+
+#define ao2_get_weakproxy(obj) \
+	__ao2_get_weakproxy(obj, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+#define ao2_t_get_weakproxy(obj, tag) \
+	__ao2_get_weakproxy(obj, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+/*! @} */
+
+
 /*! \brief Which lock to request. */
 enum ao2_lock_req {
 	/*! Request the mutex lock be acquired. */
@@ -634,6 +753,9 @@ int __ao2_trylock(void *a, enum ao2_lock
  * lock address, this allows you to correlate against
  * object address, to match objects to reported locks.
  *
+ * \warning AO2 lock objects do not include tracking fields when
+ * DEBUG_THREADS is not enabled.
+ *
  * \since 1.6.1
  */
 void *ao2_object_get_lockaddr(void *obj);
@@ -739,22 +861,11 @@ struct ao2_global_obj {
  *
  * \return Nothing
  */
-#ifdef REF_DEBUG
 #define ao2_t_global_obj_release(holder, tag)	\
 	__ao2_global_obj_replace_unref(&holder, NULL, (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 #define ao2_global_obj_release(holder)	\
 	__ao2_global_obj_replace_unref(&holder, NULL, "", __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 
-#else
-
-#define ao2_t_global_obj_release(holder, tag)	\
-	__ao2_global_obj_replace_unref(&holder, NULL, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#define ao2_global_obj_release(holder)	\
-	__ao2_global_obj_replace_unref(&holder, NULL, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#endif
-
-void __ao2_global_obj_release(struct ao2_global_obj *holder, const char *tag, const char *file, int line, const char *func, const char *name);
-
 /*!
  * \brief Replace an ao2 object in the global holder.
  * \since 11.0
@@ -770,20 +881,11 @@ void __ao2_global_obj_release(struct ao2
  * \retval Reference to previous global ao2 object stored.
  * \retval NULL if no object available.
  */
-#ifdef REF_DEBUG
 #define ao2_t_global_obj_replace(holder, obj, tag)	\
 	__ao2_global_obj_replace(&holder, (obj), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 #define ao2_global_obj_replace(holder, obj)	\
 	__ao2_global_obj_replace(&holder, (obj), "", __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 
-#else
-
-#define ao2_t_global_obj_replace(holder, obj, tag)	\
-	__ao2_global_obj_replace(&holder, (obj), NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#define ao2_global_obj_replace(holder, obj)	\
-	__ao2_global_obj_replace(&holder, (obj), NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#endif
-
 void *__ao2_global_obj_replace(struct ao2_global_obj *holder, void *obj, const char *tag, const char *file, int line, const char *func, const char *name) attribute_warn_unused_result;
 
 /*!
@@ -802,20 +904,11 @@ void *__ao2_global_obj_replace(struct ao
  * \retval 0 The global object was previously empty
  * \retval 1 The global object was not previously empty
  */
-#ifdef REF_DEBUG
 #define ao2_t_global_obj_replace_unref(holder, obj, tag)	\
 	__ao2_global_obj_replace_unref(&holder, (obj), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 #define ao2_global_obj_replace_unref(holder, obj)	\
 	__ao2_global_obj_replace_unref(&holder, (obj), "", __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 
-#else
-
-#define ao2_t_global_obj_replace_unref(holder, obj, tag)	\
-	__ao2_global_obj_replace_unref(&holder, (obj), NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#define ao2_global_obj_replace_unref(holder, obj)	\
-	__ao2_global_obj_replace_unref(&holder, (obj), NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#endif
-
 int __ao2_global_obj_replace_unref(struct ao2_global_obj *holder, void *obj, const char *tag, const char *file, int line, const char *func, const char *name);
 
 /*!
@@ -828,20 +921,11 @@ int __ao2_global_obj_replace_unref(struc
  * \retval Reference to current ao2 object stored in the holder.
  * \retval NULL if no object available.
  */
-#ifdef REF_DEBUG
 #define ao2_t_global_obj_ref(holder, tag)	\
 	__ao2_global_obj_ref(&holder, (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 #define ao2_global_obj_ref(holder)	\
 	__ao2_global_obj_ref(&holder, "", __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
 
-#else
-
-#define ao2_t_global_obj_ref(holder, tag)	\
-	__ao2_global_obj_ref(&holder, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#define ao2_global_obj_ref(holder)	\
-	__ao2_global_obj_ref(&holder, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__, #holder)
-#endif
-
 void *__ao2_global_obj_ref(struct ao2_global_obj *holder, const char *tag, const char *file, int line, const char *func, const char *name) attribute_warn_unused_result;
 
 
@@ -853,20 +937,8 @@ and perform various operations on them.
 Internally, objects are stored in lists, hash tables or other
 data structures depending on the needs.
 
-\note NOTA BENE: at the moment the only container we support is the
-    hash table and its degenerate form, the list.
-
 Operations on container include:
 
-  -  c = \b ao2_container_alloc(size, hash_fn, cmp_fn)
-    allocate a container with desired size and default compare
-    and hash function
-         -The compare function returns an int, which
-         can be 0 for not found, CMP_STOP to stop end a traversal,
-         or CMP_MATCH if they are equal
-         -The hash function returns an int. The hash function
-         takes two argument, the object pointer and a flags field,
-
   -  \b ao2_find(c, arg, flags)
     returns zero or more elements matching a given criteria
     (specified as arg). 'c' is the container pointer. Flags
@@ -1214,7 +1286,12 @@ typedef int (ao2_sort_fn)(const void *ob
 /*@{ */
 struct ao2_container;
 
+#ifndef AST_IN_CORE
+/* These macros are removed from Asterisk 17.  They are still available to modules
+ * but should only be used by third party modules that have not been updated. */
+
 /*!
+ * \deprecated
  * \brief Allocate and initialize a hash container with the desired number of buckets.
  *
  * \details
@@ -1232,16 +1309,16 @@ struct ao2_container;
  * \note Destructor is set implicitly.
  * \note This is legacy container creation that is mapped to the new method.
  */
-
 #define ao2_t_container_alloc_options(options, n_buckets, hash_fn, cmp_fn, tag) \
 	ao2_t_container_alloc_hash((options), 0, (n_buckets), (hash_fn), NULL, (cmp_fn), (tag))
 #define ao2_container_alloc_options(options, n_buckets, hash_fn, cmp_fn) \
 	ao2_container_alloc_hash((options), 0, (n_buckets), (hash_fn), NULL, (cmp_fn))
 
-#define ao2_t_container_alloc(n_buckets, hash_fn, cmp_fn, tag) \
-	ao2_t_container_alloc_options(AO2_ALLOC_OPT_LOCK_MUTEX, (n_buckets), (hash_fn), (cmp_fn), (tag))
 #define ao2_container_alloc(n_buckets, hash_fn, cmp_fn) \
-	ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_MUTEX, (n_buckets), (hash_fn), (cmp_fn))
+	ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, (n_buckets), (hash_fn), NULL, (cmp_fn))
+#define ao2_t_container_alloc(n_buckets, hash_fn, cmp_fn, tag) \
+	ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, (n_buckets), (hash_fn), NULL, (cmp_fn), (tag))
+#endif
 
 /*!
  * \brief Allocate and initialize a hash container with the desired number of buckets.
@@ -1263,36 +1340,15 @@ struct ao2_container;
  * \note Destructor is set implicitly.
  */
 
-#if defined(REF_DEBUG)
-
-#define ao2_t_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_hash_debug((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-#define ao2_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn) \
-	__ao2_container_alloc_hash_debug((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-
-#elif defined(__AST_DEBUG_MALLOC)
-
-#define ao2_t_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_hash_debug((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-#define ao2_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn) \
-	__ao2_container_alloc_hash_debug((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#else
-
 #define ao2_t_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_hash((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn))
+	__ao2_container_alloc_hash((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_container_alloc_hash(ao2_options, container_options, n_buckets, hash_fn, sort_fn, cmp_fn) \
-	__ao2_container_alloc_hash((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn))
-
-#endif
+	__ao2_container_alloc_hash((ao2_options), (container_options), (n_buckets), (hash_fn), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 struct ao2_container *__ao2_container_alloc_hash(unsigned int ao2_options,
 	unsigned int container_options, unsigned int n_buckets, ao2_hash_fn *hash_fn,
-	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn) attribute_warn_unused_result;
-struct ao2_container *__ao2_container_alloc_hash_debug(unsigned int ao2_options,
-	unsigned int container_options, unsigned int n_buckets, ao2_hash_fn *hash_fn,
 	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug) attribute_warn_unused_result;
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
 /*!
  * \brief Allocate and initialize a list container.
@@ -1309,34 +1365,14 @@ struct ao2_container *__ao2_container_al
  * \note Implemented as a degenerate hash table.
  */
 
-#if defined(REF_DEBUG)
-
 #define ao2_t_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_list_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
+	__ao2_container_alloc_list((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_list_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-
-#elif defined(__AST_DEBUG_MALLOC)
-
-#define ao2_t_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_list_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-#define ao2_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_list_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#else
-
-#define ao2_t_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_list((ao2_options), (container_options), (sort_fn), (cmp_fn))
-#define ao2_container_alloc_list(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_list((ao2_options), (container_options), (sort_fn), (cmp_fn))
-
-#endif
+	__ao2_container_alloc_list((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 struct ao2_container *__ao2_container_alloc_list(unsigned int ao2_options,
-	unsigned int container_options, ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn) attribute_warn_unused_result;
-struct ao2_container *__ao2_container_alloc_list_debug(unsigned int ao2_options,
 	unsigned int container_options, ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug) attribute_warn_unused_result;
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
 /*!
  * \brief Allocate and initialize a red-black tree container.
@@ -1352,34 +1388,14 @@ struct ao2_container *__ao2_container_al
  * \note Destructor is set implicitly.
  */
 
-#if defined(REF_DEBUG)
-
-#define ao2_t_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_rbtree_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-#define ao2_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_rbtree_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-
-#elif defined(__AST_DEBUG_MALLOC)
-
 #define ao2_t_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_rbtree_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
+	__ao2_container_alloc_rbtree((ao2_options), (container_options), (sort_fn), (cmp_fn), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_rbtree_debug((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#else
-
-#define ao2_t_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn, tag) \
-	__ao2_container_alloc_rbtree((ao2_options), (container_options), (sort_fn), (cmp_fn))
-#define ao2_container_alloc_rbtree(ao2_options, container_options, sort_fn, cmp_fn) \
-	__ao2_container_alloc_rbtree((ao2_options), (container_options), (sort_fn), (cmp_fn))
-
-#endif
+	__ao2_container_alloc_rbtree((ao2_options), (container_options), (sort_fn), (cmp_fn), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 struct ao2_container *__ao2_container_alloc_rbtree(unsigned int ao2_options, unsigned int container_options,
-	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn) attribute_warn_unused_result;
-struct ao2_container *__ao2_container_alloc_rbtree_debug(unsigned int ao2_options, unsigned int container_options,
 	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug) attribute_warn_unused_result;
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
 /*! \brief
  * Returns the number of elements in a container.
@@ -1407,6 +1423,28 @@ int ao2_container_count(struct ao2_conta
 int ao2_container_dup(struct ao2_container *dest, struct ao2_container *src, enum search_flags flags);
 
 /*!
+ * \brief Copy object references associated with src container weakproxies into the dest container.
+ *
+ * \param dest Container to copy src strong object references into.
+ * \param src Container to copy all weak object references from.
+ * \param flags OBJ_NOLOCK if a lock is already held on both containers.
+ *    Otherwise, the src container is locked first.
+ *
+ * \pre The dest container must be empty.  If the duplication fails, the
+ * dest container will be returned empty.
+ *
+ * \note This can potentially be expensive because a malloc is
+ * needed for every object in the src container.
+ *
+ * \note Every object inside the container is locked by \ref ao2_weakproxy_get_object.
+ *       Any weakproxy in \ref src with no associated object is ignored.
+ *
+ * \retval 0 on success.
+ * \retval -1 on error.
+ */
+int ao2_container_dup_weakproxy_objs(struct ao2_container *dest, struct ao2_container *src, enum search_flags flags);
+
+/*!
  * \brief Create a clone/copy of the given container.
  * \since 11.0
  *
@@ -1419,24 +1457,13 @@ int ao2_container_dup(struct ao2_contain
  * \retval Clone container on success.
  * \retval NULL on error.
  */
-struct ao2_container *__ao2_container_clone(struct ao2_container *orig, enum search_flags flags) attribute_warn_unused_result;
-struct ao2_container *__ao2_container_clone_debug(struct ao2_container *orig, enum search_flags flags, const char *tag, const char *file, int line, const char *func, int ref_debug) attribute_warn_unused_result;
-#if defined(REF_DEBUG)
+struct ao2_container *__ao2_container_clone(struct ao2_container *orig, enum search_flags flags,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
-#define ao2_t_container_clone(orig, flags, tag)	__ao2_container_clone_debug(orig, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-#define ao2_container_clone(orig, flags)		__ao2_container_clone_debug(orig, flags, "", __FILE__, __LINE__, __PRETTY_FUNCTION__, 1)
-
-#elif defined(__AST_DEBUG_MALLOC)
-
-#define ao2_t_container_clone(orig, flags, tag)	__ao2_container_clone_debug(orig, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-#define ao2_container_clone(orig, flags)		__ao2_container_clone_debug(orig, flags, "", __FILE__, __LINE__, __PRETTY_FUNCTION__, 0)
-
-#else
-
-#define ao2_t_container_clone(orig, flags, tag)	__ao2_container_clone(orig, flags)
-#define ao2_container_clone(orig, flags)		__ao2_container_clone(orig, flags)
-
-#endif
+#define ao2_t_container_clone(orig, flags, tag) \
+	__ao2_container_clone(orig, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_container_clone(orig, flags) \
+	__ao2_container_clone(orig, flags, "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Print output.
@@ -1552,10 +1579,10 @@ void ao2_container_unregister(const char
  * \note This function automatically increases the reference count to account
  *       for the reference that the container now holds to the object.
  */
-#ifdef REF_DEBUG
-
-#define ao2_t_link(container, obj, tag)					__ao2_link_debug((container), (obj), 0, (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_link(container, obj)						__ao2_link_debug((container), (obj), 0, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_t_link(container, obj, tag) \
+	__ao2_link((container), (obj), 0, (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_link(container, obj) \
+	__ao2_link((container), (obj), 0, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Add an object to a container.
@@ -1575,21 +1602,13 @@ void ao2_container_unregister(const char
  * \note This function automatically increases the reference count to account
  *       for the reference that the container now holds to the object.
  */
-#define ao2_t_link_flags(container, obj, flags, tag)	__ao2_link_debug((container), (obj), (flags), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_link_flags(container, obj, flags)			__ao2_link_debug((container), (obj), (flags), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_t_link_flags(container, obj, flags, tag) \
+	__ao2_link((container), (obj), (flags), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_link_flags(container, obj, flags) \
+	__ao2_link((container), (obj), (flags), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#else
-
-#define ao2_t_link(container, obj, tag)					__ao2_link((container), (obj), 0)
-#define ao2_link(container, obj)						__ao2_link((container), (obj), 0)
-
-#define ao2_t_link_flags(container, obj, flags, tag)	__ao2_link((container), (obj), (flags))
-#define ao2_link_flags(container, obj, flags)			__ao2_link((container), (obj), (flags))
-
-#endif
-
-int __ao2_link_debug(struct ao2_container *c, void *obj_new, int flags, const char *tag, const char *file, int line, const char *func);
-int __ao2_link(struct ao2_container *c, void *obj_new, int flags);
+int __ao2_link(struct ao2_container *c, void *obj_new, int flags,
+	const char *tag, const char *file, int line, const char *func);
 
 /*!
  * \brief Remove an object from a container
@@ -1608,10 +1627,11 @@ int __ao2_link(struct ao2_container *c,
  *       reference to the object will be automatically released. (The
  *       refcount will be decremented).
  */
-#ifdef REF_DEBUG
 
-#define ao2_t_unlink(container, obj, tag)				__ao2_unlink_debug((container), (obj), 0, (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_unlink(container, obj)						__ao2_unlink_debug((container), (obj), 0, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_t_unlink(container, obj, tag) \
+	__ao2_unlink((container), (obj), 0, (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_unlink(container, obj) \
+	__ao2_unlink((container), (obj), 0, "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Remove an object from a container
@@ -1632,25 +1652,17 @@ int __ao2_link(struct ao2_container *c,
  *       refcount will be decremented).
  */
 
-#define ao2_t_unlink_flags(container, obj, flags, tag)	__ao2_unlink_debug((container), (obj), (flags), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_unlink_flags(container, obj, flags)			__ao2_unlink_debug((container), (obj), (flags), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#else
-
-#define ao2_t_unlink(container, obj, tag)				__ao2_unlink((container), (obj), 0)
-#define ao2_unlink(container, obj)						__ao2_unlink((container), (obj), 0)
-
-#define ao2_t_unlink_flags(container, obj, flags, tag)	__ao2_unlink((container), (obj), (flags))
-#define ao2_unlink_flags(container, obj, flags)			__ao2_unlink((container), (obj), (flags))
-
-#endif
-
-void *__ao2_unlink_debug(struct ao2_container *c, void *obj, int flags, const char *tag, const char *file, int line, const char *func);
-void *__ao2_unlink(struct ao2_container *c, void *obj, int flags);
+#define ao2_t_unlink_flags(container, obj, flags, tag) \
+	__ao2_unlink((container), (obj), (flags), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_unlink_flags(container, obj, flags) \
+	__ao2_unlink((container), (obj), (flags), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
+void *__ao2_unlink(struct ao2_container *c, void *obj, int flags,
+	const char *tag, const char *file, int line, const char *func);
 
 /*@} */
 
+
 /*! \brief
  * ao2_callback() is a generic function that applies cb_fn() to all objects
  * in a container, as described below.
@@ -1719,7 +1731,7 @@ void *__ao2_unlink(struct ao2_container
  * The use of flags argument is the follow:
  *
  *      OBJ_UNLINK              unlinks the object found
- *      OBJ_NODATA              on match, do return an object
+ *      OBJ_NODATA              on match, do not return an object
  *                              Callbacks use OBJ_NODATA as a default
  *                              functions such as find() do
  *      OBJ_MULTIPLE            return multiple matches
@@ -1733,26 +1745,15 @@ void *__ao2_unlink(struct ao2_container
  *
  * @{
  */
-#ifdef REF_DEBUG
 
 #define ao2_t_callback(c, flags, cb_fn, arg, tag) \
-	__ao2_callback_debug((c), (flags), (cb_fn), (arg), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+	__ao2_callback((c), (flags), (cb_fn), (arg), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_callback(c, flags, cb_fn, arg) \
-	__ao2_callback_debug((c), (flags), (cb_fn), (arg), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+	__ao2_callback((c), (flags), (cb_fn), (arg), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#else
-
-#define ao2_t_callback(c, flags, cb_fn, arg, tag) \
-	__ao2_callback((c), (flags), (cb_fn), (arg))
-#define ao2_callback(c, flags, cb_fn, arg) \
-	__ao2_callback((c), (flags), (cb_fn), (arg))
-
-#endif
-
-void *__ao2_callback_debug(struct ao2_container *c, enum search_flags flags,
+void *__ao2_callback(struct ao2_container *c, enum search_flags flags,
 	ao2_callback_fn *cb_fn, void *arg, const char *tag, const char *file, int line,
 	const char *func);
-void *__ao2_callback(struct ao2_container *c, enum search_flags flags, ao2_callback_fn *cb_fn, void *arg);
 
 /*! @} */
 
@@ -1771,50 +1772,38 @@ void *__ao2_callback(struct ao2_containe
  *
  * \see ao2_callback()
  */
-#ifdef REF_DEBUG
 
 #define ao2_t_callback_data(container, flags, cb_fn, arg, data, tag) \
-	__ao2_callback_data_debug((container), (flags), (cb_fn), (arg), (data), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+	__ao2_callback_data((container), (flags), (cb_fn), (arg), (data), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_callback_data(container, flags, cb_fn, arg, data) \
-	__ao2_callback_data_debug((container), (flags), (cb_fn), (arg), (data), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#else
+	__ao2_callback_data((container), (flags), (cb_fn), (arg), (data), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ao2_t_callback_data(container, flags, cb_fn, arg, data, tag) \
-	__ao2_callback_data((container), (flags), (cb_fn), (arg), (data))
-#define ao2_callback_data(container, flags, cb_fn, arg, data) \
-	__ao2_callback_data((container), (flags), (cb_fn), (arg), (data))
-
-#endif
-
-void *__ao2_callback_data_debug(struct ao2_container *c, enum search_flags flags,
+void *__ao2_callback_data(struct ao2_container *c, enum search_flags flags,
 	ao2_callback_data_fn *cb_fn, void *arg, void *data, const char *tag, const char *file,
 	int line, const char *func);
-void *__ao2_callback_data(struct ao2_container *c, enum search_flags flags,
-	ao2_callback_data_fn *cb_fn, void *arg, void *data);
 
 /*! ao2_find() is a short hand for ao2_callback(c, flags, c->cmp_fn, arg)
  * XXX possibly change order of arguments ?
  */
-#ifdef REF_DEBUG
 
 #define ao2_t_find(container, arg, flags, tag) \
-	__ao2_find_debug((container), (arg), (flags), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
+	__ao2_find((container), (arg), (flags), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_find(container, arg, flags) \
-	__ao2_find_debug((container), (arg), (flags), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#else
+	__ao2_find((container), (arg), (flags), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ao2_t_find(container, arg, flags, tag) \
-	__ao2_find((container), (arg), (flags))
-#define ao2_find(container, arg, flags) \
-	__ao2_find((container), (arg), (flags))
-
-#endif
+void *__ao2_find(struct ao2_container *c, const void *arg, enum search_flags flags,
+	const char *tag, const char *file, int line, const char *func);
 
-void *__ao2_find_debug(struct ao2_container *c, const void *arg, enum search_flags flags,
+/*!
+ * \brief Perform an ao2_find on a container with ao2_weakproxy objects, returning the real object.
+ *
+ * \note Only OBJ_SEARCH_* and OBJ_NOLOCK flags are supported by this function.
+ * \see ao2_callback for description of arguments.
+ */
+#define ao2_weakproxy_find(c, arg, flags, tag) \
+	__ao2_weakproxy_find(c, arg, flags, tag, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+void *__ao2_weakproxy_find(struct ao2_container *c, const void *arg, enum search_flags flags,
 	const char *tag, const char *file, int line, const char *func);
-void *__ao2_find(struct ao2_container *c, const void *arg, enum search_flags flags);
 
 /*! \brief
  *
@@ -1974,20 +1963,13 @@ void ao2_iterator_destroy(struct ao2_ite
 void ao2_iterator_destroy(struct ao2_iterator *iter);
 #endif	/* defined(TEST_FRAMEWORK) */
 
-#ifdef REF_DEBUG
+#define ao2_t_iterator_next(iter, tag) \
+	__ao2_iterator_next((iter), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ao2_iterator_next(iter) \
+	__ao2_iterator_next((iter), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define ao2_t_iterator_next(iter, tag) __ao2_iterator_next_debug((iter), (tag),  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ao2_iterator_next(iter)        __ao2_iterator_next_debug((iter), "",  __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#else
-
-#define ao2_t_iterator_next(iter, tag) __ao2_iterator_next((iter))
-#define ao2_iterator_next(iter)        __ao2_iterator_next((iter))
-
-#endif
-
-void *__ao2_iterator_next_debug(struct ao2_iterator *iter, const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
-void *__ao2_iterator_next(struct ao2_iterator *iter) attribute_warn_unused_result;
+void *__ao2_iterator_next(struct ao2_iterator *iter,
+	const char *tag, const char *file, int line, const char *func) attribute_warn_unused_result;
 
 /*!
  * \brief Restart an iteration.
@@ -2008,13 +1990,8 @@ void ao2_iterator_restart(struct ao2_ite
  * down a NULL */
 void __ao2_cleanup(void *obj);
 void __ao2_cleanup_debug(void *obj, const char *tag, const char *file, int line, const char *function);
-#ifdef REF_DEBUG
 #define ao2_cleanup(obj) __ao2_cleanup_debug((obj), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ao2_t_cleanup(obj, tag) __ao2_cleanup_debug((obj), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-#define ao2_cleanup(obj) __ao2_cleanup(obj)
-#define ao2_t_cleanup(obj, tag) __ao2_cleanup((obj))
-#endif
 void ao2_iterator_cleanup(struct ao2_iterator *iter);
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/astobj.h 1:16.1.1~dfsg-1/include/asterisk/astobj.h
--- 1:13.23.1~dfsg-2/include/asterisk/astobj.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/astobj.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,823 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Mark Spencer <markster@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- * \brief Object Model for Asterisk
- *
- * \deprecated Use astobj2.h instead
- */
-
-#ifndef _ASTERISK_ASTOBJ_H
-#define _ASTERISK_ASTOBJ_H
-
-#include "asterisk/lock.h"
-
-/*! \file
- * \brief A set of macros implementing objects and containers.
- * Macros are used for maximum performance, to support multiple inheritance,
- * and to be easily integrated into existing structures without additional
- * malloc calls, etc.
- *
- * These macros expect to operate on two different object types, ASTOBJs and
- * ASTOBJ_CONTAINERs.  These are not actual types, as any struct can be
- * converted into an ASTOBJ compatible object or container using the supplied
- * macros.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_object {
- *    ASTOBJ_COMPONENTS(struct sample_object);
- * };
- *
- * struct sample_container {
- *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_object);
- * } super_container;
- *
- * void sample_object_destroy(struct sample_object *obj)
- * {
- *    free(obj);
- * }
- *
- * int init_stuff()
- * {
- *    struct sample_object *obj1;
- *    struct sample_object *found_obj;
- *
- *    obj1 = malloc(sizeof(struct sample_object));
- *
- *    ASTOBJ_CONTAINER_INIT(&super_container);
- *
- *    ASTOBJ_INIT(obj1);
- *    ASTOBJ_WRLOCK(obj1);
- *    ast_copy_string(obj1->name, "obj1", sizeof(obj1->name));
- *    ASTOBJ_UNLOCK(obj1);
- *
- *    ASTOBJ_CONTAINER_LINK(&super_container, obj1);
- *
- *    found_obj = ASTOBJ_CONTAINER_FIND(&super_container, "obj1");
- *
- *    if(found_obj) {
- *       printf("Found object: %s", found_obj->name);
- *       ASTOBJ_UNREF(found_obj,sample_object_destroy);
- *    }
- *
- *    ASTOBJ_CONTAINER_DESTROYALL(&super_container,sample_object_destroy);
- *    ASTOBJ_CONTAINER_DESTROY(&super_container);
- *
- *    return 0;
- * }
- * \endcode
- */
-
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
-#define ASTOBJ_DEFAULT_NAMELEN 	80
-#define ASTOBJ_DEFAULT_BUCKETS	256
-#define ASTOBJ_DEFAULT_HASH		ast_strhash
-
-#define ASTOBJ_FLAG_MARKED	(1 << 0)		/* Object has been marked for future operation */
-
-/* C++ is simply a syntactic crutch for those who cannot think for themselves
-   in an object oriented way. */
-
-/*! \brief Lock an ASTOBJ for reading.
- */
-#define ASTOBJ_RDLOCK(object) ast_mutex_lock(&(object)->_lock)
-
-/*! \brief Lock an ASTOBJ for writing.
- */
-#define ASTOBJ_WRLOCK(object) ast_mutex_lock(&(object)->_lock)
-
-#define ASTOBJ_TRYWRLOCK(object) ast_mutex_trylock(&(object)->_lock)
-
-/*! \brief Unlock a locked object. */
-#define ASTOBJ_UNLOCK(object) ast_mutex_unlock(&(object)->_lock)
-
-#ifdef ASTOBJ_CONTAINER_HASHMODEL
-#define __ASTOBJ_HASH(type,hashes) \
-	type *next[hashes]
-#else
-#define __ASTOBJ_HASH(type,hashes) \
-	type *next[1]
-#endif
-
-/*! \brief Add ASTOBJ components to a struct (without locking support).
- *
- * \param type The datatype of the object.
- * \param namelen The length to make the name char array.
- * \param hashes The number of containers the object can be present in.
- *
- * This macro adds components to a struct to make it an ASTOBJ.  This macro
- * differs from ASTOBJ_COMPONENTS_FULL in that it does not create a mutex for
- * locking.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct {
- *    ASTOBJ_COMPONENTS_NOLOCK_FULL(struct sample_struct,1,1);
- * };
- * \endcode
- */
-#define ASTOBJ_COMPONENTS_NOLOCK_FULL(type,namelen,hashes) \
-	char name[namelen]; \
-	unsigned int refcount; \
-	unsigned int objflags; \
-	__ASTOBJ_HASH(type,hashes)
-
-/*! \brief Add ASTOBJ components to a struct (without locking support).
- *
- * \param type The datatype of the object.
- *
- * This macro works like #ASTOBJ_COMPONENTS_NOLOCK_FULL() except it only accepts a
- * type and uses default values for namelen and hashes.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_componets {
- *    ASTOBJ_COMPONENTS_NOLOCK(struct sample_struct);
- * };
- * \endcode
- */
-#define ASTOBJ_COMPONENTS_NOLOCK(type) \
-	ASTOBJ_COMPONENTS_NOLOCK_FULL(type,ASTOBJ_DEFAULT_NAMELEN,1)
-
-/*! \brief Add ASTOBJ components to a struct (with locking support).
- *
- * \param type The datatype of the object.
- *
- * This macro works like #ASTOBJ_COMPONENTS_NOLOCK() except it includes locking
- * support.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct {
- *    ASTOBJ_COMPONENTS(struct sample_struct);
- * };
- * \endcode
- */
-#define ASTOBJ_COMPONENTS(type) \
-	ASTOBJ_COMPONENTS_NOLOCK(type); \
-	ast_mutex_t _lock;
-
-/*! \brief Add ASTOBJ components to a struct (with locking support).
- *
- * \param type The datatype of the object.
- * \param namelen The length to make the name char array.
- * \param hashes The number of containers the object can be present in.
- *
- * This macro adds components to a struct to make it an ASTOBJ and includes
- * support for locking.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct {
- *    ASTOBJ_COMPONENTS_FULL(struct sample_struct,1,1);
- * };
- * \endcode
- */
-#define ASTOBJ_COMPONENTS_FULL(type,namelen,hashes) \
-	ASTOBJ_COMPONENTS_NOLOCK_FULL(type,namelen,hashes); \
-	ast_mutex_t _lock;
-
-/*! \brief Increment an object reference count.
- * \param object A pointer to the object to operate on.
- * \return The object.
- */
-#define ASTOBJ_REF(object) \
-	({ \
-		ASTOBJ_WRLOCK(object); \
-		(object)->refcount++; \
-		ASTOBJ_UNLOCK(object); \
-		(object); \
-	})
-
-/*! \brief Decrement the reference count on an object.
- *
- * \param object A pointer the object to operate on.
- * \param destructor The destructor to call if the object is no longer referenced.  It will be passed the pointer as an argument.
- *
- * This macro unreferences an object and calls the specfied destructor if the
- * object is no longer referenced.  The destructor should free the object if it
- * was dynamically allocated.
- */
-#define ASTOBJ_UNREF(object,destructor) \
-	do { \
-		int newcount = 0; \
-		ASTOBJ_WRLOCK(object); \
-		if (__builtin_expect((object)->refcount > 0, 1)) \
-			newcount = --((object)->refcount); \
-		else \
-			ast_log(AST_LOG_WARNING, "Unreferencing unreferenced (object)!\n"); \
-		ASTOBJ_UNLOCK(object); \
-		if (newcount == 0) { \
-			ast_mutex_destroy(&(object)->_lock); \
-			destructor((object)); \
-		} \
-		(object) = NULL; \
-	} while(0)
-
-/*! \brief Mark an ASTOBJ by adding the #ASTOBJ_FLAG_MARKED flag to its objflags mask.
- * \param object A pointer to the object to operate on.
- *
- * This macro "marks" an object.  Marked objects can later be unlinked from a container using
- * #ASTOBJ_CONTAINER_PRUNE_MARKED().
- *
- */
-#define ASTOBJ_MARK(object) \
-	do { \
-		ASTOBJ_WRLOCK(object); \
-		(object)->objflags |= ASTOBJ_FLAG_MARKED; \
-		ASTOBJ_UNLOCK(object); \
-	} while(0)
-
-/*! \brief Unmark an ASTOBJ by subtracting the #ASTOBJ_FLAG_MARKED flag from its objflags mask.
- * \param object A pointer to the object to operate on.
- */
-#define ASTOBJ_UNMARK(object) \
-	do { \
-		ASTOBJ_WRLOCK(object); \
-		(object)->objflags &= ~ASTOBJ_FLAG_MARKED; \
-		ASTOBJ_UNLOCK(object); \
-	} while(0)
-
-/*! \brief Initialize an object.
- * \param object A pointer to the object to operate on.
- *
- * \note This should only be used on objects that support locking (objects
- * created with #ASTOBJ_COMPONENTS() or #ASTOBJ_COMPONENTS_FULL())
- */
-#define ASTOBJ_INIT(object) \
-	do { \
-		ast_mutex_init(&(object)->_lock); \
-		object->name[0] = '\0'; \
-		object->refcount = 1; \
-	} while(0)
-
-/* Containers for objects -- current implementation is linked lists, but
-   should be able to be converted to hashes relatively easily */
-
-/*! \brief Lock an ASTOBJ_CONTAINER for reading.
- */
-#define ASTOBJ_CONTAINER_RDLOCK(container) ast_mutex_lock(&(container)->_lock)
-
-/*! \brief Lock an ASTOBJ_CONTAINER for writing.
- */
-#define ASTOBJ_CONTAINER_WRLOCK(container) ast_mutex_lock(&(container)->_lock)
-
-/*! \brief Unlock an ASTOBJ_CONTAINER. */
-#define ASTOBJ_CONTAINER_UNLOCK(container) ast_mutex_unlock(&(container)->_lock)
-
-#ifdef ASTOBJ_CONTAINER_HASHMODEL
-#error "Hash model for object containers not yet implemented!"
-#else
-/* Linked lists */
-
-/*! \brief Create a container for ASTOBJs (without locking support).
- *
- * \param type The type of objects the container will hold.
- * \param hashes Currently unused.
- * \param buckets Currently unused.
- *
- * This macro is used to create a container for ASTOBJs without locking
- * support.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_nolock_container {
- *    ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(struct sample_struct,1,1);
- * };
- * \endcode
- */
-#define ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(type,hashes,buckets) \
-	type *head
-
-/*! \brief Initialize a container.
- *
- * \param container A pointer to the container to initialize.
- * \param hashes Currently unused.
- * \param buckets Currently unused.
- *
- * This macro initializes a container.  It should only be used on containers
- * that support locking.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_container {
- *    ASTOBJ_CONTAINER_COMPONENTS_FULL(struct sample_struct,1,1);
- * } container;
- *
- * int func()
- * {
- *    ASTOBJ_CONTAINER_INIT_FULL(&container,1,1);
- * }
- * \endcode
- */
-#define ASTOBJ_CONTAINER_INIT_FULL(container,hashes,buckets) \
-	do { \
-		ast_mutex_init(&(container)->_lock); \
-	} while(0)
-
-/*! \brief Destroy a container.
- *
- * \param container A pointer to the container to destroy.
- * \param hashes Currently unused.
- * \param buckets Currently unused.
- *
- * This macro frees up resources used by a container.  It does not operate on
- * the objects in the container.  To unlink the objects from the container use
- * #ASTOBJ_CONTAINER_DESTROYALL().
- *
- * \note This macro should only be used on containers with locking support.
- */
-#define ASTOBJ_CONTAINER_DESTROY_FULL(container,hashes,buckets) \
-	do { \
-		ast_mutex_destroy(&(container)->_lock); \
-	} while(0)
-
-/*! \brief Iterate through the objects in a container.
- *
- * \param container A pointer to the container to traverse.
- * \param continue A condition to allow the traversal to continue.
- * \param eval A statement to evaluate in the iteration loop.
- *
- * This is macro is a little complicated, but it may help to think of it as a
- * loop.  Basically it iterates through the specfied containter as long as the
- * condition is met.  Two variables, iterator and next, are provided for use in
- * your \p eval statement.  See the sample code for an example.
- *
- * <b>Sample Usage:</b>
- * \code
- * ASTOBJ_CONTAINER_TRAVERSE(&sample_container,1, {
- *    ASTOBJ_RDLOCK(iterator);
- *    printf("Currently iterating over '%s'\n", iterator->name);
- *    ASTOBJ_UNLOCK(iterator);
- * } );
- * \endcode
- *
- * \code
- * ASTOBJ_CONTAINER_TRAVERSE(&sample_container,1, sample_func(iterator));
- * \endcode
- */
-#define ASTOBJ_CONTAINER_TRAVERSE(container,continue,eval) \
-	do { \
-		typeof((container)->head) iterator; \
-		typeof((container)->head) next; \
-		ASTOBJ_CONTAINER_RDLOCK(container); \
-		next = (container)->head; \
-		while((continue) && (iterator = next)) { \
-			next = iterator->next[0]; \
-			eval; \
-		} \
-		ASTOBJ_CONTAINER_UNLOCK(container); \
-	} while(0)
-
-/*! \brief Find an object in a container.
- *
- * \param container A pointer to the container to search.
- * \param namestr The name to search for.
- *
- * Use this function to find an object with the specfied name in a container.
- *
- * \note When the returned object is no longer in use, #ASTOBJ_UNREF() should
- * be used to free the additional reference created by this macro.
- *
- * \return A new reference to the object located or NULL if nothing is found.
- */
-#define ASTOBJ_CONTAINER_FIND(container,namestr) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
-			if (!(strcasecmp(iterator->name, (namestr)))) \
-				found = ASTOBJ_REF(iterator); \
-		} while (0)); \
-		found; \
-	})
-
-/*! \brief Find an object in a container.
- *
- * \param container A pointer to the container to search.
- * \param data The data to search for.
- * \param field The field/member of the container's objects to search.
- * \param hashfunc The hash function to use, currently not implemented.
- * \param hashoffset The hash offset to use, currently not implemented.
- * \param comparefunc The function used to compare the field and data values.
- *
- * This macro iterates through a container passing the specified field and data
- * elements to the specified comparefunc.  The function should return 0 when a match is found.
- *
- * \note When the returned object is no longer in use, #ASTOBJ_UNREF() should
- * be used to free the additional reference created by this macro.
- *
- * \return A pointer to the object located or NULL if nothing is found.
- */
-#define ASTOBJ_CONTAINER_FIND_FULL(container,data,field,hashfunc,hashoffset,comparefunc) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
-			ASTOBJ_RDLOCK(iterator); \
-			if (!(comparefunc(iterator->field, (data)))) { \
-				found = ASTOBJ_REF(iterator); \
-			} \
-			ASTOBJ_UNLOCK(iterator); \
-		} while (0)); \
-		found; \
-	})
-
-/*! \brief Empty a container.
- *
- * \param container A pointer to the container to operate on.
- * \param destructor A destructor function to call on each object.
- *
- * This macro loops through a container removing all the items from it using
- * #ASTOBJ_UNREF().  This does not destroy the container itself, use
- * #ASTOBJ_CONTAINER_DESTROY() for that.
- *
- * \note If any object in the container is only referenced by the container,
- * the destructor will be called for that object once it has been removed.
- */
-#define ASTOBJ_CONTAINER_DESTROYALL(container,destructor) \
-	do { \
-		typeof((container)->head) iterator; \
-		ASTOBJ_CONTAINER_WRLOCK(container); \
-		while((iterator = (container)->head)) { \
-			(container)->head = (iterator)->next[0]; \
-			ASTOBJ_UNREF(iterator,destructor); \
-		} \
-		ASTOBJ_CONTAINER_UNLOCK(container); \
-	} while(0)
-
-/*! \brief Remove an object from a container.
- *
- * \param container A pointer to the container to operate on.
- * \param obj A pointer to the object to remove.
- *
- * This macro iterates through a container and removes the specfied object if
- * it exists in the container.
- *
- * \note This macro does not destroy any objects, it simply unlinks
- * them from the list.  No destructors are called.
- *
- * \return The container's reference to the removed object or NULL if no
- * matching object was found.
- */
-#define ASTOBJ_CONTAINER_UNLINK(container,obj) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		typeof((container)->head) prev = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
-			if (iterator == obj) { \
-				found = iterator; \
-				found->next[0] = NULL; \
-				ASTOBJ_CONTAINER_WRLOCK(container); \
-				if (prev) \
-					prev->next[0] = next; \
-				else \
-					(container)->head = next; \
-				ASTOBJ_CONTAINER_UNLOCK(container); \
-			} \
-			prev = iterator; \
-		} while (0)); \
-		found; \
-	})
-
-/*! \brief Find and remove an object from a container.
- *
- * \param container A pointer to the container to operate on.
- * \param namestr The name of the object to remove.
- *
- * This macro iterates through a container and removes the first object with
- * the specfied name from the container.
- *
- * \note This macro does not destroy any objects, it simply unlinks
- * them.  No destructors are called.
- *
- * \return The container's reference to the removed object or NULL if no
- * matching object was found.
- */
-#define ASTOBJ_CONTAINER_FIND_UNLINK(container,namestr) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		typeof((container)->head) prev = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
-			if (!(strcasecmp(iterator->name, (namestr)))) { \
-				found = iterator; \
-				found->next[0] = NULL; \
-				ASTOBJ_CONTAINER_WRLOCK(container); \
-				if (prev) \
-					prev->next[0] = next; \
-				else \
-					(container)->head = next; \
-				ASTOBJ_CONTAINER_UNLOCK(container); \
-			} \
-			prev = iterator; \
-		} while (0)); \
-		found; \
-	})
-
-/*! \brief Find and remove an object in a container.
- *
- * \param container A pointer to the container to search.
- * \param data The data to search for.
- * \param field The field/member of the container's objects to search.
- * \param hashfunc The hash function to use, currently not implemented.
- * \param hashoffset The hash offset to use, currently not implemented.
- * \param comparefunc The function used to compare the field and data values.
- *
- * This macro iterates through a container passing the specified field and data
- * elements to the specified comparefunc.  The function should return 0 when a match is found.
- * If a match is found it is removed from the list.
- *
- * \note This macro does not destroy any objects, it simply unlinks
- * them.  No destructors are called.
- *
- * \return The container's reference to the removed object or NULL if no match
- * was found.
- */
-#define ASTOBJ_CONTAINER_FIND_UNLINK_FULL(container,data,field,hashfunc,hashoffset,comparefunc) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		typeof((container)->head) prev = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, !found, do { \
-			ASTOBJ_RDLOCK(iterator); \
-			if (!(comparefunc(iterator->field, (data)))) { \
-				found = iterator; \
-				found->next[0] = NULL; \
-				ASTOBJ_CONTAINER_WRLOCK(container); \
-				if (prev) \
-					prev->next[0] = next; \
-				else \
-					(container)->head = next; \
-				ASTOBJ_CONTAINER_UNLOCK(container); \
-			} \
-			ASTOBJ_UNLOCK(iterator); \
-			prev = iterator; \
-		} while (0)); \
-		found; \
-	})
-
-/*! \brief Add an object to the end of a container.
- *
- * \param container A pointer to the container to operate on.
- * \param newobj A pointer to the object to be added.
- *
- * This macro adds an object to the end of a container.
- */
-#define ASTOBJ_CONTAINER_LINK_END(container,newobj) \
-	do { \
-		typeof((container)->head) iterator; \
-		typeof((container)->head) next; \
-		typeof((container)->head) prev; \
-		ASTOBJ_CONTAINER_RDLOCK(container); \
-		prev = NULL; \
-		next = (container)->head; \
-		while((iterator = next)) { \
-			next = iterator->next[0]; \
-			prev = iterator; \
-		} \
-		if(prev) { \
-			ASTOBJ_CONTAINER_WRLOCK((container)); \
-			prev->next[0] = ASTOBJ_REF(newobj); \
-			(newobj)->next[0] = NULL; \
-			ASTOBJ_CONTAINER_UNLOCK((container)); \
-		} else { \
-			ASTOBJ_CONTAINER_LINK_START((container),(newobj)); \
-		} \
-		ASTOBJ_CONTAINER_UNLOCK((container)); \
-	} while(0)
-
-/*! \brief Add an object to the front of a container.
- *
- * \param container A pointer to the container to operate on.
- * \param newobj A pointer to the object to be added.
- *
- * This macro adds an object to the start of a container.
- */
-#define ASTOBJ_CONTAINER_LINK_START(container,newobj) \
-	do { \
-		ASTOBJ_CONTAINER_WRLOCK(container); \
-		(newobj)->next[0] = (container)->head; \
-		(container)->head = ASTOBJ_REF(newobj); \
-		ASTOBJ_CONTAINER_UNLOCK(container); \
-	} while(0)
-
-/*! \brief Remove an object from the front of a container.
- *
- * \param container A pointer to the container to operate on.
- *
- * This macro removes the first object in a container.
- *
- * \note This macro does not destroy any objects, it simply unlinks
- * them from the list.  No destructors are called.
- *
- * \return The container's reference to the removed object or NULL if no
- * matching object was found.
- */
-#define ASTOBJ_CONTAINER_UNLINK_START(container) \
-	({ \
-		typeof((container)->head) found = NULL; \
-		ASTOBJ_CONTAINER_WRLOCK(container); \
-	 	if((container)->head) { \
-	 		found = (container)->head; \
-	 		(container)->head = (container)->head->next[0]; \
-	 		found->next[0] = NULL; \
-	 	} \
-		ASTOBJ_CONTAINER_UNLOCK(container); \
-	 	found; \
-	})
-
-/*! \brief Prune marked objects from a container.
- *
- * \param container A pointer to the container to prune.
- * \param destructor A destructor function to call on each marked object.
- *
- * This macro iterates through the specfied container and prunes any marked
- * objects executing the specfied destructor if necessary.
- */
-#define ASTOBJ_CONTAINER_PRUNE_MARKED(container,destructor) \
-	do { \
-		typeof((container)->head) prev = NULL; \
-		ASTOBJ_CONTAINER_TRAVERSE(container, 1, do { \
-			ASTOBJ_RDLOCK(iterator); \
-			if (iterator->objflags & ASTOBJ_FLAG_MARKED) { \
-				ASTOBJ_CONTAINER_WRLOCK(container); \
-				if (prev) \
-					prev->next[0] = next; \
-				else \
-					(container)->head = next; \
-				ASTOBJ_CONTAINER_UNLOCK(container); \
-				ASTOBJ_UNLOCK(iterator); \
-				ASTOBJ_UNREF(iterator,destructor); \
-				continue; \
-			} \
-			ASTOBJ_UNLOCK(iterator); \
-			prev = iterator; \
-		} while (0)); \
-	} while(0)
-
-/*! \brief Add an object to a container.
- *
- * \param container A pointer to the container to operate on.
- * \param newobj A pointer to the object to be added.
- * \param data Currently unused.
- * \param field Currently unused.
- * \param hashfunc Currently unused.
- * \param hashoffset Currently unused.
- * \param comparefunc Currently unused.
- *
- * Currently this function adds an object to the head of the list.  One day it
- * will support adding objects atthe position specified using the various
- * options this macro offers.
- */
-#define ASTOBJ_CONTAINER_LINK_FULL(container,newobj,data,field,hashfunc,hashoffset,comparefunc) \
-	do { \
-		ASTOBJ_CONTAINER_WRLOCK(container); \
-		(newobj)->next[0] = (container)->head; \
-		(container)->head = ASTOBJ_REF(newobj); \
-		ASTOBJ_CONTAINER_UNLOCK(container); \
-	} while(0)
-
-#endif /* List model */
-
-/* Common to hash and linked list models */
-
-/*! \brief Create a container for ASTOBJs (without locking support).
- *
- * \param type The type of objects the container will hold.
- *
- * This macro is used to create a container for ASTOBJs without locking
- * support.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_nolock_container {
- *    ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(struct sample_struct);
- * };
- * \endcode
- */
-#define ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(type) \
-	ASTOBJ_CONTAINER_COMPONENTS_NOLOCK_FULL(type,1,ASTOBJ_DEFAULT_BUCKETS)
-
-
-/*! \brief Create a container for ASTOBJs (with locking support).
- *
- * \param type The type of objects the container will hold.
- *
- * This macro is used to create a container for ASTOBJs with locking support.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_container {
- *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_struct);
- * };
- * \endcode
- */
-#define ASTOBJ_CONTAINER_COMPONENTS(type) \
-	ast_mutex_t _lock; \
-	ASTOBJ_CONTAINER_COMPONENTS_NOLOCK(type)
-
-/*! \brief Initialize a container.
- *
- * \param container A pointer to the container to initialize.
- *
- * This macro initializes a container.  It should only be used on containers
- * that support locking.
- *
- * <b>Sample Usage:</b>
- * \code
- * struct sample_struct_container {
- *    ASTOBJ_CONTAINER_COMPONENTS(struct sample_struct);
- * } container;
- *
- * int func()
- * {
- *    ASTOBJ_CONTAINER_INIT(&container);
- * }
- * \endcode
- */
-#define ASTOBJ_CONTAINER_INIT(container) \
-	ASTOBJ_CONTAINER_INIT_FULL(container,1,ASTOBJ_DEFAULT_BUCKETS)
-
-/*! \brief Destroy a container.
- *
- * \param container A pointer to the container to destory.
- *
- * This macro frees up resources used by a container.  It does not operate on
- * the objects in the container.  To unlink the objects from the container use
- * #ASTOBJ_CONTAINER_DESTROYALL().
- *
- * \note This macro should only be used on containers with locking support.
- */
-#define ASTOBJ_CONTAINER_DESTROY(container) \
-	ASTOBJ_CONTAINER_DESTROY_FULL(container,1,ASTOBJ_DEFAULT_BUCKETS)
-
-/*! \brief Add an object to a container.
- *
- * \param container A pointer to the container to operate on.
- * \param newobj A pointer to the object to be added.
- *
- * Currently this macro adds an object to the head of a container.  One day it
- * should add an object in alphabetical order.
- */
-#define ASTOBJ_CONTAINER_LINK(container,newobj) \
-	ASTOBJ_CONTAINER_LINK_FULL(container,newobj,(newobj)->name,name,ASTOBJ_DEFAULT_HASH,0,strcasecmp)
-
-/*! \brief Mark all the objects in a container.
- * \param container A pointer to the container to operate on.
- */
-#define ASTOBJ_CONTAINER_MARKALL(container) \
-	ASTOBJ_CONTAINER_TRAVERSE(container, 1, ASTOBJ_MARK(iterator))
-
-/*! \brief Unmark all the objects in a container.
- * \param container A pointer to the container to operate on.
- */
-#define ASTOBJ_CONTAINER_UNMARKALL(container) \
-	ASTOBJ_CONTAINER_TRAVERSE(container, 1, ASTOBJ_UNMARK(iterator))
-
-/*! \brief Dump information about an object into a string.
- *
- * \param s A pointer to the string buffer to use.
- * \param slen The length of s.
- * \param obj A pointer to the object to dump.
- *
- * This macro dumps a text representation of the name, objectflags, and
- * refcount fields of an object to the specfied string buffer.
- */
-#define ASTOBJ_DUMP(s,slen,obj) \
-	snprintf((s),(slen),"name: %s\nobjflags: %u\nrefcount: %u\n\n", (obj)->name, (obj)->objflags, (obj)->refcount);
-
-/*! \brief Dump information about all the objects in a container to a file descriptor.
- *
- * \param fd The file descriptor to write to.
- * \param s A string buffer, same as #ASTOBJ_DUMP().
- * \param slen The length of s, same as #ASTOBJ_DUMP().
- * \param container A pointer to the container to dump.
- *
- * This macro dumps a text representation of the name, objectflags, and
- * refcount fields of all the objects in a container to the specified file
- * descriptor.
- */
-#define ASTOBJ_CONTAINER_DUMP(fd,s,slen,container) \
-	ASTOBJ_CONTAINER_TRAVERSE(container, 1, do { ASTOBJ_DUMP(s,slen,iterator); ast_cli(fd, "%s", s); } while(0))
-
-#if defined(__cplusplus) || defined(c_plusplus)
-}
-#endif
-
-#endif /* _ASTERISK_ASTOBJ_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/astosp.h 1:16.1.1~dfsg-1/include/asterisk/astosp.h
--- 1:13.23.1~dfsg-2/include/asterisk/astosp.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/astosp.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,31 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Mark Spencer <markster@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*!
- * \file
- * \brief Open Settlement Protocol (OSP)
- */
-
-#ifndef _ASTERISK_OSP_H
-#define _ASTERISK_OSP_H
-
-#define AST_OSP_SUCCESS	((char*)"SUCCESS")	/* Return status, success */
-#define AST_OSP_FAILED	((char*)"FAILED")	/* Return status, failed */
-#define AST_OSP_ERROR	((char*)"ERROR")	/* Return status, error */
-
-#endif /* _ASTERISK_OSP_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/autoconfig.h.in 1:16.1.1~dfsg-1/include/asterisk/autoconfig.h.in
--- 1:13.23.1~dfsg-2/include/asterisk/autoconfig.h.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/autoconfig.h.in	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,12 @@
 
 #ifndef ASTERISK_AUTOCONFIG_H
 #define ASTERISK_AUTOCONFIG_H
+/* IWYU pragma: private, include "asterisk.h" */
 
-#include "asterisk/buildopts.h"
-
+#include "asterisk/buildopts.h" /* IWYU pragma: export */
 
+/* Define to 'long' or 'long long' */
+#undef AST_JSON_INT_T
 
 /* Define to 1 if internal poll should be used. */
 #undef AST_POLL_COMPAT
@@ -66,6 +68,9 @@
 /* Define to 1 if you have the `asprintf' function. */
 #undef HAVE_ASPRINTF
 
+/* Define to 1 if you have the <assert.h> header file. */
+#undef HAVE_ASSERT_H
+
 /* Define to 1 if you have the `atan' function. */
 #undef HAVE_ATAN
 
@@ -119,6 +124,9 @@
    attribute. */
 #undef HAVE_ATTRIBUTE_warn_unused_result
 
+/* Define to 1 if you have the Beanstalk Job Queue library. */
+#undef HAVE_BEANSTALK
+
 /* Define to 1 if you have the Debug symbol decoding library. */
 #undef HAVE_BFD
 
@@ -128,6 +136,10 @@
 /* Define to 1 if you have the Bluetooth library. */
 #undef HAVE_BLUETOOTH
 
+/* Define to 1 if you have the file `bridges/bridge_softmix/include/hrirs.h'.
+   */
+#undef HAVE_BRIDGES_BRIDGE_SOFTMIX_INCLUDE_HRIRS_H
+
 /* Define to 1 if byteswap.h macros are available. */
 #undef HAVE_BYTESWAP_H
 
@@ -146,6 +158,9 @@
 /* Define to 1 if you have the `closefrom' function. */
 #undef HAVE_CLOSEFROM
 
+/* Define to 1 if you have the Codec 2 Audio Decoder/Encoder library. */
+#undef HAVE_CODEC2
+
 /* Define to 1 if you have the Corosync library. */
 #undef HAVE_COROSYNC
 
@@ -167,11 +182,14 @@
 /* Define to 1 if you have the 'crypt_r' function. */
 #undef HAVE_CRYPT_R
 
+/* Define to 1 if you have the <ctype.h> header file. */
+#undef HAVE_CTYPE_H
+
 /* Define to 1 if you have a functional curl library. */
 #undef HAVE_CURL
 
-/* Define to 1 if you have the curses library. */
-#undef HAVE_CURSES
+/* Define to 1 if your C compiler provides __atomic operations. */
+#undef HAVE_C_ATOMICS
 
 /* Define if your system has the DAHDI headers. */
 #undef HAVE_DAHDI
@@ -195,6 +213,9 @@
 /* Define to 1 if your system has the dladdr() GNU extension */
 #undef HAVE_DLADDR
 
+/* Define to 1 if you have the <dlfcn.h> header file. */
+#undef HAVE_DLFCN_H
+
 /* Define to 1 if you don't have `vprintf' but do have `_doprnt.' */
 #undef HAVE_DOPRNT
 
@@ -207,6 +228,9 @@
 /* Define to 1 if you have the `endpwent' function. */
 #undef HAVE_ENDPWENT
 
+/* Define to 1 if you have the <errno.h> header file. */
+#undef HAVE_ERRNO_H
+
 /* Define to 1 if you have the `euidaccess' function. */
 #undef HAVE_EUIDACCESS
 
@@ -241,6 +265,12 @@
 /* Define to 1 if you have the `ffsll' function. */
 #undef HAVE_FFSLL
 
+/* Define to 1 if you have the LIBFFTW3 library. */
+#undef HAVE_FFTW3
+
+/* Define to 1 if you have the <float.h> header file. */
+#undef HAVE_FLOAT_H
+
 /* Define to 1 if you have the `floor' function. */
 #undef HAVE_FLOOR
 
@@ -253,9 +283,6 @@
 /* Define to 1 if you have the `fmodl' function. */
 #undef HAVE_FMODL
 
-/* Define to 1 if you have the `fopencookie' function. */
-#undef HAVE_FOPENCOOKIE
-
 /* Define to 1 if you have the `fork' function. */
 #undef HAVE_FORK
 
@@ -268,10 +295,7 @@
 /* Define to 1 if you have the `ftruncate' function. */
 #undef HAVE_FTRUNCATE
 
-/* Define to 1 if you have the `funopen' function. */
-#undef HAVE_FUNOPEN
-
-/* Define to 1 if your GCC C compiler provides atomic operations. */
+/* Define to 1 if your GCC C compiler provides __sync atomic operations. */
 #undef HAVE_GCC_ATOMICS
 
 /* Define to 1 if you have the `getcwd' function. */
@@ -316,6 +340,9 @@
 /* Define if your system has the GMIME libraries. */
 #undef HAVE_GMIME
 
+/* Define to 1 if you have the <grp.h> header file. */
+#undef HAVE_GRP_H
+
 /* Define to indicate the GSM library */
 #undef HAVE_GSM
 
@@ -386,6 +413,9 @@
 /* Define if your system has the JANSSON libraries. */
 #undef HAVE_JANSSON
 
+/* Define if your system has JANSSON_BUNDLED */
+#undef HAVE_JANSSON_BUNDLED
+
 /* Define to 1 if you have the `kevent64' function. */
 #undef HAVE_KEVENT64
 
@@ -462,6 +492,12 @@
 /* Define to 1 if you have the <malloc.h> header file. */
 #undef HAVE_MALLOC_H
 
+/* Define to 1 if you have the `malloc_trim' function. */
+#undef HAVE_MALLOC_TRIM
+
+/* Define to 1 if you have the <math.h> header file. */
+#undef HAVE_MATH_H
+
 /* Define to 1 if you have the `memchr' function. */
 #undef HAVE_MEMCHR
 
@@ -504,9 +540,6 @@
 /* Define to 1 if you have the Network Broadcast Sound library. */
 #undef HAVE_NBS
 
-/* Define to 1 if you have the ncurses library. */
-#undef HAVE_NCURSES
-
 /* Define to 1 if you have the <ndir.h> header file, and it defines `DIR'. */
 #undef HAVE_NDIR_H
 
@@ -564,9 +597,6 @@
 /* Define to 1 if you have the Open Sound System library. */
 #undef HAVE_OSS
 
-/* Define to 1 if OSX atomic operations are supported. */
-#undef HAVE_OSX_ATOMICS
-
 /* Define to 1 if your system defines the file flag O_EVTONLY in fcntl.h */
 #undef HAVE_O_EVTONLY
 
@@ -592,6 +622,9 @@
    Lock feature. */
 #undef HAVE_PJSIP_DLG_CREATE_UAS_AND_INC_LOCK
 
+/* Define if your system has the PJSIP_ENDPOINT_COMPACT_FORM headers. */
+#undef HAVE_PJSIP_ENDPOINT_COMPACT_FORM
+
 /* Define to 1 if PJPROJECT has the PJSIP EVSUB Group Lock support feature. */
 #undef HAVE_PJSIP_EVSUB_GRP_LOCK
 
@@ -754,6 +787,9 @@
 /* Define to 1 if you have the `putenv' function. */
 #undef HAVE_PUTENV
 
+/* Define to 1 if you have the <pwd.h> header file. */
+#undef HAVE_PWD_H
+
 /* Define if your system has the PYTHONDEV libraries. */
 #undef HAVE_PYTHONDEV
 
@@ -763,6 +799,9 @@
 /* Define to 1 if you have the `regcomp' function. */
 #undef HAVE_REGCOMP
 
+/* Define to 1 if you have the <regex.h> header file. */
+#undef HAVE_REGEX_H
+
 /* Define to 1 if you have the `remainder' function. */
 #undef HAVE_REMAINDER
 
@@ -772,6 +811,9 @@
 /* Define to 1 if you have the LIBRESAMPLE library. */
 #undef HAVE_RESAMPLE
 
+/* Define to 1 if you have the <resolv.h> header file. */
+#undef HAVE_RESOLV_H
+
 /* Define to 1 if your system has the close resolver function. */
 #undef HAVE_RES_CLOSE
 
@@ -808,6 +850,9 @@
 /* Define to 1 if your system has /sbin/launchd. */
 #undef HAVE_SBIN_LAUNCHD
 
+/* Define to 1 if you have the <sched.h> header file. */
+#undef HAVE_SCHED_H
+
 /* Define if your system has the SDL libraries. */
 #undef HAVE_SDL
 
@@ -826,6 +871,9 @@
 /* Define to 1 if you have the `sinl' function. */
 #undef HAVE_SINL
 
+/* Define to 1 if you have the libsndfile library. */
+#undef HAVE_SNDFILE
+
 /* Define to 1 if you have the `socket' function. */
 #undef HAVE_SOCKET
 
@@ -859,6 +907,19 @@
 /* Define to 1 if you have the Secure RTP library. */
 #undef HAVE_SRTP
 
+/* Define to 1 if SRTP has the SRTP Library AES-192 (ICM) feature. */
+#undef HAVE_SRTP_192
+
+/* Define to 1 if SRTP has the SRTP Library AES-256 (ICM) feature. */
+#undef HAVE_SRTP_256
+
+/* Define to 1 if SRTP has the SRTP Library AES-128 (GCM) and AES-256 (GCM)
+   feature. */
+#undef HAVE_SRTP_GCM
+
+/* Define to 1 if SRTP has the SRTP Library Version Function feature. */
+#undef HAVE_SRTP_GET_VERSION
+
 /* Define to 1 if SRTP has the SRTP Library Shutdown Function feature. */
 #undef HAVE_SRTP_SHUTDOWN
 
@@ -872,6 +933,9 @@
    zero-length file name argument. */
 #undef HAVE_STAT_EMPTY_STRING_BUG
 
+/* Define to 1 if you have the <stdarg.h> header file. */
+#undef HAVE_STDARG_H
+
 /* Define to 1 if stdbool.h conforms to C99. */
 #undef HAVE_STDBOOL_H
 
@@ -881,6 +945,9 @@
 /* Define to 1 if you have the <stdint.h> header file. */
 #undef HAVE_STDINT_H
 
+/* Define to 1 if you have the <stdio.h> header file. */
+#undef HAVE_STDIO_H
+
 /* Define to 1 if you have the <stdlib.h> header file. */
 #undef HAVE_STDLIB_H
 
@@ -1049,9 +1116,6 @@
 /* Define to 1 if you have the <sys/ioctl.h> header file. */
 #undef HAVE_SYS_IOCTL_H
 
-/* Define to 1 if you have the <sys/io.h> header file. */
-#undef HAVE_SYS_IO_H
-
 /* Define to 1 if you have the <sys/ndir.h> header file, and it defines `DIR'.
    */
 #undef HAVE_SYS_NDIR_H
@@ -1062,6 +1126,9 @@
 /* Define to 1 if your system has working sys/poll.h */
 #undef HAVE_SYS_POLL_H
 
+/* Define to 1 if you have the <sys/resource.h> header file. */
+#undef HAVE_SYS_RESOURCE_H
+
 /* Define to 1 if you have the <sys/select.h> header file. */
 #undef HAVE_SYS_SELECT_H
 
@@ -1080,6 +1147,9 @@
 /* Define to 1 if you have the <sys/types.h> header file. */
 #undef HAVE_SYS_TYPES_H
 
+/* Define to 1 if you have the <sys/un.h> header file. */
+#undef HAVE_SYS_UN_H
+
 /* Define to 1 if you have <sys/wait.h> that is POSIX.1 compatible. */
 #undef HAVE_SYS_WAIT_H
 
@@ -1089,9 +1159,6 @@
 /* Define to 1 if you have the `tanl' function. */
 #undef HAVE_TANL
 
-/* Define to 1 if you have the Termcap library. */
-#undef HAVE_TERMCAP
-
 /* Define to 1 if you have the <termios.h> header file. */
 #undef HAVE_TERMIOS_H
 
@@ -1101,8 +1168,8 @@
 /* Define to 1 if your system defines timersub. */
 #undef HAVE_TIMERSUB
 
-/* Define to 1 if you have the Term Info library. */
-#undef HAVE_TINFO
+/* Define to 1 if you have the <time.h> header file. */
+#undef HAVE_TIME_H
 
 /* Define to 1 if you have the tonezone library. */
 #undef HAVE_TONEZONE
@@ -1113,6 +1180,9 @@
 /* Define to 1 if you have the `truncl' function. */
 #undef HAVE_TRUNCL
 
+/* Define to 1 if you have the unbound library. */
+#undef HAVE_UNBOUND
+
 /* Define to 1 if you have the <unistd.h> header file. */
 #undef HAVE_UNISTD_H
 
@@ -1431,5 +1501,4 @@
    code using `volatile' can become incorrect without. Disable with care. */
 #undef volatile
 
-#endif
-
+#endif	/* ASTERISK_AUTOCONFIG_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/backtrace.h 1:16.1.1~dfsg-1/include/asterisk/backtrace.h
--- 1:13.23.1~dfsg-2/include/asterisk/backtrace.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/backtrace.h	2018-12-26 18:45:13.000000000 +0000
@@ -33,11 +33,13 @@
 #define ast_bt_create() __ast_bt_create()
 #define ast_bt_destroy(bt) __ast_bt_destroy((bt))
 #define ast_bt_get_symbols(addresses, num_frames) __ast_bt_get_symbols((addresses), (num_frames))
+#define ast_bt_free_symbols(string_vector) __ast_bt_free_symbols((string_vector))
 #else
 #define ast_bt_get_addresses(bt) 0
 #define ast_bt_create() NULL
 #define ast_bt_destroy(bt) NULL
 #define ast_bt_get_symbols(addresses, num_frames) NULL
+#define ast_bt_free_symbols(string_vector) NULL
 #endif
 
 /* \brief
@@ -86,11 +88,24 @@ void *__ast_bt_destroy(struct ast_bt *bt
  *
  * \param addresses A list of addresses, such as the ->addresses structure element of struct ast_bt.
  * \param num_frames Number of addresses in the addresses list
+ *
  * \retval NULL Unable to allocate memory
- * \return List of strings. Free the entire list with a single ast_std_free call.
+ * \return Vector of strings. Free with ast_bt_free_symbols
+ *
+ * \note The first frame in the addresses array will usually point to __ast_bt_create
+ * so when printing the symbols you may wish to start at position 1 rather than 0.
+ *
  * \since 1.6.2.16
  */
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames);
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames);
+
+/* \brief Free symbols returned from ast_bt_get_symbols
+ *
+ * \param symbols The symbol string vector
+ *
+ * \since 13.24.0
+ */
+void __ast_bt_free_symbols(struct ast_vector_string *symbols);
 
 #endif /* HAVE_BKTR */
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/bridge_channel.h 1:16.1.1~dfsg-1/include/asterisk/bridge_channel.h
--- 1:13.23.1~dfsg-2/include/asterisk/bridge_channel.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/bridge_channel.h	2018-12-26 18:45:13.000000000 +0000
@@ -138,7 +138,7 @@ struct ast_bridge_channel {
 	/*! Copy of write format used by chan before join */
 	struct ast_format *write_format;
 	/*! Call ID associated with bridge channel */
-	struct ast_callid *callid;
+	ast_callid callid;
 	/*! A clone of the roles living on chan when the bridge channel joins the bridge. This may require some opacification */
 	struct bridge_roles_datastore *bridge_roles;
 	/*! Linked list information */
@@ -164,6 +164,8 @@ struct ast_bridge_channel {
 		struct timeval dtmf_tv;
 		/*! Digit currently sending into the bridge. (zero if not sending) */
 		char dtmf_digit;
+		/*! Non-zero if a T.38 session terminate is owed to the bridge. */
+		char t38_terminate;
 	} owed;
 	/*! DTMF hook sequence state */
 	struct {
@@ -172,8 +174,23 @@ struct ast_bridge_channel {
 		/*! Collected DTMF digits for DTMF hooks. */
 		char collected[MAXIMUM_DTMF_FEATURE_STRING];
 	} dtmf_hook_state;
-	/*! Non-zero if a T.38 session terminate is owed to the bridge. */
-	char owed_t38_terminate;
+	union {
+		uint32_t raw;
+		struct {
+			/*! TRUE if binaural is suspended. */
+			unsigned int binaural_suspended:1;
+			/*! TRUE if a change of binaural positions has to be performed. */
+			unsigned int binaural_pos_change:1;
+			/*! Padding */
+			unsigned int padding:30;
+		};
+	};
+	struct {
+		/*! An index mapping of where a channel's media needs to be routed */
+		struct ast_vector_int to_bridge;
+		/*! An index mapping of where a bridge's media needs to be routed */
+		struct ast_vector_int to_channel;
+	} stream_map;
 };
 
 /*!
@@ -695,6 +712,32 @@ void ast_bridge_channel_feature_digit_ad
  */
 void ast_bridge_channel_feature_digit(struct ast_bridge_channel *bridge_channel, int digit);
 
+/*!
+ * \brief Maps a channel's stream topology to and from the bridge
+ * \since 15.0.0
+ *
+ * \details
+ * When a channel joins a bridge or its associated stream topology is
+ * updated, each stream in the topology needs to be mapped according
+ * to its media type to the bridge.  Calling this method creates a
+ * mapping of each stream on the channel indexed to the bridge's
+ * supported media types and vice versa (i.e. bridge's media types
+ * indexed to channel streams).
+ *
+ * The first channel to join the bridge creates the initial order for
+ * the bridge's media types (e.g. a one to one mapping is made).
+ * Subsequently added channels are mapped to that order adding more
+ * media types if/when the newly added channel has more streams and/or
+ * media types specified by the bridge.
+ *
+ * \param bridge_channel Channel to map
+ *
+ * \note The bridge_channel's bridge must be locked prior to calling this function.
+ *
+ * \return Nothing
+ */
+void ast_bridge_channel_stream_map(struct ast_bridge_channel *bridge_channel);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/bridge_features.h 1:16.1.1~dfsg-1/include/asterisk/bridge_features.h
--- 1:13.23.1~dfsg-2/include/asterisk/bridge_features.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/bridge_features.h	2018-12-26 18:45:13.000000000 +0000
@@ -53,6 +53,8 @@ enum ast_bridge_feature_flags {
 	AST_BRIDGE_FLAG_TRANSFER_PROHIBITED = (1 << 8),
 	/*! Bridge transfers require transfer of entire bridge rather than individual channels */
 	AST_BRIDGE_FLAG_TRANSFER_BRIDGE_ONLY = (1 << 9),
+	/*! Bridge is invisible to AMI/CLI/ARI/etc. */
+	AST_BRIDGE_FLAG_INVISIBLE = (1 << 10),
 };
 
 /*! \brief Flags used for per bridge channel features */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/bridge.h 1:16.1.1~dfsg-1/include/asterisk/bridge.h
--- 1:13.23.1~dfsg-2/include/asterisk/bridge.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/bridge.h	2018-12-26 18:45:13.000000000 +0000
@@ -102,6 +102,10 @@ enum ast_bridge_video_mode_type {
 	/*! A single user's video feed is distributed to all bridge channels, but
 	 *  that feed is automatically picked based on who is talking the most. */
 	AST_BRIDGE_VIDEO_MODE_TALKER_SRC,
+	/*! Operate as a selective forwarding unit. Video from each participant is
+	 * cloned to a dedicated stream on a subset of the remaining participants.
+	 */
+	AST_BRIDGE_VIDEO_MODE_SFU,
 };
 
 /*! \brief This is used for both SINGLE_SRC mode to set what channel
@@ -122,6 +126,24 @@ struct ast_bridge_video_talker_src_data
 	struct ast_channel *chan_old_vsrc;
 };
 
+/*! \brief REMB report behaviors */
+enum ast_bridge_video_sfu_remb_behavior {
+	/*! The average of all reports is sent to the sender */
+	AST_BRIDGE_VIDEO_SFU_REMB_AVERAGE = 0,
+	/*! The lowest reported bitrate is forwarded to the sender */
+	AST_BRIDGE_VIDEO_SFU_REMB_LOWEST,
+	/*! The highest reported bitrate is forwarded to the sender */
+	AST_BRIDGE_VIDEO_SFU_REMB_HIGHEST,
+};
+
+/*! \brief This is used for selective forwarding unit configuration */
+struct ast_bridge_video_sfu_data {
+	/*! The interval at which a REMB report is generated and sent */
+	unsigned int remb_send_interval;
+	/*! How the combined REMB report is generated */
+	enum ast_bridge_video_sfu_remb_behavior remb_behavior;
+};
+
 /*! \brief Data structure that defines a video source mode */
 struct ast_bridge_video_mode {
 	enum ast_bridge_video_mode_type mode;
@@ -129,7 +151,10 @@ struct ast_bridge_video_mode {
 	union {
 		struct ast_bridge_video_single_src_data single_src_data;
 		struct ast_bridge_video_talker_src_data talker_src_data;
+		struct ast_bridge_video_sfu_data sfu_data;
 	} mode_data;
+	/*! The minimum interval between video updates */
+	unsigned int video_update_discard;
 };
 
 /*!
@@ -263,8 +288,17 @@ struct ast_bridge_softmix {
 	 * for itself.
 	 */
 	unsigned int internal_mixing_interval;
+	/*! TRUE if binaural convolve is activated in configuration. */
+	unsigned int binaural_active;
+	/*!
+	 * Add a "label" attribute to each stream in the SDP containing
+	 * the channel uniqueid.  Used for participant info correlation.
+	 */
+	unsigned int send_sdp_label;
 };
 
+AST_LIST_HEAD_NOLOCK(ast_bridge_channels_list, ast_bridge_channel);
+
 /*!
  * \brief Structure that contains information about a bridge
  */
@@ -280,9 +314,9 @@ struct ast_bridge {
 	/*! Per-bridge topics */
 	struct stasis_cp_single *topics;
 	/*! Call ID associated with the bridge */
-	struct ast_callid *callid;
+	ast_callid callid;
 	/*! Linked list of channels participating in the bridge */
-	AST_LIST_HEAD_NOLOCK(, ast_bridge_channel) channels;
+	struct ast_bridge_channels_list channels;
 	/*! Queue of actions to perform on the bridge. */
 	AST_LIST_HEAD_NOLOCK(, ast_frame) action_queue;
 	/*! Softmix technology parameters. */
@@ -321,6 +355,9 @@ struct ast_bridge {
 		/*! Immutable bridge UUID. */
 		AST_STRING_FIELD(uniqueid);
 	);
+
+	/*! Type mapping used for media routing */
+	struct ast_vector_int media_types;
 };
 
 /*! \brief Bridge base class virtual method table. */
@@ -868,6 +905,14 @@ void ast_bridge_set_internal_sample_rate
 void ast_bridge_set_mixing_interval(struct ast_bridge *bridge, unsigned int mixing_interval);
 
 /*!
+ * \brief Activates the use of binaural signals in a conference bridge.
+ *
+ *  \param bridge Channel to activate the binaural signals.
+ *  \param binaural_active If true binaural signal processing will be active for the bridge.
+ */
+void ast_bridge_set_binaural_active(struct ast_bridge *bridge, unsigned int binaural_active);
+
+/*!
  * \brief Set a bridge to feed a single video source to all participants.
  */
 void ast_bridge_set_single_src_video_mode(struct ast_bridge *bridge, struct ast_channel *video_src_chan);
@@ -879,6 +924,39 @@ void ast_bridge_set_single_src_video_mod
 void ast_bridge_set_talker_src_video_mode(struct ast_bridge *bridge);
 
 /*!
+ * \brief Set the bridge to be a selective forwarding unit
+ */
+void ast_bridge_set_sfu_video_mode(struct ast_bridge *bridge);
+
+/*!
+ * \brief Set the amount of time to discard subsequent video updates after a video update has been sent
+ *
+ * \param bridge Bridge to set the minimum video update wait time on
+ * \param video_update_discard Amount of time after sending a video update that others should be discarded
+ */
+void ast_bridge_set_video_update_discard(struct ast_bridge *bridge, unsigned int video_update_discard);
+
+/*!
+ * \brief Set the interval at which a combined REMB frame will be sent to video sources
+ *
+ * \param bridge Bridge to set the REMB send interval on
+ * \param remb_send_interval The REMB send interval
+ *
+ * \note This can only be called when the bridge has been set to the SFU video mode.
+ */
+void ast_bridge_set_remb_send_interval(struct ast_bridge *bridge, unsigned int remb_send_interval);
+
+/*!
+ * \brief Set the REMB report generation behavior on a bridge
+ *
+ * \param bridge Bridge to set the REMB behavior on
+ * \param behavior How REMB reports are generated
+ *
+ * \note This can only be called when the bridge has been set to the SFU video mode.
+ */
+void ast_brige_set_remb_behavior(struct ast_bridge *bridge, enum ast_bridge_video_sfu_remb_behavior behavior);
+
+/*!
  * \brief Update information about talker energy for talker src video mode.
  */
 void ast_bridge_update_talker_src_video_mode(struct ast_bridge *bridge, struct ast_channel *chan, int talker_energy, int is_keyfame);
@@ -913,6 +991,20 @@ void ast_bridge_remove_video_src(struct
 const char *ast_bridge_video_mode_to_string(enum ast_bridge_video_mode_type video_mode);
 
 /*!
+ * \brief Controls whether to send a "label" attribute in each stream in an SDP
+ * \since 16.1.0
+ *
+ * \param bridge The bridge
+ * \param send_sdp_label Whether to send the labels or not
+ *
+ * \note The label will contain the uniqueid of the channel related to the stream.
+ * This is used to allow the recipient to correlate the stream to the participant
+ * information events sent by app_confbridge.
+ * The bridge will be locked in this function.
+ */
+void ast_bridge_set_send_sdp_label(struct ast_bridge *bridge, unsigned int send_sdp_label);
+
+/*!
  * \brief Acquire the channel's bridge for transfer purposes.
  * \since 13.21.0
  *
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/bridge_technology.h 1:16.1.1~dfsg-1/include/asterisk/bridge_technology.h
--- 1:13.23.1~dfsg-2/include/asterisk/bridge_technology.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/bridge_technology.h	2018-12-26 18:45:13.000000000 +0000
@@ -162,10 +162,34 @@ struct ast_bridge_technology {
 	 */
 	int (*write)(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel, struct ast_frame *frame);
 	/*!
-	 * \brief Formats that the bridge technology supports
-	 * \note This is no longer used.  It remains for ABI compatibility.
+	 * \brief Callback for when a request has been made to change a stream topology on a channel
+	 *
+	 * \details
+	 * This is called when a bridge receives a request to change the
+	 * topology on the channel.  A bridge technology should define a
+	 * handler for this callback if it needs to update internals or
+	 * intercept the request and not pass it on to other channels.
+	 * This can be done by returning a nonzero value.
+	 *
+	 * \retval 0 Frame can pass to the bridge technology.
+	 * \retval non-zero Frame intercepted by the bridge technology.
+	 *
+	 * \note On entry, bridge is already locked.
 	 */
-	struct ast_format_cap *format_capabilities;
+	int (*stream_topology_request_change)(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel);
+	/*!
+	 * \brief Callback for when a stream topology changes on the channel
+	 *
+	 * \details
+	 * This is called when a bridge receives an indication that a
+	 * topology has been changed on a channel and the new topology has
+	 * been mapped to the bridge.  A bridge technology should define a
+	 * handler for this callback if it needs to update internals due
+	 * to a channel's topology changing.
+	 *
+	 * \note On entry, bridge is already locked.
+	 */
+	void (*stream_topology_changed)(struct ast_bridge *bridge, struct ast_bridge_channel *bridge_channel);
 	/*! TRUE if the bridge technology is currently suspended. */
 	unsigned int suspended:1;
 	/*! Module this bridge technology belongs to. It is used for reference counting bridges using the technology. */
@@ -196,7 +220,7 @@ struct ast_bridge_technology {
 int __ast_bridge_technology_register(struct ast_bridge_technology *technology, struct ast_module *mod);
 
 /*! \brief See \ref __ast_bridge_technology_register() */
-#define ast_bridge_technology_register(technology) __ast_bridge_technology_register(technology, ast_module_info->self)
+#define ast_bridge_technology_register(technology) __ast_bridge_technology_register(technology, AST_MODULE_SELF)
 
 /*!
  * \brief Unregister a bridge technology from use
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/bucket.h 1:16.1.1~dfsg-1/include/asterisk/bucket.h
--- 1:13.23.1~dfsg-2/include/asterisk/bucket.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/bucket.h	2018-12-26 18:45:13.000000000 +0000
@@ -23,7 +23,7 @@
  */
 
 /*!
- * \page AstBucket Bucket File API
+ * \page bucket AstBucket Bucket File API
  *
  * Bucket is an API which provides directory and file access in a generic fashion. It is
  * implemented as a thin wrapper over the sorcery data access layer API and is written in
@@ -134,7 +134,7 @@ int ast_bucket_init(void);
  *
  * \note Once a scheme has been registered it can not be unregistered
  */
-#define ast_bucket_scheme_register(name, bucket, file, create_cb, destroy_cb) __ast_bucket_scheme_register(name, bucket, file, create_cb, destroy_cb, ast_module_info ? ast_module_info->self : NULL)
+#define ast_bucket_scheme_register(name, bucket, file, create_cb, destroy_cb) __ast_bucket_scheme_register(name, bucket, file, create_cb, destroy_cb, AST_MODULE_SELF)
 
 /*!
  * \brief Register support for a specific scheme
@@ -195,6 +195,17 @@ int ast_bucket_file_metadata_unset(struc
 struct ast_bucket_metadata *ast_bucket_file_metadata_get(struct ast_bucket_file *file, const char *name);
 
 /*!
+ * \brief Execute a callback function on the metadata associated with a file
+ * \since 14.0.0
+ *
+ * \param file The bucket file
+ * \param cb An ao2 callback function that will be called with each \c ast_bucket_metadata
+ *           associated with \c file
+ * \param arg An optional argument to pass to \c cb
+ */
+void ast_bucket_file_metadata_callback(struct ast_bucket_file *file, ao2_callback_fn cb, void *arg);
+
+/*!
  * \brief Allocate a new bucket
  *
  * \param uri Complete URI for the bucket
@@ -218,6 +229,23 @@ struct ast_bucket *ast_bucket_alloc(cons
 int ast_bucket_create(struct ast_bucket *bucket);
 
 /*!
+ * \brief Clone a bucket
+ *
+ * This will create a copy of the passed in \c ast_bucket structure. While
+ * all properties of the \c ast_bucket structure are copied, any metadata
+ * in the original structure simply has its reference count increased.
+ *
+ * \param file The bucket to clone
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This operation should be called prior to updating a bucket
+ * object, as \c ast_bucket instances are immutable
+ */
+struct ast_bucket *ast_bucket_clone(struct ast_bucket *bucket);
+
+/*!
  * \brief Delete a bucket from backend storage
  *
  * \param bucket The bucket
@@ -240,6 +268,23 @@ int ast_bucket_delete(struct ast_bucket
 struct ast_bucket *ast_bucket_retrieve(const char *uri);
 
 /*!
+ * \brief Retrieve whether or not the backing datastore views the bucket as stale
+ * \since 14.0.0
+ *
+ * This function will ask whatever data storage backs the bucket's schema
+ * type if the current instance of the object is stale. It will not
+ * update the bucket object itself, as said objects are immutable. If the
+ * caller of this function would like to update the object, it should perform
+ * a retrieve operation.
+ *
+ * \param bucket The bucket object to check
+ *
+ * \retval 0 if \c bucket is not stale
+ * \retval 1 if \c bucket is stale
+ */
+int ast_bucket_is_stale(struct ast_bucket *bucket);
+
+/*!
  * \brief Add an observer for bucket creation and deletion operations
  *
  * \param callbacks Implementation of the sorcery observer interface
@@ -309,6 +354,24 @@ int ast_bucket_file_create(struct ast_bu
 struct ast_bucket_file *ast_bucket_file_copy(struct ast_bucket_file *file, const char *uri);
 
 /*!
+ * \brief Clone a bucket file
+ *
+ * This will create a copy of the passed in \c ast_bucket_file structure. While
+ * all properties of the \c ast_bucket_file structure are copied, any metadata
+ * in the original structure simply has its reference count increased. Note that
+ * this copies the structure, not the underlying file.
+ *
+ * \param file The bucket file to clone
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This operation should be called prior to updating a bucket file
+ * object, as \c ast_bucket_file instances are immutable
+ */
+struct ast_bucket_file *ast_bucket_file_clone(struct ast_bucket_file *file);
+
+/*!
  * \brief Update an existing bucket file in backend storage
  *
  * \param file The bucket file
@@ -343,6 +406,23 @@ int ast_bucket_file_delete(struct ast_bu
 struct ast_bucket_file *ast_bucket_file_retrieve(const char *uri);
 
 /*!
+ * \brief Retrieve whether or not the backing datastore views the bucket file as stale
+ * \since 14.0.0
+ *
+ * This function will ask whatever data storage backs the bucket file's schema
+ * type if the current instance of the object is stale. It will not
+ * update the bucket file object itself, as said objects are immutable. If the
+ * caller of this function would like to update the object, it should perform
+ * a retrieve operation.
+ *
+ * \param bucket_file The bucket file object to check
+ *
+ * \retval 0 if \c bucket_file is not stale
+ * \retval 1 if \c bucket_file is stale
+ */
+int ast_bucket_file_is_stale(struct ast_bucket_file *file);
+
+/*!
  * \brief Add an observer for bucket file creation and deletion operations
  *
  * \param callbacks Implementation of the sorcery observer interface
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/calendar.h 1:16.1.1~dfsg-1/include/asterisk/calendar.h
--- 1:13.23.1~dfsg-2/include/asterisk/calendar.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/calendar.h	2018-12-26 18:45:13.000000000 +0000
@@ -129,11 +129,12 @@ struct ast_calendar {
 	int autoreminder;    /*!< If set, override any calendar_tech specific notification times and use this time (in mins) */
 	int notify_waittime; /*!< Maxiumum time to allow for a notification attempt */
 	int refresh;         /*!< When to refresh the calendar events */
+	int fetch_again_at_reload; /*!< To reload the calendar content when the module is reloaded */
 	int timeframe;       /*!< Span (in mins) of calendar data to pull with each request */
 	pthread_t thread;    /*!< The thread that the calendar is loaded/updated in */
 	ast_cond_t unload;
 	int unloading:1;
-	int pending_deletion:1; /*!< No longer used */
+	int pending_deletion:1;
 	struct ao2_container *events;  /*!< The events that are known at this time */
 };
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/ccss.h 1:16.1.1~dfsg-1/include/asterisk/ccss.h
--- 1:13.23.1~dfsg-2/include/asterisk/ccss.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/ccss.h	2018-12-26 18:45:13.000000000 +0000
@@ -1618,15 +1618,4 @@ typedef void (*ast_cc_callback_fn)(struc
  */
 int ast_cc_callback(struct ast_channel *inbound, const char * const tech, const char * const dest, ast_cc_callback_fn callback);
 
-/*!
- * \since 1.8
- * \brief Initialize CCSS
- *
- * Performs startup routines necessary for CC operation.
- *
- * \retval 0 Success
- * \retval nonzero Failure
- */
-int ast_cc_init(void);
-
 #endif /* _ASTERISK_CCSS_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/cdr.h 1:16.1.1~dfsg-1/include/asterisk/cdr.h
--- 1:13.23.1~dfsg-2/include/asterisk/cdr.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/cdr.h	2018-12-26 18:45:13.000000000 +0000
@@ -536,6 +536,36 @@ int ast_cdr_backend_suspend(const char *
 int ast_cdr_backend_unsuspend(const char *name);
 
 /*!
+ * \brief Register a CDR modifier
+ * \param name name associated with the particular CDR modifier
+ * \param desc description of the CDR modifier
+ * \param be function pointer to a CDR modifier
+ *
+ * Used to register a Call Detail Record modifier.
+ *
+ * This gives modules a chance to modify CDR fields before they are dispatched
+ * to registered backends (odbc, syslog, etc).
+ *
+ * \note The *modified* CDR will be passed to **all** registered backends for
+ * logging. For instance, if cdr_manager changes the CDR data, cdr_adaptive_odbc
+ * will also get the modified CDR.
+ *
+ * \retval 0 on success.
+ * \retval -1 on error
+ */
+int ast_cdr_modifier_register(const char *name, const char *desc, ast_cdrbe be);
+
+/*!
+ * \brief Unregister a CDR modifier
+ * \param name name of CDR modifier to unregister
+ * Unregisters a CDR modifier by its name
+ *
+ * \retval 0 The modifier unregistered successfully
+ * \retval -1 The modifier could not be unregistered at this time
+ */
+int ast_cdr_modifier_unregister(const char *name);
+
+/*!
  * \brief Disposition to a string
  * \param disposition input binary form
  * Converts the binary form of a disposition to string form.
@@ -551,12 +581,6 @@ const char *ast_cdr_disp2str(int disposi
  */
 void ast_cdr_setuserfield(const char *channel_name, const char *userfield);
 
-/*! \brief Reload the configuration file cdr.conf and start/stop CDR scheduling thread */
-int ast_cdr_engine_reload(void);
-
-/*! \brief Load the configuration file cdr.conf and possibly start the CDR scheduling thread */
-int ast_cdr_engine_init(void);
-
 /*! Submit any remaining CDRs and prepare for shutdown */
 void ast_cdr_engine_term(void);
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/channel.h 1:16.1.1~dfsg-1/include/asterisk/channel.h
--- 1:13.23.1~dfsg-2/include/asterisk/channel.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/channel.h	2018-12-26 18:45:13.000000000 +0000
@@ -159,7 +159,7 @@ extern "C" {
  */
 #define AST_MAX_UNIQUEID        (AST_MAX_PUBLIC_UNIQUEID + 2 + 1)
 
-#define AST_MAX_ACCOUNT_CODE    20  /*!< Max length of an account code */
+#define AST_MAX_ACCOUNT_CODE    80  /*!< Max length of an account code */
 #define AST_CHANNEL_NAME        80  /*!< Max length of an ast_channel name */
 #define MAX_LANGUAGE            40  /*!< Max length of the language setting */
 #define MAX_MUSICCLASS          80  /*!< Max length of the music class setting */
@@ -174,7 +174,7 @@ extern "C" {
 #include "asterisk/linkedlists.h"
 #include "asterisk/stringfields.h"
 #include "asterisk/datastore.h"
-#include "asterisk/data.h"
+#include "asterisk/format_cap.h"
 #include "asterisk/channelstate.h"
 #include "asterisk/ccss.h"
 #include "asterisk/framehook.h"
@@ -183,7 +183,8 @@ extern "C" {
 
 #define DATASTORE_INHERIT_FOREVER	INT_MAX
 
-#define AST_MAX_FDS		11
+#define AST_MAX_FDS		11	/*!< original maximum number of file descriptors */
+#define AST_EXTENDED_FDS	12	/*!< the start of extended file descriptor positions */
 /*
  * We have AST_MAX_FDS file descriptors in a channel.
  * Some of them have a fixed use:
@@ -203,6 +204,8 @@ enum ast_bridge_result {
 
 typedef unsigned long long ast_group_t;
 
+struct ast_stream_topology;
+
 /*! \todo Add an explanation of an Asterisk generator
 */
 struct ast_generator {
@@ -635,7 +638,28 @@ struct ast_channel_tech {
 	 */
 	struct ast_channel *(* const requester)(const char *type, struct ast_format_cap *cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause);
 
+	/*!
+	 * \brief Requester - to set up call data structures (pvt's) with stream topology
+	 *
+	 * \param type type of channel to request
+	 * \param topology Stream topology for requested channel
+	 * \param assignedid Unique ID string to assign to channel
+	 * \param requestor channel asking for data
+	 * \param addr destination of the call
+	 * \param cause Cause of failure
+	 *
+	 * \details
+	 * Request a channel of a given type, with addr as optional information used
+	 * by the low level module
+	 *
+	 * \retval NULL failure
+	 * \retval non-NULL channel on success
+	 */
+	struct ast_channel *(* const requester_with_stream_topology)(const char *type, struct ast_stream_topology *topology, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause);
+
+
 	int (* const devicestate)(const char *device_number);	/*!< Devicestate call back */
+	int (* const presencestate)(const char *presence_provider, char **subtype, char **message); /*!< Presencestate callback */
 
 	/*!
 	 * \brief Start sending a literal DTMF digit
@@ -668,12 +692,39 @@ struct ast_channel_tech {
 	/*! \brief Answer the channel */
 	int (* const answer)(struct ast_channel *chan);
 
-	/*! \brief Read a frame, in standard format (see frame.h) */
+	/*!
+	 * \brief Read a frame (or chain of frames from the same stream), in standard format (see frame.h)
+	 *
+	 * \param chan channel to read frames from
+	 *
+	 * \retval non-NULL on success
+	 * \retval NULL on failure
+	 *
+	 * \note Each media frame from this callback will have the stream_num of it changed to the default
+	 *       stream num based on the type of media returned. As a result a multistream capable channel
+	 *       should not implement this callback.
+	 */
 	struct ast_frame * (* const read)(struct ast_channel *chan);
 
+	/*!
+	 * \brief Read a frame (or chain of frames from the same stream), in standard format (see frame.h), with stream num
+	 *
+	 * \param chan channel to read frames from
+	 *
+	 * \retval non-NULL on success
+	 * \retval NULL on failure
+	 *
+	 * \note Each media frame from this callback should contain a stream_num value which is set to the
+	 *       stream that the media frame originated from.
+	 */
+	struct ast_frame * (* const read_stream)(struct ast_channel *chan);
+
 	/*! \brief Write a frame, in standard format (see frame.h) */
 	int (* const write)(struct ast_channel *chan, struct ast_frame *frame);
 
+	/*! \brief Write a frame on a specific stream, in standard format (see frame.h) */
+	int (* const write_stream)(struct ast_channel *chan, int stream_num, struct ast_frame *frame);
+
 	/*! \brief Display or transmit text */
 	int (* const send_text)(struct ast_channel *chan, const char *text);
 
@@ -1105,22 +1156,6 @@ enum ama_flags {
  *       take a channel argument.
  */
 
-/*!
- * \brief Create a channel data store object
- * \deprecated You should use the ast_datastore_alloc() generic function instead.
- * \version 1.6.1 deprecated
- */
-struct ast_datastore *ast_channel_datastore_alloc(const struct ast_datastore_info *info, const char *uid)
-	__attribute__((deprecated));
-
-/*!
- * \brief Free a channel data store object
- * \deprecated You should use the ast_datastore_free() generic function instead.
- * \version 1.6.1 deprecated
- */
-int ast_channel_datastore_free(struct ast_datastore *datastore)
-	__attribute__((deprecated));
-
 /*! \brief Inherit datastores from a parent to a child. */
 int ast_channel_datastore_inherit(struct ast_channel *from, struct ast_channel *to);
 
@@ -1197,8 +1232,6 @@ struct ast_channel * __attribute__((form
 	__ast_channel_alloc((needqueue), (state), (cid_num), (cid_name), (acctcode), (exten), (context), (assignedids), (requestor), (amaflag), (endpoint), \
 		__FILE__, __LINE__, __FUNCTION__, __VA_ARGS__)
 
-
-#if defined(REF_DEBUG) || defined(__AST_DEBUG_MALLOC)
 /*!
  * \brief Create a fake channel structure
  *
@@ -1217,25 +1250,6 @@ struct ast_channel * __attribute__((form
  */
 #define ast_dummy_channel_alloc()	__ast_dummy_channel_alloc(__FILE__, __LINE__, __PRETTY_FUNCTION__)
 struct ast_channel *__ast_dummy_channel_alloc(const char *file, int line, const char *function);
-#else
-/*!
- * \brief Create a fake channel structure
- *
- * \retval NULL failure
- * \retval non-NULL successfully allocated channel
- *
- * \note This function should ONLY be used to create a fake channel
- *       that can then be populated with data for use in variable
- *       substitution when a real channel does not exist.
- *
- * \note The created dummy channel should be destroyed by
- * ast_channel_unref().  Using ast_channel_release() needlessly
- * grabs the channel container lock and can cause a deadlock as
- * a result.  Also grabbing the channel container lock reduces
- * system performance.
- */
-struct ast_channel *ast_dummy_channel_alloc(void);
-#endif
 
 /*!
  * \brief Queue one or more frames to a channel's frame queue
@@ -1398,6 +1412,25 @@ struct ast_channel *ast_channel_release(
  */
 struct ast_channel *ast_request(const char *type, struct ast_format_cap *request_cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause);
 
+/*!
+ * \brief Requests a channel (specifying stream topology)
+ *
+ * \param type type of channel to request
+ * \param topology Stream topology for requested channel
+ * \param assignedids Unique ID to create channel with
+ * \param requestor channel asking for data
+ * \param addr destination of the call
+ * \param cause Cause of failure
+ *
+ * \details
+ * Request a channel of a given type, with addr as optional information used
+ * by the low level module
+ *
+ * \retval NULL failure
+ * \retval non-NULL channel on success
+ */
+struct ast_channel *ast_request_with_stream_topology(const char *type, struct ast_stream_topology *topology, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause);
+
 enum ast_channel_requestor_relationship {
 	/*! The requestor is the future bridge peer of the channel. */
 	AST_CHANNEL_REQUESTOR_BRIDGE_PEER,
@@ -1673,21 +1706,6 @@ void ast_channel_softhangup_withcause_lo
 
 /*!
  * \brief Compare a offset with the settings of when to hang a channel up
- * \param chan channel on which to check for hang up
- * \param offset offset in seconds from current time
- * \return 1, 0, or -1
- * \details
- * This function compares a offset from current time with the absolute time
- * out on a channel (when to hang up). If the absolute time out on a channel
- * is earlier than current time plus the offset, it returns 1, if the two
- * time values are equal, it return 0, otherwise, it return -1.
- * \sa ast_channel_cmpwhentohangup_tv()
- * \version 1.6.1 deprecated function (only had seconds precision)
- */
-int ast_channel_cmpwhentohangup(struct ast_channel *chan, time_t offset) __attribute__((deprecated));
-
-/*!
- * \brief Compare a offset with the settings of when to hang a channel up
  * \param chan channel on which to check for hangup
  * \param offset offset in seconds and microseconds from current time
  * \return 1, 0, or -1
@@ -1703,23 +1721,6 @@ int ast_channel_cmpwhentohangup_tv(struc
  * \brief Set when to hang a channel up
  *
  * \param chan channel on which to check for hang up
- * \param offset offset in seconds relative to the current time of when to hang up
- *
- * \details
- * This function sets the absolute time out on a channel (when to hang up).
- *
- * \pre chan is locked
- *
- * \return Nothing
- * \sa ast_channel_setwhentohangup_tv()
- * \version 1.6.1 deprecated function (only had seconds precision)
- */
-void ast_channel_setwhentohangup(struct ast_channel *chan, time_t offset) __attribute__((deprecated));
-
-/*!
- * \brief Set when to hang a channel up
- *
- * \param chan channel on which to check for hang up
  * \param offset offset in seconds and useconds relative to the current time of when to hang up
  *
  * This function sets the absolute time out on a channel (when to hang up).
@@ -1951,14 +1952,37 @@ int ast_waitfor_n_fd(int *fds, int n, in
 
 /*!
  * \brief Reads a frame
+ *
  * \param chan channel to read a frame from
+ *
  * \return Returns a frame, or NULL on error.  If it returns NULL, you
  * best just stop reading frames and assume the channel has been
  * disconnected.
+ *
+ * \note This function will filter frames received from the channel so
+ *       that only frames from the default stream for each media type
+ *       are returned. All other media frames from other streams will
+ *       be absorbed internally and a NULL frame returned instead.
  */
 struct ast_frame *ast_read(struct ast_channel *chan);
 
 /*!
+ * \brief Reads a frame, but does not filter to just the default streams
+ *
+ * \param chan channel to read a frame from
+ *
+ * \return Returns a frame, or NULL on error.  If it returns NULL, you
+ * best just stop reading frames and assume the channel has been
+ * disconnected.
+ *
+ * \note This function will not perform any filtering and will return
+ *       media frames from all streams on the channel. To determine which
+ *       stream a frame originated from the stream_num on it can be
+ *       examined.
+ */
+struct ast_frame *ast_read_stream(struct ast_channel *chan);
+
+/*!
  * \brief Reads a frame, returning AST_FRAME_NULL frame if audio.
  * \param chan channel to read a frame from
  * \return  Returns a frame, or NULL on error.  If it returns NULL, you
@@ -1970,6 +1994,26 @@ struct ast_frame *ast_read(struct ast_ch
 struct ast_frame *ast_read_noaudio(struct ast_channel *chan);
 
 /*!
+ * \brief Reads a frame, but does not filter to just the default streams,
+ * returning AST_FRAME_NULL frame if audio.
+ *
+ * \param chan channel to read a frame from
+ *
+ * \return Returns a frame, or NULL on error.  If it returns NULL, you
+ * best just stop reading frames and assume the channel has been
+ * disconnected.
+ *
+ * \note This function will not perform any filtering and will return
+ *       media frames from all streams on the channel. To determine which
+ *       stream a frame originated from the stream_num on it can be
+ *       examined.
+ *
+ * \note Audio is replaced with AST_FRAME_NULL to avoid
+ * transcode when the resulting audio is not necessary.
+ */
+struct ast_frame *ast_read_stream_noaudio(struct ast_channel *chan);
+
+/*!
  * \brief Write a frame to a channel
  * This function writes the given frame to the indicated channel.
  * \param chan destination channel of the frame
@@ -1996,6 +2040,18 @@ int ast_write_video(struct ast_channel *
  */
 int ast_write_text(struct ast_channel *chan, struct ast_frame *frame);
 
+/*!
+ * \brief Write a frame to a stream
+ * This function writes the given frame to the indicated stream on the channel.
+ * \param chan destination channel of the frame
+ * \param stream_num destination stream on the channel
+ * \param frame frame that will be written
+ * \return It returns 0 on success, -1 on failure.
+ * \note If -1 is provided as the stream number and a media frame is provided the
+ *       function will write to the default stream of the type of media.
+ */
+int ast_write_stream(struct ast_channel *chan, int stream_num, struct ast_frame *frame);
+
 /*! \brief Send empty audio to prime a channel driver */
 int ast_prod(struct ast_channel *chan);
 
@@ -2064,6 +2120,16 @@ int ast_set_write_format_from_cap(struct
 int ast_set_write_format(struct ast_channel *chan, struct ast_format *format);
 
 /*!
+ * \brief Sets write format for a channel.
+ * All internal data will than be handled in an interleaved format. (needed by binaural opus)
+ *
+ * \param chan channel to change
+ * \param format format to set for writing
+ * \return Returns 0 on success, -1 on failure
+ */
+int ast_set_write_format_interleaved_stereo(struct ast_channel *chan, struct ast_format *format);
+
+/*!
  * \brief Sends text to a channel
  *
  * \param chan channel to act upon
@@ -2175,11 +2241,12 @@ int ast_waitfordigit(struct ast_channel
  * Same as ast_waitfordigit() with audio fd for outputting read audio and ctrlfd to monitor for reading.
  * \param c channel to wait for a digit on
  * \param ms how many milliseconds to wait (<0 for indefinite).
+ * \param breakon string of DTMF digits to break upon or NULL for any.
  * \param audiofd audio file descriptor to write to if audio frames are received
  * \param ctrlfd control file descriptor to monitor for reading
  * \return Returns 1 if ctrlfd becomes available
  */
-int ast_waitfordigit_full(struct ast_channel *c, int ms, int audiofd, int ctrlfd);
+int ast_waitfordigit_full(struct ast_channel *c, int ms, const char *breakon, int audiofd, int ctrlfd);
 
 /*!
  * \brief Reads multiple digits
@@ -2400,12 +2467,6 @@ void ast_channel_set_caller_event(struct
 /*! Set the file descriptor on the channel */
 void ast_channel_set_fd(struct ast_channel *chan, int which, int fd);
 
-/*! Add a channel to an optimized waitfor */
-void ast_poll_channel_add(struct ast_channel *chan0, struct ast_channel *chan1);
-
-/*! Delete a channel from an optimized waitfor */
-void ast_poll_channel_del(struct ast_channel *chan0, struct ast_channel *chan1);
-
 /*! Start a tone going */
 int ast_tonepair_start(struct ast_channel *chan, int freq1, int freq2, int duration, int vol);
 /*! Stop a tone from playing */
@@ -2693,12 +2754,6 @@ int ast_namedgroups_intersect(struct ast
 /*! \brief Print named call groups and named pickup groups */
 char *ast_print_namedgroups(struct ast_str **buf, struct ast_namedgroups *groups);
 
-/*!
- * \brief Convert enum channelreloadreason to text string for manager event
- * \param reason The reason for reload (manager, cli, start etc)
- */
-const char *channelreloadreason2txt(enum channelreloadreason reason);
-
 /*! \brief return an ast_variable list of channeltypes */
 struct ast_variable *ast_channeltype_list(void);
 
@@ -3817,27 +3872,6 @@ int ast_channel_connected_line_macro(str
 int ast_channel_connected_line_sub(struct ast_channel *autoservice_chan, struct ast_channel *sub_chan, const void *connected_info, int frame);
 
 /*!
- * \brief Insert into an astdata tree, the channel structure.
- * \param[in] tree The ast data tree.
- * \param[in] chan The channel structure to add to tree.
- * \param[in] add_bridged Add the bridged channel to the structure.
- * \retval <0 on error.
- * \retval 0 on success.
- */
-int ast_channel_data_add_structure(struct ast_data *tree, struct ast_channel *chan, int add_bridged);
-
-/*!
- * \brief Compare to channel structures using the data api.
- * \param[in] tree The search tree generated by the data api.
- * \param[in] chan The channel to compare.
- * \param[in] structure_name The name of the node of the channel structure.
- * \retval 0 The structure matches.
- * \retval 1 The structure doesn't matches.
- */
-int ast_channel_data_cmp_structure(const struct ast_data_search *tree, struct ast_channel *chan,
-	const char *structure_name);
-
-/*!
  * \since 1.8
  * \brief Run a redirecting interception macro and update a channel's redirecting information
  * \deprecated You should use the ast_channel_redirecting_sub() function instead.
@@ -4201,12 +4235,12 @@ void ast_channel_tech_set(struct ast_cha
 enum ast_channel_adsicpe ast_channel_adsicpe(const struct ast_channel *chan);
 void ast_channel_adsicpe_set(struct ast_channel *chan, enum ast_channel_adsicpe value);
 enum ast_channel_state ast_channel_state(const struct ast_channel *chan);
-struct ast_callid *ast_channel_callid(const struct ast_channel *chan);
+ast_callid ast_channel_callid(const struct ast_channel *chan);
 
 /*!
  * \pre chan is locked
  */
-void ast_channel_callid_set(struct ast_channel *chan, struct ast_callid *value);
+void ast_channel_callid_set(struct ast_channel *chan, ast_callid value);
 
 /* XXX Internal use only, make sure to move later */
 void ast_channel_state_set(struct ast_channel *chan, enum ast_channel_state);
@@ -4308,11 +4342,30 @@ void ast_channel_internal_fd_set(struct
 int ast_channel_fd(const struct ast_channel *chan, int which);
 int ast_channel_fd_isset(const struct ast_channel *chan, int which);
 
-/* epoll data internal accessors */
-#ifdef HAVE_EPOLL
-struct ast_epoll_data *ast_channel_internal_epfd_data(const struct ast_channel *chan, int which);
-void ast_channel_internal_epfd_data_set(struct ast_channel *chan, int which , struct ast_epoll_data *value);
-#endif
+/*!
+ * \since 15
+ * \brief Retrieve the number of file decriptor positions present on the channel
+ *
+ * \param chan The channel to get the count of
+ *
+ * \pre chan is locked
+ *
+ * \return The number of file descriptor positions
+ */
+int ast_channel_fd_count(const struct ast_channel *chan);
+
+/*!
+ * \since 15
+ * \brief Add a file descriptor to the channel without a fixed position
+ *
+ * \param chan The channel to add the file descriptor to
+ * \param value The file descriptor
+ *
+ * \pre chan is locked
+ *
+ * \return The position of the file descriptor
+ */
+int ast_channel_fd_add(struct ast_channel *chan, int value);
 
 pthread_t ast_channel_blocker(const struct ast_channel *chan);
 void ast_channel_blocker_set(struct ast_channel *chan, pthread_t value);
@@ -4454,6 +4507,36 @@ void ast_channel_set_manager_vars(size_t
 struct varshead *ast_channel_get_manager_vars(struct ast_channel *chan);
 
 /*!
+ * \since 14.2.0
+ * \brief Return whether or not any ARI variables have been set
+ *
+ * \retval 0 if no ARI variables are expected
+ * \retval 1 if ARI variables are expected
+ */
+int ast_channel_has_ari_vars(void);
+
+/*!
+ * \since 14.2.0
+ * \brief Sets the variables to be stored in the \a ari_vars field of all
+ * snapshots.
+ * \param varc Number of variable names.
+ * \param vars Array of variable names.
+ */
+void ast_channel_set_ari_vars(size_t varc, char **vars);
+
+/*!
+ * \since 14.2.0
+ * \brief Gets the variables for a given channel, as specified by ast_channel_set_ari_vars().
+ *
+ * The returned variable list is an AO2 object, so ao2_cleanup() to free it.
+ *
+ * \param chan Channel to get variables for.
+ * \return List of channel variables.
+ * \return \c NULL on error
+ */
+struct varshead *ast_channel_get_ari_vars(struct ast_channel *chan);
+
+/*!
  * \since 12
  * \brief Gets the variables for a given channel, as set using pbx_builtin_setvar_helper().
  *
@@ -4805,4 +4888,106 @@ enum ast_channel_error ast_channel_errno
  */
 int ast_channel_get_intercept_mode(void);
 
+/*!
+ * \brief Retrieve the topology of streams on a channel
+ *
+ * \param chan The channel to get the stream topology of
+ *
+ * \pre chan is locked
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_stream_topology *ast_channel_get_stream_topology(
+	const struct ast_channel *chan);
+
+/*!
+ * \brief Set the topology of streams on a channel
+ *
+ * \param chan The channel to set the stream topology on
+ * \param topology The stream topology to set
+ *
+ * \pre chan is locked
+ *
+ * \note If topology is NULL a new empty topology will be created
+ * and returned.
+ *
+ * \retval non-NULL Success
+ * \retval NULL failure
+ */
+struct ast_stream_topology *ast_channel_set_stream_topology(
+	struct ast_channel *chan, struct ast_stream_topology *topology);
+
+/*!
+ * \brief Retrieve the default stream of a specific media type on a channel
+ *
+ * \param channel The channel to get the stream from
+ * \param type The media type of the default stream
+ *
+ * \pre chan is locked
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_stream *ast_channel_get_default_stream(struct ast_channel *chan, enum ast_media_type type);
+
+/*!
+ * \brief Determine if a channel is multi-stream capable
+ *
+ * \param channel The channel to test
+ *
+ * \pre chan is locked
+ *
+ * \return Returns true if the channel is multi-stream capable.
+ */
+int ast_channel_is_multistream(struct ast_channel *chan);
+
+/*!
+ * \brief Request that the stream topology of a channel change
+ *
+ * \param chan The channel to change
+ * \param topology The new stream topology
+ * \param change_source The source that initiated the change
+ *
+ * \note Absolutely _NO_ channel locks should be held before calling this function.
+ *
+ * \retval 0 request has been accepted to be attempted
+ * \retval -1 request could not be attempted
+ *
+ * \note This function initiates an asynchronous request to change the stream topology. It is not
+ *       guaranteed that the topology will change and until an AST_CONTROL_STREAM_TOPOLOGY_CHANGED
+ *       frame is received from the channel the current handler of the channel must tolerate the
+ *       stream topology as it currently exists.
+ *
+ * \note This interface is provided for applications and resources to request that the topology change.
+ *       It is not for use by the channel driver itself.
+ */
+int ast_channel_request_stream_topology_change(struct ast_channel *chan,
+	struct ast_stream_topology *topology, void *change_source);
+
+/*!
+ * \brief Provide notice to a channel that the stream topology has changed
+ *
+ * \param chan The channel to provide notice to
+ * \param topology The new stream topology
+ *
+ * \pre chan is locked  Absolutely _NO_ other channels can be locked.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note This interface is provided for applications and resources to accept a topology change.
+ *       It is not for use by the channel driver itself.
+ */
+int ast_channel_stream_topology_changed(struct ast_channel *chan, struct ast_stream_topology *topology);
+
+/*!
+ * \brief Retrieve the source that initiated the last stream topology change
+ *
+ * \param chan The channel
+ *
+ * \retval The channel's stream topology change source
+ */
+void *ast_channel_get_stream_topology_change_source(struct ast_channel *chan);
+
 #endif /* _ASTERISK_CHANNEL_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/channel_internal.h 1:16.1.1~dfsg-1/include/asterisk/channel_internal.h
--- 1:13.23.1~dfsg-2/include/asterisk/channel_internal.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/channel_internal.h	2018-12-26 18:45:13.000000000 +0000
@@ -27,3 +27,9 @@ int ast_channel_internal_setup_topics(st
 
 void ast_channel_internal_errno_set(enum ast_channel_error error);
 enum ast_channel_error ast_channel_internal_errno(void);
+void ast_channel_internal_set_stream_topology(struct ast_channel *chan,
+	struct ast_stream_topology *topology);
+void ast_channel_internal_set_stream_topology_change_source(
+	struct ast_channel *chan, void *change_source);
+void ast_channel_internal_swap_stream_topology(struct ast_channel *chan1,
+	struct ast_channel *chan2);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/chanvars.h 1:16.1.1~dfsg-1/include/asterisk/chanvars.h
--- 1:13.23.1~dfsg-2/include/asterisk/chanvars.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/chanvars.h	2018-12-26 18:45:13.000000000 +0000
@@ -35,12 +35,10 @@ AST_LIST_HEAD_NOLOCK(varshead, ast_var_t
 
 struct varshead *ast_var_list_create(void);
 void ast_var_list_destroy(struct varshead *head);
-#ifdef __AST_DEBUG_MALLOC
+
 struct ast_var_t *_ast_var_assign(const char *name, const char *value, const char *file, int lineno, const char *function);
-#define ast_var_assign(a,b)	_ast_var_assign(a,b,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_var_t *ast_var_assign(const char *name, const char *value);
-#endif
+#define ast_var_assign(name, value) _ast_var_assign(name, value, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 void ast_var_delete(struct ast_var_t *var);
 const char *ast_var_name(const struct ast_var_t *var);
 const char *ast_var_full_name(const struct ast_var_t *var);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/cli.h 1:16.1.1~dfsg-1/include/asterisk/cli.h
--- 1:13.23.1~dfsg-2/include/asterisk/cli.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/cli.h	2018-12-26 18:45:13.000000000 +0000
@@ -177,7 +177,7 @@ struct ast_cli_entry {
 	const char * usage; 				/*!< Detailed usage information */
 
 	int inuse; 				/*!< For keeping track of usage */
-	struct module *module;			/*!< module this belongs to */
+	struct ast_module *module;			/*!< module this belongs to */
 	char *_full_cmd;			/*!< built at load time from cmda[] */
 	int cmdlen;				/*!< len up to the first invalid char [<{% */
 	/*! \brief This gets set in ast_cli_register()
@@ -253,14 +253,19 @@ int ast_cli_command_multiple_full(int ui
  * \retval 0 on success
  * \retval -1 on failure
  */
-int ast_cli_register(struct ast_cli_entry *e);
+#define ast_cli_register(e) __ast_cli_register(e, AST_MODULE_SELF)
+
+int __ast_cli_register(struct ast_cli_entry *e, struct ast_module *mod);
 
 /*!
  * \brief Register multiple commands
  * \param e pointer to first cli entry to register
  * \param len number of entries to register
  */
-int ast_cli_register_multiple(struct ast_cli_entry *e, int len);
+#define ast_cli_register_multiple(e, len) \
+	__ast_cli_register_multiple(e, len, AST_MODULE_SELF)
+
+int __ast_cli_register_multiple(struct ast_cli_entry *e, int len, struct ast_module *mod);
 
 /*!
  * \brief Unregisters a command or an array of commands
@@ -288,8 +293,6 @@ int ast_cli_unregister_multiple(struct a
  */
 char *ast_cli_generator(const char *, const char *, int);
 
-int ast_cli_generatornummatches(const char *, const char *);
-
 /*!
  * \brief Generates a NULL-terminated array of strings that
  * 1) begin with the string in the second parameter, and
@@ -358,6 +361,17 @@ int ast_cli_completion_add(char *value);
 char *ast_complete_channels(const char *line, const char *word, int pos, int state, int rpos);
 
 /*!
+ * \since 13.8
+ * \brief Print on cli a duration in seconds in format
+ * %s year(s), %s week(s), %s day(s), %s hour(s), %s second(s)
+ *
+ * \param ast_cli_args fd to print by ast_cli
+ * \param duration The time (in seconds) to print
+ * \param prefix A Prefix string to add before of duration formatted
+ */
+void ast_cli_print_timestr_fromseconds(int fd, int seconds, const char *prefix);
+
+/*
  * \brief Allow a CLI command to be executed while Asterisk is shutting down.
  *
  * CLI commands by defeault are disabled when Asterisk is shutting down. This is
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/codec.h 1:16.1.1~dfsg-1/include/asterisk/codec.h
--- 1:13.23.1~dfsg-2/include/asterisk/codec.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/codec.h	2018-12-26 18:45:13.000000000 +0000
@@ -26,9 +26,6 @@
 #ifndef _AST_CODEC_H_
 #define _AST_CODEC_H_
 
-#define AST_SMOOTHER_FLAGS_PACK(x)   ((x) << 1)
-#define AST_SMOOTHER_FLAGS_UNPACK(x) ((x) >> 1)
-
 /*! \brief Types of media */
 enum ast_media_type {
 	AST_MEDIA_TYPE_UNKNOWN = 0,
@@ -36,6 +33,7 @@ enum ast_media_type {
 	AST_MEDIA_TYPE_VIDEO,
 	AST_MEDIA_TYPE_IMAGE,
 	AST_MEDIA_TYPE_TEXT,
+	AST_MEDIA_TYPE_END,
 };
 
 struct ast_module;
@@ -78,6 +76,8 @@ struct ast_codec {
 	int (*get_length)(unsigned int samples);
 	/*! \brief Whether the media can be smoothed or not */
 	unsigned int smooth;
+	/*! \brief Flags to be passed to the smoother */
+	unsigned int smoother_flags;
 	/*! \brief The module that registered this codec */
 	struct ast_module *mod;
 };
@@ -119,7 +119,7 @@ int __ast_codec_register(struct ast_code
  * \retval 0 success
  * \retval -1 failure
  */
-#define ast_codec_register(codec) __ast_codec_register(codec, ast_module_info->self)
+#define ast_codec_register(codec) __ast_codec_register(codec, AST_MODULE_SELF)
 
 /*!
  * \brief Retrieve a codec given a name, type, and sample rate
@@ -168,6 +168,17 @@ int ast_codec_get_max(void);
 const char *ast_codec_media_type2str(enum ast_media_type type);
 
 /*!
+ * \brief Conversion function to take a media string and convert it to a media type
+ *
+ * \param media_type_str The media type string
+ *
+ * \retval The ast_media_type that corresponds to the string
+ *
+ * \since 15.0.0
+ */
+enum ast_media_type ast_media_type_from_str(const char *media_type_str);
+
+/*!
  * \brief Get the number of samples contained within a frame
  *
  * \param frame The frame itself
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/compat.h 1:16.1.1~dfsg-1/include/asterisk/compat.h
--- 1:13.23.1~dfsg-2/include/asterisk/compat.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/compat.h	2018-12-26 18:45:13.000000000 +0000
@@ -15,45 +15,30 @@
  * especially those related to header files.
  */
 
-#include "asterisk/compiler.h"
-
 #ifndef _COMPAT_H
 #define _COMPAT_H
+/* IWYU pragma: private, include "asterisk.h" */
+/* IWYU pragma: begin_exports */
+
+#include "asterisk/compiler.h"
 
 #ifndef __STDC_VERSION__
 /* flex output wants to find this defined. */
 #define	__STDC_VERSION__ 0
 #endif
 
-#ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
-#endif
-
-#ifdef HAVE_LIMITS_H
 #include <limits.h>
-#endif
-
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 #ifdef HAVE_STDDEF_H
 #include <stddef.h>
 #endif
 
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#endif
-
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-
 #include <stdarg.h>
-
-#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
-#endif
 
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>    /* not necessarily present - could be in stdlib */
@@ -63,9 +48,7 @@
 
 #include <stdio.h>	/* this is always present */
 
-#ifdef HAVE_STRING_H
 #include <string.h>
-#endif
 
 #ifndef AST_POLL_COMPAT
 #include <poll.h>
@@ -81,7 +64,7 @@
 void closefrom(int lowfd);
 #endif
 
-#if !defined(HAVE_ASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_ASPRINTF)
 int __attribute__((format(printf, 2, 3))) asprintf(char **str, const char *fmt, ...);
 #endif
 
@@ -113,7 +96,7 @@ int setenv(const char *name, const char
 char *strcasestr(const char *, const char *);
 #endif
 
-#if !defined(HAVE_STRNDUP) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_STRNDUP)
 char *strndup(const char *, size_t);
 #endif
 
@@ -133,7 +116,7 @@ uint64_t strtoq(const char *nptr, char *
 int unsetenv(const char *name);
 #endif
 
-#if !defined(HAVE_VASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_VASPRINTF)
 int __attribute__((format(printf, 2, 0))) vasprintf(char **strp, const char *fmt, va_list ap);
 #endif
 
@@ -156,7 +139,6 @@ void timersub(struct timeval *tvend, str
 
 #include <alloca.h>
 #include <strings.h>
-#include <string.h>
 #include <pthread.h>
 #include <sys/stat.h>
 #include <signal.h>
@@ -231,5 +213,5 @@ float roundf(float x);
 #ifndef NAN
 #define NAN (0.0/0.0)
 #endif
-
+/* IWYU pragma: end_exports */
 #endif
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/config.h 1:16.1.1~dfsg-1/include/asterisk/config.h
--- 1:13.23.1~dfsg-2/include/asterisk/config.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/config.h	2018-12-26 18:45:13.000000000 +0000
@@ -784,15 +784,6 @@ int ast_realtime_append_mapping(const ch
  */
 int register_config_cli(void);
 
-/*!
- * \brief Exposed re-initialization method for core process
- *
- * \details
- * This method is intended for use only with the core re-initialization and is
- * not designed to be called from any user applications.
- */
-int read_config_maps(void);
-
 /*! \brief Create a new base configuration structure */
 struct ast_config *ast_config_new(void);
 
@@ -919,12 +910,9 @@ void ast_category_destroy(struct ast_cat
 struct ast_variable *ast_category_detach_variables(struct ast_category *cat);
 void ast_category_rename(struct ast_category *cat, const char *name);
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_variable *_ast_variable_new(const char *name, const char *value, const char *filename, const char *file, const char *function, int lineno);
-#define ast_variable_new(a, b, c) _ast_variable_new(a, b, c, __FILE__, __PRETTY_FUNCTION__, __LINE__)
-#else
-struct ast_variable *ast_variable_new(const char *name, const char *value, const char *filename);
-#endif
+#define ast_variable_new(name, value, filename) _ast_variable_new(name, value, filename, __FILE__, __PRETTY_FUNCTION__, __LINE__)
+
 struct ast_config_include *ast_include_new(struct ast_config *conf, const char *from_file, const char *included_file, int is_exec, const char *exec_file, int from_lineno, char *real_included_file_name, int real_included_file_name_size);
 struct ast_config_include *ast_include_find(struct ast_config *conf, const char *included_file);
 void ast_include_rename(struct ast_config *conf, const char *from_file, const char *to_file);
@@ -997,7 +985,6 @@ int ast_config_text_file_save2(const cha
  * \return 0 on success or -1 on failure.
  */
 int ast_config_text_file_save(const char *filename, const struct ast_config *cfg, const char *generator);
-int config_text_file_save(const char *filename, const struct ast_config *cfg, const char *generator) __attribute__((deprecated));
 
 struct ast_config *ast_config_internal_load(const char *configfile, struct ast_config *cfg, struct ast_flags flags, const char *suggested_incl_file, const char *who_asked);
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/conversions.h 1:16.1.1~dfsg-1/include/asterisk/conversions.h
--- 1:13.23.1~dfsg-2/include/asterisk/conversions.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/conversions.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,62 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Kevin Harwell <kharwell@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief Conversion utility functions
+ */
+
+#ifndef _ASTERISK_CONVERSIONS_H
+#define _ASTERISK_CONVERSIONS_H
+
+/*!
+ * \brief Convert the given string to an unsigned integer
+ *
+ * This function will return failure for the following reasons:
+ *
+ *   The given string to convert is NULL
+ *   The given string to convert is empty.
+ *   The given string to convert is negative (starts with a '-')
+ *   The given string to convert contains non numeric values
+ *   Once converted the number is out of range (greater than UINT_MAX)
+ *
+ * \param str The string to convert
+ * \param res [out] The converted value
+ *
+ * \returns -1 if it fails to convert, 0 on success
+ */
+int ast_str_to_uint(const char *str, unsigned int *res);
+
+/*!
+ * \brief Convert the given string to an unsigned long
+ *
+ * This function will return failure for the following reasons:
+ *
+ *   The given string to convert is NULL
+ *   The given string to convert is empty.
+ *   The given string to convert is negative (starts with a '-')
+ *   The given string to convert contains non numeric values
+ *   Once converted the number is out of range (greater than ULONG_MAX)
+ *
+ * \param str The string to convert
+ * \param res [out] The converted value
+ *
+ * \returns -1 if it fails to convert, 0 on success
+ */
+int ast_str_to_ulong(const char *str, unsigned long *res);
+
+#endif /* _ASTERISK_CONVERSIONS_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/core_local.h 1:16.1.1~dfsg-1/include/asterisk/core_local.h
--- 1:13.23.1~dfsg-2/include/asterisk/core_local.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/core_local.h	2018-12-26 18:45:13.000000000 +0000
@@ -42,26 +42,6 @@ struct stasis_message_type;
 /* ------------------------------------------------------------------- */
 
 /*!
- * \brief Lock the "chan" and "owner" channels (and return them) on the base
- *        private structure as well as the base private structure itself.
- *
- * \deprecated - *DO NOT USE* Please use ast_local_lock_all2 instead.
- *
- * \note This also adds references to each of the above mentioned elements and
- *       also the underlying private local structure.
- * \note None of these locks should be held prior to calling this function.
- * \note To undo this process call ast_local_unlock_all.
- *
- * \since 13.8.0
- *
- * \param chan Must be a local channel
- * \param outchan The local channel's "chan" channel
- * \param outowner The local channel's "owner" channel
- */
-void ast_local_lock_all(struct ast_channel *chan, struct ast_channel **outchan,
-			struct ast_channel **outowner);
-
-/*!
  * \brief Add a reference to the local channel's private tech, lock the local channel's
  *        private base, and add references and lock both sides of the local channel.
  *
@@ -75,26 +55,10 @@ void ast_local_lock_all(struct ast_chann
  * \param base_chan [out] One side of the local channel (ref and lock added)
  * \param base_owner [out] Other side of the local channel (ref and lock added)
  */
-void ast_local_lock_all2(struct ast_channel *chan, void **tech_pvt,
+void ast_local_lock_all(struct ast_channel *chan, void **tech_pvt,
 	struct ast_channel **base_chan, struct ast_channel **base_owner);
 
 /*!
- * \brief Unlock the "chan" and "owner" channels on the base private structure
- *        as well as the base private structure itself.
- *
- * \deprecated - *DO NOT USE* Please use ast_local_unlock_all2 instead.
- *
- * \note This also removes references to each of the above mentioned elements and
- *       also the underlying private local structure.
- * \note This function should be used in conjunction with ast_local_lock_all.
- *
- * \since 13.8.0
- *
- * \param chan Must be a local channel
- */
-void ast_local_unlock_all(struct ast_channel *chan);
-
-/*!
  * \brief Remove a reference to the given local channel's private tech, unlock the given
  *        local channel's private base, and remove references and unlock both sides of
  *        given the local channel.
@@ -107,7 +71,7 @@ void ast_local_unlock_all(struct ast_cha
  * \param base_chan One side of the local channel (ref and lock removed)
  * \param base_owner Other side of the local channel (ref and lock removed)
  */
-void ast_local_unlock_all2(void *tech_pvt, struct ast_channel *base_chan,
+void ast_local_unlock_all(void *tech_pvt, struct ast_channel *base_chan,
 	struct ast_channel *base_owner);
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/core_unreal.h 1:16.1.1~dfsg-1/include/asterisk/core_unreal.h
--- 1:13.23.1~dfsg-2/include/asterisk/core_unreal.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/core_unreal.h	2018-12-26 18:45:13.000000000 +0000
@@ -40,7 +40,6 @@ extern "C" {
 
 /* Forward declare some struct names */
 struct ast_format_cap;
-struct ast_callid;
 
 /* ------------------------------------------------------------------- */
 
@@ -207,7 +206,7 @@ struct ast_unreal_pvt *ast_unreal_alloc(
 struct ast_channel *ast_unreal_new_channels(struct ast_unreal_pvt *p,
 	const struct ast_channel_tech *tech, int semi1_state, int semi2_state,
 	const char *exten, const char *context, const struct ast_assigned_ids *assignedids,
-	const struct ast_channel *requestor, struct ast_callid *callid);
+	const struct ast_channel *requestor, ast_callid callid);
 
 /*!
  * \brief Setup unreal owner and chan channels before initiating call.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/data_buffer.h 1:16.1.1~dfsg-1/include/asterisk/data_buffer.h
--- 1:13.23.1~dfsg-2/include/asterisk/data_buffer.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/data_buffer.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,173 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2018, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ * Ben Ford <bford@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Data Buffer API
+ *
+ * A data buffer acts as a ring buffer of data. It is given a fixed
+ * number of data packets to store (which may be dynamically changed).
+ * Given a number it will store a data packet at that position relative
+ * to the others. Given a number it will retrieve the given data packet
+ * if it is present. This is purposely a storage of arbitrary things so
+ * that it can be used for multiple things.
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ * \author Ben Ford <bford@digium.com>
+ */
+
+#ifndef _AST_DATA_BUFFER_H_
+#define _AST_DATA_BUFFER_H_
+
+/*!
+ * \brief A buffer of data payloads.
+ */
+struct ast_data_buffer;
+
+/*!
+ * \brief A callback function to free a data payload in a data buffer
+ *
+ * \param The data payload
+ */
+typedef void (*ast_data_buffer_free_callback)(void *data);
+
+/*!
+ * \brief Allocate a data buffer
+ *
+ * \param free_fn Callback function to free a data payload
+ * \param size The maximum number of data payloads to contain in the data buffer
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note free_fn can be NULL. It is up to the consumer of this API to ensure that memory is
+ * managed appropriately.
+ *
+ * \since 15.4.0
+ */
+struct ast_data_buffer *ast_data_buffer_alloc(ast_data_buffer_free_callback free_fn, size_t size);
+
+/*!
+ * \brief Resize a data buffer
+ *
+ * \param buffer The data buffer
+ * \param size The new maximum size of the data buffer
+ *
+ * \note If the data buffer is shrunk any old data payloads will be freed using the configured callback.
+ * The data buffer is flexible and can be used for multiple purposes. Therefore it is up to the
+ * caller of the function to know whether or not a buffer should have its size changed. Increasing
+ * the size of the buffer may make sense in some scenarios, but shrinking should always be handled
+ * with caution since data can be lost.
+ *
+ * \since 15.4.0
+ */
+void ast_data_buffer_resize(struct ast_data_buffer *buffer, size_t size);
+
+/*!
+ * \brief Place a data payload at a position in the data buffer
+ *
+ * \param buffer The data buffer
+ * \param pos The position of the data payload
+ * \param payload The data payload
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note It is up to the consumer of this API to ensure proper memory management of data payloads
+ *
+ * \since 15.4.0
+ */
+int ast_data_buffer_put(struct ast_data_buffer *buffer, size_t pos, void *payload);
+
+/*!
+ * \brief Retrieve a data payload from the data buffer
+ *
+ * \param buffer The data buffer
+ * \param pos The position of the data payload
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This does not remove the data payload from the data buffer. It will be removed when it is displaced.
+ *
+ * \since 15.4.0
+ */
+void *ast_data_buffer_get(const struct ast_data_buffer *buffer, size_t pos);
+
+/*!
+ * \brief Remove a data payload from the data buffer
+ *
+ * \param buffer The data buffer
+ * \param pos The position of the data payload
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This DOES remove the data payload from the data buffer. It does not free it, though.
+ *
+ * \since 15.5.0
+ */
+void *ast_data_buffer_remove(struct ast_data_buffer *buffer, size_t pos);
+
+/*!
+ * \brief Remove the first payload from the data buffer
+ *
+ * \param buffer The data buffer
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This DOES remove the data payload from the data buffer.
+ *
+ * \since 15.5.0
+ */
+void *ast_data_buffer_remove_head(struct ast_data_buffer *buffer);
+
+/*!
+ * \brief Free a data buffer (and all held data payloads)
+ *
+ * \param buffer The data buffer
+ *
+ * \since 15.4.0
+ */
+void ast_data_buffer_free(struct ast_data_buffer *buffer);
+
+/*!
+ * \brief Return the number of payloads in a data buffer
+ *
+ * \param buffer The data buffer
+ *
+ * \retval the number of data payloads
+ *
+ * \since 15.4.0
+ */
+size_t ast_data_buffer_count(const struct ast_data_buffer *buffer);
+
+/*!
+ * \brief Return the maximum number of payloads a data buffer can hold
+ *
+ * \param buffer The data buffer
+ *
+ * \retval the maximum number of data payloads
+ *
+ * \since 15.4.0
+ */
+size_t ast_data_buffer_max(const struct ast_data_buffer *buffer);
+
+#endif /* _AST_DATA_BUFFER_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/data.h 1:16.1.1~dfsg-1/include/asterisk/data.h
--- 1:13.23.1~dfsg-2/include/asterisk/data.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/data.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,828 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2009, Eliel C. Sardanons (LU1ALY) <eliels@gmail.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*!
- * \file
- * \brief Data retrieval API.
- * \author Brett Bryant <brettbryant@gmail.com>
- * \author Eliel C. Sardanons (LU1ALY) <eliels@gmail.com>
- * \arg \ref AstDataRetrieval
- */
-
-#ifndef ASTERISK_DATA_H
-#define ASTERISK_DATA_H
-
-#include "asterisk/frame.h"
-#include "asterisk/format_cap.h"
-
-/*!
- * \page AstDataRetrieval The Asterisk DATA retrieval API.
- *
- * This module implements an abstraction for retrieving asterisk data and
- * export it.
- *
- * \section USAGE
- *
- * \subsection Provider
- *
- * \b Register
- *
- * To register a callback use:
- *
- * \code
- *	static const struct ast_data_handler callback_handler = {
- *		.get = callback_handler_get_function,
- *	};
- *
- *	ast_data_register("/node/path", &callback_handler);
- * \endcode
- *
- * If you instead want to register multiple nodes at once use:
- * \code
- *	static const struct ast_data_handler handler_struct1 = {
- *		.get = handler_callback_read,
- *	};
- *	... other handlers ...
- *
- *	static const struct ast_data_entry list_providers[] = {
- *		AST_DATA_ENTRY("/path1/node1", &handler_struct1),
- *		AST_DATA_ENTRY("/path2/node2", &handler_struct2),
- *		AST_DATA_ENTRY("/path3/node3", &handler_struct3),
- *	};
- *
- *      ...
- *
- *	ast_data_register_multiple(list_providers, ARRAY_LEN(list_providers));
- * \endcode
- *
- * \b Unregister
- *
- * To unregister a callback function already registered you can just call:
- *
- * \code
- *	ast_data_unregister(NULL);
- * \endcode
- * And every node registered by the current module (file) will be unregistered.
- * If you want to unregister a specific node use:
- *
- * \code
- *	ast_data_unregister("/node/path");
- * \endcode
- *
- * \b Implementation
- *
- * A simple callback function implementation:
- *
- * \code
- *	#include <data.h>
- *
- *	struct test_structure {
- *		int a;
- *		double b;
- *	};
- *
- *	DATA_EXPORT_TEST_STRUCTURE(MEMBER)			\
- *		MEMBER(test_structure, a, AST_DATA_INTEGER)	\
- *		MEMBER(test_structure, b, AST_DATA_DOUBLE)
- *
- *	AST_DATA_STRUCTURE(test_structure, DATA_EXPORT_TEST_STRUCTURE)
- *
- *	static int my_callback_function(struct ast_data_search *search,
- *		struct ast_data *root_node)
- *	{
- *		struct ast_data *internal_node;
- *		struct test_structure ts = {
- *			.a = 10,
- *			.b = 20
- *		};
- *
- *		internal_node = ast_data_add_node(root_node, "test_node");
- *		if (!internal_node) {
- *			return -1;
- *		}
- *
- *		ast_data_add_structure(test_structure, internal_node, ts);
- *
- *		if (!ast_data_search_match(search, internal_node)) {
- *			ast_data_remove_node(root_node, internal_node);
- *		}
- *
- *		return 0;
- *	}
- *
- * \endcode
- *
- * \subsection Get
- *
- * \b Getting \b the \b tree
- *
- * To get the tree you need to create a query, a query is based on three parameters
- * a \b path to the provider, a \b search condition and a \b filter condition.
- * \code
- *	struct ast_data *result;
- *	struct ast_data_query query = {
- *		.path = "/asterisk/application/app_queue/queues",
- *		.search = "/queues/queue/name=queue1",
- *		.filter = "/queues/queue/name|wrapuptime|members/member/interface"
- *	};
- *
- *	result = ast_data_get(&query);
- * \endcode
- *
- * After using it you need to release the allocated memory of the returned tree:
- * \code
- *	ast_data_free(result);
- * \endcode
- *
- * \b Iterate
- *
- * To retrieve nodes from the tree, it is possible to iterate through the returned
- * nodes of the tree using:
- * \code
- *	struct ast_data_iterator *i;
- *	struct ast_data *internal_node;
- *
- *	i = ast_data_iterator_init(result_tree, "path/node_name");
- *	while ((internal_node = ast_data_iterator_next(i))) {
- *		... do something with node ...
- *	}
- *	ast_data_iterator_end(i);
- * \endcode
- * node_name is the name of the nodes to retrieve and path is the path to the internal
- * nodes to retrieve (if needed).
- *
- * \b Retrieving
- *
- * After getting the node you where searching for, you will need to retrieve its value,
- * to do that you may use one of the ast_data_retrieve_##type functions:
- * \code
- *	int a = ast_data_retrieve_int(tree, "path/to/the/node");
- *	double b = ast_data_retrieve_dbl(tree, "path/to/the/node");
- *	unsigned int c = ast_data_retrieve_bool(tree, "path/to/the/node");
- *	char *d = ast_data_retrieve_string(tree, "path/to/the/node");
- *	struct sockaddr_in e = ast_data_retrieve_ipaddr(tree, "path/to/the/node");
- *	unsigned int f = ast_data_retrieve_uint(tree, "path/to/the/node");
- *	void *g = ast_data_retrieve_ptr(tree, "path/to/the/node");
- * \endcode
- *
- */
-
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
-/*! \brief The data type of the data node. */
-enum ast_data_type {
-	AST_DATA_CONTAINER,
-	AST_DATA_INTEGER,
-	AST_DATA_UNSIGNED_INTEGER,
-	AST_DATA_DOUBLE,
-	AST_DATA_BOOLEAN,
-	AST_DATA_STRING,
-	AST_DATA_CHARACTER,
-	AST_DATA_PASSWORD,
-	AST_DATA_IPADDR,
-	AST_DATA_TIMESTAMP,
-	AST_DATA_SECONDS,
-	AST_DATA_MILLISECONDS,
-	AST_DATA_POINTER
-};
-
-/*! \brief The Data API structures version. */
-#define AST_DATA_HANDLER_VERSION 1
-#define AST_DATA_QUERY_VERSION	 1
-
-/*! \brief opaque definition of an ast_data handler, a tree node. */
-struct ast_data;
-
-/*! \brief opaque definition of an ast_data_iterator handler. */
-struct ast_data_iterator;
-
-/*! \brief opaque definition of an ast_data_search structure. */
-struct ast_data_search;
-
-/*! \brief structure retrieved from a node, with the nodes content. */
-struct ast_data_retrieve {
-	/*! \brief The type of the node retrieved. */
-	enum ast_data_type type;
-
-	union {
-		char AST_DATA_CHARACTER;
-		char *AST_DATA_STRING;
-		char *AST_DATA_PASSWORD;
-		int AST_DATA_INTEGER;
-		unsigned int AST_DATA_TIMESTAMP;
-		unsigned int AST_DATA_SECONDS;
-		unsigned int AST_DATA_MILLISECONDS;
-		double AST_DATA_DOUBLE;
-		unsigned int AST_DATA_UNSIGNED_INTEGER;
-		unsigned int AST_DATA_BOOLEAN;
-		void *AST_DATA_POINTER;
-		struct in_addr AST_DATA_IPADDR;
-		void *AST_DATA_CONTAINER;
-	} value;
-};
-
-/*!
- * \brief The get callback definition.
- */
-typedef int (*ast_data_get_cb)(const struct ast_data_search *search,
-	struct ast_data *root);
-
-/*! \brief The structure of the node handler. */
-struct ast_data_handler {
-	/*! \brief Structure version. */
-	uint32_t version;
-	/*! \brief Data get callback implementation. */
-	ast_data_get_cb get;
-};
-
-/*! \brief This entries are for multiple registers. */
-struct ast_data_entry {
-	/*! \brief Path of the node to register. */
-	const char *path;
-	/*! \brief Data handler structure. */
-	const struct ast_data_handler *handler;
-};
-
-#define AST_DATA_ENTRY(__path, __handler) { .path = __path, .handler = __handler }
-
-/*! \brief A query to the data API is specified in this structure. */
-struct ast_data_query {
-	/*! \brief Data query version. */
-	uint32_t version;
-	/*! \brief Path to the node to retrieve. */
-	char *path;
-	/*! \brief Filter string, return the internal nodes specified here.
-	 *         Setting it to NULL will return every internal node. */
-	char *filter;
-	/*! \brief Search condition. */
-	char *search;
-};
-
-/*! \brief Map the members of a structure. */
-struct ast_data_mapping_structure {
-	/*! \brief structure member name. */
-	const char *name;
-	/*! \brief structure member type. */
-	enum ast_data_type type;
-	/*! \brief member getter. */
-	union {
-		char (*AST_DATA_CHARACTER)(void *ptr);
-		char *(*AST_DATA_STRING)(void *ptr);
-		char *(*AST_DATA_PASSWORD)(void *ptr);
-		int (*AST_DATA_INTEGER)(void *ptr);
-		int (*AST_DATA_TIMESTAMP)(void *ptr);
-		int (*AST_DATA_SECONDS)(void *ptr);
-		int (*AST_DATA_MILLISECONDS)(void *ptr);
-		double (*AST_DATA_DOUBLE)(void *ptr);
-		unsigned int (*AST_DATA_UNSIGNED_INTEGER)(void *ptr);
-		unsigned int (*AST_DATA_BOOLEAN)(void *ptr);
-		void *(*AST_DATA_POINTER)(void *ptr);
-		struct in_addr (*AST_DATA_IPADDR)(void *ptr);
-		void *(*AST_DATA_CONTAINER)(void *ptr);
-	} get;
-};
-
-/* Generate the structure and the functions to access the members of a structure. */
-#define AST_DATA_STRUCTURE(__struct, __name)								\
-	__name(__AST_DATA_MAPPING_FUNCTION);								\
-	static const struct ast_data_mapping_structure __data_mapping_structure_##__struct[] = {	\
-		__name(__AST_DATA_MAPPING_STRUCTURE)							\
-	}
-
-/* Generate the structure to access the members and setup the pointer of the getter. */
-#define __AST_DATA_MAPPING_STRUCTURE(__structure, __member, __type)				\
-	{ .name = #__member, .get.__type = data_mapping_structure_get_##__structure##__member,	\
-	.type = __type },
-
-/* based on the data type, specifify the type of return value for the getter function. */
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_PASSWORD(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_PASSWORD, char *)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_STRING(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_STRING, char *)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_CHARACTER(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_CHARACTER, char)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_INTEGER(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_INTEGER, int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_TIMESTAMP(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_INTEGER, int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_SECONDS(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_INTEGER, int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_MILLISECONDS(__structure, __member)			\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_INTEGER, int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_UNSIGNED_INTEGER(__structure, __member)			\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_UNSIGNED_INTEGER, unsigned int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_BOOLEAN(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_BOOLEAN, unsigned int)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_POINTER(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_POINTER, void *)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_IPADDR(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_IPADDR, struct in_addr)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_DOUBLE(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_DBL, double)
-#define __AST_DATA_MAPPING_FUNCTION_AST_DATA_CONTAINER(__structure, __member)				\
-	__AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, AST_DATA_CONTAINER, void *)
-
-#define __AST_DATA_MAPPING_FUNCTION(__structure, __member, __type)		\
-	__AST_DATA_MAPPING_FUNCTION_##__type(__structure, __member)
-
-/* Create the function to retrieve a member of the structure. */
-#define __AST_DATA_MAPPING_FUNCTION_TYPE(__structure, __member, __type, __real_type)		\
-	static __real_type data_mapping_structure_get_##__structure##__member(void *ptr) {	\
-		struct __structure *struct_##__member = (struct __structure *) ptr;		\
-		return (__real_type) struct_##__member->__member;				\
-	}
-
-/*!
- * \brief Register a data provider.
- * \param[in] path The path of the node to register.
- * \param[in] handler The structure defining this node handler.
- * \param[in] registrar Who is registering this node.
- * \param[in] mod The module registering this handler.
- * \see ast_data_unregister
- * \retval <0 on error.
- * \retval 0 on success.
- * \see __ast_data_unregister, __ast_data_register_multiple
- */
-int __ast_data_register(const char *path, const struct ast_data_handler *handler,
-	const char *registrar, struct ast_module *mod);
-#define ast_data_register(path, handler) __ast_data_register(path, handler, __FILE__, ast_module_info->self)
-#define ast_data_register_core(path, handler) __ast_data_register(path, handler, __FILE__, NULL)
-
-/*!
- * \brief Register multiple data providers at once.
- * \param[in] data_entries An array of data_entries structures.
- * \param[in] entries The number of entries in the data_entries array.
- * \param[in] registrar Who is registering this nodes.
- * \param[in] mod The module registering this handlers.
- * \retval <0 on error (none of the nodes are being registered on error).
- * \retval 0 on success.
- * \see __ast_data_register, __ast_data_unregister
- */
-int __ast_data_register_multiple(const struct ast_data_entry *data_entries,
-	size_t entries, const char *registrar, struct ast_module *mod);
-#define ast_data_register_multiple(data_entries, entries) \
-	__ast_data_register_multiple(data_entries, entries, __FILE__, ast_module_info->self)
-#define ast_data_register_multiple_core(data_entries, entries) \
-	__ast_data_register_multiple(data_entries, entries, __FILE__, NULL)
-
-/*!
- * \brief Unregister a data provider.
- * \param[in] path Which node to unregister, if path is NULL unregister every node
- *                 registered by the passed 'registrar'.
- * \param[in] registrar Who is trying to unregister this node, only the owner (the
- *                      one who registered the node) will be able to unregister it.
- * \see ast_data_register
- * \retval <0 on error.
- * \retval 0 on success.
- * \see __ast_data_register, __ast_data_register_multiple
- */
-int __ast_data_unregister(const char *path, const char *registrar);
-#define ast_data_unregister(path) __ast_data_unregister(path, __FILE__)
-
-/*!
- * \brief Check the current generated node to know if it matches the search
- *        condition.
- * \param[in] search The search condition.
- * \param[in] data The AstData node generated.
- * \return 1 If the "data" node matches the search condition.
- * \return 0 If the "data" node does not matches the search condition.
- * \see ast_data_remove_node
- */
-int ast_data_search_match(const struct ast_data_search *search, struct ast_data *data);
-
-/*!
- * \brief Based on a search tree, evaluate every member of a structure against it.
- * \param[in] search The search tree.
- * \param[in] mapping The structure mapping.
- * \param[in] mapping_len The lenght of the structure mapping.
- * \param[in] structure The structure pointer.
- * \param[in] structure_name The name of the structure to compare.
- * \retval 0 If the structure matches.
- * \retval 1 If the structure doesn't match.
- */
-int __ast_data_search_cmp_structure(const struct ast_data_search *search,
-	const struct ast_data_mapping_structure *mapping, size_t mapping_len,
-	void *structure, const char *structure_name);
-#define ast_data_search_cmp_structure(search, structure_name, structure, structure_name_cmp)		\
-	__ast_data_search_cmp_structure(search, __data_mapping_structure_##structure_name,		\
-	ARRAY_LEN(__data_mapping_structure_##structure_name), structure, structure_name_cmp)
-
-/*!
- * \brief Retrieve a subtree from the asterisk data API.
- * \param[in] query The query structure specifying what nodes to retrieve.
- * \retval NULL on error.
- * \retval non-NULL The dynamically allocated requested sub-tree (it needs to be
- *         released using ast_data_free.
- * \see ast_data_free, ast_data_get_xml
- */
-struct ast_data *ast_data_get(const struct ast_data_query *query);
-
-#ifdef HAVE_LIBXML2
-/*!
- * \brief Retrieve a subtree from the asterisk data API in XML format..
- * \param[in] query The query structure specifying what nodes to retrieve.
- * \retval NULL on error.
- * \retval non-NULL The dynamically allocated requested sub-tree (it needs to be
- *         released using ast_data_free.
- * \see ast_data_free, ast_data_get
- */
-struct ast_xml_doc *ast_data_get_xml(const struct ast_data_query *query);
-#endif
-
-/*!
- * \brief Release the allocated memory of a tree.
- * \param[in] root The sub-tree pointer returned by a call to ast_data_get.
- * \see ast_data_get
- */
-void ast_data_free(struct ast_data *root);
-
-/*!
- * \brief Get a node type.
- * \param[in] res A pointer to the ast_data result set.
- * \param[in] path A path to the node to get the type.
- * \return The type of the requested node type.
- */
-enum ast_data_type ast_data_retrieve_type(struct ast_data *res, const char *path);
-
-/*!
- * \brief Get the node name.
- * \param[in] node The node pointer.
- * \returns The node name.
- */
-char *ast_data_retrieve_name(struct ast_data *node);
-
-/*!
- * \brief Add a container child.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_node(struct ast_data *root, const char *childname);
-
-/*!
- * \brief Add an integer node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] value The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_int(struct ast_data *root, const char *childname,
-	int value);
-
-/*!
- * \brief Add a char node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] value The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_char(struct ast_data *root, const char *childname,
-	char value);
-
-/*!
- * \brief Add an unsigned integer node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] value The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_uint(struct ast_data *root, const char *childname,
-	unsigned int value);
-
-/*!
- * \brief Add a floating point node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] dbl The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_dbl(struct ast_data *root, const char *childname,
-	double dbl);
-/*!
- * \brief Add a ipv4 address type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] addr The ipv4 address value.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_ipaddr(struct ast_data *root, const char *childname,
-	struct in_addr addr);
-
-/*!
- * \brief Add a ptr node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] ptr The pointer value to add.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_ptr(struct ast_data *root, const char *childname,
-	void *ptr);
-
-/*!
- * \brief Add a password node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] string The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_password(struct ast_data *root, const char *childname,
-	const char *string);
-
-/*!
- * \brief Add a timestamp node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] timestamp The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_timestamp(struct ast_data *root, const char *childname,
-	unsigned int timestamp);
-
-/*!
- * \brief Add a seconds node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] seconds The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_seconds(struct ast_data *root, const char *childname,
-	unsigned int seconds);
-
-/*!
- * \brief Add a milliseconds node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] milliseconds The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_milliseconds(struct ast_data *root, const char *childname,
-	unsigned int milliseconds);
-
-/*!
- * \brief Add a string node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] string The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_str(struct ast_data *root, const char *childname,
-	const char *string);
-
-/*!
- * \brief Add a boolean node type.
- * \param[in] root The root of the ast_data to insert into.
- * \param[in] childname The name of the child element to be added.
- * \param[in] boolean The value for the new node.
- * \retval NULL on error (memory exhaustion only).
- * \retval non-NULL a newly allocated node.
- */
-struct ast_data *ast_data_add_bool(struct ast_data *root, const char *childname,
-	unsigned int boolean);
-
-/*!
- * \brief Add a complete structure to a node.
- * \param[in] root Where to add the structure.
- * \param[in] mapping The structure mapping array.
- * \param[in] mapping_len The lenght of the mapping array.
- * \param[in] structure The structure pointer.
- * \retval 0 on success.
- * \retval 1 on error.
- */
-int __ast_data_add_structure(struct ast_data *root,
-	const struct ast_data_mapping_structure *mapping,
-	size_t mapping_len, void *structure);
-#define ast_data_add_structure(structure_name, root, structure)				\
-	__ast_data_add_structure(root, __data_mapping_structure_##structure_name,	\
-		ARRAY_LEN(__data_mapping_structure_##structure_name), structure)
-
-/*!
- * \brief Remove a node that was added using ast_data_add_
- * \param[in] root The root node of the node to be removed.
- * \param[in] child The node pointer to remove.
- */
-void ast_data_remove_node(struct ast_data *root, struct ast_data *child);
-
-/*!
- * \brief Initialize an iterator.
- * \param[in] tree The returned tree by a call to ast_data_get.
- * \param[in] elements Which elements to iterate through.
- * \retval NULL on error.
- * \retval non-NULL A dinamically allocated iterator structure.
- */
-struct ast_data_iterator *ast_data_iterator_init(struct ast_data *tree,
-	const char *elements);
-
-/*!
- * \brief Release (stop using) an iterator.
- * \param[in] iterator The iterator created by ast_data_iterator_start.
- * \see ast_data_iterator_start
- */
-void ast_data_iterator_end(struct ast_data_iterator *iterator);
-
-/*!
- * \brief Get the next node of the tree.
- * \param[in] iterator The iterator structure returned by ast_data_iterator_start.
- * \retval NULL when no more nodes to return.
- * \retval non-NULL A node of the ast_data tree.
- * \see ast_data_iterator_start, ast_data_iterator_stop
- */
-struct ast_data *ast_data_iterator_next(struct ast_data_iterator *iterator);
-
-/*!
- * \brief Retrieve a value from a node in the tree.
- * \param[in] tree The structure returned by a call to ast_data_get.
- * \param[in] path The path to the node.
- * \param[out] content The node content.
- * \retval 0 on success.
- * \retval <0 on error.
- */
-int ast_data_retrieve(struct ast_data *tree, const char *path, struct ast_data_retrieve *content);
-
-/*!
- * \brief Retrieve the integer value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline int ast_data_retrieve_int(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_INTEGER;
-}
-
-/*!
- * \brief Retrieve the character value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline char ast_data_retrieve_char(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_CHARACTER;
-}
-
-/*!
- * \brief Retrieve the boolean value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline unsigned int ast_data_retrieve_bool(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_BOOLEAN;
-}
-
-/*!
- * \brief Retrieve the unsigned integer value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline unsigned int ast_data_retrieve_uint(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_UNSIGNED_INTEGER;
-}
-
-/*!
- * \brief Retrieve the password value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline const char *ast_data_retrieve_password(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_PASSWORD;
-}
-
-/*!
- * \brief Retrieve the string value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline const char *ast_data_retrieve_string(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_STRING;
-}
-
-/*!
- * \brief Retrieve the ptr value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline void *ast_data_retrieve_ptr(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_POINTER;
-}
-
-/*!
- * \brief Retrieve the double value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline double ast_data_retrieve_dbl(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_DOUBLE;
-}
-
-/*!
- * \brief Retrieve the ipv4 address value of a node.
- * \param[in] tree The tree from where to get the value.
- * \param[in] path The node name or path.
- * \returns The value of the node.
- */
-static inline struct in_addr ast_data_retrieve_ipaddr(struct ast_data *tree, const char *path)
-{
-	struct ast_data_retrieve ret;
-
-	ast_data_retrieve(tree, path, &ret);
-
-	return ret.value.AST_DATA_IPADDR;
-}
-
-/*!
- * \brief Add the codec in the root node based on the format parameter.
- * \param[in] root The astdata root node where to add the codec node.
- * \param[in] node_name The name of the node where we are going to add the codec.
- * \param[in] format The codec allowed.
- * \return < 0 on error.
- * \return 0 on success.
- */
-int ast_data_add_codec(struct ast_data *root, const char *node_name, struct ast_format *format);
-
-/*!
- * \brief Add the list of codecs in the root node based on the capability parameter.
- * \param[in] root The astdata root node where to add the codecs node.
- * \param[in] node_name The name of the node where we are going to add the list of
- *                      codecs.
- * \param[in] capability The codecs allowed.
- * \return < 0 on error.
- * \return 0 on success.
- */
-int ast_data_add_codecs(struct ast_data *root, const char *node_name, struct ast_format_cap *capability);
-
-#if defined(__cplusplus) || defined(c_plusplus)
-}
-#endif
-
-#endif /* ASTERISK_DATA_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/datastore.h 1:16.1.1~dfsg-1/include/asterisk/datastore.h
--- 1:13.23.1~dfsg-2/include/asterisk/datastore.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/datastore.h	2018-12-26 18:45:13.000000000 +0000
@@ -69,6 +69,7 @@ struct ast_datastore {
 	const char *uid;			/*!< Unique data store identifier */
 	void *data;				/*!< Contained data */
 	const struct ast_datastore_info *info;	/*!< Data store type information */
+	struct ast_module *mod;			/*!< Module referenced by this datastore */
 	unsigned int inheritance;		/*!< Number of levels this item will continue to be inherited */
 	AST_LIST_ENTRY(ast_datastore) entry; 	/*!< Used for easy linking */
 };
@@ -77,13 +78,16 @@ struct ast_datastore {
  * \brief Create a data store object
  * \param[in] info information describing the data store object
  * \param[in] uid unique identifer
+ * \param[in] mod The module to hold until this datastore is freed.
  * \param file, line, function
  * \version 1.6.1 moved here and renamed from ast_channel_datastore_alloc
  */
-struct ast_datastore *__ast_datastore_alloc(const struct ast_datastore_info *info, const char *uid,
-							      const char *file, int line, const char *function);
+struct ast_datastore *__ast_datastore_alloc(
+	const struct ast_datastore_info *info, const char *uid, struct ast_module *mod,
+	const char *file, int line, const char *function);
 
-#define ast_datastore_alloc(info, uid) __ast_datastore_alloc(info, uid, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define ast_datastore_alloc(info, uid) \
+	__ast_datastore_alloc(info, uid, AST_MODULE_SELF, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Free a data store object
@@ -92,6 +96,64 @@ struct ast_datastore *__ast_datastore_al
  */
 int ast_datastore_free(struct ast_datastore *datastore);
 
+/*!
+ * \brief Allocate a specialized data stores container
+ *
+ * \return a container for storing data stores
+ *
+ * \since 14.0.0
+ */
+struct ao2_container *ast_datastores_alloc(void);
+
+/*!
+ * \brief Add a data store to a container
+ *
+ * \param[in] datastores container to store datastore in
+ * \param[in] datastore datastore to add
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \since 14.0.0
+ */
+int ast_datastores_add(struct ao2_container *datastores, struct ast_datastore *datastore);
+
+/*!
+ * \brief Remove a data store from a container
+ *
+ * \param[in] datastores container to remove datastore from
+ * \param[in] name name of the data store to remove
+ *
+ * \since 14.0.0
+ */
+void ast_datastores_remove(struct ao2_container *datastores, const char *name);
+
+/*!
+ * \brief Find a data store in a container
+ *
+ * \param[in] datastores container to find datastore in
+ * \param[in] name name of the data store to find
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 14.0.0
+ */
+struct ast_datastore *ast_datastores_find(struct ao2_container *datastores, const char *name);
+
+/*!
+ * \brief Allocate a datastore for use with the datastores container
+ *
+ * \param[in] info information about the datastore
+ * \param[in] uid unique identifier for the datastore
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 14.0.0
+ */
+struct ast_datastore *ast_datastores_alloc_datastore(const struct ast_datastore_info *info, const char *uid);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/devicestate.h 1:16.1.1~dfsg-1/include/asterisk/devicestate.h
--- 1:13.23.1~dfsg-2/include/asterisk/devicestate.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/devicestate.h	2018-12-26 18:45:13.000000000 +0000
@@ -86,7 +86,6 @@ enum ast_device_state ast_state_chan2dev
  *
  * \param devstate Current device state
  */
-const char *devstate2str(enum ast_device_state devstate) attribute_pure __attribute__((deprecated));
 const char *ast_devstate2str(enum ast_device_state devstate) attribute_pure;
 
 /*!
@@ -167,41 +166,6 @@ int ast_devstate_changed(enum ast_device
 int ast_devstate_changed_literal(enum ast_device_state state, enum ast_devstate_cache cachable, const char *device);
 
 /*!
- * \brief Tells Asterisk the State for Device is changed.
- * (Accept change notification, add it to change queue.)
- *
- * \param fmt device name like a dial string with format parameters
- *
- * Asterisk polls the new extension states and calls the registered
- * callbacks for the changed extensions
- *
- * \retval 0 on success
- * \retval -1 on failure
- *
- * \note This is deprecated in favor of ast_devstate_changed()
- * \version 1.6.1 deprecated
- */
-int ast_device_state_changed(const char *fmt, ...)
-	__attribute__((deprecated,format(printf, 1, 2)));
-
-/*!
- * \brief Tells Asterisk the State for Device is changed
- *
- * \param device device name like a dial string
- *
- * Asterisk polls the new extension states and calls the registered
- * callbacks for the changed extensions
- *
- * \retval 0 on success
- * \retval -1 on failure
- *
- * \note This is deprecated in favor of ast_devstate_changed_literal()
- * \version 1.6.1 deprecated
- */
-int ast_device_state_changed_literal(const char *device)
-	__attribute__((deprecated));
-
-/*!
  * \brief Add device state provider
  *
  * \param label to use in hint, like label:object
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dial.h 1:16.1.1~dfsg-1/include/asterisk/dial.h
--- 1:13.23.1~dfsg-2/include/asterisk/dial.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dial.h	2018-12-26 18:45:13.000000000 +0000
@@ -76,6 +76,18 @@ struct ast_dial *ast_dial_create(void);
  */
 int ast_dial_append(struct ast_dial *dial, const char *tech, const char *device, const struct ast_assigned_ids *assignedids);
 
+/*!
+ * \brief Append a channel using an actual channel object
+ *
+ * \param dial The ast_dial to add the channel to
+ * \param chan The channel to add to the dial
+ * \retval -1 Failure
+ * \retval non-zero The position of the channel in the list of dialed channels
+ *
+ * \note The chan ref is stolen with a successful return.
+ */
+int ast_dial_append_channel(struct ast_dial *dial, struct ast_channel *chan);
+
 /*! \brief Request all appended channels, but do not dial
  * \param dial Dialing structure
  * \param chan Optional dialing channel
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_core.h 1:16.1.1~dfsg-1/include/asterisk/dns_core.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_core.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_core.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,276 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief Core DNS API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_CORE_H
+#define _ASTERISK_DNS_CORE_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*! \brief Opaque structure for an active DNS query */
+struct ast_dns_query_active;
+
+/*! \brief Opaque structure for a DNS query */
+struct ast_dns_query;
+
+/*!
+ * \brief Get the name queried in a DNS query
+ *
+ * \param query The DNS query
+ *
+ * \return the name queried
+ */
+const char *ast_dns_query_get_name(const struct ast_dns_query *query);
+
+/*!
+ * \brief Get the record resource type of a DNS query
+ *
+ * \param query The DNS query
+ *
+ * \return the record resource type
+ */
+int ast_dns_query_get_rr_type(const struct ast_dns_query *query);
+
+/*!
+ * \brief Get the record resource class of a DNS query
+ *
+ * \param query The DNS query
+ *
+ * \return the record resource class
+ */
+int ast_dns_query_get_rr_class(const struct ast_dns_query *query);
+
+/*!
+ * \brief Get the user specific data of a DNS query
+ *
+ * \param query The DNS query
+ *
+ * \return the user specific data
+ *
+ * \note The reference count of the data is NOT incremented on return
+ */
+void *ast_dns_query_get_data(const struct ast_dns_query *query);
+
+/*! \brief Opaque structure for a DNS query result, guaranteed to be immutable */
+struct ast_dns_result;
+
+/*!
+ * \brief Get the result information for a DNS query
+ *
+ * \param query The DNS query
+ *
+ * \return the DNS result information
+ *
+ * \note The result is NOT ao2 allocated
+ */
+struct ast_dns_result *ast_dns_query_get_result(const struct ast_dns_query *query);
+
+/*!
+ * \brief Get whether the result is secure or not
+ *
+ * \param result The DNS result
+ *
+ * \return whether the result is secure or not
+ */
+unsigned int ast_dns_result_get_secure(const struct ast_dns_result *result);
+
+/*!
+ * \brief Get whether the result is bogus or not
+ *
+ * \param result The DNS result
+ *
+ * \return whether the result is bogus or not
+ */
+unsigned int ast_dns_result_get_bogus(const struct ast_dns_result *result);
+
+/*!
+ * \brief Get the error rcode of a DN result
+ *
+ * \param query The DNS result
+ *
+ * \return the DNS rcode
+ */
+unsigned int ast_dns_result_get_rcode(const struct ast_dns_result *result);
+
+/*!
+ * \brief Get the canonical name of the result
+ *
+ * \param result The DNS result
+ *
+ * \return the canonical name
+ */
+const char *ast_dns_result_get_canonical(const struct ast_dns_result *result);
+
+/*!
+ * \brief Get the first record of a DNS Result
+ *
+ * \param result The DNS result
+ *
+ * \return first DNS record
+ */
+const struct ast_dns_record *ast_dns_result_get_records(const struct ast_dns_result *result);
+
+/*!
+ * \brief Get the raw DNS answer from a DNS result
+ *
+ * \param result The DNS result
+ *
+ * \return The DNS result
+ */
+const char *ast_dns_result_get_answer(const struct ast_dns_result *result);
+
+/*!
+ * \brief Retrieve the lowest TTL from a result
+ *
+ * \param result The DNS result
+ *
+ * \return the lowest TTL
+ *
+ * \note If no records exist this function will return a TTL of 0
+ */
+int ast_dns_result_get_lowest_ttl(const struct ast_dns_result *result);
+
+/*!
+ * \brief Free the DNS result information
+ *
+ * \param result The DNS result
+ */
+void ast_dns_result_free(struct ast_dns_result *result);
+
+/*! \brief Opaque structure for a DNS record */
+struct ast_dns_record;
+
+/*!
+ * \brief Callback invoked when a query completes
+ *
+ * \param query The DNS query that was invoked
+ */
+typedef void (*ast_dns_resolve_callback)(const struct ast_dns_query *query);
+
+/*!
+ * \brief Get the resource record type of a DNS record
+ *
+ * \param record The DNS record
+ *
+ * \return the resource record type
+ */
+int ast_dns_record_get_rr_type(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the resource record class of a DNS record
+ *
+ * \param record The DNS record
+ *
+ * \return the resource record class
+ */
+int ast_dns_record_get_rr_class(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the TTL of a DNS record
+ *
+ * \param record The DNS record
+ *
+ * \return the TTL
+ */
+int ast_dns_record_get_ttl(const struct ast_dns_record *record);
+
+/*!
+ * \brief Retrieve the raw DNS record
+ *
+ * \param record The DNS record
+ *
+ * \return the raw DNS record
+ */
+const char *ast_dns_record_get_data(const struct ast_dns_record *record);
+
+/*!
+ * \brief Retrieve the size of the raw DNS record
+ *
+ * \param record The DNS record
+ *
+ * \return the size of the raw DNS record
+ */
+size_t ast_dns_record_get_data_size(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the next DNS record
+ *
+ * \param record The current DNS record
+ *
+ * \return the next DNS record
+ */
+const struct ast_dns_record *ast_dns_record_get_next(const struct ast_dns_record *record);
+
+/*!
+ * \brief Asynchronously resolve a DNS query
+ *
+ * \param name The name of what to resolve
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ * \param callback The callback to invoke upon completion
+ * \param data User data to make available on the query
+ *
+ * \retval non-NULL success - query has been sent for resolution
+ * \retval NULL failure
+ *
+ * \note The result passed to the callback does not need to be freed
+ *
+ * \note The user data MUST be an ao2 object
+ *
+ * \note This function increments the reference count of the user data, it does NOT steal
+ *
+ * \note The active query must be released upon completion or cancellation using ao2_ref
+ */
+struct ast_dns_query_active *ast_dns_resolve_async(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data);
+
+/*!
+ * \brief Cancel an asynchronous DNS resolution
+ *
+ * \param active The active DNS query returned from ast_dns_resolve_async
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note If successfully cancelled the callback will not be invoked
+ */
+int ast_dns_resolve_cancel(struct ast_dns_query_active *active);
+
+/*!
+ * \brief Synchronously resolve a DNS query
+ *
+ * \param name The name of what to resolve
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ * \param result A pointer to hold the DNS result
+ *
+ * \retval 0 success - query was completed and result is available
+ * \retval -1 failure
+ */
+int ast_dns_resolve(const char *name, int rr_type, int rr_class, struct ast_dns_result **result);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_CORE_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns.h 1:16.1.1~dfsg-1/include/asterisk/dns.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns.h	2018-12-26 18:45:13.000000000 +0000
@@ -24,17 +24,63 @@
 #ifndef _ASTERISK_DNS_H
 #define _ASTERISK_DNS_H
 
-/*!	\brief	Perform DNS lookup (used by DNS, enum and SRV lookups)
-	\param	context
-	\param	dname	Domain name to lookup (host, SRV domain, TXT record name)
-	\param	class	Record Class (see "man res_search")
-	\param	type	Record type (see "man res_search")
-	\param	callback Callback function for handling DNS result
-	\note   Asterisk DNS is synchronus at this time. This means that if your DNS
-		services does not work, Asterisk may lock while waiting for response.
-*/
+/*! \brief DNS search return values */
+enum ast_dns_search_result {
+	AST_DNS_SEARCH_FAILURE = -1,     /*!< DNS search resulted in failure */
+	AST_DNS_SEARCH_NO_RECORDS = 0,   /*!< DNS search yielded no results */
+	AST_DNS_SEARCH_SUCCESS = 1       /*!< DNS search yielded at least one discovered record */
+};
+
+/*!
+ * \brief Perform DNS lookup (used by DNS, enum and SRV lookups)
+ *
+ * \param  context   Void pointer containing data to use in the callback function.
+ * \param  dname     Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  class     Record Class (see "man res_search").
+ * \param  type      Record type (see "man res_search").
+ * \param  answer    The full DNS response.
+ * \param  len       The length of the full DNS response.
+ * \param  callback  Callback function for handling the discovered resource records from
+ *                   the DNS search.
+ *
+ * \retval -1 on search failure
+ * \retval  0 on no records found
+ * \retval  1 on success
+ *
+ * \note Asterisk DNS is synchronus at this time. This means that if your DNS
+ *       service does not work, Asterisk may lock while waiting for a response.
+ */
 int ast_search_dns(void *context, const char *dname, int class, int type,
-	 int (*callback)(void *context, unsigned char *answer, int len, unsigned char *fullanswer));
+	int (*callback)(void *context, unsigned char *answer, int len, unsigned char *fullanswer));
+
+/*!
+ * \brief Extended version of the DNS search function.
+ *
+ * \details Performs a DNS lookup, (used by DNS, enum and SRV lookups), parses the
+ *          results and notifies the observer with the response and discovered records
+ *          via invoking the provided callbacks (used by ast_dns_system_resolver).
+ *
+ * \param  context           Void pointer containing data to use in the handler functions.
+ * \param  dname             Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  rr_class          Record Class (see "man res_search").
+ * \param  rr_type           Record type (see "man res_search").
+ * \param  response_handler  Callback function for handling the DNS response. Invoked upon
+ *                           completion of the DNS search.
+ * \param  record_handler    Callback function for handling the discovered resource
+ *                           records from the DNS search. Invoked n times, where n is the
+ *                           number of records discovered while parsing the DNS
+ *                           response.
+ *
+ * \retval AST_DNS_SEARCH_FAILURE    on search failure
+ * \retval AST_DNS_SEARCH_NO_RECORDS on no records found
+ * \retval AST_DNS_SEARCH_SUCCESS    on success
+ *
+ * \note Asterisk DNS is synchronus at this time. This means that if your DNS
+ *       service does not work, Asterisk may lock while waiting for a response.
+ */
+enum ast_dns_search_result ast_search_dns_ex(void *context, const char *dname, int rr_class, int rr_type,
+	int (*response_handler)(void *context, unsigned char *dns_response, int dns_response_len, int rcode),
+	int (*record_handler)(void *context, unsigned char *record, int record_len, int ttl));
 
 /*! \brief Retrieve the configured nameservers of the system */
 struct ao2_container *ast_dns_get_nameservers(void);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_internal.h 1:16.1.1~dfsg-1/include/asterisk/dns_internal.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_internal.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_internal.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,295 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Internal DNS structure definitions
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*! \brief For AST_VECTOR */
+#include "asterisk/vector.h"
+
+/*! \brief For ast_dns_query_set_callback */
+#include "asterisk/dns_query_set.h"
+
+/*! \brief Generic DNS record information */
+struct ast_dns_record {
+	/*! \brief Resource record type */
+	int rr_type;
+	/*! \brief Resource record class */
+	int rr_class;
+	/*! \brief Time-to-live of the record */
+	int ttl;
+	/*! \brief The size of the raw DNS record */
+	size_t data_len;
+	/*! \brief Linked list information */
+	AST_LIST_ENTRY(ast_dns_record) list;
+	/*! \brief pointer to record-specific data.
+	 *
+	 * For certain "subclasses" of DNS records, the
+	 * location of the raw DNS data will differ from
+	 * the generic case. This pointer will reliably
+	 * be set to point to the raw DNS data, no matter
+	 * where in the structure it may lie.
+	 */
+	char *data_ptr;
+	/*! \brief The raw DNS record */
+	char data[0];
+};
+
+/*! \brief An SRV record */
+struct ast_dns_srv_record {
+	/*! \brief Generic DNS record information */
+	struct ast_dns_record generic;
+	/*! \brief The hostname in the SRV record */
+	const char *host;
+	/*! \brief The priority of the SRV record */
+	unsigned short priority;
+	/*! \brief The weight of the SRV record */
+	unsigned short weight;
+	/*! \brief The port in the SRV record */
+	unsigned short port;
+	/*! \brief The running weight sum */
+	unsigned int weight_sum;
+	/*! \brief Additional data */
+	char data[0];
+};
+
+/*! \brief A NAPTR record */
+struct ast_dns_naptr_record {
+	/*! \brief Generic DNS record information */
+	struct ast_dns_record generic;
+	/*! \brief The flags from the NAPTR record */
+	const char *flags;
+	/*! \brief The service from the NAPTR record */
+	const char *service;
+	/*! \brief The regular expression from the NAPTR record */
+	const char *regexp;
+	/*! \brief The replacement from the NAPTR record */
+	const char *replacement;
+	/*! \brief The order for the NAPTR record */
+	unsigned short order;
+	/*! \brief The preference of the NAPTR record */
+	unsigned short preference;
+	/*! \brief Buffer for NAPTR-specific data
+	 *
+	 * This includes the raw NAPTR record, as well as
+	 * the area where the flags, service, regexp, and
+	 * replacement strings are stored.
+	 */
+	char data[0];
+};
+
+/*! \brief The result of a DNS query */
+struct ast_dns_result {
+	/*! \brief Whether the result is secure */
+	unsigned int secure;
+	/*! \brief Whether the result is bogus */
+	unsigned int bogus;
+	/*! \brief Optional rcode, set if an error occurred */
+	unsigned int rcode;
+	/*! \brief Records returned */
+	AST_LIST_HEAD_NOLOCK(dns_records, ast_dns_record) records;
+	/*! \brief The canonical name */
+	const char *canonical;
+	/*! \brief The raw DNS answer */
+	const char *answer;
+	/*! \brief The size of the raw DNS answer */
+	size_t answer_size;
+	/*! \brief Buffer for dynamic data */
+	char buf[0];
+};
+
+/*! \brief A DNS query */
+struct ast_dns_query {
+	/*! \brief Callback to invoke upon completion */
+	ast_dns_resolve_callback callback;
+	/*! \brief User-specific data */
+	void *user_data;
+	/*! \brief The resolver in use for this query */
+	struct ast_dns_resolver *resolver;
+	/*! \brief Resolver-specific data */
+	void *resolver_data;
+	/*! \brief Result of the DNS query */
+	struct ast_dns_result *result;
+	/*! \brief Resource record type */
+	int rr_type;
+	/*! \brief Resource record class */
+	int rr_class;
+	/*! \brief The name of what is being resolved */
+	char name[0];
+};
+
+/*! \brief A recurring DNS query */
+struct ast_dns_query_recurring {
+	/*! \brief Callback to invoke upon completion */
+	ast_dns_resolve_callback callback;
+	/*! \brief User-specific data */
+	void *user_data;
+	/*! \brief Current active query */
+	struct ast_dns_query_active *active;
+	/*! \brief The recurring query has been cancelled */
+	unsigned int cancelled;
+	/*! \brief Scheduled timer for next resolution */
+	int timer;
+	/*! \brief Resource record type */
+	int rr_type;
+	/*! \brief Resource record class */
+	int rr_class;
+	/*! \brief The name of what is being resolved */
+	char name[0];
+};
+
+/*! \brief A DNS query set query, which includes its state */
+struct dns_query_set_query {
+	/*! \brief Whether the query started successfully or not */
+	unsigned int started;
+	/*! \brief The query itself */
+	struct ast_dns_query *query;
+};
+
+/*! \brief A set of DNS queries */
+struct ast_dns_query_set {
+	/*! \brief DNS queries */
+	AST_VECTOR(, struct dns_query_set_query) queries;
+	/*! \brief Whether the query set is in progress or not */
+	int in_progress;
+	/*! \brief The total number of completed queries */
+	int queries_completed;
+	/*! \brief The total number of cancelled queries */
+	int queries_cancelled;
+	/*! \brief Callback to invoke upon completion */
+	ast_dns_query_set_callback callback;
+	/*! \brief User-specific data */
+	void *user_data;
+};
+
+/*! \brief An active DNS query */
+struct ast_dns_query_active {
+	/*! \brief The underlying DNS query */
+	struct ast_dns_query *query;
+};
+
+struct ast_sched_context;
+
+/*!
+ * \brief Retrieve the DNS scheduler context
+ *
+ * \return scheduler context
+ */
+struct ast_sched_context *ast_dns_get_sched(void);
+
+/*!
+ * \brief Allocate and parse a DNS NAPTR record
+ *
+ * \param query The DNS query
+ * \param data This specific NAPTR record
+ * \param size The size of the NAPTR record
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_dns_record *dns_naptr_alloc(struct ast_dns_query *query, const char *data, const size_t size);
+
+/*!
+ * \brief Sort the NAPTR records on a result
+ *
+ * \param result The DNS result
+ */
+void dns_naptr_sort(struct ast_dns_result *result);
+
+/*!
+ * \brief Allocate and parse a DNS SRV record
+ *
+ * \param query The DNS query
+ * \param data This specific SRV record
+ * \param size The size of the SRV record
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_dns_record *dns_srv_alloc(struct ast_dns_query *query, const char *data, const size_t size);
+
+/*!
+ * \brief Sort the SRV records on a result
+ *
+ * \param result The DNS result
+ */
+void dns_srv_sort(struct ast_dns_result *result);
+
+/*!
+ * \brief Find the location of a DNS record within the entire DNS answer
+ *
+ * The DNS record that has been returned by the resolver may be a copy of the record that was
+ * found in the complete DNS response. If so, then some DNS record types (specifically those that
+ * parse domains) will need to locate the DNS record within the complete DNS response. This is so
+ * that if the domain contains pointers to other sections of the DNS response, then the referenced
+ * domains may be located.
+ *
+ * \param record The DNS record returned by a resolver implementation
+ * \param record_size The size of the DNS record in bytes
+ * \param response The complete DNS answer
+ * \param response_size The size of the complete DNS response
+ */
+char *dns_find_record(const char *record, size_t record_size, const char *response, size_t response_size);
+
+/*!
+ * \brief Parse a 16-bit unsigned value from a DNS record
+ *
+ * \param cur Pointer to the location of the 16-bit value in the DNS record
+ * \param[out] val The parsed 16-bit unsigned integer
+ * \return The number of bytes consumed while parsing
+ */
+int dns_parse_short(unsigned char *cur, uint16_t *val);
+
+/*!
+ * \brief Parse a DNS string from a DNS record
+ *
+ * A DNS string consists of an 8-bit size, followed by the
+ * string value (not NULL-terminated).
+ *
+ * \param cur Pointer to the location of the DNS string
+ * \param[out] size The parsed size of the DNS string
+ * \param[out] val The contained string (not NULL-terminated)
+ * \return The number of bytes consumed while parsing
+ */
+int dns_parse_string(char *cur, uint8_t *size, char **val);
+
+/*!
+ * \brief Allocate a DNS query (but do not start resolution)
+ *
+ * \param name The name of what to resolve
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ * \param callback The callback to invoke upon completion
+ * \param data User data to make available on the query
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The result passed to the callback does not need to be freed
+ *
+ * \note The user data MUST be an ao2 object
+ *
+ * \note This function increments the reference count of the user data, it does NOT steal
+ *
+ * \note The query must be released upon completion or cancellation using ao2_ref
+ */
+struct ast_dns_query *dns_query_alloc(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_naptr.h 1:16.1.1~dfsg-1/include/asterisk/dns_naptr.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_naptr.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_naptr.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,89 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS NAPTR Record Parsing API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_NAPTR_H
+#define _ASTERISK_DNS_NAPTR_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*!
+ * \brief Get the flags from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the flags
+ */
+const char *ast_dns_naptr_get_flags(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the service from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the service
+ */
+const char *ast_dns_naptr_get_service(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the regular expression from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the regular expression
+ */
+const char *ast_dns_naptr_get_regexp(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the replacement value from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the replacement value
+ */
+const char *ast_dns_naptr_get_replacement(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the order from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the order
+ */
+unsigned short ast_dns_naptr_get_order(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the preference from a NAPTR record
+ *
+ * \param record The DNS record
+ *
+ * \return the preference
+ */
+unsigned short ast_dns_naptr_get_preference(const struct ast_dns_record *record);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_NAPTR_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_query_set.h 1:16.1.1~dfsg-1/include/asterisk/dns_query_set.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_query_set.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_query_set.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,136 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS Query Set API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_QUERY_SET_H
+#define _ASTERISK_DNS_QUERY_SET_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*! \brief Opaque structure for a set of DNS queries */
+struct ast_dns_query_set;
+
+/*!
+ * \brief Callback invoked when a query set completes
+ *
+ * \param query_set The DNS query set that was invoked
+ */
+typedef void (*ast_dns_query_set_callback)(const struct ast_dns_query_set *query_set);
+
+/*!
+ * \brief Create a query set to hold queries
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The query set must be released upon cancellation or completion using ao2_ref
+ */
+struct ast_dns_query_set *ast_dns_query_set_create(void);
+
+/*!
+ * \brief Add a query to a query set
+ *
+ * \param query_set A DNS query set
+ * \param name The name of what to resolve
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_dns_query_set_add(struct ast_dns_query_set *query_set, const char *name, int rr_type, int rr_class);
+
+/*!
+ * \brief Retrieve the number of queries in a query set
+ *
+ * \param query_set A DNS query set
+ *
+ * \return the number of queries
+ */
+size_t ast_dns_query_set_num_queries(const struct ast_dns_query_set *query_set);
+
+/*!
+ * \brief Retrieve a query from a query set
+ *
+ * \param query_set A DNS query set
+ * \param index The index of the query to retrieve
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The returned query is only valid for the lifetime of the query set itself
+ */
+struct ast_dns_query *ast_dns_query_set_get(const struct ast_dns_query_set *query_set, unsigned int index);
+
+/*!
+ * \brief Retrieve user specific data from a query set
+ *
+ * \param query_set A DNS query set
+ *
+ * \return user specific data
+ */
+void *ast_dns_query_set_get_data(const struct ast_dns_query_set *query_set);
+
+/*!
+ * \brief Asynchronously resolve queries in a query set
+ *
+ * \param query_set The query set
+ * \param callback The callback to invoke upon completion
+ * \param data User data to make available on the query set
+ *
+ * \note The callback will be invoked when all queries have completed
+ *
+ * \note The user data passed in to this function must be ao2 allocated
+ */
+void ast_dns_query_set_resolve_async(struct ast_dns_query_set *query_set, ast_dns_query_set_callback callback, void *data);
+
+/*!
+ * \brief Synchronously resolve queries in a query set
+ *
+ * \param query_set The query set
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note This function will return when all queries have been completed
+ */
+int ast_query_set_resolve(struct ast_dns_query_set *query_set);
+
+/*!
+ * \brief Cancel an asynchronous DNS query set resolution
+ *
+ * \param query_set The DNS query set
+ *
+ * \retval 0 success (all queries have been cancelled)
+ * \retval -1 failure (some queries could not be cancelled)
+ *
+ * \note If successfully cancelled the callback will not be invoked
+ */
+int ast_dns_query_set_resolve_cancel(struct ast_dns_query_set *query_set);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_QUERY_SET_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_recurring.h 1:16.1.1~dfsg-1/include/asterisk/dns_recurring.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_recurring.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_recurring.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,78 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS Recurring Resolution API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_RECURRING_H
+#define _ASTERISK_DNS_RECURRING_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*! \brief Opaque structure for a recurring DNS query */
+struct ast_dns_query_recurring;
+
+/*!
+ * \brief Asynchronously resolve a DNS query, and continue resolving it according to the lowest TTL available
+ *
+ * \param name The name of what to resolve
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ * \param callback The callback to invoke upon completion
+ * \param data User data to make available on the query
+ *
+ * \retval non-NULL success - query has been sent for resolution
+ * \retval NULL failure
+ *
+ * \note The user data passed in to this function must be ao2 allocated
+ *
+ * \note This query will continue to happen according to the lowest TTL unless cancelled using ast_dns_resolve_recurring_cancel
+ *
+ * \note It is NOT possible for the callback to be invoked concurrently for the query multiple times
+ *
+ * \note The query will occur when the TTL expires, not before. This means that there is a period of time where the previous
+ *       information can be considered stale.
+ *
+ * \note If the TTL is determined to be 0 (the record specifies 0, or no records exist) this will cease doing a recurring query.
+ *       It is the responsibility of the caller to resume querying at an interval they determine.
+ */
+struct ast_dns_query_recurring *ast_dns_resolve_recurring(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data);
+
+/*!
+ * \brief Cancel an asynchronous recurring DNS resolution
+ *
+ * \param query The DNS query returned from ast_dns_resolve_recurring
+ *
+ * \retval 0 success - any active query has been cancelled and the query will no longer occur
+ * \retval -1 failure - an active query was in progress and could not be cancelled
+ *
+ * \note If successfully cancelled the callback will not be invoked
+ *
+ * \note This function does NOT drop your reference to the recurring query, this should be dropped using ao2_ref
+ */
+int ast_dns_resolve_recurring_cancel(struct ast_dns_query_recurring *recurring);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_RECURRING_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_resolver.h 1:16.1.1~dfsg-1/include/asterisk/dns_resolver.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_resolver.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_resolver.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,146 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS Resolver API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_RESOLVER_H
+#define _ASTERISK_DNS_RESOLVER_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*! \brief DNS resolver implementation */
+struct ast_dns_resolver {
+    /*! \brief The name of the resolver implementation */
+    const char *name;
+
+    /*! \brief Priority for this resolver if multiple exist, lower being higher priority */
+    unsigned int priority;
+
+    /*!
+     * \brief Perform resolution of a DNS query
+     *
+     * \note The reference count of the query should be increased and released
+     *       upon the query completing or being successfully cancelled
+     */
+    int (*resolve)(struct ast_dns_query *query);
+
+    /*! \brief Cancel resolution of a DNS query */
+    int (*cancel)(struct ast_dns_query *query);
+
+    /*! \brief Linked list information */
+    AST_RWLIST_ENTRY(ast_dns_resolver) next;
+};
+
+/*!
+ * \brief Set resolver specific data on a query
+ *
+ * \param query The DNS query
+ * \param data The resolver specific data
+ *
+ * \note The resolver data MUST be an ao2 object
+ *
+ * \note This function increments the reference count of the resolver data, it does NOT steal
+ *
+ * \note Once resolver specific data has been set it can not be changed
+ *
+ * \retval 0 success
+ * \retval -1 failure, resolver data is already set
+ */
+int ast_dns_resolver_set_data(struct ast_dns_query *query, void *data);
+
+/*!
+ * \brief Retrieve resolver specific data
+ *
+ * \param query The DNS query
+ *
+ * \return the resolver specific data
+ *
+ * \note The reference count of the resolver data is NOT incremented on return
+ */
+void *ast_dns_resolver_get_data(const struct ast_dns_query *query);
+
+/*!
+ * \brief Set result information for a DNS query
+ *
+ * \param query The DNS query
+ * \param result Whether the result is secured or not
+ * \param bogus Whether the result is bogus or not
+ * \param rcode Optional response code
+ * \param canonical The canonical name
+ * \param answer The raw DNS answer
+ * \param answer_size The size of the raw DNS answer
+ *
+ * Zero-sized and NULL answers are permitted by this function. This may be
+ * necessary if the query fails at an early stage and no actual DNS response
+ * has been received from a DNS server.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_dns_resolver_set_result(struct ast_dns_query *query, unsigned int secure, unsigned int bogus,
+	unsigned int rcode, const char *canonical, const char *answer, size_t answer_size);
+
+/*!
+ * \brief Add a DNS record to the result of a DNS query
+ *
+ * \param query The DNS query
+ * \param rr_type Resource record type
+ * \param rr_class Resource record class
+ * \param ttl TTL of the record
+ * \param data The raw DNS record
+ * \param size The size of the raw DNS record
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_dns_resolver_add_record(struct ast_dns_query *query, int rr_type, int rr_class, int ttl, const char *data, const size_t size);
+
+/*!
+ * \brief Mark a DNS query as having been completed
+ *
+ * \param query The DNS query
+ */
+void ast_dns_resolver_completed(struct ast_dns_query *query);
+
+/*!
+ * \brief Register a DNS resolver
+ *
+ * \param resolver A DNS resolver implementation
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_dns_resolver_register(struct ast_dns_resolver *resolver);
+
+/*!
+ * \brief Unregister a DNS resolver
+ *
+ * \param resolver A DNS resolver implementation
+ */
+void ast_dns_resolver_unregister(struct ast_dns_resolver *resolver);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_RESOLVER_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_srv.h 1:16.1.1~dfsg-1/include/asterisk/dns_srv.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_srv.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_srv.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,71 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS SRV Record Parsing API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_SRV_H
+#define _ASTERISK_DNS_SRV_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*!
+ * \brief Get the hostname from an SRV record
+ *
+ * \param record The DNS record
+ *
+ * \return the hostname
+ */
+const char *ast_dns_srv_get_host(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the priority from an SRV record
+ *
+ * \param record The DNS record
+ *
+ * \return the priority
+ */
+unsigned short ast_dns_srv_get_priority(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the weight from an SRV record
+ *
+ * \param record The DNS record
+ *
+ * \return the weight
+ */
+unsigned short ast_dns_srv_get_weight(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the port from an SRV record
+ *
+ * \param record The DNS record
+ *
+ * \return the port
+ */
+unsigned short ast_dns_srv_get_port(const struct ast_dns_record *record);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_SRV_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_test.h 1:16.1.1~dfsg-1/include/asterisk/dns_test.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_test.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_test.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,109 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * Includes code and algorithms from the Zapata library.
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+#ifndef DNS_TEST_H
+#define DNS_TEST_H
+
+/*!
+ * \brief Representation of a string in DNS
+ *
+ * In DNS, a string has a byte to indicate the length,
+ * followed by a series of bytes representing the string.
+ * DNS does not NULL-terminate its strings. However, the
+ * string stored in this structure is expected to be NULL-
+ * terminated.
+ */
+struct ast_dns_test_string {
+	uint8_t len;
+	const char *val;
+};
+
+/*!
+ * \brief Write a DNS string to a buffer
+ *
+ * This writes the DNS string to the buffer and returns the total
+ * number of bytes written to the buffer.
+ *
+ * There is no buffer size passed to this function. Tests are expected to
+ * use a buffer that is sufficiently large for their tests.
+ *
+ * \param string The string to write
+ * \param buf The buffer to write the string into
+ * \return The number of bytes written to the buffer
+ */
+int ast_dns_test_write_string(const struct ast_dns_test_string *string, char *buf);
+
+/*!
+ * \brief Write a DNS domain to a buffer
+ *
+ * A DNS domain consists of a series of labels separated
+ * by dots. Each of these labels gets written as a DNS
+ * string. A DNS domain ends with a NULL label, which is
+ * essentially a zero-length DNS string.
+ *
+ * There is no buffer size passed to this function. Tests are expected to
+ * use a buffer that is sufficiently large for their tests.
+ *
+ * \param string The DNS domain to write
+ * \param buf The buffer to write the domain into
+ * \return The number of bytes written to the buffer
+ */
+int ast_dns_test_write_domain(const char *string, char *buf);
+
+/*!
+ * \brief Callback to write specific DNS record to an answer
+ *
+ * When generating a DNS result, the type of DNS record being generated
+ * will need to be performed by individual test cases. This is a callback
+ * that tests can define to write a specific type of DNS record to the
+ * provided buffer.
+ *
+ * There is no buffer size passed to this function. Tests are expected to
+ * use a buffer that is sufficiently large for their tests.
+ *
+ * \param record Pointer to test-specific DNS record data
+ * \param buf The buffer into which to write the DNS record
+ * \return The number of bytes written to the buffer
+ */
+typedef int (*record_fn)(void *record, char *buf);
+
+/*!
+ * \brief Generate a full DNS response for the given DNS records.
+ *
+ * This function takes care of generating the DNS header, question, and
+ * answer sections of a DNS response. In order to place test-specific
+ * record data into the DNS answers, a callback is provided as a parameter
+ * to this function so that the necessary records can be encoded properly
+ * by the tests.
+ *
+ * There is no buffer size passed to this function. Tests are expected to
+ * use a buffer that is sufficiently large for their tests.
+ *
+ * \param query The DNS query that is being processed
+ * \param records An array of test-specific representations of DNS records
+ * \param num_records The number of elements in the records array
+ * \param record_size The size of each element in the records array
+ * \param generate The test-specific encoder for DNS records
+ * \param buffer The buffer into which to write the DNS response
+ */
+int ast_dns_test_generate_result(struct ast_dns_query *query, void *records, size_t num_records,
+		size_t record_size, record_fn generate, char *buffer);
+
+#endif /* DNS_TEST_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dns_tlsa.h 1:16.1.1~dfsg-1/include/asterisk/dns_tlsa.h
--- 1:13.23.1~dfsg-2/include/asterisk/dns_tlsa.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dns_tlsa.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,72 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ * \brief DNS TLSA Record Parsing API
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _ASTERISK_DNS_TLSA_H
+#define _ASTERISK_DNS_TLSA_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+/*!
+ * \brief Get the certificate usage field from a TLSA record
+ *
+ * \param record The DNS record
+ *
+ * \return the certificate usage field
+ */
+
+unsigned int ast_dns_tlsa_get_usage(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the selector field from a TLSA record
+ *
+ * \param record The DNS record
+ *
+ * \return the selector field
+ */
+unsigned int ast_dns_tlsa_get_selector(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the matching type field from a TLSA record
+ *
+ * \param record The DNS record
+ *
+ * \return the matching type field
+ */
+unsigned int ast_dns_tlsa_get_matching_type(const struct ast_dns_record *record);
+
+/*!
+ * \brief Get the certificate association data from a TLSA record
+ *
+ * \param record The DNS record
+ *
+ * \return the certificate association data
+ */
+const char *ast_dns_tlsa_get_association_data(const struct ast_dns_record *record);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_DNS_TLSA_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/dsp.h 1:16.1.1~dfsg-1/include/asterisk/dsp.h
--- 1:13.23.1~dfsg-2/include/asterisk/dsp.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/dsp.h	2018-12-26 18:45:13.000000000 +0000
@@ -193,16 +193,4 @@ int ast_dsp_get_tcount(struct ast_dsp *d
  */
 int ast_dsp_get_threshold_from_settings(enum threshold which);
 
-/*!
- * \brief Reloads dsp settings from dsp.conf
- * \since 1.6.1
- */
-int ast_dsp_reload(void);
-
-/*!
- * \brief Load dsp settings from dsp.conf
- * \since 1.6.1
- */
-int ast_dsp_init(void);
-
 #endif /* _ASTERISK_DSP_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/enum.h 1:16.1.1~dfsg-1/include/asterisk/enum.h
--- 1:13.23.1~dfsg-2/include/asterisk/enum.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/enum.h	2018-12-26 18:45:13.000000000 +0000
@@ -97,7 +97,4 @@ int ast_get_enum(struct ast_channel *cha
  */
 int ast_get_txt(struct ast_channel *chan, const char *number, char *txt, int maxtxt, char *suffix);
 
-int ast_enum_init(void);
-int ast_enum_reload(void);
-
 #endif /* _ASTERISK_ENUM_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/event.h 1:16.1.1~dfsg-1/include/asterisk/event.h
--- 1:13.23.1~dfsg-2/include/asterisk/event.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/event.h	2018-12-26 18:45:13.000000000 +0000
@@ -35,7 +35,7 @@
  * modules in Asterisk.
  *  - CEL uses the \ref ast_event representation to pass information to registered
  *    backends.
- *  - The \file res_corosync module publishes \ref ast_event representations of
+ *  - The \file res_corosync.c module publishes \ref ast_event representations of
  *    information to other Asterisk instances in a cluster.
  *  - Security event represent their event types and data using this system.
  *  - Theoretically, any \ref stasis message can use this system to pass
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/features_config.h 1:16.1.1~dfsg-1/include/asterisk/features_config.h
--- 1:13.23.1~dfsg-2/include/asterisk/features_config.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/features_config.h	2018-12-26 18:45:13.000000000 +0000
@@ -265,10 +265,4 @@ struct ast_applicationmap_item {
  */
 struct ao2_container *ast_get_chan_applicationmap(struct ast_channel *chan);
 
-void ast_features_config_shutdown(void);
-
-int ast_features_config_reload(void);
-
-int ast_features_config_init(void);
-
 #endif /* _FEATURES_CONFIG_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/file.h 1:16.1.1~dfsg-1/include/asterisk/file.h
--- 1:13.23.1~dfsg-2/include/asterisk/file.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/file.h	2018-12-26 18:45:13.000000000 +0000
@@ -427,6 +427,18 @@ char *ast_format_str_reduce(char *fmts);
  */
 struct ast_format *ast_get_format_for_file_ext(const char *file_ext);
 
+/*!
+ * \brief Get a suitable filename extension for the given MIME type
+ *
+ * \param mime_type The MIME type for which to find extensions
+ * \param buffer A pointer to a buffer to receive the extension
+ * \param capacity The size of 'buffer' in bytes
+ *
+ * \retval 1 if an extension was found for the provided MIME type
+ * \retval 0 if the MIME type was not found
+ */
+int ast_get_extension_for_mime_type(const char *mime_type, char *buffer, size_t capacity);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/format_cache.h 1:16.1.1~dfsg-1/include/asterisk/format_cache.h
--- 1:13.23.1~dfsg-2/include/asterisk/format_cache.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/format_cache.h	2018-12-26 18:45:13.000000000 +0000
@@ -214,6 +214,11 @@ extern struct ast_format *ast_format_sir
 extern struct ast_format *ast_format_opus;
 
 /*!
+ * \brief Built-in cached Codec 2 format.
+ */
+extern struct ast_format *ast_format_codec2;
+
+/*!
  * \brief Built-in cached t140 format.
  */
 extern struct ast_format *ast_format_t140;
@@ -224,6 +229,11 @@ extern struct ast_format *ast_format_t14
 extern struct ast_format *ast_format_t140_red;
 
 /*!
+ * \brief Built-in cached T.38 format.
+ */
+extern struct ast_format *ast_format_t38;
+
+/*!
  * \brief Built-in "null" format.
  */
 extern struct ast_format *ast_format_none;
@@ -265,20 +275,13 @@ int ast_format_cache_set(struct ast_form
  * \note The returned format has its reference count incremented. It must be
  * dropped using ao2_ref or ao2_cleanup.
  */
-struct ast_format *__ast_format_cache_get(const char *name);
-struct ast_format *__ast_format_cache_get_debug(const char *name, const char *tag, const char *file, int line, const char *func);
+struct ast_format *__ast_format_cache_get(const char *name,
+	const char *tag, const char *file, int line, const char *func);
 
-#ifdef REF_DEBUG
-#define ast_format_cache_get(name) \
-	__ast_format_cache_get_debug((name), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ast_t_format_cache_get(name, tag) \
-	__ast_format_cache_get_debug((name), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
 #define ast_format_cache_get(name) \
-	__ast_format_cache_get((name))
+	__ast_format_cache_get((name), "ast_format_cache_get", __FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ast_t_format_cache_get(name, tag) \
-	__ast_format_cache_get((name))
-#endif
+	__ast_format_cache_get((name), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/format_cap.h 1:16.1.1~dfsg-1/include/asterisk/format_cap.h
--- 1:13.23.1~dfsg-2/include/asterisk/format_cap.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/format_cap.h	2018-12-26 18:45:13.000000000 +0000
@@ -46,20 +46,14 @@ enum ast_format_cap_flags {
  * \retval ast_format_cap object on success.
  * \retval NULL on failure.
  */
-struct ast_format_cap *__ast_format_cap_alloc(enum ast_format_cap_flags flags);
-struct ast_format_cap *__ast_format_cap_alloc_debug(enum ast_format_cap_flags flags, const char *tag, const char *file, int line, const char *func);
+struct ast_format_cap *__ast_format_cap_alloc(enum ast_format_cap_flags flags,
+	const char *tag, const char *file, int line, const char *func);
 
-#ifdef REF_DEBUG
 #define ast_format_cap_alloc(flags) \
-	__ast_format_cap_alloc_debug((flags), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
+	__ast_format_cap_alloc((flags), "ast_format_cap_alloc", \
+		__FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ast_t_format_cap_alloc(flags, tag) \
-	__ast_format_cap_alloc_debug((flags), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-#define ast_format_cap_alloc(flags) \
-	__ast_format_cap_alloc((flags))
-#define ast_t_format_cap_alloc(flags, tag) \
-	__ast_format_cap_alloc((flags))
-#endif
+	__ast_format_cap_alloc((flags), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Set the global framing.
@@ -103,20 +97,15 @@ unsigned int ast_format_cap_get_framing(
  *
  * \note If framing is specified here it overrides any global framing that has been set.
  */
-int __ast_format_cap_append(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing);
-int __ast_format_cap_append_debug(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing, const char *tag, const char *file, int line, const char *func);
+int __ast_format_cap_append(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing,
+	const char *tag, const char *file, int line, const char *func);
 
-#ifdef REF_DEBUG
-#define ast_format_cap_append(cap, format, framing) \
-	__ast_format_cap_append_debug((cap), (format), (framing), "", __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#define ast_t_format_cap_append(cap, format, framing, tag) \
-	__ast_format_cap_append_debug((cap), (format), (framing), (tag), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
 #define ast_format_cap_append(cap, format, framing) \
-	__ast_format_cap_append((cap), (format), (framing))
+	__ast_format_cap_append((cap), (format), (framing), "ast_format_cap_append", \
+		__FILE__, __LINE__, __PRETTY_FUNCTION__)
 #define ast_t_format_cap_append(cap, format, framing, tag) \
-	__ast_format_cap_append((cap), (format), (framing))
-#endif
+	__ast_format_cap_append((cap), (format), (framing), (tag), \
+		__FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Add all codecs Asterisk knows about for a specific type to
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/format.h 1:16.1.1~dfsg-1/include/asterisk/format.h
--- 1:13.23.1~dfsg-2/include/asterisk/format.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/format.h	2018-12-26 18:45:13.000000000 +0000
@@ -270,7 +270,7 @@ int __ast_format_interface_register(cons
  * \retval 0 success
  * \retval -1 failure
  */
-#define ast_format_interface_register(codec, interface) __ast_format_interface_register(codec, interface, ast_module_info->self)
+#define ast_format_interface_register(codec, interface) __ast_format_interface_register(codec, interface, AST_MODULE_SELF)
 
 /*!
  * \brief Get the attribute data on a format
@@ -299,6 +299,24 @@ void ast_format_set_attribute_data(struc
 const char *ast_format_get_name(const struct ast_format *format);
 
 /*!
+ * \brief Get the channel count on a format
+ *
+ * \param The media format
+ *
+ * \return Currently set channel count
+ */
+unsigned int ast_format_get_channel_count(const struct ast_format *format);
+
+/*!
+ * \brief Set the channel count on a format
+ *
+ * \param format The media format
+ * \param channel_count The number of audio channels used
+ *
+ */
+void ast_format_set_channel_count(struct ast_format *format, unsigned int channel_count);
+
+/*!
  * \brief Get the codec associated with a format
  *
  * \param format The media format
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/frame.h 1:16.1.1~dfsg-1/include/asterisk/frame.h
--- 1:13.23.1~dfsg-2/include/asterisk/frame.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/frame.h	2018-12-26 18:45:13.000000000 +0000
@@ -140,6 +140,8 @@ enum {
 	AST_FRFLAG_HAS_TIMING_INFO = (1 << 0),
 	/*! This frame has been requeued */
 	AST_FRFLAG_REQUEUED = (1 << 1),
+	/*! This frame contains a valid sequence number */
+	AST_FRFLAG_HAS_SEQUENCE_NUMBER = (1 << 2),
 };
 
 struct ast_frame_subclass {
@@ -184,6 +186,8 @@ struct ast_frame {
 	long len;
 	/*! Sequence number */
 	int seqno;
+	/*! Stream number the frame originated from */
+	int stream_num;
 };
 
 /*!
@@ -298,6 +302,9 @@ enum ast_control_frame_type {
 	AST_CONTROL_UPDATE_RTP_PEER = 32, /*!< Interrupt the bridge and have it update the peer */
 	AST_CONTROL_PVT_CAUSE_CODE = 33, /*!< Contains an update to the protocol-specific cause-code stored for branching dials */
 	AST_CONTROL_MASQUERADE_NOTIFY = 34,	/*!< A masquerade is about to begin/end. (Never sent as a frame but directly with ast_indicate_data().) */
+	AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE = 35,    /*!< Channel indication that a stream topology change has been requested */
+	AST_CONTROL_STREAM_TOPOLOGY_CHANGED = 36,           /*!< Channel indication that a stream topology change has occurred */
+	AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED = 37,    /*!< Channel indication that one of the source streams has changed its source */
 
 	/*
 	 * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/hashtab.h 1:16.1.1~dfsg-1/include/asterisk/hashtab.h
--- 1:13.23.1~dfsg-2/include/asterisk/hashtab.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/hashtab.h	2018-12-26 18:45:13.000000000 +0000
@@ -251,22 +251,15 @@ unsigned int ast_hashtab_hash_short(cons
  * \param hash a func ptr to do the hashing
  * \param do_locking use locks to guarantee safety of iterators/insertion/deletion -- real simpleminded right now
 */
-#ifdef __AST_DEBUG_MALLOC
-struct ast_hashtab * _ast_hashtab_create(int initial_buckets,
-					int (*compare)(const void *a, const void *b),
-					int (*resize)(struct ast_hashtab *),
-					int (*newsize)(struct ast_hashtab *tab),
-					unsigned int (*hash)(const void *obj),
-					int do_locking, const char *file, int lineno, const char *function);
-#define ast_hashtab_create(a,b,c,d,e,f)	_ast_hashtab_create(a,b,c,d,e,f,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_hashtab * ast_hashtab_create(int initial_buckets,
-					int (*compare)(const void *a, const void *b),
-					int (*resize)(struct ast_hashtab *),
-					int (*newsize)(struct ast_hashtab *tab),
-					unsigned int (*hash)(const void *obj),
-					int do_locking );
-#endif
+struct ast_hashtab *_ast_hashtab_create(int initial_buckets,
+	int (*compare)(const void *a, const void *b),
+	int (*resize)(struct ast_hashtab *),
+	int (*newsize)(struct ast_hashtab *tab),
+	unsigned int (*hash)(const void *obj),
+	int do_locking,
+	const char *file, int lineno, const char *function);
+#define ast_hashtab_create(initial_buckets, compare, resize, newsize, hash, do_locking) \
+	_ast_hashtab_create(initial_buckets, compare, resize, newsize, hash, do_locking, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief This func will free the hash table and all its memory.
@@ -294,12 +287,9 @@ void ast_hashtab_destroy( struct ast_has
  * \retval 1 on success
  * \retval 0 if there's a problem
 */
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_immediate(struct ast_hashtab *tab, const void *obj, const char *file, int lineno, const char *func);
-#define	ast_hashtab_insert_immediate(a,b)	_ast_hashtab_insert_immediate(a, b, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-int ast_hashtab_insert_immediate(struct ast_hashtab *tab, const void *obj);
-#endif
+#define ast_hashtab_insert_immediate(tab, obj) \
+	_ast_hashtab_insert_immediate(tab, obj, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Insert without checking, hashing or locking
@@ -311,12 +301,9 @@ int ast_hashtab_insert_immediate(struct
  * \retval 1 on success
  * \retval 0 if there's a problem
 */
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_immediate_bucket(struct ast_hashtab *tab, const void *obj, unsigned int h, const char *file, int lineno, const char *func);
-#define	ast_hashtab_insert_immediate_bucket(a,b,c)	_ast_hashtab_insert_immediate_bucket(a, b, c, __FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-int ast_hashtab_insert_immediate_bucket(struct ast_hashtab *tab, const void *obj, unsigned int h);
-#endif
+#define ast_hashtab_insert_immediate_bucket(tab, obj, h) \
+	_ast_hashtab_insert_immediate_bucket(tab, obj, h, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Check and insert new object only if it is not there.
@@ -324,12 +311,9 @@ int ast_hashtab_insert_immediate_bucket(
  * \retval 1 on success
  * \retval  0 if there's a problem, or it's already there.
 */
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_safe(struct ast_hashtab *tab, const void *obj, const char *file, int lineno, const char *func);
-#define	ast_hashtab_insert_safe(a,b)	_ast_hashtab_insert_safe(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-int ast_hashtab_insert_safe(struct ast_hashtab *tab, const void *obj);
-#endif
+#define ast_hashtab_insert_safe(tab, obj) \
+	_ast_hashtab_insert_safe(tab, obj, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Lookup this object in the hash table.
@@ -362,20 +346,14 @@ int ast_hashtab_size( struct ast_hashtab
 int ast_hashtab_capacity( struct ast_hashtab *tab);
 
 /*! \brief Return a copy of the hash table */
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab *_ast_hashtab_dup(struct ast_hashtab *tab, void *(*obj_dup_func)(const void *obj), const char *file, int lineno, const char *func);
-#define	ast_hashtab_dup(a,b)	_ast_hashtab_dup(a,b,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_hashtab *ast_hashtab_dup(struct ast_hashtab *tab, void *(*obj_dup_func)(const void *obj));
-#endif
+#define ast_hashtab_dup(tab, obj_dup_func) \
+	_ast_hashtab_dup(tab, obj_dup_func, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*! \brief Gives an iterator to hastable */
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab_iter *_ast_hashtab_start_traversal(struct ast_hashtab *tab, const char *file, int lineno, const char *func);
-#define	ast_hashtab_start_traversal(a)	_ast_hashtab_start_traversal(a,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_hashtab_iter *ast_hashtab_start_traversal(struct ast_hashtab *tab);
-#endif
+#define ast_hashtab_start_traversal(tab) \
+	_ast_hashtab_start_traversal(tab, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*! \brief end the traversal, free the iterator, unlock if necc. */
 void ast_hashtab_end_traversal(struct ast_hashtab_iter *it);
@@ -395,12 +373,9 @@ void *ast_hashtab_remove_this_object(str
 /* ------------------ */
 
 /*! \brief Gives an iterator to hastable */
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab_iter *_ast_hashtab_start_write_traversal(struct ast_hashtab *tab, const char *file, int lineno, const char *func);
-#define	ast_hashtab_start_write_traversal(a)	_ast_hashtab_start_write_traversal(a,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_hashtab_iter *ast_hashtab_start_write_traversal(struct ast_hashtab *tab);
-#endif
+#define ast_hashtab_start_write_traversal(tab) \
+	_ast_hashtab_start_write_traversal(tab, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*! \brief Looks up the object, removes the corresponding bucket */
 void *ast_hashtab_remove_object_via_lookup_nolock(struct ast_hashtab *tab, void *obj);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/heap.h 1:16.1.1~dfsg-1/include/asterisk/heap.h
--- 1:13.23.1~dfsg-2/include/asterisk/heap.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/heap.h	2018-12-26 18:45:13.000000000 +0000
@@ -97,14 +97,10 @@ typedef int (*ast_heap_cmp_fn)(void *elm
  * \return An instance of a max heap
  * \since 1.6.1
  */
-#ifdef __AST_DEBUG_MALLOC
 struct ast_heap *_ast_heap_create(unsigned int init_height, ast_heap_cmp_fn cmp_fn,
 		ssize_t index_offset, const char *file, int lineno, const char *func);
-#define	ast_heap_create(a,b,c)	_ast_heap_create(a,b,c,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-struct ast_heap *ast_heap_create(unsigned int init_height, ast_heap_cmp_fn cmp_fn,
-		ssize_t index_offset);
-#endif
+#define ast_heap_create(init_height, cmp_fn, index_offset) \
+	_ast_heap_create(init_height, cmp_fn, index_offset, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Destroy a max heap
@@ -126,12 +122,9 @@ struct ast_heap *ast_heap_destroy(struct
  * \retval non-zero failure
  * \since 1.6.1
  */
-#ifdef __AST_DEBUG_MALLOC
 int _ast_heap_push(struct ast_heap *h, void *elm, const char *file, int lineno, const char *func);
-#define	ast_heap_push(a,b)	_ast_heap_push(a,b,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-int ast_heap_push(struct ast_heap *h, void *elm);
-#endif
+#define ast_heap_push(h, elm) \
+	_ast_heap_push(h, elm, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Pop the max element off of the heap
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/http_websocket.h 1:16.1.1~dfsg-1/include/asterisk/http_websocket.h
--- 1:13.23.1~dfsg-2/include/asterisk/http_websocket.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/http_websocket.h	2018-12-26 18:45:13.000000000 +0000
@@ -77,13 +77,14 @@ struct ast_websocket;
  * \param ser The TCP/TLS session
  * \param parameters Parameters extracted from the request URI
  * \param headers Headers included in the request
+ * \param session_id The id of the current session.
  *
  * \retval 0 The session should be accepted
  * \retval -1 The session should be rejected. Note that the caller must send an error
  * response using \ref ast_http_error.
  * \since 13.5.0
  */
-typedef int (*ast_websocket_pre_callback)(struct ast_tcptls_session_instance *ser, struct ast_variable *parameters, struct ast_variable *headers);
+typedef int (*ast_websocket_pre_callback)(struct ast_tcptls_session_instance *ser, struct ast_variable *parameters, struct ast_variable *headers, const char *session_id);
 
 /*!
  * \brief Callback for when a new connection for a sub-protocol is established
@@ -369,6 +370,13 @@ AST_OPTIONAL_API(int, ast_websocket_is_s
 AST_OPTIONAL_API(int, ast_websocket_set_nonblock, (struct ast_websocket *session), { errno = ENOSYS; return -1;});
 
 /*!
+ * \brief Get the session ID for a WebSocket session.
+ *
+ * \retval session id
+ */
+AST_OPTIONAL_API(const char *, ast_websocket_session_id, (struct ast_websocket *session), { errno = ENOSYS; return NULL;});
+
+/*!
  * \brief Result code for a websocket client.
  */
 enum ast_websocket_result {
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/ilbc.h 1:16.1.1~dfsg-1/include/asterisk/ilbc.h
--- 1:13.23.1~dfsg-2/include/asterisk/ilbc.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/ilbc.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,8 @@
+#ifndef _AST_FORMAT_ILBC_H_
+#define _AST_FORMAT_ILBC_H_
+
+struct ilbc_attr {
+	unsigned int mode;
+};
+
+#endif /* _AST_FORMAT_ILBC_H */
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/indications.h 1:16.1.1~dfsg-1/include/asterisk/indications.h
--- 1:13.23.1~dfsg-2/include/asterisk/indications.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/indications.h	2018-12-26 18:45:13.000000000 +0000
@@ -28,7 +28,6 @@
 
 #include "asterisk/astobj2.h"
 #include "asterisk/utils.h"
-#include "asterisk/data.h"
 
 /*!
  * \brief Description of a tone
@@ -242,12 +241,4 @@ static inline struct ast_tone_zone_sound
 	return ts;
 }
 
-/*!
- * \brief Add a tone_zone structure to the data tree specified.
- *
- * \retval <0 on error.
- * \retval 0 on success.
- */
-int ast_tone_zone_data_add_structure(struct ast_data *tree, struct ast_tone_zone *zone);
-
 #endif /* _ASTERISK_INDICATIONS_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/inline_api.h 1:16.1.1~dfsg-1/include/asterisk/inline_api.h
--- 1:13.23.1~dfsg-2/include/asterisk/inline_api.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/inline_api.h	2018-12-26 18:45:13.000000000 +0000
@@ -33,7 +33,7 @@
 	copies of the function body are not built in different modules.
 	However, since this doesn't work for clang, we go with 'static'
 	anyway and hope for the best!
-	- when LOW_MEMORY is defined, inlining should be disabled
+	- when DISABLE_INLINE is defined, inlining should be disabled
 	completely, even if the compiler is configured to support it
 
   The AST_INLINE_API macro allows this to happen automatically, when
@@ -45,7 +45,7 @@
   including the header file
  */
 
-#if !defined(LOW_MEMORY) && !defined(DISABLE_INLINE)
+#if !defined(DISABLE_INLINE)
 
 #if !defined(AST_API_MODULE)
 #if defined(__clang__) || defined(__GNUC_STDC_INLINE__)
@@ -57,7 +57,7 @@
 #define AST_INLINE_API(hdr, body) hdr; hdr body
 #endif
 
-#else /* defined(LOW_MEMORY) */
+#else /* defined(DISABLE_INLINE) */
 
 #if !defined(AST_API_MODULE)
 #define AST_INLINE_API(hdr, body) hdr;
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/io.h 1:16.1.1~dfsg-1/include/asterisk/io.h
--- 1:13.23.1~dfsg-2/include/asterisk/io.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/io.h	2018-12-26 18:45:13.000000000 +0000
@@ -24,6 +24,7 @@
 #define _ASTERISK_IO_H
 
 #include "asterisk/poll-compat.h"
+#include "asterisk/netsock2.h"
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
@@ -148,6 +149,29 @@ int ast_get_termcols(int fd);
  */
 int ast_sd_notify(const char *state);
 
+/*!
+ * \brief Find a listening file descriptor provided by socket activation.
+ * \param type SOCK_STREAM or SOCK_DGRAM
+ * \param addr The socket address of the bound listener.
+ * \retval <0 No match.
+ * \retval >0 File Descriptor matching sockaddr.
+ *
+ * \note This function returns -1 if systemd's development headers were not
+ * detected on the system.
+ */
+int ast_sd_get_fd(int type, const struct ast_sockaddr *addr);
+
+/*!
+ * \brief Find a listening AF_LOCAL file descriptor provided by socket activation.
+ * \param type SOCK_STREAM or SOCK_DGRAM
+ * \param path The path of the listener.
+ * \retval <0 No match.
+ * \retval >0 File Descriptor matching path.
+ *
+ * \note This function returns -1 if systemd's development headers were not
+ * detected on the system.
+ */
+int ast_sd_get_fd_un(int type, const char *path);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/iostream.h 1:16.1.1~dfsg-1/include/asterisk/iostream.h
--- 1:13.23.1~dfsg-2/include/asterisk/iostream.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/iostream.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,261 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 1999 - 2015, Digium, Inc.
+ *
+ * Timo Teräs <timo.teras@iki.fi>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+#ifndef _ASTERISK_IOSTREAM_H
+#define _ASTERISK_IOSTREAM_H
+
+/*!
+ * \file iostream.h
+ *
+ * \brief Generic abstraction for input/output streams.
+ */
+
+#include "asterisk.h"           /* for size_t, ssize_t, HAVE_OPENSSL */
+
+#if defined(HAVE_OPENSSL)
+#define DO_SSL  /* comment in/out if you want to support ssl */
+#endif
+
+struct ssl_st;                  /* forward declaration */
+struct ssl_ctx_st;              /* forward declaration */
+struct timeval;                 /* forward declaration */
+typedef struct ssl_st SSL;
+typedef struct ssl_ctx_st SSL_CTX;
+
+struct ast_iostream;            /* forward declaration */
+
+/*!
+ * \brief Disable the iostream timeout timer.
+ *
+ * \param stream A pointer to an iostream
+ *
+ * \return Nothing
+ */
+void ast_iostream_set_timeout_disable(struct ast_iostream *stream);
+
+/*!
+ * \brief Set the iostream inactivity timeout timer.
+ *
+ * \param stream A pointer to an iostream
+ * \param timeout Number of milliseconds to wait for data transfer with the peer.
+ *
+ * \details This is basically how much time we are willing to spend
+ * in an I/O call before we declare the peer unresponsive.
+ *
+ * \note Setting timeout to -1 disables the timeout.
+ * \note Setting this timeout replaces the I/O sequence timeout timer.
+ *
+ * \return Nothing
+ */
+void ast_iostream_set_timeout_inactivity(struct ast_iostream *stream, int timeout);
+
+/*!
+ * \brief Set the iostream inactivity & idle timeout timers.
+ *
+ * \param stream A pointer to an iostream
+ * \param timeout Number of milliseconds to wait for initial data transfer with
+ *                the peer.
+ * \param timeout_reset Number of milliseconds to wait for subsequent data
+ *                      transfer with the peer.
+ *
+ * \details As an example, if you want to timeout a peer if they do not send an
+ *          initial message within 5 seconds or if they do not send a message at
+ *          least every 30 seconds, you would set \a timeout to \c 5000 and
+ *          \a timeout_reset to \c 30000.
+ *
+ * \note Setting either of these timeouts to -1 will disable them.
+ *
+ * \return Nothing
+ */
+void ast_iostream_set_timeout_idle_inactivity(struct ast_iostream *stream, int timeout, int timeout_reset);
+
+/*!
+ * \brief Set the iostream I/O sequence timeout timer.
+ *
+ * \param stream A pointer to an iostream
+ * \param start Time the I/O sequence timer starts.
+ * \param timeout Number of milliseconds from the start time before timeout.
+ *
+ * \details This is how much time are we willing to allow the peer
+ * to complete an operation that can take several I/O calls.  The
+ * main use is as an authentication timer with us.
+ *
+ * \note Setting timeout to -1 disables the timeout.
+ * \note Setting this timeout replaces the inactivity timeout timer.
+ *
+ * \return Nothing
+ */
+void ast_iostream_set_timeout_sequence(struct ast_iostream *stream, struct timeval start, int timeout);
+
+/*!
+ * \brief Set the iostream if it can exclusively depend upon the set timeouts.
+ *
+ * \param stream A pointer to an iostream
+ * \param exclusive_input TRUE if stream can exclusively wait for fd input.
+ * Otherwise, the stream will not wait for fd input.  It will wait while
+ * trying to send data.
+ *
+ * \note The stream timeouts still need to be set.
+ *
+ * \return Nothing
+ */
+void ast_iostream_set_exclusive_input(struct ast_iostream *stream, int exclusive_input);
+
+/*!
+ * \brief Get an iostream's file descriptor.
+ *
+ * \param stream A pointer to an iostream
+ *
+ * \return The file descriptor for the given iostream, or -1 if the iostream has no open
+ *         file descriptor.
+ */
+int ast_iostream_get_fd(struct ast_iostream *stream);
+
+/*!
+ * \brief Make an iostream non-blocking.
+ *
+ * \param stream A pointer to an iostream
+ *
+ * \return Nothing
+ */
+void ast_iostream_nonblock(struct ast_iostream *stream);
+
+/*!
+ * \brief Get a pointer to an iostream's OpenSSL \c SSL structure
+ *
+ * \param stream A pointer to an iostream
+ *
+ * \return A pointer to the OpenSSL \c SSL structure for the given iostream, or
+ *         \c NULL if TLS has not been initiated.
+ *
+ * \note If OpenSSL support is not included in the build, this will always return
+ *       \c NULL.
+ */
+SSL *ast_iostream_get_ssl(struct ast_iostream *stream);
+
+/*!
+ * \brief Read data from an iostream.
+ *
+ * \param stream A pointer to an iostream
+ * \param buffer Pointer to a buffer to store the read bytes.
+ * \param count The number of bytes to read.
+ *
+ * \return Upon successful completion, returns a non-negative integer indicating
+ *         the number of bytes actually read. Otherwise, returns -1 and may set
+ *         errno to indicate the error.
+ */
+ssize_t ast_iostream_read(struct ast_iostream *stream, void *buffer, size_t count);
+
+/*!
+ * \brief Read a LF-terminated string from an iostream.
+ *
+ * \param stream A pointer to an iostream
+ * \param buffer Pointer to a buffer to store the read bytes.
+ * \param size The total size of \a buffer in bytes.
+ *
+ * \return The number of bytes stored in \a buffer, excluding the null byte used
+ *         to terminate the string. If the size of \a buffer (indicated by the
+ *         caller with the \a size argument) is not sufficient to store the
+ *         entire line it will be truncated to fit the available space. The
+ *         contents of \a buffer will always be terminated with a null byte. In
+ *         the case of an error, \c -1 will be returned and \c errno may be set
+ *         indicating the error.
+ */
+ssize_t ast_iostream_gets(struct ast_iostream *stream, char *buffer, size_t size);
+
+/*!
+ * \brief Discard the specified number of bytes from an iostream.
+ *
+ * \param stream A pointer to an iostream
+ * \param count The number of bytes to discard.
+ *
+ * \return Upon successful completion, returns the number of bytes discarded.
+ *         Otherwise, \c -1 is returned and \c errno may be set indicating the
+ *         error.
+ */
+ssize_t ast_iostream_discard(struct ast_iostream *stream, size_t count);
+
+/*!
+ * \brief Write data to an iostream.
+ *
+ * \param stream A pointer to an iostream
+ * \param buffer Pointer to a buffer from which to read bytes.
+ * \param count The number of bytes from \a buffer to write.
+ *
+ * \return Upon successful completion, returns the number of bytes actually
+ *         written to the iostream. This number shall never be greater than
+ *         \a count. Otherwise, returns \c -1 and may set \c errno to indicate
+ *         the error.
+ */
+ssize_t ast_iostream_write(struct ast_iostream *stream, const void *buffer, size_t count);
+
+/*!
+ * \brief Write a formatted string to an iostream.
+ *
+ * \param stream A pointer to an iostream
+ * \param format A format string, as documented by printf(3)
+ * \param ... Arguments for the provided \a format string
+ *
+ * \return The number of bytes written, or \c -1 if an error occurs. Note that if
+ *         \c -1 is returned, the number of bytes written to the iostream is
+ *         unspecified.
+ */
+ssize_t __attribute__((format(printf, 2, 3))) ast_iostream_printf(
+	struct ast_iostream *stream, const char *format, ...);
+
+/*!
+ * \brief Create an iostream from a file descriptor.
+ *
+ * \param fd A pointer to an open file descriptor
+ *
+ * \return A newly allocated iostream or \c NULL if allocation fails.
+ */
+struct ast_iostream *ast_iostream_from_fd(int *fd);
+
+/*!
+ * \brief Begin TLS on an iostream.
+ *
+ * \param stream A pointer to an iostream pointer
+ * \param ctx A pointer to an \c SSL_CTX which will be passed to \c SSL_new()
+ * \param client Non-zero to indicate that we are the client, zero to indicate
+ *               that we are the server.
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note The iostream that is passed in \a stream may be replaced with a
+ *       different one before this function returns.
+ * \note On failure, \c errno may be set providing additional information on why
+ *       the failure occurred.
+ */
+int ast_iostream_start_tls(struct ast_iostream **stream, SSL_CTX *ctx, int client);
+
+/*!
+ * \brief Close an iostream.
+ *
+ * \param stream A pointer to an iostream
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note On failure, \c errno may be set providing additional information on why
+ *       the failure occurred.
+ */
+int ast_iostream_close(struct ast_iostream *stream);
+
+#endif /* _ASTERISK_IOSTREAM_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/json.h 1:16.1.1~dfsg-1/include/asterisk/json.h
--- 1:13.23.1~dfsg-2/include/asterisk/json.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/json.h	2018-12-26 18:45:13.000000000 +0000
@@ -27,7 +27,7 @@
  * \since 12.0.0
  *
  * This is a very thin wrapper around the Jansson API. For more details on it,
- * see its docs at http://www.digip.org/jansson/doc/2.4/apiref.html.
+ * see its docs at http://www.digip.org/jansson/doc/2.11/apiref.html.
  *
  * Rather than provide the multiple ways of doing things that the Jansson API
  * does, the Asterisk wrapper is always reference-stealing, and always NULL
@@ -43,35 +43,6 @@
  * wrap them with json_ref() when passing them to other \c ast_json_*()
  * functions.
  *
- * \par Thread Safety
- *
- * Jansson (as of 2.4) provides fairly weak thread safety guarantees. The
- * Asterisk wrapper improves upon that slightly. The remaining refcounting
- * problems are issues when slicing/sharing/mixing instances between JSON
- * objects and arrays, which we avoid.
- *
- * The \c ast_json_dump_* functions are thread safe for multiple concurrent
- * dumps of the same object, so long as the concurrent dumps start from the same
- * \c root object. But if an object is shared by other JSON objects/arrays, then
- * concurrent dumps of the outer objects/arrays are not thread safe. This can be
- * avoided by using ast_json_deep_copy() when sharing JSON instances between
- * objects.
- *
- * The ast_json_ref() and ast_json_unref() functions are thread safe. Since the
- * Asterisk wrapper exclusively uses the reference stealing API, Jansson won't
- * be performing many refcount modifications behind our backs. There are a few
- * exceptions.
- *
- * The first is the transitive json_decref() that occurs when \ref
- * AST_JSON_OBJECT and \ref AST_JSON_ARRAY instances are deleted. This can be
- * avoided by using ast_json_deep_copy() when sharing JSON instances between
- * objects.
- *
- * The second is when using the reference borrowing specifier in
- * ast_json_pack() (capital \c O). This can be avoided by using the reference
- * stealing specifier (lowercase \c o) and wrapping the JSON object parameter
- * with ast_json_ref() for an explicit ref-bump.
- *
  * \par Example code
  *
  * \code
@@ -111,6 +82,11 @@
 /*!@{*/
 
 /*!
+ * \brief Primarily used to cast when packing to an "I" type.
+ */
+typedef AST_JSON_INT_T ast_json_int_t;
+
+/*!
  * \brief Initialize the JSON library.
  */
 void ast_json_init(void);
@@ -907,7 +883,7 @@ struct ast_json *ast_json_load_new_file(
  * \brief Helper for creating complex JSON values.
  * \since 12.0.0
  *
- * See original Jansson docs at http://www.digip.org/jansson/doc/2.4/apiref.html#apiref-pack
+ * See original Jansson docs at http://www.digip.org/jansson/doc/2.11/apiref.html#apiref-pack
  * for more details.
  */
 struct ast_json *ast_json_pack(char const *format, ...);
@@ -916,7 +892,7 @@ struct ast_json *ast_json_pack(char cons
  * \brief Helper for creating complex JSON values simply.
  * \since 12.0.0
  *
- * See original Jansson docs at http://www.digip.org/jansson/doc/2.4/apiref.html#apiref-pack
+ * See original Jansson docs at http://www.digip.org/jansson/doc/2.11/apiref.html#apiref-pack
  * for more details.
  */
 struct ast_json *ast_json_vpack(char const *format, va_list ap);
@@ -1076,6 +1052,18 @@ enum ast_json_to_ast_vars_code {
  */
 enum ast_json_to_ast_vars_code ast_json_to_ast_variables(struct ast_json *json_variables, struct ast_variable **variables);
 
+struct varshead;
+
+/*!
+ * \brief Construct a JSON object from a \c ast_var_t list
+ * \since 14.2.0
+ *
+ * \param channelvars The list of \c ast_var_t to represent as JSON
+ *
+ * \return JSON object with variable names as keys and variable values as values
+ */
+struct ast_json *ast_json_channel_vars(struct varshead *channelvars);
+
 /*!@}*/
 
 #endif /* _ASTERISK_JSON_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/lock.h 1:16.1.1~dfsg-1/include/asterisk/lock.h
--- 1:13.23.1~dfsg-2/include/asterisk/lock.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/lock.h	2018-12-26 18:45:13.000000000 +0000
@@ -92,11 +92,11 @@
 #define AST_LOCK_TRACK_INIT_VALUE { { NULL }, { 0 }, 0, { NULL }, { 0 }, PTHREAD_MUTEX_INIT_VALUE }
 #endif
 
-#define AST_MUTEX_INIT_VALUE { PTHREAD_MUTEX_INIT_VALUE, NULL, 1 }
-#define AST_MUTEX_INIT_VALUE_NOTRACKING { PTHREAD_MUTEX_INIT_VALUE, NULL, 0 }
+#define AST_MUTEX_INIT_VALUE { PTHREAD_MUTEX_INIT_VALUE, NULL, {1, 0} }
+#define AST_MUTEX_INIT_VALUE_NOTRACKING { PTHREAD_MUTEX_INIT_VALUE, NULL, {0, 0} }
 
-#define AST_RWLOCK_INIT_VALUE { __AST_RWLOCK_INIT_VALUE, NULL, 1 }
-#define AST_RWLOCK_INIT_VALUE_NOTRACKING { __AST_RWLOCK_INIT_VALUE, NULL, 0 }
+#define AST_RWLOCK_INIT_VALUE { __AST_RWLOCK_INIT_VALUE, NULL, {1, 0} }
+#define AST_RWLOCK_INIT_VALUE_NOTRACKING { __AST_RWLOCK_INIT_VALUE, NULL, {0, 0} }
 
 #define AST_MAX_REENTRANCY 10
 
@@ -120,6 +120,13 @@ struct ast_lock_track {
 	pthread_mutex_t reentr_mutex;
 };
 
+struct ast_lock_track_flags {
+	/*! non-zero if lock tracking is enabled */
+	unsigned int tracking:1;
+	/*! non-zero if track is setup */
+	volatile unsigned int setup:1;
+};
+
 /*! \brief Structure for mutex and tracking information.
  *
  * We have tracking information in this structure regardless of DEBUG_THREADS being enabled.
@@ -127,9 +134,18 @@ struct ast_lock_track {
  */
 struct ast_mutex_info {
 	pthread_mutex_t mutex;
-	/*! Track which thread holds this mutex */
+#if !defined(DEBUG_THREADS) && !defined(DEBUG_THREADS_LOOSE_ABI)
+	/*!
+	 * These fields are renamed to ensure they are never used when
+	 * DEBUG_THREADS is not defined.
+	 */
+	struct ast_lock_track *_track;
+	struct ast_lock_track_flags _flags;
+#elif defined(DEBUG_THREADS)
+	/*! Track which thread holds this mutex. */
 	struct ast_lock_track *track;
-	unsigned int tracking:1;
+	struct ast_lock_track_flags flags;
+#endif
 };
 
 /*! \brief Structure for rwlock and tracking information.
@@ -139,9 +155,18 @@ struct ast_mutex_info {
  */
 struct ast_rwlock_info {
 	pthread_rwlock_t lock;
+#if !defined(DEBUG_THREADS) && !defined(DEBUG_THREADS_LOOSE_ABI)
+	/*!
+	 * These fields are renamed to ensure they are never used when
+	 * DEBUG_THREADS is not defined.
+	 */
+	struct ast_lock_track *_track;
+	struct ast_lock_track_flags _flags;
+#elif defined(DEBUG_THREADS)
 	/*! Track which thread holds this lock */
 	struct ast_lock_track *track;
-	unsigned int tracking:1;
+	struct ast_lock_track_flags flags;
+#endif
 };
 
 typedef struct ast_mutex_info ast_mutex_t;
@@ -216,8 +241,6 @@ int __ast_rwlock_trywrlock(const char *f
 
 #ifdef DEBUG_THREADS
 
-#define __ast_mutex_logger(...)  do { if (canlog) ast_log(LOG_ERROR, __VA_ARGS__); else fprintf(stderr, __VA_ARGS__); } while (0)
-
 #ifdef THREAD_CRASH
 #define DO_THREAD_CRASH do { *((int *)(0)) = 1; } while(0)
 #else
@@ -240,41 +263,18 @@ enum ast_lock_type {
  * lock info struct.  The lock is marked as pending as the thread is waiting
  * on the lock.  ast_mark_lock_acquired() will mark it as held by this thread.
  */
-#if !defined(LOW_MEMORY)
-#ifdef HAVE_BKTR
 void ast_store_lock_info(enum ast_lock_type type, const char *filename,
 	int line_num, const char *func, const char *lock_name, void *lock_addr, struct ast_bt *bt);
-#else
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-	int line_num, const char *func, const char *lock_name, void *lock_addr);
-#endif /* HAVE_BKTR */
-
-#else
-
-#ifdef HAVE_BKTR
-#define ast_store_lock_info(I,DONT,CARE,ABOUT,THE,PARAMETERS,BUD)
-#else
-#define ast_store_lock_info(I,DONT,CARE,ABOUT,THE,PARAMETERS)
-#endif /* HAVE_BKTR */
-#endif /* !defined(LOW_MEMORY) */
 
 /*!
  * \brief Mark the last lock as acquired
  */
-#if !defined(LOW_MEMORY)
 void ast_mark_lock_acquired(void *lock_addr);
-#else
-#define ast_mark_lock_acquired(ignore)
-#endif
 
 /*!
  * \brief Mark the last lock as failed (trylock)
  */
-#if !defined(LOW_MEMORY)
 void ast_mark_lock_failed(void *lock_addr);
-#else
-#define ast_mark_lock_failed(ignore)
-#endif
 
 /*!
  * \brief remove lock info for the current thread
@@ -282,23 +282,9 @@ void ast_mark_lock_failed(void *lock_add
  * this gets called by ast_mutex_unlock so that information on the lock can
  * be removed from the current thread's lock info struct.
  */
-#if !defined(LOW_MEMORY)
-#ifdef HAVE_BKTR
 void ast_remove_lock_info(void *lock_addr, struct ast_bt *bt);
-#else
-void ast_remove_lock_info(void *lock_addr);
-#endif /* HAVE_BKTR */
 void ast_suspend_lock_info(void *lock_addr);
 void ast_restore_lock_info(void *lock_addr);
-#else
-#ifdef HAVE_BKTR
-#define ast_remove_lock_info(ignore,me)
-#else
-#define ast_remove_lock_info(ignore)
-#endif /* HAVE_BKTR */
-#define ast_suspend_lock_info(ignore);
-#define ast_restore_lock_info(ignore);
-#endif /* !defined(LOW_MEMORY) */
 
 /*!
  * \brief log info for the current lock with ast_log().
@@ -333,11 +319,7 @@ struct ast_str *ast_dump_locks(void);
  * this gets called during deadlock avoidance, so that the information may
  * be preserved as to what location originally acquired the lock.
  */
-#if !defined(LOW_MEMORY)
 int ast_find_lock_info(void *lock_addr, char *filename, size_t filename_size, int *lineno, char *func, size_t func_size, char *mutex_name, size_t mutex_name_size);
-#else
-#define ast_find_lock_info(a,b,c,d,e,f,g,h) -1
-#endif
 
 /*!
  * \brief Unlock a lock briefly
@@ -658,97 +640,130 @@ static void  __attribute__((destructor))
 #define pthread_create __use_ast_pthread_create_instead__
 #endif
 
-/*
- * Support for atomic instructions.
- * For platforms that have it, use the native cpu instruction to
- * implement them. For other platforms, resort to a 'slow' version
- * (defined in utils.c) that protects the atomic instruction with
- * a single lock.
- * The slow versions is always available, for testing purposes,
- * as ast_atomic_fetchadd_int_slow()
+/*!
+ * \brief Support for atomic instructions.
+ *
+ * These macros implement a uniform interface to use built-in atomic functionality.
+ * If available __atomic built-ins are prefered.  Legacy __sync built-ins are used
+ * as a fallback for older compilers.
+ *
+ * Detailed documentation can be found in the GCC manual, all API's are modeled after
+ * the __atomic interfaces but using the namespace ast_atomic.
+ *
+ * The memorder argument is always ignored by legacy __sync functions.  Invalid
+ * memorder arguments do not produce errors unless __atomic functions are supported
+ * as the argument is erased by the preprocessor.
+ *
+ * \note ast_atomic_fetch_nand and ast_atomic_nand_fetch purposely do not exist.
+ *       It's implementation was broken prior to gcc-4.4.
+ *
+ * @{
  */
 
-int ast_atomic_fetchadd_int_slow(volatile int *p, int v);
-
 #include "asterisk/inline_api.h"
 
-#if defined(HAVE_OSX_ATOMICS)
-#include "libkern/OSAtomic.h"
+#if defined(HAVE_C_ATOMICS)
+/*! Atomic += */
+#define ast_atomic_fetch_add(ptr, val, memorder)  __atomic_fetch_add((ptr), (val), (memorder))
+#define ast_atomic_add_fetch(ptr, val, memorder)  __atomic_add_fetch((ptr), (val), (memorder))
+
+/*! Atomic -= */
+#define ast_atomic_fetch_sub(ptr, val, memorder)  __atomic_fetch_sub((ptr), (val), (memorder))
+#define ast_atomic_sub_fetch(ptr, val, memorder)  __atomic_sub_fetch((ptr), (val), (memorder))
+
+/*! Atomic &= */
+#define ast_atomic_fetch_and(ptr, val, memorder)  __atomic_fetch_and((ptr), (val), (memorder))
+#define ast_atomic_and_fetch(ptr, val, memorder)  __atomic_and_fetch((ptr), (val), (memorder))
+
+/*! Atomic |= */
+#define ast_atomic_fetch_or(ptr, val, memorder)   __atomic_fetch_or((ptr), (val), (memorder))
+#define ast_atomic_or_fetch(ptr, val, memorder)   __atomic_or_fetch((ptr), (val), (memorder))
+
+/*! Atomic xor = */
+#define ast_atomic_fetch_xor(ptr, val, memorder)  __atomic_fetch_xor((ptr), (val), (memorder))
+#define ast_atomic_xor_fetch(ptr, val, memorder)  __atomic_xor_fetch((ptr), (val), (memorder))
+
+#if 0
+/* Atomic compare and swap
+ *
+ * See comments near the __atomic implementation for why this is disabled.
+ */
+#define ast_atomic_compare_exchange_n(ptr, expected, desired, success_memorder, failure_memorder) \
+	__atomic_compare_exchange_n((ptr), (expected), (desired), 0, success_memorder, failure_memorder)
+
+#define ast_atomic_compare_exchange(ptr, expected, desired, success_memorder, failure_memorder) \
+	__atomic_compare_exchange((ptr), (expected), (desired), 0, success_memorder, failure_memorder)
 #endif
 
-/*! \brief Atomically add v to *p and return * the previous value of *p.
- * This can be used to handle reference counts, and the return value
- * can be used to generate unique identifiers.
+#elif defined(HAVE_GCC_ATOMICS)
+/*! Atomic += */
+#define ast_atomic_fetch_add(ptr, val, memorder)  __sync_fetch_and_add((ptr), (val))
+#define ast_atomic_add_fetch(ptr, val, memorder)  __sync_add_and_fetch((ptr), (val))
+
+/*! Atomic -= */
+#define ast_atomic_fetch_sub(ptr, val, memorder)  __sync_fetch_and_sub((ptr), (val))
+#define ast_atomic_sub_fetch(ptr, val, memorder)  __sync_sub_and_fetch((ptr), (val))
+
+/*! Atomic &= */
+#define ast_atomic_fetch_and(ptr, val, memorder)  __sync_fetch_and_and((ptr), (val))
+#define ast_atomic_and_fetch(ptr, val, memorder)  __sync_and_and_fetch((ptr), (val))
+
+/*! Atomic |= */
+#define ast_atomic_fetch_or(ptr, val, memorder)  __sync_fetch_and_or((ptr), (val))
+#define ast_atomic_or_fetch(ptr, val, memorder)  __sync_or_and_fetch((ptr), (val))
+
+/*! Atomic xor = */
+#define ast_atomic_fetch_xor(ptr, val, memorder)  __sync_fetch_and_xor((ptr), (val))
+#define ast_atomic_xor_fetch(ptr, val, memorder)  __sync_xor_and_fetch((ptr), (val))
+
+#if 0
+/* Atomic compare and swap
+ *
+ * The \a expected argument is a pointer, I'm guessing __atomic built-ins
+ * perform all memory reads/writes in a single atomic operation.  I don't
+ * believe this is possible to exactly replicate using __sync built-ins.
+ * Will need to determine potential use cases of this feature and write a
+ * wrapper which provides consistant behavior between __sync and __atomic
+ * implementations.
  */
+#define ast_atomic_compare_exchange_n(ptr, expected, desired, success_memorder, failure_memorder) \
+	__sync_bool_compare_and_swap((ptr), *(expected), (desired))
 
-#if defined(HAVE_GCC_ATOMICS)
-AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
-{
-	return __sync_fetch_and_add(p, v);
-})
-#elif defined(HAVE_OSX_ATOMICS) && (SIZEOF_INT == 4)
-AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
-{
-	return OSAtomicAdd32(v, (int32_t *) p) - v;
-})
-#elif defined(HAVE_OSX_ATOMICS) && (SIZEOF_INT == 8)
-AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
-{
-	return OSAtomicAdd64(v, (int64_t *) p) - v;
-})
-#elif defined (__i386__) || defined(__x86_64__)
-#ifdef sun
-AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
-{
-	__asm __volatile (
-	"       lock;  xaddl   %0, %1 ;        "
-	: "+r" (v),                     /* 0 (result) */
-	  "=m" (*p)                     /* 1 */
-	: "m" (*p));                    /* 2 */
-	return (v);
-})
-#else /* ifndef sun */
-AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
-{
-	__asm __volatile (
-	"       lock   xaddl   %0, %1 ;        "
-	: "+r" (v),                     /* 0 (result) */
-	  "=m" (*p)                     /* 1 */
-	: "m" (*p));                    /* 2 */
-	return (v);
-})
+#define ast_atomic_compare_exchange(ptr, expected, desired, success_memorder, failure_memorder) \
+	__sync_bool_compare_and_swap((ptr), *(expected), *(desired))
+#endif
+
+#else
+#error "Atomics not available."
 #endif
-#else   /* low performance version in utils.c */
+
+/*! Atomic flag set */
+#define ast_atomic_flag_set(ptr, val, memorder)   ast_atomic_fetch_or((ptr), (val), (memorder))
+
+/*! Atomic flag clear */
+#define ast_atomic_flag_clear(ptr, val, memorder) ast_atomic_fetch_and((ptr), ~(val), (memorder))
+
+/*!
+ * \brief Atomically add v to *p and return the previous value of *p.
+ *
+ * This can be used to handle reference counts, and the return value
+ * can be used to generate unique identifiers.
+ */
 AST_INLINE_API(int ast_atomic_fetchadd_int(volatile int *p, int v),
 {
-	return ast_atomic_fetchadd_int_slow(p, v);
+	return ast_atomic_fetch_add(p, v, __ATOMIC_RELAXED);
 })
-#endif
 
-/*! \brief decrement *p by 1 and return true if the variable has reached 0.
+/*!
+ * \brief decrement *p by 1 and return true if the variable has reached 0.
+ *
  * Useful e.g. to check if a refcount has reached 0.
  */
-#if defined(HAVE_GCC_ATOMICS)
 AST_INLINE_API(int ast_atomic_dec_and_test(volatile int *p),
 {
-	return __sync_sub_and_fetch(p, 1) == 0;
+	return ast_atomic_sub_fetch(p, 1, __ATOMIC_RELAXED) == 0;
 })
-#elif defined(HAVE_OSX_ATOMICS) && (SIZEOF_INT == 4)
-AST_INLINE_API(int ast_atomic_dec_and_test(volatile int *p),
-{
-	return OSAtomicAdd32( -1, (int32_t *) p) == 0;
-})
-#elif defined(HAVE_OSX_ATOMICS) && (SIZEOF_INT == 8)
-AST_INLINE_API(int ast_atomic_dec_and_test(volatile int *p),
-{
-	return OSAtomicAdd64( -1, (int64_t *) p) == 0;
-})
-#else
-AST_INLINE_API(int ast_atomic_dec_and_test(volatile int *p),
-{
-	int a = ast_atomic_fetchadd_int(p, -1);
-	return a == 1; /* true if the value is 0 now (so it was 1 previously) */
-})
-#endif
+
+/*! @} */
 
 #endif /* _ASTERISK_LOCK_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/logger.h 1:16.1.1~dfsg-1/include/asterisk/logger.h
--- 1:13.23.1~dfsg-2/include/asterisk/logger.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/logger.h	2018-12-26 18:45:13.000000000 +0000
@@ -69,6 +69,9 @@ enum ast_logger_results {
 void ast_log(int level, const char *file, int line, const char *function, const char *fmt, ...)
 	__attribute__((format(printf, 5, 6)));
 
+void ast_log_ap(int level, const char *file, int line, const char *function, const char *fmt, va_list ap)
+	 __attribute__((format(printf, 5, 0)));
+
 /*!
  * \brief Used for sending a log message with protection against recursion.
  *
@@ -81,7 +84,7 @@ void ast_log_safe(int level, const char
 	__attribute__((format(printf, 5, 6)));
 
 /* XXX needs documentation */
-struct ast_callid;
+typedef unsigned int ast_callid;
 
 /*! \brief Used for sending a log message with a known call_id
 	This is a modified logger function which is functionally identical to the above logger function,
@@ -95,7 +98,7 @@ struct ast_callid;
 	\param callid	This is the ast_callid that is associated with the log message. May be NULL.
 	\param fmt	This is what is important.  The format is the same as your favorite breed of printf.  You know how that works, right? :-)
 */
-void ast_log_callid(int level, const char *file, int line, const char *function, struct ast_callid *callid, const char *fmt, ...)
+void ast_log_callid(int level, const char *file, int line, const char *function, ast_callid callid, const char *fmt, ...)
 	__attribute__((format(printf, 6, 7)));
 
 /*!
@@ -133,9 +136,6 @@ int ast_logger_remove_channel(const char
  */
 void ast_log_backtrace(void);
 
-/*! \brief Reload logger without rotating log files */
-int logger_reload(void);
-
 /*! \brief Reload logger while rotating log files */
 int ast_logger_rotate(void);
 
@@ -172,12 +172,12 @@ void __attribute__((format(printf, 5, 6)
  * allow you to specify that a log will never display a call id even when there is a call id bound to the
  * thread.
  */
-void __attribute__((format(printf, 6, 7))) __ast_verbose_callid(const char *file, int line, const char *func, int level, struct ast_callid *callid, const char *fmt, ...);
+void __attribute__((format(printf, 6, 7))) __ast_verbose_callid(const char *file, int line, const char *func, int level, ast_callid callid, const char *fmt, ...);
 
 #define ast_verbose(...) __ast_verbose(__FILE__, __LINE__, __PRETTY_FUNCTION__, -1, __VA_ARGS__)
 #define ast_verbose_callid(callid, ...) __ast_verbose_callid(__FILE__, __LINE__, __PRETTY_FUNCTION__, -1, callid, __VA_ARGS__)
 
-void __attribute__((format(printf, 6, 0))) __ast_verbose_ap(const char *file, int line, const char *func, int level, struct ast_callid *callid, const char *fmt, va_list ap);
+void __attribute__((format(printf, 6, 0))) __ast_verbose_ap(const char *file, int line, const char *func, int level, ast_callid callid, const char *fmt, va_list ap);
 
 void __attribute__((format(printf, 2, 3))) ast_child_verbose(int level, const char *fmt, ...);
 
@@ -195,11 +195,25 @@ int ast_unregister_verbose(void (*verbos
 void ast_console_puts(const char *string);
 
 /*!
- * \brief log the string to the console, and all attached
- * console clients
+ * \brief log the string to the console, and all attached console clients
+ *
+ * \param string The message to write to the console
+ * \param level The log level of the message
+ *
  * \version 1.6.1 added level parameter
  */
 void ast_console_puts_mutable(const char *string, int level);
+
+/*!
+ * \brief log the string to the console, and all attached console clients
+ * \since 14.0.0
+ *
+ * \param message The message to write to the console
+ * \param sublevel If the log level supports it, the sub-level of the message
+ * \param level The log level of the message
+ */
+void ast_console_puts_mutable_full(const char *message, int level, int sublevel);
+
 void ast_console_toggle_mute(int fd, int silent);
 
 /*!
@@ -296,14 +310,6 @@ void ast_console_toggle_loglevel(int fd,
 unsigned int ast_debug_get_by_module(const char *module);
 
 /*!
- * \brief Get the verbose level for a module
- * \param module the name of module
- * \return the verbose level
- * \version 11.0.0 deprecated
- */
-unsigned int ast_verbose_get_by_module(const char *module) __attribute__((deprecated));
-
-/*!
  * \brief Register a new logger level
  * \param name The name of the level to be registered
  * \retval -1 if an error occurs
@@ -332,49 +338,17 @@ const char *ast_logger_get_dateformat(vo
 /*!
  * \brief factory function to create a new uniquely identifying callid.
  *
- * \retval ast_callid struct pointer containing the call id
- *
- * \note The newly created callid will be referenced upon creation and this function should be
- * paired with a call to ast_callid_unref()
+ * \retval The call id
  */
-struct ast_callid *ast_create_callid(void);
+ast_callid ast_create_callid(void);
 
 /*!
  * \brief extracts the callerid from the thread
  *
- * \retval ast_callid reference to call_id related to the thread
- * \retval NULL if no call_id is present in the thread
- *
- * This reference must be unreffed before it loses scope to prevent memory leaks.
- */
-struct ast_callid *ast_read_threadstorage_callid(void);
-
-/*!
- * \brief Increase callid reference count
- *
- * \param c the ast_callid
- *
- * \retval c always
- */
-#define ast_callid_ref(c) ({ ao2_ref(c, +1); (c); })
-
-/*!
- * \brief Decrease callid reference count
- *
- * \param c the ast_callid
- *
- * \retval NULL always
- */
-#define ast_callid_unref(c) ({ ao2_ref(c, -1); (struct ast_callid *) (NULL); })
-
-/*!
- * \brief Cleanup a callid reference (NULL safe ao2 unreference)
- *
- * \param c the ast_callid
- *
- * \retval NULL always
+ * \retval Non-zero Call id related to the thread
+ * \retval 0 if no call_id is present in the thread
  */
-#define ast_callid_cleanup(c) ({ ao2_cleanup(c); (struct ast_callid *) (NULL); })
+ast_callid ast_read_threadstorage_callid(void);
 
 /*!
  * \brief Sets what is stored in the thread storage to the given
@@ -383,7 +357,7 @@ struct ast_callid *ast_read_threadstorag
  * \retval 0 - success
  * \retval non-zero - failure
  */
-int ast_callid_threadassoc_change(struct ast_callid *callid);
+int ast_callid_threadassoc_change(ast_callid callid);
 
 /*!
  * \brief Adds a known callid to thread storage of the calling thread
@@ -391,7 +365,7 @@ int ast_callid_threadassoc_change(struct
  * \retval 0 - success
  * \retval non-zero - failure
  */
-int ast_callid_threadassoc_add(struct ast_callid *callid);
+int ast_callid_threadassoc_add(ast_callid callid);
 
 /*!
  * \brief Removes callid from thread storage of the calling thread
@@ -406,12 +380,12 @@ int ast_callid_threadassoc_remove(void);
  *        If not, then a new one will be created, bound to the thread, and a reference
  *        to it will be stored.
  *
- * \param callid pointer to struct pointer used to store the referenced callid
+ * \param callid pointer to store the callid
  * \retval 0 - callid was found
  * \retval 1 - callid was created
  * \retval -1 - the function failed somehow (presumably memory problems)
  */
-int ast_callid_threadstorage_auto(struct ast_callid **callid);
+int ast_callid_threadstorage_auto(ast_callid *callid);
 
 /*!
  * \brief Use in conjunction with ast_callid_threadstorage_auto. Cleans up the
@@ -420,7 +394,7 @@ int ast_callid_threadstorage_auto(struct
  * \param callid The callid set by ast_callid_threadstorage_auto
  * \param callid_created The integer returned through ast_callid_threadstorage_auto
  */
-void ast_callid_threadstorage_auto_clean(struct ast_callid *callid, int callid_created);
+void ast_callid_threadstorage_auto_clean(ast_callid callid, int callid_created);
 
 /*!
  * \brief copy a string representation of the callid into a target string
@@ -429,7 +403,7 @@ void ast_callid_threadstorage_auto_clean
  * \param buffer_size maximum writable length of the string (Less than 13 will result in truncation)
  * \param callid Callid for which string is being requested
  */
-void ast_callid_strnprint(char *buffer, size_t buffer_size, struct ast_callid *callid);
+void ast_callid_strnprint(char *buffer, size_t buffer_size, ast_callid callid);
 
 /*!
  * \brief Send a log message to a dynamically registered log level
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/manager.h 1:16.1.1~dfsg-1/include/asterisk/manager.h
--- 1:13.23.1~dfsg-2/include/asterisk/manager.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/manager.h	2018-12-26 18:45:13.000000000 +0000
@@ -54,7 +54,7 @@
 - \ref manager.c Main manager code file
  */
 
-#define AMI_VERSION                     "2.10.4"
+#define AMI_VERSION                     "5.0.0"
 #define DEFAULT_MANAGER_PORT 5038	/* Default port for Asterisk management via TCP */
 #define DEFAULT_MANAGER_TLS_PORT 5039	/* Default port for Asterisk management via TCP */
 
@@ -180,10 +180,10 @@ struct manager_action {
 
 /*! \brief External routines may register/unregister manager callbacks this way
  * \note  Use ast_manager_register2() to register with help text for new manager commands */
-#define ast_manager_register(action, authority, func, synopsis) ast_manager_register2(action, authority, func, ast_module_info->self, synopsis, NULL)
+#define ast_manager_register(action, authority, func, synopsis) ast_manager_register2(action, authority, func, AST_MODULE_SELF, synopsis, NULL)
 
 /*! \brief Register a manager callback using XML documentation to describe the manager. */
-#define ast_manager_register_xml(action, authority, func) ast_manager_register2(action, authority, func, ast_module_info->self, NULL, NULL)
+#define ast_manager_register_xml(action, authority, func) ast_manager_register2(action, authority, func, AST_MODULE_SELF, NULL, NULL)
 
 /*!
  * \brief Register a manager callback using XML documentation to describe the manager.
@@ -356,12 +356,6 @@ void __attribute__((format(printf, 2, 3)
 /*! \brief Determinie if a manager session ident is authenticated */
 int astman_is_authed(uint32_t ident);
 
-/*! \brief Called by Asterisk initialization */
-int init_manager(void);
-
-/*! \brief Called by Asterisk module functions and the CLI command */
-int reload_manager(void);
-
 /*!
  * \brief Add a datastore to a session
  *
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/media_cache.h 1:16.1.1~dfsg-1/include/asterisk/media_cache.h
--- 1:13.23.1~dfsg-2/include/asterisk/media_cache.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/media_cache.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,175 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Matt Jordan <mjordan@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief An in-memory media cache
+ */
+
+#ifndef _ASTERISK_MEDIA_CACHE_H
+#define _ASTERISK_MEDIA_CACHE_H
+
+#if defined(__cplusplus) || defined(c_plusplus)
+extern "C" {
+#endif
+
+struct ast_variable;
+
+/*!
+ * \brief Check if an item exists in the cache
+ *
+ * \param uri The unique URI for the media item
+ *
+ * \retval 0 uri does not exist in cache
+ * \retval 1 uri does exist in cache
+ */
+int ast_media_cache_exists(const char *uri);
+
+/*!
+ * \brief Retrieve an item from the cache
+ *
+ * \param uri The unique URI for the media item
+ * \param preferred_file_name The preferred name for the file storing the
+ *                            media once it is retrieved. Can be NULL.
+ * \param file_path Buffer to store the full path to the media in the
+ *                  cache
+ * \param len The length of the buffer pointed to by \c file_path
+ *
+ * \retval 0 The item was retrieved successfully
+ * \retval -1 The item could not be retrieved
+ *
+ * Example Usage:
+ * \code
+ * char media[PATH_MAX];
+ * int res;
+ *
+ * res = ast_media_cache_retrieve("http://localhost/foo.wav", NULL,
+ * 			media, sizeof(media));
+ * \endcode
+ *
+ * \details
+ * Retrieving an item will cause the \ref bucket Bucket backend associated
+ * with the URI scheme in \c uri to be queried. If the Bucket backend
+ * does not require an update, the cached information is used to find the
+ * file associated with \c uri, and \c file_path is populated with the
+ * location of the media file associated with \c uri.
+ *
+ * If the item is not in the cache, the item will be retrieved using the
+ * \ref bucket backend. When this occurs, if \c preferred_file_name is given,
+ * it will be used as the destination file for the retrieval. When retrieval
+ * of the media from the backend is complete, \c file_path is then populated
+ * as before.
+ */
+int ast_media_cache_retrieve(const char *uri, const char *preferred_file_name,
+	char *file_path, size_t len);
+
+/*!
+ * \brief Retrieve metadata from an item in the cache
+ *
+ * \param uri The unique URI for the media item
+ * \param key The key of the metadata to retrieve
+ * \param value Buffer to store the value in
+ * \param len The length of the buffer pointed to by \c value
+ *
+ * \retval 0 The metadata was retrieved successfully
+ * \retval -1 The metadata could not be retrieved
+ *
+ * Example Usage:
+ * \code
+ *
+ * int res;
+ * char file_size[32];
+ *
+ * res = ast_media_cache_retrieve_metadata("http://localhost/foo.wav", "size",
+ * 			file_size, sizeof(file_size));
+ * \endcode
+ */
+int ast_media_cache_retrieve_metadata(const char *uri, const char *key,
+	char *value, size_t len);
+
+/*!
+ * \brief Create/update a cached media item
+ *
+ * \param uri The unique URI for the media item to store in the cache
+ * \param file_path Full path to the media file to be cached
+ * \param metadata Metadata to store with the cached item
+ *
+ * \retval 0 The item was cached
+ * \retval -1 An error occurred when creating/updating the item
+ *
+ * Example Usage:
+ * \code
+ * int res;
+ *
+ * res = ast_media_cache_create_or_update("http://localhost/foo.wav",
+ * 		"/tmp/foo.wav", NULL);
+ * \endcode
+ *
+ * \note This method will overwrite whatever has been provided by the
+ * \ref bucket backend.
+ *
+ * \details
+ * While \ref ast_media_cache_retrieve is used to retrieve media from
+ * some \ref bucket provider, this method allows for overwriting what
+ * is provided by a backend with some local media. This is useful for
+ * reconstructing or otherwise associating local media with a remote
+ * URI, deferring updating of the media from the backend to some later
+ * retrieval.
+ */
+int ast_media_cache_create_or_update(const char *uri, const char *file_path,
+	struct ast_variable *metadata);
+
+/*!
+ * \brief Remove an item from the media cache
+ *
+ * \param uri The unique URI for the media item to store in the cache
+ *
+ * \retval 0 success
+ * \retval -1 error
+ *
+ * Example Usage:
+ * \code
+ * int res;
+ *
+ * res = ast_media_cache_delete("http://localhost/foo.wav");
+ * \endcode
+ *
+ * \details
+ * This removes an item completely from the media cache. Any files local
+ * on disk associated with the item are deleted as well.
+ *
+ * \note It is up to the \ref bucket implementation whether or not this
+ * affects any non-local storage
+ */
+int ast_media_cache_delete(const char *uri);
+
+/*!
+ * \brief Initialize the media cache
+ *
+ * \note This should only be called once, during Asterisk initialization
+ *
+ * \retval 0 success
+ * \retval -1 error
+ */
+int ast_media_cache_init(void);
+
+#if defined(__cplusplus) || defined(c_plusplus)
+}
+#endif
+
+#endif /* _ASTERISK_MEDIA_CACHE_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/mod_format.h 1:16.1.1~dfsg-1/include/asterisk/mod_format.h
--- 1:13.23.1~dfsg-2/include/asterisk/mod_format.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/mod_format.h	2018-12-26 18:45:13.000000000 +0000
@@ -44,6 +44,7 @@ struct ast_format_def {
 	char name[80];		/*!< Name of format */
 	char exts[80];		/*!< Extensions (separated by | if more than one)
 						 * this format can read.  First is assumed for writing (e.g. .mp3) */
+	char mime_types[80]; /*!< MIME Types related to the format (separated by | if more than one)*/
 	struct ast_format *format;	/*!< Format of frames it uses/provides (one only) */
 	/*!
 	 * \brief Prepare an input stream for playback.
@@ -132,7 +133,7 @@ struct ast_filestream {
  * \retval -1 on failure
  */
 int __ast_format_def_register(const struct ast_format_def *f, struct ast_module *mod);
-#define ast_format_def_register(f) __ast_format_def_register(f, ast_module_info->self)
+#define ast_format_def_register(f) __ast_format_def_register(f, AST_MODULE_SELF)
 
 /*!
  * \brief Unregisters a file format
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/module.h 1:16.1.1~dfsg-1/include/asterisk/module.h
--- 1:13.23.1~dfsg-2/include/asterisk/module.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/module.h	2018-12-26 18:45:13.000000000 +0000
@@ -66,11 +66,40 @@ enum ast_module_unload_mode {
 };
 
 enum ast_module_load_result {
-	AST_MODULE_LOAD_SUCCESS = 0,    /*!< Module loaded and configured */
-	AST_MODULE_LOAD_DECLINE = 1,    /*!< Module is not configured */
-	AST_MODULE_LOAD_SKIP = 2,       /*!< Module was skipped for some reason (For loader.c use only. Should never be returned by modules)*/
-	AST_MODULE_LOAD_PRIORITY = 3,   /*!< Module is not loaded yet, but is added to priority list */
-	AST_MODULE_LOAD_FAILURE = -1,   /*!< Module could not be loaded properly */
+	/*! Module is loaded and configured. */
+	AST_MODULE_LOAD_SUCCESS = 0,
+	/*!
+	 * \brief Module has failed to load, may be in an inconsistent state.
+	 *
+	 * This value is used when a module fails to start but does not risk
+	 * system-wide stability.  Declined modules will prevent any other
+	 * dependent module from starting.
+	 */
+	AST_MODULE_LOAD_DECLINE = 1,
+	/*! \internal
+	 * \brief Module was skipped for some reason.
+	 *
+	 * \note For loader.c use only. Should never be returned by modules.
+	 */
+	AST_MODULE_LOAD_SKIP = 2,
+	/*! \internal
+	 * \brief Module is not loaded yet, but is added to priority list.
+	 *
+	 * \note For loader.c use only. Should never be returned by modules.
+	 */
+	AST_MODULE_LOAD_PRIORITY = 3,
+	/*!
+	 * \brief Module could not be loaded properly.
+	 *
+	 * This return should only be returned by modules for unrecoverable
+	 * failures that cause the whole system to become unstable.  In almost
+	 * all cases \ref AST_MODULE_LOAD_DECLINE should be used instead.
+	 *
+	 * \warning Returning this code from any module will cause startup to abort.
+	 * If startup is already completed this code has the same effect as
+	 * \ref AST_MODULE_LOAD_DECLINE.
+	 */
+	AST_MODULE_LOAD_FAILURE = -1,
 };
 
 /*!
@@ -252,12 +281,11 @@ int ast_loader_unregister(int (*updater)
  * \param rpos The position we should be matching.  This should be the same as
  *        pos.
  * \param type The type of action that will be performed by CLI.
- *             This argument accepts values of enum ast_module_helper_type.
  *
  * \retval A possible completion of the partial match.
  * \retval NULL if no matches were found.
  */
-char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, int type);
+char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, enum ast_module_helper_type type);
 
 /* Opaque type for module handles generated by the loader */
 
@@ -293,41 +321,41 @@ enum ast_module_load_priority {
 	AST_MODPRI_REALTIME_DEPEND =    10,  /*!< Dependency for a realtime driver */
 	AST_MODPRI_REALTIME_DEPEND2 =   20,  /*!< Second level dependency for a realtime driver (func_curl needs res_curl, but is needed by res_config_curl) */
 	AST_MODPRI_REALTIME_DRIVER =    30,  /*!< A realtime driver, which provides configuration services for other modules */
-	AST_MODPRI_TIMING =             40,  /*!< Dependency for a channel (MOH needs timing interfaces to be fully loaded) */
-	AST_MODPRI_CHANNEL_DEPEND =     50,  /*!< Channel driver dependency (may depend upon realtime, e.g. MOH) */
-	AST_MODPRI_CHANNEL_DRIVER =     60,  /*!< Channel drivers (provide devicestate) */
-	AST_MODPRI_APP_DEPEND =         70,  /*!< Dependency for an application */
-	AST_MODPRI_DEVSTATE_PROVIDER =  80,  /*!< Applications and other modules that _provide_ devicestate (e.g. meetme) */
-	AST_MODPRI_DEVSTATE_PLUGIN =    90,  /*!< Plugin for a module that provides devstate (e.g. res_calendar_*) */
-	AST_MODPRI_CDR_DRIVER =        100,  /*!< CDR or CEL backend */
+	AST_MODPRI_CORE =               40,  /*!< A core module originally meant to start between preload and load. */
+	AST_MODPRI_TIMING =             50,  /*!< Dependency for a channel (MOH needs timing interfaces to be fully loaded) */
+	AST_MODPRI_CHANNEL_DEPEND =     60,  /*!< Channel driver dependency (may depend upon realtime, e.g. MOH) */
+	AST_MODPRI_CHANNEL_DRIVER =     70,  /*!< Channel drivers (provide devicestate) */
+	AST_MODPRI_APP_DEPEND =         80,  /*!< Dependency for an application */
+	AST_MODPRI_DEVSTATE_PROVIDER =  90,  /*!< Applications and other modules that _provide_ devicestate (e.g. meetme) */
+	AST_MODPRI_DEVSTATE_PLUGIN =   100,  /*!< Plugin for a module that provides devstate (e.g. res_calendar_*) */
+	AST_MODPRI_CDR_DRIVER =        110,  /*!< CDR or CEL backend */
 	AST_MODPRI_DEFAULT =           128,  /*!< Modules not otherwise defined (such as most apps) will load here */
 	AST_MODPRI_DEVSTATE_CONSUMER = 150,  /*!< Certain modules, which consume devstate, need to load after all others (e.g. app_queue) */
 };
 
 struct ast_module_info {
-
 	/*!
 	 * The 'self' pointer for a module; it will be set by the loader before
 	 * it calls the module's load_module() entrypoint, and used by various
 	 * other macros that need to identify the module.
 	 */
-
 	struct ast_module *self;
-	enum ast_module_load_result (*load)(void);	/*!< register stuff etc. Optional. */
-	int (*reload)(void);			/*!< config etc. Optional. */
-	int (*unload)(void);			/*!< unload. called with the module locked */
-	int (*backup_globals)(void);		/*!< unused, left in place for ABI compat */
-	void (*restore_globals)(void);		/*!< unused, left in place for ABI compat */
-
-	const char *name;			/*!< name of the module for loader reference and CLI commands */
-	const char *description;		/*!< user friendly description of the module. */
+	/*! Register stuff etc. Optional. */
+	enum ast_module_load_result (*load)(void);
+	/*! Config etc. Optional. */
+	int (*reload)(void);
+	/*! Unload. called with the module locked */
+	int (*unload)(void);
+	/*! Name of the module for loader reference and CLI commands */
+	const char *name;
+	/*! User friendly description of the module. */
+	const char *description;
 
 	/*!
 	 * This holds the ASTERISK_GPL_KEY, signifiying that you agree to the terms of
 	 * the Asterisk license as stated in the ASTERISK_GPL_KEY.  Your module will not
 	 * load if it does not return the EXACT key string.
 	 */
-
 	const char *key;
 	unsigned int flags;
 
@@ -341,10 +369,49 @@ struct ast_module_info {
 	 *  on load. */
 	unsigned char load_pri;
 
-	/*! Modules which should be loaded first, in comma-separated string format.
-	 * These are only required for loading, when the optional_api header file
-	 * detects that the compiler does not support the optional API featureset. */
-	const char *nonoptreq;
+	/*! Modules which must always be started first, in comma-separated string format. */
+	const char *requires;
+
+	/*!
+	 * \brief Comma-separated list of optionally required modules.
+	 *
+	 * The listed modules are optional, but load order is enforced.  For example
+	 * app_voicemail optionally requires res_adsi.  This means that app_voicemail
+	 * will happily load without res_adsi, but if both are being loaded the module
+	 * loader will force res_adsi to start first.
+	 */
+	const char *optional_modules;
+
+	/*!
+	 * \brief Modules that we provide enhanced functionality for.
+	 *
+	 * This is similar to a "requires" but specifies that we add functionality to
+	 * the other modules.  Any module that requires something we "enhances" will
+	 * also require us, but only if we are dlopen'ed.
+	 *
+	 * Example:
+	 * - res_fax_spandsp has .enhances = "res_fax".
+	 * - res_my_module has .requires = "res_fax" but has no direct knowledge
+	 *   of res_fax_spandsp.
+	 *
+	 * This forces the following startup order among the 3 modules:
+	 * 1) res_fax starts.
+	 * 2) res_fax_spandsp starts, holds a reference to res_fax.
+	 * 3) res_mymod starts, holds a reference to res_fax and res_fax_spandsp.
+	 *
+	 * If res_fax_spandsp were not being loaded res_mymod would load with
+	 * res_fax only.  If res_fax_spandsp were later loaded res_mymod would
+	 * get a reference to it.
+	 */
+	const char *enhances;
+
+	/*! These reserved fields should be NULL, they exist to allow addition to this
+	 * structure in a non-breaking way. */
+	void *reserved1;
+	void *reserved2;
+	void *reserved3;
+	void *reserved4;
+
 	/*! The support level for the given module */
 	enum ast_module_support_level support_level;
 };
@@ -356,11 +423,12 @@ struct ast_module_user *__ast_module_use
 void __ast_module_user_remove(struct ast_module *, struct ast_module_user *);
 void __ast_module_user_hangup_all(struct ast_module *);
 
-#define ast_module_user_add(chan) __ast_module_user_add(ast_module_info->self, chan)
-#define ast_module_user_remove(user) __ast_module_user_remove(ast_module_info->self, user)
-#define ast_module_user_hangup_all() __ast_module_user_hangup_all(ast_module_info->self)
+#define ast_module_user_add(chan) __ast_module_user_add(AST_MODULE_SELF, chan)
+#define ast_module_user_remove(user) __ast_module_user_remove(AST_MODULE_SELF, user)
+#define ast_module_user_hangup_all() __ast_module_user_hangup_all(AST_MODULE_SELF)
 
 struct ast_module *__ast_module_ref(struct ast_module *mod, const char *file, int line, const char *func);
+struct ast_module *__ast_module_running_ref(struct ast_module *mod, const char *file, int line, const char *func);
 void __ast_module_shutdown_ref(struct ast_module *mod, const char *file, int line, const char *func);
 void __ast_module_unref(struct ast_module *mod, const char *file, int line, const char *func);
 
@@ -373,6 +441,20 @@ void __ast_module_unref(struct ast_modul
  * from being unloaded.
  */
 #define ast_module_ref(mod)           __ast_module_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
+/*!
+ * \brief Hold a reference to the module if it is running.
+ * \param mod Module to reference
+ * \retval mod if running
+ * \retval NULL if not running
+ *
+ * The returned pointer should be released with ast_module_unref.
+ *
+ * \note A module reference will prevent the module from being unloaded.
+ */
+#define ast_module_running_ref(mod) \
+	__ast_module_running_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 /*!
  * \brief Prevent unload of the module before shutdown
  * \param mod Module to hold
@@ -393,8 +475,6 @@ void __ast_module_unref(struct ast_modul
 		load_func,                                                     \
 		reload_func,                                                   \
 		unload_func,                                                   \
-		NULL,                                                          \
-		NULL,                                                          \
 		AST_MODULE,                                                    \
 		desc,                                                          \
 		keystr,                                                        \
@@ -402,6 +482,12 @@ void __ast_module_unref(struct ast_modul
 		AST_BUILDOPT_SUM,                                              \
 		load_pri,                                                      \
 		NULL,                                                          \
+		NULL,                                                          \
+		NULL,                                                          \
+		NULL,                                                          \
+		NULL,                                                          \
+		NULL,                                                          \
+		NULL,                                                          \
 		support_level,                                                 \
 	};                                                                 \
 	static void  __attribute__((constructor)) __reg_module(void)       \
@@ -412,8 +498,13 @@ void __ast_module_unref(struct ast_modul
 	{                                                                  \
 		ast_module_unregister(&__mod_info);                            \
 	}                                                                  \
+	struct ast_module *AST_MODULE_SELF_SYM(void)                       \
+	{                                                                  \
+		return __mod_info.self;                                        \
+	}                                                                  \
 	static const __attribute__((unused)) struct ast_module_info *ast_module_info = &__mod_info
 
+
 #define AST_MODULE_INFO_STANDARD(keystr, desc)              \
 	AST_MODULE_INFO(keystr, AST_MODFLAG_LOAD_ORDER, desc,   \
 			load_module,                                    \
@@ -445,7 +536,9 @@ void __ast_module_unref(struct ast_modul
 /* forward declare this pointer in modules, so that macro/function
    calls that need it can get it, since it will actually be declared
    and populated at the end of the module's source file... */
+#if !defined(AST_IN_CORE)
 static const __attribute__((unused)) struct ast_module_info *ast_module_info;
+#endif
 
 #define AST_MODULE_INFO(keystr, flags_to_set, desc, fields...)	\
 	static struct ast_module_info 				\
@@ -465,6 +558,10 @@ static const __attribute__((unused)) str
 	{ \
 		ast_module_unregister(&__mod_info); \
 	} \
+	struct ast_module *AST_MODULE_SELF_SYM(void)                       \
+	{                                                                  \
+		return __mod_info.self;                                        \
+	}                                                                  \
 	static const struct ast_module_info *ast_module_info = &__mod_info
 
 #define AST_MODULE_INFO_STANDARD(keystr, desc)              \
@@ -510,7 +607,7 @@ static const __attribute__((unused)) str
  * \retval 0 success
  * \retval -1 failure.
  */
-#define ast_register_application(app, execute, synopsis, description) ast_register_application2(app, execute, synopsis, description, ast_module_info->self)
+#define ast_register_application(app, execute, synopsis, description) ast_register_application2(app, execute, synopsis, description, AST_MODULE_SELF)
 
 /*!
  * \brief Register an application using XML documentation.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/named_locks.h 1:16.1.1~dfsg-1/include/asterisk/named_locks.h
--- 1:13.23.1~dfsg-2/include/asterisk/named_locks.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/named_locks.h	2018-12-26 18:45:13.000000000 +0000
@@ -48,7 +48,7 @@
  * To use a named lock:
  * 	Call ast_named_lock_get with the appropriate keyspace and key.
  * 	Use the standard ao2 lock/unlock functions as needed.
- * 	Call ast_named_lock_put when you're finished with it.
+ * 	Call ao2_cleanup when you're finished with it.
  */
 
 /*!
@@ -66,9 +66,6 @@ struct ast_named_lock;
 struct ast_named_lock *__ast_named_lock_get(const char *filename, int lineno, const char *func,
 	enum ast_named_lock_type lock_type, const char *keyspace, const char *key);
 
-int __ast_named_lock_put(const char *filename, int lineno, const char *func,
-	struct ast_named_lock *lock);
-
 /*!
  * \brief Geta named lock handle
  * \since 13.9.0
@@ -92,11 +89,8 @@ int __ast_named_lock_put(const char *fil
  * \since 13.9.0
  *
  * \param lock The pointer to the ast_named_lock structure returned by ast_named_lock_get
- * \retval 0 Success
- * \retval -1 Failure
  */
-#define ast_named_lock_put(lock) \
-	__ast_named_lock_put(__FILE__, __LINE__, __PRETTY_FUNCTION__, lock)
+#define ast_named_lock_put(lock) ao2_cleanup(lock)
 
 /*!
  * @}
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/netsock2.h 1:16.1.1~dfsg-1/include/asterisk/netsock2.h
--- 1:13.23.1~dfsg-2/include/asterisk/netsock2.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/netsock2.h	2018-12-26 18:45:13.000000000 +0000
@@ -31,6 +31,18 @@ extern "C" {
 
 #include <netinet/in.h>
 
+/*
+ * String buffer size that can accommodate a fully stringified representation of a
+ * supported IP address & port:
+ *
+ * - 45 bytes for an IPv6 address
+ * -  2 bytes for brackets around an IPv6 address
+ * -  1 byte for the port separator (a colon)
+ * -  5 bytes for the port
+ * -  1 byte for the zero-terminator
+ */
+#define AST_SOCKADDR_BUFLEN (45 + 2 + 1 + 5 + 1)
+
 /*!
  * Values for address families that we support. This is reproduced from socket.h
  * because we do not want users to include that file. Only netsock2.c should
@@ -129,6 +141,22 @@ static inline void ast_sockaddr_setnull(
 }
 
 /*!
+ * \brief
+ * Copies the data from a sockaddr to an ast_sockaddr
+ *
+ * \param dst The destination ast_sockaddr
+ * \param src The source sockaddr
+ * \param len Length of the value stored in sockaddr
+ * \retval void
+ */
+static inline void ast_sockaddr_copy_sockaddr(struct ast_sockaddr *dst,
+		struct sockaddr *src, socklen_t len)
+{
+	memcpy(dst, src, len);
+	dst->len = len;
+}
+
+/*!
  * \since 1.8
  *
  * \brief
@@ -422,6 +450,44 @@ int ast_sockaddr_resolve(struct ast_sock
 			 int flags, int family);
 
 /*!
+ * \since 16.0
+ *
+ * \brief
+ * Return the first entry from ast_sockaddr_resolve filtered by address family
+ *
+ * \details
+ * Parses a string containing a host name or an IPv4 or IPv6 address followed
+ * by an optional port (separated by a colon).  This function only returns the
+ * first address into the ast_sockaddr. Allowed formats for name are the following:
+ *
+ * hostname:port
+ * host.example.com:port
+ * a.b.c.d
+ * a.b.c.d:port
+ * a:b:c:...:d
+ * [a:b:c:...:d]
+ * [a:b:c:...:d]:port
+ *
+ * \param[out] addr The resulting ast_sockaddr
+ * \param name The string to parse
+ * \param flags If set to zero, a port MAY be present. If set to
+ * PARSE_PORT_IGNORE, a port MAY be present but will be ignored. If set to
+ * PARSE_PORT_REQUIRE, a port MUST be present. If set to PARSE_PORT_FORBID, a
+ * port MUST NOT be present.
+ *
+ * \param family Only addresses of the given family will be returned. Use 0 or
+ * AST_AF_UNSPEC to specify any address family.  Behavior is ultimately determined
+ * by getaddrinfo in how it orders return results.  First result is selected to
+ * be returned.
+ *
+ * \retval 0 Success
+ * \retval non-zero Failure
+ * \warning Using this function potentially means you have a faulty design.
+ */
+int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
+				      const char* name, int flag, int family);
+
+/*!
  * \brief
  * Apply a netmask to an address and store the result in a separate structure.
  *
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/netsock.h 1:16.1.1~dfsg-1/include/asterisk/netsock.h
--- 1:13.23.1~dfsg-2/include/asterisk/netsock.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/netsock.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,74 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Mark Spencer <markster@digium.com>
- * Kevin P. Fleming <kpfleming@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- * \brief Network socket handling
- *
- * \deprecated Use netsock2.h instead
- */
-
-#ifndef _ASTERISK_NETSOCK_H
-#define _ASTERISK_NETSOCK_H
-
-#if defined(__cplusplus) || defined(c_plusplus)
-extern "C" {
-#endif
-
-#include "asterisk/network.h"
-#include "asterisk/io.h"
-#include "asterisk/netsock2.h"
-
-struct ast_netsock;
-
-struct ast_netsock_list;
-
-struct ast_netsock_list *ast_netsock_list_alloc(void);
-
-int ast_netsock_init(struct ast_netsock_list *list);
-
-struct ast_netsock *ast_netsock_bind(struct ast_netsock_list *list, struct io_context *ioc,
-				     const char *bindinfo, int defaultport, int tos, int cos, ast_io_cb callback, void *data);
-
-struct ast_netsock *ast_netsock_bindaddr(struct ast_netsock_list *list, struct io_context *ioc,
-					 struct ast_sockaddr *bindaddr, int tos, int cos, ast_io_cb callback, void *data);
-
-int ast_netsock_release(struct ast_netsock_list *list);
-
-struct ast_netsock *ast_netsock_find(struct ast_netsock_list *list,
-				     struct ast_sockaddr *addr);
-
-/*!
- * \deprecated Use ast_seq_qos in netsock2.h which properly handles IPv4 and IPv6
- *             sockets, instead.
- */
-int ast_netsock_set_qos(int sockfd, int tos, int cos, const char *desc);
-
-int ast_netsock_sockfd(const struct ast_netsock *ns);
-
-const struct ast_sockaddr *ast_netsock_boundaddr(const struct ast_netsock *ns);
-
-void *ast_netsock_data(const struct ast_netsock *ns);
-
-void ast_netsock_unref(struct ast_netsock *ns);
-
-#if defined(__cplusplus) || defined(c_plusplus)
-}
-#endif
-
-#endif /* _ASTERISK_NETSOCK_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/optional_api.h 1:16.1.1~dfsg-1/include/asterisk/optional_api.h
--- 1:13.23.1~dfsg-2/include/asterisk/optional_api.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/optional_api.h	2018-12-26 18:45:13.000000000 +0000
@@ -174,10 +174,7 @@ void ast_optional_api_unuse(const char *
 
 #define AST_OPTIONAL_API_NAME(name) __##name
 
-#if defined(AST_API_MODULE)
-/* Module defining the API */
-
-#define AST_OPTIONAL_API_IMPL_INIT(name)				\
+#define AST_OPTIONAL_API_INIT_IMPL(name)				\
 	static void __attribute__((constructor)) __init__##name##_impl(void) { \
 		ast_optional_api_provide(#name,				\
 			(ast_optional_fn)AST_OPTIONAL_API_NAME(name));	\
@@ -187,22 +184,7 @@ void ast_optional_api_unuse(const char *
 			(ast_optional_fn)AST_OPTIONAL_API_NAME(name));	\
 	}
 
-#define AST_OPTIONAL_API(result, name, proto, stub)			\
-	result AST_OPTIONAL_API_NAME(name) proto;			\
-	static attribute_unused typeof(AST_OPTIONAL_API_NAME(name)) * const \
-	     name = AST_OPTIONAL_API_NAME(name);			\
-	AST_OPTIONAL_API_IMPL_INIT(name)
-
-#define AST_OPTIONAL_API_ATTR(result, attr, name, proto, stub)		\
-	result  __attribute__((attr)) AST_OPTIONAL_API_NAME(name) proto; \
-	static attribute_unused typeof(AST_OPTIONAL_API_NAME(name)) * const \
-	     name = AST_OPTIONAL_API_NAME(name);			\
-	AST_OPTIONAL_API_IMPL_INIT(name)
-
-#else
-/* Module using the API */
-
-#define AST_OPTIONAL_API_INIT(name)					\
+#define AST_OPTIONAL_API_INIT_USER(name)				\
 	static void __attribute__((constructor)) __init__##name(void) {	\
 		ast_optional_api_use(#name, (ast_optional_fn *)&name,	\
 			(ast_optional_fn)__stub__##name,		\
@@ -213,19 +195,31 @@ void ast_optional_api_unuse(const char *
 			AST_MODULE);					\
 	}
 
-#define AST_OPTIONAL_API(result, name, proto, stub)			\
+#define AST_OPTIONAL_API_IMPL(result, name, proto, stub)			\
+	result AST_OPTIONAL_API_NAME(name) proto;			\
+	static attribute_unused typeof(AST_OPTIONAL_API_NAME(name)) * const \
+	     name = AST_OPTIONAL_API_NAME(name);			\
+	AST_OPTIONAL_API_INIT_IMPL(name)
+
+#define AST_OPTIONAL_API_USER(result, name, proto, stub)			\
 	static result __stub__##name proto stub;			\
 	static attribute_unused						\
 		typeof(__stub__##name) * name;				\
-	AST_OPTIONAL_API_INIT(name)
+	AST_OPTIONAL_API_INIT_USER(name)
+
+
+/* AST_OPTIONAL_API_ATTR */
+#define AST_OPTIONAL_API_ATTR_IMPL(result, attr, name, proto, stub)		\
+	result  __attribute__((attr)) AST_OPTIONAL_API_NAME(name) proto; \
+	static attribute_unused typeof(AST_OPTIONAL_API_NAME(name)) * const \
+	     name = AST_OPTIONAL_API_NAME(name);			\
+	AST_OPTIONAL_API_INIT_IMPL(name)
 
-#define AST_OPTIONAL_API_ATTR(result, attr, name, proto, stub)		\
+#define AST_OPTIONAL_API_ATTR_USER(result, attr, name, proto, stub)		\
 	static __attribute__((attr)) result __stub__##name proto stub;	\
 	static attribute_unused	__attribute__((attr))			\
 		typeof(__stub__##name) * name;				\
-	AST_OPTIONAL_API_INIT(name)
-
-#endif /* defined(AST_API_MODULE) */
+	AST_OPTIONAL_API_INIT_USER(name)
 
 #else /* defined(OPTIONAL_API) */
 
@@ -241,6 +235,38 @@ void ast_optional_api_unuse(const char *
 
 #endif /* defined(OPTIONAL_API) */
 
-#undef AST_API_MODULE
-
 #endif /* __ASTERISK_OPTIONAL_API_H */
+
+/*
+ * Some Asterisk sources are both consumer and provider of optional API's.  The
+ * following definitons are intentionally outside the include protected portion
+ * of this header so AST_OPTIONAL_API and AST_OPTIONAL_API_ATTR can be redefined
+ * each time the header is included.  This also ensures that AST_API_MODULE is
+ * undefined after every include of this header.
+ */
+#if defined(OPTIONAL_API)
+
+#undef AST_OPTIONAL_API
+#undef AST_OPTIONAL_API_ATTR
+
+#if defined(AST_API_MODULE)
+
+#define AST_OPTIONAL_API(result, name, proto, stub) \
+	AST_OPTIONAL_API_IMPL(result, name, proto, stub)
+
+#define AST_OPTIONAL_API_ATTR(result, attr, name, proto, stub) \
+	AST_OPTIONAL_API_ATTR_IMPL(result, attr, name, proto, stub)
+
+#else
+
+#define AST_OPTIONAL_API(result, name, proto, stub) \
+	AST_OPTIONAL_API_USER(result, name, proto, stub)
+
+#define AST_OPTIONAL_API_ATTR(result, attr, name, proto, stub) \
+	AST_OPTIONAL_API_ATTR_USER(result, attr, name, proto, stub)
+
+#endif /* defined(AST_API_MODULE) */
+
+#endif /* defined(OPTIONAL_API) */
+
+#undef AST_API_MODULE
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/options.h 1:16.1.1~dfsg-1/include/asterisk/options.h
--- 1:13.23.1~dfsg-2/include/asterisk/options.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/options.h	2018-12-26 18:45:13.000000000 +0000
@@ -66,30 +66,24 @@ enum ast_option_flags {
 	AST_OPT_FLAG_CACHE_RECORD_FILES = (1 << 13),
 	/*! Display timestamp in CLI verbose output */
 	AST_OPT_FLAG_TIMESTAMP = (1 << 14),
-	/*! Override config */
-	AST_OPT_FLAG_OVERRIDE_CONFIG = (1 << 15),
+	/*! Cache media frames for performance */
+	AST_OPT_FLAG_CACHE_MEDIA_FRAMES = (1 << 15),
 	/*! Reconnect */
 	AST_OPT_FLAG_RECONNECT = (1 << 16),
 	/*! Transmit Silence during Record() and DTMF Generation */
 	AST_OPT_FLAG_TRANSMIT_SILENCE = (1 << 17),
 	/*! Suppress some warnings */
 	AST_OPT_FLAG_DONT_WARN = (1 << 18),
-	/*! End CDRs before the 'h' extension */
-	AST_OPT_FLAG_END_CDR_BEFORE_H_EXTEN = (1 << 19),
-	/*! Cache media frames for performance */
-	AST_OPT_FLAG_CACHE_MEDIA_FRAMES = (1 << 20),
+	/*! Reference Debugging */
+	AST_OPT_FLAG_REF_DEBUG = (1 << 20),
 	/*! Always fork, even if verbose or debug settings are non-zero */
 	AST_OPT_FLAG_ALWAYS_FORK = (1 << 21),
 	/*! Disable log/verbose output to remote consoles */
 	AST_OPT_FLAG_MUTE = (1 << 22),
 	/*! There is a per-module debug setting */
 	AST_OPT_FLAG_DEBUG_MODULE = (1 << 23),
-	/*! There is a per-module verbose setting */
-	AST_OPT_FLAG_VERBOSE_MODULE = (1 << 24),
 	/*! Terminal colors should be adjusted for a light-colored background */
 	AST_OPT_FLAG_LIGHT_BACKGROUND = (1 << 25),
-	/*! Count Initiated seconds in CDR's */
-	AST_OPT_FLAG_INITIATED_SECONDS = (1 << 26),
 	/*! Force black background */
 	AST_OPT_FLAG_FORCE_BLACK_BACKGROUND = (1 << 27),
 	/*! Hide remote console connect messages on console */
@@ -122,20 +116,18 @@ enum ast_option_flags {
 #define ast_opt_cache_record_files	ast_test_flag(&ast_options, AST_OPT_FLAG_CACHE_RECORD_FILES)
 #define ast_opt_cache_media_frames	ast_test_flag(&ast_options, AST_OPT_FLAG_CACHE_MEDIA_FRAMES)
 #define ast_opt_timestamp		ast_test_flag(&ast_options, AST_OPT_FLAG_TIMESTAMP)
-#define ast_opt_override_config		ast_test_flag(&ast_options, AST_OPT_FLAG_OVERRIDE_CONFIG)
 #define ast_opt_reconnect		ast_test_flag(&ast_options, AST_OPT_FLAG_RECONNECT)
 #define ast_opt_transmit_silence	ast_test_flag(&ast_options, AST_OPT_FLAG_TRANSMIT_SILENCE)
 #define ast_opt_dont_warn		ast_test_flag(&ast_options, AST_OPT_FLAG_DONT_WARN)
-#define ast_opt_end_cdr_before_h_exten	ast_test_flag(&ast_options, AST_OPT_FLAG_END_CDR_BEFORE_H_EXTEN)
 #define ast_opt_always_fork		ast_test_flag(&ast_options, AST_OPT_FLAG_ALWAYS_FORK)
 #define ast_opt_mute			ast_test_flag(&ast_options, AST_OPT_FLAG_MUTE)
 #define ast_opt_dbg_module		ast_test_flag(&ast_options, AST_OPT_FLAG_DEBUG_MODULE)
-#define ast_opt_verb_module		ast_test_flag(&ast_options, AST_OPT_FLAG_VERBOSE_MODULE)
 #define ast_opt_light_background	ast_test_flag(&ast_options, AST_OPT_FLAG_LIGHT_BACKGROUND)
 #define ast_opt_force_black_background	ast_test_flag(&ast_options, AST_OPT_FLAG_FORCE_BLACK_BACKGROUND)
 #define ast_opt_hide_connect		ast_test_flag(&ast_options, AST_OPT_FLAG_HIDE_CONSOLE_CONNECT)
 #define ast_opt_lock_confdir		ast_test_flag(&ast_options, AST_OPT_FLAG_LOCK_CONFIG_DIR)
 #define ast_opt_generic_plc         ast_test_flag(&ast_options, AST_OPT_FLAG_GENERIC_PLC)
+#define ast_opt_ref_debug           ast_test_flag(&ast_options, AST_OPT_FLAG_REF_DEBUG)
 #define ast_opt_generic_plc_on_equal_codecs  ast_test_flag(&ast_options, AST_OPT_FLAG_GENERIC_PLC_ON_EQUAL_CODECS)
 
 /*! Maximum log level defined by PJPROJECT. */
@@ -205,6 +197,7 @@ extern char record_cache_dir[AST_CACHE_D
 
 extern int ast_language_is_prefix;
 
+extern int ast_option_rtpusedynamic;
 extern unsigned int ast_option_rtpptdynamic;
 
 #if defined(__cplusplus) || defined(c_plusplus)
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/parking.h 1:16.1.1~dfsg-1/include/asterisk/parking.h
--- 1:13.23.1~dfsg-2/include/asterisk/parking.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/parking.h	2018-12-26 18:45:13.000000000 +0000
@@ -196,7 +196,7 @@ struct ast_parking_bridge_feature_fn_tab
 	int (* parking_park_bridge_channel)(struct ast_bridge_channel *parkee, const char *parkee_uuid, const char *parker_uuid, const char *app_data);
 
 	/*! \brief The module info for the module registering this parking provider */
-	const struct ast_module_info *module_info;
+	struct ast_module *module;
 };
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/pbx.h 1:16.1.1~dfsg-1/include/asterisk/pbx.h
--- 1:13.23.1~dfsg-2/include/asterisk/pbx.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/pbx.h	2018-12-26 18:45:13.000000000 +0000
@@ -109,7 +109,7 @@ struct ast_state_cb_info {
 };
 
 /*! \brief Typedef for devicestate and hint callbacks */
-typedef int (*ast_state_cb_type)(char *context, char *id, struct ast_state_cb_info *info, void *data);
+typedef int (*ast_state_cb_type)(const char *context, const char *exten, struct ast_state_cb_info *info, void *data);
 
 /*! \brief Typedef for devicestate and hint callback removal indication callback */
 typedef void (*ast_state_cb_destroy_type)(int id, void *data);
@@ -288,6 +288,15 @@ int pbx_exec(struct ast_channel *c, stru
 struct ast_context *ast_context_find_or_create(struct ast_context **extcontexts, struct ast_hashtab *exttable, const char *name, const char *registrar);
 
 /*!
+ * \brief Enable or disable autohints support on a context
+ *
+ * \param con pointer to the context
+ * \param enabled whether autohints are enabled
+ *
+ */
+void ast_context_set_autohints(struct ast_context *con, int enabled);
+
+/*!
  * \brief Merge the temporary contexts into a global contexts list and delete from the
  *        global list the ones that are being added
  *
@@ -496,11 +505,15 @@ int ast_add_extension(const char *contex
 /*!
  * \brief Add an extension to an extension context, this time with an ast_context *.
  *
- * \note For details about the arguments, check ast_add_extension()
+ * \param registrar_file optional configuration file that defines this extension
+ * \param registrar_line optional line number of configuration file that defines extension
+ *
+ * \note For details about the other arguments, check ast_add_extension()
  */
 int ast_add_extension2(struct ast_context *con, int replace, const char *extension,
 	int priority, const char *label, const char *callerid,
-	const char *application, void *data, void (*datad)(void *), const char *registrar);
+	const char *application, void *data, void (*datad)(void *), const char *registrar,
+	const char *registrar_file, int registrar_line);
 
 /*!
  * \brief Same as ast_add_extension2, but assumes you have already locked context
@@ -511,7 +524,8 @@ int ast_add_extension2(struct ast_contex
  */
 int ast_add_extension2_nolock(struct ast_context *con, int replace, const char *extension,
 	int priority, const char *label, const char *callerid,
-	const char *application, void *data, void (*datad)(void *), const char *registrar);
+	const char *application, void *data, void (*datad)(void *), const char *registrar,
+	const char *registrar_file, int registrar_line);
 
 /*!
  * \brief Map devstate to an extension state.
@@ -1177,6 +1191,12 @@ int ast_pbx_outgoing_exten(const char *t
 	const char *account, struct ast_channel **locked_channel, int early_media,
 	const struct ast_assigned_ids *assignedids);
 
+int ast_pbx_outgoing_exten_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *context, const char *exten, int priority, int *reason,
+	int synchronous, const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel, int early_media,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee);
+
 /*!
  * \brief Synchronously or asynchronously make an outbound call and execute an
  *  application on the channel.
@@ -1215,6 +1235,12 @@ int ast_pbx_outgoing_app(const char *typ
 	const char *account, struct ast_channel **locked_channel,
 	const struct ast_assigned_ids *assignedids);
 
+int ast_pbx_outgoing_app_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *app, const char *appdata, int *reason, int synchronous,
+	const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee);
+
 /*!
  * \brief Evaluate a condition
  *
@@ -1231,11 +1257,11 @@ int pbx_checkcondition(const char *condi
 const char *ast_get_context_name(struct ast_context *con);
 const char *ast_get_extension_name(struct ast_exten *exten);
 struct ast_context *ast_get_extension_context(struct ast_exten *exten);
-const char *ast_get_include_name(struct ast_include *include);
-const char *ast_get_ignorepat_name(struct ast_ignorepat *ip);
-const char *ast_get_switch_name(struct ast_sw *sw);
-const char *ast_get_switch_data(struct ast_sw *sw);
-int ast_get_switch_eval(struct ast_sw *sw);
+const char *ast_get_include_name(const struct ast_include *include);
+const char *ast_get_ignorepat_name(const struct ast_ignorepat *ip);
+const char *ast_get_switch_name(const struct ast_sw *sw);
+const char *ast_get_switch_data(const struct ast_sw *sw);
+int ast_get_switch_eval(const struct ast_sw *sw);
 
 /*! @} */
 
@@ -1253,11 +1279,28 @@ void *ast_get_extension_app_data(struct
 /*! @{ */
 const char *ast_get_context_registrar(struct ast_context *c);
 const char *ast_get_extension_registrar(struct ast_exten *e);
-const char *ast_get_include_registrar(struct ast_include *i);
-const char *ast_get_ignorepat_registrar(struct ast_ignorepat *ip);
-const char *ast_get_switch_registrar(struct ast_sw *sw);
+const char *ast_get_include_registrar(const struct ast_include *i);
+const char *ast_get_ignorepat_registrar(const struct ast_ignorepat *ip);
+const char *ast_get_switch_registrar(const struct ast_sw *sw);
 /*! @} */
 
+/*!
+ * \brief Get name of configuration file used by registrar to register this extension
+ *
+ * \retval NULL if registrar did not indicate config file when registering the extension
+ * \retval name of the file used to register the extension
+ */
+const char *ast_get_extension_registrar_file(struct ast_exten *e);
+
+/*!
+ * \brief Get line number of configuration file used by registrar to register this extension
+ *
+ * \retval 0 if the line wasn't indicated when the extension was registered
+ * \retval positive integer indicating what line in the config file was responsible for
+ *         registering the extension.
+ */
+int ast_get_extension_registrar_line(struct ast_exten *e);
+
 /*! @name Walking functions ... */
 /*! @{ */
 struct ast_context *ast_walk_contexts(struct ast_context *con);
@@ -1265,11 +1308,22 @@ struct ast_exten *ast_walk_context_exten
 	struct ast_exten *priority);
 struct ast_exten *ast_walk_extension_priorities(struct ast_exten *exten,
 	struct ast_exten *priority);
-struct ast_include *ast_walk_context_includes(struct ast_context *con,
-	struct ast_include *inc);
-struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con,
-	struct ast_ignorepat *ip);
-struct ast_sw *ast_walk_context_switches(struct ast_context *con, struct ast_sw *sw);
+const struct ast_include *ast_walk_context_includes(const struct ast_context *con,
+	const struct ast_include *inc);
+const struct ast_ignorepat *ast_walk_context_ignorepats(const struct ast_context *con,
+	const struct ast_ignorepat *ip);
+const struct ast_sw *ast_walk_context_switches(const struct ast_context *con,
+	const struct ast_sw *sw);
+/*! @} */
+
+/*! @name Iterator functions ... */
+/*! @{ */
+int ast_context_includes_count(const struct ast_context *con);
+const struct ast_include *ast_context_includes_get(const struct ast_context *con, int idx);
+int ast_context_ignorepats_count(const struct ast_context *con);
+const struct ast_ignorepat *ast_context_ignorepats_get(const struct ast_context *con, int idx);
+int ast_context_switches_count(const struct ast_context *con);
+const struct ast_sw *ast_context_switches_get(const struct ast_context *con, int idx);
 /*! @} */
 
 /*!
@@ -1447,7 +1501,7 @@ enum ast_custom_function_escalation {
 /*!
  * \brief Register a custom function
  */
-#define ast_custom_function_register(acf) __ast_custom_function_register(acf, ast_module_info->self)
+#define ast_custom_function_register(acf) __ast_custom_function_register(acf, AST_MODULE_SELF)
 
 /*!
  * \brief Register a custom function which requires escalated privileges.
@@ -1456,7 +1510,7 @@ enum ast_custom_function_escalation {
  * arbitrary code) or FILE() (for which write needs permission to change files
  * on the filesystem).
  */
-#define ast_custom_function_register_escalating(acf, escalation) __ast_custom_function_register_escalating(acf, escalation, ast_module_info->self)
+#define ast_custom_function_register_escalating(acf, escalation) __ast_custom_function_register_escalating(acf, escalation, AST_MODULE_SELF)
 
 /*!
  * \brief Register a custom function
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/_private.h 1:16.1.1~dfsg-1/include/asterisk/_private.h
--- 1:13.23.1~dfsg-2/include/asterisk/_private.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/_private.h	2018-12-26 18:45:13.000000000 +0000
@@ -20,7 +20,7 @@ void load_asterisk_conf(void);
 void set_asterisk_conf_path(const char *path);
 void set_socket_path(const char *path);
 
-int load_modules(unsigned int);		/*!< Provided by loader.c */
+int load_modules(void);		/*!< Provided by loader.c */
 int modules_shutdown(void);		/*!< Provided by loader.c */
 int load_pbx(void);			/*!< Provided by pbx.c */
 int load_pbx_builtins(void);	/*!< Provided by pbx_builtins.c */
@@ -38,32 +38,40 @@ int astdb_init(void);			/*!< Provided by
 int ast_channels_init(void);		/*!< Provided by channel.c */
 void ast_builtins_init(void);		/*!< Provided by cli.c */
 int ast_cli_perms_init(int reload);	/*!< Provided by cli.c */
-int dnsmgr_init(void);			/*!< Provided by dnsmgr.c */
 void dnsmgr_start_refresh(void);	/*!< Provided by dnsmgr.c */
-int dnsmgr_reload(void);		/*!< Provided by dnsmgr.c */
+int ast_dns_system_resolver_init(void); /*!< Provided by dns_system_resolver.c */
 void threadstorage_init(void);		/*!< Provided by threadstorage.c */
 int ast_device_state_engine_init(void);	/*!< Provided by devicestate.c */
 int astobj2_init(void);			/*!< Provided by astobj2.c */
 int ast_named_locks_init(void);		/*!< Provided by named_locks.c */
 int ast_file_init(void);		/*!< Provided by file.c */
-int ast_features_init(void);            /*!< Provided by features.c */
 void ast_autoservice_init(void);	/*!< Provided by autoservice.c */
-int ast_data_init(void);		/*!< Provided by data.c */
-int ast_http_init(void);		/*!< Provided by http.c */
-int ast_http_reload(void);		/*!< Provided by http.c */
 int ast_tps_init(void); 		/*!< Provided by taskprocessor.c */
 int ast_timing_init(void);		/*!< Provided by timing.c */
-int ast_indications_init(void); /*!< Provided by indications.c */
-int ast_indications_reload(void);/*!< Provided by indications.c */
 void ast_stun_init(void);               /*!< Provided by stun.c */
-int ast_cel_engine_init(void);		/*!< Provided by cel.c */
-int ast_cel_engine_reload(void);	/*!< Provided by cel.c */
 int ast_ssl_init(void);                 /*!< Provided by ssl.c */
 int ast_pj_init(void);                 /*!< Provided by libasteriskpj.c */
 int ast_test_init(void);            /*!< Provided by test.c */
 int ast_msg_init(void);             /*!< Provided by message.c */
 void ast_msg_shutdown(void);        /*!< Provided by message.c */
 int aco_init(void);             /*!< Provided by config_options.c */
+int dns_core_init(void);        /*!< Provided by dns_core.c */
+
+/*!
+ * \brief Initialize malloc debug phase 1.
+ *
+ * \note Must be called first thing after forking.
+ *
+ * \return Nothing
+ */
+void load_astmm_phase_1(void);
+
+/*!
+ * \brief Initialize malloc debug phase 2.
+ *
+ * \return Nothing
+ */
+void load_astmm_phase_2(void);
 
 /*!
  * \brief Initialize the bridging system.
@@ -101,13 +109,6 @@ void ast_process_pending_reloads(void);
  */
 int ast_xmldoc_load_documentation(void);
 
-/*!
- * \brief Reload genericplc configuration value from codecs.conf
- *
- * Implementation is in main/channel.c
- */
-int ast_plc_reload(void);
-
 /*! \brief initializes the rtp engine arrays */
 int ast_rtp_engine_init(void);
 
@@ -117,9 +118,6 @@ int ast_rtp_engine_init(void);
  */
 int ast_parking_stasis_init(void);
 
-/*! \brief initialize the sounds index */
-int ast_sounds_index_init(void);
-
 /*!
  * \brief Endpoint support initialization.
  * \return 0 on success.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_mwi_external.h 1:16.1.1~dfsg-1/include/asterisk/res_mwi_external.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_mwi_external.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_mwi_external.h	2018-12-26 18:45:13.000000000 +0000
@@ -36,22 +36,6 @@ extern "C"
 
 /* ------------------------------------------------------------------- */
 
-/*!
- * \brief Increase the external MWI resource module reference count.
- * \since 12.1.0
- *
- * \return Nothing
- */
-void ast_mwi_external_ref(void);
-
-/*!
- * \brief Decrease the external MWI resource module reference count.
- * \since 12.1.0
- *
- * \return Nothing
- */
-void ast_mwi_external_unref(void);
-
 struct ast_mwi_mailbox_object;
 
 /*! \brief Convienience unref function for mailbox object. */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjproject.h 1:16.1.1~dfsg-1/include/asterisk/res_pjproject.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjproject.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjproject.h	2018-12-26 18:45:13.000000000 +0000
@@ -22,13 +22,7 @@
 #include <pj/types.h>
 #include <pj/pool.h>
 
-/*! \brief Determines whether the res_pjproject module is loaded */
-#define CHECK_PJPROJECT_MODULE_LOADED()                 \
-	do {                                                \
-		if (!ast_module_check("res_pjproject.so")) {    \
-			return AST_MODULE_LOAD_DECLINE;             \
-		}                                               \
-	} while(0)
+struct ast_sockaddr;
 
 /*!
  * \brief Retrieve a pjproject build option
@@ -83,20 +77,6 @@ void ast_pjproject_log_intercept_begin(i
 void ast_pjproject_log_intercept_end(void);
 
 /*!
- * \brief Increment the res_pjproject reference count.
- *
- * This ensures graceful shutdown happens in the proper order.
- */
-void ast_pjproject_ref(void);
-
-/*!
- * \brief Decrement the res_pjproject reference count.
- *
- * This ensures graceful shutdown happens in the proper order.
- */
-void ast_pjproject_unref(void);
-
-/*!
  * \brief Initialize the caching pool factory.
  * \since 13.21.0
  *
@@ -119,4 +99,28 @@ void ast_pjproject_caching_pool_init(pj_
  */
 void ast_pjproject_caching_pool_destroy(pj_caching_pool *cp);
 
+/*!
+ * \brief Fill a pj_sockaddr from an ast_sockaddr
+ * \since 13.24.0
+ *
+ * \param addr The source address to copy
+ * \param pjaddr The target address to receive the copied address
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ */
+int ast_sockaddr_to_pj_sockaddr(const struct ast_sockaddr *addr, pj_sockaddr *pjaddr);
+
+/*!
+ * \brief Fill an ast_sockaddr from a pj_sockaddr
+ * \since 13.24.0
+ *
+ * \param addr The target address to receive the copied address
+ * \param pjaddr The source address to copy
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ */
+int ast_sockaddr_from_pj_sockaddr(struct ast_sockaddr *addr, const pj_sockaddr *pjaddr);
+
 #endif /* _RES_PJPROJECT_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_body_generator_types.h 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_body_generator_types.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_body_generator_types.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_body_generator_types.h	2018-12-26 18:45:13.000000000 +0000
@@ -21,6 +21,9 @@
 
 #include "asterisk/pbx.h"
 
+/*! \brief Forward declaration for ao2_container so full astobj2.h is not required */
+struct ao2_container;
+
 /*!
  * \brief structure used for presence XML bodies
  *
@@ -51,6 +54,8 @@ struct ast_sip_exten_state_data {
 	char remote[PJSIP_MAX_URL_SIZE];
 	/*! Optional subscription */
 	struct ast_sip_subscription *sub;
+	/*! A datastores container to persist datastores */
+	struct ao2_container *datastores;
 	/*! Allocation pool */
 	pj_pool_t *pool;
 };
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjsip.h 1:16.1.1~dfsg-1/include/asterisk/res_pjsip.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjsip.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjsip.h	2018-12-26 18:45:13.000000000 +0000
@@ -255,6 +255,16 @@ struct ast_sip_contact {
 		AST_STRING_FIELD(path);
 		/*! Content of the User-Agent header in REGISTER request */
 		AST_STRING_FIELD(user_agent);
+		/*! The name of the aor this contact belongs to */
+		AST_STRING_FIELD(aor);
+		/*! Asterisk Server name */
+		AST_STRING_FIELD(reg_server);
+		/*! IP-address of the Via header in REGISTER request */
+		AST_STRING_FIELD(via_addr);
+		/*! Content of the Call-ID header in REGISTER request */
+		AST_STRING_FIELD(call_id);
+		/*! The name of the endpoint that added the contact */
+		AST_STRING_FIELD(endpoint_name);
 	);
 	/*! Absolute time that this contact is no longer valid after */
 	struct timeval expiration_time;
@@ -266,18 +276,8 @@ struct ast_sip_contact {
 	double qualify_timeout;
 	/*! Endpoint that added the contact, only available in observers */
 	struct ast_sip_endpoint *endpoint;
-	/*! The name of the aor this contact belongs to */
-	char *aor;
-	/*! Asterisk Server name */
-	AST_STRING_FIELD_EXTENDED(reg_server);
-	/*! IP-address of the Via header in REGISTER request */
-	AST_STRING_FIELD_EXTENDED(via_addr);
 	/*! Port of the Via header in REGISTER request */
 	int via_port;
-	/*! Content of the Call-ID header in REGISTER request */
-	AST_STRING_FIELD_EXTENDED(call_id);
-	/*! The name of the endpoint that added the contact */
-	AST_STRING_FIELD_EXTENDED(endpoint_name);
 	/*! If true delete the contact on Asterisk restart/boot */
 	int prune_on_boot;
 };
@@ -286,9 +286,13 @@ struct ast_sip_contact {
  * \brief Status type for a contact.
  */
 enum ast_sip_contact_status_type {
+	/*! Frequency > 0, but no response from remote uri */
 	UNAVAILABLE,
+	/*! Frequency > 0, and got response from remote uri */
 	AVAILABLE,
+	/*! Default last status, and when a contact status object is not found */
 	UNKNOWN,
+	/*! Frequency == 0, has a contact, but don't know status (non-qualified) */
 	CREATED,
 	REMOVED,
 };
@@ -558,6 +562,8 @@ struct ast_sip_endpoint_id_configuration
 	unsigned int send_pai;
 	/*! Do we send Remote-Party-ID headers to this endpoint? */
 	unsigned int send_rpid;
+	/*! Do we send messages for connected line updates for unanswered incoming calls immediately to this endpoint? */
+	unsigned int rpid_immediate;
 	/*! Do we add Diversion headers to applicable outgoing requests/responses? */
 	unsigned int send_diversion;
 	/*! When performing connected line update, which method should be used */
@@ -636,6 +642,10 @@ struct ast_sip_media_rtp_configuration {
 	unsigned int timeout;
 	/*! Number of seconds before terminating channel due to lack of RTP (when on hold) */
 	unsigned int timeout_hold;
+	/*! Follow forked media with a different To tag */
+	unsigned int follow_early_media_fork;
+	/*! Accept updated SDPs on non-100rel 18X and 2XX responses with the same To tag */
+	unsigned int accept_multiple_sdp_answers;
 };
 
 /*!
@@ -685,6 +695,8 @@ struct ast_sip_endpoint_media_configurat
 	struct ast_sip_t38_configuration t38;
 	/*! Configured codecs */
 	struct ast_format_cap *codecs;
+	/*! Capabilities in topology form */
+	struct ast_stream_topology *topology;
 	/*! DSCP TOS bits for audio streams */
 	unsigned int tos_audio;
 	/*! Priority for audio streams */
@@ -697,6 +709,16 @@ struct ast_sip_endpoint_media_configurat
 	unsigned int g726_non_standard;
 	/*! Bind the RTP instance to the media_address */
 	unsigned int bind_rtp_to_media_address;
+	/*! Use RTCP-MUX */
+	unsigned int rtcp_mux;
+	/*! Maximum number of audio streams to offer/accept */
+	unsigned int max_audio_streams;
+	/*! Maximum number of video streams to offer/accept */
+	unsigned int max_video_streams;
+	/*! Use BUNDLE */
+	unsigned int bundle;
+	/*! Enable webrtc settings and defaults */
+	unsigned int webrtc;
 };
 
 /*!
@@ -727,6 +749,8 @@ struct ast_sip_endpoint {
 		AST_STRING_FIELD(message_context);
 		/*! Accountcode to auto-set on channels */
 		AST_STRING_FIELD(accountcode);
+		/*! If set, we'll push incoming MWI NOTIFYs to stasis using this mailbox */
+		AST_STRING_FIELD(incoming_mwi_mailbox);
 	);
 	/*! Configuration for extensions */
 	struct ast_sip_endpoint_extensions extensions;
@@ -768,8 +792,8 @@ struct ast_sip_endpoint {
 	struct ast_variable *channel_vars;
 	/*! Whether to place a 'user=phone' parameter into the request URI if user is a number */
 	unsigned int usereqphone;
-	/*! Do we send messages for connected line updates for unanswered incoming calls immediately to this endpoint? */
-	unsigned int rpid_immediate;
+	/*! Whether to pass through hold and unhold using re-invites with recvonly and sendrecv */
+	unsigned int moh_passthrough;
 	/*! Access control list */
 	struct ast_acl_list *acl;
 	/*! Restrict what IPs are allowed in the Contact header (for registration) */
@@ -778,24 +802,22 @@ struct ast_sip_endpoint {
 	unsigned int faxdetect_timeout;
 	/*! Override the user on the outgoing Contact header with this value. */
 	char *contact_user;
+	/*! Whether to response SDP offer with single most preferred codec. */
+	unsigned int preferred_codec_only;
 	/*! Do we allow an asymmetric RTP codec? */
 	unsigned int asymmetric_rtp_codec;
-	/*! Use RTCP-MUX */
-	unsigned int rtcp_mux;
 	/*! Do we allow overlap dialling? */
 	unsigned int allow_overlap;
 	/*! Whether to notifies all the progress details on blind transfer */
 	unsigned int refer_blind_progress;
 	/*! Whether to notifies dialog-info 'early' on INUSE && RINGING state */
 	unsigned int notify_early_inuse_ringing;
-	/*! If set, we'll push incoming MWI NOTIFYs to stasis using this mailbox */
-	AST_STRING_FIELD_EXTENDED(incoming_mwi_mailbox);
-	/*! Follow forked media with a different To tag */
-	unsigned int follow_early_media_fork;
-	/*! Accept updated SDPs on non-100rel 18X and 2XX responses with the same To tag */
-	unsigned int accept_multiple_sdp_answers;
 	/*! Suppress Q.850 Reason headers on this endpoint */
 	unsigned int suppress_q850_reason_headers;
+	/*! Do we accept connected line updates from this endpoint? */
+	unsigned int trust_connected_line;
+	/*! Do we send connected line updates to this endpoint? */
+	unsigned int send_connected_line;
 };
 
 /*! URI parameter for symmetric transport */
@@ -871,24 +893,12 @@ struct ast_sip_outbound_authenticator {
 	 *
 	 * \param auths A vector of IDs of auth sorcery objects
 	 * \param challenge The SIP response with authentication challenge(s)
-	 * \param tsx The transaction in which the challenge was received
+	 * \param old_request The request that received the auth challenge(s)
 	 * \param new_request The new SIP request with challenge response(s)
 	 * \retval 0 Successfully created new request
 	 * \retval -1 Failed to create a new request
 	 */
 	int (*create_request_with_auth)(const struct ast_sip_auth_vector *auths, struct pjsip_rx_data *challenge,
-			struct pjsip_transaction *tsx, struct pjsip_tx_data **new_request);
-	/*!
-	 * \brief Create a new request with authentication credentials based on old request
-	 *
-	 * \param auths A vector of IDs of auth sorcery objects
-	 * \param challenge The SIP response with authentication challenge(s)
-	 * \param old_request The request that resulted in challenge(s)
-	 * \param new_request The new SIP request with challenge response(s)
-	 * \retval 0 Successfully created new request
-	 * \retval -1 Failed to create a new request
-	 */
-	int (*create_request_with_auth_from_old)(const struct ast_sip_auth_vector *auths, struct pjsip_rx_data *challenge,
 			struct pjsip_tx_data *old_request, struct pjsip_tx_data **new_request);
 };
 
@@ -1468,18 +1478,6 @@ typedef int (*ast_sip_task)(void *user_d
 
 /*!
  * \brief Create a new serializer for SIP tasks
- *
- * See \ref ast_threadpool_serializer for more information on serializers.
- * SIP creates serializers so that tasks operating on similar data will run
- * in sequence.
- *
- * \retval NULL Failure
- * \retval non-NULL Newly-created serializer
- */
-struct ast_taskprocessor *ast_sip_create_serializer(void);
-
-/*!
- * \brief Create a new serializer for SIP tasks
  * \since 13.8.0
  *
  * See \ref ast_threadpool_serializer for more information on serializers.
@@ -1491,27 +1489,12 @@ struct ast_taskprocessor *ast_sip_create
  * \retval NULL Failure
  * \retval non-NULL Newly-created serializer
  */
-struct ast_taskprocessor *ast_sip_create_serializer_named(const char *name);
+struct ast_taskprocessor *ast_sip_create_serializer(const char *name);
 
 struct ast_serializer_shutdown_group;
 
 /*!
  * \brief Create a new serializer for SIP tasks
- * \since 13.5.0
- *
- * See \ref ast_threadpool_serializer for more information on serializers.
- * SIP creates serializers so that tasks operating on similar data will run
- * in sequence.
- *
- * \param shutdown_group Group shutdown controller. (NULL if no group association)
- *
- * \retval NULL Failure
- * \retval non-NULL Newly-created serializer
- */
-struct ast_taskprocessor *ast_sip_create_serializer_group(struct ast_serializer_shutdown_group *shutdown_group);
-
-/*!
- * \brief Create a new serializer for SIP tasks
  * \since 13.8.0
  *
  * See \ref ast_threadpool_serializer for more information on serializers.
@@ -1524,7 +1507,7 @@ struct ast_taskprocessor *ast_sip_create
  * \retval NULL Failure
  * \retval non-NULL Newly-created serializer
  */
-struct ast_taskprocessor *ast_sip_create_serializer_group_named(const char *name, struct ast_serializer_shutdown_group *shutdown_group);
+struct ast_taskprocessor *ast_sip_create_serializer_group(const char *name, struct ast_serializer_shutdown_group *shutdown_group);
 
 /*!
  * \brief Determine the distributor serializer for the SIP message.
@@ -1538,16 +1521,6 @@ struct ast_taskprocessor *ast_sip_create
 struct ast_taskprocessor *ast_sip_get_distributor_serializer(pjsip_rx_data *rdata);
 
 /*!
- * \brief Record the task's serializer name on the tdata structure.
- * \since 13.15.0
- *
- * \param tdata The outgoing message.
- *
- * \retval PJ_SUCCESS.
- */
-pj_status_t ast_sip_record_request_serializer(pjsip_tx_data *tdata);
-
-/*!
  * \brief Set a serializer on a SIP dialog so requests and responses are automatically serialized
  *
  * Passing a NULL serializer is a way to remove a serializer from a dialog.
@@ -2160,18 +2133,7 @@ enum ast_sip_check_auth_result ast_sip_c
  * the parameters and return values.
  */
 int ast_sip_create_request_with_auth(const struct ast_sip_auth_vector *auths, pjsip_rx_data *challenge,
-		pjsip_transaction *tsx, pjsip_tx_data **new_request);
-
-/*!
- * \brief Create a response to an authentication challenge
- *
- * This will call into an outbound authenticator's create_request_with_auth callback
- * to create a new request with authentication credentials. See the create_request_with_auth_from_old
- * callback in the \ref ast_sip_outbound_authenticator structure for details about
- * the parameters and return values.
- */
-int ast_sip_create_request_with_auth_from_old(const struct ast_sip_auth_vector *auths, pjsip_rx_data *challenge,
-		pjsip_tx_data *old_request, pjsip_tx_data **new_request);
+		pjsip_tx_data *tdata, pjsip_tx_data **new_request);
 
 /*!
  * \brief Determine the endpoint that has sent a SIP message
@@ -2264,6 +2226,24 @@ int ast_sip_append_body(pjsip_tx_data *t
 void ast_copy_pj_str(char *dest, const pj_str_t *src, size_t size);
 
 /*!
+ * \brief Create and copy a pj_str_t into a standard character buffer.
+ *
+ * pj_str_t is not NULL-terminated. Any place that expects a NULL-
+ * terminated string needs to have the pj_str_t copied into a separate
+ * buffer.
+ *
+ * Copies the pj_str_t contents into a newly allocated buffer pointed to
+ * by dest. NULL-terminates the buffer.
+ *
+ * \note Caller is responsible for freeing the allocated memory.
+ *
+ * \param dest [out] The destination buffer
+ * \param src The pj_str_t to copy
+ * \retval Number of characters copied or negative value on error
+ */
+int ast_copy_pj_str2(char **dest, const pj_str_t *src);
+
+/*!
  * \brief Get the looked-up endpoint on an out-of dialog request or response
  *
  * The function may ONLY be called on out-of-dialog requests or responses. For
@@ -2559,10 +2539,8 @@ struct ast_sip_endpoint_formatter {
  * \brief Register an endpoint formatter.
  *
  * \param obj the formatter to register
- * \retval 0 Success
- * \retval -1 Failure
  */
-int ast_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
+void ast_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
 
 /*!
  * \brief Unregister an endpoint formatter.
@@ -2738,7 +2716,7 @@ struct ast_sip_supplement {
  * \retval 0 Success
  * \retval -1 Failure
  */
-int ast_sip_register_supplement(struct ast_sip_supplement *supplement);
+void ast_sip_register_supplement(struct ast_sip_supplement *supplement);
 
 /*!
  * \brief Unregister a an supplement to SIP out of dialog processing
@@ -2774,6 +2752,14 @@ int ast_sip_get_mwi_tps_queue_low(void);
 unsigned int ast_sip_get_mwi_disable_initial_unsolicited(void);
 
 /*!
+ * \brief Retrieve the global setting 'use_callerid_contact'.
+ * \since 13.24.0
+ *
+ * \retval non zero if CALLERID(num) is to be used as the default username in the contact
+ */
+unsigned int ast_sip_get_use_callerid_contact(void);
+
+/*!
  * \brief Retrieve the global setting 'ignore_uri_user_options'.
  * \since 13.12.0
  *
@@ -2869,15 +2855,6 @@ void ast_sip_get_default_realm(char *rea
  */
 void ast_sip_get_default_from_user(char *from_user, size_t size);
 
-/*! \brief Determines whether the res_pjsip module is loaded */
-#define CHECK_PJSIP_MODULE_LOADED()				\
-	do {							\
-		if (!ast_module_check("res_pjsip.so")		\
-			|| !ast_sip_get_pjsip_endpoint()) {	\
-			return AST_MODULE_LOAD_DECLINE;		\
-		}						\
-	} while(0)
-
 /*!
  * \brief Retrieve the system keep alive interval setting.
  *
@@ -2917,6 +2894,15 @@ const char *ast_sip_get_contact_status_l
 const char *ast_sip_get_contact_short_status_label(const enum ast_sip_contact_status_type status);
 
 /*!
+ * \brief Set a request to use the next value in the list of resolved addresses.
+ *
+ * \param tdata the tx data from the original request
+ * \retval 0 No more addresses to try
+ * \retval 1 The request was successfully re-intialized
+ */
+int ast_sip_failover_request(pjsip_tx_data *tdata);
+
+/*!
  * \brief Retrieve the local host address in IP form
  *
  * \param af The address family to retrieve
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_outbound_publish.h 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_outbound_publish.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_outbound_publish.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_outbound_publish.h	2018-12-26 18:45:13.000000000 +0000
@@ -93,6 +93,58 @@ void ast_sip_unregister_event_publisher_
 struct ast_sip_outbound_publish_client *ast_sip_publish_client_get(const char *name);
 
 /*!
+ * \brief Get the From URI the client will use.
+ * \since 14.0.0
+ *
+ * \param client The publication client to get the From URI
+ *
+ * \retval From-uri on success
+ * \retval Empty-string on failure
+ */
+const char *ast_sip_publish_client_get_from_uri(struct ast_sip_outbound_publish_client *client);
+
+/*!
+ * \brief Get the From URI the client will use for a specific user.
+ * \since 14.0.0
+ *
+ * \param client The publication client to get the From URI of a user
+ * \param user The user to retrieve the From URI for
+ * \param uri A buffer to place the URI into
+ * \param size The size of the buffer
+ *
+ * \retval From-uri on success
+ * \retval Empty-string on failure
+ */
+const char *ast_sip_publish_client_get_user_from_uri(struct ast_sip_outbound_publish_client *client, const char *user,
+	char *uri, size_t size);
+
+/*!
+ * \brief Get the To URI the client will use.
+ * \since 14.0.0
+ *
+ * \param client The publication client to get the To URI
+ *
+ * \retval From-uri on success
+ * \retval Empty-string on failure
+ */
+const char *ast_sip_publish_client_get_to_uri(struct ast_sip_outbound_publish_client *client);
+
+/*!
+ * \brief Get the To URI the client will use for a specific user.
+ * \since 14.0.0
+ *
+ * \param client The publication client to get the To URI of a user
+ * \param user The user to retrieve the To URI for
+ * \param uri A buffer to place the URI into
+ * \param size The size of the buffer
+ *
+ * \retval To-uri on success
+ * \retval Empty-string on failure
+ */
+const char *ast_sip_publish_client_get_user_to_uri(struct ast_sip_outbound_publish_client *client, const char *user,
+	char *uri, size_t size);
+
+/*!
  * \brief Alternative for ast_datastore_alloc()
  *
  * There are two major differences between this and ast_datastore_alloc()
@@ -162,4 +214,26 @@ void ast_sip_publish_client_remove_datas
 int ast_sip_publish_client_send(struct ast_sip_outbound_publish_client *client,
 	const struct ast_sip_body *body);
 
+/*!
+* \brief Send an outgoing PUBLISH message based on the user
+*
+* \param client The publication client to send from
+* \param user The user to send to
+* \param body An optional body to add to the PUBLISH
+*
+* \retval -1 failure
+* \retval 0 success
+*/
+int ast_sip_publish_client_user_send(struct ast_sip_outbound_publish_client *client,
+	const char *user, const struct ast_sip_body *body);
+
+/*!
+* \brief Remove the user from the client (stopping it from publishing)
+*
+* \param client The publication client
+* \param user The user to remove
+*/
+void ast_sip_publish_client_remove(struct ast_sip_outbound_publish_client *client,
+	const char *user);
+
 #endif /* RES_PJSIP_OUTBOUND_PUBLISH_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_pubsub.h 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_pubsub.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_pubsub.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_pubsub.h	2018-12-26 18:45:13.000000000 +0000
@@ -166,6 +166,20 @@ struct ast_datastore *ast_sip_publicatio
 void ast_sip_publication_remove_datastore(struct ast_sip_publication *publication, const char *name);
 
 /*!
+ * \brief Get the datastores container for a publication
+ *
+ * \param publication The publication to get the datastores container from
+ *
+ * \retval NULL datastores container not present
+ * \retval non-NULL datastores container
+ *
+ * \note The container is NOT returned with reference count bumped
+ *
+ * \since 14.0.0
+ */
+struct ao2_container *ast_sip_publication_get_datastores(const struct ast_sip_publication *publication);
+
+/*!
  * \brief Opaque structure representing an RFC 3265 SIP subscription
  */
 struct ast_sip_subscription;
@@ -518,6 +532,20 @@ struct ast_datastore *ast_sip_subscripti
 void ast_sip_subscription_remove_datastore(struct ast_sip_subscription *subscription, const char *name);
 
 /*!
+ * \brief Get the datastores container for a subscription
+ *
+ * \param subscription The subscription to get the datastores container from
+ *
+ * \retval NULL datastores container not present
+ * \retval non-NULL datastores container
+ *
+ * \note The container is NOT returned with reference count bumped
+ *
+ * \since 14.0.0
+ */
+struct ao2_container *ast_sip_subscription_get_datastores(const struct ast_sip_subscription *subscription);
+
+/*!
  * \brief Register a subscription handler
  *
  * \retval 0 Handler was registered successfully
@@ -649,6 +677,19 @@ int ast_sip_pubsub_generate_body_content
 		const char *content_subtype, struct ast_sip_body_data *data, struct ast_str **str);
 
 /*!
+ * \brief Is a body generator registered for the given type/subtype.
+ * \since 14.0.0
+ *
+ * \param type The content type of the body
+ * \param subtype The content subtype of the body
+ *
+ * \note In "plain/text", "plain" is the type and "text" is the subtype.
+ *
+ * \retval non-zero if a generator is registered.
+ */
+int ast_sip_pubsub_is_body_generator_registered(const char *type, const char *subtype);
+
+/*!
  * \since 13.0.0
  * \brief Register a body generator with the pubsub core.
  *
@@ -713,13 +754,4 @@ const char *ast_sip_subscription_get_bod
  */
 void ast_sip_subscription_destroy(struct ast_sip_subscription *sub);
 
-/*! \brief Determines whether the res_pjsip_pubsub module is loaded */
-#define CHECK_PJSIP_PUBSUB_MODULE_LOADED()			\
-	do {							\
-		CHECK_PJSIP_MODULE_LOADED();			\
-		if (!ast_module_check("res_pjsip_pubsub.so")) {	\
-			return AST_MODULE_LOAD_DECLINE;		\
-		}						\
-	} while(0)
-
 #endif /* RES_PJSIP_PUBSUB_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_session.h 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_session.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_pjsip_session.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_pjsip_session.h	2018-12-26 18:45:13.000000000 +0000
@@ -28,6 +28,8 @@
 #include "asterisk/netsock2.h"
 /* Needed for ast_sdp_srtp struct */
 #include "asterisk/sdp_srtp.h"
+/* Needed for ast_media_type */
+#include "asterisk/codec.h"
 
 /* Forward declarations */
 struct ast_sip_endpoint;
@@ -56,17 +58,21 @@ enum ast_sip_session_t38state {
 };
 
 struct ast_sip_session_sdp_handler;
+struct ast_sip_session;
+struct ast_sip_session_media;
+
+typedef struct ast_frame *(*ast_sip_session_media_read_cb)(struct ast_sip_session *session, struct ast_sip_session_media *session_media);
+typedef int (*ast_sip_session_media_write_cb)(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	struct ast_frame *frame);
 
 /*!
  * \brief A structure containing SIP session media information
  */
 struct ast_sip_session_media {
-	union {
-		/*! \brief RTP instance itself */
-		struct ast_rtp_instance *rtp;
-		/*! \brief UDPTL instance itself */
-		struct ast_udptl *udptl;
-	};
+	/*! \brief RTP instance itself */
+	struct ast_rtp_instance *rtp;
+	/*! \brief UDPTL instance itself */
+	struct ast_udptl *udptl;
 	/*! \brief Direct media address */
 	struct ast_sockaddr direct_media_addr;
 	/*! \brief SDP handler that setup the RTP */
@@ -81,14 +87,60 @@ struct ast_sip_session_media {
 	int keepalive_sched_id;
 	/*! \brief Scheduler ID for RTP timeout */
 	int timeout_sched_id;
-	/*! \brief Stream is on hold */
-	unsigned int held:1;
+	/*! \brief Stream is on hold by remote side */
+	unsigned int remotely_held:1;
+	/*! \brief Stream is on hold by local side */
+	unsigned int locally_held:1;
 	/*! \brief Does remote support rtcp_mux */
 	unsigned int remote_rtcp_mux:1;
 	/*! \brief Does remote support ice */
 	unsigned int remote_ice:1;
-	/*! \brief Stream type this session media handles */
-	char stream_type[1];
+	/*! \brief Media type of this session media */
+	enum ast_media_type type;
+	/*! \brief The write callback when writing frames */
+	ast_sip_session_media_write_cb write_callback;
+	/*! \brief The stream number to place into any resulting frames */
+	int stream_num;
+	/*! \brief Media identifier for this stream (may be shared across multiple streams) */
+	char *mid;
+	/*! \brief The bundle group the stream belongs to */
+	int bundle_group;
+	/*! \brief Whether this stream is currently bundled or not */
+	unsigned int bundled;
+	/*! \brief Media stream label */
+	char mslabel[AST_UUID_STR_LEN];
+	/*! \brief Track label */
+	char label[AST_UUID_STR_LEN];
+	/*! \brief The underlying session has been changed in some fashion */
+	unsigned int changed;
+	/*! \brief Remote media stream label */
+	char *remote_mslabel;
+};
+
+/*!
+ * \brief Structure which contains read callback information
+ */
+struct ast_sip_session_media_read_callback_state {
+	/*! \brief The file descriptor itself */
+	int fd;
+	/*! \brief The callback to invoke */
+	ast_sip_session_media_read_cb read_callback;
+	/*! \brief The media session */
+	struct ast_sip_session_media *session;
+};
+
+/*!
+ * \brief Structure which contains media state information (streams, sessions)
+ */
+struct ast_sip_session_media_state {
+	/*! \brief Mapping of stream to media sessions */
+	AST_VECTOR(, struct ast_sip_session_media *) sessions;
+	/*! \brief Added read callbacks - these are whole structs and not pointers */
+	AST_VECTOR(, struct ast_sip_session_media_read_callback_state) read_callbacks;
+	/*! \brief Default media sessions for each type */
+	struct ast_sip_session_media *default_session[AST_MEDIA_TYPE_END];
+	/*! \brief The media stream topology */
+	struct ast_stream_topology *topology;
 };
 
 /*!
@@ -123,8 +175,6 @@ struct ast_sip_session {
 	AST_LIST_HEAD(, ast_sip_session_supplement) supplements;
 	/*! Datastores added to the session by supplements to the session */
 	struct ao2_container *datastores;
-	/*! Media streams */
-	struct ao2_container *media;
 	/*! Serializer for tasks relating to this SIP session */
 	struct ast_taskprocessor *serializer;
 	/*! Non-null if the session serializer is suspended or being suspended. */
@@ -139,8 +189,10 @@ struct ast_sip_session {
 	pj_timer_entry scheduled_termination;
 	/*! Identity of endpoint this session deals with */
 	struct ast_party_id id;
-	/*! Requested capabilities */
-	struct ast_format_cap *req_caps;
+	/*! Active media state (sessions + streams) - contents are guaranteed not to change */
+	struct ast_sip_session_media_state *active_media_state;
+	/*! Pending media state (sessions + streams) */
+	struct ast_sip_session_media_state *pending_media_state;
 	/*! Optional DSP, used only for inband DTMF/Fax-CNG detection if configured */
 	struct ast_dsp *dsp;
 	/*! Whether the termination of the session should be deferred */
@@ -319,34 +371,29 @@ struct ast_sip_session_sdp_handler {
 	/*!
 	 * \brief Set session details based on a stream in an incoming SDP offer or answer
 	 * \param session The session for which the media is being negotiated
-	 * \param session_media The media to be setup for this session
+	 * \param session_media The media session
 	 * \param sdp The entire SDP. Useful for getting "global" information, such as connections or attributes
-	 * \param stream The stream on which to operate
-	 * \retval 0 The stream was not handled by this handler. If there are other registered handlers for this stream type, they will be called.
-	 * \retval <0 There was an error encountered. No further operation will take place and the current negotiation will be abandoned.
-	 * \retval >0 The stream was handled by this handler. No further handler of this stream type will be called.
-	 */
-	int (*negotiate_incoming_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *sdp, const struct pjmedia_sdp_media *stream);
-	/*!
-	 * \brief Create an SDP media stream and add it to the outgoing SDP offer or answer
-	 * \param session The session for which media is being added
-	 * \param session_media The media to be setup for this session
-	 * \param stream The stream on which to operate
+	 * \param index The index for the session media, Asterisk stream, and PJMEDIA stream being negotiated
+	 * \param asterisk_stream The Asterisk stream representation
 	 * \retval 0 The stream was not handled by this handler. If there are other registered handlers for this stream type, they will be called.
 	 * \retval <0 There was an error encountered. No further operation will take place and the current negotiation will be abandoned.
 	 * \retval >0 The stream was handled by this handler. No further handler of this stream type will be called.
 	 */
-	int (*handle_incoming_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *sdp, struct pjmedia_sdp_media *stream);
+	int (*negotiate_incoming_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+		const struct pjmedia_sdp_session *sdp, int index, struct ast_stream *asterisk_stream);
 	/*!
 	 * \brief Create an SDP media stream and add it to the outgoing SDP offer or answer
 	 * \param session The session for which media is being added
 	 * \param session_media The media to be setup for this session
 	 * \param sdp The entire SDP as currently built
+	 * \param remote Optional remote SDP if this is an answer
+	 * \param stream The stream that is to be added to the outgoing SDP
 	 * \retval 0 This handler has no stream to add. If there are other registered handlers for this stream type, they will be called.
 	 * \retval <0 There was an error encountered. No further operation will take place and the current SDP negotiation will be abandoned.
 	 * \retval >0 The handler has a stream to be added to the SDP. No further handler of this stream type will be called.
 	 */
-	int (*create_outgoing_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media, struct pjmedia_sdp_session *sdp);
+	int (*create_outgoing_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media, struct pjmedia_sdp_session *sdp,
+		const struct pjmedia_sdp_session *remote, struct ast_stream *stream);
 	/*!
 	 * \brief Update media stream with external address if applicable
 	 * \param tdata The outgoing message itself
@@ -357,17 +404,18 @@ struct ast_sip_session_sdp_handler {
 	/*!
 	 * \brief Apply a negotiated SDP media stream
 	 * \param session The session for which media is being applied
-	 * \param session_media The media to be setup for this session
+	 * \param session_media The media session
 	 * \param local The entire local negotiated SDP
-	 * \param local_stream The local stream which to apply
 	 * \param remote The entire remote negotiated SDP
-	 * \param remote_stream The remote stream which to apply
+	 * \param index The index of the session media, SDP streams, and Asterisk streams
+	 * \param asterisk_stream The Asterisk stream representation
 	 * \retval 0 The stream was not applied by this handler. If there are other registered handlers for this stream type, they will be called.
 	 * \retval <0 There was an error encountered. No further operation will take place and the current application will be abandoned.
 	 * \retval >0 The stream was handled by this handler. No further handler of this stream type will be called.
 	 */
-	int (*apply_negotiated_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *local, const struct pjmedia_sdp_media *local_stream,
-		const struct pjmedia_sdp_session *remote, const struct pjmedia_sdp_media *remote_stream);
+	int (*apply_negotiated_sdp_stream)(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+		const struct pjmedia_sdp_session *local, const struct pjmedia_sdp_session *remote, int index,
+		struct ast_stream *asterisk_stream);
 	/*!
 	 * \brief Stop a session_media created by this handler but do not destroy resources
 	 * \param session The session for which media is being stopped
@@ -397,7 +445,7 @@ struct ast_sip_channel_pvt {
 /*!
  * \brief Allocate a new SIP channel pvt structure
  *
- * \param pvt Pointer to channel specific implementation
+ * \param pvt Pointer to channel specific information
  * \param session Pointer to SIP session
  *
  * \retval non-NULL success
@@ -456,11 +504,11 @@ void ast_sip_session_unsuspend(struct as
  * \param contact The contact that this session will communicate with
  * \param location Name of the location to call, be it named location or explicit URI. Overrides contact if present.
  * \param request_user Optional request user to place in the request URI if permitted
- * \param req_caps The requested capabilities
+ * \param req_topology The requested capabilities
  */
 struct ast_sip_session *ast_sip_session_create_outgoing(struct ast_sip_endpoint *endpoint,
 	struct ast_sip_contact *contact, const char *location, const char *request_user,
-	struct ast_format_cap *req_caps);
+	struct ast_stream_topology *req_topology);
 
 /*!
  * \brief Terminate a session and, if possible, send the provided response code
@@ -533,10 +581,8 @@ void ast_sip_session_unregister_sdp_hand
  * a module could reject an incoming request if desired.
  *
  * \param supplement The supplement to register
- * \retval 0 Success
- * \retval -1 Failure
  */
-int ast_sip_session_register_supplement(struct ast_sip_session_supplement *supplement);
+void ast_sip_session_register_supplement(struct ast_sip_session_supplement *supplement);
 
 /*!
  * \brief Unregister a an supplement to SIP session processing
@@ -624,15 +670,20 @@ void ast_sip_session_remove_datastore(st
  * \param on_response Callback called when response for request is received
  * \param method The method that should be used when constructing the session refresh
  * \param generate_new_sdp Boolean to indicate if a new SDP should be created
+ * \param media_state Optional requested media state for the SDP
+ *
  * \retval 0 Successfully sent refresh
  * \retval -1 Failure to send refresh
+ *
+ * \note If a media_state is passed in ownership will be taken in all cases
  */
 int ast_sip_session_refresh(struct ast_sip_session *session,
 		ast_sip_session_request_creation_cb on_request_creation,
 		ast_sip_session_sdp_creation_cb on_sdp_creation,
 		ast_sip_session_response_cb on_response,
 		enum ast_sip_session_refresh_method method,
-		int generate_new_sdp);
+		int generate_new_sdp,
+		struct ast_sip_session_media_state *media_state);
 
 /*!
  * \brief Regenerate SDP Answer
@@ -720,13 +771,121 @@ struct ast_sip_session *ast_sip_dialog_g
  */
 void ast_sip_session_resume_reinvite(struct ast_sip_session *session);
 
-/*! \brief Determines whether the res_pjsip_session module is loaded */
-#define CHECK_PJSIP_SESSION_MODULE_LOADED()				\
-	do {								\
-		CHECK_PJSIP_MODULE_LOADED();				\
-		if (!ast_module_check("res_pjsip_session.so")) {	\
-			return AST_MODULE_LOAD_DECLINE;			\
-		}							\
-	} while(0)
+/*!
+ * \brief Determines if a provided pending stream will be the default stream or not
+ * \since 15.0.0
+ *
+ * \param session The session to check against
+ * \param stream The pending stream
+ *
+ * \retval 1 if stream will be default
+ * \retval 0 if stream will NOT be the default
+ */
+int ast_sip_session_is_pending_stream_default(const struct ast_sip_session *session, const struct ast_stream *stream);
+
+/*!
+ * \brief Allocate a session media state structure
+ * \since 15.0.0
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_sip_session_media_state *ast_sip_session_media_state_alloc(void);
+
+/*!
+ * \brief Allocate an ast_session_media and add it to the media state's vector.
+ * \since 15.0.0
+ *
+ * This allocates a session media of the specified type. The position argument
+ * determines where in the vector that the new session media will be inserted.
+ *
+ * \note The returned ast_session_media is the reference held by the vector. Callers
+ * of this function must NOT decrement the refcount of the session media.
+ *
+ * \param session Session on which to query active media state for
+ * \param media_state Media state to place the session media into
+ * \param type The type of the session media
+ * \param position Position at which to insert the new session media.
+ *
+ * \note The active media state will be queried and if a media session already
+ * exists at the given position for the same type it will be reused instead of
+ * allocating a new one.
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_sip_session_media *ast_sip_session_media_state_add(struct ast_sip_session *session,
+	struct ast_sip_session_media_state *media_state, enum ast_media_type type, int position);
+
+/*!
+ * \brief Reset a media state to a clean state
+ * \since 15.0.0
+ *
+ * \param media_state The media state to reset
+ */
+void ast_sip_session_media_state_reset(struct ast_sip_session_media_state *media_state);
+
+/*!
+ * \brief Clone a media state
+ * \since 15.0.0
+ *
+ * \param media_state The media state to clone
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+struct ast_sip_session_media_state *ast_sip_session_media_state_clone(const struct ast_sip_session_media_state *media_state);
+
+/*!
+ * \brief Free a session media state structure
+ * \since 15.0.0
+ */
+void ast_sip_session_media_state_free(struct ast_sip_session_media_state *media_state);
+
+/*!
+ * \brief Set a read callback for a media session with a specific file descriptor
+ * \since 15.0.0
+ *
+ * \param session The session
+ * \param session_media The media session
+ * \param fd The file descriptor
+ * \param callback The read callback
+ *
+ * \retval 0 the read callback was successfully added
+ * \retval -1 the read callback could not be added
+ *
+ * \note This operations on the pending media state
+ */
+int ast_sip_session_media_add_read_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	int fd, ast_sip_session_media_read_cb callback);
+
+/*!
+ * \brief Set a write callback for a media session
+ * \since 15.0.0
+ *
+ * \param session The session
+ * \param session_media The media session
+ * \param callback The write callback
+ *
+ * \retval 0 the write callback was successfully add
+ * \retval -1 the write callback is already set to something different
+ *
+ * \note This operates on the pending media state
+ */
+int ast_sip_session_media_set_write_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	ast_sip_session_media_write_cb callback);
+
+/*!
+ * \brief Retrieve the underlying media session that is acting as transport for a media session
+ * \since 15.0.0
+ *
+ * \param session The session
+ * \param session_media The media session to retrieve the transport for
+ *
+ * \note This operates on the pending media state
+ *
+ * \note This function is guaranteed to return non-NULL
+ */
+struct ast_sip_session_media *ast_sip_session_media_get_transport(struct ast_sip_session *session, struct ast_sip_session_media *session_media);
 
 #endif /* _RES_PJSIP_SESSION_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/res_srtp.h 1:16.1.1~dfsg-1/include/asterisk/res_srtp.h
--- 1:13.23.1~dfsg-2/include/asterisk/res_srtp.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/res_srtp.h	2018-12-26 18:45:13.000000000 +0000
@@ -21,6 +21,8 @@
 #ifndef _ASTERISK_RES_SRTP_H
 #define _ASTERISK_RES_SRTP_H
 
+#include "asterisk.h"                   /* for size_t */
+
 struct ast_srtp;
 struct ast_srtp_policy;
 struct ast_rtp_instance;
@@ -52,9 +54,17 @@ struct ast_srtp_res {
 
 /* Crypto suites */
 enum ast_srtp_suite {
+	/* https://www.iana.org/assignments/srtp-protection/srtp-protection.xhtml */
 	AST_AES_CM_128_HMAC_SHA1_80 = 1,
-	AST_AES_CM_128_HMAC_SHA1_32 = 2,
-	AST_F8_128_HMAC_SHA1_80     = 3
+	AST_AES_CM_128_HMAC_SHA1_32,
+	AST_AES_CM_256_HMAC_SHA1_80,
+	AST_AES_CM_256_HMAC_SHA1_32,
+	AST_AES_GCM_128             = 7,
+	AST_AES_GCM_256,
+	AST_AES_GCM_128_8,
+	AST_AES_GCM_256_8,
+	AST_AES_CM_192_HMAC_SHA1_80,
+	AST_AES_CM_192_HMAC_SHA1_32,
 };
 
 struct ast_srtp_policy_res {
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/rtp_engine.h 1:16.1.1~dfsg-1/include/asterisk/rtp_engine.h
--- 1:13.23.1~dfsg-2/include/asterisk/rtp_engine.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/rtp_engine.h	2018-12-26 18:45:13.000000000 +0000
@@ -81,6 +81,12 @@ extern "C" {
 /*! Maximum number of payload types RTP can support. */
 #define AST_RTP_MAX_PT 128
 
+/*!
+ * Last RTP payload type statically assigned, see
+ * http://www.iana.org/assignments/rtp-parameters
+ */
+#define AST_RTP_PT_LAST_STATIC 34
+
 /*! First dynamic RTP payload type */
 #define AST_RTP_PT_FIRST_DYNAMIC 96
 
@@ -116,6 +122,12 @@ enum ast_rtp_property {
 	AST_RTP_PROPERTY_RTCP,
 	/*! Enable Asymmetric RTP Codecs */
 	AST_RTP_PROPERTY_ASYMMETRIC_CODEC,
+	/*! Enable packet retransmission for received packets */
+	AST_RTP_PROPERTY_RETRANS_RECV,
+	/*! Enable packet retransmission for sent packets */
+	AST_RTP_PROPERTY_RETRANS_SEND,
+	/*! Enable REMB sending and receiving passthrough support */
+	AST_RTP_PROPERTY_REMB,
 
 	/*!
 	 * \brief Maximum number of RTP properties supported
@@ -260,15 +272,19 @@ enum ast_rtp_instance_rtcp {
 
 /*! Structure that represents a payload */
 struct ast_rtp_payload_type {
-	/*! Is this an Asterisk value */
-	int asterisk_format;
 	/*! If asterisk_format is set, this is the internal
 	 * asterisk format represented by the payload */
 	struct ast_format *format;
+	/*! Is this an Asterisk value */
+	int asterisk_format;
 	/*! Actual internal RTP specific value of the payload */
 	int rtp_code;
 	/*! Actual payload number */
 	int payload;
+	/*! TRUE if this is the primary mapping to the format. */
+	unsigned int primary_mapping:1;
+	/*! When the payload type became non-primary. */
+	struct timeval when_retired;
 };
 
 /* Common RTCP report types */
@@ -276,6 +292,20 @@ struct ast_rtp_payload_type {
 #define AST_RTP_RTCP_SR 200
 /*! Receiver Report */
 #define AST_RTP_RTCP_RR 201
+/*! Transport Layer Feed Back (From RFC4585 also RFC5104) */
+#define AST_RTP_RTCP_RTPFB	205
+/*! Payload Specific Feed Back (From RFC4585 also RFC5104) */
+#define AST_RTP_RTCP_PSFB	206
+
+/* Common RTCP feedback message types */
+/*! Generic NACK (From RFC4585 also RFC5104) */
+#define AST_RTP_RTCP_FMT_NACK	1
+/*! Picture loss indication (From RFC4585) */
+#define AST_RTP_RTCP_FMT_PLI	1
+/*! Full INTRA-frame Request (From RFC5104) */
+#define AST_RTP_RTCP_FMT_FIR	4
+/*! REMB Information (From draft-alvestrand-rmcat-remb-03) */
+#define AST_RTP_RTCP_FMT_REMB	15
 
 /*!
  * \since 12
@@ -311,6 +341,24 @@ struct ast_rtp_rtcp_report {
 	struct ast_rtp_rtcp_report_block *report_block[0];
 };
 
+/*!
+ * \since 15.4.0
+ * \brief A REMB feedback message (see draft-alvestrand-rmcat-remb-03 for details) */
+struct ast_rtp_rtcp_feedback_remb {
+	unsigned int br_exp;		/*!< Exponential scaling of the mantissa for the maximum total media bit rate value */
+	unsigned int br_mantissa;	/*!< The mantissa of the maximum total media bit rate */
+};
+
+/*!
+ * \since 15.4.0
+ * \brief An object that represents data received in a feedback report */
+struct ast_rtp_rtcp_feedback {
+	unsigned int fmt; /*!< The feedback message type */
+	union {
+		struct ast_rtp_rtcp_feedback_remb remb; /*!< REMB feedback information */
+	};
+};
+
 /*! Structure that represents statistics from an RTP instance */
 struct ast_rtp_instance_stats {
 	/*! Number of packets transmitted */
@@ -485,6 +533,18 @@ enum ast_rtp_dtls_verify {
 	AST_RTP_DTLS_VERIFY_CERTIFICATE = (1 << 1), /*!< Verify the certificate */
 };
 
+/*!
+ * \brief Known RTP extensions
+ */
+enum ast_rtp_extension {
+	/*! Per the RFC 0 should not be used, so we treat it as an unsupported extension placeholder */
+	AST_RTP_EXTENSION_UNSUPPORTED = 0,
+	/*! abs-send-time from https://tools.ietf.org/html/draft-alvestrand-rmcat-remb-03 */
+	AST_RTP_EXTENSION_ABS_SEND_TIME,
+	/*! The maximum number of known RTP extensions */
+	AST_RTP_EXTENSION_MAX,
+};
+
 /*! \brief DTLS configuration structure */
 struct ast_rtp_dtls_cfg {
 	unsigned int enabled:1;                /*!< Whether DTLS support is enabled or not */
@@ -498,6 +558,7 @@ struct ast_rtp_dtls_cfg {
 	char *cipher;                          /*!< Cipher to use */
 	char *cafile;                          /*!< Certificate authority file */
 	char *capath;                          /*!< Path to certificate authority */
+	unsigned int ephemeral_cert:1;         /*!< Whether to not to generate an ephemeral certificate - defaults to 0 (off) */
 };
 
 /*! \brief Structure that represents the optional DTLS SRTP support within an RTP engine */
@@ -591,26 +652,40 @@ struct ast_rtp_engine {
 	void (*available_formats)(struct ast_rtp_instance *instance, struct ast_format_cap *to_endpoint, struct ast_format_cap *to_asterisk, struct ast_format_cap *result);
 	/*! Callback to send CNG */
 	int (*sendcng)(struct ast_rtp_instance *instance, int level);
+	/*! Callback to retrieve local SSRC */
+	unsigned int (*ssrc_get)(struct ast_rtp_instance *instance);
+	/*! Callback to retrieve RTCP SDES CNAME */
+	const char *(*cname_get)(struct ast_rtp_instance *instance);
+	/*! Callback to bundle an RTP instance to another */
+	int (*bundle)(struct ast_rtp_instance *child, struct ast_rtp_instance *parent);
+	/*! Callback to set remote SSRC information */
+	void (*set_remote_ssrc)(struct ast_rtp_instance *instance, unsigned int ssrc);
+	/*! Callback to set the stream identifier */
+	void (*set_stream_num)(struct ast_rtp_instance *instance, int stream_num);
 	/*! Callback to pointer for optional ICE support */
 	struct ast_rtp_engine_ice *ice;
 	/*! Callback to pointer for optional DTLS SRTP support */
 	struct ast_rtp_engine_dtls *dtls;
+	/*! Callback to enable an RTP extension (returns non-zero if supported) */
+	int (*extension_enable)(struct ast_rtp_instance *instance, enum ast_rtp_extension extension);
 	/*! Linked list information */
 	AST_RWLIST_ENTRY(ast_rtp_engine) entry;
 };
 
 /*! Structure that represents codec and packetization information */
 struct ast_rtp_codecs {
-	/*! Payloads present */
-	AST_VECTOR(, struct ast_rtp_payload_type *) payloads;
-	/*! The framing for this media session */
-	unsigned int framing;
 	/*! RW lock that protects elements in this structure */
 	ast_rwlock_t codecs_lock;
+	/*! Rx payload type mapping exceptions */
+	AST_VECTOR(, struct ast_rtp_payload_type *) payload_mapping_rx;
+	/*! Tx payload type mapping */
+	AST_VECTOR(, struct ast_rtp_payload_type *) payload_mapping_tx;
+	/*! The framing for this media session */
+	unsigned int framing;
 };
 
 #define AST_RTP_CODECS_NULL_INIT \
-    { .payloads = { 0, }, .framing = 0, .codecs_lock = AST_RWLOCK_INIT_VALUE, }
+    { .codecs_lock = AST_RWLOCK_INIT_VALUE, .payload_mapping_rx = { 0, }, .payload_mapping_tx = { 0, }, .framing = 0, }
 
 /*! Structure that represents the glue that binds an RTP instance to a channel */
 struct ast_rtp_glue {
@@ -660,6 +735,22 @@ struct ast_rtp_glue {
 };
 
 /*!
+ * \brief Directions for RTP extensions
+ */
+enum ast_rtp_extension_direction {
+	/*! The extension is not negotiated and is not flowing */
+	AST_RTP_EXTENSION_DIRECTION_NONE,
+	/*! Send and receive */
+	AST_RTP_EXTENSION_DIRECTION_SENDRECV,
+	/*! Send only */
+	AST_RTP_EXTENSION_DIRECTION_SENDONLY,
+	/*! Receive only */
+	AST_RTP_EXTENSION_DIRECTION_RECVONLY,
+	/*! Negotiated but not sending or receiving */
+	AST_RTP_EXTENSION_DIRECTION_INACTIVE,
+};
+
+/*!
  * \brief Allocation routine for \ref ast_rtp_payload_type
  *
  * \retval NULL on error
@@ -671,7 +762,7 @@ struct ast_rtp_glue {
  */
 struct ast_rtp_payload_type *ast_rtp_engine_alloc_payload_type(void);
 
-#define ast_rtp_engine_register(engine) ast_rtp_engine_register2(engine, ast_module_info->self)
+#define ast_rtp_engine_register(engine) ast_rtp_engine_register2(engine, AST_MODULE_SELF)
 
 /*!
  * \brief Register an RTP engine
@@ -724,7 +815,7 @@ int ast_rtp_engine_register_srtp(struct
 void ast_rtp_engine_unregister_srtp(void);
 int ast_rtp_engine_srtp_is_registered(void);
 
-#define ast_rtp_glue_register(glue) ast_rtp_glue_register2(glue, ast_module_info->self)
+#define ast_rtp_glue_register(glue) ast_rtp_glue_register2(glue, AST_MODULE_SELF)
 
 /*!
  * \brief Register RTP glue
@@ -1186,6 +1277,109 @@ int ast_rtp_instance_get_prop(struct ast
 struct ast_rtp_codecs *ast_rtp_instance_get_codecs(struct ast_rtp_instance *instance);
 
 /*!
+ * \brief Enable support for an RTP extension on an instance
+ *
+ * \param instance The RTP instance to enable the extension on
+ * \param id The unique local identifier to use for this extension (-1 to have one auto selected)
+ * \param extension The RTP extension
+ * \param direction The initial direction that the RTP extension should be used in
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \since 15.5.0
+ */
+int ast_rtp_instance_extmap_enable(struct ast_rtp_instance *instance, int id, enum ast_rtp_extension extension,
+	enum ast_rtp_extension_direction direction);
+
+/*!
+ * \brief Negotiate received RTP extension information
+ *
+ * \param instance The RTP instance to set the extension on
+ * \param id The local identifier for the extension
+ * \param direction The direction that the extension should be used in
+ * \param uri The unique URI for the extension
+ * \param attributes Attributes specific to this extension (if NULL or empty then no attributes)
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \since 15.5.0
+ */
+int ast_rtp_instance_extmap_negotiate(struct ast_rtp_instance *instance, int id, enum ast_rtp_extension_direction direction,
+	const char *uri, const char *attributes);
+
+/*!
+ * \brief Clear negotiated RTP extension information
+ *
+ * \param instance The RTP instance to clear negotiated extension information on
+ */
+void ast_rtp_instance_extmap_clear(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Retrieve the id for an RTP extension
+ *
+ * \param instance The RTP instance to retrieve the id from
+ * \param extension The RTP extension
+ *
+ * \retval -1 not negotiated
+ * \retval id if negotiated
+ *
+ * \since 15.5.0
+ */
+int ast_rtp_instance_extmap_get_id(struct ast_rtp_instance *instance, enum ast_rtp_extension extension);
+
+/*!
+ * \brief Get the number of known unique identifiers
+ *
+ * \param instance The RTP instance to retrieve the count from
+ *
+ * \return the number of known unique identifiers
+ *
+ * \since 15.5.0
+ */
+size_t ast_rtp_instance_extmap_count(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Retrieve the extension for an RTP extension id
+ *
+ * \param instance The RTP instance to retrieve the extension from
+ * \param id The negotiated RTP extension id
+ *
+ * \retval extension the extension that maps to the id
+ *
+ * \since 15.5.0
+ *
+ * \note This will return AST_RTP_EXTENSION_UNSUPPORTED if an extension was proposed for this unique identifier
+ * but it is not supported or if the unique identifier is unused.
+ */
+enum ast_rtp_extension ast_rtp_instance_extmap_get_extension(struct ast_rtp_instance *instance, int id);
+
+/*!
+ * \brief Retrieve the negotiated direction for an RTP extension id
+ *
+ * \param instance The RTP instance to retrieve the direction from
+ * \param id The negotiated RTP extension id
+ *
+ * \retval direction the direction that has been negotiated
+ *
+ * \since 15.5.0
+ */
+enum ast_rtp_extension_direction ast_rtp_instance_extmap_get_direction(struct ast_rtp_instance *instance, int id);
+
+/*!
+ * \brief Retrieve the URI for an RTP extension id
+ *
+ * \param instance The RTP instance to retrieve the direction from
+ * \param id The negotiated RTP extension id
+ *
+ * \retval uri The URI for the RTP extension
+ *
+ * \since 15.5.0
+ */
+const char *ast_rtp_instance_extmap_get_uri(struct ast_rtp_instance *instance, int id);
+
+/*!
  * \brief Initialize an RTP codecs structure
  *
  * \param codecs The codecs structure to initialize
@@ -1221,7 +1415,7 @@ int ast_rtp_codecs_payloads_initialize(s
 void ast_rtp_codecs_payloads_destroy(struct ast_rtp_codecs *codecs);
 
 /*!
- * \brief Clear payload information from an RTP instance
+ * \brief Clear rx and tx payload mapping information from an RTP instance
  *
  * \param codecs The codecs structure that payloads will be cleared from
  * \param instance Optionally the instance that the codecs structure belongs to
@@ -1259,7 +1453,19 @@ void ast_rtp_codecs_payloads_clear(struc
 void ast_rtp_codecs_payloads_copy(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance);
 
 /*!
- * \brief Record payload information that was seen in an m= SDP line
+ * \brief Crossover copy the tx payload mapping of src to the rx payload mapping of dest.
+ * \since 14.0.0
+ *
+ * \param src The source codecs structure
+ * \param dest The destination codecs structure that the values from src will be copied to
+ * \param instance Optionally the instance that the dst codecs structure belongs to
+ *
+ * \return Nothing
+ */
+void ast_rtp_codecs_payloads_xover(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Record tx payload type information that was seen in an m= SDP line
  *
  * \param codecs The codecs structure to muck with
  * \param instance Optionally the instance that the codecs structure belongs to
@@ -1278,7 +1484,7 @@ void ast_rtp_codecs_payloads_copy(struct
 void ast_rtp_codecs_payloads_set_m_type(struct ast_rtp_codecs *codecs, struct ast_rtp_instance *instance, int payload);
 
 /*!
- * \brief Record payload information that was seen in an a=rtpmap: SDP line
+ * \brief Record tx payload type information that was seen in an a=rtpmap: SDP line
  *
  * \param codecs The codecs structure to muck with
  * \param instance Optionally the instance that the codecs structure belongs to
@@ -1304,7 +1510,7 @@ void ast_rtp_codecs_payloads_set_m_type(
 int ast_rtp_codecs_payloads_set_rtpmap_type(struct ast_rtp_codecs *codecs, struct ast_rtp_instance *instance, int payload, char *mimetype, char *mimesubtype, enum ast_rtp_options options);
 
 /*!
- * \brief Set payload type to a known MIME media type for a codec with a specific sample rate
+ * \brief Set tx payload type to a known MIME media type for a codec with a specific sample rate
  *
  * \param codecs RTP structure to modify
  * \param instance Optionally the instance that the codecs structure belongs to
@@ -1329,7 +1535,7 @@ int ast_rtp_codecs_payloads_set_rtpmap_t
 				  unsigned int sample_rate);
 
 /*!
- * \brief Remove payload information
+ * \brief Remove tx payload type mapped information
  *
  * \param codecs The codecs structure to muck with
  * \param instance Optionally the instance that the codecs structure belongs to
@@ -1358,7 +1564,7 @@ void ast_rtp_codecs_payloads_unset(struc
 enum ast_media_type ast_rtp_codecs_get_stream_type(struct ast_rtp_codecs *codecs);
 
 /*!
- * \brief Retrieve payload information by payload
+ * \brief Retrieve rx payload mapped information by payload type
  *
  * \param codecs Codecs structure to look in
  * \param payload Numerical payload to look up
@@ -1381,10 +1587,10 @@ enum ast_media_type ast_rtp_codecs_get_s
 struct ast_rtp_payload_type *ast_rtp_codecs_get_payload(struct ast_rtp_codecs *codecs, int payload);
 
 /*!
- * \brief Update the format associated with a payload in a codecs structure
+ * \brief Update the format associated with a tx payload type in a codecs structure
  *
  * \param codecs Codecs structure to operate on
- * \param payload Numerical payload to look up
+ * \param payload Numerical payload type to look up
  * \param format The format to replace the existing one
  *
  * \retval 0 success
@@ -1395,10 +1601,10 @@ struct ast_rtp_payload_type *ast_rtp_cod
 int ast_rtp_codecs_payload_replace_format(struct ast_rtp_codecs *codecs, int payload, struct ast_format *format);
 
 /*!
- * \brief Retrieve the actual ast_format stored on the codecs structure for a specific payload
+ * \brief Retrieve the actual ast_format stored on the codecs structure for a specific tx payload type
  *
  * \param codecs Codecs structure to look in
- * \param payload Numerical payload to look up
+ * \param payload Numerical payload type to look up
  *
  * \retval pointer to format structure on success
  * \retval NULL on failure
@@ -1442,7 +1648,8 @@ unsigned int ast_rtp_codecs_get_framing(
  *
  * \since 1.8
  */
-unsigned int ast_rtp_lookup_sample_rate2(int asterisk_format, struct ast_format *format, int code);
+unsigned int ast_rtp_lookup_sample_rate2(int asterisk_format,
+	const struct ast_format *format, int code);
 
 /*!
  * \brief Retrieve all formats that were found
@@ -1467,14 +1674,20 @@ unsigned int ast_rtp_lookup_sample_rate2
 void ast_rtp_codecs_payload_formats(struct ast_rtp_codecs *codecs, struct ast_format_cap *astformats, int *nonastformats);
 
 /*!
- * \brief Retrieve a payload based on whether it is an Asterisk format and the code
+ * \brief Retrieve a rx mapped payload type based on whether it is an Asterisk format and the code
  *
  * \param codecs Codecs structure to look in
  * \param asterisk_format Non-zero if the given Asterisk format is present
  * \param format Asterisk format to look for
  * \param code The format to look for
  *
- * \retval Numerical payload
+ * \details
+ * Find the currently assigned rx mapped payload type based on whether it
+ * is an Asterisk format or non-format code.  If one is currently not
+ * assigned then create a rx payload type mapping.
+ *
+ * \retval Numerical payload type
+ * \retval -1 if could not assign.
  *
  * Example usage:
  *
@@ -1486,15 +1699,43 @@ void ast_rtp_codecs_payload_formats(stru
  *
  * \since 1.8
  */
-int ast_rtp_codecs_payload_code(struct ast_rtp_codecs *codecs, int asterisk_format, const struct ast_format *format, int code);
+int ast_rtp_codecs_payload_code(struct ast_rtp_codecs *codecs, int asterisk_format, struct ast_format *format, int code);
+
+/*!
+ * \brief Set a payload code for use with a specific Asterisk format
+ *
+ * \param codecs Codecs structure to manipulate
+ * \param code The payload code
+ * \param format Asterisk format
+ *
+ * \retval 0 Payload was set to the given format
+ * \retval -1 Payload was in use or could not be set
+ *
+ * \since 15.0.0
+ */
+int ast_rtp_codecs_payload_set_rx(struct ast_rtp_codecs *codecs, int code, struct ast_format *format);
 
 /*!
- * \brief Search for a payload code in the ast_rtp_codecs structure
+ * \brief Retrieve a tx mapped payload type based on whether it is an Asterisk format and the code
+ * \since 14.0.0
  *
  * \param codecs Codecs structure to look in
+ * \param asterisk_format Non-zero if the given Asterisk format is present
+ * \param format Asterisk format to look for
  * \param code The format to look for
  *
- * \retval Numerical payload or -1 if unable to find payload in codecs
+ * \retval Numerical payload type
+ * \retval -1 if not found.
+ */
+int ast_rtp_codecs_payload_code_tx(struct ast_rtp_codecs *codecs, int asterisk_format, const struct ast_format *format, int code);
+
+/*!
+ * \brief Search for the tx payload type in the ast_rtp_codecs structure
+ *
+ * \param codecs Codecs structure to look in
+ * \param payload The payload type format to look for
+ *
+ * \retval Numerical payload type or -1 if unable to find payload in codecs
  *
  * Example usage:
  *
@@ -1503,9 +1744,8 @@ int ast_rtp_codecs_payload_code(struct a
  * \endcode
  *
  * This looks for the numerical payload for ULAW in the codecs structure.
- *
  */
-int ast_rtp_codecs_find_payload_code(struct ast_rtp_codecs *codecs, int code);
+int ast_rtp_codecs_find_payload_code(struct ast_rtp_codecs *codecs, int payload);
 
 /*!
  * \brief Retrieve mime subtype information on a payload
@@ -1528,7 +1768,8 @@ int ast_rtp_codecs_find_payload_code(str
  *
  * \since 1.8
  */
-const char *ast_rtp_lookup_mime_subtype2(const int asterisk_format, struct ast_format *format, int code, enum ast_rtp_options options);
+const char *ast_rtp_lookup_mime_subtype2(const int asterisk_format,
+	const struct ast_format *format, int code, enum ast_rtp_options options);
 
 /*!
  * \brief Convert formats into a string and put them into a buffer
@@ -2234,6 +2475,8 @@ int ast_rtp_instance_sendcng(struct ast_
  *
  * \retval 0 Success
  * \retval non-zero Failure
+ *
+ * \note If no remote policy is provided any existing SRTP policies are left and the new local policy is added
  */
 int ast_rtp_instance_add_srtp_policy(struct ast_rtp_instance *instance, struct ast_srtp_policy* remote_policy, struct ast_srtp_policy *local_policy, int rtcp);
 
@@ -2289,6 +2532,16 @@ struct ast_rtp_engine_dtls *ast_rtp_inst
 int ast_rtp_dtls_cfg_parse(struct ast_rtp_dtls_cfg *dtls_cfg, const char *name, const char *value);
 
 /*!
+ * \brief Validates DTLS related configuration options
+ *
+ * \param dtls_cfg a DTLS configuration structure
+ *
+ * \retval 0 if valid
+ * \retval -1 if invalid
+ */
+int ast_rtp_dtls_cfg_validate(struct ast_rtp_dtls_cfg *dtls_cfg);
+
+/*!
  * \brief Copy contents of a DTLS configuration structure
  *
  * \param src_cfg source DTLS configuration structure
@@ -2361,6 +2614,54 @@ time_t ast_rtp_instance_get_last_rx(cons
  */
 void ast_rtp_instance_set_last_rx(struct ast_rtp_instance *rtp, time_t time);
 
+/*!
+ * \brief Retrieve the local SSRC value that we will be using
+ *
+ * \param rtp The RTP instance
+ * \return The SSRC value
+ */
+unsigned int ast_rtp_instance_get_ssrc(struct ast_rtp_instance *rtp);
+
+/*!
+ * \brief Retrieve the CNAME used in RTCP SDES items
+ *
+ * This is a pointer directly into the RTP struct, not a copy.
+ *
+ * \param rtp The RTP instance
+ * \return the CNAME
+ */
+const char *ast_rtp_instance_get_cname(struct ast_rtp_instance *rtp);
+
+/*!
+ * \brief Request that an RTP instance be bundled with another
+ * \since 15.0.0
+ *
+ * \param child The child RTP instance
+ * \param parent The parent RTP instance the child should be bundled with
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+int ast_rtp_instance_bundle(struct ast_rtp_instance *child, struct ast_rtp_instance *parent);
+
+/*!
+ * \brief Set the remote SSRC for an RTP instance
+ * \since 15.0.0
+ *
+ * \param rtp The RTP instance
+ * \param ssrc The remote SSRC
+ */
+void ast_rtp_instance_set_remote_ssrc(struct ast_rtp_instance *rtp, unsigned int ssrc);
+
+/*!
+ * \brief Set the stream number for an RTP instance
+ * \since 15.0.0
+ *
+ * \param rtp The RTP instance
+ * \param stream_num The stream identifier number
+ */
+void ast_rtp_instance_set_stream_num(struct ast_rtp_instance *instance, int stream_num);
+
 /*! \addtogroup StasisTopicsAndMessages
  * @{
  */
@@ -2389,7 +2690,7 @@ struct stasis_message_type *ast_rtp_rtcp
  */
 struct stasis_topic *ast_rtp_topic(void);
 
-/* }@ */
+/* @} */
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/sdp_srtp.h 1:16.1.1~dfsg-1/include/asterisk/sdp_srtp.h
--- 1:13.23.1~dfsg-2/include/asterisk/sdp_srtp.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/sdp_srtp.h	2018-12-26 18:45:13.000000000 +0000
@@ -20,8 +20,7 @@
  *
  * \brief SRTP and SDP Security descriptions
  *
- * Specified in RFC 4568
- * Specified in RFC 3711
+ * Specified in RFC 3711, 6188, 7714, and 4568
  *
  * \author Mikael Magnusson <mikma@users.sourceforge.net>
  */
@@ -29,7 +28,8 @@
 #ifndef _SDP_SRTP_H
 #define _SDP_SRTP_H
 
-#include <asterisk/rtp_engine.h>
+#include "asterisk/linkedlists.h"       /* for AST_LIST_ENTRY */
+#include "asterisk/rtp_engine.h"        /* for ast_rtp_instance */
 
 struct ast_sdp_crypto;
 
@@ -37,12 +37,18 @@ struct ast_sdp_crypto;
 struct ast_sdp_srtp {
 	unsigned int flags;
 	struct ast_sdp_crypto *crypto;
+	AST_LIST_ENTRY(ast_sdp_srtp) sdp_srtp_list;
 };
 
 /* SRTP flags */
-#define AST_SRTP_CRYPTO_OFFER_OK	(1 << 1)
-#define AST_SRTP_CRYPTO_TAG_32		(1 << 2)
-#define AST_SRTP_CRYPTO_TAG_80		(1 << 3)
+#define AST_SRTP_CRYPTO_OFFER_OK (1 << 1)
+#define AST_SRTP_CRYPTO_TAG_32   (1 << 2)
+#define AST_SRTP_CRYPTO_TAG_80   (1 << 3)
+#define AST_SRTP_CRYPTO_TAG_16   (1 << 4)
+#define AST_SRTP_CRYPTO_TAG_8    (1 << 5)
+#define AST_SRTP_CRYPTO_AES_192  (1 << 6)
+#define AST_SRTP_CRYPTO_AES_256  (1 << 7)
+#define AST_SRTP_CRYPTO_OLD_NAME (1 << 8)
 
 /*!
  * \brief allocate a ast_sdp_srtp structure
@@ -57,6 +63,100 @@ struct ast_sdp_srtp *ast_sdp_srtp_alloc(
 */
 void ast_sdp_srtp_destroy(struct ast_sdp_srtp *srtp);
 
+/*! \brief Destroy a previously allocated ast_sdp_crypto struct */
+typedef void (*sdp_crypto_destroy_cb)(struct ast_sdp_crypto *crypto);
+
+/*!
+ * \brief Initialize and return an ast_sdp_crypto struct
+ *
+ * \details
+ * This function allocates a new ast_sdp_crypto struct and initializes its values
+ *
+ * \retval NULL on failure
+ * \retval a pointer to a  new ast_sdp_crypto structure
+ */
+typedef struct ast_sdp_crypto *(*sdp_crypto_alloc_cb)(void);
+
+/*!
+ * \brief Generate an SRTP a=crypto offer
+ *
+ * \details
+ * The offer is stored on the ast_sdp_crypto struct in a_crypto
+ *
+ * \param crypto A valid ast_sdp_crypto struct
+ * \param taglen Length
+ *
+ * \retval 0 success
+ * \retval nonzero failure
+ */
+typedef int (*sdp_crypto_build_offer_cb)(struct ast_sdp_crypto *crypto, int taglen);
+
+/*!
+ * \brief Parse the a=crypto line from SDP and set appropriate values on the
+ * ast_sdp_crypto struct.
+ *
+ * The attribute line should already have "a=crypto:" removed.
+ *
+ * \param p A valid ast_sdp_crypto struct
+ * \param attr the a:crypto line from SDP
+ * \param rtp The rtp instance associated with the SDP being parsed
+ * \param srtp SRTP structure
+ *
+ * \retval 0 success
+ * \retval nonzero failure
+ */
+typedef int (*sdp_crypto_parse_offer_cb)(struct ast_rtp_instance *rtp, struct ast_sdp_srtp *srtp, const char *attr);
+
+/*!
+ * \brief Get the crypto attribute line for the srtp structure
+ *
+ * \details
+ * The attribute line does not contain the initial "a=crypto:" and does
+ * not terminate with "\r\n".
+ *
+ * \param srtp The ast_sdp_srtp structure for which to get an attribute line
+ * \param dtls_enabled Whether this connection is encrypted with datagram TLS
+ * \param default_taglen_32 Whether to default to a tag length of 32 instead of 80
+ *
+ * \retval An attribute line containing cryptographic information
+ * \retval NULL if the srtp structure does not require an attribute line containing crypto information
+ */
+typedef const char *(*sdp_srtp_get_attr_cb)(struct ast_sdp_srtp *srtp, int dtls_enabled, int default_taglen_32);
+
+struct ast_sdp_crypto_api {
+	/*! Destroy a crypto struct */
+	sdp_crypto_destroy_cb dtor;
+	/*! Allocate a crypto struct */
+	sdp_crypto_alloc_cb alloc;
+	/*! Build a SDP a=crypto offer line parameter string */
+	sdp_crypto_build_offer_cb build_offer;
+	/*! Parse a SDP a=crypto offer line parameter string */
+	sdp_crypto_parse_offer_cb parse_offer;
+	/*! Get the SDP a=crypto offer line parameter string */
+	sdp_srtp_get_attr_cb get_attr;
+};
+
+/*!
+ * \brief Register SDP SRTP crypto processing routines.
+ * \since 14.0.0
+ *
+ * \param api Callbacks to register.
+ *
+ * \retval 0 on success.
+ * \retval -1 on error.
+ */
+int ast_sdp_crypto_register(struct ast_sdp_crypto_api *api);
+
+/*!
+ * \brief Unregister SDP SRTP crypto processing routines.
+ * \since 14.0.0
+ *
+ * \param api Callbacks to unregister.
+ *
+ * \return Nothing
+ */
+void ast_sdp_crypto_unregister(struct ast_sdp_crypto_api *api);
+
 /*! \brief Initialize an return an ast_sdp_crypto struct
  *
  * \details
@@ -98,7 +198,6 @@ int ast_sdp_crypto_process(struct ast_rt
  */
 int ast_sdp_crypto_build_offer(struct ast_sdp_crypto *p, int taglen);
 
-
 /*! \brief Get the crypto attribute line for the srtp structure
  *
  * The attribute line does not contain the initial "a=crypto:" and does
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/sem.h 1:16.1.1~dfsg-1/include/asterisk/sem.h
--- 1:13.23.1~dfsg-2/include/asterisk/sem.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/sem.h	2018-12-26 18:45:13.000000000 +0000
@@ -20,7 +20,9 @@
 #define ASTERISK_SEMAPHORE_H
 
 /*!
- * \file Asterisk semaphore API
+ * \file
+ *
+ * \brief Asterisk semaphore API
  *
  * This API is a thin wrapper around the POSIX semaphore API (when available),
  * so see the POSIX documentation for further details.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/sorcery.h 1:16.1.1~dfsg-1/include/asterisk/sorcery.h
--- 1:13.23.1~dfsg-2/include/asterisk/sorcery.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/sorcery.h	2018-12-26 18:45:13.000000000 +0000
@@ -298,6 +298,14 @@ struct ast_sorcery_wizard {
 	/*! \brief Callback for retrieving multiple objects using a regex on their id */
 	void (*retrieve_regex)(const struct ast_sorcery *sorcery, void *data, const char *type, struct ao2_container *objects, const char *regex);
 
+	/*! \brief Optional callback for retrieving multiple objects by matching their id with a prefix */
+	void (*retrieve_prefix)(const struct ast_sorcery *sorcery,
+			void *data,
+			const char *type,
+			struct ao2_container *objects,
+			const char *prefix,
+			const size_t prefix_len);
+
 	/*! \brief Optional callback for retrieving an object using fields */
 	void *(*retrieve_fields)(const struct ast_sorcery *sorcery, void *data, const char *type, const struct ast_variable *fields);
 
@@ -313,13 +321,8 @@ struct ast_sorcery_wizard {
 	/*! \brief Callback for closing a wizard */
 	void (*close)(void *data);
 
-	/*! \brief Optional callback for retrieving multiple objects by matching their id with a prefix */
-	void (*retrieve_prefix)(const struct ast_sorcery *sorcery,
-			void *data,
-			const char *type,
-			struct ao2_container *objects,
-			const char *prefix,
-			const size_t prefix_len);
+	/* \brief Callback for whether or not the wizard believes the object is stale */
+	int (*is_stale)(const struct ast_sorcery *sorcery, void *data, void *object);
 };
 
 /*! \brief Interface for a sorcery object type observer */
@@ -372,20 +375,9 @@ int ast_sorcery_init(void);
 int __ast_sorcery_wizard_register(const struct ast_sorcery_wizard *interface, struct ast_module *module);
 
 /*!
- * \brief Register a sorcery wizard
- *
- * \param interface Pointer to a wizard interface
- * \param module Pointer to the module implementing the interface
- *
- * \retval 0 success
- * \retval -1 failure
- */
-int __ast_sorcery_wizard_register_with_prefix(const struct ast_sorcery_wizard *interface, struct ast_module *module);
-
-/*!
  * \brief See \ref __ast_sorcery_wizard_register()
  */
-#define ast_sorcery_wizard_register(interface) __ast_sorcery_wizard_register_with_prefix(interface, ast_module_info ? ast_module_info->self : NULL)
+#define ast_sorcery_wizard_register(interface) __ast_sorcery_wizard_register(interface, AST_MODULE_SELF)
 
 /*!
  * \brief Unregister a sorcery wizard
@@ -408,9 +400,9 @@ int ast_sorcery_wizard_unregister(const
  * \retval non-NULL success
  * \retval NULL if allocation failed
  */
-struct ast_sorcery *__ast_sorcery_open(const char *module);
+struct ast_sorcery *__ast_sorcery_open(const char *module, const char *file, int line, const char *func);
 
-#define ast_sorcery_open() __ast_sorcery_open(AST_MODULE)
+#define ast_sorcery_open() __ast_sorcery_open(AST_MODULE, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief Retrieves an existing sorcery instance by module name
@@ -1004,10 +996,30 @@ int ast_sorcery_changeset_create(const s
  *
  * \retval non-NULL success
  * \retval NULL failure
+ *
+ * \note The returned object does not support AO2 locking.
  */
 void *ast_sorcery_generic_alloc(size_t size, ao2_destructor_fn destructor);
 
 /*!
+ * \since 14.1.0
+ * \brief Allocate a generic sorcery capable object with locking.
+ *
+ * \details Sorcery objects may be replaced with new allocations during reloads.
+ * If locking is required on sorcery objects it must be shared between the old
+ * object and the new one.  lockobj can be any AO2 object with locking enabled,
+ * but in most cases named locks should be used to provide stable locking.
+ *
+ * \param size Size of the object
+ * \param destructor Optional destructor function
+ * \param lockobj An AO2 object that will provide locking.
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ */
+void *ast_sorcery_lockable_alloc(size_t size, ao2_destructor_fn destructor, void *lockobj);
+
+/*!
  * \brief Allocate an object
  *
  * \param sorcery Pointer to a sorcery structure
@@ -1275,11 +1287,30 @@ int ast_sorcery_update(const struct ast_
 int ast_sorcery_delete(const struct ast_sorcery *sorcery, void *object);
 
 /*!
+ * \brief Determine if a sorcery object is stale with respect to its backing datastore
+ * \since 14.0.0
+ *
+ * This function will query the wizard(s) backing the particular sorcery object to
+ * determine if the in-memory object is now stale. No action is taken to update
+ * the object. Callers of this function may use one of the ast_sorcery_retrieve
+ * functions to obtain a new instance of the object if desired.
+ *
+ * \retval 0 the object is not stale
+ * \retval 1 the object is stale
+ */
+int ast_sorcery_is_stale(const struct ast_sorcery *sorcery, void *object);
+
+/*!
  * \brief Decrease the reference count of a sorcery structure
  *
  * \param sorcery Pointer to a sorcery structure
+ *
+ * \note Prior to 16.0.0 this was a function which had to be used.
+ *       Now you can use any variant of ao2_cleanup or ao2_ref to
+ *       release a reference.
  */
-void ast_sorcery_unref(struct ast_sorcery *sorcery);
+#define ast_sorcery_unref(sorcery) \
+	ao2_cleanup(sorcery)
 
 /*!
  * \brief Get the unique identifier of a sorcery object
@@ -1291,6 +1322,16 @@ void ast_sorcery_unref(struct ast_sorcer
 const char *ast_sorcery_object_get_id(const void *object);
 
 /*!
+ * \since 14.0.0
+ * \brief Get when the socery object was created
+ *
+ * \param object Pointer to a sorcery object
+ *
+ * \retval The time when the object was created
+ */
+const struct timeval ast_sorcery_object_get_created(const void *object);
+
+/*!
  * \brief Get the type of a sorcery object
  *
  * \param object Pointer to a sorcery object
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/sounds_index.h 1:16.1.1~dfsg-1/include/asterisk/sounds_index.h
--- 1:13.23.1~dfsg-2/include/asterisk/sounds_index.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/sounds_index.h	2018-12-26 18:45:13.000000000 +0000
@@ -33,14 +33,6 @@ extern "C" {
 struct ast_media_index;
 
 /*!
- * \brief Reload the sounds index
- *
- * \retval zero on success
- * \retval non-zero on failure
- */
-int ast_sounds_reindex(void);
-
-/*!
  * \brief Get the sounds index
  *
  * \retval sounds index (must be ao2_cleanup()'ed)
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_app.h 1:16.1.1~dfsg-1/include/asterisk/stasis_app.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_app.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_app.h	2018-12-26 18:45:13.000000000 +0000
@@ -46,8 +46,8 @@
  * functions.
  *
  * Since module unload order is based on reference counting, any module that
- * uses the API defined in this file must call stasis_app_ref() when loaded,
- * and stasis_app_unref() when unloaded.
+ * uses the API defined in this file must list "res_stasis" in the requires
+ * field.
  */
 
 #include "asterisk/channel.h"
@@ -229,18 +229,6 @@ void stasis_app_register_event_source(st
 void stasis_app_register_event_sources(void);
 
 /*!
- * \brief Checks to see if the given object is a core event source
- *
- * \note core event sources are currently only endpoint, bridge, and channel.
- *
- * \param obj event source object to check
- *
- * \return non-zero if core event source, otherwise 0 (false)
-
- */
-int stasis_app_is_core_event_source(struct stasis_app_event_source *obj);
-
-/*!
  * \brief Unregister an application event source.
  *
  * \param obj the event source to unregister
@@ -470,23 +458,6 @@ const char *stasis_app_control_get_chann
 	const struct stasis_app_control *control);
 
 /*!
- * \brief Dial an endpoint and bridge it to a channel in \c res_stasis
- *
- * If the channel is no longer in \c res_stasis, this function does nothing.
- *
- * \param control Control for \c res_stasis
- * \param endpoint The endpoint to dial.
- * \param exten Extension to dial if no endpoint specified.
- * \param context Context to use with extension.
- * \param timeout The amount of time to wait for answer, before giving up.
- *
- * \return 0 for success
- * \return -1 for error.
- */
-int stasis_app_control_dial(struct stasis_app_control *control, const char *endpoint, const char *exten,
-                            const char *context, int timeout);
-
-/*!
  * \brief Apply a bridge role to a channel controlled by a stasis app control
  *
  * \param control Control for \c res_stasis
@@ -699,6 +670,18 @@ int stasis_app_control_queue_control(str
 struct ast_bridge *stasis_app_bridge_create(const char *type, const char *name, const char *id);
 
 /*!
+ * \brief Create an invisible bridge of the specified type.
+ *
+ * \param type The type of bridge to be created
+ * \param name Optional name to give to the bridge
+ * \param id Optional Unique ID to give to the bridge
+ *
+ * \return New bridge.
+ * \return \c NULL on error.
+ */
+struct ast_bridge *stasis_app_bridge_create_invisible(const char *type, const char *name, const char *id);
+
+/*!
  * \brief Returns the bridge with the given id.
  * \param bridge_id Uniqueid of the bridge.
  *
@@ -798,6 +781,38 @@ int stasis_app_control_remove_channel_fr
 	struct stasis_app_control *control, struct ast_bridge *bridge);
 
 /*!
+ * \brief Initialize bridge features into a channel control
+ *
+ * \note Bridge features on a control are destroyed after each bridge session,
+ *       so new features need to be initialized before each bridge add.
+ *
+ * \param control Control in which to store the features
+ *
+ * \return non-zero on failure
+ * \return zero on success
+ */
+int stasis_app_control_bridge_features_init(
+	struct stasis_app_control *control);
+
+/*!
+ * \brief Set whether DTMF from the channel is absorbed instead of passing through to the bridge
+ *
+ * \param control Control whose channel should have its DTMF absorbed when bridged
+ * \param absorb Whether DTMF should be absorbed (1) instead of passed through (0).
+ */
+void stasis_app_control_absorb_dtmf_in_bridge(
+	struct stasis_app_control *control, int absorb);
+
+/*!
+ * \brief Set whether audio from the channel is muted instead of passing through to the bridge
+ *
+ * \param control Control whose channel should have its audio muted when bridged
+ * \param mute Whether audio should be muted (1) instead of passed through (0).
+ */
+void stasis_app_control_mute_in_bridge(
+	struct stasis_app_control *control, int mute);
+
+/*!
  * \since 12
  * \brief Gets the bridge currently associated with a control object.
  *
@@ -823,20 +838,6 @@ struct ast_bridge *stasis_app_get_bridge
 void stasis_app_bridge_destroy(const char *bridge_id);
 
 /*!
- * \brief Increment the res_stasis reference count.
- *
- * This ensures graceful shutdown happens in the proper order.
- */
-void stasis_app_ref(void);
-
-/*!
- * \brief Decrement the res_stasis reference count.
- *
- * This ensures graceful shutdown happens in the proper order.
- */
-void stasis_app_unref(void);
-
-/*!
  * \brief Get the Stasis message sanitizer for app_stasis applications
  *
  * \retval The stasis message sanitizer
@@ -891,6 +892,24 @@ int stasis_app_channel_unreal_set_intern
 int stasis_app_channel_set_internal(struct ast_channel *chan);
 
 /*!
+ * \brief Dial a channel
+ * \param control Control for \c res_stasis.
+ * \param dialstring The dialstring to pass to the channel driver
+ * \param timeout Optional timeout in milliseconds
+ */
+int stasis_app_control_dial(struct stasis_app_control *control,
+		const char *dialstring, unsigned int timeout);
+
+/*!
+ * \brief Let Stasis app internals shut down
+ *
+ * This is called when res_stasis is unloaded. It ensures that
+ * the Stasis app internals can free any resources they may have
+ * allocated during the time that res_stasis was loaded.
+ */
+void stasis_app_control_shutdown(void);
+
+/*!
  * \brief Enable/disable request/response and event logging on an application
  *
  * \param app The app to debug
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_app_playback.h 1:16.1.1~dfsg-1/include/asterisk/stasis_app_playback.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_app_playback.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_app_playback.h	2018-12-26 18:45:13.000000000 +0000
@@ -41,6 +41,8 @@ enum stasis_app_playback_state {
 	STASIS_PLAYBACK_STATE_PLAYING,
 	/*! The media is currently playing */
 	STASIS_PLAYBACK_STATE_PAUSED,
+	/*! The media is transitioning to the next in the list */
+	STASIS_PLAYBACK_STATE_CONTINUING,
 	/*! The media has stopped playing */
 	STASIS_PLAYBACK_STATE_COMPLETE,
 	/*! The playback was canceled. */
@@ -84,7 +86,8 @@ enum stasis_app_playback_target_type {
  * available codecs for the channel.
  *
  * \param control Control for \c res_stasis.
- * \param file Base filename for the file to play.
+ * \param media Array of const char * media files to play.
+ * \param media_count The number of media files in \c media.
  * \param language Selects the file based on language.
  * \param target_id ID of the target bridge or channel.
  * \param target_type What the target type is
@@ -95,8 +98,8 @@ enum stasis_app_playback_target_type {
  * \return \c NULL on error.
  */
 struct stasis_app_playback *stasis_app_control_play_uri(
-	struct stasis_app_control *control, const char *file,
-	const char *language, const char *target_id,
+	struct stasis_app_control *control, const char **media,
+	size_t media_count, const char *language, const char *target_id,
 	enum stasis_app_playback_target_type target_type,
 	int skipms, long offsetms, const char *id);
 
@@ -128,6 +131,14 @@ const char *stasis_app_playback_get_id(
  */
 struct stasis_app_playback *stasis_app_playback_find_by_id(const char *id);
 
+/*!
+ * \brief Convert a playback to its JSON representation
+ *
+ * \param playback The playback object to convert to JSON
+ *
+ * \retval \c NULL on error
+ * \retval A JSON object on success
+ */
 struct ast_json *stasis_app_playback_to_json(
 	const struct stasis_app_playback *playback);
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_app_recording.h 1:16.1.1~dfsg-1/include/asterisk/stasis_app_recording.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_app_recording.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_app_recording.h	2018-12-26 18:45:13.000000000 +0000
@@ -49,6 +49,30 @@ const char *stasis_app_stored_recording_
 	struct stasis_app_stored_recording *recording);
 
 /*!
+ * \brief Returns the full filename, with extension, for this recording.
+ * \since 14.0.0
+ *
+ * \param recording Recording to query.
+ *
+ * \return Absolute path to the recording file, with the extension.
+ * \return \c NULL on error
+ */
+const char *stasis_app_stored_recording_get_filename(
+	struct stasis_app_stored_recording *recording);
+
+/*!
+ * \brief Returns the extension for this recording.
+ * \since 14.0.0
+ *
+ * \param recording Recording to query.
+ *
+ * \return The extension associated with this recording.
+ * \return \c NULL on error
+ */
+const char *stasis_app_stored_recording_get_extension(
+	struct stasis_app_stored_recording *recording);
+
+/*!
  * \brief Convert stored recording info to JSON.
  *
  * \param recording Recording to convert.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_bridges.h 1:16.1.1~dfsg-1/include/asterisk/stasis_bridges.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_bridges.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_bridges.h	2018-12-26 18:45:13.000000000 +0000
@@ -46,6 +46,8 @@ struct ast_bridge_snapshot {
 		AST_STRING_FIELD(creator);
 		/*! Name given to the bridge by its creator */
 		AST_STRING_FIELD(name);
+		/*! Unique ID of the channel providing video, if one exists */
+		AST_STRING_FIELD(video_source_id);
 	);
 	/*! AO2 container of bare channel uniqueid strings participating in the bridge.
 	 * Allocated from ast_str_container_alloc() */
@@ -60,8 +62,6 @@ struct ast_bridge_snapshot {
 	unsigned int num_active;
 	/*! The video mode of the bridge */
 	enum ast_bridge_video_mode_type video_mode;
-	/*! Unique ID of the channel providing video, if one exists */
-	AST_STRING_FIELD_EXTENDED(video_source_id);
 };
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_cache_pattern.h 1:16.1.1~dfsg-1/include/asterisk/stasis_cache_pattern.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_cache_pattern.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_cache_pattern.h	2018-12-26 18:45:13.000000000 +0000
@@ -169,4 +169,39 @@ struct stasis_topic *stasis_cp_single_to
 struct stasis_topic *stasis_cp_single_topic_cached(
 	struct stasis_cp_single *one);
 
+/*!
+ * \brief Indicate to an instance that we are interested in a message type.
+ *
+ * This will cause the caching topic to receive messages of the given message
+ * type. This enables internal filtering in the stasis message bus to reduce
+ * messages.
+ *
+ * \param one One side of the cache pattern.
+ * \param type The message type we wish to receive.
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_cp_single_accept_message_type(struct stasis_cp_single *one,
+	struct stasis_message_type *type);
+
+/*!
+ * \brief Set the message type filtering level on a cache
+ *
+ * This will cause the underlying subscription to filter messages according to the
+ * provided filter level. For example if selective is used then only
+ * messages matching those provided to \ref stasis_subscription_accept_message_type
+ * will be raised to the subscription callback.
+ *
+ * \param one One side of the cache pattern.
+ * \param filter What filter to use
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_cp_single_set_filter(struct stasis_cp_single *one,
+	enum stasis_subscription_message_filter filter);
+
 #endif /* _ASTERISK_STASIS_CACHE_PATTERN_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_channels.h 1:16.1.1~dfsg-1/include/asterisk/stasis_channels.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_channels.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_channels.h	2018-12-26 18:45:13.000000000 +0000
@@ -72,6 +72,7 @@ struct ast_channel_snapshot {
 	struct ast_flags softhangup_flags;      /*!< softhangup channel flags */
 	struct varshead *manager_vars;          /*!< Variables to be appended to manager events */
 	int tech_properties;                    /*!< Properties of the channel's technology */
+	struct varshead *ari_vars;              /*!< Variables to be appended to ARI events */
 };
 
 /*!
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis.h 1:16.1.1~dfsg-1/include/asterisk/stasis.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis.h	2018-12-26 18:45:13.000000000 +0000
@@ -173,8 +173,6 @@
 #include "asterisk/utils.h"
 #include "asterisk/event.h"
 
-/*! @{ */
-
 /*!
  * \brief Metadata about a \ref stasis_message.
  * \since 12
@@ -294,6 +292,15 @@ enum stasis_message_type_result {
 };
 
 /*!
+ * \brief Stasis subscription message filters
+ */
+enum stasis_subscription_message_filter {
+	STASIS_SUBSCRIPTION_FILTER_NONE = 0,    /*!< No filter is in place, all messages are raised */
+	STASIS_SUBSCRIPTION_FILTER_FORCED_NONE, /*!< No filter is in place or can be set, all messages are raised */
+	STASIS_SUBSCRIPTION_FILTER_SELECTIVE,   /*!< Only messages of allowed message types are raised */
+};
+
+/*!
  * \brief Create a new message type.
  *
  * \ref stasis_message_type is an AO2 object, so ao2_cleanup() when you're done
@@ -329,6 +336,14 @@ const char *stasis_message_type_name(con
 unsigned int stasis_message_type_hash(const struct stasis_message_type *type);
 
 /*!
+ * \brief Gets the id of a given message type
+ * \param type The type to get the id of.
+ * \return The id
+ * \since 17.0.0
+ */
+int stasis_message_type_id(const struct stasis_message_type *type);
+
+/*!
  * \brief Check whether a message type is declined
  *
  * \param name The name of the message type to check
@@ -467,10 +482,6 @@ int stasis_message_can_be_ami(struct sta
  */
 struct ast_event *stasis_message_to_event(struct stasis_message *msg);
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \brief A topic to which messages may be posted, and subscribers, well, subscribe
  * \since 12
@@ -483,6 +494,11 @@ struct stasis_topic;
  * \return New topic instance.
  * \return \c NULL on error.
  * \since 12
+ *
+ * \note There is no explicit ability to unsubscribe all subscribers
+ * from a topic and destroy it. As a result the topic can persist until
+ * the last subscriber unsubscribes itself even if there is no
+ * publisher.
  */
 struct stasis_topic *stasis_topic_create(const char *name);
 
@@ -496,6 +512,14 @@ struct stasis_topic *stasis_topic_create
 const char *stasis_topic_name(const struct stasis_topic *topic);
 
 /*!
+ * \brief Return the number of subscribers of a topic.
+ * \param topic Topic.
+ * \return Number of subscribers of the topic.
+ * \since 17.0.0
+ */
+size_t stasis_topic_subscribers(const struct stasis_topic *topic);
+
+/*!
  * \brief Publish a message to a topic's subscribers.
  * \param topic Topic.
  * \param message Message to publish.
@@ -524,10 +548,6 @@ void stasis_publish(struct stasis_topic
  */
 void stasis_publish_sync(struct stasis_subscription *sub, struct stasis_message *message);
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \brief Callback function type for Stasis subscriptions.
  * \param data Data field provided with subscription.
@@ -564,6 +584,10 @@ void stasis_subscription_cb_noop(void *d
  * \return New \ref stasis_subscription object.
  * \return \c NULL on error.
  * \since 12
+ *
+ * \note This callback will receive a callback with a message indicating it
+ * has been subscribed. This occurs immediately before accepted message
+ * types can be set and the callback must expect to receive it.
  */
 struct stasis_subscription *stasis_subscribe(struct stasis_topic *topic,
 	stasis_subscription_cb callback, void *data);
@@ -589,11 +613,69 @@ struct stasis_subscription *stasis_subsc
  * \return New \ref stasis_subscription object.
  * \return \c NULL on error.
  * \since 12.8.0
+ *
+ * \note This callback will receive a callback with a message indicating it
+ * has been subscribed. This occurs immediately before accepted message
+ * types can be set and the callback must expect to receive it.
  */
 struct stasis_subscription *stasis_subscribe_pool(struct stasis_topic *topic,
 	stasis_subscription_cb callback, void *data);
 
 /*!
+ * \brief Indicate to a subscription that we are interested in a message type.
+ *
+ * This will cause the subscription to allow the given message type to be
+ * raised to our subscription callback. This enables internal filtering in
+ * the stasis message bus to reduce messages.
+ *
+ * \param subscription Subscription to add message type to.
+ * \param type The message type we wish to receive.
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ *
+ * \note If you are wanting to use stasis_final_message you will need to accept
+ * \ref stasis_subscription_change_type as a message type.
+ *
+ * \note Until the subscription is set to selective filtering it is possible for it
+ * to receive messages of message types that would not normally be accepted.
+ */
+int stasis_subscription_accept_message_type(struct stasis_subscription *subscription,
+	const struct stasis_message_type *type);
+
+/*!
+ * \brief Indicate to a subscription that we are not interested in a message type.
+ *
+ * \param subscription Subscription to remove message type from.
+ * \param type The message type we don't wish to receive.
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_subscription_decline_message_type(struct stasis_subscription *subscription,
+	const struct stasis_message_type *type);
+
+/*!
+ * \brief Set the message type filtering level on a subscription
+ *
+ * This will cause the subscription to filter messages according to the
+ * provided filter level. For example if selective is used then only
+ * messages matching those provided to \ref stasis_subscription_accept_message_type
+ * will be raised to the subscription callback.
+ *
+ * \param subscription Subscription that should receive all messages.
+ * \param filter What filter to use
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_subscription_set_filter(struct stasis_subscription *subscription,
+	enum stasis_subscription_message_filter filter);
+
+/*!
  * \brief Cancel a subscription.
  *
  * Note that in an asynchronous system, there may still be messages queued or
@@ -724,11 +806,9 @@ int stasis_subscription_final_message(st
  * \since 12
  */
 struct stasis_subscription_change {
-	AST_DECLARE_STRING_FIELDS(
-		AST_STRING_FIELD(uniqueid);	/*!< The unique ID associated with this subscription */
-		AST_STRING_FIELD(description);	/*!< The description of the change to the subscription associated with the uniqueid */
-	);
-	struct stasis_topic *topic;		/*!< The topic the subscription is/was subscribing to */
+	struct stasis_topic *topic; /*!< The topic the subscription is/was subscribing to */
+	char *uniqueid;             /*!< The unique ID associated with this subscription */
+	char description[0];        /*!< The description of the change to the subscription associated with the uniqueid */
 };
 
 /*!
@@ -740,8 +820,6 @@ struct stasis_message_type *stasis_subsc
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \brief Pool for topic aggregation
  */
@@ -765,6 +843,18 @@ struct stasis_topic_pool *stasis_topic_p
 struct stasis_topic *stasis_topic_pool_get_topic(struct stasis_topic_pool *pool, const char *topic_name);
 
 /*!
+ * \brief Delete a topic from the topic pool
+ *
+ * \param pool Pool from which to delete the topic
+ * \param topic_name Name of the topic to delete
+ *
+ * \since 13.24
+ * \since 15.6
+ * \since 16.1
+ */
+void stasis_topic_pool_delete_topic(struct stasis_topic_pool *pool, const char *topic_name);
+
+/*!
  * \brief Check if a topic exists in a pool
  * \param pool Pool to check
  * \param topic_name Name of the topic to check
@@ -774,8 +864,6 @@ struct stasis_topic *stasis_topic_pool_g
  */
 int stasis_topic_pool_topic_exists(const struct stasis_topic_pool *pool, const char *topic_name);
 
-/*! @} */
-
 /*! \addtogroup StasisTopicsAndMessages
  * @{
  */
@@ -808,8 +896,6 @@ struct stasis_message_type *stasis_cache
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \brief A message cache, for use with \ref stasis_caching_topic.
  * \since 12
@@ -1036,6 +1122,41 @@ struct stasis_topic *stasis_caching_get_
 	struct stasis_caching_topic *caching_topic);
 
 /*!
+ * \brief Indicate to a caching topic that we are interested in a message type.
+ *
+ * This will cause the caching topic to receive messages of the given message
+ * type. This enables internal filtering in the stasis message bus to reduce
+ * messages.
+ *
+ * \param caching_topic The caching topic.
+ * \param type The message type we wish to receive.
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_caching_accept_message_type(struct stasis_caching_topic *caching_topic,
+	struct stasis_message_type *type);
+
+/*!
+ * \brief Set the message type filtering level on a cache
+ *
+ * This will cause the underlying subscription to filter messages according to the
+ * provided filter level. For example if selective is used then only
+ * messages matching those provided to \ref stasis_subscription_accept_message_type
+ * will be raised to the subscription callback.
+ *
+ * \param caching_topic The caching topic.
+ * \param filter What filter to use
+ * \retval 0 on success
+ * \retval -1 failure
+ *
+ * \since 17.0.0
+ */
+int stasis_caching_set_filter(struct stasis_caching_topic *caching_topic,
+	enum stasis_subscription_message_filter filter);
+
+/*!
  * \brief A message which instructs the caching topic to remove an entry from
  * its cache.
  *
@@ -1141,6 +1262,10 @@ struct ao2_container *stasis_cache_dump_
  */
 struct ao2_container *stasis_cache_dump_all(struct stasis_cache *cache, struct stasis_message_type *type);
 
+/*! \addtogroup StasisTopicsAndMessages
+ * @{
+ */
+
 /*!
  * \brief Object type code for multi user object snapshots
  */
@@ -1214,8 +1339,6 @@ void ast_multi_object_blob_single_channe
 
 /*! @} */
 
-/*! @{ */
-
 /*!
  * \internal
  * \brief Log a message about invalid attempt to access a type.
@@ -1318,10 +1441,6 @@ void stasis_log_bad_type_access(const ch
 		_priv_ ## name = NULL;		\
 	})
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \brief Initialize the Stasis subsystem.
  * \return 0 on success.
@@ -1330,10 +1449,6 @@ void stasis_log_bad_type_access(const ch
  */
 int stasis_init(void);
 
-/*! @} */
-
-/*! @{ */
-
 /*!
  * \internal
  * \brief called by stasis_init() for cache initialization.
@@ -1352,12 +1467,10 @@ int stasis_cache_init(void);
  */
 int stasis_config_init(void);
 
-/*! @} */
-
 /*!
  * \defgroup StasisTopicsAndMessages Stasis topics, and their messages.
  *
- * This group contains the topics, messages and corresponding message types
+ * \brief This group contains the topics, messages and corresponding message types
  * found within Asterisk.
  */
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_message_router.h 1:16.1.1~dfsg-1/include/asterisk/stasis_message_router.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_message_router.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_message_router.h	2018-12-26 18:45:13.000000000 +0000
@@ -233,6 +233,10 @@ void stasis_message_router_remove_cache_
  * \retval -1 on failure
  *
  * \since 12
+ *
+ * \note Setting a default callback will automatically cause the underlying
+ * subscription to receive all messages and not be filtered. If filtering is
+ * desired then a specific route for each message type should be provided.
  */
 int stasis_message_router_set_default(struct stasis_message_router *router,
 				      stasis_subscription_cb callback,
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stasis_test.h 1:16.1.1~dfsg-1/include/asterisk/stasis_test.h
--- 1:13.23.1~dfsg-2/include/asterisk/stasis_test.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stasis_test.h	2018-12-26 18:45:13.000000000 +0000
@@ -20,7 +20,8 @@
 #define _ASTERISK_STASIS_TEST_H
 
 /*!
- * \file \brief Test infrastructure for dealing with Stasis.
+ * \file
+ * \brief Test infrastructure for dealing with Stasis.
  *
  * \author David M. Lee, II <dlee@digium.com>
  *
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stream.h 1:16.1.1~dfsg-1/include/asterisk/stream.h
--- 1:13.23.1~dfsg-2/include/asterisk/stream.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stream.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,526 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Media Stream API
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+#ifndef _AST_STREAM_H_
+#define _AST_STREAM_H_
+
+#include "asterisk/codec.h"
+#include "asterisk/vector.h"
+
+/*!
+ * \brief Forward declaration for a stream, as it is opaque
+ */
+struct ast_stream;
+
+/*!
+ * \brief Forward declaration for a format capability
+ */
+struct ast_format_cap;
+
+/*!
+ * \brief The topology of a set of streams
+ */
+struct ast_stream_topology;
+
+/*!
+ * \brief A mapping of two topologies.
+ */
+struct ast_stream_topology_map;
+
+typedef void (*ast_stream_data_free_fn)(void *);
+
+/*!
+ * \brief States that a stream may be in
+ */
+enum ast_stream_state {
+	/*!
+	 * \brief Set when the stream has been removed/declined
+	 */
+	AST_STREAM_STATE_REMOVED = 0,
+	/*!
+	 * \brief Set when the stream is sending and receiving media
+	 */
+	AST_STREAM_STATE_SENDRECV,
+	/*!
+	 * \brief Set when the stream is sending media only
+	 */
+	AST_STREAM_STATE_SENDONLY,
+	/*!
+	 * \brief Set when the stream is receiving media only
+	 */
+	AST_STREAM_STATE_RECVONLY,
+	/*!
+	 * \brief Set when the stream is not sending OR receiving media
+	 */
+	AST_STREAM_STATE_INACTIVE,
+};
+
+/*!
+ * \brief Create a new media stream representation
+ *
+ * \param name A name for the stream
+ * \param type The media type the stream is handling
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note This is NOT an AO2 object and has no locking. It is expected that a higher level object provides protection.
+ *
+ * \note The stream will default to an inactive state until changed.
+ *
+ * \since 15
+ */
+struct ast_stream *ast_stream_alloc(const char *name, enum ast_media_type type);
+
+/*!
+ * \brief Destroy a media stream representation
+ *
+ * \param stream The media stream
+ *
+ * \since 15
+ */
+void ast_stream_free(struct ast_stream *stream);
+
+/*!
+ * \brief Create a deep clone of an existing stream
+ *
+ * \param stream The existing stream
+ * \param Optional name for cloned stream. If NULL, then existing stream's name is copied.
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note Opaque data pointers set with ast_stream_set_data() are not part
+ * of the deep clone.  We have no way to clone the data.
+ *
+ * \since 15
+ */
+struct ast_stream *ast_stream_clone(const struct ast_stream *stream, const char *name);
+
+/*!
+ * \brief Get the name of a stream
+ *
+ * \param stream The media stream
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 15
+ */
+const char *ast_stream_get_name(const struct ast_stream *stream);
+
+/*!
+ * \brief Get the media type of a stream
+ *
+ * \param stream The media stream
+ *
+ * \return The media type of the stream (AST_MEDIA_TYPE_UNKNOWN on error)
+ *
+ * \since 15
+ */
+enum ast_media_type ast_stream_get_type(const struct ast_stream *stream);
+
+/*!
+ * \brief Change the media type of a stream
+ *
+ * \param stream The media stream
+ * \param type The new media type
+ *
+ * \since 15
+ */
+void ast_stream_set_type(struct ast_stream *stream, enum ast_media_type type);
+
+/*!
+ * \brief Get the current negotiated formats of a stream
+ *
+ * \param stream The media stream
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The reference count is not increased
+ *
+ * \since 15
+ */
+struct ast_format_cap *ast_stream_get_formats(const struct ast_stream *stream);
+
+/*!
+ * \brief Set the current negotiated formats of a stream
+ *
+ * \param stream The media stream
+ * \param caps The current negotiated formats
+ *
+ * \note The new format capabilities structure has its refcount bumped and
+ * any existing format capabilities structure has its refcount decremented.
+ *
+ * \since 15
+ */
+void ast_stream_set_formats(struct ast_stream *stream, struct ast_format_cap *caps);
+
+/*!
+ * \brief Get the current state of a stream
+ *
+ * \param stream The media stream
+ *
+ * \return The state of the stream (AST_STREAM_STATE_UNKNOWN on error)
+ *
+ * \since 15
+ */
+enum ast_stream_state ast_stream_get_state(const struct ast_stream *stream);
+
+/*!
+ * \brief Set the state of a stream
+ *
+ * \param stream The media stream
+ * \param state The new state that the stream is in
+ *
+ * \note Used by stream creator to update internal state
+ *
+ * \since 15
+ */
+void ast_stream_set_state(struct ast_stream *stream, enum ast_stream_state state);
+
+/*!
+ * \brief Convert the state of a stream into a string
+ *
+ * \param state The stream state
+ *
+ * \return The state of the stream in string format
+ *
+ * \since 15
+ */
+const char *ast_stream_state2str(enum ast_stream_state state);
+
+/*!
+ * \brief Convert a string to a stream state
+ *
+ * \param str The string to convert
+ *
+ * \return The stream state
+ *
+ * \since 15.0.0
+ */
+enum ast_stream_state ast_stream_str2state(const char *str);
+
+/*!
+ * \brief Get a stream metadata value
+ *
+ * \param stream The media stream
+ * \param m_key  An arbitrary metadata key
+ *
+ * \retval non-NULL metadata value
+ * \retval NULL failure or not found
+ *
+ * \since 15.5
+ */
+const char *ast_stream_get_metadata(const struct ast_stream *stream,
+	const char *m_key);
+
+/*!
+ * \brief Get all stream metadata keys
+ *
+ * \param stream The media stream
+ *
+ * \retval An ast_variable list of the metadata key/value pairs.
+ * \retval NULL if error or no variables are set.
+ *
+ * When you're finished with the list, you must call
+ *  ast_variables_destroy(list);
+ *
+ * \since 15.5
+ */
+struct ast_variable *ast_stream_get_metadata_list(const struct ast_stream *stream);
+
+/*!
+ * \brief Set a stream metadata value
+ *
+ * \param stream The media stream
+ * \param m_key  An arbitrary metadata key
+ * \param value  String metadata value or NULL to remove existing value
+ *
+ * \retval -1 failure
+ * \retval 0  success
+ *
+ * \since 15.5
+ */
+int ast_stream_set_metadata(struct ast_stream *stream, const char *m_key, const char *value);
+
+/*!
+ * \brief Get the position of the stream in the topology
+ *
+ * \param stream The media stream
+ *
+ * \return The position of the stream (-1 on error)
+ *
+ * \since 15
+ */
+int ast_stream_get_position(const struct ast_stream *stream);
+
+/*!
+ * \brief Get rtp_codecs associated with the stream
+ *
+ * \param stream The media stream
+ *
+ * \return The rtp_codecs
+ *
+ * \since 15.5
+ */
+struct ast_rtp_codecs *ast_stream_get_rtp_codecs(const struct ast_stream *stream);
+
+/*!
+ * \brief Set rtp_codecs associated with the stream
+ *
+ * \param stream The media stream
+ * \param rtp_codecs The rtp_codecs
+ *
+ * \since 15.5
+ */
+void ast_stream_set_rtp_codecs(struct ast_stream *stream, struct ast_rtp_codecs *rtp_codecs);
+
+/*!
+ * \brief Create a stream topology
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 15
+ *
+ * \note This returns an ao2 refcounted object
+ */
+struct ast_stream_topology *ast_stream_topology_alloc(void);
+
+/*!
+ * \brief Create a deep clone of an existing stream topology
+ *
+ * \param topology The existing topology of streams
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 15
+ *
+ * \note This returns an ao2 refcounted object
+ */
+struct ast_stream_topology *ast_stream_topology_clone(
+	const struct ast_stream_topology *topology);
+
+/*!
+ * \brief Compare two stream topologies to see if they are equal
+ *
+ * \param left The left topology
+ * \param right The right topology
+ *
+ * \retval 1 topologies are equivalent
+ * \retval 0 topologies differ
+ *
+ * \since 15
+ */
+int ast_stream_topology_equal(const struct ast_stream_topology *left,
+	const struct ast_stream_topology *right);
+
+/*!
+ * \brief Unreference and destroy a stream topology
+ *
+ * \param topology The topology of streams
+ *
+ * \note All streams contained within the topology will be destroyed
+ *
+ * \since 15
+ */
+void ast_stream_topology_free(struct ast_stream_topology *topology);
+
+/*!
+ * \brief Append a stream to the topology
+ *
+ * \param topology The topology of streams
+ * \param stream The stream to append
+ *
+ * \returns the position of the stream in the topology (-1 on error)
+ *
+ * \since 15
+ */
+int ast_stream_topology_append_stream(struct ast_stream_topology *topology,
+	struct ast_stream *stream);
+
+/*!
+ * \brief Get the number of streams in a topology
+ *
+ * \param topology The topology of streams
+ *
+ * \return the number of streams (-1 on error)
+ *
+ * \since 15
+ */
+int ast_stream_topology_get_count(const struct ast_stream_topology *topology);
+
+/*!
+ * \brief Get a specific stream from the topology
+ *
+ * \param topology The topology of streams
+ * \param position The topology position to get
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 15
+ */
+struct ast_stream *ast_stream_topology_get_stream(
+	const struct ast_stream_topology *topology, unsigned int position);
+
+/*!
+ * \brief Set a specific position in a topology
+ *
+ * \param topology The topology of streams
+ * \param position The topology position to set
+ * \param stream The stream to put in its place
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ *
+ * \note If an existing stream exists it will be destroyed
+ *
+ * \note You can overwrite an existing position in the topology or set
+ * the first unused position.  You can't set positions beyond that.
+ *
+ * \since 15
+ */
+int ast_stream_topology_set_stream(struct ast_stream_topology *topology,
+	unsigned int position, struct ast_stream *stream);
+
+/*!
+ * \brief Delete a specified stream from the given topology.
+ * \since 15.0.0
+ *
+ * \param topology The topology of streams.
+ * \param position The topology position to delete.
+ *
+ * \note Deleting a stream will completely remove it from the topology
+ * as if it never existed in it.  i.e., Any following stream positions
+ * will shift down so there is no gap.
+ *
+ * \retval 0 on success.
+ * \retval -1 on failure.
+ *
+ * \return Nothing
+ */
+int ast_stream_topology_del_stream(struct ast_stream_topology *topology,
+	unsigned int position);
+
+/*!
+ * \brief A helper function that, given a format capabilities structure,
+ * creates a topology and separates the media types in format_cap into
+ * separate streams.
+ *
+ * \param caps The format capabilities structure (NULL creates an empty topology)
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The format capabilities reference is NOT altered by this function
+ * since a new format capabilities structure is created for each media type.
+ *
+ * \note Each stream will have its name set to the corresponding media type.
+ * For example: "audio".
+ *
+ * \note Each stream will be set to the sendrecv state.
+ *
+ * \since 15
+ */
+struct ast_stream_topology *ast_stream_topology_create_from_format_cap(
+	struct ast_format_cap *cap);
+
+/*!
+ * \brief Create a format capabilities structure representing the topology.
+ *
+ * \details
+ * A helper function that, given a stream topology, creates a format
+ * capabilities structure containing all formats from all active streams.
+ *
+ * \param topology The topology of streams
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \note The stream topology is NOT altered by this function.
+ *
+ * \since 15
+ */
+struct ast_format_cap *ast_format_cap_from_stream_topology(
+	struct ast_stream_topology *topology);
+
+/*!
+ * \brief Gets the first active stream of a specific type from the topology
+ *
+ * \param topology The topology of streams
+ * \param type The media type
+ *
+ * \retval non-NULL success
+ * \retval NULL failure
+ *
+ * \since 15
+ */
+struct ast_stream *ast_stream_topology_get_first_stream_by_type(
+	const struct ast_stream_topology *topology,
+	enum ast_media_type type);
+
+/*!
+ * \brief Map a given topology's streams to the given types.
+ *
+ * \note The given vectors in which mapping values are placed are reset by
+ *       this function. This means if those vectors already contain mapping
+ *       values they will be lost.
+ *
+ * \param topology The topology to map
+ * \param types The media types to be mapped
+ * \param v0 Index mapping of topology to types
+ * \param v1 Index mapping of types to topology
+ *
+ * \since 15
+ */
+void ast_stream_topology_map(const struct ast_stream_topology *topology,
+	struct ast_vector_int *types, struct ast_vector_int *v0, struct ast_vector_int *v1);
+
+/*!
+ * \brief Get the stream group that a stream is part of
+ *
+ * \param stream The stream
+ *
+ * \return the numerical stream group (-1 if not in a group)
+ *
+ * \since 15.2.0
+ */
+int ast_stream_get_group(const struct ast_stream *stream);
+
+/*!
+ * \brief Set the stream group for a stream
+ *
+ * \param stream The stream
+ * \param group The group the stream is part of
+ *
+ * \since 15.2.0
+ */
+void ast_stream_set_group(struct ast_stream *stream, int group);
+
+#endif /* _AST_STREAM_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/stringfields.h 1:16.1.1~dfsg-1/include/asterisk/stringfields.h
--- 1:13.23.1~dfsg-2/include/asterisk/stringfields.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/stringfields.h	2018-12-26 18:45:13.000000000 +0000
@@ -222,26 +222,12 @@ AST_VECTOR(ast_string_field_vector, cons
 
 /*!
   \internal
-  \brief Structure used to hold a pointer to the embedded pool and the field vector
-  \since 13.9.0
-*/
-struct ast_string_field_header {
-	struct ast_string_field_pool *embedded_pool;	/*!< pointer to the embedded pool, if any */
-	struct ast_string_field_vector string_fields;	/*!< field vector for compare and copy */
-};
-
-/*!
-  \internal
   \brief Structure used to manage the storage for a set of string fields.
 */
 struct ast_string_field_mgr {
 	ast_string_field last_alloc;			/*!< the last field allocated */
-	struct ast_string_field_header *header;	/*!< pointer to the header */
-#if defined(__AST_DEBUG_MALLOC)
-	const char *owner_file;				/*!< filename of owner */
-	const char *owner_func;				/*!< function name of owner */
-	int owner_line;					/*!< line number of owner */
-#endif
+	struct ast_string_field_pool *embedded_pool;	/*!< pointer to the embedded pool, if any */
+	struct ast_string_field_vector string_fields;	/*!< field vector for compare and copy */
 };
 
 /*!
@@ -275,7 +261,8 @@ int __ast_string_field_ptr_grow(struct a
   an additional pool will be allocated.
 */
 ast_string_field __ast_string_field_alloc_space(struct ast_string_field_mgr *mgr,
-						struct ast_string_field_pool **pool_head, size_t needed);
+	struct ast_string_field_pool **pool_head, size_t needed,
+	const char *file, int lineno, const char *func);
 
 /*!
   \internal
@@ -286,9 +273,9 @@ ast_string_field __ast_string_field_allo
   \param format printf-style format string
   \return nothing
 */
-void __ast_string_field_ptr_build(struct ast_string_field_mgr *mgr,
-				  struct ast_string_field_pool **pool_head,
-				  ast_string_field *ptr, const char *format, ...) __attribute__((format(printf, 4, 5)));
+void __ast_string_field_ptr_build(const char *file, int lineno, const char *func,
+	struct ast_string_field_mgr *mgr, struct ast_string_field_pool **pool_head,
+	ast_string_field *ptr, const char *format, ...) __attribute__((format(printf, 7, 8)));
 
 /*!
   \internal
@@ -301,8 +288,9 @@ void __ast_string_field_ptr_build(struct
   \return nothing
 */
 void __ast_string_field_ptr_build_va(struct ast_string_field_mgr *mgr,
-				     struct ast_string_field_pool **pool_head,
-				     ast_string_field *ptr, const char *format, va_list ap) __attribute__((format(printf, 4, 0)));
+	struct ast_string_field_pool **pool_head,
+	ast_string_field *ptr, const char *format, va_list ap,
+	const char *file, int lineno, const char *func) __attribute__((format(printf, 4, 0)));
 
 /*!
   \brief Declare a string field
@@ -407,10 +395,10 @@ int __ast_string_field_free_memory(struc
 #define ast_string_field_init_extended(x, field) \
 ({ \
 	int __res__ = -1; \
-	if (((void *)(x)) != NULL && (x)->__field_mgr.header != NULL) { \
+	if (((void *)(x)) != NULL) { \
 		ast_string_field *non_const = (ast_string_field *)&(x)->field; \
 		*non_const = __ast_string_field_empty; \
-		__res__ = AST_VECTOR_APPEND(&(x)->__field_mgr.header->string_fields, non_const); \
+		__res__ = AST_VECTOR_APPEND(&(x)->__field_mgr.string_fields, non_const); \
 	} \
 	__res__; \
 })
@@ -488,7 +476,7 @@ void __ast_string_field_release_active(s
 	__res__; \
 })
 
-#define ast_string_field_ptr_set_by_fields(field_mgr_pool, field_mgr, ptr, data)               \
+#define __ast_string_field_ptr_set_by_fields(field_mgr_pool, field_mgr, ptr, data, file, lineno, func) \
 ({                                                                                             \
     int __res__ = 0;                                                                           \
     const char *__d__ = (data);                                                                \
@@ -500,7 +488,7 @@ void __ast_string_field_release_active(s
         *__p__ = __ast_string_field_empty;                                                     \
     } else if ((__dlen__ <= AST_STRING_FIELD_ALLOCATION(*__p__)) ||                            \
            (!__ast_string_field_ptr_grow(&field_mgr, &field_mgr_pool, __dlen__, __p__)) ||     \
-           (target = __ast_string_field_alloc_space(&field_mgr, &field_mgr_pool, __dlen__))) { \
+           (target = __ast_string_field_alloc_space(&field_mgr, &field_mgr_pool, __dlen__, file, lineno, func))) { \
         if (target != *__p__) {                                                                \
             __ast_string_field_release_active(field_mgr_pool, *__p__);                         \
             *__p__ = target;                                                                   \
@@ -512,6 +500,9 @@ void __ast_string_field_release_active(s
     __res__;                                                                                   \
 })
 
+#define ast_string_field_ptr_set_by_fields(field_mgr_pool, field_mgr, ptr, data) \
+	__ast_string_field_ptr_set_by_fields(field_mgr_pool, field_mgr, ptr, data, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 /*!
   \brief Set a field to a simple string value
   \param x Pointer to a structure containing fields
@@ -541,7 +532,8 @@ void __ast_string_field_release_active(s
 ({ \
 	int __res__ = -1; \
 	if (((void *)(x)) != NULL) { \
-		__ast_string_field_ptr_build(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) ptr, fmt, args); \
+		__ast_string_field_ptr_build(__FILE__, __LINE__, __PRETTY_FUNCTION__, \
+			&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) ptr, fmt, args); \
 		__res__ = 0; \
 	} \
 	__res__; \
@@ -559,7 +551,8 @@ void __ast_string_field_release_active(s
 ({ \
 	int __res__ = -1; \
 	if (((void *)(x)) != NULL) { \
-		__ast_string_field_ptr_build(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) &(x)->field, fmt, args); \
+		__ast_string_field_ptr_build(__FILE__, __LINE__, __PRETTY_FUNCTION__, \
+			&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) &(x)->field, fmt, args); \
 		__res__ = 0; \
 	} \
 	__res__; \
@@ -577,7 +570,8 @@ void __ast_string_field_release_active(s
 ({ \
 	int __res__ = -1; \
 	if (((void *)(x)) != NULL) { \
-		__ast_string_field_ptr_build_va(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) ptr, fmt, args); \
+		__ast_string_field_ptr_build_va(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) ptr, fmt, args, \
+			__FILE__, __LINE__, __PRETTY_FUNCTION__); \
 		__res__ = 0; \
 	} \
 	__res__; \
@@ -595,7 +589,8 @@ void __ast_string_field_release_active(s
 ({ \
 	int __res__ = -1; \
 	if (((void *)(x)) != NULL) { \
-		__ast_string_field_ptr_build_va(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) &(x)->field, fmt, args); \
+		__ast_string_field_ptr_build_va(&(x)->__field_mgr, &(x)->__field_mgr_pool, (ast_string_field *) &(x)->field, fmt, args, \
+			__FILE__, __LINE__, __PRETTY_FUNCTION__); \
 		__res__ = 0; \
 	} \
 	__res__; \
@@ -613,8 +608,8 @@ void __ast_string_field_release_active(s
 ({ \
 	int __res__ = -1; \
 	if (((void *)(instance1)) != NULL && ((void *)(instance2)) != NULL) { \
-		__res__ = __ast_string_fields_cmp(&(instance1)->__field_mgr.header->string_fields, \
-			&(instance2)->__field_mgr.header->string_fields); \
+		__res__ = __ast_string_fields_cmp(&(instance1)->__field_mgr.string_fields, \
+			&(instance2)->__field_mgr.string_fields); \
 	} \
 	__res__; \
 })
@@ -635,12 +630,14 @@ int __ast_string_fields_cmp(struct ast_s
 	if (((void *)(copy)) != NULL && ((void *)(orig)) != NULL) { \
 		__res__ = __ast_string_fields_copy(((copy)->__field_mgr_pool), \
 			(struct ast_string_field_mgr *)&((copy)->__field_mgr), \
-			(struct ast_string_field_mgr *)&((orig)->__field_mgr)); \
+			(struct ast_string_field_mgr *)&((orig)->__field_mgr), \
+			__FILE__, __LINE__, __PRETTY_FUNCTION__); \
 	} \
 	__res__; \
 })
 
 int __ast_string_fields_copy(struct ast_string_field_pool *copy_pool,
-	struct ast_string_field_mgr *copy_mgr, struct ast_string_field_mgr *orig_mgr);
+	struct ast_string_field_mgr *copy_mgr, struct ast_string_field_mgr *orig_mgr,
+	const char *file, int lineno, const char *func);
 
 #endif /* _ASTERISK_STRINGFIELDS_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/strings.h 1:16.1.1~dfsg-1/include/asterisk/strings.h
--- 1:13.23.1~dfsg-2/include/asterisk/strings.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/strings.h	2018-12-26 18:45:13.000000000 +0000
@@ -617,8 +617,8 @@ int ast_regex_string_to_regex_pattern(co
  * \note The result of this function is dynamically allocated memory, and must
  *       be free()'d after it is no longer needed.
  */
-#ifdef __AST_DEBUG_MALLOC
-#define	ast_str_create(a)	_ast_str_create(a,__FILE__,__LINE__,__PRETTY_FUNCTION__)
+#define ast_str_create(init_len) \
+	_ast_str_create(init_len, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 AST_INLINE_API(
 struct ast_str * attribute_malloc _ast_str_create(size_t init_len,
 		const char *file, int lineno, const char *func),
@@ -636,24 +636,6 @@ struct ast_str * attribute_malloc _ast_s
 	return buf;
 }
 )
-#else
-AST_INLINE_API(
-struct ast_str * attribute_malloc ast_str_create(size_t init_len),
-{
-	struct ast_str *buf;
-
-	buf = (struct ast_str *)ast_calloc(1, sizeof(*buf) + init_len);
-	if (buf == NULL)
-		return NULL;
-
-	buf->__AST_STR_LEN = init_len;
-	buf->__AST_STR_USED = 0;
-	buf->__AST_STR_TS = DS_MALLOC;
-
-	return buf;
-}
-)
-#endif
 
 /*! \brief Reset the content of a dynamic string.
  * Useful before a series of ast_str_append.
@@ -772,7 +754,6 @@ char *ast_str_truncate(struct ast_str *b
 /*!
  * Make space in a new string (e.g. to read in data from a file)
  */
-#ifdef __AST_DEBUG_MALLOC
 AST_INLINE_API(
 int _ast_str_make_space(struct ast_str **buf, size_t new_len, const char *file, int lineno, const char *function),
 {
@@ -796,32 +777,8 @@ int _ast_str_make_space(struct ast_str *
 	return 0;
 }
 )
-#define ast_str_make_space(a,b)	_ast_str_make_space(a,b,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-AST_INLINE_API(
-int ast_str_make_space(struct ast_str **buf, size_t new_len),
-{
-	struct ast_str *old_buf = *buf;
-
-	if (new_len <= (*buf)->__AST_STR_LEN)
-		return 0;	/* success */
-	if ((*buf)->__AST_STR_TS == DS_ALLOCA || (*buf)->__AST_STR_TS == DS_STATIC)
-		return -1;	/* cannot extend */
-	*buf = (struct ast_str *)ast_realloc(*buf, new_len + sizeof(struct ast_str));
-	if (*buf == NULL) {
-		*buf = old_buf;
-		return -1;
-	}
-	if ((*buf)->__AST_STR_TS != DS_MALLOC) {
-		pthread_setspecific((*buf)->__AST_STR_TS->key, *buf);
-		_DB1(__ast_threadstorage_object_replace(old_buf, *buf, new_len + sizeof(struct ast_str));)
-	}
-
-	(*buf)->__AST_STR_LEN = new_len;
-	return 0;
-}
-)
-#endif
+#define ast_str_make_space(buf, new_len) \
+	_ast_str_make_space(buf, new_len, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 AST_INLINE_API(
 int ast_str_copy_string(struct ast_str **dst, struct ast_str *src),
@@ -965,14 +922,12 @@ enum {
  *       through calling one of the other functions or macros defined in this
  *       file.
  */
-#ifdef __AST_DEBUG_MALLOC
-int __attribute__((format(printf, 4, 0))) __ast_debug_str_helper(struct ast_str **buf, ssize_t max_len,
-							   int append, const char *fmt, va_list ap, const char *file, int lineno, const char *func);
-#define __ast_str_helper(a,b,c,d,e)	__ast_debug_str_helper(a,b,c,d,e,__FILE__,__LINE__,__PRETTY_FUNCTION__)
-#else
-int __attribute__((format(printf, 4, 0))) __ast_str_helper(struct ast_str **buf, ssize_t max_len,
-							   int append, const char *fmt, va_list ap);
-#endif
+int __attribute__((format(printf, 4, 0))) __ast_str_helper(struct ast_str **buf,
+	ssize_t max_len, int append, const char *fmt, va_list ap,
+	const char *file, int lineno, const char *func);
+#define _ast_str_helper(buf, max_len, append, fmt, ap) \
+	__ast_str_helper(buf, max_len, append, fmt, ap, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 char *__ast_str_helper2(struct ast_str **buf, ssize_t max_len,
 	const char *src, size_t maxsrc, int append, int escapecommas);
 
@@ -1022,7 +977,7 @@ char *__ast_str_helper2(struct ast_str *
  */
 AST_INLINE_API(int __attribute__((format(printf, 3, 0))) ast_str_set_va(struct ast_str **buf, ssize_t max_len, const char *fmt, va_list ap),
 {
-	return __ast_str_helper(buf, max_len, 0, fmt, ap);
+	return _ast_str_helper(buf, max_len, 0, fmt, ap);
 }
 )
 
@@ -1040,7 +995,7 @@ AST_INLINE_API(int __attribute__((format
  */
 AST_INLINE_API(int __attribute__((format(printf, 3, 0))) ast_str_append_va(struct ast_str **buf, ssize_t max_len, const char *fmt, va_list ap),
 {
-	return __ast_str_helper(buf, max_len, 1, fmt, ap);
+	return _ast_str_helper(buf, max_len, 1, fmt, ap);
 }
 )
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/taskprocessor.h 1:16.1.1~dfsg-1/include/asterisk/taskprocessor.h
--- 1:13.23.1~dfsg-2/include/asterisk/taskprocessor.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/taskprocessor.h	2018-12-26 18:45:13.000000000 +0000
@@ -213,7 +213,8 @@ void *ast_taskprocessor_unreference(stru
  * \retval -1 failure
  * \since 1.6.1
  */
-int ast_taskprocessor_push(struct ast_taskprocessor *tps, int (*task_exe)(void *datap), void *datap);
+int ast_taskprocessor_push(struct ast_taskprocessor *tps, int (*task_exe)(void *datap), void *datap)
+	attribute_warn_unused_result;
 
 /*! \brief Local data parameter */
 struct ast_taskprocessor_local {
@@ -239,7 +240,8 @@ struct ast_taskprocessor_local {
  * \since 12.0.0
  */
 int ast_taskprocessor_push_local(struct ast_taskprocessor *tps,
-	int (*task_exe)(struct ast_taskprocessor_local *local), void *datap);
+	int (*task_exe)(struct ast_taskprocessor_local *local), void *datap)
+	attribute_warn_unused_result;
 
 /*!
  * \brief Indicate the taskprocessor is suspended.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/tcptls.h 1:16.1.1~dfsg-1/include/asterisk/tcptls.h
--- 1:13.23.1~dfsg-2/include/asterisk/tcptls.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/tcptls.h	2018-12-26 18:45:13.000000000 +0000
@@ -51,23 +51,13 @@
 #ifndef _ASTERISK_TCPTLS_H
 #define _ASTERISK_TCPTLS_H
 
-#include "asterisk.h"
-
 #include <pthread.h>            /* for pthread_t */
+#include <sys/param.h>          /* for MAXHOSTNAMELEN */
 
+#include "asterisk/iostream.h"
 #include "asterisk/netsock2.h"  /* for ast_sockaddr */
 #include "asterisk/utils.h"     /* for ast_flags */
 
-struct ssl_st;                  /* forward declaration */
-struct ssl_ctx_st;              /* forward declaration */
-struct timeval;                 /* forward declaration */
-typedef struct ssl_st SSL;
-typedef struct ssl_ctx_st SSL_CTX;
-
-#if defined(HAVE_OPENSSL) && (defined(HAVE_FUNOPEN) || defined(HAVE_FOPENCOOKIE))
-#define DO_SSL  /* comment in/out if you want to support ssl */
-#endif
-
 /*! SSL support */
 #define AST_CERTFILE "asterisk.pem"
 
@@ -153,72 +143,10 @@ struct ast_tcptls_session_args {
 	struct ast_tls_config *old_tls_cfg; /*!< copy of the SSL configuration to determine whether changes have been made */
 };
 
-struct ast_tcptls_stream;
-
-/*!
- * \brief Disable the TCP/TLS stream timeout timer.
- *
- * \param stream TCP/TLS stream control data.
- *
- * \return Nothing
- */
-void ast_tcptls_stream_set_timeout_disable(struct ast_tcptls_stream *stream);
-
-/*!
- * \brief Set the TCP/TLS stream inactivity timeout timer.
- *
- * \param stream TCP/TLS stream control data.
- * \param timeout Number of milliseconds to wait for data transfer with the peer.
- *
- * \details This is basically how much time we are willing to spend
- * in an I/O call before we declare the peer unresponsive.
- *
- * \note Setting timeout to -1 disables the timeout.
- * \note Setting this timeout replaces the I/O sequence timeout timer.
- *
- * \return Nothing
- */
-void ast_tcptls_stream_set_timeout_inactivity(struct ast_tcptls_stream *stream, int timeout);
-
-/*!
- * \brief Set the TCP/TLS stream I/O sequence timeout timer.
- *
- * \param stream TCP/TLS stream control data.
- * \param start Time the I/O sequence timer starts.
- * \param timeout Number of milliseconds from the start time before timeout.
- *
- * \details This is how much time are we willing to allow the peer
- * to complete an operation that can take several I/O calls.  The
- * main use is as an authentication timer with us.
- *
- * \note Setting timeout to -1 disables the timeout.
- * \note Setting this timeout replaces the inactivity timeout timer.
- *
- * \return Nothing
- */
-void ast_tcptls_stream_set_timeout_sequence(struct ast_tcptls_stream *stream, struct timeval start, int timeout);
-
-/*!
- * \brief Set the TCP/TLS stream I/O if it can exclusively depend upon the set timeouts.
- *
- * \param stream TCP/TLS stream control data.
- * \param exclusive_input TRUE if stream can exclusively wait for fd input.
- * Otherwise, the stream will not wait for fd input.  It will wait while
- * trying to send data.
- *
- * \note The stream timeouts still need to be set.
- *
- * \return Nothing
- */
-void ast_tcptls_stream_set_exclusive_input(struct ast_tcptls_stream *stream, int exclusive_input);
-
 /*! \brief
  * describes a server instance
  */
 struct ast_tcptls_session_instance {
-	FILE *f;    /*!< fopen/funopen result */
-	int fd;     /*!< the socket returned by accept() */
-	SSL *ssl;   /*!< ssl state */
 	int client;
 	struct ast_sockaddr remote_address;
 	struct ast_tcptls_session_args *parent;
@@ -228,20 +156,12 @@ struct ast_tcptls_session_instance {
 	 * extra data.
 	 */
 	struct ast_str *overflow_buf;
-	/*! ao2 FILE stream cookie object associated with f. */
-	struct ast_tcptls_stream *stream_cookie;
+	/*! ao2 stream object associated with this session. */
+	struct ast_iostream *stream;
 	/*! ao2 object private data of parent->worker_fn */
 	void *private_data;
 };
 
-#if defined(HAVE_FUNOPEN)
-#define HOOK_T int
-#define LEN_T int
-#else
-#define HOOK_T ssize_t
-#define LEN_T size_t
-#endif
-
 /*!
   * \brief attempts to connect and start tcptls session, on error the tcptls_session's
   * ref count is decremented, fd and file are closed, and NULL is returned.
@@ -297,7 +217,4 @@ void ast_ssl_teardown(struct ast_tls_con
  */
 int ast_tls_read_conf(struct ast_tls_config *tls_cfg, struct ast_tcptls_session_args *tls_desc, const char *varname, const char *value);
 
-HOOK_T ast_tcptls_server_read(struct ast_tcptls_session_instance *ser, void *buf, size_t count);
-HOOK_T ast_tcptls_server_write(struct ast_tcptls_session_instance *ser, const void *buf, size_t count);
-
 #endif /* _ASTERISK_TCPTLS_H */
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/term.h 1:16.1.1~dfsg-1/include/asterisk/term.h
--- 1:13.23.1~dfsg-2/include/asterisk/term.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/term.h	2018-12-26 18:45:13.000000000 +0000
@@ -137,10 +137,6 @@ char *term_strip(char *outbuf, const cha
 
 void term_filter_escapes(char *line);
 
-char *term_prompt(char *outbuf, const char *inbuf, int maxout);
-
-const char *term_prep(void);
-
 const char *term_end(void);
 
 const char *term_quit(void);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/threadpool.h 1:16.1.1~dfsg-1/include/asterisk/threadpool.h
--- 1:13.23.1~dfsg-2/include/asterisk/threadpool.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/threadpool.h	2018-12-26 18:45:13.000000000 +0000
@@ -186,7 +186,8 @@ void ast_threadpool_set_size(struct ast_
  * \retval 0 success
  * \retval -1 failure
  */
-int ast_threadpool_push(struct ast_threadpool *pool, int (*task)(void *data), void *data);
+int ast_threadpool_push(struct ast_threadpool *pool, int (*task)(void *data), void *data)
+	attribute_warn_unused_result;
 
 /*!
  * \brief Shut down a threadpool and destroy it
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/time.h 1:16.1.1~dfsg-1/include/asterisk/time.h
--- 1:13.23.1~dfsg-2/include/asterisk/time.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/time.h	2018-12-26 18:45:13.000000000 +0000
@@ -23,10 +23,16 @@
 #ifndef _ASTERISK_TIME_H
 #define _ASTERISK_TIME_H
 
+#include "asterisk/autoconfig.h"
+
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #include "asterisk/inline_api.h"
 
 /* We have to let the compiler learn what types to use for the elements of a
@@ -144,6 +150,34 @@ struct timeval ast_tvnow(void),
 )
 
 /*!
+ * \brief Returns current timespec. Meant to avoid calling ast_tvnow() just to
+ * create a timespec from the timeval it returns.
+ */
+#if defined _POSIX_TIMERS && _POSIX_TIMERS > 0
+AST_INLINE_API(
+struct timespec ast_tsnow(void),
+{
+	struct timespec ts;
+	clock_gettime(CLOCK_REALTIME, &ts);
+	return ts;
+}
+)
+#else
+AST_INLINE_API(
+struct timespec ast_tsnow(void),
+{
+	struct timeval tv = ast_tvnow();
+	struct timespec ts;
+	/* Can't use designated initializer, because it does odd things with
+	 * the AST_INLINE_API macro. Go figure. */
+	ts.tv_sec = tv.tv_sec;
+	ts.tv_nsec = tv.tv_usec * 1000;
+	return ts;
+}
+)
+#endif
+
+/*!
  * \brief Returns the sum of two timevals a + b
  */
 struct timeval ast_tvadd(struct timeval a, struct timeval b);
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/timing.h 1:16.1.1~dfsg-1/include/asterisk/timing.h
--- 1:13.23.1~dfsg-2/include/asterisk/timing.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/timing.h	2018-12-26 18:45:13.000000000 +0000
@@ -92,7 +92,7 @@ struct ast_timing_interface {
  * \retval non-Null handle to be passed to ast_unregister_timing_interface() on success
  * \since 1.6.1
  */
-#define ast_register_timing_interface(i) _ast_register_timing_interface(i, ast_module_info->self)
+#define ast_register_timing_interface(i) _ast_register_timing_interface(i, AST_MODULE_SELF)
 void *_ast_register_timing_interface(struct ast_timing_interface *funcs,
 						 struct ast_module *mod);
 
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/translate.h 1:16.1.1~dfsg-1/include/asterisk/translate.h
--- 1:13.23.1~dfsg-2/include/asterisk/translate.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/translate.h	2018-12-26 18:45:13.000000000 +0000
@@ -121,7 +121,7 @@ enum ast_trans_cost_table {
  *
  * As a minimum, a translator should supply name, srcfmt and dstfmt,
  * the required buf_size (in bytes) and buffer_samples (in samples),
- * and a few callbacks (framein, frameout, sample).
+ * and a few callbacks (framein, frameout, feedback, sample).
  * The outbuf is automatically prepended by AST_FRIENDLY_OFFSET
  * spare bytes so generic routines can place data in there.
  *
@@ -159,6 +159,10 @@ struct ast_translator {
 	                                       /*!< Output frame callback. Generate a frame
 	                                        *   with outbuf content. */
 
+	void (*feedback)(struct ast_trans_pvt *pvt, struct ast_frame *feedback);
+	                                       /*!< Feedback frame callback. Handle
+	                                        *   input frame. */
+
 	void (*destroy)(struct ast_trans_pvt *pvt);
 	                                       /*!< cleanup private data, if needed
 	                                        *   (often unnecessary). */
@@ -231,6 +235,7 @@ struct ast_trans_pvt {
 	 * explicit_dst contains an attribute which describes whether both parties
 	 * want to do forward-error correction (FEC). */
 	struct ast_format *explicit_dst;
+	int interleaved_stereo;     /*!< indicates if samples are in interleaved order, for stereo lin */
 };
 
 /*! \brief generic frameout function */
@@ -249,7 +254,7 @@ struct ast_trans_pvt;
 int __ast_register_translator(struct ast_translator *t, struct ast_module *module);
 
 /*! \brief See \ref __ast_register_translator() */
-#define ast_register_translator(t) __ast_register_translator(t, ast_module_info->self)
+#define ast_register_translator(t) __ast_register_translator(t, AST_MODULE_SELF)
 
 /*!
  * \brief Unregister a translator
@@ -315,7 +320,9 @@ void ast_translator_free_path(struct ast
 /*!
  * \brief translates one or more frames
  * Apply an input frame into the translator and receive zero or one output frames.  Consume
- * determines whether the original frame should be freed
+ * determines whether the original frame should be freed.  In case the frame type is
+ * AST_FRAME_RTCP, the frame is not translated but passed to the translator codecs
+ * via the feedback callback, and a pointer to ast_null_frame is returned after that.
  * \param path tr translator structure to use for translation
  * \param f frame to translate
  * \param consume Whether or not to free the original frame
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/udptl.h 1:16.1.1~dfsg-1/include/asterisk/udptl.h
--- 1:13.23.1~dfsg-2/include/asterisk/udptl.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/udptl.h	2018-12-26 18:45:13.000000000 +0000
@@ -126,13 +126,6 @@ void ast_udptl_setnat(struct ast_udptl *
 
 void ast_udptl_stop(struct ast_udptl *udptl);
 
-void ast_udptl_init(void);
-
-/*!
- * \version 1.6.1 return changed to int
- */
-int ast_udptl_reload(void);
-
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/utils.h 1:16.1.1~dfsg-1/include/asterisk/utils.h
--- 1:13.23.1~dfsg-2/include/asterisk/utils.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/utils.h	2018-12-26 18:45:13.000000000 +0000
@@ -423,13 +423,12 @@ int ast_careful_fwrite(FILE *f, int fd,
  * Thread management support (should be moved to lock.h or a different header)
  */
 
-#define AST_STACKSIZE (((sizeof(void *) * 8 * 8) - 16) * 1024)
+#define AST_STACKSIZE     (((sizeof(void *) * 8 * 8) - 16) * 1024)
+#define AST_STACKSIZE_LOW (((sizeof(void *) * 8 * 2) - 16) * 1024)
 
-#if defined(LOW_MEMORY)
-#define AST_BACKGROUND_STACKSIZE (((sizeof(void *) * 8 * 2) - 16) * 1024)
-#else
-#define AST_BACKGROUND_STACKSIZE AST_STACKSIZE
-#endif
+int ast_background_stacksize(void);
+
+#define AST_BACKGROUND_STACKSIZE ast_background_stacksize()
 
 void ast_register_thread(char *name);
 void ast_unregister_thread(void *id);
@@ -485,287 +484,6 @@ long int ast_random(void);
 #define ast_random_double() (((double)ast_random()) / RAND_MAX)
 
 /*!
- * \brief DEBUG_CHAOS returns failure randomly
- *
- * DEBUG_CHAOS_RETURN(failure); can be used to fake
- * failure of functions such as memory allocation,
- * for the purposes of testing failure handling.
- */
-#ifdef DEBUG_CHAOS
-#ifndef DEBUG_CHAOS_ALLOC_CHANCE
-#define DEBUG_CHAOS_ALLOC_CHANCE 100000
-#endif
-/* Could #define DEBUG_CHAOS_ENABLE ast_fully_booted */
-#ifndef DEBUG_CHAOS_ENABLE
-#define DEBUG_CHAOS_ENABLE 1
-#endif
-#define DEBUG_CHAOS_RETURN(CHANCE, FAILURE) \
-	do { \
-		if ((DEBUG_CHAOS_ENABLE) && (ast_random() % CHANCE == 0)) { \
-			return FAILURE; \
-		} \
-	} while (0)
-#else
-#define DEBUG_CHAOS_RETURN(c,f)
-#endif
-
-
-#ifndef __AST_DEBUG_MALLOC
-#define ast_std_malloc malloc
-#define ast_std_calloc calloc
-#define ast_std_realloc realloc
-#define ast_std_free free
-
-/*!
- * \brief free() wrapper
- *
- * ast_free_ptr should be used when a function pointer for free() needs to be passed
- * as the argument to a function. Otherwise, astmm will cause seg faults.
- */
-#define ast_free free
-#define ast_free_ptr ast_free
-
-#if defined(AST_IN_CORE)
-#define MALLOC_FAILURE_MSG \
-	ast_log_safe(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file)
-#else
-#define MALLOC_FAILURE_MSG \
-	ast_log(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file)
-#endif
-
-/*!
- * \brief A wrapper for malloc()
- *
- * ast_malloc() is a wrapper for malloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The argument and return value are the same as malloc()
- */
-#define ast_malloc(len) \
-	_ast_malloc((len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void * attribute_malloc _ast_malloc(size_t len, const char *file, int lineno, const char *func),
-{
-	void *p;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
-
-	if (!(p = malloc(len))) {
-		MALLOC_FAILURE_MSG;
-	}
-
-	return p;
-}
-)
-
-/*!
- * \brief A wrapper for calloc()
- *
- * ast_calloc() is a wrapper for calloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as calloc()
- */
-#define ast_calloc(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void * attribute_malloc _ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func),
-{
-	void *p;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
-
-	if (!(p = calloc(num, len))) {
-		MALLOC_FAILURE_MSG;
-	}
-
-	return p;
-}
-)
-
-/*!
- * \brief A wrapper for calloc() for use in cache pools
- *
- * ast_calloc_cache() is a wrapper for calloc() that will generate an Asterisk log
- * message in the case that the allocation fails. When memory debugging is in use,
- * the memory allocated by this function will be marked as 'cache' so it can be
- * distinguished from normal memory allocations.
- *
- * The arguments and return value are the same as calloc()
- */
-#define ast_calloc_cache(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-/*!
- * \brief A wrapper for realloc()
- *
- * ast_realloc() is a wrapper for realloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as realloc()
- */
-#define ast_realloc(p, len) \
-	_ast_realloc((p), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void *_ast_realloc(void *p, size_t len, const char *file, int lineno, const char *func),
-{
-	void *newp;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
-
-	if (!(newp = realloc(p, len))) {
-		MALLOC_FAILURE_MSG;
-	}
-
-	return newp;
-}
-)
-
-/*!
- * \brief A wrapper for strdup()
- *
- * ast_strdup() is a wrapper for strdup() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * ast_strdup(), unlike strdup(), can safely accept a NULL argument. If a NULL
- * argument is provided, ast_strdup will return NULL without generating any
- * kind of error log message.
- *
- * The argument and return value are the same as strdup()
- */
-#define ast_strdup(str) \
-	_ast_strdup((str), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-char * attribute_malloc _ast_strdup(const char *str, const char *file, int lineno, const char *func),
-{
-	char *newstr = NULL;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
-
-	if (str) {
-		if (!(newstr = strdup(str))) {
-			MALLOC_FAILURE_MSG;
-		}
-	}
-
-	return newstr;
-}
-)
-
-/*!
- * \brief A wrapper for strndup()
- *
- * ast_strndup() is a wrapper for strndup() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * ast_strndup(), unlike strndup(), can safely accept a NULL argument for the
- * string to duplicate. If a NULL argument is provided, ast_strdup will return
- * NULL without generating any kind of error log message.
- *
- * The arguments and return value are the same as strndup()
- */
-#define ast_strndup(str, len) \
-	_ast_strndup((str), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-char * attribute_malloc _ast_strndup(const char *str, size_t len, const char *file, int lineno, const char *func),
-{
-	char *newstr = NULL;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
-
-	if (str) {
-		if (!(newstr = strndup(str, len))) {
-			MALLOC_FAILURE_MSG;
-		}
-	}
-
-	return newstr;
-}
-)
-
-/*!
- * \brief A wrapper for asprintf()
- *
- * ast_asprintf() is a wrapper for asprintf() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as asprintf()
- */
-#define ast_asprintf(ret, fmt, ...) \
-	_ast_asprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt, __VA_ARGS__)
-
-int __attribute__((format(printf, 5, 6)))
-	_ast_asprintf(char **ret, const char *file, int lineno, const char *func, const char *fmt, ...);
-
-/*!
- * \brief A wrapper for vasprintf()
- *
- * ast_vasprintf() is a wrapper for vasprintf() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as vasprintf()
- */
-#define ast_vasprintf(ret, fmt, ap) \
-	_ast_vasprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, (fmt), (ap))
-
-AST_INLINE_API(
-__attribute__((format(printf, 5, 0)))
-int _ast_vasprintf(char **ret, const char *file, int lineno, const char *func, const char *fmt, va_list ap),
-{
-	int res;
-
-	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, -1);
-
-	res = vasprintf(ret, fmt, ap);
-	if (res < 0) {
-		/*
-		 * *ret is undefined so set to NULL to ensure it is
-		 * initialized to something useful.
-		 */
-		*ret = NULL;
-		MALLOC_FAILURE_MSG;
-	}
-
-	return res;
-}
-)
-
-#endif /* AST_DEBUG_MALLOC */
-
-/*!
-  \brief call __builtin_alloca to ensure we get gcc builtin semantics
-  \param size The size of the buffer we want allocated
-
-  This macro will attempt to allocate memory from the stack.  If it fails
-  you won't get a NULL returned, but a SEGFAULT if you're lucky.
-*/
-#define ast_alloca(size) __builtin_alloca(size)
-
-#if !defined(ast_strdupa) && defined(__GNUC__)
-/*!
- * \brief duplicate a string in memory from the stack
- * \param s The string to duplicate
- *
- * This macro will duplicate the given string.  It returns a pointer to the stack
- * allocatted memory for the new string.
- */
-#define ast_strdupa(s)                                                    \
-	(__extension__                                                    \
-	({                                                                \
-		const char *__old = (s);                                  \
-		size_t __len = strlen(__old) + 1;                         \
-		char *__new = __builtin_alloca(__len);                    \
-		memcpy (__new, __old, __len);                             \
-		__new;                                                    \
-	}))
-#endif
-
-/*!
  * \brief Disable PMTU discovery on a socket
  * \param sock The socket to manipulate
  * \return Nothing
diff -pruN 1:13.23.1~dfsg-2/include/asterisk/vector.h 1:16.1.1~dfsg-1/include/asterisk/vector.h
--- 1:13.23.1~dfsg-2/include/asterisk/vector.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk/vector.h	2018-12-26 18:45:13.000000000 +0000
@@ -51,8 +51,38 @@
 /*! \brief Integer vector definition */
 AST_VECTOR(ast_vector_int, int);
 
-/*! \brief String vector definition */
+/*! \brief String vector definitions */
 AST_VECTOR(ast_vector_string, char *);
+AST_VECTOR(ast_vector_const_string, const char *);
+
+/*! Options to override default processing of ast_vector_string_split. */
+enum ast_vector_string_split_flags {
+	/*! Do not trim whitespace from values. */
+	AST_VECTOR_STRING_SPLIT_NO_TRIM = 0x01,
+	/*! Append empty strings to the vector. */
+	AST_VECTOR_STRING_SPLIT_ALLOW_EMPTY = 0x02,
+};
+
+/*!
+ * \brief Append a string vector by splitting a string.
+ *
+ * \param dest Pointer to an initialized vector.
+ * \param input String buffer to split.
+ * \param delim String delimeter passed to strsep.
+ * \param flags Processing options defined by \ref enum ast_vector_string_split_flags.
+ * \param excludes_cmp NULL or a function like strcmp to exclude duplicate strings.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ *
+ * \note All elements added to the vector are allocated.  The caller is always
+ *       responsible for calling ast_free on each element in the vector even
+ *       after failure.  It's possible for this function to successfully add
+ *       some elements before failing.
+ */
+int ast_vector_string_split(struct ast_vector_string *dest,
+	const char *input, const char *delim, int flags,
+	int (*excludes_cmp)(const char *s1, const char *s2));
 
 /*!
  * \brief Define a vector structure with a read/write lock
@@ -136,7 +166,7 @@ AST_VECTOR(ast_vector_string, char *);
 /*!
  * \brief Deallocates this vector.
  *
- * If any code to free the elements of this vector need to be run, that should
+ * If any code to free the elements of this vector needs to be run, that should
  * be done prior to this call.
  *
  * \param vec Vector to deallocate.
diff -pruN 1:13.23.1~dfsg-2/include/asterisk.h 1:16.1.1~dfsg-1/include/asterisk.h
--- 1:13.23.1~dfsg-2/include/asterisk.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/include/asterisk.h	2018-12-26 18:45:13.000000000 +0000
@@ -19,12 +19,8 @@
 #define _ASTERISK_H
 
 #include "asterisk/autoconfig.h"
-
-#if !defined(NO_MALLOC_DEBUG) && !defined(STANDALONE) && !defined(STANDALONE2) && defined(MALLOC_DEBUG)
-#include "asterisk/astmm.h"
-#endif
-
 #include "asterisk/compat.h"
+#include "asterisk/astmm.h"
 
 /* Default to allowing the umask or filesystem ACLs to determine actual file
  * creation permissions
@@ -54,15 +50,15 @@
 
 #if defined(DEBUG_FD_LEAKS) && !defined(STANDALONE) && !defined(STANDALONE2) && !defined(STANDALONE_AEL)
 /* These includes are all about ordering */
-#include <stdio.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <fcntl.h>
 
 #define	open(a,...)	__ast_fdleak_open(__FILE__,__LINE__,__PRETTY_FUNCTION__, a, __VA_ARGS__)
 #define pipe(a)		__ast_fdleak_pipe(a, __FILE__,__LINE__,__PRETTY_FUNCTION__)
+#define socketpair(a,b,c,d)	__ast_fdleak_socketpair(a, b, c, d, __FILE__,__LINE__,__PRETTY_FUNCTION__)
 #define socket(a,b,c)	__ast_fdleak_socket(a, b, c, __FILE__,__LINE__,__PRETTY_FUNCTION__)
+#define accept(a,b,c)	__ast_fdleak_accept(a, b, c, __FILE__,__LINE__,__PRETTY_FUNCTION__)
 #define close(a)	__ast_fdleak_close(a)
 #define	fopen(a,b)	__ast_fdleak_fopen(a, b, __FILE__,__LINE__,__PRETTY_FUNCTION__)
 #define	fclose(a)	__ast_fdleak_fclose(a)
@@ -74,7 +70,21 @@ extern "C" {
 #endif
 int __ast_fdleak_open(const char *file, int line, const char *func, const char *path, int flags, ...);
 int __ast_fdleak_pipe(int *fds, const char *file, int line, const char *func);
+int __ast_fdleak_socketpair(int domain, int type, int protocol, int sv[2],
+	const char *file, int line, const char *func);
 int __ast_fdleak_socket(int domain, int type, int protocol, const char *file, int line, const char *func);
+int __ast_fdleak_accept(int socket, struct sockaddr *address, socklen_t *address_len,
+	const char *file, int line, const char *func);
+#if defined(HAVE_EVENTFD)
+#include <sys/eventfd.h>
+#define eventfd(a,b)	__ast_fdleak_eventfd(a,b, __FILE__,__LINE__,__PRETTY_FUNCTION__)
+int __ast_fdleak_eventfd(unsigned int initval, int flags, const char *file, int line, const char *func);
+#endif
+#if defined(HAVE_TIMERFD)
+#include <sys/timerfd.h>
+#define timerfd_create(a,b)	__ast_fdleak_timerfd_create(a,b, __FILE__,__LINE__,__PRETTY_FUNCTION__)
+int __ast_fdleak_timerfd_create(int clockid, int flags, const char *file, int line, const char *func);
+#endif
 int __ast_fdleak_close(int fd);
 FILE *__ast_fdleak_fopen(const char *path, const char *mode, const char *file, int line, const char *func);
 int __ast_fdleak_fclose(FILE *ptr);
@@ -161,126 +171,10 @@ int ast_shutting_down(void);
  */
 int ast_shutdown_final(void);
 
-#if !defined(LOW_MEMORY)
-/*!
- * \brief Register the version of a source code file with the core.
- * \param file the source file name
- * \param version the version string (typically a SVN revision keyword string)
- * \return nothing
- *
- * \note As of 13.4.0, the \c version parameter is ignored.
- *
- * This function should not be called directly, but instead the
- * ASTERISK_FILE_VERSION macro should be used to register a file with the core.
- */
-void ast_register_file_version(const char *file, const char *version);
-
-/*!
- * \brief Unregister a source code file from the core.
- * \param file the source file name
- * \return nothing
- *
- * This function should not be called directly, but instead the
- * ASTERISK_FILE_VERSION macro should be used to automatically unregister
- * the file when the module is unloaded.
- */
-void ast_unregister_file_version(const char *file);
-
-/*!
- * \brief Find version for given module name
- * \param file Module name (i.e. chan_sip.so)
- *
-* \note As of 13.4.0, the file version is no longer tracked. As such,
- * if the file exists, the Asterisk version will be returned.
- *
- * \retval NULL if the file doesn't exist.
- * \retval The Asterisk version if the file does exist.
- */
-const char *ast_file_version_find(const char *file);
-
-/*!
- * \brief Complete a source file name
- * \param partial The partial name of the file to look up.
- * \param n The n-th match to return.
- *
- * \retval NULL if there is no match for partial at the n-th position
- * \retval Matching source file name
- *
- * \note A matching source file is allocataed on the heap, and must be
- * free'd by the caller.
- */
-char *ast_complete_source_filename(const char *partial, int n);
-
-/*!
- * \brief Register/unregister a source code file with the core.
- * \param file the source file name
- * \param version the version string (typically a SVN revision keyword string)
- *
- * This macro will place a file-scope constructor and destructor into the
- * source of the module using it; this will cause the version of this file
- * to registered with the Asterisk core (and unregistered) at the appropriate
- * times.
- *
- * Example:
- *
- * \code
- * ASTERISK_FILE_VERSION(__FILE__, "\$Revision\$")
- * \endcode
- *
- * \note The dollar signs above have been protected with backslashes to keep
- * SVN from modifying them in this file; under normal circumstances they would
- * not be present and SVN would expand the Revision keyword into the file's
- * revision number.
- *
- * \deprecated All new files should use ASTERISK_REGISTER_FILE instead.
- * \version 11.22.0 deprecated
- */
 #ifdef MTX_PROFILE
 #define	HAVE_MTX_PROFILE	/* used in lock.h */
-#define ASTERISK_FILE_VERSION(file, version) \
-	static int mtx_prof = -1;       /* profile mutex */	\
-	static void __attribute__((constructor)) __register_file_version(void) \
-	{ \
-		mtx_prof = ast_add_profile("mtx_lock_" file, 0);	\
-		ast_register_file_version(file, version); \
-	} \
-	static void __attribute__((destructor)) __unregister_file_version(void) \
-	{ \
-		ast_unregister_file_version(file); \
-	}
-#else /* !MTX_PROFILE */
-#define ASTERISK_FILE_VERSION(file, version) \
-	static void __attribute__((constructor)) __register_file_version(void) \
-	{ \
-		ast_register_file_version(file, version); \
-	} \
-	static void __attribute__((destructor)) __unregister_file_version(void) \
-	{ \
-		ast_unregister_file_version(file); \
-	}
-#endif /* !MTX_PROFILE */
-#else /* LOW_MEMORY */
-#define ASTERISK_FILE_VERSION(file, x)
-#endif /* LOW_MEMORY */
-
-/*!
- * \since 11.22.0
- * \brief Register/unregister a source code file with the core.
- *
- * This macro will place a file-scope constructor and destructor into the
- * source of the module using it; this will cause the file to be
- * registered with the Asterisk core (and unregistered) at the appropriate
- * times.
- *
- * Example:
- *
- * \code
- * ASTERISK_REGISTER_FILE()
- * \endcode
- */
-#define ASTERISK_REGISTER_FILE() ASTERISK_FILE_VERSION(__FILE__, NULL)
+#endif /* MTX_PROFILE */
 
-#if !defined(LOW_MEMORY)
 /*!
  * \brief support for event profiling
  *
@@ -299,11 +193,6 @@ char *ast_complete_source_filename(const
 int ast_add_profile(const char *, uint64_t scale);
 int64_t ast_profile(int, int64_t);
 int64_t ast_mark(int, int start1_stop0);
-#else /* LOW_MEMORY */
-#define ast_add_profile(a, b) 0
-#define ast_profile(a, b) do { } while (0)
-#define ast_mark(a, b) do { } while (0)
-#endif /* LOW_MEMORY */
 
 /*! \brief
  * Definition of various structures that many asterisk files need,
@@ -323,6 +212,27 @@ struct ast_json;
 #define __stringify_1(x)	#x
 #define __stringify(x)		__stringify_1(x)
 
+#if defined(AST_IN_CORE) \
+	|| (!defined(AST_MODULE_SELF_SYM) \
+		&& (defined(STANDALONE) || defined(STANDALONE2) || defined(AST_NOT_MODULE)))
+
+#define AST_MODULE_SELF NULL
+
+#elif defined(AST_MODULE_SELF_SYM)
+
+/*! Retrieve the 'struct ast_module *' for the current module. */
+#define AST_MODULE_SELF AST_MODULE_SELF_SYM()
+
+struct ast_module;
+/* Internal/forward declaration, AST_MODULE_SELF should be used instead. */
+struct ast_module *AST_MODULE_SELF_SYM(void);
+
+#else
+
+#error "Externally compiled modules must declare AST_MODULE_SELF_SYM."
+
+#endif
+
 /*!
  * \brief Retrieve the PBX UUID
  * \param pbx_uuid A buffer of at least AST_UUID_STR_LEN (36 + 1) size to receive the UUID
diff -pruN 1:13.23.1~dfsg-2/main/abstract_jb.c 1:16.1.1~dfsg-1/main/abstract_jb.c
--- 1:13.23.1~dfsg-2/main/abstract_jb.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/abstract_jb.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/frame.h"
 #include "asterisk/channel.h"
 #include "asterisk/term.h"
diff -pruN 1:13.23.1~dfsg-2/main/acl.c 1:16.1.1~dfsg-1/main/acl.c
--- 1:13.23.1~dfsg-2/main/acl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/acl.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/network.h"
 
 #if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__Darwin__)
@@ -174,7 +172,7 @@ static int get_local_address(struct ast_
 		}
 
 		bufsz = ifn.lifn_count * sizeof(struct lifreq);
-		if (!(buf = malloc(bufsz))) {
+		if (!(buf = ast_malloc(bufsz))) {
 			close(s);
 			return -1;
 		}
@@ -187,7 +185,7 @@ static int get_local_address(struct ast_
 		ifc.lifc_flags = 0;
 		if (ioctl(s, SIOCGLIFCONF, &ifc) < 0) {
 			close(s);
-			free(buf);
+			ast_free(buf);
 			return -1;
 		}
 
@@ -201,7 +199,7 @@ static int get_local_address(struct ast_
 			}
 		}
 
-		free(buf);
+		ast_free(buf);
 #endif /* SOLARIS */
 
 		close(s);
diff -pruN 1:13.23.1~dfsg-2/main/alaw.c 1:16.1.1~dfsg-1/main/alaw.c
--- 1:13.23.1~dfsg-2/main/alaw.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/alaw.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/alaw.h"
 #include "asterisk/logger.h"
 
diff -pruN 1:13.23.1~dfsg-2/main/aoc.c 1:16.1.1~dfsg-1/main/aoc.c
--- 1:13.23.1~dfsg-2/main/aoc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/aoc.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,7 +28,6 @@
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
 
 #include "asterisk/aoc.h"
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/main/app.c 1:16.1.1~dfsg-1/main/app.c
--- 1:13.23.1~dfsg-2/main/app.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/app.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -388,6 +386,7 @@ int ast_app_run_macro(struct ast_channel
 	return res;
 }
 
+/* BUGBUG this is not thread safe. */
 static const struct ast_app_stack_funcs *app_stack_callbacks;
 
 void ast_install_stack_functions(const struct ast_app_stack_funcs *funcs)
@@ -401,16 +400,16 @@ const char *ast_app_expand_sub_args(stru
 	const char *new_args;
 
 	funcs = app_stack_callbacks;
-	if (!funcs || !funcs->expand_sub_args) {
+	if (!funcs || !funcs->expand_sub_args || !ast_module_running_ref(funcs->module)) {
 		ast_log(LOG_WARNING,
 			"Cannot expand 'Gosub(%s)' arguments.  The app_stack module is not available.\n",
 			args);
 		return NULL;
 	}
-	ast_module_ref(funcs->module);
 
 	new_args = funcs->expand_sub_args(chan, args);
 	ast_module_unref(funcs->module);
+
 	return new_args;
 }
 
@@ -420,13 +419,12 @@ int ast_app_exec_sub(struct ast_channel
 	int res;
 
 	funcs = app_stack_callbacks;
-	if (!funcs || !funcs->run_sub) {
+	if (!funcs || !funcs->run_sub || !ast_module_running_ref(funcs->module)) {
 		ast_log(LOG_WARNING,
 			"Cannot run 'Gosub(%s)'.  The app_stack module is not available.\n",
 			sub_args);
 		return -1;
 	}
-	ast_module_ref(funcs->module);
 
 	if (autoservice_chan) {
 		ast_autoservice_start(autoservice_chan);
@@ -2020,7 +2018,7 @@ int ast_app_group_set_channel(struct ast
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&groups, gi, group_list) {
 		if ((gi->chan == chan) && ((ast_strlen_zero(category) && ast_strlen_zero(gi->category)) || (!ast_strlen_zero(gi->category) && !strcasecmp(gi->category, category)))) {
 			AST_RWLIST_REMOVE_CURRENT(group_list);
-			free(gi);
+			ast_free(gi);
 			break;
 		}
 	}
@@ -2028,7 +2026,7 @@ int ast_app_group_set_channel(struct ast
 
 	if (ast_strlen_zero(group)) {
 		/* Enable unsetting the group */
-	} else if ((gi = calloc(1, len))) {
+	} else if ((gi = ast_calloc(1, len))) {
 		gi->chan = chan;
 		gi->group = (char *) gi + sizeof(*gi);
 		strcpy(gi->group, group);
@@ -2162,9 +2160,6 @@ int ast_app_group_list_unlock(void)
 	return AST_RWLIST_UNLOCK(&groups);
 }
 
-#undef ast_app_separate_args
-unsigned int ast_app_separate_args(char *buf, char delim, char **array, int arraylen);
-
 unsigned int __ast_app_separate_args(char *buf, char delim, int remove_chars, char **array, int arraylen)
 {
 	int argc;
@@ -2229,12 +2224,6 @@ unsigned int __ast_app_separate_args(cha
 	return argc;
 }
 
-/* ABI compatible function */
-unsigned int ast_app_separate_args(char *buf, char delim, char **array, int arraylen)
-{
-	return __ast_app_separate_args(buf, delim, 1, array, arraylen);
-}
-
 static enum AST_LOCK_RESULT ast_lock_path_lockfile(const char *path)
 {
 	char *s;
@@ -2304,9 +2293,9 @@ static void path_lock_destroy(struct pat
 		close(obj->fd);
 	}
 	if (obj->path) {
-		free(obj->path);
+		ast_free(obj->path);
 	}
-	free(obj);
+	ast_free(obj);
 }
 
 static enum AST_LOCK_RESULT ast_lock_path_flock(const char *path)
@@ -2350,7 +2339,7 @@ static enum AST_LOCK_RESULT ast_lock_pat
 		return AST_LOCK_FAILURE;
 	}
 	pl->fd = fd;
-	pl->path = strdup(path);
+	pl->path = ast_strdup(path);
 
 	time(&start);
 	while (
@@ -3355,6 +3344,9 @@ int ast_delete_mwi_state_full(const char
 	if (clear_msg) {
 		stasis_publish(mailbox_specific_topic, clear_msg);
 	}
+
+	stasis_topic_pool_delete_topic(mwi_topic_pool, stasis_topic_name(mailbox_specific_topic));
+
 	ao2_cleanup(clear_msg);
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/asterisk.c 1:16.1.1~dfsg-1/main/asterisk.c
--- 1:13.23.1~dfsg-2/main/asterisk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/asterisk.c	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 1999 - 2014, Digium, Inc.
+ * Copyright (C) 1999 - 2018, Digium, Inc.
  *
  * Mark Spencer <markster@digium.com>
  *
@@ -53,7 +53,7 @@
  *
  * \section copyright Copyright and Author
  *
- * Copyright (C) 1999 - 2014, Digium, Inc.
+ * Copyright (C) 1999 - 2018, Digium, Inc.
  * Asterisk is a <a href="http://www.digium.com/en/company/view-policy.php?id=Trademark-Policy">registered trademark</a>
  * of <a rel="nofollow" href="http://www.digium.com">Digium, Inc</a>.
  *
@@ -149,8 +149,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #undef sched_setscheduler
@@ -201,7 +199,6 @@ int daemon(int, int);  /* defined in lib
 #include "asterisk/channel.h"
 #include "asterisk/translate.h"
 #include "asterisk/pickup.h"
-#include "asterisk/features.h"
 #include "asterisk/acl.h"
 #include "asterisk/ulaw.h"
 #include "asterisk/alaw.h"
@@ -211,28 +208,21 @@ int daemon(int, int);  /* defined in lib
 #include "asterisk/term.h"
 #include "asterisk/manager.h"
 #include "asterisk/cdr.h"
-#include "asterisk/cel.h"
 #include "asterisk/pbx.h"
-#include "asterisk/enum.h"
-#include "asterisk/http.h"
-#include "asterisk/udptl.h"
 #include "asterisk/app.h"
 #include "asterisk/lock.h"
 #include "asterisk/utils.h"
 #include "asterisk/file.h"
 #include "asterisk/io.h"
-#include "editline/histedit.h"
 #include "asterisk/config.h"
 #include "asterisk/ast_version.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/devicestate.h"
 #include "asterisk/presencestate.h"
 #include "asterisk/module.h"
-#include "asterisk/dsp.h"
 #include "asterisk/buildinfo.h"
 #include "asterisk/xmldoc.h"
 #include "asterisk/poll-compat.h"
-#include "asterisk/ccss.h"
 #include "asterisk/test.h"
 #include "asterisk/rtp_engine.h"
 #include "asterisk/format.h"
@@ -248,6 +238,7 @@ int daemon(int, int);  /* defined in lib
 #include "asterisk/endpoints.h"
 #include "asterisk/codec.h"
 #include "asterisk/format_cache.h"
+#include "asterisk/media_cache.h"
 #include "asterisk/astdb.h"
 #include "asterisk/options.h"
 
@@ -261,6 +252,12 @@ int daemon(int, int);  /* defined in lib
 				<parameter name="Status">
 					<para>Informational message</para>
 				</parameter>
+				<parameter name="Uptime">
+					<para>Seconds since start</para>
+				</parameter>
+				<parameter name="LastReload">
+					<para>Seconds since last reload</para>
+				</parameter>
 			</syntax>
 		</managerEventInstance>
 	</managerEvent>
@@ -299,7 +296,7 @@ int daemon(int, int);  /* defined in lib
 
 /*! \brief Welcome message when starting a CLI interface */
 #define WELCOME_MESSAGE \
-    ast_verbose("Asterisk %s, Copyright (C) 1999 - 2014, Digium, Inc. and others.\n" \
+    ast_verbose("Asterisk %s, Copyright (C) 1999 - 2018, Digium, Inc. and others.\n" \
                 "Created by Mark Spencer <markster@digium.com>\n" \
                 "Asterisk comes with ABSOLUTELY NO WARRANTY; type 'core show warranty' for details.\n" \
                 "This is free software, with components licensed under the GNU General Public\n" \
@@ -308,6 +305,7 @@ int daemon(int, int);  /* defined in lib
                 "=========================================================================\n", ast_get_version()) \
 
 static int ast_socket = -1;		/*!< UNIX Socket for allowing remote control */
+static int ast_socket_is_sd = 0; /*!< Is socket activation responsible for ast_socket? */
 static int ast_consock = -1;		/*!< UNIX Socket for controlling another asterisk */
 pid_t ast_mainpid;
 struct console {
@@ -391,85 +389,10 @@ static struct {
 	 unsigned int need_reload:1;
 	 unsigned int need_quit:1;
 	 unsigned int need_quit_handler:1;
+	 unsigned int need_el_end:1;
 } sig_flags;
 
 #if !defined(LOW_MEMORY)
-struct registered_file {
-	AST_RWLIST_ENTRY(registered_file) list;
-	const char *file;
-};
-
-static AST_RWLIST_HEAD_STATIC(registered_files, registered_file);
-
-void ast_register_file_version(const char *file, const char *version)
-{
-	struct registered_file *reg;
-
-	reg = ast_calloc(1, sizeof(*reg));
-	if (!reg) {
-		return;
-	}
-
-	reg->file = file;
-	AST_RWLIST_WRLOCK(&registered_files);
-	AST_RWLIST_INSERT_HEAD(&registered_files, reg, list);
-	AST_RWLIST_UNLOCK(&registered_files);
-}
-
-void ast_unregister_file_version(const char *file)
-{
-	struct registered_file *find;
-
-	AST_RWLIST_WRLOCK(&registered_files);
-	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&registered_files, find, list) {
-		if (!strcasecmp(find->file, file)) {
-			AST_RWLIST_REMOVE_CURRENT(list);
-			break;
-		}
-	}
-	AST_RWLIST_TRAVERSE_SAFE_END;
-	AST_RWLIST_UNLOCK(&registered_files);
-
-	if (find) {
-		ast_free(find);
-	}
-}
-
-char *ast_complete_source_filename(const char *partial, int n)
-{
-	struct registered_file *find;
-	size_t len = strlen(partial);
-	int count = 0;
-	char *res = NULL;
-
-	AST_RWLIST_RDLOCK(&registered_files);
-	AST_RWLIST_TRAVERSE(&registered_files, find, list) {
-		if (!strncasecmp(find->file, partial, len) && ++count > n) {
-			res = ast_strdup(find->file);
-			break;
-		}
-	}
-	AST_RWLIST_UNLOCK(&registered_files);
-	return res;
-}
-
-const char *ast_file_version_find(const char *file)
-{
-	struct registered_file *iterator;
-
-	AST_RWLIST_RDLOCK(&registered_files);
-	AST_RWLIST_TRAVERSE(&registered_files, iterator, list) {
-		if (!strcasecmp(iterator->file, file)) {
-			break;
-		}
-	}
-	AST_RWLIST_UNLOCK(&registered_files);
-	if (iterator) {
-		return ast_get_version();
-	}
-	return NULL;
-}
-
 struct thread_list_t {
 	AST_RWLIST_ENTRY(thread_list_t) list;
 	char *name;
@@ -586,6 +509,7 @@ static char *handle_show_settings(struct
 #if !defined(LOW_MEMORY)
 	ast_cli(a->fd, "  Cache media frames:          %s\n", ast_opt_cache_media_frames ? "Enabled" : "Disabled");
 #endif
+	ast_cli(a->fd, "  RTP use dynamic payloads:    %u\n", ast_option_rtpusedynamic);
 
 	if (ast_option_rtpptdynamic == AST_RTP_PT_LAST_REASSIGN) {
 		ast_cli(a->fd, "  RTP dynamic payload types:   %u,%u-%u\n",
@@ -817,12 +741,14 @@ struct profile_data {
 };
 
 static struct profile_data *prof_data;
+#endif /* ! LOW_MEMORY */
 
 /*! \brief allocates a counter with a given name and scale.
  * \return Returns the identifier of the counter.
  */
 int ast_add_profile(const char *name, uint64_t scale)
 {
+#if !defined(LOW_MEMORY)
 	int l = sizeof(struct profile_data);
 	int n = 10;	/* default entries */
 
@@ -849,10 +775,14 @@ int ast_add_profile(const char *name, ui
 	prof_data->e[n].mark = 0;
 	prof_data->e[n].scale = scale;
 	return n;
+#else /* if defined(LOW_MEMORY) */
+	return 0;
+#endif
 }
 
 int64_t ast_profile(int i, int64_t delta)
 {
+#if !defined(LOW_MEMORY)
 	if (!prof_data || i < 0 || i > prof_data->entries)	/* invalid index */
 		return 0;
 	if (prof_data->e[i].scale > 1)
@@ -860,8 +790,12 @@ int64_t ast_profile(int i, int64_t delta
 	prof_data->e[i].value += delta;
 	prof_data->e[i].events++;
 	return prof_data->e[i].value;
+#else /* if defined(LOW_MEMORY) */
+	return 0;
+#endif
 }
 
+#if !defined(LOW_MEMORY)
 /* The RDTSC instruction was introduced on the Pentium processor and is not
  * implemented on certain clones, like the Cyrix 586. Hence, the previous
  * expectation of __i386__ was in error. */
@@ -885,9 +819,11 @@ rdtsc(void)
 	return 0;
 }
 #endif
+#endif /* ! LOW_MEMORY */
 
 int64_t ast_mark(int i, int startstop)
 {
+#if !defined(LOW_MEMORY)
 	if (!prof_data || i < 0 || i > prof_data->entries) /* invalid index */
 		return 0;
 	if (startstop == 1)
@@ -900,8 +836,12 @@ int64_t ast_mark(int i, int startstop)
 		prof_data->e[i].events++;
 	}
 	return prof_data->e[i].mark;
+#else /* if defined(LOW_MEMORY) */
+	return 0;
+#endif
 }
 
+#if !defined(LOW_MEMORY)
 #define DEFINE_PROFILE_MIN_MAX_VALUES min = 0; \
 	max = prof_data->entries;\
 	if  (a->argc > 3) { /* specific entries */ \
@@ -978,86 +918,6 @@ static char *handle_clear_profile(struct
 }
 #undef DEFINE_PROFILE_MIN_MAX_VALUES
 
-/*! \brief CLI command to list module versions */
-static char *handle_show_version_files(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
-{
-#define FORMAT "%-25.25s %-40.40s\n"
-	static const char * const completions[] = { "like", NULL };
-	struct registered_file *iterator;
-	regex_t regexbuf;
-	int havepattern = 0;
-	int havename = 0;
-	int count_files = 0;
-	char *ret = NULL;
-
-	switch (cmd) {
-	case CLI_INIT:
-		e->command = "core show file version";
-		e->usage =
-			"Usage: core show file version [<filename>|like <pattern>]\n"
-			"       Lists the files along with the Asterisk version.\n"
-			"       Optional regular expression pattern is used to filter the file list.\n";
-		return NULL;
-	case CLI_GENERATE:
-		if (a->pos != 4) {
-			return NULL;
-		}
-		ret = ast_cli_complete(a->word, completions, a->n);
-		if (!ret) {
-			ret = ast_complete_source_filename(a->word, a->n - 1);
-		}
-		return ret;
-	}
-
-
-	switch (a->argc) {
-	case 6:
-		if (!strcasecmp(a->argv[4], "like")) {
-			if (regcomp(&regexbuf, a->argv[5], REG_EXTENDED | REG_NOSUB))
-				return CLI_SHOWUSAGE;
-			havepattern = 1;
-		} else
-			return CLI_SHOWUSAGE;
-		break;
-	case 5:
-		if (!strcasecmp(a->argv[4], "like")) {
-			return CLI_SHOWUSAGE;
-		}
-		havename = 1;
-		break;
-	case 4:
-		break;
-	default:
-		return CLI_SHOWUSAGE;
-	}
-
-	ast_cli(a->fd, FORMAT, "File", "Revision");
-	ast_cli(a->fd, FORMAT, "----", "--------");
-	AST_RWLIST_RDLOCK(&registered_files);
-	AST_RWLIST_TRAVERSE(&registered_files, iterator, list) {
-		if (havename && strcasecmp(iterator->file, a->argv[4]))
-			continue;
-
-		if (havepattern && regexec(&regexbuf, iterator->file, 0, NULL, 0))
-			continue;
-
-		ast_cli(a->fd, FORMAT, iterator->file, ast_get_version());
-		count_files++;
-		if (havename)
-			break;
-	}
-	AST_RWLIST_UNLOCK(&registered_files);
-	if (!havename) {
-		ast_cli(a->fd, "%d files listed.\n", count_files);
-	}
-
-	if (havepattern)
-		regfree(&regexbuf);
-
-	return CLI_SUCCESS;
-#undef FORMAT
-}
-
 #endif /* ! LOW_MEMORY */
 
 int ast_pbx_uuid_get(char *pbx_uuid, int length)
@@ -1068,9 +928,25 @@ int ast_pbx_uuid_get(char *pbx_uuid, int
 static void publish_fully_booted(void)
 {
 	struct ast_json *json_object;
+	int uptime = 0;
+	int lastreloaded = 0;
+	struct timeval tmp;
+	struct timeval curtime = ast_tvnow();
+
+	if (ast_startuptime.tv_sec) {
+		tmp = ast_tvsub(curtime, ast_startuptime);
+		uptime = (int) tmp.tv_sec;
+	}
+
+	if (ast_lastreloadtime.tv_sec) {
+		tmp = ast_tvsub(curtime, ast_lastreloadtime);
+		lastreloaded = (int) tmp.tv_sec;
+	}
 
-	json_object = ast_json_pack("{s: s}",
-			"Status", "Fully Booted");
+	json_object = ast_json_pack("{s: s, s: i, s: i}",
+			"Status", "Fully Booted",
+			"Uptime", uptime,
+			"LastReload", lastreloaded);
 	ast_manager_publish_event("FullyBooted", EVENT_FLAG_SYSTEM, json_object);
 	ast_json_unref(json_object);
 }
@@ -1371,14 +1247,15 @@ void ast_console_toggle_mute(int fd, int
 /*!
  * \brief log the string to all attached network console clients
  */
-static void ast_network_puts_mutable(const char *string, int level)
+static void ast_network_puts_mutable(const char *string, int level, int sublevel)
 {
 	int x;
 
 	for (x = 0; x < AST_MAX_CONNECTS; ++x) {
 		if (consoles[x].fd < 0
 			|| consoles[x].mute
-			|| consoles[x].levels[level]) {
+			|| consoles[x].levels[level]
+			|| (level == __LOG_VERBOSE && consoles[x].option_verbose < sublevel)) {
 			continue;
 		}
 		fdprint(consoles[x].p[1], string);
@@ -1391,12 +1268,23 @@ static void ast_network_puts_mutable(con
  */
 void ast_console_puts_mutable(const char *string, int level)
 {
+	ast_console_puts_mutable_full(string, level, 0);
+}
+
+static int console_print(const char *s);
+
+void ast_console_puts_mutable_full(const char *message, int level, int sublevel)
+{
 	/* Send to the root console */
-	fputs(string, stdout);
-	fflush(stdout);
+	console_print(message);
+
+	/* Wake up a poll()ing console */
+	if (ast_opt_console && consolethread != AST_PTHREADT_NULL) {
+		pthread_kill(consolethread, SIGURG);
+	}
 
 	/* Send to any network console clients */
-	ast_network_puts_mutable(string, level);
+	ast_network_puts_mutable(message, level, sublevel);
 }
 
 /*!
@@ -1428,24 +1316,6 @@ void ast_console_puts(const char *string
 	ast_network_puts(string);
 }
 
-static void network_verboser(const char *string)
-{
-	int x;
-	int verb_level;
-
-	/* Send to any network console clients if client verbocity allows. */
-	verb_level = VERBOSE_MAGIC2LEVEL(string);
-	for (x = 0; x < AST_MAX_CONNECTS; ++x) {
-		if (consoles[x].fd < 0
-			|| consoles[x].mute
-			|| consoles[x].levels[__LOG_VERBOSE]
-			|| consoles[x].option_verbose < verb_level) {
-			continue;
-		}
-		fdprint(consoles[x].p[1], string);
-	}
-}
-
 static pthread_t lthread;
 
 /*!
@@ -1686,8 +1556,16 @@ static int ast_makesocket(void)
 	uid_t uid = -1;
 	gid_t gid = -1;
 
-	for (x = 0; x < AST_MAX_CONNECTS; x++)
+	for (x = 0; x < AST_MAX_CONNECTS; x++) {
 		consoles[x].fd = -1;
+	}
+
+	if (ast_socket_is_sd) {
+		ast_socket = ast_sd_get_fd_un(SOCK_STREAM, ast_config_AST_SOCKET);
+
+		goto start_lthread;
+	}
+
 	unlink(ast_config_AST_SOCKET);
 	ast_socket = socket(PF_LOCAL, SOCK_STREAM, 0);
 	if (ast_socket < 0) {
@@ -1711,16 +1589,20 @@ static int ast_makesocket(void)
 		ast_socket = -1;
 		return -1;
 	}
-	if (ast_register_verbose(network_verboser)) {
-		ast_log(LOG_WARNING, "Unable to register network verboser?\n");
-	}
 
+start_lthread:
 	if (ast_pthread_create_background(&lthread, NULL, listener, NULL)) {
 		ast_log(LOG_WARNING, "Unable to create listener thread.\n");
 		close(ast_socket);
 		return -1;
 	}
 
+	if (ast_socket_is_sd) {
+		/* owner/group/permissions are set by systemd, we might not even have access
+		 * to socket file so leave it alone */
+		return 0;
+	}
+
 	if (!ast_strlen_zero(ast_config_AST_CTL_OWNER)) {
 		struct passwd *pw;
 		if ((pw = getpwnam(ast_config_AST_CTL_OWNER)) == NULL)
@@ -1790,15 +1672,19 @@ static struct sigaction urg_handler = {
 
 static void _hup_handler(int num)
 {
-	int a = 0, save_errno = errno;
-	printf("Received HUP signal -- Reloading configs\n");
-	if (restartnow)
+	int save_errno = errno;
+
+	if (restartnow) {
+		if (el) {
+			el_end(el);
+		}
 		execvp(_argv[0], _argv);
+	}
+
+	printf("Received HUP signal -- Reloading configs\n");
 	sig_flags.need_reload = 1;
-	if (sig_alert_pipe[1] != -1) {
-		if (write(sig_alert_pipe[1], &a, sizeof(a)) < 0) {
-			fprintf(stderr, "hup_handler: write() failed: %s\n", strerror(errno));
-		}
+	if (ast_alertpipe_write(sig_alert_pipe)) {
+		fprintf(stderr, "hup_handler: write() failed: %s\n", strerror(errno));
 	}
 	errno = save_errno;
 }
@@ -2136,10 +2022,9 @@ static void really_quit(int num, shutdow
 			if (el_hist != NULL) {
 				history_end(el_hist);
 			}
-		} else if (mon_sig_flags == pthread_self()) {
-			if (consolethread != AST_PTHREADT_NULL) {
-				pthread_kill(consolethread, SIGURG);
-			}
+		} else if (!restart) {
+			sig_flags.need_el_end = 1;
+			pthread_kill(consolethread, SIGURG);
 		}
 	}
 	active_channels = ast_active_channels();
@@ -2165,7 +2050,9 @@ static void really_quit(int num, shutdow
 		pthread_cancel(lthread);
 		close(ast_socket);
 		ast_socket = -1;
-		unlink(ast_config_AST_SOCKET);
+		if (!ast_socket_is_sd) {
+			unlink(ast_config_AST_SOCKET);
+		}
 		pthread_kill(lthread, SIGURG);
 		pthread_join(lthread, NULL);
 	}
@@ -2173,10 +2060,7 @@ static void really_quit(int num, shutdow
 		close(ast_consock);
 	if (!ast_opt_remote)
 		unlink(ast_config_AST_PID);
-	if (sig_alert_pipe[0])
-		close(sig_alert_pipe[0]);
-	if (sig_alert_pipe[1])
-		close(sig_alert_pipe[1]);
+	ast_alertpipe_close(sig_alert_pipe);
 	printf("%s", term_quit());
 	if (restart) {
 		int i;
@@ -2212,12 +2096,9 @@ static void really_quit(int num, shutdow
 
 static void __quit_handler(int num)
 {
-	int a = 0;
 	sig_flags.need_quit = 1;
-	if (sig_alert_pipe[1] != -1) {
-		if (write(sig_alert_pipe[1], &a, sizeof(a)) < 0) {
-			fprintf(stderr, "quit_handler: write() failed: %s\n", strerror(errno));
-		}
+	if (ast_alertpipe_write(sig_alert_pipe)) {
+		fprintf(stderr, "quit_handler: write() failed: %s\n", strerror(errno));
 	}
 	/* There is no need to restore the signal handler here, since the app
 	 * is going to exit */
@@ -2275,7 +2156,7 @@ static int console_state_init(void *ptr)
 
 AST_THREADSTORAGE_CUSTOM(console_state, console_state_init, ast_free_ptr);
 
-static int console_print(const char *s, int local)
+static int console_print(const char *s)
 {
 	struct console_state_data *state =
 		ast_threadstorage_get(&console_state, sizeof(*state));
@@ -2346,18 +2227,6 @@ static int console_print(const char *s,
 	return res;
 }
 
-static void console_verboser(const char *s)
-{
-	if (!console_print(s, 1)) {
-		return;
-	}
-
-	/* Wake up a poll()ing console */
-	if (ast_opt_console && consolethread != AST_PTHREADT_NULL) {
-		pthread_kill(consolethread, SIGURG);
-	}
-}
-
 static int ast_all_zeros(const char *s)
 {
 	while (*s) {
@@ -2705,7 +2574,6 @@ static struct ast_cli_entry cli_asterisk
 	AST_CLI_DEFINE(handle_version, "Display version info"),
 	AST_CLI_DEFINE(handle_bang, "Execute a shell command"),
 #if !defined(LOW_MEMORY)
-	AST_CLI_DEFINE(handle_show_version_files, "List versions of files used to build Asterisk"),
 	AST_CLI_DEFINE(handle_show_threads, "Show running threads"),
 #if defined(HAVE_SYSINFO) || defined(HAVE_SYSCTL)
 	AST_CLI_DEFINE(handle_show_sysinfo, "Show System Information"),
@@ -2773,7 +2641,7 @@ static int ast_el_read_char(EditLine *ed
 		}
 		res = ast_poll(fds, max, -1);
 		if (res < 0) {
-			if (sig_flags.need_quit || sig_flags.need_quit_handler) {
+			if (sig_flags.need_quit || sig_flags.need_quit_handler || sig_flags.need_el_end) {
 				break;
 			}
 			if (errno == EINTR) {
@@ -2835,7 +2703,7 @@ static int ast_el_read_char(EditLine *ed
 				}
 			}
 
-			console_print(buf, 0);
+			console_print(buf);
 
 			if ((res < EL_BUF_SIZE - 1) && ((buf[res-1] == '\n') || (res >= 2 && buf[res-2] == '\n'))) {
 				*cp = CHAR_TO_LIBEDIT(CC_REFRESH);
@@ -2982,40 +2850,25 @@ static char *cli_prompt(EditLine *editli
 static struct ast_vector_string *ast_el_strtoarr(char *buf)
 {
 	char *retstr;
-	char *bestmatch;
 	struct ast_vector_string *vec = ast_calloc(1, sizeof(*vec));
 
 	if (!vec) {
 		return NULL;
 	}
 
-	/* bestmatch must not be deduplicated */
-	bestmatch = strsep(&buf, " ");
-	if (!bestmatch || !strcmp(bestmatch, AST_CLI_COMPLETE_EOF)) {
-		goto vector_cleanup;
-	}
-
 	while ((retstr = strsep(&buf, " "))) {
 		if (!strcmp(retstr, AST_CLI_COMPLETE_EOF)) {
 			break;
 		}
 
-		/* Older daemons sent duplicates. */
-		if (AST_VECTOR_GET_CMP(vec, retstr, !strcasecmp)) {
-			continue;
-		}
-
 		retstr = ast_strdup(retstr);
-		/* Older daemons sent unsorted. */
-		if (!retstr || AST_VECTOR_ADD_SORTED(vec, retstr, strcasecmp)) {
+		if (!retstr || AST_VECTOR_APPEND(vec, retstr)) {
 			ast_free(retstr);
 			goto vector_cleanup;
 		}
 	}
 
-	bestmatch = ast_strdup(bestmatch);
-	if (!bestmatch || AST_VECTOR_INSERT_AT(vec, 0, bestmatch)) {
-		ast_free(bestmatch);
+	if (!AST_VECTOR_SIZE(vec)) {
 		goto vector_cleanup;
 	}
 
@@ -3316,7 +3169,7 @@ static void ast_remotecontrol(char *data
 		sprintf(tmp, "%s%s", prefix, data);
 		if (write(ast_consock, tmp, strlen(tmp) + 1) < 0) {
 			ast_log(LOG_ERROR, "write() failed: %s\n", strerror(errno));
-			if (sig_flags.need_quit || sig_flags.need_quit_handler) {
+			if (sig_flags.need_quit || sig_flags.need_quit_handler || sig_flags.need_el_end) {
 				return;
 			}
 		}
@@ -3348,7 +3201,7 @@ static void ast_remotecontrol(char *data
 			char buffer[512] = "", *curline = buffer, *nextline;
 			int not_written = 1;
 
-			if (sig_flags.need_quit || sig_flags.need_quit_handler) {
+			if (sig_flags.need_quit || sig_flags.need_quit_handler || sig_flags.need_el_end) {
 				break;
 			}
 
@@ -3408,7 +3261,7 @@ static void ast_remotecontrol(char *data
 	for (;;) {
 		ebuf = (char *)el_gets(el, &num);
 
-		if (sig_flags.need_quit || sig_flags.need_quit_handler) {
+		if (sig_flags.need_quit || sig_flags.need_quit_handler || sig_flags.need_el_end) {
 			break;
 		}
 
@@ -3438,7 +3291,7 @@ static int show_version(void)
 
 static int show_cli_help(void)
 {
-	printf("Asterisk %s, Copyright (C) 1999 - 2014, Digium, Inc. and others.\n", ast_get_version());
+	printf("Asterisk %s, Copyright (C) 1999 - 2018, Digium, Inc. and others.\n", ast_get_version());
 	printf("Usage: asterisk [OPTIONS]\n");
 	printf("Valid Options:\n");
 	printf("   -V              Display version number and exit\n");
@@ -3446,7 +3299,7 @@ static int show_cli_help(void)
 	printf("   -G <group>      Run as a group other than the caller\n");
 	printf("   -U <user>       Run as a user other than the caller\n");
 	printf("   -c              Provide console CLI\n");
-	printf("   -d              Enable extra debugging\n");
+	printf("   -d              Increase debugging (multiple d's = more debugging)\n");
 #if HAVE_WORKING_FORK
 	printf("   -f              Do not fork\n");
 	printf("   -F              Always fork\n");
@@ -3469,7 +3322,7 @@ static int show_cli_help(void)
 	printf("                   of output to the CLI\n");
 	printf("   -v              Increase verbosity (multiple v's = more verbose)\n");
 	printf("   -x <cmd>        Execute command <cmd> (implies -r)\n");
-	printf("   -X              Execute includes by default (allows #exec in asterisk.conf)\n");
+	printf("   -X              Enable use of #exec in asterisk.conf\n");
 	printf("   -W              Adjust terminal colors to compensate for a light background\n");
 	printf("\n");
 	return 0;
@@ -3513,7 +3366,7 @@ static void *monitor_sig_flags(void *unu
 {
 	for (;;) {
 		struct pollfd p = { sig_alert_pipe[0], POLLIN, 0 };
-		int a;
+
 		ast_poll(&p, 1, -1);
 		if (sig_flags.need_reload) {
 			sig_flags.need_reload = 0;
@@ -3528,8 +3381,7 @@ static void *monitor_sig_flags(void *unu
 				quit_handler(0, SHUTDOWN_NORMAL, 0);
 			}
 		}
-		if (read(sig_alert_pipe[0], &a, sizeof(a)) != sizeof(a)) {
-		}
+		ast_alertpipe_read(sig_alert_pipe);
 	}
 
 	return NULL;
@@ -3617,10 +3469,6 @@ static void env_init(void)
 static void print_intro_message(const char *runuser, const char *rungroup)
 {
  	if (ast_opt_console || option_verbose || (ast_opt_remote && !ast_opt_exec)) {
-		if (ast_register_verbose(console_verboser)) {
-			fprintf(stderr, "Unable to register console verboser?\n");
-			return;
-		}
 		WELCOME_MESSAGE;
 		if (runuser) {
 			ast_verbose("Running as user '%s'\n", runuser);
@@ -3643,8 +3491,8 @@ int main(int argc, char *argv[])
 	int x;
 	int isroot = 1, rundir_exists = 0;
 	const char *runuser = NULL, *rungroup = NULL;
-	char *remotesock = NULL;
 	struct rlimit l;
+	static const char *getopt_settings = "BC:cde:FfG:ghIiL:M:mnpqRrs:TtU:VvWXx:";
 
 	/* Remember original args for restart */
 	if (argc > ARRAY_LEN(_argv) - 1) {
@@ -3664,11 +3512,53 @@ int main(int argc, char *argv[])
 	}
 	ast_mainpid = getpid();
 
+	/* Process command-line options that effect asterisk.conf load. */
+	while ((c = getopt(argc, argv, getopt_settings)) != -1) {
+		switch (c) {
+		case 'X':
+			ast_set_flag(&ast_options, AST_OPT_FLAG_EXEC_INCLUDES);
+			break;
+		case 'C':
+			set_asterisk_conf_path(optarg);
+			break;
+		case 'd':
+			option_debug++;
+			break;
+		case 'h':
+			show_cli_help();
+			exit(0);
+		case 'R':
+		case 'r':
+		case 'x':
+			/* ast_opt_remote is checked during config load.  This is only part of what
+			 * these options do, see the second loop for the rest of the actions. */
+			ast_set_flag(&ast_options, AST_OPT_FLAG_REMOTE);
+			break;
+		case 'V':
+			show_version();
+			exit(0);
+		case 'v':
+			option_verbose++;
+			break;
+		case '?':
+			exit(1);
+		}
+	}
+
+	/* Initialize env so it is available if #exec is used in asterisk.conf. */
+	env_init();
+
+	load_asterisk_conf();
+
+	/* Update env to include any systemname that was set. */
+	env_init();
+
 	/*! \brief Check for options
 	 *
 	 * \todo Document these options
 	 */
-	while ((c = getopt(argc, argv, "BC:cde:FfG:ghIiL:M:mnpqRrs:TtU:VvWXx:")) != -1) {
+	optind = 1;
+	while ((c = getopt(argc, argv, getopt_settings)) != -1) {
 		/*!\note Please keep the ordering here to alphabetical, capital letters
 		 * first.  This will make it easier in the future to select unused
 		 * option flags for new features. */
@@ -3678,18 +3568,16 @@ int main(int argc, char *argv[])
 			ast_clear_flag(&ast_options, AST_OPT_FLAG_LIGHT_BACKGROUND);
 			break;
 		case 'X':
-			ast_set_flag(&ast_options, AST_OPT_FLAG_EXEC_INCLUDES);
+			/* The command-line -X option enables #exec for asterisk.conf only. */
 			break;
 		case 'C':
-			set_asterisk_conf_path(optarg);
-			ast_set_flag(&ast_options, AST_OPT_FLAG_OVERRIDE_CONFIG);
+			/* already processed. */
 			break;
 		case 'c':
 			ast_set_flag(&ast_options, AST_OPT_FLAG_NO_FORK | AST_OPT_FLAG_CONSOLE);
 			break;
 		case 'd':
-			option_debug++;
-			ast_set_flag(&ast_options, AST_OPT_FLAG_NO_FORK);
+			/* already processed. */
 			break;
 #if defined(HAVE_SYSINFO)
 		case 'e':
@@ -3713,8 +3601,8 @@ int main(int argc, char *argv[])
 			ast_set_flag(&ast_options, AST_OPT_FLAG_DUMP_CORE);
 			break;
 		case 'h':
-			show_cli_help();
-			exit(0);
+			/* already processed. */
+			break;
 		case 'I':
 			fprintf(stderr,
 				"NOTICE: The -I option is no longer needed.\n"
@@ -3752,7 +3640,9 @@ int main(int argc, char *argv[])
 			ast_set_flag(&ast_options, AST_OPT_FLAG_NO_FORK | AST_OPT_FLAG_REMOTE);
 			break;
 		case 's':
-			remotesock = ast_strdupa(optarg);
+			if (ast_opt_remote) {
+				set_socket_path(optarg);
+			}
 			break;
 		case 'T':
 			ast_set_flag(&ast_options, AST_OPT_FLAG_TIMESTAMP);
@@ -3764,11 +3654,8 @@ int main(int argc, char *argv[])
 			runuser = ast_strdupa(optarg);
 			break;
 		case 'V':
-			show_version();
-			exit(0);
 		case 'v':
-			option_verbose++;
-			ast_set_flag(&ast_options, AST_OPT_FLAG_NO_FORK);
+			/* already processed. */
 			break;
 		case 'W': /* White background */
 			ast_set_flag(&ast_options, AST_OPT_FLAG_LIGHT_BACKGROUND);
@@ -3782,7 +3669,8 @@ int main(int argc, char *argv[])
 			xarg = ast_strdupa(optarg);
 			break;
 		case '?':
-			exit(1);
+			/* already processed. */
+			break;
 		}
 	}
 
@@ -3796,13 +3684,6 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	load_asterisk_conf();
-	env_init();
-
-	if (ast_opt_remote && remotesock != NULL) {
-		set_socket_path(remotesock);
-	}
-
 	if (!ast_language_is_prefix && !ast_opt_remote) {
 		fprintf(stderr, "The 'languageprefix' option in asterisk.conf is deprecated; in a future release it will be removed, and your sound files will need to be organized in the 'new style' language layout.\n");
 	}
@@ -3999,7 +3880,12 @@ int main(int argc, char *argv[])
 	/* Initial value of the maximum active system verbosity level. */
 	ast_verb_sys_level = option_verbose;
 
-	if (ast_tryconnect()) {
+	if (ast_sd_get_fd_un(SOCK_STREAM, ast_config_AST_SOCKET) > 0) {
+		ast_socket_is_sd = 1;
+	}
+
+	/* DO NOT perform check for existing daemon if systemd has CLI socket activation */
+	if (!ast_socket_is_sd && ast_tryconnect()) {
 		/* One is already running */
 		if (ast_opt_remote) {
 			multi_thread_safe = 1;
@@ -4054,6 +3940,9 @@ static void asterisk_daemon(int isroot,
 	char *buf;
 	char pbx_uuid[AST_UUID_STR_LEN];
 
+	/* Set time as soon as possible */
+	ast_lastreloadtime = ast_startuptime = ast_tvnow();
+
 	/* This needs to remain as high up in the initial start up as possible.
 	 * daemon causes a fork to occur, which has all sorts of unintended
 	 * consequences for things that interact with threads.  This call *must*
@@ -4078,9 +3967,7 @@ static void asterisk_daemon(int isroot,
 	 * an Asterisk instance, and that there isn't one already running. */
 	multi_thread_safe = 1;
 
-#if defined(__AST_DEBUG_MALLOC)
-	__ast_mm_init_phase_1();
-#endif	/* defined(__AST_DEBUG_MALLOC) */
+	load_astmm_phase_1();
 
 	/* Check whether high prio was succesfully set by us or some
 	 * other incantation. */
@@ -4143,12 +4030,7 @@ static void asterisk_daemon(int isroot,
 
 	print_intro_message(runuser, rungroup);
 
-	if (ast_opt_console) {
-		ast_verb(0, "[ Initializing Custom Configuration Options ]\n");
-	}
-	/* custom config setup */
 	register_config_cli();
-	read_config_maps();
 
 	check_init(astobj2_init(), "AO2");
 	check_init(ast_named_locks_init(), "Named Locks");
@@ -4225,10 +4107,9 @@ static void asterisk_daemon(int isroot,
 	srand((unsigned int) getpid() + (unsigned int) time(NULL));
 	initstate((unsigned int) getpid() * 65536 + (unsigned int) time(NULL), randompool, sizeof(randompool));
 
-	check_init(init_logger(), "Logger");
-
 	threadstorage_init();
 
+	check_init(init_logger(), "Logger");
 	check_init(ast_rtp_engine_init(), "RTP Engine");
 
 	ast_autoservice_init();
@@ -4240,7 +4121,6 @@ static void asterisk_daemon(int isroot,
 	check_init(app_init(), "App Core");
 	check_init(devstate_init(), "Device State Core");
 	check_init(ast_msg_init(), "Messaging API");
-	check_init(ast_data_init(), "Data Retrieval API");
 	check_init(ast_channels_init(), "Channel");
 	check_init(ast_endpoint_init(), "Endpoints");
 	check_init(ast_pickup_init(), "Call Pickup");
@@ -4248,20 +4128,9 @@ static void asterisk_daemon(int isroot,
 	check_init(ast_parking_stasis_init(), "Parking Core");
 	check_init(ast_device_state_engine_init(), "Device State Engine");
 	check_init(ast_presence_state_engine_init(), "Presence State Engine");
-	check_init(load_modules(1), "Module Preload");
-	check_init(ast_features_init(), "Call Features");
-	check_init(dnsmgr_init(), "DNS manager");
+	check_init(dns_core_init(), "DNS Resolver Core");
+	check_init(ast_dns_system_resolver_init(), "Default DNS resolver");
 	check_init(ast_security_stasis_init(), "Security Stasis Topic and Events");
-	check_init(ast_named_acl_init(), "Named ACL system");
-
-	ast_http_init();		/* Start the HTTP server, if needed */
-
-	check_init(ast_indications_init(), "Indication Tone Handling");
-	check_init(ast_cdr_engine_init(), "CDR Engine");
-
-	ast_dsp_init();
-	ast_udptl_init();
-
 	check_init(ast_image_init(), "Image");
 	check_init(ast_file_init(), "Generic File Format Support");
 	check_init(load_pbx(), "load_pbx");
@@ -4272,17 +4141,16 @@ static void asterisk_daemon(int isroot,
 	check_init(load_pbx_app(), "PBX Application Support");
 	check_init(load_pbx_hangup_handler(), "PBX Hangup Handler Support");
 	check_init(ast_local_init(), "Local Proxy Channel Driver");
-	check_init(ast_cel_engine_init(), "CEL Engine");
-	check_init(init_manager(), "Asterisk Manager Interface");
-	check_init(ast_enum_init(), "ENUM Support");
-	check_init(ast_cc_init(), "Call Completion Supplementary Services");
-	check_init(load_modules(0), "Module");
+
+	/* We should avoid most config loads before this point as they can't use realtime. */
+	check_init(load_modules(), "Module");
 
 	/*
-	 * This is initialized after the dynamic modules load to avoid repeatedly
-	 * reindexing sounds for every format module load.
+	 * This has to load after the dynamic modules load, as items in the media
+	 * cache can't be constructed from items in the AstDB without their
+	 * bucket backends.
 	 */
-	check_init(ast_sounds_index_init(), "Sounds Indexer");
+	check_init(ast_media_cache_init(), "Media Cache");
 
 	/* loads the cli_permissoins.conf file needed to implement cli restrictions. */
 	ast_cli_perms_init(0);
@@ -4295,9 +4163,7 @@ static void asterisk_daemon(int isroot,
 		consolethread = pthread_self();
 	}
 
-	if (pipe(sig_alert_pipe)) {
-		sig_alert_pipe[0] = sig_alert_pipe[1] = -1;
-	}
+	ast_alertpipe_init(sig_alert_pipe);
 
 	ast_process_pending_reloads();
 
@@ -4306,11 +4172,8 @@ static void asterisk_daemon(int isroot,
 
 	pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
 
-#if defined(__AST_DEBUG_MALLOC)
-	__ast_mm_init_phase_2();
-#endif	/* defined(__AST_DEBUG_MALLOC) */
+	load_astmm_phase_2();
 
-	ast_lastreloadtime = ast_startuptime = ast_tvnow();
 	ast_cli_register_multiple(cli_asterisk_shutdown, ARRAY_LEN(cli_asterisk_shutdown));
 	ast_cli_register_multiple(cli_asterisk, ARRAY_LEN(cli_asterisk));
 	ast_register_cleanup(main_atexit);
@@ -4341,6 +4204,12 @@ static void asterisk_daemon(int isroot,
 		el_set(el, EL_GETCFN, ast_el_read_char);
 
 		for (;;) {
+			if (sig_flags.need_el_end) {
+				el_end(el);
+
+				return;
+			}
+
 			if (sig_flags.need_quit || sig_flags.need_quit_handler) {
 				quit_handler(0, SHUTDOWN_FAST, 0);
 				break;
diff -pruN 1:13.23.1~dfsg-2/main/asterisk.exports.in 1:16.1.1~dfsg-1/main/asterisk.exports.in
--- 1:13.23.1~dfsg-2/main/asterisk.exports.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/asterisk.exports.in	2018-12-26 18:45:13.000000000 +0000
@@ -22,11 +22,8 @@
 		LINKER_SYMBOL_PREFIXio_*;
 		LINKER_SYMBOL_PREFIXjb_*;
 		LINKER_SYMBOL_PREFIXaes_*;
-		LINKER_SYMBOL_PREFIXconfig_*;
 		LINKER_SYMBOL_PREFIXtdd_*;
 		LINKER_SYMBOL_PREFIXterm_*;
-		LINKER_SYMBOL_PREFIXchannelreloadreason2txt;
-		LINKER_SYMBOL_PREFIXdevstate2str;
 		LINKER_SYMBOL_PREFIXstrsep;
 		LINKER_SYMBOL_PREFIXsetenv;
 		LINKER_SYMBOL_PREFIXstasis_*;
diff -pruN 1:13.23.1~dfsg-2/main/ast_expr2.c 1:16.1.1~dfsg-1/main/ast_expr2.c
--- 1:13.23.1~dfsg-2/main/ast_expr2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ast_expr2.c	2018-12-26 18:45:13.000000000 +0000
@@ -91,16 +91,14 @@
  * $FreeBSD: src/bin/expr/expr.y,v 1.16 2000/07/22 10:59:36 se Exp $
  */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
 #include <sys/cdefs.h>
 #include <sys/types.h>
 #include <stdio.h>
 
-#if !defined(STANDALONE) && !defined(STANDALONE2)	\
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#else
+#if defined(STANDALONE) || defined(STANDALONE2)
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
 #endif
diff -pruN 1:13.23.1~dfsg-2/main/ast_expr2f.c 1:16.1.1~dfsg-1/main/ast_expr2f.c
--- 1:13.23.1~dfsg-2/main/ast_expr2f.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ast_expr2f.c	2018-12-26 18:45:13.000000000 +0000
@@ -1,3 +1,4 @@
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
 #line 2 "ast_expr2f.c"
@@ -519,9 +520,7 @@ static yyconst flex_int16_t yy_chk[159]
 #include <sys/types.h>
 #include <stdio.h>
 
-#if !defined(STANDALONE)
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#else
+#if defined(STANDALONE)
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
 #endif
diff -pruN 1:13.23.1~dfsg-2/main/ast_expr2.fl 1:16.1.1~dfsg-1/main/ast_expr2.fl
--- 1:13.23.1~dfsg-2/main/ast_expr2.fl	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ast_expr2.fl	2018-12-26 18:45:13.000000000 +0000
@@ -25,9 +25,7 @@
 #include <sys/types.h>
 #include <stdio.h>
 
-#if !defined(STANDALONE)
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#else
+#if defined(STANDALONE)
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
 #endif
diff -pruN 1:13.23.1~dfsg-2/main/ast_expr2.y 1:16.1.1~dfsg-1/main/ast_expr2.y
--- 1:13.23.1~dfsg-2/main/ast_expr2.y	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ast_expr2.y	2018-12-26 18:45:13.000000000 +0000
@@ -12,16 +12,14 @@
  * $FreeBSD: src/bin/expr/expr.y,v 1.16 2000/07/22 10:59:36 se Exp $
  */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
 #include <sys/cdefs.h>
 #include <sys/types.h>
 #include <stdio.h>
 
-#if !defined(STANDALONE) && !defined(STANDALONE2)	\
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#else
+#if defined(STANDALONE) || defined(STANDALONE2)
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
 #endif
diff -pruN 1:13.23.1~dfsg-2/main/astfd.c 1:16.1.1~dfsg-1/main/astfd.c
--- 1:13.23.1~dfsg-2/main/astfd.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astfd.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #ifdef DEBUG_FD_LEAKS
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <string.h>
 #include <stddef.h>
@@ -46,6 +44,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/lock.h"
 #include "asterisk/strings.h"
 #include "asterisk/unaligned.h"
+#include "asterisk/localtime.h"
+#include "asterisk/time.h"
 
 static struct fdleaks {
 	const char *callname;
@@ -53,7 +53,8 @@ static struct fdleaks {
 	unsigned int isopen:1;
 	char file[40];
 	char function[25];
-	char callargs[60];
+	char callargs[100];
+	struct timeval now;
 } fdleaks[1024] = { { "", }, };
 
 /* COPY does ast_copy_string(dst, src, sizeof(dst)), except:
@@ -79,6 +80,7 @@ static struct fdleaks {
 #define STORE_COMMON(offset, name, ...)     \
 	do { \
 		struct fdleaks *tmp = &fdleaks[offset]; \
+		tmp->now = ast_tvnow(); \
 		COPY(tmp->file, file);      \
 		tmp->line = line;           \
 		COPY(tmp->function, func);  \
@@ -130,6 +132,19 @@ int __ast_fdleak_open(const char *file,
 	return res;
 }
 
+#undef accept
+int __ast_fdleak_accept(int socket, struct sockaddr *address, socklen_t *address_len,
+	const char *file, int line, const char *func)
+{
+	int res = accept(socket, address, address_len);
+
+	if (res >= 0) {
+		STORE_COMMON(res, "accept", "{%d}", socket);
+	}
+
+	return res;
+}
+
 #undef pipe
 int __ast_fdleak_pipe(int *fds, const char *file, int line, const char *func)
 {
@@ -145,6 +160,52 @@ int __ast_fdleak_pipe(int *fds, const ch
 	return 0;
 }
 
+#undef socketpair
+int __ast_fdleak_socketpair(int domain, int type, int protocol, int sv[2],
+	const char *file, int line, const char *func)
+{
+	int i, res = socketpair(domain, type, protocol, sv);
+	if (res) {
+		return res;
+	}
+	for (i = 0; i < 2; i++) {
+		if (sv[i] > -1 && sv[i] < ARRAY_LEN(fdleaks)) {
+			STORE_COMMON(sv[i], "socketpair", "{%d,%d}", sv[0], sv[1]);
+		}
+	}
+	return 0;
+}
+
+#if defined(HAVE_EVENTFD)
+#undef eventfd
+#include <sys/eventfd.h>
+int __ast_fdleak_eventfd(unsigned int initval, int flags, const char *file, int line, const char *func)
+{
+	int res = eventfd(initval, flags);
+
+	if (res >= 0) {
+		STORE_COMMON(res, "eventfd", "{%d}", res);
+	}
+
+	return res;
+}
+#endif
+
+#if defined(HAVE_TIMERFD)
+#undef timerfd_create
+#include <sys/timerfd.h>
+int __ast_fdleak_timerfd_create(int clockid, int flags, const char *file, int line, const char *func)
+{
+	int res = timerfd_create(clockid, flags);
+
+	if (res >= 0) {
+		STORE_COMMON(res, "timerfd_create", "{%d}", res);
+	}
+
+	return res;
+}
+#endif
+
 #undef socket
 int __ast_fdleak_socket(int domain, int type, int protocol, const char *file, int line, const char *func)
 {
@@ -272,16 +333,21 @@ static char *handle_show_fd(struct ast_c
 		return NULL;
 	}
 	getrlimit(RLIMIT_NOFILE, &rl);
-	if (rl.rlim_cur == RLIM_INFINITY || rl.rlim_max == RLIM_INFINITY) {
+	if (rl.rlim_cur == RLIM_INFINITY) {
 		ast_copy_string(line, "unlimited", sizeof(line));
 	} else {
-		snprintf(line, sizeof(line), "%d/%d", (int) rl.rlim_cur, (int) rl.rlim_max);
+		snprintf(line, sizeof(line), "%d", (int) rl.rlim_cur);
 	}
 	ast_cli(a->fd, "Current maxfiles: %s\n", line);
 	for (i = 0; i < ARRAY_LEN(fdleaks); i++) {
 		if (fdleaks[i].isopen) {
+			struct ast_tm tm = {0};
+			char datestring[256];
+
+			ast_localtime(&fdleaks[i].now, &tm, NULL);
+			ast_strftime(datestring, sizeof(datestring), "%F %T", &tm);
 			snprintf(line, sizeof(line), "%d", fdleaks[i].line);
-			ast_cli(a->fd, "%5d %15s:%-7.7s (%-25s): %s(%s)\n", i, fdleaks[i].file, line, fdleaks[i].function, fdleaks[i].callname, fdleaks[i].callargs);
+			ast_cli(a->fd, "%5d [%s] %22s:%-7.7s (%-25s): %s(%s)\n", i, datestring, fdleaks[i].file, line, fdleaks[i].function, fdleaks[i].callname, fdleaks[i].callargs);
 		}
 	}
 	return CLI_SUCCESS;
diff -pruN 1:13.23.1~dfsg-2/main/astmm.c 1:16.1.1~dfsg-1/main/astmm.c
--- 1:13.23.1~dfsg-2/main/astmm.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astmm.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,11 +28,49 @@
 	<support_level>core</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
-#if defined(__AST_DEBUG_MALLOC)
+#include "asterisk/_private.h"
+#include "asterisk/logger.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+/*!
+ * \brief DEBUG_CHAOS returns failure randomly
+ *
+ * DEBUG_CHAOS_RETURN(failure); can be used to fake
+ * failure of functions such as memory allocation,
+ * for the purposes of testing failure handling.
+ */
+#ifdef DEBUG_CHAOS
+#ifndef DEBUG_CHAOS_ALLOC_CHANCE
+#define DEBUG_CHAOS_ALLOC_CHANCE 100000
+#endif
+/* Could #define DEBUG_CHAOS_ENABLE ast_fully_booted */
+#ifndef DEBUG_CHAOS_ENABLE
+#define DEBUG_CHAOS_ENABLE 1
+#endif
+#define DEBUG_CHAOS_RETURN(CHANCE, FAILURE) \
+	do { \
+		if ((DEBUG_CHAOS_ENABLE) && (ast_random() % CHANCE == 0)) { \
+			return FAILURE; \
+		} \
+	} while (0)
+#else
+#define DEBUG_CHAOS_RETURN(c,f)
+#endif
+
+#if defined(STANDALONE) || defined(STANDALONE2)
+#define ast_log_safe ast_log
+#endif
+
+#if defined(MALLOC_DEBUG) && !defined(STANDALONE) && !defined(STANDALONE2)
+#define __AST_DEBUG_MALLOC
+#endif
+
+#define MALLOC_FAILURE_MSG \
+	ast_log_safe(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file)
+
+#if defined(__AST_DEBUG_MALLOC)
 
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
 #include <stddef.h>
@@ -61,16 +99,6 @@ enum func_type {
 	FUNC_ASPRINTF
 };
 
-/* Undefine all our macros */
-#undef malloc
-#undef calloc
-#undef realloc
-#undef strdup
-#undef strndup
-#undef free
-#undef vasprintf
-#undef asprintf
-
 #define FENCE_MAGIC		0xfeedbabe	/*!< Allocated memory high/low fence overwrite check. */
 #define FREED_MAGIC		0xdeaddead	/*!< Freed memory wipe filler. */
 #define MALLOC_FILLER	0x55		/*!< Malloced memory filler.  Must not be zero. */
@@ -160,35 +188,10 @@ AST_MUTEX_DEFINE_STATIC_NOTRACKING(reglo
 		}                                    \
 	} while (0)
 
-void *ast_std_malloc(size_t size)
-{
-	return malloc(size);
-}
-
-void *ast_std_calloc(size_t nmemb, size_t size)
-{
-	return calloc(nmemb, size);
-}
-
-void *ast_std_realloc(void *ptr, size_t size)
-{
-	return realloc(ptr, size);
-}
-
-void ast_std_free(void *ptr)
-{
-	free(ptr);
-}
-
-void ast_free_ptr(void *ptr)
-{
-	ast_free(ptr);
-}
-
 static void print_backtrace(struct ast_bt *bt)
 {
 	int i = 0;
-	char **strings;
+	struct ast_vector_string *strings;
 
 	if (!bt) {
 		return;
@@ -196,10 +199,10 @@ static void print_backtrace(struct ast_b
 
 	if ((strings = ast_bt_get_symbols(bt->addresses, bt->num_frames))) {
 		astmm_log("Memory allocation backtrace:\n");
-		for (i = 3; i < bt->num_frames - 2; i++) {
-			astmm_log("#%d: [%p] %s\n", i - 3, bt->addresses[i], strings[i]);
+		for (i = 3; i < AST_VECTOR_SIZE(strings) - 2; i++) {
+			astmm_log("#%d: %s\n", i - 3, AST_VECTOR_GET(strings, i));
 		}
-		ast_std_free(strings);
+		ast_bt_free_symbols(strings);
 	}
 }
 
@@ -463,7 +466,7 @@ static void regions_check_all_fences(voi
 	ast_mutex_unlock(&reglock);
 }
 
-static void __ast_free_region(void *ptr, const char *file, int lineno, const char *func)
+void __ast_free(void *ptr, const char *file, int lineno, const char *func)
 {
 	struct ast_region *reg;
 
@@ -492,7 +495,7 @@ static void __ast_free_region(void *ptr,
 	}
 }
 
-void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
 {
 	void *ptr;
 
@@ -504,7 +507,7 @@ void *__ast_calloc(size_t nmemb, size_t
 	return ptr;
 }
 
-void *__ast_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+static void *__ast_repl_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
 {
 	void *ptr;
 
@@ -516,7 +519,7 @@ void *__ast_calloc_cache(size_t nmemb, s
 	return ptr;
 }
 
-void *__ast_malloc(size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_malloc(size_t size, const char *file, int lineno, const char *func)
 {
 	void *ptr;
 
@@ -529,11 +532,6 @@ void *__ast_malloc(size_t size, const ch
 	return ptr;
 }
 
-void __ast_free(void *ptr, const char *file, int lineno, const char *func)
-{
-	__ast_free_region(ptr, file, lineno, func);
-}
-
 /*!
  * \note reglock must be locked before calling.
  */
@@ -552,7 +550,7 @@ static struct ast_region *region_find(vo
 	return reg;
 }
 
-void *__ast_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
 {
 	size_t len;
 	struct ast_region *found;
@@ -576,7 +574,7 @@ void *__ast_realloc(void *ptr, size_t si
 	}
 
 	if (!size) {
-		__ast_free_region(ptr, file, lineno, func);
+		__ast_free(ptr, file, lineno, func);
 		return NULL;
 	}
 
@@ -591,7 +589,7 @@ void *__ast_realloc(void *ptr, size_t si
 				/* Make sure that the added memory is not zero. */
 				memset(new_mem + len, MALLOC_FILLER, size - len);
 			}
-			__ast_free_region(ptr, file, lineno, func);
+			__ast_free(ptr, file, lineno, func);
 		} else {
 			/* Make sure that the malloced memory is not zero. */
 			memset(new_mem, MALLOC_FILLER, size);
@@ -601,30 +599,24 @@ void *__ast_realloc(void *ptr, size_t si
 	return new_mem;
 }
 
-char *__ast_strdup(const char *s, const char *file, int lineno, const char *func)
+char *__ast_repl_strdup(const char *s, const char *file, int lineno, const char *func)
 {
 	size_t len;
 	void *ptr;
 
-	if (!s)
-		return NULL;
-
 	len = strlen(s) + 1;
-	if ((ptr = __ast_alloc_region(len, FUNC_STRDUP, file, lineno, func, 0)))
+	if ((ptr = __ast_alloc_region(len, FUNC_STRDUP, file, lineno, func, 0))) {
 		strcpy(ptr, s);
+	}
 
 	return ptr;
 }
 
-char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
+char *__ast_repl_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
 {
 	size_t len;
 	char *ptr;
 
-	if (!s) {
-		return NULL;
-	}
-
 	len = strnlen(s, n);
 	if ((ptr = __ast_alloc_region(len + 1, FUNC_STRNDUP, file, lineno, func, 0))) {
 		memcpy(ptr, s, len);
@@ -634,41 +626,47 @@ char *__ast_strndup(const char *s, size_
 	return ptr;
 }
 
-int __ast_asprintf(const char *file, int lineno, const char *func, char **strp, const char *fmt, ...)
+int __ast_repl_asprintf(const char *file, int lineno, const char *func, char **strp, const char *fmt, ...)
 {
 	int size;
 	va_list ap, ap2;
 	char s;
+	void *ptr;
 
-	*strp = NULL;
 	va_start(ap, fmt);
 	va_copy(ap2, ap);
 	size = vsnprintf(&s, 1, fmt, ap2);
 	va_end(ap2);
-	if (!(*strp = __ast_alloc_region(size + 1, FUNC_ASPRINTF, file, lineno, func, 0))) {
+	ptr = __ast_alloc_region(size + 1, FUNC_ASPRINTF, file, lineno, func, 0);
+	if (!ptr) {
+		/* As with stdlib *strp is undefined if allocation fails. */
 		va_end(ap);
 		return -1;
 	}
-	vsnprintf(*strp, size + 1, fmt, ap);
+	vsnprintf(ptr, size + 1, fmt, ap);
 	va_end(ap);
+	*strp = ptr;
 
 	return size;
 }
 
-int __ast_vasprintf(char **strp, const char *fmt, va_list ap, const char *file, int lineno, const char *func)
+int __ast_repl_vasprintf(char **strp, const char *fmt, va_list ap, const char *file, int lineno, const char *func)
 {
 	int size;
 	va_list ap2;
 	char s;
+	void *ptr;
 
-	*strp = NULL;
 	va_copy(ap2, ap);
 	size = vsnprintf(&s, 1, fmt, ap2);
 	va_end(ap2);
-	if (!(*strp = __ast_alloc_region(size + 1, FUNC_VASPRINTF, file, lineno, func, 0))) {
+	ptr = __ast_alloc_region(size + 1, FUNC_VASPRINTF, file, lineno, func, 0);
+	if (!ptr) {
+		/* As with stdlib *strp is undefined if allocation fails. */
 		return -1;
 	}
-	vsnprintf(*strp, size + 1, fmt, ap);
+	vsnprintf(ptr, size + 1, fmt, ap);
+	*strp = ptr;
 
 	return size;
 }
@@ -1508,14 +1506,7 @@ static void mm_atexit_final(void)
 	}
 }
 
-/*!
- * \brief Initialize malloc debug phase 1.
- *
- * \note Must be called first thing in main().
- *
- * \return Nothing
- */
-void __ast_mm_init_phase_1(void)
+void load_astmm_phase_1(void)
 {
 	atexit(mm_atexit_final);
 }
@@ -1529,12 +1520,7 @@ static void mm_atexit_ast(void)
 	ast_cli_unregister_multiple(cli_memory, ARRAY_LEN(cli_memory));
 }
 
-/*!
- * \brief Initialize malloc debug phase 2.
- *
- * \return Nothing
- */
-void __ast_mm_init_phase_2(void)
+void load_astmm_phase_2(void)
 {
 	char filename[PATH_MAX];
 
@@ -1555,4 +1541,223 @@ void __ast_mm_init_phase_2(void)
 	ast_register_cleanup(mm_atexit_ast);
 }
 
+#else	/* !defined(__AST_DEBUG_MALLOC) */
+
+void load_astmm_phase_1(void)
+{
+}
+
+void load_astmm_phase_2(void)
+{
+}
+
+void *__ast_repl_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return calloc(nmemb, size);
+}
+
+static void *__ast_repl_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return calloc(nmemb, size);
+}
+
+void *__ast_repl_malloc(size_t size, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return malloc(size);
+}
+
+void __ast_free(void *ptr, const char *file, int lineno, const char *func)
+{
+	free(ptr);
+}
+
+void *__ast_repl_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return realloc(ptr, size);
+}
+
+char *__ast_repl_strdup(const char *s, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return strdup(s);
+}
+
+char *__ast_repl_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, NULL);
+
+	return strndup(s, n);
+}
+
+int __ast_repl_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
+{
+	int res;
+	va_list ap;
+
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, -1);
+
+	va_start(ap, format);
+	res = vasprintf(strp, format, ap);
+	va_end(ap);
+
+	return res;
+}
+
+int __ast_repl_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
+{
+	DEBUG_CHAOS_RETURN(DEBUG_CHAOS_ALLOC_CHANCE, -1);
+
+	return vasprintf(strp, format, ap);
+}
+
 #endif	/* defined(__AST_DEBUG_MALLOC) */
+
+void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+{
+	void *p;
+
+	p = __ast_repl_calloc(nmemb, size, file, lineno, func);
+	if (!p) {
+		MALLOC_FAILURE_MSG;
+	}
+
+	return p;
+}
+
+void *__ast_calloc_cache(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+{
+	void *p;
+
+	p = __ast_repl_calloc_cache(nmemb, size, file, lineno, func);
+	if (!p) {
+		MALLOC_FAILURE_MSG;
+	}
+
+	return p;
+
+}
+
+void *__ast_malloc(size_t size, const char *file, int lineno, const char *func)
+{
+	void *p;
+
+	p = __ast_repl_malloc(size, file, lineno, func);
+	if (!p) {
+		MALLOC_FAILURE_MSG;
+	}
+
+	return p;
+}
+
+void *__ast_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
+{
+	void *newp;
+
+	newp = __ast_repl_realloc(ptr, size, file, lineno, func);
+	if (!newp) {
+		MALLOC_FAILURE_MSG;
+	}
+
+	return newp;
+}
+
+char *__ast_strdup(const char *s, const char *file, int lineno, const char *func)
+{
+	char *newstr = NULL;
+
+	if (s) {
+		newstr = __ast_repl_strdup(s, file, lineno, func);
+		if (!newstr) {
+			MALLOC_FAILURE_MSG;
+		}
+	}
+
+	return newstr;
+}
+
+char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
+{
+	char *newstr = NULL;
+
+	if (s) {
+		newstr = __ast_repl_strndup(s, n, file, lineno, func);
+		if (!newstr) {
+			MALLOC_FAILURE_MSG;
+		}
+	}
+
+	return newstr;
+}
+
+int __ast_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
+{
+	int res;
+	va_list ap;
+
+	va_start(ap, format);
+	res = __ast_repl_vasprintf(strp, format, ap, file, lineno, func);
+	if (res < 0) {
+		/*
+		 * *strp is undefined so set to NULL to ensure it is
+		 * initialized to something useful.
+		 */
+		*strp = NULL;
+
+		MALLOC_FAILURE_MSG;
+	}
+	va_end(ap);
+
+	return res;
+}
+
+int __ast_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
+{
+	int res;
+
+	res = __ast_repl_vasprintf(strp, format, ap, file, lineno, func);
+	if (res < 0) {
+		/*
+		 * *strp is undefined so set to NULL to ensure it is
+		 * initialized to something useful.
+		 */
+		*strp = NULL;
+
+		MALLOC_FAILURE_MSG;
+	}
+
+	return res;
+}
+
+void *ast_std_malloc(size_t size)
+{
+	return malloc(size);
+}
+
+void *ast_std_calloc(size_t nmemb, size_t size)
+{
+	return calloc(nmemb, size);
+}
+
+void *ast_std_realloc(void *ptr, size_t size)
+{
+	return realloc(ptr, size);
+}
+
+void ast_std_free(void *ptr)
+{
+	free(ptr);
+}
+
+void ast_free_ptr(void *ptr)
+{
+	ast_free(ptr);
+}
diff -pruN 1:13.23.1~dfsg-2/main/astobj2.c 1:16.1.1~dfsg-1/main/astobj2.c
--- 1:13.23.1~dfsg-2/main/astobj2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,9 +25,11 @@
 	<support_level>core</support_level>
  ***/
 
-#include "asterisk.h"
+/* This reduces the size of lock structures within astobj2 objects when
+ * DEBUG_THREADS is not defined. */
+#define DEBUG_THREADS_LOOSE_ABI
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include "asterisk.h"
 
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
@@ -50,20 +52,50 @@ static FILE *ref_log;
  * The magic number is used for consistency check.
  */
 struct __priv_data {
-	int ref_counter;
 	ao2_destructor_fn destructor_fn;
+	/*! This field is used for astobj2 and ao2_weakproxy objects to reference each other */
+	void *weakptr;
 #if defined(AO2_DEBUG)
 	/*! User data size for stats */
 	size_t data_size;
 #endif
-	/*! The ao2 object option flags */
-	uint32_t options;
-	/*! magic number.  This is used to verify that a pointer passed in is a
-	 *  valid astobj2 */
-	uint32_t magic;
+	/*! Number of references held for this object */
+	int32_t ref_counter;
+	/*!
+	 * \brief The ao2 object option flags.
+	 *
+	 * \note This field is constant after object creation.  It shares
+	 *       a uint32_t with \ref lockused and \ref magic.
+	 */
+	uint32_t options:2;
+	/*!
+	 * \brief Set to 1 when the lock is used if refdebug is enabled.
+	 *
+	 * \note This bit-field may be modified after object creation.  It
+	 *       shares a uint32_t with \ref options and \ref magic.
+	 */
+	uint32_t lockused:1;
+	/*!
+	 * \brief Magic number.
+	 *
+	 * This is used to verify that a pointer is a valid astobj2 or ao2_weak
+	 * reference.
+	 *
+	 * \note This field is constant after object creation.  It shares
+	 *       a uint32_t with \ref options and \ref lockused.
+	 *
+	 * \warning Stealing bits for any additional writable fields would cause
+	 *          reentrancy issues if using bitfields.  If any additional
+	 *          writable bits are required in the future we will need to put
+	 *          all bitfields into a single 'uint32_t flags' field and use
+	 *          atomic operations from \file lock.h to perform writes.
+	 */
+	uint32_t magic:29;
 };
 
-#define	AO2_MAGIC	0xa570b123
+#define	AO2_MAGIC	0x1a70b123
+#define	AO2_WEAK	0x1a70b122
+#define IS_AO2_MAGIC_BAD(p) (AO2_MAGIC != (p->priv_data.magic | 1))
 
 /*!
  * What an astobj2 object looks like: fixed-size private data
@@ -74,6 +106,12 @@ struct astobj2 {
 	void *user_data[0];
 };
 
+struct ao2_weakproxy_notification {
+	ao2_weakproxy_notification_cb cb;
+	void *data;
+	AST_LIST_ENTRY(ao2_weakproxy_notification) list;
+};
+
 struct ao2_lock_priv {
 	ast_mutex_t lock;
 };
@@ -98,6 +136,17 @@ struct astobj2_rwlock {
 	void *user_data[0];
 };
 
+struct ao2_lockobj_priv {
+	void *lock;
+};
+
+/* AstObj2 with locking provided by a separate object. */
+struct astobj2_lockobj {
+	struct ao2_lockobj_priv lockobj;
+	struct __priv_data priv_data;
+	void *user_data[0];
+};
+
 #ifdef AO2_DEBUG
 struct ao2_stats ao2;
 #endif
@@ -108,32 +157,31 @@ struct ao2_stats ao2;
 #define INTERNAL_OBJ_RWLOCK(user_data) \
 	((struct astobj2_rwlock *) (((char *) (user_data)) - sizeof(struct astobj2_rwlock)))
 
+#define INTERNAL_OBJ_LOCKOBJ(user_data) \
+	((struct astobj2_lockobj *) (((char *) (user_data)) - sizeof(struct astobj2_lockobj)))
+
+#define INTERNAL_OBJ(user_data) \
+	(struct astobj2 *) ((char *) user_data - sizeof(struct astobj2))
+
 /*!
  * \brief convert from a pointer _p to a user-defined object
  *
  * \return the pointer to the astobj2 structure
  */
-static struct astobj2 *INTERNAL_OBJ(void *user_data)
-{
-	struct astobj2 *p;
+#define __INTERNAL_OBJ_CHECK(user_data, file, line, func) \
+	({ \
+		struct astobj2 *p ## __LINE__; \
+		if (!user_data \
+			|| !(p ## __LINE__ = INTERNAL_OBJ(user_data)) \
+			|| IS_AO2_MAGIC_BAD(p ## __LINE__)) { \
+			log_bad_ao2(user_data, file, line, func); \
+			p ## __LINE__ = NULL; \
+		} \
+		(p ## __LINE__); \
+	})
 
-	if (!user_data) {
-		__ast_assert_failed(0, "user_data is NULL", __FILE__, __LINE__, __PRETTY_FUNCTION__);
-		return NULL;
-	}
-
-	p = (struct astobj2 *) ((char *) user_data - sizeof(*p));
-	if (AO2_MAGIC != p->priv_data.magic) {
-		char bad_magic[100];
-
-		snprintf(bad_magic, sizeof(bad_magic), "bad magic number 0x%x for object %p",
-			p->priv_data.magic, user_data);
-		__ast_assert_failed(0, bad_magic, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-		return NULL;
-	}
-
-	return p;
-}
+#define INTERNAL_OBJ_CHECK(user_data) \
+	__INTERNAL_OBJ_CHECK(user_data, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 /*!
  * \brief convert from a pointer _p to an astobj2 object
@@ -142,22 +190,51 @@ static struct astobj2 *INTERNAL_OBJ(void
  */
 #define EXTERNAL_OBJ(_p)	((_p) == NULL ? NULL : (_p)->user_data)
 
-int is_ao2_object(void *user_data)
+int internal_is_ao2_object(void *user_data)
 {
-	return (INTERNAL_OBJ(user_data) != NULL);
+	struct astobj2 *p;
+
+	if (!user_data) {
+		return 0;
+	}
+
+	p = INTERNAL_OBJ(user_data);
+
+	return !p || IS_AO2_MAGIC_BAD(p) ? 0 : 1;
+}
+
+void log_bad_ao2(void *user_data, const char *file, int line, const char *func)
+{
+	struct astobj2 *p;
+	char bad_magic[100];
+
+	if (!user_data) {
+		__ast_assert_failed(0, "user_data is NULL", file, line, func);
+		return;
+	}
+
+	p = INTERNAL_OBJ(user_data);
+	snprintf(bad_magic, sizeof(bad_magic), "bad magic number 0x%x for object %p",
+		p->priv_data.magic, user_data);
+	__ast_assert_failed(0, bad_magic, file, line, func);
 }
 
 int __ao2_lock(void *user_data, enum ao2_lock_req lock_how, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 
 	if (obj == NULL) {
 		return -1;
 	}
 
+	if (ref_log) {
+		obj->priv_data.lockused  = 1;
+	}
+
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
@@ -195,6 +272,10 @@ int __ao2_lock(void *user_data, enum ao2
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_lock(obj_lockobj->lockobj.lock, lock_how, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
@@ -206,9 +287,10 @@ int __ao2_lock(void *user_data, enum ao2
 
 int __ao2_unlock(void *user_data, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 	int current_value;
 
@@ -244,6 +326,10 @@ int __ao2_unlock(void *user_data, const
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_unlock(obj_lockobj->lockobj.lock, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
@@ -255,15 +341,20 @@ int __ao2_unlock(void *user_data, const
 
 int __ao2_trylock(void *user_data, enum ao2_lock_req lock_how, const char *file, const char *func, int line, const char *var)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	int res = 0;
 
 	if (obj == NULL) {
 		return -1;
 	}
 
+	if (ref_log) {
+		obj->priv_data.lockused  = 1;
+	}
+
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
@@ -301,13 +392,16 @@ int __ao2_trylock(void *user_data, enum
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
 		/* The ao2 object has no lock. */
 		return 0;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		res = __ao2_trylock(obj_lockobj->lockobj.lock, lock_how, file, func, line, var);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func, "Invalid lock option on ao2 object %p\n",
 			user_data);
 		return -1;
 	}
 
-
 	return res;
 }
 
@@ -332,6 +426,7 @@ enum ao2_lock_req __adjust_lock(void *us
 {
 	struct astobj2 *obj = INTERNAL_OBJ(user_data);
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
 	enum ao2_lock_req orig_lock;
 
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
@@ -362,6 +457,10 @@ enum ao2_lock_req __adjust_lock(void *us
 			break;
 		}
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		orig_lock = __adjust_lock(obj_lockobj->lockobj.lock, lock_how, keep_stronger);
+		break;
 	default:
 		ast_log(LOG_ERROR, "Invalid lock option on ao2 object %p\n", user_data);
 		/* Fall through */
@@ -376,9 +475,11 @@ enum ao2_lock_req __adjust_lock(void *us
 
 void *ao2_object_get_lockaddr(void *user_data)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj;
 	struct astobj2_lock *obj_mutex;
 
+	obj = INTERNAL_OBJ_CHECK(user_data);
+
 	if (obj == NULL) {
 		return NULL;
 	}
@@ -394,15 +495,24 @@ void *ao2_object_get_lockaddr(void *user
 	return NULL;
 }
 
-static int internal_ao2_ref(void *user_data, int delta, const char *file, int line, const char *func)
+int __ao2_ref(void *user_data, int delta,
+	const char *tag, const char *file, int line, const char *func)
 {
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
+	struct astobj2 *obj = __INTERNAL_OBJ_CHECK(user_data, file, line, func);
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
-	int current_value;
-	int ret;
+	struct astobj2_lockobj *obj_lockobj;
+	int32_t current_value;
+	int32_t ret;
+	struct ao2_weakproxy *weakproxy = NULL;
+	const char *lock_state;
 
 	if (obj == NULL) {
+		if (ref_log && user_data) {
+			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
+				user_data, delta, ast_get_tid(), file, line, func, tag ?: "");
+			fflush(ref_log);
+		}
 		return -1;
 	}
 
@@ -411,14 +521,54 @@ static int internal_ao2_ref(void *user_d
 		return obj->priv_data.ref_counter;
 	}
 
+	if (delta < 0 && obj->priv_data.magic == AO2_MAGIC && (weakproxy = obj->priv_data.weakptr)) {
+		ao2_lock(weakproxy);
+	}
+
 	/* we modify with an atomic operation the reference counter */
-	ret = ast_atomic_fetchadd_int(&obj->priv_data.ref_counter, delta);
+	ret = ast_atomic_fetch_add(&obj->priv_data.ref_counter, delta, __ATOMIC_RELAXED);
 	current_value = ret + delta;
 
 #ifdef AO2_DEBUG
 	ast_atomic_fetchadd_int(&ao2.total_refs, delta);
 #endif
 
+	if (weakproxy) {
+		struct ao2_weakproxy cbs;
+
+		if (current_value == 1) {
+			/* The only remaining reference is the one owned by the weak object */
+			struct astobj2 *internal_weakproxy;
+
+			internal_weakproxy = INTERNAL_OBJ_CHECK(weakproxy);
+
+			/* Unlink the obj from the weak proxy */
+			internal_weakproxy->priv_data.weakptr = NULL;
+			obj->priv_data.weakptr = NULL;
+
+			/* transfer list to local copy so callbacks are run with weakproxy unlocked. */
+			cbs.destroyed_cb = weakproxy->destroyed_cb;
+			AST_LIST_HEAD_INIT_NOLOCK(&weakproxy->destroyed_cb);
+
+			/* weak is already unlinked from obj so this won't recurse */
+			ao2_ref(user_data, -1);
+		}
+
+		ao2_unlock(weakproxy);
+
+		if (current_value == 1) {
+			struct ao2_weakproxy_notification *destroyed_cb;
+
+			/* Notify the subscribers that weakproxy now points to NULL. */
+			while ((destroyed_cb = AST_LIST_REMOVE_HEAD(&cbs.destroyed_cb, list))) {
+				destroyed_cb->cb(weakproxy, destroyed_cb->data);
+				ast_free(destroyed_cb);
+			}
+
+			ao2_ref(weakproxy, -1);
+		}
+	}
+
 	if (0 < current_value) {
 		/* The object still lives. */
 #define EXCESSIVE_REF_COUNT		100000
@@ -429,18 +579,31 @@ static int internal_ao2_ref(void *user_d
 			/* We just reached or went over the excessive ref count trigger */
 			snprintf(excessive_ref_buf, sizeof(excessive_ref_buf),
 				"Excessive refcount %d reached on ao2 object %p",
-				current_value, user_data);
+				(int)current_value, user_data);
 			ast_log(__LOG_ERROR, file, line, func, "%s\n", excessive_ref_buf);
 
 			__ast_assert_failed(0, excessive_ref_buf, file, line, func);
 		}
+
+		if (ref_log && tag) {
+			fprintf(ref_log, "%p,%s%d,%d,%s,%d,%s,%d,%s\n", user_data,
+				(delta < 0 ? "" : "+"), delta, ast_get_tid(),
+				file, line, func, (int)ret, tag);
+			fflush(ref_log);
+		}
 		return ret;
 	}
 
 	/* this case must never happen */
 	if (current_value < 0) {
 		ast_log(__LOG_ERROR, file, line, func,
-			"Invalid refcount %d on ao2 object %p\n", current_value, user_data);
+			"Invalid refcount %d on ao2 object %p\n", (int)current_value, user_data);
+		if (ref_log) {
+			/* Log to ref_log invalid even if (tag == NULL) */
+			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
+				user_data, delta, ast_get_tid(), file, line, func, tag ?: "");
+			fflush(ref_log);
+		}
 		ast_assert(0);
 		/* stop here even if assert doesn't DO_CRASH */
 		return -1;
@@ -462,66 +625,49 @@ static int internal_ao2_ref(void *user_d
 	switch (obj->priv_data.options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
 		obj_mutex = INTERNAL_OBJ_MUTEX(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
 		ast_mutex_destroy(&obj_mutex->mutex.lock);
 
 		ast_free(obj_mutex);
 		break;
 	case AO2_ALLOC_OPT_LOCK_RWLOCK:
 		obj_rwlock = INTERNAL_OBJ_RWLOCK(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
 		ast_rwlock_destroy(&obj_rwlock->rwlock.lock);
 
 		ast_free(obj_rwlock);
 		break;
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
+		lock_state = "none";
 		ast_free(obj);
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		obj_lockobj = INTERNAL_OBJ_LOCKOBJ(user_data);
+		lock_state = obj->priv_data.lockused ? "used" : "unused";
+		ao2_t_ref(obj_lockobj->lockobj.lock, -1, "release lockobj");
+
+		ast_free(obj_lockobj);
+		break;
 	default:
 		ast_log(__LOG_ERROR, file, line, func,
 			"Invalid lock option on ao2 object %p\n", user_data);
+		lock_state = "invalid";
 		break;
 	}
 
-	return ret;
-}
-
-int __ao2_ref_debug(void *user_data, int delta, const char *tag, const char *file, int line, const char *func)
-{
-	struct astobj2 *obj = INTERNAL_OBJ(user_data);
-	int old_refcount = -1;
-
-	if (obj) {
-		old_refcount = internal_ao2_ref(user_data, delta, file, line, func);
-	}
-
-	if (ref_log && user_data) {
-		if (!obj) {
-			/* Invalid object: Bad magic number. */
-			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**invalid**,%s\n",
-				user_data, delta, ast_get_tid(), file, line, func, tag);
-			fflush(ref_log);
-		} else if (old_refcount + delta == 0) {
-			fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**destructor**,%s\n",
-				user_data, delta, ast_get_tid(), file, line, func, tag);
-			fflush(ref_log);
-		} else if (delta != 0) {
-			fprintf(ref_log, "%p,%s%d,%d,%s,%d,%s,%d,%s\n", user_data, (delta < 0 ? "" : "+"),
-				delta, ast_get_tid(), file, line, func, old_refcount, tag);
-			fflush(ref_log);
-		}
+	if (ref_log && tag) {
+		fprintf(ref_log, "%p,%d,%d,%s,%d,%s,**destructor**lock-state:%s**,%s\n",
+			user_data, delta, ast_get_tid(), file, line, func, lock_state, tag);
+		fflush(ref_log);
 	}
 
-	return old_refcount;
-}
-
-int __ao2_ref(void *user_data, int delta)
-{
-	return internal_ao2_ref(user_data, delta, __FILE__, __LINE__, __FUNCTION__);
+	return ret;
 }
 
 void __ao2_cleanup_debug(void *obj, const char *tag, const char *file, int line, const char *function)
 {
 	if (obj) {
-		__ao2_ref_debug(obj, -1, tag, file, line, function);
+		__ao2_ref(obj, -1, tag, file, line, function);
 	}
 }
 
@@ -532,20 +678,20 @@ void __ao2_cleanup(void *obj)
 	}
 }
 
-static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options, const char *file, int line, const char *func)
+static void *internal_ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options,
+	void *lockobj, const char *tag, const char *file, int line, const char *func)
 {
 	/* allocation */
 	struct astobj2 *obj;
 	struct astobj2_lock *obj_mutex;
 	struct astobj2_rwlock *obj_rwlock;
+	struct astobj2_lockobj *obj_lockobj;
+	size_t overhead;
 
 	switch (options & AO2_ALLOC_OPT_LOCK_MASK) {
 	case AO2_ALLOC_OPT_LOCK_MUTEX:
-#if defined(__AST_DEBUG_MALLOC)
-		obj_mutex = __ast_calloc(1, sizeof(*obj_mutex) + data_size, file, line, func);
-#else
-		obj_mutex = ast_calloc(1, sizeof(*obj_mutex) + data_size);
-#endif
+		overhead = sizeof(*obj_mutex);
+		obj_mutex = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj_mutex == NULL) {
 			return NULL;
 		}
@@ -554,11 +700,8 @@ static void *internal_ao2_alloc(size_t d
 		obj = (struct astobj2 *) &obj_mutex->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_RWLOCK:
-#if defined(__AST_DEBUG_MALLOC)
-		obj_rwlock = __ast_calloc(1, sizeof(*obj_rwlock) + data_size, file, line, func);
-#else
-		obj_rwlock = ast_calloc(1, sizeof(*obj_rwlock) + data_size);
-#endif
+		overhead = sizeof(*obj_rwlock);
+		obj_rwlock = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj_rwlock == NULL) {
 			return NULL;
 		}
@@ -567,15 +710,29 @@ static void *internal_ao2_alloc(size_t d
 		obj = (struct astobj2 *) &obj_rwlock->priv_data;
 		break;
 	case AO2_ALLOC_OPT_LOCK_NOLOCK:
-#if defined(__AST_DEBUG_MALLOC)
-		obj = __ast_calloc(1, sizeof(*obj) + data_size, file, line, func);
-#else
-		obj = ast_calloc(1, sizeof(*obj) + data_size);
-#endif
+		overhead = sizeof(*obj);
+		obj = __ast_calloc(1, overhead + data_size, file, line, func);
 		if (obj == NULL) {
 			return NULL;
 		}
 		break;
+	case AO2_ALLOC_OPT_LOCK_OBJ:
+		lockobj = ao2_t_bump(lockobj, "set lockobj");
+		if (!lockobj) {
+			ast_log(__LOG_ERROR, file, line, func, "AO2_ALLOC_OPT_LOCK_OBJ requires a non-NULL lockobj.\n");
+			return NULL;
+		}
+
+		overhead = sizeof(*obj_lockobj);
+		obj_lockobj = __ast_calloc(1, overhead + data_size, file, line, func);
+		if (obj_lockobj == NULL) {
+			ao2_t_ref(lockobj, -1, "release lockobj for failed alloc");
+			return NULL;
+		}
+
+		obj_lockobj->lockobj.lock = lockobj;
+		obj = (struct astobj2 *) &obj_lockobj->priv_data;
+		break;
 	default:
 		/* Invalid option value. */
 		ast_log(__LOG_DEBUG, file, line, func, "Invalid lock option requested\n");
@@ -583,8 +740,8 @@ static void *internal_ao2_alloc(size_t d
 	}
 
 	/* Initialize common ao2 values. */
-	obj->priv_data.ref_counter = 1;
 	obj->priv_data.destructor_fn = destructor_fn;	/* can be NULL */
+	obj->priv_data.ref_counter = 1;
 	obj->priv_data.options = options;
 	obj->priv_data.magic = AO2_MAGIC;
 
@@ -595,47 +752,38 @@ static void *internal_ao2_alloc(size_t d
 	ast_atomic_fetchadd_int(&ao2.total_refs, 1);
 #endif
 
+	if (ref_log && tag) {
+		fprintf(ref_log, "%p,+1,%d,%s,%d,%s,**constructor**%zu**%zu**,%s\n",
+			EXTERNAL_OBJ(obj), ast_get_tid(), file, line, func, overhead, data_size, tag);
+		fflush(ref_log);
+	}
+
 	/* return a pointer to the user data */
 	return EXTERNAL_OBJ(obj);
 }
 
-unsigned int ao2_options_get(void *obj)
+void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options,
+	const char *tag, const char *file, int line, const char *func)
 {
-	struct astobj2 *orig_obj = INTERNAL_OBJ(obj);
-	if (!orig_obj) {
-		return 0;
-	}
-	return orig_obj->priv_data.options;
+	return internal_ao2_alloc(data_size, destructor_fn, options, NULL, tag, file, line, func);
 }
 
-void *__ao2_alloc_debug(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options, const char *tag,
-	const char *file, int line, const char *func, int ref_debug)
+void *__ao2_alloc_with_lockobj(size_t data_size, ao2_destructor_fn destructor_fn, void *lockobj,
+	const char *tag, const char *file, int line, const char *func)
 {
-	/* allocation */
-	void *obj;
-
-	if ((obj = internal_ao2_alloc(data_size, destructor_fn, options, file, line, func)) == NULL) {
-		return NULL;
-	}
-
-	if (ref_log) {
-		fprintf(ref_log, "%p,+1,%d,%s,%d,%s,**constructor**,%s\n", obj, ast_get_tid(), file, line, func, tag);
-		fflush(ref_log);
-	}
-
-	/* return a pointer to the user data */
-	return obj;
+	return internal_ao2_alloc(data_size, destructor_fn, AO2_ALLOC_OPT_LOCK_OBJ, lockobj,
+		tag, file, line, func);
 }
 
-void *__ao2_alloc(size_t data_size, ao2_destructor_fn destructor_fn, unsigned int options)
+unsigned int ao2_options_get(void *obj)
 {
-	return internal_ao2_alloc(data_size, destructor_fn, options, __FILE__, __LINE__, __FUNCTION__);
-}
-
+	struct astobj2 *orig_obj;
 
-void __ao2_global_obj_release(struct ao2_global_obj *holder, const char *tag, const char *file, int line, const char *func, const char *name)
-{
-	__ao2_global_obj_replace_unref(holder, NULL, tag, file, line, func, name);
+	orig_obj = INTERNAL_OBJ_CHECK(obj);
+	if (!orig_obj) {
+		return 0;
+	}
+	return orig_obj->priv_data.options;
 }
 
 void *__ao2_global_obj_replace(struct ao2_global_obj *holder, void *obj, const char *tag, const char *file, int line, const char *func, const char *name)
@@ -655,11 +803,7 @@ void *__ao2_global_obj_replace(struct ao
 	}
 
 	if (obj) {
-		if (tag) {
-			__ao2_ref_debug(obj, +1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj, +1);
-		}
+		__ao2_ref(obj, +1, tag, file, line, func);
 	}
 	obj_old = holder->obj;
 	holder->obj = obj;
@@ -675,11 +819,7 @@ int __ao2_global_obj_replace_unref(struc
 
 	obj_old = __ao2_global_obj_replace(holder, obj, tag, file, line, func, name);
 	if (obj_old) {
-		if (tag) {
-			__ao2_ref_debug(obj_old, -1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj_old, -1);
-		}
+		__ao2_ref(obj_old, -1, tag, file, line, func);
 		return 1;
 	}
 	return 0;
@@ -704,11 +844,7 @@ void *__ao2_global_obj_ref(struct ao2_gl
 
 	obj = holder->obj;
 	if (obj) {
-		if (tag) {
-			__ao2_ref_debug(obj, +1, tag, file, line, func);
-		} else {
-			__ao2_ref(obj, +1);
-		}
+		__ao2_ref(obj, +1, tag, file, line, func);
 	}
 
 	__ast_rwlock_unlock(file, line, func, &holder->lock, name);
@@ -716,6 +852,223 @@ void *__ao2_global_obj_ref(struct ao2_gl
 	return obj;
 }
 
+
+void *__ao2_weakproxy_alloc(size_t data_size, ao2_destructor_fn destructor_fn,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct ao2_weakproxy *weakproxy;
+
+	if (data_size < sizeof(*weakproxy)) {
+		ast_assert(0);
+		ast_log(LOG_ERROR, "Requested data_size smaller than minimum.\n");
+		return NULL;
+	}
+
+	weakproxy = __ao2_alloc(data_size, destructor_fn, AO2_ALLOC_OPT_LOCK_MUTEX,
+		tag, file, line, func);
+
+	if (weakproxy) {
+		struct astobj2 *weakproxy_internal;
+
+		/* Just created weakproxy, no need to check if it's valid. */
+		weakproxy_internal = INTERNAL_OBJ(weakproxy);
+		weakproxy_internal->priv_data.magic = AO2_WEAK;
+	}
+
+	return weakproxy;
+}
+
+int __ao2_weakproxy_set_object(void *weakproxy, void *obj, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *weakproxy_internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	struct astobj2 *obj_internal = __INTERNAL_OBJ_CHECK(obj, file, line, func);
+	int ret = -1;
+
+	if (!weakproxy_internal
+		|| weakproxy_internal->priv_data.magic != AO2_WEAK) {
+		return -1;
+	}
+
+	if (!obj_internal
+		|| obj_internal->priv_data.weakptr
+		|| obj_internal->priv_data.magic != AO2_MAGIC) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	if (!weakproxy_internal->priv_data.weakptr) {
+		__ao2_ref(obj, +1, tag, file, line, func);
+		__ao2_ref(weakproxy, +1, tag, file, line, func);
+
+		weakproxy_internal->priv_data.weakptr = obj;
+		obj_internal->priv_data.weakptr = weakproxy;
+
+		ret = 0;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+		/* It is possible for obj to be accessed now.  It's allowed
+		 * for weakproxy to already be in a container.  Another thread
+		 * could have been waiting for a lock on weakproxy to retrieve
+		 * the object.
+		 */
+	}
+
+	return ret;
+}
+
+int __ao2_weakproxy_ref_object(void *weakproxy, int delta, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	int ret = -1;
+
+	if (!internal || internal->priv_data.magic != AO2_WEAK) {
+		/* This method is meant to be run on weakproxy objects! */
+		return -2;
+	}
+
+	/* We have a weak object, grab lock. */
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	if (internal->priv_data.weakptr) {
+		ret = __ao2_ref(internal->priv_data.weakptr, delta, tag, file, line, func);
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return ret;
+}
+
+void *__ao2_weakproxy_get_object(void *weakproxy, int flags,
+	const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *internal = __INTERNAL_OBJ_CHECK(weakproxy, file, line, func);
+	void *obj;
+
+	if (!internal || internal->priv_data.magic != AO2_WEAK) {
+		/* This method is meant to be run on weakproxy objects! */
+		return NULL;
+	}
+
+	/* We have a weak object, grab reference to object within lock */
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	obj = internal->priv_data.weakptr;
+	if (obj) {
+		__ao2_ref(obj, +1, tag, file, line, func);
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return obj;
+}
+
+void *__ao2_get_weakproxy(void *obj, const char *tag, const char *file, int line, const char *func)
+{
+	struct astobj2 *obj_internal = __INTERNAL_OBJ_CHECK(obj, file, line, func);
+
+	if (!obj_internal || obj_internal->priv_data.magic != AO2_MAGIC) {
+		/* This method is meant to be run on normal ao2 objects! */
+		return NULL;
+	}
+
+	if (!obj_internal->priv_data.weakptr) {
+		return NULL;
+	}
+
+	__ao2_ref(obj_internal->priv_data.weakptr, +1, tag, file, line, func);
+	return obj_internal->priv_data.weakptr;
+}
+
+int ao2_weakproxy_subscribe(void *weakproxy, ao2_weakproxy_notification_cb cb, void *data, int flags)
+{
+	struct astobj2 *weakproxy_internal = INTERNAL_OBJ_CHECK(weakproxy);
+	int ret = -1;
+	int hasobj;
+
+	if (!weakproxy_internal || weakproxy_internal->priv_data.magic != AO2_WEAK) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	hasobj = weakproxy_internal->priv_data.weakptr != NULL;
+	if (hasobj) {
+		struct ao2_weakproxy *weak = weakproxy;
+		struct ao2_weakproxy_notification *sub = ast_calloc(1, sizeof(*sub));
+
+		if (sub) {
+			sub->cb = cb;
+			sub->data = data;
+			AST_LIST_INSERT_HEAD(&weak->destroyed_cb, sub, list);
+			ret = 0;
+		}
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	if (!hasobj) {
+		cb(weakproxy, data);
+		ret = 0;
+	}
+
+	return ret;
+}
+
+int ao2_weakproxy_unsubscribe(void *weakproxy, ao2_weakproxy_notification_cb destroyed_cb, void *data, int flags)
+{
+	struct astobj2 *internal_weakproxy = INTERNAL_OBJ_CHECK(weakproxy);
+	struct ao2_weakproxy *weak;
+	struct ao2_weakproxy_notification *sub;
+	int ret = 0;
+
+	if (!internal_weakproxy || internal_weakproxy->priv_data.magic != AO2_WEAK || !destroyed_cb) {
+		return -1;
+	}
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_lock(weakproxy);
+	}
+
+	weak = weakproxy;
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&weak->destroyed_cb, sub, list) {
+		if (sub->cb == destroyed_cb && sub->data == data) {
+			AST_LIST_REMOVE_CURRENT(list);
+			ast_free(sub);
+			ret++;
+			if (!(flags & OBJ_MULTIPLE)) {
+				break;
+			}
+		}
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(weakproxy);
+	}
+
+	return ret;
+}
+
+
 #ifdef AO2_DEBUG
 static int print_cb(void *obj, void *arg, int flag)
 {
@@ -869,26 +1222,30 @@ static void astobj2_cleanup(void)
 #if defined(AO2_DEBUG)
 	ast_cli_unregister_multiple(cli_astobj2, ARRAY_LEN(cli_astobj2));
 #endif
-#ifdef REF_DEBUG
-	fclose(ref_log);
-	ref_log = NULL;
-#endif
+
+	if (ast_opt_ref_debug) {
+		fclose(ref_log);
+		ref_log = NULL;
+	}
 }
 
 int astobj2_init(void)
 {
-#ifdef REF_DEBUG
 	char ref_filename[1024];
 
-	snprintf(ref_filename, sizeof(ref_filename), "%s/refs", ast_config_AST_LOG_DIR);
-	ref_log = fopen(ref_filename, "w");
-	if (!ref_log) {
-		ast_log(LOG_ERROR, "Could not open ref debug log file: %s\n", ref_filename);
+	if (ast_opt_ref_debug) {
+		snprintf(ref_filename, sizeof(ref_filename), "%s/refs", ast_config_AST_LOG_DIR);
+		ref_log = fopen(ref_filename, "w");
+		if (!ref_log) {
+			ast_log(LOG_ERROR, "Could not open ref debug log file: %s\n", ref_filename);
+		}
 	}
-#endif
+
+	ast_register_cleanup(astobj2_cleanup);
 
 	if (container_init() != 0) {
 		fclose(ref_log);
+		ref_log = NULL;
 		return -1;
 	}
 
@@ -896,7 +1253,5 @@ int astobj2_init(void)
 	ast_cli_register_multiple(cli_astobj2, ARRAY_LEN(cli_astobj2));
 #endif	/* defined(AO2_DEBUG) */
 
-	ast_register_cleanup(astobj2_cleanup);
-
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/astobj2_container.c 1:16.1.1~dfsg-1/main/astobj2_container.c
--- 1:13.23.1~dfsg-2/main/astobj2_container.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2_container.c	2018-12-26 18:45:13.000000000 +0000
@@ -22,8 +22,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "astobj2_private.h"
@@ -49,11 +47,7 @@ int __container_unlink_node_debug(struct
 
 	if ((flags & AO2_UNLINK_NODE_UNLINK_OBJECT)
 		&& !(flags & AO2_UNLINK_NODE_NOUNREF_OBJECT)) {
-		if (tag) {
-			__ao2_ref_debug(node->obj, -1, tag, file, line, func);
-		} else {
-			ao2_t_ref(node->obj, -1, "Remove obj from container");
-		}
+		__ao2_ref(node->obj, -1, tag ?: "Remove obj from container", file, line, func);
 	}
 
 	node->obj = NULL;
@@ -76,7 +70,7 @@ int __container_unlink_node_debug(struct
 
 	if (flags & AO2_UNLINK_NODE_UNREF_NODE) {
 		/* Remove node from container */
-		__ao2_ref(node, -1);
+		ao2_t_ref(node, -1, NULL);
 	}
 
 	return 1;
@@ -97,19 +91,21 @@ int __container_unlink_node_debug(struct
  * \retval 0 on errors.
  * \retval 1 on success.
  */
-static int internal_ao2_link(struct ao2_container *self, void *obj_new, int flags, const char *tag, const char *file, int line, const char *func)
+int __ao2_link(struct ao2_container *self, void *obj_new, int flags,
+	const char *tag, const char *file, int line, const char *func)
 {
 	int res;
 	enum ao2_lock_req orig_lock;
 	struct ao2_container_node *node;
 
-	if (!is_ao2_object(obj_new) || !is_ao2_object(self)) {
+	if (!__is_ao2_object(obj_new, file, line, func)
+		|| !__is_ao2_object(self, file, line, func)) {
 		return 0;
 	}
 
 	if (!self->v_table || !self->v_table->new_node || !self->v_table->insert) {
 		/* Sanity checks. */
-		ast_assert(0);
+		__ast_assert_failed(0, "invalid container v_table", file, line, func);
 		return 0;
 	}
 
@@ -150,7 +146,7 @@ static int internal_ao2_link(struct ao2_
 			res = 1;
 			break;
 		case AO2_CONTAINER_INSERT_NODE_REJECTED:
-			__ao2_ref(node, -1);
+			ao2_t_ref(node, -1, NULL);
 			break;
 		}
 	}
@@ -164,16 +160,6 @@ static int internal_ao2_link(struct ao2_
 	return res;
 }
 
-int __ao2_link_debug(struct ao2_container *c, void *obj_new, int flags, const char *tag, const char *file, int line, const char *func)
-{
-	return internal_ao2_link(c, obj_new, flags, tag, file, line, func);
-}
-
-int __ao2_link(struct ao2_container *c, void *obj_new, int flags)
-{
-	return internal_ao2_link(c, obj_new, flags, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
 /*!
  * \brief another convenience function is a callback that matches on address
  */
@@ -186,29 +172,17 @@ int ao2_match_by_addr(void *user_data, v
  * Unlink an object from the container
  * and destroy the associated * bucket_entry structure.
  */
-void *__ao2_unlink_debug(struct ao2_container *c, void *user_data, int flags,
+void *__ao2_unlink(struct ao2_container *c, void *user_data, int flags,
 	const char *tag, const char *file, int line, const char *func)
 {
-	if (!is_ao2_object(user_data)) {
-		return NULL;
-	}
-
-	flags &= ~OBJ_SEARCH_MASK;
-	flags |= (OBJ_UNLINK | OBJ_SEARCH_OBJECT | OBJ_NODATA);
-	__ao2_callback_debug(c, flags, ao2_match_by_addr, user_data, tag, file, line, func);
-
-	return NULL;
-}
-
-void *__ao2_unlink(struct ao2_container *c, void *user_data, int flags)
-{
-	if (!is_ao2_object(user_data)) {
+	if (!__is_ao2_object(user_data, file, line, func)) {
+		/* Sanity checks. */
 		return NULL;
 	}
 
 	flags &= ~OBJ_SEARCH_MASK;
 	flags |= (OBJ_UNLINK | OBJ_SEARCH_OBJECT | OBJ_NODATA);
-	__ao2_callback(c, flags, ao2_match_by_addr, user_data);
+	__ao2_callback(c, flags, ao2_match_by_addr, user_data, tag, file, line, func);
 
 	return NULL;
 }
@@ -267,14 +241,14 @@ static void *internal_ao2_traverse(struc
 	struct ao2_container *multi_container = NULL;
 	struct ao2_iterator *multi_iterator = NULL;
 
-	if (!is_ao2_object(self)) {
+	if (!__is_ao2_object(self, file, line, func)) {
 		return NULL;
 	}
 
-	if (!self->v_table || !self->v_table->traverse_first
-		|| !self->v_table->traverse_next) {
+	if (!self->v_table
+		|| !self->v_table->traverse_first || !self->v_table->traverse_next) {
 		/* Sanity checks. */
-		ast_assert(0);
+		__ast_assert_failed(0, "invalid container v_table", file, line, func);
 		return NULL;
 	}
 
@@ -379,12 +353,7 @@ static void *internal_ao2_traverse(struc
 					 * Link the object into the container that will hold the
 					 * results.
 					 */
-					if (tag) {
-						__ao2_link_debug(multi_container, node->obj, flags,
-							tag, file, line, func);
-					} else {
-						__ao2_link(multi_container, node->obj, flags);
-					}
+					__ao2_link(multi_container, node->obj, flags, tag, file, line, func);
 				} else {
 					ret = node->obj;
 					/* Returning a single object. */
@@ -393,11 +362,7 @@ static void *internal_ao2_traverse(struc
 						 * Bump the ref count since we are not going to unlink and
 						 * transfer the container's object ref to the returned object.
 						 */
-						if (tag) {
-							__ao2_ref_debug(ret, 1, tag, file, line, func);
-						} else {
-							ao2_t_ref(ret, 1, "Traversal found object");
-						}
+						__ao2_ref(ret, 1, tag ?: "Traversal found object", file, line, func);
 					}
 				}
 			}
@@ -421,7 +386,7 @@ static void *internal_ao2_traverse(struc
 	}
 	if (node) {
 		/* Unref the node from self->v_table->traverse_first/traverse_next() */
-		__ao2_ref(node, -1);
+		ao2_t_ref(node, -1, NULL);
 	}
 
 	if (flags & OBJ_NOLOCK) {
@@ -442,36 +407,24 @@ static void *internal_ao2_traverse(struc
 	}
 }
 
-void *__ao2_callback_debug(struct ao2_container *c, enum search_flags flags,
+void *__ao2_callback(struct ao2_container *c, enum search_flags flags,
 	ao2_callback_fn *cb_fn, void *arg, const char *tag, const char *file, int line,
 	const char *func)
 {
 	return internal_ao2_traverse(c, flags, cb_fn, arg, NULL, AO2_CALLBACK_DEFAULT, tag, file, line, func);
 }
 
-void *__ao2_callback(struct ao2_container *c, enum search_flags flags,
-	ao2_callback_fn *cb_fn, void *arg)
-{
-	return internal_ao2_traverse(c, flags, cb_fn, arg, NULL, AO2_CALLBACK_DEFAULT, NULL, NULL, 0, NULL);
-}
-
-void *__ao2_callback_data_debug(struct ao2_container *c, enum search_flags flags,
+void *__ao2_callback_data(struct ao2_container *c, enum search_flags flags,
 	ao2_callback_data_fn *cb_fn, void *arg, void *data, const char *tag, const char *file,
 	int line, const char *func)
 {
 	return internal_ao2_traverse(c, flags, cb_fn, arg, data, AO2_CALLBACK_WITH_DATA, tag, file, line, func);
 }
 
-void *__ao2_callback_data(struct ao2_container *c, enum search_flags flags,
-	ao2_callback_data_fn *cb_fn, void *arg, void *data)
-{
-	return internal_ao2_traverse(c, flags, cb_fn, arg, data, AO2_CALLBACK_WITH_DATA, NULL, NULL, 0, NULL);
-}
-
 /*!
  * the find function just invokes the default callback with some reasonable flags.
  */
-void *__ao2_find_debug(struct ao2_container *c, const void *arg, enum search_flags flags,
+void *__ao2_find(struct ao2_container *c, const void *arg, enum search_flags flags,
 	const char *tag, const char *file, int line, const char *func)
 {
 	void *arged = (void *) arg;/* Done to avoid compiler const warning */
@@ -481,19 +434,49 @@ void *__ao2_find_debug(struct ao2_contai
 		ast_assert(0);
 		return NULL;
 	}
-	return __ao2_callback_debug(c, flags, c->cmp_fn, arged, tag, file, line, func);
+	return __ao2_callback(c, flags, c->cmp_fn, arged, tag, file, line, func);
 }
 
-void *__ao2_find(struct ao2_container *c, const void *arg, enum search_flags flags)
+void *__ao2_weakproxy_find(struct ao2_container *c, const void *arg, enum search_flags flags,
+	const char *tag, const char *file, int line, const char *func)
 {
-	void *arged = (void *) arg;/* Done to avoid compiler const warning */
+	void *proxy;
+	void *obj = NULL;
+	enum ao2_lock_req orig_lock;
 
-	if (!c) {
-		/* Sanity checks. */
-		ast_assert(0);
-		return NULL;
+	ast_assert(!!c);
+	ast_assert(flags & OBJ_SEARCH_MASK);
+	ast_assert(!(flags & ~(OBJ_SEARCH_MASK | OBJ_NOLOCK)));
+
+	if (flags & OBJ_NOLOCK) {
+		orig_lock = __adjust_lock(c, AO2_LOCK_REQ_RDLOCK, 1);
+	} else {
+		orig_lock = AO2_LOCK_REQ_RDLOCK;
+		ao2_rdlock(c);
 	}
-	return __ao2_callback(c, flags, c->cmp_fn, arged);
+
+	while ((proxy = ao2_find(c, arg, flags | OBJ_NOLOCK))) {
+		obj = __ao2_weakproxy_get_object(proxy, 0, tag ?: __PRETTY_FUNCTION__, file, line, func);
+
+		if (obj) {
+			ao2_ref(proxy, -1);
+			break;
+		}
+
+		/* Upgrade to a write lock */
+		__adjust_lock(c, AO2_LOCK_REQ_WRLOCK, 1);
+		ao2_unlink_flags(c, proxy, OBJ_NOLOCK);
+		ao2_ref(proxy, -1);
+	}
+
+	if (flags & OBJ_NOLOCK) {
+		/* We'll keep any upgraded lock */
+		__adjust_lock(c, orig_lock, 1);
+	} else {
+		ao2_unlock(c);
+	}
+
+	return obj;
 }
 
 /*!
@@ -514,7 +497,7 @@ struct ao2_iterator ao2_iterator_init(st
 void ao2_iterator_restart(struct ao2_iterator *iter)
 {
 	if (!is_ao2_object(iter->c)) {
-		ast_log(LOG_ERROR, "Iterator container is not valid.\n");
+		/* Sanity check. */
 		return;
 	}
 
@@ -534,7 +517,7 @@ void ao2_iterator_restart(struct ao2_ite
 			ao2_rdlock(iter->c);
 		}
 
-		__ao2_ref(iter->last_node, -1);
+		ao2_t_ref(iter->last_node, -1, NULL);
 		iter->last_node = NULL;
 
 		if (iter->flags & AO2_ITERATOR_DONTLOCK) {
@@ -570,22 +553,20 @@ void ao2_iterator_cleanup(struct ao2_ite
 	}
 }
 
-/*
- * move to the next element in the container.
- */
-static void *internal_ao2_iterator_next(struct ao2_iterator *iter, const char *tag, const char *file, int line, const char *func)
+void *__ao2_iterator_next(struct ao2_iterator *iter,
+	const char *tag, const char *file, int line, const char *func)
 {
 	enum ao2_lock_req orig_lock;
 	struct ao2_container_node *node;
 	void *ret;
 
-	if (!is_ao2_object(iter->c)) {
+	if (!__is_ao2_object(iter->c, file, line, func)) {
 		return NULL;
 	}
 
 	if (!iter->c->v_table || !iter->c->v_table->iterator_next) {
 		/* Sanity checks. */
-		ast_assert(0);
+		__ast_assert_failed(0, "invalid iterator container v_table", file, line, func);
 		return NULL;
 	}
 
@@ -620,14 +601,10 @@ static void *internal_ao2_iterator_next(
 			/* Transfer the container's node ref to the iterator. */
 		} else {
 			/* Bump ref of returned object */
-			if (tag) {
-				__ao2_ref_debug(ret, +1, tag, file, line, func);
-			} else {
-				ao2_t_ref(ret, +1, "Next iterator object.");
-			}
+			__ao2_ref(ret, +1, tag ?: "Next iterator object.", file, line, func);
 
 			/* Bump the container's node ref for the iterator. */
-			__ao2_ref(node, +1);
+			ao2_t_ref(node, +1, NULL);
 		}
 	} else {
 		/* The iteration has completed. */
@@ -637,7 +614,7 @@ static void *internal_ao2_iterator_next(
 
 	/* Replace the iterator's node */
 	if (iter->last_node) {
-		__ao2_ref(iter->last_node, -1);
+		ao2_t_ref(iter->last_node, -1, NULL);
 	}
 	iter->last_node = node;
 
@@ -650,16 +627,6 @@ static void *internal_ao2_iterator_next(
 	return ret;
 }
 
-void *__ao2_iterator_next_debug(struct ao2_iterator *iter, const char *tag, const char *file, int line, const char *func)
-{
-	return internal_ao2_iterator_next(iter, tag, file, line, func);
-}
-
-void *__ao2_iterator_next(struct ao2_iterator *iter)
-{
-	return internal_ao2_iterator_next(iter, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
 int ao2_iterator_count(struct ao2_iterator *iter)
 {
 	return ao2_container_count(iter->c);
@@ -671,26 +638,8 @@ void container_destruct(void *_c)
 
 	/* Unlink any stored objects in the container. */
 	c->destroying = 1;
-	__ao2_callback(c, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
-
-	/* Perform any extra container cleanup. */
-	if (c->v_table && c->v_table->destroy) {
-		c->v_table->destroy(c);
-	}
-
-#if defined(AO2_DEBUG)
-	ast_atomic_fetchadd_int(&ao2.total_containers, -1);
-#endif
-}
-
-void container_destruct_debug(void *_c)
-{
-	struct ao2_container *c = _c;
-
-	/* Unlink any stored objects in the container. */
-	c->destroying = 1;
-	__ao2_callback_debug(c, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL,
-		"container_destruct_debug called", __FILE__, __LINE__, __PRETTY_FUNCTION__);
+	ao2_t_callback(c, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL,
+		"container_destruct called");
 
 	/* Perform any extra container cleanup. */
 	if (c->v_table && c->v_table->destroy) {
@@ -718,7 +667,7 @@ static int dup_obj_cb(void *obj, void *a
 {
 	struct ao2_container *dest = arg;
 
-	return __ao2_link(dest, obj, OBJ_NOLOCK) ? 0 : (CMP_MATCH | CMP_STOP);
+	return ao2_t_link_flags(dest, obj, OBJ_NOLOCK, NULL) ? 0 : (CMP_MATCH | CMP_STOP);
 }
 
 int ao2_container_dup(struct ao2_container *dest, struct ao2_container *src, enum search_flags flags)
@@ -730,14 +679,14 @@ int ao2_container_dup(struct ao2_contain
 		ao2_rdlock(src);
 		ao2_wrlock(dest);
 	}
-	obj = __ao2_callback(src, OBJ_NOLOCK, dup_obj_cb, dest);
+	obj = ao2_callback(src, OBJ_NOLOCK, dup_obj_cb, dest);
 	if (obj) {
 		/* Failed to put this obj into the dest container. */
 		ao2_t_ref(obj, -1, "Failed to put this object into the dest container.");
 
 		/* Remove all items from the dest container. */
-		__ao2_callback(dest, OBJ_NOLOCK | OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL,
-			NULL);
+		ao2_t_callback(dest, OBJ_NOLOCK | OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL,
+			NULL, NULL);
 		res = -1;
 	}
 	if (!(flags & OBJ_NOLOCK)) {
@@ -748,61 +697,83 @@ int ao2_container_dup(struct ao2_contain
 	return res;
 }
 
-struct ao2_container *__ao2_container_clone(struct ao2_container *orig, enum search_flags flags)
+/*!
+ * \brief Copy obj associated with a weakproxy into the arg container.
+ *
+ * \param proxy pointer to the weakproxy.
+ * \param arg callback argument from ao2_callback()
+ * \param flags flags from ao2_callback()
+ *
+ * \retval 0 on success.
+ * \retval CMP_STOP|CMP_MATCH on error.
+ */
+static int dup_weakproxy_cb(void *proxy, void *arg, int flags)
 {
-	struct ao2_container *clone;
-	int failed;
+	void *obj = ao2_weakproxy_get_object(proxy, 0);
+	struct ao2_container *dest = arg;
+	int ret;
 
-	/* Create the clone container with the same properties as the original. */
-	if (!is_ao2_object(orig)) {
-		return NULL;
+	if (!obj) {
+		return 0;
 	}
 
-	if (!orig->v_table || !orig->v_table->alloc_empty_clone) {
-		/* Sanity checks. */
-		ast_assert(0);
-		return NULL;
-	}
-	clone = orig->v_table->alloc_empty_clone(orig);
-	if (!clone) {
-		return NULL;
-	}
+	ret = ao2_t_link_flags(dest, obj, OBJ_NOLOCK, NULL) ? 0 : (CMP_MATCH | CMP_STOP);
+	ao2_ref(obj, -1);
 
-	if (flags & OBJ_NOLOCK) {
-		ao2_wrlock(clone);
+	return ret;
+}
+
+int ao2_container_dup_weakproxy_objs(struct ao2_container *dest, struct ao2_container *src, enum search_flags flags)
+{
+	void *obj;
+	int res = 0;
+
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_rdlock(src);
+		ao2_wrlock(dest);
 	}
-	failed = ao2_container_dup(clone, orig, flags);
-	if (flags & OBJ_NOLOCK) {
-		ao2_unlock(clone);
+	obj = ao2_callback(src, OBJ_NOLOCK, dup_weakproxy_cb, dest);
+	if (obj) {
+		/* Failed to put this obj into the dest container. */
+		ao2_t_ref(obj, -1, "Failed to put this object into the dest container.");
+
+		/* Remove all items from the dest container. */
+		ao2_t_callback(dest, OBJ_NOLOCK | OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL,
+			NULL, NULL);
+		res = -1;
 	}
-	if (failed) {
-		/* Object copy into the clone container failed. */
-		ao2_t_ref(clone, -1, "Clone creation failed.");
-		clone = NULL;
+	if (!(flags & OBJ_NOLOCK)) {
+		ao2_unlock(dest);
+		ao2_unlock(src);
 	}
-	return clone;
+
+	return res;
 }
 
-struct ao2_container *__ao2_container_clone_debug(struct ao2_container *orig, enum search_flags flags, const char *tag, const char *file, int line, const char *func, int ref_debug)
+struct ao2_container *__ao2_container_clone(struct ao2_container *orig, enum search_flags flags, const char *tag, const char *file, int line, const char *func)
 {
 	struct ao2_container *clone;
 	int failed;
 
 	/* Create the clone container with the same properties as the original. */
-	if (!is_ao2_object(orig)) {
+	if (!__is_ao2_object(orig, file, line, func)) {
 		return NULL;
 	}
 
-	if (!orig->v_table || !orig->v_table->alloc_empty_clone_debug) {
+	if (!orig->v_table || !orig->v_table->alloc_empty_clone) {
 		/* Sanity checks. */
-		ast_assert(0);
+		__ast_assert_failed(0, "invalid container v_table", file, line, func);
 		return NULL;
 	}
-	clone = orig->v_table->alloc_empty_clone_debug(orig, tag, file, line, func, ref_debug);
+
+	clone = orig->v_table->alloc_empty_clone(orig, tag, file, line, func);
 	if (!clone) {
 		return NULL;
 	}
 
+	/* This test is correct.  clone must be locked before calling
+	 * ao2_container_dup when the OBJ_NOLOCK flag is set, otherwise
+	 * we could have errors in __adjust_lock. */
 	if (flags & OBJ_NOLOCK) {
 		ao2_wrlock(clone);
 	}
@@ -812,11 +783,7 @@ struct ao2_container *__ao2_container_cl
 	}
 	if (failed) {
 		/* Object copy into the clone container failed. */
-		if (ref_debug) {
-			__ao2_ref_debug(clone, -1, tag, file, line, func);
-		} else {
-			ao2_t_ref(clone, -1, "Clone creation failed.");
-		}
+		__ao2_ref(clone, -1, tag ?: "Clone creation failed", file, line, func);
 		clone = NULL;
 	}
 	return clone;
diff -pruN 1:13.23.1~dfsg-2/main/astobj2_container_private.h 1:16.1.1~dfsg-1/main/astobj2_container_private.h
--- 1:13.23.1~dfsg-2/main/astobj2_container_private.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2_container_private.h	2018-12-26 18:45:13.000000000 +0000
@@ -87,26 +87,15 @@ typedef void (*ao2_container_destroy_fn)
  * \brief Create an empty copy of this container.
  *
  * \param self Container to operate upon.
- *
- * \retval empty-container on success.
- * \retval NULL on error.
- */
-typedef struct ao2_container *(*ao2_container_alloc_empty_clone_fn)(struct ao2_container *self);
-
-/*!
- * \brief Create an empty copy of this container. (Debug version)
- *
- * \param self Container to operate upon.
  * \param tag used for debugging.
  * \param file Debug file name invoked from
  * \param line Debug line invoked from
  * \param func Debug function name invoked from
- * \param ref_debug TRUE if to output a debug reference message.
  *
  * \retval empty-container on success.
  * \retval NULL on error.
  */
-typedef struct ao2_container *(*ao2_container_alloc_empty_clone_debug_fn)(struct ao2_container *self, const char *tag, const char *file, int line, const char *func, int ref_debug);
+typedef struct ao2_container *(*ao2_container_alloc_empty_clone_fn)(struct ao2_container *self, const char *tag, const char *file, int line, const char *func);
 
 /*!
  * \brief Create a new container node.
@@ -250,8 +239,6 @@ struct ao2_container_methods {
 	ao2_container_destroy_fn destroy;
 	/*! \brief Create an empty copy of this container. */
 	ao2_container_alloc_empty_clone_fn alloc_empty_clone;
-	/*! \brief Create an empty copy of this container. (Debug version) */
-	ao2_container_alloc_empty_clone_debug_fn alloc_empty_clone_debug;
 	/*! Create a new container node. */
 	ao2_container_new_node_fn new_node;
 	/*! Insert a node into this container. */
@@ -336,10 +323,9 @@ int __container_unlink_node_debug(struct
 	const char *tag, const char *file, int line, const char *func);
 
 #define __container_unlink_node(node, flags) \
-	__container_unlink_node_debug(node, flags, NULL, NULL, 0, NULL)
+	__container_unlink_node_debug(node, flags, NULL, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 void container_destruct(void *_c);
-void container_destruct_debug(void *_c);
 int container_init(void);
 
 #endif /* ASTOBJ2_CONTAINER_PRIVATE_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/astobj2_hash.c 1:16.1.1~dfsg-1/main/astobj2_hash.c
--- 1:13.23.1~dfsg-2/main/astobj2_hash.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2_hash.c	2018-12-26 18:45:13.000000000 +0000
@@ -23,8 +23,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "astobj2_private.h"
@@ -104,47 +102,27 @@ struct hash_traversal_state_check {
 /*!
  * \internal
  * \brief Create an empty copy of this container.
- * \since 12.0.0
- *
- * \param self Container to operate upon.
- *
- * \retval empty-clone-container on success.
- * \retval NULL on error.
- */
-static struct ao2_container *hash_ao2_alloc_empty_clone(struct ao2_container_hash *self)
-{
-	if (!is_ao2_object(self)) {
-		return NULL;
-	}
-
-	return ao2_t_container_alloc_hash(ao2_options_get(self), self->common.options, self->n_buckets,
-		self->hash_fn, self->common.sort_fn, self->common.cmp_fn, "Clone hash container");
-}
-
-/*!
- * \internal
- * \brief Create an empty copy of this container. (Debug version)
- * \since 12.0.0
+ * \since 14.0.0
  *
  * \param self Container to operate upon.
  * \param tag used for debugging.
  * \param file Debug file name invoked from
  * \param line Debug line invoked from
  * \param func Debug function name invoked from
- * \param ref_debug TRUE if to output a debug reference message.
  *
  * \retval empty-clone-container on success.
  * \retval NULL on error.
  */
-static struct ao2_container *hash_ao2_alloc_empty_clone_debug(struct ao2_container_hash *self, const char *tag, const char *file, int line, const char *func, int ref_debug)
+static struct ao2_container *hash_ao2_alloc_empty_clone(struct ao2_container_hash *self,
+	const char *tag, const char *file, int line, const char *func)
 {
-	if (!is_ao2_object(self)) {
+	if (!__is_ao2_object(self, file, line, func)) {
 		return NULL;
 	}
 
-	return __ao2_container_alloc_hash_debug(ao2_options_get(self), self->common.options,
+	return __ao2_container_alloc_hash(ao2_options_get(self), self->common.options,
 		self->n_buckets, self->hash_fn, self->common.sort_fn, self->common.cmp_fn,
-		tag, file, line, func, ref_debug);
+		tag, file, line, func);
 }
 
 /*!
@@ -186,7 +164,7 @@ static void hash_ao2_node_destructor(voi
 		 * same node.
 		 */
 		my_container = (struct ao2_container_hash *) doomed->common.my_container;
-#if defined(AST_DEVMODE)
+#ifdef AST_DEVMODE
 		is_ao2_object(my_container);
 #endif
 
@@ -232,18 +210,14 @@ static struct hash_bucket_node *hash_ao2
 	struct hash_bucket_node *node;
 	int i;
 
-	node = __ao2_alloc(sizeof(*node), hash_ao2_node_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	node = ao2_t_alloc_options(sizeof(*node), hash_ao2_node_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK, NULL);
 	if (!node) {
 		return NULL;
 	}
 
 	i = abs(self->hash_fn(obj_new, OBJ_SEARCH_OBJECT) % self->n_buckets);
 
-	if (tag) {
-		__ao2_ref_debug(obj_new, +1, tag, file, line, func);
-	} else {
-		ao2_t_ref(obj_new, +1, "Container node creation");
-	}
+	__ao2_ref(obj_new, +1, tag ?: "Container node creation", file, line, func);
 	node->common.obj = obj_new;
 	node->common.my_container = (struct ao2_container *) self;
 	node->my_bucket = i;
@@ -300,7 +274,7 @@ static enum ao2_container_insert hash_ao
 					break;
 				case AO2_CONTAINER_ALLOC_OPT_DUPS_REPLACE:
 					SWAP(cur->common.obj, node->common.obj);
-					__ao2_ref(node, -1);
+					ao2_t_ref(node, -1, NULL);
 					return AO2_CONTAINER_INSERT_NODE_OBJ_REPLACED;
 				}
 			}
@@ -333,7 +307,7 @@ static enum ao2_container_insert hash_ao
 					break;
 				case AO2_CONTAINER_ALLOC_OPT_DUPS_REPLACE:
 					SWAP(cur->common.obj, node->common.obj);
-					__ao2_ref(node, -1);
+					ao2_t_ref(node, -1, NULL);
 					return AO2_CONTAINER_INSERT_NODE_OBJ_REPLACED;
 				}
 			}
@@ -441,7 +415,7 @@ static struct hash_bucket_node *hash_ao2
 				}
 
 				/* We have the first traversal node */
-				__ao2_ref(node, +1);
+				ao2_t_ref(node, +1, NULL);
 				return node;
 			}
 		}
@@ -483,7 +457,7 @@ static struct hash_bucket_node *hash_ao2
 				}
 
 				/* We have the first traversal node */
-				__ao2_ref(node, +1);
+				ao2_t_ref(node, +1, NULL);
 				return node;
 			}
 		}
@@ -552,7 +526,7 @@ static struct hash_bucket_node *hash_ao2
 				}
 
 				/* We have the next traversal node */
-				__ao2_ref(node, +1);
+				ao2_t_ref(node, +1, NULL);
 
 				/*
 				 * Dereferencing the prev node may result in our next node
@@ -560,7 +534,7 @@ static struct hash_bucket_node *hash_ao2
 				 * the container uses RW locks and the container was read
 				 * locked.
 				 */
-				__ao2_ref(prev, -1);
+				ao2_t_ref(prev, -1, NULL);
 				if (node->common.obj) {
 					return node;
 				}
@@ -596,7 +570,7 @@ hash_descending_resume:;
 				}
 
 				/* We have the next traversal node */
-				__ao2_ref(node, +1);
+				ao2_t_ref(node, +1, NULL);
 
 				/*
 				 * Dereferencing the prev node may result in our next node
@@ -604,7 +578,7 @@ hash_descending_resume:;
 				 * the container uses RW locks and the container was read
 				 * locked.
 				 */
-				__ao2_ref(prev, -1);
+				ao2_t_ref(prev, -1, NULL);
 				if (node->common.obj) {
 					return node;
 				}
@@ -616,7 +590,7 @@ hash_ascending_resume:;
 	}
 
 	/* No more nodes in the container left to traverse. */
-	__ao2_ref(prev, -1);
+	ao2_t_ref(prev, -1, NULL);
 	return NULL;
 }
 
@@ -1038,8 +1012,6 @@ static int hash_ao2_integrity(struct ao2
 /*! Hash container virtual method table. */
 static const struct ao2_container_methods v_table_hash = {
 	.alloc_empty_clone = (ao2_container_alloc_empty_clone_fn) hash_ao2_alloc_empty_clone,
-	.alloc_empty_clone_debug =
-		(ao2_container_alloc_empty_clone_debug_fn) hash_ao2_alloc_empty_clone_debug,
 	.new_node = (ao2_container_new_node_fn) hash_ao2_new_node,
 	.insert = (ao2_container_insert_fn) hash_ao2_insert_node,
 	.traverse_first = (ao2_container_find_first_fn) hash_ao2_find_first,
@@ -1105,25 +1077,8 @@ static struct ao2_container *hash_ao2_co
 
 struct ao2_container *__ao2_container_alloc_hash(unsigned int ao2_options,
 	unsigned int container_options, unsigned int n_buckets, ao2_hash_fn *hash_fn,
-	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn)
-{
-	unsigned int num_buckets;
-	size_t container_size;
-	struct ao2_container_hash *self;
-
-	num_buckets = hash_fn ? n_buckets : 1;
-	container_size = sizeof(struct ao2_container_hash) + num_buckets * sizeof(struct hash_bucket);
-
-	self = ao2_t_alloc_options(container_size, container_destruct, ao2_options,
-		"New hash container");
-	return hash_ao2_container_init(self, container_options, num_buckets,
-		hash_fn, sort_fn, cmp_fn);
-}
-
-struct ao2_container *__ao2_container_alloc_hash_debug(unsigned int ao2_options,
-	unsigned int container_options, unsigned int n_buckets, ao2_hash_fn *hash_fn,
 	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug)
+	const char *tag, const char *file, int line, const char *func)
 {
 	unsigned int num_buckets;
 	size_t container_size;
@@ -1132,24 +1087,16 @@ struct ao2_container *__ao2_container_al
 	num_buckets = hash_fn ? n_buckets : 1;
 	container_size = sizeof(struct ao2_container_hash) + num_buckets * sizeof(struct hash_bucket);
 
-	self = __ao2_alloc_debug(container_size,
-		ref_debug ? container_destruct_debug : container_destruct, ao2_options,
-		tag, file, line, func, ref_debug);
+	self = __ao2_alloc(container_size, container_destruct, ao2_options,
+		tag ?: __PRETTY_FUNCTION__, file, line, func);
 	return hash_ao2_container_init(self, container_options, num_buckets, hash_fn,
 		sort_fn, cmp_fn);
 }
 
 struct ao2_container *__ao2_container_alloc_list(unsigned int ao2_options,
-	unsigned int container_options, ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn)
-{
-	return __ao2_container_alloc_hash(ao2_options, container_options, 1, NULL, sort_fn,
-		cmp_fn);
-}
-
-struct ao2_container *__ao2_container_alloc_list_debug(unsigned int ao2_options,
 	unsigned int container_options, ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug)
+	const char *tag, const char *file, int line, const char *func)
 {
-	return __ao2_container_alloc_hash_debug(ao2_options, container_options, 1, NULL,
-		sort_fn, cmp_fn, tag, file, line, func, ref_debug);
+	return __ao2_container_alloc_hash(ao2_options, container_options, 1, NULL,
+		sort_fn, cmp_fn, tag, file, line, func);
 }
diff -pruN 1:13.23.1~dfsg-2/main/astobj2_private.h 1:16.1.1~dfsg-1/main/astobj2_private.h
--- 1:13.23.1~dfsg-2/main/astobj2_private.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2_private.h	2018-12-26 18:45:13.000000000 +0000
@@ -43,7 +43,24 @@ struct ao2_stats {
 extern struct ao2_stats ao2;
 #endif	/* defined(AO2_DEBUG) */
 
-int is_ao2_object(void *user_data);
+void log_bad_ao2(void *user_data, const char *file, int line, const char *func);
+int internal_is_ao2_object(void *user_data);
+
+#define __is_ao2_object(user_data, file, line, func) \
+	({ \
+		int ret ## __LINE__ = 0; \
+		if (user_data) { \
+			ret ## __LINE__ = internal_is_ao2_object(user_data); \
+		} \
+		if (!ret ## __LINE__) { \
+			log_bad_ao2(user_data, file, line, func); \
+		} \
+		(ret ## __LINE__); \
+	})
+
+#define is_ao2_object(user_data) \
+	__is_ao2_object(user_data, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 enum ao2_lock_req __adjust_lock(void *user_data, enum ao2_lock_req lock_how, int keep_stronger);
 
 #endif /* ASTOBJ2_PRIVATE_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/astobj2_rbtree.c 1:16.1.1~dfsg-1/main/astobj2_rbtree.c
--- 1:13.23.1~dfsg-2/main/astobj2_rbtree.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/astobj2_rbtree.c	2018-12-26 18:45:13.000000000 +0000
@@ -23,8 +23,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/utils.h"
@@ -545,47 +543,27 @@ static void rb_rotate_right(struct ao2_c
 
 /*!
  * \internal
- * \brief Create an empty copy of this container.
- * \since 12.0.0
- *
- * \param self Container to operate upon.
- *
- * \retval empty-clone-container on success.
- * \retval NULL on error.
- */
-static struct ao2_container *rb_ao2_alloc_empty_clone(struct ao2_container_rbtree *self)
-{
-	if (!is_ao2_object(self)) {
-		return NULL;
-	}
-
-	return ao2_t_container_alloc_rbtree(ao2_options_get(self), self->common.options,
-		self->common.sort_fn, self->common.cmp_fn, "Clone rbtree container");
-}
-
-/*!
- * \internal
  * \brief Create an empty copy of this container. (Debug version)
- * \since 12.0.0
+ * \since 14.0.0
  *
  * \param self Container to operate upon.
  * \param tag used for debugging.
  * \param file Debug file name invoked from
  * \param line Debug line invoked from
  * \param func Debug function name invoked from
- * \param ref_debug TRUE if to output a debug reference message.
  *
  * \retval empty-clone-container on success.
  * \retval NULL on error.
  */
-static struct ao2_container *rb_ao2_alloc_empty_clone_debug(struct ao2_container_rbtree *self, const char *tag, const char *file, int line, const char *func, int ref_debug)
+static struct ao2_container *rb_ao2_alloc_empty_clone(struct ao2_container_rbtree *self,
+	const char *tag, const char *file, int line, const char *func)
 {
-	if (!is_ao2_object(self)) {
+	if (!__is_ao2_object(self, file, line, func)) {
 		return NULL;
 	}
 
-	return __ao2_container_alloc_rbtree_debug(ao2_options_get(self), self->common.options,
-		self->common.sort_fn, self->common.cmp_fn, tag, file, line, func, ref_debug);
+	return __ao2_container_alloc_rbtree(ao2_options_get(self), self->common.options,
+		self->common.sort_fn, self->common.cmp_fn, tag, file, line, func);
 }
 
 /*!
@@ -878,7 +856,7 @@ static void rb_ao2_node_destructor(void
 		 * same node.
 		 */
 		my_container = (struct ao2_container_rbtree *) doomed->common.my_container;
-#if defined(AST_DEVMODE)
+#ifdef AST_DEVMODE
 		is_ao2_object(my_container);
 #endif
 
@@ -927,16 +905,12 @@ static struct rbtree_node *rb_ao2_new_no
 {
 	struct rbtree_node *node;
 
-	node = __ao2_alloc(sizeof(*node), rb_ao2_node_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	node = ao2_t_alloc_options(sizeof(*node), rb_ao2_node_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK, NULL);
 	if (!node) {
 		return NULL;
 	}
 
-	if (tag) {
-		__ao2_ref_debug(obj_new, +1, tag, file, line, func);
-	} else {
-		ao2_t_ref(obj_new, +1, "Container node creation");
-	}
+	__ao2_ref(obj_new, +1, tag ?: "Container node creation", file, line, func);
 	node->common.obj = obj_new;
 	node->common.my_container = (struct ao2_container *) self;
 
@@ -1269,7 +1243,7 @@ static enum ao2_container_insert rb_ao2_
 		break;
 	case AO2_CONTAINER_ALLOC_OPT_DUPS_REPLACE:
 		SWAP(cur->common.obj, node->common.obj);
-		__ao2_ref(node, -1);
+		ao2_t_ref(node, -1, NULL);
 		return AO2_CONTAINER_INSERT_NODE_OBJ_REPLACED;
 	}
 
@@ -1339,7 +1313,7 @@ static struct rbtree_node *rb_ao2_find_n
 		}
 
 		/* We have the next traversal node */
-		__ao2_ref(node, +1);
+		ao2_t_ref(node, +1, NULL);
 
 		/*
 		 * Dereferencing the prev node may result in our next node
@@ -1347,7 +1321,7 @@ static struct rbtree_node *rb_ao2_find_n
 		 * the container uses RW locks and the container was read
 		 * locked.
 		 */
-		__ao2_ref(prev, -1);
+		ao2_t_ref(prev, -1, NULL);
 		if (node->common.obj) {
 			return node;
 		}
@@ -1355,7 +1329,7 @@ static struct rbtree_node *rb_ao2_find_n
 	}
 
 	/* No more nodes in the container left to traverse. */
-	__ao2_ref(prev, -1);
+	ao2_t_ref(prev, -1, NULL);
 	return NULL;
 }
 
@@ -1638,7 +1612,7 @@ static struct rbtree_node *rb_ao2_find_f
 	}
 
 	/* We have the first traversal node */
-	__ao2_ref(node, +1);
+	ao2_t_ref(node, +1, NULL);
 	return node;
 }
 
@@ -2020,8 +1994,6 @@ static int rb_ao2_integrity(struct ao2_c
 /*! rbtree container virtual method table. */
 static const struct ao2_container_methods v_table_rbtree = {
 	.alloc_empty_clone = (ao2_container_alloc_empty_clone_fn) rb_ao2_alloc_empty_clone,
-	.alloc_empty_clone_debug =
-		(ao2_container_alloc_empty_clone_debug_fn) rb_ao2_alloc_empty_clone_debug,
 	.new_node = (ao2_container_new_node_fn) rb_ao2_new_node,
 	.insert = (ao2_container_insert_fn) rb_ao2_insert_node,
 	.traverse_first = (ao2_container_find_first_fn) rb_ao2_find_first,
@@ -2065,24 +2037,8 @@ static struct ao2_container *rb_ao2_cont
 }
 
 struct ao2_container *__ao2_container_alloc_rbtree(unsigned int ao2_options, unsigned int container_options,
-	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn)
-{
-	struct ao2_container_rbtree *self;
-
-	if (!sort_fn) {
-		/* Sanity checks. */
-		ast_log(LOG_ERROR, "Missing sort_fn()!\n");
-		return NULL;
-	}
-
-	self = ao2_t_alloc_options(sizeof(*self), container_destruct, ao2_options,
-		"New rbtree container");
-	return rb_ao2_container_init(self, container_options, sort_fn, cmp_fn);
-}
-
-struct ao2_container *__ao2_container_alloc_rbtree_debug(unsigned int ao2_options, unsigned int container_options,
 	ao2_sort_fn *sort_fn, ao2_callback_fn *cmp_fn,
-	const char *tag, const char *file, int line, const char *func, int ref_debug)
+	const char *tag, const char *file, int line, const char *func)
 {
 	struct ao2_container_rbtree *self;
 
@@ -2092,8 +2048,7 @@ struct ao2_container *__ao2_container_al
 		return NULL;
 	}
 
-	self = __ao2_alloc_debug(sizeof(*self),
-		ref_debug ? container_destruct_debug : container_destruct, ao2_options,
-		tag, file, line, func, ref_debug);
+	self = __ao2_alloc(sizeof(*self), container_destruct, ao2_options,
+		tag ?: __PRETTY_FUNCTION__, file, line, func);
 	return rb_ao2_container_init(self, container_options, sort_fn, cmp_fn);
 }
diff -pruN 1:13.23.1~dfsg-2/main/audiohook.c 1:16.1.1~dfsg-1/main/audiohook.c
--- 1:13.23.1~dfsg-2/main/audiohook.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/audiohook.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/channel.h"
@@ -45,6 +43,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #define AST_AUDIOHOOK_SYNC_TOLERANCE 100 /*!< Tolerance in milliseconds for audiohooks synchronization */
 #define AST_AUDIOHOOK_SMALL_QUEUE_TOLERANCE 100 /*!< When small queue is enabled, this is the maximum amount of audio that can remain queued at a time. */
+#define AST_AUDIOHOOK_LONG_QUEUE_TOLERANCE 500 /*!< Otheriwise we still don't want the queue to grow indefinitely */
 
 #define DEFAULT_INTERNAL_SAMPLE_RATE 8000
 
@@ -196,6 +195,10 @@ int ast_audiohook_write_frame(struct ast
 		ast_debug(1, "Audiohook %p has stale audio in its factories. Flushing them both\n", audiohook);
 		ast_slinfactory_flush(factory);
 		ast_slinfactory_flush(other_factory);
+	} else if ((our_factory_ms > AST_AUDIOHOOK_LONG_QUEUE_TOLERANCE) || (other_factory_ms > AST_AUDIOHOOK_LONG_QUEUE_TOLERANCE)) {
+		ast_debug(1, "Audiohook %p has stale audio in its factories. Flushing them both\n", audiohook);
+		ast_slinfactory_flush(factory);
+		ast_slinfactory_flush(other_factory);
 	}
 
 	/* Write frame out to respective factory */
diff -pruN 1:13.23.1~dfsg-2/main/autochan.c 1:16.1.1~dfsg-1/main/autochan.c
--- 1:13.23.1~dfsg-2/main/autochan.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/autochan.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/autochan.h"
 #include "asterisk/utils.h"
 #include "asterisk/linkedlists.h"
diff -pruN 1:13.23.1~dfsg-2/main/autoservice.c 1:16.1.1~dfsg-1/main/autoservice.c
--- 1:13.23.1~dfsg-2/main/autoservice.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/autoservice.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 
@@ -60,6 +58,7 @@ struct asent {
 	 *  it gets stopped for the last time. */
 	unsigned int use_count;
 	unsigned int orig_end_dtmf_flag:1;
+	unsigned int video_update:1;
 	unsigned int ignore_frame_types;
 	/*! Frames go on at the head of deferred_frames, so we have the frames
 	 *  from newest to oldest.  As we put them at the head of the readq, we'll
@@ -78,7 +77,7 @@ static int as_chan_list_state;
 
 static void *autoservice_run(void *ign)
 {
-	struct ast_callid *callid = NULL;
+	ast_callid callid = 0;
 	struct ast_frame hangup_frame = {
 		.frametype = AST_FRAME_CONTROL,
 		.subclass.integer = AST_CONTROL_HANGUP,
@@ -132,9 +131,6 @@ static void *autoservice_run(void *ign)
 
 		callid = ast_channel_callid(chan);
 		ast_callid_threadassoc_change(callid);
-		if (callid) {
-			callid = ast_callid_unref(callid);
-		}
 
 		f = ast_read(chan);
 
@@ -166,6 +162,17 @@ static void *autoservice_run(void *ign)
 					AST_LIST_INSERT_HEAD(&ents[i]->deferred_frames, dup_f, frame_list);
 				}
 			} else {
+				if (defer_frame->frametype == AST_FRAME_CONTROL &&
+					defer_frame->subclass.integer == AST_CONTROL_VIDUPDATE) {
+
+					/* If a video update is already queued don't needlessly queue another */
+					if (ents[i]->video_update) {
+						ast_frfree(defer_frame);
+						break;
+					}
+
+					ents[i]->video_update = 1;
+				}
 				if ((dup_f = ast_frisolate(defer_frame))) {
 					AST_LIST_INSERT_HEAD(&ents[i]->deferred_frames, dup_f, frame_list);
 				}
@@ -184,7 +191,7 @@ static void *autoservice_run(void *ign)
 		 * If we did, we'd need to ast_frfree(f) if (f). */
 	}
 
-	ast_callid_threadassoc_change(NULL);
+	ast_callid_threadassoc_change(0);
 	asthread = AST_PTHREADT_NULL;
 
 	return NULL;
@@ -243,7 +250,7 @@ int ast_autoservice_start(struct ast_cha
 			/* There will only be a single member in the list at this point,
 			   the one we just added. */
 			AST_LIST_REMOVE(&aslist, as, list);
-			free(as);
+			ast_free(as);
 			asthread = AST_PTHREADT_NULL;
 			res = -1;
 		} else {
@@ -327,7 +334,7 @@ int ast_autoservice_stop(struct ast_chan
 	}
 	ast_channel_unlock(chan);
 
-	free(as);
+	ast_free(as);
 
 	return res;
 }
diff -pruN 1:13.23.1~dfsg-2/main/backtrace.c 1:16.1.1~dfsg-1/main/backtrace.c
--- 1:13.23.1~dfsg-2/main/backtrace.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/backtrace.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,12 +26,33 @@
 	<support_level>core</support_level>
  ***/
 
-#include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
+/*
+ * Block automatic include of asterisk/lock.h to allow use of pthread_mutex
+ * functions directly.  We don't need or want the lock.h overhead.
+ */
+#define _ASTERISK_LOCK_H
+
+/*
+ * The astmm ast_ memory management functions can cause ast_bt_get_symbols
+ * to be invoked so we must not use them.
+ */
+#define ASTMM_LIBC ASTMM_IGNORE
 
+#include "asterisk.h"
 #include "asterisk/backtrace.h"
-#include "asterisk/utils.h"
-#include "asterisk/strings.h"
+
+/*
+ * As stated above, the vector macros call the ast_ functions so
+ * we need to remap those back to the libc ones.
+ */
+#undef ast_free
+#undef ast_calloc
+#undef ast_malloc
+#define ast_free(x) free(x)
+#define ast_calloc(n, x) calloc(n, x)
+#define ast_malloc(x) malloc(x)
+
+#include "asterisk/vector.h"
 
 #ifdef HAVE_BKTR
 #include <execinfo.h>
@@ -40,9 +61,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <bfd.h>
 #endif
 
+#include <pthread.h>
+
+/* simple definition of S_OR so we don't have include strings.h */
+#define S_OR(a, b) (a && a[0] != '\0') ? a : b
+
 struct ast_bt *__ast_bt_create(void)
 {
-	struct ast_bt *bt = ast_std_calloc(1, sizeof(*bt));
+	struct ast_bt *bt = calloc(1, sizeof(*bt));
 
 	if (!bt) {
 		return NULL;
@@ -63,164 +89,217 @@ int __ast_bt_get_addresses(struct ast_bt
 void *__ast_bt_destroy(struct ast_bt *bt)
 {
 	if (bt && bt->alloced) {
-		ast_std_free(bt);
+		free(bt);
 	}
 	return NULL;
 }
 
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames)
-{
-	char **strings;
-#if defined(BETTER_BACKTRACES)
-	int stackfr;
-	bfd *bfdobj;           /* bfd.h */
+#ifdef BETTER_BACKTRACES
+
+struct bfd_data {
+	struct ast_vector_string *return_strings;
+	bfd_vma pc;            /* bfd.h */
+	asymbol **syms;        /* bfd.h */
 	Dl_info dli;           /* dlfcn.h */
-	long allocsize;
-	asymbol **syms = NULL; /* bfd.h */
-	bfd_vma offset;        /* bfd.h */
-	const char *lastslash;
-	asection *section;
+	const char *libname;
+	int dynamic;
+	int has_syms;
+	char *msg;
+	int found;
+};
+
+#define MSG_BUFF_LEN 1024
+
+static void process_section(bfd *bfdobj, asection *section, void *obj)
+{
+	struct bfd_data *data = obj;
 	const char *file, *func;
 	unsigned int line;
-	char address_str[128];
-	char msg[1024];
-	size_t strings_size;
-	size_t *eachlen;
-#endif
+	bfd_vma offset;
+	bfd_vma vma;
+	bfd_size_type size;
+	bfd_boolean line_found = 0;
+	char *fn;
+	int inlined = 0;
 
-#if defined(BETTER_BACKTRACES)
-	strings_size = num_frames * sizeof(*strings);
+	offset = data->pc - (data->dynamic ? (bfd_vma) data->dli.dli_fbase : 0);
 
-	eachlen = ast_std_calloc(num_frames, sizeof(*eachlen));
-	strings = ast_std_calloc(num_frames, sizeof(*strings));
-	if (!eachlen || !strings) {
-		ast_std_free(eachlen);
-		ast_std_free(strings);
+	if (!(bfd_get_section_flags(bfdobj, section) & SEC_ALLOC)) {
+		return;
+	}
+
+	vma = bfd_get_section_vma(bfdobj, section);
+	size = bfd_get_section_size(section);
+
+	if (offset < vma || offset >= vma + size) {
+		/* Not in this section */
+		return;
+	}
+
+	line_found = bfd_find_nearest_line(bfdobj, section, data->syms, offset - vma, &file,
+		&func, &line);
+	if (!line_found) {
+		return;
+	}
+
+	/*
+	 * If we find a line, we will want to continue calling bfd_find_inliner_info
+	 * to capture any inlined functions that don't have their own stack frames.
+	 */
+	do {
+		data->found++;
+		/* file can possibly be null even with a success result from bfd_find_nearest_line */
+		file = file ? file : "";
+		fn = strrchr(file, '/');
+#define FMT_INLINED "[%s] %s %s:%u %s()"
+#define FMT_NOT_INLINED "[%p] %s %s:%u %s()"
+
+		snprintf(data->msg, MSG_BUFF_LEN, inlined ? FMT_INLINED : FMT_NOT_INLINED,
+			inlined ? "inlined" : (char *)data->pc,
+			data->libname,
+			fn ? fn + 1 : file,
+			line, S_OR(func, "???"));
+
+		if (AST_VECTOR_APPEND(data->return_strings, strdup(data->msg))) {
+			return;
+		}
+
+		inlined++;
+		/* Let's see if there are any inlined functions */
+	} while (bfd_find_inliner_info(bfdobj, &file, &func, &line));
+}
+
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames)
+{
+	struct ast_vector_string *return_strings;
+	int stackfr;
+	bfd *bfdobj;
+	long allocsize;
+	char msg[MSG_BUFF_LEN];
+	static pthread_mutex_t bfd_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+	return_strings = malloc(sizeof(struct ast_vector_string));
+	if (!return_strings) {
+		return NULL;
+	}
+	if (AST_VECTOR_INIT(return_strings, num_frames)) {
+		free(return_strings);
 		return NULL;
 	}
 
 	for (stackfr = 0; stackfr < num_frames; stackfr++) {
-		int found = 0, symbolcount;
+		int symbolcount;
+		struct bfd_data data = {
+			.return_strings = return_strings,
+			.msg = msg,
+			.pc = (bfd_vma)addresses[stackfr],
+			.found = 0,
+			.dynamic = 0,
+		};
 
 		msg[0] = '\0';
 
-		if (!dladdr(addresses[stackfr], &dli)) {
+		if (!dladdr((void *)data.pc, &data.dli)) {
 			continue;
 		}
+		data.libname = strrchr(data.dli.dli_fname, '/');
+		if (!data.libname) {
+			data.libname = data.dli.dli_fname;
+		} else {
+			data.libname++;
+		}
 
-		if (strcmp(dli.dli_fname, "asterisk") == 0) {
-			char asteriskpath[256];
+		pthread_mutex_lock(&bfd_mutex);
+		/* Using do while(0) here makes it easier to escape and clean up */
+		do {
+			bfdobj = bfd_openr(data.dli.dli_fname, NULL);
+			if (!bfdobj) {
+				break;
+			}
 
-			if (!(dli.dli_fname = ast_utils_which("asterisk", asteriskpath, sizeof(asteriskpath)))) {
-				/* This will fail to find symbols */
-				dli.dli_fname = "asterisk";
+			/* bfd_check_format does more than check.  It HAS to be called */
+			if (!bfd_check_format(bfdobj, bfd_object)) {
+				break;
 			}
-		}
 
-		lastslash = strrchr(dli.dli_fname, '/');
-		if ((bfdobj = bfd_openr(dli.dli_fname, NULL)) &&
-			bfd_check_format(bfdobj, bfd_object) &&
-			(allocsize = bfd_get_symtab_upper_bound(bfdobj)) > 0 &&
-			(syms = ast_std_malloc(allocsize)) &&
-			(symbolcount = bfd_canonicalize_symtab(bfdobj, syms))) {
-
-			if (bfdobj->flags & DYNAMIC) {
-				offset = addresses[stackfr] - dli.dli_fbase;
-			} else {
-				offset = addresses[stackfr] - (void *) 0;
-			}
-
-			for (section = bfdobj->sections; section; section = section->next) {
-				if (!(bfd_get_section_flags(bfdobj, section) & SEC_ALLOC) ||
-					section->vma > offset ||
-					section->size + section->vma < offset) {
-					continue;
-				}
-
-				if (!bfd_find_nearest_line(bfdobj, section, syms, offset - section->vma, &file, &func, &line)) {
-					continue;
-				}
-
-				/* file can possibly be null even with a success result from bfd_find_nearest_line */
-				file = file ? file : "";
-
-				/* Stack trace output */
-				found++;
-				if ((lastslash = strrchr(file, '/'))) {
-					const char *prevslash;
-
-					for (prevslash = lastslash - 1; *prevslash != '/' && prevslash >= file; prevslash--) {
-					}
-					if (prevslash >= file) {
-						lastslash = prevslash;
-					}
-				}
-				if (dli.dli_saddr == NULL) {
-					address_str[0] = '\0';
-				} else {
-					snprintf(address_str, sizeof(address_str), " (%p+%lX)",
-						dli.dli_saddr,
-						(unsigned long) (addresses[stackfr] - dli.dli_saddr));
-				}
-				snprintf(msg, sizeof(msg), "%s:%u %s()%s",
-					lastslash ? lastslash + 1 : file, line,
-					S_OR(func, "???"),
-					address_str);
+			data.has_syms = !!(bfd_get_file_flags(bfdobj) & HAS_SYMS);
+			data.dynamic = !!(bfd_get_file_flags(bfdobj) & DYNAMIC);
 
-				break; /* out of section iteration */
+			if (!data.has_syms) {
+				break;
 			}
-		}
+
+			allocsize = data.dynamic ?
+				bfd_get_dynamic_symtab_upper_bound(bfdobj) : bfd_get_symtab_upper_bound(bfdobj);
+			if (allocsize < 0) {
+				break;
+			}
+
+			data.syms = malloc(allocsize);
+			if (!data.syms) {
+				break;
+			}
+
+			symbolcount = data.dynamic ?
+				bfd_canonicalize_dynamic_symtab(bfdobj, data.syms) : bfd_canonicalize_symtab(bfdobj, data.syms);
+			if (symbolcount < 0) {
+				break;
+			}
+
+			bfd_map_over_sections(bfdobj, process_section, &data);
+		} while(0);
+
 		if (bfdobj) {
 			bfd_close(bfdobj);
-			ast_std_free(syms);
-			syms = NULL;
+			free(data.syms);
+			data.syms = NULL;
 		}
+		pthread_mutex_unlock(&bfd_mutex);
 
 		/* Default output, if we cannot find the information within BFD */
-		if (!found) {
-			if (dli.dli_saddr == NULL) {
-				address_str[0] = '\0';
-			} else {
-				snprintf(address_str, sizeof(address_str), " (%p+%lX)",
-					dli.dli_saddr,
-					(unsigned long) (addresses[stackfr] - dli.dli_saddr));
-			}
-			snprintf(msg, sizeof(msg), "%s %s()%s",
-				lastslash ? lastslash + 1 : dli.dli_fname,
-				S_OR(dli.dli_sname, "<unknown>"),
-				address_str);
-		}
-
-		if (!ast_strlen_zero(msg)) {
-			char **tmp;
-
-			eachlen[stackfr] = strlen(msg) + 1;
-			if (!(tmp = ast_std_realloc(strings, strings_size + eachlen[stackfr]))) {
-				ast_std_free(strings);
-				strings = NULL;
-				break; /* out of stack frame iteration */
-			}
-			strings = tmp;
-			strings[stackfr] = (char *) strings + strings_size;
-			strcpy(strings[stackfr], msg);/* Safe since we just allocated the room. */
-			strings_size += eachlen[stackfr];
+		if (!data.found) {
+			snprintf(msg, sizeof(msg), "%s %s()",
+				data.libname,
+				S_OR(data.dli.dli_sname, "<unknown>"));
+			AST_VECTOR_APPEND(return_strings, strdup(msg));
 		}
 	}
 
+	return return_strings;
+}
+
+#else
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames)
+{
+	char **strings;
+	struct ast_vector_string *return_strings;
+	int i;
+
+	return_strings = malloc(sizeof(struct ast_vector_string));
+	if (!return_strings) {
+		return NULL;
+	}
+	if (AST_VECTOR_INIT(return_strings, num_frames)) {
+		free(return_strings);
+		return NULL;
+	}
+
+	strings = backtrace_symbols(addresses, num_frames);
 	if (strings) {
-		/* Recalculate the offset pointers because of the reallocs. */
-		strings[0] = (char *) strings + num_frames * sizeof(*strings);
-		for (stackfr = 1; stackfr < num_frames; stackfr++) {
-			strings[stackfr] = strings[stackfr - 1] + eachlen[stackfr - 1];
+		for (i = 0; i < num_frames; i++) {
+			AST_VECTOR_APPEND(return_strings, strdup(strings[i]));
 		}
+		free(strings);
 	}
-	ast_std_free(eachlen);
 
-#else /* !defined(BETTER_BACKTRACES) */
+	return return_strings;
+}
+#endif /* BETTER_BACKTRACES */
 
-	strings = backtrace_symbols(addresses, num_frames);
-#endif /* defined(BETTER_BACKTRACES) */
-	return strings;
+void __ast_bt_free_symbols(struct ast_vector_string *symbols)
+{
+	AST_VECTOR_CALLBACK_VOID(symbols, free);
+	AST_VECTOR_PTR_FREE(symbols);
 }
 
 #endif /* HAVE_BKTR */
diff -pruN 1:13.23.1~dfsg-2/main/bridge_after.c 1:16.1.1~dfsg-1/main/bridge_after.c
--- 1:13.23.1~dfsg-2/main/bridge_after.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bridge_after.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/main/bridge_basic.c 1:16.1.1~dfsg-1/main/bridge_basic.c
--- 1:13.23.1~dfsg-2/main/bridge_basic.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bridge_basic.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/utils.h"
 #include "asterisk/linkedlists.h"
@@ -2434,12 +2432,11 @@ static void recall_callback(struct ast_d
  */
 static void common_recall_channel_setup(struct ast_channel *recall, struct ast_channel *transferer)
 {
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	callid = ast_read_threadstorage_callid();
 	if (callid) {
 		ast_channel_callid_set(recall, callid);
-		ast_callid_unref(callid);
 	}
 
 	ast_channel_inherit_variables(transferer, recall);
@@ -3047,7 +3044,7 @@ static enum attended_transfer_stimulus w
 static void *attended_transfer_monitor_thread(void *data)
 {
 	struct attended_transfer_properties *props = data;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	/*
 	 * Set thread callid to the transferer's callid because we
@@ -3092,7 +3089,6 @@ static void *attended_transfer_monitor_t
 	attended_transfer_properties_shutdown(props);
 
 	if (callid) {
-		ast_callid_unref(callid);
 		ast_callid_threadassoc_remove();
 	}
 
diff -pruN 1:13.23.1~dfsg-2/main/bridge.c 1:16.1.1~dfsg-1/main/bridge.c
--- 1:13.23.1~dfsg-2/main/bridge.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bridge.c	2018-12-26 18:45:13.000000000 +0000
@@ -83,8 +83,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
 #include "asterisk/options.h"
@@ -236,8 +234,21 @@ int __ast_bridge_technology_register(str
 	/* Copy module pointer so reference counting can keep the module from unloading */
 	technology->mod = module;
 
-	/* Insert our new bridge technology into the list and print out a pretty message */
-	AST_RWLIST_INSERT_TAIL(&bridge_technologies, technology, entry);
+	/* Find the correct position to insert the technology. */
+	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&bridge_technologies, current, entry) {
+		/* Put the highest preference tech's first in the list. */
+		if (technology->preference >= current->preference) {
+			AST_RWLIST_INSERT_BEFORE_CURRENT(technology, entry);
+
+			break;
+		}
+	}
+	AST_RWLIST_TRAVERSE_SAFE_END;
+
+	if (!current) {
+		/* Insert our new bridge technology to the end of the list. */
+		AST_RWLIST_INSERT_TAIL(&bridge_technologies, technology, entry);
+	}
 
 	AST_RWLIST_UNLOCK(&bridge_technologies);
 
@@ -442,6 +453,18 @@ static void bridge_channel_complete_join
 	}
 
 	bridge_channel->just_joined = 0;
+
+	/*
+	 * When a channel joins the bridge its streams need to be mapped to the bridge's
+	 * media types vector. This way all streams map to the same media type index for
+	 * a given channel.
+	 */
+	if (bridge_channel->bridge->technology->stream_topology_changed) {
+		bridge_channel->bridge->technology->stream_topology_changed(
+			bridge_channel->bridge, bridge_channel);
+	} else {
+		ast_bridge_channel_stream_map(bridge_channel);
+	}
 }
 
 /*!
@@ -507,12 +530,17 @@ static struct ast_bridge_technology *fin
 				current->name);
 			continue;
 		}
+		if (!ast_module_running_ref(current->mod)) {
+			ast_debug(1, "Bridge technology %s is not running, skipping.\n", current->name);
+			continue;
+		}
+		if (best) {
+			ast_module_unref(best->mod);
+		}
 		best = current;
 	}
 
 	if (best) {
-		/* Increment it's module reference count if present so it does not get unloaded while in use */
-		ast_module_ref(best->mod);
 		ast_debug(1, "Chose bridge technology %s\n", best->name);
 	}
 
@@ -692,9 +720,9 @@ static void destroy_bridge(void *obj)
 		bridge->technology = NULL;
 	}
 
-	if (bridge->callid) {
-		bridge->callid = ast_callid_unref(bridge->callid);
-	}
+	AST_VECTOR_FREE(&bridge->media_types);
+
+	bridge->callid = 0;
 
 	cleanup_video_mode(bridge);
 
@@ -749,6 +777,8 @@ struct ast_bridge *bridge_alloc(size_t s
 
 	bridge->v_table = v_table;
 
+	AST_VECTOR_INIT(&bridge->media_types, AST_MEDIA_TYPE_END);
+
 	return bridge;
 }
 
@@ -774,11 +804,13 @@ struct ast_bridge *bridge_base_init(stru
 	ast_set_flag(&self->feature_flags, flags);
 	self->allowed_capabilities = capabilities;
 
-	if (bridge_topics_init(self) != 0) {
-		ast_log(LOG_WARNING, "Bridge %s: Could not initialize topics\n",
-			self->uniqueid);
-		ao2_ref(self, -1);
-		return NULL;
+	if (!(flags & AST_BRIDGE_FLAG_INVISIBLE)) {
+		if (bridge_topics_init(self) != 0) {
+			ast_log(LOG_WARNING, "Bridge %s: Could not initialize topics\n",
+				self->uniqueid);
+			ao2_ref(self, -1);
+			return NULL;
+		}
 	}
 
 	/* Use our helper function to find the "best" bridge technology. */
@@ -808,9 +840,11 @@ struct ast_bridge *bridge_base_init(stru
 		return NULL;
 	}
 
-	if (!ast_bridge_topic(self)) {
-		ao2_ref(self, -1);
-		return NULL;
+	if (!(flags & AST_BRIDGE_FLAG_INVISIBLE)) {
+		if (!ast_bridge_topic(self)) {
+			ao2_ref(self, -1);
+			return NULL;
+		}
 	}
 
 	return self;
@@ -1725,7 +1759,7 @@ int ast_bridge_join(struct ast_bridge *b
 
 	ao2_ref(bridge_channel, -1);
 
-join_exit:;
+join_exit:
 	ast_bridge_run_after_callback(chan);
 	bridge_channel_impart_signal(chan);
 	if (!(ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_ASYNCGOTO)
@@ -3741,6 +3775,13 @@ void ast_bridge_set_mixing_interval(stru
 	ast_bridge_unlock(bridge);
 }
 
+void ast_bridge_set_binaural_active(struct ast_bridge *bridge, unsigned int binaural_active)
+{
+	ast_bridge_lock(bridge);
+	bridge->softmix.binaural_active = binaural_active;
+	ast_bridge_unlock(bridge);
+}
+
 void ast_bridge_set_internal_sample_rate(struct ast_bridge *bridge, unsigned int sample_rate)
 {
 	ast_bridge_lock(bridge);
@@ -3765,6 +3806,8 @@ static void cleanup_video_mode(struct as
 		if (bridge->softmix.video_mode.mode_data.talker_src_data.chan_old_vsrc) {
 			ast_channel_unref(bridge->softmix.video_mode.mode_data.talker_src_data.chan_old_vsrc);
 		}
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		break;
 	}
 	memset(&bridge->softmix.video_mode, 0, sizeof(bridge->softmix.video_mode));
 }
@@ -3792,6 +3835,39 @@ void ast_bridge_set_talker_src_video_mod
 	ast_bridge_unlock(bridge);
 }
 
+void ast_bridge_set_sfu_video_mode(struct ast_bridge *bridge)
+{
+	ast_bridge_lock(bridge);
+	cleanup_video_mode(bridge);
+	bridge->softmix.video_mode.mode = AST_BRIDGE_VIDEO_MODE_SFU;
+	ast_bridge_unlock(bridge);
+}
+
+void ast_bridge_set_video_update_discard(struct ast_bridge *bridge, unsigned int video_update_discard)
+{
+	ast_bridge_lock(bridge);
+	bridge->softmix.video_mode.video_update_discard = video_update_discard;
+	ast_bridge_unlock(bridge);
+}
+
+void ast_bridge_set_remb_send_interval(struct ast_bridge *bridge, unsigned int remb_send_interval)
+{
+	ast_assert(bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU);
+
+	ast_bridge_lock(bridge);
+	bridge->softmix.video_mode.mode_data.sfu_data.remb_send_interval = remb_send_interval;
+	ast_bridge_unlock(bridge);
+}
+
+void ast_brige_set_remb_behavior(struct ast_bridge *bridge, enum ast_bridge_video_sfu_remb_behavior behavior)
+{
+	ast_assert(bridge->softmix.video_mode.mode == AST_BRIDGE_VIDEO_MODE_SFU);
+
+	ast_bridge_lock(bridge);
+	bridge->softmix.video_mode.mode_data.sfu_data.remb_behavior = behavior;
+	ast_bridge_unlock(bridge);
+}
+
 void ast_bridge_update_talker_src_video_mode(struct ast_bridge *bridge, struct ast_channel *chan, int talker_energy, int is_keyframe)
 {
 	struct ast_bridge_video_talker_src_data *data;
@@ -3860,6 +3936,8 @@ int ast_bridge_number_video_src(struct a
 		if (bridge->softmix.video_mode.mode_data.talker_src_data.chan_old_vsrc) {
 			res++;
 		}
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		break;
 	}
 	ast_bridge_unlock(bridge);
 	return res;
@@ -3884,7 +3962,8 @@ int ast_bridge_is_video_src(struct ast_b
 		} else if (bridge->softmix.video_mode.mode_data.talker_src_data.chan_old_vsrc == chan) {
 			res = 2;
 		}
-
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		break;
 	}
 	ast_bridge_unlock(bridge);
 	return res;
@@ -3918,6 +3997,8 @@ void ast_bridge_remove_video_src(struct
 			}
 			bridge->softmix.video_mode.mode_data.talker_src_data.chan_old_vsrc = NULL;
 		}
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		break;
 	}
 	ast_bridge_unlock(bridge);
 }
@@ -3929,12 +4010,21 @@ const char *ast_bridge_video_mode_to_str
 		return "talker";
 	case AST_BRIDGE_VIDEO_MODE_SINGLE_SRC:
 		return "single";
+	case AST_BRIDGE_VIDEO_MODE_SFU:
+		return "sfu";
 	case AST_BRIDGE_VIDEO_MODE_NONE:
 	default:
 		return "none";
 	}
 }
 
+void ast_bridge_set_send_sdp_label(struct ast_bridge *bridge, unsigned int send_sdp_label)
+{
+	ast_bridge_lock(bridge);
+	bridge->softmix.send_sdp_label = send_sdp_label;
+	ast_bridge_unlock(bridge);
+}
+
 static int channel_hash(const void *obj, int flags)
 {
 	const struct ast_channel *chan = obj;
@@ -3985,8 +4075,8 @@ struct ao2_container *ast_bridge_peers_n
 	struct ao2_container *channels;
 	struct ast_bridge_channel *iter;
 
-	channels = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK,
-		13, channel_hash, channel_cmp);
+	channels = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
+		13, channel_hash, NULL, channel_cmp);
 	if (!channels) {
 		return NULL;
 	}
@@ -4249,11 +4339,11 @@ static enum ast_transfer_result attended
 		struct ast_channel *locals[2];
 
 		/* Have to lock everything just in case a hangup comes in early */
-		ast_local_lock_all2(local_chan, &tech, &locals[0], &locals[1]);
+		ast_local_lock_all(local_chan, &tech, &locals[0], &locals[1]);
 		if (!locals[0] || !locals[1]) {
 			ast_log(LOG_ERROR, "Transfer failed probably due to an early hangup - "
 				"missing other half of '%s'\n", ast_channel_name(local_chan));
-			ast_local_unlock_all2(tech, locals[0], locals[1]);
+			ast_local_unlock_all(tech, locals[0], locals[1]);
 			ao2_cleanup(local_chan);
 			return AST_BRIDGE_TRANSFER_FAIL;
 		}
@@ -4264,7 +4354,7 @@ static enum ast_transfer_result attended
 		}
 
 		ast_attended_transfer_message_add_link(transfer_msg, locals);
-		ast_local_unlock_all2(tech, locals[0], locals[1]);
+		ast_local_unlock_all(tech, locals[0], locals[1]);
 	} else {
 		ast_attended_transfer_message_add_app(transfer_msg, app, local_chan);
 	}
@@ -4363,8 +4453,8 @@ struct ast_bridge *ast_bridge_transfer_a
 	bridge = ast_channel_get_bridge(chan);
 	ast_channel_unlock(chan);
 
-	if (bridge
-		&& ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_MASQUERADE_ONLY)) {
+	if (bridge && ast_test_flag(&bridge->feature_flags,
+			(AST_BRIDGE_FLAG_MASQUERADE_ONLY | AST_BRIDGE_FLAG_INVISIBLE))) {
 		ao2_ref(bridge, -1);
 		bridge = NULL;
 	}
diff -pruN 1:13.23.1~dfsg-2/main/bridge_channel.c 1:16.1.1~dfsg-1/main/bridge_channel.c
--- 1:13.23.1~dfsg-2/main/bridge_channel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bridge_channel.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/heap.h"
@@ -57,6 +55,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/causes.h"
 #include "asterisk/test.h"
 #include "asterisk/sem.h"
+#include "asterisk/stream.h"
 #include "asterisk/message.h"
 
 /*!
@@ -628,11 +627,11 @@ void ast_bridge_channel_kick(struct ast_
 
 /*!
  * \internal
- * \brief Write an \ref ast_frame onto the bridge channel
+ * \brief Write an \ref ast_frame into the bridge
  * \since 12.0.0
  *
- * \param bridge_channel Which channel to queue the frame onto.
- * \param frame The frame to write onto the bridge_channel
+ * \param bridge_channel Which channel is queueing the frame.
+ * \param frame The frame to write into the bridge
  *
  * \retval 0 on success.
  * \retval -1 on error.
@@ -640,19 +639,74 @@ void ast_bridge_channel_kick(struct ast_
 static int bridge_channel_write_frame(struct ast_bridge_channel *bridge_channel, struct ast_frame *frame)
 {
 	const struct ast_control_t38_parameters *t38_parameters;
+	int unmapped_stream_num;
 	int deferred;
 
 	ast_assert(frame->frametype != AST_FRAME_BRIDGE_ACTION_SYNC);
 
 	ast_bridge_channel_lock_bridge(bridge_channel);
 
+	/* Map the frame to the bridge. */
+	if (ast_channel_is_multistream(bridge_channel->chan)) {
+		unmapped_stream_num = frame->stream_num;
+		switch (frame->frametype) {
+		case AST_FRAME_VOICE:
+		case AST_FRAME_VIDEO:
+		case AST_FRAME_TEXT:
+		case AST_FRAME_IMAGE:
+		case AST_FRAME_RTCP:
+			/* These frames need to be mapped to an appropriate write stream */
+			if (frame->stream_num < 0) {
+				/* Map to default stream */
+				frame->stream_num = -1;
+				break;
+			}
+			ast_bridge_channel_lock(bridge_channel);
+			if (frame->stream_num < (int)AST_VECTOR_SIZE(&bridge_channel->stream_map.to_bridge)) {
+				frame->stream_num = AST_VECTOR_GET(
+					&bridge_channel->stream_map.to_bridge, frame->stream_num);
+				if (0 <= frame->stream_num) {
+					ast_bridge_channel_unlock(bridge_channel);
+					break;
+				}
+			}
+			ast_bridge_channel_unlock(bridge_channel);
+			ast_bridge_unlock(bridge_channel->bridge);
+			/*
+			 * Ignore frame because we don't know how to map the frame
+			 * or the bridge is not expecting any media from that
+			 * stream.
+			 */
+			return 0;
+		case AST_FRAME_DTMF_BEGIN:
+		case AST_FRAME_DTMF_END:
+			/*
+			 * XXX It makes sense that DTMF could be on any audio stream.
+			 * For now we will only put it on the default audio stream.
+			 */
+		default:
+			frame->stream_num = -1;
+			break;
+		}
+	} else {
+		unmapped_stream_num = -1;
+		frame->stream_num = -1;
+	}
+
 	deferred = bridge_channel->bridge->technology->write(bridge_channel->bridge, bridge_channel, frame);
 	if (deferred) {
 		struct ast_frame *dup;
 
 		dup = ast_frdup(frame);
 		if (dup) {
+			/*
+			 * We have to unmap the deferred frame so it comes back
+			 * in like a new frame.
+			 */
+			dup->stream_num = unmapped_stream_num;
+			ast_bridge_channel_lock(bridge_channel);
 			AST_LIST_INSERT_HEAD(&bridge_channel->deferred_queue, dup, frame_list);
+			ast_bridge_channel_unlock(bridge_channel);
 		}
 	}
 
@@ -676,12 +730,12 @@ static int bridge_channel_write_frame(st
 			switch (t38_parameters->request_response) {
 			case AST_T38_REQUEST_NEGOTIATE:
 			case AST_T38_NEGOTIATED:
-				bridge_channel->owed_t38_terminate = 1;
+				bridge_channel->owed.t38_terminate = 1;
 				break;
 			case AST_T38_REQUEST_TERMINATE:
 			case AST_T38_TERMINATED:
 			case AST_T38_REFUSED:
-				bridge_channel->owed_t38_terminate = 0;
+				bridge_channel->owed.t38_terminate = 0;
 				break;
 			default:
 				break;
@@ -717,7 +771,7 @@ static int bridge_channel_write_frame(st
 static void bridge_channel_cancel_owed_events(struct ast_bridge_channel *bridge_channel)
 {
 	bridge_channel->owed.dtmf_digit = '\0';
-	bridge_channel->owed_t38_terminate = 0;
+	bridge_channel->owed.t38_terminate = 0;
 }
 
 void bridge_channel_settle_owed_events(struct ast_bridge *orig_bridge, struct ast_bridge_channel *bridge_channel)
@@ -739,7 +793,7 @@ void bridge_channel_settle_owed_events(s
 		bridge_channel->owed.dtmf_digit = '\0';
 		orig_bridge->technology->write(orig_bridge, NULL, &frame);
 	}
-	if (bridge_channel->owed_t38_terminate) {
+	if (bridge_channel->owed.t38_terminate) {
 		struct ast_control_t38_parameters t38_parameters = {
 			.request_response = AST_T38_TERMINATED,
 		};
@@ -753,7 +807,7 @@ void bridge_channel_settle_owed_events(s
 
 		ast_debug(1, "T.38 terminate simulated to bridge %s because %s left.\n",
 			orig_bridge->uniqueid, ast_channel_name(bridge_channel->chan));
-		bridge_channel->owed_t38_terminate = 0;
+		bridge_channel->owed.t38_terminate = 0;
 		orig_bridge->technology->write(orig_bridge, NULL, &frame);
 	}
 }
@@ -762,12 +816,14 @@ void bridge_channel_queue_deferred_frame
 {
 	struct ast_frame *frame;
 
+	ast_bridge_channel_lock(bridge_channel);
 	ast_channel_lock(bridge_channel->chan);
 	while ((frame = AST_LIST_REMOVE_HEAD(&bridge_channel->deferred_queue, frame_list))) {
 		ast_queue_frame_head(bridge_channel->chan, frame);
 		ast_frfree(frame);
 	}
 	ast_channel_unlock(bridge_channel->chan);
+	ast_bridge_channel_unlock(bridge_channel);
 }
 
 /*!
@@ -2285,6 +2341,9 @@ static void bridge_channel_handle_contro
 		/* Should never happen. */
 		ast_assert(0);
 		break;
+	case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+		ast_indicate_data(chan, fr->subclass.integer, fr->data.ptr, fr->datalen);
+		break;
 	default:
 		ast_indicate_data(chan, fr->subclass.integer, fr->data.ptr, fr->datalen);
 		break;
@@ -2304,6 +2363,7 @@ static void bridge_channel_handle_write(
 {
 	struct ast_frame *fr;
 	struct sync_payload *sync_payload;
+	int num;
 	struct ast_msg_data *msg;
 
 	ast_bridge_channel_lock(bridge_channel);
@@ -2376,9 +2436,27 @@ static void bridge_channel_handle_write(
 		ast_sendtext_data(bridge_channel->chan, msg);
 		break;
 	default:
+		/* Assume that there is no mapped stream for this */
+		num = -1;
+
+		if (fr->stream_num > -1) {
+			ast_bridge_channel_lock(bridge_channel);
+			if (fr->stream_num < (int)AST_VECTOR_SIZE(&bridge_channel->stream_map.to_channel)) {
+				num = AST_VECTOR_GET(&bridge_channel->stream_map.to_channel, fr->stream_num);
+			}
+			ast_bridge_channel_unlock(bridge_channel);
+
+			/* If there is no mapped stream after checking the mapping then there is nowhere
+			 * to write this frame to, so drop it.
+			 */
+			if (num == -1) {
+				break;
+			}
+		}
+
 		/* Write the frame to the channel. */
 		bridge_channel->activity = BRIDGE_CHANNEL_THREAD_SIMPLE;
-		ast_write(bridge_channel->chan, fr);
+		ast_write_stream(bridge_channel->chan, num, fr);
 		break;
 	}
 	bridge_frame_free(fr);
@@ -2430,6 +2508,14 @@ static struct ast_frame *bridge_handle_d
 	return frame;
 }
 
+static const char *controls[] = {
+	[AST_CONTROL_RINGING] = "RINGING",
+	[AST_CONTROL_PROCEEDING] = "PROCEEDING",
+	[AST_CONTROL_PROGRESS] = "PROGRESS",
+	[AST_CONTROL_BUSY] = "BUSY",
+	[AST_CONTROL_CONGESTION] = "CONGESTION",
+	[AST_CONTROL_ANSWER] = "ANSWER",
+};
 
 /*!
  * \internal
@@ -2440,24 +2526,81 @@ static struct ast_frame *bridge_handle_d
 static void bridge_handle_trip(struct ast_bridge_channel *bridge_channel)
 {
 	struct ast_frame *frame;
+	int blocked;
+
+	if (!ast_strlen_zero(ast_channel_call_forward(bridge_channel->chan))) {
+		/* TODO If early bridging is ever used by anything other than ARI,
+		 * it's important that we actually attempt to handle the call forward
+		 * attempt, as well as expand features on a bridge channel to allow/disallow
+		 * call forwarding. For now, all we do is raise an event, showing that
+		 * a call forward is being attempted.
+		 */
+		ast_channel_publish_dial_forward(NULL, bridge_channel->chan, NULL, NULL, "CANCEL",
+			ast_channel_call_forward(bridge_channel->chan));
+	}
 
 	if (bridge_channel->features->mute) {
-		frame = ast_read_noaudio(bridge_channel->chan);
+		frame = ast_read_stream_noaudio(bridge_channel->chan);
 	} else {
-		frame = ast_read(bridge_channel->chan);
+		frame = ast_read_stream(bridge_channel->chan);
 	}
 
 	if (!frame) {
 		ast_bridge_channel_kick(bridge_channel, 0);
 		return;
 	}
+
+	if (!ast_channel_is_multistream(bridge_channel->chan)) {
+		/* This may not be initialized by non-multistream channel drivers */
+		frame->stream_num = -1;
+	}
+
 	switch (frame->frametype) {
 	case AST_FRAME_CONTROL:
 		switch (frame->subclass.integer) {
+		case AST_CONTROL_CONGESTION:
+		case AST_CONTROL_BUSY:
+			ast_channel_publish_dial(NULL, bridge_channel->chan, NULL, controls[frame->subclass.integer]);
+			break;
 		case AST_CONTROL_HANGUP:
 			ast_bridge_channel_kick(bridge_channel, 0);
 			bridge_frame_free(frame);
 			return;
+		case AST_CONTROL_RINGING:
+		case AST_CONTROL_PROGRESS:
+		case AST_CONTROL_PROCEEDING:
+		case AST_CONTROL_ANSWER:
+			ast_channel_publish_dial(NULL, bridge_channel->chan, NULL, controls[frame->subclass.integer]);
+			break;
+		case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+			ast_bridge_channel_lock_bridge(bridge_channel);
+			blocked = bridge_channel->bridge->technology->stream_topology_request_change
+				&& bridge_channel->bridge->technology->stream_topology_request_change(
+					bridge_channel->bridge, bridge_channel);
+			ast_bridge_unlock(bridge_channel->bridge);
+			if (blocked) {
+				/*
+				 * Topology change was intercepted by the bridge technology
+				 * so drop frame.
+				 */
+				bridge_frame_free(frame);
+				return;
+			}
+			break;
+		case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+			/*
+			 * If a stream topology has changed then the bridge_channel's
+			 * media mapping needs to be updated.
+			 */
+			ast_bridge_channel_lock_bridge(bridge_channel);
+			if (bridge_channel->bridge->technology->stream_topology_changed) {
+				bridge_channel->bridge->technology->stream_topology_changed(
+					bridge_channel->bridge, bridge_channel);
+			} else {
+				ast_bridge_channel_stream_map(bridge_channel);
+			}
+			ast_bridge_unlock(bridge_channel->bridge);
+			break;
 		default:
 			break;
 		}
@@ -2895,7 +3038,7 @@ static void bridge_channel_destroy(void
 	struct ast_frame *fr;
 
 	if (bridge_channel->callid) {
-		bridge_channel->callid = ast_callid_unref(bridge_channel->callid);
+		bridge_channel->callid = 0;
 	}
 
 	if (bridge_channel->bridge) {
@@ -2918,6 +3061,9 @@ static void bridge_channel_destroy(void
 
 	ao2_cleanup(bridge_channel->write_format);
 	ao2_cleanup(bridge_channel->read_format);
+
+	AST_VECTOR_FREE(&bridge_channel->stream_map.to_bridge);
+	AST_VECTOR_FREE(&bridge_channel->stream_map.to_channel);
 }
 
 struct ast_bridge_channel *bridge_channel_internal_alloc(struct ast_bridge *bridge)
@@ -2938,5 +3084,18 @@ struct ast_bridge_channel *bridge_channe
 		ao2_ref(bridge_channel->bridge, +1);
 	}
 
+	/* The stream_map is initialized later - see ast_bridge_channel_stream_map */
+
 	return bridge_channel;
 }
+
+void ast_bridge_channel_stream_map(struct ast_bridge_channel *bridge_channel)
+{
+	ast_bridge_channel_lock(bridge_channel);
+	ast_channel_lock(bridge_channel->chan);
+	ast_stream_topology_map(ast_channel_get_stream_topology(bridge_channel->chan),
+		&bridge_channel->bridge->media_types, &bridge_channel->stream_map.to_bridge,
+		&bridge_channel->stream_map.to_channel);
+	ast_channel_unlock(bridge_channel->chan);
+	ast_bridge_channel_unlock(bridge_channel);
+}
diff -pruN 1:13.23.1~dfsg-2/main/bridge_roles.c 1:16.1.1~dfsg-1/main/bridge_roles.c
--- 1:13.23.1~dfsg-2/main/bridge_roles.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bridge_roles.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/logger.h"
diff -pruN 1:13.23.1~dfsg-2/main/bucket.c 1:16.1.1~dfsg-1/main/bucket.c
--- 1:13.23.1~dfsg-2/main/bucket.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/bucket.c	2018-12-26 18:45:13.000000000 +0000
@@ -60,8 +60,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #ifdef HAVE_URIPARSER
 #include <uriparser/Uri.h>
 #endif
@@ -163,12 +161,25 @@ static int bucket_wizard_delete(const st
 	return bucket->scheme_impl->bucket->delete(sorcery, data, object);
 }
 
+/*! \brief Callback function for determining if a bucket is stale */
+static int bucket_wizard_is_stale(const struct ast_sorcery *sorcery, void *data, void *object)
+{
+	struct ast_bucket *bucket = object;
+
+	if (!bucket->scheme_impl->bucket->is_stale) {
+		return 0;
+	}
+
+	return bucket->scheme_impl->bucket->is_stale(sorcery, data, object);
+}
+
 /*! \brief Intermediary bucket wizard */
 static struct ast_sorcery_wizard bucket_wizard = {
 	.name = "bucket",
 	.create = bucket_wizard_create,
 	.retrieve_id = bucket_wizard_retrieve,
 	.delete = bucket_wizard_delete,
+	.is_stale = bucket_wizard_is_stale,
 };
 
 /*! \brief Callback function for creating a bucket file */
@@ -240,6 +251,18 @@ static int bucket_file_wizard_delete(con
 	return file->scheme_impl->file->delete(sorcery, data, object);
 }
 
+/*! \brief Callback function for determining if a bucket is stale */
+static int bucket_file_wizard_is_stale(const struct ast_sorcery *sorcery, void *data, void *object)
+{
+	struct ast_bucket_file *file = object;
+
+	if (!file->scheme_impl->file->is_stale) {
+		return 0;
+	}
+
+	return file->scheme_impl->file->is_stale(sorcery, data, object);
+}
+
 /*! \brief Intermediary file wizard */
 static struct ast_sorcery_wizard bucket_file_wizard = {
 	.name = "bucket_file",
@@ -247,6 +270,7 @@ static struct ast_sorcery_wizard bucket_
 	.retrieve_id = bucket_file_wizard_retrieve,
 	.update = bucket_file_wizard_update,
 	.delete = bucket_file_wizard_delete,
+	.is_stale = bucket_file_wizard_is_stale,
 };
 
 int __ast_bucket_scheme_register(const char *name, struct ast_sorcery_wizard *bucket,
@@ -258,7 +282,7 @@ int __ast_bucket_scheme_register(const c
 
 	if (ast_strlen_zero(name) || !bucket || !file ||
 	    !bucket->create || !bucket->delete || !bucket->retrieve_id ||
-	    !create_cb) {
+	    (!bucket->create && !create_cb)) {
 		return -1;
 	}
 
@@ -336,6 +360,12 @@ struct ast_bucket_metadata *ast_bucket_f
 	return ao2_find(file->metadata, name, OBJ_KEY);
 }
 
+void ast_bucket_file_metadata_callback(struct ast_bucket_file *file, ao2_callback_fn cb, void *arg)
+{
+	ao2_callback(file->metadata, 0, cb, arg);
+}
+
+
 /*! \brief Destructor for buckets */
 static void bucket_destroy(void *obj)
 {
@@ -459,6 +489,28 @@ int ast_bucket_create(struct ast_bucket
 	return ast_sorcery_create(bucket_sorcery, bucket);
 }
 
+/*!
+ * \internal
+ * \brief Sorcery object type copy handler for \c ast_bucket
+ */
+static int bucket_copy_handler(const void *src, void *dst)
+{
+	const struct ast_bucket *src_bucket = src;
+	struct ast_bucket *dst_bucket = dst;
+
+	dst_bucket->scheme_impl = ao2_bump(src_bucket->scheme_impl);
+	ast_string_field_set(dst_bucket, scheme, src_bucket->scheme);
+	dst_bucket->created = src_bucket->created;
+	dst_bucket->modified = src_bucket->modified;
+
+	return 0;
+}
+
+struct ast_bucket *ast_bucket_clone(struct ast_bucket *bucket)
+{
+	return ast_sorcery_copy(bucket_sorcery, bucket);
+}
+
 struct ast_bucket *ast_bucket_retrieve(const char *uri)
 {
 	if (ast_strlen_zero(uri)) {
@@ -468,6 +520,11 @@ struct ast_bucket *ast_bucket_retrieve(c
 	return ast_sorcery_retrieve_by_id(bucket_sorcery, "bucket", uri);
 }
 
+int ast_bucket_is_stale(struct ast_bucket *bucket)
+{
+	return ast_sorcery_is_stale(bucket_sorcery, bucket);
+}
+
 int ast_bucket_observer_add(const struct ast_sorcery_observer *callbacks)
 {
 	return ast_sorcery_observer_add(bucket_sorcery, "bucket", callbacks);
@@ -592,8 +649,8 @@ static void *bucket_file_alloc(const cha
 		return NULL;
 	}
 
-	file->metadata = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, METADATA_BUCKETS,
-		ast_bucket_metadata_hash_fn, ast_bucket_metadata_cmp_fn);
+	file->metadata = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, METADATA_BUCKETS,
+		ast_bucket_metadata_hash_fn, NULL, ast_bucket_metadata_cmp_fn);
 	if (!file->metadata) {
 		return NULL;
 	}
@@ -656,7 +713,7 @@ struct ast_bucket_file *ast_bucket_file_
 
 	ast_string_field_set(file, scheme, uri_scheme);
 
-	if (scheme->create(file)) {
+	if (scheme->create && scheme->create(file)) {
 		ao2_ref(file, -1);
 		return NULL;
 	}
@@ -707,6 +764,29 @@ static int bucket_copy(const char *infil
 	return 0;	/* success */
 }
 
+/*!
+ * \internal
+ * \brief Sorcery object type copy handler for \c ast_bucket_file
+ */
+static int bucket_file_copy_handler(const void *src, void *dst)
+{
+	const struct ast_bucket_file *src_file = src;
+	struct ast_bucket_file *dst_file = dst;
+
+	dst_file->scheme_impl = ao2_bump(src_file->scheme_impl);
+	ast_string_field_set(dst_file, scheme, src_file->scheme);
+	dst_file->created = src_file->created;
+	dst_file->modified = src_file->modified;
+	strcpy(dst_file->path, src_file->path); /* safe */
+
+	dst_file->metadata = ao2_container_clone(src_file->metadata, 0);
+	if (!dst_file->metadata) {
+		return -1;
+	}
+
+	return 0;
+}
+
 struct ast_bucket_file *ast_bucket_file_copy(struct ast_bucket_file *file, const char *uri)
 {
 	RAII_VAR(struct ast_bucket_file *, copy, ast_bucket_file_alloc(uri), ao2_cleanup);
@@ -726,6 +806,11 @@ struct ast_bucket_file *ast_bucket_file_
 	return copy;
 }
 
+struct ast_bucket_file *ast_bucket_file_clone(struct ast_bucket_file *file)
+{
+	return ast_sorcery_copy(bucket_sorcery, file);
+}
+
 struct ast_bucket_file *ast_bucket_file_retrieve(const char *uri)
 {
 	if (ast_strlen_zero(uri)) {
@@ -735,6 +820,11 @@ struct ast_bucket_file *ast_bucket_file_
 	return ast_sorcery_retrieve_by_id(bucket_sorcery, "file", uri);
 }
 
+int ast_bucket_file_is_stale(struct ast_bucket_file *file)
+{
+	return ast_sorcery_is_stale(bucket_sorcery, file);
+}
+
 int ast_bucket_file_observer_add(const struct ast_sorcery_observer *callbacks)
 {
 	return ast_sorcery_observer_add(bucket_sorcery, "file", callbacks);
@@ -864,8 +954,8 @@ int ast_bucket_init(void)
 {
 	ast_register_cleanup(&bucket_cleanup);
 
-	schemes = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, SCHEME_BUCKETS,
-		ast_bucket_scheme_hash_fn, ast_bucket_scheme_cmp_fn);
+	schemes = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, SCHEME_BUCKETS,
+		ast_bucket_scheme_hash_fn, NULL, ast_bucket_scheme_cmp_fn);
 	if (!schemes) {
 		ast_log(LOG_ERROR, "Failed to create container for Bucket schemes\n");
 		return -1;
@@ -899,6 +989,7 @@ int ast_bucket_init(void)
 	ast_sorcery_object_field_register(bucket_sorcery, "bucket", "scheme", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_bucket, scheme));
 	ast_sorcery_object_field_register_custom(bucket_sorcery, "bucket", "created", "", timeval_str2struct, timeval_struct2str, NULL, 0, FLDSET(struct ast_bucket, created));
 	ast_sorcery_object_field_register_custom(bucket_sorcery, "bucket", "modified", "", timeval_str2struct, timeval_struct2str, NULL, 0, FLDSET(struct ast_bucket, modified));
+	ast_sorcery_object_set_copy_handler(bucket_sorcery, "bucket", bucket_copy_handler);
 
 	if (ast_sorcery_apply_default(bucket_sorcery, "file", "bucket_file", NULL) == AST_SORCERY_APPLY_FAIL) {
 		ast_log(LOG_ERROR, "Failed to apply intermediary for 'file' object type in Bucket sorcery\n");
@@ -913,6 +1004,7 @@ int ast_bucket_init(void)
 	ast_sorcery_object_field_register(bucket_sorcery, "file", "scheme", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_bucket_file, scheme));
 	ast_sorcery_object_field_register_custom(bucket_sorcery, "file", "created", "", timeval_str2struct, timeval_struct2str, NULL, 0, FLDSET(struct ast_bucket_file, created));
 	ast_sorcery_object_field_register_custom(bucket_sorcery, "file", "modified", "", timeval_str2struct, timeval_struct2str, NULL, 0, FLDSET(struct ast_bucket_file, modified));
+	ast_sorcery_object_set_copy_handler(bucket_sorcery, "file", bucket_file_copy_handler);
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/callerid.c 1:16.1.1~dfsg-1/main/callerid.c
--- 1:13.23.1~dfsg-2/main/callerid.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/callerid.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 #include <math.h>
 #include <ctype.h>
diff -pruN 1:13.23.1~dfsg-2/main/ccss.c 1:16.1.1~dfsg-1/main/ccss.c
--- 1:13.23.1~dfsg-2/main/ccss.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ccss.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/strings.h"
 #include "asterisk/ccss.h"
@@ -682,11 +680,7 @@ void ast_cc_default_config_params(struct
 
 struct ast_cc_config_params *__ast_cc_config_params_init(const char *file, int line, const char *function)
 {
-#if defined(__AST_DEBUG_MALLOC)
 	struct ast_cc_config_params *params = __ast_malloc(sizeof(*params), file, line, function);
-#else
-	struct ast_cc_config_params *params = ast_malloc(sizeof(*params));
-#endif
 
 	if (!params) {
 		return NULL;
@@ -1439,6 +1433,8 @@ static struct generic_monitor_instance_l
 		cc_unref(generic_list, "Failed to subscribe to device state");
 		return NULL;
 	}
+	stasis_subscription_accept_message_type(generic_list->sub, ast_device_state_message_type());
+	stasis_subscription_set_filter(generic_list->sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	generic_list->current_state = ast_device_state(monitor->interface->device_name);
 	ao2_t_link(generic_monitors, generic_list, "linking new generic monitor instance list");
 	return generic_list;
@@ -2810,6 +2806,9 @@ static int cc_generic_agent_start_monito
 	if (!(generic_pvt->sub = stasis_subscribe(device_specific_topic, generic_agent_devstate_cb, agent))) {
 		return -1;
 	}
+	stasis_subscription_accept_message_type(generic_pvt->sub, ast_device_state_message_type());
+	stasis_subscription_accept_message_type(generic_pvt->sub, stasis_subscription_change_type());
+	stasis_subscription_set_filter(generic_pvt->sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	cc_ref(agent, "Ref agent for subscription");
 	return 0;
 }
@@ -4621,7 +4620,7 @@ static struct ast_cli_entry cc_cli[] = {
 	AST_CLI_DEFINE(handle_cc_kill, "Kill a CC transaction"),
 };
 
-static void cc_shutdown(void)
+static int unload_module(void)
 {
 	ast_devstate_prov_del("ccss");
 	ast_cc_agent_unregister(&generic_agent_callbacks);
@@ -4647,30 +4646,37 @@ static void cc_shutdown(void)
 		ao2_t_ref(generic_monitors, -1, "Unref generic_monitor container in cc_shutdown");
 		generic_monitors = NULL;
 	}
+
+	return 0;
 }
 
-int ast_cc_init(void)
+static int load_module(void)
 {
 	int res;
 
-	if (!(cc_core_instances = ao2_t_container_alloc(CC_CORE_INSTANCES_BUCKETS,
-					cc_core_instance_hash_fn, cc_core_instance_cmp_fn,
-					"Create core instance container"))) {
-		return -1;
+	cc_core_instances = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		CC_CORE_INSTANCES_BUCKETS,
+		cc_core_instance_hash_fn, NULL, cc_core_instance_cmp_fn,
+		"Create core instance container");
+	if (!cc_core_instances) {
+		return AST_MODULE_LOAD_FAILURE;
 	}
-	if (!(generic_monitors = ao2_t_container_alloc(CC_CORE_INSTANCES_BUCKETS,
-			generic_monitor_instance_list_hash_fn, generic_monitor_instance_list_cmp_fn,
-			"Create generic monitor container"))) {
-		return -1;
+
+	generic_monitors = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		CC_CORE_INSTANCES_BUCKETS,
+		generic_monitor_instance_list_hash_fn, NULL, generic_monitor_instance_list_cmp_fn,
+		"Create generic monitor container");
+	if (!generic_monitors) {
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	if (!(cc_core_taskprocessor = ast_taskprocessor_get("CCSS_core", TPS_REF_DEFAULT))) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	if (!(cc_sched_context = ast_sched_context_create())) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	if (ast_sched_start_thread(cc_sched_context)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	res = ast_register_application2(ccreq_app, ccreq_exec, NULL, NULL, NULL);
 	res |= ast_register_application2(cccancel_app, cccancel_exec, NULL, NULL, NULL);
@@ -4686,7 +4692,12 @@ int ast_cc_init(void)
 	initialize_cc_devstate_map();
 	res |= ast_devstate_prov_add("ccss", ccss_device_state);
 
-	ast_register_cleanup(cc_shutdown);
-
-	return res;
+	return res ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Call Completion Supplementary Services",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CORE,
+);
diff -pruN 1:13.23.1~dfsg-2/main/cdr.c 1:16.1.1~dfsg-1/main/cdr.c
--- 1:13.23.1~dfsg-2/main/cdr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/cdr.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 #include <inttypes.h>
 
@@ -55,6 +53,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/cdr.h"
 #include "asterisk/callerid.h"
 #include "asterisk/manager.h"
+#include "asterisk/module.h"
 #include "asterisk/causes.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/utils.h"
@@ -62,7 +61,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/config.h"
 #include "asterisk/cli.h"
 #include "asterisk/stringfields.h"
-#include "asterisk/data.h"
 #include "asterisk/config_options.h"
 #include "asterisk/json.h"
 #include "asterisk/parking.h"
@@ -339,6 +337,9 @@ struct cdr_beitem {
 /*! \brief List of registered backends */
 static AST_RWLIST_HEAD_STATIC(be_list, cdr_beitem);
 
+/*! \brief List of registered modifiers */
+static AST_RWLIST_HEAD_STATIC(mo_list, cdr_beitem);
+
 /*! \brief Queued CDR waiting to be batched */
 struct cdr_batch_item {
 	struct ast_cdr *cdr;
@@ -1592,15 +1593,11 @@ static int base_process_party_a(struct c
 
 static int base_process_bridge_leave(struct cdr_object *cdr, struct ast_bridge_snapshot *bridge, struct ast_channel_snapshot *channel)
 {
-	/* In general, most things shouldn't get a bridge leave */
-	ast_assert(0);
-	return 1;
+	return 0;
 }
 
 static int base_process_dial_end(struct cdr_object *cdr, struct ast_channel_snapshot *caller, struct ast_channel_snapshot *peer, const char *dial_status)
 {
-	/* In general, most things shouldn't get a dial end. */
-	ast_assert(0);
 	return 0;
 }
 
@@ -2077,6 +2074,13 @@ static int filter_channel_cache_message(
 	return ret;
 }
 
+static int dial_status_end(const char *dialstatus)
+{
+	return (strcmp(dialstatus, "RINGING") &&
+			strcmp(dialstatus, "PROCEEDING") &&
+			strcmp(dialstatus, "PROGRESS"));
+}
+
 /* TOPIC ROUTER CALLBACKS */
 
 /*!
@@ -2148,7 +2152,7 @@ static void handle_dial_message(void *da
 			res &= it_cdr->fn_table->process_dial_begin(it_cdr,
 					caller,
 					peer);
-		} else {
+		} else if (dial_status_end(dial_status)) {
 			if (!it_cdr->fn_table->process_dial_end) {
 				continue;
 			}
@@ -2879,7 +2883,7 @@ int ast_cdr_backend_unsuspend(const char
 	return success;
 }
 
-int ast_cdr_register(const char *name, const char *desc, ast_cdrbe be)
+static int cdr_generic_register(struct be_list *generic_list, const char *name, const char *desc, ast_cdrbe be)
 {
 	struct cdr_beitem *i;
 	struct cdr_beitem *cur;
@@ -2903,51 +2907,61 @@ int ast_cdr_register(const char *name, c
 	ast_copy_string(i->name, name, sizeof(i->name));
 	ast_copy_string(i->desc, desc, sizeof(i->desc));
 
-	AST_RWLIST_WRLOCK(&be_list);
-	AST_RWLIST_TRAVERSE(&be_list, cur, list) {
+	AST_RWLIST_WRLOCK(generic_list);
+	AST_RWLIST_TRAVERSE(generic_list, cur, list) {
 		if (!strcasecmp(name, cur->name)) {
 			ast_log(LOG_WARNING, "Already have a CDR backend called '%s'\n", name);
-			AST_RWLIST_UNLOCK(&be_list);
+			AST_RWLIST_UNLOCK(generic_list);
 			ast_free(i);
 
 			return -1;
 		}
 	}
 
-	AST_RWLIST_INSERT_HEAD(&be_list, i, list);
-	AST_RWLIST_UNLOCK(&be_list);
+	AST_RWLIST_INSERT_HEAD(generic_list, i, list);
+	AST_RWLIST_UNLOCK(generic_list);
 
 	return 0;
 }
 
-int ast_cdr_unregister(const char *name)
+int ast_cdr_register(const char *name, const char *desc, ast_cdrbe be)
+{
+	return cdr_generic_register(&be_list, name, desc, be);
+}
+
+int ast_cdr_modifier_register(const char *name, const char *desc, ast_cdrbe be)
+{
+	return cdr_generic_register((struct be_list *)&mo_list, name, desc, be);
+}
+
+static int ast_cdr_generic_unregister(struct be_list *generic_list, const char *name)
 {
 	struct cdr_beitem *match = NULL;
 	int active_count;
 
-	AST_RWLIST_WRLOCK(&be_list);
-	AST_RWLIST_TRAVERSE(&be_list, match, list) {
+	AST_RWLIST_WRLOCK(generic_list);
+	AST_RWLIST_TRAVERSE(generic_list, match, list) {
 		if (!strcasecmp(name, match->name)) {
 			break;
 		}
 	}
 
 	if (!match) {
-		AST_RWLIST_UNLOCK(&be_list);
+		AST_RWLIST_UNLOCK(generic_list);
 		return 0;
 	}
 
 	active_count = ao2_container_count(active_cdrs_master);
 
 	if (!match->suspended && active_count != 0) {
-		AST_RWLIST_UNLOCK(&be_list);
+		AST_RWLIST_UNLOCK(generic_list);
 		ast_log(AST_LOG_WARNING, "Unable to unregister CDR backend %s; %d CDRs are still active\n",
 			name, active_count);
 		return -1;
 	}
 
-	AST_RWLIST_REMOVE(&be_list, match, list);
-	AST_RWLIST_UNLOCK(&be_list);
+	AST_RWLIST_REMOVE(generic_list, match, list);
+	AST_RWLIST_UNLOCK(generic_list);
 
 	ast_verb(2, "Unregistered '%s' CDR backend\n", name);
 	ast_free(match);
@@ -2955,6 +2969,16 @@ int ast_cdr_unregister(const char *name)
 	return 0;
 }
 
+int ast_cdr_unregister(const char *name)
+{
+	return ast_cdr_generic_unregister(&be_list, name);
+}
+
+int ast_cdr_modifier_unregister(const char *name)
+{
+	return ast_cdr_generic_unregister((struct be_list *)&mo_list, name);
+}
+
 struct ast_cdr *ast_cdr_dup(struct ast_cdr *cdr)
 {
 	struct ast_cdr *newcdr;
@@ -3484,6 +3508,13 @@ static void post_cdr(struct ast_cdr *cdr
 			continue;
 		}
 
+		/* Modify CDR's */
+		AST_RWLIST_RDLOCK(&mo_list);
+		AST_RWLIST_TRAVERSE(&mo_list, i, list) {
+			i->be(cdr);
+		}
+		AST_RWLIST_UNLOCK(&mo_list);
+
 		if (ast_test_flag(cdr, AST_CDR_FLAG_DISABLE)) {
 			continue;
 		}
@@ -4332,11 +4363,6 @@ static int process_config(int reload)
 	return 0;
 }
 
-static void cdr_engine_cleanup(void)
-{
-	destroy_subscriptions();
-}
-
 static void cdr_engine_shutdown(void)
 {
 	stasis_message_router_unsubscribe_and_join(stasis_router);
@@ -4467,26 +4493,33 @@ static int cdr_toggle_runtime_options(vo
 	return mod_cfg ? 0 : -1;
 }
 
-int ast_cdr_engine_init(void)
+static int unload_module(void)
+{
+	destroy_subscriptions();
+
+	return 0;
+}
+
+static int load_module(void)
 {
 	if (process_config(0)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	cdr_topic = stasis_topic_create("cdr_engine");
 	if (!cdr_topic) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	stasis_router = stasis_message_router_create(cdr_topic);
 	if (!stasis_router) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	stasis_message_router_set_congestion_limits(stasis_router, -1,
 		10 * AST_TASKPROCESSOR_HIGH_WATER_LEVEL);
 
 	if (STASIS_MESSAGE_TYPE_INIT(cdr_sync_message_type)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	stasis_message_router_add_cache_update(stasis_router, ast_channel_snapshot_type(), handle_channel_cache_message, NULL);
@@ -4499,28 +4532,27 @@ int ast_cdr_engine_init(void)
 	active_cdrs_master = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
 		NUM_CDR_BUCKETS, cdr_master_hash_fn, NULL, cdr_master_cmp_fn);
 	if (!active_cdrs_master) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	ao2_container_register("cdrs_master", active_cdrs_master, cdr_master_print_fn);
 
 	active_cdrs_all = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
 		NUM_CDR_BUCKETS, cdr_all_hash_fn, NULL, cdr_all_cmp_fn);
 	if (!active_cdrs_all) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	ao2_container_register("cdrs_all", active_cdrs_all, cdr_all_print_fn);
 
 	sched = ast_sched_context_create();
 	if (!sched) {
 		ast_log(LOG_ERROR, "Unable to create schedule context.\n");
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	ast_cli_register_multiple(cli_commands, ARRAY_LEN(cli_commands));
-	ast_register_cleanup(cdr_engine_cleanup);
 	ast_register_atexit(cdr_engine_shutdown);
 
-	return cdr_toggle_runtime_options();
+	return cdr_toggle_runtime_options() ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
 
 void ast_cdr_engine_term(void)
@@ -4563,7 +4595,7 @@ void ast_cdr_engine_term(void)
 	}
 }
 
-int ast_cdr_engine_reload(void)
+static int reload_module(void)
 {
 	struct module_config *old_mod_cfg;
 	struct module_config *mod_cfg;
@@ -4589,3 +4621,12 @@ int ast_cdr_engine_reload(void)
 	ao2_cleanup(old_mod_cfg);
 	return cdr_toggle_runtime_options();
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "CDR Engine",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/cel.c 1:16.1.1~dfsg-1/main/cel.c
--- 1:13.23.1~dfsg-2/main/cel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/cel.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,9 +38,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include "asterisk/_private.h"
+#include "asterisk/module.h"
 
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -1423,7 +1421,7 @@ static void destroy_subscriptions(void)
 	cel_cel_forwarder = stasis_forward_cancel(cel_cel_forwarder);
 }
 
-static void cel_engine_cleanup(void)
+static int unload_module(void)
 {
 	destroy_routes();
 	destroy_subscriptions();
@@ -1435,6 +1433,8 @@ static void cel_engine_cleanup(void)
 	ao2_global_obj_release(cel_dialstatus_store);
 	ao2_global_obj_release(cel_linkedids);
 	ao2_global_obj_release(cel_backends);
+
+	return 0;
 }
 
 /*!
@@ -1557,42 +1557,44 @@ static int create_routes(void)
 AO2_STRING_FIELD_HASH_FN(cel_linkedid, id)
 AO2_STRING_FIELD_CMP_FN(cel_linkedid, id)
 
-int ast_cel_engine_init(void)
+static int load_module(void)
 {
 	struct ao2_container *container;
 
-	container = ao2_container_alloc(NUM_APP_BUCKETS, cel_linkedid_hash_fn, cel_linkedid_cmp_fn);
+	container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NUM_APP_BUCKETS, cel_linkedid_hash_fn, NULL, cel_linkedid_cmp_fn);
 	ao2_global_obj_replace_unref(cel_linkedids, container);
 	ao2_cleanup(container);
 	if (!container) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
-	container = ao2_container_alloc(NUM_DIALSTATUS_BUCKETS,
-		cel_dialstatus_hash_fn, cel_dialstatus_cmp_fn);
+	container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NUM_DIALSTATUS_BUCKETS, cel_dialstatus_hash_fn, NULL, cel_dialstatus_cmp_fn);
 	ao2_global_obj_replace_unref(cel_dialstatus_store, container);
 	ao2_cleanup(container);
 	if (!container) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (STASIS_MESSAGE_TYPE_INIT(cel_generic_type)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (ast_cli_register(&cli_status)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
-	container = ao2_container_alloc(BACKEND_BUCKETS, cel_backend_hash_fn, cel_backend_cmp_fn);
+	container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, BACKEND_BUCKETS,
+		cel_backend_hash_fn, NULL, cel_backend_cmp_fn);
 	ao2_global_obj_replace_unref(cel_backends, container);
 	ao2_cleanup(container);
 	if (!container) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (aco_info_init(&cel_cfg_info)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	aco_option_register(&cel_cfg_info, "enable", ACO_EXACT, general_options, "no", OPT_BOOL_T, 1, FLDSET(struct ast_cel_general_config, enable));
@@ -1604,7 +1606,7 @@ int ast_cel_engine_init(void)
 		struct cel_config *cel_cfg = cel_config_alloc();
 
 		if (!cel_cfg) {
-			return -1;
+			return AST_MODULE_LOAD_FAILURE;
 		}
 
 		/* We couldn't process the configuration so create a default config. */
@@ -1616,18 +1618,17 @@ int ast_cel_engine_init(void)
 	}
 
 	if (create_subscriptions()) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (ast_cel_check_enabled() && create_routes()) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
-	ast_register_cleanup(cel_engine_cleanup);
-	return 0;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
-int ast_cel_engine_reload(void)
+static int reload_module(void)
 {
 	unsigned int was_enabled = ast_cel_check_enabled();
 	unsigned int is_enabled;
@@ -1747,3 +1748,12 @@ int ast_cel_backend_register(const char
 	ao2_ref(backend, -1);
 	return 0;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "CEL Engine",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/channel.c 1:16.1.1~dfsg-1/main/channel.c
--- 1:13.23.1~dfsg-2/main/channel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/channel.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #include <sys/time.h>
@@ -68,33 +66,24 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/autochan.h"
 #include "asterisk/stringfields.h"
 #include "asterisk/global_datastores.h"
-#include "asterisk/data.h"
 #include "asterisk/channel_internal.h"
 #include "asterisk/features.h"
 #include "asterisk/bridge.h"
 #include "asterisk/test.h"
 #include "asterisk/stasis_channels.h"
 #include "asterisk/max_forwards.h"
+#include "asterisk/stream.h"
 #include "asterisk/message.h"
 
 /*** DOCUMENTATION
  ***/
 
-#ifdef HAVE_EPOLL
-#include <sys/epoll.h>
-#endif
-
 #if defined(KEEP_TILL_CHANNEL_PARTY_NUMBER_INFO_NEEDED)
 #if defined(HAVE_PRI)
 #include "libpri.h"
 #endif	/* defined(HAVE_PRI) */
 #endif	/* defined(KEEP_TILL_CHANNEL_PARTY_NUMBER_INFO_NEEDED) */
 
-struct ast_epoll_data {
-	struct ast_channel *chan;
-	int which;
-};
-
 /* uncomment if you have problems with 'monitoring' synchronized files */
 #if 0
 #define MONITOR_CONSTANT_DELAY
@@ -273,7 +262,7 @@ static const char *party_number_plan2str
 /*! \brief Show channel types - CLI command */
 static char *handle_cli_core_show_channeltypes(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-#define FORMAT  "%-15.15s  %-40.40s %-12.12s %-12.12s %-12.12s\n"
+#define FORMAT  "%-15.15s  %-40.40s %-13.13s %-13.13s %-13.13s %-13.13s\n"
 	struct chanlist *cl;
 	int count_chan = 0;
 
@@ -292,13 +281,14 @@ static char *handle_cli_core_show_channe
 	if (a->argc != 3)
 		return CLI_SHOWUSAGE;
 
-	ast_cli(a->fd, FORMAT, "Type", "Description",       "Devicestate", "Indications", "Transfer");
-	ast_cli(a->fd, FORMAT, "-----------", "-----------", "-----------", "-----------", "-----------");
+	ast_cli(a->fd, FORMAT, "Type", "Description", "Devicestate", "Presencestate", "Indications", "Transfer");
+	ast_cli(a->fd, FORMAT, "-------------", "-------------", "-------------", "-------------", "-------------", "-------------");
 
 	AST_RWLIST_RDLOCK(&backends);
 	AST_RWLIST_TRAVERSE(&backends, cl, list) {
 		ast_cli(a->fd, FORMAT, cl->tech->type, cl->tech->description,
 			(cl->tech->devicestate) ? "yes" : "no",
+			(cl->tech->presencestate) ? "yes" : "no",
 			(cl->tech->indicate) ? "yes" : "no",
 			(cl->tech->transfer) ? "yes" : "no");
 		count_chan++;
@@ -371,6 +361,7 @@ static char *handle_cli_core_show_channe
 	ast_cli(a->fd,
 		"-- Info about channel driver: %s --\n"
 		"  Device State: %s\n"
+		"Presence State: %s\n"
 		"    Indication: %s\n"
 		"     Transfer : %s\n"
 		"  Capabilities: %s\n"
@@ -381,6 +372,7 @@ static char *handle_cli_core_show_channe
 		"  Text Support: %s\n",
 		cl->tech->type,
 		(cl->tech->devicestate) ? "yes" : "no",
+		(cl->tech->presencestate) ? "yes" : "no",
 		(cl->tech->indicate) ? "yes" : "no",
 		(cl->tech->transfer) ? "yes" : "no",
 		ast_format_cap_get_names(cl->tech->capabilities, &codec_buf),
@@ -529,12 +521,6 @@ void ast_channel_setwhentohangup_tv(stru
 	return;
 }
 
-void ast_channel_setwhentohangup(struct ast_channel *chan, time_t offset)
-{
-	struct timeval when = { offset, };
-	ast_channel_setwhentohangup_tv(chan, when);
-}
-
 /*! \brief Compare a offset with when to hangup channel */
 int ast_channel_cmpwhentohangup_tv(struct ast_channel *chan, struct timeval offset)
 {
@@ -551,12 +537,6 @@ int ast_channel_cmpwhentohangup_tv(struc
 	return ast_tvdiff_ms(whentohangup, *ast_channel_whentohangup(chan));
 }
 
-int ast_channel_cmpwhentohangup(struct ast_channel *chan, time_t offset)
-{
-	struct timeval when = { offset, };
-	return ast_channel_cmpwhentohangup_tv(chan, when);
-}
-
 /*! \brief Register a new telephony channel in Asterisk */
 int ast_channel_register(const struct ast_channel_tech *tech)
 {
@@ -771,7 +751,7 @@ static int ast_channel_by_uniqueid_cb(vo
 static int does_id_conflict(const char *uniqueid)
 {
 	struct ast_channel *conflict;
-	int length = 0;
+	size_t length = 0;
 
 	if (ast_strlen_zero(uniqueid)) {
 		return 0;
@@ -804,6 +784,7 @@ __ast_channel_alloc_ap(int needqueue, in
 	struct ast_timer *timer;
 	struct timeval now;
 	const struct ast_channel_tech *channel_tech;
+	struct ast_stream_topology *topology;
 
 	/* If shutting down, don't allocate any new channels */
 	if (ast_shutting_down()) {
@@ -811,7 +792,9 @@ __ast_channel_alloc_ap(int needqueue, in
 		return NULL;
 	}
 
-	if (!(tmp = ast_channel_internal_alloc(ast_channel_destructor, assignedids, requestor))) {
+	tmp = __ast_channel_internal_alloc(ast_channel_destructor, assignedids, requestor,
+		file, line, function);
+	if (!tmp) {
 		/* Channel structure allocation failure. */
 		return NULL;
 	}
@@ -826,10 +809,6 @@ __ast_channel_alloc_ap(int needqueue, in
 	ast_channel_internal_alertpipe_clear(tmp);
 	ast_channel_internal_fd_clear_all(tmp);
 
-#ifdef HAVE_EPOLL
-	ast_channel_epfd_set(tmp, epoll_create(25));
-#endif
-
 	nativeformats = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
 	if (!nativeformats) {
 		/*
@@ -892,6 +871,11 @@ __ast_channel_alloc_ap(int needqueue, in
 		return ast_channel_unref(tmp);
 	}
 
+	if (!(topology = ast_stream_topology_alloc())) {
+		return ast_channel_unref(tmp);
+	}
+	ast_channel_internal_set_stream_topology(tmp, topology);
+
 	/* Always watch the alertpipe */
 	ast_channel_set_fd(tmp, AST_ALERT_FD, ast_channel_internal_alert_readfd(tmp));
 	/* And timing pipe */
@@ -1028,16 +1012,14 @@ struct ast_channel *__ast_channel_alloc(
 
 /* only do the minimum amount of work needed here to make a channel
  * structure that can be used to expand channel vars */
-#if defined(REF_DEBUG) || defined(__AST_DEBUG_MALLOC)
 struct ast_channel *__ast_dummy_channel_alloc(const char *file, int line, const char *function)
-#else
-struct ast_channel *ast_dummy_channel_alloc(void)
-#endif
 {
 	struct ast_channel *tmp;
 	struct varshead *headp;
 
-	if (!(tmp = ast_channel_internal_alloc(ast_dummy_channel_destructor, NULL, NULL))) {
+	tmp = __ast_channel_internal_alloc(ast_dummy_channel_destructor, NULL, NULL,
+		file, line, function);
+	if (!tmp) {
 		/* Dummy channel structure allocation failure. */
 		return NULL;
 	}
@@ -1053,9 +1035,6 @@ struct ast_channel *ast_dummy_channel_al
 	ast_channel_timingfd_set(tmp, -1);
 	ast_channel_internal_alertpipe_clear(tmp);
 	ast_channel_internal_fd_clear_all(tmp);
-#ifdef HAVE_EPOLL
-	ast_channel_epfd_set(tmp, -1);
-#endif
 
 	ast_channel_hold_state_set(tmp, AST_CONTROL_UNHOLD);
 
@@ -2222,15 +2201,12 @@ void ast_party_redirecting_free(struct a
 static void ast_channel_destructor(void *obj)
 {
 	struct ast_channel *chan = obj;
-#ifdef HAVE_EPOLL
-	int i;
-#endif
 	struct ast_var_t *vardata;
 	struct ast_frame *f;
 	struct varshead *headp;
 	struct ast_datastore *datastore;
 	char device_name[AST_CHANNEL_NAME];
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	ast_debug(1, "Channel %p '%s' destroying\n", chan, ast_channel_name(chan));
 
@@ -2324,14 +2300,6 @@ static void ast_channel_destructor(void
 		ast_timer_close(ast_channel_timer(chan));
 		ast_channel_timer_set(chan, NULL);
 	}
-#ifdef HAVE_EPOLL
-	for (i = 0; i < AST_MAX_FDS; i++) {
-		if (ast_channel_internal_epfd_data(chan, i)) {
-			ast_free(ast_channel_internal_epfd_data(chan, i));
-		}
-	}
-	close(ast_channel_epfd(chan));
-#endif
 	while ((f = AST_LIST_REMOVE_HEAD(ast_channel_readq(chan), frame_list)))
 		ast_frfree(f);
 
@@ -2369,9 +2337,6 @@ static void ast_channel_destructor(void
 	}
 
 	ast_channel_nativeformats_set(chan, NULL);
-	if (callid) {
-		ast_callid_unref(callid);
-	}
 
 	ast_channel_named_callgroups_set(chan, NULL);
 	ast_channel_named_pickupgroups_set(chan, NULL);
@@ -2415,16 +2380,6 @@ static void ast_dummy_channel_destructor
 	ast_channel_internal_cleanup(chan);
 }
 
-struct ast_datastore *ast_channel_datastore_alloc(const struct ast_datastore_info *info, const char *uid)
-{
-	return ast_datastore_alloc(info, uid);
-}
-
-int ast_channel_datastore_free(struct ast_datastore *datastore)
-{
-	return ast_datastore_free(datastore);
-}
-
 int ast_channel_datastore_inherit(struct ast_channel *from, struct ast_channel *to)
 {
 	struct ast_datastore *datastore = NULL, *datastore2;
@@ -2485,82 +2440,10 @@ struct ast_datastore *ast_channel_datast
 /*! Set the file descriptor on the channel */
 void ast_channel_set_fd(struct ast_channel *chan, int which, int fd)
 {
-#ifdef HAVE_EPOLL
-	struct epoll_event ev;
-	struct ast_epoll_data *aed = NULL;
-
-	if (ast_channel_fd_isset(chan, which)) {
-		epoll_ctl(ast_channel_epfd(chan), EPOLL_CTL_DEL, ast_channel_fd(chan, which), &ev);
-		aed = ast_channel_internal_epfd_data(chan, which);
-	}
-
-	/* If this new fd is valid, add it to the epoll */
-	if (fd > -1) {
-		if (!aed && (!(aed = ast_calloc(1, sizeof(*aed)))))
-			return;
-
-		ast_channel_internal_epfd_data_set(chan, which, aed);
-		aed->chan = chan;
-		aed->which = which;
-
-		ev.events = EPOLLIN | EPOLLPRI | EPOLLERR | EPOLLHUP;
-		ev.data.ptr = aed;
-		epoll_ctl(ast_channel_epfd(chan), EPOLL_CTL_ADD, fd, &ev);
-	} else if (aed) {
-		/* We don't have to keep around this epoll data structure now */
-		ast_free(aed);
-		ast_channel_epfd_data_set(chan, which, NULL);
-	}
-#endif
 	ast_channel_internal_fd_set(chan, which, fd);
 	return;
 }
 
-/*! Add a channel to an optimized waitfor */
-void ast_poll_channel_add(struct ast_channel *chan0, struct ast_channel *chan1)
-{
-#ifdef HAVE_EPOLL
-	struct epoll_event ev;
-	int i = 0;
-
-	if (ast_channel_epfd(chan0) == -1)
-		return;
-
-	/* Iterate through the file descriptors on chan1, adding them to chan0 */
-	for (i = 0; i < AST_MAX_FDS; i++) {
-		if (!ast_channel_fd_isset(chan1, i)) {
-			continue;
-		}
-		ev.events = EPOLLIN | EPOLLPRI | EPOLLERR | EPOLLHUP;
-		ev.data.ptr = ast_channel_internal_epfd_data(chan1, i);
-		epoll_ctl(ast_channel_epfd(chan0), EPOLL_CTL_ADD, ast_channel_fd(chan1, i), &ev);
-	}
-
-#endif
-	return;
-}
-
-/*! Delete a channel from an optimized waitfor */
-void ast_poll_channel_del(struct ast_channel *chan0, struct ast_channel *chan1)
-{
-#ifdef HAVE_EPOLL
-	struct epoll_event ev;
-	int i = 0;
-
-	if (ast_channel_epfd(chan0) == -1)
-		return;
-
-	for (i = 0; i < AST_MAX_FDS; i++) {
-		if (!ast_channel_fd_isset(chan1, i)) {
-			continue;
-		}
-		epoll_ctl(ast_channel_epfd(chan0), EPOLL_CTL_DEL, ast_channel_fd(chan1, i), &ev);
-	}
-
-#endif
-	return;
-}
-
 void ast_channel_clear_softhangup(struct ast_channel *chan, int flag)
 {
 	ast_channel_lock(chan);
@@ -3068,20 +2951,15 @@ int ast_waitfor_n_fd(int *fds, int n, in
 }
 
 /*! \brief Wait for x amount of time on a file descriptor to have input.  */
-#ifdef HAVE_EPOLL
-static struct ast_channel *ast_waitfor_nandfds_classic(struct ast_channel **c, int n, int *fds, int nfds,
-					int *exception, int *outfd, int *ms)
-#else
 struct ast_channel *ast_waitfor_nandfds(struct ast_channel **c, int n, int *fds, int nfds,
 					int *exception, int *outfd, int *ms)
-#endif
 {
 	struct timeval start = { 0 , 0 };
 	struct pollfd *pfds = NULL;
 	int res;
 	long rms;
 	int x, y, max;
-	int sz;
+	int sz = nfds;
 	struct timeval now = { 0, 0 };
 	struct timeval whentohangup = { 0, 0 }, diff;
 	struct ast_channel *winner = NULL;
@@ -3097,14 +2975,6 @@ struct ast_channel *ast_waitfor_nandfds(
 		*exception = 0;
 	}
 
-	if ((sz = n * AST_MAX_FDS + nfds)) {
-		pfds = ast_alloca(sizeof(*pfds) * sz);
-		fdmap = ast_alloca(sizeof(*fdmap) * sz);
-	} else {
-		/* nothing to allocate and no FDs to check */
-		return NULL;
-	}
-
 	for (x = 0; x < n; x++) {
 		ast_channel_lock(c[x]);
 		if (!ast_tvzero(*ast_channel_whentohangup(c[x]))) {
@@ -3121,8 +2991,17 @@ struct ast_channel *ast_waitfor_nandfds(
 			if (ast_tvzero(whentohangup) || ast_tvcmp(diff, whentohangup) < 0)
 				whentohangup = diff;
 		}
+		sz += ast_channel_fd_count(c[x]);
 		ast_channel_unlock(c[x]);
 	}
+
+	if (!sz) {
+		return NULL;
+	}
+
+	pfds = ast_alloca(sizeof(*pfds) * sz);
+	fdmap = ast_alloca(sizeof(*fdmap) * sz);
+
 	/* Wait full interval */
 	rms = *ms;
 	/* INT_MAX, not LONG_MAX, because it matters on 64-bit */
@@ -3142,12 +3021,12 @@ struct ast_channel *ast_waitfor_nandfds(
 	 */
 	max = 0;
 	for (x = 0; x < n; x++) {
-		for (y = 0; y < AST_MAX_FDS; y++) {
+		ast_channel_lock(c[x]);
+		for (y = 0; y < ast_channel_fd_count(c[x]); y++) {
 			fdmap[max].fdno = y;  /* fd y is linked to this pfds */
 			fdmap[max].chan = x;  /* channel x is linked to this pfds */
 			max += ast_add_fd(&pfds[max], ast_channel_fd(c[x], y));
 		}
-		ast_channel_lock(c[x]);
 		CHECK_BLOCKING(c[x]);
 		ast_channel_unlock(c[x]);
 	}
@@ -3241,213 +3120,6 @@ struct ast_channel *ast_waitfor_nandfds(
 	return winner;
 }
 
-#ifdef HAVE_EPOLL
-static struct ast_channel *ast_waitfor_nandfds_simple(struct ast_channel *chan, int *ms)
-{
-	struct timeval start = { 0 , 0 };
-	int res = 0;
-	struct epoll_event ev[1];
-	long diff, rms = *ms;
-	struct ast_channel *winner = NULL;
-	struct ast_epoll_data *aed = NULL;
-
-	ast_channel_lock(chan);
-	/* Figure out their timeout */
-	if (!ast_tvzero(*ast_channel_whentohangup(chan))) {
-		if ((diff = ast_tvdiff_ms(*ast_channel_whentohangup(chan), ast_tvnow())) < 0) {
-			/* They should already be hungup! */
-			ast_channel_softhangup_internal_flag_add(chan, AST_SOFTHANGUP_TIMEOUT);
-			ast_channel_unlock(chan);
-			return NULL;
-		}
-		/* If this value is smaller then the current one... make it priority */
-		if (rms > diff) {
-			rms = diff;
-		}
-	}
-
-	/* Time to make this channel block... */
-	CHECK_BLOCKING(chan);
-
-	ast_channel_unlock(chan);
-
-	if (*ms > 0) {
-		start = ast_tvnow();
-	}
-
-	/* We don't have to add any file descriptors... they are already added, we just have to wait! */
-	res = epoll_wait(ast_channel_epfd(chan), ev, 1, rms);
-
-	/* Stop blocking */
-	ast_channel_lock(chan);
-	ast_clear_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING);
-	ast_channel_unlock(chan);
-
-	/* Simulate a timeout if we were interrupted */
-	if (res < 0) {
-		if (errno != EINTR) {
-			*ms = -1;
-		}
-		return NULL;
-	}
-
-	/* If this channel has a timeout see if it expired */
-	if (!ast_tvzero(*ast_channel_whentohangup(chan))) {
-		if (ast_tvdiff_ms(ast_tvnow(), *ast_channel_whentohangup(chan)) >= 0) {
-			ast_channel_softhangup_internal_flag_add(chan, AST_SOFTHANGUP_TIMEOUT);
-			winner = chan;
-		}
-	}
-
-	/* No fd ready, reset timeout and be done for now */
-	if (!res) {
-		*ms = 0;
-		return winner;
-	}
-
-	/* See what events are pending */
-	aed = ev[0].data.ptr;
-	ast_channel_lock(chan);
-	ast_channel_fdno_set(chan, aed->which);
-	if (ev[0].events & EPOLLPRI) {
-		ast_set_flag(ast_channel_flags(chan), AST_FLAG_EXCEPTION);
-	} else {
-		ast_clear_flag(ast_channel_flags(chan), AST_FLAG_EXCEPTION);
-	}
-	ast_channel_unlock(chan);
-
-	if (*ms > 0) {
-		*ms -= ast_tvdiff_ms(ast_tvnow(), start);
-		if (*ms < 0) {
-			*ms = 0;
-		}
-	}
-
-	return chan;
-}
-
-static struct ast_channel *ast_waitfor_nandfds_complex(struct ast_channel **c, int n, int *ms)
-{
-	struct timeval start = { 0 , 0 };
-	int res = 0, i;
-	struct epoll_event ev[25] = { { 0, } };
-	struct timeval now = { 0, 0 };
-	long whentohangup = 0, diff = 0, rms = *ms;
-	struct ast_channel *winner = NULL;
-
-	for (i = 0; i < n; i++) {
-		ast_channel_lock(c[i]);
-		if (!ast_tvzero(*ast_channel_whentohangup(c[i]))) {
-			if (whentohangup == 0) {
-				now = ast_tvnow();
-			}
-			if ((diff = ast_tvdiff_ms(*ast_channel_whentohangup(c[i]), now)) < 0) {
-				ast_channel_softhangup_internal_flag_add(c[i], AST_SOFTHANGUP_TIMEOUT);
-				ast_channel_unlock(c[i]);
-				return c[i];
-			}
-			if (!whentohangup || whentohangup > diff) {
-				whentohangup = diff;
-			}
-		}
-		CHECK_BLOCKING(c[i]);
-		ast_channel_unlock(c[i]);
-	}
-
-	rms = *ms;
-	if (whentohangup) {
-		rms = whentohangup;
-		if (*ms >= 0 && *ms < rms) {
-			rms = *ms;
-		}
-	}
-
-	if (*ms > 0) {
-		start = ast_tvnow();
-	}
-
-	res = epoll_wait(ast_channel_epfd(c[0]), ev, 25, rms);
-
-	for (i = 0; i < n; i++) {
-		ast_channel_lock(c[i]);
-		ast_clear_flag(ast_channel_flags(c[i]), AST_FLAG_BLOCKING);
-		ast_channel_unlock(c[i]);
-	}
-
-	if (res < 0) {
-		if (errno != EINTR) {
-			*ms = -1;
-		}
-		return NULL;
-	}
-
-	if (whentohangup) {
-		now = ast_tvnow();
-		for (i = 0; i < n; i++) {
-			if (!ast_tvzero(*ast_channel_whentohangup(c[i])) && ast_tvdiff_ms(now, *ast_channel_whentohangup(c[i])) >= 0) {
-				ast_channel_softhangup_internal_flag_add(c[i], AST_SOFTHANGUP_TIMEOUT);
-				if (!winner) {
-					winner = c[i];
-				}
-			}
-		}
-	}
-
-	if (!res) {
-		*ms = 0;
-		return winner;
-	}
-
-	for (i = 0; i < res; i++) {
-		struct ast_epoll_data *aed = ev[i].data.ptr;
-
-		if (!ev[i].events || !aed) {
-			continue;
-		}
-
-		winner = aed->chan;
-		ast_channel_lock(winner);
-		if (ev[i].events & EPOLLPRI) {
-			ast_set_flag(ast_channel_flags(winner), AST_FLAG_EXCEPTION);
-		} else {
-			ast_clear_flag(ast_channel_flags(winner), AST_FLAG_EXCEPTION);
-		}
-		ast_channel_fdno_set(winner, aed->which);
-		ast_channel_unlock(winner);
-	}
-
-	if (*ms > 0) {
-		*ms -= ast_tvdiff_ms(ast_tvnow(), start);
-		if (*ms < 0) {
-			*ms = 0;
-		}
-	}
-
-	return winner;
-}
-
-struct ast_channel *ast_waitfor_nandfds(struct ast_channel **c, int n, int *fds, int nfds,
-					int *exception, int *outfd, int *ms)
-{
-	/* Clear all provided values in one place. */
-	if (outfd) {
-		*outfd = -99999;
-	}
-	if (exception) {
-		*exception = 0;
-	}
-
-	/* If no epoll file descriptor is available resort to classic nandfds */
-	if (!n || nfds || ast_channel_epfd(c[0]) == -1) {
-		return ast_waitfor_nandfds_classic(c, n, fds, nfds, exception, outfd, ms);
-	} else if (!nfds && n == 1) {
-		return ast_waitfor_nandfds_simple(c[0], ms);
-	} else {
-		return ast_waitfor_nandfds_complex(c, n, ms);
-	}
-}
-#endif
-
 struct ast_channel *ast_waitfor_n(struct ast_channel **c, int n, int *ms)
 {
 	return ast_waitfor_nandfds(c, n, NULL, 0, NULL, NULL, ms);
@@ -3468,7 +3140,7 @@ int ast_waitfor(struct ast_channel *c, i
 
 int ast_waitfordigit(struct ast_channel *c, int ms)
 {
-	return ast_waitfordigit_full(c, ms, -1, -1);
+	return ast_waitfordigit_full(c, ms, NULL, -1, -1);
 }
 
 int ast_settimeout(struct ast_channel *c, unsigned int rate, int (*func)(const void *data), void *data)
@@ -3530,7 +3202,7 @@ int ast_settimeout_full(struct ast_chann
 	return res;
 }
 
-int ast_waitfordigit_full(struct ast_channel *c, int timeout_ms, int audiofd, int cmdfd)
+int ast_waitfordigit_full(struct ast_channel *c, int timeout_ms, const char *breakon, int audiofd, int cmdfd)
 {
 	struct timeval start = ast_tvnow();
 	int ms;
@@ -3581,9 +3253,12 @@ int ast_waitfordigit_full(struct ast_cha
 				break;
 			case AST_FRAME_DTMF_END:
 				res = f->subclass.integer;
-				ast_frfree(f);
-				ast_channel_clear_flag(c, AST_FLAG_END_DTMF_ONLY);
-				return res;
+				if (!breakon || strchr(breakon, res)) {
+					ast_frfree(f);
+					ast_channel_clear_flag(c, AST_FLAG_END_DTMF_ONLY);
+					return res;
+				}
+				break;
 			case AST_FRAME_CONTROL:
 				switch (f->subclass.integer) {
 				case AST_CONTROL_HANGUP:
@@ -3801,11 +3476,12 @@ static inline int calc_monitor_jump(int
 	return samples;
 }
 
-static struct ast_frame *__ast_read(struct ast_channel *chan, int dropaudio)
+static struct ast_frame *__ast_read(struct ast_channel *chan, int dropaudio, int dropnondefault)
 {
 	struct ast_frame *f = NULL;	/* the return value */
 	int prestate;
 	int cause = 0;
+	struct ast_stream *stream = NULL, *default_stream = NULL;
 
 	/* this function is very long so make sure there is only one return
 	 * point at the end (there are only two exceptions to this).
@@ -3962,6 +3638,27 @@ static struct ast_frame *__ast_read(stru
 			default:
 				break;
 			}
+		} else if (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_VIDEO) {
+			if (ast_channel_tech(chan) && ast_channel_tech(chan)->read_stream) {
+				stream = ast_stream_topology_get_stream(ast_channel_get_stream_topology(chan), f->stream_num);
+				default_stream = ast_channel_get_default_stream(chan, ast_format_get_type(f->subclass.format));
+			} else {
+				/* Since this channel driver does not support multistream determine the default stream this frame
+				 * originated from and update the frame to include it.
+				 */
+				stream = default_stream = ast_channel_get_default_stream(chan, ast_format_get_type(f->subclass.format));
+				/* In order to allow media to be passed up the underlying media type has to have a format negotiated on
+				 * the channel itself. In cases where this hasn't happened the channel driver is incorrectly passing up
+				 * a frame for a format that has not been negotiated. If this occurs just drop the frame as we have no
+				 * stream that it came from.
+				 */
+				if (!stream) {
+					ast_frfree(f);
+					f = &ast_null_frame;
+				} else {
+					f->stream_num = ast_stream_get_position(stream);
+				}
+			}
 		}
 	} else {
 		if (ast_test_flag(ast_channel_flags(chan), AST_FLAG_EXCEPTION)) {
@@ -3973,15 +3670,42 @@ static struct ast_frame *__ast_read(stru
 			}
 			/* Clear the exception flag */
 			ast_clear_flag(ast_channel_flags(chan), AST_FLAG_EXCEPTION);
-		} else if (ast_channel_tech(chan) && ast_channel_tech(chan)->read)
+		} else if (ast_channel_tech(chan) && ast_channel_tech(chan)->read_stream) {
+			f = ast_channel_tech(chan)->read_stream(chan);
+
+			/* This channel driver supports multistream so the stream_num on the frame is valid, the only
+			 * thing different is that we need to find the default stream so we know whether to invoke the
+			 * default stream logic or not (such as transcoding).
+			 */
+			if (f && (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_VIDEO)) {
+				stream = ast_stream_topology_get_stream(ast_channel_get_stream_topology(chan), f->stream_num);
+				default_stream = ast_channel_get_default_stream(chan, ast_format_get_type(f->subclass.format));
+			}
+		} else if (ast_channel_tech(chan) && ast_channel_tech(chan)->read) {
 			f = ast_channel_tech(chan)->read(chan);
+
+			/* Since this channel driver does not support multistream determine the default stream this frame
+			 * originated from and update the frame to include it.
+			 */
+			if (f && (f->frametype == AST_FRAME_VOICE || f->frametype == AST_FRAME_VIDEO)) {
+				stream = default_stream = ast_channel_get_default_stream(chan, ast_format_get_type(f->subclass.format));
+				if (!stream) {
+					ast_frfree(f);
+					f = &ast_null_frame;
+				} else {
+					f->stream_num = ast_stream_get_position(stream);
+				}
+			}
+		}
 		else
 			ast_log(LOG_WARNING, "No read routine on channel %s\n", ast_channel_name(chan));
 	}
 
-	/* Perform the framehook read event here. After the frame enters the framehook list
-	 * there is no telling what will happen, <insert mad scientist laugh here>!!! */
-	f = ast_framehook_list_read_event(ast_channel_framehooks(chan), f);
+	if (stream == default_stream) {
+		/* Perform the framehook read event here. After the frame enters the framehook list
+		 * there is no telling what will happen, <insert mad scientist laugh here>!!! */
+		f = ast_framehook_list_read_event(ast_channel_framehooks(chan), f);
+	}
 
 	/*
 	 * Reset the recorded file descriptor that triggered this read so that we can
@@ -4004,6 +3728,14 @@ static struct ast_frame *__ast_read(stru
 			AST_LIST_NEXT(f, frame_list) = NULL;
 		}
 
+		if (dropnondefault && stream != default_stream) {
+			/* If the frame originates from a non-default stream and the caller can not handle other streams
+			 * absorb the frame and replace it with a null one instead.
+			 */
+			ast_frfree(f);
+			f = &ast_null_frame;
+		}
+
 		switch (f->frametype) {
 		case AST_FRAME_CONTROL:
 			if (f->subclass.integer == AST_CONTROL_ANSWER) {
@@ -4049,6 +3781,19 @@ static struct ast_frame *__ast_read(stru
 				}
 				ast_frfree(f);
 				f = &ast_null_frame;
+			} else if (f->subclass.integer == AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE && dropnondefault) {
+				/* The caller of this function is incapable of handling streams so we don't accept the change request
+				 * and stick to the streams currently on the channel.
+				 */
+				ast_channel_stream_topology_changed(chan, ast_channel_get_stream_topology(chan));
+				ast_frfree(f);
+				f = &ast_null_frame;
+			} else if (f->subclass.integer == AST_CONTROL_STREAM_TOPOLOGY_CHANGED && dropnondefault) {
+				/* The caller of this function is incapable of handling streams so we absorb the notification that the
+				 * stream topology has changed.
+				 */
+				ast_frfree(f);
+				f = &ast_null_frame;
 			}
 			break;
 		case AST_FRAME_DTMF_END:
@@ -4185,6 +3930,11 @@ static struct ast_frame *__ast_read(stru
 			}
 			break;
 		case AST_FRAME_VOICE:
+			/* If media was received from a non-default stream don't perform any actions, let it just go through */
+			if (stream != default_stream) {
+				break;
+			}
+
 			/* The EMULATE_DTMF flag must be cleared here as opposed to when the duration
 			 * is reached , because we want to make sure we pass at least one
 			 * voice frame through before starting the next digit, to ensure a gap
@@ -4375,6 +4125,13 @@ static struct ast_frame *__ast_read(stru
 			 */
 			ast_read_generator_actions(chan, f);
 			break;
+		case AST_FRAME_RTCP:
+			/* Incoming RTCP feedback needs to get to the translator for
+			 * outgoing media, which means we treat it as an ast_write */
+			if (ast_channel_writetrans(chan)) {
+				ast_translate(ast_channel_writetrans(chan), f, 0);
+			}
+			break;
 		default:
 			/* Just pass it on! */
 			break;
@@ -4411,12 +4168,22 @@ done:
 
 struct ast_frame *ast_read(struct ast_channel *chan)
 {
-	return __ast_read(chan, 0);
+	return __ast_read(chan, 0, 1);
+}
+
+struct ast_frame *ast_read_stream(struct ast_channel *chan)
+{
+	return __ast_read(chan, 0, 0);
 }
 
 struct ast_frame *ast_read_noaudio(struct ast_channel *chan)
 {
-	return __ast_read(chan, 1);
+	return __ast_read(chan, 1, 1);
+}
+
+struct ast_frame *ast_read_stream_noaudio(struct ast_channel *chan)
+{
+	return __ast_read(chan, 1, 0);
 }
 
 int ast_indicate(struct ast_channel *chan, int condition)
@@ -4457,6 +4224,9 @@ static int attribute_const is_visible_in
 	case AST_CONTROL_UPDATE_RTP_PEER:
 	case AST_CONTROL_PVT_CAUSE_CODE:
 	case AST_CONTROL_MASQUERADE_NOTIFY:
+	case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+	case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+	case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
 	case AST_CONTROL_STREAM_STOP:
 	case AST_CONTROL_STREAM_SUSPEND:
 	case AST_CONTROL_STREAM_REVERSE:
@@ -4755,6 +4525,9 @@ static int indicate_data_internal(struct
 	case AST_CONTROL_MCID:
 	case AST_CONTROL_MASQUERADE_NOTIFY:
 	case AST_CONTROL_UPDATE_RTP_PEER:
+	case AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE:
+	case AST_CONTROL_STREAM_TOPOLOGY_CHANGED:
+	case AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED:
 	case AST_CONTROL_STREAM_STOP:
 	case AST_CONTROL_STREAM_SUSPEND:
 	case AST_CONTROL_STREAM_REVERSE:
@@ -4905,7 +4678,8 @@ int ast_sendtext_data(struct ast_channel
 		&& (ast_strlen_zero(content_type) || ast_begins_with(content_type, "text/"))
 		&& (ast_format_cap_has_type(ast_channel_nativeformats(chan), AST_MEDIA_TYPE_TEXT))) {
 		struct ast_frame f;
-		size_t body_len = strlen(body) + 1;
+		/* T.140 payload does not include the null byte terminator */
+		size_t body_len = strlen(body);
 
 		/* Process as T.140 text (moved here from ast_sendtext() */
 		memset(&f, 0, sizeof(f));
@@ -5199,6 +4973,12 @@ static void apply_plc(struct ast_channel
 
 int ast_write(struct ast_channel *chan, struct ast_frame *fr)
 {
+	return ast_write_stream(chan, -1, fr);
+}
+
+int ast_write_stream(struct ast_channel *chan, int stream_num, struct ast_frame *fr)
+{
+	struct ast_stream *stream = NULL, *default_stream = NULL;
 	int res = -1;
 	struct ast_frame *f = NULL;
 	int count = 0;
@@ -5213,18 +4993,44 @@ int ast_write(struct ast_channel *chan,
 		}
 		usleep(1);
 	}
+
 	/* Stop if we're a zombie or need a soft hangup */
-	if (ast_test_flag(ast_channel_flags(chan), AST_FLAG_ZOMBIE) || ast_check_hangup(chan))
+	if (ast_test_flag(ast_channel_flags(chan), AST_FLAG_ZOMBIE) || ast_check_hangup(chan)) {
 		goto done;
+	}
+
+	if (stream_num >= 0) {
+		/* If we were told to write to an explicit stream then allow this frame through, no matter
+		 * if the type is expected or not (a framehook could change)
+		 */
+		if (stream_num >= ast_stream_topology_get_count(ast_channel_get_stream_topology(chan))) {
+			goto done;
+		}
+		stream = ast_stream_topology_get_stream(ast_channel_get_stream_topology(chan), stream_num);
+		default_stream = ast_channel_get_default_stream(chan, ast_stream_get_type(stream));
+	} else if (fr->frametype == AST_FRAME_VOICE || fr->frametype == AST_FRAME_VIDEO || fr->frametype == AST_FRAME_MODEM) {
+		/* If we haven't been told of a stream then we need to figure out which once we need */
+		enum ast_media_type type = AST_MEDIA_TYPE_UNKNOWN;
+
+		/* Some frame types have a fixed media type */
+		if (fr->frametype == AST_FRAME_VOICE || fr->frametype == AST_FRAME_VIDEO) {
+			type = ast_format_get_type(fr->subclass.format);
+		} else if (fr->frametype == AST_FRAME_MODEM) {
+			type = AST_MEDIA_TYPE_IMAGE;
+		}
+
+		/* No stream was specified, so use the default one */
+		stream = default_stream = ast_channel_get_default_stream(chan, type);
+	}
 
 	/* Perform the framehook write event here. After the frame enters the framehook list
 	 * there is no telling what will happen, how awesome is that!!! */
-	if (!(fr = ast_framehook_list_write_event(ast_channel_framehooks(chan), fr))) {
+	if ((stream == default_stream) && !(fr = ast_framehook_list_write_event(ast_channel_framehooks(chan), fr))) {
 		res = 0;
 		goto done;
 	}
 
-	if (ast_channel_generatordata(chan) && (!fr->src || strcasecmp(fr->src, "ast_prod"))) {
+	if (ast_channel_generatordata(chan) && (fr->frametype != AST_FRAME_RTCP) && (!fr->src || strcasecmp(fr->src, "ast_prod"))) {
 		if (ast_test_flag(ast_channel_flags(chan), AST_FLAG_WRITE_INT)) {
 				ast_deactivate_generator(chan);
 		} else {
@@ -5303,20 +5109,35 @@ int ast_write(struct ast_channel *chan,
 	case AST_FRAME_VIDEO:
 		/* XXX Handle translation of video codecs one day XXX */
 		CHECK_BLOCKING(chan);
-		res = (ast_channel_tech(chan)->write_video == NULL) ? 0 :
-			ast_channel_tech(chan)->write_video(chan, fr);
+		if (ast_channel_tech(chan)->write_stream) {
+			if (stream) {
+				res = ast_channel_tech(chan)->write_stream(chan, ast_stream_get_position(stream), fr);
+			} else {
+				res = 0;
+			}
+		} else if ((stream == default_stream) && ast_channel_tech(chan)->write_video) {
+			res = ast_channel_tech(chan)->write_video(chan, fr);
+		} else {
+			res = 0;
+		}
 		ast_clear_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING);
 		break;
 	case AST_FRAME_MODEM:
 		CHECK_BLOCKING(chan);
-		res = (ast_channel_tech(chan)->write == NULL) ? 0 :
-			ast_channel_tech(chan)->write(chan, fr);
+		if (ast_channel_tech(chan)->write_stream) {
+			if (stream) {
+				res = ast_channel_tech(chan)->write_stream(chan, ast_stream_get_position(stream), fr);
+			} else {
+				res = 0;
+			}
+		} else if ((stream == default_stream) && ast_channel_tech(chan)->write) {
+			res = ast_channel_tech(chan)->write(chan, fr);
+		} else {
+			res = 0;
+		}
 		ast_clear_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING);
 		break;
 	case AST_FRAME_VOICE:
-		if (ast_channel_tech(chan)->write == NULL)
-			break;	/*! \todo XXX should return 0 maybe ? */
-
 		if (ast_opt_generic_plc && ast_format_cmp(fr->subclass.format, ast_format_slin) == AST_FORMAT_CMP_EQUAL) {
 			apply_plc(chan, fr);
 		}
@@ -5327,7 +5148,7 @@ int ast_write(struct ast_channel *chan,
 		 * Send frame to audiohooks if present, if frametype is linear (else, later as per
 		 * previous behavior)
 		 */
-		if (ast_channel_audiohooks(chan)) {
+		if ((stream == default_stream) && ast_channel_audiohooks(chan)) {
 			if (ast_format_cache_is_slinear(fr->subclass.format)) {
 				hooked = 1;
 				f = ast_audiohook_write_list(chan, ast_channel_audiohooks(chan), AST_AUDIOHOOK_DIRECTION_WRITE, fr);
@@ -5335,7 +5156,7 @@ int ast_write(struct ast_channel *chan,
 		}
 
 		/* If the frame is in the raw write format, then it's easy... just use the frame - otherwise we will have to translate */
-		if (ast_format_cmp(fr->subclass.format, ast_channel_rawwriteformat(chan)) != AST_FORMAT_CMP_EQUAL) {
+		if ((stream == default_stream) && ast_format_cmp(fr->subclass.format, ast_channel_rawwriteformat(chan)) != AST_FORMAT_CMP_EQUAL) {
 			if (ast_format_cmp(ast_channel_writeformat(chan), fr->subclass.format) != AST_FORMAT_CMP_EQUAL) {
 				struct ast_str *codec_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
 
@@ -5364,7 +5185,6 @@ int ast_write(struct ast_channel *chan,
 
 			if (ast_channel_writetrans(chan)) {
 				struct ast_frame *trans_frame = ast_translate(ast_channel_writetrans(chan), f, 0);
-
 				if (trans_frame != f && f != fr) {
 					/*
 					 * If translate gives us a new frame and so did the audio
@@ -5374,7 +5194,6 @@ int ast_write(struct ast_channel *chan,
 				}
 				f = trans_frame;
 			}
-
 		}
 
 		if (!f) {
@@ -5382,7 +5201,7 @@ int ast_write(struct ast_channel *chan,
 			break;
 		}
 
-		if (ast_channel_audiohooks(chan) && !hooked) {
+		if ((stream == default_stream) && ast_channel_audiohooks(chan) && !hooked) {
 			struct ast_frame *prev = NULL, *new_frame, *cur, *dup;
 			int freeoldlist = 0;
 
@@ -5431,7 +5250,7 @@ int ast_write(struct ast_channel *chan,
 		/* the translator on chan->writetrans may have returned multiple frames
 		   from the single frame we passed in; if so, feed each one of them to the
 		   monitor */
-		if (ast_channel_monitor(chan) && ast_channel_monitor(chan)->write_stream) {
+		if ((stream == default_stream) && ast_channel_monitor(chan) && ast_channel_monitor(chan)->write_stream) {
 			struct ast_frame *cur;
 
 			for (cur = f; cur; cur = AST_LIST_NEXT(cur, frame_list)) {
@@ -5499,7 +5318,17 @@ int ast_write(struct ast_channel *chan,
 			/* reset f so the code below doesn't attempt to free it */
 			f = NULL;
 		} else {
-			res = ast_channel_tech(chan)->write(chan, f);
+			if (ast_channel_tech(chan)->write_stream) {
+				if (stream) {
+					res = ast_channel_tech(chan)->write_stream(chan, ast_stream_get_position(stream), f);
+				} else {
+					res = 0;
+				}
+			} else if ((stream == default_stream) && ast_channel_tech(chan)->write) {
+				res = ast_channel_tech(chan)->write(chan, f);
+			} else {
+				res = 0;
+			}
 		}
 		ast_clear_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING);
 		break;
@@ -5508,6 +5337,16 @@ int ast_write(struct ast_channel *chan,
 		/* Ignore these */
 		res = 0;
 		break;
+	case AST_FRAME_RTCP:
+		/* RTCP information is on a per-stream basis and only available on multistream capable channels */
+		CHECK_BLOCKING(chan);
+		if (ast_channel_tech(chan)->write_stream && stream) {
+			res = ast_channel_tech(chan)->write_stream(chan, ast_stream_get_position(stream), fr);
+		} else {
+			res = 0;
+		}
+		ast_clear_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING);
+		break;
 	default:
 		/* At this point, fr is the incoming frame and f is NULL.  Channels do
 		 * not expect to get NULL as a frame pointer and will segfault.  Hence,
@@ -5642,7 +5481,7 @@ static const struct set_format_access se
 	.setoption = AST_OPTION_FORMAT_WRITE,
 };
 
-static int set_format(struct ast_channel *chan, struct ast_format_cap *cap_set, const int direction)
+static int set_format(struct ast_channel *chan, struct ast_format_cap *cap_set, const int direction, int interleaved_stereo)
 {
 	struct ast_trans_pvt *trans_pvt;
 	struct ast_format_cap *cap_native;
@@ -5744,16 +5583,20 @@ static int set_format(struct ast_channel
 	}
 
 	/* Now we have a good choice for both. */
+	trans_pvt = access->get_trans(chan);
 	if ((ast_format_cmp(rawformat, best_native_fmt) != AST_FORMAT_CMP_NOT_EQUAL) &&
 		(ast_format_cmp(format, best_set_fmt) != AST_FORMAT_CMP_NOT_EQUAL) &&
 		((ast_format_cmp(rawformat, format) != AST_FORMAT_CMP_NOT_EQUAL) || access->get_trans(chan))) {
-		/* the channel is already in these formats, so nothing to do */
-		ast_channel_unlock(chan);
-		return 0;
+		/* the channel is already in these formats, so nothing to do, unless the interleaved format is not set correctly */
+		if (trans_pvt != NULL) {
+			if (trans_pvt->interleaved_stereo == interleaved_stereo) {
+				ast_channel_unlock(chan);
+				return 0;
+			}
+		}
 	}
 
 	/* Free any translation we have right now */
-	trans_pvt = access->get_trans(chan);
 	if (trans_pvt) {
 		ast_translator_free_path(trans_pvt);
 		access->set_trans(chan, NULL);
@@ -5771,9 +5614,11 @@ static int set_format(struct ast_channel
 		if (!direction) {
 			/* reading */
 			trans_pvt = ast_translator_build_path(best_set_fmt, best_native_fmt);
+			trans_pvt->interleaved_stereo = 0;
 		} else {
 			/* writing */
 			trans_pvt = ast_translator_build_path(best_native_fmt, best_set_fmt);
+			trans_pvt->interleaved_stereo = interleaved_stereo;
 		}
 		access->set_trans(chan, trans_pvt);
 		res = trans_pvt ? 0 : -1;
@@ -5813,7 +5658,7 @@ int ast_set_read_format(struct ast_chann
 	}
 	ast_format_cap_append(cap, format, 0);
 
-	res = set_format(chan, cap, 0);
+	res = set_format(chan, cap, 0, 0);
 
 	ao2_cleanup(cap);
 	return res;
@@ -5821,7 +5666,25 @@ int ast_set_read_format(struct ast_chann
 
 int ast_set_read_format_from_cap(struct ast_channel *chan, struct ast_format_cap *cap)
 {
-	return set_format(chan, cap, 0);
+	return set_format(chan, cap, 0, 0);
+}
+
+int ast_set_write_format_interleaved_stereo(struct ast_channel *chan, struct ast_format *format)
+{
+	struct ast_format_cap *cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	int res;
+
+	ast_assert(format != NULL);
+
+	if (!cap) {
+		return -1;
+	}
+	ast_format_cap_append(cap, format, 0);
+
+	res = set_format(chan, cap, 1, 1);
+
+	ao2_cleanup(cap);
+	return res;
 }
 
 int ast_set_write_format(struct ast_channel *chan, struct ast_format *format)
@@ -5836,7 +5699,7 @@ int ast_set_write_format(struct ast_chan
 	}
 	ast_format_cap_append(cap, format, 0);
 
-	res = set_format(chan, cap, 1);
+	res = set_format(chan, cap, 1, 0);
 
 	ao2_cleanup(cap);
 	return res;
@@ -5844,7 +5707,7 @@ int ast_set_write_format(struct ast_chan
 
 int ast_set_write_format_from_cap(struct ast_channel *chan, struct ast_format_cap *cap)
 {
-	return set_format(chan, cap, 1);
+	return set_format(chan, cap, 1, 0);
 }
 
 const char *ast_channel_reason2str(int reason)
@@ -6243,10 +6106,11 @@ static int set_security_requirements(con
 	return 0;
 }
 
-struct ast_channel *ast_request(const char *type, struct ast_format_cap *request_cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause)
+static struct ast_channel *request_channel(const char *type, struct ast_format_cap *request_cap, struct ast_stream_topology *topology,
+	const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause)
 {
 	struct chanlist *chan;
-	struct ast_channel *c;
+	struct ast_channel *c = NULL;
 	int res;
 	int foo;
 
@@ -6260,13 +6124,44 @@ struct ast_channel *ast_request(const ch
 	}
 
 	AST_RWLIST_TRAVERSE(&backends, chan, list) {
+		if (strcasecmp(type, chan->tech->type)) {
+			continue;
+		}
+
+		break;
+	}
+
+	AST_RWLIST_UNLOCK(&backends);
+
+	if (!chan) {
+		ast_log(LOG_WARNING, "No channel type registered for '%s'\n", type);
+		*cause = AST_CAUSE_NOSUCHDRIVER;
+		return NULL;
+	}
+
+	/* Allow either format capabilities or stream topology to be provided and adapt */
+	if (chan->tech->requester_with_stream_topology) {
+		struct ast_stream_topology *tmp_converted_topology = NULL;
+
+		if (!topology && request_cap) {
+			/* Turn the requested capabilities into a stream topology */
+			topology = tmp_converted_topology = ast_stream_topology_create_from_format_cap(request_cap);
+		}
+
+		c = chan->tech->requester_with_stream_topology(type, topology, assignedids, requestor, addr, cause);
+
+		ast_stream_topology_free(tmp_converted_topology);
+	} else if (chan->tech->requester) {
+		struct ast_format_cap *tmp_converted_cap = NULL;
 		struct ast_format_cap *tmp_cap;
 		RAII_VAR(struct ast_format *, tmp_fmt, NULL, ao2_cleanup);
 		RAII_VAR(struct ast_format *, best_audio_fmt, NULL, ao2_cleanup);
 		struct ast_format_cap *joint_cap;
 
-		if (strcasecmp(type, chan->tech->type))
-			continue;
+		if (!request_cap && topology) {
+			/* Turn the request stream topology into capabilities */
+			request_cap = tmp_converted_cap = ast_format_cap_from_stream_topology(topology);
+		}
 
 		/* find the best audio format to use */
 		tmp_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
@@ -6285,13 +6180,10 @@ struct ast_channel *ast_request(const ch
 					ast_format_cap_get_names(chan->tech->capabilities, &tech_codecs),
 					ast_format_cap_get_names(request_cap, &request_codecs));
 				*cause = AST_CAUSE_BEARERCAPABILITY_NOTAVAIL;
-				AST_RWLIST_UNLOCK(&backends);
+				ao2_cleanup(tmp_converted_cap);
 				return NULL;
 			}
 		}
-		AST_RWLIST_UNLOCK(&backends);
-		if (!chan->tech->requester)
-			return NULL;
 
 		/* XXX Only the audio format calculated as being the best for translation
 		 * purposes is used for the request. This is because we don't have the ability
@@ -6300,51 +6192,56 @@ struct ast_channel *ast_request(const ch
 		 */
 		joint_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
 		if (!joint_cap) {
+			ao2_cleanup(tmp_converted_cap);
 			return NULL;
 		}
 		ast_format_cap_append_from_cap(joint_cap, request_cap, AST_MEDIA_TYPE_UNKNOWN);
 		ast_format_cap_remove_by_type(joint_cap, AST_MEDIA_TYPE_AUDIO);
 		ast_format_cap_append(joint_cap, best_audio_fmt, 0);
+		ao2_cleanup(tmp_converted_cap);
 
-		if (!(c = chan->tech->requester(type, joint_cap, assignedids, requestor, addr, cause))) {
-			ao2_ref(joint_cap, -1);
-			return NULL;
-		}
+		c = chan->tech->requester(type, joint_cap, assignedids, requestor, addr, cause);
+		ao2_ref(joint_cap, -1);
+	}
 
-		if (requestor) {
-			struct ast_callid *callid;
+	if (!c) {
+		return NULL;
+	}
 
-			ast_channel_lock_both(c, (struct ast_channel *) requestor);
+	if (requestor) {
+		ast_callid callid;
 
-			/* Set the newly created channel's callid to the same as the requestor. */
-			callid = ast_channel_callid(requestor);
-			if (callid) {
-				ast_channel_callid_set(c, callid);
-				callid = ast_callid_unref(callid);
-			}
+		ast_channel_lock_both(c, (struct ast_channel *) requestor);
 
-			ast_channel_unlock(c);
-			ast_channel_unlock((struct ast_channel *) requestor);
+		/* Set the newly created channel's callid to the same as the requestor. */
+		callid = ast_channel_callid(requestor);
+		if (callid) {
+			ast_channel_callid_set(c, callid);
 		}
 
-		ao2_ref(joint_cap, -1);
-
-		if (set_security_requirements(requestor, c)) {
-			ast_log(LOG_WARNING, "Setting security requirements failed\n");
-			ast_hangup(c);
-			*cause = AST_CAUSE_BEARERCAPABILITY_NOTAVAIL;
-			return NULL;
-		}
+		ast_channel_unlock(c);
+		ast_channel_unlock((struct ast_channel *) requestor);
+	}
 
-		/* no need to generate a Newchannel event here; it is done in the channel_alloc call */
-		return c;
+	if (set_security_requirements(requestor, c)) {
+		ast_log(LOG_WARNING, "Setting security requirements failed\n");
+		ast_hangup(c);
+		*cause = AST_CAUSE_BEARERCAPABILITY_NOTAVAIL;
+		return NULL;
 	}
 
-	ast_log(LOG_WARNING, "No channel type registered for '%s'\n", type);
-	*cause = AST_CAUSE_NOSUCHDRIVER;
-	AST_RWLIST_UNLOCK(&backends);
+	/* no need to generate a Newchannel event here; it is done in the channel_alloc call */
+	return c;
+}
 
-	return NULL;
+struct ast_channel *ast_request(const char *type, struct ast_format_cap *request_cap, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause)
+{
+	return request_channel(type, request_cap, NULL, assignedids, requestor, addr, cause);
+}
+
+struct ast_channel *ast_request_with_stream_topology(const char *type, struct ast_stream_topology *topology, const struct ast_assigned_ids *assignedids, const struct ast_channel *requestor, const char *addr, int *cause)
+{
+	return request_channel(type, NULL, topology, assignedids, requestor, addr, cause);
 }
 
 /*!
@@ -6545,11 +6442,11 @@ int ast_readstring_full(struct ast_chann
 				silgen = ast_channel_start_silence_generator(c);
 			usleep(1000);
 			if (!d)
-				d = ast_waitfordigit_full(c, to, audiofd, ctrlfd);
+				d = ast_waitfordigit_full(c, to, NULL, audiofd, ctrlfd);
 		} else {
 			if (!silgen && ast_opt_transmit_silence)
 				silgen = ast_channel_start_silence_generator(c);
-			d = ast_waitfordigit_full(c, to, audiofd, ctrlfd);
+			d = ast_waitfordigit_full(c, to, NULL, audiofd, ctrlfd);
 		}
 		if (d < 0) {
 			ast_channel_stop_silence_generator(c, silgen);
@@ -6822,6 +6719,7 @@ static void channel_do_masquerade(struct
 	int origstate;
 	unsigned int orig_disablestatecache;
 	unsigned int clone_disablestatecache;
+	int generator_fd;
 	int visible_indication;
 	int clone_hold_state;
 	int moh_is_playing;
@@ -7012,8 +6910,13 @@ static void channel_do_masquerade(struct
 	/* Keep the same parkinglot. */
 	ast_channel_parkinglot_set(original, ast_channel_parkinglot(clonechan));
 
-	/* Copy the FD's other than the generator fd */
-	for (x = 0; x < AST_MAX_FDS; x++) {
+	/* Clear all existing file descriptors but retain the generator */
+	generator_fd = ast_channel_fd(original, AST_GENERATOR_FD);
+	ast_channel_internal_fd_clear_all(original);
+	ast_channel_set_fd(original, AST_GENERATOR_FD, generator_fd);
+
+	/* Copy all file descriptors present on clonechan to original, skipping generator */
+	for (x = 0; x < ast_channel_fd_count(clonechan); x++) {
 		if (x != AST_GENERATOR_FD)
 			ast_channel_set_fd(original, x, ast_channel_fd(clonechan, x));
 	}
@@ -7159,6 +7062,8 @@ static void channel_do_masquerade(struct
 			ast_channel_tech(clonechan)->type, ast_channel_name(clonechan));
 	}
 
+	ast_channel_internal_swap_stream_topology(original, clonechan);
+
 	/*
 	 * Now, at this point, the "clone" channel is totally F'd up.
 	 * We mark it as a zombie so nothing tries to touch it.
@@ -7631,8 +7536,8 @@ struct ast_namedgroups *ast_get_namedgro
 		return NULL;
 	}
 
-	namedgroups = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 19,
-		namedgroup_hash_cb, namedgroup_cmp_cb);
+	namedgroups = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, 19,
+		namedgroup_hash_cb, NULL, namedgroup_cmp_cb);
 	if (!namedgroups) {
 		return NULL;
 	}
@@ -7741,146 +7646,6 @@ static int ast_channel_hash_cb(const voi
 	return ast_str_case_hash(name);
 }
 
-int ast_plc_reload(void)
-{
-	struct ast_variable *var;
-	struct ast_flags config_flags = { 0 };
-	struct ast_config *cfg = ast_config_load("codecs.conf", config_flags);
-	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID)
-		return 0;
-	for (var = ast_variable_browse(cfg, "plc"); var; var = var->next) {
-		if (!strcasecmp(var->name, "genericplc")) {
-			ast_set2_flag(&ast_options, ast_true(var->value), AST_OPT_FLAG_GENERIC_PLC);
-		} else if (!strcasecmp(var->name, "genericplc_on_equal_codecs")) {
-			ast_set2_flag(&ast_options, ast_true(var->value), AST_OPT_FLAG_GENERIC_PLC_ON_EQUAL_CODECS);
-		}
-	}
-
-	/*
-	 * Force on_equal_codecs to false if generic_plc is false.
-	 */
-	if (!ast_opt_generic_plc) {
-		ast_set2_flag(&ast_options, 0, AST_OPT_FLAG_GENERIC_PLC_ON_EQUAL_CODECS);
-	}
-	ast_config_destroy(cfg);
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Implements the channels provider.
- */
-static int data_channels_provider_handler(const struct ast_data_search *search,
-	struct ast_data *root)
-{
-	struct ast_channel *c;
-	struct ast_channel_iterator *iter = NULL;
-	struct ast_data *data_channel;
-
-	for (iter = ast_channel_iterator_all_new();
-		iter && (c = ast_channel_iterator_next(iter)); ast_channel_unref(c)) {
-		ast_channel_lock(c);
-
-		data_channel = ast_data_add_node(root, "channel");
-		if (!data_channel) {
-			ast_channel_unlock(c);
-			continue;
-		}
-
-		if (ast_channel_data_add_structure(data_channel, c, 1) < 0) {
-			ast_log(LOG_ERROR, "Unable to add channel structure for channel: %s\n", ast_channel_name(c));
-		}
-
-		ast_channel_unlock(c);
-
-		if (!ast_data_search_match(search, data_channel)) {
-			ast_data_remove_node(root, data_channel);
-		}
-	}
-	if (iter) {
-		ast_channel_iterator_destroy(iter);
-	}
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Implements the channeltypes provider.
- */
-static int data_channeltypes_provider_handler(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct chanlist *cl;
-	struct ast_data *data_type;
-
-	AST_RWLIST_RDLOCK(&backends);
-	AST_RWLIST_TRAVERSE(&backends, cl, list) {
-		data_type = ast_data_add_node(data_root, "type");
-		if (!data_type) {
-			continue;
-		}
-		ast_data_add_str(data_type, "name", cl->tech->type);
-		ast_data_add_str(data_type, "description", cl->tech->description);
-		ast_data_add_bool(data_type, "devicestate", cl->tech->devicestate ? 1 : 0);
-		ast_data_add_bool(data_type, "indications", cl->tech->indicate ? 1 : 0);
-		ast_data_add_bool(data_type, "transfer", cl->tech->transfer ? 1 : 0);
-		ast_data_add_bool(data_type, "send_digit_begin", cl->tech->send_digit_begin ? 1 : 0);
-		ast_data_add_bool(data_type, "send_digit_end", cl->tech->send_digit_end ? 1 : 0);
-		ast_data_add_bool(data_type, "call", cl->tech->call ? 1 : 0);
-		ast_data_add_bool(data_type, "hangup", cl->tech->hangup ? 1 : 0);
-		ast_data_add_bool(data_type, "answer", cl->tech->answer ? 1 : 0);
-		ast_data_add_bool(data_type, "read", cl->tech->read ? 1 : 0);
-		ast_data_add_bool(data_type, "write", cl->tech->write ? 1 : 0);
-		ast_data_add_bool(data_type, "send_text", cl->tech->send_text ? 1 : 0);
-		ast_data_add_bool(data_type, "send_image", cl->tech->send_image ? 1 : 0);
-		ast_data_add_bool(data_type, "send_html", cl->tech->send_html ? 1 : 0);
-		ast_data_add_bool(data_type, "exception", cl->tech->exception ? 1 : 0);
-		ast_data_add_bool(data_type, "early_bridge", cl->tech->early_bridge ? 1 : 0);
-		ast_data_add_bool(data_type, "fixup", cl->tech->fixup ? 1 : 0);
-		ast_data_add_bool(data_type, "setoption", cl->tech->setoption ? 1 : 0);
-		ast_data_add_bool(data_type, "queryoption", cl->tech->queryoption ? 1 : 0);
-		ast_data_add_bool(data_type, "write_video", cl->tech->write_video ? 1 : 0);
-		ast_data_add_bool(data_type, "write_text", cl->tech->write_text ? 1 : 0);
-		ast_data_add_bool(data_type, "func_channel_read", cl->tech->func_channel_read ? 1 : 0);
-		ast_data_add_bool(data_type, "func_channel_write", cl->tech->func_channel_write ? 1 : 0);
-		ast_data_add_bool(data_type, "get_pvt_uniqueid", cl->tech->get_pvt_uniqueid ? 1 : 0);
-		ast_data_add_bool(data_type, "cc_callback", cl->tech->cc_callback ? 1 : 0);
-
-		ast_data_add_codecs(data_type, "capabilities", cl->tech->capabilities);
-
-		if (!ast_data_search_match(search, data_type)) {
-			ast_data_remove_node(data_root, data_type);
-		}
-	}
-	AST_RWLIST_UNLOCK(&backends);
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief /asterisk/core/channels provider.
- */
-static const struct ast_data_handler channels_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = data_channels_provider_handler
-};
-
-/*!
- * \internal
- * \brief /asterisk/core/channeltypes provider.
- */
-static const struct ast_data_handler channeltypes_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = data_channeltypes_provider_handler
-};
-
-static const struct ast_data_entry channel_providers[] = {
-	AST_DATA_ENTRY("/asterisk/core/channels", &channels_provider),
-	AST_DATA_ENTRY("/asterisk/core/channeltypes", &channeltypes_provider),
-};
-
 /*!
  * \internal
  * \brief Print channel object key (name).
@@ -7911,35 +7676,48 @@ struct manager_channel_variable {
 	char name[];
 };
 
-static AST_RWLIST_HEAD_STATIC(channelvars, manager_channel_variable);
+AST_RWLIST_HEAD(external_vars, manager_channel_variable);
 
-static void free_channelvars(void)
+static struct external_vars ami_vars;
+static struct external_vars ari_vars;
+
+static void free_external_channelvars(struct external_vars *channelvars)
 {
 	struct manager_channel_variable *var;
-	AST_RWLIST_WRLOCK(&channelvars);
-	while ((var = AST_RWLIST_REMOVE_HEAD(&channelvars, entry))) {
+	AST_RWLIST_WRLOCK(channelvars);
+	while ((var = AST_RWLIST_REMOVE_HEAD(channelvars, entry))) {
 		ast_free(var);
 	}
-	AST_RWLIST_UNLOCK(&channelvars);
+	AST_RWLIST_UNLOCK(channelvars);
 }
 
-int ast_channel_has_manager_vars(void)
+static int channel_has_external_vars(struct external_vars *channelvars)
 {
 	int vars_present;
 
-	AST_RWLIST_RDLOCK(&channelvars);
-	vars_present = !AST_LIST_EMPTY(&channelvars);
-	AST_RWLIST_UNLOCK(&channelvars);
+	AST_RWLIST_RDLOCK(channelvars);
+	vars_present = !AST_LIST_EMPTY(channelvars);
+	AST_RWLIST_UNLOCK(channelvars);
 
 	return vars_present;
 }
 
-void ast_channel_set_manager_vars(size_t varc, char **vars)
+int ast_channel_has_manager_vars(void)
+{
+	return channel_has_external_vars(&ami_vars);
+}
+
+int ast_channel_has_ari_vars(void)
+{
+	return channel_has_external_vars(&ari_vars);
+}
+
+static void channel_set_external_vars(struct external_vars *channelvars, size_t varc, char **vars)
 {
 	size_t i;
 
-	free_channelvars();
-	AST_RWLIST_WRLOCK(&channelvars);
+	free_external_channelvars(channelvars);
+	AST_RWLIST_WRLOCK(channelvars);
 	for (i = 0; i < varc; ++i) {
 		const char *var = vars[i];
 		struct manager_channel_variable *mcv;
@@ -7950,9 +7728,20 @@ void ast_channel_set_manager_vars(size_t
 		if (strchr(var, '(')) {
 			mcv->isfunc = 1;
 		}
-		AST_RWLIST_INSERT_TAIL(&channelvars, mcv, entry);
+		AST_RWLIST_INSERT_TAIL(channelvars, mcv, entry);
 	}
-	AST_RWLIST_UNLOCK(&channelvars);
+	AST_RWLIST_UNLOCK(channelvars);
+
+}
+
+void ast_channel_set_manager_vars(size_t varc, char **vars)
+{
+	channel_set_external_vars(&ami_vars, varc, vars);
+}
+
+void ast_channel_set_ari_vars(size_t varc, char **vars)
+{
+	channel_set_external_vars(&ari_vars, varc, vars);
 }
 
 /*!
@@ -7994,14 +7783,15 @@ struct varshead *ast_channel_get_vars(st
 	return ret;
 }
 
-struct varshead *ast_channel_get_manager_vars(struct ast_channel *chan)
+static struct varshead *channel_get_external_vars(struct external_vars *channelvars,
+	struct ast_channel *chan)
 {
 	RAII_VAR(struct varshead *, ret, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_str *, tmp, NULL, ast_free);
 	struct manager_channel_variable *mcv;
-	SCOPED_LOCK(lock, &channelvars, AST_RWLIST_RDLOCK, AST_RWLIST_UNLOCK);
+	SCOPED_LOCK(lock, channelvars, AST_RWLIST_RDLOCK, AST_RWLIST_UNLOCK);
 
-	if (AST_LIST_EMPTY(&channelvars)) {
+	if (AST_LIST_EMPTY(channelvars)) {
 		return NULL;
 	}
 
@@ -8012,7 +7802,7 @@ struct varshead *ast_channel_get_manager
 		return NULL;
 	}
 
-	AST_LIST_TRAVERSE(&channelvars, mcv, entry) {
+	AST_LIST_TRAVERSE(channelvars, mcv, entry) {
 		const char *val = NULL;
 		struct ast_var_t *var;
 
@@ -8037,13 +7827,24 @@ struct varshead *ast_channel_get_manager
 
 	ao2_ref(ret, +1);
 	return ret;
+
+}
+
+struct varshead *ast_channel_get_manager_vars(struct ast_channel *chan)
+{
+	return channel_get_external_vars(&ami_vars, chan);
+}
+
+struct varshead *ast_channel_get_ari_vars(struct ast_channel *chan)
+{
+	return channel_get_external_vars(&ari_vars, chan);
 }
 
 static void channels_shutdown(void)
 {
-	free_channelvars();
+	free_external_channelvars(&ami_vars);
+	free_external_channelvars(&ari_vars);
 
-	ast_data_unregister(NULL);
 	ast_cli_unregister_multiple(cli_channel, ARRAY_LEN(cli_channel));
 	if (channels) {
 		ao2_container_unregister("channels");
@@ -8055,8 +7856,8 @@ static void channels_shutdown(void)
 
 int ast_channels_init(void)
 {
-	channels = ao2_container_alloc(NUM_CHANNEL_BUCKETS,
-			ast_channel_hash_cb, ast_channel_cmp_cb);
+	channels = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NUM_CHANNEL_BUCKETS,
+		ast_channel_hash_cb, NULL, ast_channel_cmp_cb);
 	if (!channels) {
 		return -1;
 	}
@@ -8068,12 +7869,11 @@ int ast_channels_init(void)
 
 	ast_cli_register_multiple(cli_channel, ARRAY_LEN(cli_channel));
 
-	ast_data_register_multiple_core(channel_providers, ARRAY_LEN(channel_providers));
-
-	ast_plc_reload();
-
 	ast_register_cleanup(channels_shutdown);
 
+	AST_RWLIST_HEAD_INIT(&ami_vars);
+	AST_RWLIST_HEAD_INIT(&ari_vars);
+
 	return 0;
 }
 
@@ -8274,24 +8074,6 @@ void ast_channel_stop_silence_generator(
 }
 
 
-/*! \ brief Convert channel reloadreason (ENUM) to text string for manager event */
-const char *channelreloadreason2txt(enum channelreloadreason reason)
-{
-	switch (reason) {
-	case CHANNEL_MODULE_LOAD:
-		return "LOAD (Channel module load)";
-
-	case CHANNEL_MODULE_RELOAD:
-		return "RELOAD (Channel module reload)";
-
-	case CHANNEL_CLI_RELOAD:
-		return "CLIRELOAD (Channel module reload by CLI command)";
-
-	default:
-		return "MANAGERRELOAD (Channel module reload by manager)";
-	}
-};
-
 /*
  * Wrappers for various ast_say_*() functions that call the full version
  * of the same functions.
@@ -11114,6 +10896,46 @@ enum ast_channel_error ast_channel_errno
 	return ast_channel_internal_errno();
 }
 
+int ast_channel_request_stream_topology_change(struct ast_channel *chan,
+		struct ast_stream_topology *topology, void *change_source)
+{
+	int res;
+
+	ast_assert(chan != NULL);
+	ast_assert(topology != NULL);
+
+	ast_channel_lock(chan);
+	if (!ast_channel_is_multistream(chan) || !ast_channel_tech(chan)->indicate) {
+		ast_channel_unlock(chan);
+		return -1;
+	}
+
+	if (ast_stream_topology_equal(ast_channel_get_stream_topology(chan), topology)) {
+		ast_debug(3, "Topology of %s already matches what is requested so ignoring topology change request\n",
+				ast_channel_name(chan));
+		ast_channel_unlock(chan);
+		return 0;
+	}
+
+	ast_channel_internal_set_stream_topology_change_source(chan, change_source);
+
+	res = ast_channel_tech(chan)->indicate(chan, AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE, topology, sizeof(topology));
+	ast_channel_unlock(chan);
+	return res;
+}
+
+int ast_channel_stream_topology_changed(struct ast_channel *chan, struct ast_stream_topology *topology)
+{
+	ast_assert(chan != NULL);
+	ast_assert(topology != NULL);
+
+	if (!ast_channel_is_multistream(chan) || !ast_channel_tech(chan)->indicate) {
+		return -1;
+	}
+
+	return ast_channel_tech(chan)->indicate(chan, AST_CONTROL_STREAM_TOPOLOGY_CHANGED, topology, sizeof(topology));
+}
+
 void ast_channel_set_flag(struct ast_channel *chan, unsigned int flag)
 {
 	ast_channel_lock(chan);
diff -pruN 1:13.23.1~dfsg-2/main/channel_internal_api.c 1:16.1.1~dfsg-1/main/channel_internal_api.c
--- 1:13.23.1~dfsg-2/main/channel_internal_api.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/channel_internal_api.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <unistd.h>
 #include <fcntl.h>
 
@@ -42,14 +40,15 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/paths.h"
 #include "asterisk/channel.h"
 #include "asterisk/channel_internal.h"
-#include "asterisk/data.h"
 #include "asterisk/endpoints.h"
 #include "asterisk/indications.h"
 #include "asterisk/stasis_cache_pattern.h"
 #include "asterisk/stasis_channels.h"
 #include "asterisk/stasis_endpoints.h"
 #include "asterisk/stringfields.h"
+#include "asterisk/stream.h"
 #include "asterisk/test.h"
+#include "asterisk/vector.h"
 
 /*!
  * \brief Channel UniqueId structure
@@ -97,10 +96,7 @@ struct ast_channel {
 	struct ast_tone_zone *zone;			/*!< Tone zone as set in indications.conf or
 							 *   in the CHANNEL dialplan function */
 	struct ast_channel_monitor *monitor;		/*!< Channel monitoring */
-	struct ast_callid *callid;			/*!< Bound call identifier pointer */
-#ifdef HAVE_EPOLL
-	struct ast_epoll_data *epfd_data[AST_MAX_FDS];
-#endif
+	ast_callid callid;			/*!< Bound call identifier pointer */
 	struct ao2_container *dialed_causes;		/*!< Contains tech-specific and Asterisk cause data from dialed channels */
 
 	AST_DECLARE_STRING_FIELDS(
@@ -170,7 +166,7 @@ struct ast_channel {
 	unsigned long outsmpl;				/*!< Track the read/written samples for monitor use */
 
 	int blocker_tid;					/*!< If anyone is blocking, this is their thread id */
-	int fds[AST_MAX_FDS];				/*!< File descriptors for channel -- Drivers will poll on
+	AST_VECTOR(, int) fds;				/*!< File descriptors for channel -- Drivers will poll on
 							 *   these file descriptors, so at least one must be non -1.
 							 *   See \arg \ref AstFileDesc */
 	int softhangup;				/*!< Whether or not we have been hung up...  Do not set this value
@@ -200,9 +196,6 @@ struct ast_channel {
 	struct ast_format *rawreadformat;         /*!< Raw read format (before translation) */
 	struct ast_format *rawwriteformat;        /*!< Raw write format (after translation) */
 	unsigned int emulate_dtmf_duration;		/*!< Number of ms left to emulate DTMF for */
-#ifdef HAVE_EPOLL
-	int epfd;
-#endif
 	int visible_indication;                         /*!< Indication currently playing on the channel */
 	int hold_state;							/*!< Current Hold/Unhold state */
 
@@ -225,216 +218,14 @@ struct ast_channel {
 	struct stasis_cp_single *topics;		/*!< Topic for all channel's events */
 	struct stasis_forward *endpoint_forward;	/*!< Subscription for event forwarding to endpoint's topic */
 	struct stasis_forward *endpoint_cache_forward; /*!< Subscription for cache updates to endpoint's topic */
+	struct ast_stream_topology *stream_topology; /*!< Stream topology */
+	void *stream_topology_change_source; /*!< Source that initiated a stream topology change */
+	struct ast_stream *default_streams[AST_MEDIA_TYPE_END]; /*!< Default streams indexed by media type */
 };
 
 /*! \brief The monotonically increasing integer counter for channel uniqueids */
 static int uniqueint;
 
-/* AST_DATA definitions, which will probably have to be re-thought since the channel will be opaque */
-
-#if 0	/* XXX AstData: ast_callerid no longer exists. (Equivalent code not readily apparent.) */
-#define DATA_EXPORT_CALLERID(MEMBER)				\
-	MEMBER(ast_callerid, cid_dnid, AST_DATA_STRING)		\
-	MEMBER(ast_callerid, cid_num, AST_DATA_STRING)		\
-	MEMBER(ast_callerid, cid_name, AST_DATA_STRING)		\
-	MEMBER(ast_callerid, cid_ani, AST_DATA_STRING)		\
-	MEMBER(ast_callerid, cid_pres, AST_DATA_INTEGER)	\
-	MEMBER(ast_callerid, cid_ani2, AST_DATA_INTEGER)	\
-	MEMBER(ast_callerid, cid_tag, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(ast_callerid, DATA_EXPORT_CALLERID);
-#endif
-
-#define DATA_EXPORT_CHANNEL(MEMBER)						\
-	MEMBER(ast_channel, blockproc, AST_DATA_STRING)				\
-	MEMBER(ast_channel, appl, AST_DATA_STRING)				\
-	MEMBER(ast_channel, data, AST_DATA_STRING)				\
-	MEMBER(ast_channel, name, AST_DATA_STRING) \
-	MEMBER(ast_channel, language, AST_DATA_STRING)				\
-	MEMBER(ast_channel, musicclass, AST_DATA_STRING)			\
-	MEMBER(ast_channel, accountcode, AST_DATA_STRING)			\
-	MEMBER(ast_channel, peeraccount, AST_DATA_STRING)			\
-	MEMBER(ast_channel, userfield, AST_DATA_STRING)				\
-	MEMBER(ast_channel, call_forward, AST_DATA_STRING)			\
-	MEMBER(ast_channel, parkinglot, AST_DATA_STRING)			\
-	MEMBER(ast_channel, hangupsource, AST_DATA_STRING)			\
-	MEMBER(ast_channel, dialcontext, AST_DATA_STRING)			\
-	MEMBER(ast_channel, rings, AST_DATA_INTEGER)				\
-	MEMBER(ast_channel, priority, AST_DATA_INTEGER)				\
-	MEMBER(ast_channel, macropriority, AST_DATA_INTEGER)			\
-	MEMBER(ast_channel, adsicpe, AST_DATA_INTEGER)				\
-	MEMBER(ast_channel, fin, AST_DATA_UNSIGNED_INTEGER)			\
-	MEMBER(ast_channel, fout, AST_DATA_UNSIGNED_INTEGER)			\
-	MEMBER(ast_channel, emulate_dtmf_duration, AST_DATA_UNSIGNED_INTEGER)	\
-	MEMBER(ast_channel, visible_indication, AST_DATA_INTEGER)		\
-	MEMBER(ast_channel, context, AST_DATA_STRING)				\
-	MEMBER(ast_channel, exten, AST_DATA_STRING)				\
-	MEMBER(ast_channel, macrocontext, AST_DATA_STRING)			\
-	MEMBER(ast_channel, macroexten, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(ast_channel, DATA_EXPORT_CHANNEL);
-
-static void channel_data_add_flags(struct ast_data *tree,
-	struct ast_channel *chan)
-{
-	ast_data_add_bool(tree, "DEFER_DTMF", ast_test_flag(ast_channel_flags(chan), AST_FLAG_DEFER_DTMF));
-	ast_data_add_bool(tree, "WRITE_INT", ast_test_flag(ast_channel_flags(chan), AST_FLAG_WRITE_INT));
-	ast_data_add_bool(tree, "BLOCKING", ast_test_flag(ast_channel_flags(chan), AST_FLAG_BLOCKING));
-	ast_data_add_bool(tree, "ZOMBIE", ast_test_flag(ast_channel_flags(chan), AST_FLAG_ZOMBIE));
-	ast_data_add_bool(tree, "EXCEPTION", ast_test_flag(ast_channel_flags(chan), AST_FLAG_EXCEPTION));
-	ast_data_add_bool(tree, "MOH", ast_test_flag(ast_channel_flags(chan), AST_FLAG_MOH));
-	ast_data_add_bool(tree, "SPYING", ast_test_flag(ast_channel_flags(chan), AST_FLAG_SPYING));
-	ast_data_add_bool(tree, "IN_AUTOLOOP", ast_test_flag(ast_channel_flags(chan), AST_FLAG_IN_AUTOLOOP));
-	ast_data_add_bool(tree, "OUTGOING", ast_test_flag(ast_channel_flags(chan), AST_FLAG_OUTGOING));
-	ast_data_add_bool(tree, "IN_DTMF", ast_test_flag(ast_channel_flags(chan), AST_FLAG_IN_DTMF));
-	ast_data_add_bool(tree, "EMULATE_DTMF", ast_test_flag(ast_channel_flags(chan), AST_FLAG_EMULATE_DTMF));
-	ast_data_add_bool(tree, "END_DTMF_ONLY", ast_test_flag(ast_channel_flags(chan), AST_FLAG_END_DTMF_ONLY));
-	ast_data_add_bool(tree, "MASQ_NOSTREAM", ast_test_flag(ast_channel_flags(chan), AST_FLAG_MASQ_NOSTREAM));
-	ast_data_add_bool(tree, "BRIDGE_HANGUP_RUN", ast_test_flag(ast_channel_flags(chan), AST_FLAG_BRIDGE_HANGUP_RUN));
-	ast_data_add_bool(tree, "DISABLE_WORKAROUNDS", ast_test_flag(ast_channel_flags(chan), AST_FLAG_DISABLE_WORKAROUNDS));
-	ast_data_add_bool(tree, "DISABLE_DEVSTATE_CACHE", ast_test_flag(ast_channel_flags(chan), AST_FLAG_DISABLE_DEVSTATE_CACHE));
-	ast_data_add_bool(tree, "BRIDGE_DUAL_REDIRECT_WAIT", ast_test_flag(ast_channel_flags(chan), AST_FLAG_BRIDGE_DUAL_REDIRECT_WAIT));
-	ast_data_add_bool(tree, "ORIGINATED", ast_test_flag(ast_channel_flags(chan), AST_FLAG_ORIGINATED));
-	ast_data_add_bool(tree, "DEAD", ast_test_flag(ast_channel_flags(chan), AST_FLAG_DEAD));
-}
-
-int ast_channel_data_add_structure(struct ast_data *tree,
-	struct ast_channel *chan, int add_bridged)
-{
-	struct ast_data *data_bridged;
-	struct ast_data *data_cdr;
-	struct ast_data *data_flags;
-	struct ast_data *data_zones;
-	struct ast_data *enum_node;
-	struct ast_data *data_softhangup;
-#if 0	/* XXX AstData: ast_callerid no longer exists. (Equivalent code not readily apparent.) */
-	struct ast_data *data_callerid;
-	char value_str[100];
-#endif
-
-	if (!tree) {
-		return -1;
-	}
-
-	ast_data_add_structure(ast_channel, tree, chan);
-
-	if (add_bridged) {
-		RAII_VAR(struct ast_channel *, bc, ast_channel_bridge_peer(chan), ast_channel_cleanup);
-		if (bc) {
-			data_bridged = ast_data_add_node(tree, "bridged");
-			if (!data_bridged) {
-				return -1;
-			}
-			ast_channel_data_add_structure(data_bridged, bc, 0);
-		}
-	}
-
-	ast_data_add_str(tree, "uniqueid", ast_channel_uniqueid(chan));
-	ast_data_add_str(tree, "linkedid", ast_channel_linkedid(chan));
-
-	ast_data_add_codec(tree, "oldwriteformat", ast_channel_oldwriteformat(chan));
-	ast_data_add_codec(tree, "readformat", ast_channel_readformat(chan));
-	ast_data_add_codec(tree, "writeformat", ast_channel_writeformat(chan));
-	ast_data_add_codec(tree, "rawreadformat", ast_channel_rawreadformat(chan));
-	ast_data_add_codec(tree, "rawwriteformat", ast_channel_rawwriteformat(chan));
-	ast_data_add_codecs(tree, "nativeformats", ast_channel_nativeformats(chan));
-
-	/* state */
-	enum_node = ast_data_add_node(tree, "state");
-	if (!enum_node) {
-		return -1;
-	}
-	ast_data_add_str(enum_node, "text", ast_state2str(ast_channel_state(chan)));
-	ast_data_add_int(enum_node, "value", ast_channel_state(chan));
-
-	/* hangupcause */
-	enum_node = ast_data_add_node(tree, "hangupcause");
-	if (!enum_node) {
-		return -1;
-	}
-	ast_data_add_str(enum_node, "text", ast_cause2str(ast_channel_hangupcause(chan)));
-	ast_data_add_int(enum_node, "value", ast_channel_hangupcause(chan));
-
-	/* amaflags */
-	enum_node = ast_data_add_node(tree, "amaflags");
-	if (!enum_node) {
-		return -1;
-	}
-	ast_data_add_str(enum_node, "text", ast_channel_amaflags2string(ast_channel_amaflags(chan)));
-	ast_data_add_int(enum_node, "value", ast_channel_amaflags(chan));
-
-	/* transfercapability */
-	enum_node = ast_data_add_node(tree, "transfercapability");
-	if (!enum_node) {
-		return -1;
-	}
-	ast_data_add_str(enum_node, "text", ast_transfercapability2str(ast_channel_transfercapability(chan)));
-	ast_data_add_int(enum_node, "value", ast_channel_transfercapability(chan));
-
-	/* _softphangup */
-	data_softhangup = ast_data_add_node(tree, "softhangup");
-	if (!data_softhangup) {
-		return -1;
-	}
-	ast_data_add_bool(data_softhangup, "dev", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_DEV);
-	ast_data_add_bool(data_softhangup, "asyncgoto", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_ASYNCGOTO);
-	ast_data_add_bool(data_softhangup, "shutdown", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_SHUTDOWN);
-	ast_data_add_bool(data_softhangup, "timeout", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_TIMEOUT);
-	ast_data_add_bool(data_softhangup, "appunload", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_APPUNLOAD);
-	ast_data_add_bool(data_softhangup, "explicit", ast_channel_softhangup_internal_flag(chan) & AST_SOFTHANGUP_EXPLICIT);
-
-	/* channel flags */
-	data_flags = ast_data_add_node(tree, "flags");
-	if (!data_flags) {
-		return -1;
-	}
-	channel_data_add_flags(data_flags, chan);
-
-	ast_data_add_uint(tree, "timetohangup", ast_channel_whentohangup(chan)->tv_sec);
-
-#if 0	/* XXX AstData: ast_callerid no longer exists. (Equivalent code not readily apparent.) */
-	/* callerid */
-	data_callerid = ast_data_add_node(tree, "callerid");
-	if (!data_callerid) {
-		return -1;
-	}
-	ast_data_add_structure(ast_callerid, data_callerid, &(chan->cid));
-	/* insert the callerid ton */
-	enum_node = ast_data_add_node(data_callerid, "cid_ton");
-	if (!enum_node) {
-		return -1;
-	}
-	ast_data_add_int(enum_node, "value", chan->cid.cid_ton);
-	snprintf(value_str, sizeof(value_str), "TON: %s/Plan: %s",
-		party_number_ton2str(chan->cid.cid_ton),
-		party_number_plan2str(chan->cid.cid_ton));
-	ast_data_add_str(enum_node, "text", value_str);
-#endif
-
-	/* tone zone */
-	if (ast_channel_zone(chan)) {
-		data_zones = ast_data_add_node(tree, "zone");
-		if (!data_zones) {
-			return -1;
-		}
-		ast_tone_zone_data_add_structure(data_zones, ast_channel_zone(chan));
-	}
-
-	/* insert cdr */
-	data_cdr = ast_data_add_node(tree, "cdr");
-	if (!data_cdr) {
-		return -1;
-	}
-
-	return 0;
-}
-
-int ast_channel_data_cmp_structure(const struct ast_data_search *tree,
-	struct ast_channel *chan, const char *structure_name)
-{
-	return ast_data_search_cmp_structure(tree, ast_channel, chan, structure_name);
-}
-
 /* ACCESSORS */
 
 #define DEFINE_STRINGFIELD_SETTERS_FOR(field, publish, assert_on_null) \
@@ -598,17 +389,6 @@ void ast_channel_amaflags_set(struct ast
 	chan->amaflags = value;
 	ast_channel_publish_snapshot(chan);
 }
-
-#ifdef HAVE_EPOLL
-int ast_channel_epfd(const struct ast_channel *chan)
-{
-	return chan->epfd;
-}
-void ast_channel_epfd_set(struct ast_channel *chan, int value)
-{
-	chan->epfd = value;
-}
-#endif
 int ast_channel_fdno(const struct ast_channel *chan)
 {
 	return chan->fdno;
@@ -829,10 +609,64 @@ struct ast_format_cap *ast_channel_nativ
 {
 	return chan->nativeformats;
 }
-void ast_channel_nativeformats_set(struct ast_channel *chan, struct ast_format_cap *value)
+
+static void channel_set_default_streams(struct ast_channel *chan)
+{
+	enum ast_media_type type;
+
+	ast_assert(chan != NULL);
+
+	for (type = AST_MEDIA_TYPE_UNKNOWN; type < AST_MEDIA_TYPE_END; type++) {
+		if (chan->stream_topology) {
+			chan->default_streams[type] =
+				ast_stream_topology_get_first_stream_by_type(chan->stream_topology, type);
+		} else {
+			chan->default_streams[type] = NULL;
+		}
+	}
+}
+
+void ast_channel_internal_set_stream_topology(struct ast_channel *chan,
+	struct ast_stream_topology *topology)
+{
+	ast_stream_topology_free(chan->stream_topology);
+	chan->stream_topology = topology;
+	channel_set_default_streams(chan);
+}
+
+void ast_channel_internal_set_stream_topology_change_source(
+	struct ast_channel *chan, void *change_source)
+{
+	chan->stream_topology_change_source = change_source;
+}
+
+void *ast_channel_get_stream_topology_change_source(struct ast_channel *chan)
+{
+	return chan->stream_topology_change_source;
+}
+
+void ast_channel_nativeformats_set(struct ast_channel *chan,
+	struct ast_format_cap *value)
 {
+	ast_assert(chan != NULL);
+
 	ao2_replace(chan->nativeformats, value);
+
+	/* If chan->stream_topology is NULL, the channel is being destroyed
+	 * and topology is destroyed.
+	 */
+	if (!chan->stream_topology) {
+		return;
+	}
+
+	if (!ast_channel_is_multistream(chan) || !value) {
+		struct ast_stream_topology *new_topology;
+
+		new_topology = ast_stream_topology_create_from_format_cap(value);
+		ast_channel_internal_set_stream_topology(chan, new_topology);
+	}
 }
+
 struct ast_framehook_list *ast_channel_framehooks(const struct ast_channel *chan)
 {
 	return chan->framehooks;
@@ -903,6 +737,10 @@ const struct ast_channel_tech *ast_chann
 }
 void ast_channel_tech_set(struct ast_channel *chan, const struct ast_channel_tech *value)
 {
+	if (value->read_stream || value->write_stream) {
+		ast_assert(value->read_stream && value->write_stream);
+	}
+
 	chan->tech = value;
 }
 enum ast_channel_adsicpe ast_channel_adsicpe(const struct ast_channel *chan)
@@ -917,15 +755,11 @@ enum ast_channel_state ast_channel_state
 {
 	return chan->state;
 }
-struct ast_callid *ast_channel_callid(const struct ast_channel *chan)
+ast_callid ast_channel_callid(const struct ast_channel *chan)
 {
-	if (chan->callid) {
-		ast_callid_ref(chan->callid);
-		return chan->callid;
-	}
-	return NULL;
+	return chan->callid;
 }
-void ast_channel_callid_set(struct ast_channel *chan, struct ast_callid *callid)
+void ast_channel_callid_set(struct ast_channel *chan, ast_callid callid)
 {
 	char call_identifier_from[AST_CALLID_BUFFER_LENGTH];
 	char call_identifier_to[AST_CALLID_BUFFER_LENGTH];
@@ -934,11 +768,9 @@ void ast_channel_callid_set(struct ast_c
 	if (chan->callid) {
 		ast_callid_strnprint(call_identifier_from, sizeof(call_identifier_from), chan->callid);
 		ast_debug(3, "Channel Call ID changing from %s to %s\n", call_identifier_from, call_identifier_to);
-		/* unbind if already set */
-		ast_callid_unref(chan->callid);
 	}
 
-	chan->callid = ast_callid_ref(callid);
+	chan->callid = callid;
 
 	ast_test_suite_event_notify("CallIDChange",
 		"State: CallIDChange\r\n"
@@ -1195,9 +1027,7 @@ void ast_channel_set_is_t38_active(struc
 
 void ast_channel_callid_cleanup(struct ast_channel *chan)
 {
-	if (chan->callid) {
-		chan->callid = ast_callid_unref(chan->callid);
-	}
+	chan->callid = 0;
 }
 
 /* Typedef accessors */
@@ -1290,38 +1120,55 @@ void ast_channel_internal_alertpipe_swap
 /* file descriptor array accessors */
 void ast_channel_internal_fd_set(struct ast_channel *chan, int which, int value)
 {
-	chan->fds[which] = value;
+	int pos;
+
+	/* This ensures that if the vector has to grow with unused positions they will be
+	 * initialized to -1.
+	 */
+	for (pos = AST_VECTOR_SIZE(&chan->fds); pos < which; pos++) {
+		AST_VECTOR_REPLACE(&chan->fds, pos, -1);
+	}
+
+	AST_VECTOR_REPLACE(&chan->fds, which, value);
 }
 void ast_channel_internal_fd_clear(struct ast_channel *chan, int which)
 {
-	ast_channel_internal_fd_set(chan, which, -1);
+	if (which >= AST_VECTOR_SIZE(&chan->fds)) {
+		return;
+	}
+
+	AST_VECTOR_REPLACE(&chan->fds, which, -1);
 }
 void ast_channel_internal_fd_clear_all(struct ast_channel *chan)
 {
-	int i;
-	for (i = 0; i < AST_MAX_FDS; i++) {
-		ast_channel_internal_fd_clear(chan, i);
-	}
+	AST_VECTOR_RESET(&chan->fds, AST_VECTOR_ELEM_CLEANUP_NOOP);
 }
 int ast_channel_fd(const struct ast_channel *chan, int which)
 {
-	return chan->fds[which];
+	return (which >= AST_VECTOR_SIZE(&chan->fds)) ? -1 : AST_VECTOR_GET(&chan->fds, which);
 }
 int ast_channel_fd_isset(const struct ast_channel *chan, int which)
 {
 	return ast_channel_fd(chan, which) > -1;
 }
 
-#ifdef HAVE_EPOLL
-struct ast_epoll_data *ast_channel_internal_epfd_data(const struct ast_channel *chan, int which)
+int ast_channel_fd_count(const struct ast_channel *chan)
 {
-	return chan->epfd_data[which];
+	return AST_VECTOR_SIZE(&chan->fds);
 }
-void ast_channel_internal_epfd_data_set(struct ast_channel *chan, int which , struct ast_epoll_data *value)
+
+int ast_channel_fd_add(struct ast_channel *chan, int value)
 {
-	chan->epfd_data[which] = value;
+	int pos = AST_EXTENDED_FDS;
+
+	while (ast_channel_fd_isset(chan, pos)) {
+		pos += 1;
+	}
+
+	AST_VECTOR_REPLACE(&chan->fds, pos, value);
+
+	return pos;
 }
-#endif
 
 pthread_t ast_channel_blocker(const struct ast_channel *chan)
 {
@@ -1444,15 +1291,8 @@ struct ast_channel *__ast_channel_intern
 {
 	struct ast_channel *tmp;
 
-#if defined(REF_DEBUG)
-	tmp = __ao2_alloc_debug(sizeof(*tmp), destructor,
-		AO2_ALLOC_OPT_LOCK_MUTEX, "", file, line, function, 1);
-#elif defined(__AST_DEBUG_MALLOC)
-	tmp = __ao2_alloc_debug(sizeof(*tmp), destructor,
-		AO2_ALLOC_OPT_LOCK_MUTEX, "", file, line, function, 0);
-#else
-	tmp = ao2_alloc(sizeof(*tmp), destructor);
-#endif
+	tmp = __ao2_alloc(sizeof(*tmp), destructor,
+		AO2_ALLOC_OPT_LOCK_MUTEX, "", file, line, function);
 
 	if (!tmp) {
 		return NULL;
@@ -1462,7 +1302,9 @@ struct ast_channel *__ast_channel_intern
 		return ast_channel_unref(tmp);
 	}
 
-	if (!(tmp->dialed_causes = ao2_container_alloc(DIALED_CAUSES_BUCKETS, pvt_cause_hash_fn, pvt_cause_cmp_fn))) {
+	tmp->dialed_causes = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DIALED_CAUSES_BUCKETS, pvt_cause_hash_fn, NULL, pvt_cause_cmp_fn);
+	if (!tmp->dialed_causes) {
 		return ast_channel_unref(tmp);
 	}
 
@@ -1491,6 +1333,8 @@ struct ast_channel *__ast_channel_intern
 		tmp->linkedid = tmp->uniqueid;
 	}
 
+	AST_VECTOR_INIT(&tmp->fds, AST_MAX_FDS);
+
 	return tmp;
 }
 
@@ -1565,6 +1409,10 @@ void ast_channel_internal_cleanup(struct
 
 	stasis_cp_single_unsubscribe(chan->topics);
 	chan->topics = NULL;
+
+	ast_channel_internal_set_stream_topology(chan, NULL);
+
+	AST_VECTOR_FREE(&chan->fds);
 }
 
 void ast_channel_internal_finalize(struct ast_channel *chan)
@@ -1657,3 +1505,66 @@ enum ast_channel_error ast_channel_inter
 
 	return *error_code;
 }
+
+struct ast_stream_topology *ast_channel_get_stream_topology(
+	const struct ast_channel *chan)
+{
+	ast_assert(chan != NULL);
+
+	return chan->stream_topology;
+}
+
+struct ast_stream_topology *ast_channel_set_stream_topology(struct ast_channel *chan,
+	struct ast_stream_topology *topology)
+{
+	struct ast_stream_topology *new_topology;
+
+	ast_assert(chan != NULL);
+
+	/* A non-MULTISTREAM channel can't manipulate topology directly */
+	ast_assert(ast_channel_is_multistream(chan));
+
+	/* Unless the channel is being destroyed, we always want a topology on
+	 * it even if its empty.
+	 */
+	if (!topology) {
+		new_topology = ast_stream_topology_alloc();
+	} else {
+		new_topology = topology;
+	}
+
+	if (new_topology) {
+		ast_channel_internal_set_stream_topology(chan, new_topology);
+	}
+
+	return new_topology;
+}
+
+struct ast_stream *ast_channel_get_default_stream(struct ast_channel *chan,
+	enum ast_media_type type)
+{
+	ast_assert(chan != NULL);
+	ast_assert(type < AST_MEDIA_TYPE_END);
+
+	return chan->default_streams[type];
+}
+
+void ast_channel_internal_swap_stream_topology(struct ast_channel *chan1,
+	struct ast_channel *chan2)
+{
+	struct ast_stream_topology *tmp_topology;
+
+	ast_assert(chan1 != NULL && chan2 != NULL);
+
+	tmp_topology = chan1->stream_topology;
+	chan1->stream_topology = chan2->stream_topology;
+	chan2->stream_topology = tmp_topology;
+
+	channel_set_default_streams(chan1);
+	channel_set_default_streams(chan2);
+}
+
+int ast_channel_is_multistream(struct ast_channel *chan)
+{
+	return (chan->tech && chan->tech->read_stream && chan->tech->write_stream);
+}
diff -pruN 1:13.23.1~dfsg-2/main/chanvars.c 1:16.1.1~dfsg-1/main/chanvars.c
--- 1:13.23.1~dfsg-2/main/chanvars.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/chanvars.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,27 +29,19 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/chanvars.h"
 #include "asterisk/strings.h"
 #include "asterisk/utils.h"
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_var_t *_ast_var_assign(const char *name, const char *value, const char *file, int lineno, const char *function)
-#else
-struct ast_var_t *ast_var_assign(const char *name, const char *value)
-#endif
 {
 	struct ast_var_t *var;
 	int name_len = strlen(name) + 1;
 	int value_len = strlen(value) + 1;
 
-#ifdef __AST_DEBUG_MALLOC
-	if (!(var = __ast_calloc(sizeof(*var) + name_len + value_len, sizeof(char), file, lineno, function))) {
-#else
-	if (!(var = ast_calloc(sizeof(*var) + name_len + value_len, sizeof(char)))) {
-#endif
+	var = __ast_calloc(sizeof(*var) + name_len + value_len, sizeof(char),
+		file, lineno, function);
+	if (!var) {
 		return NULL;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/main/cli.c 1:16.1.1~dfsg-1/main/cli.c
--- 1:13.23.1~dfsg-2/main/cli.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/cli.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,11 +38,8 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_MODULE_DIR */
-#include <sys/signal.h>
 #include <signal.h>
 #include <ctype.h>
 #include <regex.h>
@@ -63,6 +60,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/stasis_channels.h"
 #include "asterisk/stasis_bridges.h"
 #include "asterisk/vector.h"
+#include "asterisk/stream.h"
 
 /*!
  * \brief List of restrictions per user.
@@ -150,11 +148,6 @@ unsigned int ast_debug_get_by_module(con
 	return res;
 }
 
-unsigned int ast_verbose_get_by_module(const char *module)
-{
-	return 0;
-}
-
 /*! \internal
  *  \brief Check if the user with 'uid' and 'gid' is allow to execute 'command',
  *	   if command starts with '_' then not check permissions, just permit
@@ -778,7 +771,7 @@ static void print_uptimestr(int fd, stru
 		return;
 
 	if (printsec)  {	/* plain seconds output */
-		ast_cli(fd, "%s: %lu\n", prefix, (u_long)timeval.tv_sec);
+		ast_cli(fd, "%s%lu\n", prefix, (u_long)timeval.tv_sec);
 		return;
 	}
 	out = ast_str_alloca(256);
@@ -812,7 +805,7 @@ static void print_uptimestr(int fd, stru
 		/* if there is nothing, print 0 seconds */
 		ast_str_append(&out, 0, "%d second%s", x, ESS(x));
 	}
-	ast_cli(fd, "%s: %s\n", prefix, ast_str_buffer(out));
+	ast_cli(fd, "%s%s\n", prefix, ast_str_buffer(out));
 }
 
 static struct ast_cli_entry *cli_next(struct ast_cli_entry *e)
@@ -848,10 +841,12 @@ static char * handle_showuptime(struct a
 		printsec = 0;
 	else
 		return CLI_SHOWUSAGE;
-	if (ast_startuptime.tv_sec)
-		print_uptimestr(a->fd, ast_tvsub(curtime, ast_startuptime), "System uptime", printsec);
-	if (ast_lastreloadtime.tv_sec)
-		print_uptimestr(a->fd, ast_tvsub(curtime, ast_lastreloadtime), "Last reload", printsec);
+	if (ast_startuptime.tv_sec) {
+		print_uptimestr(a->fd, ast_tvsub(curtime, ast_startuptime), "System uptime: ", printsec);
+	}
+	if (ast_lastreloadtime.tv_sec) {
+		print_uptimestr(a->fd, ast_tvsub(curtime, ast_lastreloadtime), "Last reload: ", printsec);
+	}
 	return CLI_SUCCESS;
 }
 
@@ -945,7 +940,7 @@ static char *handle_showcalls(struct ast
 	ast_cli(a->fd, "%d call%s processed\n", ast_processed_calls(), ESS(ast_processed_calls()));
 
 	if (ast_startuptime.tv_sec && showuptime) {
-		print_uptimestr(a->fd, ast_tvsub(curtime, ast_startuptime), "System uptime", printsec);
+		print_uptimestr(a->fd, ast_tvsub(curtime, ast_startuptime), "System uptime: ", printsec);
 	}
 
 	return RESULT_SUCCESS;
@@ -1336,33 +1331,6 @@ static char *handle_commandmatchesarray(
 }
 
 
-
-static char *handle_commandnummatches(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
-{
-	int matches = 0;
-
-	switch (cmd) {
-	case CLI_INIT:
-		e->command = "_command nummatches";
-		e->usage =
-			"Usage: _command nummatches \"<line>\" text \n"
-			"       This function is used internally to help with command completion and should.\n"
-			"       never be called by the user directly.\n";
-		return NULL;
-	case CLI_GENERATE:
-		return NULL;
-	}
-
-	if (a->argc != 4)
-		return CLI_SHOWUSAGE;
-
-	matches = ast_cli_generatornummatches(a->argv[2], a->argv[3]);
-
-	ast_cli(a->fd, "%d", matches);
-
-	return CLI_SUCCESS;
-}
-
 struct channel_set_debug_args {
 	int fd;
 	int is_off;
@@ -1499,8 +1467,9 @@ static char *handle_showchan(struct ast_
 	struct ast_str *read_transpath = ast_str_alloca(256);
 	struct ast_str *codec_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
 	struct ast_bridge *bridge;
-	struct ast_callid *callid;
+	ast_callid callid;
 	char callid_buf[32];
+	int stream_num;
 
 	switch (cmd) {
 	case CLI_INIT:
@@ -1557,7 +1526,6 @@ static char *handle_showchan(struct ast_
 	callid = ast_channel_callid(chan);
 	if (callid) {
 		ast_callid_strnprint(callid_buf, sizeof(callid_buf), callid);
-		ast_callid_unref(callid);
 	}
 
 	ast_str_append(&output, 0,
@@ -1631,6 +1599,7 @@ static char *handle_showchan(struct ast_
 		S_OR(ast_channel_data(chan), "(Empty)"),
 		S_OR(callid_buf, "(None)")
 		);
+
 	ast_str_append(&output, 0, "      Variables:\n");
 
 	AST_LIST_TRAVERSE(ast_channel_varshead(chan), var, entries) {
@@ -1642,6 +1611,34 @@ static char *handle_showchan(struct ast_
 		ast_str_append(&output, 0, "  CDR Variables:\n%s\n", ast_str_buffer(obuf));
 	}
 
+	ast_str_append(&output, 0, " -- Streams --\n");
+	for (stream_num = 0; stream_num < ast_stream_topology_get_count(ast_channel_get_stream_topology(chan)); stream_num++) {
+		struct ast_stream *stream = ast_stream_topology_get_stream(ast_channel_get_stream_topology(chan), stream_num);
+		struct ast_variable *metadata = ast_stream_get_metadata_list(stream);
+
+		ast_str_append(&output, 0,
+			"Name: %s\n"
+			"    Type: %s\n"
+			"    State: %s\n"
+			"    Group: %d\n"
+			"    Formats: %s\n"
+			"    Metadata:\n",
+			ast_stream_get_name(stream),
+			ast_codec_media_type2str(ast_stream_get_type(stream)),
+			ast_stream_state2str(ast_stream_get_state(stream)),
+			ast_stream_get_group(stream),
+			ast_format_cap_get_names(ast_stream_get_formats(stream), &codec_buf)
+			);
+
+		if (metadata) {
+			struct ast_variable *v;
+			for(v = metadata; v; v = v->next) {
+				ast_str_append(&output, 0, "        %s: %s\n", v->name, v->value);
+			}
+			ast_variables_destroy(metadata);
+		}
+	}
+
 	ast_channel_unlock(chan);
 
 	ast_cli(a->fd, "%s", ast_str_buffer(output));
@@ -1791,10 +1788,50 @@ static char *handle_cli_wait_fullybooted
 	return CLI_SUCCESS;
 }
 
+
+#ifdef HAVE_MALLOC_TRIM
+
+/*!
+ * \internal
+ * \brief Attempt to reclaim unused heap memory.
+ *
+ * Users have reported that asterisk will sometimes be killed because it can't allocate
+ * more than around 3G of memory on a 32 bit system.
+ *
+ * Using malloc_trim() will help us to determine if it's because there's a leak or because
+ * the heap is so fragmented that there isn't enough contiguous memory available.
+ *
+ * \note malloc_trim() is a GNU extension and is therefore not portable.
+ */
+static char *handle_cli_malloc_trim(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	extern int malloc_trim(size_t __pad) __THROW;
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "malloc trim";
+		e->usage =
+			"Usage: malloc trim\n"
+			"       Try to give excess memory back to the OS.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	if (malloc_trim(0)) {
+		ast_cli(a->fd, "Returned some memory to the OS.\n");
+	} else {
+		ast_cli(a->fd, "No memory returned to the OS.\n");
+	}
+
+	return CLI_SUCCESS;
+}
+
+#endif
+
 static char *handle_help(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a);
 
 static struct ast_cli_entry cli_cli[] = {
-	AST_CLI_DEFINE(handle_commandnummatches, "Returns number of command matches"),
 	AST_CLI_DEFINE(handle_commandmatchesarray, "Returns command matches array"),
 
 	AST_CLI_DEFINE(handle_nodebugchan_deprecated, "Disable debugging on channel(s)"),
@@ -1837,6 +1874,11 @@ static struct ast_cli_entry cli_cli[] =
 	AST_CLI_DEFINE(handle_cli_check_permissions, "Try a permissions config for a user"),
 
 	AST_CLI_DEFINE(handle_cli_wait_fullybooted, "Wait for Asterisk to be fully booted"),
+
+#ifdef HAVE_MALLOC_TRIM
+	AST_CLI_DEFINE(handle_cli_malloc_trim, "Return excess memory to the OS"),
+#endif
+
 };
 
 /*!
@@ -2199,7 +2241,7 @@ static void remove_shutdown_command(stru
 	ast_rwlock_unlock(&shutdown_commands_lock);
 }
 
-static int __ast_cli_unregister(struct ast_cli_entry *e, struct ast_cli_entry *ed)
+int ast_cli_unregister(struct ast_cli_entry *e)
 {
 	if (e->inuse) {
 		ast_log(LOG_WARNING, "Can't remove command that is in use\n");
@@ -2222,7 +2264,7 @@ static int __ast_cli_unregister(struct a
 	return 0;
 }
 
-static int __ast_cli_register(struct ast_cli_entry *e, struct ast_cli_entry *ed)
+int __ast_cli_register(struct ast_cli_entry *e, struct ast_module *module)
 {
 	struct ast_cli_entry *cur;
 	int i, lf, ret = -1;
@@ -2241,7 +2283,11 @@ static int __ast_cli_register(struct ast
 	}
 
 	memset(&a, '\0', sizeof(a));
+
+	e->module = module;
+	/* No module reference needed here, the module called us. */
 	e->handler(e, CLI_INIT, &a);
+
 	/* XXX check that usage and command are filled up */
 	s = ast_skip_blanks(e->command);
 	s = e->command = ast_strdup(s);
@@ -2292,27 +2338,16 @@ done:
 	return ret;
 }
 
-/* wrapper function, so we can unregister deprecated commands recursively */
-int ast_cli_unregister(struct ast_cli_entry *e)
-{
-	return __ast_cli_unregister(e, NULL);
-}
-
-/* wrapper function, so we can register deprecated commands recursively */
-int ast_cli_register(struct ast_cli_entry *e)
-{
-	return __ast_cli_register(e, NULL);
-}
-
 /*
  * register/unregister an array of entries.
  */
-int ast_cli_register_multiple(struct ast_cli_entry *e, int len)
+int __ast_cli_register_multiple(struct ast_cli_entry *e, int len, struct ast_module *module)
 {
 	int i, res = 0;
 
-	for (i = 0; i < len; i++)
-		res |= ast_cli_register(e + i);
+	for (i = 0; i < len; i++) {
+		res |= __ast_cli_register(e + i, module);
+	}
 
 	return res;
 }
@@ -2479,23 +2514,6 @@ static char *parse_args(const char *s, i
 	return duplicate;
 }
 
-/*! \brief Return the number of unique matches for the generator */
-int ast_cli_generatornummatches(const char *text, const char *word)
-{
-	int matches;
-	struct ast_vector_string *vec = ast_cli_completion_vector(text, word);
-
-	if (!vec) {
-		return 0;
-	}
-
-	matches = AST_VECTOR_SIZE(vec) - 1;
-	AST_VECTOR_CALLBACK_VOID(vec, ast_free);
-	AST_VECTOR_PTR_FREE(vec);
-
-	return matches;
-}
-
 char **ast_cli_completion_matches(const char *text, const char *word)
 {
 	struct ast_vector_string *vec = ast_cli_completion_vector(text, word);
@@ -2726,7 +2744,12 @@ static char *__ast_cli_generator(const c
 					.n = state - matchnum,
 					.argv = argv,
 					.argc = x};
-				ret = e->handler(e, CLI_GENERATE, &a);
+
+				/* If the command is in a module it must be running. */
+				if (!e->module || ast_module_running_ref(e->module)) {
+					ret = e->handler(e, CLI_GENERATE, &a);
+					ast_module_unref(e->module);
+				}
 			}
 			if (ret)
 				break;
@@ -2767,12 +2790,12 @@ int ast_cli_command_full(int uid, int gi
 	int x;
 	char *duplicate = parse_args(s, &x, args + 1, AST_MAX_ARGS, NULL);
 	char tmp[AST_MAX_ARGS + 1];
-	char *retval = NULL;
+	char *retval = CLI_FAILURE;
 	struct ast_cli_args a = {
 		.fd = fd, .argc = x, .argv = args+1 };
 
 	if (duplicate == NULL)
-		return -1;
+		return RESULT_FAILURE;
 
 	if (x < 1)	/* We need at least one entry, otherwise ignore */
 		goto done;
@@ -2805,13 +2828,16 @@ int ast_cli_command_full(int uid, int gi
 	 */
 	args[0] = (char *)e;
 
-	retval = e->handler(e, CLI_HANDLER, &a);
+	/* If the command is in a module it must be running. */
+	if (!e->module || ast_module_running_ref(e->module)) {
+		retval = e->handler(e, CLI_HANDLER, &a);
+		ast_module_unref(e->module);
+	}
 
 	if (retval == CLI_SHOWUSAGE) {
 		ast_cli(fd, "%s", S_OR(e->usage, "Invalid usage, but no usage information available.\n"));
-	} else {
-		if (retval == CLI_FAILURE)
-			ast_cli(fd, "Command '%s' failed.\n", s);
+	} else if (retval == CLI_FAILURE) {
+		ast_cli(fd, "Command '%s' failed.\n", s);
 	}
 
 done:
@@ -2819,7 +2845,7 @@ done:
 		ast_atomic_fetchadd_int(&e->inuse, -1);
 	}
 	ast_free(duplicate);
-	return 0;
+	return retval == CLI_SUCCESS ? RESULT_SUCCESS : RESULT_FAILURE;
 }
 
 int ast_cli_command_multiple_full(int uid, int gid, int fd, size_t size, const char *s)
@@ -2839,6 +2865,11 @@ int ast_cli_command_multiple_full(int ui
 	return count;
 }
 
+void ast_cli_print_timestr_fromseconds(int fd, int seconds, const char *prefix)
+{
+	print_uptimestr(fd, ast_tv(seconds, 0), prefix, 0);
+}
+
 int ast_cli_allow_at_shutdown(struct ast_cli_entry *e)
 {
 	int res;
diff -pruN 1:13.23.1~dfsg-2/main/codec_builtin.c 1:16.1.1~dfsg-1/main/codec_builtin.c
--- 1:13.23.1~dfsg-2/main/codec_builtin.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/codec_builtin.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
+#include "asterisk/ilbc.h"
 #include "asterisk/logger.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/codec.h"
@@ -108,6 +107,30 @@ static struct ast_codec g723 = {
 	.get_length = g723_length,
 };
 
+static int codec2_samples(struct ast_frame *frame)
+{
+	return 160 * (frame->datalen / 6);
+}
+
+static int codec2_length(unsigned int samples)
+{
+	return (samples / 160) * 6;
+}
+
+static struct ast_codec codec2 = {
+	.name = "codec2",
+	.description = "Codec 2",
+	.type = AST_MEDIA_TYPE_AUDIO,
+	.sample_rate = 8000,
+	.minimum_ms = 20,
+	.maximum_ms = 300,
+	.default_ms = 20,
+	.minimum_bytes = 6,
+	.samples_count = codec2_samples,
+	.get_length = codec2_length,
+	.smooth = 1,
+};
+
 static int none_samples(struct ast_frame *frame)
 {
 	return frame->datalen;
@@ -265,7 +288,8 @@ static struct ast_codec slin8 = {
 	.minimum_bytes = 160,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin12 = {
@@ -279,7 +303,8 @@ static struct ast_codec slin12 = {
 	.minimum_bytes = 240,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin16 = {
@@ -293,7 +318,8 @@ static struct ast_codec slin16 = {
 	.minimum_bytes = 320,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin24 = {
@@ -307,7 +333,8 @@ static struct ast_codec slin24 = {
 	.minimum_bytes = 480,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin32 = {
@@ -321,7 +348,8 @@ static struct ast_codec slin32 = {
 	.minimum_bytes = 640,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin44 = {
@@ -335,7 +363,8 @@ static struct ast_codec slin44 = {
 	.minimum_bytes = 882,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin48 = {
@@ -349,7 +378,8 @@ static struct ast_codec slin48 = {
 	.minimum_bytes = 960,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin96 = {
@@ -363,7 +393,8 @@ static struct ast_codec slin96 = {
 	.minimum_bytes = 1920,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static struct ast_codec slin192 = {
@@ -377,7 +408,8 @@ static struct ast_codec slin192 = {
 	.minimum_bytes = 3840,
 	.samples_count = slin_samples,
 	.get_length = slin_length,
-	.smooth = AST_SMOOTHER_FLAGS_PACK(AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED),
+	.smooth = 1,
+	.smoother_flags = AST_SMOOTHER_FLAG_BE | AST_SMOOTHER_FLAG_FORCED,
 };
 
 static int lpc10_samples(struct ast_frame *frame)
@@ -589,7 +621,12 @@ static struct ast_codec speex32 = {
 
 static int ilbc_samples(struct ast_frame *frame)
 {
-	return 240 * (frame->datalen / 50);
+	struct ilbc_attr *attr = ast_format_get_attribute_data(frame->subclass.format);
+	const unsigned int mode = attr ? attr->mode : 30;
+	const unsigned int samples_per_frame = mode * ast_format_get_sample_rate(frame->subclass.format) / 1000;
+	const unsigned int octets_per_frame = (mode == 20) ? 38 : 50;
+
+	return samples_per_frame * frame->datalen / octets_per_frame;
 }
 
 static struct ast_codec ilbc = {
@@ -597,12 +634,12 @@ static struct ast_codec ilbc = {
 	.description = "iLBC",
 	.type = AST_MEDIA_TYPE_AUDIO,
 	.sample_rate = 8000,
-	.minimum_ms = 30,
+	.minimum_ms = 20,
 	.maximum_ms = 300,
-	.default_ms = 30,
-	.minimum_bytes = 50,
+	.default_ms = 20,
+	.minimum_bytes = 38,
 	.samples_count = ilbc_samples,
-	.smooth = 1,
+	.smooth = 0,
 };
 
 static struct ast_codec g722 = {
@@ -802,6 +839,12 @@ static struct ast_codec t140 = {
 	.type = AST_MEDIA_TYPE_TEXT,
 };
 
+static struct ast_codec t38 = {
+	.name = "t38",
+	.description = "T.38 UDPTL Fax",
+	.type = AST_MEDIA_TYPE_IMAGE,
+};
+
 static int silk_samples(struct ast_frame *frame)
 {
 	/* XXX This is likely not at all what's intended from this callback. However,
@@ -893,6 +936,7 @@ int ast_codec_builtin_init(void)
 {
 	int res = 0;
 
+	res |= CODEC_REGISTER_AND_CACHE(codec2);
 	res |= CODEC_REGISTER_AND_CACHE(g723);
 	res |= CODEC_REGISTER_AND_CACHE(ulaw);
 	res |= CODEC_REGISTER_AND_CACHE(alaw);
@@ -932,6 +976,7 @@ int ast_codec_builtin_init(void)
 	res |= CODEC_REGISTER_AND_CACHE(vp9);
 	res |= CODEC_REGISTER_AND_CACHE(t140red);
 	res |= CODEC_REGISTER_AND_CACHE(t140);
+	res |= CODEC_REGISTER_AND_CACHE(t38);
 	res |= CODEC_REGISTER_AND_CACHE(none);
 	res |= CODEC_REGISTER_AND_CACHE_NAMED("silk8", silk8);
 	res |= CODEC_REGISTER_AND_CACHE_NAMED("silk12", silk12);
diff -pruN 1:13.23.1~dfsg-2/main/codec.c 1:16.1.1~dfsg-1/main/codec.c
--- 1:13.23.1~dfsg-2/main/codec.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/codec.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/codec.h"
 #include "asterisk/format.h"
@@ -250,8 +248,8 @@ static void codec_shutdown(void)
 
 int ast_codec_init(void)
 {
-	codecs = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, CODEC_BUCKETS,
-		ast_codec_hash_fn, codec_cmp);
+	codecs = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, CODEC_BUCKETS,
+		ast_codec_hash_fn, NULL, codec_cmp);
 	if (!codecs) {
 		return -1;
 	}
@@ -362,6 +360,21 @@ const char *ast_codec_media_type2str(enu
 	}
 }
 
+enum ast_media_type ast_media_type_from_str(const char *media_type_str)
+{
+	if (!strcasecmp(media_type_str, "audio")) {
+		return AST_MEDIA_TYPE_AUDIO;
+	} else if (!strcasecmp(media_type_str, "video")) {
+		return AST_MEDIA_TYPE_VIDEO;
+	} else if (!strcasecmp(media_type_str, "image")) {
+		return AST_MEDIA_TYPE_IMAGE;
+	} else if (!strcasecmp(media_type_str, "text")) {
+		return AST_MEDIA_TYPE_TEXT;
+	} else {
+		return AST_MEDIA_TYPE_UNKNOWN;
+	}
+}
+
 unsigned int ast_codec_samples_count(struct ast_frame *frame)
 {
 	struct ast_codec *codec;
diff -pruN 1:13.23.1~dfsg-2/main/config.c 1:16.1.1~dfsg-1/main/config.c
--- 1:13.23.1~dfsg-2/main/config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/config.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,9 +30,12 @@
 	<support_level>core</support_level>
  ***/
 
-#include "asterisk.h"
+/* This maintains the original "module reload extconfig" CLI command instead
+ * of replacing it with "module reload config". */
+#undef AST_MODULE
+#define AST_MODULE "extconfig"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include "asterisk.h"
 
 #include "asterisk/paths.h"	/* use ast_config_AST_CONFIG_DIR */
 #include "asterisk/network.h"	/* we do some sockaddr manipulation here */
@@ -45,8 +48,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <math.h>	/* HUGE_VAL */
 #include <regex.h>
 
-#define AST_INCLUDE_GLOB 1
-
 #include "asterisk/config.h"
 #include "asterisk/cli.h"
 #include "asterisk/lock.h"
@@ -56,6 +57,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/astobj2.h"
 #include "asterisk/strings.h"	/* for the ast_str_*() API */
 #include "asterisk/netsock2.h"
+#include "asterisk/module.h"
 
 #define MAX_NESTED_COMMENTS 128
 #define COMMENT_START ";--"
@@ -283,11 +285,7 @@ struct ast_config_include {
 static void ast_variable_destroy(struct ast_variable *doomed);
 static void ast_includes_destroy(struct ast_config_include *incls);
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_variable *_ast_variable_new(const char *name, const char *value, const char *filename, const char *file, const char *func, int lineno)
-#else
-struct ast_variable *ast_variable_new(const char *name, const char *value, const char *filename)
-#endif
 {
 	struct ast_variable *variable;
 	int name_len = strlen(name) + 1;
@@ -299,13 +297,9 @@ struct ast_variable *ast_variable_new(co
 		fn_len = MIN_VARIABLE_FNAME_SPACE;
 	}
 
-	if (
-#ifdef __AST_DEBUG_MALLOC
-		(variable = __ast_calloc(1, fn_len + name_len + val_len + sizeof(*variable), file, lineno, func))
-#else
-		(variable = ast_calloc(1, fn_len + name_len + val_len + sizeof(*variable)))
-#endif
-		) {
+	variable = __ast_calloc(1, fn_len + name_len + val_len + sizeof(*variable),
+		file, lineno, func);
+	if (variable) {
 		char *dst = variable->stuff;	/* writable space starts here */
 
 		/* Put file first so ast_include_rename() can calculate space available. */
@@ -2051,10 +2045,8 @@ static struct ast_config *config_text_fi
 	/*! Growable string buffer */
 	struct ast_str *comment_buffer = NULL;	/*!< this will be a comment collector.*/
 	struct ast_str *lline_buffer = NULL;	/*!< A buffer for stuff behind the ; */
-#ifdef AST_INCLUDE_GLOB
 	int glob_ret;
 	glob_t globbuf;
-#endif
 
 	if (cfg) {
 		cat = ast_config_get_current_category(cfg);
@@ -2077,7 +2069,7 @@ static struct ast_config *config_text_fi
 			return NULL;
 		}
 	}
-#ifdef AST_INCLUDE_GLOB
+
 	globbuf.gl_offs = 0;	/* initialize it to silence gcc */
 	glob_ret = glob(fn, MY_GLOB_FLAGS, NULL, &globbuf);
 	if (glob_ret == GLOB_NOSPACE) {
@@ -2105,7 +2097,7 @@ static struct ast_config *config_text_fi
 		}
 		for (i=0; i<globbuf.gl_pathc; i++) {
 			ast_copy_string(fn, globbuf.gl_pathv[i], sizeof(fn));
-#endif
+
 			/*
 			 * The following is not a loop, but just a convenient way to define a block
 			 * (using do { } while(0) ), and be able to exit from it with 'continue'
@@ -2164,9 +2156,7 @@ static struct ast_config *config_text_fi
 
 					if (unchanged) {
 						AST_LIST_UNLOCK(&cfmtime_head);
-#ifdef AST_INCLUDE_GLOB
 						globfree(&globbuf);
-#endif
 						ast_free(comment_buffer);
 						ast_free(lline_buffer);
 						return CONFIG_STATUS_FILEUNCHANGED;
@@ -2340,14 +2330,12 @@ static struct ast_config *config_text_fi
 			if (comment) {
 				ast_log(LOG_WARNING,"Unterminated comment detected beginning on line %d\n", nest[comment - 1]);
 			}
-#ifdef AST_INCLUDE_GLOB
 			if (cfg == NULL || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID) {
 				break;
 			}
 		}
 		globfree(&globbuf);
 	}
-#endif
 
 	ast_free(comment_buffer);
 	ast_free(lline_buffer);
@@ -2512,11 +2500,6 @@ static void insert_leading_blank_lines(F
 	fi->lineno = lineno + 1; /* Advance the file lineno */
 }
 
-int config_text_file_save(const char *configfile, const struct ast_config *cfg, const char *generator)
-{
-	return ast_config_text_file_save2(configfile, cfg, generator, CONFIG_SAVE_FLAG_PRESERVE_EFFECTIVE_CONTEXT);
-}
-
 int ast_config_text_file_save(const char *configfile, const struct ast_config *cfg, const char *generator)
 {
 	return ast_config_text_file_save2(configfile, cfg, generator, CONFIG_SAVE_FLAG_PRESERVE_EFFECTIVE_CONTEXT);
@@ -2553,7 +2536,8 @@ int ast_config_text_file_save2(const cha
 	struct ao2_container *fileset;
 	struct inclfile *fi;
 
-	fileset = ao2_container_alloc(1023, hash_string, hashtab_compare_strings);
+	fileset = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 1023,
+		hash_string, NULL, hashtab_compare_strings);
 	if (!fileset) {
 		/* Container creation failed. */
 		return -1;
@@ -2891,7 +2875,7 @@ static int ast_realtime_append_mapping(c
 	return 0;
 }
 
-int read_config_maps(void)
+static int reload_module(void)
 {
 	struct ast_config *config, *configtmp;
 	struct ast_variable *v;
@@ -4051,6 +4035,7 @@ static void config_shutdown(void)
 int register_config_cli(void)
 {
 	ast_cli_register_multiple(cli_config, ARRAY_LEN(cli_config));
+	/* This is separate from the module load so cleanup can happen very late. */
 	ast_register_cleanup(config_shutdown);
 	return 0;
 }
@@ -4120,8 +4105,12 @@ int ast_config_hook_register(const char
 		config_hook_cb hook_cb)
 {
 	struct cfg_hook *hook;
-	if (!cfg_hooks && !(cfg_hooks = ao2_container_alloc(17, hook_hash, hook_cmp))) {
-		return -1;
+	if (!cfg_hooks) {
+		cfg_hooks = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 17,
+			hook_hash, NULL, hook_cmp);
+		if (!cfg_hooks) {
+			return -1;
+		}
 	}
 
 	if (!(hook = ao2_alloc(sizeof(*hook), hook_destroy))) {
@@ -4137,3 +4126,26 @@ int ast_config_hook_register(const char
 	ao2_ref(hook, -1);
 	return 0;
 }
+
+static int unload_module(void)
+{
+	return 0;
+}
+
+static int load_module(void)
+{
+	if (ast_opt_console) {
+		ast_verb(0, "[ Initializing Custom Configuration Options ]\n");
+	}
+
+	return reload_module() ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
+}
+
+/* This module explicitly loads before realtime drivers. */
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Configuration",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = 0,
+);
diff -pruN 1:13.23.1~dfsg-2/main/config_options.c 1:16.1.1~dfsg-1/main/config_options.c
--- 1:13.23.1~dfsg-2/main/config_options.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/config_options.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/_private.h"
@@ -224,7 +222,8 @@ int aco_option_register_deprecated(struc
 		return -1;
 	}
 
-	if (!(opt = ao2_alloc(sizeof(*opt), config_option_destroy))) {
+	opt = ao2_alloc_options(sizeof(*opt), config_option_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!opt) {
 		return -1;
 	}
 
@@ -315,7 +314,9 @@ int __aco_option_register(struct aco_inf
 		return -1;
 	}
 
-	if (!(opt = ao2_alloc(sizeof(*opt) + argc * sizeof(opt->args[0]), config_option_destroy))) {
+	opt = ao2_alloc_options(sizeof(*opt) + argc * sizeof(opt->args[0]),
+		config_option_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!opt) {
 		return -1;
 	}
 
@@ -404,7 +405,8 @@ static struct aco_option *aco_option_fin
 
 struct ao2_container *aco_option_container_alloc(void)
 {
-	return ao2_container_alloc(CONFIG_OPT_BUCKETS, config_opt_hash, config_opt_cmp);
+	return ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, CONFIG_OPT_BUCKETS,
+		config_opt_hash, NULL, config_opt_cmp);
 }
 
 static int internal_aco_type_category_check(struct aco_type *match, const char *category)
diff -pruN 1:13.23.1~dfsg-2/main/conversions.c 1:16.1.1~dfsg-1/main/conversions.c
--- 1:13.23.1~dfsg-2/main/conversions.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/conversions.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,77 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Conversion utility functions
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include <ctype.h>
+#include <errno.h>
+#include <limits.h>
+#include <stdlib.h>
+
+#include "asterisk/conversions.h"
+
+static int str_is_negative(const char *str)
+{
+	/* Ignore any preceding white space */
+	while (isspace(*str) && *++str);
+	return *str == '-';
+}
+
+int ast_str_to_uint(const char *str, unsigned int *res)
+{
+	unsigned long val;
+
+	if (ast_str_to_ulong(str, &val) || val > UINT_MAX) {
+		return -1;
+	}
+
+	*res = val;
+	return 0;
+}
+
+int ast_str_to_ulong(const char *str, unsigned long *res)
+{
+	char *end;
+	unsigned long val;
+
+	if (!str || str_is_negative(str)) {
+		return -1;
+	}
+
+	errno = 0;
+	val = strtoul(str, &end, 0);
+
+	/*
+	 * If str equals end then no digits were found. If end is not pointing to
+	 * a null character then the string contained some numbers that could be
+	 * converted, but some characters that could not, which we'll consider
+	 * invalid.
+	 */
+	if ((str == end || *end != '\0' || (errno == ERANGE && val == ULONG_MAX))) {
+		return -1;
+	}
+
+	*res = val;
+	return 0;
+
+}
diff -pruN 1:13.23.1~dfsg-2/main/core_local.c 1:16.1.1~dfsg-1/main/core_local.c
--- 1:13.23.1~dfsg-2/main/core_local.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/core_local.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 /* ------------------------------------------------------------------- */
 
 #include "asterisk/channel.h"
@@ -235,7 +233,7 @@ struct local_pvt {
 	char exten[AST_MAX_EXTENSION];
 };
 
-void ast_local_lock_all2(struct ast_channel *chan, void **tech_pvt,
+void ast_local_lock_all(struct ast_channel *chan, void **tech_pvt,
 	struct ast_channel **base_chan, struct ast_channel **base_owner)
 {
 	struct local_pvt *p = ast_channel_tech_pvt(chan);
@@ -250,14 +248,7 @@ void ast_local_lock_all2(struct ast_chan
 	}
 }
 
-void ast_local_lock_all(struct ast_channel *chan, struct ast_channel **outchan,
-			struct ast_channel **outowner)
-{
-	void *tech_pvt;
-	ast_local_lock_all2(chan, &tech_pvt, outchan, outowner);
-}
-
-void ast_local_unlock_all2(void *tech_pvt, struct ast_channel *base_chan,
+void ast_local_unlock_all(void *tech_pvt, struct ast_channel *base_chan,
 	struct ast_channel *base_owner)
 {
 	if (base_chan) {
@@ -277,19 +268,6 @@ void ast_local_unlock_all2(void *tech_pv
 	}
 }
 
-void ast_local_unlock_all(struct ast_channel *chan)
-{
-	struct local_pvt *p = ast_channel_tech_pvt(chan);
-	struct ast_unreal_pvt *base;
-
-	if (!p) {
-		return;
-	}
-
-	base = &p->base;
-	ast_local_unlock_all2(p, base->chan, base->owner);
-}
-
 struct ast_channel *ast_local_get_peer(struct ast_channel *ast)
 {
 	struct local_pvt *p = ast_channel_tech_pvt(ast);
@@ -940,7 +918,7 @@ static struct ast_channel *local_request
 {
 	struct local_pvt *p;
 	struct ast_channel *chan;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	/* Allocate a new private structure and then Asterisk channels */
 	p = local_alloc(data, cap);
@@ -953,9 +931,6 @@ static struct ast_channel *local_request
 	if (chan) {
 		ao2_link(locals, p);
 	}
-	if (callid) {
-		ast_callid_unref(callid);
-	}
 	ao2_ref(p, -1); /* kill the ref from the alloc */
 
 	return chan;
diff -pruN 1:13.23.1~dfsg-2/main/core_unreal.c 1:16.1.1~dfsg-1/main/core_unreal.c
--- 1:13.23.1~dfsg-2/main/core_unreal.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/core_unreal.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/causes.h"
 #include "asterisk/channel.h"
 #include "asterisk/stasis_channels.h"
@@ -325,6 +323,20 @@ int ast_unreal_write(struct ast_channel
 		return -1;
 	}
 
+	/* If we are told to write a frame with a type that has no corresponding
+	 * stream on the channel then drop it.
+	 */
+	if (f->frametype == AST_FRAME_VOICE) {
+		if (!ast_channel_get_default_stream(ast, AST_MEDIA_TYPE_AUDIO)) {
+			return 0;
+		}
+	} else if (f->frametype == AST_FRAME_VIDEO ||
+		(f->frametype == AST_FRAME_CONTROL && f->subclass.integer == AST_CONTROL_VIDUPDATE)) {
+		if (!ast_channel_get_default_stream(ast, AST_MEDIA_TYPE_VIDEO)) {
+			return 0;
+		}
+	}
+
 	/* Just queue for delivery to the other side */
 	ao2_ref(p, 1);
 	ao2_lock(p);
@@ -734,12 +746,11 @@ int ast_unreal_channel_push_to_bridge(st
 	struct ast_bridge_features *features;
 	struct ast_channel *chan;
 	struct ast_channel *owner;
+	ast_callid bridge_callid;
 	RAII_VAR(struct ast_unreal_pvt *, p, NULL, ao2_cleanup);
 
-	RAII_VAR(struct ast_callid *, bridge_callid, NULL, ast_callid_cleanup);
-
 	ast_bridge_lock(bridge);
-	bridge_callid = bridge->callid ? ast_callid_ref(bridge->callid) : NULL;
+	bridge_callid = bridge->callid;
 	ast_bridge_unlock(bridge);
 
 	{
@@ -768,8 +779,8 @@ int ast_unreal_channel_push_to_bridge(st
 	}
 
 	if (bridge_callid) {
-		struct ast_callid *chan_callid;
-		struct ast_callid *owner_callid;
+		ast_callid chan_callid;
+		ast_callid owner_callid;
 
 		/* chan side call ID setting */
 		ast_channel_lock(chan);
@@ -779,7 +790,6 @@ int ast_unreal_channel_push_to_bridge(st
 			ast_channel_callid_set(chan, bridge_callid);
 		}
 		ast_channel_unlock(chan);
-		ast_callid_cleanup(chan_callid);
 
 		/* owner side call ID setting */
 		ast_channel_lock(owner);
@@ -790,7 +800,6 @@ int ast_unreal_channel_push_to_bridge(st
 		}
 
 		ast_channel_unlock(owner);
-		ast_callid_cleanup(owner_callid);
 	}
 
 	/* We are done with the owner now that its call ID matches the bridge */
@@ -941,7 +950,7 @@ struct ast_unreal_pvt *ast_unreal_alloc(
 struct ast_channel *ast_unreal_new_channels(struct ast_unreal_pvt *p,
 	const struct ast_channel_tech *tech, int semi1_state, int semi2_state,
 	const char *exten, const char *context, const struct ast_assigned_ids *assignedids,
-	const struct ast_channel *requestor, struct ast_callid *callid)
+	const struct ast_channel *requestor, ast_callid callid)
 {
 	struct ast_channel *owner;
 	struct ast_channel *chan;
diff -pruN 1:13.23.1~dfsg-2/main/crypt.c 1:16.1.1~dfsg-1/main/crypt.c
--- 1:13.23.1~dfsg-2/main/crypt.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/crypt.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <unistd.h>
 #if defined(HAVE_CRYPT_R) && !defined(__FreeBSD__)
 #include <crypt.h>
diff -pruN 1:13.23.1~dfsg-2/main/data_buffer.c 1:16.1.1~dfsg-1/main/data_buffer.c
--- 1:13.23.1~dfsg-2/main/data_buffer.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/data_buffer.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,368 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2018, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ * Ben Ford <bford@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Data Buffer API
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ * \author Ben Ford <bford@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/logger.h"
+#include "asterisk/strings.h"
+#include "asterisk/data_buffer.h"
+#include "asterisk/linkedlists.h"
+
+/*!
+ * \brief The number of payloads to increment the cache by
+ */
+#define CACHED_PAYLOAD_MAX 5
+
+/*!
+ * \brief Payload entry placed inside of the data buffer list
+ */
+struct data_buffer_payload_entry {
+	/*! \brief The payload for this position */
+	void *payload;
+	/*! \brief The provided position for this */
+	size_t pos;
+	/*! \brief Linked list information */
+	AST_LIST_ENTRY(data_buffer_payload_entry) list;
+};
+
+/*!
+ * \brief Data buffer containing fixed number of data payloads
+ */
+struct ast_data_buffer {
+	/*! \brief Callback function to free a data payload */
+	ast_data_buffer_free_callback free_fn;
+	/*! \brief A linked list of data payloads */
+	AST_LIST_HEAD_NOLOCK(, data_buffer_payload_entry) payloads;
+	/*! \brief A linked list of unused cached data payloads */
+	AST_LIST_HEAD_NOLOCK(, data_buffer_payload_entry) cached_payloads;
+	/*! \brief The current number of data payloads in the buffer */
+	size_t count;
+	/*! \brief Maximum number of data payloads in the buffer */
+	size_t max;
+	/*! \brief The current number of data payloads in the cache */
+	size_t cache_count;
+};
+
+static void free_fn_do_nothing(void *data)
+{
+	return;
+}
+
+/*!
+ * \brief Helper function to allocate a data payload
+ */
+static struct data_buffer_payload_entry *data_buffer_payload_alloc(void *payload, size_t pos)
+{
+	struct data_buffer_payload_entry *data_payload;
+
+	data_payload = ast_calloc(1, sizeof(*data_payload));
+	if (!data_payload) {
+		return NULL;
+	}
+
+	data_payload->payload = payload;
+	data_payload->pos = pos;
+
+	return data_payload;
+}
+
+/*!
+ * \brief Helper function that sets the cache to its maximum number of payloads
+ */
+static void ast_data_buffer_cache_adjust(struct ast_data_buffer *buffer)
+{
+	int buffer_space;
+
+	ast_assert(buffer != NULL);
+
+	buffer_space = buffer->max - buffer->count;
+
+	if (buffer->cache_count == buffer_space) {
+		return;
+	}
+
+	if (buffer->cache_count < buffer_space) {
+		/* Add payloads to the cache, if able */
+		while (buffer->cache_count < CACHED_PAYLOAD_MAX && buffer->cache_count < buffer_space) {
+			struct data_buffer_payload_entry *buffer_payload;
+
+			buffer_payload = data_buffer_payload_alloc(NULL, -1);
+			if (buffer_payload) {
+				AST_LIST_INSERT_TAIL(&buffer->cached_payloads, buffer_payload, list);
+				buffer->cache_count++;
+				continue;
+			}
+
+			ast_log(LOG_ERROR, "Failed to allocate memory to the cache.");
+			break;
+		}
+	} else if (buffer->cache_count > buffer_space) {
+		/* Remove payloads from the cache */
+		while (buffer->cache_count > buffer_space) {
+			struct data_buffer_payload_entry *buffer_payload;
+
+			buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->cached_payloads, list);
+			if (buffer_payload) {
+				ast_free(buffer_payload);
+				buffer->cache_count--;
+				continue;
+			}
+
+			ast_log(LOG_ERROR, "Failed to remove memory from the cache.");
+			break;
+		}
+	}
+}
+
+struct ast_data_buffer *ast_data_buffer_alloc(ast_data_buffer_free_callback free_fn, size_t size)
+{
+	struct ast_data_buffer *buffer;
+
+	ast_assert(size != 0);
+
+	buffer = ast_calloc(1, sizeof(*buffer));
+	if (!buffer) {
+		return NULL;
+	}
+
+	AST_LIST_HEAD_INIT_NOLOCK(&buffer->payloads);
+	AST_LIST_HEAD_INIT_NOLOCK(&buffer->cached_payloads);
+
+	/* If free_fn is NULL, just use free_fn_do_nothing as a default */
+	buffer->free_fn = free_fn ? free_fn : free_fn_do_nothing;
+	buffer->max = size;
+
+	ast_data_buffer_cache_adjust(buffer);
+
+	return buffer;
+}
+
+void ast_data_buffer_resize(struct ast_data_buffer *buffer, size_t size)
+{
+	struct data_buffer_payload_entry *existing_payload;
+
+	ast_assert(buffer != NULL);
+
+	/* The buffer must have at least a size of 1 */
+	ast_assert(size > 0);
+
+	if (buffer->max == size) {
+		return;
+	}
+
+	/* If the size is decreasing, some payloads will need to be freed */
+	if (buffer->max > size) {
+		int remove = buffer->max - size;
+
+		AST_LIST_TRAVERSE_SAFE_BEGIN(&buffer->payloads, existing_payload, list) {
+			if (remove) {
+				AST_LIST_REMOVE_HEAD(&buffer->payloads, list);
+				buffer->free_fn(existing_payload->payload);
+				ast_free(existing_payload);
+				buffer->count--;
+				remove--;
+				continue;
+			}
+			break;
+		}
+		AST_LIST_TRAVERSE_SAFE_END;
+	}
+
+	buffer->max = size;
+	ast_data_buffer_cache_adjust(buffer);
+}
+
+int ast_data_buffer_put(struct ast_data_buffer *buffer, size_t pos, void *payload)
+{
+	struct data_buffer_payload_entry *buffer_payload = NULL;
+	struct data_buffer_payload_entry *existing_payload;
+	int inserted = 0;
+
+	ast_assert(buffer != NULL);
+	ast_assert(payload != NULL);
+
+	/* If the data buffer has reached its maximum size then the head goes away and
+	 * we will reuse its buffer payload
+	 */
+	if (buffer->count == buffer->max) {
+		buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->payloads, list);
+		buffer->free_fn(buffer_payload->payload);
+		buffer->count--;
+
+		/* Update this buffer payload with its new information */
+		buffer_payload->payload = payload;
+		buffer_payload->pos = pos;
+	}
+	if (!buffer_payload) {
+		if (!buffer->cache_count) {
+			ast_data_buffer_cache_adjust(buffer);
+		}
+		buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->cached_payloads, list);
+		buffer->cache_count--;
+
+		/* Update the payload from the cache with its new information */
+		buffer_payload->payload = payload;
+		buffer_payload->pos = pos;
+	}
+	if (!buffer_payload) {
+		return -1;
+	}
+
+	/* Given the position find its ideal spot within the buffer */
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&buffer->payloads, existing_payload, list) {
+		/* If it's already in the buffer, drop it */
+		if (existing_payload->pos == pos) {
+			ast_debug(3, "Packet with position %zu is already in buffer. Not inserting.\n", pos);
+			inserted = -1;
+			break;
+		}
+
+		if (existing_payload->pos > pos) {
+			AST_LIST_INSERT_BEFORE_CURRENT(buffer_payload, list);
+			inserted = 1;
+			break;
+		}
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	if (inserted == -1) {
+		return 0;
+	}
+
+	if (!inserted) {
+		AST_LIST_INSERT_TAIL(&buffer->payloads, buffer_payload, list);
+	}
+
+	buffer->count++;
+
+	return 0;
+}
+
+void *ast_data_buffer_get(const struct ast_data_buffer *buffer, size_t pos)
+{
+	struct data_buffer_payload_entry *buffer_payload;
+
+	ast_assert(buffer != NULL);
+
+	AST_LIST_TRAVERSE(&buffer->payloads, buffer_payload, list) {
+		if (buffer_payload->pos == pos) {
+			return buffer_payload->payload;
+		}
+	}
+
+	return NULL;
+}
+
+static void data_buffer_free_buffer_payload(struct ast_data_buffer *buffer,
+	struct data_buffer_payload_entry *buffer_payload)
+{
+	buffer_payload->payload = NULL;
+	buffer->count--;
+
+	if (buffer->cache_count < CACHED_PAYLOAD_MAX
+			&& buffer->cache_count < (buffer->max - buffer->count)) {
+		AST_LIST_INSERT_TAIL(&buffer->cached_payloads, buffer_payload, list);
+		buffer->cache_count++;
+	} else {
+		ast_free(buffer_payload);
+	}
+}
+
+void *ast_data_buffer_remove(struct ast_data_buffer *buffer, size_t pos)
+{
+	struct data_buffer_payload_entry *buffer_payload;
+
+	ast_assert(buffer != NULL);
+
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&buffer->payloads, buffer_payload, list) {
+		if (buffer_payload->pos == pos) {
+			void *payload = buffer_payload->payload;
+
+			AST_LIST_REMOVE_CURRENT(list);
+			data_buffer_free_buffer_payload(buffer, buffer_payload);
+
+			return payload;
+		}
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	return NULL;
+}
+
+void *ast_data_buffer_remove_head(struct ast_data_buffer *buffer)
+{
+	ast_assert(buffer != NULL);
+
+	if (buffer->count > 0) {
+		struct data_buffer_payload_entry *buffer_payload;
+		void *payload;
+
+		buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->payloads, list);
+		payload = buffer_payload->payload;
+		data_buffer_free_buffer_payload(buffer, buffer_payload);
+
+		return payload;
+	}
+
+	return NULL;
+}
+
+void ast_data_buffer_free(struct ast_data_buffer *buffer)
+{
+	struct data_buffer_payload_entry *buffer_payload;
+
+	ast_assert(buffer != NULL);
+
+	while ((buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->payloads, list))) {
+		buffer->free_fn(buffer_payload->payload);
+		ast_free(buffer_payload);
+	}
+
+	while ((buffer_payload = AST_LIST_REMOVE_HEAD(&buffer->cached_payloads, list))) {
+		ast_free(buffer_payload);
+	}
+
+	ast_free(buffer);
+}
+
+size_t ast_data_buffer_count(const struct ast_data_buffer *buffer)
+{
+	ast_assert(buffer != NULL);
+
+	return buffer->count;
+}
+
+size_t ast_data_buffer_max(const struct ast_data_buffer *buffer)
+{
+	ast_assert(buffer != NULL);
+
+	return buffer->max;
+}
diff -pruN 1:13.23.1~dfsg-2/main/data.c 1:16.1.1~dfsg-1/main/data.c
--- 1:13.23.1~dfsg-2/main/data.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/data.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,3352 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2009, Eliel C. Sardanons (LU1ALY) <eliels@gmail.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- *
- * \brief Data retrieval API.
- *
- * \author Brett Bryant <brettbryant@gmail.com>
- * \author Eliel C. Sardanons (LU1ALY) <eliels@gmail.com>
- */
-
-/*** MODULEINFO
-	<support_level>core</support_level>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#include "asterisk/_private.h"
-
-#include <regex.h>
-
-#include "asterisk/module.h"
-#include "asterisk/utils.h"
-#include "asterisk/lock.h"
-#include "asterisk/data.h"
-#include "asterisk/astobj2.h"
-#include "asterisk/xml.h"
-#include "asterisk/cli.h"
-#include "asterisk/term.h"
-#include "asterisk/manager.h"
-#include "asterisk/test.h"
-#include "asterisk/frame.h"
-#include "asterisk/codec.h"
-
-/*** DOCUMENTATION
-	<manager name="DataGet" language="en_US">
-		<synopsis>
-			Retrieve the data api tree.
-		</synopsis>
-		<syntax>
-			<xi:include xpointer="xpointer(/docs/manager[@name='Login']/syntax/parameter[@name='ActionID'])" />
-			<parameter name="Path" required="true" />
-			<parameter name="Search" />
-			<parameter name="Filter" />
-		</syntax>
-		<description>
-			<para>Retrieve the data api tree.</para>
-		</description>
-	</manager>
- ***/
-
-#define NUM_DATA_NODE_BUCKETS	59
-#define NUM_DATA_RESULT_BUCKETS 59
-#define NUM_DATA_SEARCH_BUCKETS 59
-#define NUM_DATA_FILTER_BUCKETS 59
-
-/*! \brief The last compatible version. */
-static const uint32_t latest_handler_compatible_version = 0;
-
-/*! \brief The last compatible version. */
-static const uint32_t latest_query_compatible_version = 0;
-
-/*! \brief Current handler structure version. */
-static const uint32_t current_handler_version = AST_DATA_HANDLER_VERSION;
-
-/*! \brief Current query structure version. */
-static const uint32_t current_query_version = AST_DATA_QUERY_VERSION;
-
-/*! \brief The data tree to be returned by the callbacks and
-	   managed by functions local to this file. */
-struct ast_data {
-	enum ast_data_type type;
-
-	/*! \brief The node content. */
-	union {
-		int32_t sint;
-		uint32_t uint;
-		double dbl;
-		unsigned int boolean;
-		char *str;
-		char character;
-		struct in_addr ipaddr;
-		void *ptr;
-	} payload;
-
-	/*! \brief The filter node that depends on the current node,
-	 * this is used only when creating the result tree. */
-	const struct data_filter *filter;
-
-	/*! \brief The list of nodes inside this node. */
-	struct ao2_container *children;
-	/*! \brief The name of the node. */
-	char name[0];
-};
-
-/*! \brief Type of comparisons allow in the search string. */
-enum data_search_comparison {
-	DATA_CMP_UNKNOWN,
-	DATA_CMP_EQ,	/* =  */
-	DATA_CMP_NEQ,	/* != */
-	DATA_CMP_GT,	/* >  */
-	DATA_CMP_GE,	/* >= */
-	DATA_CMP_LT,	/* <  */
-	DATA_CMP_LE	/* <= */
-};
-
-/*! \brief The list of nodes with their search requirement. */
-struct ast_data_search {
-	/*! \brief The value of the comparison. */
-	char *value;
-	/*! \brief The type of comparison. */
-	enum data_search_comparison cmp_type;
-	/*! \brief reference another node. */
-	struct ao2_container *children;
-	/*! \brief The name of the node we are trying to compare. */
-	char name[0];
-};
-
-struct data_filter;
-
-/*! \brief The filter node. */
-struct data_filter {
-	/*! \brief node childrens. */
-	struct ao2_container *children;
-	/*! \brief glob list */
-	AST_LIST_HEAD_NOLOCK(glob_list_t, data_filter) glob_list;
-	/*! \brief glob list entry */
-	AST_LIST_ENTRY(data_filter) list;
-	/*! \brief node name. */
-	char name[0];
-};
-
-/*! \brief A data container node pointing to the registered handler. */
-struct data_provider {
-	/*! \brief node content handler. */
-	const struct ast_data_handler *handler;
-	/*! \brief Module providing this handler. */
-	struct ast_module *module;
-	/*! \brief children nodes. */
-	struct ao2_container *children;
-	/*! \brief Who registered this node. */
-	const char *registrar;
-	/*! \brief Node name. */
-	char name[0];
-};
-
-/*! \brief This structure is used by the iterator. */
-struct ast_data_iterator {
-	/*! \brief The internal iterator. */
-	struct ao2_iterator internal_iterator;
-	/*! \brief The last returned node. */
-	struct ast_data *last;
-	/*! \brief The iterator pattern. */
-	const char *pattern;
-	/*! \brief The compiled patter. */
-	regex_t regex_pattern;
-	/*! \brief is a regular expression. */
-	unsigned int is_pattern:1;
-};
-
-struct {
-	/*! \brief The asterisk data main content structure. */
-	struct ao2_container *container;
-	/*! \brief asterisk data locking mechanism. */
-	ast_rwlock_t lock;
-} root_data;
-
-static void __data_result_print_cli(int fd, const struct ast_data *root, uint32_t depth);
-
-/*!
- * \internal
- * \brief Common string hash function.
- * \see ast_data_init
- */
-static int data_provider_hash(const void *obj, const int flags)
-{
-	const struct data_provider *node = obj;
-	return ast_str_case_hash(node->name);
-}
-
-/*!
- * \internal
- * \brief Compare two data_provider's.
- * \see ast_data_init
- */
-static int data_provider_cmp(void *obj1, void *obj2, int flags)
-{
-	struct data_provider *node1 = obj1, *node2 = obj2;
-	return strcasecmp(node1->name, node2->name) ? 0 : CMP_MATCH;
-}
-
-/*!
- * \internal
- * \brief Common string hash function for data nodes
- */
-static int data_result_hash(const void *obj, const int flags)
-{
-	const struct ast_data *node = obj;
-	return ast_str_hash(node->name);
-}
-
-/*!
- * \internal
- * \brief Common string comparison function
- */
-static int data_result_cmp(void *obj, void *arg, int flags)
-{
-	struct ast_data *node1 = obj, *node2 = arg;
-	return strcasecmp(node1->name, node2->name) ? 0 : CMP_MATCH;
-}
-
-/*!
- * \internal
- * \brief Lock the data registered handlers structure for writing.
- * \see data_unlock
- */
-#define data_write_lock() ast_rwlock_wrlock(&root_data.lock)
-
-/*!
- * \internal
- * \brief Lock the data registered handlers structure for reading.
- * \see data_unlock
- */
-#define data_read_lock() ast_rwlock_rdlock(&root_data.lock)
-
-/*!
- * \internal
- * \brief Unlock the data registered handlers structure.
- */
-#define data_unlock() ast_rwlock_unlock(&root_data.lock)
-
-/*!
- * \internal
- * \brief Check if a version is compatible with the current core.
- * \param[in] structure_version The current structure version.
- * \param[in] latest_compatible The latest compatible version.
- * \param[in] current The current Data API version.
- * \retval 1 If the module is compatible.
- * \retval 0 If the module is NOT compatible.
- */
-static int data_structure_compatible(int structure_version, uint32_t latest_compatible,
-	uint32_t current)
-{
-	if (structure_version >= latest_compatible && structure_version <= current) {
-		return 1;
-	}
-
-	ast_log(LOG_ERROR, "A module is not compatible with the"
-		"current data api version\n");
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Get the next node name in a path (/node1/node2)
- *        Avoid null nodes like //node1//node2/node3.
- * \param[in] path The path where we are going to search for the next node name.
- * \retval The next node name we found inside the given path.
- * \retval NULL if there are no more node names.
- */
-static char *next_node_name(char **path)
-{
-	char *res;
-
-	do {
-		res = strsep(path, "/");
-	} while (res && ast_strlen_zero(res));
-
-	return res;
-}
-
-/*!
- * \internal
- * \brief Release the memory allocated by a call to ao2_alloc.
- */
-static void data_provider_destructor(void *obj)
-{
-	struct data_provider *provider = obj;
-
-	ao2_ref(provider->children, -1);
-}
-
-/*!
- * \internal
- * \brief Create a new data node.
- * \param[in] name The name of the node we are going to create.
- * \param[in] handler The handler registered for this node.
- * \param[in] registrar The name of the registrar.
- * \retval NULL on error.
- * \retval The allocated data node structure.
- */
-static struct data_provider *data_provider_new(const char *name,
-	const struct ast_data_handler *handler, const char *registrar)
-{
-	struct data_provider *node;
-	size_t namelen;
-
-	namelen = strlen(name) + 1;
-
-	node = ao2_alloc(sizeof(*node) + namelen, data_provider_destructor);
-	if (!node) {
-		return NULL;
-	}
-
-	node->handler = handler;
-	node->registrar = registrar;
-	strcpy(node->name, name);
-
-	/* initialize the childrens container. */
-	if (!(node->children = ao2_container_alloc(NUM_DATA_NODE_BUCKETS,
-			data_provider_hash, data_provider_cmp))) {
-		ao2_ref(node, -1);
-		return NULL;
-	}
-
-	return node;
-}
-
-/*!
- * \internal
- * \brief Add a child node named 'name' to the 'parent' node.
- * \param[in] parent Where to add the child node.
- * \param[in] name The name of the child node.
- * \param[in] handler The handler structure.
- * \param[in] registrar Who registered this node.
- * \retval NULL on error.
- * \retval A newly allocated child in parent.
- */
-static struct data_provider *data_provider_add_child(struct ao2_container *parent,
-	const char *name, const struct ast_data_handler *handler, const char *registrar)
-{
-	struct data_provider *child;
-
-	child = data_provider_new(name, handler, registrar);
-	if (!child) {
-		return NULL;
-	}
-
-	ao2_link(parent, child);
-
-	return child;
-}
-
-/*!
- * \internal
- * \brief Find a child node, based on his name.
- * \param[in] parent Where to find the node.
- * \param[in] name The node name to find.
- * \param[in] registrar Also check if the node was being used by this registrar.
- * \retval NULL if a node wasn't found.
- * \retval The node found.
- * \note Remember to decrement the ref count of the returned node after using it.
- */
-static struct data_provider *data_provider_find(struct ao2_container *parent,
-	const char *name, const char *registrar)
-{
-	struct data_provider *find_node, *found;
-
-	/* XXX avoid allocating a new data node for searching... */
-	find_node = data_provider_new(name, NULL, NULL);
-	if (!find_node) {
-		return NULL;
-	}
-
-	found = ao2_find(parent, find_node, OBJ_POINTER);
-
-	/* free the created node used for searching. */
-	ao2_ref(find_node, -1);
-
-	if (found && found->registrar && registrar) {
-		if (strcmp(found->registrar, registrar)) {
-			/* if the name doesn't match, do not return this node. */
-			ast_debug(1, "Registrar doesn't match, node was registered"
-				" by '%s' and we are searching for '%s'\n",
-				found->registrar, registrar);
-			ao2_ref(found, -1);
-			return NULL;
-		}
-	}
-
-	return found;
-}
-
-/*!
- * \internal
- * \brief Release a group of nodes.
- * \param[in] parent The parent node.
- * \param[in] path The path of nodes to release.
- * \param[in] registrar Who registered this node.
- * \retval <0 on error.
- * \retval 0 on success.
- * \see data_provider_create
- */
-static int data_provider_release(struct ao2_container *parent, const char *path,
-	const char *registrar)
-{
-	char *node_name, *rpath;
-	struct data_provider *child;
-	int ret = 0;
-
-	rpath = ast_strdupa(path);
-
-	node_name = next_node_name(&rpath);
-	if (!node_name) {
-		return -1;
-	}
-
-	child = data_provider_find(parent, node_name, registrar);
-	if (!child) {
-		return -1;
-	}
-
-	/* if this is not a terminal node. */
-	if (!child->handler && rpath) {
-		ret = data_provider_release(child->children, rpath, registrar);
-	}
-
-	/* if this node is empty, unlink it. */
-	if (!ret && !ao2_container_count(child->children)) {
-		ao2_unlink(parent, child);
-	}
-
-	ao2_ref(child, -1);
-
-	return ret;
-}
-
-/*!
- * \internal
- * \brief Release every node registered by 'registrar'.
- * \param[in] parent The parent node.
- * \param[in] registrar
- * \see __ast_data_unregister
- */
-static void data_provider_release_all(struct ao2_container *parent,
-	const char *registrar)
-{
-	struct ao2_iterator i;
-	struct data_provider *node;
-
-	i = ao2_iterator_init(parent, 0);
-	while ((node = ao2_iterator_next(&i))) {
-		if (!node->handler) {
-			/* this is a non-terminal node, go inside it. */
-			data_provider_release_all(node->children, registrar);
-			if (!ao2_container_count(node->children)) {
-				/* if this node was left empty, unlink it. */
-				ao2_unlink(parent, node);
-			}
-		} else {
-			if (!strcmp(node->registrar, registrar)) {
-				/* if the registrars match, release it! */
-				ao2_unlink(parent, node);
-			}
-		}
-		ao2_ref(node, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-}
-
-/*!
- * \internal
- * \brief Create the middle nodes for the specified path (asterisk/testnode1/childnode)
- * \param[in] parent Where to add the middle nodes structure.
- * \param[in] path The path of nodes to add.
- * \param[in] registrar Who is trying to create this node provider.
- * \retval NULL on error.
- * \retval The created node.
- * \see data_provider_release
- */
-static struct data_provider *data_provider_create(struct ao2_container *parent,
-	const char *path, const char *registrar)
-{
-	char *rpath, *node_name;
-	struct data_provider *child, *ret = NULL;
-
-	rpath = ast_strdupa(path);
-
-	node_name = next_node_name(&rpath);
-	if (!node_name) {
-		/* no more nodes to create. */
-		return NULL;
-	}
-
-	child = data_provider_find(parent, node_name, NULL);
-
-	if (!child) {
-		/* nodes without handler are non-terminal nodes. */
-		child = data_provider_add_child(parent, node_name, NULL, registrar);
-	}
-
-	if (rpath) {
-		ret = data_provider_create(child->children, rpath, registrar);
-		if (ret) {
-			ao2_ref(child, -1);
-		}
-	}
-
-	return ret ? ret : child;
-}
-
-int __ast_data_register(const char *path, const struct ast_data_handler *handler,
-	const char *registrar, struct ast_module *mod)
-{
-	struct data_provider *node;
-
-	if (!path) {
-		return -1;
-	}
-
-	/* check if the handler structure is compatible. */
-	if (!data_structure_compatible(handler->version,
-		latest_handler_compatible_version,
-		current_handler_version)) {
-		return -1;
-	}
-
-	/* create the node structure for the registered handler. */
-	data_write_lock();
-
-	node = data_provider_create(root_data.container, path, registrar);
-	if (!node) {
-		ast_log(LOG_ERROR, "Unable to create the specified path (%s) "
-			"for '%s'.\n", path, registrar);
-		data_unlock();
-		return -1;
-	}
-
-	if (ao2_container_count(node->children) || node->handler) {
-		ast_log(LOG_ERROR, "The node '%s' was already registered. "
-			"We were unable to register '%s' for registrar '%s'.\n",
-			node->name, path, registrar);
-		ao2_ref(node, -1);
-		data_unlock();
-		return -1;
-	}
-
-	/* add handler to that node. */
-	node->handler = handler;
-	node->module = mod;
-
-	ao2_ref(node, -1);
-
-	data_unlock();
-
-	return 0;
-}
-
-int __ast_data_register_multiple(const struct ast_data_entry *data_entries,
-	size_t entries, const char *registrar, struct ast_module *mod)
-{
-	int i, res;
-
-	for (i = 0; i < entries; i++) {
-		res = __ast_data_register(data_entries[i].path, data_entries[i].handler,
-				registrar, mod);
-		if (res) {
-			/* unregister all the already registered nodes, and make
-			 * this an atomic action. */
-			while ((--i) >= 0) {
-				__ast_data_unregister(data_entries[i].path, registrar);
-			}
-			return -1;
-		}
-	}
-
-	return 0;
-}
-
-int __ast_data_unregister(const char *path, const char *registrar)
-{
-	int ret = 0;
-
-	data_write_lock();
-	if (path) {
-		ret = data_provider_release(root_data.container, path, registrar);
-	} else {
-		data_provider_release_all(root_data.container, registrar);
-	}
-	data_unlock();
-
-	if (path && ret) {
-		ast_log(LOG_ERROR, "Unable to unregister '%s' for '%s'\n",
-			path, registrar);
-	}
-
-	return ret;
-}
-
-/*!
- * \internal
- * \brief Is a char used to specify a comparison?
- * \param[in] a Character to evaluate.
- * \retval 1 It is a char used to specify a comparison.
- * \retval 0 It is NOT a char used to specify a comparison.
- */
-static int data_search_comparison_char(char a)
-{
-	switch (a) {
-	case '!':
-	case '=':
-	case '<':
-	case '>':
-		return 1;
-	}
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Get the type of comparison.
- */
-static enum data_search_comparison data_search_comparison_type(const char *comparison)
-{
-	if (!strcmp(comparison, "=")) {
-		return DATA_CMP_EQ;
-	} else if (!strcmp(comparison, "!=")) {
-		return DATA_CMP_NEQ;
-	} else if (!strcmp(comparison, "<")) {
-		return DATA_CMP_LT;
-	} else if (!strcmp(comparison, ">")) {
-		return DATA_CMP_GT;
-	} else if (!strcmp(comparison, "<=")) {
-		return DATA_CMP_LE;
-	} else if (!strcmp(comparison, ">=")) {
-		return DATA_CMP_GE;
-	}
-
-	return DATA_CMP_UNKNOWN;
-}
-
-/*!
- * \internal
- * \brief Common string hash function for data nodes
- */
-static int data_search_hash(const void *obj, const int flags)
-{
-	const struct ast_data_search *node = obj;
-	return ast_str_hash(node->name);
-}
-
-/*!
- * \internal
- * \brief Common string comparison function
- */
-static int data_search_cmp(void *obj, void *arg, int flags)
-{
-	struct ast_data_search *node1 = obj, *node2 = arg;
-	return strcasecmp(node1->name, node2->name) ? 0 : CMP_MATCH;
-}
-
-/*!
- * \internal
- * \brief Destroy the ao2 search node.
- */
-static void data_search_destructor(void *obj)
-{
-	struct ast_data_search *node = obj;
-
-	if (node->value) {
-		ast_free(node->value);
-	}
-
-	ao2_ref(node->children, -1);
-}
-
-/*!
- * \internal
- * \brief Allocate a search node.
- * \retval NULL on error.
- * \retval non-NULL The allocated search node structure.
- */
-static struct ast_data_search *data_search_alloc(const char *name)
-{
-	struct ast_data_search *res;
-	size_t name_len = strlen(name) + 1;
-
-	res = ao2_alloc(sizeof(*res) + name_len, data_search_destructor);
-	if (!res) {
-		return NULL;
-	}
-
-	res->children = ao2_container_alloc(NUM_DATA_SEARCH_BUCKETS, data_search_hash,
-		data_search_cmp);
-
-	if (!res->children) {
-		ao2_ref(res, -1);
-		return NULL;
-	}
-
-	strcpy(res->name, name);
-
-	return res;
-}
-
-/*!
- * \internal
- * \brief Find a child node, based on his name.
- * \param[in] parent Where to find the node.
- * \param[in] name The node name to find.
- * \retval NULL if a node wasn't found.
- * \retval The node found.
- * \note Remember to decrement the ref count of the returned node after using it.
- */
-static struct ast_data_search *data_search_find(struct ao2_container *parent,
-	const char *name)
-{
-	struct ast_data_search *find_node, *found;
-
-	find_node = data_search_alloc(name);
-	if (!find_node) {
-		return NULL;
-	}
-
-	found = ao2_find(parent, find_node, OBJ_POINTER);
-
-	/* free the created node used for searching. */
-	ao2_ref(find_node, -1);
-
-	return found;
-}
-
-/*!
- * \internal
- * \brief Add a child node named 'name' to the 'parent' node.
- * \param[in] parent Where to add the child node.
- * \param[in] name The name of the child node.
- * \retval NULL on error.
- * \retval A newly allocated child in parent.
- */
-static struct ast_data_search *data_search_add_child(struct ao2_container *parent,
-	const char *name)
-{
-	struct ast_data_search *child;
-
-	child = data_search_alloc(name);
-	if (!child) {
-		return NULL;
-	}
-
-	ao2_link(parent, child);
-
-	return child;
-}
-
-/*!
- * \internal
- * \brief Create the middle nodes for the specified path (asterisk/testnode1/childnode)
- * \param[in] parent Where to add the middle nodes structure.
- * \param[in] path The path of nodes to add.
- * \retval NULL on error.
- * \retval The created node.
- */
-static struct ast_data_search *data_search_create(struct ao2_container *parent,
-	const char *path)
-{
-	char *rpath, *node_name;
-	struct ast_data_search *child = NULL;
-	struct ao2_container *current = parent;
-
-	rpath = ast_strdupa(path);
-
-	node_name = next_node_name(&rpath);
-	while (node_name) {
-		child = data_search_find(current, node_name);
-		if (!child) {
-			child = data_search_add_child(current, node_name);
-		}
-		ao2_ref(child, -1);
-		current = child->children;
-		node_name = next_node_name(&rpath);
-	}
-
-	return child;
-}
-
-/*!
- * \internal
- * \brief Allocate a tree with the search string parsed.
- * \param[in] search_string The search string.
- * \retval NULL on error.
- * \retval non-NULL A dynamically allocated search tree.
- */
-static struct ast_data_search *data_search_generate(const char *search_string)
-{
-	struct ast_str *name, *value, *comparison;
-	char *elements, *search_string_dup, *saveptr;
-	int i;
-	struct ast_data_search *root, *child;
-	enum data_search_comparison cmp_type;
-	size_t search_string_len;
-
-	if (!search_string) {
-		ast_log(LOG_ERROR, "You must pass a valid search string.\n");
-		return NULL;
-	}
-
-	search_string_len = strlen(search_string);
-
-	name = ast_str_create(search_string_len);
-	if (!name) {
-		return NULL;
-	}
-	value = ast_str_create(search_string_len);
-	if (!value) {
-		ast_free(name);
-		return NULL;
-	}
-	comparison = ast_str_create(search_string_len);
-	if (!comparison) {
-		ast_free(name);
-		ast_free(value);
-		return NULL;
-	}
-
-	search_string_dup = ast_strdupa(search_string);
-
-	/* Create the root node (just used as a container) */
-	root = data_search_alloc("/");
-	if (!root) {
-		ast_free(name);
-		ast_free(value);
-		ast_free(comparison);
-		return NULL;
-	}
-
-	for (elements = strtok_r(search_string_dup, ",", &saveptr); elements;
-		elements = strtok_r(NULL, ",", &saveptr)) {
-		/* Parse the name */
-		ast_str_reset(name);
-		for (i = 0; !data_search_comparison_char(elements[i]) &&
-			elements[i]; i++) {
-			ast_str_append(&name, 0, "%c", elements[i]);
-		}
-
-		/* check if the syntax is ok. */
-		if (!data_search_comparison_char(elements[i])) {
-			/* if this is the end of the string, then this is
-			 * an error! */
-			ast_log(LOG_ERROR, "Invalid search string!\n");
-			continue;
-		}
-
-		/* parse the comparison string. */
-		ast_str_reset(comparison);
-		for (; data_search_comparison_char(elements[i]) && elements[i]; i++) {
-			ast_str_append(&comparison, 0, "%c", elements[i]);
-		}
-
-		/* parse the value string. */
-		ast_str_reset(value);
-		for (; elements[i]; i++) {
-			ast_str_append(&value, 0, "%c", elements[i]);
-		}
-
-		cmp_type = data_search_comparison_type(ast_str_buffer(comparison));
-		if (cmp_type == DATA_CMP_UNKNOWN) {
-			ast_log(LOG_ERROR, "Invalid comparison '%s'\n",
-				ast_str_buffer(comparison));
-			continue;
-		}
-
-		/* add this node to the tree. */
-		child = data_search_create(root->children, ast_str_buffer(name));
-		if (child) {
-			child->cmp_type = cmp_type;
-			child->value = ast_strdup(ast_str_buffer(value));
-		}
-	}
-
-	ast_free(name);
-	ast_free(value);
-	ast_free(comparison);
-
-	return root;
-}
-
-/*!
- * \internal
- * \brief Release the allocated memory for the search tree.
- * \param[in] search The search tree root node.
- */
-static void data_search_release(struct ast_data_search *search)
-{
-	ao2_ref(search, -1);
-}
-
-/*!
- * \internal
- * \brief Based on the kind of comparison and the result in cmpval, return
- *        if it matches.
- * \param[in] cmpval A result returned by a strcmp() for example.
- * \param[in] comparison_type The kind of comparison (<,>,=,!=,...)
- * \retval 1 If the comparison doesn't match.
- * \retval 0 If the comparison matches.
- */
-static inline int data_search_comparison_result(int cmpval,
-	enum data_search_comparison comparison_type)
-{
-	switch (comparison_type) {
-	case DATA_CMP_GE:
-		if (cmpval >= 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_LE:
-		if (cmpval <= 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_EQ:
-		if (cmpval == 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_NEQ:
-		if (cmpval != 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_LT:
-		if (cmpval < 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_GT:
-		if (cmpval > 0) {
-			return 0;
-		}
-		break;
-	case DATA_CMP_UNKNOWN:
-		break;
-	}
-	return 1;
-}
-
-/*!
- * \internal
- * \brief Get an internal node, from the search tree.
- * \param[in] node A node container.
- * \param[in] path The path to the needed internal node.
- * \retval NULL if the internal node is not found.
- * \retval non-NULL the internal node with path 'path'.
- */
-static struct ast_data_search *data_search_get_node(const struct ast_data_search *node,
-	const char *path)
-{
-	char *savepath, *node_name;
-	struct ast_data_search *child, *current = (struct ast_data_search *) node;
-
-	if (!node) {
-		return NULL;
-	}
-
-	savepath = ast_strdupa(path);
-	node_name = next_node_name(&savepath);
-
-	while (node_name) {
-		child = data_search_find(current->children, node_name);
-		if (current != node) {
-			ao2_ref(current, -1);
-		}
-		if (!child) {
-			return NULL;
-		};
-		current = child;
-		node_name = next_node_name(&savepath);
-	}
-
-	return current;
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current string value.
- *        .search = "somename=somestring"
- *        name = "somename"
- *        value is the current value of something and will be evaluated against "somestring".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The value to compare.
- * \returns The strcmp return value.
- */
-static int data_search_cmp_string(const struct ast_data_search *root, const char *name,
-	char *value)
-{
-	struct ast_data_search *child;
-	enum data_search_comparison cmp_type;
-	int ret;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	ret = strcmp(value, child->value);
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(ret, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current pointer address value.
- *        .search = "something=0x32323232"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "0x32323232".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] ptr The pointer address to compare.
- * \returns The (value - current_value) result.
- */
-static int data_search_cmp_ptr(const struct ast_data_search *root, const char *name,
-	void *ptr)
-{
-	struct ast_data_search *child;
-	enum data_search_comparison cmp_type;
-	void *node_ptr;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	cmp_type = child->cmp_type;
-
-	if (sscanf(child->value, "%p", &node_ptr) <= 0) {
-		ao2_ref(child, -1);
-		return 1;
-	}
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result((node_ptr - ptr), cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current ipv4 address value.
- *        .search = "something=192.168.2.2"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "192.168.2.2".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] addr The ipv4 address value to compare.
- * \returns The (value - current_value) result.
- */
-static int data_search_cmp_ipaddr(const struct ast_data_search *root, const char *name,
-	struct in_addr addr)
-{
-	struct ast_data_search *child;
-	enum data_search_comparison cmp_type;
-	struct in_addr node_addr;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-	cmp_type = child->cmp_type;
-
-	inet_aton(child->value, &node_addr);
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result((node_addr.s_addr - addr.s_addr), cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current boolean value.
- *        .search = "something=true"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "true".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The boolean value to compare.
- * \returns The (value - current_value) result.
- */
-static int data_search_cmp_bool(const struct ast_data_search *root, const char *name,
-	unsigned int value)
-{
-	struct ast_data_search *child;
-	unsigned int node_value;
-	enum data_search_comparison cmp_type;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	node_value = abs(ast_true(child->value));
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(value - node_value, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current double value.
- *        .search = "something=222"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "222".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The double value to compare.
- * \returns The (value - current_value) result.
- */
-static int data_search_cmp_dbl(const struct ast_data_search *root, const char *name,
-	double value)
-{
-	struct ast_data_search *child;
-	double node_value;
-	enum data_search_comparison cmp_type;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	node_value = strtod(child->value, NULL);
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(value - node_value, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current unsigned integer value.
- *        .search = "something=10"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "10".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The unsigned value to compare.
- * \returns The strcmp return value.
- */
-static int data_search_cmp_uint(const struct ast_data_search *root, const char *name,
-	unsigned int value)
-{
-	struct ast_data_search *child;
-	unsigned int node_value;
-	enum data_search_comparison cmp_type;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	node_value = atoi(child->value);
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(value - node_value, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current signed integer value.
- *        .search = "something=10"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "10".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The value to compare.
- * \returns The strcmp return value.
- */
-static int data_search_cmp_int(const struct ast_data_search *root, const char *name,
-	int value)
-{
-	struct ast_data_search *child;
-	int node_value;
-	enum data_search_comparison cmp_type;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	node_value = atoi(child->value);
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(value - node_value, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Based on a search tree, evaluate the specified 'name' inside the tree with the
- *        current character value.
- *        .search = "something=c"
- *        name = "something"
- *        value is the current value of something and will be evaluated against "c".
- * \param[in] root The root node pointer of the search tree.
- * \param[in] name The name of the specific.
- * \param[in] value The boolean value to compare.
- * \returns The (value - current_value) result.
- */
-static int data_search_cmp_char(const struct ast_data_search *root, const char *name,
-	char value)
-{
-	struct ast_data_search *child;
-	char node_value;
-	enum data_search_comparison cmp_type;
-
-	child = data_search_get_node(root, name);
-	if (!child) {
-		return 0;
-	}
-
-	node_value = *(child->value);
-	cmp_type = child->cmp_type;
-
-	ao2_ref(child, -1);
-
-	return data_search_comparison_result(value - node_value, cmp_type);
-}
-
-/*!
- * \internal
- * \brief Get the member pointer, from a mapping structure, based on its name.
- * \XXX We will need to improve performance here!!.
- * \retval <0 if the member was not found.
- * \retval >=0 The member position in the mapping structure.
- */
-static inline int data_search_mapping_find(const struct ast_data_mapping_structure *map,
-	size_t mapping_len,
-	const char *member_name)
-{
-	int i;
-
-	for (i = 0; i < mapping_len; i++) {
-		if (!strcmp(map[i].name, member_name)) {
-			return i;
-		}
-	}
-
-	return -1;
-}
-
-int __ast_data_search_cmp_structure(const struct ast_data_search *search,
-	const struct ast_data_mapping_structure *mapping, size_t mapping_len,
-	void *structure, const char *structure_name)
-{
-	struct ao2_iterator i;
-	struct ast_data_search *node, *struct_children;
-	int member, notmatch = 0;
-
-	if (!search) {
-		return 0;
-	}
-
-	struct_children = data_search_get_node(search, structure_name);
-	if (!struct_children) {
-		return 0;
-	}
-
-	i = ao2_iterator_init(struct_children->children, 0);
-	while ((node = ao2_iterator_next(&i))) {
-		member = data_search_mapping_find(mapping, mapping_len, node->name);
-		if (member < 0) {
-			/* the structure member name doesn't match! */
-			ao2_ref(node, -1);
-			ao2_ref(struct_children, -1);
-			ao2_iterator_destroy(&i);
-			return 0;
-		}
-
-		notmatch = 0;
-		switch (mapping[member].type) {
-		case AST_DATA_PASSWORD:
-			notmatch = data_search_cmp_string(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_PASSWORD(structure));
-			break;
-		case AST_DATA_TIMESTAMP:
-			notmatch = data_search_cmp_uint(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_TIMESTAMP(structure));
-			break;
-		case AST_DATA_SECONDS:
-			notmatch = data_search_cmp_uint(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_SECONDS(structure));
-			break;
-		case AST_DATA_MILLISECONDS:
-			notmatch = data_search_cmp_uint(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_MILLISECONDS(structure));
-			break;
-		case AST_DATA_STRING:
-			notmatch = data_search_cmp_string(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_STRING(structure));
-			break;
-		case AST_DATA_CHARACTER:
-			notmatch = data_search_cmp_char(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_CHARACTER(structure));
-			break;
-		case AST_DATA_INTEGER:
-			notmatch = data_search_cmp_int(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_INTEGER(structure));
-			break;
-		case AST_DATA_BOOLEAN:
-			notmatch = data_search_cmp_bool(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_BOOLEAN(structure));
-			break;
-		case AST_DATA_UNSIGNED_INTEGER:
-			notmatch = data_search_cmp_uint(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_UNSIGNED_INTEGER(structure));
-			break;
-		case AST_DATA_DOUBLE:
-			notmatch = data_search_cmp_dbl(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_DOUBLE(structure));
-			break;
-		case AST_DATA_IPADDR:
-			notmatch = data_search_cmp_ipaddr(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_IPADDR(structure));
-			break;
-		case AST_DATA_POINTER:
-			notmatch = data_search_cmp_ptr(struct_children,
-				node->name,
-				mapping[member].get.AST_DATA_POINTER(structure));
-			break;
-		case AST_DATA_CONTAINER:
-			break;
-		}
-
-		ao2_ref(node, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-	ao2_ref(struct_children, -1);
-
-	return notmatch;
-}
-
-/*!
- * \internal
- * \brief Release the memory allocated by a call to ao2_alloc.
- */
-static void data_result_destructor(void *obj)
-{
-	struct ast_data *root = obj;
-
-	switch (root->type) {
-	case AST_DATA_PASSWORD:
-	case AST_DATA_STRING:
-		ast_free(root->payload.str);
-		ao2_ref(root->children, -1);
-		break;
-	case AST_DATA_POINTER:
-	case AST_DATA_CHARACTER:
-	case AST_DATA_CONTAINER:
-	case AST_DATA_INTEGER:
-	case AST_DATA_TIMESTAMP:
-	case AST_DATA_SECONDS:
-	case AST_DATA_MILLISECONDS:
-	case AST_DATA_UNSIGNED_INTEGER:
-	case AST_DATA_DOUBLE:
-	case AST_DATA_BOOLEAN:
-	case AST_DATA_IPADDR:
-		ao2_ref(root->children, -1);
-		break;
-	}
-}
-
-static struct ast_data *data_result_create(const char *name)
-{
-	struct ast_data *res;
-	size_t namelen;
-
-	namelen = ast_strlen_zero(name) ? 1 : strlen(name) + 1;
-
-	res = ao2_alloc(sizeof(*res) + namelen, data_result_destructor);
-	if (!res) {
-		return NULL;
-	}
-
-	strcpy(res->name, namelen ? name : "");
-
-	/* initialize the children container */
-	res->children = ao2_container_alloc(NUM_DATA_RESULT_BUCKETS, data_result_hash,
-		data_result_cmp);
-	if (!res->children) {
-		ao2_ref(res, -1);
-		return NULL;
-	}
-
-	/* set this node as a container. */
-	res->type = AST_DATA_CONTAINER;
-
-	return res;
-}
-
-/*!
- * \internal
- * \brief Find a child node, based on its name.
- * \param[in] root The starting point.
- * \param[in] name The child name.
- * \retval NULL if the node wasn't found.
- * \retval non-NULL the node we were looking for.
- */
-static struct ast_data *data_result_find_child(struct ast_data *root, const char *name)
-{
-	struct ast_data *found, *find_node;
-
-	find_node = data_result_create(name);
-	if (!find_node) {
-		return NULL;
-	}
-
-	found = ao2_find(root->children, find_node, OBJ_POINTER);
-
-	/* release the temporary created node used for searching. */
-	ao2_ref(find_node, -1);
-
-	return found;
-}
-
-int ast_data_search_match(const struct ast_data_search *search, struct ast_data *data)
-{
-	struct ao2_iterator i, ii;
-	struct ast_data_search *s, *s_child;
-	struct ast_data *d_child;
-	int notmatch = 1;
-
-	if (!search) {
-		return 1;
-	}
-
-	s_child = data_search_find(search->children, data->name);
-	if (!s_child) {
-		/* nothing to compare */
-		ao2_ref(s_child, -1);
-		return 1;
-	}
-
-	i = ao2_iterator_init(s_child->children, 0);
-	while ((s = ao2_iterator_next(&i))) {
-		if (!ao2_container_count(s->children)) {
-			/* compare this search node with every data node */
-			d_child = data_result_find_child(data, s->name);
-			if (!d_child) {
-				ao2_ref(s, -1);
-				notmatch = 1;
-				continue;
-			}
-
-			switch (d_child->type) {
-			case AST_DATA_PASSWORD:
-			case AST_DATA_STRING:
-				notmatch = data_search_cmp_string(s_child, d_child->name,
-					d_child->payload.str);
-				break;
-			case AST_DATA_CHARACTER:
-				notmatch = data_search_cmp_char(s_child, d_child->name,
-					d_child->payload.character);
-				break;
-			case AST_DATA_INTEGER:
-				notmatch = data_search_cmp_int(s_child, d_child->name,
-					d_child->payload.sint);
-				break;
-			case AST_DATA_BOOLEAN:
-				notmatch = data_search_cmp_bool(s_child, d_child->name,
-					d_child->payload.boolean);
-				break;
-			case AST_DATA_UNSIGNED_INTEGER:
-				notmatch = data_search_cmp_uint(s_child, d_child->name,
-					d_child->payload.uint);
-				break;
-			case AST_DATA_TIMESTAMP:
-			case AST_DATA_SECONDS:
-			case AST_DATA_MILLISECONDS:
-			case AST_DATA_DOUBLE:
-				notmatch = data_search_cmp_uint(s_child, d_child->name,
-					d_child->payload.dbl);
-				break;
-			case AST_DATA_IPADDR:
-				notmatch = data_search_cmp_ipaddr(s_child, d_child->name,
-					d_child->payload.ipaddr);
-				break;
-			case AST_DATA_POINTER:
-				notmatch = data_search_cmp_ptr(s_child, d_child->name,
-					d_child->payload.ptr);
-				break;
-			case AST_DATA_CONTAINER:
-				break;
-			}
-			ao2_ref(d_child, -1);
-		} else {
-			ii = ao2_iterator_init(data->children, 0);
-			while ((d_child = ao2_iterator_next(&ii))) {
-				if (strcmp(d_child->name, s->name)) {
-					ao2_ref(d_child, -1);
-					continue;
-				}
-				if (!(notmatch = !ast_data_search_match(s_child, d_child))) {
-					/* do not continue if we have a match. */
-					ao2_ref(d_child, -1);
-					break;
-				}
-				ao2_ref(d_child, -1);
-			}
-			ao2_iterator_destroy(&ii);
-		}
-		ao2_ref(s, -1);
-		if (notmatch) {
-			/* do not continue if we don't have a match. */
-			break;
-		}
-	}
-	ao2_iterator_destroy(&i);
-
-	ao2_ref(s_child, -1);
-
-	return !notmatch;
-}
-
-/*!
- * \internal
- * \brief Get an internal node, from the result set.
- * \param[in] node A node container.
- * \param[in] path The path to the needed internal node.
- * \retval NULL if the internal node is not found.
- * \retval non-NULL the internal node with path 'path'.
- */
-static struct ast_data *data_result_get_node(struct ast_data *node,
-	const char *path)
-{
-	char *savepath, *node_name;
-	struct ast_data *child, *current = node;
-
-	savepath = ast_strdupa(path);
-	node_name = next_node_name(&savepath);
-
-	while (node_name) {
-		child = data_result_find_child(current, node_name);
-		if (current != node) {
-			ao2_ref(current, -1);
-		}
-		if (!child) {
-			return NULL;
-		}
-		current = child;
-		node_name = next_node_name(&savepath);
-	}
-
-	/* do not increment the refcount of the returned object. */
-	if (current != node) {
-		ao2_ref(current, -1);
-	}
-
-	return current;
-}
-
-/*!
- * \internal
- * \brief Add a child to the specified root node.
- * \param[in] root The root node pointer.
- * \param[in] child The child to add to the root node.
- */
-static void data_result_add_child(struct ast_data *root, struct ast_data *child)
-{
-	ao2_link(root->children, child);
-}
-
-/*!
- * \internal
- * \brief Common string hash function for data nodes
- */
-static int data_filter_hash(const void *obj, const int flags)
-{
-	const struct data_filter *node = obj;
-	return ast_str_hash(node->name);
-}
-
-/*!
- * \internal
- * \brief Common string comparison function
- */
-static int data_filter_cmp(void *obj, void *arg, int flags)
-{
-	struct data_filter *node1 = obj, *node2 = arg;
-	return strcasecmp(node1->name, node2->name) ? 0 : CMP_MATCH;
-}
-
-/*!
- * \internal
- * \brief Destroy a data filter tree.
- * \param[in] obj Data filter list to be destroyed.
- */
-static void data_filter_destructor(void *obj)
-{
-	struct data_filter *filter = obj, *globres;
-
-	while ((globres = AST_LIST_REMOVE_HEAD(&(filter->glob_list), list))) {
-		ao2_ref(globres, -1);
-	}
-
-	ao2_ref(filter->children, -1);
-}
-
-/*!
- * \internal
- * \brief Allocate a filter node.
- * \retval NULL on error.
- * \retval non-NULL The allocated search node structure.
- */
-static struct data_filter *data_filter_alloc(const char *name)
-{
-	char *globname, *token;
-	struct data_filter *res, *globfilter;
-	size_t name_len = strlen(name) + 1;
-
-	res = ao2_alloc(sizeof(*res) + name_len, data_filter_destructor);
-	if (!res) {
-		return NULL;
-	}
-
-	res->children = ao2_container_alloc(NUM_DATA_FILTER_BUCKETS, data_filter_hash,
-		data_filter_cmp);
-
-	if (!res->children) {
-		ao2_ref(res, -1);
-		return NULL;
-	}
-
-	strcpy(res->name, name);
-
-	if (strchr(res->name, '*')) {
-		globname = ast_strdupa(res->name);
-
-		while ((token = strsep(&globname, "*"))) {
-			globfilter = data_filter_alloc(token);
-			AST_LIST_INSERT_TAIL(&(res->glob_list), globfilter, list);
-		}
-	}
-
-	return res;
-}
-
-/*!
- * \internal
- * \brief Release a filter tree.
- * \param[in] filter The filter tree root node.
- */
-static void data_filter_release(struct data_filter *filter)
-{
-	ao2_ref(filter, -1);
-}
-
-/*!
- * \internal
- * \brief Find a child node, based on his name.
- * \param[in] parent Where to find the node.
- * \param[in] name The node name to find.
- * \retval NULL if a node wasn't found.
- * \retval The node found.
- * \note Remember to decrement the ref count of the returned node after using it.
- */
-static struct data_filter *data_filter_find(struct ao2_container *parent,
-	const char *name)
-{
-	int i, olend, orend, globfound;
-	size_t name_len = strlen(name), glob_len;
-	struct ao2_iterator iter;
-	struct data_filter *find_node, *found, *globres;
-
-	find_node = data_filter_alloc(name);
-	if (!find_node) {
-		return NULL;
-	}
-
-	found = ao2_find(parent, find_node, OBJ_POINTER);
-
-	/* free the created node used for searching. */
-	ao2_ref(find_node, -1);
-
-	if (found) {
-		return found;
-	}
-
-	iter = ao2_iterator_init(parent, 0);
-	while ((found = ao2_iterator_next(&iter))) {
-		if (!AST_LIST_EMPTY(&(found->glob_list))) {
-			i = 0;
-			globfound = 1;
-
-			olend = ast_strlen_zero(AST_LIST_FIRST(&(found->glob_list))->name);
-			orend = ast_strlen_zero(AST_LIST_LAST(&(found->glob_list))->name);
-
-			AST_LIST_TRAVERSE(&(found->glob_list), globres, list) {
-				if (!*globres->name) {
-					continue;
-				}
-
-				glob_len = strlen(globres->name);
-
-				if (!i && !olend) {
-					if (strncasecmp(name, globres->name, glob_len)) {
-						globfound = 0;
-						break;
-					}
-
-					i += glob_len;
-					continue;
-				}
-
-				for (globfound = 0; name_len - i >= glob_len; ++i) {
-					if (!strncasecmp(name + i, globres->name, glob_len)) {
-						globfound = 1;
-						i += glob_len;
-						break;
-					}
-				}
-
-				if (!globfound) {
-					break;
-				}
-			}
-
-			if (globfound && (i == name_len || orend)) {
-				ao2_iterator_destroy(&iter);
-				return found;
-			}
-		}
-
-		ao2_ref(found, -1);
-	}
-	ao2_iterator_destroy(&iter);
-
-	return NULL;
-}
-
-/*!
- * \internal
- * \brief Add a child to the specified node.
- * \param[in] root The root node where to add the child.
- * \param[in] name The name of the node to add.
- * \note Remember to decrement the ref count after using the returned node.
- */
-static struct data_filter *data_filter_add_child(struct ao2_container *root,
-	char *name)
-{
-	struct data_filter *node;
-
-	node = data_filter_find(root, name);
-	if (node) {
-		return node;
-	}
-
-	node = data_filter_alloc(name);
-	if (!node) {
-		return NULL;
-	}
-
-	ao2_link(root, node);
-
-	return node;
-}
-
-/*!
- * \internal
- * \brief Add a node to a filter list from a path
- * \param[in] Filter list to add the path onto.
- * \param[in] The path to add into the filter list.
- * \retval NULL on error.
- * \retval non-NULL A tree with the wanted nodes.
- */
-static int data_filter_add_nodes(struct ao2_container *root, char *path)
-{
-	struct data_filter *node;
-	char *savepath, *saveptr, *token, *node_name;
-	int ret = 0;
-
-	if (!path) {
-		return 0;
-	}
-
-	savepath = ast_strdupa(path);
-
-	node_name = next_node_name(&savepath);
-
-	if (!node_name) {
-		return 0;
-	}
-
-	for (token = strtok_r(node_name, "|", &saveptr);
-			token; token = strtok_r(NULL, "|", &saveptr)) {
-		node = data_filter_add_child(root, token);
-		if (!node) {
-			continue;
-		}
-		data_filter_add_nodes(node->children, savepath);
-		ret = 1;
-		ao2_ref(node, -1);
-	}
-
-	return ret;
-}
-
-/*!
- * \internal
- * \brief Generate a filter list based on a filter string provided by the API user.
- * \param[in] A filter string to create a filter from.
- */
-static struct data_filter *data_filter_generate(const char *constfilter)
-{
-	struct data_filter *filter = NULL;
-	char *strfilter, *token, *saveptr;
-	int node_added = 0;
-
-	if (!constfilter) {
-		return NULL;
-	}
-
-	strfilter = ast_strdupa(constfilter);
-
-	filter = data_filter_alloc("/");
-	if (!filter) {
-		return NULL;
-	}
-
-	for (token = strtok_r(strfilter, ",", &saveptr); token;
-			token = strtok_r(NULL, ",", &saveptr)) {
-		node_added = data_filter_add_nodes(filter->children, token);
-	}
-
-	if (!node_added) {
-		ao2_ref(filter, -1);
-		return NULL;
-	}
-
-	return filter;
-}
-
-/*!
- * \internal
- * \brief Generate all the tree from a specified provider.
- * \param[in] query The query executed.
- * \param[in] root_provider The provider specified in the path of the query.
- * \param[in] parent_node_name The root node name.
- * \retval NULL on error.
- * \retval non-NULL The generated result tree.
- */
-static struct ast_data *data_result_generate_node(const struct ast_data_query *query,
-	const struct data_provider *root_provider,
-	const char *parent_node_name,
-	const struct ast_data_search *search,
-	const struct data_filter *filter)
-{
-	struct ast_data *generated, *node;
-	struct ao2_iterator i;
-	struct data_provider *provider;
-	struct ast_data_search *search_child = NULL;
-	struct data_filter *filter_child;
-
-	node = data_result_create(parent_node_name);
-	if (!node) {
-		ast_log(LOG_ERROR, "Unable to allocate '%s' node\n", parent_node_name);
-		return NULL;
-	}
-
-	if (root_provider->module) {
-		ast_module_ref(root_provider->module);
-	}
-
-	/* if this is a terminal node, just run the callback function. */
-	if (root_provider->handler && root_provider->handler->get) {
-		node->filter = filter;
-		root_provider->handler->get(search, node);
-		if (root_provider->module) {
-			ast_module_unref(root_provider->module);
-		}
-		return node;
-	}
-
-	if (root_provider->module) {
-		ast_module_unref(root_provider->module);
-	}
-
-	/* if this is not a terminal node, generate every child node. */
-	i = ao2_iterator_init(root_provider->children, 0);
-	while ((provider = ao2_iterator_next(&i))) {
-		filter_child = NULL;
-		generated = NULL;
-
-		/* get the internal search node. */
-		if (search) {
-			search_child = data_search_find(search->children, provider->name);
-		}
-		/* get the internal filter node. */
-		if (filter) {
-			filter_child = data_filter_find(filter->children, provider->name);
-		}
-
-		if (!filter || filter_child) {
-			/* only generate the internal node, if we have something to
-			 * generate based on the filtering string. */
-			generated = data_result_generate_node(query, provider,
-				provider->name,
-				search_child, filter_child);
-		}
-
-		/* decrement the refcount of the internal search node. */
-		if (search_child) {
-			ao2_ref(search_child, -1);
-		}
-
-		/* decrement the refcount of the internal filter node. */
-		if (filter_child) {
-			ao2_ref(filter_child, -1);
-		}
-
-		if (generated) {
-			data_result_add_child(node, generated);
-			ao2_ref(generated, -1);
-		}
-
-		ao2_ref(provider, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-	return node;
-}
-
-/*!
- * \internal
- * \brief Generate a result tree based on a query.
- * \param[in] query The complete query structure.
- * \param[in] search_path The path to retrieve.
- * \retval NULL on error.
- * \retval non-NULL The generated data result.
- */
-static struct ast_data *data_result_generate(const struct ast_data_query *query,
-	const char *search_path)
-{
-	char *node_name, *tmp_path;
-	struct data_provider *provider_child, *tmp_provider_child;
-	struct ast_data *result, *result_filtered;
-	struct ast_data_search *search = NULL, *search_child = NULL;
-	struct data_filter *filter = NULL, *filter_child = NULL;
-
-	if (!search_path) {
-		/* generate all the trees?. */
-		return NULL;
-	}
-
-	tmp_path = ast_strdupa(search_path);
-
-	/* start searching the root node name */
-	node_name = next_node_name(&tmp_path);
-	if (!node_name) {
-		return NULL;
-	}
-	provider_child = data_provider_find(root_data.container, node_name, NULL);
-
-	/* continue with the rest of the path. */
-	while (provider_child) {
-		node_name = next_node_name(&tmp_path);
-		if (!node_name) {
-			break;
-		}
-
-		tmp_provider_child = data_provider_find(provider_child->children,
-				node_name, NULL);
-
-		/* release the reference from this child */
-		ao2_ref(provider_child, -1);
-
-		provider_child = tmp_provider_child;
-	}
-
-	if (!provider_child) {
-		ast_log(LOG_ERROR, "Invalid path '%s', '%s' not found.\n",
-				tmp_path, node_name);
-		return NULL;
-	}
-
-	/* generate the search tree. */
-	if (query->search) {
-		search = data_search_generate(query->search);
-		if (search) {
-			search_child = data_search_find(search->children,
-				provider_child->name);
-		}
-	}
-
-	/* generate the filter tree. */
-	if (query->filter) {
-		filter = data_filter_generate(query->filter);
-		if (filter) {
-			filter_child = data_filter_find(filter->children,
-				provider_child->name);
-		}
-	}
-
-	result = data_result_generate_node(query, provider_child, provider_child->name,
-			search_child, filter_child);
-
-	/* release the requested provider. */
-	ao2_ref(provider_child, -1);
-
-	/* release the generated search tree. */
-	if (search_child) {
-		ao2_ref(search_child, -1);
-	}
-
-	if (filter_child) {
-		ao2_ref(filter_child, -1);
-	}
-
-	if (search) {
-		data_search_release(search);
-	}
-
-	result_filtered = result;
-
-	/* release the generated filter tree. */
-	if (filter) {
-		data_filter_release(filter);
-	}
-
-	return result_filtered;
-}
-
-struct ast_data *ast_data_get(const struct ast_data_query *query)
-{
-	struct ast_data *res;
-
-	/* check compatibility */
-	if (!data_structure_compatible(query->version, latest_query_compatible_version,
-		current_query_version)) {
-		return NULL;
-	}
-
-	data_read_lock();
-	res = data_result_generate(query, query->path);
-	data_unlock();
-
-	if (!res) {
-		ast_log(LOG_ERROR, "Unable to get data from %s\n", query->path);
-		return NULL;
-	}
-
-	return res;
-}
-
-#ifdef HAVE_LIBXML2
-/*!
- * \internal
- * \brief Helper function to move an ast_data tree to xml.
- * \param[in] parent_data The initial ast_data node to be passed to xml.
- * \param[out] parent_xml The root node to insert the xml.
- */
-static void data_get_xml_add_child(struct ast_data *parent_data,
-	struct ast_xml_node *parent_xml)
-{
-	struct ao2_iterator i;
-	struct ast_data *node;
-	struct ast_xml_node *child_xml;
-	char node_content[256];
-
-	i = ao2_iterator_init(parent_data->children, 0);
-	while ((node = ao2_iterator_next(&i))) {
-		child_xml = ast_xml_new_node(node->name);
-		if (!child_xml) {
-			ao2_ref(node, -1);
-			continue;
-		}
-
-		switch (node->type) {
-		case AST_DATA_CONTAINER:
-			data_get_xml_add_child(node, child_xml);
-			break;
-		case AST_DATA_PASSWORD:
-			ast_xml_set_text(child_xml, node->payload.str);
-			break;
-		case AST_DATA_TIMESTAMP:
-			snprintf(node_content, sizeof(node_content), "%u",
-				node->payload.uint);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_SECONDS:
-			snprintf(node_content, sizeof(node_content), "%u",
-				node->payload.uint);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_MILLISECONDS:
-			snprintf(node_content, sizeof(node_content), "%u",
-				node->payload.uint);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_STRING:
-			ast_xml_set_text(child_xml, node->payload.str);
-			break;
-		case AST_DATA_CHARACTER:
-			snprintf(node_content, sizeof(node_content), "%c",
-				node->payload.character);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_INTEGER:
-			snprintf(node_content, sizeof(node_content), "%d",
-				node->payload.sint);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_UNSIGNED_INTEGER:
-			snprintf(node_content, sizeof(node_content), "%u",
-				node->payload.uint);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_DOUBLE:
-			snprintf(node_content, sizeof(node_content), "%f",
-				node->payload.dbl);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_BOOLEAN:
-			if (node->payload.boolean) {
-				ast_xml_set_text(child_xml, "true");
-			} else {
-				ast_xml_set_text(child_xml, "false");
-			}
-			break;
-		case AST_DATA_POINTER:
-			snprintf(node_content, sizeof(node_content), "%p",
-				node->payload.ptr);
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		case AST_DATA_IPADDR:
-			snprintf(node_content, sizeof(node_content), "%s",
-				ast_inet_ntoa(node->payload.ipaddr));
-			ast_xml_set_text(child_xml, node_content);
-			break;
-		}
-		ast_xml_add_child(parent_xml, child_xml);
-
-		ao2_ref(node, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-}
-
-struct ast_xml_doc *ast_data_get_xml(const struct ast_data_query *query)
-{
-	struct ast_xml_doc *doc;
-	struct ast_xml_node *root;
-	struct ast_data *res;
-
-	res = ast_data_get(query);
-	if (!res) {
-		return NULL;
-	}
-
-	doc = ast_xml_new();
-	if (!doc) {
-		ast_data_free(res);
-		return NULL;
-	}
-
-	root = ast_xml_new_node(res->name);
-	if (!root) {
-		ast_xml_close(doc);
-	}
-
-	ast_xml_set_root(doc, root);
-
-	data_get_xml_add_child(res, root);
-
-	ast_data_free(res);
-
-	return doc;
-}
-#endif
-
-enum ast_data_type ast_data_retrieve_type(struct ast_data *node, const char *path)
-{
-	struct ast_data *internal;
-
-	internal = data_result_get_node(node, path);
-	if (!internal) {
-		return -1;
-	}
-
-	return internal->type;
-}
-
-char *ast_data_retrieve_name(struct ast_data *node)
-{
-	return node->name;
-}
-
-/*!
- * \internal
- * \brief Insert a child node inside a passed parent node.
- * \param root Where we are going to insert the child node.
- * \param name The name of the child node to add.
- * \param type The type of content inside the child node.
- * \param ptr The actual content of the child node.
- * \retval NULL on error.
- * \retval non-NULL The added child node pointer.
- */
-static struct ast_data *__ast_data_add(struct ast_data *root, const char *name,
-	enum ast_data_type type, void *ptr)
-{
-	struct ast_data *node;
-	struct data_filter *filter, *filter_child = NULL;
-
-	if (!root || !root->children) {
-		/* invalid data result node. */
-		return NULL;
-	}
-
-	/* check if we need to add this node, based on the filter. */
-	if (root->filter) {
-		filter = data_filter_find(root->filter->children, name);
-		if (!filter) {
-			return NULL;
-		}
-		ao2_ref(filter, -1);
-	}
-
-	node = data_result_create(name);
-	if (!node) {
-		return NULL;
-	}
-
-	node->type = type;
-
-	switch (type) {
-	case AST_DATA_BOOLEAN:
-		node->payload.boolean = *(unsigned int *) ptr;
-		break;
-	case AST_DATA_INTEGER:
-		node->payload.sint = *(int *) ptr;
-		break;
-	case AST_DATA_TIMESTAMP:
-	case AST_DATA_SECONDS:
-	case AST_DATA_MILLISECONDS:
-	case AST_DATA_UNSIGNED_INTEGER:
-		node->payload.uint = *(unsigned int *) ptr;
-		break;
-	case AST_DATA_DOUBLE:
-		node->payload.dbl = *(double *) ptr;
-		break;
-	case AST_DATA_PASSWORD:
-	case AST_DATA_STRING:
-		node->payload.str = (char *) ptr;
-		break;
-	case AST_DATA_CHARACTER:
-		node->payload.character = *(char *) ptr;
-		break;
-	case AST_DATA_POINTER:
-		node->payload.ptr = ptr;
-		break;
-	case AST_DATA_IPADDR:
-		node->payload.ipaddr = *(struct in_addr *) ptr;
-		break;
-	case AST_DATA_CONTAINER:
-		if (root->filter) {
-			filter_child = data_filter_find(root->filter->children, name);
-			if (filter_child) {
-				/* do not increment the refcount because it is not neccesary. */
-				ao2_ref(filter_child, -1);
-			}
-		}
-		node->filter = filter_child;
-		break;
-	default:
-		break;
-	}
-
-	data_result_add_child(root, node);
-
-	ao2_ref(node, -1);
-
-	return node;
-}
-
-struct ast_data *ast_data_add_node(struct ast_data *root, const char *name)
-{
-	return __ast_data_add(root, name, AST_DATA_CONTAINER, NULL);
-}
-
-struct ast_data *ast_data_add_int(struct ast_data *root, const char *name, int value)
-{
-	return __ast_data_add(root, name, AST_DATA_INTEGER, &value);
-}
-
-struct ast_data *ast_data_add_char(struct ast_data *root, const char *name, char value)
-{
-	return __ast_data_add(root, name, AST_DATA_CHARACTER, &value);
-}
-
-struct ast_data *ast_data_add_uint(struct ast_data *root, const char *name,
-	unsigned int value)
-{
-	return __ast_data_add(root, name, AST_DATA_UNSIGNED_INTEGER, &value);
-}
-
-struct ast_data *ast_data_add_dbl(struct ast_data *root, const char *childname,
-	double dbl)
-{
-	return __ast_data_add(root, childname, AST_DATA_DOUBLE, &dbl);
-}
-
-struct ast_data *ast_data_add_bool(struct ast_data *root, const char *childname,
-	unsigned int boolean)
-{
-	return __ast_data_add(root, childname, AST_DATA_BOOLEAN, &boolean);
-}
-
-struct ast_data *ast_data_add_ipaddr(struct ast_data *root, const char *childname,
-	struct in_addr addr)
-{
-	return __ast_data_add(root, childname, AST_DATA_IPADDR, &addr);
-}
-
-struct ast_data *ast_data_add_ptr(struct ast_data *root, const char *childname,
-	void *ptr)
-{
-	return __ast_data_add(root, childname, AST_DATA_POINTER, ptr);
-}
-
-struct ast_data *ast_data_add_timestamp(struct ast_data *root, const char *childname,
-	unsigned int timestamp)
-{
-	return __ast_data_add(root, childname, AST_DATA_TIMESTAMP, &timestamp);
-}
-
-struct ast_data *ast_data_add_seconds(struct ast_data *root, const char *childname,
-	unsigned int seconds)
-{
-	return __ast_data_add(root, childname, AST_DATA_SECONDS, &seconds);
-}
-
-struct ast_data *ast_data_add_milliseconds(struct ast_data *root, const char *childname,
-	unsigned int milliseconds)
-{
-	return __ast_data_add(root, childname, AST_DATA_MILLISECONDS, &milliseconds);
-}
-
-struct ast_data *ast_data_add_password(struct ast_data *root, const char *childname,
-	const char *value)
-{
-	char *name;
-	size_t namelen = 1 + (ast_strlen_zero(value) ? 0 : strlen(value));
-	struct ast_data *res;
-
-	if (!(name = ast_malloc(namelen))) {
-		return NULL;
-	}
-
-	strcpy(name, (ast_strlen_zero(value) ? "" : value));
-
-	res = __ast_data_add(root, childname, AST_DATA_PASSWORD, name);
-	if (!res) {
-		ast_free(name);
-	}
-
-	return res;
-}
-
-struct ast_data *ast_data_add_str(struct ast_data *root, const char *childname,
-	const char *value)
-{
-	char *name;
-	size_t namelen = 1 + (ast_strlen_zero(value) ? 0 : strlen(value));
-	struct ast_data *res;
-
-	if (!(name = ast_malloc(namelen))) {
-		return NULL;
-	}
-
-	strcpy(name, (ast_strlen_zero(value) ? "" : value));
-
-	res = __ast_data_add(root, childname, AST_DATA_STRING, name);
-	if (!res) {
-		ast_free(name);
-	}
-
-	return res;
-}
-
-int __ast_data_add_structure(struct ast_data *root,
-	const struct ast_data_mapping_structure *mapping, size_t mapping_len,
-	void *structure)
-{
-	int i;
-
-	for (i = 0; i < mapping_len; i++) {
-		switch (mapping[i].type) {
-		case AST_DATA_INTEGER:
-			ast_data_add_int(root, mapping[i].name,
-				mapping[i].get.AST_DATA_INTEGER(structure));
-			break;
-		case AST_DATA_UNSIGNED_INTEGER:
-			ast_data_add_uint(root, mapping[i].name,
-				mapping[i].get.AST_DATA_UNSIGNED_INTEGER(structure));
-			break;
-		case AST_DATA_DOUBLE:
-			ast_data_add_dbl(root, mapping[i].name,
-				mapping[i].get.AST_DATA_DOUBLE(structure));
-			break;
-		case AST_DATA_BOOLEAN:
-			ast_data_add_bool(root, mapping[i].name,
-				mapping[i].get.AST_DATA_BOOLEAN(structure));
-			break;
-		case AST_DATA_PASSWORD:
-			ast_data_add_password(root, mapping[i].name,
-				mapping[i].get.AST_DATA_PASSWORD(structure));
-			break;
-		case AST_DATA_TIMESTAMP:
-			ast_data_add_timestamp(root, mapping[i].name,
-				mapping[i].get.AST_DATA_TIMESTAMP(structure));
-			break;
-		case AST_DATA_SECONDS:
-			ast_data_add_seconds(root, mapping[i].name,
-				mapping[i].get.AST_DATA_SECONDS(structure));
-			break;
-		case AST_DATA_MILLISECONDS:
-			ast_data_add_milliseconds(root, mapping[i].name,
-				mapping[i].get.AST_DATA_MILLISECONDS(structure));
-			break;
-		case AST_DATA_STRING:
-			ast_data_add_str(root, mapping[i].name,
-				mapping[i].get.AST_DATA_STRING(structure));
-			break;
-		case AST_DATA_CHARACTER:
-			ast_data_add_char(root, mapping[i].name,
-				mapping[i].get.AST_DATA_CHARACTER(structure));
-			break;
-		case AST_DATA_CONTAINER:
-			break;
-		case AST_DATA_IPADDR:
-			ast_data_add_ipaddr(root, mapping[i].name,
-				mapping[i].get.AST_DATA_IPADDR(structure));
-			break;
-		case AST_DATA_POINTER:
-			ast_data_add_ptr(root, mapping[i].name,
-				mapping[i].get.AST_DATA_POINTER(structure));
-			break;
-		}
-	}
-
-	return 0;
-}
-
-void ast_data_remove_node(struct ast_data *root, struct ast_data *child)
-{
-	ao2_unlink(root->children, child);
-}
-
-void ast_data_free(struct ast_data *root)
-{
-	/* destroy it, this will destroy all the internal nodes. */
-	ao2_ref(root, -1);
-}
-
-struct ast_data_iterator *ast_data_iterator_init(struct ast_data *tree,
-	const char *elements)
-{
-	struct ast_data_iterator *iterator;
-	struct ao2_iterator i;
-	struct ast_data *internal = tree;
-	char *path, *ptr = NULL;
-
-	if (!elements) {
-		return NULL;
-	}
-
-	/* tree is the node we want to use to iterate? or we are going
-	 * to iterate thow an internal node? */
-	path = ast_strdupa(elements);
-
-	ptr = strrchr(path, '/');
-	if (ptr) {
-		*ptr = '\0';
-		internal = data_result_get_node(tree, path);
-		if (!internal) {
-			return NULL;
-		}
-	}
-
-	iterator = ast_calloc(1, sizeof(*iterator));
-	if (!iterator) {
-		return NULL;
-	}
-
-	i = ao2_iterator_init(internal->children, 0);
-
-	iterator->pattern = (ptr ? strrchr(elements, '/') + 1 : elements);
-
-	/* is the last node a regular expression?, compile it! */
-	if (!regcomp(&(iterator->regex_pattern), iterator->pattern,
-			REG_EXTENDED | REG_NOSUB | REG_ICASE)) {
-		iterator->is_pattern = 1;
-	}
-
-	iterator->internal_iterator = i;
-
-	return iterator;
-}
-
-void ast_data_iterator_end(struct ast_data_iterator *iterator)
-{
-	/* decrement the reference counter. */
-	if (iterator->last) {
-		ao2_ref(iterator->last, -1);
-	}
-
-	/* release the generated pattern. */
-	if (iterator->is_pattern) {
-		regfree(&(iterator->regex_pattern));
-	}
-
-	ao2_iterator_destroy(&(iterator->internal_iterator));
-
-	ast_free(iterator);
-	iterator = NULL;
-}
-
-struct ast_data *ast_data_iterator_next(struct ast_data_iterator *iterator)
-{
-	struct ast_data *res;
-
-	if (iterator->last) {
-		/* release the last retrieved node reference. */
-		ao2_ref(iterator->last, -1);
-	}
-
-	while ((res = ao2_iterator_next(&iterator->internal_iterator))) {
-		/* if there is no node name pattern specified, return
-		 * the next node. */
-		if (!iterator->pattern) {
-			break;
-		}
-
-		/* if the pattern is a regular expression, check if this node
-		 * matches. */
-		if (iterator->is_pattern && !regexec(&(iterator->regex_pattern),
-			res->name, 0, NULL, 0)) {
-			break;
-		}
-
-		/* if there is a pattern specified, check if this node matches
-		 * the wanted node names. */
-		if (!iterator->is_pattern && (iterator->pattern &&
-				!strcasecmp(res->name, iterator->pattern))) {
-			break;
-		}
-
-		ao2_ref(res, -1);
-	}
-
-	iterator->last = res;
-
-	return res;
-}
-
-int ast_data_retrieve(struct ast_data *tree, const char *path,
-	struct ast_data_retrieve *content)
-{
-	struct ast_data *node;
-
-	if (!content) {
-		return -1;
-	}
-
-	node = data_result_get_node(tree, path);
-	if (!node) {
-		ast_log(LOG_ERROR, "Invalid internal node %s\n", path);
-		return -1;
-	}
-
-	content->type = node->type;
-	switch (node->type) {
-	case AST_DATA_STRING:
-		content->value.AST_DATA_STRING = node->payload.str;
-		break;
-	case AST_DATA_PASSWORD:
-		content->value.AST_DATA_PASSWORD = node->payload.str;
-		break;
-	case AST_DATA_TIMESTAMP:
-		content->value.AST_DATA_TIMESTAMP = node->payload.uint;
-		break;
-	case AST_DATA_SECONDS:
-		content->value.AST_DATA_SECONDS = node->payload.uint;
-		break;
-	case AST_DATA_MILLISECONDS:
-		content->value.AST_DATA_MILLISECONDS = node->payload.uint;
-		break;
-	case AST_DATA_CHARACTER:
-		content->value.AST_DATA_CHARACTER = node->payload.character;
-		break;
-	case AST_DATA_INTEGER:
-		content->value.AST_DATA_INTEGER = node->payload.sint;
-		break;
-	case AST_DATA_UNSIGNED_INTEGER:
-		content->value.AST_DATA_UNSIGNED_INTEGER = node->payload.uint;
-		break;
-	case AST_DATA_BOOLEAN:
-		content->value.AST_DATA_BOOLEAN = node->payload.boolean;
-		break;
-	case AST_DATA_IPADDR:
-		content->value.AST_DATA_IPADDR = node->payload.ipaddr;
-		break;
-	case AST_DATA_DOUBLE:
-		content->value.AST_DATA_DOUBLE = node->payload.dbl;
-		break;
-	case AST_DATA_CONTAINER:
-		break;
-	case AST_DATA_POINTER:
-		content->value.AST_DATA_POINTER = node->payload.ptr;
-		break;
-	}
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief One color for each node type.
- */
-static const struct {
-	enum ast_data_type type;
-	int color;
-} data_result_color[] = {
-	{ AST_DATA_STRING, COLOR_BLUE },
-	{ AST_DATA_PASSWORD, COLOR_BRBLUE },
-	{ AST_DATA_TIMESTAMP, COLOR_CYAN },
-	{ AST_DATA_SECONDS, COLOR_MAGENTA },
-	{ AST_DATA_MILLISECONDS, COLOR_BRMAGENTA },
-	{ AST_DATA_CHARACTER, COLOR_GRAY },
-	{ AST_DATA_INTEGER, COLOR_RED },
-	{ AST_DATA_UNSIGNED_INTEGER, COLOR_RED },
-	{ AST_DATA_DOUBLE, COLOR_RED },
-	{ AST_DATA_BOOLEAN, COLOR_BRRED },
-	{ AST_DATA_CONTAINER, COLOR_GREEN },
-	{ AST_DATA_IPADDR, COLOR_BROWN },
-	{ AST_DATA_POINTER, COLOR_YELLOW },
-};
-
-/*!
- * \internal
- * \brief Get the color configured for a specific node type.
- * \param[in] type The node type.
- * \returns The color specified for the passed type.
- */
-static int data_result_get_color(enum ast_data_type type)
-{
-	int i;
-	for (i = 0; i < ARRAY_LEN(data_result_color); i++) {
-		if (data_result_color[i].type == type) {
-			return data_result_color[i].color;
-		}
-	}
-
-	return COLOR_BLUE;
-}
-
-/*!
- * \internal
- * \brief Print a node to the CLI.
- * \param[in] fd The CLI file descriptor.
- * \param[in] node The node to print.
- * \param[in] depth The actual node depth in the tree.
- */
-static void data_result_print_cli_node(int fd, const struct ast_data *node, uint32_t depth)
-{
-	int i;
-	struct ast_str *tabs, *output;
-
-	tabs = ast_str_create(depth * 10 + 1);
-	if (!tabs) {
-		return;
-	}
-	ast_str_reset(tabs);
-	for (i = 0; i < depth; i++) {
-		ast_str_append(&tabs, 0, "  ");
-	}
-
-	output = ast_str_create(20);
-	if (!output) {
-		ast_free(tabs);
-		return;
-	}
-
-	ast_str_reset(output);
-	ast_term_color_code(&output, data_result_get_color(node->type), 0);
-
-	switch (node->type) {
-	case AST_DATA_POINTER:
-		ast_str_append(&output, 0, "%s%s: %p\n", ast_str_buffer(tabs),
-				node->name, node->payload.ptr);
-		break;
-	case AST_DATA_PASSWORD:
-		ast_str_append(&output, 0, "%s%s: \"%s\"\n",
-				ast_str_buffer(tabs),
-				node->name,
-				node->payload.str);
-		break;
-	case AST_DATA_STRING:
-		ast_str_append(&output, 0, "%s%s: \"%s\"\n",
-				ast_str_buffer(tabs),
-				node->name,
-				node->payload.str);
-		break;
-	case AST_DATA_CHARACTER:
-		ast_str_append(&output, 0, "%s%s: \'%c\'\n",
-				ast_str_buffer(tabs),
-				node->name,
-				node->payload.character);
-		break;
-	case AST_DATA_CONTAINER:
-		ast_str_append(&output, 0, "%s%s\n", ast_str_buffer(tabs),
-				node->name);
-		break;
-	case AST_DATA_TIMESTAMP:
-		ast_str_append(&output, 0, "%s%s: %u\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.uint);
-		break;
-	case AST_DATA_SECONDS:
-		ast_str_append(&output, 0, "%s%s: %u\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.uint);
-		break;
-	case AST_DATA_MILLISECONDS:
-		ast_str_append(&output, 0, "%s%s: %u\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.uint);
-		break;
-	case AST_DATA_INTEGER:
-		ast_str_append(&output, 0, "%s%s: %d\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.sint);
-		break;
-	case AST_DATA_UNSIGNED_INTEGER:
-		ast_str_append(&output, 0, "%s%s: %u\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.uint);
-		break;
-	case AST_DATA_DOUBLE:
-		ast_str_append(&output, 0, "%s%s: %lf\n", ast_str_buffer(tabs),
-				node->name,
-				node->payload.dbl);
-		break;
-	case AST_DATA_BOOLEAN:
-		ast_str_append(&output, 0, "%s%s: %s\n", ast_str_buffer(tabs),
-				node->name,
-				((node->payload.boolean) ? "True" : "False"));
-		break;
-	case AST_DATA_IPADDR:
-		ast_str_append(&output, 0, "%s%s: %s\n", ast_str_buffer(tabs),
-				node->name,
-				ast_inet_ntoa(node->payload.ipaddr));
-		break;
-	}
-
-	ast_free(tabs);
-
-	ast_term_color_code(&output, 0, 0);
-
-	ast_cli(fd, "%s", ast_str_buffer(output));
-
-	ast_free(output);
-
-	if (node->type == AST_DATA_CONTAINER) {
-		__data_result_print_cli(fd, node, depth + 1);
-	}
-}
-
-/*!
- * \internal
- * \brief Print out an ast_data tree to the CLI.
- * \param[in] fd The CLI file descriptor.
- * \param[in] root The root node of the tree.
- * \param[in] depth Actual depth.
- */
-
-static void __data_result_print_cli(int fd, const struct ast_data *root, uint32_t depth)
-{
-	struct ao2_iterator iter;
-	struct ast_data *node;
-
-	if (root->type == AST_DATA_CONTAINER) {
-		iter = ao2_iterator_init(root->children, 0);
-		while ((node = ao2_iterator_next(&iter))) {
-			data_result_print_cli_node(fd, node, depth + 1);
-			ao2_ref(node, -1);
-		}
-		ao2_iterator_destroy(&iter);
-	} else {
-		data_result_print_cli_node(fd, root, depth);
-	}
-}
-
-/*!
- * \internal
- * \brief
- * \param[in] fd The CLI file descriptor.
- * \param[in] root The root node of the tree.
- */
-static void data_result_print_cli(int fd, const struct ast_data *root)
-{
-	ast_cli(fd, COLORIZE_FMT "\n", COLORIZE(data_result_get_color(root->type), 0, root->name));
-
-	__data_result_print_cli(fd, root, 0);
-
-	ast_cli(fd, "\n");
-}
-
-/*!
- * \internal
- * \brief Handle the CLI command "data get".
- */
-static char *handle_cli_data_get(struct ast_cli_entry *e, int cmd,
-		struct ast_cli_args *a)
-{
-	struct ast_data_query query = {
-		.version = AST_DATA_QUERY_VERSION
-	};
-	struct ast_data *tree;
-
-	switch (cmd) {
-	case CLI_INIT:
-		e->command = "data get";
-		e->usage = ""
-			"Usage: data get <path> [<search> [<filter>]]\n"
-			"       Get the tree based on a path.\n";
-		return NULL;
-	case CLI_GENERATE:
-		return NULL;
-	}
-
-	if (a->argc < e->args + 1) {
-		return CLI_SHOWUSAGE;
-	}
-
-	query.path = (char *) a->argv[e->args];
-
-	if (a->argc > e->args + 1) {
-		query.search = (char *) a->argv[e->args + 1];
-	}
-
-	if (a->argc > e->args + 2) {
-		query.filter = (char *) a->argv[e->args + 2];
-	}
-
-	tree = ast_data_get(&query);
-	if (!tree) {
-		return CLI_FAILURE;
-	}
-
-	data_result_print_cli(a->fd, tree);
-
-	ast_data_free(tree);
-
-	return CLI_SUCCESS;
-}
-
-/*!
- * \internal
- * \brief Print the list of data providers.
- * \param[in] fd The CLI file descriptor.
- * \param[in] name The last node visited name.
- * \param[in] container The childrens of the last node.
- * \param[in] path The path to the current node.
- */
-static void data_provider_print_cli(int fd, const char *name,
-	struct ao2_container *container, struct ast_str *path)
-{
-	struct ao2_iterator i;
-	struct ast_str *current_path;
-	struct data_provider *provider;
-
-	current_path = ast_str_create(60);
-	if (!current_path) {
-		return;
-	}
-
-	ast_str_reset(current_path);
-	if (path) {
-		ast_str_set(&current_path, 0, "%s/%s", ast_str_buffer(path), name);
-	} else {
-		ast_str_set(&current_path, 0, "%s", name);
-	}
-
-	i = ao2_iterator_init(container, 0);
-	while ((provider = ao2_iterator_next(&i))) {
-		if (provider->handler) {
-			/* terminal node, print it. */
-			ast_cli(fd, "%s/%s (", ast_str_buffer(current_path),
-				provider->name);
-			if (provider->handler->get) {
-				ast_cli(fd, "get");
-			}
-			ast_cli(fd, ") [%s]\n", provider->registrar);
-		}
-		data_provider_print_cli(fd, provider->name, provider->children,
-			current_path);
-		ao2_ref(provider, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-	ast_free(current_path);
-}
-
-/*!
- * \internal
- * \brief Handle CLI command "data show providers"
- */
-static char *handle_cli_data_show_providers(struct ast_cli_entry *e, int cmd,
-		struct ast_cli_args *a)
-{
-	switch (cmd) {
-	case CLI_INIT:
-		e->command = "data show providers";
-		e->usage = ""
-			"Usage: data show providers\n"
-			"       Show the list of registered providers\n";
-		return NULL;
-	case CLI_GENERATE:
-		return NULL;
-	}
-
-	data_read_lock();
-	data_provider_print_cli(a->fd, "", root_data.container, NULL);
-	data_unlock();
-
-	return CLI_SUCCESS;
-}
-
-/*!
- * \internal
- * \brief Data API CLI commands.
- */
-static struct ast_cli_entry cli_data[] = {
-	AST_CLI_DEFINE(handle_cli_data_get, "Data API get"),
-	AST_CLI_DEFINE(handle_cli_data_show_providers, "Show data providers")
-};
-
-/*!
- * \internal
- * \brief Output a tree to the AMI.
- * \param[in] s AMI session.
- * \param[in] name The root node name.
- * \param[in] container The root container.
- * \param[in] path The current path.
- */
-static void data_result_manager_output(struct mansession *s, const char *name,
-	struct ao2_container *container, struct ast_str *path, int id)
-{
-	struct ao2_iterator i;
-	struct ast_str *current_path;
-	struct ast_data *node;
-	int current_id = id;
-
-	current_path = ast_str_create(60);
-	if (!current_path) {
-		return;
-	}
-
-	ast_str_reset(current_path);
-	if (path) {
-		ast_str_set(&current_path, 0, "%s.%s", ast_str_buffer(path), name);
-	} else {
-		ast_str_set(&current_path, 0, "%s", name);
-	}
-
-	i = ao2_iterator_init(container, 0);
-	while ((node = ao2_iterator_next(&i))) {
-		/* terminal node, print it. */
-		if (node->type != AST_DATA_CONTAINER) {
-			astman_append(s, "%d-%s.%s", id, ast_str_buffer(current_path),
-					node->name);
-		}
-		switch (node->type) {
-		case AST_DATA_CONTAINER:
-			data_result_manager_output(s, node->name, node->children, current_path, ++current_id);
-			break;
-		case AST_DATA_INTEGER:
-			astman_append(s, ": %d\r\n", node->payload.sint);
-			break;
-		case AST_DATA_TIMESTAMP:
-		case AST_DATA_SECONDS:
-		case AST_DATA_MILLISECONDS:
-		case AST_DATA_UNSIGNED_INTEGER:
-			astman_append(s, ": %u\r\n", node->payload.uint);
-			break;
-		case AST_DATA_PASSWORD:
-			astman_append(s, ": %s\r\n", node->payload.str);
-			break;
-		case AST_DATA_STRING:
-			astman_append(s, ": %s\r\n", node->payload.str);
-			break;
-		case AST_DATA_CHARACTER:
-			astman_append(s, ": %c\r\n", node->payload.character);
-			break;
-		case AST_DATA_IPADDR:
-			astman_append(s, ": %s\r\n", ast_inet_ntoa(node->payload.ipaddr));
-			break;
-		case AST_DATA_POINTER:
-			break;
-		case AST_DATA_DOUBLE:
-			astman_append(s, ": %f\r\n", node->payload.dbl);
-			break;
-		case AST_DATA_BOOLEAN:
-			astman_append(s, ": %s\r\n",
-				(node->payload.boolean ? "True" : "False"));
-			break;
-		}
-
-		ao2_ref(node, -1);
-	}
-	ao2_iterator_destroy(&i);
-
-	ast_free(current_path);
-}
-
-/*!
- * \internal
- * \brief Implements the manager action: "DataGet".
- */
-static int manager_data_get(struct mansession *s, const struct message *m)
-{
-	const char *path = astman_get_header(m, "Path");
-	const char *search = astman_get_header(m, "Search");
-	const char *filter = astman_get_header(m, "Filter");
-	const char *id = astman_get_header(m, "ActionID");
-	struct ast_data *res;
-	struct ast_data_query query = {
-		.version = AST_DATA_QUERY_VERSION,
-		.path = (char *) path,
-		.search = (char *) search,
-		.filter = (char *) filter,
-	};
-
-	if (ast_strlen_zero(path)) {
-		astman_send_error(s, m, "'Path' parameter not specified");
-		return 0;
-	}
-
-	res = ast_data_get(&query);
-	if (!res) {
-		astman_send_error(s, m, "No data returned");
-		return 0;
-	}
-
-	astman_append(s, "Event: DataGet Tree\r\n");
-	if (!ast_strlen_zero(id)) {
-		astman_append(s, "ActionID: %s\r\n", id);
-	}
-	data_result_manager_output(s, res->name, res->children, NULL, 0);
-	astman_append(s, "\r\n");
-
-	ast_data_free(res);
-
-	return RESULT_SUCCESS;
-}
-
-static int data_add_codec(struct ast_data *codecs, struct ast_format *format) {
-	struct ast_data *codec;
-	struct ast_codec *tmp;
-
-	tmp = ast_codec_get_by_id(ast_format_get_codec_id(format));
-	if (!tmp) {
-		return -1;
-	}
-
-	codec = ast_data_add_node(codecs, "codec");
-	if (!codec) {
-		ao2_ref(tmp, -1);
-		return -1;
-	}
-
-	ast_data_add_str(codec, "name", tmp->name);
-	ast_data_add_int(codec, "samplespersecond", tmp->sample_rate);
-	ast_data_add_str(codec, "description", tmp->description);
-	ast_data_add_int(codec, "frame_length", tmp->minimum_bytes);
-	ao2_ref(tmp, -1);
-
-	return 0;
-}
-
-int ast_data_add_codec(struct ast_data *root, const char *node_name, struct ast_format *format)
-{
-	struct ast_data *codecs;
-
-	codecs = ast_data_add_node(root, node_name);
-	if (!codecs) {
-		return -1;
-	}
-
-	return data_add_codec(codecs, format);
-}
-
-int ast_data_add_codecs(struct ast_data *root, const char *node_name, struct ast_format_cap *cap)
-{
-	struct ast_data *codecs;
-	size_t i;
-	size_t count;
-
-	codecs = ast_data_add_node(root, node_name);
-	if (!codecs) {
-		return -1;
-	}
-
-	if (!cap) {
-		return 0;
-	}
-
-	count = ast_format_cap_count(cap);
-	for (i = 0; i < count; ++i) {
-		struct ast_format *fmt;
-
-		fmt = ast_format_cap_get_format(cap, i);
-		if (!fmt) {
-			return -1;
-		}
-
-		if (data_add_codec(codecs, fmt)) {
-			ao2_ref(fmt, -1);
-			return -1;
-		}
-
-		ao2_ref(fmt, -1);
-	}
-
-	return 0;
-}
-
-#ifdef TEST_FRAMEWORK
-
-/*!
- * \internal
- * \brief Structure used to test how to add a complete structure,
- *        and how to compare it.
- */
-struct test_structure {
-	int a_int;
-	unsigned int b_bool:1;
-	char *c_str;
-	unsigned int a_uint;
-};
-
-/*!
- * \internal
- * \brief test_structure mapping.
- */
-#define DATA_EXPORT_TEST_STRUCTURE(MEMBER)                              \
-	MEMBER(test_structure, a_int, AST_DATA_INTEGER)                 \
-	MEMBER(test_structure, b_bool, AST_DATA_BOOLEAN)                \
-	MEMBER(test_structure, c_str, AST_DATA_STRING)                  \
-	MEMBER(test_structure, a_uint, AST_DATA_UNSIGNED_INTEGER)
-
-AST_DATA_STRUCTURE(test_structure, DATA_EXPORT_TEST_STRUCTURE);
-
-/*!
- * \internal
- * \brief Callback implementation.
- */
-static int test_data_full_provider(const struct ast_data_search *search,
-		struct ast_data *root)
-{
-	struct ast_data *test_structure;
-	struct test_structure local_test_structure = {
-		.a_int = 10,
-		.b_bool = 1,
-		.c_str = "test string",
-		.a_uint = 20
-	};
-
-	test_structure = ast_data_add_node(root, "test_structure");
-	if (!test_structure) {
-		ast_debug(1, "Internal data api error\n");
-		return 0;
-	}
-
-	/* add the complete structure. */
-	ast_data_add_structure(test_structure, test_structure, &local_test_structure);
-
-	if (!ast_data_search_match(search, test_structure)) {
-		ast_data_remove_node(root, test_structure);
-	}
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief Handler definition for the full provider.
- */
-static const struct ast_data_handler full_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = test_data_full_provider
-};
-
-/*!
- * \internal
- * \brief Structure used to define multiple providers at once.
- */
-static const struct ast_data_entry test_providers[] = {
-	AST_DATA_ENTRY("test/node1/node11/node111", &full_provider)
-};
-
-AST_TEST_DEFINE(test_data_get)
-{
-	struct ast_data *res, *node;
-	struct ast_data_iterator *i;
-	struct ast_data_query query = {
-		.version = AST_DATA_QUERY_VERSION,
-		.path = "test/node1/node11/node111",
-		.search = "node111/test_structure/a_int=10",
-		.filter = "node111/test_structure/a*int"
-	};
-
-	switch (cmd) {
-	case TEST_INIT:
-		info->name = "data_test";
-		info->category = "/main/data/";
-		info->summary = "Data API unit test";
-		info->description =
-			"Tests whether data API get implementation works as expected.";
-		return AST_TEST_NOT_RUN;
-	case TEST_EXECUTE:
-		break;
-	}
-
-	ast_data_register_multiple_core(test_providers, ARRAY_LEN(test_providers));
-
-	res = ast_data_get(&query);
-	if (!res) {
-		ast_test_status_update(test, "Unable to get tree.");
-		ast_data_unregister("test/node1/node11/node111");
-		return AST_TEST_FAIL;
-	}
-
-	/* initiate the iterator and check for errors. */
-	i = ast_data_iterator_init(res, "test_structure/");
-	if (!i) {
-		ast_test_status_update(test, "Unable to initiate the iterator.");
-		ast_data_free(res);
-		ast_data_unregister("test/node1/node11/node111");
-		return AST_TEST_FAIL;
-	}
-
-	/* walk the returned nodes. */
-	while ((node = ast_data_iterator_next(i))) {
-		if (!strcmp(ast_data_retrieve_name(node), "a_int")) {
-			if (ast_data_retrieve_int(node, "/") != 10) {
-				ast_data_iterator_end(i);
-				ast_data_free(res);
-				ast_data_unregister("test/node1/node11/node111");
-				return AST_TEST_FAIL;
-			}
-		} else if (!strcmp(ast_data_retrieve_name(node), "a_uint")) {
-			if (ast_data_retrieve_uint(node, "/") != 20) {
-				ast_data_iterator_end(i);
-				ast_data_free(res);
-				ast_data_unregister("test/node1/node11/node111");
-				return AST_TEST_FAIL;
-			}
-		}
-	}
-
-	/* finish the iterator. */
-	ast_data_iterator_end(i);
-
-	ast_data_free(res);
-
-	ast_data_unregister("test/node1/node11/node111");
-
-	return AST_TEST_PASS;
-}
-
-#endif
-
-/*!
- * \internal
- * \brief Clean up resources on Asterisk shutdown
- */
-static void data_shutdown(void)
-{
-	ast_manager_unregister("DataGet");
-	ast_cli_unregister_multiple(cli_data, ARRAY_LEN(cli_data));
-	ao2_t_ref(root_data.container, -1, "Unref root_data.container in data_shutdown");
-	root_data.container = NULL;
-	ast_rwlock_destroy(&root_data.lock);
-	AST_TEST_UNREGISTER(test_data_get);
-}
-
-int ast_data_init(void)
-{
-	int res = 0;
-
-	ast_rwlock_init(&root_data.lock);
-
-	if (!(root_data.container = ao2_container_alloc(NUM_DATA_NODE_BUCKETS,
-		data_provider_hash, data_provider_cmp))) {
-		return -1;
-	}
-
-	res |= ast_cli_register_multiple(cli_data, ARRAY_LEN(cli_data));
-
-	res |= ast_manager_register_xml_core("DataGet", 0, manager_data_get);
-
-	AST_TEST_REGISTER(test_data_get);
-
-	ast_register_cleanup(data_shutdown);
-
-	return res;
-}
diff -pruN 1:13.23.1~dfsg-2/main/datastore.c 1:16.1.1~dfsg-1/main/datastore.c
--- 1:13.23.1~dfsg-2/main/datastore.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/datastore.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,15 +25,20 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #include "asterisk/datastore.h"
 #include "asterisk/utils.h"
-
-struct ast_datastore *__ast_datastore_alloc(const struct ast_datastore_info *info, const char *uid,
-					    const char *file, int line, const char *function)
+#include "asterisk/astobj2.h"
+#include "asterisk/uuid.h"
+#include "asterisk/module.h"
+
+/*! \brief Number of buckets for datastore container */
+#define DATASTORE_BUCKETS 53
+
+struct ast_datastore *__ast_datastore_alloc(
+	const struct ast_datastore_info *info, const char *uid, struct ast_module *mod,
+	const char *file, int line, const char *function)
 {
 	struct ast_datastore *datastore = NULL;
 
@@ -42,23 +47,21 @@ struct ast_datastore *__ast_datastore_al
 		return NULL;
 	}
 
-#if defined(__AST_DEBUG_MALLOC)
-	if (!(datastore = __ast_calloc(1, sizeof(*datastore), file, line, function))) {
-		return NULL;
-	}
-#else
-	if (!(datastore = ast_calloc(1, sizeof(*datastore)))) {
+	datastore = __ast_calloc(1, sizeof(*datastore), file, line, function);
+	if (!datastore) {
 		return NULL;
 	}
-#endif
 
 	datastore->info = info;
+	datastore->mod = mod;
 
 	if (!ast_strlen_zero(uid) && !(datastore->uid = ast_strdup(uid))) {
 		ast_free(datastore);
 		datastore = NULL;
 	}
 
+	ast_module_ref(mod);
+
 	return datastore;
 }
 
@@ -78,8 +81,86 @@ int ast_datastore_free(struct ast_datast
 		datastore->uid = NULL;
 	}
 
+	ast_module_unref(datastore->mod);
+
 	/* Finally free memory used by ourselves */
 	ast_free(datastore);
 
 	return res;
 }
+
+AO2_STRING_FIELD_HASH_FN(ast_datastore, uid);
+AO2_STRING_FIELD_CMP_FN(ast_datastore, uid);
+
+struct ao2_container *ast_datastores_alloc(void)
+{
+	return ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS, ast_datastore_hash_fn, NULL, ast_datastore_cmp_fn);
+}
+
+int ast_datastores_add(struct ao2_container *datastores, struct ast_datastore *datastore)
+{
+	ast_assert(datastore != NULL);
+	ast_assert(datastore->info != NULL);
+	ast_assert(!ast_strlen_zero(datastore->uid));
+
+	if (!ao2_link(datastores, datastore)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+void ast_datastores_remove(struct ao2_container *datastores, const char *name)
+{
+	ao2_find(datastores, name, OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
+}
+
+struct ast_datastore *ast_datastores_find(struct ao2_container *datastores, const char *name)
+{
+	return ao2_find(datastores, name, OBJ_SEARCH_KEY);
+}
+
+static void datastore_destroy(void *obj)
+{
+	struct ast_datastore *datastore = obj;
+
+	/* Using the destroy function (if present) destroy the data */
+	if (datastore->info->destroy != NULL && datastore->data != NULL) {
+		datastore->info->destroy(datastore->data);
+		datastore->data = NULL;
+	}
+
+	ast_free((void *) datastore->uid);
+	datastore->uid = NULL;
+}
+
+struct ast_datastore *ast_datastores_alloc_datastore(const struct ast_datastore_info *info, const char *uid)
+{
+	struct ast_datastore *datastore;
+	char uuid_buf[AST_UUID_STR_LEN];
+	const char *uid_ptr = uid;
+
+	if (!info) {
+		return NULL;
+	}
+
+	datastore = ao2_alloc(sizeof(*datastore), datastore_destroy);
+	if (!datastore) {
+		return NULL;
+	}
+
+	datastore->info = info;
+	if (ast_strlen_zero(uid)) {
+		/* They didn't provide an ID so we'll provide one ourself */
+		uid_ptr = ast_uuid_generate_str(uuid_buf, sizeof(uuid_buf));
+	}
+
+	datastore->uid = ast_strdup(uid_ptr);
+	if (!datastore->uid) {
+		ao2_ref(datastore, -1);
+		return NULL;
+	}
+
+	return datastore;
+}
diff -pruN 1:13.23.1~dfsg-2/main/db.c 1:16.1.1~dfsg-1/main/db.c
--- 1:13.23.1~dfsg-2/main/db.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/db.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_DB */
 #include <sys/time.h>
@@ -191,8 +189,8 @@ static void clean_statements(void)
 
 static int init_statements(void)
 {
-	/* Don't initialize create_astdb_statment here as the astdb table needs to exist
-	 * brefore these statments can be initialized */
+	/* Don't initialize create_astdb_statement here as the astdb table needs to exist
+	 * brefore these statements can be initialized */
 	return init_stmt(&get_stmt, get_stmt_sql, sizeof(get_stmt_sql))
 	|| init_stmt(&del_stmt, del_stmt_sql, sizeof(del_stmt_sql))
 	|| init_stmt(&deltree_stmt, deltree_stmt_sql, sizeof(deltree_stmt_sql))
@@ -347,7 +345,7 @@ int ast_db_put(const char *family, const
 		ast_log(LOG_WARNING, "Couldn't bind value to stmt: %s\n", sqlite3_errmsg(astdb));
 		res = -1;
 	} else if (sqlite3_step(put_stmt) != SQLITE_DONE) {
-		ast_log(LOG_WARNING, "Couldn't execute statment: %s\n", sqlite3_errmsg(astdb));
+		ast_log(LOG_WARNING, "Couldn't execute statement: %s\n", sqlite3_errmsg(astdb));
 		res = -1;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/main/devicestate.c 1:16.1.1~dfsg-1/main/devicestate.c
--- 1:13.23.1~dfsg-2/main/devicestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/devicestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -143,8 +143,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/channel.h"
 #include "asterisk/utils.h"
@@ -241,12 +239,6 @@ const char *ast_devstate2str(enum ast_de
 	return devstatestring[devstate][0];
 }
 
-/* Deprecated interface (not prefixed with ast_) */
-const char *devstate2str(enum ast_device_state devstate)
-{
-	return devstatestring[devstate][0];
-}
-
 enum ast_device_state ast_state_chan2dev(enum ast_channel_state chanstate)
 {
 	int i;
@@ -515,11 +507,6 @@ int ast_devstate_changed_literal(enum as
 	return 0;
 }
 
-int ast_device_state_changed_literal(const char *dev)
-{
-	return ast_devstate_changed_literal(AST_DEVICE_UNKNOWN, AST_DEVSTATE_CACHABLE, dev);
-}
-
 int ast_devstate_changed(enum ast_device_state state, enum ast_devstate_cache cachable, const char *fmt, ...)
 {
 	char buf[AST_MAX_EXTENSION];
@@ -532,18 +519,6 @@ int ast_devstate_changed(enum ast_device
 	return ast_devstate_changed_literal(state, cachable, buf);
 }
 
-int ast_device_state_changed(const char *fmt, ...)
-{
-	char buf[AST_MAX_EXTENSION];
-	va_list ap;
-
-	va_start(ap, fmt);
-	vsnprintf(buf, sizeof(buf), fmt, ap);
-	va_end(ap);
-
-	return ast_devstate_changed_literal(AST_DEVICE_UNKNOWN, AST_DEVSTATE_CACHABLE, buf);
-}
-
 /*! \brief Go through the dev state change queue and update changes in the dev state thread */
 static void *do_devstate_changes(void *data)
 {
@@ -945,6 +920,8 @@ int devstate_init(void)
 	if (!device_state_topic_cached) {
 		return -1;
 	}
+	stasis_caching_accept_message_type(device_state_topic_cached, ast_device_state_message_type());
+	stasis_caching_set_filter(device_state_topic_cached, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	devstate_message_sub = stasis_subscribe(ast_device_state_topic_all(),
 		devstate_change_cb, NULL);
@@ -952,6 +929,8 @@ int devstate_init(void)
 		ast_log(LOG_ERROR, "Failed to create subscription creating uncached device state aggregate events.\n");
 		return -1;
 	}
+	stasis_subscription_accept_message_type(devstate_message_sub, ast_device_state_message_type());
+	stasis_subscription_set_filter(devstate_message_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/dial.c 1:16.1.1~dfsg-1/main/dial.c
--- 1:13.23.1~dfsg-2/main/dial.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dial.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 
@@ -57,7 +55,7 @@ struct ast_dial {
 	void *user_data;                                   /*!< Attached user data */
 	AST_LIST_HEAD(, ast_dial_channel) channels; /*!< Channels being dialed */
 	pthread_t thread;                                  /*!< Thread (if running in async) */
-	struct ast_callid *callid;                         /*!< callid pointer (if running in async) */
+	ast_callid callid;                                 /*!< callid (if running in async) */
 	ast_mutex_t lock;                                  /*! Lock to protect the thread information above */
 };
 
@@ -248,22 +246,9 @@ struct ast_dial *ast_dial_create(void)
 	return dial;
 }
 
-/*! \brief Append a channel
- * \note Appends a channel to a dialing structure
- * \return Returns channel reference number on success, -1 on failure
- */
-int ast_dial_append(struct ast_dial *dial, const char *tech, const char *device, const struct ast_assigned_ids *assignedids)
+static int dial_append_common(struct ast_dial *dial, struct ast_dial_channel *channel,
+		const char *tech, const char *device, const struct ast_assigned_ids *assignedids)
 {
-	struct ast_dial_channel *channel = NULL;
-
-	/* Make sure we have required arguments */
-	if (!dial || !tech || !device)
-		return -1;
-
-	/* Allocate new memory for dialed channel structure */
-	if (!(channel = ast_calloc(1, sizeof(*channel))))
-		return -1;
-
 	/* Record technology and device for when we actually dial */
 	channel->tech = ast_strdup(tech);
 	channel->device = ast_strdup(device);
@@ -287,6 +272,60 @@ int ast_dial_append(struct ast_dial *dia
 	AST_LIST_INSERT_TAIL(&dial->channels, channel, list);
 
 	return channel->num;
+
+}
+
+/*! \brief Append a channel
+ * \note Appends a channel to a dialing structure
+ * \return Returns channel reference number on success, -1 on failure
+ */
+int ast_dial_append(struct ast_dial *dial, const char *tech, const char *device, const struct ast_assigned_ids *assignedids)
+{
+	struct ast_dial_channel *channel = NULL;
+
+	/* Make sure we have required arguments */
+	if (!dial || !tech || !device)
+		return -1;
+
+	/* Allocate new memory for dialed channel structure */
+	if (!(channel = ast_calloc(1, sizeof(*channel))))
+		return -1;
+
+	return dial_append_common(dial, channel, tech, device, assignedids);
+}
+
+int ast_dial_append_channel(struct ast_dial *dial, struct ast_channel *chan)
+{
+	struct ast_dial_channel *channel;
+	char *tech;
+	char *device;
+	char *dash;
+
+	if (!dial || !chan) {
+		return -1;
+	}
+
+	channel = ast_calloc(1, sizeof(*channel));
+	if (!channel) {
+		return -1;
+	}
+	channel->owner = chan;
+
+	tech = ast_strdupa(ast_channel_name(chan));
+
+	device = strchr(tech, '/');
+	if (!device) {
+		ast_free(channel);
+		return -1;
+	}
+	*device++ = '\0';
+
+	dash = strrchr(device, '-');
+	if (dash) {
+		*dash = '\0';
+	}
+
+	return dial_append_common(dial, channel, tech, device, NULL);
 }
 
 /*! \brief Helper function that requests all channels */
@@ -315,27 +354,29 @@ static int begin_dial_prerun(struct ast_
 		}
 	}
 
-	/* Copy device string over */
-	ast_copy_string(numsubst, channel->device, sizeof(numsubst));
-
-	if (cap && ast_format_cap_count(cap)) {
-		cap_request = cap;
-	} else if (requester_cap) {
-		cap_request = requester_cap;
-	} else {
-		cap_all_audio = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-		ast_format_cap_append_by_type(cap_all_audio, AST_MEDIA_TYPE_AUDIO);
-		cap_request = cap_all_audio;
-	}
+	if (!channel->owner) {
+		/* Copy device string over */
+		ast_copy_string(numsubst, channel->device, sizeof(numsubst));
+
+		if (cap && ast_format_cap_count(cap)) {
+			cap_request = cap;
+		} else if (requester_cap) {
+			cap_request = requester_cap;
+		} else {
+			cap_all_audio = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+			ast_format_cap_append_by_type(cap_all_audio, AST_MEDIA_TYPE_AUDIO);
+			cap_request = cap_all_audio;
+		}
 
-	/* If we fail to create our owner channel bail out */
-	if (!(channel->owner = ast_request(channel->tech, cap_request, &assignedids, chan, numsubst, &channel->cause))) {
+		/* If we fail to create our owner channel bail out */
+		if (!(channel->owner = ast_request(channel->tech, cap_request, &assignedids, chan, numsubst, &channel->cause))) {
+			ao2_cleanup(cap_all_audio);
+			return -1;
+		}
+		cap_request = NULL;
+		ao2_cleanup(requester_cap);
 		ao2_cleanup(cap_all_audio);
-		return -1;
 	}
-	cap_request = NULL;
-	ao2_cleanup(requester_cap);
-	ao2_cleanup(cap_all_audio);
 
 	if (chan) {
 		ast_channel_lock_both(chan, channel->owner);
@@ -438,9 +479,6 @@ static int begin_dial_channel(struct ast
 		ast_hangup(channel->owner);
 		channel->owner = NULL;
 	} else {
-		if (chan) {
-			ast_poll_channel_add(chan, channel->owner);
-		}
 		ast_channel_publish_dial(async ? NULL : chan, channel->owner, channel->device, NULL);
 		res = 1;
 		ast_verb(3, "Called %s\n", numsubst);
@@ -548,13 +586,17 @@ static void set_state(struct ast_dial *d
 		dial->state_callback(dial);
 }
 
-/*! \brief Helper function that handles control frames WITH owner */
+/*! \brief Helper function that handles frames */
 static void handle_frame(struct ast_dial *dial, struct ast_dial_channel *channel, struct ast_frame *fr, struct ast_channel *chan)
 {
 	if (fr->frametype == AST_FRAME_CONTROL) {
 		switch (fr->subclass.integer) {
 		case AST_CONTROL_ANSWER:
-			ast_verb(3, "%s answered %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
+			if (chan) {
+				ast_verb(3, "%s answered %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
+			} else {
+				ast_verb(3, "%s answered\n", ast_channel_name(channel->owner));
+			}
 			AST_LIST_LOCK(&dial->channels);
 			AST_LIST_REMOVE(&dial->channels, channel, list);
 			AST_LIST_INSERT_HEAD(&dial->channels, channel, list);
@@ -578,28 +620,49 @@ static void handle_frame(struct ast_dial
 			break;
 		case AST_CONTROL_INCOMPLETE:
 			ast_verb(3, "%s dialed Incomplete extension %s\n", ast_channel_name(channel->owner), ast_channel_exten(channel->owner));
-			ast_indicate(chan, AST_CONTROL_INCOMPLETE);
+			if (chan) {
+				ast_indicate(chan, AST_CONTROL_INCOMPLETE);
+			} else {
+				ast_hangup(channel->owner);
+				channel->cause = AST_CAUSE_UNALLOCATED;
+				channel->owner = NULL;
+			}
 			break;
 		case AST_CONTROL_RINGING:
 			ast_verb(3, "%s is ringing\n", ast_channel_name(channel->owner));
-			if (!dial->options[AST_DIAL_OPTION_MUSIC])
+			ast_channel_publish_dial(chan, channel->owner, channel->device, "RINGING");
+			if (chan && !dial->options[AST_DIAL_OPTION_MUSIC])
 				ast_indicate(chan, AST_CONTROL_RINGING);
 			set_state(dial, AST_DIAL_RESULT_RINGING);
 			break;
 		case AST_CONTROL_PROGRESS:
-			ast_verb(3, "%s is making progress, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
-			ast_indicate(chan, AST_CONTROL_PROGRESS);
+			ast_channel_publish_dial(chan, channel->owner, channel->device, "PROGRESS");
+			if (chan) {
+				ast_verb(3, "%s is making progress, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
+				ast_indicate(chan, AST_CONTROL_PROGRESS);
+			} else {
+				ast_verb(3, "%s is making progress\n", ast_channel_name(channel->owner));
+			}
 			set_state(dial, AST_DIAL_RESULT_PROGRESS);
 			break;
 		case AST_CONTROL_VIDUPDATE:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "%s requested a video update, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
 			ast_indicate(chan, AST_CONTROL_VIDUPDATE);
 			break;
 		case AST_CONTROL_SRCUPDATE:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "%s requested a source update, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
 			ast_indicate(chan, AST_CONTROL_SRCUPDATE);
 			break;
 		case AST_CONTROL_CONNECTED_LINE:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "%s connected line has changed, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
 			if (ast_channel_connected_line_sub(channel->owner, chan, fr, 1) &&
 				ast_channel_connected_line_macro(channel->owner, chan, fr, 1, 1)) {
@@ -607,6 +670,9 @@ static void handle_frame(struct ast_dial
 			}
 			break;
 		case AST_CONTROL_REDIRECTING:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "%s redirecting info has changed, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
 			if (ast_channel_redirecting_sub(channel->owner, chan, fr, 1) &&
 				ast_channel_redirecting_macro(channel->owner, chan, fr, 1, 1)) {
@@ -614,15 +680,26 @@ static void handle_frame(struct ast_dial
 			}
 			break;
 		case AST_CONTROL_PROCEEDING:
-			ast_verb(3, "%s is proceeding, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
-			ast_indicate(chan, AST_CONTROL_PROCEEDING);
+			ast_channel_publish_dial(chan, channel->owner, channel->device, "PROCEEDING");
+			if (chan) {
+				ast_verb(3, "%s is proceeding, passing it to %s\n", ast_channel_name(channel->owner), ast_channel_name(chan));
+				ast_indicate(chan, AST_CONTROL_PROCEEDING);
+			} else {
+				ast_verb(3, "%s is proceeding\n", ast_channel_name(channel->owner));
+			}
 			set_state(dial, AST_DIAL_RESULT_PROCEEDING);
 			break;
 		case AST_CONTROL_HOLD:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "Call on %s placed on hold\n", ast_channel_name(chan));
 			ast_indicate_data(chan, AST_CONTROL_HOLD, fr->data.ptr, fr->datalen);
 			break;
 		case AST_CONTROL_UNHOLD:
+			if (!chan) {
+				break;
+			}
 			ast_verb(3, "Call on %s left from hold\n", ast_channel_name(chan));
 			ast_indicate(chan, AST_CONTROL_UNHOLD);
 			break;
@@ -630,11 +707,15 @@ static void handle_frame(struct ast_dial
 		case AST_CONTROL_FLASH:
 			break;
 		case AST_CONTROL_PVT_CAUSE_CODE:
-			ast_indicate_data(chan, AST_CONTROL_PVT_CAUSE_CODE, fr->data.ptr, fr->datalen);
+			if (chan) {
+				ast_indicate_data(chan, AST_CONTROL_PVT_CAUSE_CODE, fr->data.ptr, fr->datalen);
+			}
 			break;
 		case -1:
-			/* Prod the channel */
-			ast_indicate(chan, -1);
+			if (chan) {
+				/* Prod the channel */
+				ast_indicate(chan, -1);
+			}
 			break;
 		default:
 			break;
@@ -642,65 +723,6 @@ static void handle_frame(struct ast_dial
 	}
 }
 
-/*! \brief Helper function that handles control frames WITHOUT owner */
-static void handle_frame_ownerless(struct ast_dial *dial, struct ast_dial_channel *channel, struct ast_frame *fr)
-{
-	/* If we have no owner we can only update the state of the dial structure, so only look at control frames */
-	if (fr->frametype != AST_FRAME_CONTROL)
-		return;
-
-	switch (fr->subclass.integer) {
-	case AST_CONTROL_ANSWER:
-		ast_verb(3, "%s answered\n", ast_channel_name(channel->owner));
-		AST_LIST_LOCK(&dial->channels);
-		AST_LIST_REMOVE(&dial->channels, channel, list);
-		AST_LIST_INSERT_HEAD(&dial->channels, channel, list);
-		AST_LIST_UNLOCK(&dial->channels);
-		ast_channel_publish_dial(NULL, channel->owner, channel->device, "ANSWER");
-		set_state(dial, AST_DIAL_RESULT_ANSWERED);
-		break;
-	case AST_CONTROL_BUSY:
-		ast_verb(3, "%s is busy\n", ast_channel_name(channel->owner));
-		ast_channel_publish_dial(NULL, channel->owner, channel->device, "BUSY");
-		ast_hangup(channel->owner);
-		channel->cause = AST_CAUSE_USER_BUSY;
-		channel->owner = NULL;
-		break;
-	case AST_CONTROL_CONGESTION:
-		ast_verb(3, "%s is circuit-busy\n", ast_channel_name(channel->owner));
-		ast_channel_publish_dial(NULL, channel->owner, channel->device, "CONGESTION");
-		ast_hangup(channel->owner);
-		channel->cause = AST_CAUSE_NORMAL_CIRCUIT_CONGESTION;
-		channel->owner = NULL;
-		break;
-	case AST_CONTROL_INCOMPLETE:
-		/*
-		 * Nothing to do but abort the call since we have no
-		 * controlling channel to ask for more digits.
-		 */
-		ast_verb(3, "%s dialed Incomplete extension %s\n",
-			ast_channel_name(channel->owner), ast_channel_exten(channel->owner));
-		ast_hangup(channel->owner);
-		channel->cause = AST_CAUSE_UNALLOCATED;
-		channel->owner = NULL;
-		break;
-	case AST_CONTROL_RINGING:
-		ast_verb(3, "%s is ringing\n", ast_channel_name(channel->owner));
-		set_state(dial, AST_DIAL_RESULT_RINGING);
-		break;
-	case AST_CONTROL_PROGRESS:
-		ast_verb(3, "%s is making progress\n", ast_channel_name(channel->owner));
-		set_state(dial, AST_DIAL_RESULT_PROGRESS);
-		break;
-	case AST_CONTROL_PROCEEDING:
-		ast_verb(3, "%s is proceeding\n", ast_channel_name(channel->owner));
-		set_state(dial, AST_DIAL_RESULT_PROCEEDING);
-		break;
-	default:
-		break;
-	}
-}
-
 /*! \brief Helper function to handle when a timeout occurs on dialing attempt */
 static int handle_timeout_trip(struct ast_dial *dial, struct timeval start)
 {
@@ -843,8 +865,6 @@ static enum ast_dial_result monitor_dial
 				set_state(dial, AST_DIAL_RESULT_HANGUP);
 				break;
 			}
-			if (chan)
-				ast_poll_channel_del(chan, channel->owner);
 			ast_channel_publish_dial(chan, who, channel->device, ast_hangup_cause_to_dial_status(ast_channel_hangupcause(who)));
 			ast_hangup(who);
 			channel->owner = NULL;
@@ -852,10 +872,7 @@ static enum ast_dial_result monitor_dial
 		}
 
 		/* Process the frame */
-		if (chan)
-			handle_frame(dial, channel, fr, chan);
-		else
-			handle_frame_ownerless(dial, channel, fr);
+		handle_frame(dial, channel, fr, chan);
 
 		/* Free the received frame and start all over */
 		ast_frfree(fr);
@@ -868,8 +885,6 @@ static enum ast_dial_result monitor_dial
 		AST_LIST_TRAVERSE(&dial->channels, channel, list) {
 			if (!channel->owner || channel->owner == who)
 				continue;
-			if (chan)
-				ast_poll_channel_del(chan, channel->owner);
 			ast_channel_publish_dial(chan, channel->owner, channel->device, "CANCEL");
 			ast_hangup(channel->owner);
 			channel->cause = AST_CAUSE_ANSWERED_ELSEWHERE;
@@ -893,8 +908,6 @@ static enum ast_dial_result monitor_dial
 		AST_LIST_TRAVERSE(&dial->channels, channel, list) {
 			if (!channel->owner)
 				continue;
-			if (chan)
-				ast_poll_channel_del(chan, channel->owner);
 			ast_channel_publish_dial(chan, channel->owner, channel->device, "CANCEL");
 			ast_hangup(channel->owner);
 			channel->cause = AST_CAUSE_NORMAL_CLEARING;
@@ -1131,11 +1144,6 @@ int ast_dial_destroy(struct ast_dial *di
 	/* Lock be gone! */
 	ast_mutex_destroy(&dial->lock);
 
-	/* Get rid of the reference to the ast_callid */
-	if (dial->callid) {
-		ast_callid_unref(dial->callid);
-	}
-
 	/* Free structure */
 	ast_free(dial);
 
diff -pruN 1:13.23.1~dfsg-2/main/dns.c 1:16.1.1~dfsg-1/main/dns.c
--- 1:13.23.1~dfsg-2/main/dns.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/network.h"
 #include <arpa/nameser.h>	/* res_* functions */
 #include <resolv.h>
@@ -45,6 +43,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/dns.h"
 #include "asterisk/endian.h"
 
+/*! \brief The maximum size permitted for the answer from the DNS server */
 #define MAX_SIZE 4096
 
 #ifdef __PDP_ENDIAN
@@ -59,6 +58,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #define DETERMINED_BYTE_ORDER __LITTLE_ENDIAN
 #endif
 
+#ifndef HAVE_RES_NINIT
+AST_MUTEX_DEFINE_STATIC(res_lock);
+#endif
+
 /* The dns_HEADER structure definition below originated
    in the arpa/nameser.h header file distributed with ISC
    BIND, which contains the following copyright and license
@@ -156,12 +159,23 @@ typedef struct {
 } dns_HEADER;
 
 struct dn_answer {
-	unsigned short rtype;
-	unsigned short class;
-	unsigned int ttl;
-	unsigned short size;
+	unsigned short rtype;       /*!< The resource record type. */
+	unsigned short class;       /*!< The resource record class. */
+	unsigned int ttl;           /*!< The resource record time to live. */
+	unsigned short size;        /*!< The resource record size. */
 } __attribute__((__packed__));
 
+/*!
+ * \brief Tries to find the position of the next field in the DNS response.
+ *
+ * \internal
+ *
+ * \param  s    A char pointer to the current frame in the DNS response.
+ * \param  len  The remaining available length of the DNS response.
+ *
+ * \retval The position of the next field
+ * \retval -1 if there are no remaining fields
+ */
 static int skip_name(unsigned char *s, int len)
 {
 	int x = 0;
@@ -172,20 +186,147 @@ static int skip_name(unsigned char *s, i
 			x++;
 			break;
 		}
+
 		if ((*s & 0xc0) == 0xc0) {
 			s += 2;
 			x += 2;
 			break;
 		}
+
 		x += *s + 1;
 		s += *s + 1;
 	}
-	if (x >= len)
-		return -1;
+
+	/* If we are out of room to search, return failure. */
+	if (x >= len) {
+		return AST_DNS_SEARCH_FAILURE;
+	}
+
+	/* Return the value for the current position in the DNS response. This is the start
+	position of the next field. */
 	return x;
 }
 
-/*! \brief Parse DNS lookup result, call callback */
+/*!
+ * \brief Advances the position of the DNS response pointer by the size of the current field.
+ *
+ * \internal
+ *
+ * \param  dns_response   A pointer to a char pointer to the current field in the DNS response.
+ * \param  remaining_len  The remaining available length in the DNS response to search.
+ * \param  field_size     A positive value representing the size of the current field
+                          pointed to by the dns_response parameter.
+ *
+ * \retval The remaining length in the DNS response
+ * \retval -1 there are no frames remaining in the DNS response
+ */
+static int dns_advance_field(unsigned char **dns_response, int remaining_len, int field_size)
+{
+	if (dns_response == NULL || field_size < 0 || remaining_len < field_size) {
+		return AST_DNS_SEARCH_FAILURE;
+	}
+
+	*dns_response += field_size;
+	remaining_len -= field_size;
+
+	return remaining_len;
+}
+
+#ifndef HAVE_RES_NINIT
+/*!
+ * \brief Handles the DNS search if the system has RES_INIT.
+ *
+ * \internal
+ *
+ * \param  dname             Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  rr_class          Record Class (see "man res_search").
+ * \param  rr_type           Record type (see "man res_search").
+ * \param  dns_response      The full DNS response.
+ * \param  dns_response_len  The length of the full DNS response.
+ *
+ * \retval The length of the DNS response
+ * \retval -1 on search failure
+ */
+static int dns_search_res(const char *dname, int rr_class, int rr_type,
+	unsigned char *dns_response, int dns_response_len)
+{
+
+	int ret = AST_DNS_SEARCH_FAILURE;
+
+	ast_mutex_lock(&res_lock);
+	res_init();
+	ret = res_search(dname,
+	                 rr_class,
+	                 rr_type,
+	                 dns_response,
+	                 dns_response_len);
+
+#ifdef HAVE_RES_CLOSE
+	res_close();
+#endif
+
+	ast_mutex_unlock(&res_lock);
+
+	return ret;
+}
+#else
+/*!
+ * \brief Handles the DNS search if the system has RES_NINIT.
+ *
+ * \internal
+ *
+ * \param  dname             Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  rr_class          Record Class (see "man res_search").
+ * \param  rr_type           Record type (see "man res_search").
+ * \param  dns_response      The full DNS response.
+ * \param  dns_response_len  The length of the full DNS response.
+ *
+ * \retval The length of the DNS response
+ * \retval -1 on search failure
+ */
+static int dns_search_res(const char *dname, int rr_class, int rr_type,
+	unsigned char *dns_response, int dns_response_len)
+{
+
+	int ret = AST_DNS_SEARCH_FAILURE;
+	struct __res_state dns_state;
+
+	memset(&dns_state, 0, sizeof(dns_state));
+	res_ninit(&dns_state);
+	ret = res_nsearch(&dns_state,
+	                  dname,
+	                  rr_class,
+	                  rr_type,
+	                  dns_response,
+	                  dns_response_len);
+
+#ifdef HAVE_RES_NDESTROY
+	res_ndestroy(&dns_state);
+#else
+	res_nclose(&dns_state);
+#endif
+
+	return ret;
+}
+#endif
+
+/*!
+ * \brief Parse DNS lookup result, call callback
+ *
+ * \internal
+ *
+ * \param  context   Void pointer containing data to use in the callback functions.
+ * \param  dname     Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  class     Record Class (see "man res_search").
+ * \param  type      Record type (see "man res_search").
+ * \param  answer    The full DNS response.
+ * \param  len       The length of the full DNS response.
+ * \param  callback  Callback function for handling the discovered resource records from the DNS search.
+ *
+ * \retval -1 on search failure
+ * \retval  0 on no records found
+ * \retval  1 on success
+ */
 static int dns_parse_answer(void *context,
 	int class, int type, unsigned char *answer, int len,
 	int (*callback)(void *context, unsigned char *answer, int len, unsigned char *fullanswer))
@@ -244,13 +385,110 @@ static int dns_parse_answer(void *contex
 	return ret;
 }
 
-#ifndef HAVE_RES_NINIT
-AST_MUTEX_DEFINE_STATIC(res_lock);
-#endif
+/*!
+ * \brief Extended version of the DNS Parsing function.
+ *
+ * \details Parses the DNS lookup result and notifies the observer of each discovered
+ *          resource record with the provided callback.
+ *
+ * \internal
+ *
+ * \param  context           Void pointer containing data to use in the callback functions.
+ * \param  dname             Domain name to lookup (host, SRV domain, TXT record name).
+ * \param  rr_class          Record Class (see "man res_search").
+ * \param  rr_type           Record type (see "man res_search").
+ * \param  answer            The full DNS response.
+ * \param  answer_len        The length of the full DNS response.
+ * \param  response_handler  Callback function for handling the DNS response.
+ * \param  record_handler    Callback function for handling the discovered resource records from the DNS search.
+ *
+ * \retval -1 on search failure
+ * \retval  0 on no records found
+ * \retval  1 on success
+ */
+static int dns_parse_answer_ex(void *context, int rr_class, int rr_type, unsigned char *answer, int answer_len,
+	int (*response_handler)(void *context, unsigned char *dns_response, int dns_response_len, int rcode),
+	int (*record_handler)(void *context, unsigned char *record, int record_len, int ttl))
+{
+	unsigned char *dns_response = answer;
+	dns_HEADER *dns_header = (dns_HEADER *)answer;
+
+	struct dn_answer *ans;
+	int res, x, pos, dns_response_len, ret;
+
+	dns_response_len = answer_len;
+	ret = AST_DNS_SEARCH_NO_RECORDS;
+
+	/* Invoke the response_handler callback to notify the observer of the raw DNS response */
+	response_handler(context, dns_response, dns_response_len, ntohs(dns_header->rcode));
+
+	/* Verify there is something to parse */
+	if (answer_len == 0) {
+		return ret;
+	}
+
+	/* Try advancing the cursor for the dns header */
+	if ((pos = dns_advance_field(&answer, answer_len, sizeof(dns_HEADER))) < 0) {
+		ast_log(LOG_WARNING, "Length of DNS answer exceeds available search frames\n");
+		return AST_DNS_SEARCH_FAILURE;
+	}
+
+	/* Skip domain name and QCODE / QCLASS */
+	for (x = 0; x < ntohs(dns_header->qdcount); x++) {
+		if ((res = skip_name(answer, pos)) < 0) {
+			ast_log(LOG_WARNING, "Failed skipping name\n");
+			return AST_DNS_SEARCH_FAILURE;
+		}
 
-/*! \brief Lookup record in DNS
-\note Asterisk DNS is synchronus at this time. This means that if your DNS does
-not work properly, Asterisk might not start properly or a channel may lock.
+		/* Try advancing the cursor for the name and QCODE / QCLASS fields */
+		if ((pos = dns_advance_field(&answer, pos, res + 4)) < 0) {
+			return AST_DNS_SEARCH_FAILURE;
+		}
+	}
+
+	/* Extract the individual records */
+	for (x = 0; x < ntohs(dns_header->ancount); x++) {
+		if ((res = skip_name(answer, pos)) < 0) {
+			ast_log(LOG_WARNING, "Failed skipping name\n");
+			return AST_DNS_SEARCH_FAILURE;
+		}
+
+		/* Try advancing the cursor to the current record */
+		if ((pos = dns_advance_field(&answer, pos, res)) < 0) {
+			ast_log(LOG_WARNING, "Length of DNS answer exceeds available search frames\n");
+			return AST_DNS_SEARCH_FAILURE;
+		}
+
+		/* Cast the current value for the answer pointer as a dn_answer struct */
+		ans = (struct dn_answer *) answer;
+
+		/* Try advancing the cursor to the end of the current record  */
+		if ((pos = dns_advance_field(&answer, pos, sizeof(struct dn_answer)))  < 0) {
+			ast_log(LOG_WARNING, "Length of DNS answer exceeds available search frames\n");
+			return AST_DNS_SEARCH_FAILURE;
+		}
+
+		/* Skip over the records that do not have the same resource record class and type we care about */
+		if (ntohs(ans->class) == rr_class && ntohs(ans->rtype) == rr_type) {
+			/* Invoke the record handler callback to deliver the discovered record */
+			record_handler(context, answer, ntohs(ans->size), ntohl(ans->ttl));
+			/*At least one record was found */
+			ret = AST_DNS_SEARCH_SUCCESS;
+		}
+
+		/* Try and update the field to the next record, but ignore any errors that come
+		 * back because this may be the end of the line. */
+		pos = dns_advance_field(&answer, pos, ntohs(ans->size));
+	}
+
+	return ret;
+}
+
+/*!
+ * \brief Lookup record in DNS
+ *
+ * \note Asterisk DNS is synchronus at this time. This means that if your DNS does not
+ *       work properly, Asterisk might not start properly or a channel may lock.
 */
 int ast_search_dns(void *context,
 	   const char *dname, int class, int type,
@@ -296,6 +534,51 @@ int ast_search_dns(void *context,
 
 	return ret;
 }
+
+enum ast_dns_search_result ast_search_dns_ex(void *context, const char *dname, int rr_class, int rr_type,
+	   int (*response_handler)(void *context, unsigned char *dns_response, int dns_response_len, int rcode),
+	   int (*record_handler)(void *context, unsigned char *record, int record_len, int ttl))
+{
+	int ret, dns_response_len;
+	unsigned char dns_response[MAX_SIZE];
+
+	/* Assert that the callbacks are not NULL */
+	ast_assert(response_handler != NULL);
+	ast_assert(record_handler != NULL);
+
+	/* Try the DNS search. */
+	dns_response_len = dns_search_res(dname,
+	                                  rr_class,
+	                                  rr_type,
+	                                  dns_response,
+	                                  sizeof(dns_response));
+
+	if (dns_response_len < 0) {
+		ast_debug(1, "DNS search failed for %s\n", dname);
+		response_handler(context, (unsigned char *)"", 0, NXDOMAIN);
+		return AST_DNS_SEARCH_FAILURE;
+	}
+
+	/* Parse records from DNS response */
+	ret = dns_parse_answer_ex(context,
+	                          rr_class,
+	                          rr_type,
+	                          dns_response,
+	                          dns_response_len,
+	                          response_handler,
+	                          record_handler);
+
+	/* Handle the return code from parsing the DNS response */
+	if (ret == AST_DNS_SEARCH_FAILURE) {
+		/* Parsing Error */
+		ast_log(LOG_WARNING, "DNS Parse error for %s\n", dname);
+	} else if (ret == AST_DNS_SEARCH_NO_RECORDS) {
+		/* No results found */
+		ast_debug(1, "DNS search yielded no results for %s\n", dname);
+	}
+
+	return ret;
+}
 
 struct ao2_container *ast_dns_get_nameservers(void)
 {
diff -pruN 1:13.23.1~dfsg-2/main/dns_core.c 1:16.1.1~dfsg-1/main/dns_core.c
--- 1:13.23.1~dfsg-2/main/dns_core.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_core.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,666 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Core DNS Functionality
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/_private.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/astobj2.h"
+#include "asterisk/strings.h"
+#include "asterisk/sched.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_srv.h"
+#include "asterisk/dns_tlsa.h"
+#include "asterisk/dns_recurring.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_internal.h"
+
+#include <netinet/in.h>
+#include <arpa/nameser.h>
+
+AST_RWLIST_HEAD_STATIC(resolvers, ast_dns_resolver);
+
+static struct ast_sched_context *sched;
+
+struct ast_sched_context *ast_dns_get_sched(void)
+{
+	return sched;
+}
+
+const char *ast_dns_query_get_name(const struct ast_dns_query *query)
+{
+	return query->name;
+}
+
+int ast_dns_query_get_rr_type(const struct ast_dns_query *query)
+{
+	return query->rr_type;
+}
+
+int ast_dns_query_get_rr_class(const struct ast_dns_query *query)
+{
+	return query->rr_class;
+}
+
+void *ast_dns_query_get_data(const struct ast_dns_query *query)
+{
+	return query->user_data;
+}
+
+struct ast_dns_result *ast_dns_query_get_result(const struct ast_dns_query *query)
+{
+	return query->result;
+}
+
+unsigned int ast_dns_result_get_secure(const struct ast_dns_result *result)
+{
+	return result->secure;
+}
+
+unsigned int ast_dns_result_get_bogus(const struct ast_dns_result *result)
+{
+	return result->bogus;
+}
+
+unsigned int ast_dns_result_get_rcode(const struct ast_dns_result *result)
+{
+	return result->rcode;
+}
+
+const char *ast_dns_result_get_canonical(const struct ast_dns_result *result)
+{
+	return result->canonical;
+}
+
+const struct ast_dns_record *ast_dns_result_get_records(const struct ast_dns_result *result)
+{
+	return AST_LIST_FIRST(&result->records);
+}
+
+const char *ast_dns_result_get_answer(const struct ast_dns_result *result)
+{
+	return result->answer;
+}
+
+int ast_dns_result_get_lowest_ttl(const struct ast_dns_result *result)
+{
+	int ttl = 0;
+	const struct ast_dns_record *record;
+
+	if (ast_dns_result_get_rcode(result) == NXDOMAIN) {
+		return 0;
+	}
+
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		if (!ttl || (ast_dns_record_get_ttl(record) && (ast_dns_record_get_ttl(record) < ttl))) {
+			ttl = ast_dns_record_get_ttl(record);
+		}
+	}
+
+	return ttl;
+}
+
+void ast_dns_result_free(struct ast_dns_result *result)
+{
+	struct ast_dns_record *record;
+
+	if (!result) {
+		return;
+	}
+
+	while ((record = AST_LIST_REMOVE_HEAD(&result->records, list))) {
+		ast_free(record);
+	}
+
+	ast_free(result);
+}
+
+int ast_dns_record_get_rr_type(const struct ast_dns_record *record)
+{
+	return record->rr_type;
+}
+
+int ast_dns_record_get_rr_class(const struct ast_dns_record *record)
+{
+	return record->rr_class;
+}
+
+int ast_dns_record_get_ttl(const struct ast_dns_record *record)
+{
+	return record->ttl;
+}
+
+const char *ast_dns_record_get_data(const struct ast_dns_record *record)
+{
+	return record->data_ptr;
+}
+
+size_t ast_dns_record_get_data_size(const struct ast_dns_record *record)
+{
+	return record->data_len;
+}
+
+const struct ast_dns_record *ast_dns_record_get_next(const struct ast_dns_record *record)
+{
+	return AST_LIST_NEXT(record, list);
+}
+
+/*! \brief Destructor for an active DNS query */
+static void dns_query_active_destroy(void *data)
+{
+	struct ast_dns_query_active *active = data;
+
+	ao2_cleanup(active->query);
+}
+
+/*! \brief \brief Destructor for a DNS query */
+static void dns_query_destroy(void *data)
+{
+	struct ast_dns_query *query = data;
+
+	ao2_cleanup(query->user_data);
+	ao2_cleanup(query->resolver_data);
+	ast_dns_result_free(query->result);
+}
+
+struct ast_dns_query *dns_query_alloc(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data)
+{
+	struct ast_dns_query *query;
+
+	if (ast_strlen_zero(name)) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution, no name provided\n");
+		return NULL;
+	} else if (rr_type > 65536) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution of '%s', resource record type '%d' exceeds maximum\n",
+			name, rr_type);
+		return NULL;
+	} else if (rr_type < 0) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution of '%s', invalid resource record type '%d'\n",
+			name, rr_type);
+		return NULL;
+	} else if (rr_class > 65536) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution of '%s', resource record class '%d' exceeds maximum\n",
+			name, rr_class);
+		return NULL;
+	} else if (rr_class < 0) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution of '%s', invalid resource class '%d'\n",
+			name, rr_class);
+		return NULL;
+	} else if (!callback) {
+		ast_log(LOG_WARNING, "Could not perform asynchronous resolution of '%s', no callback provided\n",
+			name);
+		return NULL;
+	}
+
+	query = ao2_alloc_options(sizeof(*query) + strlen(name) + 1, dns_query_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!query) {
+		return NULL;
+	}
+
+	query->callback = callback;
+	query->user_data = ao2_bump(data);
+	query->rr_type = rr_type;
+	query->rr_class = rr_class;
+	strcpy(query->name, name); /* SAFE */
+
+	AST_RWLIST_RDLOCK(&resolvers);
+	query->resolver = AST_RWLIST_FIRST(&resolvers);
+	AST_RWLIST_UNLOCK(&resolvers);
+
+	if (!query->resolver) {
+		ast_log(LOG_ERROR, "Attempted to do a DNS query for '%s' of class '%d' and type '%d' but no resolver is available\n",
+			name, rr_class, rr_type);
+		ao2_ref(query, -1);
+		return NULL;
+	}
+
+	return query;
+}
+
+struct ast_dns_query_active *ast_dns_resolve_async(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data)
+{
+	struct ast_dns_query_active *active;
+
+	active = ao2_alloc_options(sizeof(*active), dns_query_active_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!active) {
+		return NULL;
+	}
+
+	active->query = dns_query_alloc(name, rr_type, rr_class, callback, data);
+	if (!active->query) {
+		ao2_ref(active, -1);
+		return NULL;
+	}
+
+	if (active->query->resolver->resolve(active->query)) {
+		ast_log(LOG_ERROR, "Resolver '%s' returned an error when resolving '%s' of class '%d' and type '%d'\n",
+			active->query->resolver->name, name, rr_class, rr_type);
+		ao2_ref(active, -1);
+		return NULL;
+	}
+
+	return active;
+}
+
+int ast_dns_resolve_cancel(struct ast_dns_query_active *active)
+{
+	return active->query->resolver->cancel(active->query);
+}
+
+/*! \brief Structure used for signaling back for synchronous resolution completion */
+struct dns_synchronous_resolve {
+	/*! \brief Lock used for signaling */
+	ast_mutex_t lock;
+	/*! \brief Condition used for signaling */
+	ast_cond_t cond;
+	/*! \brief Whether the query has completed */
+	unsigned int completed;
+	/*! \brief The result from the query */
+	struct ast_dns_result *result;
+};
+
+/*! \brief Destructor for synchronous resolution structure */
+static void dns_synchronous_resolve_destroy(void *data)
+{
+	struct dns_synchronous_resolve *synchronous = data;
+
+	ast_mutex_destroy(&synchronous->lock);
+	ast_cond_destroy(&synchronous->cond);
+
+	/* This purposely does not unref result as it has been passed to the caller */
+}
+
+/*! \brief Callback used to implement synchronous resolution */
+static void dns_synchronous_resolve_callback(const struct ast_dns_query *query)
+{
+	struct dns_synchronous_resolve *synchronous = ast_dns_query_get_data(query);
+
+	synchronous->result = query->result;
+	((struct ast_dns_query *)query)->result = NULL;
+
+	ast_mutex_lock(&synchronous->lock);
+	synchronous->completed = 1;
+	ast_cond_signal(&synchronous->cond);
+	ast_mutex_unlock(&synchronous->lock);
+}
+
+int ast_dns_resolve(const char *name, int rr_type, int rr_class, struct ast_dns_result **result)
+{
+	struct dns_synchronous_resolve *synchronous;
+	struct ast_dns_query_active *active;
+
+	if (ast_strlen_zero(name)) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution, no name provided\n");
+		return -1;
+	} else if (rr_type > 65536) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution of '%s', resource record type '%d' exceeds maximum\n",
+			name, rr_type);
+		return -1;
+	} else if (rr_type < 0) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution of '%s', invalid resource record type '%d'\n",
+			name, rr_type);
+		return -1;
+	} else if (rr_class > 65536) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution of '%s', resource record class '%d' exceeds maximum\n",
+			name, rr_class);
+		return -1;
+	} else if (rr_class < 0) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution of '%s', invalid resource class '%d'\n",
+			name, rr_class);
+		return -1;
+	} else if (!result) {
+		ast_log(LOG_WARNING, "Could not perform synchronous resolution of '%s', no result pointer provided for storing results\n",
+			name);
+		return -1;
+	}
+
+	synchronous = ao2_alloc_options(sizeof(*synchronous), dns_synchronous_resolve_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!synchronous) {
+		return -1;
+	}
+
+	ast_mutex_init(&synchronous->lock);
+	ast_cond_init(&synchronous->cond, NULL);
+
+	active = ast_dns_resolve_async(name, rr_type, rr_class, dns_synchronous_resolve_callback, synchronous);
+	if (active) {
+		/* Wait for resolution to complete */
+		ast_mutex_lock(&synchronous->lock);
+		while (!synchronous->completed) {
+			ast_cond_wait(&synchronous->cond, &synchronous->lock);
+		}
+		ast_mutex_unlock(&synchronous->lock);
+		ao2_ref(active, -1);
+	}
+
+	*result = synchronous->result;
+	ao2_ref(synchronous, -1);
+
+	return *result ? 0 : -1;
+}
+
+int ast_dns_resolver_set_data(struct ast_dns_query *query, void *data)
+{
+	if (query->resolver_data) {
+		return -1;
+	}
+
+	query->resolver_data = ao2_bump(data);
+
+	return 0;
+}
+
+void *ast_dns_resolver_get_data(const struct ast_dns_query *query)
+{
+	return query->resolver_data;
+}
+
+int ast_dns_resolver_set_result(struct ast_dns_query *query, unsigned int secure, unsigned int bogus,
+	unsigned int rcode, const char *canonical, const char *answer, size_t answer_size)
+{
+	char *buf_ptr;
+
+	if (secure && bogus) {
+		ast_debug(2, "Query '%p': Could not set result information, it can not be both secure and bogus\n",
+			query);
+		return -1;
+	}
+
+	if (ast_strlen_zero(canonical)) {
+		ast_debug(2, "Query '%p': Could not set result information since no canonical name was provided\n",
+			query);
+		return -1;
+	}
+
+	if (!answer) {
+		answer = "";
+		answer_size = 0;
+		ast_debug(2, "Query '%p': Assuming zero-sized answer on NULL input\n", query);
+	}
+
+	ast_dns_result_free(query->result);
+
+	query->result = ast_calloc(1, sizeof(*query->result) + strlen(canonical) + 1 + answer_size);
+	if (!query->result) {
+		return -1;
+	}
+
+	query->result->secure = secure;
+	query->result->bogus = bogus;
+	query->result->rcode = rcode;
+
+	buf_ptr = query->result->buf;
+	strcpy(buf_ptr, canonical); /* SAFE */
+	query->result->canonical = buf_ptr;
+
+	buf_ptr += strlen(canonical) + 1;
+	memcpy(buf_ptr, answer, answer_size); /* SAFE */
+	query->result->answer = buf_ptr;
+	query->result->answer_size = answer_size;
+
+	return 0;
+}
+
+static struct ast_dns_record *generic_record_alloc(struct ast_dns_query *query, const char *data, const size_t size)
+{
+	struct ast_dns_record *record;
+
+	record = ast_calloc(1, sizeof(*record) + size);
+	if (!record) {
+		return NULL;
+	}
+
+	record->data_ptr = record->data;
+
+	return record;
+}
+
+typedef struct ast_dns_record *(*dns_alloc_fn)(struct ast_dns_query *query, const char *data, const size_t size);
+
+static dns_alloc_fn dns_alloc_table [] = {
+	[T_NAPTR] = dns_naptr_alloc,
+	[T_SRV] = dns_srv_alloc,
+};
+
+static struct ast_dns_record *allocate_dns_record(unsigned int rr_type, struct ast_dns_query *query, const char *data, const size_t size)
+{
+	dns_alloc_fn allocator = generic_record_alloc;
+
+	if (rr_type < ARRAY_LEN(dns_alloc_table) && dns_alloc_table[rr_type]) {
+		allocator = dns_alloc_table[rr_type];
+	}
+
+	return allocator(query, data, size);
+}
+
+int ast_dns_resolver_add_record(struct ast_dns_query *query, int rr_type, int rr_class, int ttl, const char *data, const size_t size)
+{
+	struct ast_dns_record *record;
+
+	if (rr_type < 0) {
+		ast_debug(2, "Query '%p': Could not add record, invalid resource record type '%d'\n",
+			query, rr_type);
+		return -1;
+	} else if (rr_type > 65536) {
+		ast_debug(2, "Query '%p': Could not add record, resource record type '%d' exceeds maximum\n",
+			query, rr_type);
+		return -1;
+	} else if (rr_class < 0) {
+		ast_debug(2, "Query '%p': Could not add record, invalid resource record class '%d'\n",
+			query, rr_class);
+		return -1;
+	} else if (rr_class > 65536) {
+		ast_debug(2, "Query '%p': Could not add record, resource record class '%d' exceeds maximum\n",
+			query, rr_class);
+		return -1;
+	} else if (ttl < 0) {
+		ast_debug(2, "Query '%p': Could not add record, invalid TTL '%d'\n",
+			query, ttl);
+		return -1;
+	} else if (!data || !size) {
+		ast_debug(2, "Query '%p': Could not add record, no data specified\n",
+			query);
+		return -1;
+	} else if (!query->result) {
+		ast_debug(2, "Query '%p': No result was set on the query, thus records can not be added\n",
+			query);
+		return -1;
+	}
+
+	record = allocate_dns_record(rr_type, query, data, size);
+	if (!record) {
+		return -1;
+	}
+
+	record->rr_type = rr_type;
+	record->rr_class = rr_class;
+	record->ttl = ttl;
+	record->data_len = size;
+	memcpy(record->data_ptr, data, size);
+
+	AST_LIST_INSERT_TAIL(&query->result->records, record, list);
+
+	return 0;
+}
+
+typedef void (*dns_sort_fn)(struct ast_dns_result *result);
+
+static dns_sort_fn dns_sort_table [] = {
+	[T_NAPTR] = dns_naptr_sort,
+	[T_SRV] = dns_srv_sort,
+};
+
+static void sort_result(int rr_type, struct ast_dns_result *result)
+{
+	if (dns_sort_table[rr_type]) {
+		dns_sort_table[rr_type](result);
+	}
+}
+
+void ast_dns_resolver_completed(struct ast_dns_query *query)
+{
+	sort_result(ast_dns_query_get_rr_type(query), query->result);
+
+	query->callback(query);
+}
+
+static void dns_shutdown(void)
+{
+	if (sched) {
+		ast_sched_context_destroy(sched);
+		sched = NULL;
+	}
+}
+
+int dns_core_init(void)
+{
+	sched = ast_sched_context_create();
+	if (!sched) {
+		return -1;
+	}
+
+	if (ast_sched_start_thread(sched)) {
+		return -1;
+	}
+
+	ast_register_cleanup(dns_shutdown);
+
+	return 0;
+}
+
+int ast_dns_resolver_register(struct ast_dns_resolver *resolver)
+{
+	struct ast_dns_resolver *iter;
+	int inserted = 0;
+
+	if (!resolver) {
+		return -1;
+	} else if (ast_strlen_zero(resolver->name)) {
+		ast_log(LOG_ERROR, "Registration of DNS resolver failed as it does not have a name\n");
+		return -1;
+	} else if (!resolver->resolve) {
+		ast_log(LOG_ERROR, "DNS resolver '%s' does not implement the resolve callback which is required\n",
+			resolver->name);
+		return -1;
+	} else if (!resolver->cancel) {
+		ast_log(LOG_ERROR, "DNS resolver '%s' does not implement the cancel callback which is required\n",
+			resolver->name);
+		return -1;
+	}
+
+	AST_RWLIST_WRLOCK(&resolvers);
+
+	AST_LIST_TRAVERSE(&resolvers, iter, next) {
+		if (!strcmp(iter->name, resolver->name)) {
+			ast_log(LOG_ERROR, "A DNS resolver with the name '%s' is already registered\n", resolver->name);
+			AST_RWLIST_UNLOCK(&resolvers);
+			return -1;
+		}
+	}
+
+	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&resolvers, iter, next) {
+		if (iter->priority > resolver->priority) {
+			AST_RWLIST_INSERT_BEFORE_CURRENT(resolver, next);
+			inserted = 1;
+			break;
+		}
+	}
+	AST_RWLIST_TRAVERSE_SAFE_END;
+
+	if (!inserted) {
+		AST_RWLIST_INSERT_TAIL(&resolvers, resolver, next);
+	}
+
+	AST_RWLIST_UNLOCK(&resolvers);
+
+	ast_verb(2, "Registered DNS resolver '%s' with priority '%d'\n", resolver->name, resolver->priority);
+
+	return 0;
+}
+
+void ast_dns_resolver_unregister(struct ast_dns_resolver *resolver)
+{
+	struct ast_dns_resolver *iter;
+
+	if (!resolver) {
+		return;
+	}
+
+	AST_RWLIST_WRLOCK(&resolvers);
+	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&resolvers, iter, next) {
+		if (resolver == iter) {
+			AST_RWLIST_REMOVE_CURRENT(next);
+			break;
+		}
+	}
+	AST_RWLIST_TRAVERSE_SAFE_END;
+	AST_RWLIST_UNLOCK(&resolvers);
+
+	ast_verb(2, "Unregistered DNS resolver '%s'\n", resolver->name);
+}
+
+char *dns_find_record(const char *record, size_t record_size, const char *response, size_t response_size)
+{
+	size_t remaining_size = response_size;
+	const char *search_base = response;
+	char *record_offset;
+
+	while (1) {
+		record_offset = memchr(search_base, record[0], remaining_size);
+
+		ast_assert(record_offset != NULL);
+		ast_assert(search_base + remaining_size - record_offset >= record_size);
+
+		if (!memcmp(record_offset, record, record_size)) {
+			return record_offset;
+		}
+
+		remaining_size -= record_offset - search_base;
+		search_base = record_offset + 1;
+	}
+}
+
+int dns_parse_short(unsigned char *cur, uint16_t *val)
+{
+	/* This assignment takes a big-endian 16-bit value and stores it in the
+	 * machine's native byte order. Using this method allows us to avoid potential
+	 * alignment issues in case the order is not on a short-addressable boundary.
+	 * See http://commandcenter.blogspot.com/2012/04/byte-order-fallacy.html for
+	 * more information
+	 */
+	*val = (cur[1] << 0) | (cur[0] << 8);
+	return sizeof(*val);
+}
+
+int dns_parse_string(char *cur, uint8_t *size, char **val)
+{
+	*size = *cur++;
+	*val = cur;
+	return *size + 1;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dnsmgr.c 1:16.1.1~dfsg-1/main/dnsmgr.c
--- 1:13.23.1~dfsg-2/main/dnsmgr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dnsmgr.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,12 +45,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include <regex.h>
 #include <signal.h>
 
+#include "asterisk/module.h"
 #include "asterisk/dnsmgr.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/utils.h"
@@ -408,7 +407,7 @@ static struct ast_cli_entry cli_reload =
 static struct ast_cli_entry cli_refresh = AST_CLI_DEFINE(handle_cli_refresh, "Performs an immediate refresh");
 static struct ast_cli_entry cli_status = AST_CLI_DEFINE(handle_cli_status, "Display the DNS manager status");
 
-static void dnsmgr_shutdown(void)
+static int unload_module(void)
 {
 	ast_cli_unregister(&cli_reload);
 	ast_cli_unregister(&cli_status);
@@ -426,24 +425,24 @@ static void dnsmgr_shutdown(void)
 	ast_mutex_unlock(&refresh_lock);
 
 	ast_sched_context_destroy(sched);
+
+	return 0;
 }
 
-int dnsmgr_init(void)
+static int load_module(void)
 {
 	if (!(sched = ast_sched_context_create())) {
 		ast_log(LOG_ERROR, "Unable to create schedule context.\n");
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	ast_cli_register(&cli_reload);
 	ast_cli_register(&cli_status);
 	ast_cli_register(&cli_refresh);
 
-	ast_register_cleanup(dnsmgr_shutdown);
-
-	return do_reload(1);
+	return do_reload(1) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
 
-int dnsmgr_reload(void)
+static int reload_module(void)
 {
 	return do_reload(0);
 }
@@ -517,3 +516,12 @@ static int do_reload(int loading)
 
 	return 0;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "DNS Manager",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/dns_naptr.c 1:16.1.1~dfsg-1/main/dns_naptr.c
--- 1:13.23.1~dfsg-2/main/dns_naptr.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_naptr.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,646 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief DNS NAPTR Record Support
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+#include <netinet/in.h>
+#include <resolv.h>
+#include <regex.h>
+
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_naptr.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/dns_internal.h"
+#include "asterisk/utils.h"
+
+/*!
+ * \brief Result of analyzing NAPTR flags on a record
+ */
+enum flags_result {
+	/*! Terminal record, meaning the DDDS algorithm can be stopped */
+	FLAGS_TERMINAL,
+	/*! No flags provided, likely meaning another NAPTR lookup */
+	FLAGS_EMPTY,
+	/*! Unrecognized but valid flags. We cannot conclude what they mean */
+	FLAGS_UNKNOWN,
+	/*! Non-alphanumeric or invalid combination of flags */
+	FLAGS_INVALID,
+};
+
+/*!
+ * \brief Analyze and interpret NAPTR flags as per RFC 3404
+ *
+ * \note The flags string passed into this function is NOT NULL-terminated
+ *
+ * \param flags The flags string from a NAPTR record
+ * \flags_size The size of the flags string in bytes
+ * \return flag result
+ */
+static enum flags_result interpret_flags(const char *flags, uint8_t flags_size)
+{
+	int i;
+	char known_flag_found = 0;
+
+	if (flags_size == 0) {
+		return FLAGS_EMPTY;
+	}
+
+	/* Take care of the most common (and easy) case, one character */
+	if (flags_size == 1) {
+		if (*flags == 's' || *flags == 'S' ||
+				*flags == 'a' || *flags == 'A' ||
+				*flags == 'u' || *flags == 'U') {
+			return FLAGS_TERMINAL;
+		} else if (!isalnum(*flags)) {
+			return FLAGS_INVALID;
+		} else {
+			return FLAGS_UNKNOWN;
+		}
+	}
+
+	/*
+	 * Multiple flags are allowed, but you cannot mix the
+	 * S, A, U, and P flags together.
+	 */
+	for (i = 0; i < flags_size; ++i) {
+		if (!isalnum(flags[i])) {
+			return FLAGS_INVALID;
+		} else if (flags[i] == 's' || flags[i] == 'S') {
+			if (known_flag_found && known_flag_found != 's') {
+				return FLAGS_INVALID;
+			}
+			known_flag_found = 's';
+		} else if (flags[i] == 'u' || flags[i] == 'U') {
+			if (known_flag_found && known_flag_found != 'u') {
+				return FLAGS_INVALID;
+			}
+			known_flag_found = 'u';
+		} else if (flags[i] == 'a' || flags[i] == 'A') {
+			if (known_flag_found && known_flag_found != 'a') {
+				return FLAGS_INVALID;
+			}
+			known_flag_found = 'a';
+		} else if (flags[i] == 'p' || flags[i] == 'P') {
+			if (known_flag_found && known_flag_found != 'p') {
+				return FLAGS_INVALID;
+			}
+			known_flag_found = 'p';
+		}
+	}
+
+	return (!known_flag_found || known_flag_found == 'p') ? FLAGS_UNKNOWN : FLAGS_TERMINAL;
+}
+
+/*!
+ * \brief Analyze NAPTR services for validity as defined by RFC 3404
+ *
+ * \note The services string passed to this function is NOT NULL-terminated
+ * \param services The services string parsed from a NAPTR record
+ * \param services_size The size of the services string
+ * \retval 0 Services are valid
+ * \retval -1 Services are invalid
+ */
+static int services_invalid(const char *services, uint8_t services_size)
+{
+	const char *current_pos = services;
+	const char *end_of_services = services + services_size;
+
+	if (services_size == 0) {
+		return 0;
+	}
+
+	/* Services are broken into sections divided by a + sign. Each section
+	 * must start with an alphabetic character, and then can only contain
+	 * alphanumeric characters. The size of any section is limited to
+	 * 32 characters
+	 */
+	while (1) {
+		char *plus_pos = memchr(current_pos, '+', end_of_services - current_pos);
+		uint8_t current_size = plus_pos ? plus_pos - current_pos : end_of_services - current_pos;
+		int i;
+
+		if (!isalpha(current_pos[0])) {
+			return -1;
+		}
+
+		if (current_size > 32) {
+			return -1;
+		}
+
+		for (i = 1; i < current_size; ++i) {
+			if (!isalnum(current_pos[i])) {
+				return -1;
+			}
+		}
+
+		if (!plus_pos) {
+			break;
+		}
+		current_pos = plus_pos + 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Determine if flags in the regexp are invalid
+ *
+ * A NAPTR regexp is structured like so
+ * /pattern/repl/FLAGS
+ *
+ * This ensures that the flags on the regexp are valid. Regexp
+ * flags can either be zero or one character long. If the flags
+ * are one character long, that character must be "i" to indicate
+ * the regex evaluation is case-insensitive.
+ *
+ * \note The flags string passed to this function is not NULL-terminated
+ * \param flags The regexp flags from the NAPTR record
+ * \param end A pointer to the end of the flags string
+ * \retval 0 Flags are valid
+ * \retval -1 Flags are invalid
+ */
+static int regexp_flags_invalid(const char *flags, const char *end)
+{
+	if (flags >= end) {
+		return 0;
+	}
+
+	if (end - flags > 1) {
+		return -1;
+	}
+
+	if (*flags != 'i') {
+		return -1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Determine if the replacement in the regexp is invalid
+ *
+ * A NAPTR regexp is structured like so
+ * /pattern/REPL/flags
+ *
+ * This ensures that the replacement on the regexp is valid. The regexp
+ * replacement is free to use any character it wants, plus backreferences
+ * and an escaped regexp delimiter.
+ *
+ * This function does not attempt to ensure that the backreferences refer
+ * to valid portions of the regexp's regex pattern.
+ *
+ * \note The repl string passed to this function is NOT NULL-terminated
+ *
+ * \param repl The regexp replacement string
+ * \param end Pointer to the end of the replacement string
+ * \param delim The delimiter character for the regexp
+ *
+ * \retval 0 Replacement is valid
+ * \retval -1 Replacement is invalid
+ */
+static int regexp_repl_invalid(const char *repl, const char *end, char delim)
+{
+	const char *ptr = repl;
+
+	if (repl == end) {
+		/* Kind of weird, but this is fine */
+		return 0;
+	}
+
+	while (1) {
+		char *backslash_pos = memchr(ptr, '\\', end - ptr);
+		if (!backslash_pos) {
+			break;
+		}
+
+		ast_assert(backslash_pos < end - 1);
+
+		/* XXX RFC 3402 is unclear about whether other backslash-escaped characters
+		 * (such as a backslash-escaped backslash) are legal
+		 */
+		if (!strchr("12345689", backslash_pos[1]) && backslash_pos[1] != delim) {
+			return -1;
+		}
+
+		ptr = backslash_pos + 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Determine if the pattern in a regexp is invalid
+ *
+ * A NAPTR regexp is structured like so
+ * /PATTERN/repl/flags
+ *
+ * This ensures that the pattern on the regexp is valid. The pattern is
+ * passed to a regex compiler to determine its validity.
+ *
+ * \note The pattern string passed to this function is NOT NULL-terminated
+ *
+ * \param pattern The pattern from the NAPTR record
+ * \param end A pointer to the end of the pattern
+ *
+ * \retval 0 Pattern is valid
+ * \retval non-zero Pattern is invalid
+ */
+static int regexp_pattern_invalid(const char *pattern, const char *end)
+{
+	int pattern_size = end - pattern;
+	char pattern_str[pattern_size + 1];
+	regex_t reg;
+	int res;
+
+	/* regcomp requires a NULL-terminated string */
+	memcpy(pattern_str, pattern, pattern_size);
+	pattern_str[pattern_size] = '\0';
+
+	res = regcomp(&reg, pattern_str, REG_EXTENDED);
+
+	regfree(&reg);
+
+	return res;
+}
+
+/*!
+ * \brief Determine if the regexp in a NAPTR record is invalid
+ *
+ * The goal of this function is to divide the regexp into its
+ * constituent parts and then let validation subroutines determine
+ * if each part is valid. If all parts are valid, then the entire
+ * regexp is valid.
+ *
+ * \note The regexp string passed to this function is NOT NULL-terminated
+ *
+ * \param regexp The regexp from the NAPTR record
+ * \param regexp_size The size of the regexp string
+ *
+ * \retval 0 regexp is valid
+ * \retval non-zero regexp is invalid
+ */
+static int regexp_invalid(const char *regexp, uint8_t regexp_size)
+{
+	char delim;
+	const char *delim2_pos;
+	const char *delim3_pos;
+	const char *ptr = regexp;
+	const char *end_of_regexp = regexp + regexp_size;
+	const char *regex_pos;
+	const char *repl_pos;
+	const char *flags_pos;
+
+	if (regexp_size == 0) {
+		return 0;
+	}
+
+	/* The delimiter will be a ! or / in most cases, but the rules allow
+	 * for the delimiter to be nearly any character. It cannot be 'i' because
+	 * the delimiter cannot be the same as regexp flags. The delimiter cannot
+	 * be 1-9 because the delimiter cannot be a backreference number. RFC
+	 * 2915 specified that backslash was also not allowed as a delimiter, but
+	 * RFC 3402 does not say this. We've gone ahead and made the character
+	 * illegal for our purposes.
+	 */
+	delim = *ptr;
+	if (strchr("123456789\\i", delim)) {
+		return -1;
+	}
+	++ptr;
+	regex_pos = ptr;
+
+	/* Find the other two delimiters. If the delim is escaped with a backslash, it doesn't count */
+	while (1) {
+		delim2_pos = memchr(ptr, delim, end_of_regexp - ptr);
+		if (!delim2_pos) {
+			return -1;
+		}
+		ptr = delim2_pos + 1;
+		if (delim2_pos[-1] != '\\') {
+			break;
+		}
+	}
+
+	if (ptr >= end_of_regexp) {
+		return -1;
+	}
+
+	repl_pos = ptr;
+
+	while (1) {
+		delim3_pos = memchr(ptr, delim, end_of_regexp - ptr);
+		if (!delim3_pos) {
+			return -1;
+		}
+		ptr = delim3_pos + 1;
+		if (delim3_pos[-1] != '\\') {
+			break;
+		}
+	}
+	flags_pos = ptr;
+
+	if (regexp_flags_invalid(flags_pos, end_of_regexp) ||
+			regexp_repl_invalid(repl_pos, delim3_pos, delim) ||
+			regexp_pattern_invalid(regex_pos, delim2_pos)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+#define PAST_END_OF_RECORD ptr >= end_of_record
+
+struct ast_dns_record *dns_naptr_alloc(struct ast_dns_query *query, const char *data, const size_t size)
+{
+	struct ast_dns_naptr_record *naptr;
+	char *ptr = NULL;
+	uint16_t order;
+	uint16_t preference;
+	uint8_t flags_size;
+	char *flags;
+	uint8_t services_size;
+	char *services;
+	uint8_t regexp_size;
+	char *regexp;
+	char replacement[256] = "";
+	int replacement_size;
+	const char *end_of_record;
+	enum flags_result flags_res;
+	size_t naptr_len;
+
+	ptr = dns_find_record(data, size, query->result->answer, query->result->answer_size);
+	ast_assert(ptr != NULL);
+
+	end_of_record = ptr + size;
+
+	/* ORDER */
+	/* This assignment takes a big-endian 16-bit value and stores it in the
+	 * machine's native byte order. Using this method allows us to avoid potential
+	 * alignment issues in case the order is not on a short-addressable boundary.
+	 * See http://commandcenter.blogspot.com/2012/04/byte-order-fallacy.html for
+	 * more information
+	 */
+	ptr += dns_parse_short((unsigned char *) ptr, &order);
+	if (PAST_END_OF_RECORD) {
+		return NULL;
+	}
+
+	/* PREFERENCE */
+	ptr += dns_parse_short((unsigned char *) ptr, &preference);
+	if (PAST_END_OF_RECORD) {
+		return NULL;
+	}
+
+	/* FLAGS */
+	ptr += dns_parse_string(ptr, &flags_size, &flags);
+	if (PAST_END_OF_RECORD) {
+		return NULL;
+	}
+
+	/* SERVICES */
+	ptr += dns_parse_string(ptr, &services_size, &services);
+	if (PAST_END_OF_RECORD) {
+		return NULL;
+	}
+
+	/* REGEXP */
+	ptr += dns_parse_string(ptr, &regexp_size, &regexp);
+	if (PAST_END_OF_RECORD) {
+		return NULL;
+	}
+
+	/*
+	 * The return value from dn_expand represents the size of the replacement
+	 * in the buffer which MAY be compressed.  Since the expanded replacement
+	 * is NULL terminated, you can use strlen() to get the expanded size.
+	 */
+	replacement_size = dn_expand((unsigned char *)query->result->answer,
+		(unsigned char *) end_of_record, (unsigned char *) ptr,
+		replacement, sizeof(replacement) - 1);
+	if (replacement_size < 0) {
+		ast_log(LOG_ERROR, "Failed to expand domain name: %s\n", strerror(errno));
+		return NULL;
+	}
+
+	ptr += replacement_size;
+
+	if (ptr != end_of_record) {
+		ast_log(LOG_ERROR, "NAPTR record gave undersized string indications.\n");
+		return NULL;
+	}
+
+	/* We've validated the size of the NAPTR record. Now we can validate
+	 * the individual parts
+	 */
+	flags_res = interpret_flags(flags, flags_size);
+	if (flags_res == FLAGS_INVALID) {
+		ast_log(LOG_ERROR, "NAPTR Record contained invalid flags %.*s\n", flags_size, flags);
+		return NULL;
+	}
+
+	if (services_invalid(services, services_size)) {
+		ast_log(LOG_ERROR, "NAPTR record contained invalid services %.*s\n", services_size, services);
+		return NULL;
+	}
+
+	if (regexp_invalid(regexp, regexp_size)) {
+		ast_log(LOG_ERROR, "NAPTR record contained invalid regexp %.*s\n", regexp_size, regexp);
+		return NULL;
+	}
+
+	/* replacement_size takes into account the NULL label, so a NAPTR record with no replacement
+	 * will have a replacement_size of 1.
+	 */
+	if (regexp_size && replacement_size > 1) {
+		ast_log(LOG_ERROR, "NAPTR record contained both a regexp and replacement\n");
+		return NULL;
+	}
+
+	naptr_len = sizeof(*naptr) + size + flags_size + 1 + services_size + 1
+		+ regexp_size + 1 + strlen(replacement) + 1;
+	naptr = ast_calloc(1, naptr_len);
+	if (!naptr) {
+		return NULL;
+	}
+
+	naptr->order = order;
+	naptr->preference = preference;
+
+	ptr = naptr->data;
+	ptr += size;
+
+	strncpy(ptr, flags, flags_size);
+	ptr[flags_size] = '\0';
+	naptr->flags = ptr;
+	ptr += flags_size + 1;
+
+	strncpy(ptr, services, services_size);
+	ptr[services_size] = '\0';
+	naptr->service = ptr;
+	ptr += services_size + 1;
+
+	strncpy(ptr, regexp, regexp_size);
+	ptr[regexp_size] = '\0';
+	naptr->regexp = ptr;
+	ptr += regexp_size + 1;
+
+	strcpy(ptr, replacement);
+	naptr->replacement = ptr;
+
+	naptr->generic.data_ptr = naptr->data;
+
+	return (struct ast_dns_record *)naptr;
+}
+
+
+static int compare_order(const void *record1, const void *record2)
+{
+	const struct ast_dns_naptr_record **left = (const struct ast_dns_naptr_record **)record1;
+	const struct ast_dns_naptr_record **right = (const struct ast_dns_naptr_record **)record2;
+
+	if ((*left)->order < (*right)->order) {
+		return -1;
+	} else if ((*left)->order > (*right)->order) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+static int compare_preference(const void *record1, const void *record2)
+{
+	const struct ast_dns_naptr_record **left = (const struct ast_dns_naptr_record **)record1;
+	const struct ast_dns_naptr_record **right = (const struct ast_dns_naptr_record **)record2;
+
+	if ((*left)->preference < (*right)->preference) {
+		return -1;
+	} else if ((*left)->preference > (*right)->preference) {
+		return 1;
+	} else {
+		return 0;
+	}
+}
+
+void dns_naptr_sort(struct ast_dns_result *result)
+{
+	struct ast_dns_record *current;
+	size_t num_records = 0;
+	struct ast_dns_naptr_record **records;
+	int i = 0;
+	int j = 0;
+	int cur_order;
+
+	/* Determine the number of records */
+	AST_LIST_TRAVERSE(&result->records, current, list) {
+		++num_records;
+	}
+
+	/* No point in continuing if there are no records */
+	if (num_records == 0) {
+		return;
+	}
+
+	/* Allocate an array with that number of records */
+	records = ast_alloca(num_records * sizeof(*records));
+
+	/* Move records from the list to the array */
+	AST_LIST_TRAVERSE_SAFE_BEGIN(&result->records, current, list) {
+		records[i++] = (struct ast_dns_naptr_record *) current;
+		AST_LIST_REMOVE_CURRENT(list);
+	}
+	AST_LIST_TRAVERSE_SAFE_END;
+
+	/* Sort the array by order */
+	qsort(records, num_records, sizeof(*records), compare_order);
+
+	/* Sort subarrays by preference */
+	for (i = 0; i < num_records; i = j) {
+		cur_order = records[i]->order;
+		for (j = i + 1; j < num_records; ++j) {
+			if (records[j]->order != cur_order) {
+				break;
+			}
+		}
+		qsort(&records[i], j - i, sizeof(*records), compare_preference);
+	}
+
+	/* Place sorted records back into the original list */
+	for (i = 0; i < num_records; ++i) {
+		AST_LIST_INSERT_TAIL(&result->records, (struct ast_dns_record *)(records[i]), list);
+	}
+}
+
+const char *ast_dns_naptr_get_flags(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->flags;
+}
+
+const char *ast_dns_naptr_get_service(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->service;
+}
+
+const char *ast_dns_naptr_get_regexp(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->regexp;
+}
+
+const char *ast_dns_naptr_get_replacement(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->replacement;
+}
+
+unsigned short ast_dns_naptr_get_order(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->order;
+}
+
+unsigned short ast_dns_naptr_get_preference(const struct ast_dns_record *record)
+{
+	struct ast_dns_naptr_record *naptr = (struct ast_dns_naptr_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_NAPTR);
+	return naptr->preference;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dns_query_set.c 1:16.1.1~dfsg-1/main/dns_query_set.c
--- 1:13.23.1~dfsg-2/main/dns_query_set.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_query_set.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,284 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief DNS Query Set API
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/vector.h"
+#include "asterisk/astobj2.h"
+#include "asterisk/utils.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_query_set.h"
+#include "asterisk/dns_internal.h"
+#include "asterisk/dns_resolver.h"
+
+/*! \brief The default number of expected queries to be added to the query set */
+#define DNS_QUERY_SET_EXPECTED_QUERY_COUNT 5
+
+/*! \brief Destructor for DNS query set */
+static void dns_query_set_destroy(void *data)
+{
+	struct ast_dns_query_set *query_set = data;
+	int idx;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&query_set->queries); ++idx) {
+		struct dns_query_set_query *query = AST_VECTOR_GET_ADDR(&query_set->queries, idx);
+
+		ao2_ref(query->query, -1);
+	}
+	AST_VECTOR_FREE(&query_set->queries);
+
+	ao2_cleanup(query_set->user_data);
+}
+
+struct ast_dns_query_set *ast_dns_query_set_create(void)
+{
+	struct ast_dns_query_set *query_set;
+
+	query_set = ao2_alloc_options(sizeof(*query_set), dns_query_set_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!query_set) {
+		return NULL;
+	}
+
+	if (AST_VECTOR_INIT(&query_set->queries, DNS_QUERY_SET_EXPECTED_QUERY_COUNT)) {
+		ao2_ref(query_set, -1);
+		return NULL;
+	}
+
+	return query_set;
+}
+
+/*! \brief Callback invoked upon completion of a DNS query */
+static void dns_query_set_callback(const struct ast_dns_query *query)
+{
+	struct ast_dns_query_set *query_set = ast_dns_query_get_data(query);
+
+	/* The reference count of the query set is bumped here in case this query holds the last reference */
+	ao2_ref(query_set, +1);
+
+	/* Drop the query set from the query so the query set can be destroyed if this is the last one */
+	ao2_cleanup(((struct ast_dns_query *)query)->user_data);
+	((struct ast_dns_query *)query)->user_data = NULL;
+
+	if (ast_atomic_fetchadd_int(&query_set->queries_completed, +1) != (AST_VECTOR_SIZE(&query_set->queries) - 1)) {
+		ao2_ref(query_set, -1);
+		return;
+	}
+
+	/* All queries have been completed, invoke final callback */
+	if (query_set->queries_cancelled != AST_VECTOR_SIZE(&query_set->queries)) {
+		query_set->callback(query_set);
+	}
+
+	ao2_cleanup(query_set->user_data);
+	query_set->user_data = NULL;
+
+	ao2_ref(query_set, -1);
+}
+
+int ast_dns_query_set_add(struct ast_dns_query_set *query_set, const char *name, int rr_type, int rr_class)
+{
+	struct dns_query_set_query query = {
+		.started = 0,
+	};
+
+	ast_assert(!query_set->in_progress);
+	if (query_set->in_progress) {
+		ast_log(LOG_ERROR, "Attempted to add additional query to query set '%p' after resolution has started\n",
+			query_set);
+		return -1;
+	}
+
+	/*
+	 * We are intentionally passing NULL for the user data even
+	 * though dns_query_set_callback() is not NULL tolerant.  Doing
+	 * this avoids a circular reference chain until the queries are
+	 * started.  ast_dns_query_set_resolve_async() will set the
+	 * query user_data for us later when we actually kick off the
+	 * queries.
+	 */
+	query.query = dns_query_alloc(name, rr_type, rr_class, dns_query_set_callback, NULL);
+	if (!query.query) {
+		return -1;
+	}
+
+	if (AST_VECTOR_APPEND(&query_set->queries, query)) {
+		ao2_ref(query.query, -1);
+		return -1;
+	}
+
+	return 0;
+}
+
+size_t ast_dns_query_set_num_queries(const struct ast_dns_query_set *query_set)
+{
+	return AST_VECTOR_SIZE(&query_set->queries);
+}
+
+struct ast_dns_query *ast_dns_query_set_get(const struct ast_dns_query_set *query_set, unsigned int index)
+{
+	/* Only once all queries have been completed can results be retrieved */
+	if (query_set->queries_completed != AST_VECTOR_SIZE(&query_set->queries)) {
+		return NULL;
+	}
+
+	/* If the index exceeds the number of queries... no query for you */
+	if (index >= AST_VECTOR_SIZE(&query_set->queries)) {
+		return NULL;
+	}
+
+	return AST_VECTOR_GET_ADDR(&query_set->queries, index)->query;
+}
+
+void *ast_dns_query_set_get_data(const struct ast_dns_query_set *query_set)
+{
+	return query_set->user_data;
+}
+
+void ast_dns_query_set_resolve_async(struct ast_dns_query_set *query_set, ast_dns_query_set_callback callback, void *data)
+{
+	int idx;
+
+	ast_assert(!query_set->in_progress);
+	if (query_set->in_progress) {
+		ast_log(LOG_ERROR, "Attempted to start asynchronous resolution of query set '%p' when it has already started\n",
+			query_set);
+		return;
+	}
+
+	query_set->in_progress = 1;
+	query_set->callback = callback;
+	query_set->user_data = ao2_bump(data);
+
+	/*
+	 * Bump the query_set ref in case all queries complete
+	 * before we are done kicking them off.
+	 */
+	ao2_ref(query_set, +1);
+	for (idx = 0; idx < AST_VECTOR_SIZE(&query_set->queries); ++idx) {
+		struct dns_query_set_query *query = AST_VECTOR_GET_ADDR(&query_set->queries, idx);
+
+		query->query->user_data = ao2_bump(query_set);
+
+		if (!query->query->resolver->resolve(query->query)) {
+			query->started = 1;
+			continue;
+		}
+
+		dns_query_set_callback(query->query);
+	}
+	if (!idx) {
+		/*
+		 * There were no queries in the set;
+		 * therefore all queries are "completed".
+		 * Invoke the final callback.
+		 */
+		query_set->callback(query_set);
+		ao2_cleanup(query_set->user_data);
+		query_set->user_data = NULL;
+	}
+	ao2_ref(query_set, -1);
+}
+
+/*! \brief Structure used for signaling back for synchronous resolution completion */
+struct dns_synchronous_resolve {
+	/*! \brief Lock used for signaling */
+	ast_mutex_t lock;
+	/*! \brief Condition used for signaling */
+	ast_cond_t cond;
+	/*! \brief Whether the query has completed */
+	unsigned int completed;
+};
+
+/*! \brief Destructor for synchronous resolution structure */
+static void dns_synchronous_resolve_destroy(void *data)
+{
+	struct dns_synchronous_resolve *synchronous = data;
+
+	ast_mutex_destroy(&synchronous->lock);
+	ast_cond_destroy(&synchronous->cond);
+}
+
+/*! \brief Callback used to implement synchronous resolution */
+static void dns_synchronous_resolve_callback(const struct ast_dns_query_set *query_set)
+{
+	struct dns_synchronous_resolve *synchronous = ast_dns_query_set_get_data(query_set);
+
+	ast_mutex_lock(&synchronous->lock);
+	synchronous->completed = 1;
+	ast_cond_signal(&synchronous->cond);
+	ast_mutex_unlock(&synchronous->lock);
+}
+
+int ast_query_set_resolve(struct ast_dns_query_set *query_set)
+{
+	struct dns_synchronous_resolve *synchronous;
+
+	synchronous = ao2_alloc_options(sizeof(*synchronous), dns_synchronous_resolve_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!synchronous) {
+		return -1;
+	}
+
+	ast_mutex_init(&synchronous->lock);
+	ast_cond_init(&synchronous->cond, NULL);
+
+	ast_dns_query_set_resolve_async(query_set, dns_synchronous_resolve_callback, synchronous);
+
+	/* Wait for resolution to complete */
+	ast_mutex_lock(&synchronous->lock);
+	while (!synchronous->completed) {
+		ast_cond_wait(&synchronous->cond, &synchronous->lock);
+	}
+	ast_mutex_unlock(&synchronous->lock);
+
+	ao2_ref(synchronous, -1);
+
+	return 0;
+}
+
+int ast_dns_query_set_resolve_cancel(struct ast_dns_query_set *query_set)
+{
+	int idx;
+	size_t query_count = AST_VECTOR_SIZE(&query_set->queries);
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&query_set->queries); ++idx) {
+		struct dns_query_set_query *query = AST_VECTOR_GET_ADDR(&query_set->queries, idx);
+
+		if (query->started) {
+			if (!query->query->resolver->cancel(query->query)) {
+				query_set->queries_cancelled++;
+				dns_query_set_callback(query->query);
+			}
+		} else {
+			query_set->queries_cancelled++;
+		}
+	}
+
+	return (query_set->queries_cancelled == query_count) ? 0 : -1;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dns_recurring.c 1:16.1.1~dfsg-1/main/dns_recurring.c
--- 1:13.23.1~dfsg-2/main/dns_recurring.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_recurring.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,155 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief DNS Recurring Query Support
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/astobj2.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/sched.h"
+#include "asterisk/strings.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_recurring.h"
+#include "asterisk/dns_internal.h"
+
+#include <arpa/nameser.h>
+
+/*! \brief Destructor for a DNS query */
+static void dns_query_recurring_destroy(void *data)
+{
+	struct ast_dns_query_recurring *recurring = data;
+
+	ao2_cleanup(recurring->user_data);
+}
+
+static void dns_query_recurring_resolution_callback(const struct ast_dns_query *query);
+
+/*! \brief Scheduled recurring query callback */
+static int dns_query_recurring_scheduled_callback(const void *data)
+{
+	struct ast_dns_query_recurring *recurring = (struct ast_dns_query_recurring *)data;
+
+	ao2_lock(recurring);
+	recurring->timer = -1;
+	if (!recurring->cancelled) {
+		recurring->active = ast_dns_resolve_async(recurring->name, recurring->rr_type, recurring->rr_class, dns_query_recurring_resolution_callback,
+			recurring);
+	}
+	ao2_unlock(recurring);
+
+	ao2_ref(recurring, -1);
+
+	return 0;
+}
+
+/*! \brief Query resolution callback */
+static void dns_query_recurring_resolution_callback(const struct ast_dns_query *query)
+{
+	struct ast_dns_query_recurring *recurring = ast_dns_query_get_data(query);
+	struct ast_dns_query *callback_query;
+
+	/* Create a separate query to invoke the user specific callback on as the
+	 * recurring query user data may get used externally (by the unit test)
+	 * and thus changing it is problematic
+	 */
+	callback_query = dns_query_alloc(query->name, query->rr_type, query->rr_class,
+		recurring->callback, recurring->user_data);
+	if (callback_query) {
+		/* The result is immutable at this point and can be safely provided */
+		callback_query->result = query->result;
+		callback_query->callback(callback_query);
+		callback_query->result = NULL;
+		ao2_ref(callback_query, -1);
+	}
+
+	ao2_lock(recurring);
+	/* So.. if something has not externally cancelled this we can reschedule based on the TTL */
+	if (!recurring->cancelled) {
+		const struct ast_dns_result *result = ast_dns_query_get_result(query);
+		int ttl = MIN(ast_dns_result_get_lowest_ttl(result), INT_MAX / 1000);
+
+		if (ttl) {
+			recurring->timer = ast_sched_add(ast_dns_get_sched(), ttl * 1000, dns_query_recurring_scheduled_callback, ao2_bump(recurring));
+			if (recurring->timer < 0) {
+				/* It is impossible for this to be the last reference as the query has a reference to it */
+				ao2_ref(recurring, -1);
+			}
+		}
+	}
+
+	ao2_replace(recurring->active, NULL);
+	ao2_unlock(recurring);
+}
+
+struct ast_dns_query_recurring *ast_dns_resolve_recurring(const char *name, int rr_type, int rr_class, ast_dns_resolve_callback callback, void *data)
+{
+	struct ast_dns_query_recurring *recurring;
+
+	if (ast_strlen_zero(name) || !callback || !ast_dns_get_sched()) {
+		return NULL;
+	}
+
+	recurring = ao2_alloc(sizeof(*recurring) + strlen(name) + 1, dns_query_recurring_destroy);
+	if (!recurring) {
+		return NULL;
+	}
+
+	recurring->callback = callback;
+	recurring->user_data = ao2_bump(data);
+	recurring->timer = -1;
+	recurring->rr_type = rr_type;
+	recurring->rr_class = rr_class;
+	strcpy(recurring->name, name); /* SAFE */
+
+	recurring->active = ast_dns_resolve_async(name, rr_type, rr_class, dns_query_recurring_resolution_callback, recurring);
+	if (!recurring->active) {
+		ao2_ref(recurring, -1);
+		return NULL;
+	}
+
+	return recurring;
+}
+
+int ast_dns_resolve_recurring_cancel(struct ast_dns_query_recurring *recurring)
+{
+	int res = 0;
+
+	ao2_lock(recurring);
+
+	recurring->cancelled = 1;
+	AST_SCHED_DEL_UNREF(ast_dns_get_sched(), recurring->timer, ao2_ref(recurring, -1));
+
+	if (recurring->active) {
+		res = ast_dns_resolve_cancel(recurring->active);
+		ao2_replace(recurring->active, NULL);
+	}
+
+	ao2_unlock(recurring);
+
+	return res;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dns_srv.c 1:16.1.1~dfsg-1/main/dns_srv.c
--- 1:13.23.1~dfsg-2/main/dns_srv.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_srv.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,216 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief DNS SRV Record Support
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <netinet/in.h>
+#include <arpa/nameser.h>
+#include <resolv.h>
+
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_srv.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/dns_internal.h"
+#include "asterisk/utils.h"
+
+struct ast_dns_record *dns_srv_alloc(struct ast_dns_query *query, const char *data, const size_t size)
+{
+	uint16_t priority;
+	uint16_t weight;
+	uint16_t port;
+	const char *ptr;
+	const char *end_of_record;
+	struct ast_dns_srv_record *srv;
+	int host_size;
+	char host[NI_MAXHOST] = "";
+
+	ptr = dns_find_record(data, size, query->result->answer, query->result->answer_size);
+	ast_assert(ptr != NULL);
+
+	end_of_record = ptr + size;
+
+	/* PRIORITY */
+	ptr += dns_parse_short((unsigned char *) ptr, &priority);
+	if (ptr >= end_of_record) {
+		return NULL;
+	}
+
+	/* WEIGHT */
+	ptr += dns_parse_short((unsigned char *) ptr, &weight);
+	if (ptr >= end_of_record) {
+		return NULL;
+	}
+
+	/* PORT */
+	ptr += dns_parse_short((unsigned char *) ptr, &port);
+	if (ptr >= end_of_record) {
+		return NULL;
+	}
+
+	/*
+	 * The return value from dn_expand represents the size of the replacement
+	 * in the buffer which MAY be compressed.  Since the expanded replacement
+	 * is NULL terminated, you can use strlen() to get the expanded size.
+	 */
+	host_size = dn_expand((unsigned char *)query->result->answer,
+		(unsigned char *) end_of_record, (unsigned char *) ptr, host, sizeof(host) - 1);
+	if (host_size < 0) {
+		ast_log(LOG_ERROR, "Failed to expand domain name: %s\n", strerror(errno));
+		return NULL;
+	}
+
+	if (!strcmp(host, ".")) {
+		return NULL;
+	}
+
+	srv = ast_calloc(1, sizeof(*srv) + size + strlen(host) + 1);
+	if (!srv) {
+		return NULL;
+	}
+
+	srv->priority = priority;
+	srv->weight = weight;
+	srv->port = port;
+
+	srv->host = srv->data + size;
+	strcpy((char *)srv->host, host); /* SAFE */
+	srv->generic.data_ptr = srv->data;
+
+	return (struct ast_dns_record *)srv;
+}
+
+/* This implementation was taken from the existing srv.c which, after reading the RFC, implements it
+ * as it should.
+ */
+void dns_srv_sort(struct ast_dns_result *result)
+{
+	struct ast_dns_record *current;
+	struct dns_records newlist = AST_LIST_HEAD_NOLOCK_INIT_VALUE;
+
+	while (AST_LIST_FIRST(&result->records)) {
+		unsigned short cur_priority = ((struct ast_dns_srv_record *)(AST_LIST_FIRST(&result->records)))->priority;
+		struct dns_records temp_list = AST_LIST_HEAD_NOLOCK_INIT_VALUE;
+
+		/* Find the lowest current priority to work on, but if the priority is already zero there is no lower priority */
+		if (cur_priority) {
+			AST_LIST_TRAVERSE(&result->records, current, list) {
+				if (((struct ast_dns_srv_record *)current)->priority < cur_priority) {
+					cur_priority = ((struct ast_dns_srv_record *)current)->priority;
+				}
+			}
+		}
+
+		/* Find all records which match this priority */
+		AST_LIST_TRAVERSE_SAFE_BEGIN(&result->records, current, list) {
+			if (((struct ast_dns_srv_record *)current)->priority != cur_priority) {
+				continue;
+			}
+
+			AST_LIST_REMOVE_CURRENT(list);
+
+			/* Records with a weight of zero must always be at the head */
+			if (((struct ast_dns_srv_record *)current)->weight == 0) {
+				AST_LIST_INSERT_HEAD(&temp_list, current, list);
+			} else {
+				AST_LIST_INSERT_TAIL(&temp_list, current, list);
+			}
+		}
+		AST_LIST_TRAVERSE_SAFE_END;
+
+		/* Apply weighting - as each record is passed the sum of all previous weights (plus its own) is stored away, and then a random weight
+		 * is calculated. The first record with a weight sum greater than the random weight is put in the new list and the whole thing starts
+		 * once again.
+		 */
+		while (AST_LIST_FIRST(&temp_list)) {
+			unsigned int weight_sum = 0;
+			unsigned int random_weight;
+
+			AST_LIST_TRAVERSE(&temp_list, current, list) {
+				((struct ast_dns_srv_record *)current)->weight_sum = weight_sum += ((struct ast_dns_srv_record *)current)->weight;
+			}
+
+			/* if all the remaining entries have weight == 0,
+			   then just append them to the result list and quit */
+			if (weight_sum == 0) {
+				AST_LIST_APPEND_LIST(&newlist, &temp_list, list);
+				break;
+			}
+
+			random_weight = 1 + (unsigned int) ((float) weight_sum * (ast_random() / ((float) RAND_MAX + 1.0)));
+
+			AST_LIST_TRAVERSE_SAFE_BEGIN(&temp_list, current, list) {
+				if (((struct ast_dns_srv_record *)current)->weight_sum < random_weight) {
+					continue;
+				}
+
+				AST_LIST_MOVE_CURRENT(&newlist, list);
+				break;
+			}
+			AST_LIST_TRAVERSE_SAFE_END;
+		}
+
+	}
+
+	/* now that the new list has been ordered,
+	   put it in place */
+
+	AST_LIST_APPEND_LIST(&result->records, &newlist, list);
+}
+
+const char *ast_dns_srv_get_host(const struct ast_dns_record *record)
+{
+	struct ast_dns_srv_record *srv = (struct ast_dns_srv_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_SRV);
+	return srv->host;
+}
+
+unsigned short ast_dns_srv_get_priority(const struct ast_dns_record *record)
+{
+	struct ast_dns_srv_record *srv = (struct ast_dns_srv_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_SRV);
+	return srv->priority;
+}
+
+unsigned short ast_dns_srv_get_weight(const struct ast_dns_record *record)
+{
+	struct ast_dns_srv_record *srv = (struct ast_dns_srv_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_SRV);
+	return srv->weight;
+}
+
+unsigned short ast_dns_srv_get_port(const struct ast_dns_record *record)
+{
+	struct ast_dns_srv_record *srv = (struct ast_dns_srv_record *) record;
+
+	ast_assert(ast_dns_record_get_rr_type(record) == T_SRV);
+	return srv->port;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dns_system_resolver.c 1:16.1.1~dfsg-1/main/dns_system_resolver.c
--- 1:13.23.1~dfsg-2/main/dns_system_resolver.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_system_resolver.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,265 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Ashley Sanders <asanders@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief The default DNS resolver for Asterisk.
+ *
+ * \arg See also \ref res_resolver_unbound
+ *
+ * \author Ashley Sanders <asanders@digium.com>
+ */
+
+#include "asterisk.h"
+
+#include "asterisk/_private.h"
+#include "asterisk/astobj2.h"
+#include "asterisk/dns.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/taskprocessor.h"
+#include "asterisk/utils.h"
+
+/*! \brief The consideration priority for this resolver implementation. */
+#define DNS_SYSTEM_RESOLVER_PRIORITY INT_MAX
+
+/*! \brief Resolver return code upon success. */
+#define DNS_SYSTEM_RESOLVER_SUCCESS 0
+
+/*! \brief Resolver return code upon failure. */
+#define DNS_SYSTEM_RESOLVER_FAILURE -1
+
+
+static int dns_system_resolver_add_record(void *context, unsigned char *record, int record_len, int ttl);
+static int dns_system_resolver_cancel(struct ast_dns_query *query);
+static void dns_system_resolver_destroy(void);
+static int dns_system_resolver_process_query(void *data);
+static int dns_system_resolver_resolve(struct ast_dns_query *query);
+static int dns_system_resolver_set_response(void *context, unsigned char *dns_response, int dns_response_len, int rcode);
+
+
+/*! \brief The task processor to use for making DNS searches asynchronous. */
+static struct ast_taskprocessor *dns_system_resolver_tp;
+
+/*! \brief The base definition for the dns_system_resolver */
+struct ast_dns_resolver dns_system_resolver_base = {
+	.name = "system",
+	.priority = DNS_SYSTEM_RESOLVER_PRIORITY,
+	.resolve = dns_system_resolver_resolve,
+	.cancel = dns_system_resolver_cancel,
+};
+
+/*!
+ * \brief Callback to handle processing resource records.
+ *
+ * \details Adds an individual resource record discovered with ast_search_dns_ex to the
+ *          ast_dns_query currently being resolved.
+ *
+ * \internal
+ *
+ * \param context     A void pointer to the ast_dns_query being processed.
+ * \param record      An individual resource record discovered during the DNS search.
+ * \param record_len  The length of the resource record.
+ * \param ttl         The resource record's expiration time limit (time to live).
+ *
+ * \retval  0 on success
+ * \retval -1 on failure
+ */
+static int dns_system_resolver_add_record(void *context, unsigned char *record, int record_len, int ttl)
+{
+	struct ast_dns_query *query = context;
+
+	/* Add the record to the query.*/
+	return ast_dns_resolver_add_record(query,
+	                                   ast_dns_query_get_rr_type(query),
+	                                   ast_dns_query_get_rr_class(query),
+	                                   ttl,
+	                                   (const char*) record,
+	                                   record_len);
+}
+
+/*!
+ * \brief Cancels processing resolution for a given query.
+ *
+ * \note The system API calls block so there is no way to cancel them. Therefore, this function always
+ * returns failure when invoked.
+ *
+ * \internal
+ *
+ * \param query  The ast_dns_query to cancel.
+ *
+ * \retval  0 on success
+ * \retval -1 on failure
+ */
+static int dns_system_resolver_cancel(struct ast_dns_query *query)
+{
+	return DNS_SYSTEM_RESOLVER_FAILURE;
+}
+
+/*!
+ * \brief Destructor.
+ *
+ * \internal
+ */
+static void dns_system_resolver_destroy(void)
+{
+	/* Unreference the task processor */
+	dns_system_resolver_tp = ast_taskprocessor_unreference(dns_system_resolver_tp);
+
+	/* Unregister the base resolver */
+	ast_dns_resolver_unregister(&dns_system_resolver_base);
+}
+
+/*!
+ * \brief Callback to handle processing the query from the ast_taskprocessor instance.
+ *
+ * \internal
+ *
+ * \param data  A void pointer to the ast_dns_query being processed.
+ *
+ * \retval -1 on search failure
+ * \retval  0 on no records found
+ * \retval  1 on success
+ */
+static int dns_system_resolver_process_query(void *data)
+{
+	struct ast_dns_query *query = data;
+
+	/* Perform the DNS search */
+	enum ast_dns_search_result res = ast_search_dns_ex(query,
+	                                                   ast_dns_query_get_name(query),
+	                                                   ast_dns_query_get_rr_class(query),
+	                                                   ast_dns_query_get_rr_type(query),
+	                                                   dns_system_resolver_set_response,
+	                                                   dns_system_resolver_add_record);
+
+	/* Handle the possible return values from the DNS search */
+	if (res == AST_DNS_SEARCH_FAILURE) {
+		ast_debug(1, "DNS search failed for query: '%s'\n",
+		        ast_dns_query_get_name(query));
+	} else if (res == AST_DNS_SEARCH_NO_RECORDS) {
+		ast_debug(1, "DNS search failed to yield any results for query: '%s'\n",
+		        ast_dns_query_get_name(query));
+	}
+
+	/* Mark the query as complete */
+	ast_dns_resolver_completed(query);
+
+	/* Reduce the reference count on the query object */
+	ao2_ref(query, -1);
+
+	return res;
+}
+
+/*!
+ * \brief Resolves a DNS query.
+ *
+ * \internal
+ *
+ * \param query  The ast_dns_query to resolve.
+ *
+ * \retval  0 on successful load of query handler to the ast_taskprocessor instance
+ * \retval -1 on failure to load the query handler to the ast_taskprocessor instance
+ */
+static int dns_system_resolver_resolve(struct ast_dns_query *query)
+{
+	/* Add query processing handler to the task processor */
+	int res = ast_taskprocessor_push(dns_system_resolver_tp,
+	                                 dns_system_resolver_process_query,
+	                                 ao2_bump(query));
+
+	/* The query processing handler was not added to the task processor */
+	if (res < 0) {
+		ast_log(LOG_ERROR, "Failed to perform async DNS resolution of '%s'\n",
+		        ast_dns_query_get_name(query));
+		ao2_ref(query, -1);
+	}
+
+	/* Return the result of adding the query processing handler to the task processor */
+	return res;
+}
+
+/*!
+ * \brief Callback to handle initializing the results field.
+ *
+ * \internal
+ *
+ * \param dns_response  The full DNS response.
+ * \param dns_response  The length of the full DNS response.
+ * \param rcode         The DNS response code.
+ *
+ * \retval  0 on success
+ * \retval -1 on failure
+ */
+static int dns_system_resolver_set_response(void *context, unsigned char *dns_response, int dns_response_len, int rcode)
+{
+	struct ast_dns_query *query = context;
+	int res;
+
+	/* Instantiate the query's result field (if necessary). */
+	if (!ast_dns_query_get_result(query)) {
+		res = ast_dns_resolver_set_result(query,
+		                                  0,
+		                                  0,
+		                                  rcode,
+		                                  ast_dns_query_get_name(query),
+		                                  (const char*) dns_response,
+		                                  dns_response_len);
+
+		if (res) {
+			/* There was a problem instantiating the results field. */
+			ast_log(LOG_ERROR, "Could not instantiate the results field for query: '%s'\n",
+			        ast_dns_query_get_name(query));
+		}
+	} else {
+		res = DNS_SYSTEM_RESOLVER_SUCCESS;
+	}
+
+	return res;
+}
+
+/*!
+ * \brief Initializes the resolver.
+ *
+ * \retval  0 on success
+ * \retval -1 on failure
+ */
+int ast_dns_system_resolver_init(void)
+{
+	/* Register the base resolver */
+	int res = ast_dns_resolver_register(&dns_system_resolver_base);
+
+	if (res) {
+		return DNS_SYSTEM_RESOLVER_FAILURE;
+	}
+
+	/* Instantiate the task processor */
+	dns_system_resolver_tp = ast_taskprocessor_get("dns_system_resolver_tp",
+	                                                TPS_REF_DEFAULT);
+
+	/* Return error if the task processor failed to instantiate */
+	if (!dns_system_resolver_tp) {
+		return DNS_SYSTEM_RESOLVER_FAILURE;
+	}
+
+	/* Register the cleanup function */
+	ast_register_cleanup(dns_system_resolver_destroy);
+
+	return DNS_SYSTEM_RESOLVER_SUCCESS;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dns_test.c 1:16.1.1~dfsg-1/main/dns_test.c
--- 1:13.23.1~dfsg-2/main/dns_test.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_test.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,265 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * Includes code and algorithms from the Zapata library.
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_test.h"
+#include "asterisk/utils.h"
+
+#ifdef TEST_FRAMEWORK
+
+const char DNS_HEADER[] = {
+	/* ID  == 0 */
+	0x00, 0x00,
+	/* QR == 1, Opcode == 0, AA == 1, TC == 0, RD == 1 */
+	0x85,
+	/* RA == 1, Z == 0, RCODE == 0 */
+	0x80,
+	/* QDCOUNT == 1 */
+	0x00, 0x01,
+	/* ANCOUNT == 1 */
+	0x00, 0x00,
+	/* NSCOUNT == 0 */
+	0x00, 0x00,
+	/* ARCOUNT == 0 */
+	0x00, 0x00,
+};
+
+/*!
+ * \brief Generate a DNS header and write it to a buffer
+ *
+ * The DNS header is the first part of a DNS request or response. In our
+ * case, the only part of the header that a test can affect is the number
+ * of answers. The rest of the DNS header is based on hard-coded values.
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param num_records The number of DNS records in this DNS response
+ * \param buf The buffer to write the header into
+ * \retval The number of bytes written to the buffer
+ */
+static int generate_dns_header(unsigned short num_records, char *buf)
+{
+	unsigned short net_num_records = htons(num_records);
+
+	memcpy(buf, DNS_HEADER, ARRAY_LEN(DNS_HEADER));
+	/* Overwrite the ANCOUNT with the actual number of answers */
+	memcpy(&buf[6], &net_num_records, sizeof(num_records));
+
+	return ARRAY_LEN(DNS_HEADER);
+}
+
+const char DNS_QUESTION [] = {
+	/* goose */
+	0x05, 0x67, 0x6f, 0x6f, 0x73, 0x65,
+	/* feathers */
+	0x08, 0x66, 0x65, 0x61, 0x74, 0x68, 0x65, 0x72, 0x73,
+	/* end label */
+	0x00,
+	/* NAPTR type */
+	0x00, 0x23,
+	/* IN class */
+	0x00, 0x01,
+};
+
+/*!
+ * \brief Generate a DNS question and write it to a buffer
+ *
+ * The DNS question is the second part of a DNS request or response.
+ * All DNS questions in this file are for the same domain and thus
+ * the DNS question is a hard-coded value.
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param buf The buffer to write the question into
+ * \retval The number of bytes written to the buffer
+ */
+static int generate_dns_question(char *buf)
+{
+	memcpy(buf, DNS_QUESTION, ARRAY_LEN(DNS_QUESTION));
+	return ARRAY_LEN(DNS_QUESTION);
+}
+
+const char NAPTR_ANSWER [] = {
+	/* Domain points to name from question */
+	0xc0, 0x0c,
+	/* NAPTR type */
+	0x00, 0x23,
+	/* IN Class */
+	0x00, 0x01,
+	/* TTL (12345 by default) */
+	0x00, 0x00, 0x30, 0x39,
+};
+
+/*!
+ * \brief Generate a DNS answer and write it to a buffer
+ *
+ * The DNS answer is the third (and in our case final) part of a
+ * DNS response. The DNS answer generated here is only partial.
+ * The record-specific data is generated by a separate function.
+ * DNS answers in our tests may have variable TTLs, but the rest
+ * is hard-coded.
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param buf The buffer to write the answer into
+ * \retval The number of bytes written to the buffer
+ */
+static int generate_dns_answer(int ttl, char *buf)
+{
+	int net_ttl = htonl(ttl);
+
+	memcpy(buf, NAPTR_ANSWER, ARRAY_LEN(NAPTR_ANSWER));
+	/* Overwrite TTL if one is provided */
+	if (ttl) {
+		memcpy(&buf[6], &net_ttl, sizeof(int));
+	}
+
+	return ARRAY_LEN(NAPTR_ANSWER);
+}
+
+/*!
+ * \brief Write a DNS string to a buffer
+ *
+ * This writes the DNS string to the buffer and returns the total
+ * number of bytes written to the buffer.
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param string The string to write
+ * \param buf The buffer to write the string into
+ * \return The number of bytes written to the buffer
+ */
+int ast_dns_test_write_string(const struct ast_dns_test_string *string, char *buf)
+{
+	uint8_t len = string->len;
+	size_t actual_len = strlen(string->val);
+	buf[0] = len;
+	/*
+	 * We use the actual length of the string instead of
+	 * the stated value since sometimes we're going to lie about
+	 * the length of the string
+	 */
+	if (actual_len) {
+		memcpy(&buf[1], string->val, strlen(string->val));
+	}
+
+	return actual_len + 1;
+}
+
+/*!
+ * \brief Write a DNS domain to a buffer
+ *
+ * A DNS domain consists of a series of labels separated
+ * by dots. Each of these labels gets written as a DNS
+ * string. A DNS domain ends with a NULL label, which is
+ * essentially a zero-length DNS string.
+ *
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param string The DNS domain to write
+ * \param buf The buffer to write the domain into
+ * \return The number of bytes written to the buffer
+ */
+int ast_dns_test_write_domain(const char *string, char *buf)
+{
+	char *copy = ast_strdupa(string);
+	char *part;
+	char *ptr = buf;
+	static const struct ast_dns_test_string null_label = {
+		.len = 0,
+		.val = "",
+	};
+
+	while (1) {
+		struct ast_dns_test_string dns_str;
+		part = strsep(&copy, ".");
+		if (ast_strlen_zero(part)) {
+			break;
+		}
+		dns_str.len = strlen(part);
+		dns_str.val = part;
+
+		ptr += ast_dns_test_write_string(&dns_str, ptr);
+	}
+	ptr += ast_dns_test_write_string(&null_label, ptr);
+
+	return ptr - buf;
+}
+
+int ast_dns_test_generate_result(struct ast_dns_query *query, void *records, size_t num_records,
+		size_t record_size, record_fn generate, char *buffer)
+{
+	char *ptr = buffer;
+	char *record_iter;
+
+	ptr += generate_dns_header(num_records, ptr);
+	ptr += generate_dns_question(ptr);
+
+	for (record_iter = records; record_iter < (char *) records + num_records * record_size; record_iter += record_size) {
+		unsigned short rdlength;
+		unsigned short net_rdlength;
+
+		/* XXX Do we even want to override TTL? */
+		ptr += generate_dns_answer(0, ptr);
+		rdlength = generate(record_iter, ptr + 2);
+		net_rdlength = htons(rdlength);
+		memcpy(ptr, &net_rdlength, 2);
+		ptr += 2;
+		ptr += rdlength;
+	}
+
+	return ptr - buffer;
+}
+
+#else /* TEST_FRAMEWORK */
+
+int ast_dns_test_write_string(const struct ast_dns_test_string *string, char *buf)
+{
+	return 0;
+}
+
+int ast_dns_test_write_domain(const char *string, char *buf)
+{
+	return 0;
+}
+
+int ast_dns_test_generate_result(struct ast_dns_query *query, void *records, size_t num_records,
+		size_t record_size, record_fn generate, char *buffer)
+{
+	return 0;
+}
+
+#endif
diff -pruN 1:13.23.1~dfsg-2/main/dns_tlsa.c 1:16.1.1~dfsg-1/main/dns_tlsa.c
--- 1:13.23.1~dfsg-2/main/dns_tlsa.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dns_tlsa.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,53 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief DNS TLSA Record Support
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_tlsa.h"
+
+unsigned int ast_dns_tlsa_get_usage(const struct ast_dns_record *record)
+{
+	return 0;
+}
+
+unsigned int ast_dns_tlsa_get_selector(const struct ast_dns_record *record)
+{
+	return 0;
+}
+
+unsigned int ast_dns_tlsa_get_matching_type(const struct ast_dns_record *record)
+{
+	return 0;
+}
+
+const char *ast_dns_tlsa_get_association_data(const struct ast_dns_record *record)
+{
+	return NULL;
+}
diff -pruN 1:13.23.1~dfsg-2/main/dsp.c 1:16.1.1~dfsg-1/main/dsp.c
--- 1:13.23.1~dfsg-2/main/dsp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/dsp.c	2018-12-26 18:45:13.000000000 +0000
@@ -55,10 +55,9 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 
+#include "asterisk/module.h"
 #include "asterisk/frame.h"
 #include "asterisk/format_cache.h"
 #include "asterisk/channel.h"
@@ -1333,10 +1332,11 @@ int ast_dsp_busydetect(struct ast_dsp *d
 #ifndef BUSYDETECT_TONEONLY
 	if ((hittone >= dsp->busycount - 1) && (hitsilence >= dsp->busycount - 1) &&
 	    (avgtone >= BUSY_MIN && avgtone <= BUSY_MAX) &&
-	    (avgsilence >= BUSY_MIN && avgsilence <= BUSY_MAX)) {
+	    (avgsilence >= BUSY_MIN && avgsilence <= BUSY_MAX))
 #else
-	if ((hittone >= dsp->busycount - 1) && (avgtone >= BUSY_MIN && avgtone <= BUSY_MAX)) {
+	if ((hittone >= dsp->busycount - 1) && (avgtone >= BUSY_MIN && avgtone <= BUSY_MAX))
 #endif
+	{
 #ifdef BUSYDETECT_COMPARE_TONE_AND_SILENCE
 		if (avgtone > avgsilence) {
 			if (avgtone - avgtone*BUSY_PERCENT/100 <= avgsilence) {
@@ -2394,31 +2394,36 @@ AST_TEST_DEFINE(test_dsp_dtmf_detect)
 }
 #endif
 
-#ifdef TEST_FRAMEWORK
-static void test_dsp_shutdown(void)
+static int unload_module(void)
 {
 	AST_TEST_UNREGISTER(test_dsp_fax_detect);
 	AST_TEST_UNREGISTER(test_dsp_dtmf_detect);
+
+	return 0;
 }
-#endif
 
-int ast_dsp_init(void)
+static int load_module(void)
 {
 	if (_dsp_init(0)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
-#ifdef TEST_FRAMEWORK
 	AST_TEST_REGISTER(test_dsp_fax_detect);
 	AST_TEST_REGISTER(test_dsp_dtmf_detect);
 
-	ast_register_cleanup(test_dsp_shutdown);
-#endif
-
-	return 0;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
-int ast_dsp_reload(void)
+static int reload_module(void)
 {
 	return _dsp_init(1);
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "DSP",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/editline/CHANGES 1:16.1.1~dfsg-1/main/editline/CHANGES
--- 1:13.23.1~dfsg-2/main/editline/CHANGES	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/CHANGES	1970-01-01 00:00:00.000000000 +0000
@@ -1,42 +0,0 @@
-2002-02-25 : Christos Zoulas <christos@netbsd.org>
-	* Bring in constification fixes from NetBSD tree.
-	* Use NetBSD's vis, fgetln
-
-2002-02-10 : Jason Evans <jasone@freebsd.org>
-
-	* Makefile.in : Append "" arguments to for loops, to avoid syntax errors
-          with some shells that occur if there are no arguments to the for
-          loops.
-
-2002-02-09 : Jason Evans <jasone@freebsd.org>
-
-	* Install man pages in @prefix@/man/, rather than @prefix@/share/man.
-
-	* Fix the Darwin -install_name S_LDFLAGS argument to default to a prefix
-          of /usr/local if --prefix is not specified.
-
-2002-02-05 : Jason Evans <jasone@freebsd.org>
-
-	* Convert to using an autoconf-generated config.h, rather than passing
-	  -D_HAVE_<foo>=1 definitions on the command line.  Include sys.h in
-	  config.h, and include config.h in .c files rather than sys.h.
-
-	* Mangle function names for implementations in the np directory in order
-          to avoid namespace collisions with other code that may provide copies
-          of the same unimplemented functions.  For example:
-
-		#define	fgetln libedit_fgetln
-
-2002-02-03 : Jason Evans <jasone@freebsd.org>
-
-	* Add autoconf infrastructure, plus a generic Makefile that works with
-          at least BSD make, GNU make and Sun make.
-
-	* Port and/or test on FreeBSD 4.5, FreeBSD-current, NetBSD 1.5 (sparc64
-          and arm32), Apple OS X 10.1.2, Solaris 2.6, and Red Hat Linux 2.6.
-          Add the np directory, which contains implementations of non-portable
-          functions.
-
-	* Add the LIBEDIT_MAJOR and LIBEDIT_MINOR macros to histedit.h, since
-          there is otherwise no straightforward method of programmatically
-          detecting the library version.
diff -pruN 1:13.23.1~dfsg-2/main/editline/chared.c 1:16.1.1~dfsg-1/main/editline/chared.c
--- 1:13.23.1~dfsg-2/main/editline/chared.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/chared.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,695 +0,0 @@
-/*	$NetBSD: chared.c,v 1.15 2002/03/18 16:00:50 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)chared.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: chared.c,v 1.15 2002/03/18 16:00:50 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * chared.c: Character editor utilities
- */
-#include <stdlib.h>
-#include "el.h"
-
-/* value to leave unused in line buffer */
-#define	EL_LEAVE	2
-
-/* cv_undo():
- *	Handle state for the vi undo command
- */
-protected void
-cv_undo(EditLine *el,int action, size_t size, char *ptr)
-{
-	c_undo_t *vu = &el->el_chared.c_undo;
-	vu->action = action;
-	vu->ptr    = ptr;
-	vu->isize  = size;
-	(void) memcpy(vu->buf, vu->ptr, size);
-#ifdef DEBUG_UNDO
-	(void) fprintf(el->el_errfile, "Undo buffer \"%s\" size = +%d -%d\n",
-	       vu->ptr, vu->isize, vu->dsize);
-#endif
-}
-
-
-/* c_insert():
- *	Insert num characters
- */
-protected void
-c_insert(EditLine *el, int num)
-{
-	char *cp;
-
-	if (el->el_line.lastchar + num >= el->el_line.limit)
-		return;			/* can't go past end of buffer */
-
-	if (el->el_line.cursor < el->el_line.lastchar) {
-		/* if I must move chars */
-		for (cp = el->el_line.lastchar; cp >= el->el_line.cursor; cp--)
-			cp[num] = *cp;
-	}
-	el->el_line.lastchar += num;
-}
-
-
-/* c_delafter():
- *	Delete num characters after the cursor
- */
-protected void
-c_delafter(EditLine *el, int num)
-{
-
-	if (el->el_line.cursor + num > el->el_line.lastchar)
-		num = el->el_line.lastchar - el->el_line.cursor;
-
-	if (num > 0) {
-		char *cp;
-
-		if (el->el_map.current != el->el_map.emacs)
-			cv_undo(el, INSERT, (size_t)num, el->el_line.cursor);
-
-		for (cp = el->el_line.cursor; cp <= el->el_line.lastchar; cp++)
-			*cp = cp[num];
-
-		el->el_line.lastchar -= num;
-	}
-}
-
-
-/* c_delbefore():
- *	Delete num characters before the cursor
- */
-protected void
-c_delbefore(EditLine *el, int num)
-{
-
-	if (el->el_line.cursor - num < el->el_line.buffer)
-		num = el->el_line.cursor - el->el_line.buffer;
-
-	if (num > 0) {
-		char *cp;
-
-		if (el->el_map.current != el->el_map.emacs)
-			cv_undo(el, INSERT, (size_t)num,
-			    el->el_line.cursor - num);
-
-		for (cp = el->el_line.cursor - num;
-		    cp <= el->el_line.lastchar;
-		    cp++)
-			*cp = cp[num];
-
-		el->el_line.lastchar -= num;
-	}
-}
-
-
-/* ce__isword():
- *	Return if p is part of a word according to emacs
- */
-protected int
-ce__isword(int p)
-{
-	return (isalpha(p) || isdigit(p) || strchr("*?_-.[]~=", p) != NULL);
-}
-
-
-/* cv__isword():
- *	Return if p is part of a word according to vi
- */
-protected int
-cv__isword(int p)
-{
-	return (!isspace(p));
-}
-
-
-/* c__prev_word():
- *	Find the previous word
- */
-protected char *
-c__prev_word(char *p, char *low, int n, int (*wtest)(int))
-{
-	p--;
-
-	while (n--) {
-		while ((p >= low) && !(*wtest)((unsigned char) *p))
-			p--;
-		while ((p >= low) && (*wtest)((unsigned char) *p))
-			p--;
-	}
-
-	/* cp now points to one character before the word */
-	p++;
-	if (p < low)
-		p = low;
-	/* cp now points where we want it */
-	return (p);
-}
-
-
-/* c__next_word():
- *	Find the next word
- */
-protected char *
-c__next_word(char *p, char *high, int n, int (*wtest)(int))
-{
-	while (n--) {
-		while ((p < high) && !(*wtest)((unsigned char) *p))
-			p++;
-		while ((p < high) && (*wtest)((unsigned char) *p))
-			p++;
-	}
-	if (p > high)
-		p = high;
-	/* p now points where we want it */
-	return (p);
-}
-
-/* cv_next_word():
- *	Find the next word vi style
- */
-protected char *
-cv_next_word(EditLine *el, char *p, char *high, int n, int (*wtest)(int))
-{
-	int test;
-
-	while (n--) {
-		test = (*wtest)((unsigned char) *p);
-		while ((p < high) && (*wtest)((unsigned char) *p) == test)
-			p++;
-		/*
-		 * vi historically deletes with cw only the word preserving the
-		 * trailing whitespace! This is not what 'w' does..
-		 */
-		if (el->el_chared.c_vcmd.action != (DELETE|INSERT))
-			while ((p < high) && isspace((unsigned char) *p))
-				p++;
-	}
-
-	/* p now points where we want it */
-	if (p > high)
-		return (high);
-	else
-		return (p);
-}
-
-
-/* cv_prev_word():
- *	Find the previous word vi style
- */
-protected char *
-cv_prev_word(EditLine *el, char *p, char *low, int n, int (*wtest)(int))
-{
-	int test;
-
-	while (n--) {
-		p--;
-		/*
-		 * vi historically deletes with cb only the word preserving the
-		 * leading whitespace! This is not what 'b' does..
-		 */
-		if (el->el_chared.c_vcmd.action != (DELETE|INSERT))
-			while ((p > low) && isspace((unsigned char) *p))
-				p--;
-		test = (*wtest)((unsigned char) *p);
-		while ((p >= low) && (*wtest)((unsigned char) *p) == test)
-			p--;
-		p++;
-		while (isspace((unsigned char) *p))
-			p++;
-	}
-
-	/* p now points where we want it */
-	if (p < low)
-		return (low);
-	else
-		return (p);
-}
-
-
-#ifdef notdef
-/* c__number():
- *	Ignore character p points to, return number appearing after that.
- * 	A '$' by itself means a big number; "$-" is for negative; '^' means 1.
- * 	Return p pointing to last char used.
- */
-protected char *
-c__number(
-    char *p,	/* character position */
-    int *num,	/* Return value	*/
-    int dval)	/* dval is the number to subtract from like $-3 */
-{
-	int i;
-	int sign = 1;
-
-	if (*++p == '^') {
-		*num = 1;
-		return (p);
-	}
-	if (*p == '$') {
-		if (*++p != '-') {
-			*num = 0x7fffffff;	/* Handle $ */
-			return (--p);
-		}
-		sign = -1;			/* Handle $- */
-		++p;
-	}
-	for (i = 0; isdigit((unsigned char) *p); i = 10 * i + *p++ - '0')
-		continue;
-	*num = (sign < 0 ? dval - i : i);
-	return (--p);
-}
-#endif
-
-/* cv_delfini():
- *	Finish vi delete action
- */
-protected void
-cv_delfini(EditLine *el)
-{
-	int size;
-	int oaction;
-
-	if (el->el_chared.c_vcmd.action & INSERT)
-		el->el_map.current = el->el_map.key;
-
-	oaction = el->el_chared.c_vcmd.action;
-	el->el_chared.c_vcmd.action = NOP;
-
-	if (el->el_chared.c_vcmd.pos == 0)
-		return;
-
-
-	if (el->el_line.cursor > el->el_chared.c_vcmd.pos) {
-		size = (int) (el->el_line.cursor - el->el_chared.c_vcmd.pos);
-		c_delbefore(el, size);
-		el->el_line.cursor = el->el_chared.c_vcmd.pos;
-		re_refresh_cursor(el);
-	} else if (el->el_line.cursor < el->el_chared.c_vcmd.pos) {
-		size = (int)(el->el_chared.c_vcmd.pos - el->el_line.cursor);
-		c_delafter(el, size);
-	} else {
-		size = 1;
-		c_delafter(el, size);
-	}
-	switch (oaction) {
-	case DELETE|INSERT:
-		el->el_chared.c_undo.action = DELETE|INSERT;
-		break;
-	case DELETE:
-		el->el_chared.c_undo.action = INSERT;
-		break;
-	case NOP:
-	case INSERT:
-	default:
-		EL_ABORT((el->el_errfile, "Bad oaction %d\n", oaction));
-		break;
-	}
-
-
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.dsize = size;
-}
-
-
-#ifdef notdef
-/* ce__endword():
- *	Go to the end of this word according to emacs
- */
-protected char *
-ce__endword(char *p, char *high, int n)
-{
-	p++;
-
-	while (n--) {
-		while ((p < high) && isspace((unsigned char) *p))
-			p++;
-		while ((p < high) && !isspace((unsigned char) *p))
-			p++;
-	}
-
-	p--;
-	return (p);
-}
-#endif
-
-
-/* cv__endword():
- *	Go to the end of this word according to vi
- */
-protected char *
-cv__endword(char *p, char *high, int n)
-{
-	p++;
-
-	while (n--) {
-		while ((p < high) && isspace((unsigned char) *p))
-			p++;
-
-		if (isalnum((unsigned char) *p))
-			while ((p < high) && isalnum((unsigned char) *p))
-				p++;
-		else
-			while ((p < high) && !(isspace((unsigned char) *p) ||
-			    isalnum((unsigned char) *p)))
-				p++;
-	}
-	p--;
-	return (p);
-}
-
-/* ch_init():
- *	Initialize the character editor
- */
-protected int
-ch_init(EditLine *el)
-{
-	el->el_line.buffer		= (char *) el_malloc(EL_BUFSIZ);
-	if (el->el_line.buffer == NULL)
-		return (-1);
-
-	(void) memset(el->el_line.buffer, 0, EL_BUFSIZ);
-	el->el_line.cursor		= el->el_line.buffer;
-	el->el_line.lastchar		= el->el_line.buffer;
-	el->el_line.limit		= &el->el_line.buffer[EL_BUFSIZ - 2];
-
-	el->el_chared.c_undo.buf	= (char *) el_malloc(EL_BUFSIZ);
-	if (el->el_chared.c_undo.buf == NULL)
-		return (-1);
-	(void) memset(el->el_chared.c_undo.buf, 0, EL_BUFSIZ);
-	el->el_chared.c_undo.action	= NOP;
-	el->el_chared.c_undo.isize	= 0;
-	el->el_chared.c_undo.dsize	= 0;
-	el->el_chared.c_undo.ptr	= el->el_line.buffer;
-
-	el->el_chared.c_vcmd.action	= NOP;
-	el->el_chared.c_vcmd.pos	= el->el_line.buffer;
-	el->el_chared.c_vcmd.ins	= el->el_line.buffer;
-
-	el->el_chared.c_kill.buf	= (char *) el_malloc(EL_BUFSIZ);
-	if (el->el_chared.c_kill.buf == NULL)
-		return (-1);
-	(void) memset(el->el_chared.c_kill.buf, 0, EL_BUFSIZ);
-	el->el_chared.c_kill.mark	= el->el_line.buffer;
-	el->el_chared.c_kill.last	= el->el_chared.c_kill.buf;
-
-	el->el_map.current		= el->el_map.key;
-
-	el->el_state.inputmode		= MODE_INSERT; /* XXX: save a default */
-	el->el_state.doingarg		= 0;
-	el->el_state.metanext		= 0;
-	el->el_state.argument		= 1;
-	el->el_state.lastcmd		= ED_UNASSIGNED;
-
-	el->el_chared.c_macro.nline	= NULL;
-	el->el_chared.c_macro.level	= -1;
-	el->el_chared.c_macro.macro	= (char **) el_malloc(EL_MAXMACRO *
-	    sizeof(char *));
-	if (el->el_chared.c_macro.macro == NULL)
-		return (-1);
-	return (0);
-}
-
-/* ch_reset():
- *	Reset the character editor
- */
-protected void
-ch_reset(EditLine *el)
-{
-	el->el_line.cursor		= el->el_line.buffer;
-	el->el_line.lastchar		= el->el_line.buffer;
-
-	el->el_chared.c_undo.action	= NOP;
-	el->el_chared.c_undo.isize	= 0;
-	el->el_chared.c_undo.dsize	= 0;
-	el->el_chared.c_undo.ptr	= el->el_line.buffer;
-
-	el->el_chared.c_vcmd.action	= NOP;
-	el->el_chared.c_vcmd.pos	= el->el_line.buffer;
-	el->el_chared.c_vcmd.ins	= el->el_line.buffer;
-
-	el->el_chared.c_kill.mark	= el->el_line.buffer;
-
-	el->el_map.current		= el->el_map.key;
-
-	el->el_state.inputmode		= MODE_INSERT; /* XXX: save a default */
-	el->el_state.doingarg		= 0;
-	el->el_state.metanext		= 0;
-	el->el_state.argument		= 1;
-	el->el_state.lastcmd		= ED_UNASSIGNED;
-
-	el->el_chared.c_macro.level	= -1;
-
-	el->el_history.eventno		= 0;
-}
-
-/* ch_enlargebufs():
- *	Enlarge line buffer to be able to hold twice as much characters.
- *	Returns 1 if successful, 0 if not.
- */
-protected int
-ch_enlargebufs(el, addlen)
-	EditLine *el;
-	size_t addlen;
-{
-	size_t sz, newsz;
-	char *newbuffer, *oldbuf, *oldkbuf;
-
-	sz = el->el_line.limit - el->el_line.buffer + EL_LEAVE;
-	newsz = sz * 2;
-	/*
-	 * If newly required length is longer than current buffer, we need
-	 * to make the buffer big enough to hold both old and new stuff.
-	 */
-	if (addlen > sz) {
-		while(newsz - sz < addlen)
-			newsz *= 2;
-	}
-
-	/*
-	 * Reallocate line buffer.
-	 */
-	newbuffer = el_realloc(el->el_line.buffer, newsz);
-	if (!newbuffer)
-		return 0;
-
-	/* zero the newly added memory, leave old data in */
-	(void) memset(&newbuffer[sz], 0, newsz - sz);
-
-	oldbuf = el->el_line.buffer;
-
-	el->el_line.buffer = newbuffer;
-	el->el_line.cursor = newbuffer + (el->el_line.cursor - oldbuf);
-	el->el_line.lastchar = newbuffer + (el->el_line.lastchar - oldbuf);
-	el->el_line.limit  = &newbuffer[newsz - EL_LEAVE];
-
-	/*
-	 * Reallocate kill buffer.
-	 */
-	newbuffer = el_realloc(el->el_chared.c_kill.buf, newsz);
-	if (!newbuffer)
-		return 0;
-
-	/* zero the newly added memory, leave old data in */
-	(void) memset(&newbuffer[sz], 0, newsz - sz);
-
-	oldkbuf = el->el_chared.c_kill.buf;
-
-	el->el_chared.c_kill.buf = newbuffer;
-	el->el_chared.c_kill.last = newbuffer +
-					(el->el_chared.c_kill.last - oldkbuf);
-	el->el_chared.c_kill.mark = el->el_line.buffer +
-					(el->el_chared.c_kill.mark - oldbuf);
-
-	/*
-	 * Reallocate undo buffer.
-	 */
-	newbuffer = el_realloc(el->el_chared.c_undo.buf, newsz);
-	if (!newbuffer)
-		return 0;
-
-	/* zero the newly added memory, leave old data in */
-	(void) memset(&newbuffer[sz], 0, newsz - sz);
-
-	el->el_chared.c_undo.ptr = el->el_line.buffer +
-				    (el->el_chared.c_undo.ptr - oldbuf);
-	el->el_chared.c_undo.buf = newbuffer;
-
-	if (!hist_enlargebuf(el, sz, newsz))
-		return 0;
-
-	return 1;
-}
-
-/* ch_end():
- *	Free the data structures used by the editor
- */
-protected void
-ch_end(EditLine *el)
-{
-	el_free((ptr_t) el->el_line.buffer);
-	el->el_line.buffer = NULL;
-	el->el_line.limit = NULL;
-	el_free((ptr_t) el->el_chared.c_undo.buf);
-	el->el_chared.c_undo.buf = NULL;
-	el_free((ptr_t) el->el_chared.c_kill.buf);
-	el->el_chared.c_kill.buf = NULL;
-	el_free((ptr_t) el->el_chared.c_macro.macro);
-	el->el_chared.c_macro.macro = NULL;
-	ch_reset(el);
-}
-
-
-/* el_insertstr():
- *	Insert string at cursorI
- */
-public int
-el_insertstr(EditLine *el, const char *s)
-{
-	size_t len;
-
-	if ((len = strlen(s)) == 0)
-		return (-1);
-	if (el->el_line.lastchar + len >= el->el_line.limit) {
-		if (!ch_enlargebufs(el, len))
-			return (-1);
-	}
-
-	c_insert(el, (int)len);
-	while (*s)
-		*el->el_line.cursor++ = *s++;
-	return (0);
-}
-
-
-/* el_deletestr():
- *	Delete num characters before the cursor
- */
-public void
-el_deletestr(EditLine *el, int n)
-{
-	if (n <= 0)
-		return;
-
-	if (el->el_line.cursor < &el->el_line.buffer[n])
-		return;
-
-	c_delbefore(el, n);		/* delete before dot */
-	el->el_line.cursor -= n;
-	if (el->el_line.cursor < el->el_line.buffer)
-		el->el_line.cursor = el->el_line.buffer;
-}
-
-/* c_gets():
- *	Get a string
- */
-protected int
-c_gets(EditLine *el, char *buf)
-{
-	char ch;
-	int len = 0;
-
-	for (ch = 0; ch == 0;) {
-		if (el_getc(el, &ch) != 1)
-			return (ed_end_of_file(el, 0));
-		switch (ch) {
-		case 0010:	/* Delete and backspace */
-		case 0177:
-			if (len > 1) {
-				*el->el_line.cursor-- = '\0';
-				el->el_line.lastchar = el->el_line.cursor;
-				buf[len--] = '\0';
-			} else {
-				el->el_line.buffer[0] = '\0';
-				el->el_line.lastchar = el->el_line.buffer;
-				el->el_line.cursor = el->el_line.buffer;
-				return (CC_REFRESH);
-			}
-			re_refresh(el);
-			ch = 0;
-			break;
-
-		case 0033:	/* ESC */
-		case '\r':	/* Newline */
-		case '\n':
-			break;
-
-		default:
-			if (len >= EL_BUFSIZ)
-				term_beep(el);
-			else {
-				buf[len++] = ch;
-				*el->el_line.cursor++ = ch;
-				el->el_line.lastchar = el->el_line.cursor;
-			}
-			re_refresh(el);
-			ch = 0;
-			break;
-		}
-	}
-	buf[len] = ch;
-	return (len);
-}
-
-
-/* c_hpos():
- *	Return the current horizontal position of the cursor
- */
-protected int
-c_hpos(EditLine *el)
-{
-	char *ptr;
-
-	/*
-	 * Find how many characters till the beginning of this line.
-	 */
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (0);
-	else {
-		for (ptr = el->el_line.cursor - 1;
-		     ptr >= el->el_line.buffer && *ptr != '\n';
-		     ptr--)
-			continue;
-		return (el->el_line.cursor - ptr - 1);
-	}
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/chared.h 1:16.1.1~dfsg-1/main/editline/chared.h
--- 1:13.23.1~dfsg-2/main/editline/chared.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/chared.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,159 +0,0 @@
-/*	$NetBSD: chared.h,v 1.8 2002/03/18 16:00:51 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)chared.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.chared.h: Character editor interface
- */
-#ifndef _h_el_chared
-#define	_h_el_chared
-
-#include <ctype.h>
-#include <string.h>
-
-#include "histedit.h"
-
-#define	EL_MAXMACRO	10
-
-/*
- * This is a issue of basic "vi" look-and-feel. Defining VI_MOVE works
- * like real vi: i.e. the transition from command<->insert modes moves
- * the cursor.
- *
- * On the other hand we really don't want to move the cursor, because
- * all the editing commands don't include the character under the cursor.
- * Probably the best fix is to make all the editing commands aware of
- * this fact.
- */
-#define	VI_MOVE
-
-
-typedef struct c_macro_t {
-	int	  level;
-	char	**macro;
-	char	 *nline;
-} c_macro_t;
-
-/*
- * Undo information for both vi and emacs
- */
-typedef struct c_undo_t {
-	int	 action;
-	size_t	 isize;
-	size_t	 dsize;
-	char	*ptr;
-	char	*buf;
-} c_undo_t;
-
-/*
- * Current action information for vi
- */
-typedef struct c_vcmd_t {
-	int	 action;
-	char	*pos;
-	char	*ins;
-} c_vcmd_t;
-
-/*
- * Kill buffer for emacs
- */
-typedef struct c_kill_t {
-	char	*buf;
-	char	*last;
-	char	*mark;
-} c_kill_t;
-
-/*
- * Note that we use both data structures because the user can bind
- * commands from both editors!
- */
-typedef struct el_chared_t {
-	c_undo_t	c_undo;
-	c_kill_t	c_kill;
-	c_vcmd_t	c_vcmd;
-	c_macro_t	c_macro;
-} el_chared_t;
-
-
-#define	STReof		"^D\b\b"
-#define	STRQQ		"\"\""
-
-#define	isglob(a)	(strchr("*[]?", (a)) != NULL)
-#define	isword(a)	(isprint(a))
-
-#define	NOP		0x00
-#define	DELETE		0x01
-#define	INSERT		0x02
-#define	CHANGE		0x04
-
-#define	CHAR_FWD	0
-#define	CHAR_BACK	1
-
-#define	MODE_INSERT	0
-#define	MODE_REPLACE	1
-#define	MODE_REPLACE_1	2
-
-#include "common.h"
-#include "vi.h"
-#include "emacs.h"
-#include "search.h"
-#include "fcns.h"
-
-
-protected int	 cv__isword(int);
-protected void	 cv_delfini(EditLine *);
-protected char	*cv__endword(char *, char *, int);
-protected int	 ce__isword(int);
-protected void	 cv_undo(EditLine *, int, size_t, char *);
-protected char	*cv_next_word(EditLine*, char *, char *, int, int (*)(int));
-protected char	*cv_prev_word(EditLine*, char *, char *, int, int (*)(int));
-protected char	*c__next_word(char *, char *, int, int (*)(int));
-protected char	*c__prev_word(char *, char *, int, int (*)(int));
-protected void	 c_insert(EditLine *, int);
-protected void	 c_delbefore(EditLine *, int);
-protected void	 c_delafter(EditLine *, int);
-protected int	 c_gets(EditLine *, char *);
-protected int	 c_hpos(EditLine *);
-
-protected int	 ch_init(EditLine *);
-protected void	 ch_reset(EditLine *);
-protected int	 ch_enlargebufs(EditLine *, size_t);
-protected void	 ch_end(EditLine *);
-
-#endif /* _h_el_chared */
diff -pruN 1:13.23.1~dfsg-2/main/editline/common.c 1:16.1.1~dfsg-1/main/editline/common.c
--- 1:13.23.1~dfsg-2/main/editline/common.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/common.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,951 +0,0 @@
-/*	$NetBSD: common.c,v 1.11 2002/03/18 16:00:51 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)common.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: common.c,v 1.11 2002/03/18 16:00:51 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * common.c: Common Editor functions
- */
-#include "el.h"
-
-/* ed_end_of_file():
- *	Indicate end of file
- *	[^D]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_end_of_file(EditLine *el, int c)
-{
-
-	re_goto_bottom(el);
-	*el->el_line.lastchar = '\0';
-	return (CC_EOF);
-}
-
-
-/* ed_insert():
- *	Add character to the line
- *	Insert a character [bound to all insert keys]
- */
-protected el_action_t
-ed_insert(EditLine *el, int c)
-{
-	int i;
-
-	if (c == '\0')
-		return (CC_ERROR);
-
-	if (el->el_line.lastchar + el->el_state.argument >=
-	    el->el_line.limit) {
-		/* end of buffer space, try to allocate more */
-		if (!ch_enlargebufs(el, (size_t) el->el_state.argument))
-			return CC_ERROR;	/* error allocating more */
-	}
-
-	if (el->el_state.argument == 1) {
-		if (el->el_state.inputmode != MODE_INSERT) {
-			el->el_chared.c_undo.buf[el->el_chared.c_undo.isize++] =
-			    *el->el_line.cursor;
-			el->el_chared.c_undo.buf[el->el_chared.c_undo.isize] =
-			    '\0';
-			c_delafter(el, 1);
-		}
-		c_insert(el, 1);
-
-		*el->el_line.cursor++ = c;
-		el->el_state.doingarg = 0;	/* just in case */
-		re_fastaddc(el);		/* fast refresh for one char. */
-	} else {
-		if (el->el_state.inputmode != MODE_INSERT) {
-			for (i = 0; i < el->el_state.argument; i++)
-				el->el_chared.c_undo.buf[el->el_chared.c_undo.isize++] =
-				    el->el_line.cursor[i];
-
-			el->el_chared.c_undo.buf[el->el_chared.c_undo.isize] =
-			    '\0';
-			c_delafter(el, el->el_state.argument);
-		}
-		c_insert(el, el->el_state.argument);
-
-		while (el->el_state.argument--)
-			*el->el_line.cursor++ = c;
-		re_refresh(el);
-	}
-
-	if (el->el_state.inputmode == MODE_REPLACE_1)
-		(void) vi_command_mode(el, 0);
-
-	return (CC_NORM);
-}
-
-
-/* ed_delete_prev_word():
- *	Delete from beginning of current word to cursor
- *	[M-^?] [^W]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_delete_prev_word(EditLine *el, int c)
-{
-	char *cp, *p, *kp;
-
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (CC_ERROR);
-
-	cp = c__prev_word(el->el_line.cursor, el->el_line.buffer,
-	    el->el_state.argument, ce__isword);
-
-	for (p = cp, kp = el->el_chared.c_kill.buf; p < el->el_line.cursor; p++)
-		*kp++ = *p;
-	el->el_chared.c_kill.last = kp;
-
-	c_delbefore(el, el->el_line.cursor - cp);	/* delete before dot */
-	el->el_line.cursor = cp;
-	if (el->el_line.cursor < el->el_line.buffer)
-		el->el_line.cursor = el->el_line.buffer; /* bounds check */
-	return (CC_REFRESH);
-}
-
-
-/* ed_delete_next_char():
- *	Delete character under cursor
- *	[^D] [x]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_delete_next_char(EditLine *el, int c)
-{
-#ifdef notdef			/* XXX */
-#define	EL	el->el_line
-	(void) fprintf(el->el_errlfile,
-	    "\nD(b: %x(%s)  c: %x(%s) last: %x(%s) limit: %x(%s)\n",
-	    EL.buffer, EL.buffer, EL.cursor, EL.cursor, EL.lastchar,
-	    EL.lastchar, EL.limit, EL.limit);
-#endif
-	if (el->el_line.cursor == el->el_line.lastchar) {
-			/* if I'm at the end */
-		if (el->el_map.type == MAP_VI) {
-			if (el->el_line.cursor == el->el_line.buffer) {
-				/* if I'm also at the beginning */
-#ifdef KSHVI
-				return (CC_ERROR);
-#else
-				term_overwrite(el, STReof, 4);
-					/* then do a EOF */
-				term__flush();
-				return (CC_EOF);
-#endif
-			} else {
-#ifdef KSHVI
-				el->el_line.cursor--;
-#else
-				return (CC_ERROR);
-#endif
-			}
-		} else {
-			if (el->el_line.cursor != el->el_line.buffer)
-				el->el_line.cursor--;
-			else
-				return (CC_ERROR);
-		}
-	}
-	c_delafter(el, el->el_state.argument);	/* delete after dot */
-	if (el->el_line.cursor >= el->el_line.lastchar &&
-	    el->el_line.cursor > el->el_line.buffer)
-			/* bounds check */
-		el->el_line.cursor = el->el_line.lastchar - 1;
-	return (CC_REFRESH);
-}
-
-
-/* ed_kill_line():
- *	Cut to the end of line
- *	[^K] [^K]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_kill_line(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	cp = el->el_line.cursor;
-	kp = el->el_chared.c_kill.buf;
-	while (cp < el->el_line.lastchar)
-		*kp++ = *cp++;	/* copy it */
-	el->el_chared.c_kill.last = kp;
-			/* zap! -- delete to end */
-	el->el_line.lastchar = el->el_line.cursor;
-	return (CC_REFRESH);
-}
-
-
-/* ed_move_to_end():
- *	Move cursor to the end of line
- *	[^E] [^E]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_move_to_end(EditLine *el, int c)
-{
-
-	el->el_line.cursor = el->el_line.lastchar;
-	if (el->el_map.type == MAP_VI) {
-#ifdef VI_MOVE
-		el->el_line.cursor--;
-#endif
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	}
-	return (CC_CURSOR);
-}
-
-
-/* ed_move_to_beg():
- *	Move cursor to the beginning of line
- *	[^A] [^A]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_move_to_beg(EditLine *el, int c)
-{
-
-	el->el_line.cursor = el->el_line.buffer;
-
-	if (el->el_map.type == MAP_VI) {
-			/* We want FIRST non space character */
-		while (isspace((unsigned char) *el->el_line.cursor))
-			el->el_line.cursor++;
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	}
-	return (CC_CURSOR);
-}
-
-
-/* ed_transpose_chars():
- *	Exchange the character to the left of the cursor with the one under it
- *	[^T] [^T]
- */
-protected el_action_t
-ed_transpose_chars(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor < el->el_line.lastchar) {
-		if (el->el_line.lastchar <= &el->el_line.buffer[1])
-			return (CC_ERROR);
-		else
-			el->el_line.cursor++;
-	}
-	if (el->el_line.cursor > &el->el_line.buffer[1]) {
-		/* must have at least two chars entered */
-		c = el->el_line.cursor[-2];
-		el->el_line.cursor[-2] = el->el_line.cursor[-1];
-		el->el_line.cursor[-1] = c;
-		return (CC_REFRESH);
-	} else
-		return (CC_ERROR);
-}
-
-
-/* ed_next_char():
- *	Move to the right one character
- *	[^F] [^F]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_next_char(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor >= el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor += el->el_state.argument;
-	if (el->el_line.cursor > el->el_line.lastchar)
-		el->el_line.cursor = el->el_line.lastchar;
-
-	if (el->el_map.type == MAP_VI)
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	return (CC_CURSOR);
-}
-
-
-/* ed_prev_word():
- *	Move to the beginning of the current word
- *	[M-b] [b]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_prev_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (CC_ERROR);
-
-	el->el_line.cursor = c__prev_word(el->el_line.cursor,
-	    el->el_line.buffer,
-	    el->el_state.argument,
-	    ce__isword);
-
-	if (el->el_map.type == MAP_VI)
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	return (CC_CURSOR);
-}
-
-
-/* ed_prev_char():
- *	Move to the left one character
- *	[^B] [^B]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_prev_char(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor > el->el_line.buffer) {
-		el->el_line.cursor -= el->el_state.argument;
-		if (el->el_line.cursor < el->el_line.buffer)
-			el->el_line.cursor = el->el_line.buffer;
-
-		if (el->el_map.type == MAP_VI)
-			if (el->el_chared.c_vcmd.action & DELETE) {
-				cv_delfini(el);
-				return (CC_REFRESH);
-			}
-		return (CC_CURSOR);
-	} else
-		return (CC_ERROR);
-}
-
-
-/* ed_quoted_insert():
- *	Add the next character typed verbatim
- *	[^V] [^V]
- */
-protected el_action_t
-ed_quoted_insert(EditLine *el, int c)
-{
-	int num;
-	char tc;
-
-	tty_quotemode(el);
-	num = el_getc(el, &tc);
-	c = (unsigned char) tc;
-	tty_noquotemode(el);
-	if (num == 1)
-		return (ed_insert(el, c));
-	else
-		return (ed_end_of_file(el, 0));
-}
-
-
-/* ed_digit():
- *	Adds to argument or enters a digit
- */
-protected el_action_t
-ed_digit(EditLine *el, int c)
-{
-
-	if (!isdigit(c))
-		return (CC_ERROR);
-
-	if (el->el_state.doingarg) {
-			/* if doing an arg, add this in... */
-		if (el->el_state.lastcmd == EM_UNIVERSAL_ARGUMENT)
-			el->el_state.argument = c - '0';
-		else {
-			if (el->el_state.argument > 1000000)
-				return (CC_ERROR);
-			el->el_state.argument =
-			    (el->el_state.argument * 10) + (c - '0');
-		}
-		return (CC_ARGHACK);
-	} else {
-		if (el->el_line.lastchar + 1 >= el->el_line.limit) {
-			if (!ch_enlargebufs(el, 1))
-				return (CC_ERROR);
-		}
-
-		if (el->el_state.inputmode != MODE_INSERT) {
-			el->el_chared.c_undo.buf[el->el_chared.c_undo.isize++] =
-			    *el->el_line.cursor;
-			el->el_chared.c_undo.buf[el->el_chared.c_undo.isize] =
-			    '\0';
-			c_delafter(el, 1);
-		}
-		c_insert(el, 1);
-		*el->el_line.cursor++ = c;
-		el->el_state.doingarg = 0;
-		re_fastaddc(el);
-	}
-	return (CC_NORM);
-}
-
-
-/* ed_argument_digit():
- *	Digit that starts argument
- *	For ESC-n
- */
-protected el_action_t
-ed_argument_digit(EditLine *el, int c)
-{
-
-	if (!isdigit(c))
-		return (CC_ERROR);
-
-	if (el->el_state.doingarg) {
-		if (el->el_state.argument > 1000000)
-			return (CC_ERROR);
-		el->el_state.argument = (el->el_state.argument * 10) +
-		    (c - '0');
-	} else {		/* else starting an argument */
-		el->el_state.argument = c - '0';
-		el->el_state.doingarg = 1;
-	}
-	return (CC_ARGHACK);
-}
-
-
-/* ed_unassigned():
- *	Indicates unbound character
- *	Bound to keys that are not assigned
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_unassigned(EditLine *el, int c)
-{
-
-	term_beep(el);
-	term__flush();
-	return (CC_NORM);
-}
-
-
-/**
- ** TTY key handling.
- **/
-
-/* ed_tty_sigint():
- *	Tty interrupt character
- *	[^C]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_sigint(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_dsusp():
- *	Tty delayed suspend character
- *	[^Y]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_dsusp(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_flush_output():
- *	Tty flush output characters
- *	[^O]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_flush_output(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_sigquit():
- *	Tty quit character
- *	[^\]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_sigquit(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_sigtstp():
- *	Tty suspend character
- *	[^Z]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_sigtstp(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_stop_output():
- *	Tty disallow output characters
- *	[^S]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_stop_output(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_tty_start_output():
- *	Tty allow output characters
- *	[^Q]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_tty_start_output(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_newline():
- *	Execute command
- *	[^J]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_newline(EditLine *el, int c)
-{
-
-	re_goto_bottom(el);
-	*el->el_line.lastchar++ = '\n';
-	*el->el_line.lastchar = '\0';
-	if (el->el_map.type == MAP_VI)
-		el->el_chared.c_vcmd.ins = el->el_line.buffer;
-	return (CC_NEWLINE);
-}
-
-
-/* ed_delete_prev_char():
- *	Delete the character to the left of the cursor
- *	[^?]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_delete_prev_char(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor <= el->el_line.buffer)
-		return (CC_ERROR);
-
-	c_delbefore(el, el->el_state.argument);
-	el->el_line.cursor -= el->el_state.argument;
-	if (el->el_line.cursor < el->el_line.buffer)
-		el->el_line.cursor = el->el_line.buffer;
-	return (CC_REFRESH);
-}
-
-
-/* ed_clear_screen():
- *	Clear screen leaving current line at the top
- *	[^L]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_clear_screen(EditLine *el, int c)
-{
-
-	term_clear_screen(el);	/* clear the whole real screen */
-	re_clear_display(el);	/* reset everything */
-	return (CC_REFRESH);
-}
-
-
-/* ed_redisplay():
- *	Redisplay everything
- *	^R
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_redisplay(EditLine *el, int c)
-{
-
-	return (CC_REDISPLAY);
-}
-
-
-/* ed_start_over():
- *	Erase current line and start from scratch
- *	[^G]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_start_over(EditLine *el, int c)
-{
-
-	ch_reset(el);
-	return (CC_REFRESH);
-}
-
-
-/* ed_sequence_lead_in():
- *	First character in a bound sequence
- *	Placeholder for external keys
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_sequence_lead_in(EditLine *el, int c)
-{
-
-	return (CC_NORM);
-}
-
-
-/* ed_prev_history():
- *	Move to the previous history line
- *	[^P] [k]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_prev_history(EditLine *el, int c)
-{
-	char beep = 0;
-
-	el->el_chared.c_undo.action = NOP;
-	*el->el_line.lastchar = '\0';		/* just in case */
-
-	if (el->el_history.eventno == 0) {	/* save the current buffer
-						 * away */
-		(void) strncpy(el->el_history.buf, el->el_line.buffer,
-		    EL_BUFSIZ - 1);
-		el->el_history.last = el->el_history.buf +
-		    (el->el_line.lastchar - el->el_line.buffer);
-	}
-	el->el_history.eventno += el->el_state.argument;
-
-	if (hist_get(el) == CC_ERROR) {
-		beep = 1;
-		/* el->el_history.eventno was fixed by first call */
-		(void) hist_get(el);
-	}
-	re_refresh(el);
-	if (beep)
-		return (CC_ERROR);
-	else
-		return (CC_NORM);	/* was CC_UP_HIST */
-}
-
-
-/* ed_next_history():
- *	Move to the next history line
- *	[^N] [j]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_next_history(EditLine *el, int c)
-{
-
-	el->el_chared.c_undo.action = NOP;
-	*el->el_line.lastchar = '\0';	/* just in case */
-
-	el->el_history.eventno -= el->el_state.argument;
-
-	if (el->el_history.eventno < 0) {
-		el->el_history.eventno = 0;
-		return (CC_ERROR);/* make it beep */
-	}
-	return (hist_get(el));
-}
-
-
-/* ed_search_prev_history():
- *	Search previous in history for a line matching the current
- *	next search history [M-P] [K]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_search_prev_history(EditLine *el, int c)
-{
-	const char *hp;
-	int h;
-	bool_t found = 0;
-
-	el->el_chared.c_vcmd.action = NOP;
-	el->el_chared.c_undo.action = NOP;
-	*el->el_line.lastchar = '\0';	/* just in case */
-	if (el->el_history.eventno < 0) {
-#ifdef DEBUG_EDIT
-		(void) fprintf(el->el_errfile,
-		    "e_prev_search_hist(): eventno < 0;\n");
-#endif
-		el->el_history.eventno = 0;
-		return (CC_ERROR);
-	}
-	if (el->el_history.eventno == 0) {
-		(void) strncpy(el->el_history.buf, el->el_line.buffer,
-		    EL_BUFSIZ - 1);
-		el->el_history.last = el->el_history.buf +
-		    (el->el_line.lastchar - el->el_line.buffer);
-	}
-	if (el->el_history.ref == NULL)
-		return (CC_ERROR);
-
-	hp = HIST_FIRST(el);
-	if (hp == NULL)
-		return (CC_ERROR);
-
-	c_setpat(el);		/* Set search pattern !! */
-
-	for (h = 1; h <= el->el_history.eventno; h++)
-		hp = HIST_NEXT(el);
-
-	while (hp != NULL) {
-#ifdef SDEBUG
-		(void) fprintf(el->el_errfile, "Comparing with \"%s\"\n", hp);
-#endif
-		if ((strncmp(hp, el->el_line.buffer, (size_t)
-			    (el->el_line.lastchar - el->el_line.buffer)) ||
-			hp[el->el_line.lastchar - el->el_line.buffer]) &&
-		    c_hmatch(el, hp)) {
-			found++;
-			break;
-		}
-		h++;
-		hp = HIST_NEXT(el);
-	}
-
-	if (!found) {
-#ifdef SDEBUG
-		(void) fprintf(el->el_errfile, "not found\n");
-#endif
-		return (CC_ERROR);
-	}
-	el->el_history.eventno = h;
-
-	return (hist_get(el));
-}
-
-
-/* ed_search_next_history():
- *	Search next in history for a line matching the current
- *	[M-N] [J]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_search_next_history(EditLine *el, int c)
-{
-	const char *hp;
-	int h;
-	bool_t found = 0;
-
-	el->el_chared.c_vcmd.action = NOP;
-	el->el_chared.c_undo.action = NOP;
-	*el->el_line.lastchar = '\0';	/* just in case */
-
-	if (el->el_history.eventno == 0)
-		return (CC_ERROR);
-
-	if (el->el_history.ref == NULL)
-		return (CC_ERROR);
-
-	hp = HIST_FIRST(el);
-	if (hp == NULL)
-		return (CC_ERROR);
-
-	c_setpat(el);		/* Set search pattern !! */
-
-	for (h = 1; h < el->el_history.eventno && hp; h++) {
-#ifdef SDEBUG
-		(void) fprintf(el->el_errfile, "Comparing with \"%s\"\n", hp);
-#endif
-		if ((strncmp(hp, el->el_line.buffer, (size_t)
-			    (el->el_line.lastchar - el->el_line.buffer)) ||
-			hp[el->el_line.lastchar - el->el_line.buffer]) &&
-		    c_hmatch(el, hp))
-			found = h;
-		hp = HIST_NEXT(el);
-	}
-
-	if (!found) {		/* is it the current history number? */
-		if (!c_hmatch(el, el->el_history.buf)) {
-#ifdef SDEBUG
-			(void) fprintf(el->el_errfile, "not found\n");
-#endif
-			return (CC_ERROR);
-		}
-	}
-	el->el_history.eventno = found;
-
-	return (hist_get(el));
-}
-
-
-/* ed_prev_line():
- *	Move up one line
- *	Could be [k] [^p]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_prev_line(EditLine *el, int c)
-{
-	char *ptr;
-	int nchars = c_hpos(el);
-
-	/*
-         * Move to the line requested
-         */
-	if (*(ptr = el->el_line.cursor) == '\n')
-		ptr--;
-
-	for (; ptr >= el->el_line.buffer; ptr--)
-		if (*ptr == '\n' && --el->el_state.argument <= 0)
-			break;
-
-	if (el->el_state.argument > 0)
-		return (CC_ERROR);
-
-	/*
-         * Move to the beginning of the line
-         */
-	for (ptr--; ptr >= el->el_line.buffer && *ptr != '\n'; ptr--)
-		continue;
-
-	/*
-         * Move to the character requested
-         */
-	for (ptr++;
-	    nchars-- > 0 && ptr < el->el_line.lastchar && *ptr != '\n';
-	    ptr++)
-		continue;
-
-	el->el_line.cursor = ptr;
-	return (CC_CURSOR);
-}
-
-
-/* ed_next_line():
- *	Move down one line
- *	Could be [j] [^n]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_next_line(EditLine *el, int c)
-{
-	char *ptr;
-	int nchars = c_hpos(el);
-
-	/*
-         * Move to the line requested
-         */
-	for (ptr = el->el_line.cursor; ptr < el->el_line.lastchar; ptr++)
-		if (*ptr == '\n' && --el->el_state.argument <= 0)
-			break;
-
-	if (el->el_state.argument > 0)
-		return (CC_ERROR);
-
-	/*
-         * Move to the character requested
-         */
-	for (ptr++;
-	    nchars-- > 0 && ptr < el->el_line.lastchar && *ptr != '\n';
-	    ptr++)
-		continue;
-
-	el->el_line.cursor = ptr;
-	return (CC_CURSOR);
-}
-
-
-/* ed_command():
- *	Editline extended command
- *	[M-X] [:]
- */
-protected el_action_t
-/*ARGSUSED*/
-ed_command(EditLine *el, int c)
-{
-	char tmpbuf[EL_BUFSIZ];
-	int tmplen;
-
-	el->el_line.buffer[0] = '\0';
-	el->el_line.lastchar = el->el_line.buffer;
-	el->el_line.cursor = el->el_line.buffer;
-
-	c_insert(el, 3);	/* prompt + ": " */
-	*el->el_line.cursor++ = '\n';
-	*el->el_line.cursor++ = ':';
-	*el->el_line.cursor++ = ' ';
-	re_refresh(el);
-
-	tmplen = c_gets(el, tmpbuf);
-	tmpbuf[tmplen] = '\0';
-
-	el->el_line.buffer[0] = '\0';
-	el->el_line.lastchar = el->el_line.buffer;
-	el->el_line.cursor = el->el_line.buffer;
-
-	if (parse_line(el, tmpbuf) == -1)
-		return (CC_ERROR);
-	else
-		return (CC_REFRESH);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/config.guess 1:16.1.1~dfsg-1/main/editline/config.guess
--- 1:13.23.1~dfsg-2/main/editline/config.guess	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/config.guess	1970-01-01 00:00:00.000000000 +0000
@@ -1,1449 +0,0 @@
-#! /bin/sh
-# Attempt to guess a canonical system name.
-#   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
-#   2000, 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
-
-timestamp='2004-06-24'
-
-# This file is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-#
-# As a special exception to the GNU General Public License, if you
-# distribute this file as part of a program that contains a
-# configuration script generated by Autoconf, you may include it under
-# the same distribution terms that you use for the rest of that program.
-
-# Originally written by Per Bothner <per@bothner.com>.
-# Please send patches to <config-patches@gnu.org>.  Submit a context
-# diff and a properly formatted ChangeLog entry.
-#
-# This script attempts to guess a canonical system name similar to
-# config.sub.  If it succeeds, it prints the system name on stdout, and
-# exits with 0.  Otherwise, it exits with 1.
-#
-# The plan is that this can be called by configure scripts if you
-# don't specify an explicit build system type.
-
-me=`echo "$0" | sed -e 's,.*/,,'`
-
-usage="\
-Usage: $0 [OPTION]
-
-Output the configuration name of the system \`$me' is run on.
-
-Operation modes:
-  -h, --help         print this help, then exit
-  -t, --time-stamp   print date of last modification, then exit
-  -v, --version      print version number, then exit
-
-Report bugs and patches to <config-patches@gnu.org>."
-
-version="\
-GNU config.guess ($timestamp)
-
-Originally written by Per Bothner.
-Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
-Free Software Foundation, Inc.
-
-This is free software; see the source for copying conditions.  There is NO
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
-
-help="
-Try \`$me --help' for more information."
-
-# Parse command line
-while test $# -gt 0 ; do
-  case $1 in
-    --time-stamp | --time* | -t )
-       echo "$timestamp" ; exit 0 ;;
-    --version | -v )
-       echo "$version" ; exit 0 ;;
-    --help | --h* | -h )
-       echo "$usage"; exit 0 ;;
-    -- )     # Stop option processing
-       shift; break ;;
-    - )	# Use stdin as input.
-       break ;;
-    -* )
-       echo "$me: invalid option $1$help" >&2
-       exit 1 ;;
-    * )
-       break ;;
-  esac
-done
-
-if test $# != 0; then
-  echo "$me: too many arguments$help" >&2
-  exit 1
-fi
-
-trap 'exit 1' 1 2 15
-
-# CC_FOR_BUILD -- compiler used by this script. Note that the use of a
-# compiler to aid in system detection is discouraged as it requires
-# temporary files to be created and, as you can see below, it is a
-# headache to deal with in a portable fashion.
-
-# Historically, `CC_FOR_BUILD' used to be named `HOST_CC'. We still
-# use `HOST_CC' if defined, but it is deprecated.
-
-# Portable tmp directory creation inspired by the Autoconf team.
-
-set_cc_for_build='
-trap "exitcode=\$?; (rm -f \$tmpfiles 2>/dev/null; rmdir \$tmp 2>/dev/null) && exit \$exitcode" 0 ;
-trap "rm -f \$tmpfiles 2>/dev/null; rmdir \$tmp 2>/dev/null; exit 1" 1 2 13 15 ;
-: ${TMPDIR=/tmp} ;
- { tmp=`(umask 077 && mktemp -d -q "$TMPDIR/cgXXXXXX") 2>/dev/null` && test -n "$tmp" && test -d "$tmp" ; } ||
- { test -n "$RANDOM" && tmp=$TMPDIR/cg$$-$RANDOM && (umask 077 && mkdir $tmp) ; } ||
- { tmp=$TMPDIR/cg-$$ && (umask 077 && mkdir $tmp) && echo "Warning: creating insecure temp directory" >&2 ; } ||
- { echo "$me: cannot create a temporary directory in $TMPDIR" >&2 ; exit 1 ; } ;
-dummy=$tmp/dummy ;
-tmpfiles="$dummy.c $dummy.o $dummy.rel $dummy" ;
-case $CC_FOR_BUILD,$HOST_CC,$CC in
- ,,)    echo "int x;" > $dummy.c ;
-	for c in cc gcc c89 c99 ; do
-	  if ($c -c -o $dummy.o $dummy.c) >/dev/null 2>&1 ; then
-	     CC_FOR_BUILD="$c"; break ;
-	  fi ;
-	done ;
-	if test x"$CC_FOR_BUILD" = x ; then
-	  CC_FOR_BUILD=no_compiler_found ;
-	fi
-	;;
- ,,*)   CC_FOR_BUILD=$CC ;;
- ,*,*)  CC_FOR_BUILD=$HOST_CC ;;
-esac ;'
-
-# This is needed to find uname on a Pyramid OSx when run in the BSD universe.
-# (ghazi@noc.rutgers.edu 1994-08-24)
-if (test -f /.attbin/uname) >/dev/null 2>&1 ; then
-	PATH=$PATH:/.attbin ; export PATH
-fi
-
-UNAME_MACHINE=`(uname -m) 2>/dev/null` || UNAME_MACHINE=unknown
-UNAME_RELEASE=`(uname -r) 2>/dev/null` || UNAME_RELEASE=unknown
-UNAME_SYSTEM=`(uname -s) 2>/dev/null`  || UNAME_SYSTEM=unknown
-UNAME_VERSION=`(uname -v) 2>/dev/null` || UNAME_VERSION=unknown
-
-# Note: order is significant - the case branches are not exclusive.
-
-case "${UNAME_MACHINE}:${UNAME_SYSTEM}:${UNAME_RELEASE}:${UNAME_VERSION}" in
-    *:NetBSD:*:*)
-	# NetBSD (nbsd) targets should (where applicable) match one or
-	# more of the tupples: *-*-netbsdelf*, *-*-netbsdaout*,
-	# *-*-netbsdecoff* and *-*-netbsd*.  For targets that recently
-	# switched to ELF, *-*-netbsd* would select the old
-	# object file format.  This provides both forward
-	# compatibility and a consistent mechanism for selecting the
-	# object file format.
-	#
-	# Note: NetBSD doesn't particularly care about the vendor
-	# portion of the name.  We always set it to "unknown".
-	sysctl="sysctl -n hw.machine_arch"
-	UNAME_MACHINE_ARCH=`(/sbin/$sysctl 2>/dev/null || \
-	    /usr/sbin/$sysctl 2>/dev/null || echo unknown)`
-	case "${UNAME_MACHINE_ARCH}" in
-	    armeb) machine=armeb-unknown ;;
-	    arm*) machine=arm-unknown ;;
-	    sh3el) machine=shl-unknown ;;
-	    sh3eb) machine=sh-unknown ;;
-	    *) machine=${UNAME_MACHINE_ARCH}-unknown ;;
-	esac
-	# The Operating System including object format, if it has switched
-	# to ELF recently, or will in the future.
-	case "${UNAME_MACHINE_ARCH}" in
-	    arm*|i386|m68k|ns32k|sh3*|sparc|vax)
-		eval $set_cc_for_build
-		if echo __ELF__ | $CC_FOR_BUILD -E - 2>/dev/null \
-			| grep __ELF__ >/dev/null
-		then
-		    # Once all utilities can be ECOFF (netbsdecoff) or a.out (netbsdaout).
-		    # Return netbsd for either.  FIX?
-		    os=netbsd
-		else
-		    os=netbsdelf
-		fi
-		;;
-	    *)
-	        os=netbsd
-		;;
-	esac
-	# The OS release
-	# Debian GNU/NetBSD machines have a different userland, and
-	# thus, need a distinct triplet. However, they do not need
-	# kernel version information, so it can be replaced with a
-	# suitable tag, in the style of linux-gnu.
-	case "${UNAME_VERSION}" in
-	    Debian*)
-		release='-gnu'
-		;;
-	    *)
-		release=`echo ${UNAME_RELEASE}|sed -e 's/[-_].*/\./'`
-		;;
-	esac
-	# Since CPU_TYPE-MANUFACTURER-KERNEL-OPERATING_SYSTEM:
-	# contains redundant information, the shorter form:
-	# CPU_TYPE-MANUFACTURER-OPERATING_SYSTEM is used.
-	echo "${machine}-${os}${release}"
-	exit 0 ;;
-    amd64:OpenBSD:*:*)
-	echo x86_64-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    amiga:OpenBSD:*:*)
-	echo m68k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    arc:OpenBSD:*:*)
-	echo mipsel-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    cats:OpenBSD:*:*)
-	echo arm-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    hp300:OpenBSD:*:*)
-	echo m68k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    luna88k:OpenBSD:*:*)
-    	echo m88k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    mac68k:OpenBSD:*:*)
-	echo m68k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    macppc:OpenBSD:*:*)
-	echo powerpc-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    mvme68k:OpenBSD:*:*)
-	echo m68k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    mvme88k:OpenBSD:*:*)
-	echo m88k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    mvmeppc:OpenBSD:*:*)
-	echo powerpc-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    pmax:OpenBSD:*:*)
-	echo mipsel-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    sgi:OpenBSD:*:*)
-	echo mipseb-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    sun3:OpenBSD:*:*)
-	echo m68k-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    wgrisc:OpenBSD:*:*)
-	echo mipsel-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    *:OpenBSD:*:*)
-	echo ${UNAME_MACHINE}-unknown-openbsd${UNAME_RELEASE}
-	exit 0 ;;
-    *:ekkoBSD:*:*)
-	echo ${UNAME_MACHINE}-unknown-ekkobsd${UNAME_RELEASE}
-	exit 0 ;;
-    macppc:MirBSD:*:*)
-	echo powerppc-unknown-mirbsd${UNAME_RELEASE}
-	exit 0 ;;
-    *:MirBSD:*:*)
-	echo ${UNAME_MACHINE}-unknown-mirbsd${UNAME_RELEASE}
-	exit 0 ;;
-    alpha:OSF1:*:*)
-	case $UNAME_RELEASE in
-	*4.0)
-		UNAME_RELEASE=`/usr/sbin/sizer -v | awk '{print $3}'`
-		;;
-	*5.*)
-	        UNAME_RELEASE=`/usr/sbin/sizer -v | awk '{print $4}'`
-		;;
-	esac
-	# According to Compaq, /usr/sbin/psrinfo has been available on
-	# OSF/1 and Tru64 systems produced since 1995.  I hope that
-	# covers most systems running today.  This code pipes the CPU
-	# types through head -n 1, so we only detect the type of CPU 0.
-	ALPHA_CPU_TYPE=`/usr/sbin/psrinfo -v | sed -n -e 's/^  The alpha \(.*\) processor.*$/\1/p' | head -n 1`
-	case "$ALPHA_CPU_TYPE" in
-	    "EV4 (21064)")
-		UNAME_MACHINE="alpha" ;;
-	    "EV4.5 (21064)")
-		UNAME_MACHINE="alpha" ;;
-	    "LCA4 (21066/21068)")
-		UNAME_MACHINE="alpha" ;;
-	    "EV5 (21164)")
-		UNAME_MACHINE="alphaev5" ;;
-	    "EV5.6 (21164A)")
-		UNAME_MACHINE="alphaev56" ;;
-	    "EV5.6 (21164PC)")
-		UNAME_MACHINE="alphapca56" ;;
-	    "EV5.7 (21164PC)")
-		UNAME_MACHINE="alphapca57" ;;
-	    "EV6 (21264)")
-		UNAME_MACHINE="alphaev6" ;;
-	    "EV6.7 (21264A)")
-		UNAME_MACHINE="alphaev67" ;;
-	    "EV6.8CB (21264C)")
-		UNAME_MACHINE="alphaev68" ;;
-	    "EV6.8AL (21264B)")
-		UNAME_MACHINE="alphaev68" ;;
-	    "EV6.8CX (21264D)")
-		UNAME_MACHINE="alphaev68" ;;
-	    "EV6.9A (21264/EV69A)")
-		UNAME_MACHINE="alphaev69" ;;
-	    "EV7 (21364)")
-		UNAME_MACHINE="alphaev7" ;;
-	    "EV7.9 (21364A)")
-		UNAME_MACHINE="alphaev79" ;;
-	esac
-	# A Pn.n version is a patched version.
-	# A Vn.n version is a released version.
-	# A Tn.n version is a released field test version.
-	# A Xn.n version is an unreleased experimental baselevel.
-	# 1.2 uses "1.2" for uname -r.
-	echo ${UNAME_MACHINE}-dec-osf`echo ${UNAME_RELEASE} | sed -e 's/^[PVTX]//' | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
-	exit 0 ;;
-    Alpha\ *:Windows_NT*:*)
-	# How do we know it's Interix rather than the generic POSIX subsystem?
-	# Should we change UNAME_MACHINE based on the output of uname instead
-	# of the specific Alpha model?
-	echo alpha-pc-interix
-	exit 0 ;;
-    21064:Windows_NT:50:3)
-	echo alpha-dec-winnt3.5
-	exit 0 ;;
-    Amiga*:UNIX_System_V:4.0:*)
-	echo m68k-unknown-sysv4
-	exit 0;;
-    *:[Aa]miga[Oo][Ss]:*:*)
-	echo ${UNAME_MACHINE}-unknown-amigaos
-	exit 0 ;;
-    *:[Mm]orph[Oo][Ss]:*:*)
-	echo ${UNAME_MACHINE}-unknown-morphos
-	exit 0 ;;
-    *:OS/390:*:*)
-	echo i370-ibm-openedition
-	exit 0 ;;
-    *:OS400:*:*)
-        echo powerpc-ibm-os400
-	exit 0 ;;
-    arm:RISC*:1.[012]*:*|arm:riscix:1.[012]*:*)
-	echo arm-acorn-riscix${UNAME_RELEASE}
-	exit 0;;
-    SR2?01:HI-UX/MPP:*:* | SR8000:HI-UX/MPP:*:*)
-	echo hppa1.1-hitachi-hiuxmpp
-	exit 0;;
-    Pyramid*:OSx*:*:* | MIS*:OSx*:*:* | MIS*:SMP_DC-OSx*:*:*)
-	# akee@wpdis03.wpafb.af.mil (Earle F. Ake) contributed MIS and NILE.
-	if test "`(/bin/universe) 2>/dev/null`" = att ; then
-		echo pyramid-pyramid-sysv3
-	else
-		echo pyramid-pyramid-bsd
-	fi
-	exit 0 ;;
-    NILE*:*:*:dcosx)
-	echo pyramid-pyramid-svr4
-	exit 0 ;;
-    DRS?6000:unix:4.0:6*)
-	echo sparc-icl-nx6
-	exit 0 ;;
-    DRS?6000:UNIX_SV:4.2*:7*)
-	case `/usr/bin/uname -p` in
-	    sparc) echo sparc-icl-nx7 && exit 0 ;;
-	esac ;;
-    sun4H:SunOS:5.*:*)
-	echo sparc-hal-solaris2`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
-	exit 0 ;;
-    sun4*:SunOS:5.*:* | tadpole*:SunOS:5.*:*)
-	echo sparc-sun-solaris2`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
-	exit 0 ;;
-    i86pc:SunOS:5.*:*)
-	echo i386-pc-solaris2`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
-	exit 0 ;;
-    sun4*:SunOS:6*:*)
-	# According to config.sub, this is the proper way to canonicalize
-	# SunOS6.  Hard to guess exactly what SunOS6 will be like, but
-	# it's likely to be more like Solaris than SunOS4.
-	echo sparc-sun-solaris3`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
-	exit 0 ;;
-    sun4*:SunOS:*:*)
-	case "`/usr/bin/arch -k`" in
-	    Series*|S4*)
-		UNAME_RELEASE=`uname -v`
-		;;
-	esac
-	# Japanese Language versions have a version number like `4.1.3-JL'.
-	echo sparc-sun-sunos`echo ${UNAME_RELEASE}|sed -e 's/-/_/'`
-	exit 0 ;;
-    sun3*:SunOS:*:*)
-	echo m68k-sun-sunos${UNAME_RELEASE}
-	exit 0 ;;
-    sun*:*:4.2BSD:*)
-	UNAME_RELEASE=`(sed 1q /etc/motd | awk '{print substr($5,1,3)}') 2>/dev/null`
-	test "x${UNAME_RELEASE}" = "x" && UNAME_RELEASE=3
-	case "`/bin/arch`" in
-	    sun3)
-		echo m68k-sun-sunos${UNAME_RELEASE}
-		;;
-	    sun4)
-		echo sparc-sun-sunos${UNAME_RELEASE}
-		;;
-	esac
-	exit 0 ;;
-    aushp:SunOS:*:*)
-	echo sparc-auspex-sunos${UNAME_RELEASE}
-	exit 0 ;;
-    # The situation for MiNT is a little confusing.  The machine name
-    # can be virtually everything (everything which is not
-    # "atarist" or "atariste" at least should have a processor
-    # > m68000).  The system name ranges from "MiNT" over "FreeMiNT"
-    # to the lowercase version "mint" (or "freemint").  Finally
-    # the system name "TOS" denotes a system which is actually not
-    # MiNT.  But MiNT is downward compatible to TOS, so this should
-    # be no problem.
-    atarist[e]:*MiNT:*:* | atarist[e]:*mint:*:* | atarist[e]:*TOS:*:*)
-        echo m68k-atari-mint${UNAME_RELEASE}
-	exit 0 ;;
-    atari*:*MiNT:*:* | atari*:*mint:*:* | atarist[e]:*TOS:*:*)
-	echo m68k-atari-mint${UNAME_RELEASE}
-        exit 0 ;;
-    *falcon*:*MiNT:*:* | *falcon*:*mint:*:* | *falcon*:*TOS:*:*)
-        echo m68k-atari-mint${UNAME_RELEASE}
-	exit 0 ;;
-    milan*:*MiNT:*:* | milan*:*mint:*:* | *milan*:*TOS:*:*)
-        echo m68k-milan-mint${UNAME_RELEASE}
-        exit 0 ;;
-    hades*:*MiNT:*:* | hades*:*mint:*:* | *hades*:*TOS:*:*)
-        echo m68k-hades-mint${UNAME_RELEASE}
-        exit 0 ;;
-    *:*MiNT:*:* | *:*mint:*:* | *:*TOS:*:*)
-        echo m68k-unknown-mint${UNAME_RELEASE}
-        exit 0 ;;
-    m68k:machten:*:*)
-	echo m68k-apple-machten${UNAME_RELEASE}
-	exit 0 ;;
-    powerpc:machten:*:*)
-	echo powerpc-apple-machten${UNAME_RELEASE}
-	exit 0 ;;
-    RISC*:Mach:*:*)
-	echo mips-dec-mach_bsd4.3
-	exit 0 ;;
-    RISC*:ULTRIX:*:*)
-	echo mips-dec-ultrix${UNAME_RELEASE}
-	exit 0 ;;
-    VAX*:ULTRIX*:*:*)
-	echo vax-dec-ultrix${UNAME_RELEASE}
-	exit 0 ;;
-    2020:CLIX:*:* | 2430:CLIX:*:*)
-	echo clipper-intergraph-clix${UNAME_RELEASE}
-	exit 0 ;;
-    mips:*:*:UMIPS | mips:*:*:RISCos)
-	eval $set_cc_for_build
-	sed 's/^	//' << EOF >$dummy.c
-#ifdef __cplusplus
-#include <stdio.h>  /* for printf() prototype */
-	int main (int argc, char *argv[]) {
-#else
-	int main (argc, argv) int argc; char *argv[]; {
-#endif
-	#if defined (host_mips) && defined (MIPSEB)
-	#if defined (SYSTYPE_SYSV)
-	  printf ("mips-mips-riscos%ssysv\n", argv[1]); exit (0);
-	#endif
-	#if defined (SYSTYPE_SVR4)
-	  printf ("mips-mips-riscos%ssvr4\n", argv[1]); exit (0);
-	#endif
-	#if defined (SYSTYPE_BSD43) || defined(SYSTYPE_BSD)
-	  printf ("mips-mips-riscos%sbsd\n", argv[1]); exit (0);
-	#endif
-	#endif
-	  exit (-1);
-	}
-EOF
-	$CC_FOR_BUILD -o $dummy $dummy.c \
-	  && $dummy `echo "${UNAME_RELEASE}" | sed -n 's/\([0-9]*\).*/\1/p'` \
-	  && exit 0
-	echo mips-mips-riscos${UNAME_RELEASE}
-	exit 0 ;;
-    Motorola:PowerMAX_OS:*:*)
-	echo powerpc-motorola-powermax
-	exit 0 ;;
-    Motorola:*:4.3:PL8-*)
-	echo powerpc-harris-powermax
-	exit 0 ;;
-    Night_Hawk:*:*:PowerMAX_OS | Synergy:PowerMAX_OS:*:*)
-	echo powerpc-harris-powermax
-	exit 0 ;;
-    Night_Hawk:Power_UNIX:*:*)
-	echo powerpc-harris-powerunix
-	exit 0 ;;
-    m88k:CX/UX:7*:*)
-	echo m88k-harris-cxux7
-	exit 0 ;;
-    m88k:*:4*:R4*)
-	echo m88k-motorola-sysv4
-	exit 0 ;;
-    m88k:*:3*:R3*)
-	echo m88k-motorola-sysv3
-	exit 0 ;;
-    AViiON:dgux:*:*)
-        # DG/UX returns AViiON for all architectures
-        UNAME_PROCESSOR=`/usr/bin/uname -p`
-	if [ $UNAME_PROCESSOR = mc88100 ] || [ $UNAME_PROCESSOR = mc88110 ]
-	then
-	    if [ ${TARGET_BINARY_INTERFACE}x = m88kdguxelfx ] || \
-	       [ ${TARGET_BINARY_INTERFACE}x = x ]
-	    then
-		echo m88k-dg-dgux${UNAME_RELEASE}
-	    else
-		echo m88k-dg-dguxbcs${UNAME_RELEASE}
-	    fi
-	else
-	    echo i586-dg-dgux${UNAME_RELEASE}
-	fi
- 	exit 0 ;;
-    M88*:DolphinOS:*:*)	# DolphinOS (SVR3)
-	echo m88k-dolphin-sysv3
-	exit 0 ;;
-    M88*:*:R3*:*)
-	# Delta 88k system running SVR3
-	echo m88k-motorola-sysv3
-	exit 0 ;;
-    XD88*:*:*:*) # Tektronix XD88 system running UTekV (SVR3)
-	echo m88k-tektronix-sysv3
-	exit 0 ;;
-    Tek43[0-9][0-9]:UTek:*:*) # Tektronix 4300 system running UTek (BSD)
-	echo m68k-tektronix-bsd
-	exit 0 ;;
-    *:IRIX*:*:*)
-	echo mips-sgi-irix`echo ${UNAME_RELEASE}|sed -e 's/-/_/g'`
-	exit 0 ;;
-    ????????:AIX?:[12].1:2)   # AIX 2.2.1 or AIX 2.1.1 is RT/PC AIX.
-	echo romp-ibm-aix      # uname -m gives an 8 hex-code CPU id
-	exit 0 ;;              # Note that: echo "'`uname -s`'" gives 'AIX '
-    i*86:AIX:*:*)
-	echo i386-ibm-aix
-	exit 0 ;;
-    ia64:AIX:*:*)
-	if [ -x /usr/bin/oslevel ] ; then
-		IBM_REV=`/usr/bin/oslevel`
-	else
-		IBM_REV=${UNAME_VERSION}.${UNAME_RELEASE}
-	fi
-	echo ${UNAME_MACHINE}-ibm-aix${IBM_REV}
-	exit 0 ;;
-    *:AIX:2:3)
-	if grep bos325 /usr/include/stdio.h >/dev/null 2>&1; then
-		eval $set_cc_for_build
-		sed 's/^		//' << EOF >$dummy.c
-		#include <sys/systemcfg.h>
-
-		main()
-			{
-			if (!__power_pc())
-				exit(1);
-			puts("powerpc-ibm-aix3.2.5");
-			exit(0);
-			}
-EOF
-		$CC_FOR_BUILD -o $dummy $dummy.c && $dummy && exit 0
-		echo rs6000-ibm-aix3.2.5
-	elif grep bos324 /usr/include/stdio.h >/dev/null 2>&1; then
-		echo rs6000-ibm-aix3.2.4
-	else
-		echo rs6000-ibm-aix3.2
-	fi
-	exit 0 ;;
-    *:AIX:*:[45])
-	IBM_CPU_ID=`/usr/sbin/lsdev -C -c processor -S available | sed 1q | awk '{ print $1 }'`
-	if /usr/sbin/lsattr -El ${IBM_CPU_ID} | grep ' POWER' >/dev/null 2>&1; then
-		IBM_ARCH=rs6000
-	else
-		IBM_ARCH=powerpc
-	fi
-	if [ -x /usr/bin/oslevel ] ; then
-		IBM_REV=`/usr/bin/oslevel`
-	else
-		IBM_REV=${UNAME_VERSION}.${UNAME_RELEASE}
-	fi
-	echo ${IBM_ARCH}-ibm-aix${IBM_REV}
-	exit 0 ;;
-    *:AIX:*:*)
-	echo rs6000-ibm-aix
-	exit 0 ;;
-    ibmrt:4.4BSD:*|romp-ibm:BSD:*)
-	echo romp-ibm-bsd4.4
-	exit 0 ;;
-    ibmrt:*BSD:*|romp-ibm:BSD:*)            # covers RT/PC BSD and
-	echo romp-ibm-bsd${UNAME_RELEASE}   # 4.3 with uname added to
-	exit 0 ;;                           # report: romp-ibm BSD 4.3
-    *:BOSX:*:*)
-	echo rs6000-bull-bosx
-	exit 0 ;;
-    DPX/2?00:B.O.S.:*:*)
-	echo m68k-bull-sysv3
-	exit 0 ;;
-    9000/[34]??:4.3bsd:1.*:*)
-	echo m68k-hp-bsd
-	exit 0 ;;
-    hp300:4.4BSD:*:* | 9000/[34]??:4.3bsd:2.*:*)
-	echo m68k-hp-bsd4.4
-	exit 0 ;;
-    9000/[34678]??:HP-UX:*:*)
-	HPUX_REV=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*.[0B]*//'`
-	case "${UNAME_MACHINE}" in
-	    9000/31? )            HP_ARCH=m68000 ;;
-	    9000/[34]?? )         HP_ARCH=m68k ;;
-	    9000/[678][0-9][0-9])
-		if [ -x /usr/bin/getconf ]; then
-		    sc_cpu_version=`/usr/bin/getconf SC_CPU_VERSION 2>/dev/null`
-                    sc_kernel_bits=`/usr/bin/getconf SC_KERNEL_BITS 2>/dev/null`
-                    case "${sc_cpu_version}" in
-                      523) HP_ARCH="hppa1.0" ;; # CPU_PA_RISC1_0
-                      528) HP_ARCH="hppa1.1" ;; # CPU_PA_RISC1_1
-                      532)                      # CPU_PA_RISC2_0
-                        case "${sc_kernel_bits}" in
-                          32) HP_ARCH="hppa2.0n" ;;
-                          64) HP_ARCH="hppa2.0w" ;;
-			  '') HP_ARCH="hppa2.0" ;;   # HP-UX 10.20
-                        esac ;;
-                    esac
-		fi
-		if [ "${HP_ARCH}" = "" ]; then
-		    eval $set_cc_for_build
-		    sed 's/^              //' << EOF >$dummy.c
-
-              #define _HPUX_SOURCE
-              #include <stdlib.h>
-              #include <unistd.h>
-
-              int main ()
-              {
-              #if defined(_SC_KERNEL_BITS)
-                  long bits = sysconf(_SC_KERNEL_BITS);
-              #endif
-                  long cpu  = sysconf (_SC_CPU_VERSION);
-
-                  switch (cpu)
-              	{
-              	case CPU_PA_RISC1_0: puts ("hppa1.0"); break;
-              	case CPU_PA_RISC1_1: puts ("hppa1.1"); break;
-              	case CPU_PA_RISC2_0:
-              #if defined(_SC_KERNEL_BITS)
-              	    switch (bits)
-              		{
-              		case 64: puts ("hppa2.0w"); break;
-              		case 32: puts ("hppa2.0n"); break;
-              		default: puts ("hppa2.0"); break;
-              		} break;
-              #else  /* !defined(_SC_KERNEL_BITS) */
-              	    puts ("hppa2.0"); break;
-              #endif
-              	default: puts ("hppa1.0"); break;
-              	}
-                  exit (0);
-              }
-EOF
-		    (CCOPTS= $CC_FOR_BUILD -o $dummy $dummy.c 2>/dev/null) && HP_ARCH=`$dummy`
-		    test -z "$HP_ARCH" && HP_ARCH=hppa
-		fi ;;
-	esac
-	if [ ${HP_ARCH} = "hppa2.0w" ]
-	then
-	    # avoid double evaluation of $set_cc_for_build
-	    test -n "$CC_FOR_BUILD" || eval $set_cc_for_build
-	    if echo __LP64__ | (CCOPTS= $CC_FOR_BUILD -E -) | grep __LP64__ >/dev/null
-	    then
-		HP_ARCH="hppa2.0w"
-	    else
-		HP_ARCH="hppa64"
-	    fi
-	fi
-	echo ${HP_ARCH}-hp-hpux${HPUX_REV}
-	exit 0 ;;
-    ia64:HP-UX:*:*)
-	HPUX_REV=`echo ${UNAME_RELEASE}|sed -e 's/[^.]*.[0B]*//'`
-	echo ia64-hp-hpux${HPUX_REV}
-	exit 0 ;;
-    3050*:HI-UX:*:*)
-	eval $set_cc_for_build
-	sed 's/^	//' << EOF >$dummy.c
-	#include <unistd.h>
-	int
-	main ()
-	{
-	  long cpu = sysconf (_SC_CPU_VERSION);
-	  /* The order matters, because CPU_IS_HP_MC68K erroneously returns
-	     true for CPU_PA_RISC1_0.  CPU_IS_PA_RISC returns correct
-	     results, however.  */
-	  if (CPU_IS_PA_RISC (cpu))
-	    {
-	      switch (cpu)
-		{
-		  case CPU_PA_RISC1_0: puts ("hppa1.0-hitachi-hiuxwe2"); break;
-		  case CPU_PA_RISC1_1: puts ("hppa1.1-hitachi-hiuxwe2"); break;
-		  case CPU_PA_RISC2_0: puts ("hppa2.0-hitachi-hiuxwe2"); break;
-		  default: puts ("hppa-hitachi-hiuxwe2"); break;
-		}
-	    }
-	  else if (CPU_IS_HP_MC68K (cpu))
-	    puts ("m68k-hitachi-hiuxwe2");
-	  else puts ("unknown-hitachi-hiuxwe2");
-	  exit (0);
-	}
-EOF
-	$CC_FOR_BUILD -o $dummy $dummy.c && $dummy && exit 0
-	echo unknown-hitachi-hiuxwe2
-	exit 0 ;;
-    9000/7??:4.3bsd:*:* | 9000/8?[79]:4.3bsd:*:* )
-	echo hppa1.1-hp-bsd
-	exit 0 ;;
-    9000/8??:4.3bsd:*:*)
-	echo hppa1.0-hp-bsd
-	exit 0 ;;
-    *9??*:MPE/iX:*:* | *3000*:MPE/iX:*:*)
-	echo hppa1.0-hp-mpeix
-	exit 0 ;;
-    hp7??:OSF1:*:* | hp8?[79]:OSF1:*:* )
-	echo hppa1.1-hp-osf
-	exit 0 ;;
-    hp8??:OSF1:*:*)
-	echo hppa1.0-hp-osf
-	exit 0 ;;
-    i*86:OSF1:*:*)
-	if [ -x /usr/sbin/sysversion ] ; then
-	    echo ${UNAME_MACHINE}-unknown-osf1mk
-	else
-	    echo ${UNAME_MACHINE}-unknown-osf1
-	fi
-	exit 0 ;;
-    parisc*:Lites*:*:*)
-	echo hppa1.1-hp-lites
-	exit 0 ;;
-    C1*:ConvexOS:*:* | convex:ConvexOS:C1*:*)
-	echo c1-convex-bsd
-        exit 0 ;;
-    C2*:ConvexOS:*:* | convex:ConvexOS:C2*:*)
-	if getsysinfo -f scalar_acc
-	then echo c32-convex-bsd
-	else echo c2-convex-bsd
-	fi
-        exit 0 ;;
-    C34*:ConvexOS:*:* | convex:ConvexOS:C34*:*)
-	echo c34-convex-bsd
-        exit 0 ;;
-    C38*:ConvexOS:*:* | convex:ConvexOS:C38*:*)
-	echo c38-convex-bsd
-        exit 0 ;;
-    C4*:ConvexOS:*:* | convex:ConvexOS:C4*:*)
-	echo c4-convex-bsd
-        exit 0 ;;
-    CRAY*Y-MP:*:*:*)
-	echo ymp-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    CRAY*[A-Z]90:*:*:*)
-	echo ${UNAME_MACHINE}-cray-unicos${UNAME_RELEASE} \
-	| sed -e 's/CRAY.*\([A-Z]90\)/\1/' \
-	      -e y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/ \
-	      -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    CRAY*TS:*:*:*)
-	echo t90-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    CRAY*T3E:*:*:*)
-	echo alphaev5-cray-unicosmk${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    CRAY*SV1:*:*:*)
-	echo sv1-cray-unicos${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    *:UNICOS/mp:*:*)
-	echo craynv-cray-unicosmp${UNAME_RELEASE} | sed -e 's/\.[^.]*$/.X/'
-	exit 0 ;;
-    F30[01]:UNIX_System_V:*:* | F700:UNIX_System_V:*:*)
-	FUJITSU_PROC=`uname -m | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz'`
-        FUJITSU_SYS=`uname -p | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz' | sed -e 's/\///'`
-        FUJITSU_REL=`echo ${UNAME_RELEASE} | sed -e 's/ /_/'`
-        echo "${FUJITSU_PROC}-fujitsu-${FUJITSU_SYS}${FUJITSU_REL}"
-        exit 0 ;;
-    5000:UNIX_System_V:4.*:*)
-        FUJITSU_SYS=`uname -p | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz' | sed -e 's/\///'`
-        FUJITSU_REL=`echo ${UNAME_RELEASE} | tr 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz' | sed -e 's/ /_/'`
-        echo "sparc-fujitsu-${FUJITSU_SYS}${FUJITSU_REL}"
-	exit 0 ;;
-    i*86:BSD/386:*:* | i*86:BSD/OS:*:* | *:Ascend\ Embedded/OS:*:*)
-	echo ${UNAME_MACHINE}-pc-bsdi${UNAME_RELEASE}
-	exit 0 ;;
-    sparc*:BSD/OS:*:*)
-	echo sparc-unknown-bsdi${UNAME_RELEASE}
-	exit 0 ;;
-    *:BSD/OS:*:*)
-	echo ${UNAME_MACHINE}-unknown-bsdi${UNAME_RELEASE}
-	exit 0 ;;
-    *:FreeBSD:*:*)
-	echo ${UNAME_MACHINE}-unknown-freebsd`echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`
-	exit 0 ;;
-    i*:CYGWIN*:*)
-	echo ${UNAME_MACHINE}-pc-cygwin
-	exit 0 ;;
-    i*:MINGW*:*)
-	echo ${UNAME_MACHINE}-pc-mingw32
-	exit 0 ;;
-    i*:PW*:*)
-	echo ${UNAME_MACHINE}-pc-pw32
-	exit 0 ;;
-    x86:Interix*:[34]*)
-	echo i586-pc-interix${UNAME_RELEASE}|sed -e 's/\..*//'
-	exit 0 ;;
-    [345]86:Windows_95:* | [345]86:Windows_98:* | [345]86:Windows_NT:*)
-	echo i${UNAME_MACHINE}-pc-mks
-	exit 0 ;;
-    i*:Windows_NT*:* | Pentium*:Windows_NT*:*)
-	# How do we know it's Interix rather than the generic POSIX subsystem?
-	# It also conflicts with pre-2.0 versions of AT&T UWIN. Should we
-	# UNAME_MACHINE based on the output of uname instead of i386?
-	echo i586-pc-interix
-	exit 0 ;;
-    i*:UWIN*:*)
-	echo ${UNAME_MACHINE}-pc-uwin
-	exit 0 ;;
-    p*:CYGWIN*:*)
-	echo powerpcle-unknown-cygwin
-	exit 0 ;;
-    prep*:SunOS:5.*:*)
-	echo powerpcle-unknown-solaris2`echo ${UNAME_RELEASE}|sed -e 's/[^.]*//'`
-	exit 0 ;;
-    *:GNU:*:*)
-	# the GNU system
-	echo `echo ${UNAME_MACHINE}|sed -e 's,[-/].*$,,'`-unknown-gnu`echo ${UNAME_RELEASE}|sed -e 's,/.*$,,'`
-	exit 0 ;;
-    *:GNU/*:*:*)
-	# other systems with GNU libc and userland
-	echo ${UNAME_MACHINE}-unknown-`echo ${UNAME_SYSTEM} | sed 's,^[^/]*/,,' | tr '[A-Z]' '[a-z]'``echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`-gnu
-	exit 0 ;;
-    i*86:Minix:*:*)
-	echo ${UNAME_MACHINE}-pc-minix
-	exit 0 ;;
-    arm*:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    cris:Linux:*:*)
-	echo cris-axis-linux-gnu
-	exit 0 ;;
-    ia64:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    m32r*:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    m68*:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    mips:Linux:*:*)
-	eval $set_cc_for_build
-	sed 's/^	//' << EOF >$dummy.c
-	#undef CPU
-	#undef mips
-	#undef mipsel
-	#if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL) || defined(MIPSEL)
-	CPU=mipsel
-	#else
-	#if defined(__MIPSEB__) || defined(__MIPSEB) || defined(_MIPSEB) || defined(MIPSEB)
-	CPU=mips
-	#else
-	CPU=
-	#endif
-	#endif
-EOF
-	eval `$CC_FOR_BUILD -E $dummy.c 2>/dev/null | grep ^CPU=`
-	test x"${CPU}" != x && echo "${CPU}-unknown-linux-gnu" && exit 0
-	;;
-    mips64:Linux:*:*)
-	eval $set_cc_for_build
-	sed 's/^	//' << EOF >$dummy.c
-	#undef CPU
-	#undef mips64
-	#undef mips64el
-	#if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL) || defined(MIPSEL)
-	CPU=mips64el
-	#else
-	#if defined(__MIPSEB__) || defined(__MIPSEB) || defined(_MIPSEB) || defined(MIPSEB)
-	CPU=mips64
-	#else
-	CPU=
-	#endif
-	#endif
-EOF
-	eval `$CC_FOR_BUILD -E $dummy.c 2>/dev/null | grep ^CPU=`
-	test x"${CPU}" != x && echo "${CPU}-unknown-linux-gnu" && exit 0
-	;;
-    ppc:Linux:*:*)
-	echo powerpc-unknown-linux-gnu
-	exit 0 ;;
-    ppc64:Linux:*:*)
-	echo powerpc64-unknown-linux-gnu
-	exit 0 ;;
-    alpha:Linux:*:*)
-	case `sed -n '/^cpu model/s/^.*: \(.*\)/\1/p' < /proc/cpuinfo` in
-	  EV5)   UNAME_MACHINE=alphaev5 ;;
-	  EV56)  UNAME_MACHINE=alphaev56 ;;
-	  PCA56) UNAME_MACHINE=alphapca56 ;;
-	  PCA57) UNAME_MACHINE=alphapca56 ;;
-	  EV6)   UNAME_MACHINE=alphaev6 ;;
-	  EV67)  UNAME_MACHINE=alphaev67 ;;
-	  EV68*) UNAME_MACHINE=alphaev68 ;;
-        esac
-	objdump --private-headers /bin/sh | grep ld.so.1 >/dev/null
-	if test "$?" = 0 ; then LIBC="libc1" ; else LIBC="" ; fi
-	echo ${UNAME_MACHINE}-unknown-linux-gnu${LIBC}
-	exit 0 ;;
-    parisc:Linux:*:* | hppa:Linux:*:*)
-	# Look for CPU level
-	case `grep '^cpu[^a-z]*:' /proc/cpuinfo 2>/dev/null | cut -d' ' -f2` in
-	  PA7*) echo hppa1.1-unknown-linux-gnu ;;
-	  PA8*) echo hppa2.0-unknown-linux-gnu ;;
-	  *)    echo hppa-unknown-linux-gnu ;;
-	esac
-	exit 0 ;;
-    parisc64:Linux:*:* | hppa64:Linux:*:*)
-	echo hppa64-unknown-linux-gnu
-	exit 0 ;;
-    s390:Linux:*:* | s390x:Linux:*:*)
-	echo ${UNAME_MACHINE}-ibm-linux
-	exit 0 ;;
-    sh64*:Linux:*:*)
-    	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    sh*:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    sparc:Linux:*:* | sparc64:Linux:*:*)
-	echo ${UNAME_MACHINE}-unknown-linux-gnu
-	exit 0 ;;
-    x86_64:Linux:*:*)
-	echo x86_64-unknown-linux-gnu
-	exit 0 ;;
-    i*86:Linux:*:*)
-	# The BFD linker knows what the default object file format is, so
-	# first see if it will tell us. cd to the root directory to prevent
-	# problems with other programs or directories called `ld' in the path.
-	# Set LC_ALL=C to ensure ld outputs messages in English.
-	ld_supported_targets=`cd /; LC_ALL=C ld --help 2>&1 \
-			 | sed -ne '/supported targets:/!d
-				    s/[ 	][ 	]*/ /g
-				    s/.*supported targets: *//
-				    s/ .*//
-				    p'`
-        case "$ld_supported_targets" in
-	  elf32-i386)
-		TENTATIVE="${UNAME_MACHINE}-pc-linux-gnu"
-		;;
-	  a.out-i386-linux)
-		echo "${UNAME_MACHINE}-pc-linux-gnuaout"
-		exit 0 ;;
-	  coff-i386)
-		echo "${UNAME_MACHINE}-pc-linux-gnucoff"
-		exit 0 ;;
-	  "")
-		# Either a pre-BFD a.out linker (linux-gnuoldld) or
-		# one that does not give us useful --help.
-		echo "${UNAME_MACHINE}-pc-linux-gnuoldld"
-		exit 0 ;;
-	esac
-	# Determine whether the default compiler is a.out or elf
-	eval $set_cc_for_build
-	sed 's/^	//' << EOF >$dummy.c
-	#include <features.h>
-	#ifdef __ELF__
-	# ifdef __GLIBC__
-	#  if __GLIBC__ >= 2
-	LIBC=gnu
-	#  else
-	LIBC=gnulibc1
-	#  endif
-	# else
-	LIBC=gnulibc1
-	# endif
-	#else
-	#ifdef __INTEL_COMPILER
-	LIBC=gnu
-	#else
-	LIBC=gnuaout
-	#endif
-	#endif
-	#ifdef __dietlibc__
-	LIBC=dietlibc
-	#endif
-EOF
-	eval `$CC_FOR_BUILD -E $dummy.c 2>/dev/null | grep ^LIBC=`
-	test x"${LIBC}" != x && echo "${UNAME_MACHINE}-pc-linux-${LIBC}" && exit 0
-	test x"${TENTATIVE}" != x && echo "${TENTATIVE}" && exit 0
-	;;
-    i*86:DYNIX/ptx:4*:*)
-	# ptx 4.0 does uname -s correctly, with DYNIX/ptx in there.
-	# earlier versions are messed up and put the nodename in both
-	# sysname and nodename.
-	echo i386-sequent-sysv4
-	exit 0 ;;
-    i*86:UNIX_SV:4.2MP:2.*)
-        # Unixware is an offshoot of SVR4, but it has its own version
-        # number series starting with 2...
-        # I am not positive that other SVR4 systems won't match this,
-	# I just have to hope.  -- rms.
-        # Use sysv4.2uw... so that sysv4* matches it.
-	echo ${UNAME_MACHINE}-pc-sysv4.2uw${UNAME_VERSION}
-	exit 0 ;;
-    i*86:OS/2:*:*)
-	# If we were able to find `uname', then EMX Unix compatibility
-	# is probably installed.
-	echo ${UNAME_MACHINE}-pc-os2-emx
-	exit 0 ;;
-    i*86:XTS-300:*:STOP)
-	echo ${UNAME_MACHINE}-unknown-stop
-	exit 0 ;;
-    i*86:atheos:*:*)
-	echo ${UNAME_MACHINE}-unknown-atheos
-	exit 0 ;;
-	i*86:syllable:*:*)
-	echo ${UNAME_MACHINE}-pc-syllable
-	exit 0 ;;
-    i*86:LynxOS:2.*:* | i*86:LynxOS:3.[01]*:* | i*86:LynxOS:4.0*:*)
-	echo i386-unknown-lynxos${UNAME_RELEASE}
-	exit 0 ;;
-    i*86:*DOS:*:*)
-	echo ${UNAME_MACHINE}-pc-msdosdjgpp
-	exit 0 ;;
-    i*86:*:4.*:* | i*86:SYSTEM_V:4.*:*)
-	UNAME_REL=`echo ${UNAME_RELEASE} | sed 's/\/MP$//'`
-	if grep Novell /usr/include/link.h >/dev/null 2>/dev/null; then
-		echo ${UNAME_MACHINE}-univel-sysv${UNAME_REL}
-	else
-		echo ${UNAME_MACHINE}-pc-sysv${UNAME_REL}
-	fi
-	exit 0 ;;
-    i*86:*:5:[78]*)
-	case `/bin/uname -X | grep "^Machine"` in
-	    *486*)	     UNAME_MACHINE=i486 ;;
-	    *Pentium)	     UNAME_MACHINE=i586 ;;
-	    *Pent*|*Celeron) UNAME_MACHINE=i686 ;;
-	esac
-	echo ${UNAME_MACHINE}-unknown-sysv${UNAME_RELEASE}${UNAME_SYSTEM}${UNAME_VERSION}
-	exit 0 ;;
-    i*86:*:3.2:*)
-	if test -f /usr/options/cb.name; then
-		UNAME_REL=`sed -n 's/.*Version //p' </usr/options/cb.name`
-		echo ${UNAME_MACHINE}-pc-isc$UNAME_REL
-	elif /bin/uname -X 2>/dev/null >/dev/null ; then
-		UNAME_REL=`(/bin/uname -X|grep Release|sed -e 's/.*= //')`
-		(/bin/uname -X|grep i80486 >/dev/null) && UNAME_MACHINE=i486
-		(/bin/uname -X|grep '^Machine.*Pentium' >/dev/null) \
-			&& UNAME_MACHINE=i586
-		(/bin/uname -X|grep '^Machine.*Pent *II' >/dev/null) \
-			&& UNAME_MACHINE=i686
-		(/bin/uname -X|grep '^Machine.*Pentium Pro' >/dev/null) \
-			&& UNAME_MACHINE=i686
-		echo ${UNAME_MACHINE}-pc-sco$UNAME_REL
-	else
-		echo ${UNAME_MACHINE}-pc-sysv32
-	fi
-	exit 0 ;;
-    pc:*:*:*)
-	# Left here for compatibility:
-        # uname -m prints for DJGPP always 'pc', but it prints nothing about
-        # the processor, so we play safe by assuming i386.
-	echo i386-pc-msdosdjgpp
-        exit 0 ;;
-    Intel:Mach:3*:*)
-	echo i386-pc-mach3
-	exit 0 ;;
-    paragon:*:*:*)
-	echo i860-intel-osf1
-	exit 0 ;;
-    i860:*:4.*:*) # i860-SVR4
-	if grep Stardent /usr/include/sys/uadmin.h >/dev/null 2>&1 ; then
-	  echo i860-stardent-sysv${UNAME_RELEASE} # Stardent Vistra i860-SVR4
-	else # Add other i860-SVR4 vendors below as they are discovered.
-	  echo i860-unknown-sysv${UNAME_RELEASE}  # Unknown i860-SVR4
-	fi
-	exit 0 ;;
-    mini*:CTIX:SYS*5:*)
-	# "miniframe"
-	echo m68010-convergent-sysv
-	exit 0 ;;
-    mc68k:UNIX:SYSTEM5:3.51m)
-	echo m68k-convergent-sysv
-	exit 0 ;;
-    M680?0:D-NIX:5.3:*)
-	echo m68k-diab-dnix
-	exit 0 ;;
-    M68*:*:R3V[5678]*:*)
-	test -r /sysV68 && echo 'm68k-motorola-sysv' && exit 0 ;;
-    3[345]??:*:4.0:3.0 | 3[34]??A:*:4.0:3.0 | 3[34]??,*:*:4.0:3.0 | 3[34]??/*:*:4.0:3.0 | 4400:*:4.0:3.0 | 4850:*:4.0:3.0 | SKA40:*:4.0:3.0 | SDS2:*:4.0:3.0 | SHG2:*:4.0:3.0)
-	OS_REL=''
-	test -r /etc/.relid \
-	&& OS_REL=.`sed -n 's/[^ ]* [^ ]* \([0-9][0-9]\).*/\1/p' < /etc/.relid`
-	/bin/uname -p 2>/dev/null | grep 86 >/dev/null \
-	  && echo i486-ncr-sysv4.3${OS_REL} && exit 0
-	/bin/uname -p 2>/dev/null | /bin/grep entium >/dev/null \
-	  && echo i586-ncr-sysv4.3${OS_REL} && exit 0 ;;
-    3[34]??:*:4.0:* | 3[34]??,*:*:4.0:*)
-        /bin/uname -p 2>/dev/null | grep 86 >/dev/null \
-          && echo i486-ncr-sysv4 && exit 0 ;;
-    m68*:LynxOS:2.*:* | m68*:LynxOS:3.0*:*)
-	echo m68k-unknown-lynxos${UNAME_RELEASE}
-	exit 0 ;;
-    mc68030:UNIX_System_V:4.*:*)
-	echo m68k-atari-sysv4
-	exit 0 ;;
-    TSUNAMI:LynxOS:2.*:*)
-	echo sparc-unknown-lynxos${UNAME_RELEASE}
-	exit 0 ;;
-    rs6000:LynxOS:2.*:*)
-	echo rs6000-unknown-lynxos${UNAME_RELEASE}
-	exit 0 ;;
-    PowerPC:LynxOS:2.*:* | PowerPC:LynxOS:3.[01]*:* | PowerPC:LynxOS:4.0*:*)
-	echo powerpc-unknown-lynxos${UNAME_RELEASE}
-	exit 0 ;;
-    SM[BE]S:UNIX_SV:*:*)
-	echo mips-dde-sysv${UNAME_RELEASE}
-	exit 0 ;;
-    RM*:ReliantUNIX-*:*:*)
-	echo mips-sni-sysv4
-	exit 0 ;;
-    RM*:SINIX-*:*:*)
-	echo mips-sni-sysv4
-	exit 0 ;;
-    *:SINIX-*:*:*)
-	if uname -p 2>/dev/null >/dev/null ; then
-		UNAME_MACHINE=`(uname -p) 2>/dev/null`
-		echo ${UNAME_MACHINE}-sni-sysv4
-	else
-		echo ns32k-sni-sysv
-	fi
-	exit 0 ;;
-    PENTIUM:*:4.0*:*) # Unisys `ClearPath HMP IX 4000' SVR4/MP effort
-                      # says <Richard.M.Bartel@ccMail.Census.GOV>
-        echo i586-unisys-sysv4
-        exit 0 ;;
-    *:UNIX_System_V:4*:FTX*)
-	# From Gerald Hewes <hewes@openmarket.com>.
-	# How about differentiating between stratus architectures? -djm
-	echo hppa1.1-stratus-sysv4
-	exit 0 ;;
-    *:*:*:FTX*)
-	# From seanf@swdc.stratus.com.
-	echo i860-stratus-sysv4
-	exit 0 ;;
-    *:VOS:*:*)
-	# From Paul.Green@stratus.com.
-	echo hppa1.1-stratus-vos
-	exit 0 ;;
-    mc68*:A/UX:*:*)
-	echo m68k-apple-aux${UNAME_RELEASE}
-	exit 0 ;;
-    news*:NEWS-OS:6*:*)
-	echo mips-sony-newsos6
-	exit 0 ;;
-    R[34]000:*System_V*:*:* | R4000:UNIX_SYSV:*:* | R*000:UNIX_SV:*:*)
-	if [ -d /usr/nec ]; then
-	        echo mips-nec-sysv${UNAME_RELEASE}
-	else
-	        echo mips-unknown-sysv${UNAME_RELEASE}
-	fi
-        exit 0 ;;
-    BeBox:BeOS:*:*)	# BeOS running on hardware made by Be, PPC only.
-	echo powerpc-be-beos
-	exit 0 ;;
-    BeMac:BeOS:*:*)	# BeOS running on Mac or Mac clone, PPC only.
-	echo powerpc-apple-beos
-	exit 0 ;;
-    BePC:BeOS:*:*)	# BeOS running on Intel PC compatible.
-	echo i586-pc-beos
-	exit 0 ;;
-    SX-4:SUPER-UX:*:*)
-	echo sx4-nec-superux${UNAME_RELEASE}
-	exit 0 ;;
-    SX-5:SUPER-UX:*:*)
-	echo sx5-nec-superux${UNAME_RELEASE}
-	exit 0 ;;
-    SX-6:SUPER-UX:*:*)
-	echo sx6-nec-superux${UNAME_RELEASE}
-	exit 0 ;;
-    Power*:Rhapsody:*:*)
-	echo powerpc-apple-rhapsody${UNAME_RELEASE}
-	exit 0 ;;
-    *:Rhapsody:*:*)
-	echo ${UNAME_MACHINE}-apple-rhapsody${UNAME_RELEASE}
-	exit 0 ;;
-    *:Darwin:*:*)
-	case `uname -p` in
-	    *86) UNAME_PROCESSOR=i686 ;;
-	    powerpc) UNAME_PROCESSOR=powerpc ;;
-	esac
-	echo ${UNAME_PROCESSOR}-apple-darwin${UNAME_RELEASE}
-	exit 0 ;;
-    *:procnto*:*:* | *:QNX:[0123456789]*:*)
-	UNAME_PROCESSOR=`uname -p`
-	if test "$UNAME_PROCESSOR" = "x86"; then
-		UNAME_PROCESSOR=i386
-		UNAME_MACHINE=pc
-	fi
-	echo ${UNAME_PROCESSOR}-${UNAME_MACHINE}-nto-qnx${UNAME_RELEASE}
-	exit 0 ;;
-    *:QNX:*:4*)
-	echo i386-pc-qnx
-	exit 0 ;;
-    NSR-?:NONSTOP_KERNEL:*:*)
-	echo nsr-tandem-nsk${UNAME_RELEASE}
-	exit 0 ;;
-    *:NonStop-UX:*:*)
-	echo mips-compaq-nonstopux
-	exit 0 ;;
-    BS2000:POSIX*:*:*)
-	echo bs2000-siemens-sysv
-	exit 0 ;;
-    DS/*:UNIX_System_V:*:*)
-	echo ${UNAME_MACHINE}-${UNAME_SYSTEM}-${UNAME_RELEASE}
-	exit 0 ;;
-    *:Plan9:*:*)
-	# "uname -m" is not consistent, so use $cputype instead. 386
-	# is converted to i386 for consistency with other x86
-	# operating systems.
-	if test "$cputype" = "386"; then
-	    UNAME_MACHINE=i386
-	else
-	    UNAME_MACHINE="$cputype"
-	fi
-	echo ${UNAME_MACHINE}-unknown-plan9
-	exit 0 ;;
-    *:TOPS-10:*:*)
-	echo pdp10-unknown-tops10
-	exit 0 ;;
-    *:TENEX:*:*)
-	echo pdp10-unknown-tenex
-	exit 0 ;;
-    KS10:TOPS-20:*:* | KL10:TOPS-20:*:* | TYPE4:TOPS-20:*:*)
-	echo pdp10-dec-tops20
-	exit 0 ;;
-    XKL-1:TOPS-20:*:* | TYPE5:TOPS-20:*:*)
-	echo pdp10-xkl-tops20
-	exit 0 ;;
-    *:TOPS-20:*:*)
-	echo pdp10-unknown-tops20
-	exit 0 ;;
-    *:ITS:*:*)
-	echo pdp10-unknown-its
-	exit 0 ;;
-    SEI:*:*:SEIUX)
-        echo mips-sei-seiux${UNAME_RELEASE}
-	exit 0 ;;
-    *:DragonFly:*:*)
-	echo ${UNAME_MACHINE}-unknown-dragonfly`echo ${UNAME_RELEASE}|sed -e 's/[-(].*//'`
-	exit 0 ;;
-    *:*VMS:*:*)
-    	UNAME_MACHINE=`(uname -p) 2>/dev/null`
-	case "${UNAME_MACHINE}" in
-	    A*) echo alpha-dec-vms && exit 0 ;;
-	    I*) echo ia64-dec-vms && exit 0 ;;
-	    V*) echo vax-dec-vms && exit 0 ;;
-	esac
-esac
-
-#echo '(No uname command or uname output not recognized.)' 1>&2
-#echo "${UNAME_MACHINE}:${UNAME_SYSTEM}:${UNAME_RELEASE}:${UNAME_VERSION}" 1>&2
-
-eval $set_cc_for_build
-cat >$dummy.c <<EOF
-#ifdef _SEQUENT_
-# include <sys/types.h>
-# include <sys/utsname.h>
-#endif
-main ()
-{
-#if defined (sony)
-#if defined (MIPSEB)
-  /* BFD wants "bsd" instead of "newsos".  Perhaps BFD should be changed,
-     I don't know....  */
-  printf ("mips-sony-bsd\n"); exit (0);
-#else
-#include <sys/param.h>
-  printf ("m68k-sony-newsos%s\n",
-#ifdef NEWSOS4
-          "4"
-#else
-	  ""
-#endif
-         ); exit (0);
-#endif
-#endif
-
-#if defined (__arm) && defined (__acorn) && defined (__unix)
-  printf ("arm-acorn-riscix"); exit (0);
-#endif
-
-#if defined (hp300) && !defined (hpux)
-  printf ("m68k-hp-bsd\n"); exit (0);
-#endif
-
-#if defined (NeXT)
-#if !defined (__ARCHITECTURE__)
-#define __ARCHITECTURE__ "m68k"
-#endif
-  int version;
-  version=`(hostinfo | sed -n 's/.*NeXT Mach \([0-9]*\).*/\1/p') 2>/dev/null`;
-  if (version < 4)
-    printf ("%s-next-nextstep%d\n", __ARCHITECTURE__, version);
-  else
-    printf ("%s-next-openstep%d\n", __ARCHITECTURE__, version);
-  exit (0);
-#endif
-
-#if defined (MULTIMAX) || defined (n16)
-#if defined (UMAXV)
-  printf ("ns32k-encore-sysv\n"); exit (0);
-#else
-#if defined (CMU)
-  printf ("ns32k-encore-mach\n"); exit (0);
-#else
-  printf ("ns32k-encore-bsd\n"); exit (0);
-#endif
-#endif
-#endif
-
-#if defined (__386BSD__)
-  printf ("i386-pc-bsd\n"); exit (0);
-#endif
-
-#if defined (sequent)
-#if defined (i386)
-  printf ("i386-sequent-dynix\n"); exit (0);
-#endif
-#if defined (ns32000)
-  printf ("ns32k-sequent-dynix\n"); exit (0);
-#endif
-#endif
-
-#if defined (_SEQUENT_)
-    struct utsname un;
-
-    uname(&un);
-
-    if (strncmp(un.version, "V2", 2) == 0) {
-	printf ("i386-sequent-ptx2\n"); exit (0);
-    }
-    if (strncmp(un.version, "V1", 2) == 0) { /* XXX is V1 correct? */
-	printf ("i386-sequent-ptx1\n"); exit (0);
-    }
-    printf ("i386-sequent-ptx\n"); exit (0);
-
-#endif
-
-#if defined (vax)
-# if !defined (ultrix)
-#  include <sys/param.h>
-#  if defined (BSD)
-#   if BSD == 43
-      printf ("vax-dec-bsd4.3\n"); exit (0);
-#   else
-#    if BSD == 199006
-      printf ("vax-dec-bsd4.3reno\n"); exit (0);
-#    else
-      printf ("vax-dec-bsd\n"); exit (0);
-#    endif
-#   endif
-#  else
-    printf ("vax-dec-bsd\n"); exit (0);
-#  endif
-# else
-    printf ("vax-dec-ultrix\n"); exit (0);
-# endif
-#endif
-
-#if defined (alliant) && defined (i860)
-  printf ("i860-alliant-bsd\n"); exit (0);
-#endif
-
-  exit (1);
-}
-EOF
-
-$CC_FOR_BUILD -o $dummy $dummy.c 2>/dev/null && $dummy && exit 0
-
-# Apollos put the system type in the environment.
-
-test -d /usr/apollo && { echo ${ISP}-apollo-${SYSTYPE}; exit 0; }
-
-# Convex versions that predate uname can use getsysinfo(1)
-
-if [ -x /usr/convex/getsysinfo ]
-then
-    case `getsysinfo -f cpu_type` in
-    c1*)
-	echo c1-convex-bsd
-	exit 0 ;;
-    c2*)
-	if getsysinfo -f scalar_acc
-	then echo c32-convex-bsd
-	else echo c2-convex-bsd
-	fi
-	exit 0 ;;
-    c34*)
-	echo c34-convex-bsd
-	exit 0 ;;
-    c38*)
-	echo c38-convex-bsd
-	exit 0 ;;
-    c4*)
-	echo c4-convex-bsd
-	exit 0 ;;
-    esac
-fi
-
-cat >&2 <<EOF
-$0: unable to guess system type
-
-This script, last modified $timestamp, has failed to recognize
-the operating system you are using. It is advised that you
-download the most up to date version of the config scripts from
-
-    ftp://ftp.gnu.org/pub/gnu/config/
-
-If the version you run ($0) is already up to date, please
-send the following data and any information you think might be
-pertinent to <config-patches@gnu.org> in order to provide the needed
-information to handle your system.
-
-config.guess timestamp = $timestamp
-
-uname -m = `(uname -m) 2>/dev/null || echo unknown`
-uname -r = `(uname -r) 2>/dev/null || echo unknown`
-uname -s = `(uname -s) 2>/dev/null || echo unknown`
-uname -v = `(uname -v) 2>/dev/null || echo unknown`
-
-/usr/bin/uname -p = `(/usr/bin/uname -p) 2>/dev/null`
-/bin/uname -X     = `(/bin/uname -X) 2>/dev/null`
-
-hostinfo               = `(hostinfo) 2>/dev/null`
-/bin/universe          = `(/bin/universe) 2>/dev/null`
-/usr/bin/arch -k       = `(/usr/bin/arch -k) 2>/dev/null`
-/bin/arch              = `(/bin/arch) 2>/dev/null`
-/usr/bin/oslevel       = `(/usr/bin/oslevel) 2>/dev/null`
-/usr/convex/getsysinfo = `(/usr/convex/getsysinfo) 2>/dev/null`
-
-UNAME_MACHINE = ${UNAME_MACHINE}
-UNAME_RELEASE = ${UNAME_RELEASE}
-UNAME_SYSTEM  = ${UNAME_SYSTEM}
-UNAME_VERSION = ${UNAME_VERSION}
-EOF
-
-exit 1
-
-# Local variables:
-# eval: (add-hook 'write-file-hooks 'time-stamp)
-# time-stamp-start: "timestamp='"
-# time-stamp-format: "%:y-%02m-%02d"
-# time-stamp-end: "'"
-# End:
diff -pruN 1:13.23.1~dfsg-2/main/editline/config.h.in 1:16.1.1~dfsg-1/main/editline/config.h.in
--- 1:13.23.1~dfsg-2/main/editline/config.h.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/config.h.in	1970-01-01 00:00:00.000000000 +0000
@@ -1,21 +0,0 @@
-#undef SUNOS
-#undef CYGWIN
-
-#undef HAVE_SYS_CDEFS_H
-#undef HAVE_TERMCAP_H
-#undef HAVE_CURSES_H
-#undef HAVE_NCURSES_H
-#undef HAVE_TERM_H
-#undef HAVE_VIS_H
-#undef HAVE_ISSETUGID
-
-#undef HAVE_STRLCAT
-#undef HAVE_STRLCPY
-#undef HAVE_FGETLN
-#undef HAVE_STRVIS
-#undef HAVE_STRUNVIS
-
-#include "sys.h"
-#ifdef CYGWIN
-# include "cygdef.h"
-#endif
diff -pruN 1:13.23.1~dfsg-2/main/editline/config.sub 1:16.1.1~dfsg-1/main/editline/config.sub
--- 1:13.23.1~dfsg-2/main/editline/config.sub	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/config.sub	1970-01-01 00:00:00.000000000 +0000
@@ -1,1412 +0,0 @@
-#! /bin/sh
-# Configuration validation subroutine script.
-#   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
-#   Free Software Foundation, Inc.
-
-timestamp='2001-09-14'
-
-# This file is (in principle) common to ALL GNU software.
-# The presence of a machine in this file suggests that SOME GNU software
-# can handle that machine.  It does not imply ALL GNU software can.
-#
-# This file is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
-
-# As a special exception to the GNU General Public License, if you
-# distribute this file as part of a program that contains a
-# configuration script generated by Autoconf, you may include it under
-# the same distribution terms that you use for the rest of that program.
-
-# Please send patches to <config-patches@gnu.org>.
-#
-# Configuration subroutine to validate and canonicalize a configuration type.
-# Supply the specified configuration type as an argument.
-# If it is invalid, we print an error message on stderr and exit with code 1.
-# Otherwise, we print the canonical config type on stdout and succeed.
-
-# This file is supposed to be the same for all GNU packages
-# and recognize all the CPU types, system types and aliases
-# that are meaningful with *any* GNU software.
-# Each package is responsible for reporting which valid configurations
-# it does not support.  The user should be able to distinguish
-# a failure to support a valid configuration from a meaningless
-# configuration.
-
-# The goal of this file is to map all the various variations of a given
-# machine specification into a single specification in the form:
-#	CPU_TYPE-MANUFACTURER-OPERATING_SYSTEM
-# or in some cases, the newer four-part form:
-#	CPU_TYPE-MANUFACTURER-KERNEL-OPERATING_SYSTEM
-# It is wrong to echo any other type of specification.
-
-me=`echo "$0" | sed -e 's,.*/,,'`
-
-usage="\
-Usage: $0 [OPTION] CPU-MFR-OPSYS
-       $0 [OPTION] ALIAS
-
-Canonicalize a configuration name.
-
-Operation modes:
-  -h, --help         print this help, then exit
-  -t, --time-stamp   print date of last modification, then exit
-  -v, --version      print version number, then exit
-
-Report bugs and patches to <config-patches@gnu.org>."
-
-version="\
-GNU config.sub ($timestamp)
-
-Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001
-Free Software Foundation, Inc.
-
-This is free software; see the source for copying conditions.  There is NO
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE."
-
-help="
-Try \`$me --help' for more information."
-
-# Parse command line
-while test $# -gt 0 ; do
-  case $1 in
-    --time-stamp | --time* | -t )
-       echo "$timestamp" ; exit 0 ;;
-    --version | -v )
-       echo "$version" ; exit 0 ;;
-    --help | --h* | -h )
-       echo "$usage"; exit 0 ;;
-    -- )     # Stop option processing
-       shift; break ;;
-    - )	# Use stdin as input.
-       break ;;
-    -* )
-       echo "$me: invalid option $1$help"
-       exit 1 ;;
-
-    *local*)
-       # First pass through any local machine types.
-       echo $1
-       exit 0;;
-
-    * )
-       break ;;
-  esac
-done
-
-case $# in
- 0) echo "$me: missing argument$help" >&2
-    exit 1;;
- 1) ;;
- *) echo "$me: too many arguments$help" >&2
-    exit 1;;
-esac
-
-# Separate what the user gave into CPU-COMPANY and OS or KERNEL-OS (if any).
-# Here we must recognize all the valid KERNEL-OS combinations.
-maybe_os=`echo $1 | sed 's/^\(.*\)-\([^-]*-[^-]*\)$/\2/'`
-case $maybe_os in
-  nto-qnx* | linux-gnu* | storm-chaos* | os2-emx* | windows32-*)
-    os=-$maybe_os
-    basic_machine=`echo $1 | sed 's/^\(.*\)-\([^-]*-[^-]*\)$/\1/'`
-    ;;
-  *)
-    basic_machine=`echo $1 | sed 's/-[^-]*$//'`
-    if [ $basic_machine != $1 ]
-    then os=`echo $1 | sed 's/.*-/-/'`
-    else os=; fi
-    ;;
-esac
-
-### Let's recognize common machines as not being operating systems so
-### that things like config.sub decstation-3100 work.  We also
-### recognize some manufacturers as not being operating systems, so we
-### can provide default operating systems below.
-case $os in
-	-sun*os*)
-		# Prevent following clause from handling this invalid input.
-		;;
-	-dec* | -mips* | -sequent* | -encore* | -pc532* | -sgi* | -sony* | \
-	-att* | -7300* | -3300* | -delta* | -motorola* | -sun[234]* | \
-	-unicom* | -ibm* | -next | -hp | -isi* | -apollo | -altos* | \
-	-convergent* | -ncr* | -news | -32* | -3600* | -3100* | -hitachi* |\
-	-c[123]* | -convex* | -sun | -crds | -omron* | -dg | -ultra | -tti* | \
-	-harris | -dolphin | -highlevel | -gould | -cbm | -ns | -masscomp | \
-	-apple | -axis)
-		os=
-		basic_machine=$1
-		;;
-	-sim | -cisco | -oki | -wec | -winbond)
-		os=
-		basic_machine=$1
-		;;
-	-scout)
-		;;
-	-wrs)
-		os=-vxworks
-		basic_machine=$1
-		;;
-	-chorusos*)
-		os=-chorusos
-		basic_machine=$1
-		;;
- 	-chorusrdb)
- 		os=-chorusrdb
-		basic_machine=$1
- 		;;
-	-hiux*)
-		os=-hiuxwe2
-		;;
-	-sco5)
-		os=-sco3.2v5
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-sco4)
-		os=-sco3.2v4
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-sco3.2.[4-9]*)
-		os=`echo $os | sed -e 's/sco3.2./sco3.2v/'`
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-sco3.2v[4-9]*)
-		# Don't forget version if it is 3.2v4 or newer.
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-sco*)
-		os=-sco3.2v2
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-udk*)
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-isc)
-		os=-isc2.2
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-clix*)
-		basic_machine=clipper-intergraph
-		;;
-	-isc*)
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-pc/'`
-		;;
-	-lynx*)
-		os=-lynxos
-		;;
-	-ptx*)
-		basic_machine=`echo $1 | sed -e 's/86-.*/86-sequent/'`
-		;;
-	-windowsnt*)
-		os=`echo $os | sed -e 's/windowsnt/winnt/'`
-		;;
-	-psos*)
-		os=-psos
-		;;
-	-mint | -mint[0-9]*)
-		basic_machine=m68k-atari
-		os=-mint
-		;;
-esac
-
-# Decode aliases for certain CPU-COMPANY combinations.
-case $basic_machine in
-	# Recognize the basic CPU types without company name.
-	# Some are omitted here because they have special meanings below.
-	1750a | 580 \
-	| a29k \
-	| alpha | alphaev[4-8] | alphaev56 | alphaev6[78] | alphapca5[67] \
-	| arc | arm | arm[bl]e | arme[lb] | armv[2345] | armv[345][lb] | avr \
-	| c4x | clipper \
-	| d10v | d30v | dsp16xx \
-	| fr30 \
-	| h8300 | h8500 | hppa | hppa1.[01] | hppa2.0 | hppa2.0[nw] | hppa64 \
-	| i370 | i860 | i960 | ia64 \
-	| m32r | m68000 | m68k | m88k | mcore \
-	| mips16 | mips64 | mips64el | mips64orion | mips64orionel \
-	| mips64vr4100 | mips64vr4100el | mips64vr4300 \
-	| mips64vr4300el | mips64vr5000 | mips64vr5000el \
-	| mipsbe | mipseb | mipsel | mipsle | mipstx39 | mipstx39el \
-	| mipsisa32 \
-	| mn10200 | mn10300 \
-	| ns16k | ns32k \
-	| openrisc \
-	| pdp10 | pdp11 | pj | pjl \
-	| powerpc | powerpc64 | powerpc64le | powerpcle | ppcbe \
-	| pyramid \
-	| s390 | s390x \
-	| sh | sh[34] | sh[34]eb | shbe | shle \
-	| sparc | sparc64 | sparclet | sparclite | sparcv9 | sparcv9b \
-	| stormy16 | strongarm \
-	| tahoe | thumb | tic80 | tron \
-	| v850 \
-	| we32k \
-	| x86 | xscale \
-	| z8k)
-		basic_machine=$basic_machine-unknown
-		;;
-	m6811 | m68hc11 | m6812 | m68hc12)
-		# Motorola 68HC11/12.
-		basic_machine=$basic_machine-unknown
-		os=-none
-		;;
-	m88110 | m680[12346]0 | m683?2 | m68360 | m5200 | v70 | w65 | z8k)
-		;;
-
-	# We use `pc' rather than `unknown'
-	# because (1) that's what they normally are, and
-	# (2) the word "unknown" tends to confuse beginning users.
-	i*86 | x86_64)
-	  basic_machine=$basic_machine-pc
-	  ;;
-	# Object if more than one company name word.
-	*-*-*)
-		echo Invalid configuration \`$1\': machine \`$basic_machine\' not recognized 1>&2
-		exit 1
-		;;
-	# Recognize the basic CPU types with company name.
-	580-* \
-	| a29k-* \
-	| alpha-* | alphaev[4-8]-* | alphaev56-* | alphaev6[78]-* \
-	| alphapca5[67]-* | arc-* \
-	| arm-*  | armbe-* | armle-* | armv*-* \
-	| avr-* \
-	| bs2000-* \
-	| c[123]* | c30-* | [cjt]90-* | c54x-* \
-	| clipper-* | cray2-* | cydra-* \
-	| d10v-* | d30v-* \
-	| elxsi-* \
-	| f30[01]-* | f700-* | fr30-* | fx80-* \
-	| h8300-* | h8500-* \
-	| hppa-* | hppa1.[01]-* | hppa2.0-* | hppa2.0[nw]-* | hppa64-* \
-	| i*86-* | i860-* | i960-* | ia64-* \
-	| m32r-* \
-	| m68000-* | m680[01234]0-* | m68360-* | m683?2-* | m68k-* \
-	| m88110-* | m88k-* | mcore-* \
-	| mips-* | mips16-* | mips64-* | mips64el-* | mips64orion-* \
-	| mips64orionel-* | mips64vr4100-* | mips64vr4100el-* \
-	| mips64vr4300-* | mips64vr4300el-* | mipsbe-* | mipseb-* \
-	| mipsle-* | mipsel-* | mipstx39-* | mipstx39el-* \
-	| none-* | np1-* | ns16k-* | ns32k-* \
-	| orion-* \
-	| pdp10-* | pdp11-* | pj-* | pjl-* | pn-* | power-* \
-	| powerpc-* | powerpc64-* | powerpc64le-* | powerpcle-* | ppcbe-* \
-	| pyramid-* \
-	| romp-* | rs6000-* \
-	| s390-* | s390x-* \
-	| sh-* | sh[34]-* | sh[34]eb-* | shbe-* | shle-* \
-	| sparc-* | sparc64-* | sparc86x-* | sparclite-* \
-	| sparcv9-* | sparcv9b-* | stormy16-* | strongarm-* | sv1-* \
-	| t3e-* | tahoe-* | thumb-* | tic30-* | tic54x-* | tic80-* | tron-* \
-	| v850-* | vax-* \
-	| we32k-* \
-	| x86-* | x86_64-* | xmp-* | xps100-* | xscale-* \
-	| ymp-* \
-	| z8k-*)
-		;;
-	# Recognize the various machine names and aliases which stand
-	# for a CPU type and a company and sometimes even an OS.
-	386bsd)
-		basic_machine=i386-unknown
-		os=-bsd
-		;;
-	3b1 | 7300 | 7300-att | att-7300 | pc7300 | safari | unixpc)
-		basic_machine=m68000-att
-		;;
-	3b*)
-		basic_machine=we32k-att
-		;;
-	a29khif)
-		basic_machine=a29k-amd
-		os=-udi
-		;;
-	adobe68k)
-		basic_machine=m68010-adobe
-		os=-scout
-		;;
-	alliant | fx80)
-		basic_machine=fx80-alliant
-		;;
-	altos | altos3068)
-		basic_machine=m68k-altos
-		;;
-	am29k)
-		basic_machine=a29k-none
-		os=-bsd
-		;;
-	amdahl)
-		basic_machine=580-amdahl
-		os=-sysv
-		;;
-	amiga | amiga-*)
-		basic_machine=m68k-unknown
-		;;
-	amigaos | amigados)
-		basic_machine=m68k-unknown
-		os=-amigaos
-		;;
-	amigaunix | amix)
-		basic_machine=m68k-unknown
-		os=-sysv4
-		;;
-	apollo68)
-		basic_machine=m68k-apollo
-		os=-sysv
-		;;
-	apollo68bsd)
-		basic_machine=m68k-apollo
-		os=-bsd
-		;;
-	aux)
-		basic_machine=m68k-apple
-		os=-aux
-		;;
-	balance)
-		basic_machine=ns32k-sequent
-		os=-dynix
-		;;
-	convex-c1)
-		basic_machine=c1-convex
-		os=-bsd
-		;;
-	convex-c2)
-		basic_machine=c2-convex
-		os=-bsd
-		;;
-	convex-c32)
-		basic_machine=c32-convex
-		os=-bsd
-		;;
-	convex-c34)
-		basic_machine=c34-convex
-		os=-bsd
-		;;
-	convex-c38)
-		basic_machine=c38-convex
-		os=-bsd
-		;;
-	cray | ymp)
-		basic_machine=ymp-cray
-		os=-unicos
-		;;
-	cray2)
-		basic_machine=cray2-cray
-		os=-unicos
-		;;
-	[cjt]90)
-		basic_machine=${basic_machine}-cray
-		os=-unicos
-		;;
-	crds | unos)
-		basic_machine=m68k-crds
-		;;
-	cris | cris-* | etrax*)
-		basic_machine=cris-axis
-		;;
-	da30 | da30-*)
-		basic_machine=m68k-da30
-		;;
-	decstation | decstation-3100 | pmax | pmax-* | pmin | dec3100 | decstatn)
-		basic_machine=mips-dec
-		;;
-	delta | 3300 | motorola-3300 | motorola-delta \
-	      | 3300-motorola | delta-motorola)
-		basic_machine=m68k-motorola
-		;;
-	delta88)
-		basic_machine=m88k-motorola
-		os=-sysv3
-		;;
-	dpx20 | dpx20-*)
-		basic_machine=rs6000-bull
-		os=-bosx
-		;;
-	dpx2* | dpx2*-bull)
-		basic_machine=m68k-bull
-		os=-sysv3
-		;;
-	ebmon29k)
-		basic_machine=a29k-amd
-		os=-ebmon
-		;;
-	elxsi)
-		basic_machine=elxsi-elxsi
-		os=-bsd
-		;;
-	encore | umax | mmax)
-		basic_machine=ns32k-encore
-		;;
-	es1800 | OSE68k | ose68k | ose | OSE)
-		basic_machine=m68k-ericsson
-		os=-ose
-		;;
-	fx2800)
-		basic_machine=i860-alliant
-		;;
-	genix)
-		basic_machine=ns32k-ns
-		;;
-	gmicro)
-		basic_machine=tron-gmicro
-		os=-sysv
-		;;
-	go32)
-		basic_machine=i386-pc
-		os=-go32
-		;;
-	h3050r* | hiux*)
-		basic_machine=hppa1.1-hitachi
-		os=-hiuxwe2
-		;;
-	h8300hms)
-		basic_machine=h8300-hitachi
-		os=-hms
-		;;
-	h8300xray)
-		basic_machine=h8300-hitachi
-		os=-xray
-		;;
-	h8500hms)
-		basic_machine=h8500-hitachi
-		os=-hms
-		;;
-	harris)
-		basic_machine=m88k-harris
-		os=-sysv3
-		;;
-	hp300-*)
-		basic_machine=m68k-hp
-		;;
-	hp300bsd)
-		basic_machine=m68k-hp
-		os=-bsd
-		;;
-	hp300hpux)
-		basic_machine=m68k-hp
-		os=-hpux
-		;;
-	hp3k9[0-9][0-9] | hp9[0-9][0-9])
-		basic_machine=hppa1.0-hp
-		;;
-	hp9k2[0-9][0-9] | hp9k31[0-9])
-		basic_machine=m68000-hp
-		;;
-	hp9k3[2-9][0-9])
-		basic_machine=m68k-hp
-		;;
-	hp9k6[0-9][0-9] | hp6[0-9][0-9])
-		basic_machine=hppa1.0-hp
-		;;
-	hp9k7[0-79][0-9] | hp7[0-79][0-9])
-		basic_machine=hppa1.1-hp
-		;;
-	hp9k78[0-9] | hp78[0-9])
-		# FIXME: really hppa2.0-hp
-		basic_machine=hppa1.1-hp
-		;;
-	hp9k8[67]1 | hp8[67]1 | hp9k80[24] | hp80[24] | hp9k8[78]9 | hp8[78]9 | hp9k893 | hp893)
-		# FIXME: really hppa2.0-hp
-		basic_machine=hppa1.1-hp
-		;;
-	hp9k8[0-9][13679] | hp8[0-9][13679])
-		basic_machine=hppa1.1-hp
-		;;
-	hp9k8[0-9][0-9] | hp8[0-9][0-9])
-		basic_machine=hppa1.0-hp
-		;;
-	hppa-next)
-		os=-nextstep3
-		;;
-	hppaosf)
-		basic_machine=hppa1.1-hp
-		os=-osf
-		;;
-	hppro)
-		basic_machine=hppa1.1-hp
-		os=-proelf
-		;;
-	i370-ibm* | ibm*)
-		basic_machine=i370-ibm
-		;;
-# I'm not sure what "Sysv32" means.  Should this be sysv3.2?
-	i*86v32)
-		basic_machine=`echo $1 | sed -e 's/86.*/86-pc/'`
-		os=-sysv32
-		;;
-	i*86v4*)
-		basic_machine=`echo $1 | sed -e 's/86.*/86-pc/'`
-		os=-sysv4
-		;;
-	i*86v)
-		basic_machine=`echo $1 | sed -e 's/86.*/86-pc/'`
-		os=-sysv
-		;;
-	i*86sol2)
-		basic_machine=`echo $1 | sed -e 's/86.*/86-pc/'`
-		os=-solaris2
-		;;
-	i386mach)
-		basic_machine=i386-mach
-		os=-mach
-		;;
-	i386-vsta | vsta)
-		basic_machine=i386-unknown
-		os=-vsta
-		;;
-	iris | iris4d)
-		basic_machine=mips-sgi
-		case $os in
-		    -irix*)
-			;;
-		    *)
-			os=-irix4
-			;;
-		esac
-		;;
-	isi68 | isi)
-		basic_machine=m68k-isi
-		os=-sysv
-		;;
-	m88k-omron*)
-		basic_machine=m88k-omron
-		;;
-	magnum | m3230)
-		basic_machine=mips-mips
-		os=-sysv
-		;;
-	merlin)
-		basic_machine=ns32k-utek
-		os=-sysv
-		;;
-	mingw32)
-		basic_machine=i386-pc
-		os=-mingw32
-		;;
-	miniframe)
-		basic_machine=m68000-convergent
-		;;
-	*mint | -mint[0-9]* | *MiNT | *MiNT[0-9]*)
-		basic_machine=m68k-atari
-		os=-mint
-		;;
-	mipsel*-linux*)
-		basic_machine=mipsel-unknown
-		os=-linux-gnu
-		;;
-	mips*-linux*)
-		basic_machine=mips-unknown
-		os=-linux-gnu
-		;;
-	mips3*-*)
-		basic_machine=`echo $basic_machine | sed -e 's/mips3/mips64/'`
-		;;
-	mips3*)
-		basic_machine=`echo $basic_machine | sed -e 's/mips3/mips64/'`-unknown
-		;;
-	mmix*)
-		basic_machine=mmix-knuth
-		os=-mmixware
-		;;
-	monitor)
-		basic_machine=m68k-rom68k
-		os=-coff
-		;;
-	msdos)
-		basic_machine=i386-pc
-		os=-msdos
-		;;
-	mvs)
-		basic_machine=i370-ibm
-		os=-mvs
-		;;
-	ncr3000)
-		basic_machine=i486-ncr
-		os=-sysv4
-		;;
-	netbsd386)
-		basic_machine=i386-unknown
-		os=-netbsd
-		;;
-	netwinder)
-		basic_machine=armv4l-rebel
-		os=-linux
-		;;
-	news | news700 | news800 | news900)
-		basic_machine=m68k-sony
-		os=-newsos
-		;;
-	news1000)
-		basic_machine=m68030-sony
-		os=-newsos
-		;;
-	news-3600 | risc-news)
-		basic_machine=mips-sony
-		os=-newsos
-		;;
-	necv70)
-		basic_machine=v70-nec
-		os=-sysv
-		;;
-	next | m*-next )
-		basic_machine=m68k-next
-		case $os in
-		    -nextstep* )
-			;;
-		    -ns2*)
-		      os=-nextstep2
-			;;
-		    *)
-		      os=-nextstep3
-			;;
-		esac
-		;;
-	nh3000)
-		basic_machine=m68k-harris
-		os=-cxux
-		;;
-	nh[45]000)
-		basic_machine=m88k-harris
-		os=-cxux
-		;;
-	nindy960)
-		basic_machine=i960-intel
-		os=-nindy
-		;;
-	mon960)
-		basic_machine=i960-intel
-		os=-mon960
-		;;
-	nonstopux)
-		basic_machine=mips-compaq
-		os=-nonstopux
-		;;
-	np1)
-		basic_machine=np1-gould
-		;;
-	nsr-tandem)
-		basic_machine=nsr-tandem
-		;;
-	op50n-* | op60c-*)
-		basic_machine=hppa1.1-oki
-		os=-proelf
-		;;
-	OSE68000 | ose68000)
-		basic_machine=m68000-ericsson
-		os=-ose
-		;;
-	os68k)
-		basic_machine=m68k-none
-		os=-os68k
-		;;
-	pa-hitachi)
-		basic_machine=hppa1.1-hitachi
-		os=-hiuxwe2
-		;;
-	paragon)
-		basic_machine=i860-intel
-		os=-osf
-		;;
-	pbd)
-		basic_machine=sparc-tti
-		;;
-	pbb)
-		basic_machine=m68k-tti
-		;;
-        pc532 | pc532-*)
-		basic_machine=ns32k-pc532
-		;;
-	pentium | p5 | k5 | k6 | nexgen)
-		basic_machine=i586-pc
-		;;
-	pentiumpro | p6 | 6x86 | athlon)
-		basic_machine=i686-pc
-		;;
-	pentiumii | pentium2)
-		basic_machine=i686-pc
-		;;
-	pentium-* | p5-* | k5-* | k6-* | nexgen-*)
-		basic_machine=i586-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	pentiumpro-* | p6-* | 6x86-* | athlon-*)
-		basic_machine=i686-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	pentiumii-* | pentium2-*)
-		basic_machine=i686-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	pn)
-		basic_machine=pn-gould
-		;;
-	power)	basic_machine=power-ibm
-		;;
-	ppc)	basic_machine=powerpc-unknown
-	        ;;
-	ppc-*)	basic_machine=powerpc-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	ppcle | powerpclittle | ppc-le | powerpc-little)
-		basic_machine=powerpcle-unknown
-	        ;;
-	ppcle-* | powerpclittle-*)
-		basic_machine=powerpcle-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	ppc64)	basic_machine=powerpc64-unknown
-	        ;;
-	ppc64-*) basic_machine=powerpc64-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	ppc64le | powerpc64little | ppc64-le | powerpc64-little)
-		basic_machine=powerpc64le-unknown
-	        ;;
-	ppc64le-* | powerpc64little-*)
-		basic_machine=powerpc64le-`echo $basic_machine | sed 's/^[^-]*-//'`
-		;;
-	ps2)
-		basic_machine=i386-ibm
-		;;
-	pw32)
-		basic_machine=i586-unknown
-		os=-pw32
-		;;
-	rom68k)
-		basic_machine=m68k-rom68k
-		os=-coff
-		;;
-	rm[46]00)
-		basic_machine=mips-siemens
-		;;
-	rtpc | rtpc-*)
-		basic_machine=romp-ibm
-		;;
-	sa29200)
-		basic_machine=a29k-amd
-		os=-udi
-		;;
-	sequent)
-		basic_machine=i386-sequent
-		;;
-	sh)
-		basic_machine=sh-hitachi
-		os=-hms
-		;;
-	sparclite-wrs)
-		basic_machine=sparclite-wrs
-		os=-vxworks
-		;;
-	sps7)
-		basic_machine=m68k-bull
-		os=-sysv2
-		;;
-	spur)
-		basic_machine=spur-unknown
-		;;
-	st2000)
-		basic_machine=m68k-tandem
-		;;
-	stratus)
-		basic_machine=i860-stratus
-		os=-sysv4
-		;;
-	sun2)
-		basic_machine=m68000-sun
-		;;
-	sun2os3)
-		basic_machine=m68000-sun
-		os=-sunos3
-		;;
-	sun2os4)
-		basic_machine=m68000-sun
-		os=-sunos4
-		;;
-	sun3os3)
-		basic_machine=m68k-sun
-		os=-sunos3
-		;;
-	sun3os4)
-		basic_machine=m68k-sun
-		os=-sunos4
-		;;
-	sun4os3)
-		basic_machine=sparc-sun
-		os=-sunos3
-		;;
-	sun4os4)
-		basic_machine=sparc-sun
-		os=-sunos4
-		;;
-	sun4sol2)
-		basic_machine=sparc-sun
-		os=-solaris2
-		;;
-	sun3 | sun3-*)
-		basic_machine=m68k-sun
-		;;
-	sun4)
-		basic_machine=sparc-sun
-		;;
-	sun386 | sun386i | roadrunner)
-		basic_machine=i386-sun
-		;;
-	sv1)
-		basic_machine=sv1-cray
-		os=-unicos
-		;;
-	symmetry)
-		basic_machine=i386-sequent
-		os=-dynix
-		;;
-	t3e)
-		basic_machine=t3e-cray
-		os=-unicos
-		;;
-	tic54x | c54x*)
-		basic_machine=tic54x-unknown
-		os=-coff
-		;;
-	tx39)
-		basic_machine=mipstx39-unknown
-		;;
-	tx39el)
-		basic_machine=mipstx39el-unknown
-		;;
-	tower | tower-32)
-		basic_machine=m68k-ncr
-		;;
-	udi29k)
-		basic_machine=a29k-amd
-		os=-udi
-		;;
-	ultra3)
-		basic_machine=a29k-nyu
-		os=-sym1
-		;;
-	v810 | necv810)
-		basic_machine=v810-nec
-		os=-none
-		;;
-	vaxv)
-		basic_machine=vax-dec
-		os=-sysv
-		;;
-	vms)
-		basic_machine=vax-dec
-		os=-vms
-		;;
-	vpp*|vx|vx-*)
-               basic_machine=f301-fujitsu
-               ;;
-	vxworks960)
-		basic_machine=i960-wrs
-		os=-vxworks
-		;;
-	vxworks68)
-		basic_machine=m68k-wrs
-		os=-vxworks
-		;;
-	vxworks29k)
-		basic_machine=a29k-wrs
-		os=-vxworks
-		;;
-	w65*)
-		basic_machine=w65-wdc
-		os=-none
-		;;
-	w89k-*)
-		basic_machine=hppa1.1-winbond
-		os=-proelf
-		;;
-	windows32)
-		basic_machine=i386-pc
-		os=-windows32-msvcrt
-		;;
-	xmp)
-		basic_machine=xmp-cray
-		os=-unicos
-		;;
-        xps | xps100)
-		basic_machine=xps100-honeywell
-		;;
-	z8k-*-coff)
-		basic_machine=z8k-unknown
-		os=-sim
-		;;
-	none)
-		basic_machine=none-none
-		os=-none
-		;;
-
-# Here we handle the default manufacturer of certain CPU types.  It is in
-# some cases the only manufacturer, in others, it is the most popular.
-	w89k)
-		basic_machine=hppa1.1-winbond
-		;;
-	op50n)
-		basic_machine=hppa1.1-oki
-		;;
-	op60c)
-		basic_machine=hppa1.1-oki
-		;;
-	mips)
-		if [ x$os = x-linux-gnu ]; then
-			basic_machine=mips-unknown
-		else
-			basic_machine=mips-mips
-		fi
-		;;
-	romp)
-		basic_machine=romp-ibm
-		;;
-	rs6000)
-		basic_machine=rs6000-ibm
-		;;
-	vax)
-		basic_machine=vax-dec
-		;;
-	pdp10)
-		# there are many clones, so DEC is not a safe bet
-		basic_machine=pdp10-unknown
-		;;
-	pdp11)
-		basic_machine=pdp11-dec
-		;;
-	we32k)
-		basic_machine=we32k-att
-		;;
-	sh3 | sh4 | sh3eb | sh4eb)
-		basic_machine=sh-unknown
-		;;
-	sparc | sparcv9 | sparcv9b)
-		basic_machine=sparc-sun
-		;;
-        cydra)
-		basic_machine=cydra-cydrome
-		;;
-	orion)
-		basic_machine=orion-highlevel
-		;;
-	orion105)
-		basic_machine=clipper-highlevel
-		;;
-	mac | mpw | mac-mpw)
-		basic_machine=m68k-apple
-		;;
-	pmac | pmac-mpw)
-		basic_machine=powerpc-apple
-		;;
-	c4x*)
-		basic_machine=c4x-none
-		os=-coff
-		;;
-	*-unknown)
-		# Make sure to match an already-canonicalized machine name.
-		;;
-	*)
-		echo Invalid configuration \`$1\': machine \`$basic_machine\' not recognized 1>&2
-		exit 1
-		;;
-esac
-
-# Here we canonicalize certain aliases for manufacturers.
-case $basic_machine in
-	*-digital*)
-		basic_machine=`echo $basic_machine | sed 's/digital.*/dec/'`
-		;;
-	*-commodore*)
-		basic_machine=`echo $basic_machine | sed 's/commodore.*/cbm/'`
-		;;
-	*)
-		;;
-esac
-
-# Decode manufacturer-specific aliases for certain operating systems.
-
-if [ x"$os" != x"" ]
-then
-case $os in
-        # First match some system type aliases
-        # that might get confused with valid system types.
-	# -solaris* is a basic system type, with this one exception.
-	-solaris1 | -solaris1.*)
-		os=`echo $os | sed -e 's|solaris1|sunos4|'`
-		;;
-	-solaris)
-		os=-solaris2
-		;;
-	-svr4*)
-		os=-sysv4
-		;;
-	-unixware*)
-		os=-sysv4.2uw
-		;;
-	-gnu/linux*)
-		os=`echo $os | sed -e 's|gnu/linux|linux-gnu|'`
-		;;
-	# First accept the basic system types.
-	# The portable systems comes first.
-	# Each alternative MUST END IN A *, to match a version number.
-	# -sysv* is not here because it comes later, after sysvr4.
-	-gnu* | -bsd* | -mach* | -minix* | -genix* | -ultrix* | -irix* \
-	      | -*vms* | -sco* | -esix* | -isc* | -aix* | -sunos | -sunos[34]*\
-	      | -hpux* | -unos* | -osf* | -luna* | -dgux* | -solaris* | -sym* \
-	      | -amigaos* | -amigados* | -msdos* | -newsos* | -unicos* | -aof* \
-	      | -aos* \
-	      | -nindy* | -vxsim* | -vxworks* | -ebmon* | -hms* | -mvs* \
-	      | -clix* | -riscos* | -uniplus* | -iris* | -rtu* | -xenix* \
-	      | -hiux* | -386bsd* | -netbsd* | -openbsd* | -freebsd* | -riscix* \
-	      | -lynxos* | -bosx* | -nextstep* | -cxux* | -aout* | -elf* | -oabi* \
-	      | -ptx* | -coff* | -ecoff* | -winnt* | -domain* | -vsta* \
-	      | -udi* | -eabi* | -lites* | -ieee* | -go32* | -aux* \
-	      | -chorusos* | -chorusrdb* \
-	      | -cygwin* | -pe* | -psos* | -moss* | -proelf* | -rtems* \
-	      | -mingw32* | -linux-gnu* | -uxpv* | -beos* | -mpeix* | -udk* \
-	      | -interix* | -uwin* | -rhapsody* | -darwin* | -opened* \
-	      | -openstep* | -oskit* | -conix* | -pw32* | -nonstopux* \
-	      | -storm-chaos* | -tops10* | -tenex* | -tops20* | -its* \
-	      | -os2* | -vos*)
-	# Remember, each alternative MUST END IN *, to match a version number.
-		;;
-	-qnx*)
-		case $basic_machine in
-		    x86-* | i*86-*)
-			;;
-		    *)
-			os=-nto$os
-			;;
-		esac
-		;;
-	-nto*)
-		os=-nto-qnx
-		;;
-	-sim | -es1800* | -hms* | -xray | -os68k* | -none* | -v88r* \
-	      | -windows* | -osx | -abug | -netware* | -os9* | -beos* \
-	      | -macos* | -mpw* | -magic* | -mmixware* | -mon960* | -lnews*)
-		;;
-	-mac*)
-		os=`echo $os | sed -e 's|mac|macos|'`
-		;;
-	-linux*)
-		os=`echo $os | sed -e 's|linux|linux-gnu|'`
-		;;
-	-sunos5*)
-		os=`echo $os | sed -e 's|sunos5|solaris2|'`
-		;;
-	-sunos6*)
-		os=`echo $os | sed -e 's|sunos6|solaris3|'`
-		;;
-	-opened*)
-		os=-openedition
-		;;
-	-wince*)
-		os=-wince
-		;;
-	-osfrose*)
-		os=-osfrose
-		;;
-	-osf*)
-		os=-osf
-		;;
-	-utek*)
-		os=-bsd
-		;;
-	-dynix*)
-		os=-bsd
-		;;
-	-acis*)
-		os=-aos
-		;;
-	-386bsd)
-		os=-bsd
-		;;
-	-ctix* | -uts*)
-		os=-sysv
-		;;
-	-ns2 )
-	        os=-nextstep2
-		;;
-	-nsk*)
-		os=-nsk
-		;;
-	# Preserve the version number of sinix5.
-	-sinix5.*)
-		os=`echo $os | sed -e 's|sinix|sysv|'`
-		;;
-	-sinix*)
-		os=-sysv4
-		;;
-	-triton*)
-		os=-sysv3
-		;;
-	-oss*)
-		os=-sysv3
-		;;
-	-svr4)
-		os=-sysv4
-		;;
-	-svr3)
-		os=-sysv3
-		;;
-	-sysvr4)
-		os=-sysv4
-		;;
-	# This must come after -sysvr4.
-	-sysv*)
-		;;
-	-ose*)
-		os=-ose
-		;;
-	-es1800*)
-		os=-ose
-		;;
-	-xenix)
-		os=-xenix
-		;;
-        -*mint | -mint[0-9]* | -*MiNT | -MiNT[0-9]*)
-	        os=-mint
-		;;
-	-none)
-		;;
-	*)
-		# Get rid of the `-' at the beginning of $os.
-		os=`echo $os | sed 's/[^-]*-//'`
-		echo Invalid configuration \`$1\': system \`$os\' not recognized 1>&2
-		exit 1
-		;;
-esac
-else
-
-# Here we handle the default operating systems that come with various machines.
-# The value should be what the vendor currently ships out the door with their
-# machine or put another way, the most popular os provided with the machine.
-
-# Note that if you're going to try to match "-MANUFACTURER" here (say,
-# "-sun"), then you have to tell the case statement up towards the top
-# that MANUFACTURER isn't an operating system.  Otherwise, code above
-# will signal an error saying that MANUFACTURER isn't an operating
-# system, and we'll never get to this point.
-
-case $basic_machine in
-	*-acorn)
-		os=-riscix1.2
-		;;
-	arm*-rebel)
-		os=-linux
-		;;
-	arm*-semi)
-		os=-aout
-		;;
-	pdp10-*)
-		os=-tops20
-		;;
-        pdp11-*)
-		os=-none
-		;;
-	*-dec | vax-*)
-		os=-ultrix4.2
-		;;
-	m68*-apollo)
-		os=-domain
-		;;
-	i386-sun)
-		os=-sunos4.0.2
-		;;
-	m68000-sun)
-		os=-sunos3
-		# This also exists in the configure program, but was not the
-		# default.
-		# os=-sunos4
-		;;
-	m68*-cisco)
-		os=-aout
-		;;
-	mips*-cisco)
-		os=-elf
-		;;
-	mips*-*)
-		os=-elf
-		;;
-	*-tti)	# must be before sparc entry or we get the wrong os.
-		os=-sysv3
-		;;
-	sparc-* | *-sun)
-		os=-sunos4.1.1
-		;;
-	*-be)
-		os=-beos
-		;;
-	*-ibm)
-		os=-aix
-		;;
-	*-wec)
-		os=-proelf
-		;;
-	*-winbond)
-		os=-proelf
-		;;
-	*-oki)
-		os=-proelf
-		;;
-	*-hp)
-		os=-hpux
-		;;
-	*-hitachi)
-		os=-hiux
-		;;
-	i860-* | *-att | *-ncr | *-altos | *-motorola | *-convergent)
-		os=-sysv
-		;;
-	*-cbm)
-		os=-amigaos
-		;;
-	*-dg)
-		os=-dgux
-		;;
-	*-dolphin)
-		os=-sysv3
-		;;
-	m68k-ccur)
-		os=-rtu
-		;;
-	m88k-omron*)
-		os=-luna
-		;;
-	*-next )
-		os=-nextstep
-		;;
-	*-sequent)
-		os=-ptx
-		;;
-	*-crds)
-		os=-unos
-		;;
-	*-ns)
-		os=-genix
-		;;
-	i370-*)
-		os=-mvs
-		;;
-	*-next)
-		os=-nextstep3
-		;;
-        *-gould)
-		os=-sysv
-		;;
-        *-highlevel)
-		os=-bsd
-		;;
-	*-encore)
-		os=-bsd
-		;;
-        *-sgi)
-		os=-irix
-		;;
-        *-siemens)
-		os=-sysv4
-		;;
-	*-masscomp)
-		os=-rtu
-		;;
-	f30[01]-fujitsu | f700-fujitsu)
-		os=-uxpv
-		;;
-	*-rom68k)
-		os=-coff
-		;;
-	*-*bug)
-		os=-coff
-		;;
-	*-apple)
-		os=-macos
-		;;
-	*-atari*)
-		os=-mint
-		;;
-	*)
-		os=-none
-		;;
-esac
-fi
-
-# Here we handle the case where we know the os, and the CPU type, but not the
-# manufacturer.  We pick the logical manufacturer.
-vendor=unknown
-case $basic_machine in
-	*-unknown)
-		case $os in
-			-riscix*)
-				vendor=acorn
-				;;
-			-sunos*)
-				vendor=sun
-				;;
-			-aix*)
-				vendor=ibm
-				;;
-			-beos*)
-				vendor=be
-				;;
-			-hpux*)
-				vendor=hp
-				;;
-			-mpeix*)
-				vendor=hp
-				;;
-			-hiux*)
-				vendor=hitachi
-				;;
-			-unos*)
-				vendor=crds
-				;;
-			-dgux*)
-				vendor=dg
-				;;
-			-luna*)
-				vendor=omron
-				;;
-			-genix*)
-				vendor=ns
-				;;
-			-mvs* | -opened*)
-				vendor=ibm
-				;;
-			-ptx*)
-				vendor=sequent
-				;;
-			-vxsim* | -vxworks*)
-				vendor=wrs
-				;;
-			-aux*)
-				vendor=apple
-				;;
-			-hms*)
-				vendor=hitachi
-				;;
-			-mpw* | -macos*)
-				vendor=apple
-				;;
-			-*mint | -mint[0-9]* | -*MiNT | -MiNT[0-9]*)
-				vendor=atari
-				;;
-			-vos*)
-				vendor=stratus
-				;;
-		esac
-		basic_machine=`echo $basic_machine | sed "s/unknown/$vendor/"`
-		;;
-esac
-
-echo $basic_machine$os
-exit 0
-
-# Local variables:
-# eval: (add-hook 'write-file-hooks 'time-stamp)
-# time-stamp-start: "timestamp='"
-# time-stamp-format: "%:y-%02m-%02d"
-# time-stamp-end: "'"
-# End:
diff -pruN 1:13.23.1~dfsg-2/main/editline/configure 1:16.1.1~dfsg-1/main/editline/configure
--- 1:13.23.1~dfsg-2/main/editline/configure	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/configure	1970-01-01 00:00:00.000000000 +0000
@@ -1,2462 +0,0 @@
-#! /bin/sh
-
-# Guess values for system-dependent variables and create Makefiles.
-# Generated automatically using autoconf version 2.13
-# Copyright (C) 1992, 93, 94, 95, 96 Free Software Foundation, Inc.
-#
-# This configure script is free software; the Free Software Foundation
-# gives unlimited permission to copy, distribute and modify it.
-
-# Defaults:
-ac_help=
-ac_default_prefix=/usr/local
-# Any additions from configure.in:
-ac_help="$ac_help
-  --disable-readline      Disable readline compatibility"
-ac_help="$ac_help
-  --enable-debug          Enable debugging code"
-
-# Initialize some variables set by options.
-# The variables have the same names as the options, with
-# dashes changed to underlines.
-build=NONE
-cache_file=./config.cache
-exec_prefix=NONE
-host=NONE
-no_create=
-nonopt=NONE
-no_recursion=
-prefix=NONE
-program_prefix=NONE
-program_suffix=NONE
-program_transform_name=s,x,x,
-silent=
-site=
-srcdir=
-target=NONE
-verbose=
-x_includes=NONE
-x_libraries=NONE
-bindir='${exec_prefix}/bin'
-sbindir='${exec_prefix}/sbin'
-libexecdir='${exec_prefix}/libexec'
-datadir='${prefix}/share'
-sysconfdir='${prefix}/etc'
-sharedstatedir='${prefix}/com'
-localstatedir='${prefix}/var'
-libdir='${exec_prefix}/lib'
-includedir='${prefix}/include'
-oldincludedir='/usr/include'
-infodir='${prefix}/info'
-mandir='${prefix}/man'
-
-# Initialize some other variables.
-subdirs=
-MFLAGS= MAKEFLAGS=
-SHELL=${CONFIG_SHELL-/bin/sh}
-# Maximum number of lines to put in a shell here document.
-ac_max_here_lines=12
-
-ac_prev=
-for ac_option
-do
-
-  # If the previous option needs an argument, assign it.
-  if test -n "$ac_prev"; then
-    eval "$ac_prev=\$ac_option"
-    ac_prev=
-    continue
-  fi
-
-  case "$ac_option" in
-  -*=*) ac_optarg=`echo "$ac_option" | sed 's/[-_a-zA-Z0-9]*=//'` ;;
-  *) ac_optarg= ;;
-  esac
-
-  # Accept the important Cygnus configure options, so we can diagnose typos.
-
-  case "$ac_option" in
-
-  -bindir | --bindir | --bindi | --bind | --bin | --bi)
-    ac_prev=bindir ;;
-  -bindir=* | --bindir=* | --bindi=* | --bind=* | --bin=* | --bi=*)
-    bindir="$ac_optarg" ;;
-
-  -build | --build | --buil | --bui | --bu)
-    ac_prev=build ;;
-  -build=* | --build=* | --buil=* | --bui=* | --bu=*)
-    build="$ac_optarg" ;;
-
-  -cache-file | --cache-file | --cache-fil | --cache-fi \
-  | --cache-f | --cache- | --cache | --cach | --cac | --ca | --c)
-    ac_prev=cache_file ;;
-  -cache-file=* | --cache-file=* | --cache-fil=* | --cache-fi=* \
-  | --cache-f=* | --cache-=* | --cache=* | --cach=* | --cac=* | --ca=* | --c=*)
-    cache_file="$ac_optarg" ;;
-
-  -datadir | --datadir | --datadi | --datad | --data | --dat | --da)
-    ac_prev=datadir ;;
-  -datadir=* | --datadir=* | --datadi=* | --datad=* | --data=* | --dat=* \
-  | --da=*)
-    datadir="$ac_optarg" ;;
-
-  -disable-* | --disable-*)
-    ac_feature=`echo $ac_option|sed -e 's/-*disable-//'`
-    # Reject names that are not valid shell variable names.
-    if test -n "`echo $ac_feature| sed 's/[-a-zA-Z0-9_]//g'`"; then
-      { echo "configure: error: $ac_feature: invalid feature name" 1>&2; exit 1; }
-    fi
-    ac_feature=`echo $ac_feature| sed 's/-/_/g'`
-    eval "enable_${ac_feature}=no" ;;
-
-  -enable-* | --enable-*)
-    ac_feature=`echo $ac_option|sed -e 's/-*enable-//' -e 's/=.*//'`
-    # Reject names that are not valid shell variable names.
-    if test -n "`echo $ac_feature| sed 's/[-_a-zA-Z0-9]//g'`"; then
-      { echo "configure: error: $ac_feature: invalid feature name" 1>&2; exit 1; }
-    fi
-    ac_feature=`echo $ac_feature| sed 's/-/_/g'`
-    case "$ac_option" in
-      *=*) ;;
-      *) ac_optarg=yes ;;
-    esac
-    eval "enable_${ac_feature}='$ac_optarg'" ;;
-
-  -exec-prefix | --exec_prefix | --exec-prefix | --exec-prefi \
-  | --exec-pref | --exec-pre | --exec-pr | --exec-p | --exec- \
-  | --exec | --exe | --ex)
-    ac_prev=exec_prefix ;;
-  -exec-prefix=* | --exec_prefix=* | --exec-prefix=* | --exec-prefi=* \
-  | --exec-pref=* | --exec-pre=* | --exec-pr=* | --exec-p=* | --exec-=* \
-  | --exec=* | --exe=* | --ex=*)
-    exec_prefix="$ac_optarg" ;;
-
-  -gas | --gas | --ga | --g)
-    # Obsolete; use --with-gas.
-    with_gas=yes ;;
-
-  -help | --help | --hel | --he)
-    # Omit some internal or obsolete options to make the list less imposing.
-    # This message is too long to be a string in the A/UX 3.1 sh.
-    cat << EOF
-Usage: configure [options] [host]
-Options: [defaults in brackets after descriptions]
-Configuration:
-  --cache-file=FILE       cache test results in FILE
-  --help                  print this message
-  --no-create             do not create output files
-  --quiet, --silent       do not print \`checking...' messages
-  --version               print the version of autoconf that created configure
-Directory and file names:
-  --prefix=PREFIX         install architecture-independent files in PREFIX
-                          [$ac_default_prefix]
-  --exec-prefix=EPREFIX   install architecture-dependent files in EPREFIX
-                          [same as prefix]
-  --bindir=DIR            user executables in DIR [EPREFIX/bin]
-  --sbindir=DIR           system admin executables in DIR [EPREFIX/sbin]
-  --libexecdir=DIR        program executables in DIR [EPREFIX/libexec]
-  --datadir=DIR           read-only architecture-independent data in DIR
-                          [PREFIX/share]
-  --sysconfdir=DIR        read-only single-machine data in DIR [PREFIX/etc]
-  --sharedstatedir=DIR    modifiable architecture-independent data in DIR
-                          [PREFIX/com]
-  --localstatedir=DIR     modifiable single-machine data in DIR [PREFIX/var]
-  --libdir=DIR            object code libraries in DIR [EPREFIX/lib]
-  --includedir=DIR        C header files in DIR [PREFIX/include]
-  --oldincludedir=DIR     C header files for non-gcc in DIR [/usr/include]
-  --infodir=DIR           info documentation in DIR [PREFIX/info]
-  --mandir=DIR            man documentation in DIR [PREFIX/man]
-  --srcdir=DIR            find the sources in DIR [configure dir or ..]
-  --program-prefix=PREFIX prepend PREFIX to installed program names
-  --program-suffix=SUFFIX append SUFFIX to installed program names
-  --program-transform-name=PROGRAM
-                          run sed PROGRAM on installed program names
-EOF
-    cat << EOF
-Host type:
-  --build=BUILD           configure for building on BUILD [BUILD=HOST]
-  --host=HOST             configure for HOST [guessed]
-  --target=TARGET         configure for TARGET [TARGET=HOST]
-Features and packages:
-  --disable-FEATURE       do not include FEATURE (same as --enable-FEATURE=no)
-  --enable-FEATURE[=ARG]  include FEATURE [ARG=yes]
-  --with-PACKAGE[=ARG]    use PACKAGE [ARG=yes]
-  --without-PACKAGE       do not use PACKAGE (same as --with-PACKAGE=no)
-  --x-includes=DIR        X include files are in DIR
-  --x-libraries=DIR       X library files are in DIR
-EOF
-    if test -n "$ac_help"; then
-      echo "--enable and --with options recognized:$ac_help"
-    fi
-    exit 0 ;;
-
-  -host | --host | --hos | --ho)
-    ac_prev=host ;;
-  -host=* | --host=* | --hos=* | --ho=*)
-    host="$ac_optarg" ;;
-
-  -includedir | --includedir | --includedi | --included | --include \
-  | --includ | --inclu | --incl | --inc)
-    ac_prev=includedir ;;
-  -includedir=* | --includedir=* | --includedi=* | --included=* | --include=* \
-  | --includ=* | --inclu=* | --incl=* | --inc=*)
-    includedir="$ac_optarg" ;;
-
-  -infodir | --infodir | --infodi | --infod | --info | --inf)
-    ac_prev=infodir ;;
-  -infodir=* | --infodir=* | --infodi=* | --infod=* | --info=* | --inf=*)
-    infodir="$ac_optarg" ;;
-
-  -libdir | --libdir | --libdi | --libd)
-    ac_prev=libdir ;;
-  -libdir=* | --libdir=* | --libdi=* | --libd=*)
-    libdir="$ac_optarg" ;;
-
-  -libexecdir | --libexecdir | --libexecdi | --libexecd | --libexec \
-  | --libexe | --libex | --libe)
-    ac_prev=libexecdir ;;
-  -libexecdir=* | --libexecdir=* | --libexecdi=* | --libexecd=* | --libexec=* \
-  | --libexe=* | --libex=* | --libe=*)
-    libexecdir="$ac_optarg" ;;
-
-  -localstatedir | --localstatedir | --localstatedi | --localstated \
-  | --localstate | --localstat | --localsta | --localst \
-  | --locals | --local | --loca | --loc | --lo)
-    ac_prev=localstatedir ;;
-  -localstatedir=* | --localstatedir=* | --localstatedi=* | --localstated=* \
-  | --localstate=* | --localstat=* | --localsta=* | --localst=* \
-  | --locals=* | --local=* | --loca=* | --loc=* | --lo=*)
-    localstatedir="$ac_optarg" ;;
-
-  -mandir | --mandir | --mandi | --mand | --man | --ma | --m)
-    ac_prev=mandir ;;
-  -mandir=* | --mandir=* | --mandi=* | --mand=* | --man=* | --ma=* | --m=*)
-    mandir="$ac_optarg" ;;
-
-  -nfp | --nfp | --nf)
-    # Obsolete; use --without-fp.
-    with_fp=no ;;
-
-  -no-create | --no-create | --no-creat | --no-crea | --no-cre \
-  | --no-cr | --no-c)
-    no_create=yes ;;
-
-  -no-recursion | --no-recursion | --no-recursio | --no-recursi \
-  | --no-recurs | --no-recur | --no-recu | --no-rec | --no-re | --no-r)
-    no_recursion=yes ;;
-
-  -oldincludedir | --oldincludedir | --oldincludedi | --oldincluded \
-  | --oldinclude | --oldinclud | --oldinclu | --oldincl | --oldinc \
-  | --oldin | --oldi | --old | --ol | --o)
-    ac_prev=oldincludedir ;;
-  -oldincludedir=* | --oldincludedir=* | --oldincludedi=* | --oldincluded=* \
-  | --oldinclude=* | --oldinclud=* | --oldinclu=* | --oldincl=* | --oldinc=* \
-  | --oldin=* | --oldi=* | --old=* | --ol=* | --o=*)
-    oldincludedir="$ac_optarg" ;;
-
-  -prefix | --prefix | --prefi | --pref | --pre | --pr | --p)
-    ac_prev=prefix ;;
-  -prefix=* | --prefix=* | --prefi=* | --pref=* | --pre=* | --pr=* | --p=*)
-    prefix="$ac_optarg" ;;
-
-  -program-prefix | --program-prefix | --program-prefi | --program-pref \
-  | --program-pre | --program-pr | --program-p)
-    ac_prev=program_prefix ;;
-  -program-prefix=* | --program-prefix=* | --program-prefi=* \
-  | --program-pref=* | --program-pre=* | --program-pr=* | --program-p=*)
-    program_prefix="$ac_optarg" ;;
-
-  -program-suffix | --program-suffix | --program-suffi | --program-suff \
-  | --program-suf | --program-su | --program-s)
-    ac_prev=program_suffix ;;
-  -program-suffix=* | --program-suffix=* | --program-suffi=* \
-  | --program-suff=* | --program-suf=* | --program-su=* | --program-s=*)
-    program_suffix="$ac_optarg" ;;
-
-  -program-transform-name | --program-transform-name \
-  | --program-transform-nam | --program-transform-na \
-  | --program-transform-n | --program-transform- \
-  | --program-transform | --program-transfor \
-  | --program-transfo | --program-transf \
-  | --program-trans | --program-tran \
-  | --progr-tra | --program-tr | --program-t)
-    ac_prev=program_transform_name ;;
-  -program-transform-name=* | --program-transform-name=* \
-  | --program-transform-nam=* | --program-transform-na=* \
-  | --program-transform-n=* | --program-transform-=* \
-  | --program-transform=* | --program-transfor=* \
-  | --program-transfo=* | --program-transf=* \
-  | --program-trans=* | --program-tran=* \
-  | --progr-tra=* | --program-tr=* | --program-t=*)
-    program_transform_name="$ac_optarg" ;;
-
-  -q | -quiet | --quiet | --quie | --qui | --qu | --q \
-  | -silent | --silent | --silen | --sile | --sil)
-    silent=yes ;;
-
-  -sbindir | --sbindir | --sbindi | --sbind | --sbin | --sbi | --sb)
-    ac_prev=sbindir ;;
-  -sbindir=* | --sbindir=* | --sbindi=* | --sbind=* | --sbin=* \
-  | --sbi=* | --sb=*)
-    sbindir="$ac_optarg" ;;
-
-  -sharedstatedir | --sharedstatedir | --sharedstatedi \
-  | --sharedstated | --sharedstate | --sharedstat | --sharedsta \
-  | --sharedst | --shareds | --shared | --share | --shar \
-  | --sha | --sh)
-    ac_prev=sharedstatedir ;;
-  -sharedstatedir=* | --sharedstatedir=* | --sharedstatedi=* \
-  | --sharedstated=* | --sharedstate=* | --sharedstat=* | --sharedsta=* \
-  | --sharedst=* | --shareds=* | --shared=* | --share=* | --shar=* \
-  | --sha=* | --sh=*)
-    sharedstatedir="$ac_optarg" ;;
-
-  -site | --site | --sit)
-    ac_prev=site ;;
-  -site=* | --site=* | --sit=*)
-    site="$ac_optarg" ;;
-
-  -srcdir | --srcdir | --srcdi | --srcd | --src | --sr)
-    ac_prev=srcdir ;;
-  -srcdir=* | --srcdir=* | --srcdi=* | --srcd=* | --src=* | --sr=*)
-    srcdir="$ac_optarg" ;;
-
-  -sysconfdir | --sysconfdir | --sysconfdi | --sysconfd | --sysconf \
-  | --syscon | --sysco | --sysc | --sys | --sy)
-    ac_prev=sysconfdir ;;
-  -sysconfdir=* | --sysconfdir=* | --sysconfdi=* | --sysconfd=* | --sysconf=* \
-  | --syscon=* | --sysco=* | --sysc=* | --sys=* | --sy=*)
-    sysconfdir="$ac_optarg" ;;
-
-  -target | --target | --targe | --targ | --tar | --ta | --t)
-    ac_prev=target ;;
-  -target=* | --target=* | --targe=* | --targ=* | --tar=* | --ta=* | --t=*)
-    target="$ac_optarg" ;;
-
-  -v | -verbose | --verbose | --verbos | --verbo | --verb)
-    verbose=yes ;;
-
-  -version | --version | --versio | --versi | --vers)
-    echo "configure generated by autoconf version 2.13"
-    exit 0 ;;
-
-  -with-* | --with-*)
-    ac_package=`echo $ac_option|sed -e 's/-*with-//' -e 's/=.*//'`
-    # Reject names that are not valid shell variable names.
-    if test -n "`echo $ac_package| sed 's/[-_a-zA-Z0-9]//g'`"; then
-      { echo "configure: error: $ac_package: invalid package name" 1>&2; exit 1; }
-    fi
-    ac_package=`echo $ac_package| sed 's/-/_/g'`
-    case "$ac_option" in
-      *=*) ;;
-      *) ac_optarg=yes ;;
-    esac
-    eval "with_${ac_package}='$ac_optarg'" ;;
-
-  -without-* | --without-*)
-    ac_package=`echo $ac_option|sed -e 's/-*without-//'`
-    # Reject names that are not valid shell variable names.
-    if test -n "`echo $ac_package| sed 's/[-a-zA-Z0-9_]//g'`"; then
-      { echo "configure: error: $ac_package: invalid package name" 1>&2; exit 1; }
-    fi
-    ac_package=`echo $ac_package| sed 's/-/_/g'`
-    eval "with_${ac_package}=no" ;;
-
-  --x)
-    # Obsolete; use --with-x.
-    with_x=yes ;;
-
-  -x-includes | --x-includes | --x-include | --x-includ | --x-inclu \
-  | --x-incl | --x-inc | --x-in | --x-i)
-    ac_prev=x_includes ;;
-  -x-includes=* | --x-includes=* | --x-include=* | --x-includ=* | --x-inclu=* \
-  | --x-incl=* | --x-inc=* | --x-in=* | --x-i=*)
-    x_includes="$ac_optarg" ;;
-
-  -x-libraries | --x-libraries | --x-librarie | --x-librari \
-  | --x-librar | --x-libra | --x-libr | --x-lib | --x-li | --x-l)
-    ac_prev=x_libraries ;;
-  -x-libraries=* | --x-libraries=* | --x-librarie=* | --x-librari=* \
-  | --x-librar=* | --x-libra=* | --x-libr=* | --x-lib=* | --x-li=* | --x-l=*)
-    x_libraries="$ac_optarg" ;;
-
-  -*) { echo "configure: error: $ac_option: invalid option; use --help to show usage" 1>&2; exit 1; }
-    ;;
-
-  *)
-    if test -n "`echo $ac_option| sed 's/[-a-z0-9.]//g'`"; then
-      echo "configure: warning: $ac_option: invalid host type" 1>&2
-    fi
-    if test "x$nonopt" != xNONE; then
-      { echo "configure: error: can only configure for one host and one target at a time" 1>&2; exit 1; }
-    fi
-    nonopt="$ac_option"
-    ;;
-
-  esac
-done
-
-if test -n "$ac_prev"; then
-  { echo "configure: error: missing argument to --`echo $ac_prev | sed 's/_/-/g'`" 1>&2; exit 1; }
-fi
-
-trap 'rm -fr conftest* confdefs* core core.* *.core $ac_clean_files; exit 1' 1 2 15
-
-# File descriptor usage:
-# 0 standard input
-# 1 file creation
-# 2 errors and warnings
-# 3 some systems may open it to /dev/tty
-# 4 used on the Kubota Titan
-# 6 checking for... messages and results
-# 5 compiler messages saved in config.log
-if test "$silent" = yes; then
-  exec 6>/dev/null
-else
-  exec 6>&1
-fi
-exec 5>./config.log
-
-echo "\
-This file contains any messages produced by compilers while
-running configure, to aid debugging if configure makes a mistake.
-" 1>&5
-
-# Strip out --no-create and --no-recursion so they do not pile up.
-# Also quote any args containing shell metacharacters.
-ac_configure_args=
-for ac_arg
-do
-  case "$ac_arg" in
-  -no-create | --no-create | --no-creat | --no-crea | --no-cre \
-  | --no-cr | --no-c) ;;
-  -no-recursion | --no-recursion | --no-recursio | --no-recursi \
-  | --no-recurs | --no-recur | --no-recu | --no-rec | --no-re | --no-r) ;;
-  *" "*|*"	"*|*[\[\]\~\#\$\^\&\*\(\)\{\}\\\|\;\<\>\?]*)
-  ac_configure_args="$ac_configure_args '$ac_arg'" ;;
-  *) ac_configure_args="$ac_configure_args $ac_arg" ;;
-  esac
-done
-
-# NLS nuisances.
-# Only set these to C if already set.  These must not be set unconditionally
-# because not all systems understand e.g. LANG=C (notably SCO).
-# Fixing LC_MESSAGES prevents Solaris sh from translating var values in `set'!
-# Non-C LC_CTYPE values break the ctype check.
-if test "${LANG+set}"   = set; then LANG=C;   export LANG;   fi
-if test "${LC_ALL+set}" = set; then LC_ALL=C; export LC_ALL; fi
-if test "${LC_MESSAGES+set}" = set; then LC_MESSAGES=C; export LC_MESSAGES; fi
-if test "${LC_CTYPE+set}"    = set; then LC_CTYPE=C;    export LC_CTYPE;    fi
-
-# confdefs.h avoids OS command line length limits that DEFS can exceed.
-rm -rf conftest* confdefs.h
-# AIX cpp loses on an empty file, so make sure it contains at least a newline.
-echo > confdefs.h
-
-# A filename unique to this package, relative to the directory that
-# configure is in, which we can look for to find out if srcdir is correct.
-ac_unique_file=Makefile.in
-
-# Find the source files, if location was not specified.
-if test -z "$srcdir"; then
-  ac_srcdir_defaulted=yes
-  # Try the directory containing this script, then its parent.
-  ac_prog=$0
-  ac_confdir=`echo $ac_prog|sed 's%/[^/][^/]*$%%'`
-  test "x$ac_confdir" = "x$ac_prog" && ac_confdir=.
-  srcdir=$ac_confdir
-  if test ! -r $srcdir/$ac_unique_file; then
-    srcdir=..
-  fi
-else
-  ac_srcdir_defaulted=no
-fi
-if test ! -r $srcdir/$ac_unique_file; then
-  if test "$ac_srcdir_defaulted" = yes; then
-    { echo "configure: error: can not find sources in $ac_confdir or .." 1>&2; exit 1; }
-  else
-    { echo "configure: error: can not find sources in $srcdir" 1>&2; exit 1; }
-  fi
-fi
-srcdir=`echo "${srcdir}" | sed 's%\([^/]\)/*$%\1%'`
-
-# Prefer explicitly selected file to automatically selected ones.
-if test -z "$CONFIG_SITE"; then
-  if test "x$prefix" != xNONE; then
-    CONFIG_SITE="$prefix/share/config.site $prefix/etc/config.site"
-  else
-    CONFIG_SITE="$ac_default_prefix/share/config.site $ac_default_prefix/etc/config.site"
-  fi
-fi
-for ac_site_file in $CONFIG_SITE; do
-  if test -r "$ac_site_file"; then
-    echo "loading site script $ac_site_file"
-    . "$ac_site_file"
-  fi
-done
-
-if test -r "$cache_file"; then
-  echo "loading cache $cache_file"
-  . $cache_file
-else
-  echo "creating cache $cache_file"
-  > $cache_file
-fi
-
-ac_ext=c
-# CFLAGS is not in ac_cpp because -g, -O, etc. are not valid cpp options.
-ac_cpp='$CPP $CPPFLAGS'
-ac_compile='${CC-cc} -c $CFLAGS $CPPFLAGS conftest.$ac_ext 1>&5'
-ac_link='${CC-cc} -o conftest${ac_exeext} $CFLAGS $CPPFLAGS $LDFLAGS conftest.$ac_ext $LIBS 1>&5'
-cross_compiling=$ac_cv_prog_cc_cross
-
-ac_exeext=
-ac_objext=o
-if (echo "testing\c"; echo 1,2,3) | grep c >/dev/null; then
-  # Stardent Vistra SVR4 grep lacks -e, says ghazi@caip.rutgers.edu.
-  if (echo -n testing; echo 1,2,3) | sed s/-n/xn/ | grep xn >/dev/null; then
-    ac_n= ac_c='
-' ac_t='	'
-  else
-    ac_n=-n ac_c= ac_t=
-  fi
-else
-  ac_n= ac_c='\c' ac_t=
-fi
-
-
-
-CFLAGS=$CFLAGS
-# Extract the first word of "gcc", so it can be a program name with args.
-set dummy gcc; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:534: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_CC'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  if test -n "$CC"; then
-  ac_cv_prog_CC="$CC" # Let the user override the test.
-else
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      ac_cv_prog_CC="gcc"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-fi
-fi
-CC="$ac_cv_prog_CC"
-if test -n "$CC"; then
-  echo "$ac_t""$CC" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
-
-if test -z "$CC"; then
-  # Extract the first word of "cc", so it can be a program name with args.
-set dummy cc; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:564: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_CC'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  if test -n "$CC"; then
-  ac_cv_prog_CC="$CC" # Let the user override the test.
-else
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_prog_rejected=no
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      if test "$ac_dir/$ac_word" = "/usr/ucb/cc"; then
-        ac_prog_rejected=yes
-	continue
-      fi
-      ac_cv_prog_CC="cc"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-if test $ac_prog_rejected = yes; then
-  # We found a bogon in the path, so make sure we never use it.
-  set dummy $ac_cv_prog_CC
-  shift
-  if test $# -gt 0; then
-    # We chose a different compiler from the bogus one.
-    # However, it has the same basename, so the bogon will be chosen
-    # first if we set CC to just the basename; use the full file name.
-    shift
-    set dummy "$ac_dir/$ac_word" "$@"
-    shift
-    ac_cv_prog_CC="$@"
-  fi
-fi
-fi
-fi
-CC="$ac_cv_prog_CC"
-if test -n "$CC"; then
-  echo "$ac_t""$CC" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
-
-  if test -z "$CC"; then
-    case "`uname -s`" in
-    *win32* | *WIN32*)
-      # Extract the first word of "cl", so it can be a program name with args.
-set dummy cl; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:615: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_CC'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  if test -n "$CC"; then
-  ac_cv_prog_CC="$CC" # Let the user override the test.
-else
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      ac_cv_prog_CC="cl"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-fi
-fi
-CC="$ac_cv_prog_CC"
-if test -n "$CC"; then
-  echo "$ac_t""$CC" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
- ;;
-    esac
-  fi
-  test -z "$CC" && { echo "configure: error: no acceptable cc found in \$PATH" 1>&2; exit 1; }
-fi
-
-echo $ac_n "checking whether the C compiler ($CC $CFLAGS $LDFLAGS) works""... $ac_c" 1>&6
-echo "configure:647: checking whether the C compiler ($CC $CFLAGS $LDFLAGS) works" >&5
-
-ac_ext=c
-# CFLAGS is not in ac_cpp because -g, -O, etc. are not valid cpp options.
-ac_cpp='$CPP $CPPFLAGS'
-ac_compile='${CC-cc} -c $CFLAGS $CPPFLAGS conftest.$ac_ext 1>&5'
-ac_link='${CC-cc} -o conftest${ac_exeext} $CFLAGS $CPPFLAGS $LDFLAGS conftest.$ac_ext $LIBS 1>&5'
-cross_compiling=$ac_cv_prog_cc_cross
-
-cat > conftest.$ac_ext << EOF
-
-#line 658 "configure"
-#include "confdefs.h"
-
-main(){return(0);}
-EOF
-if { (eval echo configure:663: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  ac_cv_prog_cc_works=yes
-  # If we can't run a trivial program, we are probably using a cross compiler.
-  if (./conftest; exit) 2>/dev/null; then
-    ac_cv_prog_cc_cross=no
-  else
-    ac_cv_prog_cc_cross=yes
-  fi
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  ac_cv_prog_cc_works=no
-fi
-rm -fr conftest*
-ac_ext=c
-# CFLAGS is not in ac_cpp because -g, -O, etc. are not valid cpp options.
-ac_cpp='$CPP $CPPFLAGS'
-ac_compile='${CC-cc} -c $CFLAGS $CPPFLAGS conftest.$ac_ext 1>&5'
-ac_link='${CC-cc} -o conftest${ac_exeext} $CFLAGS $CPPFLAGS $LDFLAGS conftest.$ac_ext $LIBS 1>&5'
-cross_compiling=$ac_cv_prog_cc_cross
-
-echo "$ac_t""$ac_cv_prog_cc_works" 1>&6
-if test $ac_cv_prog_cc_works = no; then
-  { echo "configure: error: installation or configuration problem: C compiler cannot create executables." 1>&2; exit 1; }
-fi
-echo $ac_n "checking whether the C compiler ($CC $CFLAGS $LDFLAGS) is a cross-compiler""... $ac_c" 1>&6
-echo "configure:689: checking whether the C compiler ($CC $CFLAGS $LDFLAGS) is a cross-compiler" >&5
-echo "$ac_t""$ac_cv_prog_cc_cross" 1>&6
-cross_compiling=$ac_cv_prog_cc_cross
-
-echo $ac_n "checking whether we are using GNU C""... $ac_c" 1>&6
-echo "configure:694: checking whether we are using GNU C" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_gcc'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.c <<EOF
-#ifdef __GNUC__
-  yes;
-#endif
-EOF
-if { ac_try='${CC-cc} -E conftest.c'; { (eval echo configure:703: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }; } | egrep yes >/dev/null 2>&1; then
-  ac_cv_prog_gcc=yes
-else
-  ac_cv_prog_gcc=no
-fi
-fi
-
-echo "$ac_t""$ac_cv_prog_gcc" 1>&6
-
-if test $ac_cv_prog_gcc = yes; then
-  GCC=yes
-else
-  GCC=
-fi
-
-ac_test_CFLAGS="${CFLAGS+set}"
-ac_save_CFLAGS="$CFLAGS"
-CFLAGS=
-echo $ac_n "checking whether ${CC-cc} accepts -g""... $ac_c" 1>&6
-echo "configure:722: checking whether ${CC-cc} accepts -g" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_cc_g'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  echo 'void f(){}' > conftest.c
-if test -z "`${CC-cc} -g -c conftest.c 2>&1`"; then
-  ac_cv_prog_cc_g=yes
-else
-  ac_cv_prog_cc_g=no
-fi
-rm -f conftest*
-
-fi
-
-echo "$ac_t""$ac_cv_prog_cc_g" 1>&6
-if test "$ac_test_CFLAGS" = set; then
-  CFLAGS="$ac_save_CFLAGS"
-elif test $ac_cv_prog_cc_g = yes; then
-  if test "$GCC" = yes; then
-    CFLAGS="-g -O2"
-  else
-    CFLAGS="-g"
-  fi
-else
-  if test "$GCC" = yes; then
-    CFLAGS="-O2"
-  else
-    CFLAGS=
-  fi
-fi
-
-if test "x$CFLAGS" = "x" ; then
-  no_CFLAGS="yes"
-fi
-if test "x$no_CFLAGS" = "xyes" -a "x$GCC" = "xyes" ; then
-	CFLAGS="-Wall -pipe -g3"
-fi
-A_CFLAGS=""
-
-S_CFLAGS="-fPIC -DPIC"
-
-echo $ac_n "checking how to run the C preprocessor""... $ac_c" 1>&6
-echo "configure:764: checking how to run the C preprocessor" >&5
-# On Suns, sometimes $CPP names a directory.
-if test -n "$CPP" && test -d "$CPP"; then
-  CPP=
-fi
-if test -z "$CPP"; then
-if eval "test \"`echo '$''{'ac_cv_prog_CPP'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-    # This must be in double quotes, not single quotes, because CPP may get
-  # substituted into the Makefile and "${CC-cc}" will confuse make.
-  CPP="${CC-cc} -E"
-  # On the NeXT, cc -E runs the code through the compiler's parser,
-  # not just through cpp.
-  cat > conftest.$ac_ext <<EOF
-#line 779 "configure"
-#include "confdefs.h"
-#include <assert.h>
-Syntax Error
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:785: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  :
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  CPP="${CC-cc} -E -traditional-cpp"
-  cat > conftest.$ac_ext <<EOF
-#line 796 "configure"
-#include "confdefs.h"
-#include <assert.h>
-Syntax Error
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:802: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  :
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  CPP="${CC-cc} -nologo -E"
-  cat > conftest.$ac_ext <<EOF
-#line 813 "configure"
-#include "confdefs.h"
-#include <assert.h>
-Syntax Error
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:819: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  :
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  CPP=/lib/cpp
-fi
-rm -f conftest*
-fi
-rm -f conftest*
-fi
-rm -f conftest*
-  ac_cv_prog_CPP="$CPP"
-fi
-  CPP="$ac_cv_prog_CPP"
-else
-  ac_cv_prog_CPP="$CPP"
-fi
-echo "$ac_t""$CPP" 1>&6
-
-for ac_prog in mawk gawk nawk awk
-do
-# Extract the first word of "$ac_prog", so it can be a program name with args.
-set dummy $ac_prog; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:848: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_AWK'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  if test -n "$AWK"; then
-  ac_cv_prog_AWK="$AWK" # Let the user override the test.
-else
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      ac_cv_prog_AWK="$ac_prog"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-fi
-fi
-AWK="$ac_cv_prog_AWK"
-if test -n "$AWK"; then
-  echo "$ac_t""$AWK" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
-
-test -n "$AWK" && break
-done
-
-
-ac_aux_dir=
-for ac_dir in $srcdir $srcdir/.. $srcdir/../..; do
-  if test -f $ac_dir/install-sh; then
-    ac_aux_dir=$ac_dir
-    ac_install_sh="$ac_aux_dir/install-sh -c"
-    break
-  elif test -f $ac_dir/install.sh; then
-    ac_aux_dir=$ac_dir
-    ac_install_sh="$ac_aux_dir/install.sh -c"
-    break
-  fi
-done
-if test -z "$ac_aux_dir"; then
-  { echo "configure: error: can not find install-sh or install.sh in $srcdir $srcdir/.. $srcdir/../.." 1>&2; exit 1; }
-fi
-ac_config_guess=$ac_aux_dir/config.guess
-ac_config_sub=$ac_aux_dir/config.sub
-ac_configure=$ac_aux_dir/configure # This should be Cygnus configure.
-
-
-# Make sure we can run config.sub.
-if ${CONFIG_SHELL-/bin/sh} $ac_config_sub sun4 >/dev/null 2>&1; then :
-else { echo "configure: error: can not run $ac_config_sub" 1>&2; exit 1; }
-fi
-
-echo $ac_n "checking host system type""... $ac_c" 1>&6
-echo "configure:904: checking host system type" >&5
-
-host_alias=$host
-case "$host_alias" in
-NONE)
-  case $nonopt in
-  NONE)
-    if host_alias=`${CONFIG_SHELL-/bin/sh} $ac_config_guess`; then :
-    else { echo "configure: error: can not guess host type; you must specify one" 1>&2; exit 1; }
-    fi ;;
-  *) host_alias=$nonopt ;;
-  esac ;;
-esac
-
-host=`${CONFIG_SHELL-/bin/sh} $ac_config_sub $host_alias`
-host_cpu=`echo $host | sed 's/^\([^-]*\)-\([^-]*\)-\(.*\)$/\1/'`
-host_vendor=`echo $host | sed 's/^\([^-]*\)-\([^-]*\)-\(.*\)$/\2/'`
-host_os=`echo $host | sed 's/^\([^-]*\)-\([^-]*\)-\(.*\)$/\3/'`
-echo "$ac_t""$host" 1>&6
-
-case "${host}" in
-  *-*-darwin*)
-	CFLAGS="$CFLAGS -fno-common -no-cpp-precomp"
-	ABI="macho"
-	;;
-  *-*-freebsd*)
-	ABI="elf"
-	;;
-  *-*-linux* | *cygwin*)
-	if echo ${host} | grep -q cygwin ; then \
-		echo "cygwin detected"; \
-		S_CFLAGS=""; \
-		echo "/* cygdef.h. Generated automatically by configure. */
-#ifndef _CYGDEF_H_
-#define _CYGDEF_H_ 1
-#include <sys/ioctl.h>
-#define __linux__ 1
-
-
-typedef void (*sig_t)(int);
-
-
-#endif /* _CYGDEF_H_ */" > cygdef.h; \
-	echo "
- #define CYGWIN 1
-" > confdefs.h; \
-	fi
-	ABI="elf"
-	;;
-  *-*-netbsd*)
-	echo $ac_n "checking ABI""... $ac_c" 1>&6
-echo "configure:955: checking ABI" >&5
-	cat > conftest.$ac_ext <<EOF
-#line 957 "configure"
-#include "confdefs.h"
-#ifdef __ELF__
-  yes
-#endif
-
-EOF
-if (eval "$ac_cpp conftest.$ac_ext") 2>&5 |
-  egrep "yes" >/dev/null 2>&1; then
-  rm -rf conftest*
-  ABI="elf"
-else
-  rm -rf conftest*
-  ABI="aout"
-fi
-rm -f conftest*
-
-	echo "$ac_t""$ABI" 1>&6
-	;;
-  *-*-solaris2*)
-	ABI="elf"
-	cat >> confdefs.h <<\EOF
-#define SUNOS 1
-EOF
-
-	;;
-  *)
-	echo "$ac_t""Unsupported operating system: ${host}" 1>&6
-	ABI="elf"
-	;;
-esac
-
-# Find a good install program.  We prefer a C program (faster),
-# so one script is as good as another.  But avoid the broken or
-# incompatible versions:
-# SysV /etc/install, /usr/sbin/install
-# SunOS /usr/etc/install
-# IRIX /sbin/install
-# AIX /bin/install
-# AIX 4 /usr/bin/installbsd, which doesn't work without a -g flag
-# AFS /usr/afsws/bin/install, which mishandles nonexistent args
-# SVR4 /usr/ucb/install, which tries to use the nonexistent group "staff"
-# ./install, which can be erroneously created by make from ./install.sh.
-echo $ac_n "checking for a BSD compatible install""... $ac_c" 1>&6
-echo "configure:1001: checking for a BSD compatible install" >&5
-if test -z "$INSTALL"; then
-if eval "test \"`echo '$''{'ac_cv_path_install'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-    IFS="${IFS= 	}"; ac_save_IFS="$IFS"; IFS=":"
-  for ac_dir in $PATH; do
-    # Account for people who put trailing slashes in PATH elements.
-    case "$ac_dir/" in
-    /|./|.//|/etc/*|/usr/sbin/*|/usr/etc/*|/sbin/*|/usr/afsws/bin/*|/usr/ucb/*) ;;
-    *)
-      # OSF1 and SCO ODT 3.0 have their own names for install.
-      # Don't use installbsd from OSF since it installs stuff as root
-      # by default.
-      for ac_prog in ginstall scoinst install; do
-        if test -f $ac_dir/$ac_prog; then
-	  if test $ac_prog = install &&
-            grep dspmsg $ac_dir/$ac_prog >/dev/null 2>&1; then
-	    # AIX install.  It has an incompatible calling convention.
-	    :
-	  else
-	    ac_cv_path_install="$ac_dir/$ac_prog -c"
-	    break 2
-	  fi
-	fi
-      done
-      ;;
-    esac
-  done
-  IFS="$ac_save_IFS"
-
-fi
-  if test "${ac_cv_path_install+set}" = set; then
-    INSTALL="$ac_cv_path_install"
-  else
-    # As a last resort, use the slow shell script.  We don't cache a
-    # path for INSTALL within a source directory, because that will
-    # break other packages using the cache if that directory is
-    # removed, or if the path is relative.
-    INSTALL="$ac_install_sh"
-  fi
-fi
-echo "$ac_t""$INSTALL" 1>&6
-
-# Use test -z because SunOS4 sh mishandles braces in ${var-val}.
-# It thinks the first close brace ends the variable substitution.
-test -z "$INSTALL_PROGRAM" && INSTALL_PROGRAM='${INSTALL}'
-
-test -z "$INSTALL_SCRIPT" && INSTALL_SCRIPT='${INSTALL_PROGRAM}'
-
-test -z "$INSTALL_DATA" && INSTALL_DATA='${INSTALL} -m 644'
-
-# Extract the first word of "ranlib", so it can be a program name with args.
-set dummy ranlib; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:1056: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_prog_RANLIB'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  if test -n "$RANLIB"; then
-  ac_cv_prog_RANLIB="$RANLIB" # Let the user override the test.
-else
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      ac_cv_prog_RANLIB="ranlib"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-  test -z "$ac_cv_prog_RANLIB" && ac_cv_prog_RANLIB=":"
-fi
-fi
-RANLIB="$ac_cv_prog_RANLIB"
-if test -n "$RANLIB"; then
-  echo "$ac_t""$RANLIB" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
-
-for ac_prog in ar gar
-do
-# Extract the first word of "$ac_prog", so it can be a program name with args.
-set dummy $ac_prog; ac_word=$2
-echo $ac_n "checking for $ac_word""... $ac_c" 1>&6
-echo "configure:1088: checking for $ac_word" >&5
-if eval "test \"`echo '$''{'ac_cv_path_AR'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  case "$AR" in
-  /*)
-  ac_cv_path_AR="$AR" # Let the user override the test with a path.
-  ;;
-  ?:/*)
-  ac_cv_path_AR="$AR" # Let the user override the test with a dos path.
-  ;;
-  *)
-  IFS="${IFS= 	}"; ac_save_ifs="$IFS"; IFS=":"
-  ac_dummy="$PATH"
-  for ac_dir in $ac_dummy; do
-    test -z "$ac_dir" && ac_dir=.
-    if test -f $ac_dir/$ac_word; then
-      ac_cv_path_AR="$ac_dir/$ac_word"
-      break
-    fi
-  done
-  IFS="$ac_save_ifs"
-  ;;
-esac
-fi
-AR="$ac_cv_path_AR"
-if test -n "$AR"; then
-  echo "$ac_t""$AR" 1>&6
-else
-  echo "$ac_t""no" 1>&6
-fi
-
-test -n "$AR" && break
-done
-
-
-echo $ac_n "checking for tgetent in -ltermcap""... $ac_c" 1>&6
-echo "configure:1125: checking for tgetent in -ltermcap" >&5
-ac_lib_var=`echo termcap'_'tgetent | sed 'y%./+-%__p_%'`
-if eval "test \"`echo '$''{'ac_cv_lib_$ac_lib_var'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  ac_save_LIBS="$LIBS"
-LIBS="-ltermcap  $LIBS"
-cat > conftest.$ac_ext <<EOF
-#line 1133 "configure"
-#include "confdefs.h"
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char tgetent();
-
-int main() {
-tgetent()
-; return 0; }
-EOF
-if { (eval echo configure:1144: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=no"
-fi
-rm -f conftest*
-LIBS="$ac_save_LIBS"
-
-fi
-if eval "test \"`echo '$ac_cv_lib_'$ac_lib_var`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_lib=HAVE_LIB`echo termcap | sed -e 's/[^a-zA-Z0-9_]/_/g' \
-    -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_lib 1
-EOF
-
-  LIBS="-ltermcap $LIBS"
-
-else
-  echo "$ac_t""no" 1>&6
-\
-  echo $ac_n "checking for tgetent in -ltinfo""... $ac_c" 1>&6
-echo "configure:1171: checking for tgetent in -ltinfo" >&5
-ac_lib_var=`echo tinfo'_'tgetent | sed 'y%./+-%__p_%'`
-if eval "test \"`echo '$''{'ac_cv_lib_$ac_lib_var'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  ac_save_LIBS="$LIBS"
-LIBS="-ltinfo  $LIBS"
-cat > conftest.$ac_ext <<EOF
-#line 1179 "configure"
-#include "confdefs.h"
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char tgetent();
-
-int main() {
-tgetent()
-; return 0; }
-EOF
-if { (eval echo configure:1190: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=no"
-fi
-rm -f conftest*
-LIBS="$ac_save_LIBS"
-
-fi
-if eval "test \"`echo '$ac_cv_lib_'$ac_lib_var`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_lib=HAVE_LIB`echo tinfo | sed -e 's/^a-zA-Z0-9_/_/g' \
-    -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_lib 1
-EOF
-
-  LIBS="-ltinfo $LIBS"
-
-else
-  echo "$ac_t""no" 1>&6
-\
-    echo $ac_n "checking for tgetent in -lcurses""... $ac_c" 1>&6
-echo "configure:1217: checking for tgetent in -lcurses" >&5
-ac_lib_var=`echo curses'_'tgetent | sed 'y%./+-%__p_%'`
-if eval "test \"`echo '$''{'ac_cv_lib_$ac_lib_var'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  ac_save_LIBS="$LIBS"
-LIBS="-lcurses  $LIBS"
-cat > conftest.$ac_ext <<EOF
-#line 1225 "configure"
-#include "confdefs.h"
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char tgetent();
-
-int main() {
-tgetent()
-; return 0; }
-EOF
-if { (eval echo configure:1236: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=no"
-fi
-rm -f conftest*
-LIBS="$ac_save_LIBS"
-
-fi
-if eval "test \"`echo '$ac_cv_lib_'$ac_lib_var`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_lib=HAVE_LIB`echo curses | sed -e 's/^a-zA-Z0-9_/_/g' \
-    -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_lib 1
-EOF
-
-  LIBS="-lcurses $LIBS"
-
-else
-  echo "$ac_t""no" 1>&6
-\
-      echo $ac_n "checking for tgetent in -lncurses""... $ac_c" 1>&6
-echo "configure:1263: checking for tgetent in -lncurses" >&5
-ac_lib_var=`echo ncurses'_'tgetent | sed 'y%./+-%__p_%'`
-if eval "test \"`echo '$''{'ac_cv_lib_$ac_lib_var'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  ac_save_LIBS="$LIBS"
-LIBS="-lncurses  $LIBS"
-cat > conftest.$ac_ext <<EOF
-#line 1271 "configure"
-#include "confdefs.h"
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char tgetent();
-
-int main() {
-tgetent()
-; return 0; }
-EOF
-if { (eval echo configure:1282: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_lib_$ac_lib_var=no"
-fi
-rm -f conftest*
-LIBS="$ac_save_LIBS"
-
-fi
-if eval "test \"`echo '$ac_cv_lib_'$ac_lib_var`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_lib=HAVE_LIB`echo ncurses | sed -e 's/^a-zA-Z0-9_/_/g' \
-    -e 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_lib 1
-EOF
-
-  LIBS="-lncurses $LIBS"
-
-else
-  echo "$ac_t""no" 1>&6
-\
-        { echo "configure: error: termcap support not found" 1>&2; exit 1; }
-fi
-
-fi
-
-fi
-
-fi
-
-
-for ac_hdr in termcap.h
-do
-ac_safe=`echo "$ac_hdr" | sed 'y%./+-%__p_%'`
-echo $ac_n "checking for $ac_hdr""... $ac_c" 1>&6
-echo "configure:1322: checking for $ac_hdr" >&5
-if eval "test \"`echo '$''{'ac_cv_header_$ac_safe'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1327 "configure"
-#include "confdefs.h"
-#include <$ac_hdr>
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:1332: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=yes"
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=no"
-fi
-rm -f conftest*
-fi
-if eval "test \"`echo '$ac_cv_header_'$ac_safe`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_hdr=HAVE_`echo $ac_hdr | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_hdr 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-\
-  for ac_hdr in term.h
-do
-ac_safe=`echo "$ac_hdr" | sed 'y%./+-%__p_%'`
-echo $ac_n "checking for $ac_hdr""... $ac_c" 1>&6
-echo "configure:1360: checking for $ac_hdr" >&5
-if eval "test \"`echo '$''{'ac_cv_header_$ac_safe'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1365 "configure"
-#include "confdefs.h"
-#include <$ac_hdr>
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:1370: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=yes"
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=no"
-fi
-rm -f conftest*
-fi
-if eval "test \"`echo '$ac_cv_header_'$ac_safe`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_hdr=HAVE_`echo $ac_hdr | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_hdr 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-\
-    echo "$ac_t""Need term.h since termcap.h is missing" 1>&6
-fi
-done
-
-  for ac_hdr in curses.h
-do
-ac_safe=`echo "$ac_hdr" | sed 'y%./+-%__p_%'`
-echo $ac_n "checking for $ac_hdr""... $ac_c" 1>&6
-echo "configure:1402: checking for $ac_hdr" >&5
-if eval "test \"`echo '$''{'ac_cv_header_$ac_safe'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1407 "configure"
-#include "confdefs.h"
-#include <$ac_hdr>
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:1412: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=yes"
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=no"
-fi
-rm -f conftest*
-fi
-if eval "test \"`echo '$ac_cv_header_'$ac_safe`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_hdr=HAVE_`echo $ac_hdr | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_hdr 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-\
-    for ac_hdr in ncurses.h
-do
-ac_safe=`echo "$ac_hdr" | sed 'y%./+-%__p_%'`
-echo $ac_n "checking for $ac_hdr""... $ac_c" 1>&6
-echo "configure:1440: checking for $ac_hdr" >&5
-if eval "test \"`echo '$''{'ac_cv_header_$ac_safe'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1445 "configure"
-#include "confdefs.h"
-#include <$ac_hdr>
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:1450: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=yes"
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=no"
-fi
-rm -f conftest*
-fi
-if eval "test \"`echo '$ac_cv_header_'$ac_safe`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_hdr=HAVE_`echo $ac_hdr | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_hdr 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-\
-      echo "$ac_t""Need curses.h or ncurses.h" 1>&6
-fi
-done
-
-fi
-done
-
-fi
-done
-
-
-for ac_hdr in sys/cdefs.h vis.h
-do
-ac_safe=`echo "$ac_hdr" | sed 'y%./+-%__p_%'`
-echo $ac_n "checking for $ac_hdr""... $ac_c" 1>&6
-echo "configure:1489: checking for $ac_hdr" >&5
-if eval "test \"`echo '$''{'ac_cv_header_$ac_safe'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1494 "configure"
-#include "confdefs.h"
-#include <$ac_hdr>
-EOF
-ac_try="$ac_cpp conftest.$ac_ext >/dev/null 2>conftest.out"
-{ (eval echo configure:1499: \"$ac_try\") 1>&5; (eval $ac_try) 2>&5; }
-ac_err=`grep -v '^ *+' conftest.out | grep -v "^conftest.${ac_ext}\$"`
-if test -z "$ac_err"; then
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=yes"
-else
-  echo "$ac_err" >&5
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_header_$ac_safe=no"
-fi
-rm -f conftest*
-fi
-if eval "test \"`echo '$ac_cv_header_'$ac_safe`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_hdr=HAVE_`echo $ac_hdr | sed 'y%abcdefghijklmnopqrstuvwxyz./-%ABCDEFGHIJKLMNOPQRSTUVWXYZ___%'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_hdr 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-fi
-done
-
-
-for ac_func in issetugid
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1529: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1534 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1557: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-fi
-done
-
-for ac_func in fgetln
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1584: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1589 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1612: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-CCSRCS="$CCSRCS np/fgetln.c"
-fi
-done
-
-for ac_func in strvis
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1640: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1645 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1668: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-CCSRCS="$CCSRCS np/vis.c"
-fi
-done
-
-for ac_func in strunvis
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1696: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1701 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1724: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-CCSRCS="$CCSRCS np/unvis.c"
-fi
-done
-
-for ac_func in strlcpy
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1752: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1757 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1780: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-CCSRCS="$CCSRCS np/strlcpy.c"
-fi
-done
-
-for ac_func in strlcat
-do
-echo $ac_n "checking for $ac_func""... $ac_c" 1>&6
-echo "configure:1808: checking for $ac_func" >&5
-if eval "test \"`echo '$''{'ac_cv_func_$ac_func'+set}'`\" = set"; then
-  echo $ac_n "(cached) $ac_c" 1>&6
-else
-  cat > conftest.$ac_ext <<EOF
-#line 1813 "configure"
-#include "confdefs.h"
-/* System header to define __stub macros and hopefully few prototypes,
-    which can conflict with char $ac_func(); below.  */
-#include <assert.h>
-/* Override any gcc2 internal prototype to avoid an error.  */
-/* We use char because int might match the return type of a gcc2
-    builtin and then its argument prototype would still apply.  */
-char $ac_func();
-
-int main() {
-
-/* The GNU C library defines this for functions which it implements
-    to always fail with ENOSYS.  Some functions are actually named
-    something starting with __ and the normal name is an alias.  */
-#if defined (__stub_$ac_func) || defined (__stub___$ac_func)
-choke me
-#else
-$ac_func();
-#endif
-
-; return 0; }
-EOF
-if { (eval echo configure:1836: \"$ac_link\") 1>&5; (eval $ac_link) 2>&5; } && test -s conftest${ac_exeext}; then
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=yes"
-else
-  echo "configure: failed program was:" >&5
-  cat conftest.$ac_ext >&5
-  rm -rf conftest*
-  eval "ac_cv_func_$ac_func=no"
-fi
-rm -f conftest*
-fi
-
-if eval "test \"`echo '$ac_cv_func_'$ac_func`\" = yes"; then
-  echo "$ac_t""yes" 1>&6
-    ac_tr_func=HAVE_`echo $ac_func | tr 'abcdefghijklmnopqrstuvwxyz' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'`
-  cat >> confdefs.h <<EOF
-#define $ac_tr_func 1
-EOF
-
-else
-  echo "$ac_t""no" 1>&6
-CCSRCS="$CCSRCS np/strlcat.c"
-fi
-done
-
-
-cat > conftest.$ac_ext <<EOF
-#line 1863 "configure"
-#include "confdefs.h"
-#include <sys/cdefs.h>
-#ifdef __RCSID
-  yes
-#endif
-
-EOF
-if (eval "$ac_cpp conftest.$ac_ext") 2>&5 |
-  egrep "yes" >/dev/null 2>&1; then
-  :
-else
-  rm -rf conftest*
-  CPPFLAGS="$CPPFLAGS '-D__RCSID(x)='"
-fi
-rm -f conftest*
-
-
-cat > conftest.$ac_ext <<EOF
-#line 1882 "configure"
-#include "confdefs.h"
-#include <sys/cdefs.h>
-#ifdef __COPYRIGHT
-  yes
-#endif
-
-EOF
-if (eval "$ac_cpp conftest.$ac_ext") 2>&5 |
-  egrep "yes" >/dev/null 2>&1; then
-  :
-else
-  rm -rf conftest*
-  CPPFLAGS="$CPPFLAGS '-D__COPYRIGHT(x)='"
-fi
-rm -f conftest*
-
-
-cat > conftest.$ac_ext <<EOF
-#line 1901 "configure"
-#include "confdefs.h"
-#include <sys/cdefs.h>
-#ifdef __RENAME
-  yes
-#endif
-
-EOF
-if (eval "$ac_cpp conftest.$ac_ext") 2>&5 |
-  egrep "yes" >/dev/null 2>&1; then
-  :
-else
-  rm -rf conftest*
-  CPPFLAGS="$CPPFLAGS '-D__RENAME(x)='"
-fi
-rm -f conftest*
-
-
-cat > conftest.$ac_ext <<EOF
-#line 1920 "configure"
-#include "confdefs.h"
-#include <sys/cdefs.h>
-#ifdef _DIAGASSERT
-  yes
-#endif
-
-EOF
-if (eval "$ac_cpp conftest.$ac_ext") 2>&5 |
-  egrep "yes" >/dev/null 2>&1; then
-  :
-else
-  rm -rf conftest*
-  CPPFLAGS="$CPPFLAGS '-D_DIAGASSERT(x)='"
-fi
-rm -f conftest*
-
-
-# Check whether --enable-readline or --disable-readline was given.
-if test "${enable_readline+set}" = set; then
-  enableval="$enable_readline"
-  if test "x$enable_readline" != "xyes" ; then
-  enable_readline="no"
-fi
-
-else
-  enable_readline="yes"
-
-fi
-
-
-# Check whether --enable-debug or --disable-debug was given.
-if test "${enable_debug+set}" = set; then
-  enableval="$enable_debug"
-  if test "x$enable_debug" != "xyes" ; then
-  enable_debug="no"
-fi
-
-else
-  enable_debug="no"
-
-fi
-
-if test "x$enable_debug" = "xyes" ; then
-  CPPFLAGS="$CPPFLAGS -DDEBUG_TTY -DDEBUG_KEY -DDEBUG_READ -DDEBUG"
-  CPPFLAGS="$CPPFLAGS -DDEBUG_REFRESH -DDEBUG_PASTE"
-else
-  CFLAGS="$CFLAGS -O"
-fi
-
-
-
-ACSRCS="common.c emacs.c vi.c"
-BCSRCS="chared.c el.c hist.c key.c map.c parse.c prompt.c read.c refresh.c search.c sig.c term.c tty.c"
-CCSRCS="$CCSRCS history.c tokenizer.c"
-
-AGCSRCS="fcns.c help.c"
-BGCSRCS="editline.c"
-
-HDRS="chared.h el.h hist.h key.h map.h parse.h prompt.h refresh.h search.h sig.h sys.h term.h tokenizer.h tty.h"
-
-IHDRS="histedit.h"
-
-IHDR_LINKS=
-
-AGHDRS="common.h emacs.h vi.h"
-BGHDRS="fcns.h help.h"
-
-HDR_DIRS="include"
-
-MAN3="editline.3"
-MAN5="editrc.5"
-
-MAN3_LINKS=
-for i in  el_init.3 el_end.3 el_reset.3 el_gets.3 el_getc.3 el_push.3 \
-          el_parse.3 el_set.3 el_get.3 el_source.3 el_resize.3 el_line.3 \
-	  el_insertstr.3 el_deletestr.3 history_init.3 history_end.3 \
-	  history.3 ; do
-  MAN3_LINKS="$MAN3_LINKS editline.3 $i"
-done
-
-MAN_DIRS="man/man3 man/man5"
-
-LIB_DIRS="lib"
-LIB_MAJOR="2"
-LIB_MINOR="6"
-LIB_A="libedit.a"
-LIB_A_LINKS=
-
-if test "x$ABI" = "xelf" ; then
-  LIB_S="libedit.so.$LIB_MAJOR"
-  LIB_S_LINK="libedit.so"
-  LIB_S_LINKS="$LIB_S $LIB_S_LINK"
-  S_LDFLAGS="-shared"
-elif test "x$ABI" = "xaout" ; then
-  LIB_S="libedit.so.$LIB_MAJOR.$LIB_MINOR"
-  LIB_S_LINKS=
-  S_LDFLAGS="-shared"
-elif test "x$ABI" = "xmacho" ; then
-  S_LDFLAGS="-shared"
-  LIB_S="libedit.$LIB_MAJOR.dylib"
-  LIB_S_LINK="libedit.dylib"
-  LIB_S_LINKS="$LIB_S $LIB_S_LINK"
-  if test "x$prefix" = "xNONE" ; then
-    S_LDFLAGS="-undefined suppress -flat_namespace -dynamiclib -compatibility_version $LIB_MAJOR -current_version $LIB_MAJOR -install_name /usr/local/lib/$LIB_S"
-  else
-    S_LDFLAGS="-undefined suppress -flat_namespace -dynamiclib -compatibility_version $LIB_MAJOR -current_version $LIB_MAJOR -install_name $prefix/lib/$LIB_S"
-  fi
-fi
-
-TEST="TEST/test"
-TCSRCS="TEST/test.c"
-
-if test "x$enable_readline" = "xyes" ; then
-  CCSRCS="$CCSRCS readline.c"
-  IHDRS="$IHDRS readline.h"
-  IHDR_LINKS="readline.h readline/history.h"
-  HDR_DIRS="$HDR_DIRS include/readline"
-  LIB_A_LINKS="$LIB_A_LINKS libedit.a libreadline.a"
-  if test "x$ABI" = "xelf" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S_LINK libreadline.so"
-  elif test "x$ABI" = "xaout" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S libreadline.so.$LIB_MAJOR.$LIB_MINOR"
-  elif test "x$ABI" = "xmacho" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S_LINK libreadline.dylib"
-  fi
-fi
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-trap '' 1 2 15
-cat > confcache <<\EOF
-# This file is a shell script that caches the results of configure
-# tests run on this system so they can be shared between configure
-# scripts and configure runs.  It is not useful on other systems.
-# If it contains results you don't want to keep, you may remove or edit it.
-#
-# By default, configure uses ./config.cache as the cache file,
-# creating it if it does not exist already.  You can give configure
-# the --cache-file=FILE option to use a different cache file; that is
-# what configure does when it calls configure scripts in
-# subdirectories, so they share the cache.
-# Giving --cache-file=/dev/null disables caching, for debugging configure.
-# config.status only pays attention to the cache file if you give it the
-# --recheck option to rerun configure.
-#
-EOF
-# The following way of writing the cache mishandles newlines in values,
-# but we know of no workaround that is simple, portable, and efficient.
-# So, don't put newlines in cache variables' values.
-# Ultrix sh set writes to stderr and can't be redirected directly,
-# and sets the high bit in the cache file unless we assign to the vars.
-(set) 2>&1 |
-  case `(ac_space=' '; set | grep ac_space) 2>&1` in
-  *ac_space=\ *)
-    # `set' does not quote correctly, so add quotes (double-quote substitution
-    # turns \\\\ into \\, and sed turns \\ into \).
-    sed -n \
-      -e "s/'/'\\\\''/g" \
-      -e "s/^\\([a-zA-Z0-9_]*_cv_[a-zA-Z0-9_]*\\)=\\(.*\\)/\\1=\${\\1='\\2'}/p"
-    ;;
-  *)
-    # `set' quotes correctly as required by POSIX, so do not add quotes.
-    sed -n -e 's/^\([a-zA-Z0-9_]*_cv_[a-zA-Z0-9_]*\)=\(.*\)/\1=${\1=\2}/p'
-    ;;
-  esac >> confcache
-if cmp -s $cache_file confcache; then
-  :
-else
-  if test -w $cache_file; then
-    echo "updating cache $cache_file"
-    cat confcache > $cache_file
-  else
-    echo "not updating unwritable cache $cache_file"
-  fi
-fi
-rm -f confcache
-
-trap 'rm -fr conftest* confdefs* core core.* *.core $ac_clean_files; exit 1' 1 2 15
-
-test "x$prefix" = xNONE && prefix=$ac_default_prefix
-# Let make expand exec_prefix.
-test "x$exec_prefix" = xNONE && exec_prefix='${prefix}'
-
-# Any assignment to VPATH causes Sun make to only execute
-# the first set of double-colon rules, so remove it if not needed.
-# If there is a colon in the path, we need to keep it.
-if test "x$srcdir" = x.; then
-  ac_vpsub='/^[ 	]*VPATH[ 	]*=[^:]*$/d'
-fi
-
-trap 'rm -f $CONFIG_STATUS conftest*; exit 1' 1 2 15
-
-DEFS=-DHAVE_CONFIG_H
-
-# Without the "./", some shells look in PATH for config.status.
-: ${CONFIG_STATUS=./config.status}
-
-echo creating $CONFIG_STATUS
-rm -f $CONFIG_STATUS
-cat > $CONFIG_STATUS <<EOF
-#! /bin/sh
-# Generated automatically by configure.
-# Run this file to recreate the current configuration.
-# This directory was configured as follows,
-# on host `(hostname || uname -n) 2>/dev/null | sed 1q`:
-#
-# $0 $ac_configure_args
-#
-# Compiler output produced by configure, useful for debugging
-# configure, is in ./config.log if it exists.
-
-ac_cs_usage="Usage: $CONFIG_STATUS [--recheck] [--version] [--help]"
-for ac_option
-do
-  case "\$ac_option" in
-  -recheck | --recheck | --rechec | --reche | --rech | --rec | --re | --r)
-    echo "running \${CONFIG_SHELL-/bin/sh} $0 $ac_configure_args --no-create --no-recursion"
-    exec \${CONFIG_SHELL-/bin/sh} $0 $ac_configure_args --no-create --no-recursion ;;
-  -version | --version | --versio | --versi | --vers | --ver | --ve | --v)
-    echo "$CONFIG_STATUS generated by autoconf version 2.13"
-    exit 0 ;;
-  -help | --help | --hel | --he | --h)
-    echo "\$ac_cs_usage"; exit 0 ;;
-  *) echo "\$ac_cs_usage"; exit 1 ;;
-  esac
-done
-
-ac_given_srcdir=$srcdir
-ac_given_INSTALL="$INSTALL"
-
-trap 'rm -fr `echo "Makefile makelist config.h" | sed "s/:[^ ]*//g"` conftest*; exit 1' 1 2 15
-EOF
-cat >> $CONFIG_STATUS <<EOF
-
-# Protect against being on the right side of a sed subst in config.status.
-sed 's/%@/@@/; s/@%/@@/; s/%g\$/@g/; /@g\$/s/[\\\\&%]/\\\\&/g;
- s/@@/%@/; s/@@/@%/; s/@g\$/%g/' > conftest.subs <<\\CEOF
-$ac_vpsub
-$extrasub
-s%@SHELL@%$SHELL%g
-s%@CFLAGS@%$CFLAGS%g
-s%@CPPFLAGS@%$CPPFLAGS%g
-s%@CXXFLAGS@%$CXXFLAGS%g
-s%@FFLAGS@%$FFLAGS%g
-s%@DEFS@%$DEFS%g
-s%@LDFLAGS@%$LDFLAGS%g
-s%@LIBS@%$LIBS%g
-s%@exec_prefix@%$exec_prefix%g
-s%@prefix@%$prefix%g
-s%@program_transform_name@%$program_transform_name%g
-s%@bindir@%$bindir%g
-s%@sbindir@%$sbindir%g
-s%@libexecdir@%$libexecdir%g
-s%@datadir@%$datadir%g
-s%@sysconfdir@%$sysconfdir%g
-s%@sharedstatedir@%$sharedstatedir%g
-s%@localstatedir@%$localstatedir%g
-s%@libdir@%$libdir%g
-s%@includedir@%$includedir%g
-s%@oldincludedir@%$oldincludedir%g
-s%@infodir@%$infodir%g
-s%@mandir@%$mandir%g
-s%@CC@%$CC%g
-s%@A_CFLAGS@%$A_CFLAGS%g
-s%@S_CFLAGS@%$S_CFLAGS%g
-s%@CPP@%$CPP%g
-s%@AWK@%$AWK%g
-s%@host@%$host%g
-s%@host_alias@%$host_alias%g
-s%@host_cpu@%$host_cpu%g
-s%@host_vendor@%$host_vendor%g
-s%@host_os@%$host_os%g
-s%@INSTALL_PROGRAM@%$INSTALL_PROGRAM%g
-s%@INSTALL_SCRIPT@%$INSTALL_SCRIPT%g
-s%@INSTALL_DATA@%$INSTALL_DATA%g
-s%@RANLIB@%$RANLIB%g
-s%@AR@%$AR%g
-s%@ACSRCS@%$ACSRCS%g
-s%@BCSRCS@%$BCSRCS%g
-s%@CCSRCS@%$CCSRCS%g
-s%@AGCSRCS@%$AGCSRCS%g
-s%@BGCSRCS@%$BGCSRCS%g
-s%@HDRS@%$HDRS%g
-s%@IHDRS@%$IHDRS%g
-s%@IHDR_LINKS@%$IHDR_LINKS%g
-s%@AGHDRS@%$AGHDRS%g
-s%@BGHDRS@%$BGHDRS%g
-s%@HDR_DIRS@%$HDR_DIRS%g
-s%@MAN3@%$MAN3%g
-s%@MAN5@%$MAN5%g
-s%@MAN3_LINKS@%$MAN3_LINKS%g
-s%@MAN_DIRS@%$MAN_DIRS%g
-s%@LIB_DIRS@%$LIB_DIRS%g
-s%@LIB_VER@%$LIB_VER%g
-s%@LIB_A@%$LIB_A%g
-s%@LIB_A_LINKS@%$LIB_A_LINKS%g
-s%@LIB_S@%$LIB_S%g
-s%@LIB_S_LINKS@%$LIB_S_LINKS%g
-s%@S_LDFLAGS@%$S_LDFLAGS%g
-s%@TEST@%$TEST%g
-s%@TCSRCS@%$TCSRCS%g
-
-CEOF
-EOF
-
-cat >> $CONFIG_STATUS <<\EOF
-
-# Split the substitutions into bite-sized pieces for seds with
-# small command number limits, like on Digital OSF/1 and HP-UX.
-ac_max_sed_cmds=90 # Maximum number of lines to put in a sed script.
-ac_file=1 # Number of current file.
-ac_beg=1 # First line for current file.
-ac_end=$ac_max_sed_cmds # Line after last line for current file.
-ac_more_lines=:
-ac_sed_cmds=""
-while $ac_more_lines; do
-  if test $ac_beg -gt 1; then
-    sed "1,${ac_beg}d; ${ac_end}q" conftest.subs > conftest.s$ac_file
-  else
-    sed "${ac_end}q" conftest.subs > conftest.s$ac_file
-  fi
-  if test ! -s conftest.s$ac_file; then
-    ac_more_lines=false
-    rm -f conftest.s$ac_file
-  else
-    if test -z "$ac_sed_cmds"; then
-      ac_sed_cmds="sed -f conftest.s$ac_file"
-    else
-      ac_sed_cmds="$ac_sed_cmds | sed -f conftest.s$ac_file"
-    fi
-    ac_file=`expr $ac_file + 1`
-    ac_beg=$ac_end
-    ac_end=`expr $ac_end + $ac_max_sed_cmds`
-  fi
-done
-if test -z "$ac_sed_cmds"; then
-  ac_sed_cmds=cat
-fi
-EOF
-
-cat >> $CONFIG_STATUS <<EOF
-
-CONFIG_FILES=\${CONFIG_FILES-"Makefile makelist"}
-EOF
-cat >> $CONFIG_STATUS <<\EOF
-for ac_file in .. $CONFIG_FILES; do if test "x$ac_file" != x..; then
-  # Support "outfile[:infile[:infile...]]", defaulting infile="outfile.in".
-  case "$ac_file" in
-  *:*) ac_file_in=`echo "$ac_file"|sed 's%[^:]*:%%'`
-       ac_file=`echo "$ac_file"|sed 's%:.*%%'` ;;
-  *) ac_file_in="${ac_file}.in" ;;
-  esac
-
-  # Adjust a relative srcdir, top_srcdir, and INSTALL for subdirectories.
-
-  # Remove last slash and all that follows it.  Not all systems have dirname.
-  ac_dir=`echo $ac_file|sed 's%/[^/][^/]*$%%'`
-  if test "$ac_dir" != "$ac_file" && test "$ac_dir" != .; then
-    # The file is in a subdirectory.
-    test ! -d "$ac_dir" && mkdir "$ac_dir"
-    ac_dir_suffix="/`echo $ac_dir|sed 's%^\./%%'`"
-    # A "../" for each directory in $ac_dir_suffix.
-    ac_dots=`echo $ac_dir_suffix|sed 's%/[^/]*%../%g'`
-  else
-    ac_dir_suffix= ac_dots=
-  fi
-
-  case "$ac_given_srcdir" in
-  .)  srcdir=.
-      if test -z "$ac_dots"; then top_srcdir=.
-      else top_srcdir=`echo $ac_dots|sed 's%/$%%'`; fi ;;
-  /*) srcdir="$ac_given_srcdir$ac_dir_suffix"; top_srcdir="$ac_given_srcdir" ;;
-  *) # Relative path.
-    srcdir="$ac_dots$ac_given_srcdir$ac_dir_suffix"
-    top_srcdir="$ac_dots$ac_given_srcdir" ;;
-  esac
-
-  case "$ac_given_INSTALL" in
-  [/$]*) INSTALL="$ac_given_INSTALL" ;;
-  *) INSTALL="$ac_dots$ac_given_INSTALL" ;;
-  esac
-
-  echo creating "$ac_file"
-  rm -f "$ac_file"
-  configure_input="Generated automatically from `echo $ac_file_in|sed 's%.*/%%'` by configure."
-  case "$ac_file" in
-  *Makefile*) ac_comsub="1i\\
-# $configure_input" ;;
-  *) ac_comsub= ;;
-  esac
-
-  ac_file_inputs=`echo $ac_file_in|sed -e "s%^%$ac_given_srcdir/%" -e "s%:% $ac_given_srcdir/%g"`
-  sed -e "$ac_comsub
-s%@configure_input@%$configure_input%g
-s%@srcdir@%$srcdir%g
-s%@top_srcdir@%$top_srcdir%g
-s%@INSTALL@%$INSTALL%g
-" $ac_file_inputs | (eval "$ac_sed_cmds") > $ac_file
-fi; done
-rm -f conftest.s*
-
-# These sed commands are passed to sed as "A NAME B NAME C VALUE D", where
-# NAME is the cpp macro being defined and VALUE is the value it is being given.
-#
-# ac_d sets the value in "#define NAME VALUE" lines.
-ac_dA='s%^\([ 	]*\)#\([ 	]*define[ 	][ 	]*\)'
-ac_dB='\([ 	][ 	]*\)[^ 	]*%\1#\2'
-ac_dC='\3'
-ac_dD='%g'
-# ac_u turns "#undef NAME" with trailing blanks into "#define NAME VALUE".
-ac_uA='s%^\([ 	]*\)#\([ 	]*\)undef\([ 	][ 	]*\)'
-ac_uB='\([ 	]\)%\1#\2define\3'
-ac_uC=' '
-ac_uD='\4%g'
-# ac_e turns "#undef NAME" without trailing blanks into "#define NAME VALUE".
-ac_eA='s%^\([ 	]*\)#\([ 	]*\)undef\([ 	][ 	]*\)'
-ac_eB='$%\1#\2define\3'
-ac_eC=' '
-ac_eD='%g'
-
-if test "${CONFIG_HEADERS+set}" != set; then
-EOF
-cat >> $CONFIG_STATUS <<EOF
-  CONFIG_HEADERS="config.h"
-EOF
-cat >> $CONFIG_STATUS <<\EOF
-fi
-for ac_file in .. $CONFIG_HEADERS; do if test "x$ac_file" != x..; then
-  # Support "outfile[:infile[:infile...]]", defaulting infile="outfile.in".
-  case "$ac_file" in
-  *:*) ac_file_in=`echo "$ac_file"|sed 's%[^:]*:%%'`
-       ac_file=`echo "$ac_file"|sed 's%:.*%%'` ;;
-  *) ac_file_in="${ac_file}.in" ;;
-  esac
-
-  echo creating $ac_file
-
-  rm -f conftest.frag conftest.in conftest.out
-  ac_file_inputs=`echo $ac_file_in|sed -e "s%^%$ac_given_srcdir/%" -e "s%:% $ac_given_srcdir/%g"`
-  cat $ac_file_inputs > conftest.in
-
-EOF
-
-# Transform confdefs.h into a sed script conftest.vals that substitutes
-# the proper values into config.h.in to produce config.h.  And first:
-# Protect against being on the right side of a sed subst in config.status.
-# Protect against being in an unquoted here document in config.status.
-rm -f conftest.vals
-cat > conftest.hdr <<\EOF
-s/[\\&%]/\\&/g
-s%[\\$`]%\\&%g
-s%#define \([A-Za-z_][A-Za-z0-9_]*\) *\(.*\)%${ac_dA}\1${ac_dB}\1${ac_dC}\2${ac_dD}%gp
-s%ac_d%ac_u%gp
-s%ac_u%ac_e%gp
-EOF
-sed -n -f conftest.hdr confdefs.h > conftest.vals
-rm -f conftest.hdr
-
-# This sed command replaces #undef with comments.  This is necessary, for
-# example, in the case of _POSIX_SOURCE, which is predefined and required
-# on some systems where configure will not decide to define it.
-cat >> conftest.vals <<\EOF
-s%^[ 	]*#[ 	]*undef[ 	][ 	]*[a-zA-Z_][a-zA-Z_0-9]*%/* & */%
-EOF
-
-# Break up conftest.vals because some shells have a limit on
-# the size of here documents, and old seds have small limits too.
-
-rm -f conftest.tail
-while :
-do
-  ac_lines=`grep -c . conftest.vals`
-  # grep -c gives empty output for an empty file on some AIX systems.
-  if test -z "$ac_lines" || test "$ac_lines" -eq 0; then break; fi
-  # Write a limited-size here document to conftest.frag.
-  echo '  cat > conftest.frag <<CEOF' >> $CONFIG_STATUS
-  sed ${ac_max_here_lines}q conftest.vals >> $CONFIG_STATUS
-  echo 'CEOF
-  sed -f conftest.frag conftest.in > conftest.out
-  rm -f conftest.in
-  mv conftest.out conftest.in
-' >> $CONFIG_STATUS
-  sed 1,${ac_max_here_lines}d conftest.vals > conftest.tail
-  rm -f conftest.vals
-  mv conftest.tail conftest.vals
-done
-rm -f conftest.vals
-
-cat >> $CONFIG_STATUS <<\EOF
-  rm -f conftest.frag conftest.h
-  echo "/* $ac_file.  Generated automatically by configure.  */" > conftest.h
-  cat conftest.in >> conftest.h
-  rm -f conftest.in
-  if cmp -s $ac_file conftest.h 2>/dev/null; then
-    echo "$ac_file is unchanged"
-    rm -f conftest.h
-  else
-    # Remove last slash and all that follows it.  Not all systems have dirname.
-      ac_dir=`echo $ac_file|sed 's%/[^/][^/]*$%%'`
-      if test "$ac_dir" != "$ac_file" && test "$ac_dir" != .; then
-      # The file is in a subdirectory.
-      test ! -d "$ac_dir" && mkdir "$ac_dir"
-    fi
-    rm -f $ac_file
-    mv conftest.h $ac_file
-  fi
-fi; done
-
-EOF
-cat >> $CONFIG_STATUS <<EOF
-
-EOF
-cat >> $CONFIG_STATUS <<\EOF
-
-exit 0
-EOF
-chmod +x $CONFIG_STATUS
-rm -fr confdefs* $ac_clean_files
-test "$no_create" = yes || ${CONFIG_SHELL-/bin/sh} $CONFIG_STATUS || exit 1
diff -pruN 1:13.23.1~dfsg-2/main/editline/configure.in 1:16.1.1~dfsg-1/main/editline/configure.in
--- 1:13.23.1~dfsg-2/main/editline/configure.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/configure.in	1970-01-01 00:00:00.000000000 +0000
@@ -1,277 +0,0 @@
-dnl
-dnl Process this file with autoconf to produce a configure script.
-dnl
-AC_INIT(Makefile.in)
-
-dnl If CFLAGS isn't defined and using gcc, set CFLAGS to something reasonable.
-dnl Otherwise, just prevent autoconf from molesting CFLAGS.
-CFLAGS=$CFLAGS
-AC_PROG_CC
-if test "x$CFLAGS" = "x" ; then
-  no_CFLAGS="yes"
-fi
-if test "x$no_CFLAGS" = "xyes" -a "x$GCC" = "xyes" ; then
-	CFLAGS="-Wall -pipe -g3"
-fi
-A_CFLAGS=""
-AC_SUBST(A_CFLAGS)
-S_CFLAGS="-fPIC -DPIC"
-AC_SUBST(S_CFLAGS)
-AC_PROG_CPP
-AC_PROG_AWK
-
-dnl Platform-specific settings.  The ABI can probably be determined
-dnl programmatically, but doing so is error-prone, which makes it generally
-dnl not worth the trouble.
-AC_CANONICAL_HOST
-case "${host}" in
-  *-*-darwin*)
-	CFLAGS="$CFLAGS -fno-common -no-cpp-precomp"
-	ABI="macho"
-	;;
-  *-*-freebsd*)
-	ABI="elf"
-	;;
-  *-*-linux* | *cygwin*)
-	if echo ${host} | grep -q cygwin ; then \
-		echo "cygwin detected"; \
-		S_CFLAGS=""; \
-		echo "/* cygdef.h. Generated automatically by configure. */
-#ifndef _CYGDEF_H_
-#define _CYGDEF_H_ 1
-#include <sys/ioctl.h>
-#define __linux__ 1
-
-
-typedef void (*sig_t)(int);
-
-
-#endif /* _CYGDEF_H_ */" > cygdef.h; \
-	echo "
- #define CYGWIN 1
-" > confdefs.h; \
-	fi
-	ABI="elf"
-	;;
-  *-*-netbsd*)
-	AC_MSG_CHECKING(ABI)
-	AC_EGREP_CPP(yes,
-[#ifdef __ELF__
-  yes
-#endif
-],
-		     ABI="elf",
-		     ABI="aout")
-	AC_MSG_RESULT($ABI)
-	;;
-  *-*-solaris2*)
-	ABI="elf"
-	AC_DEFINE(SUNOS)
-	;;
-  *)
-	AC_MSG_RESULT(Unsupported operating system: ${host})
-	ABI="elf"
-	;;
-esac
-
-AC_PROG_INSTALL
-AC_PROG_RANLIB
-AC_PATH_PROGS(AR, ar gar, , $PATH)
-
-dnl Search for termcap access routines in termcap, tinfo, curses, and ncurses.
-AC_CHECK_LIB(termcap, tgetent, , \
-  AC_CHECK_LIB(tinfo, tgetent, , \
-    AC_CHECK_LIB(curses, tgetent, , \
-      AC_CHECK_LIB(ncurses, tgetent, , \
-        AC_MSG_ERROR(termcap support not found)))))
-
-dnl Use termcap.h if it exists; otherwise we need both term.h and [n]curses.h.
-AC_CHECK_HEADERS(termcap.h, , \
-  AC_CHECK_HEADERS(term.h, , \
-    AC_MSG_RESULT(Need term.h since termcap.h is missing))
-  AC_CHECK_HEADERS(curses.h, , \
-    AC_CHECK_HEADERS(ncurses.h, , \
-      AC_MSG_RESULT(Need curses.h or ncurses.h))))
-
-AC_CHECK_HEADERS(sys/cdefs.h vis.h)
-
-AC_CHECK_FUNCS(issetugid)
-AC_CHECK_FUNCS(fgetln, , CCSRCS="$CCSRCS np/fgetln.c")
-AC_CHECK_FUNCS(strvis, , CCSRCS="$CCSRCS np/vis.c")
-AC_CHECK_FUNCS(strunvis, , CCSRCS="$CCSRCS np/unvis.c")
-AC_CHECK_FUNCS(strlcpy, , CCSRCS="$CCSRCS np/strlcpy.c")
-AC_CHECK_FUNCS(strlcat, , CCSRCS="$CCSRCS np/strlcat.c")
-
-AC_EGREP_CPP(yes,
-[#include <sys/cdefs.h>
-#ifdef __RCSID
-  yes
-#endif
-], , [CPPFLAGS="$CPPFLAGS '-D__RCSID(x)='"])
-
-AC_EGREP_CPP(yes,
-[#include <sys/cdefs.h>
-#ifdef __COPYRIGHT
-  yes
-#endif
-], , [CPPFLAGS="$CPPFLAGS '-D__COPYRIGHT(x)='"])
-
-AC_EGREP_CPP(yes,
-[#include <sys/cdefs.h>
-#ifdef __RENAME
-  yes
-#endif
-], , [CPPFLAGS="$CPPFLAGS '-D__RENAME(x)='"])
-
-AC_EGREP_CPP(yes,
-[#include <sys/cdefs.h>
-#ifdef _DIAGASSERT
-  yes
-#endif
-], , [CPPFLAGS="$CPPFLAGS '-D_DIAGASSERT(x)='"])
-
-dnl Enable readline compatibility by default.
-AC_ARG_ENABLE(readline, [  --disable-readline      Disable readline compatibility],
-if test "x$enable_readline" != "xyes" ; then
-  enable_readline="no"
-fi
-,
-enable_readline="yes"
-)
-
-dnl Optionally enable debugging.
-AC_ARG_ENABLE(debug, [  --enable-debug          Enable debugging code],
-if test "x$enable_debug" != "xyes" ; then
-  enable_debug="no"
-fi
-,
-enable_debug="no"
-)
-if test "x$enable_debug" = "xyes" ; then
-  CPPFLAGS="$CPPFLAGS -DDEBUG_TTY -DDEBUG_KEY -DDEBUG_READ -DDEBUG"
-  CPPFLAGS="$CPPFLAGS -DDEBUG_REFRESH -DDEBUG_PASTE"
-else
-  CFLAGS="$CFLAGS -O"
-fi
-
-
-dnl
-dnl File lists.  This is done here instead of in the Makefile in order to avoid
-dnl the need for conditionals.
-dnl
-
-dnl .c files.
-ACSRCS="common.c emacs.c vi.c"
-BCSRCS="chared.c el.c hist.c key.c map.c parse.c prompt.c read.c refresh.c search.c sig.c term.c tty.c"
-CCSRCS="$CCSRCS history.c tokenizer.c"
-
-dnl Generated .c files.
-AGCSRCS="fcns.c help.c"
-BGCSRCS="editline.c"
-
-dnl .h files.
-HDRS="chared.h el.h hist.h key.h map.h parse.h prompt.h refresh.h search.h sig.h sys.h term.h tokenizer.h tty.h"
-
-dnl Installed .h files.
-IHDRS="histedit.h"
-
-dnl Installed headers for readline compatibility.
-IHDR_LINKS=
-
-dnl Generated .h files.
-AGHDRS="common.h emacs.h vi.h"
-BGHDRS="fcns.h help.h"
-
-dnl Header installation directories.
-HDR_DIRS="include"
-
-dnl Man pages.
-MAN3="editline.3"
-MAN5="editrc.5"
-
-MAN3_LINKS=
-for i in  el_init.3 el_end.3 el_reset.3 el_gets.3 el_getc.3 el_push.3 \
-          el_parse.3 el_set.3 el_get.3 el_source.3 el_resize.3 el_line.3 \
-	  el_insertstr.3 el_deletestr.3 history_init.3 history_end.3 \
-	  history.3 ; do
-  MAN3_LINKS="$MAN3_LINKS editline.3 $i"
-done
-
-dnl Man page installation directories.
-MAN_DIRS="man/man3 man/man5"
-
-dnl Library settings.
-LIB_DIRS="lib"
-LIB_MAJOR="2"
-LIB_MINOR="6"
-LIB_A="libedit.a"
-LIB_A_LINKS=
-
-if test "x$ABI" = "xelf" ; then
-  LIB_S="libedit.so.$LIB_MAJOR"
-  LIB_S_LINK="libedit.so"
-  LIB_S_LINKS="$LIB_S $LIB_S_LINK"
-  S_LDFLAGS="-shared"
-elif test "x$ABI" = "xaout" ; then
-  LIB_S="libedit.so.$LIB_MAJOR.$LIB_MINOR"
-  LIB_S_LINKS=
-  S_LDFLAGS="-shared"
-elif test "x$ABI" = "xmacho" ; then
-  S_LDFLAGS="-shared"
-  LIB_S="libedit.$LIB_MAJOR.dylib"
-  LIB_S_LINK="libedit.dylib"
-  LIB_S_LINKS="$LIB_S $LIB_S_LINK"
-  if test "x$prefix" = "xNONE" ; then
-    S_LDFLAGS="-undefined suppress -flat_namespace -dynamiclib -compatibility_version $LIB_MAJOR -current_version $LIB_MAJOR -install_name /usr/local/lib/$LIB_S"
-  else
-    S_LDFLAGS="-undefined suppress -flat_namespace -dynamiclib -compatibility_version $LIB_MAJOR -current_version $LIB_MAJOR -install_name $prefix/lib/$LIB_S"
-  fi
-fi
-
-dnl Test program.
-TEST="TEST/test"
-TCSRCS="TEST/test.c"
-
-dnl Add files to the lists if readline compatibility is enabled.
-if test "x$enable_readline" = "xyes" ; then
-  CCSRCS="$CCSRCS readline.c"
-  IHDRS="$IHDRS readline.h"
-  IHDR_LINKS="readline.h readline/history.h"
-  HDR_DIRS="$HDR_DIRS include/readline"
-  LIB_A_LINKS="$LIB_A_LINKS libedit.a libreadline.a"
-  if test "x$ABI" = "xelf" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S_LINK libreadline.so"
-  elif test "x$ABI" = "xaout" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S libreadline.so.$LIB_MAJOR.$LIB_MINOR"
-  elif test "x$ABI" = "xmacho" ; then
-    LIB_S_LINKS="$LIB_S_LINKS $LIB_S_LINK libreadline.dylib"
-  fi
-fi
-
-AC_SUBST(ACSRCS)
-AC_SUBST(BCSRCS)
-AC_SUBST(CCSRCS)
-AC_SUBST(AGCSRCS)
-AC_SUBST(BGCSRCS)
-AC_SUBST(HDRS)
-AC_SUBST(IHDRS)
-AC_SUBST(IHDR_LINKS)
-AC_SUBST(AGHDRS)
-AC_SUBST(BGHDRS)
-AC_SUBST(HDR_DIRS)
-AC_SUBST(MAN3)
-AC_SUBST(MAN5)
-AC_SUBST(MAN3_LINKS)
-AC_SUBST(MAN_DIRS)
-AC_SUBST(LIB_DIRS)
-AC_SUBST(LIB_VER)
-AC_SUBST(LIB_A)
-AC_SUBST(LIB_A_LINKS)
-AC_SUBST(LIB_S)
-AC_SUBST(LIB_S_LINKS)
-AC_SUBST(S_LDFLAGS)
-AC_SUBST(TEST)
-AC_SUBST(TCSRCS)
-
-AC_CONFIG_HEADER(config.h)
-AC_OUTPUT([Makefile makelist])
diff -pruN 1:13.23.1~dfsg-2/main/editline/editline.3 1:16.1.1~dfsg-1/main/editline/editline.3
--- 1:13.23.1~dfsg-2/main/editline/editline.3	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/editline.3	1970-01-01 00:00:00.000000000 +0000
@@ -1,646 +0,0 @@
-.\"	$NetBSD: editline.3,v 1.25 2002/01/15 02:46:22 wiz Exp $
-.\"
-.\" Copyright (c) 1997-1999 The NetBSD Foundation, Inc.
-.\" All rights reserved.
-.\"
-.\" This file was contributed to The NetBSD Foundation by Luke Mewburn.
-.\"
-.\" Redistribution and use in source and binary forms, with or without
-.\" modification, are permitted provided that the following conditions
-.\" are met:
-.\" 1. Redistributions of source code must retain the above copyright
-.\"    notice, this list of conditions and the following disclaimer.
-.\" 2. Redistributions in binary form must reproduce the above copyright
-.\"    notice, this list of conditions and the following disclaimer in the
-.\"    documentation and/or other materials provided with the distribution.
-.\" 3. All advertising materials mentioning features or use of this software
-.\"    must display the following acknowledgement:
-.\"        This product includes software developed by the NetBSD
-.\"        Foundation, Inc. and its contributors.
-.\" 4. Neither the name of The NetBSD Foundation nor the names of its
-.\"    contributors may be used to endorse or promote products derived
-.\"    from this software without specific prior written permission.
-.\"
-.\" THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
-.\" ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-.\" TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-.\" PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
-.\" BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-.\" CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-.\" SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-.\" INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-.\" CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-.\" ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-.\" POSSIBILITY OF SUCH DAMAGE.
-.\"
-.Dd November 12, 1999
-.Os
-.Dt EDITLINE 3
-.Sh NAME
-.Nm editline ,
-.Nm el_init ,
-.Nm el_end ,
-.Nm el_reset ,
-.Nm el_gets ,
-.Nm el_getc ,
-.Nm el_push ,
-.Nm el_parse ,
-.Nm el_set ,
-.Nm el_source ,
-.Nm el_resize ,
-.Nm el_line ,
-.Nm el_insertstr ,
-.Nm el_deletestr ,
-.Nm history_init ,
-.Nm history_end ,
-.Nm history
-.Nd line editor and history functions
-.Sh LIBRARY
-.Lb libedit
-.Sh SYNOPSIS
-.Fd #include <histedit.h>
-.Ft EditLine *
-.Fn el_init "const char *prog" "FILE *fin" "FILE *fout" "FILE *ferr"
-.Ft void
-.Fn el_end "EditLine *e"
-.Ft void
-.Fn el_reset "EditLine *e"
-.Ft const char *
-.Fn el_gets "EditLine *e" "int *count"
-.Ft int
-.Fn el_getc "EditLine *e" "char *ch"
-.Ft void
-.Fn el_push "EditLine *e" "const char *str"
-.Ft int
-.Fn el_parse "EditLine *e" "int argc" "char *argv[]"
-.Ft int
-.Fn el_set "EditLine *e" "int op" "..."
-.Ft int
-.Fn el_get "EditLine *e" "int op" "void *result"
-.Ft int
-.Fn el_source "EditLine *e" "const char *file"
-.Ft void
-.Fn el_resize "EditLine *e"
-.Ft const LineInfo *
-.Fn el_line "EditLine *e"
-.Ft int
-.Fn el_insertstr "EditLine *e" "const char *str"
-.Ft void
-.Fn el_deletestr "EditLine *e" "int count"
-.Ft History *
-.Fn history_init
-.Ft void
-.Fn history_end "History *h"
-.Ft int
-.Fn history "History *h" "HistEvent *ev" "int op" "..."
-.Sh DESCRIPTION
-The
-.Nm
-library provides generic line editing and history functions,
-similar to those found in
-.Xr sh 1 .
-.Pp
-These functions are available in the
-.Nm libedit
-library (which needs the
-.Nm libtermcap
-library).
-Programs should be linked with
-.Fl ledit ltermcap .
-.Sh LINE EDITING FUNCTIONS
-The line editing functions use a common data structure,
-.Fa EditLine ,
-which is created by
-.Fn el_init
-and freed by
-.Fn el_end .
-.Pp
-The following functions are available:
-.Bl -tag -width 4n
-.It Fn el_init
-Initialise the line editor, and return a data structure
-to be used by all other line editing functions.
-.Fa prog
-is the name of the invoking program, used when reading the
-.Xr editrc 5
-file to determine which settings to use.
-.Fa fin ,
-.Fa fout
-and
-.Fa ferr
-are the input, output, and error streams (respectively) to use.
-In this documentation, references to
-.Dq the tty
-are actually to this input/output stream combination.
-.It Fn el_end
-Clean up and finish with
-.Fa e ,
-assumed to have been created with
-.Fn el_init .
-.It Fn el_reset
-Reset the tty and the parser.
-This should be called after an error which may have upset the tty's
-state.
-.It Fn el_gets
-Read a line from the tty.
-.Fa count
-is modified to contain the number of characters read.
-Returns the line read if successful, or
-.Dv NULL
-if no characters were read or if an error occurred.
-.It Fn el_getc
-Read a character from the tty.
-.Fa ch
-is modified to contain the character read.
-Returns the number of characters read if successful, -1 otherwise.
-.It Fn el_push
-Pushes
-.Fa str
-back onto the input stream.
-This is used by the macro expansion mechanism.
-Refer to the description of
-.Ic bind
-.Fl s
-in
-.Xr editrc 5
-for more information.
-.It Fn el_parse
-Parses the
-.Fa argv
-array (which is
-.Fa argc
-elements in size)
-to execute builtin
-.Nm
-commands.
-If the command is prefixed with
-.Dq prog :
-then
-.Fn el_parse
-will only execute the command if
-.Dq prog
-matches the
-.Fa prog
-argument supplied to
-.Fn el_init .
-The return value is
--1 if the command is unknown,
-0 if there was no error or
-.Dq prog
-didn't match, or
-1 if the command returned an error.
-Refer to
-.Xr editrc 5
-for more information.
-.It Fn el_set
-Set
-.Nm
-parameters.
-.Fa op
-determines which parameter to set, and each operation has its
-own parameter list.
-.Pp
-The following values for
-.Fa op
-are supported, along with the required argument list:
-.Bl -tag -width 4n
-.It Dv EL_PROMPT , Fa "char *(*f)(EditLine *)"
-Define prompt printing function as
-.Fa f ,
-which is to return a string that contains the prompt.
-.It Dv EL_RPROMPT , Fa "char *(*f)(EditLine *)"
-Define right side prompt printing function as
-.Fa f ,
-which is to return a string that contains the prompt.
-.It Dv EL_TERMINAL , Fa "const char *type"
-Define terminal type of the tty to be
-.Fa type ,
-or to
-.Ev TERM
-if
-.Fa type
-is
-.Dv NULL .
-.It Dv EL_EDITOR , Fa "const char *mode"
-Set editing mode to
-.Fa mode ,
-which must be one of
-.Dq emacs
-or
-.Dq vi .
-.It Dv EL_SIGNAL , Fa "int flag"
-If
-.Fa flag
-is non-zero,
-.Nm
-will install its own signal handler for the following signals when
-reading command input:
-.Dv SIGCONT ,
-.Dv SIGHUP ,
-.Dv SIGINT ,
-.Dv SIGQUIT ,
-.Dv SIGSTOP ,
-.Dv SIGTERM ,
-.Dv SIGTSTP ,
-and
-.Dv SIGWINCH .
-Otherwise, the current signal handlers will be used.
-.It Dv EL_BIND , Xo
-.Fa "const char *" ,
-.Fa "..." ,
-.Dv NULL
-.Xc
-Perform the
-.Ic bind
-builtin command.
-Refer to
-.Xr editrc 5
-for more information.
-.It Dv EL_ECHOTC , Xo
-.Fa "const char *" ,
-.Fa "..." ,
-.Dv NULL
-.Xc
-Perform the
-.Ic echotc
-builtin command.
-Refer to
-.Xr editrc 5
-for more information.
-.It Dv EL_SETTC , Xo
-.Fa "const char *" ,
-.Fa "..." ,
-.Dv NULL
-.Xc
-Perform the
-.Ic settc
-builtin command.
-Refer to
-.Xr editrc 5
-for more information.
-.It Dv EL_SETTY , Xo
-.Fa "const char *" ,
-.Fa "..." ,
-.Dv NULL
-.Xc
-Perform the
-.Ic setty
-builtin command.
-Refer to
-.Xr editrc 5
-for more information.
-.It Dv EL_TELLTC , Xo
-.Fa "const char *" ,
-.Fa "..." ,
-.Dv NULL
-.Xc
-Perform the
-.Ic telltc
-builtin command.
-Refer to
-.Xr editrc 5
-for more information.
-.It Dv EL_ADDFN , Xo
-.Fa "const char *name" ,
-.Fa "const char *help" ,
-.Fa "unsigned char (*func)(EditLine *e, int ch)
-.Xc
-Add a user defined function,
-.Fn func ,
-referred to as
-.Fa name
-which is invoked when a key which is bound to
-.Fa name
-is entered.
-.Fa help
-is a description of
-.Fa name .
-At invocation time,
-.Fa ch
-is the key which caused the invocation.
-The return value of
-.Fn func
-should be one of:
-.Bl -tag -width "CC_REDISPLAY"
-.It Dv CC_NORM
-Add a normal character.
-.It Dv CC_NEWLINE
-End of line was entered.
-.It Dv CC_EOF
-EOF was entered.
-.It Dv CC_ARGHACK
-Expecting further command input as arguments, do nothing visually.
-.It Dv CC_REFRESH
-Refresh display.
-.It Dv CC_REFRESH_BEEP
-Refresh display, and beep.
-.It Dv CC_CURSOR
-Cursor moved, so update and perform
-.Dv CC_REFRESH .
-.It Dv CC_REDISPLAY
-Redisplay entire input line.
-This is useful if a key binding outputs extra information.
-.It Dv CC_ERROR
-An error occurred.
-Beep, and flush tty.
-.It Dv CC_FATAL
-Fatal error, reset tty to known state.
-.El
-.It Dv EL_HIST , Xo
-.Fa "History *(*func)(History *, int op, ...)" ,
-.Fa "const char *ptr"
-.Xc
-Defines which history function to use, which is usually
-.Fn history .
-.Fa ptr
-should be the value returned by
-.Fn history_init .
-.It Dv EL_EDITMODE , Fa "int flag"
-If
-.Fa flag
-is non-zero,
-editing is enabled (the default).
-Note that this is only an indication, and does not
-affect the operation of
-.Nm "" .
-At this time, it is the caller's responsibility to
-check this
-(using
-.Fn el_get )
-to determine if editing should be enabled or not.
-.It Dv EL_GETCFN , Fa "int (*f)(EditLine *, char *c)"
-Define the character reading function as
-.Fa f ,
-which is to return the number of characters read and store them in
-.Fa c .
-This function is called internally by
-.Fn el_gets
-and
-.Fn el_getc .
-The builtin function can be set or restored with the special function
-name ``EL_BUILTIN_GETCFN''.
-.It Dv EL_CLIENTDATA , Fa "void *data"
-Register
-.Fa data
-to be associated with this EditLine structure. It can be retrieved with
-the corresponding
-.Fn el_get
-call.
-.El
-.It Fn el_get
-Get
-.Nm
-parameters.
-.Fa op
-determines which parameter to retrieve into
-.Fa result .
-.Pp
-The following values for
-.Fa op
-are supported, along with actual type of
-.Fa result :
-.Bl -tag -width 4n
-.It Dv EL_PROMPT , Fa "char *(*f)(EditLine *)"
-Return a pointer to the function that displays the prompt.
-.It Dv EL_RPROMPT , Fa "char *(*f)(EditLine *)"
-Return a pointer to the function that displays the rightside prompt.
-.It Dv EL_EDITOR , Fa "const char *"
-Return the name of the editor, which will be one of
-.Dq emacs
-or
-.Dq vi .
-.It Dv EL_SIGNAL , Fa "int *"
-Return non-zero if
-.Nm
-has installed private signal handlers (see
-.Fn el_get
-above).
-.It Dv EL_EDITMODE, Fa "int *"
-Return non-zero if editing is enabled.
-.It Dv EL_GETCFN, Fa "int (**f)(EditLine *, char *)"
-Return a pointer to the function that read characters, which is equal to
-``EL_BUILTIN_GETCFN'' in the case of the default builtin function.
-.It Dv EL_CLIENTDATA , Fa "void **data"
-Retrieve
-.Fa data
-previously registered with the corresponding
-.Fn el_set
-call.
-.El
-.It Fn el_source
-Initialise
-.Nm
-by reading the contents of
-.Fa file .
-.Fn el_parse
-is called for each line in
-.Fa file .
-If
-.Fa file
-is
-.Dv NULL ,
-try
-.Pa $PWD/.editrc
-then
-.Pa $HOME/.editrc .
-Refer to
-.Xr editrc 5
-for details on the format of
-.Fa file .
-.It Fn el_resize
-Must be called if the terminal size changes.
-If
-.Dv EL_SIGNAL
-has been set with
-.Fn el_set ,
-then this is done automatically.
-Otherwise, it's the responsibility of the application to call
-.Fn el_resize
-on the appropriate occasions.
-.It Fn el_line
-Return the editing information for the current line in a
-.Fa LineInfo
-structure, which is defined as follows:
-.Bd -literal
-typedef struct lineinfo {
-    const char *buffer;    /* address of buffer */
-    const char *cursor;    /* address of cursor */
-    const char *lastchar;  /* address of last character */
-} LineInfo;
-.Ed
-.It Fn el_insertstr
-Insert
-.Fa str
-into the line at the cursor.
-Returns -1 if
-.Fa str
-is empty or won't fit, and 0 otherwise.
-.It Fn el_deletestr
-Delete
-.Fa num
-characters before the cursor.
-.El
-.Sh HISTORY LIST FUNCTIONS
-The history functions use a common data structure,
-.Fa History ,
-which is created by
-.Fn history_init
-and freed by
-.Fn history_end .
-.Pp
-The following functions are available:
-.Bl -tag -width 4n
-.It Fn history_init
-Initialise the history list, and return a data structure
-to be used by all other history list functions.
-.It Fn history_end
-Clean up and finish with
-.Fa h ,
-assumed to have been created with
-.Fn history_init .
-.It Fn history
-Perform operation
-.Fa op
-on the history list, with optional arguments as needed by the
-operation.
-.Fa ev
-is changed accordingly to operation.
-The following values for
-.Fa op
-are supported, along with the required argument list:
-.Bl -tag -width 4n
-.It Dv H_SETSIZE , Fa "int size"
-Set size of history to
-.Fa size
-elements.
-.It Dv H_GETSIZE
-Get number of events currently in history.
-.It Dv H_END
-Cleans up and finishes with
-.Fa h ,
-assumed to be created with
-.Fn history_init .
-.It Dv H_CLEAR
-Clear the history.
-.It Dv H_FUNC , Xo
-.Fa "void *ptr" ,
-.Fa "history_gfun_t first" ,
-.Fa "history_gfun_t next" ,
-.Fa "history_gfun_t last" ,
-.Fa "history_gfun_t prev" ,
-.Fa "history_gfun_t curr" ,
-.Fa "history_sfun_t set" ,
-.Fa "history_vfun_t clear" ,
-.Fa "history_efun_t enter" ,
-.Fa "history_efun_t add"
-.Xc
-Define functions to perform various history operations.
-.Fa ptr
-is the argument given to a function when it's invoked.
-.It Dv H_FIRST
-Return the first element in the history.
-.It Dv H_LAST
-Return the last element in the history.
-.It Dv H_PREV
-Return the previous element in the history.
-.It Dv H_NEXT
-Return the next element in the history.
-.It Dv H_CURR
-Return the current element in the history.
-.It Dv H_SET
-Set the cursor to point to the requested element.
-.It Dv H_ADD , Fa "const char *str"
-Append
-.Fa str
-to the current element of the history, or create an element with
-.It Dv H_APPEND , Fa "const char *str"
-Append
-.Fa str
-to the last new element of the history.
-.It Dv H_ENTER , Fa "const char *str"
-Add
-.Fa str
-as a new element to the history, and, if necessary,
-removing the oldest entry to keep the list to the created size.
-.It Dv H_PREV_STR , Fa "const char *str"
-Return the closest previous event that starts with
-.Fa str .
-.It Dv H_NEXT_STR , Fa "const char *str"
-Return the closest next event that starts with
-.Fa str .
-.It Dv H_PREV_EVENT , Fa "int e"
-Return the previous event numbered
-.Fa e .
-.It Dv H_NEXT_EVENT , Fa "int e"
-Return the next event numbered
-.Fa e .
-.It Dv H_LOAD , Fa "const char *file"
-Load the history list stored in
-.Fa file .
-.It Dv H_SAVE , Fa "const char *file"
-Save the history list to
-.Fa file .
-.El
-.Pp
-.Fn history
-returns 0 if the operation
-.Fa op
-succeeds. Otherwise, -1 is returned and
-.Fa ev
-is updated to contain more details about the error.
-.El
-.\"XXX.Sh EXAMPLES
-.\"XXX: provide some examples
-.Sh SEE ALSO
-.Xr sh 1 ,
-.Xr signal 3 ,
-.Xr termcap 3 ,
-.Xr editrc 5
-.Sh HISTORY
-The
-.Nm
-library first appeared in
-.Bx 4.4 .
-.Dv CC_REDISPLAY
-appeared in
-.Nx 1.3 .
-.Dv CC_REFRESH_BEEP ,
-.Dv EL_EDITMODE
-and the readline emulation appeared in
-.Nx 1.4 .
-.Dv EL_RPROMPT
-appeared in
-.Nx 1.5 .
-.Sh AUTHORS
-The
-.Nm
-library was written by Christos Zoulas.
-Luke Mewburn wrote this manual and implemented
-.Dv CC_REDISPLAY ,
-.Dv CC_REFRESH_BEEP ,
-.Dv EL_EDITMODE ,
-and
-.Dv EL_RPROMPT .
-Jaromir Dolecek implemented the readline emulation.
-.Sh BUGS
-The tokenization functions are not publicly defined in
-.Fd <histedit.h> .
-.Pp
-At this time, it is the responsibility of the caller to
-check the result of the
-.Dv EL_EDITMODE
-operation of
-.Fn el_get
-(after an
-.Fn el_source
-or
-.Fn el_parse )
-to determine if
-.Nm
-should be used for further input.
-I.e.,
-.Dv EL_EDITMODE
-is purely an indication of the result of the most recent
-.Xr editrc 5
-.Ic edit
-command.
diff -pruN 1:13.23.1~dfsg-2/main/editline/editrc.5 1:16.1.1~dfsg-1/main/editline/editrc.5
--- 1:13.23.1~dfsg-2/main/editline/editrc.5	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/editrc.5	1970-01-01 00:00:00.000000000 +0000
@@ -1,491 +0,0 @@
-.\"	$NetBSD: editrc.5,v 1.12 2002/01/15 02:46:44 wiz Exp $
-.\"
-.\" Copyright (c) 1997-2000 The NetBSD Foundation, Inc.
-.\" All rights reserved.
-.\"
-.\" This file was contributed to The NetBSD Foundation by Luke Mewburn.
-.\"
-.\" Redistribution and use in source and binary forms, with or without
-.\" modification, are permitted provided that the following conditions
-.\" are met:
-.\" 1. Redistributions of source code must retain the above copyright
-.\"    notice, this list of conditions and the following disclaimer.
-.\" 2. Redistributions in binary form must reproduce the above copyright
-.\"    notice, this list of conditions and the following disclaimer in the
-.\"    documentation and/or other materials provided with the distribution.
-.\" 3. All advertising materials mentioning features or use of this software
-.\"    must display the following acknowledgement:
-.\"        This product includes software developed by the NetBSD
-.\"        Foundation, Inc. and its contributors.
-.\" 4. Neither the name of The NetBSD Foundation nor the names of its
-.\"    contributors may be used to endorse or promote products derived
-.\"    from this software without specific prior written permission.
-.\"
-.\" THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
-.\" ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-.\" TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-.\" PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
-.\" BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-.\" CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-.\" SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-.\" INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-.\" CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-.\" ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-.\" POSSIBILITY OF SUCH DAMAGE.
-.\"
-.Dd November 8, 2000
-.Os
-.Dt EDITRC 5
-.Sh NAME
-.Nm editrc
-.Nd configuration file for editline library
-.Sh SYNOPSIS
-.Nm
-.Sh DESCRIPTION
-The
-.Nm
-file defines various settings to be used by the
-.Xr editline 3
-library.
-.Pp
-The format of each line is:
-.Dl [prog:]command [arg [...]]
-.Pp
-.Ar command
-is one of the
-.Xr editline 3
-builtin commands.
-Refer to
-.Sx BUILTIN COMMANDS
-for more information.
-.Pp
-.Ar prog
-is the program name string that a program defines when it calls
-.Xr el_init 3
-to setup
-.Xr editline 3 ,
-which is usually
-.Va argv[0] .
-.Ar command
-will be executed for any program which matches
-.Ar prog .
-.Pp
-.Ar prog
-may also be a
-.Xr regex 3
-style
-regular expression, in which case
-.Ar command
-will be executed for any program that matches the regular expression.
-.Pp
-If
-.Ar prog
-is absent,
-.Ar command
-is executed for all programs.
-.Sh BUILTIN COMMANDS
-The
-.Nm editline
-library has some builtin commands, which affect the way
-that the line editing and history functions operate.
-These are based on similar named builtins present in the
-.Xr tcsh 1
-shell.
-.Pp
-The following builtin commands are available:
-.Bl -tag -width 4n
-.It Ic bind Xo
-.Op Fl a
-.Op Fl e
-.Op Fl k
-.Op Fl l
-.Op Fl r
-.Op Fl s
-.Op Fl v
-.Op Ar key Op Ar command
-.Xc
-Without options, list all bound keys, and the editor command to which
-each is bound.
-If
-.Ar key
-is supplied, show the bindings for
-.Ar key .
-If
-.Ar key command
-is supplied, bind
-.Ar command
-to
-.Ar key .
-Options include:
-.Bl -tag -width 4n
-.It Fl e
-Bind all keys to the standard GNU Emacs-like bindings.
-.It Fl v
-Bind all keys to the standard
-.Xr vi 1 -like
-bindings.
-.It Fl a
-List or change key bindings in the
-.Xr vi 1
-mode alternate (command mode) key map.
-.It Fl k
-.Ar key
-is interpreted as a symbolic arrow key name, which may be one of
-.Sq up ,
-.Sq down ,
-.Sq left
-or
-.Sq right .
-.It Fl l
-List all editor commands and a short description of each.
-.It Fl r
-Remove a key's binding.
-.It Fl s
-.Ar command
-is taken as a literal string and treated as terminal input when
-.Ar key
-is typed.
-Bound keys in
-.Ar command
-are themselves reinterpreted, and this continues for ten levels of
-interpretation.
-.El
-.Pp
-.Ar command
-may be one of the commands documented in
-.Sx "EDITOR COMMANDS"
-below, or another key.
-.Pp
-.Ar key
-and
-.Ar command
-can contain control characters of the form
-.Sm off
-.Sq No ^ Ar character
-.Sm on
-.Po
-e.g.
-.Sq ^A
-.Pc ,
-and the following backslashed escape sequences:
-.Pp
-.Bl -tag -compact -offset indent -width 4n
-.It Ic \ea
-Bell
-.It Ic \eb
-Backspace
-.It Ic \ee
-Escape
-.It Ic \ef
-Formfeed
-.It Ic \en
-Newline
-.It Ic \er
-Carriage return
-.It Ic \et
-Horizontal tab
-.It Ic \ev
-Vertical tab
-.Sm off
-.It Sy \e Ar nnn
-.Sm on
-The ASCII character corresponding to the octal number
-.Ar nnn .
-.El
-.Pp
-.Sq \e
-nullifies the special meaning of the following character,
-if it has any, notably
-.Sq \e
-and
-.Sq ^ .
-.It Ic echotc Xo
-.Op Fl sv
-.Ar arg
-.Ar ...
-.Xc
-Exercise terminal capabilities given in
-.Ar arg Ar ... .
-If
-.Ar arg
-is
-.Sq baud ,
-.Sq cols ,
-.Sq lines ,
-.Sq rows ,
-.Sq meta or
-.Sq tabs ,
-the value of that capability is printed, with
-.Dq yes
-or
-.Dq no
-indicating that the terminal does or does not have that capability.
-.Pp
-.Fl s
-returns an emptry string for non-existent capabilities, rather than
-causing an error.
-.Fl v
-causes messages to be verbose.
-.It Ic edit Op Li on | Li off
-Enable or disable the
-.Nm editline
-functionality in a program.
-.It Ic history
-List the history.
-.It Ic telltc
-List the values of all the terminal capabilities (see
-.Xr termcap 5 ) .
-.It Ic settc Ar cap Ar val
-Set the terminal capability
-.Ar cap
-to
-.Ar val ,
-as defined in
-.Xr termcap 5 .
-No sanity checking is done.
-.It Ic setty Xo
-.Op Fl a
-.Op Fl d
-.Op Fl q
-.Op Fl x
-.Op Ar +mode
-.Op Ar -mode
-.Op Ar mode
-.Xc
-Control which tty modes that
-.Nm
-won't allow the user to change.
-.Fl d ,
-.Fl q
-or
-.Fl x
-tells
-.Ic setty
-to act on the
-.Sq edit ,
-.Sq quote
-or
-.Sq execute
-set of tty modes respectively; defaulting to
-.Fl x .
-.Pp
-Without other arguments,
-.Ic setty
-lists the modes in the chosen set which are fixed on
-.Po
-.Sq +mode
-.Pc
-or off
-.Po
-.Sq -mode
-.Pc .
-.Fl a
-lists all tty modes in the chosen set regardless of the setting.
-With
-.Ar +mode ,
-.Ar -mode
-or
-.Ar mode ,
-fixes
-.Ar mode
-on or off or removes control of
-.Ar mode
-in the chosen set.
-.El
-.Sh EDITOR COMMANDS
-The following editor commands are available for use in key bindings:
-.\" Section automatically generated with makelist
-.Bl -tag -width 4n
-.It Ic vi-paste-next
-Vi paste previous deletion to the right of the cursor.
-.It Ic vi-paste-prev
-Vi paste previous deletion to the left of the cursor.
-.It Ic vi-prev-space-word
-Vi move to the previous space delimited word.
-.It Ic vi-prev-word
-Vi move to the previous word.
-.It Ic vi-next-space-word
-Vi move to the next space delimited word.
-.It Ic vi-next-word
-Vi move to the next word.
-.It Ic vi-change-case
-Vi change case of character under the cursor and advance one character.
-.It Ic vi-change-meta
-Vi change prefix command.
-.It Ic vi-insert-at-bol
-Vi enter insert mode at the beginning of line.
-.It Ic vi-replace-char
-Vi replace character under the cursor with the next character typed.
-.It Ic vi-replace-mode
-Vi enter replace mode.
-.It Ic vi-substitute-char
-Vi replace character under the cursor and enter insert mode.
-.It Ic vi-substitute-line
-Vi substitute entire line.
-.It Ic vi-change-to-eol
-Vi change to end of line.
-.It Ic vi-insert
-Vi enter insert mode.
-.It Ic vi-add
-Vi enter insert mode after the cursor.
-.It Ic vi-add-at-eol
-Vi enter insert mode at end of line.
-.It Ic vi-delete-meta
-Vi delete prefix command.
-.It Ic vi-end-word
-Vi move to the end of the current space delimited word.
-.It Ic vi-to-end-word
-Vi move to the end of the current word.
-.It Ic vi-undo
-Vi undo last change.
-.It Ic vi-command-mode
-Vi enter command mode (use alternative key bindings).
-.It Ic vi-zero
-Vi move to the beginning of line.
-.It Ic vi-delete-prev-char
-Vi move to previous character (backspace).
-.It Ic vi-list-or-eof
-Vi list choices for completion or indicate end of file if empty line.
-.It Ic vi-kill-line-prev
-Vi cut from beginning of line to cursor.
-.It Ic vi-search-prev
-Vi search history previous.
-.It Ic vi-search-next
-Vi search history next.
-.It Ic vi-repeat-search-next
-Vi repeat current search in the same search direction.
-.It Ic vi-repeat-search-prev
-Vi repeat current search in the opposite search direction.
-.It Ic vi-next-char
-Vi move to the character specified next.
-.It Ic vi-prev-char
-Vi move to the character specified previous.
-.It Ic vi-to-next-char
-Vi move up to the character specified next.
-.It Ic vi-to-prev-char
-Vi move up to the character specified previous.
-.It Ic vi-repeat-next-char
-Vi repeat current character search in the same search direction.
-.It Ic vi-repeat-prev-char
-Vi repeat current character search in the opposite search direction.
-.It Ic em-delete-or-list
-Delete character under cursor or list completions if at end of line.
-.It Ic em-delete-next-word
-Cut from cursor to end of current word.
-.It Ic em-yank
-Paste cut buffer at cursor position.
-.It Ic em-kill-line
-Cut the entire line and save in cut buffer.
-.It Ic em-kill-region
-Cut area between mark and cursor and save in cut buffer.
-.It Ic em-copy-region
-Copy area between mark and cursor to cut buffer.
-.It Ic em-gosmacs-traspose
-Exchange the two characters before the cursor.
-.It Ic em-next-word
-Move next to end of current word.
-.It Ic em-upper-case
-Uppercase the characters from cursor to end of current word.
-.It Ic em-capitol-case
-Capitalize the characters from cursor to end of current word.
-.It Ic em-lower-case
-Lowercase the characters from cursor to end of current word.
-.It Ic em-set-mark
-Set the mark at cursor.
-.It Ic em-exchange-mark
-Exchange the cursor and mark.
-.It Ic em-universal-argument
-Universal argument (argument times 4).
-.It Ic em-meta-next
-Add 8th bit to next character typed.
-.It Ic em-toggle-overwrite
-Switch from insert to overwrite mode or vice versa.
-.It Ic em-copy-prev-word
-Copy current word to cursor.
-.It Ic em-inc-search-next
-Emacs incremental next search.
-.It Ic em-inc-search-prev
-Emacs incremental reverse search.
-.It Ic ed-end-of-file
-Indicate end of file.
-.It Ic ed-insert
-Add character to the line.
-.It Ic ed-delete-prev-word
-Delete from beginning of current word to cursor.
-.It Ic ed-delete-next-char
-Delete character under cursor.
-.It Ic ed-kill-line
-Cut to the end of line.
-.It Ic ed-move-to-end
-Move cursor to the end of line.
-.It Ic ed-move-to-beg
-Move cursor to the beginning of line.
-.It Ic ed-transpose-chars
-Exchange the character to the left of the cursor with the one under it.
-.It Ic ed-next-char
-Move to the right one character.
-.It Ic ed-prev-word
-Move to the beginning of the current word.
-.It Ic ed-prev-char
-Move to the left one character.
-.It Ic ed-quoted-insert
-Add the next character typed verbatim.
-.It Ic ed-digit
-Adds to argument or enters a digit.
-.It Ic ed-argument-digit
-Digit that starts argument.
-.It Ic ed-unassigned
-Indicates unbound character.
-.It Ic ed-tty-sigint
-Tty interrupt character.
-.It Ic ed-tty-dsusp
-Tty delayed suspend character.
-.It Ic ed-tty-flush-output
-Tty flush output characters.
-.It Ic ed-tty-sigquit
-Tty quit character.
-.It Ic ed-tty-sigtstp
-Tty suspend character.
-.It Ic ed-tty-stop-output
-Tty disallow output characters.
-.It Ic ed-tty-start-output
-Tty allow output characters.
-.It Ic ed-newline
-Execute command.
-.It Ic ed-delete-prev-char
-Delete the character to the left of the cursor.
-.It Ic ed-clear-screen
-Clear screen leaving current line at the top.
-.It Ic ed-redisplay
-Redisplay everything.
-.It Ic ed-start-over
-Erase current line and start from scratch.
-.It Ic ed-sequence-lead-in
-First character in a bound sequence.
-.It Ic ed-prev-history
-Move to the previous history line.
-.It Ic ed-next-history
-Move to the next history line.
-.It Ic ed-search-prev-history
-Search previous in history for a line matching the current.
-.It Ic ed-search-next-history
-Search next in history for a line matching the current.
-.It Ic ed-prev-line
-Move up one line.
-.It Ic ed-next-line
-Move down one line.
-.It Ic ed-command
-Editline extended command.
-.El
-.\" End of section automatically generated with makelist
-.Sh SEE ALSO
-.Xr editline 3 ,
-.Xr regex 3 ,
-.Xr termcap 5
-.Sh AUTHORS
-The
-.Nm editline
-library was written by Christos Zoulas,
-and this manual was written by Luke Mewburn,
-with some sections inspired by
-.Xr tcsh 1 .
diff -pruN 1:13.23.1~dfsg-2/main/editline/el.c 1:16.1.1~dfsg-1/main/editline/el.c
--- 1:13.23.1~dfsg-2/main/editline/el.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/el.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,509 +0,0 @@
-/*	$NetBSD: el.c,v 1.29 2002/03/18 16:00:52 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)el.c	8.2 (Berkeley) 1/3/94";
-#else
-__RCSID("$NetBSD: el.c,v 1.29 2002/03/18 16:00:52 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * el.c: EditLine interface functions
- */
-#include <sys/types.h>
-#include <sys/param.h>
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include "el.h"
-
-/* el_init():
- *	Initialize editline and set default parameters.
- */
-public EditLine *
-el_init(const char *prog, FILE *fin, FILE *fout, FILE *ferr)
-{
-
-	EditLine *el = (EditLine *) el_malloc(sizeof(EditLine));
-
-	if (el == NULL)
-		return (NULL);
-
-	memset(el, 0, sizeof(EditLine));
-
-	el->el_infd = fileno(fin);
-	el->el_outfile = fout;
-	el->el_errfile = ferr;
-	el->el_prog = strdup(prog);
-
-	/*
-         * Initialize all the modules. Order is important!!!
-         */
-	el->el_flags = 0;
-
-	if (term_init(el) == -1) {
-		free(el->el_prog);
-		el_free(el);
-		return NULL;
-	}
-	(void) key_init(el);
-	(void) map_init(el);
-	if (tty_init(el) == -1)
-		el->el_flags |= NO_TTY;
-	(void) ch_init(el);
-	(void) search_init(el);
-	(void) hist_init(el);
-	(void) prompt_init(el);
-	(void) sig_init(el);
-	(void) read_init(el);
-
-	return (el);
-}
-
-
-/* el_end():
- *	Clean up.
- */
-public void
-el_end(EditLine *el)
-{
-
-	if (el == NULL)
-		return;
-
-	el_reset(el);
-
-	term_end(el);
-	key_end(el);
-	map_end(el);
-	tty_end(el);
-	ch_end(el);
-	search_end(el);
-	hist_end(el);
-	prompt_end(el);
-	sig_end(el);
-
-	if (el->el_prog)
-		el_free((ptr_t) el->el_prog);
-	el_free((ptr_t) el);
-}
-
-
-/* el_reset():
- *	Reset the tty and the parser
- */
-public void
-el_reset(EditLine *el)
-{
-
-	tty_cookedmode(el);
-	ch_reset(el);		/* XXX: Do we want that? */
-}
-
-
-/* el_set():
- *	set the editline parameters
- */
-public int
-el_set(EditLine *el, int op, ...)
-{
-	va_list va;
-	int rv = 0;
-
-	if (el == NULL)
-		return (-1);
-	va_start(va, op);
-
-	switch (op) {
-	case EL_PROMPT:
-	case EL_RPROMPT:
-		rv = prompt_set(el, va_arg(va, el_pfunc_t), op);
-		break;
-
-	case EL_TERMINAL:
-		rv = term_set(el, va_arg(va, char *));
-		break;
-
-	case EL_EDITOR:
-		rv = map_set_editor(el, va_arg(va, char *));
-		break;
-
-	case EL_SIGNAL:
-		if (va_arg(va, int))
-			el->el_flags |= HANDLE_SIGNALS;
-		else
-			el->el_flags &= ~HANDLE_SIGNALS;
-		break;
-
-	case EL_BIND:
-	case EL_TELLTC:
-	case EL_SETTC:
-	case EL_ECHOTC:
-	case EL_SETTY:
-	{
-		const char *argv[20];
-		int i;
-
-		for (i = 1; i < 20; i++)
-			if ((argv[i] = va_arg(va, char *)) == NULL)
-				break;
-
-		switch (op) {
-		case EL_BIND:
-			argv[0] = "bind";
-			rv = map_bind(el, i, argv);
-			break;
-
-		case EL_TELLTC:
-			argv[0] = "telltc";
-			rv = term_telltc(el, i, argv);
-			break;
-
-		case EL_SETTC:
-			argv[0] = "settc";
-			rv = term_settc(el, i, argv);
-			break;
-
-		case EL_ECHOTC:
-			argv[0] = "echotc";
-			rv = term_echotc(el, i, argv);
-			break;
-
-		case EL_SETTY:
-			argv[0] = "setty";
-			rv = tty_stty(el, i, argv);
-			break;
-
-		default:
-			rv = -1;
-			EL_ABORT((el->el_errfile, "Bad op %d\n", op));
-			break;
-		}
-		break;
-	}
-
-	case EL_ADDFN:
-	{
-		char *name = va_arg(va, char *);
-		char *help = va_arg(va, char *);
-		el_func_t func = va_arg(va, el_func_t);
-
-		rv = map_addfunc(el, name, help, func);
-		break;
-	}
-
-	case EL_HIST:
-	{
-		hist_fun_t func = va_arg(va, hist_fun_t);
-		ptr_t ptr = va_arg(va, char *);
-
-		rv = hist_set(el, func, ptr);
-		break;
-	}
-
-	case EL_EDITMODE:
-		if (va_arg(va, int))
-			el->el_flags &= ~EDIT_DISABLED;
-		else
-			el->el_flags |= EDIT_DISABLED;
-		rv = 0;
-		break;
-
-	case EL_GETCFN:
-	{
-		el_rfunc_t rc = va_arg(va, el_rfunc_t);
-		rv = el_read_setfn(el, rc);
-		break;
-	}
-
-	case EL_CLIENTDATA:
-		el->el_data = va_arg(va, void *);
-		break;
-
-	default:
-		rv = -1;
-		break;
-	}
-
-	va_end(va);
-	return (rv);
-}
-
-
-/* el_get():
- *	retrieve the editline parameters
- */
-public int
-el_get(EditLine *el, int op, void *ret)
-{
-	int rv;
-
-	if (el == NULL || ret == NULL)
-		return (-1);
-	switch (op) {
-	case EL_PROMPT:
-	case EL_RPROMPT:
-		rv = prompt_get(el, (el_pfunc_t *) & ret, op);
-		break;
-
-	case EL_EDITOR:
-		rv = map_get_editor(el, (const char **) &ret);
-		break;
-
-	case EL_SIGNAL:
-		*((int *) ret) = (el->el_flags & HANDLE_SIGNALS);
-		rv = 0;
-		break;
-
-	case EL_EDITMODE:
-		*((int *) ret) = (!(el->el_flags & EDIT_DISABLED));
-		rv = 0;
-		break;
-
-#if 0				/* XXX */
-	case EL_TERMINAL:
-		rv = term_get(el, (const char *) &ret);
-		break;
-
-	case EL_BIND:
-	case EL_TELLTC:
-	case EL_SETTC:
-	case EL_ECHOTC:
-	case EL_SETTY:
-	{
-		char *argv[20];
-		int i;
-
-		for (i = 1; i < 20; i++)
-			if ((argv[i] = va_arg(va, char *)) == NULL)
-				break;
-
-		switch (op) {
-		case EL_BIND:
-			argv[0] = "bind";
-			rv = map_bind(el, i, argv);
-			break;
-
-		case EL_TELLTC:
-			argv[0] = "telltc";
-			rv = term_telltc(el, i, argv);
-			break;
-
-		case EL_SETTC:
-			argv[0] = "settc";
-			rv = term_settc(el, i, argv);
-			break;
-
-		case EL_ECHOTC:
-			argv[0] = "echotc";
-			rv = term_echotc(el, i, argv);
-			break;
-
-		case EL_SETTY:
-			argv[0] = "setty";
-			rv = tty_stty(el, i, argv);
-			break;
-
-		default:
-			rv = -1;
-			EL_ABORT((el->errfile, "Bad op %d\n", op));
-			break;
-		}
-		break;
-	}
-
-	case EL_ADDFN:
-	{
-		char *name = va_arg(va, char *);
-		char *help = va_arg(va, char *);
-		el_func_t func = va_arg(va, el_func_t);
-
-		rv = map_addfunc(el, name, help, func);
-		break;
-	}
-
-	case EL_HIST:
-		{
-			hist_fun_t func = va_arg(va, hist_fun_t);
-			ptr_t ptr = va_arg(va, char *);
-			rv = hist_set(el, func, ptr);
-		}
-		break;
-#endif /* XXX */
-
-	case EL_GETCFN:
-		*((el_rfunc_t *)ret) = el_read_getfn(el);
-		rv = 0;
-		break;
-
-	case EL_CLIENTDATA:
-		*((void **)ret) = el->el_data;
-		rv = 0;
-		break;
-
-	default:
-		rv = -1;
-	}
-
-	return (rv);
-}
-
-
-/* el_line():
- *	Return editing info
- */
-public const LineInfo *
-el_line(EditLine *el)
-{
-
-	return (const LineInfo *) (void *) &el->el_line;
-}
-
-
-/* el_source():
- *	Source a file
- */
-public int
-el_source(EditLine *el, const char *fname)
-{
-	FILE *fp;
-	size_t len;
-	char *ptr;
-
-	fp = NULL;
-	if (fname == NULL) {
-#ifdef HAVE_ISSETUGID
-		static const char elpath[] = "/.editrc";
-		char path[MAXPATHLEN];
-
-		if (issetugid())
-			return (-1);
-		if ((ptr = getenv("HOME")) == NULL)
-			return (-1);
-		if (strlcpy(path, ptr, sizeof(path)) >= sizeof(path))
-			return (-1);
-		if (strlcat(path, elpath, sizeof(path)) >= sizeof(path))
-			return (-1);
-		fname = path;
-#else
-		/*
-		 * If issetugid() is missing, always return an error, in order
-		 * to keep from inadvertently opening up the user to a security
-		 * hole.
-		 */
-		return (-1);
-#endif
-	}
-	if (fp == NULL)
-		fp = fopen(fname, "r");
-	if (fp == NULL)
-		return (-1);
-
-	while ((ptr = fgetln(fp, &len)) != NULL) {
-		if (len > 0 && ptr[len - 1] == '\n')
-			--len;
-		ptr[len] = '\0';
-		if (parse_line(el, ptr) == -1) {
-			(void) fclose(fp);
-			return (-1);
-		}
-	}
-
-	(void) fclose(fp);
-	return (0);
-}
-
-
-/* el_resize():
- *	Called from program when terminal is resized
- */
-public void
-el_resize(EditLine *el)
-{
-	int lins, cols;
-	sigset_t oset, nset;
-
-	(void) sigemptyset(&nset);
-	(void) sigaddset(&nset, SIGWINCH);
-	(void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-	/* get the correct window size */
-	if (term_get_size(el, &lins, &cols))
-		term_change_size(el, lins, cols);
-
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-}
-
-
-/* el_beep():
- *	Called from the program to beep
- */
-public void
-el_beep(EditLine *el)
-{
-
-	term_beep(el);
-}
-
-
-/* el_editmode()
- *	Set the state of EDIT_DISABLED from the `edit' command.
- */
-protected int
-/*ARGSUSED*/
-el_editmode(EditLine *el, int argc, const char **argv)
-{
-	const char *how;
-
-	if (argv == NULL || argc != 2 || argv[1] == NULL)
-		return (-1);
-
-	how = argv[1];
-	if (strcmp(how, "on") == 0)
-		el->el_flags &= ~EDIT_DISABLED;
-	else if (strcmp(how, "off") == 0)
-		el->el_flags |= EDIT_DISABLED;
-	else {
-		(void) fprintf(el->el_errfile, "edit: Bad value `%s'.\n", how);
-		return (-1);
-	}
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/el.h 1:16.1.1~dfsg-1/main/editline/el.h
--- 1:13.23.1~dfsg-2/main/editline/el.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/el.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,145 +0,0 @@
-/*	$NetBSD: el.h,v 1.11 2002/03/18 16:00:52 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)el.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.h: Internal structures.
- */
-#ifndef _h_el
-#define	_h_el
-/*
- * Local defaults
- */
-#define	KSHVI
-#define	VIDEFAULT
-#define	ANCHOR
-
-#include <stdio.h>
-#include <sys/types.h>
-
-#define	EL_BUFSIZ	1024		/* Maximum line size		*/
-
-#define	HANDLE_SIGNALS	0x01
-#define	NO_TTY		0x02
-#define	EDIT_DISABLED	0x04
-
-typedef int bool_t;			/* True or not			*/
-
-typedef unsigned char el_action_t;	/* Index to command array	*/
-
-typedef struct coord_t {		/* Position on the screen	*/
-	int	h;
-	int	v;
-} coord_t;
-
-typedef struct el_line_t {
-	char	*buffer;		/* Input line			*/
-	char	*cursor;		/* Cursor position		*/
-	char	*lastchar;		/* Last character		*/
-	const char	*limit;			/* Max position			*/
-} el_line_t;
-
-/*
- * Editor state
- */
-typedef struct el_state_t {
-	int		inputmode;	/* What mode are we in?		*/
-	int		doingarg;	/* Are we getting an argument?	*/
-	int		argument;	/* Numeric argument		*/
-	int		metanext;	/* Is the next char a meta char */
-	el_action_t	lastcmd;	/* Previous command		*/
-} el_state_t;
-
-/*
- * Until we come up with something better...
- */
-#define	el_malloc(a)	malloc(a)
-#define	el_realloc(a,b)	realloc(a, b)
-#define	el_free(a)	free(a)
-
-#include "tty.h"
-#include "prompt.h"
-#include "key.h"
-#include "term.h"
-#include "refresh.h"
-#include "chared.h"
-#include "common.h"
-#include "search.h"
-#include "hist.h"
-#include "map.h"
-#include "parse.h"
-#include "sig.h"
-#include "help.h"
-#include "read.h"
-
-struct editline {
-	char		 *el_prog;	/* the program name		*/
-	FILE		 *el_outfile;	/* Stdio stuff			*/
-	FILE		 *el_errfile;	/* Stdio stuff			*/
-	int		  el_infd;	/* Input file descriptor	*/
-	int		  el_flags;	/* Various flags.		*/
-	coord_t		  el_cursor;	/* Cursor location		*/
-	char		**el_display;	/* Real screen image = what is there */
-	char		**el_vdisplay;	/* Virtual screen image = what we see */
-	void		 *el_data;	/* Client data			*/
-	el_line_t	  el_line;	/* The current line information	*/
-	el_state_t	  el_state;	/* Current editor state		*/
-	el_term_t	  el_term;	/* Terminal dependent stuff	*/
-	el_tty_t	  el_tty;	/* Tty dependent stuff		*/
-	el_refresh_t	  el_refresh;	/* Refresh stuff		*/
-	el_prompt_t	  el_prompt;	/* Prompt stuff			*/
-	el_prompt_t	  el_rprompt;	/* Prompt stuff			*/
-	el_chared_t	  el_chared;	/* Characted editor stuff	*/
-	el_map_t	  el_map;	/* Key mapping stuff		*/
-	el_key_t	  el_key;	/* Key binding stuff		*/
-	el_history_t	  el_history;	/* History stuff		*/
-	el_search_t	  el_search;	/* Search stuff			*/
-	el_signal_t	  el_signal;	/* Signal handling stuff	*/
-	el_read_t	  el_read;	/* Character reading stuff	*/
-};
-
-protected int	el_editmode(EditLine *, int, const char **);
-
-#ifdef DEBUG
-#define EL_ABORT(a)	(void) (fprintf(el->el_errfile, "%s, %d: ", \
-				__FILE__, __LINE__), fprintf a, abort())
-#else
-#define EL_ABORT(a)	abort()
-#endif
-#endif /* _h_el */
diff -pruN 1:13.23.1~dfsg-2/main/editline/emacs.c 1:16.1.1~dfsg-1/main/editline/emacs.c
--- 1:13.23.1~dfsg-2/main/editline/emacs.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/emacs.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,488 +0,0 @@
-/*	$NetBSD: emacs.c,v 1.10 2002/03/18 16:00:52 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)emacs.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: emacs.c,v 1.10 2002/03/18 16:00:52 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * emacs.c: Emacs functions
- */
-#include "el.h"
-
-/* em_delete_or_list():
- *	Delete character under cursor or list completions if at end of line
- *	[^D]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_delete_or_list(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.lastchar) {
-					/* if I'm at the end */
-		if (el->el_line.cursor == el->el_line.buffer) {
-					/* and the beginning */
-			term_overwrite(el, STReof, 4);	/* then do a EOF */
-			term__flush();
-			return (CC_EOF);
-		} else {
-			/*
-			 * Here we could list completions, but it is an
-			 * error right now
-			 */
-			term_beep(el);
-			return (CC_ERROR);
-		}
-	} else {
-		c_delafter(el, el->el_state.argument);	/* delete after dot */
-		if (el->el_line.cursor > el->el_line.lastchar)
-			el->el_line.cursor = el->el_line.lastchar;
-				/* bounds check */
-		return (CC_REFRESH);
-	}
-}
-
-
-/* em_delete_next_word():
- *	Cut from cursor to end of current word
- *	[M-d]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_delete_next_word(EditLine *el, int c)
-{
-	char *cp, *p, *kp;
-
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	cp = c__next_word(el->el_line.cursor, el->el_line.lastchar,
-	    el->el_state.argument, ce__isword);
-
-	for (p = el->el_line.cursor, kp = el->el_chared.c_kill.buf; p < cp; p++)
-				/* save the text */
-		*kp++ = *p;
-	el->el_chared.c_kill.last = kp;
-
-	c_delafter(el, cp - el->el_line.cursor);	/* delete after dot */
-	if (el->el_line.cursor > el->el_line.lastchar)
-		el->el_line.cursor = el->el_line.lastchar;
-				/* bounds check */
-	return (CC_REFRESH);
-}
-
-
-/* em_yank():
- *	Paste cut buffer at cursor position
- *	[^Y]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_yank(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	if (el->el_chared.c_kill.last == el->el_chared.c_kill.buf) {
-		if (!ch_enlargebufs(el, 1))
-			return (CC_ERROR);
-	}
-
-	if (el->el_line.lastchar +
-	    (el->el_chared.c_kill.last - el->el_chared.c_kill.buf) >=
-	    el->el_line.limit)
-		return (CC_ERROR);
-
-	el->el_chared.c_kill.mark = el->el_line.cursor;
-	cp = el->el_line.cursor;
-
-	/* open the space, */
-	c_insert(el, el->el_chared.c_kill.last - el->el_chared.c_kill.buf);
-	/* copy the chars */
-	for (kp = el->el_chared.c_kill.buf; kp < el->el_chared.c_kill.last; kp++)
-		*cp++ = *kp;
-
-	/* if an arg, cursor at beginning else cursor at end */
-	if (el->el_state.argument == 1)
-		el->el_line.cursor = cp;
-
-	return (CC_REFRESH);
-}
-
-
-/* em_kill_line():
- *	Cut the entire line and save in cut buffer
- *	[^U]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_kill_line(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	cp = el->el_line.buffer;
-	kp = el->el_chared.c_kill.buf;
-	while (cp < el->el_line.lastchar)
-		*kp++ = *cp++;	/* copy it */
-	el->el_chared.c_kill.last = kp;
-				/* zap! -- delete all of it */
-	el->el_line.lastchar = el->el_line.buffer;
-	el->el_line.cursor = el->el_line.buffer;
-	return (CC_REFRESH);
-}
-
-
-/* em_kill_region():
- *	Cut area between mark and cursor and save in cut buffer
- *	[^W]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_kill_region(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	if (!el->el_chared.c_kill.mark)
-		return (CC_ERROR);
-
-	if (el->el_chared.c_kill.mark > el->el_line.cursor) {
-		cp = el->el_line.cursor;
-		kp = el->el_chared.c_kill.buf;
-		while (cp < el->el_chared.c_kill.mark)
-			*kp++ = *cp++;	/* copy it */
-		el->el_chared.c_kill.last = kp;
-		c_delafter(el, cp - el->el_line.cursor);
-	} else {		/* mark is before cursor */
-		cp = el->el_chared.c_kill.mark;
-		kp = el->el_chared.c_kill.buf;
-		while (cp < el->el_line.cursor)
-			*kp++ = *cp++;	/* copy it */
-		el->el_chared.c_kill.last = kp;
-		c_delbefore(el, cp - el->el_chared.c_kill.mark);
-		el->el_line.cursor = el->el_chared.c_kill.mark;
-	}
-	return (CC_REFRESH);
-}
-
-
-/* em_copy_region():
- *	Copy area between mark and cursor to cut buffer
- *	[M-W]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_copy_region(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	if (el->el_chared.c_kill.mark)
-		return (CC_ERROR);
-
-	if (el->el_chared.c_kill.mark > el->el_line.cursor) {
-		cp = el->el_line.cursor;
-		kp = el->el_chared.c_kill.buf;
-		while (cp < el->el_chared.c_kill.mark)
-			*kp++ = *cp++;	/* copy it */
-		el->el_chared.c_kill.last = kp;
-	} else {
-		cp = el->el_chared.c_kill.mark;
-		kp = el->el_chared.c_kill.buf;
-		while (cp < el->el_line.cursor)
-			*kp++ = *cp++;	/* copy it */
-		el->el_chared.c_kill.last = kp;
-	}
-	return (CC_NORM);
-}
-
-
-/* em_gosmacs_traspose():
- *	Exchange the two characters before the cursor
- *	Gosling emacs transpose chars [^T]
- */
-protected el_action_t
-em_gosmacs_traspose(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor > &el->el_line.buffer[1]) {
-		/* must have at least two chars entered */
-		c = el->el_line.cursor[-2];
-		el->el_line.cursor[-2] = el->el_line.cursor[-1];
-		el->el_line.cursor[-1] = c;
-		return (CC_REFRESH);
-	} else
-		return (CC_ERROR);
-}
-
-
-/* em_next_word():
- *	Move next to end of current word
- *	[M-f]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_next_word(EditLine *el, int c)
-{
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor = c__next_word(el->el_line.cursor,
-	    el->el_line.lastchar,
-	    el->el_state.argument,
-	    ce__isword);
-
-	if (el->el_map.type == MAP_VI)
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	return (CC_CURSOR);
-}
-
-
-/* em_upper_case():
- *	Uppercase the characters from cursor to end of current word
- *	[M-u]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_upper_case(EditLine *el, int c)
-{
-	char *cp, *ep;
-
-	ep = c__next_word(el->el_line.cursor, el->el_line.lastchar,
-	    el->el_state.argument, ce__isword);
-
-	for (cp = el->el_line.cursor; cp < ep; cp++)
-		if (islower((unsigned char) *cp))
-			*cp = toupper(*cp);
-
-	el->el_line.cursor = ep;
-	if (el->el_line.cursor > el->el_line.lastchar)
-		el->el_line.cursor = el->el_line.lastchar;
-	return (CC_REFRESH);
-}
-
-
-/* em_capitol_case():
- *	Capitalize the characters from cursor to end of current word
- *	[M-c]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_capitol_case(EditLine *el, int c)
-{
-	char *cp, *ep;
-
-	ep = c__next_word(el->el_line.cursor, el->el_line.lastchar,
-	    el->el_state.argument, ce__isword);
-
-	for (cp = el->el_line.cursor; cp < ep; cp++) {
-		if (isalpha((unsigned char) *cp)) {
-			if (islower((unsigned char) *cp))
-				*cp = toupper(*cp);
-			cp++;
-			break;
-		}
-	}
-	for (; cp < ep; cp++)
-		if (isupper((unsigned char) *cp))
-			*cp = tolower(*cp);
-
-	el->el_line.cursor = ep;
-	if (el->el_line.cursor > el->el_line.lastchar)
-		el->el_line.cursor = el->el_line.lastchar;
-	return (CC_REFRESH);
-}
-
-
-/* em_lower_case():
- *	Lowercase the characters from cursor to end of current word
- *	[M-l]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_lower_case(EditLine *el, int c)
-{
-	char *cp, *ep;
-
-	ep = c__next_word(el->el_line.cursor, el->el_line.lastchar,
-	    el->el_state.argument, ce__isword);
-
-	for (cp = el->el_line.cursor; cp < ep; cp++)
-		if (isupper((unsigned char) *cp))
-			*cp = tolower(*cp);
-
-	el->el_line.cursor = ep;
-	if (el->el_line.cursor > el->el_line.lastchar)
-		el->el_line.cursor = el->el_line.lastchar;
-	return (CC_REFRESH);
-}
-
-
-/* em_set_mark():
- *	Set the mark at cursor
- *	[^@]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_set_mark(EditLine *el, int c)
-{
-
-	el->el_chared.c_kill.mark = el->el_line.cursor;
-	return (CC_NORM);
-}
-
-
-/* em_exchange_mark():
- *	Exchange the cursor and mark
- *	[^X^X]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_exchange_mark(EditLine *el, int c)
-{
-	char *cp;
-
-	cp = el->el_line.cursor;
-	el->el_line.cursor = el->el_chared.c_kill.mark;
-	el->el_chared.c_kill.mark = cp;
-	return (CC_CURSOR);
-}
-
-
-/* em_universal_argument():
- *	Universal argument (argument times 4)
- *	[^U]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_universal_argument(EditLine *el, int c)
-{				/* multiply current argument by 4 */
-
-	if (el->el_state.argument > 1000000)
-		return (CC_ERROR);
-	el->el_state.doingarg = 1;
-	el->el_state.argument *= 4;
-	return (CC_ARGHACK);
-}
-
-
-/* em_meta_next():
- *	Add 8th bit to next character typed
- *	[<ESC>]
- */
-protected el_action_t
-/*ARGSUSED*/
-em_meta_next(EditLine *el, int c)
-{
-
-	el->el_state.metanext = 1;
-	return (CC_ARGHACK);
-}
-
-
-/* em_toggle_overwrite():
- *	Switch from insert to overwrite mode or vice versa
- */
-protected el_action_t
-/*ARGSUSED*/
-em_toggle_overwrite(EditLine *el, int c)
-{
-
-	el->el_state.inputmode = (el->el_state.inputmode == MODE_INSERT) ?
-	    MODE_REPLACE : MODE_INSERT;
-	return (CC_NORM);
-}
-
-
-/* em_copy_prev_word():
- *	Copy current word to cursor
- */
-protected el_action_t
-/*ARGSUSED*/
-em_copy_prev_word(EditLine *el, int c)
-{
-	char *cp, *oldc, *dp;
-
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (CC_ERROR);
-
-	oldc = el->el_line.cursor;
-	/* does a bounds check */
-	cp = c__prev_word(el->el_line.cursor, el->el_line.buffer,
-	    el->el_state.argument, ce__isword);
-
-	c_insert(el, oldc - cp);
-	for (dp = oldc; cp < oldc && dp < el->el_line.lastchar; cp++)
-		*dp++ = *cp;
-
-	el->el_line.cursor = dp;/* put cursor at end */
-
-	return (CC_REFRESH);
-}
-
-
-/* em_inc_search_next():
- *	Emacs incremental next search
- */
-protected el_action_t
-/*ARGSUSED*/
-em_inc_search_next(EditLine *el, int c)
-{
-
-	el->el_search.patlen = 0;
-	return (ce_inc_search(el, ED_SEARCH_NEXT_HISTORY));
-}
-
-
-/* em_inc_search_prev():
- *	Emacs incremental reverse search
- */
-protected el_action_t
-/*ARGSUSED*/
-em_inc_search_prev(EditLine *el, int c)
-{
-
-	el->el_search.patlen = 0;
-	return (ce_inc_search(el, ED_SEARCH_PREV_HISTORY));
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/.gitignore 1:16.1.1~dfsg-1/main/editline/.gitignore
--- 1:13.23.1~dfsg-2/main/editline/.gitignore	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/.gitignore	1970-01-01 00:00:00.000000000 +0000
@@ -1,13 +0,0 @@
-*.o_a
-Makefile
-common.h
-config.cache
-config.h
-editline.c
-emacs.h
-fcns.c
-fcns.h
-help.c
-help.h
-makelist
-vi.h
diff -pruN 1:13.23.1~dfsg-2/main/editline/hist.c 1:16.1.1~dfsg-1/main/editline/hist.c
--- 1:13.23.1~dfsg-2/main/editline/hist.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/hist.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,197 +0,0 @@
-/*	$NetBSD: hist.c,v 1.10 2002/03/18 16:00:53 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)hist.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: hist.c,v 1.10 2002/03/18 16:00:53 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * hist.c: History access functions
- */
-#include <stdlib.h>
-#include "el.h"
-
-/* hist_init():
- *	Initialization function.
- */
-protected int
-hist_init(EditLine *el)
-{
-
-	el->el_history.fun = NULL;
-	el->el_history.ref = NULL;
-	el->el_history.buf = (char *) el_malloc(EL_BUFSIZ);
-	el->el_history.sz  = EL_BUFSIZ;
-	if (el->el_history.buf == NULL)
-		return (-1);
-	el->el_history.last = el->el_history.buf;
-	return (0);
-}
-
-
-/* hist_end():
- *	clean up history;
- */
-protected void
-hist_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_history.buf);
-	el->el_history.buf = NULL;
-}
-
-
-/* hist_set():
- *	Set new history interface
- */
-protected int
-hist_set(EditLine *el, hist_fun_t fun, ptr_t ptr)
-{
-
-	el->el_history.ref = ptr;
-	el->el_history.fun = fun;
-	return (0);
-}
-
-
-/* hist_get():
- *	Get a history line and update it in the buffer.
- *	eventno tells us the event to get.
- */
-protected el_action_t
-hist_get(EditLine *el)
-{
-	const char *hp;
-	int h;
-
-	if (el->el_history.eventno == 0) {	/* if really the current line */
-		(void) strncpy(el->el_line.buffer, el->el_history.buf,
-		    el->el_history.sz - 1);
-		el->el_line.lastchar = el->el_line.buffer +
-		    (el->el_history.last - el->el_history.buf);
-
-#ifdef KSHVI
-		if (el->el_map.type == MAP_VI)
-			el->el_line.cursor = el->el_line.buffer;
-		else
-#endif /* KSHVI */
-			el->el_line.cursor = el->el_line.lastchar;
-
-		return (CC_REFRESH);
-	}
-	if (el->el_history.ref == NULL)
-		return (CC_ERROR);
-
-	hp = HIST_FIRST(el);
-
-	if (hp == NULL)
-		return (CC_ERROR);
-
-	for (h = 1; h < el->el_history.eventno; h++)
-		if ((hp = HIST_NEXT(el)) == NULL) {
-			el->el_history.eventno = h;
-			return (CC_ERROR);
-		}
-	(void) strncpy(el->el_line.buffer, hp,
-			(size_t)(el->el_line.limit - el->el_line.buffer));
-	el->el_line.lastchar = el->el_line.buffer + strlen(el->el_line.buffer);
-
-	if (el->el_line.lastchar > el->el_line.buffer) {
-		if (el->el_line.lastchar[-1] == '\n')
-			el->el_line.lastchar--;
-		if ((el->el_line.lastchar > el->el_line.buffer)&&(el->el_line.lastchar[-1] == ' ')) /* bill heckel */
-			el->el_line.lastchar--;
-		if (el->el_line.lastchar < el->el_line.buffer)
-			el->el_line.lastchar = el->el_line.buffer;
-	}
-#ifdef KSHVI
-	if (el->el_map.type == MAP_VI)
-		el->el_line.cursor = el->el_line.buffer;
-	else
-#endif /* KSHVI */
-		el->el_line.cursor = el->el_line.lastchar;
-
-	return (CC_REFRESH);
-}
-
-
-/* hist_list()
- *	List history entries
- */
-protected int
-/*ARGSUSED*/
-hist_list(EditLine *el, int argc, const char **argv)
-{
-	const char *str;
-
-	if (el->el_history.ref == NULL)
-		return (-1);
-	for (str = HIST_LAST(el); str != NULL; str = HIST_PREV(el))
-		(void) fprintf(el->el_outfile, "%d %s",
-		    el->el_history.ev.num, str);
-	return (0);
-}
-
-/* hist_enlargebuf()
- *	Enlarge history buffer to specified value. Called from el_enlargebufs().
- *	Return 0 for failure, 1 for success.
- */
-protected int
-/*ARGSUSED*/
-hist_enlargebuf(EditLine *el, size_t oldsz, size_t newsz)
-{
-	char *newbuf;
-
-	newbuf = realloc(el->el_history.buf, newsz);
-	if (!newbuf)
-		return 0;
-
-	(void) memset(&newbuf[oldsz], '\0', newsz - oldsz);
-
-	el->el_history.last = newbuf +
-				(el->el_history.last - el->el_history.buf);
-	el->el_history.buf = newbuf;
-	el->el_history.sz  = newsz;
-
-	return 1;
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/histedit.h 1:16.1.1~dfsg-1/main/editline/histedit.h
--- 1:13.23.1~dfsg-2/main/editline/histedit.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/histedit.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,197 +0,0 @@
-/*	$NetBSD: histedit.h,v 1.19 2002/03/18 16:00:54 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)histedit.h	8.2 (Berkeley) 1/3/94
- */
-
-/*
- * histedit.h: Line editor and history interface.
- */
-#ifndef _HISTEDIT_H_
-#define	_HISTEDIT_H_
-
-#define	LIBEDIT_MAJOR 2
-#define	LIBEDIT_MINOR 6
-
-#include <sys/types.h>
-#include <stdio.h>
-
-/*
- * ==== Editing ====
- */
-typedef struct editline EditLine;
-
-/*
- * For user-defined function interface
- */
-typedef struct lineinfo {
-	char	*buffer;
-	char	*cursor;
-	char	*lastchar;
-} LineInfo;
-
-
-/*
- * EditLine editor function return codes.
- * For user-defined function interface
- */
-#define	CC_NORM		0
-#define	CC_NEWLINE	1
-#define	CC_EOF		2
-#define	CC_ARGHACK	3
-#define	CC_REFRESH	4
-#define	CC_CURSOR	5
-#define	CC_ERROR	6
-#define	CC_FATAL	7
-#define	CC_REDISPLAY	8
-#define	CC_REFRESH_BEEP	9
-
-/*
- * Initialization, cleanup, and resetting
- */
-EditLine	*el_init(const char *, FILE *, FILE *, FILE *);
-void		 el_reset(EditLine *);
-void		 el_end(EditLine *);
-
-
-/*
- * Get a line, a character or push a string back in the input queue
- */
-const char	*el_gets(EditLine *, int *);
-int		 el_getc(EditLine *, char *);
-void		 el_push(EditLine *, char *);
-
-/*
- * Beep!
- */
-void		 el_beep(EditLine *);
-
-/*
- * High level function internals control
- * Parses argc, argv array and executes builtin editline commands
- */
-int		 el_parse(EditLine *, int, const char **);
-
-/*
- * Low level editline access functions
- */
-int		 el_set(EditLine *, int, ...);
-int		 el_get(EditLine *, int, void *);
-
-/*
- * el_set/el_get parameters
- */
-#define	EL_PROMPT	0	/* , el_pfunc_t);		*/
-#define	EL_TERMINAL	1	/* , const char *);		*/
-#define	EL_EDITOR	2	/* , const char *);		*/
-#define	EL_SIGNAL	3	/* , int);			*/
-#define	EL_BIND		4	/* , const char *, ..., NULL);	*/
-#define	EL_TELLTC	5	/* , const char *, ..., NULL);	*/
-#define	EL_SETTC	6	/* , const char *, ..., NULL);	*/
-#define	EL_ECHOTC	7	/* , const char *, ..., NULL);	*/
-#define	EL_SETTY	8	/* , const char *, ..., NULL);	*/
-#define	EL_ADDFN	9	/* , const char *, const char *	*/
-				/* , el_func_t);		*/
-#define	EL_HIST		10	/* , hist_fun_t, const char *);	*/
-#define	EL_EDITMODE	11	/* , int);			*/
-#define	EL_RPROMPT	12	/* , el_pfunc_t);		*/
-#define	EL_GETCFN	13	/* , el_rfunc_t);		*/
-#define	EL_CLIENTDATA	14	/* , void *);			*/
-
-#define EL_BUILTIN_GETCFN	(NULL)
-
-/*
- * Source named file or $PWD/.editrc or $HOME/.editrc
- */
-int		el_source(EditLine *, const char *);
-
-/*
- * Must be called when the terminal changes size; If EL_SIGNAL
- * is set this is done automatically otherwise it is the responsibility
- * of the application
- */
-void		 el_resize(EditLine *);
-
-
-/*
- * User-defined function interface.
- */
-const LineInfo	*el_line(EditLine *);
-int		 el_insertstr(EditLine *, const char *);
-void		 el_deletestr(EditLine *, int);
-
-/*
- * ==== History ====
- */
-
-typedef struct history History;
-
-typedef struct HistEvent {
-	int		 num;
-	const char	*str;
-} HistEvent;
-
-/*
- * History access functions.
- */
-History *	history_init(void);
-void		history_end(History *);
-
-int		history(History *, HistEvent *, int, ...);
-
-#define	H_FUNC		 0	/* , UTSL		*/
-#define	H_SETSIZE	 1	/* , const int);	*/
-#define	H_GETSIZE	 2	/* , void);		*/
-#define	H_FIRST		 3	/* , void);		*/
-#define	H_LAST		 4	/* , void);		*/
-#define	H_PREV		 5	/* , void);		*/
-#define	H_NEXT		 6	/* , void);		*/
-#define	H_CURR		 8	/* , const int);	*/
-#define	H_SET		 7	/* , void);		*/
-#define	H_ADD		 9	/* , const char *);	*/
-#define	H_ENTER		10	/* , const char *);	*/
-#define	H_APPEND	11	/* , const char *);	*/
-#define	H_END		12	/* , void);		*/
-#define	H_NEXT_STR	13	/* , const char *);	*/
-#define	H_PREV_STR	14	/* , const char *);	*/
-#define	H_NEXT_EVENT	15	/* , const int);	*/
-#define	H_PREV_EVENT	16	/* , const int);	*/
-#define	H_LOAD		17	/* , const char *);	*/
-#define	H_SAVE		18	/* , const char *);	*/
-#define	H_CLEAR		19	/* , void);		*/
-
-#endif /* _HISTEDIT_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/editline/hist.h 1:16.1.1~dfsg-1/main/editline/hist.h
--- 1:13.23.1~dfsg-2/main/editline/hist.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/hist.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,80 +0,0 @@
-/*	$NetBSD: hist.h,v 1.7 2002/03/18 16:00:53 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)hist.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.hist.c: History functions
- */
-#ifndef _h_el_hist
-#define	_h_el_hist
-
-#include "histedit.h"
-
-typedef int (*hist_fun_t)(ptr_t, HistEvent *, int, ...);
-
-typedef struct el_history_t {
-	char		*buf;		/* The history buffer		*/
-	size_t		sz;		/* Size of history buffer	*/
-	char		*last;		/* The last character		*/
-	int		 eventno;	/* Event we are looking for	*/
-	ptr_t		 ref;		/* Argument for history fcns	*/
-	hist_fun_t	 fun;		/* Event access			*/
-	HistEvent	 ev;		/* Event cookie			*/
-} el_history_t;
-
-#define	HIST_FUN(el, fn, arg)	\
-    ((((*(el)->el_history.fun) ((el)->el_history.ref, &(el)->el_history.ev, \
-	fn, arg)) == -1) ? NULL : (el)->el_history.ev.str)
-
-#define	HIST_NEXT(el)		HIST_FUN(el, H_NEXT, NULL)
-#define	HIST_FIRST(el)		HIST_FUN(el, H_FIRST, NULL)
-#define	HIST_LAST(el)		HIST_FUN(el, H_LAST, NULL)
-#define	HIST_PREV(el)		HIST_FUN(el, H_PREV, NULL)
-#define	HIST_EVENT(el, num)	HIST_FUN(el, H_EVENT, num)
-#define	HIST_LOAD(el, fname)	HIST_FUN(el, H_LOAD fname)
-#define	HIST_SAVE(el, fname)	HIST_FUN(el, H_SAVE fname)
-
-protected int		hist_init(EditLine *);
-protected void		hist_end(EditLine *);
-protected el_action_t	hist_get(EditLine *);
-protected int		hist_set(EditLine *, hist_fun_t, ptr_t);
-protected int		hist_list(EditLine *, int, const char **);
-protected int		hist_enlargebuf(EditLine *, size_t, size_t);
-
-#endif /* _h_el_hist */
diff -pruN 1:13.23.1~dfsg-2/main/editline/history.c 1:16.1.1~dfsg-1/main/editline/history.c
--- 1:13.23.1~dfsg-2/main/editline/history.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/history.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,875 +0,0 @@
-/*	$NetBSD: history.c,v 1.19 2002/03/18 16:00:54 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)history.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: history.c,v 1.19 2002/03/18 16:00:54 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * hist.c: History access functions
- */
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#ifdef HAVE_VIS_H
-#include <vis.h>
-#else
-#include "np/vis.h"
-#endif
-#include <sys/stat.h>
-
-static const char hist_cookie[] = "_HiStOrY_V2_\n";
-
-#include "histedit.h"
-
-typedef int (*history_gfun_t)(ptr_t, HistEvent *);
-typedef int (*history_efun_t)(ptr_t, HistEvent *, const char *);
-typedef void (*history_vfun_t)(ptr_t, HistEvent *);
-typedef int (*history_sfun_t)(ptr_t, HistEvent *, const int);
-
-struct history {
-	ptr_t h_ref;		/* Argument for history fcns	 */
-	int h_ent;		/* Last entry point for history	 */
-	history_gfun_t h_first;	/* Get the first element	 */
-	history_gfun_t h_next;	/* Get the next element		 */
-	history_gfun_t h_last;	/* Get the last element		 */
-	history_gfun_t h_prev;	/* Get the previous element	 */
-	history_gfun_t h_curr;	/* Get the current element	 */
-	history_sfun_t h_set;	/* Set the current element	 */
-	history_vfun_t h_clear;	/* Clear the history list	 */
-	history_efun_t h_enter;	/* Add an element		 */
-	history_efun_t h_add;	/* Append to an element		 */
-};
-#define	HNEXT(h, ev)		(*(h)->h_next)((h)->h_ref, ev)
-#define	HFIRST(h, ev)		(*(h)->h_first)((h)->h_ref, ev)
-#define	HPREV(h, ev)		(*(h)->h_prev)((h)->h_ref, ev)
-#define	HLAST(h, ev)		(*(h)->h_last)((h)->h_ref, ev)
-#define	HCURR(h, ev)		(*(h)->h_curr)((h)->h_ref, ev)
-#define	HSET(h, ev, n)		(*(h)->h_set)((h)->h_ref, ev, n)
-#define	HCLEAR(h, ev)		(*(h)->h_clear)((h)->h_ref, ev)
-#define	HENTER(h, ev, str)	(*(h)->h_enter)((h)->h_ref, ev, str)
-#define	HADD(h, ev, str)	(*(h)->h_add)((h)->h_ref, ev, str)
-
-#define	h_malloc(a)	malloc(a)
-#define	h_realloc(a, b)	realloc((a), (b))
-#define	h_free(a)	free(a)
-
-typedef struct {
-    int		num;
-    char	*str;
-} HistEventPrivate;
-
-
-
-private int history_setsize(History *, HistEvent *, int);
-private int history_getsize(History *, HistEvent *);
-private int history_set_fun(History *, History *);
-private int history_load(History *, const char *);
-private int history_save(History *, const char *);
-private int history_prev_event(History *, HistEvent *, int);
-private int history_next_event(History *, HistEvent *, int);
-private int history_next_string(History *, HistEvent *, const char *);
-private int history_prev_string(History *, HistEvent *, const char *);
-
-
-/***********************************************************************/
-
-/*
- * Builtin- history implementation
- */
-typedef struct hentry_t {
-	HistEvent ev;		/* What we return		 */
-	struct hentry_t *next;	/* Next entry			 */
-	struct hentry_t *prev;	/* Previous entry		 */
-}        hentry_t;
-
-typedef struct history_t {
-	hentry_t list;		/* Fake list header element	 */
-	hentry_t *cursor;	/* Current element in the list	 */
-	int max;		/* Maximum number of events	 */
-	int cur;		/* Current number of events	 */
-	int eventid;		/* For generation of unique event id	 */
-}         history_t;
-
-private int history_def_first(ptr_t, HistEvent *);
-private int history_def_last(ptr_t, HistEvent *);
-private int history_def_next(ptr_t, HistEvent *);
-private int history_def_prev(ptr_t, HistEvent *);
-private int history_def_curr(ptr_t, HistEvent *);
-private int history_def_set(ptr_t, HistEvent *, const int n);
-private int history_def_enter(ptr_t, HistEvent *, const char *);
-private int history_def_add(ptr_t, HistEvent *, const char *);
-private void history_def_init(ptr_t *, HistEvent *, int);
-private void history_def_clear(ptr_t, HistEvent *);
-private int history_def_insert(history_t *, HistEvent *, const char *);
-private void history_def_delete(history_t *, HistEvent *, hentry_t *);
-
-#define	history_def_setsize(p, num)(void) (((history_t *) p)->max = (num))
-#define	history_def_getsize(p)  (((history_t *) p)->cur)
-
-#define	he_strerror(code)	he_errlist[code]
-#define	he_seterrev(evp, code)	{\
-				    evp->num = code;\
-				    evp->str = he_strerror(code);\
-				}
-
-/* error messages */
-static const char *const he_errlist[] = {
-	"OK",
-	"unknown error",
-	"malloc() failed",
-	"first event not found",
-	"last event not found",
-	"empty list",
-	"no next event",
-	"no previous event",
-	"current event is invalid",
-	"event not found",
-	"can't read history from file",
-	"can't write history",
-	"required parameter(s) not supplied",
-	"history size negative",
-	"function not allowed with other history-functions-set the default",
-	"bad parameters"
-};
-/* error codes */
-#define	_HE_OK                   0
-#define	_HE_UNKNOWN		 1
-#define	_HE_MALLOC_FAILED        2
-#define	_HE_FIRST_NOTFOUND       3
-#define	_HE_LAST_NOTFOUND        4
-#define	_HE_EMPTY_LIST           5
-#define	_HE_END_REACHED          6
-#define	_HE_START_REACHED	 7
-#define	_HE_CURR_INVALID	 8
-#define	_HE_NOT_FOUND		 9
-#define	_HE_HIST_READ		10
-#define	_HE_HIST_WRITE		11
-#define	_HE_PARAM_MISSING	12
-#define	_HE_SIZE_NEGATIVE	13
-#define	_HE_NOT_ALLOWED		14
-#define	_HE_BAD_PARAM		15
-
-/* history_def_first():
- *	Default function to return the first event in the history.
- */
-private int
-history_def_first(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	h->cursor = h->list.next;
-	if (h->cursor != &h->list)
-		*ev = h->cursor->ev;
-	else {
-		he_seterrev(ev, _HE_FIRST_NOTFOUND);
-		return (-1);
-	}
-
-	return (0);
-}
-
-
-/* history_def_last():
- *	Default function to return the last event in the history.
- */
-private int
-history_def_last(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	h->cursor = h->list.prev;
-	if (h->cursor != &h->list)
-		*ev = h->cursor->ev;
-	else {
-		he_seterrev(ev, _HE_LAST_NOTFOUND);
-		return (-1);
-	}
-
-	return (0);
-}
-
-
-/* history_def_next():
- *	Default function to return the next event in the history.
- */
-private int
-history_def_next(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	if (h->cursor != &h->list)
-		h->cursor = h->cursor->next;
-	else {
-		he_seterrev(ev, _HE_EMPTY_LIST);
-		return (-1);
-	}
-
-	if (h->cursor != &h->list)
-		*ev = h->cursor->ev;
-	else {
-		he_seterrev(ev, _HE_END_REACHED);
-		return (-1);
-	}
-
-	return (0);
-}
-
-
-/* history_def_prev():
- *	Default function to return the previous event in the history.
- */
-private int
-history_def_prev(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	if (h->cursor != &h->list)
-		h->cursor = h->cursor->prev;
-	else {
-		he_seterrev(ev,
-		    (h->cur > 0) ? _HE_END_REACHED : _HE_EMPTY_LIST);
-		return (-1);
-	}
-
-	if (h->cursor != &h->list)
-		*ev = h->cursor->ev;
-	else {
-		he_seterrev(ev, _HE_START_REACHED);
-		return (-1);
-	}
-
-	return (0);
-}
-
-
-/* history_def_curr():
- *	Default function to return the current event in the history.
- */
-private int
-history_def_curr(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	if (h->cursor != &h->list)
-		*ev = h->cursor->ev;
-	else {
-		he_seterrev(ev,
-		    (h->cur > 0) ? _HE_CURR_INVALID : _HE_EMPTY_LIST);
-		return (-1);
-	}
-
-	return (0);
-}
-
-
-/* history_def_set():
- *	Default function to set the current event in the history to the
- *	given one.
- */
-private int
-history_def_set(ptr_t p, HistEvent *ev, const int n)
-{
-	history_t *h = (history_t *) p;
-
-	if (h->cur == 0) {
-		he_seterrev(ev, _HE_EMPTY_LIST);
-		return (-1);
-	}
-	if (h->cursor == &h->list || h->cursor->ev.num != n) {
-		for (h->cursor = h->list.next; h->cursor != &h->list;
-		    h->cursor = h->cursor->next)
-			if (h->cursor->ev.num == n)
-				break;
-	}
-	if (h->cursor == &h->list) {
-		he_seterrev(ev, _HE_NOT_FOUND);
-		return (-1);
-	}
-	return (0);
-}
-
-
-/* history_def_add():
- *	Append string to element
- */
-private int
-history_def_add(ptr_t p, HistEvent *ev, const char *str)
-{
-	history_t *h = (history_t *) p;
-	size_t len;
-	char *s;
-	HistEventPrivate *evp = (void *)&h->cursor->ev;
-
-	if (h->cursor == &h->list)
-		return (history_def_enter(p, ev, str));
-	len = strlen(evp->str) + strlen(str) + 1;
-	s = (char *) h_malloc(len);
-	if (!s) {
-		he_seterrev(ev, _HE_MALLOC_FAILED);
-		return (-1);
-	}
-	(void) strlcpy(s, h->cursor->ev.str, len);
-	(void) strlcat(s, str, len);
-	h_free(evp->str);
-	evp->str = s;
-	*ev = h->cursor->ev;
-	return (0);
-}
-
-
-/* history_def_delete():
- *	Delete element hp of the h list
- */
-/* ARGSUSED */
-private void
-history_def_delete(history_t *h, HistEvent *ev, hentry_t *hp)
-{
-	HistEventPrivate *evp = (void *)&hp->ev;
-	if (hp == &h->list)
-		abort();
-	hp->prev->next = hp->next;
-	hp->next->prev = hp->prev;
-	h_free((ptr_t) evp->str);
-	h_free(hp);
-	h->cur--;
-}
-
-
-/* history_def_insert():
- *	Insert element with string str in the h list
- */
-private int
-history_def_insert(history_t *h, HistEvent *ev, const char *str)
-{
-
-	h->cursor = (hentry_t *) h_malloc(sizeof(hentry_t));
-	if (h->cursor)
-		h->cursor->ev.str = strdup(str);
-	if (!h->cursor || !h->cursor->ev.str) {
-		he_seterrev(ev, _HE_MALLOC_FAILED);
-		return (-1);
-	}
-	h->cursor->ev.num = ++h->eventid;
-	h->cursor->next = h->list.next;
-	h->cursor->prev = &h->list;
-	h->list.next->prev = h->cursor;
-	h->list.next = h->cursor;
-	h->cur++;
-
-	*ev = h->cursor->ev;
-	return (0);
-}
-
-
-/* history_def_enter():
- *	Default function to enter an item in the history
- */
-private int
-history_def_enter(ptr_t p, HistEvent *ev, const char *str)
-{
-	history_t *h = (history_t *) p;
-
-	if (history_def_insert(h, ev, str) == -1)
-		return (-1);	/* error, keep error message */
-
-	/*
-         * Always keep at least one entry.
-         * This way we don't have to check for the empty list.
-         */
-	while (h->cur > h->max && h->cur > 0)
-		history_def_delete(h, ev, h->list.prev);
-
-	return (0);
-}
-
-
-/* history_def_init():
- *	Default history initialization function
- */
-/* ARGSUSED */
-private void
-history_def_init(ptr_t *p, HistEvent *ev, int n)
-{
-	history_t *h = (history_t *) h_malloc(sizeof(history_t));
-
-	if (n <= 0)
-		n = 0;
-	h->eventid = 0;
-	h->cur = 0;
-	h->max = n;
-	h->list.next = h->list.prev = &h->list;
-	h->list.ev.str = NULL;
-	h->list.ev.num = 0;
-	h->cursor = &h->list;
-	*p = (ptr_t) h;
-}
-
-
-/* history_def_clear():
- *	Default history cleanup function
- */
-private void
-history_def_clear(ptr_t p, HistEvent *ev)
-{
-	history_t *h = (history_t *) p;
-
-	while (h->list.prev != &h->list)
-		history_def_delete(h, ev, h->list.prev);
-	h->eventid = 0;
-	h->cur = 0;
-}
-
-
-
-
-/************************************************************************/
-
-/* history_init():
- *	Initialization function.
- */
-public History *
-history_init(void)
-{
-	History *h = (History *) h_malloc(sizeof(History));
-	HistEvent ev;
-
-	history_def_init(&h->h_ref, &ev, 0);
-	h->h_ent = -1;
-	h->h_next = history_def_next;
-	h->h_first = history_def_first;
-	h->h_last = history_def_last;
-	h->h_prev = history_def_prev;
-	h->h_curr = history_def_curr;
-	h->h_set = history_def_set;
-	h->h_clear = history_def_clear;
-	h->h_enter = history_def_enter;
-	h->h_add = history_def_add;
-
-	return (h);
-}
-
-
-/* history_end():
- *	clean up history;
- */
-public void
-history_end(History *h)
-{
-	HistEvent ev;
-
-	if (h->h_next == history_def_next)
-		history_def_clear(h->h_ref, &ev);
-}
-
-
-
-/* history_setsize():
- *	Set history number of events
- */
-private int
-history_setsize(History *h, HistEvent *ev, int num)
-{
-
-	if (h->h_next != history_def_next) {
-		he_seterrev(ev, _HE_NOT_ALLOWED);
-		return (-1);
-	}
-	if (num < 0) {
-		he_seterrev(ev, _HE_BAD_PARAM);
-		return (-1);
-	}
-	history_def_setsize(h->h_ref, num);
-	return (0);
-}
-
-
-/* history_getsize():
- *      Get number of events currently in history
- */
-private int
-history_getsize(History *h, HistEvent *ev)
-{
-	int retval = 0;
-
-	if (h->h_next != history_def_next) {
-		he_seterrev(ev, _HE_NOT_ALLOWED);
-		return (-1);
-	}
-	retval = history_def_getsize(h->h_ref);
-	if (retval < -1) {
-		he_seterrev(ev, _HE_SIZE_NEGATIVE);
-		return (-1);
-	}
-	ev->num = retval;
-	return (0);
-}
-
-
-/* history_set_fun():
- *	Set history functions
- */
-private int
-history_set_fun(History *h, History *nh)
-{
-	HistEvent ev;
-
-	if (nh->h_first == NULL || nh->h_next == NULL || nh->h_last == NULL ||
-	    nh->h_prev == NULL || nh->h_curr == NULL || nh->h_set == NULL ||
-	    nh->h_enter == NULL || nh->h_add == NULL || nh->h_clear == NULL ||
-	    nh->h_ref == NULL) {
-		if (h->h_next != history_def_next) {
-			history_def_init(&h->h_ref, &ev, 0);
-			h->h_first = history_def_first;
-			h->h_next = history_def_next;
-			h->h_last = history_def_last;
-			h->h_prev = history_def_prev;
-			h->h_curr = history_def_curr;
-			h->h_set = history_def_set;
-			h->h_clear = history_def_clear;
-			h->h_enter = history_def_enter;
-			h->h_add = history_def_add;
-		}
-		return (-1);
-	}
-	if (h->h_next == history_def_next)
-		history_def_clear(h->h_ref, &ev);
-
-	h->h_ent = -1;
-	h->h_first = nh->h_first;
-	h->h_next = nh->h_next;
-	h->h_last = nh->h_last;
-	h->h_prev = nh->h_prev;
-	h->h_curr = nh->h_curr;
-	h->h_set = nh->h_set;
-	h->h_clear = nh->h_clear;
-	h->h_enter = nh->h_enter;
-	h->h_add = nh->h_add;
-
-	return (0);
-}
-
-
-/* history_load():
- *	History load function
- */
-private int
-history_load(History *h, const char *fname)
-{
-	FILE *fp;
-	char *line;
-	size_t sz, max_size;
-	char *ptr;
-	int i = -1;
-	HistEvent ev;
-
-	if ((fp = fopen(fname, "r")) == NULL)
-		return (i);
-
-	if ((line = fgetln(fp, &sz)) == NULL)
-		goto done;
-
-	if (strncmp(line, hist_cookie, sz) != 0)
-		goto done;
-
-	ptr = h_malloc(max_size = 1024);
-	for (i = 0; (line = fgetln(fp, &sz)) != NULL; i++) {
-		char c = line[sz];
-
-		if (sz != 0 && line[sz - 1] == '\n')
-			line[--sz] = '\0';
-		else
-			line[sz] = '\0';
-
-		if (max_size < sz) {
-			max_size = (sz + 1023) & ~1023;
-			ptr = h_realloc(ptr, max_size);
-		}
-		(void) strunvis(ptr, line);
-		line[sz] = c;
-		HENTER(h, &ev, ptr);
-	}
-	h_free(ptr);
-
-done:
-	(void) fclose(fp);
-	return (i);
-}
-
-
-/* history_save():
- *	History save function
- */
-private int
-history_save(History *h, const char *fname)
-{
-	FILE *fp;
-	HistEvent ev;
-	int i = 0, retval;
-	size_t len, max_size;
-	char *ptr;
-
-	if ((fp = fopen(fname, "w")) == NULL)
-		return (-1);
-
-	(void) fchmod(fileno(fp), S_IRUSR|S_IWUSR);
-	(void) fputs(hist_cookie, fp);
-	ptr = h_malloc(max_size = 1024);
-	for (retval = HLAST(h, &ev);
-	    retval != -1;
-	    retval = HPREV(h, &ev), i++) {
-		len = strlen(ev.str) * 4;
-		if (len >= max_size) {
-			max_size = (len + 1023) & 1023;
-			ptr = h_realloc(ptr, max_size);
-		}
-		(void) strvis(ptr, ev.str, VIS_WHITE);
-		(void) fprintf(fp, "%s\n", ev.str);
-	}
-	h_free(ptr);
-	(void) fclose(fp);
-	return (i);
-}
-
-
-/* history_prev_event():
- *	Find the previous event, with number given
- */
-private int
-history_prev_event(History *h, HistEvent *ev, int num)
-{
-	int retval;
-
-	for (retval = HCURR(h, ev); retval != -1; retval = HPREV(h, ev))
-		if (ev->num == num)
-			return (0);
-
-	he_seterrev(ev, _HE_NOT_FOUND);
-	return (-1);
-}
-
-
-/* history_next_event():
- *	Find the next event, with number given
- */
-private int
-history_next_event(History *h, HistEvent *ev, int num)
-{
-	int retval;
-
-	for (retval = HCURR(h, ev); retval != -1; retval = HNEXT(h, ev))
-		if (ev->num == num)
-			return (0);
-
-	he_seterrev(ev, _HE_NOT_FOUND);
-	return (-1);
-}
-
-
-/* history_prev_string():
- *	Find the previous event beginning with string
- */
-private int
-history_prev_string(History *h, HistEvent *ev, const char *str)
-{
-	size_t len = strlen(str);
-	int retval;
-
-	for (retval = HCURR(h, ev); retval != -1; retval = HNEXT(h, ev))
-		if (strncmp(str, ev->str, len) == 0)
-			return (0);
-
-	he_seterrev(ev, _HE_NOT_FOUND);
-	return (-1);
-}
-
-
-/* history_next_string():
- *	Find the next event beginning with string
- */
-private int
-history_next_string(History *h, HistEvent *ev, const char *str)
-{
-	size_t len = strlen(str);
-	int retval;
-
-	for (retval = HCURR(h, ev); retval != -1; retval = HPREV(h, ev))
-		if (strncmp(str, ev->str, len) == 0)
-			return (0);
-
-	he_seterrev(ev, _HE_NOT_FOUND);
-	return (-1);
-}
-
-
-/* history():
- *	User interface to history functions.
- */
-int
-history(History *h, HistEvent *ev, int fun, ...)
-{
-	va_list va;
-	const char *str;
-	int retval;
-
-	va_start(va, fun);
-
-	he_seterrev(ev, _HE_OK);
-
-	switch (fun) {
-	case H_GETSIZE:
-		retval = history_getsize(h, ev);
-		break;
-
-	case H_SETSIZE:
-		retval = history_setsize(h, ev, va_arg(va, int));
-		break;
-
-	case H_ADD:
-		str = va_arg(va, const char *);
-		retval = HADD(h, ev, str);
-		break;
-
-	case H_ENTER:
-		str = va_arg(va, const char *);
-		if ((retval = HENTER(h, ev, str)) != -1)
-			h->h_ent = ev->num;
-		break;
-
-	case H_APPEND:
-		str = va_arg(va, const char *);
-		if ((retval = HSET(h, ev, h->h_ent)) != -1)
-			retval = HADD(h, ev, str);
-		break;
-
-	case H_FIRST:
-		retval = HFIRST(h, ev);
-		break;
-
-	case H_NEXT:
-		retval = HNEXT(h, ev);
-		break;
-
-	case H_LAST:
-		retval = HLAST(h, ev);
-		break;
-
-	case H_PREV:
-		retval = HPREV(h, ev);
-		break;
-
-	case H_CURR:
-		retval = HCURR(h, ev);
-		break;
-
-	case H_SET:
-		retval = HSET(h, ev, va_arg(va, const int));
-		break;
-
-	case H_CLEAR:
-		HCLEAR(h, ev);
-		retval = 0;
-		break;
-
-	case H_LOAD:
-		retval = history_load(h, va_arg(va, const char *));
-		if (retval == -1)
-			he_seterrev(ev, _HE_HIST_READ);
-		break;
-
-	case H_SAVE:
-		retval = history_save(h, va_arg(va, const char *));
-		if (retval == -1)
-			he_seterrev(ev, _HE_HIST_WRITE);
-		break;
-
-	case H_PREV_EVENT:
-		retval = history_prev_event(h, ev, va_arg(va, int));
-		break;
-
-	case H_NEXT_EVENT:
-		retval = history_next_event(h, ev, va_arg(va, int));
-		break;
-
-	case H_PREV_STR:
-		retval = history_prev_string(h, ev, va_arg(va, const char *));
-		break;
-
-	case H_NEXT_STR:
-		retval = history_next_string(h, ev, va_arg(va, const char *));
-		break;
-
-	case H_FUNC:
-	{
-		History hf;
-
-		hf.h_ref = va_arg(va, ptr_t);
-		h->h_ent = -1;
-		hf.h_first = va_arg(va, history_gfun_t);
-		hf.h_next = va_arg(va, history_gfun_t);
-		hf.h_last = va_arg(va, history_gfun_t);
-		hf.h_prev = va_arg(va, history_gfun_t);
-		hf.h_curr = va_arg(va, history_gfun_t);
-		hf.h_set = va_arg(va, history_sfun_t);
-		hf.h_clear = va_arg(va, history_vfun_t);
-		hf.h_enter = va_arg(va, history_efun_t);
-		hf.h_add = va_arg(va, history_efun_t);
-
-		if ((retval = history_set_fun(h, &hf)) == -1)
-			he_seterrev(ev, _HE_PARAM_MISSING);
-		break;
-	}
-
-	case H_END:
-		history_end(h);
-		retval = 0;
-		break;
-
-	default:
-		retval = -1;
-		he_seterrev(ev, _HE_UNKNOWN);
-		break;
-	}
-	va_end(va);
-	return (retval);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/INSTALL 1:16.1.1~dfsg-1/main/editline/INSTALL
--- 1:13.23.1~dfsg-2/main/editline/INSTALL	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/INSTALL	1970-01-01 00:00:00.000000000 +0000
@@ -1,64 +0,0 @@
-Building this distribution in many cases is as simple as typing the following
-while in the root directory of the source tree:
-
-	./configure
-	make
-
-To install, do the above, then type:
-
-	make install
-
-Additional build targets of finer granularity include:
-
-	lib_a
-	lib_s
-	install_hdr
-	install_lib
-	install_lib_a
-	install_lib_s
-	install_man
-	test
-
-Cleanup targets include:
-
-	clean
-	distclean
-
-Optionally, pass any of the following (not a definitive list) arguments to
-'configure':
-
---prefix=<install-root-dir>
-	Set the base directory in which to install.  For example:
-
-		./configure --prefix=/usr/local
-
-	will cause files to be installed into /usr/local/bin, /usr/local/man,
-	/usr/local/include, /usr/local/lib, and /usr/local/share.
-
---disable-readline
-	By default, libedit is built and installed such that it works as a
-	drop-in replacement for the readline library.  This option turns that
-	behavior off.
-
---enable-debug
-	Build debugging code (for libedit development).
-
-Optionally, define environment variables, including (not exclusively):
-
-CFLAGS="?"
-	Pass these flags to the compiler.  You probably shouldn't define this
-	unless you know what you are doing.
-
-CPPFLAGS="?"
-	Pass these flags to the C preprocessor.  Note that CFLAGS is not passed
-	to 'cpp' when 'configure' is looking for include files, so you must use
-	CPPFLAGS instead if you need to help 'configure' find header files.
-
-LD_LIBRARY_PATH="?"
-	'ld' uses this colon-separated list to find libraries.
-
-LDFLAGS="?"
-	Flags passed to 'gcc', which should normally be passed on to 'ld'.
-
-PATH="?"
-	'configure' uses this to find programs.
diff -pruN 1:13.23.1~dfsg-2/main/editline/install-sh 1:16.1.1~dfsg-1/main/editline/install-sh
--- 1:13.23.1~dfsg-2/main/editline/install-sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/install-sh	1970-01-01 00:00:00.000000000 +0000
@@ -1,250 +0,0 @@
-#! /bin/sh
-#
-# install - install a program, script, or datafile
-# This comes from X11R5 (mit/util/scripts/install.sh).
-#
-# Copyright 1991 by the Massachusetts Institute of Technology
-#
-# Permission to use, copy, modify, distribute, and sell this software and its
-# documentation for any purpose is hereby granted without fee, provided that
-# the above copyright notice appear in all copies and that both that
-# copyright notice and this permission notice appear in supporting
-# documentation, and that the name of M.I.T. not be used in advertising or
-# publicity pertaining to distribution of the software without specific,
-# written prior permission.  M.I.T. makes no representations about the
-# suitability of this software for any purpose.  It is provided "as is"
-# without express or implied warranty.
-#
-# Calling this script install-sh is preferred over install.sh, to prevent
-# `make' implicit rules from creating a file called install from it
-# when there is no Makefile.
-#
-# This script is compatible with the BSD install script, but was written
-# from scratch.  It can only install one file at a time, a restriction
-# shared with many OS's install programs.
-
-
-# set DOITPROG to echo to test this script
-
-# Don't use :- since 4.3BSD and earlier shells don't like it.
-doit="${DOITPROG-}"
-
-
-# put in absolute paths if you don't have them in your path; or use env. vars.
-
-mvprog="${MVPROG-mv}"
-cpprog="${CPPROG-cp}"
-chmodprog="${CHMODPROG-chmod}"
-chownprog="${CHOWNPROG-chown}"
-chgrpprog="${CHGRPPROG-chgrp}"
-stripprog="${STRIPPROG-strip}"
-rmprog="${RMPROG-rm}"
-mkdirprog="${MKDIRPROG-mkdir}"
-
-transformbasename=""
-transform_arg=""
-instcmd="$mvprog"
-chmodcmd="$chmodprog 0755"
-chowncmd=""
-chgrpcmd=""
-stripcmd=""
-rmcmd="$rmprog -f"
-mvcmd="$mvprog"
-src=""
-dst=""
-dir_arg=""
-
-while [ x"$1" != x ]; do
-    case $1 in
-	-c) instcmd="$cpprog"
-	    shift
-	    continue;;
-
-	-d) dir_arg=true
-	    shift
-	    continue;;
-
-	-m) chmodcmd="$chmodprog $2"
-	    shift
-	    shift
-	    continue;;
-
-	-o) chowncmd="$chownprog $2"
-	    shift
-	    shift
-	    continue;;
-
-	-g) chgrpcmd="$chgrpprog $2"
-	    shift
-	    shift
-	    continue;;
-
-	-s) stripcmd="$stripprog"
-	    shift
-	    continue;;
-
-	-t=*) transformarg=`echo $1 | sed 's/-t=//'`
-	    shift
-	    continue;;
-
-	-b=*) transformbasename=`echo $1 | sed 's/-b=//'`
-	    shift
-	    continue;;
-
-	*)  if [ x"$src" = x ]
-	    then
-		src=$1
-	    else
-		# this colon is to work around a 386BSD /bin/sh bug
-		:
-		dst=$1
-	    fi
-	    shift
-	    continue;;
-    esac
-done
-
-if [ x"$src" = x ]
-then
-	echo "install:	no input file specified"
-	exit 1
-else
-	true
-fi
-
-if [ x"$dir_arg" != x ]; then
-	dst=$src
-	src=""
-
-	if [ -d $dst ]; then
-		instcmd=:
-	else
-		instcmd=mkdir
-	fi
-else
-
-# Waiting for this to be detected by the "$instcmd $src $dsttmp" command
-# might cause directories to be created, which would be especially bad
-# if $src (and thus $dsttmp) contains '*'.
-
-	if [ -f $src -o -d $src ]
-	then
-		true
-	else
-		echo "install:  $src does not exist"
-		exit 1
-	fi
-
-	if [ x"$dst" = x ]
-	then
-		echo "install:	no destination specified"
-		exit 1
-	else
-		true
-	fi
-
-# If destination is a directory, append the input filename; if your system
-# does not like double slashes in filenames, you may need to add some logic
-
-	if [ -d $dst ]
-	then
-		dst="$dst"/`basename $src`
-	else
-		true
-	fi
-fi
-
-## this sed command emulates the dirname command
-dstdir=`echo $dst | sed -e 's,[^/]*$,,;s,/$,,;s,^$,.,'`
-
-# Make sure that the destination directory exists.
-#  this part is taken from Noah Friedman's mkinstalldirs script
-
-# Skip lots of stat calls in the usual case.
-if [ ! -d "$dstdir" ]; then
-defaultIFS='
-'
-IFS="${IFS-${defaultIFS}}"
-
-oIFS="${IFS}"
-# Some sh's can't handle IFS=/ for some reason.
-IFS='%'
-set - `echo ${dstdir} | sed -e 's@/@%@g' -e 's@^%@/@'`
-IFS="${oIFS}"
-
-pathcomp=''
-
-while [ $# -ne 0 ] ; do
-	pathcomp="${pathcomp}${1}"
-	shift
-
-	if [ ! -d "${pathcomp}" ] ;
-        then
-		$mkdirprog "${pathcomp}"
-	else
-		true
-	fi
-
-	pathcomp="${pathcomp}/"
-done
-fi
-
-if [ x"$dir_arg" != x ]
-then
-	$doit $instcmd $dst &&
-
-	if [ x"$chowncmd" != x ]; then $doit $chowncmd $dst; else true ; fi &&
-	if [ x"$chgrpcmd" != x ]; then $doit $chgrpcmd $dst; else true ; fi &&
-	if [ x"$stripcmd" != x ]; then $doit $stripcmd $dst; else true ; fi &&
-	if [ x"$chmodcmd" != x ]; then $doit $chmodcmd $dst; else true ; fi
-else
-
-# If we're going to rename the final executable, determine the name now.
-
-	if [ x"$transformarg" = x ]
-	then
-		dstfile=`basename $dst`
-	else
-		dstfile=`basename $dst $transformbasename |
-			sed $transformarg`$transformbasename
-	fi
-
-# don't allow the sed command to completely eliminate the filename
-
-	if [ x"$dstfile" = x ]
-	then
-		dstfile=`basename $dst`
-	else
-		true
-	fi
-
-# Make a temp file name in the proper directory.
-
-	dsttmp=$dstdir/#inst.$$#
-
-# Move or copy the file name to the temp name
-
-	$doit $instcmd $src $dsttmp &&
-
-	trap "rm -f ${dsttmp}" 0 &&
-
-# and set any options; do chmod last to preserve setuid bits
-
-# If any of these fail, we abort the whole thing.  If we want to
-# ignore errors from any of these, just make sure not to ignore
-# errors from the above "$doit $instcmd $src $dsttmp" command.
-
-	if [ x"$chowncmd" != x ]; then $doit $chowncmd $dsttmp; else true;fi &&
-	if [ x"$chgrpcmd" != x ]; then $doit $chgrpcmd $dsttmp; else true;fi &&
-	if [ x"$stripcmd" != x ]; then $doit $stripcmd $dsttmp; else true;fi &&
-	if [ x"$chmodcmd" != x ]; then $doit $chmodcmd $dsttmp; else true;fi &&
-
-# Now rename the file to the real destination.
-
-	$doit $rmcmd -f $dstdir/$dstfile &&
-	$doit $mvcmd $dsttmp $dstdir/$dstfile
-
-fi &&
-
-
-exit 0
diff -pruN 1:13.23.1~dfsg-2/main/editline/key.c 1:16.1.1~dfsg-1/main/editline/key.c
--- 1:13.23.1~dfsg-2/main/editline/key.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/key.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,687 +0,0 @@
-/*	$NetBSD: key.c,v 1.13 2002/03/18 16:00:55 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)key.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: key.c,v 1.13 2002/03/18 16:00:55 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * key.c: This module contains the procedures for maintaining
- *	  the extended-key map.
- *
- *      An extended-key (key) is a sequence of keystrokes introduced
- *	with an sequence introducer and consisting of an arbitrary
- *	number of characters.  This module maintains a map (the el->el_key.map)
- *	to convert these extended-key sequences into input strs
- *	(XK_STR), editor functions (XK_CMD), or unix commands (XK_EXE).
- *
- *      Warning:
- *	  If key is a substr of some other keys, then the longer
- *	  keys are lost!!  That is, if the keys "abcd" and "abcef"
- *	  are in el->el_key.map, adding the key "abc" will cause the first two
- *	  definitions to be lost.
- *
- *      Restrictions:
- *      -------------
- *      1) It is not possible to have one key that is a
- *	   substr of another.
- */
-#include <string.h>
-#include <stdlib.h>
-
-#include "el.h"
-
-/*
- * The Nodes of the el->el_key.map.  The el->el_key.map is a linked list
- * of these node elements
- */
-struct key_node_t {
-	char		ch;		/* single character of key 	 */
-	int		type;		/* node type			 */
-	key_value_t	val;		/* command code or pointer to str,  */
-					/* if this is a leaf 		 */
-	struct key_node_t *next;	/* ptr to next char of this key  */
-	struct key_node_t *sibling;	/* ptr to another key with same prefix*/
-};
-
-private int		 node_trav(EditLine *, key_node_t *, char *,
-    key_value_t *);
-private int		 node__try(EditLine *, key_node_t *, const char *,
-    key_value_t *, int);
-private key_node_t	*node__get(int);
-private void		 node__put(EditLine *, key_node_t *);
-private int		 node__delete(EditLine *, key_node_t **, const char *);
-private int		 node_lookup(EditLine *, const char *, key_node_t *,
-    int);
-private int		 node_enum(EditLine *, key_node_t *, int);
-private int		 key__decode_char(char *, int, int);
-
-#define	KEY_BUFSIZ	EL_BUFSIZ
-
-
-/* key_init():
- *	Initialize the key maps
- */
-protected int
-key_init(EditLine *el)
-{
-
-	el->el_key.buf = (char *) el_malloc(KEY_BUFSIZ);
-	if (el->el_key.buf == NULL)
-		return (-1);
-	el->el_key.map = NULL;
-	key_reset(el);
-	return (0);
-}
-
-
-/* key_end():
- *	Free the key maps
- */
-protected void
-key_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_key.buf);
-	el->el_key.buf = NULL;
-	node__put(el, el->el_key.map);
-	el->el_key.map = NULL;
-}
-
-
-/* key_map_cmd():
- *	Associate cmd with a key value
- */
-protected key_value_t *
-key_map_cmd(EditLine *el, int cmd)
-{
-
-	el->el_key.val.cmd = (el_action_t) cmd;
-	return (&el->el_key.val);
-}
-
-
-/* key_map_str():
- *	Associate str with a key value
- */
-protected key_value_t *
-key_map_str(EditLine *el, char *str)
-{
-
-	el->el_key.val.str = str;
-	return (&el->el_key.val);
-}
-
-
-/* key_reset():
- *	Takes all nodes on el->el_key.map and puts them on free list.  Then
- *	initializes el->el_key.map with arrow keys
- *	[Always bind the ansi arrow keys?]
- */
-protected void
-key_reset(EditLine *el)
-{
-
-	node__put(el, el->el_key.map);
-	el->el_key.map = NULL;
-	return;
-}
-
-
-/* key_get():
- *	Calls the recursive function with entry point el->el_key.map
- *      Looks up *ch in map and then reads characters until a
- *      complete match is found or a mismatch occurs. Returns the
- *      type of the match found (XK_STR, XK_CMD, or XK_EXE).
- *      Returns NULL in val.str and XK_STR for no match.
- *      The last character read is returned in *ch.
- */
-protected int
-key_get(EditLine *el, char *ch, key_value_t *val)
-{
-
-	return (node_trav(el, el->el_key.map, ch, val));
-}
-
-
-/* key_add():
- *      Adds key to the el->el_key.map and associates the value in val with it.
- *      If key is already is in el->el_key.map, the new code is applied to the
- *      existing key. Ntype specifies if code is a command, an
- *      out str or a unix command.
- */
-protected void
-key_add(EditLine *el, const char *key, key_value_t *val, int ntype)
-{
-
-	if (key[0] == '\0') {
-		(void) fprintf(el->el_errfile,
-		    "key_add: Null extended-key not allowed.\n");
-		return;
-	}
-	if (ntype == XK_CMD && val->cmd == ED_SEQUENCE_LEAD_IN) {
-		(void) fprintf(el->el_errfile,
-		    "key_add: sequence-lead-in command not allowed\n");
-		return;
-	}
-	if (el->el_key.map == NULL)
-		/* tree is initially empty.  Set up new node to match key[0] */
-		el->el_key.map = node__get(key[0]);
-			/* it is properly initialized */
-
-	/* Now recurse through el->el_key.map */
-	(void) node__try(el, el->el_key.map, key, val, ntype);
-	return;
-}
-
-
-/* key_clear():
- *
- */
-protected void
-key_clear(EditLine *el, el_action_t *map, const char *in)
-{
-
-	if ((map[(unsigned char)*in] == ED_SEQUENCE_LEAD_IN) &&
-	    ((map == el->el_map.key &&
-	    el->el_map.alt[(unsigned char)*in] != ED_SEQUENCE_LEAD_IN) ||
-	    (map == el->el_map.alt &&
-	    el->el_map.key[(unsigned char)*in] != ED_SEQUENCE_LEAD_IN)))
-		(void) key_delete(el, in);
-}
-
-
-/* key_delete():
- *      Delete the key and all longer keys staring with key, if
- *      they exists.
- */
-protected int
-key_delete(EditLine *el, const char *key)
-{
-
-	if (key[0] == '\0') {
-		(void) fprintf(el->el_errfile,
-		    "key_delete: Null extended-key not allowed.\n");
-		return (-1);
-	}
-	if (el->el_key.map == NULL)
-		return (0);
-
-	(void) node__delete(el, &el->el_key.map, key);
-	return (0);
-}
-
-
-/* key_print():
- *	Print the binding associated with key key.
- *	Print entire el->el_key.map if null
- */
-protected void
-key_print(EditLine *el, const char *key)
-{
-
-	/* do nothing if el->el_key.map is empty and null key specified */
-	if (el->el_key.map == NULL && *key == 0)
-		return;
-
-	el->el_key.buf[0] = '"';
-	if (node_lookup(el, key, el->el_key.map, 1) <= -1)
-		/* key is not bound */
-		(void) fprintf(el->el_errfile, "Unbound extended key \"%s\"\n",
-		    key);
-	return;
-}
-
-
-/* node_trav():
- *	recursively traverses node in tree until match or mismatch is
- * 	found.  May read in more characters.
- */
-private int
-node_trav(EditLine *el, key_node_t *ptr, char *ch, key_value_t *val)
-{
-
-	if (ptr->ch == *ch) {
-		/* match found */
-		if (ptr->next) {
-			/* key not complete so get next char */
-			if (el_getc(el, ch) != 1) {	/* if EOF or error */
-				val->cmd = ED_END_OF_FILE;
-				return (XK_CMD);
-				/* PWP: Pretend we just read an end-of-file */
-			}
-			return (node_trav(el, ptr->next, ch, val));
-		} else {
-			*val = ptr->val;
-			if (ptr->type != XK_CMD)
-				*ch = '\0';
-			return (ptr->type);
-		}
-	} else {
-		/* no match found here */
-		if (ptr->sibling) {
-			/* try next sibling */
-			return (node_trav(el, ptr->sibling, ch, val));
-		} else {
-			/* no next sibling -- mismatch */
-			val->str = NULL;
-			return (XK_STR);
-		}
-	}
-}
-
-
-/* node__try():
- * 	Find a node that matches *str or allocate a new one
- */
-private int
-node__try(EditLine *el, key_node_t *ptr, const char *str, key_value_t *val, int ntype)
-{
-
-	if (ptr->ch != *str) {
-		key_node_t *xm;
-
-		for (xm = ptr; xm->sibling != NULL; xm = xm->sibling)
-			if (xm->sibling->ch == *str)
-				break;
-		if (xm->sibling == NULL)
-			xm->sibling = node__get(*str);	/* setup new node */
-		ptr = xm->sibling;
-	}
-	if (*++str == '\0') {
-		/* we're there */
-		if (ptr->next != NULL) {
-			node__put(el, ptr->next);
-				/* lose longer keys with this prefix */
-			ptr->next = NULL;
-		}
-		switch (ptr->type) {
-		case XK_CMD:
-		case XK_NOD:
-			break;
-		case XK_STR:
-		case XK_EXE:
-			if (ptr->val.str)
-				el_free((ptr_t) ptr->val.str);
-			break;
-		default:
-			EL_ABORT((el->el_errfile, "Bad XK_ type %d\n",
-			    ptr->type));
-			break;
-		}
-
-		switch (ptr->type = ntype) {
-		case XK_CMD:
-			ptr->val = *val;
-			break;
-		case XK_STR:
-		case XK_EXE:
-			ptr->val.str = strdup(val->str);
-			break;
-		default:
-			EL_ABORT((el->el_errfile, "Bad XK_ type %d\n", ntype));
-			break;
-		}
-	} else {
-		/* still more chars to go */
-		if (ptr->next == NULL)
-			ptr->next = node__get(*str);	/* setup new node */
-		(void) node__try(el, ptr->next, str, val, ntype);
-	}
-	return (0);
-}
-
-
-/* node__delete():
- *	Delete node that matches str
- */
-private int
-node__delete(EditLine *el, key_node_t **inptr, const char *str)
-{
-	key_node_t *ptr;
-	key_node_t *prev_ptr = NULL;
-
-	ptr = *inptr;
-
-	if (ptr->ch != *str) {
-		key_node_t *xm;
-
-		for (xm = ptr; xm->sibling != NULL; xm = xm->sibling)
-			if (xm->sibling->ch == *str)
-				break;
-		if (xm->sibling == NULL)
-			return (0);
-		prev_ptr = xm;
-		ptr = xm->sibling;
-	}
-	if (*++str == '\0') {
-		/* we're there */
-		if (prev_ptr == NULL)
-			*inptr = ptr->sibling;
-		else
-			prev_ptr->sibling = ptr->sibling;
-		ptr->sibling = NULL;
-		node__put(el, ptr);
-		return (1);
-	} else if (ptr->next != NULL &&
-	    node__delete(el, &ptr->next, str) == 1) {
-		if (ptr->next != NULL)
-			return (0);
-		if (prev_ptr == NULL)
-			*inptr = ptr->sibling;
-		else
-			prev_ptr->sibling = ptr->sibling;
-		ptr->sibling = NULL;
-		node__put(el, ptr);
-		return (1);
-	} else {
-		return (0);
-	}
-}
-
-
-/* node__put():
- *	Puts a tree of nodes onto free list using free(3).
- */
-private void
-node__put(EditLine *el, key_node_t *ptr)
-{
-	if (ptr == NULL)
-		return;
-
-	if (ptr->next != NULL) {
-		node__put(el, ptr->next);
-		ptr->next = NULL;
-	}
-	node__put(el, ptr->sibling);
-
-	switch (ptr->type) {
-	case XK_CMD:
-	case XK_NOD:
-		break;
-	case XK_EXE:
-	case XK_STR:
-		if (ptr->val.str != NULL)
-			el_free((ptr_t) ptr->val.str);
-		break;
-	default:
-		EL_ABORT((el->el_errfile, "Bad XK_ type %d\n", ptr->type));
-		break;
-	}
-	el_free((ptr_t) ptr);
-}
-
-
-/* node__get():
- *	Returns pointer to an key_node_t for ch.
- */
-private key_node_t *
-node__get(int ch)
-{
-	key_node_t *ptr;
-
-	ptr = (key_node_t *) el_malloc((size_t) sizeof(key_node_t));
-	if (ptr == NULL)
-		return NULL;
-	ptr->ch = ch;
-	ptr->type = XK_NOD;
-	ptr->val.str = NULL;
-	ptr->next = NULL;
-	ptr->sibling = NULL;
-	return (ptr);
-}
-
-
-
-/* node_lookup():
- *	look for the str starting at node ptr.
- *	Print if last node
- */
-private int
-node_lookup(EditLine *el, const char *str, key_node_t *ptr, int cnt)
-{
-	int ncnt;
-
-	if (ptr == NULL)
-		return (-1);	/* cannot have null ptr */
-
-	if (*str == 0) {
-		/* no more chars in str.  node_enum from here. */
-		(void) node_enum(el, ptr, cnt);
-		return (0);
-	} else {
-		/* If match put this char into el->el_key.buf.  Recurse */
-		if (ptr->ch == *str) {
-			/* match found */
-			ncnt = key__decode_char(el->el_key.buf, cnt,
-			    (unsigned char) ptr->ch);
-			if (ptr->next != NULL)
-				/* not yet at leaf */
-				return (node_lookup(el, str + 1, ptr->next,
-				    ncnt + 1));
-			else {
-			    /* next node is null so key should be complete */
-				if (str[1] == 0) {
-					el->el_key.buf[ncnt + 1] = '"';
-					el->el_key.buf[ncnt + 2] = '\0';
-					key_kprint(el, el->el_key.buf,
-					    &ptr->val, ptr->type);
-					return (0);
-				} else
-					return (-1);
-					/* mismatch -- str still has chars */
-			}
-		} else {
-			/* no match found try sibling */
-			if (ptr->sibling)
-				return (node_lookup(el, str, ptr->sibling,
-				    cnt));
-			else
-				return (-1);
-		}
-	}
-}
-
-
-/* node_enum():
- *	Traverse the node printing the characters it is bound in buffer
- */
-private int
-node_enum(EditLine *el, key_node_t *ptr, int cnt)
-{
-	int ncnt;
-
-	if (cnt >= KEY_BUFSIZ - 5) {	/* buffer too small */
-		el->el_key.buf[++cnt] = '"';
-		el->el_key.buf[++cnt] = '\0';
-		(void) fprintf(el->el_errfile,
-		    "Some extended keys too long for internal print buffer");
-		(void) fprintf(el->el_errfile, " \"%s...\"\n", el->el_key.buf);
-		return (0);
-	}
-	if (ptr == NULL) {
-#ifdef DEBUG_EDIT
-		(void) fprintf(el->el_errfile,
-		    "node_enum: BUG!! Null ptr passed\n!");
-#endif
-		return (-1);
-	}
-	/* put this char at end of str */
-	ncnt = key__decode_char(el->el_key.buf, cnt, (unsigned char) ptr->ch);
-	if (ptr->next == NULL) {
-		/* print this key and function */
-		el->el_key.buf[ncnt + 1] = '"';
-		el->el_key.buf[ncnt + 2] = '\0';
-		key_kprint(el, el->el_key.buf, &ptr->val, ptr->type);
-	} else
-		(void) node_enum(el, ptr->next, ncnt + 1);
-
-	/* go to sibling if there is one */
-	if (ptr->sibling)
-		(void) node_enum(el, ptr->sibling, cnt);
-	return (0);
-}
-
-
-/* key_kprint():
- *	Print the specified key and its associated
- *	function specified by val
- */
-protected void
-key_kprint(EditLine *el, const char *key, key_value_t *val, int ntype)
-{
-	el_bindings_t *fp;
-	char unparsbuf[EL_BUFSIZ];
-	static const char fmt[] = "%-15s->  %s\n";
-
-	if (val != NULL)
-		switch (ntype) {
-		case XK_STR:
-		case XK_EXE:
-			(void) fprintf(el->el_outfile, fmt, key,
-			    key__decode_str(val->str, unparsbuf,
-				ntype == XK_STR ? "\"\"" : "[]"));
-			break;
-		case XK_CMD:
-			for (fp = el->el_map.help; fp->name; fp++)
-				if (val->cmd == fp->func) {
-					(void) fprintf(el->el_outfile, fmt,
-					    key, fp->name);
-					break;
-				}
-#ifdef DEBUG_KEY
-			if (fp->name == NULL)
-				(void) fprintf(el->el_outfile,
-				    "BUG! Command not found.\n");
-#endif
-
-			break;
-		default:
-			EL_ABORT((el->el_errfile, "Bad XK_ type %d\n", ntype));
-			break;
-		}
-	else
-		(void) fprintf(el->el_outfile, fmt, key, "no input");
-}
-
-
-/* key__decode_char():
- *	Put a printable form of char in buf.
- */
-private int
-key__decode_char(char *buf, int cnt, int ch)
-{
-	if (ch == 0) {
-		buf[cnt++] = '^';
-		buf[cnt] = '@';
-		return (cnt);
-	}
-	if (iscntrl(ch)) {
-		buf[cnt++] = '^';
-		if (ch == '\177')
-			buf[cnt] = '?';
-		else
-			buf[cnt] = ch | 0100;
-	} else if (ch == '^') {
-		buf[cnt++] = '\\';
-		buf[cnt] = '^';
-	} else if (ch == '\\') {
-		buf[cnt++] = '\\';
-		buf[cnt] = '\\';
-	} else if (ch == ' ' || (isprint(ch) && !isspace(ch))) {
-		buf[cnt] = ch;
-	} else {
-		buf[cnt++] = '\\';
-		buf[cnt++] = (((unsigned int) ch >> 6) & 7) + '0';
-		buf[cnt++] = (((unsigned int) ch >> 3) & 7) + '0';
-		buf[cnt] = (ch & 7) + '0';
-	}
-	return (cnt);
-}
-
-
-/* key__decode_str():
- *	Make a printable version of the ey
- */
-protected char *
-key__decode_str(const char *str, char *buf, const char *sep)
-{
-	char *b;
-	const char *p;
-
-	b = buf;
-	if (sep[0] != '\0')
-		*b++ = sep[0];
-	if (*str == 0) {
-		*b++ = '^';
-		*b++ = '@';
-		if (sep[0] != '\0' && sep[1] != '\0')
-			*b++ = sep[1];
-		*b++ = 0;
-		return (buf);
-	}
-	for (p = str; *p != 0; p++) {
-		if (iscntrl((unsigned char) *p)) {
-			*b++ = '^';
-			if (*p == '\177')
-				*b++ = '?';
-			else
-				*b++ = *p | 0100;
-		} else if (*p == '^' || *p == '\\') {
-			*b++ = '\\';
-			*b++ = *p;
-		} else if (*p == ' ' || (isprint((unsigned char) *p) &&
-			!isspace((unsigned char) *p))) {
-			*b++ = *p;
-		} else {
-			*b++ = '\\';
-			*b++ = (((unsigned int) *p >> 6) & 7) + '0';
-			*b++ = (((unsigned int) *p >> 3) & 7) + '0';
-			*b++ = (*p & 7) + '0';
-		}
-	}
-	if (sep[0] != '\0' && sep[1] != '\0')
-		*b++ = sep[1];
-	*b++ = 0;
-	return (buf);		/* should check for overflow */
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/key.h 1:16.1.1~dfsg-1/main/editline/key.h
--- 1:13.23.1~dfsg-2/main/editline/key.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/key.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,79 +0,0 @@
-/*	$NetBSD: key.h,v 1.6 2002/03/18 16:00:55 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)key.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.key.h: Key macro header
- */
-#ifndef _h_el_key
-#define	_h_el_key
-
-typedef union key_value_t {
-	el_action_t	 cmd;	/* If it is a command the #	*/
-	char		*str;	/* If it is a string...		*/
-} key_value_t;
-
-typedef struct key_node_t key_node_t;
-
-typedef struct el_key_t {
-	char		*buf;	/* Key print buffer		*/
-	key_node_t	*map;	/* Key map			*/
-	key_value_t	 val;	/* Local conversion buffer	*/
-} el_key_t;
-
-#define	XK_CMD	0
-#define	XK_STR	1
-#define	XK_NOD	2
-#define	XK_EXE	3
-
-protected int		 key_init(EditLine *);
-protected void		 key_end(EditLine *);
-protected key_value_t	*key_map_cmd(EditLine *, int);
-protected key_value_t	*key_map_str(EditLine *, char *);
-protected void		 key_reset(EditLine *);
-protected int		 key_get(EditLine *, char *, key_value_t *);
-protected void		 key_add(EditLine *, const char *, key_value_t *, int);
-protected void		 key_clear(EditLine *, el_action_t *, const char *);
-protected int		 key_delete(EditLine *, const char *);
-protected void		 key_print(EditLine *, const char *);
-protected void	         key_kprint(EditLine *, const char *, key_value_t *,
-    int);
-protected char		*key__decode_str(const char *, char *, const char *);
-
-#endif /* _h_el_key */
diff -pruN 1:13.23.1~dfsg-2/main/editline/Makefile.in 1:16.1.1~dfsg-1/main/editline/Makefile.in
--- 1:13.23.1~dfsg-2/main/editline/Makefile.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/Makefile.in	1970-01-01 00:00:00.000000000 +0000
@@ -1,234 +0,0 @@
-#
-# Generic Makefile for libedit.
-#
-
-OSTYPE=$(shell uname -s)
-define cyg_subst_sys
-	if uname -s | grep -i cygwin > /dev/null; then \
-	cat $@ | sed -e s/"sys\.h"/"config.h"/g > $@.copy; \
-	mv --force $@.copy $@; \
-	fi
-endef
-
-SHELL = /bin/sh
-
-CC = @CC@
-AR = @AR@
-RANLIB = @RANLIB@
-CPPFLAGS = @CPPFLAGS@ -I.
-CFLAGS = @CFLAGS@
-A_CFLAGS = @A_CFLAGS@
-S_CFLAGS = @S_CFLAGS@
-LDFLAGS = @LDFLAGS@
-S_LDFLAGS = @S_LDFLAGS@
-LIBS = @LIBS@
-
-INSTALL = @INSTALL@
-PREFIX = @prefix@
-
-ifeq ($(OSTYPE),SunOS)
-CFLAGS+=-DSOLARIS -I../../include/solaris-compat
-endif
-
-# .c files.
-ACSRCS = @ACSRCS@
-BCSRCS = @BCSRCS@
-CCSRCS = @CCSRCS@
-
-# Generated .c files.
-AGCSRCS = @AGCSRCS@
-BGCSRCS = @BGCSRCS@
-
-# .h files.
-HDRS = @HDRS@
-
-# Generated .h files.
-AGHDRS = @AGHDRS@
-BGHDRS = @BGHDRS@
-
-# Installed .h files.
-IHDRS = @IHDRS@
-IHDR_LINKS = @IHDR_LINKS@
-HDR_DIRS = @HDR_DIRS@
-
-# Man pages.
-MAN3 = @MAN3@
-MAN5 = @MAN5@
-MAN3_LINKS = @MAN3_LINKS@
-MAN_DIRS = @MAN_DIRS@
-
-# Library.
-LIB_DIRS = @LIB_DIRS@
-LIB_VER = @LIB_VER@
-LIB_A = @LIB_A@
-LIB_A_LINKS = @LIB_A_LINKS@
-LIB_S = @LIB_S@
-LIB_S_LINKS = @LIB_S_LINKS@
-
-# Test program.
-TEST = @TEST@
-TCSRCS = @TCSRCS@
-
-# Clear out all paths, then set just one (default path) for the main build
-# directory.
-.PATH :
-.PATH : .
-
-.SUFFIXES :
-.SUFFIXES : .c .o .o_a .o_s
-
-all :  lib_a lib_s
-
-lib_a : $(LIB_A)
-lib_s : $(LIB_S)
-
-test : $(TEST)
-
-install : install_hdr install_lib install_man
-
-install_hdr :
-	@for i in $(HDR_DIRS) ; do \
-		echo "$(INSTALL) -d $(PREFIX)/$$i/"; \
-		$(INSTALL) -d $(PREFIX)/$$i/; \
-	done
-	@for i in $(IHDRS); do \
-		echo "$(INSTALL) -m 0444 $$i $(PREFIX)/include/`dirname $$i`/"; \
-		$(INSTALL) -m 0444 $$i $(PREFIX)/include/`dirname $$i`/; \
-	done
-	@f=; \
-	for i in $(IHDR_LINKS) ""; do \
-		if test -z "$$f" ; then \
-			f=$$i; \
-		else \
-			echo "rm -f $(PREFIX)/include/$$i"; \
-			rm -f $(PREFIX)/include/$$i; \
-			echo "ln -s $$f $(PREFIX)/include/$$i"; \
-			ln -s $$f $(PREFIX)/include/$$i; \
-			f=; \
-		fi; \
-	done
-
-install_lib : install_lib_a install_lib_s
-
-install_lib_common :
-	@for i in $(LIB_DIRS) ; do \
-		echo "$(INSTALL) -d $(PREFIX)/$$i/"; \
-		$(INSTALL) -d $(PREFIX)/$$i/; \
-	done
-
-install_lib_a : $(LIB_A) install_lib_common
-	$(INSTALL) -m 0644 $(LIB_A) $(PREFIX)/lib/
-	@f=; \
-	for i in $(LIB_A_LINKS) ""; do \
-		if test -z "$$f" ; then \
-			f=$$i; \
-		else \
-			echo "rm -f $(PREFIX)/lib/$$i"; \
-			rm -f $(PREFIX)/lib/$$i; \
-			echo "ln -s $$f $(PREFIX)/lib/$$i"; \
-			ln -s $$f $(PREFIX)/lib/$$i; \
-			f=; \
-		fi; \
-	done
-
-install_lib_s : $(LIB_S) install_lib_common
-	$(INSTALL) -m 0755 $(LIB_S) $(PREFIX)/lib/
-	@f=; \
-	for i in $(LIB_S_LINKS) ""; do \
-		if test -z "$$f" ; then \
-			f=$$i; \
-		else \
-			echo "rm -f $(PREFIX)/lib/$$i"; \
-			rm -f $(PREFIX)/lib/$$i; \
-			echo "ln -s $$f $(PREFIX)/lib/$$i"; \
-			ln -s $$f $(PREFIX)/lib/$$i; \
-			f=; \
-		fi; \
-	done
-
-install_man :
-	@for i in $(MAN_DIRS) ; do \
-		echo "$(INSTALL) -d $(PREFIX)/$$i/"; \
-		$(INSTALL) -d $(PREFIX)/$$i/; \
-	done
-	@for i in $(MAN3); do \
-		echo $(INSTALL) -m 0444 $$i $(PREFIX)/man/man3/; \
-		$(INSTALL) -m 0444 $$i $(PREFIX)/man/man3/; \
-	done
-	@f=; \
-	for i in $(MAN3_LINKS) ""; do \
-		if test -z "$$f" ; then \
-			f=$$i; \
-		else \
-			echo "rm -f $(PREFIX)/man/man3/$$i"; \
-			rm -f $(PREFIX)/man/man3/$$i; \
-			echo "ln -s $$f $(PREFIX)/man/man3/$$i"; \
-			ln -s $$f $(PREFIX)/man/man3/$$i; \
-			f=; \
-		fi; \
-	done
-	@for i in $(MAN5); do\
-		echo $(INSTALL) -m 0444 $$i $(PREFIX)/man/man5/; \
-		$(INSTALL) -m 0444 $$i $(PREFIX)/man/man5/; \
-	done
-
-clean :
-	rm -f $(AGCSRCS) $(BGCSRCS) $(AGHDRS) $(BGHDRS) $(LIB_A) $(LIB_S)
-	rm -f $(BGCSRCS:.c=.o_a) $(CCSRCS:.c=.o_a)
-	rm -f $(BGCSRCS:.c=.o_s) $(CCSRCS:.c=.o_s)
-	rm -f $(TCSRCS:.c=.o) $(TEST)
-	rm -f *.s *.i
-
-distclean : clean
-	rm -f config.cache config.log config.status config.h makelist Makefile
-
-#
-# Internal targets and rules.
-#
-
-$(LIB_A) : $(BGCSRCS:.c=.o_a) $(CCSRCS:.c=.o_a)
-	$(AR) cr $@ $?
-	$(RANLIB) $@
-
-$(LIB_S) : $(BGCSRCS:.c=.o_s) $(CCSRCS:.c=.o_s)
-	$(CC) $(S_LDFLAGS) -o $@ $(BGCSRCS:.c=.o_s) $(CCSRCS:.c=.o_s) $(LIBS)
-
-$(TEST) : $(TCSRCS:.c=.o) $(LIB_A)
-	$(CC) -o $@ $(TCSRCS:.c=.o) $(LIB_A) $(LIBS)
-
-common.h : common.c
-	$(SHELL) makelist -h common.c > $@
-
-emacs.h : emacs.c
-	$(SHELL) makelist -h emacs.c> $@
-
-vi.h : vi.c
-	$(SHELL) makelist -h vi.c > $@
-
-fcns.h : $(AGHDRS)
-	$(SHELL) makelist -fh $(AGHDRS) > $@
-
-fcns.c : $(AGHDRS) fcns.h
-	$(SHELL) makelist -fc $(AGHDRS) > $@
-	$(cyg_subst_sys)
-
-help.h : $(ACSRCS)
-	$(SHELL) makelist -bh $(ACSRCS) > $@
-
-help.c : $(ACSRCS) help.h
-	$(SHELL) makelist -bc $(ACSRCS) > $@
-	$(cyg_subst_sys)
-
-editline.c : $(ACSRCS) $(BCSRCS) $(AGCSRCS)
-	$(SHELL) makelist -e $(ACSRCS) $(BCSRCS) $(AGCSRCS) > $@
-
-.c.o :
-	$(CC) -c $(A_CFLAGS) $(CFLAGS) $(CPPFLAGS) $< -o $@
-
-.c.o_a : $(AGHDRS) $(BGHDRS)
-	$(CC) -c $(A_CFLAGS) $(CFLAGS) $(CPPFLAGS) $< -o $@
-
-.c.o_s : $(AGHDRS) $(BGHDRS)
-	$(CC) -c $(S_CFLAGS) $(CFLAGS) $(CPPFLAGS) $< -o $@
-
-$(CCSRCS) : $(BGHDRS)
diff -pruN 1:13.23.1~dfsg-2/main/editline/makelist.in 1:16.1.1~dfsg-1/main/editline/makelist.in
--- 1:13.23.1~dfsg-2/main/editline/makelist.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/makelist.in	1970-01-01 00:00:00.000000000 +0000
@@ -1,254 +0,0 @@
-#!/bin/sh -
-#	$NetBSD: makelist,v 1.7 2001/01/09 19:22:31 jdolecek Exp $
-#
-# Copyright (c) 1992, 1993
-#	The Regents of the University of California.  All rights reserved.
-#
-# This code is derived from software contributed to Berkeley by
-# Christos Zoulas of Cornell University.
-#
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions
-# are met:
-# 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer.
-# 2. Redistributions in binary form must reproduce the above copyright
-#    notice, this list of conditions and the following disclaimer in the
-#    documentation and/or other materials provided with the distribution.
-# 3. All advertising materials mentioning features or use of this software
-#    must display the following acknowledgement:
-#	This product includes software developed by the University of
-#	California, Berkeley and its contributors.
-# 4. Neither the name of the University nor the names of its contributors
-#    may be used to endorse or promote products derived from this software
-#    without specific prior written permission.
-#
-# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
-# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-# SUCH DAMAGE.
-#
-#	@(#)makelist	5.3 (Berkeley) 6/4/93
-
-# makelist.sh: Automatically generate header files...
-
-AWK=@AWK@
-USAGE="Usage: $0 -h|-e|-fc|-fh|-bc|-bh|-m <filenames>"
-
-if [ "x$1" = "x" ]
-then
-    echo $USAGE 1>&2
-    exit 1
-fi
-
-FLAG="$1"
-shift
-
-FILES="$@"
-
-case $FLAG in
-
-#	generate foo.h file from foo.c
-#
--h)
-    set - `echo $FILES | sed -e 's/\\./_/g'`
-    hdr="_h_`basename $1`"
-    cat $FILES | $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#ifndef %s\n#define %s\n", "'$hdr'", "'$hdr'");
-	}
-	/\(\):/ {
-	    pr = substr($2, 1, 2);
-	    if (pr == "vi" || pr == "em" || pr == "ed") {
-		name = substr($2, 1, length($2) - 3);
-#
-# XXX:	need a space between name and prototype so that -fc and -fh
-#	parsing is much easier
-#
-		printf("protected el_action_t\t%s (EditLine *, int);\n", name);
-	    }
-	}
-	END {
-	    printf("#endif /* %s */\n", "'$hdr'");
-	}'
-	;;
-
-#	generate help.c from various .c files
-#
--bc)
-    cat $FILES | $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#include \"sys.h\"\n#include \"el.h\"\n");
-	    printf("private const struct el_bindings_t el_func_help[] = {\n");
-	    low = "abcdefghijklmnopqrstuvwxyz_";
-	    high = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_";
-	    for (i = 1; i <= length(low); i++)
-		tr[substr(low, i, 1)] = substr(high, i, 1);
-	}
-	/\(\):/ {
-	    pr = substr($2, 1, 2);
-	    if (pr == "vi" || pr == "em" || pr == "ed") {
-		name = substr($2, 1, length($2) - 3);
-		uname = "";
-		fname = "";
-		for (i = 1; i <= length(name); i++) {
-		    s = substr(name, i, 1);
-		    uname = uname tr[s];
-		    if (s == "_")
-			s = "-";
-		    fname = fname s;
-		}
-
-		printf("    { %-30.30s %-30.30s\n","\"" fname "\",", uname ",");
-		ok = 1;
-	    }
-	}
-	/^ \*/ {
-	    if (ok) {
-		printf("      \"");
-		for (i = 2; i < NF; i++)
-		    printf("%s ", $i);
-		printf("%s\" },\n", $i);
-		ok = 0;
-	    }
-	}
-	END {
-	    printf("    { NULL, 0, NULL }\n");
-	    printf("};\n");
-	    printf("\nprotected const el_bindings_t* help__get()");
-	    printf("{ return el_func_help; }\n");
-	}'
-	;;
-
-#	generate help.h from various .c files
-#
--bh)
-    $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#ifndef _h_help_c\n#define _h_help_c\n");
-	    printf("protected const el_bindings_t *help__get(void);\n");
-	    printf("#endif /* _h_help_c */\n");
-	}' /dev/null
-	;;
-
-#	generate fcns.h from various .h files
-#
--fh)
-    cat $FILES | $AWK '/el_action_t/ { print $3 }' | \
-    sort | tr '[:lower:]' '[:upper:]' | $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#ifndef _h_fcns_c\n#define _h_fcns_c\n");
-	    count = 0;
-	}
-	{
-	    printf("#define\t%-30.30s\t%3d\n", $1, count++);
-	}
-	END {
-	    printf("#define\t%-30.30s\t%3d\n", "EL_NUM_FCNS", count);
-
-	    printf("typedef el_action_t (*el_func_t)(EditLine *, int);");
-	    printf("\nprotected const el_func_t* func__get(void);\n");
-	    printf("#endif /* _h_fcns_c */\n");
-	}'
-	;;
-
-#	generate fcns.c from various .h files
-#
--fc)
-    cat $FILES | $AWK '/el_action_t/ { print $3 }' | sort | $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#include \"sys.h\"\n#include \"el.h\"\n");
-	    printf("private const el_func_t el_func[] = {");
-	    maxlen = 80;
-	    needn = 1;
-	    len = 0;
-	}
-	{
-	    clen = 25 + 2;
-	    len += clen;
-	    if (len >= maxlen)
-		needn = 1;
-	    if (needn) {
-		printf("\n    ");
-		needn = 0;
-		len = 4 + clen;
-	    }
-	    s = $1 ",";
-	    printf("%-26.26s ", s);
-	}
-	END {
-	    printf("\n};\n");
-	    printf("\nprotected const el_func_t* func__get() { return el_func; }\n");
-	}'
-	;;
-
-#	generate editline.c from various .c files
-#
--e)
-	echo "$FILES" | tr ' ' '\012' | $AWK '
-	BEGIN {
-	    printf("/* Automatically generated file, do not edit */\n");
-	    printf("#define protected static\n");
-	    printf("#define SCCSID\n");
-	}
-	{
-	    printf("#include \"%s\"\n", $1);
-	}'
-	;;
-
-#	generate man page fragment from various .c files
-#
--m)
-    cat $FILES | $AWK '
-	BEGIN {
-	    printf(".\\\" Section automatically generated with makelist\n");
-	    printf(".Bl -tag -width 4n\n");
-	}
-	/\(\):/ {
-	    pr = substr($2, 1, 2);
-	    if (pr == "vi" || pr == "em" || pr == "ed") {
-		name = substr($2, 1, length($2) - 3);
-		fname = "";
-		for (i = 1; i <= length(name); i++) {
-		    s = substr(name, i, 1);
-		    if (s == "_")
-			s = "-";
-		    fname = fname s;
-		}
-
-		printf(".It Ic %s\n", fname);
-		ok = 1;
-	    }
-	}
-	/^ \*/ {
-	    if (ok) {
-		for (i = 2; i < NF; i++)
-		    printf("%s ", $i);
-		printf("%s.\n", $i);
-		ok = 0;
-	    }
-	}
-	END {
-	    printf(".El\n");
-	    printf(".\\\" End of section automatically generated with makelist\n");
-	}'
-	;;
-
-*)
-    echo $USAGE 1>&2
-    exit 1
-    ;;
-
-esac
diff -pruN 1:13.23.1~dfsg-2/main/editline/map.c 1:16.1.1~dfsg-1/main/editline/map.c
--- 1:13.23.1~dfsg-2/main/editline/map.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/map.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,1418 +0,0 @@
-/*	$NetBSD: map.c,v 1.15 2002/03/18 16:00:55 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)map.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: map.c,v 1.15 2002/03/18 16:00:55 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * map.c: Editor function definitions
- */
-#include <stdlib.h>
-#include "el.h"
-
-#define	N_KEYS 256
-
-private void	map_print_key(EditLine *, el_action_t *, const char *);
-private void	map_print_some_keys(EditLine *, el_action_t *, int, int);
-private void	map_print_all_keys(EditLine *);
-private void	map_init_nls(EditLine *);
-private void	map_init_meta(EditLine *);
-
-/* keymap tables ; should be N_KEYS*sizeof(KEYCMD) bytes long */
-
-
-private const el_action_t  el_map_emacs[] = {
-	/*   0 */	EM_SET_MARK,		/* ^@ */
-	/*   1 */	ED_MOVE_TO_BEG,		/* ^A */
-	/*   2 */	ED_PREV_CHAR,		/* ^B */
-	/*   3 */	ED_TTY_SIGINT,		/* ^C */
-	/*   4 */	EM_DELETE_OR_LIST,	/* ^D */
-	/*   5 */	ED_MOVE_TO_END,		/* ^E */
-	/*   6 */	ED_NEXT_CHAR,		/* ^F */
-	/*   7 */	ED_UNASSIGNED,		/* ^G */
-	/*   8 */	ED_DELETE_PREV_CHAR,	/* ^H */
-	/*   9 */	ED_UNASSIGNED,		/* ^I */
-	/*  10 */	ED_NEWLINE,		/* ^J */
-	/*  11 */	ED_KILL_LINE,		/* ^K */
-	/*  12 */	ED_CLEAR_SCREEN,	/* ^L */
-	/*  13 */	ED_NEWLINE,		/* ^M */
-	/*  14 */	ED_NEXT_HISTORY,	/* ^N */
-	/*  15 */	ED_TTY_FLUSH_OUTPUT,	/* ^O */
-	/*  16 */	ED_PREV_HISTORY,	/* ^P */
-	/*  17 */	ED_TTY_START_OUTPUT,	/* ^Q */
-	/*  18 */	ED_REDISPLAY,		/* ^R */
-	/*  19 */	ED_TTY_STOP_OUTPUT,	/* ^S */
-	/*  20 */	ED_TRANSPOSE_CHARS,	/* ^T */
-	/*  21 */	EM_KILL_LINE,		/* ^U */
-	/*  22 */	ED_QUOTED_INSERT,	/* ^V */
-	/*  23 */	EM_KILL_REGION,		/* ^W */
-	/*  24 */	ED_SEQUENCE_LEAD_IN,	/* ^X */
-	/*  25 */	EM_YANK,		/* ^Y */
-	/*  26 */	ED_TTY_SIGTSTP,		/* ^Z */
-	/*  27 */	EM_META_NEXT,		/* ^[ */
-	/*  28 */	ED_TTY_SIGQUIT,		/* ^\ */
-	/*  29 */	ED_TTY_DSUSP,		/* ^] */
-	/*  30 */	ED_UNASSIGNED,		/* ^^ */
-	/*  31 */	ED_UNASSIGNED,		/* ^_ */
-	/*  32 */	ED_INSERT,		/* SPACE */
-	/*  33 */	ED_INSERT,		/* ! */
-	/*  34 */	ED_INSERT,		/* " */
-	/*  35 */	ED_INSERT,		/* # */
-	/*  36 */	ED_INSERT,		/* $ */
-	/*  37 */	ED_INSERT,		/* % */
-	/*  38 */	ED_INSERT,		/* & */
-	/*  39 */	ED_INSERT,		/* ' */
-	/*  40 */	ED_INSERT,		/* ( */
-	/*  41 */	ED_INSERT,		/* ) */
-	/*  42 */	ED_INSERT,		/* * */
-	/*  43 */	ED_INSERT,		/* + */
-	/*  44 */	ED_INSERT,		/* , */
-	/*  45 */	ED_INSERT,		/* - */
-	/*  46 */	ED_INSERT,		/* . */
-	/*  47 */	ED_INSERT,		/* / */
-	/*  48 */	ED_DIGIT,		/* 0 */
-	/*  49 */	ED_DIGIT,		/* 1 */
-	/*  50 */	ED_DIGIT,		/* 2 */
-	/*  51 */	ED_DIGIT,		/* 3 */
-	/*  52 */	ED_DIGIT,		/* 4 */
-	/*  53 */	ED_DIGIT,		/* 5 */
-	/*  54 */	ED_DIGIT,		/* 6 */
-	/*  55 */	ED_DIGIT,		/* 7 */
-	/*  56 */	ED_DIGIT,		/* 8 */
-	/*  57 */	ED_DIGIT,		/* 9 */
-	/*  58 */	ED_INSERT,		/* : */
-	/*  59 */	ED_INSERT,		/* ; */
-	/*  60 */	ED_INSERT,		/* < */
-	/*  61 */	ED_INSERT,		/* = */
-	/*  62 */	ED_INSERT,		/* > */
-	/*  63 */	ED_INSERT,		/* ? */
-	/*  64 */	ED_INSERT,		/* @ */
-	/*  65 */	ED_INSERT,		/* A */
-	/*  66 */	ED_INSERT,		/* B */
-	/*  67 */	ED_INSERT,		/* C */
-	/*  68 */	ED_INSERT,		/* D */
-	/*  69 */	ED_INSERT,		/* E */
-	/*  70 */	ED_INSERT,		/* F */
-	/*  71 */	ED_INSERT,		/* G */
-	/*  72 */	ED_INSERT,		/* H */
-	/*  73 */	ED_INSERT,		/* I */
-	/*  74 */	ED_INSERT,		/* J */
-	/*  75 */	ED_INSERT,		/* K */
-	/*  76 */	ED_INSERT,		/* L */
-	/*  77 */	ED_INSERT,		/* M */
-	/*  78 */	ED_INSERT,		/* N */
-	/*  79 */	ED_INSERT,		/* O */
-	/*  80 */	ED_INSERT,		/* P */
-	/*  81 */	ED_INSERT,		/* Q */
-	/*  82 */	ED_INSERT,		/* R */
-	/*  83 */	ED_INSERT,		/* S */
-	/*  84 */	ED_INSERT,		/* T */
-	/*  85 */	ED_INSERT,		/* U */
-	/*  86 */	ED_INSERT,		/* V */
-	/*  87 */	ED_INSERT,		/* W */
-	/*  88 */	ED_INSERT,		/* X */
-	/*  89 */	ED_INSERT,		/* Y */
-	/*  90 */	ED_INSERT,		/* Z */
-	/*  91 */	ED_INSERT,		/* [ */
-	/*  92 */	ED_INSERT,		/* \ */
-	/*  93 */	ED_INSERT,		/* ] */
-	/*  94 */	ED_INSERT,		/* ^ */
-	/*  95 */	ED_INSERT,		/* _ */
-	/*  96 */	ED_INSERT,		/* ` */
-	/*  97 */	ED_INSERT,		/* a */
-	/*  98 */	ED_INSERT,		/* b */
-	/*  99 */	ED_INSERT,		/* c */
-	/* 100 */	ED_INSERT,		/* d */
-	/* 101 */	ED_INSERT,		/* e */
-	/* 102 */	ED_INSERT,		/* f */
-	/* 103 */	ED_INSERT,		/* g */
-	/* 104 */	ED_INSERT,		/* h */
-	/* 105 */	ED_INSERT,		/* i */
-	/* 106 */	ED_INSERT,		/* j */
-	/* 107 */	ED_INSERT,		/* k */
-	/* 108 */	ED_INSERT,		/* l */
-	/* 109 */	ED_INSERT,		/* m */
-	/* 110 */	ED_INSERT,		/* n */
-	/* 111 */	ED_INSERT,		/* o */
-	/* 112 */	ED_INSERT,		/* p */
-	/* 113 */	ED_INSERT,		/* q */
-	/* 114 */	ED_INSERT,		/* r */
-	/* 115 */	ED_INSERT,		/* s */
-	/* 116 */	ED_INSERT,		/* t */
-	/* 117 */	ED_INSERT,		/* u */
-	/* 118 */	ED_INSERT,		/* v */
-	/* 119 */	ED_INSERT,		/* w */
-	/* 120 */	ED_INSERT,		/* x */
-	/* 121 */	ED_INSERT,		/* y */
-	/* 122 */	ED_INSERT,		/* z */
-	/* 123 */	ED_INSERT,		/* { */
-	/* 124 */	ED_INSERT,		/* | */
-	/* 125 */	ED_INSERT,		/* } */
-	/* 126 */	ED_INSERT,		/* ~ */
-	/* 127 */	ED_DELETE_PREV_CHAR,	/* ^? */
-	/* 128 */	ED_UNASSIGNED,		/* M-^@ */
-	/* 129 */	ED_UNASSIGNED,		/* M-^A */
-	/* 130 */	ED_UNASSIGNED,		/* M-^B */
-	/* 131 */	ED_UNASSIGNED,		/* M-^C */
-	/* 132 */	ED_UNASSIGNED,		/* M-^D */
-	/* 133 */	ED_UNASSIGNED,		/* M-^E */
-	/* 134 */	ED_UNASSIGNED,		/* M-^F */
-	/* 135 */	ED_UNASSIGNED,		/* M-^G */
-	/* 136 */	ED_DELETE_PREV_WORD,	/* M-^H */
-	/* 137 */	ED_UNASSIGNED,		/* M-^I */
-	/* 138 */	ED_UNASSIGNED,		/* M-^J */
-	/* 139 */	ED_UNASSIGNED,		/* M-^K */
-	/* 140 */	ED_CLEAR_SCREEN,	/* M-^L */
-	/* 141 */	ED_UNASSIGNED,		/* M-^M */
-	/* 142 */	ED_UNASSIGNED,		/* M-^N */
-	/* 143 */	ED_UNASSIGNED,		/* M-^O */
-	/* 144 */	ED_UNASSIGNED,		/* M-^P */
-	/* 145 */	ED_UNASSIGNED,		/* M-^Q */
-	/* 146 */	ED_UNASSIGNED,		/* M-^R */
-	/* 147 */	ED_UNASSIGNED,		/* M-^S */
-	/* 148 */	ED_UNASSIGNED,		/* M-^T */
-	/* 149 */	ED_UNASSIGNED,		/* M-^U */
-	/* 150 */	ED_UNASSIGNED,		/* M-^V */
-	/* 151 */	ED_UNASSIGNED,		/* M-^W */
-	/* 152 */	ED_UNASSIGNED,		/* M-^X */
-	/* 153 */	ED_UNASSIGNED,		/* M-^Y */
-	/* 154 */	ED_UNASSIGNED,		/* M-^Z */
-	/* 155 */	ED_UNASSIGNED,		/* M-^[ */
-	/* 156 */	ED_UNASSIGNED,		/* M-^\ */
-	/* 157 */	ED_UNASSIGNED,		/* M-^] */
-	/* 158 */	ED_UNASSIGNED,		/* M-^^ */
-	/* 159 */	EM_COPY_PREV_WORD,	/* M-^_ */
-	/* 160 */	ED_UNASSIGNED,		/* M-SPACE */
-	/* 161 */	ED_UNASSIGNED,		/* M-! */
-	/* 162 */	ED_UNASSIGNED,		/* M-" */
-	/* 163 */	ED_UNASSIGNED,		/* M-# */
-	/* 164 */	ED_UNASSIGNED,		/* M-$ */
-	/* 165 */	ED_UNASSIGNED,		/* M-% */
-	/* 166 */	ED_UNASSIGNED,		/* M-& */
-	/* 167 */	ED_UNASSIGNED,		/* M-' */
-	/* 168 */	ED_UNASSIGNED,		/* M-( */
-	/* 169 */	ED_UNASSIGNED,		/* M-) */
-	/* 170 */	ED_UNASSIGNED,		/* M-* */
-	/* 171 */	ED_UNASSIGNED,		/* M-+ */
-	/* 172 */	ED_UNASSIGNED,		/* M-, */
-	/* 173 */	ED_UNASSIGNED,		/* M-- */
-	/* 174 */	ED_UNASSIGNED,		/* M-. */
-	/* 175 */	ED_UNASSIGNED,		/* M-/ */
-	/* 176 */	ED_ARGUMENT_DIGIT,	/* M-0 */
-	/* 177 */	ED_ARGUMENT_DIGIT,	/* M-1 */
-	/* 178 */	ED_ARGUMENT_DIGIT,	/* M-2 */
-	/* 179 */	ED_ARGUMENT_DIGIT,	/* M-3 */
-	/* 180 */	ED_ARGUMENT_DIGIT,	/* M-4 */
-	/* 181 */	ED_ARGUMENT_DIGIT,	/* M-5 */
-	/* 182 */	ED_ARGUMENT_DIGIT,	/* M-6 */
-	/* 183 */	ED_ARGUMENT_DIGIT,	/* M-7 */
-	/* 184 */	ED_ARGUMENT_DIGIT,	/* M-8 */
-	/* 185 */	ED_ARGUMENT_DIGIT,	/* M-9 */
-	/* 186 */	ED_UNASSIGNED,		/* M-: */
-	/* 187 */	ED_UNASSIGNED,		/* M-; */
-	/* 188 */	ED_UNASSIGNED,		/* M-< */
-	/* 189 */	ED_UNASSIGNED,		/* M-= */
-	/* 190 */	ED_UNASSIGNED,		/* M-> */
-	/* 191 */	ED_UNASSIGNED,		/* M-? */
-	/* 192 */	ED_UNASSIGNED,		/* M-@ */
-	/* 193 */	ED_UNASSIGNED,		/* M-A */
-	/* 194 */	ED_PREV_WORD,		/* M-B */
-	/* 195 */	EM_CAPITOL_CASE,	/* M-C */
-	/* 196 */	EM_DELETE_NEXT_WORD,	/* M-D */
-	/* 197 */	ED_UNASSIGNED,		/* M-E */
-	/* 198 */	EM_NEXT_WORD,		/* M-F */
-	/* 199 */	ED_UNASSIGNED,		/* M-G */
-	/* 200 */	ED_UNASSIGNED,		/* M-H */
-	/* 201 */	ED_UNASSIGNED,		/* M-I */
-	/* 202 */	ED_UNASSIGNED,		/* M-J */
-	/* 203 */	ED_UNASSIGNED,		/* M-K */
-	/* 204 */	EM_LOWER_CASE,		/* M-L */
-	/* 205 */	ED_UNASSIGNED,		/* M-M */
-	/* 206 */	ED_SEARCH_NEXT_HISTORY,	/* M-N */
-	/* 207 */	ED_SEQUENCE_LEAD_IN,	/* M-O */
-	/* 208 */	ED_SEARCH_PREV_HISTORY,	/* M-P */
-	/* 209 */	ED_UNASSIGNED,		/* M-Q */
-	/* 210 */	ED_UNASSIGNED,		/* M-R */
-	/* 211 */	ED_UNASSIGNED,		/* M-S */
-	/* 212 */	ED_UNASSIGNED,		/* M-T */
-	/* 213 */	EM_UPPER_CASE,		/* M-U */
-	/* 214 */	ED_UNASSIGNED,		/* M-V */
-	/* 215 */	EM_COPY_REGION,		/* M-W */
-	/* 216 */	ED_COMMAND,		/* M-X */
-	/* 217 */	ED_UNASSIGNED,		/* M-Y */
-	/* 218 */	ED_UNASSIGNED,		/* M-Z */
-	/* 219 */	ED_SEQUENCE_LEAD_IN,	/* M-[ */
-	/* 220 */	ED_UNASSIGNED,		/* M-\ */
-	/* 221 */	ED_UNASSIGNED,		/* M-] */
-	/* 222 */	ED_UNASSIGNED,		/* M-^ */
-	/* 223 */	ED_UNASSIGNED,		/* M-_ */
-	/* 223 */	ED_UNASSIGNED,		/* M-` */
-	/* 224 */	ED_UNASSIGNED,		/* M-a */
-	/* 225 */	ED_PREV_WORD,		/* M-b */
-	/* 226 */	EM_CAPITOL_CASE,	/* M-c */
-	/* 227 */	EM_DELETE_NEXT_WORD,	/* M-d */
-	/* 228 */	ED_UNASSIGNED,		/* M-e */
-	/* 229 */	EM_NEXT_WORD,		/* M-f */
-	/* 230 */	ED_UNASSIGNED,		/* M-g */
-	/* 231 */	ED_UNASSIGNED,		/* M-h */
-	/* 232 */	ED_UNASSIGNED,		/* M-i */
-	/* 233 */	ED_UNASSIGNED,		/* M-j */
-	/* 234 */	ED_UNASSIGNED,		/* M-k */
-	/* 235 */	EM_LOWER_CASE,		/* M-l */
-	/* 236 */	ED_UNASSIGNED,		/* M-m */
-	/* 237 */	ED_SEARCH_NEXT_HISTORY,	/* M-n */
-	/* 238 */	ED_UNASSIGNED,		/* M-o */
-	/* 239 */	ED_SEARCH_PREV_HISTORY,	/* M-p */
-	/* 240 */	ED_UNASSIGNED,		/* M-q */
-	/* 241 */	ED_UNASSIGNED,		/* M-r */
-	/* 242 */	ED_UNASSIGNED,		/* M-s */
-	/* 243 */	ED_UNASSIGNED,		/* M-t */
-	/* 244 */	EM_UPPER_CASE,		/* M-u */
-	/* 245 */	ED_UNASSIGNED,		/* M-v */
-	/* 246 */	EM_COPY_REGION,		/* M-w */
-	/* 247 */	ED_COMMAND,		/* M-x */
-	/* 248 */	ED_UNASSIGNED,		/* M-y */
-	/* 249 */	ED_UNASSIGNED,		/* M-z */
-	/* 250 */	ED_UNASSIGNED,		/* M-{ */
-	/* 251 */	ED_UNASSIGNED,		/* M-| */
-	/* 252 */	ED_UNASSIGNED,		/* M-} */
-	/* 253 */	ED_UNASSIGNED,		/* M-~ */
-	/* 254 */	ED_DELETE_PREV_WORD	/* M-^? */
-	/* 255 */
-};
-
-
-/*
- * keymap table for vi.  Each index into above tbl; should be
- * N_KEYS entries long.  Vi mode uses a sticky-extend to do command mode:
- * insert mode characters are in the normal keymap, and command mode
- * in the extended keymap.
- */
-private const el_action_t  el_map_vi_insert[] = {
-#ifdef KSHVI
-	/*   0 */	ED_UNASSIGNED,		/* ^@ */
-	/*   1 */	ED_INSERT,		/* ^A */
-	/*   2 */	ED_INSERT,		/* ^B */
-	/*   3 */	ED_INSERT,		/* ^C */
-	/*   4 */	VI_LIST_OR_EOF,		/* ^D */
-	/*   5 */	ED_INSERT,		/* ^E */
-	/*   6 */	ED_INSERT,		/* ^F */
-	/*   7 */	ED_INSERT,		/* ^G */
-	/*   8 */	VI_DELETE_PREV_CHAR,	/* ^H */   /* BackSpace key */
-	/*   9 */	ED_INSERT,		/* ^I */   /* Tab Key  */
-	/*  10 */	ED_NEWLINE,		/* ^J */
-	/*  11 */	ED_INSERT,		/* ^K */
-	/*  12 */	ED_INSERT,		/* ^L */
-	/*  13 */	ED_NEWLINE,		/* ^M */
-	/*  14 */	ED_INSERT,		/* ^N */
-	/*  15 */	ED_INSERT,		/* ^O */
-	/*  16 */	ED_INSERT,		/* ^P */
-	/*  17 */	ED_TTY_START_OUTPUT,	/* ^Q */
-	/*  18 */	ED_INSERT,		/* ^R */
-	/*  19 */	ED_TTY_STOP_OUTPUT,	/* ^S */
-	/*  20 */	ED_INSERT,		/* ^T */
-	/*  21 */	VI_KILL_LINE_PREV,	/* ^U */
-	/*  22 */	ED_QUOTED_INSERT,	/* ^V */
-	/*  23 */	ED_DELETE_PREV_WORD,	/* ^W */
-		/* ED_DELETE_PREV_WORD: Only until strt edit pos */
-	/*  24 */	ED_INSERT,		/* ^X */
-	/*  25 */	ED_INSERT,		/* ^Y */
-	/*  26 */	ED_INSERT,		/* ^Z */
-	/*  27 */	VI_COMMAND_MODE,	/* ^[ */  /* [ Esc ] key */
-	/*  28 */	ED_TTY_SIGQUIT,		/* ^\ */
-	/*  29 */	ED_INSERT,		/* ^] */
-	/*  30 */	ED_INSERT,		/* ^^ */
-	/*  31 */	ED_INSERT,		/* ^_ */
-#else /* !KSHVI */
-				/*
-				 * NOTE: These mappings do NOT Correspond well
-				 * to the KSH VI editing assignments.
-				 * On the other and they are convenient and
-				 * many people have gotten used to them.
-				 */
-	/*   0 */	ED_UNASSIGNED,		/* ^@ */
-	/*   1 */	ED_MOVE_TO_BEG,		/* ^A */
-	/*   2 */	ED_PREV_CHAR,		/* ^B */
-	/*   3 */	ED_TTY_SIGINT,		/* ^C */
-	/*   4 */	VI_LIST_OR_EOF,		/* ^D */
-	/*   5 */	ED_MOVE_TO_END,		/* ^E */
-	/*   6 */	ED_NEXT_CHAR,		/* ^F */
-	/*   7 */	ED_UNASSIGNED,		/* ^G */
-	/*   8 */	ED_DELETE_PREV_CHAR,	/* ^H */   /* BackSpace key */
-	/*   9 */	ED_UNASSIGNED,		/* ^I */   /* Tab Key */
-	/*  10 */	ED_NEWLINE,		/* ^J */
-	/*  11 */	ED_KILL_LINE,		/* ^K */
-	/*  12 */	ED_CLEAR_SCREEN,	/* ^L */
-	/*  13 */	ED_NEWLINE,		/* ^M */
-	/*  14 */	ED_NEXT_HISTORY,	/* ^N */
-	/*  15 */	ED_TTY_FLUSH_OUTPUT,	/* ^O */
-	/*  16 */	ED_PREV_HISTORY,	/* ^P */
-	/*  17 */	ED_TTY_START_OUTPUT,	/* ^Q */
-	/*  18 */	ED_REDISPLAY,		/* ^R */
-	/*  19 */	ED_TTY_STOP_OUTPUT,	/* ^S */
-	/*  20 */	ED_TRANSPOSE_CHARS,	/* ^T */
-	/*  21 */	VI_KILL_LINE_PREV,	/* ^U */
-	/*  22 */	ED_QUOTED_INSERT,	/* ^V */
-	/*  23 */	ED_DELETE_PREV_WORD,	/* ^W */
-	/*  24 */	ED_UNASSIGNED,		/* ^X */
-	/*  25 */	ED_TTY_DSUSP,		/* ^Y */
-	/*  26 */	ED_TTY_SIGTSTP,		/* ^Z */
-	/*  27 */	VI_COMMAND_MODE,	/* ^[ */
-	/*  28 */	ED_TTY_SIGQUIT,		/* ^\ */
-	/*  29 */	ED_UNASSIGNED,		/* ^] */
-	/*  30 */	ED_UNASSIGNED,		/* ^^ */
-	/*  31 */	ED_UNASSIGNED,		/* ^_ */
-#endif  /* KSHVI */
-	/*  32 */	ED_INSERT,		/* SPACE */
-	/*  33 */	ED_INSERT,		/* ! */
-	/*  34 */	ED_INSERT,		/* " */
-	/*  35 */	ED_INSERT,		/* # */
-	/*  36 */	ED_INSERT,		/* $ */
-	/*  37 */	ED_INSERT,		/* % */
-	/*  38 */	ED_INSERT,		/* & */
-	/*  39 */	ED_INSERT,		/* ' */
-	/*  40 */	ED_INSERT,		/* ( */
-	/*  41 */	ED_INSERT,		/* ) */
-	/*  42 */	ED_INSERT,		/* * */
-	/*  43 */	ED_INSERT,		/* + */
-	/*  44 */	ED_INSERT,		/* , */
-	/*  45 */	ED_INSERT,		/* - */
-	/*  46 */	ED_INSERT,		/* . */
-	/*  47 */	ED_INSERT,		/* / */
-	/*  48 */	ED_INSERT,		/* 0 */
-	/*  49 */	ED_INSERT,		/* 1 */
-	/*  50 */	ED_INSERT,		/* 2 */
-	/*  51 */	ED_INSERT,		/* 3 */
-	/*  52 */	ED_INSERT,		/* 4 */
-	/*  53 */	ED_INSERT,		/* 5 */
-	/*  54 */	ED_INSERT,		/* 6 */
-	/*  55 */	ED_INSERT,		/* 7 */
-	/*  56 */	ED_INSERT,		/* 8 */
-	/*  57 */	ED_INSERT,		/* 9 */
-	/*  58 */	ED_INSERT,		/* : */
-	/*  59 */	ED_INSERT,		/* ; */
-	/*  60 */	ED_INSERT,		/* < */
-	/*  61 */	ED_INSERT,		/* = */
-	/*  62 */	ED_INSERT,		/* > */
-	/*  63 */	ED_INSERT,		/* ? */
-	/*  64 */	ED_INSERT,		/* @ */
-	/*  65 */	ED_INSERT,		/* A */
-	/*  66 */	ED_INSERT,		/* B */
-	/*  67 */	ED_INSERT,		/* C */
-	/*  68 */	ED_INSERT,		/* D */
-	/*  69 */	ED_INSERT,		/* E */
-	/*  70 */	ED_INSERT,		/* F */
-	/*  71 */	ED_INSERT,		/* G */
-	/*  72 */	ED_INSERT,		/* H */
-	/*  73 */	ED_INSERT,		/* I */
-	/*  74 */	ED_INSERT,		/* J */
-	/*  75 */	ED_INSERT,		/* K */
-	/*  76 */	ED_INSERT,		/* L */
-	/*  77 */	ED_INSERT,		/* M */
-	/*  78 */	ED_INSERT,		/* N */
-	/*  79 */	ED_INSERT,		/* O */
-	/*  80 */	ED_INSERT,		/* P */
-	/*  81 */	ED_INSERT,		/* Q */
-	/*  82 */	ED_INSERT,		/* R */
-	/*  83 */	ED_INSERT,		/* S */
-	/*  84 */	ED_INSERT,		/* T */
-	/*  85 */	ED_INSERT,		/* U */
-	/*  86 */	ED_INSERT,		/* V */
-	/*  87 */	ED_INSERT,		/* W */
-	/*  88 */	ED_INSERT,		/* X */
-	/*  89 */	ED_INSERT,		/* Y */
-	/*  90 */	ED_INSERT,		/* Z */
-	/*  91 */	ED_INSERT,		/* [ */
-	/*  92 */	ED_INSERT,		/* \ */
-	/*  93 */	ED_INSERT,		/* ] */
-	/*  94 */	ED_INSERT,		/* ^ */
-	/*  95 */	ED_INSERT,		/* _ */
-	/*  96 */	ED_INSERT,		/* ` */
-	/*  97 */	ED_INSERT,		/* a */
-	/*  98 */	ED_INSERT,		/* b */
-	/*  99 */	ED_INSERT,		/* c */
-	/* 100 */	ED_INSERT,		/* d */
-	/* 101 */	ED_INSERT,		/* e */
-	/* 102 */	ED_INSERT,		/* f */
-	/* 103 */	ED_INSERT,		/* g */
-	/* 104 */	ED_INSERT,		/* h */
-	/* 105 */	ED_INSERT,		/* i */
-	/* 106 */	ED_INSERT,		/* j */
-	/* 107 */	ED_INSERT,		/* k */
-	/* 108 */	ED_INSERT,		/* l */
-	/* 109 */	ED_INSERT,		/* m */
-	/* 110 */	ED_INSERT,		/* n */
-	/* 111 */	ED_INSERT,		/* o */
-	/* 112 */	ED_INSERT,		/* p */
-	/* 113 */	ED_INSERT,		/* q */
-	/* 114 */	ED_INSERT,		/* r */
-	/* 115 */	ED_INSERT,		/* s */
-	/* 116 */	ED_INSERT,		/* t */
-	/* 117 */	ED_INSERT,		/* u */
-	/* 118 */	ED_INSERT,		/* v */
-	/* 119 */	ED_INSERT,		/* w */
-	/* 120 */	ED_INSERT,		/* x */
-	/* 121 */	ED_INSERT,		/* y */
-	/* 122 */	ED_INSERT,		/* z */
-	/* 123 */	ED_INSERT,		/* { */
-	/* 124 */	ED_INSERT,		/* | */
-	/* 125 */	ED_INSERT,		/* } */
-	/* 126 */	ED_INSERT,		/* ~ */
-	/* 127 */	ED_DELETE_PREV_CHAR,	/* ^? */
-	/* 128 */	ED_UNASSIGNED,		/* M-^@ */
-	/* 129 */	ED_UNASSIGNED,		/* M-^A */
-	/* 130 */	ED_UNASSIGNED,		/* M-^B */
-	/* 131 */	ED_UNASSIGNED,		/* M-^C */
-	/* 132 */	ED_UNASSIGNED,		/* M-^D */
-	/* 133 */	ED_UNASSIGNED,		/* M-^E */
-	/* 134 */	ED_UNASSIGNED,		/* M-^F */
-	/* 135 */	ED_UNASSIGNED,		/* M-^G */
-	/* 136 */	ED_UNASSIGNED,		/* M-^H */
-	/* 137 */	ED_UNASSIGNED,		/* M-^I */
-	/* 138 */	ED_UNASSIGNED,		/* M-^J */
-	/* 139 */	ED_UNASSIGNED,		/* M-^K */
-	/* 140 */	ED_UNASSIGNED,		/* M-^L */
-	/* 141 */	ED_UNASSIGNED,		/* M-^M */
-	/* 142 */	ED_UNASSIGNED,		/* M-^N */
-	/* 143 */	ED_UNASSIGNED,		/* M-^O */
-	/* 144 */	ED_UNASSIGNED,		/* M-^P */
-	/* 145 */	ED_UNASSIGNED,		/* M-^Q */
-	/* 146 */	ED_UNASSIGNED,		/* M-^R */
-	/* 147 */	ED_UNASSIGNED,		/* M-^S */
-	/* 148 */	ED_UNASSIGNED,		/* M-^T */
-	/* 149 */	ED_UNASSIGNED,		/* M-^U */
-	/* 150 */	ED_UNASSIGNED,		/* M-^V */
-	/* 151 */	ED_UNASSIGNED,		/* M-^W */
-	/* 152 */	ED_UNASSIGNED,		/* M-^X */
-	/* 153 */	ED_UNASSIGNED,		/* M-^Y */
-	/* 154 */	ED_UNASSIGNED,		/* M-^Z */
-	/* 155 */	ED_UNASSIGNED,		/* M-^[ */
-	/* 156 */	ED_UNASSIGNED,		/* M-^\ */
-	/* 157 */	ED_UNASSIGNED,		/* M-^] */
-	/* 158 */	ED_UNASSIGNED,		/* M-^^ */
-	/* 159 */	ED_UNASSIGNED,		/* M-^_ */
-	/* 160 */	ED_UNASSIGNED,		/* M-SPACE */
-	/* 161 */	ED_UNASSIGNED,		/* M-! */
-	/* 162 */	ED_UNASSIGNED,		/* M-" */
-	/* 163 */	ED_UNASSIGNED,		/* M-# */
-	/* 164 */	ED_UNASSIGNED,		/* M-$ */
-	/* 165 */	ED_UNASSIGNED,		/* M-% */
-	/* 166 */	ED_UNASSIGNED,		/* M-& */
-	/* 167 */	ED_UNASSIGNED,		/* M-' */
-	/* 168 */	ED_UNASSIGNED,		/* M-( */
-	/* 169 */	ED_UNASSIGNED,		/* M-) */
-	/* 170 */	ED_UNASSIGNED,		/* M-* */
-	/* 171 */	ED_UNASSIGNED,		/* M-+ */
-	/* 172 */	ED_UNASSIGNED,		/* M-, */
-	/* 173 */	ED_UNASSIGNED,		/* M-- */
-	/* 174 */	ED_UNASSIGNED,		/* M-. */
-	/* 175 */	ED_UNASSIGNED,		/* M-/ */
-	/* 176 */	ED_UNASSIGNED,		/* M-0 */
-	/* 177 */	ED_UNASSIGNED,		/* M-1 */
-	/* 178 */	ED_UNASSIGNED,		/* M-2 */
-	/* 179 */	ED_UNASSIGNED,		/* M-3 */
-	/* 180 */	ED_UNASSIGNED,		/* M-4 */
-	/* 181 */	ED_UNASSIGNED,		/* M-5 */
-	/* 182 */	ED_UNASSIGNED,		/* M-6 */
-	/* 183 */	ED_UNASSIGNED,		/* M-7 */
-	/* 184 */	ED_UNASSIGNED,		/* M-8 */
-	/* 185 */	ED_UNASSIGNED,		/* M-9 */
-	/* 186 */	ED_UNASSIGNED,		/* M-: */
-	/* 187 */	ED_UNASSIGNED,		/* M-; */
-	/* 188 */	ED_UNASSIGNED,		/* M-< */
-	/* 189 */	ED_UNASSIGNED,		/* M-= */
-	/* 190 */	ED_UNASSIGNED,		/* M-> */
-	/* 191 */	ED_UNASSIGNED,		/* M-? */
-	/* 192 */	ED_UNASSIGNED,		/* M-@ */
-	/* 193 */	ED_UNASSIGNED,		/* M-A */
-	/* 194 */	ED_UNASSIGNED,		/* M-B */
-	/* 195 */	ED_UNASSIGNED,		/* M-C */
-	/* 196 */	ED_UNASSIGNED,		/* M-D */
-	/* 197 */	ED_UNASSIGNED,		/* M-E */
-	/* 198 */	ED_UNASSIGNED,		/* M-F */
-	/* 199 */	ED_UNASSIGNED,		/* M-G */
-	/* 200 */	ED_UNASSIGNED,		/* M-H */
-	/* 201 */	ED_UNASSIGNED,		/* M-I */
-	/* 202 */	ED_UNASSIGNED,		/* M-J */
-	/* 203 */	ED_UNASSIGNED,		/* M-K */
-	/* 204 */	ED_UNASSIGNED,		/* M-L */
-	/* 205 */	ED_UNASSIGNED,		/* M-M */
-	/* 206 */	ED_UNASSIGNED,		/* M-N */
-	/* 207 */	ED_UNASSIGNED,		/* M-O */
-	/* 208 */	ED_UNASSIGNED,		/* M-P */
-	/* 209 */	ED_UNASSIGNED,		/* M-Q */
-	/* 210 */	ED_UNASSIGNED,		/* M-R */
-	/* 211 */	ED_UNASSIGNED,		/* M-S */
-	/* 212 */	ED_UNASSIGNED,		/* M-T */
-	/* 213 */	ED_UNASSIGNED,		/* M-U */
-	/* 214 */	ED_UNASSIGNED,		/* M-V */
-	/* 215 */	ED_UNASSIGNED,		/* M-W */
-	/* 216 */	ED_UNASSIGNED,		/* M-X */
-	/* 217 */	ED_UNASSIGNED,		/* M-Y */
-	/* 218 */	ED_UNASSIGNED,		/* M-Z */
-	/* 219 */	ED_UNASSIGNED,		/* M-[ */
-	/* 220 */	ED_UNASSIGNED,		/* M-\ */
-	/* 221 */	ED_UNASSIGNED,		/* M-] */
-	/* 222 */	ED_UNASSIGNED,		/* M-^ */
-	/* 223 */	ED_UNASSIGNED,		/* M-_ */
-	/* 224 */	ED_UNASSIGNED,		/* M-` */
-	/* 225 */	ED_UNASSIGNED,		/* M-a */
-	/* 226 */	ED_UNASSIGNED,		/* M-b */
-	/* 227 */	ED_UNASSIGNED,		/* M-c */
-	/* 228 */	ED_UNASSIGNED,		/* M-d */
-	/* 229 */	ED_UNASSIGNED,		/* M-e */
-	/* 230 */	ED_UNASSIGNED,		/* M-f */
-	/* 231 */	ED_UNASSIGNED,		/* M-g */
-	/* 232 */	ED_UNASSIGNED,		/* M-h */
-	/* 233 */	ED_UNASSIGNED,		/* M-i */
-	/* 234 */	ED_UNASSIGNED,		/* M-j */
-	/* 235 */	ED_UNASSIGNED,		/* M-k */
-	/* 236 */	ED_UNASSIGNED,		/* M-l */
-	/* 237 */	ED_UNASSIGNED,		/* M-m */
-	/* 238 */	ED_UNASSIGNED,		/* M-n */
-	/* 239 */	ED_UNASSIGNED,		/* M-o */
-	/* 240 */	ED_UNASSIGNED,		/* M-p */
-	/* 241 */	ED_UNASSIGNED,		/* M-q */
-	/* 242 */	ED_UNASSIGNED,		/* M-r */
-	/* 243 */	ED_UNASSIGNED,		/* M-s */
-	/* 244 */	ED_UNASSIGNED,		/* M-t */
-	/* 245 */	ED_UNASSIGNED,		/* M-u */
-	/* 246 */	ED_UNASSIGNED,		/* M-v */
-	/* 247 */	ED_UNASSIGNED,		/* M-w */
-	/* 248 */	ED_UNASSIGNED,		/* M-x */
-	/* 249 */	ED_UNASSIGNED,		/* M-y */
-	/* 250 */	ED_UNASSIGNED,		/* M-z */
-	/* 251 */	ED_UNASSIGNED,		/* M-{ */
-	/* 252 */	ED_UNASSIGNED,		/* M-| */
-	/* 253 */	ED_UNASSIGNED,		/* M-} */
-	/* 254 */	ED_UNASSIGNED,		/* M-~ */
-	/* 255 */	ED_UNASSIGNED		/* M-^? */
-};
-
-private const el_action_t el_map_vi_command[] = {
-	/*   0 */	ED_UNASSIGNED,		/* ^@ */
-	/*   1 */	ED_MOVE_TO_BEG,		/* ^A */
-	/*   2 */	ED_UNASSIGNED,		/* ^B */
-	/*   3 */	ED_TTY_SIGINT,		/* ^C */
-	/*   4 */	ED_UNASSIGNED,		/* ^D */
-	/*   5 */	ED_MOVE_TO_END,		/* ^E */
-	/*   6 */	ED_UNASSIGNED,		/* ^F */
-	/*   7 */	ED_UNASSIGNED,		/* ^G */
-	/*   8 */	ED_PREV_CHAR,		/* ^H */
-	/*   9 */	ED_UNASSIGNED,		/* ^I */
-	/*  10 */	ED_NEWLINE,		/* ^J */
-	/*  11 */	ED_KILL_LINE,		/* ^K */
-	/*  12 */	ED_CLEAR_SCREEN,	/* ^L */
-	/*  13 */	ED_NEWLINE,		/* ^M */
-	/*  14 */	ED_NEXT_HISTORY,	/* ^N */
-	/*  15 */	ED_TTY_FLUSH_OUTPUT,	/* ^O */
-	/*  16 */	ED_PREV_HISTORY,	/* ^P */
-	/*  17 */	ED_TTY_START_OUTPUT,	/* ^Q */
-	/*  18 */	ED_REDISPLAY,		/* ^R */
-	/*  19 */	ED_TTY_STOP_OUTPUT,	/* ^S */
-	/*  20 */	ED_UNASSIGNED,		/* ^T */
-	/*  21 */	VI_KILL_LINE_PREV,	/* ^U */
-	/*  22 */	ED_UNASSIGNED,		/* ^V */
-	/*  23 */	ED_DELETE_PREV_WORD,	/* ^W */
-	/*  24 */	ED_UNASSIGNED,		/* ^X */
-	/*  25 */	ED_UNASSIGNED,		/* ^Y */
-	/*  26 */	ED_UNASSIGNED,		/* ^Z */
-	/*  27 */	EM_META_NEXT,		/* ^[ */
-	/*  28 */	ED_TTY_SIGQUIT,		/* ^\ */
-	/*  29 */	ED_UNASSIGNED,		/* ^] */
-	/*  30 */	ED_UNASSIGNED,		/* ^^ */
-	/*  31 */	ED_UNASSIGNED,		/* ^_ */
-	/*  32 */	ED_NEXT_CHAR,		/* SPACE */
-	/*  33 */	ED_UNASSIGNED,		/* ! */
-	/*  34 */	ED_UNASSIGNED,		/* " */
-	/*  35 */	ED_UNASSIGNED,		/* # */
-	/*  36 */	ED_MOVE_TO_END,		/* $ */
-	/*  37 */	ED_UNASSIGNED,		/* % */
-	/*  38 */	ED_UNASSIGNED,		/* & */
-	/*  39 */	ED_UNASSIGNED,		/* ' */
-	/*  40 */	ED_UNASSIGNED,		/* ( */
-	/*  41 */	ED_UNASSIGNED,		/* ) */
-	/*  42 */	ED_UNASSIGNED,		/* * */
-	/*  43 */	ED_NEXT_HISTORY,	/* + */
-	/*  44 */	VI_REPEAT_PREV_CHAR,	/* , */
-	/*  45 */	ED_PREV_HISTORY,	/* - */
-	/*  46 */	ED_UNASSIGNED,		/* . */
-	/*  47 */	VI_SEARCH_PREV,		/* / */
-	/*  48 */	VI_ZERO,		/* 0 */
-	/*  49 */	ED_ARGUMENT_DIGIT,	/* 1 */
-	/*  50 */	ED_ARGUMENT_DIGIT,	/* 2 */
-	/*  51 */	ED_ARGUMENT_DIGIT,	/* 3 */
-	/*  52 */	ED_ARGUMENT_DIGIT,	/* 4 */
-	/*  53 */	ED_ARGUMENT_DIGIT,	/* 5 */
-	/*  54 */	ED_ARGUMENT_DIGIT,	/* 6 */
-	/*  55 */	ED_ARGUMENT_DIGIT,	/* 7 */
-	/*  56 */	ED_ARGUMENT_DIGIT,	/* 8 */
-	/*  57 */	ED_ARGUMENT_DIGIT,	/* 9 */
-	/*  58 */	ED_COMMAND,		/* : */
-	/*  59 */	VI_REPEAT_NEXT_CHAR,	/* ; */
-	/*  60 */	ED_UNASSIGNED,		/* < */
-	/*  61 */	ED_UNASSIGNED,		/* = */
-	/*  62 */	ED_UNASSIGNED,		/* > */
-	/*  63 */	VI_SEARCH_NEXT,		/* ? */
-	/*  64 */	ED_UNASSIGNED,		/* @ */
-	/*  65 */	VI_ADD_AT_EOL,		/* A */
-	/*  66 */	VI_PREV_SPACE_WORD,	/* B */
-	/*  67 */	VI_CHANGE_TO_EOL,	/* C */
-	/*  68 */	ED_KILL_LINE,		/* D */
-	/*  69 */	VI_TO_END_WORD,		/* E */
-	/*  70 */	VI_PREV_CHAR,		/* F */
-	/*  71 */	ED_UNASSIGNED,		/* G */
-	/*  72 */	ED_UNASSIGNED,		/* H */
-	/*  73 */	VI_INSERT_AT_BOL,	/* I */
-	/*  74 */	ED_SEARCH_NEXT_HISTORY,	/* J */
-	/*  75 */	ED_SEARCH_PREV_HISTORY,	/* K */
-	/*  76 */	ED_UNASSIGNED,		/* L */
-	/*  77 */	ED_UNASSIGNED,		/* M */
-	/*  78 */	VI_REPEAT_SEARCH_PREV,	/* N */
-	/*  79 */	ED_SEQUENCE_LEAD_IN,	/* O */
-	/*  80 */	VI_PASTE_PREV,		/* P */
-	/*  81 */	ED_UNASSIGNED,		/* Q */
-	/*  82 */	VI_REPLACE_MODE,	/* R */
-	/*  83 */	VI_SUBSTITUTE_LINE,	/* S */
-	/*  84 */	VI_TO_PREV_CHAR,	/* T */
-	/*  85 */	ED_UNASSIGNED,		/* U */
-	/*  86 */	ED_UNASSIGNED,		/* V */
-	/*  87 */	VI_NEXT_SPACE_WORD,	/* W */
-	/*  88 */	ED_DELETE_PREV_CHAR,	/* X */
-	/*  89 */	ED_UNASSIGNED,		/* Y */
-	/*  90 */	ED_UNASSIGNED,		/* Z */
-	/*  91 */	ED_SEQUENCE_LEAD_IN,	/* [ */
-	/*  92 */	ED_UNASSIGNED,		/* \ */
-	/*  93 */	ED_UNASSIGNED,		/* ] */
-	/*  94 */	ED_MOVE_TO_BEG,		/* ^ */
-	/*  95 */	ED_UNASSIGNED,		/* _ */
-	/*  96 */	ED_UNASSIGNED,		/* ` */
-	/*  97 */	VI_ADD,			/* a */
-	/*  98 */	VI_PREV_WORD,		/* b */
-	/*  99 */	VI_CHANGE_META,		/* c */
-	/* 100 */	VI_DELETE_META,		/* d */
-	/* 101 */	VI_END_WORD,		/* e */
-	/* 102 */	VI_NEXT_CHAR,		/* f */
-	/* 103 */	ED_UNASSIGNED,		/* g */
-	/* 104 */	ED_PREV_CHAR,		/* h */
-	/* 105 */	VI_INSERT,		/* i */
-	/* 106 */	ED_NEXT_HISTORY,	/* j */
-	/* 107 */	ED_PREV_HISTORY,	/* k */
-	/* 108 */	ED_NEXT_CHAR,		/* l */
-	/* 109 */	ED_UNASSIGNED,		/* m */
-	/* 110 */	VI_REPEAT_SEARCH_NEXT,	/* n */
-	/* 111 */	ED_UNASSIGNED,		/* o */
-	/* 112 */	VI_PASTE_NEXT,		/* p */
-	/* 113 */	ED_UNASSIGNED,		/* q */
-	/* 114 */	VI_REPLACE_CHAR,	/* r */
-	/* 115 */	VI_SUBSTITUTE_CHAR,	/* s */
-	/* 116 */	VI_TO_NEXT_CHAR,	/* t */
-	/* 117 */	VI_UNDO,		/* u */
-	/* 118 */	ED_UNASSIGNED,		/* v */
-	/* 119 */	VI_NEXT_WORD,		/* w */
-	/* 120 */	ED_DELETE_NEXT_CHAR,	/* x */
-	/* 121 */	ED_UNASSIGNED,		/* y */
-	/* 122 */	ED_UNASSIGNED,		/* z */
-	/* 123 */	ED_UNASSIGNED,		/* { */
-	/* 124 */	ED_UNASSIGNED,		/* | */
-	/* 125 */	ED_UNASSIGNED,		/* } */
-	/* 126 */	VI_CHANGE_CASE,		/* ~ */
-	/* 127 */	ED_DELETE_PREV_CHAR,	/* ^? */
-	/* 128 */	ED_UNASSIGNED,		/* M-^@ */
-	/* 129 */	ED_UNASSIGNED,		/* M-^A */
-	/* 130 */	ED_UNASSIGNED,		/* M-^B */
-	/* 131 */	ED_UNASSIGNED,		/* M-^C */
-	/* 132 */	ED_UNASSIGNED,		/* M-^D */
-	/* 133 */	ED_UNASSIGNED,		/* M-^E */
-	/* 134 */	ED_UNASSIGNED,		/* M-^F */
-	/* 135 */	ED_UNASSIGNED,		/* M-^G */
-	/* 136 */	ED_UNASSIGNED,		/* M-^H */
-	/* 137 */	ED_UNASSIGNED,		/* M-^I */
-	/* 138 */	ED_UNASSIGNED,		/* M-^J */
-	/* 139 */	ED_UNASSIGNED,		/* M-^K */
-	/* 140 */	ED_UNASSIGNED,		/* M-^L */
-	/* 141 */	ED_UNASSIGNED,		/* M-^M */
-	/* 142 */	ED_UNASSIGNED,		/* M-^N */
-	/* 143 */	ED_UNASSIGNED,		/* M-^O */
-	/* 144 */	ED_UNASSIGNED,		/* M-^P */
-	/* 145 */	ED_UNASSIGNED,		/* M-^Q */
-	/* 146 */	ED_UNASSIGNED,		/* M-^R */
-	/* 147 */	ED_UNASSIGNED,		/* M-^S */
-	/* 148 */	ED_UNASSIGNED,		/* M-^T */
-	/* 149 */	ED_UNASSIGNED,		/* M-^U */
-	/* 150 */	ED_UNASSIGNED,		/* M-^V */
-	/* 151 */	ED_UNASSIGNED,		/* M-^W */
-	/* 152 */	ED_UNASSIGNED,		/* M-^X */
-	/* 153 */	ED_UNASSIGNED,		/* M-^Y */
-	/* 154 */	ED_UNASSIGNED,		/* M-^Z */
-	/* 155 */	ED_UNASSIGNED,		/* M-^[ */
-	/* 156 */	ED_UNASSIGNED,		/* M-^\ */
-	/* 157 */	ED_UNASSIGNED,		/* M-^] */
-	/* 158 */	ED_UNASSIGNED,		/* M-^^ */
-	/* 159 */	ED_UNASSIGNED,		/* M-^_ */
-	/* 160 */	ED_UNASSIGNED,		/* M-SPACE */
-	/* 161 */	ED_UNASSIGNED,		/* M-! */
-	/* 162 */	ED_UNASSIGNED,		/* M-" */
-	/* 163 */	ED_UNASSIGNED,		/* M-# */
-	/* 164 */	ED_UNASSIGNED,		/* M-$ */
-	/* 165 */	ED_UNASSIGNED,		/* M-% */
-	/* 166 */	ED_UNASSIGNED,		/* M-& */
-	/* 167 */	ED_UNASSIGNED,		/* M-' */
-	/* 168 */	ED_UNASSIGNED,		/* M-( */
-	/* 169 */	ED_UNASSIGNED,		/* M-) */
-	/* 170 */	ED_UNASSIGNED,		/* M-* */
-	/* 171 */	ED_UNASSIGNED,		/* M-+ */
-	/* 172 */	ED_UNASSIGNED,		/* M-, */
-	/* 173 */	ED_UNASSIGNED,		/* M-- */
-	/* 174 */	ED_UNASSIGNED,		/* M-. */
-	/* 175 */	ED_UNASSIGNED,		/* M-/ */
-	/* 176 */	ED_UNASSIGNED,		/* M-0 */
-	/* 177 */	ED_UNASSIGNED,		/* M-1 */
-	/* 178 */	ED_UNASSIGNED,		/* M-2 */
-	/* 179 */	ED_UNASSIGNED,		/* M-3 */
-	/* 180 */	ED_UNASSIGNED,		/* M-4 */
-	/* 181 */	ED_UNASSIGNED,		/* M-5 */
-	/* 182 */	ED_UNASSIGNED,		/* M-6 */
-	/* 183 */	ED_UNASSIGNED,		/* M-7 */
-	/* 184 */	ED_UNASSIGNED,		/* M-8 */
-	/* 185 */	ED_UNASSIGNED,		/* M-9 */
-	/* 186 */	ED_UNASSIGNED,		/* M-: */
-	/* 187 */	ED_UNASSIGNED,		/* M-; */
-	/* 188 */	ED_UNASSIGNED,		/* M-< */
-	/* 189 */	ED_UNASSIGNED,		/* M-= */
-	/* 190 */	ED_UNASSIGNED,		/* M-> */
-	/* 191 */	ED_UNASSIGNED,		/* M-? */
-	/* 192 */	ED_UNASSIGNED,		/* M-@ */
-	/* 193 */	ED_UNASSIGNED,		/* M-A */
-	/* 194 */	ED_UNASSIGNED,		/* M-B */
-	/* 195 */	ED_UNASSIGNED,		/* M-C */
-	/* 196 */	ED_UNASSIGNED,		/* M-D */
-	/* 197 */	ED_UNASSIGNED,		/* M-E */
-	/* 198 */	ED_UNASSIGNED,		/* M-F */
-	/* 199 */	ED_UNASSIGNED,		/* M-G */
-	/* 200 */	ED_UNASSIGNED,		/* M-H */
-	/* 201 */	ED_UNASSIGNED,		/* M-I */
-	/* 202 */	ED_UNASSIGNED,		/* M-J */
-	/* 203 */	ED_UNASSIGNED,		/* M-K */
-	/* 204 */	ED_UNASSIGNED,		/* M-L */
-	/* 205 */	ED_UNASSIGNED,		/* M-M */
-	/* 206 */	ED_UNASSIGNED,		/* M-N */
-	/* 207 */	ED_SEQUENCE_LEAD_IN,	/* M-O */
-	/* 208 */	ED_UNASSIGNED,		/* M-P */
-	/* 209 */	ED_UNASSIGNED,		/* M-Q */
-	/* 210 */	ED_UNASSIGNED,		/* M-R */
-	/* 211 */	ED_UNASSIGNED,		/* M-S */
-	/* 212 */	ED_UNASSIGNED,		/* M-T */
-	/* 213 */	ED_UNASSIGNED,		/* M-U */
-	/* 214 */	ED_UNASSIGNED,		/* M-V */
-	/* 215 */	ED_UNASSIGNED,		/* M-W */
-	/* 216 */	ED_UNASSIGNED,		/* M-X */
-	/* 217 */	ED_UNASSIGNED,		/* M-Y */
-	/* 218 */	ED_UNASSIGNED,		/* M-Z */
-	/* 219 */	ED_SEQUENCE_LEAD_IN,	/* M-[ */
-	/* 220 */	ED_UNASSIGNED,		/* M-\ */
-	/* 221 */	ED_UNASSIGNED,		/* M-] */
-	/* 222 */	ED_UNASSIGNED,		/* M-^ */
-	/* 223 */	ED_UNASSIGNED,		/* M-_ */
-	/* 224 */	ED_UNASSIGNED,		/* M-` */
-	/* 225 */	ED_UNASSIGNED,		/* M-a */
-	/* 226 */	ED_UNASSIGNED,		/* M-b */
-	/* 227 */	ED_UNASSIGNED,		/* M-c */
-	/* 228 */	ED_UNASSIGNED,		/* M-d */
-	/* 229 */	ED_UNASSIGNED,		/* M-e */
-	/* 230 */	ED_UNASSIGNED,		/* M-f */
-	/* 231 */	ED_UNASSIGNED,		/* M-g */
-	/* 232 */	ED_UNASSIGNED,		/* M-h */
-	/* 233 */	ED_UNASSIGNED,		/* M-i */
-	/* 234 */	ED_UNASSIGNED,		/* M-j */
-	/* 235 */	ED_UNASSIGNED,		/* M-k */
-	/* 236 */	ED_UNASSIGNED,		/* M-l */
-	/* 237 */	ED_UNASSIGNED,		/* M-m */
-	/* 238 */	ED_UNASSIGNED,		/* M-n */
-	/* 239 */	ED_UNASSIGNED,		/* M-o */
-	/* 240 */	ED_UNASSIGNED,		/* M-p */
-	/* 241 */	ED_UNASSIGNED,		/* M-q */
-	/* 242 */	ED_UNASSIGNED,		/* M-r */
-	/* 243 */	ED_UNASSIGNED,		/* M-s */
-	/* 244 */	ED_UNASSIGNED,		/* M-t */
-	/* 245 */	ED_UNASSIGNED,		/* M-u */
-	/* 246 */	ED_UNASSIGNED,		/* M-v */
-	/* 247 */	ED_UNASSIGNED,		/* M-w */
-	/* 248 */	ED_UNASSIGNED,		/* M-x */
-	/* 249 */	ED_UNASSIGNED,		/* M-y */
-	/* 250 */	ED_UNASSIGNED,		/* M-z */
-	/* 251 */	ED_UNASSIGNED,		/* M-{ */
-	/* 252 */	ED_UNASSIGNED,		/* M-| */
-	/* 253 */	ED_UNASSIGNED,		/* M-} */
-	/* 254 */	ED_UNASSIGNED,		/* M-~ */
-	/* 255 */	ED_UNASSIGNED		/* M-^? */
-};
-
-
-/* map_init():
- *	Initialize and allocate the maps
- */
-protected int
-map_init(EditLine *el)
-{
-
-	/*
-         * Make sure those are correct before starting.
-         */
-#ifdef MAP_DEBUG
-	if (sizeof(el_map_emacs) != N_KEYS * sizeof(el_action_t))
-		EL_ABORT((el->errfile, "Emacs map incorrect\n"));
-	if (sizeof(el_map_vi_command) != N_KEYS * sizeof(el_action_t))
-		EL_ABORT((el->errfile, "Vi command map incorrect\n"));
-	if (sizeof(el_map_vi_insert) != N_KEYS * sizeof(el_action_t))
-		EL_ABORT((el->errfile, "Vi insert map incorrect\n"));
-#endif
-
-	el->el_map.alt = (el_action_t *)el_malloc(sizeof(el_action_t) * N_KEYS);
-	if (el->el_map.alt == NULL)
-		return (-1);
-	el->el_map.key = (el_action_t *)el_malloc(sizeof(el_action_t) * N_KEYS);
-	if (el->el_map.key == NULL)
-		return (-1);
-	el->el_map.emacs = el_map_emacs;
-	el->el_map.vic = el_map_vi_command;
-	el->el_map.vii = el_map_vi_insert;
-	el->el_map.help = (el_bindings_t *) el_malloc(sizeof(el_bindings_t) *
-	    EL_NUM_FCNS);
-	if (el->el_map.help == NULL)
-		return (-1);
-	(void) memcpy(el->el_map.help, help__get(),
-	    sizeof(el_bindings_t) * EL_NUM_FCNS);
-	el->el_map.func = (el_func_t *)el_malloc(sizeof(el_func_t) *
-	    EL_NUM_FCNS);
-	if (el->el_map.func == NULL)
-		return (-1);
-	memcpy(el->el_map.func, func__get(), sizeof(el_func_t) * EL_NUM_FCNS);
-	el->el_map.nfunc = EL_NUM_FCNS;
-
-#ifdef VIDEFAULT
-	map_init_vi(el);
-#else
-	map_init_emacs(el);
-#endif /* VIDEFAULT */
-	return (0);
-}
-
-
-/* map_end():
- *	Free the space taken by the editor maps
- */
-protected void
-map_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_map.alt);
-	el->el_map.alt = NULL;
-	el_free((ptr_t) el->el_map.key);
-	el->el_map.key = NULL;
-	el->el_map.emacs = NULL;
-	el->el_map.vic = NULL;
-	el->el_map.vii = NULL;
-	el_free((ptr_t) el->el_map.help);
-	el->el_map.help = NULL;
-	el_free((ptr_t) el->el_map.func);
-	el->el_map.func = NULL;
-}
-
-
-/* map_init_nls():
- *	Find all the printable keys and bind them to self insert
- */
-private void
-map_init_nls(EditLine *el)
-{
-	int i;
-
-	el_action_t *map = el->el_map.key;
-
-	for (i = 0200; i <= 0377; i++)
-		if (isprint(i))
-			map[i] = ED_INSERT;
-}
-
-
-/* map_init_meta():
- *	Bind all the meta keys to the appropriate ESC-<key> sequence
- */
-private void
-map_init_meta(EditLine *el)
-{
-	char buf[3];
-	int i;
-	el_action_t *map = el->el_map.key;
-	el_action_t *alt = el->el_map.alt;
-
-	for (i = 0; i <= 0377 && map[i] != EM_META_NEXT; i++)
-		continue;
-
-	if (i > 0377) {
-		for (i = 0; i <= 0377 && alt[i] != EM_META_NEXT; i++)
-			continue;
-		if (i > 0377) {
-			i = 033;
-			if (el->el_map.type == MAP_VI)
-				map = alt;
-		} else
-			map = alt;
-	}
-	buf[0] = (char) i;
-	buf[2] = 0;
-	for (i = 0200; i <= 0377; i++)
-		switch (map[i]) {
-		case ED_INSERT:
-		case ED_UNASSIGNED:
-		case ED_SEQUENCE_LEAD_IN:
-			break;
-		default:
-			buf[1] = i & 0177;
-			key_add(el, buf, key_map_cmd(el, (int) map[i]), XK_CMD);
-			break;
-		}
-	map[(int) buf[0]] = ED_SEQUENCE_LEAD_IN;
-}
-
-
-/* map_init_vi():
- *	Initialize the vi bindings
- */
-protected void
-map_init_vi(EditLine *el)
-{
-	int i;
-	el_action_t *key = el->el_map.key;
-	el_action_t *alt = el->el_map.alt;
-	const el_action_t *vii = el->el_map.vii;
-	const el_action_t *vic = el->el_map.vic;
-
-	el->el_map.type = MAP_VI;
-	el->el_map.current = el->el_map.key;
-
-	key_reset(el);
-
-	for (i = 0; i < N_KEYS; i++) {
-		key[i] = vii[i];
-		alt[i] = vic[i];
-	}
-
-	map_init_meta(el);
-	map_init_nls(el);
-
-	tty_bind_char(el, 1);
-	term_bind_arrow(el);
-}
-
-
-/* map_init_emacs():
- *	Initialize the emacs bindings
- */
-protected void
-map_init_emacs(EditLine *el)
-{
-	int i;
-	char buf[3];
-	el_action_t *key = el->el_map.key;
-	el_action_t *alt = el->el_map.alt;
-	const el_action_t *emacs = el->el_map.emacs;
-
-	el->el_map.type = MAP_EMACS;
-	el->el_map.current = el->el_map.key;
-	key_reset(el);
-
-	for (i = 0; i < N_KEYS; i++) {
-		key[i] = emacs[i];
-		alt[i] = ED_UNASSIGNED;
-	}
-
-	map_init_meta(el);
-	map_init_nls(el);
-
-	buf[0] = CONTROL('X');
-	buf[1] = CONTROL('X');
-	buf[2] = 0;
-	key_add(el, buf, key_map_cmd(el, EM_EXCHANGE_MARK), XK_CMD);
-
-	tty_bind_char(el, 1);
-	term_bind_arrow(el);
-}
-
-
-/* map_set_editor():
- *	Set the editor
- */
-protected int
-map_set_editor(EditLine *el, char *editor)
-{
-
-	if (strcmp(editor, "emacs") == 0) {
-		map_init_emacs(el);
-		return (0);
-	}
-	if (strcmp(editor, "vi") == 0) {
-		map_init_vi(el);
-		return (0);
-	}
-	return (-1);
-}
-
-
-/* map_get_editor():
- *	Retrieve the editor
- */
-protected int
-map_get_editor(EditLine *el, const char **editor)
-{
-
-	if (editor == NULL)
-		return (-1);
-	switch (el->el_map.type) {
-	case MAP_EMACS:
-		*editor = "emacs";
-		return (0);
-	case MAP_VI:
-		*editor = "vi";
-		return (0);
-	}
-	return (-1);
-}
-
-
-/* map_print_key():
- *	Print the function description for 1 key
- */
-private void
-map_print_key(EditLine *el, el_action_t *map, const char *in)
-{
-	char outbuf[EL_BUFSIZ];
-	el_bindings_t *bp;
-
-	if (in[0] == '\0' || in[1] == '\0') {
-		(void) key__decode_str(in, outbuf, "");
-		for (bp = el->el_map.help; bp->name != NULL; bp++)
-			if (bp->func == map[(unsigned char) *in]) {
-				(void) fprintf(el->el_outfile,
-				    "%s\t->\t%s\n", outbuf, bp->name);
-				return;
-			}
-	} else
-		key_print(el, in);
-}
-
-
-/* map_print_some_keys():
- *	Print keys from first to last
- */
-private void
-map_print_some_keys(EditLine *el, el_action_t *map, int first, int last)
-{
-	el_bindings_t *bp;
-	char firstbuf[2], lastbuf[2];
-	char unparsbuf[EL_BUFSIZ], extrabuf[EL_BUFSIZ];
-
-	firstbuf[0] = first;
-	firstbuf[1] = 0;
-	lastbuf[0] = last;
-	lastbuf[1] = 0;
-	if (map[first] == ED_UNASSIGNED) {
-		if (first == last)
-			(void) fprintf(el->el_outfile,
-			    "%-15s->  is undefined\n",
-			    key__decode_str(firstbuf, unparsbuf, STRQQ));
-		return;
-	}
-	for (bp = el->el_map.help; bp->name != NULL; bp++) {
-		if (bp->func == map[first]) {
-			if (first == last) {
-				(void) fprintf(el->el_outfile, "%-15s->  %s\n",
-				    key__decode_str(firstbuf, unparsbuf, STRQQ),
-				    bp->name);
-			} else {
-				(void) fprintf(el->el_outfile,
-				    "%-4s to %-7s->  %s\n",
-				    key__decode_str(firstbuf, unparsbuf, STRQQ),
-				    key__decode_str(lastbuf, extrabuf, STRQQ),
-				    bp->name);
-			}
-			return;
-		}
-	}
-#ifdef MAP_DEBUG
-	if (map == el->el_map.key) {
-		(void) fprintf(el->el_outfile,
-		    "BUG!!! %s isn't bound to anything.\n",
-		    key__decode_str(firstbuf, unparsbuf, STRQQ));
-		(void) fprintf(el->el_outfile, "el->el_map.key[%d] == %d\n",
-		    first, el->el_map.key[first]);
-	} else {
-		(void) fprintf(el->el_outfile,
-		    "BUG!!! %s isn't bound to anything.\n",
-		    key__decode_str(firstbuf, unparsbuf, STRQQ));
-		(void) fprintf(el->el_outfile, "el->el_map.alt[%d] == %d\n",
-		    first, el->el_map.alt[first]);
-	}
-#endif
-	EL_ABORT((el->el_errfile, "Error printing keys\n"));
-}
-
-
-/* map_print_all_keys():
- *	Print the function description for all keys.
- */
-private void
-map_print_all_keys(EditLine *el)
-{
-	int prev, i;
-
-	(void) fprintf(el->el_outfile, "Standard key bindings\n");
-	prev = 0;
-	for (i = 0; i < N_KEYS; i++) {
-		if (el->el_map.key[prev] == el->el_map.key[i])
-			continue;
-		map_print_some_keys(el, el->el_map.key, prev, i - 1);
-		prev = i;
-	}
-	map_print_some_keys(el, el->el_map.key, prev, i - 1);
-
-	(void) fprintf(el->el_outfile, "Alternative key bindings\n");
-	prev = 0;
-	for (i = 0; i < N_KEYS; i++) {
-		if (el->el_map.alt[prev] == el->el_map.alt[i])
-			continue;
-		map_print_some_keys(el, el->el_map.alt, prev, i - 1);
-		prev = i;
-	}
-	map_print_some_keys(el, el->el_map.alt, prev, i - 1);
-
-	(void) fprintf(el->el_outfile, "Multi-character bindings\n");
-	key_print(el, "");
-	(void) fprintf(el->el_outfile, "Arrow key bindings\n");
-	term_print_arrow(el, "");
-}
-
-
-/* map_bind():
- *	Add/remove/change bindings
- */
-protected int
-map_bind(EditLine *el, int argc, const char **argv)
-{
-	el_action_t *map;
-	int ntype, rem;
-	const char *p;
-	char inbuf[EL_BUFSIZ];
-	char outbuf[EL_BUFSIZ];
-	const char *in = NULL;
-	char *out = NULL;
-	el_bindings_t *bp;
-	int cmd;
-	int key;
-
-	if (argv == NULL)
-		return (-1);
-
-	map = el->el_map.key;
-	ntype = XK_CMD;
-	key = rem = 0;
-	for (argc = 1; (p = argv[argc]) != NULL; argc++)
-		if (p[0] == '-')
-			switch (p[1]) {
-			case 'a':
-				map = el->el_map.alt;
-				break;
-
-			case 's':
-				ntype = XK_STR;
-				break;
-#ifdef notyet
-			case 'c':
-				ntype = XK_EXE;
-				break;
-#endif
-			case 'k':
-				key = 1;
-				break;
-
-			case 'r':
-				rem = 1;
-				break;
-
-			case 'v':
-				map_init_vi(el);
-				return (0);
-
-			case 'e':
-				map_init_emacs(el);
-				return (0);
-
-			case 'l':
-				for (bp = el->el_map.help; bp->name != NULL;
-				    bp++)
-					(void) fprintf(el->el_outfile,
-					    "%s\n\t%s\n",
-					    bp->name, bp->description);
-				return (0);
-			default:
-				(void) fprintf(el->el_errfile,
-				    "%s: Invalid switch `%c'.\n",
-				    argv[0], p[1]);
-			}
-		else
-			break;
-
-	if (argv[argc] == NULL) {
-		map_print_all_keys(el);
-		return (0);
-	}
-	if (key)
-		in = argv[argc++];
-	else if ((in = parse__string(inbuf, argv[argc++])) == NULL) {
-		(void) fprintf(el->el_errfile,
-		    "%s: Invalid \\ or ^ in instring.\n",
-		    argv[0]);
-		return (-1);
-	}
-	if (rem) {
-		if (key) {
-			(void) term_clear_arrow(el, in);
-			return (-1);
-		}
-		if (in[1])
-			(void) key_delete(el, in);
-		else if (map[(unsigned char) *in] == ED_SEQUENCE_LEAD_IN)
-			(void) key_delete(el, in);
-		else
-			map[(unsigned char) *in] = ED_UNASSIGNED;
-		return (0);
-	}
-	if (argv[argc] == NULL) {
-		if (key)
-			term_print_arrow(el, in);
-		else
-			map_print_key(el, map, in);
-		return (0);
-	}
-#ifdef notyet
-	if (argv[argc + 1] != NULL) {
-		bindkey_usage();
-		return (-1);
-	}
-#endif
-
-	switch (ntype) {
-	case XK_STR:
-	case XK_EXE:
-		if ((out = parse__string(outbuf, argv[argc])) == NULL) {
-			(void) fprintf(el->el_errfile,
-			    "%s: Invalid \\ or ^ in outstring.\n", argv[0]);
-			return (-1);
-		}
-		if (key)
-			term_set_arrow(el, in, key_map_str(el, out), ntype);
-		else
-			key_add(el, in, key_map_str(el, out), ntype);
-		map[(unsigned char) *in] = ED_SEQUENCE_LEAD_IN;
-		break;
-
-	case XK_CMD:
-		if ((cmd = parse_cmd(el, argv[argc])) == -1) {
-			(void) fprintf(el->el_errfile,
-			    "%s: Invalid command `%s'.\n", argv[0], argv[argc]);
-			return (-1);
-		}
-		if (key)
-			term_set_arrow(el, in, key_map_str(el, out), ntype);
-		else {
-			if (in[1]) {
-				key_add(el, in, key_map_cmd(el, cmd), ntype);
-				map[(unsigned char) *in] = ED_SEQUENCE_LEAD_IN;
-			} else {
-				key_clear(el, map, in);
-				map[(unsigned char) *in] = cmd;
-			}
-		}
-		break;
-
-	default:
-		EL_ABORT((el->el_errfile, "Bad XK_ type\n", ntype));
-		break;
-	}
-	return (0);
-}
-
-
-/* map_addfunc():
- *	add a user defined function
- */
-protected int
-map_addfunc(EditLine *el, const char *name, const char *help, el_func_t func)
-{
-	void *p;
-	int nf = el->el_map.nfunc + 2;
-
-	if (name == NULL || help == NULL || func == NULL)
-		return (-1);
-
-	if ((p = el_realloc(el->el_map.func, nf * sizeof(el_func_t))) == NULL)
-		return (-1);
-	el->el_map.func = (el_func_t *) p;
-	if ((p = el_realloc(el->el_map.help, nf * sizeof(el_bindings_t)))
-	    == NULL)
-		return (-1);
-	el->el_map.help = (el_bindings_t *) p;
-
-	nf = el->el_map.nfunc;
-	el->el_map.func[nf] = func;
-
-	el->el_map.help[nf].name = name;
-	el->el_map.help[nf].func = nf;
-	el->el_map.help[nf].description = help;
-	el->el_map.help[++nf].name = NULL;
-	el->el_map.nfunc++;
-
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/map.h 1:16.1.1~dfsg-1/main/editline/map.h
--- 1:13.23.1~dfsg-2/main/editline/map.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/map.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,79 +0,0 @@
-/*	$NetBSD: map.h,v 1.7 2002/03/18 16:00:56 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)map.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.map.h:	Editor maps
- */
-#ifndef _h_el_map
-#define	_h_el_map
-
-typedef struct el_bindings_t {	/* for the "bind" shell command */
-	const char	*name;		/* function name for bind command */
-	int		 func;		/* function numeric value */
-	const char	*description;	/* description of function */
-} el_bindings_t;
-
-
-typedef struct el_map_t {
-	el_action_t	*alt;		/* The current alternate key map */
-	el_action_t	*key;		/* The current normal key map	*/
-	el_action_t	*current;	/* The keymap we are using	*/
-	const el_action_t *emacs;	/* The default emacs key map	*/
-	const el_action_t *vic;		/* The vi command mode key map	*/
-	const el_action_t *vii;		/* The vi insert mode key map	*/
-	int		 type;		/* Emacs or vi			*/
-	el_bindings_t	*help;		/* The help for the editor functions */
-	el_func_t	*func;		/* List of available functions	*/
-	int		 nfunc;		/* The number of functions/help items */
-} el_map_t;
-
-#define	MAP_EMACS	0
-#define	MAP_VI		1
-
-protected int	map_bind(EditLine *, int, const char **);
-protected int	map_init(EditLine *);
-protected void	map_end(EditLine *);
-protected void	map_init_vi(EditLine *);
-protected void	map_init_emacs(EditLine *);
-protected int	map_set_editor(EditLine *, char *);
-protected int	map_get_editor(EditLine *, const char **);
-protected int	map_addfunc(EditLine *, const char *, const char *, el_func_t);
-
-#endif /* _h_el_map */
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/fgetln.c 1:16.1.1~dfsg-1/main/editline/np/fgetln.c
--- 1:13.23.1~dfsg-2/main/editline/np/fgetln.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/fgetln.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,88 +0,0 @@
-/*	$NetBSD: fgetln.c,v 1.1.1.1 1999/04/12 07:43:21 crooksa Exp $	*/
-
-/*-
- * Copyright (c) 1998 The NetBSD Foundation, Inc.
- * All rights reserved.
- *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Christos Zoulas.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *        This product includes software developed by the NetBSD
- *        Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
-#include "config.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <errno.h>
-#include <string.h>
-
-char *
-fgetln(fp, len)
-	FILE *fp;
-	size_t *len;
-{
-	static char *buf = NULL;
-	static size_t bufsiz = 0;
-	char *ptr;
-
-
-	if (buf == NULL) {
-		bufsiz = BUFSIZ;
-		if ((buf = malloc(bufsiz)) == NULL)
-			return NULL;
-	}
-
-	if (fgets(buf, bufsiz, fp) == NULL)
-		return NULL;
-	*len = 0;
-
-	while ((ptr = strchr(&buf[*len], '\n')) == NULL) {
-		size_t nbufsiz = bufsiz + BUFSIZ;
-		char *nbuf = realloc(buf, nbufsiz);
-
-		if (nbuf == NULL) {
-			int oerrno = errno;
-			free(buf);
-			errno = oerrno;
-			buf = NULL;
-			return NULL;
-		} else
-			buf = nbuf;
-
-		*len = bufsiz;
-		if (fgets(&buf[bufsiz], BUFSIZ, fp) == NULL)
-			return buf;
-
-		bufsiz = nbufsiz;
-	}
-
-	*len = (ptr - buf) + 1;
-	return buf;
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/strlcat.c 1:16.1.1~dfsg-1/main/editline/np/strlcat.c
--- 1:13.23.1~dfsg-2/main/editline/np/strlcat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/strlcat.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,67 +0,0 @@
-/*
- * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-#include "config.h"
-#include <sys/types.h>
-#include <string.h>
-
-/*
- * Appends src to string dst of size siz (unlike strncat, siz is the
- * full size of dst, not space left).  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
- * Returns strlen(initial dst) + strlen(src); if retval >= siz,
- * truncation occurred.
- */
-size_t strlcat(dst, src, siz)
-	char *dst;
-	const char *src;
-	size_t siz;
-{
-	register char *d = dst;
-	register const char *s = src;
-	register size_t n = siz;
-	size_t dlen;
-
-	/* Find the end of dst and adjust bytes left but don't go past end */
-	while (n-- != 0 && *d != '\0')
-		d++;
-	dlen = d - dst;
-	n = siz - dlen;
-
-	if (n == 0)
-		return(dlen + strlen(s));
-	while (*s != '\0') {
-		if (n != 1) {
-			*d++ = *s;
-			n--;
-		}
-		s++;
-	}
-	*d = '\0';
-
-	return(dlen + (s - src));	/* count does not include NUL */
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/strlcpy.c 1:16.1.1~dfsg-1/main/editline/np/strlcpy.c
--- 1:13.23.1~dfsg-2/main/editline/np/strlcpy.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/strlcpy.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,65 +0,0 @@
-/*	$OpenBSD: strlcpy.c,v 1.4 1999/05/01 18:56:41 millert Exp $	*/
-
-/*
- * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
- * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
- * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
- * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
-
-#include "config.h"
-#include <sys/types.h>
-#include <string.h>
-
-/*
- * Copy src to string dst of size siz.  At most siz-1 characters
- * will be copied.  Always NUL terminates (unless siz == 0).
- * Returns strlen(src); if retval >= siz, truncation occurred.
- */
-size_t strlcpy(dst, src, siz)
-	char *dst;
-	const char *src;
-	size_t siz;
-{
-	register char *d = dst;
-	register const char *s = src;
-	register size_t n = siz;
-
-	/* Copy as many bytes as will fit */
-	if (n != 0 && --n != 0) {
-		do {
-			if ((*d++ = *s++) == 0)
-				break;
-		} while (--n != 0);
-	}
-
-	/* Not enough room in dst, add NUL and traverse rest of src */
-	if (n == 0) {
-		if (siz != 0)
-			*d = '\0';		/* NUL-terminate dst */
-		while (*s++)
-			;
-	}
-
-	return(s - src - 1);	/* count does not include NUL */
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/unvis.c 1:16.1.1~dfsg-1/main/editline/np/unvis.c
--- 1:13.23.1~dfsg-2/main/editline/np/unvis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/unvis.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,322 +0,0 @@
-/*	$NetBSD: unvis.c,v 1.22 2002/03/23 17:38:27 christos Exp $	*/
-
-/*-
- * Copyright (c) 1989, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include <sys/cdefs.h>
-#if defined(LIBC_SCCS) && !defined(lint)
-#if 0
-static char sccsid[] = "@(#)unvis.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: unvis.c,v 1.22 2002/03/23 17:38:27 christos Exp $");
-#endif
-#endif /* LIBC_SCCS and not lint */
-
-#define __LIBC12_SOURCE__
-
-#include <sys/types.h>
-
-#include <assert.h>
-#include <ctype.h>
-#include <stdio.h>
-#include "np/vis.h"
-
-#ifdef __weak_alias
-__weak_alias(strunvis,_strunvis)
-__weak_alias(unvis,_unvis)
-#endif
-
-#ifdef __warn_references
-__warn_references(unvis,
-    "warning: reference to compatibility unvis(); include <vis.h> for correct reference")
-#endif
-
-#ifndef HAVE_VIS_H
-/*
- * decode driven by state machine
- */
-#define	S_GROUND	0	/* haven't seen escape char */
-#define	S_START		1	/* start decoding special sequence */
-#define	S_META		2	/* metachar started (M) */
-#define	S_META1		3	/* metachar more, regular char (-) */
-#define	S_CTRL		4	/* control char started (^) */
-#define	S_OCTAL2	5	/* octal digit 2 */
-#define	S_OCTAL3	6	/* octal digit 3 */
-#define S_HEX1		7	/* hex digit */
-#define S_HEX2		8	/* hex digit 2 */
-
-#define	isoctal(c)	(((u_char)(c)) >= '0' && ((u_char)(c)) <= '7')
-#define xtod(c)		(isdigit(c) ? (c - '0') : ((tolower(c) - 'a') + 10))
-
-int
-unvis(cp, c, astate, flag)
-	char *cp;
-	int c;
-	int *astate, flag;
-{
-	return __unvis13(cp, (int)c, astate, flag);
-}
-
-/*
- * unvis - decode characters previously encoded by vis
- */
-int
-__unvis13(cp, c, astate, flag)
-	char *cp;
-	int c;
-	int *astate, flag;
-{
-
-	_DIAGASSERT(cp != NULL);
-	_DIAGASSERT(astate != NULL);
-
-	if (flag & UNVIS_END) {
-		if (*astate == S_OCTAL2 || *astate == S_OCTAL3
-		    || *astate == S_HEX2) {
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		}
-		return (*astate == S_GROUND ? UNVIS_NOCHAR : UNVIS_SYNBAD);
-	}
-
-	switch (*astate) {
-
-	case S_GROUND:
-		*cp = 0;
-		if (c == '\\') {
-			*astate = S_START;
-			return (0);
-		}
-		if ((flag & VIS_HTTPSTYLE) && c == '%') {
-			*astate = S_HEX1;
-			return (0);
-		}
-		*cp = c;
-		return (UNVIS_VALID);
-
-	case S_START:
-		switch(c) {
-		case '\\':
-			*cp = c;
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case '0': case '1': case '2': case '3':
-		case '4': case '5': case '6': case '7':
-			*cp = (c - '0');
-			*astate = S_OCTAL2;
-			return (0);
-		case 'M':
-			*cp = (char)0200;
-			*astate = S_META;
-			return (0);
-		case '^':
-			*astate = S_CTRL;
-			return (0);
-		case 'n':
-			*cp = '\n';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'r':
-			*cp = '\r';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'b':
-			*cp = '\b';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'a':
-			*cp = '\007';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'v':
-			*cp = '\v';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 't':
-			*cp = '\t';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'f':
-			*cp = '\f';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 's':
-			*cp = ' ';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case 'E':
-			*cp = '\033';
-			*astate = S_GROUND;
-			return (UNVIS_VALID);
-		case '\n':
-			/*
-			 * hidden newline
-			 */
-			*astate = S_GROUND;
-			return (UNVIS_NOCHAR);
-		case '$':
-			/*
-			 * hidden marker
-			 */
-			*astate = S_GROUND;
-			return (UNVIS_NOCHAR);
-		}
-		*astate = S_GROUND;
-		return (UNVIS_SYNBAD);
-
-	case S_META:
-		if (c == '-')
-			*astate = S_META1;
-		else if (c == '^')
-			*astate = S_CTRL;
-		else {
-			*astate = S_GROUND;
-			return (UNVIS_SYNBAD);
-		}
-		return (0);
-
-	case S_META1:
-		*astate = S_GROUND;
-		*cp |= c;
-		return (UNVIS_VALID);
-
-	case S_CTRL:
-		if (c == '?')
-			*cp |= 0177;
-		else
-			*cp |= c & 037;
-		*astate = S_GROUND;
-		return (UNVIS_VALID);
-
-	case S_OCTAL2:	/* second possible octal digit */
-		if (isoctal(c)) {
-			/*
-			 * yes - and maybe a third
-			 */
-			*cp = (*cp << 3) + (c - '0');
-			*astate = S_OCTAL3;
-			return (0);
-		}
-		/*
-		 * no - done with current sequence, push back passed char
-		 */
-		*astate = S_GROUND;
-		return (UNVIS_VALIDPUSH);
-
-	case S_OCTAL3:	/* third possible octal digit */
-		*astate = S_GROUND;
-		if (isoctal(c)) {
-			*cp = (*cp << 3) + (c - '0');
-			return (UNVIS_VALID);
-		}
-		/*
-		 * we were done, push back passed char
-		 */
-		return (UNVIS_VALIDPUSH);
-	case S_HEX1:
-		if (isxdigit(c)) {
-			*cp = xtod(c);
-			*astate = S_HEX2;
-			return (0);
-		}
-		/*
-		 * no - done with current sequence, push back passed char
-		 */
-		*astate = S_GROUND;
-		return (UNVIS_VALIDPUSH);
-	case S_HEX2:
-                *astate = S_GROUND;
-                if (isxdigit(c)) {
-                        *cp = xtod(c) | (*cp << 4);
-			return (UNVIS_VALID);
-		}
-                return (UNVIS_VALIDPUSH);
-	default:
-		/*
-		 * decoder in unknown state - (probably uninitialized)
-		 */
-		*astate = S_GROUND;
-		return (UNVIS_SYNBAD);
-	}
-}
-
-/*
- * strunvis - decode src into dst
- *
- *	Number of chars decoded into dst is returned, -1 on error.
- *	Dst is null terminated.
- */
-
-int
-strunvisx(dst, src, flag)
-	char *dst;
-	const char *src;
-	int flag;
-{
-	char c;
-	char *start = dst;
-	int state = 0;
-
-	_DIAGASSERT(src != NULL);
-	_DIAGASSERT(dst != NULL);
-
-	while ((c = *src++) != '\0') {
-	again:
-		switch (__unvis13(dst, c, &state, flag)) {
-		case UNVIS_VALID:
-			dst++;
-			break;
-		case UNVIS_VALIDPUSH:
-			dst++;
-			goto again;
-		case 0:
-		case UNVIS_NOCHAR:
-			break;
-		default:
-			return (-1);
-		}
-	}
-	if (__unvis13(dst, c, &state, UNVIS_END) == UNVIS_VALID)
-		dst++;
-	*dst = '\0';
-	return (dst - start);
-}
-
-int
-strunvis(dst, src)
-	char *dst;
-	const char *src;
-{
-	return strunvisx(dst, src, 0);
-}
-#endif
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/vis.c 1:16.1.1~dfsg-1/main/editline/np/vis.c
--- 1:13.23.1~dfsg-2/main/editline/np/vis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/vis.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,348 +0,0 @@
-/*	$NetBSD: vis.c,v 1.22 2002/03/23 17:38:27 christos Exp $	*/
-
-/*-
- * Copyright (c) 1999 The NetBSD Foundation, Inc.
- * Copyright (c) 1989, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include <sys/cdefs.h>
-#if defined(LIBC_SCCS) && !defined(lint)
-__RCSID("$NetBSD: vis.c,v 1.22 2002/03/23 17:38:27 christos Exp $");
-#endif /* LIBC_SCCS and not lint */
-
-#include <sys/types.h>
-
-#include <assert.h>
-#include "np/vis.h"
-#include <stdlib.h>
-#include <stdint.h>
-
-#ifdef __weak_alias
-__weak_alias(strsvis,_strsvis)
-__weak_alias(strsvisx,_strsvisx)
-__weak_alias(strvis,_strvis)
-__weak_alias(strvisx,_strvisx)
-__weak_alias(svis,_svis)
-__weak_alias(vis,_vis)
-#endif
-
-#ifndef HAVE_VIS_H
-#include <ctype.h>
-#include <limits.h>
-#include <stdio.h>
-#include <string.h>
-
-#undef BELL
-#if defined(__STDC__)
-#define BELL '\a'
-#else
-#define BELL '\007'
-#endif
-
-#ifdef SOLARIS
-#include <alloca.h>
-#endif
-
-#define isoctal(c)	(((u_char)(c)) >= '0' && ((u_char)(c)) <= '7')
-#define iswhite(c)	(c == ' ' || c == '\t' || c == '\n')
-#define issafe(c)	(c == '\b' || c == BELL || c == '\r')
-#define xtoa(c)		"0123456789abcdef"[c]
-
-#define MAXEXTRAS       5
-
-
-#define MAKEEXTRALIST(flag, extra, orig)				      \
-do {									      \
-	const char *o = orig;						      \
-	char *e;						      	      \
-	while (*o++)							      \
-		continue;						      \
-	extra = alloca((size_t)((o - orig) + MAXEXTRAS));	      	      \
-	for (o = orig, e = extra; (*e++ = *o++) != '\0';)		      \
-		continue;						      \
-	e--;								      \
-	if (flag & VIS_SP) *e++ = ' ';				      	      \
-	if (flag & VIS_TAB) *e++ = '\t';				      \
-	if (flag & VIS_NL) *e++ = '\n';				      	      \
-	if ((flag & VIS_NOSLASH) == 0) *e++ = '\\';		      	      \
-	*e = '\0';							      \
-} while (/*CONSTCOND*/0)
-
-
-/*
- * This is HVIS, the macro of vis used to HTTP style (RFC 1808)
- */
-#define HVIS(dst, c, flag, nextc, extra)				      \
-do 									      \
-	if (!isascii(c) || !isalnum(c) || strchr("$-_.+!*'(),", c) != NULL) { \
-		*dst++ = '%';						      \
-		*dst++ = xtoa(((unsigned int)c >> 4) & 0xf);		      \
-		*dst++ = xtoa((unsigned int)c & 0xf);			      \
-	} else {							      \
-		SVIS(dst, c, flag, nextc, extra);			      \
-	}								      \
-while (/*CONSTCOND*/0)
-
-/*
- * This is SVIS, the central macro of vis.
- * dst:	      Pointer to the destination buffer
- * c:	      Character to encode
- * flag:      Flag word
- * nextc:     The character following 'c'
- * extra:     Pointer to the list of extra characters to be
- *	      backslash-protected.
- */
-#define SVIS(dst, c, flag, nextc, extra)				      \
-do {									      \
-	int isextra, isc;						      \
-	isextra = strchr(extra, c) != NULL;				      \
-	if (!isextra && isascii(c) && (isgraph(c) || iswhite(c) ||	      \
-	    ((flag & VIS_SAFE) && issafe(c)))) {			      \
-		*dst++ = c;						      \
-		break;							      \
-	}								      \
-	isc = 0;							      \
-	if (flag & VIS_CSTYLE) {					      \
-		switch (c) {						      \
-		case '\n':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'n';		      \
-			break;						      \
-		case '\r':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'r';		      \
-			break;						      \
-		case '\b':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'b';		      \
-			break;						      \
-		case BELL:						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'a';		      \
-			break;						      \
-		case '\v':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'v';		      \
-			break;						      \
-		case '\t':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 't';		      \
-			break;						      \
-		case '\f':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 'f';		      \
-			break;						      \
-		case ' ':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = 's';		      \
-			break;						      \
-		case '\0':						      \
-			isc = 1; *dst++ = '\\'; *dst++ = '0';		      \
-			if (isoctal(nextc)) {				      \
-				*dst++ = '0';				      \
-				*dst++ = '0';				      \
-			}						      \
-		}							      \
-	}								      \
-	if (isc) break;							      \
-	if (isextra || ((c & 0177) == ' ') || (flag & VIS_OCTAL)) {	      \
-		*dst++ = '\\';						      \
-		*dst++ = (u_char)(((uint32_t)(u_char)c >> 6) & 03) + '0';     \
-		*dst++ = (u_char)(((uint32_t)(u_char)c >> 3) & 07) + '0';     \
-		*dst++ =			     (c	      & 07) + '0';    \
-	} else {							      \
-		if ((flag & VIS_NOSLASH) == 0) *dst++ = '\\';		      \
-		if (c & 0200) {						      \
-			c &= 0177; *dst++ = 'M';			      \
-		}							      \
-		if (iscntrl(c)) {					      \
-			*dst++ = '^';					      \
-			if (c == 0177)					      \
-				*dst++ = '?';				      \
-			else						      \
-				*dst++ = c + '@';			      \
-		} else {						      \
-			*dst++ = '-'; *dst++ = c;			      \
-		}							      \
-	}								      \
-} while (/*CONSTCOND*/0)
-
-
-/*
- * svis - visually encode characters, also encoding the characters
- * 	  pointed to by `extra'
- */
-char *
-svis(dst, c, flag, nextc, extra)
-	char *dst;
-	int c, flag, nextc;
-	const char *extra;
-{
-	char *nextra;
-	_DIAGASSERT(dst != NULL);
-	_DIAGASSERT(extra != NULL);
-	MAKEEXTRALIST(flag, nextra, extra);
-	if (flag & VIS_HTTPSTYLE)
-		HVIS(dst, c, flag, nextc, nextra);
-	else
-		SVIS(dst, c, flag, nextc, nextra);
-	*dst = '\0';
-	return(dst);
-}
-
-
-/*
- * strsvis, strsvisx - visually encode characters from src into dst
- *
- *	Extra is a pointer to a \0-terminated list of characters to
- *	be encoded, too. These functions are useful e. g. to
- *	encode strings in such a way so that they are not interpreted
- *	by a shell.
- *
- *	Dst must be 4 times the size of src to account for possible
- *	expansion.  The length of dst, not including the trailing NULL,
- *	is returned.
- *
- *	Strsvisx encodes exactly len bytes from src into dst.
- *	This is useful for encoding a block of data.
- */
-int
-strsvis(dst, src, flag, extra)
-	char *dst;
-	const char *src;
-	int flag;
-	const char *extra;
-{
-	char c;
-	char *start;
-	char *nextra;
-
-	_DIAGASSERT(dst != NULL);
-	_DIAGASSERT(src != NULL);
-	_DIAGASSERT(extra != NULL);
-	MAKEEXTRALIST(flag, nextra, extra);
-	if (flag & VIS_HTTPSTYLE) {
-		for (start = dst; (c = *src++) != '\0'; /* empty */)
-			HVIS(dst, c, flag, *src, nextra);
-	} else {
-		for (start = dst; (c = *src++) != '\0'; /* empty */)
-			SVIS(dst, c, flag, *src, nextra);
-	}
-	*dst = '\0';
-	return (dst - start);
-}
-
-
-int
-strsvisx(dst, src, len, flag, extra)
-	char *dst;
-	const char *src;
-	size_t len;
-	int flag;
-	const char *extra;
-{
-	char c;
-	char *start;
-	char *nextra;
-
-	_DIAGASSERT(dst != NULL);
-	_DIAGASSERT(src != NULL);
-	_DIAGASSERT(extra != NULL);
-	MAKEEXTRALIST(flag, nextra, extra);
-
-	if (flag & VIS_HTTPSTYLE) {
-		for (start = dst; len > 0; len--) {
-			c = *src++;
-			HVIS(dst, c, flag, len ? *src : '\0', nextra);
-		}
-	} else {
-		for (start = dst; len > 0; len--) {
-			c = *src++;
-			SVIS(dst, c, flag, len ? *src : '\0', nextra);
-		}
-	}
-	*dst = '\0';
-	return (dst - start);
-}
-
-
-/*
- * vis - visually encode characters
- */
-char *
-vis(dst, c, flag, nextc)
-	char *dst;
-	int c, flag, nextc;
-
-{
-	char *extra;
-
-	_DIAGASSERT(dst != NULL);
-
-	MAKEEXTRALIST(flag, extra, "");
-	if (flag & VIS_HTTPSTYLE)
-	    HVIS(dst, c, flag, nextc, extra);
-	else
-	    SVIS(dst, c, flag, nextc, extra);
-	*dst = '\0';
-	return (dst);
-}
-
-
-/*
- * strvis, strvisx - visually encode characters from src into dst
- *
- *	Dst must be 4 times the size of src to account for possible
- *	expansion.  The length of dst, not including the trailing NULL,
- *	is returned.
- *
- *	Strvisx encodes exactly len bytes from src into dst.
- *	This is useful for encoding a block of data.
- */
-int
-strvis(dst, src, flag)
-	char *dst;
-	const char *src;
-	int flag;
-{
-	char *extra;
-
-	MAKEEXTRALIST(flag, extra, "");
-	return (strsvis(dst, src, flag, extra));
-}
-
-
-int
-strvisx(dst, src, len, flag)
-	char *dst;
-	const char *src;
-	size_t len;
-	int flag;
-{
-	char *extra;
-
-	MAKEEXTRALIST(flag, extra, "");
-	return (strsvisx(dst, src, len, flag, extra));
-}
-#endif
diff -pruN 1:13.23.1~dfsg-2/main/editline/np/vis.h 1:16.1.1~dfsg-1/main/editline/np/vis.h
--- 1:13.23.1~dfsg-2/main/editline/np/vis.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/np/vis.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,96 +0,0 @@
-/*	$NetBSD: vis.h,v 1.12 2002/03/23 17:39:05 christos Exp $	*/
-
-/*-
- * Copyright (c) 1990, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)vis.h	8.1 (Berkeley) 6/2/93
- */
-
-#ifndef _VIS_H_
-#define	_VIS_H_
-
-/*
- * to select alternate encoding format
- */
-#define	VIS_OCTAL	0x01	/* use octal \ddd format */
-#define	VIS_CSTYLE	0x02	/* use \[nrft0..] where appropiate */
-
-/*
- * to alter set of characters encoded (default is to encode all
- * non-graphic except space, tab, and newline).
- */
-#define	VIS_SP		0x04	/* also encode space */
-#define	VIS_TAB		0x08	/* also encode tab */
-#define	VIS_NL		0x10	/* also encode newline */
-#define	VIS_WHITE	(VIS_SP | VIS_TAB | VIS_NL)
-#define	VIS_SAFE	0x20	/* only encode "unsafe" characters */
-
-/*
- * other
- */
-#define	VIS_NOSLASH	0x40	/* inhibit printing '\' */
-#define	VIS_HTTPSTYLE	0x80	/* http-style escape % HEX HEX */
-
-/*
- * unvis return codes
- */
-#define	UNVIS_VALID	 1	/* character valid */
-#define	UNVIS_VALIDPUSH	 2	/* character valid, push back passed char */
-#define	UNVIS_NOCHAR	 3	/* valid sequence, no character produced */
-#define	UNVIS_SYNBAD	-1	/* unrecognized escape sequence */
-#define	UNVIS_ERROR	-2	/* decoder in unknown state (unrecoverable) */
-
-/*
- * unvis flags
- */
-#define	UNVIS_END	1	/* no more characters */
-
-#include <sys/cdefs.h>
-
-__BEGIN_DECLS
-char	*vis __P((char *, int, int, int));
-char	*svis __P((char *, int, int, int, const char *));
-int	strvis __P((char *, const char *, int));
-int	strsvis __P((char *, const char *, int, const char *));
-int	strvisx __P((char *, const char *, size_t, int));
-int	strsvisx __P((char *, const char *, size_t, int, const char *));
-int	strunvis __P((char *, const char *));
-int	strunvisx __P((char *, const char *, int));
-#ifdef __LIBC12_SOURCE__
-int	unvis __P((char *, int, int *, int));
-int	__unvis13 __P((char *, int, int *, int));
-#else
-int	unvis __P((char *, int, int *, int))	__RENAME(__unvis13);
-#endif
-__END_DECLS
-
-#endif /* !_VIS_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/editline/parse.c 1:16.1.1~dfsg-1/main/editline/parse.c
--- 1:13.23.1~dfsg-2/main/editline/parse.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/parse.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,259 +0,0 @@
-/*	$NetBSD: parse.c,v 1.15 2002/03/18 16:00:56 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)parse.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: parse.c,v 1.15 2002/03/18 16:00:56 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * parse.c: parse an editline extended command
- *
- * commands are:
- *
- *	bind
- *	echotc
- *	edit
- *	gettc
- *	history
- *	settc
- *	setty
- */
-#include "el.h"
-#include "tokenizer.h"
-#include <stdlib.h>
-
-private const struct {
-	const char *name;
-	int (*func)(EditLine *, int, const char **);
-} cmds[] = {
-	{ "bind",	map_bind	},
-	{ "echotc",	term_echotc	},
-	{ "edit",	el_editmode	},
-	{ "history",	hist_list	},
-	{ "telltc",	term_telltc	},
-	{ "settc",	term_settc	},
-	{ "setty",	tty_stty	},
-	{ NULL,		NULL		}
-};
-
-
-/* parse_line():
- *	Parse a line and dispatch it
- */
-protected int
-parse_line(EditLine *el, const char *line)
-{
-	const char **argv;
-	int argc;
-	Tokenizer *tok;
-
-	tok = tok_init(NULL);
-	tok_line(tok, line, &argc, &argv);
-	argc = el_parse(el, argc, argv);
-	tok_end(tok);
-	return (argc);
-}
-
-
-/* el_parse():
- *	Command dispatcher
- */
-public int
-el_parse(EditLine *el, int argc, const char *argv[])
-{
-	const char *ptr;
-	int i;
-
-	if (argc < 1)
-		return (-1);
-	ptr = strchr(argv[0], ':');
-	if (ptr != NULL) {
-		char *tprog;
-		size_t l;
-
-		if (ptr == argv[0])
-			return (0);
-		l = ptr - argv[0] - 1;
-		tprog = (char *) el_malloc(l + 1);
-		if (tprog == NULL)
-			return (0);
-		(void) strncpy(tprog, argv[0], l);
-		tprog[l] = '\0';
-		ptr++;
-		l = el_match(el->el_prog, tprog);
-		el_free(tprog);
-		if (!l)
-			return (0);
-	} else
-		ptr = argv[0];
-
-	for (i = 0; cmds[i].name != NULL; i++)
-		if (strcmp(cmds[i].name, ptr) == 0) {
-			i = (*cmds[i].func) (el, argc, argv);
-			return (-i);
-		}
-	return (-1);
-}
-
-
-/* parse__escape():
- *	Parse a string of the form ^<char> \<odigit> \<char> and return
- *	the appropriate character or -1 if the escape is not valid
- */
-protected int
-parse__escape(const char **const ptr)
-{
-	const char *p;
-	int c;
-
-	p = *ptr;
-
-	if (p[1] == 0)
-		return (-1);
-
-	if (*p == '\\') {
-		p++;
-		switch (*p) {
-		case 'a':
-			c = '\007';	/* Bell */
-			break;
-		case 'b':
-			c = '\010';	/* Backspace */
-			break;
-		case 't':
-			c = '\011';	/* Horizontal Tab */
-			break;
-		case 'n':
-			c = '\012';	/* New Line */
-			break;
-		case 'v':
-			c = '\013';	/* Vertical Tab */
-			break;
-		case 'f':
-			c = '\014';	/* Form Feed */
-			break;
-		case 'r':
-			c = '\015';	/* Carriage Return */
-			break;
-		case 'e':
-			c = '\033';	/* Escape */
-			break;
-		case '0':
-		case '1':
-		case '2':
-		case '3':
-		case '4':
-		case '5':
-		case '6':
-		case '7':
-		{
-			int cnt, ch;
-
-			for (cnt = 0, c = 0; cnt < 3; cnt++) {
-				ch = *p++;
-				if (ch < '0' || ch > '7') {
-					p--;
-					break;
-				}
-				c = (c << 3) | (ch - '0');
-			}
-			if ((c & 0xffffff00) != 0)
-				return (-1);
-			--p;
-			break;
-		}
-		default:
-			c = *p;
-			break;
-		}
-	} else if (*p == '^' && isalpha((unsigned char) p[1])) {
-		p++;
-		c = (*p == '?') ? '\177' : (*p & 0237);
-	} else
-		c = *p;
-	*ptr = ++p;
-	return (c);
-}
-/* parse__string():
- *	Parse the escapes from in and put the raw string out
- */
-protected char *
-parse__string(char *out, const char *in)
-{
-	char *rv = out;
-	int n;
-
-	for (;;)
-		switch (*in) {
-		case '\0':
-			*out = '\0';
-			return (rv);
-
-		case '\\':
-		case '^':
-			if ((n = parse__escape(&in)) == -1)
-				return (NULL);
-			*out++ = n;
-			break;
-
-		default:
-			*out++ = *in++;
-			break;
-		}
-}
-
-
-/* parse_cmd():
- *	Return the command number for the command string given
- *	or -1 if one is not found
- */
-protected int
-parse_cmd(EditLine *el, const char *cmd)
-{
-	el_bindings_t *b;
-
-	for (b = el->el_map.help; b->name != NULL; b++)
-		if (strcmp(b->name, cmd) == 0)
-			return (b->func);
-	return (-1);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/parse.h 1:16.1.1~dfsg-1/main/editline/parse.h
--- 1:13.23.1~dfsg-2/main/editline/parse.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/parse.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,52 +0,0 @@
-/*	$NetBSD: parse.h,v 1.4 2000/09/04 22:06:31 lukem Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)parse.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.parse.h: Parser functions
- */
-#ifndef _h_el_parse
-#define	_h_el_parse
-
-protected int	 parse_line(EditLine *, const char *);
-protected int	 parse__escape(const char ** const);
-protected char	*parse__string(char *, const char *);
-protected int	 parse_cmd(EditLine *, const char *);
-
-#endif /* _h_el_parse */
diff -pruN 1:13.23.1~dfsg-2/main/editline/PLATFORMS 1:16.1.1~dfsg-1/main/editline/PLATFORMS
--- 1:13.23.1~dfsg-2/main/editline/PLATFORMS	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/PLATFORMS	1970-01-01 00:00:00.000000000 +0000
@@ -1,13 +0,0 @@
-This distribution of libedit is expected to work on at least the following
-platforms.  It may also work on additional platforms, but no explicit support
-for them is built into the configuration system.
-
-* Apple OS X 10.1.
-
-* FreeBSD 4.x.
-
-* NetBSD 1.5.
-
-* Red Hat Linux 7.2.
-
-* Sun Solaris 2.6.
diff -pruN 1:13.23.1~dfsg-2/main/editline/prompt.c 1:16.1.1~dfsg-1/main/editline/prompt.c
--- 1:13.23.1~dfsg-2/main/editline/prompt.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/prompt.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,174 +0,0 @@
-/*	$NetBSD: prompt.c,v 1.9 2002/03/18 16:00:56 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)prompt.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: prompt.c,v 1.9 2002/03/18 16:00:56 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * prompt.c: Prompt printing functions
- */
-#include <stdio.h>
-#include "el.h"
-
-private char	*prompt_default(EditLine *);
-private char	*prompt_default_r(EditLine *);
-
-/* prompt_default():
- *	Just a default prompt, in case the user did not provide one
- */
-private char *
-/*ARGSUSED*/
-prompt_default(EditLine *el)
-{
-	static char a[3] = {'?', ' ', '\0'};
-
-	return (a);
-}
-
-
-/* prompt_default_r():
- *	Just a default rprompt, in case the user did not provide one
- */
-private char *
-/*ARGSUSED*/
-prompt_default_r(EditLine *el)
-{
-	static char a[1] = {'\0'};
-
-	return (a);
-}
-
-
-/* prompt_print():
- *	Print the prompt and update the prompt position.
- *	We use an array of integers in case we want to pass
- * 	literal escape sequences in the prompt and we want a
- *	bit to flag them
- */
-protected void
-prompt_print(EditLine *el, int op)
-{
-	el_prompt_t *elp;
-	char *p;
-
-	if (op == EL_PROMPT)
-		elp = &el->el_prompt;
-	else
-		elp = &el->el_rprompt;
-	p = (elp->p_func) (el);
-	while (*p)
-		re_putc(el, *p++, 1);
-
-	elp->p_pos.v = el->el_refresh.r_cursor.v;
-	elp->p_pos.h = el->el_refresh.r_cursor.h;
-}
-
-
-/* prompt_init():
- *	Initialize the prompt stuff
- */
-protected int
-prompt_init(EditLine *el)
-{
-
-	el->el_prompt.p_func = prompt_default;
-	el->el_prompt.p_pos.v = 0;
-	el->el_prompt.p_pos.h = 0;
-	el->el_rprompt.p_func = prompt_default_r;
-	el->el_rprompt.p_pos.v = 0;
-	el->el_rprompt.p_pos.h = 0;
-	return (0);
-}
-
-
-/* prompt_end():
- *	Clean up the prompt stuff
- */
-protected void
-/*ARGSUSED*/
-prompt_end(EditLine *el)
-{
-}
-
-
-/* prompt_set():
- *	Install a prompt printing function
- */
-protected int
-prompt_set(EditLine *el, el_pfunc_t prf, int op)
-{
-	el_prompt_t *p;
-
-	if (op == EL_PROMPT)
-		p = &el->el_prompt;
-	else
-		p = &el->el_rprompt;
-	if (prf == NULL) {
-		if (op == EL_PROMPT)
-			p->p_func = prompt_default;
-		else
-			p->p_func = prompt_default_r;
-	} else
-		p->p_func = prf;
-	p->p_pos.v = 0;
-	p->p_pos.h = 0;
-	return (0);
-}
-
-
-/* prompt_get():
- *	Retrieve the prompt printing function
- */
-protected int
-prompt_get(EditLine *el, el_pfunc_t *prf, int op)
-{
-
-	if (prf == NULL)
-		return (-1);
-	if (op == EL_PROMPT)
-		*prf = el->el_prompt.p_func;
-	else
-		*prf = el->el_rprompt.p_func;
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/prompt.h 1:16.1.1~dfsg-1/main/editline/prompt.h
--- 1:13.23.1~dfsg-2/main/editline/prompt.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/prompt.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,62 +0,0 @@
-/*	$NetBSD: prompt.h,v 1.5 2000/09/04 22:06:31 lukem Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)prompt.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.prompt.h: Prompt printing stuff
- */
-#ifndef _h_el_prompt
-#define	_h_el_prompt
-
-#include "histedit.h"
-
-typedef char * (*el_pfunc_t)(EditLine*);
-
-typedef struct el_prompt_t {
-	el_pfunc_t	p_func;	/* Function to return the prompt	*/
-	coord_t		p_pos;	/* position in the line after prompt	*/
-} el_prompt_t;
-
-protected void	prompt_print(EditLine *, int);
-protected int	prompt_set(EditLine *, el_pfunc_t, int);
-protected int	prompt_get(EditLine *, el_pfunc_t *, int);
-protected int	prompt_init(EditLine *);
-protected void	prompt_end(EditLine *);
-
-#endif /* _h_el_prompt */
diff -pruN 1:13.23.1~dfsg-2/main/editline/read.c 1:16.1.1~dfsg-1/main/editline/read.c
--- 1:13.23.1~dfsg-2/main/editline/read.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/read.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,558 +0,0 @@
-/*	$NetBSD: read.c,v 1.21 2002/03/18 16:00:57 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)read.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: read.c,v 1.21 2002/03/18 16:00:57 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * read.c: Clean this junk up! This is horrible code.
- *	   Terminal read functions
- */
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include "el.h"
-
-#define	OKCMD	-1
-
-private int	read__fixio(int, int);
-private int	read_preread(EditLine *);
-private int	read_char(EditLine *, char *);
-private int	read_getcmd(EditLine *, el_action_t *, char *);
-
-/* read_init():
- *	Initialize the read stuff
- */
-protected int
-read_init(EditLine *el)
-{
-	/* builtin read_char */
-	el->el_read.read_char = read_char;
-	return 0;
-}
-
-
-/* el_read_setfn():
- *	Set the read char function to the one provided.
- *	If it is set to EL_BUILTIN_GETCFN, then reset to the builtin one.
- */
-protected int
-el_read_setfn(EditLine *el, el_rfunc_t rc)
-{
-	el->el_read.read_char = (rc == EL_BUILTIN_GETCFN) ? read_char : rc;
-	return 0;
-}
-
-
-/* el_read_getfn():
- *	return the current read char function, or EL_BUILTIN_GETCFN
- *	if it is the default one
- */
-protected el_rfunc_t
-el_read_getfn(EditLine *el)
-{
-       return (el->el_read.read_char == read_char) ?
-	    EL_BUILTIN_GETCFN : el->el_read.read_char;
-}
-
-
-#ifdef DEBUG_EDIT
-private void
-read_debug(EditLine *el)
-{
-
-	if (el->el_line.cursor > el->el_line.lastchar)
-		(void) fprintf(el->el_errfile, "cursor > lastchar\r\n");
-	if (el->el_line.cursor < el->el_line.buffer)
-		(void) fprintf(el->el_errfile, "cursor < buffer\r\n");
-	if (el->el_line.cursor > el->el_line.limit)
-		(void) fprintf(el->el_errfile, "cursor > limit\r\n");
-	if (el->el_line.lastchar > el->el_line.limit)
-		(void) fprintf(el->el_errfile, "lastchar > limit\r\n");
-	if (el->el_line.limit != &el->el_line.buffer[EL_BUFSIZ - 2])
-		(void) fprintf(el->el_errfile, "limit != &buffer[EL_BUFSIZ-2]\r\n");
-}
-#endif /* DEBUG_EDIT */
-
-
-/* read__fixio():
- *	Try to recover from a read error
- */
-/* ARGSUSED */
-private int
-read__fixio(int fd, int e)
-{
-
-	switch (e) {
-	case -1:		/* Make sure that the code is reachable */
-
-#ifdef EWOULDBLOCK
-	case EWOULDBLOCK:
-#ifndef TRY_AGAIN
-#define	TRY_AGAIN
-#endif
-#endif /* EWOULDBLOCK */
-
-#if defined(POSIX) && defined(EAGAIN)
-#if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
-	case EAGAIN:
-#ifndef TRY_AGAIN
-#define	TRY_AGAIN
-#endif
-#endif /* EWOULDBLOCK && EWOULDBLOCK != EAGAIN */
-#endif /* POSIX && EAGAIN */
-
-		e = 0;
-#ifdef TRY_AGAIN
-#if defined(F_SETFL) && defined(O_NDELAY)
-		if ((e = fcntl(fd, F_GETFL, 0)) == -1)
-			return (-1);
-
-		if (fcntl(fd, F_SETFL, e & ~O_NDELAY) == -1)
-			return (-1);
-		else
-			e = 1;
-#endif /* F_SETFL && O_NDELAY */
-
-#ifdef FIONBIO
-		{
-			int zero = 0;
-
-			if (ioctl(fd, FIONBIO, (ioctl_t) & zero) == -1)
-				return (-1);
-			else
-				e = 1;
-		}
-#endif /* FIONBIO */
-
-#endif /* TRY_AGAIN */
-		return (e ? 0 : -1);
-
-	case EINTR:
-		return (0);
-
-	default:
-		return (-1);
-	}
-}
-
-
-/* read_preread():
- *	Try to read the stuff in the input queue;
- */
-private int
-read_preread(EditLine *el)
-{
-	int chrs = 0;
-
-	if (el->el_chared.c_macro.nline) {
-		el_free((ptr_t) el->el_chared.c_macro.nline);
-		el->el_chared.c_macro.nline = NULL;
-	}
-	if (el->el_tty.t_mode == ED_IO)
-		return (0);
-
-#ifdef FIONREAD
-	(void) ioctl(el->el_infd, FIONREAD, (ioctl_t) & chrs);
-	if (chrs > 0) {
-		char buf[EL_BUFSIZ];
-
-		chrs = read(el->el_infd, buf,
-		    (size_t) MIN(chrs, EL_BUFSIZ - 1));
-		if (chrs > 0) {
-			buf[chrs] = '\0';
-			el->el_chared.c_macro.nline = strdup(buf);
-			el_push(el, el->el_chared.c_macro.nline);
-		}
-	}
-#endif /* FIONREAD */
-
-	return (chrs > 0);
-}
-
-
-/* el_push():
- *	Push a macro
- */
-public void
-el_push(EditLine *el, char *str)
-{
-	c_macro_t *ma = &el->el_chared.c_macro;
-
-	if (str != NULL && ma->level + 1 < EL_MAXMACRO) {
-		ma->level++;
-		ma->macro[ma->level] = str;
-	} else {
-		term_beep(el);
-		term__flush();
-	}
-}
-
-
-/* read_getcmd():
- *	Return next command from the input stream.
- */
-private int
-read_getcmd(EditLine *el, el_action_t *cmdnum, char *ch)
-{
-	el_action_t cmd = ED_UNASSIGNED;
-	int num;
-
-	while (cmd == ED_UNASSIGNED || cmd == ED_SEQUENCE_LEAD_IN) {
-		if ((num = el_getc(el, ch)) != 1)	/* if EOF or error */
-			return (num);
-
-#ifdef	KANJI
-		if ((*ch & 0200)) {
-			el->el_state.metanext = 0;
-			cmd = CcViMap[' '];
-			break;
-		} else
-#endif /* KANJI */
-
-		if (el->el_state.metanext) {
-			el->el_state.metanext = 0;
-			*ch |= 0200;
-		}
-		cmd = el->el_map.current[(unsigned char) *ch];
-		if (cmd == ED_SEQUENCE_LEAD_IN) {
-			key_value_t val;
-			switch (key_get(el, ch, &val)) {
-			case XK_CMD:
-				cmd = val.cmd;
-				break;
-			case XK_STR:
-				el_push(el, val.str);
-				break;
-#ifdef notyet
-			case XK_EXE:
-				/* XXX: In the future to run a user function */
-				RunCommand(val.str);
-				break;
-#endif
-			default:
-				EL_ABORT((el->el_errfile, "Bad XK_ type \n"));
-				break;
-			}
-		}
-		if (el->el_map.alt == NULL)
-			el->el_map.current = el->el_map.key;
-	}
-	*cmdnum = cmd;
-	return (OKCMD);
-}
-
-
-/* read_char():
- *	Read a character from the tty.
- *	XXX This routine is the default, but what you are actually looking for
- *	is in main/asterisk.c, in ast_el_read_char(). XXX
- */
-private int
-read_char(EditLine *el, char *cp)
-{
-	int num_read;
-	int tried = 0;
-
-	while ((num_read = read(el->el_infd, cp, 1)) == -1)
-		if (!tried && read__fixio(el->el_infd, errno) == 0)
-			tried = 1;
-		else {
-			*cp = '\0';
-			return (-1);
-		}
-
-	return (num_read);
-}
-
-
-/* el_getc():
- *	Read a character
- */
-public int
-el_getc(EditLine *el, char *cp)
-{
-	int num_read;
-	c_macro_t *ma = &el->el_chared.c_macro;
-
-	term__flush();
-	for (;;) {
-		if (ma->level < 0) {
-			if (!read_preread(el))
-				break;
-		}
-		if (ma->level < 0)
-			break;
-
-		if (*ma->macro[ma->level] == 0) {
-			ma->level--;
-			continue;
-		}
-		*cp = *ma->macro[ma->level]++ & 0377;
-		if (*ma->macro[ma->level] == 0) {	/* Needed for QuoteMode
-							 * On */
-			ma->level--;
-		}
-		return (1);
-	}
-
-#ifdef DEBUG_READ
-	(void) fprintf(el->el_errfile, "Turning raw mode on\n");
-#endif /* DEBUG_READ */
-	if (tty_rawmode(el) < 0)/* make sure the tty is set up correctly */
-		return (0);
-
-#ifdef DEBUG_READ
-	(void) fprintf(el->el_errfile, "Reading a character\n");
-#endif /* DEBUG_READ */
-	/* See main/asterisk.c: ast_el_read_char() */
-	num_read = (*el->el_read.read_char)(el, cp);
-#ifdef DEBUG_READ
-	(void) fprintf(el->el_errfile, "Got it %c\n", *cp);
-#endif /* DEBUG_READ */
-	return (num_read);
-}
-
-
-public const char *
-el_gets(EditLine *el, int *nread)
-{
-	int retval;
-	el_action_t cmdnum = 0;
-	int num;		/* how many chars we have read at NL */
-	char ch;
-#ifdef FIONREAD
-	c_macro_t *ma = &el->el_chared.c_macro;
-#endif /* FIONREAD */
-
-	if (el->el_flags & HANDLE_SIGNALS)
-		sig_set(el);
-
-	if (el->el_flags & NO_TTY) {
-		char *cp = el->el_line.buffer;
-		size_t idx;
-
-		while ((*el->el_read.read_char)(el, cp) == 1) {
-			/* make sure there is space for next character */
-			if (cp + 1 >= el->el_line.limit) {
-				idx = (cp - el->el_line.buffer);
-				if (!ch_enlargebufs(el, 2))
-					break;
-				cp = &el->el_line.buffer[idx];
-			}
-			cp++;
-			if (cp[-1] == '\r' || cp[-1] == '\n')
-				break;
-		}
-
-		el->el_line.cursor = el->el_line.lastchar = cp;
-		*cp = '\0';
-		if (nread)
-			*nread = el->el_line.cursor - el->el_line.buffer;
-		return (el->el_line.buffer);
-	}
-	re_clear_display(el);	/* reset the display stuff */
-	ch_reset(el);
-
-#ifdef FIONREAD
-	if (el->el_tty.t_mode == EX_IO && ma->level < 0) {
-		long chrs = 0;
-
-		(void) ioctl(el->el_infd, FIONREAD, (ioctl_t) & chrs);
-		if (chrs == 0) {
-			if (tty_rawmode(el) < 0) {
-				if (nread)
-					*nread = 0;
-				return (NULL);
-			}
-		}
-	}
-#endif /* FIONREAD */
-
-	re_refresh(el);		/* print the prompt */
-
-	if (el->el_flags & EDIT_DISABLED) {
-		char *cp = el->el_line.buffer;
-		size_t idx;
-
-		term__flush();
-
-		while ((*el->el_read.read_char)(el, cp) == 1) {
-			/* make sure there is space next character */
-			if (cp + 1 >= el->el_line.limit) {
-				idx = (cp - el->el_line.buffer);
-				if (!ch_enlargebufs(el, 2))
-					break;
-				cp = &el->el_line.buffer[idx];
-			}
-			cp++;
-			if (cp[-1] == '\r' || cp[-1] == '\n')
-				break;
-		}
-
-		el->el_line.cursor = el->el_line.lastchar = cp;
-		*cp = '\0';
-		if (nread)
-			*nread = el->el_line.cursor - el->el_line.buffer;
-		return (el->el_line.buffer);
-	}
-	for (num = OKCMD; num == OKCMD;) {	/* while still editing this
-						 * line */
-#ifdef DEBUG_EDIT
-		read_debug(el);
-#endif /* DEBUG_EDIT */
-		/* if EOF or error */
-		if ((num = read_getcmd(el, &cmdnum, &ch)) != OKCMD) {
-#ifdef DEBUG_READ
-			(void) fprintf(el->el_errfile,
-			    "Returning from el_gets %d\n", num);
-#endif /* DEBUG_READ */
-			break;
-		}
-		if ((int) cmdnum >= el->el_map.nfunc) {	/* BUG CHECK command */
-#ifdef DEBUG_EDIT
-			(void) fprintf(el->el_errfile,
-			    "ERROR: illegal command from key 0%o\r\n", ch);
-#endif /* DEBUG_EDIT */
-			continue;	/* try again */
-		}
-		/* now do the real command */
-#ifdef DEBUG_READ
-		{
-			el_bindings_t *b;
-			for (b = el->el_map.help; b->name; b++)
-				if (b->func == cmdnum)
-					break;
-			if (b->name)
-				(void) fprintf(el->el_errfile,
-				    "Executing %s\n", b->name);
-			else
-				(void) fprintf(el->el_errfile,
-				    "Error command = %d\n", cmdnum);
-		}
-#endif /* DEBUG_READ */
-		retval = (*el->el_map.func[cmdnum]) (el, ch);
-
-		/* save the last command here */
-		el->el_state.lastcmd = cmdnum;
-
-		/* use any return value */
-		switch (retval) {
-		case CC_CURSOR:
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			re_refresh_cursor(el);
-			break;
-
-		case CC_REDISPLAY:
-			re_clear_lines(el);
-			re_clear_display(el);
-			/* FALLTHROUGH */
-
-		case CC_REFRESH:
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			re_refresh(el);
-			break;
-
-		case CC_REFRESH_BEEP:
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			re_refresh(el);
-			term_beep(el);
-			break;
-
-		case CC_NORM:	/* normal char */
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			break;
-
-		case CC_ARGHACK:	/* Suggested by Rich Salz */
-			/* <rsalz@pineapple.bbn.com> */
-			break;	/* keep going... */
-
-		case CC_EOF:	/* end of file typed */
-			num = 0;
-			break;
-
-		case CC_NEWLINE:	/* normal end of line */
-			num = el->el_line.lastchar - el->el_line.buffer;
-			break;
-
-		case CC_FATAL:	/* fatal error, reset to known state */
-#ifdef DEBUG_READ
-			(void) fprintf(el->el_errfile,
-			    "*** editor fatal ERROR ***\r\n\n");
-#endif /* DEBUG_READ */
-			/* put (real) cursor in a known place */
-			re_clear_display(el);	/* reset the display stuff */
-			ch_reset(el);	/* reset the input pointers */
-			re_refresh(el);	/* print the prompt again */
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			break;
-
-		case CC_ERROR:
-		default:	/* functions we don't know about */
-#ifdef DEBUG_READ
-			(void) fprintf(el->el_errfile,
-			    "*** editor ERROR ***\r\n\n");
-#endif /* DEBUG_READ */
-			el->el_state.argument = 1;
-			el->el_state.doingarg = 0;
-			term_beep(el);
-			term__flush();
-			break;
-		}
-	}
-
-				/* make sure the tty is set up correctly */
-	(void) tty_cookedmode(el);
-	term__flush();		/* flush any buffered output */
-	if (el->el_flags & HANDLE_SIGNALS)
-		sig_clr(el);
-	if (nread)
-		*nread = num;
-	return (num ? el->el_line.buffer : NULL);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/read.h 1:16.1.1~dfsg-1/main/editline/read.h
--- 1:13.23.1~dfsg-2/main/editline/read.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/read.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,55 +0,0 @@
-/*	$NetBSD: read.h,v 1.1 2001/09/27 19:29:50 christos Exp $	*/
-
-/*-
- * Copyright (c) 2001 The NetBSD Foundation, Inc.
- * All rights reserved.
- *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Anthony Mallet.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *        This product includes software developed by the NetBSD
- *        Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
-/*
- * el.read.h: Character reading functions
- */
-#ifndef	_h_el_read
-#define	_h_el_read
-
-typedef int (*el_rfunc_t)(EditLine *, char *);
-
-typedef struct el_read_t {
-	el_rfunc_t	read_char;	/* Function to read a character */
-} el_read_t;
-
-protected int		read_init(EditLine *);
-protected int		el_read_setfn(EditLine *, el_rfunc_t);
-protected el_rfunc_t	el_read_getfn(EditLine *);
-
-#endif /* _h_el_read */
diff -pruN 1:13.23.1~dfsg-2/main/editline/readline.c 1:16.1.1~dfsg-1/main/editline/readline.c
--- 1:13.23.1~dfsg-2/main/editline/readline.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/readline.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,1675 +0,0 @@
-/*	$NetBSD: readline.c,v 1.21 2002/03/18 16:20:36 christos Exp $	*/
-
-/*-
- * Copyright (c) 1997 The NetBSD Foundation, Inc.
- * All rights reserved.
- *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Jaromir Dolecek.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the NetBSD
- *	Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-__RCSID("$NetBSD: readline.c,v 1.21 2002/03/18 16:20:36 christos Exp $");
-#endif /* not lint && not SCCSID */
-
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <stdio.h>
-#include <dirent.h>
-#include <string.h>
-#include <pwd.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <limits.h>
-
-#ifdef SOLARIS
-#include <alloca.h>
-#endif
-
-#include "histedit.h"
-#include "readline.h"
-#include "el.h"
-#include "fcns.h"		/* for EL_NUM_FCNS */
-
-/* for rl_complete() */
-#define	TAB		'\r'
-
-/* see comment at the #ifdef for sense of this */
-#define	GDB_411_HACK
-
-/* readline compatibility stuff - look at readline sources/documentation */
-/* to see what these variables mean */
-const char *rl_library_version = "EditLine wrapper";
-static char empty[] = { '\0' };
-static char expand_chars[] = { ' ', '\t', '\n', '=', '(', '\0' };
-static char break_chars[] = { ' ', '\t', '\n', '"', '\\', '\'', '`', '@', '$',
-    '>', '<', '=', ';', '|', '&', '{', '(', '\0' };
-char *rl_readline_name = empty;
-FILE *rl_instream = NULL;
-FILE *rl_outstream = NULL;
-int rl_point = 0;
-int rl_end = 0;
-char *rl_line_buffer = NULL;
-
-int history_base = 1;		/* probably never subject to change */
-int history_length = 0;
-int max_input_history = 0;
-char history_expansion_char = '!';
-char history_subst_char = '^';
-char *history_no_expand_chars = expand_chars;
-Function *history_inhibit_expansion_function = NULL;
-
-int rl_inhibit_completion = 0;
-int rl_attempted_completion_over = 0;
-char *rl_basic_word_break_characters = break_chars;
-char *rl_completer_word_break_characters = NULL;
-char *rl_completer_quote_characters = NULL;
-CPFunction *rl_completion_entry_function = NULL;
-CPPFunction *rl_attempted_completion_function = NULL;
-
-/*
- * This is set to character indicating type of completion being done by
- * rl_complete_internal(); this is available for application completion
- * functions.
- */
-int rl_completion_type = 0;
-
-/*
- * If more than this number of items results from query for possible
- * completions, we ask user if they are sure to really display the list.
- */
-int rl_completion_query_items = 100;
-
-/*
- * List of characters which are word break characters, but should be left
- * in the parsed text when it is passed to the completion function.
- * Shell uses this to help determine what kind of completing to do.
- */
-char *rl_special_prefixes = (char *)NULL;
-
-/*
- * This is the character appended to the completed words if at the end of
- * the line. Default is ' ' (a space).
- */
-int rl_completion_append_character = ' ';
-
-/* stuff below is used internally by libedit for readline emulation */
-
-/* if not zero, non-unique completions always show list of possible matches */
-static int _rl_complete_show_all = 0;
-
-static History *h = NULL;
-static EditLine *e = NULL;
-static int el_rl_complete_cmdnum = 0;
-
-/* internal functions */
-static unsigned char	 _el_rl_complete(EditLine *, int);
-static char		*_get_prompt(EditLine *);
-static HIST_ENTRY	*_move_history(int);
-static int		 _history_search_gen(const char *, int, int);
-static int		 _history_expand_command(const char *, size_t, char **);
-static char		*_rl_compat_sub(const char *, const char *,
-			    const char *, int);
-static int		 rl_complete_internal(int);
-static int		 _rl_qsort_string_compare(const void *, const void *);
-
-/*
- * needed for prompt switching in readline()
- */
-static char *el_rl_prompt = NULL;
-
-
-/* ARGSUSED */
-static char *
-_get_prompt(EditLine *el)
-{
-	return (el_rl_prompt);
-}
-
-
-/*
- * generic function for moving around history
- */
-static HIST_ENTRY *
-_move_history(int op)
-{
-	HistEvent ev;
-	static HIST_ENTRY rl_he;
-
-	if (history(h, &ev, op) != 0)
-		return (HIST_ENTRY *) NULL;
-
-	rl_he.line = ev.str;
-	rl_he.data = "";
-
-	return (&rl_he);
-}
-
-
-/*
- * READLINE compatibility stuff
- */
-
-/*
- * initialize rl compat stuff
- */
-int
-rl_initialize(void)
-{
-	HistEvent ev;
-	const LineInfo *li;
-	int i;
-	int editmode = 1;
-	struct termios t;
-
-	if (e != NULL)
-		el_end(e);
-	if (h != NULL)
-		history_end(h);
-
-	if (!rl_instream)
-		rl_instream = stdin;
-	if (!rl_outstream)
-		rl_outstream = stdout;
-
-	/*
-	 * See if we don't really want to run the editor
-	 */
-	if (tcgetattr(fileno(rl_instream), &t) != -1 && (t.c_lflag & ECHO) == 0)
-		editmode = 0;
-
-	e = el_init(rl_readline_name, rl_instream, rl_outstream, stderr);
-
-	if (!editmode)
-		el_set(e, EL_EDITMODE, 0);
-
-	h = history_init();
-	if (!e || !h)
-		return (-1);
-
-	history(h, &ev, H_SETSIZE, INT_MAX);	/* unlimited */
-	history_length = 0;
-	max_input_history = INT_MAX;
-	el_set(e, EL_HIST, history, h);
-
-	/* for proper prompt printing in readline() */
-	el_rl_prompt = strdup("");
-	el_set(e, EL_PROMPT, _get_prompt);
-	el_set(e, EL_SIGNAL, 1);
-
-	/* set default mode to "emacs"-style and read setting afterwards */
-	/* so this can be overriden */
-	el_set(e, EL_EDITOR, "emacs");
-
-	/*
-	 * Word completition - this has to go AFTER rebinding keys
-	 * to emacs-style.
-	 */
-	el_set(e, EL_ADDFN, "rl_complete",
-	    "ReadLine compatible completition function",
-	    _el_rl_complete);
-	el_set(e, EL_BIND, "^I", "rl_complete", NULL);
-
-	/*
-	 * Find out where the rl_complete function was added; this is
-	 * used later to detect that lastcmd was also rl_complete.
-	 */
-	for(i=EL_NUM_FCNS; i < e->el_map.nfunc; i++) {
-		if (e->el_map.func[i] == _el_rl_complete) {
-			el_rl_complete_cmdnum = i;
-			break;
-		}
-	}
-
-	/* read settings from configuration file */
-	el_source(e, NULL);
-
-	/*
-	 * Unfortunately, some applications really do use rl_point
-	 * and rl_line_buffer directly.
-	 */
-	li = el_line(e);
-	/* a cheesy way to get rid of const cast. */
-	rl_line_buffer = memchr(li->buffer, *li->buffer, 1);
-	rl_point = rl_end = 0;
-
-	return (0);
-}
-
-
-/*
- * read one line from input stream and return it, chomping
- * trailing newline (if there is any)
- */
-char *
-readline(const char *prompt)
-{
-	HistEvent ev;
-	int count;
-	const char *ret;
-	char *buf;
-
-	if (e == NULL || h == NULL)
-		rl_initialize();
-
-	/* update prompt accordingly to what has been passed */
-	if (!prompt)
-		prompt = "";
-	if (strcmp(el_rl_prompt, prompt) != 0) {
-		free(el_rl_prompt);
-		el_rl_prompt = strdup(prompt);
-	}
-	/* get one line from input stream */
-	ret = el_gets(e, &count);
-
-	if (ret && count > 0) {
-		int lastidx;
-
-		buf = strdup(ret);
-		lastidx = count - 1;
-		if (buf[lastidx] == '\n')
-			buf[lastidx] = '\0';
-	} else
-		buf = NULL;
-
-	history(h, &ev, H_GETSIZE);
-	history_length = ev.num;
-
-	return buf;
-}
-
-/*
- * history functions
- */
-
-/*
- * is normally called before application starts to use
- * history expansion functions
- */
-void
-using_history(void)
-{
-	if (h == NULL || e == NULL)
-		rl_initialize();
-}
-
-
-/*
- * substitute ``what'' with ``with'', returning resulting string; if
- * globally == 1, substitutes all occurences of what, otherwise only the
- * first one
- */
-static char *
-_rl_compat_sub(const char *str, const char *what, const char *with,
-    int globally)
-{
-	char *result;
-	const char *temp, *new;
-	int len, with_len, what_len, add;
-	size_t size, i;
-
-	result = malloc((size = 16));
-	temp = str;
-	with_len = strlen(with);
-	what_len = strlen(what);
-	len = 0;
-	do {
-		new = strstr(temp, what);
-		if (new) {
-			i = new - temp;
-			add = i + with_len;
-			if (i + add + 1 >= size) {
-				size += add + 1;
-				result = realloc(result, size);
-			}
-			(void) strncpy(&result[len], temp, i);
-			len += i;
-			(void) strcpy(&result[len], with);	/* safe */
-			len += with_len;
-			temp = new + what_len;
-		} else {
-			add = strlen(temp);
-			if (len + add + 1 >= size) {
-				size += add + 1;
-				result = realloc(result, size);
-			}
-			(void) strcpy(&result[len], temp);	/* safe */
-			len += add;
-			temp = NULL;
-		}
-	} while (temp && globally);
-	result[len] = '\0';
-
-	return (result);
-}
-
-
-/*
- * the real function doing history expansion - takes as argument command
- * to do and data upon which the command should be executed
- * does expansion the way I've understood readline documentation
- * word designator ``%'' isn't supported (yet ?)
- *
- * returns 0 if data was not modified, 1 if it was and 2 if the string
- * should be only printed and not executed; in case of error,
- * returns -1 and *result points to NULL
- * it's callers responsibility to free() string returned in *result
- */
-static int
-_history_expand_command(const char *command, size_t cmdlen, char **result)
-{
-	char **arr, *tempcmd, *line, *search = NULL, *cmd;
-	const char *event_data = NULL;
-	static char *from = NULL, *to = NULL;
-	int start = -1, end = -1, max, i, idx;
-	int h_on = 0, t_on = 0, r_on = 0, e_on = 0, p_on = 0, g_on = 0;
-	int event_num = 0, retval;
-	size_t cmdsize;
-
-	*result = NULL;
-
-	cmd = alloca(cmdlen + 1);
-	(void) strncpy(cmd, command, cmdlen);
-	cmd[cmdlen] = 0;
-
-	idx = 1;
-	/* find out which event to take */
-	if (cmd[idx] == history_expansion_char) {
-		event_num = history_length;
-		idx++;
-	} else {
-		int off, num;
-		size_t len;
-		off = idx;
-		while (cmd[off] && !strchr(":^$*-%", cmd[off]))
-			off++;
-		num = atoi(&cmd[idx]);
-		if (num != 0) {
-			event_num = num;
-			if (num < 0)
-				event_num += history_length + 1;
-		} else {
-			int prefix = 1, curr_num;
-			HistEvent ev;
-
-			len = off - idx;
-			if (cmd[idx] == '?') {
-				idx++, len--;
-				if (cmd[off - 1] == '?')
-					len--;
-				else if (cmd[off] != '\n' && cmd[off] != '\0')
-					return (-1);
-				prefix = 0;
-			}
-			search = alloca(len + 1);
-			(void) strncpy(search, &cmd[idx], len);
-			search[len] = '\0';
-
-			if (history(h, &ev, H_CURR) != 0)
-				return (-1);
-			curr_num = ev.num;
-
-			if (prefix)
-				retval = history_search_prefix(search, -1);
-			else
-				retval = history_search(search, -1);
-
-			if (retval == -1) {
-				fprintf(rl_outstream, "%s: Event not found\n",
-				    search);
-				return (-1);
-			}
-			if (history(h, &ev, H_CURR) != 0)
-				return (-1);
-			event_data = ev.str;
-
-			/* roll back to original position */
-			history(h, &ev, H_NEXT_EVENT, curr_num);
-		}
-		idx = off;
-	}
-
-	if (!event_data && event_num >= 0) {
-		HIST_ENTRY *rl_he;
-		rl_he = history_get(event_num);
-		if (!rl_he)
-			return (0);
-		event_data = rl_he->line;
-	} else
-		return (-1);
-
-	if (cmd[idx] != ':')
-		return (-1);
-	cmd += idx + 1;
-
-	/* recognize cmd */
-	if (*cmd == '^')
-		start = end = 1, cmd++;
-	else if (*cmd == '$')
-		start = end = -1, cmd++;
-	else if (*cmd == '*')
-		start = 1, end = -1, cmd++;
-	else if (isdigit((unsigned char) *cmd)) {
-		const char *temp;
-		int shifted = 0;
-
-		start = atoi(cmd);
-		temp = cmd;
-		for (; isdigit((unsigned char) *cmd); cmd++);
-		if (temp != cmd)
-			shifted = 1;
-		if (shifted && *cmd == '-') {
-			if (!isdigit((unsigned char) *(cmd + 1)))
-				end = -2;
-			else {
-				end = atoi(cmd + 1);
-				for (; isdigit((unsigned char) *cmd); cmd++);
-			}
-		} else if (shifted && *cmd == '*')
-			end = -1, cmd++;
-		else if (shifted)
-			end = start;
-	}
-	if (*cmd == ':')
-		cmd++;
-
-	line = strdup(event_data);
-	for (; *cmd; cmd++) {
-		if (*cmd == ':')
-			continue;
-		else if (*cmd == 'h')
-			h_on = 1 | g_on, g_on = 0;
-		else if (*cmd == 't')
-			t_on = 1 | g_on, g_on = 0;
-		else if (*cmd == 'r')
-			r_on = 1 | g_on, g_on = 0;
-		else if (*cmd == 'e')
-			e_on = 1 | g_on, g_on = 0;
-		else if (*cmd == 'p')
-			p_on = 1 | g_on, g_on = 0;
-		else if (*cmd == 'g')
-			g_on = 2;
-		else if (*cmd == 's' || *cmd == '&') {
-			char *what, *with, delim;
-			int len, from_len;
-			size_t size;
-
-			if (*cmd == '&' && (from == NULL || to == NULL))
-				continue;
-			else if (*cmd == 's') {
-				delim = *(++cmd), cmd++;
-				size = 16;
-				what = realloc(from, size);
-				len = 0;
-				for (; *cmd && *cmd != delim; cmd++) {
-					if (*cmd == '\\'
-					    && *(cmd + 1) == delim)
-						cmd++;
-					if (len >= size)
-						what = realloc(what,
-						    (size <<= 1));
-					what[len++] = *cmd;
-				}
-				what[len] = '\0';
-				from = what;
-				if (*what == '\0') {
-					free(what);
-					if (search)
-						from = strdup(search);
-					else {
-						from = NULL;
-						free(line);
-						return (-1);
-					}
-				}
-				cmd++;	/* shift after delim */
-				if (!*cmd)
-					continue;
-
-				size = 16;
-				with = realloc(to, size);
-				len = 0;
-				from_len = strlen(from);
-				for (; *cmd && *cmd != delim; cmd++) {
-					if (len + from_len + 1 >= size) {
-						size += from_len + 1;
-						with = realloc(with, size);
-					}
-					if (*cmd == '&') {
-						/* safe */
-						(void) strcpy(&with[len], from);
-						len += from_len;
-						continue;
-					}
-					if (*cmd == '\\'
-					    && (*(cmd + 1) == delim
-						|| *(cmd + 1) == '&'))
-						cmd++;
-					with[len++] = *cmd;
-				}
-				with[len] = '\0';
-				to = with;
-
-				tempcmd = _rl_compat_sub(line, from, to,
-				    (g_on) ? 1 : 0);
-				free(line);
-				line = tempcmd;
-				g_on = 0;
-			}
-		}
-	}
-
-	arr = history_tokenize(line);
-	free(line);		/* no more needed */
-	if (arr && *arr == NULL)
-		free(arr), arr = NULL;
-	if (!arr)
-		return (-1);
-
-	/* find out max valid idx to array of array */
-	max = 0;
-	for (i = 0; arr[i]; i++)
-		max++;
-	max--;
-
-	/* set boundaries to something relevant */
-	if (start < 0)
-		start = 1;
-	if (end < 0)
-		end = max - ((end < -1) ? 1 : 0);
-
-	/* check boundaries ... */
-	if (start > max || end > max || start > end) {
-		for (i = 0; i <= max; i++) {
-			free(arr[i]);
-		}
-		free(arr), arr = (char **) NULL;
-		return (-1);
-	}
-
-	for (i = 0; i <= max; i++) {
-		char *temp;
-		if (h_on && (i == 1 || h_on > 1) &&
-		    (temp = strrchr(arr[i], '/')))
-			*(temp + 1) = '\0';
-		if (t_on && (i == 1 || t_on > 1) &&
-		    (temp = strrchr(arr[i], '/')))
-			(void) strcpy(arr[i], temp + 1);
-		if (r_on && (i == 1 || r_on > 1) &&
-		    (temp = strrchr(arr[i], '.')))
-			*temp = '\0';
-		if (e_on && (i == 1 || e_on > 1) &&
-		    (temp = strrchr(arr[i], '.')))
-			(void) strcpy(arr[i], temp);
-	}
-
-	cmdsize = 1, cmdlen = 0;
-	tempcmd = malloc(cmdsize);
-	for (i = start; start <= i && i <= end; i++) {
-		int arr_len;
-
-		arr_len = strlen(arr[i]);
-		if (cmdlen + arr_len + 1 >= cmdsize) {
-			cmdsize += arr_len + 1;
-			tempcmd = realloc(tempcmd, cmdsize);
-		}
-		(void) strcpy(&tempcmd[cmdlen], arr[i]);	/* safe */
-		cmdlen += arr_len;
-		tempcmd[cmdlen++] = ' ';	/* add a space */
-	}
-	while (cmdlen > 0 && isspace((unsigned char) tempcmd[cmdlen - 1]))
-		cmdlen--;
-	tempcmd[cmdlen] = '\0';
-
-	*result = tempcmd;
-
-	for (i = 0; i <= max; i++)
-		free(arr[i]);
-	free(arr), arr = (char **) NULL;
-	return (p_on) ? 2 : 1;
-}
-
-
-/*
- * csh-style history expansion
- */
-int
-history_expand(char *str, char **output)
-{
-	int i, retval = 0, idx;
-	size_t size;
-	char *temp, *result;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	*output = strdup(str);	/* do it early */
-
-	if (str[0] == history_subst_char) {
-		/* ^foo^foo2^ is equivalent to !!:s^foo^foo2^ */
-		temp = alloca(4 + strlen(str) + 1);
-		temp[0] = temp[1] = history_expansion_char;
-		temp[2] = ':';
-		temp[3] = 's';
-		(void) strcpy(temp + 4, str);
-		str = temp;
-	}
-#define	ADD_STRING(what, len) 						\
-	{								\
-		if (idx + len + 1 > size)				\
-			result = realloc(result, (size += len + 1));	\
-		(void)strncpy(&result[idx], what, len);			\
-		idx += len;						\
-		result[idx] = '\0';					\
-	}
-
-	result = NULL;
-	size = idx = 0;
-	for (i = 0; str[i];) {
-		int start, j, loop_again;
-		size_t len;
-
-		loop_again = 1;
-		start = j = i;
-loop:
-		for (; str[j]; j++) {
-			if (str[j] == '\\' &&
-			    str[j + 1] == history_expansion_char) {
-				(void) strcpy(&str[j], &str[j + 1]);
-				continue;
-			}
-			if (!loop_again) {
-				if (str[j] == '?') {
-					while (str[j] && str[++j] != '?');
-					if (str[j] == '?')
-						j++;
-				} else if (isspace((unsigned char) str[j]))
-					break;
-			}
-			if (str[j] == history_expansion_char
-			    && !strchr(history_no_expand_chars, str[j + 1])
-			    && (!history_inhibit_expansion_function ||
-			    (*history_inhibit_expansion_function)(str, j) == 0))
-				break;
-		}
-
-		if (str[j] && str[j + 1] != '#' && loop_again) {
-			i = j;
-			j++;
-			if (str[j] == history_expansion_char)
-				j++;
-			loop_again = 0;
-			goto loop;
-		}
-		len = i - start;
-		temp = &str[start];
-		ADD_STRING(temp, len);
-
-		if (str[i] == '\0' || str[i] != history_expansion_char
-		    || str[i + 1] == '#') {
-			len = j - i;
-			temp = &str[i];
-			ADD_STRING(temp, len);
-			if (start == 0)
-				retval = 0;
-			else
-				retval = 1;
-			break;
-		}
-		retval = _history_expand_command(&str[i], (size_t) (j - i),
-		    &temp);
-		if (retval != -1) {
-			len = strlen(temp);
-			ADD_STRING(temp, len);
-		}
-		i = j;
-	}			/* for(i ...) */
-
-	if (retval == 2) {
-		add_history(temp);
-#ifdef GDB_411_HACK
-		/* gdb 4.11 has been shipped with readline, where */
-		/* history_expand() returned -1 when the line	  */
-		/* should not be executed; in readline 2.1+	  */
-		/* it should return 2 in such a case		  */
-		retval = -1;
-#endif
-	}
-	free(*output);
-	*output = result;
-
-	return (retval);
-}
-
-
-/*
- * Parse the string into individual tokens, similarily to how shell would do it.
- */
-char **
-history_tokenize(const char *str)
-{
-	int size = 1, result_idx = 0, i, start;
-	size_t len;
-	char **result = NULL, *temp, delim = '\0';
-
-	for (i = 0; str[i]; i++) {
-		while (isspace((unsigned char) str[i]))
-			i++;
-		start = i;
-		for (; str[i]; i++) {
-			if (str[i] == '\\') {
-				if (str[i+1] != '\0')
-					i++;
-			} else if (str[i] == delim)
-				delim = '\0';
-			else if (!delim &&
-				    (isspace((unsigned char) str[i]) ||
-				strchr("()<>;&|$", str[i])))
-				break;
-			else if (!delim && strchr("'`\"", str[i]))
-				delim = str[i];
-		}
-
-		if (result_idx + 2 >= size) {
-			size <<= 1;
-			result = realloc(result, size * sizeof(char *));
-		}
-		len = i - start;
-		temp = malloc(len + 1);
-		(void) strncpy(temp, &str[start], len);
-		temp[len] = '\0';
-		result[result_idx++] = temp;
-		result[result_idx] = NULL;
-	}
-
-	return (result);
-}
-
-
-/*
- * limit size of history record to ``max'' events
- */
-void
-stifle_history(int max)
-{
-	HistEvent ev;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	if (history(h, &ev, H_SETSIZE, max) == 0)
-		max_input_history = max;
-}
-
-
-/*
- * "unlimit" size of history - set the limit to maximum allowed int value
- */
-int
-unstifle_history(void)
-{
-	HistEvent ev;
-	int omax;
-
-	history(h, &ev, H_SETSIZE, INT_MAX);
-	omax = max_input_history;
-	max_input_history = INT_MAX;
-	return (omax);		/* some value _must_ be returned */
-}
-
-
-int
-history_is_stifled(void)
-{
-
-	/* cannot return true answer */
-	return (max_input_history != INT_MAX);
-}
-
-
-/*
- * read history from a file given
- */
-int
-read_history(const char *filename)
-{
-	HistEvent ev;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-	return (history(h, &ev, H_LOAD, filename));
-}
-
-
-/*
- * write history to a file given
- */
-int
-write_history(const char *filename)
-{
-	HistEvent ev;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-	return (history(h, &ev, H_SAVE, filename));
-}
-
-
-/*
- * returns history ``num''th event
- *
- * returned pointer points to static variable
- */
-HIST_ENTRY *
-history_get(int num)
-{
-	static HIST_ENTRY she;
-	HistEvent ev;
-	int i = 1, curr_num;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	/* rewind to beginning */
-	if (history(h, &ev, H_CURR) != 0)
-		return (NULL);
-	curr_num = ev.num;
-	if (history(h, &ev, H_LAST) != 0)
-		return (NULL);	/* error */
-	while (i < num && history(h, &ev, H_PREV) == 0)
-		i++;
-	if (i != num)
-		return (NULL);	/* not so many entries */
-
-	she.line = ev.str;
-	she.data = NULL;
-
-	/* rewind history to the same event it was before */
-	(void) history(h, &ev, H_FIRST);
-	(void) history(h, &ev, H_NEXT_EVENT, curr_num);
-
-	return (&she);
-}
-
-
-/*
- * add the line to history table
- */
-int
-add_history(const char *line)
-{
-	HistEvent ev;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	(void) history(h, &ev, H_ENTER, line);
-	if (history(h, &ev, H_GETSIZE) == 0)
-		history_length = ev.num;
-
-	return (!(history_length > 0));	/* return 0 if all is okay */
-}
-
-
-/*
- * clear the history list - delete all entries
- */
-void
-clear_history(void)
-{
-	HistEvent ev;
-
-	history(h, &ev, H_CLEAR);
-}
-
-
-/*
- * returns offset of the current history event
- */
-int
-where_history(void)
-{
-	HistEvent ev;
-	int curr_num, off;
-
-	if (history(h, &ev, H_CURR) != 0)
-		return (0);
-	curr_num = ev.num;
-
-	history(h, &ev, H_FIRST);
-	off = 1;
-	while (ev.num != curr_num && history(h, &ev, H_NEXT) == 0)
-		off++;
-
-	return (off);
-}
-
-
-/*
- * returns current history event or NULL if there is no such event
- */
-HIST_ENTRY *
-current_history(void)
-{
-
-	return (_move_history(H_CURR));
-}
-
-
-/*
- * returns total number of bytes history events' data are using
- */
-int
-history_total_bytes(void)
-{
-	HistEvent ev;
-	int curr_num, size;
-
-	if (history(h, &ev, H_CURR) != 0)
-		return (-1);
-	curr_num = ev.num;
-
-	history(h, &ev, H_FIRST);
-	size = 0;
-	do
-		size += strlen(ev.str);
-	while (history(h, &ev, H_NEXT) == 0);
-
-	/* get to the same position as before */
-	history(h, &ev, H_PREV_EVENT, curr_num);
-
-	return (size);
-}
-
-
-/*
- * sets the position in the history list to ``pos''
- */
-int
-history_set_pos(int pos)
-{
-	HistEvent ev;
-	int off, curr_num;
-
-	if (pos > history_length || pos < 0)
-		return (-1);
-
-	history(h, &ev, H_CURR);
-	curr_num = ev.num;
-	history(h, &ev, H_FIRST);
-	off = 0;
-	while (off < pos && history(h, &ev, H_NEXT) == 0)
-		off++;
-
-	if (off != pos) {	/* do a rollback in case of error */
-		history(h, &ev, H_FIRST);
-		history(h, &ev, H_NEXT_EVENT, curr_num);
-		return (-1);
-	}
-	return (0);
-}
-
-
-/*
- * returns previous event in history and shifts pointer accordingly
- */
-HIST_ENTRY *
-previous_history(void)
-{
-
-	return (_move_history(H_PREV));
-}
-
-
-/*
- * returns next event in history and shifts pointer accordingly
- */
-HIST_ENTRY *
-next_history(void)
-{
-
-	return (_move_history(H_NEXT));
-}
-
-
-/*
- * generic history search function
- */
-static int
-_history_search_gen(const char *str, int direction, int pos)
-{
-	HistEvent ev;
-	const char *strp;
-	int curr_num;
-
-	if (history(h, &ev, H_CURR) != 0)
-		return (-1);
-	curr_num = ev.num;
-
-	for (;;) {
-		strp = strstr(ev.str, str);
-		if (strp && (pos < 0 || &ev.str[pos] == strp))
-			return (int) (strp - ev.str);
-		if (history(h, &ev, direction < 0 ? H_PREV : H_NEXT) != 0)
-			break;
-	}
-
-	history(h, &ev, direction < 0 ? H_NEXT_EVENT : H_PREV_EVENT, curr_num);
-
-	return (-1);
-}
-
-
-/*
- * searches for first history event containing the str
- */
-int
-history_search(const char *str, int direction)
-{
-
-	return (_history_search_gen(str, direction, -1));
-}
-
-
-/*
- * searches for first history event beginning with str
- */
-int
-history_search_prefix(const char *str, int direction)
-{
-
-	return (_history_search_gen(str, direction, 0));
-}
-
-
-/*
- * search for event in history containing str, starting at offset
- * abs(pos); continue backward, if pos<0, forward otherwise
- */
-/* ARGSUSED */
-int
-history_search_pos(const char *str, int direction, int pos)
-{
-	HistEvent ev;
-	int curr_num, off;
-
-	off = (pos > 0) ? pos : -pos;
-	pos = (pos > 0) ? 1 : -1;
-
-	if (history(h, &ev, H_CURR) != 0)
-		return (-1);
-	curr_num = ev.num;
-
-	if (history_set_pos(off) != 0 || history(h, &ev, H_CURR) != 0)
-		return (-1);
-
-
-	for (;;) {
-		if (strstr(ev.str, str))
-			return (off);
-		if (history(h, &ev, (pos < 0) ? H_PREV : H_NEXT) != 0)
-			break;
-	}
-
-	/* set "current" pointer back to previous state */
-	history(h, &ev, (pos < 0) ? H_NEXT_EVENT : H_PREV_EVENT, curr_num);
-
-	return (-1);
-}
-
-
-/********************************/
-/* completition functions	*/
-
-/*
- * does tilde expansion of strings of type ``~user/foo''
- * if ``user'' isn't valid user name or ``txt'' doesn't start
- * w/ '~', returns pointer to strdup()ed copy of ``txt''
- *
- * it's callers's responsibility to free() returned string
- */
-char *
-tilde_expand(char *txt)
-{
-	struct passwd *pass;
-	char *temp;
-	size_t len = 0;
-
-	if (txt[0] != '~')
-		return (strdup(txt));
-
-	temp = strchr(txt + 1, '/');
-	if (temp == NULL)
-		temp = strdup(txt + 1);
-	else {
-		len = temp - txt + 1;	/* text until string after slash */
-		temp = malloc(len);
-		(void) strncpy(temp, txt + 1, len - 2);
-		temp[len - 2] = '\0';
-	}
-	pass = getpwnam(temp);
-	free(temp);		/* value no more needed */
-	if (pass == NULL)
-		return (strdup(txt));
-
-	/* update pointer txt to point at string immedially following */
-	/* first slash */
-	txt += len;
-
-	temp = malloc(strlen(pass->pw_dir) + 1 + strlen(txt) + 1);
-	(void) sprintf(temp, "%s/%s", pass->pw_dir, txt);
-
-	return (temp);
-}
-
-
-/*
- * return first found file name starting by the ``text'' or NULL if no
- * such file can be found.
- * The first ``state'' matches are ignored.
- *
- * it's caller's responsibility to free returned string
- */
-char *
-filename_completion_function(const char *text, int state)
-{
-	DIR *dir = NULL;
-	char *filename = NULL, *dirname = NULL;
-	size_t filename_len = 0;
-	struct dirent *entry;
-	char *temp;
-	size_t len;
-
-	temp = strrchr(text, '/');
-	if (temp) {
-		temp++;
-		filename = realloc(filename, strlen(temp) + 1);
-		(void) strcpy(filename, temp);
-		len = temp - text;	/* including last slash */
-		dirname = realloc(dirname, len + 1);
-		(void) strncpy(dirname, text, len);
-		dirname[len] = '\0';
-	} else {
-		filename = strdup(text);
-		dirname = NULL;
-	}
-
-	/* support for ``~user'' syntax */
-	if (dirname && *dirname == '~') {
-		temp = tilde_expand(dirname);
-		dirname = realloc(dirname, strlen(temp) + 1);
-		(void) strcpy(dirname, temp);	/* safe */
-		free(temp);	/* no longer needed */
-	}
-	/* will be used in cycle */
-	filename_len = strlen(filename);
-
-	dir = opendir(dirname ? dirname : ".");
-	if (!dir) {
-		free(filename);
-		free(dirname);
-		return (NULL);	/* cannot open the directory */
-	}
-
-	/* find the match */
-	while ((entry = readdir(dir)) != NULL) {
-		/* otherwise, get first entry where first */
-		/* filename_len characters are equal	  */
-		if (
-#if defined(__SVR4) || defined(__linux__)
-		    strlen(entry->d_name) >= filename_len
-#else
-		    entry->d_namlen >= filename_len
-#endif
-		    && strncmp(entry->d_name, filename,
-			filename_len) == 0
-			&& (state-- == 0))
-			break;
-	}
-
-	if (entry) {		/* match found */
-
-		struct stat stbuf;
-#if defined(__SVR4) || defined(__linux__)
-		len = strlen(entry->d_name) +
-#else
-		len = entry->d_namlen +
-#endif
-		    ((dirname) ? strlen(dirname) : 0) + 1 + 1;
-		temp = malloc(len);
-		(void) sprintf(temp, "%s%s",
-		    dirname ? dirname : "", entry->d_name);	/* safe */
-
-		/* test, if it's directory */
-		if (stat(temp, &stbuf) == 0 && S_ISDIR(stbuf.st_mode))
-			strcat(temp, "/");	/* safe */
-	} else
-		temp = NULL;
-	closedir(dir);
-	free(filename);
-	free(dirname);
-
-	return (temp);
-}
-
-
-/*
- * a completion generator for usernames; returns _first_ username
- * which starts with supplied text
- * text contains a partial username preceded by random character
- * (usually '~'); state is ignored
- * it's callers responsibility to free returned value
- */
-char *
-username_completion_function(const char *text, int state)
-{
-	struct passwd *pwd;
-
-	if (text[0] == '\0')
-		return (NULL);
-
-	if (*text == '~')
-		text++;
-
-	if (state == 0)
-		setpwent();
-
-	while ((pwd = getpwent()) && text[0] == pwd->pw_name[0]
-	    && strcmp(text, pwd->pw_name) == 0);
-
-	if (pwd == NULL) {
-		endpwent();
-		return (NULL);
-	}
-	return (strdup(pwd->pw_name));
-}
-
-
-/*
- * el-compatible wrapper around rl_complete; needed for key binding
- */
-/* ARGSUSED */
-static unsigned char
-_el_rl_complete(EditLine *el, int ch)
-{
-	return (unsigned char) rl_complete(0, ch);
-}
-
-
-/*
- * returns list of completitions for text given
- */
-char **
-completion_matches(const char *text, CPFunction *genfunc)
-{
-	char **match_list = NULL, *retstr, *prevstr;
-	size_t match_list_len, max_equal, which, i;
-	int matches;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	matches = 0;
-	match_list_len = 1;
-	while ((retstr = (*genfunc) (text, matches)) != NULL) {
-		if (matches + 1 >= match_list_len) {
-			match_list_len <<= 1;
-			match_list = realloc(match_list,
-			    match_list_len * sizeof(char *));
-		}
-		match_list[++matches] = retstr;
-	}
-
-	if (!match_list)
-		return (char **) NULL;	/* nothing found */
-
-	/* find least denominator and insert it to match_list[0] */
-	which = 2;
-	prevstr = match_list[1];
-	max_equal = strlen(prevstr);
-	for (; which <= matches; which++) {
-		for (i = 0; i < max_equal &&
-		    prevstr[i] == match_list[which][i]; i++)
-			continue;
-		max_equal = i;
-	}
-
-	retstr = malloc(max_equal + 1);
-	(void) strncpy(retstr, match_list[1], max_equal);
-	retstr[max_equal] = '\0';
-	match_list[0] = retstr;
-
-	/* add NULL as last pointer to the array */
-	if (matches + 1 >= match_list_len)
-		match_list = realloc(match_list,
-		    (match_list_len + 1) * sizeof(char *));
-	match_list[matches + 1] = (char *) NULL;
-
-	return (match_list);
-}
-
-/*
- * Sort function for qsort(). Just wrapper around strcasecmp().
- */
-static int
-_rl_qsort_string_compare(i1, i2)
-	const void *i1, *i2;
-{
-	/* LINTED const castaway */
-	const char *s1 = ((const char **)i1)[0];
-	/* LINTED const castaway */
-	const char *s2 = ((const char **)i2)[0];
-
-	return strcasecmp(s1, s2);
-}
-
-/*
- * Display list of strings in columnar format on readline's output stream.
- * 'matches' is list of strings, 'len' is number of strings in 'matches',
- * 'max' is maximum length of string in 'matches'.
- */
-void
-rl_display_match_list (matches, len, max)
-     char **matches;
-     int len, max;
-{
-	int i, idx, limit, count;
-	int screenwidth = e->el_term.t_size.h;
-
-	/*
-	 * Find out how many entries can be put on one line, count
-	 * with two spaces between strings.
-	 */
-	limit = screenwidth / (max + 2);
-	if (limit == 0)
-		limit = 1;
-
-	/* how many lines of output */
-	count = len / limit;
-	if (count * limit < len)
-		count++;
-
-	/* Sort the items if they are not already sorted. */
-	qsort(&matches[1], (size_t)(len - 1), sizeof(char *),
-	    _rl_qsort_string_compare);
-
-	idx = 1;
-	for(; count > 0; count--) {
-		for(i=0; i < limit && matches[idx]; i++, idx++)
-			fprintf(e->el_outfile, "%-*s  ", max, matches[idx]);
-		fprintf(e->el_outfile, "\n");
-	}
-}
-
-/*
- * Complete the word at or before point, called by rl_complete()
- * 'what_to_do' says what to do with the completion.
- * `?' means list the possible completions.
- * TAB means do standard completion.
- * `*' means insert all of the possible completions.
- * `!' means to do standard completion, and list all possible completions if
- * there is more than one.
- *
- * Note: '*' support is not implemented
- */
-static int
-rl_complete_internal(int what_to_do)
-{
-	CPFunction *complet_func;
-	const LineInfo *li;
-	char *temp, **matches;
-	const char *ctemp;
-	size_t len;
-
-	rl_completion_type = what_to_do;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	complet_func = rl_completion_entry_function;
-	if (!complet_func)
-		complet_func = filename_completion_function;
-
-	/* We now look backwards for the start of a filename/variable word */
-	li = el_line(e);
-	ctemp = (const char *) li->cursor;
-	while (ctemp > li->buffer
-	    && !strchr(rl_basic_word_break_characters, ctemp[-1])
-	    && (!rl_special_prefixes
-			|| !strchr(rl_special_prefixes, ctemp[-1]) ) )
-		ctemp--;
-
-	len = li->cursor - ctemp;
-	temp = alloca(len + 1);
-	(void) strncpy(temp, ctemp, len);
-	temp[len] = '\0';
-
-	/* these can be used by function called in completion_matches() */
-	/* or (*rl_attempted_completion_function)() */
-	rl_point = li->cursor - li->buffer;
-	rl_end = li->lastchar - li->buffer;
-
-	if (!rl_attempted_completion_function)
-		matches = completion_matches(temp, complet_func);
-	else {
-		int end = li->cursor - li->buffer;
-		matches = (*rl_attempted_completion_function) (temp, (int)
-		    (end - len), end);
-	}
-
-	if (matches) {
-		int i, retval = CC_REFRESH;
-		int matches_num, maxlen, match_len, match_display=1;
-
-		/*
-		 * Only replace the completed string with common part of
-		 * possible matches if there is possible completion.
-		 */
-		if (matches[0][0] != '\0') {
-			el_deletestr(e, (int) len);
-			el_insertstr(e, matches[0]);
-		}
-
-		if (what_to_do == '?')
-			goto display_matches;
-
-		if (matches[2] == NULL && strcmp(matches[0], matches[1]) == 0) {
-			/*
-			 * We found exact match. Add a space after
-			 * it, unless we do filename completition and the
-			 * object is a directory.
-			 */
-			size_t alen = strlen(matches[0]);
-			if ((complet_func != filename_completion_function
-			      || (alen > 0 && (matches[0])[alen - 1] != '/'))
-			    && rl_completion_append_character) {
-				char buf[2];
-				buf[0] = rl_completion_append_character;
-				buf[1] = '\0';
-				el_insertstr(e, buf);
-			}
-		} else if (what_to_do == '!') {
-    display_matches:
-			/*
-			 * More than one match and requested to list possible
-			 * matches.
-			 */
-
-			for(i=1, maxlen=0; matches[i]; i++) {
-				match_len = strlen(matches[i]);
-				if (match_len > maxlen)
-					maxlen = match_len;
-			}
-			matches_num = i - 1;
-
-			/* newline to get on next line from command line */
-			fprintf(e->el_outfile, "\n");
-
-			/*
-			 * If there are too many items, ask user for display
-			 * confirmation.
-			 */
-			if (matches_num > rl_completion_query_items) {
-				fprintf(e->el_outfile,
-				"Display all %d possibilities? (y or n) ",
-					matches_num);
-				fflush(e->el_outfile);
-				if (getc(stdin) != 'y')
-					match_display = 0;
-				fprintf(e->el_outfile, "\n");
-			}
-
-			if (match_display)
-				rl_display_match_list(matches, matches_num,
-					maxlen);
-			retval = CC_REDISPLAY;
-		} else if (matches[0][0]) {
-			/*
-			 * There was some common match, but the name was
-			 * not complete enough. Next tab will print possible
-			 * completions.
-			 */
-			el_beep(e);
-		} else {
-			/* lcd is not a valid object - further specification */
-			/* is needed */
-			el_beep(e);
-			retval = CC_NORM;
-		}
-
-		/* free elements of array and the array itself */
-		for (i = 0; matches[i]; i++)
-			free(matches[i]);
-		free(matches), matches = NULL;
-
-		return (retval);
-	}
-	return (CC_NORM);
-}
-
-
-/*
- * complete word at current point
- */
-int
-rl_complete(int ignore, int invoking_key)
-{
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	if (rl_inhibit_completion) {
-		rl_insert(ignore, invoking_key);
-		return (CC_REFRESH);
-	} else if (e->el_state.lastcmd == el_rl_complete_cmdnum)
-		return rl_complete_internal('?');
-	else if (_rl_complete_show_all)
-		return rl_complete_internal('!');
-	else
-		return (rl_complete_internal(TAB));
-}
-
-
-/*
- * misc other functions
- */
-
-/*
- * bind key c to readline-type function func
- */
-int
-rl_bind_key(int c, int func(int, int))
-{
-	int retval = -1;
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	if (func == rl_insert) {
-		/* XXX notice there is no range checking of ``c'' */
-		e->el_map.key[c] = ED_INSERT;
-		retval = 0;
-	}
-	return (retval);
-}
-
-
-/*
- * read one key from input - handles chars pushed back
- * to input stream also
- */
-int
-rl_read_key(void)
-{
-	char fooarr[2 * sizeof(int)];
-
-	if (e == NULL || h == NULL)
-		rl_initialize();
-
-	return (el_getc(e, fooarr));
-}
-
-
-/*
- * reset the terminal
- */
-/* ARGSUSED */
-void
-rl_reset_terminal(const char *p)
-{
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-	el_reset(e);
-}
-
-
-/*
- * insert character ``c'' back into input stream, ``count'' times
- */
-int
-rl_insert(int count, int c)
-{
-	char arr[2];
-
-	if (h == NULL || e == NULL)
-		rl_initialize();
-
-	/* XXX - int -> char conversion can lose on multichars */
-	arr[0] = c;
-	arr[1] = '\0';
-
-	for (; count > 0; count--)
-		el_push(e, arr);
-
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/readline.h 1:16.1.1~dfsg-1/main/editline/readline.h
--- 1:13.23.1~dfsg-2/main/editline/readline.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/readline.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,118 +0,0 @@
-/*	$NetBSD: readline.h,v 1.1 2001/01/05 21:15:50 jdolecek Exp $	*/
-
-/*-
- * Copyright (c) 1997 The NetBSD Foundation, Inc.
- * All rights reserved.
- *
- * This code is derived from software contributed to The NetBSD Foundation
- * by Jaromir Dolecek.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the NetBSD
- *	Foundation, Inc. and its contributors.
- * 4. Neither the name of The NetBSD Foundation nor the names of its
- *    contributors may be used to endorse or promote products derived
- *    from this software without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
- * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
- * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
- */
-#ifndef _READLINE_H_
-#define	_READLINE_H_
-
-#include <sys/types.h>
-
-/* list of readline stuff supported by editline library's readline wrapper */
-
-/* typedefs */
-typedef int	  Function(const char *, int);
-typedef void	  VFunction(void);
-typedef char	 *CPFunction(const char *, int);
-typedef char	**CPPFunction(const char *, int, int);
-
-typedef struct _hist_entry {
-	const char	*line;
-	const char	*data;
-} HIST_ENTRY;
-
-/* global variables used by readline enabled applications */
-#ifdef __cplusplus
-extern "C" {
-#endif
-extern const char	*rl_library_version;
-extern char		*rl_readline_name;
-extern FILE		*rl_instream;
-extern FILE		*rl_outstream;
-extern char		*rl_line_buffer;
-extern int		 rl_point, rl_end;
-extern int		 history_base, history_length;
-extern int		 max_input_history;
-extern char		*rl_basic_word_break_characters;
-extern char		*rl_completer_word_break_characters;
-extern char		*rl_completer_quote_characters;
-extern CPFunction	*rl_completion_entry_function;
-extern CPPFunction	*rl_attempted_completion_function;
-extern int		rl_completion_type;
-extern int		rl_completion_query_items;
-extern char		*rl_special_prefixes;
-extern int		rl_completion_append_character;
-
-/* supported functions */
-char		*readline(const char *);
-int		 rl_initialize(void);
-
-void		 using_history(void);
-int		 add_history(const char *);
-void		 clear_history(void);
-void		 stifle_history(int);
-int		 unstifle_history(void);
-int		 history_is_stifled(void);
-int		 where_history(void);
-HIST_ENTRY	*current_history(void);
-HIST_ENTRY	*history_get(int);
-int		 history_total_bytes(void);
-int		 history_set_pos(int);
-HIST_ENTRY	*previous_history(void);
-HIST_ENTRY	*next_history(void);
-int		 history_search(const char *, int);
-int		 history_search_prefix(const char *, int);
-int		 history_search_pos(const char *, int, int);
-int		 read_history(const char *);
-int		 write_history(const char *);
-int		 history_expand(char *, char **);
-char	       **history_tokenize(const char *);
-
-char		*tilde_expand(char *);
-char		*filename_completion_function(const char *, int);
-char		*username_completion_function(const char *, int);
-int		 rl_complete(int, int);
-int		 rl_read_key(void);
-char	       **completion_matches(const char *, CPFunction *);
-void		 rl_display_match_list(char **, int, int);
-
-int		 rl_insert(int, int);
-void		 rl_reset_terminal(const char *);
-int		 rl_bind_key(int, int (*)(int, int));
-#ifdef __cplusplus
-}
-#endif
-
-#endif /* _READLINE_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/editline/README 1:16.1.1~dfsg-1/main/editline/README
--- 1:13.23.1~dfsg-2/main/editline/README	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/README	1970-01-01 00:00:00.000000000 +0000
@@ -1,11 +0,0 @@
-libedit is a command line editing and history library.  It is designed to be
-used by interactive programs that allow the user to type commands at a terminal
-prompt.
-
-The following files may be of direct interest to the user:
-
-* CHANGES - Software change log.
-
-* INSTALL - Installation information.
-
-* PLATFORMS - Supported platforms and platform-specific information.
diff -pruN 1:13.23.1~dfsg-2/main/editline/refresh.c 1:16.1.1~dfsg-1/main/editline/refresh.c
--- 1:13.23.1~dfsg-2/main/editline/refresh.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/refresh.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,1104 +0,0 @@
-/*	$NetBSD: refresh.c,v 1.18 2002/03/18 16:00:58 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)refresh.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: refresh.c,v 1.18 2002/03/18 16:00:58 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * refresh.c: Lower level screen refreshing functions
- */
-#include <stdio.h>
-#include <ctype.h>
-#include <unistd.h>
-#include <string.h>
-
-#include "el.h"
-
-private void	re_addc(EditLine *, int);
-private void	re_update_line(EditLine *, char *, char *, int);
-private void	re_insert (EditLine *, char *, int, int, char *, int);
-private void	re_delete(EditLine *, char *, int, int, int);
-private void	re_fastputc(EditLine *, int);
-private void	re__strncopy(char *, char *, size_t);
-private void	re__copy_and_pad(char *, const char *, size_t);
-
-#ifdef DEBUG_REFRESH
-private void	re_printstr(EditLine *, char *, char *, char *);
-#define	__F el->el_errfile
-#define	ELRE_ASSERT(a, b, c)	do 				\
-				    if (a) {			\
-					(void) fprintf b;	\
-					c;			\
-				    }				\
-				while (0)
-#define	ELRE_DEBUG(a, b)	ELRE_ASSERT(a,b,;)
-
-/* re_printstr():
- *	Print a string on the debugging pty
- */
-private void
-re_printstr(EditLine *el, char *str, char *f, char *t)
-{
-
-	ELRE_DEBUG(1, (__F, "%s:\"", str));
-	while (f < t)
-		ELRE_DEBUG(1, (__F, "%c", *f++ & 0177));
-	ELRE_DEBUG(1, (__F, "\"\r\n"));
-}
-#else
-#define	ELRE_ASSERT(a, b, c)
-#define	ELRE_DEBUG(a, b)
-#endif
-
-
-/* re_addc():
- *	Draw c, expanding tabs, control chars etc.
- */
-private void
-re_addc(EditLine *el, int c)
-{
-
-	if (isprint(c)) {
-		re_putc(el, c, 1);
-		return;
-	}
-	if (c == '\n') {				/* expand the newline */
-		int oldv = el->el_refresh.r_cursor.v;
-		re_putc(el, '\0', 0);			/* assure end of line */
-		if (oldv == el->el_refresh.r_cursor.v) { /* XXX */
-			el->el_refresh.r_cursor.h = 0;	/* reset cursor pos */
-			el->el_refresh.r_cursor.v++;
-		}
-		return;
-	}
-	if (c == '\t') {				/* expand the tab */
-		for (;;) {
-			re_putc(el, ' ', 1);
-			if ((el->el_refresh.r_cursor.h & 07) == 0)
-				break;			/* go until tab stop */
-		}
-	} else if (iscntrl(c)) {
-		re_putc(el, '^', 1);
-		if (c == '\177')
-			re_putc(el, '?', 1);
-		else
-		    /* uncontrolify it; works only for iso8859-1 like sets */
-			re_putc(el, (c | 0100), 1);
-	} else {
-		re_putc(el, '\\', 1);
-		re_putc(el, (int) ((((unsigned int) c >> 6) & 07) + '0'), 1);
-		re_putc(el, (int) ((((unsigned int) c >> 3) & 07) + '0'), 1);
-		re_putc(el, (c & 07) + '0', 1);
-	}
-}
-
-
-/* re_putc():
- *	Draw the character given
- */
-protected void
-re_putc(EditLine *el, int c, int shift)
-{
-
-	ELRE_DEBUG(1, (__F, "printing %3.3o '%c'\r\n", c, c));
-
-	el->el_vdisplay[el->el_refresh.r_cursor.v][el->el_refresh.r_cursor.h] = c;
-	if (!shift)
-		return;
-
-	el->el_refresh.r_cursor.h++;	/* advance to next place */
-	if (el->el_refresh.r_cursor.h >= el->el_term.t_size.h) {
-		el->el_vdisplay[el->el_refresh.r_cursor.v][el->el_term.t_size.h] = '\0';
-		/* assure end of line */
-		el->el_refresh.r_cursor.h = 0;	/* reset it. */
-
-		/*
-		 * If we would overflow (input is longer than terminal size),
-		 * emulate scroll by dropping first line and shuffling the rest.
-		 * We do this via pointer shuffling - it's safe in this case
-		 * and we avoid memcpy().
-		 */
-		if (el->el_refresh.r_cursor.v + 1 >= el->el_term.t_size.v) {
-			int i, lins = el->el_term.t_size.v;
-			char *firstline = el->el_vdisplay[0];
-
-			for(i=1; i < lins; i++)
-				el->el_vdisplay[i-1] = el->el_vdisplay[i];
-
-			firstline[0] = '\0';		/* empty the string */
-			el->el_vdisplay[i-1] = firstline;
-		} else
-			el->el_refresh.r_cursor.v++;
-
-		ELRE_ASSERT(el->el_refresh.r_cursor.v >= el->el_term.t_size.v,
-		    (__F, "\r\nre_putc: overflow! r_cursor.v == %d > %d\r\n",
-		    el->el_refresh.r_cursor.v, el->el_term.t_size.v),
-		    abort());
-	}
-}
-
-
-/* re_refresh():
- *	draws the new virtual screen image from the current input
- *  	line, then goes line-by-line changing the real image to the new
- *	virtual image. The routine to re-draw a line can be replaced
- *	easily in hopes of a smarter one being placed there.
- */
-protected void
-re_refresh(EditLine *el)
-{
-	int i, rhdiff;
-	char *cp, *st;
-	coord_t cur;
-#ifdef notyet
-	size_t termsz;
-#endif
-
-	ELRE_DEBUG(1, (__F, "el->el_line.buffer = :%s:\r\n",
-	    el->el_line.buffer));
-
-	/* reset the Drawing cursor */
-	el->el_refresh.r_cursor.h = 0;
-	el->el_refresh.r_cursor.v = 0;
-
-	/* temporarily draw rprompt to calculate its size */
-	prompt_print(el, EL_RPROMPT);
-
-	/* reset the Drawing cursor */
-	el->el_refresh.r_cursor.h = 0;
-	el->el_refresh.r_cursor.v = 0;
-
-	cur.h = -1;		/* set flag in case I'm not set */
-	cur.v = 0;
-
-	prompt_print(el, EL_PROMPT);
-
-	/* draw the current input buffer */
-#ifdef notyet
-	termsz = el->el_term.t_size.h * el->el_term.t_size.v;
-	if (el->el_line.lastchar - el->el_line.buffer > termsz) {
-		/*
-		 * If line is longer than terminal, process only part
-		 * of line which would influence display.
-		 */
-		size_t rem = (el->el_line.lastchar-el->el_line.buffer)%termsz;
-
-		st = el->el_line.lastchar - rem
-			- (termsz - (((rem / el->el_term.t_size.v) - 1)
-					* el->el_term.t_size.v));
-	} else
-#endif
-		st = el->el_line.buffer;
-
-	for (cp = st; cp < el->el_line.lastchar; cp++) {
-		if (cp == el->el_line.cursor) {
-			/* save for later */
-			cur.h = el->el_refresh.r_cursor.h;
-			cur.v = el->el_refresh.r_cursor.v;
-		}
-		re_addc(el, (unsigned char) *cp);
-	}
-
-	if (cur.h == -1) {	/* if I haven't been set yet, I'm at the end */
-		cur.h = el->el_refresh.r_cursor.h;
-		cur.v = el->el_refresh.r_cursor.v;
-	}
-	rhdiff = el->el_term.t_size.h - el->el_refresh.r_cursor.h -
-	    el->el_rprompt.p_pos.h;
-	if (el->el_rprompt.p_pos.h && !el->el_rprompt.p_pos.v &&
-	    !el->el_refresh.r_cursor.v && rhdiff > 1) {
-		/*
-		 * have a right-hand side prompt that will fit
-		 * on the end of the first line with at least
-		 * one character gap to the input buffer.
-		 */
-		while (--rhdiff > 0)	/* pad out with spaces */
-			re_putc(el, ' ', 1);
-		prompt_print(el, EL_RPROMPT);
-	} else {
-		el->el_rprompt.p_pos.h = 0;	/* flag "not using rprompt" */
-		el->el_rprompt.p_pos.v = 0;
-	}
-
-	re_putc(el, '\0', 0);	/* make line ended with NUL, no cursor shift */
-
-	el->el_refresh.r_newcv = el->el_refresh.r_cursor.v;
-
-	ELRE_DEBUG(1, (__F,
-		"term.h=%d vcur.h=%d vcur.v=%d vdisplay[0]=\r\n:%80.80s:\r\n",
-		el->el_term.t_size.h, el->el_refresh.r_cursor.h,
-		el->el_refresh.r_cursor.v, el->el_vdisplay[0]));
-
-	ELRE_DEBUG(1, (__F, "updating %d lines.\r\n", el->el_refresh.r_newcv));
-	for (i = 0; i <= el->el_refresh.r_newcv; i++) {
-		/* NOTE THAT re_update_line MAY CHANGE el_display[i] */
-		re_update_line(el, el->el_display[i], el->el_vdisplay[i], i);
-
-		/*
-		 * Copy the new line to be the current one, and pad out with
-		 * spaces to the full width of the terminal so that if we try
-		 * moving the cursor by writing the character that is at the
-		 * end of the screen line, it won't be a NUL or some old
-		 * leftover stuff.
-		 */
-		re__copy_and_pad(el->el_display[i], el->el_vdisplay[i],
-		    (size_t) el->el_term.t_size.h);
-	}
-	ELRE_DEBUG(1, (__F,
-	"\r\nel->el_refresh.r_cursor.v=%d,el->el_refresh.r_oldcv=%d i=%d\r\n",
-	    el->el_refresh.r_cursor.v, el->el_refresh.r_oldcv, i));
-
-	if (el->el_refresh.r_oldcv > el->el_refresh.r_newcv)
-		for (; i <= el->el_refresh.r_oldcv; i++) {
-			term_move_to_line(el, i);
-			term_move_to_char(el, 0);
-			term_clear_EOL(el, (int) strlen(el->el_display[i]));
-#ifdef DEBUG_REFRESH
-			term_overwrite(el, "C\b", 2);
-#endif /* DEBUG_REFRESH */
-			el->el_display[i][0] = '\0';
-		}
-
-	el->el_refresh.r_oldcv = el->el_refresh.r_newcv; /* set for next time */
-	ELRE_DEBUG(1, (__F,
-	    "\r\ncursor.h = %d, cursor.v = %d, cur.h = %d, cur.v = %d\r\n",
-	    el->el_refresh.r_cursor.h, el->el_refresh.r_cursor.v,
-	    cur.h, cur.v));
-	term_move_to_line(el, cur.v);	/* go to where the cursor is */
-	term_move_to_char(el, cur.h);
-}
-
-
-/* re_goto_bottom():
- *	 used to go to last used screen line
- */
-protected void
-re_goto_bottom(EditLine *el)
-{
-
-	term_move_to_line(el, el->el_refresh.r_oldcv);
-	term__putc('\r');
-	term__putc('\n');
-	re_clear_display(el);
-	term__flush();
-}
-
-
-/* re_insert():
- *	insert num characters of s into d (in front of the character)
- *	at dat, maximum length of d is dlen
- */
-private void
-/*ARGSUSED*/
-re_insert(EditLine *el, char *d, int dat, int dlen, char *s, int num)
-{
-	char *a, *b;
-
-	if (num <= 0)
-		return;
-	if (num > dlen - dat)
-		num = dlen - dat;
-
-	ELRE_DEBUG(1,
-	    (__F, "re_insert() starting: %d at %d max %d, d == \"%s\"\n",
-	    num, dat, dlen, d));
-	ELRE_DEBUG(1, (__F, "s == \"%s\"n", s));
-
-	/* open up the space for num chars */
-	if (num > 0) {
-		b = d + dlen - 1;
-		a = b - num;
-		while (a >= &d[dat])
-			*b-- = *a--;
-		d[dlen] = '\0';	/* just in case */
-	}
-	ELRE_DEBUG(1, (__F,
-		"re_insert() after insert: %d at %d max %d, d == \"%s\"\n",
-		num, dat, dlen, d));
-	ELRE_DEBUG(1, (__F, "s == \"%s\"n", s));
-
-	/* copy the characters */
-	for (a = d + dat; (a < d + dlen) && (num > 0); num--)
-		*a++ = *s++;
-
-	ELRE_DEBUG(1,
-	    (__F, "re_insert() after copy: %d at %d max %d, %s == \"%s\"\n",
-	    num, dat, dlen, d, s));
-	ELRE_DEBUG(1, (__F, "s == \"%s\"n", s));
-}
-
-
-/* re_delete():
- *	delete num characters d at dat, maximum length of d is dlen
- */
-private void
-/*ARGSUSED*/
-re_delete(EditLine *el, char *d, int dat, int dlen, int num)
-{
-	char *a, *b;
-
-	if (num <= 0)
-		return;
-	if (dat + num >= dlen) {
-		d[dat] = '\0';
-		return;
-	}
-	ELRE_DEBUG(1,
-	    (__F, "re_delete() starting: %d at %d max %d, d == \"%s\"\n",
-	    num, dat, dlen, d));
-
-	/* open up the space for num chars */
-	if (num > 0) {
-		b = d + dat;
-		a = b + num;
-		while (a < &d[dlen])
-			*b++ = *a++;
-		d[dlen] = '\0';	/* just in case */
-	}
-	ELRE_DEBUG(1,
-	    (__F, "re_delete() after delete: %d at %d max %d, d == \"%s\"\n",
-	    num, dat, dlen, d));
-}
-
-
-/* re__strncopy():
- *	Like strncpy without padding.
- */
-private void
-re__strncopy(char *a, char *b, size_t n)
-{
-
-	while (n-- && *b)
-		*a++ = *b++;
-}
-
-
-/*****************************************************************
-    re_update_line() is based on finding the middle difference of each line
-    on the screen; vis:
-
-			     /old first difference
-	/beginning of line   |              /old last same       /old EOL
-	v		     v              v                    v
-old:	eddie> Oh, my little gruntle-buggy is to me, as lurgid as
-new:	eddie> Oh, my little buggy says to me, as lurgid as
-	^		     ^        ^			   ^
-	\beginning of line   |        \new last same	   \new end of line
-			     \new first difference
-
-    all are character pointers for the sake of speed.  Special cases for
-    no differences, as well as for end of line additions must be handled.
-**************************************************************** */
-
-/* Minimum at which doing an insert it "worth it".  This should be about
- * half the "cost" of going into insert mode, inserting a character, and
- * going back out.  This should really be calculated from the termcap
- * data...  For the moment, a good number for ANSI terminals.
- */
-#define	MIN_END_KEEP	4
-
-private void
-re_update_line(EditLine *el, char *old, char *new, int i)
-{
-	char *o, *n, *p, c;
-	char *ofd, *ols, *oe, *nfd, *nls, *ne;
-	char *osb, *ose, *nsb, *nse;
-	int fx, sx;
-
-	/*
-         * find first diff
-         */
-	for (o = old, n = new; *o && (*o == *n); o++, n++)
-		continue;
-	ofd = o;
-	nfd = n;
-
-	/*
-         * Find the end of both old and new
-         */
-	while (*o)
-		o++;
-	/*
-         * Remove any trailing blanks off of the end, being careful not to
-         * back up past the beginning.
-         */
-	while (ofd < o) {
-		if (o[-1] != ' ')
-			break;
-		o--;
-	}
-	oe = o;
-	*oe = '\0';
-
-	while (*n)
-		n++;
-
-	/* remove blanks from end of new */
-	while (nfd < n) {
-		if (n[-1] != ' ')
-			break;
-		n--;
-	}
-	ne = n;
-	*ne = '\0';
-
-	/*
-         * if no diff, continue to next line of redraw
-         */
-	if (*ofd == '\0' && *nfd == '\0') {
-		ELRE_DEBUG(1, (__F, "no difference.\r\n"));
-		return;
-	}
-	/*
-         * find last same pointer
-         */
-	while ((o > ofd) && (n > nfd) && (*--o == *--n))
-		continue;
-	ols = ++o;
-	nls = ++n;
-
-	/*
-         * find same begining and same end
-         */
-	osb = ols;
-	nsb = nls;
-	ose = ols;
-	nse = nls;
-
-	/*
-         * case 1: insert: scan from nfd to nls looking for *ofd
-         */
-	if (*ofd) {
-		for (c = *ofd, n = nfd; n < nls; n++) {
-			if (c == *n) {
-				for (o = ofd, p = n;
-				    p < nls && o < ols && *o == *p;
-				    o++, p++)
-					continue;
-				/*
-				 * if the new match is longer and it's worth
-				 * keeping, then we take it
-				 */
-				if (((nse - nsb) < (p - n)) &&
-				    (2 * (p - n) > n - nfd)) {
-					nsb = n;
-					nse = p;
-					osb = ofd;
-					ose = o;
-				}
-			}
-		}
-	}
-	/*
-         * case 2: delete: scan from ofd to ols looking for *nfd
-         */
-	if (*nfd) {
-		for (c = *nfd, o = ofd; o < ols; o++) {
-			if (c == *o) {
-				for (n = nfd, p = o;
-				    p < ols && n < nls && *p == *n;
-				    p++, n++)
-					continue;
-				/*
-				 * if the new match is longer and it's worth
-				 * keeping, then we take it
-				 */
-				if (((ose - osb) < (p - o)) &&
-				    (2 * (p - o) > o - ofd)) {
-					nsb = nfd;
-					nse = n;
-					osb = o;
-					ose = p;
-				}
-			}
-		}
-	}
-	/*
-         * Pragmatics I: If old trailing whitespace or not enough characters to
-         * save to be worth it, then don't save the last same info.
-         */
-	if ((oe - ols) < MIN_END_KEEP) {
-		ols = oe;
-		nls = ne;
-	}
-	/*
-         * Pragmatics II: if the terminal isn't smart enough, make the data
-         * dumber so the smart update doesn't try anything fancy
-         */
-
-	/*
-         * fx is the number of characters we need to insert/delete: in the
-         * beginning to bring the two same begins together
-         */
-	fx = (nsb - nfd) - (osb - ofd);
-	/*
-         * sx is the number of characters we need to insert/delete: in the
-         * end to bring the two same last parts together
-         */
-	sx = (nls - nse) - (ols - ose);
-
-	if (!EL_CAN_INSERT) {
-		if (fx > 0) {
-			osb = ols;
-			ose = ols;
-			nsb = nls;
-			nse = nls;
-		}
-		if (sx > 0) {
-			ols = oe;
-			nls = ne;
-		}
-		if ((ols - ofd) < (nls - nfd)) {
-			ols = oe;
-			nls = ne;
-		}
-	}
-	if (!EL_CAN_DELETE) {
-		if (fx < 0) {
-			osb = ols;
-			ose = ols;
-			nsb = nls;
-			nse = nls;
-		}
-		if (sx < 0) {
-			ols = oe;
-			nls = ne;
-		}
-		if ((ols - ofd) > (nls - nfd)) {
-			ols = oe;
-			nls = ne;
-		}
-	}
-	/*
-         * Pragmatics III: make sure the middle shifted pointers are correct if
-         * they don't point to anything (we may have moved ols or nls).
-         */
-	/* if the change isn't worth it, don't bother */
-	/* was: if (osb == ose) */
-	if ((ose - osb) < MIN_END_KEEP) {
-		osb = ols;
-		ose = ols;
-		nsb = nls;
-		nse = nls;
-	}
-	/*
-         * Now that we are done with pragmatics we recompute fx, sx
-         */
-	fx = (nsb - nfd) - (osb - ofd);
-	sx = (nls - nse) - (ols - ose);
-
-	ELRE_DEBUG(1, (__F, "\n"));
-	ELRE_DEBUG(1, (__F, "ofd %d, osb %d, ose %d, ols %d, oe %d\n",
-		ofd - old, osb - old, ose - old, ols - old, oe - old));
-	ELRE_DEBUG(1, (__F, "nfd %d, nsb %d, nse %d, nls %d, ne %d\n",
-		nfd - new, nsb - new, nse - new, nls - new, ne - new));
-	ELRE_DEBUG(1, (__F,
-		"xxx-xxx:\"00000000001111111111222222222233333333334\"\r\n"));
-	ELRE_DEBUG(1, (__F,
-		"xxx-xxx:\"01234567890123456789012345678901234567890\"\r\n"));
-#ifdef DEBUG_REFRESH
-	re_printstr(el, "old- oe", old, oe);
-	re_printstr(el, "new- ne", new, ne);
-	re_printstr(el, "old-ofd", old, ofd);
-	re_printstr(el, "new-nfd", new, nfd);
-	re_printstr(el, "ofd-osb", ofd, osb);
-	re_printstr(el, "nfd-nsb", nfd, nsb);
-	re_printstr(el, "osb-ose", osb, ose);
-	re_printstr(el, "nsb-nse", nsb, nse);
-	re_printstr(el, "ose-ols", ose, ols);
-	re_printstr(el, "nse-nls", nse, nls);
-	re_printstr(el, "ols- oe", ols, oe);
-	re_printstr(el, "nls- ne", nls, ne);
-#endif /* DEBUG_REFRESH */
-
-	/*
-         * el_cursor.v to this line i MUST be in this routine so that if we
-         * don't have to change the line, we don't move to it. el_cursor.h to
-         * first diff char
-         */
-	term_move_to_line(el, i);
-
-	/*
-         * at this point we have something like this:
-         *
-         * /old                  /ofd    /osb               /ose    /ols     /oe
-         * v.....................v       v..................v       v........v
-         * eddie> Oh, my fredded gruntle-buggy is to me, as foo var lurgid as
-         * eddie> Oh, my fredded quiux buggy is to me, as gruntle-lurgid as
-         * ^.....................^     ^..................^       ^........^
-         * \new                  \nfd  \nsb               \nse     \nls    \ne
-         *
-         * fx is the difference in length between the chars between nfd and
-         * nsb, and the chars between ofd and osb, and is thus the number of
-         * characters to delete if < 0 (new is shorter than old, as above),
-         * or insert (new is longer than short).
-         *
-         * sx is the same for the second differences.
-         */
-
-	/*
-         * if we have a net insert on the first difference, AND inserting the
-         * net amount ((nsb-nfd) - (osb-ofd)) won't push the last useful
-         * character (which is ne if nls != ne, otherwise is nse) off the edge
-	 * of the screen (el->el_term.t_size.h) else we do the deletes first
-	 * so that we keep everything we need to.
-         */
-
-	/*
-         * if the last same is the same like the end, there is no last same
-         * part, otherwise we want to keep the last same part set p to the
-         * last useful old character
-         */
-	p = (ols != oe) ? oe : ose;
-
-	/*
-         * if (There is a diffence in the beginning) && (we need to insert
-         *   characters) && (the number of characters to insert is less than
-         *   the term width)
-	 *	We need to do an insert!
-	 * else if (we need to delete characters)
-	 *	We need to delete characters!
-	 * else
-	 *	No insert or delete
-         */
-	if ((nsb != nfd) && fx > 0 &&
-	    ((p - old) + fx <= el->el_term.t_size.h)) {
-		ELRE_DEBUG(1,
-		    (__F, "first diff insert at %d...\r\n", nfd - new));
-		/*
-		 * Move to the first char to insert, where the first diff is.
-		 */
-		term_move_to_char(el, nfd - new);
-		/*
-		 * Check if we have stuff to keep at end
-		 */
-		if (nsb != ne) {
-			ELRE_DEBUG(1, (__F, "with stuff to keep at end\r\n"));
-			/*
-		         * insert fx chars of new starting at nfd
-		         */
-			if (fx > 0) {
-				ELRE_DEBUG(!EL_CAN_INSERT, (__F,
-				"ERROR: cannot insert in early first diff\n"));
-				term_insertwrite(el, nfd, fx);
-				re_insert(el, old, ofd - old,
-				    el->el_term.t_size.h, nfd, fx);
-			}
-			/*
-		         * write (nsb-nfd) - fx chars of new starting at
-		         * (nfd + fx)
-			 */
-			term_overwrite(el, nfd + fx, (nsb - nfd) - fx);
-			re__strncopy(ofd + fx, nfd + fx,
-			    (size_t) ((nsb - nfd) - fx));
-		} else {
-			ELRE_DEBUG(1, (__F, "without anything to save\r\n"));
-			term_overwrite(el, nfd, (nsb - nfd));
-			re__strncopy(ofd, nfd, (size_t) (nsb - nfd));
-			/*
-		         * Done
-		         */
-			return;
-		}
-	} else if (fx < 0) {
-		ELRE_DEBUG(1,
-		    (__F, "first diff delete at %d...\r\n", ofd - old));
-		/*
-		 * move to the first char to delete where the first diff is
-		 */
-		term_move_to_char(el, ofd - old);
-		/*
-		 * Check if we have stuff to save
-		 */
-		if (osb != oe) {
-			ELRE_DEBUG(1, (__F, "with stuff to save at end\r\n"));
-			/*
-		         * fx is less than zero *always* here but we check
-		         * for code symmetry
-		         */
-			if (fx < 0) {
-				ELRE_DEBUG(!EL_CAN_DELETE, (__F,
-				    "ERROR: cannot delete in first diff\n"));
-				term_deletechars(el, -fx);
-				re_delete(el, old, ofd - old,
-				    el->el_term.t_size.h, -fx);
-			}
-			/*
-		         * write (nsb-nfd) chars of new starting at nfd
-		         */
-			term_overwrite(el, nfd, (nsb - nfd));
-			re__strncopy(ofd, nfd, (size_t) (nsb - nfd));
-
-		} else {
-			ELRE_DEBUG(1, (__F,
-			    "but with nothing left to save\r\n"));
-			/*
-		         * write (nsb-nfd) chars of new starting at nfd
-		         */
-			term_overwrite(el, nfd, (nsb - nfd));
-			ELRE_DEBUG(1, (__F,
-			    "cleareol %d\n", (oe - old) - (ne - new)));
-			term_clear_EOL(el, (oe - old) - (ne - new));
-			/*
-		         * Done
-		         */
-			return;
-		}
-	} else
-		fx = 0;
-
-	if (sx < 0 && (ose - old) + fx < el->el_term.t_size.h) {
-		ELRE_DEBUG(1, (__F,
-		    "second diff delete at %d...\r\n", (ose - old) + fx));
-		/*
-		 * Check if we have stuff to delete
-		 */
-		/*
-		 * fx is the number of characters inserted (+) or deleted (-)
-		 */
-
-		term_move_to_char(el, (ose - old) + fx);
-		/*
-		 * Check if we have stuff to save
-		 */
-		if (ols != oe) {
-			ELRE_DEBUG(1, (__F, "with stuff to save at end\r\n"));
-			/*
-		         * Again a duplicate test.
-		         */
-			if (sx < 0) {
-				ELRE_DEBUG(!EL_CAN_DELETE, (__F,
-				    "ERROR: cannot delete in second diff\n"));
-				term_deletechars(el, -sx);
-			}
-			/*
-		         * write (nls-nse) chars of new starting at nse
-		         */
-			term_overwrite(el, nse, (nls - nse));
-		} else {
-			ELRE_DEBUG(1, (__F,
-			    "but with nothing left to save\r\n"));
-			term_overwrite(el, nse, (nls - nse));
-			ELRE_DEBUG(1, (__F,
-			    "cleareol %d\n", (oe - old) - (ne - new)));
-			if ((oe - old) - (ne - new) != 0)
-				term_clear_EOL(el, (oe - old) - (ne - new));
-		}
-	}
-	/*
-         * if we have a first insert AND WE HAVEN'T ALREADY DONE IT...
-         */
-	if ((nsb != nfd) && (osb - ofd) <= (nsb - nfd) && (fx == 0)) {
-		ELRE_DEBUG(1, (__F, "late first diff insert at %d...\r\n",
-		    nfd - new));
-
-		term_move_to_char(el, nfd - new);
-		/*
-		 * Check if we have stuff to keep at the end
-		 */
-		if (nsb != ne) {
-			ELRE_DEBUG(1, (__F, "with stuff to keep at end\r\n"));
-			/*
-		         * We have to recalculate fx here because we set it
-		         * to zero above as a flag saying that we hadn't done
-		         * an early first insert.
-		         */
-			fx = (nsb - nfd) - (osb - ofd);
-			if (fx > 0) {
-				/*
-				 * insert fx chars of new starting at nfd
-				 */
-				ELRE_DEBUG(!EL_CAN_INSERT, (__F,
-				 "ERROR: cannot insert in late first diff\n"));
-				term_insertwrite(el, nfd, fx);
-				re_insert(el, old, ofd - old,
-				    el->el_term.t_size.h, nfd, fx);
-			}
-			/*
-		         * write (nsb-nfd) - fx chars of new starting at
-		         * (nfd + fx)
-			 */
-			term_overwrite(el, nfd + fx, (nsb - nfd) - fx);
-			re__strncopy(ofd + fx, nfd + fx,
-			    (size_t) ((nsb - nfd) - fx));
-		} else {
-			ELRE_DEBUG(1, (__F, "without anything to save\r\n"));
-			term_overwrite(el, nfd, (nsb - nfd));
-			re__strncopy(ofd, nfd, (size_t) (nsb - nfd));
-		}
-	}
-	/*
-         * line is now NEW up to nse
-         */
-	if (sx >= 0) {
-		ELRE_DEBUG(1, (__F,
-		    "second diff insert at %d...\r\n", nse - new));
-		term_move_to_char(el, nse - new);
-		if (ols != oe) {
-			ELRE_DEBUG(1, (__F, "with stuff to keep at end\r\n"));
-			if (sx > 0) {
-				/* insert sx chars of new starting at nse */
-				ELRE_DEBUG(!EL_CAN_INSERT, (__F,
-				    "ERROR: cannot insert in second diff\n"));
-				term_insertwrite(el, nse, sx);
-			}
-			/*
-		         * write (nls-nse) - sx chars of new starting at
-			 * (nse + sx)
-		         */
-			term_overwrite(el, nse + sx, (nls - nse) - sx);
-		} else {
-			ELRE_DEBUG(1, (__F, "without anything to save\r\n"));
-			term_overwrite(el, nse, (nls - nse));
-
-			/*
-	                 * No need to do a clear-to-end here because we were
-	                 * doing a second insert, so we will have over
-	                 * written all of the old string.
-		         */
-		}
-	}
-	ELRE_DEBUG(1, (__F, "done.\r\n"));
-}
-
-
-/* re__copy_and_pad():
- *	Copy string and pad with spaces
- */
-private void
-re__copy_and_pad(char *dst, const char *src, size_t width)
-{
-	int i;
-
-	for (i = 0; i < width; i++) {
-		if (*src == '\0')
-			break;
-		*dst++ = *src++;
-	}
-
-	for (; i < width; i++)
-		*dst++ = ' ';
-
-	*dst = '\0';
-}
-
-
-/* re_refresh_cursor():
- *	Move to the new cursor position
- */
-protected void
-re_refresh_cursor(EditLine *el)
-{
-	char *cp, c;
-	int h, v, th;
-
-	/* first we must find where the cursor is... */
-	h = el->el_prompt.p_pos.h;
-	v = el->el_prompt.p_pos.v;
-	th = el->el_term.t_size.h;	/* optimize for speed */
-
-	/* do input buffer to el->el_line.cursor */
-	for (cp = el->el_line.buffer; cp < el->el_line.cursor; cp++) {
-		c = *cp;
-		h++;		/* all chars at least this long */
-
-		if (c == '\n') {/* handle newline in data part too */
-			h = 0;
-			v++;
-		} else {
-			if (c == '\t') {	/* if a tab, to next tab stop */
-				while (h & 07) {
-					h++;
-				}
-			} else if (iscntrl((unsigned char) c)) {
-						/* if control char */
-				h++;
-				if (h > th) {	/* if overflow, compensate */
-					h = 1;
-					v++;
-				}
-			} else if (!isprint((unsigned char) c)) {
-				h += 3;
-				if (h > th) {	/* if overflow, compensate */
-					h = h - th;
-					v++;
-				}
-			}
-		}
-
-		if (h >= th) {	/* check, extra long tabs picked up here also */
-			h = 0;
-			v++;
-		}
-	}
-
-	/* now go there */
-	term_move_to_line(el, v);
-	term_move_to_char(el, h);
-	term__flush();
-}
-
-
-/* re_fastputc():
- *	Add a character fast.
- */
-private void
-re_fastputc(EditLine *el, int c)
-{
-
-	term__putc(c);
-	el->el_display[el->el_cursor.v][el->el_cursor.h++] = c;
-	if (el->el_cursor.h >= el->el_term.t_size.h) {
-		/* if we must overflow */
-		el->el_cursor.h = 0;
-
-		/*
-		 * If we would overflow (input is longer than terminal size),
-		 * emulate scroll by dropping first line and shuffling the rest.
-		 * We do this via pointer shuffling - it's safe in this case
-		 * and we avoid memcpy().
-		 */
-		if (el->el_cursor.v + 1 >= el->el_term.t_size.v) {
-			int i, lins = el->el_term.t_size.v;
-			char *firstline = el->el_display[0];
-
-			for(i=1; i < lins; i++)
-				el->el_display[i-1] = el->el_display[i];
-
-			re__copy_and_pad(firstline, "", 0);
-			el->el_display[i-1] = firstline;
-		} else {
-			el->el_cursor.v++;
-			el->el_refresh.r_oldcv++;
-		}
-		if (EL_HAS_AUTO_MARGINS) {
-			if (EL_HAS_MAGIC_MARGINS) {
-				term__putc(' ');
-				term__putc('\b');
-			}
-		} else {
-			term__putc('\r');
-			term__putc('\n');
-		}
-	}
-}
-
-
-/* re_fastaddc():
- *	we added just one char, handle it fast.
- *	Assumes that screen cursor == real cursor
- */
-protected void
-re_fastaddc(EditLine *el)
-{
-	char c;
-	int rhdiff;
-
-	c = el->el_line.cursor[-1];
-
-	if (c == '\t' || el->el_line.cursor != el->el_line.lastchar) {
-		re_refresh(el);	/* too hard to handle */
-		return;
-	}
-	rhdiff = el->el_term.t_size.h - el->el_cursor.h -
-	    el->el_rprompt.p_pos.h;
-	if (el->el_rprompt.p_pos.h && rhdiff < 3) {
-		re_refresh(el);	/* clear out rprompt if less than 1 char gap */
-		return;
-	}			/* else (only do at end of line, no TAB) */
-	if (iscntrl((unsigned char) c)) {	/* if control char, do caret */
-		char mc = (c == '\177') ? '?' : (c | 0100);
-		re_fastputc(el, '^');
-		re_fastputc(el, mc);
-	} else if (isprint((unsigned char) c)) {	/* normal char */
-		re_fastputc(el, c);
-	} else {
-		re_fastputc(el, '\\');
-		re_fastputc(el, (int) ((((unsigned int) c >> 6) & 7) + '0'));
-		re_fastputc(el, (int) ((((unsigned int) c >> 3) & 7) + '0'));
-		re_fastputc(el, (c & 7) + '0');
-	}
-	term__flush();
-}
-
-
-/* re_clear_display():
- *	clear the screen buffers so that new prompt starts fresh.
- */
-protected void
-re_clear_display(EditLine *el)
-{
-	int i;
-
-	el->el_cursor.v = 0;
-	el->el_cursor.h = 0;
-	for (i = 0; i < el->el_term.t_size.v; i++)
-		el->el_display[i][0] = '\0';
-	el->el_refresh.r_oldcv = 0;
-}
-
-
-/* re_clear_lines():
- *	Make sure all lines are *really* blank
- */
-protected void
-re_clear_lines(EditLine *el)
-{
-
-	if (EL_CAN_CEOL) {
-		int i;
-		term_move_to_char(el, 0);
-		for (i = 0; i <= el->el_refresh.r_oldcv; i++) {
-			/* for each line on the screen */
-			term_move_to_line(el, i);
-			term_clear_EOL(el, el->el_term.t_size.h);
-		}
-		term_move_to_line(el, 0);
-	} else {
-		term_move_to_line(el, el->el_refresh.r_oldcv);
-					/* go to last line */
-		term__putc('\r');	/* go to BOL */
-		term__putc('\n');	/* go to new line */
-	}
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/refresh.h 1:16.1.1~dfsg-1/main/editline/refresh.h
--- 1:13.23.1~dfsg-2/main/editline/refresh.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/refresh.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,63 +0,0 @@
-/*	$NetBSD: refresh.h,v 1.4 2001/01/10 07:45:42 jdolecek Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)refresh.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.refresh.h: Screen refresh functions
- */
-#ifndef _h_el_refresh
-#define	_h_el_refresh
-
-#include "histedit.h"
-
-typedef struct {
-	coord_t	r_cursor;	/* Refresh cursor position	*/
-	int	r_oldcv;	/* Vertical locations		*/
-	int	r_newcv;
-} el_refresh_t;
-
-protected void	re_putc(EditLine *, int, int);
-protected void	re_clear_lines(EditLine *);
-protected void	re_clear_display(EditLine *);
-protected void	re_refresh(EditLine *);
-protected void	re_refresh_cursor(EditLine *);
-protected void	re_fastaddc(EditLine *);
-protected void	re_goto_bottom(EditLine *);
-
-#endif /* _h_el_refresh */
diff -pruN 1:13.23.1~dfsg-2/main/editline/search.c 1:16.1.1~dfsg-1/main/editline/search.c
--- 1:13.23.1~dfsg-2/main/editline/search.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/search.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,649 +0,0 @@
-/*	$NetBSD: search.c,v 1.12 2002/03/18 16:00:58 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)search.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: search.c,v 1.12 2002/03/18 16:00:58 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * search.c: History and character search functions
- */
-#include <stdlib.h>
-#if defined(REGEX)
-#include <regex.h>
-#elif defined(REGEXP)
-#include <regexp.h>
-#endif
-#include "el.h"
-
-/*
- * Adjust cursor in vi mode to include the character under it
- */
-#define	EL_CURSOR(el) \
-    ((el)->el_line.cursor + (((el)->el_map.type == MAP_VI) && \
-			    ((el)->el_map.current == (el)->el_map.alt)))
-
-/* search_init():
- *	Initialize the search stuff
- */
-protected int
-search_init(EditLine *el)
-{
-
-	el->el_search.patbuf = (char *) el_malloc(EL_BUFSIZ);
-	if (el->el_search.patbuf == NULL)
-		return (-1);
-	el->el_search.patlen = 0;
-	el->el_search.patdir = -1;
-	el->el_search.chacha = '\0';
-	el->el_search.chadir = -1;
-	return (0);
-}
-
-
-/* search_end():
- *	Initialize the search stuff
- */
-protected void
-search_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_search.patbuf);
-	el->el_search.patbuf = NULL;
-}
-
-
-#ifdef REGEXP
-/* regerror():
- *	Handle regular expression errors
- */
-public void
-/*ARGSUSED*/
-regerror(const char *msg)
-{
-}
-#endif
-
-
-/* el_match():
- *	Return if string matches pattern
- */
-protected int
-el_match(const char *str, const char *pat)
-{
-#if defined (REGEX)
-	regex_t re;
-	int rv;
-#elif defined (REGEXP)
-	regexp *rp;
-	int rv;
-#else
-	extern char	*re_comp(const char *);
-	extern int	 re_exec(const char *);
-#endif
-
-	if (strstr(str, pat) != NULL)
-		return (1);
-
-#if defined(REGEX)
-	if (regcomp(&re, pat, 0) == 0) {
-		rv = regexec(&re, str, 0, NULL, 0) == 0;
-		regfree(&re);
-	} else {
-		rv = 0;
-	}
-	return (rv);
-#elif defined(REGEXP)
-	if ((re = regcomp(pat)) != NULL) {
-		rv = regexec(re, str);
-		free((ptr_t) re);
-	} else {
-		rv = 0;
-	}
-	return (rv);
-#else
-	if (re_comp(pat) != NULL)
-		return (0);
-	else
-		return (re_exec(str) == 1);
-#endif
-}
-
-
-/* c_hmatch():
- *	 return True if the pattern matches the prefix
- */
-protected int
-c_hmatch(EditLine *el, const char *str)
-{
-#ifdef SDEBUG
-	(void) fprintf(el->el_errfile, "match `%s' with `%s'\n",
-	    el->el_search.patbuf, str);
-#endif /* SDEBUG */
-
-	return (el_match(str, el->el_search.patbuf));
-}
-
-
-/* c_setpat():
- *	Set the history seatch pattern
- */
-protected void
-c_setpat(EditLine *el)
-{
-	if (el->el_state.lastcmd != ED_SEARCH_PREV_HISTORY &&
-	    el->el_state.lastcmd != ED_SEARCH_NEXT_HISTORY) {
-		el->el_search.patlen = EL_CURSOR(el) - el->el_line.buffer;
-		if (el->el_search.patlen >= EL_BUFSIZ)
-			el->el_search.patlen = EL_BUFSIZ - 1;
-		if (el->el_search.patlen != 0) {
-			(void) strncpy(el->el_search.patbuf, el->el_line.buffer,
-			    el->el_search.patlen);
-			el->el_search.patbuf[el->el_search.patlen] = '\0';
-		} else
-			el->el_search.patlen = strlen(el->el_search.patbuf);
-	}
-#ifdef SDEBUG
-	(void) fprintf(el->el_errfile, "\neventno = %d\n",
-	    el->el_history.eventno);
-	(void) fprintf(el->el_errfile, "patlen = %d\n", el->el_search.patlen);
-	(void) fprintf(el->el_errfile, "patbuf = \"%s\"\n",
-	    el->el_search.patbuf);
-	(void) fprintf(el->el_errfile, "cursor %d lastchar %d\n",
-	    EL_CURSOR(el) - el->el_line.buffer,
-	    el->el_line.lastchar - el->el_line.buffer);
-#endif
-}
-
-
-/* ce_inc_search():
- *	Emacs incremental search
- */
-protected el_action_t
-ce_inc_search(EditLine *el, int dir)
-{
-	static const char STRfwd[] = {'f', 'w', 'd', '\0'},
-	     STRbck[] = {'b', 'c', 'k', '\0'};
-	static char pchar = ':';/* ':' = normal, '?' = failed */
-	static char endcmd[2] = {'\0', '\0'};
-	char ch, *ocursor = el->el_line.cursor, oldpchar = pchar;
-	const char *cp;
-
-	el_action_t ret = CC_NORM;
-
-	int ohisteventno = el->el_history.eventno;
-	int oldpatlen = el->el_search.patlen;
-	int newdir = dir;
-	int done, redo;
-
-	if (el->el_line.lastchar + sizeof(STRfwd) / sizeof(char) + 2 +
-	    el->el_search.patlen >= el->el_line.limit)
-		return (CC_ERROR);
-
-	for (;;) {
-
-		if (el->el_search.patlen == 0) {	/* first round */
-			pchar = ':';
-#ifdef ANCHOR
-			el->el_search.patbuf[el->el_search.patlen++] = '.';
-			el->el_search.patbuf[el->el_search.patlen++] = '*';
-#endif
-		}
-		done = redo = 0;
-		*el->el_line.lastchar++ = '\n';
-		for (cp = (newdir == ED_SEARCH_PREV_HISTORY) ? STRbck : STRfwd;
-		    *cp; *el->el_line.lastchar++ = *cp++)
-			continue;
-		*el->el_line.lastchar++ = pchar;
-		for (cp = &el->el_search.patbuf[1];
-		    cp < &el->el_search.patbuf[el->el_search.patlen];
-		    *el->el_line.lastchar++ = *cp++)
-			continue;
-		*el->el_line.lastchar = '\0';
-		re_refresh(el);
-
-		if (el_getc(el, &ch) != 1)
-			return (ed_end_of_file(el, 0));
-
-		switch (el->el_map.current[(unsigned char) ch]) {
-		case ED_INSERT:
-		case ED_DIGIT:
-			if (el->el_search.patlen > EL_BUFSIZ - 3)
-				term_beep(el);
-			else {
-				el->el_search.patbuf[el->el_search.patlen++] =
-				    ch;
-				*el->el_line.lastchar++ = ch;
-				*el->el_line.lastchar = '\0';
-				re_refresh(el);
-			}
-			break;
-
-		case EM_INC_SEARCH_NEXT:
-			newdir = ED_SEARCH_NEXT_HISTORY;
-			redo++;
-			break;
-
-		case EM_INC_SEARCH_PREV:
-			newdir = ED_SEARCH_PREV_HISTORY;
-			redo++;
-			break;
-
-		case ED_DELETE_PREV_CHAR:
-			if (el->el_search.patlen > 1)
-				done++;
-			else
-				term_beep(el);
-			break;
-
-		default:
-			switch (ch) {
-			case 0007:	/* ^G: Abort */
-				ret = CC_ERROR;
-				done++;
-				break;
-
-			case 0027:	/* ^W: Append word */
-			/* No can do if globbing characters in pattern */
-				for (cp = &el->el_search.patbuf[1];; cp++)
-				    if (cp >= &el->el_search.patbuf[el->el_search.patlen]) {
-					el->el_line.cursor +=
-					    el->el_search.patlen - 1;
-					cp = c__next_word(el->el_line.cursor,
-					    el->el_line.lastchar, 1,
-					    ce__isword);
-					while (el->el_line.cursor < cp &&
-					    *el->el_line.cursor != '\n') {
-						if (el->el_search.patlen >
-						    EL_BUFSIZ - 3) {
-							term_beep(el);
-							break;
-						}
-						el->el_search.patbuf[el->el_search.patlen++] =
-						    *el->el_line.cursor;
-						*el->el_line.lastchar++ =
-						    *el->el_line.cursor++;
-					}
-					el->el_line.cursor = ocursor;
-					*el->el_line.lastchar = '\0';
-					re_refresh(el);
-					break;
-				    } else if (isglob(*cp)) {
-					    term_beep(el);
-					    break;
-				    }
-				break;
-
-			default:	/* Terminate and execute cmd */
-				endcmd[0] = ch;
-				el_push(el, endcmd);
-				/* FALLTHROUGH */
-
-			case 0033:	/* ESC: Terminate */
-				ret = CC_REFRESH;
-				done++;
-				break;
-			}
-			break;
-		}
-
-		while (el->el_line.lastchar > el->el_line.buffer &&
-		    *el->el_line.lastchar != '\n')
-			*el->el_line.lastchar-- = '\0';
-		*el->el_line.lastchar = '\0';
-
-		if (!done) {
-
-			/* Can't search if unmatched '[' */
-			for (cp = &el->el_search.patbuf[el->el_search.patlen-1],
-			    ch = ']';
-			    cp > el->el_search.patbuf;
-			    cp--)
-				if (*cp == '[' || *cp == ']') {
-					ch = *cp;
-					break;
-				}
-			if (el->el_search.patlen > 1 && ch != '[') {
-				if (redo && newdir == dir) {
-					if (pchar == '?') { /* wrap around */
-						el->el_history.eventno =
-						    newdir == ED_SEARCH_PREV_HISTORY ? 0 : 0x7fffffff;
-						if (hist_get(el) == CC_ERROR)
-							/* el->el_history.event
-							 * no was fixed by
-							 * first call */
-							(void) hist_get(el);
-						el->el_line.cursor = newdir ==
-						    ED_SEARCH_PREV_HISTORY ?
-						    el->el_line.lastchar :
-						    el->el_line.buffer;
-					} else
-						el->el_line.cursor +=
-						    newdir ==
-						    ED_SEARCH_PREV_HISTORY ?
-						    -1 : 1;
-				}
-#ifdef ANCHOR
-				el->el_search.patbuf[el->el_search.patlen++] =
-				    '.';
-				el->el_search.patbuf[el->el_search.patlen++] =
-				    '*';
-#endif
-				el->el_search.patbuf[el->el_search.patlen] =
-				    '\0';
-				if (el->el_line.cursor < el->el_line.buffer ||
-				    el->el_line.cursor > el->el_line.lastchar ||
-				    (ret = ce_search_line(el,
-				    &el->el_search.patbuf[1],
-				    newdir)) == CC_ERROR) {
-					/* avoid c_setpat */
-					el->el_state.lastcmd =
-					    (el_action_t) newdir;
-					ret = newdir == ED_SEARCH_PREV_HISTORY ?
-					    ed_search_prev_history(el, 0) :
-					    ed_search_next_history(el, 0);
-					if (ret != CC_ERROR) {
-						el->el_line.cursor = newdir ==
-						    ED_SEARCH_PREV_HISTORY ?
-						    el->el_line.lastchar :
-						    el->el_line.buffer;
-						(void) ce_search_line(el,
-						    &el->el_search.patbuf[1],
-						    newdir);
-					}
-				}
-				el->el_search.patbuf[--el->el_search.patlen] =
-				    '\0';
-				if (ret == CC_ERROR) {
-					term_beep(el);
-					if (el->el_history.eventno !=
-					    ohisteventno) {
-						el->el_history.eventno =
-						    ohisteventno;
-						if (hist_get(el) == CC_ERROR)
-							return (CC_ERROR);
-					}
-					el->el_line.cursor = ocursor;
-					pchar = '?';
-				} else {
-					pchar = ':';
-				}
-			}
-			ret = ce_inc_search(el, newdir);
-
-			if (ret == CC_ERROR && pchar == '?' && oldpchar == ':')
-				/*
-				 * break abort of failed search at last
-				 * non-failed
-				 */
-				ret = CC_NORM;
-
-		}
-		if (ret == CC_NORM || (ret == CC_ERROR && oldpatlen == 0)) {
-			/* restore on normal return or error exit */
-			pchar = oldpchar;
-			el->el_search.patlen = oldpatlen;
-			if (el->el_history.eventno != ohisteventno) {
-				el->el_history.eventno = ohisteventno;
-				if (hist_get(el) == CC_ERROR)
-					return (CC_ERROR);
-			}
-			el->el_line.cursor = ocursor;
-			if (ret == CC_ERROR)
-				re_refresh(el);
-		}
-		if (done || ret != CC_NORM)
-			return (ret);
-	}
-}
-
-
-/* cv_search():
- *	Vi search.
- */
-protected el_action_t
-cv_search(EditLine *el, int dir)
-{
-	char ch;
-	char tmpbuf[EL_BUFSIZ];
-	int tmplen;
-
-	tmplen = 0;
-#ifdef ANCHOR
-	tmpbuf[tmplen++] = '.';
-	tmpbuf[tmplen++] = '*';
-#endif
-
-	el->el_line.buffer[0] = '\0';
-	el->el_line.lastchar = el->el_line.buffer;
-	el->el_line.cursor = el->el_line.buffer;
-	el->el_search.patdir = dir;
-
-	c_insert(el, 2);	/* prompt + '\n' */
-	*el->el_line.cursor++ = '\n';
-	*el->el_line.cursor++ = dir == ED_SEARCH_PREV_HISTORY ? '/' : '?';
-	re_refresh(el);
-
-#ifdef ANCHOR
-#define	LEN	2
-#else
-#define	LEN	0
-#endif
-
-	tmplen = c_gets(el, &tmpbuf[LEN]) + LEN;
-	ch = tmpbuf[tmplen];
-	tmpbuf[tmplen] = '\0';
-
-	if (tmplen == LEN) {
-		/*
-		 * Use the old pattern, but wild-card it.
-		 */
-		if (el->el_search.patlen == 0) {
-			el->el_line.buffer[0] = '\0';
-			el->el_line.lastchar = el->el_line.buffer;
-			el->el_line.cursor = el->el_line.buffer;
-			re_refresh(el);
-			return (CC_ERROR);
-		}
-#ifdef ANCHOR
-		if (el->el_search.patbuf[0] != '.' &&
-		    el->el_search.patbuf[0] != '*') {
-			(void) strncpy(tmpbuf, el->el_search.patbuf,
-			    sizeof(tmpbuf) - 1);
-			el->el_search.patbuf[0] = '.';
-			el->el_search.patbuf[1] = '*';
-			(void) strncpy(&el->el_search.patbuf[2], tmpbuf,
-			    EL_BUFSIZ - 3);
-			el->el_search.patlen++;
-			el->el_search.patbuf[el->el_search.patlen++] = '.';
-			el->el_search.patbuf[el->el_search.patlen++] = '*';
-			el->el_search.patbuf[el->el_search.patlen] = '\0';
-		}
-#endif
-	} else {
-#ifdef ANCHOR
-		tmpbuf[tmplen++] = '.';
-		tmpbuf[tmplen++] = '*';
-#endif
-		tmpbuf[tmplen] = '\0';
-		(void) strncpy(el->el_search.patbuf, tmpbuf, EL_BUFSIZ - 1);
-		el->el_search.patlen = tmplen;
-	}
-	el->el_state.lastcmd = (el_action_t) dir;	/* avoid c_setpat */
-	el->el_line.cursor = el->el_line.lastchar = el->el_line.buffer;
-	if ((dir == ED_SEARCH_PREV_HISTORY ? ed_search_prev_history(el, 0) :
-		ed_search_next_history(el, 0)) == CC_ERROR) {
-		re_refresh(el);
-		return (CC_ERROR);
-	} else {
-		if (ch == 0033) {
-			re_refresh(el);
-			*el->el_line.lastchar++ = '\n';
-			*el->el_line.lastchar = '\0';
-			re_goto_bottom(el);
-			return (CC_NEWLINE);
-		} else
-			return (CC_REFRESH);
-	}
-}
-
-
-/* ce_search_line():
- *	Look for a pattern inside a line
- */
-protected el_action_t
-ce_search_line(EditLine *el, char *pattern, int dir)
-{
-	char *cp;
-
-	if (dir == ED_SEARCH_PREV_HISTORY) {
-		for (cp = el->el_line.cursor; cp >= el->el_line.buffer; cp--)
-			if (el_match(cp, pattern)) {
-				el->el_line.cursor = cp;
-				return (CC_NORM);
-			}
-		return (CC_ERROR);
-	} else {
-		for (cp = el->el_line.cursor; *cp != '\0' &&
-		    cp < el->el_line.limit; cp++)
-			if (el_match(cp, pattern)) {
-				el->el_line.cursor = cp;
-				return (CC_NORM);
-			}
-		return (CC_ERROR);
-	}
-}
-
-
-/* cv_repeat_srch():
- *	Vi repeat search
- */
-protected el_action_t
-cv_repeat_srch(EditLine *el, int c)
-{
-
-#ifdef SDEBUG
-	(void) fprintf(el->el_errfile, "dir %d patlen %d patbuf %s\n",
-	    c, el->el_search.patlen, el->el_search.patbuf);
-#endif
-
-	el->el_state.lastcmd = (el_action_t) c;	/* Hack to stop c_setpat */
-	el->el_line.lastchar = el->el_line.buffer;
-
-	switch (c) {
-	case ED_SEARCH_NEXT_HISTORY:
-		return (ed_search_next_history(el, 0));
-	case ED_SEARCH_PREV_HISTORY:
-		return (ed_search_prev_history(el, 0));
-	default:
-		return (CC_ERROR);
-	}
-}
-
-
-/* cv_csearch_back():
- *	Vi character search reverse
- */
-protected el_action_t
-cv_csearch_back(EditLine *el, int ch, int count, int tflag)
-{
-	char *cp;
-
-	cp = el->el_line.cursor;
-	while (count--) {
-		if (*cp == ch)
-			cp--;
-		while (cp > el->el_line.buffer && *cp != ch)
-			cp--;
-	}
-
-	if (cp < el->el_line.buffer || (cp == el->el_line.buffer && *cp != ch))
-		return (CC_ERROR);
-
-	if (*cp == ch && tflag)
-		cp++;
-
-	el->el_line.cursor = cp;
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		el->el_line.cursor++;
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	re_refresh_cursor(el);
-	return (CC_NORM);
-}
-
-
-/* cv_csearch_fwd():
- *	Vi character search forward
- */
-protected el_action_t
-cv_csearch_fwd(EditLine *el, int ch, int count, int tflag)
-{
-	char *cp;
-
-	cp = el->el_line.cursor;
-	while (count--) {
-		if (*cp == ch)
-			cp++;
-		while (cp < el->el_line.lastchar && *cp != ch)
-			cp++;
-	}
-
-	if (cp >= el->el_line.lastchar)
-		return (CC_ERROR);
-
-	if (*cp == ch && tflag)
-		cp--;
-
-	el->el_line.cursor = cp;
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		el->el_line.cursor++;
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	re_refresh_cursor(el);
-	return (CC_NORM);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/search.h 1:16.1.1~dfsg-1/main/editline/search.h
--- 1:13.23.1~dfsg-2/main/editline/search.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/search.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,70 +0,0 @@
-/*	$NetBSD: search.h,v 1.5 2000/09/04 22:06:32 lukem Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)search.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.search.h: Line and history searching utilities
- */
-#ifndef _h_el_search
-#define	_h_el_search
-
-#include "histedit.h"
-
-typedef struct el_search_t {
-	char	*patbuf;		/* The pattern buffer		*/
-	size_t	 patlen;		/* Length of the pattern buffer	*/
-	int	 patdir;		/* Direction of the last search	*/
-	int	 chadir;		/* Character search direction	*/
-	char	 chacha;		/* Character we are looking for	*/
-} el_search_t;
-
-
-protected int		el_match(const char *, const char *);
-protected int		search_init(EditLine *);
-protected void		search_end(EditLine *);
-protected int		c_hmatch(EditLine *, const char *);
-protected void		c_setpat(EditLine *);
-protected el_action_t	ce_inc_search(EditLine *, int);
-protected el_action_t	cv_search(EditLine *, int);
-protected el_action_t	ce_search_line(EditLine *, char *, int);
-protected el_action_t	cv_repeat_srch(EditLine *, int);
-protected el_action_t	cv_csearch_back(EditLine *, int, int, int);
-protected el_action_t	cv_csearch_fwd(EditLine *, int, int, int);
-
-#endif /* _h_el_search */
diff -pruN 1:13.23.1~dfsg-2/main/editline/sig.c 1:16.1.1~dfsg-1/main/editline/sig.c
--- 1:13.23.1~dfsg-2/main/editline/sig.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/sig.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,198 +0,0 @@
-/*	$NetBSD: sig.c,v 1.9 2002/03/18 16:00:58 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)sig.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: sig.c,v 1.9 2002/03/18 16:00:58 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * sig.c: Signal handling stuff.
- *	  our policy is to trap all signals, set a good state
- *	  and pass the ball to our caller.
- */
-#include "el.h"
-#include <stdlib.h>
-
-private EditLine *sel = NULL;
-
-private const int sighdl[] = {
-#define	_DO(a)	(a),
-	ALLSIGS
-#undef	_DO
-	- 1
-};
-
-private void sig_handler(int);
-
-/* sig_handler():
- *	This is the handler called for all signals
- *	XXX: we cannot pass any data so we just store the old editline
- *	state in a private variable
- */
-private void
-sig_handler(int signo)
-{
-	int i;
-	sigset_t nset, oset;
-
-	(void) sigemptyset(&nset);
-	(void) sigaddset(&nset, signo);
-	(void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-	switch (signo) {
-	case SIGCONT:
-		tty_rawmode(sel);
-		if (ed_redisplay(sel, 0) == CC_REFRESH)
-			re_refresh(sel);
-		term__flush();
-		break;
-
-	case SIGWINCH:
-		el_resize(sel);
-		break;
-
-	default:
-		tty_cookedmode(sel);
-		break;
-	}
-
-	for (i = 0; sighdl[i] != -1; i++)
-		if (signo == sighdl[i])
-			break;
-
-	(void) signal(signo, sel->el_signal[i]);
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-	(void) kill(0, signo);
-}
-
-
-/* sig_init():
- *	Initialize all signal stuff
- */
-protected int
-sig_init(EditLine *el)
-{
-	int i;
-	sigset_t nset, oset;
-
-	(void) sigemptyset(&nset);
-#define	_DO(a) (void) sigaddset(&nset, a);
-	ALLSIGS
-#undef	_DO
-	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-#define	SIGSIZE (sizeof(sighdl) / sizeof(sighdl[0]) * sizeof(sig_t))
-
-	el->el_signal = (sig_t *) el_malloc(SIGSIZE);
-	if (el->el_signal == NULL)
-		return (-1);
-	for (i = 0; sighdl[i] != -1; i++)
-		el->el_signal[i] = SIG_ERR;
-
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-
-	return (0);
-}
-
-
-/* sig_end():
- *	Clear all signal stuff
- */
-protected void
-sig_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_signal);
-	el->el_signal = NULL;
-}
-
-
-/* sig_set():
- *	set all the signal handlers
- */
-protected void
-sig_set(EditLine *el)
-{
-	int i;
-	sigset_t nset, oset;
-
-	(void) sigemptyset(&nset);
-#define	_DO(a) (void) sigaddset(&nset, a);
-	ALLSIGS
-#undef	_DO
-	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-	for (i = 0; sighdl[i] != -1; i++) {
-		sig_t s;
-		/* This could happen if we get interrupted */
-		if ((s = signal(sighdl[i], sig_handler)) != sig_handler)
-			el->el_signal[i] = s;
-	}
-	sel = el;
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-}
-
-
-/* sig_clr():
- *	clear all the signal handlers
- */
-protected void
-sig_clr(EditLine *el)
-{
-	int i;
-	sigset_t nset, oset;
-
-	(void) sigemptyset(&nset);
-#define	_DO(a) (void) sigaddset(&nset, a);
-	ALLSIGS
-#undef	_DO
-	    (void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-	for (i = 0; sighdl[i] != -1; i++)
-		if (el->el_signal[i] != SIG_ERR)
-			(void) signal(sighdl[i], el->el_signal[i]);
-
-	sel = NULL;		/* we are going to die if the handler is
-				 * called */
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/sig.h 1:16.1.1~dfsg-1/main/editline/sig.h
--- 1:13.23.1~dfsg-2/main/editline/sig.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/sig.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,72 +0,0 @@
-/*	$NetBSD: sig.h,v 1.3 2000/09/04 22:06:32 lukem Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)sig.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.sig.h: Signal handling functions
- */
-#ifndef _h_el_sig
-#define	_h_el_sig
-
-#include <signal.h>
-
-#include "histedit.h"
-
-/*
- * Define here all the signals we are going to handle
- * The _DO macro is used to iterate in the source code
- */
-#define	ALLSIGS		\
-	_DO(SIGINT)	\
-	_DO(SIGTSTP)	\
-	_DO(SIGSTOP)	\
-	_DO(SIGQUIT)	\
-	_DO(SIGHUP)	\
-	_DO(SIGTERM)	\
-	_DO(SIGCONT)	\
-	_DO(SIGWINCH)
-
-typedef sig_t	*el_signal_t;
-
-protected void	sig_end(EditLine*);
-protected int	sig_init(EditLine*);
-protected void	sig_set(EditLine*);
-protected void	sig_clr(EditLine*);
-
-#endif /* _h_el_sig */
diff -pruN 1:13.23.1~dfsg-2/main/editline/sys.h 1:16.1.1~dfsg-1/main/editline/sys.h
--- 1:13.23.1~dfsg-2/main/editline/sys.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/sys.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,135 +0,0 @@
-/*	$NetBSD: sys.h,v 1.5 2002/03/18 16:00:59 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)sys.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * sys.h: Put all the stupid compiler and system dependencies here...
- */
-#ifndef _h_sys
-#define	_h_sys
-
-#ifndef public
-# define public		/* Externally visible functions/variables */
-#endif
-
-#ifndef private
-# define private	static	/* Always hidden internals */
-#endif
-
-#ifndef protected
-# define protected	/* Redefined from elsewhere to "static" */
-			/* When we want to hide everything	*/
-#endif
-
-#ifndef _PTR_T
-# define _PTR_T
-typedef void	*ptr_t;
-#endif
-
-#ifndef _IOCTL_T
-# define _IOCTL_T
-typedef void	*ioctl_t;
-#endif
-
-#include <stdio.h>
-
-#ifndef HAVE_FGETLN
-#define	fgetln libedit_fgetln
-char	*fgetln(FILE *fp, size_t *len);
-#endif
-
-#ifndef HAVE_STRLCPY
-#undef strlcpy /* We define this in top-level Asterisk to point towards ast_copy_string */
-#define	strlcpy libedit_strlcpy
-size_t strlcpy(char *dst, const char *src, size_t siz);
-#endif
-
-#ifndef HAVE_STRLCAT
-#undef strlcat /* We define this in top-level Asterisk to point towards the ast_str_* APIs */
-#define	strlcat libedit_strlcat
-size_t strlcat(char *dst, const char *src, size_t siz);
-#endif
-
-#define	REGEX		/* Use POSIX.2 regular expression functions */
-#undef	REGEXP		/* Use UNIX V8 regular expression functions */
-
-#ifdef SUNOS
-# undef REGEX
-# undef REGEXP
-/* # include <malloc.h> XXX Removed for Solaris build XXX */
-typedef void (*sig_t)(int);
-# ifdef __GNUC__
-/*
- * Broken hdrs.
- */
-#ifndef SOLARIS
-extern int	tgetent(const char *bp, char *name);
-extern int	tgetflag(const char *id);
-extern int	tgetnum(const char *id);
-extern char    *tgetstr(const char *id, char **area);
-#endif
-extern char    *tgoto(const char *cap, int col, int row);
-extern int	tputs(const char *str, int affcnt, int (*putc)(int));
-extern char    *getenv(const char *);
-extern int	fprintf(FILE *, const char *, ...);
-extern int	sigsetmask(int);
-extern int	sigblock(int);
-extern int	fputc(int, FILE *);
-extern int	fgetc(FILE *);
-extern int	fflush(FILE *);
-extern int	tolower(int);
-extern int	toupper(int);
-extern int	errno, sys_nerr;
-extern char	*sys_errlist[];
-extern void	perror(const char *);
-#  include <string.h>
-#  define strerror(e)	sys_errlist[e]
-# endif
-# ifdef SABER
-extern ptr_t    memcpy(ptr_t, const ptr_t, size_t);
-extern ptr_t    memset(ptr_t, int, size_t);
-# endif
-extern char    *fgetline(FILE *, int *);
-#endif
-
-#ifdef HAVE_SYS_CDEFS_H
-#include <sys/cdefs.h>
-#endif
-
-#endif /* _h_sys */
diff -pruN 1:13.23.1~dfsg-2/main/editline/term.c 1:16.1.1~dfsg-1/main/editline/term.c
--- 1:13.23.1~dfsg-2/main/editline/term.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/term.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,1599 +0,0 @@
-/*	$NetBSD: term.c,v 1.35 2002/03/18 16:00:59 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)term.c	8.2 (Berkeley) 4/30/95";
-#else
-__RCSID("$NetBSD: term.c,v 1.35 2002/03/18 16:00:59 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * term.c: Editor/termcap-curses interface
- *	   We have to declare a static variable here, since the
- *	   termcap putchar routine does not take an argument!
- */
-#include <stdio.h>
-#include <signal.h>
-#include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#ifdef HAVE_TERMCAP_H
-#include <termcap.h>
-#endif
-#ifdef HAVE_CURSES_H
-#include <curses.h>
-#endif
-#ifdef HAVE_NCURSES_H
-#include <ncurses.h>
-#endif
-#if defined(HAVE_TERM_H)
-/* Can not use /usr/include/term.h because of a lot of incompatibilities, so just define some prototypes */
-#include "term.h"
-#endif /* defined(HAVE_TERM_H) */
-#if defined(HAVE_TERM_H) || !(defined(HAVE_TERMCAP_H) || defined(HAVE_CURSES_H) || defined(HAVE_NCURSES_H))
-extern int tgetent(char *, const char *);
-extern int tgetflag(const char *);
-extern int tgetnum(const char *);
-extern char *tgetstr(const char *, char **);
-extern int tputs (const char *, int, int (*)(int));
-extern char *tgoto (const char *, int, int);
-#endif /* defined(HAVE_TERM_H) or none_of_the_others */
-#include <sys/types.h>
-#include <sys/ioctl.h>
-
-#include "el.h"
-
-/*
- * IMPORTANT NOTE: these routines are allowed to look at the current screen
- * and the current possition assuming that it is correct.  If this is not
- * true, then the update will be WRONG!  This is (should be) a valid
- * assumption...
- */
-
-#define	TC_BUFSIZE	2048
-
-#define	GoodStr(a)	(el->el_term.t_str[a] != NULL && \
-			    el->el_term.t_str[a][0] != '\0')
-#define	Str(a)		el->el_term.t_str[a]
-#define	Val(a)		el->el_term.t_val[a]
-
-#ifdef notdef
-private const struct {
-	const char *b_name;
-	int b_rate;
-} baud_rate[] = {
-#ifdef B0
-	{ "0", B0 },
-#endif
-#ifdef B50
-	{ "50", B50 },
-#endif
-#ifdef B75
-	{ "75", B75 },
-#endif
-#ifdef B110
-	{ "110", B110 },
-#endif
-#ifdef B134
-	{ "134", B134 },
-#endif
-#ifdef B150
-	{ "150", B150 },
-#endif
-#ifdef B200
-	{ "200", B200 },
-#endif
-#ifdef B300
-	{ "300", B300 },
-#endif
-#ifdef B600
-	{ "600", B600 },
-#endif
-#ifdef B900
-	{ "900", B900 },
-#endif
-#ifdef B1200
-	{ "1200", B1200 },
-#endif
-#ifdef B1800
-	{ "1800", B1800 },
-#endif
-#ifdef B2400
-	{ "2400", B2400 },
-#endif
-#ifdef B3600
-	{ "3600", B3600 },
-#endif
-#ifdef B4800
-	{ "4800", B4800 },
-#endif
-#ifdef B7200
-	{ "7200", B7200 },
-#endif
-#ifdef B9600
-	{ "9600", B9600 },
-#endif
-#ifdef EXTA
-	{ "19200", EXTA },
-#endif
-#ifdef B19200
-	{ "19200", B19200 },
-#endif
-#ifdef EXTB
-	{ "38400", EXTB },
-#endif
-#ifdef B38400
-	{ "38400", B38400 },
-#endif
-	{ NULL, 0 }
-};
-#endif
-
-private const struct termcapstr {
-	const char *name;
-	const char *long_name;
-} tstr[] = {
-#define	T_al	0
-	{ "al", "add new blank line" },
-#define	T_bl	1
-	{ "bl", "audible bell" },
-#define	T_cd	2
-	{ "cd", "clear to bottom" },
-#define	T_ce	3
-	{ "ce", "clear to end of line" },
-#define	T_ch	4
-	{ "ch", "cursor to horiz pos" },
-#define	T_cl	5
-	{ "cl", "clear screen" },
-#define	T_dc	6
-	{ "dc", "delete a character" },
-#define	T_dl	7
-	{ "dl", "delete a line" },
-#define	T_dm	8
-	{ "dm", "start delete mode" },
-#define	T_ed	9
-	{ "ed", "end delete mode" },
-#define	T_ei	10
-	{ "ei", "end insert mode" },
-#define	T_fs	11
-	{ "fs", "cursor from status line" },
-#define	T_ho	12
-	{ "ho", "home cursor" },
-#define	T_ic	13
-	{ "ic", "insert character" },
-#define	T_im	14
-	{ "im", "start insert mode" },
-#define	T_ip	15
-	{ "ip", "insert padding" },
-#define	T_kd	16
-	{ "kd", "sends cursor down" },
-#define	T_kl	17
-	{ "kl", "sends cursor left" },
-#define	T_kr	18
-	{ "kr", "sends cursor right" },
-#define	T_ku	19
-	{ "ku", "sends cursor up" },
-#define	T_md	20
-	{ "md", "begin bold" },
-#define	T_me	21
-	{ "me", "end attributes" },
-#define	T_nd	22
-	{ "nd", "non destructive space" },
-#define	T_se	23
-	{ "se", "end standout" },
-#define	T_so	24
-	{ "so", "begin standout" },
-#define	T_ts	25
-	{ "ts", "cursor to status line" },
-#define	T_up	26
-	{ "up", "cursor up one" },
-#define	T_us	27
-	{ "us", "begin underline" },
-#define	T_ue	28
-	{ "ue", "end underline" },
-#define	T_vb	29
-	{ "vb", "visible bell" },
-#define	T_DC	30
-	{ "DC", "delete multiple chars" },
-#define	T_DO	31
-	{ "DO", "cursor down multiple" },
-#define	T_IC	32
-	{ "IC", "insert multiple chars" },
-#define	T_LE	33
-	{ "LE", "cursor left multiple" },
-#define	T_RI	34
-	{ "RI", "cursor right multiple" },
-#define	T_UP	35
-	{ "UP", "cursor up multiple" },
-#define	T_kh	36
-	{ "kh", "send cursor home" },
-#define	T_at7	37
-	{ "@7", "send cursor end" },
-#define	T_str	38
-	{ NULL, NULL }
-};
-
-private const struct termcapval {
-	const char *name;
-	const char *long_name;
-} tval[] = {
-#define	T_am	0
-	{ "am", "has automatic margins" },
-#define	T_pt	1
-	{ "pt", "has physical tabs" },
-#define	T_li	2
-	{ "li", "Number of lines" },
-#define	T_co	3
-	{ "co", "Number of columns" },
-#define	T_km	4
-	{ "km", "Has meta key" },
-#define	T_xt	5
-	{ "xt", "Tab chars destructive" },
-#define	T_xn	6
-	{ "xn", "newline ignored at right margin" },
-#define	T_MT	7
-	{ "MT", "Has meta key" },			/* XXX? */
-#define	T_val	8
-	{ NULL, NULL, }
-};
-/* do two or more of the attributes use me */
-
-private void	term_setflags(EditLine *);
-private int	term_rebuffer_display(EditLine *);
-private void	term_free_display(EditLine *);
-private int	term_alloc_display(EditLine *);
-private void	term_alloc(EditLine *, const struct termcapstr *, const char *);
-private void	term_init_arrow(EditLine *);
-private void	term_reset_arrow(EditLine *);
-
-
-private FILE *term_outfile = NULL;	/* XXX: How do we fix that? */
-
-
-/* term_setflags():
- *	Set the terminal capability flags
- */
-private void
-term_setflags(EditLine *el)
-{
-	EL_FLAGS = 0;
-	if (el->el_tty.t_tabs)
-		EL_FLAGS |= (Val(T_pt) && !Val(T_xt)) ? TERM_CAN_TAB : 0;
-
-	EL_FLAGS |= (Val(T_km) || Val(T_MT)) ? TERM_HAS_META : 0;
-	EL_FLAGS |= GoodStr(T_ce) ? TERM_CAN_CEOL : 0;
-	EL_FLAGS |= (GoodStr(T_dc) || GoodStr(T_DC)) ? TERM_CAN_DELETE : 0;
-	EL_FLAGS |= (GoodStr(T_im) || GoodStr(T_ic) || GoodStr(T_IC)) ?
-	    TERM_CAN_INSERT : 0;
-	EL_FLAGS |= (GoodStr(T_up) || GoodStr(T_UP)) ? TERM_CAN_UP : 0;
-	EL_FLAGS |= Val(T_am) ? TERM_HAS_AUTO_MARGINS : 0;
-	EL_FLAGS |= Val(T_xn) ? TERM_HAS_MAGIC_MARGINS : 0;
-
-	if (GoodStr(T_me) && GoodStr(T_ue))
-		EL_FLAGS |= (strcmp(Str(T_me), Str(T_ue)) == 0) ?
-		    TERM_CAN_ME : 0;
-	else
-		EL_FLAGS &= ~TERM_CAN_ME;
-	if (GoodStr(T_me) && GoodStr(T_se))
-		EL_FLAGS |= (strcmp(Str(T_me), Str(T_se)) == 0) ?
-		    TERM_CAN_ME : 0;
-
-
-#ifdef DEBUG_SCREEN
-	if (!EL_CAN_UP) {
-		(void) fprintf(el->el_errfile,
-		    "WARNING: Your terminal cannot move up.\n");
-		(void) fprintf(el->el_errfile,
-		    "Editing may be odd for long lines.\n");
-	}
-	if (!EL_CAN_CEOL)
-		(void) fprintf(el->el_errfile, "no clear EOL capability.\n");
-	if (!EL_CAN_DELETE)
-		(void) fprintf(el->el_errfile, "no delete char capability.\n");
-	if (!EL_CAN_INSERT)
-		(void) fprintf(el->el_errfile, "no insert char capability.\n");
-#endif /* DEBUG_SCREEN */
-}
-
-
-/* term_init():
- *	Initialize the terminal stuff
- */
-protected int
-term_init(EditLine *el)
-{
-
-	el->el_term.t_buf = (char *) el_malloc(TC_BUFSIZE);
-	if (el->el_term.t_buf == NULL)
-		return (-1);
-	el->el_term.t_cap = (char *) el_malloc(TC_BUFSIZE);
-	if (el->el_term.t_cap == NULL)
-		return (-1);
-	el->el_term.t_fkey = (fkey_t *) el_malloc(A_K_NKEYS * sizeof(fkey_t));
-	if (el->el_term.t_fkey == NULL)
-		return (-1);
-	el->el_term.t_loc = 0;
-	el->el_term.t_str = (char **) el_malloc(T_str * sizeof(char *));
-	if (el->el_term.t_str == NULL)
-		return (-1);
-	(void) memset(el->el_term.t_str, 0, T_str * sizeof(char *));
-	el->el_term.t_val = (int *) el_malloc(T_val * sizeof(int));
-	if (el->el_term.t_val == NULL)
-		return (-1);
-	(void) memset(el->el_term.t_val, 0, T_val * sizeof(int));
-	term_outfile = el->el_outfile;
-	(void) term_set(el, NULL);
-	term_init_arrow(el);
-	return (0);
-}
-/* term_end():
- *	Clean up the terminal stuff
- */
-protected void
-term_end(EditLine *el)
-{
-
-	el_free((ptr_t) el->el_term.t_buf);
-	el->el_term.t_buf = NULL;
-	el_free((ptr_t) el->el_term.t_cap);
-	el->el_term.t_cap = NULL;
-	el_free((ptr_t) el->el_term.t_fkey);
-	el->el_term.t_fkey = NULL;
-	el->el_term.t_loc = 0;
-	el_free((ptr_t) el->el_term.t_str);
-	el->el_term.t_str = NULL;
-	el_free((ptr_t) el->el_term.t_val);
-	el->el_term.t_val = NULL;
-	term_free_display(el);
-}
-
-
-/* term_alloc():
- *	Maintain a string pool for termcap strings
- */
-private void
-term_alloc(EditLine *el, const struct termcapstr *t, const char *cap)
-{
-	char termbuf[TC_BUFSIZE];
-	int tlen, clen;
-	char **tlist = el->el_term.t_str;
-	char **tmp, **str = &tlist[t - tstr];
-
-	if (cap == NULL || *cap == '\0') {
-		*str = NULL;
-		return;
-	} else
-		clen = strlen(cap);
-
-	tlen = *str == NULL ? 0 : strlen(*str);
-
-	/*
-         * New string is shorter; no need to allocate space
-         */
-	if (clen <= tlen) {
-		(void) strcpy(*str, cap);	/* XXX strcpy is safe */
-		return;
-	}
-	/*
-         * New string is longer; see if we have enough space to append
-         */
-	if (el->el_term.t_loc + 3 < TC_BUFSIZE) {
-						/* XXX strcpy is safe */
-		(void) strcpy(*str = &el->el_term.t_buf[el->el_term.t_loc],
-		    cap);
-		el->el_term.t_loc += clen + 1;	/* one for \0 */
-		return;
-	}
-	/*
-         * Compact our buffer; no need to check compaction, cause we know it
-         * fits...
-         */
-	tlen = 0;
-	for (tmp = tlist; tmp < &tlist[T_str]; tmp++)
-		if (*tmp != NULL && **tmp != '\0' && *tmp != *str) {
-			char *ptr;
-
-			for (ptr = *tmp; *ptr != '\0'; termbuf[tlen++] = *ptr++)
-				continue;
-			termbuf[tlen++] = '\0';
-		}
-	memcpy(el->el_term.t_buf, termbuf, TC_BUFSIZE);
-	el->el_term.t_loc = tlen;
-	if (el->el_term.t_loc + 3 >= TC_BUFSIZE) {
-		(void) fprintf(el->el_errfile,
-		    "Out of termcap string space.\n");
-		return;
-	}
-					/* XXX strcpy is safe */
-	(void) strcpy(*str = &el->el_term.t_buf[el->el_term.t_loc], cap);
-	el->el_term.t_loc += clen + 1;	/* one for \0 */
-	return;
-}
-
-
-/* term_rebuffer_display():
- *	Rebuffer the display after the screen changed size
- */
-private int
-term_rebuffer_display(EditLine *el)
-{
-	coord_t *c = &el->el_term.t_size;
-
-	term_free_display(el);
-
-	c->h = Val(T_co);
-	c->v = Val(T_li);
-
-	if (term_alloc_display(el) == -1)
-		return (-1);
-	return (0);
-}
-
-
-/* term_alloc_display():
- *	Allocate a new display.
- */
-private int
-term_alloc_display(EditLine *el)
-{
-	int i, j;
-	char **b;
-	coord_t *c = &el->el_term.t_size;
-
-	b = (char **) el_malloc((size_t) (sizeof(char *) * (c->v + 1)));
-	if (b == NULL)
-		return (-1);
-	for (i = 0; i < c->v; i++) {
-		b[i] = (char *) el_malloc((size_t) (sizeof(char) * (c->h + 1)));
-		if (b[i] == NULL) {
-			for (j = 0; j < i; j++) {
-				el_free(b[j]);
-			}
-			el_free(b);
-			return (-1);
-		}
-	}
-	b[c->v] = NULL;
-	el->el_display = b;
-
-	b = (char **) el_malloc((size_t) (sizeof(char *) * (c->v + 1)));
-	if (b == NULL)
-		return (-1);
-	for (i = 0; i < c->v; i++) {
-		b[i] = (char *) el_malloc((size_t) (sizeof(char) * (c->h + 1)));
-		if (b[i] == NULL) {
-			for (j = 0; j < i; j++) {
-				el_free(b[j]);
-			}
-			el_free(b);
-			return (-1);
-		}
-	}
-	b[c->v] = NULL;
-	el->el_vdisplay = b;
-	return (0);
-}
-
-
-/* term_free_display():
- *	Free the display buffers
- */
-private void
-term_free_display(EditLine *el)
-{
-	char **b;
-	char **bufp;
-
-	b = el->el_display;
-	el->el_display = NULL;
-	if (b != NULL) {
-		for (bufp = b; *bufp != NULL; bufp++)
-			el_free((ptr_t) * bufp);
-		el_free((ptr_t) b);
-	}
-	b = el->el_vdisplay;
-	el->el_vdisplay = NULL;
-	if (b != NULL) {
-		for (bufp = b; *bufp != NULL; bufp++)
-			el_free((ptr_t) * bufp);
-		el_free((ptr_t) b);
-	}
-}
-
-
-/* term_move_to_line():
- *	move to line <where> (first line == 0)
- * 	as efficiently as possible
- */
-protected void
-term_move_to_line(EditLine *el, int where)
-{
-	int del;
-
-	if (where == el->el_cursor.v)
-		return;
-
-	if (where > el->el_term.t_size.v) {
-#ifdef DEBUG_SCREEN
-		(void) fprintf(el->el_errfile,
-		    "term_move_to_line: where is ridiculous: %d\r\n", where);
-#endif /* DEBUG_SCREEN */
-		return;
-	}
-	if ((del = where - el->el_cursor.v) > 0) {
-		while (del > 0) {
-			if (EL_HAS_AUTO_MARGINS &&
-			    el->el_display[el->el_cursor.v][0] != '\0') {
-				/* move without newline */
-				term_move_to_char(el, el->el_term.t_size.h - 1);
-				term_overwrite(el,
-				    &el->el_display[el->el_cursor.v][el->el_cursor.h],
-				    1);
-				/* updates Cursor */
-				del--;
-			} else {
-				if ((del > 1) && GoodStr(T_DO)) {
-					(void) tputs(tgoto(Str(T_DO), del, del),
-					    del, term__putc);
-					del = 0;
-				} else {
-					for (; del > 0; del--)
-						term__putc('\n');
-					/* because the \n will become \r\n */
-					el->el_cursor.h = 0;
-				}
-			}
-		}
-	} else {		/* del < 0 */
-		if (GoodStr(T_UP) && (-del > 1 || !GoodStr(T_up)))
-			(void) tputs(tgoto(Str(T_UP), -del, -del), -del,
-			    term__putc);
-		else {
-			if (GoodStr(T_up))
-				for (; del < 0; del++)
-					(void) tputs(Str(T_up), 1, term__putc);
-		}
-	}
-	el->el_cursor.v = where;/* now where is here */
-}
-
-
-/* term_move_to_char():
- *	Move to the character position specified
- */
-protected void
-term_move_to_char(EditLine *el, int where)
-{
-	int del, i;
-
-mc_again:
-	if (where == el->el_cursor.h)
-		return;
-
-	if (where > el->el_term.t_size.h) {
-#ifdef DEBUG_SCREEN
-		(void) fprintf(el->el_errfile,
-		    "term_move_to_char: where is riduculous: %d\r\n", where);
-#endif /* DEBUG_SCREEN */
-		return;
-	}
-	if (!where) {		/* if where is first column */
-		term__putc('\r');	/* do a CR */
-		el->el_cursor.h = 0;
-		return;
-	}
-	del = where - el->el_cursor.h;
-
-	if ((del < -4 || del > 4) && GoodStr(T_ch))
-		/* go there directly */
-		(void) tputs(tgoto(Str(T_ch), where, where), where, term__putc);
-	else {
-		if (del > 0) {	/* moving forward */
-			if ((del > 4) && GoodStr(T_RI))
-				(void) tputs(tgoto(Str(T_RI), del, del),
-				    del, term__putc);
-			else {
-					/* if I can do tabs, use them */
-				if (EL_CAN_TAB) {
-					if ((el->el_cursor.h & 0370) !=
-					    (where & 0370)) {
-						/* if not within tab stop */
-						for (i =
-						    (el->el_cursor.h & 0370);
-						    i < (where & 0370);
-						    i += 8)
-							term__putc('\t');
-							/* then tab over */
-						el->el_cursor.h = where & 0370;
-					}
-				}
-				/*
-				 * it's usually cheaper to just write the
-				 * chars, so we do.
-				 */
-				/*
-				 * NOTE THAT term_overwrite() WILL CHANGE
-				 * el->el_cursor.h!!!
-				 */
-				term_overwrite(el,
-				    &el->el_display[el->el_cursor.v][el->el_cursor.h],
-				    where - el->el_cursor.h);
-
-			}
-		} else {	/* del < 0 := moving backward */
-			if ((-del > 4) && GoodStr(T_LE))
-				(void) tputs(tgoto(Str(T_LE), -del, -del),
-				    -del, term__putc);
-			else {	/* can't go directly there */
-				/*
-				 * if the "cost" is greater than the "cost"
-				 * from col 0
-				 */
-				if (EL_CAN_TAB ?
-				    (-del > (((unsigned int) where >> 3) +
-				     (where & 07)))
-				    : (-del > where)) {
-					term__putc('\r');	/* do a CR */
-					el->el_cursor.h = 0;
-					goto mc_again;	/* and try again */
-				}
-				for (i = 0; i < -del; i++)
-					term__putc('\b');
-			}
-		}
-	}
-	el->el_cursor.h = where;		/* now where is here */
-}
-
-
-/* term_overwrite():
- *	Overstrike num characters
- */
-protected void
-term_overwrite(EditLine *el, const char *cp, int n)
-{
-	if (n <= 0)
-		return;		/* catch bugs */
-
-	if (n > el->el_term.t_size.h) {
-#ifdef DEBUG_SCREEN
-		(void) fprintf(el->el_errfile,
-		    "term_overwrite: n is riduculous: %d\r\n", n);
-#endif /* DEBUG_SCREEN */
-		return;
-	}
-	do {
-		term__putc(*cp++);
-		el->el_cursor.h++;
-	} while (--n);
-
-	if (el->el_cursor.h >= el->el_term.t_size.h) {	/* wrap? */
-		if (EL_HAS_AUTO_MARGINS) {	/* yes */
-			el->el_cursor.h = 0;
-			el->el_cursor.v++;
-			if (EL_HAS_MAGIC_MARGINS) {
-				/* force the wrap to avoid the "magic"
-				 * situation */
-				char c;
-				if ((c = el->el_display[el->el_cursor.v][el->el_cursor.h])
-				    != '\0')
-					term_overwrite(el, &c, 1);
-				else
-					term__putc(' ');
-				el->el_cursor.h = 1;
-			}
-		} else		/* no wrap, but cursor stays on screen */
-			el->el_cursor.h = el->el_term.t_size.h;
-	}
-}
-
-
-/* term_deletechars():
- *	Delete num characters
- */
-protected void
-term_deletechars(EditLine *el, int num)
-{
-	if (num <= 0)
-		return;
-
-	if (!EL_CAN_DELETE) {
-#ifdef DEBUG_EDIT
-		(void) fprintf(el->el_errfile, "   ERROR: cannot delete   \n");
-#endif /* DEBUG_EDIT */
-		return;
-	}
-	if (num > el->el_term.t_size.h) {
-#ifdef DEBUG_SCREEN
-		(void) fprintf(el->el_errfile,
-		    "term_deletechars: num is riduculous: %d\r\n", num);
-#endif /* DEBUG_SCREEN */
-		return;
-	}
-	if (GoodStr(T_DC))	/* if I have multiple delete */
-		if ((num > 1) || !GoodStr(T_dc)) {	/* if dc would be more
-							 * expen. */
-			(void) tputs(tgoto(Str(T_DC), num, num),
-			    num, term__putc);
-			return;
-		}
-	if (GoodStr(T_dm))	/* if I have delete mode */
-		(void) tputs(Str(T_dm), 1, term__putc);
-
-	if (GoodStr(T_dc))	/* else do one at a time */
-		while (num--)
-			(void) tputs(Str(T_dc), 1, term__putc);
-
-	if (GoodStr(T_ed))	/* if I have delete mode */
-		(void) tputs(Str(T_ed), 1, term__putc);
-}
-
-
-/* term_insertwrite():
- *	Puts terminal in insert character mode or inserts num
- *	characters in the line
- */
-protected void
-term_insertwrite(EditLine *el, char *cp, int num)
-{
-	if (num <= 0)
-		return;
-	if (!EL_CAN_INSERT) {
-#ifdef DEBUG_EDIT
-		(void) fprintf(el->el_errfile, "   ERROR: cannot insert   \n");
-#endif /* DEBUG_EDIT */
-		return;
-	}
-	if (num > el->el_term.t_size.h) {
-#ifdef DEBUG_SCREEN
-		(void) fprintf(el->el_errfile,
-		    "StartInsert: num is riduculous: %d\r\n", num);
-#endif /* DEBUG_SCREEN */
-		return;
-	}
-	if (GoodStr(T_IC))	/* if I have multiple insert */
-		if ((num > 1) || !GoodStr(T_ic)) {
-				/* if ic would be more expensive */
-			(void) tputs(tgoto(Str(T_IC), num, num),
-			    num, term__putc);
-			term_overwrite(el, cp, num);
-				/* this updates el_cursor.h */
-			return;
-		}
-	if (GoodStr(T_im) && GoodStr(T_ei)) {	/* if I have insert mode */
-		(void) tputs(Str(T_im), 1, term__putc);
-
-		el->el_cursor.h += num;
-		do
-			term__putc(*cp++);
-		while (--num);
-
-		if (GoodStr(T_ip))	/* have to make num chars insert */
-			(void) tputs(Str(T_ip), 1, term__putc);
-
-		(void) tputs(Str(T_ei), 1, term__putc);
-		return;
-	}
-	do {
-		if (GoodStr(T_ic))	/* have to make num chars insert */
-			(void) tputs(Str(T_ic), 1, term__putc);
-					/* insert a char */
-
-		term__putc(*cp++);
-
-		el->el_cursor.h++;
-
-		if (GoodStr(T_ip))	/* have to make num chars insert */
-			(void) tputs(Str(T_ip), 1, term__putc);
-					/* pad the inserted char */
-
-	} while (--num);
-}
-
-
-/* term_clear_EOL():
- *	clear to end of line.  There are num characters to clear
- */
-protected void
-term_clear_EOL(EditLine *el, int num)
-{
-	int i;
-
-	if (EL_CAN_CEOL && GoodStr(T_ce))
-		(void) tputs(Str(T_ce), 1, term__putc);
-	else {
-		for (i = 0; i < num; i++)
-			term__putc(' ');
-		el->el_cursor.h += num;	/* have written num spaces */
-	}
-}
-
-
-/* term_clear_screen():
- *	Clear the screen
- */
-protected void
-term_clear_screen(EditLine *el)
-{				/* clear the whole screen and home */
-
-	if (GoodStr(T_cl))
-		/* send the clear screen code */
-		(void) tputs(Str(T_cl), Val(T_li), term__putc);
-	else if (GoodStr(T_ho) && GoodStr(T_cd)) {
-		(void) tputs(Str(T_ho), Val(T_li), term__putc);	/* home */
-		/* clear to bottom of screen */
-		(void) tputs(Str(T_cd), Val(T_li), term__putc);
-	} else {
-		term__putc('\r');
-		term__putc('\n');
-	}
-}
-
-
-/* term_beep():
- *	Beep the way the terminal wants us
- */
-protected void
-term_beep(EditLine *el)
-{
-	if (GoodStr(T_bl))
-		/* what termcap says we should use */
-		(void) tputs(Str(T_bl), 1, term__putc);
-	else
-		term__putc('\007');	/* an ASCII bell; ^G */
-}
-
-
-#ifdef notdef
-/* term_clear_to_bottom():
- *	Clear to the bottom of the screen
- */
-protected void
-term_clear_to_bottom(EditLine *el)
-{
-	if (GoodStr(T_cd))
-		(void) tputs(Str(T_cd), Val(T_li), term__putc);
-	else if (GoodStr(T_ce))
-		(void) tputs(Str(T_ce), Val(T_li), term__putc);
-}
-#endif
-
-
-/* term_set():
- *	Read in the terminal capabilities from the requested terminal
- */
-protected int
-term_set(EditLine *el, const char *term)
-{
-	int i;
-	char buf[TC_BUFSIZE];
-	char *area;
-	const struct termcapstr *t;
-	sigset_t oset, nset;
-	int lins, cols;
-
-	(void) sigemptyset(&nset);
-	(void) sigaddset(&nset, SIGWINCH);
-	(void) sigprocmask(SIG_BLOCK, &nset, &oset);
-
-	area = buf;
-
-
-	if (term == NULL)
-		term = getenv("TERM");
-
-	if (!term || !term[0])
-		term = "dumb";
-
-	if (strcmp(term, "emacs") == 0)
-		el->el_flags |= EDIT_DISABLED;
-
-	memset(el->el_term.t_cap, 0, TC_BUFSIZE);
-
-	i = tgetent(el->el_term.t_cap, term);
-
-	if (i <= 0) {
-		if (i == -1)
-			(void) fprintf(el->el_errfile,
-			    "Cannot read termcap database;\n");
-		else if (i == 0)
-			(void) fprintf(el->el_errfile,
-			    "No entry for terminal type \"%s\";\n", term);
-		(void) fprintf(el->el_errfile,
-		    "using dumb terminal settings.\n");
-		Val(T_co) = 80;	/* do a dumb terminal */
-		Val(T_pt) = Val(T_km) = Val(T_li) = 0;
-		Val(T_xt) = Val(T_MT);
-		for (t = tstr; t->name != NULL; t++)
-			term_alloc(el, t, NULL);
-	} else {
-		/* auto/magic margins */
-		Val(T_am) = tgetflag("am");
-		Val(T_xn) = tgetflag("xn");
-		/* Can we tab */
-		Val(T_pt) = tgetflag("pt");
-		Val(T_xt) = tgetflag("xt");
-		/* do we have a meta? */
-		Val(T_km) = tgetflag("km");
-		Val(T_MT) = tgetflag("MT");
-		/* Get the size */
-		Val(T_co) = tgetnum("co");
-		Val(T_li) = tgetnum("li");
-		for (t = tstr; t->name != NULL; t++)
-			term_alloc(el, t, tgetstr((char *)t->name, &area));
-	}
-
-	if (Val(T_co) < 2)
-		Val(T_co) = 80;	/* just in case */
-	if (Val(T_li) < 1)
-		Val(T_li) = 24;
-
-	el->el_term.t_size.v = Val(T_co);
-	el->el_term.t_size.h = Val(T_li);
-
-	term_setflags(el);
-
-				/* get the correct window size */
-	(void) term_get_size(el, &lins, &cols);
-	if (term_change_size(el, lins, cols) == -1)
-		return (-1);
-	(void) sigprocmask(SIG_SETMASK, &oset, NULL);
-	term_bind_arrow(el);
-	return (i <= 0 ? -1 : 0);
-}
-
-
-/* term_get_size():
- *	Return the new window size in lines and cols, and
- *	true if the size was changed.
- */
-protected int
-term_get_size(EditLine *el, int *lins, int *cols)
-{
-
-	*cols = Val(T_co);
-	*lins = Val(T_li);
-
-#ifdef TIOCGWINSZ
-	{
-		struct winsize ws;
-		if (ioctl(el->el_infd, TIOCGWINSZ, (ioctl_t) & ws) != -1) {
-			if (ws.ws_col)
-				*cols = ws.ws_col;
-			if (ws.ws_row)
-				*lins = ws.ws_row;
-		}
-	}
-#endif
-#ifdef TIOCGSIZE
-	{
-		struct ttysize ts;
-		if (ioctl(el->el_infd, TIOCGSIZE, (ioctl_t) & ts) != -1) {
-			if (ts.ts_cols)
-				*cols = ts.ts_cols;
-			if (ts.ts_lines)
-				*lins = ts.ts_lines;
-		}
-	}
-#endif
-	return (Val(T_co) != *cols || Val(T_li) != *lins);
-}
-
-
-/* term_change_size():
- *	Change the size of the terminal
- */
-protected int
-term_change_size(EditLine *el, int lins, int cols)
-{
-	/*
-         * Just in case
-         */
-	Val(T_co) = (cols < 2) ? 80 : cols;
-	Val(T_li) = (lins < 1) ? 24 : lins;
-
-	/* re-make display buffers */
-	if (term_rebuffer_display(el) == -1)
-		return (-1);
-	re_clear_display(el);
-	return (0);
-}
-
-
-/* term_init_arrow():
- *	Initialize the arrow key bindings from termcap
- */
-private void
-term_init_arrow(EditLine *el)
-{
-	fkey_t *arrow = el->el_term.t_fkey;
-
-	arrow[A_K_DN].name = "down";
-	arrow[A_K_DN].key = T_kd;
-	arrow[A_K_DN].fun.cmd = ED_NEXT_HISTORY;
-	arrow[A_K_DN].type = XK_CMD;
-
-	arrow[A_K_UP].name = "up";
-	arrow[A_K_UP].key = T_ku;
-	arrow[A_K_UP].fun.cmd = ED_PREV_HISTORY;
-	arrow[A_K_UP].type = XK_CMD;
-
-	arrow[A_K_LT].name = "left";
-	arrow[A_K_LT].key = T_kl;
-	arrow[A_K_LT].fun.cmd = ED_PREV_CHAR;
-	arrow[A_K_LT].type = XK_CMD;
-
-	arrow[A_K_RT].name = "right";
-	arrow[A_K_RT].key = T_kr;
-	arrow[A_K_RT].fun.cmd = ED_NEXT_CHAR;
-	arrow[A_K_RT].type = XK_CMD;
-
-	arrow[A_K_HO].name = "home";
-	arrow[A_K_HO].key = T_kh;
-	arrow[A_K_HO].fun.cmd = ED_MOVE_TO_BEG;
-	arrow[A_K_HO].type = XK_CMD;
-
-	arrow[A_K_EN].name = "end";
-	arrow[A_K_EN].key = T_at7;
-	arrow[A_K_EN].fun.cmd = ED_MOVE_TO_END;
-	arrow[A_K_EN].type = XK_CMD;
-}
-
-
-/* term_reset_arrow():
- *	Reset arrow key bindings
- */
-private void
-term_reset_arrow(EditLine *el)
-{
-	fkey_t *arrow = el->el_term.t_fkey;
-	static const char strA[] = {033, '[', 'A', '\0'};
-	static const char strB[] = {033, '[', 'B', '\0'};
-	static const char strC[] = {033, '[', 'C', '\0'};
-	static const char strD[] = {033, '[', 'D', '\0'};
-	static const char strH[] = {033, '[', 'H', '\0'};
-	static const char strF[] = {033, '[', 'F', '\0'};
-	static const char stOA[] = {033, 'O', 'A', '\0'};
-	static const char stOB[] = {033, 'O', 'B', '\0'};
-	static const char stOC[] = {033, 'O', 'C', '\0'};
-	static const char stOD[] = {033, 'O', 'D', '\0'};
-	static const char stOH[] = {033, 'O', 'H', '\0'};
-	static const char stOF[] = {033, 'O', 'F', '\0'};
-
-	key_add(el, strA, &arrow[A_K_UP].fun, arrow[A_K_UP].type);
-	key_add(el, strB, &arrow[A_K_DN].fun, arrow[A_K_DN].type);
-	key_add(el, strC, &arrow[A_K_RT].fun, arrow[A_K_RT].type);
-	key_add(el, strD, &arrow[A_K_LT].fun, arrow[A_K_LT].type);
-	key_add(el, strH, &arrow[A_K_HO].fun, arrow[A_K_HO].type);
-	key_add(el, strF, &arrow[A_K_EN].fun, arrow[A_K_EN].type);
-	key_add(el, stOA, &arrow[A_K_UP].fun, arrow[A_K_UP].type);
-	key_add(el, stOB, &arrow[A_K_DN].fun, arrow[A_K_DN].type);
-	key_add(el, stOC, &arrow[A_K_RT].fun, arrow[A_K_RT].type);
-	key_add(el, stOD, &arrow[A_K_LT].fun, arrow[A_K_LT].type);
-	key_add(el, stOH, &arrow[A_K_HO].fun, arrow[A_K_HO].type);
-	key_add(el, stOF, &arrow[A_K_EN].fun, arrow[A_K_EN].type);
-
-	if (el->el_map.type == MAP_VI) {
-		key_add(el, &strA[1], &arrow[A_K_UP].fun, arrow[A_K_UP].type);
-		key_add(el, &strB[1], &arrow[A_K_DN].fun, arrow[A_K_DN].type);
-		key_add(el, &strC[1], &arrow[A_K_RT].fun, arrow[A_K_RT].type);
-		key_add(el, &strD[1], &arrow[A_K_LT].fun, arrow[A_K_LT].type);
-		key_add(el, &strH[1], &arrow[A_K_HO].fun, arrow[A_K_HO].type);
-		key_add(el, &strF[1], &arrow[A_K_EN].fun, arrow[A_K_EN].type);
-		key_add(el, &stOA[1], &arrow[A_K_UP].fun, arrow[A_K_UP].type);
-		key_add(el, &stOB[1], &arrow[A_K_DN].fun, arrow[A_K_DN].type);
-		key_add(el, &stOC[1], &arrow[A_K_RT].fun, arrow[A_K_RT].type);
-		key_add(el, &stOD[1], &arrow[A_K_LT].fun, arrow[A_K_LT].type);
-		key_add(el, &stOH[1], &arrow[A_K_HO].fun, arrow[A_K_HO].type);
-		key_add(el, &stOF[1], &arrow[A_K_EN].fun, arrow[A_K_EN].type);
-	}
-}
-
-
-/* term_set_arrow():
- *	Set an arrow key binding
- */
-protected int
-term_set_arrow(EditLine *el, const char *name, key_value_t *fun, int type)
-{
-	fkey_t *arrow = el->el_term.t_fkey;
-	int i;
-
-	for (i = 0; i < A_K_NKEYS; i++)
-		if (strcmp(name, arrow[i].name) == 0) {
-			arrow[i].fun = *fun;
-			arrow[i].type = type;
-			return (0);
-		}
-	return (-1);
-}
-
-
-/* term_clear_arrow():
- *	Clear an arrow key binding
- */
-protected int
-term_clear_arrow(EditLine *el, const char *name)
-{
-	fkey_t *arrow = el->el_term.t_fkey;
-	int i;
-
-	for (i = 0; i < A_K_NKEYS; i++)
-		if (strcmp(name, arrow[i].name) == 0) {
-			arrow[i].type = XK_NOD;
-			return (0);
-		}
-	return (-1);
-}
-
-
-/* term_print_arrow():
- *	Print the arrow key bindings
- */
-protected void
-term_print_arrow(EditLine *el, const char *name)
-{
-	int i;
-	fkey_t *arrow = el->el_term.t_fkey;
-
-	for (i = 0; i < A_K_NKEYS; i++)
-		if (*name == '\0' || strcmp(name, arrow[i].name) == 0)
-			if (arrow[i].type != XK_NOD)
-				key_kprint(el, arrow[i].name, &arrow[i].fun,
-				    arrow[i].type);
-}
-
-
-/* term_bind_arrow():
- *	Bind the arrow keys
- */
-protected void
-term_bind_arrow(EditLine *el)
-{
-	el_action_t *map;
-	const el_action_t *dmap;
-	int i, j;
-	char *p;
-	fkey_t *arrow = el->el_term.t_fkey;
-
-	/* Check if the components needed are initialized */
-	if (el->el_term.t_buf == NULL || el->el_map.key == NULL)
-		return;
-
-	map = el->el_map.type == MAP_VI ? el->el_map.alt : el->el_map.key;
-	dmap = el->el_map.type == MAP_VI ? el->el_map.vic : el->el_map.emacs;
-
-	term_reset_arrow(el);
-
-	for (i = 0; i < A_K_NKEYS; i++) {
-		p = el->el_term.t_str[arrow[i].key];
-		if (p && *p) {
-			j = (unsigned char) *p;
-			/*
-			 * Assign the arrow keys only if:
-			 *
-			 * 1. They are multi-character arrow keys and the user
-			 *    has not re-assigned the leading character, or
-			 *    has re-assigned the leading character to be
-			 *	  ED_SEQUENCE_LEAD_IN
-			 * 2. They are single arrow keys pointing to an
-			 *    unassigned key.
-			 */
-			if (arrow[i].type == XK_NOD)
-				key_clear(el, map, p);
-			else {
-				if (p[1] && (dmap[j] == map[j] ||
-					map[j] == ED_SEQUENCE_LEAD_IN)) {
-					key_add(el, p, &arrow[i].fun,
-					    arrow[i].type);
-					map[j] = ED_SEQUENCE_LEAD_IN;
-				} else if (map[j] == ED_UNASSIGNED) {
-					key_clear(el, map, p);
-					if (arrow[i].type == XK_CMD)
-						map[j] = arrow[i].fun.cmd;
-					else
-						key_add(el, p, &arrow[i].fun,
-						    arrow[i].type);
-				}
-			}
-		}
-	}
-}
-
-
-/* term__putc():
- *	Add a character
- */
-protected int
-term__putc(int c)
-{
-
-	return (fputc(c, term_outfile));
-}
-
-
-/* term__flush():
- *	Flush output
- */
-protected void
-term__flush(void)
-{
-
-	(void) fflush(term_outfile);
-}
-
-
-/* term_telltc():
- *	Print the current termcap characteristics
- */
-protected int
-/*ARGSUSED*/
-term_telltc(EditLine *el, int argc, const char **argv)
-{
-	const struct termcapstr *t;
-	char **ts;
-	char upbuf[EL_BUFSIZ];
-
-	(void) fprintf(el->el_outfile, "\n\tYour terminal has the\n");
-	(void) fprintf(el->el_outfile, "\tfollowing characteristics:\n\n");
-	(void) fprintf(el->el_outfile, "\tIt has %d columns and %d lines\n",
-	    Val(T_co), Val(T_li));
-	(void) fprintf(el->el_outfile,
-	    "\tIt has %s meta key\n", EL_HAS_META ? "a" : "no");
-	(void) fprintf(el->el_outfile,
-	    "\tIt can%suse tabs\n", EL_CAN_TAB ? " " : "not ");
-	(void) fprintf(el->el_outfile, "\tIt %s automatic margins\n",
-	    EL_HAS_AUTO_MARGINS ? "has" : "does not have");
-	if (EL_HAS_AUTO_MARGINS)
-		(void) fprintf(el->el_outfile, "\tIt %s magic margins\n",
-		    EL_HAS_MAGIC_MARGINS ? "has" : "does not have");
-
-	for (t = tstr, ts = el->el_term.t_str; t->name != NULL; t++, ts++)
-		(void) fprintf(el->el_outfile, "\t%25s (%s) == %s\n",
-		    t->long_name,
-		    t->name, *ts && **ts ?
-		    key__decode_str(*ts, upbuf, "") : "(empty)");
-	(void) fputc('\n', el->el_outfile);
-	return (0);
-}
-
-
-/* term_settc():
- *	Change the current terminal characteristics
- */
-protected int
-/*ARGSUSED*/
-term_settc(EditLine *el, int argc, const char **argv)
-{
-	const struct termcapstr *ts;
-	const struct termcapval *tv;
-	const char *what, *how;
-
-	if (argv == NULL || argv[1] == NULL || argv[2] == NULL)
-		return (-1);
-
-	what = argv[1];
-	how = argv[2];
-
-	/*
-         * Do the strings first
-         */
-	for (ts = tstr; ts->name != NULL; ts++)
-		if (strcmp(ts->name, what) == 0)
-			break;
-
-	if (ts->name != NULL) {
-		term_alloc(el, ts, how);
-		term_setflags(el);
-		return (0);
-	}
-	/*
-         * Do the numeric ones second
-         */
-	for (tv = tval; tv->name != NULL; tv++)
-		if (strcmp(tv->name, what) == 0)
-			break;
-
-	if (tv->name != NULL) {
-		if (tv == &tval[T_pt] || tv == &tval[T_km] ||
-		    tv == &tval[T_am] || tv == &tval[T_xn]) {
-			if (strcmp(how, "yes") == 0)
-				el->el_term.t_val[tv - tval] = 1;
-			else if (strcmp(how, "no") == 0)
-				el->el_term.t_val[tv - tval] = 0;
-			else {
-				(void) fprintf(el->el_errfile,
-				    "settc: Bad value `%s'.\n", how);
-				return (-1);
-			}
-			term_setflags(el);
-			if (term_change_size(el, Val(T_li), Val(T_co)) == -1)
-				return (-1);
-			return (0);
-		} else {
-			long i;
-			char *ep;
-
-			i = strtol(how, &ep, 10);
-			if (*ep != '\0') {
-				(void) fprintf(el->el_errfile,
-				    "settc: Bad value `%s'.\n", how);
-				return (-1);
-			}
-			el->el_term.t_val[tv - tval] = (int) i;
-			el->el_term.t_size.v = Val(T_co);
-			el->el_term.t_size.h = Val(T_li);
-			if (tv == &tval[T_co] || tv == &tval[T_li])
-				if (term_change_size(el, Val(T_li), Val(T_co))
-				    == -1)
-					return (-1);
-			return (0);
-		}
-	}
-	return (-1);
-}
-
-
-/* term_echotc():
- *	Print the termcap string out with variable substitution
- */
-protected int
-/*ARGSUSED*/
-term_echotc(EditLine *el, int argc, const char **argv)
-{
-	char *cap, *scap, *ep;
-	int arg_need, arg_cols, arg_rows;
-	int verbose = 0, silent = 0;
-	char *area;
-	static const char fmts[] = "%s\n", fmtd[] = "%d\n";
-	const struct termcapstr *t;
-	char buf[TC_BUFSIZE];
-	long i;
-
-	area = buf;
-
-	if (argv == NULL || argv[1] == NULL)
-		return (-1);
-	argv++;
-
-	if (argv[0][0] == '-') {
-		switch (argv[0][1]) {
-		case 'v':
-			verbose = 1;
-			break;
-		case 's':
-			silent = 1;
-			break;
-		default:
-			/* stderror(ERR_NAME | ERR_TCUSAGE); */
-			break;
-		}
-		argv++;
-	}
-	if (!*argv || *argv[0] == '\0')
-		return (0);
-	if (strcmp(*argv, "tabs") == 0) {
-		(void) fprintf(el->el_outfile, fmts, EL_CAN_TAB ? "yes" : "no");
-		return (0);
-	} else if (strcmp(*argv, "meta") == 0) {
-		(void) fprintf(el->el_outfile, fmts, Val(T_km) ? "yes" : "no");
-		return (0);
-	} else if (strcmp(*argv, "xn") == 0) {
-		(void) fprintf(el->el_outfile, fmts, EL_HAS_MAGIC_MARGINS ?
-		    "yes" : "no");
-		return (0);
-	} else if (strcmp(*argv, "am") == 0) {
-		(void) fprintf(el->el_outfile, fmts, EL_HAS_AUTO_MARGINS ?
-		    "yes" : "no");
-		return (0);
-	} else if (strcmp(*argv, "baud") == 0) {
-#ifdef notdef
-		int i;
-
-		for (i = 0; baud_rate[i].b_name != NULL; i++)
-			if (el->el_tty.t_speed == baud_rate[i].b_rate) {
-				(void) fprintf(el->el_outfile, fmts,
-				    baud_rate[i].b_name);
-				return (0);
-			}
-		(void) fprintf(el->el_outfile, fmtd, 0);
-#else
-		(void) fprintf(el->el_outfile, fmtd, (int) el->el_tty.t_speed);
-#endif
-		return (0);
-	} else if (strcmp(*argv, "rows") == 0 || strcmp(*argv, "lines") == 0) {
-		(void) fprintf(el->el_outfile, fmtd, Val(T_li));
-		return (0);
-	} else if (strcmp(*argv, "cols") == 0) {
-		(void) fprintf(el->el_outfile, fmtd, Val(T_co));
-		return (0);
-	}
-	/*
-         * Try to use our local definition first
-         */
-	scap = NULL;
-	for (t = tstr; t->name != NULL; t++)
-		if (strcmp(t->name, *argv) == 0) {
-			scap = el->el_term.t_str[t - tstr];
-			break;
-		}
-	if (t->name == NULL)
-		scap = tgetstr((char *)argv, &area);
-	if (!scap || scap[0] == '\0') {
-		if (!silent)
-			(void) fprintf(el->el_errfile,
-			    "echotc: Termcap parameter `%s' not found.\n",
-			    *argv);
-		return (-1);
-	}
-	/*
-         * Count home many values we need for this capability.
-         */
-	for (cap = scap, arg_need = 0; *cap; cap++)
-		if (*cap == '%')
-			switch (*++cap) {
-			case 'd':
-			case '2':
-			case '3':
-			case '.':
-			case '+':
-				arg_need++;
-				break;
-			case '%':
-			case '>':
-			case 'i':
-			case 'r':
-			case 'n':
-			case 'B':
-			case 'D':
-				break;
-			default:
-				/*
-				 * hpux has lot's of them...
-				 */
-				if (verbose)
-					(void) fprintf(el->el_errfile,
-				"echotc: Warning: unknown termcap %% `%c'.\n",
-					    *cap);
-				/* This is bad, but I won't complain */
-				break;
-			}
-
-	switch (arg_need) {
-	case 0:
-		argv++;
-		if (*argv && *argv[0]) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Extra argument `%s'.\n",
-				    *argv);
-			return (-1);
-		}
-		(void) tputs(scap, 1, term__putc);
-		break;
-	case 1:
-		argv++;
-		if (!*argv || *argv[0] == '\0') {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Missing argument.\n");
-			return (-1);
-		}
-		arg_cols = 0;
-		i = strtol(*argv, &ep, 10);
-		if (*ep != '\0' || i < 0) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Bad value `%s' for rows.\n",
-				    *argv);
-			return (-1);
-		}
-		arg_rows = (int) i;
-		argv++;
-		if (*argv && *argv[0]) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Extra argument `%s'.\n",
-				    *argv);
-			return (-1);
-		}
-		(void) tputs(tgoto(scap, arg_cols, arg_rows), 1, term__putc);
-		break;
-	default:
-		/* This is wrong, but I will ignore it... */
-		if (verbose)
-			(void) fprintf(el->el_errfile,
-			 "echotc: Warning: Too many required arguments (%d).\n",
-			    arg_need);
-		/* FALLTHROUGH */
-	case 2:
-		argv++;
-		if (!*argv || *argv[0] == '\0') {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Missing argument.\n");
-			return (-1);
-		}
-		i = strtol(*argv, &ep, 10);
-		if (*ep != '\0' || i < 0) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Bad value `%s' for cols.\n",
-				    *argv);
-			return (-1);
-		}
-		arg_cols = (int) i;
-		argv++;
-		if (!*argv || *argv[0] == '\0') {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Missing argument.\n");
-			return (-1);
-		}
-		i = strtol(*argv, &ep, 10);
-		if (*ep != '\0' || i < 0) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Bad value `%s' for rows.\n",
-				    *argv);
-			return (-1);
-		}
-		arg_rows = (int) i;
-		if (*ep != '\0') {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Bad value `%s'.\n", *argv);
-			return (-1);
-		}
-		argv++;
-		if (*argv && *argv[0]) {
-			if (!silent)
-				(void) fprintf(el->el_errfile,
-				    "echotc: Warning: Extra argument `%s'.\n",
-				    *argv);
-			return (-1);
-		}
-		(void) tputs(tgoto(scap, arg_cols, arg_rows), arg_rows,
-		    term__putc);
-		break;
-	}
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/term.h 1:16.1.1~dfsg-1/main/editline/term.h
--- 1:13.23.1~dfsg-2/main/editline/term.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/term.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,124 +0,0 @@
-/*	$NetBSD: term.h,v 1.13 2002/03/18 16:01:00 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)term.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.term.h: Termcap header
- */
-#ifndef _h_el_term
-#define	_h_el_term
-
-#include "histedit.h"
-
-typedef struct {		/* Symbolic function key bindings	*/
-	const char	*name;	/* name of the key			*/
-	int		 key;	/* Index in termcap table		*/
-	key_value_t	 fun;	/* Function bound to it			*/
-	int		 type;	/* Type of function			*/
-} fkey_t;
-
-typedef struct {
-	coord_t	  t_size;		/* # lines and cols	*/
-	int	  t_flags;
-#define	TERM_CAN_INSERT		0x001	/* Has insert cap	*/
-#define	TERM_CAN_DELETE		0x002	/* Has delete cap	*/
-#define	TERM_CAN_CEOL		0x004	/* Has CEOL cap		*/
-#define	TERM_CAN_TAB		0x008	/* Can use tabs		*/
-#define	TERM_CAN_ME		0x010	/* Can turn all attrs.	*/
-#define	TERM_CAN_UP		0x020	/* Can move up		*/
-#define	TERM_HAS_META		0x040	/* Has a meta key	*/
-#define	TERM_HAS_AUTO_MARGINS	0x080	/* Has auto margins	*/
-#define	TERM_HAS_MAGIC_MARGINS	0x100	/* Has magic margins	*/
-	char	 *t_buf;		/* Termcap buffer	*/
-	int	  t_loc;		/* location used	*/
-	char	**t_str;		/* termcap strings	*/
-	int	 *t_val;		/* termcap values	*/
-	char	 *t_cap;		/* Termcap buffer	*/
-	fkey_t	 *t_fkey;		/* Array of keys	*/
-} el_term_t;
-
-/*
- * fKey indexes
- */
-#define	A_K_DN		0
-#define	A_K_UP		1
-#define	A_K_LT		2
-#define	A_K_RT		3
-#define	A_K_HO		4
-#define	A_K_EN		5
-#define	A_K_NKEYS	6
-
-protected void	term_move_to_line(EditLine *, int);
-protected void	term_move_to_char(EditLine *, int);
-protected void	term_clear_EOL(EditLine *, int);
-protected void	term_overwrite(EditLine *, const char *, int);
-protected void	term_insertwrite(EditLine *, char *, int);
-protected void	term_deletechars(EditLine *, int);
-protected void	term_clear_screen(EditLine *);
-protected void	term_beep(EditLine *);
-protected int	term_change_size(EditLine *, int, int);
-protected int	term_get_size(EditLine *, int *, int *);
-protected int	term_init(EditLine *);
-protected void	term_bind_arrow(EditLine *);
-protected void	term_print_arrow(EditLine *, const char *);
-protected int	term_clear_arrow(EditLine *, const char *);
-protected int	term_set_arrow(EditLine *, const char *, key_value_t *, int);
-protected void	term_end(EditLine *);
-protected int	term_set(EditLine *, const char *);
-protected int	term_settc(EditLine *, int, const char **);
-protected int	term_telltc(EditLine *, int, const char **);
-protected int	term_echotc(EditLine *, int, const char **);
-protected int	term__putc(int);
-protected void	term__flush(void);
-
-/*
- * Easy access macros
- */
-#define	EL_FLAGS	(el)->el_term.t_flags
-
-#define	EL_CAN_INSERT		(EL_FLAGS & TERM_CAN_INSERT)
-#define	EL_CAN_DELETE		(EL_FLAGS & TERM_CAN_DELETE)
-#define	EL_CAN_CEOL		(EL_FLAGS & TERM_CAN_CEOL)
-#define	EL_CAN_TAB		(EL_FLAGS & TERM_CAN_TAB)
-#define	EL_CAN_ME		(EL_FLAGS & TERM_CAN_ME)
-#define	EL_HAS_META		(EL_FLAGS & TERM_HAS_META)
-#define	EL_HAS_AUTO_MARGINS	(EL_FLAGS & TERM_HAS_AUTO_MARGINS)
-#define	EL_HAS_MAGIC_MARGINS	(EL_FLAGS & TERM_HAS_MAGIC_MARGINS)
-
-#endif /* _h_el_term */
diff -pruN 1:13.23.1~dfsg-2/main/editline/TEST/test.c 1:16.1.1~dfsg-1/main/editline/TEST/test.c
--- 1:13.23.1~dfsg-2/main/editline/TEST/test.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/TEST/test.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,268 +0,0 @@
-/*	$NetBSD: test.c,v 1.9 2000/09/04 23:36:41 lukem Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#ifndef lint
-__COPYRIGHT("@(#) Copyright (c) 1992, 1993\n\
-	The Regents of the University of California.  All rights reserved.\n");
-#endif /* not lint */
-
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)test.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: test.c,v 1.9 2000/09/04 23:36:41 lukem Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * test.c: A little test program
- */
-#include <stdio.h>
-#include <string.h>
-#include <signal.h>
-#include <sys/wait.h>
-#include <ctype.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <dirent.h>
-
-#include "histedit.h"
-#include "tokenizer.h"
-
-static int continuation = 0;
-static EditLine *el = NULL;
-
-static	u_char	complete(EditLine *, int);
-	int	main(int, char **);
-static	char   *prompt(EditLine *);
-static	void	sig(int);
-
-static char *
-prompt(EditLine *el)
-{
-	static char a[] = "Edit$";
-	static char b[] = "Edit>";
-
-	return (continuation ? b : a);
-}
-
-static void
-sig(int i)
-{
-
-	(void) fprintf(stderr, "Got signal %d.\n", i);
-	el_reset(el);
-}
-
-static unsigned char
-complete(EditLine *el, int ch)
-{
-	DIR *dd = opendir(".");
-	struct dirent *dp;
-	const char* ptr;
-	const LineInfo *lf = el_line(el);
-	int len;
-
-	/*
-	 * Find the last word
-	 */
-	for (ptr = lf->cursor - 1; !isspace(*ptr) && ptr > lf->buffer; ptr--)
-		continue;
-	len = lf->cursor - ++ptr;
-
-	for (dp = readdir(dd); dp != NULL; dp = readdir(dd)) {
-		if (len > strlen(dp->d_name))
-			continue;
-		if (strncmp(dp->d_name, ptr, len) == 0) {
-			closedir(dd);
-			if (el_insertstr(el, &dp->d_name[len]) == -1)
-				return (CC_ERROR);
-			else
-				return (CC_REFRESH);
-		}
-	}
-
-	closedir(dd);
-	return (CC_ERROR);
-}
-
-int
-main(int argc, char *argv[])
-{
-	int num;
-	const char *buf;
-	Tokenizer *tok;
-#if 0
-	int lastevent = 0;
-#endif
-	int ncontinuation;
-	History *hist;
-	HistEvent ev;
-
-	(void) signal(SIGINT, sig);
-	(void) signal(SIGQUIT, sig);
-	(void) signal(SIGHUP, sig);
-	(void) signal(SIGTERM, sig);
-
-	hist = history_init();		/* Init the builtin history	*/
-					/* Remember 100 events		*/
-	history(hist, &ev, H_SETSIZE, 100);
-
-	tok  = tok_init(NULL);		/* Initialize the tokenizer	*/
-
-					/* Initialize editline		*/
-	el = el_init(*argv, stdin, stdout, stderr);
-
-	el_set(el, EL_EDITOR, "vi");	/* Default editor is vi		*/
-	el_set(el, EL_SIGNAL, 1);	/* Handle signals gracefully	*/
-	el_set(el, EL_PROMPT, prompt);	/* Set the prompt function	*/
-
-			/* Tell editline to use this history interface	*/
-	el_set(el, EL_HIST, history, hist);
-
-					/* Add a user-defined function	*/
-	el_set(el, EL_ADDFN, "ed-complete", "Complete argument", complete);
-
-					/* Bind tab to it 		*/
-	el_set(el, EL_BIND, "^I", "ed-complete", NULL);
-
-	/*
-	 * Bind j, k in vi command mode to previous and next line, instead
-	 * of previous and next history.
-	 */
-	el_set(el, EL_BIND, "-a", "k", "ed-prev-line", NULL);
-	el_set(el, EL_BIND, "-a", "j", "ed-next-line", NULL);
-
-	/*
-	 * Source the user's defaults file.
-	 */
-	el_source(el, NULL);
-
-	while ((buf = el_gets(el, &num)) != NULL && num != 0)  {
-		int ac;
-		const char **av;
-#ifdef DEBUG
-		(void) fprintf(stderr, "got %d %s", num, buf);
-#endif
-		if (!continuation && num == 1)
-			continue;
-
-		ncontinuation = tok_line(tok, buf, &ac, &av) > 0;
-#if 0
-		if (continuation) {
-			/*
-			 * Append to the right event in case the user
-			 * moved around in history.
-			 */
-			if (history(hist, &ev, H_SET, lastevent) == -1)
-				err(1, "%d: %s\n", lastevent, ev.str);
-			history(hist, &ev, H_ADD , buf);
-		} else {
-			history(hist, &ev, H_ENTER, buf);
-			lastevent = ev.num;
-		}
-#else
-				/* Simpler */
-		history(hist, &ev, continuation ? H_APPEND : H_ENTER, buf);
-#endif
-
-		continuation = ncontinuation;
-		ncontinuation = 0;
-
-		if (strcmp(av[0], "history") == 0) {
-			int rv;
-
-			switch (ac) {
-			case 1:
-				for (rv = history(hist, &ev, H_LAST); rv != -1;
-				    rv = history(hist, &ev, H_PREV))
-					(void) fprintf(stdout, "%4d %s",
-					    ev.num, ev.str);
-				break;
-
-			case 2:
-				if (strcmp(av[1], "clear") == 0)
-					 history(hist, &ev, H_CLEAR);
-				else
-					 goto badhist;
-				break;
-
-			case 3:
-				if (strcmp(av[1], "load") == 0)
-					 history(hist, &ev, H_LOAD, av[2]);
-				else if (strcmp(av[1], "save") == 0)
-					 history(hist, &ev, H_SAVE, av[2]);
-				break;
-
-			badhist:
-			default:
-				(void) fprintf(stderr,
-				    "Bad history arguments\n");
-				break;
-			}
-		} else if (el_parse(el, ac, av) == -1) {
-			switch (fork()) {
-			case 0:
-				execvp(av[0], (char *const *)av);
-				perror(av[0]);
-				_exit(1);
-				/*NOTREACHED*/
-				break;
-
-			case -1:
-				perror("fork");
-				break;
-
-			default:
-				if (wait(&num) == -1)
-					perror("wait");
-				(void) fprintf(stderr, "Exit %x\n", num);
-				break;
-			}
-		}
-
-		tok_reset(tok);
-	}
-
-	el_end(el);
-	tok_end(tok);
-	history_end(hist);
-
-	return (0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/tokenizer.c 1:16.1.1~dfsg-1/main/editline/tokenizer.c
--- 1:13.23.1~dfsg-2/main/editline/tokenizer.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/tokenizer.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,402 +0,0 @@
-/*	$NetBSD: tokenizer.c,v 1.10 2002/03/18 16:01:00 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)tokenizer.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: tokenizer.c,v 1.10 2002/03/18 16:01:00 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * tokenize.c: Bourne shell like tokenizer
- */
-#include <string.h>
-#include <stdlib.h>
-#include "tokenizer.h"
-
-typedef enum {
-	Q_none, Q_single, Q_double, Q_one, Q_doubleone
-} quote_t;
-
-#define	IFS		"\t \n"
-
-#define	TOK_KEEP	1
-#define	TOK_EAT		2
-
-#define	WINCR		20
-#define	AINCR		10
-
-#define	tok_malloc(a)		malloc(a)
-#define	tok_free(a)		free(a)
-#define	tok_realloc(a, b)	realloc(a, b)
-
-
-struct tokenizer {
-	char	*ifs;		/* In field separator			 */
-	int	 argc, amax;	/* Current and maximum number of args	 */
-	char   **argv;		/* Argument list			 */
-	char	*wptr, *wmax;	/* Space and limit on the word buffer	 */
-	char	*wstart;	/* Beginning of next word		 */
-	char	*wspace;	/* Space of word buffer			 */
-	quote_t	 quote;		/* Quoting state			 */
-	int	 flags;		/* flags;				 */
-};
-
-
-private void tok_finish(Tokenizer *);
-
-
-/* tok_finish():
- *	Finish a word in the tokenizer.
- */
-private void
-tok_finish(Tokenizer *tok)
-{
-
-	*tok->wptr = '\0';
-	if ((tok->flags & TOK_KEEP) || tok->wptr != tok->wstart) {
-		tok->argv[tok->argc++] = tok->wstart;
-		tok->argv[tok->argc] = NULL;
-		tok->wstart = ++tok->wptr;
-	}
-	tok->flags &= ~TOK_KEEP;
-}
-
-
-/* tok_init():
- *	Initialize the tokenizer
- */
-public Tokenizer *
-tok_init(const char *ifs)
-{
-	Tokenizer *tok = (Tokenizer *) tok_malloc(sizeof(Tokenizer));
-
-	tok->ifs = strdup(ifs ? ifs : IFS);
-	tok->argc = 0;
-	tok->amax = AINCR;
-	tok->argv = (char **) tok_malloc(sizeof(char *) * tok->amax);
-	if (tok->argv == NULL) {
-		tok_free(tok);
-		return (NULL);
-	}
-	tok->argv[0] = NULL;
-	tok->wspace = (char *) tok_malloc(WINCR);
-	if (tok->wspace == NULL) {
-		tok_free(tok->argv);
-		tok_free(tok);
-		return (NULL);
-	}
-	tok->wmax = tok->wspace + WINCR;
-	tok->wstart = tok->wspace;
-	tok->wptr = tok->wspace;
-	tok->flags = 0;
-	tok->quote = Q_none;
-
-	return (tok);
-}
-
-
-/* tok_reset():
- *	Reset the tokenizer
- */
-public void
-tok_reset(Tokenizer *tok)
-{
-
-	tok->argc = 0;
-	tok->wstart = tok->wspace;
-	tok->wptr = tok->wspace;
-	tok->flags = 0;
-	tok->quote = Q_none;
-}
-
-
-/* tok_end():
- *	Clean up
- */
-public void
-tok_end(Tokenizer *tok)
-{
-
-	tok_free((ptr_t) tok->ifs);
-	tok_free((ptr_t) tok->wspace);
-	tok_free((ptr_t) tok->argv);
-	tok_free((ptr_t) tok);
-}
-
-
-
-/* tok_line():
- *	Bourne shell like tokenizing
- *	Return:
- *		-1: Internal error
- *		 3: Quoted return
- *		 2: Unmatched double quote
- *		 1: Unmatched single quote
- *		 0: Ok
- */
-public int
-tok_line(Tokenizer *tok, const char *line, int *argc, const char ***argv)
-{
-	const char *ptr;
-
-	for (;;) {
-		switch (*(ptr = line++)) {
-		case '\'':
-			tok->flags |= TOK_KEEP;
-			tok->flags &= ~TOK_EAT;
-			switch (tok->quote) {
-			case Q_none:
-				tok->quote = Q_single;	/* Enter single quote
-							 * mode */
-				break;
-
-			case Q_single:	/* Exit single quote mode */
-				tok->quote = Q_none;
-				break;
-
-			case Q_one:	/* Quote this ' */
-				tok->quote = Q_none;
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_double:	/* Stay in double quote mode */
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_doubleone:	/* Quote this ' */
-				tok->quote = Q_double;
-				*tok->wptr++ = *ptr;
-				break;
-
-			default:
-				return (-1);
-			}
-			break;
-
-		case '"':
-			tok->flags &= ~TOK_EAT;
-			tok->flags |= TOK_KEEP;
-			switch (tok->quote) {
-			case Q_none:	/* Enter double quote mode */
-				tok->quote = Q_double;
-				break;
-
-			case Q_double:	/* Exit double quote mode */
-				tok->quote = Q_none;
-				break;
-
-			case Q_one:	/* Quote this " */
-				tok->quote = Q_none;
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_single:	/* Stay in single quote mode */
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_doubleone:	/* Quote this " */
-				tok->quote = Q_double;
-				*tok->wptr++ = *ptr;
-				break;
-
-			default:
-				return (-1);
-			}
-			break;
-
-		case '\\':
-			tok->flags |= TOK_KEEP;
-			tok->flags &= ~TOK_EAT;
-			switch (tok->quote) {
-			case Q_none:	/* Quote next character */
-				tok->quote = Q_one;
-				break;
-
-			case Q_double:	/* Quote next character */
-				tok->quote = Q_doubleone;
-				break;
-
-			case Q_one:	/* Quote this, restore state */
-				*tok->wptr++ = *ptr;
-				tok->quote = Q_none;
-				break;
-
-			case Q_single:	/* Stay in single quote mode */
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_doubleone:	/* Quote this \ */
-				tok->quote = Q_double;
-				*tok->wptr++ = *ptr;
-				break;
-
-			default:
-				return (-1);
-			}
-			break;
-
-		case '\n':
-			tok->flags &= ~TOK_EAT;
-			switch (tok->quote) {
-			case Q_none:
-				tok_finish(tok);
-				*argv = (const char **)tok->argv;
-				*argc = tok->argc;
-				return (0);
-
-			case Q_single:
-			case Q_double:
-				*tok->wptr++ = *ptr;	/* Add the return */
-				break;
-
-			case Q_doubleone:   /* Back to double, eat the '\n' */
-				tok->flags |= TOK_EAT;
-				tok->quote = Q_double;
-				break;
-
-			case Q_one:	/* No quote, more eat the '\n' */
-				tok->flags |= TOK_EAT;
-				tok->quote = Q_none;
-				break;
-
-			default:
-				return (0);
-			}
-			break;
-
-		case '\0':
-			switch (tok->quote) {
-			case Q_none:
-				/* Finish word and return */
-				if (tok->flags & TOK_EAT) {
-					tok->flags &= ~TOK_EAT;
-					return (3);
-				}
-				tok_finish(tok);
-				*argv = (const char **)tok->argv;
-				*argc = tok->argc;
-				return (0);
-
-			case Q_single:
-				return (1);
-
-			case Q_double:
-				return (2);
-
-			case Q_doubleone:
-				tok->quote = Q_double;
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_one:
-				tok->quote = Q_none;
-				*tok->wptr++ = *ptr;
-				break;
-
-			default:
-				return (-1);
-			}
-			break;
-
-		default:
-			tok->flags &= ~TOK_EAT;
-			switch (tok->quote) {
-			case Q_none:
-				if (strchr(tok->ifs, *ptr) != NULL)
-					tok_finish(tok);
-				else
-					*tok->wptr++ = *ptr;
-				break;
-
-			case Q_single:
-			case Q_double:
-				*tok->wptr++ = *ptr;
-				break;
-
-
-			case Q_doubleone:
-				*tok->wptr++ = '\\';
-				tok->quote = Q_double;
-				*tok->wptr++ = *ptr;
-				break;
-
-			case Q_one:
-				tok->quote = Q_none;
-				*tok->wptr++ = *ptr;
-				break;
-
-			default:
-				return (-1);
-
-			}
-			break;
-		}
-
-		if (tok->wptr >= tok->wmax - 4) {
-			size_t size = tok->wmax - tok->wspace + WINCR;
-			char *s = (char *) tok_realloc(tok->wspace, size);
-			if (s == NULL)
-				return (-1);
-
-			if (s != tok->wspace) {
-				int i;
-				for (i = 0; i < tok->argc; i++) {
-				    tok->argv[i] =
-					(tok->argv[i] - tok->wspace) + s;
-				}
-				tok->wptr = (tok->wptr - tok->wspace) + s;
-				tok->wstart = (tok->wstart - tok->wspace) + s;
-				tok->wspace = s;
-			}
-			tok->wmax = s + size;
-		}
-		if (tok->argc >= tok->amax - 4) {
-			char **p;
-			tok->amax += AINCR;
-			p = (char **) tok_realloc(tok->argv,
-			    tok->amax * sizeof(char *));
-			if (p == NULL)
-				return (-1);
-			tok->argv = p;
-		}
-	}
-}
diff -pruN 1:13.23.1~dfsg-2/main/editline/tokenizer.h 1:16.1.1~dfsg-1/main/editline/tokenizer.h
--- 1:13.23.1~dfsg-2/main/editline/tokenizer.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/tokenizer.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,54 +0,0 @@
-/*	$NetBSD: tokenizer.h,v 1.5 2002/03/18 16:01:00 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)tokenizer.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * tokenizer.h: Header file for tokenizer routines
- */
-#ifndef _h_tokenizer
-#define	_h_tokenizer
-
-typedef struct tokenizer Tokenizer;
-
-Tokenizer	*tok_init(const char *);
-void		 tok_reset(Tokenizer *);
-void		 tok_end(Tokenizer *);
-int		 tok_line(Tokenizer *, const char *, int *, const char ***);
-
-#endif /* _h_tokenizer */
diff -pruN 1:13.23.1~dfsg-2/main/editline/tty.c 1:16.1.1~dfsg-1/main/editline/tty.c
--- 1:13.23.1~dfsg-2/main/editline/tty.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/tty.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,1182 +0,0 @@
-/*	$NetBSD: tty.c,v 1.16 2002/03/18 16:01:01 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)tty.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: tty.c,v 1.16 2002/03/18 16:01:01 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * tty.c: tty interface stuff
- */
-#include "tty.h"
-#include "el.h"
-
-typedef struct ttymodes_t {
-	const char *m_name;
-	u_int m_value;
-	int m_type;
-}          ttymodes_t;
-
-typedef struct ttymap_t {
-	int nch, och;		/* Internal and termio rep of chars */
-	el_action_t bind[3];	/* emacs, vi, and vi-cmd */
-} ttymap_t;
-
-
-private const ttyperm_t ttyperm = {
-	{
-		{"iflag:", ICRNL, (INLCR | IGNCR)},
-		{"oflag:", (OPOST | ONLCR), ONLRET},
-		{"cflag:", 0, 0},
-		{"lflag:", (ISIG | ICANON | ECHO | ECHOE | ECHOCTL | IEXTEN),
-		(NOFLSH | ECHONL | EXTPROC | FLUSHO)},
-		{"chars:", 0, 0},
-	},
-	{
-		{"iflag:", (INLCR | ICRNL), IGNCR},
-		{"oflag:", (OPOST | ONLCR), ONLRET},
-		{"cflag:", 0, 0},
-		{"lflag:", ISIG,
-		(NOFLSH | ICANON | ECHO | ECHOK | ECHONL | EXTPROC | IEXTEN | FLUSHO)},
-		{"chars:", (C_SH(C_MIN) | C_SH(C_TIME) | C_SH(C_SWTCH) | C_SH(C_DSWTCH) |
-			    C_SH(C_SUSP) | C_SH(C_DSUSP) | C_SH(C_EOL) | C_SH(C_DISCARD) |
-		    C_SH(C_PGOFF) | C_SH(C_PAGE) | C_SH(C_STATUS)), 0}
-	},
-	{
-		{"iflag:", 0, IXON | IXOFF | INLCR | ICRNL},
-		{"oflag:", 0, 0},
-		{"cflag:", 0, 0},
-		{"lflag:", 0, ISIG | IEXTEN},
-		{"chars:", 0, 0},
-	}
-};
-
-private const ttychar_t ttychar = {
-	{
-		CINTR, CQUIT, CERASE, CKILL,
-		CEOF, CEOL, CEOL2, CSWTCH,
-		CDSWTCH, CERASE2, CSTART, CSTOP,
-		CWERASE, CSUSP, CDSUSP, CREPRINT,
-		CDISCARD, CLNEXT, CSTATUS, CPAGE,
-		CPGOFF, CKILL2, CBRK, CMIN,
-		CTIME
-	},
-	{
-		CINTR, CQUIT, CERASE, CKILL,
-		_POSIX_VDISABLE, _POSIX_VDISABLE, _POSIX_VDISABLE, _POSIX_VDISABLE,
-		_POSIX_VDISABLE, CERASE2, CSTART, CSTOP,
-		_POSIX_VDISABLE, CSUSP, _POSIX_VDISABLE, _POSIX_VDISABLE,
-		CDISCARD, _POSIX_VDISABLE, _POSIX_VDISABLE, _POSIX_VDISABLE,
-		_POSIX_VDISABLE, _POSIX_VDISABLE, _POSIX_VDISABLE, 1,
-		0
-	},
-	{
-		0, 0, 0, 0,
-		0, 0, 0, 0,
-		0, 0, 0, 0,
-		0, 0, 0, 0,
-		0, 0, 0, 0,
-		0, 0, 0, 0,
-		0
-	}
-};
-
-private const ttymap_t tty_map[] = {
-#ifdef VERASE
-	{C_ERASE, VERASE,
-	{ED_DELETE_PREV_CHAR, VI_DELETE_PREV_CHAR, ED_PREV_CHAR}},
-#endif /* VERASE */
-#ifdef VERASE2
-	{C_ERASE2, VERASE2,
-	{ED_DELETE_PREV_CHAR, VI_DELETE_PREV_CHAR, ED_PREV_CHAR}},
-#endif /* VERASE2 */
-#ifdef VKILL
-	{C_KILL, VKILL,
-	{EM_KILL_LINE, VI_KILL_LINE_PREV, ED_UNASSIGNED}},
-#endif /* VKILL */
-#ifdef VKILL2
-	{C_KILL2, VKILL2,
-	{EM_KILL_LINE, VI_KILL_LINE_PREV, ED_UNASSIGNED}},
-#endif /* VKILL2 */
-#ifdef VEOF
-	{C_EOF, VEOF,
-	{EM_DELETE_OR_LIST, VI_LIST_OR_EOF, ED_UNASSIGNED}},
-#endif /* VEOF */
-#ifdef VWERASE
-	{C_WERASE, VWERASE,
-	{ED_DELETE_PREV_WORD, ED_DELETE_PREV_WORD, ED_PREV_WORD}},
-#endif /* VWERASE */
-#ifdef VREPRINT
-	{C_REPRINT, VREPRINT,
-	{ED_REDISPLAY, ED_INSERT, ED_REDISPLAY}},
-#endif /* VREPRINT */
-#ifdef VLNEXT
-	{C_LNEXT, VLNEXT,
-	{ED_QUOTED_INSERT, ED_QUOTED_INSERT, ED_UNASSIGNED}},
-#endif /* VLNEXT */
-	{-1, -1,
-	{ED_UNASSIGNED, ED_UNASSIGNED, ED_UNASSIGNED}}
-};
-
-private const ttymodes_t ttymodes[] = {
-#ifdef	IGNBRK
-	{"ignbrk", IGNBRK, MD_INP},
-#endif /* IGNBRK */
-#ifdef	BRKINT
-	{"brkint", BRKINT, MD_INP},
-#endif /* BRKINT */
-#ifdef	IGNPAR
-	{"ignpar", IGNPAR, MD_INP},
-#endif /* IGNPAR */
-#ifdef	PARMRK
-	{"parmrk", PARMRK, MD_INP},
-#endif /* PARMRK */
-#ifdef	INPCK
-	{"inpck", INPCK, MD_INP},
-#endif /* INPCK */
-#ifdef	ISTRIP
-	{"istrip", ISTRIP, MD_INP},
-#endif /* ISTRIP */
-#ifdef	INLCR
-	{"inlcr", INLCR, MD_INP},
-#endif /* INLCR */
-#ifdef	IGNCR
-	{"igncr", IGNCR, MD_INP},
-#endif /* IGNCR */
-#ifdef	ICRNL
-	{"icrnl", ICRNL, MD_INP},
-#endif /* ICRNL */
-#ifdef	IUCLC
-	{"iuclc", IUCLC, MD_INP},
-#endif /* IUCLC */
-#ifdef	IXON
-	{"ixon", IXON, MD_INP},
-#endif /* IXON */
-#ifdef	IXANY
-	{"ixany", IXANY, MD_INP},
-#endif /* IXANY */
-#ifdef	IXOFF
-	{"ixoff", IXOFF, MD_INP},
-#endif /* IXOFF */
-#ifdef  IMAXBEL
-	{"imaxbel", IMAXBEL, MD_INP},
-#endif /* IMAXBEL */
-
-#ifdef	OPOST
-	{"opost", OPOST, MD_OUT},
-#endif /* OPOST */
-#ifdef	OLCUC
-	{"olcuc", OLCUC, MD_OUT},
-#endif /* OLCUC */
-#ifdef	ONLCR
-	{"onlcr", ONLCR, MD_OUT},
-#endif /* ONLCR */
-#ifdef	OCRNL
-	{"ocrnl", OCRNL, MD_OUT},
-#endif /* OCRNL */
-#ifdef	ONOCR
-	{"onocr", ONOCR, MD_OUT},
-#endif /* ONOCR */
-#ifdef ONOEOT
-	{"onoeot", ONOEOT, MD_OUT},
-#endif /* ONOEOT */
-#ifdef	ONLRET
-	{"onlret", ONLRET, MD_OUT},
-#endif /* ONLRET */
-#ifdef	OFILL
-	{"ofill", OFILL, MD_OUT},
-#endif /* OFILL */
-#ifdef	OFDEL
-	{"ofdel", OFDEL, MD_OUT},
-#endif /* OFDEL */
-#ifdef	NLDLY
-	{"nldly", NLDLY, MD_OUT},
-#endif /* NLDLY */
-#ifdef	CRDLY
-	{"crdly", CRDLY, MD_OUT},
-#endif /* CRDLY */
-#ifdef	TABDLY
-	{"tabdly", TABDLY, MD_OUT},
-#endif /* TABDLY */
-#ifdef	XTABS
-	{"xtabs", XTABS, MD_OUT},
-#endif /* XTABS */
-#ifdef	BSDLY
-	{"bsdly", BSDLY, MD_OUT},
-#endif /* BSDLY */
-#ifdef	VTDLY
-	{"vtdly", VTDLY, MD_OUT},
-#endif /* VTDLY */
-#ifdef	FFDLY
-	{"ffdly", FFDLY, MD_OUT},
-#endif /* FFDLY */
-#ifdef	PAGEOUT
-	{"pageout", PAGEOUT, MD_OUT},
-#endif /* PAGEOUT */
-#ifdef	WRAP
-	{"wrap", WRAP, MD_OUT},
-#endif /* WRAP */
-
-#ifdef	CIGNORE
-	{"cignore", CIGNORE, MD_CTL},
-#endif /* CBAUD */
-#ifdef	CBAUD
-	{"cbaud", CBAUD, MD_CTL},
-#endif /* CBAUD */
-#ifdef	CSTOPB
-	{"cstopb", CSTOPB, MD_CTL},
-#endif /* CSTOPB */
-#ifdef	CREAD
-	{"cread", CREAD, MD_CTL},
-#endif /* CREAD */
-#ifdef	PARENB
-	{"parenb", PARENB, MD_CTL},
-#endif /* PARENB */
-#ifdef	PARODD
-	{"parodd", PARODD, MD_CTL},
-#endif /* PARODD */
-#ifdef	HUPCL
-	{"hupcl", HUPCL, MD_CTL},
-#endif /* HUPCL */
-#ifdef	CLOCAL
-	{"clocal", CLOCAL, MD_CTL},
-#endif /* CLOCAL */
-#ifdef	LOBLK
-	{"loblk", LOBLK, MD_CTL},
-#endif /* LOBLK */
-#ifdef	CIBAUD
-	{"cibaud", CIBAUD, MD_CTL},
-#endif /* CIBAUD */
-#ifdef CRTSCTS
-#ifdef CCTS_OFLOW
-	{"ccts_oflow", CCTS_OFLOW, MD_CTL},
-#else
-	{"crtscts", CRTSCTS, MD_CTL},
-#endif /* CCTS_OFLOW */
-#endif /* CRTSCTS */
-#ifdef CRTS_IFLOW
-	{"crts_iflow", CRTS_IFLOW, MD_CTL},
-#endif /* CRTS_IFLOW */
-#ifdef CDTRCTS
-	{"cdtrcts", CDTRCTS, MD_CTL},
-#endif /* CDTRCTS */
-#ifdef MDMBUF
-	{"mdmbuf", MDMBUF, MD_CTL},
-#endif /* MDMBUF */
-#ifdef RCV1EN
-	{"rcv1en", RCV1EN, MD_CTL},
-#endif /* RCV1EN */
-#ifdef XMT1EN
-	{"xmt1en", XMT1EN, MD_CTL},
-#endif /* XMT1EN */
-
-#ifdef	ISIG
-	{"isig", ISIG, MD_LIN},
-#endif /* ISIG */
-#ifdef	ICANON
-	{"icanon", ICANON, MD_LIN},
-#endif /* ICANON */
-#ifdef	XCASE
-	{"xcase", XCASE, MD_LIN},
-#endif /* XCASE */
-#ifdef	ECHO
-	{"echo", ECHO, MD_LIN},
-#endif /* ECHO */
-#ifdef	ECHOE
-	{"echoe", ECHOE, MD_LIN},
-#endif /* ECHOE */
-#ifdef	ECHOK
-	{"echok", ECHOK, MD_LIN},
-#endif /* ECHOK */
-#ifdef	ECHONL
-	{"echonl", ECHONL, MD_LIN},
-#endif /* ECHONL */
-#ifdef	NOFLSH
-	{"noflsh", NOFLSH, MD_LIN},
-#endif /* NOFLSH */
-#ifdef	TOSTOP
-	{"tostop", TOSTOP, MD_LIN},
-#endif /* TOSTOP */
-#ifdef	ECHOCTL
-	{"echoctl", ECHOCTL, MD_LIN},
-#endif /* ECHOCTL */
-#ifdef	ECHOPRT
-	{"echoprt", ECHOPRT, MD_LIN},
-#endif /* ECHOPRT */
-#ifdef	ECHOKE
-	{"echoke", ECHOKE, MD_LIN},
-#endif /* ECHOKE */
-#ifdef	DEFECHO
-	{"defecho", DEFECHO, MD_LIN},
-#endif /* DEFECHO */
-#ifdef	FLUSHO
-	{"flusho", FLUSHO, MD_LIN},
-#endif /* FLUSHO */
-#ifdef	PENDIN
-	{"pendin", PENDIN, MD_LIN},
-#endif /* PENDIN */
-#ifdef	IEXTEN
-	{"iexten", IEXTEN, MD_LIN},
-#endif /* IEXTEN */
-#ifdef	NOKERNINFO
-	{"nokerninfo", NOKERNINFO, MD_LIN},
-#endif /* NOKERNINFO */
-#ifdef	ALTWERASE
-	{"altwerase", ALTWERASE, MD_LIN},
-#endif /* ALTWERASE */
-#ifdef	EXTPROC
-	{"extproc", EXTPROC, MD_LIN},
-#endif /* EXTPROC */
-
-#if defined(VINTR)
-	{"intr", C_SH(C_INTR), MD_CHAR},
-#endif /* VINTR */
-#if defined(VQUIT)
-	{"quit", C_SH(C_QUIT), MD_CHAR},
-#endif /* VQUIT */
-#if defined(VERASE)
-	{"erase", C_SH(C_ERASE), MD_CHAR},
-#endif /* VERASE */
-#if defined(VKILL)
-	{"kill", C_SH(C_KILL), MD_CHAR},
-#endif /* VKILL */
-#if defined(VEOF)
-	{"eof", C_SH(C_EOF), MD_CHAR},
-#endif /* VEOF */
-#if defined(VEOL)
-	{"eol", C_SH(C_EOL), MD_CHAR},
-#endif /* VEOL */
-#if defined(VEOL2)
-	{"eol2", C_SH(C_EOL2), MD_CHAR},
-#endif /* VEOL2 */
-#if defined(VSWTCH)
-	{"swtch", C_SH(C_SWTCH), MD_CHAR},
-#endif /* VSWTCH */
-#if defined(VDSWTCH)
-	{"dswtch", C_SH(C_DSWTCH), MD_CHAR},
-#endif /* VDSWTCH */
-#if defined(VERASE2)
-	{"erase2", C_SH(C_ERASE2), MD_CHAR},
-#endif /* VERASE2 */
-#if defined(VSTART)
-	{"start", C_SH(C_START), MD_CHAR},
-#endif /* VSTART */
-#if defined(VSTOP)
-	{"stop", C_SH(C_STOP), MD_CHAR},
-#endif /* VSTOP */
-#if defined(VWERASE)
-	{"werase", C_SH(C_WERASE), MD_CHAR},
-#endif /* VWERASE */
-#if defined(VSUSP)
-	{"susp", C_SH(C_SUSP), MD_CHAR},
-#endif /* VSUSP */
-#if defined(VDSUSP)
-	{"dsusp", C_SH(C_DSUSP), MD_CHAR},
-#endif /* VDSUSP */
-#if defined(VREPRINT)
-	{"reprint", C_SH(C_REPRINT), MD_CHAR},
-#endif /* VREPRINT */
-#if defined(VDISCARD)
-	{"discard", C_SH(C_DISCARD), MD_CHAR},
-#endif /* VDISCARD */
-#if defined(VLNEXT)
-	{"lnext", C_SH(C_LNEXT), MD_CHAR},
-#endif /* VLNEXT */
-#if defined(VSTATUS)
-	{"status", C_SH(C_STATUS), MD_CHAR},
-#endif /* VSTATUS */
-#if defined(VPAGE)
-	{"page", C_SH(C_PAGE), MD_CHAR},
-#endif /* VPAGE */
-#if defined(VPGOFF)
-	{"pgoff", C_SH(C_PGOFF), MD_CHAR},
-#endif /* VPGOFF */
-#if defined(VKILL2)
-	{"kill2", C_SH(C_KILL2), MD_CHAR},
-#endif /* VKILL2 */
-#if defined(VBRK)
-	{"brk", C_SH(C_BRK), MD_CHAR},
-#endif /* VBRK */
-#if defined(VMIN)
-	{"min", C_SH(C_MIN), MD_CHAR},
-#endif /* VMIN */
-#if defined(VTIME)
-	{"time", C_SH(C_TIME), MD_CHAR},
-#endif /* VTIME */
-	{NULL, 0, -1},
-};
-
-
-
-#define	tty_getty(el, td)	tcgetattr((el)->el_infd, (td))
-#define	tty_setty(el, td)	tcsetattr((el)->el_infd, TCSADRAIN, (td))
-
-#define	tty__gettabs(td)	((((td)->c_oflag & TAB3) == TAB3) ? 0 : 1)
-#define	tty__geteightbit(td)	(((td)->c_cflag & CSIZE) == CS8)
-#define	tty__cooked_mode(td)	((td)->c_lflag & ICANON)
-
-private void	tty__getchar(struct termios *, unsigned char *);
-private void	tty__setchar(struct termios *, unsigned char *);
-private speed_t	tty__getspeed(struct termios *);
-private int	tty_setup(EditLine *);
-
-#define	t_qu	t_ts
-
-
-/* tty_setup():
- *	Get the tty parameters and initialize the editing state
- */
-private int
-tty_setup(EditLine *el)
-{
-	int rst = 1;
-
-	if (el->el_flags & EDIT_DISABLED)
-		return (0);
-
-	if (tty_getty(el, &el->el_tty.t_ed) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile,
-		    "tty_setup: tty_getty: %s\n", strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	el->el_tty.t_ts = el->el_tty.t_ex = el->el_tty.t_ed;
-
-	el->el_tty.t_speed = tty__getspeed(&el->el_tty.t_ex);
-	el->el_tty.t_tabs = tty__gettabs(&el->el_tty.t_ex);
-	el->el_tty.t_eight = tty__geteightbit(&el->el_tty.t_ex);
-
-	el->el_tty.t_ex.c_iflag &= ~el->el_tty.t_t[EX_IO][MD_INP].t_clrmask;
-	el->el_tty.t_ex.c_iflag |= el->el_tty.t_t[EX_IO][MD_INP].t_setmask;
-
-	el->el_tty.t_ex.c_oflag &= ~el->el_tty.t_t[EX_IO][MD_OUT].t_clrmask;
-	el->el_tty.t_ex.c_oflag |= el->el_tty.t_t[EX_IO][MD_OUT].t_setmask;
-
-	el->el_tty.t_ex.c_cflag &= ~el->el_tty.t_t[EX_IO][MD_CTL].t_clrmask;
-	el->el_tty.t_ex.c_cflag |= el->el_tty.t_t[EX_IO][MD_CTL].t_setmask;
-
-	el->el_tty.t_ex.c_lflag &= ~el->el_tty.t_t[EX_IO][MD_LIN].t_clrmask;
-	el->el_tty.t_ex.c_lflag |= el->el_tty.t_t[EX_IO][MD_LIN].t_setmask;
-
-	/*
-         * Reset the tty chars to reasonable defaults
-         * If they are disabled, then enable them.
-         */
-	if (rst) {
-		if (tty__cooked_mode(&el->el_tty.t_ts)) {
-			tty__getchar(&el->el_tty.t_ts, el->el_tty.t_c[TS_IO]);
-			/*
-	                 * Don't affect CMIN and CTIME for the editor mode
-	                 */
-			for (rst = 0; rst < C_NCC - 2; rst++)
-				if (el->el_tty.t_c[TS_IO][rst] !=
-				      el->el_tty.t_vdisable
-				    && el->el_tty.t_c[ED_IO][rst] !=
-				      el->el_tty.t_vdisable)
-					el->el_tty.t_c[ED_IO][rst] =
-					    el->el_tty.t_c[TS_IO][rst];
-			for (rst = 0; rst < C_NCC; rst++)
-				if (el->el_tty.t_c[TS_IO][rst] !=
-				    el->el_tty.t_vdisable)
-					el->el_tty.t_c[EX_IO][rst] =
-					    el->el_tty.t_c[TS_IO][rst];
-		}
-		tty__setchar(&el->el_tty.t_ex, el->el_tty.t_c[EX_IO]);
-		if (tty_setty(el, &el->el_tty.t_ex) == -1) {
-#ifdef DEBUG_TTY
-			(void) fprintf(el->el_errfile,
-			    "tty_setup: tty_setty: %s\n",
-			    strerror(errno));
-#endif /* DEBUG_TTY */
-			return (-1);
-		}
-	} else
-		tty__setchar(&el->el_tty.t_ex, el->el_tty.t_c[EX_IO]);
-
-	el->el_tty.t_ed.c_iflag &= ~el->el_tty.t_t[ED_IO][MD_INP].t_clrmask;
-	el->el_tty.t_ed.c_iflag |= el->el_tty.t_t[ED_IO][MD_INP].t_setmask;
-
-	el->el_tty.t_ed.c_oflag &= ~el->el_tty.t_t[ED_IO][MD_OUT].t_clrmask;
-	el->el_tty.t_ed.c_oflag |= el->el_tty.t_t[ED_IO][MD_OUT].t_setmask;
-
-	el->el_tty.t_ed.c_cflag &= ~el->el_tty.t_t[ED_IO][MD_CTL].t_clrmask;
-	el->el_tty.t_ed.c_cflag |= el->el_tty.t_t[ED_IO][MD_CTL].t_setmask;
-
-	el->el_tty.t_ed.c_lflag &= ~el->el_tty.t_t[ED_IO][MD_LIN].t_clrmask;
-	el->el_tty.t_ed.c_lflag |= el->el_tty.t_t[ED_IO][MD_LIN].t_setmask;
-
-	tty__setchar(&el->el_tty.t_ed, el->el_tty.t_c[ED_IO]);
-	tty_bind_char(el, 1);
-	return (0);
-}
-
-protected int
-tty_init(EditLine *el)
-{
-
-	el->el_tty.t_mode = EX_IO;
-	el->el_tty.t_vdisable = _POSIX_VDISABLE;
-	(void) memcpy(el->el_tty.t_t, ttyperm, sizeof(ttyperm_t));
-	(void) memcpy(el->el_tty.t_c, ttychar, sizeof(ttychar_t));
-	return (tty_setup(el));
-}
-
-
-/* tty_end():
- *	Restore the tty to its original settings
- */
-protected void
-/*ARGSUSED*/
-tty_end(EditLine *el)
-{
-
-	/* XXX: Maybe reset to an initial state? */
-}
-
-
-/* tty__getspeed():
- *	Get the tty speed
- */
-private speed_t
-tty__getspeed(struct termios *td)
-{
-	speed_t spd;
-
-	if ((spd = cfgetispeed(td)) == 0)
-		spd = cfgetospeed(td);
-	return (spd);
-}
-
-
-/* tty__getchar():
- *	Get the tty characters
- */
-private void
-tty__getchar(struct termios *td, unsigned char *s)
-{
-
-#ifdef VINTR
-	s[C_INTR] = td->c_cc[VINTR];
-#endif /* VINTR */
-#ifdef VQUIT
-	s[C_QUIT] = td->c_cc[VQUIT];
-#endif /* VQUIT */
-#ifdef VERASE
-	s[C_ERASE] = td->c_cc[VERASE];
-#endif /* VERASE */
-#ifdef VKILL
-	s[C_KILL] = td->c_cc[VKILL];
-#endif /* VKILL */
-#ifdef VEOF
-	s[C_EOF] = td->c_cc[VEOF];
-#endif /* VEOF */
-#ifdef VEOL
-	s[C_EOL] = td->c_cc[VEOL];
-#endif /* VEOL */
-#ifdef VEOL2
-	s[C_EOL2] = td->c_cc[VEOL2];
-#endif /* VEOL2 */
-#ifdef VSWTCH
-	s[C_SWTCH] = td->c_cc[VSWTCH];
-#endif /* VSWTCH */
-#ifdef VDSWTCH
-	s[C_DSWTCH] = td->c_cc[VDSWTCH];
-#endif /* VDSWTCH */
-#ifdef VERASE2
-	s[C_ERASE2] = td->c_cc[VERASE2];
-#endif /* VERASE2 */
-#ifdef VSTART
-	s[C_START] = td->c_cc[VSTART];
-#endif /* VSTART */
-#ifdef VSTOP
-	s[C_STOP] = td->c_cc[VSTOP];
-#endif /* VSTOP */
-#ifdef VWERASE
-	s[C_WERASE] = td->c_cc[VWERASE];
-#endif /* VWERASE */
-#ifdef VSUSP
-	s[C_SUSP] = td->c_cc[VSUSP];
-#endif /* VSUSP */
-#ifdef VDSUSP
-	s[C_DSUSP] = td->c_cc[VDSUSP];
-#endif /* VDSUSP */
-#ifdef VREPRINT
-	s[C_REPRINT] = td->c_cc[VREPRINT];
-#endif /* VREPRINT */
-#ifdef VDISCARD
-	s[C_DISCARD] = td->c_cc[VDISCARD];
-#endif /* VDISCARD */
-#ifdef VLNEXT
-	s[C_LNEXT] = td->c_cc[VLNEXT];
-#endif /* VLNEXT */
-#ifdef VSTATUS
-	s[C_STATUS] = td->c_cc[VSTATUS];
-#endif /* VSTATUS */
-#ifdef VPAGE
-	s[C_PAGE] = td->c_cc[VPAGE];
-#endif /* VPAGE */
-#ifdef VPGOFF
-	s[C_PGOFF] = td->c_cc[VPGOFF];
-#endif /* VPGOFF */
-#ifdef VKILL2
-	s[C_KILL2] = td->c_cc[VKILL2];
-#endif /* KILL2 */
-#ifdef VMIN
-	s[C_MIN] = td->c_cc[VMIN];
-#endif /* VMIN */
-#ifdef VTIME
-	s[C_TIME] = td->c_cc[VTIME];
-#endif /* VTIME */
-}				/* tty__getchar */
-
-
-/* tty__setchar():
- *	Set the tty characters
- */
-private void
-tty__setchar(struct termios *td, unsigned char *s)
-{
-
-#ifdef VINTR
-	td->c_cc[VINTR] = s[C_INTR];
-#endif /* VINTR */
-#ifdef VQUIT
-	td->c_cc[VQUIT] = s[C_QUIT];
-#endif /* VQUIT */
-#ifdef VERASE
-	td->c_cc[VERASE] = s[C_ERASE];
-#endif /* VERASE */
-#ifdef VKILL
-	td->c_cc[VKILL] = s[C_KILL];
-#endif /* VKILL */
-#ifdef VEOF
-	td->c_cc[VEOF] = s[C_EOF];
-#endif /* VEOF */
-#ifdef VEOL
-	td->c_cc[VEOL] = s[C_EOL];
-#endif /* VEOL */
-#ifdef VEOL2
-	td->c_cc[VEOL2] = s[C_EOL2];
-#endif /* VEOL2 */
-#ifdef VSWTCH
-	td->c_cc[VSWTCH] = s[C_SWTCH];
-#endif /* VSWTCH */
-#ifdef VDSWTCH
-	td->c_cc[VDSWTCH] = s[C_DSWTCH];
-#endif /* VDSWTCH */
-#ifdef VERASE2
-	td->c_cc[VERASE2] = s[C_ERASE2];
-#endif /* VERASE2 */
-#ifdef VSTART
-	td->c_cc[VSTART] = s[C_START];
-#endif /* VSTART */
-#ifdef VSTOP
-	td->c_cc[VSTOP] = s[C_STOP];
-#endif /* VSTOP */
-#ifdef VWERASE
-	td->c_cc[VWERASE] = s[C_WERASE];
-#endif /* VWERASE */
-#ifdef VSUSP
-	td->c_cc[VSUSP] = s[C_SUSP];
-#endif /* VSUSP */
-#ifdef VDSUSP
-	td->c_cc[VDSUSP] = s[C_DSUSP];
-#endif /* VDSUSP */
-#ifdef VREPRINT
-	td->c_cc[VREPRINT] = s[C_REPRINT];
-#endif /* VREPRINT */
-#ifdef VDISCARD
-	td->c_cc[VDISCARD] = s[C_DISCARD];
-#endif /* VDISCARD */
-#ifdef VLNEXT
-	td->c_cc[VLNEXT] = s[C_LNEXT];
-#endif /* VLNEXT */
-#ifdef VSTATUS
-	td->c_cc[VSTATUS] = s[C_STATUS];
-#endif /* VSTATUS */
-#ifdef VPAGE
-	td->c_cc[VPAGE] = s[C_PAGE];
-#endif /* VPAGE */
-#ifdef VPGOFF
-	td->c_cc[VPGOFF] = s[C_PGOFF];
-#endif /* VPGOFF */
-#ifdef VKILL2
-	td->c_cc[VKILL2] = s[C_KILL2];
-#endif /* VKILL2 */
-#ifdef VMIN
-	td->c_cc[VMIN] = s[C_MIN];
-#endif /* VMIN */
-#ifdef VTIME
-	td->c_cc[VTIME] = s[C_TIME];
-#endif /* VTIME */
-}				/* tty__setchar */
-
-
-/* tty_bind_char():
- *	Rebind the editline functions
- */
-protected void
-tty_bind_char(EditLine *el, int force)
-{
-
-	unsigned char *t_n = el->el_tty.t_c[ED_IO];
-	unsigned char *t_o = el->el_tty.t_ed.c_cc;
-	unsigned char new[2], old[2];
-	const ttymap_t *tp;
-	el_action_t *map, *alt;
-	const el_action_t *dmap, *dalt;
-	new[1] = old[1] = '\0';
-
-	map = el->el_map.key;
-	alt = el->el_map.alt;
-	if (el->el_map.type == MAP_VI) {
-		dmap = el->el_map.vii;
-		dalt = el->el_map.vic;
-	} else {
-		dmap = el->el_map.emacs;
-		dalt = NULL;
-	}
-
-	for (tp = tty_map; tp->nch != -1; tp++) {
-		new[0] = t_n[tp->nch];
-		old[0] = t_o[tp->och];
-		if (new[0] == old[0] && !force)
-			continue;
-		/* Put the old default binding back, and set the new binding */
-		key_clear(el, map, (char *)old);
-		map[old[0]] = dmap[old[0]];
-		key_clear(el, map, (char *)new);
-		/* MAP_VI == 1, MAP_EMACS == 0... */
-		map[new[0]] = tp->bind[el->el_map.type];
-		if (dalt) {
-			key_clear(el, alt, (char *)old);
-			alt[old[0]] = dalt[old[0]];
-			key_clear(el, alt, (char *)new);
-			alt[new[0]] = tp->bind[el->el_map.type + 1];
-		}
-	}
-}
-
-
-/* tty_rawmode():
- * 	Set terminal into 1 character at a time mode.
- */
-protected int
-tty_rawmode(EditLine *el)
-{
-
-	if (el->el_tty.t_mode == ED_IO || el->el_tty.t_mode == QU_IO)
-		return (0);
-
-	if (el->el_flags & EDIT_DISABLED)
-		return (0);
-
-	if (tty_getty(el, &el->el_tty.t_ts) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile, "tty_rawmode: tty_getty: %s\n",
-		    strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	/*
-         * We always keep up with the eight bit setting and the speed of the
-         * tty. But only we only believe changes that are made to cooked mode!
-         */
-	el->el_tty.t_eight = tty__geteightbit(&el->el_tty.t_ts);
-	el->el_tty.t_speed = tty__getspeed(&el->el_tty.t_ts);
-
-	if (tty__getspeed(&el->el_tty.t_ex) != el->el_tty.t_speed ||
-	    tty__getspeed(&el->el_tty.t_ed) != el->el_tty.t_speed) {
-		(void) cfsetispeed(&el->el_tty.t_ex, el->el_tty.t_speed);
-		(void) cfsetospeed(&el->el_tty.t_ex, el->el_tty.t_speed);
-		(void) cfsetispeed(&el->el_tty.t_ed, el->el_tty.t_speed);
-		(void) cfsetospeed(&el->el_tty.t_ed, el->el_tty.t_speed);
-	}
-	if (tty__cooked_mode(&el->el_tty.t_ts)) {
-		if (el->el_tty.t_ts.c_cflag != el->el_tty.t_ex.c_cflag) {
-			el->el_tty.t_ex.c_cflag =
-			    el->el_tty.t_ts.c_cflag;
-			el->el_tty.t_ex.c_cflag &=
-			    ~el->el_tty.t_t[EX_IO][MD_CTL].t_clrmask;
-			el->el_tty.t_ex.c_cflag |=
-			    el->el_tty.t_t[EX_IO][MD_CTL].t_setmask;
-
-			el->el_tty.t_ed.c_cflag =
-			    el->el_tty.t_ts.c_cflag;
-			el->el_tty.t_ed.c_cflag &=
-			    ~el->el_tty.t_t[ED_IO][MD_CTL].t_clrmask;
-			el->el_tty.t_ed.c_cflag |=
-			    el->el_tty.t_t[ED_IO][MD_CTL].t_setmask;
-		}
-		if ((el->el_tty.t_ts.c_lflag != el->el_tty.t_ex.c_lflag) &&
-		    (el->el_tty.t_ts.c_lflag != el->el_tty.t_ed.c_lflag)) {
-			el->el_tty.t_ex.c_lflag =
-			    el->el_tty.t_ts.c_lflag;
-			el->el_tty.t_ex.c_lflag &=
-			    ~el->el_tty.t_t[EX_IO][MD_LIN].t_clrmask;
-			el->el_tty.t_ex.c_lflag |=
-			    el->el_tty.t_t[EX_IO][MD_LIN].t_setmask;
-
-			el->el_tty.t_ed.c_lflag =
-			    el->el_tty.t_ts.c_lflag;
-			el->el_tty.t_ed.c_lflag &=
-			    ~el->el_tty.t_t[ED_IO][MD_LIN].t_clrmask;
-			el->el_tty.t_ed.c_lflag |=
-			    el->el_tty.t_t[ED_IO][MD_LIN].t_setmask;
-		}
-		if ((el->el_tty.t_ts.c_iflag != el->el_tty.t_ex.c_iflag) &&
-		    (el->el_tty.t_ts.c_iflag != el->el_tty.t_ed.c_iflag)) {
-			el->el_tty.t_ex.c_iflag =
-			    el->el_tty.t_ts.c_iflag;
-			el->el_tty.t_ex.c_iflag &=
-			    ~el->el_tty.t_t[EX_IO][MD_INP].t_clrmask;
-			el->el_tty.t_ex.c_iflag |=
-			    el->el_tty.t_t[EX_IO][MD_INP].t_setmask;
-
-			el->el_tty.t_ed.c_iflag =
-			    el->el_tty.t_ts.c_iflag;
-			el->el_tty.t_ed.c_iflag &=
-			    ~el->el_tty.t_t[ED_IO][MD_INP].t_clrmask;
-			el->el_tty.t_ed.c_iflag |=
-			    el->el_tty.t_t[ED_IO][MD_INP].t_setmask;
-		}
-		if ((el->el_tty.t_ts.c_oflag != el->el_tty.t_ex.c_oflag) &&
-		    (el->el_tty.t_ts.c_oflag != el->el_tty.t_ed.c_oflag)) {
-			el->el_tty.t_ex.c_oflag =
-			    el->el_tty.t_ts.c_oflag;
-			el->el_tty.t_ex.c_oflag &=
-			    ~el->el_tty.t_t[EX_IO][MD_OUT].t_clrmask;
-			el->el_tty.t_ex.c_oflag |=
-			    el->el_tty.t_t[EX_IO][MD_OUT].t_setmask;
-
-			el->el_tty.t_ed.c_oflag =
-			    el->el_tty.t_ts.c_oflag;
-			el->el_tty.t_ed.c_oflag &=
-			    ~el->el_tty.t_t[ED_IO][MD_OUT].t_clrmask;
-			el->el_tty.t_ed.c_oflag |=
-			    el->el_tty.t_t[ED_IO][MD_OUT].t_setmask;
-		}
-		if (tty__gettabs(&el->el_tty.t_ex) == 0)
-			el->el_tty.t_tabs = 0;
-		else
-			el->el_tty.t_tabs = EL_CAN_TAB ? 1 : 0;
-
-		{
-			int i;
-
-			tty__getchar(&el->el_tty.t_ts, el->el_tty.t_c[TS_IO]);
-			/*
-		         * Check if the user made any changes.
-		         * If he did, then propagate the changes to the
-		         * edit and execute data structures.
-		         */
-			for (i = 0; i < C_NCC; i++)
-				if (el->el_tty.t_c[TS_IO][i] !=
-				    el->el_tty.t_c[EX_IO][i])
-					break;
-
-			if (i != C_NCC) {
-				/*
-				 * Propagate changes only to the unprotected
-				 * chars that have been modified just now.
-				 */
-				for (i = 0; i < C_NCC; i++) {
-					if (!((el->el_tty.t_t[ED_IO][MD_CHAR].t_setmask & C_SH(i)))
-					    && (el->el_tty.t_c[TS_IO][i] != el->el_tty.t_c[EX_IO][i]))
-						el->el_tty.t_c[ED_IO][i] = el->el_tty.t_c[TS_IO][i];
-					if (el->el_tty.t_t[ED_IO][MD_CHAR].t_clrmask & C_SH(i))
-						el->el_tty.t_c[ED_IO][i] = el->el_tty.t_vdisable;
-				}
-				tty_bind_char(el, 0);
-				tty__setchar(&el->el_tty.t_ed, el->el_tty.t_c[ED_IO]);
-
-				for (i = 0; i < C_NCC; i++) {
-					if (!((el->el_tty.t_t[EX_IO][MD_CHAR].t_setmask & C_SH(i)))
-					    && (el->el_tty.t_c[TS_IO][i] != el->el_tty.t_c[EX_IO][i]))
-						el->el_tty.t_c[EX_IO][i] = el->el_tty.t_c[TS_IO][i];
-					if (el->el_tty.t_t[EX_IO][MD_CHAR].t_clrmask & C_SH(i))
-						el->el_tty.t_c[EX_IO][i] = el->el_tty.t_vdisable;
-				}
-				tty__setchar(&el->el_tty.t_ex, el->el_tty.t_c[EX_IO]);
-			}
-		}
-	}
-	if (tty_setty(el, &el->el_tty.t_ed) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile, "tty_rawmode: tty_setty: %s\n",
-		    strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	el->el_tty.t_mode = ED_IO;
-	return (0);
-}
-
-
-/* tty_cookedmode():
- *	Set the tty back to normal mode
- */
-protected int
-tty_cookedmode(EditLine *el)
-{				/* set tty in normal setup */
-
-	if (el->el_tty.t_mode == EX_IO)
-		return (0);
-
-	if (el->el_flags & EDIT_DISABLED)
-		return (0);
-
-	if (tty_setty(el, &el->el_tty.t_ex) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile,
-		    "tty_cookedmode: tty_setty: %s\n",
-		    strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	el->el_tty.t_mode = EX_IO;
-	return (0);
-}
-
-
-/* tty_quotemode():
- *	Turn on quote mode
- */
-protected int
-tty_quotemode(EditLine *el)
-{
-	if (el->el_tty.t_mode == QU_IO)
-		return (0);
-
-	el->el_tty.t_qu = el->el_tty.t_ed;
-
-	el->el_tty.t_qu.c_iflag &= ~el->el_tty.t_t[QU_IO][MD_INP].t_clrmask;
-	el->el_tty.t_qu.c_iflag |= el->el_tty.t_t[QU_IO][MD_INP].t_setmask;
-
-	el->el_tty.t_qu.c_oflag &= ~el->el_tty.t_t[QU_IO][MD_OUT].t_clrmask;
-	el->el_tty.t_qu.c_oflag |= el->el_tty.t_t[QU_IO][MD_OUT].t_setmask;
-
-	el->el_tty.t_qu.c_cflag &= ~el->el_tty.t_t[QU_IO][MD_CTL].t_clrmask;
-	el->el_tty.t_qu.c_cflag |= el->el_tty.t_t[QU_IO][MD_CTL].t_setmask;
-
-	el->el_tty.t_qu.c_lflag &= ~el->el_tty.t_t[QU_IO][MD_LIN].t_clrmask;
-	el->el_tty.t_qu.c_lflag |= el->el_tty.t_t[QU_IO][MD_LIN].t_setmask;
-
-	if (tty_setty(el, &el->el_tty.t_qu) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile, "QuoteModeOn: tty_setty: %s\n",
-		    strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	el->el_tty.t_mode = QU_IO;
-	return (0);
-}
-
-
-/* tty_noquotemode():
- *	Turn off quote mode
- */
-protected int
-tty_noquotemode(EditLine *el)
-{
-
-	if (el->el_tty.t_mode != QU_IO)
-		return (0);
-	if (tty_setty(el, &el->el_tty.t_ed) == -1) {
-#ifdef DEBUG_TTY
-		(void) fprintf(el->el_errfile, "QuoteModeOff: tty_setty: %s\n",
-		    strerror(errno));
-#endif /* DEBUG_TTY */
-		return (-1);
-	}
-	el->el_tty.t_mode = ED_IO;
-	return (0);
-}
-
-
-/* tty_stty():
- *	Stty builtin
- */
-protected int
-/*ARGSUSED*/
-tty_stty(EditLine *el, int argc, const char **argv)
-{
-	const ttymodes_t *m;
-	char x;
-	int aflag = 0;
-	const char *s, *d;
-	const char *name;
-	int z = EX_IO;
-
-	if (argv == NULL)
-		return (-1);
-	name = *argv++;
-
-	while (argv && *argv && argv[0][0] == '-' && argv[0][2] == '\0')
-		switch (argv[0][1]) {
-		case 'a':
-			aflag++;
-			argv++;
-			break;
-		case 'd':
-			argv++;
-			z = ED_IO;
-			break;
-		case 'x':
-			argv++;
-			z = EX_IO;
-			break;
-		case 'q':
-			argv++;
-			z = QU_IO;
-			break;
-		default:
-			(void) fprintf(el->el_errfile,
-			    "%s: Unknown switch `%c'.\n",
-			    name, argv[0][1]);
-			return (-1);
-		}
-
-	if (!argv || !*argv) {
-		int i = -1;
-		int len = 0, st = 0, cu;
-		for (m = ttymodes; m->m_name; m++) {
-			if (m->m_type != i) {
-				(void) fprintf(el->el_outfile, "%s%s",
-				    i != -1 ? "\n" : "",
-				    el->el_tty.t_t[z][m->m_type].t_name);
-				i = m->m_type;
-				st = len =
-				    strlen(el->el_tty.t_t[z][m->m_type].t_name);
-			}
-			x = (el->el_tty.t_t[z][i].t_setmask & m->m_value)
-			    ?  '+' : '\0';
-			x = (el->el_tty.t_t[z][i].t_clrmask & m->m_value)
-			    ? '-' : x;
-
-			if (x != '\0' || aflag) {
-
-				cu = strlen(m->m_name) + (x != '\0') + 1;
-
-				if (len + cu >= el->el_term.t_size.h) {
-					(void) fprintf(el->el_outfile, "\n%*s",
-					    st, "");
-					len = st + cu;
-				} else
-					len += cu;
-
-				if (x != '\0')
-					(void) fprintf(el->el_outfile, "%c%s ",
-					    x, m->m_name);
-				else
-					(void) fprintf(el->el_outfile, "%s ",
-					    m->m_name);
-			}
-		}
-		(void) fprintf(el->el_outfile, "\n");
-		return (0);
-	}
-	while (argv && (s = *argv++)) {
-		switch (*s) {
-		case '+':
-		case '-':
-			x = *s++;
-			break;
-		default:
-			x = '\0';
-			break;
-		}
-		d = s;
-		for (m = ttymodes; m->m_name; m++)
-			if (strcmp(m->m_name, d) == 0)
-				break;
-
-		if (!m->m_name) {
-			(void) fprintf(el->el_errfile,
-			    "%s: Invalid argument `%s'.\n", name, d);
-			return (-1);
-		}
-		switch (x) {
-		case '+':
-			el->el_tty.t_t[z][m->m_type].t_setmask |= m->m_value;
-			el->el_tty.t_t[z][m->m_type].t_clrmask &= ~m->m_value;
-			break;
-		case '-':
-			el->el_tty.t_t[z][m->m_type].t_setmask &= ~m->m_value;
-			el->el_tty.t_t[z][m->m_type].t_clrmask |= m->m_value;
-			break;
-		default:
-			el->el_tty.t_t[z][m->m_type].t_setmask &= ~m->m_value;
-			el->el_tty.t_t[z][m->m_type].t_clrmask &= ~m->m_value;
-			break;
-		}
-	}
-	return (0);
-}
-
-
-#ifdef notyet
-/* tty_printchar():
- *	DEbugging routine to print the tty characters
- */
-private void
-tty_printchar(EditLine *el, unsigned char *s)
-{
-	ttyperm_t *m;
-	int i;
-
-	for (i = 0; i < C_NCC; i++) {
-		for (m = el->el_tty.t_t; m->m_name; m++)
-			if (m->m_type == MD_CHAR && C_SH(i) == m->m_value)
-				break;
-		if (m->m_name)
-			(void) fprintf(el->el_errfile, "%s ^%c ",
-			    m->m_name, s[i] + 'A' - 1);
-		if (i % 5 == 0)
-			(void) fprintf(el->el_errfile, "\n");
-	}
-	(void) fprintf(el->el_errfile, "\n");
-}
-#endif /* notyet */
diff -pruN 1:13.23.1~dfsg-2/main/editline/tty.h 1:16.1.1~dfsg-1/main/editline/tty.h
--- 1:13.23.1~dfsg-2/main/editline/tty.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/tty.h	1970-01-01 00:00:00.000000000 +0000
@@ -1,484 +0,0 @@
-/*	$NetBSD: tty.h,v 1.9 2002/03/18 16:01:01 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- *
- *	@(#)tty.h	8.1 (Berkeley) 6/4/93
- */
-
-/*
- * el.tty.h: Local terminal header
- */
-#ifndef _h_el_tty
-#define	_h_el_tty
-
-#include "histedit.h"
-#include <termios.h>
-#include <unistd.h>
-
-/* Define our own since everyone gets it wrong! */
-#define	CONTROL(A)	((A) & 037)
-
-/*
- * Aix compatible names
- */
-# if defined(VWERSE) && !defined(VWERASE)
-#  define VWERASE VWERSE
-# endif /* VWERSE && !VWERASE */
-
-# if defined(VDISCRD) && !defined(VDISCARD)
-#  define VDISCARD VDISCRD
-# endif /* VDISCRD && !VDISCARD */
-
-# if defined(VFLUSHO) && !defined(VDISCARD)
-#  define VDISCARD VFLUSHO
-# endif  /* VFLUSHO && VDISCARD */
-
-# if defined(VSTRT) && !defined(VSTART)
-#  define VSTART VSTRT
-# endif /* VSTRT && ! VSTART */
-
-# if defined(VSTAT) && !defined(VSTATUS)
-#  define VSTATUS VSTAT
-# endif /* VSTAT && ! VSTATUS */
-
-# ifndef ONLRET
-#  define ONLRET 0
-# endif /* ONLRET */
-
-# ifndef TAB3
-#  ifdef OXTABS
-#   define TAB3 OXTABS
-#  else
-#   define TAB3 0
-#  endif /* OXTABS */
-# endif /* !TAB3 */
-
-# if defined(OXTABS) && !defined(XTABS)
-#  define XTABS OXTABS
-# endif /* OXTABS && !XTABS */
-
-# ifndef ONLCR
-#  define ONLCR 0
-# endif /* ONLCR */
-
-# ifndef IEXTEN
-#  define IEXTEN 0
-# endif /* IEXTEN */
-
-# ifndef ECHOCTL
-#  define ECHOCTL 0
-# endif /* ECHOCTL */
-
-# ifndef PARENB
-#  define PARENB 0
-# endif /* PARENB */
-
-# ifndef EXTPROC
-#  define EXTPROC 0
-# endif /* EXTPROC */
-
-# ifndef FLUSHO
-#  define FLUSHO  0
-# endif /* FLUSHO */
-
-
-# if defined(VDISABLE) && !defined(_POSIX_VDISABLE)
-#  define _POSIX_VDISABLE VDISABLE
-# endif /* VDISABLE && ! _POSIX_VDISABLE */
-
-/*
- * Work around ISC's definition of IEXTEN which is
- * XCASE!
- */
-# ifdef ISC
-#  if defined(IEXTEN) && defined(XCASE)
-#   if IEXTEN == XCASE
-#    undef IEXTEN
-#    define IEXTEN 0
-#   endif /* IEXTEN == XCASE */
-#  endif /* IEXTEN && XCASE */
-#  if defined(IEXTEN) && !defined(XCASE)
-#   define XCASE IEXTEN
-#   undef IEXTEN
-#   define IEXTEN 0
-#  endif /* IEXTEN && !XCASE */
-# endif /* ISC */
-
-/*
- * Work around convex weirdness where turning off IEXTEN makes us
- * lose all postprocessing!
- */
-#if defined(convex) || defined(__convex__)
-# if defined(IEXTEN) && IEXTEN != 0
-#  undef IEXTEN
-#  define IEXTEN 0
-# endif /* IEXTEN != 0 */
-#endif /* convex || __convex__ */
-
-/*
- * So that we don't lose job control.
- */
-#ifdef __SVR4
-# undef CSWTCH
-#endif
-
-#ifndef _POSIX_VDISABLE
-# define _POSIX_VDISABLE ((unsigned char) -1)
-#endif /* _POSIX_VDISABLE */
-
-#if !defined(CREPRINT) && defined(CRPRNT)
-# define CREPRINT CRPRNT
-#endif /* !CREPRINT && CRPRNT */
-#if !defined(CDISCARD) && defined(CFLUSH)
-# define CDISCARD CFLUSH
-#endif /* !CDISCARD && CFLUSH */
-
-#ifndef CINTR
-# define CINTR		CONTROL('c')
-#endif /* CINTR */
-#ifndef CQUIT
-# define CQUIT		034	/* ^\ */
-#endif /* CQUIT */
-#ifndef CERASE
-# define CERASE		0177	/* ^? */
-#endif /* CERASE */
-#ifndef CKILL
-# define CKILL		CONTROL('u')
-#endif /* CKILL */
-#ifndef CEOF
-# define CEOF		CONTROL('d')
-#endif /* CEOF */
-#ifndef CEOL
-# define CEOL		_POSIX_VDISABLE
-#endif /* CEOL */
-#ifndef CEOL2
-# define CEOL2		_POSIX_VDISABLE
-#endif /* CEOL2 */
-#ifndef CSWTCH
-# define CSWTCH		_POSIX_VDISABLE
-#endif /* CSWTCH */
-#ifndef CDSWTCH
-# define CDSWTCH	_POSIX_VDISABLE
-#endif /* CDSWTCH */
-#ifndef CERASE2
-# define CERASE2	_POSIX_VDISABLE
-#endif /* CERASE2 */
-#ifndef CSTART
-# define CSTART		CONTROL('q')
-#endif /* CSTART */
-#ifndef CSTOP
-# define CSTOP		CONTROL('s')
-#endif /* CSTOP */
-#ifndef CSUSP
-# define CSUSP		CONTROL('z')
-#endif /* CSUSP */
-#ifndef CDSUSP
-# define CDSUSP		CONTROL('y')
-#endif /* CDSUSP */
-
-#ifdef hpux
-
-# ifndef CREPRINT
-#  define CREPRINT	_POSIX_VDISABLE
-# endif /* CREPRINT */
-# ifndef CDISCARD
-#  define CDISCARD	_POSIX_VDISABLE
-# endif /* CDISCARD */
-# ifndef CLNEXT
-#  define CLNEXT	_POSIX_VDISABLE
-# endif /* CLNEXT */
-# ifndef CWERASE
-#  define CWERASE	_POSIX_VDISABLE
-# endif /* CWERASE */
-
-#else /* !hpux */
-
-# ifndef CREPRINT
-#  define CREPRINT	CONTROL('r')
-# endif /* CREPRINT */
-# ifndef CDISCARD
-#  define CDISCARD	CONTROL('o')
-# endif /* CDISCARD */
-# ifndef CLNEXT
-#  define CLNEXT	CONTROL('v')
-# endif /* CLNEXT */
-# ifndef CWERASE
-#  define CWERASE	CONTROL('w')
-# endif /* CWERASE */
-
-#endif /* hpux */
-
-#ifndef CSTATUS
-# define CSTATUS	CONTROL('t')
-#endif /* CSTATUS */
-#ifndef CPAGE
-# define CPAGE		' '
-#endif /* CPAGE */
-#ifndef CPGOFF
-# define CPGOFF		CONTROL('m')
-#endif /* CPGOFF */
-#ifndef CKILL2
-# define CKILL2		_POSIX_VDISABLE
-#endif /* CKILL2 */
-#ifndef CBRK
-# ifndef masscomp
-#  define CBRK		0377
-# else
-#  define CBRK		'\0'
-# endif /* masscomp */
-#endif /* CBRK */
-#ifndef CMIN
-# define CMIN		CEOF
-#endif /* CMIN */
-#ifndef CTIME
-# define CTIME		CEOL
-#endif /* CTIME */
-
-/*
- * Fix for sun inconsistency. On termio VSUSP and the rest of the
- * ttychars > NCC are defined. So we undefine them.
- */
-#if defined(TERMIO) || defined(POSIX)
-# if defined(POSIX) && defined(NCCS)
-#  define NUMCC		NCCS
-# else
-#  ifdef NCC
-#   define NUMCC	NCC
-#  endif /* NCC */
-# endif /* POSIX && NCCS */
-# ifdef NUMCC
-#  ifdef VINTR
-#   if NUMCC <= VINTR
-#    undef VINTR
-#   endif /* NUMCC <= VINTR */
-#  endif /* VINTR */
-#  ifdef VQUIT
-#   if NUMCC <= VQUIT
-#    undef VQUIT
-#   endif /* NUMCC <= VQUIT */
-#  endif /* VQUIT */
-#  ifdef VERASE
-#   if NUMCC <= VERASE
-#    undef VERASE
-#   endif /* NUMCC <= VERASE */
-#  endif /* VERASE */
-#  ifdef VKILL
-#   if NUMCC <= VKILL
-#    undef VKILL
-#   endif /* NUMCC <= VKILL */
-#  endif /* VKILL */
-#  ifdef VEOF
-#   if NUMCC <= VEOF
-#    undef VEOF
-#   endif /* NUMCC <= VEOF */
-#  endif /* VEOF */
-#  ifdef VEOL
-#   if NUMCC <= VEOL
-#    undef VEOL
-#   endif /* NUMCC <= VEOL */
-#  endif /* VEOL */
-#  ifdef VEOL2
-#   if NUMCC <= VEOL2
-#    undef VEOL2
-#   endif /* NUMCC <= VEOL2 */
-#  endif /* VEOL2 */
-#  ifdef VSWTCH
-#   if NUMCC <= VSWTCH
-#    undef VSWTCH
-#   endif /* NUMCC <= VSWTCH */
-#  endif /* VSWTCH */
-#  ifdef VDSWTCH
-#   if NUMCC <= VDSWTCH
-#    undef VDSWTCH
-#   endif /* NUMCC <= VDSWTCH */
-#  endif /* VDSWTCH */
-#  ifdef VERASE2
-#   if NUMCC <= VERASE2
-#    undef VERASE2
-#   endif /* NUMCC <= VERASE2 */
-#  endif /* VERASE2 */
-#  ifdef VSTART
-#   if NUMCC <= VSTART
-#    undef VSTART
-#   endif /* NUMCC <= VSTART */
-#  endif /* VSTART */
-#  ifdef VSTOP
-#   if NUMCC <= VSTOP
-#    undef VSTOP
-#   endif /* NUMCC <= VSTOP */
-#  endif /* VSTOP */
-#  ifdef VWERASE
-#   if NUMCC <= VWERASE
-#    undef VWERASE
-#   endif /* NUMCC <= VWERASE */
-#  endif /* VWERASE */
-#  ifdef VSUSP
-#   if NUMCC <= VSUSP
-#    undef VSUSP
-#   endif /* NUMCC <= VSUSP */
-#  endif /* VSUSP */
-#  ifdef VDSUSP
-#   if NUMCC <= VDSUSP
-#    undef VDSUSP
-#   endif /* NUMCC <= VDSUSP */
-#  endif /* VDSUSP */
-#  ifdef VREPRINT
-#   if NUMCC <= VREPRINT
-#    undef VREPRINT
-#   endif /* NUMCC <= VREPRINT */
-#  endif /* VREPRINT */
-#  ifdef VDISCARD
-#   if NUMCC <= VDISCARD
-#    undef VDISCARD
-#   endif /* NUMCC <= VDISCARD */
-#  endif /* VDISCARD */
-#  ifdef VLNEXT
-#   if NUMCC <= VLNEXT
-#    undef VLNEXT
-#   endif /* NUMCC <= VLNEXT */
-#  endif /* VLNEXT */
-#  ifdef VSTATUS
-#   if NUMCC <= VSTATUS
-#    undef VSTATUS
-#   endif /* NUMCC <= VSTATUS */
-#  endif /* VSTATUS */
-#  ifdef VPAGE
-#   if NUMCC <= VPAGE
-#    undef VPAGE
-#   endif /* NUMCC <= VPAGE */
-#  endif /* VPAGE */
-#  ifdef VPGOFF
-#   if NUMCC <= VPGOFF
-#    undef VPGOFF
-#   endif /* NUMCC <= VPGOFF */
-#  endif /* VPGOFF */
-#  ifdef VKILL2
-#   if NUMCC <= VKILL2
-#    undef VKILL2
-#   endif /* NUMCC <= VKILL2 */
-#  endif /* VKILL2 */
-#  ifdef VBRK
-#   if NUMCC <= VBRK
-#    undef VBRK
-#   endif /* NUMCC <= VBRK */
-#  endif /* VBRK */
-#  ifdef VMIN
-#   if NUMCC <= VMIN
-#    undef VMIN
-#   endif /* NUMCC <= VMIN */
-#  endif /* VMIN */
-#  ifdef VTIME
-#   if NUMCC <= VTIME
-#    undef VTIME
-#   endif /* NUMCC <= VTIME */
-#  endif /* VTIME */
-# endif /* NUMCC */
-#endif /* !POSIX */
-
-#define	C_INTR		 0
-#define	C_QUIT		 1
-#define	C_ERASE		 2
-#define	C_KILL		 3
-#define	C_EOF		 4
-#define	C_EOL		 5
-#define	C_EOL2		 6
-#define	C_SWTCH		 7
-#define	C_DSWTCH	 8
-#define	C_ERASE2	 9
-#define	C_START		10
-#define	C_STOP		11
-#define	C_WERASE	12
-#define	C_SUSP		13
-#define	C_DSUSP		14
-#define	C_REPRINT	15
-#define	C_DISCARD	16
-#define	C_LNEXT		17
-#define	C_STATUS	18
-#define	C_PAGE		19
-#define	C_PGOFF		20
-#define	C_KILL2		21
-#define	C_BRK		22
-#define	C_MIN		23
-#define	C_TIME		24
-#define	C_NCC		25
-#define	C_SH(A)		(1 << (A))
-
-/*
- * Terminal dependend data structures
- */
-#define	EX_IO	0	/* while we are executing	*/
-#define	ED_IO	1	/* while we are editing		*/
-#define	TS_IO	2	/* new mode from terminal	*/
-#define	QU_IO	2	/* used only for quoted chars	*/
-#define	NN_IO	3	/* The number of entries	*/
-
-#define	MD_INP	0
-#define	MD_OUT	1
-#define	MD_CTL	2
-#define	MD_LIN	3
-#define	MD_CHAR	4
-#define	MD_NN	5
-
-typedef struct {
-	const char	*t_name;
-	u_int	 t_setmask;
-	u_int	 t_clrmask;
-} ttyperm_t[NN_IO][MD_NN];
-
-typedef unsigned char ttychar_t[NN_IO][C_NCC];
-
-protected int	tty_init(EditLine *);
-protected void	tty_end(EditLine *);
-protected int	tty_stty(EditLine *, int, const char **);
-protected int	tty_rawmode(EditLine *);
-protected int	tty_cookedmode(EditLine *);
-protected int	tty_quotemode(EditLine *);
-protected int	tty_noquotemode(EditLine *);
-protected void	tty_bind_char(EditLine *, int);
-
-typedef struct {
-    ttyperm_t t_t;
-    ttychar_t t_c;
-    struct termios t_ex, t_ed, t_ts;
-    int t_tabs;
-    int t_eight;
-    speed_t t_speed;
-    int t_mode;
-    unsigned char t_vdisable;
-} el_tty_t;
-
-
-#endif /* _h_el_tty */
diff -pruN 1:13.23.1~dfsg-2/main/editline/vi.c 1:16.1.1~dfsg-1/main/editline/vi.c
--- 1:13.23.1~dfsg-2/main/editline/vi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/editline/vi.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,941 +0,0 @@
-/*	$NetBSD: vi.c,v 1.9 2002/03/18 16:01:01 christos Exp $	*/
-
-/*-
- * Copyright (c) 1992, 1993
- *	The Regents of the University of California.  All rights reserved.
- *
- * This code is derived from software contributed to Berkeley by
- * Christos Zoulas of Cornell University.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *	This product includes software developed by the University of
- *	California, Berkeley and its contributors.
- * 4. Neither the name of the University nor the names of its contributors
- *    may be used to endorse or promote products derived from this software
- *    without specific prior written permission.
- *
- * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
- * SUCH DAMAGE.
- */
-
-#include "config.h"
-#if !defined(lint) && !defined(SCCSID)
-#if 0
-static char sccsid[] = "@(#)vi.c	8.1 (Berkeley) 6/4/93";
-#else
-__RCSID("$NetBSD: vi.c,v 1.9 2002/03/18 16:01:01 christos Exp $");
-#endif
-#endif /* not lint && not SCCSID */
-
-/*
- * vi.c: Vi mode commands.
- */
-#include "el.h"
-
-private el_action_t	cv_action(EditLine *, int);
-private el_action_t	cv_paste(EditLine *, int);
-
-/* cv_action():
- *	Handle vi actions.
- */
-private el_action_t
-cv_action(EditLine *el, int c)
-{
-	char *cp, *kp;
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		el->el_chared.c_vcmd.action = NOP;
-		el->el_chared.c_vcmd.pos = 0;
-
-		el->el_chared.c_undo.isize = 0;
-		el->el_chared.c_undo.dsize = 0;
-		kp = el->el_chared.c_undo.buf;
-		for (cp = el->el_line.buffer; cp < el->el_line.lastchar; cp++) {
-			*kp++ = *cp;
-			el->el_chared.c_undo.dsize++;
-		}
-
-		el->el_chared.c_undo.action = INSERT;
-		el->el_chared.c_undo.ptr = el->el_line.buffer;
-		el->el_line.lastchar = el->el_line.buffer;
-		el->el_line.cursor = el->el_line.buffer;
-		if (c & INSERT)
-			el->el_map.current = el->el_map.key;
-
-		return (CC_REFRESH);
-	}
-	el->el_chared.c_vcmd.pos = el->el_line.cursor;
-	el->el_chared.c_vcmd.action = c;
-	return (CC_ARGHACK);
-
-#ifdef notdef
-	/*
-         * I don't think that this is needed. But we keep it for now
-         */
-	else
-	if (el_chared.c_vcmd.action == NOP) {
-		el->el_chared.c_vcmd.pos = el->el_line.cursor;
-		el->el_chared.c_vcmd.action = c;
-		return (CC_ARGHACK);
-	} else {
-		el->el_chared.c_vcmd.action = 0;
-		el->el_chared.c_vcmd.pos = 0;
-		return (CC_ERROR);
-	}
-#endif
-}
-
-
-/* cv_paste():
- *	Paste previous deletion before or after the cursor
- */
-private el_action_t
-cv_paste(EditLine *el, int c)
-{
-	char *ptr;
-	c_undo_t *un = &el->el_chared.c_undo;
-
-#ifdef DEBUG_PASTE
-	(void) fprintf(el->el_errfile, "Paste: %x \"%s\" +%d -%d\n",
-	    un->action, un->buf, un->isize, un->dsize);
-#endif
-	if (un->isize == 0)
-		return (CC_ERROR);
-
-	if (!c && el->el_line.cursor < el->el_line.lastchar)
-		el->el_line.cursor++;
-	ptr = el->el_line.cursor;
-
-	c_insert(el, (int) un->isize);
-	if (el->el_line.cursor + un->isize > el->el_line.lastchar)
-		return (CC_ERROR);
-	(void) memcpy(ptr, un->buf, un->isize);
-	return (CC_REFRESH);
-}
-
-
-/* vi_paste_next():
- *	Vi paste previous deletion to the right of the cursor
- *	[p]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_paste_next(EditLine *el, int c)
-{
-
-	return (cv_paste(el, 0));
-}
-
-
-/* vi_paste_prev():
- *	Vi paste previous deletion to the left of the cursor
- *	[P]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_paste_prev(EditLine *el, int c)
-{
-
-	return (cv_paste(el, 1));
-}
-
-
-/* vi_prev_space_word():
- *	Vi move to the previous space delimited word
- *	[B]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_prev_space_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv_prev_word(el, el->el_line.cursor,
-	    el->el_line.buffer,
-	    el->el_state.argument,
-	    cv__isword);
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	return (CC_CURSOR);
-}
-
-
-/* vi_prev_word():
- *	Vi move to the previous word
- *	[B]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_prev_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.buffer)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv_prev_word(el, el->el_line.cursor,
-	    el->el_line.buffer,
-	    el->el_state.argument,
-	    ce__isword);
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	return (CC_CURSOR);
-}
-
-
-/* vi_next_space_word():
- *	Vi move to the next space delimited word
- *	[W]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_next_space_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv_next_word(el, el->el_line.cursor,
-	    el->el_line.lastchar,
-	    el->el_state.argument,
-	    cv__isword);
-
-	if (el->el_map.type == MAP_VI)
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	return (CC_CURSOR);
-}
-
-
-/* vi_next_word():
- *	Vi move to the next word
- *	[w]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_next_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv_next_word(el, el->el_line.cursor,
-	    el->el_line.lastchar,
-	    el->el_state.argument,
-	    ce__isword);
-
-	if (el->el_map.type == MAP_VI)
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-	return (CC_CURSOR);
-}
-
-
-/* vi_change_case():
- *	Vi change case of character under the cursor and advance one character
- *	[~]
- */
-protected el_action_t
-vi_change_case(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor < el->el_line.lastchar) {
-		c = *el->el_line.cursor;
-		if (isupper(c))
-			*el->el_line.cursor++ = tolower(c);
-		else if (islower(c))
-			*el->el_line.cursor++ = toupper(c);
-		else
-			el->el_line.cursor++;
-		re_fastaddc(el);
-		return (CC_NORM);
-	}
-	return (CC_ERROR);
-}
-
-
-/* vi_change_meta():
- *	Vi change prefix command
- *	[c]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_change_meta(EditLine *el, int c)
-{
-
-	/*
-         * Delete with insert == change: first we delete and then we leave in
-         * insert mode.
-         */
-	return (cv_action(el, DELETE | INSERT));
-}
-
-
-/* vi_insert_at_bol():
- *	Vi enter insert mode at the beginning of line
- *	[I]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_insert_at_bol(EditLine *el, int c)
-{
-
-	el->el_line.cursor = el->el_line.buffer;
-	el->el_chared.c_vcmd.ins = el->el_line.cursor;
-
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.action = DELETE;
-
-	el->el_map.current = el->el_map.key;
-	return (CC_CURSOR);
-}
-
-
-/* vi_replace_char():
- *	Vi replace character under the cursor with the next character typed
- *	[r]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_replace_char(EditLine *el, int c)
-{
-
-	el->el_map.current = el->el_map.key;
-	el->el_state.inputmode = MODE_REPLACE_1;
-	el->el_chared.c_undo.action = CHANGE;
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.isize = 0;
-	el->el_chared.c_undo.dsize = 0;
-	return (CC_NORM);
-}
-
-
-/* vi_replace_mode():
- *	Vi enter replace mode
- *	[R]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_replace_mode(EditLine *el, int c)
-{
-
-	el->el_map.current = el->el_map.key;
-	el->el_state.inputmode = MODE_REPLACE;
-	el->el_chared.c_undo.action = CHANGE;
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.isize = 0;
-	el->el_chared.c_undo.dsize = 0;
-	return (CC_NORM);
-}
-
-
-/* vi_substitute_char():
- *	Vi replace character under the cursor and enter insert mode
- *	[r]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_substitute_char(EditLine *el, int c)
-{
-
-	c_delafter(el, el->el_state.argument);
-	el->el_map.current = el->el_map.key;
-	return (CC_REFRESH);
-}
-
-
-/* vi_substitute_line():
- *	Vi substitute entire line
- *	[S]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_substitute_line(EditLine *el, int c)
-{
-
-	(void) em_kill_line(el, 0);
-	el->el_map.current = el->el_map.key;
-	return (CC_REFRESH);
-}
-
-
-/* vi_change_to_eol():
- *	Vi change to end of line
- *	[C]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_change_to_eol(EditLine *el, int c)
-{
-
-	(void) ed_kill_line(el, 0);
-	el->el_map.current = el->el_map.key;
-	return (CC_REFRESH);
-}
-
-
-/* vi_insert():
- *	Vi enter insert mode
- *	[i]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_insert(EditLine *el, int c)
-{
-
-	el->el_map.current = el->el_map.key;
-
-	el->el_chared.c_vcmd.ins = el->el_line.cursor;
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.action = DELETE;
-
-	return (CC_NORM);
-}
-
-
-/* vi_add():
- *	Vi enter insert mode after the cursor
- *	[a]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_add(EditLine *el, int c)
-{
-	int ret;
-
-	el->el_map.current = el->el_map.key;
-	if (el->el_line.cursor < el->el_line.lastchar) {
-		el->el_line.cursor++;
-		if (el->el_line.cursor > el->el_line.lastchar)
-			el->el_line.cursor = el->el_line.lastchar;
-		ret = CC_CURSOR;
-	} else
-		ret = CC_NORM;
-
-	el->el_chared.c_vcmd.ins = el->el_line.cursor;
-	el->el_chared.c_undo.ptr = el->el_line.cursor;
-	el->el_chared.c_undo.action = DELETE;
-
-	return (ret);
-}
-
-
-/* vi_add_at_eol():
- *	Vi enter insert mode at end of line
- *	[A]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_add_at_eol(EditLine *el, int c)
-{
-
-	el->el_map.current = el->el_map.key;
-	el->el_line.cursor = el->el_line.lastchar;
-
-	/* Mark where insertion begins */
-	el->el_chared.c_vcmd.ins = el->el_line.lastchar;
-	el->el_chared.c_undo.ptr = el->el_line.lastchar;
-	el->el_chared.c_undo.action = DELETE;
-	return (CC_CURSOR);
-}
-
-
-/* vi_delete_meta():
- *	Vi delete prefix command
- *	[d]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_delete_meta(EditLine *el, int c)
-{
-
-	return (cv_action(el, DELETE));
-}
-
-
-/* vi_end_word():
- *	Vi move to the end of the current space delimited word
- *	[E]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_end_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv__endword(el->el_line.cursor,
-	    el->el_line.lastchar, el->el_state.argument);
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		el->el_line.cursor++;
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	return (CC_CURSOR);
-}
-
-
-/* vi_to_end_word():
- *	Vi move to the end of the current word
- *	[e]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_to_end_word(EditLine *el, int c)
-{
-
-	if (el->el_line.cursor == el->el_line.lastchar)
-		return (CC_ERROR);
-
-	el->el_line.cursor = cv__endword(el->el_line.cursor,
-	    el->el_line.lastchar, el->el_state.argument);
-
-	if (el->el_chared.c_vcmd.action & DELETE) {
-		el->el_line.cursor++;
-		cv_delfini(el);
-		return (CC_REFRESH);
-	}
-	return (CC_CURSOR);
-}
-
-
-/* vi_undo():
- *	Vi undo last change
- *	[u]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_undo(EditLine *el, int c)
-{
-	char *cp, *kp;
-	char temp;
-	int i, size;
-	c_undo_t *un = &el->el_chared.c_undo;
-
-#ifdef DEBUG_UNDO
-	(void) fprintf(el->el_errfile, "Undo: %x \"%s\" +%d -%d\n",
-	    un->action, un->buf, un->isize, un->dsize);
-#endif
-	switch (un->action) {
-	case DELETE:
-		if (un->dsize == 0)
-			return (CC_NORM);
-
-		(void) memcpy(un->buf, un->ptr, un->dsize);
-		for (cp = un->ptr; cp <= el->el_line.lastchar; cp++)
-			*cp = cp[un->dsize];
-
-		el->el_line.lastchar -= un->dsize;
-		el->el_line.cursor = un->ptr;
-
-		un->action = INSERT;
-		un->isize = un->dsize;
-		un->dsize = 0;
-		break;
-
-	case DELETE | INSERT:
-		size = un->isize - un->dsize;
-		if (size > 0)
-			i = un->dsize;
-		else
-			i = un->isize;
-		cp = un->ptr;
-		kp = un->buf;
-		while (i-- > 0) {
-			temp = *kp;
-			*kp++ = *cp;
-			*cp++ = temp;
-		}
-		if (size > 0) {
-			el->el_line.cursor = cp;
-			c_insert(el, size);
-			while (size-- > 0 && cp < el->el_line.lastchar) {
-				temp = *kp;
-				*kp++ = *cp;
-				*cp++ = temp;
-			}
-		} else if (size < 0) {
-			size = -size;
-			for (; cp <= el->el_line.lastchar; cp++) {
-				*kp++ = *cp;
-				*cp = cp[size];
-			}
-			el->el_line.lastchar -= size;
-		}
-		el->el_line.cursor = un->ptr;
-		i = un->dsize;
-		un->dsize = un->isize;
-		un->isize = i;
-		break;
-
-	case INSERT:
-		if (un->isize == 0)
-			return (CC_NORM);
-
-		el->el_line.cursor = un->ptr;
-		c_insert(el, (int) un->isize);
-		(void) memcpy(un->ptr, un->buf, un->isize);
-		un->action = DELETE;
-		un->dsize = un->isize;
-		un->isize = 0;
-		break;
-
-	case CHANGE:
-		if (un->isize == 0)
-			return (CC_NORM);
-
-		el->el_line.cursor = un->ptr;
-		size = (int) (el->el_line.cursor - el->el_line.lastchar);
-		if (size < un->isize)
-			size = un->isize;
-		cp = un->ptr;
-		kp = un->buf;
-		for (i = 0; i < size; i++) {
-			temp = *kp;
-			*kp++ = *cp;
-			*cp++ = temp;
-		}
-		un->dsize = 0;
-		break;
-
-	default:
-		return (CC_ERROR);
-	}
-
-	return (CC_REFRESH);
-}
-
-
-/* vi_command_mode():
- *	Vi enter command mode (use alternative key bindings)
- *	[<ESC>]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_command_mode(EditLine *el, int c)
-{
-	int size;
-
-	/* [Esc] cancels pending action */
-	el->el_chared.c_vcmd.ins = 0;
-	el->el_chared.c_vcmd.action = NOP;
-	el->el_chared.c_vcmd.pos = 0;
-
-	el->el_state.doingarg = 0;
-	size = el->el_chared.c_undo.ptr - el->el_line.cursor;
-	if (size < 0)
-		size = -size;
-	if (el->el_chared.c_undo.action == (INSERT | DELETE) ||
-	    el->el_chared.c_undo.action == DELETE)
-		el->el_chared.c_undo.dsize = size;
-	else
-		el->el_chared.c_undo.isize = size;
-
-	el->el_state.inputmode = MODE_INSERT;
-	el->el_map.current = el->el_map.alt;
-#ifdef VI_MOVE
-	if (el->el_line.cursor > el->el_line.buffer)
-		el->el_line.cursor--;
-#endif
-	return (CC_CURSOR);
-}
-
-
-/* vi_zero():
- *	Vi move to the beginning of line
- *	[0]
- */
-protected el_action_t
-vi_zero(EditLine *el, int c)
-{
-
-	if (el->el_state.doingarg) {
-		if (el->el_state.argument > 1000000)
-			return (CC_ERROR);
-		el->el_state.argument =
-		    (el->el_state.argument * 10) + (c - '0');
-		return (CC_ARGHACK);
-	} else {
-		el->el_line.cursor = el->el_line.buffer;
-		if (el->el_chared.c_vcmd.action & DELETE) {
-			cv_delfini(el);
-			return (CC_REFRESH);
-		}
-		return (CC_CURSOR);
-	}
-}
-
-
-/* vi_delete_prev_char():
- * 	Vi move to previous character (backspace)
- *	[^H]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_delete_prev_char(EditLine *el, int c)
-{
-
-	if (el->el_chared.c_vcmd.ins == 0)
-		return (CC_ERROR);
-
-	if (el->el_chared.c_vcmd.ins >
-	    el->el_line.cursor - el->el_state.argument)
-		return (CC_ERROR);
-
-	c_delbefore(el, el->el_state.argument);
-	el->el_line.cursor -= el->el_state.argument;
-
-	return (CC_REFRESH);
-}
-
-
-/* vi_list_or_eof():
- *	Vi list choices for completion or indicate end of file if empty line
- *	[^D]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_list_or_eof(EditLine *el, int c)
-{
-
-#ifdef notyet
-	if (el->el_line.cursor == el->el_line.lastchar &&
-	    el->el_line.cursor == el->el_line.buffer) {
-#endif
-		term_overwrite(el, STReof, 4);	/* then do a EOF */
-		term__flush();
-		return (CC_EOF);
-#ifdef notyet
-	} else {
-		re_goto_bottom(el);
-		*el->el_line.lastchar = '\0';	/* just in case */
-		return (CC_LIST_CHOICES);
-	}
-#endif
-}
-
-
-/* vi_kill_line_prev():
- *	Vi cut from beginning of line to cursor
- *	[^U]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_kill_line_prev(EditLine *el, int c)
-{
-	char *kp, *cp;
-
-	cp = el->el_line.buffer;
-	kp = el->el_chared.c_kill.buf;
-	while (cp < el->el_line.cursor)
-		*kp++ = *cp++;	/* copy it */
-	el->el_chared.c_kill.last = kp;
-	c_delbefore(el, el->el_line.cursor - el->el_line.buffer);
-	el->el_line.cursor = el->el_line.buffer;	/* zap! */
-	return (CC_REFRESH);
-}
-
-
-/* vi_search_prev():
- *	Vi search history previous
- *	[?]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_search_prev(EditLine *el, int c)
-{
-
-	return (cv_search(el, ED_SEARCH_PREV_HISTORY));
-}
-
-
-/* vi_search_next():
- *	Vi search history next
- *	[/]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_search_next(EditLine *el, int c)
-{
-
-	return (cv_search(el, ED_SEARCH_NEXT_HISTORY));
-}
-
-
-/* vi_repeat_search_next():
- *	Vi repeat current search in the same search direction
- *	[n]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_repeat_search_next(EditLine *el, int c)
-{
-
-	if (el->el_search.patlen == 0)
-		return (CC_ERROR);
-	else
-		return (cv_repeat_srch(el, el->el_search.patdir));
-}
-
-
-/* vi_repeat_search_prev():
- *	Vi repeat current search in the opposite search direction
- *	[N]
- */
-/*ARGSUSED*/
-protected el_action_t
-vi_repeat_search_prev(EditLine *el, int c)
-{
-
-	if (el->el_search.patlen == 0)
-		return (CC_ERROR);
-	else
-		return (cv_repeat_srch(el,
-		    el->el_search.patdir == ED_SEARCH_PREV_HISTORY ?
-		    ED_SEARCH_NEXT_HISTORY : ED_SEARCH_PREV_HISTORY));
-}
-
-
-/* vi_next_char():
- *	Vi move to the character specified next
- *	[f]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_next_char(EditLine *el, int c)
-{
-	char ch;
-
-	if (el_getc(el, &ch) != 1)
-		return (ed_end_of_file(el, 0));
-
-	el->el_search.chadir = CHAR_FWD;
-	el->el_search.chacha = ch;
-
-	return (cv_csearch_fwd(el, ch, el->el_state.argument, 0));
-
-}
-
-
-/* vi_prev_char():
- *	Vi move to the character specified previous
- *	[F]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_prev_char(EditLine *el, int c)
-{
-	char ch;
-
-	if (el_getc(el, &ch) != 1)
-		return (ed_end_of_file(el, 0));
-
-	el->el_search.chadir = CHAR_BACK;
-	el->el_search.chacha = ch;
-
-	return (cv_csearch_back(el, ch, el->el_state.argument, 0));
-}
-
-
-/* vi_to_next_char():
- *	Vi move up to the character specified next
- *	[t]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_to_next_char(EditLine *el, int c)
-{
-	char ch;
-
-	if (el_getc(el, &ch) != 1)
-		return (ed_end_of_file(el, 0));
-
-	return (cv_csearch_fwd(el, ch, el->el_state.argument, 1));
-
-}
-
-
-/* vi_to_prev_char():
- *	Vi move up to the character specified previous
- *	[T]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_to_prev_char(EditLine *el, int c)
-{
-	char ch;
-
-	if (el_getc(el, &ch) != 1)
-		return (ed_end_of_file(el, 0));
-
-	return (cv_csearch_back(el, ch, el->el_state.argument, 1));
-}
-
-
-/* vi_repeat_next_char():
- *	Vi repeat current character search in the same search direction
- *	[;]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_repeat_next_char(EditLine *el, int c)
-{
-
-	if (el->el_search.chacha == 0)
-		return (CC_ERROR);
-
-	return (el->el_search.chadir == CHAR_FWD
-	    ? cv_csearch_fwd(el, el->el_search.chacha,
-		el->el_state.argument, 0)
-	    : cv_csearch_back(el, el->el_search.chacha,
-		el->el_state.argument, 0));
-}
-
-
-/* vi_repeat_prev_char():
- *	Vi repeat current character search in the opposite search direction
- *	[,]
- */
-protected el_action_t
-/*ARGSUSED*/
-vi_repeat_prev_char(EditLine *el, int c)
-{
-
-	if (el->el_search.chacha == 0)
-		return (CC_ERROR);
-
-	return el->el_search.chadir == CHAR_BACK ?
-	    cv_csearch_fwd(el, el->el_search.chacha, el->el_state.argument, 0) :
-	    cv_csearch_back(el, el->el_search.chacha, el->el_state.argument, 0);
-}
diff -pruN 1:13.23.1~dfsg-2/main/endpoints.c 1:16.1.1~dfsg-1/main/endpoints.c
--- 1:13.23.1~dfsg-2/main/endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/endpoints.h"
 #include "asterisk/stasis.h"
@@ -204,7 +202,7 @@ static void endpoint_cache_clear(void *d
 	endpoint_publish_snapshot(endpoint);
 }
 
-static void endpoint_default(void *data,
+static void endpoint_subscription_change(void *data,
 	struct stasis_subscription *sub,
 	struct stasis_message *message)
 {
@@ -265,6 +263,8 @@ static struct ast_endpoint *endpoint_int
 		if (!endpoint->topics) {
 			return NULL;
 		}
+		stasis_cp_single_accept_message_type(endpoint->topics, ast_endpoint_snapshot_type());
+		stasis_cp_single_set_filter(endpoint->topics, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 		endpoint->router = stasis_message_router_create_pool(ast_endpoint_topic(endpoint));
 		if (!endpoint->router) {
@@ -273,8 +273,9 @@ static struct ast_endpoint *endpoint_int
 		r |= stasis_message_router_add(endpoint->router,
 			stasis_cache_clear_type(), endpoint_cache_clear,
 			endpoint);
-		r |= stasis_message_router_set_default(endpoint->router,
-			endpoint_default, endpoint);
+		r |= stasis_message_router_add(endpoint->router,
+			stasis_subscription_change_type(), endpoint_subscription_change,
+			endpoint);
 		if (r) {
 			return NULL;
 		}
@@ -290,6 +291,8 @@ static struct ast_endpoint *endpoint_int
 		if (!endpoint->topics) {
 			return NULL;
 		}
+		stasis_cp_single_accept_message_type(endpoint->topics, ast_endpoint_snapshot_type());
+		stasis_cp_single_set_filter(endpoint->topics, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 		ao2_link(tech_endpoints, endpoint);
 	}
@@ -479,14 +482,14 @@ int ast_endpoint_init(void)
 {
 	ast_register_cleanup(endpoint_cleanup);
 
-	endpoints = ao2_container_alloc(ENDPOINT_BUCKETS, ast_endpoint_hash_fn,
-		ast_endpoint_cmp_fn);
+	endpoints = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, ENDPOINT_BUCKETS,
+		ast_endpoint_hash_fn, NULL, ast_endpoint_cmp_fn);
 	if (!endpoints) {
 		return -1;
 	}
 
-	tech_endpoints = ao2_container_alloc(TECH_ENDPOINT_BUCKETS, ast_endpoint_hash_fn,
-		ast_endpoint_cmp_fn);
+	tech_endpoints = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		TECH_ENDPOINT_BUCKETS, ast_endpoint_hash_fn, NULL, ast_endpoint_cmp_fn);
 	if (!tech_endpoints) {
 		return -1;
 	}
diff -pruN 1:13.23.1~dfsg-2/main/enum.c 1:16.1.1~dfsg-1/main/enum.c
--- 1:13.23.1~dfsg-2/main/enum.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/enum.c	2018-12-26 18:45:13.000000000 +0000
@@ -60,8 +60,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/nameser.h>
@@ -72,6 +70,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <ctype.h>
 #include <regex.h>
 
+#include "asterisk/module.h"
 #include "asterisk/enum.h"
 #include "asterisk/dns.h"
 #include "asterisk/channel.h"
@@ -1003,12 +1002,26 @@ static int private_enum_init(int reload)
 	return 0;
 }
 
-int ast_enum_init(void)
+static int load_module(void)
+{
+	return private_enum_init(0) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
 {
-	return private_enum_init(0);
+	return 0;
 }
 
-int ast_enum_reload(void)
+static int reload_module(void)
 {
 	return private_enum_init(1);
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "ENUM Support",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/event.c 1:16.1.1~dfsg-1/main/event.c
--- 1:13.23.1~dfsg-2/main/event.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/event.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #include "asterisk/event.h"
diff -pruN 1:13.23.1~dfsg-2/main/features.c 1:16.1.1~dfsg-1/main/features.c
--- 1:13.23.1~dfsg-2/main/features.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/features.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,14 +39,13 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
+#include "features_config.h"
 
 #include <pthread.h>
 #include <signal.h>
 #include <sys/time.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <netinet/in.h>
 
 #include "asterisk/lock.h"
@@ -326,7 +325,6 @@ struct ast_bridge_thread_obj
 	struct ast_bridge_config bconfig;
 	struct ast_channel *chan;
 	struct ast_channel *peer;
-	struct ast_callid *callid;                             /*<! callid pointer (Only used to bind thread) */
 	unsigned int return_to_pbx:1;
 };
 
@@ -1148,29 +1146,33 @@ done:
 	return 0;
 }
 
-/*!
- * \internal
- * \brief Clean up resources on Asterisk shutdown
- */
-static void features_shutdown(void)
+static int unload_module(void)
 {
-	ast_features_config_shutdown();
+	unload_features_config();
 
 	ast_manager_unregister("Bridge");
 
 	ast_unregister_application(app_bridge);
 
+	return 0;
 }
 
-int ast_features_init(void)
+static int load_module(void)
 {
 	int res;
 
-	res = ast_features_config_init();
+	res = load_features_config();
 	res |= ast_register_application2(app_bridge, bridge_exec, NULL, NULL, NULL);
 	res |= ast_manager_register_xml_core("Bridge", EVENT_FLAG_CALL, action_bridge);
 
-	ast_register_cleanup(features_shutdown);
-
-	return res;
+	return res ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Call Features",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_features_config,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/features_config.c 1:16.1.1~dfsg-1/main/features_config.c
--- 1:13.23.1~dfsg-2/main/features_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/features_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,6 +26,8 @@
 #include "asterisk/app.h"
 #include "asterisk/cli.h"
 
+#include "features_config.h"
+
 /*** DOCUMENTATION
 	<configInfo name="features" language="en_US">
 		<synopsis>Features Configuration</synopsis>
@@ -378,9 +380,9 @@
 #define DEFAULT_ATXFER_COMPLETE                     "*2"
 #define DEFAULT_ATXFER_THREEWAY                     "*3"
 #define DEFAULT_ATXFER_SWAP                         "*4"
-#define DEFAULT_TRANSFER_DIAL_ATTEMPTS              1
+#define DEFAULT_TRANSFER_DIAL_ATTEMPTS              3
 #define DEFAULT_TRANSFER_RETRY_SOUND                "pbx-invalid"
-#define DEFAULT_TRANSFER_INVALID_SOUND              "pbx-invalid"
+#define DEFAULT_TRANSFER_INVALID_SOUND              "privacy-incorrect"
 
 /*! Default pickup options */
 #define DEFAULT_PICKUPEXTEN                         "*8"
@@ -757,8 +759,8 @@ static struct features_config *__feature
 			return NULL;
 		}
 
-		cfg->featuregroups = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 11, featuregroup_hash,
-			featuregroup_cmp);
+		cfg->featuregroups = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, 11,
+			featuregroup_hash, NULL, featuregroup_cmp);
 		if (!cfg->featuregroups) {
 			return NULL;
 		}
@@ -1971,7 +1973,7 @@ static struct ast_cli_entry cli_features
 	AST_CLI_DEFINE(handle_feature_show, "Lists configured features"),
 };
 
-void ast_features_config_shutdown(void)
+void unload_features_config(void)
 {
 	ast_custom_function_unregister(&featuremap_function);
 	ast_custom_function_unregister(&feature_function);
@@ -1980,7 +1982,7 @@ void ast_features_config_shutdown(void)
 	ao2_global_obj_release(globals);
 }
 
-int ast_features_config_reload(void)
+int reload_features_config(void)
 {
 	/* Rearm the parking config options have moved warning. */
 	parking_warning = 0;
@@ -1991,7 +1993,7 @@ int ast_features_config_reload(void)
 	return 0;
 }
 
-int ast_features_config_init(void)
+int load_features_config(void)
 {
 	int res;
 
diff -pruN 1:13.23.1~dfsg-2/main/features_config.h 1:16.1.1~dfsg-1/main/features_config.h
--- 1:13.23.1~dfsg-2/main/features_config.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/features_config.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,28 @@
+/*
+* Asterisk -- An open source telephony toolkit.
+*
+* Copyright (C) 2018, CFWare, LLC.
+*
+* Corey Farrell <git@cfware.com>
+*
+* See http://www.asterisk.org for more information about
+* the Asterisk project. Please do not directly contact
+* any of the maintainers of this project for assistance;
+* the project provides a web site, mailing lists and IRC
+* channels for your use.
+*
+* This program is free software, distributed under the terms of
+* the GNU General Public License Version 2. See the LICENSE file
+* at the top of the source tree.
+*/
+
+#ifndef FEATURES_CONFIG_H_
+#define FEATURES_CONFIG_H_
+
+int load_features_config(void);
+
+int reload_features_config(void);
+
+void unload_features_config(void);
+
+#endif /* FEATURES_CONFIG_H_ */
diff -pruN 1:13.23.1~dfsg-2/main/file.c 1:16.1.1~dfsg-1/main/file.c
--- 1:13.23.1~dfsg-2/main/file.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/file.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dirent.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
@@ -54,6 +52,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/stasis.h"
 #include "asterisk/json.h"
 #include "asterisk/stasis_system.h"
+#include "asterisk/media_cache.h"
 
 /*! \brief
  * The following variable controls the layout of localized sound files.
@@ -334,20 +333,21 @@ static char *build_filename(const char *
 
 /* compare type against the list 'exts' */
 /* XXX need a better algorithm */
-static int exts_compare(const char *exts, const char *type)
+static int type_in_list(const char *list, const char *type, int (*cmp)(const char *s1, const char *s2))
 {
-	char tmp[256];
-	char *stringp = tmp, *ext;
+	char *stringp = ast_strdupa(list), *item;
 
-	ast_copy_string(tmp, exts, sizeof(tmp));
-	while ((ext = strsep(&stringp, "|"))) {
-		if (!strcmp(ext, type))
+	while ((item = strsep(&stringp, "|"))) {
+		if (!cmp(item, type)) {
 			return 1;
+		}
 	}
 
 	return 0;
 }
 
+#define exts_compare(list, type) (type_in_list((list), (type), strcmp))
+
 /*!
  * \internal
  * \brief Close the file stream by canceling any pending read / write callbacks
@@ -412,17 +412,12 @@ static void filestream_destructor(void *
 		}
 	}
 
-	if (f->filename)
-		free(f->filename);
-	if (f->realfilename)
-		free(f->realfilename);
+	ast_free(f->filename);
+	ast_free(f->realfilename);
 	if (f->vfs)
 		ast_closestream(f->vfs);
-	if (f->write_buffer) {
-		ast_free(f->write_buffer);
-	}
-	if (f->orig_chan_name)
-		free((void *) f->orig_chan_name);
+	ast_free(f->write_buffer);
+	ast_free((void *)f->orig_chan_name);
 	ao2_cleanup(f->lastwriteformat);
 	ao2_cleanup(f->fr.subclass.format);
 	ast_module_unref(f->fmt->module);
@@ -431,11 +426,17 @@ static void filestream_destructor(void *
 static struct ast_filestream *get_filestream(struct ast_format_def *fmt, FILE *bfile)
 {
 	struct ast_filestream *s;
-
 	int l = sizeof(*s) + fmt->buf_size + fmt->desc_size;	/* total allocation size */
-	if ( (s = ao2_alloc(l, filestream_destructor)) == NULL)
+
+	if (!ast_module_running_ref(fmt->module)) {
 		return NULL;
-	ast_module_ref(fmt->module);
+	}
+
+	s = ao2_alloc(l, filestream_destructor);
+	if (!s) {
+		ast_module_unref(fmt->module);
+		return NULL;
+	}
 	s->fmt = fmt;
 	s->f = bfile;
 
@@ -649,6 +650,10 @@ static int fileexists_test(const char *f
 		return 0;
 	}
 
+	if (!ast_media_cache_retrieve(filename, NULL, buf, buflen)) {
+		return filehelper(buf, result_cap, NULL, ACTION_EXISTS);
+	}
+
 	if (ast_language_is_prefix && !is_absolute_path(filename)) { /* new layout */
 		if (lang) {
 			snprintf(buf, buflen, "%s/%s", lang, filename);
@@ -1922,6 +1927,27 @@ struct ast_format *ast_get_format_for_fi
 	return NULL;
 }
 
+int ast_get_extension_for_mime_type(const char *mime_type, char *buffer, size_t capacity)
+{
+	struct ast_format_def *f;
+	SCOPED_RDLOCK(lock, &formats.lock);
+
+	ast_assert(buffer && capacity);
+
+	AST_RWLIST_TRAVERSE(&formats, f, list) {
+		if (type_in_list(f->mime_types, mime_type, strcasecmp)) {
+			size_t item_len = strcspn(f->exts, "|");
+			size_t bytes_written = snprintf(buffer, capacity, ".%.*s", (int) item_len, f->exts);
+			if (bytes_written < capacity) {
+				/* Only return success if we didn't truncate */
+				return 1;
+			}
+		}
+	}
+
+	return 0;
+}
+
 static struct ast_cli_entry cli_file[] = {
 	AST_CLI_DEFINE(handle_cli_core_show_file_formats, "Displays file formats")
 };
diff -pruN 1:13.23.1~dfsg-2/main/fixedjitterbuf.c 1:16.1.1~dfsg-1/main/fixedjitterbuf.c
--- 1:13.23.1~dfsg-2/main/fixedjitterbuf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/fixedjitterbuf.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <assert.h>
 
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/main/format.c 1:16.1.1~dfsg-1/main/format.c
--- 1:13.23.1~dfsg-2/main/format.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/format.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/codec.h"
 #include "asterisk/format.h"
@@ -51,6 +49,8 @@ struct ast_format {
 	void *attribute_data;
 	/*! \brief Pointer to the optional format interface */
 	const struct ast_format_interface *interface;
+	/*! \brief The number if audio channels used, if more than one an interleaved format is required */
+	unsigned int channel_count;
 };
 
 /*! \brief Structure used when registering a format interface */
@@ -76,8 +76,8 @@ static void format_shutdown(void)
 
 int ast_format_init(void)
 {
-	interfaces = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, FORMAT_INTERFACE_BUCKETS,
-		format_interface_hash_fn, format_interface_cmp_fn);
+	interfaces = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0,
+		FORMAT_INTERFACE_BUCKETS, format_interface_hash_fn, NULL, format_interface_cmp_fn);
 	if (!interfaces) {
 		return -1;
 	}
@@ -132,6 +132,16 @@ void ast_format_set_attribute_data(struc
 	format->attribute_data = attribute_data;
 }
 
+unsigned int ast_format_get_channel_count(const struct ast_format *format)
+{
+	return format->channel_count;
+}
+
+void ast_format_set_channel_count(struct ast_format *format, unsigned int channel_count)
+{
+	format->channel_count = channel_count;
+}
+
 /*! \brief Destructor for media formats */
 static void format_destroy(void *obj)
 {
@@ -156,6 +166,7 @@ struct ast_format *ast_format_create_nam
 	}
 	format->name = format_name;
 	format->codec = ao2_bump(codec);
+	format->channel_count = 1;
 
 	format_interface = ao2_find(interfaces, codec->name, OBJ_SEARCH_KEY);
 	if (format_interface) {
@@ -332,13 +343,12 @@ const char *ast_format_get_codec_name(co
 
 int ast_format_can_be_smoothed(const struct ast_format *format)
 {
-	/* Coalesce to 1 if non-zero */
-	return format->codec->smooth ? 1 : 0;
+	return format->codec->smooth;
 }
 
 int ast_format_get_smoother_flags(const struct ast_format *format)
 {
-	return AST_SMOOTHER_FLAGS_UNPACK(format->codec->smooth);
+	return format->codec->smoother_flags;
 }
 
 enum ast_media_type ast_format_get_type(const struct ast_format *format)
diff -pruN 1:13.23.1~dfsg-2/main/format_cache.c 1:16.1.1~dfsg-1/main/format_cache.c
--- 1:13.23.1~dfsg-2/main/format_cache.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/format_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/format.h"
 #include "asterisk/format_cache.h"
@@ -223,6 +221,11 @@ struct ast_format *ast_format_siren7;
 struct ast_format *ast_format_opus;
 
 /*!
+ * \brief Built-in cached codec2 format.
+ */
+struct ast_format *ast_format_codec2;
+
+/*!
  * \brief Built-in cached t140 format.
  */
 struct ast_format *ast_format_t140;
@@ -233,6 +236,11 @@ struct ast_format *ast_format_t140;
 struct ast_format *ast_format_t140_red;
 
 /*!
+ * \brief Built-in cached T.38 format.
+ */
+struct ast_format *ast_format_t38;
+
+/*!
  * \brief Built-in "null" format.
  */
 struct ast_format *ast_format_none;
@@ -333,6 +341,7 @@ static void format_cache_shutdown(void)
 	ao2_replace(ast_format_testlaw, NULL);
 	ao2_replace(ast_format_g719, NULL);
 	ao2_replace(ast_format_opus, NULL);
+	ao2_replace(ast_format_codec2, NULL);
 	ao2_replace(ast_format_jpeg, NULL);
 	ao2_replace(ast_format_png, NULL);
 	ao2_replace(ast_format_h261, NULL);
@@ -344,6 +353,7 @@ static void format_cache_shutdown(void)
 	ao2_replace(ast_format_vp9, NULL);
 	ao2_replace(ast_format_t140_red, NULL);
 	ao2_replace(ast_format_t140, NULL);
+	ao2_replace(ast_format_t38, NULL);
 	ao2_replace(ast_format_none, NULL);
 	ao2_replace(ast_format_silk8, NULL);
 	ao2_replace(ast_format_silk12, NULL);
@@ -353,8 +363,8 @@ static void format_cache_shutdown(void)
 
 int ast_format_cache_init(void)
 {
-	formats = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, CACHE_BUCKETS,
-		format_hash_cb, format_cmp_cb);
+	formats = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, CACHE_BUCKETS,
+		format_hash_cb, NULL, format_cmp_cb);
 	if (!formats) {
 		return -1;
 	}
@@ -366,7 +376,9 @@ int ast_format_cache_init(void)
 
 static void set_cached_format(const char *name, struct ast_format *format)
 {
-	if (!strcmp(name, "g723")) {
+	if (!strcmp(name, "codec2")) {
+		ao2_replace(ast_format_codec2, format);
+	} else if (!strcmp(name, "g723")) {
 		ao2_replace(ast_format_g723, format);
 	} else if (!strcmp(name, "ulaw")) {
 		ao2_replace(ast_format_ulaw, format);
@@ -444,6 +456,8 @@ static void set_cached_format(const char
 		ao2_replace(ast_format_t140_red, format);
 	} else if (!strcmp(name, "t140")) {
 		ao2_replace(ast_format_t140, format);
+	} else if (!strcmp(name, "t38")) {
+		ao2_replace(ast_format_t38, format);
 	} else if (!strcmp(name, "none")) {
 		ao2_replace(ast_format_none, format);
 	} else if (!strcmp(name, "silk8")) {
@@ -485,22 +499,14 @@ int ast_format_cache_set(struct ast_form
 	return 0;
 }
 
-struct ast_format *__ast_format_cache_get(const char *name)
-{
-	if (ast_strlen_zero(name)) {
-		return NULL;
-	}
-
-	return ao2_find(formats, name, OBJ_SEARCH_KEY);
-}
-
-struct ast_format *__ast_format_cache_get_debug(const char *name, const char *tag, const char *file, int line, const char *func)
+struct ast_format *__ast_format_cache_get(const char *name,
+	const char *tag, const char *file, int line, const char *func)
 {
 	if (ast_strlen_zero(name)) {
 		return NULL;
 	}
 
-	return __ao2_find_debug(formats, name, OBJ_SEARCH_KEY, S_OR(tag, "ast_format_cache_get"), file, line, func);
+	return __ao2_find(formats, name, OBJ_SEARCH_KEY, tag, file, line, func);
 }
 
 struct ast_format *ast_format_cache_get_slin_by_rate(unsigned int rate)
diff -pruN 1:13.23.1~dfsg-2/main/format_cap.c 1:16.1.1~dfsg-1/main/format_cap.c
--- 1:13.23.1~dfsg-2/main/format_cap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/format_cap.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/format.h"
 #include "asterisk/format_cap.h"
@@ -116,28 +114,13 @@ static inline int format_cap_init(struct
 	return 0;
 }
 
-struct ast_format_cap *__ast_format_cap_alloc(enum ast_format_cap_flags flags)
+struct ast_format_cap *__ast_format_cap_alloc(enum ast_format_cap_flags flags,
+	const char *tag, const char *file, int line, const char *func)
 {
 	struct ast_format_cap *cap;
 
-	cap = ao2_alloc_options(sizeof(*cap), format_cap_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
-	if (!cap) {
-		return NULL;
-	}
-
-	if (format_cap_init(cap, flags)) {
-		ao2_ref(cap, -1);
-		return NULL;
-	}
-
-	return cap;
-}
-
-struct ast_format_cap *__ast_format_cap_alloc_debug(enum ast_format_cap_flags flags, const char *tag, const char *file, int line, const char *func)
-{
-	struct ast_format_cap *cap;
-
-	cap = __ao2_alloc_debug(sizeof(*cap), format_cap_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK, S_OR(tag, "ast_format_cap_alloc"), file, line, func, 1);
+	cap = __ao2_alloc(sizeof(*cap), format_cap_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK,
+		tag, file, line, func);
 	if (!cap) {
 		return NULL;
 	}
@@ -209,26 +192,7 @@ static int format_in_format_cap(struct a
 	return 0;
 }
 
-int __ast_format_cap_append(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing)
-{
-	struct format_cap_framed *framed;
-
-	ast_assert(format != NULL);
-
-	if (format_in_format_cap(cap, format)) {
-		return 0;
-	}
-
-	framed = ao2_alloc_options(sizeof(*framed), format_cap_framed_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
-	if (!framed) {
-		return -1;
-	}
-	framed->format = ao2_bump(format);
-
-	return format_cap_framed_init(framed, cap, format, framing);
-}
-
-int __ast_format_cap_append_debug(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing, const char *tag, const char *file, int line, const char *func)
+int __ast_format_cap_append(struct ast_format_cap *cap, struct ast_format *format, unsigned int framing, const char *tag, const char *file, int line, const char *func)
 {
 	struct format_cap_framed *framed;
 
@@ -243,7 +207,7 @@ int __ast_format_cap_append_debug(struct
 		return -1;
 	}
 
-	__ao2_ref_debug(format, +1, S_OR(tag, "ast_format_cap_append"), file, line, func);
+	__ao2_ref(format, +1, tag, file, line, func);
 	framed->format = format;
 
 	return format_cap_framed_init(framed, cap, format, framing);
@@ -307,6 +271,7 @@ int ast_format_cap_append_from_cap(struc
 {
 	int idx, res = 0;
 
+	/* NOTE:  The streams API is dependent on the formats being in "preference" order */
 	for (idx = 0; (idx < AST_VECTOR_SIZE(&src->preference_order)) && !res; ++idx) {
 		struct format_cap_framed *framed = AST_VECTOR_GET(&src->preference_order, idx);
 
diff -pruN 1:13.23.1~dfsg-2/main/format_compatibility.c 1:16.1.1~dfsg-1/main/format_compatibility.c
--- 1:13.23.1~dfsg-2/main/format_compatibility.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/format_compatibility.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/main/frame.c 1:16.1.1~dfsg-1/main/frame.c
--- 1:13.23.1~dfsg-2/main/frame.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/frame.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/lock.h"
 #include "asterisk/frame.h"
@@ -227,6 +225,7 @@ struct ast_frame *ast_frisolate(struct a
 			out->len = fr->len;
 			out->seqno = fr->seqno;
 		}
+		out->stream_num = fr->stream_num;
 	} else {
 		out = fr;
 	}
@@ -260,7 +259,7 @@ struct ast_frame *ast_frisolate(struct a
 
 	if (!(fr->mallocd & AST_MALLOCD_DATA))  {
 		/* The original frame has a non-malloced data buffer. */
-		if (!fr->datalen) {
+		if (!fr->datalen && fr->frametype != AST_FRAME_TEXT) {
 			/* Actually it's just an int so we can simply copy it. */
 			out->data.uint32 = fr->data.uint32;
 			return out;
@@ -357,7 +356,8 @@ struct ast_frame *ast_frdup(const struct
 	 */
 	out->mallocd = AST_MALLOCD_HDR;
 	out->offset = AST_FRIENDLY_OFFSET;
-	if (out->datalen) {
+	/* Make sure that empty text frames have a valid data.ptr */
+	if (out->datalen || f->frametype == AST_FRAME_TEXT) {
 		out->data.ptr = buf + sizeof(*out) + AST_FRIENDLY_OFFSET;
 		memcpy(out->data.ptr, f->data.ptr, out->datalen);
 	} else {
@@ -375,6 +375,7 @@ struct ast_frame *ast_frdup(const struct
 	out->ts = f->ts;
 	out->len = f->len;
 	out->seqno = f->seqno;
+	out->stream_num = f->stream_num;
 	return out;
 }
 
@@ -555,6 +556,8 @@ void ast_frame_subclass2str(struct ast_f
 			break;
 		}
 		break;
+	case AST_FRAME_RTCP:
+		ast_copy_string(subclass, "RTCP", slen);
 	default:
 		ast_copy_string(subclass, "Unknown Subclass", slen);
 		break;
@@ -609,6 +612,9 @@ void ast_frame_type2str(enum ast_frame_t
 	case AST_FRAME_VIDEO:
 		ast_copy_string(ftype, "Video", len);
 		break;
+	case AST_FRAME_RTCP:
+		ast_copy_string(ftype, "RTCP", len);
+		break;
 	default:
 		snprintf(ftype, len, "Unknown Frametype '%u'", frame_type);
 		break;
@@ -646,6 +652,9 @@ void ast_frame_dump(const char *name, st
 	if (f->frametype == AST_FRAME_VIDEO) {
 		return;
 	}
+	if (f->frametype == AST_FRAME_RTCP) {
+		return;
+	}
 
 	ast_frame_type2str(f->frametype, ftype, sizeof(ftype));
 	ast_frame_subclass2str(f, subclass, sizeof(subclass), moreinfo, sizeof(moreinfo));
diff -pruN 1:13.23.1~dfsg-2/main/framehook.c 1:16.1.1~dfsg-1/main/framehook.c
--- 1:13.23.1~dfsg-2/main/framehook.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/framehook.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/linkedlists.h"
 #include "asterisk/framehook.h"
diff -pruN 1:13.23.1~dfsg-2/main/fskmodem_float.c 1:16.1.1~dfsg-1/main/fskmodem_float.c
--- 1:13.23.1~dfsg-2/main/fskmodem_float.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/fskmodem_float.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 
 #include "asterisk/fskmodem.h"
diff -pruN 1:13.23.1~dfsg-2/main/fskmodem_int.c 1:16.1.1~dfsg-1/main/fskmodem_int.c
--- 1:13.23.1~dfsg-2/main/fskmodem_int.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/fskmodem_int.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/fskmodem.h"
 
 #define NBW	2
diff -pruN 1:13.23.1~dfsg-2/main/global_datastores.c 1:16.1.1~dfsg-1/main/global_datastores.c
--- 1:13.23.1~dfsg-2/main/global_datastores.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/global_datastores.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/global_datastores.h"
 
 static void secure_call_store_destroy(void *data)
diff -pruN 1:13.23.1~dfsg-2/main/hashtab.c 1:16.1.1~dfsg-1/main/hashtab.c
--- 1:13.23.1~dfsg-2/main/hashtab.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/hashtab.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 
 #include "asterisk/lock.h"
@@ -43,12 +41,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/hashtab.h"
 
 
-#ifdef __AST_DEBUG_MALLOC
 static void _ast_hashtab_resize(struct ast_hashtab *tab, const char *file, int lineno, const char *func);
-#define ast_hashtab_resize(a)	_ast_hashtab_resize(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
-#else
-static void ast_hashtab_resize(struct ast_hashtab *tab);
-#endif
+#define ast_hashtab_resize(tab) \
+	_ast_hashtab_resize(tab, __FILE__, __LINE__, __PRETTY_FUNCTION__)
+
 static void *ast_hashtab_lookup_internal(struct ast_hashtab *tab, const void *obj, unsigned int h);
 
 /* some standard, default routines for general use */
@@ -217,40 +213,28 @@ unsigned int ast_hashtab_hash_short(cons
 	return x;
 }
 
-struct ast_hashtab *
-#ifdef __AST_DEBUG_MALLOC
-_ast_hashtab_create
-#else
-ast_hashtab_create
-#endif
-(int initial_buckets,
+struct ast_hashtab *_ast_hashtab_create(int initial_buckets,
 	int (*compare)(const void *a, const void *b),
 	int (*resize)(struct ast_hashtab *),
 	int (*newsize)(struct ast_hashtab *tab),
 	unsigned int (*hash)(const void *obj),
-	int do_locking
-#ifdef __AST_DEBUG_MALLOC
-	, const char *file, int lineno, const char *function
-#endif
+	int do_locking,
+	const char *file, int lineno, const char *function
 )
 {
 	struct ast_hashtab *ht;
 
-#ifdef __AST_DEBUG_MALLOC
-	if (!(ht = __ast_calloc(1, sizeof(*ht), file, lineno, function)))
-#else
-	if (!(ht = ast_calloc(1, sizeof(*ht))))
-#endif
+	ht = __ast_calloc(1, sizeof(*ht), file, lineno, function);
+	if (!ht) {
 		return NULL;
+	}
 
 	while (!ast_is_prime(initial_buckets)) /* make sure this is prime */
 		initial_buckets++;
 
-#ifdef __AST_DEBUG_MALLOC
-	if (!(ht->array = __ast_calloc(initial_buckets, sizeof(*(ht->array)), file, lineno, function))) {
-#else
-	if (!(ht->array = ast_calloc(initial_buckets, sizeof(*(ht->array))))) {
-#endif
+	ht->array = __ast_calloc(initial_buckets, sizeof(*(ht->array)),
+		file, lineno, function);
+	if (!ht->array) {
 		ast_free(ht);
 		return NULL;
 	}
@@ -274,25 +258,19 @@ ast_hashtab_create
 	return ht;
 }
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab *_ast_hashtab_dup(struct ast_hashtab *tab, void *(*obj_dup_func)(const void *obj), const char *file, int lineno, const char *func)
-#else
-struct ast_hashtab *ast_hashtab_dup(struct ast_hashtab *tab, void *(*obj_dup_func)(const void *obj))
-#endif
 {
 	struct ast_hashtab *ht;
 	unsigned int i;
 
-	if (!(ht = ast_calloc(1, sizeof(*ht))))
+	ht = __ast_calloc(1, sizeof(*ht), file, lineno, func);
+	if (!ht) {
 		return NULL;
+	}
 
-	if (!(ht->array =
-#ifdef __AST_DEBUG_MALLOC
-		__ast_calloc(tab->hash_tab_size, sizeof(*(ht->array)), file, lineno, func)
-#else
-		ast_calloc(tab->hash_tab_size, sizeof(*(ht->array)))
-#endif
-		)) {
+	ht->array = __ast_calloc(tab->hash_tab_size, sizeof(*(ht->array)),
+		file, lineno, func);
+	if (!ht->array) {
 		ast_free(ht);
 		return NULL;
 	}
@@ -314,12 +292,9 @@ struct ast_hashtab *ast_hashtab_dup(stru
 		struct ast_hashtab_bucket *b = tab->array[i];
 		while (b) {
 			void *newobj = (*obj_dup_func)(b->object);
-			if (newobj)
-#ifdef __AST_DEBUG_MALLOC
+			if (newobj) {
 				_ast_hashtab_insert_immediate_bucket(ht, newobj, i, file, lineno, func);
-#else
-				ast_hashtab_insert_immediate_bucket(ht, newobj, i);
-#endif
+			}
 			b = b->next;
 		}
 	}
@@ -426,11 +401,7 @@ void ast_hashtab_destroy(struct ast_hash
 	}
 }
 
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_immediate(struct ast_hashtab *tab, const void *obj, const char *file, int lineno, const char *func)
-#else
-int ast_hashtab_insert_immediate(struct ast_hashtab *tab, const void *obj)
-#endif
 {
 	unsigned int h;
 	int res=0;
@@ -443,11 +414,7 @@ int ast_hashtab_insert_immediate(struct
 
 	h = (*tab->hash)(obj) % tab->hash_tab_size;
 
-#ifdef __AST_DEBUG_MALLOC
 	res = _ast_hashtab_insert_immediate_bucket(tab, obj, h, file, lineno, func);
-#else
-	res = ast_hashtab_insert_immediate_bucket(tab, obj, h);
-#endif
 
 	if (tab->do_locking)
 		ast_rwlock_unlock(&tab->lock);
@@ -455,11 +422,7 @@ int ast_hashtab_insert_immediate(struct
 	return res;
 }
 
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_immediate_bucket(struct ast_hashtab *tab, const void *obj, unsigned int h, const char *file, int lineno, const char *func)
-#else
-int ast_hashtab_insert_immediate_bucket(struct ast_hashtab *tab, const void *obj, unsigned int h)
-#endif
 {
 	int c;
 	struct ast_hashtab_bucket *b;
@@ -473,13 +436,10 @@ int ast_hashtab_insert_immediate_bucket(
 	if (c + 1 > tab->largest_bucket_size)
 		tab->largest_bucket_size = c + 1;
 
-	if (!(b =
-#ifdef __AST_DEBUG_MALLOC
-			__ast_calloc(1, sizeof(*b), file, lineno, func)
-#else
-			ast_calloc(1, sizeof(*b))
-#endif
-		)) return 0;
+	b = __ast_calloc(1, sizeof(*b), file, lineno, func);
+	if (!b) {
+		return 0;
+	}
 
 	b->object = obj;
 	b->next = tab->array[h];
@@ -497,11 +457,7 @@ int ast_hashtab_insert_immediate_bucket(
 	return 1;
 }
 
-#ifdef __AST_DEBUG_MALLOC
 int _ast_hashtab_insert_safe(struct ast_hashtab *tab, const void *obj, const char *file, int lineno, const char *func)
-#else
-int ast_hashtab_insert_safe(struct ast_hashtab *tab, const void *obj)
-#endif
 {
 	/* check to see if the element is already there; insert only if
 	   it is not there. */
@@ -513,11 +469,7 @@ int ast_hashtab_insert_safe(struct ast_h
 		ast_rwlock_wrlock(&tab->lock);
 
 	if (!ast_hashtab_lookup_bucket(tab, obj, &bucket)) {
-#ifdef __AST_DEBUG_MALLOC
 		int ret2 = _ast_hashtab_insert_immediate_bucket(tab, obj, bucket, file, lineno, func);
-#else
-		int ret2 = ast_hashtab_insert_immediate_bucket(tab, obj, bucket);
-#endif
 
 		if (tab->do_locking)
 			ast_rwlock_unlock(&tab->lock);
@@ -636,11 +588,7 @@ int ast_hashtab_capacity( struct ast_has
 /* the insert operation calls this, and is wrlock'd when it does. */
 /* if you want to call it, you should set the wrlock yourself */
 
-#ifdef __AST_DEBUG_MALLOC
 static void _ast_hashtab_resize(struct ast_hashtab *tab, const char *file, int lineno, const char *func)
-#else
-static void ast_hashtab_resize(struct ast_hashtab *tab)
-#endif
 {
 	/* this function is called either internally, when the resize func returns 1, or
 	   externally by the user to force a resize of the hash table */
@@ -658,14 +606,10 @@ static void ast_hashtab_resize(struct as
 		tab->array[i] = 0; /* erase old ptrs */
 	}
 	ast_free(tab->array);
-	if (!(tab->array =
-#ifdef __AST_DEBUG_MALLOC
-		__ast_calloc(newsize, sizeof(*(tab->array)), file, lineno, func)
-#else
-		ast_calloc(newsize, sizeof(*(tab->array)))
-#endif
-		))
+	tab->array = __ast_calloc(newsize, sizeof(*(tab->array)), file, lineno, func);
+	if (!tab->array) {
 		return;
+	}
 
 	/* now sort the buckets into their rightful new slots */
 	tab->resize_count++;
@@ -690,23 +634,15 @@ static void ast_hashtab_resize(struct as
 	}
 }
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab_iter *_ast_hashtab_start_traversal(struct ast_hashtab *tab, const char *file, int lineno, const char *func)
-#else
-struct ast_hashtab_iter *ast_hashtab_start_traversal(struct ast_hashtab *tab)
-#endif
 {
 	/* returns an iterator */
 	struct ast_hashtab_iter *it;
 
-	if (!(it =
-#ifdef __AST_DEBUG_MALLOC
-			__ast_calloc(1, sizeof(*it), file, lineno, func)
-#else
-			ast_calloc(1, sizeof(*it))
-#endif
-		))
+	it = __ast_calloc(1, sizeof(*it), file, lineno, func);
+	if (!it) {
 		return NULL;
+	}
 
 	it->next = tab->tlist;
 	it->tab = tab;
@@ -717,23 +653,15 @@ struct ast_hashtab_iter *ast_hashtab_sta
 }
 
 /* use this function to get a write lock */
-#ifdef __AST_DEBUG_MALLOC
 struct ast_hashtab_iter *_ast_hashtab_start_write_traversal(struct ast_hashtab *tab, const char *file, int lineno, const char *func)
-#else
-struct ast_hashtab_iter *ast_hashtab_start_write_traversal(struct ast_hashtab *tab)
-#endif
 {
 	/* returns an iterator */
 	struct ast_hashtab_iter *it;
 
-	if (!(it =
-#ifdef __AST_DEBUG_MALLOC
-			__ast_calloc(1, sizeof(*it), file, lineno, func)
-#else
-			ast_calloc(1, sizeof(*it))
-#endif
-		))
+	it = __ast_calloc(1, sizeof(*it), file, lineno, func);
+	if (!it) {
 		return NULL;
+	}
 
 	it->next = tab->tlist;
 	it->tab = tab;
diff -pruN 1:13.23.1~dfsg-2/main/heap.c 1:16.1.1~dfsg-1/main/heap.c
--- 1:13.23.1~dfsg-2/main/heap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/heap.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/heap.h"
 #include "asterisk/utils.h"
 #include "asterisk/cli.h"
@@ -111,13 +109,8 @@ int ast_heap_verify(struct ast_heap *h)
 	return 0;
 }
 
-#ifdef __AST_DEBUG_MALLOC
 struct ast_heap *_ast_heap_create(unsigned int init_height, ast_heap_cmp_fn cmp_fn,
-		ssize_t index_offset, const char *file, int lineno, const char *func)
-#else
-struct ast_heap *ast_heap_create(unsigned int init_height, ast_heap_cmp_fn cmp_fn,
-		ssize_t index_offset)
-#endif
+	ssize_t index_offset, const char *file, int lineno, const char *func)
 {
 	struct ast_heap *h;
 
@@ -130,13 +123,8 @@ struct ast_heap *ast_heap_create(unsigne
 		init_height = 8;
 	}
 
-	if (!(h =
-#ifdef __AST_DEBUG_MALLOC
-			__ast_calloc(1, sizeof(*h), file, lineno, func)
-#else
-			ast_calloc(1, sizeof(*h))
-#endif
-		)) {
+	h = __ast_calloc(1, sizeof(*h), file, lineno, func);
+	if (!h) {
 		return NULL;
 	}
 
@@ -144,13 +132,8 @@ struct ast_heap *ast_heap_create(unsigne
 	h->index_offset = index_offset;
 	h->avail_len = (1 << init_height) - 1;
 
-	if (!(h->heap =
-#ifdef __AST_DEBUG_MALLOC
-			__ast_malloc(h->avail_len * sizeof(void *), file, lineno, func)
-#else
-			ast_malloc(h->avail_len * sizeof(void *))
-#endif
-		)) {
+	h->heap = __ast_malloc(h->avail_len * sizeof(void *), file, lineno, func);
+	if (!h->heap) {
 		ast_free(h);
 		return NULL;
 	}
@@ -175,20 +158,12 @@ struct ast_heap *ast_heap_destroy(struct
 /*!
  * \brief Add a row of additional storage for the heap.
  */
-static int grow_heap(struct ast_heap *h
-#ifdef __AST_DEBUG_MALLOC
-, const char *file, int lineno, const char *func
-#endif
-)
+static int grow_heap(struct ast_heap *h, const char *file, int lineno, const char *func)
 {
 	void **new_heap;
 	size_t new_len = h->avail_len * 2 + 1;
 
-#ifdef __AST_DEBUG_MALLOC
 	new_heap = __ast_realloc(h->heap, new_len * sizeof(void *), file, lineno, func);
-#else
-	new_heap = ast_realloc(h->heap, new_len * sizeof(void *));
-#endif
 	if (!new_heap) {
 		return -1;
 	}
@@ -244,17 +219,9 @@ static int bubble_up(struct ast_heap *h,
 	return i;
 }
 
-#ifdef __AST_DEBUG_MALLOC
 int _ast_heap_push(struct ast_heap *h, void *elm, const char *file, int lineno, const char *func)
-#else
-int ast_heap_push(struct ast_heap *h, void *elm)
-#endif
-{
-	if (h->cur_len == h->avail_len && grow_heap(h
-#ifdef __AST_DEBUG_MALLOC
-		, file, lineno, func
-#endif
-		)) {
+{
+	if (h->cur_len == h->avail_len && grow_heap(h, file, lineno, func)) {
 		return -1;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/main/http.c 1:16.1.1~dfsg-1/main/http.c
--- 1:13.23.1~dfsg-2/main/http.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/http.c	2018-12-26 18:45:13.000000000 +0000
@@ -44,12 +44,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <fcntl.h>
 
 #include "asterisk/paths.h"	/* use ast_config_AST_DATA_DIR */
@@ -62,7 +60,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/stringfields.h"
 #include "asterisk/ast_version.h"
 #include "asterisk/manager.h"
-#include "asterisk/_private.h"
+#include "asterisk/module.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/netsock2.h"
 #include "asterisk/json.h"
@@ -458,7 +456,7 @@ void ast_http_send(struct ast_tcptls_ses
 	struct ast_str *server_header_field = ast_str_create(MAX_SERVER_NAME_LENGTH);
 	int send_content;
 
-	if (!ser || !ser->f || !server_header_field) {
+	if (!ser || !server_header_field) {
 		/* The connection is not open. */
 		ast_free(http_header);
 		ast_free(out);
@@ -510,7 +508,7 @@ void ast_http_send(struct ast_tcptls_ses
 	send_content = method != AST_HTTP_HEAD || status_code >= 400;
 
 	/* send http header */
-	if (fprintf(ser->f,
+	if (ast_iostream_printf(ser->stream,
 		"HTTP/1.1 %d %s\r\n"
 		"%s"
 		"Date: %s\r\n"
@@ -529,17 +527,13 @@ void ast_http_send(struct ast_tcptls_ses
 		content_length,
 		send_content && out && ast_str_strlen(out) ? ast_str_buffer(out) : ""
 		) <= 0) {
-		ast_debug(1, "fprintf() failed: %s\n", strerror(errno));
+		ast_debug(1, "ast_iostream_printf() failed: %s\n", strerror(errno));
 		close_connection = 1;
 	} else if (send_content && fd) {
 		/* send file content */
 		while ((len = read(fd, buf, sizeof(buf))) > 0) {
-			/*
-			 * NOTE: Because ser->f is a non-standard FILE *, fwrite() will probably not
-			 * behave exactly as documented.
-			 */
-			if (fwrite(buf, len, 1, ser->f) != 1) {
-				ast_debug(1, "fwrite() failed: %s\n", strerror(errno));
+			if (ast_iostream_write(ser->stream, buf, len) != len) {
+				ast_debug(1, "ast_iostream_write() failed: %s\n", strerror(errno));
 				close_connection = 1;
 				break;
 			}
@@ -569,7 +563,7 @@ void ast_http_create_response(struct ast
 		ast_free(http_header_data);
 		ast_free(server_address);
 		ast_free(out);
-		if (ser && ser->f) {
+		if (ser) {
 			ast_debug(1, "HTTP closing session. OOM.\n");
 			ast_tcptls_close_session_file(ser);
 		}
@@ -923,14 +917,9 @@ static int http_body_read_contents(struc
 {
 	int res;
 
-	/*
-	 * NOTE: Because ser->f is a non-standard FILE *, fread() does not behave as
-	 * documented.
-	 */
-
-	/* Stay in fread until get all the expected data or timeout. */
-	res = fread(buf, length, 1, ser->f);
-	if (res < 1) {
+	/* Stream is in exclusive mode so we get it all if possible. */
+	res = ast_iostream_read(ser->stream, buf, length);
+	if (res < length) {
 		ast_log(LOG_WARNING, "Short HTTP request %s (Wanted %d)\n",
 			what_getting, length);
 		return -1;
@@ -952,28 +941,12 @@ static int http_body_read_contents(struc
  */
 static int http_body_discard_contents(struct ast_tcptls_session_instance *ser, int length, const char *what_getting)
 {
-	int res;
-	char buf[MAX_HTTP_LINE_LENGTH];/* Discard buffer */
-
-	/*
-	 * NOTE: Because ser->f is a non-standard FILE *, fread() does not behave as
-	 * documented.
-	 */
+	ssize_t res;
 
-	/* Stay in fread until get all the expected data or timeout. */
-	while (sizeof(buf) < length) {
-		res = fread(buf, sizeof(buf), 1, ser->f);
-		if (res < 1) {
-			ast_log(LOG_WARNING, "Short HTTP request %s (Wanted %zu of remaining %d)\n",
-				what_getting, sizeof(buf), length);
-			return -1;
-		}
-		length -= sizeof(buf);
-	}
-	res = fread(buf, length, 1, ser->f);
-	if (res < 1) {
-		ast_log(LOG_WARNING, "Short HTTP request %s (Wanted %d of remaining %d)\n",
-			what_getting, length, length);
+	res = ast_iostream_discard(ser->stream, length);
+	if (res < length) {
+		ast_log(LOG_WARNING, "Short HTTP request %s (Wanted %d but got %zd)\n",
+			what_getting, length, res);
 		return -1;
 	}
 	return 0;
@@ -1049,7 +1022,7 @@ static int http_body_get_chunk_length(st
 	char header_line[MAX_HTTP_LINE_LENGTH];
 
 	/* get the line of hexadecimal giving chunk-size w/ optional chunk-extension */
-	if (!fgets(header_line, sizeof(header_line), ser->f)) {
+	if (ast_iostream_gets(ser->stream, header_line, sizeof(header_line)) <= 0) {
 		ast_log(LOG_WARNING, "Short HTTP read of chunked header\n");
 		return -1;
 	}
@@ -1076,14 +1049,9 @@ static int http_body_check_chunk_sync(st
 	int res;
 	char chunk_sync[2];
 
-	/*
-	 * NOTE: Because ser->f is a non-standard FILE *, fread() does not behave as
-	 * documented.
-	 */
-
 	/* Stay in fread until get the expected CRLF or timeout. */
-	res = fread(chunk_sync, sizeof(chunk_sync), 1, ser->f);
-	if (res < 1) {
+	res = ast_iostream_read(ser->stream, chunk_sync, sizeof(chunk_sync));
+	if (res < sizeof(chunk_sync)) {
 		ast_log(LOG_WARNING, "Short HTTP chunk sync read (Wanted %zu)\n",
 			sizeof(chunk_sync));
 		return -1;
@@ -1112,7 +1080,7 @@ static int http_body_discard_chunk_trail
 	char header_line[MAX_HTTP_LINE_LENGTH];
 
 	for (;;) {
-		if (!fgets(header_line, sizeof(header_line), ser->f)) {
+		if (ast_iostream_gets(ser->stream, header_line, sizeof(header_line)) <= 0) {
 			ast_log(LOG_WARNING, "Short HTTP read of chunked trailer header\n");
 			return -1;
 		}
@@ -1772,13 +1740,21 @@ static int http_request_headers_get(stru
 
 	remaining_headers = MAX_HTTP_REQUEST_HEADERS;
 	for (;;) {
+		ssize_t len;
 		char *name;
 		char *value;
 
-		if (!fgets(header_line, sizeof(header_line), ser->f)) {
+		len = ast_iostream_gets(ser->stream, header_line, sizeof(header_line));
+		if (len <= 0) {
 			ast_http_error(ser, 400, "Bad Request", "Timeout");
 			return -1;
 		}
+		if (header_line[len - 1] != '\n') {
+			/* We didn't get a full line */
+			ast_http_error(ser, 400, "Bad Request",
+				(len == sizeof(header_line) - 1) ? "Header line too long" : "Timeout");
+			return -1;
+		}
 
 		/* Trim trailing characters */
 		ast_trim_blanks(header_line);
@@ -1847,9 +1823,11 @@ static int httpd_process_request(struct
 	struct http_worker_private_data *request;
 	enum ast_http_method http_method = AST_HTTP_UNKNOWN;
 	int res;
+	ssize_t len;
 	char request_line[MAX_HTTP_LINE_LENGTH];
 
-	if (!fgets(request_line, sizeof(request_line), ser->f)) {
+	len = ast_iostream_gets(ser->stream, request_line, sizeof(request_line));
+	if (len <= 0) {
 		return -1;
 	}
 
@@ -1857,6 +1835,13 @@ static int httpd_process_request(struct
 	request = ser->private_data;
 	http_request_tracking_init(request);
 
+	if (request_line[len - 1] != '\n') {
+		/* We didn't get a full line */
+		ast_http_error(ser, 400, "Bad Request",
+			(len == sizeof(request_line) - 1) ? "Request line too long" : "Timeout");
+		return -1;
+	}
+
 	/* Get method */
 	method = ast_skip_blanks(request_line);
 	uri = ast_skip_nonblanks(method);
@@ -1930,10 +1915,10 @@ static int httpd_process_request(struct
 static void *httpd_helper_thread(void *data)
 {
 	struct ast_tcptls_session_instance *ser = data;
-	int flags = 1;
 	int timeout;
+	int arg = 1;
 
-	if (!ser || !ser->f) {
+	if (!ser) {
 		ao2_cleanup(ser);
 		return NULL;
 	}
@@ -1950,12 +1935,10 @@ static void *httpd_helper_thread(void *d
 	 * This is necessary to prevent delays (caused by buffering) as we
 	 * write to the socket in bits and pieces.
 	 */
-	if (setsockopt(ser->fd, IPPROTO_TCP, TCP_NODELAY, (char *) &flags, sizeof(flags)) < 0) {
+	if (setsockopt(ast_iostream_get_fd(ser->stream), IPPROTO_TCP, TCP_NODELAY, (char *) &arg, sizeof(arg)) < 0) {
 		ast_log(LOG_WARNING, "Failed to set TCP_NODELAY on HTTP connection: %s\n", strerror(errno));
 	}
-
-	/* make sure socket is non-blocking */
-	ast_fd_set_flags(ser->fd, O_NONBLOCK);
+	ast_iostream_nonblock(ser->stream);
 
 	/* Setup HTTP worker private data to keep track of request body reading. */
 	ao2_cleanup(ser->private_data);
@@ -1978,23 +1961,17 @@ static void *httpd_helper_thread(void *d
 	}
 
 	/* We can let the stream wait for data to arrive. */
-	ast_tcptls_stream_set_exclusive_input(ser->stream_cookie, 1);
+	ast_iostream_set_exclusive_input(ser->stream, 1);
 
 	for (;;) {
-		int ch;
-
 		/* Wait for next potential HTTP request message. */
-		ast_tcptls_stream_set_timeout_inactivity(ser->stream_cookie, timeout);
-		ch = fgetc(ser->f);
-		if (ch == EOF || ungetc(ch, ser->f) == EOF) {
-			/* Between request idle timeout */
-			ast_debug(1, "HTTP idle timeout or peer closed connection.\n");
+		ast_iostream_set_timeout_idle_inactivity(ser->stream, timeout, session_inactivity);
+		if (httpd_process_request(ser)) {
+			/* Break the connection or the connection closed */
 			break;
 		}
-
-		ast_tcptls_stream_set_timeout_inactivity(ser->stream_cookie, session_inactivity);
-		if (httpd_process_request(ser) || !ser->f || feof(ser->f)) {
-			/* Break the connection or the connection closed */
+		if (!ser->stream) {
+			/* Web-socket or similar that took the connection */
 			break;
 		}
 
@@ -2008,10 +1985,9 @@ static void *httpd_helper_thread(void *d
 done:
 	ast_atomic_fetchadd_int(&session_count, -1);
 
-	if (ser->f) {
-		ast_debug(1, "HTTP closing session.  Top level\n");
-		ast_tcptls_close_session_file(ser);
-	}
+	ast_debug(1, "HTTP closing session.  Top level\n");
+	ast_tcptls_close_session_file(ser);
+
 	ao2_ref(ser, -1);
 	return NULL;
 }
@@ -2090,7 +2066,15 @@ static int __ast_http_load(int reload)
 	int http_tls_was_enabled = 0;
 
 	cfg = ast_config_load2("http.conf", "http", config_flags);
-	if (!cfg || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID) {
+	if (!cfg || cfg == CONFIG_STATUS_FILEINVALID) {
+		return 0;
+	}
+
+	/* Even if the http.conf hasn't been updated, the TLS certs/keys may have been */
+	if (cfg == CONFIG_STATUS_FILEUNCHANGED) {
+		if (http_tls_cfg.enabled && ast_ssl_setup(https_desc.tls_cfg)) {
+			ast_tcptls_server_start(&https_desc);
+		}
 		return 0;
 	}
 
@@ -2305,7 +2289,7 @@ static char *handle_show_http(struct ast
 	return CLI_SUCCESS;
 }
 
-int ast_http_reload(void)
+static int reload_module(void)
 {
 	return __ast_http_load(1);
 }
@@ -2314,7 +2298,7 @@ static struct ast_cli_entry cli_http[] =
 	AST_CLI_DEFINE(handle_show_http, "Display HTTP server status"),
 };
 
-static void http_shutdown(void)
+static int unload_module(void)
 {
 	struct http_uri_redirect *redirect;
 	ast_cli_unregister_multiple(cli_http, ARRAY_LEN(cli_http));
@@ -2336,14 +2320,24 @@ static void http_shutdown(void)
 		ast_free(redirect);
 	}
 	AST_RWLIST_UNLOCK(&uri_redirects);
+
+	return 0;
 }
 
-int ast_http_init(void)
+static int load_module(void)
 {
 	ast_http_uri_link(&statusuri);
 	ast_http_uri_link(&staticuri);
 	ast_cli_register_multiple(cli_http, ARRAY_LEN(cli_http));
-	ast_register_cleanup(http_shutdown);
 
-	return __ast_http_load(0);
+	return __ast_http_load(0) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Built-in HTTP Server",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/image.c 1:16.1.1~dfsg-1/main/image.c
--- 1:13.23.1~dfsg-2/main/image.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/image.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <signal.h>
diff -pruN 1:13.23.1~dfsg-2/main/indications.c 1:16.1.1~dfsg-1/main/indications.c
--- 1:13.23.1~dfsg-2/main/indications.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/indications.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 
 #include "asterisk/lock.h"
@@ -43,23 +41,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/cli.h"
 #include "asterisk/module.h"
 #include "asterisk/astobj2.h"
-#include "asterisk/data.h"
 
 #include "asterisk/_private.h" /* _init(), _reload() */
 
-#define DATA_EXPORT_TONE_ZONE(MEMBER)					\
-	MEMBER(ast_tone_zone, country, AST_DATA_STRING)			\
-	MEMBER(ast_tone_zone, description, AST_DATA_STRING)		\
-	MEMBER(ast_tone_zone, nrringcadence, AST_DATA_UNSIGNED_INTEGER)
-
-AST_DATA_STRUCTURE(ast_tone_zone, DATA_EXPORT_TONE_ZONE);
-
-#define DATA_EXPORT_TONE_ZONE_SOUND(MEMBER)			\
-	MEMBER(ast_tone_zone_sound, name, AST_DATA_STRING)	\
-	MEMBER(ast_tone_zone_sound, data, AST_DATA_STRING)
-
-AST_DATA_STRUCTURE(ast_tone_zone_sound, DATA_EXPORT_TONE_ZONE_SOUND);
-
 /* Globals */
 static const char config[] = "indications.conf";
 
@@ -604,7 +588,9 @@ static int ast_register_indication(struc
 	}
 	AST_LIST_TRAVERSE_SAFE_END;
 
-	if (!(ts = ao2_alloc(sizeof(*ts), ast_tone_zone_sound_destructor))) {
+	ts = ao2_alloc_options(sizeof(*ts), ast_tone_zone_sound_destructor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!ts) {
 		return -1;
 	}
 
@@ -1125,38 +1111,11 @@ static int ast_tone_zone_cmp(void *obj,
 			CMP_MATCH | CMP_STOP : 0;
 }
 
-int ast_tone_zone_data_add_structure(struct ast_data *tree, struct ast_tone_zone *zone)
-{
-	struct ast_data *data_zone_sound;
-	struct ast_tone_zone_sound *s;
-
-	ast_data_add_structure(ast_tone_zone, tree, zone);
-
-	if (AST_LIST_EMPTY(&zone->tones)) {
-		return 0;
-	}
-
-	data_zone_sound = ast_data_add_node(tree, "tones");
-	if (!data_zone_sound) {
-		return -1;
-	}
-
-	ast_tone_zone_lock(zone);
-
-	AST_LIST_TRAVERSE(&zone->tones, s, entry) {
-		ast_data_add_structure(ast_tone_zone_sound, data_zone_sound, s);
-	}
-
-	ast_tone_zone_unlock(zone);
-
-	return 0;
-}
-
 /*!
  * \internal
  * \brief Clean up resources on Asterisk shutdown
  */
-static void indications_shutdown(void)
+static int unload_module(void)
 {
 	ast_cli_unregister_multiple(cli_indications, ARRAY_LEN(cli_indications));
 	if (default_tone_zone) {
@@ -1167,29 +1126,39 @@ static void indications_shutdown(void)
 		ao2_ref(ast_tone_zones, -1);
 		ast_tone_zones = NULL;
 	}
+
+	return 0;
 }
 
 /*! \brief Load indications module */
-int ast_indications_init(void)
+static int load_module(void)
 {
-	ast_tone_zones = ao2_container_alloc(NUM_TONE_ZONE_BUCKETS,
-			ast_tone_zone_hash, ast_tone_zone_cmp);
+	ast_tone_zones = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NUM_TONE_ZONE_BUCKETS, ast_tone_zone_hash, NULL, ast_tone_zone_cmp);
 	if (!ast_tone_zones) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (load_indications(0)) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	ast_cli_register_multiple(cli_indications, ARRAY_LEN(cli_indications));
 
-	ast_register_cleanup(indications_shutdown);
-	return 0;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
 /*! \brief Reload indications module */
-int ast_indications_reload(void)
+static int reload_module(void)
 {
 	return load_indications(1);
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Indication Tone Handling",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/io.c 1:16.1.1~dfsg-1/main/io.c
--- 1:13.23.1~dfsg-2/main/io.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/io.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <termios.h>
 #include <sys/ioctl.h>
 
@@ -38,6 +36,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/utils.h"
 #ifdef HAVE_SYSTEMD
 #include <systemd/sd-daemon.h>
+
+#ifndef SD_LISTEN_FDS_START
+#define SD_LISTEN_FDS_START 3
+#endif
 #endif
 
 #ifdef DEBUG_IO
@@ -394,3 +396,73 @@ int ast_sd_notify(const char *state) {
 	return 0;
 #endif
 }
+
+/*!
+ * \internal \brief Check the type and sockaddr of a file descriptor.
+ * \param fd File Descriptor to check.
+ * \param type SOCK_STREAM or SOCK_DGRAM
+ * \param addr The socket address to match.
+ * \retval 0 if matching
+ * \retval -1 if not matching
+ */
+#ifdef HAVE_SYSTEMD
+static int ast_sd_is_socket_sockaddr(int fd, int type, const struct ast_sockaddr* addr)
+{
+	int canretry = 1;
+	struct ast_sockaddr fd_addr;
+	struct sockaddr ss;
+	socklen_t ss_len;
+
+	if (sd_is_socket(fd, AF_UNSPEC, type, 1) <= 0) {
+		return -1;
+	}
+
+doretry:
+	if (getsockname(fd, &ss, &ss_len) != 0) {
+		return -1;
+	}
+
+	if (ss.sa_family == AF_UNSPEC && canretry) {
+		/* An unknown bug can cause silent failure from
+		 * the first call to getsockname. */
+		canretry = 0;
+		goto doretry;
+	}
+
+	ast_sockaddr_copy_sockaddr(&fd_addr, &ss, ss_len);
+
+	return ast_sockaddr_cmp(addr, &fd_addr);
+}
+#endif
+
+int ast_sd_get_fd(int type, const struct ast_sockaddr *addr)
+{
+#ifdef HAVE_SYSTEMD
+	int count = sd_listen_fds(0);
+	int idx;
+
+	for (idx = 0; idx < count; idx++) {
+		if (!ast_sd_is_socket_sockaddr(idx + SD_LISTEN_FDS_START, type, addr)) {
+			return idx + SD_LISTEN_FDS_START;
+		}
+	}
+#endif
+
+	return -1;
+}
+
+int ast_sd_get_fd_un(int type, const char *path)
+{
+#ifdef HAVE_SYSTEMD
+	int count = sd_listen_fds(0);
+	int idx;
+
+	for (idx = 0; idx < count; idx++) {
+		if (sd_is_socket_unix(idx + SD_LISTEN_FDS_START, type, 1, path, 0) > 0) {
+			return idx + SD_LISTEN_FDS_START;
+		}
+	}
+#endif
+
+	return -1;
+}
diff -pruN 1:13.23.1~dfsg-2/main/iostream.c 1:16.1.1~dfsg-1/main/iostream.c
--- 1:13.23.1~dfsg-2/main/iostream.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/iostream.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,645 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 1999 - 2015, Digium, Inc.
+ *
+ * Timo Teräs <timo.teras@iki.fi>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+#include "asterisk.h"
+
+#include "asterisk/iostream.h"          /* for DO_SSL */
+
+#include <fcntl.h>                      /* for O_NONBLOCK */
+#ifdef DO_SSL
+#include <openssl/err.h>                /* for ERR_error_string */
+#include <openssl/opensslv.h>           /* for OPENSSL_VERSION_NUMBER */
+#include <openssl/ssl.h>                /* for SSL_get_error, SSL_free, SSL_... */
+#endif
+#include <sys/socket.h>                 /* for shutdown, SHUT_RDWR */
+#include <sys/time.h>                   /* for timeval */
+
+#include "asterisk/astobj2.h"           /* for ao2_alloc_options, ao2_alloc_... */
+#include "asterisk/logger.h"            /* for ast_debug, ast_log, LOG_ERROR */
+#include "asterisk/strings.h"           /* for asterisk/threadstorage.h */
+#include "asterisk/threadstorage.h"     /* for ast_threadstorage_get, AST_TH... */
+#include "asterisk/time.h"              /* for ast_remaining_ms, ast_tvnow */
+#include "asterisk/utils.h"             /* for ast_wait_for_input, ast_wait_... */
+
+struct ast_iostream {
+	SSL *ssl;
+	struct timeval start;
+	int fd;
+	int timeout;
+	int timeout_reset;
+	int exclusive_input;
+	int rbuflen;
+	char *rbufhead;
+	char rbuf[2048];
+};
+
+#if defined(DO_SSL)
+AST_THREADSTORAGE(err2str_threadbuf);
+#define ERR2STR_BUFSIZE   128
+
+static const char *ssl_error_to_string(int sslerr, int ret)
+{
+	switch (sslerr) {
+	case SSL_ERROR_SSL:
+		return "Internal SSL error";
+	case SSL_ERROR_SYSCALL:
+		if (!ret) {
+			return "System call EOF";
+		} else if (ret == -1) {
+			char *buf;
+
+			buf = ast_threadstorage_get(&err2str_threadbuf, ERR2STR_BUFSIZE);
+			if (!buf) {
+				return "Unknown";
+			}
+
+			snprintf(buf, ERR2STR_BUFSIZE, "Underlying BIO error: %s", strerror(errno));
+			return buf;
+		} else {
+			return "System call other";
+		}
+	default:
+		break;
+	}
+
+	return "Unknown";
+}
+#endif
+
+int ast_iostream_get_fd(struct ast_iostream *stream)
+{
+	return stream->fd;
+}
+
+void ast_iostream_nonblock(struct ast_iostream *stream)
+{
+	ast_fd_set_flags(stream->fd, O_NONBLOCK);
+}
+
+SSL *ast_iostream_get_ssl(struct ast_iostream *stream)
+{
+	return stream->ssl;
+}
+
+void ast_iostream_set_timeout_disable(struct ast_iostream *stream)
+{
+	ast_assert(stream != NULL);
+
+	stream->timeout = -1;
+	stream->timeout_reset = -1;
+}
+
+void ast_iostream_set_timeout_inactivity(struct ast_iostream *stream, int timeout)
+{
+	ast_assert(stream != NULL);
+
+	stream->start.tv_sec = 0;
+	stream->timeout = timeout;
+	stream->timeout_reset = timeout;
+}
+
+void ast_iostream_set_timeout_idle_inactivity(struct ast_iostream *stream, int timeout, int timeout_reset)
+{
+	ast_assert(stream != NULL);
+
+	stream->start.tv_sec = 0;
+	stream->timeout = timeout;
+	stream->timeout_reset = timeout_reset;
+}
+
+void ast_iostream_set_timeout_sequence(struct ast_iostream *stream, struct timeval start, int timeout)
+{
+	ast_assert(stream != NULL);
+
+	stream->start = start;
+	stream->timeout = timeout;
+	stream->timeout_reset = timeout;
+}
+
+void ast_iostream_set_exclusive_input(struct ast_iostream *stream, int exclusive_input)
+{
+	ast_assert(stream != NULL);
+
+	stream->exclusive_input = exclusive_input;
+}
+
+static ssize_t iostream_read(struct ast_iostream *stream, void *buf, size_t size)
+{
+	struct timeval start;
+	int ms;
+	int res;
+
+	if (stream->start.tv_sec) {
+		start = stream->start;
+	} else {
+		start = ast_tvnow();
+	}
+
+#if defined(DO_SSL)
+	if (stream->ssl) {
+		for (;;) {
+			int sslerr;
+			char err[256];
+			res = SSL_read(stream->ssl, buf, size);
+			if (0 < res) {
+				/* We read some payload data. */
+				stream->timeout = stream->timeout_reset;
+				return res;
+			}
+			sslerr = SSL_get_error(stream->ssl, res);
+			switch (sslerr) {
+			case SSL_ERROR_ZERO_RETURN:
+				/* Report EOF for a shutdown */
+				ast_debug(1, "TLS clean shutdown alert reading data\n");
+				return 0;
+			case SSL_ERROR_WANT_READ:
+				if (!stream->exclusive_input) {
+					/* We cannot wait for data now. */
+					errno = EAGAIN;
+					return -1;
+				}
+				while ((ms = ast_remaining_ms(start, stream->timeout))) {
+					res = ast_wait_for_input(stream->fd, ms);
+					if (0 < res) {
+						/* Socket is ready to be read. */
+						break;
+					}
+					if (res < 0) {
+						if (errno == EINTR || errno == EAGAIN) {
+							/* Try again. */
+							continue;
+						}
+						ast_debug(1, "TLS socket error waiting for read data: %s\n",
+							strerror(errno));
+						return -1;
+					}
+				}
+				break;
+			case SSL_ERROR_WANT_WRITE:
+				while ((ms = ast_remaining_ms(start, stream->timeout))) {
+					res = ast_wait_for_output(stream->fd, ms);
+					if (0 < res) {
+						/* Socket is ready to be written. */
+						break;
+					}
+					if (res < 0) {
+						if (errno == EINTR || errno == EAGAIN) {
+							/* Try again. */
+							continue;
+						}
+						ast_debug(1, "TLS socket error waiting for write space: %s\n",
+							strerror(errno));
+						return -1;
+					}
+				}
+				break;
+			case SSL_ERROR_SYSCALL:
+				/* Some non-recoverable I/O error occurred. The OpenSSL error queue may
+				 * contain more information on the error. For socket I/O on Unix systems,
+				 * consult errno for details. */
+				ast_debug(1, "TLS non-recoverable I/O error occurred: %s, %s\n", ERR_error_string(sslerr, err),
+					ssl_error_to_string(sslerr, res));
+				return -1;
+			default:
+				/* Report EOF for an undecoded SSL or transport error. */
+				ast_debug(1, "TLS transport or SSL error reading data:  %s, %s\n", ERR_error_string(sslerr, err),
+					ssl_error_to_string(sslerr, res));
+				return -1;
+			}
+			if (!ms) {
+				/* Report EOF for a timeout */
+				ast_debug(1, "TLS timeout reading data\n");
+				return 0;
+			}
+		}
+	}
+#endif	/* defined(DO_SSL) */
+
+	for (;;) {
+		res = read(stream->fd, buf, size);
+		if (0 <= res) {
+			/* Got data or we cannot wait for it. */
+			stream->timeout = stream->timeout_reset;
+			return res;
+		}
+		if (!stream->exclusive_input) {
+			return res;
+		}
+		if (errno != EINTR && errno != EAGAIN) {
+			/* Not a retryable error. */
+			ast_debug(1, "TCP socket error reading data: %s\n",
+				strerror(errno));
+			return -1;
+		}
+		ms = ast_remaining_ms(start, stream->timeout);
+		if (!ms) {
+			/* Report EOF for a timeout */
+			ast_debug(1, "TCP timeout reading data\n");
+			return 0;
+		}
+		ast_wait_for_input(stream->fd, ms);
+	}
+}
+
+ssize_t ast_iostream_read(struct ast_iostream *stream, void *buffer, size_t count)
+{
+	if (!count) {
+		/* You asked for no data you got no data. */
+		return 0;
+	}
+
+	if (!stream || stream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
+
+	/* Get any remains from the read buffer */
+	if (stream->rbuflen) {
+		size_t r = count;
+		if (r > stream->rbuflen) {
+			r = stream->rbuflen;
+		}
+		memcpy(buffer, stream->rbufhead, r);
+		stream->rbuflen -= r;
+		stream->rbufhead += r;
+		return r;
+	}
+
+	return iostream_read(stream, buffer, count);
+}
+
+ssize_t ast_iostream_gets(struct ast_iostream *stream, char *buffer, size_t size)
+{
+	size_t remaining = size;
+	ssize_t accum_size = 0;
+	ssize_t len;
+	char *newline;
+
+	for (;;) {
+		/* Search for newline */
+		newline = memchr(stream->rbufhead, '\n', stream->rbuflen);
+		if (newline) {
+			len = newline - stream->rbufhead + 1;
+			if (len > remaining - 1) {
+				len = remaining - 1;
+			}
+			break;
+		}
+
+		/* Enough buffered line data to fill request buffer? */
+		if (stream->rbuflen >= remaining - 1) {
+			len = remaining - 1;
+			break;
+		}
+		if (stream->rbuflen) {
+			/* Put leftover buffered line data into request buffer */
+			memcpy(buffer + accum_size, stream->rbufhead, stream->rbuflen);
+			remaining -= stream->rbuflen;
+			accum_size += stream->rbuflen;
+			stream->rbuflen = 0;
+		}
+		stream->rbufhead = stream->rbuf;
+
+		len = iostream_read(stream, stream->rbuf, sizeof(stream->rbuf));
+		if (len == 0) {
+			/* Nothing new was read.  Return whatever we have accumulated. */
+			break;
+		}
+		if (len < 0) {
+			if (accum_size) {
+				/* We have an accumulated buffer so return that instead. */
+				len = 0;
+				break;
+			}
+			return len;
+		}
+		stream->rbuflen += len;
+	}
+
+	/* Return read buffer string length */
+	memcpy(buffer + accum_size, stream->rbufhead, len);
+	buffer[accum_size + len] = 0;
+	stream->rbuflen -= len;
+	stream->rbufhead += len;
+
+	return accum_size + len;
+}
+
+ssize_t ast_iostream_discard(struct ast_iostream *stream, size_t size)
+{
+	char buf[1024];
+	size_t remaining = size;
+	ssize_t ret;
+
+	while (remaining) {
+		ret = ast_iostream_read(stream, buf, remaining > sizeof(buf) ? sizeof(buf) : remaining);
+		if (ret <= 0) {
+			return ret;
+		}
+		remaining -= ret;
+	}
+
+	return size;
+}
+
+ssize_t ast_iostream_write(struct ast_iostream *stream, const void *buffer, size_t size)
+{
+	struct timeval start;
+	int ms;
+	int res;
+	int written;
+	int remaining;
+
+	if (!size) {
+		/* You asked to write no data you wrote no data. */
+		return 0;
+	}
+
+	if (!stream || stream->fd == -1) {
+		errno = EBADF;
+		return -1;
+	}
+
+	if (stream->start.tv_sec) {
+		start = stream->start;
+	} else {
+		start = ast_tvnow();
+	}
+
+#if defined(DO_SSL)
+	if (stream->ssl) {
+		written = 0;
+		remaining = size;
+		for (;;) {
+			int sslerr;
+			char err[256];
+			res = SSL_write(stream->ssl, buffer + written, remaining);
+			if (res == remaining) {
+				/* Everything was written. */
+				return size;
+			}
+			if (0 < res) {
+				/* Successfully wrote part of the buffer.  Try to write the rest. */
+				written += res;
+				remaining -= res;
+				continue;
+			}
+			sslerr = SSL_get_error(stream->ssl, res);
+			switch (sslerr) {
+			case SSL_ERROR_ZERO_RETURN:
+				ast_debug(1, "TLS clean shutdown alert writing data\n");
+				if (written) {
+					/* Report partial write. */
+					return written;
+				}
+				errno = EBADF;
+				return -1;
+			case SSL_ERROR_WANT_READ:
+				ms = ast_remaining_ms(start, stream->timeout);
+				if (!ms) {
+					/* Report partial write. */
+					ast_debug(1, "TLS timeout writing data (want read)\n");
+					return written;
+				}
+				ast_wait_for_input(stream->fd, ms);
+				break;
+			case SSL_ERROR_WANT_WRITE:
+				ms = ast_remaining_ms(start, stream->timeout);
+				if (!ms) {
+					/* Report partial write. */
+					ast_debug(1, "TLS timeout writing data (want write)\n");
+					return written;
+				}
+				ast_wait_for_output(stream->fd, ms);
+				break;
+			default:
+				/* Undecoded SSL or transport error. */
+				ast_debug(1, "TLS transport or SSL error writing data: %s, %s\n", ERR_error_string(sslerr, err),
+					ssl_error_to_string(sslerr, res));
+				if (written) {
+					/* Report partial write. */
+					return written;
+				}
+				errno = EBADF;
+				return -1;
+			}
+		}
+	}
+#endif	/* defined(DO_SSL) */
+
+	written = 0;
+	remaining = size;
+	for (;;) {
+		res = write(stream->fd, buffer + written, remaining);
+		if (res == remaining) {
+			/* Yay everything was written. */
+			return size;
+		}
+		if (0 < res) {
+			/* Successfully wrote part of the buffer.  Try to write the rest. */
+			written += res;
+			remaining -= res;
+			continue;
+		}
+		if (errno != EINTR && errno != EAGAIN) {
+			/* Not a retryable error. */
+			ast_debug(1, "TCP socket error writing: %s\n", strerror(errno));
+			if (written) {
+				return written;
+			}
+			return -1;
+		}
+		ms = ast_remaining_ms(start, stream->timeout);
+		if (!ms) {
+			/* Report partial write. */
+			ast_debug(1, "TCP timeout writing data\n");
+			return written;
+		}
+		ast_wait_for_output(stream->fd, ms);
+	}
+}
+
+ssize_t ast_iostream_printf(struct ast_iostream *stream, const char *format, ...)
+{
+	char sbuf[512], *buf = sbuf;
+	int len, len2, ret = -1;
+	va_list va;
+
+	va_start(va, format);
+	len = vsnprintf(buf, sizeof(sbuf), format, va);
+	va_end(va);
+
+	if (len > sizeof(sbuf) - 1) {
+		/* Add one to the string length to accommodate the NULL byte */
+		size_t buf_len = len + 1;
+
+		buf = ast_malloc(buf_len);
+		if (!buf) {
+			return -1;
+		}
+		va_start(va, format);
+		len2 = vsnprintf(buf, buf_len, format, va);
+		va_end(va);
+		if (len2 != len) {
+			goto error;
+		}
+	}
+
+	if (ast_iostream_write(stream, buf, len) == len)
+		ret = len;
+
+error:
+	if (buf != sbuf) {
+		ast_free(buf);
+	}
+
+	return ret;
+}
+
+int ast_iostream_close(struct ast_iostream *stream)
+{
+	if (!stream) {
+		errno = EBADF;
+		return -1;
+	}
+
+	if (stream->fd != -1) {
+#if defined(DO_SSL)
+		if (stream->ssl) {
+			int res;
+
+			/*
+			 * According to the TLS standard, it is acceptable for an
+			 * application to only send its shutdown alert and then
+			 * close the underlying connection without waiting for
+			 * the peer's response (this way resources can be saved,
+			 * as the process can already terminate or serve another
+			 * connection).
+			 */
+			res = SSL_shutdown(stream->ssl);
+			if (res < 0) {
+				int sslerr = SSL_get_error(stream->ssl, res);
+				char err[256];
+				ast_log(LOG_ERROR, "SSL_shutdown() failed: %s, %s\n",
+					ERR_error_string(sslerr, err), ssl_error_to_string(sslerr, res));
+			}
+
+#if !defined(LIBRESSL_VERSION_NUMBER) && (OPENSSL_VERSION_NUMBER >= 0x10100000L)
+			if (!SSL_is_server(stream->ssl)) {
+#else
+			if (!stream->ssl->server) {
+#endif
+				/* For client threads, ensure that the error stack is cleared */
+#if defined(LIBRESSL_VERSION_NUMBER) || (OPENSSL_VERSION_NUMBER < 0x10100000L)
+#if OPENSSL_VERSION_NUMBER >= 0x10000000L
+				ERR_remove_thread_state(NULL);
+#else
+				ERR_remove_state(0);
+#endif	/* OPENSSL_VERSION_NUMBER >= 0x10000000L */
+#endif  /* OPENSSL_VERSION_NUMBER  < 0x10100000L */
+			}
+
+			SSL_free(stream->ssl);
+			stream->ssl = NULL;
+		}
+#endif	/* defined(DO_SSL) */
+
+		/*
+		 * Issuing shutdown() is necessary here to avoid a race
+		 * condition where the last data written may not appear
+		 * in the TCP stream.  See ASTERISK-23548
+		 */
+		shutdown(stream->fd, SHUT_RDWR);
+		if (close(stream->fd)) {
+			ast_log(LOG_ERROR, "close() failed: %s\n", strerror(errno));
+		}
+		stream->fd = -1;
+	}
+	ao2_t_ref(stream, -1, "Closed ast_iostream");
+
+	return 0;
+}
+
+static void iostream_dtor(void *cookie)
+{
+#ifdef AST_DEVMODE
+	/* Since the ast_assert below is the only one using stream,
+	 * and ast_assert is only available with AST_DEVMODE, we
+	 * put this in a conditional to avoid compiler warnings. */
+	struct ast_iostream *stream = cookie;
+#endif
+
+	ast_assert(stream->fd == -1);
+}
+
+struct ast_iostream *ast_iostream_from_fd(int *fd)
+{
+	struct ast_iostream *stream;
+
+	stream = ao2_alloc_options(sizeof(*stream), iostream_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (stream) {
+		stream->timeout = -1;
+		stream->timeout_reset = -1;
+		stream->fd = *fd;
+		*fd = -1;
+	}
+
+	return stream;
+}
+
+int ast_iostream_start_tls(struct ast_iostream **pstream, SSL_CTX *ssl_ctx, int client)
+{
+#ifdef DO_SSL
+	struct ast_iostream *stream = *pstream;
+	int (*ssl_setup)(SSL *) = client ? SSL_connect : SSL_accept;
+	int res;
+
+	stream->ssl = SSL_new(ssl_ctx);
+	if (!stream->ssl) {
+		ast_log(LOG_ERROR, "Unable to create new SSL connection\n");
+		errno = ENOMEM;
+		return -1;
+	}
+
+	/*
+	 * This function takes struct ast_iostream **, so it can chain
+	 * SSL over any ast_iostream. For now we assume it's a file descriptor.
+	 * But later this should instead use BIO wrapper to tie SSL to another
+	 * ast_iostream.
+	 */
+	SSL_set_fd(stream->ssl, stream->fd);
+
+	res = ssl_setup(stream->ssl);
+	if (res <= 0) {
+		int sslerr = SSL_get_error(stream->ssl, res);
+		char err[256];
+
+		ast_log(LOG_ERROR, "Problem setting up ssl connection: %s, %s\n",
+			ERR_error_string(sslerr, err), ssl_error_to_string(sslerr, res));
+		errno = EIO;
+		return -1;
+	}
+
+	return 0;
+#else
+	ast_log(LOG_ERROR, "SSL not enabled in this build\n");
+	errno = ENOTSUP;
+	return -1;
+#endif
+}
diff -pruN 1:13.23.1~dfsg-2/main/jitterbuf.c 1:16.1.1~dfsg-1/main/jitterbuf.c
--- 1:13.23.1~dfsg-2/main/jitterbuf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/jitterbuf.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "jitterbuf.h"
 #include "asterisk/utils.h"
 
diff -pruN 1:13.23.1~dfsg-2/main/json.c 1:16.1.1~dfsg-1/main/json.c
--- 1:13.23.1~dfsg-2/main/json.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/json.c	2018-12-26 18:45:13.000000000 +0000
@@ -21,7 +21,7 @@
  * \brief JSON abstraction layer.
  *
  * This is a very thin wrapper around the Jansson API. For more details on it,
- * see its docs at http://www.digip.org/jansson/doc/2.4/apiref.html.
+ * see its docs at http://www.digip.org/jansson/doc/2.11/apiref.html.
  *
  * \author David M. Lee, II <dlee@digium.com>
  */
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/json.h"
 #include "asterisk/localtime.h"
 #include "asterisk/module.h"
@@ -46,7 +44,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <jansson.h>
 #include <time.h>
 
-#if defined(JANSSON_THREAD_SAFE_REFCOUNT)
 void *ast_json_malloc(size_t size)
 {
 	return ast_malloc(size);
@@ -57,155 +54,6 @@ void ast_json_free(void *p)
 	ast_free(p);
 }
 
-/* No need to lock since jansson is thread safe. */
-#define SCOPED_JSON_LOCK(json)
-
-#else
-/*! \brief Magic number, for safety checks. */
-#define JSON_MAGIC 0x1541992
-
-/*! \brief Internal structure for allocated memory blocks */
-struct json_mem {
-	/*! Magic number, for safety checks */
-	uint32_t magic;
-	/*! Mutext for locking this memory block */
-	ast_mutex_t mutex;
-	/*! Linked list pointer for the free list */
-	AST_LIST_ENTRY(json_mem) list;
-	/*! Data section of the allocation; void pointer for proper alignment */
-	void *data[];
-};
-
-/*! \brief Free a \ref json_mem block. */
-static void json_mem_free(struct json_mem *mem)
-{
-	mem->magic = 0;
-	ast_mutex_destroy(&mem->mutex);
-	ast_free(mem);
-}
-
-/*!
- * \brief Get the \ref json_mem block for a pointer allocated via
- * ast_json_malloc().
- *
- * This function properly handles Jansson singletons (null, true, false), and
- * \c NULL.
- *
- * \param p Pointer, usually to a \c json_t or \ref ast_json.
- * \return \ref json_mem object with extra allocation info.
- */
-static inline struct json_mem *to_json_mem(void *p)
-{
-	struct json_mem *mem;
-	/* Avoid ref'ing the singleton values */
-	if (p == NULL || p == json_null() || p == json_true() ||
-		p == json_false()) {
-		return NULL;
-	}
-	mem = (struct json_mem *)((char *) (p) - sizeof(*mem));
-	ast_assert(mem->magic == JSON_MAGIC);
-	return mem;
-}
-
-/*!
- * \brief Lock an \ref ast_json instance.
- *
- * If \a json is an immutable singleton (null, true, false), this function
- * safely ignores it and returns \c NULL. Otherwise, \a json must have been
- * allocates using ast_json_malloc().
- *
- * \param json JSON instance to lock.
- * \return \ref Corresponding \ref json_mem block.
- * \return \c NULL if \a json was not allocated.
- */
-static struct json_mem *json_mem_lock(struct ast_json *json)
-{
-	struct json_mem *mem = to_json_mem(json);
-	if (!mem) {
-		return NULL;
-	}
-	ast_mutex_lock(&mem->mutex);
-	return mem;
-}
-
-/*!
- * \brief Unlock a \ref json_mem instance.
- *
- * \param mem \ref json_mem, usually returned from json_mem_lock().
- */
-static void json_mem_unlock(struct json_mem *mem)
-{
-	if (!mem) {
-		return;
-	}
-	ast_mutex_unlock(&mem->mutex);
-}
-
-/*!
- * \brief Scoped lock for a \ref ast_json instance.
- *
- * \param json JSON instance to lock.
- */
-#define SCOPED_JSON_LOCK(json)				\
-	RAII_VAR(struct json_mem *, __mem_ ## __LINE__, \
-		json_mem_lock(json), json_mem_unlock)
-
-void *ast_json_malloc(size_t size)
-{
-	struct json_mem *mem = ast_malloc(size + sizeof(*mem));
-	if (!mem) {
-		return NULL;
-	}
-	mem->magic = JSON_MAGIC;
-	ast_mutex_init(&mem->mutex);
-	return mem->data;
-}
-
-AST_THREADSTORAGE(json_free_list_ts);
-
-/*!
- * \brief Struct for a linked list of \ref json_mem.
- */
-AST_LIST_HEAD_NOLOCK(json_mem_list, json_mem);
-
-/*!
- * \brief Thread local list of \ref json_mem blocks to free at the end of an
- * unref.
- */
-static struct json_mem_list *json_free_list(void)
-{
-	return ast_threadstorage_get(&json_free_list_ts,
-		sizeof(struct json_mem_list));
-}
-
-void ast_json_free(void *p)
-{
-	struct json_mem *mem;
-	struct json_mem_list *free_list;
-	mem = to_json_mem(p);
-
-	if (!mem) {
-		return;
-	}
-
-	/* Since the unref is holding a lock in mem, we can't free it
-	 * immediately. Store it off on a thread local list to be freed by
-	 * ast_json_unref().
-	 */
-	free_list = json_free_list();
-	if (!free_list) {
-		ast_log(LOG_ERROR, "Error allocating free list\n");
-		ast_assert(0);
-		/* It's not ideal to free the memory immediately, but that's the
-		 * best we can do if the threadlocal allocation fails */
-		json_mem_free(mem);
-		return;
-	}
-
-	AST_LIST_INSERT_HEAD(free_list, mem, list);
-}
-#endif
-
 void ast_json_set_alloc_funcs(void *(*malloc_fn)(size_t), void (*free_fn)(void*))
 {
 	json_set_alloc_funcs(malloc_fn, free_fn);
@@ -218,42 +66,13 @@ void ast_json_reset_alloc_funcs(void)
 
 struct ast_json *ast_json_ref(struct ast_json *json)
 {
-	/* If Jansson refcounting is non-atomic; lock it. */
-	SCOPED_JSON_LOCK(json);
 	json_incref((json_t *)json);
 	return json;
 }
 
 void ast_json_unref(struct ast_json *json)
 {
-#if defined(JANSSON_THREAD_SAFE_REFCOUNT)
 	json_decref((json_t *) json);
-#else
-	struct json_mem_list *free_list;
-	struct json_mem *mem;
-
-	if (!json) {
-		return;
-	}
-
-	/* Jansson refcounting is non-atomic; lock it. */
-	{
-		SCOPED_JSON_LOCK(json);
-
-		json_decref((json_t *) json);
-	}
-
-	/* Now free any objects that were ast_json_free()'s while the lock was
-	 * held */
-	free_list = json_free_list();
-	if (!free_list) {
-		return;
-	}
-
-	while ((mem = AST_LIST_REMOVE_HEAD(free_list, list))) {
-		json_mem_free(mem);
-	}
-#endif
 }
 
 enum ast_json_type ast_json_typeof(const struct ast_json *json)
@@ -423,11 +242,7 @@ struct ast_json *ast_json_false(void)
 
 struct ast_json *ast_json_boolean(int value)
 {
-#if JANSSON_VERSION_HEX >= 0x020400
 	return (struct ast_json *)json_boolean(value);
-#else
-	return value ? ast_json_true() : ast_json_false();
-#endif
 }
 
 struct ast_json *ast_json_null(void)
@@ -477,16 +292,25 @@ struct ast_json *ast_json_stringf(const
 
 struct ast_json *ast_json_vstringf(const char *format, va_list args)
 {
-	char *str = NULL;
 	json_t *ret = NULL;
 
 	if (format) {
+		/* json_pack was not introduced until jansson-2.0 so Asterisk could never
+		 * be compiled against older versions.  The version check can never match
+		 * anything older than 2.12. */
+#if defined(HAVE_JANSSON_BUNDLED) || JANSSON_MAJOR_VERSION > 2 || JANSSON_MINOR_VERSION > 11
+		ret = json_vsprintf(format, args);
+#else
+		char *str = NULL;
 		int err = ast_vasprintf(&str, format, args);
+
 		if (err >= 0) {
 			ret = json_string(str);
 			ast_free(str);
 		}
+#endif
 	}
+
 	return (struct ast_json *)ret;
 }
 
@@ -595,57 +419,11 @@ int ast_json_object_update(struct ast_js
 }
 int ast_json_object_update_existing(struct ast_json *object, struct ast_json *other)
 {
-#if JANSSON_VERSION_HEX >= 0x020300
 	return json_object_update_existing((json_t *)object, (json_t *)other);
-#else
-	struct ast_json_iter *iter = ast_json_object_iter(other);
-	int ret = 0;
-
-	if (object == NULL || other == NULL) {
-		return -1;
-	}
-
-	while (iter != NULL && ret == 0) {
-		const char *key = ast_json_object_iter_key(iter);
-
-		if (ast_json_object_get(object, key) != NULL) {
-			struct ast_json *value = ast_json_object_iter_value(iter);
-
-			if (!value || ast_json_object_set(object, key, ast_json_ref(value))) {
-				ret = -1;
-			}
-		}
-		iter = ast_json_object_iter_next(other, iter);
-	}
-	return ret;
-#endif
 }
 int ast_json_object_update_missing(struct ast_json *object, struct ast_json *other)
 {
-#if JANSSON_VERSION_HEX >= 0x020300
 	return json_object_update_missing((json_t *)object, (json_t *)other);
-#else
-	struct ast_json_iter *iter = ast_json_object_iter(other);
-	int ret = 0;
-
-	if (object == NULL || other == NULL) {
-		return -1;
-	}
-
-	while (iter != NULL && ret == 0) {
-		const char *key = ast_json_object_iter_key(iter);
-
-		if (ast_json_object_get(object, key) == NULL) {
-			struct ast_json *value = ast_json_object_iter_value(iter);
-
-			if (!value || ast_json_object_set(object, key, ast_json_ref(value))) {
-				ret = -1;
-			}
-		}
-		iter = ast_json_object_iter_next(other, iter);
-	}
-	return ret;
-#endif
 }
 
 struct ast_json_iter *ast_json_object_iter(struct ast_json *object)
@@ -684,14 +462,6 @@ static size_t dump_flags(enum ast_json_e
 
 char *ast_json_dump_string_format(struct ast_json *root, enum ast_json_encoding_format format)
 {
-	/* Jansson's json_dump*, even though it's a read operation, isn't
-	 * thread safe for concurrent reads. Locking is necessary.
-	 * See http://www.digip.org/jansson/doc/2.4/portability.html#thread-safety.
-	 *
-	 * This comment does not apply when JANSSON_THREAD_SAFE_REFCOUNT is defined,
-	 * in that case SCOPED_JSON_LOCK is a no-op.
-	 */
-	SCOPED_JSON_LOCK(root);
 	return json_dumps((json_t *)root, dump_flags(format));
 }
 
@@ -728,28 +498,12 @@ static int write_to_ast_str(const char *
 
 int ast_json_dump_str_format(struct ast_json *root, struct ast_str **dst, enum ast_json_encoding_format format)
 {
-	/* Jansson's json_dump*, even though it's a read operation, isn't
-	 * thread safe for concurrent reads. Locking is necessary.
-	 * See http://www.digip.org/jansson/doc/2.4/portability.html#thread-safety.
-	 *
-	 * This comment does not apply when JANSSON_THREAD_SAFE_REFCOUNT is defined,
-	 * in that case SCOPED_JSON_LOCK is a no-op.
-	 */
-	SCOPED_JSON_LOCK(root);
 	return json_dump_callback((json_t *)root, write_to_ast_str, dst, dump_flags(format));
 }
 
 
 int ast_json_dump_file_format(struct ast_json *root, FILE *output, enum ast_json_encoding_format format)
 {
-	/* Jansson's json_dump*, even though it's a read operation, isn't
-	 * thread safe for concurrent reads. Locking is necessary.
-	 * See http://www.digip.org/jansson/doc/2.4/portability.html#thread-safety.
-	 *
-	 * This comment does not apply when JANSSON_THREAD_SAFE_REFCOUNT is defined,
-	 * in that case SCOPED_JSON_LOCK is a no-op.
-	 */
-	SCOPED_JSON_LOCK(root);
 	if (!root || !output) {
 		return -1;
 	}
@@ -757,14 +511,6 @@ int ast_json_dump_file_format(struct ast
 }
 int ast_json_dump_new_file_format(struct ast_json *root, const char *path, enum ast_json_encoding_format format)
 {
-	/* Jansson's json_dump*, even though it's a read operation, isn't
-	 * thread safe for concurrent reads. Locking is necessary.
-	 * See http://www.digip.org/jansson/doc/2.4/portability.html#thread-safety.
-	 *
-	 * This comment does not apply when JANSSON_THREAD_SAFE_REFCOUNT is defined,
-	 * in that case SCOPED_JSON_LOCK is a no-op.
-	 */
-	SCOPED_JSON_LOCK(root);
 	if (!root || !path) {
 		return -1;
 	}
@@ -1010,37 +756,15 @@ static struct ast_json *json_party_subad
 
 struct ast_json *ast_json_party_id(struct ast_party_id *party)
 {
-	RAII_VAR(struct ast_json *, json_party_id, NULL, ast_json_unref);
-	int pres;
+	int pres = ast_party_id_presentation(party);
 
 	/* Combined party presentation */
-	pres = ast_party_id_presentation(party);
-	json_party_id = ast_json_pack("{s: i, s: s}",
+	return ast_json_pack("{s: i, s: s, s: o*, s: o*, s: o*}",
 		"presentation", pres,
-		"presentation_txt", ast_describe_caller_presentation(pres));
-	if (!json_party_id) {
-		return NULL;
-	}
-
-	/* Party number */
-	if (party->number.valid
-		&& ast_json_object_set(json_party_id, "number", json_party_number(&party->number))) {
-		return NULL;
-	}
-
-	/* Party name */
-	if (party->name.valid
-		&& ast_json_object_set(json_party_id, "name", json_party_name(&party->name))) {
-		return NULL;
-	}
-
-	/* Party subaddress */
-	if (party->subaddress.valid
-		&& ast_json_object_set(json_party_id, "subaddress", json_party_subaddress(&party->subaddress))) {
-		return NULL;
-	}
-
-	return ast_json_ref(json_party_id);
+		"presentation_txt", ast_describe_caller_presentation(pres),
+		"number", json_party_number(&party->number),
+		"name", json_party_name(&party->name),
+		"subaddress", json_party_subaddress(&party->subaddress));
 }
 
 enum ast_json_to_ast_vars_code ast_json_to_ast_variables(struct ast_json *json_variables, struct ast_variable **variables)
@@ -1088,3 +812,16 @@ enum ast_json_to_ast_vars_code ast_json_
 
 	return AST_JSON_TO_AST_VARS_CODE_SUCCESS;
 }
+
+struct ast_json *ast_json_channel_vars(struct varshead *channelvars)
+{
+	struct ast_json *ret;
+	struct ast_var_t *var;
+
+	ret = ast_json_object_create();
+	AST_LIST_TRAVERSE(channelvars, var, entries) {
+		ast_json_object_set(ret, var->name, ast_json_string_create(var->value));
+	}
+
+	return ret;
+}
diff -pruN 1:13.23.1~dfsg-2/main/libasteriskpj.c 1:16.1.1~dfsg-1/main/libasteriskpj.c
--- 1:13.23.1~dfsg-2/main/libasteriskpj.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/libasteriskpj.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #ifdef HAVE_PJPROJECT
 #include <pjlib.h>
 #endif
diff -pruN 1:13.23.1~dfsg-2/main/libasteriskssl.c 1:16.1.1~dfsg-1/main/libasteriskssl.c
--- 1:13.23.1~dfsg-2/main/libasteriskssl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/libasteriskssl.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"   /* ast_ssl_init() */
 
 #ifdef HAVE_OPENSSL
diff -pruN 1:13.23.1~dfsg-2/main/loader.c 1:16.1.1~dfsg-1/main/loader.c
--- 1:13.23.1~dfsg-2/main/loader.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/loader.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_MODULE_DIR */
 #include <dirent.h>
@@ -44,19 +42,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/config.h"
 #include "asterisk/channel.h"
 #include "asterisk/term.h"
-#include "asterisk/acl.h"
 #include "asterisk/manager.h"
-#include "asterisk/cdr.h"
-#include "asterisk/enum.h"
-#include "asterisk/http.h"
+#include "asterisk/io.h"
 #include "asterisk/lock.h"
-#include "asterisk/features_config.h"
-#include "asterisk/dsp.h"
-#include "asterisk/udptl.h"
 #include "asterisk/vector.h"
 #include "asterisk/app.h"
 #include "asterisk/test.h"
-#include "asterisk/sounds_index.h"
 #include "asterisk/cli.h"
 
 #include <dlfcn.h>
@@ -89,6 +80,40 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			</syntax>
 		</managerEventInstance>
 	</managerEvent>
+	<managerEvent language="en_US" name="Load">
+		<managerEventInstance class="EVENT_FLAG_SYSTEM">
+			<synopsis>Raised when a module has been loaded in Asterisk.</synopsis>
+			<syntax>
+				<parameter name="Module">
+					<para>The name of the module that was loaded</para>
+				</parameter>
+				<parameter name="Status">
+					<para>The result of the load request.</para>
+					<enumlist>
+						<enum name="Failure"><para>Module could not be loaded properly</para></enum>
+						<enum name="Success"><para>Module loaded and configured</para></enum>
+						<enum name="Decline"><para>Module is not configured</para></enum>
+					</enumlist>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
+	<managerEvent language="en_US" name="Unload">
+		<managerEventInstance class="EVENT_FLAG_SYSTEM">
+			<synopsis>Raised when a module has been unloaded in Asterisk.</synopsis>
+			<syntax>
+				<parameter name="Module">
+					<para>The name of the module that was unloaded</para>
+				</parameter>
+				<parameter name="Status">
+					<para>The result of the unload request.</para>
+					<enumlist>
+						<enum name="Success"><para>Module unloaded successfully</para></enum>
+					</enumlist>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
  ***/
 
 #ifndef RTLD_NOW
@@ -114,6 +139,39 @@ static char buildopt_sum[33] = AST_BUILD
 
 AST_VECTOR(module_vector, struct ast_module *);
 
+/*! Used with AST_VECTOR_CALLBACK_VOID to create a
+ * comma separated list of module names for error messages. */
+#define STR_APPEND_TEXT(txt, str) \
+	ast_str_append(str, 0, "%s%s", \
+		ast_str_strlen(*(str)) > 0 ? ", " : "", \
+		txt)
+
+/* Built-in module registrations need special handling at startup */
+static unsigned int loader_ready;
+
+/*! String container for deferring output of startup errors. */
+static struct ast_vector_string startup_errors;
+static struct ast_str *startup_error_builder;
+
+static __attribute__((format(printf, 1, 2))) void module_load_error(const char *fmt, ...)
+{
+	char *copy = NULL;
+	va_list ap;
+
+	va_start(ap, fmt);
+	if (startup_error_builder) {
+		ast_str_set_va(&startup_error_builder, 0, fmt, ap);
+		copy = ast_strdup(ast_str_buffer(startup_error_builder));
+		if (!copy || AST_VECTOR_APPEND(&startup_errors, copy)) {
+			ast_log(LOG_ERROR, "%s", ast_str_buffer(startup_error_builder));
+			ast_free(copy);
+		}
+	} else {
+		ast_log_ap(LOG_ERROR, fmt, ap);
+	}
+	va_end(ap);
+}
+
 /*!
  * \brief Internal flag to indicate all modules have been initially loaded.
  */
@@ -121,16 +179,29 @@ static int modules_loaded;
 
 struct ast_module {
 	const struct ast_module_info *info;
-#ifdef REF_DEBUG
 	/*! Used to get module references into refs log */
 	void *ref_debug;
-#endif
 	/*! The shared lib. */
 	void *lib;
 	/*! Number of 'users' and other references currently holding the module. */
 	int usecount;
 	/*! List of users holding the module. */
 	struct module_user_list users;
+
+	/*! List of required module names. */
+	struct ast_vector_string requires;
+	/*! List of optional api modules. */
+	struct ast_vector_string optional_modules;
+	/*! List of modules this enhances. */
+	struct ast_vector_string enhances;
+
+	/*!
+	 * \brief Vector holding pointers to modules we have a reference to.
+	 *
+	 * When one module requires another, the required module gets added
+	 * to this list with a reference.
+	 */
+	struct module_vector reffed_deps;
 	struct {
 		/*! The module running and ready to accept requests. */
 		unsigned int running:1;
@@ -138,6 +209,12 @@ struct ast_module {
 		unsigned int declined:1;
 		/*! This module is being held open until it's time to shutdown. */
 		unsigned int keepuntilshutdown:1;
+		/*! The module is built-in. */
+		unsigned int builtin:1;
+		/*! The admin has declared this module is required. */
+		unsigned int required:1;
+		/*! This module is marked for preload. */
+		unsigned int preload:1;
 	} flags;
 	AST_DLLIST_ENTRY(ast_module) entry;
 	char resource[0];
@@ -145,6 +222,34 @@ struct ast_module {
 
 static AST_DLLIST_HEAD_STATIC(module_list, ast_module);
 
+
+struct load_results_map {
+	int result;
+	const char *name;
+};
+
+static const struct load_results_map load_results[] = {
+	{ AST_MODULE_LOAD_SUCCESS, "Success" },
+	{ AST_MODULE_LOAD_DECLINE, "Decline" },
+	{ AST_MODULE_LOAD_SKIP, "Skip" },
+	{ AST_MODULE_LOAD_PRIORITY, "Priority" },
+	{ AST_MODULE_LOAD_FAILURE, "Failure" },
+};
+#define AST_MODULE_LOAD_UNKNOWN_STRING		"Unknown"		/* Status string for unknown load status */
+
+static void publish_load_message_type(const char* type, const char *name, const char *status);
+static void publish_reload_message(const char *name, enum ast_module_reload_result result);
+static void publish_load_message(const char *name, enum ast_module_load_result result);
+static void publish_unload_message(const char *name, const char* status);
+
+
+/*
+ * module_list is cleared by its constructor possibly after
+ * we start accumulating built-in modules, so we need to
+ * use another list (without the lock) to accumulate them.
+ */
+static struct module_list builtin_module_list;
+
 static int module_vector_strcasecmp(struct ast_module *a, struct ast_module *b)
 {
 	return strcasecmp(a->resource, b->resource);
@@ -152,12 +257,20 @@ static int module_vector_strcasecmp(stru
 
 static int module_vector_cmp(struct ast_module *a, struct ast_module *b)
 {
+	int preload_diff = (int)b->flags.preload - (int)a->flags.preload;
 	/* if load_pri is not set, default is 128.  Lower is better */
 	int a_pri = ast_test_flag(a->info, AST_MODFLAG_LOAD_ORDER)
 		? a->info->load_pri : AST_MODPRI_DEFAULT;
 	int b_pri = ast_test_flag(b->info, AST_MODFLAG_LOAD_ORDER)
 		? b->info->load_pri : AST_MODPRI_DEFAULT;
 
+	if (preload_diff) {
+		/* -1 preload a but not b */
+		/*  0 preload both or neither */
+		/*  1 preload b but not a */
+		return preload_diff;
+	}
+
 	/*
 	 * Returns comparison values for a vector sorted by priority.
 	 * <0 a_pri < b_pri
@@ -167,6 +280,225 @@ static int module_vector_cmp(struct ast_
 	return a_pri - b_pri;
 }
 
+static struct ast_module *find_resource(const char *resource, int do_lock);
+
+/*!
+ * \internal
+ * \brief Add a reference from mod to dep.
+ *
+ * \param mod Owner of the new reference.
+ * \param dep Module to reference
+ * \param missing Vector to store name of \a dep if it is not running.
+ *
+ * This function returns failure if \a dep is not running and \a missing
+ * is NULL.  If \a missing is not NULL errors will only be returned for
+ * allocation failures.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ *
+ * \note Adding a second reference to the same dep will return success
+ *       without doing anything.
+ */
+static int module_reffed_deps_add(struct ast_module *mod, struct ast_module *dep,
+	struct ast_vector_const_string *missing)
+{
+	if (!dep->flags.running) {
+		return !missing ? -1 : AST_VECTOR_APPEND(missing, dep->info->name);
+	}
+
+	if (AST_VECTOR_GET_CMP(&mod->reffed_deps, dep, AST_VECTOR_ELEM_DEFAULT_CMP)) {
+		/* Skip duplicate. */
+		return 0;
+	}
+
+	if (AST_VECTOR_APPEND(&mod->reffed_deps, dep)) {
+		return -1;
+	}
+
+	ast_module_ref(dep);
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Add references for modules that enhance a dependency.
+ *
+ * \param mod Owner of the new references.
+ * \param dep Module to check for enhancers.
+ * \param missing Vector to store name of any enhancer that is not running or declined.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ */
+static int module_reffed_deps_add_dep_enhancers(struct ast_module *mod,
+	struct ast_module *dep, struct ast_vector_const_string *missing)
+{
+	struct ast_module *cur;
+
+	AST_DLLIST_TRAVERSE(&module_list, cur, entry) {
+		if (cur->flags.declined) {
+			continue;
+		}
+
+		if (!AST_VECTOR_GET_CMP(&cur->enhances, dep->info->name, !strcasecmp)) {
+			/* dep is not enhanced by cur. */
+			continue;
+		}
+
+		/* dep is enhanced by cur, therefore mod requires cur. */
+		if (module_reffed_deps_add(mod, cur, missing)) {
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Add references to a list of dependencies.
+ *
+ * \param mod Owner of the new references.
+ * \param vec List of required modules to process
+ * \param missing Vector to store names of modules that are not running.
+ * \param ref_enhancers Reference all enhancers of each required module.
+ * \param isoptional Modules that are not loaded can be ignored.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ */
+static int module_deps_process_reqlist(struct ast_module *mod,
+	struct ast_vector_string *vec, struct ast_vector_const_string *missing,
+	int ref_enhancers, int isoptional)
+{
+	int idx;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(vec); idx++) {
+		const char *depname = AST_VECTOR_GET(vec, idx);
+		struct ast_module *dep = find_resource(depname, 0);
+
+		if (!dep || !dep->flags.running) {
+			if (isoptional && !dep) {
+				continue;
+			}
+
+			if (missing && !AST_VECTOR_APPEND(missing, depname)) {
+				continue;
+			}
+
+			return -1;
+		}
+
+		if (module_reffed_deps_add(mod, dep, missing)) {
+			return -1;
+		}
+
+		if (ref_enhancers && module_reffed_deps_add_dep_enhancers(mod, dep, missing)) {
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Grab all references required to start the module.
+ *
+ * \param mod The module we're trying to start.
+ * \param missing Vector to store a list of missing dependencies.
+ *
+ * \retval 0 Success
+ * \retval -1 Failure
+ *
+ * \note module_list must be locked.
+ *
+ * \note Caller is responsible for initializing and freeing the vector.
+ *       Elements are safely read only while module_list remains locked.
+ */
+static int module_deps_reference(struct ast_module *mod, struct ast_vector_const_string *missing)
+{
+	int res = 0;
+
+	/* Grab references to modules we enhance but not other enhancements. */
+	res |= module_deps_process_reqlist(mod, &mod->enhances, missing, 0, 0);
+
+	/* Grab references to modules we require plus enhancements. */
+	res |= module_deps_process_reqlist(mod, &mod->requires, missing, 1, 0);
+
+	/* Grab references to optional modules including enhancements. */
+	res |= module_deps_process_reqlist(mod, &mod->optional_modules, missing, 1, 1);
+
+	return res;
+}
+
+/*!
+ * \brief Recursively find required dependencies that are not running.
+ *
+ * \param mod Module to scan for dependencies.
+ * \param missingdeps Vector listing modules that must be started first.
+ *
+ * \retval 0 All dependencies resolved.
+ * \retval -1 Failed to resolve some dependencies.
+ *
+ * An error from this function usually means a required module is not even
+ * loaded.  This function is safe from infinite recursion, but dependency
+ * loops are not reported as an error from here.  On success missingdeps
+ * will contain a list of every module that needs to be running before this
+ * module can start.  missingdeps is sorted by load priority so any missing
+ * dependencies can be started if needed.
+ */
+static int module_deps_missing_recursive(struct ast_module *mod, struct module_vector *missingdeps)
+{
+	int i = 0;
+	int res = -1;
+	struct ast_vector_const_string localdeps;
+	struct ast_module *dep;
+
+	/*
+	 * localdeps stores a copy of all dependencies that mod could not reference.
+	 * First we discard modules that we've already found. We add all newly found
+	 * modules to the missingdeps vector then scan them recursively.  This will
+	 * ensure we quickly run out of stuff to do.
+	 */
+	AST_VECTOR_INIT(&localdeps, 0);
+	if (module_deps_reference(mod, &localdeps)) {
+		goto clean_return;
+	}
+
+	while (i < AST_VECTOR_SIZE(&localdeps)) {
+		dep = find_resource(AST_VECTOR_GET(&localdeps, i), 0);
+		if (!dep) {
+			goto clean_return;
+		}
+
+		if (AST_VECTOR_GET_CMP(missingdeps, dep, AST_VECTOR_ELEM_DEFAULT_CMP)) {
+			/* Skip common dependency.  We have already searched it. */
+			AST_VECTOR_REMOVE(&localdeps, i, 0);
+		} else {
+			/* missingdeps is the real list so keep it sorted. */
+			if (AST_VECTOR_ADD_SORTED(missingdeps, dep, module_vector_cmp)) {
+				goto clean_return;
+			}
+			i++;
+		}
+	}
+
+	res = 0;
+	for (i = 0; !res && i < AST_VECTOR_SIZE(&localdeps); i++) {
+		dep = find_resource(AST_VECTOR_GET(&localdeps, i), 0);
+		/* We've already confirmed dep is loaded in the first loop. */
+		res = module_deps_missing_recursive(dep, missingdeps);
+	}
+
+clean_return:
+	AST_VECTOR_FREE(&localdeps);
+
+	return res;
+}
+
 const char *ast_module_name(const struct ast_module *mod)
 {
 	if (!mod || !mod->info) {
@@ -215,6 +547,23 @@ void ast_module_register(const struct as
 {
 	struct ast_module *mod;
 
+	if (!loader_ready) {
+		mod = ast_std_calloc(1, sizeof(*mod) + strlen(info->name) + 1);
+		if (!mod) {
+			/* We haven't even reached main() yet, if we can't
+			 * allocate memory at this point just give up. */
+			fprintf(stderr, "Allocation failure during startup.\n");
+			exit(2);
+		}
+		strcpy(mod->resource, info->name); /* safe */
+		mod->info = info;
+		mod->flags.builtin = 1;
+		AST_DLLIST_INSERT_TAIL(&builtin_module_list, mod, entry);
+
+		/* ast_module_register for built-in modules is run again during module preload. */
+		return;
+	}
+
 	/*
 	 * This lock protects resource_being_loaded as well as the module
 	 * list.  Normally we already have a lock on module_list when we
@@ -234,10 +583,14 @@ void ast_module_register(const struct as
 	resource_being_loaded = NULL;
 
 	mod->info = info;
-#ifdef REF_DEBUG
-	mod->ref_debug = ao2_t_alloc(0, NULL, info->name);
-#endif
+	if (ast_opt_ref_debug) {
+		mod->ref_debug = ao2_t_alloc_options(0, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK, info->name);
+	}
 	AST_LIST_HEAD_INIT(&mod->users);
+	AST_VECTOR_INIT(&mod->requires, 0);
+	AST_VECTOR_INIT(&mod->optional_modules, 0);
+	AST_VECTOR_INIT(&mod->enhances, 0);
+	AST_VECTOR_INIT(&mod->reffed_deps, 0);
 
 	AST_DLLIST_INSERT_TAIL(&module_list, mod, entry);
 	AST_DLLIST_UNLOCK(&module_list);
@@ -246,13 +599,40 @@ void ast_module_register(const struct as
 	*((struct ast_module **) &(info->self)) = mod;
 }
 
+static int module_post_register(struct ast_module *mod)
+{
+	int res;
+
+	/* Split lists from mod->info. */
+	res  = ast_vector_string_split(&mod->requires, mod->info->requires, ",", 0, strcasecmp);
+	res |= ast_vector_string_split(&mod->optional_modules, mod->info->optional_modules, ",", 0, strcasecmp);
+	res |= ast_vector_string_split(&mod->enhances, mod->info->enhances, ",", 0, strcasecmp);
+
+	return res;
+}
+
 static void module_destroy(struct ast_module *mod)
 {
+	AST_VECTOR_CALLBACK_VOID(&mod->requires, ast_free);
+	AST_VECTOR_FREE(&mod->requires);
+
+	AST_VECTOR_CALLBACK_VOID(&mod->optional_modules, ast_free);
+	AST_VECTOR_FREE(&mod->optional_modules);
+
+	AST_VECTOR_CALLBACK_VOID(&mod->enhances, ast_free);
+	AST_VECTOR_FREE(&mod->enhances);
+
+	/* Release references to all dependencies. */
+	AST_VECTOR_CALLBACK_VOID(&mod->reffed_deps, ast_module_unref);
+	AST_VECTOR_FREE(&mod->reffed_deps);
+
 	AST_LIST_HEAD_DESTROY(&mod->users);
-#ifdef REF_DEBUG
 	ao2_cleanup(mod->ref_debug);
-#endif
-	ast_free(mod);
+	if (mod->flags.builtin) {
+		ast_std_free(mod);
+	} else {
+		ast_free(mod);
+	}
 }
 
 void ast_module_unregister(const struct ast_module_info *info)
@@ -273,7 +653,15 @@ void ast_module_unregister(const struct
 	AST_DLLIST_TRAVERSE_BACKWARDS_SAFE_END;
 	AST_DLLIST_UNLOCK(&module_list);
 
-	if (mod) {
+	if (mod && !mod->usecount) {
+		/*
+		 * We are intentionally leaking mod if usecount is not zero.
+		 * This is necessary if the module is being forcefully unloaded.
+		 * In addition module_destroy is not safe to run after exit()
+		 * is called.  ast_module_unregister is run during cleanup of
+		 * the process when libc releases each module's shared object
+		 * library.
+		 */
 		ast_debug(5, "Unregistering module %s\n", info->name);
 		module_destroy(mod);
 	}
@@ -294,9 +682,9 @@ struct ast_module_user *__ast_module_use
 	AST_LIST_INSERT_HEAD(&mod->users, u, entry);
 	AST_LIST_UNLOCK(&mod->users);
 
-#ifdef REF_DEBUG
-	ao2_ref(mod->ref_debug, +1);
-#endif
+	if (mod->ref_debug) {
+		ao2_ref(mod->ref_debug, +1);
+	}
 
 	ast_atomic_fetchadd_int(&mod->usecount, +1);
 
@@ -322,9 +710,9 @@ void __ast_module_user_remove(struct ast
 		return;
 	}
 
-#ifdef REF_DEBUG
-	ao2_ref(mod->ref_debug, -1);
-#endif
+	if (mod->ref_debug) {
+		ao2_ref(mod->ref_debug, -1);
+	}
 
 	ast_atomic_fetchadd_int(&mod->usecount, -1);
 	ast_free(u);
@@ -342,9 +730,9 @@ void __ast_module_user_hangup_all(struct
 			ast_softhangup(u->chan, AST_SOFTHANGUP_APPUNLOAD);
 		}
 
-#ifdef REF_DEBUG
-		ao2_ref(mod->ref_debug, -1);
-#endif
+		if (mod->ref_debug) {
+			ao2_ref(mod->ref_debug, -1);
+		}
 
 		ast_atomic_fetchadd_int(&mod->usecount, -1);
 		ast_free(u);
@@ -354,33 +742,6 @@ void __ast_module_user_hangup_all(struct
 	ast_update_use_count();
 }
 
-/*! \note
- * In addition to modules, the reload command handles some extra keywords
- * which are listed here together with the corresponding handlers.
- * This table is also used by the command completion code.
- */
-static struct reload_classes {
-	const char *name;
-	int (*reload_fn)(void);
-} reload_classes[] = {	/* list in alpha order, longest match first for cli completion */
-	{ "acl",         ast_named_acl_reload },
-	{ "cdr",         ast_cdr_engine_reload },
-	{ "cel",         ast_cel_engine_reload },
-	{ "dnsmgr",      dnsmgr_reload },
-	{ "dsp",         ast_dsp_reload},
-	{ "extconfig",   read_config_maps },
-	{ "enum",        ast_enum_reload },
-	{ "features",    ast_features_config_reload },
-	{ "http",        ast_http_reload },
-	{ "indications", ast_indications_reload },
-	{ "logger",      logger_reload },
-	{ "manager",     reload_manager },
-	{ "plc",         ast_plc_reload },
-	{ "sounds",      ast_sounds_reindex },
-	{ "udptl",       ast_udptl_reload },
-	{ NULL,          NULL }
-};
-
 static int printdigest(const unsigned char *d)
 {
 	int x, pos;
@@ -542,7 +903,21 @@ static void unload_dynamic_module(struct
 #endif
 }
 
-#define MODULE_LOCAL_ONLY (void *)-1
+static int load_dlopen_missing(struct ast_str **list, struct ast_vector_string *deps)
+{
+	int i;
+	int c = 0;
+
+	for (i = 0; i < AST_VECTOR_SIZE(deps); i++) {
+		const char *dep = AST_VECTOR_GET(deps, i);
+		if (!find_resource(dep, 0)) {
+			STR_APPEND_TEXT(dep, list);
+			c++;
+		}
+	}
+
+	return c;
+}
 
 /*!
  * \internal
@@ -575,13 +950,60 @@ static struct ast_module *load_dlopen(co
 	resource_being_loaded = mod;
 	mod->lib = dlopen(filename, flags);
 	if (resource_being_loaded) {
+		struct ast_str *list;
+		int c = 0;
+		const char *dlerror_msg = ast_strdupa(dlerror());
+
 		resource_being_loaded = NULL;
 		if (mod->lib) {
-			ast_log(LOG_ERROR, "Module '%s' did not register itself during load\n", resource_in);
+			module_load_error("Module '%s' did not register itself during load\n", resource_in);
+			logged_dlclose(resource_in, mod->lib);
+
+			goto error_return;
+		}
+
+		if (suppress_logging) {
+			goto error_return;
+		}
+
+		resource_being_loaded = mod;
+		mod->lib = dlopen(filename, RTLD_LAZY | RTLD_LOCAL);
+		if (resource_being_loaded) {
+			resource_being_loaded = NULL;
+
+			module_load_error("Error loading module '%s': %s\n", resource_in, dlerror_msg);
 			logged_dlclose(resource_in, mod->lib);
-		} else if (!suppress_logging) {
-			ast_log(LOG_WARNING, "Error loading module '%s': %s\n", resource_in, dlerror());
+
+			goto error_return;
+		}
+
+		list = ast_str_create(64);
+		if (list) {
+			if (module_post_register(mod)) {
+				goto loaded_error;
+			}
+
+			c = load_dlopen_missing(&list, &mod->requires);
+			c += load_dlopen_missing(&list, &mod->enhances);
+#ifndef OPTIONAL_API
+			c += load_dlopen_missing(&list, &mod->optional_modules);
+#endif
 		}
+
+		if (list && ast_str_strlen(list)) {
+			module_load_error("Error loading module '%s', missing %s: %s\n",
+				resource_in, c == 1 ? "dependency" : "dependencies", ast_str_buffer(list));
+		} else {
+			module_load_error("Error loading module '%s': %s\n", resource_in, dlerror_msg);
+		}
+
+loaded_error:
+		ast_free(list);
+		unload_dynamic_module(mod);
+
+		return NULL;
+
+error_return:
 		ast_free(mod);
 
 		return NULL;
@@ -590,12 +1012,11 @@ static struct ast_module *load_dlopen(co
 	return mod;
 }
 
-static struct ast_module *load_dynamic_module(const char *resource_in, unsigned int global_symbols_only, unsigned int suppress_logging)
+static struct ast_module *load_dynamic_module(const char *resource_in, unsigned int suppress_logging)
 {
 	char fn[PATH_MAX];
 	struct ast_module *mod;
 	size_t resource_in_len = strlen(resource_in);
-	int exports_globals;
 	const char *so_ext = "";
 
 	if (resource_in_len < 4 || strcasecmp(resource_in + resource_in_len - 3, ".so")) {
@@ -604,31 +1025,18 @@ static struct ast_module *load_dynamic_m
 
 	snprintf(fn, sizeof(fn), "%s/%s%s", ast_config_AST_MODULE_DIR, resource_in, so_ext);
 
-	/* Try loading in quiet mode first with flags to export global symbols.
-	 * If the module does not want to export globals we will close and reopen. */
-	mod = load_dlopen(resource_in, so_ext, fn,
-		global_symbols_only ? RTLD_LAZY | RTLD_GLOBAL : RTLD_NOW | RTLD_LOCAL,
-		suppress_logging);
-
-	if (!mod) {
-		return NULL;
-	}
+	/* Try loading in quiet mode first with RTLD_LOCAL.  The majority of modules do not
+	 * export symbols so this allows the least number of calls to dlopen. */
+	mod = load_dlopen(resource_in, so_ext, fn, RTLD_NOW | RTLD_LOCAL, suppress_logging);
 
-	exports_globals = ast_test_flag(mod->info, AST_MODFLAG_GLOBAL_SYMBOLS);
-	if ((global_symbols_only && exports_globals) || (!global_symbols_only && !exports_globals)) {
-		/* The first dlopen had the correct flags. */
+	if (!mod || !ast_test_flag(mod->info, AST_MODFLAG_GLOBAL_SYMBOLS)) {
 		return mod;
 	}
 
 	/* Close the module so we can reopen with correct flags. */
 	logged_dlclose(resource_in, mod->lib);
-	if (global_symbols_only) {
-		return MODULE_LOCAL_ONLY;
-	}
 
-	return load_dlopen(resource_in, so_ext, fn,
-		exports_globals ? RTLD_LAZY | RTLD_GLOBAL : RTLD_NOW | RTLD_LOCAL,
-		0);
+	return load_dlopen(resource_in, so_ext, fn, RTLD_NOW | RTLD_GLOBAL, 0);
 }
 
 int modules_shutdown(void)
@@ -742,6 +1150,7 @@ int ast_unload_resource(const char *reso
 		unload_dynamic_module(mod);
 		ast_test_suite_event_notify("MODULE_UNLOAD", "Message: %s", resource_name);
 		ast_update_use_count();
+		publish_unload_message(resource_name, "Success");
 	}
 
 	return res;
@@ -822,9 +1231,8 @@ static void module_load_helper(const cha
 	AST_DLLIST_UNLOCK(&module_list);
 }
 
-char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, int _type)
+char *ast_module_helper(const char *line, const char *word, int pos, int state, int rpos, enum ast_module_helper_type type)
 {
-	enum ast_module_helper_type type = _type;
 	struct ast_module *mod;
 	int which = 0;
 	int wordlen = strlen(word);
@@ -840,16 +1248,6 @@ char *ast_module_helper(const char *line
 		return NULL;
 	}
 
-	if (type == AST_MODULE_HELPER_RELOAD) {
-		int idx;
-
-		for (idx = 0; reload_classes[idx].name; idx++) {
-			if (!strncasecmp(word, reload_classes[idx].name, wordlen) && ++which > state) {
-				return ast_strdup(reload_classes[idx].name);
-			}
-		}
-	}
-
 	AST_DLLIST_LOCK(&module_list);
 	AST_DLLIST_TRAVERSE(&module_list, mod, entry) {
 		if (!module_matches_helper_type(mod, type)) {
@@ -932,29 +1330,30 @@ static void queue_reload_request(const c
 /*!
  * \since 12
  * \internal
- * \brief Publish a \ref stasis message regarding the reload result
+ * \brief Publish a \ref stasis message regarding the type.
  */
-static void publish_reload_message(const char *name, enum ast_module_reload_result result)
+static void publish_load_message_type(const char* type, const char *name, const char *status)
 {
 	RAII_VAR(struct stasis_message *, message, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_json_payload *, payload, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_json *, json_object, NULL, ast_json_unref);
 	RAII_VAR(struct ast_json *, event_object, NULL, ast_json_unref);
-	char res_buffer[8];
+
+	ast_assert(type != NULL);
+	ast_assert(!ast_strlen_zero(name));
+	ast_assert(!ast_strlen_zero(status));
 
 	if (!ast_manager_get_generic_type()) {
 		return;
 	}
 
-	snprintf(res_buffer, sizeof(res_buffer), "%u", result);
-	event_object = ast_json_pack("{s: s, s: s}",
-			"Module", S_OR(name, "All"),
-			"Status", res_buffer);
-	json_object = ast_json_pack("{s: s, s: i, s: o}",
-			"type", "Reload",
+	event_object = ast_json_pack("{s:s, s:s}",
+			"Module", name,
+			"Status", status);
+	json_object = ast_json_pack("{s:s, s:i, s:o}",
+			"type", type,
 			"class_type", EVENT_FLAG_SYSTEM,
 			"event", ast_json_ref(event_object));
-
 	if (!json_object) {
 		return;
 	}
@@ -972,61 +1371,92 @@ static void publish_reload_message(const
 	stasis_publish(ast_manager_get_topic(), message);
 }
 
-enum ast_module_reload_result ast_module_reload(const char *name)
+static const char* loadresult2str(enum ast_module_load_result result)
 {
-	struct ast_module *cur;
-	enum ast_module_reload_result res = AST_MODULE_RELOAD_NOT_FOUND;
 	int i;
-	size_t name_baselen = name ? resource_name_baselen(name) : 0;
-
-	/* If we aren't fully booted, we just pretend we reloaded but we queue this
-	   up to run once we are booted up. */
-	if (!modules_loaded) {
-		queue_reload_request(name);
-		res = AST_MODULE_RELOAD_QUEUED;
-		goto module_reload_exit;
+	for (i = 0; i < ARRAY_LEN(load_results); i++) {
+		if (load_results[i].result == result) {
+			return load_results[i].name;
+		}
 	}
 
-	if (ast_mutex_trylock(&reloadlock)) {
-		ast_verb(3, "The previous reload command didn't finish yet\n");
-		res = AST_MODULE_RELOAD_IN_PROGRESS;
-		goto module_reload_exit;
-	}
-	ast_sd_notify("RELOAD=1");
-	ast_lastreloadtime = ast_tvnow();
+	ast_log(LOG_WARNING, "Failed to find correct load result status. result %d\n", result);
+	return AST_MODULE_LOAD_UNKNOWN_STRING;
+}
+
+/*!
+ * \internal
+ * \brief Publish a \ref stasis message regarding the load result
+ */
+static void publish_load_message(const char *name, enum ast_module_load_result result)
+{
+	const char *status;
+
+	status = loadresult2str(result);
+
+	publish_load_message_type("Load", name, status);
+}
+
+/*!
+ * \internal
+ * \brief Publish a \ref stasis message regarding the unload result
+ */
+static void publish_unload_message(const char *name, const char* status)
+{
+	publish_load_message_type("Unload", name, status);
+}
+
+/*!
+ * \since 12
+ * \internal
+ * \brief Publish a \ref stasis message regarding the reload result
+ */
+static void publish_reload_message(const char *name, enum ast_module_reload_result result)
+{
+	char res_buffer[8];
+
+	snprintf(res_buffer, sizeof(res_buffer), "%u", result);
+	publish_load_message_type("Reload", S_OR(name, "All"), res_buffer);
+}
+
+enum ast_module_reload_result ast_module_reload(const char *name)
+{
+	struct ast_module *cur;
+	enum ast_module_reload_result res = AST_MODULE_RELOAD_NOT_FOUND;
+	size_t name_baselen = name ? resource_name_baselen(name) : 0;
+
+	/* If we aren't fully booted, we just pretend we reloaded but we queue this
+	   up to run once we are booted up. */
+	if (!modules_loaded) {
+		queue_reload_request(name);
+		res = AST_MODULE_RELOAD_QUEUED;
+		goto module_reload_exit;
+	}
+
+	if (ast_mutex_trylock(&reloadlock)) {
+		ast_verb(3, "The previous reload command didn't finish yet\n");
+		res = AST_MODULE_RELOAD_IN_PROGRESS;
+		goto module_reload_exit;
+	}
+	ast_sd_notify("RELOAD=1");
+	ast_lastreloadtime = ast_tvnow();
 
 	if (ast_opt_lock_confdir) {
 		int try;
-		int res;
-		for (try = 1, res = AST_LOCK_TIMEOUT; try < 6 && (res == AST_LOCK_TIMEOUT); try++) {
-			res = ast_lock_path(ast_config_AST_CONFIG_DIR);
-			if (res == AST_LOCK_TIMEOUT) {
+		int lockres;
+		for (try = 1, lockres = AST_LOCK_TIMEOUT; try < 6 && (lockres == AST_LOCK_TIMEOUT); try++) {
+			lockres = ast_lock_path(ast_config_AST_CONFIG_DIR);
+			if (lockres == AST_LOCK_TIMEOUT) {
 				ast_log(LOG_WARNING, "Failed to grab lock on %s, try %d\n", ast_config_AST_CONFIG_DIR, try);
 			}
 		}
-		if (res != AST_LOCK_SUCCESS) {
+		if (lockres != AST_LOCK_SUCCESS) {
 			ast_log(AST_LOG_WARNING, "Cannot grab lock on %s\n", ast_config_AST_CONFIG_DIR);
 			res = AST_MODULE_RELOAD_ERROR;
 			goto module_reload_done;
 		}
 	}
 
-	/* Call "predefined" reload here first */
-	for (i = 0; reload_classes[i].name; i++) {
-		if (!name || !strcasecmp(name, reload_classes[i].name)) {
-			if (reload_classes[i].reload_fn() == AST_MODULE_LOAD_SUCCESS) {
-				res = AST_MODULE_RELOAD_SUCCESS;
-			}
-		}
-	}
-
-	if (name && res == AST_MODULE_RELOAD_SUCCESS) {
-		if (ast_opt_lock_confdir) {
-			ast_unlock_path(ast_config_AST_CONFIG_DIR);
-		}
-		goto module_reload_done;
-	}
-
 	AST_DLLIST_LOCK(&module_list);
 	AST_DLLIST_TRAVERSE(&module_list, cur, entry) {
 		const struct ast_module_info *info = cur->info;
@@ -1057,6 +1487,8 @@ enum ast_module_reload_result ast_module
 		ast_verb(3, "Reloading module '%s' (%s)\n", cur->resource, info->description);
 		if (info->reload() == AST_MODULE_LOAD_SUCCESS) {
 			res = AST_MODULE_RELOAD_SUCCESS;
+		} else if (res == AST_MODULE_RELOAD_NOT_FOUND) {
+			res = AST_MODULE_RELOAD_ERROR;
 		}
 		if (name) {
 			break;
@@ -1079,24 +1511,24 @@ module_reload_exit:
 static unsigned int inspect_module(const struct ast_module *mod)
 {
 	if (!mod->info->description) {
-		ast_log(LOG_WARNING, "Module '%s' does not provide a description.\n", mod->resource);
+		module_load_error("Module '%s' does not provide a description.\n", mod->resource);
 		return 1;
 	}
 
 	if (!mod->info->key) {
-		ast_log(LOG_WARNING, "Module '%s' does not provide a license key.\n", mod->resource);
+		module_load_error("Module '%s' does not provide a license key.\n", mod->resource);
 		return 1;
 	}
 
 	if (verify_key((unsigned char *) mod->info->key)) {
-		ast_log(LOG_WARNING, "Module '%s' did not provide a valid license key.\n", mod->resource);
+		module_load_error("Module '%s' did not provide a valid license key.\n", mod->resource);
 		return 1;
 	}
 
 	if (!ast_strlen_zero(mod->info->buildopt_sum) &&
 	    strcmp(buildopt_sum, mod->info->buildopt_sum)) {
-		ast_log(LOG_WARNING, "Module '%s' was not compiled with the same compile-time options as this version of Asterisk.\n", mod->resource);
-		ast_log(LOG_WARNING, "Module '%s' will not be initialized as it may cause instability.\n", mod->resource);
+		module_load_error("Module '%s' was not compiled with the same compile-time options as this version of Asterisk.\n", mod->resource);
+		module_load_error("Module '%s' will not be initialized as it may cause instability.\n", mod->resource);
 		return 1;
 	}
 
@@ -1113,7 +1545,26 @@ static enum ast_module_load_result start
 	}
 
 	if (!mod->info->load) {
-		return AST_MODULE_LOAD_FAILURE;
+		mod->flags.declined = 1;
+
+		return mod->flags.required ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (module_deps_reference(mod, NULL)) {
+		struct module_vector missing;
+		int i;
+
+		AST_VECTOR_INIT(&missing, 0);
+		if (module_deps_missing_recursive(mod, &missing)) {
+			module_load_error("%s has one or more unknown dependencies.\n", mod->info->name);
+		}
+		for (i = 0; i < AST_VECTOR_SIZE(&missing); i++) {
+			module_load_error("%s loaded before dependency %s!\n", mod->info->name,
+				AST_VECTOR_GET(&missing, i)->info->name);
+		}
+		AST_VECTOR_FREE(&missing);
+
+		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (!ast_fully_booted) {
@@ -1130,13 +1581,22 @@ static enum ast_module_load_result start
 		}
 
 		mod->flags.running = 1;
+		if (mod->flags.builtin) {
+			/* Built-in modules cannot be unloaded. */
+			ast_module_shutdown_ref(mod);
+		}
 
 		ast_update_use_count();
 		break;
 	case AST_MODULE_LOAD_DECLINE:
 		mod->flags.declined = 1;
+		if (mod->flags.required) {
+			res = AST_MODULE_LOAD_FAILURE;
+		}
 		break;
 	case AST_MODULE_LOAD_FAILURE:
+		mod->flags.declined = 1;
+		break;
 	case AST_MODULE_LOAD_SKIP: /* modules should never return this value */
 	case AST_MODULE_LOAD_PRIORITY:
 		break;
@@ -1160,7 +1620,8 @@ static enum ast_module_load_result start
  *
  *  If the module_vector is not provided, the module's load function will be executed
  *  immediately */
-static enum ast_module_load_result load_resource(const char *resource_name, unsigned int global_symbols_only, unsigned int suppress_logging, struct module_vector *resource_heap, int required)
+static enum ast_module_load_result load_resource(const char *resource_name, unsigned int suppress_logging,
+	struct module_vector *module_priorities, int required, int preload)
 {
 	struct ast_module *mod;
 	enum ast_module_load_result res = AST_MODULE_LOAD_SUCCESS;
@@ -1170,29 +1631,28 @@ static enum ast_module_load_result load_
 			ast_log(LOG_WARNING, "Module '%s' already loaded and running.\n", resource_name);
 			return AST_MODULE_LOAD_DECLINE;
 		}
-		if (global_symbols_only && !ast_test_flag(mod->info, AST_MODFLAG_GLOBAL_SYMBOLS))
-			return AST_MODULE_LOAD_SKIP;
 	} else {
-		mod = load_dynamic_module(resource_name, global_symbols_only, suppress_logging);
-		if (mod == MODULE_LOCAL_ONLY) {
-				return AST_MODULE_LOAD_SKIP;
-		}
+		mod = load_dynamic_module(resource_name, suppress_logging);
 		if (!mod) {
-			if (!global_symbols_only) {
-				ast_log(LOG_WARNING, "Module '%s' could not be loaded.\n", resource_name);
-			}
 			return required ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_DECLINE;
 		}
+
+		if (module_post_register(mod)) {
+			goto prestart_error;
+		}
 	}
 
+	mod->flags.required |= required;
+	mod->flags.preload |= preload;
+
 	if (inspect_module(mod)) {
 		goto prestart_error;
 	}
 
 	mod->flags.declined = 0;
 
-	if (resource_heap) {
-		if (AST_VECTOR_ADD_SORTED(resource_heap, mod, module_vector_cmp)) {
+	if (module_priorities) {
+		if (AST_VECTOR_ADD_SORTED(module_priorities, mod, module_vector_cmp)) {
 			goto prestart_error;
 		}
 		res = AST_MODULE_LOAD_PRIORITY;
@@ -1200,19 +1660,27 @@ static enum ast_module_load_result load_
 		res = start_resource(mod);
 	}
 
+	if (ast_fully_booted && !ast_shutdown_final()) {
+		publish_load_message(resource_name, res);
+	}
+
 	return res;
 
 prestart_error:
-	ast_log(LOG_WARNING, "Module '%s' could not be loaded.\n", resource_name);
+	module_load_error("Module '%s' could not be loaded.\n", resource_name);
 	unload_dynamic_module(mod);
-	return required ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_DECLINE;
+	res = required ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_DECLINE;
+	if (ast_fully_booted && !ast_shutdown_final()) {
+		publish_load_message(resource_name, res);
+	}
+	return res;
 }
 
 int ast_load_resource(const char *resource_name)
 {
 	int res;
 	AST_DLLIST_LOCK(&module_list);
-	res = load_resource(resource_name, 0, 0, NULL, 0);
+	res = load_resource(resource_name, 0, NULL, 0, 0);
 	if (!res) {
 		ast_test_suite_event_notify("MODULE_LOAD", "Message: %s", resource_name);
 	}
@@ -1224,12 +1692,14 @@ int ast_load_resource(const char *resour
 struct load_order_entry {
 	char *resource;
 	int required;
+	int preload;
+	int builtin;
 	AST_LIST_ENTRY(load_order_entry) entry;
 };
 
 AST_LIST_HEAD_NOLOCK(load_order, load_order_entry);
 
-static struct load_order_entry *add_to_load_order(const char *resource, struct load_order *load_order, int required)
+static struct load_order_entry *add_to_load_order(const char *resource, struct load_order *load_order, int required, int preload, int builtin)
 {
 	struct load_order_entry *order;
 	size_t resource_baselen = resource_name_baselen(resource);
@@ -1239,12 +1709,15 @@ static struct load_order_entry *add_to_l
 			/* Make sure we have the proper setting for the required field
 			   (we might have both load= and required= lines in modules.conf) */
 			order->required |= required;
-			return NULL;
+			order->preload |= preload;
+			return order;
 		}
 	}
 
-	if (!(order = ast_calloc(1, sizeof(*order))))
+	order = ast_calloc(1, sizeof(*order));
+	if (!order) {
 		return NULL;
+	}
 
 	order->resource = ast_strdup(resource);
 	if (!order->resource) {
@@ -1253,6 +1726,8 @@ static struct load_order_entry *add_to_l
 		return NULL;
 	}
 	order->required = required;
+	order->preload = preload;
+	order->builtin = builtin;
 	AST_LIST_INSERT_TAIL(load_order, order, entry);
 
 	return order;
@@ -1260,187 +1735,400 @@ static struct load_order_entry *add_to_l
 
 AST_LIST_HEAD_NOLOCK(load_retries, load_order_entry);
 
+static enum ast_module_load_result start_resource_attempt(struct ast_module *mod, int *count)
+{
+	enum ast_module_load_result lres;
+
+	/* Try to grab required references. */
+	if (module_deps_reference(mod, NULL)) {
+		/* We're likely to retry so not an error. */
+		ast_debug(1, "Module %s is missing dependencies\n", mod->resource);
+		return AST_MODULE_LOAD_SKIP;
+	}
+
+	lres = start_resource(mod);
+	ast_debug(3, "START: %-46s[%d] %d\n",
+		mod->resource,
+		ast_test_flag(mod->info, AST_MODFLAG_LOAD_ORDER) ? mod->info->load_pri : AST_MODPRI_DEFAULT,
+		lres);
+
+	if (lres == AST_MODULE_LOAD_SUCCESS) {
+		(*count)++;
+	} else if (lres == AST_MODULE_LOAD_FAILURE) {
+		module_load_error("*** Failed to load %smodule %s\n",
+			mod->flags.required ? "required " : "",
+			mod->resource);
+	}
+
+	return lres;
+}
+
+static int resource_list_recursive_decline(struct module_vector *resources, struct ast_module *mod,
+	struct ast_str **printmissing)
+{
+	struct module_vector missingdeps;
+	struct ast_vector_const_string localdeps;
+	int i = 0;
+	int res = -1;
+
+	mod->flags.declined = 1;
+	if (mod->flags.required) {
+		module_load_error("Required module %s declined to load.\n", ast_module_name(mod));
+
+		return -2;
+	}
+
+	module_load_error("%s declined to load.\n", ast_module_name(mod));
+
+	if (!*printmissing) {
+		*printmissing = ast_str_create(64);
+		if (!*printmissing) {
+			return -1;
+		}
+	} else {
+		ast_str_reset(*printmissing);
+	}
+
+	AST_VECTOR_INIT(&missingdeps, 0);
+	AST_VECTOR_INIT(&localdeps, 0);
+
+	/* Decline everything that depends on 'mod' from resources so we can
+	 * print a concise list. */
+	while (res != -2 && i < AST_VECTOR_SIZE(resources)) {
+		struct ast_module *dep = AST_VECTOR_GET(resources, i);
+		i++;
+
+		AST_VECTOR_RESET(&missingdeps, AST_VECTOR_ELEM_CLEANUP_NOOP);
+		if (dep->flags.declined || module_deps_missing_recursive(dep, &missingdeps)) {
+			continue;
+		}
+
+		if (AST_VECTOR_GET_CMP(&missingdeps, mod, AST_VECTOR_ELEM_DEFAULT_CMP)) {
+			dep->flags.declined = 1;
+			if (dep->flags.required) {
+				module_load_error("Cannot load required module %s that depends on %s\n",
+					ast_module_name(dep), ast_module_name(mod));
+				res = -2;
+			} else {
+				AST_VECTOR_APPEND(&localdeps, ast_module_name(dep));
+			}
+		}
+	}
+	AST_VECTOR_FREE(&missingdeps);
+
+	if (res != -2 && AST_VECTOR_SIZE(&localdeps)) {
+		AST_VECTOR_CALLBACK_VOID(&localdeps, STR_APPEND_TEXT, printmissing);
+		module_load_error("Declined modules which depend on %s: %s\n",
+			ast_module_name(mod), ast_str_buffer(*printmissing));
+	}
+	AST_VECTOR_FREE(&localdeps);
+
+	return res;
+}
+
+static int start_resource_list(struct module_vector *resources, int *mod_count)
+{
+	struct module_vector missingdeps;
+	int res = 0;
+	struct ast_str *printmissing = NULL;
+
+	AST_VECTOR_INIT(&missingdeps, 0);
+	while (res != -2 && AST_VECTOR_SIZE(resources)) {
+		struct ast_module *mod = AST_VECTOR_REMOVE(resources, 0, 1);
+		enum ast_module_load_result lres;
+
+		if (mod->flags.declined) {
+			ast_debug(1, "%s is already declined, skipping\n", ast_module_name(mod));
+			continue;
+		}
+
+retry_load:
+		lres = start_resource_attempt(mod, mod_count);
+		if (lres == AST_MODULE_LOAD_SUCCESS) {
+			/* No missing dependencies, successful. */
+			continue;
+		}
+
+		if (lres == AST_MODULE_LOAD_FAILURE) {
+			res = -2;
+			break;
+		}
+
+		if (lres == AST_MODULE_LOAD_DECLINE) {
+			res = resource_list_recursive_decline(resources, mod, &printmissing);
+			continue;
+		}
+
+		if (module_deps_missing_recursive(mod, &missingdeps)) {
+			AST_VECTOR_RESET(&missingdeps, AST_VECTOR_ELEM_CLEANUP_NOOP);
+			module_load_error("Failed to resolve dependencies for %s\n", ast_module_name(mod));
+			res = resource_list_recursive_decline(resources, mod, &printmissing);
+			continue;
+		}
+
+		if (!AST_VECTOR_SIZE(&missingdeps)) {
+			module_load_error("%s load function returned an invalid result. "
+				"This is a bug in the module.\n", ast_module_name(mod));
+			/* Dependencies were met but the module failed to start and the result
+			 * code was not AST_MODULE_LOAD_FAILURE or AST_MODULE_LOAD_DECLINE. */
+			res = resource_list_recursive_decline(resources, mod, &printmissing);
+			continue;
+		}
+
+		ast_debug(1, "%s has %d dependencies\n",
+			ast_module_name(mod), (int)AST_VECTOR_SIZE(&missingdeps));
+		while (AST_VECTOR_SIZE(&missingdeps)) {
+			int didwork = 0;
+			int i = 0;
+
+			while (i < AST_VECTOR_SIZE(&missingdeps)) {
+				struct ast_module *dep = AST_VECTOR_GET(&missingdeps, i);
+
+				if (dep->flags.declined) {
+					ast_debug(1, "%s tried to start %s but it's already declined\n",
+						ast_module_name(mod), ast_module_name(dep));
+					i++;
+					continue;
+				}
+
+				ast_debug(1, "%s trying to start %s\n", ast_module_name(mod), ast_module_name(dep));
+				lres = start_resource_attempt(dep, mod_count);
+				if (lres == AST_MODULE_LOAD_SUCCESS) {
+					ast_debug(1, "%s started %s\n", ast_module_name(mod), ast_module_name(dep));
+					AST_VECTOR_REMOVE(&missingdeps, i, 1);
+					AST_VECTOR_REMOVE_CMP_ORDERED(resources, dep,
+						AST_VECTOR_ELEM_DEFAULT_CMP, AST_VECTOR_ELEM_CLEANUP_NOOP);
+					didwork++;
+					continue;
+				}
+
+				if (lres == AST_MODULE_LOAD_FAILURE) {
+					module_load_error("Failed to load %s.\n", ast_module_name(dep));
+					res = -2;
+					goto exitpoint;
+				}
+
+				ast_debug(1, "%s failed to start %s\n", ast_module_name(mod), ast_module_name(dep));
+				i++;
+			}
+
+			if (!didwork) {
+				break;
+			}
+		}
+
+		if (AST_VECTOR_SIZE(&missingdeps)) {
+			if (!printmissing) {
+				printmissing = ast_str_create(64);
+			} else {
+				ast_str_reset(printmissing);
+			}
+
+			if (printmissing) {
+				struct ast_vector_const_string localdeps;
+
+				AST_VECTOR_INIT(&localdeps, 0);
+				module_deps_reference(mod, &localdeps);
+				AST_VECTOR_CALLBACK_VOID(&localdeps, STR_APPEND_TEXT, &printmissing);
+				AST_VECTOR_FREE(&localdeps);
+			}
+
+			module_load_error("Failed to load %s due to dependencies: %s.\n",
+				ast_module_name(mod),
+				printmissing ? ast_str_buffer(printmissing) : "allocation failure creating list");
+			res = resource_list_recursive_decline(resources, mod, &printmissing);
+
+			AST_VECTOR_RESET(&missingdeps, AST_VECTOR_ELEM_CLEANUP_NOOP);
+
+			continue;
+		}
+
+		/* If we're here it means that we started with missingdeps and they're all loaded
+		 * now.  It's impossible to reach this point a second time for the same module. */
+		goto retry_load;
+	}
+
+exitpoint:
+	ast_free(printmissing);
+	AST_VECTOR_FREE(&missingdeps);
+
+	return res;
+}
+
 /*! loads modules in order by load_pri, updates mod_count
 	\return -1 on failure to load module, -2 on failure to load required module, otherwise 0
 */
-static int load_resource_list(struct load_order *load_order, unsigned int global_symbols, int *mod_count)
+static int load_resource_list(struct load_order *load_order, int *mod_count)
 {
-	struct module_vector resource_heap;
+	struct module_vector module_priorities;
 	struct load_order_entry *order;
-	struct load_retries load_retries;
+	int attempt = 0;
 	int count = 0;
 	int res = 0;
-	int i = 0;
-#define LOAD_RETRIES 4
-
-	AST_LIST_HEAD_INIT_NOLOCK(&load_retries);
+	int didwork;
+	int lasttry = 0;
 
-	if (AST_VECTOR_INIT(&resource_heap, 500)) {
+	if (AST_VECTOR_INIT(&module_priorities, 500)) {
 		ast_log(LOG_ERROR, "Failed to initialize module loader.\n");
 
 		return -1;
 	}
 
-	/* first, add find and add modules to heap */
-	AST_LIST_TRAVERSE_SAFE_BEGIN(load_order, order, entry) {
-		enum ast_module_load_result lres;
-
-		/* Suppress log messages unless this is the last pass */
-		lres = load_resource(order->resource, global_symbols, 1, &resource_heap, order->required);
-		ast_debug(3, "PASS 0: %-46s %d %d\n", order->resource, lres, global_symbols);
-		switch (lres) {
-		case AST_MODULE_LOAD_SUCCESS:
-			/* We're supplying a heap so SUCCESS isn't possible but we still have to test for it. */
-			break;
-		case AST_MODULE_LOAD_FAILURE:
-		case AST_MODULE_LOAD_DECLINE:
-			/*
-			 * DECLINE or FAILURE means there was an issue with dlopen or module_register
-			 * which might be retryable.  LOAD_FAILURE only happens for required modules
-			 * but we're still going to retry.  We need to remove the entry from the
-			 * load_order list and add it to the load_retries list.
-			 */
-			AST_LIST_REMOVE_CURRENT(entry);
-			AST_LIST_INSERT_TAIL(&load_retries, order, entry);
-			break;
-		case AST_MODULE_LOAD_SKIP:
-			/*
-			 * SKIP means that dlopen worked but global_symbols was set and this module doesn't qualify.
-			 * Leave it in load_order for the next call of load_resource_list.
-			 */
-			break;
-		case AST_MODULE_LOAD_PRIORITY:
-			/* load_resource worked and the module was added to the priority vector */
-			AST_LIST_REMOVE_CURRENT(entry);
-			ast_free(order->resource);
-			ast_free(order);
-			break;
-		}
-	}
-	AST_LIST_TRAVERSE_SAFE_END;
+	while (res != -2) {
+		didwork = 0;
 
-	/* Retry the failures until the list is empty or we reach LOAD_RETRIES */
-	for (i = 0; !AST_LIST_EMPTY(&load_retries) && i < LOAD_RETRIES; i++) {
-		AST_LIST_TRAVERSE_SAFE_BEGIN(&load_retries, order, entry) {
+		AST_LIST_TRAVERSE_SAFE_BEGIN(load_order, order, entry) {
 			enum ast_module_load_result lres;
 
 			/* Suppress log messages unless this is the last pass */
-			lres = load_resource(order->resource, global_symbols, (i < LOAD_RETRIES - 1), &resource_heap, order->required);
-			ast_debug(3, "PASS %d %-46s %d %d\n", i + 1, order->resource, lres, global_symbols);
+			lres = load_resource(order->resource, !lasttry, &module_priorities, order->required, order->preload);
+			ast_debug(3, "PASS %d: %-46s %d\n", attempt, order->resource, lres);
 			switch (lres) {
-			/* These are all retryable. */
 			case AST_MODULE_LOAD_SUCCESS:
+			case AST_MODULE_LOAD_SKIP:
+				/* We're supplying module_priorities so SUCCESS isn't possible but we
+				 * still have to test for it.  SKIP is only used when we try to start a
+				 * module that is missing dependencies. */
+				break;
 			case AST_MODULE_LOAD_DECLINE:
+				res = -1;
 				break;
 			case AST_MODULE_LOAD_FAILURE:
 				/* LOAD_FAILURE only happens for required modules */
-				if (i == LOAD_RETRIES - 1) {
-					/* This was the last chance to load a required module*/
-					ast_log(LOG_ERROR, "*** Failed to load module %s - Required\n", order->resource);
+				if (lasttry) {
+					/* This run is just to print errors. */
+					module_load_error("*** Failed to load module %s - Required\n", order->resource);
 					fprintf(stderr, "*** Failed to load module %s - Required\n", order->resource);
 					res =  -2;
-					goto done;
 				}
-				break;;
-			case AST_MODULE_LOAD_SKIP:
-				/*
-				 * SKIP means that dlopen worked but global_symbols was set and this module
-				 * doesn't qualify.  Put it back in load_order for the next call of
-				 * load_resource_list.
-				 */
-				AST_LIST_REMOVE_CURRENT(entry);
-				AST_LIST_INSERT_TAIL(load_order, order, entry);
 				break;
 			case AST_MODULE_LOAD_PRIORITY:
-				/* load_resource worked and the module was added to the priority heap */
+				/* load_resource worked and the module was added to module_priorities */
 				AST_LIST_REMOVE_CURRENT(entry);
 				ast_free(order->resource);
 				ast_free(order);
+				didwork = 1;
 				break;
 			}
 		}
 		AST_LIST_TRAVERSE_SAFE_END;
-	}
-
-	/* second remove modules from heap sorted by priority */
-	for (i = 0; i < AST_VECTOR_SIZE(&resource_heap); i++) {
-		struct ast_module *mod = AST_VECTOR_GET(&resource_heap, i);
-		enum ast_module_load_result lres;
 
-		lres = start_resource(mod);
-		ast_debug(3, "START: %-46s %d %d\n", mod->resource, lres, global_symbols);
-		switch (lres) {
-		case AST_MODULE_LOAD_SUCCESS:
-			count++;
-		case AST_MODULE_LOAD_DECLINE:
-			break;
-		case AST_MODULE_LOAD_FAILURE:
-			ast_log(LOG_ERROR, "*** Failed to load module %s\n", mod->resource);
-			res = -1;
-			goto done;
-		case AST_MODULE_LOAD_SKIP:
-		case AST_MODULE_LOAD_PRIORITY:
-			break;
+		if (!didwork) {
+			if (lasttry) {
+				break;
+			}
+			/* We know the next try is going to fail, it's only being performed
+			 * so we can print errors. */
+			lasttry = 1;
 		}
+		attempt++;
 	}
 
-done:
-
-	while ((order = AST_LIST_REMOVE_HEAD(&load_retries, entry))) {
-		ast_free(order->resource);
-		ast_free(order);
+	if (res != -2) {
+		res = start_resource_list(&module_priorities, &count);
 	}
 
 	if (mod_count) {
 		*mod_count += count;
 	}
-	AST_VECTOR_FREE(&resource_heap);
+	AST_VECTOR_FREE(&module_priorities);
 
 	return res;
 }
 
-int load_modules(unsigned int preload_only)
+static int loader_builtin_init(struct load_order *load_order)
 {
-	struct ast_config *cfg;
-	struct load_order_entry *order;
-	struct ast_variable *v;
-	unsigned int load_count;
-	struct load_order load_order;
-	int res = 0;
-	struct ast_flags config_flags = { 0 };
-	int modulecount = 0;
-	struct dirent *dirent;
-	DIR *dir;
+	struct ast_module *mod;
 
-	ast_verb(1, "Asterisk Dynamic Loader Starting:\n");
+	/*
+	 * All built-in modules have registered the first time, now it's time to complete
+	 * the registration and add them to the priority list.
+	 */
+	loader_ready = 1;
 
-	AST_LIST_HEAD_INIT_NOLOCK(&load_order);
+	while ((resource_being_loaded = AST_DLLIST_REMOVE_HEAD(&builtin_module_list, entry))) {
+		/* ast_module_register doesn't finish when first run by built-in modules. */
+		ast_module_register(resource_being_loaded->info);
+	}
 
-	AST_DLLIST_LOCK(&module_list);
+	/* Add all built-in modules to the load order. */
+	AST_DLLIST_TRAVERSE(&module_list, mod, entry) {
+		if (!mod->flags.builtin) {
+			continue;
+		}
+
+		/* Parse dependendencies from mod->info. */
+		if (module_post_register(mod)) {
+			return -1;
+		}
+
+		/* Built-in modules are not preloaded, most have an early load priority. */
+		if (!add_to_load_order(mod->resource, load_order, 0, 0, 1)) {
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+static int loader_config_init(struct load_order *load_order)
+{
+	int res = -1;
+	struct load_order_entry *order;
+	struct ast_config *cfg;
+	struct ast_variable *v;
+	struct ast_flags config_flags = { 0 };
 
 	cfg = ast_config_load2(AST_MODULE_CONFIG, "" /* core, can't reload */, config_flags);
 	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEINVALID) {
-		ast_log(LOG_WARNING, "No '%s' found, no modules will be loaded.\n", AST_MODULE_CONFIG);
-		goto done;
+		ast_log(LOG_WARNING, "'%s' invalid or missing.\n", AST_MODULE_CONFIG);
+
+		return -1;
 	}
 
 	/* first, find all the modules we have been explicitly requested to load */
 	for (v = ast_variable_browse(cfg, "modules"); v; v = v->next) {
-		if (!strcasecmp(v->name, preload_only ? "preload" : "load")) {
-			add_to_load_order(v->value, &load_order, 0);
+		int required;
+		int preload = 0;
+
+		if (!strncasecmp(v->name, "preload", strlen("preload"))) {
+			preload = 1;
+			if (!strcasecmp(v->name, "preload")) {
+				required = 0;
+			} else if (!strcasecmp(v->name, "preload-require")) {
+				required = 1;
+			} else {
+				ast_log(LOG_ERROR, "Unknown configuration option '%s'", v->name);
+				goto done;
+			}
+		} else if (!strcasecmp(v->name, "load")) {
+			required = 0;
+		} else if (!strcasecmp(v->name, "require")) {
+			required = 1;
+		} else if (!strcasecmp(v->name, "noload") || !strcasecmp(v->name, "autoload")) {
+			continue;
+		} else {
+			ast_log(LOG_ERROR, "Unknown configuration option '%s'", v->name);
+			goto done;
 		}
-		if (!strcasecmp(v->name, preload_only ? "preload-require" : "require")) {
-			/* Add the module to the list and make sure it's required */
-			add_to_load_order(v->value, &load_order, 1);
+
+		if (required) {
 			ast_debug(2, "Adding module to required list: %s (%s)\n", v->value, v->name);
 		}
+
+		if (!add_to_load_order(v->value, load_order, required, preload, 0)) {
+			goto done;
+		}
 	}
 
 	/* check if 'autoload' is on */
-	if (!preload_only && ast_true(ast_variable_retrieve(cfg, "modules", "autoload"))) {
+	if (ast_true(ast_variable_retrieve(cfg, "modules", "autoload"))) {
 		/* if we are allowed to load dynamic modules, scan the directory for
 		   for all available modules and add them as well */
-		if ((dir = opendir(ast_config_AST_MODULE_DIR))) {
+		DIR *dir = opendir(ast_config_AST_MODULE_DIR);
+		struct dirent *dirent;
+
+		if (dir) {
 			while ((dirent = readdir(dir))) {
 				int ld = strlen(dirent->d_name);
 
@@ -1457,14 +2145,16 @@ int load_modules(unsigned int preload_on
 				if (find_resource(dirent->d_name, 0))
 					continue;
 
-				add_to_load_order(dirent->d_name, &load_order, 0);
+				if (!add_to_load_order(dirent->d_name, load_order, 0, 0, 0)) {
+					closedir(dir);
+					goto done;
+				}
 			}
 
 			closedir(dir);
 		} else {
-			if (!ast_opt_quiet)
-				ast_log(LOG_WARNING, "Unable to open modules directory '%s'.\n",
-					ast_config_AST_MODULE_DIR);
+			ast_log(LOG_ERROR, "Unable to open modules directory '%s'.\n", ast_config_AST_MODULE_DIR);
+			goto done;
 		}
 	}
 
@@ -1478,8 +2168,18 @@ int load_modules(unsigned int preload_on
 		}
 
 		baselen = resource_name_baselen(v->value);
-		AST_LIST_TRAVERSE_SAFE_BEGIN(&load_order, order, entry) {
+		AST_LIST_TRAVERSE_SAFE_BEGIN(load_order, order, entry) {
 			if (!resource_name_match(v->value, baselen, order->resource)) {
+				if (order->builtin) {
+					ast_log(LOG_ERROR, "%s is a built-in module, you cannot specify 'noload'.\n", v->value);
+					goto done;
+				}
+
+				if (order->required) {
+					ast_log(LOG_ERROR, "%s is configured with '%s' and 'noload', this is impossible.\n",
+						v->value, order->preload ? "preload-require" : "require");
+					goto done;
+				}
 				AST_LIST_REMOVE_CURRENT(entry);
 				ast_free(order->resource);
 				ast_free(order);
@@ -1488,10 +2188,40 @@ int load_modules(unsigned int preload_on
 		AST_LIST_TRAVERSE_SAFE_END;
 	}
 
-	/* we are done with the config now, all the information we need is in the
-	   load_order list */
+	res = 0;
+done:
 	ast_config_destroy(cfg);
 
+	return res;
+}
+
+int load_modules(void)
+{
+	struct load_order_entry *order;
+	unsigned int load_count;
+	struct load_order load_order;
+	int res = 0;
+	int modulecount = 0;
+	int i;
+
+	ast_verb(1, "Asterisk Dynamic Loader Starting:\n");
+
+	AST_LIST_HEAD_INIT_NOLOCK(&load_order);
+	AST_DLLIST_LOCK(&module_list);
+
+	AST_VECTOR_INIT(&startup_errors, 0);
+	startup_error_builder = ast_str_create(64);
+
+	res = loader_builtin_init(&load_order);
+	if (res) {
+		goto done;
+	}
+
+	res = loader_config_init(&load_order);
+	if (res) {
+		goto done;
+	}
+
 	load_count = 0;
 	AST_LIST_TRAVERSE(&load_order, order, entry)
 		load_count++;
@@ -1499,14 +2229,10 @@ int load_modules(unsigned int preload_on
 	if (load_count)
 		ast_log(LOG_NOTICE, "%u modules will be loaded.\n", load_count);
 
-	/* first, load only modules that provide global symbols */
-	if ((res = load_resource_list(&load_order, 1, &modulecount)) < 0) {
-		goto done;
-	}
-
-	/* now load everything else */
-	if ((res = load_resource_list(&load_order, 0, &modulecount)) < 0) {
-		goto done;
+	res = load_resource_list(&load_order, &modulecount);
+	if (res == -1) {
+		ast_log(LOG_WARNING, "Some non-required modules failed to load.\n");
+		res = 0;
 	}
 
 done:
@@ -1516,6 +2242,18 @@ done:
 	}
 
 	AST_DLLIST_UNLOCK(&module_list);
+
+	for (i = 0; i < AST_VECTOR_SIZE(&startup_errors); i++) {
+		char *str = AST_VECTOR_GET(&startup_errors, i);
+
+		ast_log(LOG_ERROR, "%s", str);
+		ast_free(str);
+	}
+	AST_VECTOR_FREE(&startup_errors);
+
+	ast_free(startup_error_builder);
+	startup_error_builder = NULL;
+
 	return res;
 }
 
@@ -1695,9 +2433,9 @@ struct ast_module *__ast_module_ref(stru
 		return NULL;
 	}
 
-#ifdef REF_DEBUG
-	__ao2_ref_debug(mod->ref_debug, +1, "", file, line, func);
-#endif
+	if (mod->ref_debug) {
+		__ao2_ref(mod->ref_debug, +1, "", file, line, func);
+	}
 
 	ast_atomic_fetchadd_int(&mod->usecount, +1);
 	ast_update_use_count();
@@ -1705,6 +2443,16 @@ struct ast_module *__ast_module_ref(stru
 	return mod;
 }
 
+struct ast_module *__ast_module_running_ref(struct ast_module *mod,
+	const char *file, int line, const char *func)
+{
+	if (!mod || !mod->flags.running) {
+		return NULL;
+	}
+
+	return __ast_module_ref(mod, file, line, func);
+}
+
 void __ast_module_shutdown_ref(struct ast_module *mod, const char *file, int line, const char *func)
 {
 	if (!mod || mod->flags.keepuntilshutdown) {
@@ -1721,9 +2469,9 @@ void __ast_module_unref(struct ast_modul
 		return;
 	}
 
-#ifdef REF_DEBUG
-	__ao2_ref_debug(mod->ref_debug, -1, "", file, line, func);
-#endif
+	if (mod->ref_debug) {
+		__ao2_ref(mod->ref_debug, -1, "", file, line, func);
+	}
 
 	ast_atomic_fetchadd_int(&mod->usecount, -1);
 	ast_update_use_count();
@@ -1740,22 +2488,3 @@ const char *ast_module_support_level_to_
 {
 	return support_level_map[support_level];
 }
-
-
-
-/* The following exists for ABI compatibility */
-#undef ast_module_ref
-#undef ast_module_unref
-
-struct ast_module *ast_module_ref(struct ast_module *mod);
-void ast_module_unref(struct ast_module *mod);
-
-struct ast_module *ast_module_ref(struct ast_module *mod)
-{
-	return __ast_module_ref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
-void ast_module_unref(struct ast_module *mod)
-{
-	__ast_module_unref(mod, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
diff -pruN 1:13.23.1~dfsg-2/main/lock.c 1:16.1.1~dfsg-1/main/lock.c
--- 1:13.23.1~dfsg-2/main/lock.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/lock.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,7 +27,14 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#ifdef HAVE_MTX_PROFILE
+/* profile mutex */
+static int mtx_prof = -1;
+static void __attribute__((constructor)) __mtx_init(void)
+{
+	mtx_prof = ast_add_profile("mtx_lock_" __FILE__, 0);
+}
+#endif
 
 #include "asterisk/utils.h"
 #include "asterisk/lock.h"
@@ -46,6 +53,17 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #undef pthread_cond_wait
 #undef pthread_cond_timedwait
 
+#if defined(DEBUG_THREADS)
+#define log_mutex_error(canlog, ...) \
+	do { \
+		if (canlog) { \
+			ast_log(LOG_ERROR, __VA_ARGS__); \
+		} else { \
+			fprintf(stderr, __VA_ARGS__); \
+		} \
+	} while (0)
+#endif
+
 #if defined(DEBUG_THREADS) && defined(HAVE_BKTR)
 static void __dump_backtrace(struct ast_bt *bt, int canlog)
 {
@@ -55,7 +73,7 @@ static void __dump_backtrace(struct ast_
 	strings = backtrace_symbols(bt->addresses, bt->num_frames);
 
 	for (i = 0; i < bt->num_frames; i++) {
-		__ast_mutex_logger("%s\n", strings[i]);
+		log_mutex_error(canlog, "%s\n", strings[i]);
 	}
 
 	ast_std_free(strings);
@@ -65,21 +83,16 @@ static void __dump_backtrace(struct ast_
 #ifdef DEBUG_THREADS
 AST_MUTEX_DEFINE_STATIC(reentrancy_lock);
 
-static inline struct ast_lock_track *ast_get_reentrancy(struct ast_lock_track **plt)
+static inline struct ast_lock_track *ast_get_reentrancy(struct ast_lock_track **plt,
+	struct ast_lock_track_flags *flags, int no_setup)
 {
 	pthread_mutexattr_t reentr_attr;
 	struct ast_lock_track *lt;
 
-	/* It's a bit painful to lock a global mutex for every access to the
-	 * reentrancy structure, but it's necessary to ensure that we don't
-	 * double-allocate the structure or double-initialize the reentr_mutex.
-	 *
-	 * If you'd like to replace this with a double-checked lock, be sure to
-	 * properly volatile-ize everything to avoid optimizer bugs.
-	 *
-	 * We also have to use the underlying pthread calls for manipulating
-	 * the mutex, because this is called from the Asterisk mutex code.
-	 */
+	if (!flags->tracking || flags->setup) {
+		return *plt;
+	}
+
 	pthread_mutex_lock(&reentrancy_lock.mutex);
 
 	if (*plt) {
@@ -87,6 +100,11 @@ static inline struct ast_lock_track *ast
 		return *plt;
 	}
 
+	if (no_setup) {
+		pthread_mutex_unlock(&reentrancy_lock.mutex);
+		return NULL;
+	}
+
 	lt = *plt = ast_std_calloc(1, sizeof(*lt));
 	if (!lt) {
 		fprintf(stderr, "%s: Failed to allocate lock tracking\n", __func__);
@@ -103,6 +121,7 @@ static inline struct ast_lock_track *ast
 	pthread_mutex_init(&lt->reentr_mutex, &reentr_attr);
 	pthread_mutexattr_destroy(&reentr_attr);
 
+	flags->setup = 1;
 	pthread_mutex_unlock(&reentrancy_lock.mutex);
 	return lt;
 }
@@ -133,7 +152,7 @@ int __ast_pthread_mutex_init(int trackin
 	if ((t->mutex) != ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
 		int canlog = tracking && strcmp(filename, "logger.c");
 
-		__ast_mutex_logger("%s line %d (%s): NOTICE: mutex '%s' is already initialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): NOTICE: mutex '%s' is already initialized.\n",
 				   filename, lineno, func, mutex_name);
 		DO_THREAD_CRASH;
 		return EBUSY;
@@ -141,7 +160,8 @@ int __ast_pthread_mutex_init(int trackin
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
 	t->track = NULL;
-	t->tracking = tracking;
+	t->flags.tracking = tracking;
+	t->flags.setup = 0;
 #endif /* DEBUG_THREADS */
 
 	pthread_mutexattr_init(&attr);
@@ -158,8 +178,8 @@ int __ast_pthread_mutex_destroy(const ch
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = t->track;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 1);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
@@ -169,7 +189,7 @@ int __ast_pthread_mutex_destroy(const ch
 		 * linked with libpthread.
 		 * This is not an error condition if the mutex is created on the fly.
 		 */
-		__ast_mutex_logger("%s line %d (%s): NOTICE: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): NOTICE: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 		DO_THREAD_CRASH;
 		res = EINVAL;
@@ -183,15 +203,15 @@ int __ast_pthread_mutex_destroy(const ch
 		pthread_mutex_unlock(&t->mutex);
 		break;
 	case EINVAL:
-		__ast_mutex_logger("%s line %d (%s): Error: attempt to destroy invalid mutex '%s'.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: attempt to destroy invalid mutex '%s'.\n",
 				  filename, lineno, func, mutex_name);
 		break;
 	case EBUSY:
-		__ast_mutex_logger("%s line %d (%s): Error: attempt to destroy locked mutex '%s'.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: attempt to destroy locked mutex '%s'.\n",
 				   filename, lineno, func, mutex_name);
 		if (lt) {
 			ast_reentrancy_lock(lt);
-			__ast_mutex_logger("%s line %d (%s): Error: '%s' was locked here.\n",
+			log_mutex_error(canlog, "%s line %d (%s): Error: '%s' was locked here.\n",
 				    lt->file[ROFFSET], lt->lineno[ROFFSET], lt->func[ROFFSET], mutex_name);
 #ifdef HAVE_BKTR
 			__dump_backtrace(&lt->backtrace[ROFFSET], canlog);
@@ -206,7 +226,7 @@ int __ast_pthread_mutex_destroy(const ch
 
 #ifdef DEBUG_THREADS
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error destroying mutex %s: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error destroying mutex %s: %s\n",
 				   filename, lineno, func, mutex_name, strerror(res));
 	}
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
@@ -236,15 +256,9 @@ int __ast_pthread_mutex_lock(const char
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -261,11 +275,8 @@ int __ast_pthread_mutex_lock(const char
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t, bt);
-#else
-		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t);
 #endif
+		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -284,14 +295,14 @@ int __ast_pthread_mutex_lock(const char
 			if (res == EBUSY) {
 				wait_time = time(NULL) - seconds;
 				if (wait_time > reported_wait && (wait_time % 5) == 0) {
-					__ast_mutex_logger("%s line %d (%s): Deadlock? waited %d sec for mutex '%s'?\n",
+					log_mutex_error(canlog, "%s line %d (%s): Deadlock? waited %d sec for mutex '%s'?\n",
 							   filename, lineno, func, (int) wait_time, mutex_name);
 					if (lt) {
 						ast_reentrancy_lock(lt);
 #ifdef HAVE_BKTR
 						__dump_backtrace(&lt->backtrace[lt->reentrancy], canlog);
 #endif
-						__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+						log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 								   lt->file[ROFFSET], lt->lineno[ROFFSET],
 								   lt->func[ROFFSET], mutex_name);
 #ifdef HAVE_BKTR
@@ -325,7 +336,7 @@ int __ast_pthread_mutex_lock(const char
 			lt->thread_id[lt->reentrancy] = pthread_self();
 			lt->reentrancy++;
 		} else {
-			__ast_mutex_logger("%s line %d (%s): '%s' really deep reentrancy!\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' really deep reentrancy!\n",
 							   filename, lineno, func, mutex_name);
 		}
 		ast_reentrancy_unlock(lt);
@@ -339,13 +350,11 @@ int __ast_pthread_mutex_lock(const char
 		} else {
 			bt = NULL;
 		}
-		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
 #endif
+		ast_remove_lock_info(t, bt);
 	}
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining mutex: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining mutex: %s\n",
 				   filename, lineno, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -360,15 +369,9 @@ int __ast_pthread_mutex_trylock(const ch
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -385,11 +388,8 @@ int __ast_pthread_mutex_trylock(const ch
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t, bt);
-#else
-		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t);
 #endif
+		ast_store_lock_info(AST_MUTEX, filename, lineno, func, mutex_name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -405,7 +405,7 @@ int __ast_pthread_mutex_trylock(const ch
 			lt->thread_id[lt->reentrancy] = pthread_self();
 			lt->reentrancy++;
 		} else {
-			__ast_mutex_logger("%s line %d (%s): '%s' really deep reentrancy!\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' really deep reentrancy!\n",
 					   filename, lineno, func, mutex_name);
 		}
 		ast_reentrancy_unlock(lt);
@@ -425,30 +425,25 @@ int __ast_pthread_mutex_unlock(const cha
 
 #ifdef DEBUG_THREADS
 	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 		DO_THREAD_CRASH;
 		return EINVAL;
 	}
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
-
+	lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	if (lt) {
 		ast_reentrancy_lock(lt);
 		if (lt->reentrancy && (lt->thread_id[ROFFSET] != pthread_self())) {
-			__ast_mutex_logger("%s line %d (%s): attempted unlock mutex '%s' without owning it!\n",
+			log_mutex_error(canlog, "%s line %d (%s): attempted unlock mutex '%s' without owning it!\n",
 					   filename, lineno, func, mutex_name);
-			__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 					   lt->file[ROFFSET], lt->lineno[ROFFSET], lt->func[ROFFSET], mutex_name);
 #ifdef HAVE_BKTR
 			__dump_backtrace(&lt->backtrace[ROFFSET], canlog);
@@ -457,7 +452,7 @@ int __ast_pthread_mutex_unlock(const cha
 		}
 
 		if (--lt->reentrancy < 0) {
-			__ast_mutex_logger("%s line %d (%s): mutex '%s' freed more times than we've locked!\n",
+			log_mutex_error(canlog, "%s line %d (%s): mutex '%s' freed more times than we've locked!\n",
 					   filename, lineno, func, mutex_name);
 			lt->reentrancy = 0;
 		}
@@ -476,11 +471,7 @@ int __ast_pthread_mutex_unlock(const cha
 #endif
 		ast_reentrancy_unlock(lt);
 
-#ifdef HAVE_BKTR
 		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
-#endif
 	}
 #endif /* DEBUG_THREADS */
 
@@ -488,7 +479,7 @@ int __ast_pthread_mutex_unlock(const cha
 
 #ifdef DEBUG_THREADS
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error releasing mutex: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error releasing mutex: %s\n",
 				   filename, lineno, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -554,34 +545,31 @@ int __ast_cond_wait(const char *filename
 #ifdef DEBUG_THREADS
 	struct ast_lock_track *lt = NULL;
 	struct ast_lock_track lt_orig;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 		DO_THREAD_CRASH;
 		return EINVAL;
 	}
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
-
+	lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	if (lt) {
 		ast_reentrancy_lock(lt);
 		if (lt->reentrancy && (lt->thread_id[ROFFSET] != pthread_self())) {
-			__ast_mutex_logger("%s line %d (%s): attempted wait using mutex '%s' without owning it!\n",
+			log_mutex_error(canlog, "%s line %d (%s): attempted wait using mutex '%s' without owning it!\n",
 					   filename, lineno, func, mutex_name);
-			__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 					   lt->file[ROFFSET], lt->lineno[ROFFSET], lt->func[ROFFSET], mutex_name);
 #ifdef HAVE_BKTR
 			__dump_backtrace(&lt->backtrace[ROFFSET], canlog);
 #endif
 			DO_THREAD_CRASH;
 		} else if (lt->reentrancy <= 0) {
-			__ast_mutex_logger("%s line %d (%s): attempted wait using an unlocked mutex '%s'\n",
+			log_mutex_error(canlog, "%s line %d (%s): attempted wait using an unlocked mutex '%s'\n",
 					   filename, lineno, func, mutex_name);
 			DO_THREAD_CRASH;
 		}
@@ -601,7 +589,7 @@ int __ast_cond_wait(const char *filename
 
 #ifdef DEBUG_THREADS
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error waiting on condition mutex '%s'\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error waiting on condition mutex '%s'\n",
 				   filename, lineno, func, strerror(res));
 		DO_THREAD_CRASH;
 	} else if (lt) {
@@ -622,34 +610,31 @@ int __ast_cond_timedwait(const char *fil
 #ifdef DEBUG_THREADS
 	struct ast_lock_track *lt = NULL;
 	struct ast_lock_track lt_orig;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 		DO_THREAD_CRASH;
 		return EINVAL;
 	}
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
-
+	lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	if (lt) {
 		ast_reentrancy_lock(lt);
 		if (lt->reentrancy && (lt->thread_id[ROFFSET] != pthread_self())) {
-			__ast_mutex_logger("%s line %d (%s): attempted wait using mutex '%s' without owning it!\n",
+			log_mutex_error(canlog, "%s line %d (%s): attempted wait using mutex '%s' without owning it!\n",
 					   filename, lineno, func, mutex_name);
-			__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 					   lt->file[ROFFSET], lt->lineno[ROFFSET], lt->func[ROFFSET], mutex_name);
 #ifdef HAVE_BKTR
 			__dump_backtrace(&lt->backtrace[ROFFSET], canlog);
 #endif
 			DO_THREAD_CRASH;
 		} else if (lt->reentrancy <= 0) {
-			__ast_mutex_logger("%s line %d (%s): attempted wait using an unlocked mutex '%s'\n",
+			log_mutex_error(canlog, "%s line %d (%s): attempted wait using an unlocked mutex '%s'\n",
 					   filename, lineno, func, mutex_name);
 			DO_THREAD_CRASH;
 		}
@@ -669,7 +654,7 @@ int __ast_cond_timedwait(const char *fil
 
 #ifdef DEBUG_THREADS
 	if (res && (res != ETIMEDOUT)) {
-		__ast_mutex_logger("%s line %d (%s): Error waiting on condition mutex '%s'\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error waiting on condition mutex '%s'\n",
 				   filename, lineno, func, strerror(res));
 		DO_THREAD_CRASH;
 	} else if (lt) {
@@ -691,7 +676,7 @@ int __ast_rwlock_init(int tracking, cons
 	if (t->lock != ((pthread_rwlock_t) __AST_RWLOCK_INIT_VALUE)) {
 		int canlog = tracking && strcmp(filename, "logger.c");
 
-		__ast_mutex_logger("%s line %d (%s): Warning: rwlock '%s' is already initialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Warning: rwlock '%s' is already initialized.\n",
 				filename, lineno, func, rwlock_name);
 		DO_THREAD_CRASH;
 		return EBUSY;
@@ -699,7 +684,8 @@ int __ast_rwlock_init(int tracking, cons
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
 	t->track = NULL;
-	t->tracking = tracking;
+	t->flags.tracking = tracking;
+	t->flags.setup = 0;
 #endif /* DEBUG_THREADS */
 
 	pthread_rwlockattr_init(&attr);
@@ -717,12 +703,12 @@ int __ast_rwlock_destroy(const char *fil
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = t->track;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 1);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if (t->lock == ((pthread_rwlock_t) __AST_RWLOCK_INIT_VALUE)) {
-		__ast_mutex_logger("%s line %d (%s): Warning: rwlock '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Warning: rwlock '%s' is uninitialized.\n",
 				   filename, lineno, func, rwlock_name);
 		DO_THREAD_CRASH;
 		res = EINVAL;
@@ -735,7 +721,7 @@ int __ast_rwlock_destroy(const char *fil
 
 #ifdef DEBUG_THREADS
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error destroying rwlock %s: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error destroying rwlock %s: %s\n",
 				filename, lineno, func, rwlock_name, strerror(res));
 	}
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
@@ -765,25 +751,20 @@ int __ast_rwlock_unlock(const char *file
 
 #ifdef DEBUG_THREADS
 	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
 	int lock_found = 0;
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS) && defined(CAN_COMPARE_MUTEX_TO_INIT_VALUE)
 	if ((t->lock) == ((pthread_rwlock_t) __AST_RWLOCK_INIT_VALUE)) {
-		__ast_mutex_logger("%s line %d (%s): Warning: rwlock '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Warning: rwlock '%s' is uninitialized.\n",
 				   filename, line, func, name);
 		DO_THREAD_CRASH;
 		return EINVAL;
 	}
 #endif /* AST_MUTEX_INIT_W_CONSTRUCTORS */
 
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
-
+	lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	if (lt) {
 		ast_reentrancy_lock(lt);
 		if (lt->reentrancy) {
@@ -811,18 +792,14 @@ int __ast_rwlock_unlock(const char *file
 		}
 
 		if (lock_found && --lt->reentrancy < 0) {
-			__ast_mutex_logger("%s line %d (%s): rwlock '%s' freed more times than we've locked!\n",
+			log_mutex_error(canlog, "%s line %d (%s): rwlock '%s' freed more times than we've locked!\n",
 					filename, line, func, name);
 			lt->reentrancy = 0;
 		}
 
 		ast_reentrancy_unlock(lt);
 
-#ifdef HAVE_BKTR
 		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
-#endif
 	}
 #endif /* DEBUG_THREADS */
 
@@ -830,7 +807,7 @@ int __ast_rwlock_unlock(const char *file
 
 #ifdef DEBUG_THREADS
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error releasing rwlock: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error releasing rwlock: %s\n",
 				filename, line, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -844,15 +821,9 @@ int __ast_rwlock_rdlock(const char *file
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -869,11 +840,8 @@ int __ast_rwlock_rdlock(const char *file
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -886,14 +854,14 @@ int __ast_rwlock_rdlock(const char *file
 			if (res == EBUSY) {
 				wait_time = time(NULL) - seconds;
 				if (wait_time > reported_wait && (wait_time % 5) == 0) {
-					__ast_mutex_logger("%s line %d (%s): Deadlock? waited %d sec for readlock '%s'?\n",
+					log_mutex_error(canlog, "%s line %d (%s): Deadlock? waited %d sec for readlock '%s'?\n",
 						filename, line, func, (int)wait_time, name);
 					if (lt) {
 						ast_reentrancy_lock(lt);
 #ifdef HAVE_BKTR
 						__dump_backtrace(&lt->backtrace[lt->reentrancy], canlog);
 #endif
-						__ast_mutex_logger("%s line %d (%s): '%s' was locked  here.\n",
+						log_mutex_error(canlog, "%s line %d (%s): '%s' was locked  here.\n",
 								lt->file[lt->reentrancy-1], lt->lineno[lt->reentrancy-1],
 								lt->func[lt->reentrancy-1], name);
 #ifdef HAVE_BKTR
@@ -932,14 +900,12 @@ int __ast_rwlock_rdlock(const char *file
 		} else {
 			bt = NULL;
 		}
-		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
 #endif
+		ast_remove_lock_info(t, bt);
 	}
 
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining read lock: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining read lock: %s\n",
 				filename, line, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -953,15 +919,9 @@ int __ast_rwlock_wrlock(const char *file
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -978,11 +938,8 @@ int __ast_rwlock_wrlock(const char *file
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -995,14 +952,14 @@ int __ast_rwlock_wrlock(const char *file
 			if (res == EBUSY) {
 				wait_time = time(NULL) - seconds;
 				if (wait_time > reported_wait && (wait_time % 5) == 0) {
-					__ast_mutex_logger("%s line %d (%s): Deadlock? waited %d sec for writelock '%s'?\n",
+					log_mutex_error(canlog, "%s line %d (%s): Deadlock? waited %d sec for writelock '%s'?\n",
 						filename, line, func, (int)wait_time, name);
 					if (lt) {
 						ast_reentrancy_lock(lt);
 #ifdef HAVE_BKTR
 						__dump_backtrace(&lt->backtrace[lt->reentrancy], canlog);
 #endif
-						__ast_mutex_logger("%s line %d (%s): '%s' was locked  here.\n",
+						log_mutex_error(canlog, "%s line %d (%s): '%s' was locked  here.\n",
 								lt->file[lt->reentrancy-1], lt->lineno[lt->reentrancy-1],
 								lt->func[lt->reentrancy-1], name);
 #ifdef HAVE_BKTR
@@ -1041,13 +998,11 @@ int __ast_rwlock_wrlock(const char *file
 		} else {
 			bt = NULL;
 		}
-		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
 #endif
+		ast_remove_lock_info(t, bt);
 	}
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining write lock: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining write lock: %s\n",
 				filename, line, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -1062,15 +1017,9 @@ int __ast_rwlock_timedrdlock(const char
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -1087,11 +1036,8 @@ int __ast_rwlock_timedrdlock(const char
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -1134,13 +1080,11 @@ int __ast_rwlock_timedrdlock(const char
 		} else {
 			bt = NULL;
 		}
-		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
 #endif
+		ast_remove_lock_info(t, bt);
 	}
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining read lock: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining read lock: %s\n",
 				filename, line, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -1155,15 +1099,9 @@ int __ast_rwlock_timedwrlock(const char
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-	int canlog = t->tracking && strcmp(filename, "logger.c");
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
+	int canlog = t->flags.tracking && strcmp(filename, "logger.c");
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -1180,11 +1118,8 @@ int __ast_rwlock_timedwrlock(const char
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -1227,13 +1162,11 @@ int __ast_rwlock_timedwrlock(const char
 		} else {
 			bt = NULL;
 		}
-		ast_remove_lock_info(t, bt);
-#else
-		ast_remove_lock_info(t);
 #endif
+		ast_remove_lock_info(t, bt);
 	}
 	if (res) {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining read lock: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining read lock: %s\n",
 				filename, line, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -1247,14 +1180,8 @@ int __ast_rwlock_tryrdlock(const char *f
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -1271,11 +1198,8 @@ int __ast_rwlock_tryrdlock(const char *f
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_RDLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
@@ -1306,14 +1230,8 @@ int __ast_rwlock_trywrlock(const char *f
 	int res;
 
 #ifdef DEBUG_THREADS
-	struct ast_lock_track *lt = NULL;
-#ifdef HAVE_BKTR
+	struct ast_lock_track *lt = ast_get_reentrancy(&t->track, &t->flags, 0);
 	struct ast_bt *bt = NULL;
-#endif
-
-	if (t->tracking) {
-		lt = ast_get_reentrancy(&t->track);
-	}
 
 	if (lt) {
 #ifdef HAVE_BKTR
@@ -1330,11 +1248,8 @@ int __ast_rwlock_trywrlock(const char *f
 			bt = &lt->backtrace[lt->reentrancy];
 		}
 		ast_reentrancy_unlock(lt);
-
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
-#else
-		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t);
 #endif
+		ast_store_lock_info(AST_WRLOCK, filename, line, func, name, t, bt);
 	}
 #endif /* DEBUG_THREADS */
 
diff -pruN 1:13.23.1~dfsg-2/main/logger.c 1:16.1.1~dfsg-1/main/logger.c
--- 1:13.23.1~dfsg-2/main/logger.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/logger.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 /* When we include logger.h again it will trample on some stuff in syslog.h, but
  * nothing we care about in here. */
 #include <syslog.h>
@@ -51,6 +49,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <fcntl.h>
 
 #include "asterisk/_private.h"
+#include "asterisk/module.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_LOG_DIR */
 #include "asterisk/logger.h"
 #include "asterisk/lock.h"
@@ -69,6 +68,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/buildinfo.h"
 #include "asterisk/ast_version.h"
 #include "asterisk/backtrace.h"
+#include "asterisk/json.h"
 
 /*** DOCUMENTATION
  ***/
@@ -82,21 +82,16 @@ static int filesize_reload_needed;
 static unsigned int global_logmask = 0xFFFF;
 static int queuelog_init;
 static int logger_initialized;
-static volatile int next_unique_callid; /* Used to assign unique call_ids to calls */
+static volatile int next_unique_callid = 1; /* Used to assign unique call_ids to calls */
 static int display_callids;
-static void unique_callid_cleanup(void *data);
+
+AST_THREADSTORAGE(unique_callid);
 
 static int logger_queue_size;
 static int logger_queue_limit = 1000;
 static int logger_messages_discarded;
 static unsigned int high_water_alert;
 
-struct ast_callid {
-	int call_identifier; /* Numerical value of the call displayed in the logs */
-};
-
-AST_THREADSTORAGE_CUSTOM(unique_callid, NULL, unique_callid_cleanup);
-
 static enum rotatestrategy {
 	NONE = 0,                /* Do not rotate log files at all, instead rely on external mechanisms */
 	SEQUENTIAL = 1 << 0,     /* Original method - create a new file, in order */
@@ -114,6 +109,16 @@ static struct {
 static char hostname[MAXHOSTNAMELEN];
 AST_THREADSTORAGE_RAW(in_safe_log);
 
+struct logchannel;
+struct logmsg;
+
+struct logformatter {
+	/* The name of the log formatter */
+	const char *name;
+	/* Pointer to the function that will format the log */
+	int (* const format_log)(struct logchannel *channel, struct logmsg *msg, char *buf, size_t size);
+};
+
 enum logtypes {
 	LOGTYPE_SYSLOG,
 	LOGTYPE_FILE,
@@ -121,6 +126,8 @@ enum logtypes {
 };
 
 struct logchannel {
+	/*! How the logs sent to this channel will be formatted */
+	struct logformatter formatter;
 	/*! What to log to this channel */
 	unsigned int logmask;
 	/*! If this channel is disabled or not */
@@ -155,9 +162,10 @@ enum logmsgtypes {
 struct logmsg {
 	enum logmsgtypes type;
 	int level;
+	int sublevel;
 	int line;
 	int lwp;
-	struct ast_callid *callid;
+	ast_callid callid;
 	AST_DECLARE_STRING_FIELDS(
 		AST_STRING_FIELD(date);
 		AST_STRING_FIELD(file);
@@ -170,9 +178,6 @@ struct logmsg {
 
 static void logmsg_free(struct logmsg *msg)
 {
-	if (msg->callid) {
-		ast_callid_unref(msg->callid);
-	}
 	ast_string_field_free_memory(msg);
 	ast_free(msg);
 }
@@ -248,6 +253,176 @@ AST_THREADSTORAGE(verbose_build_buf);
 AST_THREADSTORAGE(log_buf);
 #define LOG_BUF_INIT_SIZE       256
 
+static int format_log_json(struct logchannel *channel, struct logmsg *msg, char *buf, size_t size)
+{
+	struct ast_json *json;
+	char *str;
+	char call_identifier_str[13];
+	size_t json_str_len;
+
+	if (msg->callid) {
+		snprintf(call_identifier_str, sizeof(call_identifier_str), "[C-%08x]", msg->callid);
+	} else {
+		call_identifier_str[0] = '\0';
+	}
+
+	json = ast_json_pack("{s: s, s: s, "
+		"s: {s: i, s: s} "
+		"s: {s: {s: s, s: s, s: i}, "
+		"s: s, s: s} }",
+		"hostname", ast_config_AST_SYSTEM_NAME,
+		"timestamp", msg->date,
+		"identifiers",
+		"lwp", msg->lwp,
+		"callid", S_OR(call_identifier_str, ""),
+		"logmsg",
+		"location",
+		"filename", msg->file,
+		"function", msg->function,
+		"line", msg->line,
+		"level", msg->level_name,
+		"message", msg->message);
+	if (!json) {
+		return -1;
+	}
+
+	str = ast_json_dump_string(json);
+	if (!str) {
+		ast_json_unref(json);
+		return -1;
+	}
+
+	ast_copy_string(buf, str, size);
+	json_str_len = strlen(str);
+	if (json_str_len > size - 1) {
+		json_str_len = size - 1;
+	}
+	buf[json_str_len] = '\n';
+	buf[json_str_len + 1] = '\0';
+
+	term_strip(buf, buf, size);
+
+	ast_json_free(str);
+	ast_json_unref(json);
+
+	return 0;
+}
+
+static struct logformatter logformatter_json = {
+	.name = "json",
+	.format_log = format_log_json
+};
+
+static int logger_add_verbose_magic(struct logmsg *logmsg, char *buf, size_t size)
+{
+	const char *p;
+	const char *fmt;
+	struct ast_str *prefixed;
+	signed char magic = logmsg->sublevel > 9 ? -10 : -logmsg->sublevel - 1; /* 0 => -1, 1 => -2, etc.  Can't pass NUL, as it is EOS-delimiter */
+
+	/* For compatibility with modules still calling ast_verbose() directly instead of using ast_verb() */
+	if (logmsg->sublevel < 0) {
+		if (!strncmp(logmsg->message, VERBOSE_PREFIX_4, strlen(VERBOSE_PREFIX_4))) {
+			magic = -5;
+		} else if (!strncmp(logmsg->message, VERBOSE_PREFIX_3, strlen(VERBOSE_PREFIX_3))) {
+			magic = -4;
+		} else if (!strncmp(logmsg->message, VERBOSE_PREFIX_2, strlen(VERBOSE_PREFIX_2))) {
+			magic = -3;
+		} else if (!strncmp(logmsg->message, VERBOSE_PREFIX_1, strlen(VERBOSE_PREFIX_1))) {
+			magic = -2;
+		} else {
+			magic = -1;
+		}
+	}
+
+	if (!(prefixed = ast_str_thread_get(&verbose_buf, VERBOSE_BUF_INIT_SIZE))) {
+		return -1;
+	}
+
+	ast_str_reset(prefixed);
+
+	/* for every newline found in the buffer add verbose prefix data */
+	fmt = logmsg->message;
+	do {
+		if (!(p = strchr(fmt, '\n'))) {
+			p = strchr(fmt, '\0') - 1;
+		}
+		++p;
+
+		ast_str_append(&prefixed, 0, "%c", (char)magic);
+		ast_str_append_substr(&prefixed, 0, fmt, p - fmt);
+		fmt = p;
+	} while (p && *p);
+
+	snprintf(buf, size, "%s", ast_str_buffer(prefixed));
+
+	return 0;
+}
+
+static int format_log_default(struct logchannel *chan, struct logmsg *msg, char *buf, size_t size)
+{
+	char call_identifier_str[13];
+
+	if (msg->callid) {
+		snprintf(call_identifier_str, sizeof(call_identifier_str), "[C-%08x]", msg->callid);
+	} else {
+		call_identifier_str[0] = '\0';
+	}
+
+	switch (chan->type) {
+	case LOGTYPE_SYSLOG:
+		snprintf(buf, size, "%s[%d]%s: %s:%d in %s: %s",
+		     levels[msg->level], msg->lwp, call_identifier_str, msg->file,
+		     msg->line, msg->function, msg->message);
+		term_strip(buf, buf, size);
+		break;
+	case LOGTYPE_FILE:
+		snprintf(buf, size, "[%s] %s[%d]%s %s: %s",
+		      msg->date, msg->level_name, msg->lwp, call_identifier_str,
+		      msg->file, msg->message);
+		term_strip(buf, buf, size);
+		break;
+	case LOGTYPE_CONSOLE:
+		{
+			char linestr[32];
+			int has_file = !ast_strlen_zero(msg->file);
+			int has_line = (msg->line > 0);
+			int has_func = !ast_strlen_zero(msg->function);
+
+			/*
+			 * Verbose messages are interpreted by console channels in their own
+			 * special way
+			 */
+			if (msg->level == __LOG_VERBOSE) {
+				return logger_add_verbose_magic(msg, buf, size);
+			}
+
+			/* Turn the numerical line number into a string */
+			snprintf(linestr, sizeof(linestr), "%d", msg->line);
+			/* Build string to print out */
+			snprintf(buf, size, "[%s] " COLORIZE_FMT "[%d]%s: " COLORIZE_FMT "%s" COLORIZE_FMT " " COLORIZE_FMT "%s %s",
+				msg->date,
+				COLORIZE(colors[msg->level], 0, msg->level_name),
+				msg->lwp,
+				call_identifier_str,
+				COLORIZE(COLOR_BRWHITE, 0, has_file ? msg->file : ""),
+				has_file ? ":" : "",
+				COLORIZE(COLOR_BRWHITE, 0, has_line ? linestr : ""),
+				COLORIZE(COLOR_BRWHITE, 0, has_func ? msg->function : ""),
+				has_func ? ":" : "",
+				msg->message);
+		}
+		break;
+	}
+
+	return 0;
+}
+
+static struct logformatter logformatter_default = {
+	.name = "default",
+	.format_log = format_log_default,
+};
+
 static void make_components(struct logchannel *chan)
 {
 	char *w;
@@ -259,6 +434,33 @@ static void make_components(struct logch
 	/* Default to using option_verbose as the verbosity level of the logging channel.  */
 	verb_level = -1;
 
+	w = strchr(stringp, '[');
+	if (w) {
+		char *end = strchr(w + 1, ']');
+		if (!end) {
+			fprintf(stderr, "Logger Warning: bad formatter definition for %s in logger.conf\n", chan->filename);
+		} else {
+			char *formatter_name = w + 1;
+
+			*end = '\0';
+			stringp = end + 1;
+
+			if (!strcasecmp(formatter_name, "json")) {
+				memcpy(&chan->formatter, &logformatter_json, sizeof(chan->formatter));
+			} else if (!strcasecmp(formatter_name, "default")) {
+				memcpy(&chan->formatter, &logformatter_default, sizeof(chan->formatter));
+			} else {
+				fprintf(stderr, "Logger Warning: Unknown formatter definition %s for %s in logger.conf; using 'default'\n",
+					formatter_name, chan->filename);
+				memcpy(&chan->formatter, &logformatter_default, sizeof(chan->formatter));
+			}
+		}
+	}
+
+	if (!chan->formatter.name) {
+		memcpy(&chan->formatter, &logformatter_default, sizeof(chan->formatter));
+	}
+
 	while ((w = strsep(&stringp, ","))) {
 		w = ast_strip(w);
 		if (ast_strlen_zero(w)) {
@@ -286,6 +488,7 @@ static void make_components(struct logch
 		 * with calculating the ast_verb_sys_level value.
 		 */
 		chan->verbosity = -1;
+		logmask |= (1 << __LOG_VERBOSE);
 	} else {
 		chan->verbosity = verb_level;
 	}
@@ -423,14 +626,16 @@ static struct logchannel *make_logchanne
 	return chan;
 }
 
-/* \brief Read config, setup channels.
- * \param locked The logchannels list is locked and this is a reload
+/*!
+ * \brief Read config, setup channels.
  * \param altconf Alternate configuration file to read.
  *
+ * \pre logchannels list is write locked
+ *
  * \retval 0 Success
  * \retval -1 No config found or Failed
  */
-static int init_logger_chain(int locked, const char *altconf)
+static int init_logger_chain(const char *altconf)
 {
 	struct logchannel *chan;
 	struct ast_config *cfg;
@@ -442,10 +647,6 @@ static int init_logger_chain(int locked,
 		cfg = NULL;
 	}
 
-	if (!locked) {
-		AST_RWLIST_WRLOCK(&logchannels);
-	}
-
 	/* Set defaults */
 	hostname[0] = '\0';
 	display_callids = 1;
@@ -461,9 +662,6 @@ static int init_logger_chain(int locked,
 		ast_free(chan);
 	}
 	global_logmask = 0;
-	if (!locked) {
-		AST_RWLIST_UNLOCK(&logchannels);
-	}
 
 	errno = 0;
 	/* close syslog */
@@ -471,21 +669,14 @@ static int init_logger_chain(int locked,
 
 	/* If no config file, we're fine, set default options. */
 	if (!cfg) {
-		if (!(chan = ast_calloc(1, sizeof(*chan) + 1))) {
-			fprintf(stderr, "Failed to initialize default logging\n");
+		chan = make_logchannel("console", "error,warning,notice,verbose", 0, 0);
+		if (!chan) {
+			fprintf(stderr, "ERROR: Failed to initialize default logging\n");
 			return -1;
 		}
-		chan->type = LOGTYPE_CONSOLE;
-		chan->logmask = (1 << __LOG_WARNING) | (1 << __LOG_NOTICE) | (1 << __LOG_ERROR);
 
-		if (!locked) {
-			AST_RWLIST_WRLOCK(&logchannels);
-		}
 		AST_RWLIST_INSERT_HEAD(&logchannels, chan, list);
 		global_logmask |= chan->logmask;
-		if (!locked) {
-			AST_RWLIST_UNLOCK(&logchannels);
-		}
 
 		return -1;
 	}
@@ -548,12 +739,10 @@ static int init_logger_chain(int locked,
 		}
 	}
 
-	if (!locked) {
-		AST_RWLIST_WRLOCK(&logchannels);
-	}
 	var = ast_variable_browse(cfg, "logfiles");
 	for (; var; var = var->next) {
-		if (!(chan = make_logchannel(var->name, var->value, var->lineno, 0))) {
+		chan = make_logchannel(var->name, var->value, var->lineno, 0);
+		if (!chan) {
 			/* Print error message directly to the consoles since the lock is held
 			 * and we don't want to unlock with the list partially built */
 			ast_console_puts_mutable("ERROR: Unable to create log channel '", __LOG_ERROR);
@@ -570,10 +759,6 @@ static int init_logger_chain(int locked,
 		qlog = NULL;
 	}
 
-	if (!locked) {
-		AST_RWLIST_UNLOCK(&logchannels);
-	}
-
 	ast_config_destroy(cfg);
 
 	return 0;
@@ -928,7 +1113,7 @@ static int reload_logger(int rotate, con
 
 	filesize_reload_needed = 0;
 
-	init_logger_chain(1 /* locked */, altconf);
+	init_logger_chain(altconf);
 
 	ast_unload_realtime("queue_log");
 	if (logfiles.queue_log) {
@@ -945,16 +1130,6 @@ static int reload_logger(int rotate, con
 	return res;
 }
 
-/*! \brief Reload the logger module without rotating log files (also used from loader.c during
-	a full Asterisk reload) */
-int logger_reload(void)
-{
-	if (reload_logger(0, NULL)) {
-		return RESULT_FAILURE;
-	}
-	return RESULT_SUCCESS;
-}
-
 static char *handle_logger_reload(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	switch (cmd) {
@@ -1026,7 +1201,7 @@ int ast_logger_rotate_channel(const char
 		}
 	}
 
-	init_logger_chain(1 /* locked */, NULL);
+	init_logger_chain(NULL);
 
 	AST_RWLIST_UNLOCK(&logchannels);
 
@@ -1120,7 +1295,7 @@ int ast_logger_get_channels(int (*logent
 /*! \brief CLI command to show logging system configuration */
 static char *handle_logger_show_channels(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-#define FORMATL	"%-35.35s %-8.8s %-9.9s "
+#define FORMATL	"%-35.35s %-8.8s %-10.10s %-9.9s "
 	struct logchannel *chan;
 	switch (cmd) {
 	case CLI_INIT:
@@ -1133,15 +1308,16 @@ static char *handle_logger_show_channels
 		return NULL;
 	}
 	ast_cli(a->fd, "Logger queue limit: %d\n\n", logger_queue_limit);
-	ast_cli(a->fd, FORMATL, "Channel", "Type", "Status");
+	ast_cli(a->fd, FORMATL, "Channel", "Type", "Formatter", "Status");
 	ast_cli(a->fd, "Configuration\n");
-	ast_cli(a->fd, FORMATL, "-------", "----", "------");
+	ast_cli(a->fd, FORMATL, "-------", "----", "---------", "------");
 	ast_cli(a->fd, "-------------\n");
 	AST_RWLIST_RDLOCK(&logchannels);
 	AST_RWLIST_TRAVERSE(&logchannels, chan, list) {
 		unsigned int level;
 
 		ast_cli(a->fd, FORMATL, chan->filename, chan->type == LOGTYPE_CONSOLE ? "Console" : (chan->type == LOGTYPE_SYSLOG ? "Syslog" : "File"),
+			chan->formatter.name,
 			chan->disabled ? "Disabled" : "Enabled");
 		ast_cli(a->fd, " - ");
 		for (level = 0; level < ARRAY_LEN(levels); level++) {
@@ -1197,7 +1373,9 @@ static char *handle_logger_add_channel(s
 			"       Adds a temporary logger channel. This logger channel\n"
 			"       will exist until removed or until Asterisk is restarted.\n"
 			"       <levels> is a comma-separated list of desired logger\n"
-			"       levels such as: verbose,warning,error\n";
+			"       levels such as: verbose,warning,error\n"
+			"       An optional formatter may be specified with the levels;\n"
+			"       valid values are '[json]' and '[default]'.\n";
 		return NULL;
 	case CLI_GENERATE:
 		return NULL;
@@ -1295,13 +1473,6 @@ static char *handle_logger_remove_channe
 	}
 }
 
-struct verb {
-	void (*verboser)(const char *string);
-	AST_LIST_ENTRY(verb) list;
-};
-
-static AST_RWLIST_HEAD_STATIC(verbosers, verb);
-
 static struct ast_cli_entry cli_logger[] = {
 	AST_CLI_DEFINE(handle_logger_show_channels, "List configured log channels"),
 	AST_CLI_DEFINE(handle_logger_reload, "Reopens the log files"),
@@ -1322,100 +1493,16 @@ static struct sigaction handle_SIGXFSZ =
 	.sa_flags = SA_RESTART,
 };
 
-static void ast_log_vsyslog(struct logmsg *msg, int facility)
-{
-	char buf[BUFSIZ];
-	int syslog_level = ast_syslog_priority_from_loglevel(msg->level);
-	char call_identifier_str[13];
-
-	if (msg->callid) {
-		snprintf(call_identifier_str, sizeof(call_identifier_str), "[C-%08x]", (unsigned)msg->callid->call_identifier);
-	} else {
-		call_identifier_str[0] = '\0';
-	}
-
-	if (syslog_level < 0) {
-		/* we are locked here, so cannot ast_log() */
-		fprintf(stderr, "ast_log_vsyslog called with bogus level: %d\n", msg->level);
-		return;
-	}
-
-	/* Don't use LOG_MAKEPRI because it's broken in glibc<2.17 */
-	syslog_level = facility | syslog_level; /* LOG_MAKEPRI(facility, syslog_level); */
-
-	snprintf(buf, sizeof(buf), "%s[%d]%s: %s:%d in %s: %s",
-		 levels[msg->level], msg->lwp, call_identifier_str, msg->file, msg->line, msg->function, msg->message);
-
-	term_strip(buf, buf, strlen(buf) + 1);
-	syslog(syslog_level, "%s", buf);
-}
-
-static char *logger_strip_verbose_magic(const char *message, int level)
-{
-	const char *begin, *end;
-	char *stripped_message, *dst;
-	char magic = -(level + 1);
-
-	if (!(stripped_message = ast_malloc(strlen(message) + 1))) {
-		return NULL;
-	}
-
-	begin = message;
-	dst = stripped_message;
-	do {
-		end = strchr(begin, magic);
-		if (end) {
-			size_t len = end - begin;
-			memcpy(dst, begin, len);
-			begin = end + 1;
-			dst += len;
-		} else {
-			strcpy(dst, begin); /* safe */
-			break;
-		}
-	} while (1);
-
-	return stripped_message;
-}
-
 /*! \brief Print a normal log message to the channels */
 static void logger_print_normal(struct logmsg *logmsg)
 {
 	struct logchannel *chan = NULL;
 	char buf[BUFSIZ];
-	struct verb *v = NULL;
-	char *tmpmsg;
 	int level = 0;
 
-	if (logmsg->level == __LOG_VERBOSE) {
-
-		/* Iterate through the list of verbosers and pass them the log message string */
-		AST_RWLIST_RDLOCK(&verbosers);
-		AST_RWLIST_TRAVERSE(&verbosers, v, list)
-			v->verboser(logmsg->message);
-		AST_RWLIST_UNLOCK(&verbosers);
-
-		level = VERBOSE_MAGIC2LEVEL(logmsg->message);
-
-		tmpmsg = logger_strip_verbose_magic(logmsg->message, level);
-		if (tmpmsg) {
-			ast_string_field_set(logmsg, message, tmpmsg);
-			ast_free(tmpmsg);
-		}
-	}
-
 	AST_RWLIST_RDLOCK(&logchannels);
-
 	if (!AST_RWLIST_EMPTY(&logchannels)) {
 		AST_RWLIST_TRAVERSE(&logchannels, chan, list) {
-			char call_identifier_str[13];
-
-			if (logmsg->callid) {
-				snprintf(call_identifier_str, sizeof(call_identifier_str), "[C-%08x]", (unsigned)logmsg->callid->call_identifier);
-			} else {
-				call_identifier_str[0] = '\0';
-			}
-
 
 			/* If the channel is disabled, then move on to the next one */
 			if (chan->disabled) {
@@ -1426,68 +1513,75 @@ static void logger_print_normal(struct l
 				continue;
 			}
 
-			/* Check syslog channels */
-			if (chan->type == LOGTYPE_SYSLOG && (chan->logmask & (1 << logmsg->level))) {
-				ast_log_vsyslog(logmsg, chan->facility);
-			/* Console channels */
-			} else if (chan->type == LOGTYPE_CONSOLE && (chan->logmask & (1 << logmsg->level))) {
-				char linestr[128];
-
-				/* If the level is verbose, then skip it */
-				if (logmsg->level == __LOG_VERBOSE)
-					continue;
-
-				/* Turn the numerical line number into a string */
-				snprintf(linestr, sizeof(linestr), "%d", logmsg->line);
-				/* Build string to print out */
-				snprintf(buf, sizeof(buf), "[%s] " COLORIZE_FMT "[%d]%s: " COLORIZE_FMT ":" COLORIZE_FMT " " COLORIZE_FMT ": %s",
-					 logmsg->date,
-					 COLORIZE(colors[logmsg->level], 0, logmsg->level_name),
-					 logmsg->lwp,
-					 call_identifier_str,
-					 COLORIZE(COLOR_BRWHITE, 0, logmsg->file),
-					 COLORIZE(COLOR_BRWHITE, 0, linestr),
-					 COLORIZE(COLOR_BRWHITE, 0, logmsg->function),
-					 logmsg->message);
-				/* Print out */
-				ast_console_puts_mutable(buf, logmsg->level);
-			/* File channels */
-			} else if (chan->type == LOGTYPE_FILE && (chan->logmask & (1 << logmsg->level))) {
-				int res = 0;
-
-				/* If no file pointer exists, skip it */
-				if (!chan->fileptr) {
-					continue;
+			if (!(chan->logmask & (1 << logmsg->level))) {
+				continue;
+			}
+
+			switch (chan->type) {
+			case LOGTYPE_SYSLOG:
+				{
+					int syslog_level = ast_syslog_priority_from_loglevel(logmsg->level);
+
+					if (syslog_level < 0) {
+						/* we are locked here, so cannot ast_log() */
+						fprintf(stderr, "ast_log_vsyslog called with bogus level: %d\n", logmsg->level);
+						continue;
+					}
+
+					/* Don't use LOG_MAKEPRI because it's broken in glibc<2.17 */
+					syslog_level = chan->facility | syslog_level; /* LOG_MAKEPRI(chan->facility, syslog_level); */
+					if (!chan->formatter.format_log(chan, logmsg, buf, BUFSIZ)) {
+						syslog(syslog_level, "%s", buf);
+					}
 				}
+				break;
+			case LOGTYPE_CONSOLE:
+				if (!chan->formatter.format_log(chan, logmsg, buf, BUFSIZ)) {
+					ast_console_puts_mutable_full(buf, logmsg->level, logmsg->sublevel);
+				}
+				break;
+			case LOGTYPE_FILE:
+				{
+					int res = 0;
 
-				/* Print out to the file */
-				res = fprintf(chan->fileptr, "[%s] %s[%d]%s %s: %s",
-					      logmsg->date, logmsg->level_name, logmsg->lwp, call_identifier_str,
-					      logmsg->file, term_strip(buf, logmsg->message, BUFSIZ));
-				if (res <= 0 && !ast_strlen_zero(logmsg->message)) {
-					fprintf(stderr, "**** Asterisk Logging Error: ***********\n");
-					if (errno == ENOMEM || errno == ENOSPC)
-						fprintf(stderr, "Asterisk logging error: Out of disk space, can't log to log file %s\n", chan->filename);
-					else
-						fprintf(stderr, "Logger Warning: Unable to write to log file '%s': %s (disabled)\n", chan->filename, strerror(errno));
-					/*** DOCUMENTATION
-						<managerEventInstance>
-							<synopsis>Raised when a logging channel is disabled.</synopsis>
-							<syntax>
-								<parameter name="Channel">
-									<para>The name of the logging channel.</para>
-								</parameter>
-							</syntax>
-						</managerEventInstance>
-					***/
-					manager_event(EVENT_FLAG_SYSTEM, "LogChannel", "Channel: %s\r\nEnabled: No\r\nReason: %d - %s\r\n", chan->filename, errno, strerror(errno));
-					chan->disabled = 1;
-				} else if (res > 0) {
-					fflush(chan->fileptr);
+					if (!chan->fileptr) {
+						continue;
+					}
+
+					if (chan->formatter.format_log(chan, logmsg, buf, BUFSIZ)) {
+						continue;
+					}
+
+					/* Print out to the file */
+					res = fprintf(chan->fileptr, "%s", buf);
+					if (res > 0) {
+						fflush(chan->fileptr);
+					} else if (res <= 0 && !ast_strlen_zero(logmsg->message)) {
+						fprintf(stderr, "**** Asterisk Logging Error: ***********\n");
+						if (errno == ENOMEM || errno == ENOSPC) {
+							fprintf(stderr, "Asterisk logging error: Out of disk space, can't log to log file %s\n", chan->filename);
+						} else {
+							fprintf(stderr, "Logger Warning: Unable to write to log file '%s': %s (disabled)\n", chan->filename, strerror(errno));
+						}
+
+						/*** DOCUMENTATION
+							<managerEventInstance>
+								<synopsis>Raised when a logging channel is disabled.</synopsis>
+								<syntax>
+									<parameter name="Channel">
+										<para>The name of the logging channel.</para>
+									</parameter>
+								</syntax>
+							</managerEventInstance>
+						***/
+						manager_event(EVENT_FLAG_SYSTEM, "LogChannel", "Channel: %s\r\nEnabled: No\r\nReason: %d - %s\r\n", chan->filename, errno, strerror(errno));
+						chan->disabled = 1;
+					}
 				}
+				break;
 			}
 		}
-	} else if (logmsg->level != __LOG_VERBOSE) {
+	} else if (logmsg->level != __LOG_VERBOSE || option_verbose >= logmsg->sublevel) {
 		fputs(logmsg->message, stdout);
 	}
 
@@ -1502,8 +1596,8 @@ static void logger_print_normal(struct l
 	return;
 }
 
-static struct logmsg * __attribute__((format(printf, 6, 0))) format_log_message_ap(int level,
-	const char *file, int line, const char *function, struct ast_callid *callid,
+static struct logmsg * __attribute__((format(printf, 7, 0))) format_log_message_ap(int level,
+	int sublevel, const char *file, int line, const char *function, ast_callid callid,
 	const char *fmt, va_list ap)
 {
 	struct logmsg *logmsg = NULL;
@@ -1541,8 +1635,7 @@ static struct logmsg * __attribute__((fo
 	}
 
 	if (display_callids && callid) {
-		logmsg->callid = ast_callid_ref(callid);
-		/* callid will be unreffed at logmsg destruction */
+		logmsg->callid = callid;
 	}
 
 	/* Create our date/time */
@@ -1552,6 +1645,7 @@ static struct logmsg * __attribute__((fo
 
 	/* Copy over data */
 	logmsg->level = level;
+	logmsg->sublevel = sublevel;
 	logmsg->line = line;
 	ast_string_field_set(logmsg, level_name, levels[level]);
 	ast_string_field_set(logmsg, file, file);
@@ -1561,15 +1655,15 @@ static struct logmsg * __attribute__((fo
 	return logmsg;
 }
 
-static struct logmsg * __attribute__((format(printf, 6, 0))) format_log_message(int level,
-	const char *file, int line, const char *function, struct ast_callid *callid,
+static struct logmsg * __attribute__((format(printf, 7, 0))) format_log_message(int level,
+	int sublevel, const char *file, int line, const char *function, ast_callid callid,
 	const char *fmt, ...)
 {
 	struct logmsg *logmsg;
 	va_list ap;
 
 	va_start(ap, fmt);
-	logmsg = format_log_message_ap(level, file, line, function, callid, fmt, ap);
+	logmsg = format_log_message_ap(level, sublevel, file, line, function, callid, fmt, ap);
 	va_end(ap);
 
 	return logmsg;
@@ -1593,7 +1687,7 @@ static void *logger_thread(void *data)
 		}
 
 		if (high_water_alert) {
-			msg = format_log_message(__LOG_WARNING, "logger", 0, "***", NULL,
+			msg = format_log_message(__LOG_WARNING, 0, "logger", 0, "***", 0,
 				"Logging resumed.  %d message%s discarded.\n",
 				logger_messages_discarded, logger_messages_discarded == 1 ? "" : "s");
 			if (msg) {
@@ -1713,7 +1807,9 @@ int init_logger(void)
 	ast_mkdir(ast_config_AST_LOG_DIR, 0777);
 
 	/* create log channels */
-	res = init_logger_chain(0 /* locked */, NULL);
+	AST_RWLIST_WRLOCK(&logchannels);
+	res = init_logger_chain(NULL);
+	AST_RWLIST_UNLOCK(&logchannels);
 	ast_verb_update();
 	logger_initialized = 1;
 	if (res) {
@@ -1726,7 +1822,6 @@ int init_logger(void)
 void close_logger(void)
 {
 	struct logchannel *f = NULL;
-	struct verb *cur = NULL;
 
 	ast_cli_unregister_multiple(cli_logger, ARRAY_LEN(cli_logger));
 
@@ -1738,14 +1833,9 @@ void close_logger(void)
 	ast_cond_signal(&logcond);
 	AST_LIST_UNLOCK(&logmsgs);
 
-	if (logthread != AST_PTHREADT_NULL)
+	if (logthread != AST_PTHREADT_NULL) {
 		pthread_join(logthread, NULL);
-
-	AST_RWLIST_WRLOCK(&verbosers);
-	while ((cur = AST_LIST_REMOVE_HEAD(&verbosers, list))) {
-		ast_free(cur);
 	}
-	AST_RWLIST_UNLOCK(&verbosers);
 
 	AST_RWLIST_WRLOCK(&logchannels);
 
@@ -1767,121 +1857,77 @@ void close_logger(void)
 	AST_RWLIST_UNLOCK(&logchannels);
 }
 
-void ast_callid_strnprint(char *buffer, size_t buffer_size, struct ast_callid *callid)
+void ast_callid_strnprint(char *buffer, size_t buffer_size, ast_callid callid)
 {
-	snprintf(buffer, buffer_size, "[C-%08x]", (unsigned)callid->call_identifier);
+	snprintf(buffer, buffer_size, "[C-%08x]", callid);
 }
 
-struct ast_callid *ast_create_callid(void)
+ast_callid ast_create_callid(void)
 {
-	struct ast_callid *call;
-
-	call = ao2_alloc_options(sizeof(*call), NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
-	if (!call) {
-		ast_log(LOG_ERROR, "Could not allocate callid struct.\n");
-		return NULL;
-	}
-
-	call->call_identifier = ast_atomic_fetchadd_int(&next_unique_callid, +1);
-#ifdef TEST_FRAMEWORK
-	ast_debug(3, "CALL_ID [C-%08x] created by thread.\n", (unsigned)call->call_identifier);
-#endif
-	return call;
+	return ast_atomic_fetchadd_int(&next_unique_callid, +1);
 }
 
-struct ast_callid *ast_read_threadstorage_callid(void)
+ast_callid ast_read_threadstorage_callid(void)
 {
-	struct ast_callid **callid;
+	ast_callid *callid;
 
 	callid = ast_threadstorage_get(&unique_callid, sizeof(*callid));
-	if (callid && *callid) {
-		ast_callid_ref(*callid);
-		return *callid;
-	}
-
-	return NULL;
 
+	return callid ? *callid : 0;
 }
 
-int ast_callid_threadassoc_change(struct ast_callid *callid)
+int ast_callid_threadassoc_change(ast_callid callid)
 {
-	struct ast_callid **id = ast_threadstorage_get(&unique_callid, sizeof(*id));
+	ast_callid *id = ast_threadstorage_get(&unique_callid, sizeof(*id));
 
 	if (!id) {
-		ast_log(LOG_ERROR, "Failed to allocate thread storage.\n");
 		return -1;
 	}
 
-	if (*id && (*id != callid)) {
-#ifdef TEST_FRAMEWORK
-		ast_debug(3, "CALL_ID [C-%08x] being removed from thread.\n", (unsigned)(*id)->call_identifier);
-#endif
-		*id = ast_callid_unref(*id);
-		*id = NULL;
-	}
-
-	if (!(*id) && callid) {
-		/* callid will be unreffed at thread destruction */
-		ast_callid_ref(callid);
-		*id = callid;
-#ifdef TEST_FRAMEWORK
-		ast_debug(3, "CALL_ID [C-%08x] bound to thread.\n", (unsigned)callid->call_identifier);
-#endif
-	}
+	*id = callid;
 
 	return 0;
 }
 
-int ast_callid_threadassoc_add(struct ast_callid *callid)
+int ast_callid_threadassoc_add(ast_callid callid)
 {
-	struct ast_callid **pointing;
+	ast_callid *pointing;
 
 	pointing = ast_threadstorage_get(&unique_callid, sizeof(*pointing));
-	if (!(pointing)) {
-		ast_log(LOG_ERROR, "Failed to allocate thread storage.\n");
+	if (!pointing) {
 		return -1;
 	}
 
-	if (!(*pointing)) {
-		/* callid will be unreffed at thread destruction */
-		ast_callid_ref(callid);
-		*pointing = callid;
-#ifdef TEST_FRAMEWORK
-		ast_debug(3, "CALL_ID [C-%08x] bound to thread.\n", (unsigned)callid->call_identifier);
-#endif
-	} else {
-		ast_log(LOG_WARNING, "Attempted to ast_callid_threadassoc_add on thread already associated with a callid.\n");
+	if (*pointing) {
+		ast_log(LOG_ERROR, "ast_callid_threadassoc_add(C-%08x) on thread "
+			"already associated with callid [C-%08x].\n", callid, *pointing);
 		return 1;
 	}
 
+	*pointing = callid;
 	return 0;
 }
 
 int ast_callid_threadassoc_remove(void)
 {
-	struct ast_callid **pointing;
+	ast_callid *pointing;
 
 	pointing = ast_threadstorage_get(&unique_callid, sizeof(*pointing));
-	if (!(pointing)) {
-		ast_log(LOG_ERROR, "Failed to allocate thread storage.\n");
+	if (!pointing) {
 		return -1;
 	}
 
-	if (!(*pointing)) {
-		ast_log(LOG_ERROR, "Tried to clean callid thread storage with no callid in thread storage.\n");
-		return -1;
-	} else {
-#ifdef TEST_FRAMEWORK
-		ast_debug(3, "CALL_ID [C-%08x] being removed from thread.\n", (unsigned)(*pointing)->call_identifier);
-#endif
-		*pointing = ast_callid_unref(*pointing);
+	if (*pointing) {
+		*pointing = 0;
 		return 0;
 	}
+
+	return -1;
 }
 
-int ast_callid_threadstorage_auto(struct ast_callid **callid)
+int ast_callid_threadstorage_auto(ast_callid *callid)
 {
-	struct ast_callid *tmp;
+	ast_callid tmp;
 
 	/* Start by trying to see if a callid is available from thread storage */
 	tmp = ast_read_threadstorage_callid();
@@ -1891,10 +1937,9 @@ int ast_callid_threadstorage_auto(struct
 	}
 
 	/* If that failed, try to create a new one and bind it. */
-	tmp = ast_create_callid();
-	if (tmp) {
-		ast_callid_threadassoc_add(tmp);
-		*callid = tmp;
+	*callid = ast_create_callid();
+	if (*callid) {
+		ast_callid_threadassoc_add(*callid);
 		return 1;
 	}
 
@@ -1902,42 +1947,24 @@ int ast_callid_threadstorage_auto(struct
 	return -1;
 }
 
-void ast_callid_threadstorage_auto_clean(struct ast_callid *callid, int callid_created)
+void ast_callid_threadstorage_auto_clean(ast_callid callid, int callid_created)
 {
-	if (callid) {
+	if (callid && callid_created) {
 		/* If the callid was created rather than simply grabbed from the thread storage, we need to unbind here. */
-		if (callid_created == 1) {
-			ast_callid_threadassoc_remove();
-		}
-		callid = ast_callid_unref(callid);
+		ast_callid_threadassoc_remove();
 	}
 }
 
 /*!
- * \internal
- * \brief thread storage cleanup function for unique_callid
- */
-static void unique_callid_cleanup(void *data)
-{
-	struct ast_callid **callid = data;
-
-	if (*callid) {
-		ast_callid_unref(*callid);
-	}
-
-	ast_free(data);
-}
-
-/*!
  * \brief send log messages to syslog and/or the console
  */
-static void __attribute__((format(printf, 6, 0))) ast_log_full(int level, const char *file,
-	int line, const char *function, struct ast_callid *callid, const char *fmt, va_list ap)
+static void __attribute__((format(printf, 7, 0))) ast_log_full(int level, int sublevel,
+	const char *file, int line, const char *function, ast_callid callid,
+	const char *fmt, va_list ap)
 {
 	struct logmsg *logmsg = NULL;
 
-	/* Ignore anything that never gets logged anywhere */
-	if (level != __LOG_VERBOSE && !(global_logmask & (1 << level))) {
+	if (level == __LOG_VERBOSE && ast_opt_remote && ast_opt_exec) {
 		return;
 	}
 
@@ -1945,7 +1972,7 @@ static void __attribute__((format(printf
 	if (logger_queue_size >= logger_queue_limit && !close_logger_thread) {
 		logger_messages_discarded++;
 		if (!high_water_alert && !close_logger_thread) {
-			logmsg = format_log_message(__LOG_WARNING, "logger", 0, "***", NULL,
+			logmsg = format_log_message(__LOG_WARNING, 0, "logger", 0, "***", 0,
 				"Log queue threshold (%d) exceeded.  Discarding new messages.\n", logger_queue_limit);
 			AST_LIST_INSERT_TAIL(&logmsgs, logmsg, list);
 			high_water_alert = 1;
@@ -1956,7 +1983,7 @@ static void __attribute__((format(printf
 	}
 	AST_LIST_UNLOCK(&logmsgs);
 
-	logmsg = format_log_message_ap(level, file, line, function, callid, fmt, ap);
+	logmsg = format_log_message_ap(level, sublevel, file, line, function, callid, fmt, ap);
 	if (!logmsg) {
 		return;
 	}
@@ -1981,21 +2008,23 @@ static void __attribute__((format(printf
 
 void ast_log(int level, const char *file, int line, const char *function, const char *fmt, ...)
 {
-	struct ast_callid *callid;
 	va_list ap;
 
+	va_start(ap, fmt);
+	ast_log_ap(level, file, line, function, fmt, ap);
+	va_end(ap);
+}
+
+void ast_log_ap(int level, const char *file, int line, const char *function, const char *fmt, va_list ap)
+{
+	ast_callid callid;
+
 	callid = ast_read_threadstorage_callid();
 
-	va_start(ap, fmt);
 	if (level == __LOG_VERBOSE) {
 		__ast_verbose_ap(file, line, function, 0, callid, fmt, ap);
 	} else {
-		ast_log_full(level, file, line, function, callid, fmt, ap);
-	}
-	va_end(ap);
-
-	if (callid) {
-		ast_callid_unref(callid);
+		ast_log_full(level, -1, file, line, function, callid, fmt, ap);
 	}
 }
 
@@ -2003,7 +2032,7 @@ void ast_log_safe(int level, const char
 {
 	va_list ap;
 	void *recursed = ast_threadstorage_get_ptr(&in_safe_log);
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	if (recursed) {
 		return;
@@ -2018,22 +2047,18 @@ void ast_log_safe(int level, const char
 	callid = ast_read_threadstorage_callid();
 
 	va_start(ap, fmt);
-	ast_log_full(level, file, line, function, callid, fmt, ap);
+	ast_log_full(level, -1, file, line, function, callid, fmt, ap);
 	va_end(ap);
 
-	if (callid) {
-		ast_callid_unref(callid);
-	}
-
 	/* Clear flag so the next allocation failure can be logged. */
 	ast_threadstorage_set_ptr(&in_safe_log, NULL);
 }
 
-void ast_log_callid(int level, const char *file, int line, const char *function, struct ast_callid *callid, const char *fmt, ...)
+void ast_log_callid(int level, const char *file, int line, const char *function, ast_callid callid, const char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap, fmt);
-	ast_log_full(level, file, line, function, callid, fmt, ap);
+	ast_log_full(level, -1, file, line, function, callid, fmt, ap);
 	va_end(ap);
 }
 
@@ -2043,7 +2068,7 @@ void ast_log_backtrace(void)
 #ifdef HAVE_BKTR
 	struct ast_bt *bt;
 	int i = 0;
-	char **strings;
+	struct ast_vector_string *strings;
 
 	if (!(bt = ast_bt_create())) {
 		ast_log(LOG_WARNING, "Unable to allocate space for backtrace structure\n");
@@ -2051,14 +2076,21 @@ void ast_log_backtrace(void)
 	}
 
 	if ((strings = ast_bt_get_symbols(bt->addresses, bt->num_frames))) {
-		ast_verbose("Got %d backtrace record%c\n", bt->num_frames, bt->num_frames != 1 ? 's' : ' ');
-		for (i = 3; i < bt->num_frames - 2; i++) {
-			ast_verbose("#%d: [%p] %s\n", i - 3, bt->addresses[i], strings[i]);
+		int count = AST_VECTOR_SIZE(strings);
+		struct ast_str *buf = ast_str_create(bt->num_frames * 64);
+
+		if (buf) {
+			ast_str_append(&buf, 0, "Got %d backtrace record%c\n", count - 3, count - 3 != 1 ? 's' : ' ');
+			for (i = 3; i < AST_VECTOR_SIZE(strings); i++) {
+				ast_str_append(&buf, 0, "#%2d: %s\n", i - 3, AST_VECTOR_GET(strings, i));
+			}
+			ast_log_safe(__LOG_ERROR, NULL, 0, NULL, "%s\n", ast_str_buffer(buf));
+			ast_free(buf);
 		}
 
-		ast_std_free(strings);
+		ast_bt_free_symbols(strings);
 	} else {
-		ast_verbose("Could not allocate memory for backtrace\n");
+		ast_log(LOG_ERROR, "Could not allocate memory for backtrace\n");
 	}
 	ast_bt_destroy(bt);
 #else
@@ -2066,60 +2098,14 @@ void ast_log_backtrace(void)
 #endif /* defined(HAVE_BKTR) */
 }
 
-void __ast_verbose_ap(const char *file, int line, const char *func, int level, struct ast_callid *callid, const char *fmt, va_list ap)
+void __ast_verbose_ap(const char *file, int line, const char *func, int level, ast_callid callid, const char *fmt, va_list ap)
 {
-	const char *p;
-	struct ast_str *prefixed, *buf;
-	int res = 0;
-	signed char magic = level > 9 ? -10 : -level - 1; /* 0 => -1, 1 => -2, etc.  Can't pass NUL, as it is EOS-delimiter */
-
-	/* For compatibility with modules still calling ast_verbose() directly instead of using ast_verb() */
-	if (level < 0) {
-		if (!strncmp(fmt, VERBOSE_PREFIX_4, strlen(VERBOSE_PREFIX_4))) {
-			magic = -5;
-		} else if (!strncmp(fmt, VERBOSE_PREFIX_3, strlen(VERBOSE_PREFIX_3))) {
-			magic = -4;
-		} else if (!strncmp(fmt, VERBOSE_PREFIX_2, strlen(VERBOSE_PREFIX_2))) {
-			magic = -3;
-		} else if (!strncmp(fmt, VERBOSE_PREFIX_1, strlen(VERBOSE_PREFIX_1))) {
-			magic = -2;
-		} else {
-			magic = -1;
-		}
-	}
-
-	if (!(prefixed = ast_str_thread_get(&verbose_buf, VERBOSE_BUF_INIT_SIZE)) ||
-	    !(buf = ast_str_thread_get(&verbose_build_buf, VERBOSE_BUF_INIT_SIZE))) {
-		return;
-	}
-
-	res = ast_str_set_va(&buf, 0, fmt, ap);
-	/* If the build failed then we can drop this allocated message */
-	if (res == AST_DYNSTR_BUILD_FAILED) {
-		return;
-	}
-
-	ast_str_reset(prefixed);
-
-	/* for every newline found in the buffer add verbose prefix data */
-	fmt = ast_str_buffer(buf);
-	do {
-		if (!(p = strchr(fmt, '\n'))) {
-			p = strchr(fmt, '\0') - 1;
-		}
-		++p;
-
-		ast_str_append(&prefixed, 0, "%c", (char)magic);
-		ast_str_append_substr(&prefixed, 0, fmt, p - fmt);
-		fmt = p;
-	} while (p && *p);
-
-	ast_log_callid(__LOG_VERBOSE, file, line, func, callid, "%s", ast_str_buffer(prefixed));
+	ast_log_full(__LOG_VERBOSE, level, file, line, func, callid, fmt, ap);
 }
 
 void __ast_verbose(const char *file, int line, const char *func, int level, const char *fmt, ...)
 {
-	struct ast_callid *callid;
+	ast_callid callid;
 	va_list ap;
 
 	callid = ast_read_threadstorage_callid();
@@ -2127,13 +2113,9 @@ void __ast_verbose(const char *file, int
 	va_start(ap, fmt);
 	__ast_verbose_ap(file, line, func, level, callid, fmt, ap);
 	va_end(ap);
-
-	if (callid) {
-		ast_callid_unref(callid);
-	}
 }
 
-void __ast_verbose_callid(const char *file, int line, const char *func, int level, struct ast_callid *callid, const char *fmt, ...)
+void __ast_verbose_callid(const char *file, int line, const char *func, int level, ast_callid callid, const char *fmt, ...)
 {
 	va_list ap;
 	va_start(ap, fmt);
@@ -2141,25 +2123,6 @@ void __ast_verbose_callid(const char *fi
 	va_end(ap);
 }
 
-/* No new code should use this directly, but we have the ABI for backwards compat */
-#undef ast_verbose
-void __attribute__((format(printf, 1,2))) ast_verbose(const char *fmt, ...);
-void ast_verbose(const char *fmt, ...)
-{
-	struct ast_callid *callid;
-	va_list ap;
-
-	callid = ast_read_threadstorage_callid();
-
-	va_start(ap, fmt);
-	__ast_verbose_ap("", 0, "", 0, callid, fmt, ap);
-	va_end(ap);
-
-	if (callid) {
-		ast_callid_unref(callid);
-	}
-}
-
 /*! Console verbosity level node. */
 struct verb_console {
 	/*! List node link */
@@ -2302,40 +2265,6 @@ void ast_verb_console_set(int verb_level
 	ast_verb_update();
 }
 
-int ast_register_verbose(void (*v)(const char *string))
-{
-	struct verb *verb;
-
-	if (!(verb = ast_malloc(sizeof(*verb))))
-		return -1;
-
-	verb->verboser = v;
-
-	AST_RWLIST_WRLOCK(&verbosers);
-	AST_RWLIST_INSERT_HEAD(&verbosers, verb, list);
-	AST_RWLIST_UNLOCK(&verbosers);
-
-	return 0;
-}
-
-int ast_unregister_verbose(void (*v)(const char *string))
-{
-	struct verb *cur;
-
-	AST_RWLIST_WRLOCK(&verbosers);
-	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&verbosers, cur, list) {
-		if (cur->verboser == v) {
-			AST_RWLIST_REMOVE_CURRENT(list);
-			ast_free(cur);
-			break;
-		}
-	}
-	AST_RWLIST_TRAVERSE_SAFE_END;
-	AST_RWLIST_UNLOCK(&verbosers);
-
-	return cur ? 0 : -1;
-}
-
 static void update_logchannels(void)
 {
 	struct logchannel *cur;
@@ -2448,3 +2377,28 @@ int ast_logger_get_queue_limit(void)
 {
 	return logger_queue_limit;
 }
+
+static int reload_module(void)
+{
+	return reload_logger(0, NULL);
+}
+
+static int unload_module(void)
+{
+	return 0;
+}
+
+static int load_module(void)
+{
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+/* Logger is initialized separate from the module loader, only reload_module does anything. */
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Logger",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	/* This reload does not support realtime so it does not require "extconfig". */
+	.reload = reload_module,
+	.load_pri = 0,
+);
diff -pruN 1:13.23.1~dfsg-2/main/Makefile 1:16.1.1~dfsg-1/main/Makefile
--- 1:13.23.1~dfsg-2/main/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -17,6 +17,11 @@ all: asterisk
 
 include $(ASTTOPDIR)/Makefile.moddir_rules
 
+MOD_SRC:=cdr.c cel.c config.c ccss.c dnsmgr.c dsp.c enum.c features.c http.c indications.c logger.c manager.c named_acl.c plc.c sounds.c udptl.c
+# Allow deletion of built-in modules without needing to modify this source.
+MOD_SRC:=$(wildcard $(MOD_SRC))
+MOD_OBJS:=$(sort $(MOD_SRC:.c=.o))
+
 # Must include the extra ast_expr2.c, ast_expr2f.c, in case they need to be regenerated (because to force regeneration, we delete them)
 SRC:=$(wildcard *.c) ast_expr2.c ast_expr2f.c
 ifeq ($(AST_ASTERISKSSL),yes)
@@ -25,7 +30,7 @@ endif
 ifeq ($(PJPROJECT_BUNDLED),yes)
 SRC:=$(filter-out libasteriskpj.c,$(SRC))
 endif
-OBJSFILTER=fskmodem_int.o fskmodem_float.o cygload.o buildinfo.o
+OBJSFILTER:=$(MOD_OBJS) fskmodem_int.o fskmodem_float.o cygload.o buildinfo.o
 OBJS=$(filter-out $(OBJSFILTER),$(SRC:.c=.o))
 
 # we need to link in the objects statically, not as a library, because
@@ -47,14 +52,14 @@ AST_LIBS+=$(AST_CLANG_BLOCKS_LIBS)
 AST_LIBS+=$(RT_LIB)
 AST_LIBS+=$(SYSTEMD_LIB)
 
-ifneq ($(findstring $(OSARCH), linux-gnu uclinux linux-uclibc kfreebsd-gnu),)
+ifneq ($(findstring $(OSARCH), linux-gnu uclinux linux-uclibc linux-musl kfreebsd-gnu),)
   AST_LIBS+=-ldl
   ifneq (x$(CAP_LIB),x)
     AST_LIBS+=$(CAP_LIB)
   endif
-  AST_LIBS+=-lpthread $(EDITLINE_LIB) -lm -lresolv
+  AST_LIBS+=-lpthread -lm -lresolv
 else
-  AST_LIBS+=$(EDITLINE_LIB) -lm
+  AST_LIBS+=-lm
 endif
 
 ifneq ($(findstring BETTER_BACKTRACES,$(MENUSELECT_CFLAGS)),)
@@ -85,11 +90,11 @@ ifneq ($(findstring $(OSARCH), mingw32 c
   ASTLINK+=-shared -Wl,--out-implib,libasterisk.a
 endif
 ifeq ($(OSARCH),NetBSD)
-  AST_LIBS+=-lpthread -lcrypto -lm -L/usr/pkg/lib $(EDITLINE_LIB)
+  AST_LIBS+=-lpthread -lcrypto -lm -L/usr/pkg/lib
 endif
 
 ifeq ($(OSARCH),OpenBSD)
-  AST_LIBS+=-lcrypto -lpthread -lm $(EDITLINE_LIB)
+  AST_LIBS+=-lcrypto -lpthread -lm
 endif
 
 ifeq ($(OSARCH),SunOS)
@@ -114,10 +119,6 @@ endif
 .PHONY: CHECK_SUBDIR
 CHECK_SUBDIR:	# do nothing, just make sure that we recurse in the subdir/
 
-editline/libedit.a: CHECK_SUBDIR
-	cd editline && test -f config.h || CFLAGS="$(PTHREAD_CFLAGS) $(subst $(ASTTOPDIR),../../,$(_ASTCFLAGS:-Werror=) $(ASTCFLAGS))" LDFLAGS="$(_ASTLDFLAGS) $(ASTLDFLAGS)" ./configure --build=$(BUILD_PLATFORM) --host=$(HOST_PLATFORM) --with-ncurses=$(NCURSES_DIR) --with-curses=$(CURSES_DIR) --with-termcap=$(TERMCAP_DIR) --with-tinfo=$(TINFO_DIR)
-	$(MAKE) -C editline libedit.a
-
 ifneq ($(findstring REBUILD_PARSERS,$(MENUSELECT_CFLAGS)),)
 ast_expr2.c ast_expr2.h: ast_expr2.y
 else
@@ -134,31 +135,39 @@ endif
 	$(ECHO_PREFIX) echo "   [FLEX] $< -> $@"
 	$(CMD_PREFIX) $(FLEX) -o $@ ast_expr2.fl
 	$(CMD_PREFIX) sed 's@#if __STDC_VERSION__ >= 199901L@#if !defined __STDC_VERSION__ || __STDC_VERSION__ >= 199901L@' $@ > $@.fix
-	$(CMD_PREFIX) echo "#include \"asterisk.h\"" > $@
+	$(CMD_PREFIX) echo "#define ASTMM_LIBC ASTMM_REDIRECT" > $@
+	$(CMD_PREFIX) echo "#include \"asterisk.h\"" >> $@
 	$(CMD_PREFIX) echo >> $@
 	$(CMD_PREFIX) cat $@.fix >> $@
 	$(CMD_PREFIX) rm $@.fix
 
-ast_expr2f.o: _ASTCFLAGS+=-Wno-unused
-cdr.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
-
-ifneq ($(LIBEDIT_INTERNAL),no)
-LIBEDIT_OBJ=editline/libedit.a
-LIBEDIT_INCLUDE=-I. -Ieditline
+ifneq ($(findstring ENABLE_UPLOADS,$(MENUSELECT_CFLAGS)),)
+GMIMELDFLAGS+=$(GMIME_LIB)
+GMIMECFLAGS+=$(GMIME_INCLUDE)
 endif
 
-db.o: _ASTCFLAGS+=$(SQLITE3_INCLUDE)
+# Alter CFLAGS for specific sources
+stdtime/localtime.o: _ASTCFLAGS+=$(AST_NO_STRICT_OVERFLOW) -Wno-format-nonliteral
+
 asterisk.o: _ASTCFLAGS+=$(LIBEDIT_INCLUDE)
-json.o: _ASTCFLAGS+=$(JANSSON_INCLUDE)
+ast_expr2f.o: _ASTCFLAGS+=-Wno-unused
+astmm.o: _ASTCFLAGS+=$(call get_menuselect_cflags,MALLOC_DEBUG DEBUG_CHAOS)
+astobj2.o astobj2_container.o astobj2_hash.o astobj2_rbtree.o: _ASTCFLAGS+=$(call get_menuselect_cflags,AO2_DEBUG)
+backtrace.o: _ASTCFLAGS+=$(call get_menuselect_cflags,BETTER_BACKTRACES)
 bucket.o: _ASTCFLAGS+=$(URIPARSER_INCLUDE)
+cdr.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 crypt.o: _ASTCFLAGS+=$(CRYPT_INCLUDE)
+db.o: _ASTCFLAGS+=$(SQLITE3_INCLUDE)
+dsp.o: _ASTCFLAGS+=$(call get_menuselect_cflags,RADIO_RELAX)
+http.o: _ASTCFLAGS+=$(GMIMECFLAGS)
+iostream.o: _ASTCFLAGS+=$(OPENSSL_INCLUDE)
+json.o: _ASTCFLAGS+=$(JANSSON_INCLUDE)
+lock.o: _ASTCFLAGS+=$(call get_menuselect_cflags,DETECT_DEADLOCKS)
+options.o: _ASTCFLAGS+=$(call get_menuselect_cflags,REF_DEBUG)
+sched.o: _ASTCFLAGS+=$(call get_menuselect_cflags,DEBUG_SCHEDULER DUMP_SCHEDULER)
+tcptls.o: _ASTCFLAGS+=$(OPENSSL_INCLUDE) -Wno-deprecated-declarations
 uuid.o: _ASTCFLAGS+=$(UUID_INCLUDE)
 
-ifneq ($(findstring ENABLE_UPLOADS,$(MENUSELECT_CFLAGS)),)
-http.o: _ASTCFLAGS+=$(GMIME_INCLUDE)
-endif
-
-stdtime/localtime.o: _ASTCFLAGS+=$(AST_NO_STRICT_OVERFLOW) -Wno-format-nonliteral
 
 OBJS:=$(sort $(OBJS))
 
@@ -172,11 +181,8 @@ else
 MAIN_TGT:=asterisk
 endif
 
-ifneq ($(findstring ENABLE_UPLOADS,$(MENUSELECT_CFLAGS)),)
-GMIMELDFLAGS+=$(GMIME_LIB)
-endif
-
 $(OBJS): _ASTCFLAGS+=-DAST_MODULE=\"core\" -DAST_IN_CORE
+$(MOD_OBJS): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,$*)
 
 libasteriskssl.o: _ASTCFLAGS+=$(OPENSSL_INCLUDE)
 
@@ -191,7 +197,7 @@ ifeq ($(findstring darwin,$(OSARCH)),) #
 ASTSSL_LIB:=libasteriskssl.so
 
 $(ASTSSL_LIB).$(ASTSSL_SO_VERSION): _ASTLDFLAGS+=-Wl,-soname=$(ASTSSL_LIB).$(ASTSSL_SO_VERSION)
-$(ASTSSL_LIB).$(ASTSSL_SO_VERSION): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskssl\"
+$(ASTSSL_LIB).$(ASTSSL_SO_VERSION): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskssl\" -DAST_NOT_MODULE
 $(ASTSSL_LIB).$(ASTSSL_SO_VERSION): LIBS+=$(ASTSSL_LIBS)
 ifeq ($(GNU_LD),1)
     $(ASTSSL_LIB).$(ASTSSL_SO_VERSION): SO_SUPPRESS_SYMBOLS=-Wl,--version-script,libasteriskssl.exports,--warn-common
@@ -207,7 +213,7 @@ ifeq ($(GNU_LD),1)
 	$(CMD_PREFIX) $(ASTTOPDIR)/build_tools/make_linker_version_script libasteriskssl "$(LINKER_SYMBOL_PREFIX)" "$(ASTTOPDIR)"
 endif
 	$(ECHO_PREFIX) echo "   [LD] $^ -> $@"
-	$(CMD_PREFIX) $(CC) $(STATIC_BUILD) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
+	$(CMD_PREFIX) $(CC) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
 
 $(ASTSSL_LIB): $(ASTSSL_LIB).$(ASTSSL_SO_VERSION)
 	$(ECHO_PREFIX) echo "   [LN] $< -> $@"
@@ -219,14 +225,14 @@ ASTSSL_LIB:=libasteriskssl.dylib
 # -install_name allows library to be found if installed somewhere other than
 # /lib or /usr/lib
 $(ASTSSL_LIB): _ASTLDFLAGS+=-dynamiclib -install_name $(ASTLIBDIR)/$(ASTSSL_LIB)
-$(ASTSSL_LIB): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskssl\"
+$(ASTSSL_LIB): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskssl\" -DAST_NOT_MODULE
 $(ASTSSL_LIB): LIBS+=$(ASTSSL_LIBS)
 $(ASTSSL_LIB): SOLINK=$(DYLINK)
 
 # Special rules for building a shared library (not a dynamically loadable module)
 $(ASTSSL_LIB): libasteriskssl.o
 	$(ECHO_PREFIX) echo "   [LD] $^ -> $@"
-	$(CMD_PREFIX) $(CC) $(STATIC_BUILD) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
+	$(CMD_PREFIX) $(CC) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
 endif
 
 endif
@@ -271,7 +277,7 @@ ifeq ($(GNU_LD),1)
 endif
 
 $(ASTPJ_LIB).$(ASTPJ_SO_VERSION): _ASTLDFLAGS+=-Wl,-soname=$(ASTPJ_LIB).$(ASTPJ_SO_VERSION) $(PJ_LDFLAGS)
-$(ASTPJ_LIB).$(ASTPJ_SO_VERSION): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskpj\" $(PJ_CFLAGS)
+$(ASTPJ_LIB).$(ASTPJ_SO_VERSION): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskpj\" -DAST_NOT_MODULE $(PJ_CFLAGS)
 $(ASTPJ_LIB).$(ASTPJ_SO_VERSION): LIBS+=$(PJPROJECT_LDLIBS) $(OPENSSL_LIB) $(UUID_LIB) -lm -lpthread $(RT_LIB)
 ifeq ($(GNU_LD),1)
     $(ASTPJ_LIB).$(ASTPJ_SO_VERSION): SO_SUPPRESS_SYMBOLS=-Wl,--version-script,libasteriskpj.exports,--warn-common
@@ -284,7 +290,7 @@ $(ASTPJ_LIB).$(ASTPJ_SO_VERSION): SOLINK
 # type that are built by the standard rules)
 $(ASTPJ_LIB).$(ASTPJ_SO_VERSION): libasteriskpj.o libasteriskpj.exports
 	$(ECHO_PREFIX) echo "   [LD] $< -> $@"
-	$(CMD_PREFIX) $(CC) $(STATIC_BUILD) -o $@ $(CC_LDFLAGS_SO) $< $(CC_LIBS)
+	$(CMD_PREFIX) $(CC) -o $@ $(CC_LDFLAGS_SO) $< $(CC_LIBS)
 
 $(ASTPJ_LIB): $(ASTPJ_LIB).$(ASTPJ_SO_VERSION)
 	$(ECHO_PREFIX) echo "   [LN] $< -> $@"
@@ -296,24 +302,22 @@ ASTPJ_LIB:=libasteriskpj.dylib
 # -install_name allows library to be found if installed somewhere other than
 # /lib or /usr/lib
 $(ASTPJ_LIB): _ASTLDFLAGS+=-dynamiclib -install_name $(ASTLIBDIR)/$(ASTPJ_LIB) $(PJ_LDFLAGS)
-$(ASTPJ_LIB): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskpj\" $(PJ_CFLAGS)
+$(ASTPJ_LIB): _ASTCFLAGS+=-fPIC -DAST_MODULE=\"asteriskpj\" $(PJ_CFLAGS) -DAST_NOT_MODULE
 $(ASTPJ_LIB): LIBS+=$(PJPROJECT_LIB) $(OPENSSL_LIB) $(UUID_LIB) -lm -lpthread $(RT_LIB)
 $(ASTPJ_LIB): SOLINK=$(DYLINK)
 
 # Special rules for building a shared library (not a dynamically loadable module)
 $(ASTPJ_LIB): libasteriskpj.o
 	$(ECHO_PREFIX) echo "   [LD] $^ -> $@"
-	$(CMD_PREFIX) $(CC) $(STATIC_BUILD) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
+	$(CMD_PREFIX) $(CC) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
 endif
 
 endif
 
-tcptls.o: _ASTCFLAGS+=$(OPENSSL_INCLUDE) -Wno-deprecated-declarations
-
-$(MAIN_TGT): $(OBJS) $(ASTSSL_LIB) $(ASTPJ_LIB) $(LIBEDIT_OBJ)
+$(MAIN_TGT): $(OBJS) $(MOD_OBJS) $(ASTSSL_LIB) $(ASTPJ_LIB)
 	@$(CC) -c -o buildinfo.o $(_ASTCFLAGS) buildinfo.c $(ASTCFLAGS)
-	$(ECHO_PREFIX) echo "   [LD] $(OBJS) $(LIBEDIT_OBJ) -> $@"
-	$(CMD_PREFIX) $(CXX) $(STATIC_BUILD) -o $@ $(ASTLINK) $(_ASTLDFLAGS) $(ASTLDFLAGS) $(OBJS) $(ASTSSL_LDLIBS) $(ASTPJ_LDLIBS) $(LIBEDIT_OBJ) buildinfo.o $(AST_LIBS) $(GMIMELDFLAGS) $(LIBEDIT_LIB)
+	$(ECHO_PREFIX) echo "   [LD] $(OBJS) $(MOD_OBJS) -> $@"
+	$(CMD_PREFIX) $(CXX) -o $@ $(ASTLINK) $(_ASTLDFLAGS) $(ASTLDFLAGS) $(OBJS) $(MOD_OBJS) $(ASTSSL_LDLIBS) $(ASTPJ_LDLIBS) buildinfo.o $(AST_LIBS) $(GMIMELDFLAGS) $(LIBEDIT_LIB)
 
 ifeq ($(GNU_LD),1)
 $(MAIN_TGT): asterisk.exports
@@ -357,7 +361,6 @@ endif
 	rm -f .libasteriskpj*
 
 	rm -f asterisk.exports libasteriskssl.exports libasteriskpj.exports
-	@if [ -f editline/Makefile ]; then $(MAKE) -C editline distclean ; fi
 	@$(MAKE) -C stdtime clean
 	rm -f libresample/src/*.o
 	rm -f *.tmp
diff -pruN 1:13.23.1~dfsg-2/main/manager_bridges.c 1:16.1.1~dfsg-1/main/manager_bridges.c
--- 1:13.23.1~dfsg-2/main/manager_bridges.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager_bridges.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis_bridges.h"
 #include "asterisk/stasis_channels.h"
 #include "asterisk/manager.h"
@@ -664,7 +662,7 @@ static int manager_bridge_kick(struct ma
 		}
 	} else {
 		bridge = ast_bridge_find_by_id(bridge_uniqueid);
-		if (!bridge) {
+		if (!bridge || ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
 			astman_send_error(s, m, "Bridge not found");
 			return 0;
 		}
diff -pruN 1:13.23.1~dfsg-2/main/manager.c 1:16.1.1~dfsg-1/main/manager.c
--- 1:13.23.1~dfsg-2/main/manager.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager.c	2018-12-26 18:45:13.000000000 +0000
@@ -54,8 +54,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/paths.h"	/* use various ast_config_AST_* */
 #include <ctype.h>
 #include <sys/time.h>
@@ -223,6 +221,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			<parameter name="Variables">
 				<para>Comma <literal>,</literal> separated list of variable to include.</para>
 			</parameter>
+			<parameter name="AllVariables">
+				<para>If set to "true", the Status event will include all channel variables for
+				the requested channel(s).</para>
+				<enumlist>
+					<enum name="true"/>
+					<enum name="false"/>
+				</enumlist>
+			</parameter>
 		</syntax>
 		<description>
 			<para>Will return the status information of each channel along with the
@@ -1521,6 +1527,8 @@ static void acl_change_stasis_subscribe(
 	if (!acl_change_sub) {
 		acl_change_sub = stasis_subscribe(ast_security_topic(),
 			acl_change_stasis_cb, NULL);
+		stasis_subscription_accept_message_type(acl_change_sub, ast_named_acl_change_type());
+		stasis_subscription_set_filter(acl_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 }
 
@@ -1564,8 +1572,7 @@ static void acl_change_stasis_unsubscrib
 struct mansession_session {
 				/*! \todo XXX need to document which fields it is protecting */
 	struct ast_sockaddr addr;	/*!< address we are connecting from */
-	FILE *f;		/*!< fdopen() on the underlying fd */
-	int fd;			/*!< descriptor used for output. Either the socket (AMI) or a temporary file (HTTP) */
+	struct ast_iostream *stream;	/*!< AMI stream */
 	int inuse;		/*!< number of HTTP sessions using this entry */
 	int needdestroy;	/*!< Whether an HTTP session should be destroyed */
 	pthread_t waiting_thread;	/*!< Sleeping thread using this descriptor */
@@ -1607,9 +1614,8 @@ enum mansession_message_parsing {
  */
 struct mansession {
 	struct mansession_session *session;
+	struct ast_iostream *stream;
 	struct ast_tcptls_session_instance *tcptls_session;
-	FILE *f;
-	int fd;
 	enum mansession_message_parsing parsing;
 	int write_error:1;
 	struct manager_custom_hook *hook;
@@ -2191,10 +2197,6 @@ static void session_destructor(void *obj
 		ast_datastore_free(datastore);
 	}
 
-	if (session->f != NULL) {
-		fflush(session->f);
-		fclose(session->f);
-	}
 	if (eqe) {
 		ast_atomic_fetchadd_int(&eqe->usecount, -1);
 	}
@@ -2222,14 +2224,13 @@ static struct mansession_session *build_
 		return NULL;
 	}
 
-	newsession->whitefilters = ao2_container_alloc(1, NULL, NULL);
-	newsession->blackfilters = ao2_container_alloc(1, NULL, NULL);
+	newsession->whitefilters = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
+	newsession->blackfilters = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
 	if (!newsession->whitefilters || !newsession->blackfilters) {
 		ao2_ref(newsession, -1);
 		return NULL;
 	}
 
-	newsession->fd = -1;
 	newsession->waiting_thread = AST_PTHREADT_NULL;
 	newsession->writetimeout = 100;
 	newsession->send_events = -1;
@@ -2668,7 +2669,7 @@ static char *handle_showmanconn(struct a
 				ast_sockaddr_stringify_addr(&session->addr),
 				(int) (session->sessionstart),
 				(int) (now - session->sessionstart),
-				session->fd,
+				session->stream ? ast_iostream_get_fd(session->stream) : -1,
 				session->inuse,
 				session->readperm,
 				session->writeperm);
@@ -2710,6 +2711,8 @@ static char *handle_showmaneventq(struct
 	return CLI_SUCCESS;
 }
 
+static int reload_module(void);
+
 /*! \brief CLI command manager reload */
 static char *handle_manager_reload(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
@@ -2726,7 +2729,7 @@ static char *handle_manager_reload(struc
 	if (a->argc > 2) {
 		return CLI_SHOWUSAGE;
 	}
-	reload_manager();
+	reload_module();
 	return CLI_SUCCESS;
 }
 
@@ -2959,49 +2962,53 @@ int ast_hook_send_action(struct manager_
 	}
 
 	action = astman_get_header(&m, "Action");
-	if (strcasecmp(action, "login")) {
+
+	do {
+		if (!strcasecmp(action, "login")) {
+			break;
+		}
+
 		act_found = action_find(action);
-		if (act_found) {
-			/*
-			 * we have to simulate a session for this action request
-			 * to be able to pass it down for processing
-			 * This is necessary to meet the previous design of manager.c
-			 */
-			s.hook = hook;
-			s.f = (void*)1; /* set this to something so our request will make it through all functions that test it*/
+		if (!act_found) {
+			break;
+		}
 
-			ao2_lock(act_found);
-			if (act_found->registered && act_found->func) {
-				if (act_found->module) {
-					ast_module_ref(act_found->module);
-				}
-				ao2_unlock(act_found);
+		/*
+		 * we have to simulate a session for this action request
+		 * to be able to pass it down for processing
+		 * This is necessary to meet the previous design of manager.c
+		 */
+		s.hook = hook;
+
+		ret = -1;
+		ao2_lock(act_found);
+		if (act_found->registered && act_found->func) {
+			struct ast_module *mod_ref = ast_module_running_ref(act_found->module);
+
+			ao2_unlock(act_found);
+			/* If the action is in a module it must be running. */
+			if (!act_found->module || mod_ref) {
 				ret = act_found->func(&s, &m);
-				ao2_lock(act_found);
-				if (act_found->module) {
-					ast_module_unref(act_found->module);
-				}
-			} else {
-				ret = -1;
+				ast_module_unref(mod_ref);
 			}
+		} else {
 			ao2_unlock(act_found);
-			ao2_t_ref(act_found, -1, "done with found action object");
 		}
-	}
+		ao2_t_ref(act_found, -1, "done with found action object");
+	} while (0);
+
 	ast_free(dup_str);
 	return ret;
 }
 
-
 /*!
  * helper function to send a string to the socket.
  * Return -1 on error (e.g. buffer full).
  */
 static int send_string(struct mansession *s, char *string)
 {
-	int res;
-	FILE *f = s->f ? s->f : s->session->f;
-	int fd = s->f ? s->fd : s->session->fd;
+	struct ast_iostream *stream;
+	int len, res;
 
 	/* It's a result from one of the hook's action invocation */
 	if (s->hook) {
@@ -3013,7 +3020,14 @@ static int send_string(struct mansession
 		return 0;
 	}
 
-	if ((res = ast_careful_fwrite(f, fd, string, strlen(string), s->session->writetimeout))) {
+	stream = s->stream ? s->stream : s->session->stream;
+
+	len = strlen(string);
+	ast_iostream_set_timeout_inactivity(stream, s->session->writetimeout);
+	res = ast_iostream_write(stream, string, len);
+	ast_iostream_set_timeout_disable(stream);
+
+	if (res < len) {
 		s->write_error = 1;
 	}
 
@@ -3054,10 +3068,10 @@ void astman_append(struct mansession *s,
 		return;
 	}
 
-	if (s->f != NULL || s->session->f != NULL) {
+	if (s->hook || (s->tcptls_session != NULL && s->tcptls_session->stream != NULL)) {
 		send_string(s, ast_str_buffer(buf));
 	} else {
-		ast_verbose("fd == -1 in astman_append, should not happen\n");
+		ast_verbose("No connection stream in astman_append, should not happen\n");
 	}
 }
 
@@ -4198,7 +4212,7 @@ static int action_waitevent(struct manse
 			break;
 		}
 		if (s->session->managerid == 0) {	/* AMI session */
-			if (ast_wait_for_input(s->session->fd, 1000)) {
+			if (ast_wait_for_input(ast_iostream_get_fd(s->session->stream), 1000)) {
 				break;
 			}
 		} else {	/* HTTP session */
@@ -4326,10 +4340,26 @@ static int action_login(struct mansessio
 		&& ast_test_flag(&ast_options, AST_OPT_FLAG_FULLY_BOOTED)) {
 		struct ast_str *auth = ast_str_alloca(MAX_AUTH_PERM_STRING);
 		const char *cat_str = authority_to_str(EVENT_FLAG_SYSTEM, &auth);
+		long uptime = 0;
+		long lastreloaded = 0;
+		struct timeval tmp;
+		struct timeval curtime = ast_tvnow();
+
+		if (ast_startuptime.tv_sec) {
+			tmp = ast_tvsub(curtime, ast_startuptime);
+			uptime = tmp.tv_sec;
+		}
+
+		if (ast_lastreloadtime.tv_sec) {
+			tmp = ast_tvsub(curtime, ast_lastreloadtime);
+			lastreloaded = tmp.tv_sec;
+		}
 
 		astman_append(s, "Event: FullyBooted\r\n"
 			"Privilege: %s\r\n"
-			"Status: Fully Booted\r\n\r\n", cat_str);
+			"Uptime: %ld\r\n"
+			"LastReload: %ld\r\n"
+			"Status: Fully Booted\r\n\r\n", cat_str, uptime, lastreloaded);
 	}
 	return 0;
 }
@@ -4555,19 +4585,130 @@ static int action_getvar(struct mansessi
 	return 0;
 }
 
+static void generate_status(struct mansession *s, struct ast_channel *chan, char **vars, int varc, int all_variables, char *id_text, int *count)
+{
+	struct timeval now;
+	long elapsed_seconds;
+	struct ast_bridge *bridge;
+	RAII_VAR(struct ast_str *, variable_str, NULL, ast_free);
+	struct ast_str *write_transpath = ast_str_alloca(256);
+	struct ast_str *read_transpath = ast_str_alloca(256);
+	struct ast_str *codec_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
+	struct ast_party_id effective_id;
+	int i;
+	RAII_VAR(struct ast_channel_snapshot *, snapshot,
+		ast_channel_snapshot_get_latest(ast_channel_uniqueid(chan)),
+		ao2_cleanup);
+	RAII_VAR(struct ast_str *, snapshot_str, NULL, ast_free);
+
+	if (!snapshot) {
+		return;
+	}
+
+	snapshot_str = ast_manager_build_channel_state_string(snapshot);
+	if (!snapshot_str) {
+		return;
+	}
+
+	if (all_variables) {
+		variable_str = ast_str_create(2048);
+	} else {
+		variable_str = ast_str_create(1024);
+	}
+	if (!variable_str) {
+		return;
+	}
+
+	now = ast_tvnow();
+	elapsed_seconds = ast_tvdiff_sec(now, ast_channel_creationtime(chan));
+
+	/* Even if all_variables has been specified, explicitly requested variables
+	 * may be global variables or dialplan functions */
+	for (i = 0; i < varc; i++) {
+		char valbuf[512], *ret = NULL;
+
+		if (vars[i][strlen(vars[i]) - 1] == ')') {
+			if (ast_func_read(chan, vars[i], valbuf, sizeof(valbuf)) < 0) {
+				valbuf[0] = '\0';
+			}
+			ret = valbuf;
+		} else {
+			pbx_retrieve_variable(chan, vars[i], &ret, valbuf, sizeof(valbuf), NULL);
+		}
+
+		ast_str_append(&variable_str, 0, "Variable: %s=%s\r\n", vars[i], ret);
+	}
+
+	/* Walk all channel variables and add them */
+	if (all_variables) {
+		struct ast_var_t *variables;
+
+		AST_LIST_TRAVERSE(ast_channel_varshead(chan), variables, entries) {
+			ast_str_append(&variable_str, 0, "Variable: %s=%s\r\n",
+				ast_var_name(variables), ast_var_value(variables));
+		}
+	}
+
+	bridge = ast_channel_get_bridge(chan);
+	effective_id = ast_channel_connected_effective_id(chan);
+
+	astman_append(s,
+		"Event: Status\r\n"
+		"Privilege: Call\r\n"
+		"%s"
+		"Type: %s\r\n"
+		"DNID: %s\r\n"
+		"EffectiveConnectedLineNum: %s\r\n"
+		"EffectiveConnectedLineName: %s\r\n"
+		"TimeToHangup: %ld\r\n"
+		"BridgeID: %s\r\n"
+		"Application: %s\r\n"
+		"Data: %s\r\n"
+		"Nativeformats: %s\r\n"
+		"Readformat: %s\r\n"
+		"Readtrans: %s\r\n"
+		"Writeformat: %s\r\n"
+		"Writetrans: %s\r\n"
+		"Callgroup: %llu\r\n"
+		"Pickupgroup: %llu\r\n"
+		"Seconds: %ld\r\n"
+		"%s"
+		"%s"
+		"\r\n",
+		ast_str_buffer(snapshot_str),
+		ast_channel_tech(chan)->type,
+		S_OR(ast_channel_dialed(chan)->number.str, ""),
+		S_COR(effective_id.number.valid, effective_id.number.str, "<unknown>"),
+		S_COR(effective_id.name.valid, effective_id.name.str, "<unknown>"),
+		(long)ast_channel_whentohangup(chan)->tv_sec,
+		bridge ? bridge->uniqueid : "",
+		ast_channel_appl(chan),
+		ast_channel_data(chan),
+		ast_format_cap_get_names(ast_channel_nativeformats(chan), &codec_buf),
+		ast_format_get_name(ast_channel_readformat(chan)),
+		ast_translate_path_to_str(ast_channel_readtrans(chan), &read_transpath),
+		ast_format_get_name(ast_channel_writeformat(chan)),
+		ast_translate_path_to_str(ast_channel_writetrans(chan), &write_transpath),
+		ast_channel_callgroup(chan),
+		ast_channel_pickupgroup(chan),
+		(long)elapsed_seconds,
+		ast_str_buffer(variable_str),
+		id_text);
+	++*count;
+
+	ao2_cleanup(bridge);
+}
+
 /*! \brief Manager "status" command to show channels */
-/* Needs documentation... */
 static int action_status(struct mansession *s, const struct message *m)
 {
 	const char *name = astman_get_header(m, "Channel");
 	const char *chan_variables = astman_get_header(m, "Variables");
+	const char *all_chan_variables = astman_get_header(m, "AllVariables");
+	int all_variables = 0;
 	const char *id = astman_get_header(m, "ActionID");
 	char *variables = ast_strdupa(S_OR(chan_variables, ""));
-	struct ast_str *variable_str = ast_str_create(1024);
-	struct ast_str *write_transpath = ast_str_alloca(256);
-	struct ast_str *read_transpath = ast_str_alloca(256);
 	struct ast_channel *chan;
-	struct ast_str *codec_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
 	int channels = 0;
 	int all = ast_strlen_zero(name); /* set if we want all channels */
 	char id_text[256];
@@ -4576,20 +4717,17 @@ static int action_status(struct mansessi
 		AST_APP_ARG(name)[100];
 	);
 
-	if (!variable_str) {
-		astman_send_error(s, m, "Memory Allocation Failure");
-		return 1;
+	if (!ast_strlen_zero(all_chan_variables)) {
+		all_variables = ast_true(all_chan_variables);
 	}
 
 	if (!(function_capable_string_allowed_with_auths(variables, s->session->writeperm))) {
-		ast_free(variable_str);
 		astman_send_error(s, m, "Status Access Forbidden: Variables");
 		return 0;
 	}
 
 	if (all) {
 		if (!(it_chans = ast_channel_iterator_all_new())) {
-			ast_free(variable_str);
 			astman_send_error(s, m, "Memory Allocation Failure");
 			return 1;
 		}
@@ -4598,7 +4736,6 @@ static int action_status(struct mansessi
 		chan = ast_channel_get_by_name(name);
 		if (!chan) {
 			astman_send_error(s, m, "No such channel");
-			ast_free(variable_str);
 			return 0;
 		}
 	}
@@ -4617,112 +4754,9 @@ static int action_status(struct mansessi
 
 	/* if we look by name, we break after the first iteration */
 	for (; chan; all ? chan = ast_channel_iterator_next(it_chans) : 0) {
-		struct timeval now;
-		long elapsed_seconds;
-		struct ast_bridge *bridge;
-		struct ast_party_id effective_id;
-
 		ast_channel_lock(chan);
 
-		now = ast_tvnow();
-		elapsed_seconds = ast_tvdiff_sec(now, ast_channel_creationtime(chan));
-
-		if (!ast_strlen_zero(chan_variables)) {
-			int i;
-			ast_str_reset(variable_str);
-			for (i = 0; i < vars.argc; i++) {
-				char valbuf[512], *ret = NULL;
-
-				if (vars.name[i][strlen(vars.name[i]) - 1] == ')') {
-					if (ast_func_read(chan, vars.name[i], valbuf, sizeof(valbuf)) < 0) {
-						valbuf[0] = '\0';
-					}
-					ret = valbuf;
-				} else {
-					pbx_retrieve_variable(chan, vars.name[i], &ret, valbuf, sizeof(valbuf), NULL);
-				}
-
-				ast_str_append(&variable_str, 0, "Variable: %s=%s\r\n", vars.name[i], ret);
-			}
-		}
-
-		channels++;
-
-		bridge = ast_channel_get_bridge(chan);
-		effective_id = ast_channel_connected_effective_id(chan);
-
-		astman_append(s,
-			"Event: Status\r\n"
-			"Privilege: Call\r\n"
-			/* v-- Start channel snapshot headers */
-			"Channel: %s\r\n"
-			"ChannelState: %u\r\n"
-			"ChannelStateDesc: %s\r\n"
-			"CallerIDNum: %s\r\n"
-			"CallerIDName: %s\r\n"
-			"ConnectedLineNum: %s\r\n"
-			"ConnectedLineName: %s\r\n"
-			"Accountcode: %s\r\n"
-			"Context: %s\r\n"
-			"Exten: %s\r\n"
-			"Priority: %d\r\n"
-			"Uniqueid: %s\r\n"
-			"Linkedid: %s\r\n"
-			/* ^-- End channel snapshot headers */
-			"Type: %s\r\n"
-			"DNID: %s\r\n"
-			"EffectiveConnectedLineNum: %s\r\n"
-			"EffectiveConnectedLineName: %s\r\n"
-			"TimeToHangup: %ld\r\n"
-			"BridgeID: %s\r\n"
-			"Application: %s\r\n"
-			"Data: %s\r\n"
-			"Nativeformats: %s\r\n"
-			"Readformat: %s\r\n"
-			"Readtrans: %s\r\n"
-			"Writeformat: %s\r\n"
-			"Writetrans: %s\r\n"
-			"Callgroup: %llu\r\n"
-			"Pickupgroup: %llu\r\n"
-			"Seconds: %ld\r\n"
-			"%s"
-			"%s"
-			"\r\n",
-			/* v-- Start channel snapshot headers */
-			ast_channel_name(chan),
-			ast_channel_state(chan),
-			ast_state2str(ast_channel_state(chan)),
-			S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, "<unknown>"),
-			S_COR(ast_channel_caller(chan)->id.name.valid, ast_channel_caller(chan)->id.name.str, "<unknown>"),
-			S_COR(ast_channel_connected(chan)->id.number.valid, ast_channel_connected(chan)->id.number.str, "<unknown>"),
-			S_COR(ast_channel_connected(chan)->id.name.valid, ast_channel_connected(chan)->id.name.str, "<unknown>"),
-			ast_channel_accountcode(chan),
-			ast_channel_context(chan),
-			ast_channel_exten(chan),
-			ast_channel_priority(chan),
-			ast_channel_uniqueid(chan),
-			ast_channel_linkedid(chan),
-			/* ^-- End channel snapshot headers */
-			ast_channel_tech(chan)->type,
-			S_OR(ast_channel_dialed(chan)->number.str, ""),
-			S_COR(effective_id.number.valid, effective_id.number.str, "<unknown>"),
-			S_COR(effective_id.name.valid, effective_id.name.str, "<unknown>"),
-			(long)ast_channel_whentohangup(chan)->tv_sec,
-			bridge ? bridge->uniqueid : "",
-			ast_channel_appl(chan),
-			ast_channel_data(chan),
-			ast_format_cap_get_names(ast_channel_nativeformats(chan), &codec_buf),
-			ast_format_get_name(ast_channel_readformat(chan)),
-			ast_translate_path_to_str(ast_channel_readtrans(chan), &read_transpath),
-			ast_format_get_name(ast_channel_writeformat(chan)),
-			ast_translate_path_to_str(ast_channel_writetrans(chan), &write_transpath),
-			ast_channel_callgroup(chan),
-			ast_channel_pickupgroup(chan),
-			(long)elapsed_seconds,
-			ast_str_buffer(variable_str),
-			id_text);
-
-		ao2_cleanup(bridge);
+		generate_status(s, chan, vars.name, vars.argc, all_variables, id_text, &channels);
 
 		ast_channel_unlock(chan);
 		chan = ast_channel_unref(chan);
@@ -4736,8 +4770,6 @@ static int action_status(struct mansessi
 	astman_append(s, "Items: %d\r\n", channels);
 	astman_send_list_complete_end(s);
 
-	ast_free(variable_str);
-
 	return 0;
 }
 
@@ -5120,11 +5152,10 @@ static int check_blacklist(const char *c
 static int action_command(struct mansession *s, const struct message *m)
 {
 	const char *cmd = astman_get_header(m, "Command");
-	const char *id = astman_get_header(m, "ActionID");
-	char *buf = NULL, *final_buf = NULL;
+	char *buf = NULL, *final_buf = NULL, *delim, *output;
 	char template[] = "/tmp/ast-ami-XXXXXX";	/* template for temporary file */
-	int fd;
-	off_t l;
+	int fd, ret;
+	off_t len;
 
 	if (ast_strlen_zero(cmd)) {
 		astman_send_error(s, m, "No command provided");
@@ -5137,52 +5168,59 @@ static int action_command(struct mansess
 	}
 
 	if ((fd = mkstemp(template)) < 0) {
-		ast_log(AST_LOG_WARNING, "Failed to create temporary file for command: %s\n", strerror(errno));
-		astman_send_error(s, m, "Command response construction error");
+		astman_send_error_va(s, m, "Failed to create temporary file: %s", strerror(errno));
 		return 0;
 	}
 
-	astman_append(s, "Response: Follows\r\nPrivilege: Command\r\n");
-	if (!ast_strlen_zero(id)) {
-		astman_append(s, "ActionID: %s\r\n", id);
-	}
-	/* FIXME: Wedge a ActionID response in here, waiting for later changes */
-	ast_cli_command(fd, cmd);	/* XXX need to change this to use a FILE * */
+	ret = ast_cli_command(fd, cmd);
+	astman_send_response_full(s, m, ret == RESULT_SUCCESS ? "Success" : "Error", MSG_MOREDATA, NULL);
+
 	/* Determine number of characters available */
-	if ((l = lseek(fd, 0, SEEK_END)) < 0) {
-		ast_log(LOG_WARNING, "Failed to determine number of characters for command: %s\n", strerror(errno));
+	if ((len = lseek(fd, 0, SEEK_END)) < 0) {
+		astman_append(s, "Message: Failed to determine number of characters: %s\r\n", strerror(errno));
 		goto action_command_cleanup;
 	}
 
 	/* This has a potential to overflow the stack.  Hence, use the heap. */
-	buf = ast_malloc(l + 1);
-	final_buf = ast_malloc(l + 1);
+	buf = ast_malloc(len + 1);
+	final_buf = ast_malloc(len + 1);
 
 	if (!buf || !final_buf) {
-		ast_log(LOG_WARNING, "Failed to allocate memory for temporary buffer\n");
+		astman_append(s, "Message: Memory allocation failure\r\n");
 		goto action_command_cleanup;
 	}
 
 	if (lseek(fd, 0, SEEK_SET) < 0) {
-		ast_log(LOG_WARNING, "Failed to set position on temporary file for command: %s\n", strerror(errno));
+		astman_append(s, "Message: Failed to set position on temporary file: %s\r\n", strerror(errno));
 		goto action_command_cleanup;
 	}
 
-	if (read(fd, buf, l) < 0) {
-		ast_log(LOG_WARNING, "read() failed: %s\n", strerror(errno));
+	if (read(fd, buf, len) < 0) {
+		astman_append(s, "Message: Failed to read from temporary file: %s\r\n", strerror(errno));
 		goto action_command_cleanup;
 	}
 
-	buf[l] = '\0';
-	term_strip(final_buf, buf, l);
-	final_buf[l] = '\0';
-	astman_append(s, "%s", final_buf);
+	buf[len] = '\0';
+	term_strip(final_buf, buf, len);
+	final_buf[len] = '\0';
+
+	/* Trim trailing newline */
+	if (len && final_buf[len - 1] == '\n') {
+		final_buf[len - 1] = '\0';
+	}
+
+	astman_append(s, "Message: Command output follows\r\n");
+
+	delim = final_buf;
+	while ((output = strsep(&delim, "\n"))) {
+		astman_append(s, "Output: %s\r\n", output);
+	}
 
 action_command_cleanup:
+	astman_append(s, "\r\n");
 
 	close(fd);
 	unlink(template);
-	astman_append(s, "--END COMMAND--\r\n\r\n");
 
 	ast_free(buf);
 	ast_free(final_buf);
@@ -6047,7 +6085,7 @@ static int process_events(struct mansess
 	int ret = 0;
 
 	ao2_lock(s->session);
-	if (s->session->f != NULL) {
+	if (s->session->stream != NULL) {
 		struct eventqent *eqe = s->session->last_ev;
 
 		while ((eqe = advance_event(eqe))) {
@@ -6321,7 +6359,7 @@ static int manager_modulecheck(struct ma
 	}
 	astman_append(s, "Response: Success\r\n%s", idText);
 #if !defined(LOW_MEMORY)
-	astman_append(s, "Version: %s\r\n\r\n", ast_get_version());
+	astman_append(s, "Version: %s\r\n\r\n", "");
 #endif
 	return 0;
 }
@@ -6469,21 +6507,21 @@ static int process_message(struct manses
 		if ((s->session->writeperm & act_found->authority)
 			|| act_found->authority == 0) {
 			/* We have the authority to execute the action. */
+			ret = -1;
 			ao2_lock(act_found);
 			if (act_found->registered && act_found->func) {
-				ast_debug(1, "Running action '%s'\n", act_found->action);
-				if (act_found->module) {
-					ast_module_ref(act_found->module);
-				}
+				struct ast_module *mod_ref = ast_module_running_ref(act_found->module);
+
 				ao2_unlock(act_found);
-				ret = act_found->func(s, m);
-				acted = 1;
-				ao2_lock(act_found);
-				if (act_found->module) {
-					ast_module_unref(act_found->module);
+				if (mod_ref || !act_found->module) {
+					ast_debug(1, "Running action '%s'\n", act_found->action);
+					ret = act_found->func(s, m);
+					acted = 1;
+					ast_module_unref(mod_ref);
 				}
+			} else {
+				ao2_unlock(act_found);
 			}
-			ao2_unlock(act_found);
 		}
 		if (!acted) {
 			/*
@@ -6557,9 +6595,11 @@ static int get_input(struct mansession *
 		return 1;
 	}
 	if (s->session->inlen >= maxlen) {
-		/* no crlf found, and buffer full - sorry, too long for us */
+		/* no crlf found, and buffer full - sorry, too long for us
+		 * keep the last character in case we are in the middle of a CRLF. */
 		ast_log(LOG_WARNING, "Discarding message from %s. Line too long: %.25s...\n", ast_sockaddr_stringify_addr(&s->session->addr), src);
-		s->session->inlen = 0;
+		src[0] = src[s->session->inlen - 1];
+		s->session->inlen = 1;
 		s->parsing = MESSAGE_LINE_TOO_LONG;
 	}
 	res = 0;
@@ -6587,7 +6627,7 @@ static int get_input(struct mansession *
 		s->session->waiting_thread = pthread_self();
 		ao2_unlock(s->session);
 
-		res = ast_wait_for_input(s->session->fd, timeout);
+		res = ast_wait_for_input(ast_iostream_get_fd(s->session->stream), timeout);
 
 		ao2_lock(s->session);
 		s->session->waiting_thread = AST_PTHREADT_NULL;
@@ -6605,13 +6645,7 @@ static int get_input(struct mansession *
 	}
 
 	ao2_lock(s->session);
-	/*
-	 * It is worth noting here that you can all but ignore fread()'s documentation
-	 * for the purposes of this call. The FILE * we are working with here was created
-	 * as a result of a call to fopencookie() (or equivalent) in tcptls.c, and as such
-	 * the behavior of fread() is not as documented. Frankly, I think this is gross.
-	 */
-	res = fread(src + s->session->inlen, 1, maxlen - s->session->inlen, s->session->f);
+	res = ast_iostream_read(s->session->stream, src + s->session->inlen, maxlen - s->session->inlen);
 	if (res < 1) {
 		res = -1;	/* error return */
 	} else {
@@ -6741,12 +6775,11 @@ static void *session_do(void *data)
 	struct mansession s = {
 		.tcptls_session = data,
 	};
-	int flags = 1;
 	int res;
+	int arg = 1;
 	struct ast_sockaddr ser_remote_address_tmp;
 
 	if (ast_atomic_fetchadd_int(&unauth_sessions, +1) >= authlimit) {
-		fclose(ser->f);
 		ast_atomic_fetchadd_int(&unauth_sessions, -1);
 		goto done;
 	}
@@ -6755,7 +6788,6 @@ static void *session_do(void *data)
 	session = build_mansession(&ser_remote_address_tmp);
 
 	if (session == NULL) {
-		fclose(ser->f);
 		ast_atomic_fetchadd_int(&unauth_sessions, -1);
 		goto done;
 	}
@@ -6763,12 +6795,10 @@ static void *session_do(void *data)
 	/* here we set TCP_NODELAY on the socket to disable Nagle's algorithm.
 	 * This is necessary to prevent delays (caused by buffering) as we
 	 * write to the socket in bits and pieces. */
-	if (setsockopt(ser->fd, IPPROTO_TCP, TCP_NODELAY, (char *) &flags, sizeof(flags)) < 0) {
+	if (setsockopt(ast_iostream_get_fd(ser->stream), IPPROTO_TCP, TCP_NODELAY, (char *) &arg, sizeof(arg)) < 0) {
 		ast_log(LOG_WARNING, "Failed to set TCP_NODELAY on manager connection: %s\n", strerror(errno));
 	}
-
-	/* make sure socket is non-blocking */
-	ast_fd_set_flags(ser->fd, O_NONBLOCK);
+	ast_iostream_nonblock(ser->stream);
 
 	ao2_lock(session);
 	/* Hook to the tail of the event queue */
@@ -6777,8 +6807,7 @@ static void *session_do(void *data)
 	ast_mutex_init(&s.lock);
 
 	/* these fields duplicate those in the 'ser' structure */
-	session->fd = s.fd = ser->fd;
-	session->f = s.f = ser->f;
+	session->stream = s.stream = ser->stream;
 	ast_sockaddr_copy(&session->addr, &ser_remote_address_tmp);
 	s.session = session;
 
@@ -6797,9 +6826,9 @@ static void *session_do(void *data)
 	 * We cannot let the stream exclusively wait for data to arrive.
 	 * We have to wake up the task to send async events.
 	 */
-	ast_tcptls_stream_set_exclusive_input(ser->stream_cookie, 0);
+	ast_iostream_set_exclusive_input(ser->stream, 0);
 
-	ast_tcptls_stream_set_timeout_sequence(ser->stream_cookie,
+	ast_iostream_set_timeout_sequence(ser->stream,
 		ast_tvnow(), authtimeout * 1000);
 
 	astman_append(&s, "Asterisk Call Manager/%s\r\n", AMI_VERSION);	/* welcome prompt */
@@ -6808,7 +6837,7 @@ static void *session_do(void *data)
 			break;
 		}
 		if (session->authenticated) {
-			ast_tcptls_stream_set_timeout_disable(ser->stream_cookie);
+			ast_iostream_set_timeout_disable(ser->stream);
 		}
 	}
 	/* session is over, explain why and terminate */
@@ -7090,7 +7119,7 @@ int ast_manager_unregister(const char *a
 	return 0;
 }
 
-static int manager_state_cb(char *context, char *exten, struct ast_state_cb_info *info, void *data)
+static int manager_state_cb(const char *context, const char *exten, struct ast_state_cb_info *info, void *data)
 {
 	/* Notify managers of change */
 	char hint[512];
@@ -7610,7 +7639,8 @@ static void xml_translate(struct ast_str
 			if (xml) {
 				ast_str_append(out, 0, "<response type='object' id='%s'><%s", dest, objtype);
 			}
-			vco = ao2_container_alloc(37, variable_count_hash_fn, variable_count_cmp_fn);
+			vco = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+				variable_count_hash_fn, NULL, variable_count_cmp_fn);
 			inobj = 1;
 		}
 
@@ -7668,23 +7698,9 @@ static void xml_translate(struct ast_str
 
 static void close_mansession_file(struct mansession *s)
 {
-	if (s->f) {
-		if (fclose(s->f)) {
-			ast_log(LOG_ERROR, "fclose() failed: %s\n", strerror(errno));
-		}
-		s->f = NULL;
-		s->fd = -1;
-	} else if (s->fd != -1) {
-		/*
-		 * Issuing shutdown() is necessary here to avoid a race
-		 * condition where the last data written may not appear
-		 * in the TCP stream.  See ASTERISK-23548
-		 */
-		shutdown(s->fd, SHUT_RDWR);
-		if (close(s->fd)) {
-			ast_log(LOG_ERROR, "close() failed: %s\n", strerror(errno));
-		}
-		s->fd = -1;
+	if (s->stream) {
+		ast_iostream_close(s->stream);
+		s->stream = NULL;
 	} else {
 		ast_log(LOG_ERROR, "Attempted to close file/file descriptor on mansession without a valid file or file descriptor.\n");
 	}
@@ -7693,17 +7709,20 @@ static void close_mansession_file(struct
 static void process_output(struct mansession *s, struct ast_str **out, struct ast_variable *params, enum output_format format)
 {
 	char *buf;
-	size_t l;
+	off_t l;
+	int fd;
 
-	if (!s->f)
+	if (!s->stream)
 		return;
 
 	/* Ensure buffer is NULL-terminated */
-	fprintf(s->f, "%c", 0);
-	fflush(s->f);
+	ast_iostream_write(s->stream, "", 1);
+
+	fd = ast_iostream_get_fd(s->stream);
 
-	if ((l = ftell(s->f)) > 0) {
-		if (MAP_FAILED == (buf = mmap(NULL, l, PROT_READ | PROT_WRITE, MAP_PRIVATE, s->fd, 0))) {
+	l = lseek(fd, 0, SEEK_CUR);
+	if (l > 0) {
+		if (MAP_FAILED == (buf = mmap(NULL, l, PROT_READ | PROT_WRITE, MAP_PRIVATE, fd, 0))) {
 			ast_log(LOG_WARNING, "mmap failed.  Manager output was not processed\n");
 		} else {
 			if (format == FORMAT_XML || format == FORMAT_HTML) {
@@ -7730,6 +7749,7 @@ static int generic_http_callback(struct
 	struct mansession s = { .session = NULL, .tcptls_session = ser };
 	struct mansession_session *session = NULL;
 	uint32_t ident;
+	int fd;
 	int blastaway = 0;
 	struct ast_variable *params = get_params;
 	char template[] = "/tmp/ast-http-XXXXXX";	/* template for temporary file */
@@ -7782,17 +7802,17 @@ static int generic_http_callback(struct
 	}
 
 	s.session = session;
-	s.fd = mkstemp(template);	/* create a temporary file for command output */
+	fd = mkstemp(template);	/* create a temporary file for command output */
 	unlink(template);
-	if (s.fd <= -1) {
+	if (fd <= -1) {
 		ast_http_error(ser, 500, "Server Error", "Internal Server Error (mkstemp failed)");
 		goto generic_callback_out;
 	}
-	s.f = fdopen(s.fd, "w+");
-	if (!s.f) {
+	s.stream = ast_iostream_from_fd(&fd);
+	if (!s.stream) {
 		ast_log(LOG_WARNING, "HTTP Manager, fdopen failed: %s!\n", strerror(errno));
 		ast_http_error(ser, 500, "Server Error", "Internal Server Error (fdopen failed)");
-		close(s.fd);
+		close(fd);
 		goto generic_callback_out;
 	}
 
@@ -7918,9 +7938,9 @@ generic_callback_out:
 		if (blastaway) {
 			session_destroy(session);
 		} else {
-			if (session->f) {
-				fclose(session->f);
-				session->f = NULL;
+			if (session->stream) {
+				ast_iostream_close(session->stream);
+				session->stream = NULL;
 			}
 			unref_mansession(session);
 		}
@@ -7943,6 +7963,7 @@ static int auth_http_callback(struct ast
 	struct ast_str *http_header = NULL, *out = NULL;
 	size_t result_size;
 	struct message m = { 0 };
+	int fd;
 
 	time_t time_now = time(NULL);
 	unsigned long nonce = 0, nc;
@@ -8129,17 +8150,17 @@ static int auth_http_callback(struct ast
 
 	ast_mutex_init(&s.lock);
 	s.session = session;
-	s.fd = mkstemp(template);	/* create a temporary file for command output */
+	fd = mkstemp(template);	/* create a temporary file for command output */
 	unlink(template);
-	if (s.fd <= -1) {
+	if (fd <= -1) {
 		ast_http_error(ser, 500, "Server Error", "Internal Server Error (mkstemp failed)");
 		goto auth_callback_out;
 	}
-	s.f = fdopen(s.fd, "w+");
-	if (!s.f) {
+	s.stream = ast_iostream_from_fd(&fd);
+	if (!s.stream) {
 		ast_log(LOG_WARNING, "HTTP Manager, fdopen failed: %s!\n", strerror(errno));
 		ast_http_error(ser, 500, "Server Error", "Internal Server Error (fdopen failed)");
-		close(s.fd);
+		close(fd);
 		goto auth_callback_out;
 	}
 
@@ -8176,7 +8197,7 @@ static int auth_http_callback(struct ast
 
 	astman_free_headers(&m);
 
-	result_size = ftell(s.f); /* Calculate approx. size of result */
+	result_size = lseek(ast_iostream_get_fd(s.stream), 0, SEEK_CUR); /* Calculate approx. size of result */
 
 	http_header = ast_str_create(80);
 	out = ast_str_create(result_size * 2 + 512);
@@ -8228,11 +8249,10 @@ auth_callback_out:
 	ast_free(out);
 
 	ao2_lock(session);
-	if (session->f) {
-		fclose(session->f);
+	if (session->stream) {
+		ast_iostream_close(session->stream);
+		session->stream = NULL;
 	}
-	session->f = NULL;
-	session->fd = -1;
 	ao2_unlock(session);
 
 	if (session->needdestroy) {
@@ -8977,8 +8997,6 @@ static int __init_manager(int reload, in
 #endif
 		int res;
 
-		ast_register_cleanup(manager_shutdown);
-
 		res = STASIS_MESSAGE_TYPE_INIT(ast_manager_get_generic_type);
 		if (res != 0) {
 			return -1;
@@ -9050,7 +9068,7 @@ static int __init_manager(int reload, in
 #endif
 
 		/* If you have a NULL hash fn, you only need a single bucket */
-		sessions = ao2_container_alloc(1, NULL, mansession_cmp_fn);
+		sessions = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, mansession_cmp_fn);
 		if (!sessions) {
 			return -1;
 		}
@@ -9306,8 +9324,8 @@ static int __init_manager(int reload, in
 			/* Default allowmultiplelogin from [general] */
 			user->allowmultiplelogin = allowmultiplelogin;
 			user->writetimeout = 100;
-			user->whitefilters = ao2_container_alloc(1, NULL, NULL);
-			user->blackfilters = ao2_container_alloc(1, NULL, NULL);
+			user->whitefilters = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
+			user->blackfilters = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
 			if (!user->whitefilters || !user->blackfilters) {
 				manager_free_user(user);
 				break;
@@ -9334,7 +9352,14 @@ static int __init_manager(int reload, in
 			} else if (!strcasecmp(var->name, "deny") ||
 				       !strcasecmp(var->name, "permit") ||
 				       !strcasecmp(var->name, "acl")) {
-				ast_append_acl(var->name, var->value, &user->acl, NULL, &acl_subscription_flag);
+				int acl_error = 0;
+
+				ast_append_acl(var->name, var->value, &user->acl, &acl_error, &acl_subscription_flag);
+				if (acl_error) {
+					ast_log(LOG_ERROR, "Invalid ACL '%s' for manager user '%s' on line %d. Deleting user\n",
+						var->value, user->username, var->lineno);
+					user->keep = 0;
+				}
 			}  else if (!strcasecmp(var->name, "read") ) {
 				user->readperm = get_perm(var->value);
 			}  else if (!strcasecmp(var->name, "write") ) {
@@ -9454,12 +9479,19 @@ static void acl_change_stasis_cb(void *d
 	__init_manager(1, 1);
 }
 
-int init_manager(void)
+static int unload_module(void)
 {
-	return __init_manager(0, 0);
+	return 0;
 }
 
-int reload_manager(void)
+static int load_module(void)
+{
+	ast_register_cleanup(manager_shutdown);
+
+	return __init_manager(0, 0) ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
+}
+
+static int reload_module(void)
 {
 	return __init_manager(1, 0);
 }
@@ -9556,3 +9588,12 @@ ast_manager_event_blob_create(
 
 	return ev;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Asterisk Manager Interface",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig,acl,http",
+);
diff -pruN 1:13.23.1~dfsg-2/main/manager_channels.c 1:16.1.1~dfsg-1/main/manager_channels.c
--- 1:13.23.1~dfsg-2/main/manager_channels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager_channels.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/callerid.h"
 #include "asterisk/channel.h"
 #include "asterisk/manager.h"
@@ -184,6 +182,33 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 			</see-also>
 		</managerEventInstance>
 	</managerEvent>
+	<managerEvent language="en_US" name="DialState">
+		<managerEventInstance class="EVENT_FLAG_CALL">
+			<synopsis>Raised when dial status has changed.</synopsis>
+			<syntax>
+				<channel_snapshot/>
+				<channel_snapshot prefix="Dest"/>
+				<parameter name="DialStatus">
+					<para> The new state of the outbound dial attempt.</para>
+					<enumlist>
+						<enum name="RINGING">
+							<para>The outbound channel is ringing.</para>
+						</enum>
+						<enum name="PROCEEDING">
+							<para>The call to the outbound channel is proceeding.</para>
+						</enum>
+						<enum name="PROGRESS">
+							<para>Progress has been received on the outbound channel.</para>
+						</enum>
+					</enumlist>
+				</parameter>
+				<parameter name="Forward" required="false">
+					<para>If the call was forwarded, where the call was
+					forwarded to.</para>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
 	<managerEvent language="en_US" name="DialEnd">
 		<managerEventInstance class="EVENT_FLAG_CALL">
 			<synopsis>Raised when a dial action has completed.</synopsis>
@@ -613,7 +638,7 @@ static struct ast_manager_event_blob *ch
 
 	/* DEPRECATED: Extension field deprecated in 12; remove in 14 */
 	return ast_manager_event_blob_create(
-		EVENT_FLAG_CALL, "Newexten",
+		EVENT_FLAG_DIALPLAN, "Newexten",
 		"Extension: %s\r\n"
 		"Application: %s\r\n"
 		"AppData: %s\r\n",
@@ -1096,6 +1121,13 @@ static void channel_monitor_stop_cb(void
 	publish_basic_channel_event("MonitorStop", EVENT_FLAG_CALL, payload->snapshot);
 }
 
+static int dial_status_end(const char *dialstatus)
+{
+	return (strcmp(dialstatus, "RINGING") &&
+			strcmp(dialstatus, "PROCEEDING") &&
+			strcmp(dialstatus, "PROGRESS"));
+}
+
 /*!
  * \brief Callback processing messages for channel dialing
  */
@@ -1139,7 +1171,7 @@ static void channel_dial_cb(void *data,
 	} else {
 		int forwarded = !ast_strlen_zero(forward);
 
-		manager_event(EVENT_FLAG_CALL, "DialEnd",
+		manager_event(EVENT_FLAG_CALL, dial_status_end(dialstatus) ? "DialEnd" : "DialState",
 				"%s"
 				"%s"
 				"%s%s%s"
diff -pruN 1:13.23.1~dfsg-2/main/manager_endpoints.c 1:16.1.1~dfsg-1/main/manager_endpoints.c
--- 1:13.23.1~dfsg-2/main/manager_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/callerid.h"
 #include "asterisk/channel.h"
 #include "asterisk/manager.h"
diff -pruN 1:13.23.1~dfsg-2/main/manager_mwi.c 1:16.1.1~dfsg-1/main/manager_mwi.c
--- 1:13.23.1~dfsg-2/main/manager_mwi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager_mwi.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/manager.h"
 #include "asterisk/app.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/main/manager_system.c 1:16.1.1~dfsg-1/main/manager_system.c
--- 1:13.23.1~dfsg-2/main/manager_system.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/manager_system.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_message_router.h"
 #include "asterisk/stasis_system.h"
diff -pruN 1:13.23.1~dfsg-2/main/md5.c 1:16.1.1~dfsg-1/main/md5.c
--- 1:13.23.1~dfsg-2/main/md5.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/md5.c	2018-12-26 18:45:13.000000000 +0000
@@ -22,8 +22,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/endian.h"
 #include "asterisk/md5.h"
 
diff -pruN 1:13.23.1~dfsg-2/main/media_cache.c 1:16.1.1~dfsg-1/main/media_cache.c
--- 1:13.23.1~dfsg-2/main/media_cache.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/media_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,713 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Matt Jordan
+ *
+ * Matt Jordan <mjordan@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief An in-memory media cache
+ *
+ * \author \verbatim Matt Jordan <mjordan@digium.com> \endverbatim
+ *
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <sys/stat.h>
+#include "asterisk/config.h"
+#include "asterisk/bucket.h"
+#include "asterisk/astdb.h"
+#include "asterisk/cli.h"
+#include "asterisk/file.h"
+#include "asterisk/media_cache.h"
+
+/*! The name of the AstDB family holding items in the cache. */
+#define AST_DB_FAMILY "MediaCache"
+
+/*! Length of 'MediaCache' + 2 '/' characters */
+#define AST_DB_FAMILY_LEN 12
+
+/*! Number of buckets in the ao2 container holding our media items */
+#define AO2_BUCKETS 61
+
+/*! Our one and only container holding media items */
+static struct ao2_container *media_cache;
+
+int ast_media_cache_exists(const char *uri)
+{
+	struct ast_bucket_file *bucket_file;
+
+	if (ast_strlen_zero(uri)) {
+		return 0;
+	}
+
+	bucket_file = ao2_find(media_cache, uri, OBJ_SEARCH_KEY);
+	if (bucket_file) {
+		ao2_ref(bucket_file, -1);
+		return 1;
+	}
+
+	/* Check to see if any bucket implementation could return this item */
+	bucket_file = ast_bucket_file_retrieve(uri);
+	if (bucket_file) {
+		ao2_ref(bucket_file, -1);
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Sync \c bucket_file metadata to the AstDB
+ */
+static int metadata_sync_to_astdb(void *obj, void *arg, int flags)
+{
+	struct ast_bucket_metadata *metadata = obj;
+	const char *hash = arg;
+
+	ast_db_put(hash, metadata->name, metadata->value);
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Sync a media cache item to the AstDB
+ * \param bucket_file The \c ast_bucket_file media cache item to sync
+ */
+static void media_cache_item_sync_to_astdb(struct ast_bucket_file *bucket_file)
+{
+	char hash[41]; /* 40 character SHA1 hash */
+
+	ast_sha1_hash(hash, ast_sorcery_object_get_id(bucket_file));
+	if (ast_db_put(AST_DB_FAMILY, ast_sorcery_object_get_id(bucket_file), hash)) {
+		return;
+	}
+
+	ast_db_put(hash, "path", bucket_file->path);
+	ast_bucket_file_metadata_callback(bucket_file, metadata_sync_to_astdb, hash);
+}
+
+/*!
+ * \internal
+ * \brief Delete a media cache item from the AstDB
+ * \param bucket_file The \c ast_bucket_file media cache item to delete
+ */
+static void media_cache_item_del_from_astdb(struct ast_bucket_file *bucket_file)
+{
+	char *hash_value;
+
+	if (ast_db_get_allocated(AST_DB_FAMILY, ast_sorcery_object_get_id(bucket_file), &hash_value)) {
+		return;
+	}
+
+	ast_db_deltree(hash_value, NULL);
+	ast_db_del(AST_DB_FAMILY, ast_sorcery_object_get_id(bucket_file));
+	ast_free(hash_value);
+}
+
+/*!
+ * \internal
+ * \brief Normalize the value of a Content-Type header
+ *
+ * This will trim off any optional parameters after the type/subtype.
+ */
+static void normalize_content_type_header(char *content_type)
+{
+	char *params = strchr(content_type, ';');
+
+	if (params) {
+		*params-- = 0;
+		while (params > content_type && (*params == ' ' || *params == '\t')) {
+			*params-- = 0;
+		}
+	}
+}
+
+/*!
+ * \internal
+ * \brief Update the name of the file backing a \c bucket_file
+ * \param preferred_file_name The preferred name of the backing file
+ */
+static void bucket_file_update_path(struct ast_bucket_file *bucket_file,
+	const char *preferred_file_name)
+{
+	char *ext;
+
+	if (!ast_strlen_zero(preferred_file_name) && strcmp(bucket_file->path, preferred_file_name)) {
+		/* Use the preferred file name if available */
+
+		rename(bucket_file->path, preferred_file_name);
+		ast_copy_string(bucket_file->path, preferred_file_name,
+			sizeof(bucket_file->path));
+	} else if (!strchr(bucket_file->path, '.') && (ext = strrchr(ast_sorcery_object_get_id(bucket_file), '.'))) {
+		/* If we don't have a file extension and were provided one in the URI, use it */
+		char new_path[PATH_MAX];
+		char found_ext[PATH_MAX];
+
+		ast_bucket_file_metadata_set(bucket_file, "ext", ext);
+
+		/* Don't pass '.' while checking for supported extension */
+		if (!ast_get_format_for_file_ext(ext + 1)) {
+			/* If the file extension passed in the URI isn't supported check for the
+			 * extension based on the MIME type passed in the Content-Type header before
+			 * giving up.
+			 * If a match is found then retrieve the extension from the supported list
+			 * corresponding to the mime-type and use that to rename the file */
+			struct ast_bucket_metadata *header = ast_bucket_file_metadata_get(bucket_file, "content-type");
+			if (header) {
+				char *mime_type = ast_strdup(header->value);
+				if (mime_type) {
+					normalize_content_type_header(mime_type);
+					if (!ast_strlen_zero(mime_type)) {
+						if (ast_get_extension_for_mime_type(mime_type, found_ext, sizeof(found_ext))) {
+							ext = found_ext;
+						}
+					}
+					ast_free(mime_type);
+				}
+			}
+		}
+
+		snprintf(new_path, sizeof(new_path), "%s%s", bucket_file->path, ext);
+		rename(bucket_file->path, new_path);
+		ast_copy_string(bucket_file->path, new_path, sizeof(bucket_file->path));
+	}
+}
+
+int ast_media_cache_retrieve(const char *uri, const char *preferred_file_name,
+	char *file_path, size_t len)
+{
+	struct ast_bucket_file *bucket_file;
+	char *ext;
+	SCOPED_AO2LOCK(media_lock, media_cache);
+
+	if (ast_strlen_zero(uri)) {
+		return -1;
+	}
+
+	/* First, retrieve from the ao2 cache here. If we find a bucket_file
+	 * matching the requested URI, ask the appropriate backend if it is
+	 * stale. If not; return it.
+	 */
+	bucket_file = ao2_find(media_cache, uri, OBJ_SEARCH_KEY | OBJ_NOLOCK);
+	if (bucket_file) {
+		if (!ast_bucket_file_is_stale(bucket_file)
+			&& ast_file_is_readable(bucket_file->path)) {
+			ast_copy_string(file_path, bucket_file->path, len);
+			if ((ext = strrchr(file_path, '.'))) {
+				*ext = '\0';
+			}
+			ao2_ref(bucket_file, -1);
+
+			ast_debug(5, "Returning media at local file: %s\n", file_path);
+			return 0;
+		}
+
+		/* Stale! Remove the item completely, as we're going to replace it next */
+		ao2_unlink_flags(media_cache, bucket_file, OBJ_NOLOCK);
+		ast_bucket_file_delete(bucket_file);
+		ao2_ref(bucket_file, -1);
+	}
+
+	/* Either this is new or the resource is stale; do a full retrieve
+	 * from the appropriate bucket_file backend
+	 */
+	bucket_file = ast_bucket_file_retrieve(uri);
+	if (!bucket_file) {
+		ast_debug(2, "Failed to obtain media at '%s'\n", uri);
+		return -1;
+	}
+
+	/* We can manipulate the 'immutable' bucket_file here, as we haven't
+	 * let anyone know of its existence yet
+	 */
+	bucket_file_update_path(bucket_file, preferred_file_name);
+	media_cache_item_sync_to_astdb(bucket_file);
+	ast_copy_string(file_path, bucket_file->path, len);
+	if ((ext = strrchr(file_path, '.'))) {
+		*ext = '\0';
+	}
+	ao2_link_flags(media_cache, bucket_file, OBJ_NOLOCK);
+	ao2_ref(bucket_file, -1);
+
+	ast_debug(5, "Returning media at local file: %s\n", file_path);
+
+	return 0;
+}
+
+int ast_media_cache_retrieve_metadata(const char *uri, const char *key,
+	char *value, size_t len)
+{
+	struct ast_bucket_file *bucket_file;
+	struct ast_bucket_metadata *metadata;
+
+	if (ast_strlen_zero(uri) || ast_strlen_zero(key) || !value) {
+		return -1;
+	}
+
+	bucket_file = ao2_find(media_cache, uri, OBJ_SEARCH_KEY);
+	if (!bucket_file) {
+		return -1;
+	}
+
+	metadata = ao2_find(bucket_file->metadata, key, OBJ_SEARCH_KEY);
+	if (!metadata) {
+		ao2_ref(bucket_file, -1);
+		return -1;
+	}
+	ast_copy_string(value, metadata->value, len);
+
+	ao2_ref(metadata, -1);
+	ao2_ref(bucket_file, -1);
+	return 0;
+}
+
+int ast_media_cache_create_or_update(const char *uri, const char *file_path,
+	struct ast_variable *metadata)
+{
+	struct ast_bucket_file *bucket_file;
+	struct ast_variable *it_metadata;
+	struct stat st;
+	char tmp[128];
+	char *ext;
+	char *file_path_ptr;
+	int created = 0;
+	SCOPED_AO2LOCK(media_lock, media_cache);
+
+	if (ast_strlen_zero(file_path) || ast_strlen_zero(uri)) {
+		return -1;
+	}
+	file_path_ptr = ast_strdupa(file_path);
+
+	if (stat(file_path, &st)) {
+		ast_log(LOG_WARNING, "Unable to obtain information for file %s for URI %s\n",
+			file_path, uri);
+		return -1;
+	}
+
+	bucket_file = ao2_find(media_cache, uri, OBJ_SEARCH_KEY | OBJ_NOLOCK);
+	if (bucket_file) {
+		struct ast_bucket_file *clone;
+
+		clone = ast_bucket_file_clone(bucket_file);
+		if (!clone) {
+			ao2_ref(bucket_file, -1);
+			return -1;
+		}
+
+		/* Remove the old bucket_file. We'll replace it if we succeed below. */
+		ao2_unlink_flags(media_cache, bucket_file, OBJ_NOLOCK);
+		ao2_ref(bucket_file, -1);
+
+		bucket_file = clone;
+	} else {
+		bucket_file = ast_bucket_file_alloc(uri);
+		if (!bucket_file) {
+			ast_log(LOG_WARNING, "Failed to create file storage for %s and %s\n",
+				uri, file_path);
+			return -1;
+		}
+		created = 1;
+	}
+
+	strcpy(bucket_file->path, file_path);
+	bucket_file->created.tv_sec = st.st_ctime;
+	bucket_file->modified.tv_sec = st.st_mtime;
+
+	snprintf(tmp, sizeof(tmp), "%ld", (long)st.st_atime);
+	ast_bucket_file_metadata_set(bucket_file, "accessed", tmp);
+
+	snprintf(tmp, sizeof(tmp), "%jd", (intmax_t)st.st_size);
+	ast_bucket_file_metadata_set(bucket_file, "size", tmp);
+
+	ext = strrchr(file_path_ptr, '.');
+	if (ext) {
+		ast_bucket_file_metadata_set(bucket_file, "ext", ext + 1);
+	}
+
+	for (it_metadata = metadata; it_metadata; it_metadata = it_metadata->next) {
+		ast_bucket_file_metadata_set(bucket_file, it_metadata->name, it_metadata->value);
+	}
+
+	if (created && ast_bucket_file_create(bucket_file)) {
+		ast_log(LOG_WARNING, "Failed to create media for %s\n", uri);
+		ao2_ref(bucket_file, -1);
+		return -1;
+	}
+	media_cache_item_sync_to_astdb(bucket_file);
+
+	ao2_link_flags(media_cache, bucket_file, OBJ_NOLOCK);
+	ao2_ref(bucket_file, -1);
+	return 0;
+}
+
+int ast_media_cache_delete(const char *uri)
+{
+	struct ast_bucket_file *bucket_file;
+	int res;
+
+	if (ast_strlen_zero(uri)) {
+		return -1;
+	}
+
+	bucket_file = ao2_find(media_cache, uri, OBJ_SEARCH_KEY | OBJ_UNLINK);
+	if (!bucket_file) {
+		return -1;
+	}
+
+	res = ast_bucket_file_delete(bucket_file);
+	media_cache_item_del_from_astdb(bucket_file);
+
+	ao2_ref(bucket_file, -1);
+
+	return res;
+}
+
+/*!
+ * \internal
+ * \brief Remove a media cache item from the AstDB
+ * \param uri The unique URI that represents the item in the cache
+ * \param hash The hash key for the item in the AstDB
+ */
+static void media_cache_remove_from_astdb(const char *uri, const char *hash)
+{
+	ast_db_del(AST_DB_FAMILY, uri + AST_DB_FAMILY_LEN);
+	ast_db_deltree(hash, NULL);
+}
+
+/*!
+ * \internal
+ * \brief Create an item in the media cache from entries in the AstDB
+ * \param uri The unique URI that represents the item in the cache
+ * \param hash The hash key for the item in the AstDB
+ * \retval 0 success
+ * \retval -1 failure
+ */
+static int media_cache_item_populate_from_astdb(const char *uri, const char *hash)
+{
+	struct ast_bucket_file *bucket_file;
+	struct ast_db_entry *db_tree;
+	struct ast_db_entry *db_entry;
+	struct stat st;
+
+	bucket_file = ast_bucket_file_alloc(uri);
+	if (!bucket_file) {
+		return -1;
+	}
+
+	db_tree = ast_db_gettree(hash, NULL);
+	for (db_entry = db_tree; db_entry; db_entry = db_entry->next) {
+		const char *key = strchr(db_entry->key + 1, '/');
+
+		if (ast_strlen_zero(key)) {
+			continue;
+		}
+		key++;
+
+		if (!strcasecmp(key, "path")) {
+			strcpy(bucket_file->path, db_entry->data);
+
+			if (stat(bucket_file->path, &st)) {
+				ast_log(LOG_WARNING, "Unable to obtain information for file %s for URI %s\n",
+					bucket_file->path, uri);
+				ao2_ref(bucket_file, -1);
+				ast_db_freetree(db_tree);
+				return -1;
+			}
+		} else {
+			ast_bucket_file_metadata_set(bucket_file, key, db_entry->data);
+		}
+	}
+	ast_db_freetree(db_tree);
+
+	if (ast_strlen_zero(bucket_file->path)) {
+		ao2_ref(bucket_file, -1);
+		ast_log(LOG_WARNING, "Failed to restore media cache item for '%s' from AstDB: no 'path' specified\n",
+			uri);
+		return -1;
+	}
+
+	ao2_link(media_cache, bucket_file);
+	ao2_ref(bucket_file, -1);
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Populate the media cache from entries in the AstDB
+ */
+static void media_cache_populate_from_astdb(void)
+{
+	struct ast_db_entry *db_entry;
+	struct ast_db_entry *db_tree;
+
+	db_tree = ast_db_gettree(AST_DB_FAMILY, NULL);
+	for (db_entry = db_tree; db_entry; db_entry = db_entry->next) {
+		if (media_cache_item_populate_from_astdb(db_entry->key + AST_DB_FAMILY_LEN, db_entry->data)) {
+			media_cache_remove_from_astdb(db_entry->key, db_entry->data);
+		}
+	}
+	ast_db_freetree(db_tree);
+}
+
+/*!
+ * \internal
+ * \brief ao2 callback function for \ref media_cache_handle_show_all
+ */
+static int media_cache_prnt_summary(void *obj, void *arg, int flags)
+{
+#define FORMAT_ROW "%-40s\n\t%-40s\n"
+	struct ast_bucket_file *bucket_file = obj;
+	struct ast_cli_args *a = arg;
+
+	ast_cli(a->fd, FORMAT_ROW, ast_sorcery_object_get_id(bucket_file), bucket_file->path);
+
+#undef FORMAT_ROW
+	return CMP_MATCH;
+}
+
+static char *media_cache_handle_show_all(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "media cache show all";
+		e->usage =
+			"Usage: media cache show all\n"
+			"       Display a summary of all current items\n"
+			"       in the media cache.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	if (a->argc != 4) {
+		return CLI_SHOWUSAGE;
+	}
+
+	ast_cli(a->fd, "URI\n\tLocal File\n");
+	ast_cli(a->fd, "---------------\n");
+	ao2_callback(media_cache, OBJ_NODATA | OBJ_MULTIPLE, media_cache_prnt_summary, a);
+
+	return CLI_SUCCESS;
+}
+
+/*!
+ * \internal
+ * \brief CLI tab completion function for URIs
+ */
+static char *cli_complete_uri(const char *word, int state)
+{
+	struct ast_bucket_file *bucket_file;
+	struct ao2_iterator it_media_items;
+	int wordlen = strlen(word);
+	int which = 0;
+	char *result = NULL;
+
+	it_media_items = ao2_iterator_init(media_cache, 0);
+	while ((bucket_file = ao2_iterator_next(&it_media_items))) {
+		if (!strncasecmp(word, ast_sorcery_object_get_id(bucket_file), wordlen)
+			&& ++which > state) {
+			result = ast_strdup(ast_sorcery_object_get_id(bucket_file));
+		}
+		ao2_ref(bucket_file, -1);
+		if (result) {
+			break;
+		}
+	}
+	ao2_iterator_destroy(&it_media_items);
+	return result;
+}
+
+static char *media_cache_handle_show_item(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+#define FORMAT_ROW "\t%20s: %-40.40s\n"
+	struct ast_bucket_file *bucket_file;
+	struct ao2_iterator it_metadata;
+	struct ast_bucket_metadata *metadata;
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "media cache show";
+		e->usage =
+			"Usage: media cache show <uri>\n"
+			"       Display all information about a particular\n"
+			"       item in the media cache.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return cli_complete_uri(a->word, a->n);
+	}
+
+	if (a->argc != 4) {
+		return CLI_SHOWUSAGE;
+	}
+
+	bucket_file = ao2_find(media_cache, a->argv[3], OBJ_SEARCH_KEY);
+	if (!bucket_file) {
+		ast_cli(a->fd, "Unable to find '%s' in the media cache\n", a->argv[3]);
+		return CLI_SUCCESS;
+	}
+
+	ast_cli(a->fd, "URI: %s\n", ast_sorcery_object_get_id(bucket_file));
+	ast_cli(a->fd, "%s\n", "----------------------------------------");
+	ast_cli(a->fd, FORMAT_ROW, "Path", bucket_file->path);
+
+	it_metadata = ao2_iterator_init(bucket_file->metadata, 0);
+	while ((metadata = ao2_iterator_next(&it_metadata))) {
+		ast_cli(a->fd, FORMAT_ROW, metadata->name, metadata->value);
+		ao2_ref(metadata, -1);
+	}
+	ao2_iterator_destroy(&it_metadata);
+
+	ao2_ref(bucket_file, -1);
+#undef FORMAT_ROW
+	return CLI_SUCCESS;
+}
+
+static char *media_cache_handle_delete_item(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "media cache delete";
+		e->usage =
+			"Usage: media cache delete <uri>\n"
+			"       Delete an item from the media cache.\n"
+			"       Note that this will also remove any local\n"
+			"       storage of the media associated with the URI,\n"
+			"       and will inform the backend supporting the URI\n"
+			"       scheme that it should remove the item.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return cli_complete_uri(a->word, a->n);
+	}
+
+	if (a->argc != 4) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (ast_media_cache_delete(a->argv[3])) {
+		ast_cli(a->fd, "Unable to delete '%s'\n", a->argv[3]);
+	} else {
+		ast_cli(a->fd, "Deleted '%s' from the media cache\n", a->argv[3]);
+	}
+
+	return CLI_SUCCESS;
+}
+
+static char *media_cache_handle_refresh_item(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	char file_path[PATH_MAX];
+
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "media cache refresh";
+		e->usage =
+			"Usage: media cache refresh <uri>\n"
+			"       Ask for a refresh of a particular URI.\n"
+			"       If the item does not already exist in the\n"
+			"       media cache, the item will be populated from\n"
+			"       the backend supporting the URI scheme.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return cli_complete_uri(a->word, a->n);
+	}
+
+	if (a->argc != 4) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (ast_media_cache_retrieve(a->argv[3], NULL, file_path, sizeof(file_path))) {
+		ast_cli(a->fd, "Unable to refresh '%s'\n", a->argv[3]);
+	} else {
+		ast_cli(a->fd, "Refreshed '%s' to local storage '%s'\n", a->argv[3], file_path);
+	}
+
+	return CLI_SUCCESS;
+}
+
+static char *media_cache_handle_create_item(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "media cache create";
+		e->usage =
+			"Usage: media cache create <uri> <file>\n"
+			"       Create an item in the media cache by associating\n"
+			"       a local media file with some URI.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	if (a->argc != 5) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (ast_media_cache_create_or_update(a->argv[3], a->argv[4], NULL)) {
+		ast_cli(a->fd, "Unable to create '%s' associated with local file '%s'\n",
+			a->argv[3], a->argv[4]);
+	} else {
+		ast_cli(a->fd, "Created '%s' for '%s' in the media cache\n",
+			a->argv[3], a->argv[4]);
+	}
+
+	return CLI_SUCCESS;
+}
+
+static struct ast_cli_entry cli_media_cache[] = {
+	AST_CLI_DEFINE(media_cache_handle_show_all, "Show all items in the media cache"),
+	AST_CLI_DEFINE(media_cache_handle_show_item, "Show a single item in the media cache"),
+	AST_CLI_DEFINE(media_cache_handle_delete_item, "Remove an item from the media cache"),
+	AST_CLI_DEFINE(media_cache_handle_refresh_item, "Refresh an item in the media cache"),
+	AST_CLI_DEFINE(media_cache_handle_create_item, "Create an item in the media cache"),
+};
+
+/*!
+ * \internal
+ * \brief Shutdown the media cache
+ */
+static void media_cache_shutdown(void)
+{
+	ao2_cleanup(media_cache);
+	media_cache = NULL;
+
+	ast_cli_unregister_multiple(cli_media_cache, ARRAY_LEN(cli_media_cache));
+}
+
+int ast_media_cache_init(void)
+{
+	ast_register_cleanup(media_cache_shutdown);
+
+	media_cache = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, AO2_BUCKETS,
+		ast_sorcery_object_id_hash, NULL, ast_sorcery_object_id_compare);
+	if (!media_cache) {
+		return -1;
+	}
+
+	if (ast_cli_register_multiple(cli_media_cache, ARRAY_LEN(cli_media_cache))) {
+		return -1;
+	}
+
+	media_cache_populate_from_astdb();
+
+	return 0;
+}
diff -pruN 1:13.23.1~dfsg-2/main/media_index.c 1:16.1.1~dfsg-1/main/media_index.c
--- 1:13.23.1~dfsg-2/main/media_index.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/media_index.c	2018-12-26 18:45:13.000000000 +0000
@@ -64,7 +64,8 @@ static struct media_variant *media_varia
 	size_t str_sz = strlen(variant_str) + 1;
 	struct media_variant *variant;
 
-	variant = ao2_alloc(sizeof(*variant) + str_sz, media_variant_destroy);
+	variant = ao2_alloc_options(sizeof(*variant) + str_sz, media_variant_destroy,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
 	if (!variant) {
 		return NULL;
 	}
@@ -110,15 +111,18 @@ static void media_info_destroy(void *obj
 static struct media_info *media_info_alloc(const char *name)
 {
 	size_t name_sz = strlen(name) + 1;
-	struct media_info *info = ao2_alloc(sizeof(*info) + name_sz, media_info_destroy);
+	struct media_info *info;
 
+	info = ao2_alloc_options(sizeof(*info) + name_sz, media_info_destroy,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
 	if (!info) {
 		return NULL;
 	}
 
 	memcpy(info->name, name, name_sz);
 
-	info->variants = ao2_container_alloc(VARIANT_BUCKETS, media_variant_hash, media_variant_cmp);
+	info->variants = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		VARIANT_BUCKETS, media_variant_hash, NULL, media_variant_cmp);
 	if (!info->variants) {
 		ao2_ref(info, -1);
 
@@ -166,7 +170,8 @@ struct ast_media_index *ast_media_index_
 
 	memcpy(index->base_dir, base_dir, base_dir_sz);
 
-	index->index = ao2_container_alloc(INDEX_BUCKETS, media_info_hash, media_info_cmp);
+	index->index = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, INDEX_BUCKETS,
+		media_info_hash, NULL, media_info_cmp);
 	if (!index->index) {
 		ao2_ref(index, -1);
 
diff -pruN 1:13.23.1~dfsg-2/main/message.c 1:16.1.1~dfsg-1/main/message.c
--- 1:13.23.1~dfsg-2/main/message.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/message.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #include "asterisk/module.h"
@@ -369,12 +367,6 @@ static void msg_ds_destroy(void *data)
 	ao2_ref(msg, -1);
 }
 
-static int msg_data_hash_fn(const void *obj, const int flags)
-{
-	const struct msg_data *data = obj;
-	return ast_str_case_hash(data->name);
-}
-
 static int msg_data_cmp_fn(void *obj, void *arg, int flags)
 {
 	const struct msg_data *one = obj, *two = arg;
@@ -408,7 +400,9 @@ struct ast_msg *ast_msg_alloc(void)
 		return NULL;
 	}
 
-	if (!(msg->vars = ao2_container_alloc(1, msg_data_hash_fn, msg_data_cmp_fn))) {
+	msg->vars = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NULL, msg_data_cmp_fn);
+	if (!msg->vars) {
 		ao2_ref(msg, -1);
 		return NULL;
 	}
diff -pruN 1:13.23.1~dfsg-2/main/mixmonitor.c 1:16.1.1~dfsg-1/main/mixmonitor.c
--- 1:13.23.1~dfsg-2/main/mixmonitor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/mixmonitor.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/lock.h"
 #include "asterisk/logger.h"
 #include "asterisk/mixmonitor.h"
diff -pruN 1:13.23.1~dfsg-2/main/named_acl.c 1:16.1.1~dfsg-1/main/named_acl.c
--- 1:13.23.1~dfsg-2/main/named_acl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/named_acl.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,9 +27,12 @@
  * Olle E. Johansson <oej@edvina.net>
  */
 
-#include "asterisk.h"
+/* This maintains the original "module reload acl" CLI command instead
+ * of replacing it with "module reload named_acl". */
+#undef AST_MODULE
+#define AST_MODULE "acl"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include "asterisk.h"
 
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
@@ -128,7 +131,9 @@ static void *named_acl_config_alloc(void
 		return NULL;
 	}
 
-	if (!(cfg->named_acl_list = ao2_container_alloc(37, named_acl_hash_fn, named_acl_cmp_fn))) {
+	cfg->named_acl_list = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		named_acl_hash_fn, NULL, named_acl_cmp_fn);
+	if (!cfg->named_acl_list) {
 		goto error;
 	}
 
@@ -399,36 +404,6 @@ publish_failure:
 
 /*!
  * \internal
- * \brief reload configuration for named ACLs
- *
- * \param fd file descriptor for CLI client
- */
-int ast_named_acl_reload(void)
-{
-	enum aco_process_status status;
-
-	status = aco_process_config(&cfg_info, 1);
-
-	if (status == ACO_PROCESS_ERROR) {
-		ast_log(LOG_WARNING, "Could not reload ACL config\n");
-		return 0;
-	}
-
-	if (status == ACO_PROCESS_UNCHANGED) {
-		/* We don't actually log anything if the config was unchanged,
-		 * but we don't need to send a config change event either.
-		 */
-		return 0;
-	}
-
-	/* We need to push an ACL change event with no ACL name so that all subscribers update with all ACLs */
-	publish_acl_change("");
-
-	return 0;
-}
-
-/*!
- * \internal
  * \brief secondary handler for the 'acl show <name>' command (with arg)
  *
  * \param fd file descriptor of the cli
@@ -558,32 +533,65 @@ static struct ast_cli_entry cli_named_ac
 	AST_CLI_DEFINE(handle_show_named_acl_cmd, "Show a named ACL or list all named ACLs"),
 };
 
-static void named_acl_cleanup(void)
+static int reload_module(void)
+{
+	enum aco_process_status status;
+
+	status = aco_process_config(&cfg_info, 1);
+
+	if (status == ACO_PROCESS_ERROR) {
+		ast_log(LOG_WARNING, "Could not reload ACL config\n");
+		return 0;
+	}
+
+	if (status == ACO_PROCESS_UNCHANGED) {
+		/* We don't actually log anything if the config was unchanged,
+		 * but we don't need to send a config change event either.
+		 */
+		return 0;
+	}
+
+	/* We need to push an ACL change event with no ACL name so that all subscribers update with all ACLs */
+	publish_acl_change("");
+
+	return 0;
+}
+
+static int unload_module(void)
 {
 	ast_cli_unregister_multiple(cli_named_acl, ARRAY_LEN(cli_named_acl));
 
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_named_acl_change_type);
 	aco_info_destroy(&cfg_info);
 	ao2_global_obj_release(globals);
+
+	return 0;
 }
 
-int ast_named_acl_init()
+static int load_module(void)
 {
-	ast_cli_register_multiple(cli_named_acl, ARRAY_LEN(cli_named_acl));
-
-	STASIS_MESSAGE_TYPE_INIT(ast_named_acl_change_type);
-
-	ast_register_cleanup(named_acl_cleanup);
-
 	if (aco_info_init(&cfg_info)) {
-		return 0;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
+	STASIS_MESSAGE_TYPE_INIT(ast_named_acl_change_type);
+
 	/* Register the per level options. */
 	aco_option_register(&cfg_info, "permit", ACO_EXACT, named_acl_types, NULL, OPT_ACL_T, 1, FLDSET(struct named_acl, ha));
 	aco_option_register(&cfg_info, "deny", ACO_EXACT, named_acl_types, NULL, OPT_ACL_T, 0, FLDSET(struct named_acl, ha));
 
 	aco_process_config(&cfg_info, 0);
 
-	return 0;
+	ast_cli_register_multiple(cli_named_acl, ARRAY_LEN(cli_named_acl));
+
+	return AST_MODULE_LOAD_SUCCESS;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Named ACL system",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/named_locks.c 1:16.1.1~dfsg-1/main/named_locks.c
--- 1:13.23.1~dfsg-2/main/named_locks.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/named_locks.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/named_locks.h"
@@ -35,12 +33,16 @@ ASTERISK_REGISTER_FILE()
 struct ao2_container *named_locks;
 #define NAMED_LOCKS_BUCKETS 101
 
-struct ast_named_lock {
+struct named_lock_proxy {
+	AO2_WEAKPROXY();
 	char key[0];
 };
 
-AO2_STRING_FIELD_HASH_FN(ast_named_lock, key)
-AO2_STRING_FIELD_CMP_FN(ast_named_lock, key)
+struct ast_named_lock {
+};
+
+AO2_STRING_FIELD_HASH_FN(named_lock_proxy, key)
+AO2_STRING_FIELD_CMP_FN(named_lock_proxy, key)
 
 static void named_locks_shutdown(void)
 {
@@ -50,7 +52,7 @@ static void named_locks_shutdown(void)
 int ast_named_locks_init(void)
 {
 	named_locks = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
-		NAMED_LOCKS_BUCKETS, ast_named_lock_hash_fn, NULL, ast_named_lock_cmp_fn);
+		NAMED_LOCKS_BUCKETS, named_lock_proxy_hash_fn, NULL, named_lock_proxy_cmp_fn);
 	if (!named_locks) {
 		return -1;
 	}
@@ -60,45 +62,62 @@ int ast_named_locks_init(void)
 	return 0;
 }
 
+static void named_lock_proxy_cb(void *weakproxy, void *data)
+{
+	ao2_unlink(named_locks, weakproxy);
+}
+
 struct ast_named_lock *__ast_named_lock_get(const char *filename, int lineno, const char *func,
 	enum ast_named_lock_type lock_type, const char *keyspace, const char *key)
 {
-	struct ast_named_lock *lock = NULL;
-	int concat_key_buff_len = strlen(keyspace) + strlen(key) + 2;
-	char *concat_key = ast_alloca(concat_key_buff_len);
+	struct named_lock_proxy *proxy;
+	struct ast_named_lock *lock;
+	int keylen = strlen(keyspace) + strlen(key) + 2;
+	char *concat_key = ast_alloca(keylen);
 
 	sprintf(concat_key, "%s-%s", keyspace, key); /* Safe */
 
 	ao2_lock(named_locks);
-	lock = ao2_find(named_locks, concat_key, OBJ_SEARCH_KEY | OBJ_NOLOCK);
+	lock = __ao2_weakproxy_find(named_locks, concat_key, OBJ_SEARCH_KEY | OBJ_NOLOCK,
+		__PRETTY_FUNCTION__, filename, lineno, func);
 	if (lock) {
-		ao2_unlock(named_locks);
 		ast_assert((ao2_options_get(lock) & AO2_ALLOC_OPT_LOCK_MASK) == lock_type);
+		ao2_unlock(named_locks);
+
 		return lock;
 	}
 
-	lock = ao2_alloc_options(sizeof(*lock) + concat_key_buff_len, NULL, lock_type);
-	if (lock) {
-		strcpy(lock->key, concat_key); /* Safe */
-		ao2_link_flags(named_locks, lock, OBJ_NOLOCK);
+	proxy = ao2_t_weakproxy_alloc(sizeof(*proxy) + keylen, NULL, concat_key);
+	if (!proxy) {
+		goto failure_cleanup;
 	}
-	ao2_unlock(named_locks);
-
-	return lock;
-}
 
-int __ast_named_lock_put(const char *filename, int lineno, const char *func,
-	struct ast_named_lock *lock)
-{
+	lock = __ao2_alloc(sizeof(*lock) + keylen, NULL, lock_type, concat_key, filename, lineno, func);
 	if (!lock) {
-		return -1;
+		goto failure_cleanup;
 	}
 
-	ao2_lock(named_locks);
-	if (ao2_ref(lock, -1) == 2) {
-		ao2_unlink_flags(named_locks, lock, OBJ_NOLOCK);
+	/* We have exclusive access to proxy and lock, no need for locking here. */
+	if (ao2_weakproxy_set_object(proxy, lock, OBJ_NOLOCK)) {
+		goto failure_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(proxy, named_lock_proxy_cb, NULL, OBJ_NOLOCK)) {
+		goto failure_cleanup;
 	}
+
+	strcpy(proxy->key, concat_key); /* Safe */
+	ao2_link_flags(named_locks, proxy, OBJ_NOLOCK);
 	ao2_unlock(named_locks);
+	ao2_t_ref(proxy, -1, "Release allocation reference");
 
-	return 0;
+	return lock;
+
+failure_cleanup:
+	ao2_unlock(named_locks);
+
+	ao2_cleanup(proxy);
+	ao2_cleanup(lock);
+
+	return NULL;
 }
diff -pruN 1:13.23.1~dfsg-2/main/netsock2.c 1:16.1.1~dfsg-1/main/netsock2.c
--- 1:13.23.1~dfsg-2/main/netsock2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/netsock2.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/config.h"
 #include "asterisk/netsock2.h"
 #include "asterisk/utils.h"
@@ -335,6 +333,27 @@ cleanup:
 	return res_cnt;
 }
 
+/*! \brief Pulls first resolved address and returns it */
+int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
+				      const char* name, int flag, int family)
+{
+	struct ast_sockaddr *addrs;
+	int addrs_cnt;
+
+	addrs_cnt = ast_sockaddr_resolve(&addrs, name, flag, family);
+	if (addrs_cnt <= 0) {
+		return 1;
+	}
+	if (addrs_cnt > 1) {
+		ast_debug(1, "Multiple addresses resolving %s, using the first one only\n", name);
+	}
+
+	ast_sockaddr_copy(addr, &addrs[0]);
+
+	ast_free(addrs);
+	return 0;
+}
+
 int ast_sockaddr_apply_netmask(const struct ast_sockaddr *addr, const struct ast_sockaddr *netmask,
 		struct ast_sockaddr *result)
 {
diff -pruN 1:13.23.1~dfsg-2/main/netsock.c 1:16.1.1~dfsg-1/main/netsock.c
--- 1:13.23.1~dfsg-2/main/netsock.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/netsock.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,203 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 1999 - 2005, Digium, Inc.
- *
- * Kevin P. Fleming <kpfleming@digium.com>
- * Mark Spencer <markster@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*! \file
- *
- * \brief Network socket handling
- *
- * \author Kevin P. Fleming <kpfleming@digium.com>
- * \author Mark Spencer <markster@digium.com>
- */
-
-/*** MODULEINFO
-	<support_level>core</support_level>
- ***/
-
-#include "asterisk.h"
-
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#if !defined (__linux__) && !defined (__GNU__)
-#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__Darwin__) || defined(__GLIBC__)
-#include <net/if_dl.h>
-#endif
-#endif
-
-#if defined (SOLARIS)
-#include <sys/sockio.h>
-#elif defined(HAVE_GETIFADDRS)
-#include <ifaddrs.h>
-#endif
-
-#include "asterisk/netsock.h"
-#include "asterisk/netsock2.h"
-#include "asterisk/utils.h"
-#include "asterisk/astobj.h"
-
-struct ast_netsock {
-	ASTOBJ_COMPONENTS(struct ast_netsock);
-	struct ast_sockaddr bindaddr;
-	int sockfd;
-	int *ioref;
-	struct io_context *ioc;
-	void *data;
-};
-
-struct ast_netsock_list {
-	ASTOBJ_CONTAINER_COMPONENTS(struct ast_netsock);
-	struct io_context *ioc;
-};
-
-static void ast_netsock_destroy(struct ast_netsock *netsock)
-{
-	ast_io_remove(netsock->ioc, netsock->ioref);
-	close(netsock->sockfd);
-	ast_free(netsock);
-}
-
-struct ast_netsock_list *ast_netsock_list_alloc(void)
-{
-	return ast_calloc(1, sizeof(struct ast_netsock_list));
-}
-
-int ast_netsock_init(struct ast_netsock_list *list)
-{
-	memset(list, 0, sizeof(*list));
-	ASTOBJ_CONTAINER_INIT(list);
-
-	return 0;
-}
-
-int ast_netsock_release(struct ast_netsock_list *list)
-{
-	ASTOBJ_CONTAINER_DESTROYALL(list, ast_netsock_destroy);
-	ASTOBJ_CONTAINER_DESTROY(list);
-	ast_free(list);
-
-	return 0;
-}
-
-struct ast_netsock *ast_netsock_find(struct ast_netsock_list *list, struct ast_sockaddr *addr)
-{
-	struct ast_netsock *sock = NULL;
-
-	ASTOBJ_CONTAINER_TRAVERSE(list, !sock, {
-		ASTOBJ_RDLOCK(iterator);
-		if (!ast_sockaddr_cmp(&iterator->bindaddr, addr)) {
-			sock = iterator;
-		}
-		ASTOBJ_UNLOCK(iterator);
-	});
-
-	return sock;
-}
-
-struct ast_netsock *ast_netsock_bindaddr(struct ast_netsock_list *list, struct io_context *ioc, struct ast_sockaddr *bindaddr, int tos, int cos, ast_io_cb callback, void *data)
-{
-	int netsocket = -1;
-	int *ioref;
-
-	struct ast_netsock *ns;
-	const int reuseFlag = 1;
-
-	/* Make a UDP socket */
-	netsocket = socket(ast_sockaddr_is_ipv6(bindaddr) ? AST_AF_INET6 : AST_AF_INET, SOCK_DGRAM, IPPROTO_IP);
-
-	if (netsocket < 0) {
-		ast_log(LOG_ERROR, "Unable to create network socket: %s\n", strerror(errno));
-		return NULL;
-	}
-	if (setsockopt(netsocket, SOL_SOCKET, SO_REUSEADDR, (char *)&reuseFlag, sizeof reuseFlag) < 0) {
-		ast_log(LOG_WARNING, "Error setting SO_REUSEADDR on sockfd '%d'\n", netsocket);
-	}
-	if (ast_bind(netsocket, bindaddr)) {
-		ast_log(LOG_ERROR,
-			"Unable to bind to %s: %s\n",
-			ast_sockaddr_stringify(bindaddr),
-			strerror(errno));
-		close(netsocket);
-		return NULL;
-	}
-
-	ast_set_qos(netsocket, tos, cos, "IAX2");
-
-	ast_enable_packet_fragmentation(netsocket);
-
-	if (!(ns = ast_calloc(1, sizeof(*ns)))) {
-		close(netsocket);
-		return NULL;
-	}
-
-	/* Establish I/O callback for socket read */
-	if (!(ioref = ast_io_add(ioc, netsocket, callback, AST_IO_IN, ns))) {
-		close(netsocket);
-		ast_free(ns);
-		return NULL;
-	}
-	ASTOBJ_INIT(ns);
-	ns->ioref = ioref;
-	ns->ioc = ioc;
-	ns->sockfd = netsocket;
-	ns->data = data;
-	ast_sockaddr_copy(&ns->bindaddr, bindaddr);
-	ASTOBJ_CONTAINER_LINK(list, ns);
-
-	return ns;
-}
-
-int ast_netsock_set_qos(int sockfd, int tos, int cos, const char *desc)
-{
-	return ast_set_qos(sockfd, tos, cos, desc);
-}
-
-struct ast_netsock *ast_netsock_bind(struct ast_netsock_list *list, struct io_context *ioc, const char *bindinfo, int defaultport, int tos, int cos, ast_io_cb callback, void *data)
-{
-	struct ast_sockaddr addr;
-
-	if (ast_sockaddr_parse(&addr, bindinfo, 0)) {
-
-		if (!ast_sockaddr_port(&addr)) {
-			ast_sockaddr_set_port(&addr, defaultport);
-		}
-
-		return ast_netsock_bindaddr(list, ioc, &addr, tos, cos, callback, data);
-	}
-
-	return NULL;
-}
-
-int ast_netsock_sockfd(const struct ast_netsock *ns)
-{
-	return ns ? ns-> sockfd : -1;
-}
-
-const struct ast_sockaddr *ast_netsock_boundaddr(const struct ast_netsock *ns)
-{
-	return &ns->bindaddr;
-}
-
-void *ast_netsock_data(const struct ast_netsock *ns)
-{
-	return ns->data;
-}
-
-void ast_netsock_unref(struct ast_netsock *ns)
-{
-	ASTOBJ_UNREF(ns, ast_netsock_destroy);
-}
diff -pruN 1:13.23.1~dfsg-2/main/optional_api.c 1:16.1.1~dfsg-1/main/optional_api.c
--- 1:13.23.1~dfsg-2/main/optional_api.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/optional_api.c	2018-12-26 18:45:13.000000000 +0000
@@ -18,10 +18,9 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/optional_api.h"
 #include "asterisk/utils.h"
+#include "asterisk/vector.h"
 
 #if defined(OPTIONAL_API)
 
@@ -58,16 +57,18 @@ struct optional_api_user {
 struct optional_api {
 	/*! Pointer to the implementation function; could be null */
 	ast_optional_fn impl;
-	/*! Variable length array of users of this API */
-	struct optional_api_user **users;
-	/*! Allocated size of the \a users array */
-	size_t users_maxlen;
-	/*! Number of entries in the \a users array */
-	size_t users_len;
+	/*! Users of the API */
+	AST_VECTOR(, struct optional_api_user *) users;
 	/*! Name of the optional API function */
 	char symname[];
 };
 
+/*! Vector of \ref optional_api functions */
+AST_VECTOR(, struct optional_api *) apis;
+
+#define USER_OPTIONAL_REF_CMP(ele, value) (ele->optional_ref == value)
+#define OPTIONAL_API_SYMNAME_CMP(ele, value) (!strcmp(ele->symname, value))
+
 /*!
  * \brief Free an \ref optional_api_user.
  *
@@ -76,7 +77,7 @@ struct optional_api {
 static void optional_api_user_destroy(struct optional_api_user *user)
 {
 	*user->optional_ref = user->stub;
-	ast_std_free(user);
+	ast_free(user);
 }
 
 /*!
@@ -95,8 +96,10 @@ static struct optional_api_user *optiona
 	struct optional_api_user *user;
 	size_t size = sizeof(*user) + strlen(module) + 1;
 
-	user = ast_std_calloc(1, size);
+	user = ast_calloc(1, size);
 	if (!user) {
+		ast_do_crash();
+
 		return NULL;
 	}
 
@@ -114,17 +117,15 @@ static struct optional_api_user *optiona
  */
 static void optional_api_destroy(struct optional_api *api)
 {
-	while (api->users_len--) {
-		optional_api_user_destroy(api->users[api->users_len]);
-	}
-	ast_std_free(api->users);
-	api->users = NULL;
-	api->users_maxlen = 0;
-	ast_std_free(api);
+	AST_VECTOR_REMOVE_CMP_UNORDERED(&apis, api,
+		AST_VECTOR_ELEM_DEFAULT_CMP, AST_VECTOR_ELEM_CLEANUP_NOOP);
+	AST_VECTOR_CALLBACK_VOID(&api->users, optional_api_user_destroy);
+	AST_VECTOR_FREE(&api->users);
+	ast_free(api);
 }
 
 /*!
- * \brief Create an \ref optional_api.
+ * \brief Create and link an \ref optional_api.
  *
  * \param symname Name of the optional function.
  * \return New \ref optional_api.
@@ -133,12 +134,12 @@ static void optional_api_destroy(struct
 static struct optional_api *optional_api_create(const char *symname)
 {
 	struct optional_api *api;
-	size_t size;
 
-	size = sizeof(*api) + strlen(symname) + 1;
-	api = ast_std_calloc(1, size);
-	if (!api) {
-		ast_log(LOG_ERROR, "Failed to allocate api\n");
+	api = ast_calloc(1, sizeof(*api) + strlen(symname) + 1);
+	if (!api || AST_VECTOR_APPEND(&apis, api)) {
+		ast_free(api);
+		ast_do_crash();
+
 		return NULL;
 	}
 
@@ -147,16 +148,6 @@ static struct optional_api *optional_api
 	return api;
 }
 
-/*! Array of \ref optional_api functions */
-struct {
-	/*! Variable length array of API's */
-	struct optional_api **list;
-	/*! Allocated size of the \a list array */
-	size_t maxlen;
-	/*! Number of entries in the \a list array */
-	size_t len;
-} apis;
-
 /*!
  * \brief Gets (or creates) the \ref optional_api for the given function.
  *
@@ -166,43 +157,16 @@ struct {
  */
 static struct optional_api *get_api(const char *symname)
 {
-	struct optional_api *api;
-	size_t i;
+	struct optional_api **api;
 
 	/* Find one, if we already have it */
-	if (apis.list) {
-		for (i = 0; i < apis.len; ++i) {
-			if (strcmp(symname, apis.list[i]->symname) == 0) {
-				return apis.list[i];
-			}
-		}
+	api = AST_VECTOR_GET_CMP(&apis, symname, OPTIONAL_API_SYMNAME_CMP);
+	if (api) {
+		return *api;
 	}
 
 	/* API not found. Build one */
-	api = optional_api_create(symname);
-	if (!api) {
-		return NULL;
-	}
-
-	/* Grow the list, if needed */
-	if (apis.len + 1 > apis.maxlen) {
-		size_t new_maxlen = apis.maxlen ? 2 * apis.maxlen : 1;
-		struct optional_api **new_list;
-
-		new_list = ast_std_realloc(apis.list, new_maxlen * sizeof(*new_list));
-		if (!new_list) {
-			optional_api_destroy(api);
-			ast_log(LOG_ERROR, "Failed to allocate api list\n");
-			return NULL;
-		}
-
-		apis.maxlen = new_maxlen;
-		apis.list = new_list;
-	}
-
-	apis.list[apis.len++] = api;
-
-	return api;
+	return optional_api_create(symname);
 }
 
 /*!
@@ -234,13 +198,14 @@ static void optional_api_user_relink(str
 static void optional_api_set_impl(struct optional_api *api,
 	ast_optional_fn impl)
 {
-	size_t i;
-
 	api->impl = impl;
 
 	/* re-link all users */
-	for (i = 0; i < api->users_len; ++i) {
-		optional_api_user_relink(api->users[i], api);
+	if (AST_VECTOR_SIZE(&api->users)) {
+		AST_VECTOR_CALLBACK_VOID(&api->users, optional_api_user_relink, api);
+	} else if (!impl) {
+		/* No users or impl means we should delete this api. */
+		optional_api_destroy(api);
 	}
 }
 
@@ -249,13 +214,9 @@ void ast_optional_api_provide(const char
 	struct optional_api *api;
 
 	api = get_api(symname);
-	if (!api) {
-		ast_log(LOG_ERROR, "%s: Allocation failed\n", symname);
-		ast_do_crash();
-		return;
+	if (api) {
+		optional_api_set_impl(api, impl);
 	}
-
-	optional_api_set_impl(api, impl);
 }
 
 void ast_optional_api_unprovide(const char *symname, ast_optional_fn impl)
@@ -263,13 +224,9 @@ void ast_optional_api_unprovide(const ch
 	struct optional_api *api;
 
 	api = get_api(symname);
-	if (!api) {
-		ast_log(LOG_ERROR, "%s: Could not find api\n", symname);
-		ast_do_crash();
-		return;
+	if (api) {
+		optional_api_set_impl(api, 0);
 	}
-
-	optional_api_set_impl(api, 0);
 }
 
 void ast_optional_api_use(const char *symname, ast_optional_fn *optional_ref,
@@ -278,73 +235,36 @@ void ast_optional_api_use(const char *sy
 	struct optional_api_user *user;
 	struct optional_api *api;
 
-
 	api = get_api(symname);
 	if (!api) {
-		ast_log(LOG_ERROR, "%s: Allocation failed\n", symname);
-		ast_do_crash();
 		return;
 	}
 
 	user = optional_api_user_create(optional_ref, stub, module);
 	if (!user) {
-		ast_log(LOG_ERROR, "%s: Allocation failed\n", symname);
-		ast_do_crash();
 		return;
 	}
 
 	/* Add user to the API */
-	if (api->users_len + 1 > api->users_maxlen) {
-		size_t new_maxlen = api->users_maxlen ? 2 * api->users_maxlen : 1;
-		struct optional_api_user **new_list;
-
-		new_list = ast_std_realloc(api->users, new_maxlen * sizeof(*new_list));
-		if (!new_list) {
-			optional_api_user_destroy(user);
-			ast_log(LOG_ERROR, "Failed to allocate api list\n");
-			ast_do_crash();
-			return;
-		}
-
-		api->users_maxlen = new_maxlen;
-		api->users = new_list;
+	if (!AST_VECTOR_APPEND(&api->users, user)) {
+		optional_api_user_relink(user, api);
+	} else {
+		optional_api_user_destroy(user);
+		ast_do_crash();
 	}
-
-	api->users[api->users_len++] = user;
-
-	optional_api_user_relink(user, api);
 }
 
 void ast_optional_api_unuse(const char *symname, ast_optional_fn *optional_ref,
 	const char *module)
 {
 	struct optional_api *api;
-	size_t i;
 
 	api = get_api(symname);
-	if (!api) {
-		ast_log(LOG_ERROR, "%s: Could not find api\n", symname);
-		ast_do_crash();
-		return;
-	}
-
-	for (i = 0; i < api->users_len; ++i) {
-		struct optional_api_user *user = api->users[i];
-
-		if (user->optional_ref == optional_ref) {
-			if (*user->optional_ref != user->stub) {
-				*user->optional_ref = user->stub;
-			}
-
-			/* Remove from the list */
-			api->users[i] = api->users[--api->users_len];
-
-			optional_api_user_destroy(user);
-			return;
+	if (api) {
+		AST_VECTOR_REMOVE_CMP_UNORDERED(&api->users, optional_ref, USER_OPTIONAL_REF_CMP, optional_api_user_destroy);
+		if (!api->impl && !AST_VECTOR_SIZE(&api->users)) {
+			optional_api_destroy(api);
 		}
 	}
-
-	ast_log(LOG_ERROR, "%s: Could not find user %s\n", symname, module);
 }
-
 #endif /* defined(OPTIONAL_API) */
diff -pruN 1:13.23.1~dfsg-2/main/options.c 1:16.1.1~dfsg-1/main/options.c
--- 1:13.23.1~dfsg-2/main/options.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/options.c	2018-12-26 18:45:13.000000000 +0000
@@ -83,8 +83,8 @@ unsigned int option_dtmfminduration = AS
 /*! Minimum amount of free system memory - stop accepting calls if free memory falls below this watermark */
 long option_minmemfree;
 #endif
-
-unsigned int ast_option_rtpptdynamic = AST_RTP_PT_FIRST_DYNAMIC;
+int ast_option_rtpusedynamic = 1;
+unsigned int ast_option_rtpptdynamic = 35;
 
 /*! @} */
 
@@ -206,7 +206,6 @@ void load_asterisk_conf(void)
 {
 	struct ast_config *cfg;
 	struct ast_variable *v;
-	char *config = DEFAULT_CONFIG_FILE;
 	char hostname[MAXHOSTNAMELEN] = "";
 	struct ast_flags config_flags = { CONFIG_FLAG_NOREALTIME };
 	struct {
@@ -215,20 +214,27 @@ void load_asterisk_conf(void)
 	} found = { 0, 0 };
 	/* Default to false for security */
 	int live_dangerously = 0;
+	int option_debug_new = 0;
+	int option_verbose_new = 0;
 
-	if (ast_opt_override_config) {
-		cfg = ast_config_load2(ast_config_AST_CONFIG_FILE, "" /* core, can't reload */, config_flags);
-		if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID) {
-			fprintf(stderr, "Unable to open specified master config file '%s', using built-in defaults\n", ast_config_AST_CONFIG_FILE);
-		}
-	} else {
-		cfg = ast_config_load2(config, "" /* core, can't reload */, config_flags);
-	}
+	/* init with buildtime config */
+#ifdef REF_DEBUG
+	/* The REF_DEBUG compiler flag is now only used to enable refdebug by default.
+	 * Support for debugging reference counts is always compiled in. */
+	ast_set2_flag(&ast_options, 1, AST_OPT_FLAG_REF_DEBUG);
+#endif
 
 	ast_set_default_eid(&ast_eid_default);
 
+	cfg = ast_config_load2(ast_config_AST_CONFIG_FILE, "" /* core, can't reload */, config_flags);
+
+	/* If AST_OPT_FLAG_EXEC_INCLUDES was previously enabled with -X turn it off now.
+	 * Using #exec from other configs requires that it be enabled from asterisk.conf. */
+	ast_clear_flag(&ast_options, AST_OPT_FLAG_EXEC_INCLUDES);
+
 	/* no asterisk.conf? no problem, use buildtime config! */
 	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID) {
+		fprintf(stderr, "Unable to open specified master config file '%s', using built-in defaults\n", ast_config_AST_CONFIG_FILE);
 		return;
 	}
 
@@ -288,7 +294,7 @@ void load_asterisk_conf(void)
 	for (v = ast_variable_browse(cfg, "options"); v; v = v->next) {
 		/* verbose level (-v at startup) */
 		if (!strcasecmp(v->name, "verbose")) {
-			option_verbose = atoi(v->value);
+			option_verbose_new = atoi(v->value);
 		/* whether or not to force timestamping in CLI verbose output. (-T at startup) */
 		} else if (!strcasecmp(v->name, "timestamp")) {
 			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_TIMESTAMP);
@@ -297,10 +303,12 @@ void load_asterisk_conf(void)
 			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_EXEC_INCLUDES);
 		/* debug level (-d at startup) */
 		} else if (!strcasecmp(v->name, "debug")) {
-			option_debug = 0;
-			if (sscanf(v->value, "%30d", &option_debug) != 1) {
-				option_debug = ast_true(v->value) ? 1 : 0;
+			option_debug_new = 0;
+			if (sscanf(v->value, "%30d", &option_debug_new) != 1) {
+				option_debug_new = ast_true(v->value) ? 1 : 0;
 			}
+		} else if (!strcasecmp(v->name, "refdebug")) {
+			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_REF_DEBUG);
 #if HAVE_WORKING_FORK
 		/* Disable forking (-f at startup) */
 		} else if (!strcasecmp(v->name, "nofork")) {
@@ -314,7 +322,9 @@ void load_asterisk_conf(void)
 			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_QUIET);
 		/* Run as console (-c at startup, implies nofork) */
 		} else if (!strcasecmp(v->name, "console")) {
-			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_NO_FORK | AST_OPT_FLAG_CONSOLE);
+			if (!ast_opt_remote) {
+				ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_NO_FORK | AST_OPT_FLAG_CONSOLE);
+			}
 		/* Run with high priority if the O/S permits (-p at startup) */
 		} else if (!strcasecmp(v->name, "highpriority")) {
 			ast_set2_flag(&ast_options, ast_true(v->value), AST_OPT_FLAG_HIGH_PRIORITY);
@@ -358,12 +368,13 @@ void load_asterisk_conf(void)
 			if (sscanf(v->value, "%30u", &option_dtmfminduration) != 1) {
 				option_dtmfminduration = AST_MIN_DTMF_DURATION;
 			}
+		} else if (!strcasecmp(v->name, "rtp_use_dynamic")) {
+			ast_option_rtpusedynamic = ast_true(v->value);
 		/* http://www.iana.org/assignments/rtp-parameters
 		 * RTP dynamic payload types start at 96 normally; extend down to 0 */
 		} else if (!strcasecmp(v->name, "rtp_pt_dynamic")) {
-			ast_parse_arg(v->value, PARSE_UINT32|PARSE_IN_RANGE|PARSE_DEFAULT,
-			              &ast_option_rtpptdynamic, AST_RTP_PT_FIRST_DYNAMIC,
-			              0, AST_RTP_PT_LAST_REASSIGN);
+			ast_parse_arg(v->value, PARSE_UINT32|PARSE_IN_RANGE,
+			              &ast_option_rtpptdynamic, 0, AST_RTP_PT_FIRST_DYNAMIC);
 		} else if (!strcasecmp(v->name, "maxcalls")) {
 			if ((sscanf(v->value, "%30d", &ast_option_maxcalls) != 1) || (ast_option_maxcalls < 0)) {
 				ast_option_maxcalls = 0;
@@ -459,5 +470,8 @@ void load_asterisk_conf(void)
 		pbx_live_dangerously(live_dangerously);
 	}
 
+	option_debug += option_debug_new;
+	option_verbose += option_verbose_new;
+
 	ast_config_destroy(cfg);
 }
diff -pruN 1:13.23.1~dfsg-2/main/parking.c 1:16.1.1~dfsg-1/main/parking.c
--- 1:13.23.1~dfsg-2/main/parking.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/parking.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/pbx.h"
@@ -134,8 +132,8 @@ int ast_parking_park_bridge_channel(stru
 		return -1;
 	}
 
-	if (table->module_info) {
-		SCOPED_MODULE_USE(table->module_info->self);
+	if (table->module) {
+		SCOPED_MODULE_USE(table->module);
 		return table->parking_park_bridge_channel(parkee, parkee_uuid, parker_uuid, app_data);
 	}
 
@@ -153,8 +151,8 @@ int ast_parking_blind_transfer_park(stru
 		return -1;
 	}
 
-	if (table->module_info) {
-		SCOPED_MODULE_USE(table->module_info->self);
+	if (table->module) {
+		SCOPED_MODULE_USE(table->module);
 		return table->parking_blind_transfer_park(parker, context, exten, parked_channel_cb, parked_channel_data);
 	}
 
@@ -170,8 +168,8 @@ int ast_parking_park_call(struct ast_bri
 		return -1;
 	}
 
-	if (table->module_info) {
-		SCOPED_MODULE_USE(table->module_info->self);
+	if (table->module) {
+		SCOPED_MODULE_USE(table->module);
 		return table->parking_park_call(parker, exten, length);
 	}
 
@@ -187,8 +185,8 @@ int ast_parking_is_exten_park(const char
 		return -1;
 	}
 
-	if (table->module_info) {
-		SCOPED_MODULE_USE(table->module_info->self);
+	if (table->module) {
+		SCOPED_MODULE_USE(table->module);
 		return table->parking_is_exten_park(context, exten);
 	}
 
diff -pruN 1:13.23.1~dfsg-2/main/pbx_app.c 1:16.1.1~dfsg-1/main/pbx_app.c
--- 1:13.23.1~dfsg-2/main/pbx_app.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_app.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/cli.h"
 #include "asterisk/linkedlists.h"
diff -pruN 1:13.23.1~dfsg-2/main/pbx_builtins.c 1:16.1.1~dfsg-1/main/pbx_builtins.c
--- 1:13.23.1~dfsg-2/main/pbx_builtins.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_builtins.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h"
 #include "asterisk/pbx.h"
 #include "asterisk/causes.h"
@@ -582,6 +580,42 @@ ASTERISK_REGISTER_FILE()
 			<para>This application waits for a specified number of <replaceable>seconds</replaceable>.</para>
 		</description>
 	</application>
+	<application name="WaitDigit" language="en_US">
+		<synopsis>
+			Waits for a digit to be entered.
+		</synopsis>
+		<syntax>
+			<parameter name="seconds">
+				<para>Can be passed with fractions of a second. For example, <literal>1.5</literal> will ask the
+				application to wait for 1.5 seconds.</para>
+			</parameter>
+			<parameter name="digits">
+				<para>Digits to accept, all others are ignored.</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>This application waits for the user to press one of the accepted
+			<replaceable>digits</replaceable> for a specified number of
+			<replaceable>seconds</replaceable>.</para>
+			<variablelist>
+				<variable name="WAITDIGITSTATUS">
+					<para>This is the final status of the command</para>
+					<value name="ERROR">Parameters are invalid.</value>
+					<value name="DTMF">An accepted digit was received.</value>
+					<value name="TIMEOUT">The timeout passed before any acceptable digits were received.</value>
+					<value name="CANCEL">The channel has hungup or was redirected.</value>
+				</variable>
+				<variable name="WAITDIGITRESULT">
+					<para>The digit that was received, only set if
+					<variable>WAITDIGITSTATUS</variable> is <literal>DTMF</literal>.</para>
+				</variable>
+			</variablelist>
+		</description>
+		<see-also>
+			<ref type="application">Wait</ref>
+			<ref type="application">WaitExten</ref>
+		</see-also>
+	</application>
 	<application name="WaitExten" language="en_US">
 		<synopsis>
 			Waits for an extension to be entered.
@@ -959,6 +993,47 @@ static int pbx_builtin_wait(struct ast_c
 /*!
  * \ingroup applications
  */
+static int pbx_builtin_waitdigit(struct ast_channel *chan, const char *data)
+{
+	int res;
+	int ms;
+	char *parse;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(timeout);
+		AST_APP_ARG(digits);
+	);
+
+	parse = ast_strdupa(data);
+	AST_STANDARD_APP_ARGS(args, parse);
+
+	if (ast_app_parse_timelen(args.timeout, &ms, TIMELEN_SECONDS) || ms < 0) {
+		pbx_builtin_setvar_helper(chan, "WAITDIGITSTATUS", "ERROR");
+		return 0;
+	}
+
+	/* Wait for "n" seconds */
+	res = ast_waitfordigit_full(chan, ms, S_OR(args.digits, AST_DIGIT_ANY), -1, -1);
+	if (res < 0) {
+		pbx_builtin_setvar_helper(chan, "WAITDIGITSTATUS", "CANCEL");
+		return -1;
+	}
+
+	if (res == 0) {
+		pbx_builtin_setvar_helper(chan, "WAITDIGITSTATUS", "TIMEOUT");
+	} else {
+		char key[2];
+
+		snprintf(key, sizeof(key), "%c", res);
+		pbx_builtin_setvar_helper(chan, "WAITDIGITRESULT", key);
+		pbx_builtin_setvar_helper(chan, "WAITDIGITSTATUS", "DTMF");
+	}
+
+	return 0;
+}
+
+/*!
+ * \ingroup applications
+ */
 static int pbx_builtin_waitexten(struct ast_channel *chan, const char *data)
 {
 	int ms, res;
@@ -1412,6 +1487,7 @@ struct pbx_builtin {
 	{ "SayPhonetic",    pbx_builtin_sayphonetic },
 	{ "SetAMAFlags",    pbx_builtin_setamaflags },
 	{ "Wait",           pbx_builtin_wait },
+	{ "WaitDigit",      pbx_builtin_waitdigit },
 	{ "WaitExten",      pbx_builtin_waitexten }
 };
 
diff -pruN 1:13.23.1~dfsg-2/main/pbx.c 1:16.1.1~dfsg-1/main/pbx.c
--- 1:13.23.1~dfsg-2/main/pbx.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_SYSTEM_NAME */
 #include <ctype.h>
@@ -253,38 +251,12 @@ struct ast_exten {
 	struct ast_hashtab *peer_table;    /*!< Priorities list in hashtab form -- only on the head of the peer list */
 	struct ast_hashtab *peer_label_table; /*!< labeled priorities in the peers -- only on the head of the peer list */
 	const char *registrar;		/*!< Registrar */
+	const char *registrar_file;     /*!< File name used to register extension */
+	int registrar_line;             /*!< Line number the extension was registered in text */
 	struct ast_exten *next;		/*!< Extension with a greater ID */
 	char stuff[0];
 };
 
-/*! \brief ast_include: include= support in extensions.conf */
-struct ast_include {
-	const char *name;
-	const char *rname;			/*!< Context to include */
-	const char *registrar;			/*!< Registrar */
-	int hastime;				/*!< If time construct exists */
-	struct ast_timing timing;               /*!< time construct */
-	struct ast_include *next;		/*!< Link them together */
-	char stuff[0];
-};
-
-/*! \brief ast_sw: Switch statement in extensions.conf */
-struct ast_sw {
-	char *name;
-	const char *registrar;			/*!< Registrar */
-	char *data;				/*!< Data load */
-	int eval;
-	AST_LIST_ENTRY(ast_sw) list;
-	char stuff[0];
-};
-
-/*! \brief ast_ignorepat: Ignore patterns in dial plan */
-struct ast_ignorepat {
-	const char *registrar;
-	struct ast_ignorepat *next;
-	const char pattern[0];
-};
-
 /*! \brief match_char: forms a syntax tree for quick matching of extension patterns */
 struct match_char
 {
@@ -308,18 +280,19 @@ struct scoreboard  /* make sure all fiel
 	struct ast_exten *exten;
 };
 
-/*! \brief ast_context: An extension context */
+/*! \brief ast_context: An extension context - must remain in sync with fake_context */
 struct ast_context {
 	ast_rwlock_t lock;			/*!< A lock to prevent multiple threads from clobbering the context */
 	struct ast_exten *root;			/*!< The root of the list of extensions */
 	struct ast_hashtab *root_table;            /*!< For exact matches on the extensions in the pattern tree, and for traversals of the pattern_tree  */
 	struct match_char *pattern_tree;        /*!< A tree to speed up extension pattern matching */
 	struct ast_context *next;		/*!< Link them together */
-	struct ast_include *includes;		/*!< Include other contexts */
-	struct ast_ignorepat *ignorepats;	/*!< Patterns for which to continue playing dialtone */
+	struct ast_includes includes;		/*!< Include other contexts */
+	struct ast_ignorepats ignorepats;	/*!< Patterns for which to continue playing dialtone */
+	struct ast_sws alts;			/*!< Alternative switches */
 	char *registrar;			/*!< Registrar -- make sure you malloc this, as the registrar may have to survive module unloads */
 	int refcount;                   /*!< each module that would have created this context should inc/dec this as appropriate */
-	AST_LIST_HEAD_NOLOCK(, ast_sw) alts;	/*!< Alternative switches */
+	int autohints;                  /*!< Whether autohints support is enabled or not */
 	ast_mutex_t macrolock;			/*!< A lock to implement "exclusive" macros - held whilst a call is executing in the macro */
 	char name[0];				/*!< Name of the context */
 };
@@ -402,6 +375,19 @@ struct ast_hintdevice {
 	char hintdevice[1];
 };
 
+/*! \brief Container for autohint contexts */
+static struct ao2_container *autohints;
+
+/*!
+ * \brief Structure for dial plan autohints
+ */
+struct ast_autohint {
+	/*! \brief Name of the registrar */
+	char *registrar;
+	/*! \brief Name of the context */
+	char context[1];
+};
+
 /*!
  * \note Using the device for hash
  */
@@ -425,6 +411,7 @@ static int hintdevice_hash_cb(const void
 
 	return ast_str_case_hash(key);
 }
+
 /*!
  * \note Devices on hints are not unique so no CMP_STOP is returned
  * Dont use ao2_find against hintdevices container cause there always
@@ -459,6 +446,59 @@ static int hintdevice_cmp_multiple(void
 	return cmp ? 0 : CMP_MATCH;
 }
 
+/*!
+ * \note Using the context name for hash
+ */
+static int autohint_hash_cb(const void *obj, const int flags)
+{
+	const struct ast_autohint *autohint;
+	const char *key;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_KEY:
+		key = obj;
+		break;
+	case OBJ_SEARCH_OBJECT:
+		autohint = obj;
+		key = autohint->context;
+		break;
+	default:
+		ast_assert(0);
+		return 0;
+	}
+
+	return ast_str_case_hash(key);
+}
+
+static int autohint_cmp(void *obj, void *arg, int flags)
+{
+	struct ast_autohint *left = obj;
+	struct ast_autohint *right = arg;
+	const char *right_key = arg;
+	int cmp;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_OBJECT:
+		right_key = right->context;
+		/* Fall through */
+	case OBJ_SEARCH_KEY:
+		cmp = strcasecmp(left->context, right_key);
+		break;
+	case OBJ_SEARCH_PARTIAL_KEY:
+		/*
+		* We could also use a partial key struct containing a length
+		* so strlen() does not get called for every comparison instead.
+		*/
+		cmp = strncmp(left->context, right_key, strlen(right_key));
+		break;
+	default:
+		ast_assert(0);
+		cmp = 0;
+		break;
+	}
+	return cmp ? 0 : CMP_MATCH | CMP_STOP;
+}
+
 /*! \internal \brief \c ao2_callback function to remove hintdevices */
 static int hintdevice_remove_cb(void *obj, void *arg, void *data, int flags)
 {
@@ -524,7 +564,7 @@ static int add_hintdevice(struct ast_hin
 		return -1;
 	}
 	ast_str_set(&str, 0, "%s", devicelist);
-	parse = parse_hint_device(str);
+	parse = ast_str_buffer(str);
 
 	/* Spit on '&' and ',' to handle presence hints as well */
 	while ((cur = strsep(&parse, "&,"))) {
@@ -612,7 +652,8 @@ static int ast_add_extension_nolock(cons
 static int ast_add_extension2_lockopt(struct ast_context *con,
 	int replace, const char *extension, int priority, const char *label, const char *callerid,
 	const char *application, void *data, void (*datad)(void *),
-	const char *registrar, int lock_context);
+	const char *registrar, const char *registrar_file, int registrar_line,
+	int lock_context);
 static struct ast_context *find_context_locked(const char *context);
 static struct ast_context *find_context(const char *context);
 static void get_device_state_causing_channels(struct ao2_container *c);
@@ -940,14 +981,6 @@ int check_contexts(char *file, int line
 }
 #endif
 
-static inline int include_valid(struct ast_include *i)
-{
-	if (!i->hastime)
-		return 1;
-
-	return ast_check_timing(&(i->timing));
-}
-
 static void pbx_destroy(struct ast_pbx *p)
 {
 	ast_free(p);
@@ -2373,6 +2406,7 @@ int ast_extension_close(const char *patt
 	return extension_match_core(pattern, data, needmore);
 }
 
+/* This structure must remain in sync with ast_context for proper hashtab matching */
 struct fake_context /* this struct is purely for matching in the hashtab */
 {
 	ast_rwlock_t lock;
@@ -2380,11 +2414,12 @@ struct fake_context /* this struct is pu
 	struct ast_hashtab *root_table;
 	struct match_char *pattern_tree;
 	struct ast_context *next;
-	struct ast_include *includes;
-	struct ast_ignorepat *ignorepats;
+	struct ast_includes includes;
+	struct ast_ignorepats ignorepats;
+	struct ast_sws alts;
 	const char *registrar;
 	int refcount;
-	AST_LIST_HEAD_NOLOCK(, ast_sw) alts;
+	int autohints;
 	ast_mutex_t macrolock;
 	char name[256];
 };
@@ -2439,11 +2474,10 @@ struct ast_exten *pbx_find_extension(str
 	int x, res;
 	struct ast_context *tmp = NULL;
 	struct ast_exten *e = NULL, *eroot = NULL;
-	struct ast_include *i = NULL;
-	struct ast_sw *sw = NULL;
 	struct ast_exten pattern = {NULL, };
 	struct scoreboard score = {0, };
 	struct ast_str *tmpdata = NULL;
+	int idx;
 
 	pattern.label = label;
 	pattern.priority = priority;
@@ -2663,23 +2697,28 @@ struct ast_exten *pbx_find_extension(str
 	}
 
 	/* Check alternative switches */
-	AST_LIST_TRAVERSE(&tmp->alts, sw, list) {
-		struct ast_switch *asw = pbx_findswitch(sw->name);
+	for (idx = 0; idx < ast_context_switches_count(tmp); idx++) {
+		const struct ast_sw *sw = ast_context_switches_get(tmp, idx);
+		struct ast_switch *asw = pbx_findswitch(ast_get_switch_name(sw));
 		ast_switch_f *aswf = NULL;
-		char *datap;
+		const char *datap;
 
 		if (!asw) {
-			ast_log(LOG_WARNING, "No such switch '%s'\n", sw->name);
+			ast_log(LOG_WARNING, "No such switch '%s'\n", ast_get_switch_name(sw));
 			continue;
 		}
 
 		/* Substitute variables now */
-		if (sw->eval) {
+		if (ast_get_switch_eval(sw)) {
 			if (!(tmpdata = ast_str_thread_get(&switch_data, 512))) {
 				ast_log(LOG_WARNING, "Can't evaluate switch?!\n");
 				continue;
 			}
-			pbx_substitute_variables_helper(chan, sw->data, ast_str_buffer(tmpdata), ast_str_size(tmpdata));
+			pbx_substitute_variables_helper(chan, ast_get_switch_data(sw),
+				ast_str_buffer(tmpdata), ast_str_size(tmpdata));
+			datap = ast_str_buffer(tmpdata);
+		} else {
+			datap = ast_get_switch_data(sw);
 		}
 
 		/* equivalent of extension_match_core() at the switch level */
@@ -2689,7 +2728,6 @@ struct ast_exten *pbx_find_extension(str
 			aswf = asw->matchmore;
 		else /* action == E_MATCH */
 			aswf = asw->exists;
-		datap = sw->eval ? ast_str_buffer(tmpdata) : sw->data;
 		if (!aswf)
 			res = 0;
 		else {
@@ -2709,9 +2747,11 @@ struct ast_exten *pbx_find_extension(str
 	}
 	q->incstack[q->stacklen++] = tmp->name;	/* Setup the stack */
 	/* Now try any includes we have in this context */
-	for (i = tmp->includes; i; i = i->next) {
+	for (idx = 0; idx < ast_context_includes_count(tmp); idx++) {
+		const struct ast_include *i = ast_context_includes_get(tmp, idx);
+
 		if (include_valid(i)) {
-			if ((e = pbx_find_extension(chan, bypass, q, i->rname, exten, priority, label, callerid, action))) {
+			if ((e = pbx_find_extension(chan, bypass, q, include_rname(i), exten, priority, label, callerid, action))) {
 #ifdef NEED_DEBUG_HERE
 				ast_log(LOG_NOTICE,"Returning recursive match of %s\n", e->exten);
 #endif
@@ -3014,7 +3054,7 @@ static void device_state_info_dt(void *o
 
 static struct ao2_container *alloc_device_state_info(void)
 {
-	return ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL);
+	return ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
 }
 
 static int ast_extension_state3(struct ast_str *hint_app, struct ao2_container *device_state_info)
@@ -3213,8 +3253,7 @@ static int execute_state_callback(ast_st
 		info.exten_state = AST_EXTENSION_REMOVED;
 	}
 
-	/* NOTE: The casts will not be needed for v10 and later */
-	res = cb((char *) context, (char *) exten, &info, data);
+	res = cb(context, exten, &info, data);
 
 	return res;
 }
@@ -3394,9 +3433,8 @@ static void device_state_notify_callback
 	ao2_cleanup(device_state_info);
 }
 
-static void presence_state_notify_callbacks(
-	struct stasis_message *msg, struct ast_hint *hint, struct ast_str **hint_app,
-	struct ast_presence_state_message *presence_state)
+static void presence_state_notify_callbacks(struct ast_hint *hint, struct ast_str **hint_app,
+					    struct ast_presence_state_message *presence_state)
 {
 	struct ao2_iterator cb_iter;
 	struct ast_state_cb *state_cb;
@@ -3404,38 +3442,12 @@ static void presence_state_notify_callba
 	char exten_name[AST_MAX_EXTENSION];
 
 	ao2_lock(hint);
-
 	if (!hint->exten) {
 		/* The extension has already been destroyed */
 		ao2_unlock(hint);
 		return;
 	}
 
-	if (hint_change_message_type() != stasis_message_type(msg)) {
-		const char *app;
-		char *parse;
-
-		/* Does this hint monitor the device that changed state? */
-		app = ast_get_extension_app(hint->exten);
-		if (ast_strlen_zero(app)) {
-			/* The hint does not monitor presence at all. */
-			ao2_unlock(hint);
-			return;
-		}
-
-		ast_str_set(hint_app, 0, "%s", app);
-		parse = parse_hint_presence(*hint_app);
-		if (ast_strlen_zero(parse)) {
-			ao2_unlock(hint);
-			return;
-		}
-		if (strcasecmp(parse, presence_state->provider)) {
-			/* The hint does not monitor the presence provider. */
-			ao2_unlock(hint);
-			return;
-		}
-	}
-
 	/*
 	 * Save off strings in case the hint extension gets destroyed
 	 * while we are notifying the watchers.
@@ -3446,6 +3458,7 @@ static void presence_state_notify_callba
 	ast_copy_string(exten_name, ast_get_extension_name(hint->exten),
 			sizeof(exten_name));
 	ast_str_set(hint_app, 0, "%s", ast_get_extension_app(hint->exten));
+	ao2_unlock(hint);
 
 	/* Check to see if update is necessary */
 	if ((hint->last_presence_state == presence_state->state) &&
@@ -3456,7 +3469,6 @@ static void presence_state_notify_callba
 	      !strcmp(hint->last_presence_message, presence_state->message)) ||
 	     (!hint->last_presence_message && !presence_state->message))) {
 		/* this update is the same as the last, do nothing */
-		ao2_unlock(hint);
 		return;
 	}
 
@@ -3467,13 +3479,6 @@ static void presence_state_notify_callba
 	hint->last_presence_subtype = presence_state->subtype ? ast_strdup(presence_state->subtype) : NULL;
 	hint->last_presence_message = presence_state->message ? ast_strdup(presence_state->message) : NULL;
 
-	/*
-	 * NOTE: We cannot hold any locks while notifying
-	 * the watchers without causing a deadlock.
-	 * (conlock, hints, and hint)
-	 */
-	ao2_unlock(hint);
-
 	/* For general callbacks */
 	cb_iter = ao2_iterator_init(statecbs, 0);
 	for (; (state_cb = ao2_iterator_next(&cb_iter)); ao2_ref(state_cb, -1)) {
@@ -3535,7 +3540,7 @@ static int handle_hint_change_message_ty
 					.message = presence_message
 				};
 
-				presence_state_notify_callbacks(msg, hint, &hint_app, &presence_state);
+				presence_state_notify_callbacks(hint, &hint_app, &presence_state);
 			}
 
 			ast_free(presence_subtype);
@@ -3555,6 +3560,11 @@ static void device_state_cb(void *unused
 	struct ast_hintdevice *device;
 	struct ast_hintdevice *cmpdevice;
 	struct ao2_iterator *dev_iter;
+	struct ao2_iterator auto_iter;
+	struct ast_autohint *autohint;
+	char *virtual_device;
+	char *type;
+	char *device_name;
 
 	if (handle_hint_change_message_type(msg, AST_HINT_UPDATE_DEVICE)) {
 		return;
@@ -3570,7 +3580,7 @@ static void device_state_cb(void *unused
 		return;
 	}
 
-	if (ao2_container_count(hintdevices) == 0) {
+	if (ao2_container_count(hintdevices) == 0 && ao2_container_count(autohints) == 0) {
 		/* There are no hints monitoring devices. */
 		return;
 	}
@@ -3584,25 +3594,61 @@ static void device_state_cb(void *unused
 	strcpy(cmpdevice->hintdevice, dev_state->device);
 
 	ast_mutex_lock(&context_merge_lock);/* Hold off ast_merge_contexts_and_delete */
+
+	/* Initially we find all hints for the device and notify them */
 	dev_iter = ao2_t_callback(hintdevices,
 		OBJ_SEARCH_OBJECT | OBJ_MULTIPLE,
 		hintdevice_cmp_multiple,
 		cmpdevice,
 		"find devices in container");
-	if (!dev_iter) {
-		ast_mutex_unlock(&context_merge_lock);
-		ast_free(hint_app);
-		return;
+	if (dev_iter) {
+		for (; (device = ao2_iterator_next(dev_iter)); ao2_t_ref(device, -1, "Next device")) {
+			if (device->hint) {
+				device_state_notify_callbacks(device->hint, &hint_app);
+			}
+		}
+		ao2_iterator_destroy(dev_iter);
 	}
 
-	for (; (device = ao2_iterator_next(dev_iter)); ao2_t_ref(device, -1, "Next device")) {
-		if (device->hint) {
-			device_state_notify_callbacks(device->hint, &hint_app);
+	/* Second stage we look for any autohint contexts and if the device is not already in the hints
+	 * we create it.
+	 */
+	type = ast_strdupa(dev_state->device);
+	if (ast_strlen_zero(type)) {
+		goto end;
+	}
+
+	/* Determine if this is a virtual/custom device or a real device */
+	virtual_device = strchr(type, ':');
+	device_name = strchr(type, '/');
+	if (virtual_device && (!device_name || (virtual_device < device_name))) {
+		device_name = virtual_device;
+	}
+
+	/* Invalid device state name - not a virtual/custom device and not a real device */
+	if (ast_strlen_zero(device_name)) {
+		goto end;
+	}
+
+	*device_name++ = '\0';
+
+	auto_iter = ao2_iterator_init(autohints, 0);
+	for (; (autohint = ao2_iterator_next(&auto_iter)); ao2_t_ref(autohint, -1, "Next autohint")) {
+		if (ast_get_hint(NULL, 0, NULL, 0, NULL, autohint->context, device_name)) {
+			continue;
 		}
+
+		/* The device has no hint in the context referenced by this autohint so create one */
+		ast_add_extension(autohint->context, 0, device_name,
+			PRIORITY_HINT, NULL, NULL, dev_state->device,
+			ast_strdup(dev_state->device), ast_free_ptr, autohint->registrar);
+
+		/* Since this hint was just created there are no watchers, so we don't need to notify anyone */
 	}
-	ast_mutex_unlock(&context_merge_lock);
+	ao2_iterator_destroy(&auto_iter);
 
-	ao2_iterator_destroy(dev_iter);
+end:
+	ast_mutex_unlock(&context_merge_lock);
 	ast_free(hint_app);
 	return;
 }
@@ -3907,7 +3953,7 @@ static int ast_add_hint(struct ast_exten
 	AST_VECTOR_INIT(&hint_new->devices, 8);
 
 	/* Initialize new hint. */
-	hint_new->callbacks = ao2_container_alloc(1, NULL, hint_id_cmp);
+	hint_new->callbacks = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, hint_id_cmp);
 	if (!hint_new->callbacks) {
 		ao2_ref(hint_new, -1);
 		return -1;
@@ -4220,7 +4266,7 @@ static enum ast_pbx_result __ast_pbx_run
 	int autoloopflag;
 	int error = 0;		/* set an error conditions */
 	struct ast_pbx *pbx;
-	struct ast_callid *callid;
+	ast_callid callid;
 
 	/* A little initial setup here */
 	if (ast_channel_pbx(c)) {
@@ -4248,10 +4294,7 @@ static enum ast_pbx_result __ast_pbx_run
 			}
 		}
 		ast_callid_threadassoc_add(callid);
-		callid = ast_callid_unref(callid);
-	} else {
-		/* Nothing to do here, The thread is already bound to a callid.  Let's just get rid of the reference. */
-		ast_callid_unref(callid);
+		callid = 0;
 	}
 
 	ast_channel_pbx_set(c, pbx);
@@ -4737,13 +4780,6 @@ static struct ast_context *find_context_
 	return c;
 }
 
-/*! \brief Free an ast_include and associated data. */
-static void include_free(struct ast_include *include)
-{
-	ast_destroy_timing(&(include->timing));
-	ast_free(include);
-}
-
 /*!
  * \brief Remove included contexts.
  * This function locks contexts list by &conlist, search for the right context
@@ -4775,21 +4811,22 @@ int ast_context_remove_include(const cha
  */
 int ast_context_remove_include2(struct ast_context *con, const char *include, const char *registrar)
 {
-	struct ast_include *i, *pi = NULL;
 	int ret = -1;
+	int idx;
 
 	ast_wrlock_context(con);
 
 	/* find our include */
-	for (i = con->includes; i; pi = i, i = i->next) {
-		if (!strcmp(i->name, include) &&
-				(!registrar || !strcmp(i->registrar, registrar))) {
+	for (idx = 0; idx < ast_context_includes_count(con); idx++) {
+		struct ast_include *i = AST_VECTOR_GET(&con->includes, idx);
+
+		if (!strcmp(ast_get_include_name(i), include) &&
+				(!registrar || !strcmp(ast_get_include_registrar(i), registrar))) {
+
 			/* remove from list */
 			ast_verb(3, "Removing inclusion of context '%s' in context '%s; registrar=%s'\n", include, ast_get_context_name(con), registrar);
-			if (pi)
-				pi->next = i->next;
-			else
-				con->includes = i->next;
+			AST_VECTOR_REMOVE_ORDERED(&con->includes, idx);
+
 			/* free include and return */
 			include_free(i);
 			ret = 0;
@@ -4831,24 +4868,29 @@ int ast_context_remove_switch(const char
  */
 int ast_context_remove_switch2(struct ast_context *con, const char *sw, const char *data, const char *registrar)
 {
-	struct ast_sw *i;
+	int idx;
 	int ret = -1;
 
 	ast_wrlock_context(con);
 
 	/* walk switches */
-	AST_LIST_TRAVERSE_SAFE_BEGIN(&con->alts, i, list) {
-		if (!strcmp(i->name, sw) && !strcmp(i->data, data) &&
-			(!registrar || !strcmp(i->registrar, registrar))) {
+	for (idx = 0; idx < ast_context_switches_count(con); idx++) {
+		struct ast_sw *i = AST_VECTOR_GET(&con->alts, idx);
+
+		if (!strcmp(ast_get_switch_name(i), sw) &&
+			!strcmp(ast_get_switch_data(i), data) &&
+			(!registrar || !strcmp(ast_get_switch_registrar(i), registrar))) {
+
 			/* found, remove from list */
 			ast_verb(3, "Removing switch '%s' from context '%s; registrar=%s'\n", sw, ast_get_context_name(con), registrar);
-			AST_LIST_REMOVE_CURRENT(list);
-			ast_free(i); /* free switch and return */
+			AST_VECTOR_REMOVE_ORDERED(&con->alts, idx);
+
+			/* free switch and return */
+			sw_free(i);
 			ret = 0;
 			break;
 		}
 	}
-	AST_LIST_TRAVERSE_SAFE_END;
 
 	ast_unlock_context(con);
 
@@ -5404,8 +5446,23 @@ static void print_ext(struct ast_exten *
 	}
 }
 
+/*! \brief Writes CLI output of a single extension for show dialplan */
+static void show_dialplan_helper_extension_output(int fd, char *buf1, char *buf2, struct ast_exten *exten)
+{
+	if (ast_get_extension_registrar_file(exten)) {
+		ast_cli(fd, "  %-17s %-45s [%s:%d]\n",
+			buf1, buf2,
+			ast_get_extension_registrar_file(exten),
+			ast_get_extension_registrar_line(exten));
+		return;
+	}
+
+	ast_cli(fd, "  %-17s %-45s [%s]\n",
+		buf1, buf2, ast_get_extension_registrar(exten));
+}
+
 /* XXX not verified */
-static int show_dialplan_helper(int fd, const char *context, const char *exten, struct dialplan_counters *dpc, struct ast_include *rinclude, int includecount, const char *includes[])
+static int show_dialplan_helper(int fd, const char *context, const char *exten, struct dialplan_counters *dpc, const struct ast_include *rinclude, int includecount, const char *includes[])
 {
 	struct ast_context *c = NULL;
 	int res = 0, old_total_exten = dpc->total_exten;
@@ -5414,9 +5471,8 @@ static int show_dialplan_helper(int fd,
 
 	/* walk all contexts ... */
 	while ( (c = ast_walk_contexts(c)) ) {
+		int idx;
 		struct ast_exten *e;
-		struct ast_include *i;
-		struct ast_ignorepat *ip;
 #ifndef LOW_MEMORY
 		char buf[1024], buf2[1024];
 #else
@@ -5441,6 +5497,9 @@ static int show_dialplan_helper(int fd,
 			dpc->total_context++;
 			ast_cli(fd, "[ Context '%s' created by '%s' ]\n",
 				ast_get_context_name(c), ast_get_context_registrar(c));
+			if (c->autohints) {
+				ast_cli(fd, "Autohints support enabled\n");
+			}
 			context_info_printed = 1;
 		}
 
@@ -5463,6 +5522,9 @@ static int show_dialplan_helper(int fd,
 				} else {
 					ast_cli(fd, "[ Context '%s' created by '%s' ]\n",
 						ast_get_context_name(c), ast_get_context_registrar(c));
+					if (c->autohints) {
+						ast_cli(fd, "Autohints support enabled\n");
+					}
 				}
 				context_info_printed = 1;
 			}
@@ -5476,8 +5538,7 @@ static int show_dialplan_helper(int fd,
 
 			print_ext(e, buf2, sizeof(buf2));
 
-			ast_cli(fd, "  %-17s %-45s [%s]\n", buf, buf2,
-				ast_get_extension_registrar(e));
+			show_dialplan_helper_extension_output(fd, buf, buf2, e);
 
 			dpc->total_exten++;
 			/* walk next extension peers */
@@ -5491,14 +5552,14 @@ static int show_dialplan_helper(int fd,
 					buf[0] = '\0';
 				print_ext(p, buf2, sizeof(buf2));
 
-				ast_cli(fd,"  %-17s %-45s [%s]\n", buf, buf2,
-					ast_get_extension_registrar(p));
+				show_dialplan_helper_extension_output(fd, buf, buf2, p);
 			}
 		}
 
 		/* walk included and write info ... */
-		i = NULL;
-		while ( (i = ast_walk_context_includes(c, i)) ) {
+		for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+			const struct ast_include *i = ast_context_includes_get(c, idx);
+
 			snprintf(buf, sizeof(buf), "'%s'", ast_get_include_name(i));
 			if (exten) {
 				/* Check all includes for the requested extension */
@@ -5527,10 +5588,11 @@ static int show_dialplan_helper(int fd,
 		}
 
 		/* walk ignore patterns and write info ... */
-		ip = NULL;
-		while ( (ip = ast_walk_context_ignorepats(c, ip)) ) {
+		for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+			const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
 			const char *ipname = ast_get_ignorepat_name(ip);
 			char ignorepat[AST_MAX_EXTENSION];
+
 			snprintf(buf, sizeof(buf), "'%s'", ipname);
 			snprintf(ignorepat, sizeof(ignorepat), "_%s.", ipname);
 			if (!exten || ast_extension_match(ignorepat, exten)) {
@@ -5539,8 +5601,9 @@ static int show_dialplan_helper(int fd,
 			}
 		}
 		if (!rinclude) {
-			struct ast_sw *sw = NULL;
-			while ( (sw = ast_walk_context_switches(c, sw)) ) {
+			for (idx = 0; idx < ast_context_switches_count(c); idx++) {
+				const struct ast_sw *sw = ast_context_switches_get(c, idx);
+
 				snprintf(buf, sizeof(buf), "'%s/%s'",
 					ast_get_switch_name(sw),
 					ast_get_switch_data(sw));
@@ -5750,7 +5813,7 @@ static void manager_dpsendack(struct man
 static int manager_show_dialplan_helper(struct mansession *s, const struct message *m,
 					const char *actionidtext, const char *context,
 					const char *exten, struct dialplan_counters *dpc,
-					struct ast_include *rinclude,
+					const struct ast_include *rinclude,
 					int includecount, const char *includes[])
 {
 	struct ast_context *c;
@@ -5772,9 +5835,8 @@ static int manager_show_dialplan_helper(
 
 	c = NULL;		/* walk all contexts ... */
 	while ( (c = ast_walk_contexts(c)) ) {
+		int idx;
 		struct ast_exten *e;
-		struct ast_include *i;
-		struct ast_ignorepat *ip;
 
 		if (context && strcmp(ast_get_context_name(c), context) != 0)
 			continue;	/* not the name we want */
@@ -5829,8 +5891,9 @@ static int manager_show_dialplan_helper(
 			}
 		}
 
-		i = NULL;		/* walk included and write info ... */
-		while ( (i = ast_walk_context_includes(c, i)) ) {
+		for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+			const struct ast_include *i = ast_context_includes_get(c, idx);
+
 			if (exten) {
 				/* Check all includes for the requested extension */
 				if (includecount >= AST_PBX_MAX_STACK) {
@@ -5861,8 +5924,8 @@ static int manager_show_dialplan_helper(
 			}
 		}
 
-		ip = NULL;	/* walk ignore patterns and write info ... */
-		while ( (ip = ast_walk_context_ignorepats(c, ip)) ) {
+		for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+			const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
 			const char *ipname = ast_get_ignorepat_name(ip);
 			char ignorepat[AST_MAX_EXTENSION];
 
@@ -5876,8 +5939,9 @@ static int manager_show_dialplan_helper(
 			}
 		}
 		if (!rinclude) {
-			struct ast_sw *sw = NULL;
-			while ( (sw = ast_walk_context_switches(c, sw)) ) {
+			for (idx = 0; idx < ast_context_switches_count(c); idx++) {
+				const struct ast_sw *sw = ast_context_switches_get(c, idx);
+
 				if (!dpc->total_items++)
 					manager_dpsendack(s, m);
 				astman_append(s, "Event: ListDialplan\r\n%s", actionidtext);
@@ -6126,8 +6190,9 @@ struct ast_context *ast_context_find_or_
 		tmp->root = NULL;
 		tmp->root_table = NULL;
 		tmp->registrar = ast_strdup(registrar);
-		tmp->includes = NULL;
-		tmp->ignorepats = NULL;
+		AST_VECTOR_INIT(&tmp->includes, 0);
+		AST_VECTOR_INIT(&tmp->ignorepats, 0);
+		AST_VECTOR_INIT(&tmp->alts, 0);
 		tmp->refcount = 1;
 	} else {
 		ast_log(LOG_ERROR, "Danger! We failed to allocate a context for %s!\n", name);
@@ -6153,6 +6218,11 @@ struct ast_context *ast_context_find_or_
 	return tmp;
 }
 
+void ast_context_set_autohints(struct ast_context *con, int enabled)
+{
+	con->autohints = enabled;
+}
+
 void __ast_context_destroy(struct ast_context *list, struct ast_hashtab *contexttab, struct ast_context *con, const char *registrar);
 
 struct store_hint {
@@ -6172,34 +6242,76 @@ AST_LIST_HEAD_NOLOCK(store_hints, store_
 
 static void context_merge_incls_swits_igps_other_registrars(struct ast_context *new, struct ast_context *old, const char *registrar)
 {
-	struct ast_include *i;
-	struct ast_ignorepat *ip;
-	struct ast_sw *sw;
+	int idx;
 
 	ast_debug(1, "merging incls/swits/igpats from old(%s) to new(%s) context, registrar = %s\n", ast_get_context_name(old), ast_get_context_name(new), registrar);
 	/* copy in the includes, switches, and ignorepats */
 	/* walk through includes */
-	for (i = NULL; (i = ast_walk_context_includes(old, i)) ; ) {
-		if (strcmp(ast_get_include_registrar(i), registrar) == 0)
+	for (idx = 0; idx < ast_context_includes_count(old); idx++) {
+		const struct ast_include *i = ast_context_includes_get(old, idx);
+
+		if (!strcmp(ast_get_include_registrar(i), registrar)) {
 			continue; /* not mine */
+		}
 		ast_context_add_include2(new, ast_get_include_name(i), ast_get_include_registrar(i));
 	}
 
 	/* walk through switches */
-	for (sw = NULL; (sw = ast_walk_context_switches(old, sw)) ; ) {
-		if (strcmp(ast_get_switch_registrar(sw), registrar) == 0)
+	for (idx = 0; idx < ast_context_switches_count(old); idx++) {
+		const struct ast_sw *sw = ast_context_switches_get(old, idx);
+
+		if (!strcmp(ast_get_switch_registrar(sw), registrar)) {
 			continue; /* not mine */
+		}
 		ast_context_add_switch2(new, ast_get_switch_name(sw), ast_get_switch_data(sw), ast_get_switch_eval(sw), ast_get_switch_registrar(sw));
 	}
 
 	/* walk thru ignorepats ... */
-	for (ip = NULL; (ip = ast_walk_context_ignorepats(old, ip)); ) {
-		if (strcmp(ast_get_ignorepat_registrar(ip), registrar) == 0)
+	for (idx = 0; idx < ast_context_ignorepats_count(old); idx++) {
+		const struct ast_ignorepat *ip = ast_context_ignorepats_get(old, idx);
+
+		if (strcmp(ast_get_ignorepat_registrar(ip), registrar) == 0) {
 			continue; /* not mine */
+		}
 		ast_context_add_ignorepat2(new, ast_get_ignorepat_name(ip), ast_get_ignorepat_registrar(ip));
 	}
 }
 
+/*! Set up an autohint placeholder in the hints container */
+static void context_table_create_autohints(struct ast_hashtab *table)
+{
+	struct ast_context *con;
+	struct ast_hashtab_iter *iter;
+
+	/* Remove all autohints as the below iteration will recreate them */
+	ao2_callback(autohints, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
+
+	iter = ast_hashtab_start_traversal(table);
+	while ((con = ast_hashtab_next(iter))) {
+		size_t name_len = strlen(con->name) + 1;
+		size_t registrar_len = strlen(con->registrar) + 1;
+		struct ast_autohint *autohint;
+
+		if (!con->autohints) {
+			continue;
+		}
+
+		autohint = ao2_alloc_options(sizeof(*autohint) + name_len + registrar_len, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+		if (!autohint) {
+			continue;
+		}
+
+		ast_copy_string(autohint->context, con->name, name_len);
+		autohint->registrar = autohint->context + name_len;
+		ast_copy_string(autohint->registrar, con->registrar, registrar_len);
+
+		ao2_link(autohints, autohint);
+		ao2_ref(autohint, -1);
+
+		ast_verb(3, "Enabled autohints support on context '%s'\n", con->name);
+	}
+	ast_hashtab_end_traversal(iter);
+}
 
 /* the purpose of this routine is to duplicate a context, with all its substructure,
    except for any extens that have a matching registrar */
@@ -6241,6 +6353,9 @@ static void context_merge(struct ast_con
 				/* make sure the new context exists, so we have somewhere to stick this exten/prio */
 				if (!new) {
 					new = ast_context_find_or_create(extcontexts, exttable, context->name, prio_item->registrar); /* a new context created via priority from a different context in the old dialplan, gets its registrar from the prio's registrar */
+					if (new) {
+						new->autohints = context->autohints;
+					}
 				}
 
 				/* copy in the includes, switches, and ignorepats */
@@ -6261,7 +6376,8 @@ static void context_merge(struct ast_con
 				dupdstr = ast_strdup(prio_item->data);
 
 				res1 = ast_add_extension2(new, 0, prio_item->name, prio_item->priority, prio_item->label,
-										  prio_item->matchcid ? prio_item->cidmatch : NULL, prio_item->app, dupdstr, ast_free_ptr, prio_item->registrar);
+										  prio_item->matchcid ? prio_item->cidmatch : NULL, prio_item->app, dupdstr, ast_free_ptr, prio_item->registrar,
+										  prio_item->registrar_file, prio_item->registrar_line);
 				if (!res1 && new_exten_item && new_prio_item){
 					ast_verb(3,"Dropping old dialplan item %s/%s/%d [%s(%s)] (registrar=%s) due to conflict with new dialplan\n",
 							context->name, prio_item->name, prio_item->priority, prio_item->app, (char*)prio_item->data, prio_item->registrar);
@@ -6287,6 +6403,10 @@ static void context_merge(struct ast_con
 		   but that's not available, so we give it the registrar we know about */
 		new = ast_context_find_or_create(extcontexts, exttable, context->name, context->registrar);
 
+		if (new) {
+			new->autohints = context->autohints;
+		}
+
 		/* copy in the includes, switches, and ignorepats */
 		context_merge_incls_swits_igps_other_registrars(new, context, registrar);
 	}
@@ -6332,6 +6452,9 @@ void ast_merge_contexts_and_delete(struc
 	ast_wrlock_contexts();
 
 	if (!contexts_table) {
+		/* Create any autohint contexts */
+		context_table_create_autohints(exttable);
+
 		/* Well, that's odd. There are no contexts. */
 		contexts_table = exttable;
 		contexts = *extcontexts;
@@ -6455,6 +6578,9 @@ void ast_merge_contexts_and_delete(struc
 		}
 	}
 
+	/* Create all applicable autohint contexts */
+	context_table_create_autohints(contexts_table);
+
 	ao2_unlock(hints);
 	ast_unlock_contexts();
 
@@ -6547,53 +6673,36 @@ int ast_context_add_include2(struct ast_
 	const char *registrar)
 {
 	struct ast_include *new_include;
-	char *c;
-	struct ast_include *i, *il = NULL; /* include, include_last */
-	int length;
-	char *p;
-
-	length = sizeof(struct ast_include);
-	length += 2 * (strlen(value) + 1);
+	int idx;
 
 	/* allocate new include structure ... */
-	if (!(new_include = ast_calloc(1, length)))
+	new_include = include_alloc(value, registrar);
+	if (!new_include) {
 		return -1;
-	/* Fill in this structure. Use 'p' for assignments, as the fields
-	 * in the structure are 'const char *'
-	 */
-	p = new_include->stuff;
-	new_include->name = p;
-	strcpy(p, value);
-	p += strlen(value) + 1;
-	new_include->rname = p;
-	strcpy(p, value);
-	/* Strip off timing info, and process if it is there */
-	if ( (c = strchr(p, ',')) ) {
-		*c++ = '\0';
-		new_include->hastime = ast_build_timing(&(new_include->timing), c);
 	}
-	new_include->next      = NULL;
-	new_include->registrar = registrar;
 
 	ast_wrlock_context(con);
 
 	/* ... go to last include and check if context is already included too... */
-	for (i = con->includes; i; i = i->next) {
-		if (!strcasecmp(i->name, new_include->name)) {
+	for (idx = 0; idx < ast_context_includes_count(con); idx++) {
+		const struct ast_include *i = ast_context_includes_get(con, idx);
+
+		if (!strcasecmp(ast_get_include_name(i), ast_get_include_name(new_include))) {
 			include_free(new_include);
 			ast_unlock_context(con);
 			errno = EEXIST;
 			return -1;
 		}
-		il = i;
 	}
 
 	/* ... include new context into context list, unlock, return */
-	if (il)
-		il->next = new_include;
-	else
-		con->includes = new_include;
-	ast_debug(1, "Including context '%s' in context '%s'\n", new_include->name, ast_get_context_name(con));
+	if (AST_VECTOR_APPEND(&con->includes, new_include)) {
+		include_free(new_include);
+		ast_unlock_context(con);
+		return -1;
+	}
+	ast_debug(1, "Including context '%s' in context '%s'\n",
+		ast_get_include_name(new_include), ast_get_context_name(con));
 
 	ast_unlock_context(con);
 
@@ -6628,43 +6737,24 @@ int ast_context_add_switch(const char *c
 int ast_context_add_switch2(struct ast_context *con, const char *value,
 	const char *data, int eval, const char *registrar)
 {
+	int idx;
 	struct ast_sw *new_sw;
-	struct ast_sw *i;
-	int length;
-	char *p;
-
-	length = sizeof(struct ast_sw);
-	length += strlen(value) + 1;
-	if (data)
-		length += strlen(data);
-	length++;
 
 	/* allocate new sw structure ... */
-	if (!(new_sw = ast_calloc(1, length)))
+	if (!(new_sw = sw_alloc(value, data, eval, registrar))) {
 		return -1;
-	/* ... fill in this structure ... */
-	p = new_sw->stuff;
-	new_sw->name = p;
-	strcpy(new_sw->name, value);
-	p += strlen(value) + 1;
-	new_sw->data = p;
-	if (data) {
-		strcpy(new_sw->data, data);
-		p += strlen(data) + 1;
-	} else {
-		strcpy(new_sw->data, "");
-		p++;
 	}
-	new_sw->eval	  = eval;
-	new_sw->registrar = registrar;
 
 	/* ... try to lock this context ... */
 	ast_wrlock_context(con);
 
 	/* ... go to last sw and check if context is already swd too... */
-	AST_LIST_TRAVERSE(&con->alts, i, list) {
-		if (!strcasecmp(i->name, new_sw->name) && !strcasecmp(i->data, new_sw->data)) {
-			ast_free(new_sw);
+	for (idx = 0; idx < ast_context_switches_count(con); idx++) {
+		const struct ast_sw *i = ast_context_switches_get(con, idx);
+
+		if (!strcasecmp(ast_get_switch_name(i), ast_get_switch_name(new_sw)) &&
+			!strcasecmp(ast_get_switch_data(i), ast_get_switch_data(new_sw))) {
+			sw_free(new_sw);
 			ast_unlock_context(con);
 			errno = EEXIST;
 			return -1;
@@ -6672,9 +6762,14 @@ int ast_context_add_switch2(struct ast_c
 	}
 
 	/* ... sw new context into context list, unlock, return */
-	AST_LIST_INSERT_TAIL(&con->alts, new_sw, list);
+	if (AST_VECTOR_APPEND(&con->alts, new_sw)) {
+		sw_free(new_sw);
+		ast_unlock_context(con);
+		return -1;
+	}
 
-	ast_verb(3, "Including switch '%s/%s' in context '%s'\n", new_sw->name, new_sw->data, ast_get_context_name(con));
+	ast_verb(3, "Including switch '%s/%s' in context '%s'\n",
+		ast_get_switch_name(new_sw), ast_get_switch_data(new_sw), ast_get_context_name(con));
 
 	ast_unlock_context(con);
 
@@ -6700,24 +6795,20 @@ int ast_context_remove_ignorepat(const c
 
 int ast_context_remove_ignorepat2(struct ast_context *con, const char *ignorepat, const char *registrar)
 {
-	struct ast_ignorepat *ip, *ipl = NULL;
+	int idx;
 
 	ast_wrlock_context(con);
 
-	for (ip = con->ignorepats; ip; ip = ip->next) {
-		if (!strcmp(ip->pattern, ignorepat) &&
-			(!registrar || (registrar == ip->registrar))) {
-			if (ipl) {
-				ipl->next = ip->next;
-				ast_free(ip);
-			} else {
-				con->ignorepats = ip->next;
-				ast_free(ip);
-			}
+	for (idx = 0; idx < ast_context_ignorepats_count(con); idx++) {
+		struct ast_ignorepat *ip = AST_VECTOR_GET(&con->ignorepats, idx);
+
+		if (!strcmp(ast_get_ignorepat_name(ip), ignorepat) &&
+			(!registrar || (registrar == ast_get_ignorepat_registrar(ip)))) {
+			AST_VECTOR_REMOVE_ORDERED(&con->ignorepats, idx);
+			ignorepat_free(ip);
 			ast_unlock_context(con);
 			return 0;
 		}
-		ipl = ip;
 	}
 
 	ast_unlock_context(con);
@@ -6744,41 +6835,33 @@ int ast_context_add_ignorepat(const char
 
 int ast_context_add_ignorepat2(struct ast_context *con, const char *value, const char *registrar)
 {
-	struct ast_ignorepat *ignorepat, *ignorepatc, *ignorepatl = NULL;
-	int length;
-	char *pattern;
-	length = sizeof(struct ast_ignorepat);
-	length += strlen(value) + 1;
-	if (!(ignorepat = ast_calloc(1, length)))
-		return -1;
-	/* The cast to char * is because we need to write the initial value.
-	 * The field is not supposed to be modified otherwise.  Also, gcc 4.2
-	 * sees the cast as dereferencing a type-punned pointer and warns about
-	 * it.  This is the workaround (we're telling gcc, yes, that's really
-	 * what we wanted to do).
-	 */
-	pattern = (char *) ignorepat->pattern;
-	strcpy(pattern, value);
-	ignorepat->next = NULL;
-	ignorepat->registrar = registrar;
+	struct ast_ignorepat *ignorepat = ignorepat_alloc(value, registrar);
+	int idx;
+
+	if (!ignorepat) {
+		return -1;
+	}
+
 	ast_wrlock_context(con);
-	for (ignorepatc = con->ignorepats; ignorepatc; ignorepatc = ignorepatc->next) {
-		ignorepatl = ignorepatc;
-		if (!strcasecmp(ignorepatc->pattern, value)) {
+	for (idx = 0; idx < ast_context_ignorepats_count(con); idx++) {
+		const struct ast_ignorepat *i = ast_context_ignorepats_get(con, idx);
+
+		if (!strcasecmp(ast_get_ignorepat_name(i), value)) {
 			/* Already there */
 			ast_unlock_context(con);
-			ast_free(ignorepat);
+			ignorepat_free(ignorepat);
 			errno = EEXIST;
 			return -1;
 		}
 	}
-	if (ignorepatl)
-		ignorepatl->next = ignorepat;
-	else
-		con->ignorepats = ignorepat;
+	if (AST_VECTOR_APPEND(&con->ignorepats, ignorepat)) {
+		ignorepat_free(ignorepat);
+		ast_unlock_context(con);
+		return -1;
+	}
 	ast_unlock_context(con);
-	return 0;
 
+	return 0;
 }
 
 int ast_ignore_pattern(const char *context, const char *pattern)
@@ -6789,10 +6872,12 @@ int ast_ignore_pattern(const char *conte
 	ast_rdlock_contexts();
 	con = ast_context_find(context);
 	if (con) {
-		struct ast_ignorepat *pat;
+		int idx;
+
+		for (idx = 0; idx < ast_context_ignorepats_count(con); idx++) {
+			const struct ast_ignorepat *pat = ast_context_ignorepats_get(con, idx);
 
-		for (pat = con->ignorepats; pat; pat = pat->next) {
-			if (ast_extension_match(pat->pattern, pattern)) {
+			if (ast_extension_match(ast_get_ignorepat_name(pat), pattern)) {
 				ret = 1;
 				break;
 			}
@@ -6818,7 +6903,7 @@ static int ast_add_extension_nolock(cons
 	c = find_context(context);
 	if (c) {
 		ret = ast_add_extension2_lockopt(c, replace, extension, priority, label, callerid,
-			application, data, datad, registrar, 1);
+			application, data, datad, registrar, NULL, 0, 1);
 	}
 
 	return ret;
@@ -6838,7 +6923,7 @@ int ast_add_extension(const char *contex
 	c = find_context_locked(context);
 	if (c) {
 		ret = ast_add_extension2(c, replace, extension, priority, label, callerid,
-			application, data, datad, registrar);
+			application, data, datad, registrar, NULL, 0);
 		ast_unlock_contexts();
 	}
 
@@ -7160,19 +7245,19 @@ static int add_priority(struct ast_conte
 int ast_add_extension2(struct ast_context *con,
 	int replace, const char *extension, int priority, const char *label, const char *callerid,
 	const char *application, void *data, void (*datad)(void *),
-	const char *registrar)
+	const char *registrar, const char *registrar_file, int registrar_line)
 {
 	return ast_add_extension2_lockopt(con, replace, extension, priority, label, callerid,
-		application, data, datad, registrar, 1);
+		application, data, datad, registrar, registrar_file, registrar_line, 1);
 }
 
 int ast_add_extension2_nolock(struct ast_context *con,
 	int replace, const char *extension, int priority, const char *label, const char *callerid,
 	const char *application, void *data, void (*datad)(void *),
-	const char *registrar)
+	const char *registrar, const char *registrar_file, int registrar_line)
 {
 	return ast_add_extension2_lockopt(con, replace, extension, priority, label, callerid,
-		application, data, datad, registrar, 0);
+		application, data, datad, registrar, registrar_file, registrar_line, 0);
 }
 
 
@@ -7186,7 +7271,7 @@ int ast_add_extension2_nolock(struct ast
 static int ast_add_extension2_lockopt(struct ast_context *con,
 	int replace, const char *extension, int priority, const char *label, const char *callerid,
 	const char *application, void *data, void (*datad)(void *),
-	const char *registrar, int lock_context)
+	const char *registrar, const char *registrar_file, int registrar_line, int lock_context)
 {
 	/*
 	 * Sort extensions (or patterns) according to the rules indicated above.
@@ -7257,6 +7342,9 @@ static int ast_add_extension2_lockopt(st
 	} else {
 		length ++;	/* just the '\0' */
 	}
+	if (registrar_file) {
+		length += strlen(registrar_file) + 1;
+	}
 
 	/* Be optimistic:  Build the extension structure first */
 	if (!(tmp = ast_calloc(1, length)))
@@ -7296,12 +7384,22 @@ static int ast_add_extension2_lockopt(st
 		*p++ = '\0';
 		tmp->matchcid = AST_EXT_MATCHCID_OFF;
 	}
+
+	if (registrar_file) {
+		tmp->registrar_file = p;
+		strcpy(p, registrar_file);
+		p += strlen(registrar_file) + 1;
+	} else {
+		tmp->registrar_file = NULL;
+	}
+
 	tmp->app = p;
 	strcpy(p, application);
 	tmp->parent = con;
 	tmp->data = data;
 	tmp->datad = datad;
 	tmp->registrar = registrar;
+	tmp->registrar_line = registrar_line;
 
 	if (lock_context) {
 		ast_wrlock_context(con);
@@ -7371,52 +7469,38 @@ static int ast_add_extension2_lockopt(st
 		 * so insert in the main list right before 'e' (if any)
 		 */
 		tmp->next = e;
-		if (el) {  /* there is another exten already in this context */
-			el->next = tmp;
-			tmp->peer_table = ast_hashtab_create(13,
-							hashtab_compare_exten_numbers,
+		tmp->peer_table = ast_hashtab_create(13,
+						hashtab_compare_exten_numbers,
+						ast_hashtab_resize_java,
+						ast_hashtab_newsize_java,
+						hashtab_hash_priority,
+						0);
+		tmp->peer_label_table = ast_hashtab_create(7,
+							hashtab_compare_exten_labels,
 							ast_hashtab_resize_java,
 							ast_hashtab_newsize_java,
-							hashtab_hash_priority,
+							hashtab_hash_labels,
 							0);
-			tmp->peer_label_table = ast_hashtab_create(7,
-								hashtab_compare_exten_labels,
-								ast_hashtab_resize_java,
-								ast_hashtab_newsize_java,
-								hashtab_hash_labels,
-								0);
-			if (label) {
-				ast_hashtab_insert_safe(tmp->peer_label_table, tmp);
-			}
-			ast_hashtab_insert_safe(tmp->peer_table, tmp);
+
+		if (el) {  /* there is another exten already in this context */
+			el->next = tmp;
 		} else {  /* this is the first exten in this context */
-			if (!con->root_table)
+			if (!con->root_table) {
 				con->root_table = ast_hashtab_create(27,
 													hashtab_compare_extens,
 													ast_hashtab_resize_java,
 													ast_hashtab_newsize_java,
 													hashtab_hash_extens,
 													0);
-			con->root = tmp;
-			con->root->peer_table = ast_hashtab_create(13,
-								hashtab_compare_exten_numbers,
-								ast_hashtab_resize_java,
-								ast_hashtab_newsize_java,
-								hashtab_hash_priority,
-								0);
-			con->root->peer_label_table = ast_hashtab_create(7,
-									hashtab_compare_exten_labels,
-									ast_hashtab_resize_java,
-									ast_hashtab_newsize_java,
-									hashtab_hash_labels,
-									0);
-			if (label) {
-				ast_hashtab_insert_safe(con->root->peer_label_table, tmp);
 			}
-			ast_hashtab_insert_safe(con->root->peer_table, tmp);
-
+			con->root = tmp;
+		}
+		if (label) {
+			ast_hashtab_insert_safe(tmp->peer_label_table, tmp);
 		}
+		ast_hashtab_insert_safe(tmp->peer_table, tmp);
 		ast_hashtab_insert_safe(con->root_table, tmp);
+
 		if (lock_context) {
 			ast_unlock_context(con);
 		}
@@ -7607,11 +7691,13 @@ static int pbx_outgoing_attempt(const ch
 	const char *app, const char *appdata, int *reason, int synchronous,
 	const char *cid_num, const char *cid_name, struct ast_variable *vars,
 	const char *account, struct ast_channel **locked_channel, int early_media,
-	const struct ast_assigned_ids *assignedids)
+	const struct ast_assigned_ids *assignedids, const char *predial_callee)
 {
 	RAII_VAR(struct pbx_outgoing *, outgoing, NULL, ao2_cleanup);
 	struct ast_channel *dialed;
 	pthread_t thread;
+	char tmp_cid_name[128];
+	char tmp_cid_num[128];
 
 	outgoing = ao2_alloc(sizeof(*outgoing), pbx_outgoing_destroy);
 	if (!outgoing) {
@@ -7638,6 +7724,11 @@ static int pbx_outgoing_attempt(const ch
 
 	ast_dial_set_global_timeout(outgoing->dial, timeout);
 
+	if (!ast_strlen_zero(predial_callee)) {
+		/* note casting to void * here to suppress compiler warning message (passing const to non-const function) */
+		ast_dial_option_global_enable(outgoing->dial, AST_DIAL_OPTION_PREDIAL, (void *)predial_callee);
+	}
+
 	if (ast_dial_prerun(outgoing->dial, NULL, cap)) {
 		if (synchronous && reason) {
 			*reason = pbx_dial_reason(AST_DIAL_RESULT_FAILED,
@@ -7662,6 +7753,25 @@ static int pbx_outgoing_attempt(const ch
 		ast_channel_stage_snapshot_done(dialed);
 	}
 	ast_set_flag(ast_channel_flags(dialed), AST_FLAG_ORIGINATED);
+
+	if (!ast_strlen_zero(predial_callee)) {
+		char *tmp = NULL;
+		/*
+		 * The predial sub routine may have set callerid so set this into the new channel
+		 * Note... cid_num and cid_name parameters to this function will always be NULL if
+		 * predial_callee is non-NULL so we are not overwriting anything here.
+		 */
+		tmp = S_COR(ast_channel_caller(dialed)->id.number.valid, ast_channel_caller(dialed)->id.number.str, NULL);
+		if (tmp) {
+			ast_copy_string(tmp_cid_num, tmp, sizeof(tmp_cid_num));
+			cid_num = tmp_cid_num;
+		}
+		tmp = S_COR(ast_channel_caller(dialed)->id.name.valid, ast_channel_caller(dialed)->id.name.str, NULL);
+		if (tmp) {
+			ast_copy_string(tmp_cid_name, tmp, sizeof(tmp_cid_name));
+			cid_name = tmp_cid_name;
+		}
+	}
 	ast_channel_unlock(dialed);
 
 	if (!ast_strlen_zero(cid_num) || !ast_strlen_zero(cid_name)) {
@@ -7778,6 +7888,16 @@ int ast_pbx_outgoing_exten(const char *t
 	const char *account, struct ast_channel **locked_channel, int early_media,
 	const struct ast_assigned_ids *assignedids)
 {
+	return ast_pbx_outgoing_exten_predial(type, cap, addr, timeout, context, exten, priority, reason,
+		synchronous, cid_num, cid_name, vars, account, locked_channel, early_media, assignedids, NULL);
+}
+
+int ast_pbx_outgoing_exten_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *context, const char *exten, int priority, int *reason,
+	int synchronous, const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel, int early_media,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee)
+{
 	int res;
 	int my_reason;
 
@@ -7791,7 +7911,7 @@ int ast_pbx_outgoing_exten(const char *t
 
 	res = pbx_outgoing_attempt(type, cap, addr, timeout, context, exten, priority,
 		NULL, NULL, reason, synchronous, cid_num, cid_name, vars, account, locked_channel,
-		early_media, assignedids);
+		early_media, assignedids, predial_callee);
 
 	if (res < 0 /* Call failed to get connected for some reason. */
 		&& 0 < synchronous
@@ -7832,6 +7952,16 @@ int ast_pbx_outgoing_app(const char *typ
 	const char *account, struct ast_channel **locked_channel,
 	const struct ast_assigned_ids *assignedids)
 {
+	return ast_pbx_outgoing_app_predial(type, cap, addr, timeout, app, appdata, reason, synchronous,
+		cid_num, cid_name, vars, account, locked_channel, assignedids, NULL);
+}
+
+int ast_pbx_outgoing_app_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *app, const char *appdata, int *reason, int synchronous,
+	const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee)
+{
 	if (reason) {
 		*reason = 0;
 	}
@@ -7844,7 +7974,7 @@ int ast_pbx_outgoing_app(const char *typ
 
 	return pbx_outgoing_attempt(type, cap, addr, timeout, NULL, NULL, 0, app, appdata,
 		reason, synchronous, cid_num, cid_name, vars, account, locked_channel, 0,
-		assignedids);
+		assignedids, predial_callee);
 }
 
 /* this is the guts of destroying a context --
@@ -7853,22 +7983,21 @@ int ast_pbx_outgoing_app(const char *typ
 
 static void __ast_internal_context_destroy( struct ast_context *con)
 {
-	struct ast_include *tmpi;
-	struct ast_sw *sw;
 	struct ast_exten *e, *el, *en;
-	struct ast_ignorepat *ipi;
 	struct ast_context *tmp = con;
 
-	for (tmpi = tmp->includes; tmpi; ) { /* Free includes */
-		struct ast_include *tmpil = tmpi;
-		tmpi = tmpi->next;
-		include_free(tmpil);
-	}
-	for (ipi = tmp->ignorepats; ipi; ) { /* Free ignorepats */
-		struct ast_ignorepat *ipl = ipi;
-		ipi = ipi->next;
-		ast_free(ipl);
-	}
+	/* Free includes */
+	AST_VECTOR_CALLBACK_VOID(&tmp->includes, include_free);
+	AST_VECTOR_FREE(&tmp->includes);
+
+	/* Free ignorepats */
+	AST_VECTOR_CALLBACK_VOID(&tmp->ignorepats, ignorepat_free);
+	AST_VECTOR_FREE(&tmp->ignorepats);
+
+	/* Free switches */
+	AST_VECTOR_CALLBACK_VOID(&tmp->alts, sw_free);
+	AST_VECTOR_FREE(&tmp->alts);
+
 	if (tmp->registrar)
 		ast_free(tmp->registrar);
 
@@ -7880,8 +8009,6 @@ static void __ast_internal_context_destr
 	if (tmp->pattern_tree)
 		destroy_pattern_tree(tmp->pattern_tree);
 
-	while ((sw = AST_LIST_REMOVE_HEAD(&tmp->alts, list)))
-		ast_free(sw);
 	for (e = tmp->root; e;) {
 		for (en = e->peer; en;) {
 			el = en;
@@ -7928,53 +8055,35 @@ void __ast_context_destroy(struct ast_co
 			/* then search thru and remove any extens that match registrar. */
 			struct ast_hashtab_iter *exten_iter;
 			struct ast_hashtab_iter *prio_iter;
-			struct ast_ignorepat *ip, *ipl = NULL, *ipn = NULL;
-			struct ast_include *i, *pi = NULL, *ni = NULL;
-			struct ast_sw *sw = NULL;
+			int idx;
 
 			/* remove any ignorepats whose registrar matches */
-			for (ip = tmp->ignorepats; ip; ip = ipn) {
-				ipn = ip->next;
-				if (!strcmp(ip->registrar, registrar)) {
-					if (ipl) {
-						ipl->next = ip->next;
-						ast_free(ip);
-						continue; /* don't change ipl */
-					} else {
-						tmp->ignorepats = ip->next;
-						ast_free(ip);
-						continue; /* don't change ipl */
-					}
+			for (idx = ast_context_ignorepats_count(tmp) - 1; idx >= 0; idx--) {
+				struct ast_ignorepat *ip = AST_VECTOR_GET(&tmp->ignorepats, idx);
+
+				if (!strcmp(ast_get_ignorepat_registrar(ip), registrar)) {
+					AST_VECTOR_REMOVE_ORDERED(&tmp->ignorepats, idx);
+					ignorepat_free(ip);
 				}
-				ipl = ip;
 			}
 			/* remove any includes whose registrar matches */
-			for (i = tmp->includes; i; i = ni) {
-				ni = i->next;
-				if (strcmp(i->registrar, registrar) == 0) {
-					/* remove from list */
-					if (pi) {
-						pi->next = i->next;
-						/* free include */
-						include_free(i);
-						continue; /* don't change pi */
-					} else {
-						tmp->includes = i->next;
-						/* free include */
-						include_free(i);
-						continue; /* don't change pi */
-					}
+			for (idx = ast_context_includes_count(tmp) - 1; idx >= 0; idx--) {
+				struct ast_include *i = AST_VECTOR_GET(&tmp->includes, idx);
+
+				if (!strcmp(ast_get_include_registrar(i), registrar)) {
+					AST_VECTOR_REMOVE_ORDERED(&tmp->includes, idx);
+					include_free(i);
 				}
-				pi = i;
 			}
 			/* remove any switches whose registrar matches */
-			AST_LIST_TRAVERSE_SAFE_BEGIN(&tmp->alts, sw, list) {
-				if (strcmp(sw->registrar,registrar) == 0) {
-					AST_LIST_REMOVE_CURRENT(list);
-					ast_free(sw);
+			for (idx = ast_context_switches_count(tmp) - 1; idx >= 0; idx--) {
+				struct ast_sw *sw = AST_VECTOR_GET(&tmp->alts, idx);
+
+				if (!strcmp(ast_get_switch_registrar(sw), registrar)) {
+					AST_VECTOR_REMOVE_ORDERED(&tmp->alts, idx);
+					sw_free(sw);
 				}
 			}
-			AST_LIST_TRAVERSE_SAFE_END;
 
 			if (tmp->root_table) { /* it is entirely possible that the context is EMPTY */
 				exten_iter = ast_hashtab_start_traversal(tmp->root_table);
@@ -8025,7 +8134,7 @@ void __ast_context_destroy(struct ast_co
 			/* delete the context if it's registrar matches, is empty, has refcount of 1, */
 			/* it's not empty, if it has includes, ignorepats, or switches that are registered from
 			   another registrar. It's not empty if there are any extensions */
-			if (strcmp(tmp->registrar, registrar) == 0 && tmp->refcount < 2 && !tmp->root && !tmp->ignorepats && !tmp->includes && AST_LIST_EMPTY(&tmp->alts)) {
+			if (strcmp(tmp->registrar, registrar) == 0 && tmp->refcount < 2 && !tmp->root && !ast_context_ignorepats_count(tmp) && !ast_context_includes_count(tmp) && !ast_context_switches_count(tmp)) {
 				ast_debug(1, "delete ctx %s %s\n", tmp->name, tmp->registrar);
 				ast_hashtab_remove_this_object(contexttab, tmp);
 
@@ -8157,15 +8266,20 @@ int pbx_checkcondition(const char *condi
 
 static void presence_state_cb(void *unused, struct stasis_subscription *sub, struct stasis_message *msg)
 {
-	struct ast_hint *hint;
+	struct ast_presence_state_message *presence_state;
 	struct ast_str *hint_app = NULL;
-	struct ao2_iterator hint_iter;
+	struct ast_hintdevice *device;
+	struct ast_hintdevice *cmpdevice;
+	struct ao2_iterator *dev_iter;
 
-	if (handle_hint_change_message_type(msg, AST_HINT_UPDATE_PRESENCE)) {
+	if (stasis_message_type(msg) != ast_presence_state_message_type()) {
 		return;
 	}
 
-	if (stasis_message_type(msg) != ast_presence_state_message_type()) {
+	presence_state = stasis_message_data(msg);
+
+	if (ao2_container_count(hintdevices) == 0) {
+		/* There are no hints monitoring devices. */
 		return;
 	}
 
@@ -8174,67 +8288,32 @@ static void presence_state_cb(void *unus
 		return;
 	}
 
-	ast_mutex_lock(&context_merge_lock);/* Hold off ast_merge_contexts_and_delete */
-	hint_iter = ao2_iterator_init(hints, 0);
-	for (; (hint = ao2_iterator_next(&hint_iter)); ao2_cleanup(hint)) {
-		presence_state_notify_callbacks(msg, hint, &hint_app, stasis_message_data(msg));
-	}
-	ao2_iterator_destroy(&hint_iter);
-	ast_mutex_unlock(&context_merge_lock);
-
-	ast_free(hint_app);
-}
+	cmpdevice = ast_alloca(sizeof(*cmpdevice) + strlen(presence_state->provider));
+	strcpy(cmpdevice->hintdevice, presence_state->provider);
 
-/*!
- * \internal
- * \brief Implements the hints data provider.
- */
-static int hints_data_provider_get(const struct ast_data_search *search,
-	struct ast_data *data_root)
-{
-	struct ast_data *data_hint;
-	struct ast_hint *hint;
-	int watchers;
-	struct ao2_iterator i;
-
-	if (ao2_container_count(hints) == 0) {
-		return 0;
+	ast_mutex_lock(&context_merge_lock);/* Hold off ast_merge_contexts_and_delete */
+	dev_iter = ao2_t_callback(hintdevices,
+		OBJ_POINTER | OBJ_MULTIPLE,
+		hintdevice_cmp_multiple,
+		cmpdevice,
+		"find devices in container");
+	if (!dev_iter) {
+		ast_mutex_unlock(&context_merge_lock);
+		ast_free(hint_app);
+		return;
 	}
 
-	i = ao2_iterator_init(hints, 0);
-	for (; (hint = ao2_iterator_next(&i)); ao2_ref(hint, -1)) {
-		watchers = ao2_container_count(hint->callbacks);
-		data_hint = ast_data_add_node(data_root, "hint");
-		if (!data_hint) {
-			continue;
-		}
-		ast_data_add_str(data_hint, "extension", ast_get_extension_name(hint->exten));
-		ast_data_add_str(data_hint, "context", ast_get_context_name(ast_get_extension_context(hint->exten)));
-		ast_data_add_str(data_hint, "application", ast_get_extension_app(hint->exten));
-		ast_data_add_str(data_hint, "state", ast_extension_state2str(hint->laststate));
-		ast_data_add_str(data_hint, "presence_state", ast_presence_state2str(hint->last_presence_state));
-		ast_data_add_str(data_hint, "presence_subtype", S_OR(hint->last_presence_subtype, ""));
-		ast_data_add_str(data_hint, "presence_subtype", S_OR(hint->last_presence_message, ""));
-		ast_data_add_int(data_hint, "watchers", watchers);
-
-		if (!ast_data_search_match(search, data_hint)) {
-			ast_data_remove_node(data_root, data_hint);
+	for (; (device = ao2_iterator_next(dev_iter)); ao2_t_ref(device, -1, "Next device")) {
+		if (device->hint) {
+			presence_state_notify_callbacks(device->hint, &hint_app, presence_state);
 		}
 	}
-	ao2_iterator_destroy(&i);
+	ao2_iterator_destroy(dev_iter);
+	ast_mutex_unlock(&context_merge_lock);
 
-	return 0;
+	ast_free(hint_app);
 }
 
-static const struct ast_data_handler hints_data_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = hints_data_provider_get
-};
-
-static const struct ast_data_entry pbx_data_providers[] = {
-	AST_DATA_ENTRY("asterisk/core/hints", &hints_data_provider),
-};
-
 static int action_extensionstatelist(struct mansession *s, const struct message *m)
 {
 	const char *action_id = astman_get_header(m, "ActionID");
@@ -8310,7 +8389,6 @@ static void unload_pbx(void)
 	ast_cli_unregister_multiple(pbx_cli, ARRAY_LEN(pbx_cli));
 	ast_custom_function_unregister(&exception_function);
 	ast_custom_function_unregister(&testtime_function);
-	ast_data_unregister(NULL);
 }
 
 int load_pbx(void)
@@ -8324,7 +8402,6 @@ int load_pbx(void)
 
 	ast_verb(2, "Registering builtin functions:\n");
 	ast_cli_register_multiple(pbx_cli, ARRAY_LEN(pbx_cli));
-	ast_data_register_multiple_core(pbx_data_providers, ARRAY_LEN(pbx_data_providers));
 	__ast_custom_function_register(&exception_function, NULL);
 	__ast_custom_function_register(&testtime_function, NULL);
 
@@ -8339,10 +8416,15 @@ int load_pbx(void)
 	if (!(device_state_sub = stasis_subscribe(ast_device_state_topic_all(), device_state_cb, NULL))) {
 		return -1;
 	}
+	stasis_subscription_accept_message_type(device_state_sub, ast_device_state_message_type());
+	stasis_subscription_accept_message_type(device_state_sub, hint_change_message_type());
+	stasis_subscription_set_filter(device_state_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	if (!(presence_state_sub = stasis_subscribe(ast_presence_state_topic_all(), presence_state_cb, NULL))) {
 		return -1;
 	}
+	stasis_subscription_accept_message_type(presence_state_sub, ast_presence_state_message_type());
+	stasis_subscription_set_filter(presence_state_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	return 0;
 }
@@ -8406,16 +8488,6 @@ const char *ast_get_extension_label(stru
 	return exten ? exten->label : NULL;
 }
 
-const char *ast_get_include_name(struct ast_include *inc)
-{
-	return inc ? inc->name : NULL;
-}
-
-const char *ast_get_ignorepat_name(struct ast_ignorepat *ip)
-{
-	return ip ? ip->pattern : NULL;
-}
-
 int ast_get_extension_priority(struct ast_exten *exten)
 {
 	return exten ? exten->priority : -1;
@@ -8434,14 +8506,14 @@ const char *ast_get_extension_registrar(
 	return e ? e->registrar : NULL;
 }
 
-const char *ast_get_include_registrar(struct ast_include *i)
+const char *ast_get_extension_registrar_file(struct ast_exten *e)
 {
-	return i ? i->registrar : NULL;
+	return e ? e->registrar_file : NULL;
 }
 
-const char *ast_get_ignorepat_registrar(struct ast_ignorepat *ip)
+int ast_get_extension_registrar_line(struct ast_exten *e)
 {
-	return ip ? ip->registrar : NULL;
+	return e ? e->registrar_line : 0;
 }
 
 int ast_get_extension_matchcid(struct ast_exten *e)
@@ -8464,26 +8536,6 @@ void *ast_get_extension_app_data(struct
 	return e ? e->data : NULL;
 }
 
-const char *ast_get_switch_name(struct ast_sw *sw)
-{
-	return sw ? sw->name : NULL;
-}
-
-const char *ast_get_switch_data(struct ast_sw *sw)
-{
-	return sw ? sw->data : NULL;
-}
-
-int ast_get_switch_eval(struct ast_sw *sw)
-{
-	return sw->eval;
-}
-
-const char *ast_get_switch_registrar(struct ast_sw *sw)
-{
-	return sw ? sw->registrar : NULL;
-}
-
 /*
  * Walking functions ...
  */
@@ -8501,13 +8553,43 @@ struct ast_exten *ast_walk_context_exten
 		return exten->next;
 }
 
-struct ast_sw *ast_walk_context_switches(struct ast_context *con,
-	struct ast_sw *sw)
+const struct ast_sw *ast_walk_context_switches(const struct ast_context *con,
+	const struct ast_sw *sw)
 {
-	if (!sw)
-		return con ? AST_LIST_FIRST(&con->alts) : NULL;
-	else
-		return AST_LIST_NEXT(sw, list);
+	if (sw) {
+		int idx;
+		int next = 0;
+
+		for (idx = 0; idx < ast_context_switches_count(con); idx++) {
+			const struct ast_sw *s = ast_context_switches_get(con, idx);
+
+			if (next) {
+				return s;
+			}
+
+			if (sw == s) {
+				next = 1;
+			}
+		}
+
+		return NULL;
+	}
+
+	if (!ast_context_switches_count(con)) {
+		return NULL;
+	}
+
+	return ast_context_switches_get(con, 0);
+}
+
+int ast_context_switches_count(const struct ast_context *con)
+{
+	return AST_VECTOR_SIZE(&con->alts);
+}
+
+const struct ast_sw *ast_context_switches_get(const struct ast_context *con, int idx)
+{
+	return AST_VECTOR_GET(&con->alts, idx);
 }
 
 struct ast_exten *ast_walk_extension_priorities(struct ast_exten *exten,
@@ -8516,36 +8598,103 @@ struct ast_exten *ast_walk_extension_pri
 	return priority ? priority->peer : exten;
 }
 
-struct ast_include *ast_walk_context_includes(struct ast_context *con,
-	struct ast_include *inc)
+const struct ast_include *ast_walk_context_includes(const struct ast_context *con,
+	const struct ast_include *inc)
 {
-	if (!inc)
-		return con ? con->includes : NULL;
-	else
-		return inc->next;
+	if (inc) {
+		int idx;
+		int next = 0;
+
+		for (idx = 0; idx < ast_context_includes_count(con); idx++) {
+			const struct ast_include *include = AST_VECTOR_GET(&con->includes, idx);
+
+			if (next) {
+				return include;
+			}
+
+			if (inc == include) {
+				next = 1;
+			}
+		}
+
+		return NULL;
+	}
+
+	if (!ast_context_includes_count(con)) {
+		return NULL;
+	}
+
+	return ast_context_includes_get(con, 0);
 }
 
-struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con,
-	struct ast_ignorepat *ip)
+int ast_context_includes_count(const struct ast_context *con)
 {
-	if (!ip)
-		return con ? con->ignorepats : NULL;
-	else
-		return ip->next;
+	return AST_VECTOR_SIZE(&con->includes);
+}
+
+const struct ast_include *ast_context_includes_get(const struct ast_context *con, int idx)
+{
+	return AST_VECTOR_GET(&con->includes, idx);
+}
+
+const struct ast_ignorepat *ast_walk_context_ignorepats(const struct ast_context *con,
+	const struct ast_ignorepat *ip)
+{
+	if (!con) {
+		return NULL;
+	}
+
+	if (ip) {
+		int idx;
+		int next = 0;
+
+		for (idx = 0; idx < ast_context_ignorepats_count(con); idx++) {
+			const struct ast_ignorepat *i = ast_context_ignorepats_get(con, idx);
+
+			if (next) {
+				return i;
+			}
+
+			if (ip == i) {
+				next = 1;
+			}
+		}
+
+		return NULL;
+	}
+
+	if (!ast_context_ignorepats_count(con)) {
+		return NULL;
+	}
+
+	return ast_context_ignorepats_get(con, 0);
+}
+
+int ast_context_ignorepats_count(const struct ast_context *con)
+{
+	return AST_VECTOR_SIZE(&con->ignorepats);
+}
+
+const struct ast_ignorepat *ast_context_ignorepats_get(const struct ast_context *con, int idx)
+{
+	return AST_VECTOR_GET(&con->ignorepats, idx);
 }
 
 int ast_context_verify_includes(struct ast_context *con)
 {
-	struct ast_include *inc = NULL;
+	int idx;
 	int res = 0;
 
-	while ( (inc = ast_walk_context_includes(con, inc)) ) {
-		if (ast_context_find(inc->rname))
+	for (idx = 0; idx < ast_context_includes_count(con); idx++) {
+		const struct ast_include *inc = ast_context_includes_get(con, idx);
+
+		if (ast_context_find(include_rname(inc))) {
 			continue;
+		}
 
 		res = -1;
 		ast_log(LOG_WARNING, "Context '%s' tries to include nonexistent context '%s'\n",
-			ast_get_context_name(con), inc->rname);
+			ast_get_context_name(con), include_rname(inc));
 		break;
 	}
 
@@ -8703,6 +8852,11 @@ static void pbx_shutdown(void)
 		ao2_ref(hintdevices, -1);
 		hintdevices = NULL;
 	}
+	if (autohints) {
+		ao2_container_unregister("autohints");
+		ao2_ref(autohints, -1);
+		autohints = NULL;
+	}
 	if (statecbs) {
 		ao2_container_unregister("statecbs");
 		ao2_ref(statecbs, -1);
@@ -8736,6 +8890,16 @@ static void print_hintdevices_key(void *
 		ast_get_context_name(ast_get_extension_context(hintdevice->hint->exten)));
 }
 
+static void print_autohint_key(void *v_obj, void *where, ao2_prnt_fn *prnt)
+{
+	struct ast_autohint *autohint = v_obj;
+
+	if (!autohint) {
+		return;
+	}
+	prnt(where, "%s", autohint->context);
+}
+
 static void print_statecbs_key(void *v_obj, void *where, ao2_prnt_fn *prnt)
 {
 	struct ast_state_cb *state_cb = v_obj;
@@ -8748,15 +8912,23 @@ static void print_statecbs_key(void *v_o
 
 int ast_pbx_init(void)
 {
-	hints = ao2_container_alloc(HASH_EXTENHINT_SIZE, hint_hash, hint_cmp);
+	hints = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		HASH_EXTENHINT_SIZE, hint_hash, NULL, hint_cmp);
 	if (hints) {
 		ao2_container_register("hints", hints, print_hints_key);
 	}
-	hintdevices = ao2_container_alloc(HASH_EXTENHINT_SIZE, hintdevice_hash_cb, hintdevice_cmp_multiple);
+	hintdevices = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		HASH_EXTENHINT_SIZE, hintdevice_hash_cb, NULL, hintdevice_cmp_multiple);
 	if (hintdevices) {
 		ao2_container_register("hintdevices", hintdevices, print_hintdevices_key);
 	}
-	statecbs = ao2_container_alloc(1, NULL, statecbs_cmp);
+	/* This is protected by the context_and_merge lock */
+	autohints = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, HASH_EXTENHINT_SIZE,
+		autohint_hash_cb, NULL, autohint_cmp);
+	if (autohints) {
+		ao2_container_register("autohints", autohints, print_autohint_key);
+	}
+	statecbs = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, statecbs_cmp);
 	if (statecbs) {
 		ao2_container_register("statecbs", statecbs, print_statecbs_key);
 	}
@@ -8767,5 +8939,5 @@ int ast_pbx_init(void)
 		return -1;
 	}
 
-	return (hints && hintdevices && statecbs) ? 0 : -1;
+	return (hints && hintdevices && autohints && statecbs) ? 0 : -1;
 }
diff -pruN 1:13.23.1~dfsg-2/main/pbx_functions.c 1:16.1.1~dfsg-1/main/pbx_functions.c
--- 1:13.23.1~dfsg-2/main/pbx_functions.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_functions.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/_private.h"
 #include "asterisk/cli.h"
 #include "asterisk/linkedlists.h"
diff -pruN 1:13.23.1~dfsg-2/main/pbx_hangup_handler.c 1:16.1.1~dfsg-1/main/pbx_hangup_handler.c
--- 1:13.23.1~dfsg-2/main/pbx_hangup_handler.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_hangup_handler.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/app.h"
 #include "asterisk/cli.h"
diff -pruN 1:13.23.1~dfsg-2/main/pbx_ignorepat.c 1:16.1.1~dfsg-1/main/pbx_ignorepat.c
--- 1:13.23.1~dfsg-2/main/pbx_ignorepat.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_ignorepat.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,80 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, CFWare, LLC
+ *
+ * Corey Farrell <git@cfware.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Dialplan context ignorepat routines.
+ *
+ * \author Corey Farrell <git@cfware.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/_private.h"
+#include "asterisk/pbx.h"
+#include "pbx_private.h"
+
+/*! \brief ast_ignorepat: Ignore patterns in dial plan */
+struct ast_ignorepat {
+	const char *registrar;
+	const char pattern[0];
+};
+
+const char *ast_get_ignorepat_name(const struct ast_ignorepat *ip)
+{
+	return ip ? ip->pattern : NULL;
+}
+
+const char *ast_get_ignorepat_registrar(const struct ast_ignorepat *ip)
+{
+	return ip ? ip->registrar : NULL;
+}
+
+struct ast_ignorepat *ignorepat_alloc(const char *value, const char *registrar)
+{
+	struct ast_ignorepat *ignorepat;
+	int length = strlen(value) + 1;
+	char *pattern;
+
+	/* allocate new include structure ... */
+	ignorepat = ast_calloc(1, sizeof(*ignorepat) + length);
+	if (!ignorepat) {
+		return NULL;
+	}
+
+	/* The cast to char * is because we need to write the initial value.
+	 * The field is not supposed to be modified otherwise.  Also, gcc 4.2
+	 * sees the cast as dereferencing a type-punned pointer and warns about
+	 * it.  This is the workaround (we're telling gcc, yes, that's really
+	 * what we wanted to do).
+	 */
+	pattern = (char *) ignorepat->pattern;
+	strcpy(pattern, value);
+	ignorepat->registrar = registrar;
+
+	return ignorepat;
+}
+
+void ignorepat_free(struct ast_ignorepat *ip)
+{
+	ast_free(ip);
+}
diff -pruN 1:13.23.1~dfsg-2/main/pbx_include.c 1:16.1.1~dfsg-1/main/pbx_include.c
--- 1:13.23.1~dfsg-2/main/pbx_include.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_include.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,110 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, CFWare, LLC
+ *
+ * Corey Farrell <git@cfware.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Dialplan context include routines.
+ *
+ * \author Corey Farrell <git@cfware.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/_private.h"
+#include "asterisk/pbx.h"
+#include "pbx_private.h"
+
+/*! ast_include: include= support in extensions.conf */
+struct ast_include {
+	const char *name;
+	/*! Context to include */
+	const char *rname;
+	/*! Registrar */
+	const char *registrar;
+	/*! If time construct exists */
+	int hastime;
+	/*! time construct */
+	struct ast_timing timing;
+	char stuff[0];
+};
+
+const char *ast_get_include_name(const struct ast_include *inc)
+{
+	return inc ? inc->name : NULL;
+}
+
+const char *include_rname(const struct ast_include *inc)
+{
+	return inc ? inc->rname : NULL;
+}
+
+const char *ast_get_include_registrar(const struct ast_include *inc)
+{
+	return inc ? inc->registrar : NULL;
+}
+
+int include_valid(const struct ast_include *inc)
+{
+	if (!inc->hastime) {
+		return 1;
+	}
+
+	return ast_check_timing(&(inc->timing));
+}
+
+struct ast_include *include_alloc(const char *value, const char *registrar)
+{
+	struct ast_include *new_include;
+	char *c;
+	int valuebufsz = strlen(value) + 1;
+	char *p;
+
+	/* allocate new include structure ... */
+	new_include = ast_calloc(1, sizeof(*new_include) + (valuebufsz * 2));
+	if (!new_include) {
+		return NULL;
+	}
+
+	/* Fill in this structure. Use 'p' for assignments, as the fields
+	 * in the structure are 'const char *'
+	 */
+	p = new_include->stuff;
+	new_include->name = p;
+	strcpy(p, value);
+	p += valuebufsz;
+	new_include->rname = p;
+	strcpy(p, value);
+	/* Strip off timing info, and process if it is there */
+	if ((c = strchr(p, ',')) ) {
+		*c++ = '\0';
+		new_include->hastime = ast_build_timing(&(new_include->timing), c);
+	}
+	new_include->registrar = registrar;
+
+	return new_include;
+}
+
+void include_free(struct ast_include *inc)
+{
+	ast_destroy_timing(&(inc->timing));
+	ast_free(inc);
+}
diff -pruN 1:13.23.1~dfsg-2/main/pbx_private.h 1:16.1.1~dfsg-1/main/pbx_private.h
--- 1:13.23.1~dfsg-2/main/pbx_private.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_private.h	2018-12-26 18:45:13.000000000 +0000
@@ -31,6 +31,30 @@ void set_ext_pri(struct ast_channel *c,
 /*! pbx.c function needed by pbx_app.c */
 void unreference_cached_app(struct ast_app *app);
 
+/*! pbx_ignorepat.c */
+struct ast_ignorepat;
+AST_VECTOR(ast_ignorepats, struct ast_ignorepat *);
+
+struct ast_ignorepat *ignorepat_alloc(const char *value, const char *registrar);
+void ignorepat_free(struct ast_ignorepat *ip);
+
+/*! pbx_includes.c */
+struct ast_include;
+AST_VECTOR(ast_includes, struct ast_include *);
+
+/*! Allocate and initialize an ast_include. */
+struct ast_include *include_alloc(const char *value, const char *registrar);
+/*! Free an ast_include and associated data. */
+void include_free(struct ast_include *inc);
+int include_valid(const struct ast_include *inc);
+const char *include_rname(const struct ast_include *inc);
+
+/*! pbx_sw.c */
+struct ast_sw;
+AST_VECTOR(ast_sws, struct ast_sw *);
+struct ast_sw *sw_alloc(const char *value, const char *data, int eval, const char *registrar);
+void sw_free(struct ast_sw *sw);
+
 /*! pbx_builtins.c functions needed by pbx.c */
 int indicate_congestion(struct ast_channel *, const char *);
 int indicate_busy(struct ast_channel *, const char *);
diff -pruN 1:13.23.1~dfsg-2/main/pbx_sw.c 1:16.1.1~dfsg-1/main/pbx_sw.c
--- 1:13.23.1~dfsg-2/main/pbx_sw.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_sw.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,104 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, CFWare, LLC
+ *
+ * Corey Farrell <git@cfware.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Dialplan switch routines.
+ *
+ * \author Corey Farrell <git@cfware.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/_private.h"
+#include "asterisk/pbx.h"
+#include "pbx_private.h"
+
+/*! \brief ast_sw: Switch statement in extensions.conf */
+struct ast_sw {
+	const char *name;
+	/*! Registrar */
+	const char *registrar;
+	/*! Data load */
+	const char *data;
+	int eval;
+	AST_LIST_ENTRY(ast_sw) list;
+	char stuff[0];
+};
+
+const char *ast_get_switch_name(const struct ast_sw *sw)
+{
+	return sw ? sw->name : NULL;
+}
+
+const char *ast_get_switch_data(const struct ast_sw *sw)
+{
+	return sw ? sw->data : NULL;
+}
+
+int ast_get_switch_eval(const struct ast_sw *sw)
+{
+	return sw->eval;
+}
+
+const char *ast_get_switch_registrar(const struct ast_sw *sw)
+{
+	return sw ? sw->registrar : NULL;
+}
+
+struct ast_sw *sw_alloc(const char *value, const char *data, int eval, const char *registrar)
+{
+	struct ast_sw *new_sw;
+	int length;
+	char *p;
+
+	if (!data) {
+		data = "";
+	}
+	length = sizeof(struct ast_sw);
+	length += strlen(value) + 1;
+	length += strlen(data) + 1;
+
+	/* allocate new sw structure ... */
+	if (!(new_sw = ast_calloc(1, length))) {
+		return NULL;
+	}
+
+	/* ... fill in this structure ... */
+	p = new_sw->stuff;
+	new_sw->name = p;
+	strcpy(p, value);
+
+	p += strlen(value) + 1;
+	new_sw->data = p;
+	strcpy(p, data);
+
+	new_sw->eval	  = eval;
+	new_sw->registrar = registrar;
+
+	return new_sw;
+}
+
+void sw_free(struct ast_sw *sw)
+{
+	ast_free(sw);
+}
diff -pruN 1:13.23.1~dfsg-2/main/pbx_switch.c 1:16.1.1~dfsg-1/main/pbx_switch.c
--- 1:13.23.1~dfsg-2/main/pbx_switch.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_switch.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/cli.h"
 #include "asterisk/linkedlists.h"
diff -pruN 1:13.23.1~dfsg-2/main/pbx_timing.c 1:16.1.1~dfsg-1/main/pbx_timing.c
--- 1:13.23.1~dfsg-2/main/pbx_timing.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_timing.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/localtime.h"
 #include "asterisk/logger.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/main/pbx_variables.c 1:16.1.1~dfsg-1/main/pbx_variables.c
--- 1:13.23.1~dfsg-2/main/pbx_variables.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pbx_variables.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/app.h"
 #include "asterisk/ast_expr.h"
diff -pruN 1:13.23.1~dfsg-2/main/pickup.c 1:16.1.1~dfsg-1/main/pickup.c
--- 1:13.23.1~dfsg-2/main/pickup.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/pickup.c	2018-12-26 18:45:13.000000000 +0000
@@ -47,8 +47,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/pickup.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -137,7 +135,7 @@ struct ast_channel *ast_pickup_find_by_g
 	struct ao2_container *candidates;/*!< Candidate channels found to pickup. */
 	struct ast_channel *target;/*!< Potential pickup target */
 
-	candidates = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL);
+	candidates = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
 	if (!candidates) {
 		return NULL;
 	}
diff -pruN 1:13.23.1~dfsg-2/main/plc.c 1:16.1.1~dfsg-1/main/plc.c
--- 1:13.23.1~dfsg-2/main/plc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/plc.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,10 +36,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 
+#include "asterisk/config.h"
+#include "asterisk/module.h"
 #include "asterisk/plc.h"
 
 #if !defined(FALSE)
@@ -248,3 +248,53 @@ plc_state_t *plc_init(plc_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 /*- End of file ------------------------------------------------------------*/
+
+static int reload_module(void)
+{
+	struct ast_variable *var;
+	struct ast_flags config_flags = { 0 };
+	struct ast_config *cfg = ast_config_load("codecs.conf", config_flags);
+
+	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID) {
+		return 0;
+	}
+
+	for (var = ast_variable_browse(cfg, "plc"); var; var = var->next) {
+		if (!strcasecmp(var->name, "genericplc")) {
+			ast_set2_flag(&ast_options, ast_true(var->value), AST_OPT_FLAG_GENERIC_PLC);
+		} else if (!strcasecmp(var->name, "genericplc_on_equal_codecs")) {
+			ast_set2_flag(&ast_options, ast_true(var->value), AST_OPT_FLAG_GENERIC_PLC_ON_EQUAL_CODECS);
+		}
+	}
+	ast_config_destroy(cfg);
+
+	/*
+	 * Force on_equal_codecs to false if generic_plc is false.
+	 */
+	if (!ast_opt_generic_plc) {
+		ast_set2_flag(&ast_options, 0, AST_OPT_FLAG_GENERIC_PLC_ON_EQUAL_CODECS);
+	}
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	reload_module();
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	return 0;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "PLC",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/presencestate.c 1:16.1.1~dfsg-1/main/presencestate.c
--- 1:13.23.1~dfsg-2/main/presencestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/presencestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -57,8 +57,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/utils.h"
 #include "asterisk/lock.h"
@@ -66,6 +64,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/presencestate.h"
 #include "asterisk/pbx.h"
 #include "asterisk/app.h"
+#include "asterisk/test.h"
 
 /*! \brief Device state strings for printing */
 static const struct {
@@ -146,45 +145,77 @@ static enum ast_presence_state presence_
 
 static enum ast_presence_state ast_presence_state_helper(const char *presence_provider, char **subtype, char **message, int check_cache)
 {
-	struct presence_state_provider *provider;
-	char *address;
-	char *label = ast_strdupa(presence_provider);
-	int res = AST_PRESENCE_INVALID;
+	char *labels = ast_strdupa(presence_provider);
+	char *label;
+	enum ast_presence_state state = AST_PRESENCE_INVALID;
+	enum ast_presence_state state_order[] = {
+		[AST_PRESENCE_INVALID]     = 0,
+		[AST_PRESENCE_NOT_SET]     = 1,
+		[AST_PRESENCE_AVAILABLE]   = 2,
+		[AST_PRESENCE_UNAVAILABLE] = 3,
+		[AST_PRESENCE_CHAT]        = 4,
+		[AST_PRESENCE_AWAY]        = 5,
+		[AST_PRESENCE_XA]          = 6,
+		[AST_PRESENCE_DND]         = 7
+	};
 
 	*subtype = NULL;
 	*message = NULL;
 
-	if (check_cache) {
-		res = presence_state_cached(presence_provider, subtype, message);
-		if (res != AST_PRESENCE_INVALID) {
-			return res;
+	while ((label = strsep(&labels, "&"))) {
+		enum ast_presence_state next_state = AST_PRESENCE_INVALID;
+		char *next_subtype = NULL;
+		char *next_message = NULL;
+
+		if (check_cache) {
+			next_state = presence_state_cached(label, &next_subtype, &next_message);
 		}
-	}
 
-	if ((address = strchr(label, ':'))) {
-		*address = '\0';
-		address++;
-	} else {
-		ast_log(LOG_WARNING, "No label found for presence state provider: %s\n", presence_provider);
-		return res;
-	}
+		if (next_state == AST_PRESENCE_INVALID) {
+			struct presence_state_provider *provider;
+			const struct ast_channel_tech *chan_tech;
+			char *address;
+
+			if ((address = strchr(label, '/'))) {
+				*address++ = '\0';
+
+				if ((chan_tech = ast_get_channel_tech(label)) && chan_tech->presencestate) {
+					next_state = chan_tech->presencestate(address, &next_subtype, &next_message);
+				}
+			} else if ((address = strchr(label, ':'))) {
+				*address++ = '\0';
+
+				AST_RWLIST_RDLOCK(&presence_state_providers);
+				AST_RWLIST_TRAVERSE(&presence_state_providers, provider, list) {
+					ast_debug(5, "Checking provider %s with %s\n", provider->label, label);
+
+					if (!strcasecmp(provider->label, label)) {
+						next_state = provider->callback(address, &next_subtype, &next_message);
+						break;
+					}
+				}
+				AST_RWLIST_UNLOCK(&presence_state_providers);
+
+				if (!provider) {
+					ast_log(LOG_WARNING, "No provider found for label: %s\n", label);
+				}
+			} else {
+				ast_log(LOG_WARNING, "No label found for presence state provider: %s\n", label);
+			}
+		}
 
-	AST_RWLIST_RDLOCK(&presence_state_providers);
-	AST_RWLIST_TRAVERSE(&presence_state_providers, provider, list) {
-		ast_debug(5, "Checking provider %s with %s\n", provider->label, label);
+		if (state_order[next_state] > state_order[state]) {
+			state = next_state;
 
-		if (!strcasecmp(provider->label, label)) {
-			res = provider->callback(address, subtype, message);
-			break;
-		}
-	}
-	AST_RWLIST_UNLOCK(&presence_state_providers);
+			ast_free(*subtype);
+			ast_free(*message);
 
-	if (!provider) {
-		ast_log(LOG_WARNING, "No provider found for label %s\n", label);
+			*subtype = next_subtype;
+			*message = next_message;
+		}
 	}
 
-	return res;
+	return state;
 }
 
 enum ast_presence_state ast_presence_state(const char *presence_provider, char **subtype, char **message)
@@ -357,6 +388,99 @@ static const char *presence_state_get_id
 	return presence_state->provider;
 }
 
+#if defined(TEST_FRAMEWORK)
+
+#define TEST_CATEGORY "/main/presence/"
+
+static int presence_test_alice_state = AST_PRESENCE_UNAVAILABLE;
+static int presence_test_bob_state = AST_PRESENCE_UNAVAILABLE;
+
+static int presence_test_presencestate(const char *label, char **subtype, char **message)
+{
+	if (!strcmp(label, "Alice")) {
+		return presence_test_alice_state;
+	} else if (!strcmp(label, "Bob")) {
+		return presence_test_bob_state;
+	} else {
+		return AST_PRESENCE_UNAVAILABLE;
+	}
+}
+
+static struct ast_channel_tech presence_test_tech = {
+	.type = "PresenceTestChannel",
+	.description = "Presence test technology",
+	.presencestate = presence_test_presencestate,
+};
+
+AST_TEST_DEFINE(test_presence_chan)
+{
+	int res = AST_TEST_FAIL;
+	char provider[80];
+	enum ast_presence_state state;
+	char *subtype = NULL, *message = NULL;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "channel_presence";
+		info->category = TEST_CATEGORY;
+		info->summary = "Channel presence state tests";
+		info->description = "Creates test channel technology and then test the presence state callback";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_channel_register(&presence_test_tech)) {
+		ast_log(LOG_WARNING, "Unable to register channel type '%s'\n", presence_test_tech.type);
+		goto presence_test_cleanup;
+	}
+
+	/* Check Alice's state */
+	snprintf(provider, sizeof(provider), "%s/Alice", presence_test_tech.type);
+
+	presence_test_alice_state = AST_PRESENCE_AVAILABLE;
+	state = ast_presence_state_nocache(provider, &subtype, &message);
+
+	if (state != presence_test_alice_state) {
+		ast_log(LOG_WARNING, "Presence state of '%s' returned '%s' instead of the expected value '%s'\n",
+			provider, ast_presence_state2str(state), ast_presence_state2str(presence_test_alice_state));
+		goto presence_test_cleanup;
+	}
+
+	/* Check Alice's and Bob's state, Alice's should win as DND > AVAILABLE */
+	snprintf(provider, sizeof(provider), "%s/Alice&%s/Bob", presence_test_tech.type, presence_test_tech.type);
+
+	presence_test_alice_state = AST_PRESENCE_DND;
+	presence_test_bob_state = AST_PRESENCE_UNAVAILABLE;
+	state = ast_presence_state_nocache(provider, &subtype, &message);
+
+	if (state != presence_test_alice_state) {
+		ast_log(LOG_WARNING, "Presence state of '%s' returned '%s' instead of the expected value '%s'\n",
+			provider, ast_presence_state2str(state), ast_presence_state2str(presence_test_alice_state));
+		goto presence_test_cleanup;
+        }
+
+	/* Check Alice's and Bob's state, Bob's should now win as AVAILABLE < UNAVAILABLE */
+	presence_test_alice_state = AST_PRESENCE_AVAILABLE;
+	state = ast_presence_state_nocache(provider, &subtype, &message);
+
+	if (state != presence_test_bob_state) {
+		ast_log(LOG_WARNING, "Presence state of '%s' returned '%s' instead of the expected value '%s'\n",
+			provider, ast_presence_state2str(state), ast_presence_state2str(presence_test_bob_state));
+		goto presence_test_cleanup;
+	}
+
+	res = AST_TEST_PASS;
+
+presence_test_cleanup:
+	ast_channel_unregister(&presence_test_tech);
+	ast_free(subtype);
+	ast_free(message);
+
+	return res;
+}
+#endif
+
 static void presence_state_engine_cleanup(void)
 {
 	ao2_cleanup(presence_state_topic_all);
@@ -365,6 +489,7 @@ static void presence_state_engine_cleanu
 	presence_state_cache = NULL;
 	presence_state_topic_cached = stasis_caching_unsubscribe_and_join(presence_state_topic_cached);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_presence_state_message_type);
+	AST_TEST_UNREGISTER(test_presence_chan);
 }
 
 int ast_presence_state_engine_init(void)
@@ -389,6 +514,10 @@ int ast_presence_state_engine_init(void)
 	if (!presence_state_topic_cached) {
 		return -1;
 	}
+	stasis_caching_accept_message_type(presence_state_topic_cached, ast_presence_state_message_type());
+	stasis_caching_set_filter(presence_state_topic_cached, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
+
+	AST_TEST_REGISTER(test_presence_chan);
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/privacy.c 1:16.1.1~dfsg-1/main/privacy.c
--- 1:13.23.1~dfsg-2/main/privacy.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/privacy.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 #include <dirent.h>
diff -pruN 1:13.23.1~dfsg-2/main/rtp_engine.c 1:16.1.1~dfsg-1/main/rtp_engine.c
--- 1:13.23.1~dfsg-2/main/rtp_engine.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/rtp_engine.c	2018-12-26 18:45:13.000000000 +0000
@@ -143,8 +143,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>                       /* for sqrt, MAX */
 #include <sched.h>                      /* for sched_yield */
 #include <sys/time.h>                   /* for timeval */
@@ -179,6 +177,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 struct ast_srtp_res *res_srtp = NULL;
 struct ast_srtp_policy_res *res_srtp_policy = NULL;
 
+/*! Structure that contains extmap negotiation information */
+struct rtp_extmap {
+	/*! The RTP extension */
+	enum ast_rtp_extension extension;
+	/*! The current negotiated direction */
+	enum ast_rtp_extension_direction direction;
+};
+
 /*! Structure that represents an RTP session (instance) */
 struct ast_rtp_instance {
 	/*! Engine that is handling this RTP instance */
@@ -215,6 +221,20 @@ struct ast_rtp_instance {
 	time_t last_tx;
 	/*! Time of last packet received */
 	time_t last_rx;
+	/*! Enabled RTP extensions */
+	AST_VECTOR(, enum ast_rtp_extension_direction) extmap_enabled;
+	/*! Negotiated RTP extensions (using index based on extension) */
+	AST_VECTOR(, int) extmap_negotiated;
+	/*! Negotiated RTP extensions (using index based on unique id) */
+	AST_VECTOR(, struct rtp_extmap) extmap_unique_ids;
+};
+
+/*!
+ * \brief URIs for known RTP extensions
+ */
+static const char * const rtp_extension_uris[AST_RTP_EXTENSION_MAX] = {
+	[AST_RTP_EXTENSION_UNSUPPORTED]		= "",
+	[AST_RTP_EXTENSION_ABS_SEND_TIME]	= "http://www.webrtc.org/experiments/rtp-hdrext/abs-send-time",
 };
 
 /*! List of RTP engines that are currently registered */
@@ -268,14 +288,28 @@ static void rtp_payload_type_dtor(void *
 	ao2_cleanup(payload->format);
 }
 
-struct ast_rtp_payload_type *ast_rtp_engine_alloc_payload_type(void)
+static struct ast_rtp_payload_type *rtp_payload_type_alloc(struct ast_format *format,
+	int payload, int rtp_code, int primary_mapping)
 {
-	struct ast_rtp_payload_type *payload;
+	struct ast_rtp_payload_type *type = ao2_alloc_options(
+		sizeof(*type), rtp_payload_type_dtor, AO2_ALLOC_OPT_LOCK_NOLOCK);
 
-	payload = ao2_alloc_options(sizeof(*payload), rtp_payload_type_dtor,
-		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!type) {
+		return NULL;
+	}
 
-	return payload;
+	type->format = ao2_bump(format);
+	type->asterisk_format = type->format != NULL;
+	type->payload = payload;
+	type->rtp_code = rtp_code;
+	type->primary_mapping = primary_mapping;
+
+	return type;
+}
+
+struct ast_rtp_payload_type *ast_rtp_engine_alloc_payload_type(void)
+{
+	return rtp_payload_type_alloc(NULL, 0, 0, 0);
 }
 
 int ast_rtp_engine_register2(struct ast_rtp_engine *engine, struct ast_module *module)
@@ -396,6 +430,10 @@ static void instance_destructor(void *ob
 
 	ast_rtp_codecs_payloads_destroy(&instance->codecs);
 
+	AST_VECTOR_FREE(&instance->extmap_enabled);
+	AST_VECTOR_FREE(&instance->extmap_negotiated);
+	AST_VECTOR_FREE(&instance->extmap_unique_ids);
+
 	/* Drop our engine reference */
 	ast_module_unref(instance->engine->mod);
 
@@ -416,6 +454,7 @@ struct ast_rtp_instance *ast_rtp_instanc
 	struct ast_sockaddr address = {{0,}};
 	struct ast_rtp_instance *instance = NULL;
 	struct ast_rtp_engine *engine = NULL;
+	struct ast_module *mod_ref;
 
 	AST_RWLIST_RDLOCK(&engines);
 
@@ -438,10 +477,15 @@ struct ast_rtp_instance *ast_rtp_instanc
 	}
 
 	/* Bump up the reference count before we return so the module can not be unloaded */
-	ast_module_ref(engine->mod);
+	mod_ref = ast_module_running_ref(engine->mod);
 
 	AST_RWLIST_UNLOCK(&engines);
 
+	if (!mod_ref) {
+		/* BUGBUG: improve handling of this situation. */
+		return NULL;
+	}
+
 	/* Allocate a new RTP instance */
 	if (!(instance = ao2_alloc(sizeof(*instance), instance_destructor))) {
 		ast_module_unref(engine->mod);
@@ -456,6 +500,14 @@ struct ast_rtp_instance *ast_rtp_instanc
 		return NULL;
 	}
 
+	/* Initialize RTP extension support */
+	if (AST_VECTOR_INIT(&instance->extmap_enabled, 0) ||
+		AST_VECTOR_INIT(&instance->extmap_negotiated, 0) ||
+		AST_VECTOR_INIT(&instance->extmap_unique_ids, 0)) {
+		ao2_ref(instance, -1);
+		return NULL;
+	}
+
 	ast_debug(1, "Using engine '%s' for RTP instance '%p'\n", engine->name, instance);
 
 	/*
@@ -662,28 +714,264 @@ struct ast_rtp_codecs *ast_rtp_instance_
 	return &instance->codecs;
 }
 
+int ast_rtp_instance_extmap_enable(struct ast_rtp_instance *instance, int id, enum ast_rtp_extension extension,
+	enum ast_rtp_extension_direction direction)
+{
+	struct rtp_extmap extmap = {
+		.extension = extension,
+		.direction = direction,
+	};
+
+	ao2_lock(instance);
+
+	if (!instance->engine->extension_enable || !instance->engine->extension_enable(instance, extension)) {
+		ao2_unlock(instance);
+		return 0;
+	}
+
+	/* We store enabled extensions separately so we can easily do negotiation */
+	if (AST_VECTOR_REPLACE(&instance->extmap_enabled, extension, direction)) {
+		ao2_unlock(instance);
+		return -1;
+	}
+
+	if (id <= 0) {
+		/* We find a free unique identifier for this extension by just appending it to the
+		 * vector of unique ids. The size of the vector will become its unique identifier.
+		 * As well when we are asking for information on the extensions it will be returned,
+		 * allowing it to be added to the SDP offer.
+		 */
+		if (AST_VECTOR_APPEND(&instance->extmap_unique_ids, extmap)) {
+			AST_VECTOR_REPLACE(&instance->extmap_enabled, extension, AST_RTP_EXTENSION_DIRECTION_NONE);
+			ao2_unlock(instance);
+			return -1;
+		}
+		id = AST_VECTOR_SIZE(&instance->extmap_unique_ids);
+	} else {
+		/* Otherwise we put it precisely where they want it */
+		if (AST_VECTOR_REPLACE(&instance->extmap_unique_ids, id - 1, extmap)) {
+			AST_VECTOR_REPLACE(&instance->extmap_enabled, extension, AST_RTP_EXTENSION_DIRECTION_NONE);
+			ao2_unlock(instance);
+			return -1;
+		}
+	}
+
+	/* Now that we have an id add the extension to here */
+	if (AST_VECTOR_REPLACE(&instance->extmap_negotiated, extension, id)) {
+		extmap.extension = AST_RTP_EXTENSION_UNSUPPORTED;
+		extmap.direction = AST_RTP_EXTENSION_DIRECTION_NONE;
+		AST_VECTOR_REPLACE(&instance->extmap_enabled, extension, AST_RTP_EXTENSION_DIRECTION_NONE);
+		AST_VECTOR_REPLACE(&instance->extmap_unique_ids, id - 1, extmap);
+		ao2_unlock(instance);
+		return -1;
+	}
+
+	ao2_unlock(instance);
+
+	return 0;
+}
+
+/*! \brief Helper function which negotiates two RTP extension directions to get our current direction */
+static enum ast_rtp_extension_direction rtp_extmap_negotiate_direction(enum ast_rtp_extension_direction ours,
+	enum ast_rtp_extension_direction theirs)
+{
+	if (theirs == AST_RTP_EXTENSION_DIRECTION_NONE || ours == AST_RTP_EXTENSION_DIRECTION_NONE) {
+		/* This should not occur but if it does tolerate either side not having this extension
+		 * in use.
+		 */
+		return AST_RTP_EXTENSION_DIRECTION_NONE;
+	} else if (theirs == AST_RTP_EXTENSION_DIRECTION_INACTIVE) {
+		/* Inactive is always inactive on our side */
+		return AST_RTP_EXTENSION_DIRECTION_INACTIVE;
+	} else if (theirs == AST_RTP_EXTENSION_DIRECTION_SENDRECV) {
+		return ours;
+	} else if (theirs == AST_RTP_EXTENSION_DIRECTION_SENDONLY) {
+		/* If they are send only then we become recvonly if we are configured as sendrecv or recvonly */
+		if (ours == AST_RTP_EXTENSION_DIRECTION_SENDRECV || ours == AST_RTP_EXTENSION_DIRECTION_RECVONLY) {
+			return AST_RTP_EXTENSION_DIRECTION_RECVONLY;
+		}
+	} else if (theirs == AST_RTP_EXTENSION_DIRECTION_RECVONLY) {
+		/* If they are recv only then we become sendonly if we are configured as sendrecv or sendonly */
+		if (ours == AST_RTP_EXTENSION_DIRECTION_SENDRECV || ours == AST_RTP_EXTENSION_DIRECTION_SENDONLY) {
+			return AST_RTP_EXTENSION_DIRECTION_SENDONLY;
+		}
+	}
+
+	return AST_RTP_EXTENSION_DIRECTION_NONE;
+}
+
+int ast_rtp_instance_extmap_negotiate(struct ast_rtp_instance *instance, int id, enum ast_rtp_extension_direction direction,
+	const char *uri, const char *attributes)
+{
+	/* 'attributes' is currently unused but exists in the API to ensure it does not need to be altered
+	 * in the future in case we need to use it.
+	 */
+	int idx;
+	enum ast_rtp_extension extension = AST_RTP_EXTENSION_UNSUPPORTED;
+
+	/* Per the RFC the identifier has to be 1 or above */
+	if (id < 1) {
+		return -1;
+	}
+
+	/* Convert the provided URI to the internal representation */
+	for (idx = 0; idx < ARRAY_LEN(rtp_extension_uris); ++idx) {
+		if (!strcasecmp(rtp_extension_uris[idx], uri)) {
+			extension = idx;
+			break;
+		}
+	}
+
+	ao2_lock(instance);
+	/* We only accept the extension if it is enabled */
+	if (extension < AST_VECTOR_SIZE(&instance->extmap_enabled) &&
+		AST_VECTOR_GET(&instance->extmap_enabled, extension) != AST_RTP_EXTENSION_DIRECTION_NONE) {
+		struct rtp_extmap extmap = {
+			.extension = extension,
+			.direction = rtp_extmap_negotiate_direction(AST_VECTOR_GET(&instance->extmap_enabled, extension), direction),
+		};
+
+		/* If the direction negotiation failed then don't accept or use this extension */
+		if (extmap.direction != AST_RTP_EXTENSION_DIRECTION_NONE) {
+			if (extension != AST_RTP_EXTENSION_UNSUPPORTED) {
+				AST_VECTOR_REPLACE(&instance->extmap_negotiated, extension, id);
+			}
+			AST_VECTOR_REPLACE(&instance->extmap_unique_ids, id - 1, extmap);
+		}
+	}
+	ao2_unlock(instance);
+
+	return 0;
+}
+
+void ast_rtp_instance_extmap_clear(struct ast_rtp_instance *instance)
+{
+	static const struct rtp_extmap extmap_none = {
+		.extension = AST_RTP_EXTENSION_UNSUPPORTED,
+		.direction = AST_RTP_EXTENSION_DIRECTION_NONE,
+	};
+	int idx;
+
+	ao2_lock(instance);
+
+	/* Clear both the known unique ids and the negotiated extensions as we are about to have
+	 * new results set on us.
+	 */
+	for (idx = 0; idx < AST_VECTOR_SIZE(&instance->extmap_unique_ids); ++idx) {
+		AST_VECTOR_REPLACE(&instance->extmap_unique_ids, idx, extmap_none);
+	}
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&instance->extmap_negotiated); ++idx) {
+		AST_VECTOR_REPLACE(&instance->extmap_negotiated, idx, -1);
+	}
+
+	ao2_unlock(instance);
+}
+
+int ast_rtp_instance_extmap_get_id(struct ast_rtp_instance *instance, enum ast_rtp_extension extension)
+{
+	int id = -1;
+
+	ao2_lock(instance);
+	if (extension < AST_VECTOR_SIZE(&instance->extmap_negotiated)) {
+		id = AST_VECTOR_GET(&instance->extmap_negotiated, extension);
+	}
+	ao2_unlock(instance);
+
+	return id;
+}
+
+size_t ast_rtp_instance_extmap_count(struct ast_rtp_instance *instance)
+{
+	size_t count;
+
+	ao2_lock(instance);
+	count = AST_VECTOR_SIZE(&instance->extmap_unique_ids);
+	ao2_unlock(instance);
+
+	return count;
+}
+
+enum ast_rtp_extension ast_rtp_instance_extmap_get_extension(struct ast_rtp_instance *instance, int id)
+{
+	enum ast_rtp_extension extension = AST_RTP_EXTENSION_UNSUPPORTED;
+
+	ao2_lock(instance);
+
+	/* The local unique identifier starts at '1' so the highest unique identifier
+	 * can be the actual size of the vector. We compensate (as it is 0 index based)
+	 * by dropping it down to 1 to get the correct information.
+	 */
+	if (0 < id && id <= AST_VECTOR_SIZE(&instance->extmap_unique_ids)) {
+		struct rtp_extmap *extmap = AST_VECTOR_GET_ADDR(&instance->extmap_unique_ids, id - 1);
+
+		extension = extmap->extension;
+	}
+	ao2_unlock(instance);
+
+	return extension;
+}
+
+enum ast_rtp_extension_direction ast_rtp_instance_extmap_get_direction(struct ast_rtp_instance *instance, int id)
+{
+	enum ast_rtp_extension_direction direction = AST_RTP_EXTENSION_DIRECTION_NONE;
+
+	ao2_lock(instance);
+
+	if (0 < id && id <= AST_VECTOR_SIZE(&instance->extmap_unique_ids)) {
+		struct rtp_extmap *extmap = AST_VECTOR_GET_ADDR(&instance->extmap_unique_ids, id - 1);
+
+		direction = extmap->direction;
+	}
+	ao2_unlock(instance);
+
+	return direction;
+}
+
+const char *ast_rtp_instance_extmap_get_uri(struct ast_rtp_instance *instance, int id)
+{
+	enum ast_rtp_extension extension = ast_rtp_instance_extmap_get_extension(instance, id);
+
+	if (extension == AST_RTP_EXTENSION_UNSUPPORTED ||
+		(unsigned int)extension >= ARRAY_LEN(rtp_extension_uris)) {
+		return NULL;
+	}
+
+	return rtp_extension_uris[extension];
+}
+
 int ast_rtp_codecs_payloads_initialize(struct ast_rtp_codecs *codecs)
 {
 	int res;
 
 	codecs->framing = 0;
 	ast_rwlock_init(&codecs->codecs_lock);
-	res = AST_VECTOR_INIT(&codecs->payloads, AST_RTP_MAX_PT);
+	res = AST_VECTOR_INIT(&codecs->payload_mapping_rx, AST_RTP_MAX_PT);
+	res |= AST_VECTOR_INIT(&codecs->payload_mapping_tx, AST_RTP_MAX_PT);
+	if (res) {
+		AST_VECTOR_FREE(&codecs->payload_mapping_rx);
+		AST_VECTOR_FREE(&codecs->payload_mapping_tx);
+	}
 
 	return res;
 }
 
 void ast_rtp_codecs_payloads_destroy(struct ast_rtp_codecs *codecs)
 {
-	int i;
+	int idx;
+	struct ast_rtp_payload_type *type;
 
-	for (i = 0; i < AST_VECTOR_SIZE(&codecs->payloads); i++) {
-		struct ast_rtp_payload_type *type;
+	for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++idx) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_rx, idx);
+		ao2_t_cleanup(type, "destroying ast_rtp_codec rx mapping");
+	}
+	AST_VECTOR_FREE(&codecs->payload_mapping_rx);
 
-		type = AST_VECTOR_GET(&codecs->payloads, i);
-		ao2_t_cleanup(type, "destroying ast_rtp_codec");
+	for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_tx); ++idx) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_tx, idx);
+		ao2_t_cleanup(type, "destroying ast_rtp_codec tx mapping");
 	}
-	AST_VECTOR_FREE(&codecs->payloads);
+	AST_VECTOR_FREE(&codecs->payload_mapping_tx);
 
 	ast_rwlock_destroy(&codecs->codecs_lock);
 }
@@ -704,9 +992,224 @@ void ast_rtp_codecs_payloads_clear(struc
 	}
 }
 
+/*!
+ * \internal
+ * \brief Clear the rx primary mapping flag on all other matching mappings.
+ * \since 14.0.0
+ *
+ * \param codecs Codecs that need rx clearing.
+ * \param to_match Payload type object to compare against.
+ *
+ * \note It is assumed that codecs is write locked before calling.
+ *
+ * \return Nothing
+ */
+static void payload_mapping_rx_clear_primary(struct ast_rtp_codecs *codecs, struct ast_rtp_payload_type *to_match)
+{
+	int idx;
+	struct ast_rtp_payload_type *current;
+	struct ast_rtp_payload_type *new_type;
+	struct timeval now;
+
+	if (!to_match->primary_mapping) {
+		return;
+	}
+
+	now = ast_tvnow();
+	for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++idx) {
+		current = AST_VECTOR_GET(&codecs->payload_mapping_rx, idx);
+
+		if (!current || current == to_match || !current->primary_mapping) {
+			continue;
+		}
+		if (current->asterisk_format && to_match->asterisk_format) {
+			if (ast_format_cmp(current->format, to_match->format) == AST_FORMAT_CMP_NOT_EQUAL) {
+				continue;
+			}
+		} else if (!current->asterisk_format && !to_match->asterisk_format) {
+			if (current->rtp_code != to_match->rtp_code) {
+				continue;
+			}
+		} else {
+			continue;
+		}
+
+		/* Replace current with non-primary marked version */
+		new_type = ast_rtp_engine_alloc_payload_type();
+		if (!new_type) {
+			continue;
+		}
+		*new_type = *current;
+		new_type->primary_mapping = 0;
+		new_type->when_retired = now;
+		ao2_bump(new_type->format);
+		AST_VECTOR_REPLACE(&codecs->payload_mapping_rx, idx, new_type);
+		ao2_ref(current, -1);
+	}
+}
+
+/*!
+ * \internal
+ * \brief Put the new_type into the rx payload type mapping.
+ * \since 14.0.0
+ *
+ * \param codecs Codecs structure to put new_type into
+ * \param payload type position to replace.
+ * \param new_type RTP payload mapping object to store.
+ *
+ * \note It is assumed that codecs is write locked before calling.
+ *
+ * \return Nothing
+ */
+static void rtp_codecs_payload_replace_rx(struct ast_rtp_codecs *codecs, int payload, struct ast_rtp_payload_type *new_type)
+{
+	ao2_ref(new_type, +1);
+	if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_rx)) {
+		ao2_t_cleanup(AST_VECTOR_GET(&codecs->payload_mapping_rx, payload),
+			"cleaning up rx mapping vector element about to be replaced");
+	}
+	if (AST_VECTOR_REPLACE(&codecs->payload_mapping_rx, payload, new_type)) {
+		ao2_ref(new_type, -1);
+		return;
+	}
+
+	payload_mapping_rx_clear_primary(codecs, new_type);
+}
+
+/*!
+ * \internal
+ * \brief Copy the rx payload type mapping to the destination.
+ * \since 14.0.0
+ *
+ * \param src The source codecs structure
+ * \param dest The destination codecs structure that the values from src will be copied to
+ * \param instance Optionally the instance that the dst codecs structure belongs to
+ *
+ * \note It is assumed that src is at least read locked before calling.
+ * \note It is assumed that dest is write locked before calling.
+ *
+ * \return Nothing
+ */
+static void rtp_codecs_payloads_copy_rx(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance)
+{
+	int idx;
+	struct ast_rtp_payload_type *type;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&src->payload_mapping_rx); ++idx) {
+		type = AST_VECTOR_GET(&src->payload_mapping_rx, idx);
+		if (!type) {
+			continue;
+		}
+
+		ast_debug(2, "Copying rx payload mapping %d (%p) from %p to %p\n",
+			idx, type, src, dest);
+		rtp_codecs_payload_replace_rx(dest, idx, type);
+
+		if (instance && instance->engine && instance->engine->payload_set) {
+			ao2_lock(instance);
+			instance->engine->payload_set(instance, idx, type->asterisk_format, type->format, type->rtp_code);
+			ao2_unlock(instance);
+		}
+	}
+}
+
+/*!
+ * \internal
+ * \brief Determine if a type of payload is already present in mappings.
+ * \since 14.0.0
+ *
+ * \param codecs Codecs to be checked for mappings.
+ * \param to_match Payload type object to compare against.
+ *
+ * \note It is assumed that codecs is write locked before calling.
+ *
+ * \retval 0 not found
+ * \retval 1 found
+ */
+static int payload_mapping_tx_is_present(const struct ast_rtp_codecs *codecs, const struct ast_rtp_payload_type *to_match)
+{
+	int idx;
+	struct ast_rtp_payload_type *current;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_tx); ++idx) {
+		current = AST_VECTOR_GET(&codecs->payload_mapping_tx, idx);
+
+		if (!current) {
+			continue;
+		}
+		if (current == to_match) {
+			/* The exact object is already in the mapping. */
+			return 1;
+		}
+		if (current->asterisk_format && to_match->asterisk_format) {
+			if (ast_format_get_codec_id(current->format) != ast_format_get_codec_id(to_match->format)) {
+				continue;
+			} else if (current->payload == to_match->payload) {
+				return 0;
+			}
+		} else if (!current->asterisk_format && !to_match->asterisk_format) {
+			if (current->rtp_code != to_match->rtp_code) {
+				continue;
+			}
+		} else {
+			continue;
+		}
+
+		return 1;
+	}
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Copy the tx payload type mapping to the destination.
+ * \since 14.0.0
+ *
+ * \param src The source codecs structure
+ * \param dest The destination codecs structure that the values from src will be copied to
+ * \param instance Optionally the instance that the dst codecs structure belongs to
+ *
+ * \note It is assumed that src is at least read locked before calling.
+ * \note It is assumed that dest is write locked before calling.
+ *
+ * \return Nothing
+ */
+static void rtp_codecs_payloads_copy_tx(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance)
+{
+	int idx;
+	struct ast_rtp_payload_type *type;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&src->payload_mapping_tx); ++idx) {
+		type = AST_VECTOR_GET(&src->payload_mapping_tx, idx);
+		if (!type) {
+			continue;
+		}
+
+		ast_debug(2, "Copying tx payload mapping %d (%p) from %p to %p\n",
+			idx, type, src, dest);
+		ao2_ref(type, +1);
+		if (idx < AST_VECTOR_SIZE(&dest->payload_mapping_tx)) {
+			ao2_t_cleanup(AST_VECTOR_GET(&dest->payload_mapping_tx, idx),
+				"cleaning up tx mapping vector element about to be replaced");
+		}
+		if (AST_VECTOR_REPLACE(&dest->payload_mapping_tx, idx, type)) {
+			ao2_ref(type, -1);
+			continue;
+		}
+
+		if (instance && instance->engine && instance->engine->payload_set) {
+			ao2_lock(instance);
+			instance->engine->payload_set(instance, idx, type->asterisk_format, type->format, type->rtp_code);
+			ao2_unlock(instance);
+		}
+	}
+}
+
 void ast_rtp_codecs_payloads_copy(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance)
 {
-	int i;
+	int idx;
+	struct ast_rtp_payload_type *type;
 
 	ast_rwlock_wrlock(&dest->codecs_lock);
 
@@ -717,29 +1220,66 @@ void ast_rtp_codecs_payloads_copy(struct
 		ast_rwlock_wrlock(&dest->codecs_lock);
 	}
 
-	for (i = 0; i < AST_VECTOR_SIZE(&src->payloads); i++) {
-		struct ast_rtp_payload_type *type;
+	/*
+	 * This represents a completely new mapping of what the remote party is
+	 * expecting for payloads, so we clear out the entire tx payload mapping
+	 * vector and replace it.
+	 */
+	for (idx = 0; idx < AST_VECTOR_SIZE(&dest->payload_mapping_tx); ++idx) {
+		type = AST_VECTOR_GET(&dest->payload_mapping_tx, idx);
+		ao2_t_cleanup(type, "destroying ast_rtp_codec tx mapping");
+		AST_VECTOR_REPLACE(&dest->payload_mapping_tx, idx, NULL);
+	}
+
+	rtp_codecs_payloads_copy_rx(src, dest, instance);
+	rtp_codecs_payloads_copy_tx(src, dest, instance);
+	dest->framing = src->framing;
 
-		type = AST_VECTOR_GET(&src->payloads, i);
+	ast_rwlock_unlock(&src->codecs_lock);
+	ast_rwlock_unlock(&dest->codecs_lock);
+}
+
+void ast_rtp_codecs_payloads_xover(struct ast_rtp_codecs *src, struct ast_rtp_codecs *dest, struct ast_rtp_instance *instance)
+{
+	int idx;
+	struct ast_rtp_payload_type *type;
+
+	ast_rwlock_wrlock(&dest->codecs_lock);
+	if (src != dest) {
+		/* Deadlock avoidance because of held write lock. */
+		while (ast_rwlock_tryrdlock(&src->codecs_lock)) {
+			ast_rwlock_unlock(&dest->codecs_lock);
+			sched_yield();
+			ast_rwlock_wrlock(&dest->codecs_lock);
+		}
+	}
+
+	/* Crossover copy payload type tx mapping to rx mapping. */
+	for (idx = 0; idx < AST_VECTOR_SIZE(&src->payload_mapping_tx); ++idx) {
+		type = AST_VECTOR_GET(&src->payload_mapping_tx, idx);
 		if (!type) {
 			continue;
 		}
-		if (i < AST_VECTOR_SIZE(&dest->payloads)) {
-			ao2_t_cleanup(AST_VECTOR_GET(&dest->payloads, i), "cleaning up vector element about to be replaced");
-		}
-		ast_debug(2, "Copying payload %d (%p) from %p to %p\n", i, type, src, dest);
 
-		ao2_bump(type);
-		if (AST_VECTOR_REPLACE(&dest->payloads, i, type)) {
-			ao2_cleanup(type);
-		} else if (instance && instance->engine && instance->engine->payload_set) {
+		/* All tx mapping elements should have the primary flag set. */
+		ast_assert(type->primary_mapping);
+
+		ast_debug(2, "Crossover copying tx to rx payload mapping %d (%p) from %p to %p\n",
+			idx, type, src, dest);
+		rtp_codecs_payload_replace_rx(dest, idx, type);
+
+		if (instance && instance->engine && instance->engine->payload_set) {
 			ao2_lock(instance);
-			instance->engine->payload_set(instance, i, type->asterisk_format, type->format, type->rtp_code);
+			instance->engine->payload_set(instance, idx, type->asterisk_format, type->format, type->rtp_code);
 			ao2_unlock(instance);
 		}
 	}
+
 	dest->framing = src->framing;
-	ast_rwlock_unlock(&src->codecs_lock);
+
+	if (src != dest) {
+		ast_rwlock_unlock(&src->codecs_lock);
+	}
 	ast_rwlock_unlock(&dest->codecs_lock);
 }
 
@@ -747,7 +1287,7 @@ void ast_rtp_codecs_payloads_set_m_type(
 {
 	struct ast_rtp_payload_type *new_type;
 
-	if (payload < 0 || payload >= AST_RTP_MAX_PT) {
+	if (payload < 0 || payload >= AST_RTP_MAX_PT || payload > AST_RTP_PT_LAST_STATIC) {
 		return;
 	}
 
@@ -765,16 +1305,21 @@ void ast_rtp_codecs_payloads_set_m_type(
 
 	ast_rwlock_wrlock(&codecs->codecs_lock);
 
-	if (payload < AST_VECTOR_SIZE(&codecs->payloads)) {
-		ao2_t_cleanup(AST_VECTOR_GET(&codecs->payloads, payload), "cleaning up replaced payload type");
-	}
+	if (!payload_mapping_tx_is_present(codecs, new_type)) {
+		if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+			ao2_t_cleanup(AST_VECTOR_GET(&codecs->payload_mapping_tx, payload),
+				"cleaning up replaced tx payload type");
+		}
 
-	if (AST_VECTOR_REPLACE(&codecs->payloads, payload, new_type)) {
+		if (AST_VECTOR_REPLACE(&codecs->payload_mapping_tx, payload, new_type)) {
+			ao2_ref(new_type, -1);
+		} else if (instance && instance->engine && instance->engine->payload_set) {
+			ao2_lock(instance);
+			instance->engine->payload_set(instance, payload, new_type->asterisk_format, new_type->format, new_type->rtp_code);
+			ao2_unlock(instance);
+		}
+	} else {
 		ao2_ref(new_type, -1);
-	} else if (instance && instance->engine && instance->engine->payload_set) {
-		ao2_lock(instance);
-		instance->engine->payload_set(instance, payload, new_type->asterisk_format, new_type->format, new_type->rtp_code);
-		ao2_unlock(instance);
 	}
 
 	ast_rwlock_unlock(&codecs->codecs_lock);
@@ -785,7 +1330,7 @@ int ast_rtp_codecs_payloads_set_rtpmap_t
 				 enum ast_rtp_options options,
 				 unsigned int sample_rate)
 {
-	unsigned int i;
+	unsigned int idx;
 	int found = 0;
 
 	if (pt < 0 || pt >= AST_RTP_MAX_PT) {
@@ -794,8 +1339,9 @@ int ast_rtp_codecs_payloads_set_rtpmap_t
 
 	ast_rwlock_rdlock(&mime_types_lock);
 	ast_rwlock_wrlock(&codecs->codecs_lock);
-	for (i = 0; i < mime_types_len; ++i) {
-		const struct ast_rtp_mime_type *t = &ast_rtp_mime_types[i];
+
+	for (idx = 0; idx < mime_types_len; ++idx) {
+		const struct ast_rtp_mime_type *t = &ast_rtp_mime_types[idx];
 		struct ast_rtp_payload_type *new_type;
 
 		if (strcasecmp(mimesubtype, t->subtype)) {
@@ -821,15 +1367,13 @@ int ast_rtp_codecs_payloads_set_rtpmap_t
 			continue;
 		}
 
-		if (pt < AST_VECTOR_SIZE(&codecs->payloads)) {
-			ao2_t_cleanup(AST_VECTOR_GET(&codecs->payloads, pt), "cleaning up replaced payload type");
-		}
-
-		new_type->payload = pt;
 		new_type->asterisk_format = t->payload_type.asterisk_format;
 		new_type->rtp_code = t->payload_type.rtp_code;
-		if ((ast_format_cmp(t->payload_type.format, ast_format_g726) == AST_FORMAT_CMP_EQUAL) &&
-				t->payload_type.asterisk_format && (options & AST_RTP_OPT_G726_NONSTANDARD)) {
+		new_type->payload = pt;
+		new_type->primary_mapping = 1;
+		if (t->payload_type.asterisk_format
+			&& ast_format_cmp(t->payload_type.format, ast_format_g726) == AST_FORMAT_CMP_EQUAL
+			&& (options & AST_RTP_OPT_G726_NONSTANDARD)) {
 			new_type->format = ast_format_g726_aal2;
 		} else {
 			new_type->format = t->payload_type.format;
@@ -840,16 +1384,26 @@ int ast_rtp_codecs_payloads_set_rtpmap_t
 			new_type->format = ast_format_parse_sdp_fmtp(new_type->format, "");
 		}
 
-		if (AST_VECTOR_REPLACE(&codecs->payloads, pt, new_type)) {
+		if (!payload_mapping_tx_is_present(codecs, new_type)) {
+			if (pt < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+				ao2_t_cleanup(AST_VECTOR_GET(&codecs->payload_mapping_tx, pt),
+					"cleaning up replaced tx payload type");
+			}
+
+			if (AST_VECTOR_REPLACE(&codecs->payload_mapping_tx, pt, new_type)) {
+				ao2_ref(new_type, -1);
+			} else if (instance && instance->engine && instance->engine->payload_set) {
+				ao2_lock(instance);
+				instance->engine->payload_set(instance, pt, new_type->asterisk_format, new_type->format, new_type->rtp_code);
+				ao2_unlock(instance);
+			}
+		} else {
 			ao2_ref(new_type, -1);
-		} else if (instance && instance->engine && instance->engine->payload_set) {
-			ao2_lock(instance);
-			instance->engine->payload_set(instance, pt, new_type->asterisk_format, new_type->format, new_type->rtp_code);
-			ao2_unlock(instance);
 		}
 
 		break;
 	}
+
 	ast_rwlock_unlock(&codecs->codecs_lock);
 	ast_rwlock_unlock(&mime_types_lock);
 
@@ -872,10 +1426,11 @@ void ast_rtp_codecs_payloads_unset(struc
 	ast_debug(2, "Unsetting payload %d on %p\n", payload, codecs);
 
 	ast_rwlock_wrlock(&codecs->codecs_lock);
-	if (payload < AST_VECTOR_SIZE(&codecs->payloads)) {
-		type = AST_VECTOR_GET(&codecs->payloads, payload);
+
+	if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_tx, payload);
 		ao2_cleanup(type);
-		AST_VECTOR_REPLACE(&codecs->payloads, payload, NULL);
+		AST_VECTOR_REPLACE(&codecs->payload_mapping_tx, payload, NULL);
 	}
 
 	if (instance && instance->engine && instance->engine->payload_set) {
@@ -894,8 +1449,8 @@ enum ast_media_type ast_rtp_codecs_get_s
 	struct ast_rtp_payload_type *type;
 
 	ast_rwlock_rdlock(&codecs->codecs_lock);
-	for (payload = 0; payload < AST_VECTOR_SIZE(&codecs->payloads); ++payload) {
-		type = AST_VECTOR_GET(&codecs->payloads, payload);
+	for (payload = 0; payload < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++payload) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_rx, payload);
 		if (type && type->asterisk_format) {
 			stream_type = ast_format_get_type(type->format);
 			break;
@@ -915,13 +1470,13 @@ struct ast_rtp_payload_type *ast_rtp_cod
 	}
 
 	ast_rwlock_rdlock(&codecs->codecs_lock);
-	if (payload < AST_VECTOR_SIZE(&codecs->payloads)) {
-		type = AST_VECTOR_GET(&codecs->payloads, payload);
+	if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_rx)) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_rx, payload);
 		ao2_bump(type);
 	}
 	ast_rwlock_unlock(&codecs->codecs_lock);
 
-	if (!type) {
+	if (!type && payload <= AST_RTP_PT_LAST_STATIC) {
 		ast_rwlock_rdlock(&static_RTP_PT_lock);
 		type = ao2_bump(static_RTP_PT[payload]);
 		ast_rwlock_unlock(&static_RTP_PT_lock);
@@ -946,15 +1501,18 @@ int ast_rtp_codecs_payload_replace_forma
 	type->format = format;
 	type->asterisk_format = 1;
 	type->payload = payload;
+	type->primary_mapping = 1;
 
 	ast_rwlock_wrlock(&codecs->codecs_lock);
-	if (payload < AST_VECTOR_SIZE(&codecs->payloads)) {
-		ao2_cleanup(AST_VECTOR_GET(&codecs->payloads, payload));
-	}
-	if (AST_VECTOR_REPLACE(&codecs->payloads, payload, type)) {
+	if (!payload_mapping_tx_is_present(codecs, type)) {
+		if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+			ao2_cleanup(AST_VECTOR_GET(&codecs->payload_mapping_tx, payload));
+		}
+		if (AST_VECTOR_REPLACE(&codecs->payload_mapping_tx, payload, type)) {
+			ao2_ref(type, -1);
+		}
+	} else {
 		ao2_ref(type, -1);
-		ast_rwlock_unlock(&codecs->codecs_lock);
-		return -1;
 	}
 	ast_rwlock_unlock(&codecs->codecs_lock);
 
@@ -971,8 +1529,8 @@ struct ast_format *ast_rtp_codecs_get_pa
 	}
 
 	ast_rwlock_rdlock(&codecs->codecs_lock);
-	if (payload < AST_VECTOR_SIZE(&codecs->payloads)) {
-		type = AST_VECTOR_GET(&codecs->payloads, payload);
+	if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_tx, payload);
 		if (type && type->asterisk_format) {
 			format = ao2_bump(type->format);
 		}
@@ -1006,16 +1564,17 @@ unsigned int ast_rtp_codecs_get_framing(
 
 void ast_rtp_codecs_payload_formats(struct ast_rtp_codecs *codecs, struct ast_format_cap *astformats, int *nonastformats)
 {
-	int i;
+	int idx;
 
 	ast_format_cap_remove_by_type(astformats, AST_MEDIA_TYPE_UNKNOWN);
 	*nonastformats = 0;
 
 	ast_rwlock_rdlock(&codecs->codecs_lock);
-	for (i = 0; i < AST_VECTOR_SIZE(&codecs->payloads); i++) {
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_tx); ++idx) {
 		struct ast_rtp_payload_type *type;
 
-		type = AST_VECTOR_GET(&codecs->payloads, i);
+		type = AST_VECTOR_GET(&codecs->payload_mapping_tx, idx);
 		if (!type) {
 			continue;
 		}
@@ -1026,7 +1585,6 @@ void ast_rtp_codecs_payload_formats(stru
 			*nonastformats |= type->rtp_code;
 		}
 	}
-
 	if (codecs->framing) {
 		ast_format_cap_set_framing(astformats, codecs->framing);
 	}
@@ -1034,59 +1592,381 @@ void ast_rtp_codecs_payload_formats(stru
 	ast_rwlock_unlock(&codecs->codecs_lock);
 }
 
-int ast_rtp_codecs_payload_code(struct ast_rtp_codecs *codecs, int asterisk_format, const struct ast_format *format, int code)
+/*!
+ * \internal
+ * \brief Find the static payload type mapping for the format.
+ * \since 14.0.0
+ *
+ * \param asterisk_format Non-zero if the given Asterisk format is present
+ * \param format Asterisk format to look for
+ * \param code The non-Asterisk format code to look for
+ *
+ * \note It is assumed that static_RTP_PT_lock is at least read locked before calling.
+ *
+ * \retval Numerical payload type
+ * \retval -1 if not found.
+ */
+static int find_static_payload_type(int asterisk_format, const struct ast_format *format, int code)
 {
-	struct ast_rtp_payload_type *type;
-	int i;
+	int idx;
 	int payload = -1;
 
-	ast_rwlock_rdlock(&codecs->codecs_lock);
-	for (i = 0; i < AST_VECTOR_SIZE(&codecs->payloads); i++) {
-		type = AST_VECTOR_GET(&codecs->payloads, i);
-		if (!type) {
+	if (!asterisk_format) {
+		for (idx = 0; idx < AST_RTP_MAX_PT; ++idx) {
+			if (static_RTP_PT[idx]
+				&& !static_RTP_PT[idx]->asterisk_format
+				&& static_RTP_PT[idx]->rtp_code == code) {
+				payload = idx;
+				break;
+			}
+		}
+	} else if (format) {
+		for (idx = 0; idx < AST_RTP_MAX_PT; ++idx) {
+			if (static_RTP_PT[idx]
+				&& static_RTP_PT[idx]->asterisk_format
+				&& ast_format_cmp(format, static_RTP_PT[idx]->format)
+					!= AST_FORMAT_CMP_NOT_EQUAL) {
+				payload = idx;
+				break;
+			}
+		}
+	}
+
+	return payload;
+}
+
+/*!
+ * \internal
+ * \brief Find the first unused payload type in a given range
+ *
+ * \param codecs The codec structure to look in
+ * \param start Starting index
+ * \param end Ending index
+ * \param ignore Skip these payloads types
+ *
+ * \note The static_RTP_PT_lock object must be locked before calling
+ *
+ * \retval Numerical payload type
+ * \retval -1 if not found.
+ */
+static int find_unused_payload_in_range(const struct ast_rtp_codecs *codecs,
+		int start, int end, struct ast_rtp_payload_type *ignore[])
+{
+	int x;
+
+	for (x = start; x < end; ++x) {
+		struct ast_rtp_payload_type *type;
+
+		if (ignore[x]) {
 			continue;
+		} else if (!codecs || x >= AST_VECTOR_SIZE(&codecs->payload_mapping_rx)) {
+			return x;
 		}
 
-		if ((asterisk_format && format && ast_format_cmp(format, type->format) == AST_FORMAT_CMP_EQUAL)
-			|| (!asterisk_format && type->rtp_code == code)) {
-			payload = i;
-			break;
+		type = AST_VECTOR_GET(&codecs->payload_mapping_rx, x);
+		if (!type) {
+			return x;
+		}
+	}
+	return -1;
+}
+
+/*!
+ * \internal
+ * \brief Find an unused payload type
+ *
+ * \param codecs Codecs structure to look in
+ *
+ * \note Both static_RTP_PT_lock and codecs (if given) must be at least
+ *       read locked before calling.
+ *
+ * \retval Numerical payload type
+ * \retval -1 if not found.
+ */
+static int find_unused_payload(const struct ast_rtp_codecs *codecs)
+{
+	int res;
+
+	/* find next available dynamic payload slot */
+	res = find_unused_payload_in_range(
+		codecs, AST_RTP_PT_FIRST_DYNAMIC, AST_RTP_MAX_PT, static_RTP_PT);
+	if (res != -1) {
+		return res;
+	}
+
+	if (ast_option_rtpusedynamic) {
+		/*
+		 * We're using default values for some dynamic types. So if an unused
+		 * slot was not found try again, but this time ignore the default
+		 * values declared for dynamic types (except for 101 and 121) .
+		 */
+		static struct ast_rtp_payload_type *ignore[AST_RTP_MAX_PT] = {0};
+
+		ignore[101] = static_RTP_PT[101];
+		ignore[121] = static_RTP_PT[121];
+
+		res = find_unused_payload_in_range(
+			codecs, AST_RTP_PT_FIRST_DYNAMIC, AST_RTP_MAX_PT, ignore);
+		if (res != -1) {
+			return res;
+		}
+	}
+
+	/* http://www.iana.org/assignments/rtp-parameters
+	 * RFC 3551, Section 3: "[...] applications which need to define more
+	 * than 32 dynamic payload types MAY bind codes below 96, in which case
+	 * it is RECOMMENDED that unassigned payload type numbers be used
+	 * first". Updated by RFC 5761, Section 4: "[...] values in the range
+	 * 64-95 MUST NOT be used [to avoid conflicts with RTCP]". Summaries:
+	 * https://tools.ietf.org/html/draft-roach-mmusic-unified-plan#section-3.2.1.2
+	 * https://tools.ietf.org/html/draft-wu-avtcore-dynamic-pt-usage#section-3
+	 */
+	res = find_unused_payload_in_range(
+		codecs, MAX(ast_option_rtpptdynamic, AST_RTP_PT_LAST_STATIC + 1),
+		AST_RTP_PT_LAST_REASSIGN, static_RTP_PT);
+	if (res != -1) {
+		return res;
+	}
+
+	/* Yet, reusing mappings below AST_RTP_PT_LAST_STATIC (35) is not supported
+	 * in Asterisk because when Compact Headers are activated, no rtpmap is
+	 * send for those below 35. If you want to use 35 and below
+	 * A) do not use Compact Headers,
+	 * B) remove that code in chan_sip/res_pjsip, or
+	 * C) add a flag that this RTP Payload Type got reassigned dynamically
+	 *    and requires a rtpmap even with Compact Headers enabled.
+	 */
+	res = find_unused_payload_in_range(
+		codecs, MAX(ast_option_rtpptdynamic, 20),
+		AST_RTP_PT_LAST_STATIC + 1, static_RTP_PT);
+	if (res != -1) {
+		return res;
+	}
+
+	return find_unused_payload_in_range(
+		codecs, MAX(ast_option_rtpptdynamic, 0),
+		20, static_RTP_PT);
+}
+
+/*!
+ * \internal
+ * \brief Find the oldest non-primary dynamic rx payload type.
+ * \since 14.0.0
+ *
+ * \param codecs Codecs structure to look in
+ *
+ * \note It is assumed that codecs is at least read locked before calling.
+ *
+ * \retval Numerical payload type
+ * \retval -1 if not found.
+ */
+static int rtp_codecs_find_non_primary_dynamic_rx(struct ast_rtp_codecs *codecs)
+{
+	struct ast_rtp_payload_type *type;
+	struct timeval oldest;
+	int idx;
+	int payload = -1;
+
+	idx = AST_RTP_PT_FIRST_DYNAMIC;
+	for (; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++idx) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_rx, idx);
+		if (type
+			&& !type->primary_mapping
+			&& (payload == -1
+				|| ast_tvdiff_ms(type->when_retired, oldest) < 0)) {
+			oldest = type->when_retired;
+			payload = idx;
+		}
+	}
+	return payload;
+}
+
+/*!
+ * \internal
+ * \brief Assign a payload type for the rx mapping.
+ * \since 14.0.0
+ *
+ * \param codecs Codecs structure to look in
+ * \param asterisk_format Non-zero if the given Asterisk format is present
+ * \param format Asterisk format to look for
+ * \param code The format to look for
+ * \param explicit Require the provided code to be explicitly used
+ *
+ * \note It is assumed that static_RTP_PT_lock is at least read locked before calling.
+ *
+ * \retval Numerical payload type
+ * \retval -1 if could not assign.
+ */
+static int rtp_codecs_assign_payload_code_rx(struct ast_rtp_codecs *codecs, int asterisk_format, struct ast_format *format, int code, int explicit)
+{
+	int payload = code;
+	struct ast_rtp_payload_type *new_type;
+
+	if (!explicit) {
+		payload = find_static_payload_type(asterisk_format, format, code);
+
+		if (payload < 0 && (!asterisk_format || !ast_option_rtpusedynamic)) {
+			return payload;
 		}
 	}
+
+	new_type = rtp_payload_type_alloc(format, payload, code, 1);
+	if (!new_type) {
+		return -1;
+	}
+
+	ast_rwlock_wrlock(&codecs->codecs_lock);
+	if (payload > -1 && (payload < AST_RTP_PT_FIRST_DYNAMIC
+		|| AST_VECTOR_SIZE(&codecs->payload_mapping_rx) <= payload
+		|| !AST_VECTOR_GET(&codecs->payload_mapping_rx, payload))) {
+		/*
+		 * The payload type is a static assignment
+		 * or our default dynamic position is available.
+		 */
+		rtp_codecs_payload_replace_rx(codecs, payload, new_type);
+	} else if (!explicit && (-1 < (payload = find_unused_payload(codecs))
+		|| -1 < (payload = rtp_codecs_find_non_primary_dynamic_rx(codecs)))) {
+		/*
+		 * We found the first available empty dynamic position
+		 * or we found a mapping that should no longer be
+		 * actively used.
+		 */
+		new_type->payload = payload;
+		rtp_codecs_payload_replace_rx(codecs, payload, new_type);
+	} else if (explicit) {
+		/*
+		* They explicitly requested this payload number be used but it couldn't be
+		*/
+		payload = -1;
+	} else {
+		/*
+		 * There are no empty or non-primary dynamic positions
+		 * left.  Sadness.
+		 *
+		 * I don't think this is really possible.
+		 */
+		ast_log(LOG_WARNING, "No dynamic RTP payload type values available "
+			"for %s - %d!\n", format ? ast_format_get_name(format) : "", code);
+	}
 	ast_rwlock_unlock(&codecs->codecs_lock);
 
+	ao2_ref(new_type, -1);
+
+	return payload;
+}
+
+int ast_rtp_codecs_payload_code(struct ast_rtp_codecs *codecs, int asterisk_format, struct ast_format *format, int code)
+{
+	struct ast_rtp_payload_type *type;
+	int idx;
+	int payload = -1;
+
+	ast_rwlock_rdlock(&static_RTP_PT_lock);
+	if (!asterisk_format) {
+		ast_rwlock_rdlock(&codecs->codecs_lock);
+		for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++idx) {
+			type = AST_VECTOR_GET(&codecs->payload_mapping_rx, idx);
+			if (!type) {
+				continue;
+			}
+
+			if (!type->asterisk_format
+				&& type->primary_mapping
+				&& type->rtp_code == code) {
+				payload = idx;
+				break;
+			}
+		}
+		ast_rwlock_unlock(&codecs->codecs_lock);
+	} else if (format) {
+		ast_rwlock_rdlock(&codecs->codecs_lock);
+		for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_rx); ++idx) {
+			type = AST_VECTOR_GET(&codecs->payload_mapping_rx, idx);
+			if (!type) {
+				continue;
+			}
+
+			if (type->asterisk_format
+				&& type->primary_mapping
+				&& ast_format_cmp(format, type->format) == AST_FORMAT_CMP_EQUAL) {
+				payload = idx;
+				break;
+			}
+		}
+		ast_rwlock_unlock(&codecs->codecs_lock);
+	}
+
 	if (payload < 0) {
-		ast_rwlock_rdlock(&static_RTP_PT_lock);
-		for (i = 0; i < AST_RTP_MAX_PT; i++) {
-			if (!static_RTP_PT[i]) {
+		payload = rtp_codecs_assign_payload_code_rx(codecs, asterisk_format, format,
+			code, 0);
+	}
+	ast_rwlock_unlock(&static_RTP_PT_lock);
+
+	return payload;
+}
+
+int ast_rtp_codecs_payload_set_rx(struct ast_rtp_codecs *codecs, int code, struct ast_format *format)
+{
+	return rtp_codecs_assign_payload_code_rx(codecs, 1, format, code, 1);
+}
+
+int ast_rtp_codecs_payload_code_tx(struct ast_rtp_codecs *codecs, int asterisk_format, const struct ast_format *format, int code)
+{
+	struct ast_rtp_payload_type *type;
+	int idx;
+	int payload = -1;
+
+	if (!asterisk_format) {
+		ast_rwlock_rdlock(&codecs->codecs_lock);
+		for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_tx); ++idx) {
+			type = AST_VECTOR_GET(&codecs->payload_mapping_tx, idx);
+			if (!type) {
 				continue;
 			}
-			if (static_RTP_PT[i]->asterisk_format && asterisk_format && format &&
-				(ast_format_cmp(format, static_RTP_PT[i]->format) != AST_FORMAT_CMP_NOT_EQUAL)) {
-				payload = i;
+
+			if (!type->asterisk_format
+				&& type->rtp_code == code) {
+				payload = idx;
 				break;
-			} else if (!static_RTP_PT[i]->asterisk_format && !asterisk_format &&
-				(static_RTP_PT[i]->rtp_code == code)) {
-				payload = i;
+			}
+		}
+		ast_rwlock_unlock(&codecs->codecs_lock);
+	} else if (format) {
+		ast_rwlock_rdlock(&codecs->codecs_lock);
+		for (idx = 0; idx < AST_VECTOR_SIZE(&codecs->payload_mapping_tx); ++idx) {
+			type = AST_VECTOR_GET(&codecs->payload_mapping_tx, idx);
+			if (!type) {
+				continue;
+			}
+
+			if (type->asterisk_format
+				&& ast_format_cmp(format, type->format) == AST_FORMAT_CMP_EQUAL) {
+				payload = idx;
 				break;
 			}
 		}
+		ast_rwlock_unlock(&codecs->codecs_lock);
+	}
+
+	if (payload < 0) {
+		ast_rwlock_rdlock(&static_RTP_PT_lock);
+		payload = find_static_payload_type(asterisk_format, format, code);
 		ast_rwlock_unlock(&static_RTP_PT_lock);
 	}
 
 	return payload;
 }
 
-int ast_rtp_codecs_find_payload_code(struct ast_rtp_codecs *codecs, int code)
+int ast_rtp_codecs_find_payload_code(struct ast_rtp_codecs *codecs, int payload)
 {
 	struct ast_rtp_payload_type *type;
 	int res = -1;
 
 	ast_rwlock_rdlock(&codecs->codecs_lock);
-	if (code < AST_VECTOR_SIZE(&codecs->payloads)) {
-		type = AST_VECTOR_GET(&codecs->payloads, code);
+	if (payload < AST_VECTOR_SIZE(&codecs->payload_mapping_tx)) {
+		type = AST_VECTOR_GET(&codecs->payload_mapping_tx, payload);
 		if (type) {
-			res = type->payload;
+			res = payload;
 		}
 	}
 	ast_rwlock_unlock(&codecs->codecs_lock);
@@ -1094,7 +1974,8 @@ int ast_rtp_codecs_find_payload_code(str
 	return res;
 }
 
-const char *ast_rtp_lookup_mime_subtype2(const int asterisk_format, struct ast_format *format, int code, enum ast_rtp_options options)
+const char *ast_rtp_lookup_mime_subtype2(const int asterisk_format,
+	const struct ast_format *format, int code, enum ast_rtp_options options)
 {
 	int i;
 	const char *res = "";
@@ -1123,7 +2004,8 @@ const char *ast_rtp_lookup_mime_subtype2
 	return res;
 }
 
-unsigned int ast_rtp_lookup_sample_rate2(int asterisk_format, struct ast_format *format, int code)
+unsigned int ast_rtp_lookup_sample_rate2(int asterisk_format,
+	const struct ast_format *format, int code)
 {
 	unsigned int i;
 	unsigned int res = 0;
@@ -1405,13 +2287,13 @@ void ast_rtp_instance_early_bridge_make_
 		goto done;
 	}
 
-	ast_rtp_codecs_payloads_copy(&instance_src->codecs, &instance_dst->codecs, instance_dst);
+	ast_rtp_codecs_payloads_xover(&instance_src->codecs, &instance_dst->codecs, instance_dst);
 
 	if (vinstance_dst && vinstance_src) {
-		ast_rtp_codecs_payloads_copy(&vinstance_src->codecs, &vinstance_dst->codecs, vinstance_dst);
+		ast_rtp_codecs_payloads_xover(&vinstance_src->codecs, &vinstance_dst->codecs, vinstance_dst);
 	}
 	if (tinstance_dst && tinstance_src) {
-		ast_rtp_codecs_payloads_copy(&tinstance_src->codecs, &tinstance_dst->codecs, tinstance_dst);
+		ast_rtp_codecs_payloads_xover(&tinstance_src->codecs, &tinstance_dst->codecs, tinstance_dst);
 	}
 
 	if (glue_dst->update_peer(c_dst, instance_src, vinstance_src, tinstance_src, cap_src, 0)) {
@@ -1850,7 +2732,7 @@ int ast_rtp_instance_add_srtp_policy(str
 
 	if (!*srtp) {
 		res = res_srtp->create(srtp, instance, remote_policy);
-	} else {
+	} else if (remote_policy) {
 		res = res_srtp->replace(srtp, instance, remote_policy);
 	}
 	if (!res) {
@@ -2130,6 +3012,8 @@ int ast_rtp_dtls_cfg_parse(struct ast_rt
 		if (sscanf(value, "%30u", &dtls_cfg->rekey) != 1) {
 			return -1;
 		}
+	} else if (!strcasecmp(name, "dtlsautogeneratecert")) {
+		dtls_cfg->ephemeral_cert = ast_true(value) ? 1 : 0;
 	} else if (!strcasecmp(name, "dtlscertfile")) {
 		if (!ast_strlen_zero(value) && !ast_file_is_readable(value)) {
 			ast_log(LOG_ERROR, "%s file %s does not exist or is not readable\n", name, value);
@@ -2182,6 +3066,25 @@ int ast_rtp_dtls_cfg_parse(struct ast_rt
 	return 0;
 }
 
+int ast_rtp_dtls_cfg_validate(struct ast_rtp_dtls_cfg *dtls_cfg)
+{
+	if (dtls_cfg->ephemeral_cert) {
+		if (!ast_strlen_zero(dtls_cfg->certfile)) {
+			ast_log(LOG_ERROR, "You cannot request automatically generated certificates"
+				" (dtls_auto_generate_cert) and also specify a certificate file"
+				" (dtls_cert_file) at the same time\n");
+			return -1;
+		} else if (!ast_strlen_zero(dtls_cfg->pvtfile)
+				  || !ast_strlen_zero(dtls_cfg->cafile)
+				  || !ast_strlen_zero(dtls_cfg->capath)) {
+			ast_log(LOG_NOTICE, "dtls_pvt_file, dtls_cafile, and dtls_ca_path are"
+				" ignored when dtls_auto_generate_cert is enabled\n");
+		}
+	}
+
+	return 0;
+}
+
 void ast_rtp_dtls_cfg_copy(const struct ast_rtp_dtls_cfg *src_cfg, struct ast_rtp_dtls_cfg *dst_cfg)
 {
 	ast_rtp_dtls_cfg_free(dst_cfg);         /* Prevent a double-call leaking memory via ast_strdup */
@@ -2191,6 +3094,7 @@ void ast_rtp_dtls_cfg_copy(const struct
 	dst_cfg->rekey = src_cfg->rekey;
 	dst_cfg->suite = src_cfg->suite;
 	dst_cfg->hash = src_cfg->hash;
+	dst_cfg->ephemeral_cert = src_cfg->ephemeral_cert;
 	dst_cfg->certfile = ast_strdup(src_cfg->certfile);
 	dst_cfg->pvtfile = ast_strdup(src_cfg->pvtfile);
 	dst_cfg->cipher = ast_strdup(src_cfg->cipher);
@@ -2251,94 +3155,48 @@ static void set_next_mime_type(struct as
 	ast_rwlock_unlock(&mime_types_lock);
 }
 
-static void add_static_payload(int map, struct ast_format *format, int rtp_code)
+static void add_static_payload(int payload, struct ast_format *format, int rtp_code)
 {
-	int x;
 	struct ast_rtp_payload_type *type;
 
 	/*
 	 * ARRAY_LEN's result is cast to an int so 'map' is not autocast to a size_t,
 	 * which if negative would cause an assertion.
 	 */
-	ast_assert(map < (int)ARRAY_LEN(static_RTP_PT));
+	ast_assert(payload < (int)ARRAY_LEN(static_RTP_PT));
+
+	if (ast_option_rtpusedynamic && payload < 0) {
+		/*
+		 * We're going to build dynamic payloads dynamically. An RTP code is
+		 * required otherwise one will be dynamically allocated per instance.
+		 */
+		return;
+	}
 
+	/*
+	 * Either the given payload is truly a static type, or Asterisk is
+	 * globally storing the dynamic payloads in the static_RTP_PT object.
+	 */
 	ast_rwlock_wrlock(&static_RTP_PT_lock);
-	if (map < 0) {
-		/* find next available dynamic payload slot */
-		for (x = AST_RTP_PT_FIRST_DYNAMIC; x < AST_RTP_MAX_PT; ++x) {
-			if (!static_RTP_PT[x]) {
-				map = x;
-				break;
-			}
-		}
 
-		/* http://www.iana.org/assignments/rtp-parameters
-		 * RFC 3551, Section 3: "[...] applications which need to define more
-		 * than 32 dynamic payload types MAY bind codes below 96, in which case
-		 * it is RECOMMENDED that unassigned payload type numbers be used
-		 * first". Updated by RFC 5761, Section 4: "[...] values in the range
-		 * 64-95 MUST NOT be used [to avoid conflicts with RTCP]". Summaries:
-		 * https://tools.ietf.org/html/draft-roach-mmusic-unified-plan#section-3.2.1.2
-		 * https://tools.ietf.org/html/draft-wu-avtcore-dynamic-pt-usage#section-3
-		 */
-		if (map < 0) {
-			for (x = MAX(ast_option_rtpptdynamic, 35); x <= AST_RTP_PT_LAST_REASSIGN; ++x) {
-				if (!static_RTP_PT[x]) {
-					map = x;
-					break;
-				}
-			}
-		}
-		/* Yet, reusing mappings below 35 is not supported in Asterisk because
-		 * when Compact Headers are activated, no rtpmap is send for those below
-		 * 35. If you want to use 35 and below
-		 * A) do not use Compact Headers,
-		 * B) remove that code in chan_sip/res_pjsip, or
-		 * C) add a flag that this RTP Payload Type got reassigned dynamically
-		 *    and requires a rtpmap even with Compact Headers enabled.
+	if (payload < 0) {
+		/*
+		 * This is a dynamic payload that will be stored globally,
+		 * so find the next available empty slot.
 		 */
-		if (map < 0) {
-			for (x = MAX(ast_option_rtpptdynamic, 20); x < 35; ++x) {
-				if (!static_RTP_PT[x]) {
-					map = x;
-					break;
-				}
-			}
-		}
-		if (map < 0) {
-			for (x = MAX(ast_option_rtpptdynamic, 0); x < 20; ++x) {
-				if (!static_RTP_PT[x]) {
-					map = x;
-					break;
-				}
-			}
-		}
-
-		if (map < 0) {
-			if (format) {
-				ast_log(LOG_WARNING, "No Dynamic RTP mapping available for format %s\n",
-					ast_format_get_name(format));
-			} else {
-				ast_log(LOG_WARNING, "No Dynamic RTP mapping available for RTP code %d\n",
-					rtp_code);
-			}
+		payload = find_unused_payload(NULL);
+		if (payload < 0) {
+			ast_log(LOG_WARNING, "No dynamic RTP payload type values available "
+				"for %s - %d!\n", format ? ast_format_get_name(format) : "", rtp_code);
 			ast_rwlock_unlock(&static_RTP_PT_lock);
 			return;
 		}
 	}
 
-	type = ast_rtp_engine_alloc_payload_type();
+	type = rtp_payload_type_alloc(format, payload, rtp_code, 1);
 	if (type) {
-		if (format) {
-			ao2_ref(format, +1);
-			type->format = format;
-			type->asterisk_format = 1;
-		} else {
-			type->rtp_code = rtp_code;
-		}
-		type->payload = map;
-		ao2_cleanup(static_RTP_PT[map]);
-		static_RTP_PT[map] = type;
+		ao2_cleanup(static_RTP_PT[payload]);
+		static_RTP_PT[payload] = type;
 	}
 	ast_rwlock_unlock(&static_RTP_PT_lock);
 }
@@ -2521,8 +3379,8 @@ static struct ast_json *rtcp_report_to_j
 		char str_lsr[32];
 
 		snprintf(str_lsr, sizeof(str_lsr), "%u", payload->report->report_block[i]->lsr);
-		json_report_block = ast_json_pack("{s: i, s: i, s: i, s: i, s: i, s: s, s: i}",
-			"source_ssrc", payload->report->report_block[i]->source_ssrc,
+		json_report_block = ast_json_pack("{s: I, s: i, s: i, s: i, s: i, s: s, s: i}",
+			"source_ssrc", (ast_json_int_t)payload->report->report_block[i]->source_ssrc,
 			"fraction_lost", payload->report->report_block[i]->lost_count.fraction,
 			"packets_lost", payload->report->report_block[i]->lost_count.packets,
 			"highest_seq_no", payload->report->report_block[i]->highest_seq_no,
@@ -2554,8 +3412,8 @@ static struct ast_json *rtcp_report_to_j
 		}
 	}
 
-	json_rtcp_report = ast_json_pack("{s: i, s: i, s: i, s: o, s: o}",
-		"ssrc", payload->report->ssrc,
+	json_rtcp_report = ast_json_pack("{s: I, s: i, s: i, s: o, s: o}",
+		"ssrc", (ast_json_int_t)payload->report->ssrc,
 		"type", payload->report->type,
 		"report_count", payload->report->reception_report_count,
 		"sender_information", json_rtcp_sender_info ?: ast_json_null(),
@@ -2572,10 +3430,10 @@ static struct ast_json *rtcp_report_to_j
 		}
 	}
 
-	return ast_json_pack("{s: o, s: o, s: o}",
-		"channel", payload->snapshot ? json_channel : ast_json_null(),
+	return ast_json_pack("{s: o?, s: o, s: O?}",
+		"channel", json_channel,
 		"rtcp_report", json_rtcp_report,
-		"blob", ast_json_deep_copy(payload->blob) ?: ast_json_null());
+		"blob", payload->blob);
 }
 
 static void rtp_rtcp_report_dtor(void *obj)
@@ -2701,6 +3559,13 @@ int ast_rtp_engine_init(void)
 	set_next_mime_type(ast_format_slin, 0, "audio", "L16", 8000);
 	set_next_mime_type(ast_format_slin16, 0, "audio", "L16", 16000);
 	set_next_mime_type(ast_format_slin16, 0, "audio", "L16-256", 16000);
+	set_next_mime_type(ast_format_slin12, 0, "audio", "L16", 12000);
+	set_next_mime_type(ast_format_slin24, 0, "audio", "L16", 24000);
+	set_next_mime_type(ast_format_slin32, 0, "audio", "L16", 32000);
+	set_next_mime_type(ast_format_slin44, 0, "audio", "L16", 44000);
+	set_next_mime_type(ast_format_slin48, 0, "audio", "L16", 48000);
+	set_next_mime_type(ast_format_slin96, 0, "audio", "L16", 96000);
+	set_next_mime_type(ast_format_slin192, 0, "audio", "L16", 192000);
 	set_next_mime_type(ast_format_lpc10, 0, "audio", "LPC", 8000);
 	set_next_mime_type(ast_format_g729, 0, "audio", "G729", 8000);
 	set_next_mime_type(ast_format_g729, 0, "audio", "G729A", 8000);
@@ -2754,29 +3619,43 @@ int ast_rtp_engine_init(void)
 	add_static_payload(26, ast_format_jpeg, 0);
 	add_static_payload(31, ast_format_h261, 0);
 	add_static_payload(34, ast_format_h263, 0);
+
+	/*
+	 * Dynamic payload types - Even when dynamically assigning them we'll fall
+	 * back to using the statically declared values as the default number.
+	 */
+	add_static_payload(96, ast_format_slin192, 0);
 	add_static_payload(97, ast_format_ilbc, 0);
 
 	add_static_payload(99, ast_format_h264, 0);
+	add_static_payload(100, ast_format_vp8, 0);
 	add_static_payload(101, NULL, AST_RTP_DTMF);
 	add_static_payload(102, ast_format_siren7, 0);
 	add_static_payload(103, ast_format_h263p, 0);
 	add_static_payload(104, ast_format_mp4, 0);
 	add_static_payload(105, ast_format_t140_red, 0);   /* Real time text chat (with redundancy encoding) */
 	add_static_payload(106, ast_format_t140, 0);     /* Real time text chat */
+	add_static_payload(107, ast_format_opus, 0);
 	add_static_payload(108, ast_format_vp9, 0);
 
 	add_static_payload(110, ast_format_speex, 0);
 	add_static_payload(111, ast_format_g726, 0);
 	add_static_payload(112, ast_format_g726_aal2, 0);
+
 	add_static_payload(115, ast_format_siren14, 0);
 	add_static_payload(116, ast_format_g719, 0);
 	add_static_payload(117, ast_format_speex16, 0);
 	add_static_payload(118, ast_format_slin16, 0); /* 16 Khz signed linear */
 	add_static_payload(119, ast_format_speex32, 0);
+
 	add_static_payload(121, NULL, AST_RTP_CISCO_DTMF);   /* Must be type 121 */
-	/* Opus and VP8 */
-	add_static_payload(100, ast_format_vp8, 0);
-	add_static_payload(107, ast_format_opus, 0);
+	add_static_payload(122, ast_format_slin12, 0);
+	add_static_payload(123, ast_format_slin24, 0);
+	add_static_payload(124, ast_format_slin32, 0);
+	add_static_payload(125, ast_format_slin44, 0);
+	add_static_payload(126, ast_format_slin48, 0);
+	add_static_payload(127, ast_format_slin96, 0);
+	/* payload types above 127 are not valid */
 
 	return 0;
 }
@@ -2800,3 +3679,64 @@ void ast_rtp_instance_set_last_rx(struct
 {
 	rtp->last_rx = time;
 }
+
+unsigned int ast_rtp_instance_get_ssrc(struct ast_rtp_instance *rtp)
+{
+	unsigned int ssrc = 0;
+
+	ao2_lock(rtp);
+	if (rtp->engine->ssrc_get) {
+		ssrc = rtp->engine->ssrc_get(rtp);
+	}
+	ao2_unlock(rtp);
+
+	return ssrc;
+}
+
+const char *ast_rtp_instance_get_cname(struct ast_rtp_instance *rtp)
+{
+	const char *cname = "";
+
+	ao2_lock(rtp);
+	if (rtp->engine->cname_get) {
+		cname = rtp->engine->cname_get(rtp);
+	}
+	ao2_unlock(rtp);
+
+	return cname;
+}
+
+int ast_rtp_instance_bundle(struct ast_rtp_instance *child, struct ast_rtp_instance *parent)
+{
+	int res = -1;
+
+	if (parent && (child->engine != parent->engine)) {
+		return -1;
+	}
+
+	ao2_lock(child);
+	if (child->engine->bundle) {
+		res = child->engine->bundle(child, parent);
+	}
+	ao2_unlock(child);
+
+	return res;
+}
+
+void ast_rtp_instance_set_remote_ssrc(struct ast_rtp_instance *rtp, unsigned int ssrc)
+{
+	ao2_lock(rtp);
+	if (rtp->engine->set_remote_ssrc) {
+		rtp->engine->set_remote_ssrc(rtp, ssrc);
+	}
+	ao2_unlock(rtp);
+}
+
+void ast_rtp_instance_set_stream_num(struct ast_rtp_instance *rtp, int stream_num)
+{
+	ao2_lock(rtp);
+	if (rtp->engine->set_stream_num) {
+		rtp->engine->set_stream_num(rtp, stream_num);
+	}
+	ao2_unlock(rtp);
+}
diff -pruN 1:13.23.1~dfsg-2/main/say.c 1:16.1.1~dfsg-1/main/say.c
--- 1:13.23.1~dfsg-2/main/say.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/say.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,8 +37,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <netinet/in.h>
 #include <time.h>
 #include <ctype.h>
@@ -317,6 +315,7 @@ static int say_digit_str_full(struct ast
       \arg \b es    - Spanish, Mexican
       \arg \b fr    - French
       \arg \b he    - Hebrew
+      \arg \b is    - Icelandic
       \arg \b it    - Italian
       \arg \b nl    - Dutch
       \arg \b no    - Norwegian
@@ -373,6 +372,7 @@ static int ast_say_number_full_en_GB(str
 static int ast_say_number_full_es(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_number_full_fr(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_number_full_he(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
+static int ast_say_number_full_is(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_number_full_it(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
 static int ast_say_number_full_nl(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
 static int ast_say_number_full_no(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
@@ -394,6 +394,7 @@ static int ast_say_enumeration_full_en(s
 static int ast_say_enumeration_full_da(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_enumeration_full_de(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_enumeration_full_he(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
+static int ast_say_enumeration_full_is(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd);
 static int ast_say_enumeration_full_vi(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd);
 
 /* Forward declarations of ast_say_date, ast_say_datetime and ast_say_time functions */
@@ -409,6 +410,7 @@ static int ast_say_date_ka(struct ast_ch
 static int ast_say_date_hu(struct ast_channel *chan, time_t t, const char *ints, const char *lang);
 static int ast_say_date_th(struct ast_channel *chan, time_t t, const char *ints, const char *lang);
 static int ast_say_date_he(struct ast_channel *chan, time_t t, const char *ints, const char *lang);
+static int ast_say_date_is(struct ast_channel *chan, time_t t, const char *ints, const char *lang);
 
 static int ast_say_date_with_format_en(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_da(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
@@ -416,6 +418,7 @@ static int ast_say_date_with_format_de(s
 static int ast_say_date_with_format_es(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_he(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_fr(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
+static int ast_say_date_with_format_is(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_it(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_nl(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
 static int ast_say_date_with_format_pl(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone);
@@ -511,6 +514,8 @@ static int say_number_full(struct ast_ch
 	   return ast_say_number_full_he(chan, num, ints, language, options, audiofd, ctrlfd);
 	} else if (!strncasecmp(language, "hu", 2)) { /* Hungarian syntax */
 		return ast_say_number_full_hu(chan, num, ints, language, audiofd, ctrlfd);
+	} else if (!strncasecmp(language, "is", 2)) { /* Icelandic syntax */
+		return ast_say_number_full_is(chan, num, ints, language, options, audiofd, ctrlfd);
 	} else if (!strncasecmp(language, "it", 2)) { /* Italian syntax */
 	   return ast_say_number_full_it(chan, num, ints, language, audiofd, ctrlfd);
 	} else if (!strncasecmp(language, "ka", 2)) { /* Georgian syntax */
@@ -1518,6 +1523,129 @@ static int ast_say_number_full_hu(struct
 	return res;
 }
 
+/*! \brief  ast_say_number_full_is: Icelandic syntax */
+/* New files:
+ In addition to American English, the following sounds are required:  "hundreds", "millions", "1kvk", "1hk", "2kvk", "2hk", "3kvk", "3hk", "4kvk", "4hk"
+ */
+static int ast_say_number_full_is(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd)
+{
+	int res = 0;
+	int playh = 0;
+	int playa = 0;
+	int cn = 1;		/* 1 = masc; 2 = fem; 3 = neut */
+	char fn[256] = "";
+
+	if (!num)
+		return ast_say_digits_full(chan, 0, ints, language, audiofd, ctrlfd);
+
+	if (options && !strncasecmp(options, "f", 1)) cn = 2;
+	if (options && !strncasecmp(options, "c", 1)) cn = 3;
+	/* It seems that sometimes people are using c and sometimes n. */
+	if (options && !strncasecmp(options, "n", 1)) cn = 3;
+
+	while (!res && (num || playh || playa )) {
+		if (num < 0) {
+			ast_copy_string(fn, "digits/minus", sizeof(fn));
+			if ( num > INT_MIN ) {
+				num = -num;
+			} else {
+				num = 0;
+			}
+		} else if (playh) {
+			if (playh > 1)
+				ast_copy_string(fn, "digits/hundreds", sizeof(fn));
+			else
+				ast_copy_string(fn, "digits/hundred", sizeof(fn));
+			playh = 0;
+		} else if (playa) {
+			ast_copy_string(fn, "digits/and", sizeof(fn));
+			playa = 0;
+		} else if (num < 5 && cn == 2) {
+			snprintf(fn, sizeof(fn), "digits/%dkvk", num);
+			num = 0;
+		} else if (num < 5 && cn == 3) {
+			snprintf(fn, sizeof(fn), "digits/%dhk", num);
+			num = 0;
+		} else if (num < 20) {
+			snprintf(fn, sizeof(fn), "digits/%d", num);
+			num = 0;
+		} else if (num < 100) {
+			snprintf(fn, sizeof(fn), "digits/%d", (num /10) * 10);
+			num %= 10;
+			if (num)
+				playa++;
+		} else if (num < 1000) {
+			int hundreds = num / 100;
+			/* The number prepending hundreds are in neutral */
+			if (hundreds < 5)
+				snprintf(fn, sizeof(fn), "digits/%dhk", hundreds);
+			else
+				snprintf(fn, sizeof(fn), "digits/%d", (num / 100));
+
+			playh = hundreds;
+			num -= 100 * hundreds;
+			if (num && num < 20)
+				playa++;
+			/* The 'and' moves forward on even tens. */
+			if (num && (num % 10) == 0)
+				playa++;
+		} else if (num < 1000000) {
+			res = ast_say_number_full_is(chan, num / 1000, ints, language, "n", audiofd, ctrlfd);
+			/* Play 'and' if it's an even hundred. */
+			if ((num % 100) == 0 && (num % 1000 != 0)) {
+				playa++;
+			}
+			if (res)
+				return res;
+			ast_copy_string(fn, "digits/thousand", sizeof(fn));
+			num %= 1000;
+			if (num && (num < 20 || (num % 10 == 0)))
+				playa++;
+		} else if (num < 1000000000) {
+			int millions = num / 1000000;
+			/* The number of millions is feminine */
+			res = ast_say_number_full_is(chan, millions, ints, language, "f", audiofd, ctrlfd);
+			if (res)
+				return res;
+			if (millions > 1)
+				ast_copy_string(fn, "digits/millions", sizeof(fn));
+			else
+				ast_copy_string(fn, "digits/million", sizeof(fn));
+			num %= 1000000;
+			if (num && num < 100)
+				playa++;
+		} else if (num < INT_MAX) {
+			int milliards = num / 1000000000;
+			/* The number of milliards is masculine */
+			res = ast_say_number_full_is(chan, milliards, ints, language, "m", audiofd, ctrlfd);
+			if (res)
+				return res;
+			if (milliards > 1)
+				ast_copy_string(fn, "digits/milliards", sizeof(fn));
+			else
+				ast_copy_string(fn, "digits/milliard", sizeof(fn));
+			num %= 1000000000;
+			if (num && num < 100)
+				playa++;
+		} else {
+				ast_debug(1, "Number '%d' is too big for me\n", num);
+				res = -1;
+		}
+
+		if (!res) {
+			if (!ast_streamfile(chan, fn, language)) {
+				if ((audiofd > -1) && (ctrlfd > -1))
+					res = ast_waitstream_full(chan, ints, audiofd, ctrlfd);
+				else
+					res = ast_waitstream(chan, ints);
+			}
+			ast_stopstream(chan);
+		}
+	}
+	return res;
+}
+
+
 /*! \brief  ast_say_number_full_it:  Italian */
 static int ast_say_number_full_it(struct ast_channel *chan, int num, const char *ints, const char *language, int audiofd, int ctrlfd)
 {
@@ -2789,6 +2917,8 @@ static int say_enumeration_full(struct a
 	   return ast_say_enumeration_full_de(chan, num, ints, language, options, audiofd, ctrlfd);
 	} else if (!strncasecmp(language, "he", 2)) { /* Hebrew syntax */
 		return ast_say_enumeration_full_he(chan, num, ints, language, options, audiofd, ctrlfd);
+	} else if (!strncasecmp(language, "is", 2)) { /* Icelandic syntax */
+		return ast_say_enumeration_full_is(chan, num, ints, language, options, audiofd, ctrlfd);
 	} else if (!strncasecmp(language, "vi", 2)) { /* Vietnamese syntax */
 		return ast_say_enumeration_full_vi(chan, num, ints, language, audiofd, ctrlfd);
 	}
@@ -3329,6 +3459,182 @@ static int ast_say_enumeration_full_he(s
 	return res;
 }
 
+/*! \brief  ast_say_enumeration_full_is: Icelandic syntax */
+static int ast_say_enumeration_full_is(struct ast_channel *chan, int num, const char *ints, const char *language, const char *options, int audiofd, int ctrlfd)
+{
+	/* options can be: '' or 'm' male gender; 'f' female gender; 'n' neuter gender; 'p' plural */
+	int res = 0, t = 0;
+	char fn[256] = "", fna[256] = "";
+	char *gender;
+
+	if (options && !strncasecmp(options, "f", 1)) {
+		gender = "F";
+	} else if (options && !strncasecmp(options, "n", 1)) {
+		gender = "N";
+	} else {
+		gender = "";
+	}
+
+	if (!num)
+		return ast_say_digits_full(chan, 0, ints, language, audiofd, ctrlfd);
+
+	while (!res && num) {
+		if (num < 0) {
+			ast_copy_string(fn, "digits/minus", sizeof(fn)); /* kind of senseless for enumerations, but our best effort for error checking */
+			if ( num > INT_MIN ) {
+				num = -num;
+			} else {
+				num = 0;
+			}
+		} else if (num < 100 && t) {
+			ast_copy_string(fn, "digits/and", sizeof(fn));
+			t = 0;
+		} else if (num < 20) {
+			snprintf(fn, sizeof(fn), "digits/h-%d%s", num, gender);
+			num = 0;
+		} else if (num < 100) {
+			int ones = num % 10;
+			if (ones) {
+				int tens = num - ones;
+				snprintf(fn, sizeof(fn), "digits/h-%d%s", tens, gender);
+				num = ones;
+				t++;
+			}
+			else if (t) {
+				snprintf(fn, sizeof(fn), "digits/and");
+				t = 0;
+			}
+			else {
+				snprintf(fn, sizeof(fn), "digits/h-%d%s", num, gender);
+				num = 0;
+			}
+
+		} else if (num == 100 && t == 0) {
+			snprintf(fn, sizeof(fn), "digits/h-hundred%s", gender);
+			num = 0;
+		} else if (num < 1000) {
+			int hundreds = num / 100;
+			num = num % 100;
+			if (hundreds == 1) {
+				ast_copy_string(fn, "digits/1hk", sizeof(fn));
+			} else {
+				snprintf(fn, sizeof(fn), "digits/%d", hundreds);
+			}
+			if (num) {
+				ast_copy_string(fna, "digits/hundred", sizeof(fna));
+			} else {
+				snprintf(fna, sizeof(fna), "digits/h-hundred%s", gender);
+			}
+			t = 1;
+		} else 	if (num < 1000000) {
+			int thousands = num / 1000;
+			num = num % 1000;
+			if (thousands == 1) {
+				if (num) {
+					/* Thousand is a neutral word, so use the neutral recording */
+					ast_copy_string(fn, "digits/1hk", sizeof(fn));
+					ast_copy_string(fna, "digits/thousand", sizeof(fna));
+				} else {
+					if (t) {
+						ast_copy_string(fn, "digits/1hk", sizeof(fn));
+						snprintf(fna, sizeof(fna), "digits/h-thousand%s", gender);
+					} else {
+						snprintf(fn, sizeof(fn), "digits/h-thousand%s", gender);
+					}
+				}
+			} else {
+				res = ast_say_number_full_is(chan, thousands, ints, language, options, audiofd, ctrlfd);
+				if (res) {
+					return res;
+				}
+				if (num) {
+					ast_copy_string(fn, "digits/thousand", sizeof(fn));
+				} else {
+					snprintf(fn, sizeof(fn), "digits/h-thousand%s", gender);
+				}
+			}
+			if (num)
+				t = 1;
+		} else if (num < 1000000000) {
+			int millions = num / 1000000;
+			num = num % 1000000;
+			if (millions == 1) {
+				if (num) {
+					/* Million is a feminine word, so use the female form */
+					ast_copy_string(fn, "digits/1kvk", sizeof(fn));
+					ast_copy_string(fna, "digits/million", sizeof(fna));
+				} else {
+					ast_copy_string(fn, "digits/1hk", sizeof(fn));
+					snprintf(fna, sizeof(fna), "digits/h-million%s", gender);
+				}
+			} else {
+				res = ast_say_number_full_is(chan, millions, ints, language, options, audiofd, ctrlfd);
+				if (res) {
+					return res;
+				}
+				if (num) {
+					ast_copy_string(fn, "digits/millions", sizeof(fn));
+				} else {
+					snprintf(fn, sizeof(fn), "digits/h-million%s", gender);
+				}
+			}
+			if (num)
+				t = 1;
+		} else if (num < INT_MAX) {
+			int billions = num / 1000000000;
+			num = num % 1000000000;
+			if (billions == 1) {
+				if (num) {
+					ast_copy_string(fn, "digits/1", sizeof(fn));
+					ast_copy_string(fna, "digits/milliard", sizeof(fna));
+				} else {
+					ast_copy_string(fn, "digits/1hk", sizeof(fn));
+					snprintf(fna, sizeof(fna), "digits/h-milliard%s", gender);
+				}
+			} else {
+				res = ast_say_number_full_is(chan, billions, ints, language, options, audiofd, ctrlfd);
+				if (res)
+					return res;
+				if (num) {
+					ast_copy_string(fn, "digits/milliards", sizeof(fna));
+				} else {
+					snprintf(fn, sizeof(fna), "digits/h-milliard%s", gender);
+				}
+			}
+			if (num)
+				t = 1;
+		} else if (num == INT_MAX) {
+			snprintf(fn, sizeof(fn), "digits/h-last%s", gender);
+			num = 0;
+		} else {
+			ast_debug(1, "Number '%d' is too big for me\n", num);
+			res = -1;
+		}
+
+		if (!res) {
+			if (!ast_streamfile(chan, fn, language)) {
+				if ((audiofd > -1) && (ctrlfd > -1))
+					res = ast_waitstream_full(chan, ints, audiofd, ctrlfd);
+				else
+					res = ast_waitstream(chan, ints);
+			}
+			ast_stopstream(chan);
+			if (!res) {
+				if (strlen(fna) != 0 && !ast_streamfile(chan, fna, language)) {
+					if ((audiofd > -1) && (ctrlfd > -1)) {
+						res = ast_waitstream_full(chan, ints, audiofd, ctrlfd);
+					} else {
+						res = ast_waitstream(chan, ints);
+					}
+				}
+				ast_stopstream(chan);
+				strcpy(fna, "");
+			}
+		}
+	}
+	return res;
+}
+
 static int say_date(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
 {
 	if (!strncasecmp(lang, "en", 2)) {        /* English syntax */
@@ -3353,6 +3659,8 @@ static int say_date(struct ast_channel *
 		return ast_say_date_he(chan, t, ints, lang);
 	} else if (!strncasecmp(lang, "hu", 2)) { /* Hungarian syntax */
 		return ast_say_date_hu(chan, t, ints, lang);
+	} else if (!strncasecmp(lang, "is", 2)) { /* Icelandic syntax */
+		return ast_say_date_is(chan, t, ints, lang);
 	} else if (!strncasecmp(lang, "ka", 2)) { /* Georgian syntax */
 		return ast_say_date_ka(chan, t, ints, lang);
 	} else if (!strncasecmp(lang, "nl", 2)) { /* Dutch syntax */
@@ -3682,6 +3990,55 @@ int ast_say_date_he(struct ast_channel *
 	return res;
 }
 
+/* Icelandic syntax */
+int ast_say_date_is(struct ast_channel *chan, time_t t, const char *ints, const char *lang)
+{
+	struct timeval when = { t, 0 };
+	struct ast_tm tm;
+	char fn[256];
+	int res = 0;
+	ast_localtime(&when, &tm, NULL);
+	if (!res) {
+		snprintf(fn, sizeof(fn), "digits/day-%d", tm.tm_wday);
+		res = ast_streamfile(chan, fn, lang);
+		if (!res)
+			res = ast_waitstream(chan, ints);
+	}
+	if (!res)
+		res = ast_say_enumeration(chan, tm.tm_mday, ints, lang, (char * ) NULL);
+	if (!res)
+		res = ast_waitstream(chan, ints);
+	if (!res) {
+		snprintf(fn, sizeof(fn), "digits/mon-%d", tm.tm_mon);
+		res = ast_streamfile(chan, fn, lang);
+		if (!res)
+			res = ast_waitstream(chan, ints);
+	}
+	if (!res) {
+		/* Year */
+		int year = tm.tm_year + 1900;
+		if (year > 1999) {	/* year 2000 and later */
+			res = ast_say_number(chan, year, ints, lang, (char *) NULL);
+		} else {
+			if (year < 1100) {
+				/* I'm not going to handle 1100 and prior */
+				/* We'll just be silent on the year, instead of bombing out. */
+			} else {
+			    /* year 1100 to 1999. will anybody need this?!? */
+				snprintf(fn, sizeof(fn), "digits/%d", (year / 100));
+				res = wait_file(chan, ints, fn, lang);
+				if (!res) {
+					res = wait_file(chan, ints, "digits/hundred", lang);
+					if (!res && year % 100 != 0) {
+						res = ast_say_number(chan, (year % 100), ints, lang, (char *) NULL);
+					}
+				}
+			}
+		}
+	}
+	return res;
+}
+
 static int say_date_with_format(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone)
 {
 	if (!strncasecmp(lang, "en", 2)) {      /* English syntax */
@@ -3698,6 +4055,8 @@ static int say_date_with_format(struct a
 		return ast_say_date_with_format_fr(chan, t, ints, lang, format, tzone);
 	} else if (!strncasecmp(lang, "gr", 2)) { /* Greek syntax */
 		return ast_say_date_with_format_gr(chan, t, ints, lang, format, tzone);
+	} else if (!strncasecmp(lang, "is", 2)) { /* Icelandic syntax */
+		return ast_say_date_with_format_is(chan, t, ints, lang, format, tzone);
 	} else if (!strncasecmp(lang, "ja", 2)) { /* Japanese syntax */
 		return ast_say_date_with_format_ja(chan, t, ints, lang, format, tzone);
 	} else if (!strncasecmp(lang, "it", 2)) { /* Italian syntax */
@@ -4278,7 +4637,7 @@ int ast_say_date_with_format_de(struct a
 			case 'M':
 				/* Minute */
 				if (next_item(&format[offset + 1]) == 'S') { /* zero 'digits/0' only if seconds follow */
-					res = ast_say_number(chan, tm.tm_min, ints, lang, "f"); /* female only if we say digits/minutes */
+					res = ast_say_number(chan, tm.tm_min, ints, lang, "f"); /* female only if we say minutes */
 				} else if (tm.tm_min > 0) {
 					res = ast_say_number(chan, tm.tm_min, ints, lang, (char *) NULL);
 				}
@@ -4370,6 +4729,218 @@ int ast_say_date_with_format_de(struct a
 				break;
 			case ' ':
 			case '	':
+				/* Just ignore spaces and tabs */
+				break;
+			default:
+				/* Unknown character */
+				ast_log(LOG_WARNING, "Unknown character in datetime format %s: %c at pos %d\n", format, format[offset], offset);
+		}
+		/* Jump out on DTMF */
+		if (res) {
+			break;
+		}
+	}
+	return res;
+}
+
+/* Icelandic syntax */
+int ast_say_date_with_format_is(struct ast_channel *chan, time_t t, const char *ints, const char *lang, const char *format, const char *tzone)
+{
+	struct timeval when = { t, 0 };
+	struct ast_tm tm;
+	int res=0, offset, sndoffset;
+	char sndfile[256], nextmsg[256];
+
+	if (!format)
+		format = "A dBY HMS";
+
+	ast_localtime(&when, &tm, tzone);
+
+	for (offset=0 ; format[offset] != '\0' ; offset++) {
+		ast_debug(1, "Parsing %c (offset %d) in %s\n", format[offset], offset, format);
+		switch (format[offset]) {
+			/* NOTE:  if you add more options here, please try to be consistent with strftime(3) */
+			case '\'':
+				/* Literal name of a sound file */
+				for (sndoffset = 0; !strchr("\'\0", format[++offset]) && (sndoffset < sizeof(sndfile) - 1) ; sndoffset++) {
+					sndfile[sndoffset] = format[offset];
+				}
+				sndfile[sndoffset] = '\0';
+				res = wait_file(chan, ints, sndfile, lang);
+				break;
+			case 'A':
+			case 'a':
+				/* Sunday - Saturday */
+				snprintf(nextmsg, sizeof(nextmsg), "digits/day-%d", tm.tm_wday);
+				res = wait_file(chan, ints, nextmsg, lang);
+				break;
+			case 'B':
+			case 'b':
+			case 'h':
+				/* January - December */
+				snprintf(nextmsg, sizeof(nextmsg), "digits/mon-%d", tm.tm_mon);
+				res = wait_file(chan, ints, nextmsg, lang);
+				break;
+			case 'm':
+				/* Month enumerated */
+				res = ast_say_enumeration(chan, (tm.tm_mon + 1), ints, lang, "m");
+				break;
+			case 'd':
+			case 'e':
+				/* First - Thirtyfirst */
+				res = ast_say_enumeration(chan, tm.tm_mday, ints, lang, "m");
+				break;
+			case 'Y':
+				/* Year */
+				{
+					int year = tm.tm_year + 1900;
+					if (year > 1999) {	/* year 2000 and later */
+						res = ast_say_number(chan, year, ints, lang, (char *) NULL);
+					} else {
+						if (year < 1100) {
+							/* I'm not going to handle 1100 and prior */
+							/* We'll just be silent on the year, instead of bombing out. */
+						} else {
+						    /* year 1100 to 1999. will anybody need this?!? */
+						    /* say 1967 as 'nineteen hundred seven and sixty' */
+							snprintf(nextmsg, sizeof(nextmsg), "digits/%d", (year / 100) );
+							res = wait_file(chan, ints, nextmsg, lang);
+							if (!res) {
+								res = wait_file(chan, ints, "digits/hundred", lang);
+								if (!res && year % 100 != 0) {
+									res = ast_say_number(chan, (year % 100), ints, lang, (char *) NULL);
+								}
+							}
+						}
+					}
+				}
+				break;
+			case 'I':
+			case 'l':
+				/* 12-Hour */
+				res = wait_file(chan, ints, "digits/oclock", lang);
+				if (tm.tm_hour == 0)
+					ast_copy_string(nextmsg, "digits/12", sizeof(nextmsg));
+				else if (tm.tm_hour > 12)
+					snprintf(nextmsg, sizeof(nextmsg), "digits/%d", tm.tm_hour - 12);
+				else
+					snprintf(nextmsg, sizeof(nextmsg), "digits/%d", tm.tm_hour);
+				if (!res) {
+					res = wait_file(chan, ints, nextmsg, lang);
+				}
+				break;
+			case 'H':
+				/* 24-Hour, single digit hours preceeded by "oh" (0) */
+				if (tm.tm_hour < 10 && tm.tm_hour > 0) {
+					res = wait_file(chan, ints, "digits/0", lang);
+				}
+				/* FALLTRHU */
+			case 'k':
+				/* 24-Hour */
+				res = ast_say_number(chan, tm.tm_hour, ints, lang, "n");
+				break;
+			case 'M':
+				/* Minute */
+				if (tm.tm_min > 0 || next_item(&format[offset + 1]) == 'S') { /* zero 'digits/0' only if seconds follow */
+					if (tm.tm_min < 10)
+						res = wait_file(chan, ints, "digits/0", lang);
+					/* Gender depends on whether or not seconds follow */
+					if (next_item(&format[offset + 1]) == 'S')
+						res = ast_say_number(chan, tm.tm_min, ints, lang, "f");
+					else
+						res = ast_say_number(chan, tm.tm_min, ints, lang, "n");
+				}
+				if (!res && next_item(&format[offset + 1]) == 'S') { /* minutes only if seconds follow */
+					/* Say minute/minutes depending on whether minutes end in 1 */
+					if ((tm.tm_min % 10 == 1) && (tm.tm_min != 11)) {
+						res = wait_file(chan, ints, "minute", lang);
+					} else {
+						res = wait_file(chan, ints, "minutes", lang);
+					}
+				}
+				break;
+			case 'P':
+			case 'p':
+				/* AM/PM */
+				if (tm.tm_hour > 11)
+					ast_copy_string(nextmsg, "digits/p-m", sizeof(nextmsg));
+				else
+					ast_copy_string(nextmsg, "digits/a-m", sizeof(nextmsg));
+				res = wait_file(chan, ints, nextmsg, lang);
+				break;
+			case 'Q':
+				/* Shorthand for "Today", "Yesterday", or AdBY */
+				/* XXX As emphasized elsewhere, this should the native way in your
+				 * language to say the date, with changes in what you say, depending
+				 * upon how recent the date is. XXX */
+				{
+					struct timeval now = ast_tvnow();
+					struct ast_tm tmnow;
+					time_t beg_today;
+
+					ast_localtime(&now, &tmnow, tzone);
+					/* This might be slightly off, if we transcend a leap second, but never more off than 1 second */
+					/* In any case, it saves not having to do ast_mktime() */
+					beg_today = now.tv_sec - (tmnow.tm_hour * 3600) - (tmnow.tm_min * 60) - (tmnow.tm_sec);
+					if (beg_today < t) {
+						/* Today */
+						res = wait_file(chan, ints, "digits/today", lang);
+					} else if (beg_today - 86400 < t) {
+						/* Yesterday */
+						res = wait_file(chan, ints, "digits/yesterday", lang);
+					} else {
+						res = ast_say_date_with_format_is(chan, t, ints, lang, "AdBY", tzone);
+					}
+				}
+				break;
+			case 'q':
+				/* Shorthand for "" (today), "Yesterday", A (weekday), or AdBY */
+				/* XXX As emphasized elsewhere, this should the native way in your
+				 * language to say the date, with changes in what you say, depending
+				 * upon how recent the date is. XXX */
+				{
+					struct timeval now = ast_tvnow();
+					struct ast_tm tmnow;
+					time_t beg_today;
+
+					ast_localtime(&now, &tmnow, tzone);
+					/* This might be slightly off, if we transcend a leap second, but never more off than 1 second */
+					/* In any case, it saves not having to do ast_mktime() */
+					beg_today = now.tv_sec - (tmnow.tm_hour * 3600) - (tmnow.tm_min * 60) - (tmnow.tm_sec);
+					if (beg_today < t) {
+						/* Today */
+					} else if ((beg_today - 86400) < t) {
+						/* Yesterday */
+						res = wait_file(chan, ints, "digits/yesterday", lang);
+					} else if (beg_today - 86400 * 6 < t) {
+						/* Within the last week */
+						res = ast_say_date_with_format_is(chan, t, ints, lang, "A", tzone);
+					} else {
+						res = ast_say_date_with_format_is(chan, t, ints, lang, "AdBY", tzone);
+					}
+				}
+				break;
+			case 'R':
+				res = ast_say_date_with_format_is(chan, t, ints, lang, "HM", tzone);
+				break;
+			case 'S':
+				/* Seconds */
+				res = wait_file(chan, ints, "digits/and", lang);
+				if (!res) {
+					res = ast_say_number(chan, tm.tm_sec, ints, lang, "f");
+					/* Say minute/minutes depending on whether seconds end in 1 */
+					if (!res && (tm.tm_sec % 10 == 1) && (tm.tm_sec != 11)) {
+						res = wait_file(chan, ints, "second", lang);
+					} else {
+						res = wait_file(chan, ints, "seconds", lang);
+					}
+				}
+				break;
+			case 'T':
+				res = ast_say_date_with_format_is(chan, t, ints, lang, "HMS", tzone);
+				break;
+			case ' ':
+			case '	':
 				/* Just ignore spaces and tabs */
 				break;
 			default:
diff -pruN 1:13.23.1~dfsg-2/main/sched.c 1:16.1.1~dfsg-1/main/sched.c
--- 1:13.23.1~dfsg-2/main/sched.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/sched.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #ifdef DEBUG_SCHEDULER
 #define DEBUG(a) a
 #else
diff -pruN 1:13.23.1~dfsg-2/main/sdp_srtp.c 1:16.1.1~dfsg-1/main/sdp_srtp.c
--- 1:13.23.1~dfsg-2/main/sdp_srtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/sdp_srtp.c	2018-12-26 18:45:13.000000000 +0000
@@ -16,12 +16,11 @@
  * at the top of the source tree.
  */
 
-/*! \file ast_sdp_crypto.c
+/*! \file
  *
  * \brief SRTP and SDP Security descriptions
  *
- * Specified in RFC 3711
- * Specified in RFC 4568
+ * Specified in RFC 3711, 6188, 7714, and 4568
  *
  * \author Mikael Magnusson <mikma@users.sourceforge.net>
  */
@@ -32,20 +31,12 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include "asterisk/linkedlists.h"       /* for AST_LIST_NEXT, etc */
+#include "asterisk/logger.h"            /* for ast_log, LOG_ERROR, etc */
+#include "asterisk/sdp_srtp.h"          /* for ast_sdp_srtp, etc */
 
-#include <math.h>
-#include "asterisk/options.h"
-#include "asterisk/utils.h"
-#include "asterisk/sdp_srtp.h"
-
-#define SRTP_MASTER_LEN 30
-#define SRTP_MASTERKEY_LEN 16
-#define SRTP_MASTERSALT_LEN ((SRTP_MASTER_LEN) - (SRTP_MASTERKEY_LEN))
-#define SRTP_MASTER_LEN64 (((SRTP_MASTER_LEN) * 8 + 5) / 6 + 1)
-
-extern struct ast_srtp_res *res_srtp;
-extern struct ast_srtp_policy_res *res_srtp_policy;
+/*! Registered SDP crypto API */
+static struct ast_sdp_crypto_api *sdp_crypto_api;
 
 struct ast_sdp_srtp *ast_sdp_srtp_alloc(void)
 {
@@ -59,363 +50,54 @@ struct ast_sdp_srtp *ast_sdp_srtp_alloc(
 
 void ast_sdp_srtp_destroy(struct ast_sdp_srtp *srtp)
 {
-	if (srtp->crypto) {
+	struct ast_sdp_srtp *next;
+
+	for (next = AST_LIST_NEXT(srtp, sdp_srtp_list);
+	     srtp;
+	     srtp = next, next = srtp ? AST_LIST_NEXT(srtp, sdp_srtp_list) : NULL) {
 		ast_sdp_crypto_destroy(srtp->crypto);
+		srtp->crypto = NULL;
+		ast_free(srtp);
 	}
-	srtp->crypto = NULL;
-	ast_free(srtp);
 }
 
-struct ast_sdp_crypto {
-	char *a_crypto;
-	unsigned char local_key[SRTP_MASTER_LEN];
-	int tag;
-	char local_key64[SRTP_MASTER_LEN64];
-	unsigned char remote_key[SRTP_MASTER_LEN];
-};
-
-static int set_crypto_policy(struct ast_srtp_policy *policy, int suite_val, const unsigned char *master_key, unsigned long ssrc, int inbound);
-
 void ast_sdp_crypto_destroy(struct ast_sdp_crypto *crypto)
 {
-	ast_free(crypto->a_crypto);
-	crypto->a_crypto = NULL;
-	ast_free(crypto);
+	if (sdp_crypto_api) {
+		sdp_crypto_api->dtor(crypto);
+	}
 }
 
 struct ast_sdp_crypto *ast_sdp_crypto_alloc(void)
 {
-	struct ast_sdp_crypto *p;
-	int key_len;
-	unsigned char remote_key[SRTP_MASTER_LEN];
-
-	if (!ast_rtp_engine_srtp_is_registered()) {
-		return NULL;
-	}
-
-	if (!(p = ast_calloc(1, sizeof(*p)))) {
-		return NULL;
-	}
-	p->tag = 1;
-
-	if (res_srtp->get_random(p->local_key, sizeof(p->local_key)) < 0) {
-		ast_sdp_crypto_destroy(p);
-		return NULL;
-	}
-
-	ast_base64encode(p->local_key64, p->local_key, SRTP_MASTER_LEN, sizeof(p->local_key64));
-
-	key_len = ast_base64decode(remote_key, p->local_key64, sizeof(remote_key));
-
-	if (key_len != SRTP_MASTER_LEN) {
-		ast_log(LOG_ERROR, "base64 encode/decode bad len %d != %d\n", key_len, SRTP_MASTER_LEN);
-		ast_sdp_crypto_destroy(p);
+	if (!sdp_crypto_api) {
 		return NULL;
 	}
-
-	if (memcmp(remote_key, p->local_key, SRTP_MASTER_LEN)) {
-		ast_log(LOG_ERROR, "base64 encode/decode bad key\n");
-		ast_sdp_crypto_destroy(p);
-		return NULL;
-	}
-
-	ast_debug(1 , "local_key64 %s len %zu\n", p->local_key64, strlen(p->local_key64));
-
-	return p;
-}
-
-static int set_crypto_policy(struct ast_srtp_policy *policy, int suite_val, const unsigned char *master_key, unsigned long ssrc, int inbound)
-{
-	const unsigned char *master_salt = NULL;
-
-	if (!ast_rtp_engine_srtp_is_registered()) {
-		return -1;
-	}
-
-	master_salt = master_key + SRTP_MASTERKEY_LEN;
-	if (res_srtp_policy->set_master_key(policy, master_key, SRTP_MASTERKEY_LEN, master_salt, SRTP_MASTERSALT_LEN) < 0) {
-		return -1;
-	}
-
-	if (res_srtp_policy->set_suite(policy, suite_val)) {
-		ast_log(LOG_WARNING, "Could not set remote SRTP suite\n");
-		return -1;
-	}
-
-	res_srtp_policy->set_ssrc(policy, ssrc, inbound);
-
-	return 0;
-}
-
-static int crypto_activate(struct ast_sdp_crypto *p, int suite_val, unsigned char *remote_key, struct ast_rtp_instance *rtp)
-{
-	struct ast_srtp_policy *local_policy = NULL;
-	struct ast_srtp_policy *remote_policy = NULL;
-	struct ast_rtp_instance_stats stats = {0,};
-	int res = -1;
-
-	if (!ast_rtp_engine_srtp_is_registered()) {
-		return -1;
-	}
-
-	if (!p) {
-		return -1;
-	}
-
-	if (!(local_policy = res_srtp_policy->alloc())) {
-		return -1;
-	}
-
-	if (!(remote_policy = res_srtp_policy->alloc())) {
-		goto err;
-	}
-
-	if (ast_rtp_instance_get_stats(rtp, &stats, AST_RTP_INSTANCE_STAT_LOCAL_SSRC)) {
-		goto err;
-	}
-
-	if (set_crypto_policy(local_policy, suite_val, p->local_key, stats.local_ssrc, 0) < 0) {
-		goto err;
-	}
-
-	if (set_crypto_policy(remote_policy, suite_val, remote_key, 0, 1) < 0) {
-		goto err;
-	}
-
-	/* Add the SRTP policies */
-	if (ast_rtp_instance_add_srtp_policy(rtp, remote_policy, local_policy, 0)) {
-		ast_log(LOG_WARNING, "Could not set SRTP policies\n");
-		goto err;
-	}
-
-	ast_debug(1 , "SRTP policy activated\n");
-	res = 0;
-
-err:
-	if (local_policy) {
-		res_srtp_policy->destroy(local_policy);
-	}
-
-	if (remote_policy) {
-		res_srtp_policy->destroy(remote_policy);
-	}
-
-	return res;
+	return sdp_crypto_api->alloc();
 }
 
 int ast_sdp_crypto_process(struct ast_rtp_instance *rtp, struct ast_sdp_srtp *srtp, const char *attr)
 {
-	char *str = NULL;
-	char *tag = NULL;
-	char *suite = NULL;
-	char *key_params = NULL;
-	char *key_param = NULL;
-	char *session_params = NULL;
-	char *key_salt = NULL;       /* The actual master key and key salt */
-	char *lifetime = NULL;       /* Key lifetime (# of RTP packets) */
-	char *mki = NULL;            /* Master Key Index */
-	int found = 0;
-	int key_len = 0;
-	int suite_val = 0;
-	unsigned char remote_key[SRTP_MASTER_LEN];
-	int taglen = 0;
-	double sdes_lifetime;
-	struct ast_sdp_crypto *crypto = srtp->crypto;
-
-	if (!ast_rtp_engine_srtp_is_registered()) {
+	if (!sdp_crypto_api) {
 		return -1;
 	}
-
-	str = ast_strdupa(attr);
-
-	tag = strsep(&str, " ");
-	suite = strsep(&str, " ");
-	key_params = strsep(&str, " ");
-	session_params = strsep(&str, " ");
-
-	if (!tag || !suite) {
-		ast_log(LOG_WARNING, "Unrecognized crypto attribute a=%s\n", attr);
-		return -1;
-	}
-
-	/* RFC4568 9.1 - tag is 1-9 digits */
-	if (sscanf(tag, "%30d", &crypto->tag) != 1 || crypto->tag < 0 || crypto->tag > 999999999) {
-		ast_log(LOG_WARNING, "Unacceptable a=crypto tag: %s\n", tag);
-		return -1;
-	}
-
-	if (!ast_strlen_zero(session_params)) {
-		ast_log(LOG_WARNING, "Unsupported crypto parameters: %s\n", session_params);
-		return -1;
-	}
-
-	if (!strcmp(suite, "AES_CM_128_HMAC_SHA1_80")) {
-		suite_val = AST_AES_CM_128_HMAC_SHA1_80;
-		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
-		taglen = 80;
-	} else if (!strcmp(suite, "AES_CM_128_HMAC_SHA1_32")) {
-		suite_val = AST_AES_CM_128_HMAC_SHA1_32;
-		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
-		taglen = 32;
-	} else {
-		ast_log(LOG_WARNING, "Unsupported crypto suite: %s\n", suite);
-		return -1;
-	}
-
-	while ((key_param = strsep(&key_params, ";"))) {
-		unsigned int n_lifetime;
-		char *method = NULL;
-		char *info = NULL;
-
-		method = strsep(&key_param, ":");
-		info = strsep(&key_param, ";");
-		sdes_lifetime = 0;
-
-		if (strcmp(method, "inline")) {
-			continue;
-		}
-
-		key_salt = strsep(&info, "|");
-
-		/* The next parameter can be either lifetime or MKI */
-		lifetime = strsep(&info, "|");
-		if (!lifetime) {
-			found = 1;
-			break;
-		}
-
-		mki = strchr(lifetime, ':');
-		if (mki) {
-			mki = lifetime;
-			lifetime = NULL;
-		} else {
-			mki = strsep(&info, "|");
-		}
-
-		if (mki && *mki != '1') {
-			ast_log(LOG_NOTICE, "Crypto MKI handling is not supported: ignoring attribute %s\n", attr);
-			continue;
-		}
-
-		if (lifetime) {
-			if (!strncmp(lifetime, "2^", 2)) {
-				char *lifetime_val = lifetime + 2;
-
-				/* Exponential lifetime */
-				if (sscanf(lifetime_val, "%30u", &n_lifetime) != 1) {
-					ast_log(LOG_NOTICE, "Failed to parse lifetime value in crypto attribute: %s\n", attr);
-					continue;
-				}
-
-				if (n_lifetime > 48) {
-					/* Yeah... that's a bit big. */
-					ast_log(LOG_NOTICE, "Crypto lifetime exponent of '%u' is a bit large; using 48\n", n_lifetime);
-					n_lifetime = 48;
-				}
-				sdes_lifetime = pow(2, n_lifetime);
-			} else {
-				/* Decimal lifetime */
-				if (sscanf(lifetime, "%30u", &n_lifetime) != 1) {
-					ast_log(LOG_NOTICE, "Failed to parse lifetime value in crypto attribute: %s\n", attr);
-					continue;
-				}
-				sdes_lifetime = n_lifetime;
-			}
-
-			/* Accept anything above ~5.8 hours. Less than ~5.8; reject. */
-			if (sdes_lifetime < 1048576) {
-				ast_log(LOG_NOTICE, "Rejecting crypto attribute '%s': lifetime '%f' too short\n", attr, sdes_lifetime);
-				continue;
-			}
-		}
-
-		ast_debug(2, "Crypto attribute '%s' accepted with lifetime '%f', MKI '%s'\n",
-			attr, sdes_lifetime, mki ? mki : "-");
-
-		found = 1;
-		break;
-	}
-
-	if (!found) {
-		ast_log(LOG_NOTICE, "SRTP crypto offer not acceptable: '%s'\n", attr);
-		return -1;
-	}
-
-	key_len = ast_base64decode(remote_key, key_salt, sizeof(remote_key));
-	if (key_len != SRTP_MASTER_LEN) {
-		ast_log(LOG_WARNING, "SRTP descriptions key length '%d' != master length '%d'\n",
-			key_len, SRTP_MASTER_LEN);
-		return -1;
-	}
-
-	if (!memcmp(crypto->remote_key, remote_key, sizeof(crypto->remote_key))) {
-		ast_debug(1, "SRTP remote key unchanged; maintaining current policy\n");
-		return 0;
-	}
-	memcpy(crypto->remote_key, remote_key, sizeof(crypto->remote_key));
-
-	if (crypto_activate(crypto, suite_val, remote_key, rtp) < 0) {
-		return -1;
-	}
-
-	/* Finally, rebuild the crypto line */
-	if (ast_sdp_crypto_build_offer(crypto, taglen)) {
-		return -1;
-	}
-
-	ast_set_flag(srtp, AST_SRTP_CRYPTO_OFFER_OK);
-	return 0;
+	return sdp_crypto_api->parse_offer(rtp, srtp, attr);
 }
 
 int ast_sdp_crypto_build_offer(struct ast_sdp_crypto *p, int taglen)
 {
-	/* Rebuild the crypto line */
-	if (p->a_crypto) {
-		ast_free(p->a_crypto);
-	}
-
-	if (ast_asprintf(&p->a_crypto, "%d AES_CM_128_HMAC_SHA1_%i inline:%s",
-			 p->tag, taglen, p->local_key64) == -1) {
-			ast_log(LOG_ERROR, "Could not allocate memory for crypto line\n");
+	if (!sdp_crypto_api) {
 		return -1;
 	}
-
-	ast_debug(1, "Crypto line: a=crypto:%s\n", p->a_crypto);
-
-	return 0;
+	return sdp_crypto_api->build_offer(p, taglen);
 }
 
 const char *ast_sdp_srtp_get_attrib(struct ast_sdp_srtp *srtp, int dtls_enabled, int default_taglen_32)
 {
-	int taglen = default_taglen_32 ? 32 : 80;
-
-	if (!srtp) {
+	if (!sdp_crypto_api) {
 		return NULL;
 	}
-
-	/* Set encryption properties */
-	if (!srtp->crypto) {
-		srtp->crypto = ast_sdp_crypto_alloc();
-	}
-
-	if (dtls_enabled) {
-		/* If DTLS-SRTP is enabled the key details will be pulled from TLS */
-		return NULL;
-	}
-
-	/* set the key length based on INVITE or settings */
-	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_80)) {
-		taglen = 80;
-	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_32)) {
-		taglen = 32;
-	}
-
-	if (srtp->crypto && (ast_sdp_crypto_build_offer(srtp->crypto, taglen) >= 0)) {
-		return srtp->crypto->a_crypto;
-	}
-
-	ast_log(LOG_WARNING, "No SRTP key management enabled\n");
-	return NULL;
+	return sdp_crypto_api->get_attr(srtp, dtls_enabled, default_taglen_32);
 }
 
 char *ast_sdp_get_rtp_profile(unsigned int sdes_active, struct ast_rtp_instance *instance, unsigned int using_avpf,
@@ -437,3 +119,19 @@ char *ast_sdp_get_rtp_profile(unsigned i
 		}
 	}
 }
+
+int ast_sdp_crypto_register(struct ast_sdp_crypto_api *api)
+{
+	if (sdp_crypto_api) {
+		return -1;
+	}
+	sdp_crypto_api = api;
+	return 0;
+}
+
+void ast_sdp_crypto_unregister(struct ast_sdp_crypto_api *api)
+{
+	if (sdp_crypto_api == api) {
+		sdp_crypto_api = NULL;
+	}
+}
diff -pruN 1:13.23.1~dfsg-2/main/security_events.c 1:16.1.1~dfsg-1/main/security_events.c
--- 1:13.23.1~dfsg-2/main/security_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/security_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -362,8 +362,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/strings.h"
 #include "asterisk/network.h"
diff -pruN 1:13.23.1~dfsg-2/main/sem.c 1:16.1.1~dfsg-1/main/sem.c
--- 1:13.23.1~dfsg-2/main/sem.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/sem.c	2018-12-26 18:45:13.000000000 +0000
@@ -23,8 +23,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/sem.h"
 #include "asterisk/utils.h"
 
diff -pruN 1:13.23.1~dfsg-2/main/slinfactory.c 1:16.1.1~dfsg-1/main/slinfactory.c
--- 1:13.23.1~dfsg-2/main/slinfactory.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/slinfactory.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/frame.h"
 #include "asterisk/format_cache.h"
 #include "asterisk/slinfactory.h"
diff -pruN 1:13.23.1~dfsg-2/main/smoother.c 1:16.1.1~dfsg-1/main/smoother.c
--- 1:13.23.1~dfsg-2/main/smoother.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/smoother.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/frame.h"
 #include "asterisk/astobj2.h"
diff -pruN 1:13.23.1~dfsg-2/main/sorcery.c 1:16.1.1~dfsg-1/main/sorcery.c
--- 1:13.23.1~dfsg-2/main/sorcery.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/sorcery.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/sorcery.h"
 #include "asterisk/astobj2.h"
@@ -129,6 +127,9 @@ struct ast_sorcery_object {
 
 	/*! \brief Extended object fields */
 	struct ast_variable *extended;
+
+	/*! \brief Time that the object was created */
+	struct timeval created;
 };
 
 /*! \brief Structure for registered object type */
@@ -206,6 +207,13 @@ struct ast_sorcery_object_field {
 	intptr_t args[];
 };
 
+/*! \brief Proxy object for sorcery */
+struct sorcery_proxy {
+	AO2_WEAKPROXY();
+	/*! \brief The name of the module owning this sorcery instance */
+	char module_name[0];
+};
+
 /*! \brief Full structure for sorcery */
 struct ast_sorcery {
 	/*! \brief Container for known object types */
@@ -214,8 +222,8 @@ struct ast_sorcery {
 	/*! \brief Observers */
 	struct ao2_container *observers;
 
-	/*! \brief The name of the module owning this sorcery instance */
-	char module_name[0];
+	/*! \brief Pointer to module_name in the associated sorcery_proxy. */
+	char *module_name;
 };
 
 /*! \brief Structure for passing load/reload details */
@@ -335,13 +343,10 @@ static sorcery_field_handler sorcery_fie
 	return NULL;
 }
 
-/*! \brief Hashing function for sorcery wizards */
+/*! \brief Hashing and comparison functions for sorcery wizards */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_internal_wizard, callbacks.name)
-
-/*! \brief Comparator function for sorcery wizards */
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_internal_wizard, callbacks.name)
 
-/*! \brief Hashing function for sorcery wizards */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_object_field, name)
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_object_field, name)
 
@@ -359,10 +364,10 @@ static void sorcery_cleanup(void)
 }
 
 /*! \brief Compare function for sorcery instances */
-AO2_STRING_FIELD_CMP_FN(ast_sorcery, module_name)
+AO2_STRING_FIELD_CMP_FN(sorcery_proxy, module_name)
 
 /*! \brief Hashing function for sorcery instances */
-AO2_STRING_FIELD_HASH_FN(ast_sorcery, module_name)
+AO2_STRING_FIELD_HASH_FN(sorcery_proxy, module_name)
 
 int ast_sorcery_init(void)
 {
@@ -392,7 +397,7 @@ int ast_sorcery_init(void)
 	}
 
 	instances = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, INSTANCE_BUCKETS,
-		ast_sorcery_hash_fn, NULL, ast_sorcery_cmp_fn);
+		sorcery_proxy_hash_fn, NULL, sorcery_proxy_cmp_fn);
 	if (!instances) {
 		return -1;
 	}
@@ -411,27 +416,6 @@ static void sorcery_internal_wizard_dest
 
 int __ast_sorcery_wizard_register(const struct ast_sorcery_wizard *interface, struct ast_module *module)
 {
-	struct ast_sorcery_wizard compat = {
-		.name = interface->name,
-		.open = interface->open,
-		.load = interface->load,
-		.reload = interface->reload,
-		.create = interface->create,
-		.retrieve_id = interface->retrieve_id,
-		.retrieve_regex = interface->retrieve_regex,
-		.retrieve_fields = interface->retrieve_fields,
-		.retrieve_multiple = interface->retrieve_multiple,
-		.update = interface->update,
-		.delete = interface->delete,
-		.close = interface->close,
-		.retrieve_prefix = NULL,
-	};
-
-	return __ast_sorcery_wizard_register_with_prefix(&compat, module);
-}
-
-int __ast_sorcery_wizard_register_with_prefix(const struct ast_sorcery_wizard *interface, struct ast_module *module)
-{
 	struct ast_sorcery_internal_wizard *wizard;
 	int res = -1;
 
@@ -592,61 +576,87 @@ static void sorcery_destructor(void *obj
 
 /*! \brief Hashing function for sorcery types */
 AO2_STRING_FIELD_HASH_FN(ast_sorcery_object_type, name)
-
-/*! \brief Comparator function for sorcery types */
 AO2_STRING_FIELD_CMP_FN(ast_sorcery_object_type, name)
 
-struct ast_sorcery *__ast_sorcery_open(const char *module_name)
+static void sorcery_proxy_cb(void *weakproxy, void *data)
 {
+	ao2_unlink(instances, weakproxy);
+}
+
+struct ast_sorcery *__ast_sorcery_open(const char *module_name, const char *file, int line, const char *func)
+{
+	struct sorcery_proxy *proxy;
 	struct ast_sorcery *sorcery;
 
 	ast_assert(module_name != NULL);
 
 	ao2_wrlock(instances);
-	if ((sorcery = ao2_find(instances, module_name, OBJ_SEARCH_KEY | OBJ_NOLOCK))) {
-		goto done;
+	sorcery = __ao2_weakproxy_find(instances, module_name, OBJ_SEARCH_KEY | OBJ_NOLOCK,
+		__PRETTY_FUNCTION__, file, line, func);
+	if (sorcery) {
+		ao2_unlock(instances);
+
+		return sorcery;
 	}
 
-	if (!(sorcery = ao2_alloc(sizeof(*sorcery) + strlen(module_name) + 1, sorcery_destructor))) {
-		goto done;
+	proxy = ao2_t_weakproxy_alloc(sizeof(*proxy) + strlen(module_name) + 1, NULL, module_name);
+	if (!proxy) {
+		goto failure_cleanup;
+	}
+	strcpy(proxy->module_name, module_name); /* Safe */
+
+	sorcery = __ao2_alloc(sizeof(*sorcery), sorcery_destructor, AO2_ALLOC_OPT_LOCK_MUTEX, module_name, file, line, func);
+	if (!sorcery) {
+		goto failure_cleanup;
+	}
+
+	sorcery->module_name = proxy->module_name;
+
+	/* We have exclusive access to proxy and sorcery, no need for locking here. */
+	if (ao2_t_weakproxy_set_object(proxy, sorcery, OBJ_NOLOCK, "weakproxy link")) {
+		goto failure_cleanup;
 	}
 
-	sorcery->types = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK, TYPE_BUCKETS,
-			ast_sorcery_object_type_hash_fn, ast_sorcery_object_type_cmp_fn);
+	if (ao2_weakproxy_subscribe(proxy, sorcery_proxy_cb, NULL, OBJ_NOLOCK)) {
+		goto failure_cleanup;
+	}
+
+	sorcery->types = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, TYPE_BUCKETS,
+		ast_sorcery_object_type_hash_fn, NULL, ast_sorcery_object_type_cmp_fn);
 	if (!sorcery->types) {
-		ao2_ref(sorcery, -1);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
 	if (!(sorcery->observers = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_RWLOCK, 0, NULL, NULL))) {
-		ao2_ref(sorcery, -1);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
-	strcpy(sorcery->module_name, module_name); /* Safe */
-
 	if (__ast_sorcery_apply_config(sorcery, module_name, module_name) == AST_SORCERY_APPLY_FAIL) {
 		ast_log(LOG_ERROR, "Error attempting to apply configuration %s to sorcery.\n", module_name);
-		ao2_cleanup(sorcery);
-		sorcery = NULL;
-		goto done;
+		goto failure_cleanup;
 	}
 
-	ao2_link_flags(instances, sorcery, OBJ_NOLOCK);
+	ao2_link_flags(instances, proxy, OBJ_NOLOCK);
+	ao2_ref(proxy, -1);
 
 	NOTIFY_GLOBAL_OBSERVERS(observers, instance_created, module_name, sorcery);
 
-done:
 	ao2_unlock(instances);
 	return sorcery;
+
+failure_cleanup:
+	/* cleanup of sorcery may result in locking instances, so make sure we unlock first. */
+	ao2_unlock(instances);
+	ao2_cleanup(sorcery);
+	ao2_cleanup(proxy);
+
+	return NULL;
 }
 
 /*! \brief Search function for sorcery instances */
 struct ast_sorcery *ast_sorcery_retrieve_by_module_name(const char *module_name)
 {
-	return ao2_find(instances, module_name, OBJ_SEARCH_KEY);
+	return ao2_weakproxy_find(instances, module_name, OBJ_SEARCH_KEY, "");
 }
 
 /*! \brief Destructor function for object types */
@@ -822,16 +832,19 @@ enum ast_sorcery_apply_result __ast_sorc
 	RAII_VAR(struct ast_sorcery_object_wizard *, object_wizard, ao2_alloc(sizeof(*object_wizard), sorcery_object_wizard_destructor), ao2_cleanup);
 	int created = 0;
 
-	if (!wizard) {
+	if (!object_wizard) {
+		return AST_SORCERY_APPLY_FAIL;
+	}
+
+	if (!wizard || wizard->callbacks.module != ast_module_running_ref(wizard->callbacks.module)) {
 		ast_log(LOG_ERROR, "Wizard '%s' could not be applied to object type '%s' as it was not found\n",
 			name, type);
 		return AST_SORCERY_APPLY_FAIL;
-	} else if (!object_wizard) {
-		return AST_SORCERY_APPLY_FAIL;
 	}
 
 	if (!object_type) {
 		if (!(object_type = sorcery_object_type_alloc(type, module))) {
+			ast_module_unref(wizard->callbacks.module);
 			return AST_SORCERY_APPLY_FAIL;
 		}
 		created = 1;
@@ -848,6 +861,7 @@ enum ast_sorcery_apply_result __ast_sorc
 			ast_debug(1, "Wizard %s already applied to object type %s\n",
 					wizard->callbacks.name, object_type->name);
 			AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+			ast_module_unref(wizard->callbacks.module);
 			return AST_SORCERY_APPLY_DUPLICATE;
 		}
 	}
@@ -858,11 +872,10 @@ enum ast_sorcery_apply_result __ast_sorc
 		ast_log(LOG_WARNING, "Wizard '%s' failed to open mapping for object type '%s' with data: %s\n",
 			name, object_type->name, S_OR(data, ""));
 		AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+		ast_module_unref(wizard->callbacks.module);
 		return AST_SORCERY_APPLY_FAIL;
 	}
 
-	ast_module_ref(wizard->callbacks.module);
-
 	object_wizard->wizard = ao2_bump(wizard);
 	object_wizard->caching = caching;
 
@@ -1581,9 +1594,26 @@ static void sorcery_object_destructor(vo
 	ast_free(details->object->id);
 }
 
+void *ast_sorcery_lockable_alloc(size_t size, ao2_destructor_fn destructor, void *lockobj)
+{
+	void *object = ao2_alloc_with_lockobj(size + sizeof(struct ast_sorcery_object),
+		sorcery_object_destructor, lockobj, "");
+	struct ast_sorcery_object_details *details = object;
+
+	if (!object) {
+		return NULL;
+	}
+
+	details->object = object + size;
+	details->object->destructor = destructor;
+
+	return object;
+}
+
 void *ast_sorcery_generic_alloc(size_t size, ao2_destructor_fn destructor)
 {
-	void *object = ao2_alloc_options(size + sizeof(struct ast_sorcery_object), sorcery_object_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	void *object = ao2_alloc_options(size + sizeof(struct ast_sorcery_object),
+		sorcery_object_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
 	struct ast_sorcery_object_details *details = object;
 
 	if (!object) {
@@ -1619,6 +1649,7 @@ void *ast_sorcery_alloc(const struct ast
 		return NULL;
 	}
 
+	details->object->created = ast_tvnow();
 	ast_copy_string(details->object->type, type, sizeof(details->object->type));
 
 	if (aco_set_defaults(&object_type->type, id, details)) {
@@ -1761,7 +1792,8 @@ void *ast_sorcery_retrieve_by_fields(con
 
 	/* If returning multiple objects create a container to store them in */
 	if ((flags & AST_RETRIEVE_FLAG_MULTIPLE)) {
-		if (!(object = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+		object = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+		if (!object) {
 			return NULL;
 		}
 	}
@@ -1805,7 +1837,12 @@ struct ao2_container *ast_sorcery_retrie
 	struct ao2_container *objects;
 	int i;
 
-	if (!object_type || !(objects = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+	if (!object_type) {
+		return NULL;
+	}
+
+	objects = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+	if (!objects) {
 		return NULL;
 	}
 
@@ -1835,7 +1872,12 @@ struct ao2_container *ast_sorcery_retrie
 	struct ao2_container *objects;
 	int i;
 
-	if (!object_type || !(objects = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1, NULL, NULL))) {
+	if (!object_type) {
+		return NULL;
+	}
+
+	objects = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0, NULL, NULL);
+	if (!objects) {
 		return NULL;
 	}
 
@@ -2123,16 +2165,33 @@ int ast_sorcery_delete(const struct ast_
 	return object_wizard ? 0 : -1;
 }
 
-void ast_sorcery_unref(struct ast_sorcery *sorcery)
+int ast_sorcery_is_stale(const struct ast_sorcery *sorcery, void *object)
 {
-	if (sorcery) {
-		/* One ref for what we just released, the other for the instances container. */
-		ao2_wrlock(instances);
-		if (ao2_ref(sorcery, -1) == 2) {
-			ao2_unlink_flags(instances, sorcery, OBJ_NOLOCK);
+	const struct ast_sorcery_object_details *details = object;
+	RAII_VAR(struct ast_sorcery_object_type *, object_type, ao2_find(sorcery->types, details->object->type, OBJ_KEY), ao2_cleanup);
+	struct ast_sorcery_object_wizard *found_wizard;
+	int res = 0;
+	int i;
+
+	if (!object_type) {
+		return -1;
+	}
+
+	AST_VECTOR_RW_RDLOCK(&object_type->wizards);
+	for (i = 0; i < AST_VECTOR_SIZE(&object_type->wizards); i++) {
+		found_wizard = AST_VECTOR_GET(&object_type->wizards, i);
+
+		if (found_wizard->wizard->callbacks.is_stale) {
+			res |= found_wizard->wizard->callbacks.is_stale(sorcery, found_wizard->data, object);
+			ast_debug(5, "After calling wizard '%s', object '%s' is %s\n",
+				found_wizard->wizard->callbacks.name,
+				ast_sorcery_object_get_id(object),
+				res ? "stale" : "not stale");
 		}
-		ao2_unlock(instances);
 	}
+	AST_VECTOR_RW_UNLOCK(&object_type->wizards);
+
+	return res;
 }
 
 const char *ast_sorcery_object_get_id(const void *object)
@@ -2141,6 +2200,12 @@ const char *ast_sorcery_object_get_id(co
 	return details->object->id;
 }
 
+const struct timeval ast_sorcery_object_get_created(const void *object)
+{
+	const struct ast_sorcery_object_details *details = object;
+	return details->object->created;
+}
+
 const char *ast_sorcery_object_get_type(const void *object)
 {
 	const struct ast_sorcery_object_details *details = object;
diff -pruN 1:13.23.1~dfsg-2/main/sounds.c 1:16.1.1~dfsg-1/main/sounds.c
--- 1:13.23.1~dfsg-2/main/sounds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/sounds.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,7 +34,7 @@
 #include "asterisk/sounds_index.h"
 #include "asterisk/file.h"
 #include "asterisk/cli.h"
-#include "asterisk/_private.h"
+#include "asterisk/module.h"
 #include "asterisk/stasis_message_router.h"
 #include "asterisk/stasis_system.h"
 
@@ -111,7 +111,7 @@ static int update_index_cb(void *obj, vo
 
 AST_MUTEX_DEFINE_STATIC(reload_lock);
 
-int ast_sounds_reindex(void)
+static int reload_module(void)
 {
 	RAII_VAR(struct ast_str *, sounds_dir, NULL, ast_free);
 	RAII_VAR(struct ao2_container *, languages, NULL, ao2_cleanup);
@@ -279,13 +279,15 @@ static struct ast_cli_entry cli_sounds[]
 	AST_CLI_DEFINE(handle_cli_sound_show, "Shows details about a specific sound"),
 };
 
-static void sounds_cleanup(void)
+static int unload_module(void)
 {
 	stasis_message_router_unsubscribe_and_join(sounds_system_router);
 	sounds_system_router = NULL;
 	ast_cli_unregister_multiple(cli_sounds, ARRAY_LEN(cli_sounds));
 	ao2_cleanup(sounds_index);
 	sounds_index = NULL;
+
+	return 0;
 }
 
 static void format_update_cb(void *data, struct stasis_subscription *sub,
@@ -293,21 +295,21 @@ static void format_update_cb(void *data,
 {
 	/* Reindexing during shutdown is pointless. */
 	if (!ast_shutting_down()) {
-		ast_sounds_reindex();
+		reload_module();
 	}
 }
 
-int ast_sounds_index_init(void)
+static int load_module(void)
 {
 	int res = 0;
-	if (ast_sounds_reindex()) {
-		return -1;
+	if (reload_module()) {
+		return AST_MODULE_LOAD_FAILURE;
 	}
 	res |= ast_cli_register_multiple(cli_sounds, ARRAY_LEN(cli_sounds));
 
 	sounds_system_router = stasis_message_router_create(ast_system_topic());
 	if (!sounds_system_router) {
-		return -1;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	if (ast_format_register_type()) {
@@ -326,15 +328,20 @@ int ast_sounds_index_init(void)
 			NULL);
 	}
 
-	if (res) {
-		return -1;
-	}
-
-	ast_register_cleanup(sounds_cleanup);
-	return 0;
+	return res ? AST_MODULE_LOAD_FAILURE : AST_MODULE_LOAD_SUCCESS;
 }
 
 struct ast_media_index *ast_sounds_get_index(void)
 {
 	return ao2_bump(sounds_index);
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Sounds Index",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	/* This reload doesn't use config so this module doesn't require "extconfig". */
+	.reload = reload_module,
+	/* Load after the format modules to reduce processing during startup. */
+	.load_pri = AST_MODPRI_APP_DEPEND + 1,
+);
diff -pruN 1:13.23.1~dfsg-2/main/srv.c 1:16.1.1~dfsg-1/main/srv.c
--- 1:13.23.1~dfsg-2/main/srv.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/srv.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <netinet/in.h>
 #include <arpa/nameser.h>
 #ifdef __APPLE__
diff -pruN 1:13.23.1~dfsg-2/main/stasis_bridges.c 1:16.1.1~dfsg-1/main/stasis_bridges.c
--- 1:13.23.1~dfsg-2/main/stasis_bridges.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_bridges.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_cache_pattern.h"
@@ -238,14 +236,17 @@ struct ast_bridge_snapshot *ast_bridge_s
 	struct ast_bridge_snapshot *snapshot;
 	struct ast_bridge_channel *bridge_channel;
 
+	if (ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
+		return NULL;
+	}
+
 	snapshot = ao2_alloc_options(sizeof(*snapshot), bridge_snapshot_dtor,
 		AO2_ALLOC_OPT_LOCK_NOLOCK);
 	if (!snapshot) {
 		return NULL;
 	}
 
-	if (ast_string_field_init(snapshot, 128)
-		|| ast_string_field_init_extended(snapshot, video_source_id)) {
+	if (ast_string_field_init(snapshot, 128)) {
 		ao2_ref(snapshot, -1);
 
 		return NULL;
@@ -398,6 +399,8 @@ void ast_bridge_publish_merge(struct ast
 
 	ast_assert(to != NULL);
 	ast_assert(from != NULL);
+	ast_assert(ast_test_flag(&to->feature_flags, AST_BRIDGE_FLAG_INVISIBLE) == 0);
+	ast_assert(ast_test_flag(&from->feature_flags, AST_BRIDGE_FLAG_INVISIBLE) == 0);
 
 	merge_msg = bridge_merge_message_create(to, from);
 	if (!merge_msg) {
@@ -477,6 +480,10 @@ void ast_bridge_publish_enter(struct ast
 	struct stasis_message *msg;
 	struct ast_json *blob = NULL;
 
+	if (ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
+		return;
+	}
+
 	if (swap) {
 		blob = ast_json_pack("{s: s}", "swap", ast_channel_uniqueid(swap));
 		if (!blob) {
@@ -500,6 +507,9 @@ void ast_bridge_publish_leave(struct ast
 {
 	struct stasis_message *msg;
 
+	if (ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
+		return;
+	}
 	msg = ast_bridge_blob_create(ast_channel_left_bridge_type(), bridge, chan, NULL);
 	if (!msg) {
 		return;
diff -pruN 1:13.23.1~dfsg-2/main/stasis.c 1:16.1.1~dfsg-1/main/stasis.c
--- 1:13.23.1~dfsg-2/main/stasis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis_internal.h"
 #include "asterisk/stasis.h"
@@ -372,6 +370,11 @@ const char *stasis_topic_name(const stru
 	return topic->name;
 }
 
+size_t stasis_topic_subscribers(const struct stasis_topic *topic)
+{
+	return AST_VECTOR_SIZE(&topic->subscribers);
+}
+
 /*! \internal */
 struct stasis_subscription {
 	/*! Unique ID for this subscription */
@@ -393,6 +396,11 @@ struct stasis_subscription {
 	/*! Flag set when final message for sub has been processed.
 	 *  Be sure join_lock is held before reading/setting. */
 	int final_message_processed;
+
+	/*! The message types this subscription is accepting */
+	AST_VECTOR(, char) accepted_message_types;
+	/*! The message filter currently in use */
+	enum stasis_subscription_message_filter filter;
 };
 
 static void subscription_dtor(void *obj)
@@ -411,6 +419,8 @@ static void subscription_dtor(void *obj)
 	ast_taskprocessor_unreference(sub->mailbox);
 	sub->mailbox = NULL;
 	ast_cond_destroy(&sub->join_cond);
+
+	AST_VECTOR_FREE(&sub->accepted_message_types);
 }
 
 /*!
@@ -422,19 +432,25 @@ static void subscription_dtor(void *obj)
 static void subscription_invoke(struct stasis_subscription *sub,
 				  struct stasis_message *message)
 {
+	unsigned int final = stasis_subscription_final_message(sub, message);
+	int message_type_id = stasis_message_type_id(stasis_subscription_change_type());
+
 	/* Notify that the final message has been received */
-	if (stasis_subscription_final_message(sub, message)) {
+	if (final) {
 		ao2_lock(sub);
 		sub->final_message_rxed = 1;
 		ast_cond_signal(&sub->join_cond);
 		ao2_unlock(sub);
 	}
 
-	/* Since sub is mostly immutable, no need to lock sub */
-	sub->callback(sub->data, sub, message);
+	if (!final || sub->filter != STASIS_SUBSCRIPTION_FILTER_SELECTIVE ||
+		(message_type_id < AST_VECTOR_SIZE(&sub->accepted_message_types) && AST_VECTOR_GET(&sub->accepted_message_types, message_type_id))) {
+		/* Since sub is mostly immutable, no need to lock sub */
+		sub->callback(sub->data, sub, message);
+	}
 
 	/* Notify that the final message has been processed */
-	if (stasis_subscription_final_message(sub, message)) {
+	if (final) {
 		ao2_lock(sub);
 		sub->final_message_processed = 1;
 		ast_cond_signal(&sub->join_cond);
@@ -502,6 +518,8 @@ struct stasis_subscription *internal_sta
 	sub->callback = callback;
 	sub->data = data;
 	ast_cond_init(&sub->join_cond, NULL);
+	sub->filter = STASIS_SUBSCRIPTION_FILTER_NONE;
+	AST_VECTOR_INIT(&sub->accepted_message_types, 0);
 
 	if (topic_add_subscription(topic, sub) != 0) {
 		ao2_ref(sub, -1);
@@ -563,7 +581,10 @@ struct stasis_subscription *stasis_unsub
 
 	/* When all that's done, remove the ref the mailbox has on the sub */
 	if (sub->mailbox) {
-		ast_taskprocessor_push(sub->mailbox, sub_cleanup, sub);
+		if (ast_taskprocessor_push(sub->mailbox, sub_cleanup, sub)) {
+			/* Nothing we can do here, the conditional is just to keep
+			 * the compiler happy that we're not ignoring the result. */
+		}
 	}
 
 	/* Unsubscribing unrefs the subscription */
@@ -585,6 +606,76 @@ int stasis_subscription_set_congestion_l
 	return res;
 }
 
+int stasis_subscription_accept_message_type(struct stasis_subscription *subscription,
+	const struct stasis_message_type *type)
+{
+	if (!subscription) {
+		return -1;
+	}
+
+	ast_assert(type != NULL);
+	ast_assert(stasis_message_type_name(type) != NULL);
+
+	if (!type || !stasis_message_type_name(type)) {
+		/* Filtering is unreliable as this message type is not yet initialized
+		 * so force all messages through.
+		 */
+		subscription->filter = STASIS_SUBSCRIPTION_FILTER_FORCED_NONE;
+		return 0;
+	}
+
+	ao2_lock(subscription->topic);
+	if (AST_VECTOR_REPLACE(&subscription->accepted_message_types, stasis_message_type_id(type), 1)) {
+		/* We do this for the same reason as above. The subscription can still operate, so allow
+		 * it to do so by forcing all messages through.
+		 */
+		subscription->filter = STASIS_SUBSCRIPTION_FILTER_FORCED_NONE;
+	}
+	ao2_unlock(subscription->topic);
+
+	return 0;
+}
+
+int stasis_subscription_decline_message_type(struct stasis_subscription *subscription,
+	const struct stasis_message_type *type)
+{
+	if (!subscription) {
+		return -1;
+	}
+
+	ast_assert(type != NULL);
+	ast_assert(stasis_message_type_name(type) != NULL);
+
+	if (!type || !stasis_message_type_name(type)) {
+		return 0;
+	}
+
+	ao2_lock(subscription->topic);
+	if (stasis_message_type_id(type) < AST_VECTOR_SIZE(&subscription->accepted_message_types)) {
+		/* The memory is already allocated so this can't fail */
+		AST_VECTOR_REPLACE(&subscription->accepted_message_types, stasis_message_type_id(type), 0);
+	}
+	ao2_unlock(subscription->topic);
+
+	return 0;
+}
+
+int stasis_subscription_set_filter(struct stasis_subscription *subscription,
+	enum stasis_subscription_message_filter filter)
+{
+	if (!subscription) {
+		return -1;
+	}
+
+	ao2_lock(subscription->topic);
+	if (subscription->filter != STASIS_SUBSCRIPTION_FILTER_FORCED_NONE) {
+		subscription->filter = filter;
+	}
+	ao2_unlock(subscription->topic);
+
+	return 0;
+}
+
 void stasis_subscription_join(struct stasis_subscription *subscription)
 {
 	if (subscription) {
@@ -780,6 +871,18 @@ static void dispatch_message(struct stas
 	struct stasis_message *message,
 	int synchronous)
 {
+	/* Determine if this subscription is interested in this message. Note that final
+	 * messages are special and are always invoked on the subscription.
+	 */
+	if (sub->filter == STASIS_SUBSCRIPTION_FILTER_SELECTIVE) {
+		int message_type_id = stasis_message_type_id(stasis_message_type(message));
+		if ((message_type_id >= AST_VECTOR_SIZE(&sub->accepted_message_types) ||
+			!AST_VECTOR_GET(&sub->accepted_message_types, message_type_id)) &&
+			!stasis_subscription_final_message(sub, message)) {
+			return;
+		}
+	}
+
 	if (!sub->mailbox) {
 		/* Dispatch directly */
 		subscription_invoke(sub, message);
@@ -839,6 +942,11 @@ static void publish_msg(struct stasis_to
 	ast_assert(topic != NULL);
 	ast_assert(message != NULL);
 
+	/* If there are no subscribers don't bother */
+	if (!stasis_topic_subscribers(topic)) {
+		return;
+	}
+
 	/*
 	 * The topic may be unref'ed by the subscription invocation.
 	 * Make sure we hold onto a reference while dispatching.
@@ -969,22 +1077,23 @@ static void subscription_change_dtor(voi
 {
 	struct stasis_subscription_change *change = obj;
 
-	ast_string_field_free_memory(change);
 	ao2_cleanup(change->topic);
 }
 
 static struct stasis_subscription_change *subscription_change_alloc(struct stasis_topic *topic, const char *uniqueid, const char *description)
 {
+	size_t description_len = strlen(description) + 1;
 	struct stasis_subscription_change *change;
 
-	change = ao2_alloc(sizeof(struct stasis_subscription_change), subscription_change_dtor);
-	if (!change || ast_string_field_init(change, 128)) {
-		ao2_cleanup(change);
+	change = ao2_alloc_options(sizeof(*change) + description_len + strlen(uniqueid) + 1,
+		subscription_change_dtor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!change) {
 		return NULL;
 	}
 
-	ast_string_field_set(change, uniqueid, uniqueid);
-	ast_string_field_set(change, description, description);
+	strcpy(change->description, description); /* SAFE */
+	change->uniqueid = change->description + description_len;
+	strcpy(change->uniqueid, uniqueid); /* SAFE */
 	ao2_ref(topic, +1);
 	change->topic = topic;
 
@@ -1081,6 +1190,15 @@ static void topic_pool_dtor(void *obj)
 {
 	struct stasis_topic_pool *pool = obj;
 
+#ifdef AO2_DEBUG
+	{
+		char *container_name =
+			ast_alloca(strlen(stasis_topic_name(pool->pool_topic)) + strlen("-pool") + 1);
+		sprintf(container_name, "%s-pool", stasis_topic_name(pool->pool_topic));
+		ao2_container_unregister(container_name);
+	}
+#endif
+
 	ao2_cleanup(pool->pool_container);
 	pool->pool_container = NULL;
 	ao2_cleanup(pool->pool_topic);
@@ -1145,6 +1263,18 @@ static int topic_pool_entry_cmp(void *ob
 	return CMP_MATCH;
 }
 
+#ifdef AO2_DEBUG
+static void topic_pool_prnt_obj(void *v_obj, void *where, ao2_prnt_fn *prnt)
+{
+	struct topic_pool_entry *entry = v_obj;
+
+	if (!entry) {
+		return;
+	}
+	prnt(where, "%s", stasis_topic_name(entry->topic));
+}
+#endif
+
 struct stasis_topic_pool *stasis_topic_pool_create(struct stasis_topic *pooled_topic)
 {
 	struct stasis_topic_pool *pool;
@@ -1154,18 +1284,33 @@ struct stasis_topic_pool *stasis_topic_p
 		return NULL;
 	}
 
-	pool->pool_container = ao2_container_alloc(TOPIC_POOL_BUCKETS,
-		topic_pool_entry_hash, topic_pool_entry_cmp);
+	pool->pool_container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		TOPIC_POOL_BUCKETS, topic_pool_entry_hash, NULL, topic_pool_entry_cmp);
 	if (!pool->pool_container) {
 		ao2_cleanup(pool);
 		return NULL;
 	}
+
+#ifdef AO2_DEBUG
+	{
+		char *container_name =
+			ast_alloca(strlen(stasis_topic_name(pooled_topic)) + strlen("-pool") + 1);
+		sprintf(container_name, "%s-pool", stasis_topic_name(pooled_topic));
+		ao2_container_register(container_name, pool->pool_container, topic_pool_prnt_obj);
+	}
+#endif
+
 	ao2_ref(pooled_topic, +1);
 	pool->pool_topic = pooled_topic;
 
 	return pool;
 }
 
+void stasis_topic_pool_delete_topic(struct stasis_topic_pool *pool, const char *topic_name)
+{
+	ao2_find(pool->pool_container, topic_name, OBJ_SEARCH_KEY | OBJ_NODATA | OBJ_UNLINK);
+}
+
 struct stasis_topic *stasis_topic_pool_get_topic(struct stasis_topic_pool *pool, const char *topic_name)
 {
 	RAII_VAR(struct topic_pool_entry *, topic_pool_entry, NULL, ao2_cleanup);
@@ -1334,8 +1479,8 @@ static struct ast_json *multi_user_event
 
 	ast_json_object_set(out, "type", ast_json_string_create("ChannelUserevent"));
 	ast_json_object_set(out, "timestamp", ast_json_timeval(*tv, NULL));
-	ast_json_object_set(out, "eventname", ast_json_string_create(ast_json_string_get((ast_json_object_get(blob, "eventname")))));
-	ast_json_object_set(out, "userevent", ast_json_deep_copy(blob));
+	ast_json_object_set(out, "eventname", ast_json_ref(ast_json_object_get(blob, "eventname")));
+	ast_json_object_set(out, "userevent", ast_json_ref(blob));
 
 	for (type = 0; type < STASIS_UMOS_MAX; ++type) {
 		for (i = 0; i < AST_VECTOR_SIZE(&multi->snapshots[type]); ++i) {
diff -pruN 1:13.23.1~dfsg-2/main/stasis_cache.c 1:16.1.1~dfsg-1/main/stasis_cache.c
--- 1:13.23.1~dfsg-2/main/stasis_cache.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/hashtab.h"
 #include "asterisk/stasis_internal.h"
@@ -50,6 +48,7 @@ struct stasis_cache {
 	snapshot_get_id id_fn;
 	cache_aggregate_calc_fn aggregate_calc_fn;
 	cache_aggregate_publish_fn aggregate_publish_fn;
+	int registered;
 };
 
 /*! \internal */
@@ -71,6 +70,8 @@ static void stasis_caching_topic_dtor(vo
 	 * be bad. */
 	ast_assert(stasis_subscription_is_done(caching_topic->sub));
 
+	ao2_container_unregister(stasis_topic_name(caching_topic->topic));
+
 	ao2_cleanup(caching_topic->sub);
 	caching_topic->sub = NULL;
 	ao2_cleanup(caching_topic->cache);
@@ -86,6 +87,35 @@ struct stasis_topic *stasis_caching_get_
 	return caching_topic->topic;
 }
 
+int stasis_caching_accept_message_type(struct stasis_caching_topic *caching_topic,
+	struct stasis_message_type *type)
+{
+	int res;
+
+	if (!caching_topic) {
+		return -1;
+	}
+
+	/* We wait to accept the stasis specific message types until now so that by default everything
+	 * will flow to us.
+	 */
+	res = stasis_subscription_accept_message_type(caching_topic->sub, stasis_cache_clear_type());
+	res |= stasis_subscription_accept_message_type(caching_topic->sub, stasis_subscription_change_type());
+	res |= stasis_subscription_accept_message_type(caching_topic->sub, type);
+
+	return res;
+}
+
+int stasis_caching_set_filter(struct stasis_caching_topic *caching_topic,
+	enum stasis_subscription_message_filter filter)
+{
+	if (!caching_topic) {
+		return -1;
+	}
+	return stasis_subscription_set_filter(caching_topic->sub, filter);
+}
+
+
 struct stasis_caching_topic *stasis_caching_unsubscribe(struct stasis_caching_topic *caching_topic)
 {
 	if (!caching_topic) {
@@ -155,7 +185,6 @@ static void cache_entry_dtor(void *obj)
 	struct stasis_cache_entry *entry = obj;
 	size_t idx;
 
-	ao2_cleanup(entry->key.type);
 	entry->key.type = NULL;
 	ast_free((char *) entry->key.id);
 	entry->key.id = NULL;
@@ -203,7 +232,16 @@ static struct stasis_cache_entry *cache_
 		ao2_cleanup(entry);
 		return NULL;
 	}
-	entry->key.type = ao2_bump(type);
+	/*
+	 * Normal ao2 ref counting rules says we should increment the message
+	 * type ref here and decrement it in cache_entry_dtor().  However, the
+	 * stasis message snapshot is cached here, will always have the same type
+	 * as the cache entry, and can legitimately cause the type ref count to
+	 * hit the excessive ref count assertion.  Since the cache entry will
+	 * always have a snapshot we can get away with not holding a ref here.
+	 */
+	ast_assert(type == stasis_message_type(snapshot));
+	entry->key.type = type;
 	cache_entry_compute_hash(&entry->key);
 
 	is_remote = ast_eid_cmp(&ast_eid_default, stasis_message_eid(snapshot)) ? 1 : 0;
@@ -815,7 +853,31 @@ static void caching_topic_exec(void *dat
 	}
 
 	msg_type = stasis_message_type(message);
-	if (stasis_cache_clear_type() == msg_type) {
+
+	if (stasis_subscription_change_type() == msg_type) {
+		struct stasis_subscription_change *change = stasis_message_data(message);
+
+		/*
+		 * If this change type is an unsubscribe, we need to find the original
+		 * subscribe and remove it from the cache otherwise the cache will
+		 * continue to grow unabated.
+		 */
+		if (strcmp(change->description, "Unsubscribe") == 0) {
+			struct stasis_cache_entry *sub;
+
+			ao2_wrlock(caching_topic->cache->entries);
+			sub = cache_find(caching_topic->cache->entries, stasis_subscription_change_type(), change->uniqueid);
+			if (sub) {
+				cache_remove(caching_topic->cache->entries, sub, stasis_message_eid(message));
+				ao2_cleanup(sub);
+			}
+			ao2_unlock(caching_topic->cache->entries);
+			ao2_cleanup(caching_topic_needs_unref);
+			return;
+		}
+		msg_put = message;
+		msg = message;
+	} else if (stasis_cache_clear_type() == msg_type) {
 		/* Cache clear event. */
 		msg_put = NULL;
 		msg = stasis_message_data(message);
@@ -836,11 +898,13 @@ static void caching_topic_exec(void *dat
 		/* Update the cache */
 		snapshots = cache_put(caching_topic->cache, msg_type, msg_id, msg_eid, msg_put);
 		if (snapshots.old || msg_put) {
-			update = update_create(snapshots.old, msg_put);
-			if (update) {
-				stasis_publish(caching_topic->topic, update);
+			if (stasis_topic_subscribers(caching_topic->topic)) {
+				update = update_create(snapshots.old, msg_put);
+				if (update) {
+					stasis_publish(caching_topic->topic, update);
+					ao2_ref(update, -1);
+				}
 			}
-			ao2_cleanup(update);
 		} else {
 			ast_debug(1,
 				"Attempting to remove an item from the %s cache that isn't there: %s %s\n",
@@ -853,11 +917,13 @@ static void caching_topic_exec(void *dat
 				caching_topic->cache->aggregate_publish_fn(caching_topic->original_topic,
 					snapshots.aggregate_new);
 			}
-			update = update_create(snapshots.aggregate_old, snapshots.aggregate_new);
-			if (update) {
-				stasis_publish(caching_topic->topic, update);
+			if (stasis_topic_subscribers(caching_topic->topic)) {
+				update = update_create(snapshots.aggregate_old, snapshots.aggregate_new);
+				if (update) {
+					stasis_publish(caching_topic->topic, update);
+					ao2_ref(update, -1);
+				}
 			}
-			ao2_cleanup(update);
 		}
 
 		ao2_cleanup(snapshots.old);
@@ -868,6 +934,17 @@ static void caching_topic_exec(void *dat
 	ao2_cleanup(caching_topic_needs_unref);
 }
 
+static void print_cache_entry(void *v_obj, void *where, ao2_prnt_fn *prnt)
+{
+	struct stasis_cache_entry *entry = v_obj;
+
+	if (!entry) {
+		return;
+	}
+	prnt(where, "Type: %s  ID: %s  Hash: %u", stasis_message_type_name(entry->key.type),
+		entry->key.id, entry->key.hash);
+}
+
 struct stasis_caching_topic *stasis_caching_topic_create(struct stasis_topic *original_topic, struct stasis_cache *cache)
 {
 	struct stasis_caching_topic *caching_topic;
@@ -888,15 +965,24 @@ struct stasis_caching_topic *stasis_cach
 	}
 
 	caching_topic->topic = stasis_topic_create(new_name);
-	ast_free(new_name);
 	if (caching_topic->topic == NULL) {
 		ao2_ref(caching_topic, -1);
+		ast_free(new_name);
 
 		return NULL;
 	}
 
 	ao2_ref(cache, +1);
 	caching_topic->cache = cache;
+	if (!cache->registered) {
+		if (ao2_container_register(new_name, cache->entries, print_cache_entry)) {
+			ast_log(LOG_ERROR, "Stasis cache container '%p' for '%s' did not register\n",
+				cache->entries, new_name);
+		} else {
+			cache->registered = 1;
+		}
+	}
+	ast_free(new_name);
 
 	caching_topic->sub = internal_stasis_subscribe(original_topic, caching_topic_exec, caching_topic, 0, 0);
 	if (caching_topic->sub == NULL) {
diff -pruN 1:13.23.1~dfsg-2/main/stasis_cache_pattern.c 1:16.1.1~dfsg-1/main/stasis_cache_pattern.c
--- 1:13.23.1~dfsg-2/main/stasis_cache_pattern.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_cache_pattern.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis_cache_pattern.h"
 
@@ -219,3 +217,21 @@ struct stasis_topic *stasis_cp_single_to
 	}
 	return stasis_caching_get_topic(one->topic_cached);
 }
+
+int stasis_cp_single_accept_message_type(struct stasis_cp_single *one,
+	struct stasis_message_type *type)
+{
+	if (!one) {
+		return -1;
+	}
+	return stasis_caching_accept_message_type(one->topic_cached, type);
+}
+
+int stasis_cp_single_set_filter(struct stasis_cp_single *one,
+	enum stasis_subscription_message_filter filter)
+{
+	if (!one) {
+		return -1;
+	}
+	return stasis_caching_set_filter(one->topic_cached, filter);
+}
diff -pruN 1:13.23.1~dfsg-2/main/stasis_channels.c 1:16.1.1~dfsg-1/main/stasis_channels.c
--- 1:13.23.1~dfsg-2/main/stasis_channels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_channels.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/json.h"
 #include "asterisk/pbx.h"
@@ -227,6 +225,7 @@ static void channel_snapshot_dtor(void *
 
 	ast_string_field_free_memory(snapshot);
 	ao2_cleanup(snapshot->manager_vars);
+	ao2_cleanup(snapshot->ari_vars);
 }
 
 struct ast_channel_snapshot *ast_channel_snapshot_create(struct ast_channel *chan)
@@ -288,7 +287,9 @@ struct ast_channel_snapshot *ast_channel
 	ast_string_field_set(snapshot, language, ast_channel_language(chan));
 
 	if ((bridge = ast_channel_get_bridge(chan))) {
-		ast_string_field_set(snapshot, bridgeid, bridge->uniqueid);
+		if (!ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
+			ast_string_field_set(snapshot, bridgeid, bridge->uniqueid);
+		}
 		ao2_cleanup(bridge);
 	}
 
@@ -302,6 +303,7 @@ struct ast_channel_snapshot *ast_channel
 	ast_set_flag(&snapshot->softhangup_flags, ast_channel_softhangup_internal_flag(chan));
 
 	snapshot->manager_vars = ast_channel_get_manager_vars(chan);
+	snapshot->ari_vars = ast_channel_get_ari_vars(chan);
 	snapshot->tech_properties = ast_channel_tech(chan)->properties;
 
 	return snapshot;
@@ -400,7 +402,7 @@ static void ast_channel_publish_dial_int
 	msg = stasis_message_create(ast_channel_dial_type(), payload);
 	ao2_ref(payload, -1);
 	if (msg) {
-		publish_message_for_channel_topics(msg, caller);
+		publish_message_for_channel_topics(msg, caller ?: peer);
 		ao2_ref(msg, -1);
 	}
 }
@@ -613,8 +615,8 @@ struct ast_multi_channel_blob *ast_multi
 		return NULL;
 	}
 
-	obj->channel_snapshots = ao2_container_alloc(NUM_MULTI_CHANNEL_BLOB_BUCKETS,
-		channel_role_hash_cb, channel_role_cmp_cb);
+	obj->channel_snapshots = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NUM_MULTI_CHANNEL_BLOB_BUCKETS, channel_role_hash_cb, NULL, channel_role_cmp_cb);
 	if (!obj->channel_snapshots) {
 		ao2_ref(obj, -1);
 		return NULL;
@@ -717,8 +719,9 @@ struct ao2_container *ast_multi_channel_
 		return NULL;
 	}
 
-	ret_container = ao2_container_alloc(NUM_MULTI_CHANNEL_BLOB_BUCKETS,
-		channel_snapshot_hash_cb, channel_snapshot_cmp_cb);
+	ret_container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NUM_MULTI_CHANNEL_BLOB_BUCKETS,
+		channel_snapshot_hash_cb, NULL, channel_snapshot_cmp_cb);
 	if (!ret_container) {
 		return NULL;
 	}
@@ -992,6 +995,10 @@ struct ast_json *ast_channel_snapshot_to
 		"creationtime", ast_json_timeval(snapshot->creationtime, NULL),
 		"language", snapshot->language);
 
+	if (snapshot->ari_vars && !AST_LIST_EMPTY(snapshot->ari_vars)) {
+		ast_json_object_set(json_chan, "channelvars", ast_json_channel_vars(snapshot->ari_vars));
+	}
+
 	return json_chan;
 }
 
diff -pruN 1:13.23.1~dfsg-2/main/stasis_endpoints.c 1:16.1.1~dfsg-1/main/stasis_endpoints.c
--- 1:13.23.1~dfsg-2/main/stasis_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_endpoints.h"
@@ -84,7 +82,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 						<enum name="Unknown"/>
 						<enum name="Unreachable"/>
 						<enum name="Reachable"/>
-						<enum name="Created"/>
+						<enum name="Unqualified"/>
 						<enum name="Removed"/>
 						<enum name="Updated"/>
 					</enumlist>
diff -pruN 1:13.23.1~dfsg-2/main/stasis_message.c 1:16.1.1~dfsg-1/main/stasis_message.c
--- 1:13.23.1~dfsg-2/main/stasis_message.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_message.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis.h"
 #include "asterisk/utils.h"
@@ -41,9 +39,11 @@ struct stasis_message_type {
 	struct stasis_message_vtable *vtable;
 	char *name;
 	unsigned int hash;
+	int id;
 };
 
 static struct stasis_message_vtable null_vtable = {};
+static int message_type_id;
 
 static void message_type_dtor(void *obj)
 {
@@ -63,7 +63,8 @@ int stasis_message_type_create(const cha
 		return STASIS_MESSAGE_TYPE_DECLINED;
 	}
 
-	type = ao2_t_alloc(sizeof(*type), message_type_dtor, name);
+	type = ao2_t_alloc_options(sizeof(*type), message_type_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK, name ?: "");
 	if (!type) {
 		return STASIS_MESSAGE_TYPE_ERROR;
 	}
@@ -79,6 +80,7 @@ int stasis_message_type_create(const cha
 	}
 	type->hash = ast_hashtab_hash_string(name);
 	type->vtable = vtable;
+	type->id = ast_atomic_fetchadd_int(&message_type_id, +1);
 	*result = type;
 
 	return STASIS_MESSAGE_TYPE_SUCCESS;
@@ -94,6 +96,11 @@ unsigned int stasis_message_type_hash(co
 	return type->hash;
 }
 
+int stasis_message_type_id(const struct stasis_message_type *type)
+{
+	return type->id;
+}
+
 /*! \internal */
 struct stasis_message {
 	/*! Time the message was created */
@@ -111,7 +118,6 @@ struct stasis_message {
 static void stasis_message_dtor(void *obj)
 {
 	struct stasis_message *message = obj;
-	ao2_cleanup(message->type);
 	ao2_cleanup(message->data);
 }
 
@@ -123,13 +129,21 @@ struct stasis_message *stasis_message_cr
 		return NULL;
 	}
 
-	message = ao2_t_alloc(sizeof(*message), stasis_message_dtor, type->name);
+	message = ao2_t_alloc_options(sizeof(*message), stasis_message_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK, type->name);
 	if (message == NULL) {
 		return NULL;
 	}
 
 	message->timestamp = ast_tvnow();
-	ao2_ref(type, +1);
+	/*
+	 * XXX Normal ao2 ref counting rules says we should increment the message
+	 * type ref here and decrement it in stasis_message_dtor().  However, the
+	 * stasis message could be cached and legitimately cause the type ref count
+	 * to hit the excessive ref count assertion.  Since the message type
+	 * practically has to be a global object anyway, we can get away with not
+	 * holding a ref in the stasis message.
+	 */
 	message->type = type;
 	ao2_ref(data, +1);
 	message->data = data;
diff -pruN 1:13.23.1~dfsg-2/main/stasis_message_router.c 1:16.1.1~dfsg-1/main/stasis_message_router.c
--- 1:13.23.1~dfsg-2/main/stasis_message_router.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_message_router.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis_message_router.h"
 #include "asterisk/vector.h"
@@ -237,6 +235,9 @@ static struct stasis_message_router *sta
 		return NULL;
 	}
 
+	/* We need to receive subscription change messages so we know when our subscription goes away */
+	stasis_subscription_accept_message_type(router->subscription, stasis_subscription_change_type());
+
 	return router;
 }
 
@@ -318,6 +319,14 @@ int stasis_message_router_add(struct sta
 	}
 	ao2_lock(router);
 	res = route_table_add(&router->routes, message_type, callback, data);
+	if (!res) {
+		stasis_subscription_accept_message_type(router->subscription, message_type);
+		/* Until a specific message type was added we would already drop the message, so being
+		 * selective now doesn't harm us. If we have a default route then we are already forced
+		 * to filter nothing and messages will come in regardless.
+		 */
+		stasis_subscription_set_filter(router->subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
+	}
 	ao2_unlock(router);
 	return res;
 }
@@ -336,6 +345,10 @@ int stasis_message_router_add_cache_upda
 	}
 	ao2_lock(router);
 	res = route_table_add(&router->cache_routes, message_type, callback, data);
+	if (!res) {
+		stasis_subscription_accept_message_type(router->subscription, stasis_cache_update_type());
+		stasis_subscription_set_filter(router->subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
+	}
 	ao2_unlock(router);
 	return res;
 }
@@ -380,6 +393,9 @@ int stasis_message_router_set_default(st
 	router->default_route.callback = callback;
 	router->default_route.data = data;
 	ao2_unlock(router);
+
+	stasis_subscription_set_filter(router->subscription, STASIS_SUBSCRIPTION_FILTER_FORCED_NONE);
+
 	/* While this implementation can never fail, it used to be able to */
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/main/stasis_system.c 1:16.1.1~dfsg-1/main/stasis_system.c
--- 1:13.23.1~dfsg-2/main/stasis_system.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stasis_system.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_system.h"
diff -pruN 1:13.23.1~dfsg-2/main/stdtime/localtime.c 1:16.1.1~dfsg-1/main/stdtime/localtime.c
--- 1:13.23.1~dfsg-2/main/stdtime/localtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stdtime/localtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -50,8 +50,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 #include <sys/stat.h>
 #include <fcntl.h>
diff -pruN 1:13.23.1~dfsg-2/main/strcompat.c 1:16.1.1~dfsg-1/main/strcompat.c
--- 1:13.23.1~dfsg-2/main/strcompat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/strcompat.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,6 +25,7 @@
 	<support_level>core</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
 #include <ctype.h>
@@ -139,7 +140,7 @@ size_t strnlen(const char *s, size_t n)
 }
 #endif /* !HAVE_STRNLEN */
 
-#if !defined(HAVE_STRNDUP) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_STRNDUP)
 char *strndup(const char *s, size_t n)
 {
 	size_t len = strnlen(s, n);
@@ -151,9 +152,9 @@ char *strndup(const char *s, size_t n)
 	new[len] = '\0';
 	return memcpy(new, s, len);
 }
-#endif /* !defined(HAVE_STRNDUP) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_STRNDUP) */
 
-#if !defined(HAVE_VASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_VASPRINTF)
 int vasprintf(char **strp, const char *fmt, va_list ap)
 {
 	int size;
@@ -171,7 +172,7 @@ int vasprintf(char **strp, const char *f
 
 	return size;
 }
-#endif /* !defined(HAVE_VASPRINTF) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_VASPRINTF) */
 
 #ifndef HAVE_TIMERSUB
 void timersub(struct timeval *tvend, struct timeval *tvstart, struct timeval *tvdiff)
@@ -205,7 +206,7 @@ void timersub(struct timeval *tvend, str
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#if !defined(HAVE_ASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_ASPRINTF)
 int asprintf(char **str, const char *fmt, ...)
 {
 	va_list ap;
@@ -218,7 +219,7 @@ int asprintf(char **str, const char *fmt
 
 	return ret;
 }
-#endif /* !defined(HAVE_ASPRINTF) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_ASPRINTF) */
 
 #ifndef HAVE_STRTOQ
 #ifndef LONG_MIN
diff -pruN 1:13.23.1~dfsg-2/main/stream.c 1:16.1.1~dfsg-1/main/stream.c
--- 1:13.23.1~dfsg-2/main/stream.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stream.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,687 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*! \file
+ *
+ * \brief Media Stream API
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/logger.h"
+#include "asterisk/stream.h"
+#include "asterisk/strings.h"
+#include "asterisk/format.h"
+#include "asterisk/format_cap.h"
+#include "asterisk/vector.h"
+#include "asterisk/config.h"
+#include "asterisk/rtp_engine.h"
+
+struct ast_stream_metadata_entry {
+	size_t length;
+	int value_start;
+	char name_value[0];
+};
+
+struct ast_stream {
+	/*!
+	 * \brief The type of media the stream is handling
+	 */
+	enum ast_media_type type;
+
+	/*!
+	 * \brief The position of the stream in the topology
+	 */
+	unsigned int position;
+
+	/*!
+	 * \brief Current formats negotiated on the stream
+	 */
+	struct ast_format_cap *formats;
+
+	/*!
+	 * \brief The current state of the stream
+	 */
+	enum ast_stream_state state;
+
+	/*!
+	 * \brief Stream metadata vector
+	 */
+	struct ast_variable *metadata;
+
+	/*!
+	 * \brief The group that the stream is part of
+	 */
+	int group;
+
+	/*!
+	 * \brief The rtp_codecs used by the stream
+	 */
+	struct ast_rtp_codecs *rtp_codecs;
+
+	/*!
+	 * \brief Name for the stream within the context of the channel it is on
+	 */
+	char name[0];
+};
+
+struct ast_stream_topology {
+	/*!
+	 * \brief A vector of all the streams in this topology
+	 */
+	AST_VECTOR(, struct ast_stream *) streams;
+};
+
+struct ast_stream *ast_stream_alloc(const char *name, enum ast_media_type type)
+{
+	struct ast_stream *stream;
+
+	stream = ast_calloc(1, sizeof(*stream) + strlen(S_OR(name, "")) + 1);
+	if (!stream) {
+		return NULL;
+	}
+
+	stream->type = type;
+	stream->state = AST_STREAM_STATE_INACTIVE;
+	stream->group = -1;
+	strcpy(stream->name, S_OR(name, "")); /* Safe */
+
+	return stream;
+}
+
+struct ast_stream *ast_stream_clone(const struct ast_stream *stream, const char *name)
+{
+	struct ast_stream *new_stream;
+	size_t stream_size;
+	const char *stream_name;
+
+	if (!stream) {
+		return NULL;
+	}
+
+	stream_name = name ?: stream->name;
+	stream_size = sizeof(*stream) + strlen(stream_name) + 1;
+	new_stream = ast_calloc(1, stream_size);
+	if (!new_stream) {
+		return NULL;
+	}
+
+	memcpy(new_stream, stream, sizeof(*new_stream));
+	strcpy(new_stream->name, stream_name); /* Safe */
+	new_stream->group = -1;
+	if (new_stream->formats) {
+		ao2_ref(new_stream->formats, +1);
+	}
+
+	new_stream->metadata = ast_stream_get_metadata_list(stream);
+
+	/* rtp_codecs aren't cloned */
+
+	return new_stream;
+}
+
+void ast_stream_free(struct ast_stream *stream)
+{
+	if (!stream) {
+		return;
+	}
+
+	ast_variables_destroy(stream->metadata);
+
+	if (stream->rtp_codecs) {
+		ast_rtp_codecs_payloads_destroy(stream->rtp_codecs);
+	}
+
+	ao2_cleanup(stream->formats);
+	ast_free(stream);
+}
+
+const char *ast_stream_get_name(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->name;
+}
+
+enum ast_media_type ast_stream_get_type(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->type;
+}
+
+void ast_stream_set_type(struct ast_stream *stream, enum ast_media_type type)
+{
+	ast_assert(stream != NULL);
+
+	stream->type = type;
+}
+
+struct ast_format_cap *ast_stream_get_formats(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->formats;
+}
+
+void ast_stream_set_formats(struct ast_stream *stream, struct ast_format_cap *caps)
+{
+	ast_assert(stream != NULL);
+
+	ao2_cleanup(stream->formats);
+	stream->formats = ao2_bump(caps);
+}
+
+enum ast_stream_state ast_stream_get_state(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->state;
+}
+
+void ast_stream_set_state(struct ast_stream *stream, enum ast_stream_state state)
+{
+	ast_assert(stream != NULL);
+
+	stream->state = state;
+}
+
+const char *ast_stream_state2str(enum ast_stream_state state)
+{
+	switch (state) {
+	case AST_STREAM_STATE_REMOVED:
+		return "removed";
+	case AST_STREAM_STATE_SENDRECV:
+		return "sendrecv";
+	case AST_STREAM_STATE_SENDONLY:
+		return "sendonly";
+	case AST_STREAM_STATE_RECVONLY:
+		return "recvonly";
+	case AST_STREAM_STATE_INACTIVE:
+		return "inactive";
+	default:
+		return "<unknown>";
+	}
+}
+
+enum ast_stream_state ast_stream_str2state(const char *str)
+{
+	if (!strcmp("sendrecv", str)) {
+		return AST_STREAM_STATE_SENDRECV;
+	}
+	if (!strcmp("sendonly", str)) {
+		return AST_STREAM_STATE_SENDONLY;
+	}
+	if (!strcmp("recvonly", str)) {
+		return AST_STREAM_STATE_RECVONLY;
+	}
+	if (!strcmp("inactive", str)) {
+		return AST_STREAM_STATE_INACTIVE;
+	}
+	return AST_STREAM_STATE_REMOVED;
+}
+
+const char *ast_stream_get_metadata(const struct ast_stream *stream, const char *m_key)
+{
+	struct ast_variable *v;
+
+	ast_assert_return(stream != NULL, NULL);
+	ast_assert_return(m_key != NULL, NULL);
+
+	for (v = stream->metadata; v; v = v->next) {
+		if (strcmp(v->name, m_key) == 0) {
+			return v->value;
+		}
+	}
+
+	return NULL;
+}
+
+struct ast_variable *ast_stream_get_metadata_list(const struct ast_stream *stream)
+{
+	struct ast_variable *v;
+	struct ast_variable *vout = NULL;
+
+	ast_assert_return(stream != NULL, NULL);
+
+	for (v = stream->metadata; v; v = v->next) {
+		struct ast_variable *vt = ast_variable_new(v->name, v->value, "");
+
+		if (!vt) {
+			ast_variables_destroy(vout);
+			return NULL;
+		}
+
+		ast_variable_list_append(&vout, vt);
+	}
+
+	return vout;
+}
+
+int ast_stream_set_metadata(struct ast_stream *stream, const char *m_key, const char *value)
+{
+	struct ast_variable *v;
+	struct ast_variable *prev;
+
+	ast_assert_return(stream != NULL, -1);
+	ast_assert_return(m_key != NULL, -1);
+
+	prev = NULL;
+	v = stream->metadata;
+	while(v) {
+		struct ast_variable *next = v->next;
+		if (strcmp(v->name, m_key) == 0) {
+			if (prev) {
+				prev->next = next;
+			} else {
+				stream->metadata = next;
+			}
+			ast_free(v);
+			break;
+		} else {
+			prev = v;
+		}
+		v = next;
+	}
+
+	if (!value) {
+		return 0;
+	}
+
+	v = ast_variable_new(m_key, value, "");
+	if (!v) {
+		return -1;
+	}
+
+	ast_variable_list_append(&stream->metadata, v);
+
+	return 0;
+}
+
+int ast_stream_get_position(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->position;
+}
+
+struct ast_rtp_codecs *ast_stream_get_rtp_codecs(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->rtp_codecs;
+}
+
+void ast_stream_set_rtp_codecs(struct ast_stream *stream, struct ast_rtp_codecs *rtp_codecs)
+{
+	ast_assert(stream != NULL);
+
+	if (stream->rtp_codecs) {
+		ast_rtp_codecs_payloads_destroy(rtp_codecs);
+	}
+
+	stream->rtp_codecs = rtp_codecs;
+}
+
+static void stream_topology_destroy(void *data)
+{
+	struct ast_stream_topology *topology = data;
+
+	AST_VECTOR_CALLBACK_VOID(&topology->streams, ast_stream_free);
+	AST_VECTOR_FREE(&topology->streams);
+}
+
+#define TOPOLOGY_INITIAL_STREAM_COUNT 2
+struct ast_stream_topology *ast_stream_topology_alloc(void)
+{
+	struct ast_stream_topology *topology;
+
+	topology = ao2_alloc_options(sizeof(*topology), stream_topology_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!topology) {
+		return NULL;
+	}
+
+	if (AST_VECTOR_INIT(&topology->streams, TOPOLOGY_INITIAL_STREAM_COUNT)) {
+		ao2_ref(topology, -1);
+		topology = NULL;
+	}
+
+	return topology;
+}
+
+struct ast_stream_topology *ast_stream_topology_clone(
+	const struct ast_stream_topology *topology)
+{
+	struct ast_stream_topology *new_topology;
+	int i;
+
+	ast_assert(topology != NULL);
+
+	new_topology = ast_stream_topology_alloc();
+	if (!new_topology) {
+		return NULL;
+	}
+
+	for (i = 0; i < AST_VECTOR_SIZE(&topology->streams); i++) {
+		struct ast_stream *existing = AST_VECTOR_GET(&topology->streams, i);
+		struct ast_stream *stream = ast_stream_clone(existing, NULL);
+
+		if (!stream || AST_VECTOR_APPEND(&new_topology->streams, stream)) {
+			ast_stream_free(stream);
+			ast_stream_topology_free(new_topology);
+			return NULL;
+		}
+
+		ast_stream_set_group(stream, ast_stream_get_group(existing));
+	}
+
+	return new_topology;
+}
+
+int ast_stream_topology_equal(const struct ast_stream_topology *left,
+	const struct ast_stream_topology *right)
+{
+	int index;
+
+	ast_assert(left != NULL);
+	ast_assert(right != NULL);
+
+	if (ast_stream_topology_get_count(left) != ast_stream_topology_get_count(right)) {
+		return 0;
+	}
+
+	for (index = 0; index < ast_stream_topology_get_count(left); ++index) {
+		const struct ast_stream *left_stream = ast_stream_topology_get_stream(left, index);
+		const struct ast_stream *right_stream = ast_stream_topology_get_stream(right, index);
+
+		if (ast_stream_get_type(left_stream) != ast_stream_get_type(right_stream)) {
+			return 0;
+		}
+
+		if (ast_stream_get_state(left_stream) != ast_stream_get_state(right_stream)) {
+			return 0;
+		}
+
+		if (!ast_stream_get_formats(left_stream) && ast_stream_get_formats(right_stream) &&
+			ast_format_cap_count(ast_stream_get_formats(right_stream))) {
+			/* A NULL format capabilities and an empty format capabilities are the same, as they have
+			 * no formats inside. If one does though... they are not equal.
+			 */
+			return 0;
+		} else if (!ast_stream_get_formats(right_stream) && ast_stream_get_formats(left_stream) &&
+			ast_format_cap_count(ast_stream_get_formats(left_stream))) {
+			return 0;
+		} else if (ast_stream_get_formats(left_stream) && ast_stream_get_formats(right_stream) &&
+			!ast_format_cap_identical(ast_stream_get_formats(left_stream), ast_stream_get_formats(right_stream))) {
+			/* But if both are actually present we need to do an actual identical check. */
+			return 0;
+		}
+
+		if (strcmp(ast_stream_get_name(left_stream), ast_stream_get_name(right_stream))) {
+			return 0;
+		}
+	}
+
+	return 1;
+}
+
+void ast_stream_topology_free(struct ast_stream_topology *topology)
+{
+	ao2_cleanup(topology);
+}
+
+int ast_stream_topology_append_stream(struct ast_stream_topology *topology, struct ast_stream *stream)
+{
+	ast_assert(topology && stream);
+
+	if (AST_VECTOR_APPEND(&topology->streams, stream)) {
+		return -1;
+	}
+
+	stream->position = AST_VECTOR_SIZE(&topology->streams) - 1;
+
+	return AST_VECTOR_SIZE(&topology->streams) - 1;
+}
+
+int ast_stream_topology_get_count(const struct ast_stream_topology *topology)
+{
+	ast_assert(topology != NULL);
+
+	return AST_VECTOR_SIZE(&topology->streams);
+}
+
+struct ast_stream *ast_stream_topology_get_stream(
+	const struct ast_stream_topology *topology, unsigned int stream_num)
+{
+	ast_assert(topology != NULL);
+
+	return AST_VECTOR_GET(&topology->streams, stream_num);
+}
+
+int ast_stream_topology_set_stream(struct ast_stream_topology *topology,
+	unsigned int position, struct ast_stream *stream)
+{
+	struct ast_stream *existing_stream;
+
+	ast_assert(topology && stream);
+
+	if (position > AST_VECTOR_SIZE(&topology->streams)) {
+		return -1;
+	}
+
+	if (position < AST_VECTOR_SIZE(&topology->streams)) {
+		existing_stream = AST_VECTOR_GET(&topology->streams, position);
+		ast_stream_free(existing_stream);
+	}
+
+	stream->position = position;
+
+	if (position == AST_VECTOR_SIZE(&topology->streams)) {
+		return AST_VECTOR_APPEND(&topology->streams, stream);
+	}
+
+	return AST_VECTOR_REPLACE(&topology->streams, position, stream);
+}
+
+int ast_stream_topology_del_stream(struct ast_stream_topology *topology,
+	unsigned int position)
+{
+	struct ast_stream *stream;
+
+	ast_assert(topology != NULL);
+
+	if (AST_VECTOR_SIZE(&topology->streams) <= position) {
+		return -1;
+	}
+
+	stream = AST_VECTOR_REMOVE_ORDERED(&topology->streams, position);
+	ast_stream_free(stream);
+
+	/* Fix up higher stream position indices */
+	for (; position < AST_VECTOR_SIZE(&topology->streams); ++position) {
+		stream = AST_VECTOR_GET(&topology->streams, position);
+		stream->position = position;
+	}
+
+	return 0;
+}
+
+struct ast_stream_topology *ast_stream_topology_create_from_format_cap(
+	struct ast_format_cap *cap)
+{
+	struct ast_stream_topology *topology;
+	enum ast_media_type type;
+
+	topology = ast_stream_topology_alloc();
+	if (!topology || !cap || !ast_format_cap_count(cap)) {
+		return topology;
+	}
+
+	for (type = AST_MEDIA_TYPE_UNKNOWN + 1; type < AST_MEDIA_TYPE_END; type++) {
+		struct ast_format_cap *new_cap;
+		struct ast_stream *stream;
+
+		if (!ast_format_cap_has_type(cap, type)) {
+			continue;
+		}
+
+		new_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+		if (!new_cap) {
+			ast_stream_topology_free(topology);
+			return NULL;
+		}
+
+		ast_format_cap_set_framing(new_cap, ast_format_cap_get_framing(cap));
+		if (ast_format_cap_append_from_cap(new_cap, cap, type)) {
+			ao2_cleanup(new_cap);
+			ast_stream_topology_free(topology);
+			return NULL;
+		}
+
+		stream = ast_stream_alloc(ast_codec_media_type2str(type), type);
+		if (!stream) {
+			ao2_cleanup(new_cap);
+			ast_stream_topology_free(topology);
+			return NULL;
+		}
+		/* We're transferring the initial ref so no bump needed */
+		stream->formats = new_cap;
+		stream->state = AST_STREAM_STATE_SENDRECV;
+		if (ast_stream_topology_append_stream(topology, stream) == -1) {
+			ast_stream_free(stream);
+			ast_stream_topology_free(topology);
+			return NULL;
+		}
+	}
+
+	return topology;
+}
+
+struct ast_format_cap *ast_format_cap_from_stream_topology(
+    struct ast_stream_topology *topology)
+{
+	struct ast_format_cap *caps;
+	int i;
+
+	ast_assert(topology != NULL);
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		return NULL;
+	}
+
+	for (i = 0; i < AST_VECTOR_SIZE(&topology->streams); i++) {
+		struct ast_stream *stream;
+
+		stream = AST_VECTOR_GET(&topology->streams, i);
+		if (!stream->formats
+			|| stream->state == AST_STREAM_STATE_REMOVED) {
+			continue;
+		}
+
+		ast_format_cap_append_from_cap(caps, stream->formats, AST_MEDIA_TYPE_UNKNOWN);
+	}
+
+	return caps;
+}
+
+struct ast_stream *ast_stream_topology_get_first_stream_by_type(
+	const struct ast_stream_topology *topology,
+	enum ast_media_type type)
+{
+	int i;
+
+	ast_assert(topology != NULL);
+
+	for (i = 0; i < AST_VECTOR_SIZE(&topology->streams); i++) {
+		struct ast_stream *stream;
+
+		stream = AST_VECTOR_GET(&topology->streams, i);
+		if (stream->type == type
+			&& stream->state != AST_STREAM_STATE_REMOVED) {
+			return stream;
+		}
+	}
+
+	return NULL;
+}
+
+void ast_stream_topology_map(const struct ast_stream_topology *topology,
+	struct ast_vector_int *types, struct ast_vector_int *v0, struct ast_vector_int *v1)
+{
+	int i;
+	int nths[AST_MEDIA_TYPE_END] = {0};
+	int size = ast_stream_topology_get_count(topology);
+
+	/*
+	 * Clear out any old mappings and initialize the new ones
+	 */
+	AST_VECTOR_FREE(v0);
+	AST_VECTOR_FREE(v1);
+
+	/*
+	 * Both vectors are sized to the topology. The media types vector is always
+	 * guaranteed to be the size of the given topology or greater.
+	 */
+	AST_VECTOR_INIT(v0, size);
+	AST_VECTOR_INIT(v1, size);
+
+	for (i = 0; i < size; ++i) {
+		struct ast_stream *stream = ast_stream_topology_get_stream(topology, i);
+		enum ast_media_type type = ast_stream_get_type(stream);
+		int index = AST_VECTOR_GET_INDEX_NTH(types, ++nths[type],
+			type, AST_VECTOR_ELEM_DEFAULT_CMP);
+
+		if (index == -1) {
+			/*
+			 * If a given type is not found for an index level then update the
+			 * media types vector with that type. This keeps the media types
+			 * vector always at the max topology size.
+			 */
+			AST_VECTOR_APPEND(types, type);
+			index = AST_VECTOR_SIZE(types) - 1;
+		}
+
+		/*
+		 * The mapping is reflexive in the sense that if it maps in one direction
+		 * then the reverse direction maps back to the other's index.
+		 */
+		AST_VECTOR_REPLACE(v0, i, index);
+		AST_VECTOR_REPLACE(v1, index, i);
+	}
+}
+
+int ast_stream_get_group(const struct ast_stream *stream)
+{
+	ast_assert(stream != NULL);
+
+	return stream->group;
+}
+
+void ast_stream_set_group(struct ast_stream *stream, int group)
+{
+	ast_assert(stream != NULL);
+
+	stream->group = group;
+}
diff -pruN 1:13.23.1~dfsg-2/main/stringfields.c 1:16.1.1~dfsg-1/main/stringfields.c
--- 1:13.23.1~dfsg-2/main/stringfields.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stringfields.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/stringfields.h"
 #include "asterisk/utils.h"
 
@@ -57,16 +55,6 @@ static size_t optimal_alloc_size(size_t
 	return (1 << count) - ALLOCATOR_OVERHEAD;
 }
 
-static void *calloc_wrapper(unsigned int num_structs, size_t struct_size,
-	const char *file, int lineno, const char *func)
-{
-#if defined(__AST_DEBUG_MALLOC)
-	return __ast_calloc(num_structs, struct_size, file, lineno, func);
-#else
-	return ast_calloc(num_structs, struct_size);
-#endif
-}
-
 /*! \brief add a new block to the pool.
  * We can only allocate from the topmost pool, so the
  * fields in *mgr reflect the size of that only.
@@ -77,7 +65,8 @@ static int add_string_pool(struct ast_st
 	struct ast_string_field_pool *pool;
 	size_t alloc_size = optimal_alloc_size(sizeof(*pool) + size);
 
-	if (!(pool = calloc_wrapper(1, alloc_size, file, lineno, func))) {
+	pool = __ast_calloc(1, alloc_size, file, lineno, func);
+	if (!pool) {
 		return -1;
 	}
 
@@ -118,29 +107,23 @@ int __ast_string_field_free_memory(struc
 	struct ast_string_field_pool *cur = NULL;
 	struct ast_string_field_pool *preserve = NULL;
 
-	if (!mgr->header) {
-		return -1;
-	}
-
 	/* reset all the fields regardless of cleanup type */
-	AST_VECTOR_CALLBACK_VOID(&mgr->header->string_fields, reset_field);
+	AST_VECTOR_CALLBACK_VOID(&mgr->string_fields, reset_field);
 
 	switch (cleanup_type) {
 	case AST_STRINGFIELD_DESTROY:
-		AST_VECTOR_FREE(&mgr->header->string_fields);
+		AST_VECTOR_FREE(&mgr->string_fields);
 
-		if (mgr->header->embedded_pool) { /* ALWAYS preserve the embedded pool if there is one */
-			preserve = mgr->header->embedded_pool;
+		if (mgr->embedded_pool) { /* ALWAYS preserve the embedded pool if there is one */
+			preserve = mgr->embedded_pool;
 			preserve->used = preserve->active = 0;
 		}
 
-		ast_free(mgr->header);
-		mgr->header = NULL;
 		break;
 	case AST_STRINGFIELD_RESET:
 		/* Preserve the embedded pool if there is one, otherwise the last pool */
-		if (mgr->header->embedded_pool) {
-			preserve = mgr->header->embedded_pool;
+		if (mgr->embedded_pool) {
+			preserve = mgr->embedded_pool;
 		} else {
 			if (*pool_head == NULL) {
 				ast_log(LOG_WARNING, "trying to reset empty pool\n");
@@ -196,33 +179,20 @@ int __ast_string_field_init(struct ast_s
 	}
 
 	mgr->last_alloc = NULL;
-#if defined(__AST_DEBUG_MALLOC)
-	mgr->owner_file = file;
-	mgr->owner_func = func;
-	mgr->owner_line = lineno;
-#endif
-
-	if (!(mgr->header = calloc_wrapper(1, sizeof(*mgr->header), file, lineno, func))) {
-		return -1;
-	}
 
-	if (AST_VECTOR_INIT(&mgr->header->string_fields, initial_vector_size)) {
-		ast_free(mgr->header);
-		mgr->header = NULL;
+	if (AST_VECTOR_INIT(&mgr->string_fields, initial_vector_size)) {
 		return -1;
 	}
 
 	while ((struct ast_string_field_mgr *) p != mgr) {
-		AST_VECTOR_APPEND(&mgr->header->string_fields, p);
+		AST_VECTOR_APPEND(&mgr->string_fields, p);
 		*p++ = __ast_string_field_empty;
 	}
 
 	*pool_head = NULL;
-	mgr->header->embedded_pool = NULL;
+	mgr->embedded_pool = NULL;
 	if (add_string_pool(mgr, pool_head, needed, file, lineno, func)) {
-		AST_VECTOR_FREE(&mgr->header->string_fields);
-		ast_free(mgr->header);
-		mgr->header = NULL;
+		AST_VECTOR_FREE(&mgr->string_fields);
 		return -1;
 	}
 
@@ -230,7 +200,8 @@ int __ast_string_field_init(struct ast_s
 }
 
 ast_string_field __ast_string_field_alloc_space(struct ast_string_field_mgr *mgr,
-	struct ast_string_field_pool **pool_head, size_t needed)
+	struct ast_string_field_pool **pool_head, size_t needed,
+	const char *file, int lineno, const char *func)
 {
 	char *result = NULL;
 	size_t space = (*pool_head)->size - (*pool_head)->used;
@@ -247,13 +218,9 @@ ast_string_field __ast_string_field_allo
 			new_size *= 2;
 		}
 
-#if defined(__AST_DEBUG_MALLOC)
-		if (add_string_pool(mgr, pool_head, new_size, mgr->owner_file, mgr->owner_line, mgr->owner_func))
-			return NULL;
-#else
-		if (add_string_pool(mgr, pool_head, new_size, __FILE__, __LINE__, __FUNCTION__))
+		if (add_string_pool(mgr, pool_head, new_size, file, lineno, func)) {
 			return NULL;
-#endif
+		}
 	}
 
 	/* pool->base is always aligned (gcc aligned attribute). We ensure that
@@ -318,7 +285,8 @@ void __ast_string_field_release_active(s
 
 void __ast_string_field_ptr_build_va(struct ast_string_field_mgr *mgr,
 	struct ast_string_field_pool **pool_head, ast_string_field *ptr,
-	const char *format, va_list ap)
+	const char *format, va_list ap,
+	const char *file, int lineno, const char *func)
 {
 	size_t needed;
 	size_t available;
@@ -370,7 +338,8 @@ void __ast_string_field_ptr_build_va(str
 		   (if it has one), or the space available in the pool (if it does not). allocate
 		   space for it, adding a new string pool if necessary.
 		*/
-		if (!(target = (char *) __ast_string_field_alloc_space(mgr, pool_head, needed))) {
+		target = (char *) __ast_string_field_alloc_space(mgr, pool_head, needed, file, lineno, func);
+		if (!target) {
 			return;
 		}
 		vsprintf(target, format, ap);
@@ -397,19 +366,20 @@ void __ast_string_field_ptr_build_va(str
 	}
 }
 
-void __ast_string_field_ptr_build(struct ast_string_field_mgr *mgr,
+void __ast_string_field_ptr_build(const char *file, int lineno, const char *func,
+	struct ast_string_field_mgr *mgr,
 	struct ast_string_field_pool **pool_head, ast_string_field *ptr, const char *format, ...)
 {
 	va_list ap;
 
 	va_start(ap, format);
-	__ast_string_field_ptr_build_va(mgr, pool_head, ptr, format, ap);
+	__ast_string_field_ptr_build_va(mgr, pool_head, ptr, format, ap, file, lineno, func);
 	va_end(ap);
 }
 
 void *__ast_calloc_with_stringfields(unsigned int num_structs, size_t struct_size,
-	size_t field_mgr_offset, size_t field_mgr_pool_offset, size_t pool_size, const char *file,
-	int lineno, const char *func)
+	size_t field_mgr_offset, size_t field_mgr_pool_offset, size_t pool_size,
+	const char *file, int lineno, const char *func)
 {
 	struct ast_string_field_mgr *mgr;
 	struct ast_string_field_pool *pool;
@@ -422,46 +392,31 @@ void *__ast_calloc_with_stringfields(uns
 
 	ast_assert(num_structs == 1);
 
-	if (!(allocation = calloc_wrapper(num_structs, size_to_alloc, file, lineno, func))) {
+	allocation = __ast_calloc(num_structs, size_to_alloc, file, lineno, func);
+	if (!allocation) {
 		return NULL;
 	}
 
 	mgr = allocation + field_mgr_offset;
 
-	/*
-	 * The header is calloced in __ast_string_field_init so it also gets calloced here
-	 * so __ast_string_fields_free_memory can always just free mgr->header.
-	 */
-	mgr->header = calloc_wrapper(1, sizeof(*mgr->header), file, lineno, func);
-	if (!mgr->header) {
-		ast_free(allocation);
-		return NULL;
-	}
-
 	pool = allocation + struct_size;
 	pool_head = allocation + field_mgr_pool_offset;
 	p = (const char **) pool_head + 1;
 	initial_vector_size = ((size_t) (((char *)mgr) - ((char *)p))) / sizeof(*p);
 
-	if (AST_VECTOR_INIT(&mgr->header->string_fields, initial_vector_size)) {
-		ast_free(mgr->header);
+	if (AST_VECTOR_INIT(&mgr->string_fields, initial_vector_size)) {
 		ast_free(allocation);
 		return NULL;
 	}
 
 	while ((struct ast_string_field_mgr *) p != mgr) {
-		AST_VECTOR_APPEND(&mgr->header->string_fields, p);
+		AST_VECTOR_APPEND(&mgr->string_fields, p);
 		*p++ = __ast_string_field_empty;
 	}
 
-	mgr->header->embedded_pool = pool;
+	mgr->embedded_pool = pool;
 	*pool_head = pool;
 	pool->size = size_to_alloc - struct_size - sizeof(*pool);
-#if defined(__AST_DEBUG_MALLOC)
-		mgr->owner_file = file;
-		mgr->owner_func = func;
-		mgr->owner_line = lineno;
-#endif
 
 	return allocation;
 }
@@ -484,11 +439,12 @@ int __ast_string_fields_cmp(struct ast_s
 }
 
 int __ast_string_fields_copy(struct ast_string_field_pool *copy_pool,
-	struct ast_string_field_mgr *copy_mgr, struct ast_string_field_mgr *orig_mgr)
+	struct ast_string_field_mgr *copy_mgr, struct ast_string_field_mgr *orig_mgr,
+	const char *file, int lineno, const char *func)
 {
 	int i;
-	struct ast_string_field_vector *dest = &(copy_mgr->header->string_fields);
-	struct ast_string_field_vector *src = &(orig_mgr->header->string_fields);
+	struct ast_string_field_vector *dest = &(copy_mgr->string_fields);
+	struct ast_string_field_vector *src = &(orig_mgr->string_fields);
 
 	ast_assert(AST_VECTOR_SIZE(dest) == AST_VECTOR_SIZE(src));
 
@@ -498,8 +454,8 @@ int __ast_string_fields_copy(struct ast_
 	}
 
 	for (i = 0; i < AST_VECTOR_SIZE(dest); i++) {
-		if (ast_string_field_ptr_set_by_fields(copy_pool, *copy_mgr, AST_VECTOR_GET(dest, i),
-			*AST_VECTOR_GET(src, i))) {
+		if (__ast_string_field_ptr_set_by_fields(copy_pool, *copy_mgr, AST_VECTOR_GET(dest, i),
+			*AST_VECTOR_GET(src, i), file, lineno, func)) {
 			return -1;
 		}
 	}
diff -pruN 1:13.23.1~dfsg-2/main/strings.c 1:16.1.1~dfsg-1/main/strings.c
--- 1:13.23.1~dfsg-2/main/strings.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/strings.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,11 +37,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 #include "asterisk/strings.h"
 #include "asterisk/pbx.h"
+#include "asterisk/vector.h"
 
 /*!
  * core handler for dynamic strings.
@@ -53,13 +52,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
  *	ast_str_append_va(...)
  */
 
-#ifdef __AST_DEBUG_MALLOC
-int __ast_debug_str_helper(struct ast_str **buf, ssize_t max_len,
-	int append, const char *fmt, va_list ap, const char *file, int lineno, const char *function)
-#else
 int __ast_str_helper(struct ast_str **buf, ssize_t max_len,
-	int append, const char *fmt, va_list ap)
-#endif
+	int append, const char *fmt, va_list ap,
+	const char *file, int lineno, const char *function)
 {
 	int res;
 	int added;
@@ -111,13 +106,7 @@ int __ast_str_helper(struct ast_str **bu
 			need = max_len;
 		}
 
-		if (
-#ifdef __AST_DEBUG_MALLOC
-			_ast_str_make_space(buf, need, file, lineno, function)
-#else
-			ast_str_make_space(buf, need)
-#endif
-			) {
+		if (_ast_str_make_space(buf, need, file, lineno, function)) {
 			ast_log_safe(LOG_VERBOSE, "failed to extend from %d to %d\n",
 				(int) (*buf)->__AST_STR_LEN, need);
 
@@ -391,3 +380,44 @@ char *ast_read_line_from_buffer(char **b
 
 	return start;
 }
+
+int ast_vector_string_split(struct ast_vector_string *dest,
+	const char *input, const char *delim, int flags,
+	int (*excludes_cmp)(const char *s1, const char *s2))
+{
+	char *buf;
+	char *cur;
+	int no_trim = flags & AST_VECTOR_STRING_SPLIT_NO_TRIM;
+	int allow_empty = flags & AST_VECTOR_STRING_SPLIT_ALLOW_EMPTY;
+
+	ast_assert(dest != NULL);
+	ast_assert(!ast_strlen_zero(delim));
+
+	if (ast_strlen_zero(input)) {
+		return 0;
+	}
+
+	buf = ast_strdupa(input);
+	while ((cur = strsep(&buf, delim))) {
+		if (!no_trim) {
+			cur = ast_strip(cur);
+		}
+
+		if (!allow_empty && ast_strlen_zero(cur)) {
+			continue;
+		}
+
+		if (excludes_cmp && AST_VECTOR_GET_CMP(dest, cur, !excludes_cmp)) {
+			continue;
+		}
+
+		cur = ast_strdup(cur);
+		if (!cur || AST_VECTOR_APPEND(dest, cur)) {
+			ast_free(cur);
+
+			return -1;
+		}
+	}
+
+	return 0;
+}
diff -pruN 1:13.23.1~dfsg-2/main/stun.c 1:16.1.1~dfsg-1/main/stun.c
--- 1:13.23.1~dfsg-2/main/stun.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/stun.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/stun.h"
 #include "asterisk/cli.h"
diff -pruN 1:13.23.1~dfsg-2/main/syslog.c 1:16.1.1~dfsg-1/main/syslog.c
--- 1:13.23.1~dfsg-2/main/syslog.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/syslog.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 #include "asterisk/utils.h"
 #include "asterisk/syslog.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <syslog.h>
 
 static const struct {
diff -pruN 1:13.23.1~dfsg-2/main/taskprocessor.c 1:16.1.1~dfsg-1/main/taskprocessor.c
--- 1:13.23.1~dfsg-2/main/taskprocessor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/taskprocessor.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/module.h"
 #include "asterisk/time.h"
@@ -69,10 +67,8 @@ struct tps_taskprocessor_stats {
 
 /*! \brief A ast_taskprocessor structure is a singleton by name */
 struct ast_taskprocessor {
-	/*! \brief Friendly name of the taskprocessor */
-	const char *name;
 	/*! \brief Taskprocessor statistics */
-	struct tps_taskprocessor_stats *stats;
+	struct tps_taskprocessor_stats stats;
 	void *local_data;
 	/*! \brief Taskprocessor current queue size */
 	long tps_queue_size;
@@ -93,6 +89,8 @@ struct ast_taskprocessor {
 	unsigned int high_water_alert:1;
 	/*! Indicates if the taskprocessor is currently suspended */
 	unsigned int suspended:1;
+	/*! \brief Friendly name of the taskprocessor */
+	char name[0];
 };
 
 /*!
@@ -237,7 +235,11 @@ static void default_listener_shutdown(st
 	/* Hold a reference during shutdown */
 	ao2_t_ref(listener->tps, +1, "tps-shutdown");
 
-	ast_taskprocessor_push(listener->tps, default_listener_die, pvt);
+	if (ast_taskprocessor_push(listener->tps, default_listener_die, pvt)) {
+		/* This will cause the thread to exit early without completing tasks already
+		 * in the queue.  This is probably the least bad option in this situation. */
+		default_listener_die(pvt);
+	}
 
 	ast_assert(pvt->poll_thread != AST_PTHREADT_NULL);
 
@@ -276,7 +278,9 @@ static void tps_shutdown(void)
 /* initialize the taskprocessor container and register CLI operations */
 int ast_tps_init(void)
 {
-	if (!(tps_singletons = ao2_container_alloc(TPS_MAX_BUCKETS, tps_hash_cb, tps_cmp_cb))) {
+	tps_singletons = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		TPS_MAX_BUCKETS, tps_hash_cb, NULL, tps_cmp_cb);
+	if (!tps_singletons) {
 		ast_log(LOG_ERROR, "taskprocessor container failed to initialize!\n");
 		return -1;
 	}
@@ -513,13 +517,8 @@ static char *cli_tps_report(struct ast_c
 	while ((tps = ao2_iterator_next(&iter))) {
 		ast_copy_string(name, tps->name, sizeof(name));
 		qsize = tps->tps_queue_size;
-		if (tps->stats) {
-			maxqsize = tps->stats->max_qsize;
-			processed = tps->stats->_tasks_processed_count;
-		} else {
-			maxqsize = 0;
-			processed = 0;
-		}
+		maxqsize = tps->stats.max_qsize;
+		processed = tps->stats._tasks_processed_count;
 		ast_cli(a->fd, FMT_FIELDS, name, processed, qsize, maxqsize,
 			tps->tps_queue_low, tps->tps_queue_high);
 		ast_taskprocessor_unreference(tps);
@@ -643,10 +642,6 @@ static void tps_taskprocessor_dtor(void
 		tps_alert_add(t, -1);
 	}
 
-	ast_free(t->stats);
-	t->stats = NULL;
-	ast_free((char *) t->name);
-	t->name = NULL;
 	ao2_cleanup(t->listener);
 	t->listener = NULL;
 }
@@ -738,11 +733,22 @@ static void *default_listener_pvt_alloc(
 	return pvt;
 }
 
+/*!
+ * \internal
+ * \brief Allocate a task processor structure
+ *
+ * \param name Name of the task processor.
+ * \param listener Listener to associate with the task processor.
+ *
+ * \return The newly allocated task processor.
+ *
+ * \pre tps_singletons must be locked by the caller.
+ */
 static struct ast_taskprocessor *__allocate_taskprocessor(const char *name, struct ast_taskprocessor_listener *listener)
 {
 	struct ast_taskprocessor *p;
 
-	p = ao2_alloc(sizeof(*p), tps_taskprocessor_dtor);
+	p = ao2_alloc(sizeof(*p) + strlen(name) + 1, tps_taskprocessor_dtor);
 	if (!p) {
 		ast_log(LOG_WARNING, "failed to create taskprocessor '%s'\n", name);
 		return NULL;
@@ -752,12 +758,7 @@ static struct ast_taskprocessor *__alloc
 	p->tps_queue_low = (AST_TASKPROCESSOR_HIGH_WATER_LEVEL * 9) / 10;
 	p->tps_queue_high = AST_TASKPROCESSOR_HIGH_WATER_LEVEL;
 
-	p->stats = ast_calloc(1, sizeof(*p->stats));
-	p->name = ast_strdup(name);
-	if (!p->stats || !p->name) {
-		ao2_ref(p, -1);
-		return NULL;
-	}
+	strcpy(p->name, name); /*SAFE*/
 
 	ao2_ref(listener, +1);
 	p->listener = listener;
@@ -767,17 +768,23 @@ static struct ast_taskprocessor *__alloc
 	ao2_ref(p, +1);
 	listener->tps = p;
 
-	if (!(ao2_link(tps_singletons, p))) {
+	if (!(ao2_link_flags(tps_singletons, p, OBJ_NOLOCK))) {
 		ast_log(LOG_ERROR, "Failed to add taskprocessor '%s' to container\n", p->name);
 		listener->tps = NULL;
 		ao2_ref(p, -2);
 		return NULL;
 	}
 
-	if (p->listener->callbacks->start(p->listener)) {
+	return p;
+}
+
+static struct ast_taskprocessor *__start_taskprocessor(struct ast_taskprocessor *p)
+{
+	if (p && p->listener->callbacks->start(p->listener)) {
 		ast_log(LOG_ERROR, "Unable to start taskprocessor listener for taskprocessor %s\n",
 			p->name);
 		ast_taskprocessor_unreference(p);
+
 		return NULL;
 	}
 
@@ -797,40 +804,51 @@ struct ast_taskprocessor *ast_taskproces
 		ast_log(LOG_ERROR, "requesting a nameless taskprocessor!!!\n");
 		return NULL;
 	}
-	p = ao2_find(tps_singletons, name, OBJ_KEY);
-	if (p) {
-		return p;
-	}
-	if (create & TPS_REF_IF_EXISTS) {
+	ao2_lock(tps_singletons);
+	p = ao2_find(tps_singletons, name, OBJ_KEY | OBJ_NOLOCK);
+	if (p || (create & TPS_REF_IF_EXISTS)) {
 		/* calling function does not want a new taskprocessor to be created if it doesn't already exist */
-		return NULL;
+		ao2_unlock(tps_singletons);
+		return p;
 	}
+
 	/* Create a new taskprocessor. Start by creating a default listener */
 	pvt = default_listener_pvt_alloc();
 	if (!pvt) {
+		ao2_unlock(tps_singletons);
 		return NULL;
 	}
 	listener = ast_taskprocessor_listener_alloc(&default_listener_callbacks, pvt);
 	if (!listener) {
+		ao2_unlock(tps_singletons);
 		default_listener_pvt_destroy(pvt);
 		return NULL;
 	}
 
 	p = __allocate_taskprocessor(name, listener);
-
+	ao2_unlock(tps_singletons);
+	p = __start_taskprocessor(p);
 	ao2_ref(listener, -1);
+
 	return p;
 }
 
 struct ast_taskprocessor *ast_taskprocessor_create_with_listener(const char *name, struct ast_taskprocessor_listener *listener)
 {
-	struct ast_taskprocessor *p = ao2_find(tps_singletons, name, OBJ_KEY);
+	struct ast_taskprocessor *p;
 
+	ao2_lock(tps_singletons);
+	p = ao2_find(tps_singletons, name, OBJ_KEY | OBJ_NOLOCK);
 	if (p) {
+		ao2_unlock(tps_singletons);
 		ast_taskprocessor_unreference(p);
 		return NULL;
 	}
-	return __allocate_taskprocessor(name, listener);
+
+	p = __allocate_taskprocessor(name, listener);
+	ao2_unlock(tps_singletons);
+
+	return __start_taskprocessor(p);
 }
 
 void ast_taskprocessor_set_local(struct ast_taskprocessor *tps,
@@ -983,13 +1001,11 @@ int ast_taskprocessor_execute(struct ast
 	size = ast_taskprocessor_size(tps);
 
 	/* Update the stats */
-	if (tps->stats) {
-		++tps->stats->_tasks_processed_count;
+	++tps->stats._tasks_processed_count;
 
-		/* Include the task we just executed as part of the queue size. */
-		if (size >= tps->stats->max_qsize) {
-			tps->stats->max_qsize = size + 1;
-		}
+	/* Include the task we just executed as part of the queue size. */
+	if (size >= tps->stats.max_qsize) {
+		tps->stats.max_qsize = size + 1;
 	}
 	ao2_unlock(tps);
 
diff -pruN 1:13.23.1~dfsg-2/main/tcptls.c 1:16.1.1~dfsg-1/main/tcptls.c
--- 1:13.23.1~dfsg-2/main/tcptls.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/tcptls.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,9 +27,8 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/tcptls.h"            /* for ast_tls_config, ast_tcptls_se... */
+#include "asterisk/iostream.h"          /* for DO_SSL, ast_iostream_close, a... */
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>                      /* for O_NONBLOCK */
@@ -38,7 +37,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #ifdef DO_SSL
 #include <openssl/asn1.h>               /* for ASN1_STRING_to_UTF8 */
 #include <openssl/crypto.h>             /* for OPENSSL_free */
-#include <openssl/err.h>                /* for ERR_error_string */
 #include <openssl/opensslconf.h>        /* for OPENSSL_NO_SSL3_METHOD, OPENS... */
 #include <openssl/opensslv.h>           /* for OPENSSL_VERSION_NUMBER */
 #include <openssl/safestack.h>          /* for STACK_OF */
@@ -58,573 +56,25 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <signal.h>                     /* for pthread_kill, SIGURG */
 #include <sys/socket.h>                 /* for setsockopt, shutdown, socket */
 #include <sys/stat.h>                   /* for stat */
-#include <sys/time.h>                   /* for timeval */
 
 #include "asterisk/app.h"               /* for ast_read_textfile */
 #include "asterisk/astobj2.h"           /* for ao2_ref, ao2_t_ref, ao2_alloc */
 #include "asterisk/compat.h"            /* for strcasecmp */
 #include "asterisk/config.h"            /* for ast_parse_arg, ast_parse_flag... */
+#include "asterisk/io.h"                /* for ast_sd_get_fd */
 #include "asterisk/lock.h"              /* for AST_PTHREADT_NULL */
 #include "asterisk/logger.h"            /* for ast_log, LOG_ERROR, ast_debug */
 #include "asterisk/netsock2.h"          /* for ast_sockaddr_copy, ast_sockad... */
 #include "asterisk/pbx.h"               /* for ast_thread_inhibit_escalations */
-#include "asterisk/threadstorage.h"     /* for ast_threadstorage_get, AST_TH... */
-#include "asterisk/time.h"              /* for ast_remaining_ms, ast_tvnow */
 #include "asterisk/utils.h"             /* for ast_true, ast_free, ast_wait_... */
 
-/*! ao2 object used for the FILE stream fopencookie()/funopen() cookie. */
-struct ast_tcptls_stream {
-	/*! SSL state if not NULL */
-	SSL *ssl;
-	/*!
-	 * \brief Start time from when an I/O sequence must complete
-	 * by struct ast_tcptls_stream.timeout.
-	 *
-	 * \note If struct ast_tcptls_stream.start.tv_sec is zero then
-	 * start time is the current I/O request.
-	 */
-	struct timeval start;
-	/*!
-	 * \brief The socket returned by accept().
-	 *
-	 * \note Set to -1 if the stream is closed.
-	 */
-	int fd;
-	/*!
-	 * \brief Timeout in ms relative to struct ast_tcptls_stream.start
-	 * to wait for an event on struct ast_tcptls_stream.fd.
-	 *
-	 * \note Set to -1 to disable timeout.
-	 * \note The socket needs to be set to non-blocking for the timeout
-	 * feature to work correctly.
-	 */
-	int timeout;
-	/*! TRUE if stream can exclusively wait for fd input. */
-	int exclusive_input;
-};
-
-#if defined(DO_SSL)
-AST_THREADSTORAGE(err2str_threadbuf);
-#define ERR2STR_BUFSIZE   128
-
-static const char *ssl_error_to_string(int sslerr, int ret)
-{
-	switch (sslerr) {
-	case SSL_ERROR_SSL:
-		return "Internal SSL error";
-	case SSL_ERROR_SYSCALL:
-		if (!ret) {
-			return "System call EOF";
-		} else if (ret == -1) {
-			char *buf;
-
-			buf = ast_threadstorage_get(&err2str_threadbuf, ERR2STR_BUFSIZE);
-			if (!buf) {
-				return "Unknown";
-			}
-
-			snprintf(buf, ERR2STR_BUFSIZE, "Underlying BIO error: %s", strerror(errno));
-			return buf;
-		} else {
-			return "System call other";
-		}
-	default:
-		break;
-	}
-
-	return "Unknown";
-}
-#endif
-
-void ast_tcptls_stream_set_timeout_disable(struct ast_tcptls_stream *stream)
-{
-	ast_assert(stream != NULL);
-
-	stream->timeout = -1;
-}
-
-void ast_tcptls_stream_set_timeout_inactivity(struct ast_tcptls_stream *stream, int timeout)
-{
-	ast_assert(stream != NULL);
-
-	stream->start.tv_sec = 0;
-	stream->timeout = timeout;
-}
-
-void ast_tcptls_stream_set_timeout_sequence(struct ast_tcptls_stream *stream, struct timeval start, int timeout)
-{
-	ast_assert(stream != NULL);
-
-	stream->start = start;
-	stream->timeout = timeout;
-}
-
-void ast_tcptls_stream_set_exclusive_input(struct ast_tcptls_stream *stream, int exclusive_input)
-{
-	ast_assert(stream != NULL);
-
-	stream->exclusive_input = exclusive_input;
-}
-
-/*!
- * \internal
- * \brief fopencookie()/funopen() stream read function.
- *
- * \param cookie Stream control data.
- * \param buf Where to put read data.
- * \param size Size of the buffer.
- *
- * \retval number of bytes put into buf.
- * \retval 0 on end of file.
- * \retval -1 on error.
- */
-static HOOK_T tcptls_stream_read(void *cookie, char *buf, LEN_T size)
-{
-	struct ast_tcptls_stream *stream = cookie;
-	struct timeval start;
-	int ms;
-	int res;
-
-	if (!size) {
-		/* You asked for no data you got no data. */
-		return 0;
-	}
-
-	if (!stream || stream->fd == -1) {
-		errno = EBADF;
-		return -1;
-	}
-
-	if (stream->start.tv_sec) {
-		start = stream->start;
-	} else {
-		start = ast_tvnow();
-	}
-
-#if defined(DO_SSL)
-	if (stream->ssl) {
-		for (;;) {
-			int sslerr;
-			char err[256];
-
-			res = SSL_read(stream->ssl, buf, size);
-			if (0 < res) {
-				/* We read some payload data. */
-				return res;
-			}
-
-			sslerr = SSL_get_error(stream->ssl, res);
-			switch (sslerr) {
-			case SSL_ERROR_ZERO_RETURN:
-				/* Report EOF for a shutdown */
-				ast_debug(1, "TLS clean shutdown alert reading data\n");
-				return 0;
-			case SSL_ERROR_WANT_READ:
-				if (!stream->exclusive_input) {
-					/* We cannot wait for data now. */
-					errno = EAGAIN;
-					return -1;
-				}
-				while ((ms = ast_remaining_ms(start, stream->timeout))) {
-					res = ast_wait_for_input(stream->fd, ms);
-					if (0 < res) {
-						/* Socket is ready to be read. */
-						break;
-					}
-					if (res < 0) {
-						if (errno == EINTR || errno == EAGAIN) {
-							/* Try again. */
-							continue;
-						}
-						ast_debug(1, "TLS socket error waiting for read data: %s\n",
-							strerror(errno));
-						return -1;
-					}
-				}
-				break;
-			case SSL_ERROR_WANT_WRITE:
-				while ((ms = ast_remaining_ms(start, stream->timeout))) {
-					res = ast_wait_for_output(stream->fd, ms);
-					if (0 < res) {
-						/* Socket is ready to be written. */
-						break;
-					}
-					if (res < 0) {
-						if (errno == EINTR || errno == EAGAIN) {
-							/* Try again. */
-							continue;
-						}
-						ast_debug(1, "TLS socket error waiting for write space: %s\n",
-							strerror(errno));
-						return -1;
-					}
-				}
-				break;
-			default:
-				/* Report EOF for an undecoded SSL or transport error. */
-				ast_debug(1, "TLS transport or SSL error reading data: %s, %s\n", ERR_error_string(sslerr, err),
-					ssl_error_to_string(sslerr, res));
-				return 0;
-			}
-			if (!ms) {
-				/* Report EOF for a timeout */
-				ast_debug(1, "TLS timeout reading data\n");
-				return 0;
-			}
-		}
-	}
-#endif	/* defined(DO_SSL) */
-
-	for (;;) {
-		res = read(stream->fd, buf, size);
-		if (0 <= res || !stream->exclusive_input) {
-			/* Got data or we cannot wait for it. */
-			return res;
-		}
-		if (errno != EINTR && errno != EAGAIN) {
-			/* Not a retryable error. */
-			ast_debug(1, "TCP socket error reading data: %s\n",
-				strerror(errno));
-			return -1;
-		}
-		ms = ast_remaining_ms(start, stream->timeout);
-		if (!ms) {
-			/* Report EOF for a timeout */
-			ast_debug(1, "TCP timeout reading data\n");
-			return 0;
-		}
-		ast_wait_for_input(stream->fd, ms);
-	}
-}
-
-/*!
- * \internal
- * \brief fopencookie()/funopen() stream write function.
- *
- * \param cookie Stream control data.
- * \param buf Where to get data to write.
- * \param size Size of the buffer.
- *
- * \retval number of bytes written from buf.
- * \retval -1 on error.
- */
-static HOOK_T tcptls_stream_write(void *cookie, const char *buf, LEN_T size)
-{
-	struct ast_tcptls_stream *stream = cookie;
-	struct timeval start;
-	int ms;
-	int res;
-	int written;
-	int remaining;
-
-	if (!size) {
-		/* You asked to write no data you wrote no data. */
-		return 0;
-	}
-
-	if (!stream || stream->fd == -1) {
-		errno = EBADF;
-		return -1;
-	}
-
-	if (stream->start.tv_sec) {
-		start = stream->start;
-	} else {
-		start = ast_tvnow();
-	}
-
-#if defined(DO_SSL)
-	if (stream->ssl) {
-		written = 0;
-		remaining = size;
-		for (;;) {
-			int sslerr;
-			char err[256];
-
-			res = SSL_write(stream->ssl, buf + written, remaining);
-			if (res == remaining) {
-				/* Everything was written. */
-				return size;
-			}
-			if (0 < res) {
-				/* Successfully wrote part of the buffer.  Try to write the rest. */
-				written += res;
-				remaining -= res;
-				continue;
-			}
-			sslerr = SSL_get_error(stream->ssl, res);
-			switch (sslerr) {
-			case SSL_ERROR_ZERO_RETURN:
-				ast_debug(1, "TLS clean shutdown alert writing data\n");
-				if (written) {
-					/* Report partial write. */
-					return written;
-				}
-				errno = EBADF;
-				return -1;
-			case SSL_ERROR_WANT_READ:
-				ms = ast_remaining_ms(start, stream->timeout);
-				if (!ms) {
-					/* Report partial write. */
-					ast_debug(1, "TLS timeout writing data (want read)\n");
-					return written;
-				}
-				ast_wait_for_input(stream->fd, ms);
-				break;
-			case SSL_ERROR_WANT_WRITE:
-				ms = ast_remaining_ms(start, stream->timeout);
-				if (!ms) {
-					/* Report partial write. */
-					ast_debug(1, "TLS timeout writing data (want write)\n");
-					return written;
-				}
-				ast_wait_for_output(stream->fd, ms);
-				break;
-			default:
-				/* Undecoded SSL or transport error. */
-				ast_debug(1, "TLS transport or SSL error writing data: %s, %s\n", ERR_error_string(sslerr, err),
-					ssl_error_to_string(sslerr, res));
-				if (written) {
-					/* Report partial write. */
-					return written;
-				}
-				errno = EBADF;
-				return -1;
-			}
-		}
-	}
-#endif	/* defined(DO_SSL) */
-
-	written = 0;
-	remaining = size;
-	for (;;) {
-		res = write(stream->fd, buf + written, remaining);
-		if (res == remaining) {
-			/* Yay everything was written. */
-			return size;
-		}
-		if (0 < res) {
-			/* Successfully wrote part of the buffer.  Try to write the rest. */
-			written += res;
-			remaining -= res;
-			continue;
-		}
-		if (errno != EINTR && errno != EAGAIN) {
-			/* Not a retryable error. */
-			ast_debug(1, "TCP socket error writing: %s\n", strerror(errno));
-			if (written) {
-				return written;
-			}
-			return -1;
-		}
-		ms = ast_remaining_ms(start, stream->timeout);
-		if (!ms) {
-			/* Report partial write. */
-			ast_debug(1, "TCP timeout writing data\n");
-			return written;
-		}
-		ast_wait_for_output(stream->fd, ms);
-	}
-}
-
-/*!
- * \internal
- * \brief fopencookie()/funopen() stream close function.
- *
- * \param cookie Stream control data.
- *
- * \retval 0 on success.
- * \retval -1 on error.
- */
-static int tcptls_stream_close(void *cookie)
-{
-	struct ast_tcptls_stream *stream = cookie;
-
-	if (!stream) {
-		errno = EBADF;
-		return -1;
-	}
-
-	if (stream->fd != -1) {
-#if defined(DO_SSL)
-		if (stream->ssl) {
-			int res;
-
-			/*
-			 * According to the TLS standard, it is acceptable for an
-			 * application to only send its shutdown alert and then
-			 * close the underlying connection without waiting for
-			 * the peer's response (this way resources can be saved,
-			 * as the process can already terminate or serve another
-			 * connection).
-			 */
-			res = SSL_shutdown(stream->ssl);
-			if (res < 0) {
-				int sslerr = SSL_get_error(stream->ssl, res);
-				char err[256];
-
-				ast_log(LOG_ERROR, "SSL_shutdown() failed: %s, %s\n",
-					ERR_error_string(sslerr, err), ssl_error_to_string(sslerr, res));
-			}
-
-#if !defined(LIBRESSL_VERSION_NUMBER) && (OPENSSL_VERSION_NUMBER >= 0x10100000L)
-			if (!SSL_is_server(stream->ssl)) {
-#else
-			if (!stream->ssl->server) {
-#endif
-				/* For client threads, ensure that the error stack is cleared */
-#if defined(LIBRESSL_VERSION_NUMBER) || (OPENSSL_VERSION_NUMBER < 0x10100000L)
-#if OPENSSL_VERSION_NUMBER >= 0x10000000L
-				ERR_remove_thread_state(NULL);
-#else
-				ERR_remove_state(0);
-#endif	/* openssl == 1.0 */
-#endif  /* openssl < 1.1 */
-			}
-
-			SSL_free(stream->ssl);
-			stream->ssl = NULL;
-		}
-#endif	/* defined(DO_SSL) */
-
-		/*
-		 * Issuing shutdown() is necessary here to avoid a race
-		 * condition where the last data written may not appear
-		 * in the TCP stream.  See ASTERISK-23548
-		 */
-		shutdown(stream->fd, SHUT_RDWR);
-		if (close(stream->fd)) {
-			ast_log(LOG_ERROR, "close() failed: %s\n", strerror(errno));
-		}
-		stream->fd = -1;
-	}
-	ao2_t_ref(stream, -1, "Closed tcptls stream cookie");
-
-	return 0;
-}
-
-/*!
- * \internal
- * \brief fopencookie()/funopen() stream destructor function.
- *
- * \param cookie Stream control data.
- *
- * \return Nothing
- */
-static void tcptls_stream_dtor(void *cookie)
-{
-#ifdef AST_DEVMODE
-	/* Since the ast_assert below is the only one using stream,
-	 * and ast_assert is only available with AST_DEVMODE, we
-	 * put this in a conditional to avoid compiler warnings. */
-	struct ast_tcptls_stream *stream = cookie;
-#endif
-
-	ast_assert(stream->fd == -1);
-}
-
-/*!
- * \internal
- * \brief fopencookie()/funopen() stream allocation function.
- *
- * \retval stream_cookie on success.
- * \retval NULL on error.
- */
-static struct ast_tcptls_stream *tcptls_stream_alloc(void)
-{
-	struct ast_tcptls_stream *stream;
-
-	stream = ao2_alloc_options(sizeof(*stream), tcptls_stream_dtor,
-		AO2_ALLOC_OPT_LOCK_NOLOCK);
-	if (stream) {
-		stream->fd = -1;
-		stream->timeout = -1;
-	}
-	return stream;
-}
-
-/*!
- * \internal
- * \brief Open a custom FILE stream for tcptls.
- *
- * \param stream Stream cookie control data.
- * \param ssl SSL state if not NULL.
- * \param fd Socket file descriptor.
- * \param timeout ms to wait for an event on fd. -1 if timeout disabled.
- *
- * \retval fp on success.
- * \retval NULL on error.
- */
-static FILE *tcptls_stream_fopen(struct ast_tcptls_stream *stream, SSL *ssl, int fd, int timeout)
-{
-	FILE *fp;
-
-#if defined(HAVE_FOPENCOOKIE)	/* the glibc/linux interface */
-	static const cookie_io_functions_t cookie_funcs = {
-		tcptls_stream_read,
-		tcptls_stream_write,
-		NULL,
-		tcptls_stream_close
-	};
-#endif	/* defined(HAVE_FOPENCOOKIE) */
-
-	if (fd == -1) {
-		/* Socket not open. */
-		return NULL;
-	}
-
-	stream->ssl = ssl;
-	stream->fd = fd;
-	stream->timeout = timeout;
-	ao2_t_ref(stream, +1, "Opening tcptls stream cookie");
-
-#if defined(HAVE_FUNOPEN)	/* the BSD interface */
-	fp = funopen(stream, tcptls_stream_read, tcptls_stream_write, NULL,
-		tcptls_stream_close);
-#elif defined(HAVE_FOPENCOOKIE)	/* the glibc/linux interface */
-	fp = fopencookie(stream, "w+", cookie_funcs);
-#else
-	/* could add other methods here */
-	ast_debug(2, "No stream FILE methods attempted!\n");
-	fp = NULL;
-#endif
-
-	if (!fp) {
-		stream->fd = -1;
-		ao2_t_ref(stream, -1, "Failed to open tcptls stream cookie");
-	}
-	return fp;
-}
-
-HOOK_T ast_tcptls_server_read(struct ast_tcptls_session_instance *tcptls_session, void *buf, size_t count)
-{
-	if (!tcptls_session->stream_cookie || tcptls_session->stream_cookie->fd == -1) {
-		ast_log(LOG_ERROR, "TCP/TLS read called on invalid stream.\n");
-		errno = EIO;
-		return -1;
-	}
-
-	return tcptls_stream_read(tcptls_session->stream_cookie, buf, count);
-}
-
-HOOK_T ast_tcptls_server_write(struct ast_tcptls_session_instance *tcptls_session, const void *buf, size_t count)
-{
-	if (!tcptls_session->stream_cookie || tcptls_session->stream_cookie->fd == -1) {
-		ast_log(LOG_ERROR, "TCP/TLS write called on invalid stream.\n");
-		errno = EIO;
-		return -1;
-	}
-
-	return tcptls_stream_write(tcptls_session->stream_cookie, buf, count);
-}
-
 static void session_instance_destructor(void *obj)
 {
 	struct ast_tcptls_session_instance *i = obj;
 
-	if (i->stream_cookie) {
-		ao2_t_ref(i->stream_cookie, -1, "Destroying tcptls session instance");
-		i->stream_cookie = NULL;
+	if (i->stream) {
+		ast_iostream_close(i->stream);
+		i->stream = NULL;
 	}
 	ast_free(i->overflow_buf);
 	ao2_cleanup(i->private_data);
@@ -669,8 +119,7 @@ static void *handle_tcptls_connection(vo
 {
 	struct ast_tcptls_session_instance *tcptls_session = data;
 #ifdef DO_SSL
-	int (*ssl_setup)(SSL *) = (tcptls_session->client) ? SSL_connect : SSL_accept;
-	int ret;
+	SSL *ssl;
 #endif
 
 	/* TCP/TLS connections are associated with external protocols, and
@@ -698,127 +147,94 @@ static void *handle_tcptls_connection(vo
 		return NULL;
 	}
 
-	tcptls_session->stream_cookie = tcptls_stream_alloc();
-	if (!tcptls_session->stream_cookie) {
-		ast_tcptls_close_session_file(tcptls_session);
-		ao2_ref(tcptls_session, -1);
-		return NULL;
-	}
+	if (tcptls_session->parent->tls_cfg) {
+#ifdef DO_SSL
+		if (ast_iostream_start_tls(&tcptls_session->stream, tcptls_session->parent->tls_cfg->ssl_ctx, tcptls_session->client) < 0) {
+			ast_tcptls_close_session_file(tcptls_session);
+			ao2_ref(tcptls_session, -1);
+			return NULL;
+		}
 
-	/*
-	* open a FILE * as appropriate.
-	*/
-	if (!tcptls_session->parent->tls_cfg) {
-		tcptls_session->f = tcptls_stream_fopen(tcptls_session->stream_cookie, NULL,
-			tcptls_session->fd, -1);
-		if (tcptls_session->f) {
-			if (setvbuf(tcptls_session->f, NULL, _IONBF, 0)) {
+		ssl = ast_iostream_get_ssl(tcptls_session->stream);
+		if ((tcptls_session->client && !ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_DONT_VERIFY_SERVER))
+			|| (!tcptls_session->client && ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_VERIFY_CLIENT))) {
+			X509 *peer;
+			long res;
+			peer = SSL_get_peer_certificate(ssl);
+			if (!peer) {
+				ast_log(LOG_ERROR, "No peer SSL certificate to verify\n");
 				ast_tcptls_close_session_file(tcptls_session);
+				ao2_ref(tcptls_session, -1);
+				return NULL;
 			}
-		}
-	}
-#ifdef DO_SSL
-	else if ( (tcptls_session->ssl = SSL_new(tcptls_session->parent->tls_cfg->ssl_ctx)) ) {
-		SSL_set_fd(tcptls_session->ssl, tcptls_session->fd);
-		if ((ret = ssl_setup(tcptls_session->ssl)) <= 0) {
-			char err[256];
-			int sslerr = SSL_get_error(tcptls_session->ssl, ret);
-
-			ast_log(LOG_ERROR, "Problem setting up ssl connection: %s, %s\n", ERR_error_string(sslerr, err),
-				ssl_error_to_string(sslerr, ret));
-		} else if ((tcptls_session->f = tcptls_stream_fopen(tcptls_session->stream_cookie,
-			tcptls_session->ssl, tcptls_session->fd, -1))) {
-			if ((tcptls_session->client && !ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_DONT_VERIFY_SERVER))
-				|| (!tcptls_session->client && ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_VERIFY_CLIENT))) {
-				X509 *peer;
-				long res;
-				peer = SSL_get_peer_certificate(tcptls_session->ssl);
-				if (!peer) {
-					ast_log(LOG_ERROR, "No peer SSL certificate to verify\n");
-					ast_tcptls_close_session_file(tcptls_session);
-					ao2_ref(tcptls_session, -1);
-					return NULL;
-				}
 
-				res = SSL_get_verify_result(tcptls_session->ssl);
-				if (res != X509_V_OK) {
-					ast_log(LOG_ERROR, "Certificate did not verify: %s\n", X509_verify_cert_error_string(res));
-					X509_free(peer);
-					ast_tcptls_close_session_file(tcptls_session);
-					ao2_ref(tcptls_session, -1);
-					return NULL;
+			res = SSL_get_verify_result(ssl);
+			if (res != X509_V_OK) {
+				ast_log(LOG_ERROR, "Certificate did not verify: %s\n", X509_verify_cert_error_string(res));
+				X509_free(peer);
+				ast_tcptls_close_session_file(tcptls_session);
+				ao2_ref(tcptls_session, -1);
+				return NULL;
+			}
+			if (!ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_IGNORE_COMMON_NAME)) {
+				ASN1_STRING *str;
+				X509_NAME *name = X509_get_subject_name(peer);
+				STACK_OF(GENERAL_NAME) *alt_names;
+				int pos = -1;
+				int found = 0;
+
+				for (;;) {
+					/* Walk the certificate to check all available "Common Name" */
+					/* XXX Probably should do a gethostbyname on the hostname and compare that as well */
+					pos = X509_NAME_get_index_by_NID(name, NID_commonName, pos);
+					if (pos < 0) {
+						break;
+					}
+					str = X509_NAME_ENTRY_get_data(X509_NAME_get_entry(name, pos));
+					if (!check_tcptls_cert_name(str, tcptls_session->parent->hostname, "common name")) {
+						found = 1;
+						break;
+					}
 				}
-				if (!ast_test_flag(&tcptls_session->parent->tls_cfg->flags, AST_SSL_IGNORE_COMMON_NAME)) {
-					ASN1_STRING *str;
-					X509_NAME *name = X509_get_subject_name(peer);
-					STACK_OF(GENERAL_NAME) *alt_names;
-					int pos = -1;
-					int found = 0;
-
-					for (;;) {
-						/* Walk the certificate to check all available "Common Name" */
-						/* XXX Probably should do a gethostbyname on the hostname and compare that as well */
-						pos = X509_NAME_get_index_by_NID(name, NID_commonName, pos);
-						if (pos < 0) {
-							break;
-						}
 
-						str = X509_NAME_ENTRY_get_data(X509_NAME_get_entry(name, pos));
-						if (!check_tcptls_cert_name(str, tcptls_session->parent->hostname, "common name")) {
-							found = 1;
-							break;
-						}
-					}
+				if (!found) {
+					alt_names = X509_get_ext_d2i(peer, NID_subject_alt_name, NULL, NULL);
+					if (alt_names != NULL) {
+						int alt_names_count = sk_GENERAL_NAME_num(alt_names);
 
-					if (!found) {
-						alt_names = X509_get_ext_d2i(peer, NID_subject_alt_name, NULL, NULL);
-						if (alt_names != NULL) {
-							int alt_names_count = sk_GENERAL_NAME_num(alt_names);
-
-							for (pos = 0; pos < alt_names_count; pos++) {
-								const GENERAL_NAME *alt_name = sk_GENERAL_NAME_value(alt_names, pos);
-
-								if (alt_name->type != GEN_DNS) {
-									continue;
-								}
-
-								if (!check_tcptls_cert_name(alt_name->d.dNSName, tcptls_session->parent->hostname, "alt name")) {
-									found = 1;
-									break;
-								}
+						for (pos = 0; pos < alt_names_count; pos++) {
+							const GENERAL_NAME *alt_name = sk_GENERAL_NAME_value(alt_names, pos);
+
+							if (alt_name->type != GEN_DNS) {
+								continue;
 							}
 
-							sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
+							if (!check_tcptls_cert_name(alt_name->d.dNSName, tcptls_session->parent->hostname, "alt name")) {
+								found = 1;
+								break;
+							}
 						}
-					}
 
-					if (!found) {
-						ast_log(LOG_ERROR, "Certificate common name did not match (%s)\n", tcptls_session->parent->hostname);
-						X509_free(peer);
-						ast_tcptls_close_session_file(tcptls_session);
-						ao2_ref(tcptls_session, -1);
-						return NULL;
+						sk_GENERAL_NAME_pop_free(alt_names, GENERAL_NAME_free);
 					}
 				}
-				X509_free(peer);
+
+				if (!found) {
+					ast_log(LOG_ERROR, "Certificate common name did not match (%s)\n", tcptls_session->parent->hostname);
+					X509_free(peer);
+					ast_tcptls_close_session_file(tcptls_session);
+					ao2_ref(tcptls_session, -1);
+					return NULL;
+				}
 			}
+			X509_free(peer);
 		}
-		if (!tcptls_session->f) {	/* no success opening descriptor stacking */
-			SSL_free(tcptls_session->ssl);
-		}
-	}
-#endif /* DO_SSL */
-
-	if (!tcptls_session->f) {
+#else
+		ast_log(LOG_ERROR, "Attempted a TLS connection without OpenSSL support. This will not work!\n");
 		ast_tcptls_close_session_file(tcptls_session);
-		ast_log(LOG_WARNING, "FILE * open failed!\n");
-#ifndef DO_SSL
-		if (tcptls_session->parent->tls_cfg) {
-			ast_log(LOG_ERROR, "Attempted a TLS connection without OpenSSL support. This will not work!\n");
-		}
-#endif
 		ao2_ref(tcptls_session, -1);
 		return NULL;
+#endif /* DO_SSL */
 	}
 
 	if (tcptls_session->parent->worker_fn) {
@@ -876,7 +292,14 @@ void *ast_tcptls_server_root(void *data)
 			continue;
 		}
 		ast_fd_clear_flags(fd, O_NONBLOCK);
-		tcptls_session->fd = fd;
+
+		tcptls_session->stream = ast_iostream_from_fd(&fd);
+		if (!tcptls_session->stream) {
+			ao2_ref(tcptls_session, -1);
+			close(fd);
+			continue;
+		}
+
 		tcptls_session->parent = desc;
 		ast_sockaddr_copy(&tcptls_session->remote_address, &addr);
 
@@ -886,7 +309,6 @@ void *ast_tcptls_server_root(void *data)
 		if (ast_pthread_create_detached_background(&launched, NULL, handle_tcptls_connection, tcptls_session)) {
 			ast_log(LOG_ERROR, "TCP/TLS unable to launch helper thread: %s\n",
 				strerror(errno));
-			ast_tcptls_close_session_file(tcptls_session);
 			ao2_ref(tcptls_session, -1);
 		}
 	}
@@ -902,6 +324,24 @@ void *ast_tcptls_server_root(void *data)
 	return NULL;
 }
 
+#ifdef DO_SSL
+static void __ssl_setup_certs(struct ast_tls_config *cfg, const size_t cert_file_len, const char *key_type_extension, const char *key_type)
+{
+	char *cert_file = ast_strdupa(cfg->certfile);
+
+	memcpy(cert_file + cert_file_len - 8, key_type_extension, 5);
+	if (access(cert_file, F_OK) == 0) {
+		if (SSL_CTX_use_certificate_chain_file(cfg->ssl_ctx, cert_file) == 0) {
+			ast_log(LOG_WARNING, "TLS/SSL error loading public %s key (certificate) from <%s>.\n", key_type, cert_file);
+		} else if (SSL_CTX_use_PrivateKey_file(cfg->ssl_ctx, cert_file, SSL_FILETYPE_PEM) == 0) {
+			ast_log(LOG_WARNING, "TLS/SSL error loading private %s key from <%s>.\n", key_type, cert_file);
+		} else if (SSL_CTX_check_private_key(cfg->ssl_ctx) == 0) {
+			ast_log(LOG_WARNING, "TLS/SSL error matching private %s key and certificate in <%s>.\n", key_type, cert_file);
+		}
+	}
+}
+#endif
+
 static int __ssl_setup(struct ast_tls_config *cfg, int client)
 {
 #ifndef DO_SSL
@@ -1015,6 +455,17 @@ static int __ssl_setup(struct ast_tls_co
 				return 0;
 			}
 		}
+		if (!client) {
+			size_t certfile_len = strlen(cfg->certfile);
+
+			/* expects a file name which contains _rsa. like asterisk_rsa.pem
+			 * ignores any 3-character file-extension like .pem, .cer, .crt
+			 */
+			if (certfile_len >= 8 && !strncmp(cfg->certfile + certfile_len - 8, "_rsa.", 5)) {
+				__ssl_setup_certs(cfg, certfile_len, "_ecc.", "ECC");
+				__ssl_setup_certs(cfg, certfile_len, "_dsa.", "DSA");
+			}
+		}
 	}
 	if (!ast_strlen_zero(cfg->cipher)) {
 		if (SSL_CTX_set_cipher_list(cfg->ssl_ctx, cfg->cipher) == 0 ) {
@@ -1127,7 +578,7 @@ client_start_error:
 
 struct ast_tcptls_session_instance *ast_tcptls_client_create(struct ast_tcptls_session_args *desc)
 {
-	int x = 1;
+	int fd, x = 1;
 	struct ast_tcptls_session_instance *tcptls_session = NULL;
 
 	/* Do nothing if nothing has changed */
@@ -1143,8 +594,8 @@ struct ast_tcptls_session_instance *ast_
 		close(desc->accept_fd);
 	}
 
-	desc->accept_fd = socket(ast_sockaddr_is_ipv6(&desc->remote_address) ?
-				 AF_INET6 : AF_INET, SOCK_STREAM, IPPROTO_TCP);
+	fd = desc->accept_fd = socket(ast_sockaddr_is_ipv6(&desc->remote_address) ?
+				      AF_INET6 : AF_INET, SOCK_STREAM, IPPROTO_TCP);
 	if (desc->accept_fd < 0) {
 		ast_log(LOG_ERROR, "Unable to allocate socket for %s: %s\n",
 			desc->name, strerror(errno));
@@ -1175,7 +626,11 @@ struct ast_tcptls_session_instance *ast_
 		goto error;
 	}
 	tcptls_session->client = 1;
-	tcptls_session->fd = desc->accept_fd;
+	tcptls_session->stream = ast_iostream_from_fd(&fd);
+	if (!tcptls_session->stream) {
+		goto error;
+	}
+
 	tcptls_session->parent = desc;
 	tcptls_session->parent->worker_fn = NULL;
 	ast_sockaddr_copy(&tcptls_session->remote_address,
@@ -1196,6 +651,7 @@ void ast_tcptls_server_start(struct ast_
 {
 	int x = 1;
 	int tls_changed = 0;
+	int sd_socket;
 
 	if (desc->tls_cfg) {
 		char hash[41];
@@ -1267,6 +723,19 @@ void ast_tcptls_server_start(struct ast_
 		pthread_join(desc->master, NULL);
 	}
 
+	sd_socket = ast_sd_get_fd(SOCK_STREAM, &desc->local_address);
+
+	if (sd_socket != -1) {
+		if (desc->accept_fd != sd_socket) {
+			if (desc->accept_fd != -1) {
+				close(desc->accept_fd);
+			}
+			desc->accept_fd = sd_socket;
+		}
+
+		goto systemd_socket_activation;
+	}
+
 	if (desc->accept_fd != -1) {
 		close(desc->accept_fd);
 	}
@@ -1296,6 +765,8 @@ void ast_tcptls_server_start(struct ast_
 		ast_log(LOG_ERROR, "Unable to listen for %s!\n", desc->name);
 		goto error;
 	}
+
+systemd_socket_activation:
 	ast_fd_set_flags(desc->accept_fd, O_NONBLOCK);
 	if (ast_pthread_create_background(&desc->master, NULL, desc->accept_fn, desc)) {
 		ast_log(LOG_ERROR, "Unable to launch thread for %s on %s: %s\n",
@@ -1333,26 +804,11 @@ error:
 
 void ast_tcptls_close_session_file(struct ast_tcptls_session_instance *tcptls_session)
 {
-	if (tcptls_session->f) {
-		fflush(tcptls_session->f);
-		if (fclose(tcptls_session->f)) {
-			ast_log(LOG_ERROR, "fclose() failed: %s\n", strerror(errno));
-		}
-		tcptls_session->f = NULL;
-		tcptls_session->fd = -1;
-	} else if (tcptls_session->fd != -1) {
-		/*
-		 * Issuing shutdown() is necessary here to avoid a race
-		 * condition where the last data written may not appear
-		 * in the TCP stream.  See ASTERISK-23548
-		 */
-		shutdown(tcptls_session->fd, SHUT_RDWR);
-		if (close(tcptls_session->fd)) {
-			ast_log(LOG_ERROR, "close() failed: %s\n", strerror(errno));
-		}
-		tcptls_session->fd = -1;
+	if (tcptls_session->stream) {
+		ast_iostream_close(tcptls_session->stream);
+		tcptls_session->stream = NULL;
 	} else {
-		ast_log(LOG_ERROR, "ast_tcptls_close_session_file invoked on session instance without file or file descriptor\n");
+		ast_debug(1, "ast_tcptls_close_session_file invoked on session instance without file or file descriptor\n");
 	}
 }
 
diff -pruN 1:13.23.1~dfsg-2/main/tdd.c 1:16.1.1~dfsg-1/main/tdd.c
--- 1:13.23.1~dfsg-2/main/tdd.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/tdd.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 #include <math.h>
 #include <ctype.h>
@@ -43,6 +41,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/ulaw.h"
 #include "asterisk/tdd.h"
 #include "asterisk/fskmodem.h"
+#include "asterisk/utils.h"
 #include "ecdisa.h"
 
 struct tdd_state {
@@ -104,7 +103,7 @@ void tdd_init(void)
 struct tdd_state *tdd_new(void)
 {
 	struct tdd_state *tdd;
-	tdd = calloc(1, sizeof(*tdd));
+	tdd = ast_calloc(1, sizeof(*tdd));
 	if (tdd) {
 #ifdef INTEGER_CALLERID
 		tdd->fskd.ispb = 176;        /* 45.5 baud */
@@ -166,7 +165,7 @@ int tdd_feed(struct tdd_state *tdd, unsi
 	int b = 'X';
 	int res;
 	int c,x;
-	short *buf = calloc(1, 2 * len + tdd->oldlen);
+	short *buf = ast_calloc(1, 2 * len + tdd->oldlen);
 	short *obuf = buf;
 	if (!buf) {
 		ast_log(LOG_WARNING, "Out of memory\n");
@@ -182,13 +181,13 @@ int tdd_feed(struct tdd_state *tdd, unsi
 		res = fsk_serial(&tdd->fskd, buf, &mylen, &b);
 		if (mylen < 0) {
 			ast_log(LOG_ERROR, "fsk_serial made mylen < 0 (%d) (olen was %d)\n", mylen, olen);
-			free(obuf);
+			ast_free(obuf);
 			return -1;
 		}
 		buf += (olen - mylen);
 		if (res < 0) {
 			ast_log(LOG_NOTICE, "fsk_serial failed\n");
-			free(obuf);
+			ast_free(obuf);
 			return -1;
 		}
 		if (res == 1) {
@@ -206,7 +205,7 @@ int tdd_feed(struct tdd_state *tdd, unsi
 		tdd->oldlen = mylen * 2;
 	} else
 		tdd->oldlen = 0;
-	free(obuf);
+	ast_free(obuf);
 	if (res) {
 		tdd->mode = 2;
 /* put it in mode where it
@@ -218,7 +217,7 @@ int tdd_feed(struct tdd_state *tdd, unsi
 
 void tdd_free(struct tdd_state *tdd)
 {
-	free(tdd);
+	ast_free(tdd);
 }
 
 static inline float tdd_getcarrier(float *cr, float *ci, int bit)
diff -pruN 1:13.23.1~dfsg-2/main/term.c 1:16.1.1~dfsg-1/main/term.c
--- 1:13.23.1~dfsg-2/main/term.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/term.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include <sys/time.h>
 #include <signal.h>
@@ -44,7 +42,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 static int vt100compat;
 
-static char prepdata[80] = "";
 static char enddata[80] = "";
 static char quitdata[80] = "";
 
@@ -173,13 +170,10 @@ end:
 	if (vt100compat) {
 		/* Make commands show up in nice colors */
 		if (ast_opt_light_background) {
-			snprintf(prepdata, sizeof(prepdata), "%c[%dm", ESC, COLOR_BROWN);
 			snprintf(enddata, sizeof(enddata), "%c[%dm", ESC, COLOR_BLACK);
 		} else if (ast_opt_force_black_background) {
-			snprintf(prepdata, sizeof(prepdata), "%c[%d;%d;%dm", ESC, ATTR_BRIGHT, COLOR_BROWN, COLOR_BLACK + 10);
 			snprintf(enddata, sizeof(enddata), "%c[%d;%d;%dm", ESC, ATTR_RESET, COLOR_WHITE, COLOR_BLACK + 10);
 		} else {
-			snprintf(prepdata, sizeof(prepdata), "%c[%d;%dm", ESC, ATTR_BRIGHT, COLOR_BROWN);
 			snprintf(enddata, sizeof(enddata), "%c[%dm", ESC, ATTR_RESET);
 		}
 		snprintf(quitdata, sizeof(quitdata), "%c[%dm", ESC, ATTR_RESET);
@@ -336,34 +330,6 @@ char *term_strip(char *outbuf, const cha
 	return outbuf;
 }
 
-char *term_prompt(char *outbuf, const char *inbuf, int maxout)
-{
-	if (!vt100compat) {
-		ast_copy_string(outbuf, inbuf, maxout);
-		return outbuf;
-	}
-	if (ast_opt_force_black_background) {
-		snprintf(outbuf, maxout, "%c[%d;%d;%dm%c%c[%d;%dm%s",
-			ESC, ATTR_BRIGHT, COLOR_BLUE, COLOR_BLACK + 10,
-			inbuf[0],
-			ESC, COLOR_WHITE, COLOR_BLACK + 10,
-			inbuf + 1);
-	} else if (ast_opt_light_background) {
-		snprintf(outbuf, maxout, "%c[%d;0m%c%c[0m%s",
-			ESC, COLOR_BLUE,
-			inbuf[0],
-			ESC,
-			inbuf + 1);
-	} else {
-		snprintf(outbuf, maxout, "%c[%d;%d;0m%c%c[0m%s",
-			ESC, ATTR_BRIGHT, COLOR_BLUE,
-			inbuf[0],
-			ESC,
-			inbuf + 1);
-	}
-	return outbuf;
-}
-
 /* filter escape sequences */
 void term_filter_escapes(char *line)
 {
@@ -387,11 +353,6 @@ void term_filter_escapes(char *line)
 	}
 }
 
-const char *term_prep(void)
-{
-	return prepdata;
-}
-
 const char *term_end(void)
 {
 	return enddata;
diff -pruN 1:13.23.1~dfsg-2/main/test.c 1:16.1.1~dfsg-1/main/test.c
--- 1:13.23.1~dfsg-2/main/test.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/test.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/_private.h"
 
 #ifdef TEST_FRAMEWORK
diff -pruN 1:13.23.1~dfsg-2/main/threadpool.c 1:16.1.1~dfsg-1/main/threadpool.c
--- 1:13.23.1~dfsg-2/main/threadpool.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/threadpool.c	2018-12-26 18:45:13.000000000 +0000
@@ -200,10 +200,10 @@ struct thread_worker_pair {
 /*!
  * \brief Destructor for thread_worker_pair
  */
-static void thread_worker_pair_destructor(void *obj)
+static void thread_worker_pair_free(struct thread_worker_pair *pair)
 {
-	struct thread_worker_pair *pair = obj;
 	ao2_ref(pair->worker, -1);
+	ast_free(pair);
 }
 
 /*!
@@ -214,13 +214,14 @@ static void thread_worker_pair_destructo
 static struct thread_worker_pair *thread_worker_pair_alloc(struct ast_threadpool *pool,
 		struct worker_thread *worker)
 {
-	struct thread_worker_pair *pair = ao2_alloc(sizeof(*pair), thread_worker_pair_destructor);
+	struct thread_worker_pair *pair = ast_malloc(sizeof(*pair));
 	if (!pair) {
 		return NULL;
 	}
 	pair->pool = pool;
 	ao2_ref(worker, +1);
 	pair->worker = worker;
+
 	return pair;
 }
 
@@ -240,7 +241,7 @@ static int queued_active_thread_idle(voi
 
 	threadpool_send_state_changed(pair->pool);
 
-	ao2_ref(pair, -1);
+	thread_worker_pair_free(pair);
 	return 0;
 }
 
@@ -257,14 +258,19 @@ static void threadpool_active_thread_idl
 {
 	struct thread_worker_pair *pair;
 	SCOPED_AO2LOCK(lock, pool);
+
 	if (pool->shutting_down) {
 		return;
 	}
+
 	pair = thread_worker_pair_alloc(pool, worker);
 	if (!pair) {
 		return;
 	}
-	ast_taskprocessor_push(pool->control_tps, queued_active_thread_idle, pair);
+
+	if (ast_taskprocessor_push(pool->control_tps, queued_active_thread_idle, pair)) {
+		thread_worker_pair_free(pair);
+	}
 }
 
 /*!
@@ -282,7 +288,7 @@ static int queued_zombie_thread_dead(voi
 	ao2_unlink(pair->pool->zombie_threads, pair->worker);
 	threadpool_send_state_changed(pair->pool);
 
-	ao2_ref(pair, -1);
+	thread_worker_pair_free(pair);
 	return 0;
 }
 
@@ -297,14 +303,19 @@ static void threadpool_zombie_thread_dea
 {
 	struct thread_worker_pair *pair;
 	SCOPED_AO2LOCK(lock, pool);
+
 	if (pool->shutting_down) {
 		return;
 	}
+
 	pair = thread_worker_pair_alloc(pool, worker);
 	if (!pair) {
 		return;
 	}
-	ast_taskprocessor_push(pool->control_tps, queued_zombie_thread_dead, pair);
+
+	if (ast_taskprocessor_push(pool->control_tps, queued_zombie_thread_dead, pair)) {
+		thread_worker_pair_free(pair);
+	}
 }
 
 static int queued_idle_thread_dead(void *data)
@@ -314,7 +325,7 @@ static int queued_idle_thread_dead(void
 	ao2_unlink(pair->pool->idle_threads, pair->worker);
 	threadpool_send_state_changed(pair->pool);
 
-	ao2_ref(pair, -1);
+	thread_worker_pair_free(pair);
 	return 0;
 }
 
@@ -323,14 +334,19 @@ static void threadpool_idle_thread_dead(
 {
 	struct thread_worker_pair *pair;
 	SCOPED_AO2LOCK(lock, pool);
+
 	if (pool->shutting_down) {
 		return;
 	}
+
 	pair = thread_worker_pair_alloc(pool, worker);
 	if (!pair) {
 		return;
 	}
-	ast_taskprocessor_push(pool->control_tps, queued_idle_thread_dead, pair);
+
+	if (ast_taskprocessor_push(pool->control_tps, queued_idle_thread_dead, pair)) {
+		thread_worker_pair_free(pair);
+	}
 }
 
 /*!
@@ -404,15 +420,18 @@ static struct ast_threadpool *threadpool
 	if (!pool->control_tps) {
 		return NULL;
 	}
-	pool->active_threads = ao2_container_alloc(THREAD_BUCKETS, worker_thread_hash, worker_thread_cmp);
+	pool->active_threads = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		THREAD_BUCKETS, worker_thread_hash, NULL, worker_thread_cmp);
 	if (!pool->active_threads) {
 		return NULL;
 	}
-	pool->idle_threads = ao2_container_alloc(THREAD_BUCKETS, worker_thread_hash, worker_thread_cmp);
+	pool->idle_threads = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		THREAD_BUCKETS, worker_thread_hash, NULL, worker_thread_cmp);
 	if (!pool->idle_threads) {
 		return NULL;
 	}
-	pool->zombie_threads = ao2_container_alloc(THREAD_BUCKETS, worker_thread_hash, worker_thread_cmp);
+	pool->zombie_threads = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		THREAD_BUCKETS, worker_thread_hash, NULL, worker_thread_cmp);
 	if (!pool->zombie_threads) {
 		return NULL;
 	}
@@ -447,7 +466,7 @@ struct task_pushed_data {
 static struct task_pushed_data *task_pushed_data_alloc(struct ast_threadpool *pool,
 		int was_empty)
 {
-	struct task_pushed_data *tpd = ao2_alloc(sizeof(*tpd), NULL);
+	struct task_pushed_data *tpd = ast_malloc(sizeof(*tpd));
 
 	if (!tpd) {
 		return NULL;
@@ -549,6 +568,8 @@ static int queued_task_pushed(void *data
 	int was_empty = tpd->was_empty;
 	unsigned int existing_active;
 
+	ast_free(tpd);
+
 	if (pool->listener && pool->listener->callbacks->task_pushed) {
 		pool->listener->callbacks->task_pushed(pool, pool->listener, was_empty);
 	}
@@ -565,7 +586,6 @@ static int queued_task_pushed(void *data
 	/* If no idle threads could be transitioned to active grow the pool as permitted. */
 	if (ao2_container_count(pool->active_threads) == existing_active) {
 		if (!pool->options.auto_increment) {
-			ao2_ref(tpd, -1);
 			return 0;
 		}
 		grow(pool, pool->options.auto_increment);
@@ -575,7 +595,6 @@ static int queued_task_pushed(void *data
 	}
 
 	threadpool_send_state_changed(pool);
-	ao2_ref(tpd, -1);
 	return 0;
 }
 
@@ -598,12 +617,15 @@ static void threadpool_tps_task_pushed(s
 	if (pool->shutting_down) {
 		return;
 	}
+
 	tpd = task_pushed_data_alloc(pool, was_empty);
 	if (!tpd) {
 		return;
 	}
 
-	ast_taskprocessor_push(pool->control_tps, queued_task_pushed, tpd);
+	if (ast_taskprocessor_push(pool->control_tps, queued_task_pushed, tpd)) {
+		ast_free(tpd);
+	}
 }
 
 /*!
@@ -639,7 +661,9 @@ static void threadpool_tps_emptied(struc
 	}
 
 	if (pool->listener && pool->listener->callbacks->emptied) {
-		ast_taskprocessor_push(pool->control_tps, queued_emptied, pool);
+		if (ast_taskprocessor_push(pool->control_tps, queued_emptied, pool)) {
+			/* Nothing to do here but we need the check to keep the compiler happy. */
+		}
 	}
 }
 
@@ -790,7 +814,7 @@ struct set_size_data {
 static struct set_size_data *set_size_data_alloc(struct ast_threadpool *pool,
 		unsigned int size)
 {
-	struct set_size_data *ssd = ao2_alloc(sizeof(*ssd), NULL);
+	struct set_size_data *ssd = ast_malloc(sizeof(*ssd));
 	if (!ssd) {
 		return NULL;
 	}
@@ -813,7 +837,7 @@ static struct set_size_data *set_size_da
  */
 static int queued_set_size(void *data)
 {
-	RAII_VAR(struct set_size_data *, ssd, data, ao2_cleanup);
+	struct set_size_data *ssd = data;
 	struct ast_threadpool *pool = ssd->pool;
 	unsigned int num_threads = ssd->size;
 
@@ -821,6 +845,8 @@ static int queued_set_size(void *data)
 	unsigned int current_size = ao2_container_count(pool->active_threads) +
 			ao2_container_count(pool->idle_threads);
 
+	ast_free(ssd);
+
 	if (current_size == num_threads) {
 		ast_debug(3, "Not changing threadpool size since new size %u is the same as current %u\n",
 			  num_threads, current_size);
@@ -849,6 +875,7 @@ void ast_threadpool_set_size(struct ast_
 {
 	struct set_size_data *ssd;
 	SCOPED_AO2LOCK(lock, pool);
+
 	if (pool->shutting_down) {
 		return;
 	}
@@ -858,7 +885,9 @@ void ast_threadpool_set_size(struct ast_
 		return;
 	}
 
-	ast_taskprocessor_push(pool->control_tps, queued_set_size, ssd);
+	if (ast_taskprocessor_push(pool->control_tps, queued_set_size, ssd)) {
+		ast_free(ssd);
+	}
 }
 
 struct ast_threadpool_listener *ast_threadpool_listener_alloc(
diff -pruN 1:13.23.1~dfsg-2/main/threadstorage.c 1:16.1.1~dfsg-1/main/threadstorage.c
--- 1:13.23.1~dfsg-2/main/threadstorage.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/threadstorage.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,6 @@ void threadstorage_init(void)
 
 #else /* !defined(DEBUG_THREADLOCALS) */
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/strings.h"
 #include "asterisk/utils.h"
 #include "asterisk/threadstorage.h"
diff -pruN 1:13.23.1~dfsg-2/main/timing.c 1:16.1.1~dfsg-1/main/timing.c
--- 1:13.23.1~dfsg-2/main/timing.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/timing.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 
 #include "asterisk/timing.h"
@@ -126,17 +124,22 @@ struct ast_timer *ast_timer_open(void)
 	void *data = NULL;
 	struct timing_holder *h;
 	struct ast_timer *t = NULL;
+	int idx = 1;
 
 	ast_heap_rdlock(timing_interfaces);
 
-	if ((h = ast_heap_peek(timing_interfaces, 1))) {
-		data = h->iface->timer_open();
-		ast_module_ref(h->mod);
+	while ((h = ast_heap_peek(timing_interfaces, idx))) {
+		if (ast_module_running_ref(h->mod)) {
+			data = h->iface->timer_open();
+			break;
+		}
+		idx++;
 	}
 
 	if (data) {
 		if (!(t = ast_calloc(1, sizeof(*t)))) {
 			h->iface->timer_close(data);
+			ast_module_unref(h->mod);
 		} else {
 			t->data = data;
 			t->holder = h;
diff -pruN 1:13.23.1~dfsg-2/main/translate.c 1:16.1.1~dfsg-1/main/translate.c
--- 1:13.23.1~dfsg-2/main/translate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/translate.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <math.h>
@@ -565,6 +563,17 @@ struct ast_frame *ast_translate(struct a
 	long len;
 	int seqno;
 
+	if (f->frametype == AST_FRAME_RTCP) {
+		/* Just pass the feedback to the right callback, if it exists.
+		 * This "translation" does nothing so return a null frame. */
+		struct ast_trans_pvt *tp;
+		for (tp = p; tp; tp = tp->next) {
+			if (tp->t->feedback)
+				tp->t->feedback(tp, f);
+		}
+		return &ast_null_frame;
+	}
+
 	has_timing_info = ast_test_flag(f, AST_FRFLAG_HAS_TIMING_INFO);
 	ts = f->ts;
 	len = f->len;
diff -pruN 1:13.23.1~dfsg-2/main/udptl.c 1:16.1.1~dfsg-1/main/udptl.c
--- 1:13.23.1~dfsg-2/main/udptl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/udptl.c	2018-12-26 18:45:13.000000000 +0000
@@ -63,12 +63,11 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
 
+#include "asterisk/module.h"
 #include "asterisk/udptl.h"
 #include "asterisk/frame.h"
 #include "asterisk/channel.h"
@@ -1357,9 +1356,10 @@ static int udptl_pre_apply_config(void)
 	return 0;
 }
 
-int ast_udptl_reload(void)
+static int reload_module(void)
 {
 	__ast_udptl_reload(1);
+
 	return 0;
 }
 
@@ -1367,17 +1367,19 @@ int ast_udptl_reload(void)
  * \internal
  * \brief Clean up resources on Asterisk shutdown
  */
-static void udptl_shutdown(void)
+static int unload_module(void)
 {
 	ast_cli_unregister_multiple(cli_udptl, ARRAY_LEN(cli_udptl));
 	ao2_t_global_obj_release(globals, "Unref udptl global container in shutdown");
 	aco_info_destroy(&cfg_info);
+
+	return 0;
 }
 
-void ast_udptl_init(void)
+static int load_module(void)
 {
 	if (aco_info_init(&cfg_info)) {
-		return;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	aco_option_register(&cfg_info, "udptlstart", ACO_EXACT, general_options, __stringify(DEFAULT_UDPTLSTART),
@@ -1409,5 +1411,14 @@ void ast_udptl_init(void)
 
 	ast_cli_register_multiple(cli_udptl, ARRAY_LEN(cli_udptl));
 
-	ast_register_cleanup(udptl_shutdown);
+	return AST_MODULE_LOAD_SUCCESS;
 }
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "UDPTL",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CORE,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/main/ulaw.c 1:16.1.1~dfsg-1/main/ulaw.c
--- 1:13.23.1~dfsg-2/main/ulaw.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/ulaw.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/ulaw.h"
 #include "asterisk/logger.h"
 
diff -pruN 1:13.23.1~dfsg-2/main/utils.c 1:16.1.1~dfsg-1/main/utils.c
--- 1:13.23.1~dfsg-2/main/utils.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/utils.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -75,11 +73,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 static char base64[64];
 static char b2a[256];
 
-/* This is for binary compatibility with modules built before
- * ast_log_safe existed. */
-#define _AST_MEM_BACKTRACE_BUFLEN 60
-void *_ast_mem_backtrace_buffer[_AST_MEM_BACKTRACE_BUFLEN];
-
 AST_THREADSTORAGE(inet_ntoa_buf);
 
 #if !defined(HAVE_GETHOSTBYNAME_R_5) && !defined(HAVE_GETHOSTBYNAME_R_6)
@@ -623,10 +616,9 @@ static int dev_urandom_fd = -1;
 #undef pthread_create /* For ast_pthread_create function only */
 #endif /* !__linux__ */
 
-#if !defined(LOW_MEMORY)
-
 #ifdef DEBUG_THREADS
 
+#if !defined(LOW_MEMORY)
 /*! \brief A reasonable maximum number of locks a thread would be holding ... */
 #define AST_MAX_LOCKS 64
 
@@ -729,14 +721,12 @@ static void lock_info_destroy(void *data
  * \brief The thread storage key for per-thread lock info
  */
 AST_THREADSTORAGE_CUSTOM(thread_lock_info, NULL, lock_info_destroy);
-#ifdef HAVE_BKTR
+#endif /* ! LOW_MEMORY */
+
 void ast_store_lock_info(enum ast_lock_type type, const char *filename,
 	int line_num, const char *func, const char *lock_name, void *lock_addr, struct ast_bt *bt)
-#else
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-	int line_num, const char *func, const char *lock_name, void *lock_addr)
-#endif
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	int i;
 
@@ -787,10 +777,12 @@ void ast_store_lock_info(enum ast_lock_t
 	lock_info->num_locks++;
 
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
 void ast_mark_lock_acquired(void *lock_addr)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 
 	if (!(lock_info = ast_threadstorage_get(&thread_lock_info, sizeof(*lock_info))))
@@ -801,10 +793,12 @@ void ast_mark_lock_acquired(void *lock_a
 		lock_info->locks[lock_info->num_locks - 1].pending = 0;
 	}
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
 void ast_mark_lock_failed(void *lock_addr)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 
 	if (!(lock_info = ast_threadstorage_get(&thread_lock_info, sizeof(*lock_info))))
@@ -816,10 +810,12 @@ void ast_mark_lock_failed(void *lock_add
 		lock_info->locks[lock_info->num_locks - 1].times_locked--;
 	}
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
 int ast_find_lock_info(void *lock_addr, char *filename, size_t filename_size, int *lineno, char *func, size_t func_size, char *mutex_name, size_t mutex_name_size)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	int i = 0;
 
@@ -847,10 +843,14 @@ int ast_find_lock_info(void *lock_addr,
 	pthread_mutex_unlock(&lock_info->lock);
 
 	return 0;
+#else /* if defined(LOW_MEMORY) */
+	return -1;
+#endif
 }
 
 void ast_suspend_lock_info(void *lock_addr)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	int i = 0;
 
@@ -874,10 +874,12 @@ void ast_suspend_lock_info(void *lock_ad
 	lock_info->locks[i].suspended = 1;
 
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
 void ast_restore_lock_info(void *lock_addr)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	int i = 0;
 
@@ -900,15 +902,13 @@ void ast_restore_lock_info(void *lock_ad
 	lock_info->locks[i].suspended = 0;
 
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
 
-#ifdef HAVE_BKTR
 void ast_remove_lock_info(void *lock_addr, struct ast_bt *bt)
-#else
-void ast_remove_lock_info(void *lock_addr)
-#endif
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	int i = 0;
 
@@ -946,8 +946,10 @@ void ast_remove_lock_info(void *lock_add
 	lock_info->num_locks--;
 
 	pthread_mutex_unlock(&lock_info->lock);
+#endif /* ! LOW_MEMORY */
 }
 
+#if !defined(LOW_MEMORY)
 static const char *locktype2str(enum ast_lock_type type)
 {
 	switch (type) {
@@ -965,7 +967,7 @@ static const char *locktype2str(enum ast
 #ifdef HAVE_BKTR
 static void append_backtrace_information(struct ast_str **str, struct ast_bt *bt)
 {
-	char **symbols;
+	struct ast_vector_string *symbols;
 	int num_frames;
 
 	if (!bt) {
@@ -979,11 +981,11 @@ static void append_backtrace_information
 	if ((symbols = ast_bt_get_symbols(bt->addresses, num_frames))) {
 		int frame_iterator;
 
-		for (frame_iterator = 0; frame_iterator < num_frames; ++frame_iterator) {
-			ast_str_append(str, 0, "\t%s\n", symbols[frame_iterator]);
+		for (frame_iterator = 1; frame_iterator < AST_VECTOR_SIZE(symbols); ++frame_iterator) {
+			ast_str_append(str, 0, "\t%s\n", AST_VECTOR_GET(symbols, frame_iterator));
 		}
 
-		ast_std_free(symbols);
+		ast_bt_free_symbols(symbols);
 	} else {
 		ast_str_append(str, 0, "\tCouldn't retrieve backtrace symbols\n");
 	}
@@ -1026,7 +1028,7 @@ static void append_lock_information(stru
 	}
 	ast_reentrancy_unlock(lt);
 }
-
+#endif /* ! LOW_MEMORY */
 
 /*! This function can help you find highly temporal locks; locks that happen for a
     short time, but at unexpected times, usually at times that create a deadlock,
@@ -1049,6 +1051,7 @@ static void append_lock_information(stru
 */
 void ast_log_show_lock(void *this_lock_addr)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	struct ast_str *str;
 
@@ -1075,11 +1078,13 @@ void ast_log_show_lock(void *this_lock_a
 	}
 	pthread_mutex_unlock(&lock_infos_lock.mutex);
 	ast_free(str);
+#endif /* ! LOW_MEMORY */
 }
 
 
 struct ast_str *ast_dump_locks(void)
 {
+#if !defined(LOW_MEMORY)
 	struct thr_lock_info *lock_info;
 	struct ast_str *str;
 
@@ -1146,8 +1151,12 @@ struct ast_str *ast_dump_locks(void)
 	               "\n");
 
 	return str;
+#else /* if defined(LOW_MEMORY) */
+	return NULL;
+#endif
 }
 
+#if !defined(LOW_MEMORY)
 static char *handle_show_locks(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	struct ast_str *str;
@@ -1181,9 +1190,10 @@ static char *handle_show_locks(struct as
 static struct ast_cli_entry utils_cli[] = {
 	AST_CLI_DEFINE(handle_show_locks, "Show which locks are held by which thread"),
 };
-
+#endif /* ! LOW_MEMORY */
 #endif /* DEBUG_THREADS */
 
+#if !defined(LOW_MEMORY)
 /*
  * support for 'show threads'. The start routine is wrapped by
  * dummy_start(), so that ast_register_thread() and
@@ -1245,6 +1255,15 @@ static void *dummy_start(void *data)
 
 #endif /* !LOW_MEMORY */
 
+int ast_background_stacksize(void)
+{
+#if !defined(LOW_MEMORY)
+	return AST_STACKSIZE;
+#else
+	return AST_STACKSIZE_LOW;
+#endif
+}
+
 int ast_pthread_create_stack(pthread_t *thread, pthread_attr_t *attr, void *(*start_routine)(void *),
 			     void *data, size_t stacksize, const char *file, const char *caller,
 			     int line, const char *start_fn)
@@ -1438,74 +1457,6 @@ int ast_carefulwrite(int fd, char *s, in
 	return res;
 }
 
-int ast_careful_fwrite(FILE *f, int fd, const char *src, size_t len, int timeoutms)
-{
-	struct timeval start = ast_tvnow();
-	int n = 0;
-	int elapsed = 0;
-
-	while (len) {
-		if (wait_for_output(fd, timeoutms - elapsed)) {
-			/* poll returned a fatal error, so bail out immediately. */
-			return -1;
-		}
-
-		/* Clear any errors from a previous write */
-		clearerr(f);
-
-		n = fwrite(src, 1, len, f);
-
-		if (ferror(f) && errno != EINTR && errno != EAGAIN) {
-			/* fatal error from fwrite() */
-			if (errno == EPIPE) {
-				ast_debug(1, "fwrite() failed due to reading end being closed: EPIPE\n");
-			} else if (!feof(f)) {
-				/* Don't spam the logs if it was just that the connection is closed. */
-				ast_log(LOG_ERROR, "fwrite() returned error: %s\n", strerror(errno));
-			}
-			n = -1;
-			break;
-		}
-
-		/* Update for data already written to the socket */
-		len -= n;
-		src += n;
-
-		elapsed = ast_tvdiff_ms(ast_tvnow(), start);
-		if (elapsed >= timeoutms) {
-			/* We've taken too long to write
-			 * This is only an error condition if we haven't finished writing. */
-			n = len ? -1 : 0;
-			break;
-		}
-	}
-
-	errno = 0;
-	while (fflush(f)) {
-		if (errno == EAGAIN || errno == EINTR) {
-			/* fflush() does not appear to reset errno if it flushes
-			 * and reaches EOF at the same time. It returns EOF with
-			 * the last seen value of errno, causing a possible loop.
-			 * Also usleep() to reduce CPU eating if it does loop */
-			errno = 0;
-			usleep(1);
-			continue;
-		}
-		if (errno && !feof(f)) {
-			if (errno == EPIPE) {
-				ast_debug(1, "fflush() failed due to reading end being closed: EPIPE\n");
-			} else {
-				/* Don't spam the logs if it was just that the connection is closed. */
-				ast_log(LOG_ERROR, "fflush() returned error: %s\n", strerror(errno));
-			}
-		}
-		n = -1;
-		break;
-	}
-
-	return n < 0 ? -1 : 0;
-}
-
 char *ast_strip_quoted(char *s, const char *beg_quotes, const char *end_quotes)
 {
 	char *e;
@@ -2035,18 +1986,6 @@ char *ast_to_camel_case_delim(const char
 	return res;
 }
 
-AST_MUTEX_DEFINE_STATIC(fetchadd_m); /* used for all fetc&add ops */
-
-int ast_atomic_fetchadd_int_slow(volatile int *p, int v)
-{
-	int ret;
-	ast_mutex_lock(&fetchadd_m);
-	ret = *p;
-	*p += v;
-	ast_mutex_unlock(&fetchadd_m);
-	return ret;
-}
-
 /*! \brief
  * get values from config variables.
  */
@@ -2383,28 +2322,6 @@ int ast_parse_digest(const char *digest,
 	return 0;
 }
 
-#ifndef __AST_DEBUG_MALLOC
-int _ast_asprintf(char **ret, const char *file, int lineno, const char *func, const char *fmt, ...)
-{
-	int res;
-	va_list ap;
-
-	va_start(ap, fmt);
-	res = vasprintf(ret, fmt, ap);
-	if (res < 0) {
-		/*
-		 * *ret is undefined so set to NULL to ensure it is
-		 * initialized to something useful.
-		 */
-		*ret = NULL;
-		MALLOC_FAILURE_MSG;
-	}
-	va_end(ap);
-
-	return res;
-}
-#endif
-
 int ast_get_tid(void)
 {
 	int ret = -1;
diff -pruN 1:13.23.1~dfsg-2/main/xml.c 1:16.1.1~dfsg-1/main/xml.c
--- 1:13.23.1~dfsg-2/main/xml.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/xml.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 #include "asterisk/utils.h"
 #include "asterisk/autoconfig.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #if defined(HAVE_LIBXML2)
 #include <libxml/parser.h>
 #include <libxml/tree.h>
diff -pruN 1:13.23.1~dfsg-2/main/xmldoc.c 1:16.1.1~dfsg-1/main/xmldoc.c
--- 1:13.23.1~dfsg-2/main/xmldoc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/main/xmldoc.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"
 #include "asterisk/linkedlists.h"
@@ -2292,7 +2290,9 @@ static struct ast_xml_doc_item *ast_xml_
 {
 	struct ast_xml_doc_item *item;
 
-	if (!(item = ao2_alloc(sizeof(*item), ast_xml_doc_item_destructor))) {
+	item = ao2_alloc_options(sizeof(*item), ast_xml_doc_item_destructor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!item) {
 		ast_log(AST_LOG_ERROR, "Failed to allocate memory for ast_xml_doc_item instance\n");
 		return NULL;
 	}
@@ -2657,7 +2657,9 @@ struct ao2_container *ast_xmldoc_build_d
 	struct documentation_tree *doctree;
 	const char *name;
 
-	if (!(docs = ao2_container_alloc(127, ast_xml_doc_item_hash, ast_xml_doc_item_cmp))) {
+	docs = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 127,
+		ast_xml_doc_item_hash, NULL, ast_xml_doc_item_cmp);
+	if (!docs) {
 		ast_log(AST_LOG_ERROR, "Failed to create container for xml document item instances\n");
 		return NULL;
 	}
diff -pruN 1:13.23.1~dfsg-2/Makefile 1:16.1.1~dfsg-1/Makefile
--- 1:13.23.1~dfsg-2/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -76,9 +76,6 @@ export AST_DEVMODE_STRICT # Enables shad
 export _SOLINK            # linker flags for all shared objects
 export SOLINK             # linker flags for loadable modules
 export DYLINK             # linker flags for shared libraries
-export STATIC_BUILD       # Additional cflags, set to -static
-                          # for static builds. Probably
-                          # should go directly to ASTLDFLAGS
 
 #--- paths to various commands
 # The makeopts include below tries to set these if they're found during
@@ -167,20 +164,19 @@ LINKER_SYMBOL_PREFIX=
 # Default install directory for DAHDI hooks.
 DAHDI_UDEV_HOOK_DIR = /usr/share/dahdi/span_config.d
 
-# This Makefile previously contained a note about the ability to use .asterisk.makeopts
-# from your home directory or /etc/asterisk.makeopts to set defaults for menuselect.
-# These files have never worked in this branch of Asterisk.  The work around is to
-# manually copy the file containing defaults before running 'make menuselect':
-#
-# cp ${HOME}/.asterisk.makeopts menuselect.makeopts
-#   or
-# cp /etc/asterisk.makeopts menuselect.makeopts
-#
-# As an alternative, menuselect/menuselect can be used by a script to enable or disable
-# individual options or entire categories.  To use this feature you must first
-# compile menuselect using 'make menuselect.makeopts'.  For information about parameters
-# supported run:
-# menuselect/menuselect --help
+# If the file .asterisk.makeopts is present in your home directory, you can
+# include all of your favorite menuselect options so that every time you download
+# a new version of Asterisk, you don't have to run menuselect to set them.
+# The file /etc/asterisk.makeopts will also be included but can be overridden
+# by the file in your home directory.
+
+ifeq ($(wildcard menuselect.makeopts),)
+	USER_MAKEOPTS=$(wildcard ~/.asterisk.makeopts)
+	GLOBAL_MAKEOPTS=$(wildcard /etc/asterisk.makeopts)
+else
+	USER_MAKEOPTS=
+	GLOBAL_MAKEOPTS=
+endif
 
 
 MOD_SUBDIR_CFLAGS="-I$(ASTTOPDIR)/include"
@@ -358,10 +354,10 @@ makeopts: configure
 	@echo "****"
 	@exit 1
 
-menuselect.makeopts: menuselect/menuselect menuselect-tree makeopts build_tools/menuselect-deps
-ifeq ($(filter %menuselect,$(MAKECMDGOALS)),)
-	menuselect/menuselect --check-deps $@
+menuselect.makeopts: menuselect/menuselect menuselect-tree makeopts build_tools/menuselect-deps $(GLOBAL_MAKEOPTS) $(USER_MAKEOPTS)
+ifeq ($(filter %.menuselect,$(MAKECMDGOALS)),)
 	menuselect/menuselect --check-deps $@
+	menuselect/menuselect --check-deps $@ $(GLOBAL_MAKEOPTS) $(USER_MAKEOPTS)
 endif
 
 $(MOD_SUBDIRS_MENUSELECT_TREE):
@@ -950,7 +946,7 @@ config:
 		if [ -z "$(DESTDIR)" ] ; then \
 			/sbin/chkconfig --add asterisk ; \
 		fi ; \
-	elif [ -f /etc/os-release ] && [ "opensuse" = "$(shell . /etc/os-release && echo $$ID)" ] ; then \
+	elif [ -f /etc/os-release ] && [ "opensuse" = "$(shell . /etc/os-release 2>/dev/null && echo $$ID)" ] ; then \
 		./build_tools/install_subst contrib/init.d/rc.suse.asterisk  "$(DESTDIR)/etc/init.d/asterisk"; \
 		if [ ! -f /etc/sysconfig/asterisk ] ; then \
 			$(INSTALL) -m 644 contrib/init.d/etc_default_asterisk "$(DESTDIR)/etc/sysconfig/asterisk" ; \
@@ -962,7 +958,7 @@ config:
 		./build_tools/install_subst contrib/init.d/rc.archlinux.asterisk  "$(DESTDIR)/etc/init.d/asterisk"; \
 	elif [ -f /etc/slackware-version ]; then \
 		./build_tools/install_subst contrib/init.d/rc.slackware.asterisk  "$(DESTDIR)/etc/rc.d/rc.asterisk"; \
-	elif [ -f /etc/os-release ] && [ "slackware" = "$(shell . /etc/os-release && echo $$ID)" ] ; then \
+	elif [ -f /etc/os-release ] && [ "slackware" = "$(shell . /etc/os-release 2>/dev/null && echo $$ID)" ] ; then \
 		./build_tools/install_subst contrib/init.d/rc.slackware.asterisk  "$(DESTDIR)/etc/rc.d/rc.asterisk"; \
 	elif [ -d "$(DESTDIR)/Library/LaunchDaemons" ]; then \
 		if [ ! -f "$(DESTDIR)/Library/LaunchDaemons/org.asterisk.asterisk.plist" ]; then \
diff -pruN 1:13.23.1~dfsg-2/Makefile.moddir_rules 1:16.1.1~dfsg-1/Makefile.moddir_rules
--- 1:13.23.1~dfsg-2/Makefile.moddir_rules	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/Makefile.moddir_rules	2018-12-26 18:45:13.000000000 +0000
@@ -18,10 +18,6 @@
 # Also note that we can only set one variable per rule, so we have to
 # repeat the left hand side to set multiple variables.
 
-ifneq ($(findstring STATIC_BUILD,$(MENUSELECT_CFLAGS)),)
-  STATIC_BUILD=-static
-endif
-
 include $(ASTTOPDIR)/Makefile.rules
 
 # If MODULE_PREFIX is defined, use it to run the standard functions to set
@@ -52,7 +48,29 @@ LOADABLE_MODS:=$(C_MODS) $(CC_MODS)
 # is used to collect the required flags for a module... which can
 # then be used any place they are required.
 
-MOD_ASTCFLAGS=-DAST_MODULE=\"$(1)\" $(MENUSELECT_OPTS_$(1):%=-D%) $(foreach dep,$(MENUSELECT_DEPENDS_$(1)),$(value $(dep)_INCLUDE))
+MOD_ASTCFLAGS=\
+	-DAST_MODULE=\"$(1)\" \
+	-DAST_MODULE_SELF_SYM=__internal_$(1)_self \
+	$(MENUSELECT_OPTS_$(1):%=-D%) \
+	$(foreach dep,$(MENUSELECT_DEPENDS_$(1)),$(value $(dep)_INCLUDE))
+
+define MOD_ADD_SOURCE
+$(1).so: $$(subst $(3),$(5),$(2))
+$$(subst $(3),$(5),$(2)): _ASTCFLAGS+=$$(call MOD_ASTCFLAGS,$(1))
+.$(1).moduleinfo: MODULEINFO_EXTRA_OUTPUT=" $$(addprefix $$(SUBDIR)/,$$(subst $(3),$(5),$(2)) $$(subst $(3),$(4),$(2)))"
+# The use of wildcard ensures that 'make menuselect' will not fail for modules that
+# require additional source downloads.
+.$(1).moduleinfo: $(wildcard $(2))
+
+clean:: clean-$(1)$(3)
+
+clean-$(1)$(3):
+	rm -f $$(subst $(3),$(4),$(2)) $$(subst $(3),$(5),$(2)) $$(subst $(3),$(6),$(2)) $$(subst $(3),$(7),$(2))
+
+endef
+
+MOD_ADD_C=$(eval $(call MOD_ADD_SOURCE,$(1),$(2),.c,.i,.o,.gcda,.gcno))
+MOD_ADD_CC=$(eval $(call MOD_ADD_SOURCE,$(1),$(2),.cc,.ii,.oo,.gcda,.gcno))
 
 $(addsuffix .oo,$(CC_MODS)) $(addsuffix .o,$(C_MODS)):	\
 	_ASTCFLAGS+=$(call MOD_ASTCFLAGS,$*)
@@ -110,12 +128,12 @@ dist-clean::
 	rm -f *.exports
 
 .%.moduleinfo: %.c
-	@echo "<member name=\"$*\" displayname=\"$(shell $(GREP) -e AST_MODULE_INFO $< | head -n 1 | cut -d '"' -f 2)\" remove_on_change=\"$(SUBDIR)/$*.o $(SUBDIR)/$*.so\">" > $@
+	@echo "<member name=\"$*\" displayname=\"$(shell $(GREP) -e AST_MODULE_INFO $< | head -n 1 | cut -d '"' -f 2)\" remove_on_change=\"$(SUBDIR)/$*.o $(SUBDIR)/$*.i $(SUBDIR)/$*.so$(MODULEINFO_EXTRA_OUTPUT)\">" > $@
 	$(AWK) -f $(ASTTOPDIR)/build_tools/get_moduleinfo $^ >> $@
 	echo "</member>" >> $@
 
 .%.moduleinfo: %.cc
-	@echo "<member name=\"$*\" displayname=\"$(shell $(GREP) -e AST_MODULE_INFO $< | head -n 1 | cut -d '"' -f 2)\" remove_on_change=\"$(SUBDIR)/$*.oo $(SUBDIR)/$*.so\">" > $@
+	@echo "<member name=\"$*\" displayname=\"$(shell $(GREP) -e AST_MODULE_INFO $< | head -n 1 | cut -d '"' -f 2)\" remove_on_change=\"$(SUBDIR)/$*.oo $(SUBDIR)/$*.ii $(SUBDIR)/$*.so$(MODULEINFO_EXTRA_OUTPUT)\">" > $@
 	$(AWK) -f $(ASTTOPDIR)/build_tools/get_moduleinfo $^ >> $@
 	echo "</member>" >> $@
 
diff -pruN 1:13.23.1~dfsg-2/Makefile.rules 1:16.1.1~dfsg-1/Makefile.rules
--- 1:13.23.1~dfsg-2/Makefile.rules	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/Makefile.rules	2018-12-26 18:45:13.000000000 +0000
@@ -20,6 +20,9 @@
 # Helpful functions
 # call with $(call function,...)
 tolower = $(shell echo $(1) | tr '[:upper:]' '[:lower:]')
+# Takes a list of MENUSELECT_CFLAG Id and returns CFLAGS to declare
+# the ones which are enabled.
+get_menuselect_cflags=$(patsubst %,-D%,$(filter $1,$(MENUSELECT_CFLAGS)))
 
 .PHONY: dist-clean
 
@@ -176,17 +179,17 @@ ifeq ($(GNU_LD),1)
 	$(CMD_PREFIX) $(ASTTOPDIR)/build_tools/make_linker_version_script $* "$(LINKER_SYMBOL_PREFIX)" "$(ASTTOPDIR)"
 endif
 	$(ECHO_PREFIX) echo "   [LD] $^ -> $@"
-	$(CMD_PREFIX) $(CC) $(STATIC_BUILD) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
+	$(CMD_PREFIX) $(CC) -o $@ $(CC_LDFLAGS_SO) $^ $(CC_LIBS)
 
 %.so: %.oo
 ifeq ($(GNU_LD),1)
 	$(CMD_PREFIX) $(ASTTOPDIR)/build_tools/make_linker_version_script $* "$(LINKER_SYMBOL_PREFIX)" "$(ASTTOPDIR)"
 endif
 	$(ECHO_PREFIX) echo "   [LDXX] $^ -> $@"
-	$(CMD_PREFIX) $(CXX) $(STATIC_BUILD) -o $@ $(CXX_LDFLAGS_SO) $^ $(CXX_LIBS)
+	$(CMD_PREFIX) $(CXX) -o $@ $(CXX_LDFLAGS_SO) $^ $(CXX_LIBS)
 
 %: %.o
 	$(ECHO_PREFIX) echo "   [LD] $^ -> $@"
-	$(CMD_PREFIX) $(CXX) $(STATIC_BUILD) -o $@ $(PTHREAD_CFLAGS) $(_ASTLDFLAGS) $^ $(CXX_LIBS) $(ASTLDFLAGS)
+	$(CMD_PREFIX) $(CXX) -o $@ $(PTHREAD_CFLAGS) $(_ASTLDFLAGS) $^ $(CXX_LIBS) $(ASTLDFLAGS)
 
 dist-clean:: clean
diff -pruN 1:13.23.1~dfsg-2/makeopts.in 1:16.1.1~dfsg-1/makeopts.in
--- 1:13.23.1~dfsg-2/makeopts.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/makeopts.in	2018-12-26 18:45:13.000000000 +0000
@@ -137,15 +137,12 @@ BFD_LIB=@BFD_LIB@
 BLUETOOTH_INCLUDE=@BLUETOOTH_INCLUDE@
 BLUETOOTH_LIB=@BLUETOOTH_LIB@
 
+CODEC2_INCLUDE=@CODEC2_INCLUDE@
+CODEC2_LIB=@CODEC2_LIB@
+
 CURL_INCLUDE=@CURL_INCLUDE@
 CURL_LIB=@CURL_LIB@
 
-CURSES_INCLUDE=@CURSES_INCLUDE@
-CURSES_LIB=@CURSES_LIB@
-CURSES_DIR=@CURSES_DIR@
-
-EDITLINE_LIB=@EDITLINE_LIB@
-
 FREETDS_INCLUDE=@FREETDS_INCLUDE@
 FREETDS_LIB=@FREETDS_LIB@
 
@@ -196,7 +193,6 @@ URIPARSER_LIB=@URIPARSER_LIB@
 LDAP_INCLUDE=@LDAP_INCLUDE@
 LDAP_LIB=@LDAP_LIB@
 
-LIBEDIT_INTERNAL=@LIBEDIT_INTERNAL@
 LIBEDIT_INCLUDE=@LIBEDIT_INCLUDE@
 LIBEDIT_LIB=@LIBEDIT_LIB@
 
@@ -209,10 +205,6 @@ MYSQLCLIENT_LIB=@MYSQLCLIENT_LIB@
 NBS_INCLUDE=@NBS_INCLUDE@
 NBS_LIB=@NBS_LIB@
 
-NCURSES_INCLUDE=@NCURSES_INCLUDE@
-NCURSES_LIB=@NCURSES_LIB@
-NCURSES_DIR=@NCURSES_DIR@
-
 NEON_INCLUDE=@NEON_INCLUDE@
 NEON_LIB=@NEON_LIB@
 NEON29_INCLUDE=@NEON_INCLUDE@
@@ -264,6 +256,9 @@ PYTHONDEV_LIB=@PYTHONDEV_LIB@
 RESAMPLE_INCLUDE=@RESAMPLE_INCLUDE@
 RESAMPLE_LIB=@RESAMPLE_LIB@
 
+FFTW3_INCLUDE=@FFTW3_INCLUDE@
+FFTW3_LIB=@FFTW3_LIB@
+
 RT_LIB=@RT_LIB@
 
 SS7_INCLUDE=@SS7_INCLUDE@
@@ -324,6 +319,9 @@ CRYPTO_LIB=@CRYPTO_LIB@
 TONEZONE_INCLUDE=@TONEZONE_INCLUDE@
 TONEZONE_LIB=@TONEZONE_LIB@
 
+UNBOUND_INCLUDE=@UNBOUND_INCLUDE@
+UNBOUND_LIB=@UNBOUND_LIB@
+
 UNIXODBC_INCLUDE=@UNIXODBC_INCLUDE@
 UNIXODBC_LIB=@UNIXODBC_LIB@
 
@@ -357,20 +355,18 @@ CAP_INCLUDE=@CAP_INCLUDE@
 BKTR_INCLUDE=@BKTR_INCLUDE@
 BKTR_LIB=@BKTR_LIB@
 
-TERMCAP_INCLUDE=@TERMCAP_INCLUDE@
-TERMCAP_LIB=@TERMCAP_LIB@
-TERMCAP_DIR=@TERMCAP_DIR@
-
 LIBXML2_INCLUDE=@LIBXML2_INCLUDE@
 LIBXML2_LIB=@LIBXML2_LIB@
 
 LIBXSLT_INCLUDE=@LIBXSLT_INCLUDE@
 LIBXSLT_LIB=@LIBXSLT_LIB@
 
-TINFO_INCLUDE=@TINFO_INCLUDE@
-TINFO_LIB=@TINFO_LIB@
-TINFO_DIR=@TINFO_DIR@
-
 # if poll is not present, let the makefile know.
 POLL_AVAILABLE=@HAS_POLL@
 TIMERFD_INCLUDE=@TIMERFD_INCLUDE@
+
+SNDFILE_INCLUDE=@SNDFILE_INCLUDE@
+SNDFILE_LIB=@SNDFILE_LIB@
+
+BEANSTALK_INCLUDE=@BEANSTALK_INCLUDE@
+BEANSTALK_LIB=@BEANSTALK_LIB@
diff -pruN 1:13.23.1~dfsg-2/menuselect/strcompat.c 1:16.1.1~dfsg-1/menuselect/strcompat.c
--- 1:13.23.1~dfsg-2/menuselect/strcompat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/menuselect/strcompat.c	2018-12-26 18:45:13.000000000 +0000
@@ -138,7 +138,7 @@ size_t strnlen(const char *s, size_t n)
 }
 #endif /* !HAVE_STRNLEN */
 
-#if !defined(HAVE_STRNDUP) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_STRNDUP)
 char *strndup(const char *s, size_t n)
 {
 	size_t len = strnlen(s, n);
@@ -150,9 +150,9 @@ char *strndup(const char *s, size_t n)
 	new[len] = '\0';
 	return memcpy(new, s, len);
 }
-#endif /* !defined(HAVE_STRNDUP) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_STRNDUP) */
 
-#if !defined(HAVE_VASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_VASPRINTF)
 int vasprintf(char **strp, const char *fmt, va_list ap)
 {
 	int size;
@@ -170,7 +170,7 @@ int vasprintf(char **strp, const char *f
 
 	return size;
 }
-#endif /* !defined(HAVE_VASPRINTF) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_VASPRINTF) */
 
 /*
  * Based on Code from bsd-asprintf from OpenSSH
@@ -191,7 +191,7 @@ int vasprintf(char **strp, const char *f
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
-#if !defined(HAVE_ASPRINTF) && !defined(__AST_DEBUG_MALLOC)
+#if !defined(HAVE_ASPRINTF)
 int asprintf(char **str, const char *fmt, ...)
 {
         va_list ap;
@@ -204,7 +204,7 @@ int asprintf(char **str, const char *fmt
 
         return ret;
 }
-#endif /* !defined(HAVE_ASPRINTF) && !defined(__AST_DEBUG_MALLOC) */
+#endif /* !defined(HAVE_ASPRINTF) */
 
 #ifndef HAVE_GETLOADAVG
 #ifdef linux
diff -pruN 1:13.23.1~dfsg-2/menuselect/test/menuselect-tree 1:16.1.1~dfsg-1/menuselect/test/menuselect-tree
--- 1:13.23.1~dfsg-2/menuselect/test/menuselect-tree	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/menuselect/test/menuselect-tree	2018-12-26 18:45:13.000000000 +0000
@@ -535,8 +535,6 @@
 		</member>
 		<member name="DEBUG_THREADS" displayname="Enable Thread Debugging">
 		</member>
-		<member name="STATIC_BUILD" displayname="Build static binaries">
-		</member>
 		<member name="LOADABLE_MODULES" displayname="Runtime module loading">
 			<defaultenabled>yes</defaultenabled>
 		</member>
diff -pruN 1:13.23.1~dfsg-2/pbx/dundi-parser.c 1:16.1.1~dfsg-1/pbx/dundi-parser.c
--- 1:13.23.1~dfsg-2/pbx/dundi-parser.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/dundi-parser.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -39,6 +37,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/dundi.h"
 #include "dundi-parser.h"
 
+
 static void internaloutput(const char *str)
 {
 	fputs(str, stdout);
@@ -430,7 +429,7 @@ static void dump_ies(unsigned char *ieda
 	outputf("\n");
 }
 
-void dundi_showframe(struct dundi_hdr *fhi, int rx, struct sockaddr_in *sin, int datalen)
+void dundi_showframe(struct dundi_hdr *fhi, int rx, struct ast_sockaddr *sin, int datalen)
 {
 	char *pref[] = {
 		"Tx",
@@ -471,11 +470,13 @@ void dundi_showframe(struct dundi_hdr *f
 		pref[rx],
 		fhi->oseqno, fhi->iseqno, class, fhi->cmdresp & 0x40 ? "Response" : "Command");
 	outputf(tmp);
+
 	snprintf(tmp, (int)sizeof(tmp),
-		"%s     Flags: %s STrans: %5.5d  DTrans: %5.5d [%s:%d]%s\n", (rx > 1) ? "     " : "",
+		"%s     Flags: %s STrans: %5.5d  DTrans: %5.5d [%s]%s\n", (rx > 1) ? "     " : "",
 		subclass, ntohs(fhi->strans) & ~DUNDI_FLAG_RESERVED, ntohs(fhi->dtrans) & ~DUNDI_FLAG_RETRANS,
-		ast_inet_ntoa(sin->sin_addr), ntohs(sin->sin_port),
+		ast_sockaddr_stringify(sin),
 		fhi->cmdresp & 0x80 ? " (Final)" : "");
+
 	outputf(tmp);
 	dump_ies(fhi->ies, rx > 1, datalen);
 }
diff -pruN 1:13.23.1~dfsg-2/pbx/dundi-parser.h 1:16.1.1~dfsg-1/pbx/dundi-parser.h
--- 1:13.23.1~dfsg-2/pbx/dundi-parser.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/dundi-parser.h	2018-12-26 18:45:13.000000000 +0000
@@ -60,7 +60,7 @@ struct dundi_ie_data {
 extern void dundi_set_output(void (*output)(const char *data));
 /* Choose a different function for errors */
 extern void dundi_set_error(void (*output)(const char *data));
-extern void dundi_showframe(struct dundi_hdr *fhi, int rx, struct sockaddr_in *sin, int datalen);
+extern void dundi_showframe(struct dundi_hdr *fhi, int rx, struct ast_sockaddr *sin, int datalen);
 
 extern const char *dundi_ie2str(int ie);
 
diff -pruN 1:13.23.1~dfsg-2/pbx/Makefile 1:16.1.1~dfsg-1/pbx/Makefile
--- 1:13.23.1~dfsg-2/pbx/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -23,13 +23,4 @@ ifneq ($(findstring $(OSARCH), mingw32 c
   LIBS+= -lres_ael_share.so -lres_monitor.so
 endif
 
-clean::
-	rm -f ael/*.o ael/*.i ael/*.gcda ael/*.gcno
-
-dundi-parser.o: dundi-parser.h
-dundi-parser.o: _ASTCFLAGS+=-I.
-
-pbx_gtkconsole.o: _ASTCFLAGS+=-Wno-strict-prototypes
-
-pbx_dundi.so: dundi-parser.o
-dundi-parser.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,pbx_dundi)
+$(call MOD_ADD_C,pbx_dundi,dundi-parser.c)
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_ael.c 1:16.1.1~dfsg-1/pbx/pbx_ael.c
--- 1:13.23.1~dfsg-2/pbx/pbx_ael.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_ael.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,19 +29,10 @@
 
 #include "asterisk.h"
 
-#if !defined(STANDALONE)
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-#endif
-
 #include <ctype.h>
 #include <regex.h>
 #include <sys/stat.h>
 
-#ifdef STANDALONE
-#ifdef HAVE_MTX_PROFILE
-static int mtx_prof = -1; /* helps the standalone compile with the mtx_prof flag on */
-#endif
-#endif
 #include "asterisk/pbx.h"
 #include "asterisk/config.h"
 #include "asterisk/module.h"
@@ -297,11 +288,12 @@ int ael_external_load_module(void)
 #endif
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Asterisk Extension Language Compiler",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.requires = "res_ael_share",
+);
 
 #ifdef AAL_ARGCHECK
 static const char * const ael_funclist[] =
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_config.c 1:16.1.1~dfsg-1/pbx/pbx_config.c
--- 1:13.23.1~dfsg-2/pbx/pbx_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -82,8 +82,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 
 #include "asterisk/paths.h"	/* ast_config_AST_CONFIG_DIR */
@@ -189,29 +187,49 @@ static char *handle_cli_dialplan_remove_
 /*! \brief return true if 'name' is included by context c */
 static int lookup_ci(struct ast_context *c, const char *name)
 {
-	struct ast_include *i = NULL;
+	int idx;
+	int ret = 0;
 
-	if (ast_rdlock_context(c))	/* error, skip */
+	if (ast_rdlock_context(c)) {
+		/* error, skip */
 		return 0;
-	while ( (i = ast_walk_context_includes(c, i)) )
-		if (!strcmp(name, ast_get_include_name(i)))
+	}
+
+	for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+		const struct ast_include *i = ast_context_includes_get(c, idx);
+
+		if (!strcmp(name, ast_get_include_name(i))) {
+			ret = -1;
 			break;
+		}
+	}
 	ast_unlock_context(c);
-	return i ? -1 /* success */ : 0;
+
+	return ret;
 }
 
 /*! \brief return true if 'name' is in the ignorepats for context c */
 static int lookup_c_ip(struct ast_context *c, const char *name)
 {
-	struct ast_ignorepat *ip = NULL;
+	int idx;
+	int ret = 0;
 
-	if (ast_rdlock_context(c))	/* error, skip */
+	if (ast_rdlock_context(c)) {
+		/* error, skip */
 		return 0;
-	while ( (ip = ast_walk_context_ignorepats(c, ip)) )
-		if (!strcmp(name, ast_get_ignorepat_name(ip)))
+	}
+
+	for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+		const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
+
+		if (!strcmp(name, ast_get_ignorepat_name(ip))) {
+			ret = -1;
 			break;
+		}
+	}
 	ast_unlock_context(c);
-	return ip ? -1 /* success */ : 0;
+
+	return ret;
 }
 
 /*! \brief moves to the n-th word in the string, or empty string if none */
@@ -253,7 +271,7 @@ static int split_ec(const char *src, cha
 		*c++ = '\0';
 		*ctx = c;
 		if (strchr(c, '@')) { /* two @, not allowed */
-			free(e);
+			ast_free(e);
 			return -1;
 		}
 	}
@@ -282,12 +300,13 @@ static char *complete_dialplan_remove_in
 		}
 		/* walk contexts and their includes, return the n-th match */
 		while (!res && (c = ast_walk_contexts(c))) {
-			struct ast_include *i = NULL;
+			int idx;
 
 			if (ast_rdlock_context(c))	/* error ? skip this one */
 				continue;
 
-			while ( !res && (i = ast_walk_context_includes(c, i)) ) {
+			for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+				const struct ast_include *i = ast_context_includes_get(c, idx);
 				const char *i_name = ast_get_include_name(i);
 				struct ast_context *nc = NULL;
 				int already_served = 0;
@@ -304,7 +323,7 @@ static char *complete_dialplan_remove_in
 					already_served = lookup_ci(nc, i_name);
 
 				if (!already_served && ++which > a->n) {
-					res = strdup(i_name);
+					res = ast_strdup(i_name);
 					break;
 				}
 			}
@@ -323,7 +342,7 @@ static char *complete_dialplan_remove_in
 
 		if (a->n > 0)
 			return NULL;
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
 			return NULL;
@@ -332,18 +351,18 @@ static char *complete_dialplan_remove_in
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_ERROR, "Failed to lock contexts list\n");
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
 		/* go through all contexts and check if is included ... */
 		while (!res && (c = ast_walk_contexts(c)))
 			if (lookup_ci(c, context)) /* context is really included, complete "from" command */
-				res = strdup("from");
+				res = ast_strdup("from");
 		ast_unlock_contexts();
 		if (!res)
 			ast_log(LOG_WARNING, "%s not included anywhere\n", context);
-		free(context);
+		ast_free(context);
 		return res;
 	} else if (a->pos == 5) { /* "dialplan remove include CTX from _X_" */
 		/*
@@ -351,7 +370,7 @@ static char *complete_dialplan_remove_in
 		 */
 		char *context, *dupline, *from;
 		const char *s = skip_words(a->line, 3); /* skip 'dialplan' 'remove' 'include' */
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
 			return NULL;
@@ -362,13 +381,13 @@ static char *complete_dialplan_remove_in
 		/* fourth word must be 'from' */
 		from = strsep(&dupline, " ");
 		if (!from || strcmp(from, "from")) {
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_ERROR, "Failed to lock context list\n");
-			free(context);
+			ast_free(context);
 			return NULL;
 		}
 
@@ -380,10 +399,10 @@ static char *complete_dialplan_remove_in
 				continue;
 			/* walk through all includes and check if it is our context */
 			if (lookup_ci(c, context) && ++which > a->n)
-				res = strdup(c_name);
+				res = ast_strdup(c_name);
 		}
 		ast_unlock_contexts();
-		free(context);
+		ast_free(context);
 		return res;
 	}
 
@@ -453,7 +472,7 @@ static char *handle_cli_dialplan_remove_
 	if ((!strlen(exten)) || (!(strlen(context)))) {
 		ast_cli(a->fd, "Missing extension or context name in third argument '%s'\n",
 			a->argv[3]);
-		free(exten);
+		ast_free(exten);
 		return CLI_FAILURE;
 	}
 
@@ -476,7 +495,7 @@ static char *handle_cli_dialplan_remove_
 		}
 		ret = CLI_FAILURE;
 	}
-	free(exten);
+	ast_free(exten);
 	return ret;
 }
 
@@ -588,7 +607,7 @@ static char *complete_dialplan_remove_ex
 
 		ast_unlock_contexts();
 	error2:
-		free(exten);
+		ast_free(exten);
 	} else if (a->pos == 4) { /* 'dialplan remove extension EXT _X_' (priority) */
 		char *exten = NULL, *context, *cid, *p;
 		struct ast_context *c;
@@ -636,7 +655,7 @@ static char *complete_dialplan_remove_ex
 				while ( !ret && (priority = ast_walk_extension_priorities(e, priority)) ) {
 					snprintf(buffer, sizeof(buffer), "%d", ast_get_extension_priority(priority));
 					if (partial_match(buffer, a->word, len) && ++which > a->n) /* n-th match */
-						ret = strdup(buffer);
+						ret = ast_strdup(buffer);
 				}
 				break;
 			}
@@ -644,7 +663,7 @@ static char *complete_dialplan_remove_ex
 		}
 		ast_unlock_contexts();
 	error3:
-		free(exten);
+		ast_free(exten);
 	}
 	return ret;
 }
@@ -736,16 +755,16 @@ static char *complete_dialplan_add_inclu
 		}
 		for (c = NULL; !ret && (c = ast_walk_contexts(c)); )
 			if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 		ast_unlock_contexts();
 		return ret;
 	} else if (a->pos == 4) { /* dialplan add include CTX _X_ */
 		/* always complete  as 'into' */
-		return (a->n == 0) ? strdup("into") : NULL;
+		return (a->n == 0) ? ast_strdup("into") : NULL;
 	} else if (a->pos == 5) { /* 'dialplan add include CTX into _X_' (dst context) */
 		char *context, *dupline, *into;
 		const char *s = skip_words(a->line, 3); /* should not fail */
-		context = dupline = strdup(s);
+		context = dupline = ast_strdup(s);
 
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Out of free memory\n");
@@ -772,12 +791,12 @@ static char *complete_dialplan_add_inclu
 			if (partial_match(ast_get_context_name(c), a->word, len) &&
 					!lookup_ci(c, context) /* not included yet */ &&
 					++which > a->n) {
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 			}
 		}
 		ast_unlock_contexts();
 	error3:
-		free(context);
+		ast_free(context);
 		return ret;
 	}
 
@@ -908,9 +927,7 @@ static char *handle_cli_dialplan_save(st
 	for (c = NULL; (c = ast_walk_contexts(c)); ) {
 		int context_header_written = 0;
 		struct ast_exten *ext, *last_written_e = NULL;
-		struct ast_include *i;
-		struct ast_ignorepat *ip;
-		struct ast_sw *sw;
+		int idx;
 
 		/* try to lock context and fireout all info */
 		if (ast_rdlock_context(c)) { /* lock failure */
@@ -985,17 +1002,22 @@ static char *handle_cli_dialplan_save(st
 			fprintf(output, "\n");
 
 		/* walk through includes */
-		for (i = NULL; (i = ast_walk_context_includes(c, i)) ; ) {
+		for (idx = 0; idx < ast_context_includes_count(c); idx++) {
+			const struct ast_include *i = ast_context_includes_get(c, idx);
+
 			if (strcmp(ast_get_include_registrar(i), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
 			fprintf(output, "include => %s\n", ast_get_include_name(i));
 		}
-		if (ast_walk_context_includes(c, NULL))
+		if (ast_context_includes_count(c)) {
 			fprintf(output, "\n");
+		}
 
 		/* walk through switches */
-		for (sw = NULL; (sw = ast_walk_context_switches(c, sw)) ; ) {
+		for (idx = 0; idx < ast_context_switches_count(c); idx++) {
+			const struct ast_sw *sw = ast_context_switches_get(c, idx);
+
 			if (strcmp(ast_get_switch_registrar(sw), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
@@ -1003,11 +1025,14 @@ static char *handle_cli_dialplan_save(st
 				    ast_get_switch_name(sw), ast_get_switch_data(sw));
 		}
 
-		if (ast_walk_context_switches(c, NULL))
+		if (ast_context_switches_count(c)) {
 			fprintf(output, "\n");
+		}
 
 		/* fireout ignorepats ... */
-		for (ip = NULL; (ip = ast_walk_context_ignorepats(c, ip)); ) {
+		for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+			const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
+
 			if (strcmp(ast_get_ignorepat_registrar(ip), registrar) != 0)
 				continue; /* not mine */
 			PUT_CTX_HDR;
@@ -1217,7 +1242,7 @@ static int manager_dialplan_extension_ad
 	}
 
 	if (ast_add_extension2(add_context, replace, exten, ipriority, NULL, cidmatch,
-			application, ast_strdup(application_data), ast_free_ptr, registrar)) {
+			application, ast_strdup(application_data), ast_free_ptr, registrar, NULL, 0)) {
 		ast_unlock_contexts();
 		switch (errno) {
 		case ENOMEM:
@@ -1270,7 +1295,7 @@ static char *complete_dialplan_remove_co
 	/* walk through all contexts */
 	while ( !res && (c = ast_walk_contexts(c)) ) {
 		if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n) {
-			res = strdup(ast_get_context_name(c));
+			res = ast_strdup(ast_get_context_name(c));
 		}
 	}
 	ast_unlock_contexts();
@@ -1283,7 +1308,7 @@ static char *complete_dialplan_add_exten
 	int which = 0;
 
 	if (a->pos == 4) {		/* complete 'into' word ... */
-		return (a->n == 0) ? strdup("into") : NULL;
+		return (a->n == 0) ? ast_strdup("into") : NULL;
 	} else if (a->pos == 5) { /* complete context */
 		struct ast_context *c = NULL;
 		int len = strlen(a->word);
@@ -1298,11 +1323,11 @@ static char *complete_dialplan_add_exten
 		/* walk through all contexts */
 		while ( !res && (c = ast_walk_contexts(c)) )
 			if (partial_match(ast_get_context_name(c), a->word, len) && ++which > a->n)
-				res = strdup(ast_get_context_name(c));
+				res = ast_strdup(ast_get_context_name(c));
 		ast_unlock_contexts();
 		return res;
 	} else if (a->pos == 6) {
-		return a->n == 0 ? strdup("replace") : NULL;
+		return a->n == 0 ? ast_strdup("replace") : NULL;
 	}
 	return NULL;
 }
@@ -1367,7 +1392,7 @@ static char *handle_cli_dialplan_add_ign
 static char *complete_dialplan_add_ignorepat(struct ast_cli_args *a)
 {
 	if (a->pos == 4)
-		return a->n == 0 ? strdup("into") : NULL;
+		return a->n == 0 ? ast_strdup("into") : NULL;
 	else if (a->pos == 5) {
 		struct ast_context *c;
 		int which = 0;
@@ -1380,7 +1405,7 @@ static char *complete_dialplan_add_ignor
 		s = skip_words(a->line, 3);
 		if (s == NULL)
 			return NULL;
-		dupline = strdup(s);
+		dupline = ast_strdup(s);
 		if (!dupline) {
 			ast_log(LOG_ERROR, "Malloc failure\n");
 			return NULL;
@@ -1400,10 +1425,10 @@ static char *complete_dialplan_add_ignor
 			if (ignorepat) /* there must be one, right ? */
 				found = lookup_c_ip(c, ignorepat);
 			if (!found && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 		}
 
-		free(ignorepat);
+		ast_free(ignorepat);
 		ast_unlock_contexts();
 		return ret;
 	}
@@ -1474,12 +1499,13 @@ static char *complete_dialplan_remove_ig
 		}
 
 		for (c = NULL; !ret && (c = ast_walk_contexts(c));) {
-			struct ast_ignorepat *ip;
+			int idx;
 
 			if (ast_rdlock_context(c))	/* error, skip it */
 				continue;
+			for (idx = 0; idx < ast_context_ignorepats_count(c); idx++) {
+				const struct ast_ignorepat *ip = ast_context_ignorepats_get(c, idx);
 
-			for (ip = NULL; !ret && (ip = ast_walk_context_ignorepats(c, ip));) {
 				if (partial_match(ast_get_ignorepat_name(ip), a->word, len) && ++which > a->n) {
 					/* n-th match */
 					struct ast_context *cw = NULL;
@@ -1489,7 +1515,7 @@ static char *complete_dialplan_remove_ig
 						found = lookup_c_ip(cw, ast_get_ignorepat_name(ip));
 					}
 					if (!found)
-						ret = strdup(ast_get_ignorepat_name(ip));
+						ret = ast_strdup(ast_get_ignorepat_name(ip));
 				}
 			}
 			ast_unlock_context(c);
@@ -1497,12 +1523,12 @@ static char *complete_dialplan_remove_ig
 		ast_unlock_contexts();
 		return ret;
 	} else if (a->pos == 4) {
-		 return a->n == 0 ? strdup("from") : NULL;
+		 return a->n == 0 ? ast_strdup("from") : NULL;
 	} else if (a->pos == 5) { /* XXX check this */
 		char *dupline, *duplinet, *ignorepat;
 		int len = strlen(a->word);
 
-		dupline = strdup(a->line);
+		dupline = ast_strdup(a->line);
 		if (!dupline) {
 			ast_log(LOG_WARNING, "Out of free memory\n");
 			return NULL;
@@ -1514,13 +1540,13 @@ static char *complete_dialplan_remove_ig
 		ignorepat = strsep(&duplinet, " ");
 
 		if (!ignorepat) {
-			free(dupline);
+			ast_free(dupline);
 			return NULL;
 		}
 
 		if (ast_rdlock_contexts()) {
 			ast_log(LOG_WARNING, "Failed to lock contexts list\n");
-			free(dupline);
+			ast_free(dupline);
 			return NULL;
 		}
 
@@ -1534,11 +1560,11 @@ static char *complete_dialplan_remove_ig
 				continue;
 			}
 			if (lookup_c_ip(c, ignorepat) && ++which > a->n)
-				ret = strdup(ast_get_context_name(c));
+				ret = ast_strdup(ast_get_context_name(c));
 			ast_unlock_context(c);
 		}
 		ast_unlock_contexts();
-		free(dupline);
+		ast_free(dupline);
 		return ret;
 	}
 
@@ -1584,12 +1610,11 @@ static struct ast_cli_entry cli_pbx_conf
 	AST_CLI_DEFINE(handle_cli_dialplan_remove_ignorepat, "Remove ignore pattern from context"),
 	AST_CLI_DEFINE(handle_cli_dialplan_add_include,      "Include context in other context"),
 	AST_CLI_DEFINE(handle_cli_dialplan_remove_include,   "Remove a specified include from context"),
-	AST_CLI_DEFINE(handle_cli_dialplan_reload,           "Reload extensions and *only* extensions"),
-	AST_CLI_DEFINE(handle_cli_dialplan_save,             "Save current dialplan into a file")
+	AST_CLI_DEFINE(handle_cli_dialplan_reload,           "Reload extensions and *only* extensions")
 };
 
 static struct ast_cli_entry cli_dialplan_save =
-	AST_CLI_DEFINE(handle_cli_dialplan_save, "Save dialplan");
+	AST_CLI_DEFINE(handle_cli_dialplan_save,             "Save current dialplan into a file");
 
 #define AMI_EXTENSION_ADD "DialplanExtensionAdd"
 #define AMI_EXTENSION_REMOVE "DialplanExtensionRemove"
@@ -1599,15 +1624,15 @@ static struct ast_cli_entry cli_dialplan
  */
 static int unload_module(void)
 {
-	if (static_config && !write_protect_config)
-		ast_cli_unregister(&cli_dialplan_save);
-	if (overrideswitch_config) {
-		ast_free(overrideswitch_config);
-	}
+	ast_cli_unregister(&cli_dialplan_save);
+	ast_free(overrideswitch_config);
+	overrideswitch_config = NULL;
+
 	ast_cli_unregister_multiple(cli_pbx_config, ARRAY_LEN(cli_pbx_config));
 	ast_manager_unregister(AMI_EXTENSION_ADD);
 	ast_manager_unregister(AMI_EXTENSION_REMOVE);
 	ast_context_destroy(NULL, registrar);
+
 	return 0;
 }
 
@@ -1689,10 +1714,20 @@ static int pbx_load_config(const char *c
 
 	ast_copy_string(userscontext, ast_variable_retrieve(cfg, "general", "userscontext") ?: "default", sizeof(userscontext));
 
-	for (v = ast_variable_browse(cfg, "globals"); v; v = v->next) {
-		pbx_substitute_variables_helper(NULL, v->value, realvalue, sizeof(realvalue) - 1);
-		pbx_builtin_setvar_helper(NULL, v->name, realvalue);
+	/* ast_variable_browse does not merge multiple [globals] sections */
+	for (cxt = ast_category_browse(cfg, NULL);
+	     cxt;
+	     cxt = ast_category_browse(cfg, cxt)) {
+		if (strcasecmp(cxt, "globals")) {
+			continue;
+		}
+
+		for (v = ast_variable_browse(cfg, cxt); v; v = v->next) {
+			pbx_substitute_variables_helper(NULL, v->value, realvalue, sizeof(realvalue) - 1);
+			pbx_builtin_setvar_helper(NULL, v->name, realvalue);
+		}
 	}
+
 	for (cxt = ast_category_browse(cfg, NULL);
 	     cxt;
 	     cxt = ast_category_browse(cfg, cxt)) {
@@ -1835,6 +1870,7 @@ process_extension:
 
 				appl = ast_skip_blanks(appl);
 				if (ipri) {
+					const char *registrar_file;
 					if (plus) {
 						ipri += atoi(plus);
 					}
@@ -1844,7 +1880,14 @@ process_extension:
 							"The use of '%s' for an extension is strongly discouraged and can have unexpected behavior.  Please use '_X%c' instead at line %d of %s\n",
 							realext, realext[1], v->lineno, vfile);
 					}
-					if (ast_add_extension2(con, 0, realext, ipri, label, cidmatch, appl, ast_strdup(data), ast_free_ptr, registrar)) {
+					/* Don't include full path if the configuration file includes slashes */
+					registrar_file = strrchr(vfile, '/');
+					if (!registrar_file) {
+						registrar_file = vfile;
+					} else {
+						registrar_file++; /* Skip past the end slash */
+					}
+					if (ast_add_extension2(con, 0, realext, ipri, label, cidmatch, appl, ast_strdup(data), ast_free_ptr, registrar, registrar_file, v->lineno)) {
 						ast_log(LOG_WARNING,
 							"Unable to register extension at line %d of %s\n",
 							v->lineno, vfile);
@@ -1906,6 +1949,8 @@ process_extension:
 						"Unable to include switch '%s' in context '%s' at line %d of %s\n",
 						v->value, cxt, v->lineno, vfile);
 				}
+			} else if (!strcasecmp(v->name, "autohints")) {
+				ast_context_set_autohints(con, ast_true(v->value));
 			} else {
 				ast_log(LOG_WARNING,
 					"==!!== Unknown directive: %s at line %d of %s -- IGNORING!!!\n",
@@ -2009,19 +2054,19 @@ static void pbx_load_users(void)
 			}
 
 			/* Add hint */
-			ast_add_extension2(con, 0, cat, -1, NULL, NULL, iface, NULL, NULL, registrar);
+			ast_add_extension2(con, 0, cat, -1, NULL, NULL, iface, NULL, NULL, registrar, NULL, 0);
 			/* If voicemail, use "stdexten" else use plain old dial */
 			if (hasvoicemail) {
 				if (ast_opt_stdexten_macro) {
 					/* Use legacy stdexten macro method. */
 					snprintf(tmp, sizeof(tmp), "stdexten,%s,${HINT}", cat);
-					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Macro", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Macro", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 				} else {
 					snprintf(tmp, sizeof(tmp), "%s,stdexten(${HINT})", cat);
-					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Gosub", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Gosub", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 				}
 			} else {
-				ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Dial", ast_strdup("${HINT}"), ast_free_ptr, registrar);
+				ast_add_extension2(con, 0, cat, 1, NULL, NULL, "Dial", ast_strdup("${HINT}"), ast_free_ptr, registrar, NULL, 0);
 			}
 			altexts = ast_variable_retrieve(cfg, cat, "alternateexts");
 			if (!ast_strlen_zero(altexts)) {
@@ -2030,7 +2075,7 @@ static void pbx_load_users(void)
 				c = altcopy;
 				ext = strsep(&c, ",");
 				while (ext) {
-					ast_add_extension2(con, 0, ext, 1, NULL, NULL, "Goto", ast_strdup(tmp), ast_free_ptr, registrar);
+					ast_add_extension2(con, 0, ext, 1, NULL, NULL, "Goto", ast_strdup(tmp), ast_free_ptr, registrar, NULL, 0);
 					ext = strsep(&c, ",");
 				}
 			}
@@ -2045,10 +2090,17 @@ static int pbx_load_module(void)
 
 	ast_mutex_lock(&reload_lock);
 
-	if (!local_table)
+	if (!local_table) {
 		local_table = ast_hashtab_create(17, ast_hashtab_compare_contexts, ast_hashtab_resize_java, ast_hashtab_newsize_java, ast_hashtab_hash_contexts, 0);
+		if (!local_table) {
+			ast_mutex_unlock(&reload_lock);
+			return AST_MODULE_LOAD_DECLINE;
+		}
+	}
 
 	if (!pbx_load_config(config)) {
+		ast_hashtab_destroy(local_table, NULL);
+		local_table = NULL;
 		ast_mutex_unlock(&reload_lock);
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -2075,6 +2127,11 @@ static int load_module(void)
 {
 	int res;
 
+	if (pbx_load_module()) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
 	if (static_config && !write_protect_config)
 		ast_cli_register(&cli_dialplan_save);
 	ast_cli_register_multiple(cli_pbx_config, ARRAY_LEN(cli_pbx_config));
@@ -2089,9 +2146,6 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	if (pbx_load_module())
-		return AST_MODULE_LOAD_DECLINE;
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -2103,8 +2157,8 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Text Extension Configuration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_dundi.c 1:16.1.1~dfsg-1/pbx/pbx_dundi.c
--- 1:13.23.1~dfsg-2/pbx/pbx_dundi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_dundi.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,12 +39,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/network.h"
 #include <sys/ioctl.h>
 #include <zlib.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <pthread.h>
 #include <net/if.h>
 
@@ -244,7 +242,7 @@ struct dundi_precache_queue {
 struct dundi_request;
 
 struct dundi_transaction {
-	struct sockaddr_in addr;                       /*!< Other end of transaction */
+	struct ast_sockaddr addr;                      /*!< Other end of transaction */
 	struct timeval start;                          /*!< When this transaction was created */
 	dundi_eid eids[DUNDI_MAX_STACK + 1];
 	int eidcount;                                  /*!< Number of eids in eids */
@@ -303,7 +301,7 @@ struct dundi_mapping {
 
 struct dundi_peer {
 	dundi_eid eid;
-	struct sockaddr_in addr;               /*!< Address of DUNDi peer */
+	struct ast_sockaddr addr;              /*!< Address of DUNDi peer */
 	AST_LIST_HEAD_NOLOCK(permissionlist, permission) permit;
 	struct permissionlist include;
 	dundi_eid us_eid;
@@ -409,18 +407,20 @@ static int str2tech(char *str)
 static int dundi_lookup_internal(struct dundi_result *result, int maxret, struct ast_channel *chan, const char *dcontext, const char *number, int ttl, int blockempty, struct dundi_hint_metadata *md, int *expiration, int cybpass, int modeselect, dundi_eid *skip, dundi_eid *avoid[], int direct[]);
 static int dundi_precache_internal(const char *context, const char *number, int ttl, dundi_eid *avoids[]);
 static struct dundi_transaction *create_transaction(struct dundi_peer *p);
-static struct dundi_transaction *find_transaction(struct dundi_hdr *hdr, struct sockaddr_in *sin)
+
+static struct dundi_transaction *find_transaction(struct dundi_hdr *hdr, struct ast_sockaddr *sin)
 {
 	struct dundi_transaction *trans;
 
 	/* Look for an exact match first */
 	AST_LIST_TRAVERSE(&alltrans, trans, all) {
-		if (!inaddrcmp(&trans->addr, sin) &&
-		     ((trans->strans == (ntohs(hdr->dtrans) & 32767)) /* Matches our destination */ ||
-			  ((trans->dtrans == (ntohs(hdr->strans) & 32767)) && (!hdr->dtrans))) /* We match their destination */) {
-			  if (hdr->strans)
-				  trans->dtrans = ntohs(hdr->strans) & 32767;
-			  return trans;
+		if (!ast_sockaddr_cmp(&trans->addr, sin) &&
+			((trans->strans == (ntohs(hdr->dtrans) & 32767)) /* Matches our destination */ ||
+				((trans->dtrans == (ntohs(hdr->strans) & 32767)) && (!hdr->dtrans))) /* We match their destination */) {
+			if (hdr->strans) {
+				trans->dtrans = ntohs(hdr->strans) & 32767;
+			}
+			return trans;
 		}
 	}
 
@@ -436,7 +436,7 @@ static struct dundi_transaction *find_tr
 		/* Create new transaction */
 		if (!(trans = create_transaction(NULL)))
 			break;
-		memcpy(&trans->addr, sin, sizeof(trans->addr));
+		ast_sockaddr_copy(&trans->addr, sin);
 		trans->dtrans = ntohs(hdr->strans) & 32767;
 	default:
 		break;
@@ -451,7 +451,7 @@ static int dundi_ack(struct dundi_transa
 {
 	return dundi_send(trans, DUNDI_COMMAND_ACK, 0, final, NULL);
 }
-static void dundi_reject(struct dundi_hdr *h, struct sockaddr_in *sin)
+static void dundi_reject(struct dundi_hdr *h, struct ast_sockaddr *sin)
 {
 	struct {
 		struct dundi_packet pack;
@@ -463,7 +463,7 @@ static void dundi_reject(struct dundi_hd
 		return;
 	memset(&tmp, 0, sizeof(tmp));
 	memset(&trans, 0, sizeof(trans));
-	memcpy(&trans.addr, sin, sizeof(trans.addr));
+	ast_sockaddr_copy(&trans.addr, sin);
 	tmp.hdr.strans = h->dtrans;
 	tmp.hdr.dtrans = h->strans;
 	tmp.hdr.iseqno = h->oseqno;
@@ -1293,8 +1293,9 @@ static void qualify_peer(struct dundi_pe
 
 static void apply_peer(struct dundi_transaction *trans, struct dundi_peer *p)
 {
-	if (!trans->addr.sin_addr.s_addr)
-		memcpy(&trans->addr, &p->addr, sizeof(trans->addr));
+	if (ast_sockaddr_isnull(&trans->addr)) {
+		ast_sockaddr_copy(&trans->addr, &p->addr);
+	}
 	trans->us_eid = p->us_eid;
 	trans->them_eid = p->eid;
 	/* Enable encryption if appropriate */
@@ -1325,7 +1326,7 @@ static int do_register_expire(const void
 	ast_db_del("dundi/dpeers", dundi_eid_to_str_short(eid_str, sizeof(eid_str), &peer->eid));
 	peer->registerexpire = -1;
 	peer->lastms = 0;
-	memset(&peer->addr, 0, sizeof(peer->addr));
+	ast_sockaddr_setnull(&peer->addr);
 	return 0;
 }
 
@@ -1721,17 +1722,16 @@ static int handle_command_response(struc
 				int needqual = 0;
 				AST_SCHED_DEL(sched, peer->registerexpire);
 				peer->registerexpire = ast_sched_add(sched, (expire + 10) * 1000, do_register_expire, peer);
-				snprintf(data, sizeof(data), "%s:%d:%d", ast_inet_ntoa(trans->addr.sin_addr),
-					ntohs(trans->addr.sin_port), expire);
+				snprintf(data, sizeof(data), "%s:%d", ast_sockaddr_stringify(&trans->addr), expire);
 				ast_db_put("dundi/dpeers", dundi_eid_to_str_short(eid_str, sizeof(eid_str), &peer->eid), data);
-				if (inaddrcmp(&peer->addr, &trans->addr)) {
-					ast_verb(3, "Registered DUNDi peer '%s' at '%s:%d'\n",
-							ast_eid_to_str(eid_str, sizeof(eid_str), &peer->eid),
-							ast_inet_ntoa(trans->addr.sin_addr), ntohs(trans->addr.sin_port));
+				if (ast_sockaddr_cmp(&peer->addr, &trans->addr)) {
+					ast_verb(3, "Registered DUNDi peer '%s' at '%s'\n",
+						ast_eid_to_str(eid_str, sizeof(eid_str), &peer->eid),
+						ast_sockaddr_stringify(&trans->addr));
 					needqual = 1;
 				}
 
-				memcpy(&peer->addr, &trans->addr, sizeof(peer->addr));
+				ast_sockaddr_copy(&peer->addr, &trans->addr);
 				dundi_ie_append_short(ied, DUNDI_IE_EXPIRATION, default_expiration);
 				dundi_send(trans, DUNDI_COMMAND_REGRESPONSE, 0, 1, ied);
 				if (needqual)
@@ -1855,7 +1855,7 @@ static int handle_command_response(struc
 							ast_copy_string(trans->parent->dei->ipaddr, ies.q_ipaddr, sizeof(trans->parent->dei->ipaddr));
 						if (!ast_eid_cmp(&trans->them_eid, &trans->parent->query_eid)) {
 							/* If it's them, update our address */
-							ast_copy_string(trans->parent->dei->ipaddr, ast_inet_ntoa(trans->addr.sin_addr), sizeof(trans->parent->dei->ipaddr));
+							ast_copy_string(trans->parent->dei->ipaddr, ast_sockaddr_stringify_addr(&trans->addr), sizeof(trans->parent->dei->ipaddr));
 						}
 					}
 					if (ies.hint) {
@@ -2038,7 +2038,7 @@ static int ack_trans(struct dundi_transa
 	return 0;
 }
 
-static int handle_frame(struct dundi_hdr *h, struct sockaddr_in *sin, int datalen)
+static int handle_frame(struct dundi_hdr *h, struct ast_sockaddr *sin, int datalen)
 {
 	struct dundi_transaction *trans;
 	trans = find_transaction(h, sin);
@@ -2081,13 +2081,12 @@ static int handle_frame(struct dundi_hdr
 
 static int socket_read(int *id, int fd, short events, void *cbdata)
 {
-	struct sockaddr_in sin;
+	struct ast_sockaddr sin;
 	int res;
 	struct dundi_hdr *h;
 	char buf[MAX_PACKET_SIZE];
-	socklen_t len = sizeof(sin);
 
-	res = recvfrom(netsocket, buf, sizeof(buf) - 1, 0,(struct sockaddr *) &sin, &len);
+	res = ast_recvfrom(netsocket, buf, sizeof(buf), 0, &sin);
 	if (res < 0) {
 		if (errno != ECONNREFUSED)
 			ast_log(LOG_WARNING, "Error: %s\n", strerror(errno));
@@ -2590,6 +2589,26 @@ static char *dundi_do_query(struct ast_c
 	return CLI_SUCCESS;
 }
 
+static char *dundi_sockaddr_stringify_host(const struct ast_sockaddr *addr)
+{
+	if (ast_sockaddr_isnull(addr)) {
+		return "(Unspecified)";
+	}
+	return ast_sockaddr_stringify_host(addr);
+}
+
+static uint16_t dundi_sockaddr_port(const struct ast_sockaddr *addr)
+{
+	/*
+	 * Test to avoid a debug message complaining about addr
+	 * not being an IPv4 or IPv6 address.
+	 */
+	if (ast_sockaddr_isnull(addr)) {
+		return 0;
+	}
+	return ast_sockaddr_port(addr);
+}
+
 static char *dundi_show_peer(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	struct dundi_peer *peer;
@@ -2635,8 +2654,8 @@ static char *dundi_show_peer(struct ast_
 		ast_cli(a->fd, "Peer:    %s\n", ast_eid_to_str(eid_str, sizeof(eid_str), &peer->eid));
 		ast_cli(a->fd, "Model:   %s\n", model2str(peer->model));
 		ast_cli(a->fd, "Order:   %s\n", order);
-		ast_cli(a->fd, "Host:    %s\n", peer->addr.sin_addr.s_addr ? ast_inet_ntoa(peer->addr.sin_addr) : "<Unspecified>");
-		ast_cli(a->fd, "Port:    %d\n", ntohs(peer->addr.sin_port));
+		ast_cli(a->fd, "Host:    %s\n", ast_sockaddr_isnull(&peer->addr) ? "<Unspecified>" : ast_sockaddr_stringify_host(&peer->addr));
+		ast_cli(a->fd, "Port:    %d\n", dundi_sockaddr_port(&peer->addr));
 		ast_cli(a->fd, "Dynamic: %s\n", peer->dynamic ? "yes" : "no");
 		ast_cli(a->fd, "Reg:     %s\n", peer->registerid < 0 ? "No" : "Yes");
 		ast_cli(a->fd, "In Key:  %s\n", ast_strlen_zero(peer->inkey) ? "<None>" : peer->inkey);
@@ -2668,8 +2687,8 @@ static char *dundi_show_peer(struct ast_
 
 static char *dundi_show_peers(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-#define FORMAT2 "%-20.20s %-15.15s     %-6.6s %-10.10s %-8.8s %-15.15s\n"
-#define FORMAT "%-20.20s %-15.15s %s %-6d %-10.10s %-8.8s %-15.15s\n"
+#define FORMAT2 "%-20.20s %-41s     %-6.6s %-10.10s %-8.8s %-15.15s\n"
+#define FORMAT "%-20.20s %-41s %s %-6d %-10.10s %-8.8s %-15.15s\n"
 	struct dundi_peer *peer;
 	int registeredonly=0;
 	char avgms[20];
@@ -2706,9 +2725,11 @@ static char *dundi_show_peers(struct ast
 		char status[20];
 		int print_line = -1;
 		char srch[2000];
+
 		total_peers++;
-		if (registeredonly && !peer->addr.sin_addr.s_addr)
+		if (registeredonly && ast_sockaddr_isnull(&peer->addr)) {
 			continue;
+		}
 		if (peer->maxms) {
 			if (peer->lastms < 0) {
 				strcpy(status, "UNREACHABLE");
@@ -2734,9 +2755,9 @@ static char *dundi_show_peers(struct ast
 			snprintf(avgms, sizeof(avgms), "%d ms", peer->avgms);
 		else
 			strcpy(avgms, "Unavail");
-		snprintf(srch, sizeof(srch), FORMAT, ast_eid_to_str(eid_str, sizeof(eid_str), &peer->eid),
-					peer->addr.sin_addr.s_addr ? ast_inet_ntoa(peer->addr.sin_addr) : "(Unspecified)",
-					peer->dynamic ? "(D)" : "(S)", ntohs(peer->addr.sin_port), model2str(peer->model), avgms, status);
+		snprintf(srch, sizeof(srch), FORMAT, ast_eid_to_str(eid_str, sizeof(eid_str),
+			&peer->eid), dundi_sockaddr_stringify_host(&peer->addr),
+			peer->dynamic ? "(D)" : "(S)", dundi_sockaddr_port(&peer->addr), model2str(peer->model), avgms, status);
 
                 if (a->argc == 5) {
                   if (!strcasecmp(a->argv[3],"include") && strstr(srch,a->argv[4])) {
@@ -2752,8 +2773,8 @@ static char *dundi_show_peers(struct ast
 
         if (print_line) {
 			ast_cli(a->fd, FORMAT, ast_eid_to_str(eid_str, sizeof(eid_str), &peer->eid),
-					peer->addr.sin_addr.s_addr ? ast_inet_ntoa(peer->addr.sin_addr) : "(Unspecified)",
-					peer->dynamic ? "(D)" : "(S)", ntohs(peer->addr.sin_port), model2str(peer->model), avgms, status);
+				dundi_sockaddr_stringify_host(&peer->addr),
+				peer->dynamic ? "(D)" : "(S)", dundi_sockaddr_port(&peer->addr), model2str(peer->model), avgms, status);
 		}
 	}
 	ast_cli(a->fd, "%d dundi peers [%d online, %d offline, %d unmonitored]\n", total_peers, online_peers, offline_peers, unmonitored_peers);
@@ -2765,8 +2786,8 @@ static char *dundi_show_peers(struct ast
 
 static char *dundi_show_trans(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-#define FORMAT2 "%-22.22s %-5.5s %-5.5s %-3.3s %-3.3s %-3.3s\n"
-#define FORMAT "%-16.16s:%5d %-5.5d %-5.5d %-3.3d %-3.3d %-3.3d\n"
+#define FORMAT2 "%-47s %-5.5s %-5.5s %-3.3s %-3.3s %-3.3s\n"
+#define FORMAT "%-41s:%5d %-5.5d %-5.5d %-3.3d %-3.3d %-3.3d\n"
 	struct dundi_transaction *trans;
 	switch (cmd) {
 	case CLI_INIT:
@@ -2784,8 +2805,9 @@ static char *dundi_show_trans(struct ast
 	AST_LIST_LOCK(&peers);
 	ast_cli(a->fd, FORMAT2, "Remote", "Src", "Dst", "Tx", "Rx", "Ack");
 	AST_LIST_TRAVERSE(&alltrans, trans, all) {
-		ast_cli(a->fd, FORMAT, ast_inet_ntoa(trans->addr.sin_addr),
-			ntohs(trans->addr.sin_port), trans->strans, trans->dtrans, trans->oseqno, trans->iseqno, trans->aseqno);
+		ast_cli(a->fd, FORMAT, ast_sockaddr_stringify_host(&trans->addr),
+			ast_sockaddr_port(&trans->addr), trans->strans, trans->dtrans,
+			trans->oseqno, trans->iseqno, trans->aseqno);
 	}
 	AST_LIST_UNLOCK(&peers);
 	return CLI_SUCCESS;
@@ -3105,8 +3127,9 @@ static struct dundi_transaction *create_
 	int tid;
 
 	/* Don't allow creation of transactions to non-registered peers */
-	if (p && !p->addr.sin_addr.s_addr)
+	if (p && ast_sockaddr_isnull(&p->addr)) {
 		return NULL;
+	}
 	tid = get_trans_id();
 	if (tid < 1)
 		return NULL;
@@ -3135,11 +3158,10 @@ static int dundi_xmit(struct dundi_packe
 	int res;
 	if (dundidebug)
 		dundi_showframe(pack->h, 0, &pack->parent->addr, pack->datalen - sizeof(struct dundi_hdr));
-	res = sendto(netsocket, pack->data, pack->datalen, 0, (struct sockaddr *)&pack->parent->addr, sizeof(pack->parent->addr));
+	res = ast_sendto(netsocket, pack->data, pack->datalen, 0, &pack->parent->addr);
 	if (res < 0) {
-		ast_log(LOG_WARNING, "Failed to transmit to '%s:%d': %s\n",
-			ast_inet_ntoa(pack->parent->addr.sin_addr),
-			ntohs(pack->parent->addr.sin_port), strerror(errno));
+		ast_log(LOG_WARNING, "Failed to transmit to '%s': %s\n",
+			ast_sockaddr_stringify(&pack->parent->addr), strerror(errno));
 	}
 	if (res > 0)
 		res = 0;
@@ -3244,10 +3266,10 @@ static int dundi_rexmit(const void *data
 	AST_LIST_LOCK(&peers);
 	if (pack->retrans < 1) {
 		pack->retransid = -1;
-		if (!ast_test_flag(pack->parent, FLAG_ISQUAL))
-			ast_log(LOG_NOTICE, "Max retries exceeded to host '%s:%d' msg %d on call %d\n",
-				ast_inet_ntoa(pack->parent->addr.sin_addr),
-				ntohs(pack->parent->addr.sin_port), pack->h->oseqno, ntohs(pack->h->strans));
+		if (!ast_test_flag(pack->parent, FLAG_ISQUAL)) {
+			ast_log(LOG_NOTICE, "Max retries exceeded to host '%s' msg %d on call %d\n",
+				ast_sockaddr_stringify(&pack->parent->addr), pack->h->oseqno, ntohs(pack->h->strans));
+		}
 		destroy_trans(pack->parent, 1);
 		res = 0;
 	} else {
@@ -3602,8 +3624,9 @@ static int append_transaction(struct dun
 	char eid_str2[20];
 
 	/* Ignore if not registered */
-	if (!p->addr.sin_addr.s_addr)
+	if (ast_sockaddr_isnull(&p->addr)) {
 		return 0;
+	}
 	if (p->maxms && ((p->lastms < 0) || (p->lastms >= p->maxms)))
 		return 0;
 
@@ -4548,15 +4571,31 @@ static void populate_addr(struct dundi_p
 	char eid_str[20];
 	ast_eid_to_str(eid_str, sizeof(eid_str), eid);
 	if (!ast_db_get("dundi/dpeers", eid_str, data, sizeof(data))) {
-		c = strchr(data, ':');
+		/*
+		 * data is in the form:
+		 * IPv6 address: [ffff:ffff::ffff:ffff]:port:expire
+		 * IPv4 address: a.b.c.d:port:expire
+		 */
+		c = data;
+		if (*c == '[') {
+			/* Need to skip over the IPv6 address. */
+			c = strchr(c, ']');
+		}
+		if (c) {
+			c = strchr(c, ':');
+		}
 		if (c) {
 			*c = '\0';
 			c++;
 			if (sscanf(c, "%5d:%30d", &port, &expire) == 2) {
 				/* Got it! */
-				inet_aton(data, &peer->addr.sin_addr);
-				peer->addr.sin_family = AF_INET;
-				peer->addr.sin_port = htons(port);
+				struct ast_sockaddr *addrs;
+
+				if (ast_sockaddr_resolve(&addrs, data, PARSE_PORT_FORBID, AF_UNSPEC) > 0){
+					ast_sockaddr_copy(&peer->addr, &addrs[0]);
+					ast_free(addrs);
+				}
+				ast_sockaddr_set_port(&peer->addr, port);
 				peer->registerexpire = ast_sched_add(sched, (expire + 10) * 1000, do_register_expire, peer);
 			}
 		}
@@ -4567,11 +4606,10 @@ static void populate_addr(struct dundi_p
 static void build_peer(dundi_eid *eid, struct ast_variable *v, int *globalpcmode)
 {
 	struct dundi_peer *peer;
-	struct ast_hostent he;
-	struct hostent *hp;
 	dundi_eid testeid;
 	int needregister=0;
 	char eid_str[20];
+	int port = 0;
 
 	AST_LIST_LOCK(&peers);
 	AST_LIST_TRAVERSE(&peers, peer, list) {
@@ -4588,8 +4626,6 @@ static void build_peer(dundi_eid *eid, s
 		peer->registerid = -1;
 		peer->registerexpire = -1;
 		peer->qualifyid = -1;
-		peer->addr.sin_family = AF_INET;
-		peer->addr.sin_port = htons(DUNDI_PORT);
 		populate_addr(peer, eid);
 		AST_LIST_INSERT_HEAD(&peers, peer, list);
 	}
@@ -4605,15 +4641,17 @@ static void build_peer(dundi_eid *eid, s
 		} else if (!strcasecmp(v->name, "outkey")) {
 			ast_copy_string(peer->outkey, v->value, sizeof(peer->outkey));
 		} else if (!strcasecmp(v->name, "port")) {
-			peer->addr.sin_port = htons(atoi(v->value));
+			port = atoi(v->value);
 		} else if (!strcasecmp(v->name, "host")) {
 			if (!strcasecmp(v->value, "dynamic")) {
 				peer->dynamic = 1;
 			} else {
-				hp = ast_gethostbyname(v->value, &he);
-				if (hp) {
-					memcpy(&peer->addr.sin_addr, hp->h_addr, sizeof(peer->addr.sin_addr));
+				struct ast_sockaddr *addrs;
+
+				if (ast_sockaddr_resolve(&addrs, v->value, PARSE_PORT_FORBID, AF_UNSPEC) > 0) {
+					ast_sockaddr_copy(&peer->addr, &addrs[0]);
 					peer->dynamic = 0;
+					ast_free(addrs);
 				} else {
 					ast_log(LOG_WARNING, "Unable to find host '%s' at line %d\n", v->value, v->lineno);
 					peer->dead = 1;
@@ -4684,6 +4722,11 @@ static void build_peer(dundi_eid *eid, s
 			}
 		}
 	}
+
+	if (!ast_sockaddr_isnull(&peer->addr)) {
+		ast_sockaddr_set_port(&peer->addr, (0 < port) ? port : DUNDI_PORT);
+	}
+
 	(*globalpcmode) |= peer->pcmodel;
 	if (!peer->model && !peer->pcmodel) {
 		ast_log(LOG_WARNING, "Peer '%s' lacks a model or pcmodel, discarding!\n",
@@ -4833,20 +4876,54 @@ static struct ast_switch dundi_switch =
 	.matchmore   = dundi_matchmore,
 };
 
-static int set_config(char *config_file, struct sockaddr_in* sin, int reload)
+static int get_ipaddress(char *ip, size_t size, const char *str, int family)
+{
+	struct ast_sockaddr *addrs;
+
+	if (!ast_sockaddr_resolve(&addrs, str, 0, family)) {
+		return -1;
+	}
+
+	ast_copy_string(ip, ast_sockaddr_stringify_host(&addrs[0]), size);
+	ast_free(addrs);
+
+	return 0;
+}
+
+static void set_host_ipaddr(struct ast_sockaddr *sin)
+{
+	char hn[MAXHOSTNAMELEN];
+	struct addrinfo hints;
+	int family;
+
+	memset(&hints, 0, sizeof(hints));
+
+	if (ast_sockaddr_is_ipv6(sin)) {
+		family = AF_INET6;
+	} else {
+		family = AF_INET;
+	}
+
+	if (gethostname(hn, sizeof(hn) - 1) < 0) {
+		ast_log(LOG_WARNING, "Unable to get host name!\n");
+		return;
+	}
+
+	get_ipaddress(ipaddr, sizeof(ipaddr), hn, family);
+}
+
+static int set_config(char *config_file, struct ast_sockaddr *sin, int reload)
 {
 	struct ast_config *cfg;
 	struct ast_variable *v;
 	char *cat;
 	int x;
 	struct ast_flags config_flags = { 0 };
-	char hn[MAXHOSTNAMELEN] = "";
-	struct ast_hostent he;
-	struct hostent *hp;
-	struct sockaddr_in sin2;
 	static int last_port = 0;
+	int port = 0;
 	int globalpcmodel = 0;
 	dundi_eid testeid;
+	char bind_addr[80];
 
 	if (!(cfg = ast_config_load(config_file, config_flags)) || cfg == CONFIG_STATUS_FILEINVALID) {
 		ast_log(LOG_ERROR, "Unable to load config %s\n", config_file);
@@ -4857,16 +4934,6 @@ static int set_config(char *config_file,
 	dundi_cache_time = DUNDI_DEFAULT_CACHE_TIME;
 	any_peer = NULL;
 
-	ipaddr[0] = '\0';
-	if (!gethostname(hn, sizeof(hn)-1)) {
-		hp = ast_gethostbyname(hn, &he);
-		if (hp) {
-			memcpy(&sin2.sin_addr, hp->h_addr, sizeof(sin2.sin_addr));
-			ast_copy_string(ipaddr, ast_inet_ntoa(sin2.sin_addr), sizeof(ipaddr));
-		} else
-			ast_log(LOG_WARNING, "Unable to look up host '%s'\n", hn);
-	} else
-		ast_log(LOG_WARNING, "Unable to get host name!\n");
 	AST_LIST_LOCK(&peers);
 
 	if (ast_eid_is_empty(&ast_eid_default)) {
@@ -4879,19 +4946,13 @@ static int set_config(char *config_file,
 	v = ast_variable_browse(cfg, "general");
 	while(v) {
 		if (!strcasecmp(v->name, "port")){
-			sin->sin_port = htons(atoi(v->value));
-			if(last_port==0){
-				last_port=sin->sin_port;
-			} else if(sin->sin_port != last_port)
-				ast_log(LOG_WARNING, "change to port ignored until next asterisk re-start\n");
+			port = atoi(v->value);
 		} else if (!strcasecmp(v->name, "bindaddr")) {
-			struct hostent *hep;
-			struct ast_hostent hent;
-			hep = ast_gethostbyname(v->value, &hent);
-			if (hep) {
-				memcpy(&sin->sin_addr, hep->h_addr, sizeof(sin->sin_addr));
-			} else
-				ast_log(LOG_WARNING, "Invalid host/IP '%s'\n", v->value);
+			if (get_ipaddress(bind_addr, sizeof(bind_addr), v->value, AF_UNSPEC) == 0) {
+				if (!ast_sockaddr_parse(sin, bind_addr, 0)) {
+					ast_log(LOG_WARNING, "Invalid host/IP '%s'\n", v->value);
+				}
+			}
 		} else if (!strcasecmp(v->name, "authdebug")) {
 			authdebug = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "ttl")) {
@@ -4946,6 +5007,26 @@ static int set_config(char *config_file,
 		}
 		v = v->next;
 	}
+
+	if (port == 0) {
+		port = DUNDI_PORT;
+	}
+
+	if (ast_sockaddr_isnull(sin)) {
+		sprintf(bind_addr, "0.0.0.0:%d", port);
+		ast_sockaddr_parse(sin, bind_addr, 0);
+	} else {
+		ast_sockaddr_set_port(sin, port);
+	}
+
+	if (last_port == 0) {
+		last_port = port;
+	} else if (last_port != port) {
+		ast_log(LOG_WARNING, "change to port ignored until next asterisk re-start\n");
+	}
+
+	set_host_ipaddr(sin);
+
 	AST_LIST_UNLOCK(&peers);
 
 	mark_mappings();
@@ -5032,7 +5113,9 @@ static int unload_module(void)
 
 static int reload(void)
 {
-	struct sockaddr_in sin;
+	struct ast_sockaddr sin;
+
+	ast_sockaddr_setnull(&sin);
 
 	if (set_config("dundi.conf", &sin, 1))
 		return AST_MODULE_LOAD_FAILURE;
@@ -5042,15 +5125,11 @@ static int reload(void)
 
 static int load_module(void)
 {
-	struct sockaddr_in sin;
+	struct ast_sockaddr sin;
 
 	dundi_set_output(dundi_debug_output);
 	dundi_set_error(dundi_error_output);
 
-	sin.sin_family = AF_INET;
-	sin.sin_port = htons(DUNDI_PORT);
-	sin.sin_addr.s_addr = INADDR_ANY;
-
 	/* Make a UDP socket */
 	io = io_context_create();
 	sched = ast_sched_context_create();
@@ -5059,19 +5138,25 @@ static int load_module(void)
 		goto declined;
 	}
 
+	ast_sockaddr_setnull(&sin);
+
 	if (set_config("dundi.conf", &sin, 0)) {
 		goto declined;
 	}
 
-	netsocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
+	if (ast_sockaddr_is_ipv6(&sin)) {
+		netsocket = socket(AF_INET6, SOCK_DGRAM, IPPROTO_IP);
+	} else {
+		netsocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_IP);
+	}
 
 	if (netsocket < 0) {
 		ast_log(LOG_ERROR, "Unable to create network socket: %s\n", strerror(errno));
 		goto declined;
 	}
-	if (bind(netsocket, (struct sockaddr *) &sin, sizeof(sin))) {
-		ast_log(LOG_ERROR, "Unable to bind to %s port %d: %s\n",
-			ast_inet_ntoa(sin.sin_addr), ntohs(sin.sin_port), strerror(errno));
+	if (ast_bind(netsocket, &sin))  {
+		ast_log(LOG_ERROR, "Unable to bind to %s : %s\n",
+			ast_sockaddr_stringify(&sin), strerror(errno));
 		goto declined;
 	}
 
@@ -5089,7 +5174,7 @@ static int load_module(void)
 	ast_custom_function_register(&dundi_query_function);
 	ast_custom_function_register(&dundi_result_function);
 
-	ast_verb(2, "DUNDi Ready and Listening on %s port %d\n", ast_inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
+	ast_verb(2, "DUNDi Ready and Listening on %s\n", ast_sockaddr_stringify(&sin));
 
 	return AST_MODULE_LOAD_SUCCESS;
 
@@ -5099,9 +5184,9 @@ declined:
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Distributed Universal Number Discovery (DUNDi)",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.nonoptreq = "res_crypto",
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.optional_modules = "res_crypto",
+);
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_loopback.c 1:16.1.1~dfsg-1/pbx/pbx_loopback.c
--- 1:13.23.1~dfsg-2/pbx/pbx_loopback.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_loopback.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_lua.c 1:16.1.1~dfsg-1/pbx/pbx_lua.c
--- 1:13.23.1~dfsg-2/pbx/pbx_lua.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_lua.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -62,7 +60,7 @@ static char *registrar = "pbx_lua";
  * applications might return */
 #define LUA_GOTO_DETECTED 5
 
-static char *lua_read_extensions_file(lua_State *L, long *size);
+static char *lua_read_extensions_file(lua_State *L, long *size, int *file_not_openable);
 static int lua_load_extensions(lua_State *L, struct ast_channel *chan);
 static int lua_reload_extensions(lua_State *L);
 static void lua_free_extensions(void);
@@ -807,7 +805,19 @@ static int lua_error_function(lua_State
 	lua_pushliteral(L, "\n");
 
 	lua_getglobal(L, "debug");
+	if (!lua_istable(L, -1)) {
+		/* Have no 'debug' table for whatever reason */
+		lua_pop(L, 2);
+		/*  Original err message is on stack top now */
+		return 1;
+	}
 	lua_getfield(L, -1, "traceback");
+	if (!lua_isfunction(L, -1)) {
+		/* Same here for traceback function */
+		lua_pop(L, 3);
+		/*  Original err message is on stack top now */
+		return 1;
+	}
 	lua_remove(L, -2); /* remove the 'debug' table */
 
 	lua_pushvalue(L, message_index);
@@ -1028,7 +1038,7 @@ static int lua_register_hints(lua_State
 				continue;
 			}
 
-			if (ast_add_extension2(con, 0, hint_name, PRIORITY_HINT, NULL, NULL, hint_value, NULL, NULL, registrar)) {
+			if (ast_add_extension2(con, 0, hint_name, PRIORITY_HINT, NULL, NULL, hint_value, NULL, NULL, registrar, NULL, 0)) {
 				/* remove hints table, hint name, hint value,
 				 * key copy, context name, and contex table */
 				lua_pop(L, 6);
@@ -1072,12 +1082,13 @@ static int lua_extension_cmp(lua_State *
  *
  * \param L the lua_State to use
  * \param size a pointer to store the size of the buffer
+ * \param file_not_openable a pointer to store if config file could be opened
  *
  * \note The caller is expected to free the buffer at some point.
  *
  * \return a pointer to the buffer
  */
-static char *lua_read_extensions_file(lua_State *L, long *size)
+static char *lua_read_extensions_file(lua_State *L, long *size, int *file_not_openable)
 {
 	FILE *f;
 	int error_func;
@@ -1092,6 +1103,8 @@ static char *lua_read_extensions_file(lu
 		lua_pushstring(L, strerror(errno));
 		lua_concat(L, 4);
 
+		*file_not_openable = 1;
+
 		return NULL;
 	}
 
@@ -1201,10 +1214,14 @@ static int lua_reload_extensions(lua_Sta
 {
 	long size = 0;
 	char *data = NULL;
+	int file_not_openable = 0;
 
 	luaL_openlibs(L);
 
-	if (!(data = lua_read_extensions_file(L, &size))) {
+	if (!(data = lua_read_extensions_file(L, &size, &file_not_openable))) {
+		if (file_not_openable) {
+			return -1;
+		}
 		return 1;
 	}
 
@@ -1623,17 +1640,24 @@ static struct ast_switch lua_switch = {
 static int load_or_reload_lua_stuff(void)
 {
 	int res = AST_MODULE_LOAD_SUCCESS;
+	int loaded = 0;
 
 	lua_State *L = luaL_newstate();
 	if (!L) {
 		ast_log(LOG_ERROR, "Error allocating lua_State, no memory\n");
-		return AST_MODULE_LOAD_DECLINE;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
-	if (lua_reload_extensions(L)) {
+	loaded = lua_reload_extensions(L);
+	if (loaded) {
 		const char *error = lua_tostring(L, -1);
 		ast_log(LOG_ERROR, "Error loading extensions.lua: %s\n", error);
-		res = AST_MODULE_LOAD_DECLINE;
+
+		if (loaded < 0) {
+			res = AST_MODULE_LOAD_DECLINE;
+		} else {
+			res = AST_MODULE_LOAD_FAILURE;
+		}
 	}
 
 	lua_close(L);
@@ -1662,15 +1686,15 @@ static int load_module(void)
 
 	if (ast_register_switch(&lua_switch)) {
 		ast_log(LOG_ERROR, "Unable to register LUA PBX switch\n");
-		return AST_MODULE_LOAD_DECLINE;
+		return AST_MODULE_LOAD_FAILURE;
 	}
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "Lua PBX Switch",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+);
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_realtime.c 1:16.1.1~dfsg-1/pbx/pbx_realtime.c
--- 1:13.23.1~dfsg-2/pbx/pbx_realtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_realtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <signal.h>
 
 #include "asterisk/file.h"
@@ -403,7 +401,9 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	if (!(cache = ao2_container_alloc(573, cache_hash, cache_cmp))) {
+	cache = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 573,
+		cache_hash, NULL, cache_cmp);
+	if (!cache) {
 		return AST_MODULE_LOAD_FAILURE;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/pbx/pbx_spool.c 1:16.1.1~dfsg-1/pbx/pbx_spool.c
--- 1:13.23.1~dfsg-2/pbx/pbx_spool.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/pbx/pbx_spool.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,8 +28,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 #include <time.h>
 #include <utime.h>
@@ -163,6 +161,19 @@ static struct outgoing *new_outgoing(con
 	return o;
 }
 
+static void append_variable(struct outgoing *o, const char *name, const char *value)
+{
+	struct ast_variable *var = ast_variable_new(name, value, o->fn);
+
+	if (!var) {
+		return;
+	}
+
+	/* Always insert at the end, because some people want to treat the spool
+	 * file as a script */
+	ast_variable_list_append(&o->vars, var);
+}
+
 static void parse_line(char *line, unsigned int lineno, struct outgoing *o)
 {
 	char *c;
@@ -263,20 +274,7 @@ static void parse_line(char *line, unsig
 
 		strsep(&c2, "=");
 		if (c2) {
-			struct ast_variable *var = ast_variable_new(c, c2, o->fn);
-
-			if (var) {
-				/*
-				 * Always insert at the end, because some people
-				 * want to treat the spool file as a script
-				 */
-				struct ast_variable **tail = &o->vars;
-
-				while (*tail) {
-					tail = &(*tail)->next;
-				}
-				*tail = var;
-			}
+			append_variable(o, c, c2);
 		} else {
 			ast_log(LOG_WARNING, "Malformed \"%s\" argument.  Should be \"%s: variable=value\"\n", line, line);
 		}
@@ -330,6 +328,11 @@ static int apply_outgoing(struct outgoin
 			"along with tech and dest in file %s\n", o->fn);
 		return -1;
 	}
+
+	if (snprintf(buf, sizeof(buf), "%d", o->retries + 1) < sizeof(buf)) {
+		append_variable(o, "AST_OUTGOING_ATTEMPT", buf);
+	}
+
 	return 0;
 }
 
diff -pruN 1:13.23.1~dfsg-2/README.md 1:16.1.1~dfsg-1/README.md
--- 1:13.23.1~dfsg-2/README.md	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/README.md	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
 # The Asterisk(R) Open Source PBX
 ```text
         By Mark Spencer <markster@digium.com> and the Asterisk.org developer community.
-        Copyright (C) 2001-2009 Digium, Inc. and other copyright holders.
+        Copyright (C) 2001-2016 Digium, Inc. and other copyright holders.
 ```
 ## SECURITY
 
@@ -69,7 +69,7 @@ list of new features in this version of
 
   Ensure that your system contains a compatible compiler and development
 libraries.  Asterisk requires either the GNU Compiler Collection (GCC) version
-3.0 or higher, or a compiler that supports the C99 specification and some of
+4.1 or higher, or a compiler that supports the C99 specification and some of
 the gcc language extensions.  In addition, your system needs to have the C
 library headers available, and the headers and libraries for ncurses.
 
diff -pruN 1:13.23.1~dfsg-2/res/ael/ael.flex 1:16.1.1~dfsg-1/res/ael/ael.flex
--- 1:13.23.1~dfsg-2/res/ael/ael.flex	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ael/ael.flex	2018-12-26 18:45:13.000000000 +0000
@@ -68,8 +68,8 @@
 %option bison-locations
 
 %{
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -79,6 +79,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #if !defined(GLOB_ABORTED)
 #define GLOB_ABORTED GLOB_ABEND
 #endif
+#if !defined(GLOB_BRACE)
+#define GLOB_BRACE 0
+#endif
+#if !defined(GLOB_NOMAGIC)
+#define GLOB_NOMAGIC 0
+#endif
 
 #include "asterisk/logger.h"
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/res/ael/ael_lex.c 1:16.1.1~dfsg-1/res/ael/ael_lex.c
--- 1:13.23.1~dfsg-2/res/ael/ael_lex.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ael/ael_lex.c	2018-12-26 18:45:13.000000000 +0000
@@ -1,3 +1,4 @@
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
 #line 2 "ael_lex.c"
@@ -828,7 +829,6 @@ static yyconst flex_int16_t yy_chk[1073]
  */
 #line 71 "ael.flex"
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -838,6 +838,12 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #if !defined(GLOB_ABORTED)
 #define GLOB_ABORTED GLOB_ABEND
 #endif
+#if !defined(GLOB_BRACE)
+#define GLOB_BRACE 0
+#endif
+#if !defined(GLOB_NOMAGIC)
+#define GLOB_NOMAGIC 0
+#endif
 
 #include "asterisk/logger.h"
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/res/ael/ael.tab.c 1:16.1.1~dfsg-1/res/ael/ael.tab.c
--- 1:13.23.1~dfsg-2/res/ael/ael.tab.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ael/ael.tab.c	2018-12-26 18:45:13.000000000 +0000
@@ -99,10 +99,9 @@
  *
  */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
diff -pruN 1:13.23.1~dfsg-2/res/ael/ael.y 1:16.1.1~dfsg-1/res/ael/ael.y
--- 1:13.23.1~dfsg-2/res/ael/ael.y	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ael/ael.y	2018-12-26 18:45:13.000000000 +0000
@@ -22,10 +22,9 @@
  *
  */
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
diff -pruN 1:13.23.1~dfsg-2/res/ael/pval.c 1:16.1.1~dfsg-1/res/ael/pval.c
--- 1:13.23.1~dfsg-2/res/ael/pval.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ael/pval.c	2018-12-26 18:45:13.000000000 +0000
@@ -27,10 +27,9 @@
 	<support_level>extended</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_REDIRECT
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -4235,7 +4234,7 @@ void add_extensions(struct ael_extension
 		pbx_substitute_variables_helper(NULL, exten->name, realext, sizeof(realext) - 1);
 		if (exten->hints) {
 			if (ast_add_extension2(exten->context, 0 /*no replace*/, realext, PRIORITY_HINT, NULL, exten->cidmatch,
-								  exten->hints, NULL, ast_free_ptr, registrar)) {
+								  exten->hints, NULL, ast_free_ptr, registrar, NULL, 0)) {
 				ast_log(LOG_WARNING, "Unable to add step at priority 'hint' of extension '%s'\n",
 						exten->name);
 			}
@@ -4315,7 +4314,7 @@ void add_extensions(struct ael_extension
 				label = 0;
 
 			if (ast_add_extension2(exten->context, 0 /*no replace*/, realext, pr->priority_num, (label?label:NULL), exten->cidmatch,
-								  app, strdup(appargs), ast_free_ptr, registrar)) {
+								  app, strdup(appargs), ast_free_ptr, registrar, NULL, 0)) {
 				ast_log(LOG_WARNING, "Unable to add step at priority '%d' of extension '%s'\n", pr->priority_num,
 						exten->name);
 			}
@@ -4409,7 +4408,7 @@ static int context_used(struct ael_exten
 {
 	struct ael_extension *exten;
 	/* Check the simple elements first */
-	if (ast_walk_context_extensions(context, NULL) || ast_walk_context_includes(context, NULL) || ast_walk_context_ignorepats(context, NULL) || ast_walk_context_switches(context, NULL)) {
+	if (ast_walk_context_extensions(context, NULL) || ast_context_includes_count(context) || ast_context_ignorepats_count(context) || ast_context_switches_count(context)) {
 		return 1;
 	}
 	for (exten = exten_list; exten; exten = exten->next_exten) {
diff -pruN 1:13.23.1~dfsg-2/res/ari/ari_model_validators.c 1:16.1.1~dfsg-1/res/ari/ari_model_validators.c
--- 1:13.23.1~dfsg-2/res/ari/ari_model_validators.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/ari_model_validators.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/module.h"
 #include "ari_model_validators.h"
@@ -1053,6 +1051,15 @@ int ast_ari_validate_channel(struct ast_
 				res = 0;
 			}
 		} else
+		if (strcmp("channelvars", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			prop_is_valid = ast_ari_validate_object(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI Channel field channelvars failed validation\n");
+				res = 0;
+			}
+		} else
 		if (strcmp("connected", ast_json_object_iter_key(iter)) == 0) {
 			int prop_is_valid;
 			has_connected = 1;
@@ -1762,6 +1769,15 @@ int ast_ari_validate_playback(struct ast
 				res = 0;
 			}
 		} else
+		if (strcmp("next_media_uri", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			prop_is_valid = ast_ari_validate_string(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI Playback field next_media_uri failed validation\n");
+				res = 0;
+			}
+		} else
 		if (strcmp("state", ast_json_object_iter_key(iter)) == 0) {
 			int prop_is_valid;
 			has_state = 1;
@@ -5093,6 +5109,9 @@ int ast_ari_validate_event(struct ast_js
 	if (strcmp("PeerStatusChange", discriminator) == 0) {
 		return ast_ari_validate_peer_status_change(json);
 	} else
+	if (strcmp("PlaybackContinuing", discriminator) == 0) {
+		return ast_ari_validate_playback_continuing(json);
+	} else
 	if (strcmp("PlaybackFinished", discriminator) == 0) {
 		return ast_ari_validate_playback_finished(json);
 	} else
@@ -5294,6 +5313,9 @@ int ast_ari_validate_message(struct ast_
 	if (strcmp("PeerStatusChange", discriminator) == 0) {
 		return ast_ari_validate_peer_status_change(json);
 	} else
+	if (strcmp("PlaybackContinuing", discriminator) == 0) {
+		return ast_ari_validate_playback_continuing(json);
+	} else
 	if (strcmp("PlaybackFinished", discriminator) == 0) {
 		return ast_ari_validate_playback_finished(json);
 	} else
@@ -5607,6 +5629,94 @@ ari_validator ast_ari_validate_peer_stat
 	return ast_ari_validate_peer_status_change;
 }
 
+int ast_ari_validate_playback_continuing(struct ast_json *json)
+{
+	int res = 1;
+	struct ast_json_iter *iter;
+	int has_type = 0;
+	int has_application = 0;
+	int has_playback = 0;
+
+	for (iter = ast_json_object_iter(json); iter; iter = ast_json_object_iter_next(json, iter)) {
+		if (strcmp("asterisk_id", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			prop_is_valid = ast_ari_validate_string(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI PlaybackContinuing field asterisk_id failed validation\n");
+				res = 0;
+			}
+		} else
+		if (strcmp("type", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			has_type = 1;
+			prop_is_valid = ast_ari_validate_string(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI PlaybackContinuing field type failed validation\n");
+				res = 0;
+			}
+		} else
+		if (strcmp("application", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			has_application = 1;
+			prop_is_valid = ast_ari_validate_string(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI PlaybackContinuing field application failed validation\n");
+				res = 0;
+			}
+		} else
+		if (strcmp("timestamp", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			prop_is_valid = ast_ari_validate_date(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI PlaybackContinuing field timestamp failed validation\n");
+				res = 0;
+			}
+		} else
+		if (strcmp("playback", ast_json_object_iter_key(iter)) == 0) {
+			int prop_is_valid;
+			has_playback = 1;
+			prop_is_valid = ast_ari_validate_playback(
+				ast_json_object_iter_value(iter));
+			if (!prop_is_valid) {
+				ast_log(LOG_ERROR, "ARI PlaybackContinuing field playback failed validation\n");
+				res = 0;
+			}
+		} else
+		{
+			ast_log(LOG_ERROR,
+				"ARI PlaybackContinuing has undocumented field %s\n",
+				ast_json_object_iter_key(iter));
+			res = 0;
+		}
+	}
+
+	if (!has_type) {
+		ast_log(LOG_ERROR, "ARI PlaybackContinuing missing required field type\n");
+		res = 0;
+	}
+
+	if (!has_application) {
+		ast_log(LOG_ERROR, "ARI PlaybackContinuing missing required field application\n");
+		res = 0;
+	}
+
+	if (!has_playback) {
+		ast_log(LOG_ERROR, "ARI PlaybackContinuing missing required field playback\n");
+		res = 0;
+	}
+
+	return res;
+}
+
+ari_validator ast_ari_validate_playback_continuing_fn(void)
+{
+	return ast_ari_validate_playback_continuing;
+}
+
 int ast_ari_validate_playback_finished(struct ast_json *json)
 {
 	int res = 1;
diff -pruN 1:13.23.1~dfsg-2/res/ari/ari_model_validators.h 1:16.1.1~dfsg-1/res/ari/ari_model_validators.h
--- 1:13.23.1~dfsg-2/res/ari/ari_model_validators.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/ari_model_validators.h	2018-12-26 18:45:13.000000000 +0000
@@ -23,11 +23,10 @@
  * the validator's function pointer.
  *
  * The reason for this seamingly useless indirection is the way function
- * pointers interfere with module loading. Asterisk attempts to dlopen() each
- * module using \c RTLD_LAZY in order to read some metadata from the module.
- * Unfortunately, if you take the address of a function, the function has to be
- * resolvable at load time, even if \c RTLD_LAZY is specified. By moving the
- * function-address-taking into this module, we can once again be lazy.
+ * pointers used to interfere with module loading. Previously, Asterisk
+ * attempted to dlopen() each module using \c RTLD_LAZY in order to read some
+ * metadata from the module. Using functions to get the function pointer
+ * allowed us to be lazy.
  */
 
  /*
@@ -1205,6 +1204,24 @@ int ast_ari_validate_peer_status_change(
 ari_validator ast_ari_validate_peer_status_change_fn(void);
 
 /*!
+ * \brief Validator for PlaybackContinuing.
+ *
+ * Event showing the continuation of a media playback operation from one media URI to the next in the list.
+ *
+ * \param json JSON object to validate.
+ * \returns True (non-zero) if valid.
+ * \returns False (zero) if invalid.
+ */
+int ast_ari_validate_playback_continuing(struct ast_json *json);
+
+/*!
+ * \brief Function pointer to ast_ari_validate_playback_continuing().
+ *
+ * See \ref ast_ari_model_validators.h for more details.
+ */
+ari_validator ast_ari_validate_playback_continuing_fn(void);
+
+/*!
  * \brief Validator for PlaybackFinished.
  *
  * Event showing the completion of a media playback operation.
@@ -1432,6 +1449,7 @@ ari_validator ast_ari_validate_applicati
  * Channel
  * - accountcode: string (required)
  * - caller: CallerID (required)
+ * - channelvars: object
  * - connected: CallerID (required)
  * - creationtime: Date (required)
  * - dialplan: DialplanCEP (required)
@@ -1477,6 +1495,7 @@ ari_validator ast_ari_validate_applicati
  * - id: string (required)
  * - language: string
  * - media_uri: string (required)
+ * - next_media_uri: string
  * - state: string (required)
  * - target_uri: string (required)
  * DeviceState
@@ -1727,6 +1746,12 @@ ari_validator ast_ari_validate_applicati
  * - timestamp: Date
  * - endpoint: Endpoint (required)
  * - peer: Peer (required)
+ * PlaybackContinuing
+ * - asterisk_id: string
+ * - type: string (required)
+ * - application: string (required)
+ * - timestamp: Date
+ * - playback: Playback (required)
  * PlaybackFinished
  * - asterisk_id: string
  * - type: string (required)
diff -pruN 1:13.23.1~dfsg-2/res/ari/ari_websockets.c 1:16.1.1~dfsg-1/res/ari/ari_websockets.c
--- 1:13.23.1~dfsg-2/res/ari/ari_websockets.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/ari_websockets.c	2018-12-26 18:45:13.000000000 +0000
@@ -18,8 +18,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/ari.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/http_websocket.h"
@@ -201,3 +199,9 @@ void ari_handle_websocket(struct ast_web
 	ast_websocket_uri_cb(ser, &fake_urih, uri, method, get_params,
 		headers);
 }
+
+const char *ast_ari_websocket_session_id(
+	const struct ast_ari_websocket_session *session)
+{
+	return ast_websocket_session_id(session->ws_session);
+}
diff -pruN 1:13.23.1~dfsg-2/res/ari/cli.c 1:16.1.1~dfsg-1/res/ari/cli.c
--- 1:13.23.1~dfsg-2/res/ari/cli.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/cli.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,8 +24,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/cli.h"
 #include "asterisk/stasis_app.h"
diff -pruN 1:13.23.1~dfsg-2/res/ari/config.c 1:16.1.1~dfsg-1/res/ari/config.c
--- 1:13.23.1~dfsg-2/res/ari/config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/config.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,10 +24,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/config_options.h"
 #include "asterisk/http_websocket.h"
+#include "asterisk/app.h"
+#include "asterisk/channel.h"
 #include "internal.h"
 
 /*! \brief Locking container for safe configuration access. */
@@ -318,6 +318,22 @@ static int process_config(int reload)
 	return 0;
 }
 
+#define MAX_VARS 128
+
+static int channelvars_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
+{
+	char *parse = NULL;
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(vars)[MAX_VARS];
+	);
+
+	parse = ast_strdupa(var->value);
+	AST_STANDARD_APP_ARGS(args, parse);
+
+	ast_channel_set_ari_vars(args.argc, args.vars);
+	return 0;
+}
+
 int ast_ari_config_init(void)
 {
 	if (aco_info_init(&cfg_info)) {
@@ -341,6 +357,8 @@ int ast_ari_config_init(void)
 	aco_option_register(&cfg_info, "websocket_write_timeout", ACO_EXACT, general_options,
 		AST_DEFAULT_WEBSOCKET_WRITE_TIMEOUT_STR, OPT_INT_T, PARSE_IN_RANGE,
 		FLDSET(struct ast_ari_conf_general, write_timeout), 1, INT_MAX);
+	aco_option_register_custom(&cfg_info, "channelvars", ACO_EXACT, general_options,
+		"", channelvars_handler, 0);
 
 	/* ARI type=user category options */
 	aco_option_register(&cfg_info, "type", ACO_EXACT, user, NULL,
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_applications.c 1:16.1.1~dfsg-1/res/ari/resource_applications.c
--- 1:13.23.1~dfsg-2/res/ari/resource_applications.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_applications.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis_app.h"
 #include "resource_applications.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_asterisk.c 1:16.1.1~dfsg-1/res/ari/resource_asterisk.c
--- 1:13.23.1~dfsg-2/res/ari/resource_asterisk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_asterisk.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/ast_version.h"
 #include "asterisk/buildinfo.h"
 #include "asterisk/logger.h"
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_bridges.c 1:16.1.1~dfsg-1/res/ari/resource_bridges.c
--- 1:13.23.1~dfsg-2/res/ari/resource_bridges.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_bridges.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,15 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_recording</depend>
-	<depend type="module">res_stasis_playback</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_bridges.h"
 #include "asterisk/stasis.h"
 #include "asterisk/stasis_bridges.h"
@@ -220,6 +216,12 @@ void ast_ari_bridges_add_channel(struct
 				return;
 			}
 		}
+
+		/* Apply bridge features to each of the channel controls */
+		if (!stasis_app_control_bridge_features_init(list->controls[i])) {
+			stasis_app_control_absorb_dtmf_in_bridge(list->controls[i], args->absorb_dtmf);
+			stasis_app_control_mute_in_bridge(list->controls[i], args->mute);
+		}
 	}
 
 	for (i = 0; i < list->count; ++i) {
@@ -288,10 +290,9 @@ static void *bridge_channel_control_thre
 	struct ast_channel *bridge_channel = thread_data->bridge_channel;
 	struct stasis_app_control *control = thread_data->control;
 	struct stasis_forward *forward = thread_data->forward;
+	ast_callid callid = ast_channel_callid(bridge_channel);
 	char *bridge_id = ast_strdupa(thread_data->bridge_id);
 
-	RAII_VAR(struct ast_callid *, callid, ast_channel_callid(bridge_channel), ast_callid_cleanup);
-
 	if (callid) {
 		ast_callid_threadassoc_add(callid);
 	}
@@ -337,7 +338,8 @@ static struct ast_channel *prepare_bridg
  * \brief Performs common setup for a bridge playback operation
  * with both new controls and when existing controls are  found.
  *
- * \param args_media media string split from arguments
+ * \param args_media medias to play
+ * \param args_media_count number of media items in \c media
  * \param args_lang language string split from arguments
  * \param args_offset_ms milliseconds offset split from arguments
  * \param args_playback_id string to use for playback split from
@@ -351,7 +353,8 @@ static struct ast_channel *prepare_bridg
  * \retval -1 operation failed
  * \retval operation was successful
  */
-static int ari_bridges_play_helper(const char *args_media,
+static int ari_bridges_play_helper(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -376,8 +379,8 @@ static int ari_bridges_play_helper(const
 
 	language = S_OR(args_lang, snapshot->language);
 
-	playback = stasis_app_control_play_uri(control, args_media, language,
-		bridge->uniqueid, STASIS_PLAYBACK_TARGET_BRIDGE, args_skipms,
+	playback = stasis_app_control_play_uri(control, args_media, args_media_count,
+		language, bridge->uniqueid, STASIS_PLAYBACK_TARGET_BRIDGE, args_skipms,
 		args_offset_ms, args_playback_id);
 
 	if (!playback) {
@@ -400,7 +403,8 @@ static int ari_bridges_play_helper(const
 	return 0;
 }
 
-static void ari_bridges_play_new(const char *args_media,
+static void ari_bridges_play_new(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -453,9 +457,9 @@ static void ari_bridges_play_new(const c
 	}
 
 	ao2_lock(control);
-	if (ari_bridges_play_helper(args_media, args_lang, args_offset_ms,
-			args_skipms, args_playback_id, response, bridge, control,
-			&json, &playback_url)) {
+	if (ari_bridges_play_helper(args_media, args_media_count, args_lang,
+			args_offset_ms, args_skipms, args_playback_id, response, bridge,
+			control, &json, &playback_url)) {
 		ao2_unlock(control);
 		return;
 	}
@@ -505,7 +509,8 @@ enum play_found_result {
  * \brief Performs common setup for a bridge playback operation
  * with both new controls and when existing controls are  found.
  *
- * \param args_media media string split from arguments
+ * \param args_media medias to play
+ * \param args_media_count number of media items in \c media
  * \param args_lang language string split from arguments
  * \param args_offset_ms milliseconds offset split from arguments
  * \param args_playback_id string to use for playback split from
@@ -519,7 +524,8 @@ enum play_found_result {
  * \retval PLAY_FOUND_CHANNEL_UNAVAILABLE The operation failed because
  * the channel requested to playback with is breaking down.
  */
-static enum play_found_result ari_bridges_play_found(const char *args_media,
+static enum play_found_result ari_bridges_play_found(const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -545,9 +551,9 @@ static enum play_found_result ari_bridge
 		return PLAY_FOUND_CHANNEL_UNAVAILABLE;
 	}
 
-	if (ari_bridges_play_helper(args_media, args_lang, args_offset_ms,
-			args_skipms, args_playback_id, response, bridge, control,
-			&json, &playback_url)) {
+	if (ari_bridges_play_helper(args_media, args_media_count,
+			args_lang, args_offset_ms, args_skipms, args_playback_id,
+			response, bridge, control, &json, &playback_url)) {
 		ao2_unlock(control);
 		return PLAY_FOUND_FAILURE;
 	}
@@ -559,7 +565,8 @@ static enum play_found_result ari_bridge
 
 static void ari_bridges_handle_play(
 	const char *args_bridge_id,
-	const char *args_media,
+	const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offset_ms,
 	int args_skipms,
@@ -582,15 +589,15 @@ static void ari_bridges_handle_play(
 		 * that will work or else there isn't a channel for this bridge anymore,
 		 * in which case we'll revert to ari_bridges_play_new.
 		 */
-		if (ari_bridges_play_found(args_media, args_lang, args_offset_ms,
-				args_skipms, args_playback_id, response,bridge,
+		if (ari_bridges_play_found(args_media, args_media_count, args_lang,
+				args_offset_ms, args_skipms, args_playback_id, response,bridge,
 				play_channel) == PLAY_FOUND_CHANNEL_UNAVAILABLE) {
 			continue;
 		}
 		return;
 	}
 
-	ari_bridges_play_new(args_media, args_lang, args_offset_ms,
+	ari_bridges_play_new(args_media, args_media_count, args_lang, args_offset_ms,
 		args_skipms, args_playback_id, response, bridge);
 }
 
@@ -601,6 +608,7 @@ void ast_ari_bridges_play(struct ast_var
 {
 	ari_bridges_handle_play(args->bridge_id,
 	args->media,
+	args->media_count,
 	args->lang,
 	args->offsetms,
 	args->skipms,
@@ -614,6 +622,7 @@ void ast_ari_bridges_play_with_id(struct
 {
 	ari_bridges_handle_play(args->bridge_id,
 	args->media,
+	args->media_count,
 	args->lang,
 	args->offsetms,
 	args->skipms,
@@ -816,6 +825,7 @@ void ast_ari_bridges_start_moh(struct as
 	}
 
 	ast_moh_start(moh_channel, moh_class, NULL);
+	ast_channel_cleanup(moh_channel);
 
 	ast_ari_response_no_content(response);
 
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_bridges.h 1:16.1.1~dfsg-1/res/ari/resource_bridges.h
--- 1:13.23.1~dfsg-2/res/ari/resource_bridges.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_bridges.h	2018-12-26 18:45:13.000000000 +0000
@@ -52,7 +52,7 @@ struct ast_ari_bridges_list_args {
 void ast_ari_bridges_list(struct ast_variable *headers, struct ast_ari_bridges_list_args *args, struct ast_ari_response *response);
 /*! Argument struct for ast_ari_bridges_create() */
 struct ast_ari_bridges_create_args {
-	/*! Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media). */
+	/*! Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media, video_sfu). */
 	const char *type;
 	/*! Unique ID to give to the bridge being created. */
 	const char *bridge_id;
@@ -82,7 +82,7 @@ int ast_ari_bridges_create_parse_body(
 void ast_ari_bridges_create(struct ast_variable *headers, struct ast_ari_bridges_create_args *args, struct ast_ari_response *response);
 /*! Argument struct for ast_ari_bridges_create_with_id() */
 struct ast_ari_bridges_create_with_id_args {
-	/*! Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media) to set. */
+	/*! Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media, video_sfu) to set. */
 	const char *type;
 	/*! Unique ID to give to the bridge being created. */
 	const char *bridge_id;
@@ -150,6 +150,10 @@ struct ast_ari_bridges_add_channel_args
 	char *channel_parse;
 	/*! Channel's role in the bridge */
 	const char *role;
+	/*! Absorb DTMF coming from this channel, preventing it to pass through to the bridge */
+	int absorb_dtmf;
+	/*! Mute audio from this channel, preventing it to pass through to the bridge */
+	int mute;
 };
 /*!
  * \brief Body parsing function for /bridges/{bridgeId}/addChannel.
@@ -273,11 +277,15 @@ void ast_ari_bridges_stop_moh(struct ast
 struct ast_ari_bridges_play_args {
 	/*! Bridge's id */
 	const char *bridge_id;
-	/*! Media's URI to play. */
-	const char *media;
+	/*! Array of Media URIs to play. */
+	const char **media;
+	/*! Length of media array. */
+	size_t media_count;
+	/*! Parsing context for media. */
+	char *media_parse;
 	/*! For sounds, selects language for sound. */
 	const char *lang;
-	/*! Number of media to skip before playing. */
+	/*! Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified. */
 	int offsetms;
 	/*! Number of milliseconds to skip for forward/reverse operations. */
 	int skipms;
@@ -311,11 +319,15 @@ struct ast_ari_bridges_play_with_id_args
 	const char *bridge_id;
 	/*! Playback ID. */
 	const char *playback_id;
-	/*! Media's URI to play. */
-	const char *media;
+	/*! Array of Media URIs to play. */
+	const char **media;
+	/*! Length of media array. */
+	size_t media_count;
+	/*! Parsing context for media. */
+	char *media_parse;
 	/*! For sounds, selects language for sound. */
 	const char *lang;
-	/*! Number of media to skip before playing. */
+	/*! Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified. */
 	int offsetms;
 	/*! Number of milliseconds to skip for forward/reverse operations. */
 	int skipms;
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_channels.c 1:16.1.1~dfsg-1/res/ari/resource_channels.c
--- 1:13.23.1~dfsg-2/res/ari/resource_channels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_channels.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,17 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_answer</depend>
-	<depend type="module">res_stasis_playback</depend>
-	<depend type="module">res_stasis_recording</depend>
-	<depend type="module">res_stasis_snoop</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
 #include "asterisk/bridge.h"
@@ -48,11 +42,70 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/format_cache.h"
 #include "asterisk/core_local.h"
 #include "asterisk/dial.h"
+#include "asterisk/max_forwards.h"
 #include "resource_channels.h"
 
 #include <limits.h>
 
 /*!
+ * \brief Ensure channel is in a state that allows operation to be performed.
+ *
+ * Since Asterisk 14, it has been possible for down channels, as well as unanswered
+ * outbound channels to enter Stasis. While some operations are fine to perform on
+ * such channels, operations that
+ *
+ * - Attempt to manipulate channel state
+ * - Attempt to play media
+ * - Attempt to control the channel's location in the dialplan
+ *
+ * are invalid. This function can be used to determine if the channel is in an
+ * appropriate state.
+ *
+ * \note When this function returns an error, the HTTP response is taken care of.
+ *
+ * \param control The app control
+ * \param response Response to fill in if there is an error
+ *
+ * \retval 0 Channel is in a valid state. Continue on!
+ * \retval non-zero Channel is in an invalid state. Bail!
+ */
+static int channel_state_invalid(struct stasis_app_control *control,
+	struct ast_ari_response *response)
+{
+	struct ast_channel_snapshot *snapshot;
+
+	snapshot = stasis_app_control_get_snapshot(control);
+	if (!snapshot) {
+		ast_ari_response_error(response, 404, "Not Found", "Channel not found");
+		return -1;
+	}
+
+	/* These channel states apply only to outbound channels:
+	 * - Down: Channel has been created, and nothing else has been done
+	 * - Reserved: For a PRI, an underlying B-channel is reserved,
+	 *   but the channel is not yet dialed
+	 * - Ringing: The channel has been dialed.
+	 *
+	 * This does not affect inbound channels. Inbound channels, when they
+	 * enter the dialplan, are in the "Ring" state. If they have already
+	 * been answered, then they are in the "Up" state.
+	 */
+	if (snapshot->state == AST_STATE_DOWN
+		|| snapshot->state == AST_STATE_RESERVED
+		|| snapshot->state == AST_STATE_RINGING) {
+		ast_ari_response_error(response, 412, "Precondition Failed",
+			"Channel in invalid state");
+		ao2_ref(snapshot, -1);
+
+		return -1;
+	}
+
+	ao2_ref(snapshot, -1);
+
+	return 0;
+}
+
+/*!
  * \brief Finds the control object for a channel, filling the response with an
  * error, if appropriate.
  * \param[out] response Response to fill with an error if control is not found.
@@ -106,8 +159,13 @@ void ast_ari_channels_continue_in_dialpl
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	snapshot = stasis_app_control_get_snapshot(control);
 	if (!snapshot) {
+		ast_ari_response_error(response, 404, "Not Found", "Channel not found");
 		return;
 	}
 
@@ -174,6 +232,10 @@ void ast_ari_channels_redirect(struct as
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	if (ast_strlen_zero(args->endpoint)) {
 		ast_ari_response_error(response, 400, "Not Found",
 			"Required parameter 'endpoint' not provided.");
@@ -228,6 +290,10 @@ void ast_ari_channels_answer(struct ast_
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	if (stasis_app_control_answer(control) != 0) {
 		ast_ari_response_error(
 			response, 500, "Internal Server Error",
@@ -249,6 +315,10 @@ void ast_ari_channels_ring(struct ast_va
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_ring(control);
 
 	ast_ari_response_no_content(response);
@@ -265,6 +335,10 @@ void ast_ari_channels_ring_stop(struct a
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_ring_stop(control);
 
 	ast_ari_response_no_content(response);
@@ -283,6 +357,10 @@ void ast_ari_channels_mute(struct ast_va
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	if (ast_strlen_zero(args->direction)) {
 		ast_ari_response_error(
 			response, 400, "Bad Request",
@@ -321,6 +399,10 @@ void ast_ari_channels_unmute(struct ast_
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	if (ast_strlen_zero(args->direction)) {
 		ast_ari_response_error(
 			response, 400, "Bad Request",
@@ -357,6 +439,10 @@ void ast_ari_channels_send_dtmf(struct a
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	if (ast_strlen_zero(args->dtmf)) {
 		ast_ari_response_error(
 			response, 400, "Bad Request",
@@ -381,6 +467,10 @@ void ast_ari_channels_hold(struct ast_va
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_hold(control);
 
 	ast_ari_response_no_content(response);
@@ -398,6 +488,10 @@ void ast_ari_channels_unhold(struct ast_
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_unhold(control);
 
 	ast_ari_response_no_content(response);
@@ -415,6 +509,10 @@ void ast_ari_channels_start_moh(struct a
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_moh_start(control, args->moh_class);
 	ast_ari_response_no_content(response);
 }
@@ -431,6 +529,10 @@ void ast_ari_channels_stop_moh(struct as
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_moh_stop(control);
 	ast_ari_response_no_content(response);
 }
@@ -447,6 +549,10 @@ void ast_ari_channels_start_silence(stru
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_silence_start(control);
 	ast_ari_response_no_content(response);
 }
@@ -463,13 +569,18 @@ void ast_ari_channels_stop_silence(struc
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	stasis_app_control_silence_stop(control);
 	ast_ari_response_no_content(response);
 }
 
 static void ari_channels_handle_play(
 	const char *args_channel_id,
-	const char *args_media,
+	const char **args_media,
+	size_t args_media_count,
 	const char *args_lang,
 	int args_offsetms,
 	int args_skipms,
@@ -491,6 +602,10 @@ static void ari_channels_handle_play(
 		return;
 	}
 
+	if (channel_state_invalid(control, response)) {
+		return;
+	}
+
 	snapshot = stasis_app_control_get_snapshot(control);
 	if (!snapshot) {
 		ast_ari_response_error(
@@ -515,7 +630,7 @@ static void ari_channels_handle_play(
 
 	language = S_OR(args_lang, snapshot->language);
 
-	playback = stasis_app_control_play_uri(control, args_media, language,
+	playback = stasis_app_control_play_uri(control, args_media, args_media_count, language,
 		args_channel_id, STASIS_PLAYBACK_TARGET_CHANNEL, args_skipms, args_offsetms, args_playback_id);
 	if (!playback) {
 		ast_ari_response_error(
@@ -551,6 +666,7 @@ void ast_ari_channels_play(struct ast_va
 	ari_channels_handle_play(
 		args->channel_id,
 		args->media,
+		args->media_count,
 		args->lang,
 		args->offsetms,
 		args->skipms,
@@ -565,6 +681,7 @@ void ast_ari_channels_play_with_id(struc
 	ari_channels_handle_play(
 		args->channel_id,
 		args->media,
+		args->media_count,
 		args->lang,
 		args->offsetms,
 		args->skipms,
@@ -767,6 +884,8 @@ void ast_ari_channels_hangup(struct ast_
 		cause = AST_CAUSE_CONGESTION;
 	} else if (!strcmp(args->reason, "no_answer")) {
 		cause = AST_CAUSE_NOANSWER;
+	} else if(!strcmp(args->reason, "answered_elsewhere")) {
+		cause = AST_CAUSE_ANSWERED_ELSEWHERE;
 	} else {
 		ast_ari_response_error(
 			response, 400, "Invalid Reason",
@@ -1511,3 +1630,328 @@ void ast_ari_channels_snoop_channel_with
 		args->snoop_id,
 		response);
 }
+
+struct ari_channel_thread_data {
+	struct ast_channel *chan;
+	struct ast_str *stasis_stuff;
+};
+
+static void chan_data_destroy(struct ari_channel_thread_data *chan_data)
+{
+	ast_free(chan_data->stasis_stuff);
+	ast_hangup(chan_data->chan);
+	ast_free(chan_data);
+}
+
+/*!
+ * \brief Thread that owns stasis-created channel.
+ *
+ * The channel enters into a Stasis application immediately upon creation. In this
+ * way, the channel can be manipulated by the Stasis application. Once the channel
+ * exits the Stasis application, it is hung up.
+ */
+static void *ari_channel_thread(void *data)
+{
+	struct ari_channel_thread_data *chan_data = data;
+	struct ast_app *stasis_app;
+
+	stasis_app = pbx_findapp("Stasis");
+	if (!stasis_app) {
+		ast_log(LOG_ERROR, "Stasis dialplan application is not registered");
+		chan_data_destroy(chan_data);
+		return NULL;
+	}
+
+	pbx_exec(chan_data->chan, stasis_app, ast_str_buffer(chan_data->stasis_stuff));
+
+	chan_data_destroy(chan_data);
+
+	return NULL;
+}
+
+struct ast_datastore_info dialstring_info = {
+	.type = "ARI Dialstring",
+	.destroy = ast_free_ptr,
+};
+
+/*!
+ * \brief Save dialstring onto a channel datastore
+ *
+ * This will later be retrieved when it comes time to actually dial the channel
+ *
+ * \param chan The channel on which to save the dialstring
+ * \param dialstring The dialstring to save
+ * \retval 0 SUCCESS!
+ * \reval -1 Failure :(
+ */
+static int save_dialstring(struct ast_channel *chan, const char *dialstring)
+{
+	struct ast_datastore *datastore;
+
+	datastore = ast_datastore_alloc(&dialstring_info, NULL);
+	if (!datastore) {
+		return -1;
+	}
+
+	datastore->data = ast_strdup(dialstring);
+	if (!datastore->data) {
+		ast_datastore_free(datastore);
+		return -1;
+	}
+
+	ast_channel_lock(chan);
+	if (ast_channel_datastore_add(chan, datastore)) {
+		ast_channel_unlock(chan);
+		ast_datastore_free(datastore);
+		return -1;
+	}
+	ast_channel_unlock(chan);
+
+	return 0;
+}
+
+/*!
+ * \brief Retrieve the dialstring from the channel datastore
+ *
+ * \pre chan is locked
+ * \param chan Channel that was previously created in ARI
+ * \retval NULL Failed to find datastore
+ * \retval non-NULL The dialstring
+ */
+static char *restore_dialstring(struct ast_channel *chan)
+{
+	struct ast_datastore *datastore;
+
+	datastore = ast_channel_datastore_find(chan, &dialstring_info, NULL);
+	if (!datastore) {
+		return NULL;
+	}
+
+	return datastore->data;
+}
+
+void ast_ari_channels_create(struct ast_variable *headers,
+	struct ast_ari_channels_create_args *args,
+	struct ast_ari_response *response)
+{
+	struct ast_assigned_ids assignedids = {
+		.uniqueid = args->channel_id,
+		.uniqueid2 = args->other_channel_id,
+	};
+	struct ari_channel_thread_data *chan_data;
+	struct ast_channel_snapshot *snapshot;
+	pthread_t thread;
+	char *dialtech;
+	char dialdevice[AST_CHANNEL_NAME];
+	char *stuff;
+	int cause;
+	struct ast_format_cap *request_cap;
+	struct ast_channel *originator;
+
+	if (!ast_strlen_zero(args->originator) && !ast_strlen_zero(args->formats)) {
+		ast_ari_response_error(response, 400, "Bad Request",
+			"Originator and formats can't both be specified");
+		return;
+	}
+
+	if (ast_strlen_zero(args->endpoint)) {
+		ast_ari_response_error(response, 400, "Bad Request",
+			"Endpoint must be specified");
+		return;
+	}
+
+	chan_data = ast_calloc(1, sizeof(*chan_data));
+	if (!chan_data) {
+		ast_ari_response_alloc_failed(response);
+		return;
+	}
+
+	chan_data->stasis_stuff = ast_str_create(32);
+	if (!chan_data->stasis_stuff) {
+		ast_ari_response_alloc_failed(response);
+		chan_data_destroy(chan_data);
+		return;
+	}
+
+	ast_str_append(&chan_data->stasis_stuff, 0, "%s", args->app);
+	if (!ast_strlen_zero(args->app_args)) {
+		ast_str_append(&chan_data->stasis_stuff, 0, ",%s", args->app_args);
+	}
+
+	dialtech = ast_strdupa(args->endpoint);
+	if ((stuff = strchr(dialtech, '/'))) {
+		*stuff++ = '\0';
+		ast_copy_string(dialdevice, stuff, sizeof(dialdevice));
+	}
+
+	originator = ast_channel_get_by_name(args->originator);
+	if (originator) {
+		request_cap = ao2_bump(ast_channel_nativeformats(originator));
+		if (!ast_strlen_zero(args->app)) {
+			stasis_app_subscribe_channel(args->app, originator);
+		}
+	} else if (!ast_strlen_zero(args->formats)) {
+		char *format_name;
+		char *formats_copy = ast_strdupa(args->formats);
+
+		if (!(request_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
+			ast_ari_response_alloc_failed(response);
+			chan_data_destroy(chan_data);
+			return;
+		}
+
+		while ((format_name = ast_strip(strsep(&formats_copy, ",")))) {
+			struct ast_format *fmt = ast_format_cache_get(format_name);
+
+			if (!fmt || ast_format_cap_append(request_cap, fmt, 0)) {
+				if (!fmt) {
+					ast_ari_response_error(
+						response, 400, "Bad Request",
+						"Provided format (%s) was not found", format_name);
+				} else {
+					ast_ari_response_alloc_failed(response);
+				}
+				ao2_ref(request_cap, -1);
+				ao2_cleanup(fmt);
+				chan_data_destroy(chan_data);
+				return;
+			}
+			ao2_ref(fmt, -1);
+		}
+	} else {
+		if (!(request_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
+			ast_ari_response_alloc_failed(response);
+			chan_data_destroy(chan_data);
+			return;
+		}
+
+		ast_format_cap_append_by_type(request_cap, AST_MEDIA_TYPE_AUDIO);
+	}
+
+	chan_data->chan = ast_request(dialtech, request_cap, &assignedids, originator, dialdevice, &cause);
+	ao2_cleanup(request_cap);
+
+	if (!chan_data->chan) {
+		if (ast_channel_errno() == AST_CHANNEL_ERROR_ID_EXISTS) {
+			ast_ari_response_error(response, 409, "Conflict",
+				"Channel with given unique ID already exists");
+		} else {
+			ast_ari_response_alloc_failed(response);
+		}
+		ast_channel_cleanup(originator);
+		chan_data_destroy(chan_data);
+		return;
+	}
+
+	if (!ast_strlen_zero(args->app)) {
+		stasis_app_subscribe_channel(args->app, chan_data->chan);
+	}
+
+	ast_channel_cleanup(originator);
+
+	if (save_dialstring(chan_data->chan, stuff)) {
+		ast_ari_response_alloc_failed(response);
+		chan_data_destroy(chan_data);
+		return;
+	}
+
+	snapshot = ast_channel_snapshot_get_latest(ast_channel_uniqueid(chan_data->chan));
+
+	if (ast_pthread_create_detached(&thread, NULL, ari_channel_thread, chan_data)) {
+		ast_ari_response_alloc_failed(response);
+		chan_data_destroy(chan_data);
+	} else {
+		ast_ari_response_ok(response, ast_channel_snapshot_to_json(snapshot, NULL));
+	}
+
+	ao2_ref(snapshot, -1);
+}
+
+void ast_ari_channels_dial(struct ast_variable *headers,
+	struct ast_ari_channels_dial_args *args,
+	struct ast_ari_response *response)
+{
+	RAII_VAR(struct stasis_app_control *, control, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_channel *, caller, NULL, ast_channel_cleanup);
+	RAII_VAR(struct ast_channel *, callee, NULL, ast_channel_cleanup);
+	char *dialstring;
+
+	control = find_control(response, args->channel_id);
+	if (control == NULL) {
+		/* Response filled in by find_control */
+		return;
+	}
+
+	caller = ast_channel_get_by_name(args->caller);
+
+	callee = ast_channel_get_by_name(args->channel_id);
+	if (!callee) {
+		ast_ari_response_error(response, 404, "Not Found",
+			"Callee not found");
+		return;
+	}
+
+	if (ast_channel_state(callee) != AST_STATE_DOWN
+		&& ast_channel_state(callee) != AST_STATE_RESERVED) {
+		ast_ari_response_error(response, 409, "Conflict",
+			"Channel is not in the 'Down' state");
+		return;
+	}
+
+	/* XXX This is straight up copied from main/dial.c. It's probably good
+	 * to separate this to some common method.
+	 */
+	if (caller) {
+		ast_channel_lock_both(caller, callee);
+	} else {
+		ast_channel_lock(callee);
+	}
+
+	dialstring = restore_dialstring(callee);
+	if (!dialstring) {
+		ast_channel_unlock(callee);
+		if (caller) {
+			ast_channel_unlock(caller);
+		}
+		ast_ari_response_error(response, 409, "Conflict",
+			"Dialing a channel not created by ARI");
+		return;
+	}
+	/* Make a copy of the dialstring just in case some jerk tries to hang up the
+	 * channel before we can actually dial
+	 */
+	dialstring = ast_strdupa(dialstring);
+
+	ast_channel_stage_snapshot(callee);
+	if (caller) {
+		ast_channel_inherit_variables(caller, callee);
+		ast_channel_datastore_inherit(caller, callee);
+		ast_max_forwards_decrement(callee);
+
+		/* Copy over callerid information */
+		ast_party_redirecting_copy(ast_channel_redirecting(callee), ast_channel_redirecting(caller));
+
+		ast_channel_dialed(callee)->transit_network_select = ast_channel_dialed(caller)->transit_network_select;
+
+		ast_connected_line_copy_from_caller(ast_channel_connected(callee), ast_channel_caller(caller));
+
+		ast_channel_language_set(callee, ast_channel_language(caller));
+		ast_channel_req_accountcodes(callee, caller, AST_CHANNEL_REQUESTOR_BRIDGE_PEER);
+		if (ast_strlen_zero(ast_channel_musicclass(callee)))
+			ast_channel_musicclass_set(callee, ast_channel_musicclass(caller));
+
+		ast_channel_adsicpe_set(callee, ast_channel_adsicpe(caller));
+		ast_channel_transfercapability_set(callee, ast_channel_transfercapability(caller));
+		ast_channel_unlock(caller);
+	}
+
+	ast_channel_stage_snapshot_done(callee);
+	ast_channel_unlock(callee);
+
+	if (stasis_app_control_dial(control, dialstring, args->timeout)) {
+		ast_ari_response_alloc_failed(response);
+		return;
+	}
+
+	ast_ari_response_no_content(response);
+}
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_channels.h 1:16.1.1~dfsg-1/res/ari/resource_channels.h
--- 1:13.23.1~dfsg-2/res/ari/resource_channels.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_channels.h	2018-12-26 18:45:13.000000000 +0000
@@ -102,6 +102,42 @@ int ast_ari_channels_originate_parse_bod
  * \param[out] response HTTP response
  */
 void ast_ari_channels_originate(struct ast_variable *headers, struct ast_ari_channels_originate_args *args, struct ast_ari_response *response);
+/*! Argument struct for ast_ari_channels_create() */
+struct ast_ari_channels_create_args {
+	/*! Endpoint for channel communication */
+	const char *endpoint;
+	/*! Stasis Application to place channel into */
+	const char *app;
+	/*! The application arguments to pass to the Stasis application provided by 'app'. Mutually exclusive with 'context', 'extension', 'priority', and 'label'. */
+	const char *app_args;
+	/*! The unique id to assign the channel on creation. */
+	const char *channel_id;
+	/*! The unique id to assign the second channel when using local channels. */
+	const char *other_channel_id;
+	/*! Unique ID of the calling channel */
+	const char *originator;
+	/*! The format name capability list to use if originator is not specified. Ex. "ulaw,slin16".  Format names can be found with "core show codecs". */
+	const char *formats;
+};
+/*!
+ * \brief Body parsing function for /channels/create.
+ * \param body The JSON body from which to parse parameters.
+ * \param[out] args The args structure to parse into.
+ * \retval zero on success
+ * \retval non-zero on failure
+ */
+int ast_ari_channels_create_parse_body(
+	struct ast_json *body,
+	struct ast_ari_channels_create_args *args);
+
+/*!
+ * \brief Create channel.
+ *
+ * \param headers HTTP headers
+ * \param args Swagger parameters
+ * \param[out] response HTTP response
+ */
+void ast_ari_channels_create(struct ast_variable *headers, struct ast_ari_channels_create_args *args, struct ast_ari_response *response);
 /*! Argument struct for ast_ari_channels_get() */
 struct ast_ari_channels_get_args {
 	/*! Channel's id */
@@ -475,11 +511,15 @@ void ast_ari_channels_stop_silence(struc
 struct ast_ari_channels_play_args {
 	/*! Channel's id */
 	const char *channel_id;
-	/*! Media's URI to play. */
-	const char *media;
+	/*! Array of Media URIs to play. */
+	const char **media;
+	/*! Length of media array. */
+	size_t media_count;
+	/*! Parsing context for media. */
+	char *media_parse;
 	/*! For sounds, selects language for sound. */
 	const char *lang;
-	/*! Number of media to skip before playing. */
+	/*! Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified. */
 	int offsetms;
 	/*! Number of milliseconds to skip for forward/reverse operations. */
 	int skipms;
@@ -513,11 +553,15 @@ struct ast_ari_channels_play_with_id_arg
 	const char *channel_id;
 	/*! Playback ID. */
 	const char *playback_id;
-	/*! Media's URI to play. */
-	const char *media;
+	/*! Array of Media URIs to play. */
+	const char **media;
+	/*! Length of media array. */
+	size_t media_count;
+	/*! Parsing context for media. */
+	char *media_parse;
 	/*! For sounds, selects language for sound. */
 	const char *lang;
-	/*! Number of media to skip before playing. */
+	/*! Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified. */
 	int offsetms;
 	/*! Number of milliseconds to skip for forward/reverse operations. */
 	int skipms;
@@ -709,5 +753,33 @@ int ast_ari_channels_snoop_channel_with_
  * \param[out] response HTTP response
  */
 void ast_ari_channels_snoop_channel_with_id(struct ast_variable *headers, struct ast_ari_channels_snoop_channel_with_id_args *args, struct ast_ari_response *response);
+/*! Argument struct for ast_ari_channels_dial() */
+struct ast_ari_channels_dial_args {
+	/*! Channel's id */
+	const char *channel_id;
+	/*! Channel ID of caller */
+	const char *caller;
+	/*! Dial timeout */
+	int timeout;
+};
+/*!
+ * \brief Body parsing function for /channels/{channelId}/dial.
+ * \param body The JSON body from which to parse parameters.
+ * \param[out] args The args structure to parse into.
+ * \retval zero on success
+ * \retval non-zero on failure
+ */
+int ast_ari_channels_dial_parse_body(
+	struct ast_json *body,
+	struct ast_ari_channels_dial_args *args);
+
+/*!
+ * \brief Dial a created channel.
+ *
+ * \param headers HTTP headers
+ * \param args Swagger parameters
+ * \param[out] response HTTP response
+ */
+void ast_ari_channels_dial(struct ast_variable *headers, struct ast_ari_channels_dial_args *args, struct ast_ari_response *response);
 
 #endif /* _ASTERISK_RESOURCE_CHANNELS_H */
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_device_states.c 1:16.1.1~dfsg-1/res/ari/resource_device_states.c
--- 1:13.23.1~dfsg-2/res/ari/resource_device_states.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_device_states.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,14 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_device_state</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_device_states.h"
 #include "asterisk/stasis_app_device_state.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_endpoints.c 1:16.1.1~dfsg-1/res/ari/resource_endpoints.c
--- 1:13.23.1~dfsg-2/res/ari/resource_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_endpoints.h"
 
 #include "asterisk/astobj2.h"
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_events.c 1:16.1.1~dfsg-1/res/ari/resource_events.c
--- 1:13.23.1~dfsg-2/res/ari/resource_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,185 +24,398 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_http_websocket</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
+#include "resource_events.h"
 #include "asterisk/astobj2.h"
+#include "asterisk/http_websocket.h"
 #include "asterisk/stasis_app.h"
-#include "resource_events.h"
+#include "asterisk/vector.h"
 
-/*! Number of buckets for the Stasis application hash table. Remember to keep it
- *  a prime number!
- */
+/*! Number of buckets for the event session registry. Remember to keep it a prime number! */
+#define EVENT_SESSION_NUM_BUCKETS 23
+
+/*! Number of buckets for a websocket apps container. Remember to keep it a prime number! */
 #define APPS_NUM_BUCKETS 7
 
-/*! \brief A connection to the event WebSocket */
+/*! Initial size of a message queue. */
+#define MESSAGES_INIT_SIZE 23
+
+
+/*! \brief A wrapper for the /ref ast_ari_websocket_session. */
 struct event_session {
-	struct ast_ari_websocket_session *ws_session;
-	struct ao2_container *websocket_apps;
+	struct ast_ari_websocket_session *ws_session;   /*!< Handle to the websocket session. */
+	struct ao2_container *websocket_apps;           /*!< List of Stasis apps registered to
+	                                                     the websocket session. */
+	AST_VECTOR(, struct ast_json *) message_queue;  /*!< Container for holding delayed messages. */
+	char session_id[];                              /*!< The id for the websocket session. */
 };
 
+/*! \brief \ref event_session error types. */
+enum event_session_error_type {
+	ERROR_TYPE_STASIS_REGISTRATION = 1,  /*!< Stasis failed to register the application. */
+	ERROR_TYPE_OOM = 2,                  /*!< Insufficient memory to create the event
+	                                          session. */
+	ERROR_TYPE_MISSING_APP_PARAM = 3,    /*!< HTTP request was missing an [app] parameter. */
+	ERROR_TYPE_INVALID_APP_PARAM = 4,    /*!< HTTP request contained an invalid [app]
+	                                          parameter. */
+};
+
+/*! \brief Local registry for created \ref event_session objects. */
+static struct ao2_container *event_session_registry;
+
 /*!
- * \brief Explicitly shutdown a session.
+ * \brief Callback handler for Stasis application messages.
  *
- * An explicit shutdown is necessary, since stasis-app has a reference to this
- * session. We also need to be sure to null out the \c ws_session field, since
- * the websocket is about to go away.
+ * \internal
  *
- * \param session Session info struct.
+ * \param data      Void pointer to the event session (\ref event_session).
+ * \param app_name  Name of the Stasis application that dispatched the message.
+ * \param message   The dispatched message.
  */
-static void session_shutdown(struct event_session *session)
+static void stasis_app_message_handler(
+		void *data, const char *app_name, struct ast_json *message)
 {
-        struct ao2_iterator i;
-	char *app;
-	SCOPED_AO2LOCK(lock, session);
+	struct event_session *session = data;
+	const char *msg_type, *msg_application;
+
+	ast_assert(session != NULL);
+
+	ao2_lock(session);
+
+	msg_type = S_OR(ast_json_string_get(ast_json_object_get(message, "type")), "");
+	msg_application = S_OR(
+		ast_json_string_get(ast_json_object_get(message, "application")), "");
 
-	i = ao2_iterator_init(session->websocket_apps, 0);
-	while ((app = ao2_iterator_next(&i))) {
-		stasis_app_unregister(app);
-		ao2_cleanup(app);
+	/* If we've been replaced, remove the application from our local
+	   websocket_apps container */
+	if (strcmp(msg_type, "ApplicationReplaced") == 0 &&
+		strcmp(msg_application, app_name) == 0) {
+		ao2_find(session->websocket_apps, msg_application,
+			OBJ_UNLINK | OBJ_NODATA);
 	}
-	ao2_iterator_destroy(&i);
-	ao2_cleanup(session->websocket_apps);
 
-	session->websocket_apps = NULL;
-	session->ws_session = NULL;
+	/* Now, we need to determine our state to see how we will handle the message */
+	if (ast_json_object_set(message, "application", ast_json_string_create(app_name))) {
+		/* We failed to add an application element to our json message */
+		ast_log(LOG_WARNING,
+		        "Failed to dispatch '%s' message from Stasis app '%s'; could not update message\n",
+		        msg_type,
+		        msg_application);
+	} else if (!session->ws_session) {
+		/* If the websocket is NULL, the message goes to the queue */
+		if (!AST_VECTOR_APPEND(&session->message_queue, message)) {
+			ast_json_ref(message);
+		}
+		ast_log(LOG_WARNING,
+				"Queued '%s' message for Stasis app '%s'; websocket is not ready\n",
+				msg_type,
+				msg_application);
+	} else {
+		if (stasis_app_get_debug_by_name(app_name)) {
+			char *str = ast_json_dump_string_format(message, ast_ari_json_format());
+
+			ast_verbose("<--- Sending ARI event to %s --->\n%s\n",
+				ast_sockaddr_stringify(ast_ari_websocket_session_get_remote_addr(session->ws_session)),
+				str);
+			ast_json_free(str);
+		}
+
+		/* We are ready to publish the message */
+		ast_ari_websocket_session_write(session->ws_session, message);
+	}
+
+	ao2_unlock(session);
 }
 
-static void session_dtor(void *obj)
+/*!
+ * \brief AO2 comparison function for \ref event_session objects.
+ *
+ * \internal
+ *
+ * \param obj    Void pointer to the \ref event_session container.
+ * \param arg    Void pointer to the \ref event_session object.
+ * \param flags  The \ref search_flags to use when creating the hash key.
+ *
+ * \retval 0          The objects are not equal.
+ * \retval CMP_MATCH  The objects are equal.
+ */
+static int event_session_compare(void *obj, void *arg, int flags)
 {
-#ifdef AST_DEVMODE /* Avoid unused variable warning */
-	struct event_session *session = obj;
-#endif
+	const struct event_session *object_left = obj;
+	const struct event_session *object_right = arg;
+	const char *right_key = arg;
+	int cmp = 0;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_OBJECT:
+		right_key = object_right->session_id;
+		/* Fall through */
+	case OBJ_SEARCH_KEY:
+		cmp = strcmp(object_left->session_id, right_key);
+		break;
+	case OBJ_SEARCH_PARTIAL_KEY:
+		cmp = strncmp(object_left->session_id, right_key, strlen(right_key));
+		break;
+	default:
+		break;
+	}
 
-	/* session_shutdown should have been called before */
-	ast_assert(session->ws_session == NULL);
-	ast_assert(session->websocket_apps == NULL);
+	return cmp ? 0 : CMP_MATCH;
 }
 
-static void session_cleanup(struct event_session *session)
+/*!
+ * \brief AO2 hash function for \ref event_session objects.
+ *
+ * \details Computes hash value for the given \ref event_session, with respect to the
+ *          provided search flags.
+ *
+ * \internal
+ *
+ * \param obj    Void pointer to the \ref event_session object.
+ * \param flags  The \ref search_flags to use when creating the hash key.
+ *
+ * \retval > 0  on success
+ * \retval   0  on failure
+ */
+static int event_session_hash(const void *obj, const int flags)
 {
-	session_shutdown(session);
-	ao2_cleanup(session);
+	const struct event_session *session;
+	const char *key;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_KEY:
+		key = obj;
+		break;
+	case OBJ_SEARCH_OBJECT:
+		session = obj;
+		key = session->session_id;
+		break;
+	default:
+		/* Hash can only work on something with a full key. */
+		ast_assert(0);
+		return 0;
+	}
+	return ast_str_hash(key);
 }
 
-static struct event_session *session_create(
-	struct ast_ari_websocket_session *ws_session)
+/*!
+ * \brief Explicitly shutdown a session.
+ *
+ * \details An explicit shutdown is necessary, since the \ref stasis_app has a reference
+ *          to this session. We also need to be sure to null out the \c ws_session field,
+ *          since the websocket is about to go away.
+ *
+ * \internal
+ *
+ * \param session  Event session object (\ref event_session).
+ */
+static void event_session_shutdown(struct event_session *session)
 {
-	RAII_VAR(struct event_session *, session, NULL, ao2_cleanup);
+	struct ao2_iterator i;
+	char *app;
+	int j;
+	SCOPED_AO2LOCK(lock, session);
+
+	/* Clean up the websocket_apps container */
+	if (session->websocket_apps) {
+		i = ao2_iterator_init(session->websocket_apps, 0);
+		while ((app = ao2_iterator_next(&i))) {
+			stasis_app_unregister(app);
+			ao2_cleanup(app);
+		}
+		ao2_iterator_destroy(&i);
+		ao2_cleanup(session->websocket_apps);
+		session->websocket_apps = NULL;
+	}
+
+	/* Clean up the message_queue container */
+	for (j = 0; j < AST_VECTOR_SIZE(&session->message_queue); j++) {
+		struct ast_json *msg = AST_VECTOR_GET(&session->message_queue, j);
+		ast_json_unref(msg);
+	}
+	AST_VECTOR_FREE(&session->message_queue);
+
+	/* Remove the handle to the underlying websocket session */
+	session->ws_session = NULL;
+}
 
-	session = ao2_alloc(sizeof(*session), session_dtor);
+/*!
+ * \brief Updates the websocket session for an \ref event_session.
+ *
+ * \details The websocket for the given \ref event_session will be updated to the value
+ *          of the \c ws_session argument.
+ *
+ *          If the value of the \c ws_session is not \c NULL and there are messages in the
+ *          event session's \c message_queue, the messages are dispatched and removed from
+ *          the queue.
+ *
+ * \internal
+ *
+ * \param session     The event session object to update (\ref event_session).
+ * \param ws_session  Handle to the underlying websocket session
+ *                    (\ref ast_ari_websocket_session).
+ */
+static void event_session_update_websocket(
+		struct event_session *session, struct ast_ari_websocket_session *ws_session)
+{
+	int i;
+
+	ast_assert(session != NULL);
+
+	ao2_lock(session);
 
 	session->ws_session = ws_session;
-	session->websocket_apps =
-		ast_str_container_alloc(APPS_NUM_BUCKETS);
 
-	if (!session->websocket_apps) {
-		return NULL;
+	for (i = 0; i < AST_VECTOR_SIZE(&session->message_queue); i++) {
+		struct ast_json *msg = AST_VECTOR_GET(&session->message_queue, i);
+		ast_ari_websocket_session_write(session->ws_session, msg);
+		ast_json_unref(msg);
 	}
 
-	ao2_ref(session, +1);
-	return session;
+	AST_VECTOR_RESET(&session->message_queue, AST_VECTOR_ELEM_CLEANUP_NOOP);
+	ao2_unlock(session);
 }
 
 /*!
- * \brief Callback handler for Stasis application messages.
+ * \brief Processes cleanup actions for a \ref event_session object.
+ *
+ * \internal
+ *
+ * \param session  The event session object to cleanup (\ref event_session).
  */
-static void app_handler(void *data, const char *app_name,
-			struct ast_json *message)
+static void event_session_cleanup(struct event_session *session)
 {
-	struct event_session *session = data;
-	int res;
-	const char *msg_type = S_OR(
-		ast_json_string_get(ast_json_object_get(message, "type")),
-		"");
-	const char *msg_application = S_OR(
-		ast_json_string_get(ast_json_object_get(message, "application")),
-		"");
-
 	if (!session) {
 		return;
 	}
 
-	/* Determine if we've been replaced */
-	if (strcmp(msg_type, "ApplicationReplaced") == 0 &&
-		strcmp(msg_application, app_name) == 0) {
-		ao2_find(session->websocket_apps, msg_application,
-			OBJ_UNLINK | OBJ_NODATA);
-	}
-
-	res = ast_json_object_set(message, "application",
-				  ast_json_string_create(app_name));
-	if(res != 0) {
-		return;
+	event_session_shutdown(session);
+	if (event_session_registry) {
+		ao2_unlink(event_session_registry, session);
 	}
+}
 
-	ao2_lock(session);
-	if (session->ws_session) {
-		if (stasis_app_get_debug_by_name(app_name)) {
-			char *str = ast_json_dump_string_format(message, ast_ari_json_format());
+/*!
+ * \brief Event session object destructor (\ref event_session).
+ *
+ * \internal
+ *
+ * \param obj  Void pointer to the \ref event_session object.
+ */
+static void event_session_dtor(void *obj)
+{
+#ifdef AST_DEVMODE /* Avoid unused variable warning */
+	struct event_session *session = obj;
 
-			ast_verbose("<--- Sending ARI event to %s --->\n%s\n",
-				ast_sockaddr_stringify(ast_ari_websocket_session_get_remote_addr(session->ws_session)),
-				str);
-			ast_json_free(str);
-		}
+#endif
 
-		ast_ari_websocket_session_write(session->ws_session, message);
-	}
-	ao2_unlock(session);
+	/* event_session_shutdown should have been called before now */
+	ast_assert(session->ws_session == NULL);
+	ast_assert(session->websocket_apps == NULL);
+	ast_assert(AST_VECTOR_SIZE(&session->message_queue) == 0);
 }
 
 /*!
- * \brief Register for all of the apps given.
- * \param session Session info struct.
- * \param app_name Name of application to register.
- * \param register_handler Pointer to the application registration handler
+ * \brief Handles \ref event_session error processing.
+ *
+ * \internal
+ *
+ * \param session  The \ref event_session object.
+ * \param error    The \ref event_session_error_type to handle.
+ * \param ser      HTTP TCP/TLS Server Session (\ref ast_tcptls_session_instance).
+ *
+ * \retval  -1  Always returns -1.
  */
-static int session_register_app(struct event_session *session,
-				 const char *app_name,
-				 int (* register_handler)(const char *, stasis_app_cb handler, void *data))
+static int event_session_allocation_error_handler(
+		struct event_session *session, enum event_session_error_type error,
+		struct ast_tcptls_session_instance *ser)
 {
-	SCOPED_AO2LOCK(lock, session);
+	/* Notify the client */
+	switch (error) {
+	case ERROR_TYPE_STASIS_REGISTRATION:
+		ast_http_error(ser, 500, "Internal Server Error",
+			"Stasis registration failed");
+		break;
+
+	case ERROR_TYPE_OOM:
+		ast_http_error(ser, 500, "Internal Server Error",
+			"Allocation failed");
+		break;
+
+	case ERROR_TYPE_MISSING_APP_PARAM:
+		ast_http_error(ser, 400, "Bad Request",
+			"HTTP request is missing param: [app]");
+		break;
+
+	case ERROR_TYPE_INVALID_APP_PARAM:
+		ast_http_error(ser, 400, "Bad Request",
+			"Invalid application provided in param [app].");
+		break;
+
+	default:
+		break;
+	}
+
+	/* Cleanup the session */
+	event_session_cleanup(session);
+	return -1;
+}
 
-	ast_assert(session->ws_session != NULL);
-	ast_assert(session->websocket_apps != NULL);
+/*!
+ * \brief Creates an \ref event_session object and registers its apps with Stasis.
+ *
+ * \internal
+ *
+ * \param ser         HTTP TCP/TLS Server Session (\ref ast_tcptls_session_instance).
+ * \param args        The Stasis [app] parameters as parsed from the HTTP request
+ *                    (\ref ast_ari_events_event_websocket_args).
+ * \param session_id  The id for the websocket session that will be created for this
+ *                    event session.
+ *
+ * \retval  0  on success
+ * \retval -1  on failure
+ */
+static int event_session_alloc(struct ast_tcptls_session_instance *ser,
+		struct ast_ari_events_event_websocket_args *args, const char *session_id)
+{
+	RAII_VAR(struct event_session *, session, NULL, ao2_cleanup);
+	int (* register_handler)(const char *, stasis_app_cb handler, void *data);
+	size_t size, i;
 
-	if (ast_strlen_zero(app_name)) {
-		return -1;
+	/* The request must have at least one [app] parameter */
+	if (args->app_count == 0) {
+		return event_session_allocation_error_handler(
+			session, ERROR_TYPE_MISSING_APP_PARAM, ser);
 	}
 
-	if (ast_str_container_add(session->websocket_apps, app_name)) {
-		ast_ari_websocket_session_write(session->ws_session,
-			ast_ari_oom_json());
-		return -1;
-	}
+	size = sizeof(*session) + strlen(session_id) + 1;
 
-	register_handler(app_name, app_handler, session);
+	/* Instantiate the event session */
+	session = ao2_alloc(size, event_session_dtor);
+	if (!session) {
+		return event_session_allocation_error_handler(session, ERROR_TYPE_OOM, ser);
+	}
 
-	return 0;
-}
+	strncpy(session->session_id, session_id, size - sizeof(*session));
 
-int ast_ari_websocket_events_event_websocket_attempted(struct ast_tcptls_session_instance *ser,
-	struct ast_variable *headers,
-	struct ast_ari_events_event_websocket_args *args)
-{
-	int res = 0;
-	size_t i, j;
-	int (* register_handler)(const char *, stasis_app_cb handler, void *data);
+	/* Instantiate the hash table for Stasis apps */
+	session->websocket_apps =
+		ast_str_container_alloc(APPS_NUM_BUCKETS);
 
-	ast_debug(3, "/events WebSocket attempted\n");
+	if (!session->websocket_apps) {
+		return event_session_allocation_error_handler(session, ERROR_TYPE_OOM, ser);
+	}
 
-	if (args->app_count == 0) {
-		ast_http_error(ser, 400, "Bad Request", "Missing param 'app'");
-		return -1;
+	/* Instantiate the message queue */
+	if (AST_VECTOR_INIT(&session->message_queue, MESSAGES_INIT_SIZE)) {
+		return event_session_allocation_error_handler(session, ERROR_TYPE_OOM, ser);
 	}
 
+	/* Register the apps with Stasis */
 	if (args->subscribe_all) {
 		register_handler = &stasis_app_register_all;
 	} else {
@@ -210,79 +423,104 @@ int ast_ari_websocket_events_event_webso
 	}
 
 	for (i = 0; i < args->app_count; ++i) {
-		if (ast_strlen_zero(args->app[i])) {
-			res = -1;
-			break;
+		const char *app = args->app[i];
+
+		if (ast_strlen_zero(app)) {
+			return event_session_allocation_error_handler(
+				session, ERROR_TYPE_INVALID_APP_PARAM, ser);
 		}
 
-		res |= register_handler(args->app[i], app_handler, NULL);
-	}
+		if (ast_str_container_add(session->websocket_apps, app)) {
+			return event_session_allocation_error_handler(session, ERROR_TYPE_OOM, ser);
+		}
 
-	if (res) {
-		for (j = 0; j < i; ++j) {
-			stasis_app_unregister(args->app[j]);
+		if (register_handler(app, stasis_app_message_handler, session)) {
+			ast_log(LOG_WARNING, "Stasis registration failed for application: '%s'\n", app);
+			return event_session_allocation_error_handler(
+				session, ERROR_TYPE_STASIS_REGISTRATION, ser);
 		}
-		ast_http_error(ser, 400, "Bad Request", "Invalid application provided in param 'app'.");
 	}
 
-	return res;
+	/* Add the event session to the local registry */
+	if (!ao2_link(event_session_registry, session)) {
+		return event_session_allocation_error_handler(session, ERROR_TYPE_OOM, ser);
+	}
+
+	return 0;
 }
 
-void ast_ari_websocket_events_event_websocket_established(struct ast_ari_websocket_session *ws_session,
-	struct ast_variable *headers,
-	struct ast_ari_events_event_websocket_args *args)
+static int event_session_shutdown_cb(void *session, void *arg, int flags)
 {
-	RAII_VAR(struct event_session *, session, NULL, session_cleanup);
-	struct ast_json *msg;
-	int res;
-	size_t i;
-	int (* register_handler)(const char *, stasis_app_cb handler, void *data);
+	event_session_cleanup(session);
 
-	ast_debug(3, "/events WebSocket connection\n");
+	return 0;
+}
 
-	session = session_create(ws_session);
-	if (!session) {
-		ast_ari_websocket_session_write(ws_session, ast_ari_oom_json());
-		return;
-	}
+void ast_ari_websocket_events_event_websocket_dtor(void)
+{
+	ao2_callback(event_session_registry, OBJ_MULTIPLE | OBJ_NODATA, event_session_shutdown_cb, NULL);
 
-	if (args->subscribe_all) {
-		register_handler = &stasis_app_register_all;
-	} else {
-		register_handler = &stasis_app_register;
-	}
+	ao2_cleanup(event_session_registry);
+	event_session_registry = NULL;
+}
 
-	res = 0;
-	for (i = 0; i < args->app_count; ++i) {
-		if (ast_strlen_zero(args->app[i])) {
-			continue;
-		}
-		res |= session_register_app(session, args->app[i], register_handler);
+int ast_ari_websocket_events_event_websocket_init(void)
+{
+	/* Try to instantiate the registry */
+	event_session_registry = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		EVENT_SESSION_NUM_BUCKETS, event_session_hash, NULL, event_session_compare);
+	if (!event_session_registry) {
+		/* This is bad, bad. */
+		ast_log(LOG_WARNING,
+			    "Failed to allocate the local registry for websocket applications\n");
+		return -1;
 	}
 
-	if (ao2_container_count(session->websocket_apps) == 0) {
-		RAII_VAR(struct ast_json *, msg, NULL, ast_json_unref);
+	return 0;
+}
 
-		msg = ast_json_pack("{s: s, s: [s]}",
-			"type", "MissingParams",
-			"params", "app");
-		if (!msg) {
-			msg = ast_json_ref(ast_ari_oom_json());
-		}
+int ast_ari_websocket_events_event_websocket_attempted(
+		struct ast_tcptls_session_instance *ser, struct ast_variable *headers,
+		struct ast_ari_events_event_websocket_args *args, const char *session_id)
+{
+	ast_debug(3, "/events WebSocket attempted\n");
 
-		ast_ari_websocket_session_write(session->ws_session, msg);
-		return;
-	}
+	/* Create the event session */
+	return event_session_alloc(ser, args, session_id);
+}
 
-	if (res != 0) {
-		ast_ari_websocket_session_write(ws_session, ast_ari_oom_json());
-		return;
+void ast_ari_websocket_events_event_websocket_established(
+		struct ast_ari_websocket_session *ws_session, struct ast_variable *headers,
+		struct ast_ari_events_event_websocket_args *args)
+{
+	struct event_session *session;
+
+	struct ast_json *msg;
+	const char *session_id;
+
+	ast_debug(3, "/events WebSocket established\n");
+
+	ast_assert(ws_session != NULL);
+
+	session_id = ast_ari_websocket_session_id(ws_session);
+
+	/* Find the event_session and update its websocket  */
+	session = ao2_find(event_session_registry, session_id, OBJ_SEARCH_KEY);
+	if (session) {
+		ao2_unlink(event_session_registry, session);
+		event_session_update_websocket(session, ws_session);
+	} else {
+		ast_log(LOG_WARNING,
+			"Failed to locate an event session for the provided websocket session\n");
 	}
 
 	/* We don't process any input, but we'll consume it waiting for EOF */
 	while ((msg = ast_ari_websocket_session_read(ws_session))) {
 		ast_json_unref(msg);
 	}
+
+	event_session_cleanup(session);
+	ao2_ref(session, -1);
 }
 
 void ast_ari_events_user_event(struct ast_variable *headers,
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_events.h 1:16.1.1~dfsg-1/res/ari/resource_events.h
--- 1:13.23.1~dfsg-2/res/ari/resource_events.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_events.h	2018-12-26 18:45:13.000000000 +0000
@@ -54,14 +54,31 @@ struct ast_ari_events_event_websocket_ar
 /*!
  * \brief WebSocket connection for events.
  *
+ * \retval  0 success
+ * \retval -1 error
+ */
+int ast_ari_websocket_events_event_websocket_init(void);
+
+/*!
+ * \brief WebSocket connection for events.
+ *
+ * \return Nothing
+ */
+void ast_ari_websocket_events_event_websocket_dtor(void);
+
+/*!
+ * \brief WebSocket connection for events.
+ *
  * \param ser HTTP TCP/TLS Server Session
  * \param headers HTTP headers
  * \param args Swagger parameters
+ * \param session_id The id of the current session.
  *
  * \retval 0 success
  * \retval non-zero error
  */
-int ast_ari_websocket_events_event_websocket_attempted(struct ast_tcptls_session_instance *ser, struct ast_variable *headers, struct ast_ari_events_event_websocket_args *args);
+int ast_ari_websocket_events_event_websocket_attempted(struct ast_tcptls_session_instance *ser,
+	struct ast_variable *headers, struct ast_ari_events_event_websocket_args *args, const char *session_id);
 
 /*!
  * \brief WebSocket connection for events.
@@ -69,8 +86,10 @@ int ast_ari_websocket_events_event_webso
  * \param session ARI WebSocket.
  * \param headers HTTP headers.
  * \param args Swagger parameters.
+ * \param session_id The id of the current session.
  */
-void ast_ari_websocket_events_event_websocket_established(struct ast_ari_websocket_session *session, struct ast_variable *headers, struct ast_ari_events_event_websocket_args *args);
+void ast_ari_websocket_events_event_websocket_established(struct ast_ari_websocket_session *session,
+	struct ast_variable *headers, struct ast_ari_events_event_websocket_args *args);
 /*! Argument struct for ast_ari_events_user_event() */
 struct ast_ari_events_user_event_args {
 	/*! Event name */
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_mailboxes.c 1:16.1.1~dfsg-1/res/ari/resource_mailboxes.c
--- 1:13.23.1~dfsg-2/res/ari/resource_mailboxes.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_mailboxes.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,15 +24,12 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_mailbox</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 #include "asterisk/stasis_app_mailbox.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_mailboxes.h"
 
 void ast_ari_mailboxes_list(struct ast_variable *headers,
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_playbacks.c 1:16.1.1~dfsg-1/res/ari/resource_playbacks.c
--- 1:13.23.1~dfsg-2/res/ari/resource_playbacks.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_playbacks.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,14 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_playback</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis_app_playback.h"
 #include "resource_playbacks.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_recordings.c 1:16.1.1~dfsg-1/res/ari/resource_recordings.c
--- 1:13.23.1~dfsg-2/res/ari/resource_recordings.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_recordings.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,14 +24,11 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">res_stasis_recording</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis_app_recording.h"
 #include "resource_recordings.h"
 
@@ -101,6 +98,50 @@ void ast_ari_recordings_get_stored(struc
 	ast_ari_response_ok(response, json);
 }
 
+void ast_ari_recordings_get_stored_file(struct ast_tcptls_session_instance *ser,
+	struct ast_variable *headers, struct ast_ari_recordings_get_stored_file_args *args,
+	struct ast_ari_response *response)
+{
+	RAII_VAR(struct stasis_app_stored_recording *, recording,
+		stasis_app_stored_recording_find_by_name(args->recording_name),
+		ao2_cleanup);
+	static const char *format_type_names[AST_MEDIA_TYPE_TEXT + 1] = {
+		[AST_MEDIA_TYPE_UNKNOWN] = "binary",
+		[AST_MEDIA_TYPE_AUDIO] = "audio",
+		[AST_MEDIA_TYPE_VIDEO] = "video",
+		[AST_MEDIA_TYPE_IMAGE] = "image",
+		[AST_MEDIA_TYPE_TEXT] = "text",
+	};
+	struct ast_format *format;
+
+	response->message = ast_json_null();
+
+	if (!recording) {
+		ast_ari_response_error(response, 404, "Not Found",
+			"Recording not found");
+		return;
+	}
+
+	format = ast_get_format_for_file_ext(stasis_app_stored_recording_get_extension(recording));
+	if (!format) {
+		ast_ari_response_error(response, 500, "Internal Server Error",
+			"Format specified by recording not available or loaded");
+		return;
+	}
+
+	response->fd = open(stasis_app_stored_recording_get_filename(recording), O_RDONLY);
+	if (response->fd < 0) {
+		ast_ari_response_error(response, 403, "Forbidden",
+			"Recording could not be opened");
+		return;
+	}
+
+	ast_str_append(&response->headers, 0, "Content-Type: %s/%s\r\n",
+		format_type_names[ast_format_get_type(format)],
+		stasis_app_stored_recording_get_extension(recording));
+	ast_ari_response_ok(response, ast_json_null());
+}
+
 void ast_ari_recordings_copy_stored(struct ast_variable *headers,
 	struct ast_ari_recordings_copy_stored_args *args,
 	struct ast_ari_response *response)
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_recordings.h 1:16.1.1~dfsg-1/res/ari/resource_recordings.h
--- 1:13.23.1~dfsg-2/res/ari/resource_recordings.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_recordings.h	2018-12-26 18:45:13.000000000 +0000
@@ -76,6 +76,20 @@ struct ast_ari_recordings_delete_stored_
  * \param[out] response HTTP response
  */
 void ast_ari_recordings_delete_stored(struct ast_variable *headers, struct ast_ari_recordings_delete_stored_args *args, struct ast_ari_response *response);
+/*! Argument struct for ast_ari_recordings_get_stored_file() */
+struct ast_ari_recordings_get_stored_file_args {
+	/*! The name of the recording */
+	const char *recording_name;
+};
+/*!
+ * \brief Get the file associated with the stored recording.
+ *
+ * \param ser TCP/TLS session instance
+ * \param headers HTTP headers
+ * \param args Swagger parameters
+ * \param[out] response HTTP response
+ */
+void ast_ari_recordings_get_stored_file(struct ast_tcptls_session_instance *ser, struct ast_variable *headers, struct ast_ari_recordings_get_stored_file_args *args, struct ast_ari_response *response);
 /*! Argument struct for ast_ari_recordings_copy_stored() */
 struct ast_ari_recordings_copy_stored_args {
 	/*! The name of the recording to copy */
diff -pruN 1:13.23.1~dfsg-2/res/ari/resource_sounds.c 1:16.1.1~dfsg-1/res/ari/resource_sounds.c
--- 1:13.23.1~dfsg-2/res/ari/resource_sounds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari/resource_sounds.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_sounds.h"
 #include "asterisk/media_index.h"
 #include "asterisk/sounds_index.h"
diff -pruN 1:13.23.1~dfsg-2/res/ari.make 1:16.1.1~dfsg-1/res/ari.make
--- 1:13.23.1~dfsg-2/res/ari.make	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/ari.make	2018-12-26 18:45:13.000000000 +0000
@@ -17,58 +17,14 @@
 # rest-api-templates/ari.make.mustache
 #
 
-res_ari_asterisk.so: ari/resource_asterisk.o
-.res_ari_asterisk.moduleinfo: ari/resource_asterisk.c
-
-ari/resource_asterisk.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_asterisk)
-
-res_ari_endpoints.so: ari/resource_endpoints.o
-.res_ari_endpoints.moduleinfo: ari/resource_endpoints.c
-
-ari/resource_endpoints.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_endpoints)
-
-res_ari_channels.so: ari/resource_channels.o
-.res_ari_channels.moduleinfo: ari/resource_channels.c
-
-ari/resource_channels.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_channels)
-
-res_ari_bridges.so: ari/resource_bridges.o
-.res_ari_bridges.moduleinfo: ari/resource_bridges.c
-
-ari/resource_bridges.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_bridges)
-
-res_ari_recordings.so: ari/resource_recordings.o
-.res_ari_recordings.moduleinfo: ari/resource_recordings.c
-
-ari/resource_recordings.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_recordings)
-
-res_ari_sounds.so: ari/resource_sounds.o
-.res_ari_sounds.moduleinfo: ari/resource_sounds.c
-
-ari/resource_sounds.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_sounds)
-
-res_ari_playbacks.so: ari/resource_playbacks.o
-.res_ari_playbacks.moduleinfo: ari/resource_playbacks.c
-
-ari/resource_playbacks.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_playbacks)
-
-res_ari_device_states.so: ari/resource_device_states.o
-.res_ari_device_states.moduleinfo: ari/resource_device_states.c
-
-ari/resource_device_states.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_device_states)
-
-res_ari_mailboxes.so: ari/resource_mailboxes.o
-.res_ari_mailboxes.moduleinfo: ari/resource_mailboxes.c
-
-ari/resource_mailboxes.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_mailboxes)
-
-res_ari_events.so: ari/resource_events.o
-.res_ari_events.moduleinfo: ari/resource_events.c
-
-ari/resource_events.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_events)
-
-res_ari_applications.so: ari/resource_applications.o
-.res_ari_applications.moduleinfo: ari/resource_applications.c
-
-ari/resource_applications.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_applications)
-
+$(call MOD_ADD_C,res_ari_asterisk,ari/resource_asterisk.c)
+$(call MOD_ADD_C,res_ari_endpoints,ari/resource_endpoints.c)
+$(call MOD_ADD_C,res_ari_channels,ari/resource_channels.c)
+$(call MOD_ADD_C,res_ari_bridges,ari/resource_bridges.c)
+$(call MOD_ADD_C,res_ari_recordings,ari/resource_recordings.c)
+$(call MOD_ADD_C,res_ari_sounds,ari/resource_sounds.c)
+$(call MOD_ADD_C,res_ari_playbacks,ari/resource_playbacks.c)
+$(call MOD_ADD_C,res_ari_device_states,ari/resource_device_states.c)
+$(call MOD_ADD_C,res_ari_mailboxes,ari/resource_mailboxes.c)
+$(call MOD_ADD_C,res_ari_events,ari/resource_events.c)
+$(call MOD_ADD_C,res_ari_applications,ari/resource_applications.c)
diff -pruN 1:13.23.1~dfsg-2/res/Makefile 1:16.1.1~dfsg-1/res/Makefile
--- 1:13.23.1~dfsg-2/res/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -37,21 +37,6 @@ ael/ael.tab.o: _ASTCFLAGS+=-I. -Iael -DY
 		_ASTCFLAGS+=-Wno-parentheses-equality
 	endif
 
-res_ais.so: ais/clm.o ais/evt.o
-ais/clm.o ais/evt.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ais)
-
-res_snmp.so: snmp/agent.o
-snmp/agent.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_snmp)
-
-res_ael_share.so: ael/ael_lex.o ael/ael.tab.o ael/pval.o
-ael/ael_lex.o ael/ael.tab.o ael/pval.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ael_share)
-
-res_pjsip.so: $(subst .c,.o,$(wildcard res_pjsip/*.c))
-$(subst .c,.o,$(wildcard res_pjsip/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_pjsip)
-
-res_stasis.so: $(subst .c,.o,$(wildcard stasis/*.c))
-$(subst .c,.o,$(wildcard stasis/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_stasis)
-
 ifneq ($(findstring REBUILD_PARSERS,$(MENUSELECT_CFLAGS)),)
 ael/ael_lex.c: ael/ael.flex
 else
@@ -60,7 +45,8 @@ endif
 	$(ECHO_PREFIX) echo "   [FLEX] $< -> $@"
 	$(CMD_PREFIX) (cd ael; $(FLEX) ael.flex)
 	$(CMD_PREFIX) sed 's@#if __STDC_VERSION__ >= 199901L@#if !defined __STDC_VERSION__ || __STDC_VERSION__ >= 199901L@' $@ > $@.fix
-	$(CMD_PREFIX) echo "#include \"asterisk.h\"" > $@
+	$(CMD_PREFIX) echo "#define ASTMM_LIBC ASTMM_REDIRECT" > $@
+	$(CMD_PREFIX) echo "#include \"asterisk.h\"" >> $@
 	$(CMD_PREFIX) echo >> $@
 	$(CMD_PREFIX) cat $@.fix >> $@
 	$(CMD_PREFIX) rm $@.fix
@@ -75,25 +61,14 @@ endif
 
 ael/pval.o: ael/pval.c
 
-clean::
-	rm -f snmp/*.o snmp/*.i ael/*.o ael/*.i ais/*.o ais/*.i snmp/*.gcda snmp/*.gcno ael/*.gcda ael/*.gcno
-	rm -f res_pjsip/*.[oi] res_pjsip/*.gcda res_pjsip/*.gcno
-	rm -f stasis/*.[oi] stasis/*.gcda stasis/*.gcno
-	rm -f parking/*.[oi] parking/*.gcda parking/*.gcno
-	rm -f stasis_recording/*.[oi] stasis_recording/*.gcda stasis_recording/*.gcno
-	rm -f ari/*.[oi] ari/*.gcda ari/*.gcno
-
-res_parking.so: $(subst .c,.o,$(wildcard parking/*.c))
-$(subst .c,.o,$(wildcard parking/*.c)): _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_parking)
-
-res_ari.so: ari/cli.o ari/config.o ari/ari_websockets.o
-ari/cli.o ari/config.o ari/ari_websockets.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari)
-
-res_ari_model.so: ari/ari_model_validators.o
-ari/ari_model_validators.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_model)
-
-res_stasis_recording.so: stasis_recording/stored.o
-stasis_recording/stored.o:  _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_stasis_recording)
+$(call MOD_ADD_C,res_ael_share,ael/ael_lex.c ael/ael.tab.c ael/pval.c)
+$(call MOD_ADD_C,res_stasis,$(wildcard stasis/*.c))
+$(call MOD_ADD_C,res_snmp,snmp/agent.c)
+$(call MOD_ADD_C,res_parking,$(wildcard parking/*.c))
+$(call MOD_ADD_C,res_pjsip,$(wildcard res_pjsip/*.c))
+$(call MOD_ADD_C,res_ari,ari/cli.c ari/config.c ari/ari_websockets.c)
+$(call MOD_ADD_C,res_ari_model,ari/ari_model_validators.c)
+$(call MOD_ADD_C,res_stasis_recording,stasis_recording/stored.c)
 
 res_parking.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
 
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_applications.c 1:16.1.1~dfsg-1/res/parking/parking_applications.c
--- 1:13.23.1~dfsg-2/res/parking/parking_applications.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_applications.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "res_parking.h"
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
@@ -870,6 +868,10 @@ static void park_announce_update_cb(void
 		return;
 	}
 
+	if (ast_parked_call_type() != stasis_message_type(message)) {
+		return;
+	}
+
 	if (payload->event_type != PARKED_CALL) {
 		/* We are only concerned with calls parked */
 		return;
@@ -956,6 +958,10 @@ static int park_and_announce_app_exec(st
 		return -1;
 	}
 
+	stasis_subscription_accept_message_type(parking_subscription, ast_parked_call_type());
+	stasis_subscription_accept_message_type(parking_subscription, stasis_subscription_change_type());
+	stasis_subscription_set_filter(parking_subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
+
 	/* Now for the fun part... park it! */
 	ast_bridge_join(parking_bridge, chan, NULL, &chan_features, NULL, 0);
 
@@ -982,8 +988,6 @@ static int park_and_announce_app_exec(st
 
 int load_parking_applications(void)
 {
-	const struct ast_module_info *ast_module_info = parking_get_module_info();
-
 	if (ast_register_application_xml(PARK_APPLICATION, park_app_exec)) {
 		return -1;
 	}
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_bridge_features.c 1:16.1.1~dfsg-1/res/parking/parking_bridge_features.c
--- 1:13.23.1~dfsg-2/res/parking/parking_bridge_features.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_bridge_features.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,17 +25,17 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "res_parking.h"
 #include "asterisk/utils.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/logger.h"
 #include "asterisk/pbx.h"
+#include "asterisk/app.h"
 #include "asterisk/bridge.h"
 #include "asterisk/bridge_internal.h"
 #include "asterisk/bridge_channel.h"
 #include "asterisk/bridge_features.h"
+#include "asterisk/conversions.h"
 #include "asterisk/features.h"
 #include "asterisk/say.h"
 #include "asterisk/datastore.h"
@@ -44,6 +44,24 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/core_local.h"
 #include "asterisk/causes.h"
 
+/*** DOCUMENTATION
+	<function name="PARK_GET_CHANNEL" language="en_US">
+		<synopsis>
+			Get the channel name of an occupied parking space in a parking lot.
+		</synopsis>
+			<syntax>
+				<parameter name="parking_space" required="true">
+				</parameter>
+				<parameter name="parking_lot" required="true">
+				</parameter>
+			</syntax>
+		<description>
+			<para>This function returns the channel of the specified parking space
+				if the parking lot space is occupied.</para>
+			</description>
+	</function>
+***/
+
 struct parked_subscription_datastore {
 	struct stasis_subscription *parked_subscription;
 };
@@ -195,6 +213,9 @@ static int create_parked_subscription_fu
 	if (!(parked_datastore->parked_subscription = stasis_subscribe_pool(ast_parking_topic(), parker_update_cb, subscription_data))) {
 		return -1;
 	}
+	stasis_subscription_accept_message_type(parked_datastore->parked_subscription, ast_parked_call_type());
+	stasis_subscription_accept_message_type(parked_datastore->parked_subscription, stasis_subscription_change_type());
+	stasis_subscription_set_filter(parked_datastore->parked_subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	datastore->data = parked_datastore;
 
@@ -498,7 +519,7 @@ static int parking_park_call(struct ast_
 
 static int feature_park_call(struct ast_bridge_channel *bridge_channel, void *hook_pvt)
 {
-	SCOPED_MODULE_USE(parking_get_module_info()->self);
+	SCOPED_MODULE_USE(AST_MODULE_SELF);
 
 	parking_park_call(bridge_channel, NULL, 0);
 	return 0;
@@ -635,7 +656,7 @@ static int parking_duration_callback(str
 		ast_debug(3, "An extension for '%s@%s' was already registered by another registrar '%s'\n",
 			dial_string_flat, PARK_DIAL_CONTEXT, ast_get_extension_registrar(existing_exten));
 	} else if (ast_add_extension2_nolock(park_dial_context, 1, dial_string_flat, 1, NULL, NULL,
-			"Dial", duplicate_returnexten, ast_free_ptr, BASE_REGISTRAR)) {
+			"Dial", duplicate_returnexten, ast_free_ptr, BASE_REGISTRAR, NULL, 0)) {
 			ast_free(duplicate_returnexten);
 		ast_log(LOG_ERROR, "Failed to create parking redial parker extension %s@%s - Dial(%s)\n",
 			dial_string_flat, PARK_DIAL_CONTEXT, returnexten);
@@ -706,6 +727,60 @@ void parking_set_duration(struct ast_bri
 	}
 }
 
+/*! \brief  Dial plan function to get the parking lot channel of an occupied parking lot */
+static int func_get_parkingslot_channel(struct ast_channel *chan, const char *function, char *data, char *buf, size_t len)
+{
+	RAII_VAR(struct parked_user *, pu, NULL, ao2_cleanup);
+	RAII_VAR(struct parking_lot *, lot, NULL, ao2_cleanup);
+	unsigned int space = 0;
+	const char *content = NULL;
+
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(parking_space);
+		AST_APP_ARG(parking_lot);
+		AST_APP_ARG(other);
+	);
+
+	/* Parse the arguments. */
+	AST_STANDARD_APP_ARGS(args, data);
+
+	if (args.argc < 2) {
+		/* Didn't receive enough arguments to do anything */
+		ast_log(LOG_ERROR, "Usage: %s(<parking_space>,<parking_lot>)\n",
+			function);
+		return -1;
+	}
+
+	lot = parking_lot_find_by_name(args.parking_lot);
+	if (!lot) {
+		ast_log(LOG_ERROR, "Could not find parking lot: '%s'\n", args.parking_lot);
+		return -1;
+	}
+
+	if (!ast_strlen_zero(args.parking_space)) {
+		if (ast_str_to_uint(args.parking_space, &space) != 0) {
+			ast_log(LOG_ERROR, "value '%s' for parking_space argument is invalid. Must be an integer greater than 0.\n",
+				args.parking_space);
+			return -1;
+		}
+	}
+
+	pu = parking_lot_inspect_parked_user(lot, space);
+	if (!pu) {
+		return -1;
+	}
+
+	content = ast_channel_name(pu->chan);
+	ast_copy_string(buf, content, len);
+
+	return 0;
+}
+
+static struct ast_custom_function getparkingslotchannel_function = {
+	.name = "PARK_GET_CHANNEL",
+	.read = func_get_parkingslot_channel,
+};
+
 struct ast_parking_bridge_feature_fn_table parking_provider = {
 	.module_version = PARKING_MODULE_VERSION,
 	.module_name = __FILE__,
@@ -719,11 +794,14 @@ void unload_parking_bridge_features(void
 {
 	ast_bridge_features_unregister(AST_BRIDGE_BUILTIN_PARKCALL);
 	ast_parking_unregister_bridge_features(parking_provider.module_name);
+	ast_custom_function_unregister(&getparkingslotchannel_function);
 }
 
 int load_parking_bridge_features(void)
 {
-	parking_provider.module_info = parking_get_module_info();
+	parking_provider.module = AST_MODULE_SELF;
+
+	ast_custom_function_register(&getparkingslotchannel_function);
 
 	if (ast_parking_register_bridge_features(&parking_provider)) {
 		return -1;
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_controller.c 1:16.1.1~dfsg-1/res/parking/parking_controller.c
--- 1:13.23.1~dfsg-2/res/parking/parking_controller.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_controller.c	2018-12-26 18:45:13.000000000 +0000
@@ -163,6 +163,23 @@ static int retrieve_parked_user_targeted
 	return 0;
 }
 
+struct parked_user *parking_lot_inspect_parked_user(struct parking_lot *lot, int target)
+{
+	struct parked_user *user;
+
+	if (target < 0) {
+		user = ao2_callback(lot->parked_users, 0, NULL, NULL);
+	} else {
+		user = ao2_callback(lot->parked_users, 0, retrieve_parked_user_targeted, &target);
+	}
+
+	if (!user) {
+		return NULL;
+	}
+
+	return user;
+}
+
 struct parked_user *parking_lot_retrieve_parked_user(struct parking_lot *lot, int target)
 {
 	RAII_VAR(struct parked_user *, user, NULL, ao2_cleanup);
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_manager.c 1:16.1.1~dfsg-1/res/parking/parking_manager.c
--- 1:13.23.1~dfsg-2/res/parking/parking_manager.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_manager.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "res_parking.h"
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
@@ -688,17 +686,18 @@ static void parking_manager_enable_stasi
 {
 	if (!parking_sub) {
 		parking_sub = stasis_subscribe(ast_parking_topic(), parking_event_cb, NULL);
+		stasis_subscription_accept_message_type(parking_sub, ast_parked_call_type());
+		stasis_subscription_set_filter(parking_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 }
 
 int load_parking_manager(void)
 {
 	int res;
-	const struct ast_module_info *module = parking_get_module_info();
 
-	res = ast_manager_register2("Parkinglots", EVENT_FLAG_CALL, manager_parking_lot_list, module->self, NULL, NULL);
-	res |= ast_manager_register2("ParkedCalls", EVENT_FLAG_CALL, manager_parking_status, module->self, NULL, NULL);
-	res |= ast_manager_register2("Park", EVENT_FLAG_CALL, manager_park, module->self, NULL, NULL);
+	res = ast_manager_register_xml("Parkinglots", EVENT_FLAG_CALL, manager_parking_lot_list);
+	res |= ast_manager_register_xml("ParkedCalls", EVENT_FLAG_CALL, manager_parking_status);
+	res |= ast_manager_register_xml("Park", EVENT_FLAG_CALL, manager_park);
 	parking_manager_enable_stasis();
 	return res ? -1 : 0;
 }
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_tests.c 1:16.1.1~dfsg-1/res/parking/parking_tests.c
--- 1:13.23.1~dfsg-2/res/parking/parking_tests.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_tests.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "res_parking.h"
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/res/parking/parking_ui.c 1:16.1.1~dfsg-1/res/parking/parking_ui.c
--- 1:13.23.1~dfsg-2/res/parking/parking_ui.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/parking_ui.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "res_parking.h"
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
diff -pruN 1:13.23.1~dfsg-2/res/parking/res_parking.h 1:16.1.1~dfsg-1/res/parking/res_parking.h
--- 1:13.23.1~dfsg-2/res/parking/res_parking.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/parking/res_parking.h	2018-12-26 18:45:13.000000000 +0000
@@ -196,6 +196,19 @@ struct ast_bridge *parking_lot_get_bridg
 int parking_lot_get_space(struct parking_lot *lot, int target_override);
 
 /*!
+ * \brief Determine if there is a parked user in a parking space and return it if there is.
+ *
+ * \param lot Parking lot being pulled from
+ * \param target If < 0   search for the first occupied space in the parking lot
+ *               If >= 0  Only pull from the indicated target
+ *
+ * \retval NULL if no parked user could be pulled from the requested parking lot at the requested parking space
+ * \retval reference to the requested parked user
+ *
+ */
+struct parked_user *parking_lot_inspect_parked_user(struct parking_lot *lot, int target);
+
+/*!
  * \since 12.0.0
  * \brief Determine if there is a parked user in a parking space and pull it from the parking lot if there is.
  *
@@ -561,12 +574,3 @@ int load_parking_tests(void);
  * \return Nothing
  */
 void unload_parking_tests(void);
-
-struct ast_module_info;
-/*!
- * \since 12.0.0
- * \brief Get res_parking's module info
- *
- * \retval res_parking's ast_module
- */
-const struct ast_module_info *parking_get_module_info(void);
diff -pruN 1:13.23.1~dfsg-2/res/res_adsi.c 1:16.1.1~dfsg-1/res/res_adsi.c
--- 1:13.23.1~dfsg-2/res/res_adsi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_adsi.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,13 +31,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>core</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <time.h>
 #include <math.h>
 
@@ -1210,9 +1208,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "ADSI Resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ael_share.c 1:16.1.1~dfsg-1/res/res_ael_share.c
--- 1:13.23.1~dfsg-2/res/res_ael_share.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ael_share.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -53,7 +51,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "share-able code for AEL",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-				.load = load_module,
-				.unload = unload_module
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_agi.c 1:16.1.1~dfsg-1/res/res_agi.c
--- 1:13.23.1~dfsg-2/res/res_agi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_agi.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 #include <signal.h>
 #include <sys/time.h>
@@ -2388,7 +2386,7 @@ static int handle_waitfordigit(struct as
 		return RESULT_SHOWUSAGE;
 	if (sscanf(argv[3], "%30d", &to) != 1)
 		return RESULT_SHOWUSAGE;
-	res = ast_waitfordigit_full(chan, to, agi->audio, agi->ctrl);
+	res = ast_waitfordigit_full(chan, to, NULL, agi->audio, agi->ctrl);
 	ast_agi_send(agi->fd, chan, "200 result=%d\n", res);
 	return (res >= 0) ? RESULT_SUCCESS : RESULT_FAILURE;
 }
@@ -2668,7 +2666,7 @@ static int handle_getoption(struct ast_c
 
 	/* If the user didnt press a key, wait for digitTimeout*/
 	if (res == 0 ) {
-		res = ast_waitfordigit_full(chan, timeout, agi->audio, agi->ctrl);
+		res = ast_waitfordigit_full(chan, timeout, NULL, agi->audio, agi->ctrl);
 		/* Make sure the new result is in the escape digits of the GET OPTION */
 		if ( !strchr(edigits,res) )
 			res=0;
@@ -3817,8 +3815,6 @@ int AST_OPTIONAL_API_NAME(ast_agi_regist
 		AST_RWLIST_WRLOCK(&agi_commands);
 		AST_LIST_INSERT_TAIL(&agi_commands, cmd, list);
 		AST_RWLIST_UNLOCK(&agi_commands);
-		if (mod != ast_module_info->self)
-			ast_module_ref(ast_module_info->self);
 		ast_verb(2, "AGI Command '%s' registered\n",fullcmd);
 		return 1;
 	} else {
@@ -3827,7 +3823,7 @@ int AST_OPTIONAL_API_NAME(ast_agi_regist
 	}
 }
 
-int AST_OPTIONAL_API_NAME(ast_agi_unregister)(struct ast_module *mod, agi_command *cmd)
+int AST_OPTIONAL_API_NAME(ast_agi_unregister)(agi_command *cmd)
 {
 	struct agi_command *e;
 	int unregistered = 0;
@@ -3839,8 +3835,6 @@ int AST_OPTIONAL_API_NAME(ast_agi_unregi
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&agi_commands, e, list) {
 		if (cmd == e) {
 			AST_RWLIST_REMOVE_CURRENT(list);
-			if (mod != ast_module_info->self)
-				ast_module_unref(ast_module_info->self);
 #ifdef AST_XML_DOCS
 			if (e->docsrc == AST_XML_DOC) {
 				ast_free((char *) e->summary);
@@ -3887,7 +3881,7 @@ int AST_OPTIONAL_API_NAME(ast_agi_regist
 			   to fail is if the command is not
 			   registered
 			*/
-			(void) ast_agi_unregister(mod, cmd + x - 1);
+			(void) ast_agi_unregister(cmd + x - 1);
 		}
 		return -1;
 	}
@@ -3895,7 +3889,7 @@ int AST_OPTIONAL_API_NAME(ast_agi_regist
 	return 0;
 }
 
-int AST_OPTIONAL_API_NAME(ast_agi_unregister_multiple)(struct ast_module *mod, struct agi_command *cmd, unsigned int len)
+int AST_OPTIONAL_API_NAME(ast_agi_unregister_multiple)(struct agi_command *cmd, unsigned int len)
 {
 	unsigned int i;
 	int res = 0;
@@ -3905,7 +3899,7 @@ int AST_OPTIONAL_API_NAME(ast_agi_unregi
 		   attempts failed... there is no recourse if
 		   any of them do
 		*/
-		res |= ast_agi_unregister(mod, cmd + i);
+		res |= ast_agi_unregister(cmd + i);
 	}
 
 	return res;
@@ -4042,14 +4036,19 @@ static enum agi_result agi_handle_comman
 
 	parse_args(buf, &argc, argv);
 	c = find_command(argv, 0);
-	if (c && (!dead || (dead && c->dead))) {
-		/* if this command wasn't registered by res_agi, be sure to usecount
-		the module we are using */
-		if (c->mod != ast_module_info->self)
-			ast_module_ref(c->mod);
+	if (!c || !ast_module_running_ref(c->mod)) {
+		ami_res = "Invalid or unknown command";
+		resultcode = 510;
+
+		ast_agi_send(agi->fd, chan, "%d %s\n", resultcode, ami_res);
+
+		publish_async_exec_end(chan, command_id, ami_cmd, resultcode, ami_res);
+
+		return AGI_RESULT_SUCCESS;
+	}
+
+	if (!dead || (dead && c->dead)) {
 		res = c->handler(chan, agi, argc, argv);
-		if (c->mod != ast_module_info->self)
-			ast_module_unref(c->mod);
 		switch (res) {
 		case RESULT_SHOWUSAGE:
 			ami_res = "Usage";
@@ -4096,21 +4095,15 @@ static enum agi_result agi_handle_comman
 
 			break;
 		}
-	} else if (c) {
+	} else {
 		ami_res = "Command Not Permitted on a dead channel or intercept routine";
 		resultcode = 511;
 
 		ast_agi_send(agi->fd, chan, "%d %s\n", resultcode, ami_res);
 
 		publish_async_exec_end(chan, command_id, ami_cmd, resultcode, ami_res);
-	} else {
-		ami_res = "Invalid or unknown command";
-		resultcode = 510;
-
-		ast_agi_send(agi->fd, chan, "%d %s\n", resultcode, ami_res);
-
-		publish_async_exec_end(chan, command_id, ami_cmd, resultcode, ami_res);
 	}
+	ast_module_unref(c->mod);
 
 	return AGI_RESULT_SUCCESS;
 }
@@ -4662,7 +4655,7 @@ AST_TEST_DEFINE(test_agi_null_docs)
 	}
 #endif
 
-	ast_agi_unregister(ast_module_info->self, &noop_command);
+	ast_agi_unregister(&noop_command);
 	return res;
 }
 #endif
@@ -4676,7 +4669,7 @@ static int unload_module(void)
 	STASIS_MESSAGE_TYPE_CLEANUP(agi_async_end_type);
 
 	ast_cli_unregister_multiple(cli_agi, ARRAY_LEN(cli_agi));
-	ast_agi_unregister_multiple(ast_module_info->self, commands, ARRAY_LEN(commands));
+	ast_agi_unregister_multiple(commands, ARRAY_LEN(commands));
 	ast_unregister_application(eapp);
 	ast_unregister_application(deadapp);
 	ast_manager_unregister("AGI");
@@ -4709,15 +4702,13 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Asterisk Gateway Interface (AGI)",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_speech",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_applications.c 1:16.1.1~dfsg-1/res/res_ari_applications.c
--- 1:13.23.1~dfsg-2/res/res_ari_applications.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_applications.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -495,7 +493,6 @@ static struct stasis_rest_handlers appli
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&applications);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -503,10 +500,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&applications);
 	if (res) {
 		unload_module();
@@ -520,5 +514,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_asterisk.c 1:16.1.1~dfsg-1/res/res_ari_asterisk.c
--- 1:13.23.1~dfsg-2/res/res_ari_asterisk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_asterisk.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -1216,7 +1214,6 @@ static struct stasis_rest_handlers aster
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&asterisk);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -1224,10 +1221,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&asterisk);
 	if (res) {
 		unload_module();
@@ -1241,5 +1235,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_bridges.c 1:16.1.1~dfsg-1/res/res_ari_bridges.c
--- 1:13.23.1~dfsg-2/res/res_ari_bridges.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_bridges.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,13 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_recording</depend>
+	<depend type="module">res_stasis_playback</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -432,6 +432,14 @@ int ast_ari_bridges_add_channel_parse_bo
 	if (field) {
 		args->role = ast_json_string_get(field);
 	}
+	field = ast_json_object_get(body, "absorbDTMF");
+	if (field) {
+		args->absorb_dtmf = ast_json_is_true(field);
+	}
+	field = ast_json_object_get(body, "mute");
+	if (field) {
+		args->mute = ast_json_is_true(field);
+	}
 	return 0;
 }
 
@@ -501,6 +509,12 @@ static void ast_ari_bridges_add_channel_
 		if (strcmp(i->name, "role") == 0) {
 			args.role = (i->value);
 		} else
+		if (strcmp(i->name, "absorbDTMF") == 0) {
+			args.absorb_dtmf = ast_true(i->value);
+		} else
+		if (strcmp(i->name, "mute") == 0) {
+			args.mute = ast_true(i->value);
+		} else
 		{}
 	}
 	for (i = path_vars; i; i = i->next) {
@@ -972,7 +986,32 @@ int ast_ari_bridges_play_parse_body(
 	/* Parse query parameters out of it */
 	field = ast_json_object_get(body, "media");
 	if (field) {
-		args->media = ast_json_string_get(field);
+		/* If they were silly enough to both pass in a query param and a
+		 * JSON body, free up the query value.
+		 */
+		ast_free(args->media);
+		if (ast_json_typeof(field) == AST_JSON_ARRAY) {
+			/* Multiple param passed as array */
+			size_t i;
+			args->media_count = ast_json_array_size(field);
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+
+			if (!args->media) {
+				return -1;
+			}
+
+			for (i = 0; i < args->media_count; ++i) {
+				args->media[i] = ast_json_string_get(ast_json_array_get(field, i));
+			}
+		} else {
+			/* Multiple param passed as single value */
+			args->media_count = 1;
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+			if (!args->media) {
+				return -1;
+			}
+			args->media[0] = ast_json_string_get(field);
+		}
 	}
 	field = ast_json_object_get(body, "lang");
 	if (field) {
@@ -1014,7 +1053,47 @@ static void ast_ari_bridges_play_cb(
 
 	for (i = get_params; i; i = i->next) {
 		if (strcmp(i->name, "media") == 0) {
-			args.media = (i->value);
+			/* Parse comma separated list */
+			char *vals[MAX_VALS];
+			size_t j;
+
+			args.media_parse = ast_strdup(i->value);
+			if (!args.media_parse) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (strlen(args.media_parse) == 0) {
+				/* ast_app_separate_args can't handle "" */
+				args.media_count = 1;
+				vals[0] = args.media_parse;
+			} else {
+				args.media_count = ast_app_separate_args(
+					args.media_parse, ',', vals,
+					ARRAY_LEN(vals));
+			}
+
+			if (args.media_count == 0) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (args.media_count >= MAX_VALS) {
+				ast_ari_response_error(response, 400,
+					"Bad Request",
+					"Too many values for media");
+				goto fin;
+			}
+
+			args.media = ast_malloc(sizeof(*args.media) * args.media_count);
+			if (!args.media) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			for (j = 0; j < args.media_count; ++j) {
+				args.media[j] = (vals[j]);
+			}
 		} else
 		if (strcmp(i->name, "lang") == 0) {
 			args.lang = (i->value);
@@ -1072,6 +1151,8 @@ static void ast_ari_bridges_play_cb(
 #endif /* AST_DEVMODE */
 
 fin: __attribute__((unused))
+	ast_free(args.media_parse);
+	ast_free(args.media);
 	return;
 }
 int ast_ari_bridges_play_with_id_parse_body(
@@ -1082,7 +1163,32 @@ int ast_ari_bridges_play_with_id_parse_b
 	/* Parse query parameters out of it */
 	field = ast_json_object_get(body, "media");
 	if (field) {
-		args->media = ast_json_string_get(field);
+		/* If they were silly enough to both pass in a query param and a
+		 * JSON body, free up the query value.
+		 */
+		ast_free(args->media);
+		if (ast_json_typeof(field) == AST_JSON_ARRAY) {
+			/* Multiple param passed as array */
+			size_t i;
+			args->media_count = ast_json_array_size(field);
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+
+			if (!args->media) {
+				return -1;
+			}
+
+			for (i = 0; i < args->media_count; ++i) {
+				args->media[i] = ast_json_string_get(ast_json_array_get(field, i));
+			}
+		} else {
+			/* Multiple param passed as single value */
+			args->media_count = 1;
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+			if (!args->media) {
+				return -1;
+			}
+			args->media[0] = ast_json_string_get(field);
+		}
 	}
 	field = ast_json_object_get(body, "lang");
 	if (field) {
@@ -1120,7 +1226,47 @@ static void ast_ari_bridges_play_with_id
 
 	for (i = get_params; i; i = i->next) {
 		if (strcmp(i->name, "media") == 0) {
-			args.media = (i->value);
+			/* Parse comma separated list */
+			char *vals[MAX_VALS];
+			size_t j;
+
+			args.media_parse = ast_strdup(i->value);
+			if (!args.media_parse) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (strlen(args.media_parse) == 0) {
+				/* ast_app_separate_args can't handle "" */
+				args.media_count = 1;
+				vals[0] = args.media_parse;
+			} else {
+				args.media_count = ast_app_separate_args(
+					args.media_parse, ',', vals,
+					ARRAY_LEN(vals));
+			}
+
+			if (args.media_count == 0) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (args.media_count >= MAX_VALS) {
+				ast_ari_response_error(response, 400,
+					"Bad Request",
+					"Too many values for media");
+				goto fin;
+			}
+
+			args.media = ast_malloc(sizeof(*args.media) * args.media_count);
+			if (!args.media) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			for (j = 0; j < args.media_count; ++j) {
+				args.media[j] = (vals[j]);
+			}
 		} else
 		if (strcmp(i->name, "lang") == 0) {
 			args.lang = (i->value);
@@ -1178,6 +1324,8 @@ static void ast_ari_bridges_play_with_id
 #endif /* AST_DEVMODE */
 
 fin: __attribute__((unused))
+	ast_free(args.media_parse);
+	ast_free(args.media);
 	return;
 }
 int ast_ari_bridges_record_parse_body(
@@ -1408,7 +1556,6 @@ static struct stasis_rest_handlers bridg
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&bridges);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -1416,10 +1563,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&bridges);
 	if (res) {
 		unload_module();
@@ -1433,5 +1577,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_recording,res_stasis_playback",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari.c 1:16.1.1~dfsg-1/res/res_ari.c
--- 1:13.23.1~dfsg-2/res/res_ari.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari.c	2018-12-26 18:45:13.000000000 +0000
@@ -113,6 +113,9 @@
 				<configOption name="allowed_origins">
 					<synopsis>Comma separated list of allowed origins, for Cross-Origin Resource Sharing. May be set to * to allow all origins.</synopsis>
 				</configOption>
+				<configOption name="channelvars">
+					<synopsis>Comma separated list of channel variables to display in channel json.</synopsis>
+				</configOption>
 			</configObject>
 
 			<configObject name="user">
@@ -141,8 +144,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "ari/internal.h"
 #include "asterisk/ari.h"
 #include "asterisk/astobj2.h"
@@ -195,7 +196,6 @@ int ast_ari_add_handler(struct stasis_re
 	ao2_cleanup(root_handler);
 	ao2_ref(new_handler, +1);
 	root_handler = new_handler;
-	ast_module_ref(ast_module_info->self);
 	return 0;
 }
 
@@ -221,7 +221,6 @@ int ast_ari_remove_handler(struct stasis
 	memcpy(new_handler, root_handler, sizeof(*new_handler));
 	for (i = 0, j = 0; i < root_handler->num_children; ++i) {
 		if (root_handler->children[i] == handler) {
-			ast_module_unref(ast_module_info->self);
 			continue;
 		}
 		new_handler->children[j++] = root_handler->children[i];
@@ -880,7 +879,7 @@ static int ast_ari_callback(struct ast_t
 	RAII_VAR(struct ast_ari_conf *, conf, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_str *, response_body, ast_str_create(256), ast_free);
 	RAII_VAR(struct ast_ari_conf_user *, user, NULL, ao2_cleanup);
-	struct ast_ari_response response = {};
+	struct ast_ari_response response = { .fd = -1, 0 };
 	RAII_VAR(struct ast_variable *, post_vars, NULL, ast_variables_destroy);
 	struct ast_variable *var;
 	const char *app_name = NULL;
@@ -1087,11 +1086,14 @@ request_failed:
 
 	ast_http_send(ser, method, response.response_code,
 		      response.response_text, response.headers, response_body,
-		      0, 0);
+		      response.fd != -1 ? response.fd : 0, 0);
 	/* ast_http_send takes ownership, so we don't have to free them */
 	response_body = NULL;
 
 	ast_json_unref(response.message);
+	if (response.fd >= 0) {
+		close(response.fd);
+	}
 	return 0;
 }
 
@@ -1099,7 +1101,6 @@ static struct ast_http_uri http_uri = {
 	.callback = ast_ari_callback,
 	.description = "Asterisk RESTful API",
 	.uri = "ari",
-
 	.has_subtree = 1,
 	.data = NULL,
 	.key = __FILE__,
@@ -1194,6 +1195,7 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload_module,
-	.nonoptreq = "res_http_websocket",
+	.optional_modules = "res_http_websocket",
+	.requires = "http,res_stasis",
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_channels.c 1:16.1.1~dfsg-1/res/res_ari_channels.c
--- 1:13.23.1~dfsg-2/res/res_ari_channels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_channels.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,15 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_answer</depend>
+	<depend type="module">res_stasis_playback</depend>
+	<depend type="module">res_stasis_recording</depend>
+	<depend type="module">res_stasis_snoop</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -259,6 +261,123 @@ static void ast_ari_channels_originate_c
 fin: __attribute__((unused))
 	return;
 }
+int ast_ari_channels_create_parse_body(
+	struct ast_json *body,
+	struct ast_ari_channels_create_args *args)
+{
+	struct ast_json *field;
+	/* Parse query parameters out of it */
+	field = ast_json_object_get(body, "endpoint");
+	if (field) {
+		args->endpoint = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "app");
+	if (field) {
+		args->app = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "appArgs");
+	if (field) {
+		args->app_args = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "channelId");
+	if (field) {
+		args->channel_id = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "otherChannelId");
+	if (field) {
+		args->other_channel_id = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "originator");
+	if (field) {
+		args->originator = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "formats");
+	if (field) {
+		args->formats = ast_json_string_get(field);
+	}
+	return 0;
+}
+
+/*!
+ * \brief Parameter parsing callback for /channels/create.
+ * \param get_params GET parameters in the HTTP request.
+ * \param path_vars Path variables extracted from the request.
+ * \param headers HTTP headers.
+ * \param[out] response Response to the HTTP request.
+ */
+static void ast_ari_channels_create_cb(
+	struct ast_tcptls_session_instance *ser,
+	struct ast_variable *get_params, struct ast_variable *path_vars,
+	struct ast_variable *headers, struct ast_json *body, struct ast_ari_response *response)
+{
+	struct ast_ari_channels_create_args args = {};
+	struct ast_variable *i;
+#if defined(AST_DEVMODE)
+	int is_valid;
+	int code;
+#endif /* AST_DEVMODE */
+
+	for (i = get_params; i; i = i->next) {
+		if (strcmp(i->name, "endpoint") == 0) {
+			args.endpoint = (i->value);
+		} else
+		if (strcmp(i->name, "app") == 0) {
+			args.app = (i->value);
+		} else
+		if (strcmp(i->name, "appArgs") == 0) {
+			args.app_args = (i->value);
+		} else
+		if (strcmp(i->name, "channelId") == 0) {
+			args.channel_id = (i->value);
+		} else
+		if (strcmp(i->name, "otherChannelId") == 0) {
+			args.other_channel_id = (i->value);
+		} else
+		if (strcmp(i->name, "originator") == 0) {
+			args.originator = (i->value);
+		} else
+		if (strcmp(i->name, "formats") == 0) {
+			args.formats = (i->value);
+		} else
+		{}
+	}
+	if (ast_ari_channels_create_parse_body(body, &args)) {
+		ast_ari_response_alloc_failed(response);
+		goto fin;
+	}
+	ast_ari_channels_create(headers, &args, response);
+#if defined(AST_DEVMODE)
+	code = response->response_code;
+
+	switch (code) {
+	case 0: /* Implementation is still a stub, or the code wasn't set */
+		is_valid = response->message == NULL;
+		break;
+	case 500: /* Internal Server Error */
+	case 501: /* Not Implemented */
+	case 409: /* Channel with given unique ID already exists. */
+		is_valid = 1;
+		break;
+	default:
+		if (200 <= code && code <= 299) {
+			is_valid = ast_ari_validate_channel(
+				response->message);
+		} else {
+			ast_log(LOG_ERROR, "Invalid error response %d for /channels/create\n", code);
+			is_valid = 0;
+		}
+	}
+
+	if (!is_valid) {
+		ast_log(LOG_ERROR, "Response validation failed for /channels/create\n");
+		ast_ari_response_error(response, 500,
+			"Internal Server Error", "Response validation failed");
+	}
+#endif /* AST_DEVMODE */
+
+fin: __attribute__((unused))
+	return;
+}
 /*!
  * \brief Parameter parsing callback for /channels/{channelId}.
  * \param get_params GET parameters in the HTTP request.
@@ -636,6 +755,7 @@ static void ast_ari_channels_continue_in
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -720,6 +840,7 @@ static void ast_ari_channels_redirect_cb
 	case 404: /* Channel or endpoint not found */
 	case 409: /* Channel not in a Stasis application */
 	case 422: /* Endpoint is not the same type as the channel */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -779,6 +900,7 @@ static void ast_ari_channels_answer_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -838,6 +960,7 @@ static void ast_ari_channels_ring_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -897,6 +1020,7 @@ static void ast_ari_channels_ring_stop_c
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1008,6 +1132,7 @@ static void ast_ari_channels_send_dtmf_c
 	case 400: /* DTMF is required */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1090,6 +1215,7 @@ static void ast_ari_channels_mute_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1172,6 +1298,7 @@ static void ast_ari_channels_unmute_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1231,6 +1358,7 @@ static void ast_ari_channels_hold_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1290,6 +1418,7 @@ static void ast_ari_channels_unhold_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1372,6 +1501,7 @@ static void ast_ari_channels_start_moh_c
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1431,6 +1561,7 @@ static void ast_ari_channels_stop_moh_cb
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1490,6 +1621,7 @@ static void ast_ari_channels_start_silen
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1549,6 +1681,7 @@ static void ast_ari_channels_stop_silenc
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1579,7 +1712,32 @@ int ast_ari_channels_play_parse_body(
 	/* Parse query parameters out of it */
 	field = ast_json_object_get(body, "media");
 	if (field) {
-		args->media = ast_json_string_get(field);
+		/* If they were silly enough to both pass in a query param and a
+		 * JSON body, free up the query value.
+		 */
+		ast_free(args->media);
+		if (ast_json_typeof(field) == AST_JSON_ARRAY) {
+			/* Multiple param passed as array */
+			size_t i;
+			args->media_count = ast_json_array_size(field);
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+
+			if (!args->media) {
+				return -1;
+			}
+
+			for (i = 0; i < args->media_count; ++i) {
+				args->media[i] = ast_json_string_get(ast_json_array_get(field, i));
+			}
+		} else {
+			/* Multiple param passed as single value */
+			args->media_count = 1;
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+			if (!args->media) {
+				return -1;
+			}
+			args->media[0] = ast_json_string_get(field);
+		}
 	}
 	field = ast_json_object_get(body, "lang");
 	if (field) {
@@ -1621,7 +1779,47 @@ static void ast_ari_channels_play_cb(
 
 	for (i = get_params; i; i = i->next) {
 		if (strcmp(i->name, "media") == 0) {
-			args.media = (i->value);
+			/* Parse comma separated list */
+			char *vals[MAX_VALS];
+			size_t j;
+
+			args.media_parse = ast_strdup(i->value);
+			if (!args.media_parse) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (strlen(args.media_parse) == 0) {
+				/* ast_app_separate_args can't handle "" */
+				args.media_count = 1;
+				vals[0] = args.media_parse;
+			} else {
+				args.media_count = ast_app_separate_args(
+					args.media_parse, ',', vals,
+					ARRAY_LEN(vals));
+			}
+
+			if (args.media_count == 0) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (args.media_count >= MAX_VALS) {
+				ast_ari_response_error(response, 400,
+					"Bad Request",
+					"Too many values for media");
+				goto fin;
+			}
+
+			args.media = ast_malloc(sizeof(*args.media) * args.media_count);
+			if (!args.media) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			for (j = 0; j < args.media_count; ++j) {
+				args.media[j] = (vals[j]);
+			}
 		} else
 		if (strcmp(i->name, "lang") == 0) {
 			args.lang = (i->value);
@@ -1659,6 +1857,7 @@ static void ast_ari_channels_play_cb(
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1679,6 +1878,8 @@ static void ast_ari_channels_play_cb(
 #endif /* AST_DEVMODE */
 
 fin: __attribute__((unused))
+	ast_free(args.media_parse);
+	ast_free(args.media);
 	return;
 }
 int ast_ari_channels_play_with_id_parse_body(
@@ -1689,7 +1890,32 @@ int ast_ari_channels_play_with_id_parse_
 	/* Parse query parameters out of it */
 	field = ast_json_object_get(body, "media");
 	if (field) {
-		args->media = ast_json_string_get(field);
+		/* If they were silly enough to both pass in a query param and a
+		 * JSON body, free up the query value.
+		 */
+		ast_free(args->media);
+		if (ast_json_typeof(field) == AST_JSON_ARRAY) {
+			/* Multiple param passed as array */
+			size_t i;
+			args->media_count = ast_json_array_size(field);
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+
+			if (!args->media) {
+				return -1;
+			}
+
+			for (i = 0; i < args->media_count; ++i) {
+				args->media[i] = ast_json_string_get(ast_json_array_get(field, i));
+			}
+		} else {
+			/* Multiple param passed as single value */
+			args->media_count = 1;
+			args->media = ast_malloc(sizeof(*args->media) * args->media_count);
+			if (!args->media) {
+				return -1;
+			}
+			args->media[0] = ast_json_string_get(field);
+		}
 	}
 	field = ast_json_object_get(body, "lang");
 	if (field) {
@@ -1727,7 +1953,47 @@ static void ast_ari_channels_play_with_i
 
 	for (i = get_params; i; i = i->next) {
 		if (strcmp(i->name, "media") == 0) {
-			args.media = (i->value);
+			/* Parse comma separated list */
+			char *vals[MAX_VALS];
+			size_t j;
+
+			args.media_parse = ast_strdup(i->value);
+			if (!args.media_parse) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (strlen(args.media_parse) == 0) {
+				/* ast_app_separate_args can't handle "" */
+				args.media_count = 1;
+				vals[0] = args.media_parse;
+			} else {
+				args.media_count = ast_app_separate_args(
+					args.media_parse, ',', vals,
+					ARRAY_LEN(vals));
+			}
+
+			if (args.media_count == 0) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			if (args.media_count >= MAX_VALS) {
+				ast_ari_response_error(response, 400,
+					"Bad Request",
+					"Too many values for media");
+				goto fin;
+			}
+
+			args.media = ast_malloc(sizeof(*args.media) * args.media_count);
+			if (!args.media) {
+				ast_ari_response_alloc_failed(response);
+				goto fin;
+			}
+
+			for (j = 0; j < args.media_count; ++j) {
+				args.media[j] = (vals[j]);
+			}
 		} else
 		if (strcmp(i->name, "lang") == 0) {
 			args.lang = (i->value);
@@ -1765,6 +2031,7 @@ static void ast_ari_channels_play_with_i
 	case 501: /* Not Implemented */
 	case 404: /* Channel not found */
 	case 409: /* Channel not in a Stasis application */
+	case 412: /* Channel in invalid state */
 		is_valid = 1;
 		break;
 	default:
@@ -1785,6 +2052,8 @@ static void ast_ari_channels_play_with_i
 #endif /* AST_DEVMODE */
 
 fin: __attribute__((unused))
+	ast_free(args.media_parse);
+	ast_free(args.media);
 	return;
 }
 int ast_ari_channels_record_parse_body(
@@ -2302,7 +2571,105 @@ static void ast_ari_channels_snoop_chann
 fin: __attribute__((unused))
 	return;
 }
+int ast_ari_channels_dial_parse_body(
+	struct ast_json *body,
+	struct ast_ari_channels_dial_args *args)
+{
+	struct ast_json *field;
+	/* Parse query parameters out of it */
+	field = ast_json_object_get(body, "caller");
+	if (field) {
+		args->caller = ast_json_string_get(field);
+	}
+	field = ast_json_object_get(body, "timeout");
+	if (field) {
+		args->timeout = ast_json_integer_get(field);
+	}
+	return 0;
+}
+
+/*!
+ * \brief Parameter parsing callback for /channels/{channelId}/dial.
+ * \param get_params GET parameters in the HTTP request.
+ * \param path_vars Path variables extracted from the request.
+ * \param headers HTTP headers.
+ * \param[out] response Response to the HTTP request.
+ */
+static void ast_ari_channels_dial_cb(
+	struct ast_tcptls_session_instance *ser,
+	struct ast_variable *get_params, struct ast_variable *path_vars,
+	struct ast_variable *headers, struct ast_json *body, struct ast_ari_response *response)
+{
+	struct ast_ari_channels_dial_args args = {};
+	struct ast_variable *i;
+#if defined(AST_DEVMODE)
+	int is_valid;
+	int code;
+#endif /* AST_DEVMODE */
 
+	for (i = get_params; i; i = i->next) {
+		if (strcmp(i->name, "caller") == 0) {
+			args.caller = (i->value);
+		} else
+		if (strcmp(i->name, "timeout") == 0) {
+			args.timeout = atoi(i->value);
+		} else
+		{}
+	}
+	for (i = path_vars; i; i = i->next) {
+		if (strcmp(i->name, "channelId") == 0) {
+			args.channel_id = (i->value);
+		} else
+		{}
+	}
+	if (ast_ari_channels_dial_parse_body(body, &args)) {
+		ast_ari_response_alloc_failed(response);
+		goto fin;
+	}
+	ast_ari_channels_dial(headers, &args, response);
+#if defined(AST_DEVMODE)
+	code = response->response_code;
+
+	switch (code) {
+	case 0: /* Implementation is still a stub, or the code wasn't set */
+		is_valid = response->message == NULL;
+		break;
+	case 500: /* Internal Server Error */
+	case 501: /* Not Implemented */
+	case 404: /* Channel cannot be found. */
+	case 409: /* Channel cannot be dialed. */
+		is_valid = 1;
+		break;
+	default:
+		if (200 <= code && code <= 299) {
+			is_valid = ast_ari_validate_void(
+				response->message);
+		} else {
+			ast_log(LOG_ERROR, "Invalid error response %d for /channels/{channelId}/dial\n", code);
+			is_valid = 0;
+		}
+	}
+
+	if (!is_valid) {
+		ast_log(LOG_ERROR, "Response validation failed for /channels/{channelId}/dial\n");
+		ast_ari_response_error(response, 500,
+			"Internal Server Error", "Response validation failed");
+	}
+#endif /* AST_DEVMODE */
+
+fin: __attribute__((unused))
+	return;
+}
+
+/*! \brief REST handler for /api-docs/channels.json */
+static struct stasis_rest_handlers channels_create = {
+	.path_segment = "create",
+	.callbacks = {
+		[AST_HTTP_POST] = ast_ari_channels_create_cb,
+	},
+	.num_children = 0,
+	.children = {  }
+};
 /*! \brief REST handler for /api-docs/channels.json */
 static struct stasis_rest_handlers channels_channelId_continue = {
 	.path_segment = "continue",
@@ -2447,6 +2814,15 @@ static struct stasis_rest_handlers chann
 	.children = { &channels_channelId_snoop_snoopId, }
 };
 /*! \brief REST handler for /api-docs/channels.json */
+static struct stasis_rest_handlers channels_channelId_dial = {
+	.path_segment = "dial",
+	.callbacks = {
+		[AST_HTTP_POST] = ast_ari_channels_dial_cb,
+	},
+	.num_children = 0,
+	.children = {  }
+};
+/*! \brief REST handler for /api-docs/channels.json */
 static struct stasis_rest_handlers channels_channelId = {
 	.path_segment = "channelId",
 	.is_wildcard = 1,
@@ -2455,8 +2831,8 @@ static struct stasis_rest_handlers chann
 		[AST_HTTP_POST] = ast_ari_channels_originate_with_id_cb,
 		[AST_HTTP_DELETE] = ast_ari_channels_hangup_cb,
 	},
-	.num_children = 13,
-	.children = { &channels_channelId_continue,&channels_channelId_redirect,&channels_channelId_answer,&channels_channelId_ring,&channels_channelId_dtmf,&channels_channelId_mute,&channels_channelId_hold,&channels_channelId_moh,&channels_channelId_silence,&channels_channelId_play,&channels_channelId_record,&channels_channelId_variable,&channels_channelId_snoop, }
+	.num_children = 14,
+	.children = { &channels_channelId_continue,&channels_channelId_redirect,&channels_channelId_answer,&channels_channelId_ring,&channels_channelId_dtmf,&channels_channelId_mute,&channels_channelId_hold,&channels_channelId_moh,&channels_channelId_silence,&channels_channelId_play,&channels_channelId_record,&channels_channelId_variable,&channels_channelId_snoop,&channels_channelId_dial, }
 };
 /*! \brief REST handler for /api-docs/channels.json */
 static struct stasis_rest_handlers channels = {
@@ -2465,14 +2841,13 @@ static struct stasis_rest_handlers chann
 		[AST_HTTP_GET] = ast_ari_channels_list_cb,
 		[AST_HTTP_POST] = ast_ari_channels_originate_cb,
 	},
-	.num_children = 1,
-	.children = { &channels_channelId, }
+	.num_children = 2,
+	.children = { &channels_create,&channels_channelId, }
 };
 
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&channels);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -2480,10 +2855,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&channels);
 	if (res) {
 		unload_module();
@@ -2497,5 +2869,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_answer,res_stasis_playback,res_stasis_recording,res_stasis_snoop",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_device_states.c 1:16.1.1~dfsg-1/res/res_ari_device_states.c
--- 1:13.23.1~dfsg-2/res/res_ari_device_states.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_device_states.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,12 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_device_state</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -326,7 +325,6 @@ static struct stasis_rest_handlers devic
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&deviceStates);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -334,10 +332,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&deviceStates);
 	if (res) {
 		unload_module();
@@ -351,5 +346,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_device_state",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_endpoints.c 1:16.1.1~dfsg-1/res/res_ari_endpoints.c
--- 1:13.23.1~dfsg-2/res/res_ari_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -450,7 +448,6 @@ static struct stasis_rest_handlers endpo
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&endpoints);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -458,10 +455,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&endpoints);
 	if (res) {
 		unload_module();
@@ -475,5 +469,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_events.c 1:16.1.1~dfsg-1/res/res_ari_events.c
--- 1:13.23.1~dfsg-2/res/res_ari_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,12 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_http_websocket</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -53,7 +52,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #define MAX_VALS 128
 
-static int ast_ari_events_event_websocket_ws_attempted_cb(struct ast_tcptls_session_instance *ser, struct ast_variable *get_params, struct ast_variable *headers)
+static int ast_ari_events_event_websocket_ws_attempted_cb(struct ast_tcptls_session_instance *ser,
+	struct ast_variable *get_params, struct ast_variable *headers, const char *session_id)
 {
 	struct ast_ari_events_event_websocket_args args = {};
 	int res = 0;
@@ -116,7 +116,7 @@ static int ast_ari_events_event_websocke
 		{}
 	}
 
-	res = ast_ari_websocket_events_event_websocket_attempted(ser, headers, &args);
+	res = ast_ari_websocket_events_event_websocket_attempted(ser, headers, &args, session_id);
 
 fin: __attribute__((unused))
 	if (!response) {
@@ -423,7 +423,7 @@ static int unload_module(void)
 	ast_ari_remove_handler(&events);
 	ao2_cleanup(events.ws_server);
 	events.ws_server = NULL;
-	stasis_app_unref();
+	ast_ari_websocket_events_event_websocket_dtor();
 	return 0;
 }
 
@@ -431,29 +431,29 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
+	struct ast_websocket_protocol *protocol;
 
-	/* This is scoped to not conflict with CHECK_ARI_MODULE_LOADED */
-	{
-		struct ast_websocket_protocol *protocol;
-
-		events.ws_server = ast_websocket_server_create();
-		if (!events.ws_server) {
-			return AST_MODULE_LOAD_DECLINE;
-		}
+	if (ast_ari_websocket_events_event_websocket_init() == -1) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
 
-		protocol = ast_websocket_sub_protocol_alloc("ari");
-		if (!protocol) {
-			ao2_ref(events.ws_server, -1);
-			events.ws_server = NULL;
-			return AST_MODULE_LOAD_DECLINE;
-		}
-		protocol->session_attempted = ast_ari_events_event_websocket_ws_attempted_cb;
-		protocol->session_established = ast_ari_events_event_websocket_ws_established_cb;
-		res |= ast_websocket_server_add_protocol2(events.ws_server, protocol);
+	events.ws_server = ast_websocket_server_create();
+	if (!events.ws_server) {
+		ast_ari_websocket_events_event_websocket_dtor();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	protocol = ast_websocket_sub_protocol_alloc("ari");
+	if (!protocol) {
+		ao2_ref(events.ws_server, -1);
+		events.ws_server = NULL;
+		ast_ari_websocket_events_event_websocket_dtor();
+		return AST_MODULE_LOAD_DECLINE;
 	}
+	protocol->session_attempted = ast_ari_events_event_websocket_ws_attempted_cb;
+	protocol->session_established = ast_ari_events_event_websocket_ws_established_cb;
+	res |= ast_websocket_server_add_protocol2(events.ws_server, protocol);
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&events);
 	if (res) {
 		unload_module();
@@ -467,5 +467,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_http_websocket",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_mailboxes.c 1:16.1.1~dfsg-1/res/res_ari_mailboxes.c
--- 1:13.23.1~dfsg-2/res/res_ari_mailboxes.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_mailboxes.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,12 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_mailbox</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -332,7 +331,6 @@ static struct stasis_rest_handlers mailb
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&mailboxes);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -340,10 +338,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&mailboxes);
 	if (res) {
 		unload_module();
@@ -357,5 +352,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_mailbox",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_model.c 1:16.1.1~dfsg-1/res/res_ari_model.c
--- 1:13.23.1~dfsg-2/res/res_ari_model.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_model.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "ari/ari_model_validators.h"
 #include "asterisk/logger.h"
 #include "asterisk/module.h"
@@ -208,4 +206,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-        );
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_playbacks.c 1:16.1.1~dfsg-1/res/res_ari_playbacks.c
--- 1:13.23.1~dfsg-2/res/res_ari_playbacks.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_playbacks.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,12 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_playback</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -284,7 +283,6 @@ static struct stasis_rest_handlers playb
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&playbacks);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -292,10 +290,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&playbacks);
 	if (res) {
 		unload_module();
@@ -309,5 +304,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_playback",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_recordings.c 1:16.1.1~dfsg-1/res/res_ari_recordings.c
--- 1:13.23.1~dfsg-2/res/res_ari_recordings.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_recordings.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,13 +35,12 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+	<depend type="module">res_stasis_recording</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -218,6 +217,65 @@ static void ast_ari_recordings_delete_st
 fin: __attribute__((unused))
 	return;
 }
+/*!
+ * \brief Parameter parsing callback for /recordings/stored/{recordingName}/file.
+ * \param get_params GET parameters in the HTTP request.
+ * \param path_vars Path variables extracted from the request.
+ * \param headers HTTP headers.
+ * \param[out] response Response to the HTTP request.
+ */
+static void ast_ari_recordings_get_stored_file_cb(
+	struct ast_tcptls_session_instance *ser,
+	struct ast_variable *get_params, struct ast_variable *path_vars,
+	struct ast_variable *headers, struct ast_json *body, struct ast_ari_response *response)
+{
+	struct ast_ari_recordings_get_stored_file_args args = {};
+	struct ast_variable *i;
+#if defined(AST_DEVMODE)
+	int is_valid;
+	int code;
+#endif /* AST_DEVMODE */
+
+	for (i = path_vars; i; i = i->next) {
+		if (strcmp(i->name, "recordingName") == 0) {
+			args.recording_name = (i->value);
+		} else
+		{}
+	}
+	ast_ari_recordings_get_stored_file(ser, headers, &args, response);
+#if defined(AST_DEVMODE)
+	code = response->response_code;
+
+	switch (code) {
+	case 0: /* Implementation is still a stub, or the code wasn't set */
+		is_valid = response->message == NULL;
+		break;
+	case 500: /* Internal Server Error */
+	case 501: /* Not Implemented */
+	case 403: /* The recording file could not be opened */
+	case 404: /* Recording not found */
+		is_valid = 1;
+		break;
+	default:
+		if (200 <= code && code <= 299) {
+			/* No validation on a raw binary response */
+			is_valid = 1;
+		} else {
+			ast_log(LOG_ERROR, "Invalid error response %d for /recordings/stored/{recordingName}/file\n", code);
+			is_valid = 0;
+		}
+	}
+
+	if (!is_valid) {
+		ast_log(LOG_ERROR, "Response validation failed for /recordings/stored/{recordingName}/file\n");
+		ast_ari_response_error(response, 500,
+			"Internal Server Error", "Response validation failed");
+	}
+#endif /* AST_DEVMODE */
+
+fin: __attribute__((unused))
+	return;
+}
 int ast_ari_recordings_copy_stored_parse_body(
 	struct ast_json *body,
 	struct ast_ari_recordings_copy_stored_args *args)
@@ -712,6 +770,15 @@ fin: __attribute__((unused))
 }
 
 /*! \brief REST handler for /api-docs/recordings.json */
+static struct stasis_rest_handlers recordings_stored_recordingName_file = {
+	.path_segment = "file",
+	.callbacks = {
+		[AST_HTTP_GET] = ast_ari_recordings_get_stored_file_cb,
+	},
+	.num_children = 0,
+	.children = {  }
+};
+/*! \brief REST handler for /api-docs/recordings.json */
 static struct stasis_rest_handlers recordings_stored_recordingName_copy = {
 	.path_segment = "copy",
 	.callbacks = {
@@ -728,8 +795,8 @@ static struct stasis_rest_handlers recor
 		[AST_HTTP_GET] = ast_ari_recordings_get_stored_cb,
 		[AST_HTTP_DELETE] = ast_ari_recordings_delete_stored_cb,
 	},
-	.num_children = 1,
-	.children = { &recordings_stored_recordingName_copy, }
+	.num_children = 2,
+	.children = { &recordings_stored_recordingName_file,&recordings_stored_recordingName_copy, }
 };
 /*! \brief REST handler for /api-docs/recordings.json */
 static struct stasis_rest_handlers recordings_stored = {
@@ -800,7 +867,6 @@ static struct stasis_rest_handlers recor
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&recordings);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -808,10 +874,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&recordings);
 	if (res) {
 		unload_module();
@@ -825,5 +888,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis,res_stasis_recording",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_ari_sounds.c 1:16.1.1~dfsg-1/res/res_ari_sounds.c
--- 1:13.23.1~dfsg-2/res/res_ari_sounds.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_ari_sounds.c	2018-12-26 18:45:13.000000000 +0000
@@ -40,8 +40,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -214,7 +212,6 @@ static struct stasis_rest_handlers sound
 static int unload_module(void)
 {
 	ast_ari_remove_handler(&sounds);
-	stasis_app_unref();
 	return 0;
 }
 
@@ -222,10 +219,7 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&sounds);
 	if (res) {
 		unload_module();
@@ -239,5 +233,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_calendar.c 1:16.1.1~dfsg-1/res/res_calendar.c
--- 1:13.23.1~dfsg-2/res/res_calendar.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_calendar.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,13 +33,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/_private.h"
 #include "asterisk/channel.h"
 #include "asterisk/calendar.h"
@@ -403,22 +401,35 @@ static struct ast_calendar *build_calend
 {
 	struct ast_calendar *cal;
 	struct ast_variable *v, *last = NULL;
+	int new_calendar = 0;
 
-	if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
-		ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
-		return NULL;
-	}
-
-	if (!(cal->events = ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn))) {
-		ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
+	cal = find_calendar(cat);
+	if (cal && cal->fetch_again_at_reload) {
+		/** Create new calendar, old will be removed during reload */
 		cal = unref_calendar(cal);
-		return NULL;
 	}
+	if (!cal) {
+		new_calendar = 1;
+		if (!(cal = ao2_alloc(sizeof(*cal), calendar_destructor))) {
+			ast_log(LOG_ERROR, "Could not allocate calendar structure. Stopping.\n");
+			return NULL;
+		}
 
-	if (ast_string_field_init(cal, 32)) {
-		ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
-		cal = unref_calendar(cal);
-		return NULL;
+		cal->events = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+			CALENDAR_BUCKETS, event_hash_fn, NULL, event_cmp_fn);
+		if (!cal->events) {
+			ast_log(LOG_ERROR, "Could not allocate events container for %s\n", cat);
+			cal = unref_calendar(cal);
+			return NULL;
+		}
+
+		if (ast_string_field_init(cal, 32)) {
+			ast_log(LOG_ERROR, "Couldn't create string fields for %s\n", cat);
+			cal = unref_calendar(cal);
+			return NULL;
+		}
+	} else {
+		cal->pending_deletion = 0;
 	}
 
 	ast_string_field_set(cal, name, cat);
@@ -427,6 +438,7 @@ static struct ast_calendar *build_calend
 	cal->refresh = 3600;
 	cal->timeframe = 60;
 	cal->notify_waittime = 30000;
+	cal->fetch_again_at_reload = 0;
 
 	for (v = ast_variable_browse(cfg, cat); v; v = v->next) {
 		if (!strcasecmp(v->name, "autoreminder")) {
@@ -448,6 +460,8 @@ static struct ast_calendar *build_calend
 			ast_string_field_set(cal, notify_appdata, v->value);
 		} else if (!strcasecmp(v->name, "refresh")) {
 			cal->refresh = atoi(v->value);
+		} else if (!strcasecmp(v->name, "fetch_again_at_reload")) {
+			cal->fetch_again_at_reload = ast_true(v->value);
 		} else if (!strcasecmp(v->name, "timeframe")) {
 			cal->timeframe = atoi(v->value);
 		} else if (!strcasecmp(v->name, "setvar")) {
@@ -480,15 +494,17 @@ static struct ast_calendar *build_calend
 				cal->name);
 	}
 
-	cal->thread = AST_PTHREADT_NULL;
-	ast_cond_init(&cal->unload, NULL);
-	ao2_link(calendars, cal);
-	if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
-		/* If we start failing to create threads, go ahead and return NULL
-		 * and the tech module will be unregistered
-		 */
-		ao2_unlink(calendars, cal);
-		cal = unref_calendar(cal);
+	if (new_calendar) {
+		cal->thread = AST_PTHREADT_NULL;
+		ast_cond_init(&cal->unload, NULL);
+		ao2_link(calendars, cal);
+		if (ast_pthread_create(&cal->thread, NULL, cal->tech->load_calendar, cal)) {
+			/* If we start failing to create threads, go ahead and return NULL
+			 * and the tech module will be unregistered
+			 */
+			ao2_unlink(calendars, cal);
+			cal = unref_calendar(cal);
+		}
 	}
 
 	return cal;
@@ -672,7 +688,8 @@ struct ast_calendar_event *ast_calendar_
 
 struct ao2_container *ast_calendar_event_container_alloc(void)
 {
-	return ao2_container_alloc(CALENDAR_BUCKETS, event_hash_fn, event_cmp_fn);
+	return ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, CALENDAR_BUCKETS,
+		event_hash_fn, NULL, event_cmp_fn);
 }
 
 static void event_notification_destroy(void *data)
@@ -1759,16 +1776,30 @@ static struct ast_custom_function calend
 	.read = calendar_event_read,
 };
 
+static int cb_pending_deletion(void *user_data, void *arg, int flags)
+{
+	struct ast_calendar *cal = user_data;
+
+	cal->pending_deletion = 1;
+
+	return CMP_MATCH;
+}
+
+static int cb_rm_pending_deletion(void *user_data, void *arg, int flags)
+{
+	struct ast_calendar *cal = user_data;
+
+	return cal->pending_deletion ? CMP_MATCH : 0;
+}
+
 static int reload(void)
 {
 	struct ast_calendar_tech *iter;
 
 	ast_mutex_lock(&reloadlock);
 
-	/* Delete all of the calendars */
-	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, NULL, NULL);
-
-	/* Load configuration */
+	/* Mark existing calendars for deletion */
+	ao2_callback(calendars, OBJ_NODATA | OBJ_MULTIPLE, cb_pending_deletion, NULL);
 	load_config(1);
 
 	AST_LIST_LOCK(&techs);
@@ -1779,6 +1810,9 @@ static int reload(void)
 	}
 	AST_LIST_UNLOCK(&techs);
 
+	/* Delete calendars that no longer show up in the config */
+	ao2_callback(calendars, OBJ_UNLINK | OBJ_NODATA | OBJ_MULTIPLE, cb_rm_pending_deletion, NULL);
+
 	ast_mutex_unlock(&reloadlock);
 
 	return 0;
@@ -1865,7 +1899,9 @@ static int unload_module(void)
  */
 static int load_module(void)
 {
-	if (!(calendars = ao2_container_alloc(CALENDAR_BUCKETS, calendar_hash_fn, calendar_cmp_fn))) {
+	calendars = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, CALENDAR_BUCKETS,
+		calendar_hash_fn, NULL, calendar_cmp_fn);
+	if (!calendars) {
 		ast_log(LOG_ERROR, "Unable to allocate calendars container!\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -1902,9 +1938,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Asterisk Calendar integration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_calendar_caldav.c 1:16.1.1~dfsg-1/res/res_calendar_caldav.c
--- 1:13.23.1~dfsg-2/res/res_calendar_caldav.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_calendar_caldav.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,13 +25,11 @@
 	<depend>neon</depend>
 	<depend>ical</depend>
 	<depend>libxml2</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libical/ical.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -751,8 +749,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk CalDAV Calendar Integration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
+	.requires = "res_calendar",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_calendar_ews.c 1:16.1.1~dfsg-1/res/res_calendar_ews.c
--- 1:13.23.1~dfsg-2/res/res_calendar_ews.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_calendar_ews.c	2018-12-26 18:45:13.000000000 +0000
@@ -23,13 +23,11 @@
 /*** MODULEINFO
 	<depend>res_calendar</depend>
 	<depend>neon29</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ne_request.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -939,8 +937,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk MS Exchange Web Service Calendar Integration",
-	.support_level = AST_MODULE_SUPPORT_CORE,
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
+	.requires = "res_calendar",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_calendar_exchange.c 1:16.1.1~dfsg-1/res/res_calendar_exchange.c
--- 1:13.23.1~dfsg-2/res/res_calendar_exchange.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_calendar_exchange.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,13 +25,11 @@
 	<depend>neon</depend>
 	<depend>ical</depend>
 	<depend>iksemel</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libical/ical.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -742,8 +740,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk MS Exchange Calendar Integration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
+	.requires = "res_calendar",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_calendar_icalendar.c 1:16.1.1~dfsg-1/res/res_calendar_icalendar.c
--- 1:13.23.1~dfsg-2/res/res_calendar_icalendar.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_calendar_icalendar.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,13 +24,11 @@
 	<depend>res_calendar</depend>
 	<depend>neon</depend>
 	<depend>ical</depend>
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libical/ical.h>
 #include <ne_session.h>
 #include <ne_uri.h>
@@ -540,8 +538,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk iCalendar .ics file integration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_DEVSTATE_PLUGIN,
+	.requires = "res_calendar",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_chan_stats.c 1:16.1.1~dfsg-1/res/res_chan_stats.c
--- 1:13.23.1~dfsg-2/res/res_chan_stats.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_chan_stats.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/stasis_channels.h"
 #include "asterisk/stasis_message_router.h"
@@ -184,5 +182,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_statsd"
-	);
+	.requires = "res_statsd"
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_clialiases.c 1:16.1.1~dfsg-1/res/res_clialiases.c
--- 1:13.23.1~dfsg-2/res/res_clialiases.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_clialiases.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/config.h"
 #include "asterisk/cli.h"
@@ -284,7 +282,9 @@ static int unload_module(void)
  */
 static int load_module(void)
 {
-	if (!(cli_aliases = ao2_container_alloc(MAX_ALIAS_BUCKETS, alias_hash_cb, alias_cmp_cb))) {
+	cli_aliases = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_ALIAS_BUCKETS, alias_hash_cb, NULL, alias_cmp_cb);
+	if (!cli_aliases) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -296,8 +296,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "CLI Aliases",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_clioriginate.c 1:16.1.1~dfsg-1/res/res_clioriginate.c
--- 1:13.23.1~dfsg-2/res/res_clioriginate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_clioriginate.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
@@ -160,8 +158,6 @@ static char *handle_orig(struct ast_cli_
 			"used. If no extension is given, the 's' extension will be used.\n";
 		return NULL;
 	case CLI_GENERATE:
-		/* ugly, can be removed when CLI entries have ast_module pointers */
-		ast_module_ref(ast_module_info->self);
 		if (a->pos == 3) {
 			res = ast_cli_complete(a->word, choices, a->n);
 		} else if (a->pos == 4) {
@@ -169,16 +165,12 @@ static char *handle_orig(struct ast_cli_
 				res = ast_complete_applications(a->line, a->word, a->n);
 			}
 		}
-		ast_module_unref(ast_module_info->self);
 		return res;
 	}
 
 	if (ast_strlen_zero(a->argv[2]) || ast_strlen_zero(a->argv[3]))
 		return CLI_SHOWUSAGE;
 
-	/* ugly, can be removed when CLI entries have ast_module pointers */
-	ast_module_ref(ast_module_info->self);
-
 	if (!strcasecmp("application", a->argv[3])) {
 		res = orig_app(a->fd, a->argv[2], a->argv[4], a->argv[5]);
 	} else if (!strcasecmp("extension", a->argv[3])) {
@@ -187,8 +179,6 @@ static char *handle_orig(struct ast_cli_
 		res = CLI_SHOWUSAGE;
 	}
 
-	ast_module_unref(ast_module_info->self);
-
 	return res;
 }
 
diff -pruN 1:13.23.1~dfsg-2/res/res_config_curl.c 1:16.1.1~dfsg-1/res/res_config_curl.c
--- 1:13.23.1~dfsg-2/res/res_config_curl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_curl.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <curl/curl.h>
 
 #include "asterisk/file.h"
@@ -639,20 +637,6 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	if (!ast_module_check("res_curl.so")) {
-		if (ast_load_resource("res_curl.so") != AST_MODULE_LOAD_SUCCESS) {
-			ast_log(LOG_ERROR, "Cannot load res_curl, so res_config_curl cannot be loaded\n");
-			return AST_MODULE_LOAD_DECLINE;
-		}
-	}
-
-	if (!ast_module_check("func_curl.so")) {
-		if (ast_load_resource("func_curl.so") != AST_MODULE_LOAD_SUCCESS) {
-			ast_log(LOG_ERROR, "Cannot load func_curl, so res_config_curl cannot be loaded\n");
-			return AST_MODULE_LOAD_DECLINE;
-		}
-	}
-
 	reload_module();
 
 	ast_config_engine_register(&curl_engine);
@@ -661,9 +645,10 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Realtime Curl configuration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig,res_curl,func_curl",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_config_ldap.c 1:16.1.1~dfsg-1/res/res_config_ldap.c
--- 1:13.23.1~dfsg-2/res/res_config_ldap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_ldap.c	2018-12-26 18:45:13.000000000 +0000
@@ -53,8 +53,6 @@
 #include <stdio.h>
 #include <ldap.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/logger.h"
 #include "asterisk/config.h"
@@ -89,7 +87,7 @@ struct category_and_metric {
 	int metric;
 	const char *variable_name;
 	const char *variable_value;
-	int var_metric; /*!< For organizing variables (particularly includes and switch statments) within a context */
+	int var_metric; /*!< For organizing variables (particularly includes and switch statements) within a context */
 };
 
 /*! \brief Table configuration
@@ -1032,7 +1030,7 @@ static struct ast_variable *realtime_lda
 
 /*! \brief See Asterisk doc
  *
- * this function will be called for the switch statment if no match is found with the realtime_ldap function(i.e. it is a failover);
+ * this function will be called for the switch statement if no match is found with the realtime_ldap function(i.e. it is a failover);
  * however, the ast_load_realtime wil match on wildcharacters also depending on what the mode is set to
  * this is an area of asterisk that could do with a lot of modification
  * I think this function returns Realtime dynamic objects
@@ -1987,25 +1985,8 @@ static char *realtime_ldap_status(struct
 		ast_str_append(&buf, 0, " with username %s", user);
 	}
 
-	if (ctimesec > 31536000) {
-		ast_cli(a->fd, "%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n",
-				ast_str_buffer(buf), ctimesec / 31536000,
-				(ctimesec % 31536000) / 86400, (ctimesec % 86400) / 3600,
-				(ctimesec % 3600) / 60, ctimesec % 60);
-	} else if (ctimesec > 86400) {
-		ast_cli(a->fd, "%s for %d days, %d hours, %d minutes, %d seconds.\n",
-				ast_str_buffer(buf), ctimesec / 86400, (ctimesec % 86400) / 3600,
-				(ctimesec % 3600) / 60, ctimesec % 60);
-	} else if (ctimesec > 3600) {
-		ast_cli(a->fd, "%s for %d hours, %d minutes, %d seconds.\n",
-				ast_str_buffer(buf), ctimesec / 3600, (ctimesec % 3600) / 60,
-				ctimesec % 60);
-	} else if (ctimesec > 60) {
-		ast_cli(a->fd, "%s for %d minutes, %d seconds.\n", ast_str_buffer(buf),
-					ctimesec / 60, ctimesec % 60);
-	} else {
-		ast_cli(a->fd, "%s for %d seconds.\n", ast_str_buffer(buf), ctimesec);
-	}
+	ast_str_append(&buf, 0, " for ");
+	ast_cli_print_timestr_fromseconds(a->fd, ctimesec, ast_str_buffer(buf));
 	ast_free(buf);
 
 	return CLI_SUCCESS;
@@ -2020,4 +2001,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_config_odbc.c 1:16.1.1~dfsg-1/res/res_config_odbc.c
--- 1:13.23.1~dfsg-2/res/res_config_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -118,6 +116,9 @@ static SQLHSTMT custom_prepare(struct od
 
 	res = SQLPrepare(stmt, (unsigned char *)cps->sql, SQL_NTS);
 	if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
+		if (res == SQL_ERROR) {
+			ast_odbc_print_errors(SQL_HANDLE_STMT, stmt, "SQL Prepare");
+		}
 		ast_log(LOG_WARNING, "SQL Prepare failed! [%s]\n", cps->sql);
 		SQLFreeHandle (SQL_HANDLE_STMT, stmt);
 		return NULL;
@@ -633,6 +634,9 @@ static SQLHSTMT update2_prepare(struct o
 
 	res = SQLPrepare(stmt, (unsigned char *)ast_str_buffer(sql), SQL_NTS);
 	if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
+		if (res == SQL_ERROR) {
+			ast_odbc_print_errors(SQL_HANDLE_STMT, stmt, "SQL Prepare");
+		}
 		ast_log(LOG_WARNING, "SQL Prepare failed! [%s]\n", ast_str_buffer(sql));
 		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
 		return NULL;
@@ -1235,9 +1239,10 @@ static int reload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Realtime ODBC configuration",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig,res_odbc",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_config_pgsql.c 1:16.1.1~dfsg-1/res/res_config_pgsql.c
--- 1:13.23.1~dfsg-2/res/res_config_pgsql.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_pgsql.c	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
 /*
  * Asterisk -- An open source telephony toolkit.
  *
- * Copyright (C) 1999-2010, Digium, Inc.
+ * Copyright (C) 1999 - 2017, Digium, Inc.
  *
  * Manuel Guesdon <mguesdon@oxymium.net> - PostgreSQL RealTime Driver Author/Adaptor
  * Mark Spencer <markster@digium.com>  - Asterisk Author
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <libpq-fe.h>			/* PostgreSQL */
 
 #include "asterisk/file.h"
@@ -279,63 +277,30 @@ static struct tables *find_table(const c
 
 	/* Not found, scan the table */
 	if (has_schema_support) {
-		char *schemaname, *tablename;
+		char *schemaname, *tablename, *tmp_schemaname, *tmp_tablename;
 		if (strchr(orig_tablename, '.')) {
-			schemaname = ast_strdupa(orig_tablename);
-			tablename = strchr(schemaname, '.');
-			*tablename++ = '\0';
+			tmp_schemaname = ast_strdupa(orig_tablename);
+			tmp_tablename = strchr(tmp_schemaname, '.');
+			*tmp_tablename++ = '\0';
 		} else {
-			schemaname = "";
-			tablename = ast_strdupa(orig_tablename);
+			tmp_schemaname = "";
+			tmp_tablename = ast_strdupa(orig_tablename);
 		}
 
-		/* Escape special characters in schemaname */
-		if (strchr(schemaname, '\\') || strchr(schemaname, '\'')) {
-			char *tmp = schemaname, *ptr;
-
-			ptr = schemaname = ast_alloca(strlen(tmp) * 2 + 1);
-			for (; *tmp; tmp++) {
-				if (strchr("\\'", *tmp)) {
-					*ptr++ = *tmp;
-				}
-				*ptr++ = *tmp;
-			}
-			*ptr = '\0';
-		}
-		/* Escape special characters in tablename */
-		if (strchr(tablename, '\\') || strchr(tablename, '\'')) {
-			char *tmp = tablename, *ptr;
-
-			ptr = tablename = ast_alloca(strlen(tmp) * 2 + 1);
-			for (; *tmp; tmp++) {
-				if (strchr("\\'", *tmp)) {
-					*ptr++ = *tmp;
-				}
-				*ptr++ = *tmp;
-			}
-			*ptr = '\0';
-		}
+		tablename = ast_alloca(strlen(tmp_tablename) * 2 + 1);
+		PQescapeStringConn(pgsqlConn, tablename, tmp_tablename, strlen(tmp_tablename), NULL);
+		schemaname = ast_alloca(strlen(tmp_schemaname) * 2 + 1);
+		PQescapeStringConn(pgsqlConn, schemaname, tmp_schemaname, strlen(tmp_schemaname), NULL);
 
 		ast_str_set(&sql, 0, "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM (((pg_catalog.pg_class c INNER JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace AND c.relname = '%s' AND n.nspname = %s%s%s) INNER JOIN pg_catalog.pg_attribute a ON (NOT a.attisdropped) AND a.attnum > 0 AND a.attrelid = c.oid) INNER JOIN pg_catalog.pg_type t ON t.oid = a.atttypid) LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum ORDER BY n.nspname, c.relname, attnum",
 			tablename,
 			ast_strlen_zero(schemaname) ? "" : "'", ast_strlen_zero(schemaname) ? "current_schema()" : schemaname, ast_strlen_zero(schemaname) ? "" : "'");
 	} else {
-		/* Escape special characters in tablename */
-		if (strchr(orig_tablename, '\\') || strchr(orig_tablename, '\'')) {
-			const char *tmp = orig_tablename;
-			char *ptr;
-
-			orig_tablename = ptr = ast_alloca(strlen(tmp) * 2 + 1);
-			for (; *tmp; tmp++) {
-				if (strchr("\\'", *tmp)) {
-					*ptr++ = *tmp;
-				}
-				*ptr++ = *tmp;
-			}
-			*ptr = '\0';
-		}
+		char *tablename;
+		tablename = ast_alloca(strlen(orig_tablename) * 2 + 1);
+		PQescapeStringConn(pgsqlConn, tablename, orig_tablename, strlen(orig_tablename), NULL);
 
-		ast_str_set(&sql, 0, "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM pg_class c, pg_type t, pg_attribute a LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum WHERE c.oid = a.attrelid AND a.atttypid = t.oid AND (a.attnum > 0) AND c.relname = '%s' ORDER BY c.relname, attnum", orig_tablename);
+		ast_str_set(&sql, 0, "SELECT a.attname, t.typname, a.attlen, a.attnotnull, d.adsrc, a.atttypmod FROM pg_class c, pg_type t, pg_attribute a LEFT OUTER JOIN pg_attrdef d ON a.atthasdef AND d.adrelid = a.attrelid AND d.adnum = a.attnum WHERE c.oid = a.attrelid AND a.atttypid = t.oid AND (a.attnum > 0) AND c.relname = '%s' ORDER BY c.relname, attnum", tablename);
 	}
 
 	ast_mutex_lock(&pgsql_lock);
@@ -1071,7 +1036,7 @@ static int store_pgsql(const char *datab
 	numrows = atoi(PQcmdTuples(result));
 	ast_mutex_unlock(&pgsql_lock);
 
-	ast_debug(1, "PostgreSQL RealTime: row inserted on table: %s.", table);
+	ast_debug(1, "PostgreSQL RealTime: row inserted on table: %s.\n", table);
 
 	/* From http://dev.pgsql.com/doc/pgsql/en/pgsql-affected-rows.html
 	 * An integer greater than zero indicates the number of rows affected
@@ -1476,6 +1441,9 @@ static int parse_config(int is_reload)
 
 	config = ast_config_load(RES_CONFIG_PGSQL_CONF, config_flags);
 	if (config == CONFIG_STATUS_FILEUNCHANGED) {
+		if (is_reload && pgsqlConn && PQstatus(pgsqlConn) != CONNECTION_OK) {
+			ast_log(LOG_WARNING,  "PostgreSQL RealTime: Not connected\n");
+		}
 		return 0;
 	}
 
@@ -1702,7 +1670,9 @@ static char *handle_cli_realtime_pgsql_c
 
 static char *handle_cli_realtime_pgsql_status(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
-	char status[256], credentials[100] = "";
+	char connection_info[256];
+	char credentials[100] = "";
+	char buf[376]; /* 256+100+"Connected to "+" for "+NULL */
 	int is_connected = 0, ctimesec = time(NULL) - connect_time;
 
 	switch (cmd) {
@@ -1719,49 +1689,36 @@ static char *handle_cli_realtime_pgsql_s
 	if (a->argc != 4)
 		return CLI_SHOWUSAGE;
 
+	if (!ast_strlen_zero(dbhost))
+		snprintf(connection_info, sizeof(connection_info), "%s@%s, port %d", dbname, dbhost, dbport);
+	else if (!ast_strlen_zero(dbsock))
+		snprintf(connection_info, sizeof(connection_info), "%s on socket file %s", dbname, dbsock);
+	else
+		snprintf(connection_info, sizeof(connection_info), "%s@%s", dbname, dbhost);
+
+	if (!ast_strlen_zero(dbuser))
+		snprintf(credentials, sizeof(credentials), " with username %s", dbuser);
+
 	ast_mutex_lock(&pgsql_lock);
 	is_connected = (pgsqlConn && PQstatus(pgsqlConn) == CONNECTION_OK);
 	ast_mutex_unlock(&pgsql_lock);
 
 	if (is_connected) {
-		if (!ast_strlen_zero(dbhost))
-			snprintf(status, sizeof(status), "Connected to %s@%s, port %d", dbname, dbhost, dbport);
-		else if (!ast_strlen_zero(dbsock))
-			snprintf(status, sizeof(status), "Connected to %s on socket file %s", dbname, dbsock);
-		else
-			snprintf(status, sizeof(status), "Connected to %s@%s", dbname, dbhost);
-
-		if (!ast_strlen_zero(dbuser))
-			snprintf(credentials, sizeof(credentials), " with username %s", dbuser);
-
-		if (ctimesec > 31536000)
-			ast_cli(a->fd, "%s%s for %d years, %d days, %d hours, %d minutes, %d seconds.\n",
-					status, credentials, ctimesec / 31536000, (ctimesec % 31536000) / 86400,
-					(ctimesec % 86400) / 3600, (ctimesec % 3600) / 60, ctimesec % 60);
-		else if (ctimesec > 86400)
-			ast_cli(a->fd, "%s%s for %d days, %d hours, %d minutes, %d seconds.\n", status,
-					credentials, ctimesec / 86400, (ctimesec % 86400) / 3600, (ctimesec % 3600) / 60,
-					ctimesec % 60);
-		else if (ctimesec > 3600)
-			ast_cli(a->fd, "%s%s for %d hours, %d minutes, %d seconds.\n", status, credentials,
-					ctimesec / 3600, (ctimesec % 3600) / 60, ctimesec % 60);
-		else if (ctimesec > 60)
-			ast_cli(a->fd, "%s%s for %d minutes, %d seconds.\n", status, credentials, ctimesec / 60,
-					ctimesec % 60);
-		else
-			ast_cli(a->fd, "%s%s for %d seconds.\n", status, credentials, ctimesec);
-
+		snprintf(buf, sizeof(buf), "Connected to %s%s for ", connection_info, credentials);
+		ast_cli_print_timestr_fromseconds(a->fd, ctimesec, buf);
 		return CLI_SUCCESS;
 	} else {
+		ast_cli(a->fd, "Unable to connect %s%s\n", connection_info, credentials);
 		return CLI_FAILURE;
 	}
 }
 
 /* needs usecount semantics defined */
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PostgreSQL RealTime Configuration Driver",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_config_sqlite3.c 1:16.1.1~dfsg-1/res/res_config_sqlite3.c
--- 1:13.23.1~dfsg-2/res/res_config_sqlite3.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_sqlite3.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,8 +45,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sqlite3.h>
 
 #include "asterisk/module.h"
@@ -105,6 +103,7 @@ struct realtime_sqlite3_db {
 	unsigned int exiting:1;
 	unsigned int wakeup:1;
 	unsigned int batch;
+	int busy_timeout;
 };
 
 struct ao2_container *databases;
@@ -344,7 +343,7 @@ static int db_open(struct realtime_sqlit
 		ao2_unlock(db);
 		return -1;
 	}
-	sqlite3_busy_timeout(db->handle, 1000);
+	sqlite3_busy_timeout(db->handle, db->busy_timeout);
 
 	if (db->debug) {
 		sqlite3_trace(db->handle, trace_cb, db);
@@ -402,6 +401,7 @@ static struct realtime_sqlite3_db *new_r
 	db->requirements = REALTIME_SQLITE3_REQ_WARN;
 	db->batch = 100;
 	ast_string_field_set(db, name, cat);
+	db->busy_timeout = 1000;
 
 	for (var = ast_variable_browse(config, cat); var; var = var->next) {
 		if (!strcasecmp(var->name, "dbfile")) {
@@ -412,6 +412,10 @@ static struct realtime_sqlite3_db *new_r
 			ast_app_parse_timelen(var->value, (int *) &db->batch, TIMELEN_MILLISECONDS);
 		} else if (!strcasecmp(var->name, "debug")) {
 			db->debug = ast_true(var->value);
+		} else if (!strcasecmp(var->name, "busy_timeout")) {
+			if (ast_parse_arg(var->value, PARSE_INT32|PARSE_DEFAULT, &(db->busy_timeout), 1000) != 0) {
+				ast_log(LOG_WARNING, "Invalid busy_timeout value '%s' at res_config_sqlite3.conf:%d. Using 1000 instead.\n", var->value, var->lineno);
+			}
 		}
 	}
 
@@ -456,6 +460,11 @@ static int update_realtime_sqlite3_db(st
 		db_open(db); /* Also handles setting appropriate debug on new handle */
 	}
 
+	if (db->busy_timeout != new->busy_timeout) {
+		db->busy_timeout = new->busy_timeout;
+		sqlite3_busy_timeout(db->handle, db->busy_timeout);
+	}
+
 	if (db->batch != new->batch) {
 		if (db->batch == 0) {
 			db->batch = new->batch;
@@ -1185,7 +1194,9 @@ static int realtime_sqlite3_require(cons
 		return -1;
 	}
 
-	if (!(columns = ao2_container_alloc(31, str_hash_fn, str_cmp_fn))) {
+	columns = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 31,
+		str_hash_fn, NULL, str_cmp_fn);
+	if (!columns) {
 		unref_db(&db);
 	   return -1;
 	}
@@ -1360,7 +1371,9 @@ static int load_module(void)
 {
 	discover_sqlite3_caps();
 
-	if (!((databases = ao2_container_alloc(DB_BUCKETS, db_hash_fn, db_cmp_fn)))) {
+	databases = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, DB_BUCKETS,
+		db_hash_fn, NULL, db_cmp_fn);
+	if (!databases) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -1384,4 +1397,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_config_sqlite.c 1:16.1.1~dfsg-1/res/res_config_sqlite.c
--- 1:13.23.1~dfsg-2/res/res_config_sqlite.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_config_sqlite.c	2018-12-26 18:45:13.000000000 +0000
@@ -82,11 +82,10 @@
 
 /*** MODULEINFO
 	<depend>sqlite</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include <sqlite.h>
 
@@ -1773,9 +1772,16 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
+/*
+ * This module should require "cdr" to enforce startup/shutdown ordering but it
+ * loads at REALTIME_DRIVER priority which would cause "cdr" to load too early.
+ *
+ * ast_cdr_register / ast_cdr_unregister is safe for use while "cdr" is not running.
+ */
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Realtime SQLite configuration",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+	.requires = "extconfig",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_convert.c 1:16.1.1~dfsg-1/res/res_convert.c
--- 1:13.23.1~dfsg-2/res/res_convert.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_convert.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/cli.h"
@@ -88,9 +86,6 @@ static char *handle_cli_file_convert(str
 		return NULL;
 	}
 
-	/* ugly, can be removed when CLI entries have ast_module pointers */
-	ast_module_ref(ast_module_info->self);
-
 	if (a->argc != 4 || ast_strlen_zero(a->argv[2]) || ast_strlen_zero(a->argv[3])) {
 		ret = CLI_SHOWUSAGE;
 		goto fail_out;
@@ -142,8 +137,6 @@ fail_out:
 	if (fs_in)
 		ast_closestream(fs_in);
 
-	ast_module_unref(ast_module_info->self);
-
 	return ret;
 }
 
diff -pruN 1:13.23.1~dfsg-2/res/res_corosync.c 1:16.1.1~dfsg-1/res/res_corosync.c
--- 1:13.23.1~dfsg-2/res/res_corosync.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_corosync.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include <corosync/cpg.h>
 #include <corosync/cfg.h>
 
@@ -1127,7 +1125,8 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	nodes = ao2_container_alloc(23, corosync_node_hash_fn, corosync_node_cmp_fn);
+	nodes = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 23,
+		corosync_node_hash_fn, NULL, corosync_node_cmp_fn);
 	if (!nodes) {
 		goto failed;
 	}
diff -pruN 1:13.23.1~dfsg-2/res/res_crypto.c 1:16.1.1~dfsg-1/res/res_crypto.c
--- 1:13.23.1~dfsg-2/res/res_crypto.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_crypto.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <dirent.h>                 /* for closedir, opendir, readdir, DIR */
 
 #include <openssl/aes.h>            /* for AES_decrypt, AES_encrypt, AES_set... */
@@ -660,8 +658,6 @@ static int load_module(void)
 		crypto_load(-1, -1);
 	}
 
-	/* This prevents dlclose from ever running, but allows CLI cleanup at shutdown. */
-	ast_module_shutdown_ref(ast_module_info->self);
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -672,11 +668,10 @@ static int unload_module(void)
 	return 0;
 }
 
-/* needs usecount semantics defined */
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Cryptographic Digital Signatures",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND, /*!< Since we don't have a config file, we could move up to REALTIME_DEPEND, if necessary */
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND, /*!< Since we don't have a config file, we could move up to REALTIME_DEPEND, if necessary */
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_curl.c 1:16.1.1~dfsg-1/res/res_curl.c
--- 1:13.23.1~dfsg-2/res/res_curl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_curl.c	2018-12-26 18:45:13.000000000 +0000
@@ -42,38 +42,15 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <curl/curl.h>
 
 #include "asterisk/module.h"
 
-static const char *dependents[] = {
-	"func_curl.so",
-	"res_config_curl.so",
-};
-
 static int unload_module(void)
 {
-	int res = 0;
-	size_t i;
-
-	/* If the dependent modules are still in memory, forbid unload */
-	for (i = 0; i < ARRAY_LEN(dependents); i++) {
-		if (ast_module_check(dependents[i])) {
-			if (!ast_shutting_down()) {
-				ast_log(LOG_WARNING, "%s (dependent module) is still loaded.  Cannot unload res_curl.so\n", dependents[i]);
-			}
-			res = -1;
-		}
-	}
-
-	if (res)
-		return -1;
-
 	curl_global_cleanup();
 
-	return res;
+	return 0;
 }
 
 static int load_module(void)
@@ -89,8 +66,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "cURL Resource Module",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_REALTIME_DEPEND,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_REALTIME_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_endpoint_stats.c 1:16.1.1~dfsg-1/res/res_endpoint_stats.c
--- 1:13.23.1~dfsg-2/res/res_endpoint_stats.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_endpoint_stats.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/stasis_endpoints.h"
 #include "asterisk/stasis_message_router.h"
@@ -153,5 +151,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_statsd"
+	.requires = "res_statsd"
 	);
diff -pruN 1:13.23.1~dfsg-2/res/res_fax.c 1:16.1.1~dfsg-1/res/res_fax.c
--- 1:13.23.1~dfsg-2/res/res_fax.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_fax.c	2018-12-26 18:45:13.000000000 +0000
@@ -65,8 +65,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/io.h"
 #include "asterisk/file.h"
 #include "asterisk/logger.h"
@@ -980,7 +978,6 @@ int ast_fax_tech_register(struct ast_fax
 	AST_RWLIST_WRLOCK(&faxmodules);
 	AST_RWLIST_INSERT_TAIL(&faxmodules, fax, list);
 	AST_RWLIST_UNLOCK(&faxmodules);
-	ast_module_ref(ast_module_info->self);
 
 	ast_verb(3, "Registered handler for '%s' (%s)\n", fax->tech->type, fax->tech->description);
 
@@ -1000,7 +997,6 @@ void ast_fax_tech_unregister(struct ast_
 			continue;
 		}
 		AST_RWLIST_REMOVE_CURRENT(list);
-		ast_module_unref(ast_module_info->self);
 		ast_free(fax);
 		ast_verb(4, "Unregistered FAX module type '%s'\n", tech->type);
 		break;
@@ -1163,8 +1159,10 @@ static struct ast_fax_session *fax_sessi
 		if ((faxmod->tech->caps & details->caps) != details->caps) {
 			continue;
 		}
+		if (!ast_module_running_ref(faxmod->tech->module)) {
+			continue;
+		}
 		ast_debug(4, "Reserving a FAX session from '%s'.\n", faxmod->tech->description);
-		ast_module_ref(faxmod->tech->module);
 		s->tech = faxmod->tech;
 		break;
 	}
@@ -1281,8 +1279,10 @@ static struct ast_fax_session *fax_sessi
 			if ((faxmod->tech->caps & details->caps) != details->caps) {
 				continue;
 			}
+			if (!ast_module_running_ref(faxmod->tech->module)) {
+				continue;
+			}
 			ast_debug(4, "Requesting a new FAX session from '%s'.\n", faxmod->tech->description);
-			ast_module_ref(faxmod->tech->module);
 			if (reserved) {
 				/* Balance module ref from reserved session */
 				ast_module_unref(reserved->tech->module);
@@ -2914,6 +2914,11 @@ static int fax_gateway_start(struct fax_
 	struct ast_fax_session *s;
 	int start_res;
 
+	/* if the fax gateway is already started then do nothing */
+	if (gateway->s && gateway->s->state != AST_FAX_STATE_RESERVED) {
+		return 0;
+	}
+
 	/* create the FAX session */
 	if (!(s = fax_session_new(details, chan, gateway->s, gateway->token))) {
 		gateway->token = NULL;
@@ -4719,7 +4724,9 @@ static int load_module(void)
 	/* initialize the registry */
 	faxregistry.active_sessions = 0;
 	faxregistry.reserved_sessions = 0;
-	if (!(faxregistry.container = ao2_container_alloc(FAX_MAXBUCKETS, session_hash_cb, session_cmp_cb))) {
+	faxregistry.container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		FAX_MAXBUCKETS, session_hash_cb, NULL, session_cmp_cb);
+	if (!faxregistry.container) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -4784,9 +4791,9 @@ static int reload_module(void)
 
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Generic FAX Applications",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_fax_spandsp.c 1:16.1.1~dfsg-1/res/res_fax_spandsp.c
--- 1:13.23.1~dfsg-2/res/res_fax_spandsp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_fax_spandsp.c	2018-12-26 18:45:13.000000000 +0000
@@ -48,14 +48,10 @@
 	<support_level>extended</support_level>
 ***/
 
+/* Needed for spandsp headers */
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
-#define SPANDSP_EXPOSE_INTERNAL_STRUCTURES
-#include <spandsp.h>
-#include <spandsp/version.h>
-
 #include "asterisk/logger.h"
 #include "asterisk/module.h"
 #include "asterisk/strings.h"
@@ -67,6 +63,10 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/channel.h"
 #include "asterisk/format_cache.h"
 
+#define SPANDSP_EXPOSE_INTERNAL_STRUCTURES
+#include <spandsp.h>
+#include <spandsp/version.h>
+
 #define SPANDSP_FAX_SAMPLES 160
 #define SPANDSP_FAX_TIMER_RATE 8000 / SPANDSP_FAX_SAMPLES	/* 50 ticks per second, 20ms, 160 samples per second */
 #define SPANDSP_ENGAGE_UDPTL_NAT_RETRY 3
@@ -1263,7 +1263,8 @@ static int load_module(void)
 
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Spandsp G.711 and T.38 FAX Technologies",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.enhances = "res_fax",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_celt.c 1:16.1.1~dfsg-1/res/res_format_attr_celt.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_celt.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_celt.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_g729.c 1:16.1.1~dfsg-1/res/res_format_attr_g729.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_g729.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_g729.c	2018-12-26 18:45:13.000000000 +0000
@@ -22,8 +22,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_h263.c 1:16.1.1~dfsg-1/res/res_format_attr_h263.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_h263.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_h263.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_h264.c 1:16.1.1~dfsg-1/res/res_format_attr_h264.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_h264.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_h264.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_ilbc.c 1:16.1.1~dfsg-1/res/res_format_attr_ilbc.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_ilbc.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_ilbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,172 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Alexander Traud
+ *
+ * Alexander Traud <pabstraud@compuserve.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief iLBC format attribute interface
+ *
+ * \author Alexander Traud <pabstraud@compuserve.com>
+ *
+ * \note http://tools.ietf.org/html/rfc3952
+ */
+
+/*** MODULEINFO
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/module.h"
+#include "asterisk/format.h"
+#include "asterisk/strings.h"           /* for ast_str_append */
+#include "asterisk/utils.h"             /* for ast_calloc, ast_free */
+
+#include "asterisk/ilbc.h"
+
+static struct ilbc_attr default_ilbc_attr = {
+	.mode = 20,
+};
+
+static void ilbc_destroy(struct ast_format *format)
+{
+	struct ilbc_attr *attr = ast_format_get_attribute_data(format);
+
+	ast_free(attr);
+}
+
+static int ilbc_clone(const struct ast_format *src, struct ast_format *dst)
+{
+	struct ilbc_attr *original = ast_format_get_attribute_data(src);
+	struct ilbc_attr *attr = ast_malloc(sizeof(*attr));
+
+	if (!attr) {
+		return -1;
+	}
+
+	if (original) {
+		*attr = *original;
+	} else {
+		*attr = default_ilbc_attr;
+	}
+
+	ast_format_set_attribute_data(dst, attr);
+
+	return 0;
+}
+
+static struct ast_format *ilbc_parse_sdp_fmtp(const struct ast_format *format, const char *attributes)
+{
+	struct ast_format *cloned;
+	struct ilbc_attr *attr;
+	const char *kvp;
+	unsigned int val;
+
+	cloned = ast_format_clone(format);
+	if (!cloned) {
+		return NULL;
+	}
+	attr = ast_format_get_attribute_data(cloned);
+
+	if ((kvp = strstr(attributes, "mode")) && sscanf(kvp, "mode=%30u", &val) == 1) {
+		attr->mode = val;
+	} else {
+		attr->mode = 30; /* optional attribute; 30 is default value */
+	}
+
+	return cloned;
+}
+
+static void ilbc_generate_sdp_fmtp(const struct ast_format *format, unsigned int payload, struct ast_str **str)
+{
+	struct ilbc_attr *attr = ast_format_get_attribute_data(format);
+
+	if (!attr) {
+		attr = &default_ilbc_attr;
+	}
+
+	/* When the VoIP/SIP client Zoiper calls Asterisk and its
+	 * iLBC 20 is disabled but iLBC 30 enabled, Zoiper still
+	 * falls back to iLBC 20, when there is no mode=30 in the
+	 * answer. Consequently, Zoiper defaults to iLBC 20. To
+	 * make that client happy, Asterisk sends mode always.
+	 * tested in June 2016, Zoiper Premium 1.13.2 for iPhone
+	 */
+	/* if (attr->mode != 30) */ {
+		ast_str_append(str, 0, "a=fmtp:%u mode=%u\r\n", payload, attr->mode);
+	}
+}
+
+static struct ast_format *ilbc_getjoint(const struct ast_format *format1, const struct ast_format *format2)
+{
+	struct ast_format *jointformat;
+	struct ilbc_attr *attr1 = ast_format_get_attribute_data(format1);
+	struct ilbc_attr *attr2 = ast_format_get_attribute_data(format2);
+	struct ilbc_attr *attr_res;
+
+	if (!attr1) {
+		attr1 = &default_ilbc_attr;
+	}
+
+	if (!attr2) {
+		attr2 = &default_ilbc_attr;
+	}
+
+	jointformat = ast_format_clone(format1);
+	if (!jointformat) {
+		return NULL;
+	}
+	attr_res = ast_format_get_attribute_data(jointformat);
+
+	if (attr1->mode != attr2->mode) {
+		attr_res->mode = 30;
+	}
+
+	return jointformat;
+}
+
+static struct ast_format_interface ilbc_interface = {
+	.format_destroy = ilbc_destroy,
+	.format_clone = ilbc_clone,
+	.format_cmp = NULL,
+	.format_get_joint = ilbc_getjoint,
+	.format_attribute_set = NULL,
+	.format_parse_sdp_fmtp = ilbc_parse_sdp_fmtp,
+	.format_generate_sdp_fmtp = ilbc_generate_sdp_fmtp,
+};
+
+static int load_module(void)
+{
+	if (ast_format_interface_register("ilbc", &ilbc_interface)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	return 0;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER,
+	"iLBC Format Attribute Module",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_opus.c 1:16.1.1~dfsg-1/res/res_format_attr_opus.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_opus.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_opus.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 #include "asterisk/logger.h"
@@ -96,6 +94,7 @@ static int opus_clone(const struct ast_f
 	ao2_bump(attr->data);
 
 	ast_format_set_attribute_data(dst, attr);
+	ast_format_set_channel_count(dst, ast_format_get_channel_count(src));
 
 	return 0;
 }
@@ -155,6 +154,9 @@ static struct ast_format *opus_parse_sdp
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_PTIME, &attr->ptime);
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_MAX_AVERAGE_BITRATE, &attr->maxbitrate);
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_STEREO, &attr->stereo);
+	if (attr->stereo) {
+		ast_format_set_channel_count(cloned, 2);
+	}
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_SPROP_STEREO, &attr->spropstereo);
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_CBR, &attr->cbr);
 	sdp_fmtp_get(attributes, CODEC_OPUS_ATTR_FEC, &attr->fec);
@@ -248,6 +250,10 @@ static struct ast_format *opus_getjoint(
 	if (!jointformat) {
 		return NULL;
 	}
+
+	if (ast_format_get_channel_count(format1) == 2 || ast_format_get_channel_count(format2) == 2) {
+		ast_format_set_channel_count(jointformat, 2);
+	}
 	attr_res = ast_format_get_attribute_data(jointformat);
 
 	attr_res->dtx = attr1->dtx || attr2->dtx ? 1 : 0;
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_silk.c 1:16.1.1~dfsg-1/res/res_format_attr_silk.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_silk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_silk.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_siren14.c 1:16.1.1~dfsg-1/res/res_format_attr_siren14.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_siren14.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_siren14.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_siren7.c 1:16.1.1~dfsg-1/res/res_format_attr_siren7.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_siren7.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_siren7.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 
diff -pruN 1:13.23.1~dfsg-2/res/res_format_attr_vp8.c 1:16.1.1~dfsg-1/res/res_format_attr_vp8.c
--- 1:13.23.1~dfsg-2/res/res_format_attr_vp8.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_format_attr_vp8.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 #include "asterisk/logger.h"            /* for ast_log, LOG_WARNING */
diff -pruN 1:13.23.1~dfsg-2/res/res_hep.c 1:16.1.1~dfsg-1/res/res_hep.c
--- 1:13.23.1~dfsg-2/res/res_hep.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_hep.c	2018-12-26 18:45:13.000000000 +0000
@@ -85,8 +85,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/config_options.h"
@@ -369,27 +367,6 @@ static void hepv3_data_dtor(void *obj)
 	}
 }
 
-/*! \brief Pulls first resolved address and returns it */
-static int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
-				      const char* name, int flag, int family)
-{
-	struct ast_sockaddr *addrs;
-	int addrs_cnt;
-
-	addrs_cnt = ast_sockaddr_resolve(&addrs, name, flag, family);
-	if (addrs_cnt <= 0) {
-		return 1;
-	}
-	if (addrs_cnt > 1) {
-		ast_debug(1, "Multiple addresses resolving %s, using the first one only\n", name);
-	}
-
-	ast_sockaddr_copy(addr, &addrs[0]);
-
-	ast_free(addrs);
-	return 0;
-}
-
 /*! \brief Allocate the HEPv3 run-time data */
 static struct hepv3_runtime_data *hepv3_data_alloc(struct hepv3_global_config *config)
 {
@@ -723,4 +700,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_hep_pjsip.c 1:16.1.1~dfsg-1/res/res_hep_pjsip.c
--- 1:13.23.1~dfsg-2/res/res_hep_pjsip.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_hep_pjsip.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <pjsip_ua.h>
 #include <pjlib.h>
@@ -234,10 +232,8 @@ static pjsip_module logging_module = {
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
-	if (!ast_module_check("res_hep.so") || !hepv3_is_loaded()) {
-		ast_log(AST_LOG_WARNING, "res_hep is not loaded or running; declining module load\n");
+	if (!hepv3_is_loaded()) {
+		ast_log(AST_LOG_WARNING, "res_hep is disabled; declining module load\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -252,7 +248,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "PJSIP HEPv3 Logger",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_hep,res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_hep_rtcp.c 1:16.1.1~dfsg-1/res/res_hep_rtcp.c
--- 1:13.23.1~dfsg-2/res/res_hep_rtcp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_hep_rtcp.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/res_hep.h"
 #include "asterisk/module.h"
 #include "asterisk/netsock2.h"
@@ -159,8 +157,8 @@ static void rtp_topic_handler(void *data
 
 static int load_module(void)
 {
-	if (!ast_module_check("res_hep.so") || !hepv3_is_loaded()) {
-		ast_log(AST_LOG_WARNING, "res_hep is not loaded or running; declining module load\n");
+	if (!hepv3_is_loaded()) {
+		ast_log(AST_LOG_WARNING, "res_hep is disabled; declining module load\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -169,6 +167,9 @@ static int load_module(void)
 	if (!stasis_rtp_subscription) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
+	stasis_subscription_accept_message_type(stasis_rtp_subscription, ast_rtp_rtcp_sent_type());
+	stasis_subscription_accept_message_type(stasis_rtp_subscription, ast_rtp_rtcp_received_type());
+	stasis_subscription_set_filter(stasis_rtp_subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -186,4 +187,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
-	);
+	.requires = "res_hep",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_http_media_cache.c 1:16.1.1~dfsg-1/res/res_http_media_cache.c
--- 1:13.23.1~dfsg-2/res/res_http_media_cache.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_http_media_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,446 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Matt Jordan
+ *
+ * Matt Jordan <mjordan@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief
+ *
+ * \author \verbatim Matt Jordan <mjordan@digium.com> \endverbatim
+ *
+ * HTTP backend for the core media cache
+ */
+
+/*** MODULEINFO
+	<depend>curl</depend>
+	<depend>res_curl</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <curl/curl.h>
+
+#include "asterisk/module.h"
+#include "asterisk/bucket.h"
+#include "asterisk/sorcery.h"
+#include "asterisk/threadstorage.h"
+
+#define GLOBAL_USERAGENT "asterisk-libcurl-agent/1.0"
+
+#define MAX_HEADER_LENGTH 1023
+
+/*! \brief Data passed to cURL callbacks */
+struct curl_bucket_file_data {
+	/*! The \c ast_bucket_file object that caused the operation */
+	struct ast_bucket_file *bucket_file;
+	/*! File to write data to */
+	FILE *out_file;
+};
+
+/*!
+ * \internal \brief The cURL header callback function
+ */
+static size_t curl_header_callback(char *buffer, size_t size, size_t nitems, void *data)
+{
+	struct curl_bucket_file_data *cb_data = data;
+	size_t realsize;
+	char *value;
+	char *header;
+
+	realsize = size * nitems;
+
+	if (realsize > MAX_HEADER_LENGTH) {
+		ast_log(LOG_WARNING, "cURL header length of '%zu' is too large: max %d\n",
+			realsize, MAX_HEADER_LENGTH);
+		return 0;
+	}
+
+	/* buffer may not be NULL terminated */
+	header = ast_alloca(realsize + 1);
+	memcpy(header, buffer, realsize);
+	header[realsize] = '\0';
+	value = strchr(header, ':');
+	if (!value) {
+		/* Not a header we care about; bail */
+		return realsize;
+	}
+	*value++ = '\0';
+
+	if (strcasecmp(header, "ETag")
+		&& strcasecmp(header, "Cache-Control")
+		&& strcasecmp(header, "Last-Modified")
+		&& strcasecmp(header, "Content-Type")
+		&& strcasecmp(header, "Expires")) {
+		return realsize;
+	}
+
+	value = ast_trim_blanks(ast_skip_blanks(value));
+	header = ast_str_to_lower(header);
+
+	ast_bucket_file_metadata_set(cb_data->bucket_file, header, value);
+
+	return realsize;
+}
+
+/*!
+ * \internal \brief The cURL body callback function
+ */
+static size_t curl_body_callback(void *ptr, size_t size, size_t nitems, void *data)
+{
+	struct curl_bucket_file_data *cb_data = data;
+	size_t realsize;
+
+	realsize = fwrite(ptr, size, nitems, cb_data->out_file);
+
+	return realsize;
+}
+
+/*!
+ * \internal \brief Set the expiration metadata on the bucket file based on HTTP caching rules
+ */
+static void bucket_file_set_expiration(struct ast_bucket_file *bucket_file)
+{
+	struct ast_bucket_metadata *metadata;
+	char time_buf[32];
+	struct timeval actual_expires = ast_tvnow();
+
+	metadata = ast_bucket_file_metadata_get(bucket_file, "cache-control");
+	if (metadata) {
+		char *str_max_age;
+
+		str_max_age = strstr(metadata->value, "s-maxage");
+		if (!str_max_age) {
+			str_max_age = strstr(metadata->value, "max-age");
+		}
+
+		if (str_max_age) {
+			unsigned int max_age;
+			char *equal = strchr(str_max_age, '=');
+			if (equal && (sscanf(equal + 1, "%30u", &max_age) == 1)) {
+				actual_expires.tv_sec += max_age;
+			}
+		}
+		ao2_ref(metadata, -1);
+	} else {
+		metadata = ast_bucket_file_metadata_get(bucket_file, "expires");
+		if (metadata) {
+			struct tm expires_time;
+
+			strptime(metadata->value, "%a, %d %b %Y %T %z", &expires_time);
+			expires_time.tm_isdst = -1;
+			actual_expires.tv_sec = mktime(&expires_time);
+
+			ao2_ref(metadata, -1);
+		}
+	}
+
+	/* Use 'now' if we didn't get an expiration time */
+	snprintf(time_buf, sizeof(time_buf), "%30lu", actual_expires.tv_sec);
+
+	ast_bucket_file_metadata_set(bucket_file, "__actual_expires", time_buf);
+}
+
+/*! \internal
+ * \brief Return whether or not we should always revalidate against the server
+ */
+static int bucket_file_always_revalidate(struct ast_bucket_file *bucket_file)
+{
+	RAII_VAR(struct ast_bucket_metadata *, metadata,
+		ast_bucket_file_metadata_get(bucket_file, "cache-control"),
+		ao2_cleanup);
+
+	if (!metadata) {
+		return 0;
+	}
+
+	if (strstr(metadata->value, "no-cache")
+		|| strstr(metadata->value, "must-revalidate")) {
+		return 1;
+	}
+
+	return 0;
+}
+
+/*! \internal
+ * \brief Return whether or not the item has expired
+ */
+static int bucket_file_expired(struct ast_bucket_file *bucket_file)
+{
+	RAII_VAR(struct ast_bucket_metadata *, metadata,
+		ast_bucket_file_metadata_get(bucket_file, "__actual_expires"),
+		ao2_cleanup);
+	struct timeval current_time = ast_tvnow();
+	struct timeval expires = { .tv_sec = 0, .tv_usec = 0 };
+
+	if (!metadata) {
+		return 1;
+	}
+
+	if (sscanf(metadata->value, "%lu", &expires.tv_sec) != 1) {
+		return 1;
+	}
+
+	return ast_tvcmp(current_time, expires) == -1 ? 0 : 1;
+}
+
+/*!
+ * \internal \brief Obtain a CURL handle with common setup options
+ */
+static CURL *get_curl_instance(struct curl_bucket_file_data *cb_data)
+{
+	CURL *curl;
+
+	curl = curl_easy_init();
+	if (!curl) {
+		return NULL;
+	}
+
+	curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
+	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);
+	curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, curl_header_callback);
+	curl_easy_setopt(curl, CURLOPT_USERAGENT, GLOBAL_USERAGENT);
+	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
+	curl_easy_setopt(curl, CURLOPT_URL, ast_sorcery_object_get_id(cb_data->bucket_file));
+	curl_easy_setopt(curl, CURLOPT_HEADERDATA, cb_data);
+
+	return curl;
+}
+
+/*!
+ * \brief Execute the CURL
+ */
+static long execute_curl_instance(CURL *curl)
+{
+	char curl_errbuf[CURL_ERROR_SIZE + 1];
+	long http_code;
+
+	curl_errbuf[CURL_ERROR_SIZE] = '\0';
+	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, curl_errbuf);
+
+	if (curl_easy_perform(curl)) {
+		ast_log(LOG_WARNING, "%s\n", curl_errbuf);
+		return -1;
+	}
+
+	curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_code);
+
+	curl_easy_cleanup(curl);
+
+	return http_code;
+}
+
+/*!
+ * \internal \brief CURL the URI specified by the bucket_file and store it in the provided path
+ */
+static int bucket_file_run_curl(struct ast_bucket_file *bucket_file)
+{
+	struct curl_bucket_file_data cb_data = {
+		.bucket_file = bucket_file,
+	};
+	long http_code;
+	CURL *curl;
+
+	cb_data.out_file = fopen(bucket_file->path, "wb");
+	if (!cb_data.out_file) {
+		ast_log(LOG_WARNING, "Failed to open file '%s' for writing: %s (%d)\n",
+			bucket_file->path, strerror(errno), errno);
+		return -1;
+	}
+
+	curl = get_curl_instance(&cb_data);
+	if (!curl) {
+		fclose(cb_data.out_file);
+		return -1;
+	}
+
+	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_body_callback);
+	curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void*)&cb_data);
+
+	http_code = execute_curl_instance(curl);
+
+	fclose(cb_data.out_file);
+
+	if (http_code / 100 == 2) {
+		bucket_file_set_expiration(bucket_file);
+		return 0;
+	} else {
+		ast_log(LOG_WARNING, "Failed to retrieve URL '%s': server returned %ld\n",
+			ast_sorcery_object_get_id(bucket_file), http_code);
+	}
+
+	return -1;
+}
+
+static int bucket_http_wizard_is_stale(const struct ast_sorcery *sorcery, void *data, void *object)
+{
+	struct ast_bucket_file *bucket_file = object;
+	struct ast_bucket_metadata *metadata;
+	struct curl_slist *header_list = NULL;
+	long http_code;
+	CURL *curl;
+	struct curl_bucket_file_data cb_data = {
+		.bucket_file = bucket_file
+	};
+	char etag_buf[256];
+
+	if (!bucket_file_expired(bucket_file) && !bucket_file_always_revalidate(bucket_file)) {
+		return 0;
+	}
+
+	/* See if we have an ETag for this item. If not, it's stale. */
+	metadata = ast_bucket_file_metadata_get(bucket_file, "etag");
+	if (!metadata) {
+		return 1;
+	}
+
+	curl = get_curl_instance(&cb_data);
+
+	/* Set the ETag header on our outgoing request */
+	snprintf(etag_buf, sizeof(etag_buf), "If-None-Match: %s", metadata->value);
+	header_list = curl_slist_append(header_list, etag_buf);
+	curl_easy_setopt(curl, CURLOPT_HTTPHEADER, header_list);
+	curl_easy_setopt(curl, CURLOPT_NOBODY, 1L);
+	ao2_ref(metadata, -1);
+
+	http_code = execute_curl_instance(curl);
+
+	curl_slist_free_all(header_list);
+
+	if (http_code == 304) {
+		bucket_file_set_expiration(bucket_file);
+		return 0;
+	}
+
+	return 1;
+}
+
+static int bucket_http_wizard_create(const struct ast_sorcery *sorcery, void *data,
+	void *object)
+{
+	struct ast_bucket_file *bucket_file = object;
+
+	return bucket_file_run_curl(bucket_file);
+}
+
+static void *bucket_http_wizard_retrieve_id(const struct ast_sorcery *sorcery,
+	void *data, const char *type, const char *id)
+{
+	struct ast_bucket_file *bucket_file;
+
+	if (strcmp(type, "file")) {
+		ast_log(LOG_WARNING, "Failed to create storage: invalid bucket type '%s'\n", type);
+		return NULL;
+	}
+
+	if (ast_strlen_zero(id)) {
+		ast_log(LOG_WARNING, "Failed to create storage: no URI\n");
+		return NULL;
+	}
+
+	bucket_file = ast_bucket_file_alloc(id);
+	if (!bucket_file) {
+		ast_log(LOG_WARNING, "Failed to create storage for '%s'\n", id);
+		return NULL;
+	}
+
+	if (ast_bucket_file_temporary_create(bucket_file)) {
+		ast_log(LOG_WARNING, "Failed to create temporary storage for '%s'\n", id);
+		ast_sorcery_delete(sorcery, bucket_file);
+		ao2_ref(bucket_file, -1);
+		return NULL;
+	}
+
+	if (bucket_file_run_curl(bucket_file)) {
+		ast_sorcery_delete(sorcery, bucket_file);
+		ao2_ref(bucket_file, -1);
+		return NULL;
+	}
+
+	return bucket_file;
+}
+
+static int bucket_http_wizard_delete(const struct ast_sorcery *sorcery, void *data,
+	void *object)
+{
+	struct ast_bucket_file *bucket_file = object;
+
+	unlink(bucket_file->path);
+
+	return 0;
+}
+
+static struct ast_sorcery_wizard http_bucket_wizard = {
+	.name = "http",
+	.create = bucket_http_wizard_create,
+	.retrieve_id = bucket_http_wizard_retrieve_id,
+	.delete = bucket_http_wizard_delete,
+	.is_stale = bucket_http_wizard_is_stale,
+};
+
+static struct ast_sorcery_wizard http_bucket_file_wizard = {
+	.name = "http",
+	.create = bucket_http_wizard_create,
+	.retrieve_id = bucket_http_wizard_retrieve_id,
+	.delete = bucket_http_wizard_delete,
+	.is_stale = bucket_http_wizard_is_stale,
+};
+
+static struct ast_sorcery_wizard https_bucket_wizard = {
+	.name = "https",
+	.create = bucket_http_wizard_create,
+	.retrieve_id = bucket_http_wizard_retrieve_id,
+	.delete = bucket_http_wizard_delete,
+	.is_stale = bucket_http_wizard_is_stale,
+};
+
+static struct ast_sorcery_wizard https_bucket_file_wizard = {
+	.name = "https",
+	.create = bucket_http_wizard_create,
+	.retrieve_id = bucket_http_wizard_retrieve_id,
+	.delete = bucket_http_wizard_delete,
+	.is_stale = bucket_http_wizard_is_stale,
+};
+
+static int unload_module(void)
+{
+	return 0;
+}
+
+static int load_module(void)
+{
+	if (ast_bucket_scheme_register("http", &http_bucket_wizard, &http_bucket_file_wizard,
+			NULL, NULL)) {
+		ast_log(LOG_ERROR, "Failed to register Bucket HTTP wizard scheme implementation\n");
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (ast_bucket_scheme_register("https", &https_bucket_wizard, &https_bucket_file_wizard,
+			NULL, NULL)) {
+		ast_log(LOG_ERROR, "Failed to register Bucket HTTPS wizard scheme implementation\n");
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "HTTP Media Cache Backend",
+		.support_level = AST_MODULE_SUPPORT_CORE,
+		.load = load_module,
+		.unload = unload_module,
+		.requires = "res_curl",
+	);
diff -pruN 1:13.23.1~dfsg-2/res/res_http_post.c 1:16.1.1~dfsg-1/res/res_http_post.c
--- 1:13.23.1~dfsg-2/res/res_http_post.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_http_post.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <gmime/gmime.h>
@@ -224,7 +222,7 @@ static int find_sequence(char * inbuf, i
 * This function has two modes.  The first to find a boundary marker.  The
 * second is to find the filename immediately after the boundary.
 */
-static int readmimefile(FILE *fin, FILE *fout, char *boundary, int contentlen)
+static int readmimefile(struct ast_iostream *in, FILE *fout, char *boundary, int contentlen)
 {
 	int find_filename = 0;
 	char buf[4096];
@@ -235,7 +233,7 @@ static int readmimefile(FILE *fin, FILE
 	int boundary_len;
 	char * path_end, * path_start, * filespec;
 
-	if (NULL == fin || NULL == fout || NULL == boundary || 0 >= contentlen) {
+	if (NULL == in || NULL == fout || NULL == boundary || 0 >= contentlen) {
 		return -1;
 	}
 
@@ -249,8 +247,8 @@ static int readmimefile(FILE *fin, FILE
 		}
 
 		if (0 < num_to_read) {
-			if (fread(&(buf[char_in_buf]), 1, num_to_read, fin) < num_to_read) {
-				ast_log(LOG_WARNING, "fread() failed: %s\n", strerror(errno));
+			if (ast_iostream_read(in, &(buf[char_in_buf]), num_to_read) < num_to_read) {
+				ast_log(LOG_WARNING, "read failed: %s\n", strerror(errno));
 				num_to_read = 0;
 			}
 			contentlen -= num_to_read;
@@ -391,7 +389,7 @@ static int http_post_callback(struct ast
 	 */
 	ast_http_body_read_status(ser, 0);
 
-	if (0 > readmimefile(ser->f, f, boundary_marker, content_len)) {
+	if (0 > readmimefile(ser->stream, f, boundary_marker, content_len)) {
 		ast_debug(1, "Cannot find boundary marker in POST request.\n");
 		fclose(f);
 		ast_http_error(ser, 400, "Bad Request", "Cannot find boundary marker in POST request.");
@@ -515,4 +513,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload,
+	.requires = "http",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_http_websocket.c 1:16.1.1~dfsg-1/res/res_http_websocket.c
--- 1:13.23.1~dfsg-2/res/res_http_websocket.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_http_websocket.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/http.h"
 #include "asterisk/astobj2.h"
@@ -38,6 +36,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/file.h"
 #include "asterisk/unaligned.h"
 #include "asterisk/uri.h"
+#include "asterisk/uuid.h"
 
 #define AST_API_MODULE
 #include "asterisk/http_websocket.h"
@@ -87,8 +86,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 /*! \brief Structure definition for session */
 struct ast_websocket {
-	FILE *f;                            /*!< Pointer to the file instance used for writing and reading */
-	int fd;                             /*!< File descriptor for the session, only used for polling */
+	struct ast_iostream *stream;        /*!< iostream of the connection */
 	struct ast_sockaddr remote_address; /*!< Address of the remote client */
 	struct ast_sockaddr local_address;  /*!< Our local address */
 	enum ast_websocket_opcode opcode;   /*!< Cached opcode for multi-frame messages */
@@ -100,6 +98,7 @@ struct ast_websocket {
 	unsigned int closing:1;             /*!< Bit to indicate that the session is in the process of being closed */
 	unsigned int close_sent:1;          /*!< Bit to indicate that the session close opcode has been sent and no further data will be sent */
 	struct websocket_client *client;    /*!< Client object when connected as a client websocket */
+	char session_id[AST_UUID_STR_LEN];  /*!< The identifier for the websocket session */
 };
 
 /*! \brief Hashing function for protocols */
@@ -132,29 +131,24 @@ struct ast_websocket_server {
 	struct ao2_container *protocols; /*!< Container for registered protocols */
 };
 
-static void websocket_server_internal_dtor(void *obj)
+static void websocket_server_dtor(void *obj)
 {
 	struct ast_websocket_server *server = obj;
 	ao2_cleanup(server->protocols);
 	server->protocols = NULL;
 }
 
-static void websocket_server_dtor(void *obj)
-{
-	websocket_server_internal_dtor(obj);
-	ast_module_unref(ast_module_info->self);
-}
-
-static struct ast_websocket_server *websocket_server_create_impl(void (*dtor)(void *))
+static struct ast_websocket_server *websocket_server_create_impl(void)
 {
 	RAII_VAR(struct ast_websocket_server *, server, NULL, ao2_cleanup);
 
-	server = ao2_alloc(sizeof(*server), dtor);
+	server = ao2_alloc(sizeof(*server), websocket_server_dtor);
 	if (!server) {
 		return NULL;
 	}
 
-	server->protocols = ao2_container_alloc(MAX_PROTOCOL_BUCKETS, protocol_hash_fn, protocol_cmp_fn);
+	server->protocols = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_PROTOCOL_BUCKETS, protocol_hash_fn, NULL, protocol_cmp_fn);
 	if (!server->protocols) {
 		return NULL;
 	}
@@ -165,13 +159,12 @@ static struct ast_websocket_server *webs
 
 static struct ast_websocket_server *websocket_server_internal_create(void)
 {
-	return websocket_server_create_impl(websocket_server_internal_dtor);
+	return websocket_server_create_impl();
 }
 
 struct ast_websocket_server *AST_OPTIONAL_API_NAME(ast_websocket_server_create)(void)
 {
-	ast_module_ref(ast_module_info->self);
-	return websocket_server_create_impl(websocket_server_dtor);
+	return websocket_server_create_impl();
 }
 
 /*! \brief Destructor function for sessions */
@@ -179,10 +172,11 @@ static void session_destroy_fn(void *obj
 {
 	struct ast_websocket *session = obj;
 
-	if (session->f) {
+	if (session->stream) {
 		ast_websocket_close(session, 0);
-		if (session->f) {
-			fclose(session->f);
+		if (session->stream) {
+			ast_iostream_close(session->stream);
+			session->stream = NULL;
 			ast_verb(2, "WebSocket connection %s '%s' closed\n", session->client ? "to" : "from",
 				ast_sockaddr_stringify(&session->remote_address));
 		}
@@ -309,20 +303,22 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 	session->close_sent = 1;
 
 	ao2_lock(session);
-	res = ast_careful_fwrite(session->f, session->fd, frame, 4, session->timeout);
+	ast_iostream_set_timeout_inactivity(session->stream, session->timeout);
+	res = ast_iostream_write(session->stream, frame, sizeof(frame));
+	ast_iostream_set_timeout_disable(session->stream);
 
 	/* If an error occurred when trying to close this connection explicitly terminate it now.
 	 * Doing so will cause the thread polling on it to wake up and terminate.
 	 */
-	if (res) {
-		fclose(session->f);
-		session->f = NULL;
+	if (res != sizeof(frame)) {
+		ast_iostream_close(session->stream);
+		session->stream = NULL;
 		ast_verb(2, "WebSocket connection %s '%s' forcefully closed due to fatal write error\n",
 			session->client ? "to" : "from", ast_sockaddr_stringify(&session->remote_address));
 	}
 
 	ao2_unlock(session);
-	return res;
+	return res == sizeof(frame);
 }
 
 static const char *opcode_map[] = {
@@ -390,7 +386,8 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 		return -1;
 	}
 
-	if (ast_careful_fwrite(session->f, session->fd, frame, frame_size, session->timeout)) {
+	ast_iostream_set_timeout_sequence(session->stream, ast_tvnow(), session->timeout);
+	if (ast_iostream_write(session->stream, frame, frame_size) != frame_size) {
 		ao2_unlock(session);
 		/* 1011 - server terminating connection due to not being able to fulfill the request */
 		ast_debug(1, "Closing WS with 1011 because we can't fulfill a write request\n");
@@ -398,7 +395,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 		return -1;
 	}
 
-	fflush(session->f);
+	ast_iostream_set_timeout_disable(session->stream);
 	ao2_unlock(session);
 
 	return 0;
@@ -426,7 +423,7 @@ void AST_OPTIONAL_API_NAME(ast_websocket
 
 int AST_OPTIONAL_API_NAME(ast_websocket_fd)(struct ast_websocket *session)
 {
-	return session->closing ? -1 : session->fd;
+	return session->closing ? -1 : ast_iostream_get_fd(session->stream);
 }
 
 struct ast_sockaddr * AST_OPTIONAL_API_NAME(ast_websocket_remote_address)(struct ast_websocket *session)
@@ -446,7 +443,9 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 
 int AST_OPTIONAL_API_NAME(ast_websocket_set_nonblock)(struct ast_websocket *session)
 {
-	return ast_fd_set_flags(session->fd, O_NONBLOCK);
+	ast_iostream_nonblock(session->stream);
+	ast_iostream_set_exclusive_input(session->stream, 0);
+	return 0;
 }
 
 int AST_OPTIONAL_API_NAME(ast_websocket_set_timeout)(struct ast_websocket *session, int timeout)
@@ -456,6 +455,12 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 	return 0;
 }
 
+const char * AST_OPTIONAL_API_NAME(ast_websocket_session_id)(struct ast_websocket *session)
+{
+	return session->session_id;
+}
+
+
 /* MAINTENANCE WARNING on ast_websocket_read()!
  *
  * We have to keep in mind during this function that the fact that session->fd seems ready
@@ -484,25 +489,31 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
  * Note during the header parsing stage we try to read in small chunks just what we need, this
  * is buffered data anyways, no expensive syscall required most of the time ...
  */
-static inline int ws_safe_read(struct ast_websocket *session, char *buf, int len, enum ast_websocket_opcode *opcode)
+static inline int ws_safe_read(struct ast_websocket *session, char *buf, size_t len, enum ast_websocket_opcode *opcode)
 {
-	size_t rlen;
+	ssize_t rlen;
 	int xlen = len;
 	char *rbuf = buf;
 	int sanity = 10;
 
+	ast_assert(len > 0);
+
+	if (!len) {
+		errno = EINVAL;
+		return -1;
+	}
+
 	ao2_lock(session);
-	if (!session->f) {
+	if (!session->stream) {
 		ao2_unlock(session);
 		errno = ECONNABORTED;
 		return -1;
 	}
 
 	for (;;) {
-		clearerr(session->f);
-		rlen = fread(rbuf, 1, xlen, session->f);
-		if (!rlen) {
-			if (feof(session->f)) {
+		rlen = ast_iostream_read(session->stream, rbuf, xlen);
+		if (rlen != xlen) {
+			if (rlen == 0) {
 				ast_log(LOG_WARNING, "Web socket closed abruptly\n");
 				*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
 				session->closing = 1;
@@ -510,7 +521,7 @@ static inline int ws_safe_read(struct as
 				return -1;
 			}
 
-			if (ferror(session->f) && errno != EAGAIN) {
+			if (rlen < 0 && errno != EAGAIN) {
 				ast_log(LOG_ERROR, "Error reading from web socket: %s\n", strerror(errno));
 				*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
 				session->closing = 1;
@@ -526,12 +537,14 @@ static inline int ws_safe_read(struct as
 				return -1;
 			}
 		}
-		xlen = xlen - rlen;
-		rbuf = rbuf + rlen;
-		if (!xlen) {
-			break;
+		if (rlen > 0) {
+			xlen = xlen - rlen;
+			rbuf = rbuf + rlen;
+			if (!xlen) {
+				break;
+			}
 		}
-		if (ast_wait_for_input(session->fd, 1000) < 0) {
+		if (ast_wait_for_input(ast_iostream_get_fd(session->stream), 1000) < 0) {
 			ast_log(LOG_ERROR, "ast_wait_for_input returned err: %s\n", strerror(errno));
 			*opcode = AST_WEBSOCKET_OPCODE_CLOSE;
 			session->closing = 1;
@@ -603,9 +616,12 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 			return -1;
 		}
 
-		if (ws_safe_read(session, *payload, *payload_len, opcode)) {
-			return -1;
+		if (*payload_len) {
+			if (ws_safe_read(session, *payload, *payload_len, opcode)) {
+				return -1;
+			}
 		}
+
 		/* If a mask is present unmask the payload */
 		if (mask_present) {
 			unsigned int pos;
@@ -831,17 +847,26 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 			return 0;
 		}
 
-		if (!(session = ao2_alloc(sizeof(*session), session_destroy_fn))) {
+		if (!(session = ao2_alloc(sizeof(*session) + AST_UUID_STR_LEN + 1, session_destroy_fn))) {
 			ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted\n",
 				ast_sockaddr_stringify(&ser->remote_address));
 			websocket_bad_request(ser);
 			ao2_ref(protocol_handler, -1);
 			return 0;
 		}
-		session->timeout =  AST_DEFAULT_WEBSOCKET_WRITE_TIMEOUT;
+		session->timeout = AST_DEFAULT_WEBSOCKET_WRITE_TIMEOUT;
+
+		/* Generate the session id */
+		if (!ast_uuid_generate_str(session->session_id, sizeof(session->session_id))) {
+			ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - failed to generate a session id\n",
+				ast_sockaddr_stringify(&ser->remote_address));
+			ast_http_error(ser, 500, "Internal Server Error", "Allocation failed");
+			ao2_ref(protocol_handler, -1);
+			return 0;
+		}
 
 		if (protocol_handler->session_attempted
-		    && protocol_handler->session_attempted(ser, get_vars, headers)) {
+		    && protocol_handler->session_attempted(ser, get_vars, headers, session->session_id)) {
 			ast_debug(3, "WebSocket connection from '%s' rejected by protocol handler '%s'\n",
 				ast_sockaddr_stringify(&ser->remote_address), protocol_handler->name);
 			websocket_bad_request(ser);
@@ -859,7 +884,8 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 		 *    Connection_.
 		 */
 		if (protocol) {
-			fprintf(ser->f, "HTTP/1.1 101 Switching Protocols\r\n"
+			ast_iostream_printf(ser->stream,
+				"HTTP/1.1 101 Switching Protocols\r\n"
 				"Upgrade: %s\r\n"
 				"Connection: Upgrade\r\n"
 				"Sec-WebSocket-Accept: %s\r\n"
@@ -868,15 +894,14 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 				websocket_combine_key(key, base64, sizeof(base64)),
 				protocol);
 		} else {
-			fprintf(ser->f, "HTTP/1.1 101 Switching Protocols\r\n"
+			ast_iostream_printf(ser->stream,
+				"HTTP/1.1 101 Switching Protocols\r\n"
 				"Upgrade: %s\r\n"
 				"Connection: Upgrade\r\n"
 				"Sec-WebSocket-Accept: %s\r\n\r\n",
 				upgrade,
 				websocket_combine_key(key, base64, sizeof(base64)));
 		}
-
-		fflush(ser->f);
 	} else {
 
 		/* Specification defined in http://tools.ietf.org/html/draft-hixie-thewebsocketprotocol-75 or completely unknown */
@@ -888,7 +913,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 	}
 
 	/* Enable keepalive on all sessions so the underlying user does not have to */
-	if (setsockopt(ser->fd, SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
+	if (setsockopt(ast_iostream_get_fd(ser->stream), SOL_SOCKET, SO_KEEPALIVE, &flags, sizeof(flags))) {
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - failed to enable keepalive\n",
 			ast_sockaddr_stringify(&ser->remote_address));
 		websocket_bad_request(ser);
@@ -898,7 +923,7 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 	}
 
 	/* Get our local address for the connected socket */
-	if (ast_getsockname(ser->fd, &session->local_address)) {
+	if (ast_getsockname(ast_iostream_get_fd(ser->stream), &session->local_address)) {
 		ast_log(LOG_WARNING, "WebSocket connection from '%s' could not be accepted - failed to get local address\n",
 			ast_sockaddr_stringify(&ser->remote_address));
 		websocket_bad_request(ser);
@@ -910,25 +935,23 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 	ast_verb(2, "WebSocket connection from '%s' for protocol '%s' accepted using version '%d'\n", ast_sockaddr_stringify(&ser->remote_address), protocol ? : "", version);
 
 	/* Populate the session with all the needed details */
-	session->f = ser->f;
-	session->fd = ser->fd;
+	session->stream = ser->stream;
 	ast_sockaddr_copy(&session->remote_address, &ser->remote_address);
 	session->opcode = -1;
 	session->reconstruct = DEFAULT_RECONSTRUCTION_CEILING;
-	session->secure = ser->ssl ? 1 : 0;
+	session->secure = ast_iostream_get_ssl(ser->stream) ? 1 : 0;
 
 	/* Give up ownership of the socket and pass it to the protocol handler */
-	ast_tcptls_stream_set_exclusive_input(ser->stream_cookie, 0);
+	ast_iostream_set_exclusive_input(session->stream, 0);
 	protocol_handler->session_established(session, get_vars, headers);
 	ao2_ref(protocol_handler, -1);
 
 	/*
-	 * By dropping the FILE* and fd from the session the connection
+	 * By dropping the stream from the session the connection
 	 * won't get closed when the HTTP server cleans up because we
 	 * passed the connection to the protocol handler.
 	 */
-	ser->f = NULL;
-	ser->fd = -1;
+	ser->stream = NULL;
 
 	return 0;
 }
@@ -990,11 +1013,7 @@ static int websocket_add_protocol_intern
 
 int AST_OPTIONAL_API_NAME(ast_websocket_add_protocol)(const char *name, ast_websocket_callback callback)
 {
-	int res = websocket_add_protocol_internal(name, callback);
-	if (res == 0) {
-		ast_module_ref(ast_module_info->self);
-	}
-	return res;
+	return websocket_add_protocol_internal(name, callback);
 }
 
 int AST_OPTIONAL_API_NAME(ast_websocket_add_protocol2)(struct ast_websocket_protocol *protocol)
@@ -1009,7 +1028,6 @@ int AST_OPTIONAL_API_NAME(ast_websocket_
 		return -1;
 	}
 
-	ast_module_ref(ast_module_info->self);
 	return 0;
 }
 
@@ -1024,11 +1042,7 @@ static int websocket_remove_protocol_int
 
 int AST_OPTIONAL_API_NAME(ast_websocket_remove_protocol)(const char *name, ast_websocket_callback callback)
 {
-	int res = websocket_remove_protocol_internal(name, callback);
-	if (res == 0) {
-		ast_module_unref(ast_module_info->self);
-	}
-	return res;
+	return websocket_remove_protocol_internal(name, callback);
 }
 
 /*! \brief Parse the given uri into a path and remote address.
@@ -1256,7 +1270,7 @@ static enum ast_websocket_result websock
 	int has_accept = 0;
 	int has_protocol = 0;
 
-	if (!fgets(buf, sizeof(buf), client->ser->f)) {
+	if (ast_iostream_gets(client->ser->stream, buf, sizeof(buf)) <= 0) {
 		ast_log(LOG_ERROR, "Unable to retrieve HTTP status line.");
 		return WS_BAD_STATUS;
 	}
@@ -1269,7 +1283,7 @@ static enum ast_websocket_result websock
 
 	/* Ignoring line folding - assuming header field values are contained
 	   within a single line */
-	while (fgets(buf, sizeof(buf), client->ser->f)) {
+	while (ast_iostream_gets(client->ser->stream, buf, sizeof(buf)) > 0) {
 		char *name, *value;
 		int parsed = ast_http_header_parse(buf, &name, &value);
 
@@ -1322,19 +1336,19 @@ static enum ast_websocket_result websock
 			client->protocols);
 	}
 
-	if (fprintf(client->ser->f,
-		    "GET /%s HTTP/1.1\r\n"
-		    "Sec-WebSocket-Version: %d\r\n"
-		    "Upgrade: websocket\r\n"
-		    "Connection: Upgrade\r\n"
-		    "Host: %s\r\n"
-		    "Sec-WebSocket-Key: %s\r\n"
-		    "%s\r\n",
-		    client->resource_name ? ast_str_buffer(client->resource_name) : "",
-		    client->version,
-		    client->host,
-		    client->key,
-		    protocols) < 0) {
+	if (ast_iostream_printf(client->ser->stream,
+			"GET /%s HTTP/1.1\r\n"
+			"Sec-WebSocket-Version: %d\r\n"
+			"Upgrade: websocket\r\n"
+			"Connection: Upgrade\r\n"
+			"Host: %s\r\n"
+			"Sec-WebSocket-Key: %s\r\n"
+			"%s\r\n",
+			client->resource_name ? ast_str_buffer(client->resource_name) : "",
+			client->version,
+			client->host,
+			client->key,
+			protocols) < 0) {
 		ast_log(LOG_ERROR, "Failed to send handshake.\n");
 		return WS_WRITE_ERROR;
 	}
@@ -1358,9 +1372,9 @@ static enum ast_websocket_result websock
 		return res;
 	}
 
-	ws->f = ws->client->ser->f;
-	ws->fd = ws->client->ser->fd;
-	ws->secure = ws->client->ser->ssl ? 1 : 0;
+	ws->stream = ws->client->ser->stream;
+	ws->secure = ast_iostream_get_ssl(ws->stream) ? 1 : 0;
+	ws->client->ser->stream = NULL;
 	ast_sockaddr_copy(&ws->remote_address, &ws->client->ser->remote_address);
 	return WS_OK;
 }
@@ -1450,9 +1464,6 @@ static int load_module(void)
 	ast_http_uri_link(&websocketuri);
 	websocket_add_protocol_internal("echo", websocket_echo_callback);
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return 0;
 }
 
@@ -1467,8 +1478,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "HTTP WebSocket Support",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "http",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_limit.c 1:16.1.1~dfsg-1/res/res_limit.c
--- 1:13.23.1~dfsg-2/res/res_limit.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_limit.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,8 +24,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <sys/time.h>
 #include <sys/resource.h>
diff -pruN 1:13.23.1~dfsg-2/res/res_manager_devicestate.c 1:16.1.1~dfsg-1/res/res_manager_devicestate.c
--- 1:13.23.1~dfsg-2/res/res_manager_devicestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_manager_devicestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -143,8 +143,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Manager Device State Topic Forwarder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_manager_presencestate.c 1:16.1.1~dfsg-1/res/res_manager_presencestate.c
--- 1:13.23.1~dfsg-2/res/res_manager_presencestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_manager_presencestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -142,8 +142,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Manager Presence State Topic Forwarder",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_DEVSTATE_CONSUMER,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_monitor.c 1:16.1.1~dfsg-1/res/res_monitor.c
--- 1:13.23.1~dfsg-2/res/res_monitor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_monitor.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,14 +24,13 @@
  */
 
 /*** MODULEINFO
-	<depend type="module">func_periodic_hook</depend>
-	<support_level>core</support_level>
+	<use type="module">func_periodic_hook</use>
+	<support_level>deprecated</support_level>
+	<replacement>app_mixmonitor</replacement>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 #include <libgen.h>
 
@@ -987,9 +986,6 @@ static int load_module(void)
 	ast_manager_register_xml("PauseMonitor", EVENT_FLAG_CALL, pause_monitor_action);
 	ast_manager_register_xml("UnpauseMonitor", EVENT_FLAG_CALL, unpause_monitor_action);
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -1011,8 +1007,9 @@ static int unload_module(void)
 
 /* usecount semantics need to be defined */
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Call Monitoring Resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-		);
+	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.optional_modules = "func_periodic_hook",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_musiconhold.c 1:16.1.1~dfsg-1/res/res_musiconhold.c
--- 1:13.23.1~dfsg-2/res/res_musiconhold.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_musiconhold.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,12 +39,10 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <signal.h>
 #include <sys/time.h>
-#include <sys/signal.h>
+#include <signal.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -148,9 +146,12 @@ struct moh_files_state {
 #define MOH_CUSTOM		(1 << 2)
 #define MOH_RANDOMIZE		(1 << 3)
 #define MOH_SORTALPHA		(1 << 4)
+#define MOH_RANDSTART		(MOH_RANDOMIZE | MOH_SORTALPHA) /*!< Sorted but start at random position */
+#define MOH_SORTMODE		(3 << 3)
 
-#define MOH_CACHERTCLASSES      (1 << 5)        /*!< Should we use a separate instance of MOH for each user or not */
-#define MOH_ANNOUNCEMENT	(1 << 6)			/*!< Do we play announcement files between songs on this channel? */
+#define MOH_CACHERTCLASSES	(1 << 5)	/*!< Should we use a separate instance of MOH for each user or not */
+#define MOH_ANNOUNCEMENT	(1 << 6)	/*!< Do we play announcement files between songs on this channel? */
+#define MOH_PREFERCHANNELCLASS	(1 << 7)	/*!< Should queue moh override channel moh */
 
 /* Custom astobj2 flag */
 #define MOH_NOTDELETED          (1 << 30)       /*!< Find only records that aren't deleted? */
@@ -212,11 +213,12 @@ static struct ao2_container *mohclasses;
 #define MPG_123 "/usr/bin/mpg123"
 #define MAX_MP3S 256
 
+static void moh_parse_options(struct ast_variable *var, struct mohclass *mohclass);
 static int reload(void);
 
 #define mohclass_ref(class,string)   (ao2_t_ref((class), +1, (string)), class)
 
-#ifndef REF_DEBUG
+#ifndef AST_DEVMODE
 #define mohclass_unref(class,string) ({ ao2_t_ref((class), -1, (string)); (struct mohclass *) NULL; })
 #else
 #define mohclass_unref(class,string) _mohclass_unref(class, string, __FILE__,__LINE__,__PRETTY_FUNCTION__)
@@ -225,14 +227,14 @@ static struct mohclass *_mohclass_unref(
 	struct mohclass *dup = ao2_callback(mohclasses, OBJ_POINTER, ao2_match_by_addr, class);
 
 	if (dup) {
-		if (__ao2_ref_debug(dup, -1, (char *) tag, (char *) file, line, funcname) == 2) {
+		if (__ao2_ref(dup, -1, tag, file, line, funcname) == 2) {
 			ast_log(LOG_WARNING, "Attempt to unref mohclass %p (%s) when only 1 ref remained, and class is still in a container! (at %s:%d (%s))\n",
 				class, class->name, file, line, funcname);
 		} else {
 			ao2_ref(class, -1);
 		}
 	} else {
-		__ao2_ref_debug(class, -1, (char *) tag, (char *) file, line, funcname);
+		__ao2_ref(class, -1, tag, file, line, funcname);
 	}
 	return NULL;
 }
@@ -345,7 +347,7 @@ static int ast_moh_files_next(struct ast
 		/* If a specific file has been saved confirm it still exists and that it is still valid */
 		state->pos = state->save_pos;
 		state->save_pos = -1;
-	} else if (ast_test_flag(state->class, MOH_RANDOMIZE)) {
+	} else if (ast_test_flag(state->class, MOH_SORTMODE) == MOH_RANDOMIZE) {
 		/* Get a random file and ensure we can open it */
 		for (tries = 0; tries < 20; tries++) {
 			state->pos = ast_random() % state->class->total_files;
@@ -400,11 +402,28 @@ static int ast_moh_files_next(struct ast
 
 static struct ast_frame *moh_files_readframe(struct ast_channel *chan)
 {
-	struct ast_frame *f = NULL;
+	struct ast_frame *f;
 
-	if (!(ast_channel_stream(chan) && (f = ast_readframe(ast_channel_stream(chan))))) {
-		if (!ast_moh_files_next(chan))
+	f = ast_readframe(ast_channel_stream(chan));
+	if (!f) {
+		/* Either there was no file stream setup or we reached EOF. */
+		if (!ast_moh_files_next(chan)) {
+			/*
+			 * Either we resetup the previously saved file stream position
+			 * or we started a new file stream.
+			 */
 			f = ast_readframe(ast_channel_stream(chan));
+			if (!f) {
+				/*
+				 * We can get here if we were very unlucky because the
+				 * resetup file stream was saved at EOF when MOH was
+				 * previously stopped.
+				 */
+				if (!ast_moh_files_next(chan)) {
+					f = ast_readframe(ast_channel_stream(chan));
+				}
+			}
+		}
 	}
 
 	return f;
@@ -907,12 +926,8 @@ static struct mohclass *_get_mohbyname(c
 
 	ast_copy_string(tmp_class.name, name, sizeof(tmp_class.name));
 
-#ifdef REF_DEBUG
-	moh = __ao2_find_debug(mohclasses, &tmp_class, flags,
+	moh = __ao2_find(mohclasses, &tmp_class, flags,
 		"get_mohbyname", file, lineno, funcname);
-#else
-	moh = __ao2_find(mohclasses, &tmp_class, flags);
-#endif
 
 	if (!moh && warn) {
 		ast_log(LOG_WARNING, "Music on Hold class '%s' not found in memory. Verify your configuration.\n", name);
@@ -1062,6 +1077,45 @@ static struct ast_generator mohgen = {
 	.digit    = moh_handle_digit,
 };
 
+static void moh_parse_options(struct ast_variable *var, struct mohclass *mohclass)
+{
+	for (; var; var = var->next) {
+		if (!strcasecmp(var->name, "name")) {
+			ast_copy_string(mohclass->name, var->value, sizeof(mohclass->name));
+		} else if (!strcasecmp(var->name, "mode")) {
+			ast_copy_string(mohclass->mode, var->value, sizeof(mohclass->mode));
+		} else if (!strcasecmp(var->name, "directory")) {
+			ast_copy_string(mohclass->dir, var->value, sizeof(mohclass->dir));
+		} else if (!strcasecmp(var->name, "application")) {
+			ast_copy_string(mohclass->args, var->value, sizeof(mohclass->args));
+		} else if (!strcasecmp(var->name, "digit") && (isdigit(*var->value) || strchr("*#", *var->value))) {
+			mohclass->digit = *var->value;
+		} else if (!strcasecmp(var->name, "random")) {
+			static int deprecation_warning = 0;
+			if (!deprecation_warning) {
+				ast_log(LOG_WARNING, "Music on hold 'random' setting is deprecated in 14.  Please use 'sort=random' instead.\n");
+				deprecation_warning = 1;
+			}
+			ast_set2_flag(mohclass, ast_true(var->value), MOH_RANDOMIZE);
+		} else if (!strcasecmp(var->name, "sort")) {
+			if (!strcasecmp(var->value, "random")) {
+				ast_set_flag(mohclass, MOH_RANDOMIZE);
+			} else if (!strcasecmp(var->value, "alpha")) {
+				ast_set_flag(mohclass, MOH_SORTALPHA);
+			} else if (!strcasecmp(var->value, "randstart")) {
+				ast_set_flag(mohclass, MOH_RANDSTART);
+			}
+		} else if (!strcasecmp(var->name, "format")) {
+			ao2_cleanup(mohclass->format);
+			mohclass->format = ast_format_cache_get(var->value);
+			if (!mohclass->format) {
+				ast_log(LOG_WARNING, "Unknown format '%s' -- defaulting to SLIN\n", var->value);
+				mohclass->format = ao2_bump(ast_format_slin);
+			}
+		}
+	}
+}
+
 static int moh_add_file(struct mohclass *class, const char *filepath)
 {
 	if (!class->allowed_files) {
@@ -1345,17 +1399,9 @@ static struct mohclass *_moh_class_mallo
 {
 	struct mohclass *class;
 
-	if ((class =
-#ifdef REF_DEBUG
-			__ao2_alloc_debug(sizeof(*class), moh_class_destructor,
-				AO2_ALLOC_OPT_LOCK_MUTEX, "Allocating new moh class", file, line, funcname, 1)
-#elif defined(__AST_DEBUG_MALLOC)
-			__ao2_alloc_debug(sizeof(*class), moh_class_destructor,
-				AO2_ALLOC_OPT_LOCK_MUTEX, "Allocating new moh class", file, line, funcname, 0)
-#else
-			ao2_alloc(sizeof(*class), moh_class_destructor)
-#endif
-		)) {
+	class = __ao2_alloc(sizeof(*class), moh_class_destructor, AO2_ALLOC_OPT_LOCK_MUTEX,
+		"Allocating new moh class", file, line, funcname);
+	if (class) {
 		class->format = ao2_bump(ast_format_slin);
 		class->srcfd = -1;
 		class->kill_delay = 100000;
@@ -1382,43 +1428,43 @@ static int local_ast_moh_start(struct as
 	struct moh_files_state *state = ast_channel_music_state(chan);
 	struct ast_variable *var = NULL;
 	int res = 0;
+	int i;
 	int realtime_possible = ast_check_realtime("musiconhold");
 	int warn_if_not_in_memory = !realtime_possible;
+	const char *classes[] = {NULL, NULL, interpclass, "default"};
+
+	if (ast_test_flag(global_flags, MOH_PREFERCHANNELCLASS)) {
+		classes[0] = ast_channel_musicclass(chan);
+		classes[1] = mclass;
+	} else {
+		classes[0] = mclass;
+		classes[1] = ast_channel_musicclass(chan);
+	}
 
 	/* The following is the order of preference for which class to use:
 	 * 1) The channels explicitly set musicclass, which should *only* be
 	 *    set by a call to Set(CHANNEL(musicclass)=whatever) in the dialplan.
+	 *    Unless preferchannelclass in musiconhold.conf is false
 	 * 2) The mclass argument. If a channel is calling ast_moh_start() as the
 	 *    result of receiving a HOLD control frame, this should be the
 	 *    payload that came with the frame.
-	 * 3) The interpclass argument. This would be from the mohinterpret
+	 * 3) The channels explicitly set musicclass, which should *only* be
+	 *    set by a call to Set(CHANNEL(musicclass)=whatever) in the dialplan.
+	 * 4) The interpclass argument. This would be from the mohinterpret
 	 *    option from channel drivers. This is the same as the old musicclass
 	 *    option.
-	 * 4) The default class.
+	 * 5) The default class.
 	 */
-	if (!ast_strlen_zero(ast_channel_musicclass(chan))) {
-		mohclass = get_mohbyname(ast_channel_musicclass(chan), warn_if_not_in_memory, 0);
-		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(ast_channel_musicclass(chan));
-		}
-	}
-	if (!mohclass && !var && !ast_strlen_zero(mclass)) {
-		mohclass = get_mohbyname(mclass, warn_if_not_in_memory, 0);
-		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(mclass);
-		}
-	}
-	if (!mohclass && !var && !ast_strlen_zero(interpclass)) {
-		mohclass = get_mohbyname(interpclass, warn_if_not_in_memory, 0);
-		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold(interpclass);
-		}
-	}
 
-	if (!mohclass && !var) {
-		mohclass = get_mohbyname("default", warn_if_not_in_memory, 0);
-		if (!mohclass && realtime_possible) {
-			var = load_realtime_musiconhold("default");
+	for (i = 0; i < ARRAY_LEN(classes); ++i) {
+		if (!ast_strlen_zero(classes[i])) {
+			mohclass = get_mohbyname(classes[i], warn_if_not_in_memory, 0);
+			if (!mohclass && realtime_possible) {
+				var = load_realtime_musiconhold(classes[i]);
+			}
+			if (mohclass || var) {
+				break;
+			}
 		}
 	}
 
@@ -1426,37 +1472,12 @@ static int local_ast_moh_start(struct as
 	 * above guarantees that if var is non-NULL, then mohclass must be NULL.
 	 */
 	if (var) {
-		struct ast_variable *tmp = NULL;
-
 		if ((mohclass = moh_class_malloc())) {
 			mohclass->realtime = 1;
-			for (tmp = var; tmp; tmp = tmp->next) {
-				if (!strcasecmp(tmp->name, "name"))
-					ast_copy_string(mohclass->name, tmp->value, sizeof(mohclass->name));
-				else if (!strcasecmp(tmp->name, "mode"))
-					ast_copy_string(mohclass->mode, tmp->value, sizeof(mohclass->mode));
-				else if (!strcasecmp(tmp->name, "directory"))
-					ast_copy_string(mohclass->dir, tmp->value, sizeof(mohclass->dir));
-				else if (!strcasecmp(tmp->name, "application"))
-					ast_copy_string(mohclass->args, tmp->value, sizeof(mohclass->args));
-				else if (!strcasecmp(tmp->name, "digit") && (isdigit(*tmp->value) || strchr("*#", *tmp->value)))
-					mohclass->digit = *tmp->value;
-				else if (!strcasecmp(tmp->name, "random"))
-					ast_set2_flag(mohclass, ast_true(tmp->value), MOH_RANDOMIZE);
-				else if (!strcasecmp(tmp->name, "sort") && !strcasecmp(tmp->value, "random"))
-					ast_set_flag(mohclass, MOH_RANDOMIZE);
-				else if (!strcasecmp(tmp->name, "sort") && !strcasecmp(tmp->value, "alpha"))
-					ast_set_flag(mohclass, MOH_SORTALPHA);
-				else if (!strcasecmp(tmp->name, "format")) {
-					ao2_cleanup(mohclass->format);
-					mohclass->format = ast_format_cache_get(tmp->value);
-					if (!mohclass->format) {
-						ast_log(LOG_WARNING, "Unknown format '%s' -- defaulting to SLIN\n", tmp->value);
-						mohclass->format = ao2_bump(ast_format_slin);
-					}
-				}
-			}
+
+			moh_parse_options(var, mohclass);
 			ast_variables_destroy(var);
+
 			if (ast_strlen_zero(mohclass->dir)) {
 				if (!strcasecmp(mohclass->mode, "custom")) {
 					strcpy(mohclass->dir, "nodir");
@@ -1504,8 +1525,14 @@ static int local_ast_moh_start(struct as
 						mohclass = mohclass_unref(mohclass, "unreffing potential mohclass (moh_scan_files failed)");
 						return -1;
 					}
-					if (strchr(mohclass->args, 'r'))
+					if (strchr(mohclass->args, 'r')) {
+						static int deprecation_warning = 0;
+						if (!deprecation_warning) {
+							ast_log(LOG_WARNING, "Music on hold 'application=r' setting is deprecated in 14.  Please use 'sort=random' instead.\n");
+							deprecation_warning = 1;
+						}
 						ast_set_flag(mohclass, MOH_RANDOMIZE);
+					}
 				} else if (!strcasecmp(mohclass->mode, "mp3") || !strcasecmp(mohclass->mode, "mp3nb") || !strcasecmp(mohclass->mode, "quietmp3") || !strcasecmp(mohclass->mode, "quietmp3nb") || !strcasecmp(mohclass->mode, "httpmp3") || !strcasecmp(mohclass->mode, "custom")) {
 
 					if (!strcasecmp(mohclass->mode, "custom"))
@@ -1725,6 +1752,7 @@ static int load_moh_classes(int reload)
 	}
 
 	ast_clear_flag(global_flags, AST_FLAGS_ALL);
+	ast_set2_flag(global_flags, 1, MOH_PREFERCHANNELCLASS);
 
 	cat = ast_category_browse(cfg, NULL);
 	for (; cat; cat = ast_category_browse(cfg, cat)) {
@@ -1733,14 +1761,12 @@ static int load_moh_classes(int reload)
 			for (var = ast_variable_browse(cfg, cat); var; var = var->next) {
 				if (!strcasecmp(var->name, "cachertclasses")) {
 					ast_set2_flag(global_flags, ast_true(var->value), MOH_CACHERTCLASSES);
+				} else if (!strcasecmp(var->name, "preferchannelclass")) {
+					ast_set2_flag(global_flags, ast_true(var->value), MOH_PREFERCHANNELCLASS);
 				} else {
 					ast_log(LOG_WARNING, "Unknown option '%s' in [general] section of musiconhold.conf\n", var->name);
 				}
 			}
-		}
-		/* These names were deprecated in 1.4 and should not be used until after the next major release. */
-		if (!strcasecmp(cat, "classes") || !strcasecmp(cat, "moh_files") ||
-				!strcasecmp(cat, "general")) {
 			continue;
 		}
 
@@ -1748,6 +1774,9 @@ static int load_moh_classes(int reload)
 			break;
 		}
 
+		moh_parse_options(ast_variable_browse(cfg, cat), class);
+		/* For compatibility with the past, we overwrite any name=name
+		 * with the context [name]. */
 		ast_copy_string(class->name, cat, sizeof(class->name));
 		for (var = ast_variable_browse(cfg, cat); var; var = var->next) {
 			if (!strcasecmp(var->name, "mode")) {
@@ -1976,7 +2005,9 @@ static int load_module(void)
 {
 	int res;
 
-	if (!(mohclasses = ao2_t_container_alloc(53, moh_class_hash, moh_class_cmp, "Moh class container"))) {
+	mohclasses = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 53,
+		moh_class_hash, NULL, moh_class_cmp, "Moh class container");
+	if (!mohclasses) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/res/res_mutestream.c 1:16.1.1~dfsg-1/res/res_mutestream.c
--- 1:13.23.1~dfsg-2/res/res_mutestream.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_mutestream.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/options.h"
 #include "asterisk/logger.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_mwi_external_ami.c 1:16.1.1~dfsg-1/res/res_mwi_external_ami.c
--- 1:13.23.1~dfsg-2/res/res_mwi_external_ami.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_mwi_external_ami.c	2018-12-26 18:45:13.000000000 +0000
@@ -127,8 +127,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/res_mwi_external.h"
 #include "asterisk/manager.h"
@@ -344,8 +342,6 @@ static int unload_module(void)
 	ast_manager_unregister("MWIDelete");
 	ast_manager_unregister("MWIUpdate");
 
-	/* Must be done last */
-	ast_mwi_external_unref();
 	return 0;
 }
 
@@ -353,9 +349,6 @@ static int load_module(void)
 {
 	int res;
 
-	/* Must be done first */
-	ast_mwi_external_ref();
-
 	res = 0;
 	res |= ast_manager_register_xml("MWIGet", EVENT_FLAG_CALL | EVENT_FLAG_REPORTING, mwi_mailbox_get);
 	res |= ast_manager_register_xml("MWIDelete", EVENT_FLAG_CALL, mwi_mailbox_delete);
@@ -372,4 +365,6 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 5,
+	.requires = "res_mwi_external",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_mwi_external.c 1:16.1.1~dfsg-1/res/res_mwi_external.c
--- 1:13.23.1~dfsg-2/res/res_mwi_external.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_mwi_external.c	2018-12-26 18:45:13.000000000 +0000
@@ -54,8 +54,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/res_mwi_external.h"
@@ -83,16 +81,6 @@ struct ast_mwi_mailbox_object {
 
 static struct ast_sorcery *mwi_sorcery;
 
-void ast_mwi_external_ref(void)
-{
-	ast_module_ref(ast_module_info->self);
-}
-
-void ast_mwi_external_unref(void)
-{
-	ast_module_unref(ast_module_info->self);
-}
-
 /*!
  * \internal
  * \brief Post an update event to the MWI counts.
diff -pruN 1:13.23.1~dfsg-2/res/res_odbc.c 1:16.1.1~dfsg-1/res/res_odbc.c
--- 1:13.23.1~dfsg-2/res/res_odbc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_odbc.c	2018-12-26 18:45:13.000000000 +0000
@@ -47,8 +47,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/config.h"
@@ -62,7 +60,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/app.h"
 #include "asterisk/strings.h"
 #include "asterisk/threadstorage.h"
-#include "asterisk/data.h"
 
 struct odbc_class
 {
@@ -120,15 +117,6 @@ struct odbc_txn_frame {
 	char name[0];                   /*!< Name of this transaction ID */
 };
 
-#define DATA_EXPORT_ODBC_CLASS(MEMBER)				\
-	MEMBER(odbc_class, name, AST_DATA_STRING)		\
-	MEMBER(odbc_class, dsn, AST_DATA_STRING)		\
-	MEMBER(odbc_class, username, AST_DATA_STRING)		\
-	MEMBER(odbc_class, password, AST_DATA_PASSWORD)		\
-	MEMBER(odbc_class, forcecommit, AST_DATA_BOOLEAN)
-
-AST_DATA_STRUCTURE(odbc_class, DATA_EXPORT_ODBC_CLASS);
-
 const char *ast_odbc_isolation2text(int iso)
 {
 	if (iso == SQL_TXN_READ_COMMITTED) {
@@ -190,11 +178,6 @@ static void odbc_class_destructor(void *
 	ast_cond_destroy(&class->cond);
 }
 
-static int null_hash_fn(const void *obj, const int flags)
-{
-	return 0;
-}
-
 static void odbc_obj_destructor(void *data)
 {
 	struct odbc_obj *obj = data;
@@ -443,23 +426,20 @@ struct ast_str *ast_odbc_print_errors(SQ
 {
 	struct ast_str *errors = ast_str_thread_get(&errors_buf, 16);
 	SQLINTEGER nativeerror = 0;
-	SQLINTEGER numfields = 0;
 	SQLSMALLINT diagbytes = 0;
 	SQLSMALLINT i;
 	unsigned char state[10];
 	unsigned char diagnostic[256];
 
 	ast_str_reset(errors);
-	SQLGetDiagField(handle_type, handle, 1, SQL_DIAG_NUMBER, &numfields,
-			SQL_IS_INTEGER, &diagbytes);
-	for (i = 0; i < numfields; i++) {
-		SQLGetDiagRec(handle_type, handle, i + 1, state, &nativeerror,
-				diagnostic, sizeof(diagnostic), &diagbytes);
+	i = 0;
+	while (SQLGetDiagRec(handle_type, handle, ++i, state, &nativeerror,
+		diagnostic, sizeof(diagnostic), &diagbytes) == SQL_SUCCESS) {
 		ast_str_append(&errors, 0, "%s%s", ast_str_strlen(errors) ? "," : "", state);
 		ast_log(LOG_WARNING, "%s returned an error: %s: %s\n", operation, state, diagnostic);
 		/* XXX Why is this here? */
 		if (i > 10) {
-			ast_log(LOG_WARNING, "Oh, that was good.  There are really %d diagnostics?\n", (int)numfields);
+			ast_log(LOG_WARNING, "There are more than 10 diagnostic records! Ignore the rest.\n");
 			break;
 		}
 	}
@@ -668,10 +648,14 @@ static char *handle_cli_odbc_show(struct
 			char timestr[80];
 			struct ast_tm tm;
 
-			ast_localtime(&class->last_negative_connect, &tm, NULL);
-			ast_strftime(timestr, sizeof(timestr), "%Y-%m-%d %T", &tm);
 			ast_cli(a->fd, "  Name:   %s\n  DSN:    %s\n", class->name, class->dsn);
-			ast_cli(a->fd, "    Last connection attempt: %s\n", timestr);
+
+			if (class->last_negative_connect.tv_sec > 0) {
+				ast_localtime(&class->last_negative_connect, &tm, NULL);
+				ast_strftime(timestr, sizeof(timestr), "%Y-%m-%d %T", &tm);
+				ast_cli(a->fd, "    Last fail connection attempt: %s\n", timestr);
+			}
+
 			ast_cli(a->fd, "    Number of active connections: %zd (out of %d)\n", class->connection_cnt, class->maxconnections);
 			ast_cli(a->fd, "\n");
 		}
@@ -968,65 +952,6 @@ static odbc_status odbc_obj_connect(stru
 	return ODBC_SUCCESS;
 }
 
-/*!
- * \internal
- * \brief Implements the channels provider.
- */
-static int data_odbc_provider_handler(const struct ast_data_search *search,
-		struct ast_data *root)
-{
-	struct ao2_iterator aoi;
-	struct odbc_class *class;
-	struct ast_data *data_odbc_class, *data_odbc_connections;
-	struct ast_data *enum_node;
-
-	aoi = ao2_iterator_init(class_container, 0);
-	while ((class = ao2_iterator_next(&aoi))) {
-		data_odbc_class = ast_data_add_node(root, "class");
-		if (!data_odbc_class) {
-			ao2_ref(class, -1);
-			continue;
-		}
-
-		ast_data_add_structure(odbc_class, data_odbc_class, class);
-
-		data_odbc_connections = ast_data_add_node(data_odbc_class, "connections");
-		if (!data_odbc_connections) {
-			ao2_ref(class, -1);
-			continue;
-		}
-
-		/* isolation */
-		enum_node = ast_data_add_node(data_odbc_class, "isolation");
-		if (!enum_node) {
-			ao2_ref(class, -1);
-			continue;
-		}
-		ast_data_add_int(enum_node, "value", class->isolation);
-		ast_data_add_str(enum_node, "text", ast_odbc_isolation2text(class->isolation));
-		ao2_ref(class, -1);
-
-		if (!ast_data_search_match(search, data_odbc_class)) {
-			ast_data_remove_node(root, data_odbc_class);
-		}
-	}
-	ao2_iterator_destroy(&aoi);
-	return 0;
-}
-
-/*!
- * \internal
- * \brief /asterisk/res/odbc/listprovider.
- */
-static const struct ast_data_handler odbc_provider = {
-	.version = AST_DATA_HANDLER_VERSION,
-	.get = data_odbc_provider_handler
-};
-
-static const struct ast_data_entry odbc_providers[] = {
-	AST_DATA_ENTRY("/asterisk/res/odbc", &odbc_provider),
-};
-
 static int reload(void)
 {
 	struct odbc_cache_tables *table;
@@ -1063,25 +988,33 @@ static int reload(void)
 
 static int unload_module(void)
 {
-	/* Prohibit unloading */
-	return -1;
+	ao2_cleanup(class_container);
+	ast_cli_unregister_multiple(cli_odbc, ARRAY_LEN(cli_odbc));
+
+	return 0;
 }
 
 static int load_module(void)
 {
-	if (!(class_container = ao2_container_alloc(1, null_hash_fn, ao2_match_by_addr)))
+	class_container = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, ao2_match_by_addr);
+	if (!class_container) {
 		return AST_MODULE_LOAD_DECLINE;
-	if (load_odbc_config() == -1)
+	}
+
+	if (load_odbc_config() == -1) {
 		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	ast_module_shutdown_ref(ast_module_info->self);
 	ast_cli_register_multiple(cli_odbc, ARRAY_LEN(cli_odbc));
-	ast_data_register_multiple(odbc_providers, ARRAY_LEN(odbc_providers));
+
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "ODBC resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_REALTIME_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_REALTIME_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_odbc_transaction.c 1:16.1.1~dfsg-1/res/res_odbc_transaction.c
--- 1:13.23.1~dfsg-2/res/res_odbc_transaction.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_odbc_transaction.c	2018-12-26 18:45:13.000000000 +0000
@@ -522,8 +522,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "ODBC transaction resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_REALTIME_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_REALTIME_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_parking.c 1:16.1.1~dfsg-1/res/res_parking.c
--- 1:13.23.1~dfsg-2/res/res_parking.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_parking.c	2018-12-26 18:45:13.000000000 +0000
@@ -233,8 +233,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "parking/res_parking.h"
 #include "asterisk/config.h"
 #include "asterisk/config_options.h"
@@ -387,7 +385,9 @@ static void *parking_config_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->parking_lots = ao2_container_alloc(37, parking_lot_cfg_hash_fn, parking_lot_cfg_cmp_fn))) {
+	cfg->parking_lots = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 37,
+		parking_lot_cfg_hash_fn, NULL, parking_lot_cfg_cmp_fn);
+	if (!cfg->parking_lots) {
 		return NULL;
 	}
 
@@ -723,7 +723,7 @@ static int parking_add_extension(struct
 	}
 
 	if (ast_add_extension2_nolock(context, replace, extension, priority, NULL, NULL,
-			application, data_duplicate, ast_free_ptr, registrar)) {
+			application, data_duplicate, ast_free_ptr, registrar, NULL, 0)) {
 		ast_free(data_duplicate);
 		return -1;
 	}
@@ -1191,11 +1191,6 @@ static void link_configured_disable_mark
 	disable_marked_lots();
 }
 
-const struct ast_module_info *parking_get_module_info(void)
-{
-	return ast_module_info;
-}
-
 static int unload_module(void)
 {
 	unload_parking_bridge_features();
@@ -1300,5 +1295,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload_module,
-	.load_pri = 0,
+	.load_pri = AST_MODPRI_DEVSTATE_PROVIDER,
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_phoneprov.c 1:16.1.1~dfsg-1/res/res_phoneprov.c
--- 1:13.23.1~dfsg-2/res/res_phoneprov.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_phoneprov.c	2018-12-26 18:45:13.000000000 +0000
@@ -52,7 +52,6 @@
 #ifdef SOLARIS
 #include <sys/sockio.h>
 #endif
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/channel.h"
 #include "asterisk/file.h"
@@ -393,7 +392,7 @@ static void provider_destructor(void *ob
 static void delete_file(struct phoneprov_file *file)
 {
 	ast_string_field_free_memory(file);
-	free(file);
+	ast_free(file);
 }
 
 /*! \brief Read a TEXT file into a string and return the length */
@@ -417,7 +416,7 @@ static int load_file(const char *filenam
 
 	if (len != fread(*ret, sizeof(char), len, f)) {
 		fclose(f);
-		free(*ret);
+		ast_free(*ret);
 		*ret = NULL;
 		return -3;
 	}
@@ -950,7 +949,7 @@ static int phoneprov_callback(struct ast
 			socklen_t namelen = sizeof(name.sa);
 			int res;
 
-			if ((res = getsockname(ser->fd, &name.sa, &namelen))) {
+			if ((res = getsockname(ast_iostream_get_fd(ser->stream), &name.sa, &namelen))) {
 				ast_log(LOG_WARNING, "Could not get server IP, breakage likely.\n");
 			} else {
 				struct extension *exten_iter;
@@ -1238,7 +1237,7 @@ static struct varshead *get_defaults(voi
 		}
 	}
 	var = ast_var_assign(variable_lookup[AST_PHONEPROV_STD_SERVER_PORT], S_OR(value, "5060"));
-	if(cfg && cfg != CONFIG_STATUS_FILEINVALID) {
+	if (cfg && cfg != CONFIG_STATUS_FILEINVALID) {
 		ast_config_destroy(cfg);
 	}
 	AST_VAR_LIST_INSERT_TAIL(defaults, var);
@@ -1414,13 +1413,15 @@ static int unload_module(void)
  */
 static int load_module(void)
 {
-	profiles = ao2_container_alloc(MAX_PROFILE_BUCKETS, phone_profile_hash_fn, phone_profile_cmp_fn);
+	profiles = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_PROFILE_BUCKETS,
+		phone_profile_hash_fn, NULL, phone_profile_cmp_fn);
 	if (!profiles) {
 		ast_log(LOG_ERROR, "Unable to allocate profiles container.\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	http_routes = ao2_container_alloc(MAX_ROUTE_BUCKETS, http_route_hash_fn, http_route_cmp_fn);
+	http_routes = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_ROUTE_BUCKETS,
+		http_route_hash_fn, NULL, http_route_cmp_fn);
 	if (!http_routes) {
 		ast_log(LOG_ERROR, "Unable to allocate routes container.\n");
 		goto error;
@@ -1431,13 +1432,15 @@ static int load_module(void)
 		goto error;
 	}
 
-	users = ao2_container_alloc(MAX_USER_BUCKETS, user_hash_fn, user_cmp_fn);
+	users = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, MAX_USER_BUCKETS,
+		user_hash_fn, NULL, user_cmp_fn);
 	if (!users) {
 		ast_log(LOG_ERROR, "Unable to allocate users container.\n");
 		goto error;
 	}
 
-	providers = ao2_container_alloc(MAX_PROVIDER_BUCKETS, phoneprov_provider_hash_fn, phoneprov_provider_cmp_fn);
+	providers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		MAX_PROVIDER_BUCKETS, phoneprov_provider_hash_fn, NULL, phoneprov_provider_cmp_fn);
 	if (!providers) {
 		ast_log(LOG_ERROR, "Unable to allocate providers container.\n");
 		goto error;
@@ -1494,12 +1497,13 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "HTTP Phone Provisioning",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-	);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "http",
+);
 
 /****  Public API for register/unregister, set defaults, and add extension. ****/
 
diff -pruN 1:13.23.1~dfsg-2/res/res_pjproject.c 1:16.1.1~dfsg-1/res/res_pjproject.c
--- 1:13.23.1~dfsg-2/res/res_pjproject.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjproject.c	2018-12-26 18:45:13.000000000 +0000
@@ -98,8 +98,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <stdarg.h>
 #include <pjlib.h>
 #include <pjsip.h>
@@ -112,6 +110,8 @@ ASTERISK_REGISTER_FILE()
 #include "asterisk/res_pjproject.h"
 #include "asterisk/vector.h"
 #include "asterisk/sorcery.h"
+#include "asterisk/test.h"
+#include "asterisk/netsock2.h"
 
 static struct ast_sorcery *pjproject_sorcery;
 static pj_log_func *log_cb_orig;
@@ -282,16 +282,6 @@ void ast_pjproject_log_intercept_end(voi
 	ast_mutex_unlock(&pjproject_log_intercept_lock);
 }
 
-void ast_pjproject_ref(void)
-{
-	ast_module_ref(ast_module_info->self);
-}
-
-void ast_pjproject_unref(void)
-{
-	ast_module_unref(ast_module_info->self);
-}
-
 static char *handle_pjproject_show_buildopts(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	int i;
@@ -483,6 +473,184 @@ void ast_pjproject_caching_pool_destroy(
 	pj_caching_pool_destroy(cp);
 }
 
+int ast_sockaddr_to_pj_sockaddr(const struct ast_sockaddr *addr, pj_sockaddr *pjaddr)
+{
+	if (addr->ss.ss_family == AF_INET) {
+		struct sockaddr_in *sin = (struct sockaddr_in *) &addr->ss;
+		pjaddr->ipv4.sin_family = pj_AF_INET();
+#ifdef HAVE_PJPROJECT_BUNDLED
+		pjaddr->ipv4.sin_addr = sin->sin_addr;
+#else
+		pjaddr->ipv4.sin_addr.s_addr = sin->sin_addr.s_addr;
+#endif
+		pjaddr->ipv4.sin_port   = sin->sin_port;
+	} else if (addr->ss.ss_family == AF_INET6) {
+		struct sockaddr_in6 *sin = (struct sockaddr_in6 *) &addr->ss;
+		pjaddr->ipv6.sin6_family   = pj_AF_INET6();
+		pjaddr->ipv6.sin6_port     = sin->sin6_port;
+		pjaddr->ipv6.sin6_flowinfo = sin->sin6_flowinfo;
+		pjaddr->ipv6.sin6_scope_id = sin->sin6_scope_id;
+		memcpy(&pjaddr->ipv6.sin6_addr, &sin->sin6_addr, sizeof(pjaddr->ipv6.sin6_addr));
+	} else {
+		memset(pjaddr, 0, sizeof(*pjaddr));
+		return -1;
+	}
+	return 0;
+}
+
+int ast_sockaddr_from_pj_sockaddr(struct ast_sockaddr *addr, const pj_sockaddr *pjaddr)
+{
+	if (pjaddr->addr.sa_family == pj_AF_INET()) {
+		struct sockaddr_in *sin = (struct sockaddr_in *) &addr->ss;
+		sin->sin_family = AF_INET;
+#ifdef HAVE_PJPROJECT_BUNDLED
+		sin->sin_addr = pjaddr->ipv4.sin_addr;
+#else
+		sin->sin_addr.s_addr = pjaddr->ipv4.sin_addr.s_addr;
+#endif
+		sin->sin_port   = pjaddr->ipv4.sin_port;
+		addr->len = sizeof(struct sockaddr_in);
+	} else if (pjaddr->addr.sa_family == pj_AF_INET6()) {
+		struct sockaddr_in6 *sin = (struct sockaddr_in6 *) &addr->ss;
+		sin->sin6_family   = AF_INET6;
+		sin->sin6_port     = pjaddr->ipv6.sin6_port;
+		sin->sin6_flowinfo = pjaddr->ipv6.sin6_flowinfo;
+		sin->sin6_scope_id = pjaddr->ipv6.sin6_scope_id;
+		memcpy(&sin->sin6_addr, &pjaddr->ipv6.sin6_addr, sizeof(sin->sin6_addr));
+		addr->len = sizeof(struct sockaddr_in6);
+	} else {
+		memset(addr, 0, sizeof(*addr));
+		return -1;
+	}
+	return 0;
+}
+
+#ifdef TEST_FRAMEWORK
+static void fill_with_garbage(void *x, ssize_t len)
+{
+	unsigned char *w = x;
+	while (len > 0) {
+		int r = ast_random();
+		memcpy(w, &r, len > sizeof(r) ? sizeof(r) : len);
+		w += sizeof(r);
+		len -= sizeof(r);
+	}
+}
+
+AST_TEST_DEFINE(ast_sockaddr_to_pj_sockaddr_test)
+{
+	char *candidates[] = {
+		"127.0.0.1:5555",
+		"[::]:4444",
+		"192.168.0.100:0",
+		"[fec0::1:80]:0",
+		"[fec0::1]:80",
+		NULL,
+	}, **candidate = candidates;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "ast_sockaddr_to_pj_sockaddr_test";
+		info->category = "/res/res_pjproject/";
+		info->summary = "Validate conversions from an ast_sockaddr to a pj_sockaddr";
+		info->description = "This test converts an ast_sockaddr to a pj_sockaddr and validates\n"
+			"that the two evaluate to the same string when formatted.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	while (*candidate) {
+		struct ast_sockaddr addr = {{0,}};
+		pj_sockaddr pjaddr;
+		char buffer[512];
+
+		fill_with_garbage(&pjaddr, sizeof(pj_sockaddr));
+
+		if (!ast_sockaddr_parse(&addr, *candidate, 0)) {
+			ast_test_status_update(test, "Failed to parse candidate IP: %s\n", *candidate);
+			return AST_TEST_FAIL;
+		}
+
+		if (ast_sockaddr_to_pj_sockaddr(&addr, &pjaddr)) {
+			ast_test_status_update(test, "Failed to convert ast_sockaddr to pj_sockaddr: %s\n", *candidate);
+			return AST_TEST_FAIL;
+		}
+
+		pj_sockaddr_print(&pjaddr, buffer, sizeof(buffer), 1 | 2);
+
+		if (strcmp(*candidate, buffer)) {
+			ast_test_status_update(test, "Converted sockaddrs do not match: \"%s\" and \"%s\"\n",
+				*candidate,
+				buffer);
+			return AST_TEST_FAIL;
+		}
+
+		candidate++;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(ast_sockaddr_from_pj_sockaddr_test)
+{
+	char *candidates[] = {
+		"127.0.0.1:5555",
+		"[::]:4444",
+		"192.168.0.100:0",
+		"[fec0::1:80]:0",
+		"[fec0::1]:80",
+		NULL,
+	}, **candidate = candidates;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "ast_sockaddr_from_pj_sockaddr_test";
+		info->category = "/res/res_pjproject/";
+		info->summary = "Validate conversions from a pj_sockaddr to an ast_sockaddr";
+		info->description = "This test converts a pj_sockaddr to an ast_sockaddr and validates\n"
+			"that the two evaluate to the same string when formatted.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	while (*candidate) {
+		struct ast_sockaddr addr = {{0,}};
+		pj_sockaddr pjaddr;
+		pj_str_t t;
+		char buffer[512];
+
+		fill_with_garbage(&addr, sizeof(addr));
+
+		pj_strset(&t, *candidate, strlen(*candidate));
+
+		if (pj_sockaddr_parse(pj_AF_UNSPEC(), 0, &t, &pjaddr) != PJ_SUCCESS) {
+			ast_test_status_update(test, "Failed to parse candidate IP: %s\n", *candidate);
+			return AST_TEST_FAIL;
+		}
+
+		if (ast_sockaddr_from_pj_sockaddr(&addr, &pjaddr)) {
+			ast_test_status_update(test, "Failed to convert pj_sockaddr to ast_sockaddr: %s\n", *candidate);
+			return AST_TEST_FAIL;
+		}
+
+		snprintf(buffer, sizeof(buffer), "%s", ast_sockaddr_stringify(&addr));
+
+		if (strcmp(*candidate, buffer)) {
+			ast_test_status_update(test, "Converted sockaddrs do not match: \"%s\" and \"%s\"\n",
+				*candidate,
+				buffer);
+			return AST_TEST_FAIL;
+		}
+
+		candidate++;
+	}
+
+	return AST_TEST_PASS;
+}
+#endif
+
 static int load_module(void)
 {
 	ast_debug(3, "Starting PJPROJECT logging to Asterisk logger\n");
@@ -552,6 +720,9 @@ static int load_module(void)
 
 	ast_cli_register_multiple(pjproject_cli, ARRAY_LEN(pjproject_cli));
 
+	AST_TEST_REGISTER(ast_sockaddr_to_pj_sockaddr_test);
+	AST_TEST_REGISTER(ast_sockaddr_from_pj_sockaddr_test);
+
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -575,6 +746,9 @@ static int unload_module(void)
 
 	ast_sorcery_unref(pjproject_sorcery);
 
+	AST_TEST_UNREGISTER(ast_sockaddr_to_pj_sockaddr_test);
+	AST_TEST_UNREGISTER(ast_sockaddr_from_pj_sockaddr_test);
+
 	return 0;
 }
 
@@ -593,4 +767,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 6,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjproject.exports.in 1:16.1.1~dfsg-1/res/res_pjproject.exports.in
--- 1:13.23.1~dfsg-2/res/res_pjproject.exports.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjproject.exports.in	2018-12-26 18:45:13.000000000 +0000
@@ -1,6 +1,6 @@
 {
 	global:
-		LINKER_SYMBOL_PREFIXast_pjproject_*;
+		LINKER_SYMBOL_PREFIXast_*;
 	local:
 		*;
 };
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/config_auth.c 1:16.1.1~dfsg-1/res/res_pjsip/config_auth.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/config_auth.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/config_auth.c	2018-12-26 18:45:13.000000000 +0000
@@ -195,6 +195,67 @@ static struct ast_sip_endpoint_formatter
 	.format_ami = format_ami_endpoint_auth
 };
 
+static struct ao2_container *cli_get_auths(void)
+{
+	struct ao2_container *auths;
+
+	auths = ast_sorcery_retrieve_by_fields(ast_sip_get_sorcery(), "auth",
+			AST_RETRIEVE_FLAG_MULTIPLE | AST_RETRIEVE_FLAG_ALL, NULL);
+
+	return auths;
+}
+
+static int format_ami_authlist_handler(void *obj, void *arg, int flags)
+{
+	struct ast_sip_auth *auth = obj;
+	struct ast_sip_ami *ami = arg;
+	struct ast_str *buf;
+
+	buf = ast_sip_create_ami_event("AuthList", ami);
+	if (!buf) {
+		return CMP_STOP;
+	}
+
+	sip_auth_to_ami(auth, &buf);
+
+	astman_append(ami->s, "%s\r\n", ast_str_buffer(buf));
+	ami->count++;
+
+	ast_free(buf);
+
+	return 0;
+}
+
+static int ami_show_auths(struct mansession *s, const struct message *m)
+{
+	struct ast_sip_ami ami = { .s = s, .m = m, .action_id = astman_get_header(m, "ActionID"), };
+	struct ao2_container *auths;
+
+	auths = cli_get_auths();
+	if (!auths) {
+		astman_send_error(s, m, "Could not get Auths\n");
+		return 0;
+	}
+
+	if (!ao2_container_count(auths)) {
+		astman_send_error(s, m, "No Auths found\n");
+		ao2_ref(auths, -1);
+		return 0;
+	}
+
+	astman_send_listack(s, m, "A listing of Auths follows, presented as AuthList events",
+			"start");
+
+	ao2_callback(auths, OBJ_NODATA, format_ami_authlist_handler, &ami);
+
+	astman_send_list_complete_start(s, m, "AuthListComplete", ami.count);
+	astman_send_list_complete_end(s);
+
+	ao2_ref(auths, -1);
+
+	return 0;
+}
+
 static struct ao2_container *cli_get_container(const char *regex)
 {
 	RAII_VAR(struct ao2_container *, container, NULL, ao2_cleanup);
@@ -313,7 +374,7 @@ int ast_sip_initialize_sorcery_auth(void
 	ast_sorcery_object_field_register_custom(sorcery, SIP_SORCERY_AUTH_TYPE, "auth_type",
 			"userpass", auth_type_handler, auth_type_to_str, NULL, 0, 0);
 
-	internal_sip_register_endpoint_formatter(&endpoint_auth_formatter);
+	ast_sip_register_endpoint_formatter(&endpoint_auth_formatter);
 
 	cli_formatter = ao2_alloc(sizeof(struct ast_sip_cli_formatter_entry), NULL);
 	if (!cli_formatter) {
@@ -331,6 +392,10 @@ int ast_sip_initialize_sorcery_auth(void
 	ast_sip_register_cli_formatter(cli_formatter);
 	ast_cli_register_multiple(cli_commands, ARRAY_LEN(cli_commands));
 
+	if (ast_manager_register_xml("PJSIPShowAuths", EVENT_FLAG_SYSTEM, ami_show_auths)) {
+		return -1;
+	}
+
 	return 0;
 }
 
@@ -338,7 +403,9 @@ int ast_sip_destroy_sorcery_auth(void)
 {
 	ast_cli_unregister_multiple(cli_commands, ARRAY_LEN(cli_commands));
 	ast_sip_unregister_cli_formatter(cli_formatter);
-	internal_sip_unregister_endpoint_formatter(&endpoint_auth_formatter);
+	ast_sip_unregister_endpoint_formatter(&endpoint_auth_formatter);
+
+	ast_manager_unregister("PJSIPShowAuths");
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/config_domain_aliases.c 1:16.1.1~dfsg-1/res/res_pjsip/config_domain_aliases.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/config_domain_aliases.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/config_domain_aliases.c	2018-12-26 18:45:13.000000000 +0000
@@ -62,6 +62,7 @@ static int domain_alias_apply(const stru
 		 */
 		ast_log(LOG_ERROR, "%s '%s' missing required domain being aliased.\n",
 			SIP_SORCERY_DOMAIN_ALIAS_TYPE, ast_sorcery_object_get_id(alias));
+		return -1;
 	}
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/config_global.c 1:16.1.1~dfsg-1/res/res_pjsip/config_global.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/config_global.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/config_global.c	2018-12-26 18:45:13.000000000 +0000
@@ -48,6 +48,7 @@
 #define DEFAULT_MWI_TPS_QUEUE_LOW -1
 #define DEFAULT_MWI_DISABLE_INITIAL_UNSOLICITED 0
 #define DEFAULT_IGNORE_URI_USER_OPTIONS 0
+#define DEFAULT_USE_CALLERID_CONTACT 0
 
 /*!
  * \brief Cached global config object
@@ -103,6 +104,8 @@ struct global_config {
 	} mwi;
 	/*! Nonzero if URI user field options are ignored. */
 	unsigned int ignore_uri_user_options;
+	/*! Nonzero if CALLERID(num) is to be used as the default contact username instead of default_from_user */
+	unsigned int use_callerid_contact;
 };
 
 static void global_destructor(void *obj)
@@ -402,6 +405,21 @@ unsigned int ast_sip_get_ignore_uri_user
 	return ignore_uri_user_options;
 }
 
+unsigned int ast_sip_get_use_callerid_contact(void)
+{
+	unsigned int use_callerid_contact;
+	struct global_config *cfg;
+
+	cfg = get_global_cfg();
+	if (!cfg) {
+		return DEFAULT_USE_CALLERID_CONTACT;
+	}
+
+	use_callerid_contact = cfg->use_callerid_contact;
+	ao2_ref(cfg, -1);
+	return use_callerid_contact;
+}
+
 /*!
  * \internal
  * \brief Observer to set default global object if none exist.
@@ -553,6 +571,9 @@ int ast_sip_initialize_sorcery_global(vo
 	ast_sorcery_object_field_register(sorcery, "global", "ignore_uri_user_options",
 		DEFAULT_IGNORE_URI_USER_OPTIONS ? "yes" : "no",
 		OPT_BOOL_T, 1, FLDSET(struct global_config, ignore_uri_user_options));
+	ast_sorcery_object_field_register(sorcery, "global", "use_callerid_contact",
+		DEFAULT_USE_CALLERID_CONTACT ? "yes" : "no",
+		OPT_YESNO_T, 1, FLDSET(struct global_config, use_callerid_contact));
 
 	if (ast_sorcery_instance_observer_add(sorcery, &observer_callbacks_global)) {
 		return -1;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/config_system.c 1:16.1.1~dfsg-1/res/res_pjsip/config_system.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/config_system.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/config_system.c	2018-12-26 18:45:13.000000000 +0000
@@ -114,9 +114,13 @@ static int system_apply(const struct ast
 #endif
 
 	if (system->compactheaders) {
+#ifdef HAVE_PJSIP_ENDPOINT_COMPACT_FORM
+		pjsip_cfg()->endpt.use_compact_form = PJ_TRUE;
+#else
 		extern pj_bool_t pjsip_use_compact_form;
 
 		pjsip_use_compact_form = PJ_TRUE;
+#endif
 	}
 
 	sip_threadpool_options.initial_size = system->threadpool.initial_size;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/config_transport.c 1:16.1.1~dfsg-1/res/res_pjsip/config_transport.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/config_transport.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/config_transport.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,6 +30,7 @@
 #include "asterisk/acl.h"
 #include "asterisk/utils.h"
 #include "include/res_pjsip_private.h"
+/* We're only using a #define from http_websocket.h, no OPTIONAL_API symbols are used. */
 #include "asterisk/http_websocket.h"
 
 #define MAX_POINTER_STRING 33
@@ -1376,7 +1377,8 @@ static int populate_transport_states(voi
 
 struct ao2_container *ast_sip_get_transport_states(void)
 {
-	struct ao2_container *states = ao2_container_alloc(DEFAULT_STATE_BUCKETS, transport_state_hash, transport_state_cmp);
+	struct ao2_container *states = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DEFAULT_STATE_BUCKETS, transport_state_hash, NULL, transport_state_cmp);
 
 	if (!states) {
 		return NULL;
@@ -1393,7 +1395,8 @@ int ast_sip_initialize_sorcery_transport
 	struct ao2_container *transports = NULL;
 
 	/* Create outbound registration states container. */
-	transport_states = ao2_container_alloc(DEFAULT_STATE_BUCKETS, internal_state_hash, internal_state_cmp);
+	transport_states = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DEFAULT_STATE_BUCKETS, internal_state_hash, NULL, internal_state_cmp);
 	if (!transport_states) {
 		ast_log(LOG_ERROR, "Unable to allocate transport states container\n");
 		return -1;
@@ -1433,7 +1436,7 @@ int ast_sip_initialize_sorcery_transport
 	ast_sorcery_object_field_register(sorcery, "transport", "allow_reload", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_transport, allow_reload));
 	ast_sorcery_object_field_register(sorcery, "transport", "symmetric_transport", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_transport, symmetric_transport));
 
-	internal_sip_register_endpoint_formatter(&endpoint_transport_formatter);
+	ast_sip_register_endpoint_formatter(&endpoint_transport_formatter);
 
 	cli_formatter = ao2_alloc(sizeof(struct ast_sip_cli_formatter_entry), NULL);
 	if (!cli_formatter) {
@@ -1463,7 +1466,7 @@ int ast_sip_destroy_sorcery_transport(vo
 	ast_cli_unregister_multiple(cli_commands, ARRAY_LEN(cli_commands));
 	ast_sip_unregister_cli_formatter(cli_formatter);
 
-	internal_sip_unregister_endpoint_formatter(&endpoint_transport_formatter);
+	ast_sip_unregister_endpoint_formatter(&endpoint_transport_formatter);
 
 	ao2_ref(transport_states, -1);
 	transport_states = NULL;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/include/res_pjsip_private.h 1:16.1.1~dfsg-1/res/res_pjsip/include/res_pjsip_private.h
--- 1:13.23.1~dfsg-2/res/res_pjsip/include/res_pjsip_private.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/include/res_pjsip_private.h	2018-12-26 18:45:13.000000000 +0000
@@ -25,7 +25,7 @@ struct ast_sip_cli_context;
  * \internal
  * \brief Initialize the configuration for res_pjsip
  */
-int ast_res_pjsip_initialize_configuration(const struct ast_module_info *ast_module_info);
+int ast_res_pjsip_initialize_configuration(void);
 
 /*!
  * \internal
@@ -234,24 +234,6 @@ int ast_res_pjsip_init_contact_transport
 
 /*!
  * \internal
- * \brief Initialize outbound authentication support
- *
- * \retval 0 Success
- * \retval non-zero Failure
- */
-int internal_sip_initialize_outbound_authentication(void);
-
-/*!
- * \internal
- * \brief Destroy outbound authentication support
- *
- * \retval 0 Success
- * \retval non-zero Failure
- */
-void internal_sip_destroy_outbound_authentication(void);
-
-/*!
- * \internal
  * \brief Initialize system configuration
  *
  * \retval 0 Success
@@ -273,6 +255,12 @@ void ast_sip_initialize_dns(void);
 
 /*!
  * \internal
+ * \brief Initialize our own resolver support
+ */
+void ast_sip_initialize_resolver(void);
+
+/*!
+ * \internal
  * \brief Initialize global configuration
  *
  * \retval 0 Success
@@ -335,58 +323,10 @@ int sip_cli_print_global(struct ast_sip_
  */
 int sip_cli_print_system(struct ast_sip_cli_context *context);
 
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register a service without adding a self reference
- */
-int internal_sip_register_service(pjsip_module *module);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a service without removing a self reference
- */
-int internal_sip_unregister_service(pjsip_module *module);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register a supplement without adding a self reference
- */
-void internal_sip_register_supplement(struct ast_sip_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a supplement without removing a self reference
- */
-int internal_sip_unregister_supplement(struct ast_sip_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to register an endpoint formatter without adding a self reference
- */
-void internal_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a endpoint formatter without removing a self reference
- */
-int internal_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj);
-
 struct ast_sip_session_supplement;
 
 /*!
  * \internal
- * \brief Used by res_pjsip.so to register a session supplement without adding a self reference
- */
-void internal_sip_session_register_supplement(struct ast_sip_session_supplement *supplement);
-
-/*!
- * \internal
- * \brief Used by res_pjsip.so to unregister a session supplement without removing a self reference
- */
-int internal_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement);
-
-/*!
- * \internal
  * \brief Finds or creates contact_status for a contact
  */
 struct ast_sip_contact_status *ast_res_pjsip_find_or_create_contact_status(const struct ast_sip_contact *contact);
@@ -418,18 +358,6 @@ int ast_sip_destroy_scheduler(void);
 
 /*!
  * \internal
- * \brief Add a reference to the res_pjsip module
- */
-void internal_res_pjsip_ref(void);
-
-/*!
- * \internal
- * \brief Remove a reference from the res_pjsip module
- */
-void internal_res_pjsip_unref(void);
-
-/*!
- * \internal
  * \brief Determines if a uri will still be valid after an asterisk restart
  * \since 13.20.0
  *
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/location.c 1:16.1.1~dfsg-1/res/res_pjsip/location.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/location.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/location.c	2018-12-26 18:45:13.000000000 +0000
@@ -49,11 +49,22 @@ static void aor_destroy(void *obj)
 /*! \brief Allocator for AOR */
 static void *aor_alloc(const char *name)
 {
-	struct ast_sip_aor *aor = ast_sorcery_generic_alloc(sizeof(struct ast_sip_aor), aor_destroy);
+	void *lock;
+	struct ast_sip_aor *aor;
+
+	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", name);
+	if (!lock) {
+		return NULL;
+	}
+
+	aor = ast_sorcery_lockable_alloc(sizeof(struct ast_sip_aor), aor_destroy, lock);
+	ao2_ref(lock, -1);
+
 	if (!aor) {
 		return NULL;
 	}
 	ast_string_field_init(aor, 128);
+
 	return aor;
 }
 
@@ -102,7 +113,6 @@ static void contact_destroy(void *obj)
 	struct ast_sip_contact *contact = obj;
 
 	ast_string_field_free_memory(contact);
-	ast_free(contact->aor);
 	ao2_cleanup(contact->endpoint);
 }
 
@@ -118,11 +128,7 @@ static void *contact_alloc(const char *n
 		return NULL;
 	}
 
-	if (ast_string_field_init(contact, 256)
-		|| ast_string_field_init_extended(contact, endpoint_name)
-		|| ast_string_field_init_extended(contact, reg_server)
-		|| ast_string_field_init_extended(contact, via_addr)
-		|| ast_string_field_init_extended(contact, call_id)) {
+	if (ast_string_field_init(contact, 256)) {
 		ao2_cleanup(contact);
 		return NULL;
 	}
@@ -133,11 +139,7 @@ static void *contact_alloc(const char *n
 	}
 	ast_assert(aor_separator != NULL);
 
-	contact->aor = ast_strdup(aor);
-	if (!contact->aor) {
-		ao2_cleanup(contact);
-		return NULL;
-	}
+	ast_string_field_set(contact, aor, aor);
 
 	return contact;
 }
@@ -251,17 +253,11 @@ struct ao2_container *ast_sip_location_r
 	unsigned int flags)
 {
 	struct ao2_container *contacts;
-	struct ast_named_lock *lock;
 
-	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", ast_sorcery_object_get_id(aor));
-	if (!lock) {
-		return NULL;
-	}
-
-	ao2_lock(lock);
+	/* ao2_lock / ao2_unlock do not actually write aor since it has an ao2 lockobj. */
+	ao2_lock((void*)aor);
 	contacts = ast_sip_location_retrieve_aor_contacts_nolock_filtered(aor, flags);
-	ao2_unlock(lock);
-	ast_named_lock_put(lock);
+	ao2_unlock((void*)aor);
 
 	return contacts;
 }
@@ -436,19 +432,12 @@ int ast_sip_location_add_contact(struct
 		struct ast_sip_endpoint *endpoint)
 {
 	int res;
-	struct ast_named_lock *lock;
 
-	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", ast_sorcery_object_get_id(aor));
-	if (!lock) {
-		return -1;
-	}
-
-	ao2_lock(lock);
+	ao2_lock(aor);
 	res = ast_sip_location_add_contact_nolock(aor, uri, expiration_time, path_info, user_agent,
 		via_addr, via_port, call_id,
 		endpoint);
-	ao2_unlock(lock);
-	ast_named_lock_put(lock);
+	ao2_unlock(aor);
 
 	return res;
 }
@@ -1001,35 +990,83 @@ static int cli_filter_contacts(void *obj
 	return CMP_MATCH;
 }
 
+static int cli_gather_contact(void *obj, void *arg, int flags)
+{
+	struct ast_sip_contact *contact = obj;
+	RAII_VAR(struct ast_sip_contact_wrapper *, wrapper, NULL, ao2_cleanup);
+
+	if (strcmp(contact->reg_server, ast_config_AST_SYSTEM_NAME ?: "")) {
+		return 0;
+	}
+
+	wrapper = ao2_alloc_options(sizeof(struct ast_sip_contact_wrapper),
+		contact_wrapper_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!wrapper) {
+		return -1;
+	}
+
+	wrapper->contact_id = ast_malloc(strlen(contact->aor) + strlen(contact->uri) + 2);
+	if (!wrapper->contact_id) {
+		return -1;
+	}
+	sprintf(wrapper->contact_id, "%s/%s", contact->aor, contact->uri);
+
+	wrapper->aor_id = ast_strdup(contact->aor);
+	if (!wrapper->aor_id) {
+		return -1;
+	}
+
+	wrapper->contact = ao2_bump(contact);
+
+	ao2_link(arg, wrapper);
+
+	return 0;
+}
+
 static struct ao2_container *cli_contact_get_container(const char *regex)
 {
-	RAII_VAR(struct ao2_container *, parent_container, NULL, ao2_cleanup);
-	struct ao2_container *child_container;
+	RAII_VAR(struct ao2_container *, aors, NULL, ao2_cleanup);
+	RAII_VAR(struct ao2_container *, contacts, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_variable *, var_aor, NULL, ast_variables_destroy);
+	struct ao2_container *contacts_container;
 	regex_t regexbuf;
 
-	parent_container = cli_aor_get_container("");
-	if (!parent_container) {
+	if (!(var_aor = ast_variable_new("contact !=", "", ""))) {
 		return NULL;
 	}
 
-	child_container = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
+	contacts_container = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
 		cli_contact_sort, cli_contact_compare);
-	if (!child_container) {
+	if (!contacts_container) {
+		return NULL;
+	}
+
+	contacts = ast_sorcery_retrieve_by_regex(ast_sip_get_sorcery(), "contact", regex);
+	if (!contacts) {
+		ao2_ref(contacts_container, -1);
+		return NULL;
+	}
+	ao2_callback(contacts, OBJ_NODATA, cli_gather_contact, contacts_container);
+
+	aors = ast_sorcery_retrieve_by_fields(ast_sip_get_sorcery(),
+		"aor", AST_RETRIEVE_FLAG_MULTIPLE, var_aor);
+	if (!aors) {
+		ao2_ref(contacts_container, -1);
 		return NULL;
 	}
 
-	ao2_callback(parent_container, OBJ_NODATA, cli_aor_gather_contacts, child_container);
+	ao2_callback(aors, OBJ_NODATA, cli_aor_gather_contacts, contacts_container);
 
 	if (!ast_strlen_zero(regex)) {
 		if (regcomp(&regexbuf, regex, REG_EXTENDED | REG_NOSUB)) {
-			ao2_ref(child_container, -1);
+			ao2_ref(contacts_container, -1);
 			return NULL;
 		}
-		ao2_callback(child_container, OBJ_UNLINK | OBJ_MULTIPLE | OBJ_NODATA, cli_filter_contacts, &regexbuf);
+		ao2_callback(contacts_container, OBJ_UNLINK | OBJ_MULTIPLE | OBJ_NODATA, cli_filter_contacts, &regexbuf);
 		regfree(&regexbuf);
 	}
 
-	return child_container;
+	return contacts_container;
 }
 
 static void *cli_contact_retrieve_by_id(const char *id)
@@ -1089,7 +1126,7 @@ static int cli_contact_print_body(void *
 		contact->uri,
 		hash_start,
 		ast_sip_get_contact_short_status_label(status ? status->status : UNKNOWN),
-		(status && (status->status != UNKNOWN) ? ((long long) status->rtt) / 1000.0 : NAN));
+		(status && (status->status == AVAILABLE)) ? ((long long) status->rtt) / 1000.0 : NAN);
 
 	ao2_cleanup(status);
 	return 0;
@@ -1185,6 +1222,67 @@ static int cli_aor_print_body(void *obj,
 	return 0;
 }
 
+static struct ao2_container *cli_get_aors(void)
+{
+	struct ao2_container *aors;
+
+	aors = ast_sorcery_retrieve_by_fields(ast_sip_get_sorcery(), "aor",
+			AST_RETRIEVE_FLAG_MULTIPLE | AST_RETRIEVE_FLAG_ALL, NULL);
+
+	return aors;
+}
+
+static int format_ami_aorlist_handler(void *obj, void *arg, int flags)
+{
+	struct ast_sip_aor *aor = obj;
+	struct ast_sip_ami *ami = arg;
+	struct ast_str *buf;
+
+	buf = ast_sip_create_ami_event("AorList", ami);
+	if (!buf) {
+		return -1;
+	}
+
+	sip_aor_to_ami(aor, &buf);
+
+	astman_append(ami->s, "%s\r\n", ast_str_buffer(buf));
+	ami->count++;
+
+	ast_free(buf);
+
+	return 0;
+}
+
+static int ami_show_aors(struct mansession *s, const struct message *m)
+{
+	struct ast_sip_ami ami = { .s = s, .m = m, .action_id = astman_get_header(m, "ActionID"), };
+	struct ao2_container *aors;
+
+	aors = cli_get_aors();
+	if (!aors) {
+		astman_send_error(s, m, "Could not get AORs\n");
+		return 0;
+	}
+
+	if (!ao2_container_count(aors)) {
+		astman_send_error(s, m, "No AORs found\n");
+		ao2_ref(aors, -1);
+		return 0;
+	}
+
+	astman_send_listack(s, m, "A listing of AORs follows, presented as AorList events",
+			"start");
+
+	ao2_callback(aors, OBJ_NODATA, format_ami_aorlist_handler, &ami);
+
+	astman_send_list_complete_start(s, m, "AorListComplete", ami.count);
+	astman_send_list_complete_end(s);
+
+	ao2_ref(aors, -1);
+
+	return 0;
+}
+
 static struct ast_cli_entry cli_commands[] = {
 	AST_CLI_DEFINE(ast_sip_cli_traverse_objects, "List PJSIP Aors",
 		.command = "pjsip list aors",
@@ -1291,7 +1389,7 @@ int ast_sip_initialize_sorcery_location(
 	ast_sorcery_object_field_register(sorcery, "aor", "outbound_proxy", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_aor, outbound_proxy));
 	ast_sorcery_object_field_register(sorcery, "aor", "support_path", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_aor, support_path));
 
-	internal_sip_register_endpoint_formatter(&endpoint_aor_formatter);
+	ast_sip_register_endpoint_formatter(&endpoint_aor_formatter);
 
 	contact_formatter = ao2_alloc(sizeof(struct ast_sip_cli_formatter_entry), NULL);
 	if (!contact_formatter) {
@@ -1323,6 +1421,10 @@ int ast_sip_initialize_sorcery_location(
 	ast_sip_register_cli_formatter(aor_formatter);
 	ast_cli_register_multiple(cli_commands, ARRAY_LEN(cli_commands));
 
+	if (ast_manager_register_xml("PJSIPShowAors", EVENT_FLAG_SYSTEM, ami_show_aors)) {
+		return -1;
+	}
+
 	/*
 	 * Reset StatsD gauges in case we didn't shut down cleanly.
 	 * Note that this must done here, as contacts will create the contact_status
@@ -1341,8 +1443,9 @@ int ast_sip_destroy_sorcery_location(voi
 	ast_cli_unregister_multiple(cli_commands, ARRAY_LEN(cli_commands));
 	ast_sip_unregister_cli_formatter(contact_formatter);
 	ast_sip_unregister_cli_formatter(aor_formatter);
+	ast_manager_unregister("PJSIPShowAors");
 
-	internal_sip_unregister_endpoint_formatter(&endpoint_aor_formatter);
+	ast_sip_unregister_endpoint_formatter(&endpoint_aor_formatter);
 
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_configuration.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_configuration.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_configuration.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_configuration.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,6 +33,7 @@
 #include "asterisk/test.h"
 #include "asterisk/statsd.h"
 #include "asterisk/pbx.h"
+#include "asterisk/stream.h"
 
 /*! \brief Number of buckets for persistent endpoint information */
 #define PERSISTENT_BUCKETS 53
@@ -590,6 +591,13 @@ static int caller_id_handler(const struc
 	char cid_name[80] = { '\0' };
 	char cid_num[80] = { '\0' };
 
+	ast_free(endpoint->id.self.name.str);
+	endpoint->id.self.name.str = NULL;
+	endpoint->id.self.name.valid = 0;
+	ast_free(endpoint->id.self.number.str);
+	endpoint->id.self.number.str = NULL;
+	endpoint->id.self.number.valid = 0;
+
 	ast_callerid_split(var->value, cid_name, sizeof(cid_name), cid_num, sizeof(cid_num));
 	if (!ast_strlen_zero(cid_name)) {
 		endpoint->id.self.name.str = ast_strdup(cid_name);
@@ -656,7 +664,10 @@ static int caller_id_privacy_to_str(cons
 static int caller_id_tag_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
 {
 	struct ast_sip_endpoint *endpoint = obj;
+
+	ast_free(endpoint->id.self.tag);
 	endpoint->id.self.tag = ast_strdup(var->value);
+
 	return endpoint->id.self.tag ? 0 : -1;
 }
 
@@ -826,6 +837,13 @@ static int dtlsrekey_to_str(const void *
 		buf, "%u", endpoint->media.rtp.dtls_cfg.rekey) >=0 ? 0 : -1;
 }
 
+static int dtlsautogeneratecert_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+	*buf = ast_strdup(AST_YESNO(endpoint->media.rtp.dtls_cfg.ephemeral_cert));
+	return 0;
+}
+
 static int dtlscertfile_to_str(const void *obj, const intptr_t *args, char **buf)
 {
 	const struct ast_sip_endpoint *endpoint = obj;
@@ -1059,6 +1077,7 @@ static int voicemail_extension_handler(c
 {
 	struct ast_sip_endpoint *endpoint = obj;
 
+	ast_free(endpoint->subscription.mwi.voicemail_extension);
 	endpoint->subscription.mwi.voicemail_extension = ast_strdup(var->value);
 
 	return endpoint->subscription.mwi.voicemail_extension ? 0 : -1;
@@ -1078,12 +1097,10 @@ static int contact_user_handler(const st
 {
 	struct ast_sip_endpoint *endpoint = obj;
 
+	ast_free(endpoint->contact_user);
 	endpoint->contact_user = ast_strdup(var->value);
-	if (!endpoint->contact_user) {
-		return -1;
-	}
 
-	return 0;
+	return endpoint->contact_user ? 0 : -1;
 }
 
 static int contact_user_to_str(const void *obj, const intptr_t *args, char **buf)
@@ -1243,6 +1260,39 @@ static int sip_endpoint_apply_handler(co
 		return -1;
 	}
 
+	if (ast_rtp_dtls_cfg_validate(&endpoint->media.rtp.dtls_cfg)) {
+		return -1;
+	}
+
+	endpoint->media.topology = ast_stream_topology_create_from_format_cap(endpoint->media.codecs);
+	if (!endpoint->media.topology) {
+		return -1;
+	}
+
+	endpoint->media.rtcp_mux |= endpoint->media.bundle;
+
+	/*
+	 * If webrtc has been enabled then enable those attributes, and default
+	 * some, that are needed in order for webrtc to work.
+	 */
+	endpoint->media.bundle |= endpoint->media.webrtc;
+	endpoint->media.rtcp_mux |= endpoint->media.webrtc;
+	endpoint->media.rtp.use_avpf |= endpoint->media.webrtc;
+	endpoint->media.rtp.ice_support |= endpoint->media.webrtc;
+	endpoint->media.rtp.use_received_transport |= endpoint->media.webrtc;
+
+	if (endpoint->media.webrtc) {
+		endpoint->media.rtp.encryption = AST_SIP_MEDIA_ENCRYPT_DTLS;
+		endpoint->media.rtp.dtls_cfg.enabled = 1;
+		endpoint->media.rtp.dtls_cfg.default_setup = AST_RTP_DTLS_SETUP_ACTPASS;
+		endpoint->media.rtp.dtls_cfg.verify = AST_RTP_DTLS_VERIFY_FINGERPRINT;
+
+		if (ast_strlen_zero(endpoint->media.rtp.dtls_cfg.certfile)) {
+			/* If no certificate has been specified, try to automatically create one */
+			endpoint->media.rtp.dtls_cfg.ephemeral_cert = 1;
+		}
+	}
+
 	return 0;
 }
 
@@ -1525,8 +1575,8 @@ static int ami_show_endpoints(struct man
 		return 0;
 	}
 
-	astman_send_listack(s, m, "A listing of Endpoints follows, "
-			    "presented as EndpointList events", "start");
+	astman_send_listack(s, m, "A listing of Endpoints follows, presented as EndpointList events",
+		"start");
 
 	ao2_callback(endpoints, OBJ_NODATA, format_ami_endpoints, &ami);
 
@@ -1702,7 +1752,7 @@ static void load_all_endpoints(void)
 	ao2_cleanup(endpoints);
 }
 
-int ast_res_pjsip_initialize_configuration(const struct ast_module_info *ast_module_info)
+int ast_res_pjsip_initialize_configuration(void)
 {
 	if (ast_manager_register_xml(AMI_SHOW_ENDPOINTS, EVENT_FLAG_SYSTEM, ami_show_endpoints) ||
 	    ast_manager_register_xml(AMI_SHOW_ENDPOINT, EVENT_FLAG_SYSTEM, ami_show_endpoint)) {
@@ -1769,6 +1819,8 @@ int ast_res_pjsip_initialize_configurati
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "identify_by", "username,ip", ident_handler, ident_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "direct_media", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.direct_media.enabled));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "direct_media_method", "invite", direct_media_method_handler, direct_media_method_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "trust_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, trust_connected_line));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, send_connected_line));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "connected_line_method", "invite", connected_line_method_handler, connected_line_method_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "direct_media_glare_mitigation", "none", direct_media_glare_mitigation_handler, direct_media_glare_mitigation_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "disable_direct_media_on_nat", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.direct_media.disable_on_nat));
@@ -1779,12 +1831,12 @@ int ast_res_pjsip_initialize_configurati
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "trust_id_outbound", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, id.trust_outbound));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_pai", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, id.send_pai));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_rpid", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, id.send_rpid));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rpid_immediate", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, rpid_immediate));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rpid_immediate", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, id.rpid_immediate));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_diversion", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, id.send_diversion));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "mailboxes", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, subscription.mwi.mailboxes));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "voicemail_extension", "", voicemail_extension_handler, voicemail_extension_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "aggregate_mwi", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, subscription.mwi.aggregate));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "mwi_subscribe_replaces_unsolicited", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, subscription.mwi.subscribe_replaces_unsolicited));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "mwi_subscribe_replaces_unsolicited", "no", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, subscription.mwi.subscribe_replaces_unsolicited));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "media_encryption", "no", media_encryption_handler, media_encryption_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "use_avpf", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.use_avpf));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "force_avp", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.force_avp));
@@ -1812,6 +1864,7 @@ int ast_res_pjsip_initialize_configurati
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "record_off_feature", "automixmon", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, info.recording.offfeature));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "allow_transfer", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, allowtransfer));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "user_eq_phone", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, usereqphone));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "moh_passthrough", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, moh_passthrough));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "sdp_owner", "-", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, media.sdpowner));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "sdp_session", "Asterisk", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, media.sdpsession));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "tos_audio", "0", tos_handler, tos_audio_to_str, NULL, 0, 0);
@@ -1826,6 +1879,7 @@ int ast_res_pjsip_initialize_configurati
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtp_engine", "asterisk", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, media.rtp.engine));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_verify", "no", dtls_handler, dtlsverify_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_rekey", "0", dtls_handler, dtlsrekey_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_auto_generate_cert", "no", dtls_handler, dtlsautogeneratecert_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_cert_file", "", dtls_handler, dtlscertfile_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_private_key", "", dtls_handler, dtlsprivatekey_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "dtls_cipher", "", dtls_handler, dtlscipher_to_str, NULL, 0, 0);
@@ -1848,14 +1902,19 @@ int ast_res_pjsip_initialize_configurati
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "contact_acl", "", endpoint_acl_handler, contact_acl_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "subscribe_context", "", OPT_CHAR_ARRAY_T, 0, CHARFLDSET(struct ast_sip_endpoint, subscription.context));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "contact_user", "", contact_user_handler, contact_user_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "preferred_codec_only", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, preferred_codec_only));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "asymmetric_rtp_codec", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, asymmetric_rtp_codec));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtcp_mux", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, rtcp_mux));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "rtcp_mux", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtcp_mux));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "allow_overlap", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, allow_overlap));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "refer_blind_progress", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, refer_blind_progress));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "notify_early_inuse_ringing", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, notify_early_inuse_ringing));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "max_audio_streams", "1", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, media.max_audio_streams));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "max_video_streams", "1", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, media.max_video_streams));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "bundle", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.bundle));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "webrtc", "no", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, media.webrtc));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "incoming_mwi_mailbox", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, incoming_mwi_mailbox));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "follow_early_media_fork", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, follow_early_media_fork));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "accept_multiple_sdp_answers", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, accept_multiple_sdp_answers));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "follow_early_media_fork", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.follow_early_media_fork));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "accept_multiple_sdp_answers", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.accept_multiple_sdp_answers));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "suppress_q850_reason_headers", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, suppress_q850_reason_headers));
 
 	if (ast_sip_initialize_sorcery_transport()) {
@@ -1968,7 +2027,8 @@ static void endpoint_destructor(void* ob
 
 	ast_string_field_free_memory(endpoint);
 
-	ao2_ref(endpoint->media.codecs, -1);
+	ao2_cleanup(endpoint->media.codecs);
+	ast_stream_topology_free(endpoint->media.topology);
 	subscription_configuration_destroy(&endpoint->subscription);
 	info_configuration_destroy(&endpoint->info);
 	media_configuration_destroy(&endpoint->media);
@@ -2006,8 +2066,7 @@ void *ast_sip_endpoint_alloc(const char
 	if (!endpoint) {
 		return NULL;
 	}
-	if (ast_string_field_init(endpoint, 64)
-		|| ast_string_field_init_extended(endpoint, incoming_mwi_mailbox)) {
+	if (ast_string_field_init(endpoint, 64)) {
 		ao2_cleanup(endpoint);
 		return NULL;
 	}
@@ -2028,7 +2087,9 @@ void *ast_sip_endpoint_alloc(const char
 		ao2_cleanup(endpoint);
 		return NULL;
 	}
+
 	ast_party_id_init(&endpoint->id.self);
+	endpoint->id.self.tag = ast_strdup("");
 
 	if (AST_VECTOR_INIT(&endpoint->ident_method_order, 1)) {
 		return NULL;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_distributor.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_distributor.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_distributor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_distributor.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,11 +29,12 @@
 
 static int distribute(void *data);
 static pj_bool_t distributor(pjsip_rx_data *rdata);
+static pj_status_t record_serializer(pjsip_tx_data *tdata);
 
 static pjsip_module distributor_mod = {
 	.name = {"Request Distributor", 19},
 	.priority = PJSIP_MOD_PRIORITY_TSX_LAYER - 6,
-	.on_tx_request = ast_sip_record_request_serializer,
+	.on_tx_request = record_serializer,
 	.on_rx_request = distributor,
 	.on_rx_response = distributor,
 };
@@ -63,7 +64,16 @@ struct unidentified_request{
 /*! Pool of serializers to use if not supplied. */
 static struct ast_taskprocessor *distributor_pool[DISTRIBUTOR_POOL_SIZE];
 
-pj_status_t ast_sip_record_request_serializer(pjsip_tx_data *tdata)
+/*!
+ * \internal
+ * \brief Record the task's serializer name on the tdata structure.
+ * \since 14.0.0
+ *
+ * \param tdata The outgoing message.
+ *
+ * \retval PJ_SUCCESS.
+ */
+static pj_status_t record_serializer(pjsip_tx_data *tdata)
 {
 	struct ast_taskprocessor *serializer;
 
@@ -530,12 +540,22 @@ static pj_bool_t distributor(pjsip_rx_da
 			 * we are being overloaded and need to defer adding new work to
 			 * the system.  To defer the work we will ignore the request and
 			 * rely on the peer's transport layer to retransmit the message.
-			 * We usually work off the overload within a few seconds.  The
-			 * alternative is to send back a 503 response to these requests
-			 * and be done with it.
+			 * We usually work off the overload within a few seconds.
+			 * If transport is non-UDP we send a 503 response instead.
 			 */
-			ast_debug(3, "Taskprocessor overload alert: Ignoring '%s'.\n",
-				pjsip_rx_data_get_info(rdata));
+			switch (rdata->tp_info.transport->key.type) {
+			case PJSIP_TRANSPORT_UDP6:
+			case PJSIP_TRANSPORT_UDP:
+				ast_debug(3, "Taskprocessor overload alert: Ignoring '%s'.\n",
+					pjsip_rx_data_get_info(rdata));
+				break;
+			default:
+				ast_debug(3, "Taskprocessor overload on non-udp transport. Received:'%s'. "
+					"Responding with a 503.\n", pjsip_rx_data_get_info(rdata));
+				pjsip_endpt_respond_stateless(ast_sip_get_pjsip_endpoint(), rdata,
+					PJSIP_SC_SERVICE_UNAVAILABLE, NULL, NULL, NULL);
+				break;
+			}
 			ao2_cleanup(dist);
 			return PJ_TRUE;
 		}
@@ -638,16 +658,21 @@ static void log_failed_request(pjsip_rx_
 	char from_buf[PJSIP_MAX_URL_SIZE];
 	char callid_buf[PJSIP_MAX_URL_SIZE];
 	char method_buf[PJSIP_MAX_URL_SIZE];
+	char src_addr_buf[AST_SOCKADDR_BUFLEN];
 	pjsip_uri_print(PJSIP_URI_IN_FROMTO_HDR, rdata->msg_info.from->uri, from_buf, PJSIP_MAX_URL_SIZE);
 	ast_copy_pj_str(callid_buf, &rdata->msg_info.cid->id, PJSIP_MAX_URL_SIZE);
 	ast_copy_pj_str(method_buf, &rdata->msg_info.msg->line.req.method.name, PJSIP_MAX_URL_SIZE);
 	if (count) {
-		ast_log(LOG_NOTICE, "Request '%s' from '%s' failed for '%s:%d' (callid: %s) - %s"
+		ast_log(LOG_NOTICE, "Request '%s' from '%s' failed for '%s' (callid: %s) - %s"
 			" after %u tries in %.3f ms\n",
-			method_buf, from_buf, rdata->pkt_info.src_name, rdata->pkt_info.src_port, callid_buf, msg, count, period / 1000.0);
+			method_buf, from_buf,
+			pj_sockaddr_print(&rdata->pkt_info.src_addr, src_addr_buf, sizeof(src_addr_buf), 3),
+			callid_buf, msg, count, period / 1000.0);
 	} else {
-		ast_log(LOG_NOTICE, "Request '%s' from '%s' failed for '%s:%d' (callid: %s) - %s\n",
-			method_buf, from_buf, rdata->pkt_info.src_name, rdata->pkt_info.src_port, callid_buf, msg);
+		ast_log(LOG_NOTICE, "Request '%s' from '%s' failed for '%s' (callid: %s) - %s\n",
+			method_buf, from_buf,
+			pj_sockaddr_print(&rdata->pkt_info.src_addr, src_addr_buf, sizeof(src_addr_buf), 3),
+			callid_buf, msg);
 	}
 }
 
@@ -1218,7 +1243,7 @@ static int distributor_pool_setup(void)
 		/* Create name with seq number appended. */
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/distributor");
 
-		distributor_pool[idx] = ast_sip_create_serializer_named(tps_name);
+		distributor_pool[idx] = ast_sip_create_serializer(tps_name);
 		if (!distributor_pool[idx]) {
 			return -1;
 		}
@@ -1266,15 +1291,15 @@ int ast_sip_initialize_distributor(void)
 		return -1;
 	}
 
-	if (internal_sip_register_service(&distributor_mod)) {
+	if (ast_sip_register_service(&distributor_mod)) {
 		ast_sip_destroy_distributor();
 		return -1;
 	}
-	if (internal_sip_register_service(&endpoint_mod)) {
+	if (ast_sip_register_service(&endpoint_mod)) {
 		ast_sip_destroy_distributor();
 		return -1;
 	}
-	if (internal_sip_register_service(&auth_mod)) {
+	if (ast_sip_register_service(&auth_mod)) {
 		ast_sip_destroy_distributor();
 		return -1;
 	}
@@ -1305,9 +1330,9 @@ void ast_sip_destroy_distributor(void)
 	ast_cli_unregister_multiple(cli_commands, ARRAY_LEN(cli_commands));
 	ast_sip_unregister_cli_formatter(unid_formatter);
 
-	internal_sip_unregister_service(&auth_mod);
-	internal_sip_unregister_service(&endpoint_mod);
-	internal_sip_unregister_service(&distributor_mod);
+	ast_sip_unregister_service(&auth_mod);
+	ast_sip_unregister_service(&endpoint_mod);
+	ast_sip_unregister_service(&distributor_mod);
 
 	ao2_global_obj_release(artificial_auth);
 	ao2_cleanup(artificial_endpoint);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_global_headers.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_global_headers.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_global_headers.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_global_headers.c	2018-12-26 18:45:13.000000000 +0000
@@ -165,7 +165,7 @@ void ast_sip_initialize_global_headers(v
 	AST_RWLIST_HEAD_INIT(&request_headers);
 	AST_RWLIST_HEAD_INIT(&response_headers);
 
-	internal_sip_register_service(&global_header_mod);
+	ast_sip_register_service(&global_header_mod);
 }
 
 static void destroy_headers(struct header_list *headers)
@@ -183,5 +183,5 @@ void ast_sip_destroy_global_headers(void
 	destroy_headers(&request_headers);
 	destroy_headers(&response_headers);
 
-	internal_sip_unregister_service(&global_header_mod);
+	ast_sip_unregister_service(&global_header_mod);
 }
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_message_filter.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_message_filter.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_message_filter.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_message_filter.c	2018-12-26 18:45:13.000000000 +0000
@@ -517,24 +517,24 @@ static pj_bool_t filter_on_rx_message(pj
 
 void ast_res_pjsip_cleanup_message_filter(void)
 {
-	internal_sip_unregister_service(&filter_module_tsx);
-	internal_sip_unregister_service(&filter_module_transport);
-	internal_sip_unregister_supplement(&filter_supplement);
-	internal_sip_session_unregister_supplement(&filter_session_supplement);
+	ast_sip_unregister_service(&filter_module_tsx);
+	ast_sip_unregister_service(&filter_module_transport);
+	ast_sip_unregister_supplement(&filter_supplement);
+	ast_sip_session_unregister_supplement(&filter_session_supplement);
 }
 
 int ast_res_pjsip_init_message_filter(void)
 {
-	internal_sip_session_register_supplement(&filter_session_supplement);
-	internal_sip_register_supplement(&filter_supplement);
+	ast_sip_session_register_supplement(&filter_session_supplement);
+	ast_sip_register_supplement(&filter_supplement);
 
-	if (internal_sip_register_service(&filter_module_transport)) {
+	if (ast_sip_register_service(&filter_module_transport)) {
 		ast_log(LOG_ERROR, "Could not register message filter module for incoming and outgoing requests\n");
 		ast_res_pjsip_cleanup_message_filter();
 		return -1;
 	}
 
-	if (internal_sip_register_service(&filter_module_tsx)) {
+	if (ast_sip_register_service(&filter_module_tsx)) {
 		ast_log(LOG_ERROR, "Could not register message filter module for incoming and outgoing requests\n");
 		ast_res_pjsip_cleanup_message_filter();
 		return -1;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_options.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_options.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_options.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_options.c	2018-12-26 18:45:13.000000000 +0000
@@ -321,7 +321,7 @@ static const char *status_map[] = {
 	[UNAVAILABLE] = "Unreachable",
 	[AVAILABLE] = "Reachable",
 	[UNKNOWN] = "Unknown",
-	[CREATED] = "Created",
+	[CREATED] = "NonQualified",
 	[REMOVED] = "Removed",
 };
 
@@ -329,7 +329,7 @@ static const char *short_status_map[] =
 	[UNAVAILABLE] = "Unavail",
 	[AVAILABLE] = "Avail",
 	[UNKNOWN] = "Unknown",
-	[CREATED] = "Created",
+	[CREATED] = "NonQual",
 	[REMOVED] = "Removed",
 };
 
@@ -975,7 +975,7 @@ static struct sip_options_aor *sip_optio
 
 	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/options/%s",
 		ast_sorcery_object_get_id(aor));
-	aor_options->serializer = ast_sip_create_serializer_group_named(tps_name,
+	aor_options->serializer = ast_sip_create_serializer_group(tps_name,
 		shutdown_group);
 	if (!aor_options->serializer) {
 		ao2_ref(aor_options, -1);
@@ -1164,10 +1164,10 @@ static int sip_options_set_contact_statu
 	switch (contact_status->status) {
 	case AVAILABLE:
 	case UNAVAILABLE:
-	case CREATED:
-		sip_options_set_contact_status(contact_status, UNKNOWN);
-		break;
 	case UNKNOWN:
+		sip_options_set_contact_status(contact_status, CREATED);
+		break;
+	case CREATED:
 	case REMOVED:
 		break;
 	}
@@ -2065,7 +2065,7 @@ struct sip_options_contact_observer_task
 /*!
  * \brief Check if the contact qualify options are different than local aor qualify options
  */
-static int has_qualify_changed (struct ast_sip_contact *contact, struct sip_options_aor *aor_options)
+static int has_qualify_changed (const struct ast_sip_contact *contact, const struct sip_options_aor *aor_options)
 {
 	if (!contact) {
 	    return 0;
@@ -2097,13 +2097,7 @@ static int sip_options_contact_add_task(
 	ao2_link(task_data->aor_options->contacts, task_data->contact);
 
 	contact_status = ast_res_pjsip_find_or_create_contact_status(task_data->contact);
-	if (contact_status) {
-		if (!task_data->aor_options->qualify_frequency
-			&& contact_status->status == CREATED) {
-			sip_options_set_contact_status(contact_status, UNKNOWN);
-		}
-		ao2_ref(contact_status, -1);
-	}
+	ao2_cleanup(contact_status);
 
 	if (task_data->aor_options->qualify_frequency) {
 		/* If this is the first contact we need to schedule up qualification */
@@ -2192,79 +2186,27 @@ static void contact_observer_created(con
 		sip_options_contact_add_management_task, (void *) obj);
 }
 
-/*!
- * \brief Task which updates a dynamic contact to an AOR
- * \note Run by aor_options->serializer
- */
-static int sip_options_contact_update_task(void *obj)
-{
-	struct sip_options_contact_observer_task_data *task_data = obj;
-	struct ast_sip_contact_status *contact_status;
-
-	contact_status = ast_sip_get_contact_status(task_data->contact);
-	if (contact_status) {
-		switch (contact_status->status) {
-		case CREATED:
-			sip_options_set_contact_status(contact_status, UNKNOWN);
-			break;
-		case UNAVAILABLE:
-		case AVAILABLE:
-		case UNKNOWN:
-			/* Refresh the ContactStatus AMI events. */
-			sip_options_contact_status_update(contact_status);
-			break;
-		case REMOVED:
-			break;
-		}
-		ao2_ref(contact_status, -1);
-	}
-
-	ao2_ref(task_data->contact, -1);
-	ao2_ref(task_data->aor_options, -1);
-	ast_free(task_data);
-	return 0;
-}
-
 /*! \brief Observer callback invoked on contact update */
 static void contact_observer_updated(const void *obj)
 {
-	struct sip_options_contact_observer_task_data *task_data;
+	const struct ast_sip_contact *contact = obj;
+	struct sip_options_aor *aor_options = ao2_find(sip_options_aors, contact->aor, OBJ_SEARCH_KEY);
 
-	task_data = ast_malloc(sizeof(*task_data));
-	if (!task_data) {
-		return;
-	}
-
-	task_data->contact = (struct ast_sip_contact *) obj;
-	task_data->aor_options = ao2_find(sip_options_aors, task_data->contact->aor,
-		OBJ_SEARCH_KEY);
-
-	if (has_qualify_changed(task_data->contact, task_data->aor_options)) {
+	if (has_qualify_changed(contact, aor_options)) {
 		struct ast_sip_aor *aor;
 
 		aor = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), "aor",
-			task_data->contact->aor);
+			contact->aor);
 		if (aor) {
 			ast_debug(3, "AOR '%s' qualify options have been modified. Synchronize an AOR local state\n",
-				task_data->contact->aor);
+				contact->aor);
 			ast_sip_push_task_wait_serializer(management_serializer,
 				sip_options_aor_observer_modified_task, aor);
 			ao2_ref(aor, -1);
 		}
 	}
 
-	if (!task_data->aor_options) {
-		ast_free(task_data);
-		return;
-	}
-
-	ao2_ref(task_data->contact, +1);
-	if (ast_sip_push_task(task_data->aor_options->serializer,
-		sip_options_contact_update_task, task_data)) {
-		ao2_ref(task_data->contact, -1);
-		ao2_ref(task_data->aor_options, -1);
-		ast_free(task_data);
-	}
+	ao2_cleanup(aor_options);
 }
 
 /*!
@@ -2398,6 +2340,89 @@ static char *cli_qualify(struct ast_cli_
 	return CLI_SUCCESS;
 }
 
+static struct ao2_container *get_all_contacts(void)
+{
+	struct ao2_container *contacts;
+
+	contacts = ast_sorcery_retrieve_by_fields(ast_sip_get_sorcery(), "contact",
+			AST_RETRIEVE_FLAG_MULTIPLE | AST_RETRIEVE_FLAG_ALL, NULL);
+
+	return contacts;
+}
+
+static int sip_contact_to_ami(const struct ast_sip_contact *contact,
+			   struct ast_str **buf)
+{
+	return ast_sip_sorcery_object_to_ami(contact, buf);
+}
+
+static int format_ami_contactlist_handler(void *obj, void *arg, int flags)
+{
+	struct ast_sip_contact *contact = obj;
+	struct ast_sip_ami *ami = arg;
+	struct ast_str *buf;
+	struct ast_sip_contact_status *status;
+
+	buf = ast_sip_create_ami_event("ContactList", ami);
+	if (!buf) {
+		return CMP_STOP;
+	}
+
+	if (sip_contact_to_ami(contact, &buf)) {
+		ast_free(buf);
+		return CMP_STOP;
+	}
+
+	/* Add extra info */
+	status = ast_sip_get_contact_status(contact);
+	ast_str_append(&buf, 0, "Status: %s\r\n",
+		ast_sip_get_contact_status_label(status ? status->status : UNKNOWN));
+	if (!status || status->status != AVAILABLE) {
+		ast_str_append(&buf, 0, "RoundtripUsec: N/A\r\n");
+	} else {
+		ast_str_append(&buf, 0, "RoundtripUsec: %" PRId64 "\r\n", status->rtt);
+	}
+	ao2_cleanup(status);
+
+	astman_append(ami->s, "%s\r\n", ast_str_buffer(buf));
+
+	ami->count++;
+
+	ast_free(buf);
+
+	return 0;
+}
+
+static int ami_show_contacts(struct mansession *s, const struct message *m)
+{
+	struct ast_sip_ami ami = { .s = s, .m = m, .action_id = astman_get_header(m, "ActionID"), };
+	struct ao2_container *contacts;
+
+	contacts = get_all_contacts();
+	if (!contacts) {
+		astman_send_error(s, m, "Could not get Contacts\n");
+		return 0;
+	}
+
+	if (!ao2_container_count(contacts)) {
+		astman_send_error(s, m, "No Contacts found\n");
+		ao2_ref(contacts, -1);
+		return 0;
+	}
+
+	astman_send_listack(s, m, "A listing of Contacts follows, presented as ContactList events",
+			"start");
+
+	ao2_callback(contacts, OBJ_NODATA, format_ami_contactlist_handler, &ami);
+
+	astman_send_list_complete_start(s, m, "ContactListComplete", ami.count);
+	astman_send_list_complete_end(s);
+
+	ao2_ref(contacts, -1);
+
+	return 0;
+}
+
 static char *cli_show_qualify_endpoint(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
 {
 	RAII_VAR(struct ast_sip_endpoint *, endpoint, NULL, ao2_cleanup);
@@ -2632,7 +2657,6 @@ static struct ast_cli_entry cli_options[
 	AST_CLI_DEFINE(cli_reload_qualify_aor, "Synchronize the PJSIP Aor qualify options"),
 };
 
-
 int ast_sip_format_contact_ami(void *obj, void *arg, int flags)
 {
 	struct ast_sip_contact_wrapper *wrapper = obj;
@@ -2665,7 +2689,7 @@ int ast_sip_format_contact_ami(void *obj
 	}
 	ast_str_append(&buf, 0, "Status: %s\r\n",
 		ast_sip_get_contact_status_label(status ? status->status : UNKNOWN));
-	if (!status || status->status == UNKNOWN) {
+	if (!status || status->status != AVAILABLE) {
 		ast_str_append(&buf, 0, "RoundtripUsec: N/A\r\n");
 	} else {
 		ast_str_append(&buf, 0, "RoundtripUsec: %" PRId64 "\r\n", status->rtt);
@@ -2758,7 +2782,8 @@ void ast_res_pjsip_cleanup_options_handl
 
 	ast_cli_unregister_multiple(cli_options, ARRAY_LEN(cli_options));
 	ast_manager_unregister("PJSIPQualify");
-	internal_sip_unregister_endpoint_formatter(&contact_status_formatter);
+	ast_manager_unregister("PJSIPShowContacts");
+	ast_sip_unregister_endpoint_formatter(&contact_status_formatter);
 
 	ast_sorcery_observer_remove(ast_sip_get_sorcery(), "contact",
 		&contact_observer_callbacks);
@@ -2871,7 +2896,7 @@ int ast_res_pjsip_init_options_handling(
 		return -1;
 	}
 
-	mgmt_serializer = ast_sip_create_serializer_named("pjsip/options/manage");
+	mgmt_serializer = ast_sip_create_serializer("pjsip/options/manage");
 	if (!mgmt_serializer) {
 		ast_res_pjsip_cleanup_options_handling();
 		return -1;
@@ -2898,9 +2923,10 @@ int ast_res_pjsip_init_options_handling(
 		return -1;
 	}
 
-	internal_sip_register_endpoint_formatter(&contact_status_formatter);
-	ast_manager_register2("PJSIPQualify", EVENT_FLAG_SYSTEM | EVENT_FLAG_REPORTING,
-		ami_sip_qualify, NULL, NULL, NULL);
+	ast_sip_register_endpoint_formatter(&contact_status_formatter);
+	ast_manager_register_xml("PJSIPQualify", EVENT_FLAG_SYSTEM | EVENT_FLAG_REPORTING,
+		ami_sip_qualify);
+	ast_manager_register_xml("PJSIPShowContacts", EVENT_FLAG_SYSTEM, ami_show_contacts);
 	ast_cli_register_multiple(cli_options, ARRAY_LEN(cli_options));
 
 	return 0;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_outbound_auth.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_outbound_auth.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_outbound_auth.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_outbound_auth.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,102 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2013, Digium, Inc.
- *
- * Mark Michelson <mmichelson@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-#include "asterisk.h"
-
-#undef bzero
-#define bzero bzero
-#include <pjsip.h>
-
-#include "asterisk/res_pjsip.h"
-#include "asterisk/module.h"
-#include "include/res_pjsip_private.h"
-
-static pj_bool_t outbound_auth(pjsip_rx_data *rdata);
-
-static pjsip_module outbound_auth_mod = {
-	.name = {"Outbound Authentication", 19},
-	.priority = PJSIP_MOD_PRIORITY_DIALOG_USAGE,
-	.on_rx_response = outbound_auth,
-};
-
-struct outbound_auth_cb_data {
-	ast_sip_dialog_outbound_auth_cb cb;
-	void *user_data;
-};
-
-static pj_bool_t outbound_auth(pjsip_rx_data *rdata)
-{
-	RAII_VAR(struct ast_sip_endpoint *, endpoint, NULL, ao2_cleanup);
-	pjsip_transaction *tsx;
-	pjsip_dialog *dlg;
-	struct outbound_auth_cb_data *cb_data;
-	pjsip_tx_data *tdata;
-
-	if (rdata->msg_info.msg->line.status.code != 401 &&
-			rdata->msg_info.msg->line.status.code != 407) {
-		/* Doesn't pertain to us. Move on */
-		return PJ_FALSE;
-	}
-
-	tsx = pjsip_rdata_get_tsx(rdata);
-	dlg = pjsip_rdata_get_dlg(rdata);
-	if (!dlg || !tsx) {
-		return PJ_FALSE;
-	}
-
-	endpoint = ast_sip_dialog_get_endpoint(dlg);
-	if (!endpoint) {
-		return PJ_FALSE;
-	}
-
-	if (ast_sip_create_request_with_auth(&endpoint->outbound_auths, rdata, tsx, &tdata)) {
-		return PJ_FALSE;
-	}
-
-	cb_data = dlg->mod_data[outbound_auth_mod.id];
-	if (cb_data) {
-		cb_data->cb(dlg, tdata, cb_data->user_data);
-		return PJ_TRUE;
-	}
-
-	pjsip_dlg_send_request(dlg, tdata, -1, NULL);
-	return PJ_TRUE;
-}
-
-int ast_sip_dialog_setup_outbound_authentication(pjsip_dialog *dlg, const struct ast_sip_endpoint *endpoint,
-		ast_sip_dialog_outbound_auth_cb cb, void *user_data)
-{
-	struct outbound_auth_cb_data *cb_data = PJ_POOL_ZALLOC_T(dlg->pool, struct outbound_auth_cb_data);
-	cb_data->cb = cb;
-	cb_data->user_data = user_data;
-
-	dlg->sess_count++;
-	pjsip_dlg_add_usage(dlg, &outbound_auth_mod, cb_data);
-	dlg->sess_count--;
-
-	return 0;
-}
-
-int internal_sip_initialize_outbound_authentication(void) {
-	return internal_sip_register_service(&outbound_auth_mod);
-}
-
-
-void internal_sip_destroy_outbound_authentication(void) {
-	internal_sip_unregister_service(&outbound_auth_mod);
-}
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_resolver.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_resolver.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_resolver.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_resolver.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,718 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+#include "asterisk.h"
+
+#include <pjsip.h>
+#include <pjlib-util/errno.h>
+
+#include <arpa/nameser.h>
+
+#include "asterisk/astobj2.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_query_set.h"
+#include "asterisk/dns_srv.h"
+#include "asterisk/dns_naptr.h"
+#include "asterisk/res_pjsip.h"
+#include "include/res_pjsip_private.h"
+#include "asterisk/taskprocessor.h"
+#include "asterisk/threadpool.h"
+
+#ifdef HAVE_PJSIP_EXTERNAL_RESOLVER
+
+/*! \brief Structure which contains transport+port information for an active query */
+struct sip_target {
+	/*! \brief The transport to be used */
+	pjsip_transport_type_e transport;
+	/*! \brief The port */
+	int port;
+};
+
+/*! \brief The vector used for current targets */
+AST_VECTOR(targets, struct sip_target);
+
+/*! \brief Structure which keeps track of resolution */
+struct sip_resolve {
+	/*! \brief Addresses currently being resolved, indexed based on index of queries in query set */
+	struct targets resolving;
+	/*! \brief Active queries */
+	struct ast_dns_query_set *queries;
+	/*! \brief Current viable server addresses */
+	pjsip_server_addresses addresses;
+	/*! \brief Serializer to run async callback into pjlib. */
+	struct ast_taskprocessor *serializer;
+	/*! \brief Callback to invoke upon completion */
+	pjsip_resolver_callback *callback;
+	/*! \brief User provided data */
+	void *token;
+};
+
+/*! \brief Our own defined transports, reduces the size of sip_available_transports */
+enum sip_resolver_transport {
+	SIP_RESOLVER_TRANSPORT_UDP,
+	SIP_RESOLVER_TRANSPORT_TCP,
+	SIP_RESOLVER_TRANSPORT_TLS,
+	SIP_RESOLVER_TRANSPORT_UDP6,
+	SIP_RESOLVER_TRANSPORT_TCP6,
+	SIP_RESOLVER_TRANSPORT_TLS6,
+};
+
+/*! \brief Available transports on the system */
+static int sip_available_transports[] = {
+	/* This is a list of transports with whether they are available as a valid transport
+	 * stored. We use our own identifier as to reduce the size of sip_available_transports.
+	 * As this array is only manipulated at startup it does not require a lock to protect
+	 * it.
+	 */
+	[SIP_RESOLVER_TRANSPORT_UDP] = 0,
+	[SIP_RESOLVER_TRANSPORT_TCP] = 0,
+	[SIP_RESOLVER_TRANSPORT_TLS] = 0,
+	[SIP_RESOLVER_TRANSPORT_UDP6] = 0,
+	[SIP_RESOLVER_TRANSPORT_TCP6] = 0,
+	[SIP_RESOLVER_TRANSPORT_TLS6] = 0,
+};
+
+/*!
+ * \internal
+ * \brief Destroy resolution data
+ *
+ * \param data The resolution data to destroy
+ *
+ * \return Nothing
+ */
+static void sip_resolve_destroy(void *data)
+{
+	struct sip_resolve *resolve = data;
+
+	AST_VECTOR_FREE(&resolve->resolving);
+	ao2_cleanup(resolve->queries);
+	ast_taskprocessor_unreference(resolve->serializer);
+}
+
+/*!
+ * \internal
+ * \brief Check whether a transport is available or not
+ *
+ * \param transport The PJSIP transport type
+ *
+ * \return 1 success (transport is available)
+ * \return 0 failure (transport is not available)
+ */
+static int sip_transport_is_available(enum pjsip_transport_type_e transport)
+{
+	enum sip_resolver_transport resolver_transport;
+
+	if (transport == PJSIP_TRANSPORT_UDP) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_UDP;
+	} else if (transport == PJSIP_TRANSPORT_TCP) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TCP;
+	} else if (transport == PJSIP_TRANSPORT_TLS) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TLS;
+	} else if (transport == PJSIP_TRANSPORT_UDP6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_UDP6;
+	} else if (transport == PJSIP_TRANSPORT_TCP6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TCP6;
+	} else if (transport == PJSIP_TRANSPORT_TLS6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TLS6;
+	} else {
+		return 0;
+	}
+
+	return sip_available_transports[resolver_transport];
+}
+
+/*!
+ * \internal
+ * \brief Add a query to be resolved
+ *
+ * \param resolve The ongoing resolution
+ * \param name What to resolve
+ * \param rr_type The type of record to look up
+ * \param rr_class The type of class to look up
+ * \param transport The transport to use for any resulting records
+ * \param port The port to use for any resulting records - if not specified the
+ *             default for the transport is used
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+static int sip_resolve_add(struct sip_resolve *resolve, const char *name, int rr_type, int rr_class, pjsip_transport_type_e transport, int port)
+{
+	struct sip_target target = {
+		.transport = transport,
+		.port = port,
+	};
+
+	if (!resolve->queries) {
+		resolve->queries = ast_dns_query_set_create();
+		if (!resolve->queries) {
+			return -1;
+		}
+	}
+
+	if (!port) {
+		target.port = pjsip_transport_get_default_port_for_type(transport);
+	}
+
+	if (AST_VECTOR_APPEND(&resolve->resolving, target)) {
+		return -1;
+	}
+
+	ast_debug(2, "[%p] Added target '%s' with record type '%d', transport '%s', and port '%d'\n",
+		resolve, name, rr_type, pjsip_transport_get_type_name(transport), target.port);
+
+	return ast_dns_query_set_add(resolve->queries, name, rr_type, rr_class);
+}
+
+/*!
+ * \internal
+ * \brief Task used to invoke the user specific callback
+ *
+ * \param data The complete resolution
+ *
+ * \return Nothing
+ */
+static int sip_resolve_invoke_user_callback(void *data)
+{
+	struct sip_resolve *resolve = data;
+
+	if (DEBUG_ATLEAST(2)) {
+		/* This includes space for the IP address, [, ], :, and the port */
+		char addr[PJ_INET6_ADDRSTRLEN + 10];
+		int idx;
+
+		for (idx = 0; idx < resolve->addresses.count; ++idx) {
+			pj_sockaddr_print(&resolve->addresses.entry[idx].addr, addr, sizeof(addr), 3);
+			ast_log(LOG_DEBUG, "[%p] Address '%d' is %s with transport '%s'\n",
+				resolve, idx, addr,
+				pjsip_transport_get_type_name(resolve->addresses.entry[idx].type));
+		}
+	}
+
+	ast_debug(2, "[%p] Invoking user callback with '%d' addresses\n", resolve, resolve->addresses.count);
+	resolve->callback(resolve->addresses.count ? PJ_SUCCESS : PJLIB_UTIL_EDNSNOANSWERREC, resolve->token, &resolve->addresses);
+
+	ao2_ref(resolve, -1);
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Handle a NAPTR record according to RFC3263
+ *
+ * \param resolve The ongoing resolution
+ * \param record The NAPTR record itself
+ * \param service The service to look for
+ * \param transport The transport to use for resulting queries
+ *
+ * \retval 0 success
+ * \retval -1 failure (record not handled / supported)
+ */
+static int sip_resolve_handle_naptr(struct sip_resolve *resolve, const struct ast_dns_record *record,
+	const char *service, pjsip_transport_type_e transport)
+{
+	if (strcasecmp(ast_dns_naptr_get_service(record), service)) {
+		return -1;
+	}
+
+	/* It is possible for us to receive an explicit transport that is already IPv6, in that case
+	 * we can't turn it into an IPv6 transport and check. If it's not IPv6 though we need to check
+	 * for both IPv4 and IPv6 as PJSIP does not provide enough differentiation to know that we
+	 * want only IPv4.
+	 */
+	if (!sip_transport_is_available(transport) &&
+		(!(transport & PJSIP_TRANSPORT_IPV6) && !sip_transport_is_available(transport + PJSIP_TRANSPORT_IPV6))) {
+		ast_debug(2, "[%p] NAPTR service %s skipped as transport is unavailable\n",
+			resolve, service);
+		return -1;
+	}
+
+	if (strcasecmp(ast_dns_naptr_get_flags(record), "s")) {
+		ast_debug(2, "[%p] NAPTR service %s received with unsupported flags '%s'\n",
+			resolve, service, ast_dns_naptr_get_flags(record));
+		return -1;
+	}
+
+	if (ast_strlen_zero(ast_dns_naptr_get_replacement(record))) {
+		return -1;
+	}
+
+	return sip_resolve_add(resolve, ast_dns_naptr_get_replacement(record), T_SRV, C_IN,
+		transport, 0);
+}
+
+/*!
+ * \internal
+ * \brief Query set callback function, invoked when all queries have completed
+ *
+ * \param query_set The completed query set
+ *
+ * \return Nothing
+ */
+static void sip_resolve_callback(const struct ast_dns_query_set *query_set)
+{
+	struct sip_resolve *resolve = ast_dns_query_set_get_data(query_set);
+	struct ast_dns_query_set *queries = resolve->queries;
+	struct targets resolving;
+	int idx, address_count = 0, have_naptr = 0, have_srv = 0;
+	unsigned short order = 0;
+	int strict_order = 0;
+
+	ast_debug(2, "[%p] All parallel queries completed\n", resolve);
+
+	resolve->queries = NULL;
+
+	/* This purposely steals the resolving list so we can add entries to the new one in
+	 * the same loop and also have access to the old.
+	 */
+	resolving = resolve->resolving;
+	AST_VECTOR_INIT(&resolve->resolving, 0);
+
+	/* The order of queries is what defines the preference order for the records within
+	 * this specific query set. The preference order overall is defined as a result of
+	 * drilling down from other records. Each completed query set replaces the results
+	 * of the last.
+	 */
+	for (idx = 0; idx < ast_dns_query_set_num_queries(queries); ++idx) {
+		struct ast_dns_query *query = ast_dns_query_set_get(queries, idx);
+		struct ast_dns_result *result = ast_dns_query_get_result(query);
+		struct sip_target *target;
+		const struct ast_dns_record *record;
+
+		if (!result) {
+			ast_debug(2, "[%p] No result information for target '%s' of type '%d'\n", resolve,
+				ast_dns_query_get_name(query), ast_dns_query_get_rr_type(query));
+			continue;
+		}
+
+		target = AST_VECTOR_GET_ADDR(&resolving, idx);
+		for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+
+			if (ast_dns_record_get_rr_type(record) == T_A ||
+				ast_dns_record_get_rr_type(record) == T_AAAA) {
+				/* If NAPTR or SRV records exist the subsequent results from them take preference */
+				if (have_naptr || have_srv) {
+					ast_debug(2, "[%p] %s record being skipped on target '%s' because NAPTR or SRV record exists\n",
+						resolve, ast_dns_record_get_rr_type(record) == T_A ? "A" : "AAAA",
+						ast_dns_query_get_name(query));
+					continue;
+				}
+
+				/* PJSIP has a fixed maximum number of addresses that can exist, so limit ourselves to that */
+				if (address_count == PJSIP_MAX_RESOLVED_ADDRESSES) {
+					break;
+				}
+
+				resolve->addresses.entry[address_count].type = target->transport;
+
+				/* Populate address information for the new address entry */
+				if (ast_dns_record_get_rr_type(record) == T_A) {
+					ast_debug(2, "[%p] A record received on target '%s'\n", resolve, ast_dns_query_get_name(query));
+					resolve->addresses.entry[address_count].addr_len = sizeof(pj_sockaddr_in);
+					pj_sockaddr_init(pj_AF_INET(), &resolve->addresses.entry[address_count].addr, NULL,
+						target->port);
+					resolve->addresses.entry[address_count].addr.ipv4.sin_addr = *(pj_in_addr *) ast_dns_record_get_data(record);
+				} else {
+					ast_debug(2, "[%p] AAAA record received on target '%s'\n", resolve, ast_dns_query_get_name(query));
+					resolve->addresses.entry[address_count].addr_len = sizeof(pj_sockaddr_in6);
+					pj_sockaddr_init(pj_AF_INET6(), &resolve->addresses.entry[address_count].addr, NULL,
+						target->port);
+					pj_memcpy(&resolve->addresses.entry[address_count].addr.ipv6.sin6_addr, ast_dns_record_get_data(record),
+						ast_dns_record_get_data_size(record));
+				}
+
+				address_count++;
+			} else if (ast_dns_record_get_rr_type(record) == T_SRV) {
+				if (have_naptr) {
+					ast_debug(2, "[%p] SRV record being skipped on target '%s' because NAPTR record exists\n",
+						resolve, ast_dns_query_get_name(query));
+					continue;
+				}
+
+				/* SRV records just create new queries for AAAA+A, nothing fancy */
+				ast_debug(2, "[%p] SRV record received on target '%s'\n", resolve, ast_dns_query_get_name(query));
+
+				/* If an explicit IPv6 target transport has been requested look for only AAAA records */
+				if (target->transport & PJSIP_TRANSPORT_IPV6) {
+					sip_resolve_add(resolve, ast_dns_srv_get_host(record), T_AAAA, C_IN, target->transport,
+						ast_dns_srv_get_port(record));
+					have_srv = 1;
+				} else if (sip_transport_is_available(target->transport + PJSIP_TRANSPORT_IPV6)) {
+					sip_resolve_add(resolve, ast_dns_srv_get_host(record), T_AAAA, C_IN, target->transport + PJSIP_TRANSPORT_IPV6,
+						ast_dns_srv_get_port(record));
+					have_srv = 1;
+				}
+
+				if (!(target->transport & PJSIP_TRANSPORT_IPV6) &&
+					sip_transport_is_available(target->transport)) {
+					sip_resolve_add(resolve, ast_dns_srv_get_host(record), T_A, C_IN, target->transport,
+						ast_dns_srv_get_port(record));
+					have_srv = 1;
+				}
+			} else if (ast_dns_record_get_rr_type(record) == T_NAPTR) {
+				int added = -1;
+
+				ast_debug(2, "[%p] NAPTR record received on target '%s'\n", resolve, ast_dns_query_get_name(query));
+
+				if (strict_order && (ast_dns_naptr_get_order(record) != order)) {
+					ast_debug(2, "[%p] NAPTR record skipped because order '%hu' does not match strict order '%hu'\n",
+						resolve, ast_dns_naptr_get_order(record), order);
+					continue;
+				}
+
+				if (target->transport == PJSIP_TRANSPORT_UNSPECIFIED || target->transport == PJSIP_TRANSPORT_UDP ||
+					target->transport == PJSIP_TRANSPORT_UDP6) {
+					added = sip_resolve_handle_naptr(resolve, record, "sip+d2u",
+						target->transport == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_UDP : target->transport);
+				}
+				if (target->transport == PJSIP_TRANSPORT_UNSPECIFIED || target->transport == PJSIP_TRANSPORT_TCP ||
+					target->transport == PJSIP_TRANSPORT_TCP6) {
+					added = sip_resolve_handle_naptr(resolve, record, "sip+d2t",
+						target->transport == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_TCP : target->transport);
+				}
+				if (target->transport == PJSIP_TRANSPORT_UNSPECIFIED || target->transport == PJSIP_TRANSPORT_TLS ||
+					target->transport == PJSIP_TRANSPORT_TLS6) {
+					added = sip_resolve_handle_naptr(resolve, record, "sips+d2t",
+						target->transport == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_TLS : target->transport);
+				}
+
+				/* If this record was successfully handled then we need to limit ourselves to this order */
+				if (!added) {
+					have_naptr = 1;
+					strict_order = 1;
+					order = ast_dns_naptr_get_order(record);
+				}
+			}
+		}
+	}
+
+	/* Update the server addresses count, this is not limited as it can never exceed the max allowed */
+	resolve->addresses.count = address_count;
+
+	/* Free the vector we stole as we are responsible for it */
+	AST_VECTOR_FREE(&resolving);
+
+	/* If additional queries were added start the resolution process again */
+	if (resolve->queries) {
+		ast_debug(2, "[%p] New queries added, performing parallel resolution again\n", resolve);
+		ast_dns_query_set_resolve_async(resolve->queries, sip_resolve_callback, resolve);
+		ao2_ref(queries, -1);
+		return;
+	}
+
+	ast_debug(2, "[%p] Resolution completed - %d viable targets\n", resolve, resolve->addresses.count);
+
+	/* Push a task to invoke the callback, we do this so it is guaranteed to run in a PJSIP thread */
+	ao2_ref(resolve, +1);
+	if (ast_sip_push_task(resolve->serializer, sip_resolve_invoke_user_callback, resolve)) {
+		ao2_ref(resolve, -1);
+	}
+
+	ao2_ref(queries, -1);
+}
+
+/*!
+ * \internal
+ * \brief Determine what address family a host may be if it is already an IP address
+ *
+ * \param host The host (which may be an IP address)
+ *
+ * \retval 6 The host is an IPv6 address
+ * \retval 4 The host is an IPv4 address
+ * \retval 0 The host is not an IP address
+ */
+static int sip_resolve_get_ip_addr_ver(const pj_str_t *host)
+{
+	pj_in_addr dummy;
+	pj_in6_addr dummy6;
+
+	if (pj_inet_aton(host, &dummy) > 0) {
+		return 4;
+	}
+
+	if (pj_inet_pton(pj_AF_INET6(), host, &dummy6) == PJ_SUCCESS) {
+		return 6;
+	}
+
+	return 0;
+}
+
+/*!
+ * \internal
+ * \brief Perform SIP resolution of a host
+ *
+ * \param resolver Configured resolver instance
+ * \param pool Memory pool to allocate things from
+ * \param target The target we are resolving
+ * \param token User data to pass to the resolver callback
+ * \param cb User resolver callback to invoke upon resolution completion
+ */
+static void sip_resolve(pjsip_resolver_t *resolver, pj_pool_t *pool, const pjsip_host_info *target,
+	void *token, pjsip_resolver_callback *cb)
+{
+	int ip_addr_ver;
+	pjsip_transport_type_e type = target->type;
+	struct sip_resolve *resolve;
+	char host[NI_MAXHOST];
+	int res = 0;
+
+	ast_copy_pj_str(host, &target->addr.host, sizeof(host));
+
+	ast_debug(2, "Performing SIP DNS resolution of target '%s'\n", host);
+
+	/* If the provided target is already an address don't bother resolving */
+	ip_addr_ver = sip_resolve_get_ip_addr_ver(&target->addr.host);
+
+	/* Determine the transport to use if none has been explicitly specified */
+	if (type == PJSIP_TRANSPORT_UNSPECIFIED) {
+		/* If we've been told to use a secure or reliable transport restrict ourselves to that */
+#if PJ_HAS_TCP
+		if (target->flag & PJSIP_TRANSPORT_SECURE) {
+			type = PJSIP_TRANSPORT_TLS;
+		} else if (target->flag & PJSIP_TRANSPORT_RELIABLE) {
+			type = PJSIP_TRANSPORT_TCP;
+		} else
+#endif
+		/* According to the RFC otherwise if an explicit IP address OR an explicit port is specified
+		 * we use UDP
+		 */
+		if (ip_addr_ver || target->addr.port) {
+			type = PJSIP_TRANSPORT_UDP;
+		}
+
+		if (ip_addr_ver == 6) {
+			type = (pjsip_transport_type_e)((int) type + PJSIP_TRANSPORT_IPV6);
+		}
+	}
+
+	ast_debug(2, "Transport type for target '%s' is '%s'\n", host, pjsip_transport_get_type_name(type));
+
+	/* If it's already an address call the callback immediately */
+	if (ip_addr_ver) {
+		pjsip_server_addresses addresses = {
+			.entry[0].type = type,
+			.count = 1,
+		};
+
+		if (ip_addr_ver == 4) {
+			addresses.entry[0].addr_len = sizeof(pj_sockaddr_in);
+			pj_sockaddr_init(pj_AF_INET(), &addresses.entry[0].addr, NULL, 0);
+			pj_inet_aton(&target->addr.host, &addresses.entry[0].addr.ipv4.sin_addr);
+		} else {
+			addresses.entry[0].addr_len = sizeof(pj_sockaddr_in6);
+			pj_sockaddr_init(pj_AF_INET6(), &addresses.entry[0].addr, NULL, 0);
+			pj_inet_pton(pj_AF_INET6(), &target->addr.host, &addresses.entry[0].addr.ipv6.sin6_addr);
+		}
+
+		pj_sockaddr_set_port(&addresses.entry[0].addr, !target->addr.port ? pjsip_transport_get_default_port_for_type(type) : target->addr.port);
+
+		ast_debug(2, "Target '%s' is an IP address, skipping resolution\n", host);
+
+		cb(PJ_SUCCESS, token, &addresses);
+
+		return;
+	}
+
+	resolve = ao2_alloc_options(sizeof(*resolve), sip_resolve_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!resolve) {
+		cb(PJ_ENOMEM, token, NULL);
+		return;
+	}
+
+	resolve->callback = cb;
+	resolve->token = token;
+
+	if (AST_VECTOR_INIT(&resolve->resolving, 4)) {
+		ao2_ref(resolve, -1);
+		cb(PJ_ENOMEM, token, NULL);
+		return;
+	}
+
+	ast_debug(2, "[%p] Created resolution tracking for target '%s'\n", resolve, host);
+
+	/* If no port has been specified we can do NAPTR + SRV */
+	if (!target->addr.port) {
+		char srv[NI_MAXHOST];
+
+		/* When resolving addresses PJSIP can request an explicit transport type. It will explicitly
+		 * request an IPv6 transport if a message has been tagged to use an explicitly IPv6 transport.
+		 * For other cases it can be left unspecified OR an explicit non-IPv6 transport can be requested.
+		 * In the case where a non-IPv6 transport is requested there is no way to differentiate between
+		 * a transport being requested as part of a SIP URI (sip:test.com;transport=tcp) and a message
+		 * being tagged with a specific IPv4 transport. In this case we look for both IPv4 and IPv6 addresses.
+		 * If a message has been tagged with a specific IPv4 transport the IPv6 addresses will simply
+		 * be discarded. The code below and elsewhere handles the case where we know they requested IPv6
+		 * explicitly and only looks for IPv6 records.
+		 */
+
+		res |= sip_resolve_add(resolve, host, T_NAPTR, C_IN, type, 0);
+
+		if (type == PJSIP_TRANSPORT_UNSPECIFIED ||
+			(type == PJSIP_TRANSPORT_TLS && sip_transport_is_available(PJSIP_TRANSPORT_TLS)) ||
+			(type == PJSIP_TRANSPORT_TLS6 && sip_transport_is_available(PJSIP_TRANSPORT_TLS6))) {
+			if (snprintf(srv, sizeof(srv), "_sips._tcp.%s", host) < NI_MAXHOST) {
+				res |= sip_resolve_add(resolve, srv, T_SRV, C_IN,
+					type == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_TLS : type, 0);
+			}
+		}
+		if (type == PJSIP_TRANSPORT_UNSPECIFIED ||
+			(type == PJSIP_TRANSPORT_TCP && sip_transport_is_available(PJSIP_TRANSPORT_TCP)) ||
+			(type == PJSIP_TRANSPORT_TCP6 && sip_transport_is_available(PJSIP_TRANSPORT_TCP6))) {
+			if (snprintf(srv, sizeof(srv), "_sip._tcp.%s", host) < NI_MAXHOST) {
+				res |= sip_resolve_add(resolve, srv, T_SRV, C_IN,
+					type == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_TCP : type, 0);
+			}
+		}
+		if (type == PJSIP_TRANSPORT_UNSPECIFIED ||
+			(type == PJSIP_TRANSPORT_UDP && sip_transport_is_available(PJSIP_TRANSPORT_UDP)) ||
+			(type == PJSIP_TRANSPORT_UDP6 && sip_transport_is_available(PJSIP_TRANSPORT_UDP6))) {
+			if (snprintf(srv, sizeof(srv), "_sip._udp.%s", host) < NI_MAXHOST) {
+				res |= sip_resolve_add(resolve, srv, T_SRV, C_IN,
+					type == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_UDP : type, 0);
+			}
+		}
+	}
+
+	if ((type == PJSIP_TRANSPORT_UNSPECIFIED && sip_transport_is_available(PJSIP_TRANSPORT_UDP6)) ||
+		sip_transport_is_available(type + PJSIP_TRANSPORT_IPV6)) {
+		res |= sip_resolve_add(resolve, host, T_AAAA, C_IN, (type == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_UDP6 : type + PJSIP_TRANSPORT_IPV6), target->addr.port);
+	}
+
+	if ((type == PJSIP_TRANSPORT_UNSPECIFIED && sip_transport_is_available(PJSIP_TRANSPORT_UDP)) ||
+		sip_transport_is_available(type)) {
+		res |= sip_resolve_add(resolve, host, T_A, C_IN, (type == PJSIP_TRANSPORT_UNSPECIFIED ? PJSIP_TRANSPORT_UDP : type), target->addr.port);
+	}
+
+	if (res) {
+		ao2_ref(resolve, -1);
+		cb(PJ_ENOMEM, token, NULL);
+		return;
+	}
+	if (!resolve->queries) {
+		ast_debug(2, "[%p] No resolution queries for target '%s'\n", resolve, host);
+		ao2_ref(resolve, -1);
+		cb(PJLIB_UTIL_EDNSNOANSWERREC, token, NULL);
+		return;
+	}
+
+	resolve->serializer = ao2_bump(ast_threadpool_serializer_get_current());
+
+	ast_debug(2, "[%p] Starting initial resolution using parallel queries for target '%s'\n", resolve, host);
+	ast_dns_query_set_resolve_async(resolve->queries, sip_resolve_callback, resolve);
+
+	ao2_ref(resolve, -1);
+}
+
+/*!
+ * \internal
+ * \brief Determine if a specific transport is configured on the system
+ *
+ * \param pool A memory pool to allocate things from
+ * \param transport The type of transport to check
+ * \param name A friendly name to print in the verbose message
+ *
+ * \return Nothing
+ */
+static void sip_check_transport(pj_pool_t *pool, pjsip_transport_type_e transport, const char *name)
+{
+	pjsip_tpmgr_fla2_param prm;
+	enum sip_resolver_transport resolver_transport;
+
+	pjsip_tpmgr_fla2_param_default(&prm);
+	prm.tp_type = transport;
+
+	if (transport == PJSIP_TRANSPORT_UDP) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_UDP;
+	} else if (transport == PJSIP_TRANSPORT_TCP) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TCP;
+	} else if (transport == PJSIP_TRANSPORT_TLS) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TLS;
+	} else if (transport == PJSIP_TRANSPORT_UDP6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_UDP6;
+	} else if (transport == PJSIP_TRANSPORT_TCP6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TCP6;
+	} else if (transport == PJSIP_TRANSPORT_TLS6) {
+		resolver_transport = SIP_RESOLVER_TRANSPORT_TLS6;
+	} else {
+		ast_verb(2, "'%s' is an unsupported SIP transport\n", name);
+		return;
+	}
+
+	if (pjsip_tpmgr_find_local_addr2(pjsip_endpt_get_tpmgr(ast_sip_get_pjsip_endpoint()),
+		pool, &prm) == PJ_SUCCESS) {
+		ast_verb(2, "'%s' is an available SIP transport\n", name);
+		sip_available_transports[resolver_transport] = 1;
+	} else {
+		ast_verb(2, "'%s' is not an available SIP transport, disabling resolver support for it\n",
+			name);
+	}
+}
+
+/*! \brief External resolver implementation for PJSIP */
+static pjsip_ext_resolver resolver = {
+	.resolve = sip_resolve,
+};
+
+/*!
+ * \internal
+ * \brief Task to determine available transports and set ourselves an external resolver
+ *
+ * \retval 0 success
+ * \retval -1 failure
+ */
+static int sip_replace_resolver(void *data)
+{
+	pj_pool_t *pool;
+
+
+	pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "Transport Availability", 256, 256);
+	if (!pool) {
+		return -1;
+	}
+
+	/* Determine what transports are available on the system */
+	sip_check_transport(pool, PJSIP_TRANSPORT_UDP, "UDP+IPv4");
+	sip_check_transport(pool, PJSIP_TRANSPORT_TCP, "TCP+IPv4");
+	sip_check_transport(pool, PJSIP_TRANSPORT_TLS, "TLS+IPv4");
+	sip_check_transport(pool, PJSIP_TRANSPORT_UDP6, "UDP+IPv6");
+	sip_check_transport(pool, PJSIP_TRANSPORT_TCP6, "TCP+IPv6");
+	sip_check_transport(pool, PJSIP_TRANSPORT_TLS6, "TLS+IPv6");
+
+	pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+
+	/* Replace the PJSIP resolver with our own implementation */
+	pjsip_endpt_set_ext_resolver(ast_sip_get_pjsip_endpoint(), &resolver);
+	return 0;
+}
+
+void ast_sip_initialize_resolver(void)
+{
+	/* Replace the existing PJSIP resolver with our own implementation */
+	ast_sip_push_task_wait_servant(NULL, sip_replace_resolver, NULL);
+}
+
+#else
+
+void ast_sip_initialize_resolver(void)
+{
+	/* External resolver support does not exist in the version of PJSIP in use */
+	ast_log(LOG_NOTICE, "The version of PJSIP in use does not support external resolvers, using PJSIP provided resolver\n");
+}
+
+#endif
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_scheduler.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_scheduler.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_scheduler.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_scheduler.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include "asterisk/res_pjsip.h"
 #include "include/res_pjsip_private.h"
 #include "asterisk/res_pjsip_cli.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_session.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_session.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_session.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_session.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,7 +32,7 @@
 
 AST_RWLIST_HEAD_STATIC(session_supplements, ast_sip_session_supplement);
 
-void internal_sip_session_register_supplement(struct ast_sip_session_supplement *supplement)
+void ast_sip_session_register_supplement(struct ast_sip_session_supplement *supplement)
 {
 	struct ast_sip_session_supplement *iter;
 	int inserted = 0;
@@ -56,37 +56,18 @@ void internal_sip_session_register_suppl
 	}
 }
 
-int ast_sip_session_register_supplement(struct ast_sip_session_supplement *supplement)
-{
-	internal_sip_session_register_supplement(supplement);
-	internal_res_pjsip_ref();
-
-	return 0;
-}
-
-int internal_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement)
+void ast_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement)
 {
 	struct ast_sip_session_supplement *iter;
-	int res = -1;
 	SCOPED_LOCK(lock, &session_supplements, AST_RWLIST_WRLOCK, AST_RWLIST_UNLOCK);
 
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&session_supplements, iter, next) {
 		if (supplement == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			res = 0;
 			break;
 		}
 	}
 	AST_RWLIST_TRAVERSE_SAFE_END;
-
-	return res;
-}
-
-void ast_sip_session_unregister_supplement(struct ast_sip_session_supplement *supplement)
-{
-	if (!internal_sip_session_unregister_supplement(supplement)) {
-		internal_res_pjsip_unref();
-	}
 }
 
 static struct ast_sip_session_supplement *supplement_dup(const struct ast_sip_session_supplement *src)
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_transport_management.c 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_transport_management.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/pjsip_transport_management.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/pjsip_transport_management.c	2018-12-26 18:45:13.000000000 +0000
@@ -354,8 +354,8 @@ int ast_sip_initialize_transport_managem
 {
 	struct ao2_container *transports;
 
-	transports = ao2_container_alloc(TRANSPORTS_BUCKETS, monitored_transport_hash_fn,
-		monitored_transport_cmp_fn);
+	transports = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, TRANSPORTS_BUCKETS,
+		monitored_transport_hash_fn, NULL, monitored_transport_cmp_fn);
 	if (!transports) {
 		ast_log(LOG_ERROR, "Could not create container for transports to perform keepalive on.\n");
 		return AST_MODULE_LOAD_DECLINE;
@@ -378,7 +378,7 @@ int ast_sip_initialize_transport_managem
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	internal_sip_register_service(&idle_monitor_module);
+	ast_sip_register_service(&idle_monitor_module);
 
 	ast_sip_transport_state_register(&monitored_transport_reg);
 
@@ -403,7 +403,7 @@ void ast_sip_destroy_transport_managemen
 
 	ast_sip_transport_state_unregister(&monitored_transport_reg);
 
-	internal_sip_unregister_service(&idle_monitor_module);
+	ast_sip_unregister_service(&idle_monitor_module);
 
 	ast_sched_clean_by_callback(sched, idle_sched_cb, idle_sched_cleanup);
 	ast_sched_context_destroy(sched);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip/security_events.c 1:16.1.1~dfsg-1/res/res_pjsip/security_events.c
--- 1:13.23.1~dfsg-2/res/res_pjsip/security_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip/security_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 
 #include "asterisk/res_pjsip.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_acl.c 1:16.1.1~dfsg-1/res/res_pjsip_acl.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_acl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_acl.c	2018-12-26 18:45:13.000000000 +0000
@@ -238,8 +238,21 @@ static int acl_handler(const struct aco_
 
 	if (!strncmp(var->name, "contact_", 8)) {
 		ast_append_acl(var->name + 8, var->value, &sip_acl->contact_acl, &error, &ignore);
+		if (error) {
+			ast_log(LOG_ERROR, "Bad contact ACL '%s' at line '%d' of pjsip.conf\n",
+					var->value, var->lineno);
+		}
 	} else {
 		ast_append_acl(var->name, var->value, &sip_acl->acl, &error, &ignore);
+		if (error) {
+			ast_log(LOG_ERROR, "Bad ACL '%s' at line '%d' of pjsip.conf\n",
+					var->value, var->lineno);
+		}
+	}
+
+	if (error) {
+		ast_log(LOG_ERROR, "There is an error in ACL configuration. Blocking ALL SIP traffic.\n");
+		ast_append_acl("deny", "0.0.0.0/0.0.0.0", &sip_acl->acl, NULL, &ignore);
 	}
 
 	return error;
@@ -269,8 +282,6 @@ static void *acl_alloc(const char *name)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	ast_sorcery_apply_config(ast_sip_get_sorcery(), SIP_SORCERY_ACL_TYPE);
 	ast_sorcery_apply_default(ast_sip_get_sorcery(), SIP_SORCERY_ACL_TYPE,
 				  "config", "pjsip.conf,criteria=type=acl");
@@ -304,8 +315,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP ACL Resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_authenticator_digest.c 1:16.1.1~dfsg-1/res/res_pjsip_authenticator_digest.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_authenticator_digest.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_authenticator_digest.c	2018-12-26 18:45:13.000000000 +0000
@@ -524,8 +524,6 @@ static int reload_module(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (build_entity_id()) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -549,9 +547,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP authentication resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND - 5,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 5,
+	.requires = "res_pjsip",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip.c 1:16.1.1~dfsg-1/res/res_pjsip.c
--- 1:13.23.1~dfsg-2/res/res_pjsip.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip.c	2018-12-26 18:45:13.000000000 +0000
@@ -189,6 +189,12 @@
 						</enumlist>
 					</description>
 				</configOption>
+				<configOption name="trust_connected_line">
+					<synopsis>Accept Connected Line updates from this endpoint</synopsis>
+				</configOption>
+				<configOption name="send_connected_line">
+					<synopsis>Send Connected Line updates to this endpoint</synopsis>
+				</configOption>
 				<configOption name="connected_line_method" default="invite">
 					<synopsis>Connected line method type</synopsis>
 					<description>
@@ -738,6 +744,9 @@
 				<configOption name="user_eq_phone" default="no">
 					<synopsis>Determines whether a user=phone parameter is placed into the request URI if the user is determined to be a phone number</synopsis>
 				</configOption>
+				<configOption name="moh_passthrough" default="no">
+					<synopsis>Determines whether hold and unhold will be passed through using re-INVITEs with recvonly and sendrecv to the remote side</synopsis>
+				</configOption>
 				<configOption name="sdp_owner" default="-">
 					<synopsis>String placed as the username portion of an SDP origin (o=) line.</synopsis>
 				</configOption>
@@ -799,6 +808,18 @@
 						If this is not set or the value provided is 0 rekeying will be disabled.
 					</para></description>
 				</configOption>
+				<configOption name="dtls_auto_generate_cert" default="no">
+					<synopsis>Whether or not to automatically generate an ephemeral X.509 certificate</synopsis>
+					<description>
+						<para>
+							If enabled, Asterisk will generate an X.509 certificate for each DTLS session.
+							This option only applies if <replaceable>media_encryption</replaceable> is set
+							to <literal>dtls</literal>. This option will be automatically enabled if
+							<literal>webrtc</literal> is enabled and <literal>dtls_cert_file</literal> is
+							not specified.
+						</para>
+					</description>
+				</configOption>
 				<configOption name="dtls_cert_file">
 					<synopsis>Path to certificate file to present to peer</synopsis>
 					<description><para>
@@ -900,12 +921,15 @@
 						have this accountcode set on it.
 					</para></description>
 				</configOption>
+				<configOption name="preferred_codec_only" default="no">
+					<synopsis>Respond to a SIP invite with the single most preferred codec rather than advertising all joint codec capabilities. This limits the other side's codec choice to exactly what we prefer.</synopsis>
+				</configOption>
 				<configOption name="rtp_keepalive">
 					<synopsis>Number of seconds between RTP comfort noise keepalive packets.</synopsis>
 					<description><para>
 						At the specified interval, Asterisk will send an RTP comfort noise frame. This may
-						be useful for situations where Asterisk is behind a NAT or firewall and must keep a
-						hole open in order to allow for media to arrive at Asterisk.
+						be useful for situations where Asterisk is behind a NAT or firewall and must keep
+						a hole open in order to allow for media to arrive at Asterisk.
 					</para></description>
 				</configOption>
 				<configOption name="rtp_timeout" default="0">
@@ -1022,6 +1046,41 @@
 						on Ringing when already INUSE.
 					</para></description>
 				</configOption>
+				<configOption name="max_audio_streams" default="1">
+					<synopsis>The maximum number of allowed audio streams for the endpoint</synopsis>
+					<description><para>
+						This option enforces a limit on the maximum simultaneous negotiated audio
+						streams allowed for the endpoint.
+					</para></description>
+				</configOption>
+				<configOption name="max_video_streams" default="1">
+					<synopsis>The maximum number of allowed video streams for the endpoint</synopsis>
+					<description><para>
+						This option enforces a limit on the maximum simultaneous negotiated video
+						streams allowed for the endpoint.
+					</para></description>
+				</configOption>
+				<configOption name="bundle" default="no">
+					<synopsis>Enable RTP bundling</synopsis>
+					<description><para>
+						With this option enabled, Asterisk will attempt to negotiate the use of bundle.
+						If negotiated this will result in multiple RTP streams being carried over the same
+						underlying transport. Note that enabling bundle will also enable the rtcp_mux option.
+					</para></description>
+				</configOption>
+				<configOption name="webrtc" default="no">
+					<synopsis>Defaults and enables some options that are relevant to WebRTC</synopsis>
+					<description><para>
+						When set to "yes" this also enables the following values that are needed in
+						order for basic WebRTC support to work: rtcp_mux, use_avpf, ice_support, and
+						use_received_transport. The following configuration settings also get defaulted
+						as follows:</para>
+						<para>media_encryption=dtls</para>
+						<para>dtls_auto_generate_cert=yes (if dtls_cert_file is not set)</para>
+						<para>dtls_verify=fingerprint</para>
+						<para>dtls_setup=actpass</para>
+					</description>
+				</configOption>
 				<configOption name="incoming_mwi_mailbox">
 					<synopsis>Mailbox name to use when incoming MWI NOTIFYs are received</synopsis>
 					<description><para>
@@ -1727,7 +1786,7 @@
 					<synopsis>The voicemail extension to send in the NOTIFY Message-Account header if not specified on endpoint or aor</synopsis>
 				</configOption>
 				<configOption name="debug" default="no">
-					<synopsis>Enable/Disable SIP debug logging.  Valid options include yes|no or
+					<synopsis>Enable/Disable SIP debug logging.  Valid options include yes, no, or
 						a host address</synopsis>
 				</configOption>
 				<configOption name="endpoint_identifier_order">
@@ -1828,6 +1887,13 @@
 						at the first semicolon.</para></note>
 					</description>
 				</configOption>
+				<configOption name="use_callerid_contact" default="no">
+					<synopsis>Place caller-id information into Contact header</synopsis>
+					<description><para>
+						This option will cause Asterisk to place caller-id information into
+						generated Contact headers.</para>
+					</description>
+				</configOption>
 			</configObject>
 		</configFile>
 	</configInfo>
@@ -2120,6 +2186,12 @@
 				<parameter name="DirectMediaMethod">
 					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='direct_media_method']/synopsis/node())"/></para>
 				</parameter>
+				<parameter name="TrustConnectedLine">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='trust_connected_line']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="SendConnectedLine">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='send_connected_line']/synopsis/node())"/></para>
+				</parameter>
 				<parameter name="ConnectedLineMethod">
 					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='connected_line_method']/synopsis/node())"/></para>
 				</parameter>
@@ -2231,6 +2303,9 @@
 				<parameter name="UserEqPhone">
 					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='user_eq_phone']/synopsis/node())"/></para>
 				</parameter>
+				<parameter name="MohPassthrough">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='moh_passthrough']/synopsis/node())"/></para>
+				</parameter>
 				<parameter name="SdpOwner">
 					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='sdp_owner']/synopsis/node())"/></para>
 				</parameter>
@@ -2306,6 +2381,9 @@
 				<parameter name="Accountcode">
 					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='accountcode']/synopsis/node())"/></para>
 				</parameter>
+				<parameter name="PreferredCodecOnly">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='endpoint']/configOption[@name='preferred_codec_only']/synopsis/node())"/></para>
+				</parameter>
 				<parameter name="DeviceState">
 					<para>The aggregate device state for this endpoint.</para>
 				</parameter>
@@ -2321,6 +2399,151 @@
 			</syntax>
 		</managerEventInstance>
 	</managerEvent>
+	<managerEvent language="en_US" name="AorList">
+		<managerEventInstance class="EVENT_FLAG_COMMAND">
+			<synopsis>Provide details about an Address of Record (AoR) section.</synopsis>
+			<syntax>
+				<parameter name="ObjectType">
+					<para>The object's type. This will always be 'aor'.</para>
+				</parameter>
+				<parameter name="ObjectName">
+					<para>The name of this object.</para>
+				</parameter>
+				<parameter name="MinimumExpiration">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='minimum_expiration']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="MaximumExpiration">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='maximum_expiration']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="DefaultExpiration">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='default_expiration']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="QualifyFrequency">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='qualify_frequency']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="AuthenticateQualify">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='authenticate_qualify']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="MaxContacts">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='max_contacts']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="RemoveExisting">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='remove_existing']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="Mailboxes">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='mailboxes']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="OutboundProxy">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='outbound_proxy']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="SupportPath">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='aor']/configOption[@name='support_path']/synopsis/node())"/></para>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
+	<managerEvent language="en_US" name="AuthList">
+		<managerEventInstance class="EVENT_FLAG_COMMAND">
+			<synopsis>Provide details about an Address of Record (Auth) section.</synopsis>
+			<syntax>
+				<parameter name="ObjectType">
+					<para>The object's type. This will always be 'auth'.</para>
+				</parameter>
+				<parameter name="ObjectName">
+					<para>The name of this object.</para>
+				</parameter>
+				<parameter name="Username">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='username']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="Md5Cred">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='md5_cred']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="Realm">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='realm']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="AuthType">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='auth_type']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="Password">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='password']/synopsis/node())"/></para>
+				</parameter>
+				<parameter name="NonceLifetime">
+					<para><xi:include xpointer="xpointer(/docs/configInfo[@name='res_pjsip']/configFile[@name='pjsip.conf']/configObject[@name='auth']/configOption[@name='nonce_lifetime']/synopsis/node())"/></para>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
+	<managerEvent language="en_US" name="ContactList">
+		<managerEventInstance class="EVENT_FLAG_COMMAND">
+			<synopsis>Provide details about a contact section.</synopsis>
+			<syntax>
+				<parameter name="ObjectType">
+					<para>The object's type. This will always be 'contact'.</para>
+				</parameter>
+				<parameter name="ObjectName">
+					<para>The name of this object.</para>
+				</parameter>
+				<parameter name="ViaAddr">
+					<para>IP address of the last Via header in REGISTER request.
+					Will only appear in the event if available.</para>
+				</parameter>
+				<parameter name="ViaPort">
+					<para>Port number of the last Via header in REGISTER request.
+					Will only appear in the event if available.</para>
+				</parameter>
+				<parameter name="QualifyTimeout">
+					<para>The elapsed time in decimal seconds after which an OPTIONS
+					message is sent before the contact is considered unavailable.</para>
+				</parameter>
+				<parameter name="CallId">
+					<para>Content of the Call-ID header in REGISTER request.
+					Will only appear in the event if available.</para>
+				</parameter>
+				<parameter name="RegServer">
+					<para>Asterisk Server name.</para>
+				</parameter>
+				<parameter name="PruneOnBoot">
+					<para>If true delete the contact on Asterisk restart/boot.</para>
+				</parameter>
+				<parameter name="Path">
+					<para>The Path header received on the REGISTER.</para>
+				</parameter>
+				<parameter name="Endpoint">
+					<para>The name of the endpoint associated with this information.</para>
+				</parameter>
+				<parameter name="AuthenticateQualify">
+					<para>A boolean indicating whether a qualify should be authenticated.</para>
+				</parameter>
+				<parameter name="Uri">
+					<para>This contact's URI.</para>
+				</parameter>
+				<parameter name="QualifyFrequency">
+					<para>The interval in seconds at which the contact will be qualified.</para>
+				</parameter>
+				<parameter name="UserAgent">
+					<para>Content of the User-Agent header in REGISTER request</para>
+				</parameter>
+				<parameter name="ExpirationTime">
+					<para>Absolute time that this contact is no longer valid after</para>
+				</parameter>
+				<parameter name="OutboundProxy">
+					<para>The contact's outbound proxy.</para>
+				</parameter>
+				<parameter name="Status">
+					<para>This contact's status.</para>
+					<enumlist>
+						<enum name="Reachable"/>
+						<enum name="Unreachable"/>
+						<enum name="NonQualified"/>
+						<enum name="Unknown"/>
+					</enumlist>
+				</parameter>
+				<parameter name="RoundtripUsec">
+					<para>The round trip time in microseconds.</para>
+				</parameter>
+			</syntax>
+		</managerEventInstance>
+	</managerEvent>
 	<managerEvent language="en_US" name="ContactStatusDetail">
 		<managerEventInstance class="EVENT_FLAG_COMMAND">
 			<synopsis>Provide details about a contact's status.</synopsis>
@@ -2336,6 +2559,8 @@
 					<enumlist>
 						<enum name="Reachable"/>
 						<enum name="Unreachable"/>
+						<enum name="NonQualified"/>
+						<enum name="Unknown"/>
 					</enumlist>
 				</parameter>
 				<parameter name="RoundtripUsec">
@@ -2479,6 +2704,87 @@
 			</managerEvent>
 		</responses>
 	</manager>
+	<manager name="PJSIPShowAors" language="en_US">
+		<synopsis>
+			Lists PJSIP AORs.
+		</synopsis>
+		<syntax />
+		<description>
+			<para>
+			Provides a listing of all AORs. For each AOR an <literal>AorList</literal> event
+			is raised that contains relevant attributes and status information.  Once all
+			aors have been listed an <literal>AorListComplete</literal> event is issued.
+			</para>
+		</description>
+		<responses>
+			<list-elements>
+				<xi:include xpointer="xpointer(/docs/managerEvent[@name='AorList'])" />
+			</list-elements>
+			<managerEvent language="en_US" name="AorListComplete">
+				<managerEventInstance class="EVENT_FLAG_COMMAND">
+					<synopsis>Provide final information about an aor list.</synopsis>
+					<syntax>
+						<parameter name="EventList"/>
+						<parameter name="ListItems"/>
+					</syntax>
+				</managerEventInstance>
+			</managerEvent>
+		</responses>
+	</manager>
+	<manager name="PJSIPShowAuths" language="en_US">
+		<synopsis>
+			Lists PJSIP Auths.
+		</synopsis>
+		<syntax />
+		<description>
+			<para>Provides a listing of all Auths. For each Auth an <literal>AuthList</literal> event
+			is raised that contains relevant attributes and status information.  Once all
+			auths have been listed an <literal>AuthListComplete</literal> event is issued.
+			</para>
+		</description>
+		<responses>
+			<list-elements>
+				<xi:include xpointer="xpointer(/docs/managerEvent[@name='AuthList'])" />
+			</list-elements>
+			<managerEvent language="en_US" name="AuthListComplete">
+				<managerEventInstance class="EVENT_FLAG_COMMAND">
+					<synopsis>Provide final information about an auth list.</synopsis>
+					<syntax>
+						<parameter name="EventList"/>
+						<parameter name="ListItems"/>
+					</syntax>
+				</managerEventInstance>
+			</managerEvent>
+		</responses>
+	</manager>
+	<manager name="PJSIPShowContacts" language="en_US">
+		<synopsis>
+			Lists PJSIP Contacts.
+		</synopsis>
+		<syntax />
+		<description>
+			<para>Provides a listing of all Contacts. For each Contact a <literal>ContactList</literal>
+			event is raised that contains relevant attributes and status information.
+			Once all contacts have been listed a <literal>ContactListComplete</literal> event
+			is issued.
+			</para>
+		</description>
+		<responses>
+			<list-elements>
+				<xi:include xpointer="xpointer(/docs/managerEvent[@name='ContactList'])" />
+			</list-elements>
+			<managerEvent language="en_US" name="ContactListComplete">
+				<managerEventInstance class="EVENT_FLAG_COMMAND">
+					<synopsis>Provide final information about a contact list.</synopsis>
+					<syntax>
+						<parameter name="EventList"/>
+						<parameter name="ListItems"/>
+					</syntax>
+				</managerEventInstance>
+			</managerEvent>
+		</responses>
+	</manager>
+
  ***/
 
 #define MOD_DATA_CONTACT "contact"
@@ -2486,9 +2792,6 @@
 /*! Number of serializers in pool if one not supplied. */
 #define SERIALIZER_POOL_SIZE		8
 
-/*! Next serializer pool index to use. */
-static int serializer_pool_pos;
-
 /*! Pool of serializers to use if not supplied. */
 static struct ast_taskprocessor *serializer_pool[SERIALIZER_POOL_SIZE];
 
@@ -2508,7 +2811,7 @@ static pj_sockaddr host_ip_ipv6;
 /*! Local host address for IPv6 (string form) */
 static char host_ip_ipv6_string[PJ_INET6_ADDRSTRLEN];
 
-static int register_service_noref(void *data)
+static int register_service(void *data)
 {
 	pjsip_module **module = data;
 	if (!ast_pjsip_endpoint) {
@@ -2523,28 +2826,12 @@ static int register_service_noref(void *
 	return 0;
 }
 
-static int register_service(void *data)
-{
-	int res;
-
-	if (!(res = register_service_noref(data))) {
-		ast_module_ref(ast_module_info->self);
-	}
-
-	return res;
-}
-
-int internal_sip_register_service(pjsip_module *module)
-{
-	return ast_sip_push_task_wait_servant(NULL, register_service_noref, &module);
-}
-
 int ast_sip_register_service(pjsip_module *module)
 {
 	return ast_sip_push_task_wait_servant(NULL, register_service, &module);
 }
 
-static int unregister_service_noref(void *data)
+static int unregister_service(void *data)
 {
 	pjsip_module **module = data;
 	if (!ast_pjsip_endpoint) {
@@ -2555,22 +2842,6 @@ static int unregister_service_noref(void
 	return 0;
 }
 
-static int unregister_service(void *data)
-{
-	int res;
-
-	if (!(res = unregister_service_noref(data))) {
-		ast_module_unref(ast_module_info->self);
-	}
-
-	return res;
-}
-
-int internal_sip_unregister_service(pjsip_module *module)
-{
-	return ast_sip_push_task_wait_servant(NULL, unregister_service_noref, &module);
-}
-
 void ast_sip_unregister_service(pjsip_module *module)
 {
 	ast_sip_push_task_wait_servant(NULL, unregister_service, &module);
@@ -2586,7 +2857,7 @@ int ast_sip_register_authenticator(struc
 	}
 	registered_authenticator = auth;
 	ast_debug(1, "Registered SIP authenticator module %p\n", auth);
-	ast_module_ref(ast_module_info->self);
+
 	return 0;
 }
 
@@ -2599,7 +2870,6 @@ void ast_sip_unregister_authenticator(st
 	}
 	registered_authenticator = NULL;
 	ast_debug(1, "Unregistered SIP authenticator %p\n", auth);
-	ast_module_unref(ast_module_info->self);
 }
 
 int ast_sip_requires_authentication(struct ast_sip_endpoint *endpoint, pjsip_rx_data *rdata)
@@ -2632,7 +2902,7 @@ int ast_sip_register_outbound_authentica
 	}
 	registered_outbound_authenticator = auth;
 	ast_debug(1, "Registered SIP outbound authenticator module %p\n", auth);
-	ast_module_ref(ast_module_info->self);
+
 	return 0;
 }
 
@@ -2645,27 +2915,16 @@ void ast_sip_unregister_outbound_authent
 	}
 	registered_outbound_authenticator = NULL;
 	ast_debug(1, "Unregistered SIP outbound authenticator %p\n", auth);
-	ast_module_unref(ast_module_info->self);
 }
 
 int ast_sip_create_request_with_auth(const struct ast_sip_auth_vector *auths, pjsip_rx_data *challenge,
-		pjsip_transaction *tsx, pjsip_tx_data **new_request)
-{
-	if (!registered_outbound_authenticator) {
-		ast_log(LOG_WARNING, "No SIP outbound authenticator registered. Cannot respond to authentication challenge\n");
-		return -1;
-	}
-	return registered_outbound_authenticator->create_request_with_auth(auths, challenge, tsx, new_request);
-}
-
-int ast_sip_create_request_with_auth_from_old(const struct ast_sip_auth_vector *auths, pjsip_rx_data *challenge,
 		pjsip_tx_data *old_request, pjsip_tx_data **new_request)
 {
 	if (!registered_outbound_authenticator) {
 		ast_log(LOG_WARNING, "No SIP outbound authenticator registered. Cannot respond to authentication challenge\n");
 		return -1;
 	}
-	return registered_outbound_authenticator->create_request_with_auth_from_old(auths, challenge, old_request, new_request);
+	return registered_outbound_authenticator->create_request_with_auth(auths, challenge, old_request, new_request);
 }
 
 struct endpoint_identifier_list {
@@ -2697,7 +2956,6 @@ int ast_sip_register_endpoint_identifier
 	if (ast_strlen_zero(name)) {
 		/* if an identifier has no name then place in front */
 		AST_RWLIST_INSERT_HEAD(&endpoint_identifiers, id_list_item, list);
-		ast_module_ref(ast_module_info->self);
 		return 0;
 	}
 
@@ -2707,7 +2965,6 @@ int ast_sip_register_endpoint_identifier
 	if (ast_strlen_zero(identifier_order)) {
 		id_list_item->priority = UINT_MAX;
 		AST_RWLIST_INSERT_TAIL(&endpoint_identifiers, id_list_item, list);
-		ast_module_ref(ast_module_info->self);
 		ast_free(identifier_order);
 		return 0;
 	}
@@ -2735,7 +2992,6 @@ int ast_sip_register_endpoint_identifier
 		/* if not in the endpoint_identifier_order list then consider it less in
 		   priority and add it to the end */
 		AST_RWLIST_INSERT_TAIL(&endpoint_identifiers, id_list_item, list);
-		ast_module_ref(ast_module_info->self);
 		ast_free(identifier_order);
 		return 0;
 	}
@@ -2753,7 +3009,6 @@ int ast_sip_register_endpoint_identifier
 	}
 	AST_RWLIST_TRAVERSE_SAFE_END;
 
-	ast_module_ref(ast_module_info->self);
 	ast_free(identifier_order);
 	return 0;
 }
@@ -2772,7 +3027,6 @@ void ast_sip_unregister_endpoint_identif
 			AST_RWLIST_REMOVE_CURRENT(list);
 			ast_free(iter);
 			ast_debug(1, "Unregistered endpoint identifier %p\n", identifier);
-			ast_module_unref(ast_module_info->self);
 			break;
 		}
 	}
@@ -2915,23 +3169,17 @@ static struct ast_cli_entry cli_commands
 
 AST_RWLIST_HEAD_STATIC(endpoint_formatters, ast_sip_endpoint_formatter);
 
-void internal_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
+void ast_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
 {
 	SCOPED_LOCK(lock, &endpoint_formatters, AST_RWLIST_WRLOCK, AST_RWLIST_UNLOCK);
 	AST_RWLIST_INSERT_TAIL(&endpoint_formatters, obj, next);
 }
 
-int ast_sip_register_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
-{
-	internal_sip_register_endpoint_formatter(obj);
-	ast_module_ref(ast_module_info->self);
-	return 0;
-}
-
-int internal_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
+void ast_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
 {
 	struct ast_sip_endpoint_formatter *i;
 	SCOPED_LOCK(lock, &endpoint_formatters, AST_RWLIST_WRLOCK, AST_RWLIST_UNLOCK);
+
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&endpoint_formatters, i, next) {
 		if (i == obj) {
 			AST_RWLIST_REMOVE_CURRENT(next);
@@ -2939,14 +3187,6 @@ int internal_sip_unregister_endpoint_for
 		}
 	}
 	AST_RWLIST_TRAVERSE_SAFE_END;
-	return i == obj ? 0 : -1;
-}
-
-void ast_sip_unregister_endpoint_formatter(struct ast_sip_endpoint_formatter *obj)
-{
-	if (!internal_sip_unregister_endpoint_formatter(obj)) {
-		ast_module_unref(ast_module_info->self);
-	}
 }
 
 int ast_sip_format_endpoint_ami(struct ast_sip_endpoint *endpoint,
@@ -3450,6 +3690,8 @@ int ast_sip_create_rdata_with_contact(pj
 	ast_copy_string(rdata->pkt_info.packet, packet, sizeof(rdata->pkt_info.packet));
 	ast_copy_string(rdata->pkt_info.src_name, src_name, sizeof(rdata->pkt_info.src_name));
 	rdata->pkt_info.src_port = src_port;
+	pj_sockaddr_parse(pj_AF_UNSPEC(), 0, pj_cstr(&tmp, src_name), &rdata->pkt_info.src_addr);
+	pj_sockaddr_set_port(&rdata->pkt_info.src_addr, src_port);
 
 	pjsip_parse_rdata(packet, strlen(packet), rdata);
 	if (!rdata->msg_info.msg || !pj_list_empty(&rdata->msg_info.parse_err)) {
@@ -3661,7 +3903,7 @@ int ast_sip_create_request(const char *m
 
 AST_RWLIST_HEAD_STATIC(supplements, ast_sip_supplement);
 
-void internal_sip_register_supplement(struct ast_sip_supplement *supplement)
+void ast_sip_register_supplement(struct ast_sip_supplement *supplement)
 {
 	struct ast_sip_supplement *iter;
 	int inserted = 0;
@@ -3681,37 +3923,18 @@ void internal_sip_register_supplement(st
 	}
 }
 
-int ast_sip_register_supplement(struct ast_sip_supplement *supplement)
-{
-	internal_sip_register_supplement(supplement);
-	ast_module_ref(ast_module_info->self);
-
-	return 0;
-}
-
-int internal_sip_unregister_supplement(struct ast_sip_supplement *supplement)
+void ast_sip_unregister_supplement(struct ast_sip_supplement *supplement)
 {
 	struct ast_sip_supplement *iter;
 	SCOPED_LOCK(lock, &supplements, AST_RWLIST_WRLOCK, AST_RWLIST_UNLOCK);
-	int res = -1;
 
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&supplements, iter, next) {
 		if (supplement == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			res = 0;
 			break;
 		}
 	}
 	AST_RWLIST_TRAVERSE_SAFE_END;
-
-	return res;
-}
-
-void ast_sip_unregister_supplement(struct ast_sip_supplement *supplement)
-{
-	if (!internal_sip_unregister_supplement(supplement)) {
-		ast_module_unref(ast_module_info->self);
-	}
 }
 
 static int send_in_dialog_request(pjsip_tx_data *tdata, struct pjsip_dialog *dlg)
@@ -3980,8 +4203,6 @@ static pj_status_t endpt_send_request(st
 		}
 	}
 
-	ast_sip_record_request_serializer(tdata);
-
 	/* We need to insure that the wrapper and tdata are available when the
 	 * transaction callback is executed.
 	 */
@@ -4044,59 +4265,120 @@ static pj_status_t endpt_send_request(st
 	return ret_val;
 }
 
-static void send_request_cb(void *token, pjsip_event *e)
+int ast_sip_failover_request(pjsip_tx_data *tdata)
 {
-	struct send_request_data *req_data = token;
+	pjsip_via_hdr *via;
+
+	if (!tdata || !tdata->dest_info.addr.count
+		|| (tdata->dest_info.cur_addr == tdata->dest_info.addr.count - 1)) {
+		/* No more addresses to try */
+		return 0;
+	}
+
+	/* Try next address */
+	++tdata->dest_info.cur_addr;
+
+	via = (pjsip_via_hdr*)pjsip_msg_find_hdr(tdata->msg, PJSIP_H_VIA, NULL);
+	via->branch_param.slen = 0;
+
+	pjsip_tx_data_invalidate_msg(tdata);
+
+	return 1;
+}
+
+static void send_request_cb(void *token, pjsip_event *e);
+
+static int check_request_status(struct send_request_data *req_data, pjsip_event *e)
+{
+	struct ast_sip_endpoint *endpoint;
 	pjsip_transaction *tsx;
-	pjsip_rx_data *challenge;
 	pjsip_tx_data *tdata;
-	struct ast_sip_supplement *supplement;
-	struct ast_sip_endpoint *endpoint;
-	int res;
+	int res = 0;
 
-	switch(e->body.tsx_state.type) {
-	case PJSIP_EVENT_TRANSPORT_ERROR:
-	case PJSIP_EVENT_TIMER:
-		break;
-	case PJSIP_EVENT_RX_MSG:
-		challenge = e->body.tsx_state.src.rdata;
+	if (!(endpoint = ao2_bump(req_data->endpoint))) {
+		return 0;
+	}
 
-		/*
-		 * Call any supplements that want to know about a response
-		 * with any received data.
-		 */
-		AST_RWLIST_RDLOCK(&supplements);
-		AST_LIST_TRAVERSE(&supplements, supplement, next) {
-			if (supplement->incoming_response
-				&& does_method_match(&challenge->msg_info.cseq->method.name,
-					supplement->method)) {
-				supplement->incoming_response(req_data->endpoint, challenge);
-			}
-		}
-		AST_RWLIST_UNLOCK(&supplements);
+	tsx = e->body.tsx_state.tsx;
 
+	switch (tsx->status_code) {
+	case 401:
+	case 407:
 		/* Resend the request with a challenge response if we are challenged. */
-		tsx = e->body.tsx_state.tsx;
-		endpoint = ao2_bump(req_data->endpoint);
-		res = (tsx->status_code == 401 || tsx->status_code == 407)
-			&& endpoint
-			&& ++req_data->challenge_count < MAX_RX_CHALLENGES /* Not in a challenge loop */
+		res = ++req_data->challenge_count < MAX_RX_CHALLENGES /* Not in a challenge loop */
 			&& !ast_sip_create_request_with_auth(&endpoint->outbound_auths,
-				challenge, tsx, &tdata)
-			&& endpt_send_request(endpoint, tdata, -1, req_data, send_request_cb)
-				== PJ_SUCCESS;
-		ao2_cleanup(endpoint);
-		if (res) {
+				e->body.tsx_state.src.rdata, tsx->last_tx, &tdata);
+		break;
+	case 408:
+	case 503:
+		if ((res = ast_sip_failover_request(tsx->last_tx))) {
+			tdata = tsx->last_tx;
 			/*
-			 * Request with challenge response sent.
-			 * Passed our req_data ref to the new request.
+			 * Bump the ref since it will be on a new transaction and
+			 * we don't want it to go away along with the old transaction.
 			 */
-			return;
+			pjsip_tx_data_add_ref(tdata);
 		}
 		break;
-	default:
-		ast_log(LOG_ERROR, "Unexpected PJSIP event %u\n", e->body.tsx_state.type);
-		break;
+	}
+
+	if (res) {
+		res = endpt_send_request(endpoint, tdata, -1,
+					 req_data, send_request_cb) == PJ_SUCCESS;
+	}
+
+	ao2_ref(endpoint, -1);
+	return res;
+}
+
+static void send_request_cb(void *token, pjsip_event *e)
+{
+	struct send_request_data *req_data = token;
+	pjsip_rx_data *challenge;
+	struct ast_sip_supplement *supplement;
+
+	if (e->type == PJSIP_EVENT_TSX_STATE) {
+		switch(e->body.tsx_state.type) {
+		case PJSIP_EVENT_TRANSPORT_ERROR:
+		case PJSIP_EVENT_TIMER:
+			/*
+			 * Check the request status on transport error or timeout. A transport
+			 * error can occur when a TCP socket closes and that can be the result
+			 * of a 503. Also we may need to failover on a timeout (408).
+			 */
+			if (check_request_status(req_data, e)) {
+				return;
+			}
+			break;
+		case PJSIP_EVENT_RX_MSG:
+			challenge = e->body.tsx_state.src.rdata;
+
+			/*
+			 * Call any supplements that want to know about a response
+			 * with any received data.
+			 */
+			AST_RWLIST_RDLOCK(&supplements);
+			AST_LIST_TRAVERSE(&supplements, supplement, next) {
+				if (supplement->incoming_response
+					&& does_method_match(&challenge->msg_info.cseq->method.name,
+						supplement->method)) {
+					supplement->incoming_response(req_data->endpoint, challenge);
+				}
+			}
+			AST_RWLIST_UNLOCK(&supplements);
+
+			if (check_request_status(req_data, e)) {
+				/*
+				 * Request with challenge response or failover sent.
+				 * Passed our req_data ref to the new request.
+				 */
+				return;
+			}
+			break;
+		default:
+			ast_log(LOG_ERROR, "Unexpected PJSIP event %u\n", e->body.tsx_state.type);
+			break;
+		}
 	}
 
 	if (req_data->callback) {
@@ -4236,34 +4518,14 @@ int ast_sip_append_body(pjsip_tx_data *t
 	return 0;
 }
 
-struct ast_taskprocessor *ast_sip_create_serializer_group_named(const char *name, struct ast_serializer_shutdown_group *shutdown_group)
+struct ast_taskprocessor *ast_sip_create_serializer_group(const char *name, struct ast_serializer_shutdown_group *shutdown_group)
 {
 	return ast_threadpool_serializer_group(name, sip_threadpool, shutdown_group);
 }
 
-struct ast_taskprocessor *ast_sip_create_serializer_group(struct ast_serializer_shutdown_group *shutdown_group)
-{
-	char tps_name[AST_TASKPROCESSOR_MAX_NAME + 1];
-
-	/* Create name with seq number appended. */
-	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip-group-serializer");
-
-	return ast_sip_create_serializer_group_named(tps_name, shutdown_group);
-}
-
-struct ast_taskprocessor *ast_sip_create_serializer_named(const char *name)
+struct ast_taskprocessor *ast_sip_create_serializer(const char *name)
 {
-	return ast_sip_create_serializer_group_named(name, NULL);
-}
-
-struct ast_taskprocessor *ast_sip_create_serializer(void)
-{
-	char tps_name[AST_TASKPROCESSOR_MAX_NAME + 1];
-
-	/* Create name with seq number appended. */
-	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip-serializer");
-
-	return ast_sip_create_serializer_group_named(tps_name, NULL);
+	return ast_sip_create_serializer_group(name, NULL);
 }
 
 /*!
@@ -4300,7 +4562,7 @@ static int serializer_pool_setup(void)
 		/* Create name with seq number appended. */
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/default");
 
-		serializer_pool[idx] = ast_sip_create_serializer_named(tps_name);
+		serializer_pool[idx] = ast_sip_create_serializer(tps_name);
 		if (!serializer_pool[idx]) {
 			serializer_pool_shutdown();
 			return -1;
@@ -4311,22 +4573,20 @@ static int serializer_pool_setup(void)
 
 static struct ast_taskprocessor *serializer_pool_pick(void)
 {
-	struct ast_taskprocessor *serializer;
+	int idx;
+	int pos = 0;
 
-	unsigned int pos;
+	if (!serializer_pool[0]) {
+		return NULL;
+	}
 
-	/*
-	 * Pick a serializer to use from the pool.
-	 *
-	 * Note: We don't care about any reentrancy behavior
-	 * when incrementing serializer_pool_pos.  If it gets
-	 * incorrectly incremented it doesn't matter.
-	 */
-	pos = serializer_pool_pos++;
-	pos %= SERIALIZER_POOL_SIZE;
-	serializer = serializer_pool[pos];
+	for (idx = 1; idx < SERIALIZER_POOL_SIZE; ++idx) {
+		if (ast_taskprocessor_size(serializer_pool[idx]) < ast_taskprocessor_size(serializer_pool[pos])) {
+			pos = idx;
+		}
+	}
 
-	return serializer;
+	return serializer_pool[pos];
 }
 
 int ast_sip_push_task(struct ast_taskprocessor *serializer, int (*sip_task)(void *), void *task_data)
@@ -4438,6 +4698,18 @@ void ast_copy_pj_str(char *dest, const p
 	dest[chars_to_copy] = '\0';
 }
 
+int ast_copy_pj_str2(char **dest, const pj_str_t *src)
+{
+	int res = ast_asprintf(dest, "%.*s", (int)pj_strlen(src), pj_strbuf(src));
+
+	if (res < 0) {
+		*dest = NULL;
+	}
+
+	return res;
+}
+
+
 int ast_sip_is_content_type(pjsip_media_type *content_type, char *type, char *subtype)
 {
 	pjsip_media_type compare;
@@ -4824,16 +5096,6 @@ static int reload_configuration_task(voi
 	return 0;
 }
 
-void internal_res_pjsip_ref(void)
-{
-	ast_module_ref(ast_module_info->self);
-}
-
-void internal_res_pjsip_unref(void)
-{
-	ast_module_unref(ast_module_info->self);
-}
-
 static int unload_pjsip(void *data)
 {
 	/*
@@ -4842,14 +5104,13 @@ static int unload_pjsip(void *data)
 	 */
 	if (ast_pjsip_endpoint && serializer_pool[0]) {
 		ast_res_pjsip_cleanup_options_handling();
-		internal_sip_destroy_outbound_authentication();
 		ast_res_pjsip_cleanup_message_filter();
 		ast_sip_destroy_distributor();
 		ast_sip_destroy_transport_management();
 		ast_res_pjsip_destroy_configuration();
 		ast_sip_destroy_system();
 		ast_sip_destroy_global_headers();
-		internal_sip_unregister_service(&supplement_module);
+		ast_sip_unregister_service(&supplement_module);
 		ast_sip_destroy_transport_events();
 	}
 
@@ -4949,8 +5210,6 @@ static int load_module(void)
 {
 	struct ast_threadpool_options options;
 
-	CHECK_PJPROJECT_MODULE_LOADED();
-
 	/* pjproject and config_system need to be initialized before all else */
 	if (pj_init() != PJ_SUCCESS) {
 		return AST_MODULE_LOAD_DECLINE;
@@ -5007,11 +5266,14 @@ static int load_module(void)
 		goto error;
 	}
 
-	if (ast_res_pjsip_initialize_configuration(ast_module_info)) {
+	if (ast_res_pjsip_initialize_configuration()) {
 		ast_log(LOG_ERROR, "Failed to initialize SIP configuration. Aborting load\n");
 		goto error;
 	}
 
+	ast_sip_initialize_resolver();
+	ast_sip_initialize_dns();
+
 	if (ast_sip_initialize_transport_management()) {
 		ast_log(LOG_ERROR, "Failed to initialize SIP transport management. Aborting load\n");
 		goto error;
@@ -5022,16 +5284,11 @@ static int load_module(void)
 		goto error;
 	}
 
-	if (internal_sip_register_service(&supplement_module)) {
+	if (ast_sip_register_service(&supplement_module)) {
 		ast_log(LOG_ERROR, "Failed to initialize supplement hooks. Aborting load\n");
 		goto error;
 	}
 
-	if (internal_sip_initialize_outbound_authentication()) {
-		ast_log(LOG_ERROR, "Failed to initialize outbound authentication. Aborting load\n");
-		goto error;
-	}
-
 	if (ast_res_pjsip_init_options_handling(0)) {
 		ast_log(LOG_ERROR, "Failed to initialize OPTIONS handling. Aborting load\n");
 		goto error;
@@ -5047,8 +5304,6 @@ static int load_module(void)
 	AST_TEST_REGISTER(xml_sanitization_end_null);
 	AST_TEST_REGISTER(xml_sanitization_exceeds_buffer);
 
-	ast_pjproject_ref();
-
 	return AST_MODULE_LOAD_SUCCESS;
 
 error:
@@ -5090,15 +5345,15 @@ static int unload_module(void)
 	serializer_pool_shutdown();
 	ast_threadpool_shutdown(sip_threadpool);
 
-	ast_pjproject_unref();
-
 	return 0;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Basic SIP resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND - 5,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 5,
+	.requires = "dnsmgr,res_pjproject",
+	.optional_modules = "res_statsd",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_caller_id.c 1:16.1.1~dfsg-1/res/res_pjsip_caller_id.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_caller_id.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_caller_id.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -342,7 +341,8 @@ static void update_incoming_connected_li
 {
 	struct ast_party_id id;
 
-	if (!session->endpoint->id.trust_inbound) {
+	if (!session->endpoint->trust_connected_line
+		|| !session->endpoint->id.trust_inbound) {
 		return;
 	}
 
@@ -430,8 +430,7 @@ static pjsip_fromto_hdr *create_new_id_h
 
 	id_hdr = pjsip_from_hdr_create(tdata->pool);
 	id_hdr->type = PJSIP_H_OTHER;
-	pj_strdup(tdata->pool, &id_hdr->name, hdr_name);
-	id_hdr->sname = id_hdr->name;
+	id_hdr->sname = id_hdr->name = *hdr_name;
 
 	id_name_addr = pjsip_uri_clone(tdata->pool, base->uri);
 	id_uri = pjsip_uri_get_uri(id_name_addr->uri);
@@ -750,7 +749,10 @@ static void caller_id_outgoing_response(
 	struct ast_party_id effective_id;
 	struct ast_party_id connected_id;
 
-	if (!session->channel) {
+	if (!session->channel
+		|| (!session->endpoint->send_connected_line
+			&& session->inv_session
+			&& session->inv_session->state >= PJSIP_INV_STATE_EARLY)) {
 		return;
 	}
 
@@ -776,8 +778,7 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
+	ast_module_shutdown_ref(AST_MODULE_SELF);
 	ast_sip_session_register_supplement(&caller_id_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -789,8 +790,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Caller ID Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_config_wizard.c 1:16.1.1~dfsg-1/res/res_pjsip_config_wizard.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_config_wizard.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_config_wizard.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 #include <pjsip.h>
 
@@ -476,7 +474,7 @@ static int add_extension(struct ast_cont
 	}
 
 	if (ast_add_extension2_nolock(context, 0, exten, priority, NULL, NULL,
-			app, data, free_ptr, BASE_REGISTRAR)) {
+			app, data, free_ptr, BASE_REGISTRAR, NULL, 0)) {
 		ast_free(data);
 		return -1;
 	}
@@ -1344,8 +1342,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "PJSIP Config Wizard",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_REALTIME_DRIVER,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_REALTIME_DRIVER,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_dialog_info_body_generator.c 1:16.1.1~dfsg-1/res/res_pjsip_dialog_info_body_generator.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_dialog_info_body_generator.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_dialog_info_body_generator.c	2018-12-26 18:45:13.000000000 +0000
@@ -20,7 +20,6 @@
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
 	<depend>res_pjsip_pubsub</depend>
-	<depend>res_pjsip_exten_state</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -61,20 +60,20 @@ static void *dialog_info_allocate_body(v
 	return ast_sip_presence_xml_create_node(state_data->pool, NULL, "dialog-info");
 }
 
-static struct ast_datastore *dialog_info_xml_state_find_or_create(struct ast_sip_subscription *sub)
+static struct ast_datastore *dialog_info_xml_state_find_or_create(struct ao2_container *datastores)
 {
-	struct ast_datastore *datastore = ast_sip_subscription_get_datastore(sub, "dialog-info+xml");
+	struct ast_datastore *datastore = ast_datastores_find(datastores, "dialog-info+xml");
 
 	if (datastore) {
 		return datastore;
 	}
 
-	datastore = ast_sip_subscription_alloc_datastore(&dialog_info_xml_datastore, "dialog-info+xml");
+	datastore = ast_datastores_alloc_datastore(&dialog_info_xml_datastore, "dialog-info+xml");
 	if (!datastore) {
 		return NULL;
 	}
 	datastore->data = ast_calloc(1, sizeof(struct dialog_info_xml_state));
-	if (!datastore->data || ast_sip_subscription_add_datastore(sub, datastore)) {
+	if (!datastore->data || ast_datastores_add(datastores, datastore)) {
 		ao2_ref(datastore, -1);
 		return NULL;
 	}
@@ -82,9 +81,9 @@ static struct ast_datastore *dialog_info
 	return datastore;
 }
 
-static unsigned int dialog_info_xml_get_version(struct ast_sip_subscription *sub, unsigned int *version)
+static unsigned int dialog_info_xml_get_version(struct ao2_container *datastores, unsigned int *version)
 {
-	struct ast_datastore *datastore = dialog_info_xml_state_find_or_create(sub);
+	struct ast_datastore *datastore = dialog_info_xml_state_find_or_create(datastores);
 	struct dialog_info_xml_state *state;
 
 	if (!datastore) {
@@ -110,11 +109,11 @@ static int dialog_info_generate_body_con
 	struct ast_sip_endpoint *endpoint = NULL;
 	unsigned int notify_early_inuse_ringing = 0;
 
-	if (!local || !state_data->sub) {
+	if (!local || !state_data->datastores) {
 		return -1;
 	}
 
-	if (dialog_info_xml_get_version(state_data->sub, &version)) {
+	if (dialog_info_xml_get_version(state_data->datastores, &version)) {
 		ast_log(LOG_WARNING, "dialog-info+xml version could not be retrieved from datastore\n");
 		return -1;
 	}
@@ -163,7 +162,7 @@ static int dialog_info_generate_body_con
 /* The maximum number of times the ast_str() for the body text can grow before we declare an XML body
  * too large to send.
  */
-#define MAX_STRING_GROWTHS 3
+#define MAX_STRING_GROWTHS 6
 
 static void dialog_info_to_string(void *body, struct ast_str **str)
 {
@@ -199,8 +198,6 @@ static struct ast_sip_pubsub_body_genera
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_generator(&dialog_info_body_generator)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -215,8 +212,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Extension State Dialog Info+XML Provider",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_diversion.c 1:16.1.1~dfsg-1/res/res_pjsip_diversion.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_diversion.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_diversion.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -412,8 +411,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	ast_sip_session_register_supplement(&diversion_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -425,8 +422,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Add Diversion Header Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_dlg_options.c 1:16.1.1~dfsg-1/res/res_pjsip_dlg_options.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_dlg_options.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_dlg_options.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,14 +19,11 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
 ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <pjsip_ua.h>
 #include <pjlib.h>
@@ -86,11 +83,8 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
+	ast_sip_session_register_supplement(&dlg_options_supplement);
 
-	if (ast_sip_session_register_supplement(&dlg_options_supplement)) {
-		return AST_MODULE_LOAD_DECLINE;
-	}
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -105,4 +99,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_dtmf_info.c 1:16.1.1~dfsg-1/res/res_pjsip_dtmf_info.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_dtmf_info.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_dtmf_info.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -161,8 +160,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	ast_sip_session_register_supplement(&dtmf_info_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -174,8 +171,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP DTMF INFO Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_empty_info.c 1:16.1.1~dfsg-1/res/res_pjsip_empty_info.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_empty_info.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_empty_info.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,14 +19,11 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <pjsip.h>
 #include <pjsip_ua.h>
 
@@ -69,8 +66,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	ast_sip_session_register_supplement(&empty_info_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -86,4 +81,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_anonymous.c 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_anonymous.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_anonymous.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_anonymous.c	2018-12-26 18:45:13.000000000 +0000
@@ -126,8 +126,6 @@ static struct ast_sip_endpoint_identifie
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	ast_sip_register_endpoint_identifier_with_name(&anonymous_identifier, "anonymous");
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -139,7 +137,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "PJSIP Anonymous endpoint identifier",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_ip.c 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_ip.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_ip.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_ip.c	2018-12-26 18:45:13.000000000 +0000
@@ -797,8 +797,6 @@ static struct ast_sip_cli_formatter_entr
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	ast_sorcery_apply_config(ast_sip_get_sorcery(), "res_pjsip_endpoint_identifier_ip");
 	ast_sorcery_apply_default(ast_sip_get_sorcery(), "identify", "config", "pjsip.conf,criteria=type=identify");
 
@@ -855,9 +853,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP IP endpoint identifier",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.reload = reload_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND - 4,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.reload = reload_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 4,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_user.c 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_user.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_endpoint_identifier_user.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_endpoint_identifier_user.c	2018-12-26 18:45:13.000000000 +0000
@@ -206,8 +206,6 @@ static struct ast_sip_endpoint_identifie
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	ast_sip_register_endpoint_identifier_with_name(&username_identifier, "username");
 	ast_sip_register_endpoint_identifier_with_name(&auth_username_identifier, "auth_username");
 	return AST_MODULE_LOAD_SUCCESS;
@@ -221,8 +219,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP username endpoint identifier",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND - 4,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 4,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip.exports.in 1:16.1.1~dfsg-1/res/res_pjsip.exports.in
--- 1:13.23.1~dfsg-2/res/res_pjsip.exports.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip.exports.in	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,9 @@
 {
 	global:
 		LINKER_SYMBOL_PREFIXast_sip_*;
+		LINKER_SYMBOL_PREFIX__ast_sip_*;
 		LINKER_SYMBOL_PREFIXast_copy_pj_str;
+		LINKER_SYMBOL_PREFIXast_copy_pj_str2;
 		LINKER_SYMBOL_PREFIXast_pjsip_rdata_get_endpoint;
 	local:
 		*;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_exten_state.c 1:16.1.1~dfsg-1/res/res_pjsip_exten_state.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_exten_state.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_exten_state.c	2018-12-26 18:45:13.000000000 +0000
@@ -20,16 +20,20 @@
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
 	<depend>res_pjsip_pubsub</depend>
+	<depend>res_pjsip_outbound_publish</depend>
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
+#include <regex.h>
+
 #include <pjsip.h>
 #include <pjsip_simple.h>
 #include <pjlib.h>
 
 #include "asterisk/res_pjsip.h"
+#include "asterisk/res_pjsip_outbound_publish.h"
 #include "asterisk/res_pjsip_pubsub.h"
 #include "asterisk/res_pjsip_body_generator_types.h"
 #include "asterisk/module.h"
@@ -43,6 +47,19 @@
 #define EVENT_TYPE_SIZE 50
 
 /*!
+ * \brief The number of buckets to use for storing publishers
+ */
+#define PUBLISHER_BUCKETS 31
+
+/*!
+ * \brief Container of active outbound extension state publishers
+ */
+static struct ao2_container *publishers;
+
+/*! Serializer for outbound extension state publishing. */
+static struct ast_taskprocessor *publish_exten_state_serializer;
+
+/*!
  * \brief A subscription for extension state
  *
  * This structure acts as the owner for the underlying SIP subscription. It
@@ -68,6 +85,31 @@ struct exten_state_subscription {
 	enum ast_presence_state last_presence_state;
 };
 
+/*!
+ * \brief An extension state publisher
+ *
+ */
+struct exten_state_publisher {
+	/*! Regular expression for context filtering */
+	regex_t context_regex;
+	/*! Regular expression for extension filtering */
+	regex_t exten_regex;
+	/*! Publish client to use for sending publish messages */
+	struct ast_sip_outbound_publish_client *client;
+	/*! Datastores container to hold persistent information */
+	struct ao2_container *datastores;
+	/*! Whether context filtering is active */
+	unsigned int context_filter;
+	/*! Whether extension filtering is active */
+	unsigned int exten_filter;
+	/*! The body type to use for this publisher - stored after the name */
+	char *body_type;
+	/*! The body subtype to use for this publisher - stored after the body type */
+	char *body_subtype;
+	/*! The name of this publisher */
+	char name[0];
+};
+
 #define DEFAULT_PRESENCE_BODY "application/pidf+xml"
 #define DEFAULT_DIALOG_BODY "application/dialog-info+xml"
 
@@ -77,6 +119,9 @@ static int subscription_established(stru
 static void *get_notify_data(struct ast_sip_subscription *sub);
 static void to_ami(struct ast_sip_subscription *sub,
 		   struct ast_str **buf);
+static int publisher_start(struct ast_sip_outbound_publish *configuration,
+			   struct ast_sip_outbound_publish_client *client);
+static int publisher_stop(struct ast_sip_outbound_publish_client *client);
 
 struct ast_sip_notifier presence_notifier = {
 	.default_accept = DEFAULT_PRESENCE_BODY,
@@ -101,6 +146,12 @@ struct ast_sip_subscription_handler pres
 	.notifier = &presence_notifier,
 };
 
+struct ast_sip_event_publisher_handler presence_publisher = {
+	.event_name = "presence",
+	.start_publishing = publisher_start,
+	.stop_publishing = publisher_stop,
+};
+
 struct ast_sip_subscription_handler dialog_handler = {
 	.event_name = "dialog",
 	.body_type = AST_SIP_EXTEN_STATE_DATA,
@@ -110,6 +161,12 @@ struct ast_sip_subscription_handler dial
 	.notifier = &dialog_notifier,
 };
 
+struct ast_sip_event_publisher_handler dialog_publisher = {
+	.event_name = "dialog",
+	.start_publishing = publisher_start,
+	.stop_publishing = publisher_stop,
+};
+
 static void exten_state_subscription_destructor(void *obj)
 {
 	struct exten_state_subscription *sub = obj;
@@ -191,8 +248,9 @@ static void notify_task_data_destructor(
 	ast_free(task_data->exten_state_data.user_agent);
 }
 
-static struct notify_task_data *alloc_notify_task_data(char *exten, struct exten_state_subscription *exten_state_sub,
-						       struct ast_state_cb_info *info)
+static struct notify_task_data *alloc_notify_task_data(const char *exten,
+	struct exten_state_subscription *exten_state_sub,
+	struct ast_state_cb_info *info)
 {
 	struct notify_task_data *task_data =
 		ao2_alloc(sizeof(*task_data), notify_task_data_destructor);
@@ -215,6 +273,7 @@ static struct notify_task_data *alloc_no
 	task_data->exten_state_data.user_agent = ast_strdup(exten_state_sub->user_agent);
 	task_data->exten_state_data.device_state_info = ao2_bump(info->device_state_info);
 	task_data->exten_state_data.sub = exten_state_sub->sip_sub;
+	task_data->exten_state_data.datastores = ast_sip_subscription_get_datastores(exten_state_sub->sip_sub);
 
 	if ((info->exten_state == AST_EXTENSION_DEACTIVATED) ||
 	    (info->exten_state == AST_EXTENSION_REMOVED)) {
@@ -255,6 +314,7 @@ static int notify_task(void *obj)
 	}
 
 	task_data->exten_state_data.sub = task_data->exten_state_sub->sip_sub;
+	task_data->exten_state_data.datastores = ast_sip_subscription_get_datastores(task_data->exten_state_sub->sip_sub);
 
 	ast_sip_subscription_notify(task_data->exten_state_sub->sip_sub, &data,
 			task_data->terminate);
@@ -270,8 +330,8 @@ static int notify_task(void *obj)
  *
  * Upon state change, send the appropriate notification to the subscriber.
  */
-static int state_changed(char *context, char *exten,
-			 struct ast_state_cb_info *info, void *data)
+static int state_changed(const char *context, const char *exten,
+	struct ast_state_cb_info *info, void *data)
 {
 	struct notify_task_data *task_data;
 	struct exten_state_subscription *exten_state_sub = data;
@@ -447,6 +507,7 @@ static struct ast_sip_exten_state_data *
 	ast_sip_subscription_get_remote_uri(sip_sub, exten_state_data->remote,
 			sizeof(exten_state_data->remote));
 	exten_state_data->sub = sip_sub;
+	exten_state_data->datastores = ast_sip_subscription_get_datastores(sip_sub);
 
 	exten_state_data->exten_state = ast_extension_state_extended(
 			NULL, exten_state_sub->context, exten_state_sub->exten,
@@ -494,31 +555,452 @@ static void to_ami(struct ast_sip_subscr
 			       exten_state_sub->last_exten_state));
 }
 
+struct exten_state_pub_data {
+	/*! Publishers needing state update */
+	AST_VECTOR(name, struct exten_state_publisher *) pubs;
+	/*! Body generator state data */
+	struct ast_sip_exten_state_data exten_state_data;
+};
+
+static void exten_state_pub_data_destroy(struct exten_state_pub_data *doomed)
+{
+	if (!doomed) {
+		return;
+	}
+
+	ast_free((void *) doomed->exten_state_data.exten);
+	ast_free(doomed->exten_state_data.presence_subtype);
+	ast_free(doomed->exten_state_data.presence_message);
+	ao2_cleanup(doomed->exten_state_data.device_state_info);
+
+	AST_VECTOR_CALLBACK_VOID(&doomed->pubs, ao2_ref, -1);
+	AST_VECTOR_FREE(&doomed->pubs);
+
+	ast_free(doomed);
+}
+
+static struct exten_state_pub_data *exten_state_pub_data_alloc(const char *exten, struct ast_state_cb_info *info)
+{
+	struct exten_state_pub_data *pub_data;
+
+	pub_data = ast_calloc(1, sizeof(*pub_data));
+	if (!pub_data) {
+		return NULL;
+	}
+
+	if (AST_VECTOR_INIT(&pub_data->pubs, ao2_container_count(publishers))) {
+		exten_state_pub_data_destroy(pub_data);
+		return NULL;
+	}
+
+	/* Save off currently known information for the body generators. */
+	pub_data->exten_state_data.exten = ast_strdup(exten);
+	pub_data->exten_state_data.exten_state = info->exten_state;
+	pub_data->exten_state_data.presence_state = info->presence_state;
+	pub_data->exten_state_data.presence_subtype = ast_strdup(info->presence_subtype);
+	pub_data->exten_state_data.presence_message = ast_strdup(info->presence_message);
+	pub_data->exten_state_data.device_state_info = ao2_bump(info->device_state_info);
+	if (!pub_data->exten_state_data.exten
+		|| !pub_data->exten_state_data.presence_subtype
+		|| !pub_data->exten_state_data.presence_message) {
+		exten_state_pub_data_destroy(pub_data);
+		return NULL;
+	}
+	return pub_data;
+}
+
+/*!
+ * \internal
+ * \brief Create exten state PUBLISH messages under PJSIP thread.
+ * \since 14.0.0
+ *
+ * \return 0
+ */
+static int exten_state_publisher_cb(void *data)
+{
+	struct exten_state_pub_data *pub_data = data;
+	struct exten_state_publisher *publisher;
+	size_t idx;
+	struct ast_str *body_text;
+	pj_pool_t *pool;
+	struct ast_sip_body_data gen_data = {
+		.body_type = AST_SIP_EXTEN_STATE_DATA,
+		.body_data = &pub_data->exten_state_data,
+	};
+	struct ast_sip_body body;
+
+	body_text = ast_str_create(64);
+	if (!body_text) {
+		exten_state_pub_data_destroy(pub_data);
+		return 0;
+	}
+
+	/* Need a PJSIP memory pool to generate the bodies. */
+	pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "pub_state_body",
+		1024, 1024);
+	if (!pool) {
+		ast_log(LOG_WARNING, "Exten state publishing unable to create memory pool\n");
+		exten_state_pub_data_destroy(pub_data);
+		ast_free(body_text);
+		return 0;
+	}
+	pub_data->exten_state_data.pool = pool;
+
+	for (idx = 0; idx < AST_VECTOR_SIZE(&pub_data->pubs); ++idx) {
+		const char *uri;
+		int res;
+
+		publisher = AST_VECTOR_GET(&pub_data->pubs, idx);
+
+		uri = ast_sip_publish_client_get_user_from_uri(publisher->client, pub_data->exten_state_data.exten,
+			pub_data->exten_state_data.local, sizeof(pub_data->exten_state_data.local));
+		if (ast_strlen_zero(uri)) {
+			ast_log(LOG_WARNING, "PUBLISH client '%s' has no from_uri or server_uri defined.\n",
+				publisher->name);
+			continue;
+		}
+
+		uri = ast_sip_publish_client_get_user_to_uri(publisher->client, pub_data->exten_state_data.exten,
+			pub_data->exten_state_data.remote, sizeof(pub_data->exten_state_data.remote));
+		if (ast_strlen_zero(uri)) {
+			ast_log(LOG_WARNING, "PUBLISH client '%s' has no to_uri or server_uri defined.\n",
+				publisher->name);
+			continue;
+		}
+
+		pub_data->exten_state_data.datastores = publisher->datastores;
+
+		res = ast_sip_pubsub_generate_body_content(publisher->body_type,
+			publisher->body_subtype, &gen_data, &body_text);
+		pj_pool_reset(pool);
+		if (res) {
+			ast_log(LOG_WARNING,
+				"PUBLISH client '%s' unable to generate %s/%s PUBLISH body.\n",
+				publisher->name, publisher->body_type, publisher->body_subtype);
+			continue;
+		}
+
+		body.type = publisher->body_type;
+		body.subtype = publisher->body_subtype;
+		body.body_text = ast_str_buffer(body_text);
+		ast_sip_publish_client_user_send(publisher->client, pub_data->exten_state_data.exten, &body);
+	}
+
+	pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+
+	ast_free(body_text);
+	exten_state_pub_data_destroy(pub_data);
+	return 0;
+}
+
+/*!
+ * \brief Global extension state callback function
+ */
+static int exten_state_publisher_state_cb(const char *context, const char *exten, struct ast_state_cb_info *info, void *data)
+{
+	struct ao2_iterator publisher_iter;
+	struct exten_state_publisher *publisher;
+	struct exten_state_pub_data *pub_data = NULL;
+
+	ast_debug(5, "Exten state publisher: %s@%s Reason:%s State:%s Presence:%s Subtype:'%s' Message:'%s'\n",
+		exten, context,
+		info->reason == AST_HINT_UPDATE_DEVICE
+			? "Device"
+			: info->reason == AST_HINT_UPDATE_PRESENCE
+				? "Presence"
+				: "Unknown",
+		ast_extension_state2str(info->exten_state),
+		ast_presence_state2str(info->presence_state),
+		S_OR(info->presence_subtype, ""),
+		S_OR(info->presence_message, ""));
+	publisher_iter = ao2_iterator_init(publishers, 0);
+	for (; (publisher = ao2_iterator_next(&publisher_iter)); ao2_ref(publisher, -1)) {
+		if ((publisher->context_filter && regexec(&publisher->context_regex, context, 0, NULL, 0)) ||
+		    (publisher->exten_filter && regexec(&publisher->exten_regex, exten, 0, NULL, 0))) {
+			continue;
+		}
+
+		if (!pub_data) {
+			pub_data = exten_state_pub_data_alloc(exten, info);
+			if (!pub_data) {
+				ao2_ref(publisher, -1);
+				break;
+			}
+		}
+
+		ao2_ref(publisher, +1);
+		if (AST_VECTOR_APPEND(&pub_data->pubs, publisher)) {
+			ao2_ref(publisher, -1);
+		} else {
+			ast_debug(5, "'%s' will publish exten state\n", publisher->name);
+		}
+	}
+	ao2_iterator_destroy(&publisher_iter);
+
+	if (pub_data
+		&& ast_sip_push_task(publish_exten_state_serializer, exten_state_publisher_cb,
+			pub_data)) {
+		exten_state_pub_data_destroy(pub_data);
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Hashing function for extension state publisher
+ */
+static int exten_state_publisher_hash(const void *obj, const int flags)
+{
+	const struct exten_state_publisher *object;
+	const char *key;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_KEY:
+		key = obj;
+		break;
+	case OBJ_SEARCH_OBJECT:
+		object = obj;
+		key = object->name;
+		break;
+	default:
+		ast_assert(0);
+		return 0;
+	}
+	return ast_str_hash(key);
+}
+
+/*!
+ * \brief Comparator function for extension state publisher
+ */
+static int exten_state_publisher_cmp(void *obj, void *arg, int flags)
+{
+	const struct exten_state_publisher *object_left = obj;
+	const struct exten_state_publisher *object_right = arg;
+	const char *right_key = arg;
+	int cmp;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_OBJECT:
+		right_key = object_right->name;
+		/* Fall through */
+	case OBJ_SEARCH_KEY:
+		cmp = strcmp(object_left->name, right_key);
+		break;
+	case OBJ_SEARCH_PARTIAL_KEY:
+		/* Not supported by container. */
+		ast_assert(0);
+		return 0;
+	default:
+		cmp = 0;
+		break;
+	}
+	if (cmp) {
+		return 0;
+	}
+	return CMP_MATCH;
+}
+
+/*!
+ * \brief Destructor for extension state publisher
+ */
+static void exten_state_publisher_destroy(void *obj)
+{
+	struct exten_state_publisher *publisher = obj;
+
+	if (publisher->context_filter) {
+		regfree(&publisher->context_regex);
+	}
+
+	if (publisher->exten_filter) {
+		regfree(&publisher->exten_regex);
+	}
+
+	ao2_cleanup(publisher->client);
+	ao2_cleanup(publisher->datastores);
+}
+
+static int build_regex(regex_t *regex, const char *text)
+{
+	int res;
+
+	if ((res = regcomp(regex, text, REG_EXTENDED | REG_ICASE | REG_NOSUB))) {
+		size_t len = regerror(res, regex, NULL, 0);
+		char buf[len];
+		regerror(res, regex, buf, len);
+		ast_log(LOG_ERROR, "Could not compile regex '%s': %s\n", text, buf);
+		return -1;
+	}
+
+	return 0;
+}
+
+static int publisher_start(struct ast_sip_outbound_publish *configuration, struct ast_sip_outbound_publish_client *client)
+{
+	struct exten_state_publisher *publisher;
+	size_t name_size;
+	size_t body_type_size;
+	size_t body_subtype_size;
+	char *body_subtype;
+	const char *body_full;
+	const char *body_type;
+	const char *name;
+	const char *context;
+	const char *exten;
+
+	name = ast_sorcery_object_get_id(configuration);
+
+	body_full = ast_sorcery_object_get_extended(configuration, "body");
+	if (ast_strlen_zero(body_full)) {
+		ast_log(LOG_ERROR, "Outbound extension state publisher '%s': Body not set\n",
+			name);
+		return -1;
+	}
+
+	body_subtype = ast_strdupa(body_full);
+	body_type = strsep(&body_subtype, "/");
+	if (ast_strlen_zero(body_type) || ast_strlen_zero(body_subtype)) {
+		ast_log(LOG_ERROR, "Outbound extension state publisher '%s': Body '%s' missing type or subtype\n",
+			name, body_full);
+		return -1;
+	}
+
+	if (!ast_sip_pubsub_is_body_generator_registered(body_type, body_subtype)) {
+		ast_log(LOG_ERROR, "Outbound extension state publisher '%s': '%s' body generator not registered\n",
+			name, body_full);
+		return -1;
+	}
+
+	name_size = strlen(name) + 1;
+	body_type_size = strlen(body_type) + 1;
+	body_subtype_size = strlen(body_subtype) + 1;
+
+	publisher = ao2_alloc_options(
+		sizeof(*publisher) + name_size + body_type_size + body_subtype_size,
+		exten_state_publisher_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!publisher) {
+		return -1;
+	}
+
+	ast_copy_string(publisher->name, name, name_size);
+	publisher->body_type = publisher->name + name_size;
+	ast_copy_string(publisher->body_type, body_type, body_type_size);
+	publisher->body_subtype = publisher->body_type + body_type_size;
+	ast_copy_string(publisher->body_subtype, body_subtype, body_subtype_size);
+
+	context = ast_sorcery_object_get_extended(configuration, "context");
+	if (!ast_strlen_zero(context)) {
+		if (build_regex(&publisher->context_regex, context)) {
+			ast_log(LOG_ERROR, "Outbound extension state publisher '%s': Could not build context filter '%s'\n",
+				name, context);
+			ao2_ref(publisher, -1);
+			return -1;
+		}
+
+		publisher->context_filter = 1;
+	}
+
+	exten = ast_sorcery_object_get_extended(configuration, "exten");
+	if (!ast_strlen_zero(exten)) {
+		if (build_regex(&publisher->exten_regex, exten)) {
+			ast_log(LOG_ERROR, "Outbound extension state publisher '%s': Could not build exten filter '%s'\n",
+				name, exten);
+			ao2_ref(publisher, -1);
+			return -1;
+		}
+
+		publisher->exten_filter = 1;
+	}
+
+	publisher->datastores = ast_datastores_alloc();
+	if (!publisher->datastores) {
+		ast_log(LOG_ERROR, "Outbound extension state publisher '%s': Could not create datastores container\n",
+			name);
+		ao2_ref(publisher, -1);
+		return -1;
+	}
+
+	publisher->client = ao2_bump(client);
+
+	ao2_lock(publishers);
+	if (!ao2_container_count(publishers)) {
+		ast_extension_state_add(NULL, NULL, exten_state_publisher_state_cb, NULL);
+	}
+	ao2_link_flags(publishers, publisher, OBJ_NOLOCK);
+	ao2_unlock(publishers);
+
+	ao2_ref(publisher, -1);
+
+	return 0;
+}
+
+static int publisher_stop(struct ast_sip_outbound_publish_client *client)
+{
+	ao2_find(publishers, ast_sorcery_object_get_id(client), OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
+	return 0;
+}
+
+static int unload_module(void)
+{
+	ast_sip_unregister_event_publisher_handler(&dialog_publisher);
+	ast_sip_unregister_subscription_handler(&dialog_handler);
+	ast_sip_unregister_event_publisher_handler(&presence_publisher);
+	ast_sip_unregister_subscription_handler(&presence_handler);
+
+	ast_extension_state_del(0, exten_state_publisher_state_cb);
+
+	ast_taskprocessor_unreference(publish_exten_state_serializer);
+	publish_exten_state_serializer = NULL;
+
+	ao2_cleanup(publishers);
+	publishers = NULL;
+
+	return 0;
+}
+
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
+	publishers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		PUBLISHER_BUCKETS, exten_state_publisher_hash, NULL, exten_state_publisher_cmp);
+	if (!publishers) {
+		ast_log(LOG_WARNING, "Unable to create container to store extension state publishers\n");
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	publish_exten_state_serializer = ast_sip_create_serializer("pjsip/exten_state");
+	if (!publish_exten_state_serializer) {
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
 
 	if (ast_sip_register_subscription_handler(&presence_handler)) {
 		ast_log(LOG_WARNING, "Unable to register subscription handler %s\n",
 			presence_handler.event_name);
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (ast_sip_register_event_publisher_handler(&presence_publisher)) {
+		ast_log(LOG_WARNING, "Unable to register presence publisher %s\n",
+			presence_publisher.event_name);
+		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (ast_sip_register_subscription_handler(&dialog_handler)) {
 		ast_log(LOG_WARNING, "Unable to register subscription handler %s\n",
 			dialog_handler.event_name);
-		ast_sip_unregister_subscription_handler(&presence_handler);
+		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	return AST_MODULE_LOAD_SUCCESS;
-}
+	if (ast_sip_register_event_publisher_handler(&dialog_publisher)) {
+		ast_log(LOG_WARNING, "Unable to register presence publisher %s\n",
+			dialog_publisher.event_name);
+		unload_module();
+		return AST_MODULE_LOAD_DECLINE;
+	}
 
-static int unload_module(void)
-{
-	ast_sip_unregister_subscription_handler(&dialog_handler);
-	ast_sip_unregister_subscription_handler(&presence_handler);
-	return 0;
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Extension State Notifications",
@@ -526,4 +1008,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DEPEND + 5,
+	.requires = "res_pjsip,res_pjsip_pubsub,res_pjsip_outbound_publish",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_header_funcs.c 1:16.1.1~dfsg-1/res/res_pjsip_header_funcs.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_header_funcs.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_header_funcs.c	2018-12-26 18:45:13.000000000 +0000
@@ -602,8 +602,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	ast_sip_session_register_supplement(&header_funcs_supplement);
 	ast_custom_function_register(&pjsip_header_function);
 
@@ -618,7 +616,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Header Functions",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_history.c 1:16.1.1~dfsg-1/res/res_pjsip_history.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_history.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_history.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 #include <regex.h>
 
@@ -1368,8 +1366,6 @@ static struct ast_cli_entry cli_pjsip[]
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	log_level = ast_logger_register_level("PJSIP_HISTORY");
 	if (log_level < 0) {
 		ast_log(LOG_WARNING, "Unable to register history log level\n");
@@ -1407,4 +1403,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 		.load = load_module,
 		.unload = unload_module,
 		.load_pri = AST_MODPRI_APP_DEPEND,
+		.requires = "res_pjsip",
 	);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_logger.c 1:16.1.1~dfsg-1/res/res_pjsip_logger.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_logger.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_logger.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <pjsip.h>
 
 #include "asterisk/res_pjsip.h"
@@ -43,31 +41,6 @@ enum pjsip_logging_mode {
 static enum pjsip_logging_mode logging_mode;
 static struct ast_sockaddr log_addr;
 
-/*! \brief  Return the first entry from ast_sockaddr_resolve filtered by address family
- *
- * \warning Using this function probably means you have a faulty design.
- * \note This function was taken from the function of the same name in chan_sip.c
- */
-static int ast_sockaddr_resolve_first_af(struct ast_sockaddr *addr,
-				      const char* name, int flag, int family)
-{
-	struct ast_sockaddr *addrs;
-	int addrs_cnt;
-
-	addrs_cnt = ast_sockaddr_resolve(&addrs, name, flag, family);
-	if (addrs_cnt <= 0) {
-		return 1;
-	}
-	if (addrs_cnt > 1) {
-		ast_debug(1, "Multiple addresses, using the first one only\n");
-	}
-
-	ast_sockaddr_copy(addr, &addrs[0]);
-
-	ast_free(addrs);
-	return 0;
-}
-
 /*! \brief See if we pass debug IP filter */
 static inline int pjsip_log_test_addr(const char *address, int port)
 {
@@ -101,22 +74,25 @@ static inline int pjsip_log_test_addr(co
 
 static pj_status_t logging_on_tx_msg(pjsip_tx_data *tdata)
 {
+	char buffer[AST_SOCKADDR_BUFLEN];
+
 	if (!pjsip_log_test_addr(tdata->tp_info.dst_name, tdata->tp_info.dst_port)) {
 		return PJ_SUCCESS;
 	}
 
-	ast_verbose("<--- Transmitting SIP %s (%d bytes) to %s:%s:%d --->\n%.*s\n",
+	ast_verbose("<--- Transmitting SIP %s (%d bytes) to %s:%s --->\n%.*s\n",
 		    tdata->msg->type == PJSIP_REQUEST_MSG ? "request" : "response",
 		    (int) (tdata->buf.cur - tdata->buf.start),
 		    tdata->tp_info.transport->type_name,
-		    tdata->tp_info.dst_name,
-		    tdata->tp_info.dst_port,
+		    pj_sockaddr_print(&tdata->tp_info.dst_addr, buffer, sizeof(buffer), 3),
 		    (int) (tdata->buf.end - tdata->buf.start), tdata->buf.start);
 	return PJ_SUCCESS;
 }
 
 static pj_bool_t logging_on_rx_msg(pjsip_rx_data *rdata)
 {
+	char buffer[AST_SOCKADDR_BUFLEN];
+
 	if (!pjsip_log_test_addr(rdata->pkt_info.src_name, rdata->pkt_info.src_port)) {
 		return PJ_FALSE;
 	}
@@ -125,12 +101,11 @@ static pj_bool_t logging_on_rx_msg(pjsip
 		return PJ_FALSE;
 	}
 
-	ast_verbose("<--- Received SIP %s (%d bytes) from %s:%s:%d --->\n%s\n",
+	ast_verbose("<--- Received SIP %s (%d bytes) from %s:%s --->\n%s\n",
 		    rdata->msg_info.msg->type == PJSIP_REQUEST_MSG ? "request" : "response",
 		    rdata->msg_info.len,
 		    rdata->tp_info.transport->type_name,
-		    rdata->pkt_info.src_name,
-		    rdata->pkt_info.src_port,
+		    pj_sockaddr_print(&rdata->pkt_info.src_addr, buffer, sizeof(buffer), 3),
 		    rdata->pkt_info.packet);
 	return PJ_FALSE;
 }
@@ -233,8 +208,6 @@ static const struct ast_sorcery_observer
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (ast_sorcery_observer_add(ast_sip_get_sorcery(), "global", &global_observer)) {
 		ast_log(LOG_WARNING, "Unable to add global observer\n");
 		return AST_MODULE_LOAD_DECLINE;
@@ -260,8 +233,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Packet Logger",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_messaging.c 1:16.1.1~dfsg-1/res/res_pjsip_messaging.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_messaging.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_messaging.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -894,8 +893,6 @@ static pjsip_module messaging_module = {
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	if (ast_sip_register_service(&messaging_module) != PJ_SUCCESS) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -913,7 +910,7 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	message_serializer = ast_sip_create_serializer_named("pjsip/messaging");
+	message_serializer = ast_sip_create_serializer("pjsip/messaging");
 	if (!message_serializer) {
 		ast_sip_unregister_service(&messaging_module);
 		ast_msg_tech_unregister(&msg_tech);
@@ -934,8 +931,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Messaging Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_mwi_body_generator.c 1:16.1.1~dfsg-1/res/res_pjsip_mwi_body_generator.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_mwi_body_generator.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_mwi_body_generator.c	2018-12-26 18:45:13.000000000 +0000
@@ -97,8 +97,6 @@ static struct ast_sip_pubsub_body_genera
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_generator(&mwi_generator)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -112,8 +110,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP MWI resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_mwi.c 1:16.1.1~dfsg-1/res/res_pjsip_mwi.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_mwi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_mwi.c	2018-12-26 18:45:13.000000000 +0000
@@ -160,7 +160,7 @@ static int mwi_serializer_pool_setup(voi
 		/* Create name with seq number appended. */
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/mwi");
 
-		mwi_serializer_pool[idx] = ast_sip_create_serializer_named(tps_name);
+		mwi_serializer_pool[idx] = ast_sip_create_serializer(tps_name);
 		if (!mwi_serializer_pool[idx]) {
 			mwi_serializer_pool_shutdown();
 			return -1;
@@ -269,6 +269,9 @@ static struct mwi_stasis_subscription *m
 		ao2_ref(mwi_sub, -1);
 		mwi_stasis_sub = NULL;
 	}
+	stasis_subscription_accept_message_type(mwi_stasis_sub->stasis_sub, ast_mwi_state_type());
+	stasis_subscription_accept_message_type(mwi_stasis_sub->stasis_sub, stasis_subscription_change_type());
+	stasis_subscription_set_filter(mwi_stasis_sub->stasis_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	return mwi_stasis_sub;
 }
 
@@ -358,7 +361,8 @@ static struct mwi_subscription *mwi_subs
 		sub->sip_sub = sip_sub;
 	}
 
-	sub->stasis_subs = ao2_container_alloc(STASIS_BUCKETS, stasis_sub_hash, stasis_sub_cmp);
+	sub->stasis_subs = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		STASIS_BUCKETS, stasis_sub_hash, NULL, stasis_sub_cmp);
 	if (!sub->stasis_subs) {
 		ao2_cleanup(sub);
 		return NULL;
@@ -1339,8 +1343,6 @@ static int reload(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (ast_sip_register_subscription_handler(&mwi_handler)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -1366,7 +1368,11 @@ static int load_module(void)
 		if (ast_test_flag(&ast_options, AST_OPT_FLAG_FULLY_BOOTED)) {
 			ast_sip_push_task(NULL, send_initial_notify_all, NULL);
 		} else {
-			stasis_subscribe_pool(ast_manager_get_topic(), mwi_startup_event_cb, NULL);
+			struct stasis_subscription *sub;
+
+			sub = stasis_subscribe_pool(ast_manager_get_topic(), mwi_startup_event_cb, NULL);
+			stasis_subscription_accept_message_type(sub, ast_manager_get_generic_type());
+			stasis_subscription_set_filter(sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 	}
 
@@ -1397,4 +1403,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.unload = unload_module,
 	.reload = reload,
 	.load_pri = AST_MODPRI_CHANNEL_DEPEND + 5,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_nat.c 1:16.1.1~dfsg-1/res/res_pjsip_nat.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_nat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_nat.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,10 +45,42 @@ static void rewrite_uri(pjsip_rx_data *r
 	}
 }
 
+/*
+ * Update the Record-Route headers in the request or response and in the dialog
+ * object if exists.
+ *
+ * When NAT is in use, the address of the next hop in the SIP may be incorrect.
+ * To address this  asterisk uses two strategies in parallel:
+ *  1. intercept the messages at the transaction level and rewrite the
+ *     messages before arriving at the dialog layer
+ *  2. after the application processing, update the dialog object with the
+ *     correct information
+ *
+ * The first strategy has a limitation that the SIP message may not have all
+ * the information required to determine if the next hop is in the route set
+ * or in the contact. Causing risk that asterisk will update the Contact on
+ * receipt of an in-dialog message despite there being a route set saved in
+ * the dialog.
+ *
+ * The second strategy has a limitation that not all UAC layers have interfaces
+ * available to invoke this module after dialog creation.  (pjsip_sesion does
+ * but pjsip_pubsub does not), thus this strategy can't update the dialog in
+ * all cases needed.
+ *
+ * The ideal solution would be to implement an "incomming_request" event
+ * in pubsub module that can then pass the dialog object to this module
+ * on SUBSCRIBE, this module then should add itself as a listener to the dialog
+ * for the subsequent requests and responses & then be able to properly update
+ * the dialog object for all required events.
+ */
+
 static int rewrite_route_set(pjsip_rx_data *rdata, pjsip_dialog *dlg)
 {
 	pjsip_rr_hdr *rr = NULL;
 	pjsip_sip_uri *uri;
+	int res = -1;
+	int ignore_rr = 0;
+	int pubsub = 0;
 
 	if (rdata->msg_info.msg->type == PJSIP_RESPONSE_MSG) {
 		pjsip_hdr *iter;
@@ -60,21 +92,49 @@ static int rewrite_route_set(pjsip_rx_da
 		}
 	} else if (pjsip_method_cmp(&rdata->msg_info.msg->line.req.method, &pjsip_register_method)) {
 		rr = pjsip_msg_find_hdr(rdata->msg_info.msg, PJSIP_H_RECORD_ROUTE, NULL);
+	} else {
+		/**
+		 * Record-Route header has no meaning in REGISTER requests
+		 * and should be ignored
+		 */
+		ignore_rr = 1;
+	}
+
+	if (!pjsip_method_cmp(&rdata->msg_info.cseq->method, &pjsip_subscribe_method) ||
+		!pjsip_method_cmp(&rdata->msg_info.cseq->method, &pjsip_notify_method)) {
+		/**
+		 * There is currently no good way to get the dlg object for a pubsub dialog
+		 * so we will just look at the rr & contact of the current message and
+		 * hope for the best
+		 */
+		pubsub = 1;
 	}
 
 	if (rr) {
 		uri = pjsip_uri_get_uri(&rr->name_addr);
 		rewrite_uri(rdata, uri);
-		if (dlg && !pj_list_empty(&dlg->route_set) && !dlg->route_set_frozen) {
-			pjsip_routing_hdr *route = dlg->route_set.next;
-			uri = pjsip_uri_get_uri(&route->name_addr);
-			rewrite_uri(rdata, uri);
-		}
+		res = 0;
+	}
 
-		return 0;
+	if (dlg && !pj_list_empty(&dlg->route_set) && !dlg->route_set_frozen) {
+		pjsip_routing_hdr *route = dlg->route_set.next;
+		uri = pjsip_uri_get_uri(&route->name_addr);
+		rewrite_uri(rdata, uri);
+		res = 0;
 	}
 
-	return -1;
+	if (!dlg && !rr && !ignore_rr  && !pubsub && rdata->msg_info.to->tag.slen){
+		/**
+		 * Even if this message doesn't have any route headers
+		 * the dialog may, so wait until a later invocation that
+		 * has a dialog reference to make sure there isn't a
+		 * previously saved routset in the dialog before deciding
+		 * the contact needs to be modified
+		 */
+		res = 0;
+	}
+
+	return res;
 }
 
 static int rewrite_contact(pjsip_rx_data *rdata, pjsip_dialog *dlg)
@@ -357,25 +417,20 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	if (ast_sip_register_service(&nat_module)) {
 		ast_log(LOG_ERROR, "Could not register NAT module for incoming and outgoing requests\n");
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	if (ast_sip_session_register_supplement(&nat_supplement)) {
-		ast_log(LOG_ERROR, "Could not register NAT session supplement for incoming and outgoing INVITE requests\n");
-		unload_module();
-		return AST_MODULE_LOAD_DECLINE;
-	}
+	ast_sip_session_register_supplement(&nat_supplement);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP NAT Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_notify.c 1:16.1.1~dfsg-1/res/res_pjsip_notify.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_notify.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_notify.c	2018-12-26 18:45:13.000000000 +0000
@@ -226,9 +226,9 @@ static void *notify_cfg_alloc(void)
 		return NULL;
 	}
 
-	if (!(cfg->notify_options = ao2_container_alloc_options(
-		      AO2_ALLOC_OPT_LOCK_NOLOCK, 20, notify_option_hash,
-		      notify_option_cmp))) {
+	cfg->notify_options = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_NOLOCK, 0,
+		20, notify_option_hash, NULL, notify_option_cmp);
+	if (!cfg->notify_options) {
 		ao2_cleanup(cfg);
 		return NULL;
 	}
@@ -888,30 +888,30 @@ static enum notify_result push_notify_ch
  * \internal
  * \brief Do completion on the endpoint.
  */
-static char *cli_complete_endpoint(const char *word, int state)
+static char *cli_complete_endpoint(const char *word)
 {
-	char *result = NULL;
 	int wordlen = strlen(word);
-	int which = 0;
-
+	struct ao2_container * endpoints;
 	struct ast_sip_endpoint *endpoint;
-	RAII_VAR(struct ao2_container *, endpoints,
-		 ast_sip_get_endpoints(), ao2_cleanup);
+	struct ao2_iterator i;
 
-	struct ao2_iterator i = ao2_iterator_init(endpoints, 0);
-	while ((endpoint = ao2_iterator_next(&i))) {
-		const char *name = ast_sorcery_object_get_id(endpoint);
-		if (!strncasecmp(word, name, wordlen) && ++which > state) {
-			result = ast_strdup(name);
-		}
+	endpoints = ast_sorcery_retrieve_by_prefix(ast_sip_get_sorcery(),
+		"endpoint", word, wordlen);
+	if (endpoints == NULL) {
+		return NULL;
+	}
 
+	i = ao2_iterator_init(endpoints, 0);
+	while ((endpoint = ao2_iterator_next(&i))) {
+		ast_cli_completion_add(
+			ast_strdup(ast_sorcery_object_get_id(endpoint)));
 		ao2_cleanup(endpoint);
-		if (result) {
-			break;
-		}
 	}
 	ao2_iterator_destroy(&i);
-	return result;
+
+	ao2_ref(endpoints, -1);
+
+	return NULL;
 }
 
 /*!
@@ -967,7 +967,7 @@ static char *cli_complete_notify(const c
 		return c;
 	}
 
-	return pos > 4 && !using_uri ? cli_complete_endpoint(word, state) : NULL;
+	return pos > 4 && !using_uri ? cli_complete_endpoint(word) : NULL;
 }
 
 /*!
@@ -1195,8 +1195,6 @@ static int manager_notify(struct mansess
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (aco_info_init(&notify_cfg)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -1235,9 +1233,10 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "CLI/AMI PJSIP NOTIFY Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.reload = reload_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.reload = reload_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_one_touch_record_info.c 1:16.1.1~dfsg-1/res/res_pjsip_one_touch_record_info.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_one_touch_record_info.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_one_touch_record_info.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	 <depend>pjproject</depend>
 	 <depend>res_pjsip</depend>
-	 <depend>res_pjsip_session</depend>
 	 <support_level>core</support_level>
 ***/
 
@@ -108,12 +107,7 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
-	if (ast_sip_session_register_supplement(&info_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register One Touch Recording supplement\n");
-		return AST_MODULE_LOAD_DECLINE;
-	}
+	ast_sip_session_register_supplement(&info_supplement);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -129,4 +123,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_outbound_authenticator_digest.c 1:16.1.1~dfsg-1/res/res_pjsip_outbound_authenticator_digest.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_outbound_authenticator_digest.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_outbound_authenticator_digest.c	2018-12-26 18:45:13.000000000 +0000
@@ -96,7 +96,7 @@ cleanup:
 	return res;
 }
 
-static int digest_create_request_with_auth_from_old(const struct ast_sip_auth_vector *auths,
+static int digest_create_request_with_auth(const struct ast_sip_auth_vector *auths,
 	pjsip_rx_data *challenge, pjsip_tx_data *old_request, pjsip_tx_data **new_request)
 {
 	pjsip_auth_clt_sess auth_sess;
@@ -118,8 +118,8 @@ static int digest_create_request_with_au
 	}
 	/* If there was no dialog, then this is probably a REGISTER so no endpoint */
 	if (!id) {
-		id = ast_alloca(strlen(challenge->pkt_info.src_name) + 7 /* ':' + port + NULL */);
-		sprintf(id, "%s:%d", challenge->pkt_info.src_name, challenge->pkt_info.src_port);
+		id = ast_alloca(AST_SOCKADDR_BUFLEN);
+		pj_sockaddr_print(&challenge->pkt_info.src_addr, id, AST_SOCKADDR_BUFLEN, 3);
 		id_type = "Host";
 	}
 
@@ -199,21 +199,12 @@ static int digest_create_request_with_au
 	return -1;
 }
 
-static int digest_create_request_with_auth(const struct ast_sip_auth_vector *auths, pjsip_rx_data *challenge,
-		pjsip_transaction *tsx, pjsip_tx_data **new_request)
-{
-	return digest_create_request_with_auth_from_old(auths, challenge, tsx->last_tx, new_request);
-}
-
 static struct ast_sip_outbound_authenticator digest_authenticator = {
 	.create_request_with_auth = digest_create_request_with_auth,
-	.create_request_with_auth_from_old = digest_create_request_with_auth_from_old,
 };
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (ast_sip_register_outbound_authenticator(&digest_authenticator)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -227,8 +218,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP authentication resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_outbound_publish.c 1:16.1.1~dfsg-1/res/res_pjsip_outbound_publish.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_outbound_publish.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_outbound_publish.c	2018-12-26 18:45:13.000000000 +0000
@@ -28,6 +28,7 @@
 #include <pjsip.h>
 #include <pjsip_simple.h>
 
+#include "asterisk/res_pjproject.h"
 #include "asterisk/res_pjsip.h"
 #include "asterisk/res_pjsip_outbound_publish.h"
 #include "asterisk/module.h"
@@ -108,6 +109,10 @@
 						<literal>pjsip.conf</literal>. As with other <literal>res_pjsip</literal> modules, this will use the first available transport of the appropriate type if unconfigured.</para></note>
 					</description>
 				</configOption>
+				<configOption name="multi_user" default="no">
+					<synopsis>Enable multi-user support</synopsis>
+					<description><para>When enabled the user portion of the server uri is replaced by a dynamically created user</para></description>
+				</configOption>
 				<configOption name="type">
 					<synopsis>Must be of type 'outbound-publish'.</synopsis>
 				</configOption>
@@ -116,6 +121,8 @@
 	</configInfo>
  ***/
 
+static int pjsip_max_url_size = PJSIP_MAX_URL_SIZE;
+
 /*! \brief Queued outbound publish message */
 struct sip_outbound_publish_message {
 	/*! \brief Optional body */
@@ -126,6 +133,39 @@ struct sip_outbound_publish_message {
 	char body_contents[0];
 };
 
+/*
+ * A note about some of the object types used in this module:
+ *
+ * The reason we currently have 4 separate object types that relate to configuration,
+ * publishing, state, and client information is due to object lifetimes and order of
+ * destruction dependencies.
+ *
+ * Separation of concerns is a good thing and of course it makes sense to have a
+ * configuration object type as well as an object type wrapper around pjsip's publishing
+ * client class. There also may be run time state data that needs to be tracked, so
+ * again having something to handle that is prudent. However, it may be tempting to think
+ * "why not combine the state and client object types?" Especially seeing as how they have
+ * a one-to-one relationship. The answer is, it's possible, but it'd make the code a bit
+ * more awkward.
+ *
+ * Currently this module maintains a global container of current state objects. When this
+ * states container is replaced, or deleted, it un-references all contained objects. Any
+ * state with a reference left have probably been carried over from a reload/realtime fetch.
+ * States not carried over are destructed and the associated client (and all its publishers)
+ * get unpublished.
+ *
+ * This "unpublishing" goes through a careful process of unpublishing the client, all its
+ * publishers, and making sure all the appropriate references are removed in a sane order.
+ * This process is essentially kicked off with the destruction of the state. If the state
+ * and client objects were to be merged, where clients became the globally tracked object
+ * type, this "unpublishing" process would never start because of the multiple references
+ * held to the client object over it's lifetime. Meaning the global tracking container
+ * would remove its reference to the client object when done with it, but other sources
+ * would still be holding a reference to it (namely the datastore and publisher(s)).
+ *
+ * Thus at this time it is easier to keep them separate.
+ */
+
 /*! \brief Outbound publish information */
 struct ast_sip_outbound_publish {
 	/*! \brief Sorcery object details */
@@ -151,32 +191,49 @@ struct ast_sip_outbound_publish {
 	unsigned int max_auth_attempts;
 	/*! \brief Configured authentication credentials */
 	struct ast_sip_auth_vector outbound_auths;
+	/*! \brief The publishing client is used for multiple users when true */
+	unsigned int multi_user;
 };
 
-/*! \brief Outbound publish client state information (persists for lifetime that publish should exist) */
-struct ast_sip_outbound_publish_client {
+struct sip_outbound_publisher {
+	/*! \brief The client object that 'owns' this client
+
+	     \note any potential circular reference problems are accounted
+	     for (see publisher alloc for more information)
+	*/
+	struct ast_sip_outbound_publish_client *owner;
 	/*! \brief Underlying publish client */
 	pjsip_publishc *client;
+	/*! \brief The From URI for this specific publisher */
+	char *from_uri;
+	/*! \brief The To URI for this specific publisher */
+	char *to_uri;
 	/*! \brief Timer entry for refreshing publish */
 	pj_timer_entry timer;
-	/*! \brief Publisher datastores set up by handlers */
-	struct ao2_container *datastores;
 	/*! \brief The number of auth attempts done */
 	unsigned int auth_attempts;
 	/*! \brief Queue of outgoing publish messages to send*/
 	AST_LIST_HEAD_NOLOCK(, sip_outbound_publish_message) queue;
 	/*! \brief The message currently being sent */
 	struct sip_outbound_publish_message *sending;
-	/*! \brief Publish client has been fully started and event type informed */
-	unsigned int started;
 	/*! \brief Publish client should be destroyed */
 	unsigned int destroy;
-	/*! \brief Outbound publish information */
-	struct ast_sip_outbound_publish *publish;
-	/*! \brief The name of the transport to be used for the publish */
-	char *transport_name;
 	/*! \brief Serializer for stuff and things */
 	struct ast_taskprocessor *serializer;
+	/*! \brief User, if any, associated with the publisher */
+	char user[0];
+};
+
+/*! \brief Outbound publish client state information (persists for lifetime of a publish) */
+struct ast_sip_outbound_publish_client {
+	/*! \brief Outbound publish information */
+	struct ast_sip_outbound_publish *publish;
+	/*! \brief Publisher datastores set up by handlers */
+	struct ao2_container *datastores;
+	/*! \brief Container of all the client publishing objects */
+	struct ao2_container *publishers;
+	/*! \brief Publishing has been fully started and event type informed */
+	unsigned int started;
 };
 
 /*! \brief Outbound publish state information (persists for lifetime of a publish) */
@@ -187,6 +244,20 @@ struct ast_sip_outbound_publish_state {
 	char id[0];
 };
 
+/*!
+ * \brief Used for locking while loading/reloading
+ *
+ * Mutli-user configurations make it so publishers can be dynamically added and
+ * removed. Publishers should not be added or removed during a [re]load since
+ * it could cause the current_clients container to be out of sync. Thus the
+ * reason for this lock.
+ */
+AST_RWLOCK_DEFINE_STATIC(load_lock);
+
+#define DEFAULT_PUBLISHER_BUCKETS 119
+AO2_STRING_FIELD_HASH_FN(sip_outbound_publisher, user);
+AO2_STRING_FIELD_CMP_FN(sip_outbound_publisher, user);
+
 /*! Time needs to be long enough for a transaction to timeout if nothing replies. */
 #define MAX_UNLOAD_TIMEOUT_TIME		35	/* Seconds */
 
@@ -252,6 +323,7 @@ static int outbound_publish_state_cmp(vo
 static struct ao2_container *get_publishes_and_update_state(void)
 {
 	struct ao2_container *container;
+	SCOPED_WRLOCK(lock, &load_lock);
 
 	container = ast_sorcery_retrieve_by_fields(
 		ast_sip_get_sorcery(), "outbound-publish",
@@ -273,7 +345,6 @@ AST_RWLIST_HEAD_STATIC(publisher_handler
 static void sub_add_handler(struct ast_sip_event_publisher_handler *handler)
 {
 	AST_RWLIST_INSERT_TAIL(&publisher_handlers, handler, next);
-	ast_module_ref(ast_module_info->self);
 }
 
 static struct ast_sip_event_publisher_handler *find_publisher_handler_for_event_name(const char *event_name)
@@ -288,23 +359,23 @@ static struct ast_sip_event_publisher_ha
 	return iter;
 }
 
-/*! \brief Helper function which cancels the refresh timer on a client */
-static void cancel_publish_refresh(struct ast_sip_outbound_publish_client *client)
+/*! \brief Helper function which cancels the refresh timer on a publisher */
+static void cancel_publish_refresh(struct sip_outbound_publisher *publisher)
 {
 	if (pj_timer_heap_cancel_if_active(pjsip_endpt_get_timer_heap(ast_sip_get_pjsip_endpoint()),
-		&client->timer, 0)) {
-		/* The timer was successfully cancelled, drop the refcount of the client */
-		ao2_ref(client, -1);
+		&publisher->timer, 0)) {
+		/* The timer was successfully cancelled, drop the refcount of the publisher */
+		ao2_ref(publisher, -1);
 	}
 }
 
 /*! \brief Helper function which sets up the timer to send publication */
-static void schedule_publish_refresh(struct ast_sip_outbound_publish_client *client, int expiration)
+static void schedule_publish_refresh(struct sip_outbound_publisher *publisher, int expiration)
 {
-	struct ast_sip_outbound_publish *publish = ao2_bump(client->publish);
+	struct ast_sip_outbound_publish *publish = ao2_bump(publisher->owner->publish);
 	pj_time_val delay = { .sec = 0, };
 
-	cancel_publish_refresh(client);
+	cancel_publish_refresh(publisher);
 
 	if (expiration > 0) {
 		delay.sec = expiration - PJSIP_PUBLISHC_DELAY_BEFORE_REFRESH;
@@ -316,62 +387,83 @@ static void schedule_publish_refresh(str
 		delay.sec = PJSIP_PUBLISHC_DELAY_BEFORE_REFRESH;
 	}
 
-	ao2_ref(client, +1);
-	if (pjsip_endpt_schedule_timer(ast_sip_get_pjsip_endpoint(), &client->timer, &delay) != PJ_SUCCESS) {
+	ao2_ref(publisher, +1);
+	if (pjsip_endpt_schedule_timer(ast_sip_get_pjsip_endpoint(), &publisher->timer, &delay) != PJ_SUCCESS) {
 		ast_log(LOG_WARNING, "Failed to pass timed publish refresh to scheduler\n");
-		ao2_ref(client, -1);
+		ao2_ref(publisher, -1);
 	}
 	ao2_ref(publish, -1);
 }
 
+static int publisher_client_send(void *obj, void *arg, void *data, int flags);
+
 /*! \brief Publish client timer callback function */
 static void sip_outbound_publish_timer_cb(pj_timer_heap_t *timer_heap, struct pj_timer_entry *entry)
 {
-	struct ast_sip_outbound_publish_client *client = entry->user_data;
+	struct sip_outbound_publisher *publisher = entry->user_data;
 
-	ao2_lock(client);
-	if (AST_LIST_EMPTY(&client->queue)) {
+	ao2_lock(publisher);
+	if (AST_LIST_EMPTY(&publisher->queue)) {
+		int res;
 		/* If there are no outstanding messages send an empty PUBLISH message so our publication doesn't expire */
-		ast_sip_publish_client_send(client, NULL);
+		publisher_client_send(publisher, NULL, &res, 0);
 	}
-	ao2_unlock(client);
+	ao2_unlock(publisher);
 
-	ao2_ref(client, -1);
+	ao2_ref(publisher, -1);
 }
 
 /*! \brief Task for cancelling a refresh timer */
 static int cancel_refresh_timer_task(void *data)
 {
-	struct ast_sip_outbound_publish_client *client = data;
+	struct sip_outbound_publisher *publisher = data;
 
-	cancel_publish_refresh(client);
-	ao2_ref(client, -1);
+	cancel_publish_refresh(publisher);
+	ao2_ref(publisher, -1);
 
 	return 0;
 }
 
+static void set_transport(struct sip_outbound_publisher *publisher, pjsip_tx_data *tdata)
+{
+	if (!ast_strlen_zero(publisher->owner->publish->transport)) {
+		pjsip_tpselector selector = { .type = PJSIP_TPSELECTOR_NONE, };
+		ast_sip_set_tpselector_from_transport_name(
+			publisher->owner->publish->transport, &selector);
+		pjsip_tx_data_set_transport(tdata, &selector);
+	}
+}
+
 /*! \brief Task for sending an unpublish */
 static int send_unpublish_task(void *data)
 {
-	struct ast_sip_outbound_publish_client *client = data;
+	struct sip_outbound_publisher *publisher = data;
 	pjsip_tx_data *tdata;
 
-	if (pjsip_publishc_unpublish(client->client, &tdata) == PJ_SUCCESS) {
-		if (!ast_strlen_zero(client->transport_name)) {
-			pjsip_tpselector selector = { .type = PJSIP_TPSELECTOR_NONE, };
-			ast_sip_set_tpselector_from_transport_name(client->transport_name, &selector);
-			pjsip_tx_data_set_transport(tdata, &selector);
-		}
-
-		ast_sip_record_request_serializer(tdata);
-		pjsip_publishc_send(client->client, tdata);
+	if (pjsip_publishc_unpublish(publisher->client, &tdata) == PJ_SUCCESS) {
+		set_transport(publisher, tdata);
+		pjsip_publishc_send(publisher->client, tdata);
 	}
 
-	ao2_ref(client, -1);
+	ao2_ref(publisher, -1);
 
 	return 0;
 }
 
+static void stop_publishing(struct ast_sip_outbound_publish_client *client,
+			    struct ast_sip_event_publisher_handler *handler)
+{
+	if (!handler) {
+		handler = find_publisher_handler_for_event_name(client->publish->event);
+	}
+
+	if (handler) {
+		handler->stop_publishing(client);
+	}
+}
+
+static int cancel_and_unpublish(void *obj, void *arg, int flags);
+
 /*! \brief Helper function which starts or stops publish clients when applicable */
 static void sip_outbound_publish_synchronize(struct ast_sip_event_publisher_handler *removed)
 {
@@ -405,15 +497,10 @@ static void sip_outbound_publish_synchro
 			} else {
 				state->client->started = 1;
 			}
-		} else if (!handler && removed && !strcmp(publish->event, removed->event_name)) {
-			/* If the publisher client has been started but it is going away stop it */
-			removed->stop_publishing(state->client);
+		} else if (state->client->started && !handler && removed && !strcmp(publish->event, removed->event_name)) {
+			stop_publishing(state->client, removed);
+			ao2_callback(state->client->publishers, OBJ_NODATA, cancel_and_unpublish, NULL);
 			state->client->started = 0;
-			if (ast_sip_push_task(state->client->serializer, cancel_refresh_timer_task, ao2_bump(state->client))) {
-				ast_log(LOG_WARNING, "Could not stop refresh timer on client '%s'\n",
-					ast_sorcery_object_get_id(publish));
-				ao2_ref(state->client, -1);
-			}
 		}
 		ao2_ref(publish, -1);
 		ao2_ref(state, -1);
@@ -449,6 +536,79 @@ struct ast_sip_outbound_publish_client *
 	return state->client;
 }
 
+const char *ast_sip_publish_client_get_from_uri(struct ast_sip_outbound_publish_client *client)
+{
+	struct ast_sip_outbound_publish *publish = client->publish;
+
+	return S_OR(publish->from_uri, S_OR(publish->server_uri, ""));
+}
+
+static struct sip_outbound_publisher *sip_outbound_publish_client_add_publisher(
+        struct ast_sip_outbound_publish_client *client, const char *user);
+
+static struct sip_outbound_publisher *sip_outbound_publish_client_get_publisher(
+	struct ast_sip_outbound_publish_client *client, const char *user)
+{
+	struct sip_outbound_publisher *publisher;
+
+	/*
+	 * Lock before searching since there could be a race between searching and adding.
+	 * Just use the load_lock since we might need to lock it anyway (if adding) and
+	 * also it simplifies the code (otherwise we'd have to lock the publishers, no-
+	 * lock the search and pass a flag to 'add publisher to no-lock the potential link).
+	 */
+	ast_rwlock_wrlock(&load_lock);
+	publisher = ao2_find(client->publishers, user, OBJ_SEARCH_KEY);
+	if (!publisher) {
+		if (!(publisher = sip_outbound_publish_client_add_publisher(client, user))) {
+			ast_rwlock_unlock(&load_lock);
+			return NULL;
+		}
+	}
+	ast_rwlock_unlock(&load_lock);
+
+	return publisher;
+}
+
+const char *ast_sip_publish_client_get_user_from_uri(struct ast_sip_outbound_publish_client *client, const char *user,
+	char *uri, size_t size)
+{
+	struct sip_outbound_publisher *publisher;
+
+	publisher = sip_outbound_publish_client_get_publisher(client, user);
+	if (!publisher) {
+		return NULL;
+	}
+
+	ast_copy_string(uri, publisher->from_uri, size);
+	ao2_ref(publisher, -1);
+
+	return uri;
+}
+
+const char *ast_sip_publish_client_get_to_uri(struct ast_sip_outbound_publish_client *client)
+{
+	struct ast_sip_outbound_publish *publish = client->publish;
+
+	return S_OR(publish->to_uri, S_OR(publish->server_uri, ""));
+}
+
+const char *ast_sip_publish_client_get_user_to_uri(struct ast_sip_outbound_publish_client *client, const char *user,
+	char *uri, size_t size)
+{
+	struct sip_outbound_publisher *publisher;
+
+	publisher = sip_outbound_publish_client_get_publisher(client, user);
+	if (!publisher) {
+		return NULL;
+	}
+
+	ast_copy_string(uri, publisher->to_uri, size);
+	ao2_ref(publisher, -1);
+
+	return uri;
+}
+
 int ast_sip_register_event_publisher_handler(struct ast_sip_event_publisher_handler *handler)
 {
 	struct ast_sip_event_publisher_handler *existing;
@@ -483,7 +643,6 @@ void ast_sip_unregister_event_publisher_
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&publisher_handlers, iter, next) {
 		if (handler == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			ast_module_unref(ast_module_info->self);
 			break;
 		}
 	}
@@ -585,19 +744,19 @@ void ast_sip_publish_client_remove_datas
 	ao2_find(client->datastores, name, OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
 }
 
-static int sip_publish_client_service_queue(void *data)
+static int sip_publisher_service_queue(void *data)
 {
-	RAII_VAR(struct ast_sip_outbound_publish_client *, client, data, ao2_cleanup);
-	SCOPED_AO2LOCK(lock, client);
+	RAII_VAR(struct sip_outbound_publisher *, publisher, data, ao2_cleanup);
+	SCOPED_AO2LOCK(lock, publisher);
 	struct sip_outbound_publish_message *message;
 	pjsip_tx_data *tdata;
 	pj_status_t status;
 
-	if (client->destroy || client->sending || !(message = AST_LIST_FIRST(&client->queue))) {
+	if (publisher->destroy || publisher->sending || !(message = AST_LIST_FIRST(&publisher->queue))) {
 		return 0;
 	}
 
-	if (pjsip_publishc_publish(client->client, PJ_FALSE, &tdata) != PJ_SUCCESS) {
+	if (pjsip_publishc_publish(publisher->client, PJ_FALSE, &tdata) != PJ_SUCCESS) {
 		goto fatal;
 	}
 
@@ -607,14 +766,9 @@ static int sip_publish_client_service_qu
 		goto fatal;
 	}
 
-	if (!ast_strlen_zero(client->transport_name)) {
-		pjsip_tpselector selector = { .type = PJSIP_TPSELECTOR_NONE, };
-		ast_sip_set_tpselector_from_transport_name(client->transport_name, &selector);
-		pjsip_tx_data_set_transport(tdata, &selector);
-	}
+	set_transport(publisher, tdata);
 
-	ast_sip_record_request_serializer(tdata);
-	status = pjsip_publishc_send(client->client, tdata);
+	status = pjsip_publishc_send(publisher->client, tdata);
 	if (status == PJ_EBUSY) {
 		/* We attempted to send the message but something else got there first */
 		goto service;
@@ -622,30 +776,32 @@ static int sip_publish_client_service_qu
 		goto fatal;
 	}
 
-	client->sending = message;
+	publisher->sending = message;
 
 	return 0;
 
 fatal:
-	AST_LIST_REMOVE_HEAD(&client->queue, entry);
+	AST_LIST_REMOVE_HEAD(&publisher->queue, entry);
 	ast_free(message);
 
 service:
-	if (ast_sip_push_task(client->serializer, sip_publish_client_service_queue, ao2_bump(client))) {
-		ao2_ref(client, -1);
+	if (ast_sip_push_task(publisher->serializer, sip_publisher_service_queue, ao2_bump(publisher))) {
+		ao2_ref(publisher, -1);
 	}
 	return -1;
 }
 
-int ast_sip_publish_client_send(struct ast_sip_outbound_publish_client *client,
-	const struct ast_sip_body *body)
+static int publisher_client_send(void *obj, void *arg, void *data, int flags)
 {
-	SCOPED_AO2LOCK(lock, client);
+	struct sip_outbound_publisher *publisher = obj;
+	const struct ast_sip_body *body = arg;
 	struct sip_outbound_publish_message *message;
 	size_t type_len = 0, subtype_len = 0, body_text_len = 0;
-	int res;
+	int *res = data;
+	SCOPED_AO2LOCK(lock, publisher);
 
-	if (!client->client) {
+	*res = -1;
+	if (!publisher->client) {
 		return -1;
 	}
 
@@ -671,95 +827,400 @@ int ast_sip_publish_client_send(struct a
 		message->body.body_text = strcpy(dst, body->body_text);
 	}
 
-	AST_LIST_INSERT_TAIL(&client->queue, message, entry);
+	AST_LIST_INSERT_TAIL(&publisher->queue, message, entry);
 
-	res = ast_sip_push_task(client->serializer, sip_publish_client_service_queue, ao2_bump(client));
-	if (res) {
-		ao2_ref(client, -1);
+	*res = ast_sip_push_task(publisher->serializer, sip_publisher_service_queue, ao2_bump(publisher));
+	if (*res) {
+		ao2_ref(publisher, -1);
 	}
 
+	return *res;
+}
+
+int ast_sip_publish_client_send(struct ast_sip_outbound_publish_client *client,
+	const struct ast_sip_body *body)
+{
+	SCOPED_AO2LOCK(lock, client);
+	int res = 0;
+
+	ao2_callback_data(client->publishers, OBJ_NODATA,
+			  publisher_client_send, (void *)body, &res);
 	return res;
 }
 
+static int sip_outbound_publisher_set_uri(
+	pj_pool_t *pool, const char *uri, const char *user, pj_str_t *res_uri)
+{
+	pj_str_t tmp;
+	pjsip_uri *parsed;
+	pjsip_sip_uri *parsed_uri;
+	int size;
+
+	pj_strdup2_with_null(pool, &tmp, uri);
+	if (!(parsed = pjsip_parse_uri(pool, tmp.ptr, tmp.slen, 0))) {
+		return -1;
+	}
+
+	if (!(parsed_uri = pjsip_uri_get_uri(parsed))) {
+		return -1;
+	}
+
+	if (!ast_strlen_zero(user)) {
+		pj_strdup2(pool, &parsed_uri->user, user);
+	}
+
+	res_uri->ptr = (char*) pj_pool_alloc(pool, pjsip_max_url_size);
+	if (!res_uri->ptr) {
+		return -1;
+	}
+
+	if ((size = pjsip_uri_print(PJSIP_URI_IN_OTHER, parsed_uri, res_uri->ptr,
+				    pjsip_max_url_size - 1)) <= 0) {
+		return -1;
+	}
+	res_uri->ptr[size] = '\0';
+	res_uri->slen = size;
+
+	return 0;
+}
+
+static int sip_outbound_publisher_set_uris(
+	pj_pool_t *pool, struct sip_outbound_publisher *publisher,
+	pj_str_t *server_uri, pj_str_t *to_uri, pj_str_t *from_uri)
+{
+	struct ast_sip_outbound_publish *publish = publisher->owner->publish;
+
+	if (sip_outbound_publisher_set_uri(pool, publish->server_uri, publisher->user, server_uri)) {
+		ast_log(LOG_ERROR, "Invalid server URI '%s' specified on outbound publish '%s'\n",
+			publish->server_uri, ast_sorcery_object_get_id(publish));
+		return -1;
+	}
+
+	if (ast_strlen_zero(publish->to_uri)) {
+		to_uri->ptr = server_uri->ptr;
+		to_uri->slen = server_uri->slen;
+	} else if (sip_outbound_publisher_set_uri(pool, publish->to_uri, publisher->user, to_uri)) {
+		ast_log(LOG_ERROR, "Invalid to URI '%s' specified on outbound publish '%s'\n",
+			publish->to_uri, ast_sorcery_object_get_id(publish));
+		return -1;
+	}
+
+	publisher->to_uri = ast_strdup(to_uri->ptr);
+	if (!publisher->to_uri) {
+		return -1;
+	}
+
+	if (ast_strlen_zero(publish->from_uri)) {
+		from_uri->ptr = server_uri->ptr;
+		from_uri->slen = server_uri->slen;
+	} else if (sip_outbound_publisher_set_uri(pool, publish->from_uri, publisher->user, from_uri)) {
+		ast_log(LOG_ERROR, "Invalid from URI '%s' specified on outbound publish '%s'\n",
+			publish->from_uri, ast_sorcery_object_get_id(publish));
+		return -1;
+	}
+
+	publisher->from_uri = ast_strdup(from_uri->ptr);
+	if (!publisher->from_uri) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static void sip_outbound_publish_callback(struct pjsip_publishc_cbparam *param);
+
+/*! \brief Helper function that allocates a pjsip publish client and configures it */
+static int sip_outbound_publisher_init(void *data)
+{
+	struct sip_outbound_publisher *publisher = data;
+	RAII_VAR(struct ast_sip_outbound_publish *, publish, NULL, ao2_cleanup);
+	pjsip_publishc_opt opt = {
+		.queue_request = PJ_FALSE,
+	};
+	pj_pool_t *pool;
+	pj_str_t event, server_uri, to_uri, from_uri;
+
+	if (publisher->client) {
+		return 0;
+	}
+
+	if (pjsip_publishc_create(ast_sip_get_pjsip_endpoint(), &opt,
+				  ao2_bump(publisher), sip_outbound_publish_callback,
+		&publisher->client) != PJ_SUCCESS) {
+		ao2_ref(publisher, -1);
+		return -1;
+	}
+
+	publish = ao2_bump(publisher->owner->publish);
+
+	if (!ast_strlen_zero(publish->outbound_proxy)) {
+		pjsip_route_hdr route_set, *route;
+		static const pj_str_t ROUTE_HNAME = { "Route", 5 };
+
+		pj_list_init(&route_set);
+
+		if (!(route = pjsip_parse_hdr(pjsip_publishc_get_pool(publisher->client), &ROUTE_HNAME,
+			(char*)publish->outbound_proxy, strlen(publish->outbound_proxy), NULL))) {
+			pjsip_publishc_destroy(publisher->client);
+			return -1;
+		}
+		pj_list_insert_nodes_before(&route_set, route);
+
+		pjsip_publishc_set_route_set(publisher->client, &route_set);
+	}
+
+	pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "URI Validation",
+				       pjsip_max_url_size, pjsip_max_url_size);
+	if (!pool) {
+		ast_log(LOG_ERROR, "Could not create pool for URI validation on outbound publish '%s'\n",
+			ast_sorcery_object_get_id(publish));
+		pjsip_publishc_destroy(publisher->client);
+		return -1;
+	}
+
+	if (sip_outbound_publisher_set_uris(pool, publisher, &server_uri, &from_uri, &to_uri)) {
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		pjsip_publishc_destroy(publisher->client);
+		return -1;
+	}
+
+	pj_cstr(&event, publish->event);
+	if (pjsip_publishc_init(publisher->client, &event, &server_uri, &from_uri, &to_uri,
+			publish->expiration) != PJ_SUCCESS) {
+		ast_log(LOG_ERROR, "Failed to initialize publishing client on outbound publish '%s'\n",
+			ast_sorcery_object_get_id(publish));
+		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+		pjsip_publishc_destroy(publisher->client);
+		return -1;
+	}
+
+	pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
+	return 0;
+}
+
+static int sip_outbound_publisher_reinit(void *obj, void *arg, int flags)
+{
+	return sip_outbound_publisher_init(obj);
+}
+
+static int sip_outbound_publisher_reinit_all(void *data)
+{
+	ao2_callback(data, OBJ_NODATA, sip_outbound_publisher_reinit, NULL);
+	return 0;
+}
+
 /*! \brief Destructor function for publish client */
-static void sip_outbound_publish_client_destroy(void *obj)
+static void sip_outbound_publisher_destroy(void *obj)
 {
-	struct ast_sip_outbound_publish_client *client = obj;
+	struct sip_outbound_publisher *publisher = obj;
 	struct sip_outbound_publish_message *message;
 
 	/* You might be tempted to think "the publish client isn't being destroyed" but it actually is - just elsewhere */
 
-	while ((message = AST_LIST_REMOVE_HEAD(&client->queue, entry))) {
+	while ((message = AST_LIST_REMOVE_HEAD(&publisher->queue, entry))) {
 		ast_free(message);
 	}
 
-	ao2_cleanup(client->datastores);
-	ao2_cleanup(client->publish);
-	ast_free(client->transport_name);
-	ast_taskprocessor_unreference(client->serializer);
+	ao2_cleanup(publisher->owner);
+	ast_free(publisher->from_uri);
+	ast_free(publisher->to_uri);
+
+	ast_taskprocessor_unreference(publisher->serializer);
+}
+
+static struct sip_outbound_publisher *sip_outbound_publisher_alloc(
+	struct ast_sip_outbound_publish_client *client, const char *user)
+{
+	struct sip_outbound_publisher *publisher;
+	char tps_name[AST_TASKPROCESSOR_MAX_NAME + 1];
+
+	publisher = ao2_alloc(sizeof(*publisher) + (user ? strlen(user) : 0) + 1,
+			      sip_outbound_publisher_destroy);
+	if (!publisher) {
+		return NULL;
+	}
+
+	/*
+	 * Bump the ref to the client. This essentially creates a circular reference,
+	 * but it is needed in order to make sure the client object doesn't get pulled
+	 * out from under us when the publisher stops publishing.
+	 *
+	 * The circular reference is alleviated by calling cancel_and_unpublish for
+	 * each client, from the state's destructor. By calling it there all references
+	 * to the publishers should go to zero, thus calling the publisher's destructor.
+	 * This in turn removes the client reference we added here. The state then removes
+	 * its reference to the client, which should take it to zero.
+	 */
+	publisher->owner = ao2_bump(client);
+	publisher->timer.user_data = publisher;
+	publisher->timer.cb = sip_outbound_publish_timer_cb;
+	if (user) {
+		strcpy(publisher->user, user);
+	} else {
+		*publisher->user = '\0';
+	}
+
+	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/outpub/%s",
+		ast_sorcery_object_get_id(client->publish));
+
+	publisher->serializer = ast_sip_create_serializer_group(tps_name,
+		shutdown_group);
+	if (!publisher->serializer) {
+		ao2_ref(publisher, -1);
+		return NULL;
+	}
+
+	if (ast_sip_push_task_wait_servant(NULL, sip_outbound_publisher_init, publisher)) {
+		ast_log(LOG_ERROR, "Unable to create publisher for outbound publish '%s'\n",
+			ast_sorcery_object_get_id(client->publish));
+		ao2_ref(publisher, -1);
+		return NULL;
+	}
+
+	return publisher;
+}
+
+static struct sip_outbound_publisher *sip_outbound_publish_client_add_publisher(
+	struct ast_sip_outbound_publish_client *client, const char *user)
+{
+	struct sip_outbound_publisher *publisher =
+		sip_outbound_publisher_alloc(client, user);
+
+	if (!publisher) {
+		return NULL;
+	}
+
+	if (!ao2_link(client->publishers, publisher)) {
+		/*
+		 * No need to bump the reference here. The task will take care of
+		 * removing the reference.
+		 */
+		if (ast_sip_push_task(publisher->serializer, cancel_refresh_timer_task, publisher)) {
+			ao2_ref(publisher, -1);
+		}
+		return NULL;
+	}
+
+	return publisher;
+}
+
+int ast_sip_publish_client_user_send(struct ast_sip_outbound_publish_client *client,
+				     const char *user, const struct ast_sip_body *body)
+{
+	struct sip_outbound_publisher *publisher;
+	int res;
+
+	publisher = sip_outbound_publish_client_get_publisher(client, user);
+	if (!publisher) {
+		return -1;
+	}
+
+	publisher_client_send(publisher, (void *)body, &res, 0);
+	ao2_ref(publisher, -1);
+	return res;
+}
+
+void ast_sip_publish_client_remove(struct ast_sip_outbound_publish_client *client,
+				   const char *user)
+{
+	SCOPED_WRLOCK(lock, &load_lock);
+	ao2_find(client->publishers, user, OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
 }
 
 static int explicit_publish_destroy(void *data)
 {
-	struct ast_sip_outbound_publish_client *client = data;
+	struct sip_outbound_publisher *publisher = data;
 
 	/*
 	 * If there is no pjsip publishing client then we obviously don't need
 	 * to destroy it. Also, the ref for the Asterisk publishing client that
 	 * pjsip had would not exist or should already be gone as well.
 	 */
-	if (client->client) {
-		pjsip_publishc_destroy(client->client);
-		ao2_ref(client, -1);
+	if (publisher->client) {
+		pjsip_publishc_destroy(publisher->client);
+		ao2_ref(publisher, -1);
 	}
 
+	ao2_ref(publisher, -1);
+
 	return 0;
 }
 
 /*! \brief Helper function which cancels and un-publishes a no longer used client */
-static int cancel_and_unpublish(struct ast_sip_outbound_publish_client *client)
+static int cancel_and_unpublish(void *obj, void *arg, int flags)
 {
-	struct ast_sip_event_publisher_handler *handler;
-	SCOPED_AO2LOCK(lock, client);
+	struct sip_outbound_publisher *publisher = obj;
+	struct ast_sip_outbound_publish_client *client = publisher->owner;
+
+	SCOPED_AO2LOCK(lock, publisher);
 
 	if (!client->started) {
-		/* If the client was never started, there's nothing to unpublish, so just
-		 * destroy the publication and remove its reference to the client.
+		/* If the publisher was never started, there's nothing to unpublish, so just
+		 * destroy the publication and remove its reference to the publisher.
 		 */
-		ast_sip_push_task(client->serializer, explicit_publish_destroy, client);
+		if (ast_sip_push_task(publisher->serializer, explicit_publish_destroy, ao2_bump(publisher))) {
+			ao2_ref(publisher, -1);
+		}
 		return 0;
 	}
 
-	handler = find_publisher_handler_for_event_name(client->publish->event);
-	if (handler) {
-		handler->stop_publishing(client);
-	}
-
-	client->started = 0;
-	if (ast_sip_push_task(client->serializer, cancel_refresh_timer_task, ao2_bump(client))) {
+	if (ast_sip_push_task(publisher->serializer, cancel_refresh_timer_task, ao2_bump(publisher))) {
 		ast_log(LOG_WARNING, "Could not stop refresh timer on outbound publish '%s'\n",
 			ast_sorcery_object_get_id(client->publish));
-		ao2_ref(client, -1);
+		ao2_ref(publisher, -1);
 	}
 
 	/* If nothing is being sent right now send the unpublish - the destroy will happen in the subsequent callback */
-	if (!client->sending) {
-		if (ast_sip_push_task(client->serializer, send_unpublish_task, ao2_bump(client))) {
+	if (!publisher->sending) {
+		if (ast_sip_push_task(publisher->serializer, send_unpublish_task, ao2_bump(publisher))) {
 			ast_log(LOG_WARNING, "Could not send unpublish message on outbound publish '%s'\n",
 				ast_sorcery_object_get_id(client->publish));
-			ao2_ref(client, -1);
+			ao2_ref(publisher, -1);
 		}
 	}
-	client->destroy = 1;
+	publisher->destroy = 1;
 	return 0;
 }
 
+/*! \brief Destructor function for publish client */
+static void sip_outbound_publish_client_destroy(void *obj)
+{
+	struct ast_sip_outbound_publish_client *client = obj;
+
+	ao2_cleanup(client->datastores);
+
+	/*
+	 * The client's publishers have already been unpublished and destroyed
+	 * by this point, so it is safe to finally remove the reference to the
+	 * publish object. The client needed to hold a reference to it until
+	 * the publishers were done with it.
+	 */
+	ao2_cleanup(client->publish);
+}
+
 /*! \brief Destructor function for publish state */
 static void sip_outbound_publish_state_destroy(void *obj)
 {
 	struct ast_sip_outbound_publish_state *state = obj;
 
-	cancel_and_unpublish(state->client);
+	stop_publishing(state->client, NULL);
+	/*
+	 * Since the state is being destroyed the associated client needs to also
+	 * be destroyed. However simply removing the reference to the client will
+	 * not initiate client destruction since the client's publisher(s) hold a
+	 * reference to the client object as well. So we need to unpublish the
+	 * the client's publishers here, which will remove the publisher's client
+	 * reference during that process.
+	 *
+	 * That being said we don't want to remove the client's reference to the
+	 * publish object just yet. We'll hold off on that until client destruction
+	 * itself. This is because the publishers need access to the client's
+	 * publish object while they are unpublishing.
+	 */
+	ao2_callback(state->client->publishers, OBJ_NODATA | OBJ_UNLINK, cancel_and_unpublish, NULL);
+	ao2_cleanup(state->client->publishers);
+
+	state->client->started = 0;
 	ao2_cleanup(state->client);
 }
 
@@ -793,145 +1254,46 @@ static int can_reuse_publish(struct ast_
 	return 1;
 }
 
-static void sip_outbound_publish_callback(struct pjsip_publishc_cbparam *param);
-
-/*! \brief Helper function that allocates a pjsip publish client and configures it */
-static int sip_outbound_publish_client_alloc(void *data)
-{
-	struct ast_sip_outbound_publish_client *client = data;
-	RAII_VAR(struct ast_sip_outbound_publish *, publish, NULL, ao2_cleanup);
-	pjsip_publishc_opt opt = {
-		.queue_request = PJ_FALSE,
-	};
-	pj_str_t event, server_uri, to_uri, from_uri;
-	pj_status_t status;
-
-	if (client->client) {
-		return 0;
-	} else if (pjsip_publishc_create(ast_sip_get_pjsip_endpoint(), &opt, ao2_bump(client), sip_outbound_publish_callback,
-		&client->client) != PJ_SUCCESS) {
-		ao2_ref(client, -1);
-		return -1;
-	}
-
-	publish = ao2_bump(client->publish);
-
-	if (!ast_strlen_zero(publish->outbound_proxy)) {
-		pjsip_route_hdr route_set, *route;
-		static const pj_str_t ROUTE_HNAME = { "Route", 5 };
-
-		pj_list_init(&route_set);
-
-		if (!(route = pjsip_parse_hdr(pjsip_publishc_get_pool(client->client), &ROUTE_HNAME,
-			(char*)publish->outbound_proxy, strlen(publish->outbound_proxy), NULL))) {
-			pjsip_publishc_destroy(client->client);
-			return -1;
-		}
-		pj_list_insert_nodes_before(&route_set, route);
-
-		pjsip_publishc_set_route_set(client->client, &route_set);
-	}
-
-	pj_cstr(&event, publish->event);
-	pj_cstr(&server_uri, publish->server_uri);
-	pj_cstr(&to_uri, S_OR(publish->to_uri, publish->server_uri));
-	pj_cstr(&from_uri, S_OR(publish->from_uri, publish->server_uri));
-
-	status = pjsip_publishc_init(client->client, &event, &server_uri, &from_uri, &to_uri,
-		publish->expiration);
-	if (status == PJSIP_EINVALIDURI) {
-		pj_pool_t *pool;
-		pj_str_t tmp;
-		pjsip_uri *uri;
-
-		pool = pjsip_endpt_create_pool(ast_sip_get_pjsip_endpoint(), "URI Validation", 256, 256);
-		if (!pool) {
-			ast_log(LOG_ERROR, "Could not create pool for URI validation on outbound publish '%s'\n",
-				ast_sorcery_object_get_id(publish));
-			pjsip_publishc_destroy(client->client);
-			return -1;
-		}
-
-		pj_strdup2_with_null(pool, &tmp, publish->server_uri);
-		uri = pjsip_parse_uri(pool, tmp.ptr, tmp.slen, 0);
-		if (!uri) {
-			ast_log(LOG_ERROR, "Invalid server URI '%s' specified on outbound publish '%s'\n",
-				publish->server_uri, ast_sorcery_object_get_id(publish));
-		}
-
-		if (!ast_strlen_zero(publish->to_uri)) {
-			pj_strdup2_with_null(pool, &tmp, publish->to_uri);
-			uri = pjsip_parse_uri(pool, tmp.ptr, tmp.slen, 0);
-			if (!uri) {
-				ast_log(LOG_ERROR, "Invalid to URI '%s' specified on outbound publish '%s'\n",
-					publish->to_uri, ast_sorcery_object_get_id(publish));
-			}
-		}
-
-		if (!ast_strlen_zero(publish->from_uri)) {
-			pj_strdup2_with_null(pool, &tmp, publish->from_uri);
-			uri = pjsip_parse_uri(pool, tmp.ptr, tmp.slen, 0);
-			if (!uri) {
-				ast_log(LOG_ERROR, "Invalid from URI '%s' specified on outbound publish '%s'\n",
-					publish->from_uri, ast_sorcery_object_get_id(publish));
-			}
-		}
-
-		pjsip_endpt_release_pool(ast_sip_get_pjsip_endpoint(), pool);
-		pjsip_publishc_destroy(client->client);
-		return -1;
-	} else if (status != PJ_SUCCESS) {
-		pjsip_publishc_destroy(client->client);
-		return -1;
-	}
-
-	return 0;
-}
-
 /*! \brief Callback function for publish client responses */
 static void sip_outbound_publish_callback(struct pjsip_publishc_cbparam *param)
 {
 #define DESTROY_CLIENT() do {			   \
-	pjsip_publishc_destroy(client->client); \
-	client->client = NULL; \
-	ao2_ref(client, -1); } while (0)
-
-	RAII_VAR(struct ast_sip_outbound_publish_client *, client, ao2_bump(param->token), ao2_cleanup);
-	RAII_VAR(struct ast_sip_outbound_publish *, publish, ao2_bump(client->publish), ao2_cleanup);
-	SCOPED_AO2LOCK(lock, client);
+	pjsip_publishc_destroy(publisher->client); \
+	publisher->client = NULL; \
+	ao2_ref(publisher, -1); } while (0)
+
+	RAII_VAR(struct sip_outbound_publisher *, publisher, ao2_bump(param->token), ao2_cleanup);
+	RAII_VAR(struct ast_sip_outbound_publish *, publish, ao2_bump(publisher->owner->publish), ao2_cleanup);
+	SCOPED_AO2LOCK(lock, publisher);
 	pjsip_tx_data *tdata;
 
-	if (client->destroy) {
-		if (client->sending) {
-			client->sending = NULL;
+	if (publisher->destroy) {
+		if (publisher->sending) {
+			publisher->sending = NULL;
 
-			if (!ast_sip_push_task(client->serializer, send_unpublish_task, ao2_bump(client))) {
+			if (!ast_sip_push_task(publisher->serializer, send_unpublish_task, ao2_bump(publisher))) {
 				return;
 			}
 			ast_log(LOG_WARNING, "Could not send unpublish message on outbound publish '%s'\n",
 				ast_sorcery_object_get_id(publish));
-			ao2_ref(client, -1);
+			ao2_ref(publisher, -1);
 		}
-		/* Once the destroy is called this callback will not get called any longer, so drop the client ref */
+		/* Once the destroy is called this callback will not get called any longer, so drop the publisher ref */
 		DESTROY_CLIENT();
 		return;
 	}
 
 	if (param->code == 401 || param->code == 407) {
-		if (!ast_sip_create_request_with_auth(&publish->outbound_auths,
-				param->rdata, pjsip_rdata_get_tsx(param->rdata), &tdata)) {
-			if (!ast_strlen_zero(client->transport_name)) {
-				pjsip_tpselector selector = { .type = PJSIP_TPSELECTOR_NONE, };
-				ast_sip_set_tpselector_from_transport_name(client->transport_name, &selector);
-				pjsip_tx_data_set_transport(tdata, &selector);
-			}
+		pjsip_transaction *tsx = pjsip_rdata_get_tsx(param->rdata);
 
-			ast_sip_record_request_serializer(tdata);
-			pjsip_publishc_send(client->client, tdata);
+		if (!ast_sip_create_request_with_auth(&publish->outbound_auths,
+				param->rdata, tsx->last_tx, &tdata)) {
+			set_transport(publisher, tdata);
+			pjsip_publishc_send(publisher->client, tdata);
 		}
-		client->auth_attempts++;
+		publisher->auth_attempts++;
 
-		if (client->auth_attempts == publish->max_auth_attempts) {
+		if (publisher->auth_attempts == publish->max_auth_attempts) {
 			DESTROY_CLIENT();
 			ast_log(LOG_ERROR, "Reached maximum number of PUBLISH authentication attempts on outbound publish '%s'\n",
 				ast_sorcery_object_get_id(publish));
@@ -941,18 +1303,18 @@ static void sip_outbound_publish_callbac
 		return;
 	}
 
-	client->auth_attempts = 0;
+	publisher->auth_attempts = 0;
 
 	if (param->code == 412) {
 		DESTROY_CLIENT();
-		if (sip_outbound_publish_client_alloc(client)) {
+		if (sip_outbound_publisher_init(publisher)) {
 			ast_log(LOG_ERROR, "Failed to create a new outbound publish client for '%s' on 412 response\n",
 				ast_sorcery_object_get_id(publish));
 			goto end;
 		}
 
 		/* Setting this to NULL will cause a new PUBLISH to get created and sent for the same underlying body */
-		client->sending = NULL;
+		publisher->sending = NULL;
 	} else if (param->code == 423) {
 		/* Update the expiration with the new expiration time if available */
 		pjsip_expires_hdr *expires;
@@ -965,33 +1327,33 @@ static void sip_outbound_publish_callbac
 			goto end;
 		}
 
-		pjsip_publishc_update_expires(client->client, expires->ivalue);
-		client->sending = NULL;
-	} else if (client->sending) {
+		pjsip_publishc_update_expires(publisher->client, expires->ivalue);
+		publisher->sending = NULL;
+	} else if (publisher->sending) {
 		/* Remove the message currently being sent so that when the queue is serviced another will get sent */
-		AST_LIST_REMOVE_HEAD(&client->queue, entry);
-		ast_free(client->sending);
-		client->sending = NULL;
+		AST_LIST_REMOVE_HEAD(&publisher->queue, entry);
+		ast_free(publisher->sending);
+		publisher->sending = NULL;
 		if (!param->rdata) {
 			ast_log(LOG_NOTICE, "No response received for outbound publish '%s'\n",
 				ast_sorcery_object_get_id(publish));
 		}
 	}
 
-	if (AST_LIST_EMPTY(&client->queue)) {
-		schedule_publish_refresh(client, param->expiration);
+	if (AST_LIST_EMPTY(&publisher->queue)) {
+		schedule_publish_refresh(publisher, param->expiration);
 	}
 
 end:
-	if (!client->client) {
+	if (!publisher->client) {
 		struct sip_outbound_publish_message *message;
 
-		while ((message = AST_LIST_REMOVE_HEAD(&client->queue, entry))) {
+		while ((message = AST_LIST_REMOVE_HEAD(&publisher->queue, entry))) {
 			ast_free(message);
 		}
 	} else {
-		if (ast_sip_push_task(client->serializer, sip_publish_client_service_queue, ao2_bump(client))) {
-			ao2_ref(client, -1);
+		if (ast_sip_push_task(publisher->serializer, sip_publisher_service_queue, ao2_bump(publisher))) {
+			ao2_ref(publisher, -1);
 		}
 	}
 }
@@ -1062,7 +1424,6 @@ static struct ast_sip_outbound_publish_s
 	const char *id = ast_sorcery_object_get_id(publish);
 	struct ast_sip_outbound_publish_state *state =
 		ao2_alloc(sizeof(*state) + strlen(id) + 1, sip_outbound_publish_state_destroy);
-	char tps_name[AST_TASKPROCESSOR_MAX_NAME + 1];
 
 	if (!state) {
 		return NULL;
@@ -1074,44 +1435,27 @@ static struct ast_sip_outbound_publish_s
 		return NULL;
 	}
 
-	state->client->datastores = ao2_container_alloc(DATASTORE_BUCKETS, datastore_hash, datastore_cmp);
+	state->client->datastores = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS, datastore_hash, NULL, datastore_cmp);
 	if (!state->client->datastores) {
 		ao2_ref(state, -1);
 		return NULL;
 	}
 
-	/* Create name with seq number appended. */
-	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/outpub/%s",
-		ast_sorcery_object_get_id(publish));
-
-	state->client->serializer = ast_sip_create_serializer_group_named(tps_name,
-		shutdown_group);
-	if (!state->client->serializer) {
+	state->client->publishers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS,
+		sip_outbound_publisher_hash_fn, NULL, sip_outbound_publisher_cmp_fn);
+	if (!state->client->publishers) {
 		ao2_ref(state, -1);
 		return NULL;
 	}
 
-	pj_timer_entry_init(&state->client->timer, 0, state->client, sip_outbound_publish_timer_cb);
-	state->client->transport_name = ast_strdup(publish->transport);
 	state->client->publish = ao2_bump(publish);
 
 	strcpy(state->id, id);
 	return state;
 }
 
-static int initialize_publish_client(struct ast_sip_outbound_publish *publish,
-				     struct ast_sip_outbound_publish_state *state)
-{
-	if (ast_sip_push_task_wait_serializer(state->client->serializer,
-		sip_outbound_publish_client_alloc, state->client)) {
-		ast_log(LOG_ERROR, "Unable to create client for outbound publish '%s'\n",
-			ast_sorcery_object_get_id(publish));
-		return -1;
-	}
-
-	return 0;
-}
-
 static int validate_publish_config(struct ast_sip_outbound_publish *publish)
 {
 	if (ast_strlen_zero(publish->server_uri)) {
@@ -1148,6 +1492,15 @@ static int current_state_reusable(struct
 {
 	struct ast_sip_outbound_publish *old_publish;
 
+	/*
+	 * Don't maintain the old state/client objects if the multi_user option changed.
+	 */
+	if ((!publish->multi_user && current_state->client->publish->multi_user) ||
+	    (publish->multi_user && !current_state->client->publish->multi_user)) {
+		return 0;
+	}
+
+
 	if (!can_reuse_publish(current_state->client->publish, publish)) {
 		/*
 		 * Something significant has changed in the configuration, so we are
@@ -1163,12 +1516,15 @@ static int current_state_reusable(struct
 	 */
 	old_publish = current_state->client->publish;
 	current_state->client->publish = publish;
-	if (initialize_publish_client(publish, current_state)) {
+	if (ast_sip_push_task_wait_servant(NULL, sip_outbound_publisher_reinit_all,
+		current_state->client->publishers)) {
 		/*
 		 * If the state object fails to re-initialize then swap
 		 * the old publish info back in.
 		 */
 		current_state->client->publish = publish;
+		ast_log(LOG_ERROR, "Unable to reinitialize client(s) for outbound publish '%s'\n",
+			ast_sorcery_object_get_id(current_state->client->publish));
 		return -1;
 	}
 
@@ -1193,6 +1549,7 @@ static int sip_outbound_publish_apply(co
 
 	struct ast_sip_outbound_publish *applied = obj;
 	struct ast_sip_outbound_publish_state *current_state, *new_state;
+	struct sip_outbound_publisher *publisher = NULL;
 	int res;
 
 	/*
@@ -1200,9 +1557,9 @@ static int sip_outbound_publish_apply(co
 	 * object if created/updated, or keep the old object if an error occurs.
 	 */
 	if (!new_states) {
-		new_states = ao2_container_alloc_options(
-			AO2_ALLOC_OPT_LOCK_NOLOCK, DEFAULT_STATE_BUCKETS,
-			outbound_publish_state_hash, outbound_publish_state_cmp);
+		new_states = ao2_container_alloc_hash(
+			AO2_ALLOC_OPT_LOCK_NOLOCK, 0, DEFAULT_STATE_BUCKETS,
+			outbound_publish_state_hash, NULL, outbound_publish_state_cmp);
 
 		if (!new_states) {
 			ast_log(LOG_ERROR, "Unable to allocate new states container\n");
@@ -1239,11 +1596,13 @@ static int sip_outbound_publish_apply(co
 		return -1;
 	};
 
-	if (initialize_publish_client(applied, new_state)) {
+	if (!applied->multi_user &&
+	    !(publisher = sip_outbound_publish_client_add_publisher(new_state->client, NULL))) {
 		ADD_TO_NEW_STATES(current_state);
 		ao2_ref(new_state, -1);
 		return -1;
 	}
+	ao2_cleanup(publisher);
 
 	ADD_TO_NEW_STATES(new_state);
 	ao2_cleanup(current_state);
@@ -1285,7 +1644,8 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
+	/* As of pjproject 2.4.5, PJSIP_MAX_URL_SIZE isn't exposed yet but we try anyway. */
+	ast_pjproject_get_buildopt("PJSIP_MAX_URL_SIZE", "%d", &pjsip_max_url_size);
 
 	shutdown_group = ast_serializer_shutdown_group_alloc();
 	if (!shutdown_group) {
@@ -1312,6 +1672,7 @@ static int load_module(void)
 	ast_sorcery_object_field_register(ast_sip_get_sorcery(), "outbound-publish", "max_auth_attempts", "5", OPT_UINT_T, 0, FLDSET(struct ast_sip_outbound_publish, max_auth_attempts));
 	ast_sorcery_object_field_register(ast_sip_get_sorcery(), "outbound-publish", "transport", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_outbound_publish, transport));
 	ast_sorcery_object_field_register_custom(ast_sip_get_sorcery(), "outbound-publish", "outbound_auth", "", outbound_auth_handler, NULL, NULL, 0, 0);
+	ast_sorcery_object_field_register(ast_sip_get_sorcery(), "outbound-publish", "multi_user", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_outbound_publish, multi_user));
 
 	ast_sorcery_reload_object(ast_sip_get_sorcery(), "outbound-publish");
 
@@ -1335,9 +1696,10 @@ static int reload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "PJSIP Outbound Publish Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.reload = reload_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.reload = reload_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjproject,res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_outbound_registration.c 1:16.1.1~dfsg-1/res/res_pjsip_outbound_registration.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_outbound_registration.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_outbound_registration.c	2018-12-26 18:45:13.000000000 +0000
@@ -545,7 +545,6 @@ static pj_status_t registration_client_s
 	 */
 	ast_sip_set_tpselector_from_transport_name(client_state->transport_name, &selector);
 	pjsip_regc_set_transport(client_state->client, &selector);
-	ast_sip_record_request_serializer(tdata);
 	status = pjsip_regc_send(client_state->client, tdata);
 
 	/* If the attempt to send the message failed and the callback was not invoked we need to
@@ -913,7 +912,7 @@ static int handle_registration_response(
 		pjsip_cseq_hdr *cseq_hdr;
 		pjsip_tx_data *tdata;
 
-		if (!ast_sip_create_request_with_auth_from_old(&response->client_state->outbound_auths,
+		if (!ast_sip_create_request_with_auth(&response->client_state->outbound_auths,
 				response->rdata, response->old_request, &tdata)) {
 			response->client_state->auth_attempted = 1;
 			ast_debug(1, "Sending authenticated REGISTER to server '%s' from client '%s'\n",
@@ -1153,7 +1152,7 @@ static struct sip_outbound_registration_
 	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/outreg/%s",
 		ast_sorcery_object_get_id(registration));
 
-	state->client_state->serializer = ast_sip_create_serializer_group_named(tps_name,
+	state->client_state->serializer = ast_sip_create_serializer_group(tps_name,
 		shutdown_group);
 	if (!state->client_state->serializer) {
 		ao2_cleanup(state);
@@ -2189,16 +2188,14 @@ static int load_module(void)
 {
 	struct ao2_container *new_states;
 
-	CHECK_PJSIP_MODULE_LOADED();
-
 	shutdown_group = ast_serializer_shutdown_group_alloc();
 	if (!shutdown_group) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	/* Create outbound registration states container. */
-	new_states = ao2_container_alloc(DEFAULT_STATE_BUCKETS,
-		registration_state_hash, registration_state_cmp);
+	new_states = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DEFAULT_STATE_BUCKETS, registration_state_hash, NULL, registration_state_cmp);
 	if (!new_states) {
 		ast_log(LOG_ERROR, "Unable to allocate registration states container\n");
 		unload_module();
@@ -2285,6 +2282,8 @@ static int load_module(void)
 
 	network_change_sub = stasis_subscribe(ast_system_topic(),
 		network_change_stasis_cb, NULL);
+	stasis_subscription_accept_message_type(network_change_sub, ast_network_change_type());
+	stasis_subscription_set_filter(network_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -2296,9 +2295,11 @@ static int reload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Outbound Registration Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.reload = reload_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.reload = reload_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+	.optional_modules = "res_statsd",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_path.c 1:16.1.1~dfsg-1/res/res_pjsip_path.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_path.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_path.c	2018-12-26 18:45:13.000000000 +0000
@@ -238,16 +238,8 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
-	if (ast_sip_register_supplement(&path_supplement)) {
-		return AST_MODULE_LOAD_DECLINE;
-	}
-
-	if (ast_sip_session_register_supplement(&path_session_supplement)) {
-		ast_sip_unregister_supplement(&path_supplement);
-		return AST_MODULE_LOAD_DECLINE;
-	}
+	ast_sip_register_supplement(&path_supplement);
+	ast_sip_session_register_supplement(&path_session_supplement);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -260,8 +252,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Path Header Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_phoneprov_provider.c 1:16.1.1~dfsg-1/res/res_pjsip_phoneprov_provider.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_phoneprov_provider.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_phoneprov_provider.c	2018-12-26 18:45:13.000000000 +0000
@@ -367,8 +367,6 @@ static int load_users(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	sorcery = ast_sip_get_sorcery();
 
 	ast_sorcery_apply_config(sorcery, "res_pjsip_phoneprov_provider");
@@ -413,9 +411,10 @@ static int reload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Phoneprov Provider",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.reload = reload_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.reload = reload_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_phoneprov",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_pidf_body_generator.c 1:16.1.1~dfsg-1/res/res_pjsip_pidf_body_generator.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_pidf_body_generator.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_pidf_body_generator.c	2018-12-26 18:45:13.000000000 +0000
@@ -116,8 +116,6 @@ static struct ast_sip_pubsub_body_genera
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_generator(&pidf_body_generator)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -131,8 +129,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Extension State PIDF Provider",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_pidf_digium_body_supplement.c 1:16.1.1~dfsg-1/res/res_pjsip_pidf_digium_body_supplement.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_pidf_digium_body_supplement.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_pidf_digium_body_supplement.c	2018-12-26 18:45:13.000000000 +0000
@@ -95,8 +95,6 @@ static struct ast_sip_pubsub_body_supple
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_supplement(&pidf_supplement)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -110,8 +108,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP PIDF Digium presence supplement",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_pidf_eyebeam_body_supplement.c 1:16.1.1~dfsg-1/res/res_pjsip_pidf_eyebeam_body_supplement.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_pidf_eyebeam_body_supplement.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_pidf_eyebeam_body_supplement.c	2018-12-26 18:45:13.000000000 +0000
@@ -92,8 +92,6 @@ static struct ast_sip_pubsub_body_supple
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_supplement(&pidf_supplement)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -107,8 +105,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP PIDF Eyebeam supplement",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_publish_asterisk.c 1:16.1.1~dfsg-1/res/res_pjsip_publish_asterisk.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_publish_asterisk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_publish_asterisk.c	2018-12-26 18:45:13.000000000 +0000
@@ -360,6 +360,9 @@ static int asterisk_start_devicestate_pu
 		ao2_ref(datastore, -1);
 		return -1;
 	}
+	stasis_subscription_accept_message_type(publisher_state->device_state_subscription, ast_device_state_message_type());
+	stasis_subscription_accept_message_type(publisher_state->device_state_subscription, stasis_subscription_change_type());
+	stasis_subscription_set_filter(publisher_state->device_state_subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	cached = stasis_cache_dump(ast_device_state_cache(), NULL);
 	ao2_callback(cached, OBJ_NODATA, cached_devstate_cb, datastore);
@@ -435,6 +438,9 @@ static int asterisk_start_mwi_publishing
 		ao2_ref(datastore, -1);
 		return -1;
 	}
+	stasis_subscription_accept_message_type(publisher_state->mailbox_state_subscription, ast_mwi_state_type());
+	stasis_subscription_accept_message_type(publisher_state->mailbox_state_subscription, stasis_subscription_change_type());
+	stasis_subscription_set_filter(publisher_state->mailbox_state_subscription, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	cached = stasis_cache_dump(ast_mwi_state_cache(), NULL);
 	ao2_callback(cached, OBJ_NODATA, cached_mwistate_cb, datastore);
@@ -855,8 +861,6 @@ static int regex_filter_handler(const st
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_eid_is_empty(&ast_eid_default)) {
 		ast_log(LOG_ERROR, "Entity ID is not set.\n");
 		return AST_MODULE_LOAD_DECLINE;
@@ -934,4 +938,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.reload = reload_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_CHANNEL_DEPEND + 5,
+	.requires = "res_pjsip,res_pjsip_outbound_publish,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_pubsub.c 1:16.1.1~dfsg-1/res/res_pjsip_pubsub.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_pubsub.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_pubsub.c	2018-12-26 18:45:13.000000000 +0000
@@ -1160,28 +1160,6 @@ static int build_resource_tree(struct as
 	}
 }
 
-static int datastore_hash(const void *obj, int flags)
-{
-	const struct ast_datastore *datastore = obj;
-	const char *uid = flags & OBJ_KEY ? obj : datastore->uid;
-
-	ast_assert(uid != NULL);
-
-	return ast_str_hash(uid);
-}
-
-static int datastore_cmp(void *obj, void *arg, int flags)
-{
-	const struct ast_datastore *datastore1 = obj;
-	const struct ast_datastore *datastore2 = arg;
-	const char *uid2 = flags & OBJ_KEY ? arg : datastore2->uid;
-
-	ast_assert(datastore1->uid != NULL);
-	ast_assert(uid2 != NULL);
-
-	return strcmp(datastore1->uid, uid2) ? 0 : CMP_MATCH | CMP_STOP;
-}
-
 static void add_subscription(struct sip_subscription_tree *obj)
 {
 	AST_RWLIST_WRLOCK(&subscriptions);
@@ -1251,7 +1229,7 @@ static struct ast_sip_subscription *allo
 	}
 	strcpy(sub->resource, resource); /* Safe */
 
-	sub->datastores = ao2_container_alloc(DATASTORE_BUCKETS, datastore_hash, datastore_cmp);
+	sub->datastores = ast_datastores_alloc();
 	if (!sub->datastores) {
 		destroy_subscription(sub);
 		return NULL;
@@ -1430,7 +1408,7 @@ static struct sip_subscription_tree *all
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/pubsub/%s",
 			ast_sorcery_object_get_id(endpoint));
 
-		sub_tree->serializer = ast_sip_create_serializer_named(tps_name);
+		sub_tree->serializer = ast_sip_create_serializer(tps_name);
 	}
 	if (!sub_tree->serializer) {
 		ao2_ref(sub_tree, -1);
@@ -2619,92 +2597,49 @@ static int sip_subscription_accept(struc
 	return pjsip_evsub_accept(sub_tree->evsub, rdata, response, &res_hdr) == PJ_SUCCESS ? 0 : -1;
 }
 
-static void subscription_datastore_destroy(void *obj)
-{
-	struct ast_datastore *datastore = obj;
-
-	/* Using the destroy function (if present) destroy the data */
-	if (datastore->info->destroy != NULL && datastore->data != NULL) {
-		datastore->info->destroy(datastore->data);
-		datastore->data = NULL;
-	}
-
-	ast_free((void *) datastore->uid);
-	datastore->uid = NULL;
-}
-
 struct ast_datastore *ast_sip_subscription_alloc_datastore(const struct ast_datastore_info *info, const char *uid)
 {
-	RAII_VAR(struct ast_datastore *, datastore, NULL, ao2_cleanup);
-	char uuid_buf[AST_UUID_STR_LEN];
-	const char *uid_ptr = uid;
-
-	if (!info) {
-		return NULL;
-	}
-
-	datastore = ao2_alloc(sizeof(*datastore), subscription_datastore_destroy);
-	if (!datastore) {
-		return NULL;
-	}
-
-	datastore->info = info;
-	if (ast_strlen_zero(uid)) {
-		/* They didn't provide an ID so we'll provide one ourself */
-		uid_ptr = ast_uuid_generate_str(uuid_buf, sizeof(uuid_buf));
-	}
-
-	datastore->uid = ast_strdup(uid_ptr);
-	if (!datastore->uid) {
-		return NULL;
-	}
-
-	ao2_ref(datastore, +1);
-	return datastore;
+	return ast_datastores_alloc_datastore(info, uid);
 }
 
 int ast_sip_subscription_add_datastore(struct ast_sip_subscription *subscription, struct ast_datastore *datastore)
 {
-	ast_assert(datastore != NULL);
-	ast_assert(datastore->info != NULL);
-	ast_assert(!ast_strlen_zero(datastore->uid));
-
-	if (!ao2_link(subscription->datastores, datastore)) {
-		return -1;
-	}
-	return 0;
+	return ast_datastores_add(subscription->datastores, datastore);
 }
 
 struct ast_datastore *ast_sip_subscription_get_datastore(struct ast_sip_subscription *subscription, const char *name)
 {
-	return ao2_find(subscription->datastores, name, OBJ_KEY);
+	return ast_datastores_find(subscription->datastores, name);
 }
 
 void ast_sip_subscription_remove_datastore(struct ast_sip_subscription *subscription, const char *name)
 {
-	ao2_find(subscription->datastores, name, OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
+	ast_datastores_remove(subscription->datastores, name);
 }
 
-int ast_sip_publication_add_datastore(struct ast_sip_publication *publication, struct ast_datastore *datastore)
+struct ao2_container *ast_sip_subscription_get_datastores(const struct ast_sip_subscription *subscription)
 {
-	ast_assert(datastore != NULL);
-	ast_assert(datastore->info != NULL);
-	ast_assert(!ast_strlen_zero(datastore->uid));
+	return subscription->datastores;
+}
 
-	if (!ao2_link(publication->datastores, datastore)) {
-		return -1;
-	}
-	return 0;
+int ast_sip_publication_add_datastore(struct ast_sip_publication *publication, struct ast_datastore *datastore)
+{
+	return ast_datastores_add(publication->datastores, datastore);
 }
 
 struct ast_datastore *ast_sip_publication_get_datastore(struct ast_sip_publication *publication, const char *name)
 {
-	return ao2_find(publication->datastores, name, OBJ_KEY);
+	return ast_datastores_find(publication->datastores, name);
 }
 
 void ast_sip_publication_remove_datastore(struct ast_sip_publication *publication, const char *name)
 {
-	ao2_callback(publication->datastores, OBJ_KEY | OBJ_UNLINK | OBJ_NODATA, NULL, (void *) name);
+	ast_datastores_remove(publication->datastores, name);
+}
+
+struct ao2_container *ast_sip_publication_get_datastores(const struct ast_sip_publication *publication)
+{
+	return publication->datastores;
 }
 
 AST_RWLIST_HEAD_STATIC(publish_handlers, ast_sip_publish_handler);
@@ -2741,8 +2676,9 @@ int ast_sip_register_publish_handler(str
 		return -1;
 	}
 
-	if (!(handler->publications = ao2_container_alloc(PUBLICATIONS_BUCKETS,
-		publication_hash_fn, publication_cmp_fn))) {
+	handler->publications = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		PUBLICATIONS_BUCKETS, publication_hash_fn, NULL, publication_cmp_fn);
+	if (!handler->publications) {
 		ast_log(LOG_ERROR, "Could not allocate publications container for event '%s'\n",
 			handler->event_name);
 		return -1;
@@ -2750,8 +2686,6 @@ int ast_sip_register_publish_handler(str
 
 	publish_add_handler(handler);
 
-	ast_module_ref(ast_module_info->self);
-
 	return 0;
 }
 
@@ -2764,7 +2698,6 @@ void ast_sip_unregister_publish_handler(
 		if (handler == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
 			ao2_cleanup(handler->publications);
-			ast_module_unref(ast_module_info->self);
 			break;
 		}
 	}
@@ -2778,7 +2711,6 @@ static void sub_add_handler(struct ast_s
 {
 	AST_RWLIST_WRLOCK(&subscription_handlers);
 	AST_RWLIST_INSERT_TAIL(&subscription_handlers, handler, next);
-	ast_module_ref(ast_module_info->self);
 	AST_RWLIST_UNLOCK(&subscription_handlers);
 }
 
@@ -2837,7 +2769,6 @@ void ast_sip_unregister_subscription_han
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&subscription_handlers, iter, next) {
 		if (handler == iter) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			ast_module_unref(ast_module_info->self);
 			break;
 		}
 	}
@@ -3181,7 +3112,7 @@ static struct ast_sip_publication *sip_c
 
 	ast_module_ref(ast_module_info->self);
 
-	if (!(publication->datastores = ao2_container_alloc(DATASTORE_BUCKETS, datastore_hash, datastore_cmp))) {
+	if (!(publication->datastores = ast_datastores_alloc())) {
 		ao2_ref(publication, -1);
 		return NULL;
 	}
@@ -3452,6 +3383,11 @@ const char *ast_sip_publication_get_even
 	return pub->event_configuration_name;
 }
 
+int ast_sip_pubsub_is_body_generator_registered(const char *type, const char *subtype)
+{
+	return !!find_body_generator_type_subtype(type, subtype);
+}
+
 int ast_sip_pubsub_register_body_generator(struct ast_sip_pubsub_body_generator *generator)
 {
 	struct ast_sip_pubsub_body_generator *existing;
@@ -5544,8 +5480,6 @@ static int load_module(void)
 	static const pj_str_t str_PUBLISH = { "PUBLISH", 7 };
 	struct ast_sorcery *sorcery;
 
-	CHECK_PJSIP_MODULE_LOADED();
-
 	sorcery = ast_sip_get_sorcery();
 
 	if (!(sched = ast_sched_context_create())) {
@@ -5634,7 +5568,11 @@ static int load_module(void)
 	if (ast_test_flag(&ast_options, AST_OPT_FLAG_FULLY_BOOTED)) {
 		ast_sip_push_task(NULL, subscription_persistence_load, NULL);
 	} else {
-		stasis_subscribe_pool(ast_manager_get_topic(), subscription_persistence_event_cb, NULL);
+		struct stasis_subscription *sub;
+
+		sub = stasis_subscribe_pool(ast_manager_get_topic(), subscription_persistence_event_cb, NULL);
+		stasis_subscription_accept_message_type(sub, ast_manager_get_generic_type());
+		stasis_subscription_set_filter(sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 	}
 
 	ast_manager_register_xml(AMI_SHOW_SUBSCRIPTIONS_INBOUND, EVENT_FLAG_SYSTEM,
@@ -5684,8 +5622,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "PJSIP event resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_refer.c 1:16.1.1~dfsg-1/res/res_pjsip_refer.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_refer.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_refer.c	2018-12-26 18:45:13.000000000 +0000
@@ -393,7 +393,7 @@ static int refer_progress_alloc(struct a
 	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/refer/%s",
 		ast_sorcery_object_get_id(session->endpoint));
 
-	if (!((*progress)->serializer = ast_sip_create_serializer_named(tps_name))) {
+	if (!((*progress)->serializer = ast_sip_create_serializer(tps_name))) {
 		goto error;
 	}
 
@@ -686,6 +686,10 @@ static void refer_blind_callback(struct
 			ast_channel_unlock(chan);
 
 			ao2_cleanup(refer->progress);
+		} else {
+			stasis_subscription_accept_message_type(refer->progress->bridge_sub, ast_channel_entered_bridge_type());
+			stasis_subscription_accept_message_type(refer->progress->bridge_sub, stasis_subscription_change_type());
+			stasis_subscription_set_filter(refer->progress->bridge_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 		}
 	}
 
@@ -1224,8 +1228,6 @@ static int load_module(void)
 {
 	const pj_str_t str_norefersub = { "norefersub", 10 };
 
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	pjsip_replaces_init_module(ast_sip_get_pjsip_endpoint());
 	pjsip_xfer_init_module(ast_sip_get_pjsip_endpoint());
 	pjsip_endpt_add_capability(ast_sip_get_pjsip_endpoint(), NULL, PJSIP_H_SUPPORTED, NULL, 1, &str_norefersub);
@@ -1247,8 +1249,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Blind and Attended Transfer Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-		   );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session,res_pjsip_pubsub",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_registrar.c 1:16.1.1~dfsg-1/res/res_pjsip_registrar.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_registrar.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_registrar.c	2018-12-26 18:45:13.000000000 +0000
@@ -335,15 +335,15 @@ static int register_contact_transport_re
 {
 	struct contact_transport_monitor *monitor = data;
 	struct ast_sip_contact *contact;
-	struct ast_named_lock *lock;
+	struct ast_sip_aor *aor;
 
-	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", monitor->aor_name);
-	if (!lock) {
+	aor = ast_sip_location_retrieve_aor(monitor->aor_name);
+	if (!aor) {
 		ao2_ref(monitor, -1);
 		return 0;
 	}
 
-	ao2_lock(lock);
+	ao2_lock(aor);
 	contact = ast_sip_location_retrieve_contact(monitor->contact_name);
 	if (contact) {
 		ast_sip_location_delete_contact(contact);
@@ -358,8 +358,8 @@ static int register_contact_transport_re
 			contact->user_agent);
 		ao2_ref(contact, -1);
 	}
-	ao2_unlock(lock);
-	ast_named_lock_put(lock);
+	ao2_unlock(aor);
+	ao2_ref(aor, -1);
 
 	ao2_ref(monitor, -1);
 	return 0;
@@ -850,20 +850,11 @@ static int register_aor(pjsip_rx_data *r
 		.code = 500,
 	};
 	struct ao2_container *contacts = NULL;
-	struct ast_named_lock *lock;
 
-	lock = ast_named_lock_get(AST_NAMED_LOCK_TYPE_MUTEX, "aor", aor_name);
-	if (!lock) {
-		pjsip_endpt_respond_stateless(ast_sip_get_pjsip_endpoint(),
-			rdata, response.code, NULL, NULL, NULL);
-		return PJ_TRUE;
-	}
-
-	ao2_lock(lock);
+	ao2_lock(aor);
 	contacts = ast_sip_location_retrieve_aor_contacts_nolock(aor);
 	if (!contacts) {
-		ao2_unlock(lock);
-		ast_named_lock_put(lock);
+		ao2_unlock(aor);
 		pjsip_endpt_respond_stateless(ast_sip_get_pjsip_endpoint(),
 			rdata, response.code, NULL, NULL, NULL);
 		return PJ_TRUE;
@@ -871,8 +862,7 @@ static int register_aor(pjsip_rx_data *r
 
 	register_aor_core(rdata, endpoint, aor, aor_name, contacts, &response);
 	ao2_cleanup(contacts);
-	ao2_unlock(lock);
-	ast_named_lock_put(lock);
+	ao2_unlock(aor);
 
 	/* Now send the REGISTER response to the peer */
 	if (response.tdata) {
@@ -1121,8 +1111,8 @@ static int ami_show_registrations(struct
 	int count = 0;
 	struct ast_sip_ami ami = { .s = s, .m = m, .arg = &count, .action_id = astman_get_header(m, "ActionID"), };
 
-	astman_send_listack(s, m, "Following are Events for each Inbound "
-			    "registration", "start");
+	astman_send_listack(s, m, "Following are Events for each Inbound registration",
+		"start");
 
 	ami_registrations_endpoints(&ami);
 
@@ -1280,14 +1270,10 @@ static int load_module(void)
 {
 	const pj_str_t STR_REGISTER = { "REGISTER", 8 };
 
-	CHECK_PJPROJECT_MODULE_LOADED();
-
 	ast_pjproject_get_buildopt("PJ_MAX_HOSTNAME", "%d", &pj_max_hostname);
 	/* As of pjproject 2.4.5, PJSIP_MAX_URL_SIZE isn't exposed yet but we try anyway. */
 	ast_pjproject_get_buildopt("PJSIP_MAX_URL_SIZE", "%d", &pjsip_max_url_size);
 
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (ast_sip_register_service(&registrar_module)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -1328,8 +1314,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Registrar Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND - 3,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 3,
+	.requires = "res_pjproject,res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_registrar_expire.c 1:16.1.1~dfsg-1/res/res_pjsip_registrar_expire.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_registrar_expire.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_registrar_expire.c	1970-01-01 00:00:00.000000000 +0000
@@ -1,50 +0,0 @@
-/*
- * Asterisk -- An open source telephony toolkit.
- *
- * Copyright (C) 2013, Digium, Inc.
- *
- * Joshua Colp <jcolp@digium.com>
- *
- * See http://www.asterisk.org for more information about
- * the Asterisk project. Please do not directly contact
- * any of the maintainers of this project for assistance;
- * the project provides a web site, mailing lists and IRC
- * channels for your use.
- *
- * This program is free software, distributed under the terms of
- * the GNU General Public License Version 2. See the LICENSE file
- * at the top of the source tree.
- */
-
-/*** MODULEINFO
-	<defaultenabled>no</defaultenabled>
-	<depend>pjproject</depend>
-	<depend>res_pjsip</depend>
-	<support_level>deprecated</support_level>
-	<replacement>res_pjsip_registrar</replacement>
- ***/
-
-/*
- * This module has been refactored into res_pjsip_registrar
- */
-
-#include "asterisk.h"
-
-#include "asterisk/module.h"
-
-static int unload_module(void)
-{
-	return 0;
-}
-
-static int load_module(void)
-{
-	return AST_MODULE_LOAD_SUCCESS;
-}
-
-AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "OBSOLETE PJSIP Contact Auto-Expiration",
-	.support_level = AST_MODULE_SUPPORT_DEPRECATED,
-	.load = load_module,
-	.unload = unload_module,
-	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_rfc3326.c 1:16.1.1~dfsg-1/res/res_pjsip_rfc3326.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_rfc3326.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_rfc3326.c	2018-12-26 18:45:13.000000000 +0000
@@ -19,7 +19,6 @@
 /*** MODULEINFO
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
-	<depend>res_pjsip_session</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -149,8 +148,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	ast_sip_session_register_supplement(&rfc3326_supplement);
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -162,8 +159,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP RFC3326 Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_sdp_rtp.c 1:16.1.1~dfsg-1/res/res_pjsip_sdp_rtp.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_sdp_rtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_sdp_rtp.c	2018-12-26 18:45:13.000000000 +0000
@@ -38,8 +38,7 @@
 #include <pjmedia.h>
 #include <pjlib.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
+#include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/format.h"
 #include "asterisk/format_cap.h"
@@ -51,7 +50,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/acl.h"
 #include "asterisk/sdp_srtp.h"
 #include "asterisk/dsp.h"
-#include "asterisk/utils.h"
+#include "asterisk/linkedlists.h"       /* for AST_LIST_NEXT */
+#include "asterisk/stream.h"
+#include "asterisk/format_cache.h"
 
 #include "asterisk/res_pjsip.h"
 #include "asterisk/res_pjsip_session.h"
@@ -63,47 +64,7 @@ static struct ast_sched_context *sched;
 static struct ast_sockaddr address_rtp;
 
 static const char STR_AUDIO[] = "audio";
-static const int FD_AUDIO = 0;
-
 static const char STR_VIDEO[] = "video";
-static const int FD_VIDEO = 2;
-
-/*! \brief Retrieves an ast_format_type based on the given stream_type */
-static enum ast_media_type stream_to_media_type(const char *stream_type)
-{
-	if (!strcasecmp(stream_type, STR_AUDIO)) {
-		return AST_MEDIA_TYPE_AUDIO;
-	} else if (!strcasecmp(stream_type, STR_VIDEO)) {
-		return AST_MEDIA_TYPE_VIDEO;
-	}
-
-	return 0;
-}
-
-/*! \brief Get the starting descriptor for a media type */
-static int media_type_to_fdno(enum ast_media_type media_type)
-{
-	switch (media_type) {
-	case AST_MEDIA_TYPE_AUDIO: return FD_AUDIO;
-	case AST_MEDIA_TYPE_VIDEO: return FD_VIDEO;
-	case AST_MEDIA_TYPE_TEXT:
-	case AST_MEDIA_TYPE_UNKNOWN:
-	case AST_MEDIA_TYPE_IMAGE: break;
-	}
-	return -1;
-}
-
-/*! \brief Remove all other cap types but the one given */
-static void format_cap_only_type(struct ast_format_cap *caps, enum ast_media_type media_type)
-{
-	int i = 0;
-	while (i <= AST_MEDIA_TYPE_TEXT) {
-		if (i != media_type && i != AST_MEDIA_TYPE_UNKNOWN) {
-			ast_format_cap_remove_by_type(caps, i);
-		}
-		i += 1;
-	}
-}
 
 static int send_keepalive(const void *data)
 {
@@ -181,7 +142,7 @@ static void enable_rtcp(struct ast_sip_s
 {
 	enum ast_rtp_instance_rtcp rtcp_type;
 
-	if (session->endpoint->rtcp_mux && session_media->remote_rtcp_mux) {
+	if (session->endpoint->media.rtcp_mux && session_media->remote_rtcp_mux) {
 		rtcp_type = AST_RTP_INSTANCE_RTCP_MUX;
 	} else {
 		rtcp_type = AST_RTP_INSTANCE_RTCP_STANDARD;
@@ -190,8 +151,57 @@ static void enable_rtcp(struct ast_sip_s
 	ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_RTCP, rtcp_type);
 }
 
+/*!
+ * \brief Enable an RTP extension on an RTP session.
+ */
+static void enable_rtp_extension(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	enum ast_rtp_extension extension, enum ast_rtp_extension_direction direction,
+	const pjmedia_sdp_session *sdp)
+{
+	int id = -1;
+
+	/* For a bundle group the local unique identifier space is shared across all streams within
+	 * it.
+	 */
+	if (session_media->bundle_group != -1) {
+		int index;
+
+		for (index = 0; index < sdp->media_count; ++index) {
+			struct ast_sip_session_media *other_session_media;
+			int other_id;
+
+			if (index >= AST_VECTOR_SIZE(&session->pending_media_state->sessions)) {
+				break;
+			}
+
+			other_session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, index);
+			if (!other_session_media->rtp || other_session_media->bundle_group != session_media->bundle_group) {
+				continue;
+			}
+
+			other_id = ast_rtp_instance_extmap_get_id(other_session_media->rtp, extension);
+			if (other_id == -1) {
+				/* Worst case we have to fall back to the highest available free local unique identifier
+				 * for the bundle group.
+				 */
+				other_id = ast_rtp_instance_extmap_count(other_session_media->rtp) + 1;
+				if (id < other_id) {
+					id = other_id;
+				}
+				continue;
+			}
+
+			id = other_id;
+			break;
+		}
+	}
+
+	ast_rtp_instance_extmap_enable(session_media->rtp, id, extension, direction);
+}
+
 /*! \brief Internal function which creates an RTP instance */
-static int create_rtp(struct ast_sip_session *session, struct ast_sip_session_media *session_media)
+static int create_rtp(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	const pjmedia_sdp_session *sdp)
 {
 	struct ast_rtp_engine_ice *ice;
 	struct ast_sockaddr temp_media_address;
@@ -254,14 +264,19 @@ static int create_rtp(struct ast_sip_ses
 		ast_rtp_instance_dtmf_mode_set(session_media->rtp, AST_RTP_DTMF_MODE_INBAND);
 	}
 
-	if (!strcmp(session_media->stream_type, STR_AUDIO) &&
+	if (session_media->type == AST_MEDIA_TYPE_AUDIO &&
 			(session->endpoint->media.tos_audio || session->endpoint->media.cos_audio)) {
 		ast_rtp_instance_set_qos(session_media->rtp, session->endpoint->media.tos_audio,
 				session->endpoint->media.cos_audio, "SIP RTP Audio");
-	} else if (!strcmp(session_media->stream_type, STR_VIDEO) &&
-			(session->endpoint->media.tos_video || session->endpoint->media.cos_video)) {
-		ast_rtp_instance_set_qos(session_media->rtp, session->endpoint->media.tos_video,
-				session->endpoint->media.cos_video, "SIP RTP Video");
+	} else if (session_media->type == AST_MEDIA_TYPE_VIDEO) {
+		ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_RETRANS_RECV, session->endpoint->media.webrtc);
+		ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_RETRANS_SEND, session->endpoint->media.webrtc);
+		ast_rtp_instance_set_prop(session_media->rtp, AST_RTP_PROPERTY_REMB, session->endpoint->media.webrtc);
+		enable_rtp_extension(session, session_media, AST_RTP_EXTENSION_ABS_SEND_TIME, AST_RTP_EXTENSION_DIRECTION_SENDRECV, sdp);
+		if (session->endpoint->media.tos_video || session->endpoint->media.cos_video) {
+			ast_rtp_instance_set_qos(session_media->rtp, session->endpoint->media.tos_video,
+					session->endpoint->media.cos_video, "SIP RTP Video");
+		}
 	}
 
 	ast_rtp_instance_set_last_rx(session_media->rtp, time(NULL));
@@ -355,13 +370,17 @@ static void get_codecs(struct ast_sip_se
 	}
 }
 
-static int set_caps(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-		    const struct pjmedia_sdp_media *stream)
+static int set_caps(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media,
+	struct ast_sip_session_media *session_media_transport,
+	const struct pjmedia_sdp_media *stream,
+	int is_offer, struct ast_stream *asterisk_stream)
 {
 	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_format_cap *, peer, NULL, ao2_cleanup);
 	RAII_VAR(struct ast_format_cap *, joint, NULL, ao2_cleanup);
-	enum ast_media_type media_type = stream_to_media_type(session_media->stream_type);
+	RAII_VAR(struct ast_format_cap *, endpoint_caps, NULL, ao2_cleanup);
+	enum ast_media_type media_type = session_media->type;
 	struct ast_rtp_codecs codecs = AST_RTP_CODECS_NULL_INIT;
 	int fmts = 0;
 	int direct_media_enabled = !ast_sockaddr_isnull(&session_media->direct_media_addr) &&
@@ -371,14 +390,14 @@ static int set_caps(struct ast_sip_sessi
 	if (!(caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT)) ||
 	    !(peer = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT)) ||
 	    !(joint = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
-		ast_log(LOG_ERROR, "Failed to allocate %s capabilities\n", session_media->stream_type);
+		ast_log(LOG_ERROR, "Failed to allocate %s capabilities\n",
+			ast_codec_media_type2str(session_media->type));
 		return -1;
 	}
 
 	/* get the endpoint capabilities */
 	if (direct_media_enabled) {
 		ast_format_cap_get_compatible(session->endpoint->media.codecs, session->direct_media_cap, caps);
-		format_cap_only_type(caps, media_type);
 	} else {
 		ast_format_cap_append_from_cap(caps, session->endpoint->media.codecs, media_type);
 	}
@@ -395,32 +414,61 @@ static int set_caps(struct ast_sip_sessi
 
 		ast_rtp_codecs_payloads_destroy(&codecs);
 		ast_log(LOG_NOTICE, "No joint capabilities for '%s' media stream between our configuration(%s) and incoming SDP(%s)\n",
-			session_media->stream_type,
+			ast_codec_media_type2str(session_media->type),
 			ast_format_cap_get_names(caps, &usbuf),
 			ast_format_cap_get_names(peer, &thembuf));
 		return -1;
 	}
 
+	if (is_offer) {
+		/*
+		 * Setup rx payload type mapping to prefer the mapping
+		 * from the peer that the RFC says we SHOULD use.
+		 */
+		ast_rtp_codecs_payloads_xover(&codecs, &codecs, NULL);
+	}
 	ast_rtp_codecs_payloads_copy(&codecs, ast_rtp_instance_get_codecs(session_media->rtp),
 		session_media->rtp);
 
-	ast_format_cap_append_from_cap(session->req_caps, joint, AST_MEDIA_TYPE_UNKNOWN);
+	ast_stream_set_formats(asterisk_stream, joint);
 
-	if (session->channel) {
+	/* If this is a bundled stream then apply the payloads to RTP instance acting as transport to prevent conflicts */
+	if (session_media_transport != session_media && session_media->bundled) {
+		int index;
+
+		for (index = 0; index < ast_format_cap_count(joint); ++index) {
+			struct ast_format *format = ast_format_cap_get_format(joint, index);
+			int rtp_code;
+
+			/* Ensure this payload is in the bundle group transport codecs, this purposely doesn't check the return value for
+			 * things as the format is guaranteed to have a payload already.
+			 */
+			rtp_code = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(session_media->rtp), 1, format, 0);
+			ast_rtp_codecs_payload_set_rx(ast_rtp_instance_get_codecs(session_media_transport->rtp), rtp_code, format);
+
+			ao2_ref(format, -1);
+		}
+	}
+
+	if (session->channel && ast_sip_session_is_pending_stream_default(session, asterisk_stream)) {
 		ast_channel_lock(session->channel);
 		ast_format_cap_remove_by_type(caps, AST_MEDIA_TYPE_UNKNOWN);
 		ast_format_cap_append_from_cap(caps, ast_channel_nativeformats(session->channel),
 			AST_MEDIA_TYPE_UNKNOWN);
 		ast_format_cap_remove_by_type(caps, media_type);
 
-		/*
-		 * If we don't allow the sending codec to be changed on our side
-		 * then get the best codec from the joint capabilities of the media
-		 * type and use only that. This ensures the core won't start sending
-		 * out a format that we aren't currently sending.
-		 */
-		if (!session->endpoint->asymmetric_rtp_codec) {
+		if (session->endpoint->preferred_codec_only){
+			struct ast_format *preferred_fmt = ast_format_cap_get_format(joint, 0);
+			ast_format_cap_append(caps, preferred_fmt, 0);
+			ao2_ref(preferred_fmt, -1);
+		} else if (!session->endpoint->asymmetric_rtp_codec) {
 			struct ast_format *best;
+			/*
+			 * If we don't allow the sending codec to be changed on our side
+			 * then get the best codec from the joint capabilities of the media
+			 * type and use only that. This ensures the core won't start sending
+			 * out a format that we aren't currently sending.
+			 */
 
 			best = ast_format_cap_get_best_by_type(joint, media_type);
 			if (best) {
@@ -468,6 +516,11 @@ static int set_caps(struct ast_sip_sessi
 static pjmedia_sdp_attr* generate_rtpmap_attr(struct ast_sip_session *session, pjmedia_sdp_media *media, pj_pool_t *pool,
 					      int rtp_code, int asterisk_format, struct ast_format *format, int code)
 {
+#ifndef HAVE_PJSIP_ENDPOINT_COMPACT_FORM
+	extern pj_bool_t pjsip_use_compact_form;
+#else
+	pj_bool_t pjsip_use_compact_form = pjsip_cfg()->endpt.use_compact_form;
+#endif
 	pjmedia_sdp_rtpmap rtpmap;
 	pjmedia_sdp_attr *attr = NULL;
 	char tmp[64];
@@ -476,6 +529,11 @@ static pjmedia_sdp_attr* generate_rtpmap
 
 	snprintf(tmp, sizeof(tmp), "%d", rtp_code);
 	pj_strdup2(pool, &media->desc.fmt[media->desc.fmt_count++], tmp);
+
+	if (rtp_code <= AST_RTP_PT_LAST_STATIC && pjsip_use_compact_form) {
+		return NULL;
+	}
+
 	rtpmap.pt = media->desc.fmt[media->desc.fmt_count - 1];
 	rtpmap.clock_rate = ast_rtp_lookup_sample_rate2(asterisk_format, format, code);
 	pj_strdup2(pool, &rtpmap.enc_name, ast_rtp_lookup_mime_subtype2(asterisk_format, format, code, options));
@@ -516,7 +574,8 @@ static pjmedia_sdp_attr* generate_fmtp_a
 }
 
 /*! \brief Function which adds ICE attributes to a media stream */
-static void add_ice_to_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media)
+static void add_ice_to_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media,
+	unsigned int include_candidates)
 {
 	struct ast_rtp_engine_ice *ice;
 	struct ao2_container *candidates;
@@ -526,8 +585,7 @@ static void add_ice_to_stream(struct ast
 	struct ao2_iterator it_candidates;
 	struct ast_rtp_engine_ice_candidate *candidate;
 
-	if (!session->endpoint->media.rtp.ice_support || !(ice = ast_rtp_instance_get_ice(session_media->rtp)) ||
-		!(candidates = ice->get_local_candidates(session_media->rtp))) {
+	if (!session->endpoint->media.rtp.ice_support || !(ice = ast_rtp_instance_get_ice(session_media->rtp))) {
 		return;
 	}
 
@@ -545,6 +603,15 @@ static void add_ice_to_stream(struct ast
 		media->attr[media->attr_count++] = attr;
 	}
 
+	if (!include_candidates) {
+		return;
+	}
+
+	candidates = ice->get_local_candidates(session_media->rtp);
+	if (!candidates) {
+		return;
+	}
+
 	it_candidates = ao2_iterator_init(candidates, 0);
 	for (; (candidate = ao2_iterator_next(&it_candidates)); ao2_ref(candidate, -1)) {
 		struct ast_str *attr_candidate = ast_str_create(128);
@@ -668,7 +735,7 @@ static void process_ice_attributes(struc
 			continue;
 		}
 
-		if (session->endpoint->rtcp_mux && session_media->remote_rtcp_mux && candidate.id > 1) {
+		if (session->endpoint->media.rtcp_mux && session_media->remote_rtcp_mux && candidate.id > 1) {
 			/* Remote side may have offered RTP and RTCP candidates. However, if we're using RTCP MUX,
 			 * then we should ignore RTCP candidates.
 			 */
@@ -982,7 +1049,7 @@ static void set_ice_components(struct as
 		return;
 	}
 
-	if (session->endpoint->rtcp_mux && session_media->remote_rtcp_mux) {
+	if (session->endpoint->media.rtcp_mux && session_media->remote_rtcp_mux) {
 		/* We both support RTCP mux. Only one ICE component necessary */
 		ice->change_components(session_media->rtp, 1);
 	} else {
@@ -991,25 +1058,269 @@ static void set_ice_components(struct as
 	}
 }
 
+/*! \brief Function which adds ssrc attributes to a media stream */
+static void add_ssrc_to_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media)
+{
+	pj_str_t stmp;
+	pjmedia_sdp_attr *attr;
+	char tmp[128];
+
+	if (!session->endpoint->media.bundle || session_media->bundle_group == -1) {
+		return;
+	}
+
+	snprintf(tmp, sizeof(tmp), "%u cname:%s", ast_rtp_instance_get_ssrc(session_media->rtp), ast_rtp_instance_get_cname(session_media->rtp));
+	attr = pjmedia_sdp_attr_create(pool, "ssrc", pj_cstr(&stmp, tmp));
+	media->attr[media->attr_count++] = attr;
+}
+
+/*! \brief Function which processes ssrc attributes in a stream */
+static void process_ssrc_attributes(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+				   const struct pjmedia_sdp_media *remote_stream)
+{
+	int index;
+
+	if (!session->endpoint->media.bundle) {
+		return;
+	}
+
+	for (index = 0; index < remote_stream->attr_count; ++index) {
+		pjmedia_sdp_attr *attr = remote_stream->attr[index];
+		char attr_value[pj_strlen(&attr->value) + 1];
+		char *ssrc_attribute_name, *ssrc_attribute_value = NULL;
+		unsigned int ssrc;
+
+		/* We only care about ssrc attributes */
+		if (pj_strcmp2(&attr->name, "ssrc")) {
+			continue;
+		}
+
+		ast_copy_pj_str(attr_value, &attr->value, sizeof(attr_value));
+
+		if ((ssrc_attribute_name = strchr(attr_value, ' '))) {
+			/* This has an actual attribute */
+			*ssrc_attribute_name++ = '\0';
+			ssrc_attribute_value = strchr(ssrc_attribute_name, ':');
+			if (ssrc_attribute_value) {
+				/* Values are actually optional according to the spec */
+				*ssrc_attribute_value++ = '\0';
+			}
+		}
+
+		if (sscanf(attr_value, "%30u", &ssrc) < 1) {
+			continue;
+		}
+
+		/* If we are currently negotiating as a result of the remote side renegotiating then
+		 * determine if the source for this stream has changed.
+		 */
+		if (pjmedia_sdp_neg_get_state(session->inv_session->neg) == PJMEDIA_SDP_NEG_STATE_REMOTE_OFFER &&
+			session->active_media_state) {
+			struct ast_rtp_instance_stats stats = { 0, };
+
+			if (!ast_rtp_instance_get_stats(session_media->rtp, &stats, AST_RTP_INSTANCE_STAT_REMOTE_SSRC) &&
+				stats.remote_ssrc != ssrc) {
+				session_media->changed = 1;
+			}
+		}
+
+		ast_rtp_instance_set_remote_ssrc(session_media->rtp, ssrc);
+	}
+}
+
+static void add_msid_to_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media,
+	struct ast_stream *stream)
+{
+	pj_str_t stmp;
+	pjmedia_sdp_attr *attr;
+	char msid[(AST_UUID_STR_LEN * 2) + 2];
+	const char *stream_label = ast_stream_get_metadata(stream, "SDP:LABEL");
+
+	if (!session->endpoint->media.webrtc) {
+		return;
+	}
+
+	if (ast_strlen_zero(session_media->mslabel)) {
+		/* If this stream is grouped with another then use its media stream label if possible */
+		if (ast_stream_get_group(stream) != -1) {
+			struct ast_sip_session_media *group_session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, ast_stream_get_group(stream));
+
+			ast_copy_string(session_media->mslabel, group_session_media->mslabel, sizeof(session_media->mslabel));
+		}
+
+		if (ast_strlen_zero(session_media->mslabel)) {
+			ast_uuid_generate_str(session_media->mslabel, sizeof(session_media->mslabel));
+		}
+	}
+
+	if (ast_strlen_zero(session_media->label)) {
+			ast_uuid_generate_str(session_media->label, sizeof(session_media->label));
+	}
+
+	snprintf(msid, sizeof(msid), "%s %s", session_media->mslabel, session_media->label);
+	ast_debug(3, "Stream msid: %p %s %s\n", stream,
+		ast_codec_media_type2str(ast_stream_get_type(stream)), msid);
+	attr = pjmedia_sdp_attr_create(pool, "msid", pj_cstr(&stmp, msid));
+	pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+
+	/* 'label' must come after 'msid' */
+	if (!ast_strlen_zero(stream_label)) {
+		ast_debug(3, "Stream Label: %p %s %s\n", stream,
+			ast_codec_media_type2str(ast_stream_get_type(stream)), stream_label);
+		attr = pjmedia_sdp_attr_create(pool, "label", pj_cstr(&stmp, stream_label));
+		pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+	}
+}
+
+static void add_rtcp_fb_to_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media)
+{
+	pj_str_t stmp;
+	pjmedia_sdp_attr *attr;
+
+	if (!session->endpoint->media.webrtc || session_media->type != AST_MEDIA_TYPE_VIDEO) {
+		return;
+	}
+
+	/*
+	 * For now just automatically add it the stream even though it hasn't
+	 * necessarily been negotiated.
+	 */
+	attr = pjmedia_sdp_attr_create(pool, "rtcp-fb", pj_cstr(&stmp, "* ccm fir"));
+	pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+
+	attr = pjmedia_sdp_attr_create(pool, "rtcp-fb", pj_cstr(&stmp, "* goog-remb"));
+	pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+
+	attr = pjmedia_sdp_attr_create(pool, "rtcp-fb", pj_cstr(&stmp, "* nack"));
+	pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+}
+
+static void add_extmap_to_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, pj_pool_t *pool, pjmedia_sdp_media *media)
+{
+	int idx;
+	char extmap_value[256];
+
+	if (!session->endpoint->media.webrtc || session_media->type != AST_MEDIA_TYPE_VIDEO) {
+		return;
+	}
+
+	/* RTP extension local unique identifiers start at '1' */
+	for (idx = 1; idx <= ast_rtp_instance_extmap_count(session_media->rtp); ++idx) {
+		enum ast_rtp_extension extension = ast_rtp_instance_extmap_get_extension(session_media->rtp, idx);
+		const char *direction_str = "";
+		pj_str_t stmp;
+		pjmedia_sdp_attr *attr;
+
+		/* If this is an unsupported RTP extension we can't place it into the SDP */
+		if (extension == AST_RTP_EXTENSION_UNSUPPORTED) {
+			continue;
+		}
+
+		switch (ast_rtp_instance_extmap_get_direction(session_media->rtp, idx)) {
+		case AST_RTP_EXTENSION_DIRECTION_SENDRECV:
+			/* Lack of a direction indicates sendrecv, so we leave it out */
+			direction_str = "";
+			break;
+		case AST_RTP_EXTENSION_DIRECTION_SENDONLY:
+			direction_str = "/sendonly";
+			break;
+		case AST_RTP_EXTENSION_DIRECTION_RECVONLY:
+			direction_str = "/recvonly";
+			break;
+		case AST_RTP_EXTENSION_DIRECTION_NONE:
+			/* It is impossible for a "none" direction extension to be negotiated but just in case
+			 * we treat it as inactive.
+			 */
+		case AST_RTP_EXTENSION_DIRECTION_INACTIVE:
+			direction_str = "/inactive";
+			break;
+		}
+
+		snprintf(extmap_value, sizeof(extmap_value), "%d%s %s", idx, direction_str,
+			ast_rtp_instance_extmap_get_uri(session_media->rtp, idx));
+		attr = pjmedia_sdp_attr_create(pool, "extmap", pj_cstr(&stmp, extmap_value));
+		pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+	}
+}
+
+/*! \brief Function which processes extmap attributes in a stream */
+static void process_extmap_attributes(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+				   const struct pjmedia_sdp_media *remote_stream)
+{
+	int index;
+
+	if (!session->endpoint->media.webrtc || session_media->type != AST_MEDIA_TYPE_VIDEO) {
+		return;
+	}
+
+	ast_rtp_instance_extmap_clear(session_media->rtp);
+
+	for (index = 0; index < remote_stream->attr_count; ++index) {
+		pjmedia_sdp_attr *attr = remote_stream->attr[index];
+		char attr_value[pj_strlen(&attr->value) + 1];
+		char *uri;
+		int id;
+		char direction_str[10] = "";
+		char *attributes;
+		enum ast_rtp_extension_direction direction = AST_RTP_EXTENSION_DIRECTION_SENDRECV;
+
+		/* We only care about extmap attributes */
+		if (pj_strcmp2(&attr->name, "extmap")) {
+			continue;
+		}
+
+		ast_copy_pj_str(attr_value, &attr->value, sizeof(attr_value));
+
+		/* Split the combined unique identifier and direction away from the URI and attributes for easier parsing */
+		uri = strchr(attr_value, ' ');
+		if (ast_strlen_zero(uri)) {
+			continue;
+		}
+		*uri++ = '\0';
+
+		if ((sscanf(attr_value, "%30d%9s", &id, direction_str) < 1) || (id < 1)) {
+			/* We require at a minimum the unique identifier */
+			continue;
+		}
+
+		/* Convert from the string to the internal representation */
+		if (!strcasecmp(direction_str, "/sendonly")) {
+			direction = AST_RTP_EXTENSION_DIRECTION_SENDONLY;
+		} else if (!strcasecmp(direction_str, "/recvonly")) {
+			direction = AST_RTP_EXTENSION_DIRECTION_RECVONLY;
+		} else if (!strcasecmp(direction_str, "/inactive")) {
+			direction = AST_RTP_EXTENSION_DIRECTION_INACTIVE;
+		}
+
+		attributes = strchr(uri, ' ');
+		if (!ast_strlen_zero(attributes)) {
+			*attributes++ = '\0';
+		}
+
+		ast_rtp_instance_extmap_negotiate(session_media->rtp, id, direction, uri, attributes);
+	}
+}
+
 /*! \brief Function which negotiates an incoming media stream */
-static int negotiate_incoming_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-					 const struct pjmedia_sdp_session *sdp, const struct pjmedia_sdp_media *stream)
+static int negotiate_incoming_sdp_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, const pjmedia_sdp_session *sdp,
+	int index, struct ast_stream *asterisk_stream)
 {
 	char host[NI_MAXHOST];
 	RAII_VAR(struct ast_sockaddr *, addrs, NULL, ast_free);
-	enum ast_media_type media_type = stream_to_media_type(session_media->stream_type);
+	pjmedia_sdp_media *stream = sdp->media[index];
+	struct ast_sip_session_media *session_media_transport;
+	enum ast_media_type media_type = session_media->type;
 	enum ast_sip_session_media_encryption encryption = AST_SIP_MEDIA_ENCRYPT_NONE;
 	int res;
 
-	/* If port is 0, ignore this media stream */
-	if (!stream->desc.port) {
-		ast_debug(3, "Media stream '%s' is already declined\n", session_media->stream_type);
-		return 0;
-	}
-
 	/* If no type formats have been configured reject this stream */
 	if (!ast_format_cap_has_type(session->endpoint->media.codecs, media_type)) {
-		ast_debug(3, "Endpoint has no codecs for media type '%s', declining stream\n", session_media->stream_type);
+		ast_debug(3, "Endpoint has no codecs for media type '%s', declining stream\n",
+			ast_codec_media_type2str(session_media->type));
 		return 0;
 	}
 
@@ -1031,45 +1342,58 @@ static int negotiate_incoming_sdp_stream
 	}
 
 	/* Using the connection information create an appropriate RTP instance */
-	if (!session_media->rtp && create_rtp(session, session_media)) {
+	if (!session_media->rtp && create_rtp(session, session_media, sdp)) {
 		return -1;
 	}
 
-	session_media->remote_rtcp_mux = (pjmedia_sdp_media_find_attr2(stream, "rtcp-mux", NULL) != NULL);
-	set_ice_components(session, session_media);
-
-	enable_rtcp(session, session_media, stream);
+	process_ssrc_attributes(session, session_media, stream);
+	process_extmap_attributes(session, session_media, stream);
+	session_media_transport = ast_sip_session_media_get_transport(session, session_media);
+
+	if (session_media_transport == session_media || !session_media->bundled) {
+		/* If this media session is carrying actual traffic then set up those aspects */
+		session_media->remote_rtcp_mux = (pjmedia_sdp_media_find_attr2(stream, "rtcp-mux", NULL) != NULL);
+		set_ice_components(session, session_media);
+
+		enable_rtcp(session, session_media, stream);
+
+		res = setup_media_encryption(session, session_media, sdp, stream);
+		if (res) {
+			if (!session->endpoint->media.rtp.encryption_optimistic ||
+				!pj_strncmp2(&stream->desc.transport, "RTP/SAVP", 8)) {
+				/* If optimistic encryption is disabled and crypto should have been enabled
+				 * but was not this session must fail. This must also fail if crypto was
+				 * required in the offer but could not be set up.
+				 */
+				return -1;
+			}
+			/* There is no encryption, sad. */
+			session_media->encryption = AST_SIP_MEDIA_ENCRYPT_NONE;
+		}
 
-	res = setup_media_encryption(session, session_media, sdp, stream);
-	if (res) {
-		if (!session->endpoint->media.rtp.encryption_optimistic ||
-			!pj_strncmp2(&stream->desc.transport, "RTP/SAVP", 8)) {
-			/* If optimistic encryption is disabled and crypto should have been enabled
-			 * but was not this session must fail. This must also fail if crypto was
-			 * required in the offer but could not be set up.
-			 */
-			return -1;
+		/* If we've been explicitly configured to use the received transport OR if
+		 * encryption is on and crypto is present use the received transport.
+		 * This is done in case of optimistic because it may come in as RTP/AVP or RTP/SAVP depending
+		 * on the configuration of the remote endpoint (optimistic themselves or mandatory).
+		 */
+		if ((session->endpoint->media.rtp.use_received_transport) ||
+			((encryption == AST_SIP_MEDIA_ENCRYPT_SDES) && !res)) {
+			pj_strdup(session->inv_session->pool, &session_media->transport, &stream->desc.transport);
 		}
-		/* There is no encryption, sad. */
-		session_media->encryption = AST_SIP_MEDIA_ENCRYPT_NONE;
- 	}
-
-	/* If we've been explicitly configured to use the received transport OR if
-	 * encryption is on and crypto is present use the received transport.
-	 * This is done in case of optimistic because it may come in as RTP/AVP or RTP/SAVP depending
-	 * on the configuration of the remote endpoint (optimistic themselves or mandatory).
-	 */
-	if ((session->endpoint->media.rtp.use_received_transport) ||
-		((encryption == AST_SIP_MEDIA_ENCRYPT_SDES) && !res)) {
-		pj_strdup(session->inv_session->pool, &session_media->transport, &stream->desc.transport);
- 	}
+	} else {
+		/* This is bundled with another session, so mark it as such */
+		ast_rtp_instance_bundle(session_media->rtp, session_media_transport->rtp);
+
+		enable_rtcp(session, session_media, stream);
+	}
 
 	/* If ICE support is enabled find all the needed attributes */
 	check_ice_support(session, session_media, stream);
 
-	if (set_caps(session, session_media, stream)) {
+	if (set_caps(session, session_media, session_media_transport, stream, 1, asterisk_stream)) {
 		return 0;
 	}
+
 	return 1;
 }
 
@@ -1082,12 +1406,14 @@ static int add_crypto_to_stream(struct a
 	enum ast_rtp_dtls_hash hash;
 	const char *crypto_attribute;
 	struct ast_rtp_engine_dtls *dtls;
+	struct ast_sdp_srtp *tmp;
 	static const pj_str_t STR_NEW = { "new", 3 };
 	static const pj_str_t STR_EXISTING = { "existing", 8 };
 	static const pj_str_t STR_ACTIVE = { "active", 6 };
 	static const pj_str_t STR_PASSIVE = { "passive", 7 };
 	static const pj_str_t STR_ACTPASS = { "actpass", 7 };
 	static const pj_str_t STR_HOLDCONN = { "holdconn", 8 };
+	enum ast_rtp_dtls_setup setup;
 
 	switch (session_media->encryption) {
 	case AST_SIP_MEDIA_ENCRYPT_NONE:
@@ -1101,16 +1427,22 @@ static int add_crypto_to_stream(struct a
 			}
 		}
 
-		crypto_attribute = ast_sdp_srtp_get_attrib(session_media->srtp,
-			0 /* DTLS running? No */,
-			session->endpoint->media.rtp.srtp_tag_32 /* 32 byte tag length? */);
-		if (!crypto_attribute) {
-			/* No crypto attribute to add, bad news */
-			return -1;
-		}
+		tmp = session_media->srtp;
+
+		do {
+			crypto_attribute = ast_sdp_srtp_get_attrib(tmp,
+				0 /* DTLS running? No */,
+				session->endpoint->media.rtp.srtp_tag_32 /* 32 byte tag length? */);
+			if (!crypto_attribute) {
+				/* No crypto attribute to add, bad news */
+				return -1;
+			}
+
+			attr = pjmedia_sdp_attr_create(pool, "crypto",
+				pj_cstr(&stmp, crypto_attribute));
+			media->attr[media->attr_count++] = attr;
+		} while ((tmp = AST_LIST_NEXT(tmp, sdp_srtp_list)));
 
-		attr = pjmedia_sdp_attr_create(pool, "crypto", pj_cstr(&stmp, crypto_attribute));
-		media->attr[media->attr_count++] = attr;
 		break;
 	case AST_SIP_MEDIA_ENCRYPT_DTLS:
 		if (setup_dtls_srtp(session, session_media)) {
@@ -1135,7 +1467,17 @@ static int add_crypto_to_stream(struct a
 			break;
 		}
 
-		switch (dtls->get_setup(session_media->rtp)) {
+		/* If this is an answer we need to use our current state, if it's an offer we need to use
+		 * the configured value.
+		 */
+		if (session->inv_session->neg
+			&& pjmedia_sdp_neg_get_state(session->inv_session->neg) != PJMEDIA_SDP_NEG_STATE_DONE) {
+			setup = dtls->get_setup(session_media->rtp);
+		} else {
+			setup = session->endpoint->media.rtp.dtls_cfg.default_setup;
+		}
+
+		switch (setup) {
 		case AST_RTP_DTLS_SETUP_ACTIVE:
 			attr = pjmedia_sdp_attr_create(pool, "setup", &STR_ACTIVE);
 			media->attr[media->attr_count++] = attr;
@@ -1150,7 +1492,6 @@ static int add_crypto_to_stream(struct a
 			break;
 		case AST_RTP_DTLS_SETUP_HOLDCONN:
 			attr = pjmedia_sdp_attr_create(pool, "setup", &STR_HOLDCONN);
-			media->attr[media->attr_count++] = attr;
 			break;
 		default:
 			break;
@@ -1181,13 +1522,15 @@ static int add_crypto_to_stream(struct a
 
 /*! \brief Function which creates an outgoing stream */
 static int create_outgoing_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-				      struct pjmedia_sdp_session *sdp)
+				      struct pjmedia_sdp_session *sdp, const struct pjmedia_sdp_session *remote, struct ast_stream *stream)
 {
 	pj_pool_t *pool = session->inv_session->pool_prov;
+	static const pj_str_t STR_RTP_AVP = { "RTP/AVP", 7 };
 	static const pj_str_t STR_IN = { "IN", 2 };
 	static const pj_str_t STR_IP4 = { "IP4", 3};
 	static const pj_str_t STR_IP6 = { "IP6", 3};
 	static const pj_str_t STR_SENDRECV = { "sendrecv", 8 };
+	static const pj_str_t STR_SENDONLY = { "sendonly", 8 };
 	pjmedia_sdp_media *media;
 	const char *hostip = NULL;
 	struct ast_sockaddr addr;
@@ -1199,89 +1542,159 @@ static int create_outgoing_sdp_stream(st
 	int min_packet_size = 0, max_packet_size = 0;
 	int rtp_code;
 	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
-	enum ast_media_type media_type = stream_to_media_type(session_media->stream_type);
-	int use_override_prefs = ast_format_cap_count(session->req_caps);
+	enum ast_media_type media_type = session_media->type;
+	struct ast_sip_session_media *session_media_transport;
 	pj_sockaddr ip;
 
 	int direct_media_enabled = !ast_sockaddr_isnull(&session_media->direct_media_addr) &&
 		ast_format_cap_count(session->direct_media_cap);
 
-	if ((use_override_prefs && !ast_format_cap_has_type(session->req_caps, media_type)) ||
-	    (!use_override_prefs && !ast_format_cap_has_type(session->endpoint->media.codecs, media_type))) {
-		/* If no type formats are configured don't add a stream */
-		return 0;
-	} else if (!session_media->rtp && create_rtp(session, session_media)) {
+	media = pj_pool_zalloc(pool, sizeof(struct pjmedia_sdp_media));
+	if (!media) {
 		return -1;
 	}
+	pj_strdup2(pool, &media->desc.media, ast_codec_media_type2str(session_media->type));
 
-	set_ice_components(session, session_media);
-	enable_rtcp(session, session_media, NULL);
+	/* If this is a removed (or declined) stream OR if no formats exist then construct a minimal stream in SDP */
+	if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED || !ast_stream_get_formats(stream) ||
+		!ast_format_cap_count(ast_stream_get_formats(stream))) {
+		media->desc.port = 0;
+		media->desc.port_count = 1;
+
+		if (remote && remote->media[ast_stream_get_position(stream)]) {
+			pjmedia_sdp_media *remote_media = remote->media[ast_stream_get_position(stream)];
+			int index;
+
+			media->desc.transport = remote_media->desc.transport;
+
+			/* Preserve existing behavior by copying the formats provided from the offer */
+			for (index = 0; index < remote_media->desc.fmt_count; ++index) {
+				media->desc.fmt[index] = remote_media->desc.fmt[index];
+			}
+			media->desc.fmt_count = remote_media->desc.fmt_count;
+		} else {
+			/* This is actually an offer so put a dummy payload in that is ignored and sane transport */
+			media->desc.transport = STR_RTP_AVP;
+			pj_strdup2(pool, &media->desc.fmt[media->desc.fmt_count++], "32");
+		}
 
-	if (!(media = pj_pool_zalloc(pool, sizeof(struct pjmedia_sdp_media))) ||
-		!(media->conn = pj_pool_zalloc(pool, sizeof(struct pjmedia_sdp_conn)))) {
-		return -1;
+		sdp->media[sdp->media_count++] = media;
+		ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
+
+		return 1;
 	}
 
-	if (add_crypto_to_stream(session, session_media, pool, media)) {
+	if (!session_media->rtp && create_rtp(session, session_media, sdp)) {
 		return -1;
 	}
 
-	media->desc.media = pj_str(session_media->stream_type);
-	if (pj_strlen(&session_media->transport)) {
-		/* If a transport has already been specified use it */
-		media->desc.transport = session_media->transport;
-	} else {
-		media->desc.transport = pj_str(ast_sdp_get_rtp_profile(
-			/* Optimistic encryption places crypto in the normal RTP/AVP profile */
-			!session->endpoint->media.rtp.encryption_optimistic &&
-				(session_media->encryption == AST_SIP_MEDIA_ENCRYPT_SDES),
-			session_media->rtp, session->endpoint->media.rtp.use_avpf,
-			session->endpoint->media.rtp.force_avp));
+	/* If this stream has not been bundled already it is new and we need to ensure there is no SSRC conflict */
+	if (session_media->bundle_group != -1 && !session_media->bundled) {
+		for (index = 0; index < sdp->media_count; ++index) {
+			struct ast_sip_session_media *other_session_media;
+
+			other_session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, index);
+			if (!other_session_media->rtp || other_session_media->bundle_group != session_media->bundle_group) {
+				continue;
+			}
+
+			if (ast_rtp_instance_get_ssrc(session_media->rtp) == ast_rtp_instance_get_ssrc(other_session_media->rtp)) {
+				ast_rtp_instance_change_source(session_media->rtp);
+				/* Start the conflict check over again */
+				index = -1;
+				continue;
+			}
+		}
 	}
 
-	/* Add connection level details */
-	if (direct_media_enabled) {
-		hostip = ast_sockaddr_stringify_fmt(&session_media->direct_media_addr, AST_SOCKADDR_STR_ADDR);
-	} else if (ast_strlen_zero(session->endpoint->media.address)) {
-		hostip = ast_sip_get_host_ip_string(session->endpoint->media.rtp.ipv6 ? pj_AF_INET6() : pj_AF_INET());
+	session_media_transport = ast_sip_session_media_get_transport(session, session_media);
+
+	if (session_media_transport == session_media || !session_media->bundled) {
+		set_ice_components(session, session_media);
+		enable_rtcp(session, session_media, NULL);
+
+		/* Crypto has to be added before setting the media transport so that SRTP is properly
+		 * set up according to the configuration. This ends up changing the media transport.
+		 */
+		if (add_crypto_to_stream(session, session_media, pool, media)) {
+			return -1;
+		}
+
+		if (pj_strlen(&session_media->transport)) {
+			/* If a transport has already been specified use it */
+			media->desc.transport = session_media->transport;
+		} else {
+			media->desc.transport = pj_str(ast_sdp_get_rtp_profile(
+				/* Optimistic encryption places crypto in the normal RTP/AVP profile */
+				!session->endpoint->media.rtp.encryption_optimistic &&
+					(session_media->encryption == AST_SIP_MEDIA_ENCRYPT_SDES),
+				session_media->rtp, session->endpoint->media.rtp.use_avpf,
+				session->endpoint->media.rtp.force_avp));
+		}
+
+		media->conn = pj_pool_zalloc(pool, sizeof(struct pjmedia_sdp_conn));
+		if (!media->conn) {
+			return -1;
+		}
+
+		/* Add connection level details */
+		if (direct_media_enabled) {
+			hostip = ast_sockaddr_stringify_fmt(&session_media->direct_media_addr, AST_SOCKADDR_STR_ADDR);
+		} else if (ast_strlen_zero(session->endpoint->media.address)) {
+			hostip = ast_sip_get_host_ip_string(session->endpoint->media.rtp.ipv6 ? pj_AF_INET6() : pj_AF_INET());
+		} else {
+			hostip = session->endpoint->media.address;
+		}
+
+		if (ast_strlen_zero(hostip)) {
+			ast_log(LOG_ERROR, "No local host IP available for stream %s\n",
+				ast_codec_media_type2str(session_media->type));
+			return -1;
+		}
+
+		media->conn->net_type = STR_IN;
+		/* Assume that the connection will use IPv4 until proven otherwise */
+		media->conn->addr_type = STR_IP4;
+		pj_strdup2(pool, &media->conn->addr, hostip);
+
+		if ((pj_sockaddr_parse(pj_AF_UNSPEC(), 0, &media->conn->addr, &ip) == PJ_SUCCESS) &&
+			(ip.addr.sa_family == pj_AF_INET6())) {
+			media->conn->addr_type = STR_IP6;
+		}
+
+		/* Add ICE attributes and candidates */
+		add_ice_to_stream(session, session_media, pool, media, 1);
+
+		ast_rtp_instance_get_local_address(session_media->rtp, &addr);
+		media->desc.port = direct_media_enabled ? ast_sockaddr_port(&session_media->direct_media_addr) : (pj_uint16_t) ast_sockaddr_port(&addr);
+		media->desc.port_count = 1;
 	} else {
-		hostip = session->endpoint->media.address;
-	}
+		pjmedia_sdp_media *bundle_group_stream = sdp->media[session_media_transport->stream_num];
 
-	if (ast_strlen_zero(hostip)) {
-		ast_log(LOG_ERROR, "No local host IP available for stream %s\n", session_media->stream_type);
-		return -1;
-	}
+		/* As this is in a bundle group it shares the same details as the group instance */
+		media->desc.transport = bundle_group_stream->desc.transport;
+		media->conn = bundle_group_stream->conn;
+		media->desc.port = bundle_group_stream->desc.port;
 
-	media->conn->net_type = STR_IN;
-	/* Assume that the connection will use IPv4 until proven otherwise */
-	media->conn->addr_type = STR_IP4;
-	pj_strdup2(pool, &media->conn->addr, hostip);
-
-	if ((pj_sockaddr_parse(pj_AF_UNSPEC(), 0, &media->conn->addr, &ip) == PJ_SUCCESS) &&
-		(ip.addr.sa_family == pj_AF_INET6())) {
-		media->conn->addr_type = STR_IP6;
-	}
-
-	ast_rtp_instance_get_local_address(session_media->rtp, &addr);
-	media->desc.port = direct_media_enabled ? ast_sockaddr_port(&session_media->direct_media_addr) : (pj_uint16_t) ast_sockaddr_port(&addr);
-	media->desc.port_count = 1;
+		if (add_crypto_to_stream(session, session_media_transport, pool, media)) {
+			return -1;
+		}
 
-	/* Add ICE attributes and candidates */
-	add_ice_to_stream(session, session_media, pool, media);
+		add_ice_to_stream(session, session_media_transport, pool, media, 0);
+
+		enable_rtcp(session, session_media, NULL);
+	}
 
 	if (!(caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
-		ast_log(LOG_ERROR, "Failed to allocate %s capabilities\n", session_media->stream_type);
+		ast_log(LOG_ERROR, "Failed to allocate %s capabilities\n",
+			ast_codec_media_type2str(session_media->type));
 		return -1;
 	}
 
 	if (direct_media_enabled) {
 		ast_format_cap_get_compatible(session->endpoint->media.codecs, session->direct_media_cap, caps);
-	} else if (!ast_format_cap_count(session->req_caps) ||
-		!ast_format_cap_iscompatible(session->req_caps, session->endpoint->media.codecs)) {
-		ast_format_cap_append_from_cap(caps, session->endpoint->media.codecs, media_type);
 	} else {
-		ast_format_cap_append_from_cap(caps, session->req_caps, media_type);
+		ast_format_cap_append_from_cap(caps, ast_stream_get_formats(stream), media_type);
 	}
 
 	for (index = 0; index < ast_format_cap_count(caps); ++index) {
@@ -1292,17 +1705,28 @@ static int create_outgoing_sdp_stream(st
 			continue;
 		}
 
-		if ((rtp_code = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(session_media->rtp), 1, format, 0)) == -1) {
-			ast_log(LOG_WARNING,"Unable to get rtp codec payload code for %s\n", ast_format_get_name(format));
-			ao2_ref(format, -1);
-			continue;
+		/* If this stream is not a transport we need to use the transport codecs structure for payload management to prevent
+		 * conflicts.
+		 */
+		if (session_media_transport != session_media) {
+			if ((rtp_code = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(session_media_transport->rtp), 1, format, 0)) == -1) {
+				ast_log(LOG_WARNING,"Unable to get rtp codec payload code for %s\n", ast_format_get_name(format));
+				ao2_ref(format, -1);
+				continue;
+			}
+			/* Our instance has to match the payload number though */
+			ast_rtp_codecs_payload_set_rx(ast_rtp_instance_get_codecs(session_media->rtp), rtp_code, format);
+		} else {
+			if ((rtp_code = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(session_media->rtp), 1, format, 0)) == -1) {
+				ast_log(LOG_WARNING,"Unable to get rtp codec payload code for %s\n", ast_format_get_name(format));
+				ao2_ref(format, -1);
+				continue;
+			}
 		}
 
-		if (!(attr = generate_rtpmap_attr(session, media, pool, rtp_code, 1, format, 0))) {
-			ao2_ref(format, -1);
-			continue;
+		if ((attr = generate_rtpmap_attr(session, media, pool, rtp_code, 1, format, 0))) {
+			media->attr[media->attr_count++] = attr;
 		}
-		media->attr[media->attr_count++] = attr;
 
 		if ((attr = generate_fmtp_attr(pool, format, rtp_code))) {
 			media->attr[media->attr_count++] = attr;
@@ -1320,7 +1744,8 @@ static int create_outgoing_sdp_stream(st
 	}
 
 	/* Add non-codec formats */
-	if (media_type != AST_MEDIA_TYPE_VIDEO && media->desc.fmt_count < PJMEDIA_MAX_SDP_FMT) {
+	if (ast_sip_session_is_pending_stream_default(session, stream) && media_type != AST_MEDIA_TYPE_VIDEO
+		&& media->desc.fmt_count < PJMEDIA_MAX_SDP_FMT) {
 		for (index = 1LL; index <= AST_RTP_MAX; index <<= 1) {
 			if (!(noncodec & index)) {
 				continue;
@@ -1331,12 +1756,10 @@ static int create_outgoing_sdp_stream(st
 				continue;
 			}
 
-			if (!(attr = generate_rtpmap_attr(session, media, pool, rtp_code, 0, NULL, index))) {
-				continue;
+			if ((attr = generate_rtpmap_attr(session, media, pool, rtp_code, 0, NULL, index))) {
+				media->attr[media->attr_count++] = attr;
 			}
 
-			media->attr[media->attr_count++] = attr;
-
 			if (index == AST_RTP_DTMF) {
 				snprintf(tmp, sizeof(tmp), "%d 0-16", rtp_code);
 				attr = pjmedia_sdp_attr_create(pool, "fmtp", pj_cstr(&stmp, tmp));
@@ -1349,6 +1772,7 @@ static int create_outgoing_sdp_stream(st
 		}
 	}
 
+
 	/* If no formats were actually added to the media stream don't add it to the SDP */
 	if (!media->desc.fmt_count) {
 		return 1;
@@ -1373,38 +1797,86 @@ static int create_outgoing_sdp_stream(st
 
 	/* Add the sendrecv attribute - we purposely don't keep track because pjmedia-sdp will automatically change our offer for us */
 	attr = PJ_POOL_ZALLOC_T(pool, pjmedia_sdp_attr);
-	attr->name = STR_SENDRECV;
+	attr->name = !session_media->locally_held ? STR_SENDRECV : STR_SENDONLY;
 	media->attr[media->attr_count++] = attr;
 
 	/* If we've got rtcp-mux enabled, add it unless we received an offer without it */
-	if (session->endpoint->rtcp_mux && session_media->remote_rtcp_mux) {
+	if (session->endpoint->media.rtcp_mux && session_media->remote_rtcp_mux) {
 		attr = pjmedia_sdp_attr_create(pool, "rtcp-mux", NULL);
 		pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
 	}
 
+	add_ssrc_to_stream(session, session_media, pool, media);
+	add_msid_to_stream(session, session_media, pool, media, stream);
+	add_rtcp_fb_to_stream(session, session_media, pool, media);
+	add_extmap_to_stream(session, session_media, pool, media);
+
 	/* Add the media stream to the SDP */
 	sdp->media[sdp->media_count++] = media;
 
 	return 1;
 }
 
-static int apply_negotiated_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-				       const struct pjmedia_sdp_session *local, const struct pjmedia_sdp_media *local_stream,
-				       const struct pjmedia_sdp_session *remote, const struct pjmedia_sdp_media *remote_stream)
+static struct ast_frame *media_session_rtp_read_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media)
+{
+	struct ast_frame *f;
+
+	if (!session_media->rtp) {
+		return &ast_null_frame;
+	}
+
+	f = ast_rtp_instance_read(session_media->rtp, 0);
+	if (!f) {
+		return NULL;
+	}
+
+	ast_rtp_instance_set_last_rx(session_media->rtp, time(NULL));
+
+	return f;
+}
+
+static struct ast_frame *media_session_rtcp_read_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media)
+{
+	struct ast_frame *f;
+
+	if (!session_media->rtp) {
+		return &ast_null_frame;
+	}
+
+	f = ast_rtp_instance_read(session_media->rtp, 1);
+	if (!f) {
+		return NULL;
+	}
+
+	ast_rtp_instance_set_last_rx(session_media->rtp, time(NULL));
+
+	return f;
+}
+
+static int media_session_rtp_write_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media, struct ast_frame *frame)
+{
+	if (!session_media->rtp) {
+		return 0;
+	}
+
+	return ast_rtp_instance_write(session_media->rtp, frame);
+}
+
+static int apply_negotiated_sdp_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *local,
+	const struct pjmedia_sdp_session *remote, int index, struct ast_stream *asterisk_stream)
 {
 	RAII_VAR(struct ast_sockaddr *, addrs, NULL, ast_free);
-	enum ast_media_type media_type = stream_to_media_type(session_media->stream_type);
+	struct pjmedia_sdp_media *remote_stream = remote->media[index];
+	enum ast_media_type media_type = session_media->type;
 	char host[NI_MAXHOST];
-	int fdno, res;
+	int res;
+	struct ast_sip_session_media *session_media_transport;
 
 	if (!session->channel) {
 		return 1;
 	}
 
-	if (!local_stream->desc.port || !remote_stream->desc.port) {
-		return 1;
-	}
-
 	/* Ensure incoming transport is compatible with the endpoint's configuration */
 	if (!session->endpoint->media.rtp.use_received_transport &&
 		check_endpoint_media_transport(session->endpoint, remote_stream) == AST_SIP_MEDIA_TRANSPORT_INVALID) {
@@ -1412,54 +1884,73 @@ static int apply_negotiated_sdp_stream(s
 	}
 
 	/* Create an RTP instance if need be */
-	if (!session_media->rtp && create_rtp(session, session_media)) {
+	if (!session_media->rtp && create_rtp(session, session_media, local)) {
 		return -1;
 	}
 
-	session_media->remote_rtcp_mux = (pjmedia_sdp_media_find_attr2(remote_stream, "rtcp-mux", NULL) != NULL);
-	set_ice_components(session, session_media);
+	process_ssrc_attributes(session, session_media, remote_stream);
+	process_extmap_attributes(session, session_media, remote_stream);
 
-	enable_rtcp(session, session_media, remote_stream);
+	session_media_transport = ast_sip_session_media_get_transport(session, session_media);
 
-	res = setup_media_encryption(session, session_media, remote, remote_stream);
-	if (!session->endpoint->media.rtp.encryption_optimistic && res) {
-		/* If optimistic encryption is disabled and crypto should have been enabled but was not
-		 * this session must fail.
-		 */
-		return -1;
-	}
+	if (session_media_transport == session_media || !session_media->bundled) {
+		session_media->remote_rtcp_mux = (pjmedia_sdp_media_find_attr2(remote_stream, "rtcp-mux", NULL) != NULL);
+		set_ice_components(session, session_media);
 
-	if (!remote_stream->conn && !remote->conn) {
-		return 1;
-	}
+		enable_rtcp(session, session_media, remote_stream);
 
-	ast_copy_pj_str(host, remote_stream->conn ? &remote_stream->conn->addr : &remote->conn->addr, sizeof(host));
+		res = setup_media_encryption(session, session_media, remote, remote_stream);
+		if (!session->endpoint->media.rtp.encryption_optimistic && res) {
+			/* If optimistic encryption is disabled and crypto should have been enabled but was not
+			 * this session must fail.
+			 */
+			return -1;
+		}
 
-	/* Ensure that the address provided is valid */
-	if (ast_sockaddr_resolve(&addrs, host, PARSE_PORT_FORBID, AST_AF_UNSPEC) <= 0) {
-		/* The provided host was actually invalid so we error out this negotiation */
-		return -1;
-	}
+		if (!remote_stream->conn && !remote->conn) {
+			return 1;
+		}
 
-	/* Apply connection information to the RTP instance */
-	ast_sockaddr_set_port(addrs, remote_stream->desc.port);
-	ast_rtp_instance_set_remote_address(session_media->rtp, addrs);
-	if (set_caps(session, session_media, remote_stream)) {
-		return 1;
-	}
+		ast_copy_pj_str(host, remote_stream->conn ? &remote_stream->conn->addr : &remote->conn->addr, sizeof(host));
 
-	if ((fdno = media_type_to_fdno(media_type)) < 0) {
-		return -1;
+		/* Ensure that the address provided is valid */
+		if (ast_sockaddr_resolve(&addrs, host, PARSE_PORT_FORBID, AST_AF_UNSPEC) <= 0) {
+			/* The provided host was actually invalid so we error out this negotiation */
+			return -1;
+		}
+
+		/* Apply connection information to the RTP instance */
+		ast_sockaddr_set_port(addrs, remote_stream->desc.port);
+		ast_rtp_instance_set_remote_address(session_media->rtp, addrs);
+
+		ast_sip_session_media_set_write_callback(session, session_media, media_session_rtp_write_callback);
+		ast_sip_session_media_add_read_callback(session, session_media, ast_rtp_instance_fd(session_media->rtp, 0),
+			media_session_rtp_read_callback);
+		if (!session->endpoint->media.rtcp_mux || !session_media->remote_rtcp_mux) {
+			ast_sip_session_media_add_read_callback(session, session_media, ast_rtp_instance_fd(session_media->rtp, 1),
+				media_session_rtcp_read_callback);
+		}
+
+		/* If ICE support is enabled find all the needed attributes */
+		process_ice_attributes(session, session_media, remote, remote_stream);
+	} else {
+		/* This is bundled with another session, so mark it as such */
+		ast_rtp_instance_bundle(session_media->rtp, session_media_transport->rtp);
+		ast_sip_session_media_set_write_callback(session, session_media, media_session_rtp_write_callback);
+		enable_rtcp(session, session_media, remote_stream);
 	}
-	ast_channel_set_fd(session->channel, fdno, ast_rtp_instance_fd(session_media->rtp, 0));
-	if (!session->endpoint->rtcp_mux || !session_media->remote_rtcp_mux) {
-		ast_channel_set_fd(session->channel, fdno + 1, ast_rtp_instance_fd(session_media->rtp, 1));
+
+	if (set_caps(session, session_media, session_media_transport, remote_stream, 0, asterisk_stream)) {
+		return 1;
 	}
 
-	/* If ICE support is enabled find all the needed attributes */
-	process_ice_attributes(session, session_media, remote, remote_stream);
+	/* Set the channel uniqueid on the RTP instance now that it is becoming active */
+	ast_channel_lock(session->channel);
+	ast_rtp_instance_set_channel_id(session_media->rtp, ast_channel_uniqueid(session->channel));
+	ast_channel_unlock(session->channel);
 
 	/* Ensure the RTP instance is active */
+	ast_rtp_instance_set_stream_num(session_media->rtp, ast_stream_get_position(asterisk_stream));
 	ast_rtp_instance_activate(session_media->rtp);
 
 	/* audio stream handles music on hold */
@@ -1475,18 +1966,18 @@ static int apply_negotiated_sdp_stream(s
 		ast_sockaddr_is_any(addrs) ||
 		pjmedia_sdp_media_find_attr2(remote_stream, "sendonly", NULL) ||
 		pjmedia_sdp_media_find_attr2(remote_stream, "inactive", NULL)) {
-		if (!session_media->held) {
+		if (!session_media->remotely_held) {
 			/* The remote side has put us on hold */
 			ast_queue_hold(session->channel, session->endpoint->mohsuggest);
 			ast_rtp_instance_stop(session_media->rtp);
 			ast_queue_frame(session->channel, &ast_null_frame);
-			session_media->held = 1;
+			session_media->remotely_held = 1;
 		}
-	} else if (session_media->held) {
+	} else if (session_media->remotely_held) {
 		/* The remote side has taken us off hold */
 		ast_queue_unhold(session->channel);
 		ast_queue_frame(session->channel, &ast_null_frame);
-		session_media->held = 0;
+		session_media->remotely_held = 0;
 	} else if ((pjmedia_sdp_neg_was_answer_remote(session->inv_session->neg) == PJ_FALSE)
 		&& (session->inv_session->state == PJSIP_INV_STATE_CONFIRMED)) {
 		ast_queue_control(session->channel, AST_CONTROL_UPDATE_RTP_PEER);
@@ -1496,7 +1987,7 @@ static int apply_negotiated_sdp_stream(s
 	session_media->encryption = session->endpoint->media.rtp.encryption;
 
 	if (session->endpoint->media.rtp.keepalive > 0 &&
-			stream_to_media_type(session_media->stream_type) == AST_MEDIA_TYPE_AUDIO) {
+			session_media->type == AST_MEDIA_TYPE_AUDIO) {
 		ast_rtp_instance_set_keepalive(session_media->rtp, session->endpoint->media.rtp.keepalive);
 		/* Schedule the initial keepalive early in case this is being used to punch holes through
 		 * a NAT. This way there won't be an awkward delay before media starts flowing in some
@@ -1517,9 +2008,9 @@ static int apply_negotiated_sdp_stream(s
 	 * instance itself.
 	 */
 	ast_rtp_instance_set_timeout(session_media->rtp, 0);
-	if (session->endpoint->media.rtp.timeout && !session_media->held) {
+	if (session->endpoint->media.rtp.timeout && !session_media->remotely_held) {
 		ast_rtp_instance_set_timeout(session_media->rtp, session->endpoint->media.rtp.timeout);
-	} else if (session->endpoint->media.rtp.timeout_hold && session_media->held) {
+	} else if (session->endpoint->media.rtp.timeout_hold && session_media->remotely_held) {
 		ast_rtp_instance_set_timeout(session_media->rtp, session->endpoint->media.rtp.timeout_hold);
 	}
 
@@ -1656,8 +2147,6 @@ static int unload_module(void)
  */
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	if (ast_check_ipv6()) {
 		ast_sockaddr_parse(&address_rtp, "::", 0);
 	} else {
@@ -1694,8 +2183,9 @@ end:
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP SDP RTP/AVP stream handler",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_send_to_voicemail.c 1:16.1.1~dfsg-1/res/res_pjsip_send_to_voicemail.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_send_to_voicemail.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_send_to_voicemail.c	2018-12-26 18:45:13.000000000 +0000
@@ -215,12 +215,7 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
-	if (ast_sip_session_register_supplement(&refer_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register Send to Voicemail supplement\n");
-		return AST_MODULE_LOAD_DECLINE;
-	}
+	ast_sip_session_register_supplement(&refer_supplement);
 
 	return AST_MODULE_LOAD_SUCCESS;
 }
@@ -236,4 +231,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+	.requires = "res_pjsip,res_pjsip_session",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_session.c 1:16.1.1~dfsg-1/res/res_pjsip_session.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_session.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_session.c	2018-12-26 18:45:13.000000000 +0000
@@ -47,12 +47,16 @@
 #include "asterisk/features_config.h"
 #include "asterisk/pickup.h"
 #include "asterisk/test.h"
+#include "asterisk/stream.h"
 
 #define SDP_HANDLER_BUCKETS 11
 
 #define MOD_DATA_ON_RESPONSE "on_response"
 #define MOD_DATA_NAT_HOOK "nat_hook"
 
+/* Most common case is one audio and one video stream */
+#define DEFAULT_NUM_SESSION_MEDIA 2
+
 /* Some forward declarations */
 static void handle_incoming_request(struct ast_sip_session *session, pjsip_rx_data *rdata);
 static void handle_incoming_response(struct ast_sip_session *session, pjsip_rx_data *rdata,
@@ -103,23 +107,6 @@ static int sdp_handler_list_cmp(void *ob
 	return strcmp(handler_list1->stream_type, stream_type2) ? 0 : CMP_MATCH | CMP_STOP;
 }
 
-static int session_media_hash(const void *obj, int flags)
-{
-	const struct ast_sip_session_media *session_media = obj;
-	const char *stream_type = flags & OBJ_KEY ? obj : session_media->stream_type;
-
-	return ast_str_hash(stream_type);
-}
-
-static int session_media_cmp(void *obj, void *arg, int flags)
-{
-	struct ast_sip_session_media *session_media1 = obj;
-	struct ast_sip_session_media *session_media2 = arg;
-	const char *stream_type2 = flags & OBJ_KEY ? arg : session_media2->stream_type;
-
-	return strcmp(session_media1->stream_type, stream_type2) ? 0 : CMP_MATCH | CMP_STOP;
-}
-
 int ast_sip_session_register_sdp_handler(struct ast_sip_session_sdp_handler *handler, const char *stream_type)
 {
 	RAII_VAR(struct sdp_handler_list *, handler_list,
@@ -137,7 +124,7 @@ int ast_sip_session_register_sdp_handler
 		}
 		AST_LIST_INSERT_TAIL(&handler_list->list, handler, next);
 		ast_debug(1, "Registered SDP stream handler '%s' for stream type '%s'\n", handler->id, stream_type);
-		ast_module_ref(ast_module_info->self);
+
 		return 0;
 	}
 
@@ -154,7 +141,7 @@ int ast_sip_session_register_sdp_handler
 		return -1;
 	}
 	ast_debug(1, "Registered SDP stream handler '%s' for stream type '%s'\n", handler->id, stream_type);
-	ast_module_ref(ast_module_info->self);
+
 	return 0;
 }
 
@@ -169,7 +156,6 @@ static int remove_handler(void *obj, voi
 		if (!strcmp(iter->id, handler->id)) {
 			AST_LIST_REMOVE_CURRENT(next);
 			ast_debug(1, "Unregistered SDP stream handler '%s' for stream type '%s'\n", handler->id, stream_type);
-			ast_module_unref(ast_module_info->self);
 		}
 	}
 	AST_LIST_TRAVERSE_SAFE_END;
@@ -187,6 +173,194 @@ void ast_sip_session_unregister_sdp_hand
 	ao2_callback_data(sdp_handlers, OBJ_KEY | OBJ_UNLINK | OBJ_NODATA, remove_handler, (void *)stream_type, handler);
 }
 
+static struct ast_sip_session_media_state *internal_sip_session_media_state_alloc(
+	size_t sessions, size_t read_callbacks)
+{
+	struct ast_sip_session_media_state *media_state;
+
+	media_state = ast_calloc(1, sizeof(*media_state));
+	if (!media_state) {
+		return NULL;
+	}
+
+	if (AST_VECTOR_INIT(&media_state->sessions, sessions) < 0) {
+		ast_free(media_state);
+		return NULL;
+	}
+
+	if (AST_VECTOR_INIT(&media_state->read_callbacks, read_callbacks) < 0) {
+		AST_VECTOR_FREE(&media_state->sessions);
+		ast_free(media_state);
+		return NULL;
+	}
+
+	return media_state;
+}
+
+struct ast_sip_session_media_state *ast_sip_session_media_state_alloc(void)
+{
+	return internal_sip_session_media_state_alloc(
+		DEFAULT_NUM_SESSION_MEDIA, DEFAULT_NUM_SESSION_MEDIA);
+}
+
+void ast_sip_session_media_state_reset(struct ast_sip_session_media_state *media_state)
+{
+	int index;
+
+	if (!media_state) {
+		return;
+	}
+
+	AST_VECTOR_RESET(&media_state->sessions, ao2_cleanup);
+	AST_VECTOR_RESET(&media_state->read_callbacks, AST_VECTOR_ELEM_CLEANUP_NOOP);
+
+	for (index = 0; index < AST_MEDIA_TYPE_END; ++index) {
+		media_state->default_session[index] = NULL;
+	}
+
+	ast_stream_topology_free(media_state->topology);
+	media_state->topology = NULL;
+}
+
+struct ast_sip_session_media_state *ast_sip_session_media_state_clone(const struct ast_sip_session_media_state *media_state)
+{
+	struct ast_sip_session_media_state *cloned;
+	int index;
+
+	if (!media_state) {
+		return NULL;
+	}
+
+	cloned = internal_sip_session_media_state_alloc(
+		AST_VECTOR_SIZE(&media_state->sessions),
+		AST_VECTOR_SIZE(&media_state->read_callbacks));
+	if (!cloned) {
+		return NULL;
+	}
+
+	if (media_state->topology) {
+		cloned->topology = ast_stream_topology_clone(media_state->topology);
+		if (!cloned->topology) {
+			ast_sip_session_media_state_free(cloned);
+			return NULL;
+		}
+	}
+
+	for (index = 0; index < AST_VECTOR_SIZE(&media_state->sessions); ++index) {
+		struct ast_sip_session_media *session_media = AST_VECTOR_GET(&media_state->sessions, index);
+		enum ast_media_type type = ast_stream_get_type(ast_stream_topology_get_stream(cloned->topology, index));
+
+		ao2_bump(session_media);
+		if (AST_VECTOR_REPLACE(&cloned->sessions, index, session_media)) {
+			ao2_cleanup(session_media);
+		}
+		if (ast_stream_get_state(ast_stream_topology_get_stream(cloned->topology, index)) != AST_STREAM_STATE_REMOVED &&
+			!cloned->default_session[type]) {
+			cloned->default_session[type] = session_media;
+		}
+	}
+
+	for (index = 0; index < AST_VECTOR_SIZE(&media_state->read_callbacks); ++index) {
+		struct ast_sip_session_media_read_callback_state *read_callback = AST_VECTOR_GET_ADDR(&media_state->read_callbacks, index);
+
+		AST_VECTOR_REPLACE(&cloned->read_callbacks, index, *read_callback);
+	}
+
+	return cloned;
+}
+
+void ast_sip_session_media_state_free(struct ast_sip_session_media_state *media_state)
+{
+	if (!media_state) {
+		return;
+	}
+
+	/* This will reset the internal state so we only have to free persistent things */
+	ast_sip_session_media_state_reset(media_state);
+
+	AST_VECTOR_FREE(&media_state->sessions);
+	AST_VECTOR_FREE(&media_state->read_callbacks);
+
+	ast_free(media_state);
+}
+
+int ast_sip_session_is_pending_stream_default(const struct ast_sip_session *session, const struct ast_stream *stream)
+{
+	int index;
+
+	if (!session->pending_media_state->topology) {
+		ast_log(LOG_WARNING, "Pending topology was NULL for channel '%s'\n",
+			session->channel ? ast_channel_name(session->channel) : "unknown");
+		return 0;
+	}
+
+	if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+		return 0;
+	}
+
+	for (index = 0; index < ast_stream_topology_get_count(session->pending_media_state->topology); ++index) {
+		if (ast_stream_get_type(ast_stream_topology_get_stream(session->pending_media_state->topology, index)) !=
+			ast_stream_get_type(stream)) {
+			continue;
+		}
+
+		return ast_stream_topology_get_stream(session->pending_media_state->topology, index) == stream ? 1 : 0;
+	}
+
+	return 0;
+}
+
+int ast_sip_session_media_add_read_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	int fd, ast_sip_session_media_read_cb callback)
+{
+	struct ast_sip_session_media_read_callback_state callback_state = {
+		.fd = fd,
+		.read_callback = callback,
+		.session = session_media,
+	};
+
+	/* The contents of the vector are whole structs and not pointers */
+	return AST_VECTOR_APPEND(&session->pending_media_state->read_callbacks, callback_state);
+}
+
+int ast_sip_session_media_set_write_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
+	ast_sip_session_media_write_cb callback)
+{
+	if (session_media->write_callback) {
+		if (session_media->write_callback == callback) {
+			return 0;
+		}
+
+		return -1;
+	}
+
+	session_media->write_callback = callback;
+
+	return 0;
+}
+
+struct ast_sip_session_media *ast_sip_session_media_get_transport(struct ast_sip_session *session, struct ast_sip_session_media *session_media)
+{
+	int index;
+
+	if (!session->endpoint->media.bundle || ast_strlen_zero(session_media->mid)) {
+		return session_media;
+	}
+
+	for (index = 0; index < AST_VECTOR_SIZE(&session->pending_media_state->sessions); ++index) {
+		struct ast_sip_session_media *bundle_group_session_media;
+
+		bundle_group_session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, index);
+
+		/* The first session which is in the bundle group is considered the authoritative session for transport */
+		if (bundle_group_session_media->bundle_group == session_media->bundle_group) {
+			return bundle_group_session_media;
+		}
+	}
+
+	return session_media;
+}
+
 /*!
  * \brief Set an SDP stream handler for a corresponding session media.
  *
@@ -207,50 +381,369 @@ static void session_media_set_handler(st
 	session_media->handler = handler;
 }
 
+static int stream_destroy(void *obj, void *arg, int flags)
+{
+	struct sdp_handler_list *handler_list = obj;
+	struct ast_sip_session_media *session_media = arg;
+	struct ast_sip_session_sdp_handler *handler;
+
+	AST_LIST_TRAVERSE(&handler_list->list, handler, next) {
+		handler->stream_destroy(session_media);
+	}
+
+	return 0;
+}
+
+static void session_media_dtor(void *obj)
+{
+	struct ast_sip_session_media *session_media = obj;
+
+	/* It is possible for multiple handlers to have allocated memory on the
+	 * session media (usually through a stream changing types). Therefore, we
+	 * traverse all the SDP handlers and let them all call stream_destroy on
+	 * the session_media
+	 */
+	ao2_callback(sdp_handlers, 0, stream_destroy, session_media);
+
+	if (session_media->srtp) {
+		ast_sdp_srtp_destroy(session_media->srtp);
+	}
+
+	ast_free(session_media->mid);
+	ast_free(session_media->remote_mslabel);
+}
+
+struct ast_sip_session_media *ast_sip_session_media_state_add(struct ast_sip_session *session,
+	struct ast_sip_session_media_state *media_state, enum ast_media_type type, int position)
+{
+	struct ast_sip_session_media *session_media = NULL;
+
+	/* It is possible for this media state to already contain a session for the stream. If this
+	 * is the case we simply return it.
+	 */
+	if (position < AST_VECTOR_SIZE(&media_state->sessions)) {
+		return AST_VECTOR_GET(&media_state->sessions, position);
+	}
+
+	/* Determine if we can reuse the session media from the active media state if present */
+	if (position < AST_VECTOR_SIZE(&session->active_media_state->sessions)) {
+		session_media = AST_VECTOR_GET(&session->active_media_state->sessions, position);
+		/* A stream can never exist without an accompanying media session */
+		if (session_media->type == type) {
+			ao2_ref(session_media, +1);
+		} else {
+			session_media = NULL;
+		}
+	}
+
+	if (!session_media) {
+		/* No existing media session we can use so create a new one */
+		session_media = ao2_alloc_options(sizeof(*session_media), session_media_dtor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+		if (!session_media) {
+			return NULL;
+		}
+
+		session_media->encryption = session->endpoint->media.rtp.encryption;
+		session_media->remote_ice = session->endpoint->media.rtp.ice_support;
+		session_media->remote_rtcp_mux = session->endpoint->media.rtcp_mux;
+		session_media->keepalive_sched_id = -1;
+		session_media->timeout_sched_id = -1;
+		session_media->type = type;
+		session_media->stream_num = position;
+
+		if (session->endpoint->media.bundle) {
+			/* This is a new stream so create a new mid based on media type and position, which makes it unique.
+			 * If this is the result of an offer the mid will just end up getting replaced.
+			 */
+			if (ast_asprintf(&session_media->mid, "%s-%d", ast_codec_media_type2str(type), position) < 0) {
+				ao2_ref(session_media, -1);
+				return NULL;
+			}
+			session_media->bundle_group = 0;
+
+			/* Some WebRTC clients can't handle an offer to bundle media streams. Instead they expect them to
+			 * already be bundled. Every client handles this scenario though so if WebRTC is enabled just go
+			 * ahead and treat the streams as having already been bundled.
+			 */
+			session_media->bundled = session->endpoint->media.webrtc;
+		} else {
+			session_media->bundle_group = -1;
+		}
+	}
+
+	if (AST_VECTOR_REPLACE(&media_state->sessions, position, session_media)) {
+		ao2_ref(session_media, -1);
+
+		return NULL;
+	}
+
+	/* If this stream will be active in some way and it is the first of this type then consider this the default media session to match */
+	if (!media_state->default_session[type] && ast_stream_get_state(ast_stream_topology_get_stream(media_state->topology, position)) != AST_STREAM_STATE_REMOVED) {
+		media_state->default_session[type] = session_media;
+	}
+
+	return session_media;
+}
+
+static int is_stream_limitation_reached(enum ast_media_type type, const struct ast_sip_endpoint *endpoint, int *type_streams)
+{
+	switch (type) {
+	case AST_MEDIA_TYPE_AUDIO:
+		return !(type_streams[type] < endpoint->media.max_audio_streams);
+	case AST_MEDIA_TYPE_VIDEO:
+		return !(type_streams[type] < endpoint->media.max_video_streams);
+	case AST_MEDIA_TYPE_IMAGE:
+		/* We don't have an option for image (T.38) streams so cap it to one. */
+		return (type_streams[type] > 0);
+	case AST_MEDIA_TYPE_UNKNOWN:
+	case AST_MEDIA_TYPE_TEXT:
+	default:
+		/* We don't want any unknown or "other" streams on our endpoint,
+		 * so always just say we've reached the limit
+		 */
+		return 1;
+	}
+}
+
+static int get_mid_bundle_group(const pjmedia_sdp_session *sdp, const char *mid)
+{
+	int bundle_group = 0;
+	int index;
+
+	for (index = 0; index < sdp->attr_count; ++index) {
+		pjmedia_sdp_attr *attr = sdp->attr[index];
+		char value[pj_strlen(&attr->value) + 1], *mids = value, *attr_mid;
+
+		if (pj_strcmp2(&attr->name, "group") || pj_strncmp2(&attr->value, "BUNDLE", 6)) {
+			continue;
+		}
+
+		ast_copy_pj_str(value, &attr->value, sizeof(value));
+
+		/* Skip the BUNDLE at the front */
+		mids += 7;
+
+		while ((attr_mid = strsep(&mids, " "))) {
+			if (!strcmp(attr_mid, mid)) {
+				/* The ordering of attributes determines our internal identification of the bundle group based on number,
+				 * with -1 being not in a bundle group. Since this is only exposed internally for response purposes it's
+				 * actually even fine if things move around.
+				 */
+				return bundle_group;
+			}
+		}
+
+		bundle_group++;
+	}
+
+	return -1;
+}
+
+static int set_mid_and_bundle_group(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media,
+	const pjmedia_sdp_session *sdp,
+	const struct pjmedia_sdp_media *stream)
+{
+	pjmedia_sdp_attr *attr;
+
+	if (!session->endpoint->media.bundle) {
+		return 0;
+	}
+
+	/* By default on an incoming negotiation we assume no mid and bundle group is present */
+	ast_free(session_media->mid);
+	session_media->mid = NULL;
+	session_media->bundle_group = -1;
+	session_media->bundled = 0;
+
+	/* Grab the media identifier for the stream */
+	attr = pjmedia_sdp_media_find_attr2(stream, "mid", NULL);
+	if (!attr) {
+		return 0;
+	}
+
+	session_media->mid = ast_calloc(1, attr->value.slen + 1);
+	if (!session_media->mid) {
+		return 0;
+	}
+	ast_copy_pj_str(session_media->mid, &attr->value, attr->value.slen + 1);
+
+	/* Determine what bundle group this is part of */
+	session_media->bundle_group = get_mid_bundle_group(sdp, session_media->mid);
+
+	/* If this is actually part of a bundle group then the other side requested or accepted the bundle request */
+	session_media->bundled = session_media->bundle_group != -1;
+
+	return 0;
+}
+
+static void set_remote_mslabel_and_stream_group(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media,
+	const pjmedia_sdp_session *sdp,
+	const struct pjmedia_sdp_media *stream,
+	struct ast_stream *asterisk_stream)
+{
+	int index;
+
+	ast_free(session_media->remote_mslabel);
+	session_media->remote_mslabel = NULL;
+
+	for (index = 0; index < stream->attr_count; ++index) {
+		pjmedia_sdp_attr *attr = stream->attr[index];
+		char attr_value[pj_strlen(&attr->value) + 1];
+		char *ssrc_attribute_name, *ssrc_attribute_value = NULL;
+		char *msid, *tmp = attr_value;
+		static const pj_str_t STR_msid = { "msid", 4 };
+		static const pj_str_t STR_ssrc = { "ssrc", 4 };
+
+		if (!pj_strcmp(&attr->name, &STR_msid)) {
+			ast_copy_pj_str(attr_value, &attr->value, sizeof(attr_value));
+			msid = strsep(&tmp, " ");
+			session_media->remote_mslabel = ast_strdup(msid);
+			break;
+		} else if (!pj_strcmp(&attr->name, &STR_ssrc)) {
+			ast_copy_pj_str(attr_value, &attr->value, sizeof(attr_value));
+
+			if ((ssrc_attribute_name = strchr(attr_value, ' '))) {
+				/* This has an actual attribute */
+				*ssrc_attribute_name++ = '\0';
+				ssrc_attribute_value = strchr(ssrc_attribute_name, ':');
+				if (ssrc_attribute_value) {
+					/* Values are actually optional according to the spec */
+					*ssrc_attribute_value++ = '\0';
+				}
+
+				if (!strcasecmp(ssrc_attribute_name, "mslabel") && !ast_strlen_zero(ssrc_attribute_value)) {
+					session_media->remote_mslabel = ast_strdup(ssrc_attribute_value);
+					break;
+				}
+			}
+		}
+	}
+
+	if (ast_strlen_zero(session_media->remote_mslabel)) {
+		return;
+	}
+
+	/* Iterate through the existing streams looking for a match and if so then group this with it */
+	for (index = 0; index < AST_VECTOR_SIZE(&session->pending_media_state->sessions); ++index) {
+		struct ast_sip_session_media *group_session_media;
+
+		group_session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, index);
+
+		if (ast_strlen_zero(group_session_media->remote_mslabel) ||
+			strcmp(group_session_media->remote_mslabel, session_media->remote_mslabel)) {
+			continue;
+		}
+
+		ast_stream_set_group(asterisk_stream, index);
+		break;
+	}
+}
+
+static void remove_stream_from_bundle(struct ast_sip_session_media *session_media,
+	struct ast_stream *stream)
+{
+	ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
+	ast_free(session_media->mid);
+	session_media->mid = NULL;
+	session_media->bundle_group = -1;
+	session_media->bundled = 0;
+}
+
 static int handle_incoming_sdp(struct ast_sip_session *session, const pjmedia_sdp_session *sdp)
 {
 	int i;
 	int handled = 0;
+	int type_streams[AST_MEDIA_TYPE_END] = {0};
 
 	if (session->inv_session && session->inv_session->state == PJSIP_INV_STATE_DISCONNECTED) {
 		ast_log(LOG_ERROR, "Failed to handle incoming SDP. Session has been already disconnected\n");
 		return -1;
 	}
 
+	/* It is possible for SDP deferral to have already created a pending topology */
+	if (!session->pending_media_state->topology) {
+		session->pending_media_state->topology = ast_stream_topology_alloc();
+		if (!session->pending_media_state->topology) {
+			return -1;
+		}
+	}
+
 	for (i = 0; i < sdp->media_count; ++i) {
 		/* See if there are registered handlers for this media stream type */
 		char media[20];
 		struct ast_sip_session_sdp_handler *handler;
 		RAII_VAR(struct sdp_handler_list *, handler_list, NULL, ao2_cleanup);
-		RAII_VAR(struct ast_sip_session_media *, session_media, NULL, ao2_cleanup);
+		struct ast_sip_session_media *session_media = NULL;
 		int res;
+		enum ast_media_type type;
+		struct ast_stream *stream = NULL;
+		pjmedia_sdp_media *remote_stream = sdp->media[i];
 
 		/* We need a null-terminated version of the media string */
 		ast_copy_pj_str(media, &sdp->media[i]->desc.media, sizeof(media));
+		type = ast_media_type_from_str(media);
+
+		/* See if we have an already existing stream, which can occur from SDP deferral checking */
+		if (i < ast_stream_topology_get_count(session->pending_media_state->topology)) {
+			stream = ast_stream_topology_get_stream(session->pending_media_state->topology, i);
+		}
+		if (!stream) {
+			struct ast_stream *existing_stream = NULL;
+
+			if (session->active_media_state->topology &&
+				(i < ast_stream_topology_get_count(session->active_media_state->topology))) {
+				existing_stream = ast_stream_topology_get_stream(session->active_media_state->topology, i);
+			}
+
+			stream = ast_stream_alloc(existing_stream ? ast_stream_get_name(existing_stream) : ast_codec_media_type2str(type), type);
+			if (!stream) {
+				return -1;
+			}
+			if (ast_stream_topology_set_stream(session->pending_media_state->topology, i, stream)) {
+				ast_stream_free(stream);
+				return -1;
+			}
+		}
 
-		session_media = ao2_find(session->media, media, OBJ_KEY);
+		session_media = ast_sip_session_media_state_add(session, session->pending_media_state, ast_media_type_from_str(media), i);
 		if (!session_media) {
-			/* if the session_media doesn't exist, there weren't
-			 * any handlers at the time of its creation */
+			return -1;
+		}
+
+		/* If this stream is already declined mark it as such, or mark it as such if we've reached the limit */
+		if (!remote_stream->desc.port || is_stream_limitation_reached(type, session->endpoint, type_streams)) {
+			ast_debug(1, "Declining incoming SDP media stream '%s' at position '%d'\n",
+				ast_codec_media_type2str(type), i);
+			remove_stream_from_bundle(session_media, stream);
 			continue;
 		}
 
+		set_mid_and_bundle_group(session, session_media, sdp, remote_stream);
+		set_remote_mslabel_and_stream_group(session, session_media, sdp, remote_stream, stream);
+
 		if (session_media->handler) {
 			handler = session_media->handler;
 			ast_debug(1, "Negotiating incoming SDP media stream '%s' using %s SDP handler\n",
-				session_media->stream_type,
+				ast_codec_media_type2str(session_media->type),
 				session_media->handler->id);
-			res = handler->negotiate_incoming_sdp_stream(session, session_media, sdp,
-				sdp->media[i]);
+			res = handler->negotiate_incoming_sdp_stream(session, session_media, sdp, i, stream);
 			if (res < 0) {
 				/* Catastrophic failure. Abort! */
 				return -1;
+			} else if (res == 0) {
+				ast_debug(1, "Declining incoming SDP media stream '%s' at position '%d'\n",
+					ast_codec_media_type2str(type), i);
+				remove_stream_from_bundle(session_media, stream);
+				continue;
 			} else if (res > 0) {
 				ast_debug(1, "Media stream '%s' handled by %s\n",
-					session_media->stream_type,
+					ast_codec_media_type2str(session_media->type),
 					session_media->handler->id);
 				/* Handled by this handler. Move to the next stream */
 				handled = 1;
+				++type_streams[type];
 				continue;
 			}
 		}
@@ -265,21 +758,25 @@ static int handle_incoming_sdp(struct as
 				continue;
 			}
 			ast_debug(1, "Negotiating incoming SDP media stream '%s' using %s SDP handler\n",
-				session_media->stream_type,
+				ast_codec_media_type2str(session_media->type),
 				handler->id);
-			res = handler->negotiate_incoming_sdp_stream(session, session_media, sdp,
-				sdp->media[i]);
+			res = handler->negotiate_incoming_sdp_stream(session, session_media, sdp, i, stream);
 			if (res < 0) {
 				/* Catastrophic failure. Abort! */
 				return -1;
-			}
-			if (res > 0) {
+			} else if (res == 0) {
+				ast_debug(1, "Declining incoming SDP media stream '%s' at position '%d'\n",
+					ast_codec_media_type2str(type), i);
+				remove_stream_from_bundle(session_media, stream);
+				continue;
+			} else if (res > 0) {
 				ast_debug(1, "Media stream '%s' handled by %s\n",
-					session_media->stream_type,
+					ast_codec_media_type2str(session_media->type),
 					handler->id);
 				/* Handled by this handler. Move to the next stream */
 				session_media_set_handler(session_media, handler);
 				handled = 1;
+				++type_streams[type];
 				break;
 			}
 		}
@@ -290,110 +787,240 @@ static int handle_incoming_sdp(struct as
 	return 0;
 }
 
-struct handle_negotiated_sdp_cb {
-	struct ast_sip_session *session;
-	const pjmedia_sdp_session *local;
-	const pjmedia_sdp_session *remote;
-};
+static int handle_negotiated_sdp_session_media(struct ast_sip_session_media *session_media,
+		struct ast_sip_session *session, const pjmedia_sdp_session *local,
+		const pjmedia_sdp_session *remote, int index, struct ast_stream *asterisk_stream)
+{
+	/* See if there are registered handlers for this media stream type */
+	struct pjmedia_sdp_media *local_stream = local->media[index];
+	char media[20];
+	struct ast_sip_session_sdp_handler *handler;
+	RAII_VAR(struct sdp_handler_list *, handler_list, NULL, ao2_cleanup);
+	int res;
 
-static int handle_negotiated_sdp_session_media(void *obj, void *arg, int flags)
+	/* For backwards compatibility we only reflect the stream state correctly on
+	 * the non-default streams. This is because the stream state is also used for
+	 * signaling that someone has placed us on hold. This situation is not handled
+	 * currently and can result in the remote side being sort of placed on hold too.
+	 */
+	if (!ast_sip_session_is_pending_stream_default(session, asterisk_stream)) {
+		/* Determine the state of the stream based on our local SDP */
+		if (pjmedia_sdp_media_find_attr2(local_stream, "sendonly", NULL)) {
+			ast_stream_set_state(asterisk_stream, AST_STREAM_STATE_SENDONLY);
+		} else if (pjmedia_sdp_media_find_attr2(local_stream, "recvonly", NULL)) {
+			ast_stream_set_state(asterisk_stream, AST_STREAM_STATE_RECVONLY);
+		} else if (pjmedia_sdp_media_find_attr2(local_stream, "inactive", NULL)) {
+			ast_stream_set_state(asterisk_stream, AST_STREAM_STATE_INACTIVE);
+		} else {
+			ast_stream_set_state(asterisk_stream, AST_STREAM_STATE_SENDRECV);
+		}
+	} else {
+		ast_stream_set_state(asterisk_stream, AST_STREAM_STATE_SENDRECV);
+	}
+
+	/* We need a null-terminated version of the media string */
+	ast_copy_pj_str(media, &local->media[index]->desc.media, sizeof(media));
+
+	set_mid_and_bundle_group(session, session_media, remote, remote->media[index]);
+	set_remote_mslabel_and_stream_group(session, session_media, remote, remote->media[index], asterisk_stream);
+
+	handler = session_media->handler;
+	if (handler) {
+		ast_debug(1, "Applying negotiated SDP media stream '%s' using %s SDP handler\n",
+			ast_codec_media_type2str(session_media->type),
+			handler->id);
+		res = handler->apply_negotiated_sdp_stream(session, session_media, local, remote, index, asterisk_stream);
+		if (res >= 0) {
+			ast_debug(1, "Applied negotiated SDP media stream '%s' using %s SDP handler\n",
+				ast_codec_media_type2str(session_media->type),
+				handler->id);
+			return 0;
+		}
+		return -1;
+	}
+
+	handler_list = ao2_find(sdp_handlers, media, OBJ_KEY);
+	if (!handler_list) {
+		ast_debug(1, "No registered SDP handlers for media type '%s'\n", media);
+		return -1;
+	}
+	AST_LIST_TRAVERSE(&handler_list->list, handler, next) {
+		if (handler == session_media->handler) {
+			continue;
+		}
+		ast_debug(1, "Applying negotiated SDP media stream '%s' using %s SDP handler\n",
+			ast_codec_media_type2str(session_media->type),
+			handler->id);
+		res = handler->apply_negotiated_sdp_stream(session, session_media, local, remote, index, asterisk_stream);
+		if (res < 0) {
+			/* Catastrophic failure. Abort! */
+			return -1;
+		}
+		if (res > 0) {
+			ast_debug(1, "Applied negotiated SDP media stream '%s' using %s SDP handler\n",
+				ast_codec_media_type2str(session_media->type),
+				handler->id);
+			/* Handled by this handler. Move to the next stream */
+			session_media_set_handler(session_media, handler);
+			return 0;
+		}
+	}
+
+	if (session_media->handler && session_media->handler->stream_stop) {
+		ast_debug(1, "Stopping SDP media stream '%s' as it is not currently negotiated\n",
+			ast_codec_media_type2str(session_media->type));
+		session_media->handler->stream_stop(session_media);
+	}
+
+	return 0;
+}
+
+static int handle_negotiated_sdp(struct ast_sip_session *session, const pjmedia_sdp_session *local, const pjmedia_sdp_session *remote)
 {
-	struct ast_sip_session_media *session_media = obj;
-	struct handle_negotiated_sdp_cb *callback_data = arg;
-	struct ast_sip_session *session = callback_data->session;
-	const pjmedia_sdp_session *local = callback_data->local;
-	const pjmedia_sdp_session *remote = callback_data->remote;
 	int i;
+	struct ast_stream_topology *topology;
+	unsigned int changed = 0;
+
+	if (!session->pending_media_state->topology) {
+		if (session->active_media_state->topology) {
+			/*
+			 * This happens when we have negotiated media after receiving a 183,
+			 * and we're now receiving a 200 with a new SDP.  In this case, there
+			 * is active_media_state, but the pending_media_state has been reset.
+			 */
+			struct ast_sip_session_media_state *active_media_state_clone;
+
+			active_media_state_clone =
+				ast_sip_session_media_state_clone(session->active_media_state);
+			if (!active_media_state_clone) {
+				ast_log(LOG_WARNING, "Unable to clone active media state for channel '%s'\n",
+					session->channel ? ast_channel_name(session->channel) : "unknown");
+				return -1;
+			}
+
+			ast_sip_session_media_state_free(session->pending_media_state);
+			session->pending_media_state = active_media_state_clone;
+		} else {
+			ast_log(LOG_WARNING, "No pending or active media state for channel '%s'\n",
+				session->channel ? ast_channel_name(session->channel) : "unknown");
+			return -1;
+		}
+	}
+
+	/* If we're handling negotiated streams, then we should already have set
+	 * up session media instances (and Asterisk streams) that correspond to
+	 * the local SDP, and there should be the same number of session medias
+	 * and streams as there are local SDP streams
+	 */
+	if (ast_stream_topology_get_count(session->pending_media_state->topology) != local->media_count
+		|| AST_VECTOR_SIZE(&session->pending_media_state->sessions) != local->media_count) {
+		ast_log(LOG_WARNING, "Local SDP for channel '%s' contains %d media streams while we expected it to contain %u\n",
+			session->channel ? ast_channel_name(session->channel) : "unknown",
+			ast_stream_topology_get_count(session->pending_media_state->topology), local->media_count);
+		return -1;
+	}
 
 	for (i = 0; i < local->media_count; ++i) {
-		/* See if there are registered handlers for this media stream type */
-		char media[20];
-		struct ast_sip_session_sdp_handler *handler;
-		RAII_VAR(struct sdp_handler_list *, handler_list, NULL, ao2_cleanup);
-		int res;
+		struct ast_sip_session_media *session_media;
+		struct ast_stream *stream;
 
 		if (!remote->media[i]) {
 			continue;
 		}
 
-		/* We need a null-terminated version of the media string */
-		ast_copy_pj_str(media, &local->media[i]->desc.media, sizeof(media));
+		session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, i);
+		stream = ast_stream_topology_get_stream(session->pending_media_state->topology, i);
 
-		/* stream type doesn't match the one we're looking to fill */
-		if (strcasecmp(session_media->stream_type, media)) {
+		/* Make sure that this stream is in the correct state. If we need to change
+		 * the state to REMOVED, then our work here is done, so go ahead and move on
+		 * to the next stream.
+		 */
+		if (!remote->media[i]->desc.port) {
+			ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
 			continue;
 		}
 
-		handler = session_media->handler;
-		if (handler) {
-			ast_debug(1, "Applying negotiated SDP media stream '%s' using %s SDP handler\n",
-				session_media->stream_type,
-				handler->id);
-			res = handler->apply_negotiated_sdp_stream(session, session_media, local,
-				local->media[i], remote, remote->media[i]);
-			if (res >= 0) {
-				ast_debug(1, "Applied negotiated SDP media stream '%s' using %s SDP handler\n",
-					session_media->stream_type,
-					handler->id);
-				return CMP_MATCH;
-			}
-			return 0;
+		/* If the stream state is REMOVED, nothing needs to be done, so move on to the
+		 * next stream. This can occur if an internal thing has requested it to be
+		 * removed, or if we remove it as a result of the stream limit being reached.
+		 */
+		if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+			/*
+			 * Defer removing the handler until we are ready to activate
+			 * the new topology.  The channel's thread may still be using
+			 * the stream and we could crash before we are ready.
+			 */
+			continue;
 		}
 
-		handler_list = ao2_find(sdp_handlers, media, OBJ_KEY);
-		if (!handler_list) {
-			ast_debug(1, "No registered SDP handlers for media type '%s'\n", media);
+		if (handle_negotiated_sdp_session_media(session_media, session, local, remote, i, stream)) {
+			return -1;
+		}
+
+		changed |= session_media->changed;
+		session_media->changed = 0;
+	}
+
+	/* Apply the pending media state to the channel and make it active */
+	ast_channel_lock(session->channel);
+
+	/* Now update the stream handler for any declined/removed streams */
+	for (i = 0; i < local->media_count; ++i) {
+		struct ast_sip_session_media *session_media;
+		struct ast_stream *stream;
+
+		if (!remote->media[i]) {
 			continue;
 		}
-		AST_LIST_TRAVERSE(&handler_list->list, handler, next) {
-			if (handler == session_media->handler) {
-				continue;
-			}
-			ast_debug(1, "Applying negotiated SDP media stream '%s' using %s SDP handler\n",
-				session_media->stream_type,
-				handler->id);
-			res = handler->apply_negotiated_sdp_stream(session, session_media, local,
-				local->media[i], remote, remote->media[i]);
-			if (res < 0) {
-				/* Catastrophic failure. Abort! */
-				return 0;
-			}
-			if (res > 0) {
-				ast_debug(1, "Applied negotiated SDP media stream '%s' using %s SDP handler\n",
-					session_media->stream_type,
-					handler->id);
-				/* Handled by this handler. Move to the next stream */
-				session_media_set_handler(session_media, handler);
-				return CMP_MATCH;
-			}
+
+		session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, i);
+		stream = ast_stream_topology_get_stream(session->pending_media_state->topology, i);
+
+		if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED
+			&& session_media->handler) {
+			/*
+			 * This stream is no longer being used and the channel's thread
+			 * is held off because we have the channel lock so release any
+			 * resources the handler may have on it.
+			 */
+			session_media_set_handler(session_media, NULL);
 		}
 	}
 
-	if (session_media->handler && session_media->handler->stream_stop) {
-		ast_debug(1, "Stopping SDP media stream '%s' as it is not currently negotiated\n",
-			session_media->stream_type);
-		session_media->handler->stream_stop(session_media);
+	/* Update the topology on the channel to match the accepted one */
+	topology = ast_stream_topology_clone(session->pending_media_state->topology);
+	if (topology) {
+		ast_channel_set_stream_topology(session->channel, topology);
 	}
 
-	return CMP_MATCH;
-}
+	/* Remove all current file descriptors from the channel */
+	for (i = 0; i < AST_VECTOR_SIZE(&session->active_media_state->read_callbacks); ++i) {
+		ast_channel_internal_fd_clear(session->channel, i + AST_EXTENDED_FDS);
+	}
 
-static int handle_negotiated_sdp(struct ast_sip_session *session, const pjmedia_sdp_session *local, const pjmedia_sdp_session *remote)
-{
-	RAII_VAR(struct ao2_iterator *, successful, NULL, ao2_iterator_cleanup);
-	struct handle_negotiated_sdp_cb callback_data = {
-		.session = session,
-		.local = local,
-		.remote = remote,
-	};
+	/* Add all the file descriptors from the pending media state */
+	for (i = 0; i < AST_VECTOR_SIZE(&session->pending_media_state->read_callbacks); ++i) {
+		struct ast_sip_session_media_read_callback_state *callback_state;
+
+		callback_state = AST_VECTOR_GET_ADDR(&session->pending_media_state->read_callbacks, i);
+		ast_channel_internal_fd_set(session->channel, i + AST_EXTENDED_FDS, callback_state->fd);
+	}
+
+	/* Active and pending flip flop as needed */
+	SWAP(session->active_media_state, session->pending_media_state);
+	ast_sip_session_media_state_reset(session->pending_media_state);
+
+	ast_channel_unlock(session->channel);
 
-	successful = ao2_callback(session->media, OBJ_MULTIPLE, handle_negotiated_sdp_session_media, &callback_data);
-	if (successful && ao2_iterator_count(successful) == ao2_container_count(session->media)) {
-		/* Nothing experienced a catastrophic failure */
+	if (changed) {
+		struct ast_frame f = { AST_FRAME_CONTROL, .subclass.integer = AST_CONTROL_STREAM_TOPOLOGY_SOURCE_CHANGED };
+
+		ast_queue_frame(session->channel, &f);
+	} else {
 		ast_queue_frame(session->channel, &ast_null_frame);
-		return 0;
 	}
-	return -1;
+
+	return 0;
 }
 
 #define DATASTORE_BUCKETS 53
@@ -517,6 +1144,8 @@ struct ast_sip_session_delayed_request {
 	ast_sip_session_response_cb on_response;
 	/*! Whether to generate new SDP */
 	int generate_new_sdp;
+	/*! Requested media state for the SDP */
+	struct ast_sip_session_media_state *media_state;
 	AST_LIST_ENTRY(ast_sip_session_delayed_request) next;
 };
 
@@ -525,7 +1154,8 @@ static struct ast_sip_session_delayed_re
 	ast_sip_session_request_creation_cb on_request_creation,
 	ast_sip_session_sdp_creation_cb on_sdp_creation,
 	ast_sip_session_response_cb on_response,
-	int generate_new_sdp)
+	int generate_new_sdp,
+	struct ast_sip_session_media_state *media_state)
 {
 	struct ast_sip_session_delayed_request *delay = ast_calloc(1, sizeof(*delay));
 
@@ -537,9 +1167,16 @@ static struct ast_sip_session_delayed_re
 	delay->on_sdp_creation = on_sdp_creation;
 	delay->on_response = on_response;
 	delay->generate_new_sdp = generate_new_sdp;
+	delay->media_state = media_state;
 	return delay;
 }
 
+static void delayed_request_free(struct ast_sip_session_delayed_request *delay)
+{
+	ast_sip_session_media_state_free(delay->media_state);
+	ast_free(delay);
+}
+
 static int send_delayed_request(struct ast_sip_session *session, struct ast_sip_session_delayed_request *delay)
 {
 	ast_debug(3, "Endpoint '%s(%s)' sending delayed %s request.\n",
@@ -551,12 +1188,16 @@ static int send_delayed_request(struct a
 	case DELAYED_METHOD_INVITE:
 		ast_sip_session_refresh(session, delay->on_request_creation,
 			delay->on_sdp_creation, delay->on_response,
-			AST_SIP_SESSION_REFRESH_METHOD_INVITE, delay->generate_new_sdp);
+			AST_SIP_SESSION_REFRESH_METHOD_INVITE, delay->generate_new_sdp, delay->media_state);
+		/* Ownership of media state transitions to ast_sip_session_refresh */
+		delay->media_state = NULL;
 		return 0;
 	case DELAYED_METHOD_UPDATE:
 		ast_sip_session_refresh(session, delay->on_request_creation,
 			delay->on_sdp_creation, delay->on_response,
-			AST_SIP_SESSION_REFRESH_METHOD_UPDATE, delay->generate_new_sdp);
+			AST_SIP_SESSION_REFRESH_METHOD_UPDATE, delay->generate_new_sdp, delay->media_state);
+		/* Ownership of media state transitions to ast_sip_session_refresh */
+		delay->media_state = NULL;
 		return 0;
 	case DELAYED_METHOD_BYE:
 		ast_sip_session_terminate(session, 0);
@@ -591,7 +1232,7 @@ static int invite_proceeding(void *vsess
 		case DELAYED_METHOD_UPDATE:
 			AST_LIST_REMOVE_CURRENT(next);
 			res = send_delayed_request(session, delay);
-			ast_free(delay);
+			delayed_request_free(delay);
 			found = 1;
 			break;
 		case DELAYED_METHOD_BYE:
@@ -645,7 +1286,7 @@ static int invite_terminated(void *vsess
 		if (found) {
 			AST_LIST_REMOVE_CURRENT(next);
 			res = send_delayed_request(session, delay);
-			ast_free(delay);
+			delayed_request_free(delay);
 			break;
 		}
 	}
@@ -722,12 +1363,14 @@ static int delay_request(struct ast_sip_
 	ast_sip_session_sdp_creation_cb on_sdp_creation,
 	ast_sip_session_response_cb on_response,
 	int generate_new_sdp,
-	enum delayed_method method)
+	enum delayed_method method,
+	struct ast_sip_session_media_state *media_state)
 {
 	struct ast_sip_session_delayed_request *delay = delayed_request_alloc(method,
-			on_request, on_sdp_creation, on_response, generate_new_sdp);
+			on_request, on_sdp_creation, on_response, generate_new_sdp, media_state);
 
 	if (!delay) {
+		ast_sip_session_media_state_free(media_state);
 		return -1;
 	}
 
@@ -761,8 +1404,10 @@ static void set_from_header(struct ast_s
 	struct ast_party_id connected_id;
 	pj_pool_t *dlg_pool;
 	pjsip_fromto_hdr *dlg_info;
+	pjsip_contact_hdr *dlg_contact;
 	pjsip_name_addr *dlg_info_name_addr;
 	pjsip_sip_uri *dlg_info_uri;
+	pjsip_sip_uri *dlg_contact_uri;
 	int restricted;
 
 	if (!session->channel || session->saved_from_hdr) {
@@ -783,11 +1428,16 @@ static void set_from_header(struct ast_s
 
 	dlg_pool = session->inv_session->dlg->pool;
 	dlg_info = session->inv_session->dlg->local.info;
+	dlg_contact = session->inv_session->dlg->local.contact;
 	dlg_info_name_addr = (pjsip_name_addr *) dlg_info->uri;
 	dlg_info_uri = pjsip_uri_get_uri(dlg_info_name_addr);
+	dlg_contact_uri = (pjsip_sip_uri*)pjsip_uri_get_uri(dlg_contact->uri);
 
 	if (session->endpoint->id.trust_outbound || !restricted) {
 		ast_sip_modify_id_header(dlg_pool, dlg_info, &connected_id);
+		if (ast_sip_get_use_callerid_contact() && ast_strlen_zero(session->endpoint->contact_user)) {
+			pj_strdup2(dlg_pool, &dlg_contact_uri->user, S_COR(connected_id.number.valid, connected_id.number.str, ""));
+		}
 	}
 
 	ast_party_id_free(&connected_id);
@@ -817,6 +1467,10 @@ static void set_from_header(struct ast_s
 			pj_strdup2(dlg_pool, &dlg_info_uri->user, "anonymous");
 		}
 
+		if (ast_sip_get_use_callerid_contact() && ast_strlen_zero(session->endpoint->contact_user)) {
+			pj_strdup2(dlg_pool, &dlg_contact_uri->user, "anonymous");
+		}
+
 		if (ast_strlen_zero(session->endpoint->fromdomain)) {
 			pj_strdup2(dlg_pool, &dlg_info_uri->host, "anonymous.invalid");
 		}
@@ -829,16 +1483,23 @@ int ast_sip_session_refresh(struct ast_s
 		ast_sip_session_request_creation_cb on_request_creation,
 		ast_sip_session_sdp_creation_cb on_sdp_creation,
 		ast_sip_session_response_cb on_response,
-		enum ast_sip_session_refresh_method method, int generate_new_sdp)
+		enum ast_sip_session_refresh_method method, int generate_new_sdp,
+		struct ast_sip_session_media_state *media_state)
 {
 	pjsip_inv_session *inv_session = session->inv_session;
 	pjmedia_sdp_session *new_sdp = NULL;
 	pjsip_tx_data *tdata;
 
+	if (media_state && (!media_state->topology || !generate_new_sdp)) {
+		ast_sip_session_media_state_free(media_state);
+		return -1;
+	}
+
 	if (inv_session->state == PJSIP_INV_STATE_DISCONNECTED) {
 		/* Don't try to do anything with a hung-up call */
 		ast_debug(3, "Not sending reinvite to %s because of disconnected state...\n",
 				ast_sorcery_object_get_id(session->endpoint));
+		ast_sip_session_media_state_free(media_state);
 		return 0;
 	}
 
@@ -849,7 +1510,8 @@ int ast_sip_session_refresh(struct ast_s
 		return delay_request(session, on_request_creation, on_sdp_creation, on_response,
 			generate_new_sdp,
 			method == AST_SIP_SESSION_REFRESH_METHOD_INVITE
-				? DELAYED_METHOD_INVITE : DELAYED_METHOD_UPDATE);
+				? DELAYED_METHOD_INVITE : DELAYED_METHOD_UPDATE,
+			media_state);
 	}
 
 	if (method == AST_SIP_SESSION_REFRESH_METHOD_INVITE) {
@@ -858,13 +1520,14 @@ int ast_sip_session_refresh(struct ast_s
 			ast_debug(3, "Delay sending reinvite to %s because of outstanding transaction...\n",
 					ast_sorcery_object_get_id(session->endpoint));
 			return delay_request(session, on_request_creation, on_sdp_creation,
-				on_response, generate_new_sdp, DELAYED_METHOD_INVITE);
+				on_response, generate_new_sdp, DELAYED_METHOD_INVITE, media_state);
 		} else if (inv_session->state != PJSIP_INV_STATE_CONFIRMED) {
 			/* Initial INVITE transaction failed to progress us to a confirmed state
 			 * which means re-invites are not possible
 			 */
 			ast_debug(3, "Not sending reinvite to %s because not in confirmed state...\n",
 					ast_sorcery_object_get_id(session->endpoint));
+			ast_sip_session_media_state_free(media_state);
 			return 0;
 		}
 	}
@@ -879,33 +1542,169 @@ int ast_sip_session_refresh(struct ast_s
 			return delay_request(session, on_request_creation, on_sdp_creation,
 				on_response, generate_new_sdp,
 				method == AST_SIP_SESSION_REFRESH_METHOD_INVITE
-					? DELAYED_METHOD_INVITE : DELAYED_METHOD_UPDATE);
+					? DELAYED_METHOD_INVITE : DELAYED_METHOD_UPDATE, media_state);
+		}
+
+		/* If an explicitly requested media state has been provided use it instead of any pending one */
+		if (media_state) {
+			int index;
+			int type_streams[AST_MEDIA_TYPE_END] = {0};
+			struct ast_stream *stream;
+
+			/* Prune the media state so the number of streams fit within the configured limits - we do it here
+			 * so that the index of the resulting streams in the SDP match. If we simply left the streams out
+			 * of the SDP when producing it we'd be in trouble. We also enforce formats here for media types that
+			 * are configurable on the endpoint.
+			 */
+			for (index = 0; index < ast_stream_topology_get_count(media_state->topology); ++index) {
+				struct ast_stream *existing_stream = NULL;
+
+				stream = ast_stream_topology_get_stream(media_state->topology, index);
+
+				if (session->active_media_state->topology &&
+					index < ast_stream_topology_get_count(session->active_media_state->topology)) {
+					existing_stream = ast_stream_topology_get_stream(session->active_media_state->topology, index);
+				}
+
+				if (is_stream_limitation_reached(ast_stream_get_type(stream), session->endpoint, type_streams)) {
+					if (index < AST_VECTOR_SIZE(&media_state->sessions)) {
+						struct ast_sip_session_media *session_media = AST_VECTOR_GET(&media_state->sessions, index);
+
+						ao2_cleanup(session_media);
+						AST_VECTOR_REMOVE(&media_state->sessions, index, 1);
+					}
+
+					ast_stream_topology_del_stream(media_state->topology, index);
+
+					/* A stream has potentially moved into our spot so we need to jump back so we process it */
+					index -= 1;
+					continue;
+				}
+
+				/* No need to do anything with stream if it's media state is removed */
+				if (ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+					/* If there is no existing stream we can just not have this stream in the topology at all. */
+					if (!existing_stream) {
+						ast_stream_topology_del_stream(media_state->topology, index);
+						index -= 1;
+					}
+					continue;
+				}
+
+				/* Enforce the configured allowed codecs on audio and video streams */
+				if (ast_stream_get_type(stream) == AST_MEDIA_TYPE_AUDIO || ast_stream_get_type(stream) == AST_MEDIA_TYPE_VIDEO) {
+					struct ast_format_cap *joint_cap;
+
+					joint_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+					if (!joint_cap) {
+						ast_sip_session_media_state_free(media_state);
+						return 0;
+					}
+					ast_format_cap_get_compatible(ast_stream_get_formats(stream), session->endpoint->media.codecs, joint_cap);
+					if (!ast_format_cap_count(joint_cap)) {
+						ao2_ref(joint_cap, -1);
+
+						if (!existing_stream) {
+							/* If there is no existing stream we can just not have this stream in the topology
+							 * at all.
+							 */
+							ast_stream_topology_del_stream(media_state->topology, index);
+							index -= 1;
+							continue;
+						} else if (ast_stream_get_state(stream) != ast_stream_get_state(existing_stream) ||
+								strcmp(ast_stream_get_name(stream), ast_stream_get_name(existing_stream))) {
+							/* If the underlying stream is a different type or different name then we have to
+							 * mark it as removed, as it is replacing an existing stream. We do this so order
+							 * is preserved.
+							 */
+							ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
+							continue;
+						} else {
+							/* However if the stream is otherwise remaining the same we can keep the formats
+							 * that exist on it already which allows media to continue to flow.
+							 */
+							joint_cap = ao2_bump(ast_stream_get_formats(existing_stream));
+						}
+					}
+					ast_stream_set_formats(stream, joint_cap);
+					ao2_cleanup(joint_cap);
+				}
+
+				++type_streams[ast_stream_get_type(stream)];
+			}
+
+			if (session->active_media_state->topology) {
+				/* SDP is a fun thing. Take for example the fact that streams are never removed. They just become
+				 * declined. To better handle this in the case where something requests a topology change for fewer
+				 * streams than are currently present we fill in the topology to match the current number of streams
+				 * that are active.
+				 */
+				for (index = ast_stream_topology_get_count(media_state->topology);
+					index < ast_stream_topology_get_count(session->active_media_state->topology); ++index) {
+					struct ast_stream *cloned;
+
+					stream = ast_stream_topology_get_stream(session->active_media_state->topology, index);
+					ast_assert(stream != NULL);
+
+					cloned = ast_stream_clone(stream, NULL);
+					if (!cloned) {
+						ast_sip_session_media_state_free(media_state);
+						return -1;
+					}
+
+					ast_stream_set_state(cloned, AST_STREAM_STATE_REMOVED);
+					if (ast_stream_topology_append_stream(media_state->topology, cloned) < 0) {
+						ast_stream_free(cloned);
+						ast_sip_session_media_state_free(media_state);
+						return -1;
+					}
+				}
+
+				/* If the resulting media state matches the existing active state don't bother doing a session refresh */
+				if (ast_stream_topology_equal(session->active_media_state->topology, media_state->topology)) {
+					ast_sip_session_media_state_free(media_state);
+					return 0;
+				}
+			}
+
+			ast_sip_session_media_state_free(session->pending_media_state);
+			session->pending_media_state = media_state;
 		}
 
 		new_sdp = generate_session_refresh_sdp(session);
 		if (!new_sdp) {
 			ast_log(LOG_ERROR, "Failed to generate session refresh SDP. Not sending session refresh\n");
+			ast_sip_session_media_state_reset(session->pending_media_state);
 			return -1;
 		}
 		if (on_sdp_creation) {
 			if (on_sdp_creation(session, new_sdp)) {
+				ast_sip_session_media_state_reset(session->pending_media_state);
 				return -1;
 			}
 		}
 	}
 
-
 	if (method == AST_SIP_SESSION_REFRESH_METHOD_INVITE) {
 		if (pjsip_inv_reinvite(inv_session, NULL, new_sdp, &tdata)) {
 			ast_log(LOG_WARNING, "Failed to create reinvite properly.\n");
+			if (generate_new_sdp) {
+				ast_sip_session_media_state_reset(session->pending_media_state);
+			}
 			return -1;
 		}
 	} else if (pjsip_inv_update(inv_session, NULL, new_sdp, &tdata)) {
 		ast_log(LOG_WARNING, "Failed to create UPDATE properly.\n");
+		if (generate_new_sdp) {
+			ast_sip_session_media_state_reset(session->pending_media_state);
+		}
 		return -1;
 	}
 	if (on_request_creation) {
 		if (on_request_creation(session, tdata)) {
+			if (generate_new_sdp) {
+				ast_sip_session_media_state_reset(session->pending_media_state);
+			}
 			return -1;
 		}
 	}
@@ -980,22 +1779,49 @@ static int sdp_requires_deferral(struct
 {
 	int i;
 
+	if (!session->pending_media_state->topology) {
+		session->pending_media_state->topology = ast_stream_topology_alloc();
+		if (!session->pending_media_state->topology) {
+			return -1;
+		}
+	}
+
 	for (i = 0; i < sdp->media_count; ++i) {
 		/* See if there are registered handlers for this media stream type */
 		char media[20];
 		struct ast_sip_session_sdp_handler *handler;
 		RAII_VAR(struct sdp_handler_list *, handler_list, NULL, ao2_cleanup);
-		RAII_VAR(struct ast_sip_session_media *, session_media, NULL, ao2_cleanup);
+		struct ast_stream *existing_stream = NULL;
+		struct ast_stream *stream;
+		enum ast_media_type type;
+		struct ast_sip_session_media *session_media = NULL;
 		enum ast_sip_session_sdp_stream_defer res;
 
 		/* We need a null-terminated version of the media string */
 		ast_copy_pj_str(media, &sdp->media[i]->desc.media, sizeof(media));
 
-		session_media = ao2_find(session->media, media, OBJ_KEY);
+		if (session->active_media_state->topology &&
+			(i < ast_stream_topology_get_count(session->active_media_state->topology))) {
+			existing_stream = ast_stream_topology_get_stream(session->active_media_state->topology, i);
+		}
+
+		type = ast_media_type_from_str(media);
+		stream = ast_stream_alloc(existing_stream ? ast_stream_get_name(existing_stream) : ast_codec_media_type2str(type), type);
+		if (!stream) {
+			return -1;
+		}
+
+		/* As this is only called on an incoming SDP offer before processing it is not possible
+		 * for streams and their media sessions to exist.
+		 */
+		if (ast_stream_topology_set_stream(session->pending_media_state->topology, i, stream)) {
+			ast_stream_free(stream);
+			return -1;
+		}
+
+		session_media = ast_sip_session_media_state_add(session, session->pending_media_state, ast_media_type_from_str(media), i);
 		if (!session_media) {
-			/* if the session_media doesn't exist, there weren't
-			 * any handlers at the time of its creation */
-			continue;
+			return -1;
 		}
 
 		if (session_media->handler) {
@@ -1219,7 +2045,9 @@ int ast_sip_session_create_invite(struct
 	pjsip_inv_set_local_sdp(session->inv_session, offer);
 	pjmedia_sdp_neg_set_prefer_remote_codec_order(session->inv_session->neg, PJ_FALSE);
 #ifdef PJMEDIA_SDP_NEG_ANSWER_MULTIPLE_CODECS
-	pjmedia_sdp_neg_set_answer_multiple_codecs(session->inv_session->neg, PJ_TRUE);
+	if (!session->endpoint->preferred_codec_only) {
+		pjmedia_sdp_neg_set_answer_multiple_codecs(session->inv_session->neg, PJ_TRUE);
+	}
 #endif
 
 	/*
@@ -1257,29 +2085,6 @@ static int datastore_cmp(void *obj, void
 	return strcmp(datastore1->uid, uid2) ? 0 : CMP_MATCH | CMP_STOP;
 }
 
-static void session_media_dtor(void *obj)
-{
-	struct ast_sip_session_media *session_media = obj;
-	struct sdp_handler_list *handler_list;
-	/* It is possible for SDP handlers to allocate memory on a session_media but
-	 * not end up getting set as the handler for this session_media. This traversal
-	 * ensures that all memory allocated by SDP handlers on the session_media is
-	 * cleared (as well as file descriptors, etc.).
-	 */
-	handler_list = ao2_find(sdp_handlers, session_media->stream_type, OBJ_KEY);
-	if (handler_list) {
-		struct ast_sip_session_sdp_handler *handler;
-
-		AST_LIST_TRAVERSE(&handler_list->list, handler, next) {
-			handler->stream_destroy(session_media);
-		}
-	}
-	ao2_cleanup(handler_list);
-	if (session_media->srtp) {
-		ast_sdp_srtp_destroy(session_media->srtp);
-	}
-}
-
 static void session_destructor(void *obj)
 {
 	struct ast_sip_session *session = obj;
@@ -1308,17 +2113,17 @@ static void session_destructor(void *obj
 
 	ast_taskprocessor_unreference(session->serializer);
 	ao2_cleanup(session->datastores);
-	ao2_cleanup(session->media);
+	ast_sip_session_media_state_free(session->active_media_state);
+	ast_sip_session_media_state_free(session->pending_media_state);
 
 	AST_LIST_HEAD_DESTROY(&session->supplements);
 	while ((delay = AST_LIST_REMOVE_HEAD(&session->delayed_requests, next))) {
-		ast_free(delay);
+		delayed_request_free(delay);
 	}
 	ast_party_id_free(&session->id);
 	ao2_cleanup(session->endpoint);
 	ao2_cleanup(session->aor);
 	ao2_cleanup(session->contact);
-	ao2_cleanup(session->req_caps);
 	ao2_cleanup(session->direct_media_cap);
 
 	ast_dsp_free(session->dsp);
@@ -1330,27 +2135,6 @@ static void session_destructor(void *obj
 	ast_test_suite_event_notify("SESSION_DESTROYED", "Endpoint: %s", endpoint_name);
 }
 
-static int add_session_media(void *obj, void *arg, int flags)
-{
-	struct sdp_handler_list *handler_list = obj;
-	struct ast_sip_session *session = arg;
-	RAII_VAR(struct ast_sip_session_media *, session_media, NULL, ao2_cleanup);
-
-	session_media = ao2_alloc(sizeof(*session_media) + strlen(handler_list->stream_type), session_media_dtor);
-	if (!session_media) {
-		return CMP_STOP;
-	}
-	session_media->encryption = session->endpoint->media.rtp.encryption;
-	session_media->remote_ice = session->endpoint->media.rtp.ice_support;
-	session_media->remote_rtcp_mux = session->endpoint->rtcp_mux;
-	session_media->keepalive_sched_id = -1;
-	session_media->timeout_sched_id = -1;
-	/* Safe use of strcpy */
-	strcpy(session_media->stream_type, handler_list->stream_type);
-	ao2_link(session->media, session_media);
-	return 0;
-}
-
 /*! \brief Destructor for SIP channel */
 static void sip_channel_destroy(void *obj)
 {
@@ -1397,12 +2181,17 @@ struct ast_sip_session *ast_sip_session_
 	if (!session->direct_media_cap) {
 		return NULL;
 	}
-	session->req_caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-	if (!session->req_caps) {
+	session->datastores = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DATASTORE_BUCKETS, datastore_hash, NULL, datastore_cmp);
+	if (!session->datastores) {
 		return NULL;
 	}
-	session->datastores = ao2_container_alloc(DATASTORE_BUCKETS, datastore_hash, datastore_cmp);
-	if (!session->datastores) {
+	session->active_media_state = ast_sip_session_media_state_alloc();
+	if (!session->active_media_state) {
+		return NULL;
+	}
+	session->pending_media_state = ast_sip_session_media_state_alloc();
+	if (!session->pending_media_state) {
 		return NULL;
 	}
 
@@ -1423,13 +2212,6 @@ struct ast_sip_session *ast_sip_session_
 
 	session->endpoint = ao2_bump(endpoint);
 
-	session->media = ao2_container_alloc(MEDIA_BUCKETS, session_media_hash, session_media_cmp);
-	if (!session->media) {
-		return NULL;
-	}
-	/* fill session->media with available types */
-	ao2_callback(sdp_handlers, OBJ_NODATA, add_session_media, session);
-
 	if (rdata) {
 		/*
 		 * We must continue using the serializer that the original
@@ -1446,7 +2228,7 @@ struct ast_sip_session *ast_sip_session_
 		ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/outsess/%s",
 			ast_sorcery_object_get_id(endpoint));
 
-		session->serializer = ast_sip_create_serializer_named(tps_name);
+		session->serializer = ast_sip_create_serializer(tps_name);
 	}
 	if (!session->serializer) {
 		return NULL;
@@ -1637,8 +2419,8 @@ static pj_bool_t outbound_invite_auth(pj
 
 	session = inv->mod_data[session_module.id];
 
-	if (ast_sip_create_request_with_auth(&session->endpoint->outbound_auths, rdata, tsx,
-		&tdata)) {
+	if (ast_sip_create_request_with_auth(&session->endpoint->outbound_auths, rdata,
+		tsx->last_tx, &tdata)) {
 		return PJ_FALSE;
 	}
 
@@ -1681,7 +2463,7 @@ static int setup_outbound_invite_auth(pj
 
 struct ast_sip_session *ast_sip_session_create_outgoing(struct ast_sip_endpoint *endpoint,
 	struct ast_sip_contact *contact, const char *location, const char *request_user,
-	struct ast_format_cap *req_caps)
+	struct ast_stream_topology *req_topology)
 {
 	const char *uri = NULL;
 	RAII_VAR(struct ast_sip_aor *, found_aor, NULL, ao2_cleanup);
@@ -1745,22 +2527,78 @@ struct ast_sip_session *ast_sip_session_
 	session->aor = ao2_bump(found_aor);
 	ast_party_id_copy(&session->id, &endpoint->id.self);
 
-	if (ast_format_cap_count(req_caps)) {
-		/* get joint caps between req_caps and endpoint caps */
-		struct ast_format_cap *joint_caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
-
-		ast_format_cap_get_compatible(req_caps, endpoint->media.codecs, joint_caps);
-
-		/* if joint caps */
-		if (ast_format_cap_count(joint_caps)) {
-			/* copy endpoint caps into session->req_caps */
-			ast_format_cap_append_from_cap(session->req_caps,
-				endpoint->media.codecs, AST_MEDIA_TYPE_UNKNOWN);
-			/* replace instances of joint caps equivalents in session->req_caps */
-			ast_format_cap_replace_from_cap(session->req_caps, joint_caps,
-				AST_MEDIA_TYPE_UNKNOWN);
+	if (ast_stream_topology_get_count(req_topology) > 0) {
+		/* get joint caps between req_topology and endpoint topology */
+		int i;
+
+		for (i = 0; i < ast_stream_topology_get_count(req_topology); ++i) {
+			struct ast_stream *req_stream;
+			struct ast_format_cap *req_cap;
+			struct ast_format_cap *joint_cap;
+			struct ast_stream *clone_stream;
+
+			req_stream = ast_stream_topology_get_stream(req_topology, i);
+
+			if (ast_stream_get_state(req_stream) == AST_STREAM_STATE_REMOVED) {
+				continue;
+			}
+
+			req_cap = ast_stream_get_formats(req_stream);
+
+			joint_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+			if (!joint_cap) {
+				continue;
+			}
+
+			ast_format_cap_get_compatible(req_cap, endpoint->media.codecs, joint_cap);
+			if (!ast_format_cap_count(joint_cap)) {
+				ao2_ref(joint_cap, -1);
+				continue;
+			}
+
+			clone_stream = ast_stream_clone(req_stream, NULL);
+			if (!clone_stream) {
+				ao2_ref(joint_cap, -1);
+				continue;
+			}
+
+			if (ast_stream_get_type(req_stream) == AST_MEDIA_TYPE_AUDIO) {
+				/*
+				 * By appending codecs from the endpoint after compatible ones this
+				 * guarantees that priority is given to those while also allowing
+				 * translation to occur for non-compatible.
+				 */
+				ast_format_cap_append_from_cap(joint_cap,
+					endpoint->media.codecs, AST_MEDIA_TYPE_AUDIO);
+			}
+
+			ast_stream_set_formats(clone_stream, joint_cap);
+			ao2_ref(joint_cap, -1);
+
+			if (!session->pending_media_state->topology) {
+				session->pending_media_state->topology = ast_stream_topology_alloc();
+				if (!session->pending_media_state->topology) {
+					pjsip_inv_terminate(inv_session, 500, PJ_FALSE);
+					ao2_ref(session, -1);
+					return NULL;
+				}
+			}
+
+			if (ast_stream_topology_append_stream(session->pending_media_state->topology, clone_stream) < 0) {
+				ast_stream_free(clone_stream);
+				continue;
+			}
+		}
+	}
+
+	if (!session->pending_media_state->topology) {
+		/* Use the configured topology on the endpoint as the pending one */
+		session->pending_media_state->topology = ast_stream_topology_clone(endpoint->media.topology);
+		if (!session->pending_media_state->topology) {
+			pjsip_inv_terminate(inv_session, 500, PJ_FALSE);
+			ao2_ref(session, -1);
+			return NULL;
 		}
-		ao2_cleanup(joint_caps);
 	}
 
 	if (pjsip_dlg_add_usage(dlg, &session_module, NULL) != PJ_SUCCESS) {
@@ -1795,6 +2633,15 @@ void ast_sip_session_terminate(struct as
 		response = 603;
 	}
 
+	/* The media sessions need to exist for the lifetime of the underlying channel
+	 * to ensure that anything (such as bridge_native_rtp) has access to them as
+	 * appropriate. Since ast_sip_session_terminate is called by chan_pjsip and other
+	 * places when the session is to be terminated we terminate any existing
+	 * media sessions here.
+	 */
+	SWAP(session->active_media_state, session->pending_media_state);
+	ast_sip_session_media_state_reset(session->pending_media_state);
+
 	switch (session->inv_session->state) {
 	case PJSIP_INV_STATE_NULL:
 		if (!session->inv_session->invite_tsx) {
@@ -1824,7 +2671,7 @@ void ast_sip_session_terminate(struct as
 			/* If this is delayed the only thing that will happen is a BYE request so we don't
 			 * actually need to store the response code for when it happens.
 			 */
-			delay_request(session, NULL, NULL, NULL, 0, DELAYED_METHOD_BYE);
+			delay_request(session, NULL, NULL, NULL, 0, DELAYED_METHOD_BYE, NULL);
 			break;
 		}
 		/* Fall through */
@@ -1835,7 +2682,7 @@ void ast_sip_session_terminate(struct as
 
 			/* Flush any delayed requests so they cannot overlap this transaction. */
 			while ((delay = AST_LIST_REMOVE_HEAD(&session->delayed_requests, next))) {
-				ast_free(delay);
+				delayed_request_free(delay);
 			}
 
 			if (packet->msg->type == PJSIP_RESPONSE_MSG) {
@@ -2101,6 +2948,7 @@ static int new_invite(struct new_invite
 	pjsip_timer_setting timer;
 	pjsip_rdata_sdp_info *sdp_info;
 	pjmedia_sdp_session *local = NULL;
+	char buffer[AST_SOCKADDR_BUFLEN];
 
 	/* From this point on, any calls to pjsip_inv_terminate have the last argument as PJ_TRUE
 	 * so that we will be notified so we can destroy the session properly
@@ -2128,8 +2976,11 @@ static int new_invite(struct new_invite
 		}
 		goto end;
 	case SIP_GET_DEST_EXTEN_PARTIAL:
-		ast_debug(1, "Call from '%s' (%s:%s:%d) to extension '%s' - partial match\n", ast_sorcery_object_get_id(invite->session->endpoint),
-			invite->rdata->tp_info.transport->type_name, invite->rdata->pkt_info.src_name, invite->rdata->pkt_info.src_port, invite->session->exten);
+		ast_debug(1, "Call from '%s' (%s:%s) to extension '%s' - partial match\n",
+			ast_sorcery_object_get_id(invite->session->endpoint),
+			invite->rdata->tp_info.transport->type_name,
+			pj_sockaddr_print(&invite->rdata->pkt_info.src_addr, buffer, sizeof(buffer), 3),
+			invite->session->exten);
 
 		if (pjsip_inv_initial_answer(invite->session->inv_session, invite->rdata, 484, NULL, NULL, &tdata) == PJ_SUCCESS) {
 			ast_sip_session_send_response(invite->session, tdata);
@@ -2139,9 +2990,12 @@ static int new_invite(struct new_invite
 		goto end;
 	case SIP_GET_DEST_EXTEN_NOT_FOUND:
 	default:
-		ast_log(LOG_NOTICE, "Call from '%s' (%s:%s:%d) to extension '%s' rejected because extension not found in context '%s'.\n",
-			ast_sorcery_object_get_id(invite->session->endpoint), invite->rdata->tp_info.transport->type_name, invite->rdata->pkt_info.src_name,
-			invite->rdata->pkt_info.src_port, invite->session->exten, invite->session->endpoint->context);
+		ast_log(LOG_NOTICE, "Call from '%s' (%s:%s) to extension '%s' rejected because extension not found in context '%s'.\n",
+			ast_sorcery_object_get_id(invite->session->endpoint),
+			invite->rdata->tp_info.transport->type_name,
+			pj_sockaddr_print(&invite->rdata->pkt_info.src_addr, buffer, sizeof(buffer), 3),
+			invite->session->exten,
+			invite->session->endpoint->context);
 
 		if (pjsip_inv_initial_answer(invite->session->inv_session, invite->rdata, 404, NULL, NULL, &tdata) == PJ_SUCCESS) {
 			ast_sip_session_send_response(invite->session, tdata);
@@ -2197,7 +3051,9 @@ static int new_invite(struct new_invite
 	pjsip_inv_set_local_sdp(invite->session->inv_session, local);
 	pjmedia_sdp_neg_set_prefer_remote_codec_order(invite->session->inv_session->neg, PJ_FALSE);
 #ifdef PJMEDIA_SDP_NEG_ANSWER_MULTIPLE_CODECS
-	pjmedia_sdp_neg_set_answer_multiple_codecs(invite->session->inv_session->neg, PJ_TRUE);
+	if (!invite->session->endpoint->preferred_codec_only) {
+		pjmedia_sdp_neg_set_answer_multiple_codecs(invite->session->inv_session->neg, PJ_TRUE);
+	}
 #endif
 
 	handle_incoming_request(invite->session, invite->rdata);
@@ -2368,7 +3224,7 @@ static void reschedule_reinvite(struct a
 	ast_debug(3, "Endpoint '%s(%s)' re-INVITE collision.\n",
 		ast_sorcery_object_get_id(session->endpoint),
 		session->channel ? ast_channel_name(session->channel) : "");
-	if (delay_request(session, NULL, NULL, on_response, 1, DELAYED_METHOD_INVITE)) {
+	if (delay_request(session, NULL, NULL, on_response, 1, DELAYED_METHOD_INVITE, NULL)) {
 		return;
 	}
 	if (pj_timer_entry_running(&session->rescheduled_reinvite)) {
@@ -2543,11 +3399,6 @@ static int session_end(void *vsession)
 			iter->session_end(session);
 		}
 	}
-
-	/* Release any media resources. */
-	ao2_cleanup(session->media);
-	session->media = NULL;
-
 	return 0;
 }
 
@@ -2573,6 +3424,29 @@ static int session_end_completion(void *
 	return 0;
 }
 
+static int check_request_status(pjsip_inv_session *inv, pjsip_event *e)
+{
+	struct ast_sip_session *session = inv->mod_data[session_module.id];
+	pjsip_transaction *tsx = e->body.tsx_state.tsx;
+
+	if (tsx->status_code != 503 && tsx->status_code != 408) {
+		return 0;
+	}
+
+	if (!ast_sip_failover_request(tsx->last_tx)) {
+		return 0;
+	}
+
+	pjsip_inv_uac_restart(inv, PJ_FALSE);
+	/*
+	 * Bump the ref since it will be on a new transaction and
+	 * we don't want it to go away along with the old transaction.
+	 */
+	pjsip_tx_data_add_ref(tsx->last_tx);
+	ast_sip_session_send_request(session, tsx->last_tx);
+	return 1;
+}
+
 static void handle_incoming_before_media(pjsip_inv_session *inv,
 	struct ast_sip_session *session, pjsip_rx_data *rdata)
 {
@@ -2637,10 +3511,19 @@ static void session_inv_on_state_changed
 		case PJSIP_EVENT_TX_MSG:
 			break;
 		case PJSIP_EVENT_RX_MSG:
-			handle_incoming_before_media(inv, session, e->body.tsx_state.src.rdata);
+			if (!check_request_status(inv, e)) {
+				handle_incoming_before_media(inv, session, e->body.tsx_state.src.rdata);
+			}
 			break;
 		case PJSIP_EVENT_TRANSPORT_ERROR:
 		case PJSIP_EVENT_TIMER:
+			/*
+			 * Check the request status on transport error or timeout. A transport
+			 * error can occur when a TCP socket closes and that can be the result
+			 * of a 503. Also we may need to failover on a timeout (408).
+			 */
+			check_request_status(inv, e);
+			break;
 		case PJSIP_EVENT_USER:
 		case PJSIP_EVENT_UNKNOWN:
 		case PJSIP_EVENT_TSX_STATE:
@@ -2769,13 +3652,13 @@ static void session_inv_on_tsx_state_cha
 						if ((tsx->status_code == 401 || tsx->status_code == 407)
 							&& !ast_sip_create_request_with_auth(
 								&session->endpoint->outbound_auths,
-								e->body.tsx_state.src.rdata, tsx, &tdata)) {
+								e->body.tsx_state.src.rdata, tsx->last_tx, &tdata)) {
 							/* Send authed reINVITE */
 							ast_sip_session_send_request_with_cb(session, tdata, cb);
 							return;
 						}
-						if (tsx->status_code != 488) {
-							/* Other reinvite failures (except 488) result in destroying the session. */
+						if (tsx->status_code != 488 && tsx->status_code != 500) {
+							/* Other reinvite failures (except 488 and 500) result in destroying the session. */
 							if (pjsip_inv_end_session(inv, 500, NULL, &tdata) == PJ_SUCCESS
 								&& tdata) {
 								ast_sip_session_send_request(session, tdata);
@@ -2843,7 +3726,7 @@ static void session_inv_on_tsx_state_cha
 					if ((tsx->status_code == 401 || tsx->status_code == 407)
 						&& !ast_sip_create_request_with_auth(
 							&session->endpoint->outbound_auths,
-							e->body.tsx_state.src.rdata, tsx, &tdata)) {
+							e->body.tsx_state.src.rdata, tsx->last_tx, &tdata)) {
 						/* Send authed version of the method */
 						ast_sip_session_send_request_with_cb(session, tdata, cb);
 						return;
@@ -2907,27 +3790,27 @@ static void session_inv_on_tsx_state_cha
 	}
 }
 
-static int add_sdp_streams(void *obj, void *arg, void *data, int flags)
+static int add_sdp_streams(struct ast_sip_session_media *session_media,
+	struct ast_sip_session *session, pjmedia_sdp_session *answer,
+	const struct pjmedia_sdp_session *remote,
+	struct ast_stream *stream)
 {
-	struct ast_sip_session_media *session_media = obj;
-	pjmedia_sdp_session *answer = arg;
-	struct ast_sip_session *session = data;
 	struct ast_sip_session_sdp_handler *handler = session_media->handler;
 	RAII_VAR(struct sdp_handler_list *, handler_list, NULL, ao2_cleanup);
 	int res;
 
 	if (handler) {
 		/* if an already assigned handler reports a catastrophic error, fail */
-		res = handler->create_outgoing_sdp_stream(session, session_media, answer);
+		res = handler->create_outgoing_sdp_stream(session, session_media, answer, remote, stream);
 		if (res < 0) {
-			return 0;
+			return -1;
 		}
-		return CMP_MATCH;
+		return 0;
 	}
 
-	handler_list = ao2_find(sdp_handlers, session_media->stream_type, OBJ_KEY);
+	handler_list = ao2_find(sdp_handlers, ast_codec_media_type2str(session_media->type), OBJ_KEY);
 	if (!handler_list) {
-		return CMP_MATCH;
+		return 0;
 	}
 
 	/* no handler for this stream type and we have a list to search */
@@ -2935,29 +3818,108 @@ static int add_sdp_streams(void *obj, vo
 		if (handler == session_media->handler) {
 			continue;
 		}
-		res = handler->create_outgoing_sdp_stream(session, session_media, answer);
+		res = handler->create_outgoing_sdp_stream(session, session_media, answer, remote, stream);
 		if (res < 0) {
 			/* catastrophic error */
-			return 0;
+			return -1;
 		}
 		if (res > 0) {
 			/* Handled by this handler. Move to the next stream */
 			session_media_set_handler(session_media, handler);
-			return CMP_MATCH;
+			return 0;
 		}
 	}
 
 	/* streams that weren't handled won't be included in generated outbound SDP */
-	return CMP_MATCH;
+	return 0;
+}
+
+/*! \brief Bundle group building structure */
+struct sip_session_media_bundle_group {
+	/*! \brief The media identifiers in this bundle group */
+	char *mids[PJMEDIA_MAX_SDP_MEDIA];
+	/*! \brief SDP attribute string */
+	struct ast_str *attr_string;
+};
+
+static int add_bundle_groups(struct ast_sip_session *session, pj_pool_t *pool, pjmedia_sdp_session *answer)
+{
+	pj_str_t stmp;
+	pjmedia_sdp_attr *attr;
+	struct sip_session_media_bundle_group bundle_groups[PJMEDIA_MAX_SDP_MEDIA];
+	int index, mid_id;
+	struct sip_session_media_bundle_group *bundle_group;
+
+	if (session->endpoint->media.webrtc) {
+		attr = pjmedia_sdp_attr_create(pool, "msid-semantic", pj_cstr(&stmp, "WMS *"));
+		pjmedia_sdp_attr_add(&answer->attr_count, answer->attr, attr);
+	}
+
+	if (!session->endpoint->media.bundle) {
+		return 0;
+	}
+
+	memset(bundle_groups, 0, sizeof(bundle_groups));
+
+	/* Build the bundle group layout so we can then add it to the SDP */
+	for (index = 0; index < AST_VECTOR_SIZE(&session->pending_media_state->sessions); ++index) {
+		struct ast_sip_session_media *session_media = AST_VECTOR_GET(&session->pending_media_state->sessions, index);
+
+		/* If this stream is not part of a bundle group we can't add it */
+		if (session_media->bundle_group == -1) {
+			continue;
+		}
+
+		bundle_group = &bundle_groups[session_media->bundle_group];
+
+		/* If this is the first mid then we need to allocate the attribute string and place BUNDLE in front */
+		if (!bundle_group->mids[0]) {
+			bundle_group->mids[0] = session_media->mid;
+			bundle_group->attr_string = ast_str_create(64);
+			if (!bundle_group->attr_string) {
+				continue;
+			}
+
+			ast_str_set(&bundle_group->attr_string, 0, "BUNDLE %s", session_media->mid);
+			continue;
+		}
+
+		for (mid_id = 1; mid_id < PJMEDIA_MAX_SDP_MEDIA; ++mid_id) {
+			if (!bundle_group->mids[mid_id]) {
+				bundle_group->mids[mid_id] = session_media->mid;
+				ast_str_append(&bundle_group->attr_string, 0, " %s", session_media->mid);
+				break;
+			} else if (!strcmp(bundle_group->mids[mid_id], session_media->mid)) {
+				break;
+			}
+		}
+	}
+
+	/* Add all bundle groups that have mids to the SDP */
+	for (index = 0; index < PJMEDIA_MAX_SDP_MEDIA; ++index) {
+		bundle_group = &bundle_groups[index];
+
+		if (!bundle_group->attr_string) {
+			continue;
+		}
+
+		attr = pjmedia_sdp_attr_create(pool, "group", pj_cstr(&stmp, ast_str_buffer(bundle_group->attr_string)));
+		pjmedia_sdp_attr_add(&answer->attr_count, answer->attr, attr);
+
+		ast_free(bundle_group->attr_string);
+	}
+
+	return 0;
 }
 
 static struct pjmedia_sdp_session *create_local_sdp(pjsip_inv_session *inv, struct ast_sip_session *session, const pjmedia_sdp_session *offer)
 {
-	RAII_VAR(struct ao2_iterator *, successful, NULL, ao2_iterator_cleanup);
 	static const pj_str_t STR_IN = { "IN", 2 };
 	static const pj_str_t STR_IP4 = { "IP4", 3 };
 	static const pj_str_t STR_IP6 = { "IP6", 3 };
 	pjmedia_sdp_session *local;
+	int i;
+	int stream;
 
 	if (inv->state == PJSIP_INV_STATE_DISCONNECTED) {
 		ast_log(LOG_ERROR, "Failed to create session SDP. Session has been already disconnected\n");
@@ -2978,47 +3940,105 @@ static struct pjmedia_sdp_session *creat
 	pj_strdup2(inv->pool_prov, &local->origin.user, session->endpoint->media.sdpowner);
 	pj_strdup2(inv->pool_prov, &local->name, session->endpoint->media.sdpsession);
 
-	/* Now let the handlers add streams of various types, pjmedia will automatically reorder the media streams for us */
-	successful = ao2_callback_data(session->media, OBJ_MULTIPLE, add_sdp_streams, local, session);
-	if (!successful || ao2_iterator_count(successful) != ao2_container_count(session->media)) {
-		/* Something experienced a catastrophic failure */
-		return NULL;
+	if (!session->pending_media_state->topology || !ast_stream_topology_get_count(session->pending_media_state->topology)) {
+		/* We've encountered a situation where we have been told to create a local SDP but noone has given us any indication
+		 * of what kind of stream topology they would like. We try to not alter the current state of the SDP negotiation
+		 * by using what is currently negotiated. If this is unavailable we fall back to what is configured on the endpoint.
+		 */
+		ast_stream_topology_free(session->pending_media_state->topology);
+		if (session->active_media_state->topology) {
+			session->pending_media_state->topology = ast_stream_topology_clone(session->active_media_state->topology);
+		} else {
+			session->pending_media_state->topology = ast_stream_topology_clone(session->endpoint->media.topology);
+		}
+		if (!session->pending_media_state->topology) {
+			return NULL;
+		}
 	}
 
-	/* Use the connection details of the first media stream if possible for SDP level */
-	if (local->media_count) {
-		int stream;
+	for (i = 0; i < ast_stream_topology_get_count(session->pending_media_state->topology); ++i) {
+		struct ast_sip_session_media *session_media;
+		struct ast_stream *stream;
+		unsigned int streams = local->media_count;
 
-		/* Since we are using the first media stream as the SDP level we can get rid of it
-		 * from the stream itself
+		/* This code does not enforce any maximum stream count limitations as that is done on either
+		 * the handling of an incoming SDP offer or on the handling of a session refresh.
 		 */
-		local->conn = local->media[0]->conn;
-		local->media[0]->conn = NULL;
-		pj_strassign(&local->origin.net_type, &local->conn->net_type);
-		pj_strassign(&local->origin.addr_type, &local->conn->addr_type);
-		pj_strassign(&local->origin.addr, &local->conn->addr);
 
-		/* Go through each media stream seeing if the connection details actually differ,
-		 * if not just use SDP level and reduce the SDP size
-		 */
-		for (stream = 1; stream < local->media_count; stream++) {
+		stream = ast_stream_topology_get_stream(session->pending_media_state->topology, i);
+
+		session_media = ast_sip_session_media_state_add(session, session->pending_media_state, ast_stream_get_type(stream), i);
+		if (!session_media) {
+			return NULL;
+		}
+
+		if (add_sdp_streams(session_media, session, local, offer, stream)) {
+			return NULL;
+		}
+
+		/* If a stream was actually added then add any additional details */
+		if (streams != local->media_count) {
+			pjmedia_sdp_media *media = local->media[streams];
+			pj_str_t stmp;
+			pjmedia_sdp_attr *attr;
+
+			/* Add the media identifier if present */
+			if (!ast_strlen_zero(session_media->mid)) {
+				attr = pjmedia_sdp_attr_create(inv->pool_prov, "mid", pj_cstr(&stmp, session_media->mid));
+				pjmedia_sdp_attr_add(&media->attr_count, media->attr, attr);
+			}
+		}
+
+		/* Ensure that we never exceed the maximum number of streams PJMEDIA will allow. */
+		if (local->media_count == PJMEDIA_MAX_SDP_MEDIA) {
+			break;
+		}
+	}
+
+	/* Add any bundle groups that are present on the media state */
+	if (add_bundle_groups(session, inv->pool_prov, local)) {
+		return NULL;
+	}
+
+	/* Use the connection details of an available media if possible for SDP level */
+	for (stream = 0; stream < local->media_count; stream++) {
+		if (!local->media[stream]->conn) {
+			continue;
+		}
+
+		if (local->conn) {
 			if (!pj_strcmp(&local->conn->net_type, &local->media[stream]->conn->net_type) &&
 				!pj_strcmp(&local->conn->addr_type, &local->media[stream]->conn->addr_type) &&
 				!pj_strcmp(&local->conn->addr, &local->media[stream]->conn->addr)) {
 				local->media[stream]->conn = NULL;
 			}
+			continue;
 		}
-	} else {
-		local->origin.net_type = STR_IN;
-		local->origin.addr_type = session->endpoint->media.rtp.ipv6 ? STR_IP6 : STR_IP4;
+
+		/* This stream's connection info will serve as the connection details for SDP level */
+		local->conn = local->media[stream]->conn;
+		local->media[stream]->conn = NULL;
+
+		continue;
+	}
+
+	/* If no SDP level connection details are present then create some */
+	if (!local->conn) {
+		local->conn = pj_pool_zalloc(inv->pool_prov, sizeof(struct pjmedia_sdp_conn));
+		local->conn->net_type = STR_IN;
+		local->conn->addr_type = session->endpoint->media.rtp.ipv6 ? STR_IP6 : STR_IP4;
 
 		if (!ast_strlen_zero(session->endpoint->media.address)) {
-			pj_strdup2(inv->pool_prov, &local->origin.addr, session->endpoint->media.address);
+			pj_strdup2(inv->pool_prov, &local->conn->addr, session->endpoint->media.address);
 		} else {
-			pj_strdup2(inv->pool_prov, &local->origin.addr, ast_sip_get_host_ip_string(session->endpoint->media.rtp.ipv6 ? pj_AF_INET6() : pj_AF_INET()));
+			pj_strdup2(inv->pool_prov, &local->conn->addr, ast_sip_get_host_ip_string(session->endpoint->media.rtp.ipv6 ? pj_AF_INET6() : pj_AF_INET()));
 		}
 	}
 
+	pj_strassign(&local->origin.net_type, &local->conn->net_type);
+	pj_strassign(&local->origin.addr_type, &local->conn->addr_type);
+	pj_strassign(&local->origin.addr, &local->conn->addr);
+
 	return local;
 }
 
@@ -3081,7 +4101,7 @@ static void session_inv_on_media_update(
 		 */
 
 		if (inv->following_fork) {
-			if (session->endpoint->follow_early_media_fork) {
+			if (session->endpoint->media.rtp.follow_early_media_fork) {
 				ast_debug(3, "Following early media fork with different To tags\n");
 			} else {
 				ast_debug(3, "Not following early media fork with different To tags\n");
@@ -3090,7 +4110,7 @@ static void session_inv_on_media_update(
 		}
 #ifdef HAVE_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS
 		else if (inv->updated_sdp_answer) {
-			if (session->endpoint->accept_multiple_sdp_answers) {
+			if (session->endpoint->media.rtp.accept_multiple_sdp_answers) {
 				ast_debug(3, "Accepting updated SDP with same To tag\n");
 			} else {
 				ast_debug(3, "Ignoring updated SDP answer with same To tag\n");
@@ -3239,8 +4259,6 @@ static int load_module(void)
 {
 	pjsip_endpoint *endpt;
 
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (!ast_sip_get_sorcery() || !ast_sip_get_pjsip_endpoint()) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -3249,8 +4267,8 @@ static int load_module(void)
 	}
 	nat_hook->outgoing_external_message = session_outgoing_nat_hook;
 	ast_sorcery_create(ast_sip_get_sorcery(), nat_hook);
-	sdp_handlers = ao2_container_alloc(SDP_HANDLER_BUCKETS,
-			sdp_handler_list_hash, sdp_handler_list_cmp);
+	sdp_handlers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		SDP_HANDLER_BUCKETS, sdp_handler_list_hash, NULL, sdp_handler_list_cmp);
 	if (!sdp_handlers) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -3281,8 +4299,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "PJSIP Session resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_session.exports.in 1:16.1.1~dfsg-1/res/res_pjsip_session.exports.in
--- 1:13.23.1~dfsg-2/res/res_pjsip_session.exports.in	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_session.exports.in	2018-12-26 18:45:13.000000000 +0000
@@ -1,28 +1,8 @@
 {
 	global:
-		LINKER_SYMBOL_PREFIXast_sip_session_terminate;
-		LINKER_SYMBOL_PREFIXast_sip_session_defer_termination;
-		LINKER_SYMBOL_PREFIXast_sip_session_defer_termination_cancel;
-		LINKER_SYMBOL_PREFIXast_sip_session_end_if_deferred;
-		LINKER_SYMBOL_PREFIXast_sip_session_register_sdp_handler;
-		LINKER_SYMBOL_PREFIXast_sip_session_unregister_sdp_handler;
-		LINKER_SYMBOL_PREFIXast_sip_session_register_supplement;
-		LINKER_SYMBOL_PREFIXast_sip_session_unregister_supplement;
-		LINKER_SYMBOL_PREFIXast_sip_session_alloc_datastore;
-		LINKER_SYMBOL_PREFIXast_sip_session_add_datastore;
-		LINKER_SYMBOL_PREFIXast_sip_session_get_datastore;
-		LINKER_SYMBOL_PREFIXast_sip_session_remove_datastore;
-		LINKER_SYMBOL_PREFIXast_sip_session_get_identity;
-		LINKER_SYMBOL_PREFIXast_sip_session_refresh;
-		LINKER_SYMBOL_PREFIXast_sip_session_regenerate_answer;
-		LINKER_SYMBOL_PREFIXast_sip_session_send_response;
-		LINKER_SYMBOL_PREFIXast_sip_session_send_request;
-		LINKER_SYMBOL_PREFIXast_sip_session_create_invite;
-		LINKER_SYMBOL_PREFIXast_sip_session_create_outgoing;
-		LINKER_SYMBOL_PREFIXast_sip_session_suspend;
-		LINKER_SYMBOL_PREFIXast_sip_session_unsuspend;
+		LINKER_SYMBOL_PREFIXast_sip_session_*;
+		LINKER_SYMBOL_PREFIX__ast_sip_session_*;
 		LINKER_SYMBOL_PREFIXast_sip_dialog_get_session;
-		LINKER_SYMBOL_PREFIXast_sip_session_resume_reinvite;
 		LINKER_SYMBOL_PREFIXast_sip_channel_pvt_alloc;
 	local:
 		*;
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_sips_contact.c 1:16.1.1~dfsg-1/res/res_pjsip_sips_contact.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_sips_contact.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_sips_contact.c	2018-12-26 18:45:13.000000000 +0000
@@ -90,8 +90,6 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	if (ast_sip_register_service(&sips_contact_module)) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -100,8 +98,9 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "UAC SIPS Contact support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_t38.c 1:16.1.1~dfsg-1/res/res_pjsip_t38.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_t38.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_t38.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,14 +37,14 @@
 #include <pjmedia.h>
 #include <pjlib.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
+#include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/udptl.h"
 #include "asterisk/netsock2.h"
 #include "asterisk/channel.h"
 #include "asterisk/acl.h"
-#include "asterisk/utils.h"
+#include "asterisk/stream.h"
+#include "asterisk/format_cache.h"
 
 #include "asterisk/res_pjsip.h"
 #include "asterisk/res_pjsip_session.h"
@@ -65,11 +65,16 @@ struct t38_state {
 	struct ast_control_t38_parameters their_parms;
 	/*! \brief Timer entry for automatically rejecting an inbound re-invite */
 	pj_timer_entry timer;
+	/*! Preserved media state for when T.38 ends */
+	struct ast_sip_session_media_state *media_state;
 };
 
 /*! \brief Destructor for T.38 state information */
 static void t38_state_destroy(void *obj)
 {
+	struct t38_state *state = obj;
+
+	ast_sip_session_media_state_free(state->media_state);
 	ast_free(obj);
 }
 
@@ -198,7 +203,7 @@ static int t38_automatic_reject(void *ob
 {
 	RAII_VAR(struct ast_sip_session *, session, obj, ao2_cleanup);
 	RAII_VAR(struct ast_datastore *, datastore, ast_sip_session_get_datastore(session, "t38"), ao2_cleanup);
-	RAII_VAR(struct ast_sip_session_media *, session_media, ao2_find(session->media, "image", OBJ_KEY), ao2_cleanup);
+	struct ast_sip_session_media *session_media;
 
 	if (!datastore) {
 		return 0;
@@ -207,6 +212,7 @@ static int t38_automatic_reject(void *ob
 	ast_debug(2, "Automatically rejecting T.38 request on channel '%s'\n",
 		session->channel ? ast_channel_name(session->channel) : "<gone>");
 
+	session_media = session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 	t38_change_state(session, session_media, datastore->data, T38_REJECTED);
 	ast_sip_session_resume_reinvite(session);
 
@@ -259,7 +265,6 @@ static int t38_initialize_session(struct
 		return -1;
 	}
 
-	ast_channel_set_fd(session->channel, 5, ast_udptl_fd(session_media->udptl));
 	ast_udptl_set_error_correction_scheme(session_media->udptl, session->endpoint->media.t38.error_correction);
 	ast_udptl_setnat(session_media->udptl, session->endpoint->media.t38.nat);
 	ast_udptl_set_far_max_datagram(session_media->udptl, session->endpoint->media.t38.maxdatagram);
@@ -271,19 +276,15 @@ static int t38_initialize_session(struct
 /*! \brief Callback for when T.38 reinvite SDP is created */
 static int t38_reinvite_sdp_cb(struct ast_sip_session *session, pjmedia_sdp_session *sdp)
 {
-	int stream;
+	struct t38_state *state;
 
-	/* Move the image media stream to the front and have it as the only stream, pjmedia will fill in
-	 * dummy streams for the rest
-	 */
-	for (stream = 0; stream < sdp->media_count; ++stream) {
-		if (!pj_strcmp2(&sdp->media[stream]->desc.media, "image")) {
-			sdp->media[0] = sdp->media[stream];
-			sdp->media_count = 1;
-			break;
-		}
+	state = t38_state_get_or_alloc(session);
+	if (!state) {
+		return -1;
 	}
 
+	state->media_state = ast_sip_session_media_state_clone(session->active_media_state);
+
 	return 0;
 }
 
@@ -299,32 +300,123 @@ static int t38_reinvite_response_cb(stru
 		return 0;
 	}
 
-	if (!session->channel || !(state = t38_state_get_or_alloc(session)) ||
-		!(session_media = ao2_find(session->media, "image", OBJ_KEY))) {
+	if (session->t38state != T38_LOCAL_REINVITE) {
+		/* Do nothing.  We have already processed a final response. */
+		ast_debug(3, "Received %d response to T.38 re-invite on '%s' but already had a final response (T.38 state:%d)\n",
+			status.code,
+			session->channel ? ast_channel_name(session->channel) : "unknown channel",
+			session->t38state);
+		return 0;
+	}
+
+	state = t38_state_get_or_alloc(session);
+	if (!session->channel || !state) {
 		ast_log(LOG_WARNING, "Received %d response to T.38 re-invite on '%s' but state unavailable\n",
 			status.code,
 			session->channel ? ast_channel_name(session->channel) : "unknown channel");
 		return 0;
 	}
 
-	/* Accept any 2xx response as successfully negotiated */
-	t38_change_state(session, session_media, state,
-		(status.code / 100 == 2) ? T38_ENABLED : T38_REJECTED);
+	if (status.code / 100 == 2) {
+		/* Accept any 2xx response as successfully negotiated */
+		int index;
+
+		session_media = session->active_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
+		t38_change_state(session, session_media, state, T38_ENABLED);
+
+		/* Stop all the streams in the stored away active state, they'll go back to being active once
+		 * we reinvite back.
+		 */
+		for (index = 0; index < AST_VECTOR_SIZE(&state->media_state->sessions); ++index) {
+			struct ast_sip_session_media *session_media = AST_VECTOR_GET(&state->media_state->sessions, index);
+
+			if (session_media && session_media->handler && session_media->handler->stream_stop) {
+				session_media->handler->stream_stop(session_media);
+			}
+		}
+	} else {
+		session_media = session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
+		t38_change_state(session, session_media, state, T38_REJECTED);
+
+		/* Abort this attempt at switching to T.38 by resetting the pending state and freeing our stored away active state */
+		ast_sip_session_media_state_free(state->media_state);
+		state->media_state = NULL;
+		ast_sip_session_media_state_reset(session->pending_media_state);
+	}
 
-	ao2_cleanup(session_media);
 	return 0;
 }
 
+/*! \brief Helper function which creates a media state for strictly T.38 */
+static struct ast_sip_session_media_state *t38_create_media_state(struct ast_sip_session *session)
+{
+	struct ast_sip_session_media_state *media_state;
+	struct ast_stream *stream;
+	struct ast_format_cap *caps;
+	struct ast_sip_session_media *session_media;
+
+	media_state = ast_sip_session_media_state_alloc();
+	if (!media_state) {
+		return NULL;
+	}
+
+	media_state->topology = ast_stream_topology_alloc();
+	if (!media_state->topology) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	stream = ast_stream_alloc("t38", AST_MEDIA_TYPE_IMAGE);
+	if (!stream) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	ast_stream_set_state(stream, AST_STREAM_STATE_SENDRECV);
+	if (ast_stream_topology_set_stream(media_state->topology, 0, stream)) {
+		ast_stream_free(stream);
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	ast_stream_set_formats(stream, caps);
+	/* stream holds a reference to cap, release the local reference
+	 * now so we don't have to deal with it in the error condition. */
+	ao2_ref(caps, -1);
+	if (ast_format_cap_append(caps, ast_format_t38, 0)) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	session_media = ast_sip_session_media_state_add(session, media_state, AST_MEDIA_TYPE_IMAGE, 0);
+	if (!session_media) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	if (t38_initialize_session(session, session_media)) {
+		ast_sip_session_media_state_free(media_state);
+		return NULL;
+	}
+
+	return media_state;
+}
+
 /*! \brief Task for reacting to T.38 control frame */
 static int t38_interpret_parameters(void *obj)
 {
 	RAII_VAR(struct t38_parameters_task_data *, data, obj, ao2_cleanup);
 	const struct ast_control_t38_parameters *parameters = data->frame->data.ptr;
 	struct t38_state *state = t38_state_get_or_alloc(data->session);
-	RAII_VAR(struct ast_sip_session_media *, session_media, ao2_find(data->session->media, "image", OBJ_KEY), ao2_cleanup);
+	struct ast_sip_session_media *session_media = NULL;
 
-	/* Without session media or state we can't interpret parameters */
-	if (!session_media || !state) {
+	if (!state) {
 		return 0;
 	}
 
@@ -334,12 +426,15 @@ static int t38_interpret_parameters(void
 		/* Negotiation can not take place without a valid max_ifp value. */
 		if (!parameters->max_ifp) {
 			if (data->session->t38state == T38_PEER_REINVITE) {
+				session_media = data->session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 				t38_change_state(data->session, session_media, state, T38_REJECTED);
 				ast_sip_session_resume_reinvite(data->session);
 			} else if (data->session->t38state == T38_ENABLED) {
+				session_media = data->session->active_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 				t38_change_state(data->session, session_media, state, T38_DISABLED);
 				ast_sip_session_refresh(data->session, NULL, NULL, NULL,
-					AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1);
+					AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1, state->media_state);
+				state->media_state = NULL;
 			}
 			break;
 		} else if (data->session->t38state == T38_PEER_REINVITE) {
@@ -358,37 +453,46 @@ static int t38_interpret_parameters(void
 			}
 			state->our_parms.version = MIN(state->our_parms.version, state->their_parms.version);
 			state->our_parms.rate_management = state->their_parms.rate_management;
+			session_media = data->session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 			ast_udptl_set_local_max_ifp(session_media->udptl, state->our_parms.max_ifp);
 			t38_change_state(data->session, session_media, state, T38_ENABLED);
 			ast_sip_session_resume_reinvite(data->session);
 		} else if ((data->session->t38state != T38_ENABLED) ||
 				((data->session->t38state == T38_ENABLED) &&
                                 (parameters->request_response == AST_T38_REQUEST_NEGOTIATE))) {
-			if (t38_initialize_session(data->session, session_media)) {
+			struct ast_sip_session_media_state *media_state;
+
+			media_state = t38_create_media_state(data->session);
+			if (!media_state) {
 				break;
 			}
 			state->our_parms = *parameters;
+			session_media = media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 			ast_udptl_set_local_max_ifp(session_media->udptl, state->our_parms.max_ifp);
 			t38_change_state(data->session, session_media, state, T38_LOCAL_REINVITE);
 			ast_sip_session_refresh(data->session, NULL, t38_reinvite_sdp_cb, t38_reinvite_response_cb,
-				AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1);
+				AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1, media_state);
 		}
 		break;
 	case AST_T38_TERMINATED:
 	case AST_T38_REFUSED:
 	case AST_T38_REQUEST_TERMINATE:         /* Shutdown T38 */
 		if (data->session->t38state == T38_PEER_REINVITE) {
+			session_media = data->session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 			t38_change_state(data->session, session_media, state, T38_REJECTED);
 			ast_sip_session_resume_reinvite(data->session);
 		} else if (data->session->t38state == T38_ENABLED) {
+			session_media = data->session->active_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 			t38_change_state(data->session, session_media, state, T38_DISABLED);
-			ast_sip_session_refresh(data->session, NULL, NULL, NULL, AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1);
+			ast_sip_session_refresh(data->session, NULL, NULL, NULL, AST_SIP_SESSION_REFRESH_METHOD_INVITE, 1, state->media_state);
+			state->media_state = NULL;
 		}
 		break;
 	case AST_T38_REQUEST_PARMS: {		/* Application wants remote's parameters re-sent */
 		struct ast_control_t38_parameters parameters = state->their_parms;
 
 		if (data->session->t38state == T38_PEER_REINVITE) {
+			session_media = data->session->pending_media_state->default_session[AST_MEDIA_TYPE_IMAGE];
 			parameters.max_ifp = ast_udptl_get_far_max_ifp(session_media->udptl);
 			parameters.request_response = AST_T38_REQUEST_NEGOTIATE;
 			ast_queue_control_data(data->session->channel, AST_CONTROL_T38_PARAMETERS, &parameters, sizeof(parameters));
@@ -402,18 +506,24 @@ static int t38_interpret_parameters(void
 	return 0;
 }
 
-/*! \brief Frame hook callback for writing */
-static struct ast_frame *t38_framehook_write(struct ast_channel *chan,
-	struct ast_sip_session *session, struct ast_frame *f)
+/*! \brief Frame hook callback for T.38 related stuff */
+static struct ast_frame *t38_framehook(struct ast_channel *chan, struct ast_frame *f,
+	enum ast_framehook_event event, void *data)
 {
+	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
+
+	if (event != AST_FRAMEHOOK_EVENT_WRITE) {
+		return f;
+	}
+
 	if (f->frametype == AST_FRAME_CONTROL
 		&& f->subclass.integer == AST_CONTROL_T38_PARAMETERS) {
-		if (session->endpoint->media.t38.enabled) {
+		if (channel->session->endpoint->media.t38.enabled) {
 			struct t38_parameters_task_data *data;
 
-			data = t38_parameters_task_data_alloc(session, f);
+			data = t38_parameters_task_data_alloc(channel->session, f);
 			if (data
-				&& ast_sip_push_task(session->serializer,
+				&& ast_sip_push_task(channel->session->serializer,
 					t38_interpret_parameters, data)) {
 				ao2_ref(data, -1);
 			}
@@ -443,52 +553,6 @@ static struct ast_frame *t38_framehook_w
 				break;
 			}
 		}
-	} else if (f->frametype == AST_FRAME_MODEM) {
-		struct ast_sip_session_media *session_media;
-
-		/* Avoid deadlock between chan and the session->media container lock */
-		ast_channel_unlock(chan);
-		session_media = ao2_find(session->media, "image", OBJ_SEARCH_KEY);
-		ast_channel_lock(chan);
-		if (session_media && session_media->udptl) {
-			ast_udptl_write(session_media->udptl, f);
-		}
-		ao2_cleanup(session_media);
-	}
-
-	return f;
-}
-
-/*! \brief Frame hook callback for reading */
-static struct ast_frame *t38_framehook_read(struct ast_channel *chan,
-	struct ast_sip_session *session, struct ast_frame *f)
-{
-	if (ast_channel_fdno(session->channel) == 5) {
-		struct ast_sip_session_media *session_media;
-
-		/* Avoid deadlock between chan and the session->media container lock */
-		ast_channel_unlock(chan);
-		session_media = ao2_find(session->media, "image", OBJ_SEARCH_KEY);
-		ast_channel_lock(chan);
-		if (session_media && session_media->udptl) {
-			f = ast_udptl_read(session_media->udptl);
-		}
-		ao2_cleanup(session_media);
-	}
-
-	return f;
-}
-
-/*! \brief Frame hook callback for T.38 related stuff */
-static struct ast_frame *t38_framehook(struct ast_channel *chan, struct ast_frame *f,
-	enum ast_framehook_event event, void *data)
-{
-	struct ast_sip_channel_pvt *channel = ast_channel_tech_pvt(chan);
-
-	if (event == AST_FRAMEHOOK_EVENT_READ) {
-		f = t38_framehook_read(chan, channel->session, f);
-	} else if (event == AST_FRAMEHOOK_EVENT_WRITE) {
-		f = t38_framehook_write(chan, channel->session, f);
 	}
 
 	return f;
@@ -507,7 +571,7 @@ static void t38_masq(void *data, int fra
 
 static int t38_consume(void *data, enum ast_frame_type type)
 {
-	return 0;
+	return (type == AST_FRAME_CONTROL) ? 1 : 0;
 }
 
 static const struct ast_datastore_info t38_framehook_datastore = {
@@ -704,11 +768,13 @@ static enum ast_sip_session_sdp_stream_d
 }
 
 /*! \brief Function which negotiates an incoming media stream */
-static int negotiate_incoming_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-					 const struct pjmedia_sdp_session *sdp, const struct pjmedia_sdp_media *stream)
+static int negotiate_incoming_sdp_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *sdp,
+	int index, struct ast_stream *asterisk_stream)
 {
 	struct t38_state *state;
 	char host[NI_MAXHOST];
+	pjmedia_sdp_media *stream = sdp->media[index];
 	RAII_VAR(struct ast_sockaddr *, addrs, NULL, ast_free);
 
 	if (!session->endpoint->media.t38.enabled) {
@@ -748,7 +814,7 @@ static int negotiate_incoming_sdp_stream
 
 /*! \brief Function which creates an outgoing stream */
 static int create_outgoing_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-				      struct pjmedia_sdp_session *sdp)
+				      struct pjmedia_sdp_session *sdp, const struct pjmedia_sdp_session *remote, struct ast_stream *stream)
 {
 	pj_pool_t *pool = session->inv_session->pool_prov;
 	static const pj_str_t STR_IN = { "IN", 2 };
@@ -786,7 +852,7 @@ static int create_outgoing_sdp_stream(st
 		return -1;
 	}
 
-	media->desc.media = pj_str(session_media->stream_type);
+	pj_strdup2(pool, &media->desc.media, ast_codec_media_type2str(session_media->type));
 	media->desc.transport = STR_UDPTL;
 
 	if (ast_strlen_zero(session->endpoint->media.address)) {
@@ -854,12 +920,40 @@ static int create_outgoing_sdp_stream(st
 	return 1;
 }
 
+static struct ast_frame *media_session_udptl_read_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media)
+{
+	struct ast_frame *frame;
+
+	if (!session_media->udptl) {
+		return &ast_null_frame;
+	}
+
+	frame = ast_udptl_read(session_media->udptl);
+	if (!frame) {
+		return NULL;
+	}
+
+	frame->stream_num = session_media->stream_num;
+
+	return frame;
+}
+
+static int media_session_udptl_write_callback(struct ast_sip_session *session, struct ast_sip_session_media *session_media, struct ast_frame *frame)
+{
+	if (!session_media->udptl) {
+		return 0;
+	}
+
+	return ast_udptl_write(session_media->udptl, frame);
+}
+
 /*! \brief Function which applies a negotiated stream */
-static int apply_negotiated_sdp_stream(struct ast_sip_session *session, struct ast_sip_session_media *session_media,
-				       const struct pjmedia_sdp_session *local, const struct pjmedia_sdp_media *local_stream,
-				       const struct pjmedia_sdp_session *remote, const struct pjmedia_sdp_media *remote_stream)
+static int apply_negotiated_sdp_stream(struct ast_sip_session *session,
+	struct ast_sip_session_media *session_media, const struct pjmedia_sdp_session *local,
+	const struct pjmedia_sdp_session *remote, int index, struct ast_stream *asterisk_stream)
 {
 	RAII_VAR(struct ast_sockaddr *, addrs, NULL, ast_free);
+	pjmedia_sdp_media *remote_stream = remote->media[index];
 	char host[NI_MAXHOST];
 	struct t38_state *state;
 
@@ -886,6 +980,10 @@ static int apply_negotiated_sdp_stream(s
 
 	t38_interpret_sdp(state, session, session_media, remote_stream);
 
+	ast_sip_session_media_set_write_callback(session, session_media, media_session_udptl_write_callback);
+	ast_sip_session_media_add_read_callback(session, session_media, ast_udptl_fd(session_media->udptl),
+		media_session_udptl_read_callback);
+
 	return 0;
 }
 
@@ -956,18 +1054,13 @@ static int unload_module(void)
  */
 static int load_module(void)
 {
-	CHECK_PJSIP_SESSION_MODULE_LOADED();
-
 	if (ast_check_ipv6()) {
 		ast_sockaddr_parse(&address, "::", 0);
 	} else {
 		ast_sockaddr_parse(&address, "0.0.0.0", 0);
 	}
 
-	if (ast_sip_session_register_supplement(&t38_supplement)) {
-		ast_log(LOG_ERROR, "Unable to register T.38 session supplement\n");
-		goto end;
-	}
+	ast_sip_session_register_supplement(&t38_supplement);
 
 	if (ast_sip_session_register_sdp_handler(&image_sdp_handler, "image")) {
 		ast_log(LOG_ERROR, "Unable to register SDP handler for image stream type\n");
@@ -982,8 +1075,9 @@ end:
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP T.38 UDPTL Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DRIVER,
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DRIVER,
+	.requires = "res_pjsip,res_pjsip_session,udptl",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_transport_websocket.c 1:16.1.1~dfsg-1/res/res_pjsip_transport_websocket.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_transport_websocket.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_transport_websocket.c	2018-12-26 18:45:13.000000000 +0000
@@ -293,7 +293,7 @@ static int transport_read(void *data)
 	pj_sockaddr_parse(pj_AF_UNSPEC(), 0, pj_cstr(&buf, ast_sockaddr_stringify(ast_websocket_remote_address(session))), &rdata->pkt_info.src_addr);
 	rdata->pkt_info.src_addr_len = sizeof(rdata->pkt_info.src_addr);
 
-	pj_ansi_strcpy(rdata->pkt_info.src_name, ast_sockaddr_stringify_host(ast_websocket_remote_address(session)));
+	pj_ansi_strcpy(rdata->pkt_info.src_name, ast_sockaddr_stringify_addr(ast_websocket_remote_address(session)));
 	rdata->pkt_info.src_port = ast_sockaddr_port(ast_websocket_remote_address(session));
 
 	recvd = pjsip_tpmgr_receive_packet(rdata->tp_info.transport->tpmgr, rdata);
@@ -348,7 +348,7 @@ static struct ast_taskprocessor *create_
 	/* Create name with seq number appended. */
 	ast_taskprocessor_build_name(tps_name, sizeof(tps_name), "pjsip/websocket");
 
-	return ast_sip_create_serializer_named(tps_name);
+	return ast_sip_create_serializer(tps_name);
 }
 
 /*! \brief WebSocket connection handler. */
@@ -429,12 +429,23 @@ static pj_bool_t websocket_on_rx_msg(pjs
 		pjsip_sip_uri *uri = pjsip_uri_get_uri(contact->uri);
 		const pj_str_t *txp_str = &STR_WS;
 
-		ast_debug(4, "%s re-writing Contact URI from %.*s:%d%s%.*s to %s:%d;transport=%s\n",
-			pjsip_rx_data_get_info(rdata),
-			(int)pj_strlen(&uri->host), pj_strbuf(&uri->host), uri->port,
-			pj_strlen(&uri->transport_param) ? ";transport=" : "",
-			(int)pj_strlen(&uri->transport_param), pj_strbuf(&uri->transport_param),
-			rdata->pkt_info.src_name ?: "", rdata->pkt_info.src_port, pj_strbuf(txp_str));
+		if (DEBUG_ATLEAST(4)) {
+			char src_addr_buffer[AST_SOCKADDR_BUFLEN];
+			const char *ipv6_s = "", *ipv6_e = "";
+
+			if (pj_strchr(&uri->host, ':')) {
+				ipv6_s = "[";
+				ipv6_e = "]";
+			}
+
+			ast_log(LOG_DEBUG, "%s re-writing Contact URI from %s%.*s%s:%d%s%.*s to %s;transport=%s\n",
+				pjsip_rx_data_get_info(rdata),
+				ipv6_s, (int) pj_strlen(&uri->host), pj_strbuf(&uri->host), ipv6_e, uri->port,
+				pj_strlen(&uri->transport_param) ? ";transport=" : "",
+				(int) pj_strlen(&uri->transport_param), pj_strbuf(&uri->transport_param),
+				pj_sockaddr_print(&rdata->pkt_info.src_addr, src_addr_buffer, sizeof(src_addr_buffer), 3),
+				pj_strbuf(txp_str));
+		}
 
 		pj_cstr(&uri->host, rdata->pkt_info.src_name);
 		uri->port = rdata->pkt_info.src_port;
@@ -471,8 +482,6 @@ static struct ast_sip_session_supplement
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	/*
 	 * We only need one transport type name (ws) defined.  Firefox
 	 * and Chrome do not support anything other than secure websockets
@@ -490,10 +499,7 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	if (ast_sip_session_register_supplement(&websocket_supplement)) {
-		ast_sip_unregister_service(&websocket_module);
-		return AST_MODULE_LOAD_DECLINE;
-	}
+	ast_sip_session_register_supplement(&websocket_supplement);
 
 	if (ast_websocket_add_protocol("sip", websocket_cb)) {
 		ast_sip_session_unregister_supplement(&websocket_supplement);
@@ -514,8 +520,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP WebSocket Transport Support",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND,
-	   );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND,
+	.requires = "res_pjsip,res_http_websocket",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_pjsip_xpidf_body_generator.c 1:16.1.1~dfsg-1/res/res_pjsip_xpidf_body_generator.c
--- 1:13.23.1~dfsg-2/res/res_pjsip_xpidf_body_generator.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pjsip_xpidf_body_generator.c	2018-12-26 18:45:13.000000000 +0000
@@ -20,7 +20,6 @@
 	<depend>pjproject</depend>
 	<depend>res_pjsip</depend>
 	<depend>res_pjsip_pubsub</depend>
-	<depend>res_pjsip_exten_state</depend>
 	<support_level>core</support_level>
  ***/
 
@@ -149,8 +148,6 @@ static void unregister_all(void)
 
 static int load_module(void)
 {
-	CHECK_PJSIP_PUBSUB_MODULE_LOADED();
-
 	if (ast_sip_pubsub_register_body_generator(&xpidf_body_generator)) {
 		goto fail;
 	}
@@ -173,8 +170,9 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "PJSIP Extension State PIDF Provider",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+	.requires = "res_pjsip,res_pjsip_pubsub",
 );
diff -pruN 1:13.23.1~dfsg-2/res/res_pktccops.c 1:16.1.1~dfsg-1/res/res_pktccops.c
--- 1:13.23.1~dfsg-2/res/res_pktccops.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_pktccops.c	2018-12-26 18:45:13.000000000 +0000
@@ -37,8 +37,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <fcntl.h>
@@ -340,7 +338,7 @@ static int cops_getmsg (int sfd, struct
 	recmsg->length = ntohl(*((uint32_t *) (buf + 4)));
 	/* Eg KA msg*/
 	if (recmsg->clienttype != 0x8008 ) {
-		if (!(recmsg->msg = malloc(recmsg->length - COPS_HEADER_SIZE))) {
+		if (!(recmsg->msg = ast_malloc(recmsg->length - COPS_HEADER_SIZE))) {
 			return -1;
 		}
 		lent = recv(sfd, recmsg->msg, recmsg->length - COPS_HEADER_SIZE, MSG_DONTWAIT);
@@ -353,12 +351,12 @@ static int cops_getmsg (int sfd, struct
 		while (len < recmsg->length) {
 			if (len == COPS_HEADER_SIZE) {
 				/* 1st round */
-				if (!(recmsg->object = malloc(sizeof(struct pktcobj)))) {
+				if (!(recmsg->object = ast_malloc(sizeof(struct pktcobj)))) {
 					return -1;
 				}
 				pobject = recmsg->object;
 			} else {
-		 		if (!(pobject->next = malloc(sizeof(struct pktcobj)))) {
+		 		if (!(pobject->next = ast_malloc(sizeof(struct pktcobj)))) {
 					return -1;
 				}
 				pobject = pobject->next;
@@ -373,7 +371,7 @@ static int cops_getmsg (int sfd, struct
 			pobject->length = ntohs(*ubuf);
 			pobject->cnum = *(buf + 2);
 			pobject->ctype = *(buf + 3);
-			if (!(pobject->contents = malloc(pobject->length - COPS_OBJECT_HEADER_SIZE))) {
+			if (!(pobject->contents = ast_malloc(pobject->length - COPS_OBJECT_HEADER_SIZE))) {
 				return -1;
 			}
 			lent = recv(sfd, pobject->contents, pobject->length - COPS_OBJECT_HEADER_SIZE, MSG_DONTWAIT);
@@ -402,7 +400,7 @@ static int cops_sendmsg (int sfd, struct
 		ast_log(LOG_WARNING, "COPS: invalid msg size!!!\n");
 		return -1;
 	}
-	if (!(buf = malloc((size_t) sendmsg->length))) {
+	if (!(buf = ast_malloc((size_t) sendmsg->length))) {
 		return -1;
 	}
 	*buf = sendmsg->verflag ;
@@ -419,7 +417,7 @@ static int cops_sendmsg (int sfd, struct
 			ast_debug(3, "COPS: Sending Object : cnum: %i ctype %i len: %i\n", pobject->cnum, pobject->ctype, pobject->length);
 			if (sendmsg->length < bufpos + pobject->length) {
 				ast_log(LOG_WARNING, "COPS: Invalid msg size len: %u objectlen: %i\n", sendmsg->length, pobject->length);
-				free(buf);
+				ast_free(buf);
 				return -1;
 			}
 			*(uint16_t *) (buf + bufpos) = htons(pobject->length);
@@ -427,7 +425,7 @@ static int cops_sendmsg (int sfd, struct
 			*(buf + bufpos + 3) = pobject->ctype;
 			if (sendmsg->length < pobject->length + bufpos) {
 				ast_log(LOG_WARNING, "COPS: Error sum of object len more the msg len %u < %i\n", sendmsg->length, pobject->length + bufpos);
-				free(buf);
+				ast_free(buf);
 				return -1;
 			}
 			memcpy((buf + bufpos + 4), pobject->contents, pobject->length - 4);
@@ -444,18 +442,18 @@ static int cops_sendmsg (int sfd, struct
 #endif
 	if (send(sfd, buf, sendmsg->length, SENDFLAGS) == -1) {
 		ast_log(LOG_WARNING, "COPS: Send failed errno=%i\n", errno);
-		free(buf);
+		ast_free(buf);
 		return -2;
 	}
 #undef SENDFLAGS
-	free(buf);
+	ast_free(buf);
 	return 0;
 }
 
 static void cops_freemsg(struct copsmsg *p)
 {
 	struct pktcobj *pnext;
-	free(p->msg);
+	ast_free(p->msg);
 	p->msg = NULL;
 	while (p->object != NULL) {
 			pnext = p->object->next;
@@ -555,8 +553,8 @@ static struct cops_gate *cops_gate_cmd(i
 
 	gate->in_transaction = time(NULL);
 
-	if (!(gateset = malloc(sizeof(struct copsmsg)))) {
-		free(gateset);
+	if (!(gateset = ast_malloc(sizeof(struct copsmsg)))) {
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->msg = NULL;
@@ -565,67 +563,67 @@ static struct cops_gate *cops_gate_cmd(i
 	gateset->clienttype = 0x8008; /* =PacketCable */
 
 	/* Handle object */
-	gateset->object = malloc(sizeof(struct pktcobj));
+	gateset->object = ast_malloc(sizeof(struct pktcobj));
 	if (!gateset->object) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->object->length = COPS_OBJECT_HEADER_SIZE + 4;
 	gateset->object->cnum = 1; /* Handle */
 	gateset->object->ctype = 1; /* client */
-	if (!(gateset->object->contents = malloc(sizeof(uint32_t)))) {
+	if (!(gateset->object->contents = ast_malloc(sizeof(uint32_t)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	*((uint32_t *) gateset->object->contents) = htonl(cmts->handle);
 
 	/* Context Object */
-	if (!(gateset->object->next = malloc(sizeof(struct pktcobj)))) {
+	if (!(gateset->object->next = ast_malloc(sizeof(struct pktcobj)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->object->next->length = COPS_OBJECT_HEADER_SIZE + 4;
 	gateset->object->next->cnum = 2; /* Context */
 	gateset->object->next->ctype = 1; /* Context */
-	if (!(gateset->object->next->contents = malloc(sizeof(uint32_t)))) {
+	if (!(gateset->object->next->contents = ast_malloc(sizeof(uint32_t)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	*((uint32_t *) gateset->object->next->contents) = htonl(0x00080000); /* R-Type = 8 configuration request, M-Type = 0 */
 
 	/* Decision Object: Flags */
-	if (!(gateset->object->next->next = malloc(sizeof(struct pktcobj)))) {
+	if (!(gateset->object->next->next = ast_malloc(sizeof(struct pktcobj)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->object->next->next->length = COPS_OBJECT_HEADER_SIZE + 4;
 	gateset->object->next->next->cnum = 6; /* Decision */
 	gateset->object->next->next->ctype = 1; /* Flags */
-	if (!(gateset->object->next->next->contents = malloc(sizeof(uint32_t)))) {
+	if (!(gateset->object->next->next->contents = ast_malloc(sizeof(uint32_t)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	*((uint32_t *) gateset->object->next->next->contents) = htonl(0x00010001); /* Install, Trigger Error */
 
 	/* Decision Object: Data */
-	if (!(gateset->object->next->next->next = malloc(sizeof(struct pktcobj)))) {
+	if (!(gateset->object->next->next->next = ast_malloc(sizeof(struct pktcobj)))) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->object->next->next->next->length = COPS_OBJECT_HEADER_SIZE + ((cmd != GATE_INFO && cmd != GATE_DEL) ? GATE_SET_OBJ_SIZE : GATE_INFO_OBJ_SIZE) + ((cmd == GATE_SET_HAVE_GATEID) ? GATEID_OBJ_SIZE : 0);
 	gateset->object->next->next->next->cnum = 6; /* Decision */
 	gateset->object->next->next->next->ctype = 4; /* Decision Data */
-	gateset->object->next->next->next->contents = malloc(((cmd != GATE_INFO && cmd != GATE_DEL) ? GATE_SET_OBJ_SIZE : GATE_INFO_OBJ_SIZE) + ((cmd == GATE_SET_HAVE_GATEID) ? GATEID_OBJ_SIZE : 0));
+	gateset->object->next->next->next->contents = ast_malloc(((cmd != GATE_INFO && cmd != GATE_DEL) ? GATE_SET_OBJ_SIZE : GATE_INFO_OBJ_SIZE) + ((cmd == GATE_SET_HAVE_GATEID) ? GATEID_OBJ_SIZE : 0));
 	if (!gateset->object->next->next->next->contents) {
 		cops_freemsg(gateset);
-		free(gateset);
+		ast_free(gateset);
 		return NULL;
 	}
 	gateset->object->next->next->next->next = NULL;
@@ -644,7 +642,7 @@ static struct cops_gate *cops_gate_cmd(i
 	}
 	cops_sendmsg(cmts->sfd, gateset);
 	cops_freemsg(gateset);
-	free(gateset);
+	ast_free(gateset);
 	return gate;
 }
 
@@ -928,11 +926,11 @@ static void *do_pktccops(void *data)
 							sendmsg->opcode = 7; /* Client Accept */
 							sendmsg->clienttype = 0x8008; /* =PacketCable */
 							sendmsg->length = COPS_HEADER_SIZE + COPS_OBJECT_HEADER_SIZE + 4;
-							sendmsg->object = malloc(sizeof(struct pktcobj));
+							sendmsg->object = ast_malloc(sizeof(struct pktcobj));
 							sendmsg->object->length = 4 + COPS_OBJECT_HEADER_SIZE;
 							sendmsg->object->cnum = 10; /* keppalive timer*/
 							sendmsg->object->ctype = 1;
-							sendmsg->object->contents = malloc(sizeof(uint32_t));
+							sendmsg->object->contents = ast_malloc(sizeof(uint32_t));
 							*((uint32_t *) sendmsg->object->contents) = htonl(cmts->keepalive & 0x0000ffff);
 							sendmsg->object->next = NULL;
 							cops_sendmsg(cmts->sfd, sendmsg);
@@ -979,7 +977,7 @@ static void *do_pktccops(void *data)
 						close(cmts->sfd);
 					}
 					AST_LIST_REMOVE_CURRENT(list);
-					free(cmts);
+					ast_free(cmts);
 				}
 			}
 			AST_LIST_TRAVERSE_SAFE_END;
@@ -1441,13 +1439,13 @@ static void pktccops_unregister_cmtses(v
 		if (cmts->sfd > 0) {
 			close(cmts->sfd);
 		}
-		free(cmts);
+		ast_free(cmts);
 	}
 	AST_LIST_UNLOCK(&cmts_list);
 
 	AST_LIST_LOCK(&gate_list);
 	while ((gate = AST_LIST_REMOVE_HEAD(&gate_list, list))) {
-		free(gate);
+		ast_free(gate);
 	}
 	AST_LIST_UNLOCK(&gate_list);
 }
@@ -1457,7 +1455,7 @@ static void pktccops_unregister_ippools(
 	struct cops_ippool *ippool;
 	AST_LIST_LOCK(&ippool_list);
 	while ((ippool = AST_LIST_REMOVE_HEAD(&ippool_list, list))) {
-		free(ippool);
+		ast_free(ippool);
 	}
 	AST_LIST_UNLOCK(&ippool_list);
 }
@@ -1474,9 +1472,6 @@ static int load_module(void)
 	ast_cli_register_multiple(cli_pktccops, sizeof(cli_pktccops) / sizeof(struct ast_cli_entry));
 	restart_pktc_thread();
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return 0;
 }
 
@@ -1514,8 +1509,8 @@ static int reload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS, "PktcCOPS manager for MGCP",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_realtime.c 1:16.1.1~dfsg-1/res/res_realtime.c
--- 1:13.23.1~dfsg-2/res/res_realtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_realtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_remb_modifier.c 1:16.1.1~dfsg-1/res/res_remb_modifier.c
--- 1:13.23.1~dfsg-2/res/res_remb_modifier.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_remb_modifier.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,225 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2018, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ *
+ * \brief REMB Modifier Module
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ */
+
+/*** MODULEINFO
+	<support_level>extended</support_level>
+	<defaultenabled>no</defaultenabled>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/module.h"
+#include "asterisk/cli.h"
+#include "asterisk/channel.h"
+#include "asterisk/framehook.h"
+#include "asterisk/rtp_engine.h"
+
+struct remb_values {
+	/*! \brief The amount of bitrate to use for REMB received from the channel */
+	unsigned int receive_bitrate;
+	/*! \brief The amount of bitrate to use for REMB sent to the channel */
+	unsigned int send_bitrate;
+};
+
+static void remb_values_free(void *data)
+{
+	ast_free(data);
+}
+
+static const struct ast_datastore_info remb_info = {
+	.type = "REMB Values",
+	.destroy = remb_values_free,
+};
+
+static struct ast_frame *remb_hook_event_cb(struct ast_channel *chan, struct ast_frame *frame, enum ast_framehook_event event, void *data)
+{
+	struct ast_rtp_rtcp_feedback *feedback;
+	struct ast_datastore *remb_store;
+	struct remb_values *remb_values;
+
+	if (!frame) {
+		return NULL;
+	}
+
+	switch (event) {
+	case AST_FRAMEHOOK_EVENT_READ:
+	case AST_FRAMEHOOK_EVENT_WRITE:
+		break;
+	case AST_FRAMEHOOK_EVENT_ATTACHED:
+	case AST_FRAMEHOOK_EVENT_DETACHED:
+		return frame;
+	}
+
+	/* We only care about REMB frames, all others will be unmodified */
+	if (frame->subclass.integer != AST_RTP_RTCP_PSFB) {
+		return frame;
+	}
+
+	feedback = frame->data.ptr;
+	if (feedback->fmt != AST_RTP_RTCP_FMT_REMB) {
+		return frame;
+	}
+
+	remb_store = ast_channel_datastore_find(chan, &remb_info, NULL);
+	if (!remb_store) {
+		return frame;
+	}
+	remb_values = remb_store->data;
+
+	/* If a bitrate override has been set apply it to the REMB Frame */
+	if (event == AST_FRAMEHOOK_EVENT_READ && remb_values->receive_bitrate) {
+		feedback->remb.br_mantissa = remb_values->receive_bitrate;
+		feedback->remb.br_exp = 0;
+	} else if (event == AST_FRAMEHOOK_EVENT_WRITE && remb_values->send_bitrate) {
+		feedback->remb.br_mantissa = remb_values->send_bitrate;
+		feedback->remb.br_exp = 0;
+	}
+
+	/* The mantissa only has 18 bits available, so while it exceeds them we bump
+	 * up the exp.
+	 */
+	while (feedback->remb.br_mantissa > 0x3ffff) {
+		feedback->remb.br_mantissa = feedback->remb.br_mantissa >> 1;
+		feedback->remb.br_exp++;
+	}
+
+	return frame;
+}
+
+static char *handle_remb_set(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	struct ast_channel *chan;
+	unsigned int bitrate;
+	struct ast_datastore *remb_store;
+	struct remb_values *remb_values;
+	struct ast_framehook_interface interface = {
+		.version = AST_FRAMEHOOK_INTERFACE_VERSION,
+		.event_cb = remb_hook_event_cb,
+	};
+
+	switch(cmd) {
+	case CLI_INIT:
+		e->command = "remb set {send|receive}";
+		e->usage =
+			"Usage: remb set {send|receive} <channel> <bitrate in bits>\n"
+			"       Set the REMB value which overwrites what we send or receive\n";
+		return NULL;
+	case CLI_GENERATE:
+		return ast_complete_channels(a->line, a->word, a->pos, a->n, 3);
+	}
+
+	if (a->argc != 5) {
+		return CLI_SHOWUSAGE;
+	}
+
+	if (sscanf(a->argv[4], "%30d", &bitrate) != 1) {
+		ast_cli(a->fd, "%s is not a valid bitrate in bits\n", a->argv[4]);
+		return CLI_SUCCESS;
+	} else if (strcasecmp(a->argv[2], "send") && strcasecmp(a->argv[2], "receive")) {
+		ast_cli(a->fd, "%s is not a valid direction for REMB\n", a->argv[2]);
+		return CLI_SUCCESS;
+	}
+
+	chan = ast_channel_get_by_name(a->argv[3]);
+	if (!chan) {
+		ast_cli(a->fd, "%s is not a known channel\n", a->argv[3]);
+		return CLI_SUCCESS;
+	}
+
+	ast_channel_lock(chan);
+
+	remb_store = ast_channel_datastore_find(chan, &remb_info, NULL);
+	if (!remb_store) {
+		int framehook_id;
+
+		framehook_id = ast_framehook_attach(chan, &interface);
+		if (framehook_id < 0) {
+			ast_cli(a->fd, "Could not attach framehook for modifying REMB\n");
+			ast_channel_unlock(chan);
+			ast_channel_unref(chan);
+			return CLI_SUCCESS;
+		}
+
+		remb_values = ast_calloc(1, sizeof(*remb_values));
+		if (!remb_values) {
+			ast_cli(a->fd, "Could not create a place to store provided REMB value\n");
+			ast_framehook_detach(chan, framehook_id);
+			ast_channel_unlock(chan);
+			ast_channel_unref(chan);
+			return CLI_SUCCESS;
+		}
+
+		remb_store = ast_datastore_alloc(&remb_info, NULL);
+		if (!remb_store) {
+			ast_cli(a->fd, "Could not create a place to store provided REMB value\n");
+			ast_framehook_detach(chan, framehook_id);
+			ast_channel_unlock(chan);
+			ast_channel_unref(chan);
+			ast_free(remb_values);
+			return CLI_SUCCESS;
+		}
+
+		remb_store->data = remb_values;
+		ast_channel_datastore_add(chan, remb_store);
+	} else {
+		remb_values = remb_store->data;
+	}
+
+	if (!strcasecmp(a->argv[2], "send")) {
+		remb_values->send_bitrate = bitrate;
+	} else if (!strcasecmp(a->argv[2], "receive")) {
+		remb_values->receive_bitrate = bitrate;
+	}
+
+	ast_channel_unlock(chan);
+	ast_channel_unref(chan);
+
+	ast_cli(a->fd, "Set REMB %s override to a bitrate of %s on %s\n", a->argv[2], a->argv[3], a->argv[4]);
+
+	return CLI_SUCCESS;
+}
+
+static struct ast_cli_entry remb_cli[] = {
+	AST_CLI_DEFINE(handle_remb_set, "Set the REMB value which overwrites what is sent or received"),
+};
+
+static int load_module(void)
+{
+	ast_cli_register_multiple(remb_cli, ARRAY_LEN(remb_cli));
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	ast_cli_unregister_multiple(remb_cli, ARRAY_LEN(remb_cli));
+	return 0;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "REMB Modifier Module",
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_resolver_unbound.c 1:16.1.1~dfsg-1/res/res_resolver_unbound.c
--- 1:13.23.1~dfsg-2/res/res_resolver_unbound.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_resolver_unbound.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,1474 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>unbound</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <signal.h>
+#include <unbound.h>
+#include <arpa/nameser.h>
+
+#include "asterisk/module.h"
+#include "asterisk/linkedlists.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/config.h"
+#include "asterisk/config_options.h"
+#include "asterisk/test.h"
+
+#ifdef TEST_FRAMEWORK
+#include "asterisk/dns_srv.h"
+#endif
+
+/*** DOCUMENTATION
+	<configInfo name="res_resolver_unbound" language="en_US">
+		<configFile name="resolver_unbound.conf">
+			<configObject name="general">
+				<synopsis>General options for res_resolver_unbound</synopsis>
+				<configOption name="hosts">
+					<synopsis>Full path to an optional hosts file</synopsis>
+					<description><para>Hosts specified in a hosts file will be resolved within the resolver itself. If a value
+					of system is provided the system-specific file will be used.</para></description>
+				</configOption>
+				<configOption name="resolv">
+					<synopsis>Full path to an optional resolv.conf file</synopsis>
+					<description><para>The resolv.conf file specifies the nameservers to contact when resolving queries. If a
+					value of system is provided the system-specific file will be used. If provided alongside explicit nameservers the
+					nameservers contained within the resolv.conf file will be used after all others.</para></description>
+				</configOption>
+				<configOption name="nameserver">
+					<synopsis>Nameserver to use for queries</synopsis>
+					<description><para>An explicit nameserver can be specified which is used for resolving queries. If multiple
+					nameserver lines are specified the first will be the primary with failover occurring, in order, to the other
+					nameservers as backups. If provided alongside a resolv.conf file the nameservers explicitly specified will be
+					used before all others.</para></description>
+				</configOption>
+				<configOption name="debug">
+					<synopsis>Unbound debug level</synopsis>
+					<description><para>The debugging level for the unbound resolver. While there is no explicit range generally
+					the higher the number the more debug is output.</para></description>
+				</configOption>
+				<configOption name="ta_file">
+					<synopsis>Trust anchor file</synopsis>
+					<description><para>Full path to a file with DS and DNSKEY records in zone file format. This file is provided
+					to unbound and is used as a source for trust anchors.</para></description>
+				</configOption>
+			</configObject>
+		</configFile>
+	</configInfo>
+ ***/
+
+/*! \brief Structure for an unbound resolver */
+struct unbound_resolver {
+	/*! \brief Resolver context itself */
+	struct ub_ctx *context;
+	/*! \brief Thread handling the resolver */
+	pthread_t thread;
+};
+
+/*! \brief Structure for query resolver data */
+struct unbound_resolver_data {
+	/*! \brief ID for the specific query */
+	int id;
+	/*! \brief The resolver in use for the query */
+	struct unbound_resolver *resolver;
+};
+
+/*! \brief Unbound configuration state information */
+struct unbound_config_state {
+	/*! \brief The configured resolver */
+	struct unbound_resolver *resolver;
+};
+
+/*! \brief A structure to hold global configuration-related options */
+struct unbound_global_config {
+	AST_DECLARE_STRING_FIELDS(
+		AST_STRING_FIELD(hosts);   /*!< Optional hosts file */
+		AST_STRING_FIELD(resolv);  /*!< Optional resolv.conf file */
+		AST_STRING_FIELD(ta_file); /*!< Optional trust anchor file */
+	);
+	/*! \brief List of nameservers (in order) to use for queries */
+	struct ao2_container *nameservers;
+	/*! \brief Debug level for the resolver */
+	unsigned int debug;
+	/*! \brief State information */
+	struct unbound_config_state *state;
+};
+
+/*! \brief A container for config related information */
+struct unbound_config {
+	struct unbound_global_config *global;
+};
+
+/*!
+ * \brief Allocate a unbound_config to hold a snapshot of the complete results of parsing a config
+ * \internal
+ * \returns A void pointer to a newly allocated unbound_config
+ */
+static void *unbound_config_alloc(void);
+
+/*! \brief An aco_type structure to link the "general" category to the unbound_global_config type */
+static struct aco_type global_option = {
+	.type = ACO_GLOBAL,
+	.name = "general",
+	.item_offset = offsetof(struct unbound_config, global),
+	.category_match = ACO_WHITELIST_EXACT,
+	.category = "general",
+};
+
+static struct aco_type *global_options[] = ACO_TYPES(&global_option);
+
+static struct aco_file resolver_unbound_conf = {
+	.filename = "resolver_unbound.conf",
+	.types = ACO_TYPES(&global_option),
+};
+
+/*! \brief A global object container that will contain the global_config that gets swapped out on reloads */
+static AO2_GLOBAL_OBJ_STATIC(globals);
+
+/*!
+ * \brief Finish initializing new configuration
+ * \internal
+ */
+static int unbound_config_preapply_callback(void);
+
+/*! \brief Register information about the configs being processed by this module */
+CONFIG_INFO_STANDARD(cfg_info, globals, unbound_config_alloc,
+	.files = ACO_FILES(&resolver_unbound_conf),
+	.pre_apply_config = unbound_config_preapply_callback,
+);
+
+/*! \brief Destructor for unbound resolver */
+static void unbound_resolver_destroy(void *obj)
+{
+	struct unbound_resolver *resolver = obj;
+
+	if (resolver->context) {
+		ub_ctx_delete(resolver->context);
+	}
+}
+
+/*! \brief Allocator for unbound resolver */
+static struct unbound_resolver *unbound_resolver_alloc(void)
+{
+	struct unbound_resolver *resolver;
+
+	resolver = ao2_alloc_options(sizeof(*resolver), unbound_resolver_destroy, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!resolver) {
+		return NULL;
+	}
+
+	resolver->thread = AST_PTHREADT_NULL;
+
+	resolver->context = ub_ctx_create();
+	if (!resolver->context) {
+		ao2_ref(resolver, -1);
+		return NULL;
+	}
+
+	/* Each async result should be invoked in a separate thread so others are not blocked */
+	ub_ctx_async(resolver->context, 1);
+
+	return resolver;
+}
+
+/*! \brief Resolver thread which waits and handles results */
+static void *unbound_resolver_thread(void *data)
+{
+	struct unbound_resolver *resolver = data;
+
+	ast_debug(1, "Starting processing for unbound resolver\n");
+
+	while (resolver->thread != AST_PTHREADT_STOP) {
+		/* Wait for any results to come in */
+		ast_wait_for_input(ub_fd(resolver->context), -1);
+
+		/* Finally process any results */
+		ub_process(resolver->context);
+	}
+
+	ast_debug(1, "Terminating processing for unbound resolver\n");
+
+	ao2_ref(resolver, -1);
+
+	return NULL;
+}
+
+/*! \brief Start function for the unbound resolver */
+static int unbound_resolver_start(struct unbound_resolver *resolver)
+{
+	int res;
+
+	if (resolver->thread != AST_PTHREADT_NULL) {
+		return 0;
+	}
+
+	ast_debug(1, "Starting thread for unbound resolver\n");
+
+	res = ast_pthread_create(&resolver->thread, NULL, unbound_resolver_thread, ao2_bump(resolver));
+	if (res) {
+		ast_debug(1, "Could not start thread for unbound resolver\n");
+		ao2_ref(resolver, -1);
+	}
+
+	return res;
+}
+
+/*! \brief Stop function for the unbound resolver */
+static void unbound_resolver_stop(struct unbound_resolver *resolver)
+{
+	pthread_t thread;
+
+	if (resolver->thread == AST_PTHREADT_NULL) {
+		return;
+	}
+
+	ast_debug(1, "Stopping processing thread for unbound resolver\n");
+
+	thread = resolver->thread;
+	resolver->thread = AST_PTHREADT_STOP;
+	pthread_kill(thread, SIGURG);
+	pthread_join(thread, NULL);
+
+	ast_debug(1, "Stopped processing thread for unbound resolver\n");
+}
+
+/*! \brief Callback invoked when resolution completes on a query */
+static void unbound_resolver_callback(void *data, int err, struct ub_result *ub_result)
+{
+	struct ast_dns_query *query = data;
+
+	if (!ast_dns_resolver_set_result(query, ub_result->secure, ub_result->bogus, ub_result->rcode,
+		S_OR(ub_result->canonname, ast_dns_query_get_name(query)), ub_result->answer_packet, ub_result->answer_len)) {
+		int i;
+		char *data;
+
+		for (i = 0; (data = ub_result->data[i]); i++) {
+			if (ast_dns_resolver_add_record(query, ub_result->qtype, ub_result->qclass, ub_result->ttl,
+				data, ub_result->len[i])) {
+				break;
+			}
+		}
+	}
+
+	ast_dns_resolver_completed(query);
+	ao2_ref(query, -1);
+	ub_resolve_free(ub_result);
+}
+
+static void unbound_resolver_data_dtor(void *vdoomed)
+{
+	struct unbound_resolver_data *doomed = vdoomed;
+
+	ao2_cleanup(doomed->resolver);
+}
+
+static int unbound_resolver_resolve(struct ast_dns_query *query)
+{
+	struct unbound_config *cfg = ao2_global_obj_ref(globals);
+	struct unbound_resolver_data *data;
+	int res;
+
+	data = ao2_alloc_options(sizeof(*data), unbound_resolver_data_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!data) {
+		ast_log(LOG_ERROR, "Failed to allocate resolver data for resolution of '%s'\n",
+			ast_dns_query_get_name(query));
+		return -1;
+	}
+	data->resolver = ao2_bump(cfg->global->state->resolver);
+	ast_dns_resolver_set_data(query, data);
+
+	res = ub_resolve_async(data->resolver->context, ast_dns_query_get_name(query),
+		ast_dns_query_get_rr_type(query), ast_dns_query_get_rr_class(query),
+		ao2_bump(query), unbound_resolver_callback, &data->id);
+
+	if (res) {
+		ast_log(LOG_ERROR, "Failed to perform async DNS resolution of '%s'\n",
+			ast_dns_query_get_name(query));
+		ao2_ref(query, -1);
+	}
+
+	ao2_ref(data, -1);
+	ao2_ref(cfg, -1);
+	return res;
+}
+
+static int unbound_resolver_cancel(struct ast_dns_query *query)
+{
+	struct unbound_resolver_data *data = ast_dns_resolver_get_data(query);
+	int res;
+
+	res = ub_cancel(data->resolver->context, data->id);
+	if (!res) {
+		/* When this query was started we bumped the ref, now that it has been cancelled we have ownership and
+		 * need to drop it
+		 */
+		ao2_ref(query, -1);
+	}
+
+	return res;
+}
+
+struct ast_dns_resolver unbound_resolver = {
+	.name = "unbound",
+	.priority = 100,
+	.resolve = unbound_resolver_resolve,
+	.cancel = unbound_resolver_cancel,
+};
+
+static void unbound_config_destructor(void *obj)
+{
+	struct unbound_config *cfg = obj;
+
+	ao2_cleanup(cfg->global);
+}
+
+static void unbound_global_config_destructor(void *obj)
+{
+	struct unbound_global_config *global = obj;
+
+	ast_string_field_free_memory(global);
+	ao2_cleanup(global->nameservers);
+	ao2_cleanup(global->state);
+}
+
+static void unbound_config_state_destructor(void *obj)
+{
+	struct unbound_config_state *state = obj;
+
+	if (state->resolver) {
+		unbound_resolver_stop(state->resolver);
+		ao2_ref(state->resolver, -1);
+	}
+}
+
+static void *unbound_config_alloc(void)
+{
+	struct unbound_config *cfg;
+
+	cfg = ao2_alloc_options(sizeof(*cfg), unbound_config_destructor, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!cfg) {
+		return NULL;
+	}
+
+	/* Allocate/initialize memory */
+	cfg->global = ao2_alloc_options(sizeof(*cfg->global), unbound_global_config_destructor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!cfg->global) {
+		goto error;
+	}
+
+	if (ast_string_field_init(cfg->global, 128)) {
+		goto error;
+	}
+
+	return cfg;
+error:
+	ao2_ref(cfg, -1);
+	return NULL;
+}
+
+static int unbound_config_preapply(struct unbound_config *cfg)
+{
+	int res = 0;
+
+	cfg->global->state = ao2_alloc_options(sizeof(*cfg->global->state), unbound_config_state_destructor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!cfg->global->state) {
+		ast_log(LOG_ERROR, "Could not allocate unbound resolver state structure\n");
+		return -1;
+	}
+
+	cfg->global->state->resolver = unbound_resolver_alloc();
+	if (!cfg->global->state->resolver) {
+		ast_log(LOG_ERROR, "Could not create an unbound resolver\n");
+		return -1;
+	}
+
+	ub_ctx_debuglevel(cfg->global->state->resolver->context, cfg->global->debug);
+
+	if (!strcmp(cfg->global->hosts, "system")) {
+		res = ub_ctx_hosts(cfg->global->state->resolver->context, NULL);
+	} else if (!ast_strlen_zero(cfg->global->hosts)) {
+		res = ub_ctx_hosts(cfg->global->state->resolver->context, cfg->global->hosts);
+	}
+
+	if (res) {
+		ast_log(LOG_ERROR, "Failed to set hosts file to '%s' in unbound resolver: %s\n",
+			cfg->global->hosts, ub_strerror(res));
+		return -1;
+	}
+
+	if (cfg->global->nameservers) {
+		struct ao2_iterator it_nameservers;
+		char *nameserver;
+
+		it_nameservers = ao2_iterator_init(cfg->global->nameservers, 0);
+		while (!res && (nameserver = ao2_iterator_next(&it_nameservers))) {
+			res = ub_ctx_set_fwd(cfg->global->state->resolver->context, nameserver);
+
+			if (res) {
+				ast_log(LOG_ERROR, "Failed to add nameserver '%s' to unbound resolver: %s\n",
+					nameserver, ub_strerror(res));
+			}
+			ao2_ref(nameserver, -1);
+		}
+		ao2_iterator_destroy(&it_nameservers);
+		if (res) {
+			return -1;
+		}
+	}
+
+	if (!strcmp(cfg->global->resolv, "system")) {
+		res = ub_ctx_resolvconf(cfg->global->state->resolver->context, NULL);
+	} else if (!ast_strlen_zero(cfg->global->resolv)) {
+		res = ub_ctx_resolvconf(cfg->global->state->resolver->context, cfg->global->resolv);
+	}
+
+	if (res) {
+		ast_log(LOG_ERROR, "Failed to set resolv.conf file to '%s' in unbound resolver: %s\n",
+			cfg->global->resolv, ub_strerror(res));
+		return -1;
+	}
+
+	if (!ast_strlen_zero(cfg->global->ta_file)) {
+		res = ub_ctx_add_ta_file(cfg->global->state->resolver->context, cfg->global->ta_file);
+
+		if (res) {
+			ast_log(LOG_ERROR, "Failed to set trusted anchor file to '%s' in unbound resolver: %s\n",
+				cfg->global->ta_file, ub_strerror(res));
+			return -1;
+		}
+	}
+
+	if (unbound_resolver_start(cfg->global->state->resolver)) {
+		ast_log(LOG_ERROR, "Could not start unbound resolver thread\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+static int unbound_config_apply_default(void)
+{
+	struct unbound_config *cfg;
+
+	cfg = unbound_config_alloc();
+	if (!cfg) {
+		ast_log(LOG_ERROR, "Could not create default configuration for unbound resolver\n");
+		return -1;
+	}
+
+	aco_set_defaults(&global_option, "general", cfg->global);
+
+	if (unbound_config_preapply(cfg)) {
+		ao2_ref(cfg, -1);
+		return -1;
+	}
+
+	ast_verb(1, "Starting unbound resolver using default configuration\n");
+
+	ao2_global_obj_replace_unref(globals, cfg);
+	ao2_ref(cfg, -1);
+
+	return 0;
+}
+
+static int unbound_config_preapply_callback(void)
+{
+	return unbound_config_preapply(aco_pending_config(&cfg_info));
+}
+
+#ifdef TEST_FRAMEWORK
+
+#include "asterisk/dns_naptr.h"
+
+/*!
+ * \brief A DNS record to be used during a test
+ */
+struct dns_record {
+	/*! String representation of the record, as would be found in a file */
+	const char *as_string;
+	/*! The domain this record belongs to */
+	const char *domain;
+	/*! The type of the record */
+	int rr_type;
+	/*! The class of the record */
+	int rr_class;
+	/*! The TTL of the record, in seconds */
+	int ttl;
+	/*! The RDATA of the DNS record */
+	const char *buf;
+	/*! The size of the RDATA */
+	const size_t bufsize;
+	/*! Whether a record checker has visited this record */
+	int visited;
+};
+
+/*!
+ * \brief Resolution function for tests.
+ *
+ * Several tests will have similar setups but will want to make use of a different
+ * means of actually making queries and checking their results. This pluggable
+ * function pointer allows for similar tests to be operated in different ways.
+ *
+ * \param test The test being run
+ * \param domain The domain to look up
+ * \param rr_type The record type to look up
+ * \param rr_class The class of record to look up
+ * \param records All records that exist for the test.
+ * \param num_records Number of records in the records array.
+ *
+ * \retval 0 The test has passed thus far.
+ * \retval -1 The test has failed.
+ */
+typedef int (*resolve_fn)(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, struct dns_record *records, size_t num_records);
+
+/*!
+ * \brief Pluggable function for running a synchronous query and checking its results
+ */
+static int nominal_sync_run(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, struct dns_record *records, size_t num_records)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	const struct ast_dns_record *record;
+	int i;
+
+	/* Start by making sure no records have been visited */
+	for (i = 0; i < num_records; ++i) {
+		records[i].visited = 0;
+	}
+
+	ast_test_status_update(test, "Performing DNS query '%s', type %d\n", domain, rr_type);
+
+	if (ast_dns_resolve(domain, rr_type, rr_class, &result)) {
+		ast_test_status_update(test, "Failed to perform synchronous resolution of domain %s\n", domain);
+		return -1;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "Successful synchronous resolution of domain %s gave NULL result\n", domain);
+		return -1;
+	}
+
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		int match = 0;
+
+		/* Let's make sure this matches one of our known records */
+		for (i = 0; i < num_records; ++i) {
+			if (ast_dns_record_get_rr_type(record) == records[i].rr_type &&
+					ast_dns_record_get_rr_class(record) == records[i].rr_class &&
+					ast_dns_record_get_ttl(record) == records[i].ttl &&
+					!memcmp(ast_dns_record_get_data(record), records[i].buf, records[i].bufsize)) {
+				match = 1;
+				records[i].visited = 1;
+				break;
+			}
+		}
+
+		if (!match) {
+			ast_test_status_update(test, "Unknown DNS record returned from domain %s\n", domain);
+			return -1;
+		}
+	}
+
+	return 0;
+}
+
+/*!
+ * \brief Data required for an asynchronous callback
+ */
+struct async_data {
+	/*! The set of DNS records on a test */
+	struct dns_record *records;
+	/*! The number of DNS records on the test */
+	size_t num_records;
+	/*! Whether an asynchronous query failed */
+	int failed;
+	/*! Indicates the asynchronous query is complete */
+	int complete;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+static void async_data_destructor(void *obj)
+{
+	struct async_data *adata = obj;
+
+	ast_mutex_destroy(&adata->lock);
+	ast_cond_destroy(&adata->cond);
+}
+
+static struct async_data *async_data_alloc(struct dns_record *records, size_t num_records)
+{
+	struct async_data *adata;
+
+	adata = ao2_alloc(sizeof(*adata), async_data_destructor);
+	if (!adata) {
+		return NULL;
+	}
+
+	ast_mutex_init(&adata->lock);
+	ast_cond_init(&adata->cond, NULL);
+	adata->records = records;
+	adata->num_records = num_records;
+
+	return adata;
+}
+
+/*!
+ * \brief Callback for asynchronous queries
+ *
+ * This query will check that the records in the DNS result match
+ * records that the test has created. The success or failure of the
+ * query is indicated through the async_data failed field.
+ *
+ * \param query The DNS query that has been resolved
+ */
+static void async_callback(const struct ast_dns_query *query)
+{
+	struct async_data *adata = ast_dns_query_get_data(query);
+	struct ast_dns_result *result = ast_dns_query_get_result(query);
+	const struct ast_dns_record *record;
+	int i;
+
+	if (!result) {
+		adata->failed = -1;
+		goto end;
+	}
+
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		int match = 0;
+
+		/* Let's make sure this matches one of our known records */
+		for (i = 0; i < adata->num_records; ++i) {
+			if (ast_dns_record_get_rr_type(record) == adata->records[i].rr_type &&
+					ast_dns_record_get_rr_class(record) == adata->records[i].rr_class &&
+					ast_dns_record_get_ttl(record) == adata->records[i].ttl &&
+					!memcmp(ast_dns_record_get_data(record), adata->records[i].buf, adata->records[i].bufsize)) {
+				match = 1;
+				adata->records[i].visited = 1;
+				break;
+			}
+		}
+
+		if (!match) {
+			adata->failed = -1;
+			goto end;
+		}
+	}
+
+end:
+	ast_mutex_lock(&adata->lock);
+	adata->complete = 1;
+	ast_cond_signal(&adata->cond);
+	ast_mutex_unlock(&adata->lock);
+}
+
+/*!
+ * \brief Pluggable function for performing an asynchronous query during a test
+ *
+ * Unlike the synchronous version, this does not check the records, instead leaving
+ * that to be done in the asynchronous callback.
+ */
+static int nominal_async_run(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, struct dns_record *records, size_t num_records)
+{
+	RAII_VAR(struct ast_dns_query_active *, active, NULL, ao2_cleanup);
+	RAII_VAR(struct async_data *, adata, NULL, ao2_cleanup);
+	int i;
+
+	adata = async_data_alloc(records, num_records);
+	if (!adata) {
+		ast_test_status_update(test, "Unable to allocate data for async query\n");
+		return -1;
+	}
+
+	/* Start by making sure no records have been visited */
+	for (i = 0; i < num_records; ++i) {
+		records[i].visited = 0;
+	}
+
+	ast_test_status_update(test, "Performing DNS query '%s', type %d\n", domain, rr_type);
+
+	active = ast_dns_resolve_async(domain, rr_type, rr_class, async_callback, adata);
+	if (!active) {
+		ast_test_status_update(test, "Failed to perform asynchronous resolution of domain %s\n", domain);
+		return -1;
+	}
+
+	ast_mutex_lock(&adata->lock);
+	while (!adata->complete) {
+		ast_cond_wait(&adata->cond, &adata->lock);
+	}
+	ast_mutex_unlock(&adata->lock);
+
+	if (adata->failed) {
+		ast_test_status_update(test, "Unknown DNS record returned from domain %s\n", domain);
+	}
+	return adata->failed;
+}
+
+/*!
+ * \brief Framework for running a nominal DNS test
+ *
+ * Synchronous and asynchronous tests mostly have the same setup, so this function
+ * serves as a common way to set up both types of tests by accepting a pluggable
+ * function to determine which type of lookup is used
+ *
+ * \param test The test being run
+ * \param runner The method for resolving queries on this test
+ */
+static enum ast_test_result_state nominal_test(struct ast_test *test, resolve_fn runner)
+{
+	RAII_VAR(struct unbound_resolver *, resolver, NULL, ao2_cleanup);
+	RAII_VAR(struct unbound_config *, cfg, NULL, ao2_cleanup);
+
+	static const size_t V4_SIZE = sizeof(struct in_addr);
+	static const size_t V6_SIZE = sizeof(struct in6_addr);
+
+	static const char *DOMAIN1 = "goose.feathers";
+	static const char *DOMAIN2 = "duck.feathers";
+
+	static const char *ADDR1 = "127.0.0.2";
+	static const char *ADDR2 = "127.0.0.3";
+	static const char *ADDR3 = "::1";
+	static const char *ADDR4 = "127.0.0.4";
+
+	char addr1_buf[V4_SIZE];
+	char addr2_buf[V4_SIZE];
+	char addr3_buf[V6_SIZE];
+	char addr4_buf[V4_SIZE];
+
+	struct dns_record records [] = {
+		{ "goose.feathers 12345 IN A 127.0.0.2", DOMAIN1, ns_t_a,    ns_c_in, 12345, addr1_buf, V4_SIZE, 0 },
+		{ "goose.feathers 12345 IN A 127.0.0.3", DOMAIN1, ns_t_a,    ns_c_in, 12345, addr2_buf, V4_SIZE, 0 },
+		{ "goose.feathers 12345 IN AAAA ::1",    DOMAIN1, ns_t_aaaa, ns_c_in, 12345, addr3_buf, V6_SIZE, 0 },
+		{ "duck.feathers 12345 IN A 127.0.0.4",  DOMAIN2, ns_t_a,    ns_c_in, 12345, addr4_buf, V4_SIZE, 0 },
+	};
+
+	struct {
+		const char *domain;
+		int rr_type;
+		int rr_class;
+		int visited[ARRAY_LEN(records)];
+	} runs [] = {
+		{ DOMAIN1, ns_t_a,    ns_c_in, { 1, 1, 0, 0 } },
+		{ DOMAIN1, ns_t_aaaa, ns_c_in, { 0, 0, 1, 0 } },
+		{ DOMAIN2, ns_t_a,    ns_c_in, { 0, 0, 0, 1 } },
+	};
+
+	int i;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	inet_pton(AF_INET,  ADDR1, addr1_buf);
+	inet_pton(AF_INET,  ADDR2, addr2_buf);
+	inet_pton(AF_INET6,  ADDR3, addr3_buf);
+	inet_pton(AF_INET, ADDR4, addr4_buf);
+
+	cfg = ao2_global_obj_ref(globals);
+	resolver = ao2_bump(cfg->global->state->resolver);
+
+	ub_ctx_zone_add(resolver->context, DOMAIN1, "static");
+	ub_ctx_zone_add(resolver->context, DOMAIN2, "static");
+
+	for (i = 0; i < ARRAY_LEN(records); ++i) {
+		ub_ctx_data_add(resolver->context, records[i].as_string);
+	}
+
+	for (i = 0; i < ARRAY_LEN(runs); ++i) {
+		int j;
+
+		if (runner(test, runs[i].domain, runs[i].rr_type, runs[i].rr_class, records, ARRAY_LEN(records))) {
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+
+		for (j = 0; j < ARRAY_LEN(records); ++j) {
+			if (records[j].visited != runs[i].visited[j]) {
+				ast_test_status_update(test, "DNS results match unexpected records\n");
+				res = AST_TEST_FAIL;
+				goto cleanup;
+			}
+		}
+	}
+
+cleanup:
+	for (i = 0; i < ARRAY_LEN(records); ++i) {
+		ub_ctx_data_remove(resolver->context, records[i].as_string);
+	}
+	ub_ctx_zone_remove(resolver->context, DOMAIN1);
+	ub_ctx_zone_remove(resolver->context, DOMAIN2);
+
+	return res;
+}
+
+AST_TEST_DEFINE(resolve_sync)
+{
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_sync";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Test nominal synchronous resolution using libunbound";
+		info->description = "This test performs the following:\n"
+			"\t* Set two static A records and one static AAAA record on one domain\n"
+			"\t* Set an A record for a second domain\n"
+			"\t* Perform an A record lookup on the first domain\n"
+			"\t* Ensure that both A records are returned and no AAAA record is returned\n"
+			"\t* Perform an AAAA record lookup on the first domain\n"
+			"\t* Ensure that the AAAA record is returned and no A record is returned\n"
+			"\t* Perform an A record lookup on the second domain\n"
+			"\t* Ensure that the A record from the second domain is returned";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return nominal_test(test, nominal_sync_run);
+}
+
+AST_TEST_DEFINE(resolve_async)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_async";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Test nominal asynchronous resolution using libunbound";
+		info->description = "This test performs the following:\n"
+			"\t* Set two static A records and one static AAAA record on one domain\n"
+			"\t* Set an A record for a second domain\n"
+			"\t* Perform an A record lookup on the first domain\n"
+			"\t* Ensure that both A records are returned and no AAAA record is returned\n"
+			"\t* Perform an AAAA record lookup on the first domain\n"
+			"\t* Ensure that the AAAA record is returned and no A record is returned\n"
+			"\t* Perform an A record lookup on the second domain\n"
+			"\t* Ensure that the A record from the second domain is returned";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return nominal_test(test, nominal_async_run);
+}
+
+typedef int (*off_nominal_resolve_fn)(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, int expected_rcode);
+
+static int off_nominal_sync_run(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, int expected_rcode)
+{
+	struct ast_dns_result *result;
+	int res = 0;
+
+	if (ast_dns_resolve(domain, rr_type, rr_class, &result)) {
+		ast_test_status_update(test, "Failed to perform resolution :(\n");
+		return -1;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "Resolution returned no result\n");
+		return -1;
+	}
+
+	if (ast_dns_result_get_rcode(result) != expected_rcode) {
+		ast_test_status_update(test, "Unexpected rcode from DNS resolution\n");
+		res = -1;
+	}
+
+	if (ast_dns_result_get_records(result)) {
+		ast_test_status_update(test, "DNS resolution returned records unexpectedly\n");
+		res = -1;
+	}
+
+	ast_dns_result_free(result);
+	return res;
+}
+
+/*!
+ * \brief User data for off-nominal async resolution test
+ */
+struct off_nominal_async_data {
+	/*! The DNS result's expected rcode */
+	int expected_rcode;
+	/*! Whether an asynchronous query failed */
+	int failed;
+	/*! Indicates the asynchronous query is complete */
+	int complete;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+static void off_nominal_async_data_destructor(void *obj)
+{
+	struct off_nominal_async_data *adata = obj;
+
+	ast_mutex_destroy(&adata->lock);
+	ast_cond_destroy(&adata->cond);
+}
+
+static struct off_nominal_async_data *off_nominal_async_data_alloc(int expected_rcode)
+{
+	struct off_nominal_async_data *adata;
+
+	adata = ao2_alloc(sizeof(*adata), off_nominal_async_data_destructor);
+	if (!adata) {
+		return NULL;
+	}
+
+	ast_mutex_init(&adata->lock);
+	ast_cond_init(&adata->cond, NULL);
+
+	adata->expected_rcode = expected_rcode;
+
+	return adata;
+}
+
+/*!
+ * \brief Async callback for off-nominal async test
+ *
+ * This test ensures that there is a result present on the query, then it checks
+ * that the rcode on the result is the expected value and that there are no
+ * records on the result.
+ *
+ * Once completed, the testing thread is signaled that the async query has
+ * completed.
+ */
+static void off_nominal_async_callback(const struct ast_dns_query *query)
+{
+	struct off_nominal_async_data *adata = ast_dns_query_get_data(query);
+	struct ast_dns_result *result = ast_dns_query_get_result(query);
+
+	if (!result) {
+		adata->failed = -1;
+		goto end;
+	}
+
+	if (ast_dns_result_get_rcode(result) != adata->expected_rcode) {
+		adata->failed = -1;
+	}
+
+	if (ast_dns_result_get_records(result)) {
+		adata->failed = -1;
+	}
+
+end:
+	ast_mutex_lock(&adata->lock);
+	adata->complete = 1;
+	ast_cond_signal(&adata->cond);
+	ast_mutex_unlock(&adata->lock);
+}
+
+static int off_nominal_async_run(struct ast_test *test, const char *domain, int rr_type,
+		int rr_class, int expected_rcode)
+{
+	RAII_VAR(struct ast_dns_query_active *, active, NULL, ao2_cleanup);
+	RAII_VAR(struct off_nominal_async_data *, adata, NULL, ao2_cleanup);
+
+	adata = off_nominal_async_data_alloc(expected_rcode);
+	if (!adata) {
+		ast_test_status_update(test, "Unable to allocate data for async query\n");
+		return -1;
+	}
+
+	ast_test_status_update(test, "Performing DNS query '%s', type %d\n", domain, rr_type);
+
+	active = ast_dns_resolve_async(domain, rr_type, rr_class, off_nominal_async_callback, adata);
+	if (!active) {
+		ast_test_status_update(test, "Failed to perform asynchronous resolution of domain %s\n", domain);
+		return -1;
+	}
+
+	ast_mutex_lock(&adata->lock);
+	while (!adata->complete) {
+		ast_cond_wait(&adata->cond, &adata->lock);
+	}
+	ast_mutex_unlock(&adata->lock);
+
+	if (adata->failed) {
+		ast_test_status_update(test, "Asynchronous resolution failure %s\n", domain);
+	}
+	return adata->failed;
+}
+
+static enum ast_test_result_state off_nominal_test(struct ast_test *test,
+		off_nominal_resolve_fn runner)
+{
+	RAII_VAR(struct unbound_resolver *, resolver, NULL, ao2_cleanup);
+	RAII_VAR(struct unbound_config *, cfg, NULL, ao2_cleanup);
+
+	static const size_t V4_SIZE = sizeof(struct in_addr);
+
+	static const char *DOMAIN1 = "goose.feathers";
+	static const char *DOMAIN2 = "duck.feathers";
+
+	static const char *ADDR1 = "127.0.0.2";
+
+	char addr1_buf[V4_SIZE];
+
+	struct dns_record records [] = {
+		{ "goose.feathers 12345 IN A 127.0.0.2", DOMAIN1, ns_t_a, ns_c_in, 12345, addr1_buf, V4_SIZE, 0, },
+	};
+
+	int i;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	struct {
+		const char *domain;
+		int rr_type;
+		int rr_class;
+		int rcode;
+	} runs [] = {
+		{ DOMAIN2, ns_t_a,    ns_c_in, ns_r_nxdomain },
+		{ DOMAIN1, ns_t_aaaa, ns_c_in, ns_r_noerror },
+		{ DOMAIN1, ns_t_a,    ns_c_chaos, ns_r_refused },
+	};
+
+	inet_pton(AF_INET,  ADDR1, addr1_buf);
+
+	cfg = ao2_global_obj_ref(globals);
+	resolver = ao2_bump(cfg->global->state->resolver);
+
+	ub_ctx_zone_add(resolver->context, DOMAIN1, "static");
+	ub_ctx_zone_add(resolver->context, DOMAIN2, "static");
+
+	for (i = 0; i < ARRAY_LEN(records); ++i) {
+		ub_ctx_data_add(resolver->context, records[i].as_string);
+	}
+
+	for (i = 0; i < ARRAY_LEN(runs); ++i) {
+		if (runner(test, runs[i].domain, runs[i].rr_type, runs[i].rr_class, runs[i].rcode)) {
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	return res;
+}
+
+AST_TEST_DEFINE(resolve_sync_off_nominal)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_sync_off_nominal";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Test off-nominal synchronous resolution using libunbound";
+		info->description = "This test performs the following:\n"
+			"\t* Attempt a lookup of a non-existent domain\n"
+			"\t* Attempt a lookup of a AAAA record on a domain that contains only A records\n"
+			"\t* Attempt a lookup of an A record on Chaos-net";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, off_nominal_sync_run);
+}
+
+AST_TEST_DEFINE(resolve_async_off_nominal)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_async_off_nominal";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Test off-nominal synchronous resolution using libunbound";
+		info->description = "This test performs the following:\n"
+			"\t* Attempt a lookup of a non-existent domain\n"
+			"\t* Attempt a lookup of a AAAA record on a domain that contains only A records\n"
+			"\t* Attempt a lookup of an A record on Chaos-net";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, off_nominal_async_run);
+}
+
+/*!
+ * \brief Minimal data required to signal the completion of an async resolve
+ */
+struct async_minimal_data {
+	int complete;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+static void async_minimal_data_destructor(void *obj)
+{
+	struct async_minimal_data *adata = obj;
+
+	ast_mutex_destroy(&adata->lock);
+	ast_cond_destroy(&adata->cond);
+}
+
+static struct async_minimal_data *async_minimal_data_alloc(void)
+{
+	struct async_minimal_data *adata;
+
+	adata = ao2_alloc(sizeof(*adata), async_minimal_data_destructor);
+	if (!adata) {
+		return NULL;
+	}
+
+	ast_mutex_init(&adata->lock);
+	ast_cond_init(&adata->cond, NULL);
+
+	return adata;
+}
+
+/*!
+ * \brief Async callback for off-nominal cancellation test.
+ *
+ * This simply signals the testing thread that the query completed
+ */
+static void minimal_callback(const struct ast_dns_query *query)
+{
+	struct async_minimal_data *adata = ast_dns_query_get_data(query);
+
+	ast_mutex_lock(&adata->lock);
+	adata->complete = 1;
+	ast_cond_signal(&adata->cond);
+	ast_mutex_unlock(&adata->lock);
+}
+
+AST_TEST_DEFINE(resolve_cancel_off_nominal)
+{
+	RAII_VAR(struct ast_dns_query_active *, active, NULL, ao2_cleanup);
+	RAII_VAR(struct async_minimal_data *, adata, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_cancel_off_nominal";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Off nominal cancellation test using libunbound";
+		info->description = "This test does the following:\n"
+			"\t* Perform an asynchronous query\n"
+			"\t* Once the query has completed, attempt to cancel it";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	adata = async_minimal_data_alloc();
+	if (!adata) {
+		ast_test_status_update(test, "Failed to allocate necessary data for test\n");
+		return AST_TEST_FAIL;
+	}
+
+	active = ast_dns_resolve_async("crunchy.peanut.butter", ns_t_a, ns_c_in, minimal_callback, adata);
+	if (!active) {
+		ast_test_status_update(test, "Failed to perform asynchronous query\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Wait for async query to complete */
+	ast_mutex_lock(&adata->lock);
+	while (!adata->complete) {
+		ast_cond_wait(&adata->cond, &adata->lock);
+	}
+	ast_mutex_unlock(&adata->lock);
+
+	if (!ast_dns_resolve_cancel(active)) {
+		ast_test_status_update(test, "Successfully canceled completed query\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(resolve_naptr)
+{
+	RAII_VAR(struct unbound_resolver *, resolver, NULL, ao2_cleanup);
+	RAII_VAR(struct unbound_config *, cfg, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+
+	const struct ast_dns_record *record;
+
+	static char * DOMAIN1 = "goose.feathers";
+	int i;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	struct naptr_record {
+		const char *zone_entry;
+		uint16_t order;
+		uint16_t preference;
+		const char *flags;
+		const char *services;
+		const char *regexp;
+		const char *replacement;
+		int visited;
+	} records [] = {
+		{ "goose.feathers 12345 IN NAPTR 100 100 A SIP+D2U \"\" goose.down", 100, 100, "A", "SIP+D2U", "", "goose.down", 0},
+		{ "goose.feathers 12345 IN NAPTR 100 200 A SIP+D2T \"\" duck.down", 100, 200, "A", "SIP+D2T", "", "duck.down", 0},
+		{ "goose.feathers 12345 IN NAPTR 200 100 A SIPS+D2U \"\" pheasant.down", 200, 100, "A", "SIPS+D2U", "", "pheasant.down", 0},
+		{ "goose.feathers 12345 IN NAPTR 200 200 A SIPS+D2T \"\" platypus.fur", 200, 200, "A", "SIPS+D2T", "", "platypus.fur", 0},
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_naptr";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Attempt resolution of NAPTR record";
+		info->description = "This test performs a NAPTR lookup and ensures that\n"
+			"the returned record has the appropriate values set";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	cfg = ao2_global_obj_ref(globals);
+	resolver = ao2_bump(cfg->global->state->resolver);
+
+	ub_ctx_zone_add(resolver->context, DOMAIN1, "static");
+
+	for (i = 0; i < ARRAY_LEN(records); ++i) {
+		ub_ctx_data_add(resolver->context, records[i].zone_entry);
+	}
+
+	if (ast_dns_resolve(DOMAIN1, ns_t_naptr, ns_c_in, &result)) {
+		ast_test_status_update(test, "Failed to resolve domain\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "Successful resolution set a NULL result\n");
+		return AST_TEST_FAIL;
+	}
+
+	record = ast_dns_result_get_records(result);
+	if (!record) {
+		ast_test_status_update(test, "Failed to get any DNS records from the result\n");
+		return AST_TEST_FAIL;
+	}
+
+	i = 0;
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		if (ast_dns_naptr_get_order(record) != records[i].order) {
+			ast_test_status_update(test, "Expected order %hu, got order %hu from NAPTR record\n",
+					records[i].order, ast_dns_naptr_get_order(record));
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_naptr_get_preference(record) != records[i].preference) {
+			ast_test_status_update(test, "Expected preference %hu, got preference %hu from NAPTR record\n",
+					records[i].preference, ast_dns_naptr_get_preference(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_flags(record), records[i].flags)) {
+			ast_test_status_update(test, "Expected flags %s, got flags %s from NAPTR record\n",
+					records[i].flags, ast_dns_naptr_get_flags(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_service(record), records[i].services)) {
+			ast_test_status_update(test, "Expected services %s, got services %s from NAPTR record\n",
+					records[i].services, ast_dns_naptr_get_service(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_regexp(record), records[i].regexp)) {
+			ast_test_status_update(test, "Expected regexp %s, got regexp %s from NAPTR record\n",
+					records[i].regexp, ast_dns_naptr_get_regexp(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_replacement(record), records[i].replacement)) {
+			ast_test_status_update(test, "Expected replacement %s, got replacement %s from NAPTR record\n",
+					records[i].replacement, ast_dns_naptr_get_replacement(record));
+			res = AST_TEST_FAIL;
+		}
+		records[i].visited = 1;
+		++i;
+	}
+
+	if (i != ARRAY_LEN(records)) {
+		ast_test_status_update(test, "Unexpected number of records visited\n");
+		res = AST_TEST_FAIL;
+	}
+
+	for (i = 0; i < ARRAY_LEN(records); ++i) {
+		if (!records[i].visited) {
+			ast_test_status_update(test, "Did not visit all expected NAPTR records\n");
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	return res;
+
+}
+
+AST_TEST_DEFINE(resolve_srv)
+{
+	RAII_VAR(struct unbound_resolver *, resolver, NULL, ao2_cleanup);
+	RAII_VAR(struct unbound_config *, cfg, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	const struct ast_dns_record *record;
+	static const char *DOMAIN1 = "taco.bananas";
+	static const char *DOMAIN1_SRV = "taco.bananas 12345 IN SRV 10 20 5060 sip.taco.bananas";
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolve_srv";
+		info->category = "/res/res_resolver_unbound/";
+		info->summary = "Test synchronous SRV resolution using libunbound";
+		info->description = "This test performs the following:\n"
+			"\t* Set one SRV record on one domain\n"
+			"\t* Perform an SRV lookup on the domain\n"
+			"\t* Ensure that the SRV record returned matches the expected value";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	cfg = ao2_global_obj_ref(globals);
+	resolver = ao2_bump(cfg->global->state->resolver);
+
+	ub_ctx_zone_add(resolver->context, DOMAIN1, "static");
+	ub_ctx_data_add(resolver->context, DOMAIN1_SRV);
+
+	if (ast_dns_resolve(DOMAIN1, ns_t_srv, ns_c_in, &result)) {
+		ast_test_status_update(test, "Failed to synchronously resolve SRV for domain '%s'\n", DOMAIN1);
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	record = ast_dns_result_get_records(result);
+	if (ast_dns_srv_get_priority(record) != 10) {
+		ast_test_status_update(test, "SRV Record returned priority '%d' when we expected 10\n", ast_dns_srv_get_priority(record));
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (ast_dns_srv_get_weight(record) != 20) {
+		ast_test_status_update(test, "SRV Record returned weight '%d' when we expected 20\n", ast_dns_srv_get_weight(record));
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (ast_dns_srv_get_port(record) != 5060) {
+		ast_test_status_update(test, "SRV Record returned port '%d' when we expected 5060\n", ast_dns_srv_get_port(record));
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (strcmp(ast_dns_srv_get_host(record), "sip.taco.bananas")) {
+		ast_test_status_update(test, "SRV Record returned host '%s' when we expected sip.taco.bananas\n", ast_dns_srv_get_host(record));
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ub_ctx_data_remove(resolver->context, DOMAIN1_SRV);
+	ub_ctx_zone_remove(resolver->context, DOMAIN1);
+
+	return res;
+}
+#endif
+
+static int reload_module(void)
+{
+	if (aco_process_config(&cfg_info, 1) == ACO_PROCESS_ERROR) {
+		return AST_MODULE_RELOAD_ERROR;
+	}
+
+	return 0;
+}
+
+static int unload_module(void)
+{
+	aco_info_destroy(&cfg_info);
+	ao2_global_obj_release(globals);
+
+	AST_TEST_UNREGISTER(resolve_sync);
+	AST_TEST_UNREGISTER(resolve_async);
+	AST_TEST_UNREGISTER(resolve_sync_off_nominal);
+	AST_TEST_UNREGISTER(resolve_sync_off_nominal);
+	AST_TEST_UNREGISTER(resolve_cancel_off_nominal);
+	AST_TEST_UNREGISTER(resolve_naptr);
+	AST_TEST_UNREGISTER(resolve_srv);
+	return 0;
+}
+
+static int custom_nameserver_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
+{
+	struct unbound_global_config *global = obj;
+
+	if (!global->nameservers) {
+		global->nameservers = ast_str_container_alloc_options(AO2_ALLOC_OPT_LOCK_NOLOCK, 1);
+		if (!global->nameservers) {
+			return -1;
+		}
+	}
+
+	return ast_str_container_add(global->nameservers, var->value);
+}
+
+static int load_module(void)
+{
+	struct ast_config *cfg;
+	struct ast_flags cfg_flags = { 0, };
+
+	if (aco_info_init(&cfg_info)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	aco_option_register(&cfg_info, "hosts", ACO_EXACT, global_options, "system", OPT_STRINGFIELD_T, 0, STRFLDSET(struct unbound_global_config, hosts));
+	aco_option_register(&cfg_info, "resolv", ACO_EXACT, global_options, "system", OPT_STRINGFIELD_T, 0, STRFLDSET(struct unbound_global_config, resolv));
+	aco_option_register_custom(&cfg_info, "nameserver", ACO_EXACT, global_options, "", custom_nameserver_handler, 0);
+	aco_option_register(&cfg_info, "debug", ACO_EXACT, global_options, "0", OPT_UINT_T, 0, FLDSET(struct unbound_global_config, debug));
+	aco_option_register(&cfg_info, "ta_file", ACO_EXACT, global_options, "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct unbound_global_config, ta_file));
+
+	/* This purposely checks for a configuration file so we don't output an error message in ACO if one is not present */
+	cfg = ast_config_load(resolver_unbound_conf.filename, cfg_flags);
+	if (!cfg) {
+		if (unbound_config_apply_default()) {
+			unload_module();
+			return AST_MODULE_LOAD_DECLINE;
+		}
+	} else {
+		ast_config_destroy(cfg);
+		if (aco_process_config(&cfg_info, 0) == ACO_PROCESS_ERROR) {
+			unload_module();
+			return AST_MODULE_LOAD_DECLINE;
+		}
+	}
+
+	ast_dns_resolver_register(&unbound_resolver);
+
+	ast_module_shutdown_ref(ast_module_info->self);
+
+	AST_TEST_REGISTER(resolve_sync);
+	AST_TEST_REGISTER(resolve_async);
+	AST_TEST_REGISTER(resolve_sync_off_nominal);
+	AST_TEST_REGISTER(resolve_async_off_nominal);
+	AST_TEST_REGISTER(resolve_cancel_off_nominal);
+	AST_TEST_REGISTER(resolve_naptr);
+	AST_TEST_REGISTER(resolve_srv);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Unbound DNS Resolver Support",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND - 4,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_rtp_asterisk.c 1:16.1.1~dfsg-1/res/res_rtp_asterisk.c
--- 1:13.23.1~dfsg-2/res/res_rtp_asterisk.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_rtp_asterisk.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -49,6 +47,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/bio.h>
+#if !defined(OPENSSL_NO_ECDH) && (OPENSSL_VERSION_NUMBER >= 0x10000000L)
+#include <openssl/bn.h>
+#endif
 #ifndef OPENSSL_NO_DH
 #include <openssl/dh.h>
 #endif
@@ -78,7 +79,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/module.h"
 #include "asterisk/rtp_engine.h"
 #include "asterisk/smoother.h"
+#include "asterisk/uuid.h"
 #include "asterisk/test.h"
+#include "asterisk/data_buffer.h"
 #ifdef HAVE_PJPROJECT
 #include "asterisk/res_pjproject.h"
 #endif
@@ -100,6 +103,9 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 #define TURN_STATE_WAIT_TIME 2000
 
+#define DEFAULT_RTP_SEND_BUFFER_SIZE	250
+#define DEFAULT_RTP_RECV_BUFFER_SIZE	20
+
 /*! Full INTRA-frame Request / Fast Update Request (From RFC2032) */
 #define RTCP_PT_FUR     192
 /*! Sender Report (From RFC3550) */
@@ -114,7 +120,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #define RTCP_PT_APP     204
 /* VP8: RTCP Feedback */
 /*! Payload Specific Feed Back (From RFC4585 also RFC5104) */
-#define RTCP_PT_PSFB    206
+#define RTCP_PT_PSFB    AST_RTP_RTCP_PSFB
 
 #define RTP_MTU		1200
 #define DTMF_SAMPLE_RATE_MS    8 /*!< DTMF samples per millisecond */
@@ -143,12 +149,20 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #define SRTP_MASTER_SALT_LEN 14
 #define SRTP_MASTER_LEN (SRTP_MASTER_KEY_LEN + SRTP_MASTER_SALT_LEN)
 
+#define RTP_DTLS_ESTABLISHED -37
+
 enum strict_rtp_state {
 	STRICT_RTP_OPEN = 0, /*! No RTP packets should be dropped, all sources accepted */
 	STRICT_RTP_LEARN,    /*! Accept next packet as source */
 	STRICT_RTP_CLOSED,   /*! Drop all RTP packets not coming from source that was learned */
 };
 
+enum strict_rtp_mode {
+	STRICT_RTP_NO = 0,	/*! Don't adhere to any strict RTP rules */
+	STRICT_RTP_YES,		/*! Strict RTP that restricts packets based on time and sequence number */
+	STRICT_RTP_SEQNO,	/*! Strict RTP that restricts packets based on sequence number */
+};
+
 /*!
  * \brief Strict RTP learning timeout time in milliseconds
  *
@@ -158,7 +172,7 @@ enum strict_rtp_state {
  */
 #define STRICT_RTP_LEARN_TIMEOUT	5000
 
-#define DEFAULT_STRICT_RTP -1	/*!< Enabled */
+#define DEFAULT_STRICT_RTP STRICT_RTP_YES	/*!< Enabled by default */
 #define DEFAULT_ICESUPPORT 1
 
 extern struct ast_srtp_res *res_srtp;
@@ -288,6 +302,16 @@ struct ice_wrap {
 };
 #endif
 
+/*! \brief Structure used for mapping an incoming SSRC to an RTP instance */
+struct rtp_ssrc_mapping {
+	/*! \brief The received SSRC */
+	unsigned int ssrc;
+	/*! True if the SSRC is available.  Otherwise, this is a placeholder mapping until the SSRC is set. */
+	unsigned int ssrc_valid;
+	/*! \brief The RTP instance this SSRC belongs to*/
+	struct ast_rtp_instance *instance;
+};
+
 /*! \brief RTP session description */
 struct ast_rtp {
 	int s;
@@ -295,6 +319,7 @@ struct ast_rtp {
 	struct ast_frame f;
 	unsigned char rawdata[8192 + AST_FRIENDLY_OFFSET];
 	unsigned int ssrc;		/*!< Synchronization source, RFC 3550, page 10. */
+	char cname[AST_UUID_STR_LEN]; /*!< Our local CNAME */
 	unsigned int themssrc;		/*!< Their SSRC */
 	unsigned int themssrc_valid;	/*!< True if their SSRC is available. */
 	unsigned int lastts;
@@ -304,7 +329,10 @@ struct ast_rtp {
 	unsigned int lastitexttimestamp;
 	unsigned int lastotexttimestamp;
 	unsigned int lasteventseqn;
-	int lastrxseqno;                /*!< Last received sequence number */
+	int lastrxseqno;                /*!< Last received sequence number, from the network */
+	int expectedrxseqno;		/*!< Next expected sequence number, from the network */
+	AST_VECTOR(, int) missing_seqno; /*!< A vector of sequence numbers we never received */
+	int expectedseqno;		/*!< Next expected sequence number, from the core */
 	unsigned short seedrxseqno;     /*!< What sequence number did they start with?*/
 	unsigned int seedrxts;          /*!< What RTP timestamp did they start with? */
 	unsigned int rxcount;           /*!< How many packets have we received? */
@@ -352,6 +380,11 @@ struct ast_rtp {
 	struct ast_rtp *bridged;        /*!< Who we are Packet bridged to */
 	unsigned int asymmetric_codec;  /*!< Indicate if asymmetric send/receive codecs are allowed */
 
+	struct ast_rtp_instance *bundled; /*!< The RTP instance we are bundled to */
+	int stream_num; /*!< Stream num for this RTP instance */
+	AST_VECTOR(, struct rtp_ssrc_mapping) ssrc_mapping; /*!< Mappings of SSRC to RTP instances */
+	struct ast_sockaddr bind_address; /*!< Requested bind address for the sockets */
+
 	enum strict_rtp_state strict_rtp_state; /*!< Current state that strict RTP protection is in */
 	struct ast_sockaddr strict_rtp_address;  /*!< Remote address information for strict RTP purposes */
 
@@ -363,6 +396,9 @@ struct ast_rtp {
 
 	struct rtp_red *red;
 
+	struct ast_data_buffer *send_buffer;		/*!< Buffer for storing sent packets for retransmission */
+	struct ast_data_buffer *recv_buffer;		/*!< Buffer for storing received packets for retransmission */
+
 #ifdef HAVE_PJPROJECT
 	ast_cond_t cond;            /*!< ICE/TURN condition for signaling */
 
@@ -480,6 +516,8 @@ struct ast_rtcp {
 	 */
 	char *local_addr_str;
 	enum ast_rtp_instance_rtcp type;
+	/* Buffer for frames created during RTCP interpretation */
+	unsigned char frame_buf[512 + AST_FRIENDLY_OFFSET];
 };
 
 struct rtp_red {
@@ -497,6 +535,12 @@ struct rtp_red {
 	long int prev_ts;
 };
 
+/*! \brief Structure for storing RTP packets for retransmission */
+struct ast_rtp_rtcp_nack_payload {
+	size_t size;		/*!< The size of the payload */
+	unsigned char buf[0];	/*!< The payload data */
+};
+
 AST_LIST_HEAD_NOLOCK(frame_list, ast_frame);
 
 /* Forward Declarations */
@@ -523,6 +567,12 @@ static void ast_rtp_stun_request(struct
 static void ast_rtp_stop(struct ast_rtp_instance *instance);
 static int ast_rtp_qos_set(struct ast_rtp_instance *instance, int tos, int cos, const char* desc);
 static int ast_rtp_sendcng(struct ast_rtp_instance *instance, int level);
+static unsigned int ast_rtp_get_ssrc(struct ast_rtp_instance *instance);
+static const char *ast_rtp_get_cname(struct ast_rtp_instance *instance);
+static void ast_rtp_set_remote_ssrc(struct ast_rtp_instance *instance, unsigned int ssrc);
+static void ast_rtp_set_stream_num(struct ast_rtp_instance *instance, int stream_num);
+static int ast_rtp_extension_enable(struct ast_rtp_instance *instance, enum ast_rtp_extension extension);
+static int ast_rtp_bundle(struct ast_rtp_instance *child, struct ast_rtp_instance *parent);
 
 #if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
 static int ast_rtp_activate(struct ast_rtp_instance *instance);
@@ -636,9 +686,12 @@ static void ast_rtp_ice_add_remote_candi
 		return;
 	}
 
-	if (!rtp->ice_proposed_remote_candidates &&
-			!(rtp->ice_proposed_remote_candidates = ao2_container_alloc(1, NULL, ice_candidate_cmp))) {
-		return;
+	if (!rtp->ice_proposed_remote_candidates) {
+		rtp->ice_proposed_remote_candidates = ao2_container_alloc_list(
+			AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, ice_candidate_cmp);
+		if (!rtp->ice_proposed_remote_candidates) {
+			return;
+		}
 	}
 
 	/* If this is going to exceed the maximum number of ICE candidates don't even add it */
@@ -1053,8 +1106,12 @@ static void ast_rtp_ice_add_cand(struct
 
 	pj_ice_calc_foundation(rtp->ice->real_ice->pool, &foundation, type, addr);
 
-	if (!rtp->ice_local_candidates && !(rtp->ice_local_candidates = ao2_container_alloc(1, NULL, ice_candidate_cmp))) {
-		return;
+	if (!rtp->ice_local_candidates) {
+		rtp->ice_local_candidates = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+			NULL, ice_candidate_cmp);
+		if (!rtp->ice_local_candidates) {
+			return;
+		}
 	}
 
 	if (!(candidate = ao2_alloc(sizeof(*candidate), ast_rtp_ice_candidate_destroy))) {
@@ -1612,45 +1669,32 @@ static int dtls_setup_rtcp(struct ast_rt
 	return dtls_details_initialize(&rtp->rtcp->dtls, rtp->ssl_ctx, rtp->dtls.dtls_setup);
 }
 
-/*! \pre instance is locked */
-static int ast_rtp_dtls_set_configuration(struct ast_rtp_instance *instance, const struct ast_rtp_dtls_cfg *dtls_cfg)
+static const SSL_METHOD *get_dtls_method(void)
 {
-	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	int res;
-#if !defined(OPENSSL_NO_ECDH) && (OPENSSL_VERSION_NUMBER >= 0x10000000L) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
-	EC_KEY *ecdh;
-#endif
-
-	if (!dtls_cfg->enabled) {
-		return 0;
-	}
-
-	if (!ast_rtp_engine_srtp_is_registered()) {
-		ast_log(LOG_ERROR, "SRTP support module is not loaded or available. Try loading res_srtp.so.\n");
-		return -1;
-	}
-
-	if (rtp->ssl_ctx) {
-		return 0;
-	}
-
 #if OPENSSL_VERSION_NUMBER < 0x10002000L || defined(LIBRESSL_VERSION_NUMBER)
-	rtp->ssl_ctx = SSL_CTX_new(DTLSv1_method());
+	return DTLSv1_method();
 #else
-	rtp->ssl_ctx = SSL_CTX_new(DTLS_method());
+	return DTLS_method();
 #endif
-	if (!rtp->ssl_ctx) {
-		return -1;
-	}
+}
 
-	SSL_CTX_set_read_ahead(rtp->ssl_ctx, 1);
+struct dtls_cert_info {
+	EVP_PKEY *private_key;
+	X509 *certificate;
+};
+
+static void configure_dhparams(const struct ast_rtp *rtp, const struct ast_rtp_dtls_cfg *dtls_cfg)
+{
+#if !defined(OPENSSL_NO_ECDH) && (OPENSSL_VERSION_NUMBER >= 0x10000000L) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
+	EC_KEY *ecdh;
+#endif
 
 #ifndef OPENSSL_NO_DH
 	if (!ast_strlen_zero(dtls_cfg->pvtfile)) {
 		BIO *bio = BIO_new_file(dtls_cfg->pvtfile, "r");
-		if (bio != NULL) {
+		if (bio) {
 			DH *dh = PEM_read_bio_DHparams(bio, NULL, NULL, NULL);
-			if (dh != NULL) {
+			if (dh) {
 				if (SSL_CTX_set_tmp_dh(rtp->ssl_ctx, dh)) {
 					long options = SSL_OP_CIPHER_SERVER_PREFERENCE |
 						SSL_OP_SINGLE_DH_USE | SSL_OP_SINGLE_ECDH_USE;
@@ -1663,10 +1707,11 @@ static int ast_rtp_dtls_set_configuratio
 		}
 	}
 #endif /* !OPENSSL_NO_DH */
+
 #if !defined(OPENSSL_NO_ECDH) && (OPENSSL_VERSION_NUMBER >= 0x10000000L) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
 	/* enables AES-128 ciphers, to get AES-256 use NID_secp384r1 */
 	ecdh = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
-	if (ecdh != NULL) {
+	if (ecdh) {
 		if (SSL_CTX_set_tmp_ecdh(rtp->ssl_ctx, ecdh)) {
 			#ifndef SSL_CTRL_SET_ECDH_AUTO
 				#define SSL_CTRL_SET_ECDH_AUTO 94
@@ -1681,6 +1726,256 @@ static int ast_rtp_dtls_set_configuratio
 		EC_KEY_free(ecdh);
 	}
 #endif /* !OPENSSL_NO_ECDH */
+}
+
+#if !defined(OPENSSL_NO_ECDH) && (OPENSSL_VERSION_NUMBER >= 0x10000000L)
+
+static int create_ephemeral_ec_keypair(EVP_PKEY **keypair)
+{
+	EC_KEY *eckey = NULL;
+	EC_GROUP *group = NULL;
+
+	group = EC_GROUP_new_by_curve_name(NID_X9_62_prime256v1);
+	if (!group) {
+		goto error;
+	}
+
+	EC_GROUP_set_asn1_flag(group, OPENSSL_EC_NAMED_CURVE);
+	EC_GROUP_set_point_conversion_form(group, POINT_CONVERSION_UNCOMPRESSED);
+
+	eckey = EC_KEY_new();
+	if (!eckey) {
+		goto error;
+	}
+
+	if (!EC_KEY_set_group(eckey, group)) {
+		goto error;
+	}
+
+	if (!EC_KEY_generate_key(eckey)) {
+		goto error;
+	}
+
+	*keypair = EVP_PKEY_new();
+	if (!*keypair) {
+		goto error;
+	}
+
+	EVP_PKEY_assign_EC_KEY(*keypair, eckey);
+	EC_GROUP_free(group);
+
+	return 0;
+
+error:
+	EC_KEY_free(eckey);
+	EC_GROUP_free(group);
+
+	return -1;
+}
+
+/* From OpenSSL's x509 command */
+#define SERIAL_RAND_BITS 159
+
+static int create_ephemeral_certificate(EVP_PKEY *keypair, X509 **certificate)
+{
+	X509 *cert = NULL;
+	BIGNUM *serial = NULL;
+	X509_NAME *name = NULL;
+
+	cert = X509_new();
+	if (!cert) {
+		goto error;
+	}
+
+	if (!X509_set_version(cert, 2)) {
+		goto error;
+	}
+
+	/* Set the public key */
+	X509_set_pubkey(cert, keypair);
+
+	/* Generate a random serial number */
+	if (!(serial = BN_new())
+	   || !BN_rand(serial, SERIAL_RAND_BITS, -1, 0)
+	   || !BN_to_ASN1_INTEGER(serial, X509_get_serialNumber(cert))) {
+		goto error;
+	}
+
+	/*
+	 * Validity period - Current Chrome & Firefox make it 31 days starting
+	 * with yesterday at the current time, so we will do the same.
+	 */
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+	if (!X509_time_adj_ex(X509_get_notBefore(cert), -1, 0, NULL)
+	   || !X509_time_adj_ex(X509_get_notAfter(cert), 30, 0, NULL)) {
+		goto error;
+	}
+#else
+	if (!X509_time_adj_ex(X509_getm_notBefore(cert), -1, 0, NULL)
+	   || !X509_time_adj_ex(X509_getm_notAfter(cert), 30, 0, NULL)) {
+		goto error;
+	}
+#endif
+
+	/* Set the name and issuer */
+	if (!(name = X509_get_subject_name(cert))
+	   || !X509_NAME_add_entry_by_NID(name, NID_commonName, MBSTRING_ASC,
+									  (unsigned char *) "asterisk", -1, -1, 0)
+	   || !X509_set_issuer_name(cert, name)) {
+		goto error;
+	}
+
+	/* Sign it */
+	if (!X509_sign(cert, keypair, EVP_sha256())) {
+		goto error;
+	}
+
+	*certificate = cert;
+
+	return 0;
+
+error:
+	BN_free(serial);
+	X509_free(cert);
+
+	return -1;
+}
+
+static int create_certificate_ephemeral(struct ast_rtp_instance *instance,
+										const struct ast_rtp_dtls_cfg *dtls_cfg,
+										struct dtls_cert_info *cert_info)
+{
+	/* Make sure these are initialized */
+	cert_info->private_key = NULL;
+	cert_info->certificate = NULL;
+
+	if (create_ephemeral_ec_keypair(&cert_info->private_key)) {
+		ast_log(LOG_ERROR, "Failed to create ephemeral ECDSA keypair\n");
+		goto error;
+	}
+
+	if (create_ephemeral_certificate(cert_info->private_key, &cert_info->certificate)) {
+		ast_log(LOG_ERROR, "Failed to create ephemeral X509 certificate\n");
+		goto error;
+	}
+
+	return 0;
+
+  error:
+	X509_free(cert_info->certificate);
+	EVP_PKEY_free(cert_info->private_key);
+
+	return -1;
+}
+
+#else
+
+static int create_certificate_ephemeral(struct ast_rtp_instance *instance,
+										const struct ast_rtp_dtls_cfg *dtls_cfg,
+										struct dtls_cert_info *cert_info)
+{
+	ast_log(LOG_ERROR, "Your version of OpenSSL does not support ECDSA keys\n");
+	return -1;
+}
+
+#endif /* !OPENSSL_NO_ECDH */
+
+static int create_certificate_from_file(struct ast_rtp_instance *instance,
+										const struct ast_rtp_dtls_cfg *dtls_cfg,
+										struct dtls_cert_info *cert_info)
+{
+	FILE *fp;
+	BIO *certbio = NULL;
+	EVP_PKEY *private_key = NULL;
+	X509 *cert = NULL;
+	char *private_key_file = ast_strlen_zero(dtls_cfg->pvtfile) ? dtls_cfg->certfile : dtls_cfg->pvtfile;
+
+	fp = fopen(private_key_file, "r");
+	if (!fp) {
+		ast_log(LOG_ERROR, "Failed to read private key from file '%s': %s\n", private_key_file, strerror(errno));
+		goto error;
+	}
+
+	if (!PEM_read_PrivateKey(fp, &private_key, NULL, NULL)) {
+		ast_log(LOG_ERROR, "Failed to read private key from PEM file '%s'\n", private_key_file);
+		fclose(fp);
+		goto error;
+	}
+
+	if (fclose(fp)) {
+		ast_log(LOG_ERROR, "Failed to close private key file '%s': %s\n", private_key_file, strerror(errno));
+		goto error;
+	}
+
+	certbio = BIO_new(BIO_s_file());
+	if (!certbio) {
+		ast_log(LOG_ERROR, "Failed to allocate memory for certificate fingerprinting on RTP instance '%p'\n",
+				instance);
+		goto error;
+	}
+
+	if (!BIO_read_filename(certbio, dtls_cfg->certfile)
+	   || !(cert = PEM_read_bio_X509(certbio, NULL, 0, NULL))) {
+		ast_log(LOG_ERROR, "Failed to read certificate from file '%s'\n", dtls_cfg->certfile);
+		goto error;
+	}
+
+	cert_info->private_key = private_key;
+	cert_info->certificate = cert;
+
+	BIO_free_all(certbio);
+
+	return 0;
+
+error:
+	X509_free(cert);
+	BIO_free_all(certbio);
+	EVP_PKEY_free(private_key);
+
+	return -1;
+}
+
+static int load_dtls_certificate(struct ast_rtp_instance *instance,
+								 const struct ast_rtp_dtls_cfg *dtls_cfg,
+								 struct dtls_cert_info *cert_info)
+{
+	if (dtls_cfg->ephemeral_cert) {
+		return create_certificate_ephemeral(instance, dtls_cfg, cert_info);
+	} else if (!ast_strlen_zero(dtls_cfg->certfile)) {
+		return create_certificate_from_file(instance, dtls_cfg, cert_info);
+	} else {
+		return -1;
+	}
+}
+
+/*! \pre instance is locked */
+static int ast_rtp_dtls_set_configuration(struct ast_rtp_instance *instance, const struct ast_rtp_dtls_cfg *dtls_cfg)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct dtls_cert_info cert_info = { 0 };
+	int res;
+
+	if (!dtls_cfg->enabled) {
+		return 0;
+	}
+
+	if (!ast_rtp_engine_srtp_is_registered()) {
+		ast_log(LOG_ERROR, "SRTP support module is not loaded or available. Try loading res_srtp.so.\n");
+		return -1;
+	}
+
+	if (rtp->ssl_ctx) {
+		return 0;
+	}
+
+	rtp->ssl_ctx = SSL_CTX_new(get_dtls_method());
+	if (!rtp->ssl_ctx) {
+		return -1;
+	}
+
+	SSL_CTX_set_read_ahead(rtp->ssl_ctx, 1);
+
+	configure_dhparams(rtp, dtls_cfg);
 
 	rtp->dtls_verify = dtls_cfg->verify;
 
@@ -1699,25 +1994,22 @@ static int ast_rtp_dtls_set_configuratio
 
 	rtp->local_hash = dtls_cfg->hash;
 
-	if (!ast_strlen_zero(dtls_cfg->certfile)) {
-		char *private = ast_strlen_zero(dtls_cfg->pvtfile) ? dtls_cfg->certfile : dtls_cfg->pvtfile;
-		BIO *certbio;
-		X509 *cert = NULL;
+	if (!load_dtls_certificate(instance, dtls_cfg, &cert_info)) {
 		const EVP_MD *type;
 		unsigned int size, i;
 		unsigned char fingerprint[EVP_MAX_MD_SIZE];
 		char *local_fingerprint = rtp->local_fingerprint;
 
-		if (!SSL_CTX_use_certificate_file(rtp->ssl_ctx, dtls_cfg->certfile, SSL_FILETYPE_PEM)) {
-			ast_log(LOG_ERROR, "Specified certificate file '%s' for RTP instance '%p' could not be used\n",
-				dtls_cfg->certfile, instance);
+		if (!SSL_CTX_use_certificate(rtp->ssl_ctx, cert_info.certificate)) {
+			ast_log(LOG_ERROR, "Specified certificate for RTP instance '%p' could not be used\n",
+					instance);
 			return -1;
 		}
 
-		if (!SSL_CTX_use_PrivateKey_file(rtp->ssl_ctx, private, SSL_FILETYPE_PEM) ||
-		    !SSL_CTX_check_private_key(rtp->ssl_ctx)) {
-			ast_log(LOG_ERROR, "Specified private key file '%s' for RTP instance '%p' could not be used\n",
-				private, instance);
+		if (!SSL_CTX_use_PrivateKey(rtp->ssl_ctx, cert_info.private_key)
+		    || !SSL_CTX_check_private_key(rtp->ssl_ctx)) {
+			ast_log(LOG_ERROR, "Specified private key for RTP instance '%p' could not be used\n",
+					instance);
 			return -1;
 		}
 
@@ -1731,22 +2023,9 @@ static int ast_rtp_dtls_set_configuratio
 			return -1;
 		}
 
-		if (!(certbio = BIO_new(BIO_s_file()))) {
-			ast_log(LOG_ERROR, "Failed to allocate memory for certificate fingerprinting on RTP instance '%p'\n",
-				instance);
-			return -1;
-		}
-
-		if (!BIO_read_filename(certbio, dtls_cfg->certfile) ||
-		    !(cert = PEM_read_bio_X509(certbio, NULL, 0, NULL)) ||
-		    !X509_digest(cert, type, fingerprint, &size) ||
-		    !size) {
-			ast_log(LOG_ERROR, "Could not produce fingerprint from certificate '%s' for RTP instance '%p'\n",
-				dtls_cfg->certfile, instance);
-			BIO_free_all(certbio);
-			if (cert) {
-				X509_free(cert);
-			}
+		if (!X509_digest(cert_info.certificate, type, fingerprint, &size) || !size) {
+			ast_log(LOG_ERROR, "Could not produce fingerprint from certificate for RTP instance '%p'\n",
+					instance);
 			return -1;
 		}
 
@@ -1755,10 +2034,10 @@ static int ast_rtp_dtls_set_configuratio
 			local_fingerprint += 3;
 		}
 
-		*(local_fingerprint-1) = 0;
+		*(local_fingerprint - 1) = 0;
 
-		BIO_free_all(certbio);
-		X509_free(cert);
+		EVP_PKEY_free(cert_info.private_key);
+		X509_free(cert_info.certificate);
 	}
 
 	if (!ast_strlen_zero(dtls_cfg->cipher)) {
@@ -2003,6 +2282,12 @@ static struct ast_rtp_engine asterisk_rt
 	.dtls = &ast_rtp_dtls,
 	.activate = ast_rtp_activate,
 #endif
+	.ssrc_get = ast_rtp_get_ssrc,
+	.cname_get = ast_rtp_get_cname,
+	.set_remote_ssrc = ast_rtp_set_remote_ssrc,
+	.set_stream_num = ast_rtp_set_stream_num,
+	.extension_enable = ast_rtp_extension_enable,
+	.bundle = ast_rtp_bundle,
 };
 
 #if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
@@ -2039,6 +2324,23 @@ static void dtls_perform_handshake(struc
 }
 #endif
 
+#if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
+static void dtls_perform_setup(struct dtls_details *dtls)
+{
+	if (!dtls->ssl || !SSL_is_init_finished(dtls->ssl)) {
+		return;
+	}
+
+	SSL_clear(dtls->ssl);
+	if (dtls->dtls_setup == AST_RTP_DTLS_SETUP_PASSIVE) {
+		SSL_set_accept_state(dtls->ssl);
+	} else {
+		SSL_set_connect_state(dtls->ssl);
+	}
+	dtls->connection = AST_RTP_DTLS_CONNECTION_NEW;
+}
+#endif
+
 #ifdef HAVE_PJPROJECT
 static void rtp_learning_start(struct ast_rtp *rtp);
 
@@ -2067,9 +2369,12 @@ static void ast_rtp_on_ice_complete(pj_i
 	}
 
 #if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
+
+	dtls_perform_setup(&rtp->dtls);
 	dtls_perform_handshake(instance, &rtp->dtls, 0);
 
 	if (rtp->rtcp && rtp->rtcp->type == AST_RTP_INSTANCE_RTCP_STANDARD) {
+		dtls_perform_setup(&rtp->rtcp->dtls);
 		dtls_perform_handshake(instance, &rtp->rtcp->dtls, 1);
 	}
 #endif
@@ -2337,33 +2642,109 @@ static int dtls_srtp_renegotiate(const v
 	return 0;
 }
 
-static int dtls_srtp_setup(struct ast_rtp *rtp, struct ast_srtp *srtp, struct ast_rtp_instance *instance, int rtcp)
+static int dtls_srtp_add_local_ssrc(struct ast_rtp *rtp, struct ast_rtp_instance *instance, int rtcp, unsigned int ssrc, int set_remote_policy)
 {
 	unsigned char material[SRTP_MASTER_LEN * 2];
 	unsigned char *local_key, *local_salt, *remote_key, *remote_salt;
 	struct ast_srtp_policy *local_policy, *remote_policy = NULL;
-	struct ast_rtp_instance_stats stats = { 0, };
 	int res = -1;
 	struct dtls_details *dtls = !rtcp ? &rtp->dtls : &rtp->rtcp->dtls;
 
-	/* If a fingerprint is present in the SDP make sure that the peer certificate matches it */
-	if (rtp->dtls_verify & AST_RTP_DTLS_VERIFY_FINGERPRINT) {
-		X509 *certificate;
+	/* Produce key information and set up SRTP */
+	if (!SSL_export_keying_material(dtls->ssl, material, SRTP_MASTER_LEN * 2, "EXTRACTOR-dtls_srtp", 19, NULL, 0, 0)) {
+		ast_log(LOG_WARNING, "Unable to extract SRTP keying material from DTLS-SRTP negotiation on RTP instance '%p'\n",
+			instance);
+		return -1;
+	}
 
-		if (!(certificate = SSL_get_peer_certificate(dtls->ssl))) {
-			ast_log(LOG_WARNING, "No certificate was provided by the peer on RTP instance '%p'\n", instance);
-			return -1;
-		}
+	/* Whether we are acting as a server or client determines where the keys/salts are */
+	if (rtp->dtls.dtls_setup == AST_RTP_DTLS_SETUP_ACTIVE) {
+		local_key = material;
+		remote_key = local_key + SRTP_MASTER_KEY_LEN;
+		local_salt = remote_key + SRTP_MASTER_KEY_LEN;
+		remote_salt = local_salt + SRTP_MASTER_SALT_LEN;
+	} else {
+		remote_key = material;
+		local_key = remote_key + SRTP_MASTER_KEY_LEN;
+		remote_salt = local_key + SRTP_MASTER_KEY_LEN;
+		local_salt = remote_salt + SRTP_MASTER_SALT_LEN;
+	}
 
-		/* If a fingerprint is present in the SDP make sure that the peer certificate matches it */
-		if (rtp->remote_fingerprint[0]) {
-			const EVP_MD *type;
-			unsigned char fingerprint[EVP_MAX_MD_SIZE];
-			unsigned int size;
+	if (!(local_policy = res_srtp_policy->alloc())) {
+		return -1;
+	}
 
-			if (rtp->remote_hash == AST_RTP_DTLS_HASH_SHA1) {
-				type = EVP_sha1();
-			} else if (rtp->remote_hash == AST_RTP_DTLS_HASH_SHA256) {
+	if (res_srtp_policy->set_master_key(local_policy, local_key, SRTP_MASTER_KEY_LEN, local_salt, SRTP_MASTER_SALT_LEN) < 0) {
+		ast_log(LOG_WARNING, "Could not set key/salt information on local policy of '%p' when setting up DTLS-SRTP\n", rtp);
+		goto error;
+	}
+
+	if (res_srtp_policy->set_suite(local_policy, rtp->suite)) {
+		ast_log(LOG_WARNING, "Could not set suite to '%u' on local policy of '%p' when setting up DTLS-SRTP\n", rtp->suite, rtp);
+		goto error;
+	}
+
+	res_srtp_policy->set_ssrc(local_policy, ssrc, 0);
+
+	if (set_remote_policy) {
+		if (!(remote_policy = res_srtp_policy->alloc())) {
+			goto error;
+		}
+
+		if (res_srtp_policy->set_master_key(remote_policy, remote_key, SRTP_MASTER_KEY_LEN, remote_salt, SRTP_MASTER_SALT_LEN) < 0) {
+			ast_log(LOG_WARNING, "Could not set key/salt information on remote policy of '%p' when setting up DTLS-SRTP\n", rtp);
+			goto error;
+		}
+
+		if (res_srtp_policy->set_suite(remote_policy, rtp->suite)) {
+			ast_log(LOG_WARNING, "Could not set suite to '%u' on remote policy of '%p' when setting up DTLS-SRTP\n", rtp->suite, rtp);
+			goto error;
+		}
+
+		res_srtp_policy->set_ssrc(remote_policy, 0, 1);
+	}
+
+	if (ast_rtp_instance_add_srtp_policy(instance, remote_policy, local_policy, rtcp)) {
+		ast_log(LOG_WARNING, "Could not set policies when setting up DTLS-SRTP on '%p'\n", rtp);
+		goto error;
+	}
+
+	res = 0;
+
+error:
+	/* policy->destroy() called even on success to release local reference to these resources */
+	res_srtp_policy->destroy(local_policy);
+
+	if (remote_policy) {
+		res_srtp_policy->destroy(remote_policy);
+	}
+
+	return res;
+}
+
+static int dtls_srtp_setup(struct ast_rtp *rtp, struct ast_rtp_instance *instance, int rtcp)
+{
+	struct dtls_details *dtls = !rtcp ? &rtp->dtls : &rtp->rtcp->dtls;
+	int index;
+
+	/* If a fingerprint is present in the SDP make sure that the peer certificate matches it */
+	if (rtp->dtls_verify & AST_RTP_DTLS_VERIFY_FINGERPRINT) {
+		X509 *certificate;
+
+		if (!(certificate = SSL_get_peer_certificate(dtls->ssl))) {
+			ast_log(LOG_WARNING, "No certificate was provided by the peer on RTP instance '%p'\n", instance);
+			return -1;
+		}
+
+		/* If a fingerprint is present in the SDP make sure that the peer certificate matches it */
+		if (rtp->remote_fingerprint[0]) {
+			const EVP_MD *type;
+			unsigned char fingerprint[EVP_MAX_MD_SIZE];
+			unsigned int size;
+
+			if (rtp->remote_hash == AST_RTP_DTLS_HASH_SHA1) {
+				type = EVP_sha1();
+			} else if (rtp->remote_hash == AST_RTP_DTLS_HASH_SHA256) {
 				type = EVP_sha256();
 			} else {
 				ast_log(LOG_WARNING, "Unsupported fingerprint hash type on RTP instance '%p'\n", instance);
@@ -2383,95 +2764,41 @@ static int dtls_srtp_setup(struct ast_rt
 		X509_free(certificate);
 	}
 
-	/* Ensure that certificate verification was successful */
-	if ((rtp->dtls_verify & AST_RTP_DTLS_VERIFY_CERTIFICATE) && SSL_get_verify_result(dtls->ssl) != X509_V_OK) {
-		ast_log(LOG_WARNING, "Peer certificate on RTP instance '%p' failed verification test\n",
-			instance);
-		return -1;
-	}
-
-	/* Produce key information and set up SRTP */
-	if (!SSL_export_keying_material(dtls->ssl, material, SRTP_MASTER_LEN * 2, "EXTRACTOR-dtls_srtp", 19, NULL, 0, 0)) {
-		ast_log(LOG_WARNING, "Unable to extract SRTP keying material from DTLS-SRTP negotiation on RTP instance '%p'\n",
-			instance);
-		return -1;
-	}
-
-	/* Whether we are acting as a server or client determines where the keys/salts are */
-	if (rtp->dtls.dtls_setup == AST_RTP_DTLS_SETUP_ACTIVE) {
-		local_key = material;
-		remote_key = local_key + SRTP_MASTER_KEY_LEN;
-		local_salt = remote_key + SRTP_MASTER_KEY_LEN;
-		remote_salt = local_salt + SRTP_MASTER_SALT_LEN;
-	} else {
-		remote_key = material;
-		local_key = remote_key + SRTP_MASTER_KEY_LEN;
-		remote_salt = local_key + SRTP_MASTER_KEY_LEN;
-		local_salt = remote_salt + SRTP_MASTER_SALT_LEN;
-	}
-
-	if (!(local_policy = res_srtp_policy->alloc())) {
+	if (dtls_srtp_add_local_ssrc(rtp, instance, rtcp, ast_rtp_instance_get_ssrc(instance), 1)) {
 		return -1;
 	}
 
-	if (res_srtp_policy->set_master_key(local_policy, local_key, SRTP_MASTER_KEY_LEN, local_salt, SRTP_MASTER_SALT_LEN) < 0) {
-		ast_log(LOG_WARNING, "Could not set key/salt information on local policy of '%p' when setting up DTLS-SRTP\n", rtp);
-		goto error;
-	}
-
-	if (res_srtp_policy->set_suite(local_policy, rtp->suite)) {
-		ast_log(LOG_WARNING, "Could not set suite to '%u' on local policy of '%p' when setting up DTLS-SRTP\n", rtp->suite, rtp);
-		goto error;
-	}
-
-	if (ast_rtp_instance_get_stats(instance, &stats, AST_RTP_INSTANCE_STAT_LOCAL_SSRC)) {
-		goto error;
-	}
-
-	res_srtp_policy->set_ssrc(local_policy, stats.local_ssrc, 0);
-
-	if (!(remote_policy = res_srtp_policy->alloc())) {
-		goto error;
-	}
-
-	if (res_srtp_policy->set_master_key(remote_policy, remote_key, SRTP_MASTER_KEY_LEN, remote_salt, SRTP_MASTER_SALT_LEN) < 0) {
-		ast_log(LOG_WARNING, "Could not set key/salt information on remote policy of '%p' when setting up DTLS-SRTP\n", rtp);
-		goto error;
-	}
-
-	if (res_srtp_policy->set_suite(remote_policy, rtp->suite)) {
-		ast_log(LOG_WARNING, "Could not set suite to '%u' on remote policy of '%p' when setting up DTLS-SRTP\n", rtp->suite, rtp);
-		goto error;
-	}
+	for (index = 0; index < AST_VECTOR_SIZE(&rtp->ssrc_mapping); ++index) {
+		struct rtp_ssrc_mapping *mapping = AST_VECTOR_GET_ADDR(&rtp->ssrc_mapping, index);
 
-	res_srtp_policy->set_ssrc(remote_policy, 0, 1);
-
-	if (ast_rtp_instance_add_srtp_policy(instance, remote_policy, local_policy, rtcp)) {
-		ast_log(LOG_WARNING, "Could not set policies when setting up DTLS-SRTP on '%p'\n", rtp);
-		goto error;
+		if (dtls_srtp_add_local_ssrc(rtp, instance, rtcp, ast_rtp_instance_get_ssrc(mapping->instance), 0)) {
+			return -1;
+		}
 	}
 
 	if (rtp->rekey) {
 		ao2_ref(instance, +1);
 		if ((rtp->rekeyid = ast_sched_add(rtp->sched, rtp->rekey * 1000, dtls_srtp_renegotiate, instance)) < 0) {
 			ao2_ref(instance, -1);
-			goto error;
+			return -1;
 		}
 	}
 
-	res = 0;
-
-error:
-	/* policy->destroy() called even on success to release local reference to these resources */
-	res_srtp_policy->destroy(local_policy);
+	return 0;
+}
+#endif
 
-	if (remote_policy) {
-		res_srtp_policy->destroy(remote_policy);
-	}
+/*! \brief Helper function to compare an elem in a vector by value */
+static int compare_by_value(int elem, int value)
+{
+	return elem - value;
+}
 
-	return res;
+/*! \brief Helper function to find an elem in a vector by value */
+static int find_by_value(int elem, int value)
+{
+	return elem == value;
 }
-#endif
 
 static int rtcp_mux(struct ast_rtp *rtp, const unsigned char *packet)
 {
@@ -2511,8 +2838,9 @@ static int __rtp_recvfrom(struct ast_rtp
 {
 	int len;
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	struct ast_srtp *srtp = ast_rtp_instance_get_srtp(instance, rtcp);
+#if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
 	char *in = buf;
+#endif
 #ifdef HAVE_PJPROJECT
 	struct ast_sockaddr *loop = rtcp ? &rtp->rtcp_loop : &rtp->rtp_loop;
 #endif
@@ -2573,7 +2901,11 @@ static int __rtp_recvfrom(struct ast_rtp
 			/* Any further connections will be existing since this is now established */
 			dtls->connection = AST_RTP_DTLS_CONNECTION_EXISTING;
 			/* Use the keying material to set up key/salt information */
-			res = dtls_srtp_setup(rtp, srtp, instance, rtcp);
+			if ((res = dtls_srtp_setup(rtp, instance, rtcp))) {
+				return res;
+			}
+			/* Notify that dtls has been established */
+			res = RTP_DTLS_ESTABLISHED;
 		} else {
 			/* Since we've sent additional traffic start the timeout timer for retransmission */
 			dtls_srtp_start_timeout_timer(instance, rtp, rtcp);
@@ -2639,11 +2971,6 @@ static int __rtp_recvfrom(struct ast_rtp
 	}
 #endif
 
-	if ((*in & 0xC0) && res_srtp && srtp && res_srtp->unprotect(
-		    srtp, buf, &len, rtcp || rtcp_mux(rtp, buf)) < 0) {
-	   return -1;
-	}
-
 	return len;
 }
 
@@ -2665,7 +2992,9 @@ static int __rtp_sendto(struct ast_rtp_i
 	int len = size;
 	void *temp = buf;
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	struct ast_srtp *srtp = ast_rtp_instance_get_srtp(instance, rtcp);
+	struct ast_rtp_instance *transport = rtp->bundled ? rtp->bundled : instance;
+	struct ast_rtp *transport_rtp = ast_rtp_instance_get_data(transport);
+	struct ast_srtp *srtp = ast_rtp_instance_get_srtp(transport, rtcp);
 	int res;
 
 	*via_ice = 0;
@@ -2675,7 +3004,7 @@ static int __rtp_sendto(struct ast_rtp_i
 	}
 
 #ifdef HAVE_PJPROJECT
-	if (rtp->ice) {
+	if (transport_rtp->ice) {
 		enum ast_rtp_ice_component_type component = rtcp ? AST_RTP_ICE_COMPONENT_RTCP : AST_RTP_ICE_COMPONENT_RTP;
 		pj_status_t status;
 		struct ice_wrap *ice;
@@ -2688,12 +3017,16 @@ static int __rtp_sendto(struct ast_rtp_i
 		pj_thread_register_check();
 
 		/* Release the instance lock to avoid deadlock with PJPROJECT group lock */
-		ice = rtp->ice;
+		ice = transport_rtp->ice;
 		ao2_ref(ice, +1);
-		ao2_unlock(instance);
+		if (instance == transport) {
+			ao2_unlock(instance);
+		}
 		status = pj_ice_sess_send_data(ice->real_ice, component, temp, len);
 		ao2_ref(ice, -1);
-		ao2_lock(instance);
+		if (instance == transport) {
+			ao2_lock(instance);
+		}
 		if (status == PJ_SUCCESS) {
 			*via_ice = 1;
 			return len;
@@ -2701,7 +3034,7 @@ static int __rtp_sendto(struct ast_rtp_i
 	}
 #endif
 
-	res = ast_sendto(rtcp ? rtp->rtcp->s : rtp->s, temp, len, flags, sa);
+	res = ast_sendto(rtcp ? transport_rtp->rtcp->s : transport_rtp->s, temp, len, flags, sa);
 	if (res > 0) {
 		ast_rtp_instance_set_last_tx(instance, time(NULL));
 	}
@@ -2834,27 +3167,31 @@ static int rtp_learning_rtp_seq_update(s
 		info->received = ast_tvnow();
 	}
 
-	switch (info->stream_type) {
-	case AST_MEDIA_TYPE_UNKNOWN:
-	case AST_MEDIA_TYPE_AUDIO:
-		/*
-		 * Protect against packet floods by checking that we
-		 * received the packet sequence in at least the minimum
-		 * allowed time.
-		 */
-		if (ast_tvzero(info->received)) {
-			info->received = ast_tvnow();
-		} else if (!info->packets
-			&& ast_tvdiff_ms(ast_tvnow(), info->received) < learning_min_duration) {
-			/* Packet flood; reset */
-			info->packets = learning_min_sequential - 1;
-			info->received = ast_tvnow();
+	/* Only check time if strictrtp is set to yes. Otherwise, we only needed to check seqno */
+	if (strictrtp == STRICT_RTP_YES) {
+		switch (info->stream_type) {
+		case AST_MEDIA_TYPE_UNKNOWN:
+		case AST_MEDIA_TYPE_AUDIO:
+			/*
+			 * Protect against packet floods by checking that we
+			 * received the packet sequence in at least the minimum
+			 * allowed time.
+			 */
+			if (ast_tvzero(info->received)) {
+				info->received = ast_tvnow();
+			} else if (!info->packets
+				&& ast_tvdiff_ms(ast_tvnow(), info->received) < learning_min_duration) {
+				/* Packet flood; reset */
+				info->packets = learning_min_sequential - 1;
+				info->received = ast_tvnow();
+			}
+			break;
+		case AST_MEDIA_TYPE_VIDEO:
+		case AST_MEDIA_TYPE_IMAGE:
+		case AST_MEDIA_TYPE_TEXT:
+		case AST_MEDIA_TYPE_END:
+			break;
 		}
-		break;
-	case AST_MEDIA_TYPE_VIDEO:
-	case AST_MEDIA_TYPE_IMAGE:
-	case AST_MEDIA_TYPE_TEXT:
-		break;
 	}
 
 	info->max_seq = seq;
@@ -3145,31 +3482,18 @@ static int ice_create(struct ast_rtp_ins
 }
 #endif
 
-/*! \pre instance is locked */
-static int ast_rtp_new(struct ast_rtp_instance *instance,
-		       struct ast_sched_context *sched, struct ast_sockaddr *addr,
-		       void *data)
+static int rtp_allocate_transport(struct ast_rtp_instance *instance, struct ast_rtp *rtp)
 {
-	struct ast_rtp *rtp = NULL;
 	int x, startplace;
 
-	/* Create a new RTP structure to hold all of our data */
-	if (!(rtp = ast_calloc(1, sizeof(*rtp)))) {
-		return -1;
-	}
-
-	/* Set default parameters on the newly created RTP structure */
-	rtp->ssrc = ast_random();
-	rtp->seqno = ast_random() & 0x7fff;
 	rtp->strict_rtp_state = (strictrtp ? STRICT_RTP_CLOSED : STRICT_RTP_OPEN);
 
 	/* Create a new socket for us to listen on and use */
 	if ((rtp->s =
 	     create_new_socket("RTP",
-			       ast_sockaddr_is_ipv4(addr) ? AF_INET  :
-			       ast_sockaddr_is_ipv6(addr) ? AF_INET6 : -1)) < 0) {
+			       ast_sockaddr_is_ipv4(&rtp->bind_address) ? AF_INET  :
+			       ast_sockaddr_is_ipv6(&rtp->bind_address) ? AF_INET6 : -1)) < 0) {
 		ast_log(LOG_WARNING, "Failed to create a new socket for RTP instance '%p'\n", instance);
-		ast_free(rtp);
 		return -1;
 	}
 
@@ -3179,11 +3503,12 @@ static int ast_rtp_new(struct ast_rtp_in
 	startplace = x;
 
 	for (;;) {
-		ast_sockaddr_set_port(addr, x);
+		ast_sockaddr_set_port(&rtp->bind_address, x);
 		/* Try to bind, this will tell us whether the port is available or not */
-		if (!ast_bind(rtp->s, addr)) {
+		if (!ast_bind(rtp->s, &rtp->bind_address)) {
 			ast_debug(1, "Allocated port %d for RTP instance '%p'\n", x, instance);
-			ast_rtp_instance_set_local_address(instance, addr);
+			ast_rtp_instance_set_local_address(instance, &rtp->bind_address);
+			ast_test_suite_event_notify("RTP_PORT_ALLOCATED", "Port: %d", x);
 			break;
 		}
 
@@ -3196,7 +3521,7 @@ static int ast_rtp_new(struct ast_rtp_in
 		if (x == startplace || (errno != EADDRINUSE && errno != EACCES)) {
 			ast_log(LOG_ERROR, "Oh dear... we couldn't allocate a port for RTP instance '%p'\n", instance);
 			close(rtp->s);
-			ast_free(rtp);
+			rtp->s = -1;
 			return -1;
 		}
 	}
@@ -3207,40 +3532,31 @@ static int ast_rtp_new(struct ast_rtp_in
 
 	generate_random_string(rtp->local_ufrag, sizeof(rtp->local_ufrag));
 	generate_random_string(rtp->local_passwd, sizeof(rtp->local_passwd));
-#endif
-	ast_rtp_instance_set_data(instance, rtp);
-#ifdef HAVE_PJPROJECT
+
 	/* Create an ICE session for ICE negotiation */
 	if (icesupport) {
 		rtp->ice_num_components = 2;
-		ast_debug(3, "Creating ICE session %s (%d) for RTP instance '%p'\n", ast_sockaddr_stringify(addr), x, instance);
-		if (ice_create(instance, addr, x, 0)) {
+		ast_debug(3, "Creating ICE session %s (%d) for RTP instance '%p'\n", ast_sockaddr_stringify(&rtp->bind_address), x, instance);
+		if (ice_create(instance, &rtp->bind_address, x, 0)) {
 			ast_log(LOG_NOTICE, "Failed to create ICE session\n");
 		} else {
 			rtp->ice_port = x;
-			ast_sockaddr_copy(&rtp->ice_original_rtp_addr, addr);
+			ast_sockaddr_copy(&rtp->ice_original_rtp_addr, &rtp->bind_address);
 		}
 	}
 #endif
-	/* Record any information we may need */
-	rtp->sched = sched;
 
 #if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
 	rtp->rekeyid = -1;
 	rtp->dtls.timeout_timer = -1;
 #endif
 
-	rtp->f.subclass.format = ao2_bump(ast_format_none);
-	rtp->lastrxformat = ao2_bump(ast_format_none);
-	rtp->lasttxformat = ao2_bump(ast_format_none);
-
 	return 0;
 }
 
-/*! \pre instance is locked */
-static int ast_rtp_destroy(struct ast_rtp_instance *instance)
+static void rtp_deallocate_transport(struct ast_rtp_instance *instance, struct ast_rtp *rtp)
 {
-	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	int saved_rtp_s = rtp->s;
 #ifdef HAVE_PJPROJECT
 	struct timeval wait = ast_tvadd(ast_tvnow(), ast_samp2tv(TURN_STATE_WAIT_TIME, 1000));
 	struct timespec ts = { .tv_sec = wait.tv_sec, .tv_nsec = wait.tv_usec * 1000, };
@@ -3250,37 +3566,18 @@ static int ast_rtp_destroy(struct ast_rt
 	ast_rtp_dtls_stop(instance);
 #endif
 
-	/* Destroy the smoother that was smoothing out audio if present */
-	if (rtp->smoother) {
-		ast_smoother_free(rtp->smoother);
-	}
-
 	/* Close our own socket so we no longer get packets */
 	if (rtp->s > -1) {
 		close(rtp->s);
+		rtp->s = -1;
 	}
 
 	/* Destroy RTCP if it was being used */
-	if (rtp->rtcp) {
-		/*
-		 * It is not possible for there to be an active RTCP scheduler
-		 * entry at this point since it holds a reference to the
-		 * RTP instance while it's active.
-		 */
-		if (rtp->rtcp->s > -1 && rtp->s != rtp->rtcp->s) {
+	if (rtp->rtcp && rtp->rtcp->s > -1) {
+		if (saved_rtp_s != rtp->rtcp->s) {
 			close(rtp->rtcp->s);
 		}
-		ast_free(rtp->rtcp->local_addr_str);
-		ast_free(rtp->rtcp);
-	}
-
-	/* Destroy RED if it was being used */
-	if (rtp->red) {
-		ao2_unlock(instance);
-		AST_SCHED_DEL(rtp->sched, rtp->red->schedid);
-		ao2_lock(instance);
-		ast_free(rtp->red);
-		rtp->red = NULL;
+		rtp->rtcp->s = -1;
 	}
 
 #ifdef HAVE_PJPROJECT
@@ -3301,6 +3598,7 @@ static int ast_rtp_destroy(struct ast_rt
 		while (rtp->turn_state != PJ_TURN_STATE_DESTROYING) {
 			ast_cond_timedwait(&rtp->cond, ao2_object_get_lockaddr(instance), &ts);
 		}
+		rtp->turn_rtp = NULL;
 	}
 
 	/* Destroy the RTCP TURN relay if being used */
@@ -3314,6 +3612,7 @@ static int ast_rtp_destroy(struct ast_rt
 		while (rtp->turn_state != PJ_TURN_STATE_DESTROYING) {
 			ast_cond_timedwait(&rtp->cond, ao2_object_get_lockaddr(instance), &ts);
 		}
+		rtp->turn_rtcp = NULL;
 	}
 
 	/* Destroy any ICE session */
@@ -3322,10 +3621,12 @@ static int ast_rtp_destroy(struct ast_rt
 	/* Destroy any candidates */
 	if (rtp->ice_local_candidates) {
 		ao2_ref(rtp->ice_local_candidates, -1);
+		rtp->ice_local_candidates = NULL;
 	}
 
 	if (rtp->ice_active_remote_candidates) {
 		ao2_ref(rtp->ice_active_remote_candidates, -1);
+		rtp->ice_active_remote_candidates = NULL;
 	}
 
 	if (rtp->ioqueue) {
@@ -3337,38 +3638,145 @@ static int ast_rtp_destroy(struct ast_rt
 		ao2_unlock(instance);
 		rtp_ioqueue_thread_remove(rtp->ioqueue);
 		ao2_lock(instance);
+		rtp->ioqueue = NULL;
 	}
 #endif
+}
 
-	ao2_cleanup(rtp->lasttxformat);
-	ao2_cleanup(rtp->lastrxformat);
-	ao2_cleanup(rtp->f.subclass.format);
+/*! \pre instance is locked */
+static int ast_rtp_new(struct ast_rtp_instance *instance,
+		       struct ast_sched_context *sched, struct ast_sockaddr *addr,
+		       void *data)
+{
+	struct ast_rtp *rtp = NULL;
 
-#ifdef HAVE_PJPROJECT
-	/* Destroy synchronization items */
-	ast_cond_destroy(&rtp->cond);
-#endif
+	/* Create a new RTP structure to hold all of our data */
+	if (!(rtp = ast_calloc(1, sizeof(*rtp)))) {
+		return -1;
+	}
 
-	/* Finally destroy ourselves */
-	ast_free(rtp);
+	/* Set default parameters on the newly created RTP structure */
+	rtp->ssrc = ast_random();
+	ast_uuid_generate_str(rtp->cname, sizeof(rtp->cname));
+	rtp->seqno = ast_random() & 0x7fff;
+	rtp->expectedrxseqno = -1;
+	rtp->expectedseqno = -1;
+	rtp->sched = sched;
+	ast_sockaddr_copy(&rtp->bind_address, addr);
 
-	return 0;
-}
+	/* Transport creation operations can grab the RTP data from the instance, so set it */
+	ast_rtp_instance_set_data(instance, rtp);
+
+	if (rtp_allocate_transport(instance, rtp)) {
+		return -1;
+	}
+
+	if (AST_VECTOR_INIT(&rtp->ssrc_mapping, 1)) {
+		return -1;
+	}
+
+	rtp->f.subclass.format = ao2_bump(ast_format_none);
+	rtp->lastrxformat = ao2_bump(ast_format_none);
+	rtp->lasttxformat = ao2_bump(ast_format_none);
+	rtp->stream_num = -1;
 
-/*! \pre instance is locked */
-static int ast_rtp_dtmf_mode_set(struct ast_rtp_instance *instance, enum ast_rtp_dtmf_mode dtmf_mode)
-{
-	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	rtp->dtmfmode = dtmf_mode;
 	return 0;
 }
 
+/*!
+ * \brief SSRC mapping comparator for AST_VECTOR_REMOVE_CMP_UNORDERED()
+ *
+ * \param elem Element to compare against
+ * \param value Value to compare with the vector element.
+ *
+ * \return 0 if element does not match.
+ * \return Non-zero if element matches.
+ */
+#define SSRC_MAPPING_ELEM_CMP(elem, value) ((elem).instance == (value))
+
 /*! \pre instance is locked */
-static enum ast_rtp_dtmf_mode ast_rtp_dtmf_mode_get(struct ast_rtp_instance *instance)
+static int ast_rtp_destroy(struct ast_rtp_instance *instance)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	return rtp->dtmfmode;
-}
+
+	if (rtp->bundled) {
+		struct ast_rtp *bundled_rtp;
+
+		/* We can't hold our instance lock while removing ourselves from the parent */
+		ao2_unlock(instance);
+
+		ao2_lock(rtp->bundled);
+		bundled_rtp = ast_rtp_instance_get_data(rtp->bundled);
+		AST_VECTOR_REMOVE_CMP_UNORDERED(&bundled_rtp->ssrc_mapping, instance, SSRC_MAPPING_ELEM_CMP, AST_VECTOR_ELEM_CLEANUP_NOOP);
+		ao2_unlock(rtp->bundled);
+
+		ao2_lock(instance);
+		ao2_ref(rtp->bundled, -1);
+	}
+
+	rtp_deallocate_transport(instance, rtp);
+
+	/* Destroy the smoother that was smoothing out audio if present */
+	if (rtp->smoother) {
+		ast_smoother_free(rtp->smoother);
+	}
+
+	/* Destroy RTCP if it was being used */
+	if (rtp->rtcp) {
+		/*
+		 * It is not possible for there to be an active RTCP scheduler
+		 * entry at this point since it holds a reference to the
+		 * RTP instance while it's active.
+		 */
+		ast_free(rtp->rtcp->local_addr_str);
+		ast_free(rtp->rtcp);
+	}
+
+	/* Destroy RED if it was being used */
+	if (rtp->red) {
+		ao2_unlock(instance);
+		AST_SCHED_DEL(rtp->sched, rtp->red->schedid);
+		ao2_lock(instance);
+		ast_free(rtp->red);
+		rtp->red = NULL;
+	}
+
+	/* Destroy the send buffer if it was being used */
+	if (rtp->send_buffer) {
+		ast_data_buffer_free(rtp->send_buffer);
+	}
+
+	/* Destroy the recv buffer if it was being used */
+	if (rtp->recv_buffer) {
+		ast_data_buffer_free(rtp->recv_buffer);
+	}
+
+	ao2_cleanup(rtp->lasttxformat);
+	ao2_cleanup(rtp->lastrxformat);
+	ao2_cleanup(rtp->f.subclass.format);
+	AST_VECTOR_FREE(&rtp->ssrc_mapping);
+	AST_VECTOR_FREE(&rtp->missing_seqno);
+
+	/* Finally destroy ourselves */
+	ast_free(rtp);
+
+	return 0;
+}
+
+/*! \pre instance is locked */
+static int ast_rtp_dtmf_mode_set(struct ast_rtp_instance *instance, enum ast_rtp_dtmf_mode dtmf_mode)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	rtp->dtmfmode = dtmf_mode;
+	return 0;
+}
+
+/*! \pre instance is locked */
+static enum ast_rtp_dtmf_mode ast_rtp_dtmf_mode_get(struct ast_rtp_instance *instance)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	return rtp->dtmfmode;
+}
 
 /*! \pre instance is locked */
 static int ast_rtp_dtmf_begin(struct ast_rtp_instance *instance, char digit)
@@ -3403,7 +3811,7 @@ static int ast_rtp_dtmf_begin(struct ast
 	}
 
 	/* Grab the payload that they expect the RFC2833 packet to be received in */
-	payload = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(instance), 0, NULL, AST_RTP_DTMF);
+	payload = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(instance), 0, NULL, AST_RTP_DTMF);
 
 	rtp->dtmfmute = ast_tvadd(ast_tvnow(), ast_tv(0, 500000));
 	rtp->send_duration = 160;
@@ -3597,14 +4005,11 @@ static void ast_rtp_change_source(struct
 	struct ast_srtp *rtcp_srtp = ast_rtp_instance_get_srtp(instance, 1);
 	unsigned int ssrc = ast_random();
 
-	if (!rtp->lastts) {
-		ast_debug(3, "Not changing SSRC since we haven't sent any RTP yet\n");
-		return;
+	if (rtp->lastts) {
+		/* We simply set this bit so that the next packet sent will have the marker bit turned on */
+		ast_set_flag(rtp, FLAG_NEED_MARKER_BIT);
 	}
 
-	/* We simply set this bit so that the next packet sent will have the marker bit turned on */
-	ast_set_flag(rtp, FLAG_NEED_MARKER_BIT);
-
 	ast_debug(3, "Changing ssrc from %u to %u due to a source change\n", rtp->ssrc, ssrc);
 
 	if (srtp) {
@@ -3617,6 +4022,9 @@ static void ast_rtp_change_source(struct
 
 	rtp->ssrc = ssrc;
 
+	/* Since the source is changing, we don't know what sequence number to expect next */
+	rtp->expectedrxseqno = -1;
+
 	return;
 }
 
@@ -3712,39 +4120,24 @@ static void calculate_lost_packet_statis
 	rtp->rtcp->rxlost_count++;
 }
 
-/*!
- * \brief Send RTCP SR or RR report
- *
- * \pre instance is locked
- */
-static int ast_rtcp_write_report(struct ast_rtp_instance *instance, int sr)
+static int ast_rtcp_generate_report(struct ast_rtp_instance *instance, unsigned char *rtcpheader,
+		struct ast_rtp_rtcp_report *rtcp_report, int *sr)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	RAII_VAR(struct ast_json *, message_blob, NULL, ast_json_unref);
-	int res;
 	int len = 0;
 	struct timeval now;
 	unsigned int now_lsw;
 	unsigned int now_msw;
-	unsigned int *rtcpheader;
 	unsigned int lost_packets;
 	int fraction_lost;
 	struct timeval dlsr = { 0, };
-	char bdata[512];
-	int rate = rtp_get_rate(rtp->f.subclass.format);
-	int ice;
-	int header_offset = 0;
-	struct ast_sockaddr remote_address = { { 0, } };
 	struct ast_rtp_rtcp_report_block *report_block = NULL;
-	RAII_VAR(struct ast_rtp_rtcp_report *, rtcp_report,
-			ast_rtp_rtcp_report_alloc(rtp->themssrc_valid ? 1 : 0),
-			ao2_cleanup);
 
 	if (!rtp || !rtp->rtcp) {
 		return 0;
 	}
 
-	if (ast_sockaddr_isnull(&rtp->rtcp->them)) {  /* This'll stop rtcp for this rtp session */
+	if (ast_sockaddr_isnull(&rtp->rtcp->them)) { /* This'll stop rtcp for this rtp session */
 		/* RTCP was stopped. */
 		return 0;
 	}
@@ -3753,14 +4146,16 @@ static int ast_rtcp_write_report(struct
 		return 1;
 	}
 
+	*sr = rtp->txcount > rtp->rtcp->lastsrtxcount ? 1 : 0;
+
 	/* Compute statistics */
 	calculate_lost_packet_statistics(rtp, &lost_packets, &fraction_lost);
 
 	gettimeofday(&now, NULL);
 	rtcp_report->reception_report_count = rtp->themssrc_valid ? 1 : 0;
 	rtcp_report->ssrc = rtp->ssrc;
-	rtcp_report->type = sr ? RTCP_PT_SR : RTCP_PT_RR;
-	if (sr) {
+	rtcp_report->type = *sr ? RTCP_PT_SR : RTCP_PT_RR;
+	if (*sr) {
 		rtcp_report->sender_information.ntp_timestamp = now;
 		rtcp_report->sender_information.rtp_timestamp = rtp->lastts;
 		rtcp_report->sender_information.packet_count = rtp->txcount;
@@ -3778,7 +4173,7 @@ static int ast_rtcp_write_report(struct
 		report_block->lost_count.fraction = (fraction_lost & 0xff);
 		report_block->lost_count.packets = (lost_packets & 0xffffff);
 		report_block->highest_seq_no = (rtp->cycles | (rtp->lastrxseqno & 0xffff));
-		report_block->ia_jitter = (unsigned int)(rtp->rxjitter * rate);
+		report_block->ia_jitter = (unsigned int)(rtp->rxjitter * rtp_get_rate(rtp->f.subclass.format));
 		report_block->lsr = rtp->rtcp->themrxlsr;
 		/* If we haven't received an SR report, DLSR should be 0 */
 		if (!ast_tvzero(rtp->rtcp->rxlsr)) {
@@ -3787,48 +4182,53 @@ static int ast_rtcp_write_report(struct
 		}
 	}
 	timeval2ntp(rtcp_report->sender_information.ntp_timestamp, &now_msw, &now_lsw);
-	rtcpheader = (unsigned int *)bdata;
-	rtcpheader[1] = htonl(rtcp_report->ssrc);            /* Our SSRC */
+	put_unaligned_uint32(rtcpheader + 4, htonl(rtcp_report->ssrc)); /* Our SSRC */
 	len += 8;
-	if (sr) {
-		header_offset = 5;
-		rtcpheader[2] = htonl(now_msw);                 /* now, MSW. gettimeofday() + SEC_BETWEEN_1900_AND_1970*/
-		rtcpheader[3] = htonl(now_lsw);                 /* now, LSW */
-		rtcpheader[4] = htonl(rtcp_report->sender_information.rtp_timestamp);
-		rtcpheader[5] = htonl(rtcp_report->sender_information.packet_count);
-		rtcpheader[6] = htonl(rtcp_report->sender_information.octet_count);
+	if (*sr) {
+		put_unaligned_uint32(rtcpheader + len, htonl(now_msw)); /* now, MSW. gettimeofday() + SEC_BETWEEN_1900_AND_1970 */
+		put_unaligned_uint32(rtcpheader + len + 4, htonl(now_lsw)); /* now, LSW */
+		put_unaligned_uint32(rtcpheader + len + 8, htonl(rtcp_report->sender_information.rtp_timestamp));
+		put_unaligned_uint32(rtcpheader + len + 12, htonl(rtcp_report->sender_information.packet_count));
+		put_unaligned_uint32(rtcpheader + len + 16, htonl(rtcp_report->sender_information.octet_count));
 		len += 20;
 	}
 	if (report_block) {
-		rtcpheader[2 + header_offset] = htonl(report_block->source_ssrc);     /* Their SSRC */
-		rtcpheader[3 + header_offset] = htonl((report_block->lost_count.fraction << 24) | report_block->lost_count.packets);
-		rtcpheader[4 + header_offset] = htonl(report_block->highest_seq_no);
-		rtcpheader[5 + header_offset] = htonl(report_block->ia_jitter);
-		rtcpheader[6 + header_offset] = htonl(report_block->lsr);
-		rtcpheader[7 + header_offset] = htonl(report_block->dlsr);
+		put_unaligned_uint32(rtcpheader + len, htonl(report_block->source_ssrc)); /* Their SSRC */
+		put_unaligned_uint32(rtcpheader + len + 4, htonl((report_block->lost_count.fraction << 24) | report_block->lost_count.packets));
+		put_unaligned_uint32(rtcpheader + len + 8, htonl(report_block->highest_seq_no));
+		put_unaligned_uint32(rtcpheader + len + 12, htonl(report_block->ia_jitter));
+		put_unaligned_uint32(rtcpheader + len + 16, htonl(report_block->lsr));
+		put_unaligned_uint32(rtcpheader + len + 20, htonl(report_block->dlsr));
 		len += 24;
 	}
-	rtcpheader[0] = htonl((2 << 30) | (rtcp_report->reception_report_count << 24)
-					| ((sr ? RTCP_PT_SR : RTCP_PT_RR) << 16) | ((len/4)-1));
 
-	/* Insert SDES here. Probably should make SDES text equal to mimetypes[code].type (not subtype 'cos */
-	/* it can change mid call, and SDES can't) */
-	rtcpheader[len/4]     = htonl((2 << 30) | (1 << 24) | (RTCP_PT_SDES << 16) | 2);
-	rtcpheader[(len/4)+1] = htonl(rtcp_report->ssrc);
-	rtcpheader[(len/4)+2] = htonl(0x01 << 24);
-	len += 12;
+	put_unaligned_uint32(rtcpheader, htonl((2 << 30) | (rtcp_report->reception_report_count << 24)
+				| ((*sr ? RTCP_PT_SR : RTCP_PT_RR) << 16) | ((len/4)-1)));
 
-	ast_sockaddr_copy(&remote_address, &rtp->rtcp->them);
-	res = rtcp_sendto(instance, (unsigned int *)rtcpheader, len, 0, &remote_address, &ice);
-	if (res < 0) {
-		ast_log(LOG_ERROR, "RTCP %s transmission error to %s, rtcp halted %s\n",
-			sr ? "SR" : "RR",
-			ast_sockaddr_stringify(&rtp->rtcp->them),
-			strerror(errno));
+	return len;
+}
+
+static int ast_rtcp_calculate_sr_rr_statistics(struct ast_rtp_instance *instance,
+		struct ast_rtp_rtcp_report *rtcp_report, struct ast_sockaddr remote_address, int ice, int sr)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct ast_rtp_rtcp_report_block *report_block = NULL;
+	RAII_VAR(struct ast_json *, message_blob, NULL, ast_json_unref);
+
+	if (!rtp || !rtp->rtcp) {
 		return 0;
 	}
 
-	/* Update RTCP SR/RR statistics */
+	if (ast_sockaddr_isnull(&rtp->rtcp->them)) {
+		return 0;
+	}
+
+	if (!rtcp_report) {
+		return -1;
+	}
+
+	report_block = rtcp_report->report_block[0];
+
 	if (sr) {
 		rtp->rtcp->txlsr = rtcp_report->sender_information.ntp_timestamp;
 		rtp->rtcp->sr_count++;
@@ -3855,7 +4255,7 @@ static int ast_rtcp_write_report(struct
 			ast_verbose("    Fraction lost: %d\n", report_block->lost_count.fraction);
 			ast_verbose("    Cumulative loss: %u\n", report_block->lost_count.packets);
 			ast_verbose("    Highest seq no: %u\n", report_block->highest_seq_no);
-			ast_verbose("    IA jitter: %.4f\n", (double)report_block->ia_jitter / rate);
+			ast_verbose("    IA jitter: %.4f\n", (double)report_block->ia_jitter / rtp_get_rate(rtp->f.subclass.format));
 			ast_verbose("    Their last SR: %u\n", report_block->lsr);
 			ast_verbose("    DLSR: %4.4f (sec)\n\n", (double)(report_block->dlsr / 65536.0));
 		}
@@ -3865,9 +4265,121 @@ static int ast_rtcp_write_report(struct
 			"to", ast_sockaddr_stringify(&remote_address),
 			"from", rtp->rtcp->local_addr_str);
 	ast_rtp_publish_rtcp_message(instance, ast_rtp_rtcp_sent_type(),
-			rtcp_report,
-			message_blob);
-	return res;
+			rtcp_report, message_blob);
+
+	return 1;
+}
+
+static int ast_rtcp_generate_sdes(struct ast_rtp_instance *instance, unsigned char *rtcpheader,
+		struct ast_rtp_rtcp_report *rtcp_report)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	int len = 0;
+	uint16_t sdes_packet_len_bytes;
+	uint16_t sdes_packet_len_rounded;
+
+	if (!rtp || !rtp->rtcp) {
+		return 0;
+	}
+
+	if (ast_sockaddr_isnull(&rtp->rtcp->them)) {
+		return 0;
+	}
+
+	if (!rtcp_report) {
+		return -1;
+	}
+
+	sdes_packet_len_bytes =
+		4 + /* RTCP Header */
+		4 + /* SSRC */
+		1 + /* Type (CNAME) */
+		1 + /* Text Length */
+		AST_UUID_STR_LEN /* Text and NULL terminator */
+		;
+
+	/* Round to 32 bit boundary */
+	sdes_packet_len_rounded = (sdes_packet_len_bytes + 3) & ~0x3;
+
+	put_unaligned_uint32(rtcpheader, htonl((2 << 30) | (1 << 24) | (RTCP_PT_SDES << 16) | ((sdes_packet_len_rounded / 4) - 1)));
+	put_unaligned_uint32(rtcpheader + 4, htonl(rtcp_report->ssrc));
+	rtcpheader[8] = 0x01; /* CNAME */
+	rtcpheader[9] = AST_UUID_STR_LEN - 1; /* Number of bytes of text */
+	memcpy(rtcpheader + 10, rtp->cname, AST_UUID_STR_LEN);
+	len += 10 + AST_UUID_STR_LEN;
+
+	/* Padding - Note that we don't set the padded bit on the packet. From
+	 * RFC 3550 Section 6.5:
+	 *
+	 *   No length octet follows the null item type octet, but additional null
+	 *   octets MUST be included if needd to pad until the next 32-bit
+	 *   boundary. Note that this padding is separate from that indicated by
+	 *   the P bit in the RTCP header.
+	 *
+	 * These bytes will already be zeroed out during array initialization.
+	 */
+	len += (sdes_packet_len_rounded - sdes_packet_len_bytes);
+
+	return len;
+}
+
+static int ast_rtcp_generate_nack(struct ast_rtp_instance *instance, unsigned char *rtcpheader)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	int packet_len;
+	int blp_index;
+	int current_seqno;
+	int seqno;
+	unsigned int fci;
+
+	if (!rtp || !rtp->rtcp) {
+		return 0;
+	}
+
+	if (ast_sockaddr_isnull(&rtp->rtcp->them)) {
+		return 0;
+	}
+
+	current_seqno = rtp->expectedrxseqno;
+	seqno = rtp->lastrxseqno;
+	packet_len = 12; /* The header length is 12 (version line, packet source SSRC, media source SSRC) */
+
+	/* Get the missing sequence numbers for the FCI section of the NACK request */
+	for (blp_index = 0, fci = 0; current_seqno < seqno; current_seqno++, blp_index++) {
+		int *missing_seqno;
+
+		missing_seqno = AST_VECTOR_GET_CMP(&rtp->missing_seqno, current_seqno,
+				find_by_value);
+
+		if (!missing_seqno) {
+			continue;
+		}
+
+		/* We hit the max blp size, reset */
+		if (blp_index >= 17) {
+			put_unaligned_uint32(rtcpheader + packet_len, htonl(fci));
+			fci = 0;
+			blp_index = 0;
+			packet_len += 4;
+		}
+
+		if (blp_index == 0) {
+			fci |= (current_seqno << 16);
+		} else {
+			fci |= (1 << (blp_index - 1));
+		}
+	}
+
+	put_unaligned_uint32(rtcpheader + packet_len, htonl(fci));
+	packet_len += 4;
+
+	/* Length MUST be 2+n, where n is the number of NACKs. Same as length in words minus 1 */
+	put_unaligned_uint32(rtcpheader, htonl((2 << 30) | (AST_RTP_RTCP_FMT_NACK << 24)
+				| (AST_RTP_RTCP_RTPFB << 16) | ((packet_len / 4) - 1)));
+	put_unaligned_uint32(rtcpheader + 4, htonl(rtp->ssrc));
+	put_unaligned_uint32(rtcpheader + 8, htonl(rtp->themssrc));
+
+	return packet_len;
 }
 
 /*!
@@ -3883,6 +4395,15 @@ static int ast_rtcp_write(const void *da
 	struct ast_rtp_instance *instance = (struct ast_rtp_instance *) data;
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 	int res;
+	int sr = 0;
+	int packet_len = 0;
+	int ice;
+	struct ast_sockaddr remote_address = { { 0, } };
+	unsigned char *rtcpheader;
+	unsigned char bdata[AST_UUID_STR_LEN + 128] = ""; /* More than enough */
+	RAII_VAR(struct ast_rtp_rtcp_report *, rtcp_report,
+			ast_rtp_rtcp_report_alloc(rtp->themssrc_valid ? 1 : 0),
+			ao2_cleanup);
 
 	if (!rtp || !rtp->rtcp || rtp->rtcp->schedid == -1) {
 		ao2_ref(instance, -1);
@@ -3890,13 +4411,44 @@ static int ast_rtcp_write(const void *da
 	}
 
 	ao2_lock(instance);
-	if (rtp->txcount > rtp->rtcp->lastsrtxcount) {
-		/* Send an SR */
-		res = ast_rtcp_write_report(instance, 1);
+	rtcpheader = bdata;
+
+	res = ast_rtcp_generate_report(instance, rtcpheader, rtcp_report, &sr);
+
+	if (res == 0 || res == 1) {
+		ast_debug(1, "Failed to add %s report to RTCP packet!\n", sr ? "SR" : "RR");
+		goto cleanup;
+	}
+
+	packet_len += res;
+
+	res = ast_rtcp_generate_sdes(instance, rtcpheader + packet_len, rtcp_report);
+
+	if (res == 0 || res == 1) {
+		ast_debug(1, "Failed to add SDES to RTCP packet!\n");
+		goto cleanup;
+	}
+
+	packet_len += res;
+
+	if (rtp->bundled) {
+		ast_rtp_instance_get_remote_address(instance, &remote_address);
+	} else {
+		ast_sockaddr_copy(&remote_address, &rtp->rtcp->them);
+	}
+
+	res = rtcp_sendto(instance, (unsigned int *)rtcpheader, packet_len, 0, &remote_address, &ice);
+	if (res < 0) {
+		ast_log(LOG_ERROR, "RTCP %s transmission error to %s, rtcp halted %s\n",
+				sr ? "SR" : "RR",
+				ast_sockaddr_stringify(&rtp->rtcp->them),
+				strerror(errno));
+		res = 0;
 	} else {
-		/* Send an RR */
-		res = ast_rtcp_write_report(instance, 0);
+		ast_rtcp_calculate_sr_rr_statistics(instance, rtcp_report, remote_address, ice, sr);
 	}
+
+cleanup:
 	ao2_unlock(instance);
 
 	if (!res) {
@@ -3910,6 +4462,20 @@ static int ast_rtcp_write(const void *da
 	return res;
 }
 
+static void put_unaligned_time24(void *p, uint32_t time_msw, uint32_t time_lsw)
+{
+	unsigned char *cp = p;
+	uint32_t datum;
+
+	/* Convert the time to 6.18 format */
+	datum = (time_msw << 18) & 0x00fc0000;
+	datum |= (time_lsw >> 14) & 0x0003ffff;
+
+	cp[0] = datum >> 16;
+	cp[1] = datum >> 8;
+	cp[2] = datum;
+}
+
 /*! \pre instance is locked */
 static int rtp_raw_write(struct ast_rtp_instance *instance, struct ast_frame *frame, int codec)
 {
@@ -3918,6 +4484,7 @@ static int rtp_raw_write(struct ast_rtp_
 	unsigned int ms = calc_txstamp(rtp, &frame->delivery);
 	struct ast_sockaddr remote_address = { {0,} };
 	int rate = rtp_get_rate(frame->subclass.format) / 1000;
+	unsigned int seqno;
 
 	if (ast_format_cmp(frame->subclass.format, ast_format_g722) == AST_FORMAT_CMP_EQUAL) {
 		frame->samples /= 2;
@@ -3984,6 +4551,40 @@ static int rtp_raw_write(struct ast_rtp_
 		rtp->lastdigitts = rtp->lastts;
 	}
 
+	/* Assume that the sequence number we expect to use is what will be used until proven otherwise */
+	seqno = rtp->seqno;
+
+	/* If the frame contains sequence number information use it to influence our sequence number */
+	if (ast_test_flag(frame, AST_FRFLAG_HAS_SEQUENCE_NUMBER)) {
+		if (rtp->expectedseqno != -1) {
+			/* Determine where the frame from the core is in relation to where we expected */
+			int difference = frame->seqno - rtp->expectedseqno;
+
+			/* If there is a substantial difference then we've either got packets really out
+			 * of order, or the source is RTP and it has cycled. If this happens we resync
+			 * the sequence number adjustments to this frame. If we also have packet loss
+			 * things won't be reflected correctly but it will sort itself out after a bit.
+			 */
+			if (abs(difference) > 100) {
+				difference = 0;
+			}
+
+			/* Adjust the sequence number being used for this packet accordingly */
+			seqno += difference;
+
+			if (difference >= 0) {
+				/* This frame is on time or in the future */
+				rtp->expectedseqno = frame->seqno + 1;
+				rtp->seqno += difference;
+			}
+		} else {
+			/* This is the first frame with sequence number we've seen, so start keeping track */
+			rtp->expectedseqno = frame->seqno + 1;
+		}
+	} else {
+		rtp->expectedseqno = -1;
+	}
+
 	if (ast_test_flag(frame, AST_FRFLAG_HAS_TIMING_INFO)) {
 		rtp->lastts = frame->ts * rate;
 	}
@@ -3992,14 +4593,60 @@ static int rtp_raw_write(struct ast_rtp_
 
 	/* If we know the remote address construct a packet and send it out */
 	if (!ast_sockaddr_isnull(&remote_address)) {
-		int hdrlen = 12, res, ice;
-		unsigned char *rtpheader = (unsigned char *)(frame->data.ptr - hdrlen);
+		int hdrlen = 12;
+		int res;
+		int ice;
+		int ext = 0;
+		int abs_send_time_id;
+		int packet_len;
+		unsigned char *rtpheader;
+
+		/* If the abs-send-time extension has been negotiated determine how much space we need */
+		abs_send_time_id = ast_rtp_instance_extmap_get_id(instance, AST_RTP_EXTENSION_ABS_SEND_TIME);
+		if (abs_send_time_id != -1) {
+			/* 4 bytes for the shared information, 1 byte for identifier, 3 bytes for abs-send-time */
+			hdrlen += 8;
+			ext = 1;
+		}
+
+		packet_len = frame->datalen + hdrlen;
+		rtpheader = (unsigned char *)(frame->data.ptr - hdrlen);
 
-		put_unaligned_uint32(rtpheader, htonl((2 << 30) | (codec << 16) | (rtp->seqno) | (mark << 23)));
+		put_unaligned_uint32(rtpheader, htonl((2 << 30) | (ext << 28) | (codec << 16) | (seqno) | (mark << 23)));
 		put_unaligned_uint32(rtpheader + 4, htonl(rtp->lastts));
 		put_unaligned_uint32(rtpheader + 8, htonl(rtp->ssrc));
 
-		if ((res = rtp_sendto(instance, (void *)rtpheader, frame->datalen + hdrlen, 0, &remote_address, &ice)) < 0) {
+		/* We assume right now that we will only ever have the abs-send-time extension in the packet
+		 * which simplifies things a bit.
+		 */
+		if (abs_send_time_id != -1) {
+			unsigned int now_msw;
+			unsigned int now_lsw;
+
+			/* This happens before being placed into the retransmission buffer so that when we
+			 * retransmit we only have to update the timestamp, not everything else.
+			 */
+			put_unaligned_uint32(rtpheader + 12, htonl((0xBEDE << 16) | 1));
+			rtpheader[16] = (abs_send_time_id << 4) | 2;
+
+			timeval2ntp(ast_tvnow(), &now_msw, &now_lsw);
+			put_unaligned_time24(rtpheader + 17, now_msw, now_lsw);
+		}
+
+		/* If retransmissions are enabled, we need to store this packet for future use */
+		if (rtp->send_buffer) {
+			struct ast_rtp_rtcp_nack_payload *payload;
+
+			payload = ast_malloc(sizeof(*payload) + packet_len);
+			if (payload) {
+				payload->size = packet_len;
+				memcpy(payload->buf, rtpheader, packet_len);
+				ast_data_buffer_put(rtp->send_buffer, rtp->seqno, payload);
+			}
+		}
+
+		res = rtp_sendto(instance, (void *)rtpheader, packet_len, 0, &remote_address, &ice);
+		if (res < 0) {
 			if (!ast_rtp_instance_get_prop(instance, AST_RTP_PROPERTY_NAT) || (ast_rtp_instance_get_prop(instance, AST_RTP_PROPERTY_NAT) && (ast_test_flag(rtp, FLAG_NAT_ACTIVE) == FLAG_NAT_ACTIVE))) {
 				ast_debug(1, "RTP Transmission error of packet %d to %s: %s\n",
 					  rtp->seqno,
@@ -4032,7 +4679,13 @@ static int rtp_raw_write(struct ast_rtp_
 		}
 	}
 
-	rtp->seqno++;
+	/* If the sequence number that has been used doesn't match what we expected then this is an out of
+	 * order late packet, so we don't need to increment as we haven't yet gotten the expected frame from
+	 * the core.
+	 */
+	if (seqno == rtp->seqno) {
+		rtp->seqno++;
+	}
 
 	return 0;
 }
@@ -4077,63 +4730,119 @@ static struct ast_frame *red_t140_to_red
 	return &red->t140red;
 }
 
-/*! \pre instance is locked */
-static int ast_rtp_write(struct ast_rtp_instance *instance, struct ast_frame *frame)
+static void rtp_write_rtcp_fir(struct ast_rtp_instance *instance, struct ast_rtp *rtp, struct ast_sockaddr *remote_address)
 {
-	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-	struct ast_sockaddr remote_address = { {0,} };
-	struct ast_format *format;
-	int codec;
-
-	ast_rtp_instance_get_remote_address(instance, &remote_address);
+	unsigned int *rtcpheader;
+	char bdata[1024];
+	int len = 20;
+	int ice;
+	int res;
 
-	/* If we don't actually know the remote address don't even bother doing anything */
-	if (ast_sockaddr_isnull(&remote_address)) {
-		ast_debug(1, "No remote address on RTP instance '%p' so dropping frame\n", instance);
-		return 0;
+	if (!rtp || !rtp->rtcp) {
+		return;
 	}
 
-	/* VP8: is this a request to send a RTCP FIR? */
-	if (frame->frametype == AST_FRAME_CONTROL && frame->subclass.integer == AST_CONTROL_VIDUPDATE) {
-		struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
-		unsigned int *rtcpheader;
-		char bdata[1024];
-		int len = 20;
-		int ice;
-		int res;
-
-		if (!rtp || !rtp->rtcp) {
-			return 0;
-		}
-
-		if (ast_sockaddr_isnull(&rtp->rtcp->them) || rtp->rtcp->schedid < 0) {
-			/*
-			 * RTCP was stopped.
-			 */
-			return 0;
-		}
-		if (!rtp->themssrc_valid) {
-			/* We don't know their SSRC value so we don't know who to update. */
-			return 0;
-		}
+	if (ast_sockaddr_isnull(&rtp->rtcp->them) || rtp->rtcp->schedid < 0) {
+		/*
+		 * RTCP was stopped.
+		 */
+		return;
+	}
 
-		/* Prepare RTCP FIR (PT=206, FMT=4) */
-		rtp->rtcp->firseq++;
-		if(rtp->rtcp->firseq == 256) {
-			rtp->rtcp->firseq = 0;
-		}
+	if (!rtp->themssrc_valid) {
+		/* We don't know their SSRC value so we don't know who to update. */
+		return;
+	}
 
-		rtcpheader = (unsigned int *)bdata;
-		rtcpheader[0] = htonl((2 << 30) | (4 << 24) | (RTCP_PT_PSFB << 16) | ((len/4)-1));
-		rtcpheader[1] = htonl(rtp->ssrc);
-		rtcpheader[2] = htonl(rtp->themssrc);
-		rtcpheader[3] = htonl(rtp->themssrc);	/* FCI: SSRC */
-		rtcpheader[4] = htonl(rtp->rtcp->firseq << 24);			/* FCI: Sequence number */
-		res = rtcp_sendto(instance, (unsigned int *)rtcpheader, len, 0, &rtp->rtcp->them, &ice);
-		if (res < 0) {
-			ast_log(LOG_ERROR, "RTCP FIR transmission error: %s\n", strerror(errno));
-		}
-		return 0;
+	/* Prepare RTCP FIR (PT=206, FMT=4) */
+	rtp->rtcp->firseq++;
+	if(rtp->rtcp->firseq == 256) {
+		rtp->rtcp->firseq = 0;
+	}
+
+	rtcpheader = (unsigned int *)bdata;
+	rtcpheader[0] = htonl((2 << 30) | (4 << 24) | (RTCP_PT_PSFB << 16) | ((len/4)-1));
+	rtcpheader[1] = htonl(rtp->ssrc);
+	rtcpheader[2] = htonl(rtp->themssrc);
+	rtcpheader[3] = htonl(rtp->themssrc);	/* FCI: SSRC */
+	rtcpheader[4] = htonl(rtp->rtcp->firseq << 24);			/* FCI: Sequence number */
+	res = rtcp_sendto(instance, (unsigned int *)rtcpheader, len, 0, rtp->bundled ? remote_address : &rtp->rtcp->them, &ice);
+	if (res < 0) {
+		ast_log(LOG_ERROR, "RTCP FIR transmission error: %s\n", strerror(errno));
+	}
+}
+
+static void rtp_write_rtcp_psfb(struct ast_rtp_instance *instance, struct ast_rtp *rtp, struct ast_frame *frame, struct ast_sockaddr *remote_address)
+{
+	struct ast_rtp_rtcp_feedback *feedback = frame->data.ptr;
+	unsigned int *rtcpheader;
+	char bdata[1024];
+	int len = 24;
+	int ice;
+	int res;
+
+	if (feedback->fmt != AST_RTP_RTCP_FMT_REMB) {
+		ast_debug(1, "Provided an RTCP feedback frame of format %d to write on RTP instance '%p' but only REMB is supported\n",
+			feedback->fmt, instance);
+		return;
+	}
+
+	if (!rtp || !rtp->rtcp) {
+		return;
+	}
+
+	/* If REMB support is not enabled don't send this RTCP packet */
+	if (!ast_rtp_instance_get_prop(instance, AST_RTP_PROPERTY_REMB)) {
+		ast_debug(1, "Provided an RTCP feedback REMB report to write on RTP instance '%p' but REMB support not enabled\n",
+			instance);
+		return;
+	}
+
+	if (ast_sockaddr_isnull(&rtp->rtcp->them) || rtp->rtcp->schedid < 0) {
+		/*
+		 * RTCP was stopped.
+		 */
+		return;
+	}
+
+	rtcpheader = (unsigned int *)bdata;
+	rtcpheader[0] = htonl((2 << 30) | (AST_RTP_RTCP_FMT_REMB << 24) | (RTCP_PT_PSFB << 16) | ((len/4)-1));
+	rtcpheader[1] = htonl(rtp->ssrc);
+	rtcpheader[2] = htonl(0); /* Per the draft this should always be 0 */
+	rtcpheader[3] = htonl(('R' << 24) | ('E' << 16) | ('M' << 8) | ('B')); /* Unique identifier 'R' 'E' 'M' 'B' */
+	rtcpheader[4] = htonl((1 << 24) | (feedback->remb.br_exp << 18) | (feedback->remb.br_mantissa)); /* Number of SSRCs / BR Exp / BR Mantissa */
+	rtcpheader[5] = htonl(rtp->ssrc); /* The SSRC this feedback message applies to */
+	res = rtcp_sendto(instance, (unsigned int *)rtcpheader, len, 0, rtp->bundled ? remote_address : &rtp->rtcp->them, &ice);
+	if (res < 0) {
+		ast_log(LOG_ERROR, "RTCP PSFB transmission error: %s\n", strerror(errno));
+	}
+}
+
+/*! \pre instance is locked */
+static int ast_rtp_write(struct ast_rtp_instance *instance, struct ast_frame *frame)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct ast_sockaddr remote_address = { {0,} };
+	struct ast_format *format;
+	int codec;
+
+	ast_rtp_instance_get_remote_address(instance, &remote_address);
+
+	/* If we don't actually know the remote address don't even bother doing anything */
+	if (ast_sockaddr_isnull(&remote_address)) {
+		ast_debug(1, "No remote address on RTP instance '%p' so dropping frame\n", instance);
+		return 0;
+	}
+
+	/* VP8: is this a request to send a RTCP FIR? */
+	if (frame->frametype == AST_FRAME_CONTROL && frame->subclass.integer == AST_CONTROL_VIDUPDATE) {
+		rtp_write_rtcp_fir(instance, rtp, &remote_address);
+		return 0;
+	} else if (frame->frametype == AST_FRAME_RTCP) {
+		if (frame->subclass.integer == AST_RTP_RTCP_PSFB) {
+			rtp_write_rtcp_psfb(instance, rtp, frame, &remote_address);
+		}
+		return 0;
 	}
 
 	/* If there is no data length we can't very well send the packet */
@@ -4156,10 +4865,8 @@ static int ast_rtp_write(struct ast_rtp_
 	}
 
 	/* Grab the subclass and look up the payload we are going to use */
-	codec = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(instance),
-	                                    1,
-	                                    frame->subclass.format,
-	                                    0);
+	codec = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(instance),
+		1, frame->subclass.format, 0);
 	if (codec < 0) {
 		ast_log(LOG_WARNING, "Don't know how to send format %s packets with RTP\n",
 			ast_format_get_name(frame->subclass.format));
@@ -4324,7 +5031,7 @@ static struct ast_frame *create_dtmf_fra
 	return &rtp->f;
 }
 
-static void process_dtmf_rfc2833(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, struct ast_sockaddr *addr, int payloadtype, int mark, struct frame_list *frames)
+static void process_dtmf_rfc2833(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, int payloadtype, int mark, struct frame_list *frames)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 	struct ast_sockaddr remote_address = { {0,} };
@@ -4459,7 +5166,7 @@ static void process_dtmf_rfc2833(struct
 	return;
 }
 
-static struct ast_frame *process_dtmf_cisco(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, struct ast_sockaddr *addr, int payloadtype, int mark)
+static struct ast_frame *process_dtmf_cisco(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, int payloadtype, int mark)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 	unsigned int event, flags, power;
@@ -4539,7 +5246,7 @@ static struct ast_frame *process_dtmf_ci
 	return f;
 }
 
-static struct ast_frame *process_cn_rfc3389(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, struct ast_sockaddr *addr, int payloadtype, int mark)
+static struct ast_frame *process_cn_rfc3389(struct ast_rtp_instance *instance, unsigned char *data, int len, unsigned int seqno, unsigned int timestamp, int payloadtype, int mark)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 
@@ -4694,6 +5401,48 @@ static void update_lost_stats(struct ast
 	rtp->rtcp->reported_normdev_lost = reported_normdev_lost_current;
 }
 
+/*! \pre instance is locked */
+static struct ast_rtp_instance *__rtp_find_instance_by_ssrc(struct ast_rtp_instance *instance,
+	struct ast_rtp *rtp, unsigned int ssrc, int source)
+{
+	int index;
+
+	if (!AST_VECTOR_SIZE(&rtp->ssrc_mapping)) {
+		/* This instance is not bundled */
+		return instance;
+	}
+
+	/* Find the bundled child instance */
+	for (index = 0; index < AST_VECTOR_SIZE(&rtp->ssrc_mapping); ++index) {
+		struct rtp_ssrc_mapping *mapping = AST_VECTOR_GET_ADDR(&rtp->ssrc_mapping, index);
+		unsigned int mapping_ssrc = source ? ast_rtp_get_ssrc(mapping->instance) : mapping->ssrc;
+
+		if (mapping->ssrc_valid && mapping_ssrc == ssrc) {
+			return mapping->instance;
+		}
+	}
+
+	/* Does the SSRC match the bundled parent? */
+	if (rtp->themssrc_valid && rtp->themssrc == ssrc) {
+		return instance;
+	}
+	return NULL;
+}
+
+/*! \pre instance is locked */
+static struct ast_rtp_instance *rtp_find_instance_by_packet_source_ssrc(struct ast_rtp_instance *instance,
+	struct ast_rtp *rtp, unsigned int ssrc)
+{
+	return __rtp_find_instance_by_ssrc(instance, rtp, ssrc, 0);
+}
+
+/*! \pre instance is locked */
+static struct ast_rtp_instance *rtp_find_instance_by_media_source_ssrc(struct ast_rtp_instance *instance,
+	struct ast_rtp *rtp, unsigned int ssrc)
+{
+	return __rtp_find_instance_by_ssrc(instance, rtp, ssrc, 1);
+}
+
 static const char *rtcp_payload_type2str(unsigned int pt)
 {
 	const char *str;
@@ -4726,6 +5475,86 @@ static const char *rtcp_payload_type2str
 	return str;
 }
 
+/*! \pre instance is locked */
+static int ast_rtp_rtcp_handle_nack(struct ast_rtp_instance *instance, unsigned int *nackdata, unsigned int position,
+	unsigned int length)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	int res = 0;
+	int blp_index;
+	int packet_index;
+	int ice;
+	struct ast_rtp_rtcp_nack_payload *payload;
+	unsigned int current_word;
+	unsigned int pid;	/* Packet ID which refers to seqno of lost packet */
+	unsigned int blp;	/* Bitmask of following lost packets */
+	struct ast_sockaddr remote_address = { {0,} };
+	int abs_send_time_id;
+	unsigned int now_msw = 0;
+	unsigned int now_lsw = 0;
+
+	if (!rtp->send_buffer) {
+		ast_debug(1, "Tried to handle NACK request, but we don't have a RTP packet storage!\n");
+		return res;
+	}
+
+	abs_send_time_id = ast_rtp_instance_extmap_get_id(instance, AST_RTP_EXTENSION_ABS_SEND_TIME);
+	if (abs_send_time_id != -1) {
+		timeval2ntp(ast_tvnow(), &now_msw, &now_lsw);
+	}
+
+	ast_rtp_instance_get_remote_address(instance, &remote_address);
+
+	/*
+	 * We use index 3 because with feedback messages, the FCI (Feedback Control Information)
+	 * does not begin until after the version, packet SSRC, and media SSRC words.
+	 */
+	for (packet_index = 3; packet_index < length; packet_index++) {
+		current_word = ntohl(nackdata[position + packet_index]);
+		pid = current_word >> 16;
+		/* We know the remote end is missing this packet. Go ahead and send it if we still have it. */
+		payload = (struct ast_rtp_rtcp_nack_payload *)ast_data_buffer_get(rtp->send_buffer, pid);
+		if (payload) {
+			if (abs_send_time_id != -1) {
+				/* On retransmission we need to update the timestamp within the packet, as it
+				 * is supposed to contain when the packet was actually sent.
+				 */
+				put_unaligned_time24(payload->buf + 17, now_msw, now_lsw);
+			}
+			res += rtp_sendto(instance, payload->buf, payload->size, 0, &remote_address, &ice);
+		} else {
+			ast_debug(1, "Received NACK request for RTP packet with seqno %d, but we don't have it\n", pid);
+		}
+		/*
+		 * The bitmask. Denoting the least significant bit as 1 and its most significant bit
+		 * as 16, then bit i of the bitmask is set to 1 if the receiver has not received RTP
+		 * packet (pid+i)(modulo 2^16). Otherwise, it is set to 0. We cannot assume bits set
+		 * to 0 after a bit set to 1 have actually been received.
+		 */
+		blp = current_word & 0xFF;
+		blp_index = 1;
+		while (blp) {
+			if (blp & 1) {
+				/* Packet (pid + i)(modulo 2^16) is missing too. */
+				unsigned int seqno = (pid + blp_index) % 65536;
+				payload = (struct ast_rtp_rtcp_nack_payload *)ast_data_buffer_get(rtp->send_buffer, seqno);
+				if (payload) {
+					if (abs_send_time_id != -1) {
+						put_unaligned_time24(payload->buf + 17, now_msw, now_lsw);
+					}
+					res += rtp_sendto(instance, payload->buf, payload->size, 0, &remote_address, &ice);
+				} else {
+					ast_debug(1, "Remote end also requested RTP packet with seqno %d, but we don't have it\n", seqno);
+				}
+			}
+			blp >>= 1;
+			blp_index++;
+		}
+	}
+
+	return res;
+}
+
 /*
  * Unshifted RTCP header bit field masks
  */
@@ -4765,10 +5594,15 @@ static const char *rtcp_payload_type2str
 #define RTCP_SR_BLOCK_WORD_LENGTH 5
 #define RTCP_RR_BLOCK_WORD_LENGTH 6
 #define RTCP_HEADER_SSRC_LENGTH   2
+#define RTCP_FB_REMB_BLOCK_WORD_LENGTH 4
+#define RTCP_FB_NACK_BLOCK_WORD_LENGTH 2
 
-static struct ast_frame *ast_rtcp_interpret(struct ast_rtp_instance *instance, const unsigned char *rtcpdata, size_t size, struct ast_sockaddr *addr)
+static struct ast_frame *ast_rtcp_interpret(struct ast_rtp_instance *instance, struct ast_srtp *srtp,
+	const unsigned char *rtcpdata, size_t size, struct ast_sockaddr *addr)
 {
-	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct ast_rtp_instance *transport = instance;
+	struct ast_rtp *transport_rtp = ast_rtp_instance_get_data(instance);
+	int len = size;
 	unsigned int *rtcpheader = (unsigned int *)(rtcpdata);
 	unsigned int packetwords;
 	unsigned int position;
@@ -4778,10 +5612,16 @@ static struct ast_frame *ast_rtcp_interp
 	struct ast_rtp_rtcp_report_block *report_block;
 	struct ast_frame *f = &ast_null_frame;
 
-	packetwords = size / 4;
+	/* If this is encrypted then decrypt the payload */
+	if ((*rtcpheader & 0xC0) && res_srtp && srtp && res_srtp->unprotect(
+		    srtp, rtcpheader, &len, 1) < 0) {
+	   return &ast_null_frame;
+	}
+
+	packetwords = len / 4;
 
-	ast_debug(1, "Got RTCP report of %zu bytes from %s\n",
-		size, ast_sockaddr_stringify(addr));
+	ast_debug(1, "Got RTCP report of %d bytes from %s\n",
+		len, ast_sockaddr_stringify(addr));
 
 	/*
 	 * Validate the RTCP packet according to an adapted and slightly
@@ -4789,14 +5629,14 @@ static struct ast_frame *ast_rtcp_interp
 	 */
 	if (packetwords < RTCP_HEADER_SSRC_LENGTH) {
 		ast_debug(1, "%p -- RTCP from %s: Frame size (%u words) is too short\n",
-			rtp, ast_sockaddr_stringify(addr), packetwords);
+			transport_rtp, ast_sockaddr_stringify(addr), packetwords);
 		return &ast_null_frame;
 	}
 	position = 0;
 	first_word = ntohl(rtcpheader[position]);
 	if ((first_word & RTCP_VALID_MASK) != RTCP_VALID_VALUE) {
 		ast_debug(1, "%p -- RTCP from %s: Failed first packet validity check\n",
-			rtp, ast_sockaddr_stringify(addr));
+			transport_rtp, ast_sockaddr_stringify(addr));
 		return &ast_null_frame;
 	}
 	do {
@@ -4808,7 +5648,7 @@ static struct ast_frame *ast_rtcp_interp
 	} while ((first_word & RTCP_VERSION_MASK_SHIFTED) == RTCP_VERSION_SHIFTED);
 	if (position != packetwords) {
 		ast_debug(1, "%p -- RTCP from %s: Failed packet version or length check\n",
-			rtp, ast_sockaddr_stringify(addr));
+			transport_rtp, ast_sockaddr_stringify(addr));
 		return &ast_null_frame;
 	}
 
@@ -4828,7 +5668,7 @@ static struct ast_frame *ast_rtcp_interp
 	 * a matching SSRC so we'll accept this packet's address for
 	 * symmetrical RTP.
 	 */
-	ssrc_seen = rtp->strict_rtp_state == STRICT_RTP_OPEN;
+	ssrc_seen = transport_rtp->strict_rtp_state == STRICT_RTP_OPEN;
 
 	position = 0;
 	while (position < packetwords) {
@@ -4840,9 +5680,14 @@ static struct ast_frame *ast_rtcp_interp
 		unsigned int ssrc_valid;
 		unsigned int length;
 		unsigned int min_length;
+		/*! Always use packet source SSRC to find the rtp instance unless explicitly told not to. */
+		unsigned int use_packet_source = 1;
 
 		struct ast_json *message_blob;
 		RAII_VAR(struct ast_rtp_rtcp_report *, rtcp_report, NULL, ao2_cleanup);
+		struct ast_rtp_instance *child;
+		struct ast_rtp *rtp;
+		struct ast_rtp_rtcp_feedback *feedback;
 
 		i = position;
 		first_word = ntohl(rtcpheader[i]);
@@ -4859,9 +5704,28 @@ static struct ast_frame *ast_rtcp_interp
 			/* fall through */
 		case RTCP_PT_RR:
 			min_length += (rc * RTCP_RR_BLOCK_WORD_LENGTH);
+			use_packet_source = 0;
 			break;
 		case RTCP_PT_FUR:
+			break;
+		case AST_RTP_RTCP_RTPFB:
+			switch (rc) {
+			case AST_RTP_RTCP_FMT_NACK:
+				min_length += RTCP_FB_NACK_BLOCK_WORD_LENGTH;
+				break;
+			default:
+				break;
+			}
+			use_packet_source = 0;
+			break;
 		case RTCP_PT_PSFB:
+			switch (rc) {
+			case AST_RTP_RTCP_FMT_REMB:
+				min_length += RTCP_FB_REMB_BLOCK_WORD_LENGTH;
+				break;
+			default:
+				break;
+			}
 			break;
 		case RTCP_PT_SDES:
 		case RTCP_PT_BYE:
@@ -4875,7 +5739,7 @@ static struct ast_frame *ast_rtcp_interp
 			break;
 		default:
 			ast_debug(1, "%p -- RTCP from %s: %u(%s) skipping record\n",
-				rtp, ast_sockaddr_stringify(addr), pt, rtcp_payload_type2str(pt));
+				transport_rtp, ast_sockaddr_stringify(addr), pt, rtcp_payload_type2str(pt));
 			if (rtcp_debug_test_addr(addr)) {
 				ast_verbose("\n");
 				ast_verbose("RTCP from %s: %u(%s) skipping record\n",
@@ -4886,7 +5750,7 @@ static struct ast_frame *ast_rtcp_interp
 		}
 		if (length < min_length) {
 			ast_debug(1, "%p -- RTCP from %s: %u(%s) length field less than expected minimum.  Min:%u Got:%u\n",
-				rtp, ast_sockaddr_stringify(addr), pt, rtcp_payload_type2str(pt),
+				transport_rtp, ast_sockaddr_stringify(addr), pt, rtcp_payload_type2str(pt),
 				min_length - 1, length - 1);
 			return &ast_null_frame;
 		}
@@ -4902,13 +5766,16 @@ static struct ast_frame *ast_rtcp_interp
 			}
 			rtcp_report->reception_report_count = rc;
 
-			ssrc = ntohl(rtcpheader[i + 1]);
+			ssrc = ntohl(rtcpheader[i + 2]);
 			rtcp_report->ssrc = ssrc;
 			break;
 		case RTCP_PT_FUR:
 		case RTCP_PT_PSFB:
 			ssrc = ntohl(rtcpheader[i + 1]);
 			break;
+		case AST_RTP_RTCP_RTPFB:
+			ssrc = ntohl(rtcpheader[i + 2]);
+			break;
 		case RTCP_PT_SDES:
 		case RTCP_PT_BYE:
 		default:
@@ -4925,8 +5792,48 @@ static struct ast_frame *ast_rtcp_interp
 			ast_verbose("SSRC of sender: %u\n", ssrc);
 		}
 
+		/* Determine the appropriate instance for this */
+		if (ssrc_valid) {
+			/*
+			 * Depending on the payload type, either the packet source or media source
+			 * SSRC is used.
+			 */
+			if (use_packet_source) {
+				child = rtp_find_instance_by_packet_source_ssrc(transport, transport_rtp, ssrc);
+			} else {
+				child = rtp_find_instance_by_media_source_ssrc(transport, transport_rtp, ssrc);
+			}
+			if (child && child != transport) {
+				/*
+				 * It is safe to hold the child lock while holding the parent lock.
+				 * We guarantee that the locking order is always parent->child or
+				 * that the child lock is not held when acquiring the parent lock.
+				 */
+				ao2_lock(child);
+				instance = child;
+				rtp = ast_rtp_instance_get_data(instance);
+			} else {
+				/* The child is the parent! We don't need to unlock it. */
+				child = NULL;
+				rtp = transport_rtp;
+			}
+		} else {
+			child = NULL;
+			rtp = transport_rtp;
+		}
+
 		if (ssrc_valid && rtp->themssrc_valid) {
-			if (ssrc != rtp->themssrc) {
+			/*
+			 * If the SSRC is 1, we still need to handle RTCP since this could be a
+			 * special case. For example, if we have a unidirectional video stream, the
+			 * SSRC may be set to 1 by the browser (in the case of chromium), and requests
+			 * will still need to be processed so that video can flow as expected. This
+			 * should only be done for PLI and FUR, since there is not a way to get the
+			 * appropriate rtp instance when the SSRC is 1.
+			 */
+			int exception = (ssrc == 1 && !((pt == RTCP_PT_PSFB && rc == AST_RTP_RTCP_FMT_PLI) || pt == RTCP_PT_FUR));
+			if ((ssrc != rtp->themssrc && use_packet_source && ssrc != 1)
+					|| exception) {
 				/*
 				 * Skip over this RTCP record as it does not contain the
 				 * correct SSRC.  We should not act upon RTCP records
@@ -4935,6 +5842,9 @@ static struct ast_frame *ast_rtcp_interp
 				position += length;
 				ast_debug(1, "%p -- RTCP from %s: Skipping record, received SSRC '%u' != expected '%u'\n",
 					rtp, ast_sockaddr_stringify(addr), ssrc, rtp->themssrc);
+				if (child) {
+					ao2_unlock(child);
+				}
 				continue;
 			}
 			ssrc_seen = 1;
@@ -4986,6 +5896,9 @@ static struct ast_frame *ast_rtcp_interp
 				/* Don't handle multiple reception reports (rc > 1) yet */
 				report_block = ast_calloc(1, sizeof(*report_block));
 				if (!report_block) {
+					if (child) {
+						ao2_unlock(child);
+					}
 					return &ast_null_frame;
 				}
 				rtcp_report->report_block[0] = report_block;
@@ -5032,26 +5945,106 @@ static struct ast_frame *ast_rtcp_interp
 
 			message_blob = ast_json_pack("{s: s, s: s, s: f}",
 				"from", ast_sockaddr_stringify(addr),
-				"to", rtp->rtcp->local_addr_str,
+				"to", transport_rtp->rtcp->local_addr_str,
 				"rtt", rtp->rtcp->rtt);
 			ast_rtp_publish_rtcp_message(instance, ast_rtp_rtcp_received_type(),
 					rtcp_report,
 					message_blob);
 			ast_json_unref(message_blob);
+
+			/* Return an AST_FRAME_RTCP frame with the ast_rtp_rtcp_report
+			 * object as a its data */
+			transport_rtp->f.frametype = AST_FRAME_RTCP;
+			transport_rtp->f.subclass.integer = pt;
+			transport_rtp->f.data.ptr = rtp->rtcp->frame_buf + AST_FRIENDLY_OFFSET;
+			memcpy(transport_rtp->f.data.ptr, rtcp_report, sizeof(struct ast_rtp_rtcp_report));
+			transport_rtp->f.datalen = sizeof(struct ast_rtp_rtcp_report);
+			if (rc > 0) {
+				/* There's always a single report block stored, here */
+				struct ast_rtp_rtcp_report *rtcp_report2;
+				report_block = transport_rtp->f.data.ptr + transport_rtp->f.datalen + sizeof(struct ast_rtp_rtcp_report_block *);
+				memcpy(report_block, rtcp_report->report_block[0], sizeof(struct ast_rtp_rtcp_report_block));
+				rtcp_report2 = (struct ast_rtp_rtcp_report *)transport_rtp->f.data.ptr;
+				rtcp_report2->report_block[0] = report_block;
+				transport_rtp->f.datalen += sizeof(struct ast_rtp_rtcp_report_block);
+			}
+			transport_rtp->f.offset = AST_FRIENDLY_OFFSET;
+			transport_rtp->f.samples = 0;
+			transport_rtp->f.mallocd = 0;
+			transport_rtp->f.delivery.tv_sec = 0;
+			transport_rtp->f.delivery.tv_usec = 0;
+			transport_rtp->f.src = "RTP";
+			f = &transport_rtp->f;
+			break;
+		case AST_RTP_RTCP_RTPFB:
+			switch (rc) {
+			case AST_RTP_RTCP_FMT_NACK:
+				/* If retransmissions are not enabled ignore this message */
+				if (!rtp->send_buffer) {
+					break;
+				}
+
+				if (rtcp_debug_test_addr(addr)) {
+					ast_verbose("Received generic RTCP NACK message\n");
+				}
+
+				ast_rtp_rtcp_handle_nack(instance, rtcpheader, position, length);
+				break;
+			default:
+				break;
+			}
 			break;
 		case RTCP_PT_FUR:
-		/* Handle RTCP FIR as FUR */
+			/* Handle RTCP FUR as FIR by setting the format to 4 */
+			rc = AST_RTP_RTCP_FMT_FIR;
 		case RTCP_PT_PSFB:
-			if (rtcp_debug_test_addr(addr)) {
-				ast_verbose("Received an RTCP Fast Update Request\n");
+			switch (rc) {
+			case AST_RTP_RTCP_FMT_PLI:
+			case AST_RTP_RTCP_FMT_FIR:
+				if (rtcp_debug_test_addr(addr)) {
+					ast_verbose("Received an RTCP Fast Update Request\n");
+				}
+				transport_rtp->f.frametype = AST_FRAME_CONTROL;
+				transport_rtp->f.subclass.integer = AST_CONTROL_VIDUPDATE;
+				transport_rtp->f.datalen = 0;
+				transport_rtp->f.samples = 0;
+				transport_rtp->f.mallocd = 0;
+				transport_rtp->f.src = "RTP";
+				f = &transport_rtp->f;
+				break;
+			case AST_RTP_RTCP_FMT_REMB:
+				/* If REMB support is not enabled ignore this message */
+				if (!ast_rtp_instance_get_prop(instance, AST_RTP_PROPERTY_REMB)) {
+					break;
+				}
+
+				if (rtcp_debug_test_addr(addr)) {
+					ast_verbose("Received REMB report\n");
+				}
+				transport_rtp->f.frametype = AST_FRAME_RTCP;
+				transport_rtp->f.subclass.integer = pt;
+				transport_rtp->f.stream_num = rtp->stream_num;
+				transport_rtp->f.data.ptr = rtp->rtcp->frame_buf + AST_FRIENDLY_OFFSET;
+				feedback = transport_rtp->f.data.ptr;
+				feedback->fmt = rc;
+
+				/* We don't actually care about the SSRC information in the feedback message */
+				first_word = ntohl(rtcpheader[i + 2]);
+				feedback->remb.br_exp = (first_word >> 18) & ((1 << 6) - 1);
+				feedback->remb.br_mantissa = first_word & ((1 << 18) - 1);
+
+				transport_rtp->f.datalen = sizeof(struct ast_rtp_rtcp_feedback);
+				transport_rtp->f.offset = AST_FRIENDLY_OFFSET;
+				transport_rtp->f.samples = 0;
+				transport_rtp->f.mallocd = 0;
+				transport_rtp->f.delivery.tv_sec = 0;
+				transport_rtp->f.delivery.tv_usec = 0;
+				transport_rtp->f.src = "RTP";
+				f = &transport_rtp->f;
+				break;
+			default:
+				break;
 			}
-			rtp->f.frametype = AST_FRAME_CONTROL;
-			rtp->f.subclass.integer = AST_CONTROL_VIDUPDATE;
-			rtp->f.datalen = 0;
-			rtp->f.samples = 0;
-			rtp->f.mallocd = 0;
-			rtp->f.src = "RTP";
-			f = &rtp->f;
 			break;
 		case RTCP_PT_SDES:
 			if (rtcp_debug_test_addr(addr)) {
@@ -5069,8 +6062,12 @@ static struct ast_frame *ast_rtcp_interp
 			break;
 		}
 		position += length;
+		rtp->rtcp->rtcp_info = 1;
+
+		if (child) {
+			ao2_unlock(child);
+		}
 	}
-	rtp->rtcp->rtcp_info = 1;
 
 	return f;
 }
@@ -5079,6 +6076,7 @@ static struct ast_frame *ast_rtcp_interp
 static struct ast_frame *ast_rtcp_read(struct ast_rtp_instance *instance)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct ast_srtp *srtp = ast_rtp_instance_get_srtp(instance, 1);
 	struct ast_sockaddr addr;
 	unsigned char rtcpdata[8192 + AST_FRIENDLY_OFFSET];
 	unsigned char *read_area = rtcpdata + AST_FRIENDLY_OFFSET;
@@ -5088,6 +6086,12 @@ static struct ast_frame *ast_rtcp_read(s
 	/* Read in RTCP data from the socket */
 	if ((res = rtcp_recvfrom(instance, read_area, read_area_size,
 				0, &addr)) < 0) {
+		if (res == RTP_DTLS_ESTABLISHED) {
+			rtp->f.frametype = AST_FRAME_CONTROL;
+			rtp->f.subclass.integer = AST_CONTROL_SRCCHANGE;
+			return &rtp->f;
+		}
+
 		ast_assert(errno != EBADF);
 		if (errno != EAGAIN) {
 			ast_log(LOG_WARNING, "RTCP Read error: %s.  Hanging up.\n",
@@ -5124,7 +6128,7 @@ static struct ast_frame *ast_rtcp_read(s
 		return &ast_null_frame;
 	}
 
-	return ast_rtcp_interpret(instance, read_area, res, &addr);
+	return ast_rtcp_interpret(instance, srtp, read_area, res, &addr);
 }
 
 /*! \pre instance is locked */
@@ -5151,7 +6155,8 @@ static int bridge_p2p_rtp_write(struct a
 	}
 
 	/* Otherwise adjust bridged payload to match */
-	bridged_payload = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(instance1), payload_type->asterisk_format, payload_type->format, payload_type->rtp_code);
+	bridged_payload = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(instance1),
+		payload_type->asterisk_format, payload_type->format, payload_type->rtp_code);
 
 	/* If no codec could be matched between instance and instance1, then somehow things were made incompatible while we were still bridged.  Bail. */
 	if (bridged_payload < 0) {
@@ -5289,25 +6294,26 @@ static int bridge_p2p_rtp_write(struct a
 	return 0;
 }
 
-/*! \pre instance is locked */
-static struct ast_frame *ast_rtp_read(struct ast_rtp_instance *instance, int rtcp)
+static void rtp_instance_unlock(struct ast_rtp_instance *instance)
+{
+	if (instance) {
+		ao2_unlock(instance);
+	}
+}
+
+static struct ast_frame *ast_rtp_interpret(struct ast_rtp_instance *instance, struct ast_srtp *srtp,
+	const struct ast_sockaddr *remote_address, unsigned char *read_area, int length, int prev_seqno)
 {
+	unsigned int *rtpheader = (unsigned int*)(read_area);
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 	struct ast_rtp_instance *instance1;
-	struct ast_sockaddr addr;
-	int res, hdrlen = 12, version, payloadtype, padding, mark, ext, cc, prev_seqno;
-	unsigned char *read_area = rtp->rawdata + AST_FRIENDLY_OFFSET;
-	size_t read_area_size = sizeof(rtp->rawdata) - AST_FRIENDLY_OFFSET;
-	unsigned int *rtpheader = (unsigned int*)(read_area), seqno, ssrc, timestamp;
+	int res = length, hdrlen = 12, seqno, timestamp, payloadtype, padding, mark, ext, cc;
 	RAII_VAR(struct ast_rtp_payload_type *, payload, NULL, ao2_cleanup);
-	struct ast_sockaddr remote_address = { {0,} };
 	struct frame_list frames;
 
-	/* If this is actually RTCP let's hop on over and handle it */
-	if (rtcp) {
-		if (rtp->rtcp && rtp->rtcp->type == AST_RTP_INSTANCE_RTCP_STANDARD) {
-			return ast_rtcp_read(instance);
-		}
+	/* If this payload is encrypted then decrypt it using the given SRTP instance */
+	if ((*read_area & 0xC0) && res_srtp && srtp && res_srtp->unprotect(
+		    srtp, read_area, &res, 0) < 0) {
 		return &ast_null_frame;
 	}
 
@@ -5316,56 +6322,348 @@ static struct ast_frame *ast_rtp_read(st
 		ast_rtp_dtmf_continuation(instance);
 	}
 
-	/* Actually read in the data from the socket */
-	if ((res = rtp_recvfrom(instance, read_area, read_area_size, 0,
-				&addr)) < 0) {
-		ast_assert(errno != EBADF);
-		if (errno != EAGAIN) {
-			ast_log(LOG_WARNING, "RTP Read error: %s.  Hanging up.\n",
-				(errno) ? strerror(errno) : "Unspecified");
-			return NULL;
-		}
-		return &ast_null_frame;
-	}
+	/* Pull out the various other fields we will need */
+	seqno = ntohl(rtpheader[0]);
+	payloadtype = (seqno & 0x7f0000) >> 16;
+	padding = seqno & (1 << 29);
+	mark = seqno & (1 << 23);
+	ext = seqno & (1 << 28);
+	cc = (seqno & 0xF000000) >> 24;
+	seqno &= 0xffff;
+	timestamp = ntohl(rtpheader[1]);
 
-	/* If this was handled by the ICE session don't do anything */
-	if (!res) {
-		return &ast_null_frame;
+	AST_LIST_HEAD_INIT_NOLOCK(&frames);
+
+	/* Remove any padding bytes that may be present */
+	if (padding) {
+		res -= read_area[res - 1];
 	}
 
-	/* This could be a multiplexed RTCP packet. If so, be sure to interpret it correctly */
-	if (rtcp_mux(rtp, read_area)) {
-		return ast_rtcp_interpret(instance, read_area, res, &addr);
+	/* Skip over any CSRC fields */
+	if (cc) {
+		hdrlen += cc * 4;
 	}
 
-	/* Make sure the data that was read in is actually enough to make up an RTP packet */
-	if (res < hdrlen) {
-		/* If this is a keepalive containing only nulls, don't bother with a warning */
-		int i;
-		for (i = 0; i < res; ++i) {
-			if (read_area[i] != '\0') {
-				ast_log(LOG_WARNING, "RTP Read too short\n");
-				return &ast_null_frame;
+	/* Look for any RTP extensions, currently we do not support any */
+	if (ext) {
+		hdrlen += (ntohl(rtpheader[hdrlen/4]) & 0xffff) << 2;
+		hdrlen += 4;
+		if (DEBUG_ATLEAST(1)) {
+			unsigned int profile;
+			profile = (ntohl(rtpheader[3]) & 0xffff0000) >> 16;
+			if (profile == 0x505a) {
+				ast_log(LOG_DEBUG, "Found Zfone extension in RTP stream - zrtp - not supported.\n");
+			} else if (profile != 0xbede) {
+				/* SDP negotiated RTP extensions can not currently be output in logging */
+				ast_log(LOG_DEBUG, "Found unknown RTP Extensions %x\n", profile);
 			}
 		}
-		return &ast_null_frame;
 	}
 
-	/* Get fields and verify this is an RTP packet */
-	seqno = ntohl(rtpheader[0]);
+	/* Make sure after we potentially mucked with the header length that it is once again valid */
+	if (res < hdrlen) {
+		ast_log(LOG_WARNING, "RTP Read too short (%d, expecting %d\n", res, hdrlen);
+		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+	}
 
-	ast_rtp_instance_get_remote_address(instance, &remote_address);
+	rtp->rxcount++;
+	rtp->rxoctetcount += (res - hdrlen);
+	if (rtp->rxcount == 1) {
+		rtp->seedrxseqno = seqno;
+	}
 
-	if (!(version = (seqno & 0xC0000000) >> 30)) {
-		struct sockaddr_in addr_tmp;
-		struct ast_sockaddr addr_v4;
-		if (ast_sockaddr_is_ipv4(&addr)) {
-			ast_sockaddr_to_sin(&addr, &addr_tmp);
-		} else if (ast_sockaddr_ipv4_mapped(&addr, &addr_v4)) {
-			ast_debug(1, "Using IPv6 mapped address %s for STUN\n",
-				  ast_sockaddr_stringify(&addr));
-			ast_sockaddr_to_sin(&addr_v4, &addr_tmp);
-		} else {
+	/* Do not schedule RR if RTCP isn't run */
+	if (rtp->rtcp && !ast_sockaddr_isnull(&rtp->rtcp->them) && rtp->rtcp->schedid < 0) {
+		/* Schedule transmission of Receiver Report */
+		ao2_ref(instance, +1);
+		rtp->rtcp->schedid = ast_sched_add(rtp->sched, ast_rtcp_calc_interval(rtp), ast_rtcp_write, instance);
+		if (rtp->rtcp->schedid < 0) {
+			ao2_ref(instance, -1);
+			ast_log(LOG_WARNING, "scheduling RTCP transmission failed.\n");
+		}
+	}
+	if ((int)rtp->lastrxseqno - (int)seqno  > 100) /* if so it would indicate that the sender cycled; allow for misordering */
+		rtp->cycles += RTP_SEQ_MOD;
+
+	/* If we are directly bridged to another instance send the audio directly out,
+	 * but only after updating core information about the received traffic so that
+	 * outgoing RTCP reflects it.
+	 */
+	instance1 = ast_rtp_instance_get_bridged(instance);
+	if (instance1
+		&& !bridge_p2p_rtp_write(instance, instance1, rtpheader, res, hdrlen)) {
+		struct timeval rxtime;
+		struct ast_frame *f;
+
+		/* Update statistics for jitter so they are correct in RTCP */
+		calc_rxstamp(&rxtime, rtp, timestamp, mark);
+
+		/* When doing P2P we don't need to raise any frames about SSRC change to the core */
+		while ((f = AST_LIST_REMOVE_HEAD(&frames, frame_list)) != NULL) {
+			ast_frfree(f);
+		}
+
+		return &ast_null_frame;
+	}
+
+	payload = ast_rtp_codecs_get_payload(ast_rtp_instance_get_codecs(instance), payloadtype);
+	if (!payload) {
+		/* Unknown payload type. */
+		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+	}
+
+	/* If the payload is not actually an Asterisk one but a special one pass it off to the respective handler */
+	if (!payload->asterisk_format) {
+		struct ast_frame *f = NULL;
+		if (payload->rtp_code == AST_RTP_DTMF) {
+			/* process_dtmf_rfc2833 may need to return multiple frames. We do this
+			 * by passing the pointer to the frame list to it so that the method
+			 * can append frames to the list as needed.
+			 */
+			process_dtmf_rfc2833(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, payloadtype, mark, &frames);
+		} else if (payload->rtp_code == AST_RTP_CISCO_DTMF) {
+			f = process_dtmf_cisco(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, payloadtype, mark);
+		} else if (payload->rtp_code == AST_RTP_CN) {
+			f = process_cn_rfc3389(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, payloadtype, mark);
+		} else {
+			ast_log(LOG_NOTICE, "Unknown RTP codec %d received from '%s'\n",
+				payloadtype,
+				ast_sockaddr_stringify(remote_address));
+		}
+
+		if (f) {
+			AST_LIST_INSERT_TAIL(&frames, f, frame_list);
+		}
+		/* Even if no frame was returned by one of the above methods,
+		 * we may have a frame to return in our frame list
+		 */
+		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+	}
+
+	ao2_replace(rtp->lastrxformat, payload->format);
+	ao2_replace(rtp->f.subclass.format, payload->format);
+	switch (ast_format_get_type(rtp->f.subclass.format)) {
+	case AST_MEDIA_TYPE_AUDIO:
+		rtp->f.frametype = AST_FRAME_VOICE;
+		break;
+	case AST_MEDIA_TYPE_VIDEO:
+		rtp->f.frametype = AST_FRAME_VIDEO;
+		break;
+	case AST_MEDIA_TYPE_TEXT:
+		rtp->f.frametype = AST_FRAME_TEXT;
+		break;
+	case AST_MEDIA_TYPE_IMAGE:
+		/* Fall through */
+	default:
+		ast_log(LOG_WARNING, "Unknown or unsupported media type: %s\n",
+			ast_codec_media_type2str(ast_format_get_type(rtp->f.subclass.format)));
+		return &ast_null_frame;
+	}
+	rtp->rxseqno = seqno;
+
+	if (rtp->dtmf_timeout && rtp->dtmf_timeout < timestamp) {
+		rtp->dtmf_timeout = 0;
+
+		if (rtp->resp) {
+			struct ast_frame *f;
+			f = create_dtmf_frame(instance, AST_FRAME_DTMF_END, 0);
+			f->len = ast_tvdiff_ms(ast_samp2tv(rtp->dtmf_duration, rtp_get_rate(f->subclass.format)), ast_tv(0, 0));
+			rtp->resp = 0;
+			rtp->dtmf_timeout = rtp->dtmf_duration = 0;
+			AST_LIST_INSERT_TAIL(&frames, f, frame_list);
+			return AST_LIST_FIRST(&frames);
+		}
+	}
+
+	rtp->lastrxts = timestamp;
+
+	rtp->f.src = "RTP";
+	rtp->f.mallocd = 0;
+	rtp->f.datalen = res - hdrlen;
+	rtp->f.data.ptr = read_area + hdrlen;
+	rtp->f.offset = hdrlen + AST_FRIENDLY_OFFSET;
+	ast_set_flag(&rtp->f, AST_FRFLAG_HAS_SEQUENCE_NUMBER);
+	rtp->f.seqno = seqno;
+	rtp->f.stream_num = rtp->stream_num;
+
+	if ((ast_format_cmp(rtp->f.subclass.format, ast_format_t140) == AST_FORMAT_CMP_EQUAL)
+		&& ((int)seqno - (prev_seqno + 1) > 0)
+		&& ((int)seqno - (prev_seqno + 1) < 10)) {
+		unsigned char *data = rtp->f.data.ptr;
+
+		memmove(rtp->f.data.ptr+3, rtp->f.data.ptr, rtp->f.datalen);
+		rtp->f.datalen +=3;
+		*data++ = 0xEF;
+		*data++ = 0xBF;
+		*data = 0xBD;
+	}
+
+	if (ast_format_cmp(rtp->f.subclass.format, ast_format_t140_red) == AST_FORMAT_CMP_EQUAL) {
+		unsigned char *data = rtp->f.data.ptr;
+		unsigned char *header_end;
+		int num_generations;
+		int header_length;
+		int len;
+		int diff =(int)seqno - (prev_seqno+1); /* if diff = 0, no drop*/
+		int x;
+
+		ao2_replace(rtp->f.subclass.format, ast_format_t140);
+		header_end = memchr(data, ((*data) & 0x7f), rtp->f.datalen);
+		if (header_end == NULL) {
+			return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+		}
+		header_end++;
+
+		header_length = header_end - data;
+		num_generations = header_length / 4;
+		len = header_length;
+
+		if (!diff) {
+			for (x = 0; x < num_generations; x++)
+				len += data[x * 4 + 3];
+
+			if (!(rtp->f.datalen - len))
+				return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+
+			rtp->f.data.ptr += len;
+			rtp->f.datalen -= len;
+		} else if (diff > num_generations && diff < 10) {
+			len -= 3;
+			rtp->f.data.ptr += len;
+			rtp->f.datalen -= len;
+
+			data = rtp->f.data.ptr;
+			*data++ = 0xEF;
+			*data++ = 0xBF;
+			*data = 0xBD;
+		} else {
+			for ( x = 0; x < num_generations - diff; x++)
+				len += data[x * 4 + 3];
+
+			rtp->f.data.ptr += len;
+			rtp->f.datalen -= len;
+		}
+	}
+
+	if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_AUDIO) {
+		rtp->f.samples = ast_codec_samples_count(&rtp->f);
+		if (ast_format_cache_is_slinear(rtp->f.subclass.format)) {
+			ast_frame_byteswap_be(&rtp->f);
+		}
+		calc_rxstamp(&rtp->f.delivery, rtp, timestamp, mark);
+		/* Add timing data to let ast_generic_bridge() put the frame into a jitterbuf */
+		ast_set_flag(&rtp->f, AST_FRFLAG_HAS_TIMING_INFO);
+		rtp->f.ts = timestamp / (rtp_get_rate(rtp->f.subclass.format) / 1000);
+		rtp->f.len = rtp->f.samples / ((ast_format_get_sample_rate(rtp->f.subclass.format) / 1000));
+	} else if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_VIDEO) {
+		/* Video -- samples is # of samples vs. 90000 */
+		if (!rtp->lastividtimestamp)
+			rtp->lastividtimestamp = timestamp;
+		ast_set_flag(&rtp->f, AST_FRFLAG_HAS_TIMING_INFO);
+		rtp->f.ts = timestamp / (rtp_get_rate(rtp->f.subclass.format) / 1000);
+		rtp->f.samples = timestamp - rtp->lastividtimestamp;
+		rtp->lastividtimestamp = timestamp;
+		rtp->f.delivery.tv_sec = 0;
+		rtp->f.delivery.tv_usec = 0;
+		/* Pass the RTP marker bit as bit */
+		rtp->f.subclass.frame_ending = mark ? 1 : 0;
+	} else if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_TEXT) {
+		/* TEXT -- samples is # of samples vs. 1000 */
+		if (!rtp->lastitexttimestamp)
+			rtp->lastitexttimestamp = timestamp;
+		rtp->f.samples = timestamp - rtp->lastitexttimestamp;
+		rtp->lastitexttimestamp = timestamp;
+		rtp->f.delivery.tv_sec = 0;
+		rtp->f.delivery.tv_usec = 0;
+	} else {
+		ast_log(LOG_WARNING, "Unknown or unsupported media type: %s\n",
+			ast_codec_media_type2str(ast_format_get_type(rtp->f.subclass.format)));
+		return &ast_null_frame;
+	}
+
+	AST_LIST_INSERT_TAIL(&frames, &rtp->f, frame_list);
+	return AST_LIST_FIRST(&frames);
+}
+
+/*! \pre instance is locked */
+static struct ast_frame *ast_rtp_read(struct ast_rtp_instance *instance, int rtcp)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+	struct ast_srtp *srtp;
+	RAII_VAR(struct ast_rtp_instance *, child, NULL, rtp_instance_unlock);
+	struct ast_sockaddr addr;
+	int res, hdrlen = 12, version, payloadtype, mark;
+	unsigned char *read_area = rtp->rawdata + AST_FRIENDLY_OFFSET;
+	size_t read_area_size = sizeof(rtp->rawdata) - AST_FRIENDLY_OFFSET;
+	unsigned int *rtpheader = (unsigned int*)(read_area), seqno, ssrc, timestamp, prev_seqno;
+	struct ast_sockaddr remote_address = { {0,} };
+	struct frame_list frames;
+	struct ast_frame *frame;
+
+	/* If this is actually RTCP let's hop on over and handle it */
+	if (rtcp) {
+		if (rtp->rtcp && rtp->rtcp->type == AST_RTP_INSTANCE_RTCP_STANDARD) {
+			return ast_rtcp_read(instance);
+		}
+		return &ast_null_frame;
+	}
+
+	/* Actually read in the data from the socket */
+	if ((res = rtp_recvfrom(instance, read_area, read_area_size, 0,
+				&addr)) < 0) {
+		if (res == RTP_DTLS_ESTABLISHED) {
+			rtp->f.frametype = AST_FRAME_CONTROL;
+			rtp->f.subclass.integer = AST_CONTROL_SRCCHANGE;
+			return &rtp->f;
+		}
+
+		ast_assert(errno != EBADF);
+		if (errno != EAGAIN) {
+			ast_log(LOG_WARNING, "RTP Read error: %s.  Hanging up.\n",
+				(errno) ? strerror(errno) : "Unspecified");
+			return NULL;
+		}
+		return &ast_null_frame;
+	}
+
+	/* If this was handled by the ICE session don't do anything */
+	if (!res) {
+		return &ast_null_frame;
+	}
+
+	/* This could be a multiplexed RTCP packet. If so, be sure to interpret it correctly */
+	if (rtcp_mux(rtp, read_area)) {
+		return ast_rtcp_interpret(instance, ast_rtp_instance_get_srtp(instance, 1), read_area, res, &addr);
+	}
+
+	/* Make sure the data that was read in is actually enough to make up an RTP packet */
+	if (res < hdrlen) {
+		/* If this is a keepalive containing only nulls, don't bother with a warning */
+		int i;
+		for (i = 0; i < res; ++i) {
+			if (read_area[i] != '\0') {
+				ast_log(LOG_WARNING, "RTP Read too short\n");
+				return &ast_null_frame;
+			}
+		}
+		return &ast_null_frame;
+	}
+
+	/* Get fields and verify this is an RTP packet */
+	seqno = ntohl(rtpheader[0]);
+
+	ast_rtp_instance_get_remote_address(instance, &remote_address);
+
+	if (!(version = (seqno & 0xC0000000) >> 30)) {
+		struct sockaddr_in addr_tmp;
+		struct ast_sockaddr addr_v4;
+		if (ast_sockaddr_is_ipv4(&addr)) {
+			ast_sockaddr_to_sin(&addr, &addr_tmp);
+		} else if (ast_sockaddr_ipv4_mapped(&addr, &addr_v4)) {
+			ast_debug(1, "Using IPv6 mapped address %s for STUN\n",
+				  ast_sockaddr_stringify(&addr));
+			ast_sockaddr_to_sin(&addr_v4, &addr_tmp);
+		} else {
 			ast_debug(1, "Cannot do STUN for non IPv4 address %s\n",
 				  ast_sockaddr_stringify(&addr));
 			return &ast_null_frame;
@@ -5383,6 +6681,30 @@ static struct ast_frame *ast_rtp_read(st
 		return &ast_null_frame;
 	}
 
+	/* We use the SSRC to determine what RTP instance this packet is actually for */
+	ssrc = ntohl(rtpheader[2]);
+
+	/* We use the SRTP data from the provided instance that it came in on, not the child */
+	srtp = ast_rtp_instance_get_srtp(instance, 0);
+
+	/* Determine the appropriate instance for this */
+	child = rtp_find_instance_by_packet_source_ssrc(instance, rtp, ssrc);
+	if (!child) {
+		/* Neither the bundled parent nor any child has this SSRC */
+		return &ast_null_frame;
+	}
+	if (child != instance) {
+		/* It is safe to hold the child lock while holding the parent lock, we guarantee that the locking order
+		 * is always parent->child or that the child lock is not held when acquiring the parent lock.
+		 */
+		ao2_lock(child);
+		instance = child;
+		rtp = ast_rtp_instance_get_data(instance);
+	} else {
+		/* The child is the parent! We don't need to unlock it. */
+		child = NULL;
+	}
+
 	/* If strict RTP protection is enabled see if we need to learn the remote address or if we need to drop the packet */
 	switch (rtp->strict_rtp_state) {
 	case STRICT_RTP_LEARN:
@@ -5431,6 +6753,8 @@ static struct ast_frame *ast_rtp_read(st
 			&& STRICT_RTP_LEARN_TIMEOUT < ast_tvdiff_ms(ast_tvnow(), rtp->rtp_source_learn.start)) {
 			ast_verb(4, "%p -- Strict RTP learning complete - Locking on source address %s\n",
 				rtp, ast_sockaddr_stringify(&rtp->strict_rtp_address));
+			ast_test_suite_event_notify("STRICT_RTP_LEARN", "Source: %s",
+				ast_sockaddr_stringify(&rtp->strict_rtp_address));
 			rtp->strict_rtp_state = STRICT_RTP_CLOSED;
 		} else {
 			struct ast_sockaddr target_address;
@@ -5517,6 +6841,16 @@ static struct ast_frame *ast_rtp_read(st
 		}
 		ast_debug(1, "%p -- Received RTP packet from %s, dropping due to strict RTP protection.\n",
 			rtp, ast_sockaddr_stringify(&addr));
+#ifdef TEST_FRAMEWORK
+	{
+		static int strict_rtp_test_event = 1;
+		if (strict_rtp_test_event) {
+			ast_test_suite_event_notify("STRICT_RTP_CLOSED", "Source: %s",
+				ast_sockaddr_stringify(&addr));
+			strict_rtp_test_event = 0; /* Only run this event once to prevent possible spam */
+		}
+	}
+#endif
 		return &ast_null_frame;
 	case STRICT_RTP_OPEN:
 		break;
@@ -5542,303 +6876,312 @@ static struct ast_frame *ast_rtp_read(st
 
 	/* Pull out the various other fields we will need */
 	payloadtype = (seqno & 0x7f0000) >> 16;
-	padding = seqno & (1 << 29);
 	mark = seqno & (1 << 23);
-	ext = seqno & (1 << 28);
-	cc = (seqno & 0xF000000) >> 24;
 	seqno &= 0xffff;
 	timestamp = ntohl(rtpheader[1]);
-	ssrc = ntohl(rtpheader[2]);
+
+	if (rtp_debug_test_addr(&addr)) {
+		ast_verbose("Got  RTP packet from    %s (type %-2.2d, seq %-6.6u, ts %-6.6u, len %-6.6d)\n",
+			    ast_sockaddr_stringify(&addr),
+			    payloadtype, seqno, timestamp, res - hdrlen);
+	}
 
 	AST_LIST_HEAD_INIT_NOLOCK(&frames);
-	/* Force a marker bit and change SSRC if the SSRC changes */
-	if (rtp->themssrc_valid && rtp->themssrc != ssrc) {
-		struct ast_frame *f, srcupdate = {
-			AST_FRAME_CONTROL,
-			.subclass.integer = AST_CONTROL_SRCCHANGE,
-		};
 
-		if (!mark) {
-			if (rtpdebug) {
-				ast_debug(1, "Forcing Marker bit, because SSRC has changed\n");
-			}
-			mark = 1;
-		}
-
-		f = ast_frisolate(&srcupdate);
-		AST_LIST_INSERT_TAIL(&frames, f, frame_list);
-
-		rtp->seedrxseqno = 0;
-		rtp->rxcount = 0;
-		rtp->rxoctetcount = 0;
-		rtp->cycles = 0;
-		rtp->lastrxseqno = 0;
-		rtp->last_seqno = 0;
-		rtp->last_end_timestamp = 0;
-		if (rtp->rtcp) {
-			rtp->rtcp->expected_prior = 0;
-			rtp->rtcp->received_prior = 0;
-		}
-	}
-	rtp->themssrc = ssrc; /* Record their SSRC to put in future RR */
-	rtp->themssrc_valid = 1;
+	/* Only non-bundled instances can change/learn the remote's SSRC implicitly. */
+	if (!child && !AST_VECTOR_SIZE(&rtp->ssrc_mapping)) {
+		/* Force a marker bit and change SSRC if the SSRC changes */
+		if (rtp->themssrc_valid && rtp->themssrc != ssrc) {
+			struct ast_frame *f, srcupdate = {
+				AST_FRAME_CONTROL,
+				.subclass.integer = AST_CONTROL_SRCCHANGE,
+			};
 
-	/* Remove any padding bytes that may be present */
-	if (padding) {
-		res -= read_area[res - 1];
-	}
+			if (!mark) {
+				if (rtpdebug) {
+					ast_debug(1, "Forcing Marker bit, because SSRC has changed\n");
+				}
+				mark = 1;
+			}
 
-	/* Skip over any CSRC fields */
-	if (cc) {
-		hdrlen += cc * 4;
-	}
+			f = ast_frisolate(&srcupdate);
+			AST_LIST_INSERT_TAIL(&frames, f, frame_list);
 
-	/* Look for any RTP extensions, currently we do not support any */
-	if (ext) {
-		hdrlen += (ntohl(rtpheader[hdrlen/4]) & 0xffff) << 2;
-		hdrlen += 4;
-		if (DEBUG_ATLEAST(1)) {
-			unsigned int profile;
-			profile = (ntohl(rtpheader[3]) & 0xffff0000) >> 16;
-			if (profile == 0x505a) {
-				ast_log(LOG_DEBUG, "Found Zfone extension in RTP stream - zrtp - not supported.\n");
-			} else {
-				ast_log(LOG_DEBUG, "Found unknown RTP Extensions %x\n", profile);
+			rtp->seedrxseqno = 0;
+			rtp->rxcount = 0;
+			rtp->rxoctetcount = 0;
+			rtp->cycles = 0;
+			rtp->lastrxseqno = 0;
+			rtp->last_seqno = 0;
+			rtp->last_end_timestamp = 0;
+			if (rtp->rtcp) {
+				rtp->rtcp->expected_prior = 0;
+				rtp->rtcp->received_prior = 0;
 			}
 		}
-	}
 
-	/* Make sure after we potentially mucked with the header length that it is once again valid */
-	if (res < hdrlen) {
-		ast_log(LOG_WARNING, "RTP Read too short (%d, expecting %d\n", res, hdrlen);
-		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+		rtp->themssrc = ssrc; /* Record their SSRC to put in future RR */
+		rtp->themssrc_valid = 1;
 	}
 
-	rtp->rxcount++;
-	rtp->rxoctetcount += (res - hdrlen);
-	if (rtp->rxcount == 1) {
-		rtp->seedrxseqno = seqno;
-	}
+	prev_seqno = rtp->lastrxseqno;
+	rtp->lastrxseqno = seqno;
 
-	/* Do not schedule RR if RTCP isn't run */
-	if (rtp->rtcp && !ast_sockaddr_isnull(&rtp->rtcp->them) && rtp->rtcp->schedid < 0) {
-		/* Schedule transmission of Receiver Report */
-		ao2_ref(instance, +1);
-		rtp->rtcp->schedid = ast_sched_add(rtp->sched, ast_rtcp_calc_interval(rtp), ast_rtcp_write, instance);
-		if (rtp->rtcp->schedid < 0) {
-			ao2_ref(instance, -1);
-			ast_log(LOG_WARNING, "scheduling RTCP transmission failed.\n");
+	if (!rtp->recv_buffer) {
+		/* If there is no receive buffer then we can pass back the frame directly */
+		return ast_rtp_interpret(instance, srtp, &addr, read_area, res, prev_seqno);
+	} else if (rtp->expectedrxseqno == -1 || seqno == rtp->expectedrxseqno) {
+		rtp->expectedrxseqno = seqno + 1;
+
+		/* If there are no buffered packets that will be placed after this frame then we can
+		 * return it directly without duplicating it.
+		 */
+		if (!ast_data_buffer_count(rtp->recv_buffer)) {
+			return ast_rtp_interpret(instance, srtp, &addr, read_area, res, prev_seqno);
 		}
-	}
-	if ((int)rtp->lastrxseqno - (int)seqno  > 100) /* if so it would indicate that the sender cycled; allow for misordering */
-		rtp->cycles += RTP_SEQ_MOD;
 
-	prev_seqno = rtp->lastrxseqno;
-	rtp->lastrxseqno = seqno;
+		if (!AST_VECTOR_REMOVE_CMP_ORDERED(&rtp->missing_seqno, seqno, find_by_value,
+			AST_VECTOR_ELEM_CLEANUP_NOOP)) {
+			ast_debug(2, "Packet with sequence number '%d' on RTP instance '%p' is no longer missing\n",
+				seqno, instance);
+		}
 
+		/* If we don't have the next packet after this we can directly return the frame, as there is no
+		 * chance it will be overwritten.
+		 */
+		if (!ast_data_buffer_get(rtp->recv_buffer, seqno + 1)) {
+			return ast_rtp_interpret(instance, srtp, &addr, read_area, res, prev_seqno);
+		}
 
-	/* If we are directly bridged to another instance send the audio directly out,
-	 * but only after updating core information about the received traffic so that
-	 * outgoing RTCP reflects it.
-	 */
-	instance1 = ast_rtp_instance_get_bridged(instance);
-	if (instance1
-		&& !bridge_p2p_rtp_write(instance, instance1, rtpheader, res, hdrlen)) {
-		struct timeval rxtime;
-		struct ast_frame *f;
+		/* Otherwise we need to dupe the frame so that the potential processing of frames placed after
+		 * it do not overwrite the data. You may be thinking that we could just add the current packet
+		 * to the head of the frames list and avoid having to duplicate it but this would result in out
+		 * of order packet processing by libsrtp which we are trying to avoid.
+		 */
+		frame = ast_frdup(ast_rtp_interpret(instance, srtp, &addr, read_area, res, seqno - 1));
+		if (frame) {
+			AST_LIST_INSERT_TAIL(&frames, frame, frame_list);
+		}
 
-		/* Update statistics for jitter so they are correct in RTCP */
-		calc_rxstamp(&rxtime, rtp, timestamp, mark);
+		/* Add any additional packets that we have buffered and that are available */
+		while (ast_data_buffer_count(rtp->recv_buffer)) {
+			struct ast_rtp_rtcp_nack_payload *payload;
 
-		/* When doing P2P we don't need to raise any frames about SSRC change to the core */
-		while ((f = AST_LIST_REMOVE_HEAD(&frames, frame_list)) != NULL) {
-			ast_frfree(f);
+			payload = (struct ast_rtp_rtcp_nack_payload *)ast_data_buffer_remove(rtp->recv_buffer, rtp->expectedrxseqno);
+			if (!payload) {
+				break;
+			}
+
+			frame = ast_frdup(ast_rtp_interpret(instance, srtp, &addr, payload->buf, payload->size, rtp->expectedrxseqno - 1));
+			ast_free(payload);
+
+			if (!frame) {
+				/* If this packet can't be interpeted due to being out of memory we return what we have and assume
+				 * that we will determine it is a missing packet later and NACK for it.
+				 */
+				return AST_LIST_FIRST(&frames);
+			}
+
+			ast_debug(2, "Pulled buffered packet with sequence number '%d' to additionally return on RTP instance '%p'\n",
+				frame->seqno, instance);
+			AST_LIST_INSERT_TAIL(&frames, frame, frame_list);
+			rtp->expectedrxseqno++;
 		}
 
-		return &ast_null_frame;
-	}
+		return AST_LIST_FIRST(&frames);
+	} else if ((abs(seqno - rtp->expectedrxseqno) > 100) ||
+		ast_data_buffer_count(rtp->recv_buffer) == ast_data_buffer_max(rtp->recv_buffer)) {
+		int inserted = 0;
 
-	if (rtp_debug_test_addr(&addr)) {
-		ast_verbose("Got  RTP packet from    %s (type %-2.2d, seq %-6.6u, ts %-6.6u, len %-6.6d)\n",
-			    ast_sockaddr_stringify(&addr),
-			    payloadtype, seqno, timestamp,res - hdrlen);
-	}
+		/* We have a large number of outstanding buffered packets or we've jumped far ahead in time.
+		 * To compensate we dump what we have in the buffer and place the current packet in a logical
+		 * spot. In the case of video we also require a full frame to give the decoding side a fighting
+		 * chance.
+		 */
 
-	payload = ast_rtp_codecs_get_payload(ast_rtp_instance_get_codecs(instance), payloadtype);
-	if (!payload) {
-		/* Unknown payload type. */
-		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
-	}
+		if (rtp->rtp_source_learn.stream_type == AST_MEDIA_TYPE_VIDEO) {
+			ast_debug(2, "Source on RTP instance '%p' has wild gap or packet loss, sending FIR\n",
+				instance);
+			rtp_write_rtcp_fir(instance, rtp, &remote_address);
+		}
+
+		while (ast_data_buffer_count(rtp->recv_buffer)) {
+			struct ast_rtp_rtcp_nack_payload *payload;
+
+			payload = (struct ast_rtp_rtcp_nack_payload *)ast_data_buffer_remove_head(rtp->recv_buffer);
+			if (!payload) {
+				continue;
+			}
+
+			/* Even when dumping the receive buffer we do our best to order things, so we ensure that the
+			 * packet we just received is processed in the correct order, so see if we need to insert it now.
+			 */
+			if (!inserted) {
+				int buffer_seqno;
+
+				buffer_seqno = ntohl(payload->buf[0]) & 0xffff;
+				if (seqno < buffer_seqno) {
+					frame = ast_frdup(ast_rtp_interpret(instance, srtp, &addr, read_area, res, prev_seqno));
+					if (frame) {
+						AST_LIST_INSERT_TAIL(&frames, frame, frame_list);
+						rtp->expectedrxseqno = seqno + 1;
+						prev_seqno = seqno;
+						ast_debug(2, "Inserted just received packet with sequence number '%d' in correct order on RTP instance '%p'\n",
+							seqno, instance);
+					}
+					inserted = 1;
+				}
+			}
 
-	/* If the payload is not actually an Asterisk one but a special one pass it off to the respective handler */
-	if (!payload->asterisk_format) {
-		struct ast_frame *f = NULL;
-		if (payload->rtp_code == AST_RTP_DTMF) {
-			/* process_dtmf_rfc2833 may need to return multiple frames. We do this
-			 * by passing the pointer to the frame list to it so that the method
-			 * can append frames to the list as needed.
+			frame = ast_frdup(ast_rtp_interpret(instance, srtp, &addr, payload->buf, payload->size, prev_seqno));
+			if (frame) {
+				AST_LIST_INSERT_TAIL(&frames, frame, frame_list);
+				prev_seqno = frame->seqno;
+				ast_debug(2, "Emptying queue and returning packet with sequence number '%d' from RTP instance '%p'\n",
+					frame->seqno, instance);
+			}
+
+			ast_free(payload);
+		}
+
+		if (!inserted) {
+			/* This current packet goes after them, and we assume that packets going forward will follow
+			 * that new sequence number increment. It is okay for this to not be duplicated as it is guaranteed
+			 * to be the last packet processed right now and it is also guaranteed that it will always return
+			 * non-NULL.
 			 */
-			process_dtmf_rfc2833(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, &addr, payloadtype, mark, &frames);
-		} else if (payload->rtp_code == AST_RTP_CISCO_DTMF) {
-			f = process_dtmf_cisco(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, &addr, payloadtype, mark);
-		} else if (payload->rtp_code == AST_RTP_CN) {
-			f = process_cn_rfc3389(instance, read_area + hdrlen, res - hdrlen, seqno, timestamp, &addr, payloadtype, mark);
-		} else {
-			ast_log(LOG_NOTICE, "Unknown RTP codec %d received from '%s'\n",
-				payloadtype,
-				ast_sockaddr_stringify(&remote_address));
-		}
+			frame = ast_rtp_interpret(instance, srtp, &addr, read_area, res, prev_seqno);
+			AST_LIST_INSERT_TAIL(&frames, frame, frame_list);
+			rtp->expectedrxseqno = seqno + 1;
 
-		if (f) {
-			AST_LIST_INSERT_TAIL(&frames, f, frame_list);
+			ast_debug(2, "Adding just received packet with sequence number '%d' to end of dumped queue on RTP instance '%p'\n",
+				seqno, instance);
 		}
-		/* Even if no frame was returned by one of the above methods,
-		 * we may have a frame to return in our frame list
+
+		/* As there is such a large gap we don't want to flood the order side with missing packets, so we
+		 * give up and start anew.
 		 */
-		return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
-	}
+		AST_VECTOR_RESET(&rtp->missing_seqno, AST_VECTOR_ELEM_CLEANUP_NOOP);
 
-	ao2_replace(rtp->lastrxformat, payload->format);
-	ao2_replace(rtp->f.subclass.format, payload->format);
-	switch (ast_format_get_type(rtp->f.subclass.format)) {
-	case AST_MEDIA_TYPE_AUDIO:
-		rtp->f.frametype = AST_FRAME_VOICE;
-		break;
-	case AST_MEDIA_TYPE_VIDEO:
-		rtp->f.frametype = AST_FRAME_VIDEO;
-		break;
-	case AST_MEDIA_TYPE_TEXT:
-		rtp->f.frametype = AST_FRAME_TEXT;
-		break;
-	case AST_MEDIA_TYPE_IMAGE:
-		/* Fall through */
-	default:
-		ast_log(LOG_WARNING, "Unknown or unsupported media type: %s\n",
-			ast_codec_media_type2str(ast_format_get_type(rtp->f.subclass.format)));
+		return AST_LIST_FIRST(&frames);
+	} else if (seqno < rtp->expectedrxseqno) {
+		/* If this is a packet from the past then we have received a duplicate packet, so just drop it */
+		ast_debug(2, "Received an old packet with sequence number '%d' on RTP instance '%p', dropping it\n",
+			seqno, instance);
 		return &ast_null_frame;
-	}
-	rtp->rxseqno = seqno;
-
-	if (rtp->dtmf_timeout && rtp->dtmf_timeout < timestamp) {
-		rtp->dtmf_timeout = 0;
-
-		if (rtp->resp) {
-			struct ast_frame *f;
-			f = create_dtmf_frame(instance, AST_FRAME_DTMF_END, 0);
-			f->len = ast_tvdiff_ms(ast_samp2tv(rtp->dtmf_duration, rtp_get_rate(f->subclass.format)), ast_tv(0, 0));
-			rtp->resp = 0;
-			rtp->dtmf_timeout = rtp->dtmf_duration = 0;
-			AST_LIST_INSERT_TAIL(&frames, f, frame_list);
-			return AST_LIST_FIRST(&frames);
+	} else if (ast_data_buffer_get(rtp->recv_buffer, seqno)) {
+		/* If this is a packet we already have buffered then it is a duplicate, so just drop it */
+		ast_debug(2, "Received a duplicate transmission of packet with sequence number '%d' on RTP instance '%p', dropping it\n",
+			seqno, instance);
+		return &ast_null_frame;
+	} else {
+		/* This is an out of order packet from the future */
+		struct ast_rtp_rtcp_nack_payload *payload;
+		int difference;
+
+		ast_debug(2, "Received an out of order packet with sequence number '%d' from the future on RTP instance '%p'\n",
+			seqno, instance);
+
+		payload = ast_malloc(sizeof(*payload) + res);
+		if (!payload) {
+			/* If the payload can't be allocated then we can't defer this packet right now.
+			 * Instead of dumping what we have we pretend we lost this packet. It will then
+			 * get NACKed later or the existing buffer will be returned entirely. Well, we may
+			 * try since we're seemingly out of memory. It's a bad situation all around and
+			 * packets are likely to get lost anyway.
+			 */
+			return &ast_null_frame;
 		}
-	}
 
-	rtp->lastrxts = timestamp;
+		payload->size = res;
+		memcpy(payload->buf, rtpheader, res);
+		ast_data_buffer_put(rtp->recv_buffer, seqno, payload);
+		AST_VECTOR_REMOVE_CMP_ORDERED(&rtp->missing_seqno, seqno, find_by_value,
+			AST_VECTOR_ELEM_CLEANUP_NOOP);
+
+		difference = seqno - (prev_seqno + 1);
+		while (difference > 0) {
+			/* We don't want missing sequence number duplicates. If, for some reason,
+			 * packets are really out of order, we could end up in this scenario:
+			 *
+			 * We are expecting sequence number 100
+			 * We receive sequence number 105
+			 * Sequence numbers 100 through 104 get added to the vector
+			 * We receive sequence number 101 (this section is skipped)
+			 * We receive sequence number 103
+			 * Sequence number 102 is added to the vector
+			 *
+			 * This will prevent the duplicate from being added.
+			 */
+			if (AST_VECTOR_GET_CMP(&rtp->missing_seqno, seqno - difference,
+						find_by_value)) {
+				difference--;
+				continue;
+			}
 
-	rtp->f.src = "RTP";
-	rtp->f.mallocd = 0;
-	rtp->f.datalen = res - hdrlen;
-	rtp->f.data.ptr = read_area + hdrlen;
-	rtp->f.offset = hdrlen + AST_FRIENDLY_OFFSET;
-	rtp->f.seqno = seqno;
+			ast_debug(2, "Added missing sequence number '%d' to RTP instance '%p'\n",
+				seqno - difference, instance);
+			AST_VECTOR_ADD_SORTED(&rtp->missing_seqno, seqno - difference,
+					compare_by_value);
+			difference--;
+		}
 
-	if ((ast_format_cmp(rtp->f.subclass.format, ast_format_t140) == AST_FORMAT_CMP_EQUAL)
-		&& ((int)seqno - (prev_seqno + 1) > 0)
-		&& ((int)seqno - (prev_seqno + 1) < 10)) {
-		unsigned char *data = rtp->f.data.ptr;
+		/* When our data buffer is half full we assume that the packets aren't just out of order but
+		 * have actually been lost. To get them back we construct and send a NACK causing the sender to
+		 * retransmit them.
+		 */
+		if (ast_data_buffer_count(rtp->recv_buffer) == ast_data_buffer_max(rtp->recv_buffer) / 2) {
+			int packet_len = 0;
+			int res = 0;
+			int ice;
+			int sr;
+			size_t data_size = AST_UUID_STR_LEN + 128 + (seqno - rtp->expectedrxseqno) / 17;
+			RAII_VAR(unsigned char *, rtcpheader, NULL, ast_free_ptr);
+			RAII_VAR(struct ast_rtp_rtcp_report *, rtcp_report,
+					ast_rtp_rtcp_report_alloc(rtp->themssrc_valid ? 1 : 0),
+					ao2_cleanup);
+
+			rtcpheader = ast_malloc(sizeof(*rtcpheader) + data_size);
+			if (!rtcpheader) {
+				ast_debug(1, "Failed to allocate memory for NACK\n");
+				return &ast_null_frame;
+			}
 
-		memmove(rtp->f.data.ptr+3, rtp->f.data.ptr, rtp->f.datalen);
-		rtp->f.datalen +=3;
-		*data++ = 0xEF;
-		*data++ = 0xBF;
-		*data = 0xBD;
-	}
+			memset(rtcpheader, 0, data_size);
 
-	if (ast_format_cmp(rtp->f.subclass.format, ast_format_t140_red) == AST_FORMAT_CMP_EQUAL) {
-		unsigned char *data = rtp->f.data.ptr;
-		unsigned char *header_end;
-		int num_generations;
-		int header_length;
-		int len;
-		int diff =(int)seqno - (prev_seqno+1); /* if diff = 0, no drop*/
-		int x;
+			res = ast_rtcp_generate_report(instance, rtcpheader, rtcp_report, &sr);
 
-		ao2_replace(rtp->f.subclass.format, ast_format_t140);
-		header_end = memchr(data, ((*data) & 0x7f), rtp->f.datalen);
-		if (header_end == NULL) {
-			return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
-		}
-		header_end++;
+			if (res == 0 || res == 1) {
+				ast_debug(1, "Failed to add %s report to NACK, stopping here\n", sr ? "SR" : "RR");
+				return &ast_null_frame;
+			}
 
-		header_length = header_end - data;
-		num_generations = header_length / 4;
-		len = header_length;
+			packet_len += res;
 
-		if (!diff) {
-			for (x = 0; x < num_generations; x++)
-				len += data[x * 4 + 3];
+			res = ast_rtcp_generate_nack(instance, rtcpheader + packet_len);
 
-			if (!(rtp->f.datalen - len))
-				return AST_LIST_FIRST(&frames) ? AST_LIST_FIRST(&frames) : &ast_null_frame;
+			if (res == 0) {
+				ast_debug(1, "Failed to construct NACK, stopping here\n");
+				return &ast_null_frame;
+			}
 
-			rtp->f.data.ptr += len;
-			rtp->f.datalen -= len;
-		} else if (diff > num_generations && diff < 10) {
-			len -= 3;
-			rtp->f.data.ptr += len;
-			rtp->f.datalen -= len;
+			packet_len += res;
 
-			data = rtp->f.data.ptr;
-			*data++ = 0xEF;
-			*data++ = 0xBF;
-			*data = 0xBD;
-		} else {
-			for ( x = 0; x < num_generations - diff; x++)
-				len += data[x * 4 + 3];
+			res = rtcp_sendto(instance, rtcpheader, packet_len, 0, &remote_address, &ice);
+			if (res < 0) {
+				ast_debug(1, "Failed to send NACK request out\n");
+			} else {
+				/* Update RTCP SR/RR statistics */
+				ast_rtcp_calculate_sr_rr_statistics(instance, rtcp_report, remote_address, ice, sr);
+			}
 
-			rtp->f.data.ptr += len;
-			rtp->f.datalen -= len;
+			ast_debug(2, "Sending a NACK request on RTP instance '%p' to get missing packets\n", instance);
 		}
-	}
 
-	if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_AUDIO) {
-		rtp->f.samples = ast_codec_samples_count(&rtp->f);
-		if (ast_format_cache_is_slinear(rtp->f.subclass.format)) {
-			ast_frame_byteswap_be(&rtp->f);
-		}
-		calc_rxstamp(&rtp->f.delivery, rtp, timestamp, mark);
-		/* Add timing data to let ast_generic_bridge() put the frame into a jitterbuf */
-		ast_set_flag(&rtp->f, AST_FRFLAG_HAS_TIMING_INFO);
-		rtp->f.ts = timestamp / (rtp_get_rate(rtp->f.subclass.format) / 1000);
-		rtp->f.len = rtp->f.samples / ((ast_format_get_sample_rate(rtp->f.subclass.format) / 1000));
-	} else if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_VIDEO) {
-		/* Video -- samples is # of samples vs. 90000 */
-		if (!rtp->lastividtimestamp)
-			rtp->lastividtimestamp = timestamp;
-		ast_set_flag(&rtp->f, AST_FRFLAG_HAS_TIMING_INFO);
-		rtp->f.ts = timestamp / (rtp_get_rate(rtp->f.subclass.format) / 1000);
-		rtp->f.samples = timestamp - rtp->lastividtimestamp;
-		rtp->lastividtimestamp = timestamp;
-		rtp->f.delivery.tv_sec = 0;
-		rtp->f.delivery.tv_usec = 0;
-		/* Pass the RTP marker bit as bit */
-		rtp->f.subclass.frame_ending = mark ? 1 : 0;
-	} else if (ast_format_get_type(rtp->f.subclass.format) == AST_MEDIA_TYPE_TEXT) {
-		/* TEXT -- samples is # of samples vs. 1000 */
-		if (!rtp->lastitexttimestamp)
-			rtp->lastitexttimestamp = timestamp;
-		rtp->f.samples = timestamp - rtp->lastitexttimestamp;
-		rtp->lastitexttimestamp = timestamp;
-		rtp->f.delivery.tv_sec = 0;
-		rtp->f.delivery.tv_usec = 0;
-	} else {
-		ast_log(LOG_WARNING, "Unknown or unsupported media type: %s\n",
-			ast_codec_media_type2str(ast_format_get_type(rtp->f.subclass.format)));
-		return &ast_null_frame;;
+		return &ast_null_frame;
 	}
 
-	AST_LIST_INSERT_TAIL(&frames, &rtp->f, frame_list);
-	return AST_LIST_FIRST(&frames);
+	return &ast_null_frame;
 }
 
 /*! \pre instance is locked */
@@ -5988,6 +7331,11 @@ static void ast_rtp_prop_set(struct ast_
 		}
 	} else if (property == AST_RTP_PROPERTY_ASYMMETRIC_CODEC) {
 		rtp->asymmetric_codec = value;
+	} else if (property == AST_RTP_PROPERTY_RETRANS_SEND) {
+		rtp->send_buffer = ast_data_buffer_alloc(ast_free_ptr, DEFAULT_RTP_SEND_BUFFER_SIZE);
+	} else if (property == AST_RTP_PROPERTY_RETRANS_RECV) {
+		rtp->recv_buffer = ast_data_buffer_alloc(ast_free_ptr, DEFAULT_RTP_RECV_BUFFER_SIZE);
+		AST_VECTOR_INIT(&rtp->missing_seqno, 0);
 	}
 }
 
@@ -6004,6 +7352,7 @@ static void ast_rtp_remote_address_set(s
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 	struct ast_sockaddr local;
+	int index;
 
 	ast_rtp_instance_get_local_address(instance, &local);
 	if (!ast_sockaddr_isnull(addr)) {
@@ -6032,6 +7381,13 @@ static void ast_rtp_remote_address_set(s
 		rtp->rtcp->local_addr_str = ast_strdup(ast_sockaddr_stringify(&local));
 	}
 
+	/* Update any bundled RTP instances */
+	for (index = 0; index < AST_VECTOR_SIZE(&rtp->ssrc_mapping); ++index) {
+		struct rtp_ssrc_mapping *mapping = AST_VECTOR_GET_ADDR(&rtp->ssrc_mapping, index);
+
+		ast_rtp_instance_set_remote_address(mapping->instance, addr);
+	}
+
 	rtp->rxseqno = 0;
 
 	if (strictrtp && rtp->strict_rtp_state != STRICT_RTP_OPEN
@@ -6299,7 +7655,7 @@ static int ast_rtp_sendcng(struct ast_rt
 		return -1;
 	}
 
-	payload = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(instance), 0, NULL, AST_RTP_CN);
+	payload = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(instance), 0, NULL, AST_RTP_CN);
 
 	level = 127 - (level & 0x7f);
 
@@ -6331,32 +7687,157 @@ static int ast_rtp_sendcng(struct ast_rt
 	return res;
 }
 
-#if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
-static void dtls_perform_setup(struct dtls_details *dtls)
+/*! \pre instance is locked */
+static unsigned int ast_rtp_get_ssrc(struct ast_rtp_instance *instance)
 {
-	if (!dtls->ssl || !SSL_is_init_finished(dtls->ssl)) {
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+
+	return rtp->ssrc;
+}
+
+/*! \pre instance is locked */
+static const char *ast_rtp_get_cname(struct ast_rtp_instance *instance)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+
+	return rtp->cname;
+}
+
+/*! \pre instance is locked */
+static void ast_rtp_set_remote_ssrc(struct ast_rtp_instance *instance, unsigned int ssrc)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
+
+	if (rtp->themssrc_valid && rtp->themssrc == ssrc) {
 		return;
 	}
 
-	SSL_clear(dtls->ssl);
-	if (dtls->dtls_setup == AST_RTP_DTLS_SETUP_PASSIVE) {
-		SSL_set_accept_state(dtls->ssl);
-	} else {
-		SSL_set_connect_state(dtls->ssl);
+	rtp->themssrc = ssrc;
+	rtp->themssrc_valid = 1;
+
+	/* If this is bundled we need to update the SSRC mapping */
+	if (rtp->bundled) {
+		struct ast_rtp *bundled_rtp;
+		int index;
+
+		ao2_unlock(instance);
+
+		/* The child lock can't be held while accessing the parent */
+		ao2_lock(rtp->bundled);
+		bundled_rtp = ast_rtp_instance_get_data(rtp->bundled);
+
+		for (index = 0; index < AST_VECTOR_SIZE(&bundled_rtp->ssrc_mapping); ++index) {
+			struct rtp_ssrc_mapping *mapping = AST_VECTOR_GET_ADDR(&bundled_rtp->ssrc_mapping, index);
+
+			if (mapping->instance == instance) {
+				mapping->ssrc = ssrc;
+				mapping->ssrc_valid = 1;
+				break;
+			}
+		}
+
+		ao2_unlock(rtp->bundled);
+
+		ao2_lock(instance);
 	}
-	dtls->connection = AST_RTP_DTLS_CONNECTION_NEW;
 }
 
-/*! \pre instance is locked */
-static int ast_rtp_activate(struct ast_rtp_instance *instance)
+static void ast_rtp_set_stream_num(struct ast_rtp_instance *instance, int stream_num)
 {
 	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 
-	dtls_perform_setup(&rtp->dtls);
+	rtp->stream_num = stream_num;
+}
 
-	if (rtp->rtcp) {
-		dtls_perform_setup(&rtp->rtcp->dtls);
+static int ast_rtp_extension_enable(struct ast_rtp_instance *instance, enum ast_rtp_extension extension)
+{
+	switch (extension) {
+	case AST_RTP_EXTENSION_ABS_SEND_TIME:
+		return 1;
+	default:
+		return 0;
+	}
+}
+
+/*! \pre child is locked */
+static int ast_rtp_bundle(struct ast_rtp_instance *child, struct ast_rtp_instance *parent)
+{
+	struct ast_rtp *child_rtp = ast_rtp_instance_get_data(child);
+	struct ast_rtp *parent_rtp;
+	struct rtp_ssrc_mapping mapping;
+	struct ast_sockaddr them = { { 0, } };
+
+	if (child_rtp->bundled == parent) {
+		return 0;
+	}
+
+	/* If this instance was already bundled then remove the SSRC mapping */
+	if (child_rtp->bundled) {
+		struct ast_rtp *bundled_rtp;
+
+		ao2_unlock(child);
+
+		/* The child lock can't be held while accessing the parent */
+		ao2_lock(child_rtp->bundled);
+		bundled_rtp = ast_rtp_instance_get_data(child_rtp->bundled);
+		AST_VECTOR_REMOVE_CMP_UNORDERED(&bundled_rtp->ssrc_mapping, child, SSRC_MAPPING_ELEM_CMP, AST_VECTOR_ELEM_CLEANUP_NOOP);
+		ao2_unlock(child_rtp->bundled);
+
+		ao2_lock(child);
+		ao2_ref(child_rtp->bundled, -1);
+		child_rtp->bundled = NULL;
+	}
+
+	if (!parent) {
+		/* We transitioned away from bundle so we need our own transport resources once again */
+		rtp_allocate_transport(child, child_rtp);
+		return 0;
+	}
+
+	parent_rtp = ast_rtp_instance_get_data(parent);
+
+	/* We no longer need any transport related resources as we will use our parent RTP instance instead */
+	rtp_deallocate_transport(child, child_rtp);
+
+	/* Children maintain a reference to the parent to guarantee that the transport doesn't go away on them */
+	child_rtp->bundled = ao2_bump(parent);
+
+	mapping.ssrc = child_rtp->themssrc;
+	mapping.ssrc_valid = child_rtp->themssrc_valid;
+	mapping.instance = child;
+
+	ao2_unlock(child);
+
+	ao2_lock(parent);
+
+	AST_VECTOR_APPEND(&parent_rtp->ssrc_mapping, mapping);
+
+#if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
+	/* If DTLS-SRTP is already in use then add the local SSRC to it, otherwise it will get added once DTLS
+	 * negotiation has been completed.
+	 */
+	if (parent_rtp->dtls.connection == AST_RTP_DTLS_CONNECTION_EXISTING) {
+		dtls_srtp_add_local_ssrc(parent_rtp, parent, 0, child_rtp->ssrc, 0);
 	}
+#endif
+
+	/* Bundle requires that RTCP-MUX be in use so only the main remote address needs to match */
+	ast_rtp_instance_get_remote_address(parent, &them);
+
+	ao2_unlock(parent);
+
+	ao2_lock(child);
+
+	ast_rtp_instance_set_remote_address(child, &them);
+
+	return 0;
+}
+
+#if defined(HAVE_OPENSSL) && (OPENSSL_VERSION_NUMBER >= 0x10001000L) && !defined(OPENSSL_NO_SRTP)
+/*! \pre instance is locked */
+static int ast_rtp_activate(struct ast_rtp_instance *instance)
+{
+	struct ast_rtp *rtp = ast_rtp_instance_get_data(instance);
 
 	/* If ICE negotiation is enabled the DTLS Handshake will be performed upon completion of it */
 #ifdef HAVE_PJPROJECT
@@ -6365,9 +7846,11 @@ static int ast_rtp_activate(struct ast_r
 	}
 #endif
 
+	dtls_perform_setup(&rtp->dtls);
 	dtls_perform_handshake(instance, &rtp->dtls, 0);
 
 	if (rtp->rtcp && rtp->rtcp->type == AST_RTP_INSTANCE_RTCP_STANDARD) {
+		dtls_perform_setup(&rtp->rtcp->dtls);
 		dtls_perform_handshake(instance, &rtp->rtcp->dtls, 1);
 	}
 
@@ -6584,8 +8067,13 @@ static int rtp_reload(int reload)
 		return 0;
 	}
 
+#ifdef SO_NO_CHECK
+	nochecksums = 0;
+#endif
+
 	rtpstart = DEFAULT_RTP_START;
 	rtpend = DEFAULT_RTP_END;
+	rtcpinterval = RTCP_DEFAULT_INTERVALMS;
 	dtmftimeout = DEFAULT_DTMF_TIMEOUT;
 	strictrtp = DEFAULT_STRICT_RTP;
 	learning_min_sequential = DEFAULT_LEARNING_MIN_SEQUENTIAL;
@@ -6651,7 +8139,13 @@ static int rtp_reload(int reload)
 		};
 	}
 	if ((s = ast_variable_retrieve(cfg, "general", "strictrtp"))) {
-		strictrtp = ast_true(s);
+		if (ast_true(s)) {
+			strictrtp = STRICT_RTP_YES;
+		} else if (!strcasecmp(s, "seqno")) {
+			strictrtp = STRICT_RTP_SEQNO;
+		} else {
+			strictrtp = STRICT_RTP_NO;
+		}
 	}
 	if ((s = ast_variable_retrieve(cfg, "general", "probation"))) {
 		if ((sscanf(s, "%d", &learning_min_sequential) != 1) || learning_min_sequential <= 1) {
@@ -6838,9 +8332,12 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Asterisk RTP Stack",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+#ifdef HAVE_PJPROJECT
+	.requires = "res_pjproject",
+#endif
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_rtp_multicast.c 1:16.1.1~dfsg-1/res/res_rtp_multicast.c
--- 1:13.23.1~dfsg-2/res/res_rtp_multicast.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_rtp_multicast.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -481,7 +479,9 @@ static int multicast_rtp_write(struct as
 	}
 
 	/* Grab the actual payload number for when we create the RTP packet */
-	if ((codec = ast_rtp_codecs_payload_code(ast_rtp_instance_get_codecs(instance), 1, frame->subclass.format, 0)) < 0) {
+	codec = ast_rtp_codecs_payload_code_tx(ast_rtp_instance_get_codecs(instance),
+		1, frame->subclass.format, 0);
+	if (codec < 0) {
 		return -1;
 	}
 
diff -pruN 1:13.23.1~dfsg-2/res/res_security_log.c 1:16.1.1~dfsg-1/res/res_security_log.c
--- 1:13.23.1~dfsg-2/res/res_security_log.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_security_log.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/module.h"
 #include "asterisk/logger.h"
 #include "asterisk/threadstorage.h"
@@ -143,6 +141,8 @@ static int load_module(void)
 		LOG_SECURITY = -1;
 		return AST_MODULE_LOAD_DECLINE;
 	}
+	stasis_subscription_accept_message_type(security_stasis_sub, ast_security_event_type());
+	stasis_subscription_set_filter(security_stasis_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	ast_verb(3, "Security Logging Enabled\n");
 
diff -pruN 1:13.23.1~dfsg-2/res/res_smdi.c 1:16.1.1~dfsg-1/res/res_smdi.c
--- 1:13.23.1~dfsg-2/res/res_smdi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_smdi.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,13 +41,11 @@
  */
 
 /*** MODULEINFO
-	<support_level>core</support_level>
+	<support_level>extended</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <termios.h>
 #include <sys/time.h>
 #include <time.h>
@@ -164,7 +162,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
  ***/
 
 static const char config_file[] = "smdi.conf";
-static int smdi_loaded;
 
 struct ast_smdi_interface {
 	char name[SMDI_MAX_FILENAME_LEN];
@@ -226,10 +223,12 @@ static struct {
 static void smdi_interface_destroy(void *obj)
 {
 	struct ast_smdi_interface *iface = obj;
+	int mod_unref_defer = 0;
 
 	if (iface->thread != AST_PTHREADT_NULL && iface->thread != AST_PTHREADT_STOP) {
 		pthread_cancel(iface->thread);
 		pthread_join(iface->thread, NULL);
+		mod_unref_defer = 1;
 	}
 
 	iface->thread = AST_PTHREADT_STOP;
@@ -246,9 +245,9 @@ static void smdi_interface_destroy(void
 	ast_mutex_destroy(&iface->mwi_q_lock);
 	ast_cond_destroy(&iface->mwi_q_cond);
 
-	free(iface);
-
-	ast_module_unref(ast_module_info->self);
+	if (mod_unref_defer) {
+		ast_module_unref(ast_module_info->self);
+	}
 }
 
 /*!
@@ -611,7 +610,6 @@ static void *smdi_read(void *iface_p)
 
 			md_msg = ao2_alloc(sizeof(*md_msg), NULL);
 			if (!md_msg) {
-				ao2_ref(iface, -1);
 				return NULL;
 			}
 
@@ -621,7 +619,6 @@ static void *smdi_read(void *iface_p)
 				if (c == EOF) {
 					ast_log(LOG_ERROR, "Unexpected EOF while reading MD message\n");
 					ao2_ref(md_msg, -1);
-					ao2_ref(iface, -1);
 					return NULL;
 				}
 				md_msg->mesg_desk_num[i] = (char) c;
@@ -638,7 +635,6 @@ static void *smdi_read(void *iface_p)
 				if (c == EOF) {
 					ast_log(LOG_ERROR, "Unexpected EOF while reading SMDI message\n");
 					ao2_ref(md_msg, -1);
-					ao2_ref(iface, -1);
 					return NULL;
 				}
 				md_msg->mesg_desk_term[i] = (char) c;
@@ -654,7 +650,6 @@ static void *smdi_read(void *iface_p)
 			if (c == EOF) {
 				ast_log(LOG_ERROR, "Unexpected EOF while reading SMDI message\n");
 				ao2_ref(md_msg, -1);
-				ao2_ref(iface, -1);
 				return NULL;
 			}
 			md_msg->type = (char) c;
@@ -733,7 +728,6 @@ static void *smdi_read(void *iface_p)
 
 			mwi_msg = ao2_alloc(sizeof(*mwi_msg), NULL);
 			if (!mwi_msg) {
-				ao2_ref(iface, -1);
 				return NULL;
 			}
 
@@ -767,7 +761,6 @@ static void *smdi_read(void *iface_p)
 				if (c == EOF) {
 					ast_log(LOG_ERROR, "Unexpected EOF while reading MWI message\n");
 					ao2_ref(mwi_msg, -1);
-					ao2_ref(iface, -1);
 					return NULL;
 				}
 				mwi_msg->cause[i] = (char) c;
@@ -788,7 +781,6 @@ static void *smdi_read(void *iface_p)
 	}
 
 	ast_log(LOG_ERROR, "Error reading from SMDI interface %s, stopping listener thread\n", iface->name);
-	ao2_ref(iface, -1);
 	return NULL;
 }
 
@@ -796,7 +788,7 @@ static void destroy_mailbox_mapping(stru
 {
 	ast_string_field_free_memory(mm);
 	ao2_ref(mm->iface, -1);
-	free(mm);
+	ast_free(mm);
 }
 
 static void destroy_all_mailbox_mappings(void)
@@ -985,6 +977,11 @@ static int smdi_load(int reload)
 		return 0;
 
 	new_ifaces = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, smdi_ifaces_cmp_fn);
+	if (!new_ifaces) {
+		ast_config_destroy(conf);
+		return -1;
+	}
+
 	for (v = ast_variable_browse(conf, "interfaces"); v; v = v->next) {
 		RAII_VAR(struct ast_smdi_interface *, iface, NULL, ao2_cleanup);
 
@@ -1037,7 +1034,7 @@ static int smdi_load(int reload)
 		} else if (!strcasecmp(v->name, "twostopbits")) {
 			stopbits = ast_true(v->name);
 		} else if (!strcasecmp(v->name, "smdiport")) {
-			if (reload) {
+			if (reload && old_ifaces) {
 				/* we are reloading, check if we are already
 				 * monitoring this interface, if we are we do
 				 * not want to start it again.  This also has
@@ -1107,7 +1104,13 @@ static int smdi_load(int reload)
 			/* set the message expiry time */
 			iface->msg_expiry = msg_expiry;
 
-			/* start the listener thread */
+			/*
+			 * start the listener thread
+			 *
+			 * The listener thread does not actually hold a ref to iface.  When all
+			 * external refs go away, the destructor will stop the listener thread
+			 * before actually destroying the iface object.
+			 */
 			ast_verb(3, "Starting SMDI monitor thread for %s\n", iface->name);
 			if (ast_pthread_create_background(&iface->thread, NULL, smdi_read, iface)) {
 				ast_log(LOG_ERROR, "Error starting SMDI monitor thread for %s\n", iface->name);
@@ -1157,7 +1160,7 @@ static int smdi_load(int reload)
 		return -1;
 	}
 
-	if (ao2_container_count(new_ifaces)) {
+	if (!ao2_container_count(new_ifaces)) {
 		res = 1;
 	}
 
@@ -1177,7 +1180,7 @@ static void smdi_msg_datastore_destroy(v
 	ao2_cleanup(smd->iface);
 	ao2_cleanup(smd->md_msg);
 
-	free(smd);
+	ast_free(smd);
 }
 
 static const struct ast_datastore_info smdi_msg_datastore_info = {
@@ -1373,7 +1376,26 @@ static struct ast_custom_function smdi_m
 	.read = smdi_msg_read,
 };
 
-static int _unload_module(int fromload);
+static int unload_module(void)
+{
+	ao2_global_obj_release(smdi_ifaces);
+
+	destroy_all_mailbox_mappings();
+
+	ast_mutex_lock(&mwi_monitor.lock);
+	mwi_monitor.stop = 1;
+	ast_cond_signal(&mwi_monitor.cond);
+	ast_mutex_unlock(&mwi_monitor.lock);
+
+	if (mwi_monitor.thread != AST_PTHREADT_NULL) {
+		pthread_join(mwi_monitor.thread, NULL);
+	}
+
+	ast_custom_function_unregister(&smdi_msg_retrieve_function);
+	ast_custom_function_unregister(&smdi_msg_function);
+
+	return 0;
+}
 
 /*!
  * \brief Load the module
@@ -1388,7 +1410,6 @@ static int _unload_module(int fromload);
 static int load_module(void)
 {
 	int res;
-	smdi_loaded = 1;
 
 	ast_mutex_init(&mwi_monitor.lock);
 	ast_cond_init(&mwi_monitor.cond, NULL);
@@ -1396,76 +1417,35 @@ static int load_module(void)
 	/* load the config and start the listener threads*/
 	res = smdi_load(0);
 	if (res < 0) {
-		_unload_module(1);
+		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	} else if (res == 1) {
-		_unload_module(1);
 		ast_log(LOG_NOTICE, "No SMDI interfaces are available to listen on, not starting SMDI listener.\n");
-		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	ast_custom_function_register(&smdi_msg_retrieve_function);
 	ast_custom_function_register(&smdi_msg_function);
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-static int _unload_module(int fromload)
-{
-	if (!smdi_loaded) {
-		return 0;
-	}
-
-	ao2_global_obj_release(smdi_ifaces);
-
-	destroy_all_mailbox_mappings();
-
-	ast_mutex_lock(&mwi_monitor.lock);
-	mwi_monitor.stop = 1;
-	ast_cond_signal(&mwi_monitor.cond);
-	ast_mutex_unlock(&mwi_monitor.lock);
-
-	if (mwi_monitor.thread != AST_PTHREADT_NULL) {
-		pthread_join(mwi_monitor.thread, NULL);
-	}
-
-	if (!fromload) {
-		ast_custom_function_unregister(&smdi_msg_retrieve_function);
-		ast_custom_function_unregister(&smdi_msg_function);
-	}
-
-	smdi_loaded = 0;
-
-	return 0;
-}
-
-static int unload_module(void)
-{
-	return _unload_module(0);
-}
-
 static int reload(void)
 {
 	int res;
 
 	res = smdi_load(1);
-
 	if (res < 0) {
 		return res;
 	} else if (res == 1) {
 		ast_log(LOG_WARNING, "No SMDI interfaces were specified to listen on, not starting SDMI listener.\n");
-		return 0;
-	} else
-		return 0;
+	}
+	return 0;
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Simplified Message Desk Interface (SMDI) Resource",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_snmp.c 1:16.1.1~dfsg-1/res/res_snmp.c
--- 1:13.23.1~dfsg-2/res/res_snmp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_snmp.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 
@@ -136,7 +134,7 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "SNMP [Sub]Agent for Asterisk",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_sorcery_astdb.c 1:16.1.1~dfsg-1/res/res_sorcery_astdb.c
--- 1:13.23.1~dfsg-2/res/res_sorcery_astdb.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_sorcery_astdb.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_sorcery_config.c 1:16.1.1~dfsg-1/res/res_sorcery_config.c
--- 1:13.23.1~dfsg-2/res/res_sorcery_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_sorcery_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_sorcery_memory.c 1:16.1.1~dfsg-1/res/res_sorcery_memory.c
--- 1:13.23.1~dfsg-2/res/res_sorcery_memory.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_sorcery_memory.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
@@ -252,7 +250,8 @@ static int sorcery_memory_delete(const s
 
 static void *sorcery_memory_open(const char *data)
 {
-	return ao2_container_alloc(OBJECT_BUCKETS, sorcery_memory_hash, sorcery_memory_cmp);
+	return ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, OBJECT_BUCKETS,
+		sorcery_memory_hash, NULL, sorcery_memory_cmp);
 }
 
 static void sorcery_memory_close(void *data)
diff -pruN 1:13.23.1~dfsg-2/res/res_sorcery_memory_cache.c 1:16.1.1~dfsg-1/res/res_sorcery_memory_cache.c
--- 1:13.23.1~dfsg-2/res/res_sorcery_memory_cache.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_sorcery_memory_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -1556,9 +1556,9 @@ static void *sorcery_memory_cache_open(c
 		}
 	}
 
-	cache->objects = ao2_container_alloc_options(AO2_ALLOC_OPT_LOCK_RWLOCK,
+	cache->objects = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK, 0,
 		cache->maximum_objects ? cache->maximum_objects : CACHE_CONTAINER_BUCKET_SIZE,
-		sorcery_memory_cached_object_hash, sorcery_memory_cached_object_cmp);
+		sorcery_memory_cached_object_hash, NULL, sorcery_memory_cached_object_cmp);
 	if (!cache->objects) {
 		ast_log(LOG_ERROR, "Could not create a container to hold cached objects for memory cache\n");
 		return NULL;
@@ -3528,7 +3528,8 @@ static int load_module(void)
 {
 	int res;
 
-	caches = ao2_container_alloc(CACHES_CONTAINER_BUCKET_SIZE, sorcery_memory_cache_hash,
+	caches = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		CACHES_CONTAINER_BUCKET_SIZE, sorcery_memory_cache_hash, NULL,
 		sorcery_memory_cache_cmp);
 	if (!caches) {
 		ast_log(LOG_ERROR, "Failed to create container for configured caches\n");
diff -pruN 1:13.23.1~dfsg-2/res/res_sorcery_realtime.c 1:16.1.1~dfsg-1/res/res_sorcery_realtime.c
--- 1:13.23.1~dfsg-2/res/res_sorcery_realtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_sorcery_realtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/res/res_speech.c 1:16.1.1~dfsg-1/res/res_speech.c
--- 1:13.23.1~dfsg-2/res/res_speech.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_speech.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/channel.h"
 #include "asterisk/module.h"
 #include "asterisk/lock.h"
@@ -362,8 +360,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Generic Speech Recognition API",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_APP_DEPEND - 1,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_APP_DEPEND - 1,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_srtp.c 1:16.1.1~dfsg-1/res/res_srtp.c
--- 1:13.23.1~dfsg-2/res/res_srtp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_srtp.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,9 +36,9 @@
 
 /* See https://wiki.asterisk.org/wiki/display/AST/Secure+Calling */
 
-#include "asterisk.h"
+#include "asterisk.h"                   /* for NULL, size_t, memcpy, etc */
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#include <math.h>                       /* for pow */
 
 #if HAVE_SRTP_VERSION > 1
 # include <srtp2/srtp.h>
@@ -53,12 +53,14 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 # endif
 #endif
 
-#include "asterisk/lock.h"
-#include "asterisk/sched.h"
-#include "asterisk/module.h"
-#include "asterisk/options.h"
-#include "asterisk/rtp_engine.h"
-#include "asterisk/astobj2.h"
+#include "asterisk/astobj2.h"           /* for ao2_t_ref, etc */
+#include "asterisk/frame.h"             /* for AST_FRIENDLY_OFFSET */
+#include "asterisk/logger.h"            /* for ast_log, ast_debug, etc */
+#include "asterisk/module.h"            /* for ast_module_info, etc */
+#include "asterisk/sdp_srtp.h"
+#include "asterisk/res_srtp.h"          /* for ast_srtp_cb, ast_srtp_suite, etc */
+#include "asterisk/rtp_engine.h"        /* for ast_rtp_engine_register_srtp, etc */
+#include "asterisk/utils.h"             /* for ast_free, ast_calloc */
 
 struct ast_srtp {
 	struct ast_rtp_instance *rtp;
@@ -192,7 +194,9 @@ static struct ast_srtp *res_srtp_new(voi
 		return NULL;
 	}
 
-	if (!(srtp->policies = ao2_t_container_alloc(5, policy_hash_fn, policy_cmp_fn, "SRTP policy container"))) {
+	srtp->policies = ao2_t_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 5,
+		policy_hash_fn, NULL, policy_cmp_fn, "SRTP policy container");
+	if (!srtp->policies) {
 		ast_free(srtp);
 		return NULL;
 	}
@@ -264,23 +268,49 @@ static int policy_set_suite(crypto_polic
 {
 	switch (suite) {
 	case AST_AES_CM_128_HMAC_SHA1_80:
-		p->cipher_type = AES_128_ICM;
-		p->cipher_key_len = 30;
-		p->auth_type = HMAC_SHA1;
-		p->auth_key_len = 20;
-		p->auth_tag_len = 10;
-		p->sec_serv = sec_serv_conf_and_auth;
+		crypto_policy_set_aes_cm_128_hmac_sha1_80(p);
 		return 0;
 
 	case AST_AES_CM_128_HMAC_SHA1_32:
-		p->cipher_type = AES_128_ICM;
-		p->cipher_key_len = 30;
-		p->auth_type = HMAC_SHA1;
-		p->auth_key_len = 20;
-		p->auth_tag_len = 4;
-		p->sec_serv = sec_serv_conf_and_auth;
+		crypto_policy_set_aes_cm_128_hmac_sha1_32(p);
 		return 0;
 
+#ifdef HAVE_SRTP_192
+	case AST_AES_CM_192_HMAC_SHA1_80:
+		crypto_policy_set_aes_cm_192_hmac_sha1_80(p);
+		return 0;
+
+	case AST_AES_CM_192_HMAC_SHA1_32:
+		crypto_policy_set_aes_cm_192_hmac_sha1_32(p);
+		return 0;
+#endif
+#ifdef HAVE_SRTP_256
+	case AST_AES_CM_256_HMAC_SHA1_80:
+		crypto_policy_set_aes_cm_256_hmac_sha1_80(p);
+		return 0;
+
+	case AST_AES_CM_256_HMAC_SHA1_32:
+		crypto_policy_set_aes_cm_256_hmac_sha1_32(p);
+		return 0;
+#endif
+#ifdef HAVE_SRTP_GCM
+	case AST_AES_GCM_128:
+		crypto_policy_set_aes_gcm_128_16_auth(p);
+		return 0;
+
+	case AST_AES_GCM_256:
+		crypto_policy_set_aes_gcm_256_16_auth(p);
+		return 0;
+
+	case AST_AES_GCM_128_8:
+		crypto_policy_set_aes_gcm_128_8_auth(p);
+		return 0;
+
+	case AST_AES_GCM_256_8:
+		crypto_policy_set_aes_gcm_256_8_auth(p);
+		return 0;
+#endif
+
 	default:
 		ast_log(LOG_ERROR, "Invalid crypto suite: %u\n", suite);
 		return -1;
@@ -428,11 +458,12 @@ tryagain:
 		 * whether they use a custom library or an outdated version of libSRTP.
 		 */
 		if (rtcp) {
-			ast_verb(2, "SRTCP unprotect failed because of %s\n", srtp_errstr(res));
+			ast_verb(2, "SRTCP unprotect failed on SSRC %u because of %s\n",
+				ast_rtp_instance_get_ssrc(srtp->rtp), srtp_errstr(res));
 		} else {
 			if ((srtp->warned >= 10) && !((srtp->warned - 10) % 150)) {
-				ast_verb(2, "SRTP unprotect failed because of %s %d\n",
-					srtp_errstr(res), srtp->warned);
+				ast_verb(2, "SRTP unprotect failed on SSRC %u because of %s %d\n",
+					ast_rtp_instance_get_ssrc(srtp->rtp), srtp_errstr(res), srtp->warned);
 				srtp->warned = 11;
 			} else {
 				srtp->warned++;
@@ -572,10 +603,586 @@ static int ast_srtp_change_source(struct
 	return 0;
 }
 
+struct ast_sdp_crypto {
+	char *a_crypto;
+	unsigned char local_key[SRTP_MAX_KEY_LEN];
+	int tag;
+	char local_key64[((SRTP_MAX_KEY_LEN) * 8 + 5) / 6 + 1];
+	unsigned char remote_key[SRTP_MAX_KEY_LEN];
+	int key_len;
+};
+
+static void res_sdp_crypto_dtor(struct ast_sdp_crypto *crypto)
+{
+	if (crypto) {
+		ast_free(crypto->a_crypto);
+		crypto->a_crypto = NULL;
+		ast_free(crypto);
+
+		ast_module_unref(ast_module_info->self);
+	}
+}
+
+static struct ast_sdp_crypto *crypto_init_keys(struct ast_sdp_crypto *p, const int key_len)
+{
+	unsigned char remote_key[key_len];
+
+	if (srtp_res.get_random(p->local_key, key_len) < 0) {
+		return NULL;
+	}
+
+	ast_base64encode(p->local_key64, p->local_key, key_len, sizeof(p->local_key64));
+
+	p->key_len = ast_base64decode(remote_key, p->local_key64, sizeof(remote_key));
+
+	if (p->key_len != key_len) {
+		ast_log(LOG_ERROR, "base64 encode/decode bad len %d != %d\n", p->key_len, key_len);
+		return NULL;
+	}
+
+	if (memcmp(remote_key, p->local_key, p->key_len)) {
+		ast_log(LOG_ERROR, "base64 encode/decode bad key\n");
+		return NULL;
+	}
+
+	ast_debug(1 , "local_key64 %s len %zu\n", p->local_key64, strlen(p->local_key64));
+
+	return p;
+}
+
+static struct ast_sdp_crypto *sdp_crypto_alloc(const int key_len)
+{
+	struct ast_sdp_crypto *p, *result;
+
+	if (!(p = ast_calloc(1, sizeof(*p)))) {
+		return NULL;
+	}
+	p->tag = 1;
+	ast_module_ref(ast_module_info->self);
+
+	/* default is a key which uses AST_AES_CM_128_HMAC_SHA1_xx */
+	result = crypto_init_keys(p, key_len);
+	if (!result) {
+		res_sdp_crypto_dtor(p);
+	}
+
+	return result;
+}
+
+static struct ast_sdp_crypto *res_sdp_crypto_alloc(void)
+{
+	return sdp_crypto_alloc(SRTP_MASTER_KEY_LEN);
+}
+
+static int res_sdp_crypto_build_offer(struct ast_sdp_crypto *p, int taglen)
+{
+	int res;
+
+	/* Rebuild the crypto line */
+	ast_free(p->a_crypto);
+	p->a_crypto = NULL;
+
+	if ((taglen & 0x007f) == 8) {
+		res = ast_asprintf(&p->a_crypto, "%d AEAD_AES_%d_GCM_%d inline:%s",
+			p->tag, 128 + ((taglen & 0x0300) >> 2), taglen & 0x007f, p->local_key64);
+	} else if ((taglen & 0x007f) == 16) {
+		res = ast_asprintf(&p->a_crypto, "%d AEAD_AES_%d_GCM inline:%s",
+			p->tag, 128 + ((taglen & 0x0300) >> 2), p->local_key64);
+	} else if ((taglen & 0x0300) && !(taglen & 0x0080)) {
+		res = ast_asprintf(&p->a_crypto, "%d AES_%d_CM_HMAC_SHA1_%d inline:%s",
+			p->tag, 128 + ((taglen & 0x0300) >> 2), taglen & 0x007f, p->local_key64);
+	} else {
+		res = ast_asprintf(&p->a_crypto, "%d AES_CM_%d_HMAC_SHA1_%d inline:%s",
+			p->tag, 128 + ((taglen & 0x0300) >> 2), taglen & 0x007f, p->local_key64);
+	}
+	if (res == -1 || !p->a_crypto) {
+		ast_log(LOG_ERROR, "Could not allocate memory for crypto line\n");
+		return -1;
+	}
+
+	ast_debug(1, "Crypto line: a=crypto:%s\n", p->a_crypto);
+
+	return 0;
+}
+
+static int set_crypto_policy(struct ast_srtp_policy *policy, int suite_val, const unsigned char *master_key, int key_len, unsigned long ssrc, int inbound)
+{
+	if (policy_res.set_master_key(policy, master_key, key_len, NULL, 0) < 0) {
+		return -1;
+	}
+
+	if (policy_res.set_suite(policy, suite_val)) {
+		ast_log(LOG_WARNING, "Could not set remote SRTP suite\n");
+		return -1;
+	}
+
+	policy_res.set_ssrc(policy, ssrc, inbound);
+
+	return 0;
+}
+
+static int crypto_activate(struct ast_sdp_crypto *p, int suite_val, unsigned char *remote_key, int key_len, struct ast_rtp_instance *rtp)
+{
+	struct ast_srtp_policy *local_policy = NULL;
+	struct ast_srtp_policy *remote_policy = NULL;
+	struct ast_rtp_instance_stats stats = {0,};
+	int res = -1;
+
+	if (!p) {
+		return -1;
+	}
+
+	if (!(local_policy = policy_res.alloc())) {
+		return -1;
+	}
+
+	if (!(remote_policy = policy_res.alloc())) {
+		goto err;
+	}
+
+	if (ast_rtp_instance_get_stats(rtp, &stats, AST_RTP_INSTANCE_STAT_LOCAL_SSRC)) {
+		goto err;
+	}
+
+	if (set_crypto_policy(local_policy, suite_val, p->local_key, key_len, stats.local_ssrc, 0) < 0) {
+		goto err;
+	}
+
+	if (set_crypto_policy(remote_policy, suite_val, remote_key, key_len, 0, 1) < 0) {
+		goto err;
+	}
+
+	/* Add the SRTP policies */
+	if (ast_rtp_instance_add_srtp_policy(rtp, remote_policy, local_policy, 0)) {
+		ast_log(LOG_WARNING, "Could not set SRTP policies\n");
+		goto err;
+	}
+
+	ast_debug(1 , "SRTP policy activated\n");
+	res = 0;
+
+err:
+	if (local_policy) {
+		policy_res.destroy(local_policy);
+	}
+
+	if (remote_policy) {
+		policy_res.destroy(remote_policy);
+	}
+
+	return res;
+}
+
+static int res_sdp_crypto_parse_offer(struct ast_rtp_instance *rtp, struct ast_sdp_srtp *srtp, const char *attr)
+{
+	char *str = NULL;
+	char *tag = NULL;
+	char *suite = NULL;
+	char *key_params = NULL;
+	char *key_param = NULL;
+	char *session_params = NULL;
+	char *key_salt = NULL;       /* The actual master key and key salt */
+	char *lifetime = NULL;       /* Key lifetime (# of RTP packets) */
+	char *mki = NULL;            /* Master Key Index */
+	int found = 0;
+	int key_len_from_sdp;
+	int key_len_expected;
+	int tag_from_sdp;
+	int suite_val = 0;
+	unsigned char remote_key[SRTP_MAX_KEY_LEN];
+	int taglen;
+	double sdes_lifetime;
+	struct ast_sdp_crypto *crypto;
+	struct ast_sdp_srtp *tmp;
+
+	str = ast_strdupa(attr);
+
+	tag = strsep(&str, " ");
+	suite = strsep(&str, " ");
+	key_params = strsep(&str, " ");
+	session_params = strsep(&str, " ");
+
+	if (!tag || !suite) {
+		ast_log(LOG_WARNING, "Unrecognized crypto attribute a=%s\n", attr);
+		return -1;
+	}
+
+	/* RFC4568 9.1 - tag is 1-9 digits */
+	if (sscanf(tag, "%30d", &tag_from_sdp) != 1 || tag_from_sdp < 0 || tag_from_sdp > 999999999) {
+		ast_log(LOG_WARNING, "Unacceptable a=crypto tag: %s\n", tag);
+		return -1;
+	}
+
+	if (!ast_strlen_zero(session_params)) {
+		ast_log(LOG_WARNING, "Unsupported crypto parameters: %s\n", session_params);
+		return -1;
+	}
+
+	/* On egress, Asterisk sent several crypto lines in the SIP/SDP offer
+	   The remote party might have choosen another line than the first */
+	for (tmp = srtp; tmp && tmp->crypto && tmp->crypto->tag != tag_from_sdp;) {
+		tmp = AST_LIST_NEXT(tmp, sdp_srtp_list);
+	}
+	if (tmp) { /* tag matched an already created crypto line */
+		unsigned int flags = tmp->flags;
+
+		/* Make that crypto line the head of the list, not by changing the
+		   list structure but by exchanging the content of the list members */
+		crypto = tmp->crypto;
+		tmp->crypto = srtp->crypto;
+		tmp->flags = srtp->flags;
+		srtp->crypto = crypto;
+		srtp->flags = flags;
+	} else {
+		crypto = srtp->crypto;
+		crypto->tag = tag_from_sdp;
+	}
+
+	if (!strcmp(suite, "AES_CM_128_HMAC_SHA1_80")) {
+		suite_val = AST_AES_CM_128_HMAC_SHA1_80;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
+		key_len_expected = 30;
+	} else if (!strcmp(suite, "AES_CM_128_HMAC_SHA1_32")) {
+		suite_val = AST_AES_CM_128_HMAC_SHA1_32;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
+		key_len_expected = 30;
+#ifdef HAVE_SRTP_192
+	} else if (!strcmp(suite, "AES_192_CM_HMAC_SHA1_80")) {
+		suite_val = AST_AES_CM_192_HMAC_SHA1_80;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_192);
+		key_len_expected = 38;
+	} else if (!strcmp(suite, "AES_192_CM_HMAC_SHA1_32")) {
+		suite_val = AST_AES_CM_192_HMAC_SHA1_32;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_192);
+		key_len_expected = 38;
+	/* RFC used a different name while in draft, some still use that */
+	} else if (!strcmp(suite, "AES_CM_192_HMAC_SHA1_80")) {
+		suite_val = AST_AES_CM_192_HMAC_SHA1_80;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_192);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME);
+		key_len_expected = 38;
+	} else if (!strcmp(suite, "AES_CM_192_HMAC_SHA1_32")) {
+		suite_val = AST_AES_CM_192_HMAC_SHA1_32;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_192);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME);
+		key_len_expected = 38;
+#endif
+#ifdef HAVE_SRTP_256
+	} else if (!strcmp(suite, "AES_256_CM_HMAC_SHA1_80")) {
+		suite_val = AST_AES_CM_256_HMAC_SHA1_80;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		key_len_expected = 46;
+	} else if (!strcmp(suite, "AES_256_CM_HMAC_SHA1_32")) {
+		suite_val = AST_AES_CM_256_HMAC_SHA1_32;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		key_len_expected = 46;
+	/* RFC used a different name while in draft, some still use that */
+	} else if (!strcmp(suite, "AES_CM_256_HMAC_SHA1_80")) {
+		suite_val = AST_AES_CM_256_HMAC_SHA1_80;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_80);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME);
+		key_len_expected = 46;
+	} else if (!strcmp(suite, "AES_CM_256_HMAC_SHA1_32")) {
+		suite_val = AST_AES_CM_256_HMAC_SHA1_32;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_32);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME);
+		key_len_expected = 46;
+#endif
+#ifdef HAVE_SRTP_GCM
+	} else if (!strcmp(suite, "AEAD_AES_128_GCM")) {
+		suite_val = AST_AES_GCM_128;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_16);
+		key_len_expected = AES_128_GCM_KEYSIZE_WSALT;
+	} else if (!strcmp(suite, "AEAD_AES_256_GCM")) {
+		suite_val = AST_AES_GCM_256;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_16);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		key_len_expected = AES_256_GCM_KEYSIZE_WSALT;
+	/* RFC contained a (too) short auth tag for RTP media, some still use that */
+	} else if (!strcmp(suite, "AEAD_AES_128_GCM_8")) {
+		suite_val = AST_AES_GCM_128_8;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_8);
+		key_len_expected = AES_128_GCM_KEYSIZE_WSALT;
+	} else if (!strcmp(suite, "AEAD_AES_256_GCM_8")) {
+		suite_val = AST_AES_GCM_256_8;
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_TAG_8);
+		ast_set_flag(srtp, AST_SRTP_CRYPTO_AES_256);
+		key_len_expected = AES_256_GCM_KEYSIZE_WSALT;
+#endif
+	} else {
+		ast_verb(1, "Unsupported crypto suite: %s\n", suite);
+		return -1;
+	}
+
+	while ((key_param = strsep(&key_params, ";"))) {
+		unsigned int n_lifetime;
+		char *method = NULL;
+		char *info = NULL;
+
+		method = strsep(&key_param, ":");
+		info = strsep(&key_param, ";");
+		sdes_lifetime = 0;
+
+		if (strcmp(method, "inline")) {
+			continue;
+		}
+
+		key_salt = strsep(&info, "|");
+
+		/* The next parameter can be either lifetime or MKI */
+		lifetime = strsep(&info, "|");
+		if (!lifetime) {
+			found = 1;
+			break;
+		}
+
+		mki = strchr(lifetime, ':');
+		if (mki) {
+			mki = lifetime;
+			lifetime = NULL;
+		} else {
+			mki = strsep(&info, "|");
+		}
+
+		if (mki && *mki != '1') {
+			ast_log(LOG_NOTICE, "Crypto MKI handling is not supported: ignoring attribute %s\n", attr);
+			continue;
+		}
+
+		if (lifetime) {
+			if (!strncmp(lifetime, "2^", 2)) {
+				char *lifetime_val = lifetime + 2;
+
+				/* Exponential lifetime */
+				if (sscanf(lifetime_val, "%30u", &n_lifetime) != 1) {
+					ast_log(LOG_NOTICE, "Failed to parse lifetime value in crypto attribute: %s\n", attr);
+					continue;
+				}
+
+				if (n_lifetime > 48) {
+					/* Yeah... that's a bit big. */
+					ast_log(LOG_NOTICE, "Crypto lifetime exponent of '%u' is a bit large; using 48\n", n_lifetime);
+					n_lifetime = 48;
+				}
+				sdes_lifetime = pow(2, n_lifetime);
+			} else {
+				/* Decimal lifetime */
+				if (sscanf(lifetime, "%30u", &n_lifetime) != 1) {
+					ast_log(LOG_NOTICE, "Failed to parse lifetime value in crypto attribute: %s\n", attr);
+					continue;
+				}
+				sdes_lifetime = n_lifetime;
+			}
+
+			/* Accept anything above ~5.8 hours. Less than ~5.8; reject. */
+			if (sdes_lifetime < 1048576) {
+				ast_log(LOG_NOTICE, "Rejecting crypto attribute '%s': lifetime '%f' too short\n", attr, sdes_lifetime);
+				continue;
+			}
+		}
+
+		ast_debug(2, "Crypto attribute '%s' accepted with lifetime '%f', MKI '%s'\n",
+			attr, sdes_lifetime, mki ? mki : "-");
+
+		found = 1;
+		break;
+	}
+
+	if (!found) {
+		ast_log(LOG_NOTICE, "SRTP crypto offer not acceptable: '%s'\n", attr);
+		return -1;
+	}
+
+	key_len_from_sdp = ast_base64decode(remote_key, key_salt, sizeof(remote_key));
+	if (key_len_from_sdp != key_len_expected) {
+		ast_log(LOG_WARNING, "SRTP descriptions key length is '%d', not '%d'\n",
+			key_len_from_sdp, key_len_expected);
+		return -1;
+	}
+
+	/* on default, the key is 30 (AES-128); throw that away (only) when the suite changed actually */
+	/* ingress: optional, but saves one expensive call to get_random(.) */
+	/*  egress: required, because the local key was communicated before the remote key is processed */
+	if (crypto->key_len != key_len_from_sdp) {
+		if (!crypto_init_keys(crypto, key_len_from_sdp)) {
+			return -1;
+		}
+	} else if (!memcmp(crypto->remote_key, remote_key, key_len_from_sdp)) {
+		ast_debug(1, "SRTP remote key unchanged; maintaining current policy\n");
+		return 0;
+	}
+
+	if (key_len_from_sdp > sizeof(crypto->remote_key)) {
+		ast_log(LOG_ERROR,
+			"SRTP key buffer is %zu although it must be at least %d bytes\n",
+			sizeof(crypto->remote_key), key_len_from_sdp);
+		return -1;
+	}
+	memcpy(crypto->remote_key, remote_key, key_len_from_sdp);
+
+	if (crypto_activate(crypto, suite_val, remote_key, key_len_from_sdp, rtp) < 0) {
+		return -1;
+	}
+
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_32)) {
+		taglen = 32;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_16)) {
+		taglen = 16;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_8)) {
+		taglen = 8;
+	} else {
+		taglen = 80;
+	}
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_AES_256)) {
+		taglen |= 0x0200;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_AES_192)) {
+		taglen |= 0x0100;
+	}
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME)) {
+		taglen |= 0x0080;
+	}
+
+	/* Finally, rebuild the crypto line */
+	if (res_sdp_crypto_build_offer(crypto, taglen)) {
+		return -1;
+	}
+
+	ast_set_flag(srtp, AST_SRTP_CRYPTO_OFFER_OK);
+	return 0;
+}
+
+static const char *res_sdp_srtp_get_attr(struct ast_sdp_srtp *srtp, int dtls_enabled, int default_taglen_32)
+{
+	int taglen;
+
+	if (!srtp) {
+		return NULL;
+	}
+
+	/* Set encryption properties */
+	if (!srtp->crypto) {
+		if (AST_LIST_NEXT(srtp, sdp_srtp_list)) {
+			srtp->crypto = res_sdp_crypto_alloc();
+			ast_log(LOG_ERROR, "SRTP SDP list was not empty\n");
+		} else {
+			const int len = default_taglen_32 ? AST_SRTP_CRYPTO_TAG_32 : AST_SRTP_CRYPTO_TAG_80;
+			const int attr[][3] = {
+			/* This array creates the following list:
+			 * a=crypto:1 AES_CM_128_HMAC_SHA1_ ...
+			 * a=crypto:2 AEAD_AES_128_GCM ...
+			 * a=crypto:3 AES_256_CM_HMAC_SHA1_ ...
+			 * a=crypto:4 AEAD_AES_256_GCM ...
+			 * a=crypto:5 AES_192_CM_HMAC_SHA1_ ...
+			 * something like 'AEAD_AES_192_GCM' is not specified by the RFCs
+			 *
+			 * If you want to prefer another crypto suite or you want to
+			 * exclude a suite, change this array and recompile Asterisk.
+			 * This list cannot be changed from rtp.conf because you should
+			 * know what you are doing. Especially AES-192 and AES-GCM are
+			 * broken in many VoIP clients, see
+			 * https://github.com/cisco/libsrtp/pull/170
+			 * https://github.com/cisco/libsrtp/pull/184
+			 * Furthermore, AES-GCM uses a shorter crypto-suite string which
+			 * causes Nokia phones based on Symbian/S60 to reject the whole
+			 * INVITE with status 500, even if a matching suite was offered.
+			 * AES-256 might just waste your processor cycles, especially if
+			 * your TLS transport is not secured with equivalent grade, see
+			 * https://security.stackexchange.com/q/61361
+			 * Therefore, AES-128 was preferred here.
+			 *
+			 * If you want to enable one of those defines, please, go for
+			 * CFLAGS='-DENABLE_SRTP_AES_GCM' ./configure && sudo make install
+			 */
+				{ len, 0, 30 },
+#if defined(HAVE_SRTP_GCM) && defined(ENABLE_SRTP_AES_GCM)
+				{ AST_SRTP_CRYPTO_TAG_16, 0, AES_128_GCM_KEYSIZE_WSALT },
+#endif
+#if defined(HAVE_SRTP_256) && defined(ENABLE_SRTP_AES_256)
+				{ len, AST_SRTP_CRYPTO_AES_256, 46 },
+#endif
+#if defined(HAVE_SRTP_GCM) && defined(ENABLE_SRTP_AES_GCM) && defined(ENABLE_SRTP_AES_256)
+				{ AST_SRTP_CRYPTO_TAG_16, AST_SRTP_CRYPTO_AES_256, AES_256_GCM_KEYSIZE_WSALT },
+#endif
+#if defined(HAVE_SRTP_192) && defined(ENABLE_SRTP_AES_192)
+				{ len, AST_SRTP_CRYPTO_AES_192, 38 },
+#endif
+			};
+			struct ast_sdp_srtp *tmp = srtp;
+			int i;
+
+			for (i = 0; i < ARRAY_LEN(attr); i++) {
+				if (attr[i][0]) {
+					ast_set_flag(tmp, attr[i][0]);
+				}
+				if (attr[i][1]) {
+					ast_set_flag(tmp, attr[i][1]);
+				}
+				tmp->crypto = sdp_crypto_alloc(attr[i][2]); /* key_len */
+				tmp->crypto->tag = (i + 1); /* tag starts at 1 */
+
+				if (i < ARRAY_LEN(attr) - 1) {
+					AST_LIST_NEXT(tmp, sdp_srtp_list) = ast_sdp_srtp_alloc();
+					tmp = AST_LIST_NEXT(tmp, sdp_srtp_list);
+				}
+			}
+		}
+	}
+
+	if (dtls_enabled) {
+		/* If DTLS-SRTP is enabled the key details will be pulled from TLS */
+		return NULL;
+	}
+
+	/* set the key length based on INVITE or settings */
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_80)) {
+		taglen = 80;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_32)) {
+		taglen = 32;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_16)) {
+		taglen = 16;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_TAG_8)) {
+		taglen = 8;
+	} else {
+		taglen = default_taglen_32 ? 32 : 80;
+	}
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_AES_256)) {
+		taglen |= 0x0200;
+	} else if (ast_test_flag(srtp, AST_SRTP_CRYPTO_AES_192)) {
+		taglen |= 0x0100;
+	}
+	if (ast_test_flag(srtp, AST_SRTP_CRYPTO_OLD_NAME)) {
+		taglen |= 0x0080;
+	}
+
+	if (srtp->crypto && (res_sdp_crypto_build_offer(srtp->crypto, taglen) >= 0)) {
+		return srtp->crypto->a_crypto;
+	}
+
+	ast_log(LOG_WARNING, "No SRTP key management enabled\n");
+	return NULL;
+}
+
+static struct ast_sdp_crypto_api res_sdp_crypto_api = {
+	.dtor = res_sdp_crypto_dtor,
+	.alloc = res_sdp_crypto_alloc,
+	.build_offer = res_sdp_crypto_build_offer,
+	.parse_offer = res_sdp_crypto_parse_offer,
+	.get_attr = res_sdp_srtp_get_attr,
+};
+
 static void res_srtp_shutdown(void)
 {
-	srtp_install_event_handler(NULL);
+	ast_sdp_crypto_unregister(&res_sdp_crypto_api);
 	ast_rtp_engine_unregister_srtp();
+	srtp_install_event_handler(NULL);
 #ifdef HAVE_SRTP_SHUTDOWN
 	srtp_shutdown();
 #endif
@@ -601,6 +1208,18 @@ static int res_srtp_init(void)
 		return -1;
 	}
 
+	if (ast_sdp_crypto_register(&res_sdp_crypto_api)) {
+		ast_log(AST_LOG_WARNING, "Failed to register SDP SRTP crypto API\n");
+		res_srtp_shutdown();
+		return -1;
+	}
+
+#ifdef HAVE_SRTP_GET_VERSION
+	ast_verb(2, "%s initialized\n", srtp_get_version_string());
+#else
+	ast_verb(2, "libsrtp initialized\n");
+#endif
+
 	g_initialized = 1;
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_answer.c 1:16.1.1~dfsg-1/res/res_stasis_answer.c
--- 1:13.23.1~dfsg-2/res/res_stasis_answer.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_answer.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/stasis_app_impl.h"
 
@@ -75,4 +73,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis");
+	.requires = "res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis.c 1:16.1.1~dfsg-1/res/res_stasis.c
--- 1:13.23.1~dfsg-2/res/res_stasis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis.c	2018-12-26 18:45:13.000000000 +0000
@@ -53,8 +53,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/callerid.h"
 #include "asterisk/module.h"
@@ -150,10 +148,10 @@ static struct ast_json *stasis_start_to_
 		return NULL;
 	}
 
-	msg = ast_json_pack("{s: s, s: o, s: o, s: o}",
+	msg = ast_json_pack("{s: s, s: O, s: O, s: o}",
 		"type", "StasisStart",
-		"timestamp", ast_json_copy(ast_json_object_get(payload->blob, "timestamp")),
-		"args", ast_json_deep_copy(ast_json_object_get(payload->blob, "args")),
+		"timestamp", ast_json_object_get(payload->blob, "timestamp"),
+		"args", ast_json_object_get(payload->blob, "args"),
 		"channel", ast_channel_snapshot_to_json(payload->channel, NULL));
 	if (!msg) {
 		ast_log(LOG_ERROR, "Failed to pack JSON for StasisStart message\n");
@@ -474,29 +472,6 @@ static int bridges_channel_sort_fn(const
 	return cmp;
 }
 
-/*! Removes the bridge to music on hold channel link */
-static void remove_bridge_moh(char *bridge_id)
-{
-	ao2_find(app_bridges_moh, bridge_id, OBJ_SEARCH_KEY | OBJ_UNLINK | OBJ_NODATA);
-	ast_free(bridge_id);
-}
-
-/*! After bridge failure callback for moh channels */
-static void moh_after_bridge_cb_failed(enum ast_bridge_after_cb_reason reason, void *data)
-{
-	char *bridge_id = data;
-
-	remove_bridge_moh(bridge_id);
-}
-
-/*! After bridge callback for moh channels */
-static void moh_after_bridge_cb(struct ast_channel *chan, void *data)
-{
-	char *bridge_id = data;
-
-	remove_bridge_moh(bridge_id);
-}
-
 /*! Request a bridge MOH channel */
 static struct ast_channel *prepare_bridge_moh_channel(void)
 {
@@ -519,11 +494,34 @@ static struct ast_channel *prepare_bridg
 /*! Provides the moh channel with a thread so it can actually play its music */
 static void *moh_channel_thread(void *data)
 {
-	struct ast_channel *moh_channel = data;
+	struct stasis_app_bridge_channel_wrapper *moh_wrapper = data;
+	struct ast_channel *moh_channel = ast_channel_get_by_name(moh_wrapper->channel_id);
+	struct ast_frame *f;
+
+	if (!moh_channel) {
+		ao2_unlink(app_bridges_moh, moh_wrapper);
+		ao2_ref(moh_wrapper, -1);
+		return NULL;
+	}
+
+	/* Read and discard any frame coming from the stasis bridge. */
+	for (;;) {
+		if (ast_waitfor(moh_channel, -1) < 0) {
+			/* Error or hungup */
+			break;
+		}
 
-	while (!ast_safe_sleep(moh_channel, 1000)) {
+		f = ast_read(moh_channel);
+		if (!f) {
+			/* Hungup */
+			break;
+		}
+		ast_frfree(f);
 	}
 
+	ao2_unlink(app_bridges_moh, moh_wrapper);
+	ao2_ref(moh_wrapper, -1);
+
 	ast_moh_stop(moh_channel);
 	ast_hangup(moh_channel);
 
@@ -541,15 +539,10 @@ static void *moh_channel_thread(void *da
  */
 static struct ast_channel *bridge_moh_create(struct ast_bridge *bridge)
 {
-	RAII_VAR(struct stasis_app_bridge_channel_wrapper *, new_wrapper, NULL, ao2_cleanup);
-	RAII_VAR(char *, bridge_id, ast_strdup(bridge->uniqueid), ast_free);
+	struct stasis_app_bridge_channel_wrapper *new_wrapper;
 	struct ast_channel *chan;
 	pthread_t threadid;
 
-	if (!bridge_id) {
-		return NULL;
-	}
-
 	chan = prepare_bridge_moh_channel();
 	if (!chan) {
 		return NULL;
@@ -560,14 +553,6 @@ static struct ast_channel *bridge_moh_cr
 		return NULL;
 	}
 
-	/* The after bridge callback assumes responsibility of the bridge_id. */
-	if (ast_bridge_set_after_callback(chan,
-		moh_after_bridge_cb, moh_after_bridge_cb_failed, bridge_id)) {
-		ast_hangup(chan);
-		return NULL;
-	}
-	bridge_id = NULL;
-
 	if (ast_unreal_channel_push_to_bridge(chan, bridge,
 		AST_BRIDGE_CHANNEL_FLAG_IMMOVABLE | AST_BRIDGE_CHANNEL_FLAG_LONELY)) {
 		ast_hangup(chan);
@@ -581,21 +566,25 @@ static struct ast_channel *bridge_moh_cr
 		return NULL;
 	}
 
-	if (ast_string_field_init(new_wrapper, 32)) {
+	if (ast_string_field_init(new_wrapper, AST_UUID_STR_LEN + AST_CHANNEL_NAME)
+		|| ast_string_field_set(new_wrapper, bridge_id, bridge->uniqueid)
+		|| ast_string_field_set(new_wrapper, channel_id, ast_channel_uniqueid(chan))) {
+		ao2_ref(new_wrapper, -1);
 		ast_hangup(chan);
 		return NULL;
 	}
-	ast_string_field_set(new_wrapper, bridge_id, bridge->uniqueid);
-	ast_string_field_set(new_wrapper, channel_id, ast_channel_uniqueid(chan));
 
 	if (!ao2_link_flags(app_bridges_moh, new_wrapper, OBJ_NOLOCK)) {
+		ao2_ref(new_wrapper, -1);
 		ast_hangup(chan);
 		return NULL;
 	}
 
-	if (ast_pthread_create_detached(&threadid, NULL, moh_channel_thread, chan)) {
+	/* Pass the new_wrapper ref to moh_channel_thread() */
+	if (ast_pthread_create_detached(&threadid, NULL, moh_channel_thread, new_wrapper)) {
 		ast_log(LOG_ERROR, "Failed to create channel thread. Abandoning MOH channel creation.\n");
 		ao2_unlink_flags(app_bridges_moh, new_wrapper, OBJ_NOLOCK);
+		ao2_ref(new_wrapper, -1);
 		ast_hangup(chan);
 		return NULL;
 	}
@@ -769,7 +758,7 @@ static void control_unlink(struct stasis
 	ao2_cleanup(control);
 }
 
-struct ast_bridge *stasis_app_bridge_create(const char *type, const char *name, const char *id)
+static struct ast_bridge *bridge_create_common(const char *type, const char *name, const char *id, int invisible)
 {
 	struct ast_bridge *bridge;
 	char *requested_type, *requested_types = ast_strdupa(S_OR(type, "mixing"));
@@ -777,6 +766,11 @@ struct ast_bridge *stasis_app_bridge_cre
 	int flags = AST_BRIDGE_FLAG_MERGE_INHIBIT_FROM | AST_BRIDGE_FLAG_MERGE_INHIBIT_TO
 		| AST_BRIDGE_FLAG_SWAP_INHIBIT_FROM | AST_BRIDGE_FLAG_SWAP_INHIBIT_TO
 		| AST_BRIDGE_FLAG_TRANSFER_BRIDGE_ONLY;
+	enum ast_bridge_video_mode_type video_mode = AST_BRIDGE_VIDEO_MODE_TALKER_SRC;
+
+	if (invisible) {
+		flags |= AST_BRIDGE_FLAG_INVISIBLE;
+	}
 
 	while ((requested_type = strsep(&requested_types, ","))) {
 		requested_type = ast_strip(requested_type);
@@ -789,9 +783,17 @@ struct ast_bridge *stasis_app_bridge_cre
 		} else if (!strcmp(requested_type, "dtmf_events") ||
 			!strcmp(requested_type, "proxy_media")) {
 			capabilities &= ~AST_BRIDGE_CAPABILITY_NATIVE;
+		} else if (!strcmp(requested_type, "video_sfu")) {
+			video_mode = AST_BRIDGE_VIDEO_MODE_SFU;
 		}
 	}
 
+	/* For an SFU video bridge we ensure it always remains in multimix for the best experience. */
+	if (video_mode == AST_BRIDGE_VIDEO_MODE_SFU) {
+		capabilities = AST_BRIDGE_CAPABILITY_MULTIMIX;
+		flags &= ~AST_BRIDGE_FLAG_SMART;
+	}
+
 	if (!capabilities
 		/* Holding and mixing capabilities don't mix. */
 		|| ((capabilities & AST_BRIDGE_CAPABILITY_HOLDING)
@@ -801,7 +803,15 @@ struct ast_bridge *stasis_app_bridge_cre
 
 	bridge = bridge_stasis_new(capabilities, flags, name, id);
 	if (bridge) {
-		ast_bridge_set_talker_src_video_mode(bridge);
+		if (video_mode == AST_BRIDGE_VIDEO_MODE_SFU) {
+			ast_bridge_set_sfu_video_mode(bridge);
+			/* We require a minimum 5 seconds between video updates to stop floods from clients,
+			 * this should rarely be changed but should become configurable in the future.
+			 */
+			ast_bridge_set_video_update_discard(bridge, 5);
+		} else {
+			ast_bridge_set_talker_src_video_mode(bridge);
+		}
 		if (!ao2_link(app_bridges, bridge)) {
 			ast_bridge_destroy(bridge, 0);
 			bridge = NULL;
@@ -810,6 +820,16 @@ struct ast_bridge *stasis_app_bridge_cre
 	return bridge;
 }
 
+struct ast_bridge *stasis_app_bridge_create(const char *type, const char *name, const char *id)
+{
+	return bridge_create_common(type, name, id, 0);
+}
+
+struct ast_bridge *stasis_app_bridge_create_invisible(const char *type, const char *name, const char *id)
+{
+	return bridge_create_common(type, name, id, 1);
+}
+
 void stasis_app_bridge_destroy(const char *bridge_id)
 {
 	struct ast_bridge *bridge = stasis_app_bridge_find_by_id(bridge_id);
@@ -1273,8 +1293,6 @@ static void remove_stasis_end_published(
 int stasis_app_exec(struct ast_channel *chan, const char *app_name, int argc,
 		    char *argv[])
 {
-	SCOPED_MODULE_USE(ast_module_info->self);
-
 	RAII_VAR(struct stasis_app *, app, NULL, ao2_cleanup);
 	RAII_VAR(struct stasis_app_control *, control, NULL, control_unlink);
 	struct ast_bridge *bridge = NULL;
@@ -1352,7 +1370,7 @@ int stasis_app_exec(struct ast_channel *
 		}
 
 		if (bridge) {
-			/* Bridge is handling channel frames */
+			/* Bridge/dial is handling channel frames */
 			control_wait(control);
 			control_dispatch_all(control, chan);
 			continue;
@@ -1625,11 +1643,6 @@ void stasis_app_register_event_source(st
 {
 	AST_RWLIST_WRLOCK(&event_sources);
 	AST_LIST_INSERT_TAIL(&event_sources, obj, next);
-	/* only need to bump the module ref on non-core sources because the
-	   core ones are [un]registered by this module. */
-	if (!stasis_app_is_core_event_source(obj)) {
-		ast_module_ref(ast_module_info->self);
-	}
 	AST_RWLIST_UNLOCK(&event_sources);
 }
 
@@ -1641,9 +1654,6 @@ void stasis_app_unregister_event_source(
 	AST_RWLIST_TRAVERSE_SAFE_BEGIN(&event_sources, source, next) {
 		if (source == obj) {
 			AST_RWLIST_REMOVE_CURRENT(next);
-			if (!stasis_app_is_core_event_source(obj)) {
-				ast_module_unref(ast_module_info->self);
-			}
 			break;
 		}
 	}
@@ -2007,16 +2017,6 @@ enum stasis_app_user_event_res stasis_ap
 	return STASIS_APP_USER_OK;
 }
 
-void stasis_app_ref(void)
-{
-	ast_module_ref(ast_module_info->self);
-}
-
-void stasis_app_unref(void)
-{
-	ast_module_unref(ast_module_info->self);
-}
-
 static int unload_module(void)
 {
 	stasis_app_unregister_event_sources();
@@ -2024,6 +2024,9 @@ static int unload_module(void)
 	messaging_cleanup();
 
 	cleanup();
+
+	stasis_app_control_shutdown();
+
 	ao2_cleanup(apps_registry);
 	apps_registry = NULL;
 
@@ -2164,12 +2167,15 @@ static int load_module(void)
 	if (STASIS_MESSAGE_TYPE_INIT(end_message_type) != 0) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
-	apps_registry = ao2_container_alloc(APPS_NUM_BUCKETS, app_hash, app_compare);
-	app_controls = ao2_container_alloc(CONTROLS_NUM_BUCKETS, control_hash, control_compare);
-	app_bridges = ao2_container_alloc(BRIDGES_NUM_BUCKETS, bridges_hash, bridges_compare);
+	apps_registry = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		APPS_NUM_BUCKETS, app_hash, NULL, app_compare);
+	app_controls = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		CONTROLS_NUM_BUCKETS, control_hash, NULL, control_compare);
+	app_bridges = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		BRIDGES_NUM_BUCKETS, bridges_hash, NULL, bridges_compare);
 	app_bridges_moh = ao2_container_alloc_hash(
-		AO2_ALLOC_OPT_LOCK_MUTEX, AO2_CONTAINER_ALLOC_OPT_DUPS_REJECT,
-		37, bridges_channel_hash_fn, bridges_channel_sort_fn, NULL);
+		AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		37, bridges_channel_hash_fn, NULL, NULL);
 	app_bridges_playback = ao2_container_alloc_hash(
 		AO2_ALLOC_OPT_LOCK_MUTEX, AO2_CONTAINER_ALLOC_OPT_DUPS_REJECT,
 		37, bridges_channel_hash_fn, bridges_channel_sort_fn, NULL);
@@ -2195,4 +2201,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_device_state.c 1:16.1.1~dfsg-1/res/res_stasis_device_state.c
--- 1:13.23.1~dfsg-2/res/res_stasis_device_state.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_device_state.c	2018-12-26 18:45:13.000000000 +0000
@@ -23,8 +23,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astdb.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
@@ -396,6 +394,9 @@ static int subscribe_device_state(struct
 		ao2_ref(sub, -1);
 		return -1;
 	}
+	stasis_subscription_accept_message_type(sub->sub, ast_device_state_message_type());
+	stasis_subscription_accept_message_type(sub->sub, stasis_subscription_change_type());
+	stasis_subscription_set_filter(sub->sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	ao2_link_flags(device_state_subscriptions, sub, OBJ_NOLOCK);
 	ao2_unlock(device_state_subscriptions);
@@ -460,9 +461,10 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	if (!(device_state_subscriptions = ao2_container_alloc(
-		      DEVICE_STATE_BUCKETS, device_state_subscriptions_hash,
-		      device_state_subscriptions_cmp))) {
+	device_state_subscriptions = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		DEVICE_STATE_BUCKETS, device_state_subscriptions_hash, NULL,
+		device_state_subscriptions_cmp);
+	if (!device_state_subscriptions) {
 		ast_devstate_prov_del(DEVICE_STATE_PROVIDER_STASIS);
 		return AST_MODULE_LOAD_DECLINE;
 	}
@@ -484,4 +486,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis");
+	.requires = "res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_mailbox.c 1:16.1.1~dfsg-1/res/res_stasis_mailbox.c
--- 1:13.23.1~dfsg-2/res/res_stasis_mailbox.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_mailbox.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,8 +24,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astdb.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
@@ -145,17 +143,11 @@ enum stasis_mailbox_result stasis_app_ma
 
 static int load_module(void)
 {
-	/* Must be done first */
-	ast_mwi_external_ref();
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
 static int unload_module(void)
 {
-	/* Must be done last */
-	ast_mwi_external_unref();
-
 	return 0;
 }
 
@@ -163,5 +155,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis,res_mwi_external"
-	);
+	.requires = "res_stasis,res_mwi_external"
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_playback.c 1:16.1.1~dfsg-1/res/res_stasis_playback.c
--- 1:13.23.1~dfsg-2/res/res_stasis_playback.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_playback.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/astobj2.h"
 #include "asterisk/bridge.h"
@@ -70,10 +68,16 @@ static struct ao2_container *playbacks;
 struct stasis_app_playback {
 	AST_DECLARE_STRING_FIELDS(
 		AST_STRING_FIELD(id);	/*!< Playback unique id */
-		AST_STRING_FIELD(media);	/*!< Playback media uri */
+		AST_STRING_FIELD(media);	/*!< The current media playing */
 		AST_STRING_FIELD(language);	/*!< Preferred language */
 		AST_STRING_FIELD(target);       /*!< Playback device uri */
-		);
+	);
+	/*! The list of medias to play back */
+	AST_VECTOR(, char *) medias;
+
+	/*! The current index in \c medias we're playing */
+	size_t media_index;
+
 	/*! Control object for the channel we're playing back to */
 	struct stasis_app_control *control;
 	/*! Number of milliseconds to skip before playing */
@@ -99,15 +103,17 @@ static struct ast_json *playback_to_json
 
 	if (!strcmp(state, "playing")) {
 		type = "PlaybackStarted";
+	} else if (!strcmp(state, "continuing")) {
+		type = "PlaybackContinuing";
 	} else if (!strcmp(state, "done")) {
 		type = "PlaybackFinished";
 	} else {
 		return NULL;
 	}
 
-	return ast_json_pack("{s: s, s: o}",
+	return ast_json_pack("{s: s, s: O}",
 		"type", type,
-		"playback", ast_json_deep_copy(blob));
+		"playback", blob);
 }
 
 STASIS_MESSAGE_TYPE_DEFN(stasis_app_playback_snapshot_type,
@@ -117,6 +123,14 @@ STASIS_MESSAGE_TYPE_DEFN(stasis_app_play
 static void playback_dtor(void *obj)
 {
 	struct stasis_app_playback *playback = obj;
+	int i;
+
+	for (i = 0; i < AST_VECTOR_SIZE(&playback->medias); i++) {
+		char *media = AST_VECTOR_GET(&playback->medias, i);
+
+		ast_free(media);
+	}
+	AST_VECTOR_FREE(&playback->medias);
 
 	ao2_cleanup(playback->control);
 	ast_string_field_free_memory(playback);
@@ -137,6 +151,11 @@ static struct stasis_app_playback *playb
 		return NULL;
 	}
 
+	if (AST_VECTOR_INIT(&playback->medias, 8)) {
+		ao2_ref(playback, -1);
+		return NULL;
+	}
+
 	if (!ast_strlen_zero(id)) {
 		ast_string_field_set(playback, id, id);
 	} else {
@@ -180,6 +199,8 @@ static const char *state_to_string(enum
 		return "playing";
 	case STASIS_PLAYBACK_STATE_PAUSED:
 		return "paused";
+	case STASIS_PLAYBACK_STATE_CONTINUING:
+		return "continuing";
 	case STASIS_PLAYBACK_STATE_STOPPED:
 	case STASIS_PLAYBACK_STATE_COMPLETE:
 	case STASIS_PLAYBACK_STATE_CANCELED:
@@ -241,7 +262,11 @@ static void playback_final_update(struct
 
 	playback->playedms = playedms;
 	if (res == 0) {
-		playback->state = STASIS_PLAYBACK_STATE_COMPLETE;
+		if (playback->media_index == AST_VECTOR_SIZE(&playback->medias) - 1) {
+			playback->state = STASIS_PLAYBACK_STATE_COMPLETE;
+		} else {
+			playback->state = STASIS_PLAYBACK_STATE_CONTINUING;
+		}
 	} else {
 		if (playback->state == STASIS_PLAYBACK_STATE_STOPPED) {
 			ast_log(LOG_NOTICE, "%s: Playback stopped for %s\n",
@@ -262,7 +287,7 @@ static void play_on_channel(struct stasi
 	int res;
 	long offsetms;
 
-	/* Even though these local variables look fairly pointless, the avoid
+	/* Even though these local variables look fairly pointless, they avoid
 	 * having a bunch of NULL's passed directly into
 	 * ast_control_streamfile() */
 	const char *fwd = NULL;
@@ -273,73 +298,83 @@ static void play_on_channel(struct stasi
 
 	ast_assert(playback != NULL);
 
-	offsetms = playback->offsetms;
-
-	res = playback_first_update(playback, ast_channel_uniqueid(chan));
-
-	if (res != 0) {
-		return;
-	}
-
 	if (ast_channel_state(chan) != AST_STATE_UP) {
 		ast_indicate(chan, AST_CONTROL_PROGRESS);
 	}
 
-	if (ast_begins_with(playback->media, SOUND_URI_SCHEME)) {
-		playback->controllable = 1;
+	offsetms = playback->offsetms;
 
-		/* Play sound */
-		res = ast_control_streamfile_lang(chan, playback->media + strlen(SOUND_URI_SCHEME),
-				fwd, rev, stop, pause, restart, playback->skipms, playback->language,
-				&offsetms);
-	} else if (ast_begins_with(playback->media, RECORDING_URI_SCHEME)) {
-		/* Play recording */
-		RAII_VAR(struct stasis_app_stored_recording *, recording, NULL,
-			ao2_cleanup);
-		const char *relname =
-			playback->media + strlen(RECORDING_URI_SCHEME);
-		recording = stasis_app_stored_recording_find_by_name(relname);
-
-		if (!recording) {
-			ast_log(LOG_ERROR, "Attempted to play recording '%s' on channel '%s' but recording does not exist",
-				relname, ast_channel_name(chan));
-			return;
-		}
+	for (; playback->media_index < AST_VECTOR_SIZE(&playback->medias); playback->media_index++) {
 
-		playback->controllable = 1;
+		/* Set the current media to play */
+		ast_string_field_set(playback, media, AST_VECTOR_GET(&playback->medias, playback->media_index));
 
-		res = ast_control_streamfile_lang(chan,
-			stasis_app_stored_recording_get_file(recording), fwd, rev, stop, pause,
-			restart, playback->skipms, playback->language, &offsetms);
-	} else if (ast_begins_with(playback->media, NUMBER_URI_SCHEME)) {
-		int number;
-
-		if (sscanf(playback->media + strlen(NUMBER_URI_SCHEME), "%30d", &number) != 1) {
-			ast_log(LOG_ERROR, "Attempted to play number '%s' on channel '%s' but number is invalid",
-				playback->media + strlen(NUMBER_URI_SCHEME), ast_channel_name(chan));
+		res = playback_first_update(playback, ast_channel_uniqueid(chan));
+		if (res != 0) {
 			return;
 		}
 
-		res = ast_say_number(chan, number, stop, playback->language, NULL);
-	} else if (ast_begins_with(playback->media, DIGITS_URI_SCHEME)) {
-		res = ast_say_digit_str(chan, playback->media + strlen(DIGITS_URI_SCHEME),
-			stop, playback->language);
-	} else if (ast_begins_with(playback->media, CHARACTERS_URI_SCHEME)) {
-		res = ast_say_character_str(chan, playback->media + strlen(CHARACTERS_URI_SCHEME),
-			stop, playback->language, AST_SAY_CASE_NONE);
-	} else if (ast_begins_with(playback->media, TONE_URI_SCHEME)) {
-		playback->controllable = 1;
-		res = ast_control_tone(chan, playback->media + strlen(TONE_URI_SCHEME));
-	} else {
-		/* Play URL */
-		ast_log(LOG_ERROR, "Attempted to play URI '%s' on channel '%s' but scheme is unsupported\n",
-			playback->media, ast_channel_name(chan));
-		return;
-	}
+		if (ast_begins_with(playback->media, SOUND_URI_SCHEME)) {
+			playback->controllable = 1;
+
+			/* Play sound */
+			res = ast_control_streamfile_lang(chan, playback->media + strlen(SOUND_URI_SCHEME),
+					fwd, rev, stop, pause, restart, playback->skipms, playback->language,
+					&offsetms);
+		} else if (ast_begins_with(playback->media, RECORDING_URI_SCHEME)) {
+			/* Play recording */
+			RAII_VAR(struct stasis_app_stored_recording *, recording, NULL,
+				ao2_cleanup);
+			const char *relname =
+				playback->media + strlen(RECORDING_URI_SCHEME);
+			recording = stasis_app_stored_recording_find_by_name(relname);
+
+			if (!recording) {
+				ast_log(LOG_ERROR, "Attempted to play recording '%s' on channel '%s' but recording does not exist",
+					relname, ast_channel_name(chan));
+				continue;
+			}
+
+			playback->controllable = 1;
+
+			res = ast_control_streamfile_lang(chan,
+				stasis_app_stored_recording_get_file(recording), fwd, rev, stop, pause,
+				restart, playback->skipms, playback->language, &offsetms);
+		} else if (ast_begins_with(playback->media, NUMBER_URI_SCHEME)) {
+			int number;
+
+			if (sscanf(playback->media + strlen(NUMBER_URI_SCHEME), "%30d", &number) != 1) {
+				ast_log(LOG_ERROR, "Attempted to play number '%s' on channel '%s' but number is invalid",
+					playback->media + strlen(NUMBER_URI_SCHEME), ast_channel_name(chan));
+				continue;
+			}
+
+			res = ast_say_number(chan, number, stop, playback->language, NULL);
+		} else if (ast_begins_with(playback->media, DIGITS_URI_SCHEME)) {
+			res = ast_say_digit_str(chan, playback->media + strlen(DIGITS_URI_SCHEME),
+				stop, playback->language);
+		} else if (ast_begins_with(playback->media, CHARACTERS_URI_SCHEME)) {
+			res = ast_say_character_str(chan, playback->media + strlen(CHARACTERS_URI_SCHEME),
+				stop, playback->language, AST_SAY_CASE_NONE);
+		} else if (ast_begins_with(playback->media, TONE_URI_SCHEME)) {
+			playback->controllable = 1;
+			res = ast_control_tone(chan, playback->media + strlen(TONE_URI_SCHEME));
+		} else {
+			/* Play URL */
+			ast_log(LOG_ERROR, "Attempted to play URI '%s' on channel '%s' but scheme is unsupported\n",
+				playback->media, ast_channel_name(chan));
+			continue;
+		}
 
-	playback_final_update(playback, offsetms, res,
-		ast_channel_uniqueid(chan));
+		playback_final_update(playback, offsetms, res,
+			ast_channel_uniqueid(chan));
+		if (res == AST_CONTROL_STREAM_STOP) {
+			break;
+		}
 
+		/* Reset offset for any subsequent media */
+		offsetms = 0;
+	}
 	return;
 }
 
@@ -431,30 +466,49 @@ static void set_target_uri(
 }
 
 struct stasis_app_playback *stasis_app_control_play_uri(
-	struct stasis_app_control *control, const char *uri,
-	const char *language, const char *target_id,
+	struct stasis_app_control *control, const char **media,
+	size_t media_count, const char *language, const char *target_id,
 	enum stasis_app_playback_target_type target_type,
 	int skipms, long offsetms, const char *id)
 {
 	struct stasis_app_playback *playback;
+	size_t i;
 
-	if (skipms < 0 || offsetms < 0) {
+	if (skipms < 0 || offsetms < 0 || media_count == 0) {
 		return NULL;
 	}
 
-	ast_debug(3, "%s: Sending play(%s) command\n",
-		stasis_app_control_get_channel_id(control), uri);
-
 	playback = playback_create(control, id);
 	if (!playback) {
 		return NULL;
 	}
 
+	for (i = 0; i < media_count; i++) {
+		char *media_uri;
+
+		media_uri = ast_malloc(strlen(media[i]) + 1);
+	 	if (!media_uri) {
+			ao2_ref(playback, -1);
+			return NULL;
+		}
+
+		ast_debug(3, "%s: Sending play(%s) command\n",
+			stasis_app_control_get_channel_id(control), media[i]);
+
+	    /* safe */
+		strcpy(media_uri, media[i]);
+		if (AST_VECTOR_APPEND(&playback->medias, media_uri)) {
+			ao2_ref(playback, -1);
+			ast_free(media_uri);
+			return NULL;
+		}
+	}
+
 	if (skipms == 0) {
 		skipms = PLAYBACK_DEFAULT_SKIPMS;
 	}
 
-	ast_string_field_set(playback, media, uri);
+	ast_string_field_set(playback, media, AST_VECTOR_GET(&playback->medias, 0));
 	ast_string_field_set(playback, language, language);
 	set_target_uri(playback, target_type, target_id);
 	playback->skipms = skipms;
@@ -497,12 +551,22 @@ struct ast_json *stasis_app_playback_to_
 		return NULL;
 	}
 
-	json = ast_json_pack("{s: s, s: s, s: s, s: s, s: s}",
-		"id", playback->id,
-		"media_uri", playback->media,
-		"target_uri", playback->target,
-		"language", playback->language,
-		"state", state_to_string(playback->state));
+	if (playback->media_index == AST_VECTOR_SIZE(&playback->medias) - 1) {
+		json = ast_json_pack("{s: s, s: s, s: s, s: s, s: s}",
+			"id", playback->id,
+			"media_uri", playback->media,
+			"target_uri", playback->target,
+			"language", playback->language,
+			"state", state_to_string(playback->state));
+	} else {
+		json = ast_json_pack("{s: s, s: s, s: s, s: s, s: s, s: s}",
+			"id", playback->id,
+			"media_uri", playback->media,
+			"next_media_uri", AST_VECTOR_GET(&playback->medias, playback->media_index + 1),
+			"target_uri", playback->target,
+			"language", playback->language,
+			"state", state_to_string(playback->state));
+	}
 
 	return ast_json_ref(json);
 }
@@ -615,6 +679,13 @@ playback_opreation_cb operations[STASIS_
 	[STASIS_PLAYBACK_STATE_PLAYING][STASIS_PLAYBACK_REVERSE] = playback_reverse,
 	[STASIS_PLAYBACK_STATE_PLAYING][STASIS_PLAYBACK_FORWARD] = playback_forward,
 
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_STOP] = playback_stop,
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_RESTART] = playback_restart,
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_PAUSE] = playback_pause,
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_UNPAUSE] = playback_noop,
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_REVERSE] = playback_reverse,
+	[STASIS_PLAYBACK_STATE_CONTINUING][STASIS_PLAYBACK_FORWARD] = playback_forward,
+
 	[STASIS_PLAYBACK_STATE_PAUSED][STASIS_PLAYBACK_STOP] = playback_stop,
 	[STASIS_PLAYBACK_STATE_PAUSED][STASIS_PLAYBACK_PAUSE] = playback_noop,
 	[STASIS_PLAYBACK_STATE_PAUSED][STASIS_PLAYBACK_UNPAUSE] = playback_unpause,
@@ -667,8 +738,8 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	playbacks = ao2_container_alloc(PLAYBACK_BUCKETS, playback_hash,
-		playback_cmp);
+	playbacks = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, PLAYBACK_BUCKETS,
+		playback_hash, NULL, playback_cmp);
 	if (!playbacks) {
 		STASIS_MESSAGE_TYPE_CLEANUP(stasis_app_playback_snapshot_type);
 		return AST_MODULE_LOAD_DECLINE;
@@ -688,4 +759,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis,res_stasis_recording");
+	.requires = "res_stasis,res_stasis_recording"
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_recording.c 1:16.1.1~dfsg-1/res/res_stasis_recording.c
--- 1:13.23.1~dfsg-2/res/res_stasis_recording.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_recording.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/dsp.h"
 #include "asterisk/file.h"
 #include "asterisk/module.h"
@@ -91,9 +89,9 @@ static struct ast_json *recording_to_jso
 		return NULL;
 	}
 
-	return ast_json_pack("{s: s, s: o}",
+	return ast_json_pack("{s: s, s: O}",
 		"type", type,
-		"recording", ast_json_deep_copy(blob));
+		"recording", blob);
 }
 
 STASIS_MESSAGE_TYPE_DEFN(stasis_app_recording_snapshot_type,
@@ -636,8 +634,8 @@ static int load_module(void)
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	recordings = ao2_container_alloc(RECORDING_BUCKETS, recording_hash,
-		recording_cmp);
+	recordings = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, RECORDING_BUCKETS,
+		recording_hash, NULL, recording_cmp);
 	if (!recordings) {
 		STASIS_MESSAGE_TYPE_CLEANUP(stasis_app_recording_snapshot_type);
 		return AST_MODULE_LOAD_DECLINE;
@@ -657,5 +655,6 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis",
-	.load_pri = AST_MODPRI_APP_DEPEND);
+	.requires = "res_stasis",
+	.load_pri = AST_MODPRI_APP_DEPEND
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_snoop.c 1:16.1.1~dfsg-1/res/res_stasis_snoop.c
--- 1:13.23.1~dfsg-2/res/res_stasis_snoop.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_snoop.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/stasis_app_impl.h"
 #include "asterisk/stasis_app_snoop.h"
@@ -447,4 +445,5 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis");
+	.requires = "res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stasis_test.c 1:16.1.1~dfsg-1/res/res_stasis_test.c
--- 1:13.23.1~dfsg-2/res/res_stasis_test.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stasis_test.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test infrastructure for dealing with Stasis.
+ * \file
+ * \brief Test infrastructure for dealing with Stasis.
  *
  * \author David M. Lee, II <dlee@digium.com>
  */
@@ -29,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_test.h"
@@ -101,23 +100,10 @@ struct stasis_message_sink *stasis_messa
  * \brief Implementation of the stasis_message_sink_cb() callback.
  *
  * Why the roundabout way of exposing this via stasis_message_sink_cb()? Well,
- * it has to do with how we load modules.
- *
- * Modules have their own metadata compiled into them in the module info block
- * at the end of the file.  This includes dependency information in the
- * \c nonoptreq field.
- *
- * Asterisk loads the module, inspects the field, then loads any needed
- * dependencies. This works because Asterisk passes \c RTLD_LAZY to the initial
- * dlopen(), which defers binding function references until they are called.
- *
- * But when you take the address of a function, that function needs to be
- * available at load time. So if some module used the address of
- * message_sink_cb() directly, and \c res_stasis_test.so wasn't loaded yet, then
- * that module would fail to load.
+ * it has to do with how we previously loaded modules, using \c RTLD_LAZY.
  *
- * The stasis_message_sink_cb() function gives us a layer of indirection so that
- * the initial lazy binding will still work as expected.
+ * The stasis_message_sink_cb() function gave us a layer of indirection so that
+ * the initial lazy binding would still work as expected.
  */
 static void message_sink_cb(void *data, struct stasis_subscription *sub,
 	struct stasis_message *message)
@@ -284,4 +270,4 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.load = load_module,
 	.unload = unload_module,
 	.load_pri = AST_MODPRI_APP_DEPEND,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_statsd.c 1:16.1.1~dfsg-1/res/res_statsd.c
--- 1:13.23.1~dfsg-2/res/res_statsd.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_statsd.c	2018-12-26 18:45:13.000000000 +0000
@@ -52,8 +52,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/config_options.h"
 #include "asterisk/module.h"
 #include "asterisk/netsock2.h"
@@ -318,6 +316,14 @@ static void statsd_shutdown(void)
 	}
 }
 
+static int unload_module(void)
+{
+	statsd_shutdown();
+	aco_info_destroy(&cfg_info);
+	ao2_global_obj_release(confs);
+	return 0;
+}
+
 static int load_module(void)
 {
 	if (aco_info_init(&cfg_info)) {
@@ -341,55 +347,68 @@ static int load_module(void)
 		"", OPT_CHAR_ARRAY_T, 0,
 		CHARFLDSET(struct conf_global_options, prefix));
 
-	if (aco_process_config(&cfg_info, 0)) {
-		aco_info_destroy(&cfg_info);
-		return AST_MODULE_LOAD_DECLINE;
+	if (aco_process_config(&cfg_info, 0) == ACO_PROCESS_ERROR) {
+		struct conf *cfg;
+
+		ast_log(LOG_NOTICE, "Could not load statsd config; using defaults\n");
+		cfg = conf_alloc();
+		if (!cfg) {
+			aco_info_destroy(&cfg_info);
+			return AST_MODULE_LOAD_DECLINE;
+		}
+
+		if (aco_set_defaults(&global_option, "general", cfg->global)) {
+			ast_log(LOG_ERROR, "Failed to initialize statsd defaults.\n");
+			ao2_ref(cfg, -1);
+			aco_info_destroy(&cfg_info);
+			return AST_MODULE_LOAD_DECLINE;
+		}
+
+		ao2_global_obj_replace_unref(confs, cfg);
+		ao2_ref(cfg, -1);
 	}
 
 	if (!is_enabled()) {
 		return AST_MODULE_LOAD_SUCCESS;
 	}
 
-	if (statsd_init() != 0) {
-		aco_info_destroy(&cfg_info);
+	if (statsd_init()) {
+		unload_module();
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
-	/* For Optional API. */
-	ast_module_shutdown_ref(ast_module_info->self);
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-static int unload_module(void)
-{
-	statsd_shutdown();
-	aco_info_destroy(&cfg_info);
-	ao2_global_obj_release(confs);
-	return 0;
-}
-
 static int reload_module(void)
 {
-	if (aco_process_config(&cfg_info, 1)) {
+	switch (aco_process_config(&cfg_info, 1)) {
+	case ACO_PROCESS_OK:
+		break;
+	case ACO_PROCESS_UNCHANGED:
+		return AST_MODULE_LOAD_SUCCESS;
+	case ACO_PROCESS_ERROR:
+	default:
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
 	if (is_enabled()) {
-		return statsd_init();
+		if (statsd_init()) {
+			return AST_MODULE_LOAD_DECLINE;
+		}
 	} else {
 		statsd_shutdown();
-		return AST_MODULE_LOAD_SUCCESS;
 	}
+	return AST_MODULE_LOAD_SUCCESS;
 }
 
-/* The priority of this module is set to be as low as possible, since it could
- * be used by any other sort of module.
+/* The priority of this module is set just after realtime, since it loads
+ * configuration and could be used by any other sort of module.
  */
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Statsd client support",
 	.support_level = AST_MODULE_SUPPORT_EXTENDED,
 	.load = load_module,
 	.unload = unload_module,
 	.reload = reload_module,
-	.load_pri = 0,
-	);
+	.load_pri = AST_MODPRI_REALTIME_DRIVER + 5,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_stun_monitor.c 1:16.1.1~dfsg-1/res/res_stun_monitor.c
--- 1:13.23.1~dfsg-2/res/res_stun_monitor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_stun_monitor.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/sched.h"
 #include "asterisk/config.h"
@@ -480,9 +478,9 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "STUN Network Monitor",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_timing_dahdi.c 1:16.1.1~dfsg-1/res/res_timing_dahdi.c
--- 1:13.23.1~dfsg-2/res/res_timing_dahdi.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_timing_dahdi.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -235,8 +233,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "DAHDI Timing Interface",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_TIMING,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_TIMING,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_timing_kqueue.c 1:16.1.1~dfsg-1/res/res_timing_kqueue.c
--- 1:13.23.1~dfsg-2/res/res_timing_kqueue.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_timing_kqueue.c	2018-12-26 18:45:13.000000000 +0000
@@ -501,8 +501,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "KQueue Timing Interface",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_timing_pthread.c 1:16.1.1~dfsg-1/res/res_timing_pthread.c
--- 1:13.23.1~dfsg-2/res/res_timing_pthread.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_timing_pthread.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include <stdbool.h>
 #include <math.h>
 #include <unistd.h>
@@ -442,8 +440,9 @@ static int init_timing_thread(void)
 
 static int load_module(void)
 {
-	if (!(pthread_timers = ao2_container_alloc(PTHREAD_TIMER_BUCKETS,
-		pthread_timer_hash, pthread_timer_cmp))) {
+	pthread_timers = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		PTHREAD_TIMER_BUCKETS, pthread_timer_hash, NULL, pthread_timer_cmp);
+	if (!pthread_timers) {
 		return AST_MODULE_LOAD_DECLINE;
 	}
 
@@ -475,8 +474,8 @@ static int unload_module(void)
 	return res;
 }
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "pthread Timing Interface",
-		.support_level = AST_MODULE_SUPPORT_EXTENDED,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_TIMING,
-		);
+	.support_level = AST_MODULE_SUPPORT_EXTENDED,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_TIMING,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res_timing_timerfd.c 1:16.1.1~dfsg-1/res/res_timing_timerfd.c
--- 1:13.23.1~dfsg-2/res/res_timing_timerfd.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_timing_timerfd.c	2018-12-26 18:45:13.000000000 +0000
@@ -273,8 +273,8 @@ static int unload_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_LOAD_ORDER, "Timerfd Timing Interface",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.load_pri = AST_MODPRI_TIMING,
-		);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.load_pri = AST_MODPRI_TIMING,
+);
diff -pruN 1:13.23.1~dfsg-2/res/res.xml 1:16.1.1~dfsg-1/res/res.xml
--- 1:13.23.1~dfsg-2/res/res.xml	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res.xml	2018-12-26 18:45:13.000000000 +0000
@@ -1,5 +1,6 @@
 <member name="res_digium_phone" displayname="Download the Digium Phone Module for Asterisk.  See http://downloads.digium.com/pub/telephony/res_digium_phone/README.">
 	<support_level>external</support_level>
+	<conflict>no_binary_modules</conflict>
 	<depend>xmlstarlet</depend>
 	<depend>bash</depend>
 	<defaultenabled>no</defaultenabled>
diff -pruN 1:13.23.1~dfsg-2/res/res_xmpp.c 1:16.1.1~dfsg-1/res/res_xmpp.c
--- 1:13.23.1~dfsg-2/res/res_xmpp.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/res_xmpp.c	2018-12-26 18:45:13.000000000 +0000
@@ -48,8 +48,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <ctype.h>
 #include <iksemel.h>
 
@@ -643,7 +641,9 @@ static struct ast_xmpp_client *xmpp_clie
 		return NULL;
 	}
 
-	if (!(client->buddies = ao2_container_alloc(BUDDY_BUCKETS, xmpp_buddy_hash, xmpp_buddy_cmp))) {
+	client->buddies = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, BUDDY_BUCKETS,
+		xmpp_buddy_hash, NULL, xmpp_buddy_cmp);
+	if (!client->buddies) {
 		ast_log(LOG_ERROR, "Could not initialize buddy container for '%s'\n", name);
 		ao2_ref(client, -1);
 		return NULL;
@@ -709,7 +709,9 @@ static void *ast_xmpp_client_config_allo
 		return NULL;
 	}
 
-	if (!(cfg->buddies = ao2_container_alloc(BUDDY_BUCKETS, xmpp_buddy_hash, xmpp_buddy_cmp))) {
+	cfg->buddies = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, BUDDY_BUCKETS,
+		xmpp_buddy_hash, NULL, xmpp_buddy_cmp);
+	if (!cfg->buddies) {
 		ao2_ref(cfg, -1);
 		return NULL;
 	}
@@ -727,14 +729,6 @@ static void xmpp_config_destructor(void
 	ao2_cleanup(cfg->clients);
 }
 
-/*! \brief Hashing function for configuration */
-static int xmpp_config_hash(const void *obj, const int flags)
-{
-	const struct ast_xmpp_client_config *cfg = obj;
-	const char *name = (flags & OBJ_KEY) ? obj : cfg->name;
-	return ast_str_case_hash(name);
-}
-
 /*! \brief Comparator function for configuration */
 static int xmpp_config_cmp(void *obj, void *arg, int flags)
 {
@@ -756,7 +750,9 @@ static void *xmpp_config_alloc(void)
 		goto error;
 	}
 
-	if (!(cfg->clients = ao2_container_alloc(1, xmpp_config_hash, xmpp_config_cmp))) {
+	cfg->clients = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		NULL, xmpp_config_cmp);
+	if (!cfg->clients) {
 		goto error;
 	}
 
@@ -1628,11 +1624,15 @@ static void xmpp_init_event_distribution
 	if (!(client->mwi_sub = stasis_subscribe_pool(ast_mwi_topic_all(), xmpp_pubsub_mwi_cb, client))) {
 		return;
 	}
+	stasis_subscription_accept_message_type(client->mwi_sub, ast_mwi_state_type());
+	stasis_subscription_set_filter(client->mwi_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	if (!(client->device_state_sub = stasis_subscribe(ast_device_state_topic_all(), xmpp_pubsub_devstate_cb, client))) {
 		client->mwi_sub = stasis_unsubscribe(client->mwi_sub);
 		return;
 	}
+	stasis_subscription_accept_message_type(client->device_state_sub, ast_device_state_message_type());
+	stasis_subscription_set_filter(client->device_state_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
 
 	cached = stasis_cache_dump(ast_device_state_cache(), NULL);
 	ao2_callback(cached, OBJ_NODATA, cached_devstate_cb, client);
@@ -2252,7 +2252,9 @@ static struct ast_xmpp_buddy *xmpp_clien
 		return NULL;
 	}
 
-	if (!(buddy->resources = ao2_container_alloc(RESOURCE_BUCKETS, xmpp_resource_hash, xmpp_resource_cmp))) {
+	buddy->resources = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		RESOURCE_BUCKETS, xmpp_resource_hash, NULL, xmpp_resource_cmp);
+	if (!buddy->resources) {
 		ao2_ref(buddy, -1);
 		return NULL;
 	}
@@ -4779,9 +4781,9 @@ static int reload(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_GLOBAL_SYMBOLS | AST_MODFLAG_LOAD_ORDER, "Asterisk XMPP Interface",
-		.support_level = AST_MODULE_SUPPORT_CORE,
-		.load = load_module,
-		.unload = unload_module,
-		.reload = reload,
-		.load_pri = AST_MODPRI_CHANNEL_DEPEND,
-	       );
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.reload = reload,
+	.load_pri = AST_MODPRI_CHANNEL_DEPEND,
+);
diff -pruN 1:13.23.1~dfsg-2/res/snmp/agent.c 1:16.1.1~dfsg-1/res/snmp/agent.c
--- 1:13.23.1~dfsg-2/res/snmp/agent.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/snmp/agent.c	2018-12-26 18:45:13.000000000 +0000
@@ -18,10 +18,10 @@
 	<support_level>extended</support_level>
  ***/
 
+/* Needed for net-snmp headers */
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 /*
  * There is some collision collision between netsmp and asterisk names,
  * causing build under AST_DEVMODE to fail.
diff -pruN 1:13.23.1~dfsg-2/res/srtp/srtp_compat.h 1:16.1.1~dfsg-1/res/srtp/srtp_compat.h
--- 1:13.23.1~dfsg-2/res/srtp/srtp_compat.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/srtp/srtp_compat.h	2018-12-26 18:45:13.000000000 +0000
@@ -5,8 +5,28 @@
 
 #define crypto_policy_t srtp_crypto_policy_t
 
-#define AES_128_ICM 1
-#define HMAC_SHA1 3
+#define crypto_policy_set_aes_cm_128_hmac_sha1_80 srtp_crypto_policy_set_aes_cm_128_hmac_sha1_80
+#define crypto_policy_set_aes_cm_128_hmac_sha1_32 srtp_crypto_policy_set_aes_cm_128_hmac_sha1_32
+#define crypto_policy_set_aes_cm_192_hmac_sha1_80 srtp_crypto_policy_set_aes_cm_192_hmac_sha1_80
+#define crypto_policy_set_aes_cm_192_hmac_sha1_32 srtp_crypto_policy_set_aes_cm_192_hmac_sha1_32
+#define crypto_policy_set_aes_cm_256_hmac_sha1_80 srtp_crypto_policy_set_aes_cm_256_hmac_sha1_80
+#define crypto_policy_set_aes_cm_256_hmac_sha1_32 srtp_crypto_policy_set_aes_cm_256_hmac_sha1_32
+#define crypto_policy_set_aes_gcm_128_16_auth srtp_crypto_policy_set_aes_gcm_128_16_auth
+#define crypto_policy_set_aes_gcm_256_16_auth srtp_crypto_policy_set_aes_gcm_256_16_auth
+#define crypto_policy_set_aes_gcm_128_8_auth srtp_crypto_policy_set_aes_gcm_128_8_auth
+#define crypto_policy_set_aes_gcm_256_8_auth srtp_crypto_policy_set_aes_gcm_256_8_auth
+
+#if defined(SRTP_AES_GCM_128_KEY_LEN_WSALT)
+#define AES_128_GCM_KEYSIZE_WSALT SRTP_AES_GCM_128_KEY_LEN_WSALT
+#else
+#define AES_128_GCM_KEYSIZE_WSALT SRTP_AES_128_GCM_KEYSIZE_WSALT
+#endif
+
+#if defined(SRTP_AES_GCM_256_KEY_LEN_WSALT)
+#define AES_256_GCM_KEYSIZE_WSALT SRTP_AES_GCM_256_KEY_LEN_WSALT
+#else
+#define AES_256_GCM_KEYSIZE_WSALT SRTP_AES_256_GCM_KEYSIZE_WSALT
+#endif
 
 #define err_status_t srtp_err_status_t
 #define err_status_ok srtp_err_status_ok
diff -pruN 1:13.23.1~dfsg-2/res/stasis/app.c 1:16.1.1~dfsg-1/res/stasis/app.c
--- 1:13.23.1~dfsg-2/res/stasis/app.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis/app.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "app.h"
 #include "control.h"
 #include "messaging.h"
@@ -210,7 +208,6 @@ static void endpoint_state_cb(void *data
 static struct app_forwards *forwards_create_endpoint(struct stasis_app *app,
 	struct ast_endpoint *endpoint)
 {
-
 	struct app_forwards *forwards;
 	int ret = 0;
 
@@ -1603,13 +1600,6 @@ void stasis_app_register_event_sources(v
 	stasis_app_register_event_source(&endpoint_event_source);
 }
 
-int stasis_app_is_core_event_source(struct stasis_app_event_source *obj)
-{
-	return obj == &endpoint_event_source ||
-		obj == &bridge_event_source ||
-		obj == &channel_event_source;
-}
-
 void stasis_app_unregister_event_sources(void)
 {
 	stasis_app_unregister_event_source(&endpoint_event_source);
diff -pruN 1:13.23.1~dfsg-2/res/stasis/command.c 1:16.1.1~dfsg-1/res/stasis/command.c
--- 1:13.23.1~dfsg-2/res/stasis/command.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis/command.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "command.h"
 
 #include "asterisk/lock.h"
diff -pruN 1:13.23.1~dfsg-2/res/stasis/control.c 1:16.1.1~dfsg-1/res/stasis/control.c
--- 1:13.23.1~dfsg-2/res/stasis/control.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis/control.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,9 +25,8 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/stasis_channels.h"
+#include "asterisk/stasis_app.h"
 
 #include "command.h"
 #include "control.h"
@@ -36,6 +35,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 #include "asterisk/bridge.h"
 #include "asterisk/bridge_after.h"
 #include "asterisk/bridge_basic.h"
+#include "asterisk/bridge_features.h"
 #include "asterisk/frame.h"
 #include "asterisk/pbx.h"
 #include "asterisk/musiconhold.h"
@@ -43,6 +43,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 AST_LIST_HEAD(app_control_rules, stasis_app_control_rule);
 
+/*!
+ * \brief Indicates if the Stasis app internals are being shut down
+ */
+static int shutting_down;
+
 struct stasis_app_control {
 	ast_cond_t wait_cond;
 	/*! Queue of commands to dispatch on the channel */
@@ -58,6 +63,10 @@ struct stasis_app_control {
 	 */
 	struct ast_bridge *bridge;
 	/*!
+	 * Bridge features which should be applied to the channel when it enters the next bridge.  These only apply to the next bridge and will be emptied thereafter.
+	 */
+	struct ast_bridge_features *bridge_features;
+	/*!
 	 * Holding place for channel's PBX while imparted to a bridge.
 	 */
 	struct ast_pbx *pbx;
@@ -95,6 +104,8 @@ static void control_dtor(void *obj)
 	ast_cond_destroy(&control->wait_cond);
 	AST_LIST_HEAD_DESTROY(&control->add_rules);
 	AST_LIST_HEAD_DESTROY(&control->remove_rules);
+	ast_bridge_features_destroy(control->bridge_features);
+
 }
 
 struct stasis_app_control *control_create(struct ast_channel *channel, struct stasis_app *app)
@@ -134,7 +145,7 @@ struct stasis_app_control *control_creat
 }
 
 static void app_control_register_rule(
-	const struct stasis_app_control *control,
+	struct stasis_app_control *control,
 	struct app_control_rules *list, struct stasis_app_control_rule *obj)
 {
 	ao2_lock(control->command_queue);
@@ -143,7 +154,7 @@ static void app_control_register_rule(
 }
 
 static void app_control_unregister_rule(
-	const struct stasis_app_control *control,
+	struct stasis_app_control *control,
 	struct app_control_rules *list, struct stasis_app_control_rule *obj)
 {
 	struct stasis_app_control_rule *rule;
@@ -279,89 +290,6 @@ static struct stasis_app_command *exec_c
 	return exec_command_on_condition(control, command_fn, data, data_destructor, NULL);
 }
 
-struct stasis_app_control_dial_data {
-	char endpoint[AST_CHANNEL_NAME];
-	int timeout;
-};
-
-static int app_control_dial(struct stasis_app_control *control,
-	struct ast_channel *chan, void *data)
-{
-	RAII_VAR(struct ast_dial *, dial, ast_dial_create(), ast_dial_destroy);
-	struct stasis_app_control_dial_data *dial_data = data;
-	enum ast_dial_result res;
-	char *tech, *resource;
-	struct ast_channel *new_chan;
-	RAII_VAR(struct ast_bridge *, bridge, NULL, ao2_cleanup);
-
-	tech = dial_data->endpoint;
-	if (!(resource = strchr(tech, '/'))) {
-		return -1;
-	}
-	*resource++ = '\0';
-
-	if (!dial) {
-		ast_log(LOG_ERROR, "Failed to create dialing structure.\n");
-		return -1;
-	}
-
-	if (ast_dial_append(dial, tech, resource, NULL) < 0) {
-		ast_log(LOG_ERROR, "Failed to add %s/%s to dialing structure.\n", tech, resource);
-		return -1;
-	}
-
-	ast_dial_set_global_timeout(dial, dial_data->timeout);
-
-	res = ast_dial_run(dial, NULL, 0);
-	if (res != AST_DIAL_RESULT_ANSWERED || !(new_chan = ast_dial_answered_steal(dial))) {
-		return -1;
-	}
-
-	if (!(bridge = ast_bridge_basic_new())) {
-		ast_log(LOG_ERROR, "Failed to create basic bridge.\n");
-		return -1;
-	}
-
-	if (ast_bridge_impart(bridge, new_chan, NULL, NULL,
-		AST_BRIDGE_IMPART_CHAN_INDEPENDENT)) {
-		ast_hangup(new_chan);
-	} else {
-		control_swap_channel_in_bridge(control, bridge, chan, NULL);
-	}
-
-	return 0;
-}
-
-int stasis_app_control_dial(struct stasis_app_control *control, const char *endpoint, const char *exten, const char *context,
-			    int timeout)
-{
-	struct stasis_app_control_dial_data *dial_data;
-
-	if (!(dial_data = ast_calloc(1, sizeof(*dial_data)))) {
-		return -1;
-	}
-
-	if (!ast_strlen_zero(endpoint)) {
-		ast_copy_string(dial_data->endpoint, endpoint, sizeof(dial_data->endpoint));
-	} else if (!ast_strlen_zero(exten) && !ast_strlen_zero(context)) {
-		snprintf(dial_data->endpoint, sizeof(dial_data->endpoint), "Local/%s@%s", exten, context);
-	} else {
-		return -1;
-	}
-
-	if (timeout > 0) {
-		dial_data->timeout = timeout * 1000;
-	} else if (timeout == -1) {
-		dial_data->timeout = -1;
-	} else {
-		dial_data->timeout = 30000;
-	}
-
-	stasis_app_send_command_async(control, app_control_dial, dial_data, ast_free_ptr);
-
-	return 0;
-}
-
 static int app_control_add_role(struct stasis_app_control *control,
 		struct ast_channel *chan, void *data)
 {
@@ -943,6 +871,141 @@ struct ast_bridge *stasis_app_get_bridge
 	return ret;
 }
 
+/*!
+ * \brief Singleton dial bridge
+ *
+ * The dial bridge is a holding bridge used to hold all
+ * outbound dialed channels that are not in any "real" ARI-created
+ * bridge. The dial bridge is invisible, meaning that it does not
+ * show up in channel snapshots, AMI or ARI output, and no events
+ * get raised for it.
+ *
+ * This is used to keep dialed channels confined to the bridging system
+ * and unify the threading model used for dialing outbound channels.
+ */
+static struct ast_bridge *dial_bridge;
+AST_MUTEX_DEFINE_STATIC(dial_bridge_lock);
+
+/*!
+ * \brief Retrieve a reference to the dial bridge.
+ *
+ * If the dial bridge has not been created yet, it will
+ * be created, otherwise, a reference to the existing bridge
+ * will be returned.
+ *
+ * The caller will need to unreference the dial bridge once
+ * they are finished with it.
+ *
+ * \retval NULL Unable to find/create the dial bridge
+ * \retval non-NULL A reference to the dial bridge
+ */
+static struct ast_bridge *get_dial_bridge(void)
+{
+	struct ast_bridge *ret_bridge = NULL;
+
+	ast_mutex_lock(&dial_bridge_lock);
+
+	if (shutting_down) {
+		goto end;
+	}
+
+	if (dial_bridge) {
+		ret_bridge = ao2_bump(dial_bridge);
+		goto end;
+	}
+
+	dial_bridge = stasis_app_bridge_create_invisible("holding", "dial_bridge", NULL);
+	if (!dial_bridge) {
+		goto end;
+	}
+	ret_bridge = ao2_bump(dial_bridge);
+
+end:
+	ast_mutex_unlock(&dial_bridge_lock);
+	return ret_bridge;
+}
+
+static int bridge_channel_depart(struct stasis_app_control *control,
+	struct ast_channel *chan, void *data);
+
+/*!
+ * \brief after bridge callback for the dial bridge
+ *
+ * The only purpose of this callback is to ensure that the control structure's
+ * bridge pointer is NULLed
+ */
+static void dial_bridge_after_cb(struct ast_channel *chan, void *data)
+{
+	struct stasis_app_control *control = data;
+	struct ast_bridge_channel *bridge_channel;
+
+	ast_channel_lock(chan);
+	bridge_channel = ast_channel_get_bridge_channel(chan);
+	ast_channel_unlock(chan);
+
+	ast_debug(3, "Channel: <%s>  Reason: %d\n", ast_channel_name(control->channel), ast_channel_hangupcause(chan));
+
+	stasis_app_send_command_async(control, bridge_channel_depart, bridge_channel, __ao2_cleanup);
+
+	control->bridge = NULL;
+}
+
+static void dial_bridge_after_cb_failed(enum ast_bridge_after_cb_reason reason, void *data)
+{
+	struct stasis_app_control *control = data;
+
+	ast_debug(3, "Channel: <%s>  Reason: %d\n", ast_channel_name(control->channel), reason);
+	dial_bridge_after_cb(control->channel, data);
+}
+
+/*!
+ * \brief Add a channel to the singleton dial bridge.
+ *
+ * \param control The Stasis control structure
+ * \param chan The channel to add to the bridge
+ * \retval -1 Failed
+ * \retval 0 Success
+ */
+static int add_to_dial_bridge(struct stasis_app_control *control, struct ast_channel *chan)
+{
+	struct ast_bridge *bridge;
+
+	bridge = get_dial_bridge();
+	if (!bridge) {
+		return -1;
+	}
+
+	control->bridge = bridge;
+	ast_bridge_set_after_callback(chan, dial_bridge_after_cb, dial_bridge_after_cb_failed, control);
+	if (ast_bridge_impart(bridge, chan, NULL, NULL, AST_BRIDGE_IMPART_CHAN_DEPARTABLE)) {
+		control->bridge = NULL;
+		ao2_ref(bridge, -1);
+		return -1;
+	}
+
+	ao2_ref(bridge, -1);
+
+	return 0;
+}
+
+/*!
+ * \brief Depart a channel from a bridge, and potentially add it back to the dial bridge
+ *
+ * \param control Take a guess
+ * \param chan Take another guess
+ */
+static int depart_channel(struct stasis_app_control *control, struct ast_channel *chan)
+{
+	ast_bridge_depart(chan);
+
+	if (!ast_check_hangup(chan) && ast_channel_state(chan) != AST_STATE_UP) {
+		/* Channel is still being dialed, so put it back in the dialing bridge */
+		add_to_dial_bridge(control, chan);
+	}
+
+	return 0;
+}
+
 static int bridge_channel_depart(struct stasis_app_control *control,
 	struct ast_channel *chan, void *data)
 {
@@ -961,7 +1024,7 @@ static int bridge_channel_depart(struct
 	ast_debug(3, "%s: Channel departing bridge\n",
 		ast_channel_uniqueid(chan));
 
-	ast_bridge_depart(chan);
+	depart_channel(control, chan);
 
 	return 0;
 }
@@ -1039,9 +1102,113 @@ static void bridge_after_cb_failed(enum
 		ast_bridge_after_cb_reason_string(reason));
 }
 
+/*!
+ * \brief Dial timeout datastore
+ *
+ * A datastore is used because a channel may change
+ * bridges during the course of a dial attempt. This
+ * may be because the channel changes from the dial bridge
+ * to a standard bridge, or it may move between standard
+ * bridges. In order to keep the dial timeout, we need
+ * to keep the timeout information local to the channel.
+ * That is what this datastore is for
+ */
+struct ast_datastore_info timeout_datastore = {
+	.type = "ARI dial timeout",
+};
+
+static int hangup_channel(struct stasis_app_control *control,
+	struct ast_channel *chan, void *data)
+{
+	ast_softhangup(chan, AST_SOFTHANGUP_EXPLICIT);
+	return 0;
+}
+
+/*!
+ * \brief Dial timeout
+ *
+ * This is a bridge interval hook callback. The interval hook triggering
+ * means that the dial timeout has been reached. If the channel has not
+ * been answered by the time this callback is called, then the channel
+ * is hung up
+ *
+ * \param bridge_channel Bridge channel on which interval hook has been called
+ * \param ignore Ignored
+ * \return -1 (i.e. remove the interval hook)
+ */
+static int bridge_timeout(struct ast_bridge_channel *bridge_channel, void *ignore)
+{
+	struct ast_datastore *datastore;
+	RAII_VAR(struct stasis_app_control *, control, NULL, ao2_cleanup);
+
+	control = stasis_app_control_find_by_channel(bridge_channel->chan);
+
+	ast_channel_lock(bridge_channel->chan);
+	if (ast_channel_state(bridge_channel->chan) != AST_STATE_UP) {
+		/* Don't bother removing the datastore because it will happen when the channel is hung up */
+		ast_channel_unlock(bridge_channel->chan);
+		stasis_app_send_command_async(control, hangup_channel, NULL, NULL);
+		return -1;
+	}
+
+	datastore = ast_channel_datastore_find(bridge_channel->chan, &timeout_datastore, NULL);
+	if (!datastore) {
+		ast_channel_unlock(bridge_channel->chan);
+		return -1;
+	}
+	ast_channel_datastore_remove(bridge_channel->chan, datastore);
+	ast_channel_unlock(bridge_channel->chan);
+	ast_datastore_free(datastore);
+
+	return -1;
+}
+
+/*!
+ * \brief Set a dial timeout interval hook on the channel.
+ *
+ * The absolute time that the timeout should occur is stored on
+ * a datastore on the channel. This time is converted into a relative
+ * number of milliseconds in the future. Then an interval hook is set
+ * to trigger in that number of milliseconds.
+ *
+ * \pre chan is locked
+ *
+ * \param chan The channel on which to set the interval hook
+ */
+static void set_interval_hook(struct ast_channel *chan)
+{
+	struct ast_datastore *datastore;
+	struct timeval *hangup_time;
+	int64_t ms;
+	struct ast_bridge_channel *bridge_channel;
+
+	datastore = ast_channel_datastore_find(chan, &timeout_datastore, NULL);
+	if (!datastore) {
+		return;
+	}
+
+	hangup_time = datastore->data;
+
+	ms = ast_tvdiff_ms(*hangup_time, ast_tvnow());
+	bridge_channel = ast_channel_get_bridge_channel(chan);
+	if (!bridge_channel) {
+		return;
+	}
+
+	if (ast_bridge_interval_hook(bridge_channel->features, 0, ms > 0 ? ms : 1,
+			bridge_timeout, NULL, NULL, 0)) {
+		ao2_ref(bridge_channel, -1);
+		return;
+	}
+
+	ast_queue_frame(bridge_channel->chan, &ast_null_frame);
+	ao2_ref(bridge_channel, -1);
+}
+
 int control_swap_channel_in_bridge(struct stasis_app_control *control, struct ast_bridge *bridge, struct ast_channel *chan, struct ast_channel *swap)
 {
 	int res;
+	struct ast_bridge_features *features;
 
 	if (!control || !bridge) {
 		return -1;
@@ -1086,6 +1253,10 @@ int control_swap_channel_in_bridge(struc
 		ast_channel_pbx_set(chan, NULL);
 	}
 
+	/* Pull bridge features from the control */
+	features = control->bridge_features;
+	control->bridge_features = NULL;
+
 	ast_assert(stasis_app_get_bridge(control) == NULL);
 	/* We need to set control->bridge here since bridge_after_cb may be run
 	 * before ast_bridge_impart returns.  bridge_after_cb gets a reason
@@ -1101,7 +1272,7 @@ int control_swap_channel_in_bridge(struc
 	res = ast_bridge_impart(bridge,
 		chan,
 		swap,
-		NULL, /* features */
+		features, /* features */
 		AST_BRIDGE_IMPART_CHAN_DEPARTABLE);
 	if (res != 0) {
 		/* ast_bridge_impart failed before it could spawn the depart
@@ -1115,6 +1286,10 @@ int control_swap_channel_in_bridge(struc
 		control->pbx = NULL;
 		control->bridge = NULL;
 		ao2_unlock(control);
+	} else {
+		ast_channel_lock(chan);
+		set_interval_hook(chan);
+		ast_channel_unlock(chan);
 	}
 
 	return res;
@@ -1158,7 +1333,7 @@ static int app_control_remove_channel_fr
 		return -1;
 	}
 
-	ast_bridge_depart(chan);
+	depart_channel(control, chan);
 	return 0;
 }
 
@@ -1193,6 +1368,31 @@ int stasis_app_control_queue_control(str
 	return ast_queue_control(control->channel, frame_type);
 }
 
+int stasis_app_control_bridge_features_init(
+	struct stasis_app_control *control)
+{
+	struct ast_bridge_features *features;
+
+	features = ast_bridge_features_new();
+	if (!features) {
+		return 1;
+	}
+	control->bridge_features = features;
+	return 0;
+}
+
+void stasis_app_control_absorb_dtmf_in_bridge(
+	struct stasis_app_control *control, int absorb)
+{
+	control->bridge_features->dtmf_passthrough = !absorb;
+}
+
+void stasis_app_control_mute_in_bridge(
+	struct stasis_app_control *control, int mute)
+{
+	control->bridge_features->mute = mute;
+}
+
 void control_flush_queue(struct stasis_app_control *control)
 {
 	struct ao2_iterator iter;
@@ -1278,3 +1478,115 @@ struct stasis_app *control_app(struct st
 {
 	return control->app;
 }
+
+struct control_dial_args {
+	unsigned int timeout;
+	char dialstring[0];
+};
+
+static struct control_dial_args *control_dial_args_alloc(const char *dialstring,
+	unsigned int timeout)
+{
+	struct control_dial_args *args;
+
+	args = ast_malloc(sizeof(*args) + strlen(dialstring) + 1);
+	if (!args) {
+		return NULL;
+	}
+
+	args->timeout = timeout;
+	/* Safe */
+	strcpy(args->dialstring, dialstring);
+
+	return args;
+}
+
+static void control_dial_args_destroy(void *data)
+{
+	struct control_dial_args *args = data;
+
+	ast_free(args);
+}
+
+/*!
+ * \brief Set dial timeout on a channel to be dialed.
+ *
+ * \param chan The channel on which to set the dial timeout
+ * \param timeout The timeout in seconds
+ */
+static int set_timeout(struct ast_channel *chan, unsigned int timeout)
+{
+	struct ast_datastore *datastore;
+	struct timeval *hangup_time;
+
+	hangup_time = ast_malloc(sizeof(struct timeval));
+
+	datastore = ast_datastore_alloc(&timeout_datastore, NULL);
+	if (!datastore) {
+		return -1;
+	}
+	*hangup_time = ast_tvadd(ast_tvnow(), ast_samp2tv(timeout, 1));
+	datastore->data = hangup_time;
+
+	ast_channel_lock(chan);
+	ast_channel_datastore_add(chan, datastore);
+
+	if (ast_channel_is_bridged(chan)) {
+		set_interval_hook(chan);
+	}
+	ast_channel_unlock(chan);
+
+	return 0;
+}
+
+static int app_control_dial(struct stasis_app_control *control,
+	struct ast_channel *chan, void *data)
+{
+	struct control_dial_args *args = data;
+	int bridged;
+
+	ast_channel_lock(chan);
+	bridged = ast_channel_is_bridged(chan);
+	ast_channel_unlock(chan);
+
+	if (!bridged && add_to_dial_bridge(control, chan)) {
+		return -1;
+	}
+
+	if (args->timeout && set_timeout(chan, args->timeout)) {
+		return -1;
+	}
+
+	if (ast_call(chan, args->dialstring, 0)) {
+		return -1;
+	}
+
+	ast_channel_publish_dial(NULL, chan, args->dialstring, NULL);
+
+	return 0;
+}
+
+int stasis_app_control_dial(struct stasis_app_control *control,
+		const char *dialstring, unsigned int timeout)
+{
+	struct control_dial_args *args;
+
+	args = control_dial_args_alloc(dialstring, timeout);
+	if (!args) {
+		return -1;
+	}
+
+	return stasis_app_send_command_async(control, app_control_dial,
+		args, control_dial_args_destroy);
+}
+
+void stasis_app_control_shutdown(void)
+{
+	ast_mutex_lock(&dial_bridge_lock);
+	shutting_down = 1;
+	if (dial_bridge) {
+		ast_bridge_destroy(dial_bridge, 0);
+		dial_bridge = NULL;
+	}
+	ast_mutex_unlock(&dial_bridge_lock);
+}
diff -pruN 1:13.23.1~dfsg-2/res/stasis/messaging.c 1:16.1.1~dfsg-1/res/stasis/messaging.c
--- 1:13.23.1~dfsg-2/res/stasis/messaging.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis/messaging.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/message.h"
 #include "asterisk/endpoints.h"
 #include "asterisk/astobj2.h"
diff -pruN 1:13.23.1~dfsg-2/res/stasis/stasis_bridge.c 1:16.1.1~dfsg-1/res/stasis/stasis_bridge.c
--- 1:13.23.1~dfsg-2/res/stasis/stasis_bridge.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis/stasis_bridge.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/bridge.h"
 #include "asterisk/bridge_after.h"
 #include "asterisk/bridge_internal.h"
diff -pruN 1:13.23.1~dfsg-2/res/stasis_recording/stored.c 1:16.1.1~dfsg-1/res/stasis_recording/stored.c
--- 1:13.23.1~dfsg-2/res/stasis_recording/stored.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/res/stasis_recording/stored.c	2018-12-26 18:45:13.000000000 +0000
@@ -25,8 +25,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/paths.h"
 #include "asterisk/stasis_app_recording.h"
@@ -61,6 +59,24 @@ const char *stasis_app_stored_recording_
 	return recording->file;
 }
 
+const char *stasis_app_stored_recording_get_filename(
+	struct stasis_app_stored_recording *recording)
+{
+	if (!recording) {
+		return NULL;
+	}
+	return recording->file_with_ext;
+}
+
+const char *stasis_app_stored_recording_get_extension(
+	struct stasis_app_stored_recording *recording)
+{
+	if (!recording) {
+		return NULL;
+	}
+	return recording->format;
+}
+
 /*!
  * \brief Split a path into directory and file, resolving canonical directory.
  *
@@ -107,18 +123,9 @@ static int split_path(const char *path,
 		return -1;
 	}
 
-#if defined(__AST_DEBUG_MALLOC)
 	*dir = ast_strdup(real_dir); /* Dupe so we can ast_free() */
-#else
-	/*
-	 * ast_std_free() and ast_free() are the same thing at this time
-	 * so we don't need to dupe.
-	 */
-	*dir = real_dir;
-	real_dir = NULL;
-#endif	/* defined(__AST_DEBUG_MALLOC) */
 	*file = ast_strdup(file_portion);
-	return 0;
+	return (*dir && *file) ? 0 : -1;
 }
 
 struct match_recording_data {
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/applications.json 1:16.1.1~dfsg-1/rest-api/api-docs/applications.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/applications.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/applications.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,7 +2,7 @@
 	"_copyright": "Copyright (C) 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/applications.{format}",
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/asterisk.json 1:16.1.1~dfsg-1/rest-api/api-docs/asterisk.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/asterisk.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/asterisk.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,7 +2,7 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/asterisk.{format}",
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/bridges.json 1:16.1.1~dfsg-1/rest-api/api-docs/bridges.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/bridges.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/bridges.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,14 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/bridges.{format}",
+	"requiresModules": [
+		"res_stasis_recording",
+		"res_stasis_playback"
+	],
 	"apis": [
 		{
 			"path": "/bridges",
@@ -26,7 +30,7 @@
 					"parameters": [
 						{
 							"name": "type",
-							"description": "Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media).",
+							"description": "Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media, video_sfu).",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
@@ -65,7 +69,7 @@
 					"parameters": [
 						{
 							"name": "type",
-							"description": "Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media) to set.",
+							"description": "Comma separated list of bridge type attributes (mixing, holding, dtmf_events, proxy_media, video_sfu) to set.",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
@@ -169,6 +173,24 @@
 							"required": false,
 							"allowMultiple": false,
 							"dataType": "string"
+						},
+						{
+							"name": "absorbDTMF",
+							"description": "Absorb DTMF coming from this channel, preventing it to pass through to the bridge",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "boolean",
+							"defaultValue": false
+						},
+						{
+							"name": "mute",
+							"description": "Mute audio from this channel, preventing it to pass through to the bridge",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "boolean",
+							"defaultValue": false
 						}
 					],
 					"errorResponses": [
@@ -400,10 +422,10 @@
 						},
 						{
 							"name": "media",
-							"description": "Media's URI to play.",
+							"description": "Media URIs to play.",
 							"paramType": "query",
 							"required": true,
-							"allowMultiple": false,
+							"allowMultiple": true,
 							"dataType": "string"
 						},
 						{
@@ -416,7 +438,7 @@
 						},
 						{
 							"name": "offsetms",
-							"description": "Number of media to skip before playing.",
+							"description": "Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified.",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
@@ -492,10 +514,10 @@
 						},
 						{
 							"name": "media",
-							"description": "Media's URI to play.",
+							"description": "Media URIs to play.",
 							"paramType": "query",
 							"required": true,
-							"allowMultiple": false,
+							"allowMultiple": true,
 							"dataType": "string"
 						},
 						{
@@ -508,7 +530,7 @@
 						},
 						{
 							"name": "offsetms",
-							"description": "Number of media to skip before playing.",
+							"description": "Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified.",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/channels.json 1:16.1.1~dfsg-1/rest-api/api-docs/channels.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/channels.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/channels.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,16 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/channels.{format}",
+	"requiresModules": [
+		"res_stasis_answer",
+		"res_stasis_playback",
+		"res_stasis_recording",
+		"res_stasis_snoop"
+	],
 	"apis": [
 		{
 			"path": "/channels",
@@ -152,6 +158,82 @@
 			]
 		},
 		{
+			"path": "/channels/create",
+			"description": "Create a channel and place it in a Stasis app, but do not dial the channel yet.",
+			"operations": [
+				{
+					"httpMethod": "POST",
+					"summary": "Create channel.",
+					"nickname": "create",
+					"responseClass": "Channel",
+					"parameters": [
+						{
+							"name": "endpoint",
+							"description": "Endpoint for channel communication",
+							"paramType": "query",
+							"required": true,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "app",
+							"description": "Stasis Application to place channel into",
+							"paramType": "query",
+							"required": true,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "appArgs",
+							"description": "The application arguments to pass to the Stasis application provided by 'app'. Mutually exclusive with 'context', 'extension', 'priority', and 'label'.",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "channelId",
+							"description": "The unique id to assign the channel on creation.",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "otherChannelId",
+							"description": "The unique id to assign the second channel when using local channels.",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "originator",
+							"description": "Unique ID of the calling channel",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "formats",
+							"description": "The format name capability list to use if originator is not specified. Ex. \"ulaw,slin16\".  Format names can be found with \"core show codecs\".",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						}
+					],
+					"errorResponses": [
+						{
+							"code": 409,
+							"reason": "Channel with given unique ID already exists."
+						}
+					]
+				}
+			]
+		},
+		{
 			"path": "/channels/{channelId}",
 			"description": "Active channel",
 			"operations": [
@@ -308,6 +390,7 @@
 							"reason": "Channel with given unique ID already exists."
 						}
 					]
+
 				},
 				{
 					"httpMethod": "DELETE",
@@ -337,7 +420,8 @@
 									"normal",
 									"busy",
 									"congestion",
-									"no_answer"
+									"no_answer",
+									"answered_elsewhere"
 								]
 							}
 						}
@@ -414,6 +498,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -462,6 +550,10 @@
 						{
 							"code": 422,
 							"reason": "Endpoint is not the same type as the channel"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -494,6 +586,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -526,6 +622,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				},
@@ -552,6 +652,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -632,6 +736,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -681,6 +789,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				},
@@ -724,6 +836,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -756,6 +872,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				},
@@ -782,6 +902,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -823,6 +947,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				},
@@ -849,6 +977,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -882,6 +1014,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				},
@@ -908,6 +1044,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -934,10 +1074,10 @@
 						},
 						{
 							"name": "media",
-							"description": "Media's URI to play.",
+							"description": "Media URIs to play.",
 							"paramType": "query",
 							"required": true,
-							"allowMultiple": false,
+							"allowMultiple": true,
 							"dataType": "string"
 						},
 						{
@@ -950,7 +1090,7 @@
 						},
 						{
 							"name": "offsetms",
-							"description": "Number of media to skip before playing.",
+							"description": "Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified.",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
@@ -982,6 +1122,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -1016,10 +1160,10 @@
 						},
 						{
 							"name": "media",
-							"description": "Media's URI to play.",
+							"description": "Media URIs to play.",
 							"paramType": "query",
 							"required": true,
-							"allowMultiple": false,
+							"allowMultiple": true,
 							"dataType": "string"
 						},
 						{
@@ -1032,7 +1176,7 @@
 						},
 						{
 							"name": "offsetms",
-							"description": "Number of media to skip before playing.",
+							"description": "Number of milliseconds to skip before playing. Only applies to the first URI if multiple media URIs are specified.",
 							"paramType": "query",
 							"required": false,
 							"allowMultiple": false,
@@ -1056,6 +1200,10 @@
 						{
 							"code": 409,
 							"reason": "Channel not in a Stasis application"
+						},
+						{
+							"code": 412,
+							"reason": "Channel in invalid state"
 						}
 					]
 				}
@@ -1463,6 +1611,59 @@
 					]
 				}
 			]
+		},
+		{
+			"path": "/channels/{channelId}/dial",
+			"description": "Dial a channel",
+			"operations": [
+				{
+					"httpMethod": "POST",
+					"summary": "Dial a created channel.",
+					"nickname": "dial",
+					"responseClass": "void",
+					"parameters": [
+						{
+							"name": "channelId",
+							"description": "Channel's id",
+							"paramType": "path",
+							"required": true,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "caller",
+							"description": "Channel ID of caller",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "string"
+						},
+						{
+							"name": "timeout",
+							"description": "Dial timeout",
+							"paramType": "query",
+							"required": false,
+							"allowMultiple": false,
+							"dataType": "int",
+							"defaultValue": 0,
+							"allowableValues": {
+								"valueType": "RANGE",
+								"min": 0
+							}
+						}
+					],
+					"errorResponses": [
+						{
+							"code": 404,
+							"reason": "Channel cannot be found."
+						},
+						{
+							"code": 409,
+							"reason": "Channel cannot be dialed."
+						}
+					]
+				}
+			]
 		}
 	],
 	"models": {
@@ -1566,6 +1767,11 @@
 					"required": true,
 					"type": "string",
 					"description": "The default spoken language"
+				},
+				"channelvars": {
+					"required": false,
+					"type": "object",
+					"description": "Channel variables"
 				}
 			}
 		}
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/deviceStates.json 1:16.1.1~dfsg-1/rest-api/api-docs/deviceStates.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/deviceStates.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/deviceStates.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "Kevin Harwell <kharwell@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/deviceStates.{format}",
+	"requiresModules": [
+		"res_stasis_device_state"
+	],
 	"apis": [
 		{
 			"path": "/deviceStates",
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/endpoints.json 1:16.1.1~dfsg-1/rest-api/api-docs/endpoints.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/endpoints.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/endpoints.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,7 +2,7 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/endpoints.{format}",
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/events.json 1:16.1.1~dfsg-1/rest-api/api-docs/events.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/events.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/events.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.2",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/events.{format}",
+	"requiresModules": [
+		"res_http_websocket"
+	],
 	"apis": [
 		{
 			"path": "/events",
@@ -151,6 +154,7 @@
 			"subTypes": [
 				"DeviceStateChanged",
 				"PlaybackStarted",
+				"PlaybackContinuing",
 				"PlaybackFinished",
 				"RecordingStarted",
 				"RecordingFinished",
@@ -206,7 +210,7 @@
 							"Unreachable",
 							"Reachable",
 							"Unknown",
-							"Created",
+							"NonQualified",
 							"Removed"
 						]
 					}
@@ -271,6 +275,17 @@
 			"properties": {
 				"playback": {
 					"type": "Playback",
+					"description": "Playback control object",
+					"required": true
+				}
+			}
+		},
+		"PlaybackContinuing": {
+			"id": "PlaybackContinuing",
+			"description": "Event showing the continuation of a media playback operation from one media URI to the next in the list.",
+			"properties": {
+				"playback": {
+					"type": "Playback",
 					"description": "Playback control object",
 					"required": true
 				}
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/mailboxes.json 1:16.1.1~dfsg-1/rest-api/api-docs/mailboxes.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/mailboxes.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/mailboxes.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 	"_copyright": "Copyright (C) 2013, Digium, Inc.",
 	"_author": "Jonathan Rose <jrose@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/mailboxes.{format}",
+	"requiresModules": [
+		"res_stasis_mailbox"
+	],
 	"apis": [
 		{
 			"path": "/mailboxes",
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/playbacks.json 1:16.1.1~dfsg-1/rest-api/api-docs/playbacks.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/playbacks.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/playbacks.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/playbacks.{format}",
+	"requiresModules": [
+		"res_stasis_playback"
+	],
 	"apis": [
 		{
 			"path": "/playbacks/{playbackId}",
@@ -124,9 +127,14 @@
 				},
 				"media_uri": {
 					"type": "string",
-					"description": "URI for the media to play back.",
+					"description": "The URI for the media currently being played back.",
 					"required": true
 				},
+				"next_media_uri": {
+					"type": "string",
+					"description": "If a list of URIs is being played, the next media URI to be played back.",
+					"required": false
+				},
 				"target_uri": {
 					"type": "string",
 					"description": "URI for the channel or bridge to play the media on",
@@ -145,7 +153,8 @@
 						"values": [
 							"queued",
 							"playing",
-							"complete"
+							"continuing",
+							"done"
 						]
 					}
 				}
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/recordings.json 1:16.1.1~dfsg-1/rest-api/api-docs/recordings.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/recordings.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/recordings.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,10 +2,13 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/recordings.{format}",
+	"requiresModules": [
+		"res_stasis_recording"
+	],
 	"apis": [
 		{
 			"path": "/recordings/stored",
@@ -64,6 +67,38 @@
 						{
 							"code": 404,
 							"reason": "Recording not found"
+						}
+					]
+				}
+			]
+		},
+		{
+			"path": "/recordings/stored/{recordingName}/file",
+			"description": "The actual file associated with the stored recording",
+			"operations": [
+				{
+					"httpMethod": "GET",
+					"summary": "Get the file associated with the stored recording.",
+					"nickname": "getStoredFile",
+					"responseClass": "binary",
+					"parameters": [
+						{
+							"name": "recordingName",
+							"description": "The name of the recording",
+							"paramType": "path",
+							"required": true,
+							"allowMultiple": false,
+							"dataType": "string"
+						}
+					],
+					"errorResponses": [
+						{
+							"code": 403,
+							"reason": "The recording file could not be opened"
+						},
+						{
+							"code": 404,
+							"reason": "Recording not found"
 						}
 					]
 				}
diff -pruN 1:13.23.1~dfsg-2/rest-api/api-docs/sounds.json 1:16.1.1~dfsg-1/rest-api/api-docs/sounds.json
--- 1:13.23.1~dfsg-2/rest-api/api-docs/sounds.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/api-docs/sounds.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,7 +2,7 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "2.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"resourcePath": "/api-docs/sounds.{format}",
diff -pruN 1:13.23.1~dfsg-2/rest-api/resources.json 1:16.1.1~dfsg-1/rest-api/resources.json
--- 1:13.23.1~dfsg-2/rest-api/resources.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api/resources.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,7 +2,7 @@
 	"_copyright": "Copyright (C) 2012 - 2013, Digium, Inc.",
 	"_author": "David M. Lee, II <dlee@digium.com>",
 	"_svn_revision": "$Revision$",
-	"apiVersion": "1.10.0",
+	"apiVersion": "4.0.0",
 	"swaggerVersion": "1.1",
 	"basePath": "http://localhost:8088/ari",
 	"apis": [
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/ari.make.mustache 1:16.1.1~dfsg-1/rest-api-templates/ari.make.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/ari.make.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/ari.make.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -19,9 +19,5 @@
 #
 
 {{#apis}}
-res_ari_{{c_name}}.so: ari/resource_{{c_name}}.o
-.res_ari_{{c_name}}.moduleinfo: ari/resource_{{c_name}}.c
-
-ari/resource_{{c_name}}.o: _ASTCFLAGS+=$(call MOD_ASTCFLAGS,res_ari_{{c_name}})
-
+$(call MOD_ADD_C,res_ari_{{c_name}},ari/resource_{{c_name}}.c)
 {{/apis}}
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/ari_model_validators.c.mustache 1:16.1.1~dfsg-1/rest-api-templates/ari_model_validators.c.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/ari_model_validators.c.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/ari_model_validators.c.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -27,8 +27,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/logger.h"
 #include "asterisk/module.h"
 #include "ari_model_validators.h"
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/ari_model_validators.h.mustache 1:16.1.1~dfsg-1/rest-api-templates/ari_model_validators.h.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/ari_model_validators.h.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/ari_model_validators.h.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -23,11 +23,10 @@
  * the validator's function pointer.
  *
  * The reason for this seamingly useless indirection is the way function
- * pointers interfere with module loading. Asterisk attempts to dlopen() each
- * module using \c RTLD_LAZY in order to read some metadata from the module.
- * Unfortunately, if you take the address of a function, the function has to be
- * resolvable at load time, even if \c RTLD_LAZY is specified. By moving the
- * function-address-taking into this module, we can once again be lazy.
+ * pointers used to interfere with module loading. Previously, Asterisk
+ * attempted to dlopen() each module using \c RTLD_LAZY in order to read some
+ * metadata from the module. Using functions to get the function pointer
+ * allowed us to be lazy.
  */
 
  /*
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/ari_resource.c.mustache 1:16.1.1~dfsg-1/rest-api-templates/ari_resource.c.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/ari_resource.c.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/ari_resource.c.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "resource_{{c_name}}.h"
 
 {{#apis}}
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/ari_resource.h.mustache 1:16.1.1~dfsg-1/rest-api-templates/ari_resource.h.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/ari_resource.h.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/ari_resource.h.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -82,11 +82,19 @@ int ast_ari_{{c_name}}_{{c_nickname}}_pa
  * {{{notes}}}
 {{/notes}}
  *
+{{#is_binary_response}}
+ * \param ser TCP/TLS session instance
+{{/is_binary_response}}
  * \param headers HTTP headers
  * \param args Swagger parameters
  * \param[out] response HTTP response
  */
+{{^is_binary_response}}
 void ast_ari_{{c_name}}_{{c_nickname}}(struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args, struct ast_ari_response *response);
+{{/is_binary_response}}
+{{#is_binary_response}}
+void ast_ari_{{c_name}}_{{c_nickname}}(struct ast_tcptls_session_instance *ser, struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args, struct ast_ari_response *response);
+{{/is_binary_response}}
 {{/is_req}}
 {{#is_websocket}}
 
@@ -97,17 +105,42 @@ void ast_ari_{{c_name}}_{{c_nickname}}(s
  * {{{notes}}}
 {{/notes}}
  *
+ * \retval  0 success
+ * \retval -1 error
+ */
+int ast_ari_websocket_{{c_name}}_{{c_nickname}}_init(void);
+
+/*!
+ * \brief {{{summary}}}
+{{#notes}}
+ *
+ * {{{notes}}}
+{{/notes}}
+ *
+ * \return Nothing
+ */
+void ast_ari_websocket_{{c_name}}_{{c_nickname}}_dtor(void);
+
+/*!
+ * \brief {{summary}}
+{{#notes}}
+ *
+ * {{{notes}}}
+{{/notes}}
+ *
  * \param ser HTTP TCP/TLS Server Session
  * \param headers HTTP headers
  * \param args Swagger parameters
+ * \param session_id The id of the current session.
  *
  * \retval 0 success
  * \retval non-zero error
  */
-int ast_ari_websocket_{{c_name}}_{{c_nickname}}_attempted(struct ast_tcptls_session_instance *ser, struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args);
+int ast_ari_websocket_{{c_name}}_{{c_nickname}}_attempted(struct ast_tcptls_session_instance *ser,
+	struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args, const char *session_id);
 
 /*!
- * \brief {{{summary}}}
+ * \brief {{summary}}
 {{#notes}}
  *
  * {{{notes}}}
@@ -116,8 +149,10 @@ int ast_ari_websocket_{{c_name}}_{{c_nic
  * \param session ARI WebSocket.
  * \param headers HTTP headers.
  * \param args Swagger parameters.
+ * \param session_id The id of the current session.
  */
-void ast_ari_websocket_{{c_name}}_{{c_nickname}}_established(struct ast_ari_websocket_session *session, struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args);
+void ast_ari_websocket_{{c_name}}_{{c_nickname}}_established(struct ast_ari_websocket_session *session,
+	struct ast_variable *headers, struct ast_ari_{{c_name}}_{{c_nickname}}_args *args);
 {{/is_websocket}}
 {{/operations}}
 {{/apis}}
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/make_ari_stubs.py 1:16.1.1~dfsg-1/rest-api-templates/make_ari_stubs.py
--- 1:13.23.1~dfsg-2/rest-api-templates/make_ari_stubs.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/make_ari_stubs.py	2018-12-26 18:45:13.000000000 +0000
@@ -42,7 +42,7 @@ def rel(file):
     """
     return os.path.join(TOPDIR, file)
 
-WIKI_PREFIX = 'Asterisk 13'
+WIKI_PREFIX = 'Asterisk 16'
 
 API_TRANSFORMS = [
     Transform(rel('api.wiki.mustache'),
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/res_ari_resource.c.mustache 1:16.1.1~dfsg-1/rest-api-templates/res_ari_resource.c.mustache
--- 1:13.23.1~dfsg-2/rest-api-templates/res_ari_resource.c.mustache	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/res_ari_resource.c.mustache	2018-12-26 18:45:13.000000000 +0000
@@ -40,13 +40,14 @@
 	<depend type="module">res_ari</depend>
 	<depend type="module">res_ari_model</depend>
 	<depend type="module">res_stasis</depend>
+{{#requires_modules}}
+	<depend type="module">{{.}}</depend>
+{{/requires_modules}}
 	<support_level>core</support_level>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/app.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis_app.h"
@@ -90,7 +91,12 @@ static void ast_ari_{{c_name}}_{{c_nickn
 #endif /* AST_DEVMODE */
 
 {{> param_parsing}}
+{{^is_binary_response}}
 	ast_ari_{{c_name}}_{{c_nickname}}(headers, &args, response);
+{{/is_binary_response}}
+{{#is_binary_response}}
+	ast_ari_{{c_name}}_{{c_nickname}}(ser, headers, &args, response);
+{{/is_binary_response}}
 #if defined(AST_DEVMODE)
 	code = response->response_code;
 
@@ -113,8 +119,14 @@ static void ast_ari_{{c_name}}_{{c_nickn
 				ast_ari_validate_{{c_singular_name}}_fn());
 {{/is_list}}
 {{^is_list}}
+{{^is_binary_response}}
 			is_valid = ast_ari_validate_{{c_name}}(
 				response->message);
+{{/is_binary_response}}
+{{#is_binary_response}}
+			/* No validation on a raw binary response */
+			is_valid = 1;
+{{/is_binary_response}}
 {{/is_list}}
 {{/response_class}}
 		} else {
@@ -136,7 +148,8 @@ fin: __attribute__((unused))
 }
 {{/is_req}}
 {{#is_websocket}}
-static int ast_ari_{{c_name}}_{{c_nickname}}_ws_attempted_cb(struct ast_tcptls_session_instance *ser, struct ast_variable *get_params, struct ast_variable *headers)
+static int ast_ari_{{c_name}}_{{c_nickname}}_ws_attempted_cb(struct ast_tcptls_session_instance *ser,
+	struct ast_variable *get_params, struct ast_variable *headers, const char *session_id)
 {
 	struct ast_ari_{{c_name}}_{{c_nickname}}_args args = {};
 {{#has_parameters}}
@@ -155,7 +168,7 @@ static int ast_ari_{{c_name}}_{{c_nickna
 
 {{> param_parsing}}
 
-	res = ast_ari_websocket_{{c_name}}_{{c_nickname}}_attempted(ser, headers, &args);
+	res = ast_ari_websocket_{{c_name}}_{{c_nickname}}_attempted(ser, headers, &args, session_id);
 
 fin: __attribute__((unused))
 	if (!response) {
@@ -253,9 +266,9 @@ static int unload_module(void)
 {{#has_websocket}}
 	ao2_cleanup({{full_name}}.ws_server);
 	{{full_name}}.ws_server = NULL;
+	ast_ari_websocket_events_event_websocket_dtor();
 {{/has_websocket}}
 {{/apis}}
-	stasis_app_unref();
 	return 0;
 }
 
@@ -263,35 +276,35 @@ static int load_module(void)
 {
 	int res = 0;
 
-	CHECK_ARI_MODULE_LOADED();
-
 {{#apis}}
 {{#operations}}
 {{#is_websocket}}
-	/* This is scoped to not conflict with CHECK_ARI_MODULE_LOADED */
-	{
-		struct ast_websocket_protocol *protocol;
-
-		{{full_name}}.ws_server = ast_websocket_server_create();
-		if (!{{full_name}}.ws_server) {
-			return AST_MODULE_LOAD_DECLINE;
-		}
+	struct ast_websocket_protocol *protocol;
 
-		protocol = ast_websocket_sub_protocol_alloc("{{websocket_protocol}}");
-		if (!protocol) {
-			ao2_ref({{full_name}}.ws_server, -1);
-			{{full_name}}.ws_server = NULL;
-			return AST_MODULE_LOAD_DECLINE;
-		}
-		protocol->session_attempted = ast_ari_{{c_name}}_{{c_nickname}}_ws_attempted_cb;
-		protocol->session_established = ast_ari_{{c_name}}_{{c_nickname}}_ws_established_cb;
-		res |= ast_websocket_server_add_protocol2({{full_name}}.ws_server, protocol);
+	if (ast_ari_websocket_{{c_name}}_{{c_nickname}}_init() == -1) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	{{full_name}}.ws_server = ast_websocket_server_create();
+	if (!{{full_name}}.ws_server) {
+		ast_ari_websocket_events_event_websocket_dtor();
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	protocol = ast_websocket_sub_protocol_alloc("{{websocket_protocol}}");
+	if (!protocol) {
+		ao2_ref({{full_name}}.ws_server, -1);
+		{{full_name}}.ws_server = NULL;
+		ast_ari_websocket_events_event_websocket_dtor();
+		return AST_MODULE_LOAD_DECLINE;
 	}
+	protocol->session_attempted = ast_ari_{{c_name}}_{{c_nickname}}_ws_attempted_cb;
+	protocol->session_established = ast_ari_{{c_name}}_{{c_nickname}}_ws_established_cb;
+	res |= ast_websocket_server_add_protocol2({{full_name}}.ws_server, protocol);
 {{/is_websocket}}
 {{/operations}}
 {{/apis}}
 
-	stasis_app_ref();
 	res |= ast_ari_add_handler(&{{root_full_name}});
 	if (res) {
 		unload_module();
@@ -305,6 +318,6 @@ AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MO
 	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari,res_stasis",
-	);
+	.requires = "res_ari,res_ari_model,res_stasis{{#requires_modules}},{{.}}{{/requires_modules}}",
+);
 {{/api_declaration}}
diff -pruN 1:13.23.1~dfsg-2/rest-api-templates/swagger_model.py 1:16.1.1~dfsg-1/rest-api-templates/swagger_model.py
--- 1:13.23.1~dfsg-2/rest-api-templates/swagger_model.py	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/rest-api-templates/swagger_model.py	2018-12-26 18:45:13.000000000 +0000
@@ -333,6 +333,7 @@ class SwaggerType(Stringify):
         self.is_list = None
         self.singular_name = None
         self.is_primitive = None
+        self.is_binary = None
 
     def load(self, type_name, processor, context):
         # Some common errors
@@ -347,6 +348,7 @@ class SwaggerType(Stringify):
         else:
             self.singular_name = self.name
         self.is_primitive = self.singular_name in SWAGGER_PRIMITIVES
+        self.is_binary = (self.singular_name == 'binary')
         processor.process_type(self, context)
         return self
 
@@ -402,6 +404,7 @@ class Operation(Stringify):
         self.has_header_parameters = self.header_parameters and True
         self.has_parameters = self.has_query_parameters or \
             self.has_path_parameters or self.has_header_parameters
+        self.is_binary_response = self.response_class.is_binary
 
         # Body param is different, since there's at most one
         self.body_parameter = [
@@ -644,6 +647,7 @@ class ApiDeclaration(Stringify):
         self.api_version = api_decl_json.get('apiVersion')
         self.base_path = api_decl_json.get('basePath')
         self.resource_path = api_decl_json.get('resourcePath')
+        self.requires_modules = api_decl_json.get('requiresModules') or []
         api_json = api_decl_json.get('apis') or []
         self.apis = [
             Api().load(j, processor, context) for j in api_json]
diff -pruN 1:13.23.1~dfsg-2/tests/CI/buildAsterisk.sh 1:16.1.1~dfsg-1/tests/CI/buildAsterisk.sh
--- 1:13.23.1~dfsg-2/tests/CI/buildAsterisk.sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/buildAsterisk.sh	2018-12-26 18:45:13.000000000 +0000
@@ -3,8 +3,13 @@
 CIDIR=$(dirname $(readlink -fn $0))
 COVERAGE=0
 REF_DEBUG=0
+DISABLE_BINARY_MODULES=0
 source $CIDIR/ci.functions
 
+if [ -z $BRANCH_NAME ]; then
+	BRANCH_NAME=$(git config -f .gitreview --get gerrit.defaultbranch)
+fi
+
 gen_cats() {
 	set +x
 	action=$1
@@ -68,14 +73,20 @@ runner ccache -s
 runner ulimit -a
 
 MAKE=`which make`
+PKGCONFIG=`which pkg-config`
 [ -d /usr/lib64 ] && _libdir=/usr/lib64
 
 common_config_args="--prefix=/usr ${_libdir:+--libdir=${_libdir}} --sysconfdir=/etc --with-pjproject-bundled"
+$PKGCONFIG 'jansson >= 2.11' || common_config_args+=" --with-jansson-bundled"
 common_config_args+=" ${CACHE_DIR:+--with-sounds-cache=${CACHE_DIR}/sounds --with-externals-cache=${CACHE_DIR}/externals}"
 common_config_args+=" --enable-dev-mode"
 if [ $COVERAGE -eq 1 ] ; then
 	common_config_args+=" --enable-coverage"
 fi
+if [ "$BRANCH_NAME" == "master" -o $DISABLE_BINARY_MODULES -eq 1 ] ; then
+	common_config_args+=" --disable-binary-modules"
+fi
+
 export WGET_EXTRA_ARGS="--quiet"
 
 runner ./configure ${common_config_args} > ${OUTPUT_DIR:+${OUTPUT_DIR}/}configure.txt
@@ -113,14 +124,7 @@ if [ $TESTED_ONLY -eq 1 ] ; then
 	mod_disables+=" res_ael_share res_calendar res_config_ldap res_config_pgsql res_corosync"
 	mod_disables+=" res_http_post res_pktccops res_rtp_multicast res_snmp res_xmpp"
 fi
-if [ $REF_DEBUG -eq 1 ] ; then
-	# res_odbc does not unload at shutdown, including it with REF_DEBUG testing would cause
-	# every test to fail due to that leak.
-	# Note: --ref-debug and --realtime cannot be used together in this version of Asterisk.
-	# To test for reference leaks with realtime usage you must test against Asterisk 16+.
-	mod_disables+=" res_odbc"
-fi
-[ "$BRANCH_NAME" == "master" ] && mod_disables+=" codec_opus codec_silk codec_g729a codec_siren7 codec_siren14"
+
 runner menuselect/menuselect `gen_mods disable $mod_disables` menuselect.makeopts
 
 mod_enables="app_voicemail app_directory FILE_STORAGE"
diff -pruN 1:13.23.1~dfsg-2/tests/CI/gates.jenkinsfile 1:16.1.1~dfsg-1/tests/CI/gates.jenkinsfile
--- 1:13.23.1~dfsg-2/tests/CI/gates.jenkinsfile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/gates.jenkinsfile	2018-12-26 18:45:13.000000000 +0000
@@ -10,7 +10,19 @@
  * we need to dynamically determine which docker image we're going to use and
  * you can't do that in a delcarative pipeline.
  */
+def timeoutTime = 60
+def timeoutUnits = 'MINUTES'
+if (env.TIMEOUT_GATES) {
+	def _timeout = env.TIMEOUT_GATES.split()
+	timeoutTime = _timeout[0].toInteger()
+	timeoutUnits = _timeout[1]
+}
+
 pipeline {
+	options {
+		timestamps()
+		timeout(time: timeoutTime, unit: timeoutUnits)
+	}
 	triggers {
 		/*
 		 * This trigger will match either the "asterisk" or "Security-asterisk"
@@ -125,8 +137,9 @@ pipeline {
 					def r = currentBuild.startTimeInMillis % images.length
 					def ri = images[(int)r]
 					def randomImage = env.DOCKER_REGISTRY + "/" + ri
+					/* FYI... Jenkins takes care of mouting the workspace for the container */
 					def dockerOptions = "--privileged --ulimit core=0 --ulimit nofile=10240 " +
-						" -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
+						" --mount type=tmpfs,tmpfs-size=1g,dst=/tmp -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
 						" --entrypoint=''"
 					def bt = env.BUILD_TAG.replaceAll(/[^a-zA-Z0-9_.-]/, '-')
 					def outputdir = "tests/CI/output/Testsuite"
@@ -215,8 +228,8 @@ pipeline {
 	}
 	post {
 		cleanup {
-			sh "sudo make distclean 2&>/dev/null || : "
-			sh "sudo rm -rf tests/CI/output  2&>/dev/null || : "
+			sh "sudo make distclean >/dev/null 2>&1 || : "
+			sh "sudo rm -rf tests/CI/output >/dev/null 2>&1 || : "
 		}
 		/*
 		 * The Gerrit Trigger will automatically post the "Verified" results back
diff -pruN 1:13.23.1~dfsg-2/tests/CI/gateTestGroups.json 1:16.1.1~dfsg-1/tests/CI/gateTestGroups.json
--- 1:13.23.1~dfsg-2/tests/CI/gateTestGroups.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/gateTestGroups.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,46 +2,55 @@
 	{
 		"name": "ari1",
 		"dir": "tests/CI/output/ari1",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/rest_api/[Ca-d]"
 	},
 	{
 		"name": "ari2",
 		"dir": "tests/CI/output/ari2",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/rest_api/[e-z]"
 	},
 	{
 		"name": "pjs1",
 		"dir": "tests/CI/output/pjsip1",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/channels/pjsip/[a-f]"
 	},
 	{
 		"name": "pjs2",
 		"dir": "tests/CI/output/pjsip2",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/channels/pjsip/[g-r]"
 	},
 	{
 		"name": "pjs3",
 		"dir": "tests/CI/output/pjsip3",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/channels/pjsip/[s-z]"
 	},
 	{
 		"name": "sip1",
 		"dir": "tests/CI/output/sip1",
+		"runTestsuiteOptions": "--test-timeout=240",
 		"testcmd": "--test-regex=tests/channels/SIP/[Sa-r]"
 	},
 	{
 		"name": "sip2",
 		"dir": "tests/CI/output/sip2",
+		"runTestsuiteOptions": "--test-timeout=240",
 		"testcmd": "--test-regex=tests/channels/SIP/[s-z]"
 	},
 	{
 		"name": "iax ",
 		"dir": "tests/CI/output/iax2_local",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": " -t tests/channels/iax2 -t tests/channels/local"
 	},
 	{
 		"name": "mwi ",
 		"dir": "tests/CI/output/extmwi",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "--test-regex=tests/channels/pjsip/.*mwi"
 	}
 ]
diff -pruN 1:13.23.1~dfsg-2/tests/CI/periodic-dailyTestGroups.json 1:16.1.1~dfsg-1/tests/CI/periodic-dailyTestGroups.json
--- 1:13.23.1~dfsg-2/tests/CI/periodic-dailyTestGroups.json	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/periodic-dailyTestGroups.json	2018-12-26 18:45:13.000000000 +0000
@@ -2,37 +2,43 @@
 	{
 		"name": "ari ",
 		"dir": "tests/CI/output/ari",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "-t tests/rest_api/"
 	},
 	{
 		"name": "pjs ",
 		"dir": "tests/CI/output/pjsip",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": "-t tests/channels/pjsip"
 	},
 	{
 		"name": "sip ",
 		"dir": "tests/CI/output/sip",
+		"runTestsuiteOptions": "--test-timeout=240",
 		"testcmd": "-t tests/channels/SIP"
 	},
 	{
 		"name": "iax ",
 		"dir": "tests/CI/output/iax2_local",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": " -t tests/channels/iax2 -t tests/channels/local"
 	},
 	{
 		"name": "apps",
 		"dir": "tests/CI/output/agi-apps",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": " -t tests/agi -t tests/apps -t blind-transfer-parkingtimeout"
 	},
 	{
 		"name": "othr",
 		"dir": "tests/CI/output/other",
+		"runTestsuiteOptions": "--test-timeout=180",
 		"testcmd": " -T tests/(apps|agi|blind-transfer-parkingtimeout|rest_api|channels|realtime|example|skeleton_test|remote-test)"
 	},
 	{
 		"name": "real",
 		"dir": "tests/CI/output/realtime",
-		"runTestsuiteOptions": "--realtime --initialize-db --cleanup-db",
+		"runTestsuiteOptions": "--test-timeout=180 --realtime --initialize-db --cleanup-db",
 		"testcmd": " -t tests/channels/pjsip -G realtime-incompatible"
 	}
 ]
diff -pruN 1:13.23.1~dfsg-2/tests/CI/periodics-daily.jenkinsfile 1:16.1.1~dfsg-1/tests/CI/periodics-daily.jenkinsfile
--- 1:13.23.1~dfsg-2/tests/CI/periodics-daily.jenkinsfile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/periodics-daily.jenkinsfile	2018-12-26 18:45:13.000000000 +0000
@@ -10,7 +10,19 @@
  * we need to dynamically determine which docker image we're going to use and
  * you can't do that in a delcarative pipeline.
  */
+def timeoutTime = 3
+def timeoutUnits = 'HOURS'
+if (env.TIMEOUT_DAILIES) {
+	def _timeout = env.TIMEOUT_DAILIES.split()
+	timeoutTime = _timeout[0].toInteger()
+	timeoutUnits = _timeout[1]
+}
+
 pipeline {
+	options {
+		timestamps()
+		timeout(time: timeoutTime, unit: timeoutUnits)
+	}
 	triggers {
 		cron 'H H(0-4) * * *'
 	}
@@ -38,7 +50,7 @@ pipeline {
 					def ri = images[(int)r]
 					def randomImage = env.DOCKER_REGISTRY + "/" + ri
 					def dockerOptions = "--privileged --ulimit core=0 --ulimit nofile=10240 " +
-						" -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
+						" --mount type=tmpfs,tmpfs-size=1g,dst=/tmp -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
 						" --entrypoint=''"
 					def bt = env.BUILD_TAG.replaceAll(/[^a-zA-Z0-9_.-]/, '-')
 					def outputdir = "tests/CI/output/Testsuite"
@@ -140,8 +152,8 @@ pipeline {
 	}
 	post {
 		cleanup {
-			sh "sudo make distclean 2&>/dev/null || : "
-			sh "sudo rm -rf tests/CI/output  2&>/dev/null || : "
+			sh "sudo make distclean >/dev/null 2>&1 || : "
+			sh "sudo rm -rf tests/CI/output >/dev/null 2>&1 || : "
 		}
 		success {
 			echo "Reporting ${currentBuild.currentResult} Passed"
diff -pruN 1:13.23.1~dfsg-2/tests/CI/publishAsteriskDocs.sh 1:16.1.1~dfsg-1/tests/CI/publishAsteriskDocs.sh
--- 1:13.23.1~dfsg-2/tests/CI/publishAsteriskDocs.sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/publishAsteriskDocs.sh	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
+#!/usr/bin/env bash
 #
 # Publish Asterisk documentation to the wiki
 #
-#!/usr/bin/env bash
 CIDIR=$(dirname $(readlink -fn $0))
 source $CIDIR/ci.functions
 ASTETCDIR=$DESTDIR/etc/asterisk
@@ -113,7 +113,7 @@ ${OUTPUTDIR}/publish-rest-api.py --usern
 rm -f ${OUTPUTDIR}/full-en_US.xml
 
 sudo $ASTERISK ${USER_GROUP:+-U ${USER_GROUP%%:*} -G ${USER_GROUP##*:}} -gn -C $CONFFILE
-for n in `seq 1 5` ; do
+for n in {1..5} ; do
 	sleep 3
 	$ASTERISK -rx "core waitfullybooted" -C $CONFFILE && break
 done
diff -pruN 1:13.23.1~dfsg-2/tests/CI/ref_debug.jenkinsfile 1:16.1.1~dfsg-1/tests/CI/ref_debug.jenkinsfile
--- 1:13.23.1~dfsg-2/tests/CI/ref_debug.jenkinsfile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/ref_debug.jenkinsfile	2018-12-26 18:45:13.000000000 +0000
@@ -10,7 +10,19 @@
  * we need to dynamically determine which docker image we're going to use and
  * you can't do that in a delcarative pipeline.
  */
+def timeoutTime = 24
+def timeoutUnits = 'HOURS'
+if (env.TIMEOUT_REF_DEBUG) {
+	def _timeout = env.TIMEOUT_REF_DEBUG.split()
+	timeoutTime = _timeout[0].toInteger()
+	timeoutUnits = _timeout[1]
+}
+
 pipeline {
+	options {
+		timestamps()
+		timeout(time: timeoutTime, unit: timeoutUnits)
+	}
 	triggers {
 		cron 'H H(0-4) * * 0'
 	}
@@ -38,7 +50,7 @@ pipeline {
 					def ri = images[(int)r]
 					def randomImage = env.DOCKER_REGISTRY + "/" + ri
 					def dockerOptions = "--privileged --ulimit core=0 --ulimit nofile=10240 " +
-						" -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
+						" --mount type=tmpfs,tmpfs-size=1g,dst=/tmp -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
 						" --entrypoint=''"
 					def bt = env.BUILD_TAG.replaceAll(/[^a-zA-Z0-9_.-]/, '-')
 					def outputdir = "tests/CI/output/Testsuite"
@@ -115,8 +127,8 @@ pipeline {
 	}
 	post {
 		cleanup {
-			sh "sudo make distclean 2&>/dev/null || : "
-			sh "sudo rm -rf tests/CI/output  2&>/dev/null || : "
+			sh "sudo make distclean >/dev/null 2>&1 || : "
+			sh "sudo rm -rf tests/CI/output >/dev/null 2>&1 || : "
 		}
 		success {
 			echo "Reporting ${currentBuild.currentResult} Passed"
diff -pruN 1:13.23.1~dfsg-2/tests/CI/runTestsuite.sh 1:16.1.1~dfsg-1/tests/CI/runTestsuite.sh
--- 1:13.23.1~dfsg-2/tests/CI/runTestsuite.sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/runTestsuite.sh	2018-12-26 18:45:13.000000000 +0000
@@ -1,9 +1,15 @@
 #!/usr/bin/env bash
 CIDIR=$(dirname $(readlink -fn $0))
 REALTIME=0
+TEST_TIMEOUT=600
 source $CIDIR/ci.functions
 ASTETCDIR=$DESTDIR/etc/asterisk
 
+if [ x"$WORK_DIR" != x ] ; then
+	export AST_WORK_DIR="$(readlink -f $WORK_DIR)"
+	mkdir -p "$AST_WORK_DIR"
+fi
+
 pushd $TESTSUITE_DIR
 
 ./cleanup-test-remnants.sh
@@ -13,8 +19,8 @@ if [ $REALTIME -eq 1 ] ; then
 fi
 
 export PYTHONPATH=./lib/python/
-echo "Running tests ${TESTSUITE_COMMAND}"
-./runtests.py --cleanup ${TESTSUITE_COMMAND} | contrib/scripts/pretty_print --no-color --no-timer --term-width=120 --show-errors || :
+echo "Running tests ${TESTSUITE_COMMAND} ${AST_WORK_DIR:+with work directory ${AST_WORK_DIR}}"
+./runtests.py --cleanup --timeout=${TEST_TIMEOUT} ${TESTSUITE_COMMAND} | contrib/scripts/pretty_print --no-color --no-timer --term-width=120 --show-errors || :
 
 if [ $REALTIME -eq 1 ] ; then
 	$CIDIR/teardownRealtime.sh --cleanup-db=${CLEANUP_DB:?0}
diff -pruN 1:13.23.1~dfsg-2/tests/CI/runUnittests.sh 1:16.1.1~dfsg-1/tests/CI/runUnittests.sh
--- 1:13.23.1~dfsg-2/tests/CI/runUnittests.sh	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/runUnittests.sh	2018-12-26 18:45:13.000000000 +0000
@@ -1,11 +1,68 @@
 #!/usr/bin/env bash
 CIDIR=$(dirname $(readlink -fn $0))
+NO_EXPECT=0
 source $CIDIR/ci.functions
 ASTETCDIR=$DESTDIR/etc/asterisk
 
+asterisk_corefile_glob() {
+	local pattern=$(/sbin/sysctl -n kernel.core_pattern)
+
+	# If core_pattern is a pipe there isn't much we can do
+	if [[ ${pattern:0:1} == "|" ]] ; then
+		echo "core*"
+	else
+		echo "${pattern%%%*}*"
+	fi
+}
+
+run_tests_expect() {
+$EXPECT <<-EOF
+	spawn sudo $ASTERISK ${USER_GROUP:+-U ${USER_GROUP%%:*} -G ${USER_GROUP##*:}} -fcng -C $CONFFILE
+	match_max 512
+	set timeout 600
+	expect -notransfer "Asterisk Ready."
+	send "core show settings\r"
+	expect -notransfer "CLI>"
+	send "${UNITTEST_COMMAND:-test execute all}\r"
+	expect -notransfer -ex "Test(s) Executed"
+	expect -notransfer "CLI>"
+	send "test show results failed\r"
+	expect -notransfer "CLI>"
+	send "test generate results xml ${OUTPUTFILE}\r"
+	expect -notransfer "CLI>"
+	send "core stop now\r"
+	expect -notransfer "Executing last minute cleanups"
+	wait
+EOF
+}
+
+run_tests_socket() {
+	sudo $ASTERISK ${USER_GROUP:+-U ${USER_GROUP%%:*} -G ${USER_GROUP##*:}} -gn -C $CONFFILE
+	for n in {1..5} ; do
+		sleep 3
+		$ASTERISK -rx "core waitfullybooted" -C $CONFFILE && break
+	done
+	sleep 1
+	$ASTERISK -rx "core show settings" -C $CONFFILE
+	$ASTERISK -rx "${UNITTEST_COMMAND:-test execute all}" -C $CONFFILE
+	$ASTERISK -rx "test show results failed" -C $CONFFILE
+	$ASTERISK -rx "test generate results xml $OUTPUTFILE" -C $CONFFILE
+	$ASTERISK -rx "core stop now" -C $CONFFILE
+}
+
+# If DESTDIR is used to install and run asterisk from non standard locations,
+# the directory entries in asterisk.conf need to be munged to prepend DESTDIR.
+ALTERED=$(head -10 ../tmp/DESTDIR/etc/asterisk/asterisk.conf | grep -q "DESTDIR" && echo yes)
+if [ x"$ALTERED" = x ] ; then
+	# In the section that starts with [directories and ends with a blank line,
+	# replace "=> " with "=> ${DESTDIR}"
+	sed -i -r -e "/^\[directories/,/^$/ s@=>\s+@=> ${DESTDIR}@" "$ASTETCDIR/asterisk.conf"
+fi
+
 cat <<-EOF > "$ASTETCDIR/logger.conf"
 	[logfiles]
 	full => notice,warning,error,debug,verbose
+	console => notice,warning,error
 EOF
 
 echo "[default]" > "$ASTETCDIR/extensions.conf"
@@ -26,7 +83,7 @@ cat <<-EOF > "$ASTETCDIR/http.conf"
 	[general]
 	enabled=yes
 	bindaddr=127.0.0.1
-	port=8088
+	bindport=8088
 EOF
 
 cat <<-EOF > "$ASTETCDIR/modules.conf"
@@ -47,6 +104,7 @@ ASTERISK="$DESTDIR/usr/sbin/asterisk"
 CONFFILE=$ASTETCDIR/asterisk.conf
 OUTPUTDIR=${OUTPUT_DIR:-tests/CI/output/}
 OUTPUTFILE=${OUTPUT_XML:-${OUTPUTDIR}/unittests-results.xml}
+EXPECT="$(which expect 2>/dev/null || : )"
 
 [ ! -d ${OUTPUTDIR} ] && mkdir -p $OUTPUTDIR
 [ x"$USER_GROUP" != x ] && sudo chown -R $USER_GROUP $OUTPUTDIR
@@ -54,23 +112,28 @@ OUTPUTFILE=${OUTPUT_XML:-${OUTPUTDIR}/un
 rm -rf $ASTETCDIR/extensions.{ael,lua} || :
 
 set -x
-sudo $ASTERISK ${USER_GROUP:+-U ${USER_GROUP%%:*} -G ${USER_GROUP##*:}} -gn -C $CONFFILE
-for n in `seq 1 5` ; do
-	sleep 3
-	$ASTERISK -rx "core waitfullybooted" -C $CONFFILE && break
-done
-sleep 1
-$ASTERISK -rx "${UNITTEST_COMMAND:-test execute all}" -C $CONFFILE
-$ASTERISK -rx "test show results failed" -C $CONFFILE
-$ASTERISK -rx "test generate results xml $OUTPUTFILE" -C $CONFFILE
-$ASTERISK -rx "core stop now" -C $CONFFILE
+if [ x"$EXPECT" != x -a $NO_EXPECT -eq 0 ] ; then
+	run_tests_expect
+else
+	run_tests_socket
+fi
+
+# Cleanup "just in case"
+sudo killall -qe -ABRT $ASTERISK 
 
 runner rsync -vaH $DESTDIR/var/log/asterisk/. $OUTPUTDIR
 set +x
 
 [ x"$USER_GROUP" != x ] && sudo chown -R $USER_GROUP $OUTPUTDIR
-if [ -f core* ] ; then
-	echo "*** Found a core file after running unit tests ***"
-	$DESTDIR/var/lib/asterisk/scripts/ast_coredumper --no-default-search core*
-	exit 1
-fi
+
+for core in $(asterisk_corefile_glob)
+do
+	if [ -f $core ]
+	then
+		echo "*** Found a core file ($core) after running unit tests ***"
+		set -x
+		sudo OUTPUTDIR=$OUTPUTDIR $DESTDIR/var/lib/asterisk/scripts/ast_coredumper --no-default-search $core
+	fi
+done
+
+exit 0
diff -pruN 1:13.23.1~dfsg-2/tests/CI/unittests.jenkinsfile 1:16.1.1~dfsg-1/tests/CI/unittests.jenkinsfile
--- 1:13.23.1~dfsg-2/tests/CI/unittests.jenkinsfile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/CI/unittests.jenkinsfile	2018-12-26 18:45:13.000000000 +0000
@@ -10,7 +10,19 @@
  * we need to dynamically determine which docker image we're going to use and
  * you can't do that in a delcarative pipeline.
  */
+def timeoutTime = 30
+def timeoutUnits = 'MINUTES'
+if (env.TIMEOUT_UNITTESTS) {
+	def _timeout = env.TIMEOUT_UNITTESTS.split()
+	timeoutTime = _timeout[0].toInteger()
+	timeoutUnits = _timeout[1]
+}
+
 pipeline {
+	options {
+		timestamps()
+		timeout(time: timeoutTime, unit: timeoutUnits)
+	}
 	triggers {
 		/*
 		 * This trigger will match either the "asterisk" or "Security-asterisk"
@@ -129,7 +141,7 @@ pipeline {
 					def randomImage = env.DOCKER_REGISTRY + "/" + ri;
 					def bt = env.BUILD_TAG.replaceAll(/[^a-zA-Z0-9_.-]/, '-')
 					def dockerOptions = "--privileged --ulimit core=0 --ulimit nofile=10240 " +
-						" -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
+						" --mount type=tmpfs,tmpfs-size=1g,dst=/tmp -v /srv/jenkins:/srv/jenkins:rw -v /srv/cache:/srv/cache:rw " +
 						" --entrypoint='' --name ${bt}-build"
 					def outputdir = "tests/CI/output/UnitTests"
 
@@ -169,8 +181,8 @@ pipeline {
 	}
 	post {
 		cleanup {
-			sh "sudo make distclean 2&>/dev/null || : "
-			sh "sudo rm -rf tests/CI/output 2&>/dev/null || : "
+			sh "sudo make distclean >/dev/null 2>&1 || : "
+			sh "sudo rm -rf tests/CI/output >/dev/null 2>&1 || : "
 		}
 		/*
 		 * The Gerrit Trigger will automatically post the "Verified" results back
diff -pruN 1:13.23.1~dfsg-2/tests/Makefile 1:16.1.1~dfsg-1/tests/Makefile
--- 1:13.23.1~dfsg-2/tests/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -19,5 +19,6 @@ all: _all
 
 include $(ASTTOPDIR)/Makefile.moddir_rules
 
+test_astobj2.o: _ASTCFLAGS+=$(call get_menuselect_cflags,AO2_DEBUG)
 test_strings.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION) $(AST_NO_STRINGOP_TRUNCATION)
 test_voicemail_api.o: _ASTCFLAGS+=$(AST_NO_FORMAT_TRUNCATION)
diff -pruN 1:13.23.1~dfsg-2/tests/test_abstract_jb.c 1:16.1.1~dfsg-1/tests/test_abstract_jb.c
--- 1:13.23.1~dfsg-2/tests/test_abstract_jb.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_abstract_jb.c	2018-12-26 18:45:13.000000000 +0000
@@ -36,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_acl.c 1:16.1.1~dfsg-1/tests/test_acl.c
--- 1:13.23.1~dfsg-2/tests/test_acl.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_acl.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/acl.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_amihooks.c 1:16.1.1~dfsg-1/tests/test_amihooks.c
--- 1:13.23.1~dfsg-2/tests/test_amihooks.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_amihooks.c	2018-12-26 18:45:13.000000000 +0000
@@ -22,8 +22,10 @@
  *
  * \author David Brooks <dbrooks@digium.com> based off of code written by Russell Bryant <russell@digium.com>
  *
- * This is simply an example or test module illustrating the ability for a custom module
- * to hook into AMI. Registration for AMI events and sending of AMI actions is shown.
+ * This started, and continues to serves, as an example illustrating the ability
+ * for a custom module to hook into AMI. Registration for AMI events and sending
+ * of AMI actions is shown. A test has also been created that utilizes the original
+ * example in order to make sure the ami event hook gets raised.
  */
 
 /*** MODULEINFO
@@ -33,18 +35,71 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/cli.h"
 #include "asterisk/utils.h"
 #include "asterisk/manager.h"
+#include "asterisk/test.h"
+
+#define CATEGORY "/main/amihooks/"
+
+AST_MUTEX_DEFINE_STATIC(hook_lock);
+ast_cond_t hook_cond;
+int done;
+
+static int wait_for_hook(struct ast_test *test)
+{
+	struct timeval start = ast_tvnow();
+	struct timespec timeout = {
+		.tv_sec = start.tv_sec + 2,
+		.tv_nsec = start.tv_usec * 1000
+	};
+	int res = 0;
+
+	ast_mutex_lock(&hook_lock);
+	while (!done) {
+		if (ast_cond_timedwait(&hook_cond, &hook_lock, &timeout) == ETIMEDOUT) {
+			ast_test_status_update(test, "Test timed out while waiting for hook event\n");
+			res = -1;
+			break;
+		}
+	}
+	ast_mutex_unlock(&hook_lock);
+
+	return res;
+}
+
+AST_TEST_DEFINE(amihook_cli_send)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Execute an action using an AMI hook";
+		info->description = info->summary;
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	done = 0;
+	if (ast_cli_command(-1, "amihook send")) {
+		return AST_TEST_FAIL;
+	}
+
+	return wait_for_hook(test) ? AST_TEST_FAIL : AST_TEST_PASS;
+}
 
 /* The helper function is required by struct manager_custom_hook.
  * See __ast_manager_event_multichan for details */
 static int amihook_helper(int category, const char *event, char *content)
 {
 	ast_log(LOG_NOTICE, "AMI Event: \nCategory: %d Event: %s\n%s\n", category, event, content);
+
+	ast_mutex_lock(&hook_lock);
+	done = 1;
+	ast_cond_signal(&hook_cond);
+	ast_mutex_unlock(&hook_lock);
 	return 0;
 }
 
@@ -144,6 +199,7 @@ static struct ast_cli_entry cli_amihook_
 
 static int unload_module(void)
 {
+	AST_TEST_UNREGISTER(amihook_cli_send);
 	ast_manager_unregister_hook(&test_hook);
 	return ast_cli_unregister_multiple(cli_amihook_evt, ARRAY_LEN(cli_amihook_evt));
 }
@@ -154,6 +210,8 @@ static int load_module(void)
 
 	res = ast_cli_register_multiple(cli_amihook_evt, ARRAY_LEN(cli_amihook_evt));
 
+	AST_TEST_REGISTER(amihook_cli_send);
+
 	return res ? AST_MODULE_LOAD_DECLINE : AST_MODULE_LOAD_SUCCESS;
 }
 
diff -pruN 1:13.23.1~dfsg-2/tests/test_aoc.c 1:16.1.1~dfsg-1/tests/test_aoc.c
--- 1:13.23.1~dfsg-2/tests/test_aoc.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_aoc.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_app.c 1:16.1.1~dfsg-1/tests/test_app.c
--- 1:13.23.1~dfsg-2/tests/test_app.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_app.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_ari.c 1:16.1.1~dfsg-1/tests/test_ari.c
--- 1:13.23.1~dfsg-2/tests/test_ari.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_ari.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test ARI API.
+ * \file
+ * \brief Test ARI API.
  * \author\verbatim David M. Lee, II <dlee@digium.com> \endverbatim
  *
  * \ingroup tests
@@ -31,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/test.h"
 #include "asterisk/ari.h"
@@ -572,7 +571,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ARI testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_ari",
-	);
+	.requires = "res_ari",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_ari_model.c 1:16.1.1~dfsg-1/tests/test_ari_model.c
--- 1:13.23.1~dfsg-2/tests/test_ari_model.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_ari_model.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
@@ -42,8 +40,8 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 /*!
  * Wrapper of ast_test_validate_int() so an external function pointer is not used.
  *
- * \note Must do this because using an external function pointer
- * does not play nicely when loading with RTLD_LAZY.
+ * \note We do this because using an external function pointer
+ * did not play nicely when we loaded with RTLD_LAZY.
  */
 static int wrap_ast_ari_validate_int(struct ast_json *json)
 {
@@ -55,8 +53,8 @@ static int wrap_ast_ari_validate_int(str
 /*!
  * Wrapper of ast_ari_validate_string() so an external function pointer is not used.
  *
- * \note Must do this because using an external function pointer
- * does not play nicely when loading with RTLD_LAZY.
+ * \note We do this because using an external function pointer
+ * did not play nicely when we loaded with RTLD_LAZY.
  */
 static int wrap_ast_ari_validate_string(struct ast_json *json)
 {
@@ -454,4 +452,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Skeleton (sample) Test");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Skeleton (sample) Test",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_ari_model",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_ast_format_str_reduce.c 1:16.1.1~dfsg-1/tests/test_ast_format_str_reduce.c
--- 1:13.23.1~dfsg-2/tests/test_ast_format_str_reduce.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_ast_format_str_reduce.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/file.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_astobj2.c 1:16.1.1~dfsg-1/tests/test_astobj2.c
--- 1:13.23.1~dfsg-2/tests/test_astobj2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_astobj2.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
@@ -458,7 +456,7 @@ static int astobj2_test_1_helper(int tst
 			test_sort_cb, test_cmp_cb, "test");
 		break;
 	}
-	c2 = ao2_t_container_alloc(1, NULL, NULL, "test");
+	c2 = ao2_t_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL, "test");
 
 	if (!c1 || !c2) {
 		ast_test_status_update(test, "ao2_container_alloc failed.\n");
@@ -714,9 +712,9 @@ AST_TEST_DEFINE(astobj2_test_2)
 		break;
 	}
 
-	c = ao2_container_alloc(1, NULL, test_cmp_cb);
+	c = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, test_cmp_cb);
 	if (!c) {
-		ast_test_status_update(test, "ao2_container_alloc failed.\n");
+		ast_test_status_update(test, "ao2_container_alloc_list failed.\n");
 		res = AST_TEST_FAIL;
 		goto cleanup;
 	}
diff -pruN 1:13.23.1~dfsg-2/tests/test_astobj2_thrash.c 1:16.1.1~dfsg-1/tests/test_astobj2_thrash.c
--- 1:13.23.1~dfsg-2/tests/test_astobj2_thrash.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_astobj2_thrash.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,7 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include <pthread.h>
 #include "asterisk/astobj2.h"
 #include "asterisk/hashtab.h"
@@ -268,8 +267,8 @@ AST_TEST_DEFINE(hash_test)
 	data.preload = MAX_HASH_ENTRIES / 2;
 	data.max_grow = MAX_HASH_ENTRIES - data.preload;
 	data.deadline = ast_tvadd(ast_tvnow(), ast_tv(MAX_TEST_SECONDS, 0));
-	data.to_be_thrashed = ao2_container_alloc(HASH_BUCKETS, hash_string,
-		compare_strings);
+	data.to_be_thrashed = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		HASH_BUCKETS, hash_string, NULL, compare_strings);
 
 	if (data.to_be_thrashed == NULL) {
 		ast_test_status_update(test, "Allocation failed\n");
diff -pruN 1:13.23.1~dfsg-2/tests/test_astobj2_weaken.c 1:16.1.1~dfsg-1/tests/test_astobj2_weaken.c
--- 1:13.23.1~dfsg-2/tests/test_astobj2_weaken.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_astobj2_weaken.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,437 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, CFWare, LLC
+ *
+ * Corey Farrell <git@cfware.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief astobj2 weakproxy test module
+ *
+ * \author Corey Farrell <git@cfware.com>
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/utils.h"
+#include "asterisk/module.h"
+#include "asterisk/test.h"
+#include "asterisk/astobj2.h"
+
+static int destructor_called;
+static int weakproxydestroyed;
+
+static void test_obj_destructor(void *obj)
+{
+	destructor_called++;
+}
+
+static void weakproxy_destructor(void *obj)
+{
+	weakproxydestroyed++;
+}
+
+static void test_obj_destroy_notify(void *obj, void *data)
+{
+	int *i = data;
+
+	++*i;
+}
+
+struct my_weakproxy {
+	AO2_WEAKPROXY();
+	int f1;
+};
+
+AST_TEST_DEFINE(astobj2_weak1)
+{
+	void *obj1 = NULL;
+	void *obj2 = NULL;
+	void *obj3 = NULL;
+	void *strong1 = NULL;
+	struct my_weakproxy *weakref1 = NULL;
+	struct my_weakproxy *weakref2 = NULL;
+	int notify0_called = 0;
+	int notify1_called = 0;
+	int notify2_called = 0;
+	int notify3_called = 0;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "astobj2_weak1";
+		info->category = "/main/astobj2/";
+		info->summary = "Test ao2 weak objects";
+		info->description = "Test ao2 weak objects.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	destructor_called = weakproxydestroyed = 0;
+	obj1 = ao2_t_alloc(0, test_obj_destructor, "obj1");
+	if (!obj1) {
+		return AST_TEST_FAIL;
+	}
+
+	weakref1 = ao2_t_weakproxy_alloc(sizeof(*weakref1), weakproxy_destructor, "weakref1");
+	if (!weakref1) {
+		ast_test_status_update(test, "Failed to allocate weakref1.\n");
+		goto fail_cleanup;
+	}
+	weakref1->f1 = 5315;
+
+	if (ao2_weakproxy_subscribe(weakref1, test_obj_destroy_notify, &notify0_called, 0)) {
+		ast_test_status_update(test, "Failed to subscribe to weakref1.\n");
+		goto fail_cleanup;
+	}
+	if (!notify0_called) {
+		ast_test_status_update(test, "Subscribe failed to immediately run callback for empty weakproxy.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_weakproxy_set_object(weakref1, obj1, 0, "set weakref1 to obj1")) {
+		ast_test_status_update(test, "Failed to set obj1 on weakref1.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(weakref1, test_obj_destroy_notify, &notify1_called, 0)) {
+		ast_test_status_update(test, "Failed to add a subscription to weakref1.\n");
+		goto fail_cleanup;
+	}
+
+	weakref2 = ao2_t_get_weakproxy(obj1, "get weakref2 from obj1");
+	if (weakref1 != weakref2) {
+		ast_test_status_update(test, "weakref1 != weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(weakref2, test_obj_destroy_notify, &notify2_called, 0)) {
+		ast_test_status_update(test, "Failed to add a subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(weakref2, test_obj_destroy_notify, &notify2_called, 0)) {
+		ast_test_status_update(test, "Failed to add a duplicate subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(weakref2, test_obj_destroy_notify, &notify2_called, 0)) {
+		ast_test_status_update(test, "Failed to add a second duplicate subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_unsubscribe(weakref2, test_obj_destroy_notify, &notify2_called, 0) != 1) {
+		ast_test_status_update(test, "Failed to remove a subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	ao2_t_cleanup(weakref1, "weakref1");
+	ao2_t_cleanup(weakref2, "weakref2");
+
+	weakref2 = ao2_t_get_weakproxy(obj1, "get weakref2 from obj1");
+	if (weakref1 != weakref2) {
+		weakref1 = NULL;
+		ast_test_status_update(test, "weakref1 != weakref2.\n");
+		goto fail_cleanup;
+	}
+	weakref1 = NULL;
+
+	obj2 = ao2_t_alloc(0, NULL, "obj2");
+	if (obj2) {
+		int ret = ao2_t_weakproxy_set_object(weakref2, obj2, 0, "set weakref2 to obj2");
+
+		ao2_ref(obj2, -1);
+		if (!ret) {
+			ast_test_status_update(test, "Set obj2 to weakref2 when it already had an object.\n");
+			goto fail_cleanup;
+		}
+	}
+
+	if (ao2_weakproxy_subscribe(weakref2, test_obj_destroy_notify, &notify3_called, 0)) {
+		ast_test_status_update(test, "Failed to add a subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_subscribe(weakref2, test_obj_destroy_notify, &notify3_called, 0)) {
+		ast_test_status_update(test, "Failed to add a duplicate subscription to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_weakproxy_unsubscribe(weakref2, test_obj_destroy_notify, &notify3_called, OBJ_MULTIPLE) != 2) {
+		ast_test_status_update(test, "Failed to remove the correct number of subscriptions to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (destructor_called || notify1_called || notify2_called || notify3_called) {
+		ast_test_status_update(test, "Destructor or notifications called early.\n");
+		goto fail_cleanup;
+	}
+
+	strong1 = ao2_t_weakproxy_get_object(weakref2, 0, "get strong1 from weakref2");
+	ao2_t_cleanup(strong1, "strong1");
+
+	if (obj1 != strong1) {
+		ast_test_status_update(test, "obj1 != strong1.\n");
+		goto fail_cleanup;
+	}
+
+	if (destructor_called || notify1_called || notify2_called || notify3_called) {
+		ast_test_status_update(test, "Destructor or notification called early.\n");
+		goto fail_cleanup;
+	}
+
+	ao2_t_ref(obj1, -1, "obj1");
+	obj1 = NULL;
+
+	if (destructor_called != 1 || notify1_called != 1 || notify2_called != 2 || notify3_called != 0) {
+		ast_test_status_update(test, "Destructor or notification not called the expected number of times.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_weakproxy_get_object(weakref2, 0, "impossible get of weakref2") != NULL) {
+		ast_test_status_update(test, "Get object on weakref2 worked when it shouldn't\n");
+		goto fail_cleanup;
+	}
+
+	obj3 = ao2_t_alloc(0, test_obj_destructor, "obj3");
+	if (!obj3) {
+		ast_test_status_update(test, "Failed to allocate obj3.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_weakproxy_set_object(weakref2, obj3, 0, "set weakref2 to obj3")) {
+		ast_test_status_update(test, "Failed to set obj3 to weakref2.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_weakproxy_ref_object(obj3, +1, 0, "ao2_ref should never see this") != -2) {
+		ast_test_status_update(test,
+			"Expected -2 from ao2_t_weakproxy_ref_object against normal ao2 object.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_weakproxy_ref_object(weakref2, +1, 0, "weakref2 ref_object") != 2) {
+		ast_test_status_update(test, "Expected 2 from weakref2 ref_object.\n");
+		goto fail_cleanup;
+	}
+
+	if (ao2_t_ref(obj3, -1, "balance weakref2 ref_object") != 3) {
+		ast_test_status_update(test, "Expected 3 from obj3 ao2_t_ref.\n");
+		goto fail_cleanup;
+	}
+
+	ao2_ref(obj3, -1);
+
+	if (ao2_weakproxy_ref_object(weakref2, +1, 0) != -1) {
+		ast_test_status_update(test, "Expected -1 from weakref2 ref_object because obj3 is gone.\n");
+		goto fail_cleanup;
+	}
+
+	ao2_t_ref(weakref2, -1, "weakref2");
+
+	if (!weakproxydestroyed) {
+		ast_test_status_update(test, "Destructor never called for weakproxy, likely a leak.\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+
+fail_cleanup:
+	ao2_cleanup(obj1);
+	ao2_cleanup(obj3);
+	ao2_cleanup(weakref1);
+	ao2_cleanup(weakref2);
+
+	return AST_TEST_FAIL;
+}
+
+struct strong_str {
+	char *value;
+};
+
+struct weakproxy_str {
+	AO2_WEAKPROXY();
+	char value[0];
+};
+
+static struct strong_str *alloc_str(struct ao2_container *weakcontainer, const char *value)
+{
+	struct strong_str *strong = ao2_t_alloc(sizeof(*strong), NULL, value);
+	struct weakproxy_str *weak = ao2_weakproxy_alloc(sizeof(*weak) + strlen(value) + 1, NULL);
+
+	if (!weak || !strong) {
+		goto error_return;
+	}
+
+	strcpy(weak->value, value); /*SAFE*/
+	strong->value = weak->value;
+
+	if (ao2_weakproxy_set_object(weak, strong, 0)) {
+		goto error_return;
+	}
+
+	if (!ao2_link(weakcontainer, weak)) {
+		goto error_return;
+	}
+
+	ao2_ref(weak, -1);
+	return strong;
+
+error_return:
+	ao2_cleanup(weak);
+	ao2_cleanup(strong);
+
+	return NULL;
+}
+
+AO2_STRING_FIELD_HASH_FN(weakproxy_str, value);
+AO2_STRING_FIELD_CMP_FN(weakproxy_str, value);
+AO2_STRING_FIELD_SORT_FN(strong_str, value);
+
+#define ITERATOR_CHECK_NEXT(iter, var, expected) \
+	do { \
+		var = ao2_iterator_next(iter); \
+		ast_test_validate_cleanup(test, var == expected, ret, cleanup); \
+		ao2_cleanup(var); \
+	} while (0)
+
+#define WEAKFIND_CHECK(c, key, var, expected) \
+	do { \
+		var = ao2_weakproxy_find(c, key, OBJ_SEARCH_KEY, ""); \
+		ast_test_validate_cleanup(test, var == expected, ret, cleanup); \
+		ao2_cleanup(var); \
+	} while (0)
+
+AST_TEST_DEFINE(astobj2_weak_container)
+{
+	int ret = AST_TEST_FAIL;
+
+	struct strong_str *strong1 = NULL;
+	struct strong_str *strong2 = NULL;
+	struct strong_str *strong3 = NULL;
+
+	struct strong_str *strong = NULL;
+
+	struct ao2_container *weakcontainer = NULL;
+	struct ao2_container *dupcontainer = NULL;
+
+	struct ao2_iterator iter;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "astobj2_weak_container";
+		info->category = "/main/astobj2/";
+		info->summary = "Test ao2 weak containers";
+		info->description = "Test ao2 weak containers.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	weakcontainer = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 7,
+		weakproxy_str_hash_fn, NULL, weakproxy_str_cmp_fn);
+	dupcontainer = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0,
+		strong_str_sort_fn, NULL);
+
+	if (!weakcontainer || !dupcontainer) {
+		goto cleanup;
+	}
+
+	strong1 = alloc_str(weakcontainer, "obj1");
+	strong2 = alloc_str(weakcontainer, "obj2");
+	strong3 = alloc_str(weakcontainer, "obj3");
+
+	if (!strong1 || !strong2 || !strong3) {
+		goto cleanup;
+	}
+
+	if (ao2_container_dup_weakproxy_objs(dupcontainer, weakcontainer, 0)) {
+		goto cleanup;
+	}
+
+	iter = ao2_iterator_init(dupcontainer, 0);
+	ITERATOR_CHECK_NEXT(&iter, strong, strong1);
+	ITERATOR_CHECK_NEXT(&iter, strong, strong2);
+	ITERATOR_CHECK_NEXT(&iter, strong, strong3);
+	ITERATOR_CHECK_NEXT(&iter, strong, NULL);
+	ao2_iterator_cleanup(&iter);
+
+	ao2_callback(dupcontainer, OBJ_NODATA | OBJ_UNLINK | OBJ_MULTIPLE, NULL, NULL);
+
+	WEAKFIND_CHECK(weakcontainer, "obj1", strong, strong1);
+	WEAKFIND_CHECK(weakcontainer, "obj2", strong, strong2);
+	WEAKFIND_CHECK(weakcontainer, "obj3", strong, strong3);
+	WEAKFIND_CHECK(weakcontainer, "unknown", strong, NULL);
+
+	/* This will orphan "obj2" in weakcontainer. */
+	ao2_replace(strong2, NULL);
+
+	if (ao2_container_dup_weakproxy_objs(dupcontainer, weakcontainer, 0)) {
+		goto cleanup;
+	}
+
+	ast_test_validate_cleanup(test,
+		ao2_container_count(weakcontainer) == ao2_container_count(dupcontainer) + 1,
+		ret,
+		cleanup);
+
+	iter = ao2_iterator_init(dupcontainer, 0);
+	ITERATOR_CHECK_NEXT(&iter, strong, strong1);
+	ITERATOR_CHECK_NEXT(&iter, strong, strong3);
+	ITERATOR_CHECK_NEXT(&iter, strong, NULL);
+	ao2_iterator_cleanup(&iter);
+
+	WEAKFIND_CHECK(weakcontainer, "obj1", strong, strong1);
+	WEAKFIND_CHECK(weakcontainer, "obj2", strong, NULL);
+	WEAKFIND_CHECK(weakcontainer, "obj3", strong, strong3);
+	WEAKFIND_CHECK(weakcontainer, "unknown", strong, NULL);
+
+	ret = AST_TEST_PASS;
+
+cleanup:
+	ao2_cleanup(strong1);
+	ao2_cleanup(strong2);
+	ao2_cleanup(strong3);
+
+	ao2_cleanup(weakcontainer);
+	ao2_cleanup(dupcontainer);
+
+	ao2_cleanup(strong);
+
+	return ret;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(astobj2_weak1);
+	AST_TEST_UNREGISTER(astobj2_weak_container);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(astobj2_weak1);
+	AST_TEST_REGISTER(astobj2_weak_container);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "ASTOBJ2 Weak Reference Unit Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_bridging.c 1:16.1.1~dfsg-1/tests/test_bridging.c
--- 1:13.23.1~dfsg-2/tests/test_bridging.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_bridging.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/test.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_bucket.c 1:16.1.1~dfsg-1/tests/test_bucket.c
--- 1:13.23.1~dfsg-2/tests/test_bucket.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_bucket.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include <sys/stat.h>
 
 #include "asterisk/test.h"
@@ -50,6 +48,8 @@ struct bucket_test_state {
 	unsigned int updated:1;
 	/*! \brief Whether the object has been deleted or not */
 	unsigned int deleted:1;
+	/*! \brief Whether the object is stale or not */
+	unsigned int is_stale:1;
 };
 
 /*! \brief Global scope structure for testing bucket wizards */
@@ -60,6 +60,7 @@ static void bucket_test_wizard_clear(voi
 	bucket_test_wizard_state.created = 0;
 	bucket_test_wizard_state.updated = 0;
 	bucket_test_wizard_state.deleted = 0;
+	bucket_test_wizard_state.is_stale = 0;
 }
 
 static int bucket_test_wizard_create(const struct ast_sorcery *sorcery, void *data, void *object)
@@ -107,11 +108,17 @@ static int bucket_test_wizard_delete(con
 	return 0;
 }
 
+static int bucket_test_wizard_is_stale(const struct ast_sorcery *sorcery, void *data, void *object)
+{
+	return bucket_test_wizard_state.is_stale;
+}
+
 static struct ast_sorcery_wizard bucket_test_wizard = {
 	.name = "test",
 	.create = bucket_test_wizard_create,
 	.retrieve_id = bucket_test_wizard_retrieve_id,
 	.delete = bucket_test_wizard_delete,
+	.is_stale = bucket_test_wizard_is_stale,
 };
 
 static struct ast_sorcery_wizard bucket_file_test_wizard = {
@@ -120,6 +127,7 @@ static struct ast_sorcery_wizard bucket_
 	.update = bucket_test_wizard_update,
 	.retrieve_id = bucket_test_wizard_retrieve_id,
 	.delete = bucket_test_wizard_delete,
+	.is_stale = bucket_test_wizard_is_stale,
 };
 
 AST_TEST_DEFINE(bucket_scheme_register)
@@ -233,6 +241,50 @@ AST_TEST_DEFINE(bucket_create)
 	return AST_TEST_PASS;
 }
 
+AST_TEST_DEFINE(bucket_clone)
+{
+	RAII_VAR(struct ast_bucket *, bucket, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_bucket *, clone, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "bucket_clone";
+		info->category = "/main/bucket/";
+		info->summary = "bucket clone unit test";
+		info->description =
+			"Test cloning a bucket";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (!(bucket = ast_bucket_alloc("test:///tmp/bob"))) {
+		ast_test_status_update(test, "Failed to allocate bucket\n");
+		return AST_TEST_FAIL;
+	}
+
+	bucket_test_wizard_clear();
+
+	if (ast_bucket_create(bucket)) {
+		ast_test_status_update(test, "Failed to create bucket with URI '%s'\n",
+			ast_sorcery_object_get_id(bucket));
+		return AST_TEST_FAIL;
+	}
+
+	clone = ast_bucket_clone(bucket);
+	if (!clone) {
+		ast_test_status_update(test, "Failed to clone bucket with URI '%s'\n",
+			ast_sorcery_object_get_id(bucket));
+		return AST_TEST_FAIL;
+	}
+
+	ast_test_validate(test, strcmp(ast_sorcery_object_get_id(bucket), ast_sorcery_object_get_id(clone)) == 0);
+	ast_test_validate(test, bucket->scheme_impl == clone->scheme_impl);
+	ast_test_validate(test, strcmp(bucket->scheme, clone->scheme) == 0);
+
+	return AST_TEST_PASS;
+}
+
 AST_TEST_DEFINE(bucket_delete)
 {
 	RAII_VAR(struct ast_bucket *, bucket, NULL, ao2_cleanup);
@@ -276,6 +328,38 @@ AST_TEST_DEFINE(bucket_delete)
 	return AST_TEST_PASS;
 }
 
+AST_TEST_DEFINE(bucket_is_stale)
+{
+	RAII_VAR(struct ast_bucket *, bucket, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "bucket_is_stale";
+		info->category = "/main/bucket/";
+		info->summary = "bucket staleness unit test";
+		info->description =
+			"Test if staleness of a bucket is reported correctly";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (!(bucket = ast_bucket_alloc("test:///tmp/bob"))) {
+		ast_test_status_update(test, "Failed to allocate bucket\n");
+		return AST_TEST_FAIL;
+	}
+
+	bucket_test_wizard_clear();
+
+	ast_test_validate(test, ast_bucket_is_stale(bucket) == 0);
+
+	bucket_test_wizard_state.is_stale = 1;
+
+	ast_test_validate(test, ast_bucket_is_stale(bucket) == 1);
+
+	return AST_TEST_PASS;
+}
+
 AST_TEST_DEFINE(bucket_json)
 {
 	RAII_VAR(struct ast_bucket *, bucket, NULL, ao2_cleanup);
@@ -440,6 +524,52 @@ AST_TEST_DEFINE(bucket_file_create)
 	return AST_TEST_PASS;
 }
 
+AST_TEST_DEFINE(bucket_file_clone)
+{
+	RAII_VAR(struct ast_bucket_file *, file, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_bucket_file *, clone, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "bucket_file_clone";
+		info->category = "/main/bucket/";
+		info->summary = "file clone unit test";
+		info->description =
+			"Test cloning a file";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (!(file = ast_bucket_file_alloc("test:///tmp/bob"))) {
+		ast_test_status_update(test, "Failed to allocate file\n");
+		return AST_TEST_FAIL;
+	}
+
+	bucket_test_wizard_clear();
+
+	if (ast_bucket_file_create(file)) {
+		ast_test_status_update(test, "Failed to create file with URI '%s'\n",
+			ast_sorcery_object_get_id(file));
+		return AST_TEST_FAIL;
+	}
+	ast_bucket_file_metadata_set(file, "bob", "joe");
+
+	clone = ast_bucket_file_clone(file);
+	if (!clone) {
+		ast_test_status_update(test, "Failed to clone file with URI '%s'\n",
+			ast_sorcery_object_get_id(file));
+		return AST_TEST_FAIL;
+	}
+
+	ast_test_validate(test, strcmp(ast_sorcery_object_get_id(file), ast_sorcery_object_get_id(clone)) == 0);
+	ast_test_validate(test, file->scheme_impl == clone->scheme_impl);
+	ast_test_validate(test, strcmp(file->scheme, clone->scheme) == 0);
+	ast_test_validate(test, ao2_container_count(file->metadata) == ao2_container_count(clone->metadata));
+
+	return AST_TEST_PASS;
+}
+
 AST_TEST_DEFINE(bucket_file_copy)
 {
 	RAII_VAR(struct ast_bucket_file *, file, NULL, ao2_cleanup);
@@ -625,6 +755,38 @@ AST_TEST_DEFINE(bucket_file_delete)
 	return AST_TEST_PASS;
 }
 
+AST_TEST_DEFINE(bucket_file_is_stale)
+{
+	RAII_VAR(struct ast_bucket_file *, file, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "bucket_file_is_stale";
+		info->category = "/main/bucket/";
+		info->summary = "file staleness unit test";
+		info->description =
+			"Test if staleness of a bucket file is reported correctly";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (!(file = ast_bucket_file_alloc("test:///tmp/bob"))) {
+		ast_test_status_update(test, "Failed to allocate file\n");
+		return AST_TEST_FAIL;
+	}
+
+	bucket_test_wizard_clear();
+
+	ast_test_validate(test, ast_bucket_file_is_stale(file) == 0);
+
+	bucket_test_wizard_state.is_stale = 1;
+
+	ast_test_validate(test, ast_bucket_file_is_stale(file) == 1);
+
+	return AST_TEST_PASS;
+}
+
 AST_TEST_DEFINE(bucket_file_metadata_set)
 {
 	RAII_VAR(struct ast_bucket_file *, file, NULL, ao2_cleanup);
@@ -827,11 +989,13 @@ static int unload_module(void)
 	AST_TEST_UNREGISTER(bucket_scheme_register);
 	AST_TEST_UNREGISTER(bucket_alloc);
 	AST_TEST_UNREGISTER(bucket_create);
+	AST_TEST_UNREGISTER(bucket_clone);
 	AST_TEST_UNREGISTER(bucket_delete);
 	AST_TEST_UNREGISTER(bucket_retrieve);
 	AST_TEST_UNREGISTER(bucket_json);
 	AST_TEST_UNREGISTER(bucket_file_alloc);
 	AST_TEST_UNREGISTER(bucket_file_create);
+	AST_TEST_UNREGISTER(bucket_file_clone);
 	AST_TEST_UNREGISTER(bucket_file_copy);
 	AST_TEST_UNREGISTER(bucket_file_retrieve);
 	AST_TEST_UNREGISTER(bucket_file_update);
@@ -854,15 +1018,19 @@ static int load_module(void)
 	AST_TEST_REGISTER(bucket_scheme_register);
 	AST_TEST_REGISTER(bucket_alloc);
 	AST_TEST_REGISTER(bucket_create);
+	AST_TEST_REGISTER(bucket_clone);
 	AST_TEST_REGISTER(bucket_delete);
 	AST_TEST_REGISTER(bucket_retrieve);
+	AST_TEST_REGISTER(bucket_is_stale);
 	AST_TEST_REGISTER(bucket_json);
 	AST_TEST_REGISTER(bucket_file_alloc);
 	AST_TEST_REGISTER(bucket_file_create);
+	AST_TEST_REGISTER(bucket_file_clone);
 	AST_TEST_REGISTER(bucket_file_copy);
 	AST_TEST_REGISTER(bucket_file_retrieve);
 	AST_TEST_REGISTER(bucket_file_update);
 	AST_TEST_REGISTER(bucket_file_delete);
+	AST_TEST_REGISTER(bucket_file_is_stale);
 	AST_TEST_REGISTER(bucket_file_metadata_set);
 	AST_TEST_REGISTER(bucket_file_metadata_unset);
 	AST_TEST_REGISTER(bucket_file_metadata_get);
diff -pruN 1:13.23.1~dfsg-2/tests/test_callerid.c 1:16.1.1~dfsg-1/tests/test_callerid.c
--- 1:13.23.1~dfsg-2/tests/test_callerid.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_callerid.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 #include "asterisk.h"
 #include "asterisk/callerid.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_cdr.c 1:16.1.1~dfsg-1/tests/test_cdr.c
--- 1:13.23.1~dfsg-2/tests/test_cdr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_cdr.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_cel.c 1:16.1.1~dfsg-1/tests/test_cel.c
--- 1:13.23.1~dfsg-2/tests/test_cel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_cel.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <math.h>
 #include "asterisk/module.h"
 #include "asterisk/test.h"
@@ -1792,7 +1790,7 @@ static int append_expected_event(
 
 static void test_sub(struct ast_event *event)
 {
-	struct ast_event *event_dup = ao2_dup_event(event);
+	RAII_VAR(struct ast_event *, event_dup, ao2_dup_event(event), ao2_cleanup);
 	const char *chan_name;
 	SCOPED_MUTEX(mid_test_lock, &mid_test_sync_lock);
 
@@ -1839,8 +1837,8 @@ static int test_cel_init_cb(struct ast_t
 	ast_cel_set_config(cel_test_config);
 
 	/* init CEL event storage (degenerate hash table becomes a linked list) */
-	cel_received_events = ao2_container_alloc(1, NULL, NULL);
-	cel_expected_events = ao2_container_alloc(1, NULL, NULL);
+	cel_received_events = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
+	cel_expected_events = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, NULL);
 
 	/* start the CEL event callback */
 	if (ast_cel_backend_register(TEST_BACKEND_NAME, test_sub)) {
@@ -1857,11 +1855,15 @@ static int test_cel_init_cb(struct ast_t
  */
 static int test_cel_peer_strings_match(const char *str1, const char *str2)
 {
-	struct ao2_container *intersection = ast_str_container_alloc(11);
+	RAII_VAR(struct ao2_container *, intersection, ast_str_container_alloc(11), ao2_cleanup);
 	RAII_VAR(char *, str1_dup, ast_strdup(str1), ast_free);
 	RAII_VAR(char *, str2_dup, ast_strdup(str2), ast_free);
 	char *chan;
 
+	if (!intersection) {
+		return 1;
+	}
+
 	while ((chan = strsep(&str1_dup, ","))) {
 		ast_str_container_add(intersection, chan);
 	}
@@ -2142,6 +2144,10 @@ static int unload_module(void)
 
 	ast_channel_unregister(&test_cel_chan_tech);
 
+	ao2_cleanup(cel_expected_events);
+	cel_expected_events = NULL;
+	ao2_cleanup(cel_received_events);
+	cel_received_events = NULL;
 	ao2_cleanup(cel_test_config);
 	cel_test_config = NULL;
 
diff -pruN 1:13.23.1~dfsg-2/tests/test_channel.c 1:16.1.1~dfsg-1/tests/test_channel.c
--- 1:13.23.1~dfsg-2/tests/test_channel.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_channel.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,119 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Channel unit tests
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ *
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/module.h"
+#include "asterisk/test.h"
+#include "asterisk/channel.h"
+
+AST_TEST_DEFINE(set_fd_grow)
+{
+	struct ast_channel *mock_channel;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int pos;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "set_fd_grow";
+		info->category = "/main/channel/";
+		info->summary = "channel setting file descriptor with growth test";
+		info->description =
+			"Test that setting a file descriptor on a high position of a channel results in -1 set on any new positions";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	ast_channel_set_fd(mock_channel, AST_EXTENDED_FDS + 10, 1);
+	ast_test_validate_cleanup(test, ast_channel_fd_count(mock_channel) == AST_EXTENDED_FDS + 11, res, done);
+
+	for (pos = AST_EXTENDED_FDS; (pos < AST_EXTENDED_FDS + 10); pos++) {
+		ast_test_validate_cleanup(test, ast_channel_fd(mock_channel, pos) == -1, res, done);
+	}
+
+done:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(add_fd)
+{
+	struct ast_channel *mock_channel;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int pos;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "add_fd";
+		info->category = "/main/channel/";
+		info->summary = "channel adding file descriptor test";
+		info->description =
+			"Test that adding a file descriptor to a channel places it in the expected position";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pos = ast_channel_fd_add(mock_channel, 1);
+	ast_test_validate_cleanup(test, pos == AST_EXTENDED_FDS, res, done);
+
+	ast_channel_set_fd(mock_channel, pos, -1);
+	ast_test_validate_cleanup(test, ast_channel_fd(mock_channel, pos) == -1, res, done);
+
+done:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(set_fd_grow);
+	AST_TEST_UNREGISTER(add_fd);
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(set_fd_grow);
+	AST_TEST_REGISTER(add_fd);
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Channel Unit Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_channel_feature_hooks.c 1:16.1.1~dfsg-1/tests/test_channel_feature_hooks.c
--- 1:13.23.1~dfsg-2/tests/test_channel_feature_hooks.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_channel_feature_hooks.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/test.h"
 #include "asterisk/channel.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_config.c 1:16.1.1~dfsg-1/tests/test_config.c
--- 1:13.23.1~dfsg-2/tests/test_config.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_config.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include <math.h> /* HUGE_VAL */
 #include <sys/stat.h>
 
@@ -1366,12 +1364,6 @@ struct test_config {
 	struct ao2_container *items;
 };
 
-static int test_item_hash(const void *obj, const int flags)
-{
-	const struct test_item *item = obj;
-	const char *name = (flags & OBJ_KEY) ? obj : item->name;
-	return ast_str_case_hash(name);
-}
 static int test_item_cmp(void *obj, void *arg, int flags)
 {
 	struct test_item *one = obj, *two = arg;
@@ -1424,7 +1416,8 @@ static void *test_config_alloc(void)
 	if (!(cfg->global_defaults = test_item_alloc("global_defaults"))) {
 		goto error;
 	}
-	if (!(cfg->items = ao2_container_alloc(1, test_item_hash, test_item_cmp))) {
+	cfg->items = ao2_container_alloc_list(AO2_ALLOC_OPT_LOCK_MUTEX, 0, NULL, test_item_cmp);
+	if (!cfg->items) {
 		goto error;
 	}
 	return cfg;
diff -pruN 1:13.23.1~dfsg-2/tests/test_conversions.c 1:16.1.1~dfsg-1/tests/test_conversions.c
--- 1:13.23.1~dfsg-2/tests/test_conversions.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_conversions.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,136 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Kevin Harwell <kharwell@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Conversions Unit Tests
+ *
+ * \author Kevin Harwell <kharwell@digium.com>
+ *
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/conversions.h"
+
+#define CATEGORY "/main/conversions/"
+
+AST_TEST_DEFINE(str_to_uint)
+{
+	const char *invalid = "abc";
+	const char *invalid_partial = "7abc";
+	const char *negative = "-7";
+	const char *negative_spaces = "  -7";
+	const char *out_of_range = "9999999999";
+	const char *spaces = "  ";
+	const char *valid = "7";
+	const char *valid_spaces = "  7";
+	unsigned int val;
+	char str[64];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "convert a string to an unsigned integer";
+		info->description = info->summary;
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_test_validate(test, ast_str_to_uint(NULL, &val));
+	ast_test_validate(test, ast_str_to_uint(invalid, &val));
+	ast_test_validate(test, ast_str_to_uint(invalid_partial, &val));
+	ast_test_validate(test, ast_str_to_uint(negative, &val));
+	ast_test_validate(test, ast_str_to_uint(negative_spaces, &val));
+	ast_test_validate(test, ast_str_to_uint(out_of_range, &val));
+	ast_test_validate(test, ast_str_to_uint(spaces, &val));
+	ast_test_validate(test, !ast_str_to_uint(valid, &val));
+	ast_test_validate(test, !ast_str_to_uint(valid_spaces, &val));
+
+	ast_test_validate(test, snprintf(str, sizeof(str), "%u", UINT_MAX) > 0);
+	ast_test_validate(test, !ast_str_to_uint(str, &val));
+	ast_test_validate(test, val == UINT_MAX);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(str_to_ulong)
+{
+	const char *invalid = "abc";
+	const char *invalid_partial = "7abc";
+	const char *negative = "-7";
+	const char *negative_spaces = "  -7";
+	const char *out_of_range = "99999999999999999999";
+	const char *spaces = "  ";
+	const char *valid = "7";
+	const char *valid_spaces = "  7";
+	unsigned long val;
+	char str[64];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "convert a string to an unsigned long";
+		info->description = info->summary;
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_test_validate(test, ast_str_to_ulong(NULL, &val));
+	ast_test_validate(test, ast_str_to_ulong(invalid, &val));
+	ast_test_validate(test, ast_str_to_ulong(invalid_partial, &val));
+	ast_test_validate(test, ast_str_to_ulong(negative, &val));
+	ast_test_validate(test, ast_str_to_ulong(negative_spaces, &val));
+	ast_test_validate(test, ast_str_to_ulong(out_of_range, &val));
+	ast_test_validate(test, ast_str_to_ulong(spaces, &val));
+	ast_test_validate(test, !ast_str_to_ulong(valid, &val));
+	ast_test_validate(test, !ast_str_to_ulong(valid_spaces, &val));
+
+	ast_test_validate(test, snprintf(str, sizeof(str), "%lu", ULONG_MAX) > 0);
+	ast_test_validate(test, !ast_str_to_ulong(str, &val));
+	ast_test_validate(test, val == ULONG_MAX);
+
+	return AST_TEST_PASS;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(str_to_uint);
+	AST_TEST_REGISTER(str_to_ulong);
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(str_to_uint);
+	AST_TEST_UNREGISTER(str_to_ulong);
+	return 0;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "URI test module");
diff -pruN 1:13.23.1~dfsg-2/tests/test_core_codec.c 1:16.1.1~dfsg-1/tests/test_core_codec.c
--- 1:13.23.1~dfsg-2/tests/test_core_codec.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_core_codec.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_core_format.c 1:16.1.1~dfsg-1/tests/test_core_format.c
--- 1:13.23.1~dfsg-2/tests/test_core_format.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_core_format.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_data_buffer.c 1:16.1.1~dfsg-1/tests/test_data_buffer.c
--- 1:13.23.1~dfsg-2/tests/test_data_buffer.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_data_buffer.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,345 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2018, Digium, Inc.
+ *
+ * Ben Ford <bford@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Media Stream API Unit Tests
+ *
+ * \author Ben Ford <bford@digium.com>
+ *
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/module.h"
+#include "asterisk/test.h"
+#include "asterisk/data_buffer.h"
+
+#define BUFFER_MAX_NOMINAL 10
+
+struct mock_payload
+{
+	int id;
+};
+
+/* Ensures that RAII_VAR will not trip ast_assert(buffer != NULL) in the callback */
+static void ast_data_buffer_free_wrapper(struct ast_data_buffer *buffer)
+{
+	if (!buffer) {
+		return;
+	}
+
+	ast_data_buffer_free(buffer);
+}
+
+AST_TEST_DEFINE(buffer_create)
+{
+	RAII_VAR(struct ast_data_buffer *, buffer, NULL, ast_data_buffer_free_wrapper);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "buffer_create";
+		info->category = "/main/data_buffer/";
+		info->summary = "buffer create unit test";
+		info->description =
+			"Test that creating a data buffer results in a buffer with the expected values";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	buffer = ast_data_buffer_alloc(ast_free_ptr, BUFFER_MAX_NOMINAL);
+
+	ast_test_validate(test, buffer != NULL,
+			"Failed to create buffer with valid arguments");
+	ast_test_validate(test, ast_data_buffer_count(buffer) == 0,
+			"Newly created buffer does not have the expected payload count");
+	ast_test_validate(test, ast_data_buffer_max(buffer) == BUFFER_MAX_NOMINAL,
+			"Newly created buffer does not have the expected max size");
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(buffer_put)
+{
+	RAII_VAR(struct ast_data_buffer *, buffer, NULL, ast_data_buffer_free_wrapper);
+	struct mock_payload *payload;
+	struct mock_payload *fetched_payload;
+	int ret;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "buffer_put";
+		info->category = "/main/data_buffer/";
+		info->summary = "buffer put unit test";
+		info->description =
+			"Test that putting payloads in the buffer yields the expected results";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	buffer = ast_data_buffer_alloc(ast_free_ptr, 2);
+
+	ast_test_validate(test, buffer != NULL,
+			"Failed to create buffer with valid arguments");
+	ast_test_validate(test, ast_data_buffer_count(buffer) == 0,
+			"Newly created buffer is not empty");
+
+	payload = ast_calloc(1, sizeof(*payload));
+
+	ast_test_validate(test, payload != NULL,
+			"Failed to allocate memory for first payload");
+
+	payload->id = 2;
+	ret = ast_data_buffer_put(buffer, 2, payload);
+
+	ast_test_validate(test, ret == 0,
+			"Adding a payload to an empty buffer did not return the expected value");
+	ast_test_validate(test, ast_data_buffer_count(buffer) == 1,
+			"Adding a payload to an empty buffer did not update count to the expected value");
+
+	fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, 2);
+
+	ast_test_validate(test, fetched_payload != NULL,
+			"Failed to get only payload from buffer given valid arguments");
+
+	ast_data_buffer_put(buffer, 2, payload);
+
+	ast_test_validate(test, ast_data_buffer_count(buffer) == 1,
+			"Adding a payload that is already in the buffer should not do anything");
+
+	payload = ast_calloc(1, sizeof(*payload));
+
+	ast_test_validate(test, payload != NULL,
+			"Failed to allocate memory for second payload");
+
+	payload->id = 1;
+	ast_data_buffer_put(buffer, 1, payload);
+	fetched_payload = ast_data_buffer_get(buffer, 1);
+
+	ast_test_validate(test, fetched_payload != NULL,
+			"Failed to get a payload from buffer given valid arguments");
+	ast_test_validate(test, ast_data_buffer_count(buffer) == 2,
+			"Buffer does not have the expected count after removing a payload");
+	ast_test_validate(test, fetched_payload->id == 1,
+			"Did not get the expected payload from the buffer");
+
+	payload = ast_calloc(1, sizeof(*payload));
+
+	ast_test_validate(test, payload != NULL,
+			"Failed to allocate memory for third payload");
+
+	payload->id = 3;
+	ret = ast_data_buffer_put(buffer, 3, payload);
+
+	ast_test_validate(test, ret == 0,
+			"Failed to replace a payload in the buffer");
+	ast_test_validate(test, ast_data_buffer_count(buffer) <= 2,
+			"Buffer count exceeded the max");
+
+	fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, 3);
+
+	ast_test_validate(test, fetched_payload != NULL,
+			"Failed to get a payload from buffer at position 3 given valid arguments");
+	ast_test_validate(test, fetched_payload->id == 3,
+			"Did not get the expected payload at position 3 from the buffer");
+
+	fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, 2);
+
+	ast_test_validate(test, fetched_payload != NULL,
+			"Failed to get a payload from buffer at position 2 given valid arguments");
+	ast_test_validate(test, fetched_payload->id == 2,
+			"Did not get the expected payload at position 2 from the buffer");
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(buffer_resize)
+{
+	RAII_VAR(struct ast_data_buffer *, buffer, NULL, ast_data_buffer_free_wrapper);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "buffer_resize";
+		info->category = "/main/data_buffer/";
+		info->summary = "buffer resize unit test";
+		info->description =
+			"Tests resizing a data buffer to make sure it has the expected outcome";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	buffer = ast_data_buffer_alloc(ast_free_ptr, BUFFER_MAX_NOMINAL);
+
+	ast_test_validate(test, buffer != NULL,
+			"Failed to create buffer with valid arguments");
+
+	ast_data_buffer_resize(buffer, BUFFER_MAX_NOMINAL);
+
+	ast_test_validate(test, ast_data_buffer_max(buffer) == BUFFER_MAX_NOMINAL,
+			"Trying to resize buffer to same size should not change its max size");
+
+	ast_data_buffer_resize(buffer, BUFFER_MAX_NOMINAL + 2);
+
+	ast_test_validate(test, ast_data_buffer_max(buffer) == BUFFER_MAX_NOMINAL + 2,
+			"Increasing buffer size did not return the expected max");
+
+	ast_data_buffer_resize(buffer, 1);
+
+	ast_test_validate(test, ast_data_buffer_max(buffer) == 1,
+			"Decreasing buffer size did not return the expected max");
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(buffer_nominal)
+{
+	RAII_VAR(struct ast_data_buffer *, buffer, NULL, ast_data_buffer_free_wrapper);
+	RAII_VAR(struct mock_payload *, removed_payload, NULL, ast_free_ptr);
+	struct mock_payload *payload;
+	struct mock_payload *fetched_payload;
+	int ret;
+	int i;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "buffer_nominal";
+		info->category = "/main/data_buffer/";
+		info->summary = "buffer nominal unit test";
+		info->description =
+			"Tests the normal usage of a data buffer to ensure the expected payloads "
+			"are present after multiple insertions";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	buffer = ast_data_buffer_alloc(ast_free_ptr, BUFFER_MAX_NOMINAL);
+
+	ast_test_validate(test, buffer != NULL,
+			"Failed to create buffer with valid arguments");
+
+	for (i = 1; i <= BUFFER_MAX_NOMINAL; i++) {
+		payload = ast_calloc(1, sizeof(*payload));
+
+		ast_test_validate(test, payload != NULL,
+				"Failed to allocate memory for payload %d", i);
+
+		ret = ast_data_buffer_put(buffer, i, payload);
+		if (ret) {
+			ast_free(payload);
+		}
+
+		ast_test_validate(test, ret == 0,
+				"Failed to add payload %d to buffer", i);
+	}
+
+	ast_test_validate(test, ast_data_buffer_count(buffer) == BUFFER_MAX_NOMINAL,
+			"Buffer does not have the expected count after adding payloads");
+
+	for (i = 1; i <= BUFFER_MAX_NOMINAL; i++) {
+		fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, i);
+
+		ast_test_validate(test, fetched_payload != NULL,
+				"Failed to get payload at position %d during first loop", i);
+	}
+
+	for (i = 1; i <= BUFFER_MAX_NOMINAL; i++) {
+		payload = ast_calloc(1, sizeof(*payload));
+
+		ast_test_validate(test, payload != NULL,
+				"Failed to allocate memory for payload %d", i + BUFFER_MAX_NOMINAL);
+
+		payload->id = i;
+		ret = ast_data_buffer_put(buffer, i + BUFFER_MAX_NOMINAL, payload);
+		if (ret) {
+			ast_free(payload);
+		}
+
+		ast_test_validate(test, ret == 0,
+				"Failed to add payload %d to buffer", i + BUFFER_MAX_NOMINAL);
+	}
+
+	ast_test_validate(test, ast_data_buffer_count(buffer) == BUFFER_MAX_NOMINAL,
+			"Buffer does not have the expected count after replacing payloads");
+
+	for (i = 1; i <= BUFFER_MAX_NOMINAL; i++) {
+		fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, i);
+
+		ast_test_validate(test, fetched_payload == NULL,
+				"Got an unexpected payload at position %d", i);
+
+		fetched_payload = (struct mock_payload *)ast_data_buffer_get(buffer, i + BUFFER_MAX_NOMINAL);
+
+		ast_test_validate(test, fetched_payload != NULL,
+				"Failed to get payload at position %d during second loop", i + BUFFER_MAX_NOMINAL);
+	}
+
+	removed_payload = (struct mock_payload *)ast_data_buffer_remove_head(buffer);
+
+	ast_test_validate(test, removed_payload != NULL,
+			"Failed to get the payload at the HEAD of the buffer");
+
+	ast_test_validate(test, ast_data_buffer_count(buffer) == BUFFER_MAX_NOMINAL - 1,
+			"Removing payload from HEAD of buffer did not decrease buffer size");
+
+	ast_test_validate(test, removed_payload->id == 1,
+			"Removing payload from HEAD of buffer did not return expected payload");
+
+	ast_free(removed_payload);
+
+	removed_payload = (struct mock_payload *)ast_data_buffer_remove(buffer, BUFFER_MAX_NOMINAL * 2);
+
+	ast_test_validate(test, removed_payload != NULL,
+			"Failed to get payload at position %d from buffer", BUFFER_MAX_NOMINAL * 2);
+
+	ast_test_validate(test, ast_data_buffer_count(buffer) == BUFFER_MAX_NOMINAL - 2,
+			"Removing payload from buffer did not decrease buffer size");
+
+	ast_test_validate(test, removed_payload->id == BUFFER_MAX_NOMINAL,
+			"Removing payload from buffer did not return expected payload");
+
+	return AST_TEST_PASS;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(buffer_create);
+	AST_TEST_UNREGISTER(buffer_put);
+	AST_TEST_UNREGISTER(buffer_resize);
+	AST_TEST_UNREGISTER(buffer_nominal);
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(buffer_create);
+	AST_TEST_REGISTER(buffer_put);
+	AST_TEST_REGISTER(buffer_resize);
+	AST_TEST_REGISTER(buffer_nominal);
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Data buffer API test module");
diff -pruN 1:13.23.1~dfsg-2/tests/test_db.c 1:16.1.1~dfsg-1/tests/test_db.c
--- 1:13.23.1~dfsg-2/tests/test_db.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_db.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/astdb.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_devicestate.c 1:16.1.1~dfsg-1/tests/test_devicestate.c
--- 1:13.23.1~dfsg-2/tests/test_devicestate.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_devicestate.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_dlinklists.c 1:16.1.1~dfsg-1/tests/test_dlinklists.c
--- 1:13.23.1~dfsg-2/tests/test_dlinklists.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dlinklists.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
@@ -106,10 +104,10 @@ static void destroy_test_container(struc
 	struct test1 *t1;
 	AST_DLLIST_TRAVERSE_BACKWARDS_SAFE_BEGIN(&x->entries, t1, list) {
 		AST_DLLIST_REMOVE_CURRENT(list);
-		free(t1);
+		ast_free(t1);
 	}
 	AST_DLLIST_TRAVERSE_BACKWARDS_SAFE_END;
-	free(x);
+	ast_free(x);
 }
 
 /* Macros to test:
@@ -337,7 +335,7 @@ static void dll_tests(void)
 
 		if (e == b) {
 			AST_DLLIST_REMOVE_CURRENT(list);  /* C A */
-			free(b);
+			ast_free(b);
 			print_list(tc, "C <=> A");
 		}
 		if (e == a) {
diff -pruN 1:13.23.1~dfsg-2/tests/test_dns.c 1:16.1.1~dfsg-1/tests/test_dns.c
--- 1:13.23.1~dfsg-2/tests/test_dns.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dns.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,1337 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Mark Michelson
+ *
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+#include <arpa/inet.h>
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_internal.h"
+
+/* Used when a stub is needed for certain tests */
+static int stub_resolve(struct ast_dns_query *query)
+{
+	return 0;
+}
+
+/* Used when a stub is needed for certain tests */
+static int stub_cancel(struct ast_dns_query *query)
+{
+	return 0;
+}
+
+AST_TEST_DEFINE(resolver_register_unregister)
+{
+	struct ast_dns_resolver cool_guy_resolver = {
+		.name = "A snake that swallowed a deer",
+		.priority = 19890504,
+		.resolve = stub_resolve,
+		.cancel = stub_cancel,
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_register_unregister";
+		info->category = "/main/dns/";
+		info->summary = "Test nominal resolver registration and unregistration";
+		info->description =
+			"The test performs the following steps:\n"
+			"\t* Register a valid resolver.\n"
+			"\t* Unregister the resolver.\n"
+			"If either step fails, the test fails";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&cool_guy_resolver)) {
+		ast_test_status_update(test, "Unable to register a perfectly good resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_dns_resolver_unregister(&cool_guy_resolver);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(resolver_register_off_nominal)
+{
+	struct ast_dns_resolver valid = {
+		.name = "valid",
+		.resolve = stub_resolve,
+		.cancel = stub_cancel,
+	};
+
+	struct ast_dns_resolver incomplete1 = {
+		.name = NULL,
+		.resolve = stub_resolve,
+		.cancel = stub_cancel,
+	};
+
+	struct ast_dns_resolver incomplete2 = {
+		.name = "incomplete2",
+		.resolve = NULL,
+		.cancel = stub_cancel,
+	};
+
+	struct ast_dns_resolver incomplete3 = {
+		.name = "incomplete3",
+		.resolve = stub_resolve,
+		.cancel = NULL,
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_register_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test off-nominal resolver registration";
+		info->description =
+			"Test off-nominal resolver registration:\n"
+			"\t* Register a duplicate resolver\n"
+			"\t* Register a resolver without a name\n"
+			"\t* Register a resolver without a resolve() method\n"
+			"\t* Register a resolver without a cancel() method";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&valid)) {
+		ast_test_status_update(test, "Failed to register valid resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_register(&valid)) {
+		ast_test_status_update(test, "Successfully registered the same resolver multiple times\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_dns_resolver_unregister(&valid);
+
+	if (!ast_dns_resolver_register(NULL)) {
+		ast_test_status_update(test, "Successfully registered a NULL resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_register(&incomplete1)) {
+		ast_test_status_update(test, "Successfully registered a DNS resolver with no name\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_register(&incomplete2)) {
+		ast_test_status_update(test, "Successfully registered a DNS resolver with no resolve() method\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_register(&incomplete3)) {
+		ast_test_status_update(test, "Successfully registered a DNS resolver with no cancel() method\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(resolver_unregister_off_nominal)
+{
+	struct ast_dns_resolver non_existent = {
+		.name = "I do not exist",
+		.priority = 20141004,
+		.resolve = stub_resolve,
+		.cancel = stub_cancel,
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_unregister_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test off-nominal DNS resolver unregister";
+		info->description =
+			"The test attempts the following:\n"
+			"\t* Unregister a resolver that is not registered.\n"
+			"\t* Unregister a NULL pointer.\n"
+			"Because unregistering a resolver does not return an indicator of success, the best\n"
+			"this test can do is verify that nothing blows up when this is attempted.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_dns_resolver_unregister(&non_existent);
+	ast_dns_resolver_unregister(NULL);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(resolver_data)
+{
+	struct ast_dns_query some_query;
+
+	struct digits {
+		int fingers;
+		int toes;
+	};
+
+	RAII_VAR(struct digits *, average, NULL, ao2_cleanup);
+	RAII_VAR(struct digits *, polydactyl, NULL, ao2_cleanup);
+
+	struct digits *data_ptr;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_data";
+		info->category = "/main/dns/";
+		info->summary = "Test getting and setting data on a DNS resolver";
+		info->description = "This test does the following:\n"
+			"\t* Ensure that requesting resolver data results in a NULL return if no data has been set.\n"
+			"\t* Ensure that setting resolver data does not result in an error.\n"
+			"\t* Ensure that retrieving the set resolver data returns the data we expect\n"
+			"\t* Ensure that setting new resolver data on the query does not result in an error\n"
+			"\t* Ensure that retrieving the resolver data returns the new data that we set";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	memset(&some_query, 0, sizeof(some_query));
+
+	average = ao2_alloc(sizeof(*average), NULL);
+	polydactyl = ao2_alloc(sizeof(*average), NULL);
+
+	if (!average || !polydactyl) {
+		ast_test_status_update(test, "Allocation failure during unit test\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Ensure that NULL is retrieved if we haven't set anything on the query */
+	data_ptr = ast_dns_resolver_get_data(&some_query);
+	if (data_ptr) {
+		ast_test_status_update(test, "Retrieved non-NULL resolver data from query unexpectedly\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_dns_resolver_set_data(&some_query, average)) {
+		ast_test_status_update(test, "Failed to set resolver data on query\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Go ahead now and remove the query's reference to the resolver data to prevent memory leaks */
+	ao2_ref(average, -1);
+
+	/* Ensure that data can be set and retrieved */
+	data_ptr = ast_dns_resolver_get_data(&some_query);
+	if (!data_ptr) {
+		ast_test_status_update(test, "Unable to retrieve resolver data from DNS query\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (data_ptr != average) {
+		ast_test_status_update(test, "Unexpected resolver data retrieved from DNS query\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Ensure that attempting to set new resolver data on the query fails */
+	if (!ast_dns_resolver_set_data(&some_query, polydactyl)) {
+		ast_test_status_update(test, "Successfully overwrote resolver data on a query. We shouldn't be able to do that\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+static int test_results(struct ast_test *test, const struct ast_dns_query *query,
+		unsigned int expected_secure, unsigned int expected_bogus,
+		unsigned int expected_rcode, const char *expected_canonical,
+		const char *expected_answer, size_t answer_size)
+{
+	struct ast_dns_result *result;
+
+	result = ast_dns_query_get_result(query);
+	if (!result) {
+		ast_test_status_update(test, "Unable to retrieve result from query\n");
+		return -1;
+	}
+
+	if (ast_dns_result_get_secure(result) != expected_secure ||
+			ast_dns_result_get_bogus(result) != expected_bogus ||
+			ast_dns_result_get_rcode(result) != expected_rcode ||
+			strcmp(ast_dns_result_get_canonical(result), expected_canonical) ||
+			memcmp(ast_dns_result_get_answer(result), expected_answer, answer_size)) {
+		ast_test_status_update(test, "Unexpected values in result from query\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+/* When setting a DNS result, we have to provide the raw DNS answer. This
+ * is not happening. Sorry. Instead, we provide a dummy string and call it
+ * a day
+ */
+#define DNS_ANSWER "Grumble Grumble"
+#define DNS_ANSWER_SIZE strlen(DNS_ANSWER)
+
+AST_TEST_DEFINE(resolver_set_result)
+{
+	struct ast_dns_query some_query;
+	struct ast_dns_result *result;
+
+	struct dns_result {
+		unsigned int secure;
+		unsigned int bogus;
+		unsigned int rcode;
+	} results[] = {
+		{ 0, 0, NOERROR, },
+		{ 0, 1, NOERROR, },
+		{ 1, 0, NOERROR, },
+		{ 0, 0, NXDOMAIN, },
+	};
+	int i;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_set_result";
+		info->category = "/main/dns/";
+		info->summary = "Test setting and getting results on DNS queries";
+		info->description =
+			"This test performs the following:\n"
+			"\t* Sets a result that is not secure, bogus, and has rcode 0\n"
+			"\t* Sets a result that is not secure, has rcode 0, but is secure\n"
+			"\t* Sets a result that is not bogus, has rcode 0, but is secure\n"
+			"\t* Sets a result that is not secure or bogus, but has rcode NXDOMAIN\n"
+			"After each result is set, we ensure that parameters retrieved from\n"
+			"the result have the expected values.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	memset(&some_query, 0, sizeof(some_query));
+
+	for (i = 0; i < ARRAY_LEN(results); ++i) {
+		if (ast_dns_resolver_set_result(&some_query, results[i].secure, results[i].bogus,
+				results[i].rcode, "asterisk.org", DNS_ANSWER, DNS_ANSWER_SIZE)) {
+			ast_test_status_update(test, "Unable to add DNS result to query\n");
+			res = AST_TEST_FAIL;
+		}
+
+		if (test_results(test, &some_query, results[i].secure, results[i].bogus,
+				results[i].rcode, "asterisk.org", DNS_ANSWER, DNS_ANSWER_SIZE)) {
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	/* The final result we set needs to be freed */
+	result = ast_dns_query_get_result(&some_query);
+	ast_dns_result_free(result);
+
+	return res;
+}
+
+AST_TEST_DEFINE(resolver_set_result_off_nominal)
+{
+	struct ast_dns_query some_query;
+	struct ast_dns_result *result;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_set_result_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test setting off-nominal DNS results";
+		info->description =
+			"This test performs the following:\n"
+			"\t* Attempt to add a DNS result that is both bogus and secure\n"
+			"\t* Attempt to add a DNS result that has no canonical name";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	memset(&some_query, 0, sizeof(some_query));
+
+	if (!ast_dns_resolver_set_result(&some_query, 1, 1, NOERROR, "asterisk.org",
+				DNS_ANSWER, DNS_ANSWER_SIZE)) {
+		ast_test_status_update(test, "Successfully added a result that was both secure and bogus\n");
+		result = ast_dns_query_get_result(&some_query);
+		ast_dns_result_free(result);
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_set_result(&some_query, 0, 0, NOERROR, NULL,
+				DNS_ANSWER, DNS_ANSWER_SIZE)) {
+		ast_test_status_update(test, "Successfully added result with no canonical name\n");
+		result = ast_dns_query_get_result(&some_query);
+		ast_dns_result_free(result);
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+static int test_record(struct ast_test *test, const struct ast_dns_record *record,
+		int rr_type, int rr_class, int ttl, const char *data, const size_t size)
+{
+	if (ast_dns_record_get_rr_type(record) != rr_type) {
+		ast_test_status_update(test, "Unexpected rr_type from DNS record\n");
+		return -1;
+	}
+
+	if (ast_dns_record_get_rr_class(record) != rr_class) {
+		ast_test_status_update(test, "Unexpected rr_class from DNS record\n");
+		return -1;
+	}
+
+	if (ast_dns_record_get_ttl(record) != ttl) {
+		ast_test_status_update(test, "Unexpected ttl from DNS record\n");
+		return -1;
+	}
+
+	if (memcmp(ast_dns_record_get_data(record), data, size)) {
+		ast_test_status_update(test, "Unexpected data in DNS record\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+AST_TEST_DEFINE(resolver_add_record)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	struct ast_dns_query some_query;
+	const struct ast_dns_record *record;
+
+	static const char *V4 = "127.0.0.1";
+	static const size_t V4_BUFSIZE = sizeof(struct in_addr);
+	char v4_buf[V4_BUFSIZE];
+
+	static const char *V6 = "::1";
+	static const size_t V6_BUFSIZE = sizeof(struct in6_addr);
+	char v6_buf[V6_BUFSIZE];
+
+	struct dns_record_details {
+		int type;
+		int class;
+		int ttl;
+		const char *data;
+		const size_t size;
+		int visited;
+	} records[] = {
+		{ T_A, C_IN, 12345, v4_buf, V4_BUFSIZE, 0, },
+		{ T_AAAA, C_IN, 12345, v6_buf, V6_BUFSIZE, 0, },
+	};
+
+	int num_records_visited = 0;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_add_record";
+		info->category = "/main/dns/";
+		info->summary = "Test adding DNS records to a query";
+		info->description =
+			"This test performs the following:\n"
+			"\t* Ensure a nominal A record can be added to a query result\n"
+			"\t* Ensures that the record can be retrieved\n"
+			"\t* Ensure that a second record can be added to the query result\n"
+			"\t* Ensures that both records can be retrieved";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	memset(&some_query, 0, sizeof(some_query));
+
+	if (ast_dns_resolver_set_result(&some_query, 0, 0, NOERROR, "asterisk.org",
+				DNS_ANSWER, DNS_ANSWER_SIZE)) {
+		ast_test_status_update(test, "Unable to set result for DNS query\n");
+		return AST_TEST_FAIL;
+	}
+
+	result = ast_dns_query_get_result(&some_query);
+	if (!result) {
+		ast_test_status_update(test, "Unable to retrieve result from query\n");
+		return AST_TEST_FAIL;
+	}
+
+	inet_pton(AF_INET, V4, v4_buf);
+
+	/* Nominal Record */
+	if (ast_dns_resolver_add_record(&some_query, records[0].type, records[0].class,
+				records[0].ttl, records[0].data, records[0].size)) {
+		ast_test_status_update(test, "Unable to add nominal record to query result\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* I should only be able to retrieve one record */
+	record = ast_dns_result_get_records(result);
+	if (!record) {
+		ast_test_status_update(test, "Unable to retrieve record from result\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (test_record(test, record, records[0].type, records[0].class, records[0].ttl,
+				records[0].data, records[0].size)) {
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_dns_record_get_next(record)) {
+		ast_test_status_update(test, "Multiple records returned when only one was expected\n");
+		return AST_TEST_FAIL;
+	}
+
+	inet_pton(AF_INET6, V6, v6_buf);
+
+	if (ast_dns_resolver_add_record(&some_query, records[1].type, records[1].class,
+				records[1].ttl, records[1].data, records[1].size)) {
+		ast_test_status_update(test, "Unable to add second record to query result\n");
+		return AST_TEST_FAIL;
+	}
+
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		int res;
+
+		/* The order of returned records is not specified by the API. We use the record type
+		 * as the discriminator to determine which record data to expect.
+		 */
+		if (ast_dns_record_get_rr_type(record) == records[0].type) {
+			res = test_record(test, record, records[0].type, records[0].class, records[0].ttl, records[0].data, records[0].size);
+			records[0].visited = 1;
+		} else if (ast_dns_record_get_rr_type(record) == records[1].type) {
+			res = test_record(test, record, records[1].type, records[1].class, records[1].ttl, records[1].data, records[1].size);
+			records[1].visited = 1;
+		} else {
+			ast_test_status_update(test, "Unknown record type found in DNS results\n");
+			return AST_TEST_FAIL;
+		}
+
+		if (res) {
+			return AST_TEST_FAIL;
+		}
+
+		++num_records_visited;
+	}
+
+	if (!records[0].visited || !records[1].visited) {
+		ast_test_status_update(test, "Did not visit all added DNS records\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (num_records_visited != ARRAY_LEN(records)) {
+		ast_test_status_update(test, "Did not visit the expected number of DNS records\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(resolver_add_record_off_nominal)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	struct ast_dns_query some_query;
+	static const char *V4 = "127.0.0.1";
+	static const size_t V4_BUFSIZE = sizeof(struct in_addr);
+	char v4_buf[V4_BUFSIZE];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_add_record_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test adding off-nominal DNS records to a query";
+		info->description =
+			"This test performs the following:\n"
+			"\t* Ensure a nominal A record cannot be added if no result has been set.\n"
+			"\t* Ensure that an A record with invalid RR types cannot be added to a query\n"
+			"\t* Ensure that an A record with invalid RR classes cannot be added to a query\n"
+			"\t* Ensure that an A record with invalid TTL cannot be added to a query\n"
+			"\t* Ensure that an A record with NULL data cannot be added to a query\n"
+			"\t* Ensure that an A record with invalid length cannot be added to a query";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	memset(&some_query, 0, sizeof(some_query));
+
+	inet_ntop(AF_INET, V4, v4_buf, V4_BUFSIZE);
+
+	/* Add record before setting result */
+	if (!ast_dns_resolver_add_record(&some_query, T_A, C_IN, 12345, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record to query before setting a result\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_dns_resolver_set_result(&some_query, 0, 0, NOERROR, "asterisk.org",
+				DNS_ANSWER, DNS_ANSWER_SIZE)) {
+		ast_test_status_update(test, "Unable to set result for DNS query\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* We get the result so it will be cleaned up when the function exits */
+	result = ast_dns_query_get_result(&some_query);
+
+	/* Invalid RR types */
+	if (!ast_dns_resolver_add_record(&some_query, -1, C_IN, 12345, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record with negative RR type\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_add_record(&some_query, 65536 + 1, C_IN, 12345, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record with too large RR type\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Invalid RR classes */
+	if (!ast_dns_resolver_add_record(&some_query, T_A, -1, 12345, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record with negative RR class\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolver_add_record(&some_query, T_A, 65536 + 1, 12345, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record with too large RR class\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Invalid TTL */
+	if (!ast_dns_resolver_add_record(&some_query, T_A, C_IN, -1, v4_buf, V4_BUFSIZE)) {
+		ast_test_status_update(test, "Successfully added DNS record with negative TTL\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* No data */
+	if (!ast_dns_resolver_add_record(&some_query, T_A, C_IN, 12345, NULL, 0)) {
+		ast_test_status_update(test, "Successfully added a DNS record with no data\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Lie about the length */
+	if (!ast_dns_resolver_add_record(&some_query, T_A, C_IN, 12345, v4_buf, 0)) {
+		ast_test_status_update(test, "Successfully added a DNS record with length zero\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+/*!
+ * \brief File-scoped data used during resolver tests
+ *
+ * This data has to live at file-scope since it needs to be
+ * accessible by multiple threads.
+ */
+static struct resolver_data {
+	/*! True if the resolver's resolve() method has been called */
+	int resolve_called;
+	/*! True if the resolver's cancel() method has been called */
+	int canceled;
+	/*! True if resolution successfully completed. This is mutually exclusive with \ref canceled */
+	int resolution_complete;
+	/*! Lock used for protecting \ref cancel_cond */
+	ast_mutex_t lock;
+	/*! Condition variable used to coordinate canceling a query */
+	ast_cond_t cancel_cond;
+} test_resolver_data;
+
+/*!
+ * \brief Thread spawned by the mock resolver
+ *
+ * All DNS resolvers are required to be asynchronous. The mock resolver
+ * spawns this thread for every DNS query that is executed.
+ *
+ * This thread waits for 5 seconds and then returns the same A record
+ * every time. The 5 second wait is to allow for the query to be
+ * canceled if desired
+ *
+ * \param dns_query The ast_dns_query that is being resolved
+ * \return NULL
+ */
+static void *resolution_thread(void *dns_query)
+{
+	struct ast_dns_query *query = dns_query;
+	struct timespec timeout;
+
+	static const char *V4 = "127.0.0.1";
+	static const size_t V4_BUFSIZE = sizeof(struct in_addr);
+	char v4_buf[V4_BUFSIZE];
+
+	timeout = ast_tsnow();
+	timeout.tv_sec += 5;
+
+	ast_mutex_lock(&test_resolver_data.lock);
+	while (!test_resolver_data.canceled) {
+		if (ast_cond_timedwait(&test_resolver_data.cancel_cond, &test_resolver_data.lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&test_resolver_data.lock);
+
+	if (test_resolver_data.canceled) {
+		ast_dns_resolver_completed(query);
+		ao2_ref(query, -1);
+		return NULL;
+	}
+
+	ast_dns_resolver_set_result(query, 0, 0, NOERROR, "asterisk.org", DNS_ANSWER, DNS_ANSWER_SIZE);
+
+	inet_pton(AF_INET, V4, v4_buf);
+	ast_dns_resolver_add_record(query, T_A, C_IN, 12345, v4_buf, V4_BUFSIZE);
+
+	test_resolver_data.resolution_complete = 1;
+	ast_dns_resolver_completed(query);
+
+	ao2_ref(query, -1);
+	return NULL;
+}
+
+/*!
+ * \brief Mock resolver's resolve method
+ *
+ * \param query The query to resolve
+ * \retval 0 Successfully spawned resolution thread
+ * \retval non-zero Failed to spawn the resolution thread
+ */
+static int test_resolve(struct ast_dns_query *query)
+{
+	pthread_t resolver_thread;
+
+	test_resolver_data.resolve_called = 1;
+	return ast_pthread_create_detached(&resolver_thread, NULL, resolution_thread, ao2_bump(query));
+}
+
+/*!
+ * \brief Mock resolver's cancel method
+ *
+ * This signals the resolution thread not to return any DNS results.
+ *
+ * \param query DNS query to cancel
+ * \return 0
+ */
+static int test_cancel(struct ast_dns_query *query)
+{
+	ast_mutex_lock(&test_resolver_data.lock);
+	test_resolver_data.canceled = 1;
+	ast_cond_signal(&test_resolver_data.cancel_cond);
+	ast_mutex_unlock(&test_resolver_data.lock);
+
+	return 0;
+}
+
+/*!
+ * \brief Initialize global mock resolver data.
+ *
+ * This must be called at the beginning of tests that use the mock resolver
+ */
+static void resolver_data_init(void)
+{
+	test_resolver_data.resolve_called = 0;
+	test_resolver_data.canceled = 0;
+	test_resolver_data.resolution_complete = 0;
+
+	ast_mutex_init(&test_resolver_data.lock);
+	ast_cond_init(&test_resolver_data.cancel_cond, NULL);
+}
+
+/*!
+ * \brief Cleanup global mock resolver data
+ *
+ * This must be called at the end of tests that use the mock resolver
+ */
+static void resolver_data_cleanup(void)
+{
+	ast_mutex_destroy(&test_resolver_data.lock);
+	ast_cond_destroy(&test_resolver_data.cancel_cond);
+}
+
+/*!
+ * \brief The mock resolver
+ *
+ * The mock resolver does not care about the DNS query that is
+ * actually being made on it. It simply regurgitates the same
+ * DNS record no matter what.
+ */
+static struct ast_dns_resolver test_resolver = {
+	.name = "test",
+	.priority = 0,
+	.resolve = test_resolve,
+	.cancel = test_cancel,
+};
+
+AST_TEST_DEFINE(resolver_resolve_sync)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_resolve_sync";
+		info->category = "/main/dns/";
+		info->summary = "Test a nominal synchronous DNS resolution";
+		info->description =
+			"This test performs a synchronous DNS resolution of a domain. The goal of this\n"
+			"test is not to check the records for accuracy. Rather, the goal is to ensure that\n"
+			"the resolver is called into as expected, that the query completes entirely before\n"
+			"returning from the synchronous resolution, that nothing tried to cancel the resolution\n,"
+			"and that some records were returned.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&test_resolver)) {
+		ast_test_status_update(test, "Unable to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	resolver_data_init();
+
+	if (ast_dns_resolve("asterisk.org", T_A, C_IN, &result)) {
+		ast_test_status_update(test, "Resolution of address failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "DNS resolution returned a NULL result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!test_resolver_data.resolve_called) {
+		ast_test_status_update(test, "DNS resolution did not call resolver's resolve() method\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (test_resolver_data.canceled) {
+		ast_test_status_update(test, "Resolver's cancel() method called for no reason\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!test_resolver_data.resolution_complete) {
+		ast_test_status_update(test, "Synchronous resolution completed early?\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!ast_dns_result_get_records(result)) {
+		ast_test_status_update(test, "Synchronous resolution yielded no records.\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&test_resolver);
+	resolver_data_cleanup();
+	return res;
+}
+
+/*!
+ * \brief A resolve() method that simply fails
+ *
+ * \param query The DNS query to resolve. This is ignored.
+ * \return -1
+ */
+static int fail_resolve(struct ast_dns_query *query)
+{
+	return -1;
+}
+
+AST_TEST_DEFINE(resolver_resolve_sync_off_nominal)
+{
+	struct ast_dns_resolver terrible_resolver = {
+		.name = "Uwe Boll's Filmography",
+		.priority = 0,
+		.resolve = fail_resolve,
+		.cancel = stub_cancel,
+	};
+
+	struct ast_dns_result *result = NULL;
+
+	struct dns_resolve_data {
+		const char *name;
+		int rr_type;
+		int rr_class;
+		struct ast_dns_result **result;
+	} resolves [] = {
+		{ NULL,           T_A,       C_IN,      &result },
+		{ "asterisk.org", -1,        C_IN,      &result },
+		{ "asterisk.org", 65536 + 1, C_IN,      &result },
+		{ "asterisk.org", T_A,       -1,        &result },
+		{ "asterisk.org", T_A,       65536 + 1, &result },
+		{ "asterisk.org", T_A,       C_IN,      NULL },
+	};
+
+	int i;
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_resolve_sync_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test off-nominal synchronous DNS resolution";
+		info->description =
+			"This test performs several off-nominal synchronous DNS resolutions:\n"
+			"\t* Attempt resolution with NULL name\n"
+			"\t* Attempt resolution with invalid RR type\n"
+			"\t* Attempt resolution with invalid RR class\n"
+			"\t* Attempt resolution with NULL result pointer\n"
+			"\t* Attempt resolution with resolver that returns an error";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&test_resolver)) {
+		ast_test_status_update(test, "Failed to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	for (i = 0; i < ARRAY_LEN(resolves); ++i) {
+		if (!ast_dns_resolve(resolves[i].name, resolves[i].rr_type, resolves[i].rr_class, resolves[i].result)) {
+			ast_test_status_update(test, "Successfully resolved DNS query with invalid parameters\n");
+			res = AST_TEST_FAIL;
+		} else if (result) {
+			ast_test_status_update(test, "Failed resolution set a non-NULL result\n");
+			ast_dns_result_free(result);
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	ast_dns_resolver_unregister(&test_resolver);
+
+	/* As a final test, try a legitimate query with a bad resolver */
+	if (ast_dns_resolver_register(&terrible_resolver)) {
+		ast_test_status_update(test, "Failed to register the terrible resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!ast_dns_resolve("asterisk.org", T_A, C_IN, &result)) {
+		ast_test_status_update(test, "DNS resolution succeeded when we expected it not to\n");
+		ast_dns_resolver_unregister(&terrible_resolver);
+		return AST_TEST_FAIL;
+	}
+
+	ast_dns_resolver_unregister(&terrible_resolver);
+
+	if (result) {
+		ast_test_status_update(test, "Failed DNS resolution set the result to something non-NULL\n");
+		ast_dns_result_free(result);
+		return AST_TEST_FAIL;
+	}
+
+	return res;
+}
+
+/*!
+ * \brief Data used by async result callback
+ *
+ * This is the typical combination of boolean, lock, and condition
+ * used to synchronize the activities of two threads. In this case,
+ * the testing thread waits on the condition, and the async callback
+ * signals the condition when the asynchronous callback is complete.
+ */
+struct async_resolution_data {
+	int complete;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+/*!
+ * \brief Destructor for async_resolution_data
+ */
+static void async_data_destructor(void *obj)
+{
+	struct async_resolution_data *async_data = obj;
+
+	ast_mutex_destroy(&async_data->lock);
+	ast_cond_destroy(&async_data->cond);
+}
+
+/*!
+ * \brief Allocation/initialization for async_resolution_data
+ *
+ * The DNS core mandates that a query's user data has to be ao2 allocated,
+ * so this is a helper method for doing that.
+ *
+ * \retval NULL Failed allocation
+ * \retval non-NULL Newly allocated async_resolution_data
+ */
+static struct async_resolution_data *async_data_alloc(void)
+{
+	struct async_resolution_data *async_data;
+
+	async_data = ao2_alloc(sizeof(*async_data), async_data_destructor);
+	if (!async_data) {
+		return NULL;
+	}
+
+	async_data->complete = 0;
+	ast_mutex_init(&async_data->lock);
+	ast_cond_init(&async_data->cond, NULL);
+
+	return async_data;
+}
+
+/*!
+ * \brief Async DNS callback
+ *
+ * This is called when an async query completes, either because it resolved or
+ * because it was canceled. In our case, this callback is used to signal to the
+ * test that it can continue
+ *
+ * \param query The DNS query that has completed
+ */
+static void async_callback(const struct ast_dns_query *query)
+{
+	struct async_resolution_data *async_data = ast_dns_query_get_data(query);
+
+	ast_mutex_lock(&async_data->lock);
+	async_data->complete = 1;
+	ast_cond_signal(&async_data->cond);
+	ast_mutex_unlock(&async_data->lock);
+}
+
+AST_TEST_DEFINE(resolver_resolve_async)
+{
+	RAII_VAR(struct async_resolution_data *, async_data, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_dns_query_active *, active, NULL, ao2_cleanup);
+	struct ast_dns_result *result;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	struct timespec timeout;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_resolve_async";
+		info->category = "/main/dns/";
+		info->summary = "Test a nominal asynchronous DNS resolution";
+		info->description =
+			"This test performs an asynchronous DNS resolution of a domain. The goal of this\n"
+			"test is not to check the records for accuracy. Rather, the goal is to ensure that\n"
+			"the resolver is called into as expected, that we regain control before the query\n"
+			"is completed, and to ensure that nothing tried to cancel the resolution.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&test_resolver)) {
+		ast_test_status_update(test, "Unable to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	resolver_data_init();
+
+	async_data = async_data_alloc();
+	if (!async_data) {
+		ast_test_status_update(test, "Failed to allocate asynchronous data\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	active = ast_dns_resolve_async("asterisk.org", T_A, C_IN, async_callback, async_data);
+	if (!active) {
+		ast_test_status_update(test, "Asynchronous resolution of address failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!test_resolver_data.resolve_called) {
+		ast_test_status_update(test, "DNS resolution did not call resolver's resolve() method\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (test_resolver_data.canceled) {
+		ast_test_status_update(test, "Resolver's cancel() method called for no reason\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	timeout = ast_tsnow();
+	timeout.tv_sec += 10;
+	ast_mutex_lock(&async_data->lock);
+	while (!async_data->complete) {
+		if (ast_cond_timedwait(&async_data->cond, &async_data->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&async_data->lock);
+
+	if (!async_data->complete) {
+		ast_test_status_update(test, "Asynchronous resolution timed out\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!test_resolver_data.resolution_complete) {
+		ast_test_status_update(test, "Asynchronous resolution completed early?\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	result = ast_dns_query_get_result(active->query);
+	if (!result) {
+		ast_test_status_update(test, "Asynchronous resolution yielded no result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!ast_dns_result_get_records(result)) {
+		ast_test_status_update(test, "Asynchronous result had no records\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&test_resolver);
+	resolver_data_cleanup();
+	return res;
+}
+
+/*! Stub async resolution callback */
+static void stub_callback(const struct ast_dns_query *query)
+{
+	return;
+}
+
+AST_TEST_DEFINE(resolver_resolve_async_off_nominal)
+{
+	struct ast_dns_resolver terrible_resolver = {
+		.name = "Ed Wood's Filmography",
+		.priority = 0,
+		.resolve = fail_resolve,
+		.cancel = stub_cancel,
+	};
+
+	struct dns_resolve_data {
+		const char *name;
+		int rr_type;
+		int rr_class;
+		ast_dns_resolve_callback callback;
+	} resolves [] = {
+		{ NULL,           T_A,       C_IN,      stub_callback },
+		{ "asterisk.org", -1,        C_IN,      stub_callback },
+		{ "asterisk.org", 65536 + 1, C_IN,      stub_callback },
+		{ "asterisk.org", T_A,       -1,        stub_callback },
+		{ "asterisk.org", T_A,       65536 + 1, stub_callback },
+		{ "asterisk.org", T_A,       C_IN,      NULL },
+	};
+
+	struct ast_dns_query_active *active;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int i;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_resolve_async_off_nominal";
+		info->category = "/main/dns/";
+		info->summary = "Test off-nominal asynchronous DNS resolution";
+		info->description =
+			"This test performs several off-nominal asynchronous DNS resolutions:\n"
+			"\t* Attempt resolution with NULL name\n"
+			"\t* Attempt resolution with invalid RR type\n"
+			"\t* Attempt resolution with invalid RR class\n"
+			"\t* Attempt resolution with NULL callback pointer\n"
+			"\t* Attempt resolution with resolver that returns an error";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&test_resolver)) {
+		ast_test_status_update(test, "Failed to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	for (i = 0; i < ARRAY_LEN(resolves); ++i) {
+		active = ast_dns_resolve_async(resolves[i].name, resolves[i].rr_type, resolves[i].rr_class,
+				resolves[i].callback, NULL);
+		if (active) {
+			ast_test_status_update(test, "Successfully performed asynchronous resolution with invalid data\n");
+			ao2_ref(active, -1);
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	ast_dns_resolver_unregister(&test_resolver);
+
+	if (ast_dns_resolver_register(&terrible_resolver)) {
+		ast_test_status_update(test, "Failed to register the DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	active = ast_dns_resolve_async("asterisk.org", T_A, C_IN, stub_callback, NULL);
+
+	ast_dns_resolver_unregister(&terrible_resolver);
+
+	if (active) {
+		ast_test_status_update(test, "Successfully performed asynchronous resolution with invalid data\n");
+		ao2_ref(active, -1);
+		return AST_TEST_FAIL;
+	}
+
+	return res;
+}
+
+AST_TEST_DEFINE(resolver_resolve_async_cancel)
+{
+	RAII_VAR(struct async_resolution_data *, async_data, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_dns_query_active *, active, NULL, ao2_cleanup);
+	struct ast_dns_result *result;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	struct timespec timeout;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "resolver_resolve_async_cancel";
+		info->category = "/main/dns/";
+		info->summary = "Test canceling an asynchronous DNS resolution";
+		info->description =
+			"This test performs an asynchronous DNS resolution of a domain and then cancels\n"
+			"the resolution. The goal of this test is to ensure that the cancel() callback of\n"
+			"the resolver is called and that it properly interrupts the resolution such that no\n"
+			"records are returned.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&test_resolver)) {
+		ast_test_status_update(test, "Unable to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	resolver_data_init();
+
+	async_data = async_data_alloc();
+	if (!async_data) {
+		ast_test_status_update(test, "Failed to allocate asynchronous data\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	active = ast_dns_resolve_async("asterisk.org", T_A, C_IN, async_callback, async_data);
+	if (!active) {
+		ast_test_status_update(test, "Asynchronous resolution of address failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!test_resolver_data.resolve_called) {
+		ast_test_status_update(test, "DNS resolution did not call resolver's resolve() method\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (test_resolver_data.canceled) {
+		ast_test_status_update(test, "Resolver's cancel() method called for no reason\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	ast_dns_resolve_cancel(active);
+
+	if (!test_resolver_data.canceled) {
+		ast_test_status_update(test, "Resolver's cancel() method was not called\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	timeout = ast_tsnow();
+	timeout.tv_sec += 10;
+	ast_mutex_lock(&async_data->lock);
+	while (!async_data->complete) {
+		if (ast_cond_timedwait(&async_data->cond, &async_data->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&async_data->lock);
+
+	if (!async_data->complete) {
+		ast_test_status_update(test, "Asynchronous resolution timed out\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (test_resolver_data.resolution_complete) {
+		ast_test_status_update(test, "Resolution completed without cancelation\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	result = ast_dns_query_get_result(active->query);
+	if (result) {
+		ast_test_status_update(test, "Canceled resolution had a result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&test_resolver);
+	resolver_data_cleanup();
+	return res;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(resolver_register_unregister);
+	AST_TEST_UNREGISTER(resolver_register_off_nominal);
+	AST_TEST_UNREGISTER(resolver_unregister_off_nominal);
+	AST_TEST_UNREGISTER(resolver_data);
+	AST_TEST_UNREGISTER(resolver_set_result);
+	AST_TEST_UNREGISTER(resolver_set_result_off_nominal);
+	AST_TEST_UNREGISTER(resolver_add_record);
+	AST_TEST_UNREGISTER(resolver_add_record_off_nominal);
+	AST_TEST_UNREGISTER(resolver_resolve_sync);
+	AST_TEST_UNREGISTER(resolver_resolve_sync_off_nominal);
+	AST_TEST_UNREGISTER(resolver_resolve_async);
+	AST_TEST_UNREGISTER(resolver_resolve_async_off_nominal);
+	AST_TEST_UNREGISTER(resolver_resolve_async_cancel);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(resolver_register_unregister);
+	AST_TEST_REGISTER(resolver_register_off_nominal);
+	AST_TEST_REGISTER(resolver_unregister_off_nominal);
+	AST_TEST_REGISTER(resolver_data);
+	AST_TEST_REGISTER(resolver_set_result);
+	AST_TEST_REGISTER(resolver_set_result_off_nominal);
+	AST_TEST_REGISTER(resolver_add_record);
+	AST_TEST_REGISTER(resolver_add_record_off_nominal);
+	AST_TEST_REGISTER(resolver_resolve_sync);
+	AST_TEST_REGISTER(resolver_resolve_sync_off_nominal);
+	AST_TEST_REGISTER(resolver_resolve_async);
+	AST_TEST_REGISTER(resolver_resolve_async_off_nominal);
+	AST_TEST_REGISTER(resolver_resolve_async_cancel);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "DNS API Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_dns_naptr.c 1:16.1.1~dfsg-1/tests/test_dns_naptr.c
--- 1:13.23.1~dfsg-2/tests/test_dns_naptr.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dns_naptr.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,502 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Mark Michelson
+ *
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_naptr.h"
+#include "asterisk/dns_test.h"
+
+struct naptr_record {
+	uint16_t order;
+	uint16_t preference;
+	struct ast_dns_test_string flags;
+	struct ast_dns_test_string services;
+	struct ast_dns_test_string regexp;
+	const char *replacement;
+};
+
+/*!
+ * \brief Given a NAPTR record, generate a binary form, as would appear in DNS RDATA
+ *
+ * This is part of a DNS answer, specific to NAPTR. It consists of all parts of
+ * the NAPTR record, encoded as it should be in a DNS record.
+ *
+ * There is no buffer size passed to this function since we provide
+ * the data ourselves and have sized the buffer to be way larger
+ * than necessary for the tests.
+ *
+ * \param string The NAPTR record to encode
+ * \param buf The buffer to write the record into
+ * \return The number of bytes written to the buffer
+ */
+static int generate_naptr_record(void *dns_record, char *buf)
+{
+	struct naptr_record *record = dns_record;
+	uint16_t net_order = htons(record->order);
+	uint16_t net_preference = htons(record->preference);
+	char *ptr = buf;
+
+	memcpy(ptr, &net_order, sizeof(net_order));
+	ptr += sizeof(net_order);
+
+	memcpy(ptr, &net_preference, sizeof(net_preference));
+	ptr += sizeof(net_preference);
+
+	ptr += ast_dns_test_write_string(&record->flags, ptr);
+	ptr += ast_dns_test_write_string(&record->services, ptr);
+	ptr += ast_dns_test_write_string(&record->regexp, ptr);
+	ptr += ast_dns_test_write_domain(record->replacement, ptr);
+
+	return ptr - buf;
+}
+
+/*!
+ * \brief A pointer to an array of records for a test
+ *
+ * Each test is expected to set this pointer to its local
+ * array of records and then re-set tis pointer to NULL
+ * at the end of the test
+ */
+static struct naptr_record *test_records;
+/*!
+ * \brief The number of records in the test_records array.
+ *
+ * Each test must set this to the appropriate value at the
+ * beginning of the test and must set this back to zero at
+ * the end of the test.
+ */
+static int num_test_records;
+/*!
+ * \brief A buffer to place raw DNS records into.
+ *
+ * This buffer is way larger than any DNS records we actually
+ * wish to create during any of the tests, but that's fine.
+ */
+static char ans_buffer[1024];
+
+/*!
+ * \brief Asynchronous NAPTR resolution thread.
+ *
+ * This builds an appropriate DNS response based on the NAPTR
+ * records for a given test. Once the records have been created,
+ * the records are added to the DNS result
+ */
+static void *naptr_thread(void *dns_query)
+{
+	struct ast_dns_query *query = dns_query;
+	int i;
+	int ans_size;
+
+	ans_size = ast_dns_test_generate_result(query, test_records, num_test_records,
+			sizeof(struct naptr_record), generate_naptr_record, ans_buffer);
+
+	ast_dns_resolver_set_result(query, 0, 0, NOERROR, "goose.feathers", ans_buffer, ans_size);
+
+	for (i = 0; i < num_test_records; ++i) {
+		char record[128];
+		int naptr_size;
+
+		naptr_size = generate_naptr_record(&test_records[i], record);
+		ast_dns_resolver_add_record(query, T_NAPTR, C_IN, 12345, record, naptr_size);
+	}
+
+	ast_dns_resolver_completed(query);
+
+	ao2_ref(query, -1);
+	return NULL;
+}
+
+/*!
+ * \brief Mock NAPTR resolution method.
+ *
+ * This spawns a thread to handle generation of the necessary NAPTR records
+ */
+static int naptr_resolve(struct ast_dns_query *query)
+{
+	pthread_t thread;
+
+	return ast_pthread_create_detached(&thread, NULL, naptr_thread, ao2_bump(query));
+}
+
+/*!
+ * \brief A STUB
+ */
+static int naptr_cancel(struct ast_dns_query *query)
+{
+	return 0;
+}
+
+/*!
+ * \brief Mock NAPTR resolver
+ */
+static struct ast_dns_resolver naptr_resolver = {
+	.name = "naptr_test",
+	.priority = 0,
+	.resolve = naptr_resolve,
+	.cancel = naptr_cancel,
+};
+
+AST_TEST_DEFINE(naptr_resolve_nominal)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	const struct ast_dns_record *record;
+	struct naptr_record records[] = {
+		/* Incredibly plain record */
+		{ 200, 100, {1, "A"}, {4, "BLAH"}, {0, ""}, "goose.down" },
+		/* Records with valid but unusual flags */
+		{ 300,   8, {0, ""}, {4, "BLAH"}, {0, ""}, "goose.down" },
+		{ 300,   6, {1, "3"}, {4, "BLAH"}, {0, ""}, "goose.down" },
+		{ 100,   2, {2, "32"}, {4, "BLAH"}, {0, ""}, "goose.down" },
+		{ 400, 100, {3, "A32"}, {4, "BLAH"}, {0, ""}, "goose.down" },
+		/* Records with valid but unusual services */
+		{ 100, 700, {0, ""}, {0, ""}, {0, ""}, "goose.down" },
+		{ 500, 102, {1, "A"}, {42, "A+B12+C+D+EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE"}, {0, ""}, "goose.down" },
+		{ 500, 100, {1, "A"}, {14, "A+B12+C+D+EEEE"}, {0, ""}, "goose.down" },
+		/* Records with valid regexes (regexes are always unusual) */
+		{ 500, 101, {1, "A"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, "" },
+		{ 500,  99, {1, "A"}, {4, "BLAH"}, {15, "0.*0horse.mane0"}, "" },
+		{  10, 100, {1, "A"}, {4, "BLAH"}, {11, "!.*!\\!\\!\\!!"}, "" },
+		{ 700, 999, {1, "A"}, {4, "BLAH"}, {30, "!(.)(.)(.)(.)!\\1.m.\\2.n\\3.o\\4!"}, "" },
+	};
+
+	int naptr_record_order[] = { 10, 3, 5, 0, 2, 1, 4, 9, 7, 8, 6, 11};
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int i;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Test nominal resolution of NAPTR records";
+		info->description = "This test defines four valid NAPTR records and\n"
+			"performs a resolution of the domain to which they belong. The test\n"
+			"ensures that all fields of the NAPTR records are parsed correctly\n"
+			"and that the records are returned in sorted order";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	test_records = records;
+	num_test_records = ARRAY_LEN(records);
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	ast_dns_resolver_register(&naptr_resolver);
+
+	if (ast_dns_resolve("goose.feathers", T_NAPTR, C_IN, &result)) {
+		ast_test_status_update(test, "DNS resolution failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "DNS resolution returned no result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	i = 0;
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		if (ast_dns_naptr_get_order(record) != records[naptr_record_order[i]].order) {
+			ast_test_status_update(test, "Expected order %hu, got order %hu from NAPTR record\n",
+					records[naptr_record_order[i]].order, ast_dns_naptr_get_order(record));
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_naptr_get_preference(record) != records[naptr_record_order[i]].preference) {
+			ast_test_status_update(test, "Expected preference %hu, got preference %hu from NAPTR record\n",
+					records[naptr_record_order[i]].preference, ast_dns_naptr_get_preference(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_flags(record), records[naptr_record_order[i]].flags.val)) {
+			ast_test_status_update(test, "Expected flags %s, got flags %s from NAPTR record\n",
+					records[naptr_record_order[i]].flags.val, ast_dns_naptr_get_flags(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_service(record), records[naptr_record_order[i]].services.val)) {
+			ast_test_status_update(test, "Expected services %s, got services %s from NAPTR record\n",
+					records[naptr_record_order[i]].services.val, ast_dns_naptr_get_service(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_regexp(record), records[naptr_record_order[i]].regexp.val)) {
+			ast_test_status_update(test, "Expected regexp %s, got regexp %s from NAPTR record\n",
+					records[naptr_record_order[i]].regexp.val, ast_dns_naptr_get_regexp(record));
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_naptr_get_replacement(record), records[naptr_record_order[i]].replacement)) {
+			ast_test_status_update(test, "Expected replacement %s, got replacement %s from NAPTR record\n",
+					records[naptr_record_order[i]].replacement, ast_dns_naptr_get_replacement(record));
+			res = AST_TEST_FAIL;
+		}
+		++i;
+	}
+
+	if (i != ARRAY_LEN(records)) {
+		ast_test_status_update(test, "Unexpected number of records returned in NAPTR lookup\n");
+		res = AST_TEST_FAIL;
+	}
+
+cleanup:
+
+	ast_dns_resolver_unregister(&naptr_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+static enum ast_test_result_state off_nominal_test(struct ast_test *test, struct naptr_record *records, int num_records)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	enum ast_test_result_state res = AST_TEST_PASS;
+	const struct ast_dns_record *record;
+
+	test_records = records;
+	num_test_records = num_records;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	ast_dns_resolver_register(&naptr_resolver);
+
+	if (ast_dns_resolve("goose.feathers", T_NAPTR, C_IN, &result)) {
+		ast_test_status_update(test, "Failed to perform DNS resolution, despite using valid inputs\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "Synchronous DNS resolution failed to set a result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	record = ast_dns_result_get_records(result);
+	if (record) {
+		ast_test_status_update(test, "DNS resolution returned records when it was not expected to\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&naptr_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+AST_TEST_DEFINE(naptr_resolve_off_nominal_length)
+{
+	struct naptr_record records[] = {
+		{ 100, 100, {255, "A"}, {4, "BLAH"},   {15, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {0, "A"},   {4, "BLAH"},   {15, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {1, "A"},   {255, "BLAH"}, {15, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {1, "A"},   {2, "BLAH"},   {15, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {1, "A"},   {4, "BLAH"},   {255, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {1, "A"},   {4, "BLAH"},   {3, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {255, "A"}, {255, "BLAH"}, {255, "!.*!horse.mane!"}, "" },
+		{ 100, 100, {0, "A"},   {2, "BLAH"},   {3, "!.*!horse.mane!"}, "" },
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve_off_nominal_length";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Test resolution of NAPTR records with off-nominal lengths";
+		info->description = "This test defines a set of records where the strings provided\n"
+			"within the record are valid, but the lengths of the strings in the record are\n"
+			"invalid, either too large or too small. The goal of this test is to ensure that\n"
+			"these invalid lengths result in resolution failures";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(naptr_resolve_off_nominal_flags)
+{
+	struct naptr_record records[] = {
+		/* Non-alphanumeric flag */
+		{ 100, 100, {1, "!"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		/* Mix of valid and non-alphanumeric */
+		{ 100, 100, {2, "A!"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "!A"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		/* Invalid combinations of flags */
+		{ 100, 100, {2, "sa"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "su"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "sp"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "as"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "au"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "ap"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "ua"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "us"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "up"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "pa"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "ps"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {2, "pu"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, ""},
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve_off_nominal_flags";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Ensure that NAPTR records with invalid flags are not presented in results";
+		info->description = "This test defines a set of records where the flags provided are\n"
+			"invalid in some way. This may be due to providing non-alphanumeric characters or\n"
+			"by providing clashing flags. The result should be that none of the defined records\n"
+			"are returned by the resolver";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(naptr_resolve_off_nominal_services)
+{
+	struct naptr_record records[] = {
+		{ 100, 100, {1, "A"}, {5, "BLAH!"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {5, "BL!AH"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {8, "1SIP+D2U"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {8, "SIP+1D2U"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {4, "+D2U"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {4, "SIP+"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {8, "SIP++D2U"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {37, "SIPSIPSIPSIPSIPSIPSIPSIPSIPSIPSIP+D2U"}, {15, "!.*!horse.mane!"}, ""},
+		{ 100, 100, {1, "A"}, {37, "SIP+D2UD2UD2UD2UD2UD2UD2UD2UD2UD2UD2U"}, {15, "!.*!horse.mane!"}, ""},
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve_off_nominal_services";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Ensure that NAPTR records with invalid services are not presented in results";
+		info->description = "This test defines a set of records where the services provided are\n"
+			"invalid in some way. This may be due to providing non-alphanumeric characters, providing\n"
+			"protocols or resolution services that start with a non-alphabetic character, or\n"
+			"providing fields that are too long.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(naptr_resolve_off_nominal_regexp)
+{
+	struct naptr_record records[] = {
+		/* Invalid delim-char */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {15, "1.*1horse.mane1"}, ""},
+		/* Not enough delim-chars */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {14, "!.*!horse.mane"}, ""},
+		/* Not enough delim-chars, part 2 */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {16, "!.*!horse.mane\\!"}, ""},
+		/* Too many delim-chars */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {15, "!.*!horse!mane!"}, ""},
+		/* Invalid regex flag */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {16, "!.*!horse.mane!o"}, ""},
+		/* Invalid backreference */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {13, "!.*!horse.\\0!"}, ""},
+		/* Invalid regex */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {16, "!(.*!horse.mane!"}, ""},
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve_off_nominal_regexp";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Ensure that NAPTR records with invalid regexps are not presented in results";
+		info->description = "This test defines a set of records where the regexps provided are\n"
+			"invalid in some way. The test ensures that none of the invalid records are returned\n"
+			"when performing a NAPTR lookup";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(naptr_resolve_off_nominal_interactions)
+{
+	struct naptr_record records[] = {
+		/* Both regexp and replacement are specified */
+		{ 100, 100, {1, "A"}, {4, "BLAH"}, {15, "!.*!horse.mane!"}, "goose.down"},
+		/* XXX RFC 2915 says that a service MUST be present if terminal flags are
+		 * specified. However, RFCs 3401-3404 do not specify this behavior, so
+		 * I am not putting in a test for it
+		 */
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "naptr_resolve_off_nominal_interactions";
+		info->category = "/main/dns/naptr/";
+		info->summary = "Ensure that NAPTR records with invalid interactions are not presented in results";
+		info->description = "This test defines a set of records where all parts are individually valid,\n"
+			"but when combined do not make sense and are thus invalid.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return off_nominal_test(test, records, ARRAY_LEN(records));
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(naptr_resolve_nominal);
+	AST_TEST_UNREGISTER(naptr_resolve_off_nominal_length);
+	AST_TEST_UNREGISTER(naptr_resolve_off_nominal_flags);
+	AST_TEST_UNREGISTER(naptr_resolve_off_nominal_services);
+	AST_TEST_UNREGISTER(naptr_resolve_off_nominal_regexp);
+	AST_TEST_UNREGISTER(naptr_resolve_off_nominal_interactions);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(naptr_resolve_nominal);
+	AST_TEST_REGISTER(naptr_resolve_off_nominal_length);
+	AST_TEST_REGISTER(naptr_resolve_off_nominal_flags);
+	AST_TEST_REGISTER(naptr_resolve_off_nominal_services);
+	AST_TEST_REGISTER(naptr_resolve_off_nominal_regexp);
+	AST_TEST_REGISTER(naptr_resolve_off_nominal_interactions);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "DNS API Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_dns_query_set.c 1:16.1.1~dfsg-1/tests/test_dns_query_set.c
--- 1:13.23.1~dfsg-2/tests/test_dns_query_set.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dns_query_set.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,392 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+#include <arpa/inet.h>
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/vector.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_query_set.h"
+#include "asterisk/dns_internal.h"
+
+struct query_set_data {
+	/*! Boolean indicator if query set has completed */
+	int query_set_complete;
+	/*! Number of times resolve() method has been called */
+	int resolves;
+	/*! Number of times resolve() method is allowed to be called */
+	int resolves_allowed;
+	/*! Number of times cancel() method has been called */
+	int cancel;
+	/*! Number of times cancel() method is allowed to be called */
+	int cancel_allowed;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+static void query_set_data_destructor(void *obj)
+{
+	struct query_set_data *qsdata = obj;
+
+	ast_mutex_destroy(&qsdata->lock);
+	ast_cond_destroy(&qsdata->cond);
+}
+
+static struct query_set_data *query_set_data_alloc(void)
+{
+	struct query_set_data *qsdata;
+
+	qsdata = ao2_alloc(sizeof(*qsdata), query_set_data_destructor);
+	if (!qsdata) {
+		return NULL;
+	}
+
+	ast_mutex_init(&qsdata->lock);
+	ast_cond_init(&qsdata->cond, NULL);
+
+	return qsdata;
+}
+
+#define DNS_ANSWER "Yes sirree"
+#define DNS_ANSWER_SIZE strlen(DNS_ANSWER)
+
+/*!
+ * \brief Thread that performs asynchronous resolution.
+ *
+ * This thread uses the query's user data to determine how to
+ * perform the resolution. If the allowed number of resolutions
+ * has not been reached then this will succeed, otherwise the
+ * query is expected to have been canceled.
+ *
+ * \param dns_query The ast_dns_query that is being performed
+ * \return NULL
+ */
+static void *resolution_thread(void *dns_query)
+{
+	struct ast_dns_query *query = dns_query;
+	struct ast_dns_query_set *query_set = ast_dns_query_get_data(query);
+	struct query_set_data *qsdata = query_set->user_data;
+
+	ast_assert(qsdata != NULL);
+
+	ast_dns_resolver_set_result(query, 0, 0, NOERROR, "asterisk.org", DNS_ANSWER, DNS_ANSWER_SIZE);
+	ast_dns_resolver_completed(query);
+
+	ao2_ref(query, -1);
+	return NULL;
+}
+
+/*!
+ * \brief Resolver's resolve() method
+ *
+ * \param query The query that is to be resolved
+ * \retval 0 Successfully created thread to perform the resolution
+ * \retval non-zero Failed to create resolution thread
+ */
+static int query_set_resolve(struct ast_dns_query *query)
+{
+	struct ast_dns_query_set *query_set = ast_dns_query_get_data(query);
+	struct query_set_data *qsdata = query_set->user_data;
+	pthread_t resolver_thread;
+
+	/* Only the queries which will not be canceled actually start a thread */
+	if (qsdata->resolves++ < qsdata->cancel_allowed) {
+		return 0;
+	}
+
+	return ast_pthread_create_detached(&resolver_thread, NULL, resolution_thread, ao2_bump(query));
+}
+
+/*!
+ * \brief Resolver's cancel() method
+ *
+ * \param query The query to cancel
+ * \return 0
+ */
+static int query_set_cancel(struct ast_dns_query *query)
+{
+	struct ast_dns_query_set *query_set = ast_dns_query_get_data(query);
+	struct query_set_data *qsdata;
+	int res = -1;
+
+	if (!query_set) {
+		return -1;
+	}
+	qsdata = query_set->user_data;
+
+	if (qsdata->cancel++ < qsdata->cancel_allowed) {
+		res = 0;
+	}
+
+	return res;
+}
+
+static struct ast_dns_resolver query_set_resolver = {
+	.name = "query_set",
+	.priority = 0,
+	.resolve = query_set_resolve,
+	.cancel = query_set_cancel,
+};
+
+/*!
+ * \brief Callback which is invoked upon query set completion
+ *
+ * \param query_set The query set
+ */
+static void query_set_callback(const struct ast_dns_query_set *query_set)
+{
+	struct query_set_data *qsdata = ast_dns_query_set_get_data(query_set);
+
+	ast_mutex_lock(&qsdata->lock);
+	qsdata->query_set_complete = 1;
+	ast_cond_signal(&qsdata->cond);
+	ast_mutex_unlock(&qsdata->lock);
+}
+
+/*!
+ * \brief Framework for running a query set DNS test
+ *
+ * This function serves as a common way of testing various numbers of queries in a
+ * query set and optional canceling of them.
+ *
+ * \param test The test being run
+ * \param resolve The number of queries that should be allowed to complete resolution
+ * \param cancel The number of queries that should be allowed to be canceled
+ */
+static enum ast_test_result_state query_set_test(struct ast_test *test, int resolve, int cancel)
+{
+	int total = resolve + cancel;
+	RAII_VAR(struct ast_dns_query_set *, query_set, NULL, ao2_cleanup);
+	RAII_VAR(struct query_set_data *, qsdata, NULL, ao2_cleanup);
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int idx;
+	struct timespec timeout;
+
+	if (ast_dns_resolver_register(&query_set_resolver)) {
+		ast_test_status_update(test, "Failed to register query set DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	qsdata = query_set_data_alloc();
+	if (!qsdata) {
+		ast_test_status_update(test, "Failed to allocate data necessary for query set test\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	query_set = ast_dns_query_set_create();
+	if (!query_set) {
+		ast_test_status_update(test, "Failed to create DNS query set\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	qsdata->resolves_allowed = resolve;
+	qsdata->cancel_allowed = cancel;
+
+	for (idx = 0; idx < total; ++idx) {
+		if (ast_dns_query_set_add(query_set, "asterisk.org", T_A, C_IN)) {
+			ast_test_status_update(test, "Failed to add query to DNS query set\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+	}
+
+	if (ast_dns_query_set_num_queries(query_set) != total) {
+		ast_test_status_update(test, "DNS query set does not contain the correct number of queries\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	ast_dns_query_set_resolve_async(query_set, query_set_callback, qsdata);
+
+	if (cancel && (cancel == total)) {
+		if (ast_dns_query_set_resolve_cancel(query_set)) {
+			ast_test_status_update(test, "Failed to cancel DNS query set when it should be cancellable\n");
+			res = AST_TEST_FAIL;
+		}
+
+		if (qsdata->query_set_complete) {
+			ast_test_status_update(test, "Query set callback was invoked despite all queries being cancelled\n");
+			res = AST_TEST_FAIL;
+		}
+
+		goto cleanup;
+	} else if (cancel) {
+		if (!ast_dns_query_set_resolve_cancel(query_set)) {
+			ast_test_status_update(test, "Successfully cancelled DNS query set when it should not be possible\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+	}
+
+	timeout = ast_tsnow();
+	timeout.tv_sec += 10;
+
+	ast_mutex_lock(&qsdata->lock);
+	while (!qsdata->query_set_complete) {
+		if (ast_cond_timedwait(&qsdata->cond, &qsdata->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&qsdata->lock);
+
+	if (!qsdata->query_set_complete) {
+		ast_test_status_update(test, "Query set did not complete when it should have\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	for (idx = 0; idx < ast_dns_query_set_num_queries(query_set); ++idx) {
+		const struct ast_dns_query *query = ast_dns_query_set_get(query_set, idx);
+
+		if (strcmp(ast_dns_query_get_name(query), "asterisk.org")) {
+			ast_test_status_update(test, "Query did not have expected name\n");
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_query_get_rr_type(query) != T_A) {
+			ast_test_status_update(test, "Query did not have expected type\n");
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_query_get_rr_class(query) != C_IN) {
+			ast_test_status_update(test, "Query did not have expected class\n");
+			res = AST_TEST_FAIL;
+		}
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&query_set_resolver);
+	return res;
+}
+
+AST_TEST_DEFINE(query_set)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "query_set";
+		info->category = "/main/dns/query_set/";
+		info->summary = "Test nominal asynchronous DNS query set";
+		info->description =
+			"This tests nominal query set in the following ways:\n"
+			"\t* Multiple queries are added to a query set\n"
+			"\t* The mock resolver is configured to respond to all queries\n"
+			"\t* Asynchronous resolution of the query set is started\n"
+			"\t* The mock resolver responds to all queries\n"
+			"\t* We ensure that the query set callback is invoked upon completion";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return query_set_test(test, 4, 0);
+}
+
+AST_TEST_DEFINE(query_set_empty)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "query_set_empty";
+		info->category = "/main/dns/query_set/";
+		info->summary = "Test nominal asynchronous empty DNS query set";
+		info->description =
+			"This tests nominal query set in the following ways:\n"
+			"\t* No queries are added to a query set\n"
+			"\t* Asynchronous resolution of the query set is started\n"
+			"\t* We ensure that the query set callback is invoked upon completion";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return query_set_test(test, 0, 0);
+}
+
+AST_TEST_DEFINE(query_set_nominal_cancel)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "query_set_nominal_cancel";
+		info->category = "/main/dns/query_set/";
+		info->summary = "Test nominal asynchronous DNS query set cancellation";
+		info->description =
+			"This tests nominal query set cancellation in the following ways:\n"
+			"\t* Multiple queries are added to a query set\n"
+			"\t* The mock resolver is configured to NOT respond to any queries\n"
+			"\t* Asynchronous resolution of the query set is started\n"
+			"\t* The query set is canceled and is confirmed to return with success";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return query_set_test(test, 0, 4);
+}
+
+AST_TEST_DEFINE(query_set_off_nominal_cancel)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "query_set_off_nominal_cancel";
+		info->category = "/main/dns/query_set/";
+		info->summary = "Test off-nominal asynchronous DNS query set cancellation";
+		info->description =
+			"This tests nominal query set cancellation in the following ways:\n"
+			"\t* Multiple queries are added to a query set\n"
+			"\t* The mock resolver is configured to respond to half the queries\n"
+			"\t* Asynchronous resolution of the query set is started\n"
+			"\t* The query set is canceled and is confirmed to return failure\n"
+			"\t* The query set callback is confirmed to run, since it could not be fully canceled";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return query_set_test(test, 2, 2);
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(query_set);
+	AST_TEST_UNREGISTER(query_set_empty);
+	AST_TEST_UNREGISTER(query_set_nominal_cancel);
+	AST_TEST_UNREGISTER(query_set_off_nominal_cancel);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(query_set);
+	AST_TEST_REGISTER(query_set_empty);
+	AST_TEST_REGISTER(query_set_nominal_cancel);
+	AST_TEST_REGISTER(query_set_off_nominal_cancel);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "DNS query set tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_dns_recurring.c 1:16.1.1~dfsg-1/tests/test_dns_recurring.c
--- 1:13.23.1~dfsg-2/tests/test_dns_recurring.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dns_recurring.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,647 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Mark Michelson
+ *
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+#include <arpa/inet.h>
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_recurring.h"
+#include "asterisk/dns_internal.h"
+
+struct recurring_data {
+	/*! TTL to place in first returned result */
+	int ttl1;
+	/*! TTL to place in second returned result */
+	int ttl2;
+	/*! Boolean indicator if query has completed */
+	int query_complete;
+	/*! Number of times recurring resolution has completed */
+	int complete_resolutions;
+	/*! Number of times resolve() method has been called */
+	int resolves;
+	/*! Indicates that the query is expected to be canceled */
+	int cancel_expected;
+	/*! Indicates that the query is ready to be canceled */
+	int cancel_ready;
+	/*! Indicates that the query has been canceled */
+	int canceled;
+	ast_mutex_t lock;
+	ast_cond_t cond;
+};
+
+static void recurring_data_destructor(void *obj)
+{
+	struct recurring_data *rdata = obj;
+
+	ast_mutex_destroy(&rdata->lock);
+	ast_cond_destroy(&rdata->cond);
+}
+
+static struct recurring_data *recurring_data_alloc(void)
+{
+	struct recurring_data *rdata;
+
+	rdata = ao2_alloc(sizeof(*rdata), recurring_data_destructor);
+	if (!rdata) {
+		return NULL;
+	}
+
+	ast_mutex_init(&rdata->lock);
+	ast_cond_init(&rdata->cond, NULL);
+
+	return rdata;
+}
+
+#define DNS_ANSWER "Yes sirree"
+#define DNS_ANSWER_SIZE strlen(DNS_ANSWER)
+
+/*!
+ * \brief Thread that performs asynchronous resolution.
+ *
+ * This thread uses the query's user data to determine how to
+ * perform the resolution. The query may either be canceled or
+ * it may be completed with records.
+ *
+ * \param dns_query The ast_dns_query that is being performed
+ * \return NULL
+ */
+static void *resolution_thread(void *dns_query)
+{
+	struct ast_dns_query *query = dns_query;
+
+	static const char *ADDR1 = "127.0.0.1";
+	static const size_t ADDR1_BUFSIZE = sizeof(struct in_addr);
+	char addr1_buf[ADDR1_BUFSIZE];
+
+	static const char *ADDR2 = "192.168.0.1";
+	static const size_t ADDR2_BUFSIZE = sizeof(struct in_addr);
+	char addr2_buf[ADDR2_BUFSIZE];
+
+	struct ast_dns_query_recurring *recurring = ast_dns_query_get_data(query);
+	struct recurring_data *rdata = recurring->user_data;
+
+	ast_assert(rdata != NULL);
+
+	/* Canceling is an interesting dance. This thread needs to signal that it is
+	 * ready to be canceled. Then it needs to wait until the query is actually canceled.
+	 */
+	if (rdata->cancel_expected) {
+		ast_mutex_lock(&rdata->lock);
+		rdata->cancel_ready = 1;
+		ast_cond_signal(&rdata->cond);
+
+		while (!rdata->canceled) {
+			ast_cond_wait(&rdata->cond, &rdata->lock);
+		}
+		ast_mutex_unlock(&rdata->lock);
+
+		ast_dns_resolver_completed(query);
+		ao2_ref(query, -1);
+
+		return NULL;
+	}
+
+	/* When the query isn't canceled, we set the TTL of the results based on what
+	 * we've been told to set it to
+	 */
+	ast_dns_resolver_set_result(query, 0, 0, NOERROR, "asterisk.org", DNS_ANSWER, DNS_ANSWER_SIZE);
+
+	inet_pton(AF_INET, ADDR1, addr1_buf);
+	ast_dns_resolver_add_record(query, T_A, C_IN, rdata->ttl1, addr1_buf, ADDR1_BUFSIZE);
+
+	inet_pton(AF_INET, ADDR2, addr2_buf);
+	ast_dns_resolver_add_record(query, T_A, C_IN, rdata->ttl2, addr2_buf, ADDR2_BUFSIZE);
+
+	++rdata->complete_resolutions;
+
+	ast_dns_resolver_completed(query);
+
+	ao2_ref(query, -1);
+	return NULL;
+}
+
+/*!
+ * \brief Resolver's resolve() method
+ *
+ * \param query The query that is to be resolved
+ * \retval 0 Successfully created thread to perform the resolution
+ * \retval non-zero Failed to create resolution thread
+ */
+static int recurring_resolve(struct ast_dns_query *query)
+{
+	struct ast_dns_query_recurring *recurring = ast_dns_query_get_data(query);
+	struct recurring_data *rdata = recurring->user_data;
+	pthread_t resolver_thread;
+
+	ast_assert(rdata != NULL);
+	++rdata->resolves;
+	return ast_pthread_create_detached(&resolver_thread, NULL, resolution_thread, ao2_bump(query));
+}
+
+/*!
+ * \brief Resolver's cancel() method
+ *
+ * \param query The query to cancel
+ * \return 0
+ */
+static int recurring_cancel(struct ast_dns_query *query)
+{
+	struct ast_dns_query_recurring *recurring = ast_dns_query_get_data(query);
+	struct recurring_data *rdata = recurring->user_data;
+
+	ast_mutex_lock(&rdata->lock);
+	rdata->canceled = 1;
+	ast_cond_signal(&rdata->cond);
+	ast_mutex_unlock(&rdata->lock);
+
+	return 0;
+}
+
+static struct ast_dns_resolver recurring_resolver = {
+	.name = "test_recurring",
+	.priority = 0,
+	.resolve = recurring_resolve,
+	.cancel = recurring_cancel,
+};
+
+/*!
+ * \brief Wait for a successful resolution to complete
+ *
+ * This is called whenever a successful DNS resolution occurs. This function
+ * serves to ensure that parameters are as we expect them to be.
+ *
+ * \param test The test being executed
+ * \param rdata DNS query user data
+ * \param expected_lapse The amount of time we expect to wait for the query to complete
+ * \param num_resolves The number of DNS resolutions that have been executed
+ * \param num_completed The number of DNS resolutions we expect to have completed successfully
+ * \param canceled Whether the query is expected to have been canceled
+ */
+static int wait_for_resolution(struct ast_test *test, struct recurring_data *rdata,
+		int expected_lapse, int num_resolves, int num_completed, int canceled)
+{
+	struct timespec begin;
+	struct timespec end;
+	struct timespec timeout;
+	int secdiff;
+
+	begin = ast_tsnow();
+
+	timeout.tv_sec = begin.tv_sec + 20;
+	timeout.tv_nsec = begin.tv_nsec;
+
+	ast_mutex_lock(&rdata->lock);
+	while (!rdata->query_complete) {
+		if (ast_cond_timedwait(&rdata->cond, &rdata->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&rdata->lock);
+
+	if (!rdata->query_complete) {
+		ast_test_status_update(test, "Query timed out\n");
+		return -1;
+	}
+
+	rdata->query_complete = 0;
+	end = ast_tsnow();
+
+	secdiff = end.tv_sec - begin.tv_sec;
+
+	/* Give ourselves some wiggle room */
+	if (secdiff < expected_lapse - 2 || secdiff > expected_lapse + 2) {
+		ast_test_status_update(test, "Query did not complete in expected time\n");
+		return -1;
+	}
+
+	if (rdata->resolves != num_resolves || rdata->complete_resolutions != num_completed) {
+		ast_test_status_update(test, "Query has not undergone expected number of resolutions\n");
+		return -1;
+	}
+
+	if (rdata->canceled != canceled) {
+		ast_test_status_update(test, "Query was canceled unexpectedly\n");
+		return -1;
+	}
+
+	ast_test_status_update(test, "Query completed in expected time frame\n");
+
+	return 0;
+}
+
+static void async_callback(const struct ast_dns_query *query)
+{
+	struct recurring_data *rdata = ast_dns_query_get_data(query);
+
+	ast_assert(rdata != NULL);
+
+	ast_mutex_lock(&rdata->lock);
+	rdata->query_complete = 1;
+	ast_cond_signal(&rdata->cond);
+	ast_mutex_unlock(&rdata->lock);
+}
+
+AST_TEST_DEFINE(recurring_query)
+{
+	RAII_VAR(struct ast_dns_query_recurring *, recurring_query, NULL, ao2_cleanup);
+	RAII_VAR(struct recurring_data *, rdata, NULL, ao2_cleanup);
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int expected_lapse;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "recurring_query";
+		info->category = "/main/dns/recurring/";
+		info->summary = "Test nominal asynchronous recurring DNS queries";
+		info->description =
+			"This tests nominal recurring queries in the following ways:\n"
+			"\t* An asynchronous query is sent to a mock resolver\n"
+			"\t* The mock resolver returns two records with different TTLs\n"
+			"\t* We ensure that the query re-occurs according to the lower of the TTLs\n"
+			"\t* The mock resolver returns two records, this time with different TTLs\n"
+			"\t  from the first time the query was resolved\n"
+			"\t* We ensure that the query re-occurs according to the new lower TTL";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&recurring_resolver)) {
+		ast_test_status_update(test, "Failed to register recurring DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	rdata = recurring_data_alloc();
+	if (!rdata) {
+		ast_test_status_update(test, "Failed to allocate data necessary for recurring test\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	expected_lapse = 0;
+	rdata->ttl1 = 5;
+	rdata->ttl2 = 20;
+
+	recurring_query = ast_dns_resolve_recurring("asterisk.org", T_A, C_IN, async_callback, rdata);
+	if (!recurring_query) {
+		ast_test_status_update(test, "Failed to create recurring DNS query\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	/* This should be near instantaneous */
+	if (wait_for_resolution(test, rdata, expected_lapse, 1, 1, 0)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	expected_lapse = rdata->ttl1;
+	rdata->ttl1 = 45;
+	rdata->ttl2 = 10;
+
+	/* This should take approximately 5 seconds */
+	if (wait_for_resolution(test, rdata, expected_lapse, 2, 2, 0)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	expected_lapse = rdata->ttl2;
+
+	/* This should take approximately 10 seconds */
+	if (wait_for_resolution(test, rdata, expected_lapse, 3, 3, 0)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	if (recurring_query) {
+		/* XXX I don't like calling this here since I'm not testing
+		 * canceling recurring queries, but I'm forced into it here
+		 */
+		ast_dns_resolve_recurring_cancel(recurring_query);
+	}
+	ast_dns_resolver_unregister(&recurring_resolver);
+	return res;
+}
+
+static int fail_resolve(struct ast_dns_query *query)
+{
+	return -1;
+}
+
+static int stub_cancel(struct ast_dns_query *query)
+{
+	return 0;
+}
+
+static void stub_callback(const struct ast_dns_query *query)
+{
+	return;
+}
+
+AST_TEST_DEFINE(recurring_query_off_nominal)
+{
+	struct ast_dns_resolver terrible_resolver = {
+		.name = "Harold P. Warren's Filmography",
+		.priority = 0,
+		.resolve = fail_resolve,
+		.cancel = stub_cancel,
+	};
+
+	struct ast_dns_query_recurring *recurring;
+
+	struct dns_resolve_data {
+		const char *name;
+		int rr_type;
+		int rr_class;
+		ast_dns_resolve_callback callback;
+	} resolves [] = {
+		{ NULL,           T_A,       C_IN,      stub_callback },
+		{ "asterisk.org", -1,        C_IN,      stub_callback },
+		{ "asterisk.org", 65536 + 1, C_IN,      stub_callback },
+		{ "asterisk.org", T_A,       -1,        stub_callback },
+		{ "asterisk.org", T_A,       65536 + 1, stub_callback },
+		{ "asterisk.org", T_A,       C_IN,      NULL },
+	};
+	int i;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "recurring_query_off_nominal";
+		info->category = "/main/dns/recurring/";
+		info->summary = "Test off-nominal recurring DNS resolution";
+		info->description =
+			"This test performs several off-nominal recurring DNS resolutions:\n"
+			"\t* Attempt resolution with NULL name\n"
+			"\t* Attempt resolution with invalid RR type\n"
+			"\t* Attempt resolution with invalid RR class\n"
+			"\t* Attempt resolution with NULL callback pointer\n"
+			"\t* Attempt resolution with resolver that returns an error";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&recurring_resolver)) {
+		ast_test_status_update(test, "Failed to register test resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	for (i = 0; i < ARRAY_LEN(resolves); ++i) {
+		recurring = ast_dns_resolve_recurring(resolves[i].name, resolves[i].rr_type, resolves[i].rr_class,
+				resolves[i].callback, NULL);
+		if (recurring) {
+			ast_test_status_update(test, "Successfully performed recurring resolution with invalid data\n");
+			ast_dns_resolve_recurring_cancel(recurring);
+			ao2_ref(recurring, -1);
+			res = AST_TEST_FAIL;
+		}
+	}
+
+	ast_dns_resolver_unregister(&recurring_resolver);
+
+	if (ast_dns_resolver_register(&terrible_resolver)) {
+		ast_test_status_update(test, "Failed to register the DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	recurring = ast_dns_resolve_recurring("asterisk.org", T_A, C_IN, stub_callback, NULL);
+
+	ast_dns_resolver_unregister(&terrible_resolver);
+
+	if (recurring) {
+		ast_test_status_update(test, "Successfully performed recurring resolution with invalid data\n");
+		ast_dns_resolve_recurring_cancel(recurring);
+		ao2_ref(recurring, -1);
+		return AST_TEST_FAIL;
+	}
+
+	return res;
+}
+
+AST_TEST_DEFINE(recurring_query_cancel_between)
+{
+	RAII_VAR(struct ast_dns_query_recurring *, recurring_query, NULL, ao2_cleanup);
+	RAII_VAR(struct recurring_data *, rdata, NULL, ao2_cleanup);
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+	struct timespec timeout;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "recurring_query_cancel_between";
+		info->category = "/main/dns/recurring/";
+		info->summary = "Test canceling a recurring DNS query during the downtime between queries";
+		info->description = "This test does the following:\n"
+			"\t* Issue a recurring DNS query.\n"
+			"\t* Once results have been returned, cancel the recurring query.\n"
+			"\t* Wait a while to ensure that no more queries are occurring.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&recurring_resolver)) {
+		ast_test_status_update(test, "Failed to register recurring DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	rdata = recurring_data_alloc();
+	if (!rdata) {
+		ast_test_status_update(test, "Failed to allocate data necessary for recurring test\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	rdata->ttl1 = 5;
+	rdata->ttl2 = 20;
+
+	recurring_query = ast_dns_resolve_recurring("asterisk.org", T_A, C_IN, async_callback, rdata);
+	if (!recurring_query) {
+		ast_test_status_update(test, "Unable to make recurring query\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (wait_for_resolution(test, rdata, 0, 1, 1, 0)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (ast_dns_resolve_recurring_cancel(recurring_query)) {
+		ast_test_status_update(test, "Failed to cancel recurring query\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	/* Query has been canceled, so let's wait to make sure that we don't get
+	 * told another query has occurred.
+	 */
+	timeout = ast_tsnow();
+	timeout.tv_sec += 10;
+
+	ast_mutex_lock(&rdata->lock);
+	while (!rdata->query_complete) {
+		if (ast_cond_timedwait(&rdata->cond, &rdata->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&rdata->lock);
+
+	if (rdata->query_complete) {
+		ast_test_status_update(test, "Recurring query occurred after cancellation\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&recurring_resolver);
+	return res;
+}
+
+AST_TEST_DEFINE(recurring_query_cancel_during)
+{
+
+	RAII_VAR(struct ast_dns_query_recurring *, recurring_query, NULL, ao2_cleanup);
+	RAII_VAR(struct recurring_data *, rdata, NULL, ao2_cleanup);
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+	struct timespec timeout;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "recurring_query_cancel_during";
+		info->category = "/main/dns/recurring/";
+		info->summary = "Cancel a recurring DNS query while a query is actually happening";
+		info->description = "This test does the following:\n"
+			"\t* Initiate a recurring DNS query.\n"
+			"\t* Allow the initial query to complete, and a second query to start\n"
+			"\t* Cancel the recurring query while the second query is executing\n"
+			"\t* Ensure that the resolver's cancel() method was called\n"
+			"\t* Wait a while to make sure that recurring queries are no longer occurring";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_dns_resolver_register(&recurring_resolver)) {
+		ast_test_status_update(test, "Failed to register recurring DNS resolver\n");
+		return AST_TEST_FAIL;
+	}
+
+	rdata = recurring_data_alloc();
+	if (!rdata) {
+		ast_test_status_update(test, "Failed to allocate data necessary for recurring test\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	rdata->ttl1 = 5;
+	rdata->ttl2 = 20;
+
+	recurring_query = ast_dns_resolve_recurring("asterisk.org", T_A, C_IN, async_callback, rdata);
+	if (!recurring_query) {
+		ast_test_status_update(test, "Failed to make recurring DNS query\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (wait_for_resolution(test, rdata, 0, 1, 1, 0)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	/* Initial query has completed. Now let's make the next query expect a cancelation */
+	rdata->cancel_expected = 1;
+
+	/* Wait to be told that the query should be canceled  */
+	ast_mutex_lock(&rdata->lock);
+	while (!rdata->cancel_ready) {
+		ast_cond_wait(&rdata->cond, &rdata->lock);
+	}
+	rdata->cancel_expected = 0;
+	ast_mutex_unlock(&rdata->lock);
+
+	if (ast_dns_resolve_recurring_cancel(recurring_query)) {
+		ast_test_status_update(test, "Failed to cancel recurring DNS query\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	/* Query has been canceled. We'll be told that the query in flight has completed. */
+	if (wait_for_resolution(test, rdata, 0, 2, 1, 1)) {
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	/* Now ensure that no more queries get completed after cancellation. */
+	timeout = ast_tsnow();
+	timeout.tv_sec += 10;
+
+	ast_mutex_lock(&rdata->lock);
+	while (!rdata->query_complete) {
+		if (ast_cond_timedwait(&rdata->cond, &rdata->lock, &timeout) == ETIMEDOUT) {
+			break;
+		}
+	}
+	ast_mutex_unlock(&rdata->lock);
+
+	if (rdata->query_complete) {
+		ast_test_status_update(test, "Recurring query occurred after cancellation\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+cleanup:
+	ast_dns_resolver_unregister(&recurring_resolver);
+	return res;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(recurring_query);
+	AST_TEST_UNREGISTER(recurring_query_off_nominal);
+	AST_TEST_UNREGISTER(recurring_query_cancel_between);
+	AST_TEST_UNREGISTER(recurring_query_cancel_during);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(recurring_query);
+	AST_TEST_REGISTER(recurring_query_off_nominal);
+	AST_TEST_REGISTER(recurring_query_cancel_between);
+	AST_TEST_REGISTER(recurring_query_cancel_during);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Recurring DNS query tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_dns_srv.c 1:16.1.1~dfsg-1/tests/test_dns_srv.c
--- 1:13.23.1~dfsg-2/tests/test_dns_srv.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_dns_srv.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,589 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ * Mark Michelson <mmichelson@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <arpa/nameser.h>
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/dns_core.h"
+#include "asterisk/dns_resolver.h"
+#include "asterisk/dns_srv.h"
+#include "asterisk/dns_test.h"
+
+struct srv_record {
+	uint16_t priority;
+	uint16_t weight;
+	uint16_t port;
+	const char *host;
+	unsigned int ignore_priority;
+	unsigned int ignore_weight;
+	unsigned int ignore_port;
+	unsigned int ignore_host;
+};
+
+static int generate_srv_record(void *dns_record, char *buf)
+{
+	struct srv_record *record = dns_record;
+	uint16_t priority = htons(record->priority);
+	uint16_t weight = htons(record->weight);
+	uint16_t port = htons(record->port);
+	char *ptr = buf;
+
+	if (!record->ignore_priority) {
+		memcpy(ptr, &priority, sizeof(priority));
+		ptr += sizeof(priority);
+	}
+
+	if (!record->ignore_weight) {
+		memcpy(ptr, &weight, sizeof(weight));
+		ptr += sizeof(weight);
+	}
+
+	if (!record->ignore_port) {
+		memcpy(ptr, &port, sizeof(port));
+		ptr += sizeof(port);
+	}
+
+	if (!record->ignore_host) {
+		ptr += ast_dns_test_write_domain(record->host, ptr);
+	}
+
+	return ptr - buf;
+}
+
+static struct srv_record *test_records;
+static int num_test_records;
+static char ans_buffer[1024];
+
+static void *srv_thread(void *dns_query)
+{
+	struct ast_dns_query *query = dns_query;
+	int i;
+	int ans_size;
+
+	ans_size = ast_dns_test_generate_result(query, test_records, num_test_records,
+			sizeof(struct srv_record), generate_srv_record, ans_buffer);
+
+	ast_dns_resolver_set_result(query, 0, 0, NOERROR, "goose.feathers", ans_buffer, ans_size);
+
+	for (i = 0; i < num_test_records; ++i) {
+		char record[128];
+		int srv_size;
+
+		srv_size = generate_srv_record(&test_records[i], record);
+		ast_dns_resolver_add_record(query, T_SRV, C_IN, 12345, record, srv_size);
+	}
+
+	ast_dns_resolver_completed(query);
+
+	ao2_ref(query, -1);
+	return NULL;
+}
+
+static int srv_resolve(struct ast_dns_query *query)
+{
+	pthread_t thread;
+
+	return ast_pthread_create_detached(&thread, NULL, srv_thread, ao2_bump(query));
+}
+
+static int srv_cancel(struct ast_dns_query *query)
+{
+	return -1;
+}
+
+static struct ast_dns_resolver srv_resolver = {
+	.name = "srv_test",
+	.priority = 0,
+	.resolve = srv_resolve,
+	.cancel = srv_cancel,
+};
+
+static enum ast_test_result_state nominal_test(struct ast_test *test, struct srv_record *records,
+	int *srv_record_order, int num_records)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	const struct ast_dns_record *record;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int i;
+
+	test_records = records;
+	num_test_records = num_records;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	ast_dns_resolver_register(&srv_resolver);
+
+	if (ast_dns_resolve("goose.feathers", T_SRV, C_IN, &result)) {
+		ast_test_status_update(test, "DNS resolution failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "DNS resolution returned no result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	i = 0;
+	for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+		if (ast_dns_srv_get_priority(record) != records[srv_record_order[i]].priority) {
+			ast_test_status_update(test, "Unexpected priority in returned SRV record\n");
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_srv_get_weight(record) != records[srv_record_order[i]].weight) {
+			ast_test_status_update(test, "Unexpected weight in returned SRV record\n");
+			res = AST_TEST_FAIL;
+		}
+		if (ast_dns_srv_get_port(record) != records[srv_record_order[i]].port) {
+			ast_test_status_update(test, "Unexpected port in returned SRV record\n");
+			res = AST_TEST_FAIL;
+		}
+		if (strcmp(ast_dns_srv_get_host(record), records[srv_record_order[i]].host)) {
+			ast_test_status_update(test, "Unexpected host in returned SRV record\n");
+			res = AST_TEST_FAIL;
+		}
+		++i;
+	}
+
+	if (i != num_records) {
+		ast_test_status_update(test, "Unexpected number of records returned in SRV lookup\n");
+		res = AST_TEST_FAIL;
+	}
+
+cleanup:
+
+	ast_dns_resolver_unregister(&srv_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+AST_TEST_DEFINE(srv_resolve_single_record)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "goose.down" },
+	};
+
+	int srv_record_order[] = { 0, };
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_single_record";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns a single record";
+		info->description = "This test defines a single SRV record and performs a\n"
+			"resolution of the domain to which they belong. The test ensures that all\n"
+			"fields of the SRV record are parsed correctly";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return nominal_test(test, records, srv_record_order, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(srv_resolve_sort_priority)
+{
+	struct srv_record records[] = {
+		{ 20, 10, 5060, "tacos" },
+		{ 10, 10, 5060, "goose.down" },
+	};
+	int srv_record_order[] = { 1, 0};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_sort_priority";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns two records with differing priorities";
+		info->description = "This test defines two SRV records with differing priorities and\n"
+			"performs a resolution of the domain to which they belong. The test ensures that\n"
+			"the two records are sorted according to priority and that all fields of the SRV\n"
+			"records are parsed correctly";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return nominal_test(test, records, srv_record_order, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(srv_resolve_same_priority_zero_weight)
+{
+	struct srv_record records[] = {
+		{ 10, 0, 5060, "tacos" },
+		{ 10, 10, 5060, "goose.down" },
+	};
+	int srv_record_order[] = { 1, 0};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_same_priority_zero_weight";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns two records with same priority but different weights";
+		info->description = "This test defines two SRV records with same priority but different weights and\n"
+			"performs a resolution of the domain to which they belong. The test ensures that\n"
+			"the record with zero weight comes last and that all fields of the SRV\n"
+			"records are parsed correctly";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return nominal_test(test, records, srv_record_order, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(srv_resolve_same_priority_different_weights)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "tacos" },
+		{ 10, 20, 5060, "goose.down" },
+	};
+
+	int srv_record_occurence[2] = { 0, };
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int count = 0;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_same_priority_different_weights";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns two records with same priority but different weights";
+		info->description = "This test defines two SRV records with same priority but different weights and\n"
+			"performs a resolution of the domain to which they belong. The test ensures that\n"
+			"the record with higher weight occurs more often than the one of lesser weight";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	test_records = records;
+	num_test_records = ARRAY_LEN(records);
+
+	ast_dns_resolver_register(&srv_resolver);
+
+	for (count = 0; count < 100; count++) {
+		struct ast_dns_result *result;
+		const struct ast_dns_record *record;
+		int i;
+
+		memset(ans_buffer, 0, sizeof(ans_buffer));
+
+		if (ast_dns_resolve("goose.feathers", T_SRV, C_IN, &result)) {
+			ast_test_status_update(test, "DNS resolution failed\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+
+		if (!result) {
+			ast_test_status_update(test, "DNS resolution returned no result\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+
+		record = ast_dns_result_get_records(result);
+		for (i = 0; i < ARRAY_LEN(records); i++) {
+			if (ast_dns_srv_get_priority(record) != records[i].priority) {
+				continue;
+			}
+			if (ast_dns_srv_get_weight(record) != records[i].weight) {
+				continue;
+			}
+			if (ast_dns_srv_get_port(record) != records[i].port) {
+				continue;
+			}
+			if (strcmp(ast_dns_srv_get_host(record), records[i].host)) {
+				continue;
+			}
+
+			srv_record_occurence[i]++;
+			break;
+		}
+
+		ast_dns_result_free(result);
+	}
+
+	if (srv_record_occurence[0] > srv_record_occurence[1]) {
+		ast_test_status_update(test, "SRV sorting resulted in lesser weight being returned more often\n");
+		res = AST_TEST_FAIL;
+	}
+
+cleanup:
+
+	ast_dns_resolver_unregister(&srv_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+AST_TEST_DEFINE(srv_resolve_different_priorities_different_weights)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "tacos" },
+		{ 10, 20, 5060, "goose.down" },
+		{ 5, 80, 5060, "moo" },
+		{ 5, 10, 5060, "Canada" },
+	};
+
+	int srv_record_priority[4] = { 5, 5, 10, 10 };
+	int srv_record_occurence[4] = { 0, };
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int count = 0;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_different_priorities_different_weights";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns four records with different priority and different weights";
+		info->description = "This test defines four SRV records, two with one priority and two with another priority,\n"
+			"and different weights and performs a resolution of the domain to which they belong.\n"
+			"The test ensures that the priorities are sorted properly and that the records with higher weight\n"
+			"occur more often than the ones of less weight.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	test_records = records;
+	num_test_records = ARRAY_LEN(records);
+
+	ast_dns_resolver_register(&srv_resolver);
+
+	for (count = 0; count < 100; count++) {
+		struct ast_dns_result *result;
+		const struct ast_dns_record *record;
+		int i;
+
+		memset(ans_buffer, 0, sizeof(ans_buffer));
+
+		if (ast_dns_resolve("goose.feathers", T_SRV, C_IN, &result)) {
+			ast_test_status_update(test, "DNS resolution failed\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+
+		if (!result) {
+			ast_test_status_update(test, "DNS resolution returned no result\n");
+			res = AST_TEST_FAIL;
+			goto cleanup;
+		}
+
+		i = 0;
+		for (record = ast_dns_result_get_records(result); record; record = ast_dns_record_get_next(record)) {
+			if (ast_dns_srv_get_priority(record) != srv_record_priority[i]) {
+				ast_test_status_update(test, "Unexpected priority in returned SRV record\n");
+				res = AST_TEST_FAIL;
+			}
+			i++;
+		}
+
+		record = ast_dns_result_get_records(result);
+		for (i = 0; i < ARRAY_LEN(records); i++) {
+			if (ast_dns_srv_get_priority(record) != records[i].priority) {
+				continue;
+			}
+			if (ast_dns_srv_get_weight(record) != records[i].weight) {
+				continue;
+			}
+			if (ast_dns_srv_get_port(record) != records[i].port) {
+				continue;
+			}
+			if (strcmp(ast_dns_srv_get_host(record), records[i].host)) {
+				continue;
+			}
+
+			srv_record_occurence[i]++;
+			break;
+		}
+
+		ast_dns_result_free(result);
+	}
+
+	if (srv_record_occurence[0] > srv_record_occurence[1]) {
+		ast_test_status_update(test, "SRV sorting resulted in lesser weight being returned more often for priority 10\n");
+		res = AST_TEST_FAIL;
+	}
+
+	if (srv_record_occurence[3] > srv_record_occurence[2]) {
+		ast_test_status_update(test, "SRV sorting resulted in lesser weight being returned more often for priority 5\n");
+		res = AST_TEST_FAIL;
+	}
+
+cleanup:
+
+	ast_dns_resolver_unregister(&srv_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+static enum ast_test_result_state invalid_record_test(struct ast_test *test, struct srv_record *records,
+	int num_records)
+{
+	RAII_VAR(struct ast_dns_result *, result, NULL, ast_dns_result_free);
+	const struct ast_dns_record *record;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	test_records = records;
+	num_test_records = num_records;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	ast_dns_resolver_register(&srv_resolver);
+
+	if (ast_dns_resolve("goose.feathers", T_SRV, C_IN, &result)) {
+		ast_test_status_update(test, "DNS resolution failed\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	if (!result) {
+		ast_test_status_update(test, "DNS resolution returned no result\n");
+		res = AST_TEST_FAIL;
+		goto cleanup;
+	}
+
+	record = ast_dns_result_get_records(result);
+	if (record) {
+		ast_test_status_update(test, "Unexpected record returned from SRV query\n");
+		res = AST_TEST_FAIL;
+	}
+
+cleanup:
+
+	ast_dns_resolver_unregister(&srv_resolver);
+
+	test_records = NULL;
+	num_test_records = 0;
+	memset(ans_buffer, 0, sizeof(ans_buffer));
+
+	return res;
+}
+
+AST_TEST_DEFINE(srv_resolve_record_missing_weight_port_host)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "tacos.com", 0, 1, 1, 1 },
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_record_missing_weight_port_host";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns a single invalid record";
+		info->description = "This test defines a single SRV record and performs a\n"
+			"resolution of the domain to which they belong. The test ensures that the\n"
+			"record is determined to be corrupt as it contains only a priority";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return invalid_record_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(srv_resolve_record_missing_port_host)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "tacos.com", 0, 0, 1, 1 },
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_record_missing_port_host";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns a single invalid record";
+		info->description = "This test defines a single SRV record and performs a\n"
+			"resolution of the domain to which they belong. The test ensures that the\n"
+			"record is determined to be corrupt as it contains only a priority and weight";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return invalid_record_test(test, records, ARRAY_LEN(records));
+}
+
+AST_TEST_DEFINE(srv_resolve_record_missing_host)
+{
+	struct srv_record records[] = {
+		{ 10, 10, 5060, "tacos.com", 0, 0, 0, 1 },
+	};
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "srv_resolve_record_missing_host";
+		info->category = "/main/dns/srv/";
+		info->summary = "Test an SRV lookup which returns a single invalid record";
+		info->description = "This test defines a single SRV record and performs a\n"
+			"resolution of the domain to which they belong. The test ensures that the\n"
+			"record is determined to be corrupt as it contains only a priority, weight,\n"
+			"and port";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	return invalid_record_test(test, records, ARRAY_LEN(records));
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(srv_resolve_single_record);
+	AST_TEST_UNREGISTER(srv_resolve_sort_priority);
+	AST_TEST_UNREGISTER(srv_resolve_same_priority_zero_weight);
+	AST_TEST_UNREGISTER(srv_resolve_same_priority_different_weights);
+	AST_TEST_UNREGISTER(srv_resolve_different_priorities_different_weights);
+	AST_TEST_UNREGISTER(srv_resolve_record_missing_weight_port_host);
+	AST_TEST_UNREGISTER(srv_resolve_record_missing_port_host);
+	AST_TEST_UNREGISTER(srv_resolve_record_missing_host);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(srv_resolve_single_record);
+	AST_TEST_REGISTER(srv_resolve_sort_priority);
+	AST_TEST_REGISTER(srv_resolve_same_priority_zero_weight);
+	AST_TEST_REGISTER(srv_resolve_same_priority_different_weights);
+	AST_TEST_REGISTER(srv_resolve_different_priorities_different_weights);
+	AST_TEST_REGISTER(srv_resolve_record_missing_weight_port_host);
+	AST_TEST_REGISTER(srv_resolve_record_missing_port_host);
+	AST_TEST_REGISTER(srv_resolve_record_missing_host);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "DNS SRV Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_endpoints.c 1:16.1.1~dfsg-1/tests/test_endpoints.c
--- 1:13.23.1~dfsg-2/tests/test_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test endpoints.
+ * \file
+ * \brief Test endpoints.
  *
  * \author\verbatim David M. Lee, II <dlee@digium.com> \endverbatim
  *
@@ -31,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/endpoints.h"
 #include "asterisk/module.h"
@@ -152,6 +151,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Endpoint testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_event.c 1:16.1.1~dfsg-1/tests/test_event.c
--- 1:13.23.1~dfsg-2/tests/test_event.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_event.c	2018-12-26 18:45:13.000000000 +0000
@@ -41,8 +41,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/utils.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_expr.c 1:16.1.1~dfsg-1/tests/test_expr.c
--- 1:13.23.1~dfsg-2/tests/test_expr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_expr.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_file.c 1:16.1.1~dfsg-1/tests/test_file.c
--- 1:13.23.1~dfsg-2/tests/test_file.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_file.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,6 @@
 #include <sys/stat.h>
 #include <stdio.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/paths.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_format_cache.c 1:16.1.1~dfsg-1/tests/test_format_cache.c
--- 1:13.23.1~dfsg-2/tests/test_format_cache.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_format_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_format_cap.c 1:16.1.1~dfsg-1/tests/test_format_cap.c
--- 1:13.23.1~dfsg-2/tests/test_format_cap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_format_cap.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/codec.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_func_file.c 1:16.1.1~dfsg-1/tests/test_func_file.c
--- 1:13.23.1~dfsg-2/tests/test_func_file.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_func_file.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/app.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_gosub.c 1:16.1.1~dfsg-1/tests/test_gosub.c
--- 1:13.23.1~dfsg-2/tests/test_gosub.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_gosub.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_hashtab_thrash.c 1:16.1.1~dfsg-1/tests/test_hashtab_thrash.c
--- 1:13.23.1~dfsg-2/tests/test_hashtab_thrash.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_hashtab_thrash.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,7 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include <pthread.h>
 #include "asterisk/hashtab.h"
 #include "asterisk/lock.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_heap.c 1:16.1.1~dfsg-1/tests/test_heap.c
--- 1:13.23.1~dfsg-2/tests/test_heap.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_heap.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/utils.h"
 #include "asterisk/heap.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_http_media_cache.c 1:16.1.1~dfsg-1/tests/test_http_media_cache.c
--- 1:13.23.1~dfsg-2/tests/test_http_media_cache.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_http_media_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,700 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Matt Jordan
+ *
+ * Matt Jordan <mjordan@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Tests for the HTTP media cache backend
+ *
+ * \author \verbatim Matt Jordan <mjordan@digium.com> \endverbatim
+ *
+ * \ingroup tests
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<depend>curl</depend>
+	<depend>res_http_media_cache</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include <fcntl.h>
+
+#include "asterisk/module.h"
+#include "asterisk/http.h"
+#include "asterisk/bucket.h"
+#include "asterisk/test.h"
+
+#define CATEGORY "/res/http_media_cache/"
+
+#define TEST_URI "test_media_cache"
+
+struct test_options {
+	int status_code;
+	int send_file;
+	struct {
+		int s_maxage;
+		int maxage;
+		int no_cache;
+		int must_revalidate;
+	} cache_control;
+	struct timeval expires;
+	const char *status_text;
+	const char *etag;
+};
+
+static struct test_options options;
+
+static char server_uri[512];
+
+#define VALIDATE_EXPIRES(test, bucket_file, expected, delta) do { \
+	RAII_VAR(struct ast_bucket_metadata *, metadata, ast_bucket_file_metadata_get((bucket_file), "__actual_expires"), ao2_cleanup); \
+	int actual_expires; \
+	ast_test_validate(test, metadata != NULL); \
+	ast_test_validate(test, sscanf(metadata->value, "%d", &actual_expires) == 1); \
+	ast_test_status_update(test, "Checking %d >= %d and %d <= %d\n", \
+			(int) ((expected) + (delta)), actual_expires, \
+			(int) ((expected) - (delta)), actual_expires); \
+	ast_test_validate(test, (((expected) + (delta) >= actual_expires) && ((expected) - (delta) <= actual_expires))); \
+} while (0)
+
+#define VALIDATE_STR_METADATA(test, bucket_file, key, expected) do { \
+	RAII_VAR(struct ast_bucket_metadata *, metadata, ast_bucket_file_metadata_get((bucket_file), (key)), ao2_cleanup); \
+	ast_test_validate(test, metadata != NULL); \
+	ast_test_validate(test, !strcmp(metadata->value, (expected))); \
+} while (0)
+
+#define SET_OR_APPEND_CACHE_CONTROL(str) do { \
+	if (!ast_str_strlen((str))) { \
+		ast_str_set(&(str), 0, "%s", "cache-control: "); \
+	} else { \
+		ast_str_append(&(str), 0, "%s", ", "); \
+	} \
+} while (0)
+
+static int http_callback(struct ast_tcptls_session_instance *ser, const struct ast_http_uri *urih, const char *uri, enum ast_http_method method, struct ast_variable *get_params, struct ast_variable *headers)
+{
+	char file_name[64] = "/tmp/test-media-cache-XXXXXX";
+	struct ast_str *http_header = ast_str_create(128);
+	struct ast_str *cache_control = ast_str_create(128);
+	int fd = -1;
+	int unmodified = 0;
+	int send_file = options.send_file && method == AST_HTTP_GET;
+
+	if (!http_header) {
+		goto error;
+	}
+
+	if (send_file) {
+		char buf[1024];
+
+		fd = mkstemp(file_name);
+		if (fd == -1) {
+			ast_log(LOG_ERROR, "Unable to open temp file for testing: %s (%d)", strerror(errno), errno);
+			goto error;
+		}
+
+		memset(buf, 1, sizeof(buf));
+		if (write(fd, buf, sizeof(buf)) != sizeof(buf)) {
+			ast_log(LOG_ERROR, "Failed to write expected number of bytes to pipe\n");
+			close(fd);
+			goto error;
+		}
+		close(fd);
+
+		fd = open(file_name, 0);
+		if (fd == -1) {
+			ast_log(LOG_ERROR, "Unable to open temp file for testing: %s (%d)", strerror(errno), errno);
+			goto error;
+		}
+	}
+
+	if (options.cache_control.maxage) {
+		SET_OR_APPEND_CACHE_CONTROL(cache_control);
+		ast_str_append(&cache_control, 0, "max-age=%d", options.cache_control.maxage);
+	}
+
+	if (options.cache_control.s_maxage) {
+		SET_OR_APPEND_CACHE_CONTROL(cache_control);
+		ast_str_append(&cache_control, 0, "s-maxage=%d", options.cache_control.s_maxage);
+	}
+
+	if (options.cache_control.no_cache) {
+		SET_OR_APPEND_CACHE_CONTROL(cache_control);
+		ast_str_append(&cache_control, 0, "%s", "no-cache");
+	}
+
+	if (options.cache_control.must_revalidate) {
+		SET_OR_APPEND_CACHE_CONTROL(cache_control);
+		ast_str_append(&cache_control, 0, "%s", "must-revalidate");
+	}
+
+	if (ast_str_strlen(cache_control)) {
+		ast_str_append(&http_header, 0, "%s\r\n", ast_str_buffer(cache_control));
+	}
+
+	if (options.expires.tv_sec) {
+		struct ast_tm now_time;
+		char tmbuf[64];
+
+		ast_localtime(&options.expires, &now_time, NULL);
+		ast_strftime(tmbuf, sizeof(tmbuf), "%a, %d %b %Y %T %z", &now_time);
+		ast_str_append(&http_header, 0, "Expires: %s\r\n", tmbuf);
+	}
+
+	if (!ast_strlen_zero(options.etag)) {
+		struct ast_variable *v;
+
+		ast_str_append(&http_header, 0, "ETag: %s\r\n", options.etag);
+		for (v = headers; v; v = v->next) {
+			if (!strcasecmp(v->name, "If-None-Match") && !strcasecmp(v->value, options.etag)) {
+				unmodified = 1;
+				break;
+			}
+		}
+	}
+
+	if (!unmodified) {
+		ast_http_send(ser, method, options.status_code, options.status_text, http_header, NULL, send_file ? fd : 0, 1);
+	} else {
+		ast_http_send(ser, method, 304, "Not Modified", http_header, NULL, 0, 1);
+	}
+
+	if (send_file) {
+		close(fd);
+		unlink(file_name);
+	}
+
+	ast_free(cache_control);
+
+	return 0;
+
+error:
+	ast_free(http_header);
+	ast_free(cache_control);
+	ast_http_request_close_on_completion(ser);
+	ast_http_error(ser, 418, "I'm a Teapot", "Please don't ask me to brew coffee.");
+
+	return 0;
+}
+
+static struct ast_http_uri test_uri = {
+	.description = "HTTP Media Cache Test URI",
+	.uri = TEST_URI,
+	.callback = http_callback,
+	.has_subtree = 1,
+	.data = NULL,
+	.key = __FILE__,
+};
+
+static int pre_test_cb(struct ast_test_info *info, struct ast_test *test)
+{
+	memset(&options, 0, sizeof(options));
+
+	return 0;
+}
+
+static void bucket_file_cleanup(void *obj)
+{
+	struct ast_bucket_file *bucket_file = obj;
+
+	if (bucket_file) {
+		ast_bucket_file_delete(bucket_file);
+		ao2_ref(bucket_file, -1);
+	}
+}
+
+AST_TEST_DEFINE(retrieve_cache_control_directives)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test retrieval of a resource with Cache-Control directives that affect staleness";
+		info->description =
+			"This test covers retrieval of a resource with the Cache-Control header,\n"
+			"which specifies no-cache and/or must-revalidate.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+
+	ast_test_status_update(test, "Testing no-cache...\n");
+	options.cache_control.no_cache = 1;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing no-cache with ETag...\n");
+	options.cache_control.no_cache = 1;
+	options.etag = "123456789";
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	options.etag = NULL;
+
+	ast_test_status_update(test, "Testing no-cache with max-age...\n");
+	options.cache_control.no_cache = 1;
+	options.cache_control.maxage = 300;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+	bucket_file_cleanup(bucket_file);
+
+	options.cache_control.maxage = 0;
+	options.cache_control.no_cache = 0;
+
+	ast_test_status_update(test, "Testing must-revalidate...\n");
+	options.cache_control.must_revalidate = 1;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing must-revalidate with ETag...\n");
+	options.cache_control.must_revalidate = 1;
+	options.etag = "123456789";
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	options.etag = NULL;
+
+	ast_test_status_update(test, "Testing must-revalidate with max-age...\n");
+	options.cache_control.must_revalidate = 1;
+	options.cache_control.maxage = 300;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(retrieve_cache_control_age)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test retrieval of a resource with age specifiers in Cache-Control";
+		info->description =
+			"This test covers retrieval of a resource with the Cache-Control header,\n"
+			"which specifies max-age and/or s-maxage. The test verifies proper precedence\n"
+			"ordering of the header attributes, along with its relation if the Expires\n"
+			"header is present.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+
+	ast_test_status_update(test, "Testing max-age...\n");
+	options.cache_control.maxage = 300;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing s-maxage...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 0;
+	options.cache_control.s_maxage = 300;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing max-age and s-maxage...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 300;
+	options.cache_control.s_maxage = 600;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 600, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing max-age and Expires...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 300;
+	options.cache_control.s_maxage = 0;
+	options.expires.tv_sec = now.tv_sec + 3000;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing s-maxage and Expires...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 0;
+	options.cache_control.s_maxage = 300;
+	options.expires.tv_sec = now.tv_sec + 3000;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing s-maxage and Expires...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 0;
+	options.cache_control.s_maxage = 300;
+	options.expires.tv_sec = now.tv_sec + 3000;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 300, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+	bucket_file_cleanup(bucket_file);
+
+	ast_test_status_update(test, "Testing max-age, s-maxage, and Expires...\n");
+	now = ast_tvnow();
+	options.cache_control.maxage = 300;
+	options.cache_control.s_maxage = 600;
+	options.expires.tv_sec = now.tv_sec + 3000;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 600, 3);
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(retrieve_etag_expired)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test retrieval of an expired resource with an ETag";
+		info->description =
+			"This test covers a staleness check of a resource with an ETag\n"
+			"that has also expired. It guarantees that even if a resource\n"
+			"is expired, we will still not consider it stale if the resource\n"
+			"has not changed per the ETag value.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+	options.etag = "123456789";
+	options.expires.tv_sec = now.tv_sec - 1;
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, !strcmp(uri, ast_sorcery_object_get_id(bucket_file)));
+	ast_test_validate(test, !ast_strlen_zero(bucket_file->path));
+	VALIDATE_STR_METADATA(test, bucket_file, "etag", options.etag);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec - 1, 3);
+
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(retrieve_expires)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test retrieval with explicit expiration";
+		info->description =
+			"This test covers retrieving a resource that has an Expires.\n"
+			"After retrieval of the resource, staleness is checked. With\n"
+			"a non-expired resource, we expect the resource to not be stale.\n"
+			"When the expiration has occurred, we expect the staleness check\n"
+			"to fail.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+	options.expires.tv_sec = now.tv_sec + 3000;
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, !strcmp(uri, ast_sorcery_object_get_id(bucket_file)));
+	ast_test_validate(test, !ast_strlen_zero(bucket_file->path));
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec + 3000, 3);
+
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+
+	/* Clean up previous result */
+	bucket_file_cleanup(bucket_file);
+
+	options.expires.tv_sec = now.tv_sec - 1;
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec - 1, 3);
+
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(retrieve_etag)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test retrieval with an ETag";
+		info->description =
+			"This test covers retrieving a resource that has an ETag.\n"
+			"After retrieval of the resource, staleness is checked. With\n"
+			"matching ETags, we expect the resource to not be stale. When\n"
+			"the ETag does not match, we expect the resource to be stale.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+	options.etag = "123456789";
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, !strcmp(uri, ast_sorcery_object_get_id(bucket_file)));
+	ast_test_validate(test, !ast_strlen_zero(bucket_file->path));
+	VALIDATE_STR_METADATA(test, bucket_file, "etag", options.etag);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec, 3);
+
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 0);
+
+	options.etag = "99999999";
+	ast_test_validate(test, ast_bucket_file_is_stale(bucket_file) == 1);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(retrieve_nominal)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test nominal retrieval";
+		info->description =
+			"Test nominal retrieval of a resource.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	bucket_file = ast_bucket_file_retrieve(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, !strcmp(uri, ast_sorcery_object_get_id(bucket_file)));
+	ast_test_validate(test, !ast_strlen_zero(bucket_file->path));
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec, 3);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(create_nominal)
+{
+	RAII_VAR(struct ast_bucket_file *, bucket_file, NULL, bucket_file_cleanup);
+	struct timeval now = ast_tvnow();
+	char uri[1024];
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test nominal creation";
+		info->description =
+			"Test nominal creation of a resource.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	options.send_file = 1;
+	options.status_code = 200;
+	options.status_text = "OK";
+
+	snprintf(uri, sizeof(uri), "%s/%s", server_uri, "foo.wav");
+
+	bucket_file = ast_bucket_file_alloc(uri);
+	ast_test_validate(test, bucket_file != NULL);
+	ast_test_validate(test, ast_bucket_file_temporary_create(bucket_file) == 0);
+	ast_test_validate(test, ast_bucket_file_create(bucket_file) == 0);
+	VALIDATE_EXPIRES(test, bucket_file, now.tv_sec, 3);
+
+	return AST_TEST_PASS;
+}
+
+
+static int process_config(int reload)
+{
+	struct ast_config *config;
+	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
+	const char *bindaddr;
+	const char *bindport;
+	const char *prefix;
+	const char *enabled;
+
+	config = ast_config_load("http.conf", config_flags);
+	if (!config || config == CONFIG_STATUS_FILEINVALID) {
+		return -1;
+	} else if (config == CONFIG_STATUS_FILEUNCHANGED) {
+		return 0;
+	}
+
+	enabled = ast_config_option(config, "general", "enabled");
+	if (!enabled || ast_false(enabled)) {
+		ast_config_destroy(config);
+		return -1;
+	}
+
+	/* Construct our Server URI */
+	bindaddr = ast_config_option(config, "general", "bindaddr");
+	if (!bindaddr) {
+		ast_config_destroy(config);
+		return -1;
+	}
+
+	bindport = ast_config_option(config, "general", "bindport");
+	if (!bindport) {
+		bindport = "8088";
+	}
+
+	prefix = ast_config_option(config, "general", "prefix");
+
+	snprintf(server_uri, sizeof(server_uri), "http://%s:%s%s/%s", bindaddr, bindport, S_OR(prefix, ""), TEST_URI);
+
+	ast_config_destroy(config);
+
+	return 0;
+}
+
+static int reload_module(void)
+{
+	return process_config(1);
+}
+
+static int load_module(void)
+{
+	if (process_config(0)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	if (ast_http_uri_link(&test_uri)) {
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	AST_TEST_REGISTER(create_nominal);
+
+	AST_TEST_REGISTER(retrieve_nominal);
+	AST_TEST_REGISTER(retrieve_etag);
+	AST_TEST_REGISTER(retrieve_expires);
+	AST_TEST_REGISTER(retrieve_etag_expired);
+	AST_TEST_REGISTER(retrieve_cache_control_age);
+	AST_TEST_REGISTER(retrieve_cache_control_directives);
+
+	ast_test_register_init(CATEGORY, pre_test_cb);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+static int unload_module(void)
+{
+	ast_http_uri_unlink(&test_uri);
+
+	AST_TEST_UNREGISTER(create_nominal);
+
+	AST_TEST_UNREGISTER(retrieve_nominal);
+	AST_TEST_UNREGISTER(retrieve_etag);
+	AST_TEST_UNREGISTER(retrieve_expires);
+	AST_TEST_UNREGISTER(retrieve_etag_expired);
+	AST_TEST_UNREGISTER(retrieve_cache_control_age);
+	AST_TEST_UNREGISTER(retrieve_cache_control_directives);
+
+	return 0;
+}
+
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "HTTP Media Cache Backend Tests",
+		.support_level = AST_MODULE_SUPPORT_CORE,
+		.load = load_module,
+		.reload = reload_module,
+		.unload = unload_module,
+	);
diff -pruN 1:13.23.1~dfsg-2/tests/test_jitterbuf.c 1:16.1.1~dfsg-1/tests/test_jitterbuf.c
--- 1:13.23.1~dfsg-2/tests/test_jitterbuf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_jitterbuf.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_json.c 1:16.1.1~dfsg-1/tests/test_json.c
--- 1:13.23.1~dfsg-2/tests/test_json.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_json.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test JSON API.
+ * \file
+ * \brief Test JSON API.
  *
  * While some of these tests are actually testing our JSON library wrapper, the bulk of
  * them are exploratory tests to determine what the behavior of the underlying JSON
@@ -36,7 +37,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 #include "asterisk/json.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
@@ -1816,5 +1816,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, 0, "JSON testing",
-		.load = load_module,
-		.unload = unload_module);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_linkedlists.c 1:16.1.1~dfsg-1/tests/test_linkedlists.c
--- 1:13.23.1~dfsg-2/tests/test_linkedlists.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_linkedlists.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/test.h"
 #include "asterisk/strings.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_locale.c 1:16.1.1~dfsg-1/tests/test_locale.c
--- 1:13.23.1~dfsg-2/tests/test_locale.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_locale.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <dirent.h>
 #ifndef __USE_GNU
diff -pruN 1:13.23.1~dfsg-2/tests/test_logger.c 1:16.1.1~dfsg-1/tests/test_logger.c
--- 1:13.23.1~dfsg-2/tests/test_logger.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_logger.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_media_cache.c 1:16.1.1~dfsg-1/tests/test_media_cache.c
--- 1:13.23.1~dfsg-2/tests/test_media_cache.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_media_cache.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,413 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2015, Matt Jordan
+ *
+ * Matt Jordan <mjordan@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Tests for the media cache API
+ *
+ * \author \verbatim Matt Jordan <mjordan@digium.com> \endverbatim
+ *
+ * \ingroup tests
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/utils.h"
+#include "asterisk/module.h"
+#include "asterisk/test.h"
+#include "asterisk/bucket.h"
+#include "asterisk/media_cache.h"
+
+/*! The unit test category */
+#define CATEGORY "/main/media_cache/"
+
+/*! A 'valid' resource for the test bucket behind the media cache facade */
+#define VALID_RESOURCE "httptest://localhost:8088/test_media_cache/monkeys.wav"
+
+/*! An 'invalid' resource for the test bucket behind the media cache facade */
+#define INVALID_RESOURCE "httptest://localhost:8088/test_media_cache/bad.wav"
+
+/*! An 'invalid' scheme, not mapping to a valid bucket backend */
+#define INVALID_SCHEME "foo://localhost:8088/test_media_cache/monkeys.wav"
+
+/*! A URI with no scheme */
+#define NO_SCHEME "localhost:8088/test_media_cache/monkeys.wav"
+
+/*!
+ * \internal
+ * \brief Create callback for the httptest bucket backend
+ */
+static int bucket_http_test_wizard_create(const struct ast_sorcery *sorcery, void *data,
+	void *object)
+{
+	if (!strcmp(ast_sorcery_object_get_id(object), VALID_RESOURCE)) {
+		return 0;
+	}
+
+	return -1;
+}
+
+/*!
+ * \internal
+ * \brief Update callback for the httptest bucket backend
+ */
+static int bucket_http_test_wizard_update(const struct ast_sorcery *sorcery, void *data,
+	void *object)
+{
+	if (!strcmp(ast_sorcery_object_get_id(object), VALID_RESOURCE)) {
+		return 0;
+	}
+
+	return -1;
+}
+
+/*!
+ * \internal
+ * \brief Retrieve callback for the httptest bucket backend
+ */
+static void *bucket_http_test_wizard_retrieve_id(const struct ast_sorcery *sorcery,
+	void *data, const char *type, const char *id)
+{
+	struct ast_bucket_file *bucket_file;
+
+	if (!strcmp(type, "file") && !strcmp(id, VALID_RESOURCE)) {
+		bucket_file = ast_bucket_file_alloc(id);
+		if (!bucket_file) {
+			return NULL;
+		}
+
+		ast_bucket_file_temporary_create(bucket_file);
+		return bucket_file;
+	}
+	return NULL;
+}
+
+/*!
+ * \internal
+ * \brief Delete callback for the httptest bucket backend
+ */
+static int bucket_http_test_wizard_delete(const struct ast_sorcery *sorcery, void *data,
+	void *object)
+{
+	if (!strcmp(ast_sorcery_object_get_id(object), VALID_RESOURCE)) {
+		return 0;
+	}
+
+	return -1;
+}
+
+static struct ast_sorcery_wizard bucket_test_wizard = {
+	.name = "httptest",
+	.create = bucket_http_test_wizard_create,
+	.retrieve_id = bucket_http_test_wizard_retrieve_id,
+	.delete = bucket_http_test_wizard_delete,
+};
+
+static struct ast_sorcery_wizard bucket_file_test_wizard = {
+	.name = "httptest",
+	.create = bucket_http_test_wizard_create,
+	.update = bucket_http_test_wizard_update,
+	.retrieve_id = bucket_http_test_wizard_retrieve_id,
+	.delete = bucket_http_test_wizard_delete,
+};
+
+AST_TEST_DEFINE(exists_nominal)
+{
+	int res;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test nominal existance of resources in the cache";
+		info->description =
+			"This test verifies that if a known resource is in the cache, "
+			"calling ast_media_cache_exists will return logical True. If "
+			"a resource does not exist, the same function call will return "
+			"logical False.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	res = ast_media_cache_exists(INVALID_RESOURCE);
+	ast_test_validate(test, res == 0);
+
+	res = ast_media_cache_exists(VALID_RESOURCE);
+	ast_test_validate(test, res == 1);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(exists_off_nominal)
+{
+	int res;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test off nominal existance of resources in the cache";
+		info->description =
+			"This test verifies that checking for bad resources (NULL, bad "
+			"scheme, etc.) does not result in false positivies.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	res = ast_media_cache_exists("");
+	ast_test_validate(test, res != 1);
+
+	res = ast_media_cache_exists(NULL);
+	ast_test_validate(test, res != 1);
+
+	res = ast_media_cache_exists(NO_SCHEME);
+	ast_test_validate(test, res != 1);
+
+	res = ast_media_cache_exists(INVALID_SCHEME);
+	ast_test_validate(test, res != 1);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(create_update_nominal)
+{
+	int res;
+	char file_path[PATH_MAX];
+	char tmp_path_one[PATH_MAX] = "/tmp/test-media-cache-XXXXXX";
+	char tmp_path_two[PATH_MAX] = "/tmp/test-media-cache-XXXXXX";
+	int fd;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test nominal creation/updating of a resource";
+		info->description =
+			"This test creates a resource and associates it with a file. "
+			"It then updates the resource with a new file. In both cases, "
+			"the test verifies that the resource is associated with the "
+			"file.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	/* Create two local files to associate with a resource */
+	fd = mkstemp(tmp_path_one);
+	if (fd < 0) {
+		ast_test_status_update(test, "Failed to create first tmp file: %s\n",
+			tmp_path_one);
+		return AST_TEST_FAIL;
+	}
+	/* We don't need anything in the file */
+	close(fd);
+
+	fd = mkstemp(tmp_path_two);
+	if (fd < 0) {
+		ast_test_status_update(test, "Failed to create second tmp file: %s\n",
+			tmp_path_two);
+		return AST_TEST_FAIL;
+	}
+	close(fd);
+
+	ast_test_status_update(test, "Creating resource with %s\n", tmp_path_one);
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, tmp_path_one, NULL);
+	ast_test_validate(test, res == 0);
+
+	res = ast_media_cache_retrieve(VALID_RESOURCE, NULL, file_path, PATH_MAX);
+	ast_test_status_update(test, "Got %s for first file path\n", file_path);
+	ast_test_validate(test, res == 0);
+	ast_test_validate(test, strcmp(file_path, tmp_path_one) == 0);
+
+	ast_test_status_update(test, "Creating resource with %s\n", tmp_path_two);
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, tmp_path_two, NULL);
+	ast_test_validate(test, res == 0);
+
+	res = ast_media_cache_retrieve(VALID_RESOURCE, NULL, file_path, PATH_MAX);
+	ast_test_status_update(test, "Got %s for second file path\n", file_path);
+	ast_test_validate(test, res == 0);
+	ast_test_validate(test, strcmp(file_path, tmp_path_two) == 0);
+
+	unlink(tmp_path_one);
+	unlink(tmp_path_two);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(create_update_off_nominal)
+{
+	int res;
+	char tmp_path[PATH_MAX] = "/tmp/test-media-cache-XXXXXX";
+	int fd;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test off nominal creation/updating of a resource";
+		info->description =
+			"Test creation/updating of a resource with a variety of invalid\n"
+			"inputs.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	/* Create two local files to associate with a resource */
+	fd = mkstemp(tmp_path);
+	if (fd < 0) {
+		ast_test_status_update(test, "Failed to create first tmp file: %s\n",
+			tmp_path);
+		return AST_TEST_FAIL;
+	}
+	/* We don't need anything in the file */
+	close(fd);
+
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, NULL, NULL);
+	ast_test_validate(test, res != 0);
+
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, "", NULL);
+	ast_test_validate(test, res != 0);
+
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, "I don't exist", NULL);
+	ast_test_validate(test, res != 0);
+
+	res = ast_media_cache_create_or_update(INVALID_RESOURCE, tmp_path, NULL);
+	ast_test_validate(test, res != 0);
+
+	res = ast_media_cache_create_or_update(INVALID_SCHEME, tmp_path, NULL);
+	ast_test_validate(test, res != 0);
+
+	res = ast_media_cache_create_or_update(NO_SCHEME, tmp_path, NULL);
+	ast_test_validate(test, res != 0);
+
+	unlink(tmp_path);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(create_update_metadata)
+{
+	int res;
+	char tmp_path[PATH_MAX] = "/tmp/test-media-cache-XXXXXX";
+	char file_path[PATH_MAX];
+	char actual_metadata[32];
+	struct ast_variable *meta_list = NULL;
+	struct ast_variable *tmp;
+	int fd;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = __func__;
+		info->category = CATEGORY;
+		info->summary = "Test nominal creation/updating of a resource";
+		info->description =
+			"This test creates a resource and associates it with a file. "
+			"It then updates the resource with a new file. In both cases, "
+			"the test verifies that the resource is associated with the "
+			"file.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	/* Create two local files to associate with a resource */
+	fd = mkstemp(tmp_path);
+	if (fd < 0) {
+		ast_test_status_update(test, "Failed to create first tmp file: %s\n",
+			tmp_path);
+		return AST_TEST_FAIL;
+	}
+	/* We don't need anything in the file */
+	close(fd);
+
+	tmp = ast_variable_new("meta1", "value1", __FILE__);
+	if (!tmp) {
+		ast_test_status_update(test, "Failed to create metadata 1 for test\n");
+		return AST_TEST_FAIL;
+	}
+	ast_variable_list_append(&meta_list, tmp);
+
+	tmp = ast_variable_new("meta2", "value2", __FILE__);
+	if (!tmp) {
+		ast_test_status_update(test, "Failed to create metadata 2 for test\n");
+		return AST_TEST_FAIL;
+	}
+	ast_variable_list_append(&meta_list, tmp);
+
+	res = ast_media_cache_create_or_update(VALID_RESOURCE, tmp_path, meta_list);
+	ast_test_validate(test, res == 0);
+
+	res = ast_media_cache_retrieve(VALID_RESOURCE, NULL, file_path, PATH_MAX);
+	ast_test_status_update(test, "Got %s for second file path\n", file_path);
+	ast_test_validate(test, res == 0);
+	ast_test_validate(test, strcmp(file_path, tmp_path) == 0);
+
+	res = ast_media_cache_retrieve_metadata(VALID_RESOURCE, "meta1",
+		actual_metadata, sizeof(actual_metadata));
+	ast_test_validate(test, res == 0);
+	ast_test_validate(test, strcmp(actual_metadata, "value1") == 0);
+
+	res = ast_media_cache_retrieve_metadata(VALID_RESOURCE, "meta2",
+		actual_metadata, sizeof(actual_metadata));
+	ast_test_validate(test, res == 0);
+	ast_test_validate(test, strcmp(actual_metadata, "value2") == 0);
+
+	unlink(tmp_path);
+
+	return AST_TEST_PASS;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(exists_nominal);
+	AST_TEST_UNREGISTER(exists_off_nominal);
+
+	AST_TEST_UNREGISTER(create_update_nominal);
+	AST_TEST_UNREGISTER(create_update_metadata);
+	AST_TEST_UNREGISTER(create_update_off_nominal);
+
+	return 0;
+}
+
+static int load_module(void)
+{
+	if (ast_bucket_scheme_register("httptest", &bucket_test_wizard,
+		&bucket_file_test_wizard, NULL, NULL)) {
+		ast_log(LOG_ERROR, "Failed to register Bucket HTTP test wizard scheme implementation\n");
+		return AST_MODULE_LOAD_DECLINE;
+	}
+
+	AST_TEST_REGISTER(exists_nominal);
+	AST_TEST_REGISTER(exists_off_nominal);
+
+	AST_TEST_REGISTER(create_update_nominal);
+	AST_TEST_REGISTER(create_update_metadata);
+	AST_TEST_REGISTER(create_update_off_nominal);
+
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Media Cache Tests");
diff -pruN 1:13.23.1~dfsg-2/tests/test_message.c 1:16.1.1~dfsg-1/tests/test_message.c
--- 1:13.23.1~dfsg-2/tests/test_message.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_message.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <regex.h>
 
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_netsock2.c 1:16.1.1~dfsg-1/tests/test_netsock2.c
--- 1:13.23.1~dfsg-2/tests/test_netsock2.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_netsock2.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/netsock2.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_optional_api.c 1:16.1.1~dfsg-1/tests/test_optional_api.c
--- 1:13.23.1~dfsg-2/tests/test_optional_api.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_optional_api.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test optional API.
+ * \file
+ * \brief Test optional API.
  *
  * This tests exercise the underlying implementation functions. Acutal usage
  * won't look anything like this; it would use the wrapper macros.
@@ -35,8 +36,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/optional_api.h"
 #include "asterisk/test.h"
@@ -182,6 +181,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "ARI testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	);
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_pbx.c 1:16.1.1~dfsg-1/tests/test_pbx.c
--- 1:13.23.1~dfsg-2/tests/test_pbx.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_pbx.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"
 #include "asterisk/test.h"
@@ -345,8 +343,50 @@ AST_TEST_DEFINE(segv)
 	return AST_TEST_FAIL;
 }
 
+AST_TEST_DEFINE(call_assert)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "CALL_ASSERT";
+		info->category = "/DO_NOT_RUN/";
+		info->summary = "Calls ast_asert()!!! (will only be run if explicitly called)";
+		info->description = "Calls ast_asert()!!! (will only be run if explicitly called). "
+			"This test is mainly used for testing CI and tool failure scenarios.";
+		info->explicit_only = 1;
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_assert(0);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(call_backtrace)
+{
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "CALL_BACKTRACE";
+		info->category = "/DO_NOT_RUN/";
+		info->summary = "Calls ast_log_backtrace()!!! (will only be run if explicitly called)";
+		info->description = "Calls ast_log_backtrace()!!! (will only be run if explicitly called). "
+			"This test is mainly used for testing CI and tool failure scenarios.";
+		info->explicit_only = 1;
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_log_backtrace();
+
+	return AST_TEST_PASS;
+}
+
 static int unload_module(void)
 {
+	AST_TEST_UNREGISTER(call_backtrace);
+	AST_TEST_UNREGISTER(call_assert);
 	AST_TEST_UNREGISTER(segv);
 	AST_TEST_UNREGISTER(pattern_match_test);
 	return 0;
@@ -356,6 +396,8 @@ static int load_module(void)
 {
 	AST_TEST_REGISTER(pattern_match_test);
 	AST_TEST_REGISTER(segv);
+	AST_TEST_REGISTER(call_assert);
+	AST_TEST_REGISTER(call_backtrace);
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
diff -pruN 1:13.23.1~dfsg-2/tests/test_poll.c 1:16.1.1~dfsg-1/tests/test_poll.c
--- 1:13.23.1~dfsg-2/tests/test_poll.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_poll.c	2018-12-26 18:45:13.000000000 +0000
@@ -39,8 +39,6 @@
 #include <errno.h>
 #include <unistd.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_res_pjsip_scheduler.c 1:16.1.1~dfsg-1/tests/test_res_pjsip_scheduler.c
--- 1:13.23.1~dfsg-2/tests/test_res_pjsip_scheduler.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_res_pjsip_scheduler.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_REGISTER_FILE()
-
 #include <pjsip.h>
 #include "asterisk/test.h"
 #include "asterisk/module.h"
@@ -55,6 +53,7 @@ struct test_data {
 	int interval;
 	int sleep;
 	int done;
+	int no_clear_done;
 	struct ast_test *test;
 };
 
@@ -65,7 +64,9 @@ static int task_1(void *data)
 {
 	struct test_data *test = data;
 
-	test->done = 0;
+	if (!test->no_clear_done) {
+		test->done = 0;
+	}
 	test->task_start = ast_tvnow();
 	test->tid = pthread_self();
 	test->is_servant = ast_sip_thread_is_servant();
@@ -73,7 +74,7 @@ static int task_1(void *data)
 	test->task_end = ast_tvnow();
 
 	ast_mutex_lock(&test->lock);
-	test->done = 1;
+	test->done++;
 	ast_mutex_unlock(&test->lock);
 	ast_cond_signal(&test->cond);
 
@@ -129,7 +130,7 @@ static int scheduler(struct ast_test *te
 	if (serialized) {
 		ast_test_status_update(test, "This test will take about %3.1f seconds\n",
 			(test_data1->interval + test_data1->sleep + (MAX(test_data1->interval - test_data2->interval, 0)) + test_data2->sleep) / 1000.0);
-		tp1 = ast_sip_create_serializer();
+		tp1 = ast_sip_create_serializer("test-scheduler-serializer");
 		ast_test_validate(test, (tp1 != NULL));
 	} else {
 		ast_test_status_update(test, "This test will take about %3.1f seconds\n",
@@ -347,11 +348,12 @@ AST_TEST_DEFINE(scheduler_policy)
 	test_data1->test_start = ast_tvnow();
 	test_data1->interval = 1000;
 	test_data1->sleep = 500;
+	test_data1->no_clear_done = 1;
 	ast_mutex_init(&test_data1->lock);
 	ast_cond_init(&test_data1->cond, NULL);
 
 	ast_test_status_update(test, "This test will take about %3.1f seconds\n",
-		((test_data1->interval * 3) + test_data1->sleep) / 1000.0);
+		((test_data1->interval * 4) + test_data1->sleep) / 1000.0);
 
 	task = ast_sip_schedule_task(NULL, test_data1->interval, task_1, "test_1", test_data1,
 		AST_SIP_SCHED_TASK_DATA_NO_CLEANUP | AST_SIP_SCHED_TASK_PERIODIC);
@@ -370,16 +372,39 @@ AST_TEST_DEFINE(scheduler_policy)
 	ast_test_validate(test, when > test_data1->interval * 3 * 0.9 && when < test_data1->interval * 3 * 1.1);
 
 	ast_sip_sched_task_cancel(task);
-	ao2_ref(task, -1);
-	task = NULL;
+
+	/* Wait a full interval in case a 4th call to test_1 happened before the cancel */
+	usleep(M2U(test_data1->interval));
+
+	ast_mutex_lock(&test_data1->lock);
+	if (test_data1->done) {
+		int done = test_data1->done;
+
+		test_data1->done = 0;
+		ast_mutex_unlock(&test_data1->lock);
+
+		ast_test_validate(test, done == 1);
+
+		/* Wait two full intervals to be certain no further calls to test_1. */
+		usleep(M2U(test_data1->interval * 2));
+
+		ast_mutex_lock(&test_data1->lock);
+		if (test_data1->done != 0) {
+			ast_mutex_unlock(&test_data1->lock);
+			/* The cancelation failed so we need to prevent cleanup of
+			 * test_data1 to prevent a crash from write-after-free. */
+			test_data1 = NULL;
+			ast_test_status_update(test, "Failed to cancel task");
+			return AST_TEST_FAIL;
+		}
+	}
+	ast_mutex_unlock(&test_data1->lock);
 
 	return AST_TEST_PASS;
 }
 
 static int load_module(void)
 {
-	CHECK_PJSIP_MODULE_LOADED();
-
 	AST_TEST_REGISTER(serialized_scheduler);
 	AST_TEST_REGISTER(unserialized_scheduler);
 	AST_TEST_REGISTER(scheduler_cleanup);
@@ -398,4 +423,9 @@ static int unload_module(void)
 	return 0;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "res_pjsip scheduler test module");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "res_pjsip scheduler test module",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_pjsip",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_res_stasis.c 1:16.1.1~dfsg-1/tests/test_res_stasis.c
--- 1:13.23.1~dfsg-2/tests/test_res_stasis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_res_stasis.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test Stasis Application API.
+ * \file
+ * \brief Test Stasis Application API.
  * \author\verbatim David M. Lee, II <dlee@digium.com> \endverbatim
  *
  * \ingroup tests
@@ -31,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/test.h"
 #include "asterisk/stasis_app.h"
@@ -180,13 +179,11 @@ static int unload_module(void)
 	AST_TEST_UNREGISTER(app_invoke_dne);
 	AST_TEST_UNREGISTER(app_invoke_one);
 	AST_TEST_UNREGISTER(app_replaced);
-	stasis_app_unref();
 	return 0;
 }
 
 static int load_module(void)
 {
-	stasis_app_ref();
 	AST_TEST_REGISTER(app_replaced);
 	AST_TEST_REGISTER(app_invoke_one);
 	AST_TEST_REGISTER(app_invoke_dne);
@@ -194,7 +191,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Stasis Core testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis",
-	);
+	.requires = "res_stasis",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_sched.c 1:16.1.1~dfsg-1/tests/test_sched.c
--- 1:13.23.1~dfsg-2/tests/test_sched.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_sched.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include <inttypes.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/utils.h"
 #include "asterisk/sched.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_scoped_lock.c 1:16.1.1~dfsg-1/tests/test_scoped_lock.c
--- 1:13.23.1~dfsg-2/tests/test_scoped_lock.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_scoped_lock.c	2018-12-26 18:45:13.000000000 +0000
@@ -199,8 +199,8 @@ AST_TEST_DEFINE(cleanup_order)
 	enum ast_test_result_state res = AST_TEST_PASS;
 	struct ao2_iterator iter;
 	struct test_struct *object_iter;
-	RAII_VAR(struct ao2_container*, container, ao2_container_alloc(13, NULL, NULL), ao2_cleanup);
-	RAII_VAR(struct test_struct *, object, ao2_alloc(sizeof(*object), NULL), ao2_cleanup);
+	RAII_VAR(struct ao2_container*, container, NULL, ao2_cleanup);
+	RAII_VAR(struct test_struct *, object, NULL, ao2_cleanup);
 
 	switch(cmd) {
 	case TEST_INIT:
@@ -216,6 +216,8 @@ AST_TEST_DEFINE(cleanup_order)
 	}
 	current_test = test;
 
+	container = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_MUTEX, 0, 13, NULL, NULL, NULL);
+	object = ao2_alloc(sizeof(*object), NULL);
 	if (!object || !container) {
 		/* Allocation failure. We can't even pretend to do this test properly */
 		return AST_TEST_FAIL;
diff -pruN 1:13.23.1~dfsg-2/tests/test_security_events.c 1:16.1.1~dfsg-1/tests/test_security_events.c
--- 1:13.23.1~dfsg-2/tests/test_security_events.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_security_events.c	2018-12-26 18:45:13.000000000 +0000
@@ -30,8 +30,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/module.h"
 #include "asterisk/cli.h"
 #include "asterisk/utils.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_skel.c 1:16.1.1~dfsg-1/tests/test_skel.c
--- 1:13.23.1~dfsg-2/tests/test_skel.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_skel.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_sorcery_astdb.c 1:16.1.1~dfsg-1/tests/test_sorcery_astdb.c
--- 1:13.23.1~dfsg-2/tests/test_sorcery_astdb.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_sorcery_astdb.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/sorcery.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_sorcery.c 1:16.1.1~dfsg-1/tests/test_sorcery.c
--- 1:13.23.1~dfsg-2/tests/test_sorcery.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_sorcery.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/astobj2.h"
@@ -138,6 +136,9 @@ struct sorcery_test_caching {
 	/*! \brief Whether the object has been deleted from the cache or not */
 	unsigned int deleted:1;
 
+	/*! \brief Whether the object is stale or not */
+	unsigned int is_stale:1;
+
 	/*! \brief Object to return when asked */
 	struct test_sorcery_object object;
 };
@@ -217,6 +218,12 @@ static int sorcery_test_delete(const str
 	return 0;
 }
 
+static int sorcery_test_is_stale(const struct ast_sorcery *sorcery, void *data, void *object)
+{
+	cache.is_stale = 1;
+	return 1;
+}
+
 /*! \brief Dummy sorcery wizards, not actually used so we only populate the name and nothing else */
 static struct ast_sorcery_wizard test_wizard = {
 	.name = "test",
@@ -234,6 +241,7 @@ static struct ast_sorcery_wizard test_wi
 	.retrieve_id = sorcery_test_retrieve_id,
 	.update = sorcery_test_update,
 	.delete = sorcery_test_delete,
+	.is_stale = sorcery_test_is_stale,
 };
 
 static void sorcery_observer_created(const void *object)
@@ -2223,6 +2231,84 @@ AST_TEST_DEFINE(object_delete_uncreated)
 	return AST_TEST_PASS;
 }
 
+AST_TEST_DEFINE(object_is_stale)
+{
+	RAII_VAR(struct ast_sorcery *, sorcery, NULL, ast_sorcery_unref);
+	RAII_VAR(struct ast_sorcery_wizard *, wizard1, &test_wizard, ast_sorcery_wizard_unregister);
+	RAII_VAR(struct ast_sorcery_wizard *, wizard2, &test_wizard2, ast_sorcery_wizard_unregister);
+	RAII_VAR(struct test_sorcery_object *, obj1, NULL, ao2_cleanup);
+	RAII_VAR(struct test_sorcery_object *, obj2, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "object_is_stale";
+		info->category = "/main/sorcery/";
+		info->summary = "sorcery object staleness unit test";
+		info->description =
+			"Test whether sorcery will query a wizard correctly if asked\n"
+			"if an object is stale.";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	if (ast_sorcery_wizard_register(&test_wizard)) {
+		ast_test_status_update(test, "Failed to register a perfectly valid sorcery wizard\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_sorcery_wizard_register(&test_wizard2)) {
+		ast_test_status_update(test, "Failed to register a perfectly valid sorcery wizard\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!(sorcery = ast_sorcery_open())) {
+		ast_test_status_update(test, "Failed to open sorcery structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if ((ast_sorcery_apply_default(sorcery, "test", "test", NULL) != AST_SORCERY_APPLY_SUCCESS) ||
+		ast_sorcery_internal_object_register(sorcery, "test", test_sorcery_object_alloc, NULL, NULL)) {
+		return AST_TEST_FAIL;
+	}
+
+	ast_sorcery_object_field_register_nodoc(sorcery, "test", "bob", "5", OPT_UINT_T, 0, FLDSET(struct test_sorcery_object, bob));
+	ast_sorcery_object_field_register_nodoc(sorcery, "test", "joe", "10", OPT_UINT_T, 0, FLDSET(struct test_sorcery_object, joe));
+	ast_sorcery_object_field_register_custom_nodoc(sorcery, "test", "jim", "444", jim_handler, NULL, jim_vl, 0, 0);
+	ast_sorcery_object_field_register_custom_nodoc(sorcery, "test", "jack", "888,999", jack_handler, jack_str, NULL, 0, 0);
+
+
+	if ((ast_sorcery_apply_default(sorcery, "test2", "test2", "test2data") != AST_SORCERY_APPLY_SUCCESS) ||
+		ast_sorcery_internal_object_register(sorcery, "test2", test_sorcery_object_alloc, NULL, NULL)) {
+		return AST_TEST_FAIL;
+	}
+
+	ast_sorcery_object_field_register_nodoc(sorcery, "test2", "bob", "5", OPT_UINT_T, 0, FLDSET(struct test_sorcery_object, bob));
+	ast_sorcery_object_field_register_nodoc(sorcery, "test2", "joe", "10", OPT_UINT_T, 0, FLDSET(struct test_sorcery_object, joe));
+	ast_sorcery_object_field_register_custom_nodoc(sorcery, "test2", "jim", "444", jim_handler, NULL, jim_vl, 0, 0);
+	ast_sorcery_object_field_register_custom_nodoc(sorcery, "test2", "jack", "888,999", jack_handler, jack_str, NULL, 0, 0);
+
+
+	if (!(obj1 = ast_sorcery_alloc(sorcery, "test", "blah"))) {
+		ast_test_status_update(test, "Failed to allocate a known object type\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (!(obj2 = ast_sorcery_alloc(sorcery, "test2", "blah"))) {
+		ast_test_status_update(test, "Failed to allocate a known object type\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* The 'test' wizard has no is_stale callback */
+	ast_test_validate(test, ast_sorcery_is_stale(sorcery, obj1) == 0);
+
+	/* The 'test2' wizard should return stale */
+	ast_test_validate(test, ast_sorcery_is_stale(sorcery, obj2) == 1);
+	ast_test_validate(test, cache.is_stale == 1);
+
+	return AST_TEST_PASS;
+}
+
 AST_TEST_DEFINE(caching_wizard_behavior)
 {
 	struct ast_flags flags = { CONFIG_FLAG_NOCACHE };
@@ -3505,6 +3591,7 @@ static int unload_module(void)
 	AST_TEST_UNREGISTER(object_update_uncreated);
 	AST_TEST_UNREGISTER(object_delete);
 	AST_TEST_UNREGISTER(object_delete_uncreated);
+	AST_TEST_UNREGISTER(object_is_stale);
 	AST_TEST_UNREGISTER(caching_wizard_behavior);
 	AST_TEST_UNREGISTER(object_type_observer);
 	AST_TEST_UNREGISTER(configuration_file_wizard);
@@ -3561,6 +3648,7 @@ static int load_module(void)
 	AST_TEST_REGISTER(object_update_uncreated);
 	AST_TEST_REGISTER(object_delete);
 	AST_TEST_REGISTER(object_delete_uncreated);
+	AST_TEST_REGISTER(object_is_stale);
 	AST_TEST_REGISTER(caching_wizard_behavior);
 	AST_TEST_REGISTER(object_type_observer);
 	AST_TEST_REGISTER(configuration_file_wizard);
diff -pruN 1:13.23.1~dfsg-2/tests/test_sorcery_realtime.c 1:16.1.1~dfsg-1/tests/test_sorcery_realtime.c
--- 1:13.23.1~dfsg-2/tests/test_sorcery_realtime.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_sorcery_realtime.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/sorcery.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_stasis.c 1:16.1.1~dfsg-1/tests/test_stasis.c
--- 1:13.23.1~dfsg-2/tests/test_stasis.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_stasis.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test Stasis message bus.
+ * \file
+ * \brief Test Stasis message bus.
  *
  * \author\verbatim David M. Lee, II <dlee@digium.com> \endverbatim
  *
@@ -31,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis.h"
@@ -2104,6 +2103,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, 0, "Stasis testing",
-		.load = load_module,
-		.unload = unload_module
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_stasis_channels.c 1:16.1.1~dfsg-1/tests/test_stasis_channels.c
--- 1:13.23.1~dfsg-2/tests/test_stasis_channels.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_stasis_channels.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test Stasis Channel messages and objects
+ * \file
+ * \brief Test Stasis Channel messages and objects
  *
  * \author\verbatim Matt Jordan <mjordan@digium.com> \endverbatim
  *
@@ -31,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/module.h"
 #include "asterisk/stasis.h"
@@ -328,6 +327,7 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, 0, "Stasis Channel Testing",
-		.load = load_module,
-		.unload = unload_module
-	);
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_stasis_endpoints.c 1:16.1.1~dfsg-1/tests/test_stasis_endpoints.c
--- 1:13.23.1~dfsg-2/tests/test_stasis_endpoints.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_stasis_endpoints.c	2018-12-26 18:45:13.000000000 +0000
@@ -17,7 +17,8 @@
  */
 
 /*!
- * \file \brief Test endpoints.
+ * \file
+ * \brief Test endpoints.
  *
  * \author\verbatim David M. Lee, II <dlee@digium.com> \endverbatim
  *
@@ -32,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/astobj2.h"
 #include "asterisk/channel.h"
 #include "asterisk/endpoints.h"
@@ -305,7 +304,8 @@ static int load_module(void)
 }
 
 AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Endpoint stasis-related testing",
+	.support_level = AST_MODULE_SUPPORT_CORE,
 	.load = load_module,
 	.unload = unload_module,
-	.nonoptreq = "res_stasis_test",
-	);
+	.requires = "res_stasis_test",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_stream.c 1:16.1.1~dfsg-1/tests/test_stream.c
--- 1:13.23.1~dfsg-2/tests/test_stream.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_stream.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,2314 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2017, Digium, Inc.
+ *
+ * Joshua Colp <jcolp@digium.com>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * \brief Media Stream API Unit Tests
+ *
+ * \author Joshua Colp <jcolp@digium.com>
+ *
+ */
+
+/*** MODULEINFO
+	<depend>TEST_FRAMEWORK</depend>
+	<support_level>core</support_level>
+ ***/
+
+#include "asterisk.h"
+
+#include "asterisk/test.h"
+#include "asterisk/module.h"
+#include "asterisk/stream.h"
+#include "asterisk/format.h"
+#include "asterisk/format_cap.h"
+#include "asterisk/format_cache.h"
+#include "asterisk/channel.h"
+#include "asterisk/uuid.h"
+
+AST_TEST_DEFINE(stream_create)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_create";
+		info->category = "/main/stream/";
+		info->summary = "stream create unit test";
+		info->description =
+			"Test that creating a stream results in a stream with the expected values";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	stream = ast_stream_alloc("test", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_state(stream) != AST_STREAM_STATE_INACTIVE) {
+		ast_test_status_update(test, "Newly created stream does not have expected inactive stream state\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO) {
+		ast_test_status_update(test, "Newly created stream does not have expected audio media type\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (strcmp(ast_stream_get_name(stream), "test")) {
+		ast_test_status_update(test, "Newly created stream does not have expected name of test\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_create_no_name)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_create_no_name";
+		info->category = "/main/stream/";
+		info->summary = "stream create (without a name) unit test";
+		info->description =
+			"Test that creating a stream with no name works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	stream = ast_stream_alloc(NULL, AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_set_type)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_set_type";
+		info->category = "/main/stream/";
+		info->summary = "stream type setting unit test";
+		info->description =
+			"Test that changing the type of a stream works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	stream = ast_stream_alloc("test", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO) {
+		ast_test_status_update(test, "Newly created stream does not have expected audio media type\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_set_type(stream, AST_MEDIA_TYPE_VIDEO);
+
+	if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_VIDEO) {
+		ast_test_status_update(test, "Changed stream does not have expected video media type\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_set_formats)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_set_formats";
+		info->category = "/main/stream/";
+		info->summary = "stream formats setting unit test";
+		info->description =
+			"Test that changing the formats of a stream works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Failed to create a format capabilities structure for testing\n");
+		return AST_TEST_FAIL;
+	}
+
+	stream = ast_stream_alloc("test", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_set_formats(stream, caps);
+
+	if (ast_stream_get_formats(stream) != caps) {
+		ast_test_status_update(test, "Changed stream does not have expected formats\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_set_formats(stream, NULL);
+
+	if (ast_stream_get_formats(stream)) {
+		ast_test_status_update(test, "Retrieved formats from stream despite removing them\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_set_state)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_set_state";
+		info->category = "/main/stream/";
+		info->summary = "stream state setting unit test";
+		info->description =
+			"Test that changing the state of a stream works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	stream = ast_stream_alloc("test", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_state(stream) != AST_STREAM_STATE_INACTIVE) {
+		ast_test_status_update(test, "Newly created stream does not have expected inactive stream state\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_set_state(stream, AST_STREAM_STATE_SENDRECV);
+
+	if (ast_stream_get_state(stream) != AST_STREAM_STATE_SENDRECV) {
+		ast_test_status_update(test, "Changed stream does not have expected sendrecv state\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_metadata)
+{
+	RAII_VAR(struct ast_stream *, stream, NULL, ast_stream_free);
+	char track_label[AST_UUID_STR_LEN + 1];
+	const char *stream_track_label;
+	int rc;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_metadata";
+		info->category = "/main/stream/";
+		info->summary = "stream metadata unit test";
+		info->description =
+			"Test that metadata operations on a stream works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	stream = ast_stream_alloc("test", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create media stream given proper arguments\n");
+		return AST_TEST_FAIL;
+	}
+
+	stream_track_label = ast_stream_get_metadata(stream, "AST_STREAM_METADATA_TRACK_LABEL");
+	if (stream_track_label) {
+		ast_test_status_update(test, "New stream HAD a track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_uuid_generate_str(track_label, sizeof(track_label));
+	rc = ast_stream_set_metadata(stream, "AST_STREAM_METADATA_TRACK_LABEL", track_label);
+	if (rc != 0) {
+		ast_test_status_update(test, "Failed to add track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	stream_track_label = ast_stream_get_metadata(stream, "AST_STREAM_METADATA_TRACK_LABEL");
+	if (!stream_track_label) {
+		ast_test_status_update(test, "Changed stream does not have a track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (strcmp(stream_track_label, track_label) != 0) {
+		ast_test_status_update(test, "Changed stream did not return same track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	rc = ast_stream_set_metadata(stream, "AST_STREAM_METADATA_TRACK_LABEL", NULL);
+	if (rc != 0) {
+		ast_test_status_update(test, "Failed to remove track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	stream_track_label = ast_stream_get_metadata(stream, "AST_STREAM_METADATA_TRACK_LABEL");
+	if (stream_track_label) {
+		ast_test_status_update(test, "Changed stream still had a track label after we removed it\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_create)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_create";
+		info->category = "/main/stream/";
+		info->summary = "stream topology creation unit test";
+		info->description =
+			"Test that creating a stream topology works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_clone)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	RAII_VAR(struct ast_stream_topology *, cloned, NULL, ast_stream_topology_free);
+	struct ast_stream *audio_stream, *video_stream;
+	char audio_track_label[AST_UUID_STR_LEN + 1];
+	char video_track_label[AST_UUID_STR_LEN + 1];
+	const char *original_track_label;
+	const char *cloned_track_label;
+	int rc;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_clone";
+		info->category = "/main/stream/";
+		info->summary = "stream topology cloning unit test";
+		info->description =
+			"Test that cloning a stream topology results in a clone with the same contents";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	audio_stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!audio_stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_uuid_generate_str(audio_track_label, sizeof(audio_track_label));
+	rc = ast_stream_set_metadata(audio_stream, "AST_STREAM_METADATA_TRACK_LABEL", audio_track_label);
+	if (rc != 0) {
+		ast_test_status_update(test, "Failed to add track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, audio_stream) == -1) {
+		ast_test_status_update(test, "Failed to append valid audio stream to stream topology\n");
+		ast_stream_free(audio_stream);
+		return AST_TEST_FAIL;
+	}
+
+	video_stream = ast_stream_alloc("video", AST_MEDIA_TYPE_VIDEO);
+	if (!video_stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_uuid_generate_str(video_track_label, sizeof(video_track_label));
+	rc = ast_stream_set_metadata(video_stream, "AST_STREAM_METADATA_TRACK_LABEL", video_track_label);
+	if (rc != 0) {
+		ast_test_status_update(test, "Failed to add track label\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, video_stream) == -1) {
+		ast_test_status_update(test, "Failed to append valid video stream to stream topology\n");
+		ast_stream_free(video_stream);
+		return AST_TEST_FAIL;
+	}
+
+	cloned = ast_stream_topology_clone(topology);
+	if (!cloned) {
+		ast_test_status_update(test, "Failed to clone a perfectly good stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(cloned) != ast_stream_topology_get_count(topology)) {
+		ast_test_status_update(test, "Cloned stream topology does not contain same number of streams as original\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(cloned, 0)) != ast_stream_get_type(ast_stream_topology_get_stream(topology, 0))) {
+		ast_test_status_update(test, "Cloned audio stream does not contain same type as original\n");
+		return AST_TEST_FAIL;
+	}
+
+	original_track_label = ast_stream_get_metadata(ast_stream_topology_get_stream(topology, 0),
+		"AST_STREAM_METADATA_TRACK_LABEL");
+	if (!original_track_label) {
+		ast_test_status_update(test, "Original topology stream 0 does not contain metadata\n");
+		return AST_TEST_FAIL;
+	}
+	cloned_track_label = ast_stream_get_metadata(ast_stream_topology_get_stream(cloned, 0),
+		"AST_STREAM_METADATA_TRACK_LABEL");
+	if (!cloned_track_label) {
+		ast_test_status_update(test, "Cloned topology stream 0 does not contain metadata\n");
+		return AST_TEST_FAIL;
+	}
+	if (strcmp(original_track_label, cloned_track_label) != 0) {
+		ast_test_status_update(test, "Cloned topology stream 0 track label was not the same as the original\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(cloned, 1)) != ast_stream_get_type(ast_stream_topology_get_stream(topology, 1))) {
+		ast_test_status_update(test, "Cloned video stream does not contain same type as original\n");
+		return AST_TEST_FAIL;
+	}
+
+	original_track_label = ast_stream_get_metadata(ast_stream_topology_get_stream(topology, 1),
+		"AST_STREAM_METADATA_TRACK_LABEL");
+	if (!original_track_label) {
+		ast_test_status_update(test, "Original topology stream 1 does not contain metadata\n");
+		return AST_TEST_FAIL;
+	}
+	cloned_track_label = ast_stream_get_metadata(ast_stream_topology_get_stream(cloned, 1),
+		"AST_STREAM_METADATA_TRACK_LABEL");
+	if (!cloned_track_label) {
+		ast_test_status_update(test, "Cloned topology stream 1 does not contain metadata\n");
+		return AST_TEST_FAIL;
+	}
+	if (strcmp(original_track_label, cloned_track_label) != 0) {
+		ast_test_status_update(test, "Cloned topology stream 1 track label was not the same as the original\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_append_stream)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_stream *audio_stream, *video_stream;
+	int position;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_append_stream";
+		info->category = "/main/stream/";
+		info->summary = "stream topology stream appending unit test";
+		info->description =
+			"Test that appending streams to a stream topology works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	audio_stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!audio_stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	position = ast_stream_topology_append_stream(topology, audio_stream);
+	if (position == -1) {
+		ast_test_status_update(test, "Failed to append valid audio stream to stream topology\n");
+		ast_stream_free(audio_stream);
+		return AST_TEST_FAIL;
+	} else if (position != 0) {
+		ast_test_status_update(test, "Appended audio stream to stream topology but position is '%d' instead of 0\n",
+			position);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 1) {
+		ast_test_status_update(test, "Appended an audio stream to the stream topology but stream count is '%d' on it, not 1\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_stream(topology, 0) != audio_stream) {
+		ast_test_status_update(test, "Appended an audio stream to the stream topology but returned stream doesn't match\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_position(audio_stream) != 0) {
+		ast_test_status_update(test, "Appended audio stream says it is at position '%d' instead of 0\n",
+			ast_stream_get_position(audio_stream));
+		return AST_TEST_FAIL;
+	}
+
+	video_stream = ast_stream_alloc("video", AST_MEDIA_TYPE_VIDEO);
+	if (!video_stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	position = ast_stream_topology_append_stream(topology, video_stream);
+	if (position == -1) {
+		ast_test_status_update(test, "Failed to append valid video stream to stream topology\n");
+		ast_stream_free(video_stream);
+		return AST_TEST_FAIL;
+	} else if (position != 1) {
+		ast_test_status_update(test, "Appended video stream to stream topology but position is '%d' instead of 1\n",
+			position);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 2) {
+		ast_test_status_update(test, "Appended a video stream to the stream topology but stream count is '%d' on it, not 2\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_stream(topology, 1) != video_stream) {
+		ast_test_status_update(test, "Appended a video stream to the stream topology but returned stream doesn't match\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_position(video_stream) != 1) {
+		ast_test_status_update(test, "Appended video stream says it is at position '%d' instead of 1\n",
+			ast_stream_get_position(video_stream));
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_set_stream)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_stream *audio_stream, *video_stream;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_set_stream";
+		info->category = "/main/stream/";
+		info->summary = "stream topology stream setting unit test";
+		info->description =
+			"Test that setting streams at a specific position in a topology works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	audio_stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!audio_stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_set_stream(topology, 0, audio_stream)) {
+		ast_test_status_update(test, "Failed to set an audio stream to a position where it is permitted\n");
+		ast_stream_free(audio_stream);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 1) {
+		ast_test_status_update(test, "Set an audio stream on the stream topology but stream count is '%d' on it, not 1\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_stream(topology, 0) != audio_stream) {
+		ast_test_status_update(test, "Set an audio stream on the stream topology but returned stream doesn't match\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_position(audio_stream) != 0) {
+		ast_test_status_update(test, "Set audio stream says it is at position '%d' instead of 0\n",
+			ast_stream_get_position(audio_stream));
+		return AST_TEST_FAIL;
+	}
+
+	video_stream = ast_stream_alloc("video", AST_MEDIA_TYPE_VIDEO);
+	if (!video_stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_set_stream(topology, 0, video_stream)) {
+		ast_test_status_update(test, "Failed to set a video stream to a position where it is permitted\n");
+		ast_stream_free(video_stream);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 1) {
+		ast_test_status_update(test, "Set a video stream on the stream topology but stream count is '%d' on it, not 1\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_stream(topology, 0) != video_stream) {
+		ast_test_status_update(test, "Set a video stream on the stream topology but returned stream doesn't match\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_position(video_stream) != 0) {
+		ast_test_status_update(test, "Set video stream says it is at position '%d' instead of 0\n",
+			ast_stream_get_position(video_stream));
+		return AST_TEST_FAIL;
+	}
+
+	audio_stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!audio_stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_set_stream(topology, 1, audio_stream)) {
+		ast_test_status_update(test, "Failed to set an audio stream to a position where it is permitted\n");
+		ast_stream_free(audio_stream);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 2) {
+		ast_test_status_update(test, "Set an audio stream on the stream topology but stream count is '%d' on it, not 2\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_stream(topology, 1) != audio_stream) {
+		ast_test_status_update(test, "Set an audio stream on the stream topology but returned stream doesn't match\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_position(audio_stream) != 1) {
+		ast_test_status_update(test, "Set audio stream says it is at position '%d' instead of 1\n",
+			ast_stream_get_position(audio_stream));
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+static int check_stream_positions(struct ast_test *test, const struct ast_stream_topology *topology)
+{
+	const struct ast_stream *stream;
+	int idx;
+	int pos;
+	enum ast_media_type type;
+
+	for (idx = 0; idx < ast_stream_topology_get_count(topology); ++idx) {
+		stream = ast_stream_topology_get_stream(topology, idx);
+		pos = ast_stream_get_position(stream);
+		if (idx != pos) {
+			type = ast_stream_get_type(stream);
+			ast_test_status_update(test, "Failed: '%s' stream says it is at position %d instead of %d\n",
+				ast_codec_media_type2str(type), pos, idx);
+			return -1;
+		}
+	}
+	return 0;
+}
+
+AST_TEST_DEFINE(stream_topology_del_stream)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_stream *stream;
+	enum ast_media_type type;
+	int idx;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_del_stream";
+		info->category = "/main/stream/";
+		info->summary = "stream topology stream delete unit test";
+		info->description =
+			"Test that deleting streams at a specific position in a topology works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Create streams */
+	for (type = AST_MEDIA_TYPE_UNKNOWN; type < AST_MEDIA_TYPE_END; ++type) {
+		stream = ast_stream_alloc(ast_codec_media_type2str(type), type);
+		if (!stream) {
+			ast_test_status_update(test, "Failed to create '%s' stream for testing stream topology\n",
+				ast_codec_media_type2str(type));
+			return AST_TEST_FAIL;
+		}
+		if (ast_stream_topology_append_stream(topology, stream) == -1) {
+			ast_test_status_update(test, "Failed to append '%s' stream to topology\n",
+				ast_codec_media_type2str(type));
+			ast_stream_free(stream);
+			return AST_TEST_FAIL;
+		}
+	}
+
+	/* Check initial stream positions and types for sanity. */
+	type = AST_MEDIA_TYPE_UNKNOWN;
+	for (idx = 0; idx < ast_stream_topology_get_count(topology); ++idx, ++type) {
+		stream = ast_stream_topology_get_stream(topology, idx);
+		if (type != ast_stream_get_type(stream)) {
+			ast_test_status_update(test, "Initial topology types failed: Expected:%s Got:%s\n",
+				ast_codec_media_type2str(type),
+				ast_codec_media_type2str(ast_stream_get_type(stream)));
+			return AST_TEST_FAIL;
+		}
+	}
+	if (check_stream_positions(test, topology)) {
+		ast_test_status_update(test, "Initial topology positions failed.\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Try to delete outside of topology size */
+	if (!ast_stream_topology_del_stream(topology, ast_stream_topology_get_count(topology))) {
+		ast_test_status_update(test, "Deleting stream outside of topology succeeded!\n");
+		return AST_TEST_FAIL;
+	}
+
+	/* Try to delete the last topology stream */
+	if (ast_stream_topology_del_stream(topology, ast_stream_topology_get_count(topology) - 1)) {
+		ast_test_status_update(test, "Failed deleting last stream of topology.\n");
+		return AST_TEST_FAIL;
+	}
+	if (check_stream_positions(test, topology)) {
+		ast_test_status_update(test, "Last stream delete topology positions failed.\n");
+		return AST_TEST_FAIL;
+	}
+	stream = ast_stream_topology_get_stream(topology, ast_stream_topology_get_count(topology) - 1);
+	type = ast_stream_get_type(stream);
+	if (type != AST_MEDIA_TYPE_END - 2) {
+		ast_test_status_update(test, "Last stream delete types failed: Expected:%s Got:%s\n",
+			ast_codec_media_type2str(AST_MEDIA_TYPE_END - 2),
+			ast_codec_media_type2str(type));
+		return AST_TEST_FAIL;
+	}
+
+	/* Try to delete the second stream in the topology */
+	if (ast_stream_topology_del_stream(topology, 1)) {
+		ast_test_status_update(test, "Failed deleting second stream in topology.\n");
+		return AST_TEST_FAIL;
+	}
+	if (check_stream_positions(test, topology)) {
+		ast_test_status_update(test, "Second stream delete topology positions failed.\n");
+		return AST_TEST_FAIL;
+	}
+	stream = ast_stream_topology_get_stream(topology, 1);
+	type = ast_stream_get_type(stream);
+	if (type != AST_MEDIA_TYPE_UNKNOWN + 2) {
+		ast_test_status_update(test, "Second stream delete types failed: Expected:%s Got:%s\n",
+			ast_codec_media_type2str(AST_MEDIA_TYPE_UNKNOWN + 2),
+			ast_codec_media_type2str(type));
+		return AST_TEST_FAIL;
+	}
+
+	/* Try to delete the first stream in the topology */
+	if (ast_stream_topology_del_stream(topology, 0)) {
+		ast_test_status_update(test, "Failed deleting first stream in topology.\n");
+		return AST_TEST_FAIL;
+	}
+	if (check_stream_positions(test, topology)) {
+		ast_test_status_update(test, "First stream delete topology positions failed.\n");
+		return AST_TEST_FAIL;
+	}
+	stream = ast_stream_topology_get_stream(topology, 0);
+	type = ast_stream_get_type(stream);
+	if (type != AST_MEDIA_TYPE_UNKNOWN + 2) {
+		ast_test_status_update(test, "First stream delete types failed: Expected:%s Got:%s\n",
+			ast_codec_media_type2str(AST_MEDIA_TYPE_UNKNOWN + 2),
+			ast_codec_media_type2str(type));
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_create_from_format_cap)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_create_from_format_cap";
+		info->category = "/main/stream/";
+		info->summary = "stream topology creation from format capabilities unit test";
+		info->description =
+			"Test that creating a stream topology from format capabilities results in the expected streams";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_ulaw, 0)) {
+		ast_test_status_update(test, "Failed to append a ulaw format to capabilities for stream topology creation\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_alaw, 0)) {
+		ast_test_status_update(test, "Failed to append an alaw format to capabilities for stream topology creation\n");
+		return AST_TEST_FAIL;
+	}
+
+	topology = ast_stream_topology_create_from_format_cap(caps);
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create a stream topology using a perfectly good format capabilities\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 1) {
+		ast_test_status_update(test, "Expected a stream topology with 1 stream but it has %d streams\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(topology, 0)) != AST_MEDIA_TYPE_AUDIO) {
+		ast_test_status_update(test, "Produced stream topology has a single stream of type %s instead of audio\n",
+			ast_codec_media_type2str(ast_stream_get_type(ast_stream_topology_get_stream(topology, 0))));
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_topology_free(topology);
+	topology = NULL;
+
+	ast_format_cap_append(caps, ast_format_h264, 0);
+
+	topology = ast_stream_topology_create_from_format_cap(caps);
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create a stream topology using a perfectly good format capabilities\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_count(topology) != 2) {
+		ast_test_status_update(test, "Expected a stream topology with 2 streams but it has %d streams\n",
+			ast_stream_topology_get_count(topology));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(topology, 0)) != AST_MEDIA_TYPE_AUDIO) {
+		ast_test_status_update(test, "Produced stream topology has a first stream of type %s instead of audio\n",
+			ast_codec_media_type2str(ast_stream_get_type(ast_stream_topology_get_stream(topology, 0))));
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(topology, 1)) != AST_MEDIA_TYPE_VIDEO) {
+		ast_test_status_update(test, "Produced stream topology has a second stream of type %s instead of video\n",
+			ast_codec_media_type2str(ast_stream_get_type(ast_stream_topology_get_stream(topology, 1))));
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(stream_topology_get_first_stream_by_type)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_stream *first_stream;
+	struct ast_stream *second_stream;
+	struct ast_stream *third_stream;
+	struct ast_stream *fourth_stream;
+	struct ast_stream *fifth_stream;
+	struct ast_stream *sixth_stream;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_get_first_stream_by_type";
+		info->category = "/main/stream/";
+		info->summary = "stream topology getting first stream by type unit test";
+		info->description =
+			"Test that getting the first stream by type from a topology actually returns the first stream";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	first_stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!first_stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+	ast_stream_set_state(first_stream, AST_STREAM_STATE_REMOVED);
+
+	if (ast_stream_topology_append_stream(topology, first_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(first_stream);
+		return AST_TEST_FAIL;
+	}
+
+	second_stream = ast_stream_alloc("audio2", AST_MEDIA_TYPE_AUDIO);
+	if (!second_stream) {
+		ast_test_status_update(test, "Failed to create a second audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, second_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(second_stream);
+		return AST_TEST_FAIL;
+	}
+
+	third_stream = ast_stream_alloc("audio3", AST_MEDIA_TYPE_AUDIO);
+	if (!third_stream) {
+		ast_test_status_update(test, "Failed to create a third audio stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, third_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(third_stream);
+		return AST_TEST_FAIL;
+	}
+
+	fourth_stream = ast_stream_alloc("video", AST_MEDIA_TYPE_VIDEO);
+	if (!fourth_stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+	ast_stream_set_state(fourth_stream, AST_STREAM_STATE_REMOVED);
+
+	if (ast_stream_topology_append_stream(topology, fourth_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(fourth_stream);
+		return AST_TEST_FAIL;
+	}
+
+	fifth_stream = ast_stream_alloc("video2", AST_MEDIA_TYPE_VIDEO);
+	if (!fifth_stream) {
+		ast_test_status_update(test, "Failed to create a second video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, fifth_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(fifth_stream);
+		return AST_TEST_FAIL;
+	}
+
+	sixth_stream = ast_stream_alloc("video3", AST_MEDIA_TYPE_VIDEO);
+	if (!sixth_stream) {
+		ast_test_status_update(test, "Failed to create a third video stream for testing stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, sixth_stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(sixth_stream);
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_first_stream_by_type(topology, AST_MEDIA_TYPE_AUDIO) != second_stream) {
+		ast_test_status_update(test, "Retrieved first audio stream from topology but it is not the correct one\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_get_first_stream_by_type(topology, AST_MEDIA_TYPE_VIDEO) != fifth_stream) {
+		ast_test_status_update(test, "Retrieved first video stream from topology but it is not the correct one\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+static const struct ast_channel_tech mock_channel_tech = {
+};
+
+AST_TEST_DEFINE(stream_topology_create_from_channel_nativeformats)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+	struct ast_channel *mock_channel;
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct ast_str *codec_have_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
+	struct ast_str *codec_wanted_buf = ast_str_alloca(AST_FORMAT_CAP_NAMES_LEN);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_create_from_channel_nativeformats";
+		info->category = "/main/stream/";
+		info->summary = "stream topology creation from channel native formats unit test";
+		info->description =
+			"Test that creating a stream topology from the setting of channel nativeformats results in the expected streams";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_ulaw, 0)) {
+		ast_test_status_update(test, "Failed to append a ulaw format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_alaw, 0)) {
+		ast_test_status_update(test, "Failed to append an alaw format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_h264, 0)) {
+		ast_test_status_update(test, "Failed to append an h264 format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	if (!mock_channel) {
+		ast_test_status_update(test, "Failed to create a mock channel for testing\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_channel_tech_set(mock_channel, &mock_channel_tech);
+	ast_channel_nativeformats_set(mock_channel, caps);
+
+	if (!ast_channel_get_stream_topology(mock_channel)) {
+		ast_test_status_update(test, "Set nativeformats with ulaw, alaw, and h264 on channel but it did not create a topology\n");
+		goto end;
+	}
+
+	if (ast_stream_topology_get_count(ast_channel_get_stream_topology(mock_channel)) != 2) {
+		ast_test_status_update(test, "Set nativeformats on a channel to ulaw, alaw, and h264 and received '%d' streams instead of expected 2\n",
+			ast_stream_topology_get_count(ast_channel_get_stream_topology(mock_channel)));
+		goto end;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 0)) != AST_MEDIA_TYPE_AUDIO) {
+		ast_test_status_update(test, "First stream on channel is of %s when it should be audio\n",
+			ast_codec_media_type2str(ast_stream_get_type(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 0))));
+		goto end;
+	}
+
+	ast_format_cap_remove_by_type(caps, AST_MEDIA_TYPE_VIDEO);
+	if (!ast_format_cap_identical(ast_stream_get_formats(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 0)), caps)) {
+		ast_test_status_update(test, "Formats on audio stream of channel are '%s' when they should be '%s'\n",
+			ast_format_cap_get_names(ast_stream_get_formats(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 0)), &codec_have_buf),
+			ast_format_cap_get_names(caps, &codec_wanted_buf));
+		goto end;
+	}
+
+	if (ast_stream_get_type(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 1)) != AST_MEDIA_TYPE_VIDEO) {
+		ast_test_status_update(test, "Second stream on channel is of type %s when it should be video\n",
+			ast_codec_media_type2str(ast_stream_get_type(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 1))));
+		goto end;
+	}
+
+	ast_format_cap_remove_by_type(caps, AST_MEDIA_TYPE_AUDIO);
+
+	if (ast_format_cap_append(caps, ast_format_h264, 0)) {
+		ast_test_status_update(test, "Failed to append h264 video codec to capabilities for capabilities comparison\n");
+		goto end;
+	}
+
+	if (!ast_format_cap_identical(ast_stream_get_formats(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 1)), caps)) {
+		ast_test_status_update(test, "Formats on video stream of channel are '%s' when they should be '%s'\n",
+			ast_format_cap_get_names(ast_stream_get_formats(ast_stream_topology_get_stream(ast_channel_get_stream_topology(mock_channel), 1)), &codec_wanted_buf),
+			ast_format_cap_get_names(caps, &codec_wanted_buf));
+		goto end;
+	}
+
+	res = AST_TEST_PASS;
+
+end:
+	ast_channel_unlock(mock_channel);
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+struct mock_channel_pvt {
+	int mallocd;
+	unsigned int wrote;
+	unsigned int wrote_stream;
+	int stream_num;
+	int frame_limit;
+	int frame_count;
+	int streams;
+	int frames_per_read;
+	unsigned int indicated_change_request;
+	unsigned int indicated_changed;
+};
+
+static struct ast_frame *mock_channel_read(struct ast_channel *chan)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+	struct ast_frame f = { 0, };
+	struct ast_frame *head_frame = NULL;
+	struct ast_frame *tail_frame = NULL;
+	int i;
+
+	if (pvt->frames_per_read == 0) {
+		pvt->frames_per_read = 1;
+	}
+	for (i = 0; i < pvt->frames_per_read && pvt->frame_count < pvt->frame_limit; i++) {
+		struct ast_frame *fr;
+
+		if (pvt->frame_count % 2 == 0) {
+			f.frametype = AST_FRAME_VOICE;
+			f.subclass.format = ast_format_ulaw;
+		} else {
+			f.frametype = AST_FRAME_VIDEO;
+			f.subclass.format = ast_format_h264;
+		}
+		f.seqno = pvt->frame_count;
+		f.stream_num = pvt->frame_count % pvt->streams;
+		pvt->frame_count++;
+		fr = ast_frdup(&f);
+		if (!head_frame) {
+			head_frame = fr;
+		} else  {
+			tail_frame->frame_list.next = fr;
+		}
+		tail_frame = fr;
+	}
+
+	return(head_frame);
+}
+
+static int mock_channel_write(struct ast_channel *chan, struct ast_frame *fr)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+
+	pvt->wrote = 1;
+
+	return 0;
+}
+
+static int mock_channel_indicate(struct ast_channel *chan, int condition, const void *data, size_t datalen)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+
+	if (condition == AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE) {
+		pvt->indicated_change_request = 1;
+	} else if (condition == AST_CONTROL_STREAM_TOPOLOGY_CHANGED) {
+		pvt->indicated_changed = 1;
+	}
+
+	return 0;
+}
+
+static int mock_channel_write_stream(struct ast_channel *chan, int stream_num, struct ast_frame *fr)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+
+	pvt->wrote_stream = 1;
+	pvt->stream_num = stream_num;
+
+	return 0;
+}
+
+static const struct ast_channel_tech mock_stream_channel_tech = {
+	.read_stream = mock_channel_read,
+	.write_stream = mock_channel_write_stream,
+};
+
+AST_TEST_DEFINE(stream_topology_channel_set)
+{
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_channel *mock_channel;
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_channel_set";
+		info->category = "/main/stream/";
+		info->summary = "stream topology setting on a channel unit test";
+		info->description =
+			"Test that setting a stream topology on a channel works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	if (!mock_channel) {
+		ast_test_status_update(test, "Failed to create a mock channel for testing\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_channel_tech_set(mock_channel, &mock_stream_channel_tech);
+	ast_channel_set_stream_topology(mock_channel, topology);
+
+	if (ast_channel_get_stream_topology(mock_channel) != topology) {
+		ast_test_status_update(test, "Set an explicit stream topology on a channel but the returned one did not match it\n");
+		res = AST_TEST_FAIL;
+	}
+
+	topology = NULL;
+	ast_channel_unlock(mock_channel);
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+static int mock_channel_hangup(struct ast_channel *chan)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+
+	if (pvt->mallocd) {
+		ast_free(pvt);
+	}
+
+	ast_channel_tech_pvt_set(chan, NULL);
+	return 0;
+}
+
+static const struct ast_channel_tech mock_channel_old_write_tech = {
+	.write = mock_channel_write,
+	.write_video = mock_channel_write,
+	.hangup = mock_channel_hangup,
+};
+
+AST_TEST_DEFINE(stream_write_non_multistream)
+{
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt pvt = { 0, };
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct ast_frame frame = { 0, };
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_write_non_multistream";
+		info->category = "/main/stream/";
+		info->summary = "stream writing to non-multistream capable channel test";
+		info->description =
+			"Test that writing frames to a non-multistream channel works as expected";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_ulaw, 0)) {
+		ast_test_status_update(test, "Failed to append a ulaw format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_h264, 0)) {
+		ast_test_status_update(test, "Failed to append an h264 format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	if (!mock_channel) {
+		ast_test_status_update(test, "Failed to create a mock channel for testing\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_channel_tech_set(mock_channel, &mock_channel_old_write_tech);
+	ast_channel_nativeformats_set(mock_channel, caps);
+
+	pvt.wrote = 0;
+	ast_channel_tech_pvt_set(mock_channel, &pvt);
+	ast_channel_unlock(mock_channel);
+
+	frame.frametype = AST_FRAME_VOICE;
+	frame.subclass.format = ast_format_ulaw;
+
+	if (ast_write(mock_channel, &frame)) {
+		ast_test_status_update(test, "Failed to write a ulaw frame to the mock channel when it should be fine\n");
+		goto end;
+	}
+
+	if (!pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw but it never reached the channel driver\n");
+		goto end;
+	}
+
+	pvt.wrote = 0;
+
+	if (!ast_write_stream(mock_channel, 2, &frame) || pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to a non-existent stream\n");
+		goto end;
+	}
+
+	frame.frametype = AST_FRAME_VIDEO;
+	frame.subclass.format = ast_format_h264;
+
+	if (ast_write(mock_channel, &frame)) {
+		ast_test_status_update(test, "Failed to write an h264 frame to the mock channel when it should be fine\n");
+		goto end;
+	}
+
+	if (!pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 but it never reached the channel driver\n");
+		goto end;
+	}
+
+	res = AST_TEST_PASS;
+
+end:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+static const struct ast_channel_tech mock_channel_write_stream_tech = {
+	.write = mock_channel_write,
+	.write_video = mock_channel_write,
+	.write_stream = mock_channel_write_stream,
+	.read_stream = mock_channel_read,
+	.hangup = mock_channel_hangup,
+};
+
+AST_TEST_DEFINE(stream_write_multistream)
+{
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_stream *stream;
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt pvt = { 0, };
+	enum ast_test_result_state res = AST_TEST_FAIL;
+	struct ast_frame frame = { 0, };
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_write_multistream";
+		info->category = "/main/stream/";
+		info->summary = "stream writing to multistream capable channel test";
+		info->description =
+			"Test that writing frames to a multistream channel works as expected";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	topology = ast_stream_topology_alloc();
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create media stream topology\n");
+		return AST_TEST_FAIL;
+	}
+
+	stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing multistream writing\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(stream);
+		return AST_TEST_FAIL;
+	}
+
+	stream = ast_stream_alloc("audio2", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing multistream writing\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(stream);
+		return AST_TEST_FAIL;
+	}
+
+	stream = ast_stream_alloc("video", AST_MEDIA_TYPE_VIDEO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing multistream writing\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(stream);
+		return AST_TEST_FAIL;
+	}
+
+	stream = ast_stream_alloc("video2", AST_MEDIA_TYPE_VIDEO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create a video stream for testing multistream writing\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_stream_topology_append_stream(topology, stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(stream);
+		return AST_TEST_FAIL;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_ulaw, 0)) {
+		ast_test_status_update(test, "Failed to append a ulaw format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_h264, 0)) {
+		ast_test_status_update(test, "Failed to append an h264 format to capabilities for channel nativeformats\n");
+		return AST_TEST_FAIL;
+	}
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	if (!mock_channel) {
+		ast_test_status_update(test, "Failed to create a mock channel for testing\n");
+		return AST_TEST_FAIL;
+	}
+
+	ast_channel_tech_set(mock_channel, &mock_channel_write_stream_tech);
+	ast_channel_set_stream_topology(mock_channel, topology);
+	ast_channel_nativeformats_set(mock_channel, caps);
+	topology = NULL;
+
+	ast_channel_tech_pvt_set(mock_channel, &pvt);
+	ast_channel_unlock(mock_channel);
+
+	frame.frametype = AST_FRAME_VOICE;
+	frame.subclass.format = ast_format_ulaw;
+	pvt.stream_num = -1;
+
+	if (ast_write(mock_channel, &frame)) {
+		ast_test_status_update(test, "Failed to write a ulaw frame to the mock channel when it should be fine\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 0) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the default stream but it ended up on stream %d and not 0\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	if (ast_write_stream(mock_channel, 0, &frame)) {
+		ast_test_status_update(test, "Failed to write a ulaw frame to the first audio stream\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the first audio stream but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the first audio stream but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 0) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the first audio stream but it ended up on stream %d and not 0\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	if (ast_write_stream(mock_channel, 1, &frame)) {
+		ast_test_status_update(test, "Failed to write a ulaw frame to the second audio stream\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the second audio stream but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the second audio stream but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 1) {
+		ast_test_status_update(test, "Successfully wrote a frame of ulaw to the second audio stream but it ended up on stream %d and not 1\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	frame.frametype = AST_FRAME_VIDEO;
+	frame.subclass.format = ast_format_h264;
+
+	if (ast_write(mock_channel, &frame)) {
+		ast_test_status_update(test, "Failed to write an h264 frame to the mock channel when it should be fine\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 2) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the default stream but it ended up on stream %d and not 2\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	if (ast_write_stream(mock_channel, 2, &frame)) {
+		ast_test_status_update(test, "Failed to write an h264 frame to the first video stream\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the first video stream but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the first video stream but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 2) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the first video stream but it ended up on stream %d and not 2\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	if (ast_write_stream(mock_channel, 3, &frame)) {
+		ast_test_status_update(test, "Failed to write an h264 frame to the second video stream\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the second video stream but it ended up on the old write callback instead of write_stream\n");
+		goto end;
+	}
+
+	if (!pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the second video stream but it never reached the channel driver\n");
+		goto end;
+	}
+
+	if (pvt.stream_num != 3) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to the second video stream but it ended up on stream %d and not 3\n",
+			pvt.stream_num);
+		goto end;
+	}
+
+	pvt.wrote_stream = 0;
+	pvt.stream_num = -1;
+
+	if (!ast_write_stream(mock_channel, 9, &frame)) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to a non-existent stream\n");
+		goto end;
+	}
+
+	if (pvt.wrote) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to a non-existent stream and it ended up on the old write callback\n");
+		goto end;
+	}
+
+	if (pvt.wrote_stream) {
+		ast_test_status_update(test, "Successfully wrote a frame of h264 to a non-existent stream and it ended up on the write_stream callback\n");
+		goto end;
+	}
+
+	res = AST_TEST_PASS;
+
+end:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+static int load_stream_readqueue(struct ast_channel *chan, int frames)
+{
+	struct mock_channel_pvt *pvt = ast_channel_tech_pvt(chan);
+	struct ast_frame f = { 0, };
+	struct ast_frame *frame = NULL;
+	int i;
+
+	while ((frame = AST_LIST_REMOVE_HEAD(ast_channel_readq(chan), frame_list)))
+			ast_frfree(frame);
+
+	for (i = 0; i < frames; i++) {
+		if (pvt->frame_count % 2 == 0) {
+			f.frametype = AST_FRAME_VOICE;
+			f.subclass.format = ast_format_ulaw;
+		} else {
+			f.frametype = AST_FRAME_VIDEO;
+			f.subclass.format = ast_format_h264;
+		}
+		f.stream_num = pvt->frame_count % pvt->streams;
+		f.seqno = pvt->frame_count;
+		ast_queue_frame(chan, &f);
+		pvt->frame_count++;
+	}
+
+	return 0;
+}
+
+static struct ast_channel *make_channel(struct ast_test *test, int streams,
+		struct ast_channel_tech *tech)
+{
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+	struct ast_channel *mock_channel = NULL;
+	struct mock_channel_pvt *pvt = NULL;
+	struct ast_stream_topology *topology = NULL;
+	struct ast_stream *stream;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int i;
+
+	mock_channel = ast_channel_alloc(0, AST_STATE_DOWN, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, "TestChannel");
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+	ast_channel_tech_set(mock_channel, tech);
+
+	if (tech->read_stream) {
+		topology = ast_stream_topology_alloc();
+		ast_test_validate_cleanup(test, topology, res, done);
+
+		for (i = 0; i < streams; i++) {
+			stream = ast_stream_alloc((i % 2 ? "video": "audio"), (i % 2 ? AST_MEDIA_TYPE_VIDEO : AST_MEDIA_TYPE_AUDIO));
+			ast_test_validate_cleanup(test, stream, res, done);
+			ast_test_validate_cleanup(test, ast_stream_topology_append_stream(topology, stream) == i, res, done);
+		}
+		ast_test_validate_cleanup(test, ast_stream_topology_get_count(topology) == streams, res, done);
+		ast_channel_set_stream_topology(mock_channel, topology);
+		topology = NULL;
+	} else {
+		caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+		ast_test_validate_cleanup(test, caps, res, done);
+
+		ast_test_validate_cleanup(test, ast_format_cap_append(caps, ast_format_ulaw, 0) == 0, res, done);
+		ast_test_validate_cleanup(test, ast_format_cap_append(caps, ast_format_h264, 0) == 0, res, done);
+		ast_channel_nativeformats_set(mock_channel, caps);
+	}
+
+	pvt = ast_calloc(1, sizeof(*pvt));
+	ast_test_validate_cleanup(test, pvt, res, done);
+	pvt->mallocd = 1;
+	ast_channel_tech_pvt_set(mock_channel, pvt);
+
+	ast_channel_unlock(mock_channel);
+
+done:
+	ast_stream_topology_free(topology);
+	if (res == AST_TEST_FAIL && mock_channel) {
+		ast_hangup(mock_channel);
+	}
+
+	return mock_channel;
+}
+
+enum CHANNEL_READ_TYPE {
+	CHANNEL_READ,
+	CHANNEL_READ_STREAM
+};
+
+static struct ast_frame *read_from_chan(enum CHANNEL_READ_TYPE rt, struct ast_channel *chan)
+{
+	if (rt == CHANNEL_READ_STREAM) {
+		return ast_read_stream(chan);
+	} else {
+		return ast_read(chan);
+	}
+}
+
+static enum ast_test_result_state read_test(struct ast_test *test, struct ast_channel_tech *tech,
+		enum CHANNEL_READ_TYPE rt, int streams, int frames, int frames_per_read, int expected_nulls)
+{
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt *pvt;
+	struct ast_frame *fr = NULL;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int i = 0;
+	int null_frames = 0;
+
+	ast_test_status_update(test, "ChanType: %s ReadType: %s Streams: %d Frames: %d Frames per read: %d Expected Nulls: %d\n",
+			tech->read_stream ? "MULTI" : "NON-MULTI",
+			rt == CHANNEL_READ_STREAM ? "STREAM" : "NON-STREAM",
+			streams, frames, frames_per_read, expected_nulls);
+	mock_channel = make_channel(test, 4, tech);
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pvt = ast_channel_tech_pvt(mock_channel);
+	pvt->frame_count = 0;
+	pvt->frame_limit = frames;
+	pvt->streams = streams;
+	pvt->frames_per_read = frames_per_read;
+
+	load_stream_readqueue(mock_channel, frames / 2);
+	ast_channel_fdno_set(mock_channel, 0);
+
+	while ((fr = read_from_chan(rt, mock_channel))) {
+		ast_channel_fdno_set(mock_channel, 0);
+		if (fr->frametype != AST_FRAME_NULL) {
+			ast_test_validate_cleanup(test, i == fr->seqno, res, done);
+			ast_test_validate_cleanup(test, fr->frametype == ( i % 2 ? AST_FRAME_VIDEO : AST_FRAME_VOICE), res, done);
+			ast_test_validate_cleanup(test, fr->stream_num == ( i % streams ), res, done);
+			ast_frfree(fr);
+		} else {
+			null_frames++;
+		}
+		fr = NULL;
+		i++;
+	}
+	ast_test_validate_cleanup(test, i == frames, res, done);
+	ast_test_validate_cleanup(test, null_frames == expected_nulls, res, done);
+
+done:
+	ast_test_status_update(test, "    Frames read: %d NULL frames: %d\n", i, null_frames);
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_read_non_multistream)
+{
+	struct ast_channel_tech tech = {
+		.read = mock_channel_read,
+		.hangup = mock_channel_hangup,
+	};
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_read_non_multistream";
+		info->category = "/main/stream/";
+		info->summary = "stream reading from non-multistream capable channel test";
+		info->description =
+			"Test that reading frames from a non-multistream channel works as expected";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	res = read_test(test, &tech, CHANNEL_READ, 2, 16, 1, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "non multi, non read stream, 2 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 2, 16, 1, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "non multi, read stream, 2 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ, 2, 16, 3, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "non multi, non read stream, 2 stream, 3 frames per read");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 2, 16, 3, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "non multi, read stream, 2 stream, 3 frames per read");
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_read_multistream)
+{
+	struct ast_channel_tech tech = {
+		.read_stream = mock_channel_read,
+		.write_stream = mock_channel_write_stream,
+		.hangup = mock_channel_hangup,
+	};
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_read_multistream";
+		info->category = "/main/stream/";
+		info->summary = "stream reading from multistream capable channel test";
+		info->description =
+			"Test that reading frames from a multistream channel works as expected";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	res = read_test(test, &tech, CHANNEL_READ, 2, 16, 1, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, non read stream, 2 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 2, 16, 1, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, read stream, 2 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ, 4, 16, 1, 8);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, non read stream, 4 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 4, 16, 1, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, read stream, 4 stream");
+
+	res = read_test(test, &tech, CHANNEL_READ, 2, 16, 3, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, non read stream, 2 stream, 3 frames per read");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 2, 16, 3, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, read stream, 2 stream, 3 frames per read");
+
+	res = read_test(test, &tech, CHANNEL_READ, 4, 16, 3, 8);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, non read stream, 4 stream, 3 frames per read");
+
+	res = read_test(test, &tech, CHANNEL_READ_STREAM, 4, 16, 3, 0);
+	ast_test_validate(test, res == AST_TEST_PASS, "multi, read stream, 4 stream, 3 frames per read");
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_topology_change_request_from_application_non_multistream)
+{
+	struct ast_channel_tech tech = {
+		.read = mock_channel_read,
+		.indicate = mock_channel_indicate,
+		.hangup = mock_channel_hangup,
+	};
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt *pvt;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int change_res;
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_change_request_from_application_non_multistream";
+		info->category = "/main/stream/";
+		info->summary = "stream topology changing on non-multistream channel test";
+		info->description =
+			"Test that an application trying to change the stream topology of a non-multistream channel gets a failure";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = make_channel(test, 1, &tech);
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pvt = ast_channel_tech_pvt(mock_channel);
+	pvt->indicated_change_request = 0;
+	pvt->indicated_changed = 0;
+
+	topology = ast_stream_topology_alloc();
+	ast_test_validate_cleanup(test, topology, res, done);
+
+	change_res = ast_channel_request_stream_topology_change(mock_channel, topology, NULL);
+
+	ast_test_validate_cleanup(test, change_res == -1, res, done);
+	ast_test_validate_cleanup(test, !pvt->indicated_change_request, res, done);
+
+	ast_channel_lock(mock_channel);
+	change_res = ast_channel_stream_topology_changed(mock_channel, topology);
+	ast_channel_unlock(mock_channel);
+
+	ast_test_validate_cleanup(test, change_res == -1, res, done);
+	ast_test_validate_cleanup(test, !pvt->indicated_changed, res, done);
+
+done:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_topology_change_request_from_channel_non_multistream)
+{
+	struct ast_channel_tech tech = {
+		.read_stream = mock_channel_read,
+		.write_stream = mock_channel_write_stream,
+		.indicate = mock_channel_indicate,
+		.hangup = mock_channel_hangup,
+	};
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt *pvt;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_frame request_change = {
+		.frametype = AST_FRAME_CONTROL,
+		.subclass.integer = AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE,
+	};
+	struct ast_frame *fr = NULL;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_change_request_from_channel_non_multistream";
+		info->category = "/main/stream/";
+		info->summary = "channel requesting stream topology change to non-multistream application test";
+		info->description =
+			"Test that a channel requesting a stream topology change from a non-multistream application does not work";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = make_channel(test, 1, &tech);
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pvt = ast_channel_tech_pvt(mock_channel);
+	pvt->indicated_changed = 0;
+
+	topology = ast_stream_topology_alloc();
+	ast_test_validate_cleanup(test, topology, res, done);
+
+	request_change.data.ptr = topology;
+	ast_queue_frame(mock_channel, &request_change);
+
+	fr = ast_read(mock_channel);
+	ast_test_validate_cleanup(test, fr, res, done);
+	ast_test_validate_cleanup(test, fr == &ast_null_frame, res, done);
+	ast_test_validate_cleanup(test, pvt->indicated_changed, res, done);
+
+done:
+	if (fr) {
+		ast_frfree(fr);
+	}
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_topology_change_request_from_application)
+{
+	struct ast_channel_tech tech = {
+		.read_stream = mock_channel_read,
+		.write_stream = mock_channel_write_stream,
+		.indicate = mock_channel_indicate,
+		.hangup = mock_channel_hangup,
+	};
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt *pvt;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	int change_res;
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_change_request_from_application";
+		info->category = "/main/stream/";
+		info->summary = "stream topology change request from application test";
+		info->description =
+			"Test that an application changing the stream topology of a multistream capable channel receives success";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = make_channel(test, 1, &tech);
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pvt = ast_channel_tech_pvt(mock_channel);
+	pvt->indicated_change_request = 0;
+	pvt->indicated_changed = 0;
+
+	topology = ast_stream_topology_alloc();
+	ast_test_validate_cleanup(test, topology, res, done);
+
+	change_res = ast_channel_request_stream_topology_change(mock_channel, topology, NULL);
+
+	ast_test_validate_cleanup(test, !change_res, res, done);
+	ast_test_validate_cleanup(test, pvt->indicated_change_request, res, done);
+
+	ast_channel_lock(mock_channel);
+	change_res = ast_channel_stream_topology_changed(mock_channel, topology);
+	ast_channel_unlock(mock_channel);
+
+	ast_test_validate_cleanup(test, !change_res, res, done);
+	ast_test_validate_cleanup(test, pvt->indicated_changed, res, done);
+
+done:
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(stream_topology_change_request_from_channel)
+{
+	struct ast_channel_tech tech = {
+		.read_stream = mock_channel_read,
+		.write_stream = mock_channel_write_stream,
+		.indicate = mock_channel_indicate,
+		.hangup = mock_channel_hangup,
+	};
+	struct ast_channel *mock_channel;
+	struct mock_channel_pvt *pvt;
+	enum ast_test_result_state res = AST_TEST_PASS;
+	RAII_VAR(struct ast_stream_topology *, topology, NULL, ast_stream_topology_free);
+	struct ast_frame request_change = {
+		.frametype = AST_FRAME_CONTROL,
+		.subclass.integer = AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE,
+	};
+	struct ast_frame *fr = NULL;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_change_request_from_channel";
+		info->category = "/main/stream/";
+		info->summary = "channel requesting stream topology change to multistream application test";
+		info->description =
+			"Test that a channel requesting a stream topology change from a multistream application works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	mock_channel = make_channel(test, 1, &tech);
+	ast_test_validate_cleanup(test, mock_channel, res, done);
+
+	pvt = ast_channel_tech_pvt(mock_channel);
+	pvt->indicated_changed = 0;
+
+	topology = ast_stream_topology_alloc();
+	ast_test_validate_cleanup(test, topology, res, done);
+
+	request_change.data.ptr = topology;
+	ast_queue_frame(mock_channel, &request_change);
+
+	fr = ast_read_stream(mock_channel);
+	ast_test_validate_cleanup(test, fr, res, done);
+	ast_test_validate_cleanup(test, fr->frametype == AST_FRAME_CONTROL, res, done);
+	ast_test_validate_cleanup(test, fr->subclass.integer == AST_CONTROL_STREAM_TOPOLOGY_REQUEST_CHANGE, res, done);
+	ast_test_validate_cleanup(test, !pvt->indicated_changed, res, done);
+
+done:
+	if (fr) {
+		ast_frfree(fr);
+	}
+	ast_hangup(mock_channel);
+
+	return res;
+}
+
+AST_TEST_DEFINE(format_cap_from_stream_topology)
+{
+	RAII_VAR(struct ast_format_cap *, caps, NULL, ao2_cleanup);
+	RAII_VAR(struct ast_format_cap *, stream_caps, NULL, ao2_cleanup);
+	struct ast_stream_topology *topology;
+	struct ast_stream *stream;
+	struct ast_format_cap *new_cap;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "format_cap_from_stream_topology";
+		info->category = "/main/stream/";
+		info->summary = "stream topology to format capabilities conversion test";
+		info->description =
+			"Test that converting a stream topology to format capabilities results in expected formats";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	caps = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!caps) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_ulaw, 0)) {
+		ast_test_status_update(test, "Failed to append ulaw format to capabilities\n");
+		return AST_TEST_FAIL;
+	}
+
+	if (ast_format_cap_append(caps, ast_format_h264, 0)) {
+		ast_test_status_update(test, "Failed to append h264 format to capabilities\n");
+		return AST_TEST_FAIL;
+	}
+
+	topology = ast_stream_topology_create_from_format_cap(caps);
+	if (!topology) {
+		ast_test_status_update(test, "Failed to create a stream topology from format capabilities of ulaw and h264\n");
+		return AST_TEST_FAIL;
+	}
+
+	/*
+	 * Append declined stream with formats that should not be included
+	 * in combined topology caps.
+	 */
+	stream = ast_stream_alloc("audio", AST_MEDIA_TYPE_AUDIO);
+	if (!stream) {
+		ast_test_status_update(test, "Failed to create an audio stream for testing stream topology\n");
+		ast_stream_topology_free(topology);
+		return AST_TEST_FAIL;
+	}
+	ast_stream_set_state(stream, AST_STREAM_STATE_REMOVED);
+	new_cap = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT);
+	if (!new_cap) {
+		ast_test_status_update(test, "Could not allocate an empty format capabilities structure\n");
+		ast_stream_free(stream);
+		ast_stream_topology_free(topology);
+		return AST_TEST_FAIL;
+	}
+	if (ast_format_cap_append(new_cap, ast_format_alaw, 0)) {
+		ast_test_status_update(test, "Failed to append alaw format to capabilities\n");
+		ao2_cleanup(new_cap);
+		ast_stream_free(stream);
+		ast_stream_topology_free(topology);
+		return AST_TEST_FAIL;
+	}
+	ast_stream_set_formats(stream, new_cap);
+	ao2_cleanup(new_cap);
+	if (ast_stream_topology_append_stream(topology, stream) == -1) {
+		ast_test_status_update(test, "Failed to append a perfectly good stream to a topology\n");
+		ast_stream_free(stream);
+		ast_stream_topology_free(topology);
+		return AST_TEST_FAIL;
+	}
+
+	stream_caps = ast_format_cap_from_stream_topology(topology);
+	if (!stream_caps) {
+		ast_test_status_update(test, "Failed to create a format capabilities from a stream topology\n");
+		ast_stream_topology_free(topology);
+		return AST_TEST_FAIL;
+	}
+
+	ast_stream_topology_free(topology);
+
+	if (!ast_format_cap_identical(caps, stream_caps)) {
+		ast_test_status_update(test, "Converting format capabilities into topology and back resulted in different formats\n");
+		return AST_TEST_FAIL;
+	}
+
+	return AST_TEST_PASS;
+}
+
+#define topology_append_stream(topology, name, type, res, label) \
+	do { \
+		struct ast_stream *__stream = ast_stream_alloc((name), (type)); \
+		ast_test_validate_cleanup(test, __stream, res, label); \
+		if (ast_stream_topology_append_stream((topology), __stream) < 0) { \
+			ast_stream_free(__stream); \
+			res = AST_TEST_FAIL; \
+			goto label;	     \
+		} \
+	} while(0)
+
+AST_TEST_DEFINE(stream_topology_map_create)
+{
+	RAII_VAR(struct ast_stream_topology *, t0, NULL, ast_stream_topology_free);
+
+	struct ast_vector_int types = { NULL };
+	struct ast_vector_int v0 = { NULL };
+	struct ast_vector_int v1 = { NULL };
+
+	enum ast_test_result_state res = AST_TEST_PASS;
+
+	switch (cmd) {
+	case TEST_INIT:
+		info->name = "stream_topology_map_create";
+		info->category = "/main/stream/";
+		info->summary = "stream topology map creation unit test";
+		info->description =
+			"Test that creating a stream topology map works";
+		return AST_TEST_NOT_RUN;
+	case TEST_EXECUTE:
+		break;
+	}
+
+	ast_test_validate(test, AST_VECTOR_INIT(&types, 5) == 0);
+
+	/* Map a first topology and check that it mapped one to one */
+	ast_test_validate_cleanup(test, (t0 = ast_stream_topology_alloc()), res, done);
+	topology_append_stream(t0, "audio", AST_MEDIA_TYPE_AUDIO, res, done);
+	topology_append_stream(t0, "video", AST_MEDIA_TYPE_VIDEO, res, done);
+
+	ast_stream_topology_map(t0, &types, &v0, &v1);
+	ast_test_validate_cleanup(test, AST_VECTOR_SIZE(&types) == 2, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 0) == AST_MEDIA_TYPE_AUDIO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 1) == AST_MEDIA_TYPE_VIDEO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 0) == 0, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 1) == 1, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 0) == 0, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 1) == 1, res, done);
+
+	/* Map a second topology and check that it merged */
+	ast_stream_topology_free(t0);
+	ast_test_validate_cleanup(test, (t0 = ast_stream_topology_alloc()), res, done);
+	topology_append_stream(t0, "video", AST_MEDIA_TYPE_VIDEO, res, done);
+	topology_append_stream(t0, "audio", AST_MEDIA_TYPE_AUDIO, res, done);
+
+	ast_stream_topology_map(t0, &types, &v0, &v1);
+	ast_test_validate_cleanup(test, AST_VECTOR_SIZE(&types) == 2, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 0) == AST_MEDIA_TYPE_AUDIO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 1) == AST_MEDIA_TYPE_VIDEO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 0) == 1, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 1) == 0, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 0) == 1, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 1) == 0, res, done);
+
+	/* Map a third topology with more streams and check that it merged */
+	ast_stream_topology_free(t0);
+	ast_test_validate_cleanup(test, (t0 = ast_stream_topology_alloc()), res, done);
+	topology_append_stream(t0, "video", AST_MEDIA_TYPE_VIDEO, res, done);
+	topology_append_stream(t0, "audio", AST_MEDIA_TYPE_AUDIO, res, done);
+	topology_append_stream(t0, "audio", AST_MEDIA_TYPE_AUDIO, res, done);
+
+	ast_stream_topology_map(t0, &types, &v0, &v1);
+	ast_test_validate_cleanup(test, AST_VECTOR_SIZE(&types) == 3, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 0) == AST_MEDIA_TYPE_AUDIO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 1) == AST_MEDIA_TYPE_VIDEO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&types, 2) == AST_MEDIA_TYPE_AUDIO, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 0) == 1, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 1) == 0, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v0, 2) == 2, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 0) == 1, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 1) == 0, res, done);
+	ast_test_validate_cleanup(test, AST_VECTOR_GET(&v1, 2) == 2, res, done);
+
+done:
+	AST_VECTOR_FREE(&v1);
+	AST_VECTOR_FREE(&v0);
+	AST_VECTOR_FREE(&types);
+
+	return res;
+}
+
+static int unload_module(void)
+{
+	AST_TEST_UNREGISTER(stream_create);
+	AST_TEST_UNREGISTER(stream_create_no_name);
+	AST_TEST_UNREGISTER(stream_set_type);
+	AST_TEST_UNREGISTER(stream_set_formats);
+	AST_TEST_UNREGISTER(stream_set_state);
+	AST_TEST_UNREGISTER(stream_metadata);
+	AST_TEST_UNREGISTER(stream_topology_create);
+	AST_TEST_UNREGISTER(stream_topology_clone);
+	AST_TEST_UNREGISTER(stream_topology_clone);
+	AST_TEST_UNREGISTER(stream_topology_append_stream);
+	AST_TEST_UNREGISTER(stream_topology_set_stream);
+	AST_TEST_UNREGISTER(stream_topology_del_stream);
+	AST_TEST_UNREGISTER(stream_topology_create_from_format_cap);
+	AST_TEST_UNREGISTER(stream_topology_get_first_stream_by_type);
+	AST_TEST_UNREGISTER(stream_topology_create_from_channel_nativeformats);
+	AST_TEST_UNREGISTER(stream_topology_channel_set);
+	AST_TEST_UNREGISTER(stream_write_non_multistream);
+	AST_TEST_UNREGISTER(stream_write_multistream);
+	AST_TEST_UNREGISTER(stream_read_non_multistream);
+	AST_TEST_UNREGISTER(stream_read_multistream);
+	AST_TEST_UNREGISTER(stream_topology_change_request_from_application_non_multistream);
+	AST_TEST_UNREGISTER(stream_topology_change_request_from_channel_non_multistream);
+	AST_TEST_UNREGISTER(stream_topology_change_request_from_application);
+	AST_TEST_UNREGISTER(stream_topology_change_request_from_channel);
+	AST_TEST_UNREGISTER(format_cap_from_stream_topology);
+	AST_TEST_UNREGISTER(stream_topology_map_create);
+	return 0;
+}
+
+static int load_module(void)
+{
+	AST_TEST_REGISTER(stream_create);
+	AST_TEST_REGISTER(stream_create_no_name);
+	AST_TEST_REGISTER(stream_set_type);
+	AST_TEST_REGISTER(stream_set_formats);
+	AST_TEST_REGISTER(stream_set_state);
+	AST_TEST_REGISTER(stream_metadata);
+	AST_TEST_REGISTER(stream_topology_create);
+	AST_TEST_REGISTER(stream_topology_clone);
+	AST_TEST_REGISTER(stream_topology_append_stream);
+	AST_TEST_REGISTER(stream_topology_set_stream);
+	AST_TEST_REGISTER(stream_topology_del_stream);
+	AST_TEST_REGISTER(stream_topology_create_from_format_cap);
+	AST_TEST_REGISTER(stream_topology_get_first_stream_by_type);
+	AST_TEST_REGISTER(stream_topology_create_from_channel_nativeformats);
+	AST_TEST_REGISTER(stream_topology_channel_set);
+	AST_TEST_REGISTER(stream_write_non_multistream);
+	AST_TEST_REGISTER(stream_write_multistream);
+	AST_TEST_REGISTER(stream_read_non_multistream);
+	AST_TEST_REGISTER(stream_read_multistream);
+	AST_TEST_REGISTER(stream_topology_change_request_from_application_non_multistream);
+	AST_TEST_REGISTER(stream_topology_change_request_from_channel_non_multistream);
+	AST_TEST_REGISTER(stream_topology_change_request_from_application);
+	AST_TEST_REGISTER(stream_topology_change_request_from_channel);
+	AST_TEST_REGISTER(format_cap_from_stream_topology);
+	AST_TEST_REGISTER(stream_topology_map_create);
+	return AST_MODULE_LOAD_SUCCESS;
+}
+
+AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Media Stream API test module");
diff -pruN 1:13.23.1~dfsg-2/tests/test_stringfields.c 1:16.1.1~dfsg-1/tests/test_stringfields.c
--- 1:13.23.1~dfsg-2/tests/test_stringfields.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_stringfields.c	2018-12-26 18:45:13.000000000 +0000
@@ -369,7 +369,7 @@ AST_TEST_DEFINE(string_field_aggregate_t
 	ast_string_field_ptr_set_by_fields(inst2->__field_mgr_pool, inst2->__field_mgr, &inst2->string1, "foo");
 	inst2->foo = 2;
 
-	if (inst3->__field_mgr.header->embedded_pool->prev) {
+	if (inst3->__field_mgr.embedded_pool->prev) {
 		ast_test_status_update(test, "Structure 3 embedded pool should not have a previous pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
@@ -377,13 +377,13 @@ AST_TEST_DEFINE(string_field_aggregate_t
 
 	ast_string_field_set(inst3, string1, "foo");
 
-	if (inst3->__field_mgr.header->embedded_pool != inst3->__field_mgr_pool) {
+	if (inst3->__field_mgr.embedded_pool != inst3->__field_mgr_pool) {
 		ast_test_status_update(test, "Structure 3 embedded pool should have been the current pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
 	}
 
-	if (inst3->__field_mgr.header->embedded_pool->prev) {
+	if (inst3->__field_mgr.embedded_pool->prev) {
 		ast_test_status_update(test, "Structure 3 embedded pool should not have a previous pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
@@ -395,13 +395,13 @@ AST_TEST_DEFINE(string_field_aggregate_t
 	ast_string_field_set(inst3, string2, "baz 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890 1234567890");
 	inst3->foo = 3;
 
-	if (inst3->__field_mgr_pool == inst3->__field_mgr.header->embedded_pool) {
+	if (inst3->__field_mgr_pool == inst3->__field_mgr.embedded_pool) {
 		ast_test_status_update(test, "Structure 3 embedded pool should not have been the current pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
 	}
 
-	if (inst3->__field_mgr.header->embedded_pool != inst3->__field_mgr_pool->prev) {
+	if (inst3->__field_mgr.embedded_pool != inst3->__field_mgr_pool->prev) {
 		ast_test_status_update(test, "Structure 3 embedded pool should be the current pool's previous!\n");
 		res = AST_TEST_FAIL;
 		goto error;
@@ -484,7 +484,7 @@ AST_TEST_DEFINE(string_field_aggregate_t
 		ast_test_status_update(test, "Structures 1/2 are the same (empty) as expected.\n");
 	}
 
-	if (inst4->__field_mgr.header->embedded_pool != inst4->__field_mgr_pool) {
+	if (inst4->__field_mgr.embedded_pool != inst4->__field_mgr_pool) {
 		ast_test_status_update(test, "Structure 4 embedded pool should have been the current pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
@@ -492,7 +492,7 @@ AST_TEST_DEFINE(string_field_aggregate_t
 		ast_test_status_update(test, "Structure 4 embedded pool is the current pool as expected.\n");
 	}
 
-	if (inst4->__field_mgr.header->embedded_pool->prev) {
+	if (inst4->__field_mgr.embedded_pool->prev) {
 		ast_test_status_update(test, "Structure 4 embedded pool should not have a previous pool!\n");
 		res = AST_TEST_FAIL;
 		goto error;
diff -pruN 1:13.23.1~dfsg-2/tests/test_strings.c 1:16.1.1~dfsg-1/tests/test_strings.c
--- 1:13.23.1~dfsg-2/tests/test_strings.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_strings.c	2018-12-26 18:45:13.000000000 +0000
@@ -34,8 +34,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/test.h"
 #include "asterisk/utils.h"
 #include "asterisk/strings.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_substitution.c 1:16.1.1~dfsg-1/tests/test_substitution.c
--- 1:13.23.1~dfsg-2/tests/test_substitution.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_substitution.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
 #include "asterisk/pbx.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_taskprocessor.c 1:16.1.1~dfsg-1/tests/test_taskprocessor.c
--- 1:13.23.1~dfsg-2/tests/test_taskprocessor.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_taskprocessor.c	2018-12-26 18:45:13.000000000 +0000
@@ -151,7 +151,10 @@ AST_TEST_DEFINE(default_taskprocessor)
 		return AST_TEST_FAIL;
 	}
 
-	ast_taskprocessor_push(tps, task, task_data);
+	if (ast_taskprocessor_push(tps, task, task_data)) {
+		ast_test_status_update(test, "Failed to queue task\n");
+		return AST_TEST_FAIL;
+	}
 
 	res = task_wait(task_data);
 	if (res != 0) {
@@ -240,7 +243,11 @@ AST_TEST_DEFINE(default_taskprocessor_lo
 
 	for (i = 0; i < NUM_TASKS; ++i) {
 		rand_data[i] = ast_random();
-		ast_taskprocessor_push(tps, load_task, &rand_data[i]);
+		if (ast_taskprocessor_push(tps, load_task, &rand_data[i])) {
+			ast_test_status_update(test, "Failed to queue task\n");
+			res = AST_TEST_FAIL;
+			goto test_end;
+		}
 	}
 
 	ast_mutex_lock(&load_task_results.lock);
@@ -438,14 +445,22 @@ AST_TEST_DEFINE(taskprocessor_listener)
 		goto test_exit;
 	}
 
-	ast_taskprocessor_push(tps, listener_test_task, NULL);
+	if (ast_taskprocessor_push(tps, listener_test_task, NULL)) {
+		ast_test_status_update(test, "Failed to queue task\n");
+		res = AST_TEST_FAIL;
+		goto test_exit;
+	}
 
 	if (check_stats(test, pvt, 1, 0, 1) < 0) {
 		res = AST_TEST_FAIL;
 		goto test_exit;
 	}
 
-	ast_taskprocessor_push(tps, listener_test_task, NULL);
+	if (ast_taskprocessor_push(tps, listener_test_task, NULL)) {
+		ast_test_status_update(test, "Failed to queue task\n");
+		res = AST_TEST_FAIL;
+		goto test_exit;
+	}
 
 	if (check_stats(test, pvt, 2, 0, 1) < 0) {
 		res = AST_TEST_FAIL;
@@ -710,7 +725,10 @@ AST_TEST_DEFINE(taskprocessor_push_local
 	local_data = 0;
 	ast_taskprocessor_set_local(tps, &local_data);
 
-	ast_taskprocessor_push_local(tps, local_task_exe, task_data);
+	if (ast_taskprocessor_push_local(tps, local_task_exe, task_data)) {
+		ast_test_status_update(test, "Failed to queue task\n");
+		return AST_TEST_FAIL;
+	}
 
 	res = task_wait(task_data);
 	if (res != 0) {
diff -pruN 1:13.23.1~dfsg-2/tests/test_threadpool.c 1:16.1.1~dfsg-1/tests/test_threadpool.c
--- 1:13.23.1~dfsg-2/tests/test_threadpool.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_threadpool.c	2018-12-26 18:45:13.000000000 +0000
@@ -127,6 +127,18 @@ static struct simple_task_data *simple_t
 	return std;
 }
 
+static void simple_task_data_free(struct simple_task_data *std)
+{
+	if (!std) {
+		return;
+	}
+
+	ast_mutex_destroy(&std->lock);
+	ast_cond_destroy(&std->cond);
+
+	ast_free(std);
+}
+
 static int simple_task(void *data)
 {
 	struct simple_task_data *std = data;
@@ -319,7 +331,9 @@ AST_TEST_DEFINE(threadpool_push)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	wait_for_task_pushed(listener);
 
@@ -328,7 +342,7 @@ AST_TEST_DEFINE(threadpool_push)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
@@ -635,11 +649,13 @@ AST_TEST_DEFINE(threadpool_thread_timeou
 		}
 		ast_mutex_unlock(&tld->lock);
 
-		ast_threadpool_push(pool, simple_task, std);
-
-		res = wait_for_completion(test, std);
+		if (ast_threadpool_push(pool, simple_task, std)) {
+			res = AST_TEST_FAIL;
+		} else {
+			res = wait_for_completion(test, std);
+		}
 
-		ast_free(std);
+		simple_task_data_free(std);
 
 		if (res == AST_TEST_FAIL) {
 			goto end;
@@ -707,7 +723,9 @@ AST_TEST_DEFINE(threadpool_one_task_one_
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 1);
 
@@ -736,7 +754,7 @@ AST_TEST_DEFINE(threadpool_one_task_one_
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 
@@ -796,7 +814,10 @@ AST_TEST_DEFINE(threadpool_one_thread_on
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		res = AST_TEST_FAIL;
+		goto end;
+	}
 
 	res = wait_for_completion(test, std);
 	if (res == AST_TEST_FAIL) {
@@ -819,7 +840,7 @@ AST_TEST_DEFINE(threadpool_one_thread_on
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
@@ -882,9 +903,18 @@ AST_TEST_DEFINE(threadpool_one_thread_mu
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
-	ast_threadpool_push(pool, simple_task, std2);
-	ast_threadpool_push(pool, simple_task, std3);
+	res = AST_TEST_FAIL;
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std3)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std1);
 	if (res == AST_TEST_FAIL) {
@@ -914,9 +944,9 @@ AST_TEST_DEFINE(threadpool_one_thread_mu
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
-	ast_free(std3);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
+	simple_task_data_free(std3);
 	ast_free(tld);
 	return res;
 }
@@ -1011,7 +1041,9 @@ AST_TEST_DEFINE(threadpool_auto_incremen
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
 
 	/* Pushing the task should result in the threadpool growing
 	 * by three threads. This will allow the task to actually execute
@@ -1034,9 +1066,19 @@ AST_TEST_DEFINE(threadpool_auto_incremen
 	/* Now push three tasks into the pool and ensure the pool does not
 	 * grow.
 	 */
-	ast_threadpool_push(pool, simple_task, std2);
-	ast_threadpool_push(pool, simple_task, std3);
-	ast_threadpool_push(pool, simple_task, std4);
+	res = AST_TEST_FAIL;
+
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std3)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, simple_task, std4)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std2);
 	if (res == AST_TEST_FAIL) {
@@ -1064,10 +1106,10 @@ AST_TEST_DEFINE(threadpool_auto_incremen
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
-	ast_free(std3);
-	ast_free(std4);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
+	simple_task_data_free(std3);
+	simple_task_data_free(std4);
 	ast_free(tld);
 	return res;
 }
@@ -1121,7 +1163,9 @@ AST_TEST_DEFINE(threadpool_max_size)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std);
+	if (ast_threadpool_push(pool, simple_task, std)) {
+		goto end;
+	}
 
 	res = wait_for_completion(test, std);
 	if (res == AST_TEST_FAIL) {
@@ -1137,7 +1181,7 @@ AST_TEST_DEFINE(threadpool_max_size)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std);
+	simple_task_data_free(std);
 	ast_free(tld);
 	return res;
 }
@@ -1193,7 +1237,9 @@ AST_TEST_DEFINE(threadpool_reactivation)
 		goto end;
 	}
 
-	ast_threadpool_push(pool, simple_task, std1);
+	if (ast_threadpool_push(pool, simple_task, std1)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 1);
 
@@ -1218,7 +1264,10 @@ AST_TEST_DEFINE(threadpool_reactivation)
 	}
 
 	/* Now make sure the threadpool reactivates when we add a second task */
-	ast_threadpool_push(pool, simple_task, std2);
+	if (ast_threadpool_push(pool, simple_task, std2)) {
+		res = AST_TEST_FAIL;
+		goto end;
+	}
 
 	res = wait_for_completion(test, std2);
 	if (res == AST_TEST_FAIL) {
@@ -1240,8 +1289,8 @@ AST_TEST_DEFINE(threadpool_reactivation)
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(std1);
-	ast_free(std2);
+	simple_task_data_free(std1);
+	simple_task_data_free(std2);
 	ast_free(tld);
 	return res;
 
@@ -1269,6 +1318,19 @@ static struct complex_task_data *complex
 	return ctd;
 }
 
+static void complex_task_data_free(struct complex_task_data *ctd)
+{
+	if (!ctd) {
+		return;
+	}
+
+	ast_mutex_destroy(&ctd->lock);
+	ast_cond_destroy(&ctd->stall_cond);
+	ast_cond_destroy(&ctd->notify_cond);
+
+	ast_free(ctd);
+}
+
 static int complex_task(void *data)
 {
 	struct complex_task_data *ctd = data;
@@ -1400,8 +1462,13 @@ AST_TEST_DEFINE(threadpool_task_distribu
 		goto end;
 	}
 
-	ast_threadpool_push(pool, complex_task, ctd1);
-	ast_threadpool_push(pool, complex_task, ctd2);
+	if (ast_threadpool_push(pool, complex_task, ctd1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, complex_task, ctd2)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 2);
 
@@ -1438,8 +1505,8 @@ AST_TEST_DEFINE(threadpool_task_distribu
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(ctd1);
-	ast_free(ctd2);
+	complex_task_data_free(ctd1);
+	complex_task_data_free(ctd2);
 	ast_free(tld);
 	return res;
 }
@@ -1496,8 +1563,13 @@ AST_TEST_DEFINE(threadpool_more_destruct
 		goto end;
 	}
 
-	ast_threadpool_push(pool, complex_task, ctd1);
-	ast_threadpool_push(pool, complex_task, ctd2);
+	if (ast_threadpool_push(pool, complex_task, ctd1)) {
+		goto end;
+	}
+
+	if (ast_threadpool_push(pool, complex_task, ctd2)) {
+		goto end;
+	}
 
 	ast_threadpool_set_size(pool, 4);
 
@@ -1549,8 +1621,8 @@ AST_TEST_DEFINE(threadpool_more_destruct
 end:
 	ast_threadpool_shutdown(pool);
 	ao2_cleanup(listener);
-	ast_free(ctd1);
-	ast_free(ctd2);
+	complex_task_data_free(ctd1);
+	complex_task_data_free(ctd2);
 	ast_free(tld);
 	return res;
 }
@@ -1666,9 +1738,9 @@ end:
 	poke_worker(data3);
 	ast_taskprocessor_unreference(uut);
 	ast_threadpool_shutdown(pool);
-	ast_free(data1);
-	ast_free(data2);
-	ast_free(data3);
+	complex_task_data_free(data1);
+	complex_task_data_free(data2);
+	complex_task_data_free(data3);
 	return res;
 }
 
diff -pruN 1:13.23.1~dfsg-2/tests/test_time.c 1:16.1.1~dfsg-1/tests/test_time.c
--- 1:13.23.1~dfsg-2/tests/test_time.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_time.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/app.h"
 #include "asterisk/module.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_uri.c 1:16.1.1~dfsg-1/tests/test_uri.c
--- 1:13.23.1~dfsg-2/tests/test_uri.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_uri.c	2018-12-26 18:45:13.000000000 +0000
@@ -31,8 +31,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/uri.h"
diff -pruN 1:13.23.1~dfsg-2/tests/test_utils.c 1:16.1.1~dfsg-1/tests/test_utils.c
--- 1:13.23.1~dfsg-2/tests/test_utils.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_utils.c	2018-12-26 18:45:13.000000000 +0000
@@ -35,8 +35,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$");
-
 #include "asterisk/utils.h"
 #include "asterisk/test.h"
 #include "asterisk/crypto.h"
@@ -394,7 +392,7 @@ AST_TEST_DEFINE(agi_loaded_test)
 	}
 #endif
 
-	ast_agi_unregister(ast_module_info->self, &noop_command);
+	ast_agi_unregister(&noop_command);
 	return res;
 }
 
@@ -649,4 +647,9 @@ static int load_module(void)
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Utils test module");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Utils test module",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_agi,res_crypto",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_voicemail_api.c 1:16.1.1~dfsg-1/tests/test_voicemail_api.c
--- 1:13.23.1~dfsg-2/tests/test_voicemail_api.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_voicemail_api.c	2018-12-26 18:45:13.000000000 +0000
@@ -33,8 +33,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/stat.h>
 
 #include "asterisk/utils.h"
@@ -827,12 +825,12 @@ static struct ast_channel *test_vm_api_c
 	}
 
 	ast_channel_set_writeformat(mock_channel, ast_format_gsm);
-	native_formats = ast_channel_nativeformats(mock_channel);
-	ast_format_cap_append(native_formats, ast_channel_writeformat(mock_channel), 0);
 	ast_channel_set_rawwriteformat(mock_channel, ast_format_gsm);
 	ast_channel_set_readformat(mock_channel, ast_format_gsm);
 	ast_channel_set_rawreadformat(mock_channel, ast_format_gsm);
 	ast_channel_tech_set(mock_channel, &mock_channel_tech);
+	native_formats = ast_channel_nativeformats(mock_channel);
+	ast_format_cap_append(native_formats, ast_channel_writeformat(mock_channel), 0);
 
 	ast_channel_unlock(mock_channel);
 
diff -pruN 1:13.23.1~dfsg-2/tests/test_websocket_client.c 1:16.1.1~dfsg-1/tests/test_websocket_client.c
--- 1:13.23.1~dfsg-2/tests/test_websocket_client.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_websocket_client.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "")
-
 #include "asterisk/test.h"
 #include "asterisk/module.h"
 #include "asterisk/astobj2.h"
@@ -158,4 +156,9 @@ static int unload_module(void)
 	return 0;
 }
 
-AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Websocket client test module");
+AST_MODULE_INFO(ASTERISK_GPL_KEY, AST_MODFLAG_DEFAULT, "Websocket client test module",
+	.support_level = AST_MODULE_SUPPORT_CORE,
+	.load = load_module,
+	.unload = unload_module,
+	.requires = "res_http_websocket",
+);
diff -pruN 1:13.23.1~dfsg-2/tests/test_xml_escape.c 1:16.1.1~dfsg-1/tests/test_xml_escape.c
--- 1:13.23.1~dfsg-2/tests/test_xml_escape.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/tests/test_xml_escape.c	2018-12-26 18:45:13.000000000 +0000
@@ -32,8 +32,6 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/utils.h"
 #include "asterisk/module.h"
 #include "asterisk/test.h"
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/configure.m4 1:16.1.1~dfsg-1/third-party/jansson/configure.m4
--- 1:13.23.1~dfsg-2/third-party/jansson/configure.m4	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/configure.m4	2018-12-26 18:45:13.000000000 +0000
@@ -74,11 +74,15 @@ AC_DEFUN([_JANSSON_CONFIGURE],
 	JANSSON_LIB="-L${JANSSON_DIR}/dest/lib -ljansson"
 	PBX_JANSSON=1
 
+	# We haven't run install yet
+	JANSSON_DEFINE_JSON_INT([$JANSSON_DIR]/source/src/)
+
 	AC_SUBST([JANSSON_BUNDLED])
 	AC_SUBST([PBX_JANSSON])
 	AC_SUBST([JANSSON_LIB])
 	AC_SUBST([JANSSON_INCLUDE])
 	AC_MSG_RESULT(yes)
+	AC_DEFINE([HAVE_JANSSON_BUNDLED], 1, [Define if your system has JANSSON_BUNDLED])
 ])
 
 AC_DEFUN([JANSSON_CONFIGURE],
@@ -87,3 +91,19 @@ AC_DEFUN([JANSSON_CONFIGURE],
 		_JANSSON_CONFIGURE()
 	fi
 ])
+
+AC_DEFUN([JANSSON_DEFINE_JSON_INT],
+[
+	# Define the ast_json_int_t (large integer type) to match jansson's
+	saved_cppflags="${CPPFLAGS}"
+	CPPFLAGS="${CPPFLAGS} ${JANSSON_INCLUDE}"
+	AC_COMPILE_IFELSE(
+		[AC_LANG_PROGRAM([#include <$1jansson.h>],
+		[#if !JSON_INTEGER_IS_LONG_LONG
+		#error "not long long"
+		#endif
+		])],
+		[AC_DEFINE([AST_JSON_INT_T], [long long], [Define to 'long' or 'long long'])],
+		[AC_DEFINE([AST_JSON_INT_T], [long], [Define to 'long' or 'long long'])])
+	CPPFLAGS="${saved_cppflags}"
+])
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/jansson-2.11.tar.bz2.md5 1:16.1.1~dfsg-1/third-party/jansson/jansson-2.11.tar.bz2.md5
--- 1:13.23.1~dfsg-2/third-party/jansson/jansson-2.11.tar.bz2.md5	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/jansson-2.11.tar.bz2.md5	1970-01-01 00:00:00.000000000 +0000
@@ -1 +0,0 @@
-289ca8cbd2df31de9bda7e5220754d25  jansson-2.11.tar.bz2
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/jansson-2.12.tar.bz2.md5 1:16.1.1~dfsg-1/third-party/jansson/jansson-2.12.tar.bz2.md5
--- 1:13.23.1~dfsg-2/third-party/jansson/jansson-2.12.tar.bz2.md5	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/jansson-2.12.tar.bz2.md5	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1 @@
+317dbaf90a9f85ea0ec7b12d080d173d  jansson-2.12.tar.bz2
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/Makefile 1:16.1.1~dfsg-1/third-party/jansson/Makefile
--- 1:13.23.1~dfsg-2/third-party/jansson/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -55,7 +55,7 @@ SHELL_ECHO_PREFIX := echo '[jansson] '
 dest/include/jansson.h: source/config.status
 	$(ECHO_PREFIX) Building bundled jansson.
 	$(CMD_PREFIX) (cd source; make $(REALLY_QUIET))
-	$(CMD_PREFIX) (cd source; make install $(REALLY_QUIET))
+	$(CMD_PREFIX) (cd source; make install DESTDIR= $(REALLY_QUIET))
 
 .DELETE_ON_ERROR:
 
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/patches/0001-Improve-test-coverage.patch 1:16.1.1~dfsg-1/third-party/jansson/patches/0001-Improve-test-coverage.patch
--- 1:13.23.1~dfsg-2/third-party/jansson/patches/0001-Improve-test-coverage.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/patches/0001-Improve-test-coverage.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,128 +0,0 @@
-From 73c22de51672cb40fdc29c95331923d4dcebb6fa Mon Sep 17 00:00:00 2001
-From: Corey Farrell <git@cfware.com>
-Date: Tue, 13 Feb 2018 04:35:37 -0500
-Subject: [PATCH 01/22] Improve test coverage.
-
-Changes to test/ removed for bundled use in Asterisk.
-
-* Test equality of different length strings.
-* Add tab to json_pack whitespace test.
-* Test json_sprintf with empty result and invalid UTF.
-* Test json_get_alloc_funcs with NULL arguments.
-* Test invalid arguments.
-* Add test_chaos to test allocation failure code paths.
-* Remove redundant json_is_string checks from json_string_equal and
-  json_string_copy.  Both functions are static and can only be called
-  with a json string.
-
-Fixes to issues found by test_chaos:
-* Fix crash on OOM in pack_unpack.c:read_string().
-* Unconditionally free string in string_create upon allocation failure.
-  Update load.c:parse_value() to reflect this.  This resolves a leak on
-  allocation failure for pack_unpack.c:pack_string() and
-  value.c:json_sprintf().
-
-Although not visible from CodeCoverage these changes significantly
-increase branch coverage.  Especially in src/value.c where we previously
-covered 67.4% of branches and now cover 96.3% of branches.
----
- CMakeLists.txt                      |   1 +
- src/load.c                          |   6 +-
- src/pack_unpack.c                   |   5 +-
- src/value.c                         |   9 +-
- test/.gitignore                     |   1 +
- test/suites/api/Makefile.am         |   2 +
- test/suites/api/test_array.c        |  73 +++++++++++++++++
- test/suites/api/test_chaos.c        | 115 ++++++++++++++++++++++++++
- test/suites/api/test_equal.c        |   7 ++
- test/suites/api/test_memory_funcs.c |   7 ++
- test/suites/api/test_number.c       |  36 ++++++++
- test/suites/api/test_object.c       | 122 ++++++++++++++++++++++++++++
- test/suites/api/test_pack.c         |  10 ++-
- test/suites/api/test_simple.c       |  52 ++++++++++++
- test/suites/api/test_sprintf.c      |  12 +++
- 15 files changed, 444 insertions(+), 14 deletions(-)
- create mode 100644 test/suites/api/test_chaos.c
-
-diff --git a/CMakeLists.txt b/CMakeLists.txt
-index 16cf552..2f6cfec 100644
---- a/CMakeLists.txt
-+++ b/CMakeLists.txt
-@@ -487,6 +487,7 @@ if (NOT JANSSON_WITHOUT_TESTS)
-    set(api_tests
-          test_array
-          test_copy
-+         test_chaos
-          test_dump
-          test_dump_callback
-          test_equal
-diff --git a/src/load.c b/src/load.c
-index deb36f3..25efe2e 100644
---- a/src/load.c
-+++ b/src/load.c
-@@ -829,10 +829,8 @@ static json_t *parse_value(lex_t *lex, size_t flags, json_error_t *error)
-             }
- 
-             json = jsonp_stringn_nocheck_own(value, len);
--            if(json) {
--                lex->value.string.val = NULL;
--                lex->value.string.len = 0;
--            }
-+            lex->value.string.val = NULL;
-+            lex->value.string.len = 0;
-             break;
-         }
- 
-diff --git a/src/pack_unpack.c b/src/pack_unpack.c
-index 153f64d..19dbf93 100644
---- a/src/pack_unpack.c
-+++ b/src/pack_unpack.c
-@@ -159,7 +159,10 @@ static char *read_string(scanner_t *s, va_list *ap,
-         return (char *)str;
-     }
- 
--    strbuffer_init(&strbuff);
-+    if(strbuffer_init(&strbuff)) {
-+        set_error(s, "<internal>", json_error_out_of_memory, "Out of memory");
-+        s->has_error = 1;
-+    }
- 
-     while(1) {
-         str = va_arg(*ap, const char *);
-diff --git a/src/value.c b/src/value.c
-index b3b3141..29a978c 100644
---- a/src/value.c
-+++ b/src/value.c
-@@ -652,8 +652,7 @@ static json_t *string_create(const char *value, size_t len, int own)
- 
-     string = jsonp_malloc(sizeof(json_string_t));
-     if(!string) {
--        if(!own)
--            jsonp_free(v);
-+        jsonp_free(v);
-         return NULL;
-     }
-     json_init(&string->json, JSON_STRING);
-@@ -768,9 +767,6 @@ static int json_string_equal(const json_t *string1, const json_t *string2)
- {
-     json_string_t *s1, *s2;
- 
--    if(!json_is_string(string1) || !json_is_string(string2))
--        return 0;
--
-     s1 = json_to_string(string1);
-     s2 = json_to_string(string2);
-     return s1->length == s2->length && !memcmp(s1->value, s2->value, s1->length);
-@@ -780,9 +776,6 @@ static json_t *json_string_copy(const json_t *string)
- {
-     json_string_t *s;
- 
--    if(!json_is_string(string))
--        return NULL;
--
-     s = json_to_string(string);
-     return json_stringn_nocheck(s->value, s->length);
- }
--- 
-2.17.1
-
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/patches/0017-Fix-error-handling-in-json_pack.patch 1:16.1.1~dfsg-1/third-party/jansson/patches/0017-Fix-error-handling-in-json_pack.patch
--- 1:13.23.1~dfsg-2/third-party/jansson/patches/0017-Fix-error-handling-in-json_pack.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/patches/0017-Fix-error-handling-in-json_pack.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,103 +0,0 @@
-From 15105b66b4df387037b670ac713584194ea10c2f Mon Sep 17 00:00:00 2001
-From: Maxim Zhukov <mussitantesmortem@gmail.com>
-Date: Mon, 12 Mar 2018 17:39:04 +0300
-Subject: [PATCH 17/22] Fix error handling in json_pack
-
-Changes to test/ removed.
-
-Fixed a bug where the error message was not filled if an empty object
-was passed to the json_pack.
-
-Fixes #271
----
- src/pack_unpack.c           | 64 ++++++++++++++++++-------------------
- test/suites/api/test_pack.c |  8 +++++
- 2 files changed, 40 insertions(+), 32 deletions(-)
-
-diff --git a/src/pack_unpack.c b/src/pack_unpack.c
-index 4026fd9..6461c06 100644
---- a/src/pack_unpack.c
-+++ b/src/pack_unpack.c
-@@ -348,6 +348,36 @@ static json_t *pack_string(scanner_t *s, va_list *ap)
-     }
- }
- 
-+static json_t *pack_object_inter(scanner_t *s, va_list *ap, int need_incref)
-+{
-+    json_t *json;
-+    char ntoken;
-+
-+    next_token(s);
-+    ntoken = token(s);
-+
-+    if (ntoken != '?')
-+        prev_token(s);
-+
-+    json = va_arg(*ap, json_t *);
-+
-+    if (json)
-+        return need_incref ? json_incref(json) : json;
-+
-+    switch (ntoken) {
-+        case '?':
-+            return json_null();
-+        case '*':
-+            return NULL;
-+        default:
-+            break;
-+    }
-+
-+    set_error(s, "<args>", json_error_null_value, "NULL object key");
-+    s->has_error = 1;
-+    return NULL;
-+}
-+
- static json_t *pack(scanner_t *s, va_list *ap)
- {
-     switch(token(s)) {
-@@ -376,40 +406,10 @@ static json_t *pack(scanner_t *s, va_list *ap)
-             return json_real(va_arg(*ap, double));
- 
-         case 'O': /* a json_t object; increments refcount */
--        {
--            int nullable;
--            json_t *json;
--
--            next_token(s);
--            nullable = token(s) == '?';
--            if (!nullable)
--                prev_token(s);
--
--            json = va_arg(*ap, json_t *);
--            if (!json && nullable) {
--                return json_null();
--            } else {
--                return json_incref(json);
--            }
--        }
-+            return pack_object_inter(s, ap, 1);
- 
-         case 'o': /* a json_t object; doesn't increment refcount */
--        {
--            int nullable;
--            json_t *json;
--
--            next_token(s);
--            nullable = token(s) == '?';
--            if (!nullable)
--                prev_token(s);
--
--            json = va_arg(*ap, json_t *);
--            if (!json && nullable) {
--                return json_null();
--            } else {
--                return json;
--            }
--        }
-+            return pack_object_inter(s, ap, 0);
- 
-         default:
-             set_error(s, "<format>", json_error_invalid_format, "Unexpected format character '%c'",
--- 
-2.17.1
-
diff -pruN 1:13.23.1~dfsg-2/third-party/jansson/patches/README 1:16.1.1~dfsg-1/third-party/jansson/patches/README
--- 1:13.23.1~dfsg-2/third-party/jansson/patches/README	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/jansson/patches/README	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,10 @@
+Patches should be added here once merged to the upstream jansson project at
+https://github.com/akheron/jansson/.  Patch filenames should be generated by
+running 'git format-patch ${JANSSON_VERSION}' from the jansson repository, then
+copying the required patches to this folder.
+
+Any changes to configure.ac or Makefile.am must be stripped from the upstream
+patches.  The upstream project does not commit `autoreconf -i` output files
+and we must ensure bundled jansson does not require autoconf or automake.
+Changes to test files can also be stripped as the bundled builds do not run
+jansson tests.
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/configure.m4 1:16.1.1~dfsg-1/third-party/pjproject/configure.m4
--- 1:13.23.1~dfsg-2/third-party/pjproject/configure.m4	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/configure.m4	2018-12-26 18:45:13.000000000 +0000
@@ -73,7 +73,7 @@ AC_DEFUN([_PJPROJECT_CONFIGURE],
 	fi
 
 	export TAR PATCH SED NM EXTERNALS_CACHE_DIR AST_DOWNLOAD_CACHE DOWNLOAD_TO_STDOUT DOWNLOAD_TIMEOUT DOWNLOAD MD5 CAT CUT GREP
-	export NOISY_BUILD
+	export NOISY_BUILD AST_DEVMODE
 	${GNU_MAKE} --quiet --no-print-directory -C ${PJPROJECT_DIR} \
 		PJPROJECT_CONFIGURE_OPTS="$PJPROJECT_CONFIGURE_OPTS" \
 		EXTERNALS_CACHE_DIR="${EXTERNALS_CACHE_DIR:-${AST_DOWNLOAD_CACHE}}" \
@@ -105,6 +105,7 @@ AC_DEFUN([_PJPROJECT_CONFIGURE],
 	AC_DEFINE([HAVE_PJSIP_AUTH_CLT_DEINIT], 1, [Define if your system has pjsip_auth_clt_deinit declared.])
 	AC_DEFINE([HAVE_PJSIP_TSX_LAYER_FIND_TSX2], 1, [Define if your system has pjsip_tsx_layer_find_tsx2 declared.])
 	AC_DEFINE([HAVE_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS], 1, [Define if your system has HAVE_PJSIP_INV_ACCEPT_MULTIPLE_SDP_ANSWERS declared.])
+	AC_DEFINE([HAVE_PJSIP_ENDPOINT_COMPACT_FORM], 1, [Define if your system has HAVE_PJSIP_ENDPOINT_COMPACT_FORM declared.])
 
 	AC_SUBST([PJPROJECT_BUNDLED])
 	AC_SUBST([PJPROJECT_DIR])
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/Makefile 1:16.1.1~dfsg-1/third-party/pjproject/Makefile
--- 1:13.23.1~dfsg-2/third-party/pjproject/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -54,18 +54,16 @@ ifeq ($(SPECIAL_TARGETS),)
         include source/build.mak
         CF := $(filter-out -W%,$(CC_CFLAGS))
         CF := $(filter-out -I%,$(CF))
-        ifeq ($(findstring TEST_FRAMEWORK,$(MENUSELECT_CFLAGS)),TEST_FRAMEWORK)
+        ifeq ($(AST_DEVMODE),yes)
             apps := source/pjsip-apps/bin/pjsua-$(TARGET_NAME) source/pjsip-apps/bin/pjsystest-$(TARGET_NAME)
             TARGETS += $(apps)
             ifneq ($(PYTHONDEV_LIB),)
                 TARGETS += source/pjsip-apps/src/python/_pjsua.so
             endif
+            CF += -DPJPROJECT_BUNDLED_ASSERTIONS=yes
         endif
-        ifeq ($(findstring MALLOC_DEBUG,$(MENUSELECT_CFLAGS)),MALLOC_DEBUG)
-            CF += -DMALLOC_DEBUG
-            MALLOC_DEBUG_LIBS = source/pjsip-apps/lib/libasterisk_malloc_debug.a
-            MALLOC_DEBUG_LDFLAGS = -L$(PJDIR)/pjsip-apps/lib -Wl,-whole-archive -lasterisk_malloc_debug -Wl,-no-whole-archive
-        endif
+        MALLOC_DEBUG_LIBS = source/pjsip-apps/lib/libasterisk_malloc_debug.a
+        MALLOC_DEBUG_LDFLAGS = -L$(PJDIR)/pjsip-apps/lib -Wl,-whole-archive -lasterisk_malloc_debug -Wl,-no-whole-archive
         ifeq ($(findstring DONT_OPTIMIZE,$(MENUSELECT_CFLAGS)),)
             CF += -O3
         endif
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/Makefile.rules 1:16.1.1~dfsg-1/third-party/pjproject/Makefile.rules
--- 1:13.23.1~dfsg-2/third-party/pjproject/Makefile.rules	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/Makefile.rules	2018-12-26 18:45:13.000000000 +0000
@@ -30,7 +30,6 @@ PJPROJECT_CONFIG_OPTS = $(PJPROJECT_CONF
 	--disable-v4l2 \
 	--disable-sound \
 	--disable-ext-sound \
-	--disable-oss \
 	--disable-sdl \
 	--disable-libyuv \
 	--disable-ffmpeg \
@@ -40,7 +39,7 @@ PJPROJECT_CONFIG_OPTS = $(PJPROJECT_CONF
 	--without-external-pa \
 	--without-external-srtp
 
-ifeq ($(findstring TEST_FRAMEWORK,$(MENUSELECT_CFLAGS)),)
+ifneq ($(AST_DEVMODE),yes)
     PJPROJECT_CONFIG_OPTS += --disable-resample --disable-g711-codec
 endif
 
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-configure-ssl-library-path.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-configure-ssl-library-path.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-configure-ssl-library-path.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-configure-ssl-library-path.patch	2018-12-26 18:45:13.000000000 +0000
@@ -1,18 +1,47 @@
---- a/aconfigure.ac	(PJSIP 2.7.2)
-+++ b/aconfigure.ac	(working copy)
-@@ -1571 +1571,5 @@
+From e8000cc80e5f8ba02cc52852edc02cdb0e949525 Mon Sep 17 00:00:00 2001
+From: Richard Mudgett <rmudgett@digium.com>
+Date: Mon, 6 Aug 2018 11:24:25 -0500
+Subject: [PATCH 1/5] 0000-configure-ssl-library-path.patch
+
+---
+ aconfigure    | 6 +++++-
+ aconfigure.ac | 6 +++++-
+ 2 files changed, 10 insertions(+), 2 deletions(-)
+
+diff --git a/aconfigure b/aconfigure
+index 1c449b8..c4c6060 100755
+--- a/aconfigure
++++ b/aconfigure
+@@ -7954,7 +7954,11 @@ else
+                 if test "x$with_ssl" != "xno" -a "x$with_ssl" != "x"; then
+                     CFLAGS="$CFLAGS -I$with_ssl/include"
+                     CPPFLAGS="$CPPFLAGS -I$with_ssl/include"
 -                    LDFLAGS="$LDFLAGS -L$with_ssl/lib"
 +                    if test -d $with_ssl/lib; then
 +                        LDFLAGS="$LDFLAGS -L$with_ssl/lib"
 +                    else
 +                        LDFLAGS="$LDFLAGS -L$with_ssl"
 +                    fi
---- a/aconfigure	(PJSIP 2.7.2)
-+++ b/aconfigure	(working copy)
-@@ -7884 +7884,5 @@
+                     { $as_echo "$as_me:${as_lineno-$LINENO}: result: Using SSL prefix... $with_ssl" >&5
+ $as_echo "Using SSL prefix... $with_ssl" >&6; }
+                 fi
+diff --git a/aconfigure.ac b/aconfigure.ac
+index 2c272cd..a5d6d97 100644
+--- a/aconfigure.ac
++++ b/aconfigure.ac
+@@ -1580,7 +1580,11 @@ AC_ARG_ENABLE(ssl,
+                 if test "x$with_ssl" != "xno" -a "x$with_ssl" != "x"; then
+                     CFLAGS="$CFLAGS -I$with_ssl/include"
+                     CPPFLAGS="$CPPFLAGS -I$with_ssl/include"
 -                    LDFLAGS="$LDFLAGS -L$with_ssl/lib"
 +                    if test -d $with_ssl/lib; then
 +                        LDFLAGS="$LDFLAGS -L$with_ssl/lib"
 +                    else
 +                        LDFLAGS="$LDFLAGS -L$with_ssl"
 +                    fi
+                     AC_MSG_RESULT([Using SSL prefix... $with_ssl])
+                 fi
+ 
+-- 
+2.7.4
+
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-remove-third-party.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-remove-third-party.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-remove-third-party.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-remove-third-party.patch	2018-12-26 18:45:13.000000000 +0000
@@ -1,14 +1,14 @@
-From 1ec66eb253c603e06fedff5710e701dc0e55c697 Mon Sep 17 00:00:00 2001
+From 665a2fbc3a09a71cd77988ae2deb3f5d3e205f63 Mon Sep 17 00:00:00 2001
 From: Richard Mudgett <rmudgett@digium.com>
 Date: Thu, 23 Feb 2017 17:10:07 -0600
-Subject: [PATCH 1/3] 0000-remove-third-party.patch
+Subject: [PATCH 2/5] 0000-remove-third-party.patch
 
 ---
- build.mak.in | 107 -----------------------------------------------------------
- 1 file changed, 107 deletions(-)
+ build.mak.in | 97 ------------------------------------------------------------
+ 1 file changed, 97 deletions(-)
 
 diff --git a/build.mak.in b/build.mak.in
-index 6c263a9..36e7ff4 100644
+index 80ccad1..41ec64e 100644
 --- a/build.mak.in
 +++ b/build.mak.in
 @@ -1,4 +1,3 @@
@@ -16,7 +16,7 @@ index 6c263a9..36e7ff4 100644
  include $(PJDIR)/version.mak
  export PJ_DIR := $(PJDIR)
  
-@@ -28,19 +27,6 @@ export APP_THIRD_PARTY_EXT :=
+@@ -37,19 +36,6 @@ export APP_THIRD_PARTY_EXT :=
  export APP_THIRD_PARTY_LIBS :=
  export APP_THIRD_PARTY_LIB_FILES :=
  
@@ -36,7 +36,7 @@ index 6c263a9..36e7ff4 100644
  ifeq (@ac_pjmedia_resample@,libresample)
  APP_THIRD_PARTY_LIB_FILES += $(PJ_DIR)/third_party/lib/libresample-$(LIB_SUFFIX)
  ifeq ($(PJ_SHARED_LIBRARIES),)
-@@ -57,99 +43,6 @@ APP_THIRD_PARTY_LIB_FILES += $(PJ_DIR)/third_party/lib/libresample.$(SHLIB_SUFFI
+@@ -66,89 +52,6 @@ APP_THIRD_PARTY_LIB_FILES += $(PJ_DIR)/third_party/lib/libresample.$(SHLIB_SUFFI
  endif
  endif
  
@@ -89,19 +89,9 @@ index 6c263a9..36e7ff4 100644
 -endif
 -endif
 -
--ifneq ($(findstring pa,@ac_pjmedia_snd@),)
 -ifeq (@ac_external_pa@,1)
 -# External PA
 -APP_THIRD_PARTY_EXT += -lportaudio
--else
--APP_THIRD_PARTY_LIB_FILES += $(PJ_DIR)/third_party/lib/libportaudio-$(LIB_SUFFIX)
--ifeq ($(PJ_SHARED_LIBRARIES),)
--APP_THIRD_PARTY_LIBS += -lportaudio-$(TARGET_NAME)
--else
--APP_THIRD_PARTY_LIBS += -lportaudio
--APP_THIRD_PARTY_LIB_FILES += $(PJ_DIR)/third_party/lib/libportaudio.$(SHLIB_SUFFIX).$(PJ_VERSION_MAJOR) $(PJ_DIR)/third_party/lib/libportaudio.$(SHLIB_SUFFIX)
--endif
--endif
 -endif
 -
 -ifneq (@ac_no_yuv@,1)
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-set_apps_initial_log_level.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-set_apps_initial_log_level.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-set_apps_initial_log_level.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-set_apps_initial_log_level.patch	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
-From 34f25c67f17be80814ddb16edc6c76d30213ca1b Mon Sep 17 00:00:00 2001
+From c40ad6ba454fdf6456d8ffa92faa4cd49f2c807d Mon Sep 17 00:00:00 2001
 From: Richard Mudgett <rmudgett@digium.com>
 Date: Thu, 23 Feb 2017 17:11:00 -0600
-Subject: [PATCH 2/3] 0000-set_apps_initial_log_level.patch
+Subject: [PATCH 3/5] 0000-set_apps_initial_log_level.patch
 
 ---
  pjsip-apps/src/pjsua/main.c             | 2 ++
@@ -35,10 +35,10 @@ index 122cdc7..dc79eab 100644
  	return 1;
  
 diff --git a/pjsip-apps/src/python/_pjsua.c b/pjsip-apps/src/python/_pjsua.c
-index fb80e23..c9b21d8 100644
+index 31b835e..3e15030 100644
 --- a/pjsip-apps/src/python/_pjsua.c
 +++ b/pjsip-apps/src/python/_pjsua.c
-@@ -4437,7 +4437,8 @@ init_pjsua(void)
+@@ -4434,7 +4434,8 @@ init_pjsua(void)
      PyObject* m = NULL;
  #define ADD_CONSTANT(mod,name)	PyModule_AddIntConstant(mod,#name,name)
  
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-solaris.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-solaris.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0000-solaris.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0000-solaris.patch	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
-From 8f8a9dc7203bfcd5557ad5c2f15359308535c4bb Mon Sep 17 00:00:00 2001
+From 1ac599a0f29500a15faf0dbbdc2565cc7dce2420 Mon Sep 17 00:00:00 2001
 From: Shaun Ruffell <sruffell@digium.com>
 Date: Fri, 7 Sep 2012 14:31:19 -0500
-Subject: [PATCH] pjproject: Fix for Solaris builds. Do not undef s_addr.
+Subject: [PATCH 4/5] pjproject: Fix for Solaris builds. Do not undef s_addr.
 
 pjproject, in order to solve build problems on Windows [1], undefines s_addr in
 one of it's headers that is included in res_rtp_asterisk.c. On Solaris s_addr is
@@ -38,16 +38,15 @@ Reported-by: Ben Klang
 
 Updated by ASTERISK-27997
 ---
- pjlib/include/pj/sock.h          | 8 +++++++-
- pjlib/src/pj/sock_bsd.c          | 2 +-
- pjlib/src/pj/sock_linux_kernel.c | 2 +-
- pjlib/src/pj/sock_symbian.cpp    | 2 +-
- pjlib/src/pj/sock_uwp.cpp        | 2 +-
- pjsip/src/test/transport_test.c  | 2 +-
- 6 files changed, 12 insertions(+), 6 deletions(-)
+ pjlib/include/pj/sock.h         | 8 +++++++-
+ pjlib/src/pj/sock_bsd.c         | 2 +-
+ pjlib/src/pj/sock_symbian.cpp   | 2 +-
+ pjlib/src/pj/sock_uwp.cpp       | 2 +-
+ pjsip/src/test/transport_test.c | 2 +-
+ 5 files changed, 11 insertions(+), 5 deletions(-)
 
 diff --git a/pjlib/include/pj/sock.h b/pjlib/include/pj/sock.h
-index 8fc833c..c415358 100644
+index 4daf298..c35833c 100644
 --- a/pjlib/include/pj/sock.h
 +++ b/pjlib/include/pj/sock.h
 @@ -484,6 +484,7 @@ typedef enum pj_socket_sd_type
@@ -70,7 +69,7 @@ index 8fc833c..c415358 100644
  
  /**
   * Maximum length of text representation of an IPv4 address.
-@@ -714,7 +720,7 @@ PJ_DECL(char*) pj_inet_ntoa(pj_in_addr inaddr);
+@@ -712,7 +718,7 @@ PJ_DECL(char*) pj_inet_ntoa(pj_in_addr inaddr);
   *
   * @return	nonzero if the address is valid, zero if not.
   */
@@ -80,7 +79,7 @@ index 8fc833c..c415358 100644
  /**
   * This function converts an address in its standard text presentation form
 diff --git a/pjlib/src/pj/sock_bsd.c b/pjlib/src/pj/sock_bsd.c
-index 95c3804..daefc54 100644
+index e416991..940fce1 100644
 --- a/pjlib/src/pj/sock_bsd.c
 +++ b/pjlib/src/pj/sock_bsd.c
 @@ -244,7 +244,7 @@ PJ_DEF(char*) pj_inet_ntoa(pj_in_addr inaddr)
@@ -92,19 +91,6 @@ index 95c3804..daefc54 100644
  {
      char tempaddr[PJ_INET_ADDRSTRLEN];
  
-diff --git a/pjlib/src/pj/sock_linux_kernel.c b/pjlib/src/pj/sock_linux_kernel.c
-index 899b486..10d84cd 100644
---- a/pjlib/src/pj/sock_linux_kernel.c
-+++ b/pjlib/src/pj/sock_linux_kernel.c
-@@ -149,7 +149,7 @@ PJ_DEF(char*) pj_inet_ntoa(pj_in_addr in)
-  * numbers-and-dots notation into binary data and stores it in the structure
-  * that inp points to. 
-  */
--PJ_DEF(int) pj_inet_aton(const pj_str_t *ccp, struct pj_in_addr *addr)
-+PJ_DEF(int) pj_inet_aton(const pj_str_t *ccp, pj_in_addr *addr)
- {
-     pj_uint32_t val;
-     int base, n;
 diff --git a/pjlib/src/pj/sock_symbian.cpp b/pjlib/src/pj/sock_symbian.cpp
 index 09239b0..e72bbda 100644
 --- a/pjlib/src/pj/sock_symbian.cpp
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0021-sip_parser-Fix-return-code-in-pjsip_find_msg-and-add.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0021-sip_parser-Fix-return-code-in-pjsip_find_msg-and-add.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0021-sip_parser-Fix-return-code-in-pjsip_find_msg-and-add.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0021-sip_parser-Fix-return-code-in-pjsip_find_msg-and-add.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,41 +0,0 @@
-From 186f82627c40d0c3a56a6a94ce55c055ad1f7620 Mon Sep 17 00:00:00 2001
-From: George Joseph <gjoseph@digium.com>
-Date: Fri, 10 Nov 2017 09:26:29 -0700
-Subject: [PATCH] sip_parser: Fix return code in pjsip_find_msg and add
- "volatile"
-
-The default return code for pjsip_find_msg was PJ_SUCCESS so if
-a Content-Length header wasn't found at all, pjsip_find_msg was
-returning PJ_SUCCESS instead of PJSIP_EMISSINGHDR.
-
-Also added the volatile keyword to a few variables what are used
-both insude and outsude the PJ_TRY/PJ_CATCH block.
----
- pjsip/src/pjsip/sip_parser.c | 8 ++++----
- 1 file changed, 4 insertions(+), 4 deletions(-)
-
-diff --git a/pjsip/src/pjsip/sip_parser.c b/pjsip/src/pjsip/sip_parser.c
-index f9a0e65b5..f65ffca5a 100644
---- a/pjsip/src/pjsip/sip_parser.c
-+++ b/pjsip/src/pjsip/sip_parser.c
-@@ -834,13 +834,13 @@ PJ_DEF(pj_status_t) pjsip_find_msg( const char *buf, pj_size_t size,
- 				  pj_bool_t is_datagram, pj_size_t *msg_size)
- {
- #if PJ_HAS_TCP
--    const char *hdr_end;
--    const char *body_start;
-+    const char *volatile hdr_end;
-+    const char *volatile body_start;
-     const char *pos;
--    const char *line;
-+    const char *volatile line;
-     int content_length = -1;
-     pj_str_t cur_msg;
--    pj_status_t status = PJ_SUCCESS;
-+    volatile pj_status_t status = PJSIP_EMISSINGHDR;
-     const pj_str_t end_hdr = { "\n\r\n", 3};
- 
-     *msg_size = size;
--- 
-2.13.6
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0030-sip_transport-Destroy-transports-not-in-hash.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0030-sip_transport-Destroy-transports-not-in-hash.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0030-sip_transport-Destroy-transports-not-in-hash.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0030-sip_transport-Destroy-transports-not-in-hash.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,27 +0,0 @@
-diff --git a/pjsip/src/pjsip/sip_transport.c b/pjsip/src/pjsip/sip_transport.c
-index e4bec24..a39b56e 100644
---- a/pjsip/src/pjsip/sip_transport.c
-+++ b/pjsip/src/pjsip/sip_transport.c
-@@ -957,7 +957,21 @@ static pj_bool_t is_transport_valid(pjsip_tpmgr *tpmgr, pjsip_transport *tp,
- 				    const pjsip_transport_key *key,
- 				    int key_len)
- {
--    return (pj_hash_get(tpmgr->table, key, key_len, NULL) == (void*)tp);
-+    transport *tp_iter;
-+
-+    if (pj_hash_get(tpmgr->table, key, key_len, NULL) == (void*)tp) {
-+        return PJ_TRUE;
-+    }
-+
-+    tp_iter = tpmgr->tp_list.next;
-+    while (tp_iter != &tpmgr->tp_list) {
-+        if (tp_iter->tp == tp) {
-+            return PJ_TRUE;
-+        }
-+        tp_iter = tp_iter->next;
-+    }
-+
-+    return PJ_FALSE;
- }
- 
- /*
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0040-183_without_to_tag.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0040-183_without_to_tag.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0040-183_without_to_tag.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0040-183_without_to_tag.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,17 +0,0 @@
-diff --git a/pjsip/src/pjsip-ua/sip_inv.c b/pjsip/src/pjsip-ua/sip_inv.c
-index c9686a0..fc52a63 100644
---- a/pjsip/src/pjsip-ua/sip_inv.c
-+++ b/pjsip/src/pjsip-ua/sip_inv.c
-@@ -4156,9 +4156,10 @@ static void inv_on_state_calling( pjsip_inv_session *inv, pjsip_event *e)
- 		    status = pjsip_inv_send_msg(inv, cancel);
- 	    }
- 
--	    if (dlg->remote.info->tag.slen) {
-+	    if (tsx->status_code != 100) {
- 
--		inv_set_state(inv, PJSIP_INV_STATE_EARLY, e);
-+		if (dlg->remote.info->tag.slen)
-+		    inv_set_state(inv, PJSIP_INV_STATE_EARLY, e);
- 
- 		inv_check_sdp_in_incoming_msg(inv, tsx, 
- 					      e->body.tsx_state.src.rdata);
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0050-dont_terminate_session_early.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0050-dont_terminate_session_early.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0050-dont_terminate_session_early.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0050-dont_terminate_session_early.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,71 +0,0 @@
-commit ca0b723e92bd76bbda1bbd14477a829eaeeb675e
-Author: Joshua Colp <jcolp@digium.com>
-Date:   Wed Dec 13 10:58:57 2017 +0000
-
-    Ignore transport error on completed transaction.
-    Don't disconnect call if transport error happens on transaction that is not initial INVITE transaction.
-    
-    Scenario:
-    
-    DNS lookup returning two servers.
-    Sending INVITE to first server over TCP.
-    Response received with code 503 (Service Unavailable).
-    Failover to second server, sending second INVITE after restarting the session.
-    TCP connection for the first INVITE getting disconnected and causing call disconnection (while second INVITE is still outstanding).
-    
-    This is a backport of 5714 from upstream PJSIP.
-
-diff --git a/pjsip/src/pjsip-ua/sip_inv.c b/pjsip/src/pjsip-ua/sip_inv.c
-index ac4d1949..0173cb4c 100644
---- a/pjsip/src/pjsip-ua/sip_inv.c
-+++ b/pjsip/src/pjsip-ua/sip_inv.c
-@@ -4254,8 +4254,7 @@ static void inv_on_state_calling( pjsip_inv_session *inv, pjsip_event *e)
- 	if ((tsx->status_code == PJSIP_SC_CALL_TSX_DOES_NOT_EXIST &&
- 		tsx->method.id != PJSIP_CANCEL_METHOD) ||
- 	    tsx->status_code == PJSIP_SC_REQUEST_TIMEOUT ||
--	    tsx->status_code == PJSIP_SC_TSX_TIMEOUT ||
--	    tsx->status_code == PJSIP_SC_TSX_TRANSPORT_ERROR)
-+	    tsx->status_code == PJSIP_SC_TSX_TIMEOUT)
- 	{
- 	    inv_set_cause(inv, tsx->status_code, &tsx->status_text);
- 	    inv_set_state(inv, PJSIP_INV_STATE_DISCONNECTED, e);
-diff --git a/pjsip/src/pjsip/sip_transaction.c b/pjsip/src/pjsip/sip_transaction.c
-index 7ac3d1b7..d52b12a7 100644
---- a/pjsip/src/pjsip/sip_transaction.c
-+++ b/pjsip/src/pjsip/sip_transaction.c
-@@ -2044,9 +2044,14 @@ static void transport_callback(void *token, pjsip_tx_data *tdata,
- 	 */
- 	lock_timer(tsx);
- 	tsx->transport_err = (pj_status_t)-sent;
--	tsx_cancel_timer(tsx, &tsx->timeout_timer);
--	tsx_schedule_timer(tsx, &tsx->timeout_timer, &delay,
--	                   TRANSPORT_ERR_TIMER);
-+	/* Don't cancel timeout timer if tsx state is already
-+	 * PJSIP_TSX_STATE_COMPLETED (see #2076).
-+	 */
-+	if (tsx->state < PJSIP_TSX_STATE_COMPLETED) {
-+	    tsx_cancel_timer(tsx, &tsx->timeout_timer);
-+	    tsx_schedule_timer(tsx, &tsx->timeout_timer, &delay,
-+			       TRANSPORT_ERR_TIMER);
-+	}
- 	unlock_timer(tsx);
-    }
- 
-@@ -2077,9 +2082,14 @@ static void tsx_tp_state_callback( pjsip_transport *tp,
- 	 */
- 	lock_timer(tsx);
- 	tsx->transport_err = info->status;
--	tsx_cancel_timer(tsx, &tsx->timeout_timer);
--	tsx_schedule_timer(tsx, &tsx->timeout_timer, &delay,
--	                   TRANSPORT_ERR_TIMER);
-+	/* Don't cancel timeout timer if tsx state is already
-+	 * PJSIP_TSX_STATE_COMPLETED (see #2076).
-+	 */
-+	if (tsx->state < PJSIP_TSX_STATE_COMPLETED) {
-+	    tsx_cancel_timer(tsx, &tsx->timeout_timer);
-+	    tsx_schedule_timer(tsx, &tsx->timeout_timer, &delay,
-+			       TRANSPORT_ERR_TIMER);
-+	}
- 	unlock_timer(tsx);
-     }
- }
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0060-sip_msg-Prevent-crash-on-header-without-vptr.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0060-sip_msg-Prevent-crash-on-header-without-vptr.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0060-sip_msg-Prevent-crash-on-header-without-vptr.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0060-sip_msg-Prevent-crash-on-header-without-vptr.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,56 +0,0 @@
-From 88409082a4cbf27b308c76c148270ee72fdcb503 Mon Sep 17 00:00:00 2001
-From: Riza Sulistyo <riza@teluu.com>
-Date: Wed, 17 Jan 2018 11:30:52 +0000
-Subject: [PATCH] r5727 svn backport sip_msg: Prevent crash on header without
- vptr
-
-Re #2059 (misc): Prevent corrupt header from causing a crash when printed.
-Thanks to George Joseph for the patch.
-
-Occasionally a header with no vptr gets into the list of header.  This
-causes a crash when printing them.  We still need to figure out how
-the header got there but this patch at least prevents the crash by checking
-for a non-NULL vptr before attempting to call its print function.
----
- pjsip/src/pjsip/sip_msg.c | 11 +++++++++--
- 1 file changed, 9 insertions(+), 2 deletions(-)
-
-diff --git a/pjsip/src/pjsip/sip_msg.c b/pjsip/src/pjsip/sip_msg.c
-index a13f80e..8819dd9 100644
---- a/pjsip/src/pjsip/sip_msg.c
-+++ b/pjsip/src/pjsip/sip_msg.c
-@@ -26,6 +26,7 @@
- #include <pj/string.h>
- #include <pj/pool.h>
- #include <pj/assert.h>
-+#include <pj/log.h>
- #include <pjlib-util/string.h>
- 
- PJ_DEF_DATA(const pjsip_method) pjsip_invite_method =
-@@ -461,8 +462,13 @@ PJ_DEF(pj_ssize_t) pjsip_msg_print( const pjsip_msg *msg,
-     /* Print each of the headers. */
-     for (hdr=msg->hdr.next; hdr!=&msg->hdr; hdr=hdr->next) {
- 	len = pjsip_hdr_print_on(hdr, p, end-p);
--	if (len < 0)
--	    return -1;
-+	if (len < 0) {
-+	   if (len == -2) {
-+	       PJ_LOG(5, ("sip_msg", "Header with no vptr encountered!! "\
-+			  "Current buffer: %.*s", (int)(p-buf), buf));
-+	   }
-+	   return len;
-+	}
- 
- 	if (len > 0) {
- 	    p += len;
-@@ -578,6 +584,7 @@ PJ_DEF(void*) pjsip_hdr_shallow_clone( pj_pool_t *pool, const void *hdr_ptr )
- PJ_DEF(int) pjsip_hdr_print_on( void *hdr_ptr, char *buf, pj_size_t len)
- {
-     pjsip_hdr *hdr = (pjsip_hdr*) hdr_ptr;
-+    PJ_ASSERT_RETURN(hdr->vptr, -2);
-     return (*hdr->vptr->print_on)(hdr_ptr, buf, len);
- }
- 
--- 
-2.7.4
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0070-os_core_unix-Set-mutex-NULL-in-atomic-destroy-and-ad.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0070-os_core_unix-Set-mutex-NULL-in-atomic-destroy-and-ad.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0070-os_core_unix-Set-mutex-NULL-in-atomic-destroy-and-ad.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0070-os_core_unix-Set-mutex-NULL-in-atomic-destroy-and-ad.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,114 +0,0 @@
-From 67485f3a6c711f67a40ff46288cb6be1658023bd Mon Sep 17 00:00:00 2001
-From: nanang <nanang@localhost>
-Date: Mon, 26 Mar 2018 10:33:50 +0000
-Subject: [PATCH] Close #2101:  - set atomic's mutex to NULL in atomic destroy 
- - added few sanity checks to the atomic functions.
-
----
- pjlib/src/pj/os_core_unix.c  | 20 ++++++++++++++++++--
- pjlib/src/pj/os_core_win32.c |  4 ++++
- 2 files changed, 22 insertions(+), 2 deletions(-)
-
-diff --git a/pjlib/src/pj/os_core_unix.c b/pjlib/src/pj/os_core_unix.c
-index ebfe84348..c17ad4ef0 100644
---- a/pjlib/src/pj/os_core_unix.c
-+++ b/pjlib/src/pj/os_core_unix.c
-@@ -879,9 +879,16 @@ PJ_DEF(pj_status_t) pj_atomic_create( pj_pool_t *pool,
-  */
- PJ_DEF(pj_status_t) pj_atomic_destroy( pj_atomic_t *atomic_var )
- {
-+    pj_status_t status;
-+
-     PJ_ASSERT_RETURN(atomic_var, PJ_EINVAL);
-+    
- #if PJ_HAS_THREADS
--    return pj_mutex_destroy( atomic_var->mutex );
-+    status = pj_mutex_destroy( atomic_var->mutex );
-+    if (status == PJ_SUCCESS) {
-+        atomic_var->mutex = NULL;
-+    }
-+    return status;
- #else
-     return 0;
- #endif
-@@ -892,10 +899,16 @@ PJ_DEF(pj_status_t) pj_atomic_destroy( pj_atomic_t *atomic_var )
-  */
- PJ_DEF(void) pj_atomic_set(pj_atomic_t *atomic_var, pj_atomic_value_t value)
- {
-+    pj_status_t status;
-+
-     PJ_CHECK_STACK();
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
- 
- #if PJ_HAS_THREADS
--    pj_mutex_lock( atomic_var->mutex );
-+    status = pj_mutex_lock( atomic_var->mutex );
-+    if (status != PJ_SUCCESS) {
-+        return;
-+    }
- #endif
-     atomic_var->value = value;
- #if PJ_HAS_THREADS
-@@ -946,6 +959,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_inc_and_get(pj_atomic_t *atomic_var)
-  */
- PJ_DEF(void) pj_atomic_inc(pj_atomic_t *atomic_var)
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
-     pj_atomic_inc_and_get(atomic_var);
- }
- 
-@@ -974,6 +988,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_dec_and_get(pj_atomic_t *atomic_var)
-  */
- PJ_DEF(void) pj_atomic_dec(pj_atomic_t *atomic_var)
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
-     pj_atomic_dec_and_get(atomic_var);
- }
- 
-@@ -1005,6 +1020,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_add_and_get( pj_atomic_t *atomic_var,
- PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var,
-                             pj_atomic_value_t value )
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
-     pj_atomic_add_and_get(atomic_var, value);
- }
- 
-diff --git a/pjlib/src/pj/os_core_win32.c b/pjlib/src/pj/os_core_win32.c
-index 1cb6004d3..8c934b34d 100644
---- a/pjlib/src/pj/os_core_win32.c
-+++ b/pjlib/src/pj/os_core_win32.c
-@@ -750,6 +750,7 @@ PJ_DEF(pj_status_t) pj_atomic_destroy( pj_atomic_t *var )
- PJ_DEF(void) pj_atomic_set( pj_atomic_t *atomic_var, pj_atomic_value_t value)
- {
-     PJ_CHECK_STACK();
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
- 
-     InterlockedExchange(&atomic_var->value, value);
- }
-@@ -784,6 +785,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_inc_and_get(pj_atomic_t *atomic_var)
-  */
- PJ_DEF(void) pj_atomic_inc(pj_atomic_t *atomic_var)
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
-     pj_atomic_inc_and_get(atomic_var);
- }
- 
-@@ -806,6 +808,7 @@ PJ_DEF(pj_atomic_value_t) pj_atomic_dec_and_get(pj_atomic_t *atomic_var)
-  */
- PJ_DEF(void) pj_atomic_dec(pj_atomic_t *atomic_var)
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
-     pj_atomic_dec_and_get(atomic_var);
- }
- 
-@@ -815,6 +818,7 @@ PJ_DEF(void) pj_atomic_dec(pj_atomic_t *atomic_var)
- PJ_DEF(void) pj_atomic_add( pj_atomic_t *atomic_var,
- 			    pj_atomic_value_t value )
- {
-+    PJ_ASSERT_ON_FAIL(atomic_var, return);
- #if defined(PJ_WIN32_WINNT) && PJ_WIN32_WINNT >= 0x0400
-     InterlockedExchangeAdd( &atomic_var->value, value );
- #else
--- 
-2.14.3
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0080-timer-Clean-up-usage-of-timer-heap.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0080-timer-Clean-up-usage-of-timer-heap.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0080-timer-Clean-up-usage-of-timer-heap.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0080-timer-Clean-up-usage-of-timer-heap.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,434 +0,0 @@
-From 853005378de2ffecee7774e095d8cbfbfa0ab706 Mon Sep 17 00:00:00 2001
-From: George Joseph <gjoseph@digium.com>
-Date: Tue, 2 Jan 2018 06:36:46 -0700
-Subject: [PATCH] timer: Clean up usage of timer heap
-
-Added a new pj_timer_entry_reset function that resets a timer_entry
-for re-use.
-
-Changed direct settings of timer_entry fields to use
-pj_timer_entry_init and pj_timer_entry_reset.
-
-Fixed issues where timers were being rescheduled incorrectly.
----
- pjlib/include/pj/timer.h             | 14 ++++++++++++++
- pjlib/src/pj/ssl_sock_ossl.c         |  8 +++++---
- pjlib/src/pj/timer.c                 | 12 ++++++++++++
- pjnath/src/pjnath/ice_session.c      |  9 ++++++++-
- pjnath/src/pjnath/nat_detect.c       |  2 ++
- pjnath/src/pjnath/stun_sock.c        |  2 +-
- pjnath/src/pjnath/stun_transaction.c | 10 +++++-----
- pjnath/src/pjnath/turn_session.c     |  3 +++
- pjnath/src/pjnath/turn_sock.c        |  1 +
- pjnath/src/pjturn-srv/allocation.c   |  4 ++--
- pjnath/src/pjturn-srv/listener_tcp.c |  2 +-
- pjsip/src/pjsip-simple/evsub.c       |  6 +++---
- pjsip/src/pjsip/sip_endpoint.c       |  4 +++-
- pjsip/src/pjsip/sip_transaction.c    |  9 +++------
- pjsip/src/pjsip/sip_transport.c      |  3 +--
- 15 files changed, 64 insertions(+), 25 deletions(-)
-
-diff --git a/pjlib/include/pj/timer.h b/pjlib/include/pj/timer.h
-index df6155a81..90fc8ac85 100644
---- a/pjlib/include/pj/timer.h
-+++ b/pjlib/include/pj/timer.h
-@@ -212,6 +212,20 @@ PJ_DECL(pj_timer_entry*) pj_timer_entry_init( pj_timer_entry *entry,
-                                               void *user_data,
-                                               pj_timer_heap_callback *cb );
- 
-+/**
-+ * Reset a timer entry. Application should call this function before reusing
-+ * the timer entry.
-+ *
-+ * @param entry     The timer entry to be initialized.
-+ * @param id        Arbitrary ID assigned by the user/owner of this entry.
-+ *                  Applications can use this ID to distinguish multiple
-+ *                  timer entries that share the same callback and user_data.
-+ *
-+ * @return          The timer entry itself.
-+ */
-+PJ_DECL(pj_timer_entry*) pj_timer_entry_reset( pj_timer_entry *entry,
-+                                             int id);
-+
- /**
-  * Queries whether a timer entry is currently running.
-  *
-diff --git a/pjlib/src/pj/ssl_sock_ossl.c b/pjlib/src/pj/ssl_sock_ossl.c
-index 969cc1420..ead1a8fbb 100644
---- a/pjlib/src/pj/ssl_sock_ossl.c
-+++ b/pjlib/src/pj/ssl_sock_ossl.c
-@@ -291,6 +291,7 @@ struct pj_ssl_cert_t
- static write_data_t* alloc_send_data(pj_ssl_sock_t *ssock, pj_size_t len);
- static void free_send_data(pj_ssl_sock_t *ssock, write_data_t *wdata);
- static pj_status_t flush_delayed_send(pj_ssl_sock_t *ssock);
-+static void on_timer(pj_timer_heap_t *th, struct pj_timer_entry *te);
- 
- /*
-  *******************************************************************
-@@ -1621,7 +1622,8 @@ static pj_bool_t on_handshake_complete(pj_ssl_sock_t *ssock,
- 		    pj_timer_heap_cancel(ssock->param.timer_heap,
- 					 &ssock->timer);
- 		}
--		ssock->timer.id = TIMER_CLOSE;
-+
-+		pj_timer_entry_reset(&ssock->timer, TIMER_CLOSE);
- 		pj_time_val_normalize(&interval);
- 		if (pj_timer_heap_schedule(ssock->param.timer_heap, 
- 					   &ssock->timer, &interval) != 0)
-@@ -2387,7 +2389,7 @@ static pj_bool_t asock_on_accept_complete (pj_activesock_t *asock,
- 	ssock->param.timeout.msec != 0))
-     {
- 	pj_assert(ssock->timer.id == TIMER_NONE);
--	ssock->timer.id = TIMER_HANDSHAKE_TIMEOUT;
-+	pj_timer_entry_reset(&ssock->timer, TIMER_HANDSHAKE_TIMEOUT);
- 	status = pj_timer_heap_schedule(ssock->param.timer_heap, 
- 				        &ssock->timer,
- 					&ssock->param.timeout);
-@@ -3405,7 +3407,7 @@ PJ_DEF(pj_status_t) pj_ssl_sock_start_connect( pj_ssl_sock_t *ssock,
- 	ssock->param.timeout.msec != 0))
-     {
- 	pj_assert(ssock->timer.id == TIMER_NONE);
--	ssock->timer.id = TIMER_HANDSHAKE_TIMEOUT;
-+	pj_timer_entry_reset(&ssock->timer, TIMER_HANDSHAKE_TIMEOUT);
- 	status = pj_timer_heap_schedule(ssock->param.timer_heap,
- 					&ssock->timer,
- 				        &ssock->param.timeout);
-diff --git a/pjlib/src/pj/timer.c b/pjlib/src/pj/timer.c
-index 90a95e37b..13126116f 100644
---- a/pjlib/src/pj/timer.c
-+++ b/pjlib/src/pj/timer.c
-@@ -472,6 +472,18 @@ PJ_DEF(pj_timer_entry*) pj_timer_entry_init( pj_timer_entry *entry,
-     return entry;
- }
- 
-+PJ_DEF(pj_timer_entry*) pj_timer_entry_reset( pj_timer_entry *entry,
-+                                             int id)
-+{
-+    entry->id = id;
-+    entry->_grp_lock = NULL;
-+    entry->_timer_id = -1;
-+    entry->_timer_value = (pj_time_val){0, 0};
-+
-+    return entry;
-+}
-+
-+
- PJ_DEF(pj_bool_t) pj_timer_entry_running( pj_timer_entry *entry )
- {
-     return (entry->_timer_id >= 1);
-diff --git a/pjnath/src/pjnath/ice_session.c b/pjnath/src/pjnath/ice_session.c
-index 63a0d1c9c..6d0e6abc9 100644
---- a/pjnath/src/pjnath/ice_session.c
-+++ b/pjnath/src/pjnath/ice_session.c
-@@ -1246,6 +1246,7 @@ done:
- 		     ice->comp_cnt;
- 	pj_time_val_normalize(&delay);
- 
-+	pj_timer_entry_reset(&ice->timer, TIMER_KEEP_ALIVE);
- 	pj_timer_heap_schedule_w_grp_lock(ice->stun_cfg.timer_heap,
- 	                                  &ice->timer, &delay,
- 	                                  TIMER_KEEP_ALIVE,
-@@ -1276,7 +1277,7 @@ static void on_ice_complete(pj_ice_sess *ice, pj_status_t status)
- 	/* Call callback */
- 	if (ice->cb.on_ice_complete) {
- 	    pj_time_val delay = {0, 0};
--
-+	    pj_timer_entry_reset(&ice->timer, TIMER_COMPLETION_CALLBACK);
- 	    pj_timer_heap_schedule_w_grp_lock(ice->stun_cfg.timer_heap,
- 	                                      &ice->timer, &delay,
- 	                                      TIMER_COMPLETION_CALLBACK,
-@@ -1507,6 +1508,7 @@ static pj_bool_t on_check_complete(pj_ice_sess *ice,
- 		    delay.sec = 0;
- 		    delay.msec = ice->opt.controlled_agent_want_nom_timeout;
- 		    pj_time_val_normalize(&delay);
-+		    pj_timer_entry_reset(&ice->timer, TIMER_CONTROLLED_WAIT_NOM);
- 
- 		    pj_timer_heap_schedule_w_grp_lock(
- 					ice->stun_cfg.timer_heap,
-@@ -1597,6 +1599,7 @@ static pj_bool_t on_check_complete(pj_ice_sess *ice,
- 	delay.sec = 0;
- 	delay.msec = ice->opt.nominated_check_delay;
- 	pj_time_val_normalize(&delay);
-+        pj_timer_entry_reset(&ice->timer, TIMER_START_NOMINATED_CHECK);
- 
- 	pj_timer_heap_schedule_w_grp_lock(ice->stun_cfg.timer_heap,
- 	                                  &ice->timer, &delay,
-@@ -1929,6 +1932,8 @@ static pj_status_t start_periodic_check(pj_timer_heap_t *th,
- 	pj_time_val timeout = {0, PJ_ICE_TA_VAL};
- 
- 	pj_time_val_normalize(&timeout);
-+        pj_timer_entry_reset(&ice->timer, PJ_TRUE);
-+
- 	pj_timer_heap_schedule_w_grp_lock(th, te, &timeout, PJ_TRUE,
- 	                                  ice->grp_lock);
-     }
-@@ -1986,6 +1991,7 @@ static void start_nominated_check(pj_ice_sess *ice)
-                                    &ice->clist.timer, PJ_FALSE);
- 
-     delay.sec = delay.msec = 0;
-+    pj_timer_entry_reset(&ice->timer, PJ_TRUE);
-     status = pj_timer_heap_schedule_w_grp_lock(ice->stun_cfg.timer_heap,
-                                                &ice->clist.timer, &delay,
-                                                PJ_TRUE,
-@@ -2125,6 +2131,7 @@ PJ_DEF(pj_status_t) pj_ice_sess_start_check(pj_ice_sess *ice)
-      * return start_periodic_check(ice->stun_cfg.timer_heap, &clist->timer);
-      */
-     delay.sec = delay.msec = 0;
-+    pj_timer_entry_reset(&ice->timer, PJ_TRUE);
-     status = pj_timer_heap_schedule_w_grp_lock(ice->stun_cfg.timer_heap,
-                                                &clist->timer, &delay,
-                                                PJ_TRUE, ice->grp_lock);
-diff --git a/pjnath/src/pjnath/nat_detect.c b/pjnath/src/pjnath/nat_detect.c
-index 8a2408374..7bb364798 100644
---- a/pjnath/src/pjnath/nat_detect.c
-+++ b/pjnath/src/pjnath/nat_detect.c
-@@ -414,6 +414,7 @@ static void end_session(nat_detect_session *sess,
-     delay.msec = 0;
- 
-     sess->timer.id = TIMER_DESTROY;
-+    pj_timer_entry_init(&sess->timer, TIMER_DESTROY, sess, &on_sess_timer);
-     pj_timer_heap_schedule(sess->timer_heap, &sess->timer, &delay);
- }
- 
-@@ -933,6 +934,7 @@ static void on_sess_timer(pj_timer_heap_t *th,
- 
- 	if (next_timer) {
- 	    pj_time_val delay = {0, TEST_INTERVAL};
-+	    pj_timer_entry_reset(te, TIMER_TEST);
- 	    pj_timer_heap_schedule(th, te, &delay);
- 	} else {
- 	    te->id = 0;
-diff --git a/pjnath/src/pjnath/stun_sock.c b/pjnath/src/pjnath/stun_sock.c
-index 6028e0c47..3aab27a1d 100644
---- a/pjnath/src/pjnath/stun_sock.c
-+++ b/pjnath/src/pjnath/stun_sock.c
-@@ -864,7 +864,7 @@ static void start_ka_timer(pj_stun_sock *stun_sock)
- 
- 	delay.sec = stun_sock->ka_interval;
- 	delay.msec = 0;
--
-+	pj_timer_entry_reset(&stun_sock->ka_timer, PJ_TRUE);
- 	pj_timer_heap_schedule_w_grp_lock(stun_sock->stun_cfg.timer_heap,
- 	                                  &stun_sock->ka_timer,
- 	                                  &delay, PJ_TRUE,
-diff --git a/pjnath/src/pjnath/stun_transaction.c b/pjnath/src/pjnath/stun_transaction.c
-index 28f623005..ad87b7b6c 100644
---- a/pjnath/src/pjnath/stun_transaction.c
-+++ b/pjnath/src/pjnath/stun_transaction.c
-@@ -86,11 +86,8 @@ PJ_DEF(pj_status_t) pj_stun_client_tsx_create(pj_stun_config *cfg,
-     tsx->grp_lock = grp_lock;
-     pj_memcpy(&tsx->cb, cb, sizeof(*cb));
- 
--    tsx->retransmit_timer.cb = &retransmit_timer_callback;
--    tsx->retransmit_timer.user_data = tsx;
--
--    tsx->destroy_timer.cb = &destroy_timer_callback;
--    tsx->destroy_timer.user_data = tsx;
-+    pj_timer_entry_init(&tsx->retransmit_timer, 0, tsx, &retransmit_timer_callback);
-+    pj_timer_entry_init(&tsx->destroy_timer, 0, tsx, &destroy_timer_callback);
- 
-     pj_ansi_snprintf(tsx->obj_name, sizeof(tsx->obj_name), "utsx%p", tsx);
- 
-@@ -120,6 +117,7 @@ PJ_DEF(pj_status_t) pj_stun_client_tsx_schedule_destroy(
-     pj_timer_heap_cancel_if_active(tsx->timer_heap, &tsx->retransmit_timer,
-                                    TIMER_INACTIVE);
- 
-+    pj_timer_entry_reset(&tsx->destroy_timer, TIMER_ACTIVE);
-     status = pj_timer_heap_schedule_w_grp_lock(tsx->timer_heap,
-                                                &tsx->destroy_timer, delay,
-                                                TIMER_ACTIVE, tsx->grp_lock);
-@@ -237,6 +235,7 @@ static pj_status_t tsx_transmit_msg(pj_stun_client_tsx *tsx,
- 	 * cancel it (as opposed to when schedule_timer() failed we cannot
- 	 * cancel transmission).
- 	 */;
-+	pj_timer_entry_reset(&tsx->retransmit_timer, TIMER_ACTIVE);
- 	status = pj_timer_heap_schedule_w_grp_lock(tsx->timer_heap,
- 						   &tsx->retransmit_timer,
- 						   &tsx->retransmit_time,
-@@ -315,6 +314,7 @@ PJ_DEF(pj_status_t) pj_stun_client_tsx_send_msg(pj_stun_client_tsx *tsx,
- 	 * cancel it (as opposed to when schedule_timer() failed we cannot
- 	 * cancel transmission).
- 	 */;
-+        pj_timer_entry_reset(&tsx->retransmit_timer, TIMER_ACTIVE);
- 	status = pj_timer_heap_schedule_w_grp_lock(tsx->timer_heap,
- 	                                           &tsx->retransmit_timer,
- 	                                           &tsx->retransmit_time,
-diff --git a/pjnath/src/pjnath/turn_session.c b/pjnath/src/pjnath/turn_session.c
-index bbea027f3..e4685e625 100644
---- a/pjnath/src/pjnath/turn_session.c
-+++ b/pjnath/src/pjnath/turn_session.c
-@@ -431,6 +431,7 @@ static void sess_shutdown(pj_turn_session *sess,
- 
- 	pj_timer_heap_cancel_if_active(sess->timer_heap, &sess->timer,
- 	                               TIMER_NONE);
-+	pj_timer_entry_reset(&sess->timer, TIMER_DESTROY);
- 	pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
- 	                                  &delay, TIMER_DESTROY,
- 	                                  sess->grp_lock);
-@@ -1434,6 +1435,7 @@ static void on_allocate_success(pj_turn_session *sess,
- 	timeout.sec = sess->ka_interval;
- 	timeout.msec = 0;
- 
-+	pj_timer_entry_reset(&sess->timer, TIMER_KEEP_ALIVE);
- 	pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
- 					  &timeout, TIMER_KEEP_ALIVE,
- 					  sess->grp_lock);
-@@ -2080,6 +2082,7 @@ static void on_timer_event(pj_timer_heap_t *th, pj_timer_entry *e)
- 	    delay.sec = sess->ka_interval;
- 	    delay.msec = 0;
- 
-+	    pj_timer_entry_reset(&sess->timer, TIMER_KEEP_ALIVE);
- 	    pj_timer_heap_schedule_w_grp_lock(sess->timer_heap, &sess->timer,
- 	                                      &delay, TIMER_KEEP_ALIVE,
- 	                                      sess->grp_lock);
-diff --git a/pjnath/src/pjnath/turn_sock.c b/pjnath/src/pjnath/turn_sock.c
-index a30ab5153..507858048 100644
---- a/pjnath/src/pjnath/turn_sock.c
-+++ b/pjnath/src/pjnath/turn_sock.c
-@@ -928,6 +928,7 @@ static void turn_on_state(pj_turn_session *sess,
- 
- 	pj_timer_heap_cancel_if_active(turn_sock->cfg.timer_heap,
- 	                               &turn_sock->timer, 0);
-+	pj_timer_entry_reset(&turn_sock->timer, TIMER_DESTROY);
- 	pj_timer_heap_schedule_w_grp_lock(turn_sock->cfg.timer_heap,
- 	                                  &turn_sock->timer,
- 	                                  &delay, TIMER_DESTROY,
-diff --git a/pjnath/src/pjturn-srv/allocation.c b/pjnath/src/pjturn-srv/allocation.c
-index 6c9c9ce11..88533926b 100644
---- a/pjnath/src/pjturn-srv/allocation.c
-+++ b/pjnath/src/pjturn-srv/allocation.c
-@@ -513,7 +513,7 @@ static void alloc_shutdown(pj_turn_allocation *alloc)
-      */
- 
-     /* Schedule destroy timer */
--    alloc->relay.timer.id = TIMER_ID_DESTROY;
-+    pj_timer_entry_reset(&alloc->relay.timer, TIMER_ID_DESTROY);
-     pj_timer_heap_schedule(alloc->server->core.timer_heap,
- 			   &alloc->relay.timer, &destroy_delay);
- }
-@@ -538,7 +538,7 @@ static pj_status_t resched_timeout(pj_turn_allocation *alloc)
-     delay.sec = alloc->relay.lifetime;
-     delay.msec = 0;
- 
--    alloc->relay.timer.id = TIMER_ID_TIMEOUT;
-+    pj_timer_entry_reset(&alloc->relay.timer, TIMER_ID_TIMEOUT);
-     status = pj_timer_heap_schedule(alloc->server->core.timer_heap,
- 				    &alloc->relay.timer, &delay);
-     if (status != PJ_SUCCESS) {
-diff --git a/pjnath/src/pjturn-srv/listener_tcp.c b/pjnath/src/pjturn-srv/listener_tcp.c
-index 796ed471b..4a9550c2e 100644
---- a/pjnath/src/pjturn-srv/listener_tcp.c
-+++ b/pjnath/src/pjturn-srv/listener_tcp.c
-@@ -475,7 +475,7 @@ static void tcp_dec_ref(pj_turn_transport *tp,
- 
-     if (tcp->ref_cnt == 0 && tcp->timer.id == TIMER_NONE) {
- 	pj_time_val delay = { SHUTDOWN_DELAY, 0 };
--	tcp->timer.id = TIMER_DESTROY;
-+	pj_timer_entry_reset(&tcp->timer, TIMER_DESTROY);
- 	pj_timer_heap_schedule(tcp->base.listener->server->core.timer_heap,
- 			       &tcp->timer, &delay);
-     }
-diff --git a/pjsip/src/pjsip-simple/evsub.c b/pjsip/src/pjsip-simple/evsub.c
-index eb666654f..7748853d2 100644
---- a/pjsip/src/pjsip-simple/evsub.c
-+++ b/pjsip/src/pjsip-simple/evsub.c
-@@ -518,6 +518,7 @@ static void set_timer( pjsip_evsub *sub, int timer_id,
- 	timeout.sec = seconds;
- 	timeout.msec = 0;
- 
-+	pj_timer_entry_reset(&sub->timer, timer_id);
- 	pj_timer_heap_schedule_w_grp_lock(
- 			    pjsip_endpt_get_timer_heap(sub->endpt),
- 			    &sub->timer, &timeout, timer_id, sub->grp_lock);
-@@ -655,7 +656,7 @@ static void on_timer( pj_timer_heap_t *timer_heap,
-     /* If this timer entry has just been rescheduled or cancelled
-      * while waiting for dialog mutex, just return (see #1885 scenario 1).
-      */
--    if (pj_timer_entry_running(entry) || entry->id == TIMER_TYPE_NONE) {
-+    if (entry->id == TIMER_TYPE_NONE) {
- 	pjsip_dlg_dec_lock(sub->dlg);
- 	return;
-     }
-@@ -786,8 +787,7 @@ static pj_status_t evsub_create( pjsip_dialog *dlg,
-     		  pjsip_hdr_clone(sub->pool, pkg->pkg_accept);
-     pj_list_init(&sub->sub_hdr_list);
- 
--    sub->timer.user_data = sub;
--    sub->timer.cb = &on_timer;
-+    pj_timer_entry_init(&sub->timer, 0, sub, &on_timer);
- 
-     /* Set name. */
-     pj_ansi_snprintf(sub->obj_name, PJ_ARRAY_SIZE(sub->obj_name),
-diff --git a/pjsip/src/pjsip/sip_endpoint.c b/pjsip/src/pjsip/sip_endpoint.c
-index d810781d5..5c98a5bf6 100644
---- a/pjsip/src/pjsip/sip_endpoint.c
-+++ b/pjsip/src/pjsip/sip_endpoint.c
-@@ -788,6 +788,7 @@ PJ_DEF(pj_status_t) pjsip_endpt_schedule_timer_dbg(pjsip_endpoint *endpt,
- {
-     PJ_LOG(6, (THIS_FILE, "pjsip_endpt_schedule_timer(entry=%p, delay=%u.%u)",
- 			 entry, delay->sec, delay->msec));
-+    pj_timer_entry_reset(entry, entry->id);
-     return pj_timer_heap_schedule_dbg(endpt->timer_heap, entry, delay,
-                                       src_file, src_line);
- }
-@@ -798,6 +799,7 @@ PJ_DEF(pj_status_t) pjsip_endpt_schedule_timer( pjsip_endpoint *endpt,
- {
-     PJ_LOG(6, (THIS_FILE, "pjsip_endpt_schedule_timer(entry=%p, delay=%u.%u)",
- 			 entry, delay->sec, delay->msec));
-+    pj_timer_entry_reset(entry, entry->id);
-     return pj_timer_heap_schedule( endpt->timer_heap, entry, delay );
- }
- #endif
-@@ -809,7 +811,7 @@ PJ_DEF(void) pjsip_endpt_cancel_timer( pjsip_endpoint *endpt,
- 				       pj_timer_entry *entry )
- {
-     PJ_LOG(6, (THIS_FILE, "pjsip_endpt_cancel_timer(entry=%p)", entry));
--    pj_timer_heap_cancel( endpt->timer_heap, entry );
-+    pj_timer_heap_cancel_if_active( endpt->timer_heap, entry, 0 );
- }
- 
- /*
-diff --git a/pjsip/src/pjsip/sip_transaction.c b/pjsip/src/pjsip/sip_transaction.c
-index d52b12a72..5236e63f7 100644
---- a/pjsip/src/pjsip/sip_transaction.c
-+++ b/pjsip/src/pjsip/sip_transaction.c
-@@ -978,6 +978,7 @@ static pj_status_t tsx_schedule_timer(pjsip_transaction *tsx,
-     pj_status_t status;
- 
-     pj_assert(active_id != 0);
-+    pj_timer_entry_reset(entry, active_id);
-     status = pj_timer_heap_schedule_w_grp_lock(timer_heap, entry,
-                                                delay, active_id,
-                                                tsx->grp_lock);
-@@ -1019,12 +1020,8 @@ static pj_status_t tsx_create( pjsip_module *tsx_user,
-     pj_memcpy(pool->obj_name, tsx->obj_name, sizeof(pool->obj_name));
- 
-     tsx->handle_200resp = 1;
--    tsx->retransmit_timer.id = TIMER_INACTIVE;
--    tsx->retransmit_timer.user_data = tsx;
--    tsx->retransmit_timer.cb = &tsx_timer_callback;
--    tsx->timeout_timer.id = TIMER_INACTIVE;
--    tsx->timeout_timer.user_data = tsx;
--    tsx->timeout_timer.cb = &tsx_timer_callback;
-+    pj_timer_entry_init(&tsx->retransmit_timer, TIMER_INACTIVE, tsx, &tsx_timer_callback);
-+    pj_timer_entry_init(&tsx->timeout_timer, TIMER_INACTIVE, tsx, &tsx_timer_callback);
-     
-     if (grp_lock) {
- 	tsx->grp_lock = grp_lock;
-diff --git a/pjsip/src/pjsip/sip_transport.c b/pjsip/src/pjsip/sip_transport.c
-index 3d27d5d71..834babeae 100644
---- a/pjsip/src/pjsip/sip_transport.c
-+++ b/pjsip/src/pjsip/sip_transport.c
-@@ -1092,8 +1092,7 @@ PJ_DEF(pj_status_t) pjsip_transport_register( pjsip_tpmgr *mgr,
-     /* Init. */
-     tp->tpmgr = mgr;
-     pj_bzero(&tp->idle_timer, sizeof(tp->idle_timer));
--    tp->idle_timer.user_data = tp;
--    tp->idle_timer.cb = &transport_idle_callback;
-+    pj_timer_entry_init(&tp->idle_timer, 0, tp, &transport_idle_callback);
- 
-     /* 
-      * Register to hash table (see Trac ticket #42).
--- 
-2.14.3
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0090-sip_transaction-In-tsx_timer_callback-check-if-tsx-i.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0090-sip_transaction-In-tsx_timer_callback-check-if-tsx-i.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0090-sip_transaction-In-tsx_timer_callback-check-if-tsx-i.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0090-sip_transaction-In-tsx_timer_callback-check-if-tsx-i.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,31 +0,0 @@
-From beaa7874ff8e3b1d2951218c94e7e6bbba9c0531 Mon Sep 17 00:00:00 2001
-From: George Joseph <gjoseph@digium.com>
-Date: Sun, 25 Mar 2018 12:30:05 -0600
-Subject: [PATCH] sip_transaction: In tsx_timer_callback, check if tsx is
- already gone
-
-There have been cases that when the transaction timer callback is called
-the tsx is already destroyed.  This causes a crash.  We now check the
-tsx state and return if the tsx is already destroyed.
----
- pjsip/src/pjsip/sip_transaction.c | 4 ++++
- 1 file changed, 4 insertions(+)
-
-diff --git a/pjsip/src/pjsip/sip_transaction.c b/pjsip/src/pjsip/sip_transaction.c
-index d52b12a72..6d4cdc65f 100644
---- a/pjsip/src/pjsip/sip_transaction.c
-+++ b/pjsip/src/pjsip/sip_transaction.c
-@@ -1119,6 +1119,10 @@ static void tsx_timer_callback( pj_timer_heap_t *theap, pj_timer_entry *entry)
- 
-     PJ_UNUSED_ARG(theap);
- 
-+    if (tsx->state >= PJSIP_TSX_STATE_DESTROYED) {
-+        return;
-+    }
-+
-     if (entry->id == TRANSPORT_ERR_TIMER) {
- 	/* Posted transport error event */
- 	entry->id = 0;
--- 
-2.14.3
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0100-sip_inv-Add-option-to-accept-updated-SDP-on-same-To-.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0100-sip_inv-Add-option-to-accept-updated-SDP-on-same-To-.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0100-sip_inv-Add-option-to-accept-updated-SDP-on-same-To-.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0100-sip_inv-Add-option-to-accept-updated-SDP-on-same-To-.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,215 +0,0 @@
-From 13e20772cd3c8735a6b78e30391a33f3eba4c023 Mon Sep 17 00:00:00 2001
-From: George Joseph <gjoseph@digium.com>
-Date: Fri, 22 Jun 2018 09:33:34 -0600
-Subject: [PATCH] sip_inv:  Add option to accept updated SDP on same To tag
-
-Currently, setting pjsip_cfg()->endpt.follow_early_media_fork allows
-sip_inv to process media updates when the To tag is different.  There
-are some cases where media updates need to be processed when the tags
-are the same.  Since removing the requirement for different tags would
-change default behavior, a new option "accept_multiple_sdp_answers"
-has been added along with a new pjsip_inv_session flag
-"updated_sdp_answer" to indicate under which condition we're
-updating.
-
-The logic was also updated to more closely follow RFC6337 in that
-if 100rel is efffect, do not accept updated SDPs.
-
-See
-https://tools.ietf.org/html/rfc6337#section-3.1.1
-for more information.
----
- pjsip/include/pjsip-ua/sip_inv.h |  2 ++
- pjsip/include/pjsip/sip_config.h | 25 ++++++++++++++++
- pjsip/src/pjsip-ua/sip_inv.c     | 62 +++++++++++++++++++++++++---------------
- pjsip/src/pjsip/sip_config.c     |  3 +-
- 4 files changed, 68 insertions(+), 24 deletions(-)
-
-diff --git a/pjsip/include/pjsip-ua/sip_inv.h b/pjsip/include/pjsip-ua/sip_inv.h
-index 1bb7b8adc..77ef070c3 100644
---- a/pjsip/include/pjsip-ua/sip_inv.h
-+++ b/pjsip/include/pjsip-ua/sip_inv.h
-@@ -442,6 +442,8 @@ struct pjsip_inv_session
-     pj_bool_t		 following_fork;	    /**< Internal, following
- 							 forked media?	    */
-     pj_atomic_t		*ref_cnt;		    /**< Reference counter. */
-+    pj_bool_t            updated_sdp_answer;        /**< SDP answer just been
-+							 updated?	    */
- };
- 
- 
-diff --git a/pjsip/include/pjsip/sip_config.h b/pjsip/include/pjsip/sip_config.h
-index b3a9468e2..b7cf6feed 100644
---- a/pjsip/include/pjsip/sip_config.h
-+++ b/pjsip/include/pjsip/sip_config.h
-@@ -157,6 +157,17 @@ typedef struct pjsip_cfg_t
- 	 */
- 	pj_bool_t disable_secure_dlg_check;
- 
-+        /**
-+         * Accept multiple SDP answers on non-reliable 18X responses and the 2XX
-+         * response when they are all received from the same source (same To tag).
-+         *
-+         * See also:
-+         * https://tools.ietf.org/html/rfc6337#section-3.1.1
-+         *
-+         * Default is PJSIP_ACCEPT_MULTIPLE_SDP_ANSWERS.
-+         */
-+        pj_bool_t accept_multiple_sdp_answers;
-+
-     } endpt;
- 
-     /** Transaction layer settings. */
-@@ -402,6 +413,20 @@ PJ_INLINE(pjsip_cfg_t*) pjsip_cfg(void)
- #endif
- 
- 
-+/**
-+ * Accept multiple SDP answers on non-reliable 18X responses and the 2XX
-+ * response when they are all received from the same source (same To tag).
-+ *
-+ * This option can also be controlled at run-time by the
-+ * \a accept_multiple_sdp_answers setting in pjsip_cfg_t.
-+ *
-+ * Default is PJ_FALSE.
-+ */
-+#ifndef PJSIP_ACCEPT_MULTIPLE_SDP_ANSWERS
-+#   define PJSIP_ACCEPT_MULTIPLE_SDP_ANSWERS        PJ_TRUE
-+#endif
-+
-+
- /**
-  * Specify whether "alias" param should be added to the Via header
-  * in any outgoing request with connection oriented transport.
-diff --git a/pjsip/src/pjsip-ua/sip_inv.c b/pjsip/src/pjsip-ua/sip_inv.c
-index c9686a088..c22726bad 100644
---- a/pjsip/src/pjsip-ua/sip_inv.c
-+++ b/pjsip/src/pjsip-ua/sip_inv.c
-@@ -162,6 +162,7 @@ struct tsx_inv_data
-     pj_bool_t		 retrying;  /* Resend (e.g. due to 401/407)         */
-     pj_str_t		 done_tag;  /* To tag in RX response with answer    */
-     pj_bool_t		 done_early;/* Negotiation was done for early med?  */
-+    pj_bool_t		 done_early_rel;/* Early med was realiable?	    */
-     pj_bool_t		 has_sdp;   /* Message with SDP?		    */
- };
- 
-@@ -2000,18 +2001,20 @@ static pj_status_t inv_check_sdp_in_incoming_msg( pjsip_inv_session *inv,
- 
-     /* Initialize info that we are following forked media */
-     inv->following_fork = PJ_FALSE;
-+    inv->updated_sdp_answer = PJ_FALSE;
- 
-     /* MUST NOT do multiple SDP offer/answer in a single transaction,
--     * EXCEPT if:
--     *	- this is an initial UAC INVITE transaction (i.e. not re-INVITE), and
--     *	- the previous negotiation was done on an early media (18x) and
--     *    this response is a final/2xx response, and
--     *  - the 2xx response has different To tag than the 18x response
--     *    (i.e. the request has forked).
-+     * EXCEPT previous nego was in 18x (early media) and any of the following
-+     * condition is met:
-+     *  - Non-forking scenario:
-+     *	  - 'accept_multiple_sdp_answers' is set, and
-+     *    - previous early response was not reliable (rfc6337 section 3.1.1).
-+     *  - Forking scenario:
-+     *    - This response has different To tag than the previous response, and
-+     *    - This response is 18x/2xx (early or final). If this is 18x,
-+     *      only do multiple SDP nego if 'follow_early_media_fork' is set.
-      *
--     * The exception above is to add a rudimentary support for early media
--     * forking (sample case: custom ringback). See this ticket for more
--     * info: http://trac.pjsip.org/repos/ticket/657
-+     * See also ticket #657, #1644, #1764 for more info.
-      */
-     if (tsx_inv_data->sdp_done) {
- 	pj_str_t res_tag;
-@@ -2020,21 +2023,29 @@ static pj_status_t inv_check_sdp_in_incoming_msg( pjsip_inv_session *inv,
- 	res_tag = rdata->msg_info.to->tag;
- 	st_code = rdata->msg_info.msg->line.status.code;
- 
--	/* Allow final/early response after SDP has been negotiated in early
--	 * media, IF this response is a final/early response with different
--	 * tag.
--         * See ticket #1644 and #1764 for forked early media case.
--	 */
--	if (tsx->role == PJSIP_ROLE_UAC &&
--	    (st_code/100 == 2 ||
--	     (st_code/10 == 18 /* st_code == 18x */
--              && pjsip_cfg()->endpt.follow_early_media_fork)) &&
--	    tsx_inv_data->done_early &&
--	    pj_stricmp(&tsx_inv_data->done_tag, &res_tag))
-+	if (tsx->role == PJSIP_ROLE_UAC && tsx_inv_data->done_early &&
-+	       (
-+	           /* Non-forking scenario */
-+	           (
-+	               !tsx_inv_data->done_early_rel &&
-+	               (st_code/100 == 2 || st_code/10 == 18) &&
-+                       pjsip_cfg()->endpt.accept_multiple_sdp_answers &&
-+	               !pj_stricmp(&tsx_inv_data->done_tag, &res_tag)
-+	           )
-+	           ||
-+	           /* Forking scenario */
-+	           (
-+	               (st_code/100 == 2 ||
-+	                   (st_code/10 == 18 &&
-+	                       pjsip_cfg()->endpt.follow_early_media_fork)) &&
-+	               pj_stricmp(&tsx_inv_data->done_tag, &res_tag)
-+	           )
-+	       )
-+	   )
- 	{
- 	    const pjmedia_sdp_session *reoffer_sdp = NULL;
- 
--	    PJ_LOG(4,(inv->obj_name, "Received forked %s response "
-+	    PJ_LOG(4,(inv->obj_name, "Received %s response "
- 		      "after SDP negotiation has been done in early "
- 		      "media. Renegotiating SDP..",
- 		      (st_code/10==18? "early" : "final" )));
-@@ -2054,7 +2065,9 @@ static pj_status_t inv_check_sdp_in_incoming_msg( pjsip_inv_session *inv,
- 		return status;
- 	    }
- 
--	    inv->following_fork = PJ_TRUE;
-+	    inv->following_fork = !!pj_stricmp(&tsx_inv_data->done_tag,
-+					       &res_tag);
-+	    inv->updated_sdp_answer = PJ_TRUE;
- 
- 	} else {
- 
-@@ -2135,6 +2148,7 @@ static pj_status_t inv_check_sdp_in_incoming_msg( pjsip_inv_session *inv,
- 		PJMEDIA_SDP_NEG_STATE_LOCAL_OFFER) 
-     {
- 	int status_code;
-+	pjsip_msg *msg = rdata->msg_info.msg;
- 
- 	/* This is an answer. 
- 	 * Process and negotiate remote answer.
-@@ -2161,8 +2175,10 @@ static pj_status_t inv_check_sdp_in_incoming_msg( pjsip_inv_session *inv,
- 	 */
- 
- 	tsx_inv_data->sdp_done = 1;
--	status_code = rdata->msg_info.msg->line.status.code;
-+	status_code = msg->line.status.code;
- 	tsx_inv_data->done_early = (status_code/100==1);
-+	tsx_inv_data->done_early_rel = tsx_inv_data->done_early &&
-+				       pjsip_100rel_is_reliable(rdata);
- 	pj_strdup(tsx->pool, &tsx_inv_data->done_tag, 
- 		  &rdata->msg_info.to->tag);
- 
-diff --git a/pjsip/src/pjsip/sip_config.c b/pjsip/src/pjsip/sip_config.c
-index 3576f351e..316824a00 100644
---- a/pjsip/src/pjsip/sip_config.c
-+++ b/pjsip/src/pjsip/sip_config.c
-@@ -34,7 +34,8 @@ pjsip_cfg_t pjsip_sip_cfg_var =
-        PJSIP_FOLLOW_EARLY_MEDIA_FORK,
-        PJSIP_REQ_HAS_VIA_ALIAS,
-        PJSIP_RESOLVE_HOSTNAME_TO_GET_INTERFACE,
--       0
-+       0,
-+       PJSIP_ACCEPT_MULTIPLE_SDP_ANSWERS
-     },
- 
-     /* Transaction settings */
--- 
-2.14.4
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/0110_fix_tdata_rexmit_deadlock.patch 1:16.1.1~dfsg-1/third-party/pjproject/patches/0110_fix_tdata_rexmit_deadlock.patch
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/0110_fix_tdata_rexmit_deadlock.patch	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/0110_fix_tdata_rexmit_deadlock.patch	1970-01-01 00:00:00.000000000 +0000
@@ -1,203 +0,0 @@
-From 3c9ad14b32ddb881559c05ea742aba3153a8cb8f Mon Sep 17 00:00:00 2001
-From: Riza Sulistyo <riza@teluu.com>
-Date: Wed, 1 Aug 2018 10:49:51 -0500
-Subject: [PATCH] r5851 svn backport sip_inv.c: Fix race condition in 183
- re-transmission deadlock
-
-Fixed #2137: Race condition in 183 re-transmission can result in a deadlock.
-
-The tdata containing the response can be shared by both the dialog
-object and the tsx object.  In order to prevent the race condition
-between the tsx retransmission and the dialog sending a response,
-clone the tdata before modifying it for the dialog send response.
-
-ASTERISK-27966
----
- pjsip/include/pjsip/sip_transport.h | 16 +++++++++++++
- pjsip/src/pjsip-ua/sip_100rel.c     | 43 +--------------------------------
- pjsip/src/pjsip-ua/sip_inv.c        | 14 ++++++++++-
- pjsip/src/pjsip/sip_transport.c     | 47 +++++++++++++++++++++++++++++++++++++
- 4 files changed, 77 insertions(+), 43 deletions(-)
-
-diff --git a/pjsip/include/pjsip/sip_transport.h b/pjsip/include/pjsip/sip_transport.h
-index 10a50ef..36581dc 100644
---- a/pjsip/include/pjsip/sip_transport.h
-+++ b/pjsip/include/pjsip/sip_transport.h
-@@ -721,6 +721,22 @@ PJ_DECL(char*) pjsip_tx_data_get_info( pjsip_tx_data *tdata );
- PJ_DECL(pj_status_t) pjsip_tx_data_set_transport(pjsip_tx_data *tdata,
- 						 const pjsip_tpselector *sel);
- 
-+/**
-+ * Clone pjsip_tx_data. This will duplicate the message contents of
-+ * pjsip_tx_data (pjsip_tx_data.msg) and add reference count to the tdata.
-+ * Once application has finished using the cloned pjsip_tx_data,
-+ * it must release it by calling  #pjsip_tx_data_dec_ref().
-+ * Currently, this will only clone response message.
-+ *
-+ * @param src	    The source to be cloned.
-+ * @param flags	    Optional flags. Must be zero for now.
-+ * @param p_rdata   Pointer to receive the cloned tdata.
-+ *
-+ * @return	    PJ_SUCCESS on success or the appropriate error.
-+ */
-+PJ_DECL(pj_status_t) pjsip_tx_data_clone(const pjsip_tx_data *src,
-+                                         unsigned flags,
-+                                         pjsip_tx_data **p_rdata);
- 
- /*****************************************************************************
-  *
-diff --git a/pjsip/src/pjsip-ua/sip_100rel.c b/pjsip/src/pjsip-ua/sip_100rel.c
-index eb9e587..7bf0ad1 100644
---- a/pjsip/src/pjsip-ua/sip_100rel.c
-+++ b/pjsip/src/pjsip-ua/sip_100rel.c
-@@ -634,47 +634,6 @@ static void on_retransmit(pj_timer_heap_t *timer_heap,
- }
- 
- 
--/* Clone response. */
--static pjsip_tx_data *clone_tdata(dlg_data *dd,
--				  const pjsip_tx_data *src)
--{
--    pjsip_tx_data *dst;
--    const pjsip_hdr *hsrc;
--    pjsip_msg *msg;
--    pj_status_t status;
--
--    status = pjsip_endpt_create_tdata(dd->inv->dlg->endpt, &dst);
--    if (status != PJ_SUCCESS)
--	return NULL;
--
--    msg = pjsip_msg_create(dst->pool, PJSIP_RESPONSE_MSG);
--    dst->msg = msg;
--    pjsip_tx_data_add_ref(dst);
--
--    /* Duplicate status line */
--    msg->line.status.code = src->msg->line.status.code;
--    pj_strdup(dst->pool, &msg->line.status.reason, 
--	      &src->msg->line.status.reason);
--
--    /* Duplicate all headers */
--    hsrc = src->msg->hdr.next;
--    while (hsrc != &src->msg->hdr) {
--	pjsip_hdr *h = (pjsip_hdr*) pjsip_hdr_clone(dst->pool, hsrc);
--	pjsip_msg_add_hdr(msg, h);
--	hsrc = hsrc->next;
--    }
--
--    /* Duplicate message body */
--    if (src->msg->body)
--	msg->body = pjsip_msg_body_clone(dst->pool, src->msg->body);
--
--    PJ_LOG(5,(dd->inv->dlg->obj_name,
--	     "Reliable response %s created",
--	     pjsip_tx_data_get_info(dst)));
--
--    return dst;
--}
--
- 
- /* Check if any pending response in transmission list has SDP */
- static pj_bool_t has_sdp(dlg_data *dd)
-@@ -725,7 +684,7 @@ PJ_DEF(pj_status_t) pjsip_100rel_tx_response(pjsip_inv_session *inv,
-      * if it wants to send another response.
-      */
-     old_tdata = tdata;
--    tdata = clone_tdata(dd, old_tdata);
-+    pjsip_tx_data_clone(old_tdata, 0, &tdata);
-     pjsip_tx_data_dec_ref(old_tdata);
-     
- 
-diff --git a/pjsip/src/pjsip-ua/sip_inv.c b/pjsip/src/pjsip-ua/sip_inv.c
-index 16a0e17..da28903 100644
---- a/pjsip/src/pjsip-ua/sip_inv.c
-+++ b/pjsip/src/pjsip-ua/sip_inv.c
-@@ -2382,6 +2382,7 @@ PJ_DEF(pj_status_t) pjsip_inv_answer(	pjsip_inv_session *inv,
- 					pjsip_tx_data **p_tdata )
- {
-     pjsip_tx_data *last_res;
-+    pjsip_tx_data *old_res;
-     pj_status_t status;
- 
-     /* Verify arguments. */
-@@ -2397,8 +2398,19 @@ PJ_DEF(pj_status_t) pjsip_inv_answer(	pjsip_inv_session *inv,
- 
-     pjsip_dlg_inc_lock(inv->dlg);
- 
-+    /* Clone last response.
-+     * The tdata (last_answer) is a shared object used by the transaction.
-+     * Modifying a shared object might lead to a deadlock.
-+     * Refer to ticket #2137 for more detail.
-+     */
-+    status = pjsip_tx_data_clone(inv->last_answer, 0, &last_res);
-+    if (status != PJ_SUCCESS)
-+	goto on_return;
-+    old_res = inv->last_answer;
-+    inv->last_answer = last_res;
-+    pjsip_tx_data_dec_ref(old_res);
-+
-     /* Modify last response. */
--    last_res = inv->last_answer;
-     status = pjsip_dlg_modify_response(inv->dlg, last_res, st_code, st_text);
-     if (status != PJ_SUCCESS)
- 	goto on_return;
-diff --git a/pjsip/src/pjsip/sip_transport.c b/pjsip/src/pjsip/sip_transport.c
-index f350fe7..3a8baca 100644
---- a/pjsip/src/pjsip/sip_transport.c
-+++ b/pjsip/src/pjsip/sip_transport.c
-@@ -643,6 +643,53 @@ PJ_DEF(pj_status_t) pjsip_tx_data_set_transport(pjsip_tx_data *tdata,
-     return PJ_SUCCESS;
- }
- 
-+/* Clone pjsip_tx_data. */
-+PJ_DEF(pj_status_t) pjsip_tx_data_clone(const pjsip_tx_data *src,
-+                                        unsigned flags,
-+				  	pjsip_tx_data ** p_tdata)
-+{
-+    pjsip_tx_data *dst;
-+    const pjsip_hdr *hsrc;
-+    pjsip_msg *msg;
-+    pj_status_t status;
-+
-+    PJ_UNUSED_ARG(flags);
-+
-+    status = pjsip_tx_data_create(src->mgr, p_tdata);
-+    if (status != PJ_SUCCESS)
-+	return status;
-+
-+    dst = *p_tdata;
-+
-+    msg = pjsip_msg_create(dst->pool, PJSIP_RESPONSE_MSG);
-+    dst->msg = msg;
-+    pjsip_tx_data_add_ref(dst);
-+
-+    /* Duplicate status line */
-+    msg->line.status.code = src->msg->line.status.code;
-+    pj_strdup(dst->pool, &msg->line.status.reason,
-+	      &src->msg->line.status.reason);
-+
-+    /* Duplicate all headers */
-+    hsrc = src->msg->hdr.next;
-+    while (hsrc != &src->msg->hdr) {
-+	pjsip_hdr *h = (pjsip_hdr*) pjsip_hdr_clone(dst->pool, hsrc);
-+	pjsip_msg_add_hdr(msg, h);
-+	hsrc = hsrc->next;
-+    }
-+
-+    /* Duplicate message body */
-+    if (src->msg->body)
-+	msg->body = pjsip_msg_body_clone(dst->pool, src->msg->body);
-+
-+    dst->is_pending = src->is_pending;
-+
-+    PJ_LOG(5,(THIS_FILE,
-+	     "Tx data %s cloned",
-+	     pjsip_tx_data_get_info(dst)));
-+
-+    return PJ_SUCCESS;
-+}
- 
- PJ_DEF(char*) pjsip_rx_data_get_info(pjsip_rx_data *rdata)
- {
--- 
-2.7.4
-
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/asterisk_malloc_debug.c 1:16.1.1~dfsg-1/third-party/pjproject/patches/asterisk_malloc_debug.c
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/asterisk_malloc_debug.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/asterisk_malloc_debug.c	2018-12-26 18:45:13.000000000 +0000
@@ -22,7 +22,7 @@
 #include <string.h>
 #include <stdarg.h>
 
-int __ast_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
+int __ast_repl_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
 {
 	va_list ap;
 	int rc = 0;
@@ -34,7 +34,7 @@ int __ast_asprintf(const char *file, int
 	return rc;
 }
 
-void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func)
 {
 	return calloc(nmemb, size);
 }
@@ -44,27 +44,27 @@ void __ast_free(void *ptr, const char *f
 	free(ptr);
 }
 
-void *__ast_malloc(size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_malloc(size_t size, const char *file, int lineno, const char *func)
 {
 	return malloc(size);
 }
 
-void *__ast_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
+void *__ast_repl_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func)
 {
 	return realloc(ptr, size);
 }
 
-char *__ast_strdup(const char *s, const char *file, int lineno, const char *func)
+char *__ast_repl_strdup(const char *s, const char *file, int lineno, const char *func)
 {
 	return strdup(s);
 }
 
-char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
+char *__ast_repl_strndup(const char *s, size_t n, const char *file, int lineno, const char *func)
 {
 	return strndup(s, n);
 }
 
-int __ast_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
+int __ast_repl_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
 {
 	return vasprintf(strp, format, ap);
 }
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/asterisk_malloc_debug.h 1:16.1.1~dfsg-1/third-party/pjproject/patches/asterisk_malloc_debug.h
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/asterisk_malloc_debug.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/asterisk_malloc_debug.h	2018-12-26 18:45:13.000000000 +0000
@@ -25,15 +25,25 @@
 extern "C" {
 #endif
 
-int __ast_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
+#ifndef attribute_malloc
+#ifdef HAVE_ATTRIBUTE_malloc
+/* HAVE_ATTRIBUTE_malloc is never defined from pjproject.  This is here as a placeholder
+ * hopefully we can just use __attribute__((malloc)) unconditionally. */
+#define attribute_malloc __attribute__((malloc))
+#else
+#define attribute_malloc
+#endif
+#endif
+
+int __ast_repl_asprintf(const char *file, int lineno, const char *func, char **strp, const char *format, ...)
 	__attribute__((format(printf, 5, 6)));
-void *__ast_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func);
+void *__ast_repl_calloc(size_t nmemb, size_t size, const char *file, int lineno, const char *func) attribute_malloc;
 void __ast_free(void *ptr, const char *file, int lineno, const char *func);
-void *__ast_malloc(size_t size, const char *file, int lineno, const char *func);
-void *__ast_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func);
-char *__ast_strdup(const char *s, const char *file, int lineno, const char *func);
-char *__ast_strndup(const char *s, size_t n, const char *file, int lineno, const char *func);
-int __ast_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
+void *__ast_repl_malloc(size_t size, const char *file, int lineno, const char *func) attribute_malloc;
+void *__ast_repl_realloc(void *ptr, size_t size, const char *file, int lineno, const char *func);
+char *__ast_repl_strdup(const char *s, const char *file, int lineno, const char *func) attribute_malloc;
+char *__ast_repl_strndup(const char *s, size_t n, const char *file, int lineno, const char *func) attribute_malloc;
+int __ast_repl_vasprintf(char **strp, const char *format, va_list ap, const char *file, int lineno, const char *func)
 	__attribute__((format(printf, 2, 0)));
 
 /* Undefine any macros */
@@ -47,29 +57,29 @@ int __ast_vasprintf(char **strp, const c
 #undef vasprintf
 
  /* Provide our own definitions */
-#define asprintf(a, b, c...) \
-	__ast_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, a, b, c)
+#define asprintf(strp, format, args...) \
+	__ast_repl_asprintf(__FILE__, __LINE__, __PRETTY_FUNCTION__, strp, format, args)
 
-#define calloc(a,b) \
-	__ast_calloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define calloc(nmemb, size) \
+	__ast_repl_calloc(nmemb, size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define free(a) \
-	__ast_free(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define free(ptr) \
+	__ast_free(ptr, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define malloc(a) \
-	__ast_malloc(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define malloc(size) \
+	__ast_repl_malloc(size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define realloc(a,b) \
-	__ast_realloc(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define realloc(ptr, size) \
+	__ast_repl_realloc(ptr, size, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define strdup(a) \
-	__ast_strdup(a,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define strdup(s) \
+	__ast_repl_strdup(s, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define strndup(a,b) \
-	__ast_strndup(a,b,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define strndup(s, n) \
+	__ast_repl_strndup(s, n, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
-#define vasprintf(a,b,c) \
-	__ast_vasprintf(a,b,c,__FILE__, __LINE__, __PRETTY_FUNCTION__)
+#define vasprintf(strp, format, ap) \
+	__ast_repl_vasprintf(strp, format, ap, __FILE__, __LINE__, __PRETTY_FUNCTION__)
 
 #ifdef __cplusplus
 }
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/patches/config_site.h 1:16.1.1~dfsg-1/third-party/pjproject/patches/config_site.h
--- 1:13.23.1~dfsg-2/third-party/pjproject/patches/config_site.h	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/patches/config_site.h	2018-12-26 18:45:13.000000000 +0000
@@ -8,7 +8,7 @@
  * Since both pjproject and asterisk source files will include config_site.h,
  * we need to make sure that only pjproject source files include asterisk_malloc_debug.h.
  */
-#if defined(MALLOC_DEBUG) && !defined(_ASTERISK_ASTMM_H)
+#if !defined(_ASTERISK_ASTMM_H)
 #include "asterisk_malloc_debug.h"
 #endif
 
@@ -21,7 +21,10 @@
 #define PJMEDIA_HAS_SRTP 0
 
 #define PJ_HAS_IPV6 1
+#if !defined(AST_DEVMODE) && !defined(PJPROJECT_BUNDLED_ASSERTIONS)
 #define NDEBUG 1
+#endif
+
 #define PJ_MAX_HOSTNAME (256)
 #define PJSIP_MAX_URL_SIZE (512)
 #ifdef PJ_HAS_LINUX_EPOLL
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/pjproject-2.7.2.tar.bz2.md5 1:16.1.1~dfsg-1/third-party/pjproject/pjproject-2.7.2.tar.bz2.md5
--- 1:13.23.1~dfsg-2/third-party/pjproject/pjproject-2.7.2.tar.bz2.md5	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/pjproject-2.7.2.tar.bz2.md5	1970-01-01 00:00:00.000000000 +0000
@@ -1,2 +0,0 @@
-8119f0d91a00b6f553099e6ee5358ade *pjproject-2.7.2.zip
-fa3f0bc098c4bff48ddd92db1c016a7a  pjproject-2.7.2.tar.bz2
diff -pruN 1:13.23.1~dfsg-2/third-party/pjproject/pjproject-2.8.tar.bz2.md5 1:16.1.1~dfsg-1/third-party/pjproject/pjproject-2.8.tar.bz2.md5
--- 1:13.23.1~dfsg-2/third-party/pjproject/pjproject-2.8.tar.bz2.md5	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/pjproject/pjproject-2.8.tar.bz2.md5	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,2 @@
+1d3cb8fd2752c724dc8798900f102e75 *pjproject-2.8.zip
+6487d54213f270d307eaa60efc9f56f3  pjproject-2.8.tar.bz2
\ No newline at end of file
diff -pruN 1:13.23.1~dfsg-2/third-party/versions.mak 1:16.1.1~dfsg-1/third-party/versions.mak
--- 1:13.23.1~dfsg-2/third-party/versions.mak	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/third-party/versions.mak	2018-12-26 18:45:13.000000000 +0000
@@ -1,2 +1,2 @@
-JANSSON_VERSION = 2.11
-PJPROJECT_VERSION = 2.7.2
+JANSSON_VERSION = 2.12
+PJPROJECT_VERSION = 2.8
diff -pruN 1:13.23.1~dfsg-2/UPGRADE-13.txt 1:16.1.1~dfsg-1/UPGRADE-13.txt
--- 1:13.23.1~dfsg-2/UPGRADE-13.txt	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/UPGRADE-13.txt	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,399 @@
+===========================================================
+===
+=== Information for upgrading between Asterisk versions
+===
+=== These files document all the changes that MUST be taken
+=== into account when upgrading between the Asterisk
+=== versions listed below. These changes may require that
+=== you modify your configuration files, dialplan or (in
+=== some cases) source code if you have your own Asterisk
+=== modules or patches. These files also include advance
+=== notice of any functionality that has been marked as
+=== 'deprecated' and may be removed in a future release,
+=== along with the suggested replacement functionality.
+===
+=== UPGRADE-1.2.txt -- Upgrade info for 1.0 to 1.2
+=== UPGRADE-1.4.txt -- Upgrade info for 1.2 to 1.4
+=== UPGRADE-1.6.txt -- Upgrade info for 1.4 to 1.6
+=== UPGRADE-1.8.txt -- Upgrade info for 1.6 to 1.8
+=== UPGRADE-10.txt  -- Upgrade info for 1.8 to 10
+=== UPGRADE-11.txt  -- Upgrade info for 10 to 11
+=== UPGRADE-12.txt  -- Upgrade info for 11 to 12
+===========================================================
+
+General Asterisk Changes:
+ - The asterisk command line -I option and the asterisk.conf internal_timing
+   option are removed and always enabled if any timing module is loaded.
+
+ - The per console verbose level feature as previously implemented caused a
+   large performance penalty.  The fix required some minor incompatibilities
+   if the new rasterisk is used to connect to an earlier version.  If the new
+   rasterisk connects to an older Asterisk version then the root console verbose
+   level is always affected by the "core set verbose" command of the remote
+   console even though it may appear to only affect the current console.  If
+   an older version of rasterisk connects to the new version then the
+   "core set verbose" command will have no effect.
+
+ - The asterisk compatibility options in asterisk.conf have been removed.
+   These options enabled certain backwards compatibility features for
+   pbx_realtime, res_agi, and app_set that made their behaviour similar to
+   Asterisk 1.4. Users who used these backwards compatibility settings should
+   update their dialplans to use ',' instead of '|' as a delimiter, and should
+   use the Set dialplan application instead of the MSet dialplan application.
+
+Build System:
+ - Sample config files have been moved from configs/ to a subfolder of that
+   directory, 'samples'.
+
+ - The menuselect utility has been pulled into the Asterisk repository. As a
+   result, the libxml2 development library is now a required dependency for
+   Asterisk.
+
+ - Added a new Compiler Flag, REF_DEBUG. When enabled, reference counted
+   objects will emit additional debug information to the refs log file located
+   in the standard Asterisk log file directory. This log file is useful in
+   tracking down object leaks and other reference counting issues. Prior to
+   this version, this option was only available by modifying the source code
+   directly. This change also includes a new script, refcounter.py, in the
+   contrib folder that will process the refs log file.
+
+Applications:
+
+ConfBridge:
+ - The sound_place_into_conference sound used in Confbridge is now deprecated
+   and is no longer functional since it has been broken since its inception
+   and the fix involved using a different method to achieve the same goal. The
+   new method to achieve this functionality is by using sound_begin to play
+   a sound to the conference when waitmarked users are moved into the conference.
+
+ - Added 'Admin' header to ConfbridgeJoin, ConfbridgeLeave, ConfbridgeMute,
+   ConfbridgeUnmute, and ConfbridgeTalking AMI events.
+
+ControlPlayback:
+ - The ControlPlayback and 'control stream file' AGI command will no longer
+   implicitly answer the channel. If you do not answer the channel prior to
+   using either this application or AGI command, you must send Progress
+   first.
+
+Queue:
+ - Queue rules provided in queuerules.conf can no longer be named "general".
+
+SetMusicOnHold:
+ - The SetMusicOnHold dialplan application was deprecated and has been removed.
+   Users of the application should use the CHANNEL function's musicclass
+   setting instead.
+
+WaitMusicOnHold:
+ - The WaitMusicOnHold dialplan application was deprecated and has been
+   removed. Users of the application should use MusicOnHold with a duration
+   parameter instead.
+
+CDR Backends:
+ - The cdr_sqlite module was deprecated and has been removed. Users of this
+   module should use the cdr_sqlite3_custom module instead.
+
+Channel Drivers:
+
+chan_dahdi:
+ - SS7 support now requires libss7 v2.0 or later.
+
+ - Added the inband_on_setup_ack compatibility option to chan_dahdi.conf to
+   deal with switches that don't send an inband progress indication in the
+   SETUP ACKNOWLEDGE message.
+   Default is now no.
+
+chan_gtalk
+ - This module was deprecated and has been removed. Users of chan_gtalk
+   should use chan_motif.
+
+chan_h323
+ - This module was deprecated and has been removed. Users of chan_h323
+   should use chan_ooh323.
+
+chan_jingle
+ - This module was deprecated and has been removed. Users of chan_jingle
+   should use chan_motif.
+
+chan_pjsip:
+ - Added a 'force_avp' option to chan_pjsip which will force the usage of
+   'RTP/AVP', 'RTP/AVPF', 'RTP/SAVP', or 'RTP/SAVPF' as the media transport type
+   in SDP offers depending on settings, even when DTLS is used for media
+   encryption.
+
+ - Added a 'media_use_received_transport' option to chan_pjsip which will
+   cause the SDP answer to use the media transport as received in the SDP
+   offer.
+
+chan_sip:
+ - Made set SIPREFERREDBYHDR as inheritable for better chan_pjsip
+   interoperability.
+
+ - The SIPPEER dialplan function no longer supports using a colon as a
+   delimiter for parameters. The parameters for the function should be
+   delimited using a comma.
+
+ - The SIPCHANINFO dialplan function was deprecated and has been removed. Users
+   of the function should use the CHANNEL function instead.
+
+ - Added a 'force_avp' option for chan_sip. When enabled this option will
+   cause the media transport in the offer or answer SDP to be 'RTP/AVP',
+   'RTP/AVPF', 'RTP/SAVP', or 'RTP/SAVPF' even if a DTLS stream has been
+   configured. This option can be set to improve interoperability with WebRTC
+   clients that don't use the RFC defined transport for DTLS.
+
+ - The 'dtlsverify' option in chan_sip now has additional values besides
+   'yes' and 'no'. If 'yes' is specified both the certificate and fingerprint
+   will be verified. If 'no' is specified then neither the certificate or
+   fingerprint is verified. If 'certificate' is specified then only the
+   certificate is verified. If 'fingerprint' is specified then only the
+   fingerprint is verified.
+
+ - A 'dtlsfingerprint' option has been added to chan_sip which allows the
+   hash to be specified for the DTLS fingerprint placed in SDP. Supported
+   values are 'sha-1' and 'sha-256' with 'sha-256' being the default.
+
+ - The 'progressinband=never' option is now more zealous in the persecution of
+   progress messages coming from Asterisk. Channels bridged with a SIP channel
+   that has 'progressinband=never' set will not be able to forward their
+   progress indications through to the SIP device. chan_sip will now turn such
+   progress indications into a 180 Ringing (if a 180 has not yet been
+   transmitted) if 'progressinband=never'.
+
+  - The codec preference order in an SDP during an offer is slightly different
+    than previous releases. Prior to Asterisk 13, the preference order of
+    codecs used to be:
+    (1) Our preferred codec
+    (2) Our configured codecs
+    (3) Any non-audio joint codecs
+
+    One of the ways the new media format architecture in Asterisk 13 improves
+    performance is by reference counting formats, such that they can be reused
+    in many places without additional allocation. To not require a large
+    amount of locking, an instance of a format is immutable by convention.
+    This works well except for formats with attributes. Since a media format
+    with an attribute is a different object than the same format without an
+    attribute, we have to carry over the formats with attributes from an
+    inbound offer so that the correct attributes are offered in an outgoing
+    INVITE request. This requires some subtle tweaks to the preference order
+    to ensure that the media format with attributes is offered to a remote
+    peer, as opposed to the same media format (but without attributes) that
+    may be stored in the peer object.
+
+    All of this means that our offer offer list will now be:
+    (1) Our preferred codec
+    (2) Any joint codecs offered by the inbound offer
+    (3) All other codecs that are not the preferred codec and not a joint
+        codec offered by the inbound offer
+
+chan_unistim:
+ - The unistim.conf 'dateformat' has changed meaning of options values to conform
+   values used inside Unistim protocol
+
+ - Added 'dtmf_duration' option with changing default operation to disable
+   receivied dtmf playback on unistim phone
+
+Core:
+
+Account Codes:
+ - accountcode behavior changed somewhat to add functional peeraccount
+   support.  The main change is that local channels now cross accountcode
+   and peeraccount across the special bridge between the ;1 and ;2 channels
+   just like channels between normal bridges.  See the CHANGES file for
+   more information.
+
+ARI:
+ - The ARI version has been changed to 1.5.0. This is to reflect backwards
+   compatible changes made since 12.0.0 was released.
+
+ - Added a new ARI resource 'mailboxes' which allows the creation and
+   modification of mailboxes managed by external MWI. Modules res_mwi_external
+   and res_stasis_mailbox must be enabled to use this resource.
+
+ - Added new events for externally initiated transfers. The event
+   BridgeBlindTransfer is now raised when a channel initiates a blind transfer
+   of a bridge in the ARI controlled application to the dialplan; the
+   BridgeAttendedTransfer event is raised when a channel initiates an
+   attended transfer of a bridge in the ARI controlled application to the
+   dialplan.
+
+ - Channel variables may now be specified as a body parameter to the
+   POST /channels operation. The 'variables' key in the JSON is interpreted
+   as a sequence of key/value pairs that will be added to the created channel
+   as channel variables. Other parameters in the JSON body are treated as
+   query parameters of the same name.
+
+ - A bug fix in bridge creation has caused a behavioural change in how
+   subscriptions are created for bridges. A bridge created through ARI, does
+   not, by itself, have a subscription created for any particular Stasis
+   application. When a channel in a Stasis application joins a bridge, an
+   implicit event subscription is created for that bridge as well. Previously,
+   when a channel left such a bridge, the subscription was leaked; this allowed
+   for later bridge events to continue to be pushed to the subscribed
+   applications. That leak has been fixed; as a result, bridge events that were
+   delivered after a channel left the bridge are no longer delivered. An
+   application must subscribe to a bridge through the applications resource if
+   it wishes to receive all events related to a bridge.
+
+AMI:
+ - The AMI version has been changed to 2.5.0. This is to reflect backwards
+   compatible changes made since 12.0.0 was released.
+
+ - The DialStatus field in the DialEnd event can now have additional values.
+   This includes ABORT, CONTINUE, and GOTO.
+
+ - The res_mwi_external_ami module can, if loaded, provide additional AMI
+   actions and events that convey MWI state within Asterisk. This includes
+   the MWIGet, MWIUpdate, and MWIDelete actions, as well as the MWIGet and
+   MWIGetComplete events that occur in response to an MWIGet action.
+
+ - AMI now contains a new class authorization, 'security'. This is used with
+   the following new events: FailedACL, InvalidAccountID, SessionLimit,
+   MemoryLimit, LoadAverageLimit, RequestNotAllowed, AuthMethodNotAllowed,
+   RequestBadFormat, SuccessfulAuth, UnexpectedAddress, ChallengeResponseFailed,
+   InvalidPassword, ChallengeSent, and InvalidTransport.
+
+ - Bridge related events now have two additional fields: BridgeName and
+   BridgeCreator. BridgeName is a descriptive name for the bridge;
+   BridgeCreator is the name of the entity that created the bridge. This
+   affects the following events: ConfbridgeStart, ConfbridgeEnd,
+   ConfbridgeJoin, ConfbridgeLeave, ConfbridgeRecord, ConfbridgeStopRecord,
+   ConfbridgeMute, ConfbridgeUnmute, ConfbridgeTalking, BlindTransfer,
+   AttendedTransfer, BridgeCreate, BridgeDestroy, BridgeEnter, BridgeLeave
+
+ - MixMonitor AMI actions now require users to have authorization classes.
+   * MixMonitor - system
+   * MixMonitorMute - call or system
+   * StopMixMonitor - call or system
+
+ - Removed the undocumented manager.conf block-sockets option.  It interferes with
+   TCP/TLS inactivity timeouts.
+
+ - The response to the PresenceState AMI action has historically contained two
+   Message keys. The first of these is used as an informative message regarding
+   the success/failure of the action; the second contains a Presence state
+   specific message. Having two keys with the same unique name in an AMI
+   message is cumbersome for some client; hence, the Presence specific Message
+   has been deprecated. The message will now contain a PresenceMessage key
+   for the presence specific information; the Message key containing presence
+   information will be removed in the next major version of AMI.
+
+ - The manager.conf 'eventfilter' now takes an "extended" regular expression
+   instead of a "basic" one.
+
+CDRs:
+ - The "endbeforehexten" setting now defaults to "yes", instead of "no".
+   When set to "no", yhis setting will cause a new CDR to be generated when a
+   channel enters into hangup logic (either the 'h' extension or a hangup
+   handler subroutine). In general, this is not the preferred default: this
+   causes extra CDRs to be generated for a channel in many common dialplans.
+
+CLI commands:
+ - "core show settings" now lists the current console verbosity in addition
+   to the root console verbosity.
+
+ - "core set verbose" has not been able to support the by module verbose
+   logging levels since verbose logging levels were made per console.  That
+   syntax is now removed and a silence option added in its place.
+
+Logging:
+ - The 'verbose' setting in logger.conf still takes an optional argument,
+   specifying the verbosity level for each logging destination.  However,
+   the default is now to once again follow the current root console level.
+   As a result, using the AMI Command action with "core set verbose" could
+   again set the root console verbose level and affect the verbose level
+   logged.
+
+HTTP:
+ - Added http.conf session_inactivity timer option to close HTTP connections
+   that aren't doing anything.
+
+ - Added support for persistent HTTP connections.  To enable persistent
+   HTTP connections configure the keep alive time between HTTP requests.  The
+   keep alive time between HTTP requests is configured in http.conf with the
+   session_keep_alive parameter.
+
+Realtime Configuration:
+ - WARNING: The database migration script that adds the 'extensions' table for
+   realtime had to be modified due to an error when installing for MySQL.  The
+   'extensions' table's 'id' column was changed to be a primary key.  This could
+   potentially cause a migration problem.  If so, it may be necessary to
+   manually alter the affected table/column to bring it back in line with the
+   migration scripts.
+
+ - New columns have been added to realtime tables for 'support_path' on
+   ps_registrations and ps_aors and for 'path' on ps_contacts for the new
+   SIP Path support in chan_pjsip.
+
+ - The following new tables have been added for pjsip realtime: 'ps_systems',
+   'ps_globals', 'ps_tranports', 'ps_registrations'.
+
+ - The following columns were added to the 'ps_aors' realtime table:
+   'maximum_expiration', 'outbound_proxy', and 'support_path'.
+
+ - The following columns were added to the 'ps_contacts' realtime table:
+   'outbound_proxy', 'user_agent', and 'path'.
+
+ - New columns have been added to the ps_endpoints realtime table for the
+   'media_address', 'redirect_method' and 'set_var' options.  Also the
+   'mwi_fromuser' column was renamed to 'mwi_from_user'. A new column
+   'message_context' was added to let users configure how MESSAGE requests are
+   routed to the dialplan.
+
+ - A new column was added to the 'ps_globals' realtime table for the 'debug'
+   option.
+
+ - PJSIP endpoint columns 'tos_audio' and 'tos_video' have been changed from
+   yes/no enumerators to string values. 'cos_audio' and 'cos_video' have been
+   changed from yes/no enumerators to integer values. PJSIP transport column
+   'tos' has been changed from a yes/no enumerator to a string value. 'cos' has
+   been changed from a yes/no enumerator to an integer value.
+
+ - The 'queues' and 'queue_members' realtime tables have been added to the
+   config Alembic scripts.
+
+ - A new set of Alembic scripts has been added for CDR tables. This will create
+   a 'cdr' table with the default schema that Asterisk expects.
+
+ - A new upgrade script has been added that adds a 'queue_rules' table for
+   app_queue. Users of app_queue can store queue rules in a database. It is
+   important to note that app_queue only looks for this table on module load or
+   module reload; for more information, see the CHANGES file.
+
+Resources:
+
+res_odbc:
+- The compatibility setting, allow_empty_string_in_nontext, has been removed.
+  Empty column values will be stored as empty strings during realtime updates.
+
+res_jabber:
+ - This module was deprecated and has been removed. Users of this module should
+   use res_xmpp instead.
+
+res_http_websocket:
+ - Added a compatibility option to ari.conf, sip.conf, and pjsip.conf
+   'websocket_write_timeout'. When a websocket connection exists where Asterisk
+   writes a substantial amount of data to the connected client, and the connected
+   client is slow to process the received data, the socket may be disconnected.
+   In such cases, it may be necessary to adjust this value.
+   Default is 100 ms.
+Scripts:
+
+safe_asterisk:
+ - The safe_asterisk script was previously not installed on top of an existing
+   version. This caused bug-fixes in that script not to be deployed. If your
+   safe_asterisk script is customized, be sure to keep your changes. Custom
+   values for variables should be created in *.sh file(s) inside
+   ASTETCDIR/startup.d/. See ASTERISK-21965.
+
+ - Changed a log message in safe_asterisk and the $NOTIFY mail subject. If
+   you use tools to parse either of them, update your parse functions
+   accordingly. The changed strings are:
+   - "Exited on signal $EXITSIGNAL" => "Asterisk exited on signal $EXITSIGNAL."
+   - "Asterisk Died" => "Asterisk on $MACHINE died (sig $EXITSIGNAL)"
+
+Utilities:
+ - The refcounter program has been removed in favor of the refcounter.py script
+   in contrib/scripts.
+
+===========================================================
+===========================================================
diff -pruN 1:13.23.1~dfsg-2/UPGRADE-14.txt 1:16.1.1~dfsg-1/UPGRADE-14.txt
--- 1:13.23.1~dfsg-2/UPGRADE-14.txt	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/UPGRADE-14.txt	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,115 @@
+===========================================================
+===
+=== Information for upgrading between Asterisk versions
+===
+=== These files document all the changes that MUST be taken
+=== into account when upgrading between the Asterisk
+=== versions listed below. These changes may require that
+=== you modify your configuration files, dialplan or (in
+=== some cases) source code if you have your own Asterisk
+=== modules or patches. These files also include advance
+=== notice of any functionality that has been marked as
+=== 'deprecated' and may be removed in a future release,
+=== along with the suggested replacement functionality.
+===
+=== UPGRADE-1.2.txt -- Upgrade info for 1.0 to 1.2
+=== UPGRADE-1.4.txt -- Upgrade info for 1.2 to 1.4
+=== UPGRADE-1.6.txt -- Upgrade info for 1.4 to 1.6
+=== UPGRADE-1.8.txt -- Upgrade info for 1.6 to 1.8
+=== UPGRADE-10.txt  -- Upgrade info for 1.8 to 10
+=== UPGRADE-11.txt  -- Upgrade info for 10 to 11
+=== UPGRADE-12.txt  -- Upgrade info for 11 to 12
+=== UPGRADE-13.txt  -- Upgrade info for 12 to 13
+===========================================================
+
+From 14.6.0 to 14.7.0:
+
+Core:
+ - ast_app_parse_timelen now returns an error if it encounters extra characters
+   at the end of the string to be parsed.
+
+From 14.4.0 to 14.5.0:
+
+Core:
+ - Support for embedded modules has been removed.  This has not worked in
+   many years.  LOADABLE_MODULES menuselect option is also removed as
+   loadable module support is now always enabled.
+
+From 14.3.0 to 14.4.0:
+
+res_rtp_asterisk:
+ - The RTP layer of Asterisk now has support for RFC 5761: "Multiplexing RTP
+   Data and Control Packets on a Single Port." For the PJSIP channel driver,
+   chan_pjsip, you can set "rtcp_mux = yes" on a PJSIP endpoint in pjsip.conf
+   to enable the feature. For chan_sip you can set "rtcp_mux = yes" either
+   globally or on a per-peer basis in sip.conf.
+
+New in 14.0.0
+
+ARI:
+ - The policy for when to send "Dial" events has changed. Previously, "Dial"
+   events were sent on the calling channel's topic. However, starting in Asterisk
+   14, if there is no calling channel on which to send the event, the event is
+   instead sent on the called channel's topic. Note that for the ARI channels
+   resource's dial operation, this means that the "Dial" events will always be
+   sent on the called channel's topic.
+
+Channel Drivers:
+
+chan_dahdi:
+ - For users using the FXO port (FXS signaling) distinctive ring detection
+   feature, you will need to adjust the dringX count values.  The count
+   values now only record ring end events instead of any DAHDI event.  A
+   ring-ring-ring pattern would exceed the pattern limits and stop
+   Caller-ID detection.
+
+chan_sip:
+ - The SIP dial string has been extended past the [!dnid] option by another
+   exclamation mark: [!dnid[!fromuri].  An exclamation mark in the To-URI
+   will now mean changes to the From-URI.
+
+Core:
+ - The REF_DEBUG compiler flag is now used to enable refdebug by default.
+   The setting can be overridden in asterisk.conf by setting refdebug in
+   the options category.  No recompile is required to enable/disable it.
+
+ - Modified processing of command-line options to first parse only what
+   is necessary to read asterisk.conf. Once asterisk.conf is fully loaded,
+   the remaining options are processed.  The -X option now applies to
+   asterisk.conf only.  To enable #exec for other config files you must
+   set execincludes=yes in asterisk.conf.  Any other option set on the
+   command-line will now override the equivalent setting from asterisk.conf.
+
+AMI:
+ - The 'ModuleCheck' Action's Version key will no longer show the module
+   version. The value will always be blank.
+
+CLI:
+ - The 'core show file version' command has been removed. When Asterisk
+   moved to Git, the source control version support was removed. As a
+   result, the CLi command was no longer useful and was removed as well.
+
+Logging:
+ - The first callid created is now 1 instead of 0.  The value 0
+   is now reserved to represent a lack of callid.
+
+AMI:
+ - The Command action now sends the output from the CLI command as a series
+   of Output headers for each line instead of as a block of text with the
+   --END COMMAND-- delimiter to match the output from other actions.
+
+   Commands that fail to execute (no such command, invalid syntax etc.) now
+   return an Error response instead of Success.
+
+app_amd:
+ - The 'maximum_number_of_words' configuration option and parameter to the AMD
+   application previously did not match the documented functionality + variable
+   name.  In Asterisk 13, a value of '3' would mean that if '3' words were detected,
+   the result would be detection as a 'MACHINE'.  As of this version, the value
+   reflects the maximum words that if EXCEEDED (rather than reached), would
+   result in detection as a machine.  This means that you should update this
+   value to be one higher than your previos value, if your previous value
+   was working well for you.
+
+===========================================================
+===========================================================
diff -pruN 1:13.23.1~dfsg-2/UPGRADE-15.txt 1:16.1.1~dfsg-1/UPGRADE-15.txt
--- 1:13.23.1~dfsg-2/UPGRADE-15.txt	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/UPGRADE-15.txt	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,63 @@
+===========================================================
+===
+=== Information for upgrading between Asterisk versions
+===
+=== These files document all the changes that MUST be taken
+=== into account when upgrading between the Asterisk
+=== versions listed below. These changes may require that
+=== you modify your configuration files, dialplan or (in
+=== some cases) source code if you have your own Asterisk
+=== modules or patches. These files also include advance
+=== notice of any functionality that has been marked as
+=== 'deprecated' and may be removed in a future release,
+=== along with the suggested replacement functionality.
+===
+=== UPGRADE-1.2.txt -- Upgrade info for 1.0 to 1.2
+=== UPGRADE-1.4.txt -- Upgrade info for 1.2 to 1.4
+=== UPGRADE-1.6.txt -- Upgrade info for 1.4 to 1.6
+=== UPGRADE-1.8.txt -- Upgrade info for 1.6 to 1.8
+=== UPGRADE-10.txt  -- Upgrade info for 1.8 to 10
+=== UPGRADE-11.txt  -- Upgrade info for 10 to 11
+=== UPGRADE-12.txt  -- Upgrade info for 11 to 12
+=== UPGRADE-13.txt  -- Upgrade info for 12 to 13
+=== UPGRADE-14.txt  -- Upgrade info for 13 to 14
+===========================================================
+
+From 15.2.0 to 15.3.0:
+
+res_pjsip
+------------------
+ * Users who are matching endpoints by SIP header need to reevaluate their
+   global "endpoint_identifier_order" option in light of the "ip" endpoint
+   identifier method split into the "ip" and "header" endpoint identifier
+   methods.
+
+res_pjsip_endpoint_identifier_ip
+------------------
+ * The endpoint identifier "ip" method previously recognized endpoints either
+   by IP address or a matching SIP header.  The "ip" endpoint identifier method
+   is now split into the "ip" and "header" endpoint identifier methods.  The
+   "ip" endpoint identifier method only matches by IP address and the "header"
+   endpoint identifier method only matches by SIP header.  The split allows the
+   user to control the relative priority of the IP address and the SIP header
+   identification methods in the global "endpoint_identifier_order" option.
+   e.g., If you have two type=identify sections where one matches by IP address
+   for endpoint alice and the other matches by SIP header for endpoint bob then
+   you can now predict which endpoint is matched when a request comes in that
+   matches both.
+
+New in 15.0.0:
+
+Build System:
+ - '--with-pjproject-bundled' is now the default when running ./configure
+   It can be disabled with '--without-pjproject-bundled'.
+
+Core:
+ - Multi-stream support has been added so a channel can have multiple
+   streams of the same type such as audio and video.
+
+ - The 'Data Retrieval API' has been removed. This API was not actively
+   maintained, was not added to new modules (such as res_pjsip), and there
+   exist better alternatives to acquire the same information, such as the
+   ARI. As a result, the 'DataGet' AMI action as well as the 'data get'
+   CLI command have been removed.
diff -pruN 1:13.23.1~dfsg-2/UPGRADE.txt 1:16.1.1~dfsg-1/UPGRADE.txt
--- 1:13.23.1~dfsg-2/UPGRADE.txt	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/UPGRADE.txt	2018-12-26 18:45:13.000000000 +0000
@@ -19,603 +19,74 @@
 === UPGRADE-10.txt  -- Upgrade info for 1.8 to 10
 === UPGRADE-11.txt  -- Upgrade info for 10 to 11
 === UPGRADE-12.txt  -- Upgrade info for 11 to 12
+=== UPGRADE-13.txt  -- Upgrade info for 12 to 13
+=== UPGRADE-14.txt  -- Upgrade info for 13 to 14
+=== UPGRADE-15.txt  -- Upgrade info for 14 to 15
 ===========================================================
 
-From 13.20.0 to 13.21.0:
-
-app_dial
-------------------
- * The Dial application now supports early-media video (in addition to
-   audio) on both the calling as well as the called party.
-   Be aware that this is a change in behavior.
-
-From 13.19.0 to 13.20.0:
-
-app_confbridge
-------------------
- * Made the AMI ConfbridgeList action's ConfbridgeList events output all
-   the standard channel snapshot headers instead of a few hand-coded channel
-   snapshot headers.  The benefit is that the CallerIDName gets disruptive
-   characters like CR, LF, Tab, and a few others escaped.  However, an empty
-   CallerIDName is now output as "<unknown>" instead of "<no name>".
-
-res_pjsip
-------------------
- * Users who are matching endpoints by SIP header need to reevaluate their
-   global "endpoint_identifier_order" option in light of the "ip" endpoint
-   identifier method split into the "ip" and "header" endpoint identifier
-   methods.
-
- * The pjsip_transport_event feature introduced in 13.18.0 has been refactored.
-   Any external modules that may have used that feature (highly unlikey) will
-   need to be changed as the API has been altered slightly.
-
-res_pjsip_endpoint_identifier_ip
-------------------
- * The endpoint identifier "ip" method previously recognized endpoints either
-   by IP address or a matching SIP header.  The "ip" endpoint identifier method
-   is now split into the "ip" and "header" endpoint identifier methods.  The
-   "ip" endpoint identifier method only matches by IP address and the "header"
-   endpoint identifier method only matches by SIP header.  The split allows the
-   user to control the relative priority of the IP address and the SIP header
-   identification methods in the global "endpoint_identifier_order" option.
-   e.g., If you have two type=identify sections where one matches by IP address
-   for endpoint alice and the other matches by SIP header for endpoint bob then
-   you can now predict which endpoint is matched when a request comes in that
-   matches both.
-
-res_pjsip_transport_management
-------------------
- * Since res_pjsip_transport_management provides several attack
-   mitigation features, its functionality moved to res_pjsip and
-   this module has been removed.  This way the features will always
-   be available if res_pjsip is loaded.
-
-From 13.17.0 to 13.18.0:
-
-Core:
- - ast_app_parse_timelen now returns an error if it encounters extra characters
-   at the end of the string to be parsed.
-
-From 13.15.0 to 13.16.0:
+From 16.0.0 to 16.1.0:
 
 Core:
- - Support for embedded modules has been removed.  This has not worked in
-   many years.  LOADABLE_MODULES menuselect option is also removed as
-   loadable module support is now always enabled.
-
-From 13.14.0 to 13.15.0:
-
-res_rtp_asterisk:
- - The RTP layer of Asterisk now has support for RFC 5761: "Multiplexing RTP
-   Data and Control Packets on a Single Port." For the PJSIP channel driver,
-   chan_pjsip, you can set "rtcp_mux = yes" on a PJSIP endpoint in pjsip.conf
-   to enable the feature. For chan_sip you can set "rtcp_mux = yes" either
-   globally or on a per-peer basis in sip.conf.
-
-From 13.8.0 to 13.9.0:
-
-res_parking:
- - The dynamic parking lot creation channel variables PARKINGDYNAMIC,
-   PARKINGDYNCONTEXT, PARKINGDYNEXTEN, and PARKINGDYNPOS are now looked
-   for in the parker's channel instead of the parked channel.  This is only
-   of significance if the parker uses blind transfer or the DTMF one-step
-   parking feature.  You need to use the double underscore '__' inheritance
-   for these variables.  The indefinite inheritance is also recommended
-   for the PARKINGEXTEN variable.
-
-From 13.7.0 to 13.8.0:
-
-res_pjsip:
- - res_pjsip now depends on res_pjproject.  If autoload=no in modules.conf,
-   res_pjproject must be explicitly loaded or res_pjsip and all of its
-   dependents will fail to load.
-
-REDIRECTING(reason):
- - See the CHANGES file for a description of the behavior change.
-
-ODBC:
- - Connection pooling/sharing has been completely removed from Asterisk
-   in favor of letting ODBC take care of it instead. It is strongly
-   recommended that you enable connection pooling in unixODBC. As a result
-   of this, the "pooling", "shared_connection", "limit", and "idlecheck"
-   options in res_odbc.conf are deprecated and provide no function.
-
-From 13.5.0 to 13.6.0:
-
-ARI:
- - The version of ARI has been updated to 1.9.0 to reflect the backwards
-   compatible changes outlined in the CHANGES file.
-
-From 13.4.0 to 13.5.0:
-
-AMI:
- - The version of AMI has been bumped to 2.8.0 to account for backwards
-   compatible features included with this release.  See CHANGES for more
-   information.
-
-ARI:
- - The version of ARI has been updated to 1.8.0 to reflect the backwards
-   compatible changes outlined in the CHANGES file.
+ - ast_bt_get_symbols() now returns a vector of strings instead of an
+   array of strings.  This must be freed with ast_bt_free_symbols.
 
-From 13.3.0 to 13.4.0:
+New in 16.0.0:
 
-Source Control:
- - Asterisk has moved from Subversion to Git. As a result, several changes
-   were required in functionality. These are listed individually in the
-   notes below.
+app_fax:
+ - The app_fax module is now deprecated, users should migrate to the
+   replacement module res_fax.
+
+app_macro:
+ - The app_macro module is now deprecated and by default it is no longer
+   built.  Users should migrate to app_stack (Gosub).  A warning is logged
+   the first time any Macro is used.
 
 AMI:
- - The 'ModuleCheck' Action's Version key will now always report the
-   current version of Asterisk.
-
-ARI:
- - The version of ARI has been updated to 1.7.0 to reflect the backwards
-   compatible changes outlined in the CHANGES file.
-
-CLI:
- - The 'core show file version' command has been altered. In the past,
-   this command would show the SVN revision of the source files compiled
-   in Asterisk. However, when Asterisk moved to Git, the source control
-   version support was removed. As a result, the version information shown
-   by the CLI command is always the Asterisk version. This CLI command
-   will be removed in Asterisk 14.
-
-chan_dahdi:
- - Added the force_restart_unavailable_chans compatibility option.  When
-   enabled it causes Asterisk to restart the ISDN B channel if an outgoing
-   call receives cause 44 (Requested channel not available).  The new option
-   is enabled by default in current release branches for backward
-   compatibility.
-
-res_pjsip:
- - The dtmf_mode now supports a new option, 'auto'. This mode will attempt to
-   detect if the device supports RFC4733 DTMF. If so, it will choose that
-   DTMF type; if not, it will choose 'inband' DTMF.
-
-res_pjsip_dlg_options:
- - A new module, this handles OPTIONS requests sent in-dialog. This module
-   should have no adverse effects for those upgrading; this note merely
-   serves as an indication that a new module exists.
-
-cdr_odbc:
- - Added support for post-1.8 CDR columns 'peeraccount', 'linkedid', and
-   'sequence'. Support for the new columns can be enabled via the newcdrcolumns
-   option in cdr_odbc.conf.
-
-cdr_csv:
- - Added a new configuration option, "newcdrcolumns", which enables use of the
-   post-1.8 CDR columns 'peeraccount', 'linkedid', and 'sequence'.
-
-From 13.2.0 to 13.3.0:
-
-chan_dahdi:
- - For users using the FXO port (FXS signaling) distinctive ring detection
-   feature, you will need to adjust the dringX count values.  The count
-   values now only record ring end events instead of any DAHDI event.  A
-   ring-ring-ring pattern would exceed the pattern limits and stop
-   Caller-ID detection.
-
-From 13.1.0 to 13.2.0:
+ - The ContactStatus and Status fields for the manager events ContactStatus
+   and ContactStatusDetail are now set to "NonQualified" when a contact exists
+   but has not been qualified.
+ - The ContactStatus event will no longer be sent by PJSIP when a device
+   refreshes its registration.
+ - The "Newexten" event is now part of the "dialplan" class. The documentation
+   for Asterisk 15 already specified this, but the implementation was actually
+   using the "call" class instead.
 
 ARI:
- - The version of ARI has been bumped to 1.7.0 to account for backwards
-   compatible features included with this release. See CHANGES for more
-   information.
-
-AMI:
- - The version of AMI has been bumped to 2.7.0 to account for backwards
-   compatible features included with this release. See CHANGES for more
-   information.
-
-chan_dahdi:
- - The CALLERID(ani2) value for incoming calls is now populated in featdmf
-   signaling mode.  The information was previously discarded.
-
-chan_iax2:
- - The iax.conf forcejitterbuffer option has been removed.  It is now always
-   forced if you set iax.conf jitterbuffer=yes.  If you put a jitter buffer
-   on a channel it will be on the channel.
-
-From 13.0.0 to 13.1.0:
-
-ARI:
- - The version of ARI has been bumped to 1.6.0 to account for backwards
-   compatible features included with this release. See CHANGES for more
-   information.
-
-AMI:
- - The version of AMI has been bumped to 2.6.0 to account for backwards
-   compatible features included with this release. See CHANGES for more
-   information.
-
-Core:
- - The core of Asterisk uses a message bus called "Stasis" to distribute
-   information to internal components. For performance reasons, the message
-   distribution was modified to make use of a thread pool instead of a
-   dedicated thread per consumer in certain cases. The initial settings for
-   the thread pool can now be configured in 'stasis.conf'.
-
-PJSIP:
- - Added the pjsip.conf system type disable_tcp_switch option.  The option
-   allows the user to disable switching from UDP to TCP transports described
-   by RFC 3261 section 18.1.1.
-
-From 12 to 13:
-General Asterisk Changes:
- - The asterisk command line -I option and the asterisk.conf internal_timing
-   option are removed and always enabled if any timing module is loaded.
-
- - The per console verbose level feature as previously implemented caused a
-   large performance penalty.  The fix required some minor incompatibilities
-   if the new rasterisk is used to connect to an earlier version.  If the new
-   rasterisk connects to an older Asterisk version then the root console verbose
-   level is always affected by the "core set verbose" command of the remote
-   console even though it may appear to only affect the current console.  If
-   an older version of rasterisk connects to the new version then the
-   "core set verbose" command will have no effect.
-
- - The asterisk compatibility options in asterisk.conf have been removed.
-   These options enabled certain backwards compatibility features for
-   pbx_realtime, res_agi, and app_set that made their behaviour similar to
-   Asterisk 1.4. Users who used these backwards compatibility settings should
-   update their dialplans to use ',' instead of '|' as a delimiter, and should
-   use the Set dialplan application instead of the MSet dialplan application.
+ - The ContactInfo event's contact_status field is now set to "NonQualified"
+   when a contact exists but has not been qualified.
 
 Build System:
- - Sample config files have been moved from configs/ to a subfolder of that
-   directory, 'samples'.
-
- - The menuselect utility has been pulled into the Asterisk repository. As a
-   result, the libxml2 development library is now a required dependency for
-   Asterisk.
-
- - Added a new Compiler Flag, REF_DEBUG. When enabled, reference counted
-   objects will emit additional debug information to the refs log file located
-   in the standard Asterisk log file directory. This log file is useful in
-   tracking down object leaks and other reference counting issues. Prior to
-   this version, this option was only available by modifying the source code
-   directly. This change also includes a new script, refcounter.py, in the
-   contrib folder that will process the refs log file.
-
-Applications:
-
-ConfBridge:
- - The sound_place_into_conference sound used in Confbridge is now deprecated
-   and is no longer functional since it has been broken since its inception
-   and the fix involved using a different method to achieve the same goal. The
-   new method to achieve this functionality is by using sound_begin to play
-   a sound to the conference when waitmarked users are moved into the conference.
-
- - Added 'Admin' header to ConfbridgeJoin, ConfbridgeLeave, ConfbridgeMute,
-   ConfbridgeUnmute, and ConfbridgeTalking AMI events.
-
-ControlPlayback:
- - The ControlPlayback and 'control stream file' AGI command will no longer
-   implicitly answer the channel. If you do not answer the channel prior to
-   using either this application or AGI command, you must send Progress
-   first.
-
-Queue:
- - Queue rules provided in queuerules.conf can no longer be named "general".
-
-SetMusicOnHold:
- - The SetMusicOnHold dialplan application was deprecated and has been removed.
-   Users of the application should use the CHANNEL function's musicclass
-   setting instead.
-
-WaitMusicOnHold:
- - The WaitMusicOnHold dialplan application was deprecated and has been
-   removed. Users of the application should use MusicOnHold with a duration
-   parameter instead.
-
-CDR Backends:
- - The cdr_sqlite module was deprecated and has been removed. Users of this
-   module should use the cdr_sqlite3_custom module instead.
+ - MALLOC_DEBUG no longer has an effect on Asterisk's ABI.  Asterisk built
+   with MALLOC_DEBUG can now successfully load binary modules built without
+   MALLOC_DEBUG and vice versa.  Third-party pre-compiled modules no longer
+   need to have a special build with it enabled.
 
-Channel Drivers:
+ - Asterisk now depends on libjansson >= 2.11.  If this version is not
+   available on your distro you can use `./configure --with-jansson-bundled`.
 
 chan_dahdi:
- - SS7 support now requires libss7 v2.0 or later.
-
- - Added the inband_on_setup_ack compatibility option to chan_dahdi.conf to
-   deal with switches that don't send an inband progress indication in the
-   SETUP ACKNOWLEDGE message.
-   Default is now no.
-
-chan_gtalk
- - This module was deprecated and has been removed. Users of chan_gtalk
-   should use chan_motif.
-
-chan_h323
- - This module was deprecated and has been removed. Users of chan_h323
-   should use chan_ooh323.
-
-chan_jingle
- - This module was deprecated and has been removed. Users of chan_jingle
-   should use chan_motif.
-
-chan_pjsip:
- - Added a 'force_avp' option to chan_pjsip which will force the usage of
-   'RTP/AVP', 'RTP/AVPF', 'RTP/SAVP', or 'RTP/SAVPF' as the media transport type
-   in SDP offers depending on settings, even when DTLS is used for media
-   encryption.
-
- - Added a 'media_use_received_transport' option to chan_pjsip which will
-   cause the SDP answer to use the media transport as received in the SDP
-   offer.
-
-chan_sip:
- - Made set SIPREFERREDBYHDR as inheritable for better chan_pjsip
-   interoperability.
-
- - The SIPPEER dialplan function no longer supports using a colon as a
-   delimiter for parameters. The parameters for the function should be
-   delimited using a comma.
-
- - The SIPCHANINFO dialplan function was deprecated and has been removed. Users
-   of the function should use the CHANNEL function instead.
-
- - Added a 'force_avp' option for chan_sip. When enabled this option will
-   cause the media transport in the offer or answer SDP to be 'RTP/AVP',
-   'RTP/AVPF', 'RTP/SAVP', or 'RTP/SAVPF' even if a DTLS stream has been
-   configured. This option can be set to improve interoperability with WebRTC
-   clients that don't use the RFC defined transport for DTLS.
-
- - The 'dtlsverify' option in chan_sip now has additional values besides
-   'yes' and 'no'. If 'yes' is specified both the certificate and fingerprint
-   will be verified. If 'no' is specified then neither the certificate or
-   fingerprint is verified. If 'certificate' is specified then only the
-   certificate is verified. If 'fingerprint' is specified then only the
-   fingerprint is verified.
-
- - A 'dtlsfingerprint' option has been added to chan_sip which allows the
-   hash to be specified for the DTLS fingerprint placed in SDP. Supported
-   values are 'sha-1' and 'sha-256' with 'sha-256' being the default.
-
- - The 'progressinband=never' option is now more zealous in the persecution of
-   progress messages coming from Asterisk. Channels bridged with a SIP channel
-   that has 'progressinband=never' set will not be able to forward their
-   progress indications through to the SIP device. chan_sip will now turn such
-   progress indications into a 180 Ringing (if a 180 has not yet been
-   transmitted) if 'progressinband=never'.
-
-  - The codec preference order in an SDP during an offer is slightly different
-    than previous releases. Prior to Asterisk 13, the preference order of
-    codecs used to be:
-    (1) Our preferred codec
-    (2) Our configured codecs
-    (3) Any non-audio joint codecs
-
-    One of the ways the new media format architecture in Asterisk 13 improves
-    performance is by reference counting formats, such that they can be reused
-    in many places without additional allocation. To not require a large
-    amount of locking, an instance of a format is immutable by convention.
-    This works well except for formats with attributes. Since a media format
-    with an attribute is a different object than the same format without an
-    attribute, we have to carry over the formats with attributes from an
-    inbound offer so that the correct attributes are offered in an outgoing
-    INVITE request. This requires some subtle tweaks to the preference order
-    to ensure that the media format with attributes is offered to a remote
-    peer, as opposed to the same media format (but without attributes) that
-    may be stored in the peer object.
-
-    All of this means that our offer offer list will now be:
-    (1) Our preferred codec
-    (2) Any joint codecs offered by the inbound offer
-    (3) All other codecs that are not the preferred codec and not a joint
-        codec offered by the inbound offer
-
-chan_unistim:
- - The unistim.conf 'dateformat' has changed meaning of options values to conform
-   values used inside Unistim protocol
+ - Timeouts for reading digits from analog phones are now configurable in
+   chan_dahdi.conf: firstdigit_timeout, interdigit_timeout, matchdigit_timeout.
 
- - Added 'dtmf_duration' option with changing default operation to disable
-   receivied dtmf playback on unistim phone
+cdr_syslog:
+ - The cdr_syslog module is now deprecated and by default it is no longer
+   built.
+
+res_config_sqlite:
+ - The res_config_sqlite module is now deprecated, users should migrate to the
+   replacement module res_config_sqlite3.
+
+res_monitor:
+ - The res_monitor module is now deprecated, users should migrate to the
+   replacement module app_mixmonitor.
 
 Core:
-
-Account Codes:
- - accountcode behavior changed somewhat to add functional peeraccount
-   support.  The main change is that local channels now cross accountcode
-   and peeraccount across the special bridge between the ;1 and ;2 channels
-   just like channels between normal bridges.  See the CHANGES file for
-   more information.
-
-ARI:
- - The ARI version has been changed to 1.5.0. This is to reflect backwards
-   compatible changes made since 12.0.0 was released.
-
- - Added a new ARI resource 'mailboxes' which allows the creation and
-   modification of mailboxes managed by external MWI. Modules res_mwi_external
-   and res_stasis_mailbox must be enabled to use this resource.
-
- - Added new events for externally initiated transfers. The event
-   BridgeBlindTransfer is now raised when a channel initiates a blind transfer
-   of a bridge in the ARI controlled application to the dialplan; the
-   BridgeAttendedTransfer event is raised when a channel initiates an
-   attended transfer of a bridge in the ARI controlled application to the
-   dialplan.
-
- - Channel variables may now be specified as a body parameter to the
-   POST /channels operation. The 'variables' key in the JSON is interpreted
-   as a sequence of key/value pairs that will be added to the created channel
-   as channel variables. Other parameters in the JSON body are treated as
-   query parameters of the same name.
-
- - A bug fix in bridge creation has caused a behavioural change in how
-   subscriptions are created for bridges. A bridge created through ARI, does
-   not, by itself, have a subscription created for any particular Stasis
-   application. When a channel in a Stasis application joins a bridge, an
-   implicit event subscription is created for that bridge as well. Previously,
-   when a channel left such a bridge, the subscription was leaked; this allowed
-   for later bridge events to continue to be pushed to the subscribed
-   applications. That leak has been fixed; as a result, bridge events that were
-   delivered after a channel left the bridge are no longer delivered. An
-   application must subscribe to a bridge through the applications resource if
-   it wishes to receive all events related to a bridge.
-
-AMI:
- - The AMI version has been changed to 2.5.0. This is to reflect backwards
-   compatible changes made since 12.0.0 was released.
-
- - The DialStatus field in the DialEnd event can now have additional values.
-   This includes ABORT, CONTINUE, and GOTO.
-
- - The res_mwi_external_ami module can, if loaded, provide additional AMI
-   actions and events that convey MWI state within Asterisk. This includes
-   the MWIGet, MWIUpdate, and MWIDelete actions, as well as the MWIGet and
-   MWIGetComplete events that occur in response to an MWIGet action.
-
- - AMI now contains a new class authorization, 'security'. This is used with
-   the following new events: FailedACL, InvalidAccountID, SessionLimit,
-   MemoryLimit, LoadAverageLimit, RequestNotAllowed, AuthMethodNotAllowed,
-   RequestBadFormat, SuccessfulAuth, UnexpectedAddress, ChallengeResponseFailed,
-   InvalidPassword, ChallengeSent, and InvalidTransport.
-
- - Bridge related events now have two additional fields: BridgeName and
-   BridgeCreator. BridgeName is a descriptive name for the bridge;
-   BridgeCreator is the name of the entity that created the bridge. This
-   affects the following events: ConfbridgeStart, ConfbridgeEnd,
-   ConfbridgeJoin, ConfbridgeLeave, ConfbridgeRecord, ConfbridgeStopRecord,
-   ConfbridgeMute, ConfbridgeUnmute, ConfbridgeTalking, BlindTransfer,
-   AttendedTransfer, BridgeCreate, BridgeDestroy, BridgeEnter, BridgeLeave
-
- - MixMonitor AMI actions now require users to have authorization classes.
-   * MixMonitor - system
-   * MixMonitorMute - call or system
-   * StopMixMonitor - call or system
-
- - Removed the undocumented manager.conf block-sockets option.  It interferes with
-   TCP/TLS inactivity timeouts.
-
- - The response to the PresenceState AMI action has historically contained two
-   Message keys. The first of these is used as an informative message regarding
-   the success/failure of the action; the second contains a Presence state
-   specific message. Having two keys with the same unique name in an AMI
-   message is cumbersome for some client; hence, the Presence specific Message
-   has been deprecated. The message will now contain a PresenceMessage key
-   for the presence specific information; the Message key containing presence
-   information will be removed in the next major version of AMI.
-
- - The manager.conf 'eventfilter' now takes an "extended" regular expression
-   instead of a "basic" one.
-
-CDRs:
- - The "endbeforehexten" setting now defaults to "yes", instead of "no".
-   When set to "no", yhis setting will cause a new CDR to be generated when a
-   channel enters into hangup logic (either the 'h' extension or a hangup
-   handler subroutine). In general, this is not the preferred default: this
-   causes extra CDRs to be generated for a channel in many common dialplans.
-
-CLI commands:
- - "core show settings" now lists the current console verbosity in addition
-   to the root console verbosity.
-
- - "core set verbose" has not been able to support the by module verbose
-   logging levels since verbose logging levels were made per console.  That
-   syntax is now removed and a silence option added in its place.
-
-Logging:
- - The 'verbose' setting in logger.conf still takes an optional argument,
-   specifying the verbosity level for each logging destination.  However,
-   the default is now to once again follow the current root console level.
-   As a result, using the AMI Command action with "core set verbose" could
-   again set the root console verbose level and affect the verbose level
-   logged.
-
-HTTP:
- - Added http.conf session_inactivity timer option to close HTTP connections
-   that aren't doing anything.
-
- - Added support for persistent HTTP connections.  To enable persistent
-   HTTP connections configure the keep alive time between HTTP requests.  The
-   keep alive time between HTTP requests is configured in http.conf with the
-   session_keep_alive parameter.
-
-Realtime Configuration:
- - WARNING: The database migration script that adds the 'extensions' table for
-   realtime had to be modified due to an error when installing for MySQL.  The
-   'extensions' table's 'id' column was changed to be a primary key.  This could
-   potentially cause a migration problem.  If so, it may be necessary to
-   manually alter the affected table/column to bring it back in line with the
-   migration scripts.
-
- - New columns have been added to realtime tables for 'support_path' on
-   ps_registrations and ps_aors and for 'path' on ps_contacts for the new
-   SIP Path support in chan_pjsip.
-
- - The following new tables have been added for pjsip realtime: 'ps_systems',
-   'ps_globals', 'ps_tranports', 'ps_registrations'.
-
- - The following columns were added to the 'ps_aors' realtime table:
-   'maximum_expiration', 'outbound_proxy', and 'support_path'.
-
- - The following columns were added to the 'ps_contacts' realtime table:
-   'outbound_proxy', 'user_agent', and 'path'.
-
- - New columns have been added to the ps_endpoints realtime table for the
-   'media_address', 'redirect_method' and 'set_var' options.  Also the
-   'mwi_fromuser' column was renamed to 'mwi_from_user'. A new column
-   'message_context' was added to let users configure how MESSAGE requests are
-   routed to the dialplan.
-
- - A new column was added to the 'ps_globals' realtime table for the 'debug'
-   option.
-
- - PJSIP endpoint columns 'tos_audio' and 'tos_video' have been changed from
-   yes/no enumerators to string values. 'cos_audio' and 'cos_video' have been
-   changed from yes/no enumerators to integer values. PJSIP transport column
-   'tos' has been changed from a yes/no enumerator to a string value. 'cos' has
-   been changed from a yes/no enumerator to an integer value.
-
- - The 'queues' and 'queue_members' realtime tables have been added to the
-   config Alembic scripts.
-
- - A new set of Alembic scripts has been added for CDR tables. This will create
-   a 'cdr' table with the default schema that Asterisk expects.
-
- - A new upgrade script has been added that adds a 'queue_rules' table for
-   app_queue. Users of app_queue can store queue rules in a database. It is
-   important to note that app_queue only looks for this table on module load or
-   module reload; for more information, see the CHANGES file.
-
-Resources:
-
-res_odbc:
-- The compatibility setting, allow_empty_string_in_nontext, has been removed.
-  Empty column values will be stored as empty strings during realtime updates.
-
-res_jabber:
- - This module was deprecated and has been removed. Users of this module should
-   use res_xmpp instead.
-
-res_http_websocket:
- - Added a compatibility option to ari.conf, sip.conf, and pjsip.conf
-   'websocket_write_timeout'. When a websocket connection exists where Asterisk
-   writes a substantial amount of data to the connected client, and the connected
-   client is slow to process the received data, the socket may be disconnected.
-   In such cases, it may be necessary to adjust this value.
-   Default is 100 ms.
-
-Scripts:
-
-safe_asterisk:
- - The safe_asterisk script was previously not installed on top of an existing
-   version. This caused bug-fixes in that script not to be deployed. If your
-   safe_asterisk script is customized, be sure to keep your changes. Custom
-   values for variables should be created in *.sh file(s) inside
-   ASTETCDIR/startup.d/. See ASTERISK-21965.
-
- - Changed a log message in safe_asterisk and the $NOTIFY mail subject. If
-   you use tools to parse either of them, update your parse functions
-   accordingly. The changed strings are:
-   - "Exited on signal $EXITSIGNAL" => "Asterisk exited on signal $EXITSIGNAL."
-   - "Asterisk Died" => "Asterisk on $MACHINE died (sig $EXITSIGNAL)"
-
-Utilities:
- - The refcounter program has been removed in favor of the refcounter.py script
-   in contrib/scripts.
-
-===========================================================
-===========================================================
+ - libedit is no longer available as an embedded library and must be provided
+   by the system.
+ - The module loader now enforces inter-module dependencies.  This ensures that
+   a module is not started before another it depends on, even if preload is used.
+   If a dependency is not available or fails to startup this will block any
+   dependants from startup.
+ - Parts of the Asterisk core which can load configuration from realtime are now
+   built-in modules.  It is no longer necessary to preload realtime drivers as
+   they are always initialized before the built-in modules.
diff -pruN 1:13.23.1~dfsg-2/utils/ael_main.c 1:16.1.1~dfsg-1/utils/ael_main.c
--- 1:13.23.1~dfsg-2/utils/ael_main.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/ael_main.c	2018-12-26 18:45:13.000000000 +0000
@@ -11,6 +11,7 @@
 	<support_level>extended</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
 #include <locale.h>
@@ -18,8 +19,6 @@
 #include <regex.h>
 #include <limits.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include "asterisk/backtrace.h"
 #include "asterisk/channel.h"
 #include "asterisk/ast_expr.h"
@@ -32,10 +31,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revisi
 
 int option_debug = 0;
 int option_verbose = 0;
-#if !defined(LOW_MEMORY)
-void ast_register_file_version(const char *file, const char *version) { }
-void ast_unregister_file_version(const char *file) { }
-#endif
 
 /*** MODULEINFO
   	<depend>res_ael_share</depend>
@@ -103,7 +98,7 @@ static char var_dir[PATH_MAX];
 const char *ast_config_AST_CONFIG_DIR = config_dir;
 const char *ast_config_AST_VAR_DIR = var_dir;
 
-void ast_cli_register_multiple(void);
+void __ast_cli_register_multiple(void);
 int ast_add_extension2(struct ast_context *con,
 					   int replace, const char *extension, int priority, const char *label, const char *callerid,
 						const char *application, void *data, void (*datad)(void *),
@@ -180,7 +175,6 @@ struct ast_custom_function *ast_custom_f
 	return 0; /* in "standalone" mode, functions are just not avail */
 }
 
-#if !defined(LOW_MEMORY)
 int ast_add_profile(const char *x, uint64_t scale)
 {
 	if (!no_comp)
@@ -188,7 +182,6 @@ int ast_add_profile(const char *x, uint6
 
 	return 0;
 }
-#endif
 
 int ast_loader_register(int (*updater)(void))
 {
@@ -208,7 +201,7 @@ void ast_module_unregister(const struct
 }
 
 
-void ast_cli_register_multiple(void)
+void __ast_cli_register_multiple(void)
 {
 	if(!no_comp)
         	printf("Executed ast_cli_register_multiple();\n");
@@ -606,11 +599,9 @@ unsigned int ast_hashtab_hash_contexts(c
 }
 
 #ifdef DEBUG_THREADS
-#if !defined(LOW_MEMORY)
 void ast_mark_lock_acquired(void *lock_addr)
 {
 }
-#ifdef HAVE_BKTR
 void ast_remove_lock_info(void *lock_addr, struct ast_bt *bt)
 {
 }
@@ -620,30 +611,15 @@ void ast_store_lock_info(enum ast_lock_t
 {
 }
 
+#ifdef HAVE_BKTR
 int __ast_bt_get_addresses(struct ast_bt *bt)
 {
 	return 0;
 }
 
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames)
-{
-	char **foo = calloc(num_frames, sizeof(char *) + 1);
-	if (foo) {
-		int i;
-		for (i = 0; i < num_frames; i++) {
-			foo[i] = (char *) foo + sizeof(char *) * num_frames;
-		}
-	}
-	return foo;
-}
-#else
-void ast_remove_lock_info(void *lock_addr)
-{
-}
-
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-	int line_num, const char *func, const char *lock_name, void *lock_addr)
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames)
 {
+	return NULL;
 }
 #endif /* HAVE_BKTR */
 void ast_suspend_lock_info(void *lock_addr)
@@ -652,5 +628,4 @@ void ast_suspend_lock_info(void *lock_ad
 void ast_restore_lock_info(void *lock_addr)
 {
 }
-#endif /* !defined(LOW_MEMORY) */
 #endif /* DEBUG_THREADS */
diff -pruN 1:13.23.1~dfsg-2/utils/astdb2sqlite3.c 1:16.1.1~dfsg-1/utils/astdb2sqlite3.c
--- 1:13.23.1~dfsg-2/utils/astdb2sqlite3.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/astdb2sqlite3.c	2018-12-26 18:45:13.000000000 +0000
@@ -29,8 +29,6 @@
 
 #include "asterisk.h"
 
-//ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
-
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -95,7 +93,7 @@ static int db_put_raw(const char *key, s
 		fprintf(stderr, "Couldn't bind value to stmt: %s\n", sqlite3_errmsg(astdb));
 		res = -1;
 	} else if (sqlite3_step(put_stmt) != SQLITE_DONE) {
-		fprintf(stderr, "Couldn't execute statment: %s\n", sqlite3_errmsg(astdb));
+		fprintf(stderr, "Couldn't execute statement: %s\n", sqlite3_errmsg(astdb));
 		res = -1;
 	}
 	sqlite3_reset(put_stmt);
@@ -168,8 +166,8 @@ static int db_create_astdb(void)
 
 static int init_statements(void)
 {
-	/* Don't initialize create_astdb_statment here as the astdb table needs to exist
-	 * brefore these statments can be initialized */
+	/* Don't initialize create_astdb_statement here as the astdb table needs to exist
+	 * brefore these statements can be initialized */
 	return init_stmt(&put_stmt, put_stmt_sql, sizeof(put_stmt_sql));
 }
 
diff -pruN 1:13.23.1~dfsg-2/utils/astman.c 1:16.1.1~dfsg-1/utils/astman.c
--- 1:13.23.1~dfsg-2/utils/astman.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/astman.c	2018-12-26 18:45:13.000000000 +0000
@@ -26,8 +26,7 @@
 	<support_level>extended</support_level>
  ***/
 
-#include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
 #include <newt.h>
@@ -89,17 +88,6 @@ struct ast_chan {
 static AST_LIST_HEAD_NOLOCK_STATIC(chans, ast_chan);
 
 /* dummy functions to be compatible with the Asterisk core for md5.c */
-void ast_register_file_version(const char *file, const char *version);
-void ast_register_file_version(const char *file, const char *version)
-{
-}
-
-void ast_unregister_file_version(const char *file);
-void ast_unregister_file_version(const char *file)
-{
-}
-
-#if !defined(LOW_MEMORY)
 int ast_add_profile(const char *, uint64_t scale);
 int ast_add_profile(const char *s, uint64_t scale)
 {
@@ -116,7 +104,6 @@ int64_t ast_mark(int key, int start1_sto
 {
 	return 0;
 }
-#endif /* LOW_MEMORY */
 
 /* end of dummy functions */
 
diff -pruN 1:13.23.1~dfsg-2/utils/check_expr.c 1:16.1.1~dfsg-1/utils/check_expr.c
--- 1:13.23.1~dfsg-2/utils/check_expr.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/check_expr.c	2018-12-26 18:45:13.000000000 +0000
@@ -20,8 +20,8 @@
 	<support_level>extended</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/ast_expr.h"
 
@@ -42,9 +42,9 @@ enum ast_lock_type {
 #define MALLOC_FAILURE_MSG \
 	ast_log(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file);
 
-void * attribute_malloc _ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func);
+void * attribute_malloc __ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func);
 
-void * attribute_malloc _ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func)
+void * attribute_malloc __ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func)
 {
 	void *p;
 
@@ -56,8 +56,6 @@ void * attribute_malloc _ast_calloc(size
 #endif
 
 #ifdef DEBUG_THREADS
-#if !defined(LOW_MEMORY)
-#ifdef HAVE_BKTR
 void ast_store_lock_info(enum ast_lock_type type, const char *filename,
 		        int line_num, const char *func, const char *lock_name, void *lock_addr, struct ast_bt *bt);
 void ast_store_lock_info(enum ast_lock_type type, const char *filename,
@@ -72,37 +70,17 @@ void ast_remove_lock_info(void *lock_add
     /* not a lot to do in a standalone w/o threading! */
 }
 
+#ifdef HAVE_BKTR
 int __ast_bt_get_addresses(struct ast_bt *bt);
 int __ast_bt_get_addresses(struct ast_bt *bt)
 {
 	/* Suck it, you stupid utils directory! */
 	return 0;
 }
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames);
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames)
-{
-	char **foo = calloc(num_frames, sizeof(char *) + 1);
-	if (foo) {
-		int i;
-		for (i = 0; i < num_frames; i++) {
-			foo[i] = (char *) foo + sizeof(char *) * num_frames;
-		}
-	}
-	return foo;
-}
-#else
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-		        int line_num, const char *func, const char *lock_name, void *lock_addr);
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-		        int line_num, const char *func, const char *lock_name, void *lock_addr)
-{
-    /* not a lot to do in a standalone w/o threading! */
-}
-
-void ast_remove_lock_info(void *lock_addr);
-void ast_remove_lock_info(void *lock_addr)
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames);
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames)
 {
-    /* not a lot to do in a standalone w/o threading! */
+	return NULL;
 }
 #endif /* HAVE_BKTR */
 
@@ -117,7 +95,6 @@ void ast_mark_lock_acquired(void *foo)
 {
     /* not a lot to do in a standalone w/o threading! */
 }
-#endif
 #endif /* DEBUG_THREADS */
 
 
@@ -152,8 +129,6 @@ void ast_log(int level, const char *file
 	fflush(stdout);
 	va_end(vars);
 }
-//void ast_register_file_version(const char *file, const char *version);
-//void ast_unregister_file_version(const char *file);
 
 char *find_var(const char *varname);
 void set_var(const char *varname, const char *varval);
@@ -161,23 +136,7 @@ unsigned int check_expr(char* buffer, ch
 int check_eval(char *buffer, char *error_report);
 void parse_file(const char *fname);
 
-void ast_register_file_version(const char *file, const char *version);
-void ast_register_file_version(const char *file, const char *version) { }
-#if !defined(LOW_MEMORY)
 int ast_add_profile(const char *x, uint64_t scale) { return 0;}
-#endif
-int ast_atomic_fetchadd_int_slow(volatile int *p, int v)
-{
-        int ret;
-        ret = *p;
-        *p += v;
-        return ret;
-}
-
-void ast_unregister_file_version(const char *file);
-void ast_unregister_file_version(const char *file)
-{
-}
 
 char *find_var(const char *varname) /* the list should be pretty short, if there's any list at all */
 {
diff -pruN 1:13.23.1~dfsg-2/utils/clicompat.c 1:16.1.1~dfsg-1/utils/clicompat.c
--- 1:13.23.1~dfsg-2/utils/clicompat.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/clicompat.c	2018-12-26 18:45:13.000000000 +0000
@@ -21,8 +21,8 @@ int ast_register_cleanup(void (*func)(vo
 	return 0;
 }
 
-int ast_cli_register_multiple(struct ast_cli_entry *e, int len);
-int ast_cli_register_multiple(struct ast_cli_entry *e, int len)
+int __ast_cli_register_multiple(struct ast_cli_entry *e, int len);
+int __ast_cli_register_multiple(struct ast_cli_entry *e, int len)
 {
 	return 0;
 }
diff -pruN 1:13.23.1~dfsg-2/utils/conf2ael.c 1:16.1.1~dfsg-1/utils/conf2ael.c
--- 1:13.23.1~dfsg-2/utils/conf2ael.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/conf2ael.c	2018-12-26 18:45:13.000000000 +0000
@@ -24,11 +24,11 @@
 
 /*** MODULEINFO
 	<depend>res_ael_share</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision$")
 
 #include "asterisk/paths.h"	/* CONFIG_DIR */
 #include <locale.h>
@@ -79,18 +79,7 @@ char ast_config_AST_SYSTEM_NAME[20] = ""
 int option_debug = 0;
 int option_verbose = 0;
 
-void ast_register_file_version(const char *file, const char *version);
-void ast_register_file_version(const char *file, const char *version)
-{
-}
-
-void ast_unregister_file_version(const char *file);
-void ast_unregister_file_version(const char *file)
-{
-}
-#if !defined(LOW_MEMORY)
 int ast_add_profile(const char *x, uint64_t scale) { return 0;}
-#endif
 /* Our own version of ast_log, since the expr parser uses it. -- stolen from utils/check_expr.c */
 void ast_log(int level, const char *file, int line, const char *function, const char *fmt, ...) __attribute__((format(printf,5,6)));
 
@@ -573,7 +562,7 @@ void pbx_substitute_variables_helper(str
 int ast_add_extension2(struct ast_context *con,
 					   int replace, const char *extension, int priority, const char *label, const char *callerid,
 					   const char *application, void *data, void (*datad)(void *),
-					   const char *registrar)
+					   const char *registrar, const char *registrar_file, int registrar_line)
 {
 	return localized_add_extension2(con, replace, extension, priority, label, callerid, application, data, datad, registrar);
 }
@@ -605,9 +594,9 @@ struct ast_context *ast_context_find_or_
 	return localized_context_find_or_create(extcontexts, exttable, name, registrar);
 }
 
-void ast_cli_register_multiple(void);
+void __ast_cli_register_multiple(void);
 
-void ast_cli_register_multiple(void)
+void __ast_cli_register_multiple(void)
 {
 }
 
@@ -664,20 +653,8 @@ struct ast_exten *ast_walk_context_exten
 	return NULL;
 }
 
-struct ast_include *ast_walk_context_includes(struct ast_context *con, struct ast_include *inc);
-struct ast_include *ast_walk_context_includes(struct ast_context *con, struct ast_include *inc)
-{
-	return NULL;
-}
-
-struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con, struct ast_ignorepat *ip);
-struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con, struct ast_ignorepat *ip)
-{
-	return NULL;
-}
-
-struct ast_sw *ast_walk_context_switches(struct ast_context *con, struct ast_sw *sw);
-struct ast_sw *ast_walk_context_switches(struct ast_context *con, struct ast_sw *sw)
+const struct ast_include *ast_walk_context_includes(const struct ast_context *con, const struct ast_include *inc);
+const struct ast_include *ast_walk_context_includes(const struct ast_context *con, const struct ast_include *inc)
 {
 	return NULL;
 }
@@ -720,11 +697,9 @@ unsigned int ast_hashtab_hash_contexts(c
 }
 
 #ifdef DEBUG_THREADS
-#if !defined(LOW_MEMORY)
 void ast_mark_lock_acquired(void *lock_addr)
 {
 }
-#ifdef HAVE_BKTR
 void ast_remove_lock_info(void *lock_addr, struct ast_bt *bt)
 {
 }
@@ -734,31 +709,15 @@ void ast_store_lock_info(enum ast_lock_t
 {
 }
 
+#ifdef HAVE_BKTR
 int __ast_bt_get_addresses(struct ast_bt *bt)
 {
 	return 0;
 }
 
-char **__ast_bt_get_symbols(void **addresses, size_t num_frames)
-{
-	char **foo = calloc(num_frames, sizeof(char *) + 1);
-	if (foo) {
-		int i;
-		for (i = 0; i < num_frames; i++) {
-			foo[i] = (char *) foo + sizeof(char *) * num_frames;
-		}
-	}
-	return foo;
-}
-
-#else
-void ast_remove_lock_info(void *lock_addr)
-{
-}
-
-void ast_store_lock_info(enum ast_lock_type type, const char *filename,
-	int line_num, const char *func, const char *lock_name, void *lock_addr)
+struct ast_vector_string *__ast_bt_get_symbols(void **addresses, size_t num_frames)
 {
+	return NULL;
 }
 #endif /* HAVE_BKTR */
 void ast_suspend_lock_info(void *lock_addr)
@@ -767,5 +726,4 @@ void ast_suspend_lock_info(void *lock_ad
 void ast_restore_lock_info(void *lock_addr)
 {
 }
-#endif /* !defined(LOW_MEMORY) */
 #endif /* DEBUG_THREADS */
diff -pruN 1:13.23.1~dfsg-2/utils/conf_bridge_binaural_hrir_importer.c 1:16.1.1~dfsg-1/utils/conf_bridge_binaural_hrir_importer.c
--- 1:13.23.1~dfsg-2/utils/conf_bridge_binaural_hrir_importer.c	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/conf_bridge_binaural_hrir_importer.c	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,147 @@
+/*
+ * Asterisk -- An open source telephony toolkit.
+ *
+ * Copyright (C) 2016, Frank Haase, Dennis Guse
+ *
+ * Frank Haase <fra.haase@gmail.com>
+ * Dennis Guse <dennis.guse@alumni.tu-berlin.de>
+ *
+ * See http://www.asterisk.org for more information about
+ * the Asterisk project. Please do not directly contact
+ * any of the maintainers of this project for assistance;
+ * the project provides a web site, mailing lists and IRC
+ * channels for your use.
+ *
+ * This program is free software, distributed under the terms of
+ * the GNU General Public License Version 2. See the LICENSE file
+ * at the top of the source tree.
+ */
+
+/*!
+ * \file
+ * Converts a Head Related Impulse Response (HRIR) database (a multi-channel wave) into a C header file.
+ * HRIR for the left ear and HRIR for right ear have to be interleaved.
+ * No further signal processing is applied (e.g., resampling).
+ *
+ * Info messages are printed to stderror and the generated header file to output.
+ */
+
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <sndfile.h>
+#include "conf_bridge_binaural_hrir_importer.h"
+
+int main (int argc, char **argv)
+{
+	char *hrir_filename;
+	unsigned int binaural_index_start;
+	unsigned int binaural_index_end;
+
+	SNDFILE *hrir_file;
+	SF_INFO hrir_info;
+	float *hrir_data;
+
+	unsigned int impulse_response_index_start;
+	unsigned int impulse_response_index_end;
+
+	int j;
+	int ir_current;
+
+	if(argc != 4) {
+		puts("HRIR database to C header file converter.");
+		puts("Usage: conf_bridge_binaural_hrir_importer HRIR.wav INDEX_START INDEX_END > OUTPUT.h");
+		puts("Example: conf_bridge_binaural_hrir_importer hrirs.wav 0 180 > ../bridges/bridge_softmix/include/hrirs.h");
+
+		return -1;
+	}
+
+	/* Parse arguments */
+	hrir_filename = argv[1];
+	binaural_index_start = atoi(argv[2]);
+	binaural_index_end = atoi(argv[3]);
+
+	/* Read HRIR database */
+	hrir_file = sf_open(hrir_filename, SFM_READ, &hrir_info);
+	if(hrir_file == NULL) {
+		fprintf(stderr, "ERROR: Could not open HRIR database (%s).\n", hrir_filename);
+
+		return -1;
+	}
+	fprintf(stderr, "INFO: Opened HRIR database (%s) with: number channels: %d; samplerate: %d; samples per channel: %ld\n", hrir_filename, hrir_info.channels, hrir_info.samplerate, hrir_info.frames);
+
+	hrir_data = (float *)malloc(hrir_info.channels * hrir_info.frames * sizeof(float));
+	if(hrir_data == NULL) {
+		fprintf(stderr, "ERROR: Out of memory!");
+
+		return -1;
+	}
+
+	/* Channels are interleaved */
+	sf_read_float(hrir_file, hrir_data, hrir_info.channels * hrir_info.frames);
+	sf_close(hrir_file);
+
+	if(binaural_index_start >= binaural_index_end) {
+		fprintf(stderr, "ERROR: INDEX_START (%d) must be smaller than INDEX_END (%d).", binaural_index_start, binaural_index_end);
+		free(hrir_data);
+
+		return -1;
+	}
+
+	if (binaural_index_end * 2 >= hrir_info.channels) {
+		fprintf(stderr, "ERROR: END_INDEX (%d) is out of range for HRIR database (%s).\n", binaural_index_end, hrir_filename);
+		free(hrir_data);
+
+		return -1;
+	}
+
+	/* Convert indices */
+	impulse_response_index_start = 2 * binaural_index_start;
+	impulse_response_index_end = (binaural_index_end + 1) * 2;
+
+	/* Write header */
+	printf(FILE_HEADER, hrir_filename, binaural_index_start, binaural_index_end);
+
+	printf("#define HRIRS_IMPULSE_LEN %ld\n", hrir_info.frames);
+	printf("#define HRIRS_IMPULSE_SIZE %d\n", binaural_index_end - binaural_index_start + 1);
+	printf("#define HRIRS_SAMPLE_RATE %d\n\n", hrir_info.samplerate);
+
+	printf("float hrirs_left[HRIRS_IMPULSE_SIZE][HRIRS_IMPULSE_LEN] = {\n");
+	for (ir_current = impulse_response_index_start; ir_current < impulse_response_index_end; ir_current += 2) {
+		printf("{");
+
+		for (j = 0; j < hrir_info.frames - 1; j++) {
+			printf("%.16f,%s", hrir_data[ir_current * hrir_info.frames + j], ((j + 1) % 4 ? " " : "\n"));
+		}
+		/* Write last without trailing "," */
+		printf("%.16f", hrir_data[ir_current * hrir_info.frames + hrir_info.frames - 1]);
+
+		if (ir_current + 2 < impulse_response_index_end) {
+			printf("},\n");
+		}	else {
+			printf("}};");
+		}
+	}
+
+	printf("\nfloat hrirs_right[HRIRS_IMPULSE_SIZE][HRIRS_IMPULSE_LEN] = {\n");
+	for (ir_current = impulse_response_index_start + 1; ir_current < impulse_response_index_end + 1; ir_current += 2) {
+		printf("{");
+
+		for (j = 0; j < hrir_info.frames - 1; j++) {
+			printf("%.16f,%s", hrir_data[ir_current * hrir_info.frames + j], ((j + 1) % 4 ? " " : "\n"));
+		}
+		 /* Write last without trailing "," */
+		printf("%.16f", hrir_data[ir_current * hrir_info.frames + hrir_info.frames - 1]);
+
+		if (ir_current + 2 < impulse_response_index_end) {
+			printf("},\n");
+		}	else {
+			printf("}};");
+		}
+	}
+
+	fprintf(stderr, "INFO: Successfully converted: imported %d impulse responses.\n", impulse_response_index_end - impulse_response_index_start);
+	free(hrir_data);
+
+	return 0;
+}
diff -pruN 1:13.23.1~dfsg-2/utils/conf_bridge_binaural_hrir_importer.h 1:16.1.1~dfsg-1/utils/conf_bridge_binaural_hrir_importer.h
--- 1:13.23.1~dfsg-2/utils/conf_bridge_binaural_hrir_importer.h	1970-01-01 00:00:00.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/conf_bridge_binaural_hrir_importer.h	2018-12-26 18:45:13.000000000 +0000
@@ -0,0 +1,46 @@
+#define FILE_HEADER "\
+ * Asterisk -- An open source telephony toolkit.\n\
+ *\n\
+ * Copyright (C) 2016, Frank Haase, Dennis Guse\n\
+ *\n\
+ * Frank Haase <fra.haase@gmail.com>\n\
+ * Dennis Guse <dennis.guse@alumni.tu-berlin.de>\n\
+ *\n\
+ * See http://www.asterisk.org for more information about\n\
+ * the Asterisk project. Please do not directly contact\n\
+ * any of the maintainers of this project for assistance;\n\
+ * the project provides a web site, mailing lists and IRC\n\
+ * channels for your use.\n\
+ *\n\
+ * Copyright (c) 2001 The Regents of the University of California. All Rights Reserved.\n\
+ *\n\
+ * The HRIRs used here are obtained from The CIPIC HRTF Database\n\
+ * (http://interface.cipic.ucdavis.edu/CIL_html/CIL_HRTF_database.htm)\n\
+ * Note that the above mentioned material is Copyright (c) 2001 The\n\
+ * Regents of the University of California. All Rights Reserved.\n\
+ *\n\
+ * Download the file\n\
+ * http://interface.cipic.ucdavis.edu/data/special_kemar_hrir.tar and\n\
+ * uncompress it in the folder where this Matlab script resides. Finally,\n\
+ * run the script.\n\
+ *\n\
+ * This program is free software, distributed under the terms of\n\
+ * the GNU General Public License Version 2. See the LICENSE file\n\
+ * at the top of the source tree.\n\
+ */\n\
+\n\
+/*! \\file\n\
+ *\n\
+ * \\brief Multi-party software binaural channel HRIRS\n\
+ *\n\
+ * \\author Frank Haase <fra.haase@googlemail.com>\n\
+ * \\author Dennis Guse <dennis.guse@alumni.tu-berlin.de>\n\
+ *\n\
+ * \\ingroup bridges\n\
+ */\n\
+\n\
+/*\n\
+ * This file was created with command:\n\
+ * $ conf_bridge_binaural_hrir_importer %s %d %d\n\
+ */\n\
+"
diff -pruN 1:13.23.1~dfsg-2/utils/extconf.c 1:16.1.1~dfsg-1/utils/extconf.c
--- 1:13.23.1~dfsg-2/utils/extconf.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/extconf.c	2018-12-26 18:45:13.000000000 +0000
@@ -18,7 +18,7 @@
 
 
 /*!
- * \file extconf
+ * \file
  * A condensation of the pbx_config stuff, to read into exensions.conf, and provide an interface to the data there,
  * for operations outside of asterisk. A huge, awful hack.
  *
@@ -43,6 +43,7 @@
 	<support_level>extended</support_level>
  ***/
 
+#define ASTMM_LIBC ASTMM_IGNORE
 #include "asterisk.h"
 
 #undef DEBUG_THREADS
@@ -74,16 +75,6 @@
 static void ast_log(int level, const char *file, int line, const char *function, const char *fmt, ...) __attribute__((format(printf, 5, 6)));
 void ast_verbose(const char *fmt, ...) __attribute__((format(printf, 1, 2)));
 
-#define ASINCLUDE_GLOB 1
-#ifdef AST_INCLUDE_GLOB
-
-#if !defined(GLOB_ABORTED)
-#define GLOB_ABORTED GLOB_ABEND
-#endif
-
-# include <glob.h>
-#endif
-
 #define AST_API_MODULE  1 /* gimme the inline defs! */
 struct ast_channel
 {
@@ -119,9 +110,6 @@ void ast_queue_log(const char *queuename
 /* IN CONFLICT: void ast_verbose(const char *fmt, ...)
    __attribute__((format(printf, 1, 2))); */
 
-int ast_register_verbose(void (*verboser)(const char *string));
-int ast_unregister_verbose(void (*verboser)(const char *string));
-
 void ast_console_puts(const char *string);
 
 #define _A_ __FILE__, __LINE__, __PRETTY_FUNCTION__
@@ -208,7 +196,7 @@ int mtx_prof = -1;
 
 #ifdef DEBUG_THREADS
 
-#define __ast_mutex_logger(...)  do { if (canlog) ast_log(LOG_ERROR, __VA_ARGS__); else fprintf(stderr, __VA_ARGS__); } while (0)
+#define log_mutex_error(canlog, ...)  do { if (canlog) ast_log(LOG_ERROR, __VA_ARGS__); else fprintf(stderr, __VA_ARGS__); } while (0)
 
 #ifdef THREAD_CRASH
 #define DO_THREAD_CRASH do { *((int *)(0)) = 1; } while(0)
@@ -251,9 +239,9 @@ static inline int __ast_pthread_mutex_in
 
 	if ((t->mutex) != ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
 		if ((t->mutex) != (empty_mutex)) {
-			__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is already initialized.\n",
+			log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is already initialized.\n",
 					   filename, lineno, func, mutex_name);
-			__ast_mutex_logger("%s line %d (%s): Error: previously initialization of mutex '%s'.\n",
+			log_mutex_error(canlog, "%s line %d (%s): Error: previously initialization of mutex '%s'.\n",
 					   t->file[0], t->lineno[0], t->func[0], mutex_name);
 			DO_THREAD_CRASH;
 			return 0;
@@ -290,7 +278,7 @@ static inline int __ast_pthread_mutex_de
 
 #ifdef AST_MUTEX_INIT_W_CONSTRUCTORS
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 	}
 #endif
@@ -301,19 +289,19 @@ static inline int __ast_pthread_mutex_de
 		pthread_mutex_unlock(&t->mutex);
 		break;
 	case EINVAL:
-		__ast_mutex_logger("%s line %d (%s): Error: attempt to destroy invalid mutex '%s'.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: attempt to destroy invalid mutex '%s'.\n",
 				  filename, lineno, func, mutex_name);
 		break;
 	case EBUSY:
-		__ast_mutex_logger("%s line %d (%s): Error: attempt to destroy locked mutex '%s'.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: attempt to destroy locked mutex '%s'.\n",
 				   filename, lineno, func, mutex_name);
-		__ast_mutex_logger("%s line %d (%s): Error: '%s' was locked here.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: '%s' was locked here.\n",
 				   t->file[t->reentrancy-1], t->lineno[t->reentrancy-1], t->func[t->reentrancy-1], mutex_name);
 		break;
 	}
 
 	if ((res = pthread_mutex_destroy(&t->mutex)))
-		__ast_mutex_logger("%s line %d (%s): Error destroying mutex: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error destroying mutex: %s\n",
 				   filename, lineno, func, strerror(res));
 #ifndef PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
 	else
@@ -334,7 +322,7 @@ static inline int __ast_pthread_mutex_lo
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				 filename, lineno, func, mutex_name);
 		ast_mutex_init(t);
 	}
@@ -355,9 +343,9 @@ static inline int __ast_pthread_mutex_lo
 			if (res == EBUSY) {
 				current = time(NULL);
 				if ((current - seconds) && (!((current - seconds) % 5))) {
-					__ast_mutex_logger("%s line %d (%s): Deadlock? waited %d sec for mutex '%s'?\n",
+					log_mutex_error(canlog, "%s line %d (%s): Deadlock? waited %d sec for mutex '%s'?\n",
 							   filename, lineno, func, (int)(current - seconds), mutex_name);
-					__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+					log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 							   t->file[t->reentrancy-1], t->lineno[t->reentrancy-1],
 							   t->func[t->reentrancy-1], mutex_name);
 				}
@@ -383,11 +371,11 @@ static inline int __ast_pthread_mutex_lo
 			t->thread[t->reentrancy] = pthread_self();
 			t->reentrancy++;
 		} else {
-			__ast_mutex_logger("%s line %d (%s): '%s' really deep reentrancy!\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' really deep reentrancy!\n",
 							   filename, lineno, func, mutex_name);
 		}
 	} else {
-		__ast_mutex_logger("%s line %d (%s): Error obtaining mutex: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error obtaining mutex: %s\n",
 				   filename, lineno, func, strerror(errno));
 		DO_THREAD_CRASH;
 	}
@@ -403,7 +391,7 @@ static inline int __ast_pthread_mutex_tr
 
 #if defined(AST_MUTEX_INIT_W_CONSTRUCTORS)
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 		ast_mutex_init(t);
 	}
@@ -417,11 +405,11 @@ static inline int __ast_pthread_mutex_tr
 			t->thread[t->reentrancy] = pthread_self();
 			t->reentrancy++;
 		} else {
-			__ast_mutex_logger("%s line %d (%s): '%s' really deep reentrancy!\n",
+			log_mutex_error(canlog, "%s line %d (%s): '%s' really deep reentrancy!\n",
 					   filename, lineno, func, mutex_name);
 		}
 	} else {
-		__ast_mutex_logger("%s line %d (%s): Warning: '%s' was locked here.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Warning: '%s' was locked here.\n",
                                    t->file[t->reentrancy-1], t->lineno[t->reentrancy-1], t->func[t->reentrancy-1], mutex_name);
 	}
 
@@ -436,21 +424,21 @@ static inline int __ast_pthread_mutex_un
 
 #ifdef AST_MUTEX_INIT_W_CONSTRUCTORS
 	if ((t->mutex) == ((pthread_mutex_t) PTHREAD_MUTEX_INITIALIZER)) {
-		__ast_mutex_logger("%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error: mutex '%s' is uninitialized.\n",
 				   filename, lineno, func, mutex_name);
 	}
 #endif
 
 	if (t->reentrancy && (t->thread[t->reentrancy-1] != pthread_self())) {
-		__ast_mutex_logger("%s line %d (%s): attempted unlock mutex '%s' without owning it!\n",
+		log_mutex_error(canlog, "%s line %d (%s): attempted unlock mutex '%s' without owning it!\n",
 				   filename, lineno, func, mutex_name);
-		__ast_mutex_logger("%s line %d (%s): '%s' was locked here.\n",
+		log_mutex_error(canlog, "%s line %d (%s): '%s' was locked here.\n",
 				   t->file[t->reentrancy-1], t->lineno[t->reentrancy-1], t->func[t->reentrancy-1], mutex_name);
 		DO_THREAD_CRASH;
 	}
 
 	if (--t->reentrancy < 0) {
-		__ast_mutex_logger("%s line %d (%s): mutex '%s' freed more times than we've locked!\n",
+		log_mutex_error(canlog, "%s line %d (%s): mutex '%s' freed more times than we've locked!\n",
 				   filename, lineno, func, mutex_name);
 		t->reentrancy = 0;
 	}
@@ -463,7 +451,7 @@ static inline int __ast_pthread_mutex_un
 	}
 
 	if ((res = pthread_mutex_unlock(&t->mutex))) {
-		__ast_mutex_logger("%s line %d (%s): Error releasing mutex: %s\n",
+		log_mutex_error(canlog, "%s line %d (%s): Error releasing mutex: %s\n",
 				   filename, lineno, func, strerror(res));
 		DO_THREAD_CRASH;
 	}
@@ -683,44 +671,6 @@ int ast_channel_trylock(struct ast_chann
 
 /* from utils.h */
 
-#define ast_free free
-#define ast_free_ptr free
-
-#define MALLOC_FAILURE_MSG \
-	ast_log(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file);
-
-/*!
- * \brief A wrapper for malloc()
- *
- * ast_malloc() is a wrapper for malloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The argument and return value are the same as malloc()
- */
-#define ast_malloc(len) \
-	_ast_malloc((len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_calloc(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_calloc_cache(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_realloc(p, len) \
-	_ast_realloc((p), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_strdup(str) \
-	_ast_strdup((str), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_strndup(str, len) \
-	_ast_strndup((str), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-#define ast_asprintf(ret, fmt, ...) \
-	_ast_asprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt, __VA_ARGS__)
-
-#define ast_vasprintf(ret, fmt, ap) \
-	_ast_vasprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, (fmt), (ap))
-
 struct ast_flags {  /* stolen from utils.h */
 	unsigned int flags;
 };
@@ -741,221 +691,6 @@ struct ast_flags {  /* stolen from utils
 						(p)->flags &= ~(flag); \
 					} while (0)
 
-
-
-#define MALLOC_FAILURE_MSG \
-	ast_log(LOG_ERROR, "Memory Allocation Failure in function %s at line %d of %s\n", func, lineno, file);
-/*!
- * \brief A wrapper for malloc()
- *
- * ast_malloc() is a wrapper for malloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The argument and return value are the same as malloc()
- */
-#define ast_malloc(len) \
-	_ast_malloc((len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void * attribute_malloc _ast_malloc(size_t len, const char *file, int lineno, const char *func),
-{
-	void *p;
-
-	if (!(p = malloc(len)))
-		MALLOC_FAILURE_MSG;
-
-	return p;
-}
-)
-
-/*!
- * \brief A wrapper for calloc()
- *
- * ast_calloc() is a wrapper for calloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as calloc()
- */
-#define ast_calloc(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void * attribute_malloc _ast_calloc(size_t num, size_t len, const char *file, int lineno, const char *func),
-{
-	void *p;
-
-	if (!(p = calloc(num, len)))
-		MALLOC_FAILURE_MSG;
-
-	return p;
-}
-)
-
-/*!
- * \brief A wrapper for calloc() for use in cache pools
- *
- * ast_calloc_cache() is a wrapper for calloc() that will generate an Asterisk log
- * message in the case that the allocation fails. When memory debugging is in use,
- * the memory allocated by this function will be marked as 'cache' so it can be
- * distinguished from normal memory allocations.
- *
- * The arguments and return value are the same as calloc()
- */
-#define ast_calloc_cache(num, len) \
-	_ast_calloc((num), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-/*!
- * \brief A wrapper for realloc()
- *
- * ast_realloc() is a wrapper for realloc() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as realloc()
- */
-#define ast_realloc(p, len) \
-	_ast_realloc((p), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-void *_ast_realloc(void *p, size_t len, const char *file, int lineno, const char *func),
-{
-	void *newp;
-
-	if (!(newp = realloc(p, len)))
-		MALLOC_FAILURE_MSG;
-
-	return newp;
-}
-)
-
-/*!
- * \brief A wrapper for strdup()
- *
- * ast_strdup() is a wrapper for strdup() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * ast_strdup(), unlike strdup(), can safely accept a NULL argument. If a NULL
- * argument is provided, ast_strdup will return NULL without generating any
- * kind of error log message.
- *
- * The argument and return value are the same as strdup()
- */
-#define ast_strdup(str) \
-	_ast_strdup((str), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-char * attribute_malloc _ast_strdup(const char *str, const char *file, int lineno, const char *func),
-{
-	char *newstr = NULL;
-
-	if (str) {
-		if (!(newstr = strdup(str)))
-			MALLOC_FAILURE_MSG;
-	}
-
-	return newstr;
-}
-)
-
-/*!
- * \brief A wrapper for strndup()
- *
- * ast_strndup() is a wrapper for strndup() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * ast_strndup(), unlike strndup(), can safely accept a NULL argument for the
- * string to duplicate. If a NULL argument is provided, ast_strdup will return
- * NULL without generating any kind of error log message.
- *
- * The arguments and return value are the same as strndup()
- */
-#define ast_strndup(str, len) \
-	_ast_strndup((str), (len), __FILE__, __LINE__, __PRETTY_FUNCTION__)
-
-AST_INLINE_API(
-char * attribute_malloc _ast_strndup(const char *str, size_t len, const char *file, int lineno, const char *func),
-{
-	char *newstr = NULL;
-
-	if (str) {
-		if (!(newstr = strndup(str, len)))
-			MALLOC_FAILURE_MSG;
-	}
-
-	return newstr;
-}
-)
-
-/*!
- * \brief A wrapper for asprintf()
- *
- * ast_asprintf() is a wrapper for asprintf() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as asprintf()
- */
-#define ast_asprintf(ret, fmt, ...) \
-	_ast_asprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, fmt, __VA_ARGS__)
-
-AST_INLINE_API(
-__attribute__((format(printf, 5, 6)))
-int _ast_asprintf(char **ret, const char *file, int lineno, const char *func, const char *fmt, ...),
-{
-	int res;
-	va_list ap;
-
-	va_start(ap, fmt);
-	if ((res = vasprintf(ret, fmt, ap)) == -1)
-		MALLOC_FAILURE_MSG;
-	va_end(ap);
-
-	return res;
-}
-)
-
-/*!
- * \brief A wrapper for vasprintf()
- *
- * ast_vasprintf() is a wrapper for vasprintf() that will generate an Asterisk log
- * message in the case that the allocation fails.
- *
- * The arguments and return value are the same as vasprintf()
- */
-#define ast_vasprintf(ret, fmt, ap) \
-	_ast_vasprintf((ret), __FILE__, __LINE__, __PRETTY_FUNCTION__, (fmt), (ap))
-
-AST_INLINE_API(
-__attribute__((format(printf, 5, 0)))
-int _ast_vasprintf(char **ret, const char *file, int lineno, const char *func, const char *fmt, va_list ap),
-{
-	int res;
-
-	if ((res = vasprintf(ret, fmt, ap)) == -1)
-		MALLOC_FAILURE_MSG;
-
-	return res;
-}
-)
-
-#if !defined(ast_strdupa) && defined(__GNUC__)
-/*!
-  \brief duplicate a string in memory from the stack
-  \param s The string to duplicate
-
-  This macro will duplicate the given string.  It returns a pointer to the stack
-  allocatted memory for the new string.
-*/
-#define ast_strdupa(s)                                                    \
-	(__extension__                                                    \
-	({                                                                \
-		const char *__old = (s);                                  \
-		size_t __len = strlen(__old) + 1;                         \
-		char *__new = __builtin_alloca(__len);                    \
-		memcpy (__new, __old, __len);                             \
-		__new;                                                    \
-	}))
-#endif
-
-
 /* from config.c */
 
 #define MAX_NESTED_COMMENTS 128
@@ -1606,20 +1341,14 @@ enum ast_option_flags {
 	AST_OPT_FLAG_TRANSMIT_SILENCE = (1 << 17),
 	/*! Suppress some warnings */
 	AST_OPT_FLAG_DONT_WARN = (1 << 18),
-	/*! End CDRs before the 'h' extension */
-	AST_OPT_FLAG_END_CDR_BEFORE_H_EXTEN = (1 << 19),
 	/*! Always fork, even if verbose or debug settings are non-zero */
 	AST_OPT_FLAG_ALWAYS_FORK = (1 << 21),
 	/*! Disable log/verbose output to remote consoles */
 	AST_OPT_FLAG_MUTE = (1 << 22),
 	/*! There is a per-file debug setting */
 	AST_OPT_FLAG_DEBUG_FILE = (1 << 23),
-	/*! There is a per-file verbose setting */
-	AST_OPT_FLAG_VERBOSE_FILE = (1 << 24),
 	/*! Terminal colors should be adjusted for a light-colored background */
 	AST_OPT_FLAG_LIGHT_BACKGROUND = (1 << 25),
-	/*! Count Initiated seconds in CDR's */
-	AST_OPT_FLAG_INITIATED_SECONDS = (1 << 26),
 	/*! Force black background */
 	AST_OPT_FLAG_FORCE_BLACK_BACKGROUND = (1 << 27),
 };
@@ -1652,7 +1381,6 @@ struct ast_flags ast_options = { AST_DEF
 #define ast_opt_reconnect		ast_test_flag(&ast_options, AST_OPT_FLAG_RECONNECT)
 #define ast_opt_transmit_silence	ast_test_flag(&ast_options, AST_OPT_FLAG_TRANSMIT_SILENCE)
 #define ast_opt_dont_warn		ast_test_flag(&ast_options, AST_OPT_FLAG_DONT_WARN)
-#define ast_opt_end_cdr_before_h_exten	ast_test_flag(&ast_options, AST_OPT_FLAG_END_CDR_BEFORE_H_EXTEN)
 #define ast_opt_always_fork		ast_test_flag(&ast_options, AST_OPT_FLAG_ALWAYS_FORK)
 #define ast_opt_mute			ast_test_flag(&ast_options, AST_OPT_FLAG_MUTE)
 
@@ -3415,29 +3143,6 @@ static struct ast_config *config_text_fi
 		CB_INIT();
 	}
 
-#ifdef AST_INCLUDE_GLOB
-	{
-		int glob_ret;
-		glob_t globbuf;
-
-		globbuf.gl_offs = 0;	/* initialize it to silence gcc */
-#ifdef SOLARIS
-		glob_ret = glob(fn, GLOB_NOCHECK, NULL, &globbuf);
-#else
-		glob_ret = glob(fn, GLOB_NOMAGIC|GLOB_BRACE, NULL, &globbuf);
-#endif
-		if (glob_ret == GLOB_NOSPACE)
-			ast_log(LOG_WARNING,
-				"Glob Expansion of pattern '%s' failed: Not enough memory\n", fn);
-		else if (glob_ret  == GLOB_ABORTED)
-			ast_log(LOG_WARNING,
-				"Glob Expansion of pattern '%s' failed: Read error\n", fn);
-		else  {
-			/* loop over expanded files */
-			int i;
-			for (i=0; i<globbuf.gl_pathc; i++) {
-				ast_copy_string(fn, globbuf.gl_pathv[i], sizeof(fn));
-#endif
 	do {
 		if (stat(fn, &statbuf))
 			continue;
@@ -3547,14 +3252,6 @@ static struct ast_config *config_text_fi
 	if (comment) {
 		ast_log(LOG_WARNING,"Unterminated comment detected beginning on line %d\n", nest[comment]);
 	}
-#ifdef AST_INCLUDE_GLOB
-					if (!cfg)
-						break;
-				}
-				globfree(&globbuf);
-			}
-		}
-#endif
 	if (cfg && cfg->include_level == 1 && withcomments && comment_buffer) {
 		if (comment_buffer) {
 			free(comment_buffer);
@@ -4389,6 +4086,19 @@ static struct ast_include *ast_walk_cont
 		return inc->next;
 }
 
+int ast_context_includes_count(struct ast_context *con);
+int ast_context_includes_count(struct ast_context *con)
+{
+	int c = 0;
+	struct ast_include *inc = NULL;
+
+	while ((inc = ast_walk_context_includes(con, inc))) {
+		c++;
+	}
+
+	return c;
+}
+
 struct ast_include *localized_walk_context_includes(struct ast_context *con,
 													struct ast_include *inc);
 struct ast_include *localized_walk_context_includes(struct ast_context *con,
@@ -4397,6 +4107,31 @@ struct ast_include *localized_walk_conte
 	return ast_walk_context_includes(con, inc);
 }
 
+static struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con,
+	struct ast_ignorepat *ip);
+
+static struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con,
+	struct ast_ignorepat *ip)
+{
+	if (!ip)
+		return con ? con->ignorepats : NULL;
+	else
+		return ip->next;
+}
+
+int ast_context_ignorepats_count(struct ast_context *con);
+int ast_context_ignorepats_count(struct ast_context *con)
+{
+	int c = 0;
+	struct ast_ignorepat *ip = NULL;
+
+	while ((ip = ast_walk_context_ignorepats(con, ip))) {
+		c++;
+	}
+
+	return c;
+}
+
 
 static struct ast_sw *ast_walk_context_switches(struct ast_context *con,
 													 struct ast_sw *sw);
@@ -4418,6 +4153,19 @@ struct ast_sw *localized_walk_context_sw
 	return ast_walk_context_switches(con, sw);
 }
 
+int ast_context_switches_count(struct ast_context *con);
+int ast_context_switches_count(struct ast_context *con)
+{
+	int c = 0;
+	struct ast_sw *sw = NULL;
+
+	while ((sw = ast_walk_context_switches(con, sw))) {
+		c++;
+	}
+
+	return c;
+}
+
 
 static struct ast_context *ast_context_find(const char *name);
 
diff -pruN 1:13.23.1~dfsg-2/utils/.gitignore 1:16.1.1~dfsg-1/utils/.gitignore
--- 1:13.23.1~dfsg-2/utils/.gitignore	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/.gitignore	2018-12-26 18:45:13.000000000 +0000
@@ -24,3 +24,4 @@ stereorize
 strcompat.c
 streamplayer
 threadstorage.c
+conf_bridge_binaural_hrir_importer
diff -pruN 1:13.23.1~dfsg-2/utils/Makefile 1:16.1.1~dfsg-1/utils/Makefile
--- 1:13.23.1~dfsg-2/utils/Makefile	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/Makefile	2018-12-26 18:45:13.000000000 +0000
@@ -86,24 +86,28 @@ clean:
 	rm -f *.o $(ALL_UTILS) check_expr
 	rm -f .*.d
 	rm -f *.s *.i
-	rm -f md5.c strcompat.c ast_expr2.c ast_expr2.h ast_expr2f.c pbx_ael.c pval.c hashtab.c lock.c
+	rm -f astmm.c md5.c strcompat.c ast_expr2.c ast_expr2.h ast_expr2f.c pbx_ael.c pval.c hashtab.c lock.c
 	rm -f aelparse.c aelbison.c conf2ael
 	rm -f threadstorage.c
 	rm -f utils.c strings.c poll.c version.c sha1.c astobj2.c refcounter
 	rm -f db1-ast/.*.d
 	@$(MAKE) -C db1-ast clean
 
+
 md5.c: $(ASTTOPDIR)/main/md5.c
 	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
 	$(CMD_PREFIX) cp "$<" "$@"
 
 astman: astman.o md5.o
 astman: LIBS+=$(NEWT_LIB)
-astman.o: _ASTCFLAGS+=-DNO_MALLOC_DEBUG
 
 stereorize: stereorize.o frame.o
 stereorize: LIBS+=-lm
 
+astmm.c: $(ASTTOPDIR)/main/astmm.c
+	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
+	$(CMD_PREFIX) cp "$<" "$@"
+
 hashtab.c: $(ASTTOPDIR)/main/hashtab.c
 	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
 	$(CMD_PREFIX) cp "$<" "$@"
@@ -133,7 +137,7 @@ ast_expr2f.c: $(ASTTOPDIR)/main/ast_expr
 	$(CMD_PREFIX) cp "$<" "$@"
 ast_expr2f.o: _ASTCFLAGS+=-I$(ASTTOPDIR)/main -Wno-unused
 
-check_expr: check_expr.o ast_expr2.o ast_expr2f.o strcompat.o threadstorage.o clicompat.o
+check_expr: check_expr.o ast_expr2.o ast_expr2f.o strcompat.o threadstorage.o clicompat.o astmm.o
 
 aelbison.c: $(ASTTOPDIR)/res/ael/ael.tab.c
 	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
@@ -150,6 +154,8 @@ pbx_ael.c: $(ASTTOPDIR)/pbx/pbx_ael.c
 	$(CMD_PREFIX) sed 's/ast_debug([[:digit:]][[:digit:]]*/ast_log(LOG_DEBUG/' "$@" > "$@.new"
 	$(CMD_PREFIX) mv "$@.new" "$@"
 
+pbx_ael.o: _ASTCFLAGS+=-DAST_MODULE_SELF_SYM=__internal_pbx_ael_self
+
 aelparse.c: $(ASTTOPDIR)/res/ael/ael_lex.c
 	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
 	$(CMD_PREFIX) cp "$<" "$@"
@@ -159,25 +165,26 @@ aelparse.c: $(ASTTOPDIR)/res/ael/ael_lex
 
 aelparse.o: _ASTCFLAGS+=-I$(ASTTOPDIR)/res -Wno-unused $(AST_NO_FORMAT_TRUNCATION)
 aelparse: LIBS+=-lm $(AST_CLANG_BLOCKS_LIBS)
-aelparse: aelparse.o aelbison.o pbx_ael.o hashtab.o lock.o ael_main.o ast_expr2f.o ast_expr2.o strcompat.o pval.o extconf.o
+aelparse: aelparse.o aelbison.o pbx_ael.o hashtab.o lock.o ael_main.o ast_expr2f.o ast_expr2.o strcompat.o pval.o extconf.o astmm.o
 
 threadstorage.c: $(ASTTOPDIR)/main/threadstorage.c
 	$(ECHO_PREFIX) echo "   [CP] $(subst $(ASTTOPDIR)/,,$<) -> $@"
 	$(CMD_PREFIX) cp "$<" "$@"
 
 
+extconf.o: _ASTCFLAGS+=$(call get_menuselect_cflags,DETECT_DEADLOCKS)
 extconf.o: extconf.c
 
 conf2ael: LIBS+=$(AST_CLANG_BLOCKS_LIBS)
-conf2ael: conf2ael.o ast_expr2f.o ast_expr2.o hashtab.o lock.o aelbison.o aelparse.o pbx_ael.o pval.o extconf.o strcompat.o
+conf2ael: conf2ael.o ast_expr2f.o ast_expr2.o hashtab.o lock.o aelbison.o aelparse.o pbx_ael.o pval.o extconf.o strcompat.o astmm.o
 
-check_expr2: $(ASTTOPDIR)/main/ast_expr2f.c $(ASTTOPDIR)/main/ast_expr2.c $(ASTTOPDIR)/main/ast_expr2.h
+check_expr2: $(ASTTOPDIR)/main/ast_expr2f.c $(ASTTOPDIR)/main/ast_expr2.c $(ASTTOPDIR)/main/ast_expr2.h astmm.o
 	$(ECHO_PREFIX) echo "   [CC] ast_expr2f.c -> ast_expr2fz.o"
 	$(CC) -g -c -I$(ASTTOPDIR)/include -DSTANDALONE $(ASTTOPDIR)/main/ast_expr2f.c -o ast_expr2fz.o
 	$(ECHO_PREFIX) echo "   [CC] ast_expr2.c -> ast_expr2z.o"
 	$(CC) -g -c -I$(ASTTOPDIR)/include -DSTANDALONE2 $(ASTTOPDIR)/main/ast_expr2.c -o ast_expr2z.o
 	$(ECHO_PREFIX) echo "   [LD] ast_expr2fz.o ast_expr2z.o  -> check_expr2"
-	$(CC) -g -o check_expr2 ast_expr2fz.o ast_expr2z.o -lm $(_ASTLDFLAGS)
+	$(CC) -g -o check_expr2 ast_expr2fz.o ast_expr2z.o astmm.o -lm $(_ASTLDFLAGS)
 	$(ECHO_PREFIX) echo "   [RM] ast_expr2fz.o ast_expr2z.o"
 	rm ast_expr2z.o ast_expr2fz.o
 	./check_expr2 expr2.testinput
@@ -203,6 +210,9 @@ astdb2bdb: LIBS+=$(SQLITE3_LIB)
 astdb2bdb: _ASTCFLAGS+=$(SQLITE3_INCLUDE)
 astdb2bdb: db1-ast/libdb1.a
 
+conf_bridge_binaural_hrir_importer: LIBS+=$(SNDFILE_LIB)
+conf_bridge_binaural_hrir_importer.o: _ASTCFLAGS+=$(SNDFILE_INCLUDE)
+
 ifneq ($(wildcard .*.d),)
    include .*.d
 endif
diff -pruN 1:13.23.1~dfsg-2/utils/muted.c 1:16.1.1~dfsg-1/utils/muted.c
--- 1:13.23.1~dfsg-2/utils/muted.c	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/muted.c	2018-12-26 18:45:13.000000000 +0000
@@ -45,7 +45,7 @@
  */
 
 /*** MODULEINFO
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
  ***/
 
 #include "asterisk/autoconfig.h"
diff -pruN 1:13.23.1~dfsg-2/utils/utils.xml 1:16.1.1~dfsg-1/utils/utils.xml
--- 1:13.23.1~dfsg-2/utils/utils.xml	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/utils/utils.xml	2018-12-26 18:45:13.000000000 +0000
@@ -1,7 +1,7 @@
 <category name="MENUSELECT_UTILS" displayname="Utilities" positive_output="yes">
   <member name="aelparse">
 	<defaultenabled>no</defaultenabled>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
   </member>
   <member name="astcanary">
 	<defaultenabled>yes</defaultenabled>
@@ -18,7 +18,7 @@
   <member name="astman">
 	<defaultenabled>no</defaultenabled>
 	<depend>newt</depend>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
   </member>
   <member name="check_expr">
 	<defaultenabled>no</defaultenabled>
@@ -30,11 +30,11 @@
   </member>
   <member name="conf2ael">
 	<defaultenabled>no</defaultenabled>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
   </member>
   <member name="muted">
 	<defaultenabled>no</defaultenabled>
-	<support_level>extended</support_level>
+	<support_level>deprecated</support_level>
   </member>
   <member name="smsq">
 	<defaultenabled>no</defaultenabled>
@@ -49,4 +49,12 @@
 	<defaultenabled>no</defaultenabled>
 	<support_level>extended</support_level>
   </member>
+  <member name="conf_bridge_binaural_hrir_importer"
+    displayname="Impulse Noise wav to hrirs.h generator"
+    remove_on_change="conf_bridge_binaural_hrir_importer">
+    <defaultenabled>no</defaultenabled>
+    <depend>sndfile</depend>
+    <support_level>extended</support_level>
+    <defaultenabled>no</defaultenabled>
+  </member>
 </category>
diff -pruN 1:13.23.1~dfsg-2/.version 1:16.1.1~dfsg-1/.version
--- 1:13.23.1~dfsg-2/.version	2018-09-20 18:38:45.000000000 +0000
+++ 1:16.1.1~dfsg-1/.version	2018-12-26 18:45:13.000000000 +0000
@@ -1 +1 @@
-13.23.1
\ No newline at end of file
+16.1.1
\ No newline at end of file
